"""Data containing all current emoji
   Extracted from https://unicode.org/Public/emoji/latest/emoji-test.txt
   and https://www.unicode.org/Public/UCD/latest/ucd/emoji/emoji-variation-sequences.txt
   See utils/get_codes_from_unicode_emoji_data_files.py

   +----------------+-------------+------------------+-------------------+
   | Emoji Version  |    Date     | Unicode Version  | Data File Comment |
   +----------------+-------------+------------------+-------------------+
   | N/A            | 2010-10-11  | Unicode 6.0      | E0.6              |
   | N/A            | 2014-06-16  | Unicode 7.0      | E0.7              |
   | Emoji 1.0      | 2015-06-09  | Unicode 8.0      | E1.0              |
   | Emoji 2.0      | 2015-11-12  | Unicode 8.0      | E2.0              |
   | Emoji 3.0      | 2016-06-03  | Unicode 9.0      | E3.0              |
   | Emoji 4.0      | 2016-11-22  | Unicode 9.0      | E4.0              |
   | Emoji 5.0      | 2017-06-20  | Unicode 10.0     | E5.0              |
   | Emoji 11.0     | 2018-05-21  | Unicode 11.0     | E11.0             |
   | Emoji 12.0     | 2019-03-05  | Unicode 12.0     | E12.0             |
   | Emoji 12.1     | 2019-10-21  | Unicode 12.1     | E12.1             |
   | Emoji 13.0     | 2020-03-10  | Unicode 13.0     | E13.0             |
   | Emoji 13.1     | 2020-09-15  | Unicode 13.0     | E13.1             |
   | Emoji 14.0     | 2021-09-14  | Unicode 14.0     | E14.0             |
   | Emoji 15.0     | 2022-09-13  | Unicode 15.0     | E15.0             |
   | Emoji 15.1     | 2023-09-12  | Unicode 15.1     | E15.1             |

                  http://www.unicode.org/reports/tr51/#Versioning

"""

__all__ = [
    'EMOJI_DATA', 'STATUS', 'LANGUAGES'
]

from typing import Any, Dict, List


component = 1
fully_qualified = 2
minimally_qualified = 3
unqualified = 4

STATUS: Dict[str, int] = {
    "component": component,
    "fully_qualified": fully_qualified,
    "minimally_qualified": minimally_qualified,
    "unqualified": unqualified
}

LANGUAGES: List[str] = ['en', 'es', 'ja', 'ko', 'pt', 'it', 'fr', 'de', 'fa', 'id', 'zh', 'ru', 'tr', 'ar']

EMOJI_DATA: Dict[str, Dict[str, Any]] = {'\U0001F947':{'en':':1st_place_medal:','status':fully_qualified,'E':3,'de':':goldmedaille:','es':':medalla_de_oro:','fr':':médaille_d’or:','ja':':金メダル:','ko':':금메달:','pt':':medalha_de_ouro:','it':':medaglia_d’oro:','fa':':مدال_طلا:','id':':medali_emas:','zh':':金牌:','ru':':золотая_медаль:','tr':':birincilik_madalyası:','ar':':ميدالية_مركز_أول:'},'\U0001F948':{'en':':2nd_place_medal:','status':fully_qualified,'E':3,'de':':silbermedaille:','es':':medalla_de_plata:','fr':':médaille_d’argent:','ja':':銀メダル:','ko':':은메달:','pt':':medalha_de_prata:','it':':medaglia_d’argento:','fa':':مدال_نقره:','id':':medali_perak:','zh':':银牌:','ru':':серебряная_медаль:','tr':':ikincilik_madalyası:','ar':':ميدالية_مركز_ثان:'},'\U0001F949':{'en':':3rd_place_medal:','status':fully_qualified,'E':3,'de':':bronzemedaille:','es':':medalla_de_bronce:','fr':':médaille_de_bronze:','ja':':銅メダル:','ko':':동메달:','pt':':medalha_de_bronze:','it':':medaglia_di_bronzo:','fa':':مدال_برنز:','id':':medali_perunggu:','zh':':铜牌:','ru':':бронзовая_медаль:','tr':':üçüncülük_madalyası:','ar':':ميدالية_مركز_ثالث:'},'\U0001F18E':{'en':':AB_button_(blood_type):','status':fully_qualified,'E':0.6,'alias':[':ab:', ':ab_button_blood_type:'],'de':':großbuchstaben_ab_in_rotem_quadrat:','es':':grupo_sanguíneo_ab:','fr':':groupe_sanguin_ab:','ja':':血液型ab型:','ko':':에이비형:','pt':':botão_ab_(tipo_sanguíneo):','it':':gruppo_sanguigno_ab:','fa':':دکمه_آ_ب_(گروه_خونی):','id':':tombol_ab_(golongan_darah):','zh':':AB型血:','ru':':IV_группа_крови:','tr':':ab_düğmesi_(kan_grubu):','ar':':زر_ab_(فئة_الدم):'},'\U0001F3E7':{'en':':ATM_sign:','status':fully_qualified,'E':0.6,'alias':[':atm:', ':atm_sign:'],'de':':symbol_geldautomat:','es':':señal_de_cajero_automático:','fr':':distributeur_de_billets:','ja':':atm:','ko':':에이티엠:','pt':':símbolo_de_caixa_automático:','it':':simbolo_dello_sportello_bancomat:','fa':':نشان_عابربانک:','id':':tanda_atm:','zh':':取款机:','ru':':значок_банкомата:','tr':':atm_işareti:','ar':':علامة_ماكينة_صرف_آلي:'},'\U0001F170\U0000FE0F':{'en':':A_button_(blood_type):','status':fully_qualified,'E':0.6,'alias':[':a:', ':a_button_blood_type:'],'variant':True,'de':':großbuchstabe_a_in_rotem_quadrat:','es':':grupo_sanguíneo_a:','fr':':groupe_sanguin_a:','ja':':血液型a型:','ko':':에이형:','pt':':botão_a_(tipo_sanguíneo):','it':':gruppo_sanguigno_a:','fa':':دکمه_آ_(گروه_خونی):','id':':tombol_a_(golongan_darah):','zh':':A型血:','ru':':ii_группа_крови:','tr':':a_düğmesi_(kan_grubu):','ar':':زر_a:'},'\U0001F170':{'en':':A_button_(blood_type):','status':unqualified,'E':0.6,'alias':[':a:', ':a_button_blood_type:'],'variant':True,'de':':großbuchstabe_a_in_rotem_quadrat:','es':':grupo_sanguíneo_a:','fr':':groupe_sanguin_a:','ja':':血液型a型:','ko':':에이형:','pt':':botão_a_(tipo_sanguíneo):','it':':gruppo_sanguigno_a:','fa':':دکمه_آ_(گروه_خونی):','id':':tombol_a_(golongan_darah):','zh':':A型血:','ru':':II_группа_крови:','tr':':a_düğmesi_(kan_grubu):','ar':':زر_a:'},'\U0001F1E6\U0001F1EB':{'en':':Afghanistan:','status':fully_qualified,'E':2,'alias':[':flag_for_Afghanistan:', ':afghanistan:'],'de':':flagge_afghanistan:','es':':bandera_afganistán:','fr':':drapeau_afghanistan:','ja':':旗_アフガニスタン:','ko':':깃발_아프가니스탄:','pt':':bandeira_afeganistão:','it':':bandiera_afghanistan:','fa':':پرچم_افغانستان:','id':':bendera_afganistan:','zh':':阿富汗:','ru':':флаг_Афганистан:','tr':':bayrak_afganistan:','ar':':علم_أفغانستان:'},'\U0001F1E6\U0001F1F1':{'en':':Albania:','status':fully_qualified,'E':2,'alias':[':flag_for_Albania:', ':albania:'],'de':':flagge_albanien:','es':':bandera_albania:','fr':':drapeau_albanie:','ja':':旗_アルバニア:','ko':':깃발_알바니아:','pt':':bandeira_albânia:','it':':bandiera_albania:','fa':':پرچم_آلبانی:','id':':bendera_albania:','zh':':阿尔巴尼亚:','ru':':флаг_Албания:','tr':':bayrak_arnavutluk:','ar':':علم_ألبانيا:'},'\U0001F1E9\U0001F1FF':{'en':':Algeria:','status':fully_qualified,'E':2,'alias':[':flag_for_Algeria:', ':algeria:'],'de':':flagge_algerien:','es':':bandera_argelia:','fr':':drapeau_algérie:','ja':':旗_アルジェリア:','ko':':깃발_알제리:','pt':':bandeira_argélia:','it':':bandiera_algeria:','fa':':پرچم_الجزایر:','id':':bendera_aljazair:','zh':':阿尔及利亚:','ru':':флаг_Алжир:','tr':':bayrak_cezayir:','ar':':علم_الجزائر:'},'\U0001F1E6\U0001F1F8':{'en':':American_Samoa:','status':fully_qualified,'E':2,'alias':[':flag_for_American_Samoa:', ':american_samoa:'],'de':':flagge_amerikanisch-samoa:','es':':bandera_samoa_americana:','fr':':drapeau_samoa_américaines:','ja':':旗_米領サモア:','ko':':깃발_아메리칸_사모아:','pt':':bandeira_samoa_americana:','it':':bandiera_samoa_americane:','fa':':پرچم_ساموآی_امریکا:','id':':bendera_samoa_amerika:','zh':':美属萨摩亚:','ru':':флаг_Американское_Самоа:','tr':':bayrak_amerikan_samoası:','ar':':علم_ساموا_الأمريكية:'},'\U0001F1E6\U0001F1E9':{'en':':Andorra:','status':fully_qualified,'E':2,'alias':[':flag_for_Andorra:', ':andorra:'],'de':':flagge_andorra:','es':':bandera_andorra:','fr':':drapeau_andorre:','ja':':旗_アンドラ:','ko':':깃발_안도라:','pt':':bandeira_andorra:','it':':bandiera_andorra:','fa':':پرچم_آندورا:','id':':bendera_andorra:','zh':':安道尔:','ru':':флаг_Андорра:','tr':':bayrak_andorra:','ar':':علم_أندورا:'},'\U0001F1E6\U0001F1F4':{'en':':Angola:','status':fully_qualified,'E':2,'alias':[':flag_for_Angola:', ':angola:'],'de':':flagge_angola:','es':':bandera_angola:','fr':':drapeau_angola:','ja':':旗_アンゴラ:','ko':':깃발_앙골라:','pt':':bandeira_angola:','it':':bandiera_angola:','fa':':پرچم_آنگولا:','id':':bendera_angola:','zh':':安哥拉:','ru':':флаг_Ангола:','tr':':bayrak_angola:','ar':':علم_أنغولا:'},'\U0001F1E6\U0001F1EE':{'en':':Anguilla:','status':fully_qualified,'E':2,'alias':[':flag_for_Anguilla:', ':anguilla:'],'de':':flagge_anguilla:','es':':bandera_anguila:','fr':':drapeau_anguilla:','ja':':旗_アンギラ:','ko':':깃발_앵귈라:','pt':':bandeira_anguila:','it':':bandiera_anguilla:','fa':':پرچم_آنگویلا:','id':':bendera_anguilla:','zh':':安圭拉:','ru':':флаг_Ангилья:','tr':':bayrak_anguilla:','ar':':علم_أنغويلا:'},'\U0001F1E6\U0001F1F6':{'en':':Antarctica:','status':fully_qualified,'E':2,'alias':[':flag_for_Antarctica:', ':antarctica:'],'de':':flagge_antarktis:','es':':bandera_antártida:','fr':':drapeau_antarctique:','ja':':旗_南極:','ko':':깃발_남극_대륙:','pt':':bandeira_antártida:','it':':bandiera_antartide:','fa':':پرچم_جنوبگان:','id':':bendera_antarktika:','zh':':南极洲:','ru':':флаг_Антарктида:','tr':':bayrak_antarktika:','ar':':علم_أنتاركتيكا:'},'\U0001F1E6\U0001F1EC':{'en':':Antigua_&_Barbuda:','status':fully_qualified,'E':2,'alias':[':flag_for_Antigua_&_Barbuda:', ':antigua_barbuda:'],'de':':flagge_antigua_und_barbuda:','es':':bandera_antigua_y_barbuda:','fr':':drapeau_antigua-et-barbuda:','ja':':旗_アンティグア・バーブーダ:','ko':':깃발_앤티가_바부다:','pt':':bandeira_antígua_e_barbuda:','it':':bandiera_antigua_e_barbuda:','fa':':پرچم_آنتیگوا_و_باربودا:','id':':bendera_antigua_dan_barbuda:','zh':':安提瓜和巴布达:','ru':':флаг_Антигуа_и_Барбуда:','tr':':bayrak_antigua_ve_barbuda:','ar':':علم_أنتيغوا_وبربودا:'},'\U00002652':{'en':':Aquarius:','status':fully_qualified,'E':0.6,'alias':[':aquarius:'],'variant':True,'de':':wassermann_(sternzeichen):','es':':acuario:','fr':':verseau:','ja':':みずがめ座:','ko':':물병자리:','pt':':signo_de_aquário:','it':':segno_zodiacale_dell’acquario:','fa':':برج_دلو:','id':':aquarius:','zh':':水瓶座:','ru':':знак_зодиака_Водолей:','tr':':kova_burcu:','ar':':الدلو:'},'\U0001F1E6\U0001F1F7':{'en':':Argentina:','status':fully_qualified,'E':2,'alias':[':flag_for_Argentina:', ':argentina:'],'de':':flagge_argentinien:','es':':bandera_argentina:','fr':':drapeau_argentine:','ja':':旗_アルゼンチン:','ko':':깃발_아르헨티나:','pt':':bandeira_argentina:','it':':bandiera_argentina:','fa':':پرچم_آرژانتین:','id':':bendera_argentina:','zh':':阿根廷:','ru':':флаг_Аргентина:','tr':':bayrak_arjantin:','ar':':علم_الأرجنتين:'},'\U00002648':{'en':':Aries:','status':fully_qualified,'E':0.6,'alias':[':aries:'],'variant':True,'de':':widder_(sternzeichen):','es':':aries:','fr':':bélier_zodiaque:','ja':':おひつじ座:','ko':':양자리:','pt':':signo_de_áries:','it':':segno_zodiacale_dell’ariete:','fa':':برج_حمل:','id':':aries:','zh':':白羊座:','ru':':знак_зодиака_Овен:','tr':':koç_burcu:','ar':':الحمل:'},'\U0001F1E6\U0001F1F2':{'en':':Armenia:','status':fully_qualified,'E':2,'alias':[':flag_for_Armenia:', ':armenia:'],'de':':flagge_armenien:','es':':bandera_armenia:','fr':':drapeau_arménie:','ja':':旗_アルメニア:','ko':':깃발_아르메니아:','pt':':bandeira_armênia:','it':':bandiera_armenia:','fa':':پرچم_ارمنستان:','id':':bendera_armenia:','zh':':亚美尼亚:','ru':':флаг_Армения:','tr':':bayrak_ermenistan:','ar':':علم_أرمينيا:'},'\U0001F1E6\U0001F1FC':{'en':':Aruba:','status':fully_qualified,'E':2,'alias':[':flag_for_Aruba:', ':aruba:'],'de':':flagge_aruba:','es':':bandera_aruba:','fr':':drapeau_aruba:','ja':':旗_アルバ:','ko':':깃발_아루바:','pt':':bandeira_aruba:','it':':bandiera_aruba:','fa':':پرچم_آروبا:','id':':bendera_aruba:','zh':':阿鲁巴:','ru':':флаг_Аруба:','tr':':bayrak_aruba:','ar':':علم_أروبا:'},'\U0001F1E6\U0001F1E8':{'en':':Ascension_Island:','status':fully_qualified,'E':2,'alias':[':flag_for_Ascension_Island:', ':ascension_island:'],'de':':flagge_ascension:','es':':bandera_isla_de_la_ascensión:','fr':':drapeau_île_de_l’ascension:','ja':':旗_アセンション島:','ko':':깃발_어센션섬:','pt':':bandeira_ilha_de_ascensão:','it':':bandiera_isola_ascensione:','fa':':پرچم_جزایر_آسنسیون:','id':':bendera_pulau_ascension:','zh':':阿森松岛:','ru':':флаг_о_в_Вознесения:','tr':':bayrak_ascension_adası:','ar':':علم_جزيرة_أسينشيون:'},'\U0001F1E6\U0001F1FA':{'en':':Australia:','status':fully_qualified,'E':2,'alias':[':flag_for_Australia:', ':australia:'],'de':':flagge_australien:','es':':bandera_australia:','fr':':drapeau_australie:','ja':':旗_オーストラリア:','ko':':깃발_오스트레일리아:','pt':':bandeira_austrália:','it':':bandiera_australia:','fa':':پرچم_استرالیا:','id':':bendera_australia:','zh':':澳大利亚:','ru':':флаг_Австралия:','tr':':bayrak_avustralya:','ar':':علم_أستراليا:'},'\U0001F1E6\U0001F1F9':{'en':':Austria:','status':fully_qualified,'E':2,'alias':[':flag_for_Austria:', ':austria:'],'de':':flagge_österreich:','es':':bandera_austria:','fr':':drapeau_autriche:','ja':':旗_オーストリア:','ko':':깃발_오스트리아:','pt':':bandeira_áustria:','it':':bandiera_austria:','fa':':پرچم_اتریش:','id':':bendera_austria:','zh':':奥地利:','ru':':флаг_Австрия:','tr':':bayrak_avusturya:','ar':':علم_النمسا:'},'\U0001F1E6\U0001F1FF':{'en':':Azerbaijan:','status':fully_qualified,'E':2,'alias':[':flag_for_Azerbaijan:', ':azerbaijan:'],'de':':flagge_aserbaidschan:','es':':bandera_azerbaiyán:','fr':':drapeau_azerbaïdjan:','ja':':旗_アゼルバイジャン:','ko':':깃발_아제르바이잔:','pt':':bandeira_azerbaijão:','it':':bandiera_azerbaigian:','fa':':پرچم_جمهوری_آذربایجان:','id':':bendera_azerbaijan:','zh':':阿塞拜疆:','ru':':флаг_Азербайджан:','tr':':bayrak_azerbaycan:','ar':':علم_أذربيجان:'},'\U0001F519':{'en':':BACK_arrow:','status':fully_qualified,'E':0.6,'alias':[':back:', ':back_arrow:'],'de':':back-pfeil:','es':':flecha_back:','fr':':flèche_retour:','ja':':back矢印:','ko':':뒤로:','pt':':seta_back:','it':':freccia_back:','fa':':پیکان_عقب:','id':':tanda_panah_back:','zh':':返回箭头:','ru':':стрелка_«назад»:','tr':':geri_oku:','ar':':سهم_رجوع:'},'\U0001F171\U0000FE0F':{'en':':B_button_(blood_type):','status':fully_qualified,'E':0.6,'alias':[':b:', ':b_button_blood_type:'],'variant':True,'de':':großbuchstabe_b_in_rotem_quadrat:','es':':grupo_sanguíneo_b:','fr':':groupe_sanguin_b:','ja':':血液型b型:','ko':':비형:','pt':':botão_b_(tipo_sanguíneo):','it':':gruppo_sanguigno_b:','fa':':دکمه_ب_(گروه_خونی):','id':':tombol_b_(golongan_darah):','zh':':B型血:','ru':':iii_группа_крови:','tr':':b_grubu_(kan_grubu):','ar':':زر_b:'},'\U0001F171':{'en':':B_button_(blood_type):','status':unqualified,'E':0.6,'alias':[':b:', ':b_button_blood_type:'],'variant':True,'de':':großbuchstabe_b_in_rotem_quadrat:','es':':grupo_sanguíneo_b:','fr':':groupe_sanguin_b:','ja':':血液型b型:','ko':':비형:','pt':':botão_b_(tipo_sanguíneo):','it':':gruppo_sanguigno_b:','fa':':دکمه_ب_(گروه_خونی):','id':':tombol_b_(golongan_darah):','zh':':B型血:','ru':':III_группа_крови:','tr':':b_grubu_(kan_grubu):','ar':':زر_b:'},'\U0001F1E7\U0001F1F8':{'en':':Bahamas:','status':fully_qualified,'E':2,'alias':[':flag_for_Bahamas:', ':bahamas:'],'de':':flagge_bahamas:','es':':bandera_bahamas:','fr':':drapeau_bahamas:','ja':':旗_バハマ:','ko':':깃발_바하마:','pt':':bandeira_bahamas:','it':':bandiera_bahamas:','fa':':پرچم_باهاما:','id':':bendera_bahama:','zh':':巴哈马:','ru':':флаг_Багамы:','tr':':bayrak_bahamalar:','ar':':علم_جزر_البهاما:'},'\U0001F1E7\U0001F1ED':{'en':':Bahrain:','status':fully_qualified,'E':2,'alias':[':flag_for_Bahrain:', ':bahrain:'],'de':':flagge_bahrain:','es':':bandera_baréin:','fr':':drapeau_bahreïn:','ja':':旗_バーレーン:','ko':':깃발_바레인:','pt':':bandeira_barein:','it':':bandiera_bahrein:','fa':':پرچم_بحرین:','id':':bendera_bahrain:','zh':':巴林:','ru':':флаг_Бахрейн:','tr':':bayrak_bahreyn:','ar':':علم_البحرين:'},'\U0001F1E7\U0001F1E9':{'en':':Bangladesh:','status':fully_qualified,'E':2,'alias':[':flag_for_Bangladesh:', ':bangladesh:'],'de':':flagge_bangladesch:','es':':bandera_bangladés:','fr':':drapeau_bangladesh:','ja':':旗_バングラデシュ:','ko':':깃발_방글라데시:','pt':':bandeira_bangladesh:','it':':bandiera_bangladesh:','fa':':پرچم_بنگلادش:','id':':bendera_bangladesh:','zh':':孟加拉国:','ru':':флаг_Бангладеш:','tr':':bayrak_bangladeş:','ar':':علم_بنغلاديش:'},'\U0001F1E7\U0001F1E7':{'en':':Barbados:','status':fully_qualified,'E':2,'alias':[':flag_for_Barbados:', ':barbados:'],'de':':flagge_barbados:','es':':bandera_barbados:','fr':':drapeau_barbade:','ja':':旗_バルバドス:','ko':':깃발_바베이도스:','pt':':bandeira_barbados:','it':':bandiera_barbados:','fa':':پرچم_باربادوس:','id':':bendera_barbados:','zh':':巴巴多斯:','ru':':флаг_Барбадос:','tr':':bayrak_barbados:','ar':':علم_بربادوس:'},'\U0001F1E7\U0001F1FE':{'en':':Belarus:','status':fully_qualified,'E':2,'alias':[':flag_for_Belarus:', ':belarus:'],'de':':flagge_belarus:','es':':bandera_bielorrusia:','fr':':drapeau_biélorussie:','ja':':旗_ベラルーシ:','ko':':깃발_벨라루스:','pt':':bandeira_bielorrússia:','it':':bandiera_bielorussia:','fa':':پرچم_بلاروس:','id':':bendera_belarus:','zh':':白俄罗斯:','ru':':флаг_Беларусь:','tr':':bayrak_belarus:','ar':':علم_بيلاروس:'},'\U0001F1E7\U0001F1EA':{'en':':Belgium:','status':fully_qualified,'E':2,'alias':[':flag_for_Belgium:', ':belgium:'],'de':':flagge_belgien:','es':':bandera_bélgica:','fr':':drapeau_belgique:','ja':':旗_ベルギー:','ko':':깃발_벨기에:','pt':':bandeira_bélgica:','it':':bandiera_belgio:','fa':':پرچم_بلژیک:','id':':bendera_belgia:','zh':':比利时:','ru':':флаг_Бельгия:','tr':':bayrak_belçika:','ar':':علم_بلجيكا:'},'\U0001F1E7\U0001F1FF':{'en':':Belize:','status':fully_qualified,'E':2,'alias':[':flag_for_Belize:', ':belize:'],'de':':flagge_belize:','es':':bandera_belice:','fr':':drapeau_belize:','ja':':旗_ベリーズ:','ko':':깃발_벨리즈:','pt':':bandeira_belize:','it':':bandiera_belize:','fa':':پرچم_بلیز:','id':':bendera_belize:','zh':':伯利兹:','ru':':флаг_Белиз:','tr':':bayrak_belize:','ar':':علم_بليز:'},'\U0001F1E7\U0001F1EF':{'en':':Benin:','status':fully_qualified,'E':2,'alias':[':flag_for_Benin:', ':benin:'],'de':':flagge_benin:','es':':bandera_benín:','fr':':drapeau_bénin:','ja':':旗_ベナン:','ko':':깃발_베냉:','pt':':bandeira_benin:','it':':bandiera_benin:','fa':':پرچم_بنین:','id':':bendera_benin:','zh':':贝宁:','ru':':флаг_Бенин:','tr':':bayrak_benin:','ar':':علم_بنين:'},'\U0001F1E7\U0001F1F2':{'en':':Bermuda:','status':fully_qualified,'E':2,'alias':[':flag_for_Bermuda:', ':bermuda:'],'de':':flagge_bermuda:','es':':bandera_bermudas:','fr':':drapeau_bermudes:','ja':':旗_バミューダ:','ko':':깃발_버뮤다:','pt':':bandeira_bermudas:','it':':bandiera_bermuda:','fa':':پرچم_برمودا:','id':':bendera_bermuda:','zh':':百慕大:','ru':':флаг_Бермудские_о_ва:','tr':':bayrak_bermuda:','ar':':علم_برمودا:'},'\U0001F1E7\U0001F1F9':{'en':':Bhutan:','status':fully_qualified,'E':2,'alias':[':flag_for_Bhutan:', ':bhutan:'],'de':':flagge_bhutan:','es':':bandera_bután:','fr':':drapeau_bhoutan:','ja':':旗_ブータン:','ko':':깃발_부탄:','pt':':bandeira_butão:','it':':bandiera_bhutan:','fa':':پرچم_بوتان:','id':':bendera_bhutan:','zh':':不丹:','ru':':флаг_Бутан:','tr':':bayrak_butan:','ar':':علم_بوتان:'},'\U0001F1E7\U0001F1F4':{'en':':Bolivia:','status':fully_qualified,'E':2,'alias':[':flag_for_Bolivia:', ':bolivia:'],'de':':flagge_bolivien:','es':':bandera_bolivia:','fr':':drapeau_bolivie:','ja':':旗_ボリビア:','ko':':깃발_볼리비아:','pt':':bandeira_bolívia:','it':':bandiera_bolivia:','fa':':پرچم_بولیوی:','id':':bendera_bolivia:','zh':':玻利维亚:','ru':':флаг_Боливия:','tr':':bayrak_bolivya:','ar':':علم_بوليفيا:'},'\U0001F1E7\U0001F1E6':{'en':':Bosnia_&_Herzegovina:','status':fully_qualified,'E':2,'alias':[':flag_for_Bosnia_&_Herzegovina:', ':bosnia_herzegovina:'],'de':':flagge_bosnien_und_herzegowina:','es':':bandera_bosnia_y_herzegovina:','fr':':drapeau_bosnie-herzégovine:','ja':':旗_ボスニア・ヘルツェゴビナ:','ko':':깃발_보스니아_헤르체고비나:','pt':':bandeira_bósnia_e_herzegovina:','it':':bandiera_bosnia_ed_erzegovina:','fa':':پرچم_بوسنی_و_هرزگوین:','id':':bendera_bosnia_dan_herzegovina:','zh':':波斯尼亚和黑塞哥维那:','ru':':флаг_Босния_и_Герцеговина:','tr':':bayrak_bosna-hersek:','ar':':علم_البوسنة_والهرسك:'},'\U0001F1E7\U0001F1FC':{'en':':Botswana:','status':fully_qualified,'E':2,'alias':[':flag_for_Botswana:', ':botswana:'],'de':':flagge_botsuana:','es':':bandera_botsuana:','fr':':drapeau_botswana:','ja':':旗_ボツワナ:','ko':':깃발_보츠와나:','pt':':bandeira_botsuana:','it':':bandiera_botswana:','fa':':پرچم_بوتسوانا:','id':':bendera_botswana:','zh':':博茨瓦纳:','ru':':флаг_Ботсвана:','tr':':bayrak_botsvana:','ar':':علم_بوتسوانا:'},'\U0001F1E7\U0001F1FB':{'en':':Bouvet_Island:','status':fully_qualified,'E':2,'alias':[':flag_for_Bouvet_Island:', ':bouvet_island:'],'de':':flagge_bouvetinsel:','es':':bandera_isla_bouvet:','fr':':drapeau_île_bouvet:','ja':':旗_ブーベ島:','ko':':깃발_부베섬:','pt':':bandeira_ilha_bouvet:','it':':bandiera_isola_bouvet:','fa':':پرچم_جزیرهٔ_بووه:','id':':bendera_pulau_bouvet:','zh':':布韦岛:','ru':':флаг_о_в_Буве:','tr':':bayrak_bouvet_adası:','ar':':علم_جزيرة_بوفيه:'},'\U0001F1E7\U0001F1F7':{'en':':Brazil:','status':fully_qualified,'E':2,'alias':[':flag_for_Brazil:', ':brazil:'],'de':':flagge_brasilien:','es':':bandera_brasil:','fr':':drapeau_brésil:','ja':':旗_ブラジル:','ko':':깃발_브라질:','pt':':bandeira_brasil:','it':':bandiera_brasile:','fa':':پرچم_برزیل:','id':':bendera_brasil:','zh':':巴西:','ru':':флаг_Бразилия:','tr':':bayrak_brezilya:','ar':':علم_البرازيل:'},'\U0001F1EE\U0001F1F4':{'en':':British_Indian_Ocean_Territory:','status':fully_qualified,'E':2,'alias':[':flag_for_British_Indian_Ocean_Territory:', ':british_indian_ocean_territory:'],'de':':flagge_britisches_territorium_im_indischen_ozean:','es':':bandera_territorio_británico_del_océano_índico:','fr':':drapeau_territoire_britannique_de_l’océan_indien:','ja':':旗_英領インド洋地域:','ko':':깃발_영국령_인도양_지역:','pt':':bandeira_território_britânico_do_oceano_índico:','it':':bandiera_territorio_britannico_dell’oceano_indiano:','fa':':پرچم_قلمرو_بریتانیا_در_اقیانوس_هند:','id':':bendera_wilayah_inggris_di_samudra_hindia:','zh':':英属印度洋领地:','ru':':флаг_Британская_территория_в_Индийском_океане:','tr':':bayrak_britanya_hint_okyanusu_toprakları:','ar':':علم_الإقليم_البريطاني_في_المحيط_الهندي:'},'\U0001F1FB\U0001F1EC':{'en':':British_Virgin_Islands:','status':fully_qualified,'E':2,'alias':[':flag_for_British_Virgin_Islands:', ':british_virgin_islands:'],'de':':flagge_britische_jungferninseln:','es':':bandera_islas_vírgenes_británicas:','fr':':drapeau_îles_vierges_britanniques:','ja':':旗_英領ヴァージン諸島:','ko':':깃발_영국령_버진아일랜드:','pt':':bandeira_ilhas_virgens_britânicas:','it':':bandiera_isole_vergini_britanniche:','fa':':پرچم_جزایر_ویرجین_بریتانیا:','id':':bendera_kepulauan_virgin_britania_raya:','zh':':英属维尔京群岛:','ru':':флаг_Виргинские_о_ва_(Великобритания):','tr':':bayrak_britanya_virjin_adaları:','ar':':علم_جزر_فيرجن_البريطانية:'},'\U0001F1E7\U0001F1F3':{'en':':Brunei:','status':fully_qualified,'E':2,'alias':[':flag_for_Brunei:', ':brunei:'],'de':':flagge_brunei_darussalam:','es':':bandera_brunéi:','fr':':drapeau_brunei:','ja':':旗_ブルネイ:','ko':':깃발_브루나이:','pt':':bandeira_brunei:','it':':bandiera_brunei:','fa':':پرچم_برونئی:','id':':bendera_brunei:','zh':':文莱:','ru':':флаг_Бруней:','tr':':bayrak_brunei:','ar':':علم_بروناي:'},'\U0001F1E7\U0001F1EC':{'en':':Bulgaria:','status':fully_qualified,'E':2,'alias':[':flag_for_Bulgaria:', ':bulgaria:'],'de':':flagge_bulgarien:','es':':bandera_bulgaria:','fr':':drapeau_bulgarie:','ja':':旗_ブルガリア:','ko':':깃발_불가리아:','pt':':bandeira_bulgária:','it':':bandiera_bulgaria:','fa':':پرچم_بلغارستان:','id':':bendera_bulgaria:','zh':':保加利亚:','ru':':флаг_Болгария:','tr':':bayrak_bulgaristan:','ar':':علم_بلغاريا:'},'\U0001F1E7\U0001F1EB':{'en':':Burkina_Faso:','status':fully_qualified,'E':2,'alias':[':flag_for_Burkina_Faso:', ':burkina_faso:'],'de':':flagge_burkina_faso:','es':':bandera_burkina_faso:','fr':':drapeau_burkina_faso:','ja':':旗_ブルキナファソ:','ko':':깃발_부르키나파소:','pt':':bandeira_burquina_faso:','it':':bandiera_burkina_faso:','fa':':پرچم_بورکینافاسو:','id':':bendera_burkina_faso:','zh':':布基纳法索:','ru':':флаг_Буркина_Фасо:','tr':':bayrak_burkina_faso:','ar':':علم_بوركينا_فاسو:'},'\U0001F1E7\U0001F1EE':{'en':':Burundi:','status':fully_qualified,'E':2,'alias':[':flag_for_Burundi:', ':burundi:'],'de':':flagge_burundi:','es':':bandera_burundi:','fr':':drapeau_burundi:','ja':':旗_ブルンジ:','ko':':깃발_부룬디:','pt':':bandeira_burundi:','it':':bandiera_burundi:','fa':':پرچم_بوروندی:','id':':bendera_burundi:','zh':':布隆迪:','ru':':флаг_Бурунди:','tr':':bayrak_burundi:','ar':':علم_بوروندي:'},'\U0001F191':{'en':':CL_button:','status':fully_qualified,'E':0.6,'alias':[':cl:', ':cl_button:'],'de':':großbuchstaben_cl_in_rotem_quadrat:','es':':borrar:','fr':':bouton_effacer:','ja':':clマーク:','ko':':씨엘:','pt':':botão_cl:','it':':pulsante_cl:','fa':':دکمهٔ_cl:','id':':tombol_cl:','zh':':CL按钮:','ru':':кнопка_«очистить»:','tr':':cl_düğmesi:','ar':':مسح_داخل_مربع:'},'\U0001F192':{'en':':COOL_button:','status':fully_qualified,'E':0.6,'alias':[':cool:', ':cool_button:'],'de':':wort_cool_in_blauem_quadrat:','es':':botón_cool:','fr':':bouton_cool:','ja':':coolマーク:','ko':':쿨:','pt':':botão_cool:','it':':pulsante_cool:','fa':':دکمهٔ_cool:','id':':tombol_cool:','zh':':cool按钮:','ru':':значок_«круто»:','tr':':cool_düğmesi:','ar':':بارد_داخل_مربع:'},'\U0001F1F0\U0001F1ED':{'en':':Cambodia:','status':fully_qualified,'E':2,'alias':[':flag_for_Cambodia:', ':cambodia:'],'de':':flagge_kambodscha:','es':':bandera_camboya:','fr':':drapeau_cambodge:','ja':':旗_カンボジア:','ko':':깃발_캄보디아:','pt':':bandeira_camboja:','it':':bandiera_cambogia:','fa':':پرچم_کامبوج:','id':':bendera_kamboja:','zh':':柬埔寨:','ru':':флаг_Камбоджа:','tr':':bayrak_kamboçya:','ar':':علم_كمبوديا:'},'\U0001F1E8\U0001F1F2':{'en':':Cameroon:','status':fully_qualified,'E':2,'alias':[':flag_for_Cameroon:', ':cameroon:'],'de':':flagge_kamerun:','es':':bandera_camerún:','fr':':drapeau_cameroun:','ja':':旗_カメルーン:','ko':':깃발_카메룬:','pt':':bandeira_camarões:','it':':bandiera_camerun:','fa':':پرچم_کامرون:','id':':bendera_kamerun:','zh':':喀麦隆:','ru':':флаг_Камерун:','tr':':bayrak_kamerun:','ar':':علم_الكاميرون:'},'\U0001F1E8\U0001F1E6':{'en':':Canada:','status':fully_qualified,'E':2,'alias':[':flag_for_Canada:', ':canada:'],'de':':flagge_kanada:','es':':bandera_canadá:','fr':':drapeau_canada:','ja':':旗_カナダ:','ko':':깃발_캐나다:','pt':':bandeira_canadá:','it':':bandiera_canada:','fa':':پرچم_کانادا:','id':':bendera_kanada:','zh':':加拿大:','ru':':флаг_Канада:','tr':':bayrak_kanada:','ar':':علم_كندا:'},'\U0001F1EE\U0001F1E8':{'en':':Canary_Islands:','status':fully_qualified,'E':2,'alias':[':flag_for_Canary_Islands:', ':canary_islands:'],'de':':flagge_kanarische_inseln:','es':':bandera_canarias:','fr':':drapeau_îles_canaries:','ja':':旗_カナリア諸島:','ko':':깃발_카나리아_제도:','pt':':bandeira_ilhas_canárias:','it':':bandiera_isole_canarie:','fa':':پرچم_جزایر_قناری:','id':':bendera_kepulauan_canaria:','zh':':加纳利群岛:','ru':':флаг_Канарские_о_ва:','tr':':bayrak_kanarya_adaları:','ar':':علم_جزر_الكناري:'},'\U0000264B':{'en':':Cancer:','status':fully_qualified,'E':0.6,'alias':[':cancer:'],'variant':True,'de':':krebs_(sternzeichen):','es':':cáncer:','fr':':cancer:','ja':':かに座:','ko':':게자리:','pt':':signo_de_câncer:','it':':segno_zodiacale_del_cancro:','fa':':برج_سرطان:','id':':cancer:','zh':':巨蟹座:','ru':':знак_зодиака_Рак:','tr':':yengeç_burcu:','ar':':السرطان:'},'\U0001F1E8\U0001F1FB':{'en':':Cape_Verde:','status':fully_qualified,'E':2,'alias':[':flag_for_Cape_Verde:', ':cape_verde:'],'de':':flagge_cabo_verde:','es':':bandera_cabo_verde:','fr':':drapeau_cap-vert:','ja':':旗_カーボベルデ:','ko':':깃발_카보베르데:','pt':':bandeira_cabo_verde:','it':':bandiera_capo_verde:','fa':':پرچم_کیپ_ورد:','id':':bendera_tanjung_verde:','zh':':佛得角:','ru':':флаг_Кабо_Верде:','tr':':bayrak_cabo_verde:','ar':':علم_الرأس_الأخضر:'},'\U00002651':{'en':':Capricorn:','status':fully_qualified,'E':0.6,'alias':[':capricorn:'],'variant':True,'de':':steinbock_(sternzeichen):','es':':capricornio:','fr':':capricorne:','ja':':やぎ座:','ko':':염소자리:','pt':':signo_de_capricórnio:','it':':segno_zodiacale_del_capricorno:','fa':':برج_جدی:','id':':capricorn:','zh':':摩羯座:','ru':':знак_зодиака_Козерог:','tr':':oğlak_burcu:','ar':':الجدي:'},'\U0001F1E7\U0001F1F6':{'en':':Caribbean_Netherlands:','status':fully_qualified,'E':2,'alias':[':flag_for_Caribbean_Netherlands:', ':caribbean_netherlands:'],'de':':flagge_karibische_niederlande:','es':':bandera_caribe_neerlandés:','fr':':drapeau_pays-bas_caribéens:','ja':':旗_オランダ領カリブ:','ko':':깃발_네덜란드령_카리브:','pt':':bandeira_países_baixos_caribenhos:','it':':bandiera_caraibi_olandesi:','fa':':پرچم_جزایر_کارائیب_هلند:','id':':bendera_belanda_karibia:','zh':':荷属加勒比区:','ru':':флаг_Бонэйр_Синт_Эстатиус_и_Саба:','tr':':bayrak_karayip_hollandası:','ar':':علم_هولندا_الكاريبية:'},'\U0001F1F0\U0001F1FE':{'en':':Cayman_Islands:','status':fully_qualified,'E':2,'alias':[':flag_for_Cayman_Islands:', ':cayman_islands:'],'de':':flagge_kaimaninseln:','es':':bandera_islas_caimán:','fr':':drapeau_îles_caïmans:','ja':':旗_ケイマン諸島:','ko':':깃발_케이맨_제도:','pt':':bandeira_ilhas_cayman:','it':':bandiera_isole_cayman:','fa':':پرچم_جزایر_کِیمن:','id':':bendera_kepulauan_cayman:','zh':':开曼群岛:','ru':':флаг_о_ва_Кайман:','tr':':bayrak_cayman_adaları:','ar':':علم_جزر_كايمان:'},'\U0001F1E8\U0001F1EB':{'en':':Central_African_Republic:','status':fully_qualified,'E':2,'alias':[':flag_for_Central_African_Republic:', ':central_african_republic:'],'de':':flagge_zentralafrikanische_republik:','es':':bandera_república_centroafricana:','fr':':drapeau_république_centrafricaine:','ja':':旗_中央アフリカ共和国:','ko':':깃발_중앙_아프리카_공화국:','pt':':bandeira_república_centro-africana:','it':':bandiera_repubblica_centrafricana:','fa':':پرچم_جمهوری_افریقای_مرکزی:','id':':bendera_republik_afrika_tengah:','zh':':中非共和国:','ru':':флаг_Центрально_Африканская_Республика:','tr':':bayrak_orta_afrika_cumhuriyeti:','ar':':علم_جمهورية_أفريقيا_الوسطى:'},'\U0001F1EA\U0001F1E6':{'en':':Ceuta_&_Melilla:','status':fully_qualified,'E':2,'alias':[':flag_for_Ceuta_&_Melilla:', ':ceuta_melilla:'],'de':':flagge_ceuta_und_melilla:','es':':bandera_ceuta_y_melilla:','fr':':drapeau_ceuta_et_melilla:','ja':':旗_セウタ・メリリャ:','ko':':깃발_세우타_및_멜리야:','pt':':bandeira_ceuta_e_melilla:','it':':bandiera_ceuta_e_melilla:','fa':':پرچم_سبته_و_ملیله:','id':':bendera_ceuta_dan_melilla:','zh':':休达及梅利利亚:','ru':':флаг_Сеута_и_Мелилья:','tr':':bayrak_ceuta_ve_melilla:','ar':':علم_سيوتا_وميليلا:'},'\U0001F1F9\U0001F1E9':{'en':':Chad:','status':fully_qualified,'E':2,'alias':[':flag_for_Chad:', ':chad:'],'de':':flagge_tschad:','es':':bandera_chad:','fr':':drapeau_tchad:','ja':':旗_チャド:','ko':':깃발_차드:','pt':':bandeira_chade:','it':':bandiera_ciad:','fa':':پرچم_چاد:','id':':bendera_chad:','zh':':乍得:','ru':':флаг_Чад:','tr':':bayrak_çad:','ar':':علم_تشاد:'},'\U0001F1E8\U0001F1F1':{'en':':Chile:','status':fully_qualified,'E':2,'alias':[':flag_for_Chile:', ':chile:'],'de':':flagge_chile:','es':':bandera_chile:','fr':':drapeau_chili:','ja':':旗_チリ:','ko':':깃발_칠레:','pt':':bandeira_chile:','it':':bandiera_cile:','fa':':پرچم_شیلی:','id':':bendera_cile:','zh':':智利:','ru':':флаг_Чили:','tr':':bayrak_şili:','ar':':علم_تشيلي:'},'\U0001F1E8\U0001F1F3':{'en':':China:','status':fully_qualified,'E':0.6,'alias':[':flag_for_China:', ':cn:'],'de':':flagge_china:','es':':bandera_china:','fr':':drapeau_chine:','ja':':旗_中国:','ko':':깃발_중국:','pt':':bandeira_china:','it':':bandiera_cina:','fa':':پرچم_چین:','id':':bendera_tiongkok:','zh':':中国:','ru':':флаг_Китай:','tr':':bayrak_çin:','ar':':علم_الصين:'},'\U0001F1E8\U0001F1FD':{'en':':Christmas_Island:','status':fully_qualified,'E':2,'alias':[':flag_for_Christmas_Island:', ':christmas_island:'],'de':':flagge_weihnachtsinsel:','es':':bandera_isla_de_navidad:','fr':':drapeau_île_christmas:','ja':':旗_クリスマス島:','ko':':깃발_크리스마스섬:','pt':':bandeira_ilha_christmas:','it':':bandiera_isola_christmas:','fa':':پرچم_جزیرهٔ_کریسمس:','id':':bendera_pulau_natal:','zh':':圣诞岛:','ru':':флаг_о_в_Рождества:','tr':':bayrak_christmas_adası:','ar':':علم_جزيرة_كريسماس:'},'\U0001F384':{'en':':Christmas_tree:','status':fully_qualified,'E':0.6,'alias':[':christmas_tree:'],'de':':weihnachtsbaum:','es':':árbol_de_navidad:','fr':':sapin_de_noël:','ja':':クリスマスツリー:','ko':':크리스마스_트리:','pt':':árvore_de_natal:','it':':albero_di_natale:','fa':':درخت_کریسمس:','id':':pohon_natal:','zh':':圣诞树:','ru':':новогодняя_елка:','tr':':noel_ağacı:','ar':':شجرة_عيد_الميلاد:'},'\U0001F1E8\U0001F1F5':{'en':':Clipperton_Island:','status':fully_qualified,'E':2,'alias':[':flag_for_Clipperton_Island:', ':clipperton_island:'],'de':':flagge_clipperton-insel:','es':':bandera_isla_clipperton:','fr':':drapeau_île_clipperton:','ja':':旗_クリッパートン島:','ko':':깃발_클리퍼턴섬:','pt':':bandeira_ilha_de_clipperton:','it':':bandiera_isola_di_clipperton:','fa':':پرچم_جزایر_کلیپرتون:','id':':bendera_pulau_clipperton:','zh':':克利珀顿岛:','ru':':флаг_о_в_Клиппертон:','tr':':bayrak_clipperton_adası:','ar':':علم_جزيرة_كليبيرتون:'},'\U0001F1E8\U0001F1E8':{'en':':Cocos_(Keeling)_Islands:','status':fully_qualified,'E':2,'alias':[':flag_for_Cocos_Islands:', ':cocos_islands:'],'de':':flagge_kokosinseln:','es':':bandera_islas_cocos:','fr':':drapeau_îles_cocos:','ja':':旗_ココス(キーリング)諸島:','ko':':깃발_코코스_제도:','pt':':bandeira_ilhas_cocos_(keeling):','it':':bandiera_isole_cocos_(keeling):','fa':':پرچم_جزایر_کوکوس:','id':':bendera_kepulauan_cocos_(keeling):','zh':':科科斯(基林)群岛:','ru':':флаг_Кокосовые_о_ва:','tr':':bayrak_cocos_(keeling)_adaları:','ar':':علم_جزر_كوكوس_(كيلينغ):'},'\U0001F1E8\U0001F1F4':{'en':':Colombia:','status':fully_qualified,'E':2,'alias':[':flag_for_Colombia:', ':colombia:'],'de':':flagge_kolumbien:','es':':bandera_colombia:','fr':':drapeau_colombie:','ja':':旗_コロンビア:','ko':':깃발_콜롬비아:','pt':':bandeira_colômbia:','it':':bandiera_colombia:','fa':':پرچم_کلمبیا:','id':':bendera_kolombia:','zh':':哥伦比亚:','ru':':флаг_Колумбия:','tr':':bayrak_kolombiya:','ar':':علم_كولومبيا:'},'\U0001F1F0\U0001F1F2':{'en':':Comoros:','status':fully_qualified,'E':2,'alias':[':flag_for_Comoros:', ':comoros:'],'de':':flagge_komoren:','es':':bandera_comoras:','fr':':drapeau_comores:','ja':':旗_コモロ:','ko':':깃발_코모로:','pt':':bandeira_comores:','it':':bandiera_comore:','fa':':پرچم_کومور:','id':':bendera_komoro:','zh':':科摩罗:','ru':':флаг_Коморы:','tr':':bayrak_komorlar:','ar':':علم_جزر_القمر:'},'\U0001F1E8\U0001F1EC':{'en':':Congo-Brazzaville:','status':fully_qualified,'E':2,'alias':[':flag_for_Congo_Brazzaville:', ':congo_brazzaville:'],'de':':flagge_kongo-brazzaville:','es':':bandera_congo:','fr':':drapeau_congo-brazzaville:','ja':':旗_コンゴ共和国(ブラザビル):','ko':':깃발_콩고-브라자빌:','pt':':bandeira_república_do_congo:','it':':bandiera_congo-brazzaville:','fa':':پرچم_کنگو-برازویل:','id':':bendera_kongo-brazzaville:','zh':':刚果_布:','ru':':флаг_Конго_Браззавиль:','tr':':bayrak_kongo-brazavil:','ar':':علم_الكونغو-برازافيل:'},'\U0001F1E8\U0001F1E9':{'en':':Congo-Kinshasa:','status':fully_qualified,'E':2,'alias':[':flag_for_Congo_Kinshasa:', ':congo_kinshasa:'],'de':':flagge_kongo-kinshasa:','es':':bandera_república_democrática_del_congo:','fr':':drapeau_congo-kinshasa:','ja':':旗_コンゴ民主共和国(キンシャサ):','ko':':깃발_콩고-킨샤사:','pt':':bandeira_congo-kinshasa:','it':':bandiera_congo-kinshasa:','fa':':پرچم_کنگو-کینشاسا:','id':':bendera_kongo-kinshasa:','zh':':刚果_金:','ru':':флаг_Конго_Киншаса:','tr':':bayrak_kongo-kinşasa:','ar':':علم_الكونغو-كينشاسا:'},'\U0001F1E8\U0001F1F0':{'en':':Cook_Islands:','status':fully_qualified,'E':2,'alias':[':flag_for_Cook_Islands:', ':cook_islands:'],'de':':flagge_cookinseln:','es':':bandera_islas_cook:','fr':':drapeau_îles_cook:','ja':':旗_クック諸島:','ko':':깃발_쿡_제도:','pt':':bandeira_ilhas_cook:','it':':bandiera_isole_cook:','fa':':پرچم_جزایر_کوک:','id':':bendera_kepulauan_cook:','zh':':库克群岛:','ru':':флаг_о_ва_Кука:','tr':':bayrak_cook_adaları:','ar':':علم_جزر_كوك:'},'\U0001F1E8\U0001F1F7':{'en':':Costa_Rica:','status':fully_qualified,'E':2,'alias':[':flag_for_Costa_Rica:', ':costa_rica:'],'de':':flagge_costa_rica:','es':':bandera_costa_rica:','fr':':drapeau_costa_rica:','ja':':旗_コスタリカ:','ko':':깃발_코스타리카:','pt':':bandeira_costa_rica:','it':':bandiera_costa_rica:','fa':':پرچم_کاستاریکا:','id':':bendera_kosta_rika:','zh':':哥斯达黎加:','ru':':флаг_Коста_Рика:','tr':':bayrak_kosta_rika:','ar':':علم_كوستاريكا:'},'\U0001F1ED\U0001F1F7':{'en':':Croatia:','status':fully_qualified,'E':2,'alias':[':flag_for_Croatia:', ':croatia:'],'de':':flagge_kroatien:','es':':bandera_croacia:','fr':':drapeau_croatie:','ja':':旗_クロアチア:','ko':':깃발_크로아티아:','pt':':bandeira_croácia:','it':':bandiera_croazia:','fa':':پرچم_کرواسی:','id':':bendera_kroasia:','zh':':克罗地亚:','ru':':флаг_Хорватия:','tr':':bayrak_hırvatistan:','ar':':علم_كرواتيا:'},'\U0001F1E8\U0001F1FA':{'en':':Cuba:','status':fully_qualified,'E':2,'alias':[':flag_for_Cuba:', ':cuba:'],'de':':flagge_kuba:','es':':bandera_cuba:','fr':':drapeau_cuba:','ja':':旗_キューバ:','ko':':깃발_쿠바:','pt':':bandeira_cuba:','it':':bandiera_cuba:','fa':':پرچم_کوبا:','id':':bendera_kuba:','zh':':古巴:','ru':':флаг_Куба:','tr':':bayrak_küba:','ar':':علم_كوبا:'},'\U0001F1E8\U0001F1FC':{'en':':Curaçao:','status':fully_qualified,'E':2,'alias':[':flag_for_Curaçao:', ':curacao:'],'de':':flagge_curaçao:','es':':bandera_curazao:','fr':':drapeau_curaçao:','ja':':旗_キュラソー:','ko':':깃발_퀴라소:','pt':':bandeira_curaçao:','it':':bandiera_curaçao:','fa':':پرچم_کوراسائو:','id':':bendera_curaçao:','zh':':库拉索:','ru':':флаг_Кюрасао:','tr':':bayrak_curaçao:','ar':':علم_كوراساو:'},'\U0001F1E8\U0001F1FE':{'en':':Cyprus:','status':fully_qualified,'E':2,'alias':[':flag_for_Cyprus:', ':cyprus:'],'de':':flagge_zypern:','es':':bandera_chipre:','fr':':drapeau_chypre:','ja':':旗_キプロス:','ko':':깃발_키프로스:','pt':':bandeira_chipre:','it':':bandiera_cipro:','fa':':پرچم_قبرس:','id':':bendera_siprus:','zh':':塞浦路斯:','ru':':флаг_Кипр:','tr':':bayrak_kıbrıs:','ar':':علم_قبرص:'},'\U0001F1E8\U0001F1FF':{'en':':Czechia:','status':fully_qualified,'E':2,'alias':[':flag_for_Czech_Republic:', ':czech_republic:'],'de':':flagge_tschechien:','es':':bandera_chequia:','fr':':drapeau_tchéquie:','ja':':旗_チェコ:','ko':':깃발_체코:','pt':':bandeira_tchéquia:','it':':bandiera_cechia:','fa':':پرچم_چک:','id':':bendera_ceko:','zh':':捷克:','ru':':флаг_Чехия:','tr':':bayrak_çekya:','ar':':علم_التشيك:'},'\U0001F1E8\U0001F1EE':{'en':':Côte_d’Ivoire:','status':fully_qualified,'E':2,'alias':[':flag_for_Côte_d’Ivoire:', ':cote_divoire:'],'de':':flagge_côte_d’ivoire:','es':':bandera_côte_d’ivoire:','fr':':drapeau_côte_d’ivoire:','ja':':旗_コートジボワール:','ko':':깃발_코트디부아르:','pt':':bandeira_costa_do_marfim:','it':':bandiera_costa_d’avorio:','fa':':پرچم_ساحل_عاج:','id':':bendera_côte_d’ivoire:','zh':':科特迪瓦:','ru':':флаг_Кот_д’Ивуар:','tr':':bayrak_côte_d’ivoire:','ar':':علم_ساحل_العاج:'},'\U0001F1E9\U0001F1F0':{'en':':Denmark:','status':fully_qualified,'E':2,'alias':[':flag_for_Denmark:', ':denmark:'],'de':':flagge_dänemark:','es':':bandera_dinamarca:','fr':':drapeau_danemark:','ja':':旗_デンマーク:','ko':':깃발_덴마크:','pt':':bandeira_dinamarca:','it':':bandiera_danimarca:','fa':':پرچم_دانمارک:','id':':bendera_denmark:','zh':':丹麦:','ru':':флаг_Дания:','tr':':bayrak_danimarka:','ar':':علم_الدانمرك:'},'\U0001F1E9\U0001F1EC':{'en':':Diego_Garcia:','status':fully_qualified,'E':2,'alias':[':flag_for_Diego_Garcia:', ':diego_garcia:'],'de':':flagge_diego_garcia:','es':':bandera_diego_garcía:','fr':':drapeau_diego_garcia:','ja':':旗_ディエゴガルシア島:','ko':':깃발_디에고_가르시아:','pt':':bandeira_diego_garcia:','it':':bandiera_diego_garcia:','fa':':پرچم_دیه_گو_گارسیا:','id':':bendera_diego_garcia:','zh':':迪戈加西亚岛:','ru':':флаг_Диего_Гарсия:','tr':':bayrak_diego_garcia:','ar':':علم_دييغو_غارسيا:'},'\U0001F1E9\U0001F1EF':{'en':':Djibouti:','status':fully_qualified,'E':2,'alias':[':flag_for_Djibouti:', ':djibouti:'],'de':':flagge_dschibuti:','es':':bandera_yibuti:','fr':':drapeau_djibouti:','ja':':旗_ジブチ:','ko':':깃발_지부티:','pt':':bandeira_djibuti:','it':':bandiera_gibuti:','fa':':پرچم_جیبوتی:','id':':bendera_jibuti:','zh':':吉布提:','ru':':флаг_Джибути:','tr':':bayrak_cibuti:','ar':':علم_جيبوتي:'},'\U0001F1E9\U0001F1F2':{'en':':Dominica:','status':fully_qualified,'E':2,'alias':[':flag_for_Dominica:', ':dominica:'],'de':':flagge_dominica:','es':':bandera_dominica:','fr':':drapeau_dominique:','ja':':旗_ドミニカ国:','ko':':깃발_도미니카:','pt':':bandeira_dominica:','it':':bandiera_dominica:','fa':':پرچم_دومینیکا:','id':':bendera_dominika:','zh':':多米尼克:','ru':':флаг_Доминика:','tr':':bayrak_dominika:','ar':':علم_دومينيكا:'},'\U0001F1E9\U0001F1F4':{'en':':Dominican_Republic:','status':fully_qualified,'E':2,'alias':[':flag_for_Dominican_Republic:', ':dominican_republic:'],'de':':flagge_dominikanische_republik:','es':':bandera_república_dominicana:','fr':':drapeau_république_dominicaine:','ja':':旗_ドミニカ共和国:','ko':':깃발_도미니카_공화국:','pt':':bandeira_república_dominicana:','it':':bandiera_repubblica_dominicana:','fa':':پرچم_جمهوری_دومینیکن:','id':':bendera_republik_dominika:','zh':':多米尼加共和国:','ru':':флаг_Доминиканская_Республика:','tr':':bayrak_dominik_cumhuriyeti:','ar':':علم_جمهورية_الدومينيكان:'},'\U0001F51A':{'en':':END_arrow:','status':fully_qualified,'E':0.6,'alias':[':end:', ':end_arrow:'],'de':':end-pfeil:','es':':flecha_end:','fr':':flèche_fin:','ja':':end矢印:','ko':':종료:','pt':':seta_end:','it':':freccia_end:','fa':':پیکان_پایان:','id':':tanda_panah_end:','zh':':结束箭头:','ru':':стрелка_«конец»:','tr':':bitiş_oku:','ar':':سهم_النهاية:'},'\U0001F1EA\U0001F1E8':{'en':':Ecuador:','status':fully_qualified,'E':2,'alias':[':flag_for_Ecuador:', ':ecuador:'],'de':':flagge_ecuador:','es':':bandera_ecuador:','fr':':drapeau_équateur:','ja':':旗_エクアドル:','ko':':깃발_에콰도르:','pt':':bandeira_equador:','it':':bandiera_ecuador:','fa':':پرچم_اکوادور:','id':':bendera_ekuador:','zh':':厄瓜多尔:','ru':':флаг_Эквадор:','tr':':bayrak_ekvador:','ar':':علم_الإكوادور:'},'\U0001F1EA\U0001F1EC':{'en':':Egypt:','status':fully_qualified,'E':2,'alias':[':flag_for_Egypt:', ':egypt:'],'de':':flagge_ägypten:','es':':bandera_egipto:','fr':':drapeau_égypte:','ja':':旗_エジプト:','ko':':깃발_이집트:','pt':':bandeira_egito:','it':':bandiera_egitto:','fa':':پرچم_مصر:','id':':bendera_mesir:','zh':':埃及:','ru':':флаг_Египет:','tr':':bayrak_mısır:','ar':':علم_مصر:'},'\U0001F1F8\U0001F1FB':{'en':':El_Salvador:','status':fully_qualified,'E':2,'alias':[':flag_for_El_Salvador:', ':el_salvador:'],'de':':flagge_el_salvador:','es':':bandera_el_salvador:','fr':':drapeau_salvador:','ja':':旗_エルサルバドル:','ko':':깃발_엘살바도르:','pt':':bandeira_el_salvador:','it':':bandiera_el_salvador:','fa':':پرچم_السالوادور:','id':':bendera_el_salvador:','zh':':萨尔瓦多:','ru':':флаг_Сальвадор:','tr':':bayrak_el_salvador:','ar':':علم_السلفادور:'},'\U0001F3F4\U000E0067\U000E0062\U000E0065\U000E006E\U000E0067\U000E007F':{'en':':England:','status':fully_qualified,'E':5,'alias':[':england:'],'de':':flagge_england:','es':':bandera_inglaterra:','fr':':drapeau_angleterre:','ja':':旗_イングランド:','ko':':깃발_잉글랜드:','pt':':bandeira_inglaterra:','it':':bandiera_inghilterra:','fa':':پرچم_انگلستان:','id':':bendera_inggris:','zh':':英格兰:','ru':':флаг_Англия:','tr':':bayrak_ingiltere:','ar':':علم_إنجلترا:'},'\U0001F1EC\U0001F1F6':{'en':':Equatorial_Guinea:','status':fully_qualified,'E':2,'alias':[':flag_for_Equatorial_Guinea:', ':equatorial_guinea:'],'de':':flagge_äquatorialguinea:','es':':bandera_guinea_ecuatorial:','fr':':drapeau_guinée_équatoriale:','ja':':旗_赤道ギニア:','ko':':깃발_적도_기니:','pt':':bandeira_guiné_equatorial:','it':':bandiera_guinea_equatoriale:','fa':':پرچم_گینهٔ_استوایی:','id':':bendera_guinea_ekuatorial:','zh':':赤道几内亚:','ru':':флаг_Экваториальная_Гвинея:','tr':':bayrak_ekvator_ginesi:','ar':':علم_غينيا_الاستوائية:'},'\U0001F1EA\U0001F1F7':{'en':':Eritrea:','status':fully_qualified,'E':2,'alias':[':flag_for_Eritrea:', ':eritrea:'],'de':':flagge_eritrea:','es':':bandera_eritrea:','fr':':drapeau_érythrée:','ja':':旗_エリトリア:','ko':':깃발_에리트리아:','pt':':bandeira_eritreia:','it':':bandiera_eritrea:','fa':':پرچم_اریتره:','id':':bendera_eritrea:','zh':':厄立特里亚:','ru':':флаг_Эритрея:','tr':':bayrak_eritre:','ar':':علم_إريتريا:'},'\U0001F1EA\U0001F1EA':{'en':':Estonia:','status':fully_qualified,'E':2,'alias':[':flag_for_Estonia:', ':estonia:'],'de':':flagge_estland:','es':':bandera_estonia:','fr':':drapeau_estonie:','ja':':旗_エストニア:','ko':':깃발_에스토니아:','pt':':bandeira_estônia:','it':':bandiera_estonia:','fa':':پرچم_استونی:','id':':bendera_estonia:','zh':':爱沙尼亚:','ru':':флаг_Эстония:','tr':':bayrak_estonya:','ar':':علم_إستونيا:'},'\U0001F1F8\U0001F1FF':{'en':':Eswatini:','status':fully_qualified,'E':2,'alias':[':flag_for_Swaziland:', ':swaziland:'],'de':':flagge_eswatini:','es':':bandera_esuatini:','fr':':drapeau_eswatini:','ja':':旗_エスワティニ:','ko':':깃발_에스와티니:','pt':':bandeira_essuatíni:','it':':bandiera_eswatini:','fa':':پرچم_اسواتینی:','id':':bendera_eswatini:','zh':':斯威士兰:','ru':':флаг_Эсватини:','tr':':bayrak_esvatini:','ar':':علم_إسواتيني:'},'\U0001F1EA\U0001F1F9':{'en':':Ethiopia:','status':fully_qualified,'E':2,'alias':[':flag_for_Ethiopia:', ':ethiopia:'],'de':':flagge_äthiopien:','es':':bandera_etiopía:','fr':':drapeau_éthiopie:','ja':':旗_エチオピア:','ko':':깃발_에티오피아:','pt':':bandeira_etiópia:','it':':bandiera_etiopia:','fa':':پرچم_اتیوپی:','id':':bendera_etiopia:','zh':':埃塞俄比亚:','ru':':флаг_Эфиопия:','tr':':bayrak_etiyopya:','ar':':علم_إثيوبيا:'},'\U0001F1EA\U0001F1FA':{'en':':European_Union:','status':fully_qualified,'E':2,'alias':[':flag_for_European_Union:', ':eu:', ':european_union:'],'de':':flagge_europäische_union:','es':':bandera_unión_europea:','fr':':drapeau_union_européenne:','ja':':旗_欧州連合:','ko':':깃발_유럽_연합:','pt':':bandeira_união_europeia:','it':':bandiera_unione_europea:','fa':':پرچم_اتحادیهٔ_اروپا:','id':':bendera_uni_eropa:','zh':':欧盟:','ru':':флаг_Европейский_союз:','tr':':bayrak_avrupa_birliği:','ar':':علم_الاتحاد_الأوروبي:'},'\U0001F193':{'en':':FREE_button:','status':fully_qualified,'E':0.6,'alias':[':free:', ':free_button:'],'de':':wort_free_in_blauem_quadrat:','es':':botón_free:','fr':':bouton_gratuit:','ja':':freeマーク:','ko':':프리:','pt':':botão_free:','it':':pulsante_free:','fa':':دکمهٔ_free:','id':':tombol_free:','zh':':FREE按钮:','ru':':значок_«бесплатно»:','tr':':free_düğmesi:','ar':':زر_مجاني:'},'\U0001F1EB\U0001F1F0':{'en':':Falkland_Islands:','status':fully_qualified,'E':2,'alias':[':flag_for_Falkland_Islands:', ':falkland_islands:'],'de':':flagge_falklandinseln:','es':':bandera_islas_malvinas:','fr':':drapeau_îles_malouines:','ja':':旗_フォークランド諸島:','ko':':깃발_포클랜드_제도:','pt':':bandeira_ilhas_malvinas:','it':':bandiera_isole_falkland:','fa':':پرچم_جزایر_فالکلند:','id':':bendera_kepulauan_falkland:','zh':':福克兰群岛:','ru':':флаг_Фолклендские_о_ва:','tr':':bayrak_falkland_adaları:','ar':':علم_جزر_فوكلاند:'},'\U0001F1EB\U0001F1F4':{'en':':Faroe_Islands:','status':fully_qualified,'E':2,'alias':[':flag_for_Faroe_Islands:', ':faroe_islands:'],'de':':flagge_färöer:','es':':bandera_islas_feroe:','fr':':drapeau_îles_féroé:','ja':':旗_フェロー諸島:','ko':':깃발_페로_제도:','pt':':bandeira_ilhas_faroé:','it':':bandiera_isole_fær_øer:','fa':':پرچم_جزایر_فارو:','id':':bendera_kepulauan_faroe:','zh':':法罗群岛:','ru':':флаг_Фарерские_о_ва:','tr':':bayrak_faroe_adaları:','ar':':علم_جزر_فارو:'},'\U0001F1EB\U0001F1EF':{'en':':Fiji:','status':fully_qualified,'E':2,'alias':[':flag_for_Fiji:', ':fiji:'],'de':':flagge_fidschi:','es':':bandera_fiyi:','fr':':drapeau_fidji:','ja':':旗_フィジー:','ko':':깃발_피지:','pt':':bandeira_fiji:','it':':bandiera_figi:','fa':':پرچم_فیجی:','id':':bendera_fiji:','zh':':斐济:','ru':':флаг_Фиджи:','tr':':bayrak_fiji:','ar':':علم_فيجي:'},'\U0001F1EB\U0001F1EE':{'en':':Finland:','status':fully_qualified,'E':2,'alias':[':flag_for_Finland:', ':finland:'],'de':':flagge_finnland:','es':':bandera_finlandia:','fr':':drapeau_finlande:','ja':':旗_フィンランド:','ko':':깃발_핀란드:','pt':':bandeira_finlândia:','it':':bandiera_finlandia:','fa':':پرچم_فنلاند:','id':':bendera_finlandia:','zh':':芬兰:','ru':':флаг_Финляндия:','tr':':bayrak_finlandiya:','ar':':علم_فنلندا:'},'\U0001F1EB\U0001F1F7':{'en':':France:','status':fully_qualified,'E':0.6,'alias':[':flag_for_France:', ':fr:'],'de':':flagge_frankreich:','es':':bandera_francia:','fr':':drapeau_france:','ja':':旗_フランス:','ko':':깃발_프랑스:','pt':':bandeira_frança:','it':':bandiera_francia:','fa':':پرچم_فرانسه:','id':':bendera_prancis:','zh':':法国:','ru':':флаг_Франция:','tr':':bayrak_fransa:','ar':':علم_فرنسا:'},'\U0001F1EC\U0001F1EB':{'en':':French_Guiana:','status':fully_qualified,'E':2,'alias':[':flag_for_French_Guiana:', ':french_guiana:'],'de':':flagge_französisch-guayana:','es':':bandera_guayana_francesa:','fr':':drapeau_guyane_française:','ja':':旗_仏領ギアナ:','ko':':깃발_프랑스령_기아나:','pt':':bandeira_guiana_francesa:','it':':bandiera_guyana_francese:','fa':':پرچم_گویان_فرانسه:','id':':bendera_guyana_prancis:','zh':':法属圭亚那:','ru':':флаг_Французская_Гвиана:','tr':':bayrak_fransız_guyanası:','ar':':علم_غويانا_الفرنسية:'},'\U0001F1F5\U0001F1EB':{'en':':French_Polynesia:','status':fully_qualified,'E':2,'alias':[':flag_for_French_Polynesia:', ':french_polynesia:'],'de':':flagge_französisch-polynesien:','es':':bandera_polinesia_francesa:','fr':':drapeau_polynésie_française:','ja':':旗_仏領ポリネシア:','ko':':깃발_프랑스령_폴리네시아:','pt':':bandeira_polinésia_francesa:','it':':bandiera_polinesia_francese:','fa':':پرچم_پلی_نزی_فرانسه:','id':':bendera_polinesia_prancis:','zh':':法属波利尼西亚:','ru':':флаг_Французская_Полинезия:','tr':':bayrak_fransız_polinezyası:','ar':':علم_بولينيزيا_الفرنسية:'},'\U0001F1F9\U0001F1EB':{'en':':French_Southern_Territories:','status':fully_qualified,'E':2,'alias':[':flag_for_French_Southern_Territories:', ':french_southern_territories:'],'de':':flagge_französische_süd-_und_antarktisgebiete:','es':':bandera_territorios_australes_franceses:','fr':':drapeau_terres_australes_françaises:','ja':':旗_仏領極南諸島:','ko':':깃발_프랑스령_남방_지역:','pt':':bandeira_territórios_franceses_do_sul:','it':':bandiera_terre_australi_francesi:','fa':':پرچم_سرزمین_های_جنوبی_فرانسه:','id':':bendera_wilayah_selatan_prancis:','zh':':法属南部领地:','ru':':флаг_Французские_Южные_территории:','tr':':bayrak_fransız_güney_toprakları:','ar':':علم_الأقاليم_الجنوبية_الفرنسية:'},'\U0001F1EC\U0001F1E6':{'en':':Gabon:','status':fully_qualified,'E':2,'alias':[':flag_for_Gabon:', ':gabon:'],'de':':flagge_gabun:','es':':bandera_gabón:','fr':':drapeau_gabon:','ja':':旗_ガボン:','ko':':깃발_가봉:','pt':':bandeira_gabão:','it':':bandiera_gabon:','fa':':پرچم_گابن:','id':':bendera_gabon:','zh':':加蓬:','ru':':флаг_Габон:','tr':':bayrak_gabon:','ar':':علم_الغابون:'},'\U0001F1EC\U0001F1F2':{'en':':Gambia:','status':fully_qualified,'E':2,'alias':[':flag_for_Gambia:', ':gambia:'],'de':':flagge_gambia:','es':':bandera_gambia:','fr':':drapeau_gambie:','ja':':旗_ガンビア:','ko':':깃발_감비아:','pt':':bandeira_gâmbia:','it':':bandiera_gambia:','fa':':پرچم_گامبیا:','id':':bendera_gambia:','zh':':冈比亚:','ru':':флаг_Гамбия:','tr':':bayrak_gambiya:','ar':':علم_غامبيا:'},'\U0000264A':{'en':':Gemini:','status':fully_qualified,'E':0.6,'alias':[':gemini:'],'variant':True,'de':':zwillinge_(sternzeichen):','es':':géminis:','fr':':gémeaux:','ja':':ふたご座:','ko':':쌍둥이_자리:','pt':':signo_de_gêmeos:','it':':segno_zodiacale_dei_gemelli:','fa':':برج_جوزا:','id':':gemini:','zh':':双子座:','ru':':знак_зодиака_Близнецы:','tr':':ikizler_burcu:','ar':':الجوزاء:'},'\U0001F1EC\U0001F1EA':{'en':':Georgia:','status':fully_qualified,'E':2,'alias':[':flag_for_Georgia:', ':georgia:'],'de':':flagge_georgien:','es':':bandera_georgia:','fr':':drapeau_géorgie:','ja':':旗_ジョージア:','ko':':깃발_조지아:','pt':':bandeira_geórgia:','it':':bandiera_georgia:','fa':':پرچم_گرجستان:','id':':bendera_georgia:','zh':':格鲁吉亚:','ru':':флаг_Грузия:','tr':':bayrak_gürcistan:','ar':':علم_جورجيا:'},'\U0001F1E9\U0001F1EA':{'en':':Germany:','status':fully_qualified,'E':0.6,'alias':[':flag_for_Germany:', ':de:'],'de':':flagge_deutschland:','es':':bandera_alemania:','fr':':drapeau_allemagne:','ja':':旗_ドイツ:','ko':':깃발_독일:','pt':':bandeira_alemanha:','it':':bandiera_germania:','fa':':پرچم_آلمان:','id':':bendera_jerman:','zh':':德国:','ru':':флаг_Германия:','tr':':bayrak_almanya:','ar':':علم_ألمانيا:'},'\U0001F1EC\U0001F1ED':{'en':':Ghana:','status':fully_qualified,'E':2,'alias':[':flag_for_Ghana:', ':ghana:'],'de':':flagge_ghana:','es':':bandera_ghana:','fr':':drapeau_ghana:','ja':':旗_ガーナ:','ko':':깃발_가나:','pt':':bandeira_gana:','it':':bandiera_ghana:','fa':':پرچم_غنا:','id':':bendera_ghana:','zh':':加纳:','ru':':флаг_Гана:','tr':':bayrak_gana:','ar':':علم_غانا:'},'\U0001F1EC\U0001F1EE':{'en':':Gibraltar:','status':fully_qualified,'E':2,'alias':[':flag_for_Gibraltar:', ':gibraltar:'],'de':':flagge_gibraltar:','es':':bandera_gibraltar:','fr':':drapeau_gibraltar:','ja':':旗_ジブラルタル:','ko':':깃발_지브롤터:','pt':':bandeira_gibraltar:','it':':bandiera_gibilterra:','fa':':پرچم_جبل_الطارق:','id':':bendera_gibraltar:','zh':':直布罗陀:','ru':':флаг_Гибралтар:','tr':':bayrak_cebelitarık:','ar':':علم_جبل_طارق:'},'\U0001F1EC\U0001F1F7':{'en':':Greece:','status':fully_qualified,'E':2,'alias':[':flag_for_Greece:', ':greece:'],'de':':flagge_griechenland:','es':':bandera_grecia:','fr':':drapeau_grèce:','ja':':旗_ギリシャ:','ko':':깃발_그리스:','pt':':bandeira_grécia:','it':':bandiera_grecia:','fa':':پرچم_یونان:','id':':bendera_yunani:','zh':':希腊:','ru':':флаг_Греция:','tr':':bayrak_yunanistan:','ar':':علم_اليونان:'},'\U0001F1EC\U0001F1F1':{'en':':Greenland:','status':fully_qualified,'E':2,'alias':[':flag_for_Greenland:', ':greenland:'],'de':':flagge_grönland:','es':':bandera_groenlandia:','fr':':drapeau_groenland:','ja':':旗_グリーンランド:','ko':':깃발_그린란드:','pt':':bandeira_groenlândia:','it':':bandiera_groenlandia:','fa':':پرچم_گرینلند:','id':':bendera_greenland:','zh':':格陵兰:','ru':':флаг_Гренландия:','tr':':bayrak_grönland:','ar':':علم_غرينلاند:'},'\U0001F1EC\U0001F1E9':{'en':':Grenada:','status':fully_qualified,'E':2,'alias':[':flag_for_Grenada:', ':grenada:'],'de':':flagge_grenada:','es':':bandera_granada:','fr':':drapeau_grenade:','ja':':旗_グレナダ:','ko':':깃발_그레나다:','pt':':bandeira_granada:','it':':bandiera_grenada:','fa':':پرچم_گرنادا:','id':':bendera_grenada:','zh':':格林纳达:','ru':':флаг_Гренада:','tr':':bayrak_grenada:','ar':':علم_غرينادا:'},'\U0001F1EC\U0001F1F5':{'en':':Guadeloupe:','status':fully_qualified,'E':2,'alias':[':flag_for_Guadeloupe:', ':guadeloupe:'],'de':':flagge_guadeloupe:','es':':bandera_guadalupe:','fr':':drapeau_guadeloupe:','ja':':旗_グアドループ:','ko':':깃발_과들루프:','pt':':bandeira_guadalupe:','it':':bandiera_guadalupa:','fa':':پرچم_گوادلوپ:','id':':bendera_guadeloupe:','zh':':瓜德罗普:','ru':':флаг_Гваделупа:','tr':':bayrak_guadeloupe:','ar':':علم_غوادلوب:'},'\U0001F1EC\U0001F1FA':{'en':':Guam:','status':fully_qualified,'E':2,'alias':[':flag_for_Guam:', ':guam:'],'de':':flagge_guam:','es':':bandera_guam:','fr':':drapeau_guam:','ja':':旗_グアム:','ko':':깃발_괌:','pt':':bandeira_guam:','it':':bandiera_guam:','fa':':پرچم_گوام:','id':':bendera_guam:','zh':':关岛:','ru':':флаг_Гуам:','tr':':bayrak_guam:','ar':':علم_غوام:'},'\U0001F1EC\U0001F1F9':{'en':':Guatemala:','status':fully_qualified,'E':2,'alias':[':flag_for_Guatemala:', ':guatemala:'],'de':':flagge_guatemala:','es':':bandera_guatemala:','fr':':drapeau_guatemala:','ja':':旗_グアテマラ:','ko':':깃발_과테말라:','pt':':bandeira_guatemala:','it':':bandiera_guatemala:','fa':':پرچم_گواتمالا:','id':':bendera_guatemala:','zh':':危地马拉:','ru':':флаг_Гватемала:','tr':':bayrak_guatemala:','ar':':علم_غواتيمالا:'},'\U0001F1EC\U0001F1EC':{'en':':Guernsey:','status':fully_qualified,'E':2,'alias':[':flag_for_Guernsey:', ':guernsey:'],'de':':flagge_guernsey:','es':':bandera_guernesey:','fr':':drapeau_guernesey:','ja':':旗_ガーンジー:','ko':':깃발_건지:','pt':':bandeira_guernsey:','it':':bandiera_guernsey:','fa':':پرچم_گرنزی:','id':':bendera_guernsey:','zh':':根西岛:','ru':':флаг_Гернси:','tr':':bayrak_guernsey:','ar':':علم_غيرنزي:'},'\U0001F1EC\U0001F1F3':{'en':':Guinea:','status':fully_qualified,'E':2,'alias':[':flag_for_Guinea:', ':guinea:'],'de':':flagge_guinea:','es':':bandera_guinea:','fr':':drapeau_guinée:','ja':':旗_ギニア:','ko':':깃발_기니:','pt':':bandeira_guiné:','it':':bandiera_guinea:','fa':':پرچم_گینه:','id':':bendera_guinea:','zh':':几内亚:','ru':':флаг_Гвинея:','tr':':bayrak_gine:','ar':':علم_غينيا:'},'\U0001F1EC\U0001F1FC':{'en':':Guinea-Bissau:','status':fully_qualified,'E':2,'alias':[':flag_for_Guinea_Bissau:', ':guinea_bissau:'],'de':':flagge_guinea-bissau:','es':':bandera_guinea-bisáu:','fr':':drapeau_guinée-bissau:','ja':':旗_ギニアビサウ:','ko':':깃발_기니비사우:','pt':':bandeira_guiné-bissau:','it':':bandiera_guinea-bissau:','fa':':پرچم_گینهٔ_بیسائو:','id':':bendera_guinea-bissau:','zh':':几内亚比绍:','ru':':флаг_Гвинея_Бисау:','tr':':bayrak_gine-bissau:','ar':':علم_غينيا_بيساو:'},'\U0001F1EC\U0001F1FE':{'en':':Guyana:','status':fully_qualified,'E':2,'alias':[':flag_for_Guyana:', ':guyana:'],'de':':flagge_guyana:','es':':bandera_guyana:','fr':':drapeau_guyana:','ja':':旗_ガイアナ:','ko':':깃발_가이아나:','pt':':bandeira_guiana:','it':':bandiera_guyana:','fa':':پرچم_گویان:','id':':bendera_guyana:','zh':':圭亚那:','ru':':флаг_Гайана:','tr':':bayrak_guyana:','ar':':علم_غيانا:'},'\U0001F1ED\U0001F1F9':{'en':':Haiti:','status':fully_qualified,'E':2,'alias':[':flag_for_Haiti:', ':haiti:'],'de':':flagge_haiti:','es':':bandera_haití:','fr':':drapeau_haïti:','ja':':旗_ハイチ:','ko':':깃발_아이티:','pt':':bandeira_haiti:','it':':bandiera_haiti:','fa':':پرچم_هائیتی:','id':':bendera_haiti:','zh':':海地:','ru':':флаг_Гаити:','tr':':bayrak_haiti:','ar':':علم_هايتي:'},'\U0001F1ED\U0001F1F2':{'en':':Heard_&_McDonald_Islands:','status':fully_qualified,'E':2,'alias':[':flag_for_Heard_&_McDonald_Islands:', ':heard_mcdonald_islands:'],'de':':flagge_heard_und_mcdonaldinseln:','es':':bandera_islas_heard_y_mcdonald:','fr':':drapeau_îles_heard-et-macdonald:','ja':':旗_ハード島・マクドナルド諸島:','ko':':깃발_허드_맥도널드_제도:','pt':':bandeira_ilhas_heard_e_mcdonald:','it':':bandiera_isole_heard_e_mcdonald:','fa':':پرچم_هرد_و_جزایر_مک_دونالد:','id':':bendera_pulau_heard_dan_kepulauan_mcdonald:','zh':':赫德岛和麦克唐纳群岛:','ru':':флаг_о_ва_Херд_и_Макдональд:','tr':':bayrak_heard_adası_ve_mcdonald_adaları:','ar':':علم_جزيرة_هيرد_وجزر_ماكدونالد:'},'\U0001F1ED\U0001F1F3':{'en':':Honduras:','status':fully_qualified,'E':2,'alias':[':flag_for_Honduras:', ':honduras:'],'de':':flagge_honduras:','es':':bandera_honduras:','fr':':drapeau_honduras:','ja':':旗_ホンジュラス:','ko':':깃발_온두라스:','pt':':bandeira_honduras:','it':':bandiera_honduras:','fa':':پرچم_هندوراس:','id':':bendera_honduras:','zh':':洪都拉斯:','ru':':флаг_Гондурас:','tr':':bayrak_honduras:','ar':':علم_هندوراس:'},'\U0001F1ED\U0001F1F0':{'en':':Hong_Kong_SAR_China:','status':fully_qualified,'E':2,'alias':[':flag_for_Hong_Kong:', ':hong_kong:'],'de':':flagge_sonderverwaltungsregion_hongkong:','es':':bandera_rae_de_hong_kong_(china):','fr':':drapeau_r.a.s._chinoise_de_hong_kong:','ja':':旗_中華人民共和国香港特別行政区:','ko':':깃발_홍콩(중국_특별행정구):','pt':':bandeira_hong_kong_rae_da_china:','it':':bandiera_ras_di_hong_kong:','fa':':پرچم_هنگ_کنگ_منطقهٔ_ویژهٔ_اداری_چین:','id':':bendera_hong_kong_dak_tiongkok:','zh':':香港:','ru':':флаг_Гонконг_(САР):','tr':':bayrak_çin_hong_kong_öib:','ar':':علم_هونغ_كونغ_الصينية_(منطقة_إدارية_خاصة):'},'\U0001F1ED\U0001F1FA':{'en':':Hungary:','status':fully_qualified,'E':2,'alias':[':flag_for_Hungary:', ':hungary:'],'de':':flagge_ungarn:','es':':bandera_hungría:','fr':':drapeau_hongrie:','ja':':旗_ハンガリー:','ko':':깃발_헝가리:','pt':':bandeira_hungria:','it':':bandiera_ungheria:','fa':':پرچم_مجارستان:','id':':bendera_hungaria:','zh':':匈牙利:','ru':':флаг_Венгрия:','tr':':bayrak_macaristan:','ar':':علم_هنغاريا:'},'\U0001F194':{'en':':ID_button:','status':fully_qualified,'E':0.6,'alias':[':id:', ':id_button:'],'de':':großbuchstaben_id_in_lila_quadrat:','es':':símbolo_de_identificación:','fr':':bouton_identifiant:','ja':':idマーク:','ko':':아이디:','pt':':botão_id:','it':':pulsante_id:','fa':':دکمهٔ_شناسایی:','id':':tombol_id:','zh':':ID按钮:','ru':':значок_«идентификация»:','tr':':id_düğmesi:','ar':':زر_معرف_الهوية:'},'\U0001F1EE\U0001F1F8':{'en':':Iceland:','status':fully_qualified,'E':2,'alias':[':flag_for_Iceland:', ':iceland:'],'de':':flagge_island:','es':':bandera_islandia:','fr':':drapeau_islande:','ja':':旗_アイスランド:','ko':':깃발_아이슬란드:','pt':':bandeira_islândia:','it':':bandiera_islanda:','fa':':پرچم_ایسلند:','id':':bendera_islandia:','zh':':冰岛:','ru':':флаг_Исландия:','tr':':bayrak_izlanda:','ar':':علم_آيسلندا:'},'\U0001F1EE\U0001F1F3':{'en':':India:','status':fully_qualified,'E':2,'alias':[':flag_for_India:', ':india:'],'de':':flagge_indien:','es':':bandera_india:','fr':':drapeau_inde:','ja':':旗_インド:','ko':':깃발_인도:','pt':':bandeira_índia:','it':':bandiera_india:','fa':':پرچم_هند:','id':':bendera_india:','zh':':印度:','ru':':флаг_Индия:','tr':':bayrak_hindistan:','ar':':علم_الهند:'},'\U0001F1EE\U0001F1E9':{'en':':Indonesia:','status':fully_qualified,'E':2,'alias':[':flag_for_Indonesia:', ':indonesia:'],'de':':flagge_indonesien:','es':':bandera_indonesia:','fr':':drapeau_indonésie:','ja':':旗_インドネシア:','ko':':깃발_인도네시아:','pt':':bandeira_indonésia:','it':':bandiera_indonesia:','fa':':پرچم_اندونزی:','id':':bendera_indonesia:','zh':':印度尼西亞:','ru':':флаг_Индонезия:','tr':':bayrak_endonezya:','ar':':علم_إندونيسيا:'},'\U0001F1EE\U0001F1F7':{'en':':Iran:','status':fully_qualified,'E':2,'alias':[':flag_for_Iran:', ':iran:'],'de':':flagge_iran:','es':':bandera_irán:','fr':':drapeau_iran:','ja':':旗_イラン:','ko':':깃발_이란:','pt':':bandeira_irã:','it':':bandiera_iran:','fa':':پرچم_ایران:','id':':bendera_iran:','zh':':伊朗:','ru':':флаг_Иран:','tr':':bayrak_iran:','ar':':علم_إيران:'},'\U0001F1EE\U0001F1F6':{'en':':Iraq:','status':fully_qualified,'E':2,'alias':[':flag_for_Iraq:', ':iraq:'],'de':':flagge_irak:','es':':bandera_irak:','fr':':drapeau_irak:','ja':':旗_イラク:','ko':':깃발_이라크:','pt':':bandeira_iraque:','it':':bandiera_iraq:','fa':':پرچم_عراق:','id':':bendera_irak:','zh':':伊拉克:','ru':':флаг_Ирак:','tr':':bayrak_irak:','ar':':علم_العراق:'},'\U0001F1EE\U0001F1EA':{'en':':Ireland:','status':fully_qualified,'E':2,'alias':[':flag_for_Ireland:', ':ireland:'],'de':':flagge_irland:','es':':bandera_irlanda:','fr':':drapeau_irlande:','ja':':旗_アイルランド:','ko':':깃발_아일랜드:','pt':':bandeira_irlanda:','it':':bandiera_irlanda:','fa':':پرچم_ایرلند:','id':':bendera_irlandia:','zh':':爱尔兰:','ru':':флаг_Ирландия:','tr':':bayrak_irlanda:','ar':':علم_أيرلندا:'},'\U0001F1EE\U0001F1F2':{'en':':Isle_of_Man:','status':fully_qualified,'E':2,'alias':[':flag_for_Isle_of_Man:', ':isle_of_man:'],'de':':flagge_isle_of_man:','es':':bandera_isla_de_man:','fr':':drapeau_île_de_man:','ja':':旗_マン島:','ko':':깃발_맨섬:','pt':':bandeira_ilha_de_man:','it':':bandiera_isola_di_man:','fa':':پرچم_جزیرهٔ_من:','id':':bendera_pulau_man:','zh':':马恩岛:','ru':':флаг_о_в_Мэн:','tr':':bayrak_man_adası:','ar':':علم_جزيرة_مان:'},'\U0001F1EE\U0001F1F1':{'en':':Israel:','status':fully_qualified,'E':2,'alias':[':flag_for_Israel:', ':israel:'],'de':':flagge_israel:','es':':bandera_israel:','fr':':drapeau_israël:','ja':':旗_イスラエル:','ko':':깃발_이스라엘:','pt':':bandeira_israel:','it':':bandiera_israele:','fa':':پرچم_اسرائیل:','id':':bendera_israel:','zh':':以色列:','ru':':флаг_Израиль:','tr':':bayrak_israil:','ar':':علم_إسرائيل:'},'\U0001F1EE\U0001F1F9':{'en':':Italy:','status':fully_qualified,'E':0.6,'alias':[':flag_for_Italy:', ':it:'],'de':':flagge_italien:','es':':bandera_italia:','fr':':drapeau_italie:','ja':':旗_イタリア:','ko':':깃발_이탈리아:','pt':':bandeira_itália:','it':':bandiera_italia:','fa':':پرچم_ایتالیا:','id':':bendera_italia:','zh':':意大利:','ru':':флаг_Италия:','tr':':bayrak_italya:','ar':':علم_إيطاليا:'},'\U0001F1EF\U0001F1F2':{'en':':Jamaica:','status':fully_qualified,'E':2,'alias':[':flag_for_Jamaica:', ':jamaica:'],'de':':flagge_jamaika:','es':':bandera_jamaica:','fr':':drapeau_jamaïque:','ja':':旗_ジャマイカ:','ko':':깃발_자메이카:','pt':':bandeira_jamaica:','it':':bandiera_giamaica:','fa':':پرچم_جامائیکا:','id':':bendera_jamaika:','zh':':牙买加:','ru':':флаг_Ямайка:','tr':':bayrak_jamaika:','ar':':علم_جامايكا:'},'\U0001F1EF\U0001F1F5':{'en':':Japan:','status':fully_qualified,'E':0.6,'alias':[':flag_for_Japan:', ':jp:'],'de':':flagge_japan:','es':':bandera_japón:','fr':':drapeau_japon:','ja':':旗_日本:','ko':':깃발_일본:','pt':':bandeira_japão:','it':':bandiera_giappone:','fa':':پرچم_ژاپن:','id':':bendera_jepang:','zh':':日本:','ru':':флаг_Япония:','tr':':bayrak_japonya:','ar':':علم_اليابان:'},'\U0001F251':{'en':':Japanese_acceptable_button:','status':fully_qualified,'E':0.6,'alias':[':accept:', ':japanese_acceptable_button:'],'de':':schriftzeichen_für_akzeptieren:','es':':ideograma_japonés_para_aceptable:','fr':':bouton_accepter_en_japonais:','ja':':可マーク:','ko':':옳을_가:','pt':':botão_japonês_de_“aceitável”:','it':':ideogramma_giapponese_di_“accettabile”:','fa':':دکمۀ_«قابل_قبول»_به_ژاپنی:','id':':tombol_jepang_dapat_diterima:','zh':':可:','ru':':допустимость:','tr':':japonca_kabul_edilebilir_düğmesi:','ar':':الزر_/مقبول/_باليابانية:'},'\U0001F238':{'en':':Japanese_application_button:','status':fully_qualified,'E':0.6,'alias':[':u7533:', ':japanese_application_button:'],'de':':schriftzeichen_für_anwenden:','es':':ideograma_japonés_para_aplicación:','fr':':bouton_application_en_japonais:','ja':':申マーク:','ko':':거듭_신:','pt':':botão_japonês_de_“aplicação”:','it':':ideogramma_giapponese_di_“candidatura”:','fa':':دکمۀ_«تقاضانامه»_به_ژاپنی:','id':':tombol_jepang_lamaran_kerja:','zh':':申:','ru':':заявка:','tr':':japonca_uygulama_düğmesi:','ar':':الزر_/تطبيق/_باليابانية:'},'\U0001F250':{'en':':Japanese_bargain_button:','status':fully_qualified,'E':0.6,'alias':[':ideograph_advantage:', ':japanese_bargain_button:'],'de':':schriftzeichen_für_schnäppchen:','es':':ideograma_japonés_para_ganga:','fr':':bouton_bonne_affaire_en_japonais:','ja':':マル得:','ko':':얻을_득:','pt':':botão_japonês_de_“barganha”:','it':':ideogramma_giapponese_di_“occasione”:','fa':':دکمهٔ_«قیمت_مناسب»_به_ژاپنی:','id':':karakter_keunggulan_dalam_lingkaran:','zh':':得:','ru':':удачная_сделка:','tr':':japonca_avantaj_düğmesi:','ar':':الزر_/صفقة/_باليابانية:'},'\U0001F3EF':{'en':':Japanese_castle:','status':fully_qualified,'E':0.6,'alias':[':japanese_castle:'],'de':':japanisches_schloss:','es':':castillo_japonés:','fr':':château_japonais:','ja':':城:','ko':':일본_성:','pt':':castelo_japonês:','it':':castello_giapponese:','fa':':قلعهٔ_ژاپنی:','id':':kastel_jepang:','zh':':日本城堡:','ru':':японский_замок:','tr':':japon_şatosu:','ar':':قلعة_يابانية:'},'\U00003297\U0000FE0F':{'en':':Japanese_congratulations_button:','status':fully_qualified,'E':0.6,'alias':[':congratulations:', ':japanese_congratulations_button:'],'variant':True,'de':':schriftzeichen_für_gratulation:','es':':ideograma_japonés_para_enhorabuena:','fr':':bouton_félicitations_en_japonais:','ja':':祝マーク:','ko':':원_안의_축하_표의문자:','pt':':botão_japonês_de_“parabéns”:','it':':ideogramma_giapponese_di_“congratulazioni”:','fa':':دکمه_تبریک_به_ژاپنی:','id':':tombol_jepang_selamat:','zh':':祝:','ru':':поздравление:','tr':':japonca_tebrikler_düğmesi:','ar':':الزر_/تهانينا/_باليابانية:'},'\U00003297':{'en':':Japanese_congratulations_button:','status':unqualified,'E':0.6,'alias':[':congratulations:', ':japanese_congratulations_button:'],'variant':True,'de':':schriftzeichen_für_gratulation:','es':':ideograma_japonés_para_enhorabuena:','fr':':bouton_félicitations_en_japonais:','ja':':祝マーク:','ko':':원_안의_축하_표의문자:','pt':':botão_japonês_de_“parabéns”:','it':':ideogramma_giapponese_di_“congratulazioni”:','fa':':دکمهٔ_«تبریک»_به_ژاپنی:','id':':tombol_jepang_selamat:','zh':':祝:','ru':':поздравление:','tr':':japonca_tebrikler_düğmesi:','ar':':الزر_/تهانينا/_باليابانية:'},'\U0001F239':{'en':':Japanese_discount_button:','status':fully_qualified,'E':0.6,'alias':[':u5272:', ':japanese_discount_button:'],'de':':schriftzeichen_für_rabatt:','es':':ideograma_japonés_para_descuento:','fr':':bouton_réduction_en_japonais:','ja':':割マーク:','ko':':나눌_할:','pt':':botão_japonês_de_“desconto”:','it':':ideogramma_giapponese_di_“sconto”:','fa':':دکمۀ_«تخفیف»_به_ژاپنی:','id':':tombol_jepang_diskon:','zh':':割:','ru':':скидка:','tr':':japonca_indirim_düğmesi:','ar':':الزر_/خصم/_باليابانية:'},'\U0001F38E':{'en':':Japanese_dolls:','status':fully_qualified,'E':0.6,'alias':[':dolls:', ':japanese_dolls:'],'de':':japanische_puppen:','es':':muñecas_japonesas:','fr':':poupées_japonaises:','ja':':ひな祭り:','ko':':일본_인형:','pt':':bonecas_japonesas:','it':':bambole_giapponesi:','fa':':عروسک_های_ژاپنی:','id':':boneka_jepang:','zh':':日本人形:','ru':':японские_куклы:','tr':':japon_oyuncak_bebekleri:','ar':':دميتان_يابانيتان:'},'\U0001F21A':{'en':':Japanese_free_of_charge_button:','status':fully_qualified,'E':0.6,'alias':[':u7121:', ':japanese_free_of_charge_button:'],'variant':True,'de':':schriftzeichen_für_gratis:','es':':ideograma_japonés_para_gratis:','fr':':bouton_gratuit_en_japonais:','ja':':無マーク:','ko':':없을_무:','pt':':botão_japonês_de_“gratuito”:','it':':ideogramma_giapponese_di_“gratis”:','fa':':دکمۀ_«رایگان»_به_ژاپنی:','id':':tombol_jepang_gratis:','zh':':无:','ru':':бесплатно:','tr':':japonca_ücretsiz_düğmesi:','ar':':الزر_/مجانا/_باليابانية:'},'\U0001F201':{'en':':Japanese_here_button:','status':fully_qualified,'E':0.6,'alias':[':koko:', ':japanese_here_button:'],'de':':schriftzeichen_koko:','es':':ideograma_japonés_para_aquí:','fr':':bouton_ici_en_japonais:','ja':':ココのマーク:','ko':':일본어_여기_버튼:','pt':':botão_japonês_de_“aqui”:','it':':ideogramma_giapponese_per_“qui”:','fa':':دکمهٔ_«اینجا»_به_ژاپنی:','id':':tombol_jepang_di_sini:','zh':':这里:','ru':':значок_«здесь»:','tr':':japonca_burada_düğmesi:','ar':':هنا_بالياباني_في_مربع:'},'\U0001F237\U0000FE0F':{'en':':Japanese_monthly_amount_button:','status':fully_qualified,'E':0.6,'alias':[':u6708:', ':japanese_monthly_amount_button:'],'variant':True,'de':':schriftzeichen_für_monatsbetrag:','es':':ideograma_japonés_para_cantidad_mensual:','fr':':bouton_montant_mensuel_en_japonais:','ja':':月マーク:','ko':':달_월:','pt':':botão_japonês_de_“quantidade_mensal”:','it':':ideogramma_giapponese_di_“importo_mensile”:','fa':':دکمه_مقدار_ماهانه_به_ژاپنی:','id':':tombol_jepang_jumlah_bulanan:','zh':':月:','ru':':месяц:','tr':':japonca_aylık_ücret_düğmesi:','ar':':الزر_/مبلغ_شهري/_باليابانية:'},'\U0001F237':{'en':':Japanese_monthly_amount_button:','status':unqualified,'E':0.6,'alias':[':u6708:', ':japanese_monthly_amount_button:'],'variant':True,'de':':schriftzeichen_für_monatsbetrag:','es':':ideograma_japonés_para_cantidad_mensual:','fr':':bouton_montant_mensuel_en_japonais:','ja':':月マーク:','ko':':달_월:','pt':':botão_japonês_de_“quantidade_mensal”:','it':':ideogramma_giapponese_di_“importo_mensile”:','fa':':دکمهٔ_«مقدار_ماهانه»_به_ژاپنی:','id':':tombol_jepang_jumlah_bulanan:','zh':':月:','ru':':месяц:','tr':':japonca_aylık_ücret_düğmesi:','ar':':الزر_/مبلغ_شهري/_باليابانية:'},'\U0001F235':{'en':':Japanese_no_vacancy_button:','status':fully_qualified,'E':0.6,'alias':[':u6e80:', ':japanese_no_vacancy_button:'],'de':':schriftzeichen_für_kein_zimmer_frei:','es':':ideograma_japonés_para_completo:','fr':':bouton_complet_en_japonais:','ja':':満マーク:','ko':':찰_만:','pt':':botão_japonês_de_“sem_vagas”:','it':':ideogramma_giapponese_di_“nessun_posto_libero”:','fa':':دکمهٔ_«پُر»_به_ژاپنی:','id':':tombol_jepang_tidak_ada_lowongan:','zh':':满:','ru':':нет_мест:','tr':':japonca_yer_yok_düğmesi:','ar':':الزر_/غير_خال/_باليابانية:'},'\U0001F236':{'en':':Japanese_not_free_of_charge_button:','status':fully_qualified,'E':0.6,'alias':[':u6709:', ':japanese_not_free_of_charge_button:'],'de':':schriftzeichen_für_nicht_gratis:','es':':ideograma_japonés_para_de_pago:','fr':':bouton_pas_gratuit_en_japonais:','ja':':有マーク:','ko':':있을_유:','pt':':botão_japonês_de_“não_gratuito”:','it':':ideogramma_giapponese_di_“a_pagamento”:','fa':':دکمهٔ_غیرمجانی_به_ژاپنی:','id':':tombol_jepang_tidak_gratis:','zh':':有:','ru':':платно:','tr':':japonca_ücretsiz_değil_düğmesi:','ar':':الزر_/غير_مجاني/_باليابانية:'},'\U0001F23A':{'en':':Japanese_open_for_business_button:','status':fully_qualified,'E':0.6,'alias':[':u55b6:', ':japanese_open_for_business_button:'],'de':':schriftzeichen_für_geöffnet:','es':':ideograma_japonés_para_abierto:','fr':':bouton_ouvert_pour_affaires_en_japonais:','ja':':営マーク:','ko':':벼슬_관:','pt':':botão_japonês_de_“aberto_para_negócios”:','it':':ideogramma_giapponese_di_“aperto_al_pubblico”:','fa':':دکمۀ_«آماده_برای_کسب_وکار»_به_ژاپنی:','id':':tombol_jepang_siap_berbisnis:','zh':':营:','ru':':открыто:','tr':':japonca_hizmete_hazır_düğmesi:','ar':':الزر_/عمل/_باليابانية:'},'\U0001F234':{'en':':Japanese_passing_grade_button:','status':fully_qualified,'E':0.6,'alias':[':u5408:', ':japanese_passing_grade_button:'],'de':':schriftzeichen_für_note_zum_bestehen:','es':':ideograma_japonés_para_aprobado:','fr':':bouton_note_pour_réussir_en_japonais:','ja':':合マーク:','ko':':합할_합:','pt':':botão_japonês_de_“nota_mínima”:','it':':ideogramma_giapponese_di_“voto_di_sufficienza”:','fa':':دکمۀ_«نمرهٔ_قبولی»_به_ژاپنی:','id':':tombol_jepang_nilai_kelulusan:','zh':':合:','ru':':проходной_балл:','tr':':japonca_geçer_not_düğmesi:','ar':':الزر_/درجة_النجاح/_باليابانية:'},'\U0001F3E3':{'en':':Japanese_post_office:','status':fully_qualified,'E':0.6,'alias':[':post_office:', ':japanese_post_office:'],'de':':japanisches_postgebäude:','es':':oficina_de_correos_japonesa:','fr':':bureau_de_poste_japonais:','ja':':郵便局:','ko':':일본_우체국:','pt':':correio_japonês:','it':':ufficio_postale_giapponese:','fa':':پست_خانهٔ_ژاپنی:','id':':kantor_pos_jepang:','zh':':日本邮局:','ru':':японская_почта:','tr':':japon_postanesi:','ar':':مكتب_بريد_ياباني:'},'\U0001F232':{'en':':Japanese_prohibited_button:','status':fully_qualified,'E':0.6,'alias':[':u7981:', ':japanese_prohibited_button:'],'de':':schriftzeichen_für_verbieten:','es':':ideograma_japonés_para_prohibido:','fr':':bouton_interdit_en_japonais:','ja':':禁マーク:','ko':':금할_금:','pt':':botão_japonês_de_“proibido”:','it':':ideogramma_giapponese_di_“proibito”:','fa':':دکمۀ_«ممنوع»_به_ژاپنی:','id':':tombol_jepang_dilarang:','zh':':禁:','ru':':запрет:','tr':':japonca_yasak_düğmesi:','ar':':الزر_/ممنوع/_باليابانية:'},'\U0001F22F':{'en':':Japanese_reserved_button:','status':fully_qualified,'E':0.6,'alias':[':u6307:', ':japanese_reserved_button:'],'variant':True,'de':':schriftzeichen_für_reserviert:','es':':ideograma_japonés_para_reservado:','fr':':bouton_réservé_en_japonais:','ja':':指マーク:','ko':':가리킬_지:','pt':':botão_japonês_de_“reservado”:','it':':ideogramma_giapponese_di_“riservato”:','fa':':دکمۀ_«رزروشده»_به_ژاپنی:','id':':tombol_jepang_sudah_dipesan:','zh':':指:','ru':':забронировано:','tr':':japonca_rezerve_düğmesi:','ar':':الزر_/محجوز/_باليابانية:'},'\U00003299\U0000FE0F':{'en':':Japanese_secret_button:','status':fully_qualified,'E':0.6,'alias':[':secret:', ':japanese_secret_button:'],'variant':True,'de':':schriftzeichen_für_geheimnis:','es':':ideograma_japonés_para_secreto:','fr':':bouton_secret_en_japonais:','ja':':マル秘:','ko':':원_표시된_비밀_표의문자:','pt':':botão_japonês_de_“segredo”:','it':':ideogramma_giapponese_di_“segreto”:','fa':':دکمه_راز_به_ژاپنی:','id':':tombol_jepang_rahasia:','zh':':秘:','ru':':секрет:','tr':':japonca_sır_düğmesi:','ar':':الزر_/سر/_باليابانية:'},'\U00003299':{'en':':Japanese_secret_button:','status':unqualified,'E':0.6,'alias':[':secret:', ':japanese_secret_button:'],'variant':True,'de':':schriftzeichen_für_geheimnis:','es':':ideograma_japonés_para_secreto:','fr':':bouton_secret_en_japonais:','ja':':マル秘:','ko':':원_표시된_비밀_표의문자:','pt':':botão_japonês_de_“segredo”:','it':':ideogramma_giapponese_di_“segreto”:','fa':':دکمهٔ_«راز»_به_ژاپنی:','id':':tombol_jepang_rahasia:','zh':':秘:','ru':':секрет:','tr':':japonca_sır_düğmesi:','ar':':الزر_/سر/_باليابانية:'},'\U0001F202\U0000FE0F':{'en':':Japanese_service_charge_button:','status':fully_qualified,'E':0.6,'alias':[':sa:', ':japanese_service_charge_button:'],'variant':True,'de':':schriftzeichen_sa:','es':':ideograma_japonés_para_de_cortesía:','fr':':bouton_frais_de_service_en_japonais:','ja':':サのマーク:','ko':':일본어_봉사료_버튼:','pt':':botão_japonês_de_“taxa_de_serviço”:','it':':ideogramma_giapponese_di_“costo_del_servizio”:','fa':':دکمه_هزینه_خدمات_به_ژاپنی:','id':':tombol_jepang_biaya_layanan:','zh':':服务费:','ru':':цена_услуги:','tr':':japonca_servis_ücreti_düğmesi:','ar':':الزر_/خدمة_برسوم/_باليابانية:'},'\U0001F202':{'en':':Japanese_service_charge_button:','status':unqualified,'E':0.6,'alias':[':sa:', ':japanese_service_charge_button:'],'variant':True,'de':':schriftzeichen_sa:','es':':ideograma_japonés_para_de_cortesía:','fr':':bouton_frais_de_service_en_japonais:','ja':':サのマーク:','ko':':일본어_봉사료_버튼:','pt':':botão_japonês_de_“taxa_de_serviço”:','it':':ideogramma_giapponese_di_“costo_del_servizio”:','fa':':دکمهٔ_«هزینهٔ_خدمات»_به_ژاپنی:','id':':tombol_jepang_biaya_layanan:','zh':':服务:','ru':':цена_услуги:','tr':':japonca_servis_ücreti_düğmesi:','ar':':الزر_/خدمة_برسوم/_باليابانية:'},'\U0001F530':{'en':':Japanese_symbol_for_beginner:','status':fully_qualified,'E':0.6,'alias':[':beginner:', ':japanese_symbol_for_beginner:'],'de':':japanisches_anfänger-zeichen:','es':':símbolo_japonés_para_principiante:','fr':':symbole_japonais_de_débutant:','ja':':初心者マーク:','ko':':노란색_초록색_이파리_모양:','pt':':símbolo_japonês_de_principiante:','it':':ideogramma_giapponese_di_“principiante”:','fa':':نماد_ژاپنی_سطح_مبتدی:','id':':simbol_jepang_untuk_pemula:','zh':':日本新手驾驶标志:','ru':':символ_новичка:','tr':':japonca_yeni_başlayan_sembolü:','ar':':ورقة_خضراء_وصفراء:'},'\U0001F233':{'en':':Japanese_vacancy_button:','status':fully_qualified,'E':0.6,'alias':[':u7a7a:', ':japanese_vacancy_button:'],'de':':schriftzeichen_für_zimmer_frei:','es':':ideograma_japonés_para_vacante:','fr':':bouton_chambres_disponibles_en_japonais:','ja':':空マーク:','ko':':빌_공:','pt':':botão_japonês_de_“vago”:','it':':ideogramma_giapponese_di_“posto_libero”:','fa':':دکمۀ_«پست_خالی»_به_ژاپنی:','id':':tombol_jepang_ada_lowongan:','zh':':空:','ru':':есть_места:','tr':':japonca_yer_var_düğmesi:','ar':':الزر_/خال/_باليابانية:'},'\U0001F1EF\U0001F1EA':{'en':':Jersey:','status':fully_qualified,'E':2,'alias':[':flag_for_Jersey:', ':jersey:'],'de':':flagge_jersey:','es':':bandera_jersey:','fr':':drapeau_jersey:','ja':':旗_ジャージー:','ko':':깃발_저지:','pt':':bandeira_jersey:','it':':bandiera_jersey:','fa':':پرچم_جرزی:','id':':bendera_jersey:','zh':':泽西岛:','ru':':флаг_Джерси:','tr':':bayrak_jersey:','ar':':علم_جيرسي:'},'\U0001F1EF\U0001F1F4':{'en':':Jordan:','status':fully_qualified,'E':2,'alias':[':flag_for_Jordan:', ':jordan:'],'de':':flagge_jordanien:','es':':bandera_jordania:','fr':':drapeau_jordanie:','ja':':旗_ヨルダン:','ko':':깃발_요르단:','pt':':bandeira_jordânia:','it':':bandiera_giordania:','fa':':پرچم_اردن:','id':':bendera_yordania:','zh':':约旦:','ru':':флаг_Иордания:','tr':':bayrak_ürdün:','ar':':علم_الأردن:'},'\U0001F1F0\U0001F1FF':{'en':':Kazakhstan:','status':fully_qualified,'E':2,'alias':[':flag_for_Kazakhstan:', ':kazakhstan:'],'de':':flagge_kasachstan:','es':':bandera_kazajistán:','fr':':drapeau_kazakhstan:','ja':':旗_カザフスタン:','ko':':깃발_카자흐스탄:','pt':':bandeira_cazaquistão:','it':':bandiera_kazakistan:','fa':':پرچم_قزاقستان:','id':':bendera_kazakhstan:','zh':':哈薩克:','ru':':флаг_Казахстан:','tr':':bayrak_kazakistan:','ar':':علم_كازاخستان:'},'\U0001F1F0\U0001F1EA':{'en':':Kenya:','status':fully_qualified,'E':2,'alias':[':flag_for_Kenya:', ':kenya:'],'de':':flagge_kenia:','es':':bandera_kenia:','fr':':drapeau_kenya:','ja':':旗_ケニア:','ko':':깃발_케냐:','pt':':bandeira_quênia:','it':':bandiera_kenya:','fa':':پرچم_کنیا:','id':':bendera_kenya:','zh':':肯尼亚:','ru':':флаг_Кения:','tr':':bayrak_kenya:','ar':':علم_كينيا:'},'\U0001F1F0\U0001F1EE':{'en':':Kiribati:','status':fully_qualified,'E':2,'alias':[':flag_for_Kiribati:', ':kiribati:'],'de':':flagge_kiribati:','es':':bandera_kiribati:','fr':':drapeau_kiribati:','ja':':旗_キリバス:','ko':':깃발_키리바시:','pt':':bandeira_quiribati:','it':':bandiera_kiribati:','fa':':پرچم_کیریباتی:','id':':bendera_kiribati:','zh':':基里巴斯:','ru':':флаг_Кирибати:','tr':':bayrak_kiribati:','ar':':علم_كيريباتي:'},'\U0001F1FD\U0001F1F0':{'en':':Kosovo:','status':fully_qualified,'E':2,'alias':[':flag_for_Kosovo:', ':kosovo:'],'de':':flagge_kosovo:','es':':bandera_kosovo:','fr':':drapeau_kosovo:','ja':':旗_コソボ:','ko':':깃발_코소보:','pt':':bandeira_kosovo:','it':':bandiera_kosovo:','fa':':پرچم_کوزوو:','id':':bendera_kosovo:','zh':':科索沃:','ru':':флаг_Косово:','tr':':bayrak_kosova:','ar':':علم_كوسوفو:'},'\U0001F1F0\U0001F1FC':{'en':':Kuwait:','status':fully_qualified,'E':2,'alias':[':flag_for_Kuwait:', ':kuwait:'],'de':':flagge_kuwait:','es':':bandera_kuwait:','fr':':drapeau_koweït:','ja':':旗_クウェート:','ko':':깃발_쿠웨이트:','pt':':bandeira_kuwait:','it':':bandiera_kuwait:','fa':':پرچم_کویت:','id':':bendera_kuwait:','zh':':科威特:','ru':':флаг_Кувейт:','tr':':bayrak_kuveyt:','ar':':علم_الكويت:'},'\U0001F1F0\U0001F1EC':{'en':':Kyrgyzstan:','status':fully_qualified,'E':2,'alias':[':flag_for_Kyrgyzstan:', ':kyrgyzstan:'],'de':':flagge_kirgisistan:','es':':bandera_kirguistán:','fr':':drapeau_kirghizstan:','ja':':旗_キルギス:','ko':':깃발_키르기스스탄:','pt':':bandeira_quirguistão:','it':':bandiera_kirghizistan:','fa':':پرچم_قرقیزستان:','id':':bendera_kirgizstan:','zh':':吉尔吉斯斯坦:','ru':':флаг_Киргизия:','tr':':bayrak_kırgızistan:','ar':':علم_قيرغيزستان:'},'\U0001F1F1\U0001F1E6':{'en':':Laos:','status':fully_qualified,'E':2,'alias':[':flag_for_Laos:', ':laos:'],'de':':flagge_laos:','es':':bandera_laos:','fr':':drapeau_laos:','ja':':旗_ラオス:','ko':':깃발_라오스:','pt':':bandeira_laos:','it':':bandiera_laos:','fa':':پرچم_لائوس:','id':':bendera_laos:','zh':':老挝:','ru':':флаг_Лаос:','tr':':bayrak_laos:','ar':':علم_لاوس:'},'\U0001F1F1\U0001F1FB':{'en':':Latvia:','status':fully_qualified,'E':2,'alias':[':flag_for_Latvia:', ':latvia:'],'de':':flagge_lettland:','es':':bandera_letonia:','fr':':drapeau_lettonie:','ja':':旗_ラトビア:','ko':':깃발_라트비아:','pt':':bandeira_letônia:','it':':bandiera_lettonia:','fa':':پرچم_لتونی:','id':':bendera_latvia:','zh':':拉脱维亚:','ru':':флаг_Латвия:','tr':':bayrak_letonya:','ar':':علم_لاتفيا:'},'\U0001F1F1\U0001F1E7':{'en':':Lebanon:','status':fully_qualified,'E':2,'alias':[':flag_for_Lebanon:', ':lebanon:'],'de':':flagge_libanon:','es':':bandera_líbano:','fr':':drapeau_liban:','ja':':旗_レバノン:','ko':':깃발_레바논:','pt':':bandeira_líbano:','it':':bandiera_libano:','fa':':پرچم_لبنان:','id':':bendera_lebanon:','zh':':黎巴嫩:','ru':':флаг_Ливан:','tr':':bayrak_lübnan:','ar':':علم_لبنان:'},'\U0000264C':{'en':':Leo:','status':fully_qualified,'E':0.6,'alias':[':leo:'],'variant':True,'de':':löwe_(sternzeichen):','es':':leo:','fr':':lion:','ja':':しし座:','ko':':사자자리:','pt':':signo_de_leão:','it':':segno_zodiacale_del_leone:','fa':':برج_اسد:','id':':leo:','zh':':狮子座:','ru':':знак_зодиака_Лев:','tr':':aslan_burcu:','ar':':الأسد:'},'\U0001F1F1\U0001F1F8':{'en':':Lesotho:','status':fully_qualified,'E':2,'alias':[':flag_for_Lesotho:', ':lesotho:'],'de':':flagge_lesotho:','es':':bandera_lesoto:','fr':':drapeau_lesotho:','ja':':旗_レソト:','ko':':깃발_레소토:','pt':':bandeira_lesoto:','it':':bandiera_lesotho:','fa':':پرچم_لسوتو:','id':':bendera_lesotho:','zh':':莱索托:','ru':':флаг_Лесото:','tr':':bayrak_lesotho:','ar':':علم_ليسوتو:'},'\U0001F1F1\U0001F1F7':{'en':':Liberia:','status':fully_qualified,'E':2,'alias':[':flag_for_Liberia:', ':liberia:'],'de':':flagge_liberia:','es':':bandera_liberia:','fr':':drapeau_liberia:','ja':':旗_リベリア:','ko':':깃발_라이베리아:','pt':':bandeira_libéria:','it':':bandiera_liberia:','fa':':پرچم_لیبریا:','id':':bendera_liberia:','zh':':利比里亚:','ru':':флаг_Либерия:','tr':':bayrak_liberya:','ar':':علم_ليبيريا:'},'\U0000264E':{'en':':Libra:','status':fully_qualified,'E':0.6,'alias':[':libra:'],'variant':True,'de':':waage_(sternzeichen):','es':':libra:','fr':':balance:','ja':':てんびん座:','ko':':천칭자리:','pt':':signo_de_libra:','it':':segno_zodiacale_della_bilancia:','fa':':برج_میزان:','id':':libra:','zh':':天秤座:','ru':':знак_зодиака_Весы:','tr':':terazi_burcu:','ar':':الميزان:'},'\U0001F1F1\U0001F1FE':{'en':':Libya:','status':fully_qualified,'E':2,'alias':[':flag_for_Libya:', ':libya:'],'de':':flagge_libyen:','es':':bandera_libia:','fr':':drapeau_libye:','ja':':旗_リビア:','ko':':깃발_리비아:','pt':':bandeira_líbia:','it':':bandiera_libia:','fa':':پرچم_لیبی:','id':':bendera_libya:','zh':':利比亚:','ru':':флаг_Ливия:','tr':':bayrak_libya:','ar':':علم_ليبيا:'},'\U0001F1F1\U0001F1EE':{'en':':Liechtenstein:','status':fully_qualified,'E':2,'alias':[':flag_for_Liechtenstein:', ':liechtenstein:'],'de':':flagge_liechtenstein:','es':':bandera_liechtenstein:','fr':':drapeau_liechtenstein:','ja':':旗_リヒテンシュタイン:','ko':':깃발_리히텐슈타인:','pt':':bandeira_liechtenstein:','it':':bandiera_liechtenstein:','fa':':پرچم_لیختن_اشتاین:','id':':bendera_liechtenstein:','zh':':列支敦士登:','ru':':флаг_Лихтенштейн:','tr':':bayrak_liechtenstein:','ar':':علم_ليختنشتاين:'},'\U0001F1F1\U0001F1F9':{'en':':Lithuania:','status':fully_qualified,'E':2,'alias':[':flag_for_Lithuania:', ':lithuania:'],'de':':flagge_litauen:','es':':bandera_lituania:','fr':':drapeau_lituanie:','ja':':旗_リトアニア:','ko':':깃발_리투아니아:','pt':':bandeira_lituânia:','it':':bandiera_lituania:','fa':':پرچم_لیتوانی:','id':':bendera_lituania:','zh':':立陶宛:','ru':':флаг_Литва:','tr':':bayrak_litvanya:','ar':':علم_ليتوانيا:'},'\U0001F1F1\U0001F1FA':{'en':':Luxembourg:','status':fully_qualified,'E':2,'alias':[':flag_for_Luxembourg:', ':luxembourg:'],'de':':flagge_luxemburg:','es':':bandera_luxemburgo:','fr':':drapeau_luxembourg:','ja':':旗_ルクセンブルク:','ko':':깃발_룩셈부르크:','pt':':bandeira_luxemburgo:','it':':bandiera_lussemburgo:','fa':':پرچم_لوکزامبورگ:','id':':bendera_luksemburg:','zh':':卢森堡:','ru':':флаг_Люксембург:','tr':':bayrak_lüksemburg:','ar':':علم_لوكسمبورغ:'},'\U0001F1F2\U0001F1F4':{'en':':Macao_SAR_China:','status':fully_qualified,'E':2,'alias':[':flag_for_Macau:', ':macau:'],'de':':flagge_sonderverwaltungsregion_macau:','es':':bandera_rae_de_macao_(china):','fr':':drapeau_r.a.s._chinoise_de_macao:','ja':':旗_中華人民共和国マカオ特別行政区:','ko':':깃발_마카오(중국_특별행정구):','pt':':bandeira_macau_rae_da_china:','it':':bandiera_ras_di_macao:','fa':':پرچم_ماکائو_منطقهٔ_ویژهٔ_اداری_چین:','id':':bendera_makau_dak_tiongkok:','zh':':澳門:','ru':':флаг_Макао_(САР):','tr':':bayrak_çin_makao_öib:','ar':':علم_منطقة_ماكاو_الإدارية_الخاصة:'},'\U0001F1F2\U0001F1EC':{'en':':Madagascar:','status':fully_qualified,'E':2,'alias':[':flag_for_Madagascar:', ':madagascar:'],'de':':flagge_madagaskar:','es':':bandera_madagascar:','fr':':drapeau_madagascar:','ja':':旗_マダガスカル:','ko':':깃발_마다가스카르:','pt':':bandeira_madagascar:','it':':bandiera_madagascar:','fa':':پرچم_ماداگاسکار:','id':':bendera_madagaskar:','zh':':马达加斯加:','ru':':флаг_Мадагаскар:','tr':':bayrak_madagaskar:','ar':':علم_مدغشقر:'},'\U0001F1F2\U0001F1FC':{'en':':Malawi:','status':fully_qualified,'E':2,'alias':[':flag_for_Malawi:', ':malawi:'],'de':':flagge_malawi:','es':':bandera_malaui:','fr':':drapeau_malawi:','ja':':旗_マラウイ:','ko':':깃발_말라위:','pt':':bandeira_malaui:','it':':bandiera_malawi:','fa':':پرچم_مالاوی:','id':':bendera_malawi:','zh':':马拉维:','ru':':флаг_Малави:','tr':':bayrak_malavi:','ar':':علم_ملاوي:'},'\U0001F1F2\U0001F1FE':{'en':':Malaysia:','status':fully_qualified,'E':2,'alias':[':flag_for_Malaysia:', ':malaysia:'],'de':':flagge_malaysia:','es':':bandera_malasia:','fr':':drapeau_malaisie:','ja':':旗_マレーシア:','ko':':깃발_말레이시아:','pt':':bandeira_malásia:','it':':bandiera_malaysia:','fa':':پرچم_مالزی:','id':':bendera_malaysia:','zh':':马来西亚:','ru':':флаг_Малайзия:','tr':':bayrak_malezya:','ar':':علم_ماليزيا:'},'\U0001F1F2\U0001F1FB':{'en':':Maldives:','status':fully_qualified,'E':2,'alias':[':flag_for_Maldives:', ':maldives:'],'de':':flagge_malediven:','es':':bandera_maldivas:','fr':':drapeau_maldives:','ja':':旗_モルディブ:','ko':':깃발_몰디브:','pt':':bandeira_maldivas:','it':':bandiera_maldive:','fa':':پرچم_مالدیو:','id':':bendera_maladewa:','zh':':马尔代夫:','ru':':флаг_Мальдивы:','tr':':bayrak_maldivler:','ar':':علم_جزر_المالديف:'},'\U0001F1F2\U0001F1F1':{'en':':Mali:','status':fully_qualified,'E':2,'alias':[':flag_for_Mali:', ':mali:'],'de':':flagge_mali:','es':':bandera_mali:','fr':':drapeau_mali:','ja':':旗_マリ:','ko':':깃발_말리:','pt':':bandeira_mali:','it':':bandiera_mali:','fa':':پرچم_مالی:','id':':bendera_mali:','zh':':马里:','ru':':флаг_Мали:','tr':':bayrak_mali:','ar':':علم_مالي:'},'\U0001F1F2\U0001F1F9':{'en':':Malta:','status':fully_qualified,'E':2,'alias':[':flag_for_Malta:', ':malta:'],'de':':flagge_malta:','es':':bandera_malta:','fr':':drapeau_malte:','ja':':旗_マルタ:','ko':':깃발_몰타:','pt':':bandeira_malta:','it':':bandiera_malta:','fa':':پرچم_مالت:','id':':bendera_malta:','zh':':马耳他:','ru':':флаг_Мальта:','tr':':bayrak_malta:','ar':':علم_مالطا:'},'\U0001F1F2\U0001F1ED':{'en':':Marshall_Islands:','status':fully_qualified,'E':2,'alias':[':flag_for_Marshall_Islands:', ':marshall_islands:'],'de':':flagge_marshallinseln:','es':':bandera_islas_marshall:','fr':':drapeau_îles_marshall:','ja':':旗_マーシャル諸島:','ko':':깃발_마셜_제도:','pt':':bandeira_ilhas_marshall:','it':':bandiera_isole_marshall:','fa':':پرچم_جزایر_مارشال:','id':':bendera_kepulauan_marshall:','zh':':马绍尔群岛:','ru':':флаг_Маршалловы_о_ва:','tr':':bayrak_marshall_adaları:','ar':':علم_جزر_مارشال:'},'\U0001F1F2\U0001F1F6':{'en':':Martinique:','status':fully_qualified,'E':2,'alias':[':flag_for_Martinique:', ':martinique:'],'de':':flagge_martinique:','es':':bandera_martinica:','fr':':drapeau_martinique:','ja':':旗_マルティニーク:','ko':':깃발_마르티니크:','pt':':bandeira_martinica:','it':':bandiera_martinica:','fa':':پرچم_مارتینیک:','id':':bendera_martinik:','zh':':马提尼克:','ru':':флаг_Мартиника:','tr':':bayrak_martinik:','ar':':علم_جزر_المارتينيك:'},'\U0001F1F2\U0001F1F7':{'en':':Mauritania:','status':fully_qualified,'E':2,'alias':[':flag_for_Mauritania:', ':mauritania:'],'de':':flagge_mauretanien:','es':':bandera_mauritania:','fr':':drapeau_mauritanie:','ja':':旗_モーリタニア:','ko':':깃발_모리타니:','pt':':bandeira_mauritânia:','it':':bandiera_mauritania:','fa':':پرچم_موریتانی:','id':':bendera_mauritania:','zh':':毛里塔尼亚:','ru':':флаг_Мавритания:','tr':':bayrak_moritanya:','ar':':علم_موريتانيا:'},'\U0001F1F2\U0001F1FA':{'en':':Mauritius:','status':fully_qualified,'E':2,'alias':[':flag_for_Mauritius:', ':mauritius:'],'de':':flagge_mauritius:','es':':bandera_mauricio:','fr':':drapeau_maurice:','ja':':旗_モーリシャス:','ko':':깃발_모리셔스:','pt':':bandeira_maurício:','it':':bandiera_mauritius:','fa':':پرچم_موریس:','id':':bendera_mauritius:','zh':':毛里求斯:','ru':':флаг_Маврикий:','tr':':bayrak_mauritius:','ar':':علم_موريشيوس:'},'\U0001F1FE\U0001F1F9':{'en':':Mayotte:','status':fully_qualified,'E':2,'alias':[':flag_for_Mayotte:', ':mayotte:'],'de':':flagge_mayotte:','es':':bandera_mayotte:','fr':':drapeau_mayotte:','ja':':旗_マヨット:','ko':':깃발_마요트:','pt':':bandeira_mayotte:','it':':bandiera_mayotte:','fa':':پرچم_مایوت:','id':':bendera_mayotte:','zh':':马约特:','ru':':флаг_Майотта:','tr':':bayrak_mayotte:','ar':':علم_مايوت:'},'\U0001F1F2\U0001F1FD':{'en':':Mexico:','status':fully_qualified,'E':2,'alias':[':flag_for_Mexico:', ':mexico:'],'de':':flagge_mexiko:','es':':bandera_méxico:','fr':':drapeau_mexique:','ja':':旗_メキシコ:','ko':':깃발_멕시코:','pt':':bandeira_méxico:','it':':bandiera_messico:','fa':':پرچم_مکزیک:','id':':bendera_meksiko:','zh':':墨西哥:','ru':':флаг_Мексика:','tr':':bayrak_meksika:','ar':':علم_المكسيك:'},'\U0001F1EB\U0001F1F2':{'en':':Micronesia:','status':fully_qualified,'E':2,'alias':[':flag_for_Micronesia:', ':micronesia:'],'de':':flagge_mikronesien:','es':':bandera_micronesia:','fr':':drapeau_micronésie:','ja':':旗_ミクロネシア連邦:','ko':':깃발_미크로네시아:','pt':':bandeira_micronésia:','it':':bandiera_micronesia:','fa':':پرچم_میکرونزی:','id':':bendera_mikronesia:','zh':':密克罗尼西亚:','ru':':флаг_Федеративные_Штаты_Микронезии:','tr':':bayrak_mikronezya:','ar':':علم_ميكرونيزيا:'},'\U0001F1F2\U0001F1E9':{'en':':Moldova:','status':fully_qualified,'E':2,'alias':[':flag_for_Moldova:', ':moldova:'],'de':':flagge_republik_moldau:','es':':bandera_moldavia:','fr':':drapeau_moldavie:','ja':':旗_モルドバ:','ko':':깃발_몰도바:','pt':':bandeira_moldávia:','it':':bandiera_moldavia:','fa':':پرچم_مولداوی:','id':':bendera_moldova:','zh':':摩尔多瓦:','ru':':флаг_Молдова:','tr':':bayrak_moldova:','ar':':علم_مولدوفا:'},'\U0001F1F2\U0001F1E8':{'en':':Monaco:','status':fully_qualified,'E':2,'alias':[':flag_for_Monaco:', ':monaco:'],'de':':flagge_monaco:','es':':bandera_mónaco:','fr':':drapeau_monaco:','ja':':旗_モナコ:','ko':':깃발_모나코:','pt':':bandeira_mônaco:','it':':bandiera_monaco:','fa':':پرچم_موناکو:','id':':bendera_monako:','zh':':摩纳哥:','ru':':флаг_Монако:','tr':':bayrak_monako:','ar':':علم_موناكو:'},'\U0001F1F2\U0001F1F3':{'en':':Mongolia:','status':fully_qualified,'E':2,'alias':[':flag_for_Mongolia:', ':mongolia:'],'de':':flagge_mongolei:','es':':bandera_mongolia:','fr':':drapeau_mongolie:','ja':':旗_モンゴル:','ko':':깃발_몽골:','pt':':bandeira_mongólia:','it':':bandiera_mongolia:','fa':':پرچم_مغولستان:','id':':bendera_mongolia:','zh':':蒙古:','ru':':флаг_Монголия:','tr':':bayrak_moğolistan:','ar':':علم_منغوليا:'},'\U0001F1F2\U0001F1EA':{'en':':Montenegro:','status':fully_qualified,'E':2,'alias':[':flag_for_Montenegro:', ':montenegro:'],'de':':flagge_montenegro:','es':':bandera_montenegro:','fr':':drapeau_monténégro:','ja':':旗_モンテネグロ:','ko':':깃발_몬테네그로:','pt':':bandeira_montenegro:','it':':bandiera_montenegro:','fa':':پرچم_مونته_نگرو:','id':':bendera_montenegro:','zh':':黑山:','ru':':флаг_Черногория:','tr':':bayrak_karadağ:','ar':':علم_الجبل_الأسود:'},'\U0001F1F2\U0001F1F8':{'en':':Montserrat:','status':fully_qualified,'E':2,'alias':[':flag_for_Montserrat:', ':montserrat:'],'de':':flagge_montserrat:','es':':bandera_montserrat:','fr':':drapeau_montserrat:','ja':':旗_モントセラト:','ko':':깃발_몬트세라트:','pt':':bandeira_montserrat:','it':':bandiera_montserrat:','fa':':پرچم_مونت_سرات:','id':':bendera_montserrat:','zh':':蒙特塞拉特:','ru':':флаг_Монтсеррат:','tr':':bayrak_montserrat:','ar':':علم_مونتسرات:'},'\U0001F1F2\U0001F1E6':{'en':':Morocco:','status':fully_qualified,'E':2,'alias':[':flag_for_Morocco:', ':morocco:'],'de':':flagge_marokko:','es':':bandera_marruecos:','fr':':drapeau_maroc:','ja':':旗_モロッコ:','ko':':깃발_모로코:','pt':':bandeira_marrocos:','it':':bandiera_marocco:','fa':':پرچم_مراکش:','id':':bendera_maroko:','zh':':摩洛哥:','ru':':флаг_Марокко:','tr':':bayrak_fas:','ar':':علم_المغرب:'},'\U0001F1F2\U0001F1FF':{'en':':Mozambique:','status':fully_qualified,'E':2,'alias':[':flag_for_Mozambique:', ':mozambique:'],'de':':flagge_mosambik:','es':':bandera_mozambique:','fr':':drapeau_mozambique:','ja':':旗_モザンビーク:','ko':':깃발_모잠비크:','pt':':bandeira_moçambique:','it':':bandiera_mozambico:','fa':':پرچم_موزامبیک:','id':':bendera_mozambik:','zh':':莫桑比克:','ru':':флаг_Мозамбик:','tr':':bayrak_mozambik:','ar':':علم_موزمبيق:'},'\U0001F936':{'en':':Mrs._Claus:','status':fully_qualified,'E':3,'alias':[':mrs_claus:'],'de':':weihnachtsfrau:','es':':mamá_noel:','fr':':mère_noël:','ja':':女性のサンタ:','ko':':산타할머니:','pt':':mamãe_noel:','it':':mamma_natale:','fa':':مامان_نوئل:','id':':nyonya_sinterklas:','zh':':圣诞奶奶:','ru':':миссис_Клаус:','tr':':noel_anne:','ar':':سيدة_سانتا_كلوز:'},'\U0001F936\U0001F3FF':{'en':':Mrs._Claus_dark_skin_tone:','status':fully_qualified,'E':3,'de':':weihnachtsfrau_dunkle_hautfarbe:','es':':mamá_noel_tono_de_piel_oscuro:','fr':':mère_noël_peau_foncée:','ja':':女性のサンタ_濃い肌色:','ko':':산타할머니_검은색_피부:','pt':':mamãe_noel_pele_escura:','it':':mamma_natale_carnagione_scura:','fa':':مامان_نوئل_پوست_آبنوسی:','id':':nyonya_sinterklas_warna_kulit_gelap:','zh':':圣诞奶奶_较深肤色:','ru':':миссис_Клаус_очень_темный_тон_кожи:','tr':':noel_anne_koyu_cilt_tonu:','ar':':سيدة_سانتا_كلوز_بشرة_بلون_غامق:'},'\U0001F936\U0001F3FB':{'en':':Mrs._Claus_light_skin_tone:','status':fully_qualified,'E':3,'de':':weihnachtsfrau_helle_hautfarbe:','es':':mamá_noel_tono_de_piel_claro:','fr':':mère_noël_peau_claire:','ja':':女性のサンタ_薄い肌色:','ko':':산타할머니_하얀_피부:','pt':':mamãe_noel_pele_clara:','it':':mamma_natale_carnagione_chiara:','fa':':مامان_نوئل_پوست_سفید:','id':':nyonya_sinterklas_warna_kulit_cerah:','zh':':圣诞奶奶_较浅肤色:','ru':':миссис_Клаус_очень_светлый_тон_кожи:','tr':':noel_anne_açık_cilt_tonu:','ar':':سيدة_سانتا_كلوز_بشرة_بلون_فاتح:'},'\U0001F936\U0001F3FE':{'en':':Mrs._Claus_medium-dark_skin_tone:','status':fully_qualified,'E':3,'de':':weihnachtsfrau_mitteldunkle_hautfarbe:','es':':mamá_noel_tono_de_piel_oscuro_medio:','fr':':mère_noël_peau_mate:','ja':':女性のサンタ_やや濃い肌色:','ko':':산타할머니_진한_갈색_피부:','pt':':mamãe_noel_pele_morena_escura:','it':':mamma_natale_carnagione_abbastanza_scura:','fa':':مامان_نوئل_پوست_گندمی:','id':':nyonya_sinterklas_warna_kulit_gelap-sedang:','zh':':圣诞奶奶_中等深肤色:','ru':':миссис_Клаус_темный_тон_кожи:','tr':':noel_anne_orta_koyu_cilt_tonu:','ar':':سيدة_سانتا_كلوز_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F936\U0001F3FC':{'en':':Mrs._Claus_medium-light_skin_tone:','status':fully_qualified,'E':3,'de':':weihnachtsfrau_mittelhelle_hautfarbe:','es':':mamá_noel_tono_de_piel_claro_medio:','fr':':mère_noël_peau_moyennement_claire:','ja':':女性のサンタ_やや薄い肌色:','ko':':산타할머니_연한_갈색_피부:','pt':':mamãe_noel_pele_morena_clara:','it':':mamma_natale_carnagione_abbastanza_chiara:','fa':':مامان_نوئل_پوست_روشن:','id':':nyonya_sinterklas_warna_kulit_cerah-sedang:','zh':':圣诞奶奶_中等浅肤色:','ru':':миссис_Клаус_светлый_тон_кожи:','tr':':noel_anne_orta_açık_cilt_tonu:','ar':':سيدة_سانتا_كلوز_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F936\U0001F3FD':{'en':':Mrs._Claus_medium_skin_tone:','status':fully_qualified,'E':3,'de':':weihnachtsfrau_mittlere_hautfarbe:','es':':mamá_noel_tono_de_piel_medio:','fr':':mère_noël_peau_légèrement_mate:','ja':':女性のサンタ_中間の肌色:','ko':':산타할머니_갈색_피부:','pt':':mamãe_noel_pele_morena:','it':':mamma_natale_carnagione_olivastra:','fa':':مامان_نوئل_پوست_طلایی:','id':':nyonya_sinterklas_warna_kulit_sedang:','zh':':圣诞奶奶_中等肤色:','ru':':миссис_Клаус_средний_тон_кожи:','tr':':noel_anne_orta_cilt_tonu:','ar':':سيدة_سانتا_كلوز_بشرة_بلون_معتدل:'},'\U0001F1F2\U0001F1F2':{'en':':Myanmar_(Burma):','status':fully_qualified,'E':2,'alias':[':flag_for_Myanmar:', ':myanmar:'],'de':':flagge_myanmar:','es':':bandera_myanmar_(birmania):','fr':':drapeau_myanmar_(birmanie):','ja':':旗_ミャンマー_(ビルマ):','ko':':깃발_미얀마:','pt':':bandeira_mianmar_(birmânia):','it':':bandiera_myanmar_(birmania):','fa':':پرچم_میانمار_(برمه):','id':':bendera_myanmar_(burma):','zh':':缅甸:','ru':':флаг_Мьянма_(Бирма):','tr':':bayrak_myanmar_(burma):','ar':':علم_ميانمار_(بورما):'},'\U0001F195':{'en':':NEW_button:','status':fully_qualified,'E':0.6,'alias':[':new:', ':new_button:'],'de':':wort_new_in_blauem_quadrat:','es':':botón_new:','fr':':bouton_nouveau:','ja':':newマーク:','ko':':뉴:','pt':':botão_new:','it':':pulsante_new:','fa':':دکمهٔ_new:','id':':tombol_new:','zh':':new按钮:','ru':':значок_«новинка»:','tr':':new_düğmesi:','ar':':زر_جديد:'},'\U0001F196':{'en':':NG_button:','status':fully_qualified,'E':0.6,'alias':[':ng:', ':ng_button:'],'de':':großbuchstaben_ng_in_blauem_quadrat:','es':':botón_ng:','fr':':bouton_pas_bien:','ja':':ngマーク:','ko':':엔지:','pt':':botão_ng:','it':':pulsante_ng:','fa':':دکمهٔ_ng:','id':':tombol_ng:','zh':':NG按钮:','ru':':значок_«не_годится»:','tr':':ng_düğmesi:','ar':':زر_غير_جيد:'},'\U0001F1F3\U0001F1E6':{'en':':Namibia:','status':fully_qualified,'E':2,'alias':[':flag_for_Namibia:', ':namibia:'],'de':':flagge_namibia:','es':':bandera_namibia:','fr':':drapeau_namibie:','ja':':旗_ナミビア:','ko':':깃발_나미비아:','pt':':bandeira_namíbia:','it':':bandiera_namibia:','fa':':پرچم_نامیبیا:','id':':bendera_namibia:','zh':':纳米比亚:','ru':':флаг_Намибия:','tr':':bayrak_namibya:','ar':':علم_ناميبيا:'},'\U0001F1F3\U0001F1F7':{'en':':Nauru:','status':fully_qualified,'E':2,'alias':[':flag_for_Nauru:', ':nauru:'],'de':':flagge_nauru:','es':':bandera_nauru:','fr':':drapeau_nauru:','ja':':旗_ナウル:','ko':':깃발_나우루:','pt':':bandeira_nauru:','it':':bandiera_nauru:','fa':':پرچم_نائورو:','id':':bendera_nauru:','zh':':瑙鲁:','ru':':флаг_Науру:','tr':':bayrak_nauru:','ar':':علم_ناورو:'},'\U0001F1F3\U0001F1F5':{'en':':Nepal:','status':fully_qualified,'E':2,'alias':[':flag_for_Nepal:', ':nepal:'],'de':':flagge_nepal:','es':':bandera_nepal:','fr':':drapeau_népal:','ja':':旗_ネパール:','ko':':깃발_네팔:','pt':':bandeira_nepal:','it':':bandiera_nepal:','fa':':پرچم_نپال:','id':':bendera_nepal:','zh':':尼泊尔:','ru':':флаг_Непал:','tr':':bayrak_nepal:','ar':':علم_نيبال:'},'\U0001F1F3\U0001F1F1':{'en':':Netherlands:','status':fully_qualified,'E':2,'alias':[':flag_for_Netherlands:', ':netherlands:'],'de':':flagge_niederlande:','es':':bandera_países_bajos:','fr':':drapeau_pays-bas:','ja':':旗_オランダ:','ko':':깃발_네덜란드:','pt':':bandeira_países_baixos:','it':':bandiera_paesi_bassi:','fa':':پرچم_هلند:','id':':bendera_belanda:','zh':':荷兰:','ru':':флаг_Нидерланды:','tr':':bayrak_hollanda:','ar':':علم_هولندا:'},'\U0001F1F3\U0001F1E8':{'en':':New_Caledonia:','status':fully_qualified,'E':2,'alias':[':flag_for_New_Caledonia:', ':new_caledonia:'],'de':':flagge_neukaledonien:','es':':bandera_nueva_caledonia:','fr':':drapeau_nouvelle-calédonie:','ja':':旗_ニューカレドニア:','ko':':깃발_뉴칼레도니아:','pt':':bandeira_nova_caledônia:','it':':bandiera_nuova_caledonia:','fa':':پرچم_کالدونیای_جدید:','id':':bendera_kaledonia_baru:','zh':':新喀里多尼亚:','ru':':флаг_Новая_Каледония:','tr':':bayrak_yeni_kaledonya:','ar':':علم_كاليدونيا_الجديدة:'},'\U0001F1F3\U0001F1FF':{'en':':New_Zealand:','status':fully_qualified,'E':2,'alias':[':flag_for_New_Zealand:', ':new_zealand:'],'de':':flagge_neuseeland:','es':':bandera_nueva_zelanda:','fr':':drapeau_nouvelle-zélande:','ja':':旗_ニュージーランド:','ko':':깃발_뉴질랜드:','pt':':bandeira_nova_zelândia:','it':':bandiera_nuova_zelanda:','fa':':پرچم_نیوزیلند:','id':':bendera_selandia_baru:','zh':':新西兰:','ru':':флаг_Новая_Зеландия:','tr':':bayrak_yeni_zelanda:','ar':':علم_نيوزيلندا:'},'\U0001F1F3\U0001F1EE':{'en':':Nicaragua:','status':fully_qualified,'E':2,'alias':[':flag_for_Nicaragua:', ':nicaragua:'],'de':':flagge_nicaragua:','es':':bandera_nicaragua:','fr':':drapeau_nicaragua:','ja':':旗_ニカラグア:','ko':':깃발_니카라과:','pt':':bandeira_nicarágua:','it':':bandiera_nicaragua:','fa':':پرچم_نیکاراگوئه:','id':':bendera_nikaragua:','zh':':尼加拉瓜:','ru':':флаг_Никарагуа:','tr':':bayrak_nikaragua:','ar':':علم_نيكاراغوا:'},'\U0001F1F3\U0001F1EA':{'en':':Niger:','status':fully_qualified,'E':2,'alias':[':flag_for_Niger:', ':niger:'],'de':':flagge_niger:','es':':bandera_níger:','fr':':drapeau_niger:','ja':':旗_ニジェール:','ko':':깃발_니제르:','pt':':bandeira_níger:','it':':bandiera_niger:','fa':':پرچم_نیجر:','id':':bendera_niger:','zh':':尼日尔:','ru':':флаг_Нигер:','tr':':bayrak_nijer:','ar':':علم_النيجر:'},'\U0001F1F3\U0001F1EC':{'en':':Nigeria:','status':fully_qualified,'E':2,'alias':[':flag_for_Nigeria:', ':nigeria:'],'de':':flagge_nigeria:','es':':bandera_nigeria:','fr':':drapeau_nigeria:','ja':':旗_ナイジェリア:','ko':':깃발_나이지리아:','pt':':bandeira_nigéria:','it':':bandiera_nigeria:','fa':':پرچم_نیجریه:','id':':bendera_nigeria:','zh':':尼日利亚:','ru':':флаг_Нигерия:','tr':':bayrak_nijerya:','ar':':علم_نيجيريا:'},'\U0001F1F3\U0001F1FA':{'en':':Niue:','status':fully_qualified,'E':2,'alias':[':flag_for_Niue:', ':niue:'],'de':':flagge_niue:','es':':bandera_niue:','fr':':drapeau_niue:','ja':':旗_ニウエ:','ko':':깃발_니우에:','pt':':bandeira_niue:','it':':bandiera_niue:','fa':':پرچم_نیوئه:','id':':bendera_niue:','zh':':纽埃:','ru':':флаг_Ниуэ:','tr':':bayrak_niue:','ar':':علم_نيوي:'},'\U0001F1F3\U0001F1EB':{'en':':Norfolk_Island:','status':fully_qualified,'E':2,'alias':[':flag_for_Norfolk_Island:', ':norfolk_island:'],'de':':flagge_norfolkinsel:','es':':bandera_isla_norfolk:','fr':':drapeau_île_norfolk:','ja':':旗_ノーフォーク島:','ko':':깃발_노퍽섬:','pt':':bandeira_ilha_norfolk:','it':':bandiera_isola_norfolk:','fa':':پرچم_جزیرهٔ_نورفولک:','id':':bendera_kepulauan_norfolk:','zh':':诺福克岛:','ru':':флаг_о_в_Норфолк:','tr':':bayrak_norfolk_adası:','ar':':علم_جزيرة_نورفولك:'},'\U0001F1F0\U0001F1F5':{'en':':North_Korea:','status':fully_qualified,'E':2,'alias':[':flag_for_North_Korea:', ':north_korea:'],'de':':flagge_nordkorea:','es':':bandera_corea_del_norte:','fr':':drapeau_corée_du_nord:','ja':':旗_北朝鮮:','ko':':깃발_북한:','pt':':bandeira_coreia_do_norte:','it':':bandiera_corea_del_nord:','fa':':پرچم_کرهٔ_شمالی:','id':':bendera_korea_utara:','zh':':朝鲜:','ru':':флаг_КНДР:','tr':':bayrak_kuzey_kore:','ar':':علم_كوريا_الشمالية:'},'\U0001F1F2\U0001F1F0':{'en':':North_Macedonia:','status':fully_qualified,'E':2,'alias':[':flag_for_Macedonia:', ':macedonia:'],'de':':flagge_nordmazedonien:','es':':bandera_macedonia_del_norte:','fr':':drapeau_macédoine_du_nord:','ja':':旗_北マケドニア:','ko':':깃발_북마케도니아:','pt':':bandeira_macedônia_do_norte:','it':':bandiera_macedonia_del_nord:','fa':':پرچم_مقدونیهٔ_شمالی:','id':':bendera_makedonia_utara:','zh':':北马其顿:','ru':':флаг_Северная_Македония:','tr':':bayrak_kuzey_makedonya:','ar':':علم_مقدونيا_الشمالية:'},'\U0001F1F2\U0001F1F5':{'en':':Northern_Mariana_Islands:','status':fully_qualified,'E':2,'alias':[':flag_for_Northern_Mariana_Islands:', ':northern_mariana_islands:'],'de':':flagge_nördliche_marianen:','es':':bandera_islas_marianas_del_norte:','fr':':drapeau_îles_mariannes_du_nord:','ja':':旗_北マリアナ諸島:','ko':':깃발_북마리아나제도:','pt':':bandeira_ilhas_marianas_do_norte:','it':':bandiera_isole_marianne_settentrionali:','fa':':پرچم_جزایر_ماریانای_شمالی:','id':':bendera_kepulauan_mariana_utara:','zh':':北马里亚纳群岛:','ru':':флаг_Северные_Марианские_о_ва:','tr':':bayrak_kuzey_mariana_adaları:','ar':':علم_جزر_ماريانا_الشمالية:'},'\U0001F1F3\U0001F1F4':{'en':':Norway:','status':fully_qualified,'E':2,'alias':[':flag_for_Norway:', ':norway:'],'de':':flagge_norwegen:','es':':bandera_noruega:','fr':':drapeau_norvège:','ja':':旗_ノルウェー:','ko':':깃발_노르웨이:','pt':':bandeira_noruega:','it':':bandiera_norvegia:','fa':':پرچم_نروژ:','id':':bendera_norwegia:','zh':':挪威:','ru':':флаг_Норвегия:','tr':':bayrak_norveç:','ar':':علم_النرويج:'},'\U0001F197':{'en':':OK_button:','status':fully_qualified,'E':0.6,'alias':[':ok:', ':ok_button:'],'de':':großbuchstaben_ok_in_blauem_quadrat:','es':':botón_ok:','fr':':bouton_ok:','ja':':okマーク:','ko':':오케이:','pt':':botão_ok:','it':':pulsante_ok:','fa':':دکمهٔ_ok:','id':':tombol_ok:','zh':':OK按钮:','ru':':значок_«ОК»:','tr':':ok_düğmesi:','ar':':زر_موافق:'},'\U0001F44C':{'en':':OK_hand:','status':fully_qualified,'E':0.6,'alias':[':ok_hand:'],'de':':ok-zeichen:','es':':señal_de_aprobación_con_la_mano:','fr':':ok:','ja':':okの手:','ko':':오케이를_표시한_손:','pt':':sinal_de_ok:','it':':mano_che_fa_ok:','fa':':علامت_اوکی:','id':':oke:','zh':':OK:','ru':':жест_«все_хорошо»:','tr':':tamam_işareti:','ar':':يد_تشير_للموافقة:'},'\U0001F44C\U0001F3FF':{'en':':OK_hand_dark_skin_tone:','status':fully_qualified,'E':1,'de':':ok-zeichen_dunkle_hautfarbe:','es':':señal_de_aprobación_con_la_mano_tono_de_piel_oscuro:','fr':':ok_peau_foncée:','ja':':okの手_濃い肌色:','ko':':오케이를_표시한_손_검은색_피부:','pt':':sinal_de_ok_pele_escura:','it':':mano_che_fa_ok_carnagione_scura:','fa':':علامت_اوکی_پوست_آبنوسی:','id':':oke_warna_kulit_gelap:','zh':':OK_较深肤色:','ru':':жест_«все_хорошо»_очень_темный_тон_кожи:','tr':':tamam_işareti_koyu_cilt_tonu:','ar':':يد_تشير_للموافقة_بشرة_بلون_غامق:'},'\U0001F44C\U0001F3FB':{'en':':OK_hand_light_skin_tone:','status':fully_qualified,'E':1,'de':':ok-zeichen_helle_hautfarbe:','es':':señal_de_aprobación_con_la_mano_tono_de_piel_claro:','fr':':ok_peau_claire:','ja':':okの手_薄い肌色:','ko':':오케이를_표시한_손_하얀_피부:','pt':':sinal_de_ok_pele_clara:','it':':mano_che_fa_ok_carnagione_chiara:','fa':':علامت_اوکی_پوست_سفید:','id':':oke_warna_kulit_cerah:','zh':':OK_较浅肤色:','ru':':жест_«все_хорошо»_очень_светлый_тон_кожи:','tr':':tamam_işareti_açık_cilt_tonu:','ar':':يد_تشير_للموافقة_بشرة_بلون_فاتح:'},'\U0001F44C\U0001F3FE':{'en':':OK_hand_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':ok-zeichen_mitteldunkle_hautfarbe:','es':':señal_de_aprobación_con_la_mano_tono_de_piel_oscuro_medio:','fr':':ok_peau_mate:','ja':':okの手_やや濃い肌色:','ko':':오케이를_표시한_손_진한_갈색_피부:','pt':':sinal_de_ok_pele_morena_escura:','it':':mano_che_fa_ok_carnagione_abbastanza_scura:','fa':':علامت_اوکی_پوست_گندمی:','id':':oke_warna_kulit_gelap-sedang:','zh':':OK_中等深肤色:','ru':':жест_«все_хорошо»_темный_тон_кожи:','tr':':tamam_işareti_orta_koyu_cilt_tonu:','ar':':يد_تشير_للموافقة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F44C\U0001F3FC':{'en':':OK_hand_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':ok-zeichen_mittelhelle_hautfarbe:','es':':señal_de_aprobación_con_la_mano_tono_de_piel_claro_medio:','fr':':ok_peau_moyennement_claire:','ja':':okの手_やや薄い肌色:','ko':':오케이를_표시한_손_연한_갈색_피부:','pt':':sinal_de_ok_pele_morena_clara:','it':':mano_che_fa_ok_carnagione_abbastanza_chiara:','fa':':علامت_اوکی_پوست_روشن:','id':':oke_warna_kulit_cerah-sedang:','zh':':OK_中等浅肤色:','ru':':жест_«все_хорошо»_светлый_тон_кожи:','tr':':tamam_işareti_orta_açık_cilt_tonu:','ar':':يد_تشير_للموافقة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F44C\U0001F3FD':{'en':':OK_hand_medium_skin_tone:','status':fully_qualified,'E':1,'de':':ok-zeichen_mittlere_hautfarbe:','es':':señal_de_aprobación_con_la_mano_tono_de_piel_medio:','fr':':ok_peau_légèrement_mate:','ja':':okの手_中間の肌色:','ko':':오케이를_표시한_손_갈색_피부:','pt':':sinal_de_ok_pele_morena:','it':':mano_che_fa_ok_carnagione_olivastra:','fa':':علامت_اوکی_پوست_طلایی:','id':':oke_warna_kulit_sedang:','zh':':OK_中等肤色:','ru':':жест_«все_хорошо»_средний_тон_кожи:','tr':':tamam_işareti_orta_cilt_tonu:','ar':':يد_تشير_للموافقة_بشرة_بلون_معتدل:'},'\U0001F51B':{'en':':ON!_arrow:','status':fully_qualified,'E':0.6,'alias':[':on:', ':on_arrow:', ':on!_arrow:'],'de':':on!-pfeil:','es':':flecha_on!:','fr':':flèche_activé:','ja':':on矢印:','ko':':켜짐:','pt':':seta_on!:','it':':freccia_on:','fa':':پیکان_روشن:','id':':tanda_panah_on!:','zh':':ON箭头:','ru':':стрелка_«активно»:','tr':':açık_oku:','ar':':سهم_تشغيل:'},'\U0001F17E\U0000FE0F':{'en':':O_button_(blood_type):','status':fully_qualified,'E':0.6,'alias':[':o2:', ':o_button_blood_type:'],'variant':True,'de':':großbuchstabe_o_in_rotem_quadrat:','es':':grupo_sanguíneo_tipo_o:','fr':':groupe_sanguin_o:','ja':':血液型o型:','ko':':오형:','pt':':botão_o_(tipo_sanguíneo):','it':':gruppo_sanguigno_0:','fa':':دکمه_ا_(گروه_خونی):','id':':tombol_o_(golongan_darah):','zh':':O型血:','ru':':i_группа_крови:','tr':':0_düğmesi_(kan_grubu):','ar':':زر_o:'},'\U0001F17E':{'en':':O_button_(blood_type):','status':unqualified,'E':0.6,'alias':[':o2:', ':o_button_blood_type:'],'variant':True,'de':':großbuchstabe_o_in_rotem_quadrat:','es':':grupo_sanguíneo_tipo_o:','fr':':groupe_sanguin_o:','ja':':血液型o型:','ko':':오형:','pt':':botão_o_(tipo_sanguíneo):','it':':gruppo_sanguigno_0:','fa':':دکمه_اُ_(گروه_خونی):','id':':tombol_o_(golongan_darah):','zh':':O型血:','ru':':I_группа_крови:','tr':':0_düğmesi_(kan_grubu):','ar':':زر_o:'},'\U0001F1F4\U0001F1F2':{'en':':Oman:','status':fully_qualified,'E':2,'alias':[':flag_for_Oman:', ':oman:'],'de':':flagge_oman:','es':':bandera_omán:','fr':':drapeau_oman:','ja':':旗_オマーン:','ko':':깃발_오만:','pt':':bandeira_omã:','it':':bandiera_oman:','fa':':پرچم_عمان:','id':':bendera_oman:','zh':':阿曼:','ru':':флаг_Оман:','tr':':bayrak_umman:','ar':':علم_عُمان:'},'\U000026CE':{'en':':Ophiuchus:','status':fully_qualified,'E':0.6,'alias':[':ophiuchus:'],'variant':True,'de':':schlangenträger:','es':':ofiuco:','fr':':serpentaire:','ja':':へびつかい座:','ko':':뱀주인자리:','pt':':signo_de_ofiúco:','it':':segno_zodiacale_dell’ofiuco:','fa':':برج_حوا:','id':':ophiuchus:','zh':':蛇夫座:','ru':':знак_зодиака_Змееносец:','tr':':yılancı:','ar':':الحواء:'},'\U0001F17F\U0000FE0F':{'en':':P_button:','status':fully_qualified,'E':0.6,'alias':[':parking:', ':p_button:'],'variant':True,'de':':großbuchstabe_p_in_blauem_quadrat:','es':':aparcamiento:','fr':':bouton_p:','ja':':pマーク:','ko':':주차_표시:','pt':':botão_p:','it':':pulsante_p:','fa':':دکمه_پی:','id':':tombol_p:','zh':':停车按钮:','ru':':парковка:','tr':':p_düğmesi:','ar':':انتظار_سيارات:'},'\U0001F17F':{'en':':P_button:','status':unqualified,'E':0.6,'alias':[':parking:', ':p_button:'],'variant':True,'de':':großbuchstabe_p_in_blauem_quadrat:','es':':aparcamiento:','fr':':bouton_p:','ja':':pマーク:','ko':':주차_표시:','pt':':botão_p:','it':':pulsante_p:','fa':':دکمه_پی:','id':':tombol_p:','zh':':停车按钮:','ru':':парковка:','tr':':p_düğmesi:','ar':':انتظار_سيارات:'},'\U0001F1F5\U0001F1F0':{'en':':Pakistan:','status':fully_qualified,'E':2,'alias':[':flag_for_Pakistan:', ':pakistan:'],'de':':flagge_pakistan:','es':':bandera_pakistán:','fr':':drapeau_pakistan:','ja':':旗_パキスタン:','ko':':깃발_파키스탄:','pt':':bandeira_paquistão:','it':':bandiera_pakistan:','fa':':پرچم_پاکستان:','id':':bendera_pakistan:','zh':':巴基斯坦:','ru':':флаг_Пакистан:','tr':':bayrak_pakistan:','ar':':علم_باكستان:'},'\U0001F1F5\U0001F1FC':{'en':':Palau:','status':fully_qualified,'E':2,'alias':[':flag_for_Palau:', ':palau:'],'de':':flagge_palau:','es':':bandera_palaos:','fr':':drapeau_palaos:','ja':':旗_パラオ:','ko':':깃발_팔라우:','pt':':bandeira_palau:','it':':bandiera_palau:','fa':':پرچم_پالائو:','id':':bendera_palau:','zh':':帕劳:','ru':':флаг_Палау:','tr':':bayrak_palau:','ar':':علم_بالاو:'},'\U0001F1F5\U0001F1F8':{'en':':Palestinian_Territories:','status':fully_qualified,'E':2,'alias':[':flag_for_Palestinian_Territories:', ':palestinian_territories:'],'de':':flagge_palästinensische_autonomiegebiete:','es':':bandera_territorios_palestinos:','fr':':drapeau_territoires_palestiniens:','ja':':旗_パレスチナ自治区:','ko':':깃발_팔레스타인_지구:','pt':':bandeira_territórios_palestinos:','it':':bandiera_territori_palestinesi:','fa':':پرچم_سرزمین_های_فلسطینی:','id':':bendera_wilayah_palestina:','zh':':巴勒斯坦领土:','ru':':флаг_Палестинские_территории:','tr':':bayrak_filistin_bölgeleri:','ar':':علم_الأراضي_الفلسطينية:'},'\U0001F1F5\U0001F1E6':{'en':':Panama:','status':fully_qualified,'E':2,'alias':[':flag_for_Panama:', ':panama:'],'de':':flagge_panama:','es':':bandera_panamá:','fr':':drapeau_panama:','ja':':旗_パナマ:','ko':':깃발_파나마:','pt':':bandeira_panamá:','it':':bandiera_panama:','fa':':پرچم_پاناما:','id':':bendera_panama:','zh':':巴拿马:','ru':':флаг_Панама:','tr':':bayrak_panama:','ar':':علم_بنما:'},'\U0001F1F5\U0001F1EC':{'en':':Papua_New_Guinea:','status':fully_qualified,'E':2,'alias':[':flag_for_Papua_New_Guinea:', ':papua_new_guinea:'],'de':':flagge_papua-neuguinea:','es':':bandera_papúa_nueva_guinea:','fr':':drapeau_papouasie-nouvelle-guinée:','ja':':旗_パプアニューギニア:','ko':':깃발_파푸아뉴기니:','pt':':bandeira_papua-nova_guiné:','it':':bandiera_papua_nuova_guinea:','fa':':پرچم_پاپوا_گینهٔ_نو:','id':':bendera_papua_nugini:','zh':':巴布亚新几内亚:','ru':':флаг_Папуа_Новая_Гвинея:','tr':':bayrak_papua_yeni_gine:','ar':':علم_بابوا_غينيا_الجديدة:'},'\U0001F1F5\U0001F1FE':{'en':':Paraguay:','status':fully_qualified,'E':2,'alias':[':flag_for_Paraguay:', ':paraguay:'],'de':':flagge_paraguay:','es':':bandera_paraguay:','fr':':drapeau_paraguay:','ja':':旗_パラグアイ:','ko':':깃발_파라과이:','pt':':bandeira_paraguai:','it':':bandiera_paraguay:','fa':':پرچم_پاراگوئه:','id':':bendera_paraguay:','zh':':巴拉圭:','ru':':флаг_Парагвай:','tr':':bayrak_paraguay:','ar':':علم_باراغواي:'},'\U0001F1F5\U0001F1EA':{'en':':Peru:','status':fully_qualified,'E':2,'alias':[':flag_for_Peru:', ':peru:'],'de':':flagge_peru:','es':':bandera_perú:','fr':':drapeau_pérou:','ja':':旗_ペルー:','ko':':깃발_페루:','pt':':bandeira_peru:','it':':bandiera_perù:','fa':':پرچم_پرو:','id':':bendera_peru:','zh':':秘鲁:','ru':':флаг_Перу:','tr':':bayrak_peru:','ar':':علم_بيرو:'},'\U0001F1F5\U0001F1ED':{'en':':Philippines:','status':fully_qualified,'E':2,'alias':[':flag_for_Philippines:', ':philippines:'],'de':':flagge_philippinen:','es':':bandera_filipinas:','fr':':drapeau_philippines:','ja':':旗_フィリピン:','ko':':깃발_필리핀:','pt':':bandeira_filipinas:','it':':bandiera_filippine:','fa':':پرچم_فیلیپین:','id':':bendera_filipina:','zh':':菲律宾:','ru':':флаг_Филиппины:','tr':':bayrak_filipinler:','ar':':علم_الفلبين:'},'\U00002653':{'en':':Pisces:','status':fully_qualified,'E':0.6,'alias':[':pisces:'],'variant':True,'de':':fische_(sternzeichen):','es':':piscis:','fr':':poissons:','ja':':うお座:','ko':':물고기자리:','pt':':signo_de_peixes:','it':':segno_zodiacale_dei_pesci:','fa':':برج_حوت:','id':':pisces:','zh':':双鱼座:','ru':':знак_зодиака_Рыбы:','tr':':balık_burcu:','ar':':الحوت:'},'\U0001F1F5\U0001F1F3':{'en':':Pitcairn_Islands:','status':fully_qualified,'E':2,'alias':[':flag_for_Pitcairn_Islands:', ':pitcairn_islands:'],'de':':flagge_pitcairninseln:','es':':bandera_islas_pitcairn:','fr':':drapeau_îles_pitcairn:','ja':':旗_ピトケアン諸島:','ko':':깃발_핏케언_제도:','pt':':bandeira_ilhas_pitcairn:','it':':bandiera_isole_pitcairn:','fa':':پرچم_جزایر_پیت_کرن:','id':':bendera_kepulauan_pitcairn:','zh':':皮特凯恩群岛:','ru':':флаг_о_ва_Питкэрн:','tr':':bayrak_pitcairn_adaları:','ar':':علم_جزر_بيتكيرن:'},'\U0001F1F5\U0001F1F1':{'en':':Poland:','status':fully_qualified,'E':2,'alias':[':flag_for_Poland:', ':poland:'],'de':':flagge_polen:','es':':bandera_polonia:','fr':':drapeau_pologne:','ja':':旗_ポーランド:','ko':':깃발_폴란드:','pt':':bandeira_polônia:','it':':bandiera_polonia:','fa':':پرچم_لهستان:','id':':bendera_polandia:','zh':':波兰:','ru':':флаг_Польша:','tr':':bayrak_polonya:','ar':':علم_بولندا:'},'\U0001F1F5\U0001F1F9':{'en':':Portugal:','status':fully_qualified,'E':2,'alias':[':flag_for_Portugal:', ':portugal:'],'de':':flagge_portugal:','es':':bandera_portugal:','fr':':drapeau_portugal:','ja':':旗_ポルトガル:','ko':':깃발_포르투갈:','pt':':bandeira_portugal:','it':':bandiera_portogallo:','fa':':پرچم_پرتغال:','id':':bendera_portugal:','zh':':葡萄牙:','ru':':флаг_Португалия:','tr':':bayrak_portekiz:','ar':':علم_البرتغال:'},'\U0001F1F5\U0001F1F7':{'en':':Puerto_Rico:','status':fully_qualified,'E':2,'alias':[':flag_for_Puerto_Rico:', ':puerto_rico:'],'de':':flagge_puerto_rico:','es':':bandera_puerto_rico:','fr':':drapeau_porto_rico:','ja':':旗_プエルトリコ:','ko':':깃발_푸에르토리코:','pt':':bandeira_porto_rico:','it':':bandiera_portorico:','fa':':پرچم_پورتوریکو:','id':':bendera_puerto_riko:','zh':':波多黎各:','ru':':флаг_Пуэрто_Рико:','tr':':bayrak_porto_riko:','ar':':علم_بورتوريكو:'},'\U0001F1F6\U0001F1E6':{'en':':Qatar:','status':fully_qualified,'E':2,'alias':[':flag_for_Qatar:', ':qatar:'],'de':':flagge_katar:','es':':bandera_catar:','fr':':drapeau_qatar:','ja':':旗_カタール:','ko':':깃발_카타르:','pt':':bandeira_catar:','it':':bandiera_qatar:','fa':':پرچم_قطر:','id':':bendera_qatar:','zh':':卡塔尔:','ru':':флаг_Катар:','tr':':bayrak_katar:','ar':':علم_قطر:'},'\U0001F1F7\U0001F1F4':{'en':':Romania:','status':fully_qualified,'E':2,'alias':[':flag_for_Romania:', ':romania:'],'de':':flagge_rumänien:','es':':bandera_rumanía:','fr':':drapeau_roumanie:','ja':':旗_ルーマニア:','ko':':깃발_루마니아:','pt':':bandeira_romênia:','it':':bandiera_romania:','fa':':پرچم_رومانی:','id':':bendera_rumania:','zh':':罗马尼亚:','ru':':флаг_Румыния:','tr':':bayrak_romanya:','ar':':علم_رومانيا:'},'\U0001F1F7\U0001F1FA':{'en':':Russia:','status':fully_qualified,'E':0.6,'alias':[':flag_for_Russia:', ':ru:'],'de':':flagge_russland:','es':':bandera_rusia:','fr':':drapeau_russie:','ja':':旗_ロシア:','ko':':깃발_러시아:','pt':':bandeira_rússia:','it':':bandiera_russia:','fa':':پرچم_روسیه:','id':':bendera_rusia:','zh':':俄罗斯:','ru':':флаг_Россия:','tr':':bayrak_rusya:','ar':':علم_روسيا:'},'\U0001F1F7\U0001F1FC':{'en':':Rwanda:','status':fully_qualified,'E':2,'alias':[':flag_for_Rwanda:', ':rwanda:'],'de':':flagge_ruanda:','es':':bandera_ruanda:','fr':':drapeau_rwanda:','ja':':旗_ルワンダ:','ko':':깃발_르완다:','pt':':bandeira_ruanda:','it':':bandiera_ruanda:','fa':':پرچم_رواندا:','id':':bendera_rwanda:','zh':':卢旺达:','ru':':флаг_Руанда:','tr':':bayrak_ruanda:','ar':':علم_رواندا:'},'\U0001F1F7\U0001F1EA':{'en':':Réunion:','status':fully_qualified,'E':2,'alias':[':flag_for_Réunion:', ':reunion:'],'de':':flagge_réunion:','es':':bandera_reunión:','fr':':drapeau_la_réunion:','ja':':旗_レユニオン:','ko':':깃발_레위니옹:','pt':':bandeira_reunião:','it':':bandiera_riunione:','fa':':پرچم_رئونیون:','id':':bendera_réunion:','zh':':留尼汪:','ru':':флаг_Реюньон:','tr':':bayrak_reunion:','ar':':علم_روينيون:'},'\U0001F51C':{'en':':SOON_arrow:','status':fully_qualified,'E':0.6,'alias':[':soon:', ':soon_arrow:'],'de':':soon-pfeil:','es':':flecha_soon:','fr':':flèche_bientôt:','ja':':soon矢印:','ko':':곧:','pt':':seta_soon:','it':':freccia_soon:','fa':':پیکان_به_زودی:','id':':tanda_panah_soon:','zh':':SOON箭头:','ru':':стрелка_«скоро»:','tr':':yakında_işareti:','ar':':سهم_علامة_قريبا:'},'\U0001F198':{'en':':SOS_button:','status':fully_qualified,'E':0.6,'alias':[':sos:', ':sos_button:'],'de':':sos-zeichen:','es':':símbolo_de_socorro:','fr':':bouton_sos:','ja':':sosマーク:','ko':':에스오에스:','pt':':botão_sos:','it':':pulsante_sos:','fa':':دکمه_اس_او_اس:','id':':tombol_sos:','zh':':SOS按钮:','ru':':значок_SOS:','tr':':sos_düğmesi:','ar':':زر_طوارئ:'},'\U00002650':{'en':':Sagittarius:','status':fully_qualified,'E':0.6,'alias':[':sagittarius:'],'variant':True,'de':':schütze_(sternzeichen):','es':':sagitario:','fr':':sagittaire:','ja':':いて座:','ko':':사수자리:','pt':':signo_de_sagitário:','it':':segno_zodiacale_del_saggitario:','fa':':برج_قوس:','id':':sagitarius:','zh':':射手座:','ru':':знак_зодиака_Стрелец:','tr':':yay_burcu:','ar':':القوس:'},'\U0001F1FC\U0001F1F8':{'en':':Samoa:','status':fully_qualified,'E':2,'alias':[':flag_for_Samoa:', ':samoa:'],'de':':flagge_samoa:','es':':bandera_samoa:','fr':':drapeau_samoa:','ja':':旗_サモア:','ko':':깃발_사모아:','pt':':bandeira_samoa:','it':':bandiera_samoa:','fa':':پرچم_ساموآ:','id':':bendera_samoa:','zh':':萨摩亚:','ru':':флаг_Самоа:','tr':':bayrak_samoa:','ar':':علم_ساموا:'},'\U0001F1F8\U0001F1F2':{'en':':San_Marino:','status':fully_qualified,'E':2,'alias':[':flag_for_San_Marino:', ':san_marino:'],'de':':flagge_san_marino:','es':':bandera_san_marino:','fr':':drapeau_saint-marin:','ja':':旗_サンマリノ:','ko':':깃발_산마리노:','pt':':bandeira_san_marino:','it':':bandiera_san_marino:','fa':':پرچم_سان_مارینو:','id':':bendera_san_marino:','zh':':圣马力诺:','ru':':флаг_Сан_Марино:','tr':':bayrak_san_marino:','ar':':علم_سان_مارينو:'},'\U0001F385':{'en':':Santa_Claus:','status':fully_qualified,'E':0.6,'alias':[':santa:', ':santa_claus:'],'de':':weihnachtsmann:','es':':papá_noel:','fr':':père_noël:','ja':':サンタ:','ko':':산타클로스:','pt':':papai_noel:','it':':babbo_natale:','fa':':بابا_نوئل:','id':':sinterklas:','zh':':圣诞老人:','ru':':Санта_Клаус:','tr':':noel_baba:','ar':':بابا_نويل:'},'\U0001F385\U0001F3FF':{'en':':Santa_Claus_dark_skin_tone:','status':fully_qualified,'E':1,'de':':weihnachtsmann_dunkle_hautfarbe:','es':':papá_noel_tono_de_piel_oscuro:','fr':':père_noël_peau_foncée:','ja':':サンタ_濃い肌色:','ko':':산타클로스_검은색_피부:','pt':':papai_noel_pele_escura:','it':':babbo_natale_carnagione_scura:','fa':':بابا_نوئل_پوست_آبنوسی:','id':':sinterklas_warna_kulit_gelap:','zh':':圣诞老人_较深肤色:','ru':':Санта_Клаус_очень_темный_тон_кожи:','tr':':noel_baba_koyu_cilt_tonu:','ar':':بابا_نويل_بشرة_بلون_غامق:'},'\U0001F385\U0001F3FB':{'en':':Santa_Claus_light_skin_tone:','status':fully_qualified,'E':1,'de':':weihnachtsmann_helle_hautfarbe:','es':':papá_noel_tono_de_piel_claro:','fr':':père_noël_peau_claire:','ja':':サンタ_薄い肌色:','ko':':산타클로스_하얀_피부:','pt':':papai_noel_pele_clara:','it':':babbo_natale_carnagione_chiara:','fa':':بابا_نوئل_پوست_سفید:','id':':sinterklas_warna_kulit_cerah:','zh':':圣诞老人_较浅肤色:','ru':':Санта_Клаус_очень_светлый_тон_кожи:','tr':':noel_baba_açık_cilt_tonu:','ar':':بابا_نويل_بشرة_بلون_فاتح:'},'\U0001F385\U0001F3FE':{'en':':Santa_Claus_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':weihnachtsmann_mitteldunkle_hautfarbe:','es':':papá_noel_tono_de_piel_oscuro_medio:','fr':':père_noël_peau_mate:','ja':':サンタ_やや濃い肌色:','ko':':산타클로스_진한_갈색_피부:','pt':':papai_noel_pele_morena_escura:','it':':babbo_natale_carnagione_abbastanza_scura:','fa':':بابا_نوئل_پوست_گندمی:','id':':sinterklas_warna_kulit_gelap-sedang:','zh':':圣诞老人_中等深肤色:','ru':':Санта_Клаус_темный_тон_кожи:','tr':':noel_baba_orta_koyu_cilt_tonu:','ar':':بابا_نويل_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F385\U0001F3FC':{'en':':Santa_Claus_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':weihnachtsmann_mittelhelle_hautfarbe:','es':':papá_noel_tono_de_piel_claro_medio:','fr':':père_noël_peau_moyennement_claire:','ja':':サンタ_やや薄い肌色:','ko':':산타클로스_연한_갈색_피부:','pt':':papai_noel_pele_morena_clara:','it':':babbo_natale_carnagione_abbastanza_chiara:','fa':':بابا_نوئل_پوست_روشن:','id':':sinterklas_warna_kulit_cerah-sedang:','zh':':圣诞老人_中等浅肤色:','ru':':Санта_Клаус_светлый_тон_кожи:','tr':':noel_baba_orta_açık_cilt_tonu:','ar':':بابا_نويل_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F385\U0001F3FD':{'en':':Santa_Claus_medium_skin_tone:','status':fully_qualified,'E':1,'de':':weihnachtsmann_mittlere_hautfarbe:','es':':papá_noel_tono_de_piel_medio:','fr':':père_noël_peau_légèrement_mate:','ja':':サンタ_中間の肌色:','ko':':산타클로스_갈색_피부:','pt':':papai_noel_pele_morena:','it':':babbo_natale_carnagione_olivastra:','fa':':بابا_نوئل_پوست_طلایی:','id':':sinterklas_warna_kulit_sedang:','zh':':圣诞老人_中等肤色:','ru':':Санта_Клаус_средний_тон_кожи:','tr':':noel_baba_orta_cilt_tonu:','ar':':بابا_نويل_بشرة_بلون_معتدل:'},'\U0001F1F8\U0001F1E6':{'en':':Saudi_Arabia:','status':fully_qualified,'E':2,'alias':[':flag_for_Saudi_Arabia:', ':saudi_arabia:'],'de':':flagge_saudi-arabien:','es':':bandera_arabia_saudí:','fr':':drapeau_arabie_saoudite:','ja':':旗_サウジアラビア:','ko':':깃발_사우디아라비아:','pt':':bandeira_arábia_saudita:','it':':bandiera_arabia_saudita:','fa':':پرچم_عربستان_سعودی:','id':':bendera_arab_saudi:','zh':':沙特阿拉伯:','ru':':флаг_Саудовская_Аравия:','tr':':bayrak_suudi_arabistan:','ar':':علم_المملكة_العربية_السعودية:'},'\U0000264F':{'en':':Scorpio:','status':fully_qualified,'E':0.6,'alias':[':scorpius:', ':scorpio:'],'variant':True,'de':':skorpion_(sternzeichen):','es':':escorpio:','fr':':scorpion_zodiaque:','ja':':さそり座:','ko':':전갈자리:','pt':':signo_de_escorpião:','it':':segno_zodiacale_dello_scorpione:','fa':':برج_عقرب:','id':':scorpio:','zh':':天蝎座:','ru':':знак_зодиака_Скорпион:','tr':':akrep_burcu:','ar':':العقرب:'},'\U0001F3F4\U000E0067\U000E0062\U000E0073\U000E0063\U000E0074\U000E007F':{'en':':Scotland:','status':fully_qualified,'E':5,'alias':[':scotland:'],'de':':flagge_schottland:','es':':bandera_escocia:','fr':':drapeau_écosse:','ja':':旗_スコットランド:','ko':':깃발_스코틀랜드:','pt':':bandeira_escócia:','it':':bandiera_scozia:','fa':':پرچم_اسکاتلند:','id':':bendera_skotlandia:','zh':':苏格兰:','ru':':флаг_Шотландия:','tr':':bayrak_iskoçya:','ar':':علم_اسكتلندا:'},'\U0001F1F8\U0001F1F3':{'en':':Senegal:','status':fully_qualified,'E':2,'alias':[':flag_for_Senegal:', ':senegal:'],'de':':flagge_senegal:','es':':bandera_senegal:','fr':':drapeau_sénégal:','ja':':旗_セネガル:','ko':':깃발_세네갈:','pt':':bandeira_senegal:','it':':bandiera_senegal:','fa':':پرچم_سنگال:','id':':bendera_senegal:','zh':':塞内加尔:','ru':':флаг_Сенегал:','tr':':bayrak_senegal:','ar':':علم_السنغال:'},'\U0001F1F7\U0001F1F8':{'en':':Serbia:','status':fully_qualified,'E':2,'alias':[':flag_for_Serbia:', ':serbia:'],'de':':flagge_serbien:','es':':bandera_serbia:','fr':':drapeau_serbie:','ja':':旗_セルビア:','ko':':깃발_세르비아:','pt':':bandeira_sérvia:','it':':bandiera_serbia:','fa':':پرچم_صربستان:','id':':bendera_serbia:','zh':':塞尔维亚:','ru':':флаг_Сербия:','tr':':bayrak_sırbistan:','ar':':علم_صربيا:'},'\U0001F1F8\U0001F1E8':{'en':':Seychelles:','status':fully_qualified,'E':2,'alias':[':flag_for_Seychelles:', ':seychelles:'],'de':':flagge_seychellen:','es':':bandera_seychelles:','fr':':drapeau_seychelles:','ja':':旗_セーシェル:','ko':':깃발_세이셸:','pt':':bandeira_seicheles:','it':':bandiera_seychelles:','fa':':پرچم_سیشل:','id':':bendera_seychelles:','zh':':塞舌尔:','ru':':флаг_Сейшельские_о_ва:','tr':':bayrak_seyşeller:','ar':':علم_سيشل:'},'\U0001F1F8\U0001F1F1':{'en':':Sierra_Leone:','status':fully_qualified,'E':2,'alias':[':flag_for_Sierra_Leone:', ':sierra_leone:'],'de':':flagge_sierra_leone:','es':':bandera_sierra_leona:','fr':':drapeau_sierra_leone:','ja':':旗_シエラレオネ:','ko':':깃발_시에라리온:','pt':':bandeira_serra_leoa:','it':':bandiera_sierra_leone:','fa':':پرچم_سیرالئون:','id':':bendera_sierra_leone:','zh':':塞拉利昂:','ru':':флаг_Сьерра_Леоне:','tr':':bayrak_sierra_leone:','ar':':علم_سيراليون:'},'\U0001F1F8\U0001F1EC':{'en':':Singapore:','status':fully_qualified,'E':2,'alias':[':flag_for_Singapore:', ':singapore:'],'de':':flagge_singapur:','es':':bandera_singapur:','fr':':drapeau_singapour:','ja':':旗_シンガポール:','ko':':깃발_싱가포르:','pt':':bandeira_singapura:','it':':bandiera_singapore:','fa':':پرچم_سنگاپور:','id':':bendera_singapura:','zh':':新加坡:','ru':':флаг_Сингапур:','tr':':bayrak_singapur:','ar':':علم_سنغافورة:'},'\U0001F1F8\U0001F1FD':{'en':':Sint_Maarten:','status':fully_qualified,'E':2,'alias':[':flag_for_Sint_Maarten:', ':sint_maarten:'],'de':':flagge_sint_maarten:','es':':bandera_sint_maarten:','fr':':drapeau_saint-martin_(partie_néerlandaise):','ja':':旗_シント・マールテン:','ko':':깃발_신트마르턴:','pt':':bandeira_sint_maarten:','it':':bandiera_sint_maarten:','fa':':پرچم_سنت_مارتن:','id':':bendera_sint_maarten:','zh':':荷属圣马丁:','ru':':флаг_Синт_Мартен:','tr':':bayrak_sint_maarten:','ar':':علم_سانت_مارتن:'},'\U0001F1F8\U0001F1F0':{'en':':Slovakia:','status':fully_qualified,'E':2,'alias':[':flag_for_Slovakia:', ':slovakia:'],'de':':flagge_slowakei:','es':':bandera_eslovaquia:','fr':':drapeau_slovaquie:','ja':':旗_スロバキア:','ko':':깃발_슬로바키아:','pt':':bandeira_eslováquia:','it':':bandiera_slovacchia:','fa':':پرچم_اسلواکی:','id':':bendera_slovakia:','zh':':斯洛伐克:','ru':':флаг_Словакия:','tr':':bayrak_slovakya:','ar':':علم_سلوفاكيا:'},'\U0001F1F8\U0001F1EE':{'en':':Slovenia:','status':fully_qualified,'E':2,'alias':[':flag_for_Slovenia:', ':slovenia:'],'de':':flagge_slowenien:','es':':bandera_eslovenia:','fr':':drapeau_slovénie:','ja':':旗_スロベニア:','ko':':깃발_슬로베니아:','pt':':bandeira_eslovênia:','it':':bandiera_slovenia:','fa':':پرچم_اسلوونی:','id':':bendera_slovenia:','zh':':斯洛文尼亚:','ru':':флаг_Словения:','tr':':bayrak_slovenya:','ar':':علم_سلوفينيا:'},'\U0001F1F8\U0001F1E7':{'en':':Solomon_Islands:','status':fully_qualified,'E':2,'alias':[':flag_for_Solomon_Islands:', ':solomon_islands:'],'de':':flagge_salomonen:','es':':bandera_islas_salomón:','fr':':drapeau_îles_salomon:','ja':':旗_ソロモン諸島:','ko':':깃발_솔로몬_제도:','pt':':bandeira_ilhas_salomão:','it':':bandiera_isole_salomone:','fa':':پرچم_جزایر_سلیمان:','id':':bendera_kepulauan_solomon:','zh':':所罗门群岛:','ru':':флаг_Соломоновы_о_ва:','tr':':bayrak_solomon_adaları:','ar':':علم_جزر_سليمان:'},'\U0001F1F8\U0001F1F4':{'en':':Somalia:','status':fully_qualified,'E':2,'alias':[':flag_for_Somalia:', ':somalia:'],'de':':flagge_somalia:','es':':bandera_somalia:','fr':':drapeau_somalie:','ja':':旗_ソマリア:','ko':':깃발_소말리아:','pt':':bandeira_somália:','it':':bandiera_somalia:','fa':':پرچم_سومالی:','id':':bendera_somalia:','zh':':索马里:','ru':':флаг_Сомали:','tr':':bayrak_somali:','ar':':علم_الصومال:'},'\U0001F1FF\U0001F1E6':{'en':':South_Africa:','status':fully_qualified,'E':2,'alias':[':flag_for_South_Africa:', ':south_africa:'],'de':':flagge_südafrika:','es':':bandera_sudáfrica:','fr':':drapeau_afrique_du_sud:','ja':':旗_南アフリカ:','ko':':깃발_남아프리카:','pt':':bandeira_áfrica_do_sul:','it':':bandiera_sudafrica:','fa':':پرچم_افریقای_جنوبی:','id':':bendera_afrika_selatan:','zh':':南非:','ru':':флаг_Южно_Африканская_Республика:','tr':':bayrak_güney_afrika:','ar':':علم_جنوب_أفريقيا:'},'\U0001F1EC\U0001F1F8':{'en':':South_Georgia_&_South_Sandwich_Islands:','status':fully_qualified,'E':2,'alias':[':flag_for_South_Georgia_&_South_Sandwich_Islands:', ':south_georgia_south_sandwich_islands:'],'de':':flagge_südgeorgien_und_die_südlichen_sandwichinseln:','es':':bandera_islas_georgia_del_sur_y_sandwich_del_sur:','fr':':drapeau_géorgie_du_sud-et-les_îles_sandwich_du_sud:','ja':':旗_サウスジョージア・サウスサンドウィッチ諸島:','ko':':깃발_사우스조지아_사우스샌드위치_제도:','pt':':bandeira_ilhas_geórgia_do_sul_e_sandwich_do_sul:','it':':bandiera_georgia_del_sud_e_sandwich_australi:','fa':':پرچم_جورجیای_جنوبی_و_جزایر_ساندویچ_جنوبی:','id':':bendera_georgia_selatan_&_kep._sandwich_selatan:','zh':':南乔治亚和南桑威奇群岛:','ru':':флаг_Южная_Георгия_и_Южные_Сандвичевы_о_ва:','tr':':bayrak_güney_georgia_ve_güney_sandwich_adaları:','ar':':علم_جورجيا_الجنوبية_وجزر_ساندويتش_الجنوبية:'},'\U0001F1F0\U0001F1F7':{'en':':South_Korea:','status':fully_qualified,'E':0.6,'alias':[':flag_for_South_Korea:', ':kr:'],'de':':flagge_südkorea:','es':':bandera_corea_del_sur:','fr':':drapeau_corée_du_sud:','ja':':旗_韓国:','ko':':깃발_대한민국:','pt':':bandeira_coreia_do_sul:','it':':bandiera_corea_del_sud:','fa':':پرچم_کرهٔ_جنوبی:','id':':bendera_korea_selatan:','zh':':韩国:','ru':':флаг_Республика_Корея:','tr':':bayrak_güney_kore:','ar':':علم_كوريا_الجنوبية:'},'\U0001F1F8\U0001F1F8':{'en':':South_Sudan:','status':fully_qualified,'E':2,'alias':[':flag_for_South_Sudan:', ':south_sudan:'],'de':':flagge_südsudan:','es':':bandera_sudán_del_sur:','fr':':drapeau_soudan_du_sud:','ja':':旗_南スーダン:','ko':':깃발_남수단:','pt':':bandeira_sudão_do_sul:','it':':bandiera_sud_sudan:','fa':':پرچم_سودان_جنوبی:','id':':bendera_sudan_selatan:','zh':':南苏丹:','ru':':флаг_Южный_Судан:','tr':':bayrak_güney_sudan:','ar':':علم_جنوب_السودان:'},'\U0001F1EA\U0001F1F8':{'en':':Spain:','status':fully_qualified,'E':0.6,'alias':[':flag_for_Spain:', ':es:'],'de':':flagge_spanien:','es':':bandera_españa:','fr':':drapeau_espagne:','ja':':旗_スペイン:','ko':':깃발_스페인:','pt':':bandeira_espanha:','it':':bandiera_spagna:','fa':':پرچم_اسپانیا:','id':':bendera_spanyol:','zh':':西班牙:','ru':':флаг_Испания:','tr':':bayrak_ispanya:','ar':':علم_إسبانيا:'},'\U0001F1F1\U0001F1F0':{'en':':Sri_Lanka:','status':fully_qualified,'E':2,'alias':[':flag_for_Sri_Lanka:', ':sri_lanka:'],'de':':flagge_sri_lanka:','es':':bandera_sri_lanka:','fr':':drapeau_sri_lanka:','ja':':旗_スリランカ:','ko':':깃발_스리랑카:','pt':':bandeira_sri_lanka:','it':':bandiera_sri_lanka:','fa':':پرچم_سری_لانکا:','id':':bendera_sri_lanka:','zh':':斯里兰卡:','ru':':флаг_Шри_Ланка:','tr':':bayrak_sri_lanka:','ar':':علم_سريلانكا:'},'\U0001F1E7\U0001F1F1':{'en':':St._Barthélemy:','status':fully_qualified,'E':2,'alias':[':flag_for_St._Barthélemy:', ':st_barthelemy:'],'de':':flagge_st._barthélemy:','es':':bandera_san_bartolomé:','fr':':drapeau_saint-barthélemy:','ja':':旗_サン・バルテルミー:','ko':':깃발_생바르텔레미:','pt':':bandeira_são_bartolomeu:','it':':bandiera_saint-barthélemy:','fa':':پرچم_سن_بارتلمی:','id':':bendera_saint_barthélemy:','zh':':圣巴泰勒米:','ru':':флаг_Сен_Бартелеми:','tr':':bayrak_saint_barthelemy:','ar':':علم_سان_بارتليمي:'},'\U0001F1F8\U0001F1ED':{'en':':St._Helena:','status':fully_qualified,'E':2,'alias':[':flag_for_St._Helena:', ':st_helena:'],'de':':flagge_st._helena:','es':':bandera_santa_elena:','fr':':drapeau_sainte-hélène:','ja':':旗_セントヘレナ:','ko':':깃발_세인트헬레나:','pt':':bandeira_santa_helena:','it':':bandiera_sant’elena:','fa':':پرچم_سنت_هلن:','id':':bendera_saint_helena:','zh':':圣赫勒拿:','ru':':флаг_о_в_Св._Елены:','tr':':bayrak_saint_helena:','ar':':علم_سانت_هيلينا:'},'\U0001F1F0\U0001F1F3':{'en':':St._Kitts_&_Nevis:','status':fully_qualified,'E':2,'alias':[':flag_for_St._Kitts_&_Nevis:', ':st_kitts_nevis:'],'de':':flagge_st._kitts_und_nevis:','es':':bandera_san_cristóbal_y_nieves:','fr':':drapeau_saint-christophe-et-niévès:','ja':':旗_セントクリストファー・ネーヴィス:','ko':':깃발_세인트키츠_네비스:','pt':':bandeira_são_cristóvão_e_névis:','it':':bandiera_saint_kitts_e_nevis:','fa':':پرچم_سنت_کیتس_و_نویس:','id':':bendera_saint_kitts_dan_nevis:','zh':':圣基茨和尼维斯:','ru':':флаг_Сент_Китс_и_Невис:','tr':':bayrak_saint_kitts_ve_nevis:','ar':':علم_سانت_كيتس_ونيفيس:'},'\U0001F1F1\U0001F1E8':{'en':':St._Lucia:','status':fully_qualified,'E':2,'alias':[':flag_for_St._Lucia:', ':st_lucia:'],'de':':flagge_st._lucia:','es':':bandera_santa_lucía:','fr':':drapeau_sainte-lucie:','ja':':旗_セントルシア:','ko':':깃발_세인트루시아:','pt':':bandeira_santa_lúcia:','it':':bandiera_saint_lucia:','fa':':پرچم_سنت_لوسیا:','id':':bendera_saint_lucia:','zh':':圣卢西亚:','ru':':флаг_Сент_Люсия:','tr':':bayrak_saint_lucia:','ar':':علم_سانت_لوسيا:'},'\U0001F1F2\U0001F1EB':{'en':':St._Martin:','status':fully_qualified,'E':2,'alias':[':flag_for_St._Martin:', ':st_martin:'],'de':':flagge_st._martin:','es':':bandera_san_martín:','fr':':drapeau_saint-martin:','ja':':旗_サン・マルタン:','ko':':깃발_생마르탱:','pt':':bandeira_são_martinho:','it':':bandiera_saint_martin:','fa':':پرچم_سنت_مارتین:','id':':bendera_saint_martin:','zh':':法属圣马丁:','ru':':флаг_Сен_Мартен:','tr':':bayrak_saint_martin:','ar':':علم_سان_مارتن:'},'\U0001F1F5\U0001F1F2':{'en':':St._Pierre_&_Miquelon:','status':fully_qualified,'E':2,'alias':[':flag_for_St._Pierre_&_Miquelon:', ':st_pierre_miquelon:'],'de':':flagge_st._pierre_und_miquelon:','es':':bandera_san_pedro_y_miquelón:','fr':':drapeau_saint-pierre-et-miquelon:','ja':':旗_サンピエール島・ミクロン島:','ko':':깃발_생피에르_미클롱:','pt':':bandeira_são_pedro_e_miquelão:','it':':bandiera_saint-pierre_e_miquelon:','fa':':پرچم_سن_پیر_و_میکلن:','id':':bendera_saint_pierre_dan_miquelon:','zh':':圣皮埃尔和密克隆群岛:','ru':':флаг_Сен_Пьер_и_Микелон:','tr':':bayrak_saint_pierre_ve_miquelon:','ar':':علم_سان_بيير_ومكويلون:'},'\U0001F1FB\U0001F1E8':{'en':':St._Vincent_&_Grenadines:','status':fully_qualified,'E':2,'alias':[':flag_for_St._Vincent_&_Grenadines:', ':st_vincent_grenadines:'],'de':':flagge_st._vincent_und_die_grenadinen:','es':':bandera_san_vicente_y_las_granadinas:','fr':':drapeau_saint-vincent-et-les_grenadines:','ja':':旗_セントビンセント及びグレナディーン諸島:','ko':':깃발_세인트빈센트그레나딘:','pt':':bandeira_são_vicente_e_granadinas:','it':':bandiera_saint_vincent_e_grenadine:','fa':':پرچم_سنت_وینسنت_و_گرنادین:','id':':bendera_saint_vincent_dan_grenadine:','zh':':圣文森特和格林纳丁斯:','ru':':флаг_Сент_Винсент_и_Гренадины:','tr':':bayrak_saint_vincent_ve_grenadinler:','ar':':علم_سانت_فنسنت_وجزر_غرينادين:'},'\U0001F5FD':{'en':':Statue_of_Liberty:','status':fully_qualified,'E':0.6,'alias':[':statue_of_liberty:'],'de':':freiheitsstatue:','es':':estatua_de_la_libertad:','fr':':statue_de_la_liberté:','ja':':自由の女神:','ko':':자유의_여신상:','pt':':estátua_da_liberdade:','it':':statua_della_libertà:','fa':':مجسمهٔ_آزادی:','id':':patung_liberty:','zh':':自由女神像:','ru':':статуя_Свободы:','tr':':özgürlük_heykeli:','ar':':تمثال_الحرية:'},'\U0001F1F8\U0001F1E9':{'en':':Sudan:','status':fully_qualified,'E':2,'alias':[':flag_for_Sudan:', ':sudan:'],'de':':flagge_sudan:','es':':bandera_sudán:','fr':':drapeau_soudan:','ja':':旗_スーダン:','ko':':깃발_수단:','pt':':bandeira_sudão:','it':':bandiera_sudan:','fa':':پرچم_سودان:','id':':bendera_sudan:','zh':':苏丹:','ru':':флаг_Судан:','tr':':bayrak_sudan:','ar':':علم_السودان:'},'\U0001F1F8\U0001F1F7':{'en':':Suriname:','status':fully_qualified,'E':2,'alias':[':flag_for_Suriname:', ':suriname:'],'de':':flagge_suriname:','es':':bandera_surinam:','fr':':drapeau_suriname:','ja':':旗_スリナム:','ko':':깃발_수리남:','pt':':bandeira_suriname:','it':':bandiera_suriname:','fa':':پرچم_سورینام:','id':':bendera_suriname:','zh':':苏里南:','ru':':флаг_Суринам:','tr':':bayrak_surinam:','ar':':علم_سورينام:'},'\U0001F1F8\U0001F1EF':{'en':':Svalbard_&_Jan_Mayen:','status':fully_qualified,'E':2,'alias':[':flag_for_Svalbard_&_Jan_Mayen:', ':svalbard_jan_mayen:'],'de':':flagge_spitzbergen_und_jan_mayen:','es':':bandera_svalbard_y_jan_mayen:','fr':':drapeau_svalbard_et_jan_mayen:','ja':':旗_スバールバル諸島・ヤンマイエン島:','ko':':깃발_스발바르제도-얀마웬섬:','pt':':bandeira_svalbard_e_jan_mayen:','it':':bandiera_svalbard_e_jan_mayen:','fa':':پرچم_سوالبارد_و_یان_ماین:','id':':bendera_kepulauan_svalbard_dan_jan_mayen:','zh':':斯瓦尔巴和扬马延:','ru':':флаг_Шпицберген_и_Ян_Майен:','tr':':bayrak_svalbard_ve_jan_mayen:','ar':':علم_سفالبارد_وجان_ماين:'},'\U0001F1F8\U0001F1EA':{'en':':Sweden:','status':fully_qualified,'E':2,'alias':[':flag_for_Sweden:', ':sweden:'],'de':':flagge_schweden:','es':':bandera_suecia:','fr':':drapeau_suède:','ja':':旗_スウェーデン:','ko':':깃발_스웨덴:','pt':':bandeira_suécia:','it':':bandiera_svezia:','fa':':پرچم_سوئد:','id':':bendera_swedia:','zh':':瑞典:','ru':':флаг_Швеция:','tr':':bayrak_isveç:','ar':':علم_السويد:'},'\U0001F1E8\U0001F1ED':{'en':':Switzerland:','status':fully_qualified,'E':2,'alias':[':flag_for_Switzerland:', ':switzerland:'],'de':':flagge_schweiz:','es':':bandera_suiza:','fr':':drapeau_suisse:','ja':':旗_スイス:','ko':':깃발_스위스:','pt':':bandeira_suíça:','it':':bandiera_svizzera:','fa':':پرچم_سوئیس:','id':':bendera_swiss:','zh':':瑞士:','ru':':флаг_Швейцария:','tr':':bayrak_isviçre:','ar':':علم_سويسرا:'},'\U0001F1F8\U0001F1FE':{'en':':Syria:','status':fully_qualified,'E':2,'alias':[':flag_for_Syria:', ':syria:'],'de':':flagge_syrien:','es':':bandera_siria:','fr':':drapeau_syrie:','ja':':旗_シリア:','ko':':깃발_시리아:','pt':':bandeira_síria:','it':':bandiera_siria:','fa':':پرچم_سوریه:','id':':bendera_suriah:','zh':':叙利亚:','ru':':флаг_Сирия:','tr':':bayrak_suriye:','ar':':علم_سوريا:'},'\U0001F1F8\U0001F1F9':{'en':':São_Tomé_&_Príncipe:','status':fully_qualified,'E':2,'alias':[':flag_for_São_Tomé_&_Príncipe:', ':sao_tome_principe:'],'de':':flagge_são_tomé_und_príncipe:','es':':bandera_santo_tomé_y_príncipe:','fr':':drapeau_sao_tomé-et-principe:','ja':':旗_サントメ・プリンシペ:','ko':':깃발_상투메_프린시페:','pt':':bandeira_são_tomé_e_príncipe:','it':':bandiera_são_tomé_e_príncipe:','fa':':پرچم_سائوتومه_و_پرینسیپ:','id':':bendera_sao_tome_dan_principe:','zh':':圣多美和普林西比:','ru':':флаг_Сан_Томе_и_Принсипи:','tr':':bayrak_sao_tome_ve_principe:','ar':':علم_ساو_تومي_وبرينسيبي:'},'\U0001F996':{'en':':T-Rex:','status':fully_qualified,'E':5,'alias':[':t-rex:', ':t_rex:'],'de':':t-rex:','es':':t-rex:','fr':':t-rex:','ja':':ティラノサウルス:','ko':':티라노사우루스:','pt':':tiranossauro_rex:','it':':t-rex:','fa':':تی_رکس:','id':':t-rex:','zh':':霸王龙:','ru':':Ти_Рекс:','tr':':t-rex:','ar':':التيرانوصور:'},'\U0001F51D':{'en':':TOP_arrow:','status':fully_qualified,'E':0.6,'alias':[':top:', ':top_arrow:'],'de':':top-pfeil:','es':':flecha_top:','fr':':flèche_en_haut:','ja':':top矢印:','ko':':위로_화살표:','pt':':seta_top:','it':':freccia_top:','fa':':پیکان_رو:','id':':tanda_panah_top:','zh':':置顶:','ru':':стрелка_«верх»:','tr':':yukarı_oku:','ar':':السهم_لأعلى:'},'\U0001F1F9\U0001F1FC':{'en':':Taiwan:','status':fully_qualified,'E':2,'alias':[':flag_for_Taiwan:', ':taiwan:'],'de':':flagge_taiwan:','es':':bandera_taiwán:','fr':':drapeau_taïwan:','ja':':旗_台湾:','ko':':깃발_대만:','pt':':bandeira_taiwan:','it':':bandiera_taiwan:','fa':':پرچم_تایوان:','id':':bendera_taiwan:','zh':':台湾:','ru':':флаг_Тайвань:','tr':':bayrak_tayvan:','ar':':علم_تايوان:'},'\U0001F1F9\U0001F1EF':{'en':':Tajikistan:','status':fully_qualified,'E':2,'alias':[':flag_for_Tajikistan:', ':tajikistan:'],'de':':flagge_tadschikistan:','es':':bandera_tayikistán:','fr':':drapeau_tadjikistan:','ja':':旗_タジキスタン:','ko':':깃발_타지키스탄:','pt':':bandeira_tadjiquistão:','it':':bandiera_tagikistan:','fa':':پرچم_تاجیکستان:','id':':bendera_tajikistan:','zh':':塔吉克斯坦:','ru':':флаг_Таджикистан:','tr':':bayrak_tacikistan:','ar':':علم_طاجيكستان:'},'\U0001F1F9\U0001F1FF':{'en':':Tanzania:','status':fully_qualified,'E':2,'alias':[':flag_for_Tanzania:', ':tanzania:'],'de':':flagge_tansania:','es':':bandera_tanzania:','fr':':drapeau_tanzanie:','ja':':旗_タンザニア:','ko':':깃발_탄자니아:','pt':':bandeira_tanzânia:','it':':bandiera_tanzania:','fa':':پرچم_تانزانیا:','id':':bendera_tanzania:','zh':':坦桑尼亚:','ru':':флаг_Танзания:','tr':':bayrak_tanzanya:','ar':':علم_تنزانيا:'},'\U00002649':{'en':':Taurus:','status':fully_qualified,'E':0.6,'alias':[':taurus:'],'variant':True,'de':':stier_(sternzeichen):','es':':tauro:','fr':':taureau:','ja':':おうし座:','ko':':황소자리:','pt':':signo_de_touro:','it':':segno_zodiacale_del_toro:','fa':':برج_ثور:','id':':taurus:','zh':':金牛座:','ru':':знак_зодиака_Телец:','tr':':boğa_burcu:','ar':':الثور:'},'\U0001F1F9\U0001F1ED':{'en':':Thailand:','status':fully_qualified,'E':2,'alias':[':flag_for_Thailand:', ':thailand:'],'de':':flagge_thailand:','es':':bandera_tailandia:','fr':':drapeau_thaïlande:','ja':':旗_タイ:','ko':':깃발_태국:','pt':':bandeira_tailândia:','it':':bandiera_thailandia:','fa':':پرچم_تایلند:','id':':bendera_thailand:','zh':':泰国:','ru':':флаг_Таиланд:','tr':':bayrak_tayland:','ar':':علم_تايلاند:'},'\U0001F1F9\U0001F1F1':{'en':':Timor-Leste:','status':fully_qualified,'E':2,'alias':[':flag_for_Timor_Leste:', ':timor_leste:'],'de':':flagge_timor-leste:','es':':bandera_timor-leste:','fr':':drapeau_timor_oriental:','ja':':旗_東ティモール:','ko':':깃발_동티모르:','pt':':bandeira_timor-leste:','it':':bandiera_timor_est:','fa':':پرچم_تیمور-لسته:','id':':bendera_timor_leste:','zh':':东帝汶:','ru':':флаг_Восточный_Тимор:','tr':':bayrak_timor-leste:','ar':':علم_تيمور-ليشتي:'},'\U0001F1F9\U0001F1EC':{'en':':Togo:','status':fully_qualified,'E':2,'alias':[':flag_for_Togo:', ':togo:'],'de':':flagge_togo:','es':':bandera_togo:','fr':':drapeau_togo:','ja':':旗_トーゴ:','ko':':깃발_토고:','pt':':bandeira_togo:','it':':bandiera_togo:','fa':':پرچم_توگو:','id':':bendera_togo:','zh':':多哥:','ru':':флаг_Того:','tr':':bayrak_togo:','ar':':علم_توغو:'},'\U0001F1F9\U0001F1F0':{'en':':Tokelau:','status':fully_qualified,'E':2,'alias':[':flag_for_Tokelau:', ':tokelau:'],'de':':flagge_tokelau:','es':':bandera_tokelau:','fr':':drapeau_tokelau:','ja':':旗_トケラウ:','ko':':깃발_토켈라우:','pt':':bandeira_tokelau:','it':':bandiera_tokelau:','fa':':پرچم_توکلائو:','id':':bendera_tokelau:','zh':':托克劳:','ru':':флаг_Токелау:','tr':':bayrak_tokelau:','ar':':علم_توكيلاو:'},'\U0001F5FC':{'en':':Tokyo_tower:','status':fully_qualified,'E':0.6,'alias':[':tokyo_tower:'],'de':':tokyo_tower:','es':':torre_de_tokio:','fr':':tour_de_tokyo:','ja':':東京タワー:','ko':':도쿄_타워:','pt':':torre_de_tóquio:','it':':torre_di_tokyo:','fa':':برج_توکیو:','id':':menara_tokyo:','zh':':东京塔:','ru':':Токийская_башня:','tr':':tokyo_kulesi:','ar':':برج_طوكيو:'},'\U0001F1F9\U0001F1F4':{'en':':Tonga:','status':fully_qualified,'E':2,'alias':[':flag_for_Tonga:', ':tonga:'],'de':':flagge_tonga:','es':':bandera_tonga:','fr':':drapeau_tonga:','ja':':旗_トンガ:','ko':':깃발_통가:','pt':':bandeira_tonga:','it':':bandiera_tonga:','fa':':پرچم_تونگا:','id':':bendera_tonga:','zh':':汤加:','ru':':флаг_Тонга:','tr':':bayrak_tonga:','ar':':علم_تونغا:'},'\U0001F1F9\U0001F1F9':{'en':':Trinidad_&_Tobago:','status':fully_qualified,'E':2,'alias':[':flag_for_Trinidad_&_Tobago:', ':trinidad_tobago:'],'de':':flagge_trinidad_und_tobago:','es':':bandera_trinidad_y_tobago:','fr':':drapeau_trinité-et-tobago:','ja':':旗_トリニダード・トバゴ:','ko':':깃발_트리니다드_토바고:','pt':':bandeira_trinidad_e_tobago:','it':':bandiera_trinidad_e_tobago:','fa':':پرچم_ترینیداد_و_توباگو:','id':':bendera_trinidad_dan_tobago:','zh':':特立尼达和多巴哥:','ru':':флаг_Тринидад_и_Тобаго:','tr':':bayrak_trinidad_ve_tobago:','ar':':علم_ترينيداد_وتوباغو:'},'\U0001F1F9\U0001F1E6':{'en':':Tristan_da_Cunha:','status':fully_qualified,'E':2,'alias':[':flag_for_Tristan_da_Cunha:', ':tristan_da_cunha:'],'de':':flagge_tristan_da_cunha:','es':':bandera_tristán_de_acuña:','fr':':drapeau_tristan_da_cunha:','ja':':旗_トリスタン・ダ・クーニャ:','ko':':깃발_트리스탄다쿠나:','pt':':bandeira_tristão_da_cunha:','it':':bandiera_tristan_da_cunha:','fa':':پرچم_تریستان_دا_کونا:','id':':bendera_tristan_da_cunha:','zh':':特里斯坦达库尼亚群岛:','ru':':флаг_Тристан_да_Кунья:','tr':':bayrak_tristan_da_cunha:','ar':':علم_تريستان_دا_كونا:'},'\U0001F1F9\U0001F1F3':{'en':':Tunisia:','status':fully_qualified,'E':2,'alias':[':flag_for_Tunisia:', ':tunisia:'],'de':':flagge_tunesien:','es':':bandera_túnez:','fr':':drapeau_tunisie:','ja':':旗_チュニジア:','ko':':깃발_튀니지:','pt':':bandeira_tunísia:','it':':bandiera_tunisia:','fa':':پرچم_تونس:','id':':bendera_tunisia:','zh':':突尼斯:','ru':':флаг_Тунис:','tr':':bayrak_tunus:','ar':':علم_تونس:'},'\U0001F1F9\U0001F1F2':{'en':':Turkmenistan:','status':fully_qualified,'E':2,'alias':[':flag_for_Turkmenistan:', ':turkmenistan:'],'de':':flagge_turkmenistan:','es':':bandera_turkmenistán:','fr':':drapeau_turkménistan:','ja':':旗_トルクメニスタン:','ko':':깃발_투르크메니스탄:','pt':':bandeira_turcomenistão:','it':':bandiera_turkmenistan:','fa':':پرچم_ترکمنستان:','id':':bendera_turkmenistan:','zh':':土库曼斯坦:','ru':':флаг_Туркменистан:','tr':':bayrak_türkmenistan:','ar':':علم_تركمانستان:'},'\U0001F1F9\U0001F1E8':{'en':':Turks_&_Caicos_Islands:','status':fully_qualified,'E':2,'alias':[':flag_for_Turks_&_Caicos_Islands:', ':turks_caicos_islands:'],'de':':flagge_turks-_und_caicosinseln:','es':':bandera_islas_turcas_y_caicos:','fr':':drapeau_îles_turques-et-caïques:','ja':':旗_タークス・カイコス諸島:','ko':':깃발_터크스_케이커스_제도:','pt':':bandeira_ilhas_turcas_e_caicos:','it':':bandiera_isole_turks_e_caicos:','fa':':پرچم_جزایر_تورکس_و_کایکوس:','id':':bendera_kepulauan_turks_dan_caicos:','zh':':特克斯和凯科斯群岛:','ru':':флаг_Тёркс_и_Кайкос:','tr':':bayrak_turks_ve_caicos_adaları:','ar':':علم_جزر_توركس_وكايكوس:'},'\U0001F1F9\U0001F1FB':{'en':':Tuvalu:','status':fully_qualified,'E':2,'alias':[':flag_for_Tuvalu:', ':tuvalu:'],'de':':flagge_tuvalu:','es':':bandera_tuvalu:','fr':':drapeau_tuvalu:','ja':':旗_ツバル:','ko':':깃발_투발루:','pt':':bandeira_tuvalu:','it':':bandiera_tuvalu:','fa':':پرچم_تووالو:','id':':bendera_tuvalu:','zh':':图瓦卢:','ru':':флаг_Тувалу:','tr':':bayrak_tuvalu:','ar':':علم_توفالو:'},'\U0001F1F9\U0001F1F7':{'en':':Türkiye:','status':fully_qualified,'E':2,'alias':[':flag_for_Turkey:', ':Turkey:', ':tr:'],'de':':flagge_türkei:','es':':bandera_turquía:','fr':':drapeau_turquie:','ja':':旗_トルコ:','ko':':깃발_튀르키예:','pt':':bandeira_turquia:','it':':bandiera_turchia:','fa':':پرچم_ترکیه:','id':':bendera_turki:','zh':':土耳其:','ru':':флаг_Турция:','tr':':bayrak_türkiye:','ar':':علم_تركيا:'},'\U0001F1FA\U0001F1F2':{'en':':U.S._Outlying_Islands:','status':fully_qualified,'E':2,'alias':[':flag_for_U.S._Outlying_Islands:', ':us_outlying_islands:'],'de':':flagge_amerikanische_überseeinseln:','es':':bandera_islas_menores_alejadas_de_ee._uu.:','fr':':drapeau_îles_mineures_éloignées_des_états-unis:','ja':':旗_合衆国領有小離島:','ko':':깃발_미국령_해외_제도:','pt':':bandeira_ilhas_menores_distantes_dos_eua:','it':':bandiera_isole_minori_esterne_degli_stati_uniti:','fa':':پرچم_جزایر_دورافتادهٔ_ایالات_متحده:','id':':bendera_kepulauan_terluar_as:','zh':':美国本土外小岛屿:','ru':':флаг_Внешние_малые_о_ва_(США):','tr':':bayrak_abd_küçük_harici_adaları:','ar':':علم_جزر_الولايات_المتحدة_النائية:'},'\U0001F1FB\U0001F1EE':{'en':':U.S._Virgin_Islands:','status':fully_qualified,'E':2,'alias':[':flag_for_U.S._Virgin_Islands:', ':us_virgin_islands:'],'de':':flagge_amerikanische_jungferninseln:','es':':bandera_islas_vírgenes_de_ee._uu.:','fr':':drapeau_îles_vierges_des_états-unis:','ja':':旗_米領ヴァージン諸島:','ko':':깃발_미국령_버진아일랜드:','pt':':bandeira_ilhas_virgens_americanas:','it':':bandiera_isole_vergini_americane:','fa':':پرچم_جزایر_ویرجین_ایالات_متحده:','id':':bendera_kepulauan_virgin_amerika_serikat:','zh':':美属维尔京群岛:','ru':':флаг_Виргинские_о_ва_(США):','tr':':bayrak_abd_virjin_adaları:','ar':':علم_جزر_فيرجن_الأمريكية:'},'\U0001F199':{'en':':UP!_button:','status':fully_qualified,'E':0.6,'alias':[':up:', ':up_button:'],'de':':schriftzug_up!_im_blauen_quadrat:','es':':botón_up!:','fr':':bouton_vers_le_haut:','ja':':upマーク:','ko':':위로_버튼:','pt':':botão_up!:','it':':pulsante_up!:','fa':':دکمهٔ_!up:','id':':tombol_up!:','zh':':up按钮:','ru':':значок_«обновление»:','tr':':up!_düğmesi:','ar':':أعلى:'},'\U0001F1FA\U0001F1EC':{'en':':Uganda:','status':fully_qualified,'E':2,'alias':[':flag_for_Uganda:', ':uganda:'],'de':':flagge_uganda:','es':':bandera_uganda:','fr':':drapeau_ouganda:','ja':':旗_ウガンダ:','ko':':깃발_우간다:','pt':':bandeira_uganda:','it':':bandiera_uganda:','fa':':پرچم_اوگاندا:','id':':bendera_uganda:','zh':':乌干达:','ru':':флаг_Уганда:','tr':':bayrak_uganda:','ar':':علم_أوغندا:'},'\U0001F1FA\U0001F1E6':{'en':':Ukraine:','status':fully_qualified,'E':2,'alias':[':flag_for_Ukraine:', ':ukraine:'],'de':':flagge_ukraine:','es':':bandera_ucrania:','fr':':drapeau_ukraine:','ja':':旗_ウクライナ:','ko':':깃발_우크라이나:','pt':':bandeira_ucrânia:','it':':bandiera_ucraina:','fa':':پرچم_اوکراین:','id':':bendera_ukraina:','zh':':乌克兰:','ru':':флаг_Украина:','tr':':bayrak_ukrayna:','ar':':علم_أوكرانيا:'},'\U0001F1E6\U0001F1EA':{'en':':United_Arab_Emirates:','status':fully_qualified,'E':2,'alias':[':flag_for_United_Arab_Emirates:', ':united_arab_emirates:'],'de':':flagge_vereinigte_arabische_emirate:','es':':bandera_emiratos_árabes_unidos:','fr':':drapeau_émirats_arabes_unis:','ja':':旗_アラブ首長国連邦:','ko':':깃발_아랍에미리트:','pt':':bandeira_emirados_árabes_unidos:','it':':bandiera_emirati_arabi_uniti:','fa':':پرچم_امارات_متحدهٔ_عربی:','id':':bendera_uni_emirat_arab:','zh':':阿拉伯联合酋长国:','ru':':флаг_ОАЭ:','tr':':bayrak_birleşik_arap_emirlikleri:','ar':':علم_الإمارات_العربية_المتحدة:'},'\U0001F1EC\U0001F1E7':{'en':':United_Kingdom:','status':fully_qualified,'E':0.6,'alias':[':flag_for_United_Kingdom:', ':gb:', ':uk:'],'de':':flagge_vereinigtes_königreich:','es':':bandera_reino_unido:','fr':':drapeau_royaume-uni:','ja':':旗_イギリス:','ko':':깃발_영국:','pt':':bandeira_reino_unido:','it':':bandiera_regno_unito:','fa':':پرچم_بریتانیا:','id':':bendera_inggris_raya:','zh':':英国:','ru':':флаг_Великобритания:','tr':':bayrak_birleşik_krallık:','ar':':علم_المملكة_المتحدة:'},'\U0001F1FA\U0001F1F3':{'en':':United_Nations:','status':fully_qualified,'E':4,'alias':[':united_nations:'],'de':':flagge_vereinte_nationen:','es':':bandera_naciones_unidas:','fr':':drapeau_nations_unies:','ja':':旗_国際連合:','ko':':깃발_국제연합:','pt':':bandeira_nações_unidas:','it':':bandiera_nazioni_unite:','fa':':پرچم_سازمان_ملل_متحد:','id':':bendera_perserikatan_bangsa-bangsa:','zh':':联合国:','ru':':флаг_Организация_Объединенных_Наций:','tr':':bayrak_birleşmiş_milletler:','ar':':علم_الأمم_المتحدة:'},'\U0001F1FA\U0001F1F8':{'en':':United_States:','status':fully_qualified,'E':0.6,'alias':[':flag_for_United_States:', ':us:'],'de':':flagge_vereinigte_staaten:','es':':bandera_estados_unidos:','fr':':drapeau_états-unis:','ja':':旗_アメリカ合衆国:','ko':':깃발_미국:','pt':':bandeira_estados_unidos:','it':':bandiera_stati_uniti:','fa':':پرچم_ایالات_متحده:','id':':bendera_amerika_serikat:','zh':':美国:','ru':':флаг_Соединенные_Штаты:','tr':':bayrak_amerika_birleşik_devletleri:','ar':':علم_الولايات_المتحدة:'},'\U0001F1FA\U0001F1FE':{'en':':Uruguay:','status':fully_qualified,'E':2,'alias':[':flag_for_Uruguay:', ':uruguay:'],'de':':flagge_uruguay:','es':':bandera_uruguay:','fr':':drapeau_uruguay:','ja':':旗_ウルグアイ:','ko':':깃발_우루과이:','pt':':bandeira_uruguai:','it':':bandiera_uruguay:','fa':':پرچم_اروگوئه:','id':':bendera_uruguay:','zh':':乌拉圭:','ru':':флаг_Уругвай:','tr':':bayrak_uruguay:','ar':':علم_أورغواي:'},'\U0001F1FA\U0001F1FF':{'en':':Uzbekistan:','status':fully_qualified,'E':2,'alias':[':flag_for_Uzbekistan:', ':uzbekistan:'],'de':':flagge_usbekistan:','es':':bandera_uzbekistán:','fr':':drapeau_ouzbékistan:','ja':':旗_ウズベキスタン:','ko':':깃발_우즈베키스탄:','pt':':bandeira_uzbequistão:','it':':bandiera_uzbekistan:','fa':':پرچم_ازبکستان:','id':':bendera_uzbekistan:','zh':':乌兹别克斯坦:','ru':':флаг_Узбекистан:','tr':':bayrak_özbekistan:','ar':':علم_أوزبكستان:'},'\U0001F19A':{'en':':VS_button:','status':fully_qualified,'E':0.6,'alias':[':vs:', ':vs_button:'],'de':':schriftzug_vs_in_orangem_quadrat:','es':':botón_vs:','fr':':bouton_vs:','ja':':vsマーク:','ko':':대:','pt':':botão_vs:','it':':pulsante_vs:','fa':':دکمهٔ_vs:','id':':tombol_vs:','zh':':VS按钮:','ru':':значок_«против»:','tr':':vs_düğmesi:','ar':':ضد:'},'\U0001F1FB\U0001F1FA':{'en':':Vanuatu:','status':fully_qualified,'E':2,'alias':[':flag_for_Vanuatu:', ':vanuatu:'],'de':':flagge_vanuatu:','es':':bandera_vanuatu:','fr':':drapeau_vanuatu:','ja':':旗_バヌアツ:','ko':':깃발_바누아투:','pt':':bandeira_vanuatu:','it':':bandiera_vanuatu:','fa':':پرچم_وانواتو:','id':':bendera_vanuatu:','zh':':瓦努阿图:','ru':':флаг_Вануату:','tr':':bayrak_vanuatu:','ar':':علم_فانواتو:'},'\U0001F1FB\U0001F1E6':{'en':':Vatican_City:','status':fully_qualified,'E':2,'alias':[':flag_for_Vatican_City:', ':vatican_city:'],'de':':flagge_vatikanstadt:','es':':bandera_ciudad_del_vaticano:','fr':':drapeau_état_de_la_cité_du_vatican:','ja':':旗_バチカン市国:','ko':':깃발_바티칸_시국:','pt':':bandeira_cidade_do_vaticano:','it':':bandiera_città_del_vaticano:','fa':':پرچم_واتیکان:','id':':bendera_vatikan:','zh':':梵蒂冈:','ru':':флаг_Ватикан:','tr':':bayrak_vatikan:','ar':':علم_الفاتيكان:'},'\U0001F1FB\U0001F1EA':{'en':':Venezuela:','status':fully_qualified,'E':2,'alias':[':flag_for_Venezuela:', ':venezuela:'],'de':':flagge_venezuela:','es':':bandera_venezuela:','fr':':drapeau_venezuela:','ja':':旗_ベネズエラ:','ko':':깃발_베네수엘라:','pt':':bandeira_venezuela:','it':':bandiera_venezuela:','fa':':پرچم_ونزوئلا:','id':':bendera_venezuela:','zh':':委内瑞拉:','ru':':флаг_Венесуэла:','tr':':bayrak_venezuela:','ar':':علم_فنزويلا:'},'\U0001F1FB\U0001F1F3':{'en':':Vietnam:','status':fully_qualified,'E':2,'alias':[':flag_for_Vietnam:', ':vietnam:'],'de':':flagge_vietnam:','es':':bandera_vietnam:','fr':':drapeau_viêt_nam:','ja':':旗_ベトナム:','ko':':깃발_베트남:','pt':':bandeira_vietnã:','it':':bandiera_vietnam:','fa':':پرچم_ویتنام:','id':':bendera_vietnam:','zh':':越南:','ru':':флаг_Вьетнам:','tr':':bayrak_vietnam:','ar':':علم_فيتنام:'},'\U0000264D':{'en':':Virgo:','status':fully_qualified,'E':0.6,'alias':[':virgo:'],'variant':True,'de':':jungfrau_(sternzeichen):','es':':virgo:','fr':':vierge:','ja':':おとめ座:','ko':':처녀자리:','pt':':signo_de_virgem:','it':':segno_zodiacale_della_vergine:','fa':':برج_سنبله:','id':':virgo:','zh':':处女座:','ru':':знак_зодиака_Дева:','tr':':başak_burcu:','ar':':العذراء:'},'\U0001F3F4\U000E0067\U000E0062\U000E0077\U000E006C\U000E0073\U000E007F':{'en':':Wales:','status':fully_qualified,'E':5,'alias':[':wales:'],'de':':flagge_wales:','es':':bandera_gales:','fr':':drapeau_pays_de_galles:','ja':':旗_ウェールズ:','ko':':깃발_웨일즈:','pt':':bandeira_país_de_gales:','it':':bandiera_galles:','fa':':پرچم_ویلز:','id':':bendera_wales:','zh':':威尔士:','ru':':флаг_Уэльс:','tr':':bayrak_galler:','ar':':علم_ويلز:'},'\U0001F1FC\U0001F1EB':{'en':':Wallis_&_Futuna:','status':fully_qualified,'E':2,'alias':[':flag_for_Wallis_&_Futuna:', ':wallis_futuna:'],'de':':flagge_wallis_und_futuna:','es':':bandera_wallis_y_futuna:','fr':':drapeau_wallis-et-futuna:','ja':':旗_ウォリス・フツナ:','ko':':깃발_왈리스-푸투나_제도:','pt':':bandeira_wallis_e_futuna:','it':':bandiera_wallis_e_futuna:','fa':':پرچم_والیس_و_فوتونا:','id':':bendera_kepulauan_wallis_dan_futuna:','zh':':瓦利斯和富图纳:','ru':':флаг_Уоллис_и_Футуна:','tr':':bayrak_wallis_ve_futuna:','ar':':علم_جزر_والس_وفوتونا:'},'\U0001F1EA\U0001F1ED':{'en':':Western_Sahara:','status':fully_qualified,'E':2,'alias':[':flag_for_Western_Sahara:', ':western_sahara:'],'de':':flagge_westsahara:','es':':bandera_sáhara_occidental:','fr':':drapeau_sahara_occidental:','ja':':旗_西サハラ:','ko':':깃발_서사하라:','pt':':bandeira_saara_ocidental:','it':':bandiera_sahara_occidentale:','fa':':پرچم_صحرای_غربی:','id':':bendera_sahara_barat:','zh':':西撒哈拉:','ru':':флаг_Западная_Сахара:','tr':':bayrak_batı_sahra:','ar':':علم_الصحراء_الغربية:'},'\U0001F1FE\U0001F1EA':{'en':':Yemen:','status':fully_qualified,'E':2,'alias':[':flag_for_Yemen:', ':yemen:'],'de':':flagge_jemen:','es':':bandera_yemen:','fr':':drapeau_yémen:','ja':':旗_イエメン:','ko':':깃발_예멘:','pt':':bandeira_iêmen:','it':':bandiera_yemen:','fa':':پرچم_یمن:','id':':bendera_yaman:','zh':':也门:','ru':':флаг_Йемен:','tr':':bayrak_yemen:','ar':':علم_اليمن:'},'\U0001F4A4':{'en':':ZZZ:','status':fully_qualified,'E':0.6,'alias':[':zzz:'],'de':':schlafen:','es':':símbolo_de_sueño:','fr':':endormi:','ja':':グーグー:','ko':':졸림:','pt':':zzz:','it':':sonno:','fa':':خروپف:','id':':zzz:','zh':':睡着:','ru':':сон:','tr':':zzz:','ar':':نعسان:'},'\U0001F1FF\U0001F1F2':{'en':':Zambia:','status':fully_qualified,'E':2,'alias':[':flag_for_Zambia:', ':zambia:'],'de':':flagge_sambia:','es':':bandera_zambia:','fr':':drapeau_zambie:','ja':':旗_ザンビア:','ko':':깃발_잠비아:','pt':':bandeira_zâmbia:','it':':bandiera_zambia:','fa':':پرچم_زامبیا:','id':':bendera_zambia:','zh':':赞比亚:','ru':':флаг_Замбия:','tr':':bayrak_zambiya:','ar':':علم_زامبيا:'},'\U0001F1FF\U0001F1FC':{'en':':Zimbabwe:','status':fully_qualified,'E':2,'alias':[':flag_for_Zimbabwe:', ':zimbabwe:'],'de':':flagge_simbabwe:','es':':bandera_zimbabue:','fr':':drapeau_zimbabwe:','ja':':旗_ジンバブエ:','ko':':깃발_짐바브웨:','pt':':bandeira_zimbábue:','it':':bandiera_zimbabwe:','fa':':پرچم_زیمبابوه:','id':':bendera_zimbabwe:','zh':':津巴布韦:','ru':':флаг_Зимбабве:','tr':':bayrak_zimbabve:','ar':':علم_زيمبابوي:'},'\U0001F9EE':{'en':':abacus:','status':fully_qualified,'E':11,'de':':abakus:','es':':ábaco:','fr':':abaque:','ja':':そろばん:','ko':':주판:','pt':':ábaco:','it':':abaco:','fa':':چرتکه:','id':':sempoa:','zh':':算盘:','ru':':счеты:','tr':':abaküs:','ar':':معداد:'},'\U0001FA97':{'en':':accordion:','status':fully_qualified,'E':13,'de':':akkordeon:','es':':acordeón:','fr':':accordéon:','ja':':アコーディオン:','ko':':아코디언:','pt':':acordeão:','it':':fisarmonica:','fa':':آکوردئون:','id':':akordeon:','zh':':手风琴:','ru':':аккордеон:','tr':':akordiyon:','ar':':أكورديون:'},'\U0001FA79':{'en':':adhesive_bandage:','status':fully_qualified,'E':12,'de':':heftpflaster:','es':':tirita:','fr':':sparadrap:','ja':':絆創膏:','ko':':반창고:','pt':':atadura_adesiva:','it':':cerotto:','fa':':چسب_زخم:','id':':plester_luka:','zh':':创可贴:','ru':':пластырь:','tr':':yara_bandı:','ar':':ضمادة_لاصقة:'},'\U0001F39F\U0000FE0F':{'en':':admission_tickets:','status':fully_qualified,'E':0.7,'alias':[':tickets:'],'variant':True,'de':':eintrittskarten:','es':':entradas:','fr':':billet_d’entrée:','ja':':入場券:','ko':':입장_티켓:','pt':':ingresso_de_cinema:','it':':biglietto_d’ingresso:','fa':':بلیت_ورود:','id':':tiket_masuk:','zh':':入场券:','ru':':билеты:','tr':':giriş_bileti:','ar':':تذكرتا_دخول:'},'\U0001F39F':{'en':':admission_tickets:','status':unqualified,'E':0.7,'alias':[':tickets:'],'variant':True,'de':':eintrittskarten:','es':':entradas:','fr':':billet_d’entrée:','ja':':入場券:','ko':':입장_티켓:','pt':':ingresso_de_cinema:','it':':biglietto_d’ingresso:','fa':':بلیت_ورود:','id':':tiket_masuk:','zh':':入场券:','ru':':билеты:','tr':':giriş_bileti:','ar':':تذكرتا_دخول:'},'\U0001F6A1':{'en':':aerial_tramway:','status':fully_qualified,'E':1,'de':':bergseilbahn:','es':':teleférico:','fr':':tramway_aérien:','ja':':ロープウェイ:','ko':':케이블카:','pt':':teleférico:','it':':funivia_aerea:','fa':':تراموای_هوایی:','id':':trem_gantung:','zh':':索道:','ru':':канатная_дорога:','tr':':teleferik:','ar':':عربة_قطار_هوائي:'},'\U00002708\U0000FE0F':{'en':':airplane:','status':fully_qualified,'E':0.6,'variant':True,'de':':flugzeug:','es':':avión:','fr':':avion:','ja':':飛行機:','ko':':비행기:','pt':':avião:','it':':aeroplano:','fa':':هواپیما:','id':':pesawat:','zh':':飞机:','ru':':самолет:','tr':':uçak:','ar':':طائرة:'},'\U00002708':{'en':':airplane:','status':unqualified,'E':0.6,'variant':True,'de':':flugzeug:','es':':avión:','fr':':avion:','ja':':飛行機:','ko':':비행기:','pt':':avião:','it':':aeroplano:','fa':':هواپیما:','id':':pesawat:','zh':':飞机:','ru':':самолет:','tr':':uçak:','ar':':طائرة:'},'\U0001F6EC':{'en':':airplane_arrival:','status':fully_qualified,'E':1,'alias':[':airplane_arriving:', ':flight_arrival:'],'de':':landung_eines_flugzeugs:','es':':avión_aterrizando:','fr':':avion_à_l’atterrissage:','ja':':飛行機着陸:','ko':':비행기_착륙:','pt':':avião_aterrissando:','it':':atterraggio:','fa':':زمان_ورود_هواپیما:','id':':pesawat_mendarat:','zh':':航班降落:','ru':':посадка:','tr':':inen_uçak:','ar':':وصول_طائرة:'},'\U0001F6EB':{'en':':airplane_departure:','status':fully_qualified,'E':1,'alias':[':flight_departure:'],'de':':abflug:','es':':avión_despegando:','fr':':avion_au_décollage:','ja':':飛行機離陸:','ko':':비행기_이륙:','pt':':avião_decolando:','it':':decollo:','fa':':زمان_حرکت_هواپیما:','id':':pesawat_lepas_landas:','zh':':航班起飞:','ru':':взлет:','tr':':kalkan_uçak:','ar':':مغادرة_طائرة:'},'\U000023F0':{'en':':alarm_clock:','status':fully_qualified,'E':0.6,'variant':True,'de':':wecker:','es':':reloj_despertador:','fr':':réveil:','ja':':目覚まし時計:','ko':':알람_시계:','pt':':despertador:','it':':sveglia:','fa':':ساعت_رومیزی:','id':':jam_weker:','zh':':闹钟:','ru':':будильник:','tr':':çalar_saat:','ar':':ساعة_منبه:'},'\U00002697\U0000FE0F':{'en':':alembic:','status':fully_qualified,'E':1,'variant':True,'de':':destillierapparat:','es':':alambique:','fr':':alambic:','ja':':蒸留器:','ko':':증류기:','pt':':alambique:','it':':alambicco:','fa':':انبیق:','id':':alembic:','zh':':蒸馏器:','ru':':перегонныи_сосуд:','tr':':imbik:','ar':':أمبيق:'},'\U00002697':{'en':':alembic:','status':unqualified,'E':1,'variant':True,'de':':destillierapparat:','es':':alambique:','fr':':alambic:','ja':':蒸留器:','ko':':증류기:','pt':':alambique:','it':':alambicco:','fa':':انبیق:','id':':alembic:','zh':':蒸馏器:','ru':':перегонный_сосуд:','tr':':imbik:','ar':':أمبيق:'},'\U0001F47D':{'en':':alien:','status':fully_qualified,'E':0.6,'variant':True,'de':':außerirdischer:','es':':alienígena:','fr':':alien:','ja':':エイリアン:','ko':':외계인:','pt':':alienígena:','it':':alieno:','fa':':موجود_فضایی:','id':':alien:','zh':':外星人:','ru':':инопланетянин:','tr':':uzaylı:','ar':':كائن_فضائي:'},'\U0001F47E':{'en':':alien_monster:','status':fully_qualified,'E':0.6,'alias':[':space_invader:'],'de':':computerspiel-monster:','es':':monstruo_alienígena:','fr':':monstre_de_l’espace:','ja':':インベーダー:','ko':':에일리언:','pt':':monstro_alienígena:','it':':mostro_alieno:','fa':':هیولای_فضایی:','id':':monster_alien:','zh':':外星怪物:','ru':':космический_монстр:','tr':':uzaylı_canavar:','ar':':وحش_غريب:'},'\U0001F691':{'en':':ambulance:','status':fully_qualified,'E':0.6,'variant':True,'de':':krankenwagen:','es':':ambulancia:','fr':':ambulance:','ja':':救急車:','ko':':구급차:','pt':':ambulância:','it':':ambulanza:','fa':':آمبولانس:','id':':ambulans:','zh':':救护车:','ru':':машина_скорой_помощи:','tr':':ambulans:','ar':':سيارة_إسعاف:'},'\U0001F3C8':{'en':':american_football:','status':fully_qualified,'E':0.6,'alias':[':football:'],'de':':football:','es':':balón_de_fútbol_americano:','fr':':football_américain:','ja':':アメフト:','ko':':미식축구공:','pt':':bola_de_futebol_americano:','it':':football_americano:','fa':':فوتبال_امریکایی:','id':':american_football:','zh':':美式橄榄球:','ru':':американский_футбол:','tr':':amerikan_futbolu:','ar':':كرة_قدم_أمريكية:'},'\U0001F3FA':{'en':':amphora:','status':fully_qualified,'E':1,'de':':amphore:','es':':ánfora:','fr':':amphore:','ja':':壺:','ko':':암포라:','pt':':ânfora:','it':':anfora:','fa':':سبو:','id':':amphora:','zh':':双耳瓶:','ru':':амфора:','tr':':amfora:','ar':':جرة:'},'\U0001FAC0':{'en':':anatomical_heart:','status':fully_qualified,'E':13,'de':':herz_(organ):','es':':corazón_humano:','fr':':cœur:','ja':':心臓:','ko':':심장:','pt':':coração_humano:','it':':organo_del_cuore:','fa':':قلب:','id':':organ_jantung:','zh':':心脏器官:','ru':':сердце_человека:','tr':':kalp:','ar':':عضو_القلب:'},'\U00002693':{'en':':anchor:','status':fully_qualified,'E':0.6,'variant':True,'de':':anker:','es':':ancla:','fr':':ancre:','ja':':錨:','ko':':닻:','pt':':âncora:','it':':ancora:','fa':':لنگر:','id':':jangkar:','zh':':锚:','ru':':якорь:','tr':':çıpa:','ar':':مرساة:'},'\U0001F4A2':{'en':':anger_symbol:','status':fully_qualified,'E':0.6,'alias':[':anger:'],'de':':ärger:','es':':símbolo_de_enfado:','fr':':symbole_de_colère:','ja':':むかっ:','ko':':화남:','pt':':símbolo_de_raiva:','it':':rabbia:','fa':':نماد_عصبانیت:','id':':marah:','zh':':怒:','ru':':значок_«гнев»:','tr':':öfke_simgesi:','ar':':رمز_الغضب:'},'\U0001F620':{'en':':angry_face:','status':fully_qualified,'E':0.6,'alias':[':angry:'],'de':':verärgertes_gesicht:','es':':cara_enfadada:','fr':':visage_en_colère:','ja':':ぷんぷん:','ko':':화난_얼굴:','pt':':rosto_zangado:','it':':faccina_arrabbiata:','fa':':عصبانی:','id':':wajah_marah:','zh':':生气:','ru':':сердится:','tr':':öfkeli_yüz:','ar':':وجه_غاضب:'},'\U0001F47F':{'en':':angry_face_with_horns:','status':fully_qualified,'E':0.6,'alias':[':imp:'],'de':':wütendes_gesicht_mit_hörnern:','es':':cara_enfadada_con_cuernos:','fr':':visage_en_colère_avec_des_cornes:','ja':':怒った悪魔:','ko':':머리에_뿔이_달린_화난_얼굴:','pt':':rosto_zangado_com_chifres:','it':':faccina_arrabbiata_con_corna:','fa':':چهره_عصبانی_با_دو_شاخ:','id':':wajah_marah_dengan_tanduk:','zh':':生气的恶魔:','ru':':злой_чертенок:','tr':':boynuzlu_kızgın_yüz:','ar':':وجه_غاضب_بقرنين:'},'\U0001F627':{'en':':anguished_face:','status':fully_qualified,'E':1,'alias':[':anguished:'],'de':':qualvolles_gesicht:','es':':cara_angustiada:','fr':':visage_angoissé:','ja':':苦悩:','ko':':괴로워하는_얼굴:','pt':':rosto_angustiado:','it':':faccina_angosciata:','fa':':دردمند:','id':':wajah_menderita:','zh':':极度痛苦:','ru':':страдает:','tr':':kederli_yüz:','ar':':وجه_مكروب:'},'\U0001F41C':{'en':':ant:','status':fully_qualified,'E':0.6,'de':':ameise:','es':':hormiga:','fr':':fourmi:','ja':':アリ:','ko':':개미:','pt':':formiga:','it':':formica:','fa':':مورچه:','id':':semut:','zh':':蚂蚁:','ru':':муравей:','tr':':karınca:','ar':':نملة:'},'\U0001F4F6':{'en':':antenna_bars:','status':fully_qualified,'E':0.6,'alias':[':signal_strength:'],'de':':balkenförmige_signalstärkenanzeige:','es':':barras_de_cobertura:','fr':':barres_de_réseau:','ja':':アンテナマーク:','ko':':안테나_신호:','pt':':barras_de_sinal:','it':':segnale_cellulare:','fa':':نوار_سیگنال:','id':':batang_antena:','zh':':信号强度条:','ru':':сигнал_сети:','tr':':sinyal_gücü:','ar':':إشارات_هوائي:'},'\U0001F630':{'en':':anxious_face_with_sweat:','status':fully_qualified,'E':0.6,'alias':[':cold_sweat:'],'de':':besorgtes_gesicht_mit_schweißtropfen:','es':':cara_con_ansiedad_y_sudor:','fr':':visage_anxieux_avec_goutte_de_sueur:','ja':':冷や汗青ざめ:','ko':':땀_흘리며_불안해하는_얼굴:','pt':':rosto_ansioso_com_gota_de_suor:','it':':faccina_sudata_in_ansia:','fa':':عرق_ترس:','id':':wajah_ketakutan_berkeringat:','zh':':冷汗:','ru':':сильно_беспокоится:','tr':':terli_ve_endişeli_yüz:','ar':':وجه_بفم_مفتوح_وعرق_بارد:'},'\U0001F69B':{'en':':articulated_lorry:','status':fully_qualified,'E':1,'de':':sattelzug:','es':':camión_articulado:','fr':':semi-remorque:','ja':':トレーラー:','ko':':트레일러_트럭:','pt':':caminhão:','it':':autoarticolato:','fa':':کامیون_مفصلی:','id':':lori_gandeng:','zh':':铰接式货车:','ru':':фура:','tr':':tır:','ar':':لوري:'},'\U0001F9D1\U0000200D\U0001F3A8':{'en':':artist:','status':fully_qualified,'E':12.1,'de':':künstler(in):','es':':artista:','fr':':artiste:','ja':':芸術家:','ko':':화가:','pt':':artista:','it':':artista:','fa':':هنرمند:','id':':seniman:','zh':':艺术家:','ru':':художник:','tr':':ressam:','ar':':فنان_رسم:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F3A8':{'en':':artist_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':künstler(in)_dunkle_hautfarbe:','es':':artista_tono_de_piel_oscuro:','fr':':artiste_peau_foncée:','ja':':芸術家_濃い肌色:','ko':':화가_검은색_피부:','pt':':artista_pele_escura:','it':':artista_carnagione_scura:','fa':':هنرمند_پوست_آبنوسی:','id':':seniman_warna_kulit_gelap:','zh':':艺术家_较深肤色:','ru':':художник_очень_темный_тон_кожи:','tr':':ressam_koyu_cilt_tonu:','ar':':فنان_رسم_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F3A8':{'en':':artist_light_skin_tone:','status':fully_qualified,'E':12.1,'de':':künstler(in)_helle_hautfarbe:','es':':artista_tono_de_piel_claro:','fr':':artiste_peau_claire:','ja':':芸術家_薄い肌色:','ko':':화가_하얀_피부:','pt':':artista_pele_clara:','it':':artista_carnagione_chiara:','fa':':هنرمند_پوست_سفید:','id':':seniman_warna_kulit_cerah:','zh':':艺术家_较浅肤色:','ru':':художник_очень_светлый_тон_кожи:','tr':':ressam_açık_cilt_tonu:','ar':':فنان_رسم_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F3A8':{'en':':artist_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':künstler(in)_mitteldunkle_hautfarbe:','es':':artista_tono_de_piel_oscuro_medio:','fr':':artiste_peau_mate:','ja':':芸術家_やや濃い肌色:','ko':':화가_진한_갈색_피부:','pt':':artista_pele_morena_escura:','it':':artista_carnagione_abbastanza_scura:','fa':':هنرمند_پوست_گندمی:','id':':seniman_warna_kulit_gelap-sedang:','zh':':艺术家_中等深肤色:','ru':':художник_темный_тон_кожи:','tr':':ressam_orta_koyu_cilt_tonu:','ar':':فنان_رسم_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F3A8':{'en':':artist_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':künstler(in)_mittelhelle_hautfarbe:','es':':artista_tono_de_piel_claro_medio:','fr':':artiste_peau_moyennement_claire:','ja':':芸術家_やや薄い肌色:','ko':':화가_연한_갈색_피부:','pt':':artista_pele_morena_clara:','it':':artista_carnagione_abbastanza_chiara:','fa':':هنرمند_پوست_روشن:','id':':seniman_warna_kulit_cerah-sedang:','zh':':艺术家_中等浅肤色:','ru':':художник_светлый_тон_кожи:','tr':':ressam_orta_açık_cilt_tonu:','ar':':فنان_رسم_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F3A8':{'en':':artist_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':künstler(in)_mittlere_hautfarbe:','es':':artista_tono_de_piel_medio:','fr':':artiste_peau_légèrement_mate:','ja':':芸術家_中間の肌色:','ko':':화가_갈색_피부:','pt':':artista_pele_morena:','it':':artista_carnagione_olivastra:','fa':':هنرمند_پوست_طلایی:','id':':seniman_warna_kulit_sedang:','zh':':艺术家_中等肤色:','ru':':художник_средний_тон_кожи:','tr':':ressam_orta_cilt_tonu:','ar':':فنان_رسم_بشرة_بلون_معتدل:'},'\U0001F3A8':{'en':':artist_palette:','status':fully_qualified,'E':0.6,'alias':[':art:'],'de':':mischpalette:','es':':paleta_de_pintor:','fr':':palette_de_peinture:','ja':':絵の具パレット:','ko':':팔레트:','pt':':paleta_de_tintas:','it':':tavolozza:','fa':':پالت_نقاشی:','id':':palet_pelukis:','zh':':调色盘:','ru':':палитра_с_красками:','tr':':boya_paleti:','ar':':لوحة_ألوان_الرسام:'},'\U0001F632':{'en':':astonished_face:','status':fully_qualified,'E':0.6,'alias':[':astonished:'],'de':':erstauntes_gesicht:','es':':cara_asombrada:','fr':':visage_stupéfait:','ja':':びっくり:','ko':':깜짝_놀란_얼굴:','pt':':rosto_espantado:','it':':faccina_stupita:','fa':':بهت_زده:','id':':wajah_terpukau:','zh':':震惊:','ru':':в_шоке:','tr':':afallamış_yüz:','ar':':وجه_مندهش:'},'\U0001F9D1\U0000200D\U0001F680':{'en':':astronaut:','status':fully_qualified,'E':12.1,'de':':astronaut(in):','es':':astronauta:','fr':':astronaute:','ja':':宇宙飛行士:','ko':':우주비행사:','pt':':astronauta:','it':':astronauta:','fa':':فضانورد:','id':':astronaut:','zh':':宇航员:','ru':':космонавт:','tr':':astronot:','ar':':رائد_الفضاء:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F680':{'en':':astronaut_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':astronaut(in)_dunkle_hautfarbe:','es':':astronauta_tono_de_piel_oscuro:','fr':':astronaute_peau_foncée:','ja':':宇宙飛行士_濃い肌色:','ko':':우주비행사_검은색_피부:','pt':':astronauta_pele_escura:','it':':astronauta_carnagione_scura:','fa':':فضانورد_پوست_آبنوسی:','id':':astronaut_warna_kulit_gelap:','zh':':宇航员_较深肤色:','ru':':космонавт_очень_темный_тон_кожи:','tr':':astronot_koyu_cilt_tonu:','ar':':رائد_الفضاء_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F680':{'en':':astronaut_light_skin_tone:','status':fully_qualified,'E':12.1,'de':':astronaut(in)_helle_hautfarbe:','es':':astronauta_tono_de_piel_claro:','fr':':astronaute_peau_claire:','ja':':宇宙飛行士_薄い肌色:','ko':':우주비행사_하얀_피부:','pt':':astronauta_pele_clara:','it':':astronauta_carnagione_chiara:','fa':':فضانورد_پوست_سفید:','id':':astronaut_warna_kulit_cerah:','zh':':宇航员_较浅肤色:','ru':':космонавт_очень_светлый_тон_кожи:','tr':':astronot_açık_cilt_tonu:','ar':':رائد_الفضاء_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F680':{'en':':astronaut_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':astronaut(in)_mitteldunkle_hautfarbe:','es':':astronauta_tono_de_piel_oscuro_medio:','fr':':astronaute_peau_mate:','ja':':宇宙飛行士_やや濃い肌色:','ko':':우주비행사_진한_갈색_피부:','pt':':astronauta_pele_morena_escura:','it':':astronauta_carnagione_abbastanza_scura:','fa':':فضانورد_پوست_گندمی:','id':':astronaut_warna_kulit_gelap-sedang:','zh':':宇航员_中等深肤色:','ru':':космонавт_темный_тон_кожи:','tr':':astronot_orta_koyu_cilt_tonu:','ar':':رائد_الفضاء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F680':{'en':':astronaut_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':astronaut(in)_mittelhelle_hautfarbe:','es':':astronauta_tono_de_piel_claro_medio:','fr':':astronaute_peau_moyennement_claire:','ja':':宇宙飛行士_やや薄い肌色:','ko':':우주비행사_연한_갈색_피부:','pt':':astronauta_pele_morena_clara:','it':':astronauta_carnagione_abbastanza_chiara:','fa':':فضانورد_پوست_روشن:','id':':astronaut_warna_kulit_cerah-sedang:','zh':':宇航员_中等浅肤色:','ru':':космонавт_светлый_тон_кожи:','tr':':astronot_orta_açık_cilt_tonu:','ar':':رائد_الفضاء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F680':{'en':':astronaut_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':astronaut(in)_mittlere_hautfarbe:','es':':astronauta_tono_de_piel_medio:','fr':':astronaute_peau_légèrement_mate:','ja':':宇宙飛行士_中間の肌色:','ko':':우주비행사_갈색_피부:','pt':':astronauta_pele_morena:','it':':astronauta_carnagione_olivastra:','fa':':فضانورد_پوست_طلایی:','id':':astronaut_warna_kulit_sedang:','zh':':宇航员_中等肤色:','ru':':космонавт_средний_тон_кожи:','tr':':astronot_orta_cilt_tonu:','ar':':رائد_الفضاء_بشرة_بلون_معتدل:'},'\U0000269B\U0000FE0F':{'en':':atom_symbol:','status':fully_qualified,'E':1,'variant':True,'de':':atomzeichen:','es':':símbolo_de_átomo:','fr':':symbole_de_l’atome:','ja':':原子のシンボル:','ko':':원자_기호:','pt':':símbolo_de_átomo:','it':':simbolo_dell’atomo:','fa':':نماد_اتم:','id':':simbol_atom:','zh':':原子符号:','ru':':символ_атома:','tr':':atom_sembolü:','ar':':نووي:'},'\U0000269B':{'en':':atom_symbol:','status':unqualified,'E':1,'variant':True,'de':':atomzeichen:','es':':símbolo_de_átomo:','fr':':symbole_de_l’atome:','ja':':原子のシンボル:','ko':':원자_기호:','pt':':símbolo_de_átomo:','it':':simbolo_dell’atomo:','fa':':نماد_اتم:','id':':simbol_atom:','zh':':原子符号:','ru':':символ_атома:','tr':':atom_sembolü:','ar':':نووي:'},'\U0001F6FA':{'en':':auto_rickshaw:','status':fully_qualified,'E':12,'de':':autorikscha:','es':':mototaxi:','fr':':tuk_tuk:','ja':':三輪タクシー:','ko':':경삼륜차:','pt':':automóvel_riquixá:','it':':risciò_a_motore:','fa':':اتوریکشا:','id':':bajaj:','zh':':三轮摩托车:','ru':':авторикша:','tr':':elektrikli_çekçek:','ar':':توكتوك:'},'\U0001F697':{'en':':automobile:','status':fully_qualified,'E':0.6,'alias':[':car:', ':red_car:'],'de':':auto:','es':':coche:','fr':':voiture:','ja':':自動車:','ko':':자동차:','pt':':carro:','it':':auto:','fa':':خودرو:','id':':mobil:','zh':':汽车:','ru':':автомобиль:','tr':':araba:','ar':':سيارة:'},'\U0001F951':{'en':':avocado:','status':fully_qualified,'E':3,'de':':avocado:','es':':aguacate:','fr':':avocat:','ja':':アボカド:','ko':':아보카도:','pt':':abacate:','it':':avocado:','fa':':آووکادو:','id':':alpukat:','zh':':鳄梨:','ru':':авокадо:','tr':':avokado:','ar':':أفوكادو:'},'\U0001FA93':{'en':':axe:','status':fully_qualified,'E':12,'de':':axt:','es':':hacha:','fr':':hache:','ja':':斧:','ko':':도끼:','pt':':machado:','it':':ascia:','fa':':تبر:','id':':kapak:','zh':':斧头:','ru':':топор:','tr':':balta:','ar':':بلطة:'},'\U0001F476':{'en':':baby:','status':fully_qualified,'E':0.6,'de':':baby:','es':':bebé:','fr':':bébé:','ja':':赤ん坊:','ko':':아기:','pt':':bebê:','it':':neonato:','fa':':نوزاد:','id':':bayi:','zh':':小宝贝:','ru':':младенец:','tr':':bebek:','ar':':رضيع:'},'\U0001F47C':{'en':':baby_angel:','status':fully_qualified,'E':0.6,'alias':[':angel:'],'de':':putte:','es':':bebé_ángel:','fr':':bébé_ange:','ja':':天使:','ko':':아기_천사:','pt':':bebê_anjo:','it':':angioletto:','fa':':فرشته_کوچولو:','id':':malaikat_bayi:','zh':':小天使:','ru':':ангелочек:','tr':':bebek_melek:','ar':':طفل_ملاك:'},'\U0001F47C\U0001F3FF':{'en':':baby_angel_dark_skin_tone:','status':fully_qualified,'E':1,'de':':putte_dunkle_hautfarbe:','es':':bebé_ángel_tono_de_piel_oscuro:','fr':':bébé_ange_peau_foncée:','ja':':天使_濃い肌色:','ko':':아기_천사_검은색_피부:','pt':':bebê_anjo_pele_escura:','it':':angioletto_carnagione_scura:','fa':':فرشته_کوچولو_پوست_آبنوسی:','id':':malaikat_bayi_warna_kulit_gelap:','zh':':小天使_较深肤色:','ru':':ангелочек_очень_темный_тон_кожи:','tr':':bebek_melek_koyu_cilt_tonu:','ar':':طفل_ملاك_بشرة_بلون_غامق:'},'\U0001F47C\U0001F3FB':{'en':':baby_angel_light_skin_tone:','status':fully_qualified,'E':1,'de':':putte_helle_hautfarbe:','es':':bebé_ángel_tono_de_piel_claro:','fr':':bébé_ange_peau_claire:','ja':':天使_薄い肌色:','ko':':아기_천사_하얀_피부:','pt':':bebê_anjo_pele_clara:','it':':angioletto_carnagione_chiara:','fa':':فرشته_کوچولو_پوست_سفید:','id':':malaikat_bayi_warna_kulit_cerah:','zh':':小天使_较浅肤色:','ru':':ангелочек_очень_светлый_тон_кожи:','tr':':bebek_melek_açık_cilt_tonu:','ar':':طفل_ملاك_بشرة_بلون_فاتح:'},'\U0001F47C\U0001F3FE':{'en':':baby_angel_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':putte_mitteldunkle_hautfarbe:','es':':bebé_ángel_tono_de_piel_oscuro_medio:','fr':':bébé_ange_peau_mate:','ja':':天使_やや濃い肌色:','ko':':아기_천사_진한_갈색_피부:','pt':':bebê_anjo_pele_morena_escura:','it':':angioletto_carnagione_abbastanza_scura:','fa':':فرشته_کوچولو_پوست_گندمی:','id':':malaikat_bayi_warna_kulit_gelap-sedang:','zh':':小天使_中等深肤色:','ru':':ангелочек_темный_тон_кожи:','tr':':bebek_melek_orta_koyu_cilt_tonu:','ar':':طفل_ملاك_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F47C\U0001F3FC':{'en':':baby_angel_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':putte_mittelhelle_hautfarbe:','es':':bebé_ángel_tono_de_piel_claro_medio:','fr':':bébé_ange_peau_moyennement_claire:','ja':':天使_やや薄い肌色:','ko':':아기_천사_연한_갈색_피부:','pt':':bebê_anjo_pele_morena_clara:','it':':angioletto_carnagione_abbastanza_chiara:','fa':':فرشته_کوچولو_پوست_روشن:','id':':malaikat_bayi_warna_kulit_cerah-sedang:','zh':':小天使_中等浅肤色:','ru':':ангелочек_светлый_тон_кожи:','tr':':bebek_melek_orta_açık_cilt_tonu:','ar':':طفل_ملاك_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F47C\U0001F3FD':{'en':':baby_angel_medium_skin_tone:','status':fully_qualified,'E':1,'de':':putte_mittlere_hautfarbe:','es':':bebé_ángel_tono_de_piel_medio:','fr':':bébé_ange_peau_légèrement_mate:','ja':':天使_中間の肌色:','ko':':아기_천사_갈색_피부:','pt':':bebê_anjo_pele_morena:','it':':angioletto_carnagione_olivastra:','fa':':فرشته_کوچولو_پوست_طلایی:','id':':malaikat_bayi_warna_kulit_sedang:','zh':':小天使_中等肤色:','ru':':ангелочек_средний_тон_кожи:','tr':':bebek_melek_orta_cilt_tonu:','ar':':طفل_ملاك_بشرة_بلون_معتدل:'},'\U0001F37C':{'en':':baby_bottle:','status':fully_qualified,'E':1,'de':':babyflasche:','es':':biberón:','fr':':biberon:','ja':':ほにゅう瓶:','ko':':젖병:','pt':':mamadeira:','it':':biberon:','fa':':شیشه_شیر:','id':':botol_bayi:','zh':':奶瓶:','ru':':бутылочка_с_соской:','tr':':biberon:','ar':':زجاجة_الرضاعة:'},'\U0001F424':{'en':':baby_chick:','status':fully_qualified,'E':0.6,'de':':küken:','es':':pollito:','fr':':poussin:','ja':':ひよこ:','ko':':병아리:','pt':':pintinho_de_perfil:','it':':pulcino:','fa':':جوجه:','id':':itik_bayi:','zh':':小鸡:','ru':':цыпленок_боком:','tr':':civciv:','ar':':كتكوت_صغير:'},'\U0001F476\U0001F3FF':{'en':':baby_dark_skin_tone:','status':fully_qualified,'E':1,'de':':baby_dunkle_hautfarbe:','es':':bebé_tono_de_piel_oscuro:','fr':':bébé_peau_foncée:','ja':':赤ん坊_濃い肌色:','ko':':아기_검은색_피부:','pt':':bebê_pele_escura:','it':':neonato_carnagione_scura:','fa':':نوزاد_پوست_آبنوسی:','id':':bayi_warna_kulit_gelap:','zh':':小宝贝_较深肤色:','ru':':младенец_очень_темный_тон_кожи:','tr':':bebek_koyu_cilt_tonu:','ar':':رضيع_بشرة_بلون_غامق:'},'\U0001F476\U0001F3FB':{'en':':baby_light_skin_tone:','status':fully_qualified,'E':1,'de':':baby_helle_hautfarbe:','es':':bebé_tono_de_piel_claro:','fr':':bébé_peau_claire:','ja':':赤ん坊_薄い肌色:','ko':':아기_하얀_피부:','pt':':bebê_pele_clara:','it':':neonato_carnagione_chiara:','fa':':نوزاد_پوست_سفید:','id':':bayi_warna_kulit_cerah:','zh':':小宝贝_较浅肤色:','ru':':младенец_очень_светлый_тон_кожи:','tr':':bebek_açık_cilt_tonu:','ar':':رضيع_بشرة_بلون_فاتح:'},'\U0001F476\U0001F3FE':{'en':':baby_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':baby_mitteldunkle_hautfarbe:','es':':bebé_tono_de_piel_oscuro_medio:','fr':':bébé_peau_mate:','ja':':赤ん坊_やや濃い肌色:','ko':':아기_진한_갈색_피부:','pt':':bebê_pele_morena_escura:','it':':neonato_carnagione_abbastanza_scura:','fa':':نوزاد_پوست_گندمی:','id':':bayi_warna_kulit_gelap-sedang:','zh':':小宝贝_中等深肤色:','ru':':младенец_темный_тон_кожи:','tr':':bebek_orta_koyu_cilt_tonu:','ar':':رضيع_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F476\U0001F3FC':{'en':':baby_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':baby_mittelhelle_hautfarbe:','es':':bebé_tono_de_piel_claro_medio:','fr':':bébé_peau_moyennement_claire:','ja':':赤ん坊_やや薄い肌色:','ko':':아기_연한_갈색_피부:','pt':':bebê_pele_morena_clara:','it':':neonato_carnagione_abbastanza_chiara:','fa':':نوزاد_پوست_روشن:','id':':bayi_warna_kulit_cerah-sedang:','zh':':小宝贝_中等浅肤色:','ru':':младенец_светлый_тон_кожи:','tr':':bebek_orta_açık_cilt_tonu:','ar':':رضيع_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F476\U0001F3FD':{'en':':baby_medium_skin_tone:','status':fully_qualified,'E':1,'de':':baby_mittlere_hautfarbe:','es':':bebé_tono_de_piel_medio:','fr':':bébé_peau_légèrement_mate:','ja':':赤ん坊_中間の肌色:','ko':':아기_갈색_피부:','pt':':bebê_pele_morena:','it':':neonato_carnagione_olivastra:','fa':':نوزاد_پوست_طلایی:','id':':bayi_warna_kulit_sedang:','zh':':小宝贝_中等肤色:','ru':':младенец_средний_тон_кожи:','tr':':bebek_orta_cilt_tonu:','ar':':رضيع_بشرة_بلون_معتدل:'},'\U0001F6BC':{'en':':baby_symbol:','status':fully_qualified,'E':0.6,'variant':True,'de':':symbol_baby:','es':':señal_de_bebé:','fr':':symbole_bébé:','ja':':ベビーシート:','ko':':수유실:','pt':':símbolo_de_bebê:','it':':simbolo_con_immagine_di_bambino:','fa':':نماد_نوزاد:','id':':simbol_bayi:','zh':':宝宝:','ru':':пеленальная:','tr':':bebek_işareti:','ar':':إشارة_طفل:'},'\U0001F447':{'en':':backhand_index_pointing_down:','status':fully_qualified,'E':0.6,'alias':[':point_down:'],'variant':True,'de':':nach_unten_weisender_zeigefinger:','es':':dorso_de_mano_con_índice_hacia_abajo:','fr':':main_avec_index_pointant_vers_le_bas:','ja':':下指差し:','ko':':아래쪽을_가리키는_손등:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_baixo:','it':':indice_abbassato:','fa':':اشاره_به_پایین:','id':':menunjuk_ke_bawah:','zh':':反手食指向下指:','ru':':указывающий_вниз_палец:','tr':':elinin_tersiyle_aşağıyı_gösteren_işaret_parmağı:','ar':':إصبع_لأسفل:'},'\U0001F447\U0001F3FF':{'en':':backhand_index_pointing_down_dark_skin_tone:','status':fully_qualified,'E':1,'de':':nach_unten_weisender_zeigefinger_dunkle_hautfarbe:','es':':dorso_de_mano_con_índice_hacia_abajo_tono_de_piel_oscuro:','fr':':main_avec_index_pointant_vers_le_bas_peau_foncée:','ja':':下指差し_濃い肌色:','ko':':아래쪽을_가리키는_손등_검은색_피부:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_baixo_pele_escura:','it':':indice_abbassato_carnagione_scura:','fa':':اشاره_به_پایین_پوست_آبنوسی:','id':':menunjuk_ke_bawah_warna_kulit_gelap:','zh':':反手食指向下指_较深肤色:','ru':':указывающий_вниз_палец_очень_темный_тон_кожи:','tr':':elinin_tersiyle_aşağıyı_gösteren_işaret_parmağı_koyu_cilt_tonu:','ar':':إصبع_لأسفل_بشرة_بلون_غامق:'},'\U0001F447\U0001F3FB':{'en':':backhand_index_pointing_down_light_skin_tone:','status':fully_qualified,'E':1,'de':':nach_unten_weisender_zeigefinger_helle_hautfarbe:','es':':dorso_de_mano_con_índice_hacia_abajo_tono_de_piel_claro:','fr':':main_avec_index_pointant_vers_le_bas_peau_claire:','ja':':下指差し_薄い肌色:','ko':':아래쪽을_가리키는_손등_하얀_피부:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_baixo_pele_clara:','it':':indice_abbassato_carnagione_chiara:','fa':':اشاره_به_پایین_پوست_سفید:','id':':menunjuk_ke_bawah_warna_kulit_cerah:','zh':':反手食指向下指_较浅肤色:','ru':':указывающий_вниз_палец_очень_светлый_тон_кожи:','tr':':elinin_tersiyle_aşağıyı_gösteren_işaret_parmağı_açık_cilt_tonu:','ar':':إصبع_لأسفل_بشرة_بلون_فاتح:'},'\U0001F447\U0001F3FE':{'en':':backhand_index_pointing_down_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':nach_unten_weisender_zeigefinger_mitteldunkle_hautfarbe:','es':':dorso_de_mano_con_índice_hacia_abajo_tono_de_piel_oscuro_medio:','fr':':main_avec_index_pointant_vers_le_bas_peau_mate:','ja':':下指差し_やや濃い肌色:','ko':':아래쪽을_가리키는_손등_진한_갈색_피부:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_baixo_pele_morena_escura:','it':':indice_abbassato_carnagione_abbastanza_scura:','fa':':اشاره_به_پایین_پوست_گندمی:','id':':menunjuk_ke_bawah_warna_kulit_gelap-sedang:','zh':':反手食指向下指_中等深肤色:','ru':':указывающий_вниз_палец_темный_тон_кожи:','tr':':elinin_tersiyle_aşağıyı_gösteren_işaret_parmağı_orta_koyu_cilt_tonu:','ar':':إصبع_لأسفل_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F447\U0001F3FC':{'en':':backhand_index_pointing_down_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':nach_unten_weisender_zeigefinger_mittelhelle_hautfarbe:','es':':dorso_de_mano_con_índice_hacia_abajo_tono_de_piel_claro_medio:','fr':':main_avec_index_pointant_vers_le_bas_peau_moyennement_claire:','ja':':下指差し_やや薄い肌色:','ko':':아래쪽을_가리키는_손등_연한_갈색_피부:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_baixo_pele_morena_clara:','it':':indice_abbassato_carnagione_abbastanza_chiara:','fa':':اشاره_به_پایین_پوست_روشن:','id':':menunjuk_ke_bawah_warna_kulit_cerah-sedang:','zh':':反手食指向下指_中等浅肤色:','ru':':указывающий_вниз_палец_светлый_тон_кожи:','tr':':elinin_tersiyle_aşağıyı_gösteren_işaret_parmağı_orta_açık_cilt_tonu:','ar':':إصبع_لأسفل_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F447\U0001F3FD':{'en':':backhand_index_pointing_down_medium_skin_tone:','status':fully_qualified,'E':1,'de':':nach_unten_weisender_zeigefinger_mittlere_hautfarbe:','es':':dorso_de_mano_con_índice_hacia_abajo_tono_de_piel_medio:','fr':':main_avec_index_pointant_vers_le_bas_peau_légèrement_mate:','ja':':下指差し_中間の肌色:','ko':':아래쪽을_가리키는_손등_갈색_피부:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_baixo_pele_morena:','it':':indice_abbassato_carnagione_olivastra:','fa':':اشاره_به_پایین_پوست_طلایی:','id':':menunjuk_ke_bawah_warna_kulit_sedang:','zh':':反手食指向下指_中等肤色:','ru':':указывающий_вниз_палец_средний_тон_кожи:','tr':':elinin_tersiyle_aşağıyı_gösteren_işaret_parmağı_orta_cilt_tonu:','ar':':إصبع_لأسفل_بشرة_بلون_معتدل:'},'\U0001F448':{'en':':backhand_index_pointing_left:','status':fully_qualified,'E':0.6,'alias':[':point_left:'],'variant':True,'de':':nach_links_weisender_zeigefinger:','es':':dorso_de_mano_con_índice_a_la_izquierda:','fr':':main_avec_index_pointant_à_gauche:','ja':':左指差し:','ko':':왼쪽을_가리키는_손등:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_a_esquerda:','it':':indice_verso_sinistra:','fa':':اشاره_به_چپ:','id':':menunjuk_ke_arah_kiri:','zh':':反手食指向左指:','ru':':указывающий_налево_палец:','tr':':elinin_tersiyle_solu_gösteren_işaret_parmağı:','ar':':إصبع_لليسار:'},'\U0001F448\U0001F3FF':{'en':':backhand_index_pointing_left_dark_skin_tone:','status':fully_qualified,'E':1,'de':':nach_links_weisender_zeigefinger_dunkle_hautfarbe:','es':':dorso_de_mano_con_índice_a_la_izquierda_tono_de_piel_oscuro:','fr':':main_avec_index_pointant_à_gauche_peau_foncée:','ja':':左指差し_濃い肌色:','ko':':왼쪽을_가리키는_손등_검은색_피부:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_a_esquerda_pele_escura:','it':':indice_verso_sinistra_carnagione_scura:','fa':':اشاره_به_چپ_پوست_آبنوسی:','id':':menunjuk_ke_arah_kiri_warna_kulit_gelap:','zh':':反手食指向左指_较深肤色:','ru':':указывающий_налево_палец_очень_темный_тон_кожи:','tr':':elinin_tersiyle_solu_gösteren_işaret_parmağı_koyu_cilt_tonu:','ar':':إصبع_لليسار_بشرة_بلون_غامق:'},'\U0001F448\U0001F3FB':{'en':':backhand_index_pointing_left_light_skin_tone:','status':fully_qualified,'E':1,'de':':nach_links_weisender_zeigefinger_helle_hautfarbe:','es':':dorso_de_mano_con_índice_a_la_izquierda_tono_de_piel_claro:','fr':':main_avec_index_pointant_à_gauche_peau_claire:','ja':':左指差し_薄い肌色:','ko':':왼쪽을_가리키는_손등_하얀_피부:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_a_esquerda_pele_clara:','it':':indice_verso_sinistra_carnagione_chiara:','fa':':اشاره_به_چپ_پوست_سفید:','id':':menunjuk_ke_arah_kiri_warna_kulit_cerah:','zh':':反手食指向左指_较浅肤色:','ru':':указывающий_налево_палец_очень_светлый_тон_кожи:','tr':':elinin_tersiyle_solu_gösteren_işaret_parmağı_açık_cilt_tonu:','ar':':إصبع_لليسار_بشرة_بلون_فاتح:'},'\U0001F448\U0001F3FE':{'en':':backhand_index_pointing_left_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':nach_links_weisender_zeigefinger_mitteldunkle_hautfarbe:','es':':dorso_de_mano_con_índice_a_la_izquierda_tono_de_piel_oscuro_medio:','fr':':main_avec_index_pointant_à_gauche_peau_mate:','ja':':左指差し_やや濃い肌色:','ko':':왼쪽을_가리키는_손등_진한_갈색_피부:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_a_esquerda_pele_morena_escura:','it':':indice_verso_sinistra_carnagione_abbastanza_scura:','fa':':اشاره_به_چپ_پوست_گندمی:','id':':menunjuk_ke_arah_kiri_warna_kulit_gelap-sedang:','zh':':反手食指向左指_中等深肤色:','ru':':указывающий_налево_палец_темный_тон_кожи:','tr':':elinin_tersiyle_solu_gösteren_işaret_parmağı_orta_koyu_cilt_tonu:','ar':':إصبع_لليسار_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F448\U0001F3FC':{'en':':backhand_index_pointing_left_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':nach_links_weisender_zeigefinger_mittelhelle_hautfarbe:','es':':dorso_de_mano_con_índice_a_la_izquierda_tono_de_piel_claro_medio:','fr':':main_avec_index_pointant_à_gauche_peau_moyennement_claire:','ja':':左指差し_やや薄い肌色:','ko':':왼쪽을_가리키는_손등_연한_갈색_피부:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_a_esquerda_pele_morena_clara:','it':':indice_verso_sinistra_carnagione_abbastanza_chiara:','fa':':اشاره_به_چپ_پوست_روشن:','id':':menunjuk_ke_arah_kiri_warna_kulit_cerah-sedang:','zh':':反手食指向左指_中等浅肤色:','ru':':указывающий_налево_палец_светлый_тон_кожи:','tr':':elinin_tersiyle_solu_gösteren_işaret_parmağı_orta_açık_cilt_tonu:','ar':':إصبع_لليسار_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F448\U0001F3FD':{'en':':backhand_index_pointing_left_medium_skin_tone:','status':fully_qualified,'E':1,'de':':nach_links_weisender_zeigefinger_mittlere_hautfarbe:','es':':dorso_de_mano_con_índice_a_la_izquierda_tono_de_piel_medio:','fr':':main_avec_index_pointant_à_gauche_peau_légèrement_mate:','ja':':左指差し_中間の肌色:','ko':':왼쪽을_가리키는_손등_갈색_피부:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_a_esquerda_pele_morena:','it':':indice_verso_sinistra_carnagione_olivastra:','fa':':اشاره_به_چپ_پوست_طلایی:','id':':menunjuk_ke_arah_kiri_warna_kulit_sedang:','zh':':反手食指向左指_中等肤色:','ru':':указывающий_налево_палец_средний_тон_кожи:','tr':':elinin_tersiyle_solu_gösteren_işaret_parmağı_orta_cilt_tonu:','ar':':إصبع_لليسار_بشرة_بلون_معتدل:'},'\U0001F449':{'en':':backhand_index_pointing_right:','status':fully_qualified,'E':0.6,'alias':[':point_right:'],'variant':True,'de':':nach_rechts_weisender_zeigefinger:','es':':dorso_de_mano_con_índice_a_la_derecha:','fr':':main_avec_index_pointant_à_droite:','ja':':右指差し:','ko':':오른쪽을_가리키는_손등:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_a_direita:','it':':indice_verso_destra:','fa':':اشاره_به_راست:','id':':menunjuk_ke_arah_kanan:','zh':':反手食指向右指:','ru':':указывающий_направо_палец:','tr':':elinin_tersiyle_sağı_gösteren_işaret_parmağı:','ar':':إصبع_لليمين:'},'\U0001F449\U0001F3FF':{'en':':backhand_index_pointing_right_dark_skin_tone:','status':fully_qualified,'E':1,'de':':nach_rechts_weisender_zeigefinger_dunkle_hautfarbe:','es':':dorso_de_mano_con_índice_a_la_derecha_tono_de_piel_oscuro:','fr':':main_avec_index_pointant_à_droite_peau_foncée:','ja':':右指差し_濃い肌色:','ko':':오른쪽을_가리키는_손등_검은색_피부:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_a_direita_pele_escura:','it':':indice_verso_destra_carnagione_scura:','fa':':اشاره_به_راست_پوست_آبنوسی:','id':':menunjuk_ke_arah_kanan_warna_kulit_gelap:','zh':':反手食指向右指_较深肤色:','ru':':указывающий_направо_палец_очень_темный_тон_кожи:','tr':':elinin_tersiyle_sağı_gösteren_işaret_parmağı_koyu_cilt_tonu:','ar':':إصبع_لليمين_بشرة_بلون_غامق:'},'\U0001F449\U0001F3FB':{'en':':backhand_index_pointing_right_light_skin_tone:','status':fully_qualified,'E':1,'de':':nach_rechts_weisender_zeigefinger_helle_hautfarbe:','es':':dorso_de_mano_con_índice_a_la_derecha_tono_de_piel_claro:','fr':':main_avec_index_pointant_à_droite_peau_claire:','ja':':右指差し_薄い肌色:','ko':':오른쪽을_가리키는_손등_하얀_피부:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_a_direita_pele_clara:','it':':indice_verso_destra_carnagione_chiara:','fa':':اشاره_به_راست_پوست_سفید:','id':':menunjuk_ke_arah_kanan_warna_kulit_cerah:','zh':':反手食指向右指_较浅肤色:','ru':':указывающий_направо_палец_очень_светлый_тон_кожи:','tr':':elinin_tersiyle_sağı_gösteren_işaret_parmağı_açık_cilt_tonu:','ar':':إصبع_لليمين_بشرة_بلون_فاتح:'},'\U0001F449\U0001F3FE':{'en':':backhand_index_pointing_right_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':nach_rechts_weisender_zeigefinger_mitteldunkle_hautfarbe:','es':':dorso_de_mano_con_índice_a_la_derecha_tono_de_piel_oscuro_medio:','fr':':main_avec_index_pointant_à_droite_peau_mate:','ja':':右指差し_やや濃い肌色:','ko':':오른쪽을_가리키는_손등_진한_갈색_피부:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_a_direita_pele_morena_escura:','it':':indice_verso_destra_carnagione_abbastanza_scura:','fa':':اشاره_به_راست_پوست_گندمی:','id':':menunjuk_ke_arah_kanan_warna_kulit_gelap-sedang:','zh':':反手食指向右指_中等深肤色:','ru':':указывающий_направо_палец_темный_тон_кожи:','tr':':elinin_tersiyle_sağı_gösteren_işaret_parmağı_orta_koyu_cilt_tonu:','ar':':إصبع_لليمين_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F449\U0001F3FC':{'en':':backhand_index_pointing_right_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':nach_rechts_weisender_zeigefinger_mittelhelle_hautfarbe:','es':':dorso_de_mano_con_índice_a_la_derecha_tono_de_piel_claro_medio:','fr':':main_avec_index_pointant_à_droite_peau_moyennement_claire:','ja':':右指差し_やや薄い肌色:','ko':':오른쪽을_가리키는_손등_연한_갈색_피부:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_a_direita_pele_morena_clara:','it':':indice_verso_destra_carnagione_abbastanza_chiara:','fa':':اشاره_به_راست_پوست_روشن:','id':':menunjuk_ke_arah_kanan_warna_kulit_cerah-sedang:','zh':':反手食指向右指_中等浅肤色:','ru':':указывающий_направо_палец_светлый_тон_кожи:','tr':':elinin_tersiyle_sağı_gösteren_işaret_parmağı_orta_açık_cilt_tonu:','ar':':إصبع_لليمين_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F449\U0001F3FD':{'en':':backhand_index_pointing_right_medium_skin_tone:','status':fully_qualified,'E':1,'de':':nach_rechts_weisender_zeigefinger_mittlere_hautfarbe:','es':':dorso_de_mano_con_índice_a_la_derecha_tono_de_piel_medio:','fr':':main_avec_index_pointant_à_droite_peau_légèrement_mate:','ja':':右指差し_中間の肌色:','ko':':오른쪽을_가리키는_손등_갈색_피부:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_a_direita_pele_morena:','it':':indice_verso_destra_carnagione_olivastra:','fa':':اشاره_به_راست_پوست_طلایی:','id':':menunjuk_ke_arah_kanan_warna_kulit_sedang:','zh':':反手食指向右指_中等肤色:','ru':':указывающий_направо_палец_средний_тон_кожи:','tr':':elinin_tersiyle_sağı_gösteren_işaret_parmağı_orta_cilt_tonu:','ar':':إصبع_لليمين_بشرة_بلون_معتدل:'},'\U0001F446':{'en':':backhand_index_pointing_up:','status':fully_qualified,'E':0.6,'alias':[':point_up_2:'],'variant':True,'de':':nach_oben_weisender_zeigefinger_von_hinten:','es':':dorso_de_mano_con_índice_hacia_arriba:','fr':':main_avec_index_pointant_vers_le_haut:','ja':':手の甲上指差し:','ko':':위쪽을_가리키는_손등:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_cima:','it':':indice_alzato:','fa':':اشاره_به_بالا_با_پشت_دست:','id':':menunjuk_ke_atas_(dibalik):','zh':':反手食指向上指:','ru':':указывающий_вверх_палец:','tr':':elinin_tersiyle_yukarıyı_gösteren_işaret_parmağı:','ar':':إصبع_لأعلى:'},'\U0001F446\U0001F3FF':{'en':':backhand_index_pointing_up_dark_skin_tone:','status':fully_qualified,'E':1,'de':':nach_oben_weisender_zeigefinger_von_hinten_dunkle_hautfarbe:','es':':dorso_de_mano_con_índice_hacia_arriba_tono_de_piel_oscuro:','fr':':main_avec_index_pointant_vers_le_haut_peau_foncée:','ja':':手の甲上指差し_濃い肌色:','ko':':위쪽을_가리키는_손등_검은색_피부:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_cima_pele_escura:','it':':indice_alzato_carnagione_scura:','fa':':اشاره_به_بالا_با_پشت_دست_پوست_آبنوسی:','id':':menunjuk_ke_atas_(dibalik)_warna_kulit_gelap:','zh':':反手食指向上指_较深肤色:','ru':':указывающий_вверх_палец_очень_темный_тон_кожи:','tr':':elinin_tersiyle_yukarıyı_gösteren_işaret_parmağı_koyu_cilt_tonu:','ar':':إصبع_لأعلى_بشرة_بلون_غامق:'},'\U0001F446\U0001F3FB':{'en':':backhand_index_pointing_up_light_skin_tone:','status':fully_qualified,'E':1,'de':':nach_oben_weisender_zeigefinger_von_hinten_helle_hautfarbe:','es':':dorso_de_mano_con_índice_hacia_arriba_tono_de_piel_claro:','fr':':main_avec_index_pointant_vers_le_haut_peau_claire:','ja':':手の甲上指差し_薄い肌色:','ko':':위쪽을_가리키는_손등_하얀_피부:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_cima_pele_clara:','it':':indice_alzato_carnagione_chiara:','fa':':اشاره_به_بالا_با_پشت_دست_پوست_سفید:','id':':menunjuk_ke_atas_(dibalik)_warna_kulit_cerah:','zh':':反手食指向上指_较浅肤色:','ru':':указывающий_вверх_палец_очень_светлый_тон_кожи:','tr':':elinin_tersiyle_yukarıyı_gösteren_işaret_parmağı_açık_cilt_tonu:','ar':':إصبع_لأعلى_بشرة_بلون_فاتح:'},'\U0001F446\U0001F3FE':{'en':':backhand_index_pointing_up_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':nach_oben_weisender_zeigefinger_von_hinten_mitteldunkle_hautfarbe:','es':':dorso_de_mano_con_índice_hacia_arriba_tono_de_piel_oscuro_medio:','fr':':main_avec_index_pointant_vers_le_haut_peau_mate:','ja':':手の甲上指差し_やや濃い肌色:','ko':':위쪽을_가리키는_손등_진한_갈색_피부:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_cima_pele_morena_escura:','it':':indice_alzato_carnagione_abbastanza_scura:','fa':':اشاره_به_بالا_با_پشت_دست_پوست_گندمی:','id':':menunjuk_ke_atas_(dibalik)_warna_kulit_gelap-sedang:','zh':':反手食指向上指_中等深肤色:','ru':':указывающий_вверх_палец_темный_тон_кожи:','tr':':elinin_tersiyle_yukarıyı_gösteren_işaret_parmağı_orta_koyu_cilt_tonu:','ar':':إصبع_لأعلى_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F446\U0001F3FC':{'en':':backhand_index_pointing_up_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':nach_oben_weisender_zeigefinger_von_hinten_mittelhelle_hautfarbe:','es':':dorso_de_mano_con_índice_hacia_arriba_tono_de_piel_claro_medio:','fr':':main_avec_index_pointant_vers_le_haut_peau_moyennement_claire:','ja':':手の甲上指差し_やや薄い肌色:','ko':':위쪽을_가리키는_손등_연한_갈색_피부:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_cima_pele_morena_clara:','it':':indice_alzato_carnagione_abbastanza_chiara:','fa':':اشاره_به_بالا_با_پشت_دست_پوست_روشن:','id':':menunjuk_ke_atas_(dibalik)_warna_kulit_cerah-sedang:','zh':':反手食指向上指_中等浅肤色:','ru':':указывающий_вверх_палец_светлый_тон_кожи:','tr':':elinin_tersiyle_yukarıyı_gösteren_işaret_parmağı_orta_açık_cilt_tonu:','ar':':إصبع_لأعلى_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F446\U0001F3FD':{'en':':backhand_index_pointing_up_medium_skin_tone:','status':fully_qualified,'E':1,'de':':nach_oben_weisender_zeigefinger_von_hinten_mittlere_hautfarbe:','es':':dorso_de_mano_con_índice_hacia_arriba_tono_de_piel_medio:','fr':':main_avec_index_pointant_vers_le_haut_peau_légèrement_mate:','ja':':手の甲上指差し_中間の肌色:','ko':':위쪽을_가리키는_손등_갈색_피부:','pt':':dorso_da_mão_com_dedo_indicador_apontando_para_cima_pele_morena:','it':':indice_alzato_carnagione_olivastra:','fa':':اشاره_به_بالا_با_پشت_دست_پوست_طلایی:','id':':menunjuk_ke_atas_(dibalik)_warna_kulit_sedang:','zh':':反手食指向上指_中等肤色:','ru':':указывающий_вверх_палец_средний_тон_кожи:','tr':':elinin_tersiyle_yukarıyı_gösteren_işaret_parmağı_orta_cilt_tonu:','ar':':إصبع_لأعلى_بشرة_بلون_معتدل:'},'\U0001F392':{'en':':backpack:','status':fully_qualified,'E':0.6,'alias':[':school_satchel:'],'de':':schulranzen:','es':':mochila_escolar:','fr':':cartable:','ja':':バックパック:','ko':':학교_가방:','pt':':mochila:','it':':zaino:','fa':':کوله_پشتی:','id':':tas_sekolah:','zh':':书包:','ru':':ранец:','tr':':okul_çantası:','ar':':حقيبة_مدرسة:'},'\U0001F953':{'en':':bacon:','status':fully_qualified,'E':3,'de':':bacon:','es':':beicon:','fr':':lard:','ja':':ベーコン:','ko':':베이컨:','pt':':bacon:','it':':pancetta:','fa':':بیکن:','id':':babi_asap:','zh':':培根:','ru':':бекон:','tr':':domuz_pastırması:','ar':':لحم_مقدد:'},'\U0001F9A1':{'en':':badger:','status':fully_qualified,'E':11,'de':':dachs:','es':':tejón:','fr':':blaireau:','ja':':アナグマ:','ko':':오소리:','pt':':texugo:','it':':tasso:','fa':':گورکن:','id':':musang:','zh':':獾:','ru':':барсук:','tr':':porsuk:','ar':':غرير:'},'\U0001F3F8':{'en':':badminton:','status':fully_qualified,'E':1,'alias':[':badminton_racquet_and_shuttlecock:'],'de':':badminton:','es':':bádminton:','fr':':badminton:','ja':':バドミントン:','ko':':배드민턴:','pt':':badminton:','it':':badminton:','fa':':بدمینتون:','id':':bulu_tangkis:','zh':':羽毛球:','ru':':бадминтон:','tr':':badminton:','ar':':تنس_الريشة:'},'\U0001F96F':{'en':':bagel:','status':fully_qualified,'E':11,'de':':bagel:','es':':bagel:','fr':':bagel:','ja':':ベーグル:','ko':':베이글:','pt':':rosca:','it':':bagel:','fa':':نان_بیگل:','id':':bagel:','zh':':面包圈:','ru':':бублик:','tr':':simit:','ar':':خبز_البيغل:'},'\U0001F6C4':{'en':':baggage_claim:','status':fully_qualified,'E':1,'de':':gepäckausgabe:','es':':recogida_de_equipajes:','fr':':retrait_des_bagages:','ja':':手荷物受取:','ko':':짐_찾는_곳:','pt':':restituição_de_bagagem:','it':':simbolo_del_ritiro_bagagli:','fa':':دریافت_بار:','id':':pengambilan_bagasi:','zh':':提取行李:','ru':':получение_багажа:','tr':':bagaj_teslim:','ar':':استرداد_الحقائب:'},'\U0001F956':{'en':':baguette_bread:','status':fully_qualified,'E':3,'de':':baguette:','es':':baguete:','fr':':baguette:','ja':':バゲット:','ko':':바게트:','pt':':baguete:','it':':baguette:','fa':':نان_باگت:','id':':roti_baguette:','zh':':法式长棍面包:','ru':':багет:','tr':':baget_ekmek:','ar':':الخبز_الفرنسي:'},'\U00002696\U0000FE0F':{'en':':balance_scale:','status':fully_qualified,'E':1,'alias':[':scales:'],'variant':True,'de':':waage:','es':':balanza:','fr':':balance_à_poids:','ja':':天秤:','ko':':접시저울:','pt':':balança:','it':':bilancia_a_doppio_piatto:','fa':':ترازو:','id':':timbangan_gantung:','zh':':天平:','ru':':весы:','tr':':terazi:','ar':':ميزان:'},'\U00002696':{'en':':balance_scale:','status':unqualified,'E':1,'alias':[':scales:'],'variant':True,'de':':waage:','es':':balanza:','fr':':balance_à_poids:','ja':':天秤:','ko':':접시저울:','pt':':balança:','it':':bilancia_a_doppio_piatto:','fa':':ترازو:','id':':timbangan_gantung:','zh':':天平:','ru':':весы:','tr':':terazi:','ar':':ميزان:'},'\U0001F9B2':{'en':':bald:','status':component,'E':11,'de':':glatze:','es':':sin_pelo:','fr':':chauve:','ja':':はげ頭:','ko':':대머리:','pt':':careca:','it':':calvo:','fa':':تاس:','id':':gundul:','zh':':秃顶:','ru':':без_волос:','tr':':kel:','ar':':أصلع:'},'\U0001FA70':{'en':':ballet_shoes:','status':fully_qualified,'E':12,'de':':ballettschuhe:','es':':zapatillas_de_ballet:','fr':':chaussons_de_danse:','ja':':トウシューズ:','ko':':발레_슈즈:','pt':':sapatilha_de_balé:','it':':scarpette_da_ballerina:','fa':':کفش_باله:','id':':sepatu_balet:','zh':':芭蕾舞鞋:','ru':':пуанты:','tr':':bale_ayakkabıları:','ar':':حذاء_باليه:'},'\U0001F388':{'en':':balloon:','status':fully_qualified,'E':0.6,'de':':luftballon:','es':':globo:','fr':':ballon_gonflable:','ja':':風船:','ko':':풍선:','pt':':balão:','it':':palloncino:','fa':':بادکنک:','id':':balon:','zh':':气球:','ru':':воздушный_шарик:','tr':':balon:','ar':':بالون:'},'\U0001F5F3\U0000FE0F':{'en':':ballot_box_with_ballot:','status':fully_qualified,'E':0.7,'alias':[':ballot_box:'],'variant':True,'de':':urne_mit_wahlzettel:','es':':urna_con_papeleta:','fr':':urne_électorale:','ja':':投票箱:','ko':':투표_용지가_있는_투표_상자:','pt':':urna_eleitoral_com_cédula:','it':':urna_elettorale_con_scheda:','fa':':صندوق_رأی:','id':':kotak_dengan_surat_suara:','zh':':投票箱:','ru':':избирательная_урна:','tr':':oy_sandığı:','ar':':صندوق_اقتراع_وورقة_اقتراع:'},'\U0001F5F3':{'en':':ballot_box_with_ballot:','status':unqualified,'E':0.7,'alias':[':ballot_box:'],'variant':True,'de':':urne_mit_wahlzettel:','es':':urna_con_papeleta:','fr':':urne_électorale:','ja':':投票箱:','ko':':투표_용지가_있는_투표_상자:','pt':':urna_eleitoral_com_cédula:','it':':urna_elettorale_con_scheda:','fa':':صندوق_رأی:','id':':kotak_dengan_surat_suara:','zh':':投票箱:','ru':':избирательная_урна:','tr':':oy_sandığı:','ar':':صندوق_اقتراع_وورقة_اقتراع:'},'\U0001F34C':{'en':':banana:','status':fully_qualified,'E':0.6,'de':':banane:','es':':plátano:','fr':':banane:','ja':':バナナ:','ko':':바나나:','pt':':banana:','it':':banana:','fa':':موز:','id':':pisang:','zh':':香蕉:','ru':':банан:','tr':':muz:','ar':':موز:'},'\U0001FA95':{'en':':banjo:','status':fully_qualified,'E':12,'de':':banjo:','es':':banjo:','fr':':banjo:','ja':':バンジョー:','ko':':밴조:','pt':':banjo:','it':':banjo:','fa':':بانجو:','id':':banjo:','zh':':班卓琴:','ru':':банджо:','tr':':banjo:','ar':':آلة_بانجو:'},'\U0001F3E6':{'en':':bank:','status':fully_qualified,'E':0.6,'de':':bank:','es':':banco:','fr':':banque:','ja':':銀行:','ko':':은행:','pt':':banco:','it':':banca:','fa':':بانک:','id':':bank:','zh':':银行:','ru':':банк:','tr':':banka:','ar':':بنك:'},'\U0001F4CA':{'en':':bar_chart:','status':fully_qualified,'E':0.6,'de':':balkendiagramm:','es':':gráfico_de_barras:','fr':':graphiques_à_barres:','ja':':棒グラフ:','ko':':막대_그래프:','pt':':gráfico_de_barras:','it':':grafico_a_barre:','fa':':نمودار_ستونی:','id':':diagram_batang:','zh':':条形图:','ru':':гистограмма:','tr':':çubuk_grafiği:','ar':':مخطط_بياني_شريطي:'},'\U0001F488':{'en':':barber_pole:','status':fully_qualified,'E':0.6,'alias':[':barber:'],'de':':barbershop-säule:','es':':poste_de_barbero:','fr':':enseigne_de_barbier:','ja':':床屋:','ko':':이발소:','pt':':barbearia:','it':':barbiere:','fa':':چراغ_سلمانی:','id':':tanda_salon:','zh':':理发店:','ru':':парикмахерская:','tr':':berber_dükkanı:','ar':':محل_حلاقة:'},'\U000026BE':{'en':':baseball:','status':fully_qualified,'E':0.6,'variant':True,'de':':baseball:','es':':béisbol:','fr':':baseball:','ja':':野球:','ko':':야구공:','pt':':bola_de_beisebol:','it':':palla_da_baseball:','fa':':بیسبال:','id':':bisbol:','zh':':棒球:','ru':':бейсбол:','tr':':beyzbol:','ar':':بيسبول:'},'\U0001F9FA':{'en':':basket:','status':fully_qualified,'E':11,'de':':korb:','es':':cesta:','fr':':panier:','ja':':かご:','ko':':바구니:','pt':':cesta:','it':':cesto:','fa':':سبد:','id':':keranjang:','zh':':筐:','ru':':корзина:','tr':':sepet:','ar':':سلة:'},'\U0001F3C0':{'en':':basketball:','status':fully_qualified,'E':0.6,'de':':basketball:','es':':balón_de_baloncesto:','fr':':basket:','ja':':バスケットボール:','ko':':농구:','pt':':bola_de_basquete:','it':':palla_da_pallacanestro:','fa':':بسکتبال:','id':':bola_basket:','zh':':篮球:','ru':':баскетбол:','tr':':basketbol:','ar':':كرة_سلة:'},'\U0001F987':{'en':':bat:','status':fully_qualified,'E':3,'de':':fledermaus:','es':':murciélago:','fr':':chauve-souris:','ja':':コウモリ:','ko':':박쥐:','pt':':morcego:','it':':pipistrello:','fa':':خفاش:','id':':kelelawar:','zh':':蝙蝠:','ru':':летучая_мышь:','tr':':yarasa:','ar':':خفاش:'},'\U0001F6C1':{'en':':bathtub:','status':fully_qualified,'E':1,'de':':badewanne:','es':':bañera:','fr':':baignoire:','ja':':バスタブ:','ko':':욕조:','pt':':banheira:','it':':vasca:','fa':':وان_حمام:','id':':bathtub:','zh':':浴缸:','ru':':ванна:','tr':':banyo_küveti:','ar':':حوض_استحمام:'},'\U0001F50B':{'en':':battery:','status':fully_qualified,'E':0.6,'de':':batterie:','es':':pila:','fr':':pile:','ja':':電池:','ko':':배터리:','pt':':pilha:','it':':batteria:','fa':':باتری:','id':':baterai:','zh':':电池:','ru':':батарейка:','tr':':pil:','ar':':بطارية:'},'\U0001F3D6\U0000FE0F':{'en':':beach_with_umbrella:','status':fully_qualified,'E':0.7,'alias':[':beach_umbrella:'],'variant':True,'de':':strand_mit_sonnenschirm:','es':':playa_y_sombrilla:','fr':':plage_avec_parasol:','ja':':ビーチパラソル:','ko':':파라솔이_있는_해변:','pt':':praia_e_guarda-sol:','it':':spiaggia_con_ombrellone:','fa':':ساحل_و_سایبان:','id':':pantai_dan_payung:','zh':':沙滩伞:','ru':':пляж:','tr':':plaj_ve_şemsiye:','ar':':شاطئ_وشمسية:'},'\U0001F3D6':{'en':':beach_with_umbrella:','status':unqualified,'E':0.7,'alias':[':beach_umbrella:'],'variant':True,'de':':strand_mit_sonnenschirm:','es':':playa_y_sombrilla:','fr':':plage_avec_parasol:','ja':':ビーチパラソル:','ko':':파라솔이_있는_해변:','pt':':praia_e_guarda-sol:','it':':spiaggia_con_ombrellone:','fa':':ساحل_و_سایبان:','id':':pantai_dan_payung:','zh':':沙滩伞:','ru':':пляж:','tr':':plaj_ve_şemsiye:','ar':':شاطئ_وشمسية:'},'\U0001F601':{'en':':beaming_face_with_smiling_eyes:','status':fully_qualified,'E':0.6,'alias':[':grin:'],'de':':strahlendes_gesicht_mit_lachenden_augen:','es':':cara_radiante_con_ojos_sonrientes:','fr':':visage_souriant_aux_yeux_rieurs:','ja':':にやっと笑う:','ko':':미소_짓는_눈으로_웃는_얼굴:','pt':':rosto_contente_com_olhos_sorridentes:','it':':faccina_raggiante_con_occhi_felici:','fa':':خندهٔ_دندانی:','id':':wajah_gembira_dengan_mata_bahagia:','zh':':嘻嘻:','ru':':светится_от_счастья:','tr':':gülen_gözlerle_sırıtma:','ar':':وجه_مشرق_بعينين_باسمتين:'},'\U0001FAD8':{'en':':beans:','status':fully_qualified,'E':14,'de':':bohnen:','es':':alubias:','fr':':haricots:','ja':':豆:','ko':':콩:','pt':':feijões:','it':':fagioli:','fa':':لوبیا:','id':':kacang_merah:','zh':':豆:','ru':':фасоль:','tr':':fasulye:','ar':':بقول:'},'\U0001F43B':{'en':':bear:','status':fully_qualified,'E':0.6,'de':':bär:','es':':oso:','fr':':ours:','ja':':クマの顔:','ko':':곰_얼굴:','pt':':rosto_de_urso:','it':':orso:','fa':':خرس:','id':':beruang:','zh':':熊:','ru':':морда_медведя:','tr':':ayı:','ar':':وجه_دب:'},'\U0001F493':{'en':':beating_heart:','status':fully_qualified,'E':0.6,'alias':[':heartbeat:'],'de':':schlagendes_herz:','es':':corazón_latiendo:','fr':':cœur_battant:','ja':':ドキドキするハート:','ko':':두근거리는_하트:','pt':':coração_pulsante:','it':':cuore_che_batte:','fa':':قلب_تپنده:','id':':hati_berdetak:','zh':':心跳:','ru':':пульсирующее_сердце:','tr':':çarpan_kalp:','ar':':قلب_يخفق:'},'\U0001F9AB':{'en':':beaver:','status':fully_qualified,'E':13,'de':':biber:','es':':castor:','fr':':castor:','ja':':ビーバー:','ko':':비버:','pt':':castor:','it':':castoro:','fa':':بیدستر:','id':':biwara:','zh':':海狸:','ru':':бобр:','tr':':kunduz:','ar':':سمور:'},'\U0001F6CF\U0000FE0F':{'en':':bed:','status':fully_qualified,'E':0.7,'variant':True,'de':':bett:','es':':cama:','fr':':lit:','ja':':ベッド:','ko':':침대:','pt':':cama:','it':':letto:','fa':':تختخواب:','id':':tempat_tidur:','zh':':床:','ru':':кровать:','tr':':yatak:','ar':':سرير:'},'\U0001F6CF':{'en':':bed:','status':unqualified,'E':0.7,'variant':True,'de':':bett:','es':':cama:','fr':':lit:','ja':':ベッド:','ko':':침대:','pt':':cama:','it':':letto:','fa':':تختخواب:','id':':tempat_tidur:','zh':':床:','ru':':кровать:','tr':':yatak:','ar':':سرير:'},'\U0001F37A':{'en':':beer_mug:','status':fully_qualified,'E':0.6,'alias':[':beer:'],'de':':bierkrug:','es':':jarra_de_cerveza:','fr':':chope:','ja':':ビールジョッキ:','ko':':맥주잔:','pt':':cerveja:','it':':boccale_di_birra:','fa':':لیوان_آب_جو:','id':':mug_bir:','zh':':啤酒:','ru':':кружка_пива:','tr':':bira_kupası:','ar':':كأس_بيرة:'},'\U0001FAB2':{'en':':beetle:','status':fully_qualified,'E':13,'de':':käfer:','es':':escarabajo:','fr':':scarabée:','ja':':カブトムシ:','ko':':딱정벌레:','pt':':besouro:','it':':coleottero:','fa':':سوسک:','id':':kumbang:','zh':':甲虫:','ru':':жук:','tr':':böcek:','ar':':خنفساء:'},'\U0001F514':{'en':':bell:','status':fully_qualified,'E':0.6,'de':':glocke:','es':':campana:','fr':':cloche:','ja':':ベル:','ko':':종:','pt':':sino:','it':':campana:','fa':':زنگ:','id':':lonceng:','zh':':铃铛:','ru':':колокольчик:','tr':':zil:','ar':':جرس:'},'\U0001FAD1':{'en':':bell_pepper:','status':fully_qualified,'E':13,'de':':paprika:','es':':pimiento:','fr':':poivron:','ja':':赤ピーマン:','ko':':피망:','pt':':pimentão:','it':':peperone:','fa':':فلفل_دلمه_ای:','id':':paprika:','zh':':灯笼椒:','ru':':болгарский_перец:','tr':':dolma_biber:','ar':':فلفل_حلو:'},'\U0001F515':{'en':':bell_with_slash:','status':fully_qualified,'E':1,'alias':[':no_bell:'],'de':':durchgestrichene_glocke:','es':':campana_con_signo_de_cancelación:','fr':':alarme_désactivée:','ja':':ベル消音:','ko':':음소거:','pt':':sino_silenciado:','it':':campana_sbarrata:','fa':':خط_قرمز_روی_زنگوله:','id':':lonceng_dicoret:','zh':':禁止响铃:','ru':':перечеркнутый_колокольчик:','tr':':üzeri_çizili_zil:','ar':':ممنوع_استخدام_الجرس:'},'\U0001F6CE\U0000FE0F':{'en':':bellhop_bell:','status':fully_qualified,'E':0.7,'variant':True,'de':':rezeptionsklingel:','es':':timbre_de_hotel:','fr':':cloche_de_comptoir:','ja':':ベルボーイベル:','ko':':호출_벨:','pt':':sineta:','it':':campanello_da_hotel:','fa':':زنگ_میز_پذیرش:','id':':bel_meja:','zh':':服务铃:','ru':':гостиничныи_звонок:','tr':':resepsiyon_zili:','ar':':جرس_فندق:'},'\U0001F6CE':{'en':':bellhop_bell:','status':unqualified,'E':0.7,'variant':True,'de':':rezeptionsklingel:','es':':timbre_de_hotel:','fr':':cloche_de_comptoir:','ja':':ベルボーイベル:','ko':':호출_벨:','pt':':sineta:','it':':campanello_da_hotel:','fa':':زنگ_میز_پذیرش:','id':':bel_meja:','zh':':服务铃:','ru':':гостиничный_звонок:','tr':':resepsiyon_zili:','ar':':جرس_فندق:'},'\U0001F371':{'en':':bento_box:','status':fully_qualified,'E':0.6,'alias':[':bento:'],'de':':bento-box:','es':':caja_de_bento:','fr':':boîte_déjeuner:','ja':':弁当:','ko':':도시락:','pt':':bentô:','it':':bento_box:','fa':':جعبه_بنتو:','id':':kotak_bento:','zh':':盒饭:','ru':':бэнто:','tr':':bento:','ar':':علبة_طعام:'},'\U0001F9C3':{'en':':beverage_box:','status':fully_qualified,'E':12,'de':':trinkpäckchen:','es':':tetrabrik:','fr':':briquette_de_jus:','ja':':紙パック飲料:','ko':':음료_팩:','pt':':suco_de_caixa:','it':':bevanda_monodose:','fa':':پاکت_نوشیدنی:','id':':kotak_minuman:','zh':':饮料盒:','ru':':пакетик_сока:','tr':':içecek_kutusu:','ar':':علبة_عصير:'},'\U0001F6B2':{'en':':bicycle:','status':fully_qualified,'E':0.6,'alias':[':bike:'],'variant':True,'de':':fahrrad:','es':':bicicleta:','fr':':vélo:','ja':':自転車:','ko':':자전거:','pt':':bicicleta:','it':':bicicletta:','fa':':دوچرخه:','id':':sepeda:','zh':':自行车:','ru':':велосипед:','tr':':bisiklet:','ar':':دراجة:'},'\U0001F459':{'en':':bikini:','status':fully_qualified,'E':0.6,'de':':bikini:','es':':bikini:','fr':':bikini:','ja':':ビキニ:','ko':':비키니:','pt':':biquíni:','it':':bikini:','fa':':بیکینی:','id':':bikini:','zh':':比基尼:','ru':':бикини:','tr':':bikini:','ar':':ملابس_البحر:'},'\U0001F9E2':{'en':':billed_cap:','status':fully_qualified,'E':5,'de':':baseballmütze:','es':':gorra_con_visera:','fr':':casquette_américaine:','ja':':キャップ:','ko':':야구모자:','pt':':boné:','it':':cappello_con_visiera:','fa':':کلاه_لبه_دار:','id':':topi_berlidah:','zh':':鸭舌帽:','ru':':бейсболка:','tr':':siperli_şapka:','ar':':قبعة_رياضية:'},'\U00002623\U0000FE0F':{'en':':biohazard:','status':fully_qualified,'E':1,'alias':[':biohazard_sign:'],'variant':True,'de':':biogefährdung:','es':':riesgo_biológico:','fr':':danger_biologique:','ja':':バイオハザード:','ko':':생물학적_위험:','pt':':risco_biológico:','it':':simbolo_del_rischio_biologico:','fa':':خطر_زیستی:','id':':biohazard:','zh':':生物危害:','ru':':биологическая_угроза:','tr':':biyolojik_tehlike:','ar':':خطورة_بيولوجية:'},'\U00002623':{'en':':biohazard:','status':unqualified,'E':1,'alias':[':biohazard_sign:'],'variant':True,'de':':biogefährdung:','es':':riesgo_biológico:','fr':':danger_biologique:','ja':':バイオハザード:','ko':':생물학적_위험:','pt':':risco_biológico:','it':':simbolo_del_rischio_biologico:','fa':':خطر_زیستی:','id':':biohazard:','zh':':生物危害:','ru':':биологическая_угроза:','tr':':biyolojik_tehlike:','ar':':خطورة_بيولوجية:'},'\U0001F426':{'en':':bird:','status':fully_qualified,'E':0.6,'variant':True,'de':':vogel:','es':':pájaro:','fr':':oiseau:','ja':':鳥:','ko':':새:','pt':':pássaro:','it':':uccello:','fa':':پرنده:','id':':burung:','zh':':鸟:','ru':':птица:','tr':':kuş:','ar':':طائر:'},'\U0001F382':{'en':':birthday_cake:','status':fully_qualified,'E':0.6,'alias':[':birthday:'],'de':':geburtstagskuchen:','es':':tarta_de_cumpleaños:','fr':':gâteau_d’anniversaire:','ja':':バースデーケーキ:','ko':':생일_케이크:','pt':':bolo_de_aniversário:','it':':torta_di_compleanno:','fa':':کیک_تولد:','id':':kue_ulang_tahun:','zh':':生日蛋糕:','ru':':торт_ко_дню_рождения:','tr':':doğum_günü_pastası:','ar':':كعكة_عيد_ميلاد:'},'\U0001F9AC':{'en':':bison:','status':fully_qualified,'E':13,'de':':bison:','es':':bisonte:','fr':':bison:','ja':':バイソン:','ko':':들소:','pt':':bisão:','it':':bisonte:','fa':':گاومیش:','id':':bison:','zh':':大野牛:','ru':':бизон:','tr':':bizon:','ar':':ثور_أمريكي:'},'\U0001FAE6':{'en':':biting_lip:','status':fully_qualified,'E':14,'de':':auf_lippe_beißen:','es':':labio_mordido:','fr':':lèvres_qui_se_mordent:','ja':':唇を咬んでいる口:','ko':':입술을_깨물고_있는_입:','pt':':mordendo_o_lábio:','it':':bocca_che_morde_il_labbro:','fa':':گازگرفتن_لب:','id':':menggigit_bibir:','zh':':咬住嘴唇:','ru':':кусать_губы:','tr':':ısırılan_dudak:','ar':':عض_الشفاه:'},'\U0001F426\U0000200D\U00002B1B':{'en':':black_bird:','status':fully_qualified,'E':15,'alias':[':raven:', ':crow:', ':rook:'],'de':':schwarzer_vogel:','es':':pájaro_negro:','fr':':oiseau_noir:','ja':':黒い鳥:','ko':':검은_새:','pt':':pássaro_preto:','it':':merlo:','fa':':پرنده_سیاه:','id':':burung_hitam:','zh':':黑色的鸟:','ru':':черная_птица:','tr':':siyah_kuş:','ar':':طائر_أسود:'},'\U0001F408\U0000200D\U00002B1B':{'en':':black_cat:','status':fully_qualified,'E':13,'de':':schwarze_katze:','es':':gato_negro:','fr':':chat_noir:','ja':':黒猫:','ko':':검은_고양이:','pt':':gato_preto:','it':':gatto_nero:','fa':':گربه_سیاه:','id':':kucing_hitam:','zh':':黑猫:','ru':':черная_кошка:','tr':':kara_kedi:','ar':':هر_أسود:'},'\U000026AB':{'en':':black_circle:','status':fully_qualified,'E':0.6,'variant':True,'de':':schwarzer_punkt:','es':':círculo_negro:','fr':':disque_noir:','ja':':黒い丸:','ko':':검은색_원:','pt':':círculo_preto:','it':':cerchio_nero:','fa':':دایره_مشکی:','id':':lingkaran_hitam:','zh':':黑色圆:','ru':':черный_круг:','tr':':siyah_daire:','ar':':دائرة_سوداء:'},'\U0001F3F4':{'en':':black_flag:','status':fully_qualified,'E':1,'alias':[':waving_black_flag:'],'de':':schwarze_flagge:','es':':bandera_negra:','fr':':drapeau_noir:','ja':':黒旗:','ko':':펄럭이는_검은색_깃발:','pt':':bandeira_preta:','it':':bandiera_nera:','fa':':پرچم_سیاه:','id':':bendera_hitam:','zh':':举黑旗:','ru':':черный_флаг:','tr':':siyah_bayrak:','ar':':علم_أسود_يرفرف:'},'\U0001F5A4':{'en':':black_heart:','status':fully_qualified,'E':3,'de':':schwarzes_herz:','es':':corazón_negro:','fr':':cœur_noir:','ja':':黒いハート:','ko':':검은색_하트:','pt':':coração_preto:','it':':cuore_nero:','fa':':قلب_سیاه:','id':':hati_hitam:','zh':':黑心:','ru':':черное_сердце:','tr':':siyah_kalp:','ar':':قلب_أسود:'},'\U00002B1B':{'en':':black_large_square:','status':fully_qualified,'E':0.6,'variant':True,'de':':großes_schwarzes_quadrat:','es':':cuadrado_negro_grande:','fr':':grand_carré_noir:','ja':':黒四角特大:','ko':':큰_검은색_사각형:','pt':':quadrado_preto_grande:','it':':quadrato_nero_grande:','fa':':مربع_مشکی_بزرگ:','id':':persegi_besar_hitam:','zh':':黑线大方框:','ru':':большой_черный_квадрат:','tr':':büyük_siyah_kare:','ar':':مربع_كبير_أسود:'},'\U000025FE':{'en':':black_medium-small_square:','status':fully_qualified,'E':0.6,'alias':[':black_medium_small_square:'],'variant':True,'de':':mittelkleines_schwarzes_quadrat:','es':':cuadrado_negro_mediano-pequeño:','fr':':carré_petit_moyen_noir:','ja':':黒四角中:','ko':':중각_작은_검은색_사각형:','pt':':quadrado_preto_médio_menor:','it':':quadrato_nero_medio-piccolo:','fa':':مربع_مشکی_نسبتاً_کوچک:','id':':persegi_kecil_medium_hitam:','zh':':黑色中小方块:','ru':':небольшой_черный_квадрат:','tr':':küçük-orta_siyah_kare:','ar':':مربع_صغير_متوسط_أسود:'},'\U000025FC\U0000FE0F':{'en':':black_medium_square:','status':fully_qualified,'E':0.6,'variant':True,'de':':mittelgroßes_schwarzes_quadrat:','es':':cuadrado_negro_mediano:','fr':':carré_moyen_noir:','ja':':黒四角大:','ko':':중간_검은색_사각형:','pt':':quadrado_preto_médio:','it':':quadrato_nero_medio:','fa':':مربع_مشکی_متوسط:','id':':persegi_medium_hitam:','zh':':黑色中方块:','ru':':черныи_квадрат:','tr':':orta_boy_siyah_kare:','ar':':مربع_متوسط_أسود:'},'\U000025FC':{'en':':black_medium_square:','status':unqualified,'E':0.6,'variant':True,'de':':mittelgroßes_schwarzes_quadrat:','es':':cuadrado_negro_mediano:','fr':':carré_moyen_noir:','ja':':黒四角大:','ko':':중간_검은색_사각형:','pt':':quadrado_preto_médio:','it':':quadrato_nero_medio:','fa':':مربع_مشکی_متوسط:','id':':persegi_medium_hitam:','zh':':黑色中方块:','ru':':черный_квадрат:','tr':':orta_boy_siyah_kare:','ar':':مربع_متوسط_أسود:'},'\U00002712\U0000FE0F':{'en':':black_nib:','status':fully_qualified,'E':0.6,'variant':True,'de':':schwarzer_federhalter:','es':':pluma_negra:','fr':':stylo_plume_noir:','ja':':ペン先:','ko':':검은색_펜촉:','pt':':ponta_de_caneta_tinteiro:','it':':pennino:','fa':':قلم_مشکی:','id':':ujung_pena:','zh':':钢笔尖:','ru':':перьевои_наконечник:','tr':':siyah_uç:','ar':':سن_قلم_أسود:'},'\U00002712':{'en':':black_nib:','status':unqualified,'E':0.6,'variant':True,'de':':schwarzer_federhalter:','es':':pluma_negra:','fr':':stylo_plume_noir:','ja':':ペン先:','ko':':검은색_펜촉:','pt':':ponta_de_caneta_tinteiro:','it':':pennino:','fa':':قلم_مشکی:','id':':ujung_pena:','zh':':钢笔尖:','ru':':перьевой_наконечник:','tr':':siyah_uç:','ar':':سن_قلم_أسود:'},'\U000025AA\U0000FE0F':{'en':':black_small_square:','status':fully_qualified,'E':0.6,'variant':True,'de':':kleines_schwarzes_quadrat:','es':':cuadrado_negro_pequeño:','fr':':petit_carré_noir:','ja':':黒四角小:','ko':':작은_검은색_사각형:','pt':':quadrado_preto_pequeno:','it':':quadrato_nero_piccolo:','fa':':مربع_مشکی_کوچک:','id':':persegi_kecil_hitam:','zh':':黑色小方块:','ru':':черныи_квадратик:','tr':':küçük_siyah_kare:','ar':':مربع_صغير_أسود:'},'\U000025AA':{'en':':black_small_square:','status':unqualified,'E':0.6,'variant':True,'de':':kleines_schwarzes_quadrat:','es':':cuadrado_negro_pequeño:','fr':':petit_carré_noir:','ja':':黒四角小:','ko':':작은_검은색_사각형:','pt':':quadrado_preto_pequeno:','it':':quadrato_nero_piccolo:','fa':':مربع_مشکی_کوچک:','id':':persegi_kecil_hitam:','zh':':黑色小方块:','ru':':черный_квадратик:','tr':':küçük_siyah_kare:','ar':':مربع_صغير_أسود:'},'\U0001F532':{'en':':black_square_button:','status':fully_qualified,'E':0.6,'de':':schwarze_quadratische_schaltfläche:','es':':botón_cuadrado_con_borde_negro:','fr':':carré_noir:','ja':':黒枠四角:','ko':':테두리가_검은_사각_버튼:','pt':':botão_quadrado_preto:','it':':tasto_quadrato_bianco_con_bordo_nero:','fa':':دکمۀ_مربع_مشکی:','id':':tombol_persegi_hitam:','zh':':黑色方形按钮:','ru':':кнопка_с_черной_рамкой:','tr':':siyah_kare_düğme:','ar':':زر_مربع_أسود:'},'\U0001F33C':{'en':':blossom:','status':fully_qualified,'E':0.6,'de':':gelbe_blüte:','es':':flor:','fr':':bourgeon:','ja':':開花:','ko':':꽃송이:','pt':':flor:','it':':fiore:','fa':':شکوفه:','id':':mekar:','zh':':开花:','ru':':цветок:','tr':':çiçek_açma:','ar':':زهرة:'},'\U0001F421':{'en':':blowfish:','status':fully_qualified,'E':0.6,'de':':kugelfisch:','es':':pez_globo:','fr':':poisson-lune:','ja':':フグ:','ko':':복어:','pt':':baiacu:','it':':pesce_palla:','fa':':ماهی_بادکنکی:','id':':ikan_gembung:','zh':':河豚:','ru':':рыба_еж:','tr':':balon_balığı:','ar':':سمكة_منتفخة:'},'\U0001F4D8':{'en':':blue_book:','status':fully_qualified,'E':0.6,'de':':blaues_buch:','es':':libro_azul:','fr':':livre_bleu:','ja':':青の本:','ko':':파란색_책:','pt':':livro_azul:','it':':libro_blu:','fa':':کتاب_آبی:','id':':buku_biru:','zh':':蓝色书本:','ru':':синяя_книга:','tr':':mavi_kitap:','ar':':كتاب_أزرق:'},'\U0001F535':{'en':':blue_circle:','status':fully_qualified,'E':0.6,'alias':[':large_blue_circle:'],'de':':blauer_punkt:','es':':círculo_azul:','fr':':disque_bleu:','ja':':青い丸:','ko':':파란_원:','pt':':círculo_azul:','it':':cerchio_blu:','fa':':دایره_آبی:','id':':lingkaran_biru:','zh':':蓝色圆:','ru':':голубой_круг:','tr':':mavi_daire:','ar':':دائرة_زرقاء:'},'\U0001F499':{'en':':blue_heart:','status':fully_qualified,'E':0.6,'de':':blaues_herz:','es':':corazón_azul:','fr':':cœur_bleu:','ja':':青いハート:','ko':':파란색_하트:','pt':':coração_azul:','it':':cuore_blu:','fa':':قلب_آبی:','id':':hati_biru:','zh':':蓝心:','ru':':синее_сердце:','tr':':mavi_kalp:','ar':':قلب_أزرق:'},'\U0001F7E6':{'en':':blue_square:','status':fully_qualified,'E':12,'de':':blaues_quadrat:','es':':cuadrado_azul:','fr':':carré_bleu:','ja':':青い四角:','ko':':파란색_사각형:','pt':':quadrado_azul:','it':':quadrato_blu:','fa':':مربع_آبی:','id':':persegi_biru:','zh':':蓝色方块:','ru':':синий_квадрат:','tr':':mavi_kare:','ar':':مربع_أزرق:'},'\U0001FAD0':{'en':':blueberries:','status':fully_qualified,'E':13,'de':':blaubeeren:','es':':arándanos:','fr':':myrtilles:','ja':':ブルーベリー:','ko':':블루베리:','pt':':mirtilos:','it':':mirtilli:','fa':':توت_آبی:','id':':blueberry:','zh':':蓝莓:','ru':':голубика:','tr':':yaban_mersini:','ar':':توت_بري:'},'\U0001F417':{'en':':boar:','status':fully_qualified,'E':0.6,'de':':wildschwein:','es':':jabalí:','fr':':sanglier:','ja':':イノシシ:','ko':':멧돼지:','pt':':javali:','it':':cinghiale:','fa':':گراز_نر:','id':':babi_hutan:','zh':':野猪:','ru':':кабан:','tr':':yaban_domuzu:','ar':':خنزير_بري:'},'\U0001F4A3':{'en':':bomb:','status':fully_qualified,'E':0.6,'variant':True,'de':':bombe:','es':':bomba:','fr':':bombe:','ja':':爆弾:','ko':':폭탄:','pt':':bomba:','it':':bomba:','fa':':بمب:','id':':bom:','zh':':炸弹:','ru':':бомба:','tr':':bomba:','ar':':قنبلة:'},'\U0001F9B4':{'en':':bone:','status':fully_qualified,'E':11,'de':':knochen:','es':':hueso:','fr':':os:','ja':':骨:','ko':':뼈다귀:','pt':':osso:','it':':osso:','fa':':استخوان:','id':':tulang:','zh':':骨头:','ru':':кость:','tr':':kemik:','ar':':عظمة:'},'\U0001F516':{'en':':bookmark:','status':fully_qualified,'E':0.6,'de':':lesezeichen:','es':':marcapáginas:','fr':':marque-page:','ja':':しおり:','ko':':북마크:','pt':':marcador_de_página:','it':':segnalibro:','fa':':نشان_گذار_کتاب:','id':':pembatas_buku:','zh':':书签:','ru':':закладка:','tr':':yer_işareti:','ar':':علامة_توقف_قراءة:'},'\U0001F4D1':{'en':':bookmark_tabs:','status':fully_qualified,'E':0.6,'de':':pagemarker:','es':':marcadores:','fr':':signets:','ja':':ページに付箋:','ko':':북마크_탭:','pt':':marcadores_de_página:','it':':etichette_segnalibro:','fa':':برچسب_های_نشانه_گذاری:','id':':penanda_halaman_buku:','zh':':标签页:','ru':':стикеры_закладки:','tr':':sayfa_ayraçları:','ar':':لصاقات_علامات_توقف_قراءة:'},'\U0001F4DA':{'en':':books:','status':fully_qualified,'E':0.6,'variant':True,'de':':bücherstapel:','es':':libros:','fr':':livres:','ja':':本の山:','ko':':책_여러_권:','pt':':livros:','it':':libri:','fa':':چند_کتاب:','id':':buku:','zh':':书:','ru':':книги:','tr':':kitaplar:','ar':':كتب:'},'\U0001FA83':{'en':':boomerang:','status':fully_qualified,'E':13,'de':':bumerang:','es':':bumerán:','fr':':boomerang:','ja':':ブーメラン:','ko':':부메랑:','pt':':bumerangue:','it':':boomerang:','fa':':بومرنگ:','id':':bumerang:','zh':':回旋镖:','ru':':бумеранг:','tr':':bumerang:','ar':':خذوف_مرتد:'},'\U0001F37E':{'en':':bottle_with_popping_cork:','status':fully_qualified,'E':1,'alias':[':champagne:'],'de':':flasche_mit_knallendem_korken:','es':':botella_descorchada:','fr':':bouteille_de_champagne:','ja':':シャンパン:','ko':':코르크가_튀어나오고_있는_병:','pt':':garrafa_de_champanhe:','it':':bottiglia_stappata:','fa':':شامپاین_جوشان:','id':':botol_dengan_penyumbat_terlepas:','zh':':开香槟:','ru':':шампанское:','tr':':patlayan_mantarlı_şişe:','ar':':زجاجة_بسدادة_مفرقعة:'},'\U0001F490':{'en':':bouquet:','status':fully_qualified,'E':0.6,'de':':blumenstrauß:','es':':ramo_de_flores:','fr':':bouquet:','ja':':花束:','ko':':꽃다발:','pt':':buquê:','it':':mazzo_di_fiori:','fa':':دسته_گل:','id':':karangan_bunga:','zh':':花束:','ru':':букет:','tr':':buket:','ar':':باقة_زهور:'},'\U0001F3F9':{'en':':bow_and_arrow:','status':fully_qualified,'E':1,'de':':pfeil_und_bogen:','es':':arco_y_flecha:','fr':':arc_et_flèche:','ja':':弓矢:','ko':':활과_화살:','pt':':arco_e_flecha:','it':':arco_e_freccia:','fa':':تیر_و_کمان:','id':':busur_dan_panah:','zh':':弓和箭:','ru':':лук_со_стрелой:','tr':':ok_ve_yay:','ar':':قوس_وسهم:'},'\U0001F963':{'en':':bowl_with_spoon:','status':fully_qualified,'E':5,'de':':schüssel_mit_löffel:','es':':cuenco_con_cuchara:','fr':':bol_avec_cuillère:','ja':':ボウルとスプーン:','ko':':스푼과_사발:','pt':':tigela_com_colher:','it':':ciotola_con_cucchiaio:','fa':':کاسه_و_قاشق:','id':':mangkuk_dengan_sendok:','zh':':碗勺:','ru':':миска_с_ложкой:','tr':':kaşıklı_kase:','ar':':طبق_وملعقة:'},'\U0001F3B3':{'en':':bowling:','status':fully_qualified,'E':0.6,'de':':bowling:','es':':bolos:','fr':':bowling:','ja':':ボウリング:','ko':':볼링:','pt':':boliche:','it':':bowling:','fa':':بولینگ:','id':':bowling:','zh':':保龄球:','ru':':боулинг:','tr':':bovling:','ar':':بولينغ:'},'\U0001F94A':{'en':':boxing_glove:','status':fully_qualified,'E':3,'de':':boxhandschuh:','es':':guante_de_boxeo:','fr':':gant_de_boxe:','ja':':ボクシング:','ko':':권투_글러브:','pt':':luva_de_boxe:','it':':guantone_da_pugilato:','fa':':دست_کش_بوکس:','id':':sarung_tinju:','zh':':拳击手套:','ru':':боксерская_перчатка:','tr':':boks_eldiveni:','ar':':قفاز_ملاكمة:'},'\U0001F466':{'en':':boy:','status':fully_qualified,'E':0.6,'de':':junge:','es':':niño:','fr':':garçon:','ja':':男の子:','ko':':남자_아이:','pt':':menino:','it':':bambino:','fa':':پسر:','id':':anak_laki-laki:','zh':':男孩:','ru':':мальчик:','tr':':erkek_çocuk:','ar':':صبي:'},'\U0001F466\U0001F3FF':{'en':':boy_dark_skin_tone:','status':fully_qualified,'E':1,'de':':junge_dunkle_hautfarbe:','es':':niño_tono_de_piel_oscuro:','fr':':garçon_peau_foncée:','ja':':男の子_濃い肌色:','ko':':남자_아이_검은색_피부:','pt':':menino_pele_escura:','it':':bambino_carnagione_scura:','fa':':پسر_پوست_آبنوسی:','id':':anak_laki-laki_warna_kulit_gelap:','zh':':男孩_较深肤色:','ru':':мальчик_очень_темный_тон_кожи:','tr':':erkek_çocuk_koyu_cilt_tonu:','ar':':صبي_بشرة_بلون_غامق:'},'\U0001F466\U0001F3FB':{'en':':boy_light_skin_tone:','status':fully_qualified,'E':1,'de':':junge_helle_hautfarbe:','es':':niño_tono_de_piel_claro:','fr':':garçon_peau_claire:','ja':':男の子_薄い肌色:','ko':':남자_아이_하얀_피부:','pt':':menino_pele_clara:','it':':bambino_carnagione_chiara:','fa':':پسر_پوست_سفید:','id':':anak_laki-laki_warna_kulit_cerah:','zh':':男孩_较浅肤色:','ru':':мальчик_очень_светлый_тон_кожи:','tr':':erkek_çocuk_açık_cilt_tonu:','ar':':صبي_بشرة_بلون_فاتح:'},'\U0001F466\U0001F3FE':{'en':':boy_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':junge_mitteldunkle_hautfarbe:','es':':niño_tono_de_piel_oscuro_medio:','fr':':garçon_peau_mate:','ja':':男の子_やや濃い肌色:','ko':':남자_아이_진한_갈색_피부:','pt':':menino_pele_morena_escura:','it':':bambino_carnagione_abbastanza_scura:','fa':':پسر_پوست_گندمی:','id':':anak_laki-laki_warna_kulit_gelap-sedang:','zh':':男孩_中等深肤色:','ru':':мальчик_темный_тон_кожи:','tr':':erkek_çocuk_orta_koyu_cilt_tonu:','ar':':صبي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F466\U0001F3FC':{'en':':boy_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':junge_mittelhelle_hautfarbe:','es':':niño_tono_de_piel_claro_medio:','fr':':garçon_peau_moyennement_claire:','ja':':男の子_やや薄い肌色:','ko':':남자_아이_연한_갈색_피부:','pt':':menino_pele_morena_clara:','it':':bambino_carnagione_abbastanza_chiara:','fa':':پسر_پوست_روشن:','id':':anak_laki-laki_warna_kulit_cerah-sedang:','zh':':男孩_中等浅肤色:','ru':':мальчик_светлый_тон_кожи:','tr':':erkek_çocuk_orta_açık_cilt_tonu:','ar':':صبي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F466\U0001F3FD':{'en':':boy_medium_skin_tone:','status':fully_qualified,'E':1,'de':':junge_mittlere_hautfarbe:','es':':niño_tono_de_piel_medio:','fr':':garçon_peau_légèrement_mate:','ja':':男の子_中間の肌色:','ko':':남자_아이_갈색_피부:','pt':':menino_pele_morena:','it':':bambino_carnagione_olivastra:','fa':':پسر_پوست_طلایی:','id':':anak_laki-laki_warna_kulit_sedang:','zh':':男孩_中等肤色:','ru':':мальчик_средний_тон_кожи:','tr':':erkek_çocuk_orta_cilt_tonu:','ar':':صبي_بشرة_بلون_معتدل:'},'\U0001F9E0':{'en':':brain:','status':fully_qualified,'E':5,'de':':gehirn:','es':':cerebro:','fr':':cerveau:','ja':':脳:','ko':':뇌:','pt':':cérebro:','it':':cervello:','fa':':مغز:','id':':otak:','zh':':脑:','ru':':мозг:','tr':':beyin:','ar':':دماغ:'},'\U0001F35E':{'en':':bread:','status':fully_qualified,'E':0.6,'de':':brot:','es':':pan_de_molde:','fr':':pain:','ja':':食パン:','ko':':빵:','pt':':pão:','it':':pane_in_cassetta:','fa':':نان:','id':':roti:','zh':':面包:','ru':':хлеб:','tr':':ekmek:','ar':':خبز:'},'\U0001F931':{'en':':breast-feeding:','status':fully_qualified,'E':5,'alias':[':breast_feeding:'],'de':':stillen:','es':':lactancia_materna:','fr':':allaitement:','ja':':授乳:','ko':':모유_수유:','pt':':amamentando:','it':':allattare:','fa':':تغذیه_با_شیر_مادر:','id':':menyusui:','zh':':母乳喂养:','ru':':кормление_грудью:','tr':':emziren_kadın:','ar':':رضاعة:'},'\U0001F931\U0001F3FF':{'en':':breast-feeding_dark_skin_tone:','status':fully_qualified,'E':5,'de':':stillen_dunkle_hautfarbe:','es':':lactancia_materna_tono_de_piel_oscuro:','fr':':allaitement_peau_foncée:','ja':':授乳_濃い肌色:','ko':':모유_수유_검은색_피부:','pt':':amamentando_pele_escura:','it':':allattare_carnagione_scura:','fa':':تغذیه_با_شیر_مادر_پوست_آبنوسی:','id':':menyusui_warna_kulit_gelap:','zh':':母乳喂养_较深肤色:','ru':':кормление_грудью_очень_темный_тон_кожи:','tr':':emziren_kadın_koyu_cilt_tonu:','ar':':رضاعة_بشرة_بلون_غامق:'},'\U0001F931\U0001F3FB':{'en':':breast-feeding_light_skin_tone:','status':fully_qualified,'E':5,'de':':stillen_helle_hautfarbe:','es':':lactancia_materna_tono_de_piel_claro:','fr':':allaitement_peau_claire:','ja':':授乳_薄い肌色:','ko':':모유_수유_하얀_피부:','pt':':amamentando_pele_clara:','it':':allattare_carnagione_chiara:','fa':':تغذیه_با_شیر_مادر_پوست_سفید:','id':':menyusui_warna_kulit_cerah:','zh':':母乳喂养_较浅肤色:','ru':':кормление_грудью_очень_светлый_тон_кожи:','tr':':emziren_kadın_açık_cilt_tonu:','ar':':رضاعة_بشرة_بلون_فاتح:'},'\U0001F931\U0001F3FE':{'en':':breast-feeding_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':stillen_mitteldunkle_hautfarbe:','es':':lactancia_materna_tono_de_piel_oscuro_medio:','fr':':allaitement_peau_mate:','ja':':授乳_やや濃い肌色:','ko':':모유_수유_진한_갈색_피부:','pt':':amamentando_pele_morena_escura:','it':':allattare_carnagione_abbastanza_scura:','fa':':تغذیه_با_شیر_مادر_پوست_گندمی:','id':':menyusui_warna_kulit_gelap-sedang:','zh':':母乳喂养_中等深肤色:','ru':':кормление_грудью_темный_тон_кожи:','tr':':emziren_kadın_orta_koyu_cilt_tonu:','ar':':رضاعة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F931\U0001F3FC':{'en':':breast-feeding_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':stillen_mittelhelle_hautfarbe:','es':':lactancia_materna_tono_de_piel_claro_medio:','fr':':allaitement_peau_moyennement_claire:','ja':':授乳_やや薄い肌色:','ko':':모유_수유_연한_갈색_피부:','pt':':amamentando_pele_morena_clara:','it':':allattare_carnagione_abbastanza_chiara:','fa':':تغذیه_با_شیر_مادر_پوست_روشن:','id':':menyusui_warna_kulit_cerah-sedang:','zh':':母乳喂养_中等浅肤色:','ru':':кормление_грудью_светлый_тон_кожи:','tr':':emziren_kadın_orta_açık_cilt_tonu:','ar':':رضاعة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F931\U0001F3FD':{'en':':breast-feeding_medium_skin_tone:','status':fully_qualified,'E':5,'de':':stillen_mittlere_hautfarbe:','es':':lactancia_materna_tono_de_piel_medio:','fr':':allaitement_peau_légèrement_mate:','ja':':授乳_中間の肌色:','ko':':모유_수유_갈색_피부:','pt':':amamentando_pele_morena:','it':':allattare_carnagione_olivastra:','fa':':تغذیه_با_شیر_مادر_پوست_طلایی:','id':':menyusui_warna_kulit_sedang:','zh':':母乳喂养_中等肤色:','ru':':кормление_грудью_средний_тон_кожи:','tr':':emziren_kadın_orta_cilt_tonu:','ar':':رضاعة_بشرة_بلون_معتدل:'},'\U0001F9F1':{'en':':brick:','status':fully_qualified,'E':11,'alias':[':bricks:'],'de':':ziegelstein:','es':':ladrillo:','fr':':brique:','ja':':れんが:','ko':':벽돌:','pt':':tijolo:','it':':mattoni:','fa':':دیوار_آجری:','id':':batu_bata:','zh':':砖:','ru':':кирпичи:','tr':':tuğla:','ar':':طوبة:'},'\U0001F309':{'en':':bridge_at_night:','status':fully_qualified,'E':0.6,'de':':brücke_vor_nachthimmel:','es':':puente_de_noche:','fr':':pont_de_nuit:','ja':':夜の橋:','ko':':밤하늘을_배경으로_하는_다리:','pt':':ponte_à_noite:','it':':ponte_di_notte:','fa':':پل_در_شب:','id':':jembatan_di_malam_hari:','zh':':夜幕下的桥:','ru':':мост_ночью:','tr':':gece_vakti_köprü:','ar':':جسر_في_الليل:'},'\U0001F4BC':{'en':':briefcase:','status':fully_qualified,'E':0.6,'de':':aktentasche:','es':':maletín:','fr':':porte-documents:','ja':':ブリーフケース:','ko':':서류_가방:','pt':':maleta:','it':':valigetta_24_ore:','fa':':کیف_اسناد:','id':':tas_kantor:','zh':':公文包:','ru':':портфель:','tr':':evrak_çantası:','ar':':حقيبة_كتب:'},'\U0001FA72':{'en':':briefs:','status':fully_qualified,'E':12,'alias':[':swim_brief:'],'de':':slip:','es':':ropa_interior:','fr':':slip:','ja':':ブリーフ:','ko':':삼각_수영복:','pt':':cueca:','it':':slip:','fa':':شورت_اسلیپ:','id':':celana_renang:','zh':':三角裤:','ru':':плавки:','tr':':slip_mayo:','ar':':ملابس_تحتية:'},'\U0001F506':{'en':':bright_button:','status':fully_qualified,'E':1,'alias':[':high_brightness:'],'de':':heller-taste:','es':':brillo_alto:','fr':':luminosité_élevée:','ja':':高輝度:','ko':':밝기_높음:','pt':':botão_de_aumentar_brilho:','it':':luminosità_elevata:','fa':':دکمهٔ_روشن_کردن_صفحه:','id':':tombol_cerahkan:','zh':':高亮度按钮:','ru':':высокая_яркость:','tr':':yüksek_parlaklık:','ar':':زر_سطوع_مرتفع:'},'\U0001F966':{'en':':broccoli:','status':fully_qualified,'E':5,'de':':brokkoli:','es':':brócoli:','fr':':brocoli:','ja':':ブロッコリー:','ko':':브로콜리:','pt':':brócolis:','it':':broccoli:','fa':':بروکلی:','id':':brokoli:','zh':':西兰花:','ru':':брокколи:','tr':':brokoli:','ar':':بروكولي:'},'\U000026D3\U0000FE0F\U0000200D\U0001F4A5':{'en':':broken_chain:','status':fully_qualified,'E':15.1,'de':':gebrochene_kette:','es':':cadena_rota:','fr':':chaîne_brisée:','ja':':壊れた鎖:','ko':':끊어진_쇠사슬:','pt':':corrente_quebrada:','it':':catena_spezzata:','fa':':زنجیر_پاره:','id':':rantai_putus:','zh':':断链:','ru':':разорванная_цепь:','tr':':zincirleri_kırmak:','ar':':سلسلة_مكسورة:'},'\U000026D3\U0000200D\U0001F4A5':{'en':':broken_chain:','status':unqualified,'E':15.1,'de':':gebrochene_kette:','es':':cadena_rota:','fr':':chaîne_brisée:','ja':':壊れた鎖:','ko':':끊어진_쇠사슬:','pt':':corrente_quebrada:','it':':catena_spezzata:','fa':':زنجیر_پاره:','id':':rantai_putus:','zh':':断链:','ru':':разорванная_цепь:','tr':':zincirleri_kırmak:','ar':':سلسلة_مكسورة:'},'\U0001F494':{'en':':broken_heart:','status':fully_qualified,'E':0.6,'de':':gebrochenes_herz:','es':':corazón_roto:','fr':':cœur_brisé:','ja':':割れたハート:','ko':':깨진_하트:','pt':':coração_partido:','it':':cuore_infranto:','fa':':قلب_شکسته:','id':':patah_hati:','zh':':心碎:','ru':':разбитое_сердце:','tr':':kırık_kalp:','ar':':قلب_مكسور:'},'\U0001F9F9':{'en':':broom:','status':fully_qualified,'E':11,'de':':besen:','es':':escoba:','fr':':balai:','ja':':ほうき:','ko':':빗자루:','pt':':vassoura:','it':':scopa:','fa':':جارو:','id':':sapu:','zh':':扫帚:','ru':':метла:','tr':':süpürge:','ar':':مكنسة:'},'\U0001F7E4':{'en':':brown_circle:','status':fully_qualified,'E':12,'de':':brauner_punkt:','es':':círculo_marrón:','fr':':disque_marron:','ja':':茶色の丸:','ko':':갈색_원:','pt':':círculo_marrom:','it':':cerchio_marrone:','fa':':دایره_قهوه_ای:','id':':lingkaran_cokelat:','zh':':棕色圆:','ru':':коричневый_круг:','tr':':kahverengi_daire:','ar':':دائرة_بنية:'},'\U0001F90E':{'en':':brown_heart:','status':fully_qualified,'E':12,'de':':braunes_herz:','es':':corazón_marrón:','fr':':cœur_marron:','ja':':茶色いハート:','ko':':갈색_하트:','pt':':coração_marrom:','it':':cuore_marrone:','fa':':قلب_قهوه_ای:','id':':hati_cokelat:','zh':':棕心:','ru':':коричневое_сердце:','tr':':kahverengi_kalp:','ar':':قلب_بني:'},'\U0001F344\U0000200D\U0001F7EB':{'en':':brown_mushroom:','status':fully_qualified,'E':15.1,'de':':brauner_pilz:','es':':champiñón_marrón:','fr':':champignon_marron:','ja':':きのこ:','ko':':갈색_양송이_버섯:','pt':':cogumelo_marrom:','it':':fungo_marrone:','fa':':قارچ_قهوه_ای:','id':':jamur_cokelat:','zh':':褐色蘑菇:','ru':':бурый_гриб:','tr':':kahverengi_mantar:','ar':':فطر_بني:'},'\U0001F7EB':{'en':':brown_square:','status':fully_qualified,'E':12,'de':':braunes_quadrat:','es':':cuadrado_marrón:','fr':':carré_marron:','ja':':茶色い四角:','ko':':갈색_사각형:','pt':':quadrado_marrom:','it':':quadrato_marrone:','fa':':مربع_قهوه_ای:','id':':persegi_cokelat:','zh':':棕色方块:','ru':':коричневый_квадрат:','tr':':kahverengi_kare:','ar':':مربع_بني:'},'\U0001F9CB':{'en':':bubble_tea:','status':fully_qualified,'E':13,'de':':bubble_tea:','es':':té_de_burbujas:','fr':':thé_aux_perles:','ja':':タピオカドリンク:','ko':':버블티:','pt':':chá_perolado:','it':':bubble_tea:','fa':':چای_حُبابی:','id':':teh_bubble:','zh':':珍珠奶茶:','ru':':чай_с_шариками:','tr':':bubble_tea:','ar':':شاي_الفقاعات:'},'\U0001FAE7':{'en':':bubbles:','status':fully_qualified,'E':14,'de':':blasen:','es':':burbujas:','fr':':bulles:','ja':':泡:','ko':':거품:','pt':':bolhas:','it':':bolle:','fa':':حباب:','id':':gelembung:','zh':':气泡:','ru':':пузыри:','tr':':baloncuk:','ar':':فقاعات:'},'\U0001FAA3':{'en':':bucket:','status':fully_qualified,'E':13,'de':':eimer:','es':':cubo:','fr':':seau:','ja':':バケツ:','ko':':양동이:','pt':':balde:','it':':secchio:','fa':':سطل:','id':':ember:','zh':':桶:','ru':':ведро:','tr':':kova:','ar':':دلو:'},'\U0001F41B':{'en':':bug:','status':fully_qualified,'E':0.6,'de':':raupe:','es':':bicho:','fr':':chenille:','ja':':毛虫:','ko':':송충이:','pt':':inseto:','it':':insetto:','fa':':حشره:','id':':serangga_kecil:','zh':':毛毛虫:','ru':':гусеница:','tr':':tırtıl:','ar':':بق:'},'\U0001F3D7\U0000FE0F':{'en':':building_construction:','status':fully_qualified,'E':0.7,'variant':True,'de':':kran:','es':':construcción:','fr':':construction_d’un_bâtiment:','ja':':建設中:','ko':':건물_공사:','pt':':construção:','it':':edificio_in_costruzione:','fa':':ساخت_وساز_ساختمانی:','id':':konstruksi_bangunan:','zh':':施工:','ru':':строика:','tr':':bina_inşaatı:','ar':':إنشاءات:'},'\U0001F3D7':{'en':':building_construction:','status':unqualified,'E':0.7,'variant':True,'de':':kran:','es':':construcción:','fr':':construction_d’un_bâtiment:','ja':':建設中:','ko':':건물_공사:','pt':':construção:','it':':edificio_in_costruzione:','fa':':ساخت_وساز_ساختمانی:','id':':konstruksi_bangunan:','zh':':施工:','ru':':стройка:','tr':':bina_inşaatı:','ar':':إنشاءات:'},'\U0001F685':{'en':':bullet_train:','status':fully_qualified,'E':0.6,'alias':[':bullettrain_front:'],'de':':hochgeschwindigkeitszug:','es':':tren_bala:','fr':':train_à_grande_vitesse:','ja':':0系新幹線:','ko':':고속철:','pt':':trem_de_alta_velocidade_japonês:','it':':treno_alta_velocità_punta_arrotondata:','fa':':قطار_تندرو:','id':':kereta_ekspres_peluru:','zh':':子弹头高速列车:','ru':':поезд_пуля:','tr':':yuvarlak_uçlu_yüksek_hızlı_tren:','ar':':قطار_سريع_بوجه_محدب:'},'\U0001F3AF':{'en':':bullseye:','status':fully_qualified,'E':0.6,'alias':[':dart:'],'de':':darts:','es':':diana:','fr':':dans_le_mille:','ja':':的:','ko':':과녁_명중:','pt':':no_alvo:','it':':bersaglio:','fa':':پیکان_در_قلب_هدف:','id':':dart:','zh':':正中靶心的飞镖:','ru':':мишень:','tr':':tam_isabet:','ar':':نشان:'},'\U0001F32F':{'en':':burrito:','status':fully_qualified,'E':1,'de':':burrito:','es':':burrito:','fr':':burrito:','ja':':ブリトー:','ko':':부리또:','pt':':burrito:','it':':burrito:','fa':':بوریتو:','id':':burrito:','zh':':墨西哥玉米煎饼:','ru':':буррито:','tr':':burito:','ar':':بوريتو:'},'\U0001F68C':{'en':':bus:','status':fully_qualified,'E':0.6,'de':':bus:','es':':autobús:','fr':':bus:','ja':':バス:','ko':':버스:','pt':':ônibus:','it':':bus:','fa':':اتوبوس:','id':':bus:','zh':':公交车:','ru':':автобус:','tr':':otobüs:','ar':':حافلة:'},'\U0001F68F':{'en':':bus_stop:','status':fully_qualified,'E':0.6,'alias':[':busstop:'],'de':':bushaltestelle:','es':':parada_de_autobús:','fr':':arrêt_de_bus:','ja':':バス停:','ko':':버스_정류장:','pt':':ponto_de_ônibus:','it':':fermata_dell’autobus:','fa':':ایستگاه_اتوبوس:','id':':halte_bus:','zh':':公交车站:','ru':':автобусная_остановка:','tr':':otobüs_durağı:','ar':':موقف_حافلات:'},'\U0001F464':{'en':':bust_in_silhouette:','status':fully_qualified,'E':0.6,'de':':silhouette_einer_büste:','es':':silueta_de_busto:','fr':':silhouette_de_buste:','ja':':人のシルエット:','ko':':사람_그림자:','pt':':silhueta_de_busto:','it':':profilo_di_persona:','fa':':تندیس_سایه_نما:','id':':siluet_foto_setengah_badan:','zh':':人像:','ru':':силуэт_человека:','tr':':siluet_büst:','ar':':صورة_ظلية:'},'\U0001F465':{'en':':busts_in_silhouette:','status':fully_qualified,'E':1,'de':':silhouette_mehrerer_büsten:','es':':dos_siluetas_de_bustos:','fr':':silhouettes_de_bustes:','ja':':2人のシルエット:','ko':':사람들_그림자:','pt':':silhueta_de_bustos:','it':':profilo_di_due_persone:','fa':':تندیس_های_سایه_نما:','id':':beberapa_siluet_foto_setengah_badan:','zh':':双人像:','ru':':два_силуэта:','tr':':siluet_büstler:','ar':':صورتان_ظليتان:'},'\U0001F9C8':{'en':':butter:','status':fully_qualified,'E':12,'de':':butter:','es':':mantequilla:','fr':':beurre:','ja':':バター:','ko':':버터:','pt':':manteiga:','it':':burro:','fa':':کره:','id':':mentega:','zh':':黄油:','ru':':масло:','tr':':tereyağı:','ar':':زبد:'},'\U0001F98B':{'en':':butterfly:','status':fully_qualified,'E':3,'de':':schmetterling:','es':':mariposa:','fr':':papillon:','ja':':チョウ:','ko':':나비:','pt':':borboleta:','it':':farfalla:','fa':':پروانه:','id':':kupu-kupu:','zh':':蝴蝶:','ru':':бабочка:','tr':':kelebek:','ar':':فراشة:'},'\U0001F335':{'en':':cactus:','status':fully_qualified,'E':0.6,'de':':kaktus:','es':':cactus:','fr':':cactus:','ja':':サボテン:','ko':':선인장:','pt':':cacto:','it':':cactus:','fa':':کاکتوس:','id':':kaktus:','zh':':仙人掌:','ru':':кактус:','tr':':kaktüs:','ar':':صبار:'},'\U0001F4C5':{'en':':calendar:','status':fully_qualified,'E':0.6,'alias':[':date:'],'de':':kalender:','es':':calendario:','fr':':calendrier:','ja':':カレンダー:','ko':':달력:','pt':':calendário:','it':':calendario:','fa':':تقویم:','id':':kalender:','zh':':日历:','ru':':календарь:','tr':':takvim:','ar':':تقويم:'},'\U0001F919':{'en':':call_me_hand:','status':fully_qualified,'E':3,'de':':ruf-mich-an-handzeichen:','es':':mano_haciendo_el_gesto_de_llamar:','fr':':signe_appel_téléphonique_avec_les_doigts:','ja':':電話の合図:','ko':':전화를_걸라는_손_모양:','pt':':sinal_me_liga:','it':':mano_con_gesto_di_chiamata:','fa':':دست_به_شکل_تلفن:','id':':panggil_saya:','zh':':给我打电话:','ru':':жест_«позвони_мне»:','tr':':beni_ara_işareti:','ar':':إشارة_للاتصال:'},'\U0001F919\U0001F3FF':{'en':':call_me_hand_dark_skin_tone:','status':fully_qualified,'E':3,'de':':ruf-mich-an-handzeichen_dunkle_hautfarbe:','es':':mano_haciendo_el_gesto_de_llamar_tono_de_piel_oscuro:','fr':':signe_appel_téléphonique_avec_les_doigts_peau_foncée:','ja':':電話の合図_濃い肌色:','ko':':전화를_걸라는_손_모양_검은색_피부:','pt':':sinal_me_liga_pele_escura:','it':':mano_con_gesto_di_chiamata_carnagione_scura:','fa':':دست_به_شکل_تلفن_پوست_آبنوسی:','id':':panggil_saya_warna_kulit_gelap:','zh':':给我打电话_较深肤色:','ru':':жест_«позвони_мне»_очень_темный_тон_кожи:','tr':':beni_ara_işareti_koyu_cilt_tonu:','ar':':إشارة_للاتصال_بشرة_بلون_غامق:'},'\U0001F919\U0001F3FB':{'en':':call_me_hand_light_skin_tone:','status':fully_qualified,'E':3,'de':':ruf-mich-an-handzeichen_helle_hautfarbe:','es':':mano_haciendo_el_gesto_de_llamar_tono_de_piel_claro:','fr':':signe_appel_téléphonique_avec_les_doigts_peau_claire:','ja':':電話の合図_薄い肌色:','ko':':전화를_걸라는_손_모양_하얀_피부:','pt':':sinal_me_liga_pele_clara:','it':':mano_con_gesto_di_chiamata_carnagione_chiara:','fa':':دست_به_شکل_تلفن_پوست_سفید:','id':':panggil_saya_warna_kulit_cerah:','zh':':给我打电话_较浅肤色:','ru':':жест_«позвони_мне»_очень_светлый_тон_кожи:','tr':':beni_ara_işareti_açık_cilt_tonu:','ar':':إشارة_للاتصال_بشرة_بلون_فاتح:'},'\U0001F919\U0001F3FE':{'en':':call_me_hand_medium-dark_skin_tone:','status':fully_qualified,'E':3,'de':':ruf-mich-an-handzeichen_mitteldunkle_hautfarbe:','es':':mano_haciendo_el_gesto_de_llamar_tono_de_piel_oscuro_medio:','fr':':signe_appel_téléphonique_avec_les_doigts_peau_mate:','ja':':電話の合図_やや濃い肌色:','ko':':전화를_걸라는_손_모양_진한_갈색_피부:','pt':':sinal_me_liga_pele_morena_escura:','it':':mano_con_gesto_di_chiamata_carnagione_abbastanza_scura:','fa':':دست_به_شکل_تلفن_پوست_گندمی:','id':':panggil_saya_warna_kulit_gelap-sedang:','zh':':给我打电话_中等深肤色:','ru':':жест_«позвони_мне»_темный_тон_кожи:','tr':':beni_ara_işareti_orta_koyu_cilt_tonu:','ar':':إشارة_للاتصال_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F919\U0001F3FC':{'en':':call_me_hand_medium-light_skin_tone:','status':fully_qualified,'E':3,'de':':ruf-mich-an-handzeichen_mittelhelle_hautfarbe:','es':':mano_haciendo_el_gesto_de_llamar_tono_de_piel_claro_medio:','fr':':signe_appel_téléphonique_avec_les_doigts_peau_moyennement_claire:','ja':':電話の合図_やや薄い肌色:','ko':':전화를_걸라는_손_모양_연한_갈색_피부:','pt':':sinal_me_liga_pele_morena_clara:','it':':mano_con_gesto_di_chiamata_carnagione_abbastanza_chiara:','fa':':دست_به_شکل_تلفن_پوست_روشن:','id':':panggil_saya_warna_kulit_cerah-sedang:','zh':':给我打电话_中等浅肤色:','ru':':жест_«позвони_мне»_светлый_тон_кожи:','tr':':beni_ara_işareti_orta_açık_cilt_tonu:','ar':':إشارة_للاتصال_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F919\U0001F3FD':{'en':':call_me_hand_medium_skin_tone:','status':fully_qualified,'E':3,'de':':ruf-mich-an-handzeichen_mittlere_hautfarbe:','es':':mano_haciendo_el_gesto_de_llamar_tono_de_piel_medio:','fr':':signe_appel_téléphonique_avec_les_doigts_peau_légèrement_mate:','ja':':電話の合図_中間の肌色:','ko':':전화를_걸라는_손_모양_갈색_피부:','pt':':sinal_me_liga_pele_morena:','it':':mano_con_gesto_di_chiamata_carnagione_olivastra:','fa':':دست_به_شکل_تلفن_پوست_طلایی:','id':':panggil_saya_warna_kulit_sedang:','zh':':给我打电话_中等肤色:','ru':':жест_«позвони_мне»_средний_тон_кожи:','tr':':beni_ara_işareti_orta_cilt_tonu:','ar':':إشارة_للاتصال_بشرة_بلون_معتدل:'},'\U0001F42A':{'en':':camel:','status':fully_qualified,'E':1,'alias':[':dromedary_camel:'],'de':':dromedar:','es':':dromedario:','fr':':dromadaire:','ja':':ラクダ:','ko':':낙타:','pt':':camelo:','it':':dromedario:','fa':':شتر:','id':':unta:','zh':':骆驼:','ru':':одногорбый_верблюд:','tr':':deve:','ar':':جمل_وحيد_السنام:'},'\U0001F4F7':{'en':':camera:','status':fully_qualified,'E':0.6,'variant':True,'de':':fotoapparat:','es':':cámara_de_fotos:','fr':':appareil_photo:','ja':':カメラ:','ko':':카메라:','pt':':câmera:','it':':fotocamera:','fa':':دوربین:','id':':kamera:','zh':':相机:','ru':':фотоаппарат:','tr':':fotoğraf_makinesi:','ar':':كاميرا:'},'\U0001F4F8':{'en':':camera_with_flash:','status':fully_qualified,'E':1,'alias':[':camera_flash:'],'de':':fotoapparat_mit_blitz:','es':':cámara_con_flash:','fr':':appareil_photo_avec_flash:','ja':':フラッシュを焚いているカメラ:','ko':':플래시를_터트리고_있는_카메라:','pt':':câmera_com_flash:','it':':fotocamera_con_flash:','fa':':دوربین_با_فلاش:','id':':kamera_dengan_flash:','zh':':开闪光灯的相机:','ru':':фотоаппарат_со_вспышкой:','tr':':flaşlı_fotoğraf_makinesi:','ar':':كاميرا_بفلاش:'},'\U0001F3D5\U0000FE0F':{'en':':camping:','status':fully_qualified,'E':0.7,'variant':True,'de':':camping:','es':':camping:','fr':':camping:','ja':':キャンプ:','ko':':캠핑:','pt':':acampamento:','it':':campeggio:','fa':':چادرزنی:','id':':berkemah:','zh':':露营:','ru':':кемпинг:','tr':':kamp:','ar':':تخييم:'},'\U0001F3D5':{'en':':camping:','status':unqualified,'E':0.7,'variant':True,'de':':camping:','es':':camping:','fr':':camping:','ja':':キャンプ:','ko':':캠핑:','pt':':acampamento:','it':':campeggio:','fa':':چادرزنی:','id':':berkemah:','zh':':露营:','ru':':кемпинг:','tr':':kamp:','ar':':تخييم:'},'\U0001F56F\U0000FE0F':{'en':':candle:','status':fully_qualified,'E':0.7,'variant':True,'de':':kerze:','es':':vela:','fr':':bougie:','ja':':ろうそく:','ko':':양초:','pt':':vela:','it':':candela:','fa':':شمع:','id':':lilin:','zh':':蜡烛:','ru':':свеча:','tr':':mum:','ar':':شمعة:'},'\U0001F56F':{'en':':candle:','status':unqualified,'E':0.7,'variant':True,'de':':kerze:','es':':vela:','fr':':bougie:','ja':':ろうそく:','ko':':양초:','pt':':vela:','it':':candela:','fa':':شمع:','id':':lilin:','zh':':蜡烛:','ru':':свеча:','tr':':mum:','ar':':شمعة:'},'\U0001F36C':{'en':':candy:','status':fully_qualified,'E':0.6,'de':':bonbon:','es':':caramelo:','fr':':bonbon:','ja':':キャンディ:','ko':':사탕:','pt':':bala:','it':':caramella:','fa':':آب_نبات_:','id':':permen:','zh':':糖:','ru':':конфета:','tr':':şekerleme:','ar':':حلوى:'},'\U0001F96B':{'en':':canned_food:','status':fully_qualified,'E':5,'de':':konserve:','es':':comida_enlatada:','fr':':aliments_en_conserve:','ja':':缶詰:','ko':':통조림:','pt':':comida_enlatada:','it':':cibo_in_scatola:','fa':':غذای_کنسروشده:','id':':makanan_kaleng:','zh':':罐头食品:','ru':':консервы:','tr':':konserve_gıda:','ar':':طعام_معلب:'},'\U0001F6F6':{'en':':canoe:','status':fully_qualified,'E':3,'de':':kanu:','es':':canoa:','fr':':canoë:','ja':':カヌー:','ko':':카누:','pt':':canoa:','it':':canoa:','fa':':بلم:','id':':kano:','zh':':独木舟:','ru':':каноэ:','tr':':kano:','ar':':زورق:'},'\U0001F5C3\U0000FE0F':{'en':':card_file_box:','status':fully_qualified,'E':0.7,'variant':True,'de':':karteikasten:','es':':archivador_de_tarjetas:','fr':':boîte_à_dossiers:','ja':':カードファイルボックス:','ko':':카드_파일_상자:','pt':':caixa_de_arquivos:','it':':schedario_da_tavolo:','fa':':جعبه_نگه_داری_کارت:','id':':kotak_file_kartu:','zh':':卡片盒:','ru':':картотека:','tr':':dosya_kutusu:','ar':':صندوق_بطاقات:'},'\U0001F5C3':{'en':':card_file_box:','status':unqualified,'E':0.7,'variant':True,'de':':karteikasten:','es':':archivador_de_tarjetas:','fr':':boîte_à_dossiers:','ja':':カードファイルボックス:','ko':':카드_파일_상자:','pt':':caixa_de_arquivos:','it':':schedario_da_tavolo:','fa':':جعبه_نگه_داری_کارت:','id':':kotak_file_kartu:','zh':':卡片盒:','ru':':картотека:','tr':':dosya_kutusu:','ar':':صندوق_بطاقات:'},'\U0001F4C7':{'en':':card_index:','status':fully_qualified,'E':0.6,'de':':rotationskartei:','es':':organizador_de_fichas:','fr':':carnet_d’adresses:','ja':':カードインデックス:','ko':':카드_인덱스:','pt':':índice_de_cartões:','it':':schedario:','fa':':کارت_های_راهنما:','id':':indeks_kartu:','zh':':卡片索引:','ru':':органайзер:','tr':':kart_dizini:','ar':':فهرس_بطاقات:'},'\U0001F5C2\U0000FE0F':{'en':':card_index_dividers:','status':fully_qualified,'E':0.7,'variant':True,'de':':karteibahnen:','es':':separador_de_fichas:','fr':':intercalaires:','ja':':カードフォルダー:','ko':':카드_색인_파일:','pt':':divisores_de_pastas:','it':':divisori_per_schedario:','fa':':جداکننده:','id':':pembagi_indeks_kartu:','zh':':索引分隔文件夹:','ru':':маркированные_папки:','tr':':dosya_ayırıcı:','ar':':حاجز_بطاقات:'},'\U0001F5C2':{'en':':card_index_dividers:','status':unqualified,'E':0.7,'variant':True,'de':':karteireiter:','es':':separador_de_fichas:','fr':':intercalaires:','ja':':カードフォルダー:','ko':':카드_색인_파일:','pt':':divisores_de_pastas:','it':':divisori_per_schedario:','fa':':جداکننده:','id':':pembagi_indeks_kartu:','zh':':索引分隔文件夹:','ru':':маркированные_папки:','tr':':dosya_ayırıcı:','ar':':حاجز_بطاقات:'},'\U0001F3A0':{'en':':carousel_horse:','status':fully_qualified,'E':0.6,'de':':karussellpferd:','es':':caballo_de_tiovivo:','fr':':cheval_de_manège:','ja':':メリーゴーランド:','ko':':회전_목마:','pt':':carrossel:','it':':cavallo_da_giostra:','fa':':چرخ_وفلک_اسبی:','id':':komidi_putar:','zh':':旋转木马:','ru':':лошадь_на_карусели:','tr':':atlıkarınca:','ar':':حصان_الدوامة:'},'\U0001F38F':{'en':':carp_streamer:','status':fully_qualified,'E':0.6,'alias':[':flags:'],'de':':traditionelle_japanische_windsäcke:','es':':banderín_de_carpas:','fr':':koinobori:','ja':':こいのぼり:','ko':':물고기_깃발:','pt':':bandeira_de_carpas:','it':':aquilone_a_forma_di_carpa:','fa':':پرچم_های_ماهی_در_باد:','id':':bendera_ikan_koi:','zh':':鲤鱼旗:','ru':':вымпелы_в_виде_карпов:','tr':':sazan_şekilli_flama:','ar':':قصاصات_ملونة:'},'\U0001FA9A':{'en':':carpentry_saw:','status':fully_qualified,'E':13,'de':':handsäge:','es':':sierra_de_carpintería:','fr':':scie:','ja':':のこぎり:','ko':':목공_톱:','pt':':serrote:','it':':sega_da_falegname:','fa':':اره_نجاری:','id':':gergaji_kayu:','zh':':木工锯:','ru':':пила_по_дереву:','tr':':testere:','ar':':منشار_يدوي:'},'\U0001F955':{'en':':carrot:','status':fully_qualified,'E':3,'de':':karotte:','es':':zanahoria:','fr':':carotte:','ja':':人参:','ko':':당근:','pt':':cenoura:','it':':carota:','fa':':هویج:','id':':wortel:','zh':':胡萝卜:','ru':':морковь:','tr':':havuç:','ar':':جزر:'},'\U0001F3F0':{'en':':castle:','status':fully_qualified,'E':0.6,'alias':[':european_castle:'],'de':':schloss:','es':':castillo_europeo:','fr':':château:','ja':':西洋の城:','ko':':유럽_성:','pt':':castelo:','it':':castello:','fa':':قلعه:','id':':kastel:','zh':':欧洲城堡:','ru':':замок:','tr':':şato:','ar':':قلعة:'},'\U0001F408':{'en':':cat:','status':fully_qualified,'E':0.7,'alias':[':cat2:'],'variant':True,'de':':katze:','es':':gato:','fr':':chat:','ja':':ネコ:','ko':':고양이:','pt':':gato:','it':':gatto:','fa':':گربه:','id':':kucing:','zh':':猫:','ru':':кошка:','tr':':kedi:','ar':':قطة:'},'\U0001F431':{'en':':cat_face:','status':fully_qualified,'E':0.6,'alias':[':cat:'],'de':':katzengesicht:','es':':cara_de_gato:','fr':':tête_de_chat:','ja':':ネコの顔:','ko':':고양이_얼굴:','pt':':rosto_de_gato:','it':':muso_di_gatto:','fa':':صورت_گربه:','id':':wajah_kucing:','zh':':猫脸:','ru':':морда_кошки:','tr':':kedi_yüzü:','ar':':وجه_قطة:'},'\U0001F639':{'en':':cat_with_tears_of_joy:','status':fully_qualified,'E':0.6,'alias':[':joy_cat:'],'de':':katze_mit_freudentränen:','es':':gato_llorando_de_risa:','fr':':chat_qui_pleure_de_joie:','ja':':嬉し泣きする猫:','ko':':기쁨의_눈물을_흘리는_고양이_얼굴:','pt':':rosto_de_gato_com_lágrimas_de_alegria:','it':':gatto_con_lacrime_di_gioia:','fa':':گربهٔ_خندان_با_اشک_شوق:','id':':wajah_kucing_gembira_berurai_air_mata:','zh':':笑出眼泪的猫:','ru':':смеющийся_до_слез_кот:','tr':':sevinç_gözyaşları_döken_kedi:','ar':':وجه_قطة_بدموع_فرح:'},'\U0001F63C':{'en':':cat_with_wry_smile:','status':fully_qualified,'E':0.6,'alias':[':smirk_cat:'],'de':':verwegen_lächelnde_katze:','es':':gato_haciendo_una_mueca:','fr':':chat_avec_sourire_en_coin:','ja':':にやりとする猫:','ko':':썩소_짓는_고양이:','pt':':rosto_de_gato_com_sorriso_irônico:','it':':gatto_con_sorriso_sarcastico:','fa':':گربه_با_پوزخند:','id':':wajah_kucing_tersenyum_sombong:','zh':':奸笑的猫:','ru':':ухмыляющийся_кот:','tr':':iğneleyici_kedi_yüzü:','ar':':وجه_قطة_بابتسامة_ممتعضة:'},'\U000026D3\U0000FE0F':{'en':':chains:','status':fully_qualified,'E':0.7,'variant':True,'de':':ketten:','es':':cadenas:','fr':':chaînes:','ja':':鎖:','ko':':쇠사슬:','pt':':correntes:','it':':catene:','fa':':غل_و_زنجیر:','id':':rantai:','zh':':链条:','ru':':цепь:','tr':':zincir:','ar':':سلاسل:'},'\U000026D3':{'en':':chains:','status':unqualified,'E':0.7,'variant':True,'de':':ketten:','es':':cadenas:','fr':':chaînes:','ja':':鎖:','ko':':쇠사슬:','pt':':correntes:','it':':catene:','fa':':غل_و_زنجیر:','id':':rantai:','zh':':链条:','ru':':цепь:','tr':':zincir:','ar':':سلاسل:'},'\U0001FA91':{'en':':chair:','status':fully_qualified,'E':12,'de':':stuhl:','es':':silla:','fr':':chaise:','ja':':椅子:','ko':':의자:','pt':':cadeira:','it':':sedia:','fa':':صندلی:','id':':bangku:','zh':':椅子:','ru':':стул:','tr':':sandalye:','ar':':كرسي:'},'\U0001F4C9':{'en':':chart_decreasing:','status':fully_qualified,'E':0.6,'alias':[':chart_with_downwards_trend:'],'de':':abwärtstrend:','es':':gráfica_de_evolución_descendente:','fr':':graphique_en_baisse:','ja':':グラフ下向き:','ko':':내려가는_그래프:','pt':':gráfico_caindo:','it':':grafico_con_andamento_negativo:','fa':':نمودار_نزولی:','id':':bagan_menurun:','zh':':趋势向下的图表:','ru':':диаграмма_падения:','tr':':iniş_grafiği:','ar':':مخطط_بياني_لأسفل:'},'\U0001F4C8':{'en':':chart_increasing:','status':fully_qualified,'E':0.6,'alias':[':chart_with_upwards_trend:'],'de':':aufwärtstrend:','es':':gráfica_de_evolución_ascendente:','fr':':graphique_en_hausse:','ja':':グラフ上向き:','ko':':상승하는_그래프:','pt':':gráfico_subindo:','it':':grafico_con_andamento_positivo:','fa':':نمودار_صعودی:','id':':bagan_meningkat:','zh':':趋势向上的图表:','ru':':диаграмма_роста:','tr':':artış_grafiği:','ar':':اتجاه_لأعلى:'},'\U0001F4B9':{'en':':chart_increasing_with_yen:','status':fully_qualified,'E':0.6,'alias':[':chart:'],'de':':steigende_kurve_mit_yen-zeichen:','es':':mercado_al_alza:','fr':':courbe_avec_yen_en_hausse:','ja':':為替:','ko':':엔화_관련_상승하는_그래프:','pt':':gráfico_subindo_com_iene:','it':':grafico_ascendente:','fa':':نمودار_صعودی_با_ین:','id':':bagan_meningkat_dengan_yen:','zh':':趋势向上且带有日元符号的图表:','ru':':биржа:','tr':':yen_simgeli_artış_grafiği:','ar':':مخطط_لبيانات_متزايدة_مع_الين:'},'\U00002611\U0000FE0F':{'en':':check_box_with_check:','status':fully_qualified,'E':0.6,'alias':[':ballot_box_with_check:'],'variant':True,'de':':abstimmungsfeld_mit_häkchen:','es':':casilla_con_marca_de_verificación:','fr':':case_cochée:','ja':':チェックボックス:','ko':':체크박스:','pt':':caixa_de_seleção_marcada_com_tique:','it':':riquadro_con_spunta:','fa':':مربع_با_علامت_تیک:','id':':kotak_bercentang:','zh':':勾选框:','ru':':галочка_в_квадрате:','tr':':seçilmiş_onay_kutusu:','ar':':مربع_اقتراع_محدد:'},'\U00002611':{'en':':check_box_with_check:','status':unqualified,'E':0.6,'alias':[':ballot_box_with_check:'],'variant':True,'de':':abgehaktes_kästchen:','es':':casilla_con_marca_de_verificación:','fr':':case_cochée:','ja':':チェックボックス:','ko':':체크박스:','pt':':caixa_de_seleção_marcada_com_tique:','it':':riquadro_con_spunta:','fa':':مربع_با_علامت_تیک:','id':':kotak_bercentang:','zh':':勾选框:','ru':':галочка_в_квадрате:','tr':':seçilmiş_onay_kutusu:','ar':':مربع_اقتراع_محدد:'},'\U00002714\U0000FE0F':{'en':':check_mark:','status':fully_qualified,'E':0.6,'alias':[':heavy_check_mark:'],'variant':True,'de':':kräftiges_häkchen:','es':':marca_de_verificación:','fr':':coche:','ja':':太字のチェック:','ko':':진한_체크_표시:','pt':':marca_de_seleção:','it':':segno_di_spunta_nero:','fa':':علامت_تیک:','id':':tanda_centang_tebal:','zh':':勾号:','ru':':галочка:','tr':':onay_işareti:','ar':':علامة_صح:'},'\U00002714':{'en':':check_mark:','status':unqualified,'E':0.6,'alias':[':heavy_check_mark:'],'variant':True,'de':':kräftiges_häkchen:','es':':marca_de_verificación:','fr':':coche:','ja':':太字のチェック:','ko':':진한_체크_표시:','pt':':marca_de_seleção:','it':':segno_di_spunta:','fa':':علامت_تیک:','id':':tanda_centang_tebal:','zh':':勾号:','ru':':галочка:','tr':':onay_işareti:','ar':':علامة_صح:'},'\U00002705':{'en':':check_mark_button:','status':fully_qualified,'E':0.6,'alias':[':white_check_mark:'],'variant':True,'de':':weißes_häkchen:','es':':botón_de_marca_de_verificación:','fr':':bouton_coché:','ja':':白抜きのチェック:','ko':':체크_표시:','pt':':marca_de_seleção_branca:','it':':pulsante_segno_di_spunta:','fa':':علامت_تأیید:','id':':tanda_centang_putih_tebal:','zh':':勾号按钮:','ru':':белая_галочка:','tr':':onay_işareti_düğmesi:','ar':':علامة_تحديد_بيضاء:'},'\U0001F9C0':{'en':':cheese_wedge:','status':fully_qualified,'E':1,'alias':[':cheese:'],'de':':käsestück:','es':':cuña_de_queso:','fr':':part_de_fromage:','ja':':チーズ:','ko':':치즈_조각:','pt':':queijo:','it':':fetta_di_formaggio:','fa':':قاچ_پنیر:','id':':irisan_keju:','zh':':芝士:','ru':':сыр:','tr':':peynir_dilimi:','ar':':قطعة_جبن:'},'\U0001F3C1':{'en':':chequered_flag:','status':fully_qualified,'E':0.6,'alias':[':checkered_flag:'],'de':':zielflagge:','es':':bandera_de_cuadros:','fr':':drapeau_à_damier:','ja':':チェッカーフラッグ:','ko':':격자_무늬_깃발:','pt':':bandeira_quadriculada:','it':':bandiera_a_scacchi:','fa':':پرچم_شطرنجی:','id':':bendera_kotak-kotak:','zh':':终点旗:','ru':':клетчатый_флаг:','tr':':damalı_bayrak:','ar':':علم_مربعات:'},'\U0001F352':{'en':':cherries:','status':fully_qualified,'E':0.6,'de':':kirschen:','es':':cerezas:','fr':':cerises:','ja':':さくらんぼ:','ko':':체리:','pt':':cereja:','it':':ciliegie:','fa':':گیلاس:','id':':ceri:','zh':':樱桃:','ru':':вишня:','tr':':kiraz:','ar':':كرز:'},'\U0001F338':{'en':':cherry_blossom:','status':fully_qualified,'E':0.6,'de':':kirschblüte:','es':':flor_de_cerezo:','fr':':fleur_de_cerisier:','ja':':桜:','ko':':벚꽃:','pt':':flor_de_cerejeira:','it':':fiore_di_ciliegio:','fa':':شکوفه_گیلاس:','id':':sakura:','zh':':樱花:','ru':':цветок_вишни:','tr':':kiraz_çiçeği:','ar':':زهرة_الكرز:'},'\U0000265F\U0000FE0F':{'en':':chess_pawn:','status':fully_qualified,'E':11,'variant':True,'de':':bauer_schach:','es':':peón_de_ajedrez:','fr':':pion_d’échec:','ja':':チェスの駒:','ko':':체스_폰:','pt':':peão_de_xadrez:','it':':pedina_degli_scacchi:','fa':':پیاده_شطرنج:','id':':pion_catur:','zh':':兵:','ru':':пешка:','tr':':piyon:','ar':':بيدق:'},'\U0000265F':{'en':':chess_pawn:','status':unqualified,'E':11,'variant':True,'de':':bauer_schach:','es':':peón_de_ajedrez:','fr':':pion_d’échec:','ja':':チェスの駒:','ko':':체스_폰:','pt':':peão_de_xadrez:','it':':pedina_degli_scacchi:','fa':':پیاده_شطرنج:','id':':pion_catur:','zh':':兵:','ru':':пешка:','tr':':piyon:','ar':':بيدق:'},'\U0001F330':{'en':':chestnut:','status':fully_qualified,'E':0.6,'de':':kastanie:','es':':castaña:','fr':':châtaigne:','ja':':くり:','ko':':밤:','pt':':castanha:','it':':castagna:','fa':':فندق:','id':':kastanye:','zh':':栗子:','ru':':каштан:','tr':':kestane:','ar':':كستناء:'},'\U0001F414':{'en':':chicken:','status':fully_qualified,'E':0.6,'de':':huhn:','es':':gallina:','fr':':poule:','ja':':にわとり:','ko':':닭:','pt':':galinha:','it':':gallina:','fa':':مرغ:','id':':ayam:','zh':':鸡:','ru':':курица:','tr':':tavuk:','ar':':دجاجة:'},'\U0001F9D2':{'en':':child:','status':fully_qualified,'E':5,'de':':kind:','es':':infante:','fr':':enfant:','ja':':子供:','ko':':어린이:','pt':':criança:','it':':bimbo:','fa':':بچه:','id':':anak:','zh':':儿童:','ru':':ребенок:','tr':':çocuk:','ar':':طفل:'},'\U0001F9D2\U0001F3FF':{'en':':child_dark_skin_tone:','status':fully_qualified,'E':5,'de':':kind_dunkle_hautfarbe:','es':':infante_tono_de_piel_oscuro:','fr':':enfant_peau_foncée:','ja':':子供_濃い肌色:','ko':':어린이_검은색_피부:','pt':':criança_pele_escura:','it':':bimbo_carnagione_scura:','fa':':بچه_پوست_آبنوسی:','id':':anak_warna_kulit_gelap:','zh':':儿童_较深肤色:','ru':':ребенок_очень_темный_тон_кожи:','tr':':çocuk_koyu_cilt_tonu:','ar':':طفل_بشرة_بلون_غامق:'},'\U0001F9D2\U0001F3FB':{'en':':child_light_skin_tone:','status':fully_qualified,'E':5,'de':':kind_helle_hautfarbe:','es':':infante_tono_de_piel_claro:','fr':':enfant_peau_claire:','ja':':子供_薄い肌色:','ko':':어린이_하얀_피부:','pt':':criança_pele_clara:','it':':bimbo_carnagione_chiara:','fa':':بچه_پوست_سفید:','id':':anak_warna_kulit_cerah:','zh':':儿童_较浅肤色:','ru':':ребенок_очень_светлый_тон_кожи:','tr':':çocuk_açık_cilt_tonu:','ar':':طفل_بشرة_بلون_فاتح:'},'\U0001F9D2\U0001F3FE':{'en':':child_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':kind_mitteldunkle_hautfarbe:','es':':infante_tono_de_piel_oscuro_medio:','fr':':enfant_peau_mate:','ja':':子供_やや濃い肌色:','ko':':어린이_진한_갈색_피부:','pt':':criança_pele_morena_escura:','it':':bimbo_carnagione_abbastanza_scura:','fa':':بچه_پوست_گندمی:','id':':anak_warna_kulit_gelap-sedang:','zh':':儿童_中等深肤色:','ru':':ребенок_темный_тон_кожи:','tr':':çocuk_orta_koyu_cilt_tonu:','ar':':طفل_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D2\U0001F3FC':{'en':':child_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':kind_mittelhelle_hautfarbe:','es':':infante_tono_de_piel_claro_medio:','fr':':enfant_peau_moyennement_claire:','ja':':子供_やや薄い肌色:','ko':':어린이_연한_갈색_피부:','pt':':criança_pele_morena_clara:','it':':bimbo_carnagione_abbastanza_chiara:','fa':':بچه_پوست_روشن:','id':':anak_warna_kulit_cerah-sedang:','zh':':儿童_中等浅肤色:','ru':':ребенок_светлый_тон_кожи:','tr':':çocuk_orta_açık_cilt_tonu:','ar':':طفل_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D2\U0001F3FD':{'en':':child_medium_skin_tone:','status':fully_qualified,'E':5,'de':':kind_mittlere_hautfarbe:','es':':infante_tono_de_piel_medio:','fr':':enfant_peau_légèrement_mate:','ja':':子供_中間の肌色:','ko':':어린이_갈색_피부:','pt':':criança_pele_morena:','it':':bimbo_carnagione_olivastra:','fa':':بچه_پوست_طلایی:','id':':anak_warna_kulit_sedang:','zh':':儿童_中等肤色:','ru':':ребенок_средний_тон_кожи:','tr':':çocuk_orta_cilt_tonu:','ar':':طفل_بشرة_بلون_معتدل:'},'\U0001F6B8':{'en':':children_crossing:','status':fully_qualified,'E':1,'de':':kinder_überqueren_die_straße:','es':':niños_cruzando:','fr':':traversée_d’enfants:','ja':':児童横断:','ko':':어린이_보호_구역:','pt':':crianças_atravessando:','it':':attraversamento_bambini:','fa':':عبور_کودکان:','id':':anak-anak_menyeberang:','zh':':儿童过街:','ru':':знак_«дети»:','tr':':karşıya_geçen_çocuklar:','ar':':عبور_أطفال:'},'\U0001F43F\U0000FE0F':{'en':':chipmunk:','status':fully_qualified,'E':0.7,'variant':True,'de':':streifenhörnchen:','es':':ardilla:','fr':':écureuil:','ja':':リス:','ko':':얼룩다람쥐:','pt':':esquilo:','it':':scoiattolo:','fa':':سنجاب_راه_راه:','id':':tupai:','zh':':松鼠:','ru':':бурундук:','tr':':sincap:','ar':':سنجاب:'},'\U0001F43F':{'en':':chipmunk:','status':unqualified,'E':0.7,'variant':True,'de':':streifenhörnchen:','es':':ardilla:','fr':':écureuil:','ja':':リス:','ko':':얼룩다람쥐:','pt':':esquilo:','it':':scoiattolo:','fa':':سنجاب_راه_راه:','id':':tupai:','zh':':松鼠:','ru':':бурундук:','tr':':sincap:','ar':':سنجاب:'},'\U0001F36B':{'en':':chocolate_bar:','status':fully_qualified,'E':0.6,'de':':schokoladentafel:','es':':tableta_de_chocolate:','fr':':barre_chocolatée:','ja':':チョコレート:','ko':':초콜렛:','pt':':chocolate:','it':':cioccolato:','fa':':تخته_شکلات:','id':':sebatang_cokelat:','zh':':巧克力:','ru':':шоколад:','tr':':çikolata:','ar':':لوح_شوكولاتة:'},'\U0001F962':{'en':':chopsticks:','status':fully_qualified,'E':5,'de':':essstäbchen:','es':':palillos:','fr':':baguettes:','ja':':はし:','ko':':젓가락:','pt':':hashi:','it':':bacchette:','fa':':چاپ_استیک:','id':':sumpit:','zh':':筷子:','ru':':палочки_для_еды:','tr':':yemek_çubukları:','ar':':عيدان_للأكل:'},'\U000026EA':{'en':':church:','status':fully_qualified,'E':0.6,'variant':True,'de':':kirche:','es':':iglesia:','fr':':église:','ja':':教会:','ko':':교회:','pt':':igreja:','it':':chiesa:','fa':':کلیسا:','id':':gereja:','zh':':教堂:','ru':':церковь:','tr':':kilise:','ar':':كنيسة:'},'\U0001F6AC':{'en':':cigarette:','status':fully_qualified,'E':0.6,'alias':[':smoking:'],'de':':zigarette:','es':':cigarrillo:','fr':':cigarette:','ja':':煙草:','ko':':흡연_구역_신호:','pt':':cigarro:','it':':sigaretta:','fa':':سیگار:','id':':rokok:','zh':':香烟:','ru':':сигарета:','tr':':sigara:','ar':':سيجارة:'},'\U0001F3A6':{'en':':cinema:','status':fully_qualified,'E':0.6,'de':':kinosymbol:','es':':cine:','fr':':cinéma:','ja':':映画:','ko':':영화:','pt':':cinema:','it':':simbolo_del_cinema:','fa':':سینما:','id':':film:','zh':':电影院:','ru':':видеозапись:','tr':':sinema:','ar':':سينما:'},'\U000024C2\U0000FE0F':{'en':':circled_M:','status':fully_qualified,'E':0.6,'alias':[':m:', ':circled_m:'],'variant':True,'de':':buchstabe_m_in_kreis:','es':':m_en_círculo:','fr':':m_encerclé:','ja':':丸いmマーク:','ko':':원글자_엠:','pt':':círculo_com_a_letra_m:','it':':pulsante_m_cerchiata:','fa':':حرف_ام_در_دایره:','id':':huruf_m_dalam_lingkaran:','zh':':圆圈包围的M:','ru':':буква_«м»_в_кружке:','tr':':daire_içinde_m_harfi:','ar':':حرف_إم:'},'\U000024C2':{'en':':circled_M:','status':unqualified,'E':0.6,'alias':[':m:', ':circled_m:'],'variant':True,'de':':buchstabe_m_in_kreis:','es':':m_en_círculo:','fr':':m_encerclé:','ja':':丸いmマーク:','ko':':원글자_엠:','pt':':círculo_com_a_letra_m:','it':':pulsante_m_cerchiata:','fa':':حرف_ام_در_دایره:','id':':huruf_m_dalam_lingkaran:','zh':':圆圈包围的M:','ru':':буква_«м»_в_кружке:','tr':':daire_içinde_m_harfi:','ar':':حرف_إم:'},'\U0001F3AA':{'en':':circus_tent:','status':fully_qualified,'E':0.6,'de':':zirkuszelt:','es':':carpa_de_circo:','fr':':chapiteau:','ja':':サーカス:','ko':':서커스:','pt':':circo:','it':':circo:','fa':':چادر_سیرک:','id':':tenda_sirkus:','zh':':马戏团帐篷:','ru':':цирковой_шатер:','tr':':sirk_çadırı:','ar':':خيمة_سيرك:'},'\U0001F3D9\U0000FE0F':{'en':':cityscape:','status':fully_qualified,'E':0.7,'variant':True,'de':':skyline:','es':':paisaje_urbano:','fr':':ville:','ja':':高層ビル:','ko':':도시_전경:','pt':':cidade:','it':':paesaggio_urbano:','fa':':نمای_شهر:','id':':panorama_kota:','zh':':城市风光:','ru':':ночнои_город:','tr':':şehir_manzarası:','ar':':ناطحات_سحاب:'},'\U0001F3D9':{'en':':cityscape:','status':unqualified,'E':0.7,'variant':True,'de':':skyline:','es':':paisaje_urbano:','fr':':ville:','ja':':高層ビル:','ko':':도시_전경:','pt':':cidade:','it':':paesaggio_urbano:','fa':':نمای_شهر:','id':':panorama_kota:','zh':':城市风光:','ru':':ночной_город:','tr':':şehir_manzarası:','ar':':ناطحات_سحاب:'},'\U0001F306':{'en':':cityscape_at_dusk:','status':fully_qualified,'E':0.6,'alias':[':city_sunset:'],'de':':abendstimmung_in_der_stadt:','es':':ciudad_al_atardecer:','fr':':ville_au_crépuscule:','ja':':夕暮れの都会:','ko':':도시_야경:','pt':':cidade_ao_anoitecer:','it':':città_al_tramonto:','fa':':نمای_شهر_هنگام_غروب:','id':':panorama_kota_di_sore_hari:','zh':':城市黄昏:','ru':':закат_в_городе:','tr':':şehirde_alacakaranlık:','ar':':غروب_مدينة:'},'\U0001F5DC\U0000FE0F':{'en':':clamp:','status':fully_qualified,'E':0.7,'alias':[':compression:'],'variant':True,'de':':schraubzwinge:','es':':tornillo_de_banco:','fr':':serre-joint:','ja':':万力:','ko':':압축기:','pt':':braçadeira:','it':':morsetto:','fa':':پرس:','id':':klem:','zh':':夹钳:','ru':':компрессор:','tr':':mengene:','ar':':آلة_ضغط:'},'\U0001F5DC':{'en':':clamp:','status':unqualified,'E':0.7,'alias':[':compression:'],'variant':True,'de':':schraubzwinge:','es':':tornillo_de_banco:','fr':':serre-joint:','ja':':万力:','ko':':압축기:','pt':':braçadeira:','it':':morsetto:','fa':':پرس:','id':':klem:','zh':':夹钳:','ru':':компрессор:','tr':':mengene:','ar':':آلة_ضغط:'},'\U0001F3AC':{'en':':clapper_board:','status':fully_qualified,'E':0.6,'alias':[':clapper:'],'variant':True,'de':':filmklappe:','es':':claqueta:','fr':':clap:','ja':':カチンコ:','ko':':슬레이트:','pt':':claquete:','it':':ciak:','fa':':کلاکت:','id':':papan_sutradara:','zh':':场记板:','ru':':хлопушка_нумератор:','tr':':film_tahtası:','ar':':كلاكيت:'},'\U0001F44F':{'en':':clapping_hands:','status':fully_qualified,'E':0.6,'alias':[':clap:'],'de':':klatschende_hände:','es':':manos_aplaudiendo:','fr':':applaudissements:','ja':':拍手:','ko':':손뼉:','pt':':mãos_aplaudindo:','it':':mani_che_applaudono:','fa':':دست_زدن:','id':':tepuk_tangan:','zh':':鼓掌:','ru':':аплодисменты:','tr':':el_çırpma:','ar':':يدان_تصفقان:'},'\U0001F44F\U0001F3FF':{'en':':clapping_hands_dark_skin_tone:','status':fully_qualified,'E':1,'de':':klatschende_hände_dunkle_hautfarbe:','es':':manos_aplaudiendo_tono_de_piel_oscuro:','fr':':applaudissements_peau_foncée:','ja':':拍手_濃い肌色:','ko':':손뼉_검은색_피부:','pt':':mãos_aplaudindo_pele_escura:','it':':mani_che_applaudono_carnagione_scura:','fa':':دست_زدن_پوست_آبنوسی:','id':':tepuk_tangan_warna_kulit_gelap:','zh':':鼓掌_较深肤色:','ru':':аплодисменты_очень_темный_тон_кожи:','tr':':el_çırpma_koyu_cilt_tonu:','ar':':يدان_تصفقان_بشرة_بلون_غامق:'},'\U0001F44F\U0001F3FB':{'en':':clapping_hands_light_skin_tone:','status':fully_qualified,'E':1,'de':':klatschende_hände_helle_hautfarbe:','es':':manos_aplaudiendo_tono_de_piel_claro:','fr':':applaudissements_peau_claire:','ja':':拍手_薄い肌色:','ko':':손뼉_하얀_피부:','pt':':mãos_aplaudindo_pele_clara:','it':':mani_che_applaudono_carnagione_chiara:','fa':':دست_زدن_پوست_سفید:','id':':tepuk_tangan_warna_kulit_cerah:','zh':':鼓掌_较浅肤色:','ru':':аплодисменты_очень_светлый_тон_кожи:','tr':':el_çırpma_açık_cilt_tonu:','ar':':يدان_تصفقان_بشرة_بلون_فاتح:'},'\U0001F44F\U0001F3FE':{'en':':clapping_hands_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':klatschende_hände_mitteldunkle_hautfarbe:','es':':manos_aplaudiendo_tono_de_piel_oscuro_medio:','fr':':applaudissements_peau_mate:','ja':':拍手_やや濃い肌色:','ko':':손뼉_진한_갈색_피부:','pt':':mãos_aplaudindo_pele_morena_escura:','it':':mani_che_applaudono_carnagione_abbastanza_scura:','fa':':دست_زدن_پوست_گندمی:','id':':tepuk_tangan_warna_kulit_gelap-sedang:','zh':':鼓掌_中等深肤色:','ru':':аплодисменты_темный_тон_кожи:','tr':':el_çırpma_orta_koyu_cilt_tonu:','ar':':يدان_تصفقان_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F44F\U0001F3FC':{'en':':clapping_hands_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':klatschende_hände_mittelhelle_hautfarbe:','es':':manos_aplaudiendo_tono_de_piel_claro_medio:','fr':':applaudissements_peau_moyennement_claire:','ja':':拍手_やや薄い肌色:','ko':':손뼉_연한_갈색_피부:','pt':':mãos_aplaudindo_pele_morena_clara:','it':':mani_che_applaudono_carnagione_abbastanza_chiara:','fa':':دست_زدن_پوست_روشن:','id':':tepuk_tangan_warna_kulit_cerah-sedang:','zh':':鼓掌_中等浅肤色:','ru':':аплодисменты_светлый_тон_кожи:','tr':':el_çırpma_orta_açık_cilt_tonu:','ar':':يدان_تصفقان_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F44F\U0001F3FD':{'en':':clapping_hands_medium_skin_tone:','status':fully_qualified,'E':1,'de':':klatschende_hände_mittlere_hautfarbe:','es':':manos_aplaudiendo_tono_de_piel_medio:','fr':':applaudissements_peau_légèrement_mate:','ja':':拍手_中間の肌色:','ko':':손뼉_갈색_피부:','pt':':mãos_aplaudindo_pele_morena:','it':':mani_che_applaudono_carnagione_olivastra:','fa':':دست_زدن_پوست_طلایی:','id':':tepuk_tangan_warna_kulit_sedang:','zh':':鼓掌_中等肤色:','ru':':аплодисменты_средний_тон_кожи:','tr':':el_çırpma_orta_cilt_tonu:','ar':':يدان_تصفقان_بشرة_بلون_معتدل:'},'\U0001F3DB\U0000FE0F':{'en':':classical_building:','status':fully_qualified,'E':0.7,'variant':True,'de':':antikes_gebäude:','es':':edificio_clásico:','fr':':monument_classique:','ja':':歴史的な建物:','ko':':고전_양식의_건축물:','pt':':prédio_grego:','it':':edificio_classico:','fa':':ساختمان_باستانی:','id':':bangunan_klasik:','zh':':古典建筑:','ru':':античное_здание:','tr':':tarihi_yapı:','ar':':مبنى_أثري:'},'\U0001F3DB':{'en':':classical_building:','status':unqualified,'E':0.7,'variant':True,'de':':antikes_gebäude:','es':':edificio_clásico:','fr':':monument_classique:','ja':':歴史的な建物:','ko':':고전_양식의_건축물:','pt':':prédio_grego:','it':':edificio_classico:','fa':':ساختمان_باستانی:','id':':bangunan_klasik:','zh':':古典建筑:','ru':':античное_здание:','tr':':tarihi_yapı:','ar':':مبنى_أثري:'},'\U0001F37B':{'en':':clinking_beer_mugs:','status':fully_qualified,'E':0.6,'alias':[':beers:'],'de':':bierkrüge:','es':':jarras_de_cerveza_brindando:','fr':':chopes_de_bière:','ja':':ビールで乾杯:','ko':':맥주:','pt':':canecas_de_cerveja:','it':':boccali_di_birra:','fa':':لیوان_های_آبجو_را_به_هم_زدن:','id':':mug_bir_berdenting:','zh':':干杯:','ru':':чокающиеся_кружки:','tr':':bira_kupaları_tokuşturma:','ar':':كأسا_بيرة:'},'\U0001F942':{'en':':clinking_glasses:','status':fully_qualified,'E':3,'de':':sektgläser:','es':':copas_brindando:','fr':':trinquer:','ja':':グラスで乾杯:','ko':':건배하는_샴페인_잔:','pt':':taças_brindando:','it':':brindisi:','fa':':صدای_به_هم_خوردن_پیاله_ها:','id':':mendentingkan_gelas:','zh':':碰杯:','ru':':чокающиеся_бокалы:','tr':':kadeh_tokuşturma:','ar':':كأسان_يرتطمان:'},'\U0001F4CB':{'en':':clipboard:','status':fully_qualified,'E':0.6,'variant':True,'de':':klemmbrett:','es':':portapapeles:','fr':':porte-bloc:','ja':':クリップボード:','ko':':클립보드:','pt':':prancheta:','it':':portablocco:','fa':':تختهٔ_گیره_دار:','id':':papan_klip:','zh':':剪贴板:','ru':':планшет_с_зажимом:','tr':':pano:','ar':':لوح_كتابة_مشبكي:'},'\U0001F503':{'en':':clockwise_vertical_arrows:','status':fully_qualified,'E':0.6,'alias':[':arrows_clockwise:'],'de':':kreisförmige_pfeile_im_uhrzeigersinn:','es':':flechas_verticales_en_sentido_horario:','fr':':flèches_dans_le_sens_horaire:','ja':':右回り縦矢印:','ko':':시계_방향_화살표:','pt':':setas_verticais_no_sentido_horário:','it':':frecce_verticali_che_ruotano_in_senso_orario:','fa':':جهت_عقربه_های_ساعت:','id':':tanda_panah_searah_jarum_jam:','zh':':顺时针垂直箭头:','ru':':по_часовой_стрелке:','tr':':saat_yönünde_dönen_oklar:','ar':':سهمان_رأسيان_مع_عقارب_الساعة:'},'\U0001F4D5':{'en':':closed_book:','status':fully_qualified,'E':0.6,'de':':geschlossenes_buch:','es':':libro_cerrado:','fr':':livre_fermé:','ja':':閉じた本:','ko':':펼치지_않은_책:','pt':':livro_fechado:','it':':libro_chiuso:','fa':':کتاب_بسته:','id':':buku_tertutup:','zh':':合上的书本:','ru':':закрытая_книга:','tr':':kapalı_kitap:','ar':':كتاب_مغلق:'},'\U0001F4EA':{'en':':closed_mailbox_with_lowered_flag:','status':fully_qualified,'E':0.6,'alias':[':mailbox_closed:'],'variant':True,'de':':geschlossener_briefkasten_ohne_post:','es':':buzón_cerrado_con_la_bandera_bajada:','fr':':boîte_aux_lettres_fermée_drapeau_baissé:','ja':':閉じた郵便受け(手紙なし):','ko':':표지가_내려간_닫힌_우편함:','pt':':caixa_de_correio_fechada_com_bandeira_abaixada:','it':':cassetta_postale_chiusa_bandierina_abbassata:','fa':':صندوق_پستی_بسته_با_پرچم_پایین:','id':':kotak_surat_tertutup_dengan_bendera_turun:','zh':':无待收信件:','ru':':закрытый_почтовый_ящик_с_опущенным_флажком:','tr':':işareti_aşağıda_kapalı_posta_kutusu:','ar':':صندوق_بريد_مغلق_بإشارة_لأسفل:'},'\U0001F4EB':{'en':':closed_mailbox_with_raised_flag:','status':fully_qualified,'E':0.6,'alias':[':mailbox:'],'variant':True,'de':':geschlossener_briefkasten_mit_post:','es':':buzón_cerrado_con_la_bandera_levantada:','fr':':boîte_aux_lettres_fermée_drapeau_levé:','ja':':閉じた郵便受け(手紙あり):','ko':':표지가_올라간_닫힌_우편함:','pt':':caixa_de_correio_fechada_com_bandeira_levantada:','it':':cassetta_postale_chiusa_bandierina_alzata:','fa':':صندوق_پستی_بسته_با_پرچم_بالا:','id':':kotak_surat_tertutup_dengan_bendera_terangkat:','zh':':有待收信件:','ru':':закрытый_почтовый_ящик_с_поднятым_флажком:','tr':':işareti_yukarıda_kapalı_posta_kutusu:','ar':':صندوق_بريد_مغلق_بإشارة_مرفوعة:'},'\U0001F302':{'en':':closed_umbrella:','status':fully_qualified,'E':0.6,'de':':geschlossener_regenschirm:','es':':paraguas_cerrado:','fr':':parapluie_fermé:','ja':':閉じた傘:','ko':':접힌_우산:','pt':':guarda-chuva_fechado:','it':':ombrello_chiuso:','fa':':چتر_بسته:','id':':payung_tertutup:','zh':':收起的伞:','ru':':закрытый_зонт:','tr':':kapalı_şemsiye:','ar':':شمسية_مغلقة:'},'\U00002601\U0000FE0F':{'en':':cloud:','status':fully_qualified,'E':0.6,'variant':True,'de':':wolke:','es':':nube:','fr':':nuage:','ja':':雲:','ko':':구름:','pt':':nuvem:','it':':nuvola:','fa':':ابر:','id':':awan:','zh':':云:','ru':':облако:','tr':':bulut:','ar':':سحابة:'},'\U00002601':{'en':':cloud:','status':unqualified,'E':0.6,'variant':True,'de':':wolke:','es':':nube:','fr':':nuage:','ja':':雲:','ko':':구름:','pt':':nuvem:','it':':nuvola:','fa':':ابر:','id':':awan:','zh':':云:','ru':':облако:','tr':':bulut:','ar':':سحابة:'},'\U0001F329\U0000FE0F':{'en':':cloud_with_lightning:','status':fully_qualified,'E':0.7,'variant':True,'de':':wolke_mit_blitz:','es':':nube_con_rayo:','fr':':nuage_avec_éclair:','ja':':雷雲:','ko':':번개구름:','pt':':nuvem_com_trovão:','it':':fulmini:','fa':':ابر_با_برق:','id':':awan_petir:','zh':':打雷:','ru':':молния:','tr':':şimşek_bulutu:','ar':':سحابة_وبرق:'},'\U0001F329':{'en':':cloud_with_lightning:','status':unqualified,'E':0.7,'variant':True,'de':':wolke_mit_blitz:','es':':nube_con_rayo:','fr':':nuage_avec_éclair:','ja':':雷雲:','ko':':번개구름:','pt':':nuvem_com_trovão:','it':':fulmini:','fa':':ابر_با_برق:','id':':awan_petir:','zh':':打雷:','ru':':молния:','tr':':şimşek_bulutu:','ar':':سحابة_وبرق:'},'\U000026C8\U0000FE0F':{'en':':cloud_with_lightning_and_rain:','status':fully_qualified,'E':0.7,'alias':[':thunder_cloud_and_rain:'],'variant':True,'de':':wolke_mit_blitz_und_regen:','es':':nube_con_rayo_y_lluvia:','fr':':nuage_avec_éclair_et_pluie:','ja':':雷雨:','ko':':번개가_치는_비구름:','pt':':chuva_com_trovão:','it':':temporale:','fa':':ابر_با_رعدوبرق_و_باران:','id':':awan_dengan_petir_dan_hujan:','zh':':雷阵雨:','ru':':гроза:','tr':':şimşek_ve_yağmurlu_bulut:','ar':':سحابة_وبرق_ورعد:'},'\U000026C8':{'en':':cloud_with_lightning_and_rain:','status':unqualified,'E':0.7,'alias':[':thunder_cloud_and_rain:'],'variant':True,'de':':wolke_mit_blitz_und_regen:','es':':nube_con_rayo_y_lluvia:','fr':':nuage_avec_éclair_et_pluie:','ja':':雷雨:','ko':':번개가_치는_비구름:','pt':':chuva_com_trovão:','it':':temporale:','fa':':ابر_با_رعدوبرق_و_باران:','id':':awan_dengan_petir_dan_hujan:','zh':':雷阵雨:','ru':':гроза:','tr':':şimşek_ve_yağmurlu_bulut:','ar':':سحابة_وبرق_ورعد:'},'\U0001F327\U0000FE0F':{'en':':cloud_with_rain:','status':fully_qualified,'E':0.7,'variant':True,'de':':wolke_mit_regen:','es':':nube_con_lluvia:','fr':':nuage_avec_pluie:','ja':':雨雲:','ko':':비구름:','pt':':nuvem_com_chuva:','it':':pioggia:','fa':':ابر_بارانی:','id':':awan_hujan:','zh':':下雨:','ru':':дождь:','tr':':yağmur_bulutu:','ar':':سحابة_ومطر:'},'\U0001F327':{'en':':cloud_with_rain:','status':unqualified,'E':0.7,'variant':True,'de':':wolke_mit_regen:','es':':nube_con_lluvia:','fr':':nuage_avec_pluie:','ja':':雨雲:','ko':':비구름:','pt':':nuvem_com_chuva:','it':':pioggia:','fa':':ابر_بارانی:','id':':awan_hujan:','zh':':下雨:','ru':':дождь:','tr':':yağmur_bulutu:','ar':':سحابة_ومطر:'},'\U0001F328\U0000FE0F':{'en':':cloud_with_snow:','status':fully_qualified,'E':0.7,'variant':True,'de':':wolke_mit_schnee:','es':':nube_con_nieve:','fr':':nuage_avec_neige:','ja':':雪雲:','ko':':눈구름:','pt':':nuvem_com_neve:','it':':neve:','fa':':ابر_برفی:','id':':awan_salju:','zh':':下雪:','ru':':снег:','tr':':kar_bulutu:','ar':':سحابة_وثلج:'},'\U0001F328':{'en':':cloud_with_snow:','status':unqualified,'E':0.7,'variant':True,'de':':wolke_mit_schnee:','es':':nube_con_nieve:','fr':':nuage_avec_neige:','ja':':雪雲:','ko':':눈구름:','pt':':nuvem_com_neve:','it':':neve:','fa':':ابر_برفی:','id':':awan_salju:','zh':':下雪:','ru':':снег:','tr':':kar_bulutu:','ar':':سحابة_وثلج:'},'\U0001F921':{'en':':clown_face:','status':fully_qualified,'E':3,'de':':clown-gesicht:','es':':cara_de_payaso:','fr':':visage_de_clown:','ja':':ピエロの顔:','ko':':어릿광대_얼굴:','pt':':rosto_de_palhaço:','it':':faccina_pagliaccio:','fa':':دلقک:','id':':badut:','zh':':小丑脸:','ru':':клоун:','tr':':palyaço_yüzü:','ar':':وجه_مهرج:'},'\U00002663\U0000FE0F':{'en':':club_suit:','status':fully_qualified,'E':0.6,'alias':[':clubs:'],'variant':True,'de':':kreuz:','es':':palo_de_tréboles:','fr':':trèfle_cartes:','ja':':クラブ:','ko':':클럽:','pt':':naipe_de_paus:','it':':fiori:','fa':':خال_گشنیز:','id':':keriting:','zh':':梅花:','ru':':трефы:','tr':':sinek:','ar':':ورقة_لعب_بشكل_شجرة:'},'\U00002663':{'en':':club_suit:','status':unqualified,'E':0.6,'alias':[':clubs:'],'variant':True,'de':':kreuz:','es':':palo_de_tréboles:','fr':':trèfle_cartes:','ja':':クラブ:','ko':':클럽:','pt':':naipe_de_paus:','it':':fiori:','fa':':خال_گشنیز:','id':':keriting:','zh':':梅花:','ru':':трефы:','tr':':sinek:','ar':':ورقة_لعب_بشكل_شجرة:'},'\U0001F45D':{'en':':clutch_bag:','status':fully_qualified,'E':0.6,'alias':[':pouch:'],'de':':clutch:','es':':bolso_de_mano:','fr':':pochette:','ja':':ポーチ:','ko':':파우치:','pt':':bolsa_pequena:','it':':pochette:','fa':':کیف_بدون_تسمه_زنانه:','id':':kantung:','zh':':手袋:','ru':':клатч:','tr':':para_kesesi:','ar':':محفظة_يد:'},'\U0001F9E5':{'en':':coat:','status':fully_qualified,'E':5,'de':':mantel:','es':':abrigo:','fr':':manteau:','ja':':コート:','ko':':코트:','pt':':casaco:','it':':cappotto:','fa':':پالتو:','id':':mantel:','zh':':外套:','ru':':пальто:','tr':':kaban:','ar':':معطف:'},'\U0001FAB3':{'en':':cockroach:','status':fully_qualified,'E':13,'de':':kakerlake:','es':':cucaracha:','fr':':cafard:','ja':':ゴキブリ:','ko':':바퀴벌레:','pt':':barata:','it':':scarafaggio:','fa':':سوسک_حمام:','id':':kecoak:','zh':':蟑螂:','ru':':таракан:','tr':':hamam_böceği:','ar':':صرصور:'},'\U0001F378':{'en':':cocktail_glass:','status':fully_qualified,'E':0.6,'alias':[':cocktail:'],'variant':True,'de':':cocktailglas:','es':':copa_de_cóctel:','fr':':cocktail:','ja':':カクテルグラス:','ko':':칵테일:','pt':':coquetel:','it':':cocktail:','fa':':جام_کوکتل:','id':':gelas_cocktail:','zh':':鸡尾酒:','ru':':коктейль:','tr':':kokteyl_kadehi:','ar':':كأس_عصير_كوكتيل:'},'\U0001F965':{'en':':coconut:','status':fully_qualified,'E':5,'de':':kokosnuss:','es':':coco:','fr':':noix_de_coco:','ja':':ココナツ:','ko':':코코넛:','pt':':coco:','it':':cocco:','fa':':نارگیل:','id':':kelapa:','zh':':椰子:','ru':':кокос:','tr':':hindistan_cevizi:','ar':':جوز_الهند:'},'\U000026B0\U0000FE0F':{'en':':coffin:','status':fully_qualified,'E':1,'variant':True,'de':':sarg:','es':':ataúd:','fr':':cercueil:','ja':':棺桶:','ko':':관:','pt':':caixão:','it':':bara:','fa':':تابوت:','id':':peti_mati:','zh':':棺材:','ru':':гроб:','tr':':tabut:','ar':':تابوت:'},'\U000026B0':{'en':':coffin:','status':unqualified,'E':1,'variant':True,'de':':sarg:','es':':ataúd:','fr':':cercueil:','ja':':棺桶:','ko':':관:','pt':':caixão:','it':':bara:','fa':':تابوت:','id':':peti_mati:','zh':':棺材:','ru':':гроб:','tr':':tabut:','ar':':تابوت:'},'\U0001FA99':{'en':':coin:','status':fully_qualified,'E':13,'de':':münze:','es':':moneda:','fr':':pièce:','ja':':コイン:','ko':':동전:','pt':':moeda:','it':':moneta:','fa':':سکه:','id':':koin:','zh':':硬币:','ru':':монета:','tr':':bozuk_para:','ar':':عملة_معدنية:'},'\U0001F976':{'en':':cold_face:','status':fully_qualified,'E':11,'de':':frierendes_gesicht:','es':':cara_con_frío:','fr':':visage_bleu_et_froid:','ja':':寒い顔:','ko':':추워하는_얼굴:','pt':':rosto_gelado:','it':':faccina_congelata:','fa':':خیلی_سرد:','id':':wajah_kedinginan:','zh':':冷脸:','ru':':мерзнет:','tr':':üşümüş_yüz:','ar':':وجه_بارد:'},'\U0001F4A5':{'en':':collision:','status':fully_qualified,'E':0.6,'alias':[':boom:'],'de':':zusammenstoß:','es':':colisión:','fr':':explosion:','ja':':衝突:','ko':':충돌:','pt':':colisão:','it':':collisione:','fa':':برخورد:','id':':tabrakan:','zh':':爆炸:','ru':':взрыв:','tr':':çarpışma:','ar':':تصادم:'},'\U00002604\U0000FE0F':{'en':':comet:','status':fully_qualified,'E':1,'variant':True,'de':':komet:','es':':meteorito:','fr':':comète:','ja':':彗星:','ko':':혜성:','pt':':cometa:','it':':cometa:','fa':':ستاره_دنباله_دار:','id':':komet:','zh':':彗星:','ru':':комета:','tr':':kuyruklu_yıldız:','ar':':مذنب:'},'\U00002604':{'en':':comet:','status':unqualified,'E':1,'variant':True,'de':':komet:','es':':meteorito:','fr':':comète:','ja':':彗星:','ko':':혜성:','pt':':cometa:','it':':cometa:','fa':':ستاره_دنباله_دار:','id':':komet:','zh':':彗星:','ru':':комета:','tr':':kuyruklu_yıldız:','ar':':مذنب:'},'\U0001F9ED':{'en':':compass:','status':fully_qualified,'E':11,'de':':kompass:','es':':brújula:','fr':':boussole:','ja':':コンパス:','ko':':나침반:','pt':':bússola:','it':':bussola:','fa':':قطب_نما:','id':':kompas:','zh':':指南针:','ru':':компас:','tr':':pusula:','ar':':بوصلة:'},'\U0001F4BD':{'en':':computer_disk:','status':fully_qualified,'E':0.6,'alias':[':minidisc:'],'de':':minidisc:','es':':minidisc:','fr':':disque_d’ordinateur:','ja':':md:','ko':':엠디:','pt':':minidisc:','it':':minidisc:','fa':':مینی_دیسک:','id':':disk_komputer:','zh':':电脑光盘:','ru':':диск:','tr':':mini_disk:','ar':':قرص_كمبيوتر:'},'\U0001F5B1\U0000FE0F':{'en':':computer_mouse:','status':fully_qualified,'E':0.7,'alias':[':three_button_mouse:'],'variant':True,'de':':computermaus:','es':':ratón_de_ordenador:','fr':':souris_d’ordinateur:','ja':':マウス:','ko':':컴퓨터_마우스:','pt':':mouse:','it':':mouse:','fa':':موشواره:','id':':mouse_komputer:','zh':':电脑鼠标:','ru':':мышка:','tr':':bilgisayar_faresi:','ar':':ماوس:'},'\U0001F5B1':{'en':':computer_mouse:','status':unqualified,'E':0.7,'alias':[':three_button_mouse:'],'variant':True,'de':':computermaus:','es':':ratón_de_ordenador:','fr':':souris_d’ordinateur:','ja':':マウス:','ko':':컴퓨터_마우스:','pt':':mouse:','it':':mouse:','fa':':موشواره:','id':':mouse_komputer:','zh':':电脑鼠标:','ru':':мышка:','tr':':bilgisayar_faresi:','ar':':ماوس:'},'\U0001F38A':{'en':':confetti_ball:','status':fully_qualified,'E':0.6,'de':':konfettiball:','es':':bola_de_confeti:','fr':':confettis:','ja':':くす玉:','ko':':박_터트리기:','pt':':confete:','it':':coriandoli:','fa':':توپ_پولکی:','id':':bola_konfeti:','zh':':五彩纸屑球:','ru':':конфетти:','tr':':konfeti:','ar':':كرة_أوراق_ملونة:'},'\U0001F616':{'en':':confounded_face:','status':fully_qualified,'E':0.6,'alias':[':confounded:'],'de':':verwirrtes_gesicht:','es':':cara_de_frustración:','fr':':visage_déconcerté:','ja':':困惑した顔:','ko':':당혹한_얼굴:','pt':':rosto_perplexo:','it':':faccina_frustrata:','fa':':سردرگم:','id':':wajah_stres:','zh':':困惑:','ru':':в_растерянности:','tr':':kafası_çok_karışık_yüz:','ar':':وجه_مرتبك:'},'\U0001F615':{'en':':confused_face:','status':fully_qualified,'E':1,'alias':[':confused:'],'de':':verwundertes_gesicht:','es':':cara_de_confusión:','fr':':visage_confus:','ja':':混乱:','ko':':혼란스러워하는_얼굴:','pt':':rosto_confuso:','it':':faccina_confusa:','fa':':گیج_شده:','id':':wajah_bingung:','zh':':困扰:','ru':':в_замешательстве:','tr':':kafası_karışık_yüz:','ar':':وجه_حائر:'},'\U0001F6A7':{'en':':construction:','status':fully_qualified,'E':0.6,'de':':baustellenabsperrung:','es':':obras:','fr':':travaux:','ja':':工事中:','ko':':공사_중:','pt':':em_construção:','it':':area_cantiere:','fa':':ساخت_وساز:','id':':konstruksi:','zh':':路障:','ru':':строительные_работы:','tr':':inşaat:','ar':':تحت_الإنشاء:'},'\U0001F477':{'en':':construction_worker:','status':fully_qualified,'E':0.6,'de':':bauarbeiter(in):','es':':profesional_de_la_construcción:','fr':':personnel_du_bâtiment:','ja':':建設作業員:','ko':':건설_노동자:','pt':':trabalhador_de_construção_civil:','it':':operaio_edile:','fa':':کارگر:','id':':pekerja_konstruksi:','zh':':建筑工人:','ru':':строитель:','tr':':inşaat_işçisi:','ar':':شخص_يعمل_في_البناء:'},'\U0001F477\U0001F3FF':{'en':':construction_worker_dark_skin_tone:','status':fully_qualified,'E':1,'de':':bauarbeiter(in)_dunkle_hautfarbe:','es':':profesional_de_la_construcción_tono_de_piel_oscuro:','fr':':personnel_du_bâtiment_peau_foncée:','ja':':建設作業員_濃い肌色:','ko':':건설_노동자_검은색_피부:','pt':':trabalhador_de_construção_civil_pele_escura:','it':':operaio_edile_carnagione_scura:','fa':':کارگر_پوست_آبنوسی:','id':':pekerja_konstruksi_warna_kulit_gelap:','zh':':建筑工人_较深肤色:','ru':':строитель_очень_темный_тон_кожи:','tr':':inşaat_işçisi_koyu_cilt_tonu:','ar':':شخص_يعمل_في_البناء_بشرة_بلون_غامق:'},'\U0001F477\U0001F3FB':{'en':':construction_worker_light_skin_tone:','status':fully_qualified,'E':1,'de':':bauarbeiter(in)_helle_hautfarbe:','es':':profesional_de_la_construcción_tono_de_piel_claro:','fr':':personnel_du_bâtiment_peau_claire:','ja':':建設作業員_薄い肌色:','ko':':건설_노동자_하얀_피부:','pt':':trabalhador_de_construção_civil_pele_clara:','it':':operaio_edile_carnagione_chiara:','fa':':کارگر_پوست_سفید:','id':':pekerja_konstruksi_warna_kulit_cerah:','zh':':建筑工人_较浅肤色:','ru':':строитель_очень_светлый_тон_кожи:','tr':':inşaat_işçisi_açık_cilt_tonu:','ar':':شخص_يعمل_في_البناء_بشرة_بلون_فاتح:'},'\U0001F477\U0001F3FE':{'en':':construction_worker_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':bauarbeiter(in)_mitteldunkle_hautfarbe:','es':':profesional_de_la_construcción_tono_de_piel_oscuro_medio:','fr':':personnel_du_bâtiment_peau_mate:','ja':':建設作業員_やや濃い肌色:','ko':':건설_노동자_진한_갈색_피부:','pt':':trabalhador_de_construção_civil_pele_morena_escura:','it':':operaio_edile_carnagione_abbastanza_scura:','fa':':کارگر_پوست_گندمی:','id':':pekerja_konstruksi_warna_kulit_gelap-sedang:','zh':':建筑工人_中等深肤色:','ru':':строитель_темный_тон_кожи:','tr':':inşaat_işçisi_orta_koyu_cilt_tonu:','ar':':شخص_يعمل_في_البناء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F477\U0001F3FC':{'en':':construction_worker_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':bauarbeiter(in)_mittelhelle_hautfarbe:','es':':profesional_de_la_construcción_tono_de_piel_claro_medio:','fr':':personnel_du_bâtiment_peau_moyennement_claire:','ja':':建設作業員_やや薄い肌色:','ko':':건설_노동자_연한_갈색_피부:','pt':':trabalhador_de_construção_civil_pele_morena_clara:','it':':operaio_edile_carnagione_abbastanza_chiara:','fa':':کارگر_پوست_روشن:','id':':pekerja_konstruksi_warna_kulit_cerah-sedang:','zh':':建筑工人_中等浅肤色:','ru':':строитель_светлый_тон_кожи:','tr':':inşaat_işçisi_orta_açık_cilt_tonu:','ar':':شخص_يعمل_في_البناء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F477\U0001F3FD':{'en':':construction_worker_medium_skin_tone:','status':fully_qualified,'E':1,'de':':bauarbeiter(in)_mittlere_hautfarbe:','es':':profesional_de_la_construcción_tono_de_piel_medio:','fr':':personnel_du_bâtiment_peau_légèrement_mate:','ja':':建設作業員_中間の肌色:','ko':':건설_노동자_갈색_피부:','pt':':trabalhador_de_construção_civil_pele_morena:','it':':operaio_edile_carnagione_olivastra:','fa':':کارگر_پوست_طلایی:','id':':pekerja_konstruksi_warna_kulit_sedang:','zh':':建筑工人_中等肤色:','ru':':строитель_средний_тон_кожи:','tr':':inşaat_işçisi_orta_cilt_tonu:','ar':':شخص_يعمل_في_البناء_بشرة_بلون_معتدل:'},'\U0001F39B\U0000FE0F':{'en':':control_knobs:','status':fully_qualified,'E':0.7,'variant':True,'de':':drehregler:','es':':ruedas_de_control:','fr':':boutons_de_réglage:','ja':':コントロールつまみ:','ko':':컨트롤_레버:','pt':':botões_giratórios:','it':':manopole_di_controllo:','fa':':دسته_های_کنترل:','id':':kenop_kontrol:','zh':':控制旋钮:','ru':':регуляторы:','tr':':kontrol_düğmeleri:','ar':':أزرار_تحكم:'},'\U0001F39B':{'en':':control_knobs:','status':unqualified,'E':0.7,'variant':True,'de':':drehregler:','es':':ruedas_de_control:','fr':':boutons_de_réglage:','ja':':コントロールつまみ:','ko':':컨트롤_레버:','pt':':botões_giratórios:','it':':manopole_di_controllo:','fa':':دسته_های_کنترل:','id':':kenop_kontrol:','zh':':控制旋钮:','ru':':регуляторы:','tr':':kontrol_düğmeleri:','ar':':أزرار_تحكم:'},'\U0001F3EA':{'en':':convenience_store:','status':fully_qualified,'E':0.6,'de':':minimarkt:','es':':tienda_24_horas:','fr':':supérette:','ja':':コンビニ:','ko':':편의점:','pt':':loja_de_conveniência:','it':':minimarket:','fa':':بقالی:','id':':minimarket:','zh':':便利店:','ru':':круглосуточный_магазин:','tr':':sürekli_açık_market:','ar':':متجر_بقالة:'},'\U0001F9D1\U0000200D\U0001F373':{'en':':cook:','status':fully_qualified,'E':12.1,'de':':koch/köchin:','es':':chef:','fr':':cuisinier_(tous_genres):','ja':':コック:','ko':':요리사:','pt':':chef_de_cozinha:','it':':persona_che_cucina:','fa':':آشپز:','id':':koki:','zh':':厨师:','ru':':повар:','tr':':aşçı:','ar':':طاهي:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F373':{'en':':cook_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':koch/köchin_dunkle_hautfarbe:','es':':chef_tono_de_piel_oscuro:','fr':':cuisinier_(tous_genres)_peau_foncée:','ja':':コック_濃い肌色:','ko':':요리사_검은색_피부:','pt':':chef_de_cozinha_pele_escura:','it':':persona_che_cucina_carnagione_scura:','fa':':آشپز_پوست_آبنوسی:','id':':koki_warna_kulit_gelap:','zh':':厨师_较深肤色:','ru':':повар_очень_темный_тон_кожи:','tr':':aşçı_koyu_cilt_tonu:','ar':':طاهي_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F373':{'en':':cook_light_skin_tone:','status':fully_qualified,'E':12.1,'de':':koch/köchin_helle_hautfarbe:','es':':chef_tono_de_piel_claro:','fr':':cuisinier_(tous_genres)_peau_claire:','ja':':コック_薄い肌色:','ko':':요리사_하얀_피부:','pt':':chef_de_cozinha_pele_clara:','it':':persona_che_cucina_carnagione_chiara:','fa':':آشپز_پوست_سفید:','id':':koki_warna_kulit_cerah:','zh':':厨师_较浅肤色:','ru':':повар_очень_светлый_тон_кожи:','tr':':aşçı_açık_cilt_tonu:','ar':':طاهي_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F373':{'en':':cook_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':koch/köchin_mitteldunkle_hautfarbe:','es':':chef_tono_de_piel_oscuro_medio:','fr':':cuisinier_(tous_genres)_peau_mate:','ja':':コック_やや濃い肌色:','ko':':요리사_진한_갈색_피부:','pt':':chef_de_cozinha_pele_morena_escura:','it':':persona_che_cucina_carnagione_abbastanza_scura:','fa':':آشپز_پوست_گندمی:','id':':koki_warna_kulit_gelap-sedang:','zh':':厨师_中等深肤色:','ru':':повар_темный_тон_кожи:','tr':':aşçı_orta_koyu_cilt_tonu:','ar':':طاهي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F373':{'en':':cook_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':koch/köchin_mittelhelle_hautfarbe:','es':':chef_tono_de_piel_claro_medio:','fr':':cuisinier_(tous_genres)_peau_moyennement_claire:','ja':':コック_やや薄い肌色:','ko':':요리사_연한_갈색_피부:','pt':':chef_de_cozinha_pele_morena_clara:','it':':persona_che_cucina_carnagione_abbastanza_chiara:','fa':':آشپز_پوست_روشن:','id':':koki_warna_kulit_cerah-sedang:','zh':':厨师_中等浅肤色:','ru':':повар_светлый_тон_кожи:','tr':':aşçı_orta_açık_cilt_tonu:','ar':':طاهي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F373':{'en':':cook_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':koch/köchin_mittlere_hautfarbe:','es':':chef_tono_de_piel_medio:','fr':':cuisinier_(tous_genres)_peau_légèrement_mate:','ja':':コック_中間の肌色:','ko':':요리사_갈색_피부:','pt':':chef_de_cozinha_pele_morena:','it':':persona_che_cucina_carnagione_olivastra:','fa':':آشپز_پوست_طلایی:','id':':koki_warna_kulit_sedang:','zh':':厨师_中等肤色:','ru':':повар_средний_тон_кожи:','tr':':aşçı_orta_cilt_tonu:','ar':':طاهي_بشرة_بلون_معتدل:'},'\U0001F35A':{'en':':cooked_rice:','status':fully_qualified,'E':0.6,'alias':[':rice:'],'de':':reis_in_schüssel:','es':':arroz_cocido:','fr':':bol_de_riz:','ja':':ごはん:','ko':':밥:','pt':':arroz_cozido:','it':':riso_bollito:','fa':':برنج_پخته:','id':':nasi_matang:','zh':':米饭:','ru':':рис:','tr':':pilav:','ar':':أرز_مطبوخ:'},'\U0001F36A':{'en':':cookie:','status':fully_qualified,'E':0.6,'de':':keks:','es':':galleta:','fr':':cookie:','ja':':クッキー:','ko':':쿠키:','pt':':biscoito:','it':':biscotto:','fa':':کلوچه:','id':':biskuit:','zh':':饼干:','ru':':печенье:','tr':':kurabiye:','ar':':كعكة:'},'\U0001F373':{'en':':cooking:','status':fully_qualified,'E':0.6,'alias':[':egg:', ':fried_egg:'],'de':':spiegelei_in_bratpfanne:','es':':cocinar:','fr':':œuf_au_plat:','ja':':料理:','ko':':프라이팬:','pt':':ovo_frito:','it':':cucinare:','fa':':آشپزی:','id':':memasak:','zh':':煎蛋:','ru':':яичница_на_сковороде:','tr':':yemek_pişirme:','ar':':مقلاة:'},'\U000000A9\U0000FE0F':{'en':':copyright:','status':fully_qualified,'E':0.6,'variant':True,'de':':copyright:','es':':copyright:','fr':':symbole_copyright:','ja':':著作権マーク:','ko':':저작권:','pt':':símbolo_de_copyright:','it':':copyright:','fa':':حق_نسخه_برداری:','id':':hak_cipta:','zh':':版权:','ru':':значок_защиты_авторского_права:','tr':':telif_hakkı_işareti:','ar':':علامة_حقوق_نشر:'},'\U000000A9':{'en':':copyright:','status':unqualified,'E':0.6,'variant':True,'de':':copyright:','es':':copyright:','fr':':symbole_copyright:','ja':':著作権マーク:','ko':':저작권:','pt':':símbolo_de_copyright:','it':':copyright:','fa':':حق_نسخه_برداری:','id':':hak_cipta:','zh':':版权:','ru':':значок_защиты_авторского_права:','tr':':telif_hakkı_işareti:','ar':':علامة_حقوق_نشر:'},'\U0001FAB8':{'en':':coral:','status':fully_qualified,'E':14,'de':':koralle:','es':':coral:','fr':':corail:','ja':':サンゴ:','ko':':산호초:','pt':':coral:','it':':corallo:','fa':':مرجان:','id':':terumbu_karang:','zh':':珊瑚:','ru':':коралл:','tr':':mercan:','ar':':مرجان:'},'\U0001F6CB\U0000FE0F':{'en':':couch_and_lamp:','status':fully_qualified,'E':0.7,'variant':True,'de':':sofa_und_lampe:','es':':sofá_y_lámpara:','fr':':canapé_et_lampe:','ja':':ソファとランプ:','ko':':소파와_전등:','pt':':sofá_e_luminária:','it':':divano_e_lampada:','fa':':مبل_راحتی_و_چراغ:','id':':sofa_dan_lampu:','zh':':沙发和灯:','ru':':диван_и_торшер:','tr':':kanepe_ve_lamba:','ar':':كنبة_ومصباح:'},'\U0001F6CB':{'en':':couch_and_lamp:','status':unqualified,'E':0.7,'variant':True,'de':':sofa_und_lampe:','es':':sofá_y_lámpara:','fr':':canapé_et_lampe:','ja':':ソファとランプ:','ko':':소파와_전등:','pt':':sofá_e_luminária:','it':':divano_e_lampada:','fa':':مبل_راحتی_و_چراغ:','id':':sofa_dan_lampu:','zh':':沙发和灯:','ru':':диван_и_торшер:','tr':':kanepe_ve_lamba:','ar':':كنبة_ومصباح:'},'\U0001F504':{'en':':counterclockwise_arrows_button:','status':fully_qualified,'E':1,'alias':[':arrows_counterclockwise:'],'de':':pfeile_gegen_den_uhrzeigersinn:','es':':flechas_en_sentido_antihorario:','fr':':flèches_dans_le_sens_antihoraire:','ja':':左回り矢印:','ko':':반시계_방향_화살표:','pt':':botão_de_setas_em_sentido_anti-horário:','it':':frecce_che_ruotano_in_senso_antiorario:','fa':':خلاف_جهت_عقربه_های_ساعت:','id':':tanda_panah_berlawanan_arah_jarum_jam:','zh':':逆时针箭头按钮:','ru':':против_часовой_стрелки:','tr':':saat_yönünün_tersinde_dönen_oklar:','ar':':زر_عكس_اتجاه_عقارب_الساعة:'},'\U0001F491':{'en':':couple_with_heart:','status':fully_qualified,'E':0.6,'de':':liebespaar:','es':':pareja_enamorada:','fr':':couple_avec_cœur:','ja':':カップルとハート:','ko':':연인:','pt':':casal_apaixonado:','it':':coppia_con_cuore:','fa':':زوج_عاشق:','id':':pasangan_dengan_hati:','zh':':情侣:','ru':':влюбленная_пара:','tr':':kalpli_çift:','ar':':زوج_وقلب:'},'\U0001F491\U0001F3FF':{'en':':couple_with_heart_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_dunkle_hautfarbe:','es':':pareja_enamorada_tono_de_piel_oscuro:','fr':':couple_avec_cœur_peau_foncée:','ja':':カップルとハート_濃い肌色:','ko':':연인_검은색_피부:','pt':':casal_apaixonado_pele_escura:','it':':coppia_con_cuore_carnagione_scura:','fa':':زوج_عاشق_پوست_آبنوسی:','id':':pasangan_dengan_hati_warna_kulit_gelap:','zh':':情侣_较深肤色:','ru':':влюбленная_пара_очень_темный_тон_кожи:','tr':':kalpli_çift_koyu_cilt_tonu:','ar':':زوج_وقلب_بشرة_بلون_غامق:'},'\U0001F491\U0001F3FB':{'en':':couple_with_heart_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_helle_hautfarbe:','es':':pareja_enamorada_tono_de_piel_claro:','fr':':couple_avec_cœur_peau_claire:','ja':':カップルとハート_薄い肌色:','ko':':연인_하얀_피부:','pt':':casal_apaixonado_pele_clara:','it':':coppia_con_cuore_carnagione_chiara:','fa':':زوج_عاشق_پوست_سفید:','id':':pasangan_dengan_hati_warna_kulit_cerah:','zh':':情侣_较浅肤色:','ru':':влюбленная_пара_очень_светлый_тон_кожи:','tr':':kalpli_çift_açık_cilt_tonu:','ar':':زوج_وقلب_بشرة_بلون_فاتح:'},'\U0001F468\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468':{'en':':couple_with_heart_man_man:','status':fully_qualified,'E':2,'de':':liebespaar_mann,_mann:','es':':pareja_enamorada_hombre_y_hombre:','fr':':couple_avec_cœur_homme_et_homme:','ja':':カップルとハート_男性_男性:','ko':':연인_남자_남자:','pt':':casal_apaixonado_homem_e_homem:','it':':coppia_con_cuore_uomo_e_uomo:','fa':':زوج_عاشق_مرد_مرد:','id':':pasangan_dengan_hati_pria_pria:','zh':':情侣_男人男人:','ru':':влюбленная_пара_мужчина_мужчина:','tr':':kalpli_çift_erkek_erkek:','ar':':زوج_وقلب_رجل_ورجل:'},'\U0001F468\U0000200D\U00002764\U0000200D\U0001F468':{'en':':couple_with_heart_man_man:','status':minimally_qualified,'E':2,'de':':liebespaar_mann_mann:','es':':pareja_enamorada_hombre_y_hombre:','fr':':couple_avec_cœur_homme_et_homme:','ja':':カップルとハート_男性_男性:','ko':':연인_남자_남자:','pt':':casal_apaixonado_homem_e_homem:','it':':coppia_con_cuore_uomo_e_uomo:','fa':':زوج_عاشق_مرد_مرد:','id':':pasangan_dengan_hati_pria_pria:','zh':':情侣_男人男人:','ru':':влюбленная_пара_мужчина_мужчина:','tr':':kalpli_çift_erkek_erkek:','ar':':زوج_وقلب_رجل_ورجل:'},'\U0001F468\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FF':{'en':':couple_with_heart_man_man_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_und_dunkle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_oscuro:','fr':':couple_avec_cœur_homme_homme_et_peau_foncée:','ja':':カップルとハート_男性_男性_濃い肌色:','ko':':연인_남자_남자_검은색_피부:','pt':':casal_apaixonado_homem_homem_e_pele_escura:','it':':coppia_con_cuore_uomo_uomo_e_carnagione_scura:','fa':':زوج_عاشق_مرد_مرد_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_gelap:','zh':':情侣_男人男人较深肤色:','ru':':влюбленная_пара_мужчина_мужчина_очень_темныи_тон_кожи:','tr':':kalpli_çift_erkek_erkek_koyu_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FF':{'en':':couple_with_heart_man_man_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_und_dunkle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_oscuro:','fr':':couple_avec_cœur_homme_homme_et_peau_foncée:','ja':':カップルとハート_男性_男性_濃い肌色:','ko':':연인_남자_남자_검은색_피부:','pt':':casal_apaixonado_homem_homem_e_pele_escura:','it':':coppia_con_cuore_uomo_uomo_e_carnagione_scura:','fa':':زوج_عاشق_مرد_مرد_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_gelap:','zh':':情侣_男人男人较深肤色:','ru':':влюбленная_пара_мужчина_мужчина_очень_темный_тон_кожи:','tr':':kalpli_çift_erkek_erkek_koyu_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FB':{'en':':couple_with_heart_man_man_dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_dunkle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_oscuro_tono_de_piel_claro:','fr':':couple_avec_cœur_homme_homme_peau_foncée_et_peau_claire:','ja':':カップルとハート_男性_男性_濃い肌色_薄い肌色:','ko':':연인_남자_남자_검은색_피부_하얀_피부:','pt':':casal_apaixonado_homem_homem_pele_escura_e_pele_clara:','it':':coppia_con_cuore_uomo_uomo_carnagione_scura_e_carnagione_chiara:','fa':':زوج_عاشق_مرد_مرد_پوست_آبنوسی_و_پوست_سفید:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_gelap_warna_kulit_cerah:','zh':':情侣_男人男人较深肤色较浅肤色:','ru':':влюбленная_пара_мужчина_мужчина_очень_темныи_тон_кожи_очень_светлыи_тон_кожи:','tr':':kalpli_çift_erkek_erkek_koyu_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FB':{'en':':couple_with_heart_man_man_dark_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_dunkle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_oscuro_tono_de_piel_claro:','fr':':couple_avec_cœur_homme_homme_peau_foncée_et_peau_claire:','ja':':カップルとハート_男性_男性_濃い肌色_薄い肌色:','ko':':연인_남자_남자_검은색_피부_하얀_피부:','pt':':casal_apaixonado_homem_homem_pele_escura_e_pele_clara:','it':':coppia_con_cuore_uomo_uomo_carnagione_scura_e_carnagione_chiara:','fa':':زوج_عاشق_مرد_مرد_پوست_آبنوسی_و_پوست_سفید:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_gelap_warna_kulit_cerah:','zh':':情侣_男人男人较深肤色较浅肤色:','ru':':влюбленная_пара_мужчина_мужчина_очень_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':kalpli_çift_erkek_erkek_koyu_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FE':{'en':':couple_with_heart_man_man_dark_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_dunkle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_oscuro_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_homme_homme_peau_foncée_et_peau_mate:','ja':':カップルとハート_男性_男性_濃い肌色_やや濃い肌色:','ko':':연인_남자_남자_검은색_피부_진한_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_escura_e_pele_morena_escura:','it':':coppia_con_cuore_uomo_uomo_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_مرد_مرد_پوست_آبنوسی_و_پوست_گندمی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':情侣_男人男人较深肤色中等深肤色:','ru':':влюбленная_пара_мужчина_мужчина_очень_темныи_тон_кожи_темныи_тон_кожи:','tr':':kalpli_çift_erkek_erkek_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FE':{'en':':couple_with_heart_man_man_dark_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_dunkle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_oscuro_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_homme_homme_peau_foncée_et_peau_mate:','ja':':カップルとハート_男性_男性_濃い肌色_やや濃い肌色:','ko':':연인_남자_남자_검은색_피부_진한_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_escura_e_pele_morena_escura:','it':':coppia_con_cuore_uomo_uomo_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_مرد_مرد_پوست_آبنوسی_و_پوست_گندمی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':情侣_男人男人较深肤色中等深肤色:','ru':':влюбленная_пара_мужчина_мужчина_очень_темный_тон_кожи_темный_тон_кожи:','tr':':kalpli_çift_erkek_erkek_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FC':{'en':':couple_with_heart_man_man_dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_dunkle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_oscuro_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_homme_homme_peau_foncée_et_peau_moyennement_claire:','ja':':カップルとハート_男性_男性_濃い肌色_やや薄い肌色:','ko':':연인_남자_남자_검은색_피부_연한_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_escura_e_pele_morena_clara:','it':':coppia_con_cuore_uomo_uomo_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_مرد_مرد_پوست_آبنوسی_و_پوست_روشن:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':情侣_男人男人较深肤色中等浅肤色:','ru':':влюбленная_пара_мужчина_мужчина_очень_темныи_тон_кожи_светлыи_тон_кожи:','tr':':kalpli_çift_erkek_erkek_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FC':{'en':':couple_with_heart_man_man_dark_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_dunkle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_oscuro_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_homme_homme_peau_foncée_et_peau_moyennement_claire:','ja':':カップルとハート_男性_男性_濃い肌色_やや薄い肌色:','ko':':연인_남자_남자_검은색_피부_연한_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_escura_e_pele_morena_clara:','it':':coppia_con_cuore_uomo_uomo_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_مرد_مرد_پوست_آبنوسی_و_پوست_روشن:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':情侣_男人男人较深肤色中等浅肤色:','ru':':влюбленная_пара_мужчина_мужчина_очень_темный_тон_кожи_светлый_тон_кожи:','tr':':kalpli_çift_erkek_erkek_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FD':{'en':':couple_with_heart_man_man_dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_dunkle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_oscuro_tono_de_piel_medio:','fr':':couple_avec_cœur_homme_homme_peau_foncée_et_peau_légèrement_mate:','ja':':カップルとハート_男性_男性_濃い肌色_中間の肌色:','ko':':연인_남자_남자_검은색_피부_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_escura_e_pele_morena:','it':':coppia_con_cuore_uomo_uomo_carnagione_scura_e_carnagione_olivastra:','fa':':زوج_عاشق_مرد_مرد_پوست_آبنوسی_و_پوست_طلایی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_gelap_warna_kulit_sedang:','zh':':情侣_男人男人较深肤色中等肤色:','ru':':влюбленная_пара_мужчина_мужчина_очень_темныи_тон_кожи_среднии_тон_кожи:','tr':':kalpli_çift_erkek_erkek_koyu_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F468\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FD':{'en':':couple_with_heart_man_man_dark_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_dunkle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_oscuro_tono_de_piel_medio:','fr':':couple_avec_cœur_homme_homme_peau_foncée_et_peau_légèrement_mate:','ja':':カップルとハート_男性_男性_濃い肌色_中間の肌色:','ko':':연인_남자_남자_검은색_피부_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_escura_e_pele_morena:','it':':coppia_con_cuore_uomo_uomo_carnagione_scura_e_carnagione_olivastra:','fa':':زوج_عاشق_مرد_مرد_پوست_آبنوسی_و_پوست_طلایی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_gelap_warna_kulit_sedang:','zh':':情侣_男人男人较深肤色中等肤色:','ru':':влюбленная_пара_мужчина_мужчина_очень_темный_тон_кожи_средний_тон_кожи:','tr':':kalpli_çift_erkek_erkek_koyu_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F468\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FB':{'en':':couple_with_heart_man_man_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_und_helle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_claro:','fr':':couple_avec_cœur_homme_homme_et_peau_claire:','ja':':カップルとハート_男性_男性_薄い肌色:','ko':':연인_남자_남자_하얀_피부:','pt':':casal_apaixonado_homem_homem_e_pele_clara:','it':':coppia_con_cuore_uomo_uomo_e_carnagione_chiara:','fa':':زوج_عاشق_مرد_مرد_و_پوست_سفید:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_cerah:','zh':':情侣_男人男人较浅肤色:','ru':':влюбленная_пара_мужчина_мужчина_очень_светлыи_тон_кожи:','tr':':kalpli_çift_erkek_erkek_açık_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FB':{'en':':couple_with_heart_man_man_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_und_helle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_claro:','fr':':couple_avec_cœur_homme_homme_et_peau_claire:','ja':':カップルとハート_男性_男性_薄い肌色:','ko':':연인_남자_남자_하얀_피부:','pt':':casal_apaixonado_homem_homem_e_pele_clara:','it':':coppia_con_cuore_uomo_uomo_e_carnagione_chiara:','fa':':زوج_عاشق_مرد_مرد_و_پوست_سفید:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_cerah:','zh':':情侣_男人男人较浅肤色:','ru':':влюбленная_пара_мужчина_мужчина_очень_светлый_тон_кожи:','tr':':kalpli_çift_erkek_erkek_açık_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FF':{'en':':couple_with_heart_man_man_light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_helle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_claro_tono_de_piel_oscuro:','fr':':couple_avec_cœur_homme_homme_peau_claire_et_peau_foncée:','ja':':カップルとハート_男性_男性_薄い肌色_濃い肌色:','ko':':연인_남자_남자_하얀_피부_검은색_피부:','pt':':casal_apaixonado_homem_homem_pele_clara_e_pele_escura:','it':':coppia_con_cuore_uomo_uomo_carnagione_chiara_e_carnagione_scura:','fa':':زوج_عاشق_مرد_مرد_پوست_سفید_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_cerah_warna_kulit_gelap:','zh':':情侣_男人男人较浅肤色较深肤色:','ru':':влюбленная_пара_мужчина_мужчина_очень_светлыи_тон_кожи_очень_темныи_тон_кожи:','tr':':kalpli_çift_erkek_erkek_açık_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FF':{'en':':couple_with_heart_man_man_light_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_helle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_claro_tono_de_piel_oscuro:','fr':':couple_avec_cœur_homme_homme_peau_claire_et_peau_foncée:','ja':':カップルとハート_男性_男性_薄い肌色_濃い肌色:','ko':':연인_남자_남자_하얀_피부_검은색_피부:','pt':':casal_apaixonado_homem_homem_pele_clara_e_pele_escura:','it':':coppia_con_cuore_uomo_uomo_carnagione_chiara_e_carnagione_scura:','fa':':زوج_عاشق_مرد_مرد_پوست_سفید_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_cerah_warna_kulit_gelap:','zh':':情侣_男人男人较浅肤色较深肤色:','ru':':влюбленная_пара_мужчина_мужчина_очень_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':kalpli_çift_erkek_erkek_açık_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FE':{'en':':couple_with_heart_man_man_light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_helle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_claro_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_homme_homme_peau_claire_et_peau_mate:','ja':':カップルとハート_男性_男性_薄い肌色_やや濃い肌色:','ko':':연인_남자_남자_하얀_피부_진한_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_clara_e_pele_morena_escura:','it':':coppia_con_cuore_uomo_uomo_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_مرد_مرد_پوست_سفید_و_پوست_گندمی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':情侣_男人男人较浅肤色中等深肤色:','ru':':влюбленная_пара_мужчина_мужчина_очень_светлыи_тон_кожи_темныи_тон_кожи:','tr':':kalpli_çift_erkek_erkek_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FE':{'en':':couple_with_heart_man_man_light_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_helle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_claro_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_homme_homme_peau_claire_et_peau_mate:','ja':':カップルとハート_男性_男性_薄い肌色_やや濃い肌色:','ko':':연인_남자_남자_하얀_피부_진한_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_clara_e_pele_morena_escura:','it':':coppia_con_cuore_uomo_uomo_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_مرد_مرد_پوست_سفید_و_پوست_گندمی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':情侣_男人男人较浅肤色中等深肤色:','ru':':влюбленная_пара_мужчина_мужчина_очень_светлый_тон_кожи_темный_тон_кожи:','tr':':kalpli_çift_erkek_erkek_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FC':{'en':':couple_with_heart_man_man_light_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_helle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_claro_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_homme_homme_peau_claire_et_peau_moyennement_claire:','ja':':カップルとハート_男性_男性_薄い肌色_やや薄い肌色:','ko':':연인_남자_남자_하얀_피부_연한_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_clara_e_pele_morena_clara:','it':':coppia_con_cuore_uomo_uomo_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_مرد_مرد_پوست_سفید_و_پوست_روشن:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':情侣_男人男人较浅肤色中等浅肤色:','ru':':влюбленная_пара_мужчина_мужчина_очень_светлыи_тон_кожи_светлыи_тон_кожи:','tr':':kalpli_çift_erkek_erkek_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FC':{'en':':couple_with_heart_man_man_light_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_helle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_claro_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_homme_homme_peau_claire_et_peau_moyennement_claire:','ja':':カップルとハート_男性_男性_薄い肌色_やや薄い肌色:','ko':':연인_남자_남자_하얀_피부_연한_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_clara_e_pele_morena_clara:','it':':coppia_con_cuore_uomo_uomo_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_مرد_مرد_پوست_سفید_و_پوست_روشن:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':情侣_男人男人较浅肤色中等浅肤色:','ru':':влюбленная_пара_мужчина_мужчина_очень_светлый_тон_кожи_светлый_тон_кожи:','tr':':kalpli_çift_erkek_erkek_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FD':{'en':':couple_with_heart_man_man_light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_helle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_claro_tono_de_piel_medio:','fr':':couple_avec_cœur_homme_homme_peau_claire_et_peau_légèrement_mate:','ja':':カップルとハート_男性_男性_薄い肌色_中間の肌色:','ko':':연인_남자_남자_하얀_피부_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_clara_e_pele_morena:','it':':coppia_con_cuore_uomo_uomo_carnagione_chiara_e_carnagione_olivastra:','fa':':زوج_عاشق_مرد_مرد_پوست_سفید_و_پوست_طلایی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_cerah_warna_kulit_sedang:','zh':':情侣_男人男人较浅肤色中等肤色:','ru':':влюбленная_пара_мужчина_мужчина_очень_светлыи_тон_кожи_среднии_тон_кожи:','tr':':kalpli_çift_erkek_erkek_açık_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F468\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FD':{'en':':couple_with_heart_man_man_light_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_helle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_claro_tono_de_piel_medio:','fr':':couple_avec_cœur_homme_homme_peau_claire_et_peau_légèrement_mate:','ja':':カップルとハート_男性_男性_薄い肌色_中間の肌色:','ko':':연인_남자_남자_하얀_피부_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_clara_e_pele_morena:','it':':coppia_con_cuore_uomo_uomo_carnagione_chiara_e_carnagione_olivastra:','fa':':زوج_عاشق_مرد_مرد_پوست_سفید_و_پوست_طلایی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_cerah_warna_kulit_sedang:','zh':':情侣_男人男人较浅肤色中等肤色:','ru':':влюбленная_пара_мужчина_мужчина_очень_светлый_тон_кожи_средний_тон_кожи:','tr':':kalpli_çift_erkek_erkek_açık_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F468\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FE':{'en':':couple_with_heart_man_man_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_und_mitteldunkle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_homme_homme_et_peau_mate:','ja':':カップルとハート_男性_男性_やや濃い肌色:','ko':':연인_남자_남자_진한_갈색_피부:','pt':':casal_apaixonado_homem_homem_e_pele_morena_escura:','it':':coppia_con_cuore_uomo_uomo_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_مرد_مرد_و_پوست_گندمی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_gelap-sedang:','zh':':情侣_男人男人中等深肤色:','ru':':влюбленная_пара_мужчина_мужчина_темныи_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FE':{'en':':couple_with_heart_man_man_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_und_mitteldunkle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_homme_homme_et_peau_mate:','ja':':カップルとハート_男性_男性_やや濃い肌色:','ko':':연인_남자_남자_진한_갈색_피부:','pt':':casal_apaixonado_homem_homem_e_pele_morena_escura:','it':':coppia_con_cuore_uomo_uomo_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_مرد_مرد_و_پوست_گندمی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_gelap-sedang:','zh':':情侣_男人男人中等深肤色:','ru':':влюбленная_пара_мужчина_мужчина_темный_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FF':{'en':':couple_with_heart_man_man_medium-dark_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_mitteldunkle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_oscuro_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_homme_homme_peau_mate_et_peau_foncée:','ja':':カップルとハート_男性_男性_やや濃い肌色_濃い肌色:','ko':':연인_남자_남자_진한_갈색_피부_검은색_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_escura_e_pele_escura:','it':':coppia_con_cuore_uomo_uomo_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':زوج_عاشق_مرد_مرد_پوست_گندمی_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':情侣_男人男人中等深肤色较深肤色:','ru':':влюбленная_пара_мужчина_мужчина_темныи_тон_кожи_очень_темныи_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FF':{'en':':couple_with_heart_man_man_medium-dark_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_mitteldunkle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_oscuro_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_homme_homme_peau_mate_et_peau_foncée:','ja':':カップルとハート_男性_男性_やや濃い肌色_濃い肌色:','ko':':연인_남자_남자_진한_갈색_피부_검은색_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_escura_e_pele_escura:','it':':coppia_con_cuore_uomo_uomo_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':زوج_عاشق_مرد_مرد_پوست_گندمی_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':情侣_男人男人中等深肤色较深肤色:','ru':':влюбленная_пара_мужчина_мужчина_темный_тон_кожи_очень_темный_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FB':{'en':':couple_with_heart_man_man_medium-dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_mitteldunkle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_oscuro_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_homme_homme_peau_mate_et_peau_claire:','ja':':カップルとハート_男性_男性_やや濃い肌色_薄い肌色:','ko':':연인_남자_남자_진한_갈색_피부_하얀_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_escura_e_pele_clara:','it':':coppia_con_cuore_uomo_uomo_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':زوج_عاشق_مرد_مرد_پوست_گندمی_و_پوست_سفید:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':情侣_男人男人中等深肤色较浅肤色:','ru':':влюбленная_пара_мужчина_мужчина_темныи_тон_кожи_очень_светлыи_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FB':{'en':':couple_with_heart_man_man_medium-dark_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_mitteldunkle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_oscuro_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_homme_homme_peau_mate_et_peau_claire:','ja':':カップルとハート_男性_男性_やや濃い肌色_薄い肌色:','ko':':연인_남자_남자_진한_갈색_피부_하얀_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_escura_e_pele_clara:','it':':coppia_con_cuore_uomo_uomo_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':زوج_عاشق_مرد_مرد_پوست_گندمی_و_پوست_سفید:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':情侣_男人男人中等深肤色较浅肤色:','ru':':влюбленная_пара_мужчина_мужчина_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FC':{'en':':couple_with_heart_man_man_medium-dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_mitteldunkle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_oscuro_medio_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_homme_homme_peau_mate_et_peau_moyennement_claire:','ja':':カップルとハート_男性_男性_やや濃い肌色_やや薄い肌色:','ko':':연인_남자_남자_진한_갈색_피부_연한_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_escura_e_pele_morena_clara:','it':':coppia_con_cuore_uomo_uomo_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_مرد_مرد_پوست_گندمی_و_پوست_روشن:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':情侣_男人男人中等深肤色中等浅肤色:','ru':':влюбленная_пара_мужчина_мужчина_темныи_тон_кожи_светлыи_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FC':{'en':':couple_with_heart_man_man_medium-dark_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_mitteldunkle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_oscuro_medio_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_homme_homme_peau_mate_et_peau_moyennement_claire:','ja':':カップルとハート_男性_男性_やや濃い肌色_やや薄い肌色:','ko':':연인_남자_남자_진한_갈색_피부_연한_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_escura_e_pele_morena_clara:','it':':coppia_con_cuore_uomo_uomo_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_مرد_مرد_پوست_گندمی_و_پوست_روشن:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':情侣_男人男人中等深肤色中等浅肤色:','ru':':влюбленная_пара_мужчина_мужчина_темный_тон_кожи_светлый_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FD':{'en':':couple_with_heart_man_man_medium-dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_mitteldunkle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_oscuro_medio_tono_de_piel_medio:','fr':':couple_avec_cœur_homme_homme_peau_mate_et_peau_légèrement_mate:','ja':':カップルとハート_男性_男性_やや濃い肌色_中間の肌色:','ko':':연인_남자_남자_진한_갈색_피부_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_escura_e_pele_morena:','it':':coppia_con_cuore_uomo_uomo_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':زوج_عاشق_مرد_مرد_پوست_گندمی_و_پوست_طلایی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':情侣_男人男人中等深肤色中等肤色:','ru':':влюбленная_пара_мужчина_мужчина_темныи_тон_кожи_среднии_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F468\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FD':{'en':':couple_with_heart_man_man_medium-dark_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_mitteldunkle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_oscuro_medio_tono_de_piel_medio:','fr':':couple_avec_cœur_homme_homme_peau_mate_et_peau_légèrement_mate:','ja':':カップルとハート_男性_男性_やや濃い肌色_中間の肌色:','ko':':연인_남자_남자_진한_갈색_피부_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_escura_e_pele_morena:','it':':coppia_con_cuore_uomo_uomo_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':زوج_عاشق_مرد_مرد_پوست_گندمی_و_پوست_طلایی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':情侣_男人男人中等深肤色中等肤色:','ru':':влюбленная_пара_мужчина_мужчина_темный_тон_кожи_средний_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F468\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FC':{'en':':couple_with_heart_man_man_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_und_mittelhelle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_homme_homme_et_peau_moyennement_claire:','ja':':カップルとハート_男性_男性_やや薄い肌色:','ko':':연인_남자_남자_연한_갈색_피부:','pt':':casal_apaixonado_homem_homem_e_pele_morena_clara:','it':':coppia_con_cuore_uomo_uomo_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_مرد_مرد_و_پوست_روشن:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_cerah-sedang:','zh':':情侣_男人男人中等浅肤色:','ru':':влюбленная_пара_мужчина_мужчина_светлыи_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_açık_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FC':{'en':':couple_with_heart_man_man_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_und_mittelhelle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_homme_homme_et_peau_moyennement_claire:','ja':':カップルとハート_男性_男性_やや薄い肌色:','ko':':연인_남자_남자_연한_갈색_피부:','pt':':casal_apaixonado_homem_homem_e_pele_morena_clara:','it':':coppia_con_cuore_uomo_uomo_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_مرد_مرد_و_پوست_روشن:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_cerah-sedang:','zh':':情侣_男人男人中等浅肤色:','ru':':влюбленная_пара_мужчина_мужчина_светлый_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_açık_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FF':{'en':':couple_with_heart_man_man_medium-light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_mittelhelle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_claro_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_homme_homme_peau_moyennement_claire_et_peau_foncée:','ja':':カップルとハート_男性_男性_やや薄い肌色_濃い肌色:','ko':':연인_남자_남자_연한_갈색_피부_검은색_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_clara_e_pele_escura:','it':':coppia_con_cuore_uomo_uomo_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':زوج_عاشق_مرد_مرد_پوست_روشن_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':情侣_男人男人中等浅肤色较深肤色:','ru':':влюбленная_пара_мужчина_мужчина_светлыи_тон_кожи_очень_темныи_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FF':{'en':':couple_with_heart_man_man_medium-light_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_mittelhelle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_claro_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_homme_homme_peau_moyennement_claire_et_peau_foncée:','ja':':カップルとハート_男性_男性_やや薄い肌色_濃い肌色:','ko':':연인_남자_남자_연한_갈색_피부_검은색_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_clara_e_pele_escura:','it':':coppia_con_cuore_uomo_uomo_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':زوج_عاشق_مرد_مرد_پوست_روشن_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':情侣_男人男人中等浅肤色较深肤色:','ru':':влюбленная_пара_мужчина_мужчина_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FB':{'en':':couple_with_heart_man_man_medium-light_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_mittelhelle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_claro_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_homme_homme_peau_moyennement_claire_et_peau_claire:','ja':':カップルとハート_男性_男性_やや薄い肌色_薄い肌色:','ko':':연인_남자_남자_연한_갈색_피부_하얀_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_clara_e_pele_clara:','it':':coppia_con_cuore_uomo_uomo_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':زوج_عاشق_مرد_مرد_پوست_روشن_و_پوست_سفید:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':情侣_男人男人中等浅肤色较浅肤色:','ru':':влюбленная_пара_мужчина_мужчина_светлыи_тон_кожи_очень_светлыи_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FB':{'en':':couple_with_heart_man_man_medium-light_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_mittelhelle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_claro_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_homme_homme_peau_moyennement_claire_et_peau_claire:','ja':':カップルとハート_男性_男性_やや薄い肌色_薄い肌色:','ko':':연인_남자_남자_연한_갈색_피부_하얀_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_clara_e_pele_clara:','it':':coppia_con_cuore_uomo_uomo_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':زوج_عاشق_مرد_مرد_پوست_روشن_و_پوست_سفید:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':情侣_男人男人中等浅肤色较浅肤色:','ru':':влюбленная_пара_мужчина_мужчина_светлый_тон_кожи_очень_светлый_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FE':{'en':':couple_with_heart_man_man_medium-light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_mittelhelle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_claro_medio_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_homme_homme_peau_moyennement_claire_et_peau_mate:','ja':':カップルとハート_男性_男性_やや薄い肌色_やや濃い肌色:','ko':':연인_남자_남자_연한_갈색_피부_진한_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_clara_e_pele_morena_escura:','it':':coppia_con_cuore_uomo_uomo_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_مرد_مرد_پوست_روشن_و_پوست_گندمی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':情侣_男人男人中等浅肤色中等深肤色:','ru':':влюбленная_пара_мужчина_мужчина_светлыи_тон_кожи_темныи_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FE':{'en':':couple_with_heart_man_man_medium-light_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_mittelhelle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_claro_medio_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_homme_homme_peau_moyennement_claire_et_peau_mate:','ja':':カップルとハート_男性_男性_やや薄い肌色_やや濃い肌色:','ko':':연인_남자_남자_연한_갈색_피부_진한_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_clara_e_pele_morena_escura:','it':':coppia_con_cuore_uomo_uomo_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_مرد_مرد_پوست_روشن_و_پوست_گندمی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':情侣_男人男人中等浅肤色中等深肤色:','ru':':влюбленная_пара_мужчина_мужчина_светлый_тон_кожи_темный_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FD':{'en':':couple_with_heart_man_man_medium-light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_mittelhelle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_claro_medio_tono_de_piel_medio:','fr':':couple_avec_cœur_homme_homme_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':カップルとハート_男性_男性_やや薄い肌色_中間の肌色:','ko':':연인_남자_남자_연한_갈색_피부_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_clara_e_pele_morena:','it':':coppia_con_cuore_uomo_uomo_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':زوج_عاشق_مرد_مرد_پوست_روشن_و_پوست_طلایی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':情侣_男人男人中等浅肤色中等肤色:','ru':':влюбленная_пара_мужчина_мужчина_светлыи_тон_кожи_среднии_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F468\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FD':{'en':':couple_with_heart_man_man_medium-light_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_mittelhelle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_claro_medio_tono_de_piel_medio:','fr':':couple_avec_cœur_homme_homme_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':カップルとハート_男性_男性_やや薄い肌色_中間の肌色:','ko':':연인_남자_남자_연한_갈색_피부_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_clara_e_pele_morena:','it':':coppia_con_cuore_uomo_uomo_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':زوج_عاشق_مرد_مرد_پوست_روشن_و_پوست_طلایی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':情侣_男人男人中等浅肤色中等肤色:','ru':':влюбленная_пара_мужчина_мужчина_светлый_тон_кожи_средний_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F468\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FD':{'en':':couple_with_heart_man_man_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_und_mittlere_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_medio:','fr':':couple_avec_cœur_homme_homme_et_peau_légèrement_mate:','ja':':カップルとハート_男性_男性_中間の肌色:','ko':':연인_남자_남자_갈색_피부:','pt':':casal_apaixonado_homem_homem_e_pele_morena:','it':':coppia_con_cuore_uomo_uomo_e_carnagione_olivastra:','fa':':زوج_عاشق_مرد_مرد_و_پوست_طلایی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_sedang:','zh':':情侣_男人男人中等肤色:','ru':':влюбленная_пара_мужчина_мужчина_среднии_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_معتدل:'},'\U0001F468\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FD':{'en':':couple_with_heart_man_man_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_und_mittlere_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_medio:','fr':':couple_avec_cœur_homme_homme_et_peau_légèrement_mate:','ja':':カップルとハート_男性_男性_中間の肌色:','ko':':연인_남자_남자_갈색_피부:','pt':':casal_apaixonado_homem_homem_e_pele_morena:','it':':coppia_con_cuore_uomo_uomo_e_carnagione_olivastra:','fa':':زوج_عاشق_مرد_مرد_و_پوست_طلایی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_sedang:','zh':':情侣_男人男人中等肤色:','ru':':влюбленная_пара_мужчина_мужчина_средний_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_معتدل:'},'\U0001F468\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FF':{'en':':couple_with_heart_man_man_medium_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_mittlere_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_homme_homme_peau_légèrement_mate_et_peau_foncée:','ja':':カップルとハート_男性_男性_中間の肌色_濃い肌色:','ko':':연인_남자_남자_갈색_피부_검은색_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_e_pele_escura:','it':':coppia_con_cuore_uomo_uomo_carnagione_olivastra_e_carnagione_scura:','fa':':زوج_عاشق_مرد_مرد_پوست_طلایی_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_sedang_warna_kulit_gelap:','zh':':情侣_男人男人中等肤色较深肤色:','ru':':влюбленная_пара_мужчина_мужчина_среднии_тон_кожи_очень_темныи_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FF':{'en':':couple_with_heart_man_man_medium_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_mittlere_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_homme_homme_peau_légèrement_mate_et_peau_foncée:','ja':':カップルとハート_男性_男性_中間の肌色_濃い肌色:','ko':':연인_남자_남자_갈색_피부_검은색_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_e_pele_escura:','it':':coppia_con_cuore_uomo_uomo_carnagione_olivastra_e_carnagione_scura:','fa':':زوج_عاشق_مرد_مرد_پوست_طلایی_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_sedang_warna_kulit_gelap:','zh':':情侣_男人男人中等肤色较深肤色:','ru':':влюбленная_пара_мужчина_мужчина_средний_тон_кожи_очень_темный_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FB':{'en':':couple_with_heart_man_man_medium_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_mittlere_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_homme_homme_peau_légèrement_mate_et_peau_claire:','ja':':カップルとハート_男性_男性_中間の肌色_薄い肌色:','ko':':연인_남자_남자_갈색_피부_하얀_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_e_pele_clara:','it':':coppia_con_cuore_uomo_uomo_carnagione_olivastra_e_carnagione_chiara:','fa':':زوج_عاشق_مرد_مرد_پوست_طلایی_و_پوست_سفید:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_sedang_warna_kulit_cerah:','zh':':情侣_男人男人中等肤色较浅肤色:','ru':':влюбленная_пара_мужчина_мужчина_среднии_тон_кожи_очень_светлыи_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FB':{'en':':couple_with_heart_man_man_medium_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_mittlere_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_homme_homme_peau_légèrement_mate_et_peau_claire:','ja':':カップルとハート_男性_男性_中間の肌色_薄い肌色:','ko':':연인_남자_남자_갈색_피부_하얀_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_e_pele_clara:','it':':coppia_con_cuore_uomo_uomo_carnagione_olivastra_e_carnagione_chiara:','fa':':زوج_عاشق_مرد_مرد_پوست_طلایی_و_پوست_سفید:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_sedang_warna_kulit_cerah:','zh':':情侣_男人男人中等肤色较浅肤色:','ru':':влюбленная_пара_мужчина_мужчина_средний_тон_кожи_очень_светлый_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FE':{'en':':couple_with_heart_man_man_medium_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_mittlere_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_medio_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_homme_homme_peau_légèrement_mate_et_peau_mate:','ja':':カップルとハート_男性_男性_中間の肌色_やや濃い肌色:','ko':':연인_남자_남자_갈색_피부_진한_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_e_pele_morena_escura:','it':':coppia_con_cuore_uomo_uomo_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_مرد_مرد_پوست_طلایی_و_پوست_گندمی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':情侣_男人男人中等肤色中等深肤色:','ru':':влюбленная_пара_мужчина_мужчина_среднии_тон_кожи_темныи_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FE':{'en':':couple_with_heart_man_man_medium_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_mittlere_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_medio_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_homme_homme_peau_légèrement_mate_et_peau_mate:','ja':':カップルとハート_男性_男性_中間の肌色_やや濃い肌色:','ko':':연인_남자_남자_갈색_피부_진한_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_e_pele_morena_escura:','it':':coppia_con_cuore_uomo_uomo_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_مرد_مرد_پوست_طلایی_و_پوست_گندمی:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':情侣_男人男人中等肤色中等深肤色:','ru':':влюбленная_пара_мужчина_мужчина_средний_тон_кожи_темный_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FC':{'en':':couple_with_heart_man_man_medium_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mann_mann_mittlere_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_medio_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_homme_homme_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':カップルとハート_男性_男性_中間の肌色_やや薄い肌色:','ko':':연인_남자_남자_갈색_피부_연한_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_e_pele_morena_clara:','it':':coppia_con_cuore_uomo_uomo_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_مرد_مرد_پوست_طلایی_و_پوست_روشن:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':情侣_男人男人中等肤色中等浅肤色:','ru':':влюбленная_пара_мужчина_мужчина_среднии_тон_кожи_светлыи_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FC':{'en':':couple_with_heart_man_man_medium_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_mann_mann_mittlere_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_hombre_hombre_tono_de_piel_medio_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_homme_homme_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':カップルとハート_男性_男性_中間の肌色_やや薄い肌色:','ko':':연인_남자_남자_갈색_피부_연한_갈색_피부:','pt':':casal_apaixonado_homem_homem_pele_morena_e_pele_morena_clara:','it':':coppia_con_cuore_uomo_uomo_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_مرد_مرد_پوست_طلایی_و_پوست_روشن:','id':':pasangan_dengan_hati_pria_pria_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':情侣_男人男人中等肤色中等浅肤色:','ru':':влюбленная_пара_мужчина_мужчина_средний_тон_кожи_светлый_тон_кожи:','tr':':kalpli_çift_erkek_erkek_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_رجل_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F491\U0001F3FE':{'en':':couple_with_heart_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mitteldunkle_hautfarbe:','es':':pareja_enamorada_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_peau_mate:','ja':':カップルとハート_やや濃い肌色:','ko':':연인_진한_갈색_피부:','pt':':casal_apaixonado_pele_morena_escura:','it':':coppia_con_cuore_carnagione_abbastanza_scura:','fa':':زوج_عاشق_پوست_گندمی:','id':':pasangan_dengan_hati_warna_kulit_gelap-sedang:','zh':':情侣_中等深肤色:','ru':':влюбленная_пара_темный_тон_кожи:','tr':':kalpli_çift_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F491\U0001F3FC':{'en':':couple_with_heart_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mittelhelle_hautfarbe:','es':':pareja_enamorada_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_peau_moyennement_claire:','ja':':カップルとハート_やや薄い肌色:','ko':':연인_연한_갈색_피부:','pt':':casal_apaixonado_pele_morena_clara:','it':':coppia_con_cuore_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_پوست_روشن:','id':':pasangan_dengan_hati_warna_kulit_cerah-sedang:','zh':':情侣_中等浅肤色:','ru':':влюбленная_пара_светлый_тон_кожи:','tr':':kalpli_çift_orta_açık_cilt_tonu:','ar':':زوج_وقلب_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F491\U0001F3FD':{'en':':couple_with_heart_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_mittlere_hautfarbe:','es':':pareja_enamorada_tono_de_piel_medio:','fr':':couple_avec_cœur_peau_légèrement_mate:','ja':':カップルとハート_中間の肌色:','ko':':연인_갈색_피부:','pt':':casal_apaixonado_pele_morena:','it':':coppia_con_cuore_carnagione_olivastra:','fa':':زوج_عاشق_پوست_طلایی:','id':':pasangan_dengan_hati_warna_kulit_sedang:','zh':':情侣_中等肤色:','ru':':влюбленная_пара_средний_тон_кожи:','tr':':kalpli_çift_orta_cilt_tonu:','ar':':زوج_وقلب_بشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FB':{'en':':couple_with_heart_person_person_dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_erwachsener_erwachsener_dunkle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_oscuro_tono_de_piel_claro:','fr':':couple_avec_cœur_adulte_adulte_peau_foncée_et_peau_claire:','ja':':カップルとハート_大人_大人_濃い肌色_薄い肌色:','ko':':연인_사람_사람_검은색_피부_하얀_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_escura_e_pele_clara:','it':':coppia_con_cuore_persona_persona_carnagione_scura_e_carnagione_chiara:','fa':':زوج_عاشق_بزرگسال_بزرگسال_پوست_آبنوسی_و_پوست_سفید:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_gelap_warna_kulit_cerah:','zh':':情侣_成人成人较深肤色较浅肤色:','ru':':влюбленная_пара_взрослыи_взрослыи_очень_темныи_тон_кожи_очень_светлыи_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_koyu_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F9D1\U0001F3FB':{'en':':couple_with_heart_person_person_dark_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_person_person_dunkle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_oscuro_tono_de_piel_claro:','fr':':couple_avec_cœur_adulte_adulte_peau_foncée_et_peau_claire:','ja':':カップルとハート_大人_大人_濃い肌色_薄い肌色:','ko':':연인_사람_사람_검은색_피부_하얀_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_escura_e_pele_clara:','it':':coppia_con_cuore_persona_persona_carnagione_scura_e_carnagione_chiara:','fa':':زوج_عاشق_شخص_شخص_پوست_آبنوسی_و_پوست_سفید:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_gelap_warna_kulit_cerah:','zh':':情侣_成人成人较深肤色较浅肤色:','ru':':влюбленная_пара_взрослый_взрослый_очень_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_koyu_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FE':{'en':':couple_with_heart_person_person_dark_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_erwachsener_erwachsener_dunkle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_oscuro_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_foncée_et_peau_mate:','ja':':カップルとハート_大人_大人_濃い肌色_やや濃い肌色:','ko':':연인_사람_사람_검은색_피부_진한_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_escura_e_pele_morena_escura:','it':':coppia_con_cuore_persona_persona_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_بزرگسال_بزرگسال_پوست_آبنوسی_و_پوست_گندمی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':情侣_成人成人较深肤色中等深肤色:','ru':':влюбленная_пара_взрослыи_взрослыи_очень_темныи_тон_кожи_темныи_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F9D1\U0001F3FE':{'en':':couple_with_heart_person_person_dark_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_person_person_dunkle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_oscuro_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_foncée_et_peau_mate:','ja':':カップルとハート_大人_大人_濃い肌色_やや濃い肌色:','ko':':연인_사람_사람_검은색_피부_진한_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_escura_e_pele_morena_escura:','it':':coppia_con_cuore_persona_persona_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_شخص_شخص_پوست_آبنوسی_و_پوست_گندمی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':情侣_成人成人较深肤色中等深肤色:','ru':':влюбленная_пара_взрослый_взрослый_очень_темный_тон_кожи_темный_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FC':{'en':':couple_with_heart_person_person_dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_erwachsener_erwachsener_dunkle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_oscuro_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_foncée_et_peau_moyennement_claire:','ja':':カップルとハート_大人_大人_濃い肌色_やや薄い肌色:','ko':':연인_사람_사람_검은색_피부_연한_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_escura_e_pele_morena_clara:','it':':coppia_con_cuore_persona_persona_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_بزرگسال_بزرگسال_پوست_آبنوسی_و_پوست_روشن:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':情侣_成人成人较深肤色中等浅肤色:','ru':':влюбленная_пара_взрослыи_взрослыи_очень_темныи_тон_кожи_светлыи_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F9D1\U0001F3FC':{'en':':couple_with_heart_person_person_dark_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_person_person_dunkle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_oscuro_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_foncée_et_peau_moyennement_claire:','ja':':カップルとハート_大人_大人_濃い肌色_やや薄い肌色:','ko':':연인_사람_사람_검은색_피부_연한_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_escura_e_pele_morena_clara:','it':':coppia_con_cuore_persona_persona_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_شخص_شخص_پوست_آبنوسی_و_پوست_روشن:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':情侣_成人成人较深肤色中等浅肤色:','ru':':влюбленная_пара_взрослый_взрослый_очень_темный_тон_кожи_светлый_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FD':{'en':':couple_with_heart_person_person_dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_erwachsener_erwachsener_dunkle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_oscuro_tono_de_piel_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_foncée_et_peau_légèrement_mate:','ja':':カップルとハート_大人_大人_濃い肌色_中間の肌色:','ko':':연인_사람_사람_검은색_피부_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_escura_e_pele_morena:','it':':coppia_con_cuore_persona_persona_carnagione_scura_e_carnagione_olivastra:','fa':':زوج_عاشق_بزرگسال_بزرگسال_پوست_آبنوسی_و_پوست_طلایی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_gelap_warna_kulit_sedang:','zh':':情侣_成人成人较深肤色中等肤色:','ru':':влюбленная_пара_взрослыи_взрослыи_очень_темныи_тон_кожи_среднии_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_koyu_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F9D1\U0001F3FD':{'en':':couple_with_heart_person_person_dark_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_person_person_dunkle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_oscuro_tono_de_piel_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_foncée_et_peau_légèrement_mate:','ja':':カップルとハート_大人_大人_濃い肌色_中間の肌色:','ko':':연인_사람_사람_검은색_피부_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_escura_e_pele_morena:','it':':coppia_con_cuore_persona_persona_carnagione_scura_e_carnagione_olivastra:','fa':':زوج_عاشق_شخص_شخص_پوست_آبنوسی_و_پوست_طلایی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_gelap_warna_kulit_sedang:','zh':':情侣_成人成人较深肤色中等肤色:','ru':':влюбленная_пара_взрослый_взрослый_очень_темный_тон_кожи_средний_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_koyu_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FF':{'en':':couple_with_heart_person_person_light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_erwachsener_erwachsener_helle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_claro_tono_de_piel_oscuro:','fr':':couple_avec_cœur_adulte_adulte_peau_claire_et_peau_foncée:','ja':':カップルとハート_大人_大人_薄い肌色_濃い肌色:','ko':':연인_사람_사람_하얀_피부_검은색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_clara_e_pele_escura:','it':':coppia_con_cuore_persona_persona_carnagione_chiara_e_carnagione_scura:','fa':':زوج_عاشق_بزرگسال_بزرگسال_پوست_سفید_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_cerah_warna_kulit_gelap:','zh':':情侣_成人成人较浅肤色较深肤色:','ru':':влюбленная_пара_взрослыи_взрослыи_очень_светлыи_тон_кожи_очень_темныи_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_açık_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F9D1\U0001F3FF':{'en':':couple_with_heart_person_person_light_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_person_person_helle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_claro_tono_de_piel_oscuro:','fr':':couple_avec_cœur_adulte_adulte_peau_claire_et_peau_foncée:','ja':':カップルとハート_大人_大人_薄い肌色_濃い肌色:','ko':':연인_사람_사람_하얀_피부_검은색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_clara_e_pele_escura:','it':':coppia_con_cuore_persona_persona_carnagione_chiara_e_carnagione_scura:','fa':':زوج_عاشق_شخص_شخص_پوست_سفید_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_cerah_warna_kulit_gelap:','zh':':情侣_成人成人较浅肤色较深肤色:','ru':':влюбленная_пара_взрослый_взрослый_очень_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_açık_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FE':{'en':':couple_with_heart_person_person_light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_erwachsener_erwachsener_helle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_claro_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_claire_et_peau_mate:','ja':':カップルとハート_大人_大人_薄い肌色_やや濃い肌色:','ko':':연인_사람_사람_하얀_피부_진한_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_clara_e_pele_morena_escura:','it':':coppia_con_cuore_persona_persona_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_بزرگسال_بزرگسال_پوست_سفید_و_پوست_گندمی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':情侣_成人成人较浅肤色中等深肤色:','ru':':влюбленная_пара_взрослыи_взрослыи_очень_светлыи_тон_кожи_темныи_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F9D1\U0001F3FE':{'en':':couple_with_heart_person_person_light_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_person_person_helle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_claro_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_claire_et_peau_mate:','ja':':カップルとハート_大人_大人_薄い肌色_やや濃い肌色:','ko':':연인_사람_사람_하얀_피부_진한_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_clara_e_pele_morena_escura:','it':':coppia_con_cuore_persona_persona_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_شخص_شخص_پوست_سفید_و_پوست_گندمی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':情侣_成人成人较浅肤色中等深肤色:','ru':':влюбленная_пара_взрослый_взрослый_очень_светлый_тон_кожи_темный_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FC':{'en':':couple_with_heart_person_person_light_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_erwachsener_erwachsener_helle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_claro_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_claire_et_peau_moyennement_claire:','ja':':カップルとハート_大人_大人_薄い肌色_やや薄い肌色:','ko':':연인_사람_사람_하얀_피부_연한_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_clara_e_pele_morena_clara:','it':':coppia_con_cuore_persona_persona_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_بزرگسال_بزرگسال_پوست_سفید_و_پوست_روشن:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':情侣_成人成人较浅肤色中等浅肤色:','ru':':влюбленная_пара_взрослыи_взрослыи_очень_светлыи_тон_кожи_светлыи_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F9D1\U0001F3FC':{'en':':couple_with_heart_person_person_light_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_person_person_helle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_claro_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_claire_et_peau_moyennement_claire:','ja':':カップルとハート_大人_大人_薄い肌色_やや薄い肌色:','ko':':연인_사람_사람_하얀_피부_연한_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_clara_e_pele_morena_clara:','it':':coppia_con_cuore_persona_persona_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_شخص_شخص_پوست_سفید_و_پوست_روشن:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':情侣_成人成人较浅肤色中等浅肤色:','ru':':влюбленная_пара_взрослый_взрослый_очень_светлый_тон_кожи_светлый_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FD':{'en':':couple_with_heart_person_person_light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_erwachsener_erwachsener_helle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_claro_tono_de_piel_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_claire_et_peau_légèrement_mate:','ja':':カップルとハート_大人_大人_薄い肌色_中間の肌色:','ko':':연인_사람_사람_하얀_피부_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_clara_e_pele_morena:','it':':coppia_con_cuore_persona_persona_carnagione_chiara_e_carnagione_olivastra:','fa':':زوج_عاشق_بزرگسال_بزرگسال_پوست_سفید_و_پوست_طلایی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_cerah_warna_kulit_sedang:','zh':':情侣_成人成人较浅肤色中等肤色:','ru':':влюбленная_пара_взрослыи_взрослыи_очень_светлыи_тон_кожи_среднии_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_açık_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F9D1\U0001F3FD':{'en':':couple_with_heart_person_person_light_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_person_person_helle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_claro_tono_de_piel_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_claire_et_peau_légèrement_mate:','ja':':カップルとハート_大人_大人_薄い肌色_中間の肌色:','ko':':연인_사람_사람_하얀_피부_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_clara_e_pele_morena:','it':':coppia_con_cuore_persona_persona_carnagione_chiara_e_carnagione_olivastra:','fa':':زوج_عاشق_شخص_شخص_پوست_سفید_و_پوست_طلایی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_cerah_warna_kulit_sedang:','zh':':情侣_成人成人较浅肤色中等肤色:','ru':':влюбленная_пара_взрослый_взрослый_очень_светлый_тон_кожи_средний_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_açık_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FF':{'en':':couple_with_heart_person_person_medium-dark_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_erwachsener_erwachsener_mitteldunkle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_oscuro_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_adulte_adulte_peau_mate_et_peau_foncée:','ja':':カップルとハート_大人_大人_やや濃い肌色_濃い肌色:','ko':':연인_사람_사람_진한_갈색_피부_검은색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_escura_e_pele_escura:','it':':coppia_con_cuore_persona_persona_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':زوج_عاشق_بزرگسال_بزرگسال_پوست_گندمی_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':情侣_成人成人中等深肤色较深肤色:','ru':':влюбленная_пара_взрослыи_взрослыи_темныи_тон_кожи_очень_темныи_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F9D1\U0001F3FF':{'en':':couple_with_heart_person_person_medium-dark_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_person_person_mitteldunkle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_oscuro_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_adulte_adulte_peau_mate_et_peau_foncée:','ja':':カップルとハート_大人_大人_やや濃い肌色_濃い肌色:','ko':':연인_사람_사람_진한_갈색_피부_검은색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_escura_e_pele_escura:','it':':coppia_con_cuore_persona_persona_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':زوج_عاشق_شخص_شخص_پوست_گندمی_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':情侣_成人成人中等深肤色较深肤色:','ru':':влюбленная_пара_взрослый_взрослый_темный_тон_кожи_очень_темный_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FB':{'en':':couple_with_heart_person_person_medium-dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_erwachsener_erwachsener_mitteldunkle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_oscuro_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_adulte_adulte_peau_mate_et_peau_claire:','ja':':カップルとハート_大人_大人_やや濃い肌色_薄い肌色:','ko':':연인_사람_사람_진한_갈색_피부_하얀_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_escura_e_pele_clara:','it':':coppia_con_cuore_persona_persona_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':زوج_عاشق_بزرگسال_بزرگسال_پوست_گندمی_و_پوست_سفید:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':情侣_成人成人中等深肤色较浅肤色:','ru':':влюбленная_пара_взрослыи_взрослыи_темныи_тон_кожи_очень_светлыи_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F9D1\U0001F3FB':{'en':':couple_with_heart_person_person_medium-dark_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_person_person_mitteldunkle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_oscuro_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_adulte_adulte_peau_mate_et_peau_claire:','ja':':カップルとハート_大人_大人_やや濃い肌色_薄い肌色:','ko':':연인_사람_사람_진한_갈색_피부_하얀_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_escura_e_pele_clara:','it':':coppia_con_cuore_persona_persona_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':زوج_عاشق_شخص_شخص_پوست_گندمی_و_پوست_سفید:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':情侣_成人成人中等深肤色较浅肤色:','ru':':влюбленная_пара_взрослый_взрослый_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FC':{'en':':couple_with_heart_person_person_medium-dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_erwachsener_erwachsener_mitteldunkle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_oscuro_medio_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_mate_et_peau_moyennement_claire:','ja':':カップルとハート_大人_大人_やや濃い肌色_やや薄い肌色:','ko':':연인_사람_사람_진한_갈색_피부_연한_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_escura_e_pele_morena_clara:','it':':coppia_con_cuore_persona_persona_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_بزرگسال_بزرگسال_پوست_گندمی_و_پوست_روشن:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':情侣_成人成人中等深肤色中等浅肤色:','ru':':влюбленная_пара_взрослыи_взрослыи_темныи_тон_кожи_светлыи_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F9D1\U0001F3FC':{'en':':couple_with_heart_person_person_medium-dark_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_person_person_mitteldunkle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_oscuro_medio_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_mate_et_peau_moyennement_claire:','ja':':カップルとハート_大人_大人_やや濃い肌色_やや薄い肌色:','ko':':연인_사람_사람_진한_갈색_피부_연한_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_escura_e_pele_morena_clara:','it':':coppia_con_cuore_persona_persona_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_شخص_شخص_پوست_گندمی_و_پوست_روشن:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':情侣_成人成人中等深肤色中等浅肤色:','ru':':влюбленная_пара_взрослый_взрослый_темный_тон_кожи_светлый_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FD':{'en':':couple_with_heart_person_person_medium-dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_erwachsener_erwachsener_mitteldunkle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_oscuro_medio_tono_de_piel_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_mate_et_peau_légèrement_mate:','ja':':カップルとハート_大人_大人_やや濃い肌色_中間の肌色:','ko':':연인_사람_사람_진한_갈색_피부_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_escura_e_pele_morena:','it':':coppia_con_cuore_persona_persona_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':زوج_عاشق_بزرگسال_بزرگسال_پوست_گندمی_و_پوست_طلایی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':情侣_成人成人中等深肤色中等肤色:','ru':':влюбленная_пара_взрослыи_взрослыи_темныи_тон_кожи_среднии_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F9D1\U0001F3FD':{'en':':couple_with_heart_person_person_medium-dark_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_person_person_mitteldunkle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_oscuro_medio_tono_de_piel_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_mate_et_peau_légèrement_mate:','ja':':カップルとハート_大人_大人_やや濃い肌色_中間の肌色:','ko':':연인_사람_사람_진한_갈색_피부_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_escura_e_pele_morena:','it':':coppia_con_cuore_persona_persona_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':زوج_عاشق_شخص_شخص_پوست_گندمی_و_پوست_طلایی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':情侣_成人成人中等深肤色中等肤色:','ru':':влюбленная_пара_взрослый_взрослый_темный_тон_кожи_средний_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FF':{'en':':couple_with_heart_person_person_medium-light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_erwachsener_erwachsener_mittelhelle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_claro_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_adulte_adulte_peau_moyennement_claire_et_peau_foncée:','ja':':カップルとハート_大人_大人_やや薄い肌色_濃い肌色:','ko':':연인_사람_사람_연한_갈색_피부_검은색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_clara_e_pele_escura:','it':':coppia_con_cuore_persona_persona_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':زوج_عاشق_بزرگسال_بزرگسال_پوست_روشن_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':情侣_成人成人中等浅肤色较深肤色:','ru':':влюбленная_пара_взрослыи_взрослыи_светлыи_тон_кожи_очень_темныи_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F9D1\U0001F3FF':{'en':':couple_with_heart_person_person_medium-light_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_person_person_mittelhelle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_claro_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_adulte_adulte_peau_moyennement_claire_et_peau_foncée:','ja':':カップルとハート_大人_大人_やや薄い肌色_濃い肌色:','ko':':연인_사람_사람_연한_갈색_피부_검은색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_clara_e_pele_escura:','it':':coppia_con_cuore_persona_persona_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':زوج_عاشق_شخص_شخص_پوست_روشن_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':情侣_成人成人中等浅肤色较深肤色:','ru':':влюбленная_пара_взрослый_взрослый_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FB':{'en':':couple_with_heart_person_person_medium-light_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_erwachsener_erwachsener_mittelhelle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_claro_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_adulte_adulte_peau_moyennement_claire_et_peau_claire:','ja':':カップルとハート_大人_大人_やや薄い肌色_薄い肌色:','ko':':연인_사람_사람_연한_갈색_피부_하얀_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_clara_e_pele_clara:','it':':coppia_con_cuore_persona_persona_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':زوج_عاشق_بزرگسال_بزرگسال_پوست_روشن_و_پوست_سفید:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':情侣_成人成人中等浅肤色较浅肤色:','ru':':влюбленная_пара_взрослыи_взрослыи_светлыи_тон_кожи_очень_светлыи_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F9D1\U0001F3FB':{'en':':couple_with_heart_person_person_medium-light_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_person_person_mittelhelle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_claro_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_adulte_adulte_peau_moyennement_claire_et_peau_claire:','ja':':カップルとハート_大人_大人_やや薄い肌色_薄い肌色:','ko':':연인_사람_사람_연한_갈색_피부_하얀_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_clara_e_pele_clara:','it':':coppia_con_cuore_persona_persona_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':زوج_عاشق_شخص_شخص_پوست_روشن_و_پوست_سفید:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':情侣_成人成人中等浅肤色较浅肤色:','ru':':влюбленная_пара_взрослый_взрослый_светлый_тон_кожи_очень_светлый_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FE':{'en':':couple_with_heart_person_person_medium-light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_erwachsener_erwachsener_mittelhelle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_claro_medio_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_moyennement_claire_et_peau_mate:','ja':':カップルとハート_大人_大人_やや薄い肌色_やや濃い肌色:','ko':':연인_사람_사람_연한_갈색_피부_진한_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_clara_e_pele_morena_escura:','it':':coppia_con_cuore_persona_persona_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_بزرگسال_بزرگسال_پوست_روشن_و_پوست_گندمی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':情侣_成人成人中等浅肤色中等深肤色:','ru':':влюбленная_пара_взрослыи_взрослыи_светлыи_тон_кожи_темныи_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F9D1\U0001F3FE':{'en':':couple_with_heart_person_person_medium-light_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_person_person_mittelhelle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_claro_medio_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_moyennement_claire_et_peau_mate:','ja':':カップルとハート_大人_大人_やや薄い肌色_やや濃い肌色:','ko':':연인_사람_사람_연한_갈색_피부_진한_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_clara_e_pele_morena_escura:','it':':coppia_con_cuore_persona_persona_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_شخص_شخص_پوست_روشن_و_پوست_گندمی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':情侣_成人成人中等浅肤色中等深肤色:','ru':':влюбленная_пара_взрослый_взрослый_светлый_тон_кожи_темный_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FD':{'en':':couple_with_heart_person_person_medium-light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_erwachsener_erwachsener_mittelhelle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_claro_medio_tono_de_piel_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':カップルとハート_大人_大人_やや薄い肌色_中間の肌色:','ko':':연인_사람_사람_연한_갈색_피부_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_clara_e_pele_morena:','it':':coppia_con_cuore_persona_persona_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':زوج_عاشق_بزرگسال_بزرگسال_پوست_روشن_و_پوست_طلایی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':情侣_成人成人中等浅肤色中等肤色:','ru':':влюбленная_пара_взрослыи_взрослыи_светлыи_тон_кожи_среднии_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F9D1\U0001F3FD':{'en':':couple_with_heart_person_person_medium-light_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_person_person_mittelhelle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_claro_medio_tono_de_piel_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':カップルとハート_大人_大人_やや薄い肌色_中間の肌色:','ko':':연인_사람_사람_연한_갈색_피부_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_clara_e_pele_morena:','it':':coppia_con_cuore_persona_persona_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':زوج_عاشق_شخص_شخص_پوست_روشن_و_پوست_طلایی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':情侣_成人成人中等浅肤色中等肤色:','ru':':влюбленная_пара_взрослый_взрослый_светлый_тон_кожи_средний_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FF':{'en':':couple_with_heart_person_person_medium_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_erwachsener_erwachsener_mittlere_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_adulte_adulte_peau_légèrement_mate_et_peau_foncée:','ja':':カップルとハート_大人_大人_中間の肌色_濃い肌色:','ko':':연인_사람_사람_갈색_피부_검은색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_e_pele_escura:','it':':coppia_con_cuore_persona_persona_carnagione_olivastra_e_carnagione_scura:','fa':':زوج_عاشق_بزرگسال_بزرگسال_پوست_طلایی_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_sedang_warna_kulit_gelap:','zh':':情侣_成人成人中等肤色较深肤色:','ru':':влюбленная_пара_взрослыи_взрослыи_среднии_тон_кожи_очень_темныи_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F9D1\U0001F3FF':{'en':':couple_with_heart_person_person_medium_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_person_person_mittlere_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_adulte_adulte_peau_légèrement_mate_et_peau_foncée:','ja':':カップルとハート_大人_大人_中間の肌色_濃い肌色:','ko':':연인_사람_사람_갈색_피부_검은색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_e_pele_escura:','it':':coppia_con_cuore_persona_persona_carnagione_olivastra_e_carnagione_scura:','fa':':زوج_عاشق_شخص_شخص_پوست_طلایی_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_sedang_warna_kulit_gelap:','zh':':情侣_成人成人中等肤色较深肤色:','ru':':влюбленная_пара_взрослый_взрослый_средний_тон_кожи_очень_темный_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FB':{'en':':couple_with_heart_person_person_medium_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_erwachsener_erwachsener_mittlere_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_adulte_adulte_peau_légèrement_mate_et_peau_claire:','ja':':カップルとハート_大人_大人_中間の肌色_薄い肌色:','ko':':연인_사람_사람_갈색_피부_하얀_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_e_pele_clara:','it':':coppia_con_cuore_persona_persona_carnagione_olivastra_e_carnagione_chiara:','fa':':زوج_عاشق_بزرگسال_بزرگسال_پوست_طلایی_و_پوست_سفید:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_sedang_warna_kulit_cerah:','zh':':情侣_成人成人中等肤色较浅肤色:','ru':':влюбленная_пара_взрослыи_взрослыи_среднии_тон_кожи_очень_светлыи_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F9D1\U0001F3FB':{'en':':couple_with_heart_person_person_medium_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_person_person_mittlere_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_adulte_adulte_peau_légèrement_mate_et_peau_claire:','ja':':カップルとハート_大人_大人_中間の肌色_薄い肌色:','ko':':연인_사람_사람_갈색_피부_하얀_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_e_pele_clara:','it':':coppia_con_cuore_persona_persona_carnagione_olivastra_e_carnagione_chiara:','fa':':زوج_عاشق_شخص_شخص_پوست_طلایی_و_پوست_سفید:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_sedang_warna_kulit_cerah:','zh':':情侣_成人成人中等肤色较浅肤色:','ru':':влюбленная_пара_взрослый_взрослый_средний_тон_кожи_очень_светлый_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FE':{'en':':couple_with_heart_person_person_medium_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_erwachsener_erwachsener_mittlere_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_medio_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_légèrement_mate_et_peau_mate:','ja':':カップルとハート_大人_大人_中間の肌色_やや濃い肌色:','ko':':연인_사람_사람_갈색_피부_진한_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_e_pele_morena_escura:','it':':coppia_con_cuore_persona_persona_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_بزرگسال_بزرگسال_پوست_طلایی_و_پوست_گندمی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':情侣_成人成人中等肤色中等深肤色:','ru':':влюбленная_пара_взрослыи_взрослыи_среднии_тон_кожи_темныи_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F9D1\U0001F3FE':{'en':':couple_with_heart_person_person_medium_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_person_person_mittlere_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_medio_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_légèrement_mate_et_peau_mate:','ja':':カップルとハート_大人_大人_中間の肌色_やや濃い肌色:','ko':':연인_사람_사람_갈색_피부_진한_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_e_pele_morena_escura:','it':':coppia_con_cuore_persona_persona_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_شخص_شخص_پوست_طلایی_و_پوست_گندمی:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':情侣_成人成人中等肤色中等深肤色:','ru':':влюбленная_пара_взрослый_взрослый_средний_тон_кожи_темный_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F9D1\U0001F3FC':{'en':':couple_with_heart_person_person_medium_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_erwachsener_erwachsener_mittlere_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_medio_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':カップルとハート_大人_大人_中間の肌色_やや薄い肌色:','ko':':연인_사람_사람_갈색_피부_연한_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_e_pele_morena_clara:','it':':coppia_con_cuore_persona_persona_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_بزرگسال_بزرگسال_پوست_طلایی_و_پوست_روشن:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':情侣_成人成人中等肤色中等浅肤色:','ru':':влюбленная_пара_взрослыи_взрослыи_среднии_тон_кожи_светлыи_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F9D1\U0001F3FC':{'en':':couple_with_heart_person_person_medium_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_person_person_mittlere_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_persona_adulta_persona_adulta_tono_de_piel_medio_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_adulte_adulte_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':カップルとハート_大人_大人_中間の肌色_やや薄い肌色:','ko':':연인_사람_사람_갈색_피부_연한_갈색_피부:','pt':':casal_apaixonado_pessoa_pessoa_pele_morena_e_pele_morena_clara:','it':':coppia_con_cuore_persona_persona_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_شخص_شخص_پوست_طلایی_و_پوست_روشن:','id':':pasangan_dengan_hati_dewasa_dewasa_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':情侣_成人成人中等肤色中等浅肤色:','ru':':влюбленная_пара_взрослый_взрослый_средний_тон_кожи_светлый_тон_кожи:','tr':':kalpli_çift_yetişkin_yetişkin_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_بالغ_وبالغ_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468':{'en':':couple_with_heart_woman_man:','status':fully_qualified,'E':2,'de':':liebespaar_frau,_mann:','es':':pareja_enamorada_mujer_y_hombre:','fr':':couple_avec_cœur_femme_et_homme:','ja':':カップルとハート_女性_男性:','ko':':연인_여자_남자:','pt':':casal_apaixonado_mulher_e_homem:','it':':coppia_con_cuore_donna_e_uomo:','fa':':زوج_عاشق_زن_مرد:','id':':pasangan_dengan_hati_wanita_pria:','zh':':情侣_女人男人:','ru':':влюбленная_пара_женщина_мужчина:','tr':':kalpli_çift_kadın_erkek:','ar':':زوج_وقلب_سيدة_ورجل:'},'\U0001F469\U0000200D\U00002764\U0000200D\U0001F468':{'en':':couple_with_heart_woman_man:','status':minimally_qualified,'E':2,'de':':liebespaar_frau_mann:','es':':pareja_enamorada_mujer_y_hombre:','fr':':couple_avec_cœur_femme_et_homme:','ja':':カップルとハート_女性_男性:','ko':':연인_여자_남자:','pt':':casal_apaixonado_mulher_e_homem:','it':':coppia_con_cuore_donna_e_uomo:','fa':':زوج_عاشق_زن_مرد:','id':':pasangan_dengan_hati_wanita_pria:','zh':':情侣_女人男人:','ru':':влюбленная_пара_женщина_мужчина:','tr':':kalpli_çift_kadın_erkek:','ar':':زوج_وقلب_سيدة_ورجل:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FF':{'en':':couple_with_heart_woman_man_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_und_dunkle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_oscuro:','fr':':couple_avec_cœur_femme_homme_et_peau_foncée:','ja':':カップルとハート_女性_男性_濃い肌色:','ko':':연인_여자_남자_검은색_피부:','pt':':casal_apaixonado_mulher_homem_e_pele_escura:','it':':coppia_con_cuore_donna_uomo_e_carnagione_scura:','fa':':زوج_عاشق_زن_مرد_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_gelap:','zh':':情侣_女人男人较深肤色:','ru':':влюбленная_пара_женщина_мужчина_очень_темныи_тон_кожи:','tr':':kalpli_çift_kadın_erkek_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FF':{'en':':couple_with_heart_woman_man_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_und_dunkle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_oscuro:','fr':':couple_avec_cœur_femme_homme_et_peau_foncée:','ja':':カップルとハート_女性_男性_濃い肌色:','ko':':연인_여자_남자_검은색_피부:','pt':':casal_apaixonado_mulher_homem_e_pele_escura:','it':':coppia_con_cuore_donna_uomo_e_carnagione_scura:','fa':':زوج_عاشق_زن_مرد_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_gelap:','zh':':情侣_女人男人较深肤色:','ru':':влюбленная_пара_женщина_мужчина_очень_темный_тон_кожи:','tr':':kalpli_çift_kadın_erkek_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FB':{'en':':couple_with_heart_woman_man_dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_dunkle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_oscuro_tono_de_piel_claro:','fr':':couple_avec_cœur_femme_homme_peau_foncée_et_peau_claire:','ja':':カップルとハート_女性_男性_濃い肌色_薄い肌色:','ko':':연인_여자_남자_검은색_피부_하얀_피부:','pt':':casal_apaixonado_mulher_homem_pele_escura_e_pele_clara:','it':':coppia_con_cuore_donna_uomo_carnagione_scura_e_carnagione_chiara:','fa':':زوج_عاشق_زن_مرد_پوست_آبنوسی_و_پوست_سفید:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_gelap_warna_kulit_cerah:','zh':':情侣_女人男人较深肤色较浅肤色:','ru':':влюбленная_пара_женщина_мужчина_очень_темныи_тон_кожи_очень_светлыи_тон_кожи:','tr':':kalpli_çift_kadın_erkek_koyu_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FB':{'en':':couple_with_heart_woman_man_dark_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_dunkle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_oscuro_tono_de_piel_claro:','fr':':couple_avec_cœur_femme_homme_peau_foncée_et_peau_claire:','ja':':カップルとハート_女性_男性_濃い肌色_薄い肌色:','ko':':연인_여자_남자_검은색_피부_하얀_피부:','pt':':casal_apaixonado_mulher_homem_pele_escura_e_pele_clara:','it':':coppia_con_cuore_donna_uomo_carnagione_scura_e_carnagione_chiara:','fa':':زوج_عاشق_زن_مرد_پوست_آبنوسی_و_پوست_سفید:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_gelap_warna_kulit_cerah:','zh':':情侣_女人男人较深肤色较浅肤色:','ru':':влюбленная_пара_женщина_мужчина_очень_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':kalpli_çift_kadın_erkek_koyu_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FE':{'en':':couple_with_heart_woman_man_dark_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_dunkle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_oscuro_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_femme_homme_peau_foncée_et_peau_mate:','ja':':カップルとハート_女性_男性_濃い肌色_やや濃い肌色:','ko':':연인_여자_남자_검은색_피부_진한_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_escura_e_pele_morena_escura:','it':':coppia_con_cuore_donna_uomo_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_زن_مرد_پوست_آبنوسی_و_پوست_گندمی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':情侣_女人男人较深肤色中等深肤色:','ru':':влюбленная_пара_женщина_мужчина_очень_темныи_тон_кожи_темныи_тон_кожи:','tr':':kalpli_çift_kadın_erkek_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FE':{'en':':couple_with_heart_woman_man_dark_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_dunkle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_oscuro_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_femme_homme_peau_foncée_et_peau_mate:','ja':':カップルとハート_女性_男性_濃い肌色_やや濃い肌色:','ko':':연인_여자_남자_검은색_피부_진한_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_escura_e_pele_morena_escura:','it':':coppia_con_cuore_donna_uomo_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_زن_مرد_پوست_آبنوسی_و_پوست_گندمی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':情侣_女人男人较深肤色中等深肤色:','ru':':влюбленная_пара_женщина_мужчина_очень_темный_тон_кожи_темный_тон_кожи:','tr':':kalpli_çift_kadın_erkek_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FC':{'en':':couple_with_heart_woman_man_dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_dunkle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_oscuro_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_femme_homme_peau_foncée_et_peau_moyennement_claire:','ja':':カップルとハート_女性_男性_濃い肌色_やや薄い肌色:','ko':':연인_여자_남자_검은색_피부_연한_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_escura_e_pele_morena_clara:','it':':coppia_con_cuore_donna_uomo_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_زن_مرد_پوست_آبنوسی_و_پوست_روشن:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':情侣_女人男人较深肤色中等浅肤色:','ru':':влюбленная_пара_женщина_мужчина_очень_темныи_тон_кожи_светлыи_тон_кожи:','tr':':kalpli_çift_kadın_erkek_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FC':{'en':':couple_with_heart_woman_man_dark_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_dunkle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_oscuro_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_femme_homme_peau_foncée_et_peau_moyennement_claire:','ja':':カップルとハート_女性_男性_濃い肌色_やや薄い肌色:','ko':':연인_여자_남자_검은색_피부_연한_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_escura_e_pele_morena_clara:','it':':coppia_con_cuore_donna_uomo_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_زن_مرد_پوست_آبنوسی_و_پوست_روشن:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':情侣_女人男人较深肤色中等浅肤色:','ru':':влюбленная_пара_женщина_мужчина_очень_темный_тон_кожи_светлый_тон_кожи:','tr':':kalpli_çift_kadın_erkek_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FD':{'en':':couple_with_heart_woman_man_dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_dunkle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_oscuro_tono_de_piel_medio:','fr':':couple_avec_cœur_femme_homme_peau_foncée_et_peau_légèrement_mate:','ja':':カップルとハート_女性_男性_濃い肌色_中間の肌色:','ko':':연인_여자_남자_검은색_피부_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_escura_e_pele_morena:','it':':coppia_con_cuore_donna_uomo_carnagione_scura_e_carnagione_olivastra:','fa':':زوج_عاشق_زن_مرد_پوست_آبنوسی_و_پوست_طلایی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_gelap_warna_kulit_sedang:','zh':':情侣_女人男人较深肤色中等肤色:','ru':':влюбленная_пара_женщина_мужчина_очень_темныи_тон_кожи_среднии_тон_кожи:','tr':':kalpli_çift_kadın_erkek_koyu_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FD':{'en':':couple_with_heart_woman_man_dark_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_dunkle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_oscuro_tono_de_piel_medio:','fr':':couple_avec_cœur_femme_homme_peau_foncée_et_peau_légèrement_mate:','ja':':カップルとハート_女性_男性_濃い肌色_中間の肌色:','ko':':연인_여자_남자_검은색_피부_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_escura_e_pele_morena:','it':':coppia_con_cuore_donna_uomo_carnagione_scura_e_carnagione_olivastra:','fa':':زوج_عاشق_زن_مرد_پوست_آبنوسی_و_پوست_طلایی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_gelap_warna_kulit_sedang:','zh':':情侣_女人男人较深肤色中等肤色:','ru':':влюбленная_пара_женщина_мужчина_очень_темный_тон_кожи_средний_тон_кожи:','tr':':kalpli_çift_kadın_erkek_koyu_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FB':{'en':':couple_with_heart_woman_man_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_und_helle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_claro:','fr':':couple_avec_cœur_femme_homme_et_peau_claire:','ja':':カップルとハート_女性_男性_薄い肌色:','ko':':연인_여자_남자_하얀_피부:','pt':':casal_apaixonado_mulher_homem_e_pele_clara:','it':':coppia_con_cuore_donna_uomo_e_carnagione_chiara:','fa':':زوج_عاشق_زن_مرد_و_پوست_سفید:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_cerah:','zh':':情侣_女人男人较浅肤色:','ru':':влюбленная_пара_женщина_мужчина_очень_светлыи_тон_кожи:','tr':':kalpli_çift_kadın_erkek_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FB':{'en':':couple_with_heart_woman_man_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_und_helle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_claro:','fr':':couple_avec_cœur_femme_homme_et_peau_claire:','ja':':カップルとハート_女性_男性_薄い肌色:','ko':':연인_여자_남자_하얀_피부:','pt':':casal_apaixonado_mulher_homem_e_pele_clara:','it':':coppia_con_cuore_donna_uomo_e_carnagione_chiara:','fa':':زوج_عاشق_زن_مرد_و_پوست_سفید:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_cerah:','zh':':情侣_女人男人较浅肤色:','ru':':влюбленная_пара_женщина_мужчина_очень_светлый_тон_кожи:','tr':':kalpli_çift_kadın_erkek_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FF':{'en':':couple_with_heart_woman_man_light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_helle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_claro_tono_de_piel_oscuro:','fr':':couple_avec_cœur_femme_homme_peau_claire_et_peau_foncée:','ja':':カップルとハート_女性_男性_薄い肌色_濃い肌色:','ko':':연인_여자_남자_하얀_피부_검은색_피부:','pt':':casal_apaixonado_mulher_homem_pele_clara_e_pele_escura:','it':':coppia_con_cuore_donna_uomo_carnagione_chiara_e_carnagione_scura:','fa':':زوج_عاشق_زن_مرد_پوست_سفید_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_cerah_warna_kulit_gelap:','zh':':情侣_女人男人较浅肤色较深肤色:','ru':':влюбленная_пара_женщина_мужчина_очень_светлыи_тон_кожи_очень_темныи_тон_кожи:','tr':':kalpli_çift_kadın_erkek_açık_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FF':{'en':':couple_with_heart_woman_man_light_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_helle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_claro_tono_de_piel_oscuro:','fr':':couple_avec_cœur_femme_homme_peau_claire_et_peau_foncée:','ja':':カップルとハート_女性_男性_薄い肌色_濃い肌色:','ko':':연인_여자_남자_하얀_피부_검은색_피부:','pt':':casal_apaixonado_mulher_homem_pele_clara_e_pele_escura:','it':':coppia_con_cuore_donna_uomo_carnagione_chiara_e_carnagione_scura:','fa':':زوج_عاشق_زن_مرد_پوست_سفید_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_cerah_warna_kulit_gelap:','zh':':情侣_女人男人较浅肤色较深肤色:','ru':':влюбленная_пара_женщина_мужчина_очень_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':kalpli_çift_kadın_erkek_açık_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FE':{'en':':couple_with_heart_woman_man_light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_helle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_claro_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_femme_homme_peau_claire_et_peau_mate:','ja':':カップルとハート_女性_男性_薄い肌色_やや濃い肌色:','ko':':연인_여자_남자_하얀_피부_진한_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_clara_e_pele_morena_escura:','it':':coppia_con_cuore_donna_uomo_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_زن_مرد_پوست_سفید_و_پوست_گندمی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':情侣_女人男人较浅肤色中等深肤色:','ru':':влюбленная_пара_женщина_мужчина_очень_светлыи_тон_кожи_темныи_тон_кожи:','tr':':kalpli_çift_kadın_erkek_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FE':{'en':':couple_with_heart_woman_man_light_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_helle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_claro_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_femme_homme_peau_claire_et_peau_mate:','ja':':カップルとハート_女性_男性_薄い肌色_やや濃い肌色:','ko':':연인_여자_남자_하얀_피부_진한_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_clara_e_pele_morena_escura:','it':':coppia_con_cuore_donna_uomo_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_زن_مرد_پوست_سفید_و_پوست_گندمی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':情侣_女人男人较浅肤色中等深肤色:','ru':':влюбленная_пара_женщина_мужчина_очень_светлый_тон_кожи_темный_тон_кожи:','tr':':kalpli_çift_kadın_erkek_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FC':{'en':':couple_with_heart_woman_man_light_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_helle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_claro_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_femme_homme_peau_claire_et_peau_moyennement_claire:','ja':':カップルとハート_女性_男性_薄い肌色_やや薄い肌色:','ko':':연인_여자_남자_하얀_피부_연한_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_clara_e_pele_morena_clara:','it':':coppia_con_cuore_donna_uomo_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_زن_مرد_پوست_سفید_و_پوست_روشن:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':情侣_女人男人较浅肤色中等浅肤色:','ru':':влюбленная_пара_женщина_мужчина_очень_светлыи_тон_кожи_светлыи_тон_кожи:','tr':':kalpli_çift_kadın_erkek_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FC':{'en':':couple_with_heart_woman_man_light_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_helle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_claro_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_femme_homme_peau_claire_et_peau_moyennement_claire:','ja':':カップルとハート_女性_男性_薄い肌色_やや薄い肌色:','ko':':연인_여자_남자_하얀_피부_연한_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_clara_e_pele_morena_clara:','it':':coppia_con_cuore_donna_uomo_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_زن_مرد_پوست_سفید_و_پوست_روشن:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':情侣_女人男人较浅肤色中等浅肤色:','ru':':влюбленная_пара_женщина_мужчина_очень_светлый_тон_кожи_светлый_тон_кожи:','tr':':kalpli_çift_kadın_erkek_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FD':{'en':':couple_with_heart_woman_man_light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_helle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_claro_tono_de_piel_medio:','fr':':couple_avec_cœur_femme_homme_peau_claire_et_peau_légèrement_mate:','ja':':カップルとハート_女性_男性_薄い肌色_中間の肌色:','ko':':연인_여자_남자_하얀_피부_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_clara_e_pele_morena:','it':':coppia_con_cuore_donna_uomo_carnagione_chiara_e_carnagione_olivastra:','fa':':زوج_عاشق_زن_مرد_پوست_سفید_و_پوست_طلایی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_cerah_warna_kulit_sedang:','zh':':情侣_女人男人较浅肤色中等肤色:','ru':':влюбленная_пара_женщина_мужчина_очень_светлыи_тон_кожи_среднии_тон_кожи:','tr':':kalpli_çift_kadın_erkek_açık_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FD':{'en':':couple_with_heart_woman_man_light_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_helle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_claro_tono_de_piel_medio:','fr':':couple_avec_cœur_femme_homme_peau_claire_et_peau_légèrement_mate:','ja':':カップルとハート_女性_男性_薄い肌色_中間の肌色:','ko':':연인_여자_남자_하얀_피부_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_clara_e_pele_morena:','it':':coppia_con_cuore_donna_uomo_carnagione_chiara_e_carnagione_olivastra:','fa':':زوج_عاشق_زن_مرد_پوست_سفید_و_پوست_طلایی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_cerah_warna_kulit_sedang:','zh':':情侣_女人男人较浅肤色中等肤色:','ru':':влюбленная_пара_женщина_мужчина_очень_светлый_тон_кожи_средний_тон_кожи:','tr':':kalpli_çift_kadın_erkek_açık_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FE':{'en':':couple_with_heart_woman_man_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_und_mitteldunkle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_femme_homme_et_peau_mate:','ja':':カップルとハート_女性_男性_やや濃い肌色:','ko':':연인_여자_남자_진한_갈색_피부:','pt':':casal_apaixonado_mulher_homem_e_pele_morena_escura:','it':':coppia_con_cuore_donna_uomo_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_زن_مرد_و_پوست_گندمی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_gelap-sedang:','zh':':情侣_女人男人中等深肤色:','ru':':влюбленная_пара_женщина_мужчина_темныи_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FE':{'en':':couple_with_heart_woman_man_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_und_mitteldunkle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_femme_homme_et_peau_mate:','ja':':カップルとハート_女性_男性_やや濃い肌色:','ko':':연인_여자_남자_진한_갈색_피부:','pt':':casal_apaixonado_mulher_homem_e_pele_morena_escura:','it':':coppia_con_cuore_donna_uomo_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_زن_مرد_و_پوست_گندمی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_gelap-sedang:','zh':':情侣_女人男人中等深肤色:','ru':':влюбленная_пара_женщина_мужчина_темный_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FF':{'en':':couple_with_heart_woman_man_medium-dark_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_mitteldunkle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_oscuro_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_femme_homme_peau_mate_et_peau_foncée:','ja':':カップルとハート_女性_男性_やや濃い肌色_濃い肌色:','ko':':연인_여자_남자_진한_갈색_피부_검은색_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_escura_e_pele_escura:','it':':coppia_con_cuore_donna_uomo_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':زوج_عاشق_زن_مرد_پوست_گندمی_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':情侣_女人男人中等深肤色较深肤色:','ru':':влюбленная_пара_женщина_мужчина_темныи_тон_кожи_очень_темныи_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FF':{'en':':couple_with_heart_woman_man_medium-dark_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_mitteldunkle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_oscuro_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_femme_homme_peau_mate_et_peau_foncée:','ja':':カップルとハート_女性_男性_やや濃い肌色_濃い肌色:','ko':':연인_여자_남자_진한_갈색_피부_검은색_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_escura_e_pele_escura:','it':':coppia_con_cuore_donna_uomo_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':زوج_عاشق_زن_مرد_پوست_گندمی_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':情侣_女人男人中等深肤色较深肤色:','ru':':влюбленная_пара_женщина_мужчина_темный_тон_кожи_очень_темный_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FB':{'en':':couple_with_heart_woman_man_medium-dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_mitteldunkle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_oscuro_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_femme_homme_peau_mate_et_peau_claire:','ja':':カップルとハート_女性_男性_やや濃い肌色_薄い肌色:','ko':':연인_여자_남자_진한_갈색_피부_하얀_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_escura_e_pele_clara:','it':':coppia_con_cuore_donna_uomo_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':زوج_عاشق_زن_مرد_پوست_گندمی_و_پوست_سفید:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':情侣_女人男人中等深肤色较浅肤色:','ru':':влюбленная_пара_женщина_мужчина_темныи_тон_кожи_очень_светлыи_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FB':{'en':':couple_with_heart_woman_man_medium-dark_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_mitteldunkle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_oscuro_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_femme_homme_peau_mate_et_peau_claire:','ja':':カップルとハート_女性_男性_やや濃い肌色_薄い肌色:','ko':':연인_여자_남자_진한_갈색_피부_하얀_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_escura_e_pele_clara:','it':':coppia_con_cuore_donna_uomo_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':زوج_عاشق_زن_مرد_پوست_گندمی_و_پوست_سفید:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':情侣_女人男人中等深肤色较浅肤色:','ru':':влюбленная_пара_женщина_мужчина_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FC':{'en':':couple_with_heart_woman_man_medium-dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_mitteldunkle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_oscuro_medio_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_femme_homme_peau_mate_et_peau_moyennement_claire:','ja':':カップルとハート_女性_男性_やや濃い肌色_やや薄い肌色:','ko':':연인_여자_남자_진한_갈색_피부_연한_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_escura_e_pele_morena_clara:','it':':coppia_con_cuore_donna_uomo_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_زن_مرد_پوست_گندمی_و_پوست_روشن:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':情侣_女人男人中等深肤色中等浅肤色:','ru':':влюбленная_пара_женщина_мужчина_темныи_тон_кожи_светлыи_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FC':{'en':':couple_with_heart_woman_man_medium-dark_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_mitteldunkle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_oscuro_medio_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_femme_homme_peau_mate_et_peau_moyennement_claire:','ja':':カップルとハート_女性_男性_やや濃い肌色_やや薄い肌色:','ko':':연인_여자_남자_진한_갈색_피부_연한_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_escura_e_pele_morena_clara:','it':':coppia_con_cuore_donna_uomo_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_زن_مرد_پوست_گندمی_و_پوست_روشن:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':情侣_女人男人中等深肤色中等浅肤色:','ru':':влюбленная_пара_женщина_мужчина_темный_тон_кожи_светлый_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FD':{'en':':couple_with_heart_woman_man_medium-dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_mitteldunkle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_oscuro_medio_tono_de_piel_medio:','fr':':couple_avec_cœur_femme_homme_peau_mate_et_peau_légèrement_mate:','ja':':カップルとハート_女性_男性_やや濃い肌色_中間の肌色:','ko':':연인_여자_남자_진한_갈색_피부_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_escura_e_pele_morena:','it':':coppia_con_cuore_donna_uomo_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':زوج_عاشق_زن_مرد_پوست_گندمی_و_پوست_طلایی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':情侣_女人男人中等深肤色中等肤色:','ru':':влюбленная_пара_женщина_мужчина_темныи_тон_кожи_среднии_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FD':{'en':':couple_with_heart_woman_man_medium-dark_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_mitteldunkle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_oscuro_medio_tono_de_piel_medio:','fr':':couple_avec_cœur_femme_homme_peau_mate_et_peau_légèrement_mate:','ja':':カップルとハート_女性_男性_やや濃い肌色_中間の肌色:','ko':':연인_여자_남자_진한_갈색_피부_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_escura_e_pele_morena:','it':':coppia_con_cuore_donna_uomo_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':زوج_عاشق_زن_مرد_پوست_گندمی_و_پوست_طلایی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':情侣_女人男人中等深肤色中等肤色:','ru':':влюбленная_пара_женщина_мужчина_темный_тон_кожи_средний_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FC':{'en':':couple_with_heart_woman_man_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_und_mittelhelle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_femme_homme_et_peau_moyennement_claire:','ja':':カップルとハート_女性_男性_やや薄い肌色:','ko':':연인_여자_남자_연한_갈색_피부:','pt':':casal_apaixonado_mulher_homem_e_pele_morena_clara:','it':':coppia_con_cuore_donna_uomo_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_زن_مرد_و_پوست_روشن:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_cerah-sedang:','zh':':情侣_女人男人中等浅肤色:','ru':':влюбленная_пара_женщина_мужчина_светлыи_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FC':{'en':':couple_with_heart_woman_man_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_und_mittelhelle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_femme_homme_et_peau_moyennement_claire:','ja':':カップルとハート_女性_男性_やや薄い肌色:','ko':':연인_여자_남자_연한_갈색_피부:','pt':':casal_apaixonado_mulher_homem_e_pele_morena_clara:','it':':coppia_con_cuore_donna_uomo_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_زن_مرد_و_پوست_روشن:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_cerah-sedang:','zh':':情侣_女人男人中等浅肤色:','ru':':влюбленная_пара_женщина_мужчина_светлый_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FF':{'en':':couple_with_heart_woman_man_medium-light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_mittelhelle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_claro_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_femme_homme_peau_moyennement_claire_et_peau_foncée:','ja':':カップルとハート_女性_男性_やや薄い肌色_濃い肌色:','ko':':연인_여자_남자_연한_갈색_피부_검은색_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_clara_e_pele_escura:','it':':coppia_con_cuore_donna_uomo_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':زوج_عاشق_زن_مرد_پوست_روشن_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':情侣_女人男人中等浅肤色较深肤色:','ru':':влюбленная_пара_женщина_мужчина_светлыи_тон_кожи_очень_темныи_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FF':{'en':':couple_with_heart_woman_man_medium-light_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_mittelhelle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_claro_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_femme_homme_peau_moyennement_claire_et_peau_foncée:','ja':':カップルとハート_女性_男性_やや薄い肌色_濃い肌色:','ko':':연인_여자_남자_연한_갈색_피부_검은색_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_clara_e_pele_escura:','it':':coppia_con_cuore_donna_uomo_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':زوج_عاشق_زن_مرد_پوست_روشن_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':情侣_女人男人中等浅肤色较深肤色:','ru':':влюбленная_пара_женщина_мужчина_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FB':{'en':':couple_with_heart_woman_man_medium-light_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_mittelhelle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_claro_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_femme_homme_peau_moyennement_claire_et_peau_claire:','ja':':カップルとハート_女性_男性_やや薄い肌色_薄い肌色:','ko':':연인_여자_남자_연한_갈색_피부_하얀_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_clara_e_pele_clara:','it':':coppia_con_cuore_donna_uomo_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':زوج_عاشق_زن_مرد_پوست_روشن_و_پوست_سفید:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':情侣_女人男人中等浅肤色较浅肤色:','ru':':влюбленная_пара_женщина_мужчина_светлыи_тон_кожи_очень_светлыи_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FB':{'en':':couple_with_heart_woman_man_medium-light_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_mittelhelle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_claro_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_femme_homme_peau_moyennement_claire_et_peau_claire:','ja':':カップルとハート_女性_男性_やや薄い肌色_薄い肌色:','ko':':연인_여자_남자_연한_갈색_피부_하얀_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_clara_e_pele_clara:','it':':coppia_con_cuore_donna_uomo_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':زوج_عاشق_زن_مرد_پوست_روشن_و_پوست_سفید:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':情侣_女人男人中等浅肤色较浅肤色:','ru':':влюбленная_пара_женщина_мужчина_светлый_тон_кожи_очень_светлый_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FE':{'en':':couple_with_heart_woman_man_medium-light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_mittelhelle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_claro_medio_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_femme_homme_peau_moyennement_claire_et_peau_mate:','ja':':カップルとハート_女性_男性_やや薄い肌色_やや濃い肌色:','ko':':연인_여자_남자_연한_갈색_피부_진한_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_clara_e_pele_morena_escura:','it':':coppia_con_cuore_donna_uomo_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_زن_مرد_پوست_روشن_و_پوست_گندمی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':情侣_女人男人中等浅肤色中等深肤色:','ru':':влюбленная_пара_женщина_мужчина_светлыи_тон_кожи_темныи_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FE':{'en':':couple_with_heart_woman_man_medium-light_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_mittelhelle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_claro_medio_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_femme_homme_peau_moyennement_claire_et_peau_mate:','ja':':カップルとハート_女性_男性_やや薄い肌色_やや濃い肌色:','ko':':연인_여자_남자_연한_갈색_피부_진한_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_clara_e_pele_morena_escura:','it':':coppia_con_cuore_donna_uomo_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_زن_مرد_پوست_روشن_و_پوست_گندمی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':情侣_女人男人中等浅肤色中等深肤色:','ru':':влюбленная_пара_женщина_мужчина_светлый_тон_кожи_темный_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FD':{'en':':couple_with_heart_woman_man_medium-light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_mittelhelle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_claro_medio_tono_de_piel_medio:','fr':':couple_avec_cœur_femme_homme_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':カップルとハート_女性_男性_やや薄い肌色_中間の肌色:','ko':':연인_여자_남자_연한_갈색_피부_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_clara_e_pele_morena:','it':':coppia_con_cuore_donna_uomo_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':زوج_عاشق_زن_مرد_پوست_روشن_و_پوست_طلایی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':情侣_女人男人中等浅肤色中等肤色:','ru':':влюбленная_пара_женщина_мужчина_светлыи_тон_кожи_среднии_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FD':{'en':':couple_with_heart_woman_man_medium-light_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_mittelhelle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_claro_medio_tono_de_piel_medio:','fr':':couple_avec_cœur_femme_homme_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':カップルとハート_女性_男性_やや薄い肌色_中間の肌色:','ko':':연인_여자_남자_연한_갈색_피부_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_clara_e_pele_morena:','it':':coppia_con_cuore_donna_uomo_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':زوج_عاشق_زن_مرد_پوست_روشن_و_پوست_طلایی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':情侣_女人男人中等浅肤色中等肤色:','ru':':влюбленная_пара_женщина_мужчина_светлый_тон_кожи_средний_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FD':{'en':':couple_with_heart_woman_man_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_und_mittlere_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_medio:','fr':':couple_avec_cœur_femme_homme_et_peau_légèrement_mate:','ja':':カップルとハート_女性_男性_中間の肌色:','ko':':연인_여자_남자_갈색_피부:','pt':':casal_apaixonado_mulher_homem_e_pele_morena:','it':':coppia_con_cuore_donna_uomo_e_carnagione_olivastra:','fa':':زوج_عاشق_زن_مرد_و_پوست_طلایی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_sedang:','zh':':情侣_女人男人中等肤色:','ru':':влюбленная_пара_женщина_мужчина_среднии_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FD':{'en':':couple_with_heart_woman_man_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_und_mittlere_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_medio:','fr':':couple_avec_cœur_femme_homme_et_peau_légèrement_mate:','ja':':カップルとハート_女性_男性_中間の肌色:','ko':':연인_여자_남자_갈색_피부:','pt':':casal_apaixonado_mulher_homem_e_pele_morena:','it':':coppia_con_cuore_donna_uomo_e_carnagione_olivastra:','fa':':زوج_عاشق_زن_مرد_و_پوست_طلایی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_sedang:','zh':':情侣_女人男人中等肤色:','ru':':влюбленная_пара_женщина_мужчина_средний_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FF':{'en':':couple_with_heart_woman_man_medium_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_mittlere_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_femme_homme_peau_légèrement_mate_et_peau_foncée:','ja':':カップルとハート_女性_男性_中間の肌色_濃い肌色:','ko':':연인_여자_남자_갈색_피부_검은색_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_e_pele_escura:','it':':coppia_con_cuore_donna_uomo_carnagione_olivastra_e_carnagione_scura:','fa':':زوج_عاشق_زن_مرد_پوست_طلایی_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_sedang_warna_kulit_gelap:','zh':':情侣_女人男人中等肤色较深肤色:','ru':':влюбленная_пара_женщина_мужчина_среднии_тон_кожи_очень_темныи_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FF':{'en':':couple_with_heart_woman_man_medium_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_mittlere_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_femme_homme_peau_légèrement_mate_et_peau_foncée:','ja':':カップルとハート_女性_男性_中間の肌色_濃い肌色:','ko':':연인_여자_남자_갈색_피부_검은색_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_e_pele_escura:','it':':coppia_con_cuore_donna_uomo_carnagione_olivastra_e_carnagione_scura:','fa':':زوج_عاشق_زن_مرد_پوست_طلایی_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_sedang_warna_kulit_gelap:','zh':':情侣_女人男人中等肤色较深肤色:','ru':':влюбленная_пара_женщина_мужчина_средний_тон_кожи_очень_темный_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FB':{'en':':couple_with_heart_woman_man_medium_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_mittlere_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_femme_homme_peau_légèrement_mate_et_peau_claire:','ja':':カップルとハート_女性_男性_中間の肌色_薄い肌色:','ko':':연인_여자_남자_갈색_피부_하얀_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_e_pele_clara:','it':':coppia_con_cuore_donna_uomo_carnagione_olivastra_e_carnagione_chiara:','fa':':زوج_عاشق_زن_مرد_پوست_طلایی_و_پوست_سفید:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_sedang_warna_kulit_cerah:','zh':':情侣_女人男人中等肤色较浅肤色:','ru':':влюбленная_пара_женщина_мужчина_среднии_тон_кожи_очень_светлыи_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FB':{'en':':couple_with_heart_woman_man_medium_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_mittlere_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_femme_homme_peau_légèrement_mate_et_peau_claire:','ja':':カップルとハート_女性_男性_中間の肌色_薄い肌色:','ko':':연인_여자_남자_갈색_피부_하얀_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_e_pele_clara:','it':':coppia_con_cuore_donna_uomo_carnagione_olivastra_e_carnagione_chiara:','fa':':زوج_عاشق_زن_مرد_پوست_طلایی_و_پوست_سفید:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_sedang_warna_kulit_cerah:','zh':':情侣_女人男人中等肤色较浅肤色:','ru':':влюбленная_пара_женщина_мужчина_средний_тон_кожи_очень_светлый_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FE':{'en':':couple_with_heart_woman_man_medium_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_mittlere_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_medio_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_femme_homme_peau_légèrement_mate_et_peau_mate:','ja':':カップルとハート_女性_男性_中間の肌色_やや濃い肌色:','ko':':연인_여자_남자_갈색_피부_진한_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_e_pele_morena_escura:','it':':coppia_con_cuore_donna_uomo_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_زن_مرد_پوست_طلایی_و_پوست_گندمی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':情侣_女人男人中等肤色中等深肤色:','ru':':влюбленная_пара_женщина_мужчина_среднии_тон_кожи_темныи_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FE':{'en':':couple_with_heart_woman_man_medium_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_mittlere_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_medio_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_femme_homme_peau_légèrement_mate_et_peau_mate:','ja':':カップルとハート_女性_男性_中間の肌色_やや濃い肌色:','ko':':연인_여자_남자_갈색_피부_진한_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_e_pele_morena_escura:','it':':coppia_con_cuore_donna_uomo_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_زن_مرد_پوست_طلایی_و_پوست_گندمی:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':情侣_女人男人中等肤色中等深肤色:','ru':':влюбленная_пара_женщина_мужчина_средний_тон_кожи_темный_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F468\U0001F3FC':{'en':':couple_with_heart_woman_man_medium_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_mann_mittlere_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_medio_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_femme_homme_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':カップルとハート_女性_男性_中間の肌色_やや薄い肌色:','ko':':연인_여자_남자_갈색_피부_연한_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_e_pele_morena_clara:','it':':coppia_con_cuore_donna_uomo_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_زن_مرد_پوست_طلایی_و_پوست_روشن:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':情侣_女人男人中等肤色中等浅肤色:','ru':':влюбленная_пара_женщина_мужчина_среднии_тон_кожи_светлыи_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F468\U0001F3FC':{'en':':couple_with_heart_woman_man_medium_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_mann_mittlere_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_mujer_hombre_tono_de_piel_medio_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_femme_homme_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':カップルとハート_女性_男性_中間の肌色_やや薄い肌色:','ko':':연인_여자_남자_갈색_피부_연한_갈색_피부:','pt':':casal_apaixonado_mulher_homem_pele_morena_e_pele_morena_clara:','it':':coppia_con_cuore_donna_uomo_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_زن_مرد_پوست_طلایی_و_پوست_روشن:','id':':pasangan_dengan_hati_wanita_pria_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':情侣_女人男人中等肤色中等浅肤色:','ru':':влюбленная_пара_женщина_мужчина_средний_тон_кожи_светлый_тон_кожи:','tr':':kalpli_çift_kadın_erkek_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469':{'en':':couple_with_heart_woman_woman:','status':fully_qualified,'E':2,'de':':liebespaar_frau,_frau:','es':':pareja_enamorada_mujer_y_mujer:','fr':':couple_avec_cœur_femme_et_femme:','ja':':カップルとハート_女性_女性:','ko':':연인_여자_여자:','pt':':casal_apaixonado_mulher_e_mulher:','it':':coppia_con_cuore_donna_e_donna:','fa':':زوج_عاشق_زن_زن:','id':':pasangan_dengan_hati_wanita_wanita:','zh':':情侣_女人女人:','ru':':влюбленная_пара_женщина_женщина:','tr':':kalpli_çift_kadın_kadın:','ar':':زوج_وقلب_سيدة_وسيدة:'},'\U0001F469\U0000200D\U00002764\U0000200D\U0001F469':{'en':':couple_with_heart_woman_woman:','status':minimally_qualified,'E':2,'de':':liebespaar_frau_frau:','es':':pareja_enamorada_mujer_y_mujer:','fr':':couple_avec_cœur_femme_et_femme:','ja':':カップルとハート_女性_女性:','ko':':연인_여자_여자:','pt':':casal_apaixonado_mulher_e_mulher:','it':':coppia_con_cuore_donna_e_donna:','fa':':زوج_عاشق_زن_زن:','id':':pasangan_dengan_hati_wanita_wanita:','zh':':情侣_女人女人:','ru':':влюбленная_пара_женщина_женщина:','tr':':kalpli_çift_kadın_kadın:','ar':':زوج_وقلب_سيدة_وسيدة:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FF':{'en':':couple_with_heart_woman_woman_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_und_dunkle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_oscuro:','fr':':couple_avec_cœur_femme_femme_et_peau_foncée:','ja':':カップルとハート_女性_女性_濃い肌色:','ko':':연인_여자_여자_검은색_피부:','pt':':casal_apaixonado_mulher_mulher_e_pele_escura:','it':':coppia_con_cuore_donna_donna_e_carnagione_scura:','fa':':زوج_عاشق_زن_زن_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_gelap:','zh':':情侣_女人女人较深肤色:','ru':':влюбленная_пара_женщина_женщина_очень_темныи_тон_кожи:','tr':':kalpli_çift_kadın_kadın_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FF':{'en':':couple_with_heart_woman_woman_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_und_dunkle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_oscuro:','fr':':couple_avec_cœur_femme_femme_et_peau_foncée:','ja':':カップルとハート_女性_女性_濃い肌色:','ko':':연인_여자_여자_검은색_피부:','pt':':casal_apaixonado_mulher_mulher_e_pele_escura:','it':':coppia_con_cuore_donna_donna_e_carnagione_scura:','fa':':زوج_عاشق_زن_زن_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_gelap:','zh':':情侣_女人女人较深肤色:','ru':':влюбленная_пара_женщина_женщина_очень_темный_тон_кожи:','tr':':kalpli_çift_kadın_kadın_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FB':{'en':':couple_with_heart_woman_woman_dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_dunkle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_oscuro_tono_de_piel_claro:','fr':':couple_avec_cœur_femme_femme_peau_foncée_et_peau_claire:','ja':':カップルとハート_女性_女性_濃い肌色_薄い肌色:','ko':':연인_여자_여자_검은색_피부_하얀_피부:','pt':':casal_apaixonado_mulher_mulher_pele_escura_e_pele_clara:','it':':coppia_con_cuore_donna_donna_carnagione_scura_e_carnagione_chiara:','fa':':زوج_عاشق_زن_زن_پوست_آبنوسی_و_پوست_سفید:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_gelap_warna_kulit_cerah:','zh':':情侣_女人女人较深肤色较浅肤色:','ru':':влюбленная_пара_женщина_женщина_очень_темныи_тон_кожи_очень_светлыи_тон_кожи:','tr':':kalpli_çift_kadın_kadın_koyu_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FB':{'en':':couple_with_heart_woman_woman_dark_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_dunkle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_oscuro_tono_de_piel_claro:','fr':':couple_avec_cœur_femme_femme_peau_foncée_et_peau_claire:','ja':':カップルとハート_女性_女性_濃い肌色_薄い肌色:','ko':':연인_여자_여자_검은색_피부_하얀_피부:','pt':':casal_apaixonado_mulher_mulher_pele_escura_e_pele_clara:','it':':coppia_con_cuore_donna_donna_carnagione_scura_e_carnagione_chiara:','fa':':زوج_عاشق_زن_زن_پوست_آبنوسی_و_پوست_سفید:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_gelap_warna_kulit_cerah:','zh':':情侣_女人女人较深肤色较浅肤色:','ru':':влюбленная_пара_женщина_женщина_очень_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':kalpli_çift_kadın_kadın_koyu_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FE':{'en':':couple_with_heart_woman_woman_dark_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_dunkle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_oscuro_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_femme_femme_peau_foncée_et_peau_mate:','ja':':カップルとハート_女性_女性_濃い肌色_やや濃い肌色:','ko':':연인_여자_여자_검은색_피부_진한_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_escura_e_pele_morena_escura:','it':':coppia_con_cuore_donna_donna_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_زن_زن_پوست_آبنوسی_و_پوست_گندمی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':情侣_女人女人较深肤色中等深肤色:','ru':':влюбленная_пара_женщина_женщина_очень_темныи_тон_кожи_темныи_тон_кожи:','tr':':kalpli_çift_kadın_kadın_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FE':{'en':':couple_with_heart_woman_woman_dark_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_dunkle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_oscuro_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_femme_femme_peau_foncée_et_peau_mate:','ja':':カップルとハート_女性_女性_濃い肌色_やや濃い肌色:','ko':':연인_여자_여자_검은색_피부_진한_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_escura_e_pele_morena_escura:','it':':coppia_con_cuore_donna_donna_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_زن_زن_پوست_آبنوسی_و_پوست_گندمی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':情侣_女人女人较深肤色中等深肤色:','ru':':влюбленная_пара_женщина_женщина_очень_темный_тон_кожи_темный_тон_кожи:','tr':':kalpli_çift_kadın_kadın_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FC':{'en':':couple_with_heart_woman_woman_dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_dunkle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_oscuro_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_femme_femme_peau_foncée_et_peau_moyennement_claire:','ja':':カップルとハート_女性_女性_濃い肌色_やや薄い肌色:','ko':':연인_여자_여자_검은색_피부_연한_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_escura_e_pele_morena_clara:','it':':coppia_con_cuore_donna_donna_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_زن_زن_پوست_آبنوسی_و_پوست_روشن:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':情侣_女人女人较深肤色中等浅肤色:','ru':':влюбленная_пара_женщина_женщина_очень_темныи_тон_кожи_светлыи_тон_кожи:','tr':':kalpli_çift_kadın_kadın_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FC':{'en':':couple_with_heart_woman_woman_dark_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_dunkle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_oscuro_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_femme_femme_peau_foncée_et_peau_moyennement_claire:','ja':':カップルとハート_女性_女性_濃い肌色_やや薄い肌色:','ko':':연인_여자_여자_검은색_피부_연한_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_escura_e_pele_morena_clara:','it':':coppia_con_cuore_donna_donna_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_زن_زن_پوست_آبنوسی_و_پوست_روشن:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':情侣_女人女人较深肤色中等浅肤色:','ru':':влюбленная_пара_женщина_женщина_очень_темный_тон_кожи_светлый_тон_кожи:','tr':':kalpli_çift_kadın_kadın_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FD':{'en':':couple_with_heart_woman_woman_dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_dunkle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_oscuro_tono_de_piel_medio:','fr':':couple_avec_cœur_femme_femme_peau_foncée_et_peau_légèrement_mate:','ja':':カップルとハート_女性_女性_濃い肌色_中間の肌色:','ko':':연인_여자_여자_검은색_피부_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_escura_e_pele_morena:','it':':coppia_con_cuore_donna_donna_carnagione_scura_e_carnagione_olivastra:','fa':':زوج_عاشق_زن_زن_پوست_آبنوسی_و_پوست_طلایی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_gelap_warna_kulit_sedang:','zh':':情侣_女人女人较深肤色中等肤色:','ru':':влюбленная_пара_женщина_женщина_очень_темныи_тон_кожи_среднии_тон_кожи:','tr':':kalpli_çift_kadın_kadın_koyu_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FD':{'en':':couple_with_heart_woman_woman_dark_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_dunkle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_oscuro_tono_de_piel_medio:','fr':':couple_avec_cœur_femme_femme_peau_foncée_et_peau_légèrement_mate:','ja':':カップルとハート_女性_女性_濃い肌色_中間の肌色:','ko':':연인_여자_여자_검은색_피부_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_escura_e_pele_morena:','it':':coppia_con_cuore_donna_donna_carnagione_scura_e_carnagione_olivastra:','fa':':زوج_عاشق_زن_زن_پوست_آبنوسی_و_پوست_طلایی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_gelap_warna_kulit_sedang:','zh':':情侣_女人女人较深肤色中等肤色:','ru':':влюбленная_пара_женщина_женщина_очень_темный_тон_кожи_средний_тон_кожи:','tr':':kalpli_çift_kadın_kadın_koyu_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FB':{'en':':couple_with_heart_woman_woman_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_und_helle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_claro:','fr':':couple_avec_cœur_femme_femme_et_peau_claire:','ja':':カップルとハート_女性_女性_薄い肌色:','ko':':연인_여자_여자_하얀_피부:','pt':':casal_apaixonado_mulher_mulher_e_pele_clara:','it':':coppia_con_cuore_donna_donna_e_carnagione_chiara:','fa':':زوج_عاشق_زن_زن_و_پوست_سفید:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_cerah:','zh':':情侣_女人女人较浅肤色:','ru':':влюбленная_пара_женщина_женщина_очень_светлыи_тон_кожи:','tr':':kalpli_çift_kadın_kadın_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FB':{'en':':couple_with_heart_woman_woman_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_und_helle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_claro:','fr':':couple_avec_cœur_femme_femme_et_peau_claire:','ja':':カップルとハート_女性_女性_薄い肌色:','ko':':연인_여자_여자_하얀_피부:','pt':':casal_apaixonado_mulher_mulher_e_pele_clara:','it':':coppia_con_cuore_donna_donna_e_carnagione_chiara:','fa':':زوج_عاشق_زن_زن_و_پوست_سفید:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_cerah:','zh':':情侣_女人女人较浅肤色:','ru':':влюбленная_пара_женщина_женщина_очень_светлый_тон_кожи:','tr':':kalpli_çift_kadın_kadın_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FF':{'en':':couple_with_heart_woman_woman_light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_helle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_claro_tono_de_piel_oscuro:','fr':':couple_avec_cœur_femme_femme_peau_claire_et_peau_foncée:','ja':':カップルとハート_女性_女性_薄い肌色_濃い肌色:','ko':':연인_여자_여자_하얀_피부_검은색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_clara_e_pele_escura:','it':':coppia_con_cuore_donna_donna_carnagione_chiara_e_carnagione_scura:','fa':':زوج_عاشق_زن_زن_پوست_سفید_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_cerah_warna_kulit_gelap:','zh':':情侣_女人女人较浅肤色较深肤色:','ru':':влюбленная_пара_женщина_женщина_очень_светлыи_тон_кожи_очень_темныи_тон_кожи:','tr':':kalpli_çift_kadın_kadın_açık_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FF':{'en':':couple_with_heart_woman_woman_light_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_helle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_claro_tono_de_piel_oscuro:','fr':':couple_avec_cœur_femme_femme_peau_claire_et_peau_foncée:','ja':':カップルとハート_女性_女性_薄い肌色_濃い肌色:','ko':':연인_여자_여자_하얀_피부_검은색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_clara_e_pele_escura:','it':':coppia_con_cuore_donna_donna_carnagione_chiara_e_carnagione_scura:','fa':':زوج_عاشق_زن_زن_پوست_سفید_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_cerah_warna_kulit_gelap:','zh':':情侣_女人女人较浅肤色较深肤色:','ru':':влюбленная_пара_женщина_женщина_очень_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':kalpli_çift_kadın_kadın_açık_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FE':{'en':':couple_with_heart_woman_woman_light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_helle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_claro_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_femme_femme_peau_claire_et_peau_mate:','ja':':カップルとハート_女性_女性_薄い肌色_やや濃い肌色:','ko':':연인_여자_여자_하얀_피부_진한_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_clara_e_pele_morena_escura:','it':':coppia_con_cuore_donna_donna_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_زن_زن_پوست_سفید_و_پوست_گندمی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':情侣_女人女人较浅肤色中等深肤色:','ru':':влюбленная_пара_женщина_женщина_очень_светлыи_тон_кожи_темныи_тон_кожи:','tr':':kalpli_çift_kadın_kadın_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FE':{'en':':couple_with_heart_woman_woman_light_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_helle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_claro_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_femme_femme_peau_claire_et_peau_mate:','ja':':カップルとハート_女性_女性_薄い肌色_やや濃い肌色:','ko':':연인_여자_여자_하얀_피부_진한_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_clara_e_pele_morena_escura:','it':':coppia_con_cuore_donna_donna_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_زن_زن_پوست_سفید_و_پوست_گندمی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':情侣_女人女人较浅肤色中等深肤色:','ru':':влюбленная_пара_женщина_женщина_очень_светлый_тон_кожи_темный_тон_кожи:','tr':':kalpli_çift_kadın_kadın_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FC':{'en':':couple_with_heart_woman_woman_light_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_helle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_claro_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_femme_femme_peau_claire_et_peau_moyennement_claire:','ja':':カップルとハート_女性_女性_薄い肌色_やや薄い肌色:','ko':':연인_여자_여자_하얀_피부_연한_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_clara_e_pele_morena_clara:','it':':coppia_con_cuore_donna_donna_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_زن_زن_پوست_سفید_و_پوست_روشن:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':情侣_女人女人较浅肤色中等浅肤色:','ru':':влюбленная_пара_женщина_женщина_очень_светлыи_тон_кожи_светлыи_тон_кожи:','tr':':kalpli_çift_kadın_kadın_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FC':{'en':':couple_with_heart_woman_woman_light_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_helle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_claro_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_femme_femme_peau_claire_et_peau_moyennement_claire:','ja':':カップルとハート_女性_女性_薄い肌色_やや薄い肌色:','ko':':연인_여자_여자_하얀_피부_연한_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_clara_e_pele_morena_clara:','it':':coppia_con_cuore_donna_donna_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_زن_زن_پوست_سفید_و_پوست_روشن:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':情侣_女人女人较浅肤色中等浅肤色:','ru':':влюбленная_пара_женщина_женщина_очень_светлый_тон_кожи_светлый_тон_кожи:','tr':':kalpli_çift_kadın_kadın_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FD':{'en':':couple_with_heart_woman_woman_light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_helle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_claro_tono_de_piel_medio:','fr':':couple_avec_cœur_femme_femme_peau_claire_et_peau_légèrement_mate:','ja':':カップルとハート_女性_女性_薄い肌色_中間の肌色:','ko':':연인_여자_여자_하얀_피부_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_clara_e_pele_morena:','it':':coppia_con_cuore_donna_donna_carnagione_chiara_e_carnagione_olivastra:','fa':':زوج_عاشق_زن_زن_پوست_سفید_و_پوست_طلایی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_cerah_warna_kulit_sedang:','zh':':情侣_女人女人较浅肤色中等肤色:','ru':':влюбленная_пара_женщина_женщина_очень_светлыи_тон_кожи_среднии_тон_кожи:','tr':':kalpli_çift_kadın_kadın_açık_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FD':{'en':':couple_with_heart_woman_woman_light_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_helle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_claro_tono_de_piel_medio:','fr':':couple_avec_cœur_femme_femme_peau_claire_et_peau_légèrement_mate:','ja':':カップルとハート_女性_女性_薄い肌色_中間の肌色:','ko':':연인_여자_여자_하얀_피부_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_clara_e_pele_morena:','it':':coppia_con_cuore_donna_donna_carnagione_chiara_e_carnagione_olivastra:','fa':':زوج_عاشق_زن_زن_پوست_سفید_و_پوست_طلایی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_cerah_warna_kulit_sedang:','zh':':情侣_女人女人较浅肤色中等肤色:','ru':':влюбленная_пара_женщина_женщина_очень_светлый_тон_кожи_средний_тон_кожи:','tr':':kalpli_çift_kadın_kadın_açık_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FE':{'en':':couple_with_heart_woman_woman_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_und_mitteldunkle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_femme_femme_et_peau_mate:','ja':':カップルとハート_女性_女性_やや濃い肌色:','ko':':연인_여자_여자_진한_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_e_pele_morena_escura:','it':':coppia_con_cuore_donna_donna_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_زن_زن_و_پوست_گندمی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_gelap-sedang:','zh':':情侣_女人女人中等深肤色:','ru':':влюбленная_пара_женщина_женщина_темныи_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FE':{'en':':couple_with_heart_woman_woman_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_und_mitteldunkle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_femme_femme_et_peau_mate:','ja':':カップルとハート_女性_女性_やや濃い肌色:','ko':':연인_여자_여자_진한_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_e_pele_morena_escura:','it':':coppia_con_cuore_donna_donna_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_زن_زن_و_پوست_گندمی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_gelap-sedang:','zh':':情侣_女人女人中等深肤色:','ru':':влюбленная_пара_женщина_женщина_темный_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FF':{'en':':couple_with_heart_woman_woman_medium-dark_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_mitteldunkle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_oscuro_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_femme_femme_peau_mate_et_peau_foncée:','ja':':カップルとハート_女性_女性_やや濃い肌色_濃い肌色:','ko':':연인_여자_여자_진한_갈색_피부_검은색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_escura_e_pele_escura:','it':':coppia_con_cuore_donna_donna_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':زوج_عاشق_زن_زن_پوست_گندمی_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':情侣_女人女人中等深肤色较深肤色:','ru':':влюбленная_пара_женщина_женщина_темныи_тон_кожи_очень_темныи_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FF':{'en':':couple_with_heart_woman_woman_medium-dark_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_mitteldunkle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_oscuro_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_femme_femme_peau_mate_et_peau_foncée:','ja':':カップルとハート_女性_女性_やや濃い肌色_濃い肌色:','ko':':연인_여자_여자_진한_갈색_피부_검은색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_escura_e_pele_escura:','it':':coppia_con_cuore_donna_donna_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':زوج_عاشق_زن_زن_پوست_گندمی_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':情侣_女人女人中等深肤色较深肤色:','ru':':влюбленная_пара_женщина_женщина_темный_тон_кожи_очень_темный_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FB':{'en':':couple_with_heart_woman_woman_medium-dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_mitteldunkle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_oscuro_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_femme_femme_peau_mate_et_peau_claire:','ja':':カップルとハート_女性_女性_やや濃い肌色_薄い肌色:','ko':':연인_여자_여자_진한_갈색_피부_하얀_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_escura_e_pele_clara:','it':':coppia_con_cuore_donna_donna_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':زوج_عاشق_زن_زن_پوست_گندمی_و_پوست_سفید:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':情侣_女人女人中等深肤色较浅肤色:','ru':':влюбленная_пара_женщина_женщина_темныи_тон_кожи_очень_светлыи_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FB':{'en':':couple_with_heart_woman_woman_medium-dark_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_mitteldunkle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_oscuro_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_femme_femme_peau_mate_et_peau_claire:','ja':':カップルとハート_女性_女性_やや濃い肌色_薄い肌色:','ko':':연인_여자_여자_진한_갈색_피부_하얀_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_escura_e_pele_clara:','it':':coppia_con_cuore_donna_donna_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':زوج_عاشق_زن_زن_پوست_گندمی_و_پوست_سفید:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':情侣_女人女人中等深肤色较浅肤色:','ru':':влюбленная_пара_женщина_женщина_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FC':{'en':':couple_with_heart_woman_woman_medium-dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_mitteldunkle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_oscuro_medio_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_femme_femme_peau_mate_et_peau_moyennement_claire:','ja':':カップルとハート_女性_女性_やや濃い肌色_やや薄い肌色:','ko':':연인_여자_여자_진한_갈색_피부_연한_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_escura_e_pele_morena_clara:','it':':coppia_con_cuore_donna_donna_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_زن_زن_پوست_گندمی_و_پوست_روشن:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':情侣_女人女人中等深肤色中等浅肤色:','ru':':влюбленная_пара_женщина_женщина_темныи_тон_кожи_светлыи_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FC':{'en':':couple_with_heart_woman_woman_medium-dark_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_mitteldunkle_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_oscuro_medio_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_femme_femme_peau_mate_et_peau_moyennement_claire:','ja':':カップルとハート_女性_女性_やや濃い肌色_やや薄い肌色:','ko':':연인_여자_여자_진한_갈색_피부_연한_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_escura_e_pele_morena_clara:','it':':coppia_con_cuore_donna_donna_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_زن_زن_پوست_گندمی_و_پوست_روشن:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':情侣_女人女人中等深肤色中等浅肤色:','ru':':влюбленная_пара_женщина_женщина_темный_тон_кожи_светлый_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FD':{'en':':couple_with_heart_woman_woman_medium-dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_mitteldunkle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_oscuro_medio_tono_de_piel_medio:','fr':':couple_avec_cœur_femme_femme_peau_mate_et_peau_légèrement_mate:','ja':':カップルとハート_女性_女性_やや濃い肌色_中間の肌色:','ko':':연인_여자_여자_진한_갈색_피부_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_escura_e_pele_morena:','it':':coppia_con_cuore_donna_donna_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':زوج_عاشق_زن_زن_پوست_گندمی_و_پوست_طلایی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':情侣_女人女人中等深肤色中等肤色:','ru':':влюбленная_пара_женщина_женщина_темныи_тон_кожи_среднии_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FD':{'en':':couple_with_heart_woman_woman_medium-dark_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_mitteldunkle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_oscuro_medio_tono_de_piel_medio:','fr':':couple_avec_cœur_femme_femme_peau_mate_et_peau_légèrement_mate:','ja':':カップルとハート_女性_女性_やや濃い肌色_中間の肌色:','ko':':연인_여자_여자_진한_갈색_피부_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_escura_e_pele_morena:','it':':coppia_con_cuore_donna_donna_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':زوج_عاشق_زن_زن_پوست_گندمی_و_پوست_طلایی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':情侣_女人女人中等深肤色中等肤色:','ru':':влюбленная_пара_женщина_женщина_темный_тон_кожи_средний_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FC':{'en':':couple_with_heart_woman_woman_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_und_mittelhelle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_femme_femme_et_peau_moyennement_claire:','ja':':カップルとハート_女性_女性_やや薄い肌色:','ko':':연인_여자_여자_연한_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_e_pele_morena_clara:','it':':coppia_con_cuore_donna_donna_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_زن_زن_و_پوست_روشن:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_cerah-sedang:','zh':':情侣_女人女人中等浅肤色:','ru':':влюбленная_пара_женщина_женщина_светлыи_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FC':{'en':':couple_with_heart_woman_woman_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_und_mittelhelle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_femme_femme_et_peau_moyennement_claire:','ja':':カップルとハート_女性_女性_やや薄い肌色:','ko':':연인_여자_여자_연한_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_e_pele_morena_clara:','it':':coppia_con_cuore_donna_donna_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_زن_زن_و_پوست_روشن:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_cerah-sedang:','zh':':情侣_女人女人中等浅肤色:','ru':':влюбленная_пара_женщина_женщина_светлый_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FF':{'en':':couple_with_heart_woman_woman_medium-light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_mittelhelle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_claro_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_femme_femme_peau_moyennement_claire_et_peau_foncée:','ja':':カップルとハート_女性_女性_やや薄い肌色_濃い肌色:','ko':':연인_여자_여자_연한_갈색_피부_검은색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_clara_e_pele_escura:','it':':coppia_con_cuore_donna_donna_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':زوج_عاشق_زن_زن_پوست_روشن_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':情侣_女人女人中等浅肤色较深肤色:','ru':':влюбленная_пара_женщина_женщина_светлыи_тон_кожи_очень_темныи_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FF':{'en':':couple_with_heart_woman_woman_medium-light_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_mittelhelle_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_claro_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_femme_femme_peau_moyennement_claire_et_peau_foncée:','ja':':カップルとハート_女性_女性_やや薄い肌色_濃い肌色:','ko':':연인_여자_여자_연한_갈색_피부_검은색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_clara_e_pele_escura:','it':':coppia_con_cuore_donna_donna_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':زوج_عاشق_زن_زن_پوست_روشن_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':情侣_女人女人中等浅肤色较深肤色:','ru':':влюбленная_пара_женщина_женщина_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FB':{'en':':couple_with_heart_woman_woman_medium-light_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_mittelhelle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_claro_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_femme_femme_peau_moyennement_claire_et_peau_claire:','ja':':カップルとハート_女性_女性_やや薄い肌色_薄い肌色:','ko':':연인_여자_여자_연한_갈색_피부_하얀_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_clara_e_pele_clara:','it':':coppia_con_cuore_donna_donna_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':زوج_عاشق_زن_زن_پوست_روشن_و_پوست_سفید:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':情侣_女人女人中等浅肤色较浅肤色:','ru':':влюбленная_пара_женщина_женщина_светлыи_тон_кожи_очень_светлыи_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FB':{'en':':couple_with_heart_woman_woman_medium-light_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_mittelhelle_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_claro_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_femme_femme_peau_moyennement_claire_et_peau_claire:','ja':':カップルとハート_女性_女性_やや薄い肌色_薄い肌色:','ko':':연인_여자_여자_연한_갈색_피부_하얀_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_clara_e_pele_clara:','it':':coppia_con_cuore_donna_donna_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':زوج_عاشق_زن_زن_پوست_روشن_و_پوست_سفید:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':情侣_女人女人中等浅肤色较浅肤色:','ru':':влюбленная_пара_женщина_женщина_светлый_тон_кожи_очень_светлый_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FE':{'en':':couple_with_heart_woman_woman_medium-light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_mittelhelle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_claro_medio_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_femme_femme_peau_moyennement_claire_et_peau_mate:','ja':':カップルとハート_女性_女性_やや薄い肌色_やや濃い肌色:','ko':':연인_여자_여자_연한_갈색_피부_진한_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_clara_e_pele_morena_escura:','it':':coppia_con_cuore_donna_donna_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_زن_زن_پوست_روشن_و_پوست_گندمی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':情侣_女人女人中等浅肤色中等深肤色:','ru':':влюбленная_пара_женщина_женщина_светлыи_тон_кожи_темныи_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FE':{'en':':couple_with_heart_woman_woman_medium-light_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_mittelhelle_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_claro_medio_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_femme_femme_peau_moyennement_claire_et_peau_mate:','ja':':カップルとハート_女性_女性_やや薄い肌色_やや濃い肌色:','ko':':연인_여자_여자_연한_갈색_피부_진한_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_clara_e_pele_morena_escura:','it':':coppia_con_cuore_donna_donna_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_زن_زن_پوست_روشن_و_پوست_گندمی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':情侣_女人女人中等浅肤色中等深肤色:','ru':':влюбленная_пара_женщина_женщина_светлый_тон_кожи_темный_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FD':{'en':':couple_with_heart_woman_woman_medium-light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_mittelhelle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_claro_medio_tono_de_piel_medio:','fr':':couple_avec_cœur_femme_femme_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':カップルとハート_女性_女性_やや薄い肌色_中間の肌色:','ko':':연인_여자_여자_연한_갈색_피부_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_clara_e_pele_morena:','it':':coppia_con_cuore_donna_donna_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':زوج_عاشق_زن_زن_پوست_روشن_و_پوست_طلایی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':情侣_女人女人中等浅肤色中等肤色:','ru':':влюбленная_пара_женщина_женщина_светлыи_тон_кожи_среднии_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FD':{'en':':couple_with_heart_woman_woman_medium-light_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_mittelhelle_hautfarbe,mittlere_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_claro_medio_tono_de_piel_medio:','fr':':couple_avec_cœur_femme_femme_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':カップルとハート_女性_女性_やや薄い肌色_中間の肌色:','ko':':연인_여자_여자_연한_갈색_피부_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_clara_e_pele_morena:','it':':coppia_con_cuore_donna_donna_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':زوج_عاشق_زن_زن_پوست_روشن_و_پوست_طلایی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':情侣_女人女人中等浅肤色中等肤色:','ru':':влюбленная_пара_женщина_женщина_светлый_тон_кожи_средний_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FD':{'en':':couple_with_heart_woman_woman_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_und_mittlere_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_medio:','fr':':couple_avec_cœur_femme_femme_et_peau_légèrement_mate:','ja':':カップルとハート_女性_女性_中間の肌色:','ko':':연인_여자_여자_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_e_pele_morena:','it':':coppia_con_cuore_donna_donna_e_carnagione_olivastra:','fa':':زوج_عاشق_زن_زن_و_پوست_طلایی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_sedang:','zh':':情侣_女人女人中等肤色:','ru':':влюбленная_пара_женщина_женщина_среднии_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FD':{'en':':couple_with_heart_woman_woman_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_und_mittlere_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_medio:','fr':':couple_avec_cœur_femme_femme_et_peau_légèrement_mate:','ja':':カップルとハート_女性_女性_中間の肌色:','ko':':연인_여자_여자_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_e_pele_morena:','it':':coppia_con_cuore_donna_donna_e_carnagione_olivastra:','fa':':زوج_عاشق_زن_زن_و_پوست_طلایی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_sedang:','zh':':情侣_女人女人中等肤色:','ru':':влюбленная_пара_женщина_женщина_средний_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FF':{'en':':couple_with_heart_woman_woman_medium_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_mittlere_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_femme_femme_peau_légèrement_mate_et_peau_foncée:','ja':':カップルとハート_女性_女性_中間の肌色_濃い肌色:','ko':':연인_여자_여자_갈색_피부_검은색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_e_pele_escura:','it':':coppia_con_cuore_donna_donna_carnagione_olivastra_e_carnagione_scura:','fa':':زوج_عاشق_زن_زن_پوست_طلایی_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_sedang_warna_kulit_gelap:','zh':':情侣_女人女人中等肤色较深肤色:','ru':':влюбленная_пара_женщина_женщина_среднии_тон_кожи_очень_темныи_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FF':{'en':':couple_with_heart_woman_woman_medium_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_mittlere_hautfarbe,dunkle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_medio_tono_de_piel_oscuro:','fr':':couple_avec_cœur_femme_femme_peau_légèrement_mate_et_peau_foncée:','ja':':カップルとハート_女性_女性_中間の肌色_濃い肌色:','ko':':연인_여자_여자_갈색_피부_검은색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_e_pele_escura:','it':':coppia_con_cuore_donna_donna_carnagione_olivastra_e_carnagione_scura:','fa':':زوج_عاشق_زن_زن_پوست_طلایی_و_پوست_آبنوسی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_sedang_warna_kulit_gelap:','zh':':情侣_女人女人中等肤色较深肤色:','ru':':влюбленная_пара_женщина_женщина_средний_тон_кожи_очень_темный_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_cilt_tonu_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FB':{'en':':couple_with_heart_woman_woman_medium_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_mittlere_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_femme_femme_peau_légèrement_mate_et_peau_claire:','ja':':カップルとハート_女性_女性_中間の肌色_薄い肌色:','ko':':연인_여자_여자_갈색_피부_하얀_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_e_pele_clara:','it':':coppia_con_cuore_donna_donna_carnagione_olivastra_e_carnagione_chiara:','fa':':زوج_عاشق_زن_زن_پوست_طلایی_و_پوست_سفید:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_sedang_warna_kulit_cerah:','zh':':情侣_女人女人中等肤色较浅肤色:','ru':':влюбленная_пара_женщина_женщина_среднии_тон_кожи_очень_светлыи_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FB':{'en':':couple_with_heart_woman_woman_medium_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_mittlere_hautfarbe,helle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_medio_tono_de_piel_claro:','fr':':couple_avec_cœur_femme_femme_peau_légèrement_mate_et_peau_claire:','ja':':カップルとハート_女性_女性_中間の肌色_薄い肌色:','ko':':연인_여자_여자_갈색_피부_하얀_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_e_pele_clara:','it':':coppia_con_cuore_donna_donna_carnagione_olivastra_e_carnagione_chiara:','fa':':زوج_عاشق_زن_زن_پوست_طلایی_و_پوست_سفید:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_sedang_warna_kulit_cerah:','zh':':情侣_女人女人中等肤色较浅肤色:','ru':':влюбленная_пара_женщина_женщина_средний_тон_кожи_очень_светлый_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_cilt_tonu_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FE':{'en':':couple_with_heart_woman_woman_medium_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_mittlere_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_medio_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_femme_femme_peau_légèrement_mate_et_peau_mate:','ja':':カップルとハート_女性_女性_中間の肌色_やや濃い肌色:','ko':':연인_여자_여자_갈색_피부_진한_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_e_pele_morena_escura:','it':':coppia_con_cuore_donna_donna_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_زن_زن_پوست_طلایی_و_پوست_گندمی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':情侣_女人女人中等肤色中等深肤色:','ru':':влюбленная_пара_женщина_женщина_среднии_тон_кожи_темныи_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FE':{'en':':couple_with_heart_woman_woman_medium_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_mittlere_hautfarbe,mitteldunkle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_medio_tono_de_piel_oscuro_medio:','fr':':couple_avec_cœur_femme_femme_peau_légèrement_mate_et_peau_mate:','ja':':カップルとハート_女性_女性_中間の肌色_やや濃い肌色:','ko':':연인_여자_여자_갈색_피부_진한_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_e_pele_morena_escura:','it':':coppia_con_cuore_donna_donna_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':زوج_عاشق_زن_زن_پوست_طلایی_و_پوست_گندمی:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':情侣_女人女人中等肤色中等深肤色:','ru':':влюбленная_пара_женщина_женщина_средний_тон_кожи_темный_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F469\U0001F3FC':{'en':':couple_with_heart_woman_woman_medium_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':liebespaar_frau_frau_mittlere_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_medio_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_femme_femme_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':カップルとハート_女性_女性_中間の肌色_やや薄い肌色:','ko':':연인_여자_여자_갈색_피부_연한_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_e_pele_morena_clara:','it':':coppia_con_cuore_donna_donna_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_زن_زن_پوست_طلایی_و_پوست_روشن:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':情侣_女人女人中等肤色中等浅肤色:','ru':':влюбленная_пара_женщина_женщина_среднии_тон_кожи_светлыи_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F469\U0001F3FC':{'en':':couple_with_heart_woman_woman_medium_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':liebespaar_frau_frau_mittlere_hautfarbe,mittelhelle_hautfarbe:','es':':pareja_enamorada_mujer_mujer_tono_de_piel_medio_tono_de_piel_claro_medio:','fr':':couple_avec_cœur_femme_femme_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':カップルとハート_女性_女性_中間の肌色_やや薄い肌色:','ko':':연인_여자_여자_갈색_피부_연한_갈색_피부:','pt':':casal_apaixonado_mulher_mulher_pele_morena_e_pele_morena_clara:','it':':coppia_con_cuore_donna_donna_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':زوج_عاشق_زن_زن_پوست_طلایی_و_پوست_روشن:','id':':pasangan_dengan_hati_wanita_wanita_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':情侣_女人女人中等肤色中等浅肤色:','ru':':влюбленная_пара_женщина_женщина_средний_тон_кожи_светлый_тон_кожи:','tr':':kalpli_çift_kadın_kadın_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':زوج_وقلب_سيدة_وسيدة_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F404':{'en':':cow:','status':fully_qualified,'E':1,'alias':[':cow2:'],'de':':kuh:','es':':vaca:','fr':':vache:','ja':':牝牛:','ko':':젖소:','pt':':vaca:','it':':mucca:','fa':':گاو_ماده:','id':':sapi:','zh':':奶牛:','ru':':корова:','tr':':inek:','ar':':بقرة:'},'\U0001F42E':{'en':':cow_face:','status':fully_qualified,'E':0.6,'alias':[':cow:'],'de':':kuhgesicht:','es':':cara_de_vaca:','fr':':tête_de_vache:','ja':':牛の顔:','ko':':소_얼굴:','pt':':rosto_de_vaca:','it':':muso_di_mucca:','fa':':صورت_گاو:','id':':wajah_sapi:','zh':':奶牛头:','ru':':морда_коровы:','tr':':inek_yüzü:','ar':':وجه_بقرة:'},'\U0001F920':{'en':':cowboy_hat_face:','status':fully_qualified,'E':3,'de':':gesicht_mit_cowboyhut:','es':':cara_con_sombrero_de_vaquero:','fr':':visage_avec_chapeau_de_cowboy:','ja':':カウボーイの顔:','ko':':카우보이_모자_쓴_얼굴:','pt':':rosto_com_chapéu_de_caubói:','it':':faccina_con_cappello_da_cowboy:','fa':':کلاه_کابوی:','id':':wajah_koboi:','zh':':牛仔帽脸:','ru':':в_ковбойской_шляпе:','tr':':kovboy_şapkalı_yüz:','ar':':وجه_بقبعة_راعي_بقر:'},'\U0001F980':{'en':':crab:','status':fully_qualified,'E':1,'de':':krebs:','es':':cangrejo:','fr':':crabe:','ja':':カニ:','ko':':꽃게:','pt':':caranguejo:','it':':granchio:','fa':':خرچنگ:','id':':kepiting:','zh':':蟹:','ru':':краб:','tr':':yengeç:','ar':':سرطان:'},'\U0001F58D\U0000FE0F':{'en':':crayon:','status':fully_qualified,'E':0.7,'alias':[':lower_left_crayon:'],'variant':True,'de':':wachsmalstift:','es':':lápiz_de_cera:','fr':':crayon_pastel:','ja':':クレヨン:','ko':':크레용:','pt':':giz_de_cera:','it':':pastello_a_cera:','fa':':مدادشمعی:','id':':krayon:','zh':':蜡笔:','ru':':восковои_мелок:','tr':':pastel_boya:','ar':':قلم_شمع_ملون:'},'\U0001F58D':{'en':':crayon:','status':unqualified,'E':0.7,'alias':[':lower_left_crayon:'],'variant':True,'de':':wachsmalstift:','es':':lápiz_de_cera:','fr':':crayon_pastel:','ja':':クレヨン:','ko':':크레용:','pt':':giz_de_cera:','it':':pastello_a_cera:','fa':':مدادشمعی:','id':':krayon:','zh':':蜡笔:','ru':':восковой_мелок:','tr':':pastel_boya:','ar':':قلم_شمع_ملون:'},'\U0001F4B3':{'en':':credit_card:','status':fully_qualified,'E':0.6,'variant':True,'de':':kreditkarte:','es':':tarjeta_de_crédito:','fr':':carte_bancaire:','ja':':クレジットカード:','ko':':신용카드:','pt':':cartão_de_crédito:','it':':carta_di_credito:','fa':':کارت_اعتباری:','id':':kartu_kredit:','zh':':信用卡:','ru':':кредитная_карта:','tr':':kredi_kartı:','ar':':بطاقة_ائتمان:'},'\U0001F319':{'en':':crescent_moon:','status':fully_qualified,'E':0.6,'de':':mondsichel:','es':':luna:','fr':':croissant_de_lune:','ja':':三日月:','ko':':초승달:','pt':':lua_crescente:','it':':spicchio_di_luna:','fa':':هلال_ماه:','id':':bulan_sabit:','zh':':弯月:','ru':':полумесяц:','tr':':hilal:','ar':':هلال:'},'\U0001F997':{'en':':cricket:','status':fully_qualified,'E':5,'de':':grille:','es':':grillo:','fr':':criquet:','ja':':バッタ:','ko':':귀뚜라미:','pt':':grilo:','it':':grillo:','fa':':جیرجیرک:','id':':jangkrik:','zh':':蟋蟀:','ru':':сверчок:','tr':':cırcır_böceği:','ar':':جدجد:'},'\U0001F3CF':{'en':':cricket_game:','status':fully_qualified,'E':1,'alias':[':cricket_bat_and_ball:'],'de':':kricket:','es':':críquet:','fr':':cricket:','ja':':クリケット:','ko':':크리켓:','pt':':críquete:','it':':cricket:','fa':':کریکت:','id':':kriket:','zh':':板球:','ru':':крикет:','tr':':kriket:','ar':':كريكيت:'},'\U0001F40A':{'en':':crocodile:','status':fully_qualified,'E':1,'de':':krokodil:','es':':cocodrilo:','fr':':crocodile:','ja':':ワニ:','ko':':악어:','pt':':crocodilo:','it':':coccodrillo:','fa':':تمساح:','id':':buaya:','zh':':鳄鱼:','ru':':крокодил:','tr':':timsah:','ar':':تمساح:'},'\U0001F950':{'en':':croissant:','status':fully_qualified,'E':3,'de':':croissant:','es':':cruasán:','fr':':croissant:','ja':':クロワッサン:','ko':':크루아상:','pt':':croissant:','it':':croissant:','fa':':کرواسان:','id':':croissant:','zh':':羊角面包:','ru':':круассан:','tr':':kruvasan:','ar':':كرواسون:'},'\U0000274C':{'en':':cross_mark:','status':fully_qualified,'E':0.6,'alias':[':x:'],'variant':True,'de':':kreuzzeichen:','es':':marca_de_cruz:','fr':':croix:','ja':':バツ:','ko':':엑스_표시:','pt':':xis:','it':':croce:','fa':':ضربدر:','id':':tanda_silang:','zh':':叉号:','ru':':крестик:','tr':':çarpı_işareti:','ar':':علامة_x:'},'\U0000274E':{'en':':cross_mark_button:','status':fully_qualified,'E':0.6,'alias':[':negative_squared_cross_mark:'],'variant':True,'de':':angekreuztes_kästchen:','es':':botón_con_marca_de_cruz:','fr':':bouton_croix:','ja':':白抜きのバツ:','ko':':초록_상자_안_엑스표:','pt':':botão_de_xis:','it':':croce_con_quadrato:','fa':':دکمه_ایکس:','id':':tombol_tanda_silang:','zh':':叉号按钮:','ru':':кнопка_«крестик»:','tr':':çarpı_işareti_düğmesi:','ar':':علامة_x_في_مربع:'},'\U0001F91E':{'en':':crossed_fingers:','status':fully_qualified,'E':3,'de':':hand_mit_gekreuzten_fingern:','es':':dedos_cruzados:','fr':':doigts_croisés:','ja':':指をクロス:','ko':':검지와_중지_크로스:','pt':':dedos_cruzados:','it':':dita_incrociate:','fa':':انگشتان_گره_خورده:','id':':jari_menyilang:','zh':':交叉的手指:','ru':':скрещенные_пальцы:','tr':':şans_dileme:','ar':':أصابع_مشبكة:'},'\U0001F91E\U0001F3FF':{'en':':crossed_fingers_dark_skin_tone:','status':fully_qualified,'E':3,'de':':hand_mit_gekreuzten_fingern_dunkle_hautfarbe:','es':':dedos_cruzados_tono_de_piel_oscuro:','fr':':doigts_croisés_peau_foncée:','ja':':指をクロス_濃い肌色:','ko':':검지와_중지_크로스_검은색_피부:','pt':':dedos_cruzados_pele_escura:','it':':dita_incrociate_carnagione_scura:','fa':':انگشتان_گره_خورده_پوست_آبنوسی:','id':':jari_menyilang_warna_kulit_gelap:','zh':':交叉的手指_较深肤色:','ru':':скрещенные_пальцы_очень_темный_тон_кожи:','tr':':şans_dileme_koyu_cilt_tonu:','ar':':أصابع_مشبكة_بشرة_بلون_غامق:'},'\U0001F91E\U0001F3FB':{'en':':crossed_fingers_light_skin_tone:','status':fully_qualified,'E':3,'de':':hand_mit_gekreuzten_fingern_helle_hautfarbe:','es':':dedos_cruzados_tono_de_piel_claro:','fr':':doigts_croisés_peau_claire:','ja':':指をクロス_薄い肌色:','ko':':검지와_중지_크로스_하얀_피부:','pt':':dedos_cruzados_pele_clara:','it':':dita_incrociate_carnagione_chiara:','fa':':انگشتان_گره_خورده_پوست_سفید:','id':':jari_menyilang_warna_kulit_cerah:','zh':':交叉的手指_较浅肤色:','ru':':скрещенные_пальцы_очень_светлый_тон_кожи:','tr':':şans_dileme_açık_cilt_tonu:','ar':':أصابع_مشبكة_بشرة_بلون_فاتح:'},'\U0001F91E\U0001F3FE':{'en':':crossed_fingers_medium-dark_skin_tone:','status':fully_qualified,'E':3,'de':':hand_mit_gekreuzten_fingern_mitteldunkle_hautfarbe:','es':':dedos_cruzados_tono_de_piel_oscuro_medio:','fr':':doigts_croisés_peau_mate:','ja':':指をクロス_やや濃い肌色:','ko':':검지와_중지_크로스_진한_갈색_피부:','pt':':dedos_cruzados_pele_morena_escura:','it':':dita_incrociate_carnagione_abbastanza_scura:','fa':':انگشتان_گره_خورده_پوست_گندمی:','id':':jari_menyilang_warna_kulit_gelap-sedang:','zh':':交叉的手指_中等深肤色:','ru':':скрещенные_пальцы_темный_тон_кожи:','tr':':şans_dileme_orta_koyu_cilt_tonu:','ar':':أصابع_مشبكة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F91E\U0001F3FC':{'en':':crossed_fingers_medium-light_skin_tone:','status':fully_qualified,'E':3,'de':':hand_mit_gekreuzten_fingern_mittelhelle_hautfarbe:','es':':dedos_cruzados_tono_de_piel_claro_medio:','fr':':doigts_croisés_peau_moyennement_claire:','ja':':指をクロス_やや薄い肌色:','ko':':검지와_중지_크로스_연한_갈색_피부:','pt':':dedos_cruzados_pele_morena_clara:','it':':dita_incrociate_carnagione_abbastanza_chiara:','fa':':انگشتان_گره_خورده_پوست_روشن:','id':':jari_menyilang_warna_kulit_cerah-sedang:','zh':':交叉的手指_中等浅肤色:','ru':':скрещенные_пальцы_светлый_тон_кожи:','tr':':şans_dileme_orta_açık_cilt_tonu:','ar':':أصابع_مشبكة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F91E\U0001F3FD':{'en':':crossed_fingers_medium_skin_tone:','status':fully_qualified,'E':3,'de':':hand_mit_gekreuzten_fingern_mittlere_hautfarbe:','es':':dedos_cruzados_tono_de_piel_medio:','fr':':doigts_croisés_peau_légèrement_mate:','ja':':指をクロス_中間の肌色:','ko':':검지와_중지_크로스_갈색_피부:','pt':':dedos_cruzados_pele_morena:','it':':dita_incrociate_carnagione_olivastra:','fa':':انگشتان_گره_خورده_پوست_طلایی:','id':':jari_menyilang_warna_kulit_sedang:','zh':':交叉的手指_中等肤色:','ru':':скрещенные_пальцы_средний_тон_кожи:','tr':':şans_dileme_orta_cilt_tonu:','ar':':أصابع_مشبكة_بشرة_بلون_معتدل:'},'\U0001F38C':{'en':':crossed_flags:','status':fully_qualified,'E':0.6,'de':':überkreuzte_flaggen:','es':':banderas_cruzadas:','fr':':drapeaux_croisés:','ja':':祝日の旗:','ko':':엇갈린_일본_국기:','pt':':bandeiras_cruzadas:','it':':bandiere_del_giappone_incrociate:','fa':':پرچم_های_ضرب_دری:','id':':bendera_menyilang:','zh':':交叉旗:','ru':':скрещенные_флаги:','tr':':çapraz_bayraklar:','ar':':علمان_متقاطعان:'},'\U00002694\U0000FE0F':{'en':':crossed_swords:','status':fully_qualified,'E':1,'variant':True,'de':':gekreuzte_schwerter:','es':':espadas_cruzadas:','fr':':épées_croisées:','ja':':クロスした剣:','ko':':쌍검:','pt':':espadas_cruzadas:','it':':spade_incrociate:','fa':':شمشیرهای_ضرب_دری:','id':':pedang_bersilang:','zh':':交叉放置的剑:','ru':':скрещенные_мечи:','tr':':çapraz_kılıçlar:','ar':':سيفان_متصالبان:'},'\U00002694':{'en':':crossed_swords:','status':unqualified,'E':1,'variant':True,'de':':gekreuzte_schwerter:','es':':espadas_cruzadas:','fr':':épées_croisées:','ja':':クロスした剣:','ko':':쌍검:','pt':':espadas_cruzadas:','it':':spade_incrociate:','fa':':شمشیرهای_ضرب_دری:','id':':pedang_bersilang:','zh':':交叉放置的剑:','ru':':скрещенные_мечи:','tr':':çapraz_kılıçlar:','ar':':سيفان_متصالبان:'},'\U0001F451':{'en':':crown:','status':fully_qualified,'E':0.6,'de':':krone:','es':':corona:','fr':':couronne:','ja':':王冠:','ko':':왕관:','pt':':coroa:','it':':corona:','fa':':تاج:','id':':mahkota:','zh':':皇冠:','ru':':корона:','tr':':taç:','ar':':تاج:'},'\U0001FA7C':{'en':':crutch:','status':fully_qualified,'E':14,'de':':krücke:','es':':muleta:','fr':':béquille:','ja':':松葉杖:','ko':':목발:','pt':':muleta:','it':':stampella:','fa':':عصای_زیر_بغل:','id':':kruk:','zh':':拐杖:','ru':':костыль:','tr':':koltuk_değneği:','ar':':عكاز:'},'\U0001F63F':{'en':':crying_cat:','status':fully_qualified,'E':0.6,'alias':[':crying_cat_face:'],'de':':weinende_katze:','es':':gato_llorando:','fr':':chat_qui_pleure:','ja':':泣いている猫:','ko':':울고_있는_고양이_얼굴:','pt':':rosto_de_gato_chorando:','it':':gatto_che_piange:','fa':':گربهٔ_گریان:','id':':wajah_kucing_menangis:','zh':':哭泣的猫:','ru':':плачущий_кот:','tr':':ağlayan_kedi:','ar':':وجه_قطة_يبكي:'},'\U0001F622':{'en':':crying_face:','status':fully_qualified,'E':0.6,'alias':[':cry:'],'de':':weinendes_gesicht:','es':':cara_llorando:','fr':':visage_qui_pleure:','ja':':泣き顔:','ko':':우는_얼굴:','pt':':rosto_chorando:','it':':faccina_che_piange:','fa':':گریان:','id':':wajah_menangis:','zh':':哭:','ru':':плачет:','tr':':ağlayan_yüz:','ar':':وجه_يبكي:'},'\U0001F52E':{'en':':crystal_ball:','status':fully_qualified,'E':0.6,'de':':kristallkugel:','es':':bola_de_cristal:','fr':':boule_de_cristal:','ja':':水晶玉:','ko':':수정_구슬:','pt':':bola_de_cristal:','it':':sfera_di_cristallo:','fa':':گوی:','id':':bola_kristal:','zh':':水晶球:','ru':':хрустальный_шар:','tr':':kristal_küre:','ar':':كرة_كريستال:'},'\U0001F952':{'en':':cucumber:','status':fully_qualified,'E':3,'de':':gurke:','es':':pepino:','fr':':concombre:','ja':':キュウリ:','ko':':오이:','pt':':pepino:','it':':cetriolo:','fa':':خیار:','id':':ketimun:','zh':':黄瓜:','ru':':огурец:','tr':':salatalık:','ar':':خيار:'},'\U0001F964':{'en':':cup_with_straw:','status':fully_qualified,'E':5,'de':':becher_mit_strohhalm:','es':':vaso_con_pajita:','fr':':gobelet_avec_paille:','ja':':ストローカップ:','ko':':빨대와_컵:','pt':':copo_com_canudo:','it':':bicchiere_con_cannuccia:','fa':':فنجان_با_نی:','id':':cangkir_dengan_sedotan:','zh':':带吸管杯:','ru':':стакан_с_соломинкой:','tr':':pipetli_bardak:','ar':':كأس_وقشة_للشرب:'},'\U0001F9C1':{'en':':cupcake:','status':fully_qualified,'E':11,'de':':cupcake:','es':':magdalena:','fr':':cupcake:','ja':':カップケーキ:','ko':':컵케이크:','pt':':cupcake:','it':':cupcake:','fa':':کیک_فنجانی:','id':':kue_mangkuk:','zh':':纸杯蛋糕:','ru':':капкейк:','tr':':cupcake:','ar':':كعكة_صغيرة:'},'\U0001F94C':{'en':':curling_stone:','status':fully_qualified,'E':5,'de':':curlingstein:','es':':piedra_de_curling:','fr':':pierre_de_curling:','ja':':カーリング:','ko':':컬링_스톤:','pt':':pedra_de_curling:','it':':stone_da_curling:','fa':':سنگ_کرلینگ:','id':':batu_curling:','zh':':冰壶:','ru':':керлинг:','tr':':curling_taşı:','ar':':حجرة_لعبة_الكرلنغ:'},'\U0001F9B1':{'en':':curly_hair:','status':component,'E':11,'de':':lockiges_haar:','es':':pelo_rizado:','fr':':cheveux_bouclés:','ja':':巻き毛:','ko':':곱슬머리:','pt':':cabelo_cacheado:','it':':capelli_ricci:','fa':':موفرفری:','id':':rambut_keriting:','zh':':卷发:','ru':':кудрявые_волосы:','tr':':kıvırcık_saç:','ar':':شعر_مجعد:'},'\U000027B0':{'en':':curly_loop:','status':fully_qualified,'E':0.6,'variant':True,'de':':schleife:','es':':bucle:','fr':':boucle:','ja':':ループ:','ko':':고리:','pt':':laço_encaracolado:','it':':occhiello:','fa':':حلقه_مجعد:','id':':untaian_bergelombang:','zh':':卷曲环:','ru':':петля:','tr':':kıvrık_döngü:','ar':':عقدة:'},'\U0001F4B1':{'en':':currency_exchange:','status':fully_qualified,'E':0.6,'de':':geldwechsel:','es':':cambio_de_divisas:','fr':':conversion_de_devise:','ja':':外貨両替:','ko':':환전:','pt':':câmbio_de_moeda:','it':':cambio_valuta:','fa':':تبدیل_ارز:','id':':kurs:','zh':':货币兑换:','ru':':обмен_валюты:','tr':':kambiyo:','ar':':صرف_عملة:'},'\U0001F35B':{'en':':curry_rice:','status':fully_qualified,'E':0.6,'alias':[':curry:'],'de':':reis_mit_curry:','es':':arroz_con_curry:','fr':':riz_au_curry:','ja':':カレーライス:','ko':':카레라이스:','pt':':arroz_com_curry:','it':':riso_al_curry:','fa':':برنج_با_کاری:','id':':nasi_kari:','zh':':咖喱饭:','ru':':рис_с_карри:','tr':':körili_pilav:','ar':':أرز_وكاري:'},'\U0001F36E':{'en':':custard:','status':fully_qualified,'E':0.6,'de':':pudding:','es':':flan:','fr':':crème_renversée:','ja':':プリン:','ko':':커스타드_푸딩:','pt':':pudim:','it':':budino:','fa':':کاستارد:','id':':puding_susu:','zh':':奶黄:','ru':':пудинг:','tr':':krem_karamel:','ar':':كريم_كراميل:'},'\U0001F6C3':{'en':':customs:','status':fully_qualified,'E':1,'de':':zollkontrolle:','es':':aduana:','fr':':douane:','ja':':税関:','ko':':세관:','pt':':alfândega:','it':':simbolo_della_dogana:','fa':':گمرک:','id':':bea_cukai:','zh':':海关:','ru':':таможня:','tr':':gümrük:','ar':':جمارك:'},'\U0001F969':{'en':':cut_of_meat:','status':fully_qualified,'E':5,'de':':fleischstück:','es':':corte_de_carne:','fr':':morceau_de_viande:','ja':':ステーキ肉:','ko':':고깃덩이:','pt':':corte_de_carne:','it':':taglio_di_carne:','fa':':برش_گوشت:','id':':potongan_daging:','zh':':肉块:','ru':':кусок_мяса:','tr':':parça_et:','ar':':قطعة_لحم:'},'\U0001F300':{'en':':cyclone:','status':fully_qualified,'E':0.6,'de':':wirbel:','es':':ciclón:','fr':':cyclone:','ja':':渦巻き:','ko':':태풍:','pt':':ciclone:','it':':ciclone:','fa':':چرخند:','id':':topan:','zh':':台风:','ru':':циклон:','tr':':siklon:','ar':':عاصفة:'},'\U0001F5E1\U0000FE0F':{'en':':dagger:','status':fully_qualified,'E':0.7,'alias':[':dagger_knife:'],'variant':True,'de':':dolch:','es':':puñal:','fr':':dague:','ja':':短刀:','ko':':단검:','pt':':adaga:','it':':pugnale:','fa':':خنجر:','id':':belati:','zh':':匕首:','ru':':кинжал:','tr':':hançer:','ar':':خنجر:'},'\U0001F5E1':{'en':':dagger:','status':unqualified,'E':0.7,'alias':[':dagger_knife:'],'variant':True,'de':':dolch:','es':':puñal:','fr':':dague:','ja':':短刀:','ko':':단검:','pt':':adaga:','it':':pugnale:','fa':':خنجر:','id':':belati:','zh':':匕首:','ru':':кинжал:','tr':':hançer:','ar':':خنجر:'},'\U0001F361':{'en':':dango:','status':fully_qualified,'E':0.6,'de':':dango:','es':':dango:','fr':':brochette_de_bonbons:','ja':':だんご:','ko':':떡꼬치:','pt':':dango:','it':':dango:','fa':':دانگو:','id':':dango:','zh':':团子:','ru':':данго:','tr':':pirinç_tatlısı:','ar':':حلوى_الدانغو:'},'\U0001F3FF':{'en':':dark_skin_tone:','status':component,'E':1,'alias':[':emoji_modifier_fitzpatrick_type_6:'],'de':':dunkle_hautfarbe:','es':':tono_de_piel_oscuro:','fr':':peau_foncée:','ja':':濃い肌色:','ko':':검은색_피부:','pt':':pele_escura:','it':':carnagione_scura:','fa':':پوست_آبنوسی:','id':':warna_kulit_gelap:','zh':':较深肤色:','ru':':очень_темный_тон_кожи:','tr':':koyu_cilt_tonu:','ar':':بشرة_بلون_غامق:'},'\U0001F4A8':{'en':':dashing_away:','status':fully_qualified,'E':0.6,'alias':[':dash:'],'de':':staubwolke:','es':':salir_corriendo:','fr':':décamper:','ja':':ダッシュ:','ko':':달려나감:','pt':':rapidez:','it':':nuvola_di_polvere:','fa':':به_سرعت_باد:','id':':berlari:','zh':':尾气:','ru':':исчезновение:','tr':':hızla_fırlama:','ar':':انطلاقة_جري:'},'\U0001F9CF\U0000200D\U00002642\U0000FE0F':{'en':':deaf_man:','status':fully_qualified,'E':12,'de':':gehörloser_mann:','es':':hombre_sordo:','fr':':homme_sourd:','ja':':耳の不自由な男性:','ko':':청각장애가_있는_남자:','pt':':homem_surdo:','it':':uomo_con_problemi_di_udito:','fa':':مرد_ناشنوا:','id':':pria_tuli:','zh':':失聪的男人:','ru':':глухои_мужчина:','tr':':işitme_engelli_erkek:','ar':':رجل_أصم:'},'\U0001F9CF\U0000200D\U00002642':{'en':':deaf_man:','status':minimally_qualified,'E':12,'de':':gehörloser_mann:','es':':hombre_sordo:','fr':':homme_sourd:','ja':':耳の不自由な男性:','ko':':청각장애가_있는_남자:','pt':':homem_surdo:','it':':uomo_con_problemi_di_udito:','fa':':مرد_ناشنوا:','id':':pria_tuli:','zh':':失聪的男人:','ru':':глухой_мужчина:','tr':':işitme_engelli_erkek:','ar':':رجل_أصم:'},'\U0001F9CF\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':deaf_man_dark_skin_tone:','status':fully_qualified,'E':12,'de':':gehörloser_mann_dunkle_hautfarbe:','es':':hombre_sordo_tono_de_piel_oscuro:','fr':':homme_sourd_peau_foncée:','ja':':耳の不自由な男性_濃い肌色:','ko':':청각장애가_있는_남자_검은색_피부:','pt':':homem_surdo_pele_escura:','it':':uomo_con_problemi_di_udito_carnagione_scura:','fa':':مرد_ناشنوا_پوست_آبنوسی:','id':':pria_tuli_warna_kulit_gelap:','zh':':失聪的男人_较深肤色:','ru':':глухои_мужчина_очень_темныи_тон_кожи:','tr':':işitme_engelli_erkek_koyu_cilt_tonu:','ar':':رجل_أصم_بشرة_بلون_غامق:'},'\U0001F9CF\U0001F3FF\U0000200D\U00002642':{'en':':deaf_man_dark_skin_tone:','status':minimally_qualified,'E':12,'de':':gehörloser_mann_dunkle_hautfarbe:','es':':hombre_sordo_tono_de_piel_oscuro:','fr':':homme_sourd_peau_foncée:','ja':':耳の不自由な男性_濃い肌色:','ko':':청각장애가_있는_남자_검은색_피부:','pt':':homem_surdo_pele_escura:','it':':uomo_con_problemi_di_udito_carnagione_scura:','fa':':مرد_ناشنوا_پوست_آبنوسی:','id':':pria_tuli_warna_kulit_gelap:','zh':':失聪的男人_较深肤色:','ru':':глухой_мужчина_очень_темный_тон_кожи:','tr':':işitme_engelli_erkek_koyu_cilt_tonu:','ar':':رجل_أصم_بشرة_بلون_غامق:'},'\U0001F9CF\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':deaf_man_light_skin_tone:','status':fully_qualified,'E':12,'de':':gehörloser_mann_helle_hautfarbe:','es':':hombre_sordo_tono_de_piel_claro:','fr':':homme_sourd_peau_claire:','ja':':耳の不自由な男性_薄い肌色:','ko':':청각장애가_있는_남자_하얀_피부:','pt':':homem_surdo_pele_clara:','it':':uomo_con_problemi_di_udito_carnagione_chiara:','fa':':مرد_ناشنوا_پوست_سفید:','id':':pria_tuli_warna_kulit_cerah:','zh':':失聪的男人_较浅肤色:','ru':':глухои_мужчина_очень_светлыи_тон_кожи:','tr':':işitme_engelli_erkek_açık_cilt_tonu:','ar':':رجل_أصم_بشرة_بلون_فاتح:'},'\U0001F9CF\U0001F3FB\U0000200D\U00002642':{'en':':deaf_man_light_skin_tone:','status':minimally_qualified,'E':12,'de':':gehörloser_mann_helle_hautfarbe:','es':':hombre_sordo_tono_de_piel_claro:','fr':':homme_sourd_peau_claire:','ja':':耳の不自由な男性_薄い肌色:','ko':':청각장애가_있는_남자_하얀_피부:','pt':':homem_surdo_pele_clara:','it':':uomo_con_problemi_di_udito_carnagione_chiara:','fa':':مرد_ناشنوا_پوست_سفید:','id':':pria_tuli_warna_kulit_cerah:','zh':':失聪的男人_较浅肤色:','ru':':глухой_мужчина_очень_светлый_тон_кожи:','tr':':işitme_engelli_erkek_açık_cilt_tonu:','ar':':رجل_أصم_بشرة_بلون_فاتح:'},'\U0001F9CF\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':deaf_man_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':gehörloser_mann_mitteldunkle_hautfarbe:','es':':hombre_sordo_tono_de_piel_oscuro_medio:','fr':':homme_sourd_peau_mate:','ja':':耳の不自由な男性_やや濃い肌色:','ko':':청각장애가_있는_남자_진한_갈색_피부:','pt':':homem_surdo_pele_morena_escura:','it':':uomo_con_problemi_di_udito_carnagione_abbastanza_scura:','fa':':مرد_ناشنوا_پوست_گندمی:','id':':pria_tuli_warna_kulit_gelap-sedang:','zh':':失聪的男人_中等深肤色:','ru':':глухои_мужчина_темныи_тон_кожи:','tr':':işitme_engelli_erkek_orta_koyu_cilt_tonu:','ar':':رجل_أصم_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9CF\U0001F3FE\U0000200D\U00002642':{'en':':deaf_man_medium-dark_skin_tone:','status':minimally_qualified,'E':12,'de':':gehörloser_mann_mitteldunkle_hautfarbe:','es':':hombre_sordo_tono_de_piel_oscuro_medio:','fr':':homme_sourd_peau_mate:','ja':':耳の不自由な男性_やや濃い肌色:','ko':':청각장애가_있는_남자_진한_갈색_피부:','pt':':homem_surdo_pele_morena_escura:','it':':uomo_con_problemi_di_udito_carnagione_abbastanza_scura:','fa':':مرد_ناشنوا_پوست_گندمی:','id':':pria_tuli_warna_kulit_gelap-sedang:','zh':':失聪的男人_中等深肤色:','ru':':глухой_мужчина_темный_тон_кожи:','tr':':işitme_engelli_erkek_orta_koyu_cilt_tonu:','ar':':رجل_أصم_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9CF\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':deaf_man_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':gehörloser_mann_mittelhelle_hautfarbe:','es':':hombre_sordo_tono_de_piel_claro_medio:','fr':':homme_sourd_peau_moyennement_claire:','ja':':耳の不自由な男性_やや薄い肌色:','ko':':청각장애가_있는_남자_연한_갈색_피부:','pt':':homem_surdo_pele_morena_clara:','it':':uomo_con_problemi_di_udito_carnagione_abbastanza_chiara:','fa':':مرد_ناشنوا_پوست_روشن:','id':':pria_tuli_warna_kulit_cerah-sedang:','zh':':失聪的男人_中等浅肤色:','ru':':глухои_мужчина_светлыи_тон_кожи:','tr':':işitme_engelli_erkek_orta_açık_cilt_tonu:','ar':':رجل_أصم_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9CF\U0001F3FC\U0000200D\U00002642':{'en':':deaf_man_medium-light_skin_tone:','status':minimally_qualified,'E':12,'de':':gehörloser_mann_mittelhelle_hautfarbe:','es':':hombre_sordo_tono_de_piel_claro_medio:','fr':':homme_sourd_peau_moyennement_claire:','ja':':耳の不自由な男性_やや薄い肌色:','ko':':청각장애가_있는_남자_연한_갈색_피부:','pt':':homem_surdo_pele_morena_clara:','it':':uomo_con_problemi_di_udito_carnagione_abbastanza_chiara:','fa':':مرد_ناشنوا_پوست_روشن:','id':':pria_tuli_warna_kulit_cerah-sedang:','zh':':失聪的男人_中等浅肤色:','ru':':глухой_мужчина_светлый_тон_кожи:','tr':':işitme_engelli_erkek_orta_açık_cilt_tonu:','ar':':رجل_أصم_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9CF\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':deaf_man_medium_skin_tone:','status':fully_qualified,'E':12,'de':':gehörloser_mann_mittlere_hautfarbe:','es':':hombre_sordo_tono_de_piel_medio:','fr':':homme_sourd_peau_légèrement_mate:','ja':':耳の不自由な男性_中間の肌色:','ko':':청각장애가_있는_남자_갈색_피부:','pt':':homem_surdo_pele_morena:','it':':uomo_con_problemi_di_udito_carnagione_olivastra:','fa':':مرد_ناشنوا_پوست_طلایی:','id':':pria_tuli_warna_kulit_sedang:','zh':':失聪的男人_中等肤色:','ru':':глухои_мужчина_среднии_тон_кожи:','tr':':işitme_engelli_erkek_orta_cilt_tonu:','ar':':رجل_أصم_بشرة_بلون_معتدل:'},'\U0001F9CF\U0001F3FD\U0000200D\U00002642':{'en':':deaf_man_medium_skin_tone:','status':minimally_qualified,'E':12,'de':':gehörloser_mann_mittlere_hautfarbe:','es':':hombre_sordo_tono_de_piel_medio:','fr':':homme_sourd_peau_légèrement_mate:','ja':':耳の不自由な男性_中間の肌色:','ko':':청각장애가_있는_남자_갈색_피부:','pt':':homem_surdo_pele_morena:','it':':uomo_con_problemi_di_udito_carnagione_olivastra:','fa':':مرد_ناشنوا_پوست_طلایی:','id':':pria_tuli_warna_kulit_sedang:','zh':':失聪的男人_中等肤色:','ru':':глухой_мужчина_средний_тон_кожи:','tr':':işitme_engelli_erkek_orta_cilt_tonu:','ar':':رجل_أصم_بشرة_بلون_معتدل:'},'\U0001F9CF':{'en':':deaf_person:','status':fully_qualified,'E':12,'de':':gehörlose_person:','es':':persona_sorda:','fr':':personne_sourde:','ja':':耳の不自由な人:','ko':':청각_장애가_있는_사람:','pt':':pessoa_surda:','it':':persona_con_problemi_di_udito:','fa':':آدم_ناشنوا:','id':':orang_tuli:','zh':':失聪者:','ru':':глухой_человек:','tr':':işitme_engelli_kişi:','ar':':شخص_أصم:'},'\U0001F9CF\U0001F3FF':{'en':':deaf_person_dark_skin_tone:','status':fully_qualified,'E':12,'de':':gehörlose_person_dunkle_hautfarbe:','es':':persona_sorda_tono_de_piel_oscuro:','fr':':personne_sourde_peau_foncée:','ja':':耳の不自由な人_濃い肌色:','ko':':청각_장애가_있는_사람_검은색_피부:','pt':':pessoa_surda_pele_escura:','it':':persona_con_problemi_di_udito_carnagione_scura:','fa':':آدم_ناشنوا_پوست_آبنوسی:','id':':orang_tuli_warna_kulit_gelap:','zh':':失聪者_较深肤色:','ru':':глухой_человек_очень_темный_тон_кожи:','tr':':işitme_engelli_kişi_koyu_cilt_tonu:','ar':':شخص_أصم_بشرة_بلون_غامق:'},'\U0001F9CF\U0001F3FB':{'en':':deaf_person_light_skin_tone:','status':fully_qualified,'E':12,'de':':gehörlose_person_helle_hautfarbe:','es':':persona_sorda_tono_de_piel_claro:','fr':':personne_sourde_peau_claire:','ja':':耳の不自由な人_薄い肌色:','ko':':청각_장애가_있는_사람_하얀_피부:','pt':':pessoa_surda_pele_clara:','it':':persona_con_problemi_di_udito_carnagione_chiara:','fa':':آدم_ناشنوا_پوست_سفید:','id':':orang_tuli_warna_kulit_cerah:','zh':':失聪者_较浅肤色:','ru':':глухой_человек_очень_светлый_тон_кожи:','tr':':işitme_engelli_kişi_açık_cilt_tonu:','ar':':شخص_أصم_بشرة_بلون_فاتح:'},'\U0001F9CF\U0001F3FE':{'en':':deaf_person_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':gehörlose_person_mitteldunkle_hautfarbe:','es':':persona_sorda_tono_de_piel_oscuro_medio:','fr':':personne_sourde_peau_mate:','ja':':耳の不自由な人_やや濃い肌色:','ko':':청각_장애가_있는_사람_진한_갈색_피부:','pt':':pessoa_surda_pele_morena_escura:','it':':persona_con_problemi_di_udito_carnagione_abbastanza_scura:','fa':':آدم_ناشنوا_پوست_گندمی:','id':':orang_tuli_warna_kulit_gelap-sedang:','zh':':失聪者_中等深肤色:','ru':':глухой_человек_темный_тон_кожи:','tr':':işitme_engelli_kişi_orta_koyu_cilt_tonu:','ar':':شخص_أصم_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9CF\U0001F3FC':{'en':':deaf_person_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':gehörlose_person_mittelhelle_hautfarbe:','es':':persona_sorda_tono_de_piel_claro_medio:','fr':':personne_sourde_peau_moyennement_claire:','ja':':耳の不自由な人_やや薄い肌色:','ko':':청각_장애가_있는_사람_연한_갈색_피부:','pt':':pessoa_surda_pele_morena_clara:','it':':persona_con_problemi_di_udito_carnagione_abbastanza_chiara:','fa':':آدم_ناشنوا_پوست_روشن:','id':':orang_tuli_warna_kulit_cerah-sedang:','zh':':失聪者_中等浅肤色:','ru':':глухой_человек_светлый_тон_кожи:','tr':':işitme_engelli_kişi_orta_açık_cilt_tonu:','ar':':شخص_أصم_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9CF\U0001F3FD':{'en':':deaf_person_medium_skin_tone:','status':fully_qualified,'E':12,'de':':gehörlose_person_mittlere_hautfarbe:','es':':persona_sorda_tono_de_piel_medio:','fr':':personne_sourde_peau_légèrement_mate:','ja':':耳の不自由な人_中間の肌色:','ko':':청각_장애가_있는_사람_갈색_피부:','pt':':pessoa_surda_pele_morena:','it':':persona_con_problemi_di_udito_carnagione_olivastra:','fa':':آدم_ناشنوا_پوست_طلایی:','id':':orang_tuli_warna_kulit_sedang:','zh':':失聪者_中等肤色:','ru':':глухой_человек_средний_тон_кожи:','tr':':işitme_engelli_kişi_orta_cilt_tonu:','ar':':شخص_أصم_بشرة_بلون_معتدل:'},'\U0001F9CF\U0000200D\U00002640\U0000FE0F':{'en':':deaf_woman:','status':fully_qualified,'E':12,'de':':gehörlose_frau:','es':':mujer_sorda:','fr':':femme_sourde:','ja':':耳の不自由な女性:','ko':':청각장애가_있는_여자:','pt':':mulher_surda:','it':':donna_con_problemi_di_udito:','fa':':زن_ناشنوا:','id':':wanita_tuli:','zh':':失聪的女人:','ru':':глухая_женщина:','tr':':işitme_engelli_kadın:','ar':':سيدة_صماء:'},'\U0001F9CF\U0000200D\U00002640':{'en':':deaf_woman:','status':minimally_qualified,'E':12,'de':':gehörlose_frau:','es':':mujer_sorda:','fr':':femme_sourde:','ja':':耳の不自由な女性:','ko':':청각장애가_있는_여자:','pt':':mulher_surda:','it':':donna_con_problemi_di_udito:','fa':':زن_ناشنوا:','id':':wanita_tuli:','zh':':失聪的女人:','ru':':глухая_женщина:','tr':':işitme_engelli_kadın:','ar':':سيدة_صماء:'},'\U0001F9CF\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':deaf_woman_dark_skin_tone:','status':fully_qualified,'E':12,'de':':gehörlose_frau_dunkle_hautfarbe:','es':':mujer_sorda_tono_de_piel_oscuro:','fr':':femme_sourde_peau_foncée:','ja':':耳の不自由な女性_濃い肌色:','ko':':청각장애가_있는_여자_검은색_피부:','pt':':mulher_surda_pele_escura:','it':':donna_con_problemi_di_udito_carnagione_scura:','fa':':زن_ناشنوا_پوست_آبنوسی:','id':':wanita_tuli_warna_kulit_gelap:','zh':':失聪的女人_较深肤色:','ru':':глухая_женщина_очень_темныи_тон_кожи:','tr':':işitme_engelli_kadın_koyu_cilt_tonu:','ar':':سيدة_صماء_بشرة_بلون_غامق:'},'\U0001F9CF\U0001F3FF\U0000200D\U00002640':{'en':':deaf_woman_dark_skin_tone:','status':minimally_qualified,'E':12,'de':':gehörlose_frau_dunkle_hautfarbe:','es':':mujer_sorda_tono_de_piel_oscuro:','fr':':femme_sourde_peau_foncée:','ja':':耳の不自由な女性_濃い肌色:','ko':':청각장애가_있는_여자_검은색_피부:','pt':':mulher_surda_pele_escura:','it':':donna_con_problemi_di_udito_carnagione_scura:','fa':':زن_ناشنوا_پوست_آبنوسی:','id':':wanita_tuli_warna_kulit_gelap:','zh':':失聪的女人_较深肤色:','ru':':глухая_женщина_очень_темный_тон_кожи:','tr':':işitme_engelli_kadın_koyu_cilt_tonu:','ar':':سيدة_صماء_بشرة_بلون_غامق:'},'\U0001F9CF\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':deaf_woman_light_skin_tone:','status':fully_qualified,'E':12,'de':':gehörlose_frau_helle_hautfarbe:','es':':mujer_sorda_tono_de_piel_claro:','fr':':femme_sourde_peau_claire:','ja':':耳の不自由な女性_薄い肌色:','ko':':청각장애가_있는_여자_하얀_피부:','pt':':mulher_surda_pele_clara:','it':':donna_con_problemi_di_udito_carnagione_chiara:','fa':':زن_ناشنوا_پوست_سفید:','id':':wanita_tuli_warna_kulit_cerah:','zh':':失聪的女人_较浅肤色:','ru':':глухая_женщина_очень_светлыи_тон_кожи:','tr':':işitme_engelli_kadın_açık_cilt_tonu:','ar':':سيدة_صماء_بشرة_بلون_فاتح:'},'\U0001F9CF\U0001F3FB\U0000200D\U00002640':{'en':':deaf_woman_light_skin_tone:','status':minimally_qualified,'E':12,'de':':gehörlose_frau_helle_hautfarbe:','es':':mujer_sorda_tono_de_piel_claro:','fr':':femme_sourde_peau_claire:','ja':':耳の不自由な女性_薄い肌色:','ko':':청각장애가_있는_여자_하얀_피부:','pt':':mulher_surda_pele_clara:','it':':donna_con_problemi_di_udito_carnagione_chiara:','fa':':زن_ناشنوا_پوست_سفید:','id':':wanita_tuli_warna_kulit_cerah:','zh':':失聪的女人_较浅肤色:','ru':':глухая_женщина_очень_светлый_тон_кожи:','tr':':işitme_engelli_kadın_açık_cilt_tonu:','ar':':سيدة_صماء_بشرة_بلون_فاتح:'},'\U0001F9CF\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':deaf_woman_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':gehörlose_frau_mitteldunkle_hautfarbe:','es':':mujer_sorda_tono_de_piel_oscuro_medio:','fr':':femme_sourde_peau_mate:','ja':':耳の不自由な女性_やや濃い肌色:','ko':':청각장애가_있는_여자_진한_갈색_피부:','pt':':mulher_surda_pele_morena_escura:','it':':donna_con_problemi_di_udito_carnagione_abbastanza_scura:','fa':':زن_ناشنوا_پوست_گندمی:','id':':wanita_tuli_warna_kulit_gelap-sedang:','zh':':失聪的女人_中等深肤色:','ru':':глухая_женщина_темныи_тон_кожи:','tr':':işitme_engelli_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_صماء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9CF\U0001F3FE\U0000200D\U00002640':{'en':':deaf_woman_medium-dark_skin_tone:','status':minimally_qualified,'E':12,'de':':gehörlose_frau_mitteldunkle_hautfarbe:','es':':mujer_sorda_tono_de_piel_oscuro_medio:','fr':':femme_sourde_peau_mate:','ja':':耳の不自由な女性_やや濃い肌色:','ko':':청각장애가_있는_여자_진한_갈색_피부:','pt':':mulher_surda_pele_morena_escura:','it':':donna_con_problemi_di_udito_carnagione_abbastanza_scura:','fa':':زن_ناشنوا_پوست_گندمی:','id':':wanita_tuli_warna_kulit_gelap-sedang:','zh':':失聪的女人_中等深肤色:','ru':':глухая_женщина_темный_тон_кожи:','tr':':işitme_engelli_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_صماء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9CF\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':deaf_woman_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':gehörlose_frau_mittelhelle_hautfarbe:','es':':mujer_sorda_tono_de_piel_claro_medio:','fr':':femme_sourde_peau_moyennement_claire:','ja':':耳の不自由な女性_やや薄い肌色:','ko':':청각장애가_있는_여자_연한_갈색_피부:','pt':':mulher_surda_pele_morena_clara:','it':':donna_con_problemi_di_udito_carnagione_abbastanza_chiara:','fa':':زن_ناشنوا_پوست_روشن:','id':':wanita_tuli_warna_kulit_cerah-sedang:','zh':':失聪的女人_中等浅肤色:','ru':':глухая_женщина_светлыи_тон_кожи:','tr':':işitme_engelli_kadın_orta_açık_cilt_tonu:','ar':':سيدة_صماء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9CF\U0001F3FC\U0000200D\U00002640':{'en':':deaf_woman_medium-light_skin_tone:','status':minimally_qualified,'E':12,'de':':gehörlose_frau_mittelhelle_hautfarbe:','es':':mujer_sorda_tono_de_piel_claro_medio:','fr':':femme_sourde_peau_moyennement_claire:','ja':':耳の不自由な女性_やや薄い肌色:','ko':':청각장애가_있는_여자_연한_갈색_피부:','pt':':mulher_surda_pele_morena_clara:','it':':donna_con_problemi_di_udito_carnagione_abbastanza_chiara:','fa':':زن_ناشنوا_پوست_روشن:','id':':wanita_tuli_warna_kulit_cerah-sedang:','zh':':失聪的女人_中等浅肤色:','ru':':глухая_женщина_светлый_тон_кожи:','tr':':işitme_engelli_kadın_orta_açık_cilt_tonu:','ar':':سيدة_صماء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9CF\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':deaf_woman_medium_skin_tone:','status':fully_qualified,'E':12,'de':':gehörlose_frau_mittlere_hautfarbe:','es':':mujer_sorda_tono_de_piel_medio:','fr':':femme_sourde_peau_légèrement_mate:','ja':':耳の不自由な女性_中間の肌色:','ko':':청각장애가_있는_여자_갈색_피부:','pt':':mulher_surda_pele_morena:','it':':donna_con_problemi_di_udito_carnagione_olivastra:','fa':':زن_ناشنوا_پوست_طلایی:','id':':wanita_tuli_warna_kulit_sedang:','zh':':失聪的女人_中等肤色:','ru':':глухая_женщина_среднии_тон_кожи:','tr':':işitme_engelli_kadın_orta_cilt_tonu:','ar':':سيدة_صماء_بشرة_بلون_معتدل:'},'\U0001F9CF\U0001F3FD\U0000200D\U00002640':{'en':':deaf_woman_medium_skin_tone:','status':minimally_qualified,'E':12,'de':':gehörlose_frau_mittlere_hautfarbe:','es':':mujer_sorda_tono_de_piel_medio:','fr':':femme_sourde_peau_légèrement_mate:','ja':':耳の不自由な女性_中間の肌色:','ko':':청각장애가_있는_여자_갈색_피부:','pt':':mulher_surda_pele_morena:','it':':donna_con_problemi_di_udito_carnagione_olivastra:','fa':':زن_ناشنوا_پوست_طلایی:','id':':wanita_tuli_warna_kulit_sedang:','zh':':失聪的女人_中等肤色:','ru':':глухая_женщина_средний_тон_кожи:','tr':':işitme_engelli_kadın_orta_cilt_tonu:','ar':':سيدة_صماء_بشرة_بلون_معتدل:'},'\U0001F333':{'en':':deciduous_tree:','status':fully_qualified,'E':1,'de':':laubbaum:','es':':árbol_de_hoja_caduca:','fr':':arbre_à_feuilles_caduques:','ja':':落葉樹:','ko':':나무:','pt':':árvore_caidiça:','it':':albero_deciduo:','fa':':درخت_سبز:','id':':pohon_meranggas:','zh':':落叶树:','ru':':дерево:','tr':':yaprak_döken_ağaç:','ar':':شجرة_زائلة_الخضرة:'},'\U0001F98C':{'en':':deer:','status':fully_qualified,'E':3,'de':':hirsch:','es':':ciervo:','fr':':cerf:','ja':':シカ:','ko':':사슴:','pt':':cervo:','it':':cervo:','fa':':گوزن:','id':':rusa:','zh':':鹿:','ru':':олень:','tr':':geyik:','ar':':غزال:'},'\U0001F69A':{'en':':delivery_truck:','status':fully_qualified,'E':0.6,'alias':[':truck:'],'de':':lieferwagen:','es':':camión_de_reparto:','fr':':camion_de_livraison:','ja':':トラック:','ko':':운송_트럭:','pt':':caminhão_de_entrega:','it':':camion:','fa':':کامیون_پخش:','id':':truk_pengiriman:','zh':':货车:','ru':':грузовик:','tr':':nakliye_kamyonu:','ar':':شاحنة_توصيل_طلبات:'},'\U0001F3EC':{'en':':department_store:','status':fully_qualified,'E':0.6,'de':':kaufhaus:','es':':grandes_almacenes:','fr':':grand_magasin:','ja':':デパート:','ko':':백화점:','pt':':loja_de_departamentos:','it':':grande_magazzino:','fa':':فروشگاه_بزرگ:','id':':toserba:','zh':':商场:','ru':':универмаг:','tr':':çok_katlı_mağaza:','ar':':متجر_شامل:'},'\U0001F3DA\U0000FE0F':{'en':':derelict_house:','status':fully_qualified,'E':0.7,'alias':[':derelict_house_building:'],'variant':True,'de':':verfallenes_haus:','es':':casa_abandonada:','fr':':maison_abandonnée:','ja':':廃屋:','ko':':낡은_주택_건물:','pt':':casa_abandonada:','it':':casa_in_rovina:','fa':':خانه_متروکه:','id':':rumah_terlantar:','zh':':废墟:','ru':':заброшенныи_дом:','tr':':terkedilmiş_ev:','ar':':مبنى_مهجور:'},'\U0001F3DA':{'en':':derelict_house:','status':unqualified,'E':0.7,'alias':[':derelict_house_building:'],'variant':True,'de':':verfallenes_haus:','es':':casa_abandonada:','fr':':maison_abandonnée:','ja':':廃屋:','ko':':낡은_주택_건물:','pt':':casa_abandonada:','it':':casa_in_rovina:','fa':':خانهٔ_متروکه:','id':':rumah_terlantar:','zh':':废墟:','ru':':заброшенный_дом:','tr':':terkedilmiş_ev:','ar':':مبنى_مهجور:'},'\U0001F3DC\U0000FE0F':{'en':':desert:','status':fully_qualified,'E':0.7,'variant':True,'de':':wüste:','es':':desierto:','fr':':désert:','ja':':砂漠:','ko':':사막:','pt':':deserto:','it':':deserto:','fa':':بیابان:','id':':gurun:','zh':':沙漠:','ru':':пустыня:','tr':':çöl:','ar':':صحراء:'},'\U0001F3DC':{'en':':desert:','status':unqualified,'E':0.7,'variant':True,'de':':wüste:','es':':desierto:','fr':':désert:','ja':':砂漠:','ko':':사막:','pt':':deserto:','it':':deserto:','fa':':بیابان:','id':':gurun:','zh':':沙漠:','ru':':пустыня:','tr':':çöl:','ar':':صحراء:'},'\U0001F3DD\U0000FE0F':{'en':':desert_island:','status':fully_qualified,'E':0.7,'variant':True,'de':':einsame_insel:','es':':isla_desierta:','fr':':île_déserte:','ja':':無人島:','ko':':사막_섬:','pt':':ilha_deserta:','it':':isola_deserta:','fa':':جزیره_خالی_از_سکنه:','id':':pulau_terpencil:','zh':':无人荒岛:','ru':':остров:','tr':':ıssız_ada:','ar':':جزيرة_صحراوية:'},'\U0001F3DD':{'en':':desert_island:','status':unqualified,'E':0.7,'variant':True,'de':':einsame_insel:','es':':isla_desierta:','fr':':île_déserte:','ja':':無人島:','ko':':사막_섬:','pt':':ilha_deserta:','it':':isola_deserta:','fa':':جزیرهٔ_خالی_از_سکنه:','id':':pulau_terpencil:','zh':':无人荒岛:','ru':':остров:','tr':':ıssız_ada:','ar':':جزيرة_صحراوية:'},'\U0001F5A5\U0000FE0F':{'en':':desktop_computer:','status':fully_qualified,'E':0.7,'variant':True,'de':':desktopcomputer:','es':':ordenador_de_sobremesa:','fr':':ordinateur_de_bureau:','ja':':デスクトップパソコン:','ko':':데스크톱_컴퓨터:','pt':':computador_de_mesa:','it':':computer_fisso:','fa':':رایانه_میزی:','id':':komputer_desktop:','zh':':台式电脑:','ru':':компьютер:','tr':':masaüstü_bilgisayar:','ar':':كمبيوتر:'},'\U0001F5A5':{'en':':desktop_computer:','status':unqualified,'E':0.7,'variant':True,'de':':desktopcomputer:','es':':ordenador_de_sobremesa:','fr':':ordinateur_de_bureau:','ja':':デスクトップパソコン:','ko':':데스크톱_컴퓨터:','pt':':computador_de_mesa:','it':':computer_fisso:','fa':':رایانهٔ_میزی:','id':':komputer_desktop:','zh':':台式电脑:','ru':':компьютер:','tr':':masaüstü_bilgisayar:','ar':':كمبيوتر:'},'\U0001F575\U0000FE0F':{'en':':detective:','status':fully_qualified,'E':0.7,'alias':[':sleuth_or_spy:'],'variant':True,'de':':detektiv(in):','es':':detective:','fr':':détective:','ja':':探偵:','ko':':탐정:','pt':':detetive:','it':':detective:','fa':':کارآگاه:','id':':detektif:','zh':':侦探:','ru':':детектив:','tr':':dedektif:','ar':':شخص_يحقق:'},'\U0001F575':{'en':':detective:','status':unqualified,'E':0.7,'alias':[':sleuth_or_spy:'],'variant':True,'de':':detektiv(in):','es':':detective:','fr':':détective:','ja':':探偵:','ko':':탐정:','pt':':detetive:','it':':detective:','fa':':کارآگاه:','id':':detektif:','zh':':侦探:','ru':':детектив:','tr':':dedektif:','ar':':شخص_يحقق:'},'\U0001F575\U0001F3FF':{'en':':detective_dark_skin_tone:','status':fully_qualified,'E':2,'de':':detektiv(in)_dunkle_hautfarbe:','es':':detective_tono_de_piel_oscuro:','fr':':détective_peau_foncée:','ja':':探偵_濃い肌色:','ko':':탐정_검은색_피부:','pt':':detetive_pele_escura:','it':':detective_carnagione_scura:','fa':':کارآگاه_پوست_آبنوسی:','id':':detektif_warna_kulit_gelap:','zh':':侦探_较深肤色:','ru':':детектив_очень_темный_тон_кожи:','tr':':dedektif_koyu_cilt_tonu:','ar':':شخص_يحقق_بشرة_بلون_غامق:'},'\U0001F575\U0001F3FB':{'en':':detective_light_skin_tone:','status':fully_qualified,'E':2,'de':':detektiv(in)_helle_hautfarbe:','es':':detective_tono_de_piel_claro:','fr':':détective_peau_claire:','ja':':探偵_薄い肌色:','ko':':탐정_하얀_피부:','pt':':detetive_pele_clara:','it':':detective_carnagione_chiara:','fa':':کارآگاه_پوست_سفید:','id':':detektif_warna_kulit_cerah:','zh':':侦探_较浅肤色:','ru':':детектив_очень_светлый_тон_кожи:','tr':':dedektif_açık_cilt_tonu:','ar':':شخص_يحقق_بشرة_بلون_فاتح:'},'\U0001F575\U0001F3FE':{'en':':detective_medium-dark_skin_tone:','status':fully_qualified,'E':2,'de':':detektiv(in)_mitteldunkle_hautfarbe:','es':':detective_tono_de_piel_oscuro_medio:','fr':':détective_peau_mate:','ja':':探偵_やや濃い肌色:','ko':':탐정_진한_갈색_피부:','pt':':detetive_pele_morena_escura:','it':':detective_carnagione_abbastanza_scura:','fa':':کارآگاه_پوست_گندمی:','id':':detektif_warna_kulit_gelap-sedang:','zh':':侦探_中等深肤色:','ru':':детектив_темный_тон_кожи:','tr':':dedektif_orta_koyu_cilt_tonu:','ar':':شخص_يحقق_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F575\U0001F3FC':{'en':':detective_medium-light_skin_tone:','status':fully_qualified,'E':2,'de':':detektiv(in)_mittelhelle_hautfarbe:','es':':detective_tono_de_piel_claro_medio:','fr':':détective_peau_moyennement_claire:','ja':':探偵_やや薄い肌色:','ko':':탐정_연한_갈색_피부:','pt':':detetive_pele_morena_clara:','it':':detective_carnagione_abbastanza_chiara:','fa':':کارآگاه_پوست_روشن:','id':':detektif_warna_kulit_cerah-sedang:','zh':':侦探_中等浅肤色:','ru':':детектив_светлый_тон_кожи:','tr':':dedektif_orta_açık_cilt_tonu:','ar':':شخص_يحقق_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F575\U0001F3FD':{'en':':detective_medium_skin_tone:','status':fully_qualified,'E':2,'de':':detektiv(in)_mittlere_hautfarbe:','es':':detective_tono_de_piel_medio:','fr':':détective_peau_légèrement_mate:','ja':':探偵_中間の肌色:','ko':':탐정_갈색_피부:','pt':':detetive_pele_morena:','it':':detective_carnagione_olivastra:','fa':':کارآگاه_پوست_طلایی:','id':':detektif_warna_kulit_sedang:','zh':':侦探_中等肤色:','ru':':детектив_средний_тон_кожи:','tr':':dedektif_orta_cilt_tonu:','ar':':شخص_يحقق_بشرة_بلون_معتدل:'},'\U00002666\U0000FE0F':{'en':':diamond_suit:','status':fully_qualified,'E':0.6,'alias':[':diamonds:'],'variant':True,'de':':karo:','es':':palo_de_diamantes:','fr':':carreau:','ja':':ダイヤ:','ko':':다이아몬드:','pt':':naipe_de_ouros:','it':':quadri:','fa':':خال_خشت:','id':':wajik:','zh':':方片:','ru':':бубны:','tr':':karo:','ar':':ورقة_لعب_بشكل_ماسة:'},'\U00002666':{'en':':diamond_suit:','status':unqualified,'E':0.6,'alias':[':diamonds:'],'variant':True,'de':':karo:','es':':palo_de_diamantes:','fr':':carreau:','ja':':ダイヤ:','ko':':다이아몬드:','pt':':naipe_de_ouros:','it':':quadri:','fa':':خال_خشت:','id':':wajik:','zh':':方片:','ru':':бубны:','tr':':karo:','ar':':ورقة_لعب_بشكل_ماسة:'},'\U0001F4A0':{'en':':diamond_with_a_dot:','status':fully_qualified,'E':0.6,'alias':[':diamond_shape_with_a_dot_inside:'],'de':':rautenform_mit_punkt:','es':':rombo_con_pétalo:','fr':':diamant_avec_un_point:','ja':':ドット付きひし形:','ko':':점이_찍힌_다이아몬드_모양:','pt':':diamante_com_um_ponto:','it':':petalo_di_fiore:','fa':':لوزی_نقطه_دار:','id':':wajik_bertitik:','zh':':带圆点的菱形:','ru':':ромб_с_точкой:','tr':':içinde_noktalı_karo:','ar':':ماسة_مع_نقطة:'},'\U0001F505':{'en':':dim_button:','status':fully_qualified,'E':1,'alias':[':low_brightness:'],'de':':taste_dimmen:','es':':brillo_bajo:','fr':':luminosité_faible:','ja':':低輝度:','ko':':밝기_낮음:','pt':':botão_de_diminuir_brilho:','it':':luminosità_bassa:','fa':':دکمهٔ_تار_کردن_صفحه:','id':':tombol_redupkan:','zh':':低亮度按钮:','ru':':низкая_яркость:','tr':':düşük_parlaklık:','ar':':زر_سطوع_منخفض:'},'\U0001F61E':{'en':':disappointed_face:','status':fully_qualified,'E':0.6,'alias':[':disappointed:'],'de':':enttäuschtes_gesicht:','es':':cara_decepcionada:','fr':':visage_déçu:','ja':':失望した顔:','ko':':실망한_얼굴:','pt':':rosto_desapontado:','it':':faccina_delusa:','fa':':ناامید:','id':':wajah_kecewa:','zh':':失望:','ru':':разочарование:','tr':':memnuniyetsiz_yüz:','ar':':وجه_محبط:'},'\U0001F978':{'en':':disguised_face:','status':fully_qualified,'E':13,'de':':verkleidet:','es':':cara_disfrazada:','fr':':visage_déguisé:','ja':':変装した顔:','ko':':변장한_얼굴:','pt':':rosto_disfarçado:','it':':faccina_travestita:','fa':':تغییرقیافه_داده:','id':':wajah_menyamar:','zh':':伪装的脸:','ru':':маскировка:','tr':':gizlenmiş_yüz:','ar':':وجع_مقنع:'},'\U00002797':{'en':':divide:','status':fully_qualified,'E':0.6,'alias':[':heavy_division_sign:'],'variant':True,'de':':geteilt_durch:','es':':división:','fr':':signe_diviser:','ja':':割り算:','ko':':나누기:','pt':':símbolo_de_divisão:','it':':divisione:','fa':':تقسیم_کردن:','id':':bagi:','zh':':除:','ru':':знак_деления:','tr':':bölü_işareti:','ar':':علامة_قسمة:'},'\U0001F93F':{'en':':diving_mask:','status':fully_qualified,'E':12,'de':':tauchmaske:','es':':máscara_de_buceo:','fr':':masque_de_plongée:','ja':':ダイビング_マスク:','ko':':다이빙_마스크:','pt':':máscara_de_mergulho:','it':':maschera_da_sub:','fa':':ماسک_غواصی:','id':':kacamata_selam:','zh':':潜水面罩:','ru':':маска_с_трубкой:','tr':':dalgıç_maskesi:','ar':':قناع_غطس:'},'\U0001FA94':{'en':':diya_lamp:','status':fully_qualified,'E':12,'de':':öllampe:','es':':lámpara_de_aceite:','fr':':diya:','ja':':ディヤランプ:','ko':':기름_램프:','pt':':lâmpada_de_óleo:','it':':diya:','fa':':چراغ_دایا:','id':':lampu_diya:','zh':':印度油灯:','ru':':масляная_лампа:','tr':':diya_lamba:','ar':':مصباح_ضياء:'},'\U0001F4AB':{'en':':dizzy:','status':fully_qualified,'E':0.6,'de':':schwindlig:','es':':símbolo_de_mareo:','fr':':étourdissement:','ja':':くらくら:','ko':':어지러움:','pt':':zonzo:','it':':stella_con_scia:','fa':':سرگیجه:','id':':pusing:','zh':':头晕:','ru':':кружащаяся_звезда:','tr':':baş_dönmesi:','ar':':دوار:'},'\U0001F9EC':{'en':':dna:','status':fully_qualified,'E':11,'de':':dna:','es':':adn:','fr':':adn:','ja':':dna:','ko':':dna:','pt':':dna:','it':':dna:','fa':':دی_ان_ای:','id':':dna:','zh':':DNA:','ru':':ДНК:','tr':':dna:','ar':':الحامض_النووي:'},'\U0001F9A4':{'en':':dodo:','status':fully_qualified,'E':13,'de':':dodo:','es':':dodo:','fr':':dodo:','ja':':ドードー:','ko':':도도새:','pt':':dodô:','it':':dodo:','fa':':دودو:','id':':dodo:','zh':':渡渡鸟:','ru':':дронт:','tr':':dodo_kuşu:','ar':':طائر_الدودو:'},'\U0001F415':{'en':':dog:','status':fully_qualified,'E':0.7,'alias':[':dog2:'],'variant':True,'de':':hund:','es':':perro:','fr':':chien:','ja':':イヌ:','ko':':개:','pt':':cachorro:','it':':cane:','fa':':سگ:','id':':anjing:','zh':':狗:','ru':':собака:','tr':':köpek:','ar':':كلب:'},'\U0001F436':{'en':':dog_face:','status':fully_qualified,'E':0.6,'alias':[':dog:'],'de':':hundegesicht:','es':':cara_de_perro:','fr':':tête_de_chien:','ja':':イヌの顔:','ko':':강아지_얼굴:','pt':':rosto_de_cachorro:','it':':muso_di_cane:','fa':':صورت_سگ:','id':':wajah_anjing:','zh':':狗脸:','ru':':морда_собаки:','tr':':köpek_yüzü:','ar':':وجه_كلب:'},'\U0001F4B5':{'en':':dollar_banknote:','status':fully_qualified,'E':0.6,'alias':[':dollar:'],'de':':dollar-banknote:','es':':billete_de_dólar:','fr':':billet_en_dollars:','ja':':ドル札:','ko':':달러:','pt':':nota_de_dólar:','it':':banconota_dollaro:','fa':':اسکناس_دلار:','id':':uang_kertas_dolar:','zh':':美元:','ru':':банкнота_доллара:','tr':':dolar_banknot:','ar':':ورقة_دولار:'},'\U0001F42C':{'en':':dolphin:','status':fully_qualified,'E':0.6,'alias':[':flipper:'],'de':':delfin:','es':':delfín:','fr':':dauphin:','ja':':イルカ:','ko':':돌고래:','pt':':golfinho:','it':':delfino:','fa':':دلفین:','id':':lumba-lumba:','zh':':海豚:','ru':':дельфин:','tr':':yunus:','ar':':دولفين:'},'\U0001FACF':{'en':':donkey:','status':fully_qualified,'E':15,'de':':esel:','es':':burro:','fr':':âne:','ja':':ロバ:','ko':':당나귀:','pt':':burro:','it':':asino:','fa':':الاغ:','id':':keledai:','zh':':驴:','ru':':осел:','tr':':eşek:','ar':':حمار:'},'\U0001F6AA':{'en':':door:','status':fully_qualified,'E':0.6,'de':':tür:','es':':puerta:','fr':':porte:','ja':':ドア:','ko':':문:','pt':':porta:','it':':porta:','fa':':در:','id':':pintu:','zh':':门:','ru':':дверь:','tr':':kapı:','ar':':باب:'},'\U0001FAE5':{'en':':dotted_line_face:','status':fully_qualified,'E':14,'de':':gestricheltes_gesicht:','es':':cara_con_línea_de_puntos:','fr':':visage_en_pointillés:','ja':':点線の顔:','ko':':테두리가_점선으로_된_얼굴:','pt':':rosto_com_linha_pontilhada:','it':':faccina_tratteggiata:','fa':':چهره_با_خط_چین:','id':':wajah_dengan_garis_titik-titik:','zh':':虚线脸:','ru':':лицо_пунктиром:','tr':':noktalı_çizgili_yüz:','ar':':وجه_محدد_بنقاط:'},'\U0001F52F':{'en':':dotted_six-pointed_star:','status':fully_qualified,'E':0.6,'alias':[':six_pointed_star:', ':dotted_six_pointed_star:'],'de':':hexagramm_mit_punkt:','es':':estrella_de_seis_puntas:','fr':':étoile_à_6_branches:','ja':':六芒星:','ko':':육각_별:','pt':':estrela_de_seis_pontas:','it':':stella_a_sei_punte:','fa':':ستارهٔ_شش_پر:','id':':bintang_sudut_enam_bertitik:','zh':':带中间点的六芒星:','ru':':шестиконечная_звезда:','tr':':altı_köşeli_yıldız:','ar':':نجمة_سداسية:'},'\U000027BF':{'en':':double_curly_loop:','status':fully_qualified,'E':1,'alias':[':loop:'],'variant':True,'de':':doppelschleife:','es':':bucle_doble:','fr':':double_boucle:','ja':':ダブルループ:','ko':':두_개의_고리:','pt':':loop_encaracolado_duas_vezes:','it':':doppio_occhiello:','fa':':حلقه_مجعد_دوتایی:','id':':untaian_bergelombang_ganda:','zh':':双卷曲环:','ru':':двойная_петля:','tr':':çift_kıvrık_döngü:','ar':':عقدة_مزدوجة:'},'\U0000203C\U0000FE0F':{'en':':double_exclamation_mark:','status':fully_qualified,'E':0.6,'alias':[':bangbang:'],'variant':True,'de':':doppeltes_ausrufezeichen:','es':':exclamación_doble:','fr':':double_point_d’exclamation:','ja':':二重感嘆符:','ko':':두_개의_느낌표:','pt':':dupla_exclamação:','it':':doppio_punto_esclamativo:','fa':':علامت_تعجب_دوتایی:','id':':tanda_seru_ganda:','zh':':双感叹号:','ru':':два_восклицательных_знака:','tr':':çift_ünlem:','ar':':تعجب_مزدوج:'},'\U0000203C':{'en':':double_exclamation_mark:','status':unqualified,'E':0.6,'alias':[':bangbang:'],'variant':True,'de':':doppeltes_ausrufezeichen:','es':':exclamación_doble:','fr':':double_point_d’exclamation:','ja':':二重感嘆符:','ko':':두_개의_느낌표:','pt':':dupla_exclamação:','it':':doppio_punto_esclamativo:','fa':':علامت_تعجب_دوتایی:','id':':tanda_seru_ganda:','zh':':双感叹号:','ru':':два_восклицательных_знака:','tr':':çift_ünlem:','ar':':تعجب_مزدوج:'},'\U0001F369':{'en':':doughnut:','status':fully_qualified,'E':0.6,'de':':donut:','es':':dónut:','fr':':doughnut:','ja':':ドーナツ:','ko':':도넛:','pt':':donut:','it':':ciambella:','fa':':دونات:','id':':donat:','zh':':甜甜圈:','ru':':пончик:','tr':':donut:','ar':':كعكة_محلاة:'},'\U0001F54A\U0000FE0F':{'en':':dove:','status':fully_qualified,'E':0.7,'alias':[':dove_of_peace:'],'variant':True,'de':':taube:','es':':paloma:','fr':':colombe:','ja':':ハト:','ko':':비둘기:','pt':':pomba_branca:','it':':colomba:','fa':':کبوتر:','id':':merpati:','zh':':鸽:','ru':':голубь:','tr':':güvercin:','ar':':حمامة:'},'\U0001F54A':{'en':':dove:','status':unqualified,'E':0.7,'alias':[':dove_of_peace:'],'variant':True,'de':':taube:','es':':paloma:','fr':':colombe:','ja':':ハト:','ko':':비둘기:','pt':':pomba_branca:','it':':colomba:','fa':':کبوتر:','id':':merpati:','zh':':鸽:','ru':':голубь:','tr':':güvercin:','ar':':حمامة:'},'\U00002199\U0000FE0F':{'en':':down-left_arrow:','status':fully_qualified,'E':0.6,'alias':[':arrow_lower_left:', ':down_left_arrow:'],'variant':True,'de':':pfeil_nach_links_unten:','es':':flecha_hacia_la_esquina_inferior_izquierda:','fr':':flèche_bas_gauche:','ja':':左下矢印:','ko':':좌하향_화살표:','pt':':seta_para_baixo_e_para_a_esquerda:','it':':freccia_in_basso_a_sinistra:','fa':':پیکان_پایین_چپ:','id':':tanda_panah_kiri_bawah:','zh':':左下箭头:','ru':':стрелка_влево-вниз:','tr':':sol_aşağı_ok:','ar':':سهم_لأسفل_اليسار:'},'\U00002199':{'en':':down-left_arrow:','status':unqualified,'E':0.6,'alias':[':arrow_lower_left:', ':down_left_arrow:'],'variant':True,'de':':pfeil_nach_links_unten:','es':':flecha_hacia_la_esquina_inferior_izquierda:','fr':':flèche_bas_gauche:','ja':':左下矢印:','ko':':좌하향_화살표:','pt':':seta_para_baixo_e_para_a_esquerda:','it':':freccia_in_basso_a_sinistra:','fa':':پیکان_پایین_چپ:','id':':tanda_panah_kiri_bawah:','zh':':左下箭头:','ru':':стрелка_влево_вниз:','tr':':sol_aşağı_ok:','ar':':سهم_لأسفل_اليسار:'},'\U00002198\U0000FE0F':{'en':':down-right_arrow:','status':fully_qualified,'E':0.6,'alias':[':arrow_lower_right:', ':down_right_arrow:'],'variant':True,'de':':pfeil_nach_rechts_unten:','es':':flecha_hacia_la_esquina_inferior_derecha:','fr':':flèche_bas_droite:','ja':':右下矢印:','ko':':우하향_화살표:','pt':':seta_para_baixo_e_para_a_direita:','it':':freccia_in_basso_a_destra:','fa':':پیکان_پایین_راست:','id':':tanda_panah_kanan_bawah:','zh':':右下箭头:','ru':':стрелка_вправо-вниз:','tr':':sağ_aşağı_ok:','ar':':سهم_لأسفل_اليمين:'},'\U00002198':{'en':':down-right_arrow:','status':unqualified,'E':0.6,'alias':[':arrow_lower_right:', ':down_right_arrow:'],'variant':True,'de':':pfeil_nach_rechts_unten:','es':':flecha_hacia_la_esquina_inferior_derecha:','fr':':flèche_bas_droite:','ja':':右下矢印:','ko':':우하향_화살표:','pt':':seta_para_baixo_e_para_a_direita:','it':':freccia_in_basso_a_destra:','fa':':پیکان_پایین_راست:','id':':tanda_panah_kanan_bawah:','zh':':右下箭头:','ru':':стрелка_вправо_вниз:','tr':':sağ_aşağı_ok:','ar':':سهم_لأسفل_اليمين:'},'\U00002B07\U0000FE0F':{'en':':down_arrow:','status':fully_qualified,'E':0.6,'alias':[':arrow_down:'],'variant':True,'de':':pfeil_nach_unten:','es':':flecha_hacia_abajo:','fr':':flèche_bas:','ja':':下矢印:','ko':':하향_화살표:','pt':':seta_para_baixo:','it':':freccia_rivolta_verso_il_basso:','fa':':پیکان_پایین:','id':':tanda_panah_bawah:','zh':':向下箭头:','ru':':стрелка_вниз:','tr':':aşağı_ok:','ar':':سهم_لأسفل:'},'\U00002B07':{'en':':down_arrow:','status':unqualified,'E':0.6,'alias':[':arrow_down:'],'variant':True,'de':':pfeil_nach_unten:','es':':flecha_hacia_abajo:','fr':':flèche_bas:','ja':':下矢印:','ko':':하향_화살표:','pt':':seta_para_baixo:','it':':freccia_rivolta_verso_il_basso:','fa':':پیکان_پایین:','id':':tanda_panah_bawah:','zh':':向下箭头:','ru':':стрелка_вниз:','tr':':aşağı_ok:','ar':':سهم_لأسفل:'},'\U0001F613':{'en':':downcast_face_with_sweat:','status':fully_qualified,'E':0.6,'alias':[':sweat:'],'de':':bedrücktes_gesicht_mit_schweiß:','es':':cara_con_sudor_frío:','fr':':visage_démoralisé_avec_goutte_de_sueur:','ja':':冷や汗:','ko':':식은땀_흘리는_얼굴:','pt':':rosto_cabisbaixo_com_gota_de_suor:','it':':faccina_abbattuta_sudata:','fa':':عرق_سرد:','id':':wajah_berkeringat:','zh':':汗:','ru':':в_холодном_поту:','tr':':soğuk_ter:','ar':':وجه_بعرق_بارد:'},'\U0001F53D':{'en':':downwards_button:','status':fully_qualified,'E':0.6,'alias':[':arrow_down_small:'],'de':':abwärts-schaltfläche:','es':':triángulo_hacia_abajo:','fr':':petit_triangle_bas:','ja':':下ボタン:','ko':':아래쪽_삼각형:','pt':':botão_apontando_para_baixo:','it':':pulsante_a_triangolo_rivolto_verso_il_basso:','fa':':دکمهٔ_پایین:','id':':tombol_turun:','zh':':向下三角形按钮:','ru':':треугольник_острием_вниз:','tr':':aşağı_düğmesi:','ar':':زر_لأسفل:'},'\U0001F409':{'en':':dragon:','status':fully_qualified,'E':1,'de':':drache:','es':':dragón:','fr':':dragon:','ja':':ドラゴン:','ko':':용:','pt':':dragão:','it':':drago:','fa':':اژدها:','id':':naga:','zh':':龙:','ru':':дракон:','tr':':ejderha:','ar':':تنين:'},'\U0001F432':{'en':':dragon_face:','status':fully_qualified,'E':0.6,'de':':drachengesicht:','es':':cara_de_dragón:','fr':':tête_de_dragon:','ja':':ドラゴンの顔:','ko':':용_얼굴:','pt':':rosto_de_dragão:','it':':testa_di_drago:','fa':':صورت_اژدها:','id':':wajah_naga:','zh':':龙头:','ru':':голова_дракона:','tr':':ejderha_yüzü:','ar':':وجه_تنين:'},'\U0001F457':{'en':':dress:','status':fully_qualified,'E':0.6,'de':':kleid:','es':':vestido:','fr':':robe:','ja':':ワンピース:','ko':':원피스:','pt':':vestido:','it':':vestito:','fa':':پیراهن_زنانه:','id':':gaun:','zh':':连衣裙:','ru':':платье:','tr':':elbise:','ar':':فستان:'},'\U0001F924':{'en':':drooling_face:','status':fully_qualified,'E':3,'de':':sabberndes_gesicht:','es':':cara_babeando:','fr':':visage_qui_bave:','ja':':よだれを垂らした顔:','ko':':침_흘리는_얼굴:','pt':':rosto_babando:','it':':faccina_che_sbava:','fa':':آب_افتادن_دهان:','id':':ngiler:','zh':':流口水:','ru':':пускает_слюни:','tr':':salya_akıtan_yüz:','ar':':وجه_بلعاب_سائل:'},'\U0001FA78':{'en':':drop_of_blood:','status':fully_qualified,'E':12,'de':':blutstropfen:','es':':gota_de_sangre:','fr':':goutte_de_sang:','ja':':血液:','ko':':핏방울:','pt':':gota_de_sangue:','it':':goccia_di_sangue:','fa':':قطره_خون:','id':':tetesan_darah:','zh':':血滴:','ru':':капля_крови:','tr':':kan_damlası:','ar':':قطرة_دم:'},'\U0001F4A7':{'en':':droplet:','status':fully_qualified,'E':0.6,'de':':tropfen:','es':':gota:','fr':':goutte_d’eau:','ja':':水滴:','ko':':물방울:','pt':':gota:','it':':goccia:','fa':':قطره:','id':':tetesan_air:','zh':':水滴:','ru':':капля:','tr':':damlacık:','ar':':قطرة:'},'\U0001F941':{'en':':drum:','status':fully_qualified,'E':3,'de':':trommel:','es':':tambor:','fr':':batterie:','ja':':ドラム:','ko':':드럼:','pt':':tambor:','it':':tamburo:','fa':':طبل:','id':':drum:','zh':':鼓:','ru':':барабан:','tr':':davul:','ar':':طبلة:'},'\U0001F986':{'en':':duck:','status':fully_qualified,'E':3,'de':':ente:','es':':pato:','fr':':canard:','ja':':カモ:','ko':':오리:','pt':':pato:','it':':anatra:','fa':':مرغابی:','id':':bebek:','zh':':鸭子:','ru':':утка:','tr':':ördek:','ar':':بطة:'},'\U0001F95F':{'en':':dumpling:','status':fully_qualified,'E':5,'de':':teigtasche:','es':':dumpling:','fr':':boulette_de_pâte:','ja':':点心:','ko':':만두:','pt':':bolinho_asiático:','it':':raviolo:','fa':':کوفته:','id':':dumpling:','zh':':饺子:','ru':':вареник:','tr':':mantı:','ar':':زلابية_صينية:'},'\U0001F4C0':{'en':':dvd:','status':fully_qualified,'E':0.6,'de':':dvd:','es':':disco_dvd:','fr':':dvd:','ja':':dvd:','ko':':디비디:','pt':':dvd:','it':':dvd:','fa':':دی_وی_دی:','id':':dvd:','zh':':DVD:','ru':':DVD:','tr':':dvd:','ar':':قرص_dvd:'},'\U0001F4E7':{'en':':e-mail:','status':fully_qualified,'E':0.6,'alias':[':email:', ':e_mail:'],'de':':e-mail:','es':':correo_electrónico:','fr':':e-mail:','ja':':eメール:','ko':':이메일:','pt':':e-mail:','it':':e-mail:','fa':':ایمیل:','id':':email:','zh':':电子邮件:','ru':':электронная_почта:','tr':':e-posta:','ar':':بريد_إلكتروني:'},'\U0001F985':{'en':':eagle:','status':fully_qualified,'E':3,'de':':adler:','es':':águila:','fr':':aigle:','ja':':ワシ:','ko':':독수리:','pt':':águia:','it':':aquila:','fa':':عقاب:','id':':elang:','zh':':鹰:','ru':':орел:','tr':':kartal:','ar':':نسر:'},'\U0001F442':{'en':':ear:','status':fully_qualified,'E':0.6,'variant':True,'de':':ohr:','es':':oreja:','fr':':oreille:','ja':':耳:','ko':':귀:','pt':':orelha:','it':':orecchio:','fa':':گوش:','id':':telinga:','zh':':耳朵:','ru':':ухо:','tr':':kulak:','ar':':أذن:'},'\U0001F442\U0001F3FF':{'en':':ear_dark_skin_tone:','status':fully_qualified,'E':1,'de':':ohr_dunkle_hautfarbe:','es':':oreja_tono_de_piel_oscuro:','fr':':oreille_peau_foncée:','ja':':耳_濃い肌色:','ko':':귀_검은색_피부:','pt':':orelha_pele_escura:','it':':orecchio_carnagione_scura:','fa':':گوش_پوست_آبنوسی:','id':':telinga_warna_kulit_gelap:','zh':':耳朵_较深肤色:','ru':':ухо_очень_темный_тон_кожи:','tr':':kulak_koyu_cilt_tonu:','ar':':أذن_بشرة_بلون_غامق:'},'\U0001F442\U0001F3FB':{'en':':ear_light_skin_tone:','status':fully_qualified,'E':1,'de':':ohr_helle_hautfarbe:','es':':oreja_tono_de_piel_claro:','fr':':oreille_peau_claire:','ja':':耳_薄い肌色:','ko':':귀_하얀_피부:','pt':':orelha_pele_clara:','it':':orecchio_carnagione_chiara:','fa':':گوش_پوست_سفید:','id':':telinga_warna_kulit_cerah:','zh':':耳朵_较浅肤色:','ru':':ухо_очень_светлый_тон_кожи:','tr':':kulak_açık_cilt_tonu:','ar':':أذن_بشرة_بلون_فاتح:'},'\U0001F442\U0001F3FE':{'en':':ear_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':ohr_mitteldunkle_hautfarbe:','es':':oreja_tono_de_piel_oscuro_medio:','fr':':oreille_peau_mate:','ja':':耳_やや濃い肌色:','ko':':귀_진한_갈색_피부:','pt':':orelha_pele_morena_escura:','it':':orecchio_carnagione_abbastanza_scura:','fa':':گوش_پوست_گندمی:','id':':telinga_warna_kulit_gelap-sedang:','zh':':耳朵_中等深肤色:','ru':':ухо_темный_тон_кожи:','tr':':kulak_orta_koyu_cilt_tonu:','ar':':أذن_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F442\U0001F3FC':{'en':':ear_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':ohr_mittelhelle_hautfarbe:','es':':oreja_tono_de_piel_claro_medio:','fr':':oreille_peau_moyennement_claire:','ja':':耳_やや薄い肌色:','ko':':귀_연한_갈색_피부:','pt':':orelha_pele_morena_clara:','it':':orecchio_carnagione_abbastanza_chiara:','fa':':گوش_پوست_روشن:','id':':telinga_warna_kulit_cerah-sedang:','zh':':耳朵_中等浅肤色:','ru':':ухо_светлый_тон_кожи:','tr':':kulak_orta_açık_cilt_tonu:','ar':':أذن_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F442\U0001F3FD':{'en':':ear_medium_skin_tone:','status':fully_qualified,'E':1,'de':':ohr_mittlere_hautfarbe:','es':':oreja_tono_de_piel_medio:','fr':':oreille_peau_légèrement_mate:','ja':':耳_中間の肌色:','ko':':귀_갈색_피부:','pt':':orelha_pele_morena:','it':':orecchio_carnagione_olivastra:','fa':':گوش_پوست_طلایی:','id':':telinga_warna_kulit_sedang:','zh':':耳朵_中等肤色:','ru':':ухо_средний_тон_кожи:','tr':':kulak_orta_cilt_tonu:','ar':':أذن_بشرة_بلون_معتدل:'},'\U0001F33D':{'en':':ear_of_corn:','status':fully_qualified,'E':0.6,'alias':[':corn:'],'de':':maiskolben:','es':':espiga_de_maíz:','fr':':épi_de_maïs:','ja':':とうもろこし:','ko':':옥수수:','pt':':milho:','it':':pannocchia:','fa':':بلال_ذرت:','id':':bonggol_jagung:','zh':':玉米:','ru':':кукурузный_початок:','tr':':mısır_koçanı:','ar':':ذرة:'},'\U0001F9BB':{'en':':ear_with_hearing_aid:','status':fully_qualified,'E':12,'de':':ohr_mit_hörgerät:','es':':oreja_con_audífono:','fr':':oreille_appareillée:','ja':':補聴器を付けた耳:','ko':':보청기를_낀_귀:','pt':':ouvido_com_aparelho_auditivo:','it':':orecchio_con_apparecchio_acustico:','fa':':سمعک_در_گوش:','id':':telinga_dengan_alat_bantu_dengar:','zh':':戴助听器的耳朵:','ru':':ухо_со_слуховым_аппаратом:','tr':':işitme_cihazı_olan_kulak:','ar':':أذن_مع_جهاز_سمعي:'},'\U0001F9BB\U0001F3FF':{'en':':ear_with_hearing_aid_dark_skin_tone:','status':fully_qualified,'E':12,'de':':ohr_mit_hörgerät_dunkle_hautfarbe:','es':':oreja_con_audífono_tono_de_piel_oscuro:','fr':':oreille_appareillée_peau_foncée:','ja':':補聴器を付けた耳_濃い肌色:','ko':':보청기를_낀_귀_검은색_피부:','pt':':ouvido_com_aparelho_auditivo_pele_escura:','it':':orecchio_con_apparecchio_acustico_carnagione_scura:','fa':':سمعک_در_گوش_پوست_آبنوسی:','id':':telinga_dengan_alat_bantu_dengar_warna_kulit_gelap:','zh':':戴助听器的耳朵_较深肤色:','ru':':ухо_со_слуховым_аппаратом_очень_темный_тон_кожи:','tr':':işitme_cihazı_olan_kulak_koyu_cilt_tonu:','ar':':أذن_مع_جهاز_سمعي_بشرة_بلون_غامق:'},'\U0001F9BB\U0001F3FB':{'en':':ear_with_hearing_aid_light_skin_tone:','status':fully_qualified,'E':12,'de':':ohr_mit_hörgerät_helle_hautfarbe:','es':':oreja_con_audífono_tono_de_piel_claro:','fr':':oreille_appareillée_peau_claire:','ja':':補聴器を付けた耳_薄い肌色:','ko':':보청기를_낀_귀_하얀_피부:','pt':':ouvido_com_aparelho_auditivo_pele_clara:','it':':orecchio_con_apparecchio_acustico_carnagione_chiara:','fa':':سمعک_در_گوش_پوست_سفید:','id':':telinga_dengan_alat_bantu_dengar_warna_kulit_cerah:','zh':':戴助听器的耳朵_较浅肤色:','ru':':ухо_со_слуховым_аппаратом_очень_светлый_тон_кожи:','tr':':işitme_cihazı_olan_kulak_açık_cilt_tonu:','ar':':أذن_مع_جهاز_سمعي_بشرة_بلون_فاتح:'},'\U0001F9BB\U0001F3FE':{'en':':ear_with_hearing_aid_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':ohr_mit_hörgerät_mitteldunkle_hautfarbe:','es':':oreja_con_audífono_tono_de_piel_oscuro_medio:','fr':':oreille_appareillée_peau_mate:','ja':':補聴器を付けた耳_やや濃い肌色:','ko':':보청기를_낀_귀_진한_갈색_피부:','pt':':ouvido_com_aparelho_auditivo_pele_morena_escura:','it':':orecchio_con_apparecchio_acustico_carnagione_abbastanza_scura:','fa':':سمعک_در_گوش_پوست_گندمی:','id':':telinga_dengan_alat_bantu_dengar_warna_kulit_gelap-sedang:','zh':':戴助听器的耳朵_中等深肤色:','ru':':ухо_со_слуховым_аппаратом_темный_тон_кожи:','tr':':işitme_cihazı_olan_kulak_orta_koyu_cilt_tonu:','ar':':أذن_مع_جهاز_سمعي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9BB\U0001F3FC':{'en':':ear_with_hearing_aid_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':ohr_mit_hörgerät_mittelhelle_hautfarbe:','es':':oreja_con_audífono_tono_de_piel_claro_medio:','fr':':oreille_appareillée_peau_moyennement_claire:','ja':':補聴器を付けた耳_やや薄い肌色:','ko':':보청기를_낀_귀_연한_갈색_피부:','pt':':ouvido_com_aparelho_auditivo_pele_morena_clara:','it':':orecchio_con_apparecchio_acustico_carnagione_abbastanza_chiara:','fa':':سمعک_در_گوش_پوست_روشن:','id':':telinga_dengan_alat_bantu_dengar_warna_kulit_cerah-sedang:','zh':':戴助听器的耳朵_中等浅肤色:','ru':':ухо_со_слуховым_аппаратом_светлый_тон_кожи:','tr':':işitme_cihazı_olan_kulak_orta_açık_cilt_tonu:','ar':':أذن_مع_جهاز_سمعي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9BB\U0001F3FD':{'en':':ear_with_hearing_aid_medium_skin_tone:','status':fully_qualified,'E':12,'de':':ohr_mit_hörgerät_mittlere_hautfarbe:','es':':oreja_con_audífono_tono_de_piel_medio:','fr':':oreille_appareillée_peau_légèrement_mate:','ja':':補聴器を付けた耳_中間の肌色:','ko':':보청기를_낀_귀_갈색_피부:','pt':':ouvido_com_aparelho_auditivo_pele_morena:','it':':orecchio_con_apparecchio_acustico_carnagione_olivastra:','fa':':سمعک_در_گوش_پوست_طلایی:','id':':telinga_dengan_alat_bantu_dengar_warna_kulit_sedang:','zh':':戴助听器的耳朵_中等肤色:','ru':':ухо_со_слуховым_аппаратом_средний_тон_кожи:','tr':':işitme_cihazı_olan_kulak_orta_cilt_tonu:','ar':':أذن_مع_جهاز_سمعي_بشرة_بلون_معتدل:'},'\U0001F95A':{'en':':egg:','status':fully_qualified,'E':3,'alias':[':egg2:'],'de':':ei:','es':':huevo:','fr':':œuf:','ja':':卵:','ko':':달걀:','pt':':ovo:','it':':uovo:','fa':':تخم_مرغ:','id':':telur:','zh':':蛋:','ru':':яйцо:','tr':':yumurta:','ar':':بيض:'},'\U0001F346':{'en':':eggplant:','status':fully_qualified,'E':0.6,'de':':aubergine:','es':':berenjena:','fr':':aubergine:','ja':':ナス:','ko':':가지:','pt':':berinjela:','it':':melanzana:','fa':':بادمجان:','id':':terung:','zh':':茄子:','ru':':баклажан:','tr':':patlıcan:','ar':':باذنجان:'},'\U00002734\U0000FE0F':{'en':':eight-pointed_star:','status':fully_qualified,'E':0.6,'alias':[':eight_pointed_black_star:', ':eight_pointed_star:'],'variant':True,'de':':stern_mit_acht_zacken:','es':':estrella_de_ocho_puntas:','fr':':étoile_huit_branches:','ja':':八角星:','ko':':팔각_별:','pt':':estrela_de_oito_pontas:','it':':stella_stilizzata:','fa':':ستاره_هشت_پر:','id':':bintang_berujung_delapan:','zh':':八角星:','ru':':восьмиконечная_звезда:','tr':':sekiz_noktalı_yıldız:','ar':':نجمة_بثمان_نقاط:'},'\U00002734':{'en':':eight-pointed_star:','status':unqualified,'E':0.6,'alias':[':eight_pointed_black_star:', ':eight_pointed_star:'],'variant':True,'de':':achtstrahliger_stern:','es':':estrella_de_ocho_puntas:','fr':':étoile_huit_branches:','ja':':八角星:','ko':':팔각_별:','pt':':estrela_de_oito_pontas:','it':':stella_stilizzata:','fa':':ستاره_هشت_پر:','id':':bintang_berujung_delapan:','zh':':八角星:','ru':':восьмиконечная_звезда:','tr':':sekiz_noktalı_yıldız:','ar':':نجمة_بثمان_نقاط:'},'\U00002733\U0000FE0F':{'en':':eight-spoked_asterisk:','status':fully_qualified,'E':0.6,'alias':[':eight_spoked_asterisk:'],'variant':True,'de':':achtzackiger_stern:','es':':asterisco_de_ocho_puntas:','fr':':astérisque_huit_branches:','ja':':八角アスタリスク:','ko':':여덟_갈래_별표:','pt':':asterisco_de_oito_pontas:','it':':asterisco:','fa':':ستاره_هشت_پره:','id':':bintang_berujung_lancip_delapan:','zh':':八轮辐星号:','ru':':восьмиконечная_звездочка:','tr':':sekiz_uçlu_yıldız:','ar':':نجمة_ثمانية:'},'\U00002733':{'en':':eight-spoked_asterisk:','status':unqualified,'E':0.6,'alias':[':eight_spoked_asterisk:'],'variant':True,'de':':achtzackiger_stern:','es':':asterisco_de_ocho_puntas:','fr':':astérisque_huit_branches:','ja':':八角アスタリスク:','ko':':여덟_갈래_별표:','pt':':asterisco_de_oito_pontas:','it':':asterisco:','fa':':ستاره_هشت_پره:','id':':bintang_berujung_lancip_delapan:','zh':':八轮辐星号:','ru':':восьмиконечная_звездочка:','tr':':sekiz_uçlu_yıldız:','ar':':نجمة_ثمانية:'},'\U0001F563':{'en':':eight-thirty:','status':fully_qualified,'E':0.7,'alias':[':clock830:', ':eight_thirty:'],'variant':True,'de':':8.30_uhr:','es':':ocho_y_media:','fr':':huit_heures_et_demie:','ja':':8時半:','ko':':여덟_시_반:','pt':':oito_e_meia:','it':':ore_otto_e_mezza:','fa':':ساعت_هشت_و_نیم:','id':':jam_setengah_sembilan:','zh':':八点半:','ru':':полдевятого:','tr':':sekiz_buçuk:','ar':':الثامنة_والنصف:'},'\U0001F557':{'en':':eight_o’clock:','status':fully_qualified,'E':0.6,'alias':[':clock8:', ':eight_oclock:'],'variant':True,'de':':8.00_uhr:','es':':8_en_punto:','fr':':huit_heures:','ja':':8時:','ko':':여덟_시:','pt':':8_horas:','it':':ore_otto:','fa':':ساعت_هشت:','id':':jam_delapan_tepat:','zh':':八点:','ru':':восемь_часов:','tr':':saat_8:','ar':':الساعة_الثامنة:'},'\U000023CF\U0000FE0F':{'en':':eject_button:','status':fully_qualified,'E':1,'alias':[':eject_symbol:'],'variant':True,'de':':auswerfen:','es':':expulsar:','fr':':bouton_éjecter:','ja':':取り出しボタン:','ko':':꺼냄_버튼:','pt':':botão_ejetar:','it':':pulsante_di_espulsione:','fa':':دکمه_خروج:','id':':tombol_keluarkan:','zh':':推出按钮:','ru':':выброс:','tr':':çıkarma_düğmesi:','ar':':زر_إخراج:'},'\U000023CF':{'en':':eject_button:','status':unqualified,'E':1,'alias':[':eject_symbol:'],'variant':True,'de':':auswerfen:','es':':expulsar:','fr':':bouton_éjecter:','ja':':取り出しボタン:','ko':':꺼냄_버튼:','pt':':botão_ejetar:','it':':pulsante_di_espulsione:','fa':':دکمهٔ_خروج:','id':':tombol_keluarkan:','zh':':推出按钮:','ru':':выброс:','tr':':çıkarma_düğmesi:','ar':':زر_إخراج:'},'\U0001F50C':{'en':':electric_plug:','status':fully_qualified,'E':0.6,'de':':netzstecker:','es':':enchufe_eléctrico:','fr':':câble_avec_fiche_électrique:','ja':':コンセント:','ko':':전기_플러그:','pt':':tomada_elétrica:','it':':spina_elettrica:','fa':':دوشاخه:','id':':steker:','zh':':电源插头:','ru':':вилка:','tr':':elektrik_fişi:','ar':':فيشة_كهرباء:'},'\U0001F418':{'en':':elephant:','status':fully_qualified,'E':0.6,'de':':elefant:','es':':elefante:','fr':':éléphant:','ja':':ゾウ:','ko':':코끼리:','pt':':elefante:','it':':elefante:','fa':':فیل:','id':':gajah:','zh':':大象:','ru':':слон:','tr':':fil:','ar':':فيل:'},'\U0001F6D7':{'en':':elevator:','status':fully_qualified,'E':13,'de':':fahrstuhl:','es':':ascensor:','fr':':ascenseur:','ja':':エレベーター:','ko':':엘리베이터:','pt':':elevador:','it':':ascensore:','fa':':بالابر:','id':':elevator:','zh':':电梯:','ru':':лифт:','tr':':asansör:','ar':':مصعد:'},'\U0001F566':{'en':':eleven-thirty:','status':fully_qualified,'E':0.7,'alias':[':clock1130:', ':eleven_thirty:'],'variant':True,'de':':11.30_uhr:','es':':once_y_media:','fr':':onze_heures_et_demie:','ja':':11時半:','ko':':열한_시_반:','pt':':onze_e_meia:','it':':ore_undici_e_mezza:','fa':':ساعت_یازده_و_نیم:','id':':jam_setengah_dua_belas:','zh':':十一点半:','ru':':полдвенадцатого:','tr':':on_bir_buçuk:','ar':':الحادية_عشر_ونصف:'},'\U0001F55A':{'en':':eleven_o’clock:','status':fully_qualified,'E':0.6,'alias':[':clock11:', ':eleven_oclock:'],'variant':True,'de':':11.00_uhr:','es':':11_en_punto:','fr':':onze_heures:','ja':':11時:','ko':':열한_시:','pt':':11_horas:','it':':ore_undici:','fa':':ساعت_یازده:','id':':jam_sebelas_tepat:','zh':':十一点:','ru':':одиннадцать_часов:','tr':':saat_11:','ar':':الساعة_الحادية_عشرة:'},'\U0001F9DD':{'en':':elf:','status':fully_qualified,'E':5,'de':':elf(e):','es':':elfo:','fr':':elfe:','ja':':エルフ:','ko':':엘프:','pt':':elfo:','it':':elfo:','fa':':اِلف:','id':':kurcaci:','zh':':小精灵:','ru':':эльф:','tr':':elf:','ar':':جني_صغير:'},'\U0001F9DD\U0001F3FF':{'en':':elf_dark_skin_tone:','status':fully_qualified,'E':5,'de':':elf(e)_dunkle_hautfarbe:','es':':elfo_tono_de_piel_oscuro:','fr':':elfe_peau_foncée:','ja':':エルフ_濃い肌色:','ko':':엘프_검은색_피부:','pt':':elfo_pele_escura:','it':':elfo_carnagione_scura:','fa':':اِلف_پوست_آبنوسی:','id':':kurcaci_warna_kulit_gelap:','zh':':小精灵_较深肤色:','ru':':эльф_очень_темный_тон_кожи:','tr':':elf_koyu_cilt_tonu:','ar':':جني_صغير_بشرة_بلون_غامق:'},'\U0001F9DD\U0001F3FB':{'en':':elf_light_skin_tone:','status':fully_qualified,'E':5,'de':':elf(e)_helle_hautfarbe:','es':':elfo_tono_de_piel_claro:','fr':':elfe_peau_claire:','ja':':エルフ_薄い肌色:','ko':':엘프_하얀_피부:','pt':':elfo_pele_clara:','it':':elfo_carnagione_chiara:','fa':':اِلف_پوست_سفید:','id':':kurcaci_warna_kulit_cerah:','zh':':小精灵_较浅肤色:','ru':':эльф_очень_светлый_тон_кожи:','tr':':elf_açık_cilt_tonu:','ar':':جني_صغير_بشرة_بلون_فاتح:'},'\U0001F9DD\U0001F3FE':{'en':':elf_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':elf(e)_mitteldunkle_hautfarbe:','es':':elfo_tono_de_piel_oscuro_medio:','fr':':elfe_peau_mate:','ja':':エルフ_やや濃い肌色:','ko':':엘프_진한_갈색_피부:','pt':':elfo_pele_morena_escura:','it':':elfo_carnagione_abbastanza_scura:','fa':':اِلف_پوست_گندمی:','id':':kurcaci_warna_kulit_gelap-sedang:','zh':':小精灵_中等深肤色:','ru':':эльф_темный_тон_кожи:','tr':':elf_orta_koyu_cilt_tonu:','ar':':جني_صغير_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9DD\U0001F3FC':{'en':':elf_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':elf(e)_mittelhelle_hautfarbe:','es':':elfo_tono_de_piel_claro_medio:','fr':':elfe_peau_moyennement_claire:','ja':':エルフ_やや薄い肌色:','ko':':엘프_연한_갈색_피부:','pt':':elfo_pele_morena_clara:','it':':elfo_carnagione_abbastanza_chiara:','fa':':اِلف_پوست_روشن:','id':':kurcaci_warna_kulit_cerah-sedang:','zh':':小精灵_中等浅肤色:','ru':':эльф_светлый_тон_кожи:','tr':':elf_orta_açık_cilt_tonu:','ar':':جني_صغير_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9DD\U0001F3FD':{'en':':elf_medium_skin_tone:','status':fully_qualified,'E':5,'de':':elf(e)_mittlere_hautfarbe:','es':':elfo_tono_de_piel_medio:','fr':':elfe_peau_légèrement_mate:','ja':':エルフ_中間の肌色:','ko':':엘프_갈색_피부:','pt':':elfo_pele_morena:','it':':elfo_carnagione_olivastra:','fa':':اِلف_پوست_طلایی:','id':':kurcaci_warna_kulit_sedang:','zh':':小精灵_中等肤色:','ru':':эльф_средний_тон_кожи:','tr':':elf_orta_cilt_tonu:','ar':':جني_صغير_بشرة_بلون_معتدل:'},'\U0001FAB9':{'en':':empty_nest:','status':fully_qualified,'E':14,'de':':leeres_nest:','es':':nido_vacío:','fr':':nid_vide:','ja':':空っぽの巣:','ko':':빈_둥지:','pt':':ninho_vazio:','it':':nido_vuoto:','fa':':آشیانه_خالی:','id':':sarang_burung_kosong:','zh':':空巢:','ru':':пустое_гнездо:','tr':':boş_yuva:','ar':':عش_فارغ:'},'\U0001F621':{'en':':enraged_face:','status':fully_qualified,'E':0.6,'alias':[':rage:', ':pout:'],'de':':schmollendes_gesicht:','es':':cara_cabreada:','fr':':visage_boudeur:','ja':':ふくれっ面:','ko':':뾰로통한_얼굴:','pt':':rosto_furioso:','it':':faccina_accigliata:','fa':':خشمگین:','id':':wajah_sangat_marah:','zh':':怒火中烧:','ru':':злится:','tr':':somurtkan_yüz:','ar':':وجه_مشتاط_غضبا:'},'\U00002709\U0000FE0F':{'en':':envelope:','status':fully_qualified,'E':0.6,'variant':True,'de':':briefumschlag:','es':':sobre:','fr':':enveloppe:','ja':':封筒:','ko':':봉투:','pt':':envelope:','it':':busta:','fa':':پاکت_نامه:','id':':amplop:','zh':':信封:','ru':':конверт:','tr':':zarf:','ar':':ظرف:'},'\U00002709':{'en':':envelope:','status':unqualified,'E':0.6,'variant':True,'de':':briefumschlag:','es':':sobre:','fr':':enveloppe:','ja':':封筒:','ko':':봉투:','pt':':envelope:','it':':busta:','fa':':پاکت_نامه:','id':':amplop:','zh':':信封:','ru':':конверт:','tr':':zarf:','ar':':ظرف:'},'\U0001F4E9':{'en':':envelope_with_arrow:','status':fully_qualified,'E':0.6,'de':':umschlag_mit_pfeil:','es':':sobre_con_flecha:','fr':':enveloppe_avec_flèche:','ja':':メール受信:','ko':':보낸_편지:','pt':':envelope_com_seta:','it':':posta_in_uscita:','fa':':پاکت_نامه_با_پیکان:','id':':amplop_dengan_panah:','zh':':收邮件:','ru':':исходящее_письмо:','tr':':oklu_zarf:','ar':':بريد_صادر:'},'\U0001F4B6':{'en':':euro_banknote:','status':fully_qualified,'E':1,'alias':[':euro:'],'de':':euro-banknote:','es':':billete_de_euro:','fr':':billet_en_euros:','ja':':ユーロ札:','ko':':유로:','pt':':nota_de_euro:','it':':banconota_euro:','fa':':اسکناس_یورو:','id':':uang_kertas_euro:','zh':':欧元:','ru':':банкнота_евро:','tr':':euro_banknot:','ar':':ورقة_يورو:'},'\U0001F332':{'en':':evergreen_tree:','status':fully_qualified,'E':1,'de':':nadelbaum:','es':':árbol_de_hoja_perenne:','fr':':conifère:','ja':':常緑樹:','ko':':소나무:','pt':':conífera:','it':':albero_sempreverde:','fa':':همیشه_سبز:','id':':pohon_hijau_abadi:','zh':':松树:','ru':':елка:','tr':':yaprak_dökmeyen_ağaç:','ar':':شجرة_دائمة_الخضرة:'},'\U0001F411':{'en':':ewe:','status':fully_qualified,'E':0.6,'alias':[':sheep:'],'de':':schaf:','es':':oveja:','fr':':mouton:','ja':':羊:','ko':':양:','pt':':ovelha:','it':':pecora:','fa':':میش:','id':':domba_betina:','zh':':母羊:','ru':':овца:','tr':':koyun:','ar':':خروف:'},'\U00002049\U0000FE0F':{'en':':exclamation_question_mark:','status':fully_qualified,'E':0.6,'alias':[':interrobang:'],'variant':True,'de':':ausrufe-_und_fragezeichen:','es':':exclamación_e_interrogación:','fr':':points_d’exclamation_et_d’interrogation:','ja':':感嘆符疑問符:','ko':':느낌표와_물음표:','pt':':exclamação_com_interrogação:','it':':punto_esclamativo_e_interrogativo:','fa':':علامت_تعجب_و_سؤال:','id':':tanda_seru_dan_tanya:','zh':':感叹疑问号:','ru':':восклицательныи_и_вопросительныи_знаки:','tr':':ünlemli_soru_işareti:','ar':':تعجب_واستفهام:'},'\U00002049':{'en':':exclamation_question_mark:','status':unqualified,'E':0.6,'alias':[':interrobang:'],'variant':True,'de':':ausrufe-_und_fragezeichen:','es':':exclamación_e_interrogación:','fr':':points_d’exclamation_et_d’interrogation:','ja':':感嘆符疑問符:','ko':':느낌표와_물음표:','pt':':exclamação_com_interrogação:','it':':punto_esclamativo_e_interrogativo:','fa':':علامت_تعجب_و_سؤال:','id':':tanda_seru_dan_tanya:','zh':':感叹疑问号:','ru':':восклицательный_и_вопросительный_знаки:','tr':':ünlemli_soru_işareti:','ar':':تعجب_واستفهام:'},'\U0001F92F':{'en':':exploding_head:','status':fully_qualified,'E':5,'de':':explodierender_kopf:','es':':cabeza_explotando:','fr':':tête_qui_explose:','ja':':頭爆発:','ko':':폭발하는_얼굴:','pt':':cabeça_explodindo:','it':':testa_che_esplode:','fa':':سر_منفجرشده:','id':':kepala_meledak:','zh':':爆炸头:','ru':':взрыв_мозга:','tr':':patlayan_kafa:','ar':':وجه_برأس_منفجر:'},'\U0001F611':{'en':':expressionless_face:','status':fully_qualified,'E':1,'alias':[':expressionless:'],'de':':ausdrucksloses_gesicht:','es':':cara_sin_expresión:','fr':':visage_sans_expression:','ja':':無表情:','ko':':무표정한_얼굴:','pt':':rosto_inexpressivo:','it':':faccina_inespressiva:','fa':':صورت_بی_حالت:','id':':wajah_tanpa_ekspresi:','zh':':无语:','ru':':без_эмоций:','tr':':ifadesiz_yüz:','ar':':وجه_بدون_تعبير:'},'\U0001F441\U0000FE0F':{'en':':eye:','status':fully_qualified,'E':0.7,'variant':True,'de':':auge:','es':':ojo:','fr':':œil:','ja':':片目:','ko':':눈:','pt':':olho:','it':':occhio:','fa':':چشم:','id':':satu_mata:','zh':':眼睛:','ru':':глаз:','tr':':göz:','ar':':عين:'},'\U0001F441':{'en':':eye:','status':unqualified,'E':0.7,'variant':True,'de':':auge:','es':':ojo:','fr':':œil:','ja':':片目:','ko':':눈:','pt':':olho:','it':':occhio:','fa':':چشم:','id':':satu_mata:','zh':':眼睛:','ru':':глаз:','tr':':göz:','ar':':عين:'},'\U0001F441\U0000FE0F\U0000200D\U0001F5E8\U0000FE0F':{'en':':eye_in_speech_bubble:','status':fully_qualified,'E':2,'alias':[':eye_speech_bubble:'],'de':':auge_in_sprechblase:','es':':ojo_en_bocadillo_de_texto:','fr':':œil_dans_une_bulle_de_bd:','ja':':吹き出しの目:','ko':':말풍선_속_눈:','pt':':olho_no_balão_de_diálogo:','it':':occhio_nel_fumetto:','fa':':چشم_در_حباب_گفتار:','id':':mata_di_dalam_gelembung_ucapan:','zh':':眼睛对话框:','ru':':значок_свидетеля:','tr':':konuşma_balonunda_göz:','ar':':عين_في_فقاعة_كلام:'},'\U0001F441\U0000200D\U0001F5E8\U0000FE0F':{'en':':eye_in_speech_bubble:','status':unqualified,'E':2,'alias':[':eye_speech_bubble:'],'de':':auge_in_sprechblase:','es':':ojo_en_bocadillo_de_texto:','fr':':œil_dans_une_bulle_de_bd:','ja':':吹き出しの目:','ko':':말풍선_속_눈:','pt':':olho_no_balão_de_diálogo:','it':':occhio_nel_fumetto:','fa':':چشم_در_حباب_گفتار:','id':':mata_di_dalam_gelembung_ucapan:','zh':':眼睛对话框:','ru':':значок_свидетеля:','tr':':konuşma_balonunda_göz:','ar':':عين_في_فقاعة_كلام:'},'\U0001F441\U0000FE0F\U0000200D\U0001F5E8':{'en':':eye_in_speech_bubble:','status':minimally_qualified,'E':2,'alias':[':eye_speech_bubble:'],'de':':auge_in_sprechblase:','es':':ojo_en_bocadillo_de_texto:','fr':':œil_dans_une_bulle_de_bd:','ja':':吹き出しの目:','ko':':말풍선_속_눈:','pt':':olho_no_balão_de_diálogo:','it':':occhio_nel_fumetto:','fa':':چشم_در_حباب_گفتار:','id':':mata_di_dalam_gelembung_ucapan:','zh':':眼睛对话框:','ru':':значок_свидетеля:','tr':':konuşma_balonunda_göz:','ar':':عين_في_فقاعة_كلام:'},'\U0001F441\U0000200D\U0001F5E8':{'en':':eye_in_speech_bubble:','status':unqualified,'E':2,'alias':[':eye_speech_bubble:'],'de':':auge_in_sprechblase:','es':':ojo_en_bocadillo_de_texto:','fr':':œil_dans_une_bulle_de_bd:','ja':':吹き出しの目:','ko':':말풍선_속_눈:','pt':':olho_no_balão_de_diálogo:','it':':occhio_nel_fumetto:','fa':':چشم_در_حباب_گفتار:','id':':mata_di_dalam_gelembung_ucapan:','zh':':眼睛对话框:','ru':':значок_свидетеля:','tr':':konuşma_balonunda_göz:','ar':':عين_في_فقاعة_كلام:'},'\U0001F440':{'en':':eyes:','status':fully_qualified,'E':0.6,'de':':augen:','es':':ojos:','fr':':yeux:','ja':':目:','ko':':왕눈이_눈알:','pt':':olhos:','it':':occhi:','fa':':دو_چشم:','id':':dua_mata:','zh':':双眼:','ru':':глаза:','tr':':gözler:','ar':':عينان:'},'\U0001F618':{'en':':face_blowing_a_kiss:','status':fully_qualified,'E':0.6,'alias':[':kissing_heart:'],'de':':kuss_zuwerfendes_gesicht:','es':':cara_lanzando_un_beso:','fr':':visage_envoyant_un_bisou:','ja':':投げキッス:','ko':':키스를_보내는_얼굴:','pt':':rosto_mandando_um_beijo:','it':':faccina_che_manda_un_bacio:','fa':':بوسه_فرستادن:','id':':wajah_memberikan_ciuman_jauh:','zh':':飞吻:','ru':':воздушный_поцелуй:','tr':':öpücük_gönderen_yüz:','ar':':وجه_يرسل_قبلة:'},'\U0001F62E\U0000200D\U0001F4A8':{'en':':face_exhaling:','status':fully_qualified,'E':13.1,'de':':gesicht_das_ausatmet:','es':':cara_exhalando:','fr':':visage_expirant:','ja':':息を吐く顔:','ko':':날숨_쉬는_얼굴:','pt':':rosto_exalando:','it':':faccina_che_espira:','fa':':صورتک_در_حال_بازدم:','id':':wajah_mengembuskan_napas:','zh':':呼气:','ru':':выдыхает:','tr':':nefes_veren_yüz:','ar':':وجه_يزفر_الهواء:'},'\U0001F979':{'en':':face_holding_back_tears:','status':fully_qualified,'E':14,'de':':gesicht_das_tränen_zurückhält:','es':':cara_aguantándose_las_lágrimas:','fr':':visage_retenant_ses_larmes:','ja':':涙をこらえた顔:','ko':':눈물을_참는_얼굴:','pt':':rosto_segurando_as_lágrimas:','it':':faccina_che_trattiene_le_lacrime:','fa':':چهره_با_چشمان_پراشک:','id':':wajah_menahan_air_mata:','zh':':忍住泪水:','ru':':еле_сдерживает_слезы:','tr':':gözleri_dolmuş_yüz:','ar':':وجه_يحبس_دموعه:'},'\U0001F636\U0000200D\U0001F32B\U0000FE0F':{'en':':face_in_clouds:','status':fully_qualified,'E':13.1,'de':':gesicht_in_wolken:','es':':cara_en_las_nubes:','fr':':visage_dans_les_nuages:','ja':':雲の中の顔:','ko':':공상에_잠긴_얼굴:','pt':':rosto_nas_nuvens:','it':':testa_tra_le_nuvole:','fa':':صورتک_میان_ابر:','id':':wajah_di_awan:','zh':':迷茫:','ru':':лицо_в_облаках:','tr':':bulutlardaki_yüz:','ar':':وجه_في_الغيوم:'},'\U0001F636\U0000200D\U0001F32B':{'en':':face_in_clouds:','status':minimally_qualified,'E':13.1,'de':':gesicht_in_wolken:','es':':cara_entre_las_nubes:','fr':':visage_dans_les_nuages:','ja':':雲の中の顔:','ko':':공상에_잠긴_얼굴:','pt':':rosto_nas_nuvens:','it':':testa_tra_le_nuvole:','fa':':صورتک_میان_ابر:','id':':wajah_di_awan:','zh':':迷茫:','ru':':лицо_в_облаках:','tr':':bulutlardaki_yüz:','ar':':وجه_في_الغيوم:'},'\U0001F60B':{'en':':face_savoring_food:','status':fully_qualified,'E':0.6,'alias':[':yum:'],'de':':sich_die_lippen_leckendes_gesicht:','es':':cara_saboreando_comida:','fr':':miam:','ja':':にこにこぺろり:','ko':':맛있는_음식을_음미하는_얼굴:','pt':':rosto_saboreando_comida:','it':':faccina_che_si_lecca_i_baffi:','fa':':صورت_درحال_لیسیدن_لب:','id':':wajah_menikmati:','zh':':好吃:','ru':':вкусно:','tr':':lezzetli_yiyecek_beğenen_yüz:','ar':':وجه_يتلذذ_بالطعام:'},'\U0001F631':{'en':':face_screaming_in_fear:','status':fully_qualified,'E':0.6,'alias':[':scream:'],'de':':vor_angst_schreiendes_gesicht:','es':':cara_gritando_de_miedo:','fr':':visage_qui_hurle_de_peur:','ja':':恐怖:','ko':':공포에_절규하는_얼굴:','pt':':rosto_gritando_de_medo:','it':':faccina_terrorizzata:','fa':':جیغ_ترس:','id':':wajah_sangat_ketakutan:','zh':':吓死了:','ru':':в_ужасе:','tr':':korkuyla_çığlık_atan_yüz:','ar':':وجه_خائف_يصرخ:'},'\U0001F92E':{'en':':face_vomiting:','status':fully_qualified,'E':5,'alias':[':vomiting_face:'],'de':':kotzendes_gesicht:','es':':cara_vomitando:','fr':':visage_qui_vomit:','ja':':嘔吐する顔:','ko':':토하는_얼굴:','pt':':rosto_vomitando:','it':':faccina_che_vomita:','fa':':درحال_استفراغ:','id':':wajah_muntah:','zh':':呕吐:','ru':':рвота:','tr':':kusan_yüz:','ar':':وجه_يتقيأ:'},'\U0001F635':{'en':':face_with_crossed-out_eyes:','status':fully_qualified,'E':0.6,'alias':[':dizzy_face:', ':face_with_crossed_out_eyes:', ':knocked_out_face:'],'de':':benommenes_gesicht:','es':':cara_mareada:','fr':':visage_étourdi:','ja':':めまい:','ko':':어지러운_얼굴:','pt':':rosto_atordoado:','it':':faccina_frastornata:','fa':':گیج:','id':':wajah_pusing:','zh':':晕头转向:','ru':':головокружение:','tr':':sersemlemiş_yüz:','ar':':وجه_دائخ:'},'\U0001FAE4':{'en':':face_with_diagonal_mouth:','status':fully_qualified,'E':14,'de':':gesicht_mit_schrägem_mund:','es':':cara_con_boca_diagonal:','fr':':visage_avec_bouche_en_diagonale:','ja':':口が斜めの顔:','ko':':입이_한쪽으로_올라간_얼굴:','pt':':rosto_com_boca_diagonal:','it':':faccina_perplessa:','fa':':صورت_با_دهان_کج:','id':':wajah_dengan_mulut_diagonal:','zh':':郁闷:','ru':':с_перекошенным_ртом:','tr':':yamuk_ağızlı_yüz:','ar':':وجه_بفم_مائل:'},'\U0001F92D':{'en':':face_with_hand_over_mouth:','status':fully_qualified,'E':5,'alias':[':hand_over_mouth:'],'de':':verlegen_kicherndes_gesicht:','es':':cara_con_mano_sobre_la_boca:','fr':':visage_avec_une_main_sur_la_bouche:','ja':':口に手を当てた顔:','ko':':손으로_입을_가린_얼굴:','pt':':rosto_com_a_mão_sobre_a_boca:','it':':faccina_con_mano_sulla_bocca:','fa':':صورتک_با_دست_روی_دهان:','id':':wajah_dengan_tangan_menutup_mulut:','zh':':不说:','ru':':прикрывает_рот_рукой:','tr':':eliyle_ağzını_kapatan_yüz:','ar':':وجه_ضاحك_مع_يد_تغطي_الفم:'},'\U0001F915':{'en':':face_with_head-bandage:','status':fully_qualified,'E':1,'alias':[':face_with_head_bandage:'],'de':':gesicht_mit_kopfverband:','es':':cara_con_la_cabeza_vendada:','fr':':visage_avec_bandage_autour_de_la_tête:','ja':':包帯を巻いた顔:','ko':':머리에_붕대를_감은_얼굴:','pt':':rosto_com_atadura_na_cabeça:','it':':faccina_con_la_testa_bendata:','fa':':سر_باندپیچی_شده:','id':':wajah_diperban_di_kepala:','zh':':受伤:','ru':':с_перевязанной_головой:','tr':':kafası_sarılı_yüz:','ar':':وجه_بعصابة_رأس:'},'\U0001F637':{'en':':face_with_medical_mask:','status':fully_qualified,'E':0.6,'alias':[':mask:'],'de':':gesicht_mit_atemschutzmaske:','es':':cara_con_mascarilla_médica:','fr':':visage_avec_masque:','ja':':マスク顔:','ko':':마스크_낀_얼굴:','pt':':rosto_com_máscara_médica:','it':':faccina_con_mascherina:','fa':':صورت_با_ماسک_پزشکی:','id':':wajah_bermasker:','zh':':感冒:','ru':':в_медицинской_маске:','tr':':maskeli_yüz:','ar':':وجه_بكمامة:'},'\U0001F9D0':{'en':':face_with_monocle:','status':fully_qualified,'E':5,'alias':[':monocle_face:'],'de':':gesicht_mit_monokel:','es':':cara_con_monóculo:','fr':':visage_avec_un_monocle:','ja':':モノクルを付けた顔:','ko':':단안경을_쓴_얼굴:','pt':':rosto_com_monóculo:','it':':faccina_con_monocolo:','fa':':صورت_با_عینک_یک_چشمی:','id':':wajah_dengan_monokel:','zh':':带单片眼镜的脸:','ru':':с_моноклем:','tr':':tek_cam_gözlüklü_yüz:','ar':':وجه_مع_نظارة_لعين_واحدة:'},'\U0001FAE2':{'en':':face_with_open_eyes_and_hand_over_mouth:','status':fully_qualified,'E':14,'de':':gesicht_mit_offenen_augen_und_hand_über_dem_mund:','es':':cara_con_ojos_abiertos_y_boca_tapada:','fr':':visage_avec_yeux_ouverts_et_main_sur_la_bouche:','ja':':目を開けて口に手を当てた顔:','ko':':눈을_뜨고_손으로_입을_가린_얼굴:','pt':':rosto_com_olhos_abertos_e_mão_sobre_a_boca:','it':':faccina_con_occhi_aperti_e_mano_sulla_bocca:','fa':':چهره_با_چشمان_باز_و_دست_روی_دهان:','id':':wajah_dengan_mata_terbuka_dan_tangan_menutupi_mulut:','zh':':睁眼捂嘴:','ru':':прикрывает_рот_рукой_с_открытыми_глазами:','tr':':gözleri_açık_ve_eliyle_ağzını_kapatan_yüz:','ar':':وجه_بعينين_مفتوحتين_ويد_تغطي_الفم:'},'\U0001F62E':{'en':':face_with_open_mouth:','status':fully_qualified,'E':1,'alias':[':open_mouth:'],'de':':gesicht_mit_offenem_mund:','es':':cara_con_la_boca_abierta:','fr':':visage_avec_bouche_ouverte:','ja':':口が開いた顔:','ko':':입벌린_얼굴:','pt':':rosto_com_boca_aberta:','it':':faccina_con_bocca_aperta:','fa':':دهان_باز_از_تعجب:','id':':wajah_dengan_mulut_terbuka:','zh':':吃惊:','ru':':удивление:','tr':':ağzı_açık_yüz:','ar':':وجه_بفم_مفتوح:'},'\U0001FAE3':{'en':':face_with_peeking_eye:','status':fully_qualified,'E':14,'de':':gesicht_mit_durch_die_finger_linsendem_auge:','es':':cara_tapada_con_ojo_espiando:','fr':':visage_qui_regarde_entre_ses_doigts:','ja':':指の間からのぞき見る顔:','ko':':한쪽_눈을_가리고_훔쳐보는_얼굴:','pt':':rosto_com_olho_espiando:','it':':faccina_che_sbircia_tra_le_dita:','fa':':چهره_با_نگاه_دزدکی:','id':':wajah_dengan_mata_mengintip:','zh':':偷看:','ru':':подсматривает:','tr':':gizlice_bakan_yüz:','ar':':وجه_يختلس_النظر:'},'\U0001F928':{'en':':face_with_raised_eyebrow:','status':fully_qualified,'E':5,'alias':[':raised_eyebrow:'],'de':':gesicht_mit_hochgezogenen_augenbrauen:','es':':cara_con_ceja_alzada:','fr':':visage_avec_les_sourcils_relevés:','ja':':眉を上げた顔:','ko':':눈썹을_치켜올린_얼굴:','pt':':rosto_com_sobrancelha_levantada:','it':':faccia_con_sopracciglia_alzate:','fa':':صورت_با_ابروی_بالاداده:','id':':wajah_dengan_alis_terangkat:','zh':':挑眉:','ru':':не_верит:','tr':':tek_kaşı_çatık_yüz:','ar':':وجه_بحاجب_مرفوع:'},'\U0001F644':{'en':':face_with_rolling_eyes:','status':fully_qualified,'E':1,'alias':[':roll_eyes:'],'de':':augen_verdrehendes_gesicht:','es':':cara_con_ojos_en_blanco:','fr':':visage_roulant_des_yeux:','ja':':上を見る顔:','ko':':눈을_굴리고_있는_얼굴:','pt':':rosto_com_olhos_revirados:','it':':faccina_con_occhi_al_cielo:','fa':':نگاه_کلافه:','id':':wajah_dengan_mata_ke_atas:','zh':':翻白眼:','ru':':закатывает_глаза:','tr':':gözlerini_deviren_yüz:','ar':':وجه_بعينين_تدوران:'},'\U0001F635\U0000200D\U0001F4AB':{'en':':face_with_spiral_eyes:','status':fully_qualified,'E':13.1,'de':':gesicht_mit_spiralen_als_augen:','es':':cara_con_ojos_en_espiral:','fr':':visage_aux_yeux_en_spirales:','ja':':目を回した顔:','ko':':현기증_난_얼굴:','pt':':rosto_com_olhos_em_espiral:','it':':faccina_con_occhi_a_spirale:','fa':':صورتک_با_چشمان_چرخشی:','id':':wajah_dengan_mata_berputar:','zh':':晕:','ru':':ошеломление:','tr':':sarmal_gözlü_yüz:','ar':':وجه_بعينين_دائختين:'},'\U0001F624':{'en':':face_with_steam_from_nose:','status':fully_qualified,'E':0.6,'alias':[':triumph:'],'de':':schnaubendes_gesicht:','es':':cara_resoplando:','fr':':visage_avec_fumée_sortant_des_narines:','ja':':勝ち誇った顔:','ko':':화내며_씩씩거리는_얼굴:','pt':':rosto_soltando_vapor_pelo_nariz:','it':':faccina_che_sbuffa:','fa':':ژست_برندگی_با_دلخوری:','id':':wajah_bersungguh-sungguh:','zh':':傲慢:','ru':':в_ожидании_успеха:','tr':':burnundan_soluyan_yüz:','ar':':وجه_بأنف_يصدر_بخارا:'},'\U0001F92C':{'en':':face_with_symbols_on_mouth:','status':fully_qualified,'E':5,'alias':[':cursing_face:'],'de':':gesicht_mit_symbolen_über_dem_mund:','es':':cara_con_símbolos_en_la_boca:','fr':':visage_avec_des_symboles_dans_la_bouche:','ja':':ののしる:','ko':':욕하는_얼굴:','pt':':rosto_com_símbolos_na_boca:','it':':faccina_con_simboli_sulla_bocca:','fa':':صورت_با_نمادهایی_روی_دهان:','id':':wajah_dengan_simbol_pada_mulut:','zh':':嘴上有符号的脸:','ru':':цензура:','tr':':ağzında_semboller_olan_yüz:','ar':':وجه_مع_رموز_على_الفم:'},'\U0001F602':{'en':':face_with_tears_of_joy:','status':fully_qualified,'E':0.6,'alias':[':joy:'],'de':':gesicht_mit_freudentränen:','es':':cara_llorando_de_risa:','fr':':visage_riant_aux_larmes:','ja':':嬉し泣き:','ko':':기쁨의_눈물을_흘리는_얼굴:','pt':':rosto_chorando_de_rir:','it':':faccina_con_lacrime_di_gioia:','fa':':روده_بر:','id':':wajah_gembira_berurai_air_mata:','zh':':笑哭了:','ru':':смеется_до_слез:','tr':':sevinç_gözyaşları:','ar':':وجه_بدموع_فرح:'},'\U0001F912':{'en':':face_with_thermometer:','status':fully_qualified,'E':1,'de':':gesicht_mit_fieberthermometer:','es':':cara_con_termómetro:','fr':':visage_avec_thermomètre:','ja':':熱がある顔:','ko':':체온계를_물고_있는_얼굴:','pt':':rosto_com_termômetro:','it':':faccina_con_termometro:','fa':':تب_دار:','id':':wajah_bertermometer:','zh':':发烧:','ru':':с_градусником_во_рту:','tr':':ağzında_termometre_olan_yüz:','ar':':وجه_مع_ميزان_للحرارة:'},'\U0001F61B':{'en':':face_with_tongue:','status':fully_qualified,'E':1,'alias':[':stuck_out_tongue:'],'de':':gesicht_mit_herausgestreckter_zunge:','es':':cara_sacando_la_lengua:','fr':':visage_qui_tire_la_langue:','ja':':舌を出した顔:','ko':':혀를_내민_얼굴:','pt':':rosto_mostrando_a_língua:','it':':faccina_che_mostra_la_lingua:','fa':':صورت_با_زبان_بیرون_آمده:','id':':wajah_menjulurkan_lidah:','zh':':吐舌:','ru':':показывает_язык:','tr':':dil_dışarıda:','ar':':وجه_بلسان_متدل:'},'\U0001F636':{'en':':face_without_mouth:','status':fully_qualified,'E':1,'alias':[':no_mouth:'],'de':':gesicht_ohne_mund:','es':':cara_sin_boca:','fr':':visage_sans_bouche:','ja':':口のない顔:','ko':':입이_없는_얼굴:','pt':':rosto_sem_boca:','it':':faccina_senza_bocca:','fa':':صورت_بدون_لب:','id':':wajah_tanpa_mulut:','zh':':沉默:','ru':':лицо_без_рта:','tr':':ağzı_olmayan_yüz:','ar':':وجه_بدون_فم:'},'\U0001F3ED':{'en':':factory:','status':fully_qualified,'E':0.6,'variant':True,'de':':fabrik:','es':':fábrica:','fr':':usine:','ja':':工場:','ko':':공장:','pt':':fábrica:','it':':fabbrica:','fa':':کارخانه:','id':':pabrik:','zh':':工厂:','ru':':завод:','tr':':fabrika:','ar':':مصنع:'},'\U0001F9D1\U0000200D\U0001F3ED':{'en':':factory_worker:','status':fully_qualified,'E':12.1,'de':':fabrikarbeiter(in):','es':':profesional_industrial:','fr':':ouvrier_(tous_genres):','ja':':溶接工:','ko':':공장_직원:','pt':':funcionário_de_fábrica:','it':':persona_che_lavora_in_fabbrica:','fa':':کارگر_کارخانه:','id':':pekerja_pabrik:','zh':':工人:','ru':':рабочий:','tr':':fabrika_işçisi:','ar':':عامل_في_مصنع:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F3ED':{'en':':factory_worker_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':fabrikarbeiter(in)_dunkle_hautfarbe:','es':':profesional_industrial_tono_de_piel_oscuro:','fr':':ouvrier_(tous_genres)_peau_foncée:','ja':':溶接工_濃い肌色:','ko':':공장_직원_검은색_피부:','pt':':funcionário_de_fábrica_pele_escura:','it':':persona_che_lavora_in_fabbrica_carnagione_scura:','fa':':کارگر_کارخانه_پوست_آبنوسی:','id':':pekerja_pabrik_warna_kulit_gelap:','zh':':工人_较深肤色:','ru':':рабочий_очень_темный_тон_кожи:','tr':':fabrika_işçisi_koyu_cilt_tonu:','ar':':عامل_في_مصنع_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F3ED':{'en':':factory_worker_light_skin_tone:','status':fully_qualified,'E':12.1,'de':':fabrikarbeiter(in)_helle_hautfarbe:','es':':profesional_industrial_tono_de_piel_claro:','fr':':ouvrier_(tous_genres)_peau_claire:','ja':':溶接工_薄い肌色:','ko':':공장_직원_하얀_피부:','pt':':funcionário_de_fábrica_pele_clara:','it':':persona_che_lavora_in_fabbrica_carnagione_chiara:','fa':':کارگر_کارخانه_پوست_سفید:','id':':pekerja_pabrik_warna_kulit_cerah:','zh':':工人_较浅肤色:','ru':':рабочий_очень_светлый_тон_кожи:','tr':':fabrika_işçisi_açık_cilt_tonu:','ar':':عامل_في_مصنع_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F3ED':{'en':':factory_worker_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':fabrikarbeiter(in)_mitteldunkle_hautfarbe:','es':':profesional_industrial_tono_de_piel_oscuro_medio:','fr':':ouvrier_(tous_genres)_peau_mate:','ja':':溶接工_やや濃い肌色:','ko':':공장_직원_진한_갈색_피부:','pt':':funcionário_de_fábrica_pele_morena_escura:','it':':persona_che_lavora_in_fabbrica_carnagione_abbastanza_scura:','fa':':کارگر_کارخانه_پوست_گندمی:','id':':pekerja_pabrik_warna_kulit_gelap-sedang:','zh':':工人_中等深肤色:','ru':':рабочий_темный_тон_кожи:','tr':':fabrika_işçisi_orta_koyu_cilt_tonu:','ar':':عامل_في_مصنع_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F3ED':{'en':':factory_worker_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':fabrikarbeiter(in)_mittelhelle_hautfarbe:','es':':profesional_industrial_tono_de_piel_claro_medio:','fr':':ouvrier_(tous_genres)_peau_moyennement_claire:','ja':':溶接工_やや薄い肌色:','ko':':공장_직원_연한_갈색_피부:','pt':':funcionário_de_fábrica_pele_morena_clara:','it':':persona_che_lavora_in_fabbrica_carnagione_abbastanza_chiara:','fa':':کارگر_کارخانه_پوست_روشن:','id':':pekerja_pabrik_warna_kulit_cerah-sedang:','zh':':工人_中等浅肤色:','ru':':рабочий_светлый_тон_кожи:','tr':':fabrika_işçisi_orta_açık_cilt_tonu:','ar':':عامل_في_مصنع_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F3ED':{'en':':factory_worker_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':fabrikarbeiter(in)_mittlere_hautfarbe:','es':':profesional_industrial_tono_de_piel_medio:','fr':':ouvrier_(tous_genres)_peau_légèrement_mate:','ja':':溶接工_中間の肌色:','ko':':공장_직원_갈색_피부:','pt':':funcionário_de_fábrica_pele_morena:','it':':persona_che_lavora_in_fabbrica_carnagione_olivastra:','fa':':کارگر_کارخانه_پوست_طلایی:','id':':pekerja_pabrik_warna_kulit_sedang:','zh':':工人_中等肤色:','ru':':рабочий_средний_тон_кожи:','tr':':fabrika_işçisi_orta_cilt_tonu:','ar':':عامل_في_مصنع_بشرة_بلون_معتدل:'},'\U0001F9DA':{'en':':fairy:','status':fully_qualified,'E':5,'de':':märchenfee:','es':':hada:','fr':':personnage_féérique:','ja':':妖精:','ko':':요정:','pt':':fada:','it':':fata:','fa':':پری:','id':':peri:','zh':':精灵:','ru':':фея:','tr':':peri:','ar':':جنية_خيالية:'},'\U0001F9DA\U0001F3FF':{'en':':fairy_dark_skin_tone:','status':fully_qualified,'E':5,'de':':märchenfee_dunkle_hautfarbe:','es':':hada_tono_de_piel_oscuro:','fr':':personnage_féérique_peau_foncée:','ja':':妖精_濃い肌色:','ko':':요정_검은색_피부:','pt':':fada_pele_escura:','it':':fata_carnagione_scura:','fa':':پری_پوست_آبنوسی:','id':':peri_warna_kulit_gelap:','zh':':精灵_较深肤色:','ru':':фея_очень_темный_тон_кожи:','tr':':peri_koyu_cilt_tonu:','ar':':جنية_خيالية_بشرة_بلون_غامق:'},'\U0001F9DA\U0001F3FB':{'en':':fairy_light_skin_tone:','status':fully_qualified,'E':5,'de':':märchenfee_helle_hautfarbe:','es':':hada_tono_de_piel_claro:','fr':':personnage_féérique_peau_claire:','ja':':妖精_薄い肌色:','ko':':요정_하얀_피부:','pt':':fada_pele_clara:','it':':fata_carnagione_chiara:','fa':':پری_پوست_سفید:','id':':peri_warna_kulit_cerah:','zh':':精灵_较浅肤色:','ru':':фея_очень_светлый_тон_кожи:','tr':':peri_açık_cilt_tonu:','ar':':جنية_خيالية_بشرة_بلون_فاتح:'},'\U0001F9DA\U0001F3FE':{'en':':fairy_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':märchenfee_mitteldunkle_hautfarbe:','es':':hada_tono_de_piel_oscuro_medio:','fr':':personnage_féérique_peau_mate:','ja':':妖精_やや濃い肌色:','ko':':요정_진한_갈색_피부:','pt':':fada_pele_morena_escura:','it':':fata_carnagione_abbastanza_scura:','fa':':پری_پوست_گندمی:','id':':peri_warna_kulit_gelap-sedang:','zh':':精灵_中等深肤色:','ru':':фея_темный_тон_кожи:','tr':':peri_orta_koyu_cilt_tonu:','ar':':جنية_خيالية_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9DA\U0001F3FC':{'en':':fairy_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':märchenfee_mittelhelle_hautfarbe:','es':':hada_tono_de_piel_claro_medio:','fr':':personnage_féérique_peau_moyennement_claire:','ja':':妖精_やや薄い肌色:','ko':':요정_연한_갈색_피부:','pt':':fada_pele_morena_clara:','it':':fata_carnagione_abbastanza_chiara:','fa':':پری_پوست_روشن:','id':':peri_warna_kulit_cerah-sedang:','zh':':精灵_中等浅肤色:','ru':':фея_светлый_тон_кожи:','tr':':peri_orta_açık_cilt_tonu:','ar':':جنية_خيالية_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9DA\U0001F3FD':{'en':':fairy_medium_skin_tone:','status':fully_qualified,'E':5,'de':':märchenfee_mittlere_hautfarbe:','es':':hada_tono_de_piel_medio:','fr':':personnage_féérique_peau_légèrement_mate:','ja':':妖精_中間の肌色:','ko':':요정_갈색_피부:','pt':':fada_pele_morena:','it':':fata_carnagione_olivastra:','fa':':پری_پوست_طلایی:','id':':peri_warna_kulit_sedang:','zh':':精灵_中等肤色:','ru':':фея_средний_тон_кожи:','tr':':peri_orta_cilt_tonu:','ar':':جنية_خيالية_بشرة_بلون_معتدل:'},'\U0001F9C6':{'en':':falafel:','status':fully_qualified,'E':12,'de':':falafel:','es':':falafel:','fr':':falafels:','ja':':ファラフェル:','ko':':팔라펠:','pt':':falafel:','it':':falafel:','fa':':فلافل:','id':':falafel:','zh':':炸豆丸子:','ru':':фалафель:','tr':':falafel:','ar':':فلافل:'},'\U0001F342':{'en':':fallen_leaf:','status':fully_qualified,'E':0.6,'de':':laub:','es':':hojas_caídas:','fr':':feuille_morte:','ja':':落ち葉:','ko':':낙엽:','pt':':folhas_caídas:','it':':foglia_caduta:','fa':':برگ_ریزان:','id':':daun_berguguran:','zh':':落叶:','ru':':падающие_листья:','tr':':düşen_yapraklar:','ar':':أوراق_متساقطة:'},'\U0001F46A':{'en':':family:','status':fully_qualified,'E':0.6,'variant':True,'de':':familie:','es':':familia:','fr':':famille:','ja':':家族:','ko':':가족:','pt':':família:','it':':famiglia:','fa':':خانواده:','id':':keluarga:','zh':':家庭:','ru':':семья:','tr':':aile:','ar':':أسرة:'},'\U0001F9D1\U0000200D\U0001F9D1\U0000200D\U0001F9D2':{'en':':family_adult_adult_child:','status':fully_qualified,'E':15.1,'de':':familie_2_erwachsene_1_kind:','es':':familia_adulto_adulto_niño:','fr':':famille_adulte_adulte_enfant:','ja':':家族_大人二人と子供一人:','ko':':가족_성인_성인_아이:','pt':':família_adulto_adulto_criança:','it':':famiglia_persona_adulta_persona_adulta_minore:','fa':':خانواده_بزرگسال_بالغ_کودک:','id':':keluarga_orang_dewasa_orang_dewasa_anak:','zh':':一孩家庭:','ru':':семья_двое_взрослых_и_ребенок:','tr':':aile_yetişkin_yetişkin_çocuk:','ar':':أسرة_بالغ_بالغة_طفل:'},'\U0001F9D1\U0000200D\U0001F9D1\U0000200D\U0001F9D2\U0000200D\U0001F9D2':{'en':':family_adult_adult_child_child:','status':fully_qualified,'E':15.1,'de':':familie_2_erwachsene_2_kinder:','es':':familia_adulto_adulto_niño_niño:','fr':':famille_adulte_adulte_enfant_enfant:','ja':':家族_大人二人と子供二人:','ko':':가족_성인_성인_아이_아이:','pt':':família_adulto_adulto_criança_criança:','it':':famiglia_persona_adulta_persona_adulta_minore_minore:','fa':':خانواده_بزرگسال_بالغ_کودک_فرزند:','id':':keluarga_orang_dewasa_orang_dewasa_anak_anak:','zh':':二孩家庭:','ru':':семья_двое_взрослых_и_двое_детей:','tr':':aile_yetişkin_yetişkin_çocuk_çocuk:','ar':':أسرة_بالغ_بالغة_طفل_طفلة:'},'\U0001F9D1\U0000200D\U0001F9D2':{'en':':family_adult_child:','status':fully_qualified,'E':15.1,'de':':familie_1_erwachsene(r)_1_kind:','es':':familia_adulto_niño:','fr':':famille_adulte_enfant:','ja':':家族_大人一人と子供一人:','ko':':가족_성인_아이:','pt':':família_adulto_criança:','it':':famiglia_persona_adulta_minore:','fa':':خانواده_بزرگسال_کودک:','id':':keluarga_orang_dewasa_anak:','zh':':单亲一孩家庭:','ru':':семья_взрослый_и_ребенок:','tr':':aile_yetişkin_çocuk:','ar':':أسرة_بالغ_طفل:'},'\U0001F9D1\U0000200D\U0001F9D2\U0000200D\U0001F9D2':{'en':':family_adult_child_child:','status':fully_qualified,'E':15.1,'de':':familie_1_erwachsene(r)_2_kinder:','es':':familia_adulto_niño_niño:','fr':':famille_adulte_enfant_enfant:','ja':':家族_大人一人と子供二人:','ko':':가족_성인_아이_아이:','pt':':família_adulto_criança_criança:','it':':famiglia_persona_adulta_minore_minore:','fa':':خانواده_بزرگسال_کودک_فرزند:','id':':keluarga_orang_dewasa_anak_anak:','zh':':单亲二孩家庭:','ru':':семья_взрослый_и_двое_детей:','tr':':aile_yetişkin_çocuk_çocuk:','ar':':أسرة_بالغ_طفل_طفلة:'},'\U0001F468\U0000200D\U0001F466':{'en':':family_man_boy:','status':fully_qualified,'E':4,'de':':familie_mann_junge:','es':':familia_hombre_y_niño:','fr':':famille_homme_et_garçon:','ja':':家族_男性_男の子:','ko':':가족_남자_남자_아이:','pt':':família_homem_e_menino:','it':':famiglia_uomo_e_bambino:','fa':':خانواده_مرد_پسر:','id':':keluarga_pria_anak_laki-laki:','zh':':家庭_男人男孩:','ru':':семья_мужчина_мальчик:','tr':':aile_erkek_erkek_çocuk:','ar':':أسرة_رجل_وصبي:'},'\U0001F468\U0000200D\U0001F466\U0000200D\U0001F466':{'en':':family_man_boy_boy:','status':fully_qualified,'E':4,'de':':familie_mann_junge_und_junge:','es':':familia_hombre_niño_niño:','fr':':famille_homme_garçon_et_garçon:','ja':':家族_男性_男の子_男の子:','ko':':가족_남자_남자_아이_남자_아이:','pt':':família_homem_menino_e_menino:','it':':famiglia_uomo_bambino_e_bambino:','fa':':خانواده_مرد_پسر_و_پسر:','id':':keluarga_pria_anak_laki-laki_anak_laki-laki:','zh':':家庭_男人男孩男孩:','ru':':семья_мужчина_мальчик_мальчик:','tr':':aile_erkek_erkek_çocuk_erkek_çocuk:','ar':':أسرة_رجل_وصبي_وصبي:'},'\U0001F468\U0000200D\U0001F467':{'en':':family_man_girl:','status':fully_qualified,'E':4,'de':':familie_mann_mädchen:','es':':familia_hombre_y_niña:','fr':':famille_homme_et_fille:','ja':':家族_男性_女の子:','ko':':가족_남자_여자_아이:','pt':':família_homem_e_menina:','it':':famiglia_uomo_e_bambina:','fa':':خانواده_مرد_دختر:','id':':keluarga_pria_anak_perempuan:','zh':':家庭_男人女孩:','ru':':семья_мужчина_девочка:','tr':':aile_erkek_kız_çocuk:','ar':':أسرة_رجل_وفتاة:'},'\U0001F468\U0000200D\U0001F467\U0000200D\U0001F466':{'en':':family_man_girl_boy:','status':fully_qualified,'E':4,'de':':familie_mann_mädchen_und_junge:','es':':familia_hombre_niña_niño:','fr':':famille_homme_fille_et_garçon:','ja':':家族_男性_女の子_男の子:','ko':':가족_남자_여자_아이_남자_아이:','pt':':família_homem_menina_e_menino:','it':':famiglia_uomo_bambina_e_bambino:','fa':':خانواده_مرد_دختر_و_پسر:','id':':keluarga_pria_anak_perempuan_anak_laki-laki:','zh':':家庭_男人女孩男孩:','ru':':семья_мужчина_девочка_мальчик:','tr':':aile_erkek_kız_çocuk_erkek_çocuk:','ar':':أسرة_رجل_وفتاة_وصبي:'},'\U0001F468\U0000200D\U0001F467\U0000200D\U0001F467':{'en':':family_man_girl_girl:','status':fully_qualified,'E':4,'de':':familie_mann_mädchen_und_mädchen:','es':':familia_hombre_niña_niña:','fr':':famille_homme_fille_et_fille:','ja':':家族_男性_女の子_女の子:','ko':':가족_남자_여자_아이_여자_아이:','pt':':família_homem_menina_e_menina:','it':':famiglia_uomo_bambina_e_bambina:','fa':':خانواده_مرد_دختر_و_دختر:','id':':keluarga_pria_anak_perempuan_anak_perempuan:','zh':':家庭_男人女孩女孩:','ru':':семья_мужчина_девочка_девочка:','tr':':aile_erkek_kız_çocuk_kız_çocuk:','ar':':أسرة_رجل_وفتاة_وفتاة:'},'\U0001F468\U0000200D\U0001F468\U0000200D\U0001F466':{'en':':family_man_man_boy:','status':fully_qualified,'E':2,'de':':familie_mann_mann_und_junge:','es':':familia_hombre_hombre_niño:','fr':':famille_homme_homme_et_garçon:','ja':':家族_男性_男性_男の子:','ko':':가족_남자_남자_남자_아이:','pt':':família_homem_homem_e_menino:','it':':famiglia_uomo_uomo_e_bambino:','fa':':خانواده_مرد_مرد_و_پسر:','id':':keluarga_pria_pria_anak_laki-laki:','zh':':家庭_男人男人男孩:','ru':':семья_мужчина_мужчина_мальчик:','tr':':aile_erkek_erkek_erkek_çocuk:','ar':':أسرة_رجل_ورجل_وصبي:'},'\U0001F468\U0000200D\U0001F468\U0000200D\U0001F466\U0000200D\U0001F466':{'en':':family_man_man_boy_boy:','status':fully_qualified,'E':2,'de':':familie_mann_mann_junge_und_junge:','es':':familia_hombre_hombre_niño_niño:','fr':':famille_homme_homme_garçon_et_garçon:','ja':':家族_男性_男性_男の子_男の子:','ko':':가족_남자_남자_남자_아이_남자_아이:','pt':':família_homem_homem_menino_e_menino:','it':':famiglia_uomo_uomo_bambino_e_bambino:','fa':':خانواده_مرد_مرد_پسر_و_پسر:','id':':keluarga_pria_pria_anak_laki-laki_anak_laki-laki:','zh':':家庭_男人男人男孩男孩:','ru':':семья_мужчина_мужчина_мальчик_мальчик:','tr':':aile_erkek_erkek_erkek_çocuk_erkek_çocuk:','ar':':أسرة_رجل_ورجل_وصبي_وصبي:'},'\U0001F468\U0000200D\U0001F468\U0000200D\U0001F467':{'en':':family_man_man_girl:','status':fully_qualified,'E':2,'de':':familie_mann_mann_und_mädchen:','es':':familia_hombre_hombre_niña:','fr':':famille_homme_homme_et_fille:','ja':':家族_男性_男性_女の子:','ko':':가족_남자_남자_여자_아이:','pt':':família_homem_homem_e_menina:','it':':famiglia_uomo_uomo_e_bambina:','fa':':خانواده_مرد_مرد_و_دختر:','id':':keluarga_pria_pria_anak_perempuan:','zh':':家庭_男人男人女孩:','ru':':семья_мужчина_мужчина_девочка:','tr':':aile_erkek_erkek_kız_çocuk:','ar':':أسرة_رجل_ورجل_وفتاة:'},'\U0001F468\U0000200D\U0001F468\U0000200D\U0001F467\U0000200D\U0001F466':{'en':':family_man_man_girl_boy:','status':fully_qualified,'E':2,'de':':familie_mann_mann_mädchen_und_junge:','es':':familia_hombre_hombre_niña_niño:','fr':':famille_homme_homme_fille_et_garçon:','ja':':家族_男性_男性_女の子_男の子:','ko':':가족_남자_남자_여자_아이_남자_아이:','pt':':família_homem_homem_menina_e_menino:','it':':famiglia_uomo_uomo_bambina_e_bambino:','fa':':خانواده_مرد_مرد_دختر_و_پسر:','id':':keluarga_pria_pria_anak_perempuan_anak_laki-laki:','zh':':家庭_男人男人女孩男孩:','ru':':семья_мужчина_мужчина_девочка_мальчик:','tr':':aile_erkek_erkek_kız_çocuk_erkek_çocuk:','ar':':أسرة_رجل_ورجل_وفتاة_وصبي:'},'\U0001F468\U0000200D\U0001F468\U0000200D\U0001F467\U0000200D\U0001F467':{'en':':family_man_man_girl_girl:','status':fully_qualified,'E':2,'de':':familie_mann_mann_mädchen_und_mädchen:','es':':familia_hombre_hombre_niña_niña:','fr':':famille_homme_homme_fille_et_fille:','ja':':家族_男性_男性_女の子_女の子:','ko':':가족_남자_남자_여자_아이_여자_아이:','pt':':família_homem_homem_menina_e_menina:','it':':famiglia_uomo_uomo_bambina_e_bambina:','fa':':خانواده_مرد_مرد_دختر_و_دختر:','id':':keluarga_pria_pria_anak_perempuan_anak_perempuan:','zh':':家庭_男人男人女孩女孩:','ru':':семья_мужчина_мужчина_девочка_девочка:','tr':':aile_erkek_erkek_kız_çocuk_kız_çocuk:','ar':':أسرة_رجل_ورجل_وفتاة_وفتاة:'},'\U0001F468\U0000200D\U0001F469\U0000200D\U0001F466':{'en':':family_man_woman_boy:','status':fully_qualified,'E':2,'de':':familie_mann_frau_und_junge:','es':':familia_hombre_mujer_niño:','fr':':famille_homme_femme_et_garçon:','ja':':家族_男性_女性_男の子:','ko':':가족_남자_여자_남자_아이:','pt':':família_homem_mulher_e_menino:','it':':famiglia_uomo_donna_e_bambino:','fa':':خانواده_مرد_زن_و_پسر:','id':':keluarga_pria_wanita_anak_laki-laki:','zh':':家庭_男人女人男孩:','ru':':семья_мужчина_женщина_мальчик:','tr':':aile_erkek_kadın_erkek_çocuk:','ar':':أسرة_رجل_وسيدة_وصبي:'},'\U0001F468\U0000200D\U0001F469\U0000200D\U0001F466\U0000200D\U0001F466':{'en':':family_man_woman_boy_boy:','status':fully_qualified,'E':2,'de':':familie_mann_frau_junge_und_junge:','es':':familia_hombre_mujer_niño_niño:','fr':':famille_homme_femme_garçon_et_garçon:','ja':':家族_男性_女性_男の子_男の子:','ko':':가족_남자_여자_남자_아이_남자_아이:','pt':':família_homem_mulher_menino_e_menino:','it':':famiglia_uomo_donna_bambino_e_bambino:','fa':':خانواده_مرد_زن_پسر_و_پسر:','id':':keluarga_pria_wanita_anak_laki-laki_anak_laki-laki:','zh':':家庭_男人女人男孩男孩:','ru':':семья_мужчина_женщина_мальчик_мальчик:','tr':':aile_erkek_kadın_erkek_çocuk_erkek_çocuk:','ar':':أسرة_رجل_وسيدة_وصبي_وصبي:'},'\U0001F468\U0000200D\U0001F469\U0000200D\U0001F467':{'en':':family_man_woman_girl:','status':fully_qualified,'E':2,'de':':familie_mann_frau_und_mädchen:','es':':familia_hombre_mujer_niña:','fr':':famille_homme_femme_et_fille:','ja':':家族_男性_女性_女の子:','ko':':가족_남자_여자_여자_아이:','pt':':família_homem_mulher_e_menina:','it':':famiglia_uomo_donna_e_bambina:','fa':':خانواده_مرد_زن_و_دختر:','id':':keluarga_pria_wanita_anak_perempuan:','zh':':家庭_男人女人女孩:','ru':':семья_мужчина_женщина_девочка:','tr':':aile_erkek_kadın_kız_çocuk:','ar':':أسرة_رجل_وسيدة_وفتاة:'},'\U0001F468\U0000200D\U0001F469\U0000200D\U0001F467\U0000200D\U0001F466':{'en':':family_man_woman_girl_boy:','status':fully_qualified,'E':2,'de':':familie_mann_frau_mädchen_und_junge:','es':':familia_hombre_mujer_niña_niño:','fr':':famille_homme_femme_fille_et_garçon:','ja':':家族_男性_女性_女の子_男の子:','ko':':가족_남자_여자_여자_아이_남자_아이:','pt':':família_homem_mulher_menina_e_menino:','it':':famiglia_uomo_donna_bambina_e_bambino:','fa':':خانواده_مرد_زن_دختر_و_پسر:','id':':keluarga_pria_wanita_anak_perempuan_anak_laki-laki:','zh':':家庭_男人女人女孩男孩:','ru':':семья_мужчина_женщина_девочка_мальчик:','tr':':aile_erkek_kadın_kız_çocuk_erkek_çocuk:','ar':':أسرة_رجل_وسيدة_وفتاة_وصبي:'},'\U0001F468\U0000200D\U0001F469\U0000200D\U0001F467\U0000200D\U0001F467':{'en':':family_man_woman_girl_girl:','status':fully_qualified,'E':2,'de':':familie_mann_frau_mädchen_und_mädchen:','es':':familia_hombre_mujer_niña_niña:','fr':':famille_homme_femme_fille_et_fille:','ja':':家族_男性_女性_女の子_女の子:','ko':':가족_남자_여자_여자_아이_여자_아이:','pt':':família_homem_mulher_menina_e_menina:','it':':famiglia_uomo_donna_bambina_e_bambina:','fa':':خانواده_مرد_زن_دختر_و_دختر:','id':':keluarga_pria_wanita_anak_perempuan_anak_perempuan:','zh':':家庭_男人女人女孩女孩:','ru':':семья_мужчина_женщина_девочка_девочка:','tr':':aile_erkek_kadın_kız_çocuk_kız_çocuk:','ar':':أسرة_رجل_وسيدة_وفتاة_وفتاة:'},'\U0001F469\U0000200D\U0001F466':{'en':':family_woman_boy:','status':fully_qualified,'E':4,'de':':familie_frau_junge:','es':':familia_mujer_y_niño:','fr':':famille_femme_et_garçon:','ja':':家族_女性_男の子:','ko':':가족_여자_남자_아이:','pt':':família_mulher_e_menino:','it':':famiglia_donna_e_bambino:','fa':':خانواده_زن_پسر:','id':':keluarga_wanita_anak_laki-laki:','zh':':家庭_女人男孩:','ru':':семья_женщина_мальчик:','tr':':aile_kadın_erkek_çocuk:','ar':':أسرة_سيدة_وصبي:'},'\U0001F469\U0000200D\U0001F466\U0000200D\U0001F466':{'en':':family_woman_boy_boy:','status':fully_qualified,'E':4,'de':':familie_frau_junge_und_junge:','es':':familia_mujer_niño_niño:','fr':':famille_femme_garçon_et_garçon:','ja':':家族_女性_男の子_男の子:','ko':':가족_여자_남자_아이_남자_아이:','pt':':família_mulher_menino_e_menino:','it':':famiglia_donna_bambino_e_bambino:','fa':':خانواده_زن_پسر_و_پسر:','id':':keluarga_wanita_anak_laki-laki_anak_laki-laki:','zh':':家庭_女人男孩男孩:','ru':':семья_женщина_мальчик_мальчик:','tr':':aile_kadın_erkek_çocuk_erkek_çocuk:','ar':':أسرة_سيدة_وصبي_وصبي:'},'\U0001F469\U0000200D\U0001F467':{'en':':family_woman_girl:','status':fully_qualified,'E':4,'de':':familie_frau_mädchen:','es':':familia_mujer_y_niña:','fr':':famille_femme_et_fille:','ja':':家族_女性_女の子:','ko':':가족_여자_여자_아이:','pt':':família_mulher_e_menina:','it':':famiglia_donna_e_bambina:','fa':':خانواده_زن_دختر:','id':':keluarga_wanita_anak_perempuan:','zh':':家庭_女人女孩:','ru':':семья_женщина_девочка:','tr':':aile_kadın_kız_çocuk:','ar':':أسرة_سيدة_وفتاة:'},'\U0001F469\U0000200D\U0001F467\U0000200D\U0001F466':{'en':':family_woman_girl_boy:','status':fully_qualified,'E':4,'de':':familie_frau_mädchen_und_junge:','es':':familia_mujer_niña_niño:','fr':':famille_femme_fille_et_garçon:','ja':':家族_女性_女の子_男の子:','ko':':가족_여자_여자_아이_남자_아이:','pt':':família_mulher_menina_e_menino:','it':':famiglia_donna_bambina_e_bambino:','fa':':خانواده_زن_دختر_و_پسر:','id':':keluarga_wanita_anak_perempuan_anak_laki-laki:','zh':':家庭_女人女孩男孩:','ru':':семья_женщина_девочка_мальчик:','tr':':aile_kadın_kız_çocuk_erkek_çocuk:','ar':':أسرة_سيدة_وفتاة_وصبي:'},'\U0001F469\U0000200D\U0001F467\U0000200D\U0001F467':{'en':':family_woman_girl_girl:','status':fully_qualified,'E':4,'de':':familie_frau_mädchen_und_mädchen:','es':':familia_mujer_niña_niña:','fr':':famille_femme_fille_et_fille:','ja':':家族_女性_女の子_女の子:','ko':':가족_여자_여자_아이_여자_아이:','pt':':família_mulher_menina_e_menina:','it':':famiglia_donna_bambina_e_bambina:','fa':':خانواده_زن_دختر_و_دختر:','id':':keluarga_wanita_anak_perempuan_anak_perempuan:','zh':':家庭_女人女孩女孩:','ru':':семья_женщина_девочка_девочка:','tr':':aile_kadın_kız_çocuk_kız_çocuk:','ar':':أسرة_سيدة_وفتاة_وفتاة:'},'\U0001F469\U0000200D\U0001F469\U0000200D\U0001F466':{'en':':family_woman_woman_boy:','status':fully_qualified,'E':2,'de':':familie_frau_frau_und_junge:','es':':familia_mujer_mujer_niño:','fr':':famille_femme_femme_et_garçon:','ja':':家族_女性_女性_男の子:','ko':':가족_여자_여자_남자_아이:','pt':':família_mulher_mulher_e_menino:','it':':famiglia_donna_donna_e_bambino:','fa':':خانواده_زن_زن_و_پسر:','id':':keluarga_wanita_wanita_anak_laki-laki:','zh':':家庭_女人女人男孩:','ru':':семья_женщина_женщина_мальчик:','tr':':aile_kadın_kadın_erkek_çocuk:','ar':':أسرة_سيدة_وسيدة_وصبي:'},'\U0001F469\U0000200D\U0001F469\U0000200D\U0001F466\U0000200D\U0001F466':{'en':':family_woman_woman_boy_boy:','status':fully_qualified,'E':2,'de':':familie_frau_frau_junge_und_junge:','es':':familia_mujer_mujer_niño_niño:','fr':':famille_femme_femme_garçon_et_garçon:','ja':':家族_女性_女性_男の子_男の子:','ko':':가족_여자_여자_남자_아이_남자_아이:','pt':':família_mulher_mulher_menino_e_menino:','it':':famiglia_donna_donna_bambino_e_bambino:','fa':':خانواده_زن_زن_پسر_و_پسر:','id':':keluarga_wanita_wanita_anak_laki-laki_anak_laki-laki:','zh':':家庭_女人女人男孩男孩:','ru':':семья_женщина_женщина_мальчик_мальчик:','tr':':aile_kadın_kadın_erkek_çocuk_erkek_çocuk:','ar':':أسرة_سيدة_وسيدة_وصبي_وصبي:'},'\U0001F469\U0000200D\U0001F469\U0000200D\U0001F467':{'en':':family_woman_woman_girl:','status':fully_qualified,'E':2,'de':':familie_frau_frau_und_mädchen:','es':':familia_mujer_mujer_niña:','fr':':famille_femme_femme_et_fille:','ja':':家族_女性_女性_女の子:','ko':':가족_여자_여자_여자_아이:','pt':':família_mulher_mulher_e_menina:','it':':famiglia_donna_donna_e_bambina:','fa':':خانواده_زن_زن_و_دختر:','id':':keluarga_wanita_wanita_anak_perempuan:','zh':':家庭_女人女人女孩:','ru':':семья_женщина_женщина_девочка:','tr':':aile_kadın_kadın_kız_çocuk:','ar':':أسرة_سيدة_وسيدة_وفتاة:'},'\U0001F469\U0000200D\U0001F469\U0000200D\U0001F467\U0000200D\U0001F466':{'en':':family_woman_woman_girl_boy:','status':fully_qualified,'E':2,'de':':familie_frau_frau_mädchen_und_junge:','es':':familia_mujer_mujer_niña_niño:','fr':':famille_femme_femme_fille_et_garçon:','ja':':家族_女性_女性_女の子_男の子:','ko':':가족_여자_여자_여자_아이_남자_아이:','pt':':família_mulher_mulher_menina_e_menino:','it':':famiglia_donna_donna_bambina_e_bambino:','fa':':خانواده_زن_زن_دختر_و_پسر:','id':':keluarga_wanita_wanita_anak_perempuan_anak_laki-laki:','zh':':家庭_女人女人女孩男孩:','ru':':семья_женщина_женщина_девочка_мальчик:','tr':':aile_kadın_kadın_kız_çocuk_erkek_çocuk:','ar':':أسرة_سيدة_وسيدة_وفتاة_وصبي:'},'\U0001F469\U0000200D\U0001F469\U0000200D\U0001F467\U0000200D\U0001F467':{'en':':family_woman_woman_girl_girl:','status':fully_qualified,'E':2,'de':':familie_frau_frau_mädchen_und_mädchen:','es':':familia_mujer_mujer_niña_niña:','fr':':famille_femme_femme_fille_et_fille:','ja':':家族_女性_女性_女の子_女の子:','ko':':가족_여자_여자_여자_아이_여자_아이:','pt':':família_mulher_mulher_menina_e_menina:','it':':famiglia_donna_donna_bambina_e_bambina:','fa':':خانواده_زن_زن_دختر_و_دختر:','id':':keluarga_wanita_wanita_anak_perempuan_anak_perempuan:','zh':':家庭_女人女人女孩女孩:','ru':':семья_женщина_женщина_девочка_девочка:','tr':':aile_kadın_kadın_kız_çocuk_kız_çocuk:','ar':':أسرة_سيدة_وسيدة_وفتاة_وفتاة:'},'\U0001F9D1\U0000200D\U0001F33E':{'en':':farmer:','status':fully_qualified,'E':12.1,'de':':bauer/bäuerin:','es':':profesional_de_la_agricultura:','fr':':fermier_(tous_genres):','ja':':農家:','ko':':농부:','pt':':agricultor:','it':':agricoltore:','fa':':کشاورز:','id':':petani:','zh':':农民:','ru':':фермер:','tr':':çiftçi:','ar':':مُزارع:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F33E':{'en':':farmer_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':bauer/bäuerin_dunkle_hautfarbe:','es':':profesional_de_la_agricultura_tono_de_piel_oscuro:','fr':':fermier_(tous_genres)_peau_foncée:','ja':':農家_濃い肌色:','ko':':농부_검은색_피부:','pt':':agricultor_pele_escura:','it':':agricoltore_carnagione_scura:','fa':':کشاورز_پوست_آبنوسی:','id':':petani_warna_kulit_gelap:','zh':':农民_较深肤色:','ru':':фермер_очень_темный_тон_кожи:','tr':':çiftçi_koyu_cilt_tonu:','ar':':مُزارع_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F33E':{'en':':farmer_light_skin_tone:','status':fully_qualified,'E':12.1,'de':':bauer/bäuerin_helle_hautfarbe:','es':':profesional_de_la_agricultura_tono_de_piel_claro:','fr':':fermier_(tous_genres)_peau_claire:','ja':':農家_薄い肌色:','ko':':농부_하얀_피부:','pt':':agricultor_pele_clara:','it':':agricoltore_carnagione_chiara:','fa':':کشاورز_پوست_سفید:','id':':petani_warna_kulit_cerah:','zh':':农民_较浅肤色:','ru':':фермер_очень_светлый_тон_кожи:','tr':':çiftçi_açık_cilt_tonu:','ar':':مُزارع_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F33E':{'en':':farmer_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':bauer/bäuerin_mitteldunkle_hautfarbe:','es':':profesional_de_la_agricultura_tono_de_piel_oscuro_medio:','fr':':fermier_(tous_genres)_peau_mate:','ja':':農家_やや濃い肌色:','ko':':농부_진한_갈색_피부:','pt':':agricultor_pele_morena_escura:','it':':agricoltore_carnagione_abbastanza_scura:','fa':':کشاورز_پوست_گندمی:','id':':petani_warna_kulit_gelap-sedang:','zh':':农民_中等深肤色:','ru':':фермер_темный_тон_кожи:','tr':':çiftçi_orta_koyu_cilt_tonu:','ar':':مُزارع_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F33E':{'en':':farmer_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':bauer/bäuerin_mittelhelle_hautfarbe:','es':':profesional_de_la_agricultura_tono_de_piel_claro_medio:','fr':':fermier_(tous_genres)_peau_moyennement_claire:','ja':':農家_やや薄い肌色:','ko':':농부_연한_갈색_피부:','pt':':agricultor_pele_morena_clara:','it':':agricoltore_carnagione_abbastanza_chiara:','fa':':کشاورز_پوست_روشن:','id':':petani_warna_kulit_cerah-sedang:','zh':':农民_中等浅肤色:','ru':':фермер_светлый_тон_кожи:','tr':':çiftçi_orta_açık_cilt_tonu:','ar':':مُزارع_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F33E':{'en':':farmer_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':bauer/bäuerin_mittlere_hautfarbe:','es':':profesional_de_la_agricultura_tono_de_piel_medio:','fr':':fermier_(tous_genres)_peau_légèrement_mate:','ja':':農家_中間の肌色:','ko':':농부_갈색_피부:','pt':':agricultor_pele_morena:','it':':agricoltore_carnagione_olivastra:','fa':':کشاورز_پوست_طلایی:','id':':petani_warna_kulit_sedang:','zh':':农民_中等肤色:','ru':':фермер_средний_тон_кожи:','tr':':çiftçi_orta_cilt_tonu:','ar':':مُزارع_بشرة_بلون_معتدل:'},'\U000023E9':{'en':':fast-forward_button:','status':fully_qualified,'E':0.6,'alias':[':fast_forward:', ':fast_forward_button:'],'variant':True,'de':':doppelpfeile_nach_rechts:','es':':avance_rápido:','fr':':bouton_avance_rapide:','ja':':早送りボタン:','ko':':빨리_감기:','pt':':botão_avançar:','it':':pulsante_di_avanzamento_rapido:','fa':':دکمهٔ_سریع_به_جلو:','id':':tombol_maju_cepat:','zh':':快进按钮:','ru':':перемотка_вперед:','tr':':ileri_sarma_düğmesi:','ar':':زر_تقديم_سريع:'},'\U000023EC':{'en':':fast_down_button:','status':fully_qualified,'E':0.6,'alias':[':arrow_double_down:'],'variant':True,'de':':doppelpfeile_nach_unten:','es':':triángulo_doble_hacia_abajo:','fr':':double_flèche_vers_le_bas:','ja':':高速下ボタン:','ko':':아래쪽_화살표:','pt':':botão_de_avanço_para_baixo:','it':':doppia_freccia_in_basso:','fa':':دکمهٔ_سریع_به_پایین:','id':':tombol_turun_cepat:','zh':':快速下降按钮:','ru':':два_треугольника_острием_вниз:','tr':':hızlı_aşağı_düğmesi:','ar':':زر_سريع_لأسفل:'},'\U000023EA':{'en':':fast_reverse_button:','status':fully_qualified,'E':0.6,'alias':[':rewind:'],'variant':True,'de':':zurückspulen:','es':':retroceso_rápido:','fr':':bouton_retour_rapide:','ja':':早戻しボタン:','ko':':되감기:','pt':':botão_de_retroceder:','it':':pulsante_di_riavvolgimento_rapido:','fa':':دکمهٔ_سریع_به_عقب:','id':':tombol_mundur_cepat:','zh':':快退按钮:','ru':':перемотка_назад:','tr':':hızlı_geri_düğmesi:','ar':':زر_ترجيع_سريع:'},'\U000023EB':{'en':':fast_up_button:','status':fully_qualified,'E':0.6,'alias':[':arrow_double_up:'],'variant':True,'de':':doppelpfeile_nach_oben:','es':':triángulo_doble_hacia_arriba:','fr':':double_flèche_vers_le_haut:','ja':':高速上ボタン:','ko':':위쪽_화살표:','pt':':botão_de_avanço_para_cima:','it':':pulsante_doppia_freccia_in_alto:','fa':':دکمهٔ_سریع_به_بالا:','id':':tombol_naik_cepat:','zh':':快速上升按钮:','ru':':два_треугольника_острием_вверх:','tr':':hızlı_yukarı_düğmesi:','ar':':زر_سريع_لأعلى:'},'\U0001F4E0':{'en':':fax_machine:','status':fully_qualified,'E':0.6,'alias':[':fax:'],'de':':faxgerät:','es':':máquina_de_fax:','fr':':fax:','ja':':ファックス:','ko':':팩스:','pt':':fax:','it':':fax:','fa':':دستگاه_نمابر:','id':':mesin_faks:','zh':':传真机:','ru':':факс:','tr':':faks_makinesi:','ar':':فاكس:'},'\U0001F628':{'en':':fearful_face:','status':fully_qualified,'E':0.6,'alias':[':fearful:'],'de':':ängstliches_gesicht:','es':':cara_asustada:','fr':':visage_effrayé:','ja':':青ざめ:','ko':':두려워하는_얼굴:','pt':':rosto_amedrontado:','it':':faccina_impaurita:','fa':':ترسیده:','id':':wajah_ketakutan:','zh':':害怕:','ru':':боится:','tr':':korkulu_yüz:','ar':':وجه_خائف:'},'\U0001FAB6':{'en':':feather:','status':fully_qualified,'E':13,'de':':feder:','es':':pluma:','fr':':plume:','ja':':羽:','ko':':깃털:','pt':':pena:','it':':piuma:','fa':':پَر:','id':':bulu:','zh':':羽毛:','ru':':перо:','tr':':kuş_tüyü:','ar':':ريشة:'},'\U00002640\U0000FE0F':{'en':':female_sign:','status':fully_qualified,'E':4,'variant':True,'de':':frauensymbol:','es':':signo_femenino:','fr':':symbole_de_la_femme:','ja':':女性のマーク:','ko':':여성_기호:','pt':':símbolo_de_feminino:','it':':simbolo_genere_femminile:','fa':':نشان_ناهید:','id':':simbol_wanita:','zh':':女性符号:','ru':':женскии_символ:','tr':':kadın_sembolü:','ar':':رمز_أنثى:'},'\U00002640':{'en':':female_sign:','status':unqualified,'E':4,'variant':True,'de':':frauensymbol:','es':':signo_femenino:','fr':':symbole_de_la_femme:','ja':':女性のマーク:','ko':':여성_기호:','pt':':símbolo_de_feminino:','it':':simbolo_genere_femminile:','fa':':نشان_ناهید:','id':':simbol_wanita:','zh':':女性符号:','ru':':женский_символ:','tr':':kadın_sembolü:','ar':':رمز_أنثى:'},'\U0001F3A1':{'en':':ferris_wheel:','status':fully_qualified,'E':0.6,'de':':riesenrad:','es':':noria_de_feria:','fr':':grande_roue:','ja':':観覧車:','ko':':관람차:','pt':':roda_gigante:','it':':ruota_panoramica:','fa':':چرخ_وفلک:','id':':kincir_ria:','zh':':摩天轮:','ru':':колесо_обозрения:','tr':':dönme_dolap:','ar':':عجلة_دوارة:'},'\U000026F4\U0000FE0F':{'en':':ferry:','status':fully_qualified,'E':0.7,'variant':True,'de':':fähre:','es':':ferri:','fr':':ferry:','ja':':フェリー:','ko':':페리:','pt':':balsa:','it':':traghetto:','fa':':فرابر:','id':':kapal_feri:','zh':':渡轮:','ru':':паром:','tr':':feribot:','ar':':عبارة:'},'\U000026F4':{'en':':ferry:','status':unqualified,'E':0.7,'variant':True,'de':':fähre:','es':':ferri:','fr':':ferry:','ja':':フェリー:','ko':':페리:','pt':':balsa:','it':':traghetto:','fa':':فرابر:','id':':kapal_feri:','zh':':渡轮:','ru':':паром:','tr':':feribot:','ar':':عبارة:'},'\U0001F3D1':{'en':':field_hockey:','status':fully_qualified,'E':1,'alias':[':field_hockey_stick_and_ball:'],'de':':feldhockey:','es':':hockey_sobre_hierba:','fr':':hockey_sur_gazon:','ja':':ホッケー:','ko':':필드_하키:','pt':':hóquei_de_campo:','it':':hockey_su_prato:','fa':':هاکی_روی_چمن:','id':':hoki_lapangan:','zh':':曲棍球:','ru':':хоккей_на_траве:','tr':':çim_hokeyi:','ar':':هوكي_أرضي:'},'\U0001F5C4\U0000FE0F':{'en':':file_cabinet:','status':fully_qualified,'E':0.7,'variant':True,'de':':aktenschrank:','es':':archivador:','fr':':meuble_à_dossiers:','ja':':ファイルキャビネット:','ko':':파일_보관_서랍장:','pt':':gavetas_de_escritório:','it':':schedario_da_ufficio:','fa':':کشوی_پرونده:','id':':lemari_file:','zh':':文件柜:','ru':':архивныи_шкаф:','tr':':dosya_dolabı:','ar':':خزانة_ملفات:'},'\U0001F5C4':{'en':':file_cabinet:','status':unqualified,'E':0.7,'variant':True,'de':':aktenschrank:','es':':archivador:','fr':':meuble_à_dossiers:','ja':':ファイルキャビネット:','ko':':파일_보관_서랍장:','pt':':gavetas_de_escritório:','it':':schedario_da_ufficio:','fa':':کشوی_پرونده:','id':':lemari_file:','zh':':文件柜:','ru':':архивный_шкаф:','tr':':dosya_dolabı:','ar':':خزانة_ملفات:'},'\U0001F4C1':{'en':':file_folder:','status':fully_qualified,'E':0.6,'de':':ordner:','es':':carpeta_de_archivos:','fr':':dossier:','ja':':フォルダー:','ko':':폴더:','pt':':pasta_de_arquivos:','it':':cartella_file:','fa':':پوشه:','id':':folder_file:','zh':':文件夹:','ru':':папка:','tr':':dosya_klasörü:','ar':':مجلد_ملفات:'},'\U0001F39E\U0000FE0F':{'en':':film_frames:','status':fully_qualified,'E':0.7,'alias':[':film_strip:'],'variant':True,'de':':filmstreifen:','es':':fotograma_de_película:','fr':':pellicule:','ja':':映画フィルム:','ko':':영화_프레임:','pt':':rolo_de_filmes:','it':':pellicola_cinematografica:','fa':':حلقه_فیلم:','id':':frame_film:','zh':':影片帧:','ru':':кинопленка:','tr':':film_kareleri:','ar':':شريط_سينمائي:'},'\U0001F39E':{'en':':film_frames:','status':unqualified,'E':0.7,'alias':[':film_strip:'],'variant':True,'de':':filmstreifen:','es':':fotograma_de_película:','fr':':pellicule:','ja':':映画フィルム:','ko':':영화_프레임:','pt':':rolo_de_filmes:','it':':pellicola_cinematografica:','fa':':حلقهٔ_فیلم:','id':':frame_film:','zh':':影片帧:','ru':':кинопленка:','tr':':film_kareleri:','ar':':شريط_سينمائي:'},'\U0001F4FD\U0000FE0F':{'en':':film_projector:','status':fully_qualified,'E':0.7,'variant':True,'de':':filmprojektor:','es':':proyector_de_cine:','fr':':projecteur_cinématographique:','ja':':映写機:','ko':':영화_프로젝터:','pt':':projetor_de_filmes:','it':':proiettore_cinematografico:','fa':':پروژکتور:','id':':proyektor_film:','zh':':电影放映机:','ru':':кинопроектор:','tr':':film_projektörü:','ar':':جهاز_عرض_أفلام:'},'\U0001F4FD':{'en':':film_projector:','status':unqualified,'E':0.7,'variant':True,'de':':filmprojektor:','es':':proyector_de_cine:','fr':':projecteur_cinématographique:','ja':':映写機:','ko':':영화_프로젝터:','pt':':projetor_de_filmes:','it':':proiettore_cinematografico:','fa':':پروژکتور:','id':':proyektor_film:','zh':':电影放映机:','ru':':кинопроектор:','tr':':film_projektörü:','ar':':جهاز_عرض_أفلام:'},'\U0001F525':{'en':':fire:','status':fully_qualified,'E':0.6,'de':':feuer:','es':':fuego:','fr':':feu:','ja':':火:','ko':':불:','pt':':fogo:','it':':fuoco:','fa':':آتش:','id':':api:','zh':':火焰:','ru':':огонь:','tr':':ateş:','ar':':حريق:'},'\U0001F692':{'en':':fire_engine:','status':fully_qualified,'E':0.6,'de':':feuerwehrauto:','es':':coche_de_bomberos:','fr':':camion_de_pompier:','ja':':消防車:','ko':':소방차:','pt':':carro_do_corpo_de_bombeiros:','it':':camion_dei_pompieri:','fa':':ماشین_آتش_نشانی:','id':':mobil_pemadam_kebakaran:','zh':':消防车:','ru':':пожарная_машина:','tr':':itfaiye_arabası:','ar':':سيارة_إطفاء:'},'\U0001F9EF':{'en':':fire_extinguisher:','status':fully_qualified,'E':11,'de':':feuerlöscher:','es':':extintor:','fr':':extincteur:','ja':':消火器:','ko':':소화기:','pt':':extintor_de_incêndio:','it':':estintore:','fa':':کپسول_آتش_نشانی:','id':':alat_pemadam_api_ringan:','zh':':灭火器:','ru':':огнетушитель:','tr':':yangın_söndürücü:','ar':':مطفأة_حريق:'},'\U0001F9E8':{'en':':firecracker:','status':fully_qualified,'E':11,'de':':feuerwerkskörper:','es':':petardo:','fr':':pétard:','ja':':爆竹:','ko':':폭죽:','pt':':bombinha:','it':':petardo:','fa':':ترقه:','id':':petasan:','zh':':爆竹:','ru':':динамитная_шашка:','tr':':maytap:','ar':':مفرقعات:'},'\U0001F9D1\U0000200D\U0001F692':{'en':':firefighter:','status':fully_qualified,'E':12.1,'de':':feuerwehrmann/-frau:','es':':bombero:','fr':':pompier:','ja':':消防士:','ko':':소방관:','pt':':bombeiro:','it':':pompiere:','fa':':مأمور_آتش_نشانی:','id':':pemadam_kebakaran:','zh':':消防员:','ru':':пожарный:','tr':':itfaiyeci:','ar':':إطفائي:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F692':{'en':':firefighter_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':feuerwehrmann/-frau_dunkle_hautfarbe:','es':':bombero_tono_de_piel_oscuro:','fr':':pompier_peau_foncée:','ja':':消防士_濃い肌色:','ko':':소방관_검은색_피부:','pt':':bombeiro_pele_escura:','it':':pompiere_carnagione_scura:','fa':':مأمور_آتش_نشانی_پوست_آبنوسی:','id':':pemadam_kebakaran_warna_kulit_gelap:','zh':':消防员_较深肤色:','ru':':пожарный_очень_темный_тон_кожи:','tr':':itfaiyeci_koyu_cilt_tonu:','ar':':إطفائي_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F692':{'en':':firefighter_light_skin_tone:','status':fully_qualified,'E':12.1,'de':':feuerwehrmann/-frau_helle_hautfarbe:','es':':bombero_tono_de_piel_claro:','fr':':pompier_peau_claire:','ja':':消防士_薄い肌色:','ko':':소방관_하얀_피부:','pt':':bombeiro_pele_clara:','it':':pompiere_carnagione_chiara:','fa':':مأمور_آتش_نشانی_پوست_سفید:','id':':pemadam_kebakaran_warna_kulit_cerah:','zh':':消防员_较浅肤色:','ru':':пожарный_очень_светлый_тон_кожи:','tr':':itfaiyeci_açık_cilt_tonu:','ar':':إطفائي_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F692':{'en':':firefighter_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':feuerwehrmann/-frau_mitteldunkle_hautfarbe:','es':':bombero_tono_de_piel_oscuro_medio:','fr':':pompier_peau_mate:','ja':':消防士_やや濃い肌色:','ko':':소방관_진한_갈색_피부:','pt':':bombeiro_pele_morena_escura:','it':':pompiere_carnagione_abbastanza_scura:','fa':':مأمور_آتش_نشانی_پوست_گندمی:','id':':pemadam_kebakaran_warna_kulit_gelap-sedang:','zh':':消防员_中等深肤色:','ru':':пожарный_темный_тон_кожи:','tr':':itfaiyeci_orta_koyu_cilt_tonu:','ar':':إطفائي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F692':{'en':':firefighter_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':feuerwehrmann/-frau_mittelhelle_hautfarbe:','es':':bombero_tono_de_piel_claro_medio:','fr':':pompier_peau_moyennement_claire:','ja':':消防士_やや薄い肌色:','ko':':소방관_연한_갈색_피부:','pt':':bombeiro_pele_morena_clara:','it':':pompiere_carnagione_abbastanza_chiara:','fa':':مأمور_آتش_نشانی_پوست_روشن:','id':':pemadam_kebakaran_warna_kulit_cerah-sedang:','zh':':消防员_中等浅肤色:','ru':':пожарный_светлый_тон_кожи:','tr':':itfaiyeci_orta_açık_cilt_tonu:','ar':':إطفائي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F692':{'en':':firefighter_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':feuerwehrmann/-frau_mittlere_hautfarbe:','es':':bombero_tono_de_piel_medio:','fr':':pompier_peau_légèrement_mate:','ja':':消防士_中間の肌色:','ko':':소방관_갈색_피부:','pt':':bombeiro_pele_morena:','it':':pompiere_carnagione_olivastra:','fa':':مأمور_آتش_نشانی_پوست_طلایی:','id':':pemadam_kebakaran_warna_kulit_sedang:','zh':':消防员_中等肤色:','ru':':пожарный_средний_тон_кожи:','tr':':itfaiyeci_orta_cilt_tonu:','ar':':إطفائي_بشرة_بلون_معتدل:'},'\U0001F386':{'en':':fireworks:','status':fully_qualified,'E':0.6,'de':':feuerwerk:','es':':fuegos_artificiales:','fr':':feu_d’artifice:','ja':':打ち上げ花火:','ko':':불꽃놀이:','pt':':fogos_de_artifício:','it':':fuochi_d’artificio:','fa':':آتش_بازی:','id':':kembang_api:','zh':':焰火:','ru':':фейерверк:','tr':':havai_fişek:','ar':':ألعاب_نارية:'},'\U0001F313':{'en':':first_quarter_moon:','status':fully_qualified,'E':0.6,'de':':zunehmender_halbmond:','es':':luna_en_cuarto_creciente:','fr':':premier_quartier_de_lune:','ja':':上弦の月:','ko':':오른쪽_반달:','pt':':quarto_crescente:','it':':primo_quarto_di_luna:','fa':':یک_چهارم_نخست:','id':':bulan_perbani_awal:','zh':':上弦月:','ru':':луна_в_первой_четверти:','tr':':ilk_dördün:','ar':':تربيع_أول:'},'\U0001F31B':{'en':':first_quarter_moon_face:','status':fully_qualified,'E':0.6,'alias':[':first_quarter_moon_with_face:'],'de':':mondsichel_mit_gesicht_links:','es':':luna_de_cuarto_creciente_con_cara:','fr':':premier_quartier_de_lune_avec_visage:','ja':':顔のある上弦の月:','ko':':얼굴이_있는_초승달:','pt':':rosto_da_lua_de_quarto_crescente:','it':':faccina_primo_quarto_di_luna:','fa':':صورت_یک_چهارم_اول:','id':':wajah_bulan_perbani_awal:','zh':':微笑的上弦月:','ru':':луна_в_первой_четверти_с_лицом:','tr':':ilk_dördün_ay_yüzü:','ar':':وجه_تربيع_أول:'},'\U0001F41F':{'en':':fish:','status':fully_qualified,'E':0.6,'variant':True,'de':':fisch:','es':':pez:','fr':':poisson:','ja':':魚:','ko':':물고기:','pt':':peixe:','it':':pesce:','fa':':ماهی:','id':':ikan:','zh':':鱼:','ru':':рыба:','tr':':balık:','ar':':سمكة:'},'\U0001F365':{'en':':fish_cake_with_swirl:','status':fully_qualified,'E':0.6,'alias':[':fish_cake:'],'de':':fischfrikadelle:','es':':pastel_de_pescado_japonés:','fr':':croquette_de_poisson:','ja':':なると:','ko':':어묵:','pt':':bolinho_de_peixe:','it':':tortino_di_pesce_a_spirale:','fa':':کیک_ماهی:','id':':kue_ikan_dengan_desain_pusaran:','zh':':鱼板:','ru':':камабоко:','tr':':balık_köftesi:','ar':':كعكة_سمك_بشكل_حلزوني:'},'\U0001F3A3':{'en':':fishing_pole:','status':fully_qualified,'E':0.6,'alias':[':fishing_pole_and_fish:'],'de':':angel_mit_fisch:','es':':caña_de_pescar:','fr':':pêche_à_la_ligne:','ja':':釣り:','ko':':낚싯대:','pt':':pesca:','it':':canna_da_pesca:','fa':':چوب_ماهیگیری:','id':':tongkat_pancing:','zh':':钓鱼竿:','ru':':удочка_с_рыбой:','tr':':olta:','ar':':صنارة_صيد:'},'\U0001F560':{'en':':five-thirty:','status':fully_qualified,'E':0.7,'alias':[':clock530:', ':five_thirty:'],'variant':True,'de':':5.30_uhr:','es':':cinco_y_media:','fr':':cinq_heures_et_demie:','ja':':5時半:','ko':':다섯_시_반:','pt':':cinco_e_meia:','it':':ore_cinque_e_mezza:','fa':':ساعت_پنج_و_نیم:','id':':jam_setengah_enam:','zh':':五点半:','ru':':полшестого:','tr':':beş_buçuk:','ar':':الخامسة_والنصف:'},'\U0001F554':{'en':':five_o’clock:','status':fully_qualified,'E':0.6,'alias':[':clock5:', ':five_oclock:'],'variant':True,'de':':5.00_uhr:','es':':5_en_punto:','fr':':cinq_heures:','ja':':5時:','ko':':다섯_시:','pt':':5_horas:','it':':ore_cinque:','fa':':ساعت_پنج:','id':':jam_lima_tepat:','zh':':五点:','ru':':пять_часов:','tr':':saat_5:','ar':':الساعة_الخامسة:'},'\U000026F3':{'en':':flag_in_hole:','status':fully_qualified,'E':0.6,'alias':[':golf:'],'variant':True,'de':':golffahne:','es':':banderín_en_hoyo:','fr':':drapeau_de_golf:','ja':':ゴルフ:','ko':':골프:','pt':':bandeira_no_buraco:','it':':bandiera_in_buca:','fa':':پرچم_در_سوراخ:','id':':bendera_di_lubang_golf:','zh':':高尔夫球洞:','ru':':гольф:','tr':':delikteki_bayrak:','ar':':علم_على_حفرة:'},'\U0001F9A9':{'en':':flamingo:','status':fully_qualified,'E':12,'de':':flamingo:','es':':flamenco:','fr':':flamant:','ja':':フラミンゴ:','ko':':홍학:','pt':':flamingo:','it':':fenicottero:','fa':':فلامینگو:','id':':flamingo:','zh':':火烈鸟:','ru':':фламинго:','tr':':flamingo:','ar':':فلامينغو:'},'\U0001F526':{'en':':flashlight:','status':fully_qualified,'E':0.6,'de':':taschenlampe:','es':':linterna:','fr':':torche:','ja':':懐中電灯:','ko':':손전등:','pt':':lanterna:','it':':torcia:','fa':':چراغ_قوه:','id':':lampu_senter:','zh':':手电筒:','ru':':фонарик:','tr':':el_feneri:','ar':':ضوء_فلاش:'},'\U0001F97F':{'en':':flat_shoe:','status':fully_qualified,'E':11,'de':':flacher_schuh:','es':':bailarina:','fr':':chaussure_plate:','ja':':フラットシューズ:','ko':':플랫_슈즈:','pt':':sapatilha:','it':':ballerina:','fa':':کفش_بی_پاشنه:','id':':sepatu_flat:','zh':':平底鞋:','ru':':балетки:','tr':':düz_ayakkabı:','ar':':حذاء_نسائي_بدون_كعب:'},'\U0001FAD3':{'en':':flatbread:','status':fully_qualified,'E':13,'de':':fladenbrot:','es':':pan_sin_levadura:','fr':':galette:','ja':':フラットブレッド:','ko':':플랫브레드:','pt':':pão_sírio:','it':':focaccia:','fa':':نان_مسطح:','id':':roti_pipih:','zh':':扁面包:','ru':':лепешка:','tr':':yassı_ekmek:','ar':':خبز_مفرود:'},'\U0000269C\U0000FE0F':{'en':':fleur-de-lis:','status':fully_qualified,'E':1,'alias':[':fleur_de_lis:'],'variant':True,'de':':lilie:','es':':flor_de_lis:','fr':':fleur_de_lys:','ja':':フルール・ド・リス:','ko':':백합_문장:','pt':':flor-de-lis:','it':':giglio_araldico:','fa':':نماد_گل_زنبق:','id':':fleur-de-lis:','zh':':百合花饰:','ru':':геральдическая_лилия:','tr':':zambak:','ar':':زنبقة:'},'\U0000269C':{'en':':fleur-de-lis:','status':unqualified,'E':1,'alias':[':fleur_de_lis:'],'variant':True,'de':':lilie:','es':':flor_de_lis:','fr':':fleur_de_lys:','ja':':フルール・ド・リス:','ko':':백합_문장:','pt':':flor-de-lis:','it':':giglio_araldico:','fa':':نماد_گل_زنبق:','id':':fleur-de-lis:','zh':':百合花饰:','ru':':геральдическая_лилия:','tr':':zambak:','ar':':زنبقة:'},'\U0001F4AA':{'en':':flexed_biceps:','status':fully_qualified,'E':0.6,'alias':[':muscle:'],'de':':angespannter_bizeps:','es':':bíceps_flexionado:','fr':':biceps_contracté:','ja':':力こぶ:','ko':':알통:','pt':':bíceps:','it':':bicipite:','fa':':عضلهٔ_بازو:','id':':lengan_berotot:','zh':':肌肉:','ru':':бицепс:','tr':':pazı:','ar':':عضلة_الذراع:'},'\U0001F4AA\U0001F3FF':{'en':':flexed_biceps_dark_skin_tone:','status':fully_qualified,'E':1,'de':':angespannter_bizeps_dunkle_hautfarbe:','es':':bíceps_flexionado_tono_de_piel_oscuro:','fr':':biceps_contracté_peau_foncée:','ja':':力こぶ_濃い肌色:','ko':':알통_검은색_피부:','pt':':bíceps_pele_escura:','it':':bicipite_carnagione_scura:','fa':':عضلهٔ_بازو_پوست_آبنوسی:','id':':lengan_berotot_warna_kulit_gelap:','zh':':肌肉_较深肤色:','ru':':бицепс_очень_темный_тон_кожи:','tr':':pazı_koyu_cilt_tonu:','ar':':عضلة_الذراع_بشرة_بلون_غامق:'},'\U0001F4AA\U0001F3FB':{'en':':flexed_biceps_light_skin_tone:','status':fully_qualified,'E':1,'de':':angespannter_bizeps_helle_hautfarbe:','es':':bíceps_flexionado_tono_de_piel_claro:','fr':':biceps_contracté_peau_claire:','ja':':力こぶ_薄い肌色:','ko':':알통_하얀_피부:','pt':':bíceps_pele_clara:','it':':bicipite_carnagione_chiara:','fa':':عضلهٔ_بازو_پوست_سفید:','id':':lengan_berotot_warna_kulit_cerah:','zh':':肌肉_较浅肤色:','ru':':бицепс_очень_светлый_тон_кожи:','tr':':pazı_açık_cilt_tonu:','ar':':عضلة_الذراع_بشرة_بلون_فاتح:'},'\U0001F4AA\U0001F3FE':{'en':':flexed_biceps_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':angespannter_bizeps_mitteldunkle_hautfarbe:','es':':bíceps_flexionado_tono_de_piel_oscuro_medio:','fr':':biceps_contracté_peau_mate:','ja':':力こぶ_やや濃い肌色:','ko':':알통_진한_갈색_피부:','pt':':bíceps_pele_morena_escura:','it':':bicipite_carnagione_abbastanza_scura:','fa':':عضلهٔ_بازو_پوست_گندمی:','id':':lengan_berotot_warna_kulit_gelap-sedang:','zh':':肌肉_中等深肤色:','ru':':бицепс_темный_тон_кожи:','tr':':pazı_orta_koyu_cilt_tonu:','ar':':عضلة_الذراع_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F4AA\U0001F3FC':{'en':':flexed_biceps_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':angespannter_bizeps_mittelhelle_hautfarbe:','es':':bíceps_flexionado_tono_de_piel_claro_medio:','fr':':biceps_contracté_peau_moyennement_claire:','ja':':力こぶ_やや薄い肌色:','ko':':알통_연한_갈색_피부:','pt':':bíceps_pele_morena_clara:','it':':bicipite_carnagione_abbastanza_chiara:','fa':':عضلهٔ_بازو_پوست_روشن:','id':':lengan_berotot_warna_kulit_cerah-sedang:','zh':':肌肉_中等浅肤色:','ru':':бицепс_светлый_тон_кожи:','tr':':pazı_orta_açık_cilt_tonu:','ar':':عضلة_الذراع_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F4AA\U0001F3FD':{'en':':flexed_biceps_medium_skin_tone:','status':fully_qualified,'E':1,'de':':angespannter_bizeps_mittlere_hautfarbe:','es':':bíceps_flexionado_tono_de_piel_medio:','fr':':biceps_contracté_peau_légèrement_mate:','ja':':力こぶ_中間の肌色:','ko':':알통_갈색_피부:','pt':':bíceps_pele_morena:','it':':bicipite_carnagione_olivastra:','fa':':عضلهٔ_بازو_پوست_طلایی:','id':':lengan_berotot_warna_kulit_sedang:','zh':':肌肉_中等肤色:','ru':':бицепс_средний_тон_кожи:','tr':':pazı_orta_cilt_tonu:','ar':':عضلة_الذراع_بشرة_بلون_معتدل:'},'\U0001F4BE':{'en':':floppy_disk:','status':fully_qualified,'E':0.6,'de':':diskette:','es':':disquete:','fr':':disquette:','ja':':フロッピー:','ko':':플로피_디스크:','pt':':disquete:','it':':floppy_disc:','fa':':فلاپی_دیسک:','id':':disket:','zh':':软盘:','ru':':дискета:','tr':':disket:','ar':':قرص_مرن:'},'\U0001F3B4':{'en':':flower_playing_cards:','status':fully_qualified,'E':0.6,'de':':japanische_blumenkarte:','es':':cartas_de_flores:','fr':':jeu_des_fleurs:','ja':':花札:','ko':':화투:','pt':':carta_de_flores:','it':':carta_da_gioco_hanafuda:','fa':':کارت_بازی_گل_دار:','id':':kartu_bunga:','zh':':花札:','ru':':цветочная_карта:','tr':':çiçekli_oyun_kağıtları:','ar':':بطاقات_لعب:'},'\U0001F633':{'en':':flushed_face:','status':fully_qualified,'E':0.6,'alias':[':flushed:'],'de':':errötetes_gesicht_mit_großen_augen:','es':':cara_sonrojada:','fr':':visage_qui_rougit:','ja':':赤面:','ko':':상기된_얼굴:','pt':':rosto_ruborizado:','it':':faccina_imbarazzata:','fa':':سرخ_شده:','id':':wajah_tersipu:','zh':':脸红:','ru':':краснеет:','tr':':kızarmış_ve_şaşkın_yüz:','ar':':وجه_متورد:'},'\U0001FA88':{'en':':flute:','status':fully_qualified,'E':15,'de':':flöte:','es':':flauta:','fr':':flûte:','ja':':笛:','ko':':플루트:','pt':':flauta:','it':':flauto:','fa':':فلوت:','id':':seruling:','zh':':长笛:','ru':':флейта:','tr':':flüt:','ar':':فلوت:'},'\U0001FAB0':{'en':':fly:','status':fully_qualified,'E':13,'de':':fliege:','es':':mosca:','fr':':mouche:','ja':':ハエ:','ko':':파리:','pt':':mosca:','it':':mosca:','fa':':مگس:','id':':lalat:','zh':':苍蝇:','ru':':муха:','tr':':karasinek:','ar':':ذبابة:'},'\U0001F94F':{'en':':flying_disc:','status':fully_qualified,'E':11,'de':':frisbee:','es':':disco_volador:','fr':':disque_volant:','ja':':フリスビー:','ko':':원반:','pt':':frisbee:','it':':frisbee:','fa':':فریزبی:','id':':lempar_cakram:','zh':':飞盘:','ru':':летающий_диск:','tr':':frizbi:','ar':':قرص_طائر:'},'\U0001F6F8':{'en':':flying_saucer:','status':fully_qualified,'E':5,'de':':fliegende_untertasse:','es':':platillo_volante:','fr':':soucoupe_volante:','ja':':空飛ぶ円盤:','ko':':비행접시:','pt':':disco_voador:','it':':disco_volante:','fa':':بشقاب_پرنده:','id':':piring_terbang:','zh':':飞碟:','ru':':летающая_тарелка:','tr':':uçan_daire:','ar':':صحن_طائر:'},'\U0001F32B\U0000FE0F':{'en':':fog:','status':fully_qualified,'E':0.7,'variant':True,'de':':nebel:','es':':niebla:','fr':':brouillard:','ja':':霧:','ko':':안개:','pt':':neblina:','it':':nebbia:','fa':':مه:','id':':kabut:','zh':':雾:','ru':':густои_туман:','tr':':sis:','ar':':ضبابي:'},'\U0001F32B':{'en':':fog:','status':unqualified,'E':0.7,'variant':True,'de':':nebel:','es':':niebla:','fr':':brouillard:','ja':':霧:','ko':':안개:','pt':':nevoeiro:','it':':nebbia:','fa':':مه:','id':':kabut:','zh':':雾:','ru':':густой_туман:','tr':':sis:','ar':':ضبابي:'},'\U0001F301':{'en':':foggy:','status':fully_qualified,'E':0.6,'de':':neblig:','es':':bruma:','fr':':brume:','ja':':霧の都会:','ko':':자욱한_안개:','pt':':enevoado:','it':':nebbioso:','fa':':مه_آلود:','id':':berkabut:','zh':':有雾:','ru':':туман:','tr':':sisli:','ar':':ضباب:'},'\U0001F64F':{'en':':folded_hands:','status':fully_qualified,'E':0.6,'alias':[':pray:'],'de':':zusammengelegte_handflächen:','es':':manos_en_oración:','fr':':mains_en_prière:','ja':':祈り:','ko':':기도:','pt':':mãos_juntas:','it':':mani_giunte:','fa':':خواهش_می_کنم:','id':':melipat_tangan:','zh':':双手合十:','ru':':сложенные_руки:','tr':':birleşen_eller:','ar':':يدان_مضمومتان:'},'\U0001F64F\U0001F3FF':{'en':':folded_hands_dark_skin_tone:','status':fully_qualified,'E':1,'de':':zusammengelegte_handflächen_dunkle_hautfarbe:','es':':manos_en_oración_tono_de_piel_oscuro:','fr':':mains_en_prière_peau_foncée:','ja':':祈り_濃い肌色:','ko':':기도_검은색_피부:','pt':':mãos_juntas_pele_escura:','it':':mani_giunte_carnagione_scura:','fa':':خواهش_می_کنم_پوست_آبنوسی:','id':':melipat_tangan_warna_kulit_gelap:','zh':':双手合十_较深肤色:','ru':':сложенные_руки_очень_темный_тон_кожи:','tr':':birleşen_eller_koyu_cilt_tonu:','ar':':يدان_مضمومتان_بشرة_بلون_غامق:'},'\U0001F64F\U0001F3FB':{'en':':folded_hands_light_skin_tone:','status':fully_qualified,'E':1,'de':':zusammengelegte_handflächen_helle_hautfarbe:','es':':manos_en_oración_tono_de_piel_claro:','fr':':mains_en_prière_peau_claire:','ja':':祈り_薄い肌色:','ko':':기도_하얀_피부:','pt':':mãos_juntas_pele_clara:','it':':mani_giunte_carnagione_chiara:','fa':':خواهش_می_کنم_پوست_سفید:','id':':melipat_tangan_warna_kulit_cerah:','zh':':双手合十_较浅肤色:','ru':':сложенные_руки_очень_светлый_тон_кожи:','tr':':birleşen_eller_açık_cilt_tonu:','ar':':يدان_مضمومتان_بشرة_بلون_فاتح:'},'\U0001F64F\U0001F3FE':{'en':':folded_hands_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':zusammengelegte_handflächen_mitteldunkle_hautfarbe:','es':':manos_en_oración_tono_de_piel_oscuro_medio:','fr':':mains_en_prière_peau_mate:','ja':':祈り_やや濃い肌色:','ko':':기도_진한_갈색_피부:','pt':':mãos_juntas_pele_morena_escura:','it':':mani_giunte_carnagione_abbastanza_scura:','fa':':خواهش_می_کنم_پوست_گندمی:','id':':melipat_tangan_warna_kulit_gelap-sedang:','zh':':双手合十_中等深肤色:','ru':':сложенные_руки_темный_тон_кожи:','tr':':birleşen_eller_orta_koyu_cilt_tonu:','ar':':يدان_مضمومتان_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F64F\U0001F3FC':{'en':':folded_hands_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':zusammengelegte_handflächen_mittelhelle_hautfarbe:','es':':manos_en_oración_tono_de_piel_claro_medio:','fr':':mains_en_prière_peau_moyennement_claire:','ja':':祈り_やや薄い肌色:','ko':':기도_연한_갈색_피부:','pt':':mãos_juntas_pele_morena_clara:','it':':mani_giunte_carnagione_abbastanza_chiara:','fa':':خواهش_می_کنم_پوست_روشن:','id':':melipat_tangan_warna_kulit_cerah-sedang:','zh':':双手合十_中等浅肤色:','ru':':сложенные_руки_светлый_тон_кожи:','tr':':birleşen_eller_orta_açık_cilt_tonu:','ar':':يدان_مضمومتان_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F64F\U0001F3FD':{'en':':folded_hands_medium_skin_tone:','status':fully_qualified,'E':1,'de':':zusammengelegte_handflächen_mittlere_hautfarbe:','es':':manos_en_oración_tono_de_piel_medio:','fr':':mains_en_prière_peau_légèrement_mate:','ja':':祈り_中間の肌色:','ko':':기도_갈색_피부:','pt':':mãos_juntas_pele_morena:','it':':mani_giunte_carnagione_olivastra:','fa':':خواهش_می_کنم_پوست_طلایی:','id':':melipat_tangan_warna_kulit_sedang:','zh':':双手合十_中等肤色:','ru':':сложенные_руки_средний_тон_кожи:','tr':':birleşen_eller_orta_cilt_tonu:','ar':':يدان_مضمومتان_بشرة_بلون_معتدل:'},'\U0001FAAD':{'en':':folding_hand_fan:','status':fully_qualified,'E':15,'de':':faltfächer:','es':':abanico_abierto:','fr':':éventail:','ja':':扇子:','ko':':접이식_손_부채:','pt':':leque_dobrável:','it':':ventaglio:','fa':':بادبزن:','id':':kipas_tangan_lipat:','zh':':折扇:','ru':':веер:','tr':':yelpaze:','ar':':مروحة_يدوية:'},'\U0001FAD5':{'en':':fondue:','status':fully_qualified,'E':13,'de':':fondue:','es':':fondue:','fr':':fondue:','ja':':フォンデュ:','ko':':퐁듀:','pt':':fondue:','it':':fonduta:','fa':':فوندو:','id':':fondue:','zh':':奶酪火锅:','ru':':фондю:','tr':':fondü:','ar':':فوندو:'},'\U0001F9B6':{'en':':foot:','status':fully_qualified,'E':11,'de':':fuß:','es':':pie:','fr':':pied:','ja':':足:','ko':':한쪽_발:','pt':':pé:','it':':piede:','fa':':پا:','id':':telapak_kaki:','zh':':脚:','ru':':щиколотка:','tr':':ayak:','ar':':قدم:'},'\U0001F9B6\U0001F3FF':{'en':':foot_dark_skin_tone:','status':fully_qualified,'E':11,'de':':fuß_dunkle_hautfarbe:','es':':pie_tono_de_piel_oscuro:','fr':':pied_peau_foncée:','ja':':足_濃い肌色:','ko':':한쪽_발_검은색_피부:','pt':':pé_pele_escura:','it':':piede_carnagione_scura:','fa':':پا_پوست_آبنوسی:','id':':telapak_kaki_warna_kulit_gelap:','zh':':脚_较深肤色:','ru':':щиколотка_очень_темный_тон_кожи:','tr':':ayak_koyu_cilt_tonu:','ar':':قدم_بشرة_بلون_غامق:'},'\U0001F9B6\U0001F3FB':{'en':':foot_light_skin_tone:','status':fully_qualified,'E':11,'de':':fuß_helle_hautfarbe:','es':':pie_tono_de_piel_claro:','fr':':pied_peau_claire:','ja':':足_薄い肌色:','ko':':한쪽_발_하얀_피부:','pt':':pé_pele_clara:','it':':piede_carnagione_chiara:','fa':':پا_پوست_سفید:','id':':telapak_kaki_warna_kulit_cerah:','zh':':脚_较浅肤色:','ru':':щиколотка_очень_светлый_тон_кожи:','tr':':ayak_açık_cilt_tonu:','ar':':قدم_بشرة_بلون_فاتح:'},'\U0001F9B6\U0001F3FE':{'en':':foot_medium-dark_skin_tone:','status':fully_qualified,'E':11,'de':':fuß_mitteldunkle_hautfarbe:','es':':pie_tono_de_piel_oscuro_medio:','fr':':pied_peau_mate:','ja':':足_やや濃い肌色:','ko':':한쪽_발_진한_갈색_피부:','pt':':pé_pele_morena_escura:','it':':piede_carnagione_abbastanza_scura:','fa':':پا_پوست_گندمی:','id':':telapak_kaki_warna_kulit_gelap-sedang:','zh':':脚_中等深肤色:','ru':':щиколотка_темный_тон_кожи:','tr':':ayak_orta_koyu_cilt_tonu:','ar':':قدم_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9B6\U0001F3FC':{'en':':foot_medium-light_skin_tone:','status':fully_qualified,'E':11,'de':':fuß_mittelhelle_hautfarbe:','es':':pie_tono_de_piel_claro_medio:','fr':':pied_peau_moyennement_claire:','ja':':足_やや薄い肌色:','ko':':한쪽_발_연한_갈색_피부:','pt':':pé_pele_morena_clara:','it':':piede_carnagione_abbastanza_chiara:','fa':':پا_پوست_روشن:','id':':telapak_kaki_warna_kulit_cerah-sedang:','zh':':脚_中等浅肤色:','ru':':щиколотка_светлый_тон_кожи:','tr':':ayak_orta_açık_cilt_tonu:','ar':':قدم_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9B6\U0001F3FD':{'en':':foot_medium_skin_tone:','status':fully_qualified,'E':11,'de':':fuß_mittlere_hautfarbe:','es':':pie_tono_de_piel_medio:','fr':':pied_peau_légèrement_mate:','ja':':足_中間の肌色:','ko':':한쪽_발_갈색_피부:','pt':':pé_pele_morena:','it':':piede_carnagione_olivastra:','fa':':پا_پوست_طلایی:','id':':telapak_kaki_warna_kulit_sedang:','zh':':脚_中等肤色:','ru':':щиколотка_средний_тон_кожи:','tr':':ayak_orta_cilt_tonu:','ar':':قدم_بشرة_بلون_معتدل:'},'\U0001F463':{'en':':footprints:','status':fully_qualified,'E':0.6,'de':':fußabdrücke:','es':':huellas_de_pies:','fr':':traces_de_pas:','ja':':足あと:','ko':':발자국:','pt':':pegadas:','it':':impronta_di_piedi:','fa':':رد_پا:','id':':jejak_kaki:','zh':':脚印:','ru':':следы:','tr':':ayak_izi:','ar':':أثر_قدمين:'},'\U0001F374':{'en':':fork_and_knife:','status':fully_qualified,'E':0.6,'de':':messer_und_gabel:','es':':tenedor_y_cuchillo:','fr':':fourchette_et_couteau:','ja':':ナイフとフォーク:','ko':':포크와_나이프:','pt':':garfo_e_faca:','it':':forchetta_e_coltello:','fa':':کارد_و_چنگال:','id':':garpu_dan_pisau:','zh':':刀叉:','ru':':вилка_и_нож:','tr':':çatal_ve_bıçak:','ar':':شوكة_وسكينة:'},'\U0001F37D\U0000FE0F':{'en':':fork_and_knife_with_plate:','status':fully_qualified,'E':0.7,'alias':[':plate_with_cutlery:'],'variant':True,'de':':teller_mit_messer_und_gabel:','es':':cuchillo_y_tenedor_con_un_plato:','fr':':fourchette_et_couteau_avec_assiette:','ja':':ナイフとフォークと皿:','ko':':포크와_나이프가_있는_접시:','pt':':prato_com_talheres:','it':':piatto_e_posate:','fa':':بشقاب_و_کارد_و_چنگال:','id':':garpu_dan_pisau_dengan_piring:','zh':':餐具:','ru':':тарелка_с_приборами:','tr':':tabakla_birlikte_çatal_ve_bıçak:','ar':':طبق_وشوكة_وسكينة:'},'\U0001F37D':{'en':':fork_and_knife_with_plate:','status':unqualified,'E':0.7,'alias':[':plate_with_cutlery:'],'variant':True,'de':':teller_mit_messer_und_gabel:','es':':cuchillo_y_tenedor_con_un_plato:','fr':':fourchette_et_couteau_avec_assiette:','ja':':ナイフとフォークと皿:','ko':':포크와_나이프가_있는_접시:','pt':':prato_com_talheres:','it':':piatto_e_posate:','fa':':بشقاب_و_کارد_و_چنگال:','id':':garpu_dan_pisau_dengan_piring:','zh':':餐具:','ru':':тарелка_с_приборами:','tr':':tabakla_birlikte_çatal_ve_bıçak:','ar':':طبق_وشوكة_وسكينة:'},'\U0001F960':{'en':':fortune_cookie:','status':fully_qualified,'E':5,'de':':glückskeks:','es':':galleta_de_la_fortuna:','fr':':biscuit_chinois:','ja':':フォーチュンクッキー:','ko':':포츈_쿠키:','pt':':biscoito_da_sorte:','it':':biscotto_della_fortuna:','fa':':شیرینی_شانس:','id':':kue_keberuntungan:','zh':':幸运饼干:','ru':':печенье_с_предсказанием:','tr':':fal_kurabiyesi:','ar':':كعكة_الحظ:'},'\U000026F2':{'en':':fountain:','status':fully_qualified,'E':0.6,'variant':True,'de':':springbrunnen:','es':':fuente:','fr':':fontaine:','ja':':噴水:','ko':':분수:','pt':':fonte:','it':':fontana:','fa':':فواره:','id':':air_mancur:','zh':':喷泉:','ru':':фонтан:','tr':':fıskiye:','ar':':نافورة:'},'\U0001F58B\U0000FE0F':{'en':':fountain_pen:','status':fully_qualified,'E':0.7,'alias':[':lower_left_fountain_pen:'],'variant':True,'de':':füllhalter:','es':':estilográfica:','fr':':stylo_plume:','ja':':万年筆:','ko':':만년필:','pt':':caneta_tinteiro:','it':':penna_stilografica:','fa':':خودنویس:','id':':pena_tinta:','zh':':钢笔:','ru':':перьевая_ручка:','tr':':dolma_kalem:','ar':':قلم_حبر:'},'\U0001F58B':{'en':':fountain_pen:','status':unqualified,'E':0.7,'alias':[':lower_left_fountain_pen:'],'variant':True,'de':':füllhalter:','es':':estilográfica:','fr':':stylo_plume:','ja':':万年筆:','ko':':만년필:','pt':':caneta_tinteiro:','it':':penna_stilografica:','fa':':خودنویس:','id':':pena_tinta:','zh':':钢笔:','ru':':перьевая_ручка:','tr':':dolma_kalem:','ar':':قلم_حبر:'},'\U0001F55F':{'en':':four-thirty:','status':fully_qualified,'E':0.7,'alias':[':clock430:', ':four_thirty:'],'variant':True,'de':':4.30_uhr:','es':':cuatro_y_media:','fr':':quatre_heures_et_demie:','ja':':4時半:','ko':':네_시_반:','pt':':quatro_e_meia:','it':':ore_quattro_e_mezza:','fa':':ساعت_چهار_و_نیم:','id':':jam_setengah_lima:','zh':':四点半:','ru':':полпятого:','tr':':dört_buçuk:','ar':':الرابعة_والنصف:'},'\U0001F340':{'en':':four_leaf_clover:','status':fully_qualified,'E':0.6,'de':':glücksklee:','es':':trébol_de_cuatro_hojas:','fr':':trèfle_à_quatre_feuilles:','ja':':四つ葉のクローバー:','ko':':네잎클로버:','pt':':trevo_de_quatro_folhas:','it':':quadrifoglio:','fa':':شبدر_چهاربرگ:','id':':empat_daun_semanggi:','zh':':四叶草:','ru':':четырехлистный_клевер:','tr':':dört_yapraklı_yonca:','ar':':ورقة_برسيم_رباعية:'},'\U0001F553':{'en':':four_o’clock:','status':fully_qualified,'E':0.6,'alias':[':clock4:', ':four_oclock:'],'variant':True,'de':':4.00_uhr:','es':':4_en_punto:','fr':':quatre_heures:','ja':':4時:','ko':':네_시:','pt':':4_horas:','it':':ore_quattro:','fa':':ساعت_چهار:','id':':jam_empat_tepat:','zh':':四点:','ru':':четыре_часа:','tr':':saat_4:','ar':':الساعة_الرابعة:'},'\U0001F98A':{'en':':fox:','status':fully_qualified,'E':3,'alias':[':fox_face:'],'de':':fuchs:','es':':zorro:','fr':':renard:','ja':':キツネの顔:','ko':':여우_얼굴:','pt':':rosto_de_raposa:','it':':volpe:','fa':':روباه:','id':':rubah:','zh':':狐狸:','ru':':морда_лисицы:','tr':':tilki:','ar':':وجه_ثعلب:'},'\U0001F5BC\U0000FE0F':{'en':':framed_picture:','status':fully_qualified,'E':0.7,'alias':[':frame_with_picture:'],'variant':True,'de':':gerahmtes_bild:','es':':cuadro_enmarcado:','fr':':cadre_avec_image:','ja':':絵画:','ko':':그림_액자:','pt':':quadro_emoldurado:','it':':quadro_incorniciato:','fa':':عکس_در_قاب:','id':':bingkai_bergambar:','zh':':带框的画:','ru':':картина:','tr':':resimli_çerçeve:','ar':':إطار_وصورة:'},'\U0001F5BC':{'en':':framed_picture:','status':unqualified,'E':0.7,'alias':[':frame_with_picture:'],'variant':True,'de':':gerahmtes_bild:','es':':cuadro_enmarcado:','fr':':cadre_avec_image:','ja':':絵画:','ko':':그림_액자:','pt':':quadro_emoldurado:','it':':quadro_incorniciato:','fa':':عکس_در_قاب:','id':':bingkai_bergambar:','zh':':带框的画:','ru':':картина:','tr':':resimli_çerçeve:','ar':':إطار_وصورة:'},'\U0001F35F':{'en':':french_fries:','status':fully_qualified,'E':0.6,'alias':[':fries:'],'de':':pommes_frites:','es':':patatas_fritas:','fr':':frites:','ja':':フライドポテト:','ko':':감자튀김:','pt':':batata_frita:','it':':patatine:','fa':':سیب_زمینی_سرخ_کرده:','id':':kentang_goreng:','zh':':薯条:','ru':':картофель_фри:','tr':':patates_kızartması:','ar':':بطاطس_مقلية:'},'\U0001F364':{'en':':fried_shrimp:','status':fully_qualified,'E':0.6,'de':':frittierte_garnele:','es':':gamba_frita:','fr':':beignet_de_crevette:','ja':':エビフライ:','ko':':새우튀김:','pt':':camarão_frito:','it':':gambero_fritto:','fa':':میگو_سوخاری:','id':':udang_goreng:','zh':':天妇罗:','ru':':жареная_креветка:','tr':':kızarmış_karides:','ar':':جمبري_مقلي:'},'\U0001F438':{'en':':frog:','status':fully_qualified,'E':0.6,'de':':frosch:','es':':rana:','fr':':grenouille:','ja':':カエルの顔:','ko':':개구리_얼굴:','pt':':sapo:','it':':rana:','fa':':قورباغه:','id':':katak:','zh':':青蛙:','ru':':голова_лягушки:','tr':':kurbağa:','ar':':وجه_ضفدع:'},'\U0001F425':{'en':':front-facing_baby_chick:','status':fully_qualified,'E':0.6,'alias':[':hatched_chick:', ':front_facing_baby_chick:'],'de':':küken_von_vorne:','es':':pollito_de_frente:','fr':':poussin_de_face:','ja':':前を向いているひよこ:','ko':':정면을_향해_날개를_편_병아리:','pt':':pintinho_de_frente:','it':':pulcino_visto_di_fronte:','fa':':جوجه_از_روبرو:','id':':itik_bayi_menghadap_ke_depan:','zh':':正面朝向的小鸡:','ru':':цыпленок:','tr':':önden_civciv:','ar':':كتكوت_صغير_بوجه_للأمام:'},'\U00002639\U0000FE0F':{'en':':frowning_face:','status':fully_qualified,'E':0.7,'alias':[':white_frowning_face:'],'variant':True,'de':':düsteres_gesicht:','es':':cara_con_el_ceño_fruncido:','fr':':visage_mécontent:','ja':':困った顔:','ko':':찡그린_얼굴:','pt':':rosto_descontente:','it':':faccina_imbronciata:','fa':':اخمو:','id':':wajah_cemberut:','zh':':不满:','ru':':грустит:','tr':':asık_surat:','ar':':وجه_عابس:'},'\U00002639':{'en':':frowning_face:','status':unqualified,'E':0.7,'alias':[':white_frowning_face:'],'variant':True,'de':':düsteres_gesicht:','es':':cara_con_el_ceño_fruncido:','fr':':visage_mécontent:','ja':':困った顔:','ko':':찡그린_얼굴:','pt':':rosto_descontente:','it':':faccina_imbronciata:','fa':':اخمو:','id':':wajah_cemberut:','zh':':不满:','ru':':грустит:','tr':':asık_surat:','ar':':وجه_عابس:'},'\U0001F626':{'en':':frowning_face_with_open_mouth:','status':fully_qualified,'E':1,'alias':[':frowning:'],'de':':entsetztes_gesicht:','es':':cara_con_el_ceño_fruncido_y_la_boca_abierta:','fr':':visage_mécontent_avec_bouche_ouverte:','ja':':あきれ顔:','ko':':입_벌리고_찌푸린_얼굴:','pt':':rosto_franzido_com_boca_aberta:','it':':faccina_imbronciata_con_bocca_aperta:','fa':':چهره_درهم_کشیده_با_دهان_باز:','id':':wajah_mengerutkan_kening_dengan_mulut_terbuka:','zh':':啊:','ru':':в_изумлении:','tr':':hoşnutsuz_yüz:','ar':':وجه_عابس_بفم_مفتوح:'},'\U000026FD':{'en':':fuel_pump:','status':fully_qualified,'E':0.6,'alias':[':fuelpump:'],'variant':True,'de':':tanksäule:','es':':surtidor_de_gasolina:','fr':':pompe_à_essence:','ja':':ガソリンスタンド:','ko':':주유소:','pt':':posto_de_gasolina:','it':':stazione_di_servizio:','fa':':پمپ_بنزین:','id':':pompa_bahan_bakar:','zh':':油泵:','ru':':заправка:','tr':':akaryakıt_istasyonu:','ar':':مضخة_وقود:'},'\U0001F315':{'en':':full_moon:','status':fully_qualified,'E':0.6,'variant':True,'de':':vollmond:','es':':luna_llena:','fr':':pleine_lune:','ja':':満月:','ko':':보름달:','pt':':lua_cheia:','it':':luna_piena:','fa':':ماه_کامل:','id':':bulan_purnama:','zh':':满月:','ru':':полнолуние:','tr':':dolunay:','ar':':بدر:'},'\U0001F31D':{'en':':full_moon_face:','status':fully_qualified,'E':1,'alias':[':full_moon_with_face:'],'de':':vollmond_mit_gesicht:','es':':luna_llena_con_cara:','fr':':pleine_lune_avec_visage:','ja':':顔のある満月:','ko':':얼굴이_있는_보름달:','pt':':rosto_da_lua_cheia:','it':':faccina_luna_piena:','fa':':صورت_ماه_کامل:','id':':wajah_bulan_purnama:','zh':':微笑的月亮:','ru':':полная_луна_с_лицом:','tr':':dolunay_yüzü:','ar':':وجه_البدر:'},'\U000026B1\U0000FE0F':{'en':':funeral_urn:','status':fully_qualified,'E':1,'variant':True,'de':':urne:','es':':urna_funeraria:','fr':':urne_funéraire:','ja':':骨壺:','ko':':납골_단지:','pt':':urna_funerária:','it':':urna_funeraria:','fa':':خاکستردان:','id':':guci_abu:','zh':':骨灰缸:','ru':':урна_с_прахом:','tr':':kül_kavanozu:','ar':':رماد_موتى:'},'\U000026B1':{'en':':funeral_urn:','status':unqualified,'E':1,'variant':True,'de':':urne:','es':':urna_funeraria:','fr':':urne_funéraire:','ja':':骨壺:','ko':':납골_단지:','pt':':urna_funerária:','it':':urna_funeraria:','fa':':خاکستردان:','id':':guci_abu:','zh':':骨灰缸:','ru':':урна_с_прахом:','tr':':kül_kavanozu:','ar':':رماد_موتى:'},'\U0001F3B2':{'en':':game_die:','status':fully_qualified,'E':0.6,'de':':spielwürfel:','es':':dado:','fr':':dés:','ja':':サイコロ:','ko':':주사위:','pt':':jogo_de_dado:','it':':dado:','fa':':تاس_بازی:','id':':dadu:','zh':':骰子:','ru':':игральная_кость:','tr':':oyun_zarı:','ar':':زهر:'},'\U0001F9C4':{'en':':garlic:','status':fully_qualified,'E':12,'de':':knoblauch:','es':':ajo:','fr':':ail:','ja':':ニンニク:','ko':':마늘:','pt':':alho:','it':':aglio:','fa':':سیر:','id':':bawang_putih:','zh':':蒜:','ru':':чеснок:','tr':':sarımsak:','ar':':ثوم:'},'\U00002699\U0000FE0F':{'en':':gear:','status':fully_qualified,'E':1,'variant':True,'de':':zahnrad:','es':':engranaje:','fr':':roue_dentée:','ja':':歯車:','ko':':톱니바퀴:','pt':':engrenagem:','it':':ingranaggio:','fa':':چرخ_دنده:','id':':roda_gigi:','zh':':齿轮:','ru':':шестеренка:','tr':':dişli:','ar':':ترس:'},'\U00002699':{'en':':gear:','status':unqualified,'E':1,'variant':True,'de':':zahnrad:','es':':engranaje:','fr':':roue_dentée:','ja':':歯車:','ko':':톱니바퀴:','pt':':engrenagem:','it':':ingranaggio:','fa':':چرخ_دنده:','id':':roda_gigi:','zh':':齿轮:','ru':':шестеренка:','tr':':dişli:','ar':':ترس:'},'\U0001F48E':{'en':':gem_stone:','status':fully_qualified,'E':0.6,'alias':[':gem:'],'de':':edelstein:','es':':piedra_preciosa:','fr':':pierre_précieuse:','ja':':宝石:','ko':':원석:','pt':':pedra_preciosa:','it':':gemma:','fa':':جواهر:','id':':batu_permata:','zh':':宝石:','ru':':драгоценный_камень:','tr':':mücevher:','ar':':جوهرة:'},'\U0001F9DE':{'en':':genie:','status':fully_qualified,'E':5,'de':':flaschengeist:','es':':genio:','fr':':génie:','ja':':精霊:','ko':':지니:','pt':':gênio:','it':':genio:','fa':':جن:','id':':jin:','zh':':妖怪:','ru':':джинн:','tr':':cin:','ar':':مارد_سحري:'},'\U0001F47B':{'en':':ghost:','status':fully_qualified,'E':0.6,'de':':gespenst:','es':':fantasma:','fr':':fantôme:','ja':':お化け:','ko':':유령:','pt':':fantasma:','it':':fantasma:','fa':':روح:','id':':hantu:','zh':':鬼:','ru':':привидение:','tr':':hayalet:','ar':':شبح:'},'\U0001FADA':{'en':':ginger_root:','status':fully_qualified,'E':15,'de':':ingwer:','es':':raíz_de_jengibre:','fr':':racine_de_gingembre:','ja':':ショウガ:','ko':':생강_뿌리:','pt':':gengibre:','it':':radice_di_zenzero:','fa':':زنجبیل:','id':':akar_jahe:','zh':':姜:','ru':':корень_имбиря:','tr':':kök_zencefil:','ar':':جذر_الزنجبيل:'},'\U0001F992':{'en':':giraffe:','status':fully_qualified,'E':5,'de':':giraffe:','es':':jirafa:','fr':':girafe:','ja':':キリン:','ko':':기린:','pt':':girafa:','it':':giraffa:','fa':':زرافه:','id':':jerapah:','zh':':长颈鹿:','ru':':жираф:','tr':':zürafa:','ar':':زرافة:'},'\U0001F467':{'en':':girl:','status':fully_qualified,'E':0.6,'de':':mädchen:','es':':niña:','fr':':fille:','ja':':女の子:','ko':':여자_아이:','pt':':menina:','it':':bambina:','fa':':دختر:','id':':anak_perempuan:','zh':':女孩:','ru':':девочка:','tr':':kız_çocuk:','ar':':فتاة:'},'\U0001F467\U0001F3FF':{'en':':girl_dark_skin_tone:','status':fully_qualified,'E':1,'de':':mädchen_dunkle_hautfarbe:','es':':niña_tono_de_piel_oscuro:','fr':':fille_peau_foncée:','ja':':女の子_濃い肌色:','ko':':여자_아이_검은색_피부:','pt':':menina_pele_escura:','it':':bambina_carnagione_scura:','fa':':دختر_پوست_آبنوسی:','id':':anak_perempuan_warna_kulit_gelap:','zh':':女孩_较深肤色:','ru':':девочка_очень_темный_тон_кожи:','tr':':kız_çocuk_koyu_cilt_tonu:','ar':':فتاة_بشرة_بلون_غامق:'},'\U0001F467\U0001F3FB':{'en':':girl_light_skin_tone:','status':fully_qualified,'E':1,'de':':mädchen_helle_hautfarbe:','es':':niña_tono_de_piel_claro:','fr':':fille_peau_claire:','ja':':女の子_薄い肌色:','ko':':여자_아이_하얀_피부:','pt':':menina_pele_clara:','it':':bambina_carnagione_chiara:','fa':':دختر_پوست_سفید:','id':':anak_perempuan_warna_kulit_cerah:','zh':':女孩_较浅肤色:','ru':':девочка_очень_светлый_тон_кожи:','tr':':kız_çocuk_açık_cilt_tonu:','ar':':فتاة_بشرة_بلون_فاتح:'},'\U0001F467\U0001F3FE':{'en':':girl_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':mädchen_mitteldunkle_hautfarbe:','es':':niña_tono_de_piel_oscuro_medio:','fr':':fille_peau_mate:','ja':':女の子_やや濃い肌色:','ko':':여자_아이_진한_갈색_피부:','pt':':menina_pele_morena_escura:','it':':bambina_carnagione_abbastanza_scura:','fa':':دختر_پوست_گندمی:','id':':anak_perempuan_warna_kulit_gelap-sedang:','zh':':女孩_中等深肤色:','ru':':девочка_темный_тон_кожи:','tr':':kız_çocuk_orta_koyu_cilt_tonu:','ar':':فتاة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F467\U0001F3FC':{'en':':girl_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':mädchen_mittelhelle_hautfarbe:','es':':niña_tono_de_piel_claro_medio:','fr':':fille_peau_moyennement_claire:','ja':':女の子_やや薄い肌色:','ko':':여자_아이_연한_갈색_피부:','pt':':menina_pele_morena_clara:','it':':bambina_carnagione_abbastanza_chiara:','fa':':دختر_پوست_روشن:','id':':anak_perempuan_warna_kulit_cerah-sedang:','zh':':女孩_中等浅肤色:','ru':':девочка_светлый_тон_кожи:','tr':':kız_çocuk_orta_açık_cilt_tonu:','ar':':فتاة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F467\U0001F3FD':{'en':':girl_medium_skin_tone:','status':fully_qualified,'E':1,'de':':mädchen_mittlere_hautfarbe:','es':':niña_tono_de_piel_medio:','fr':':fille_peau_légèrement_mate:','ja':':女の子_中間の肌色:','ko':':여자_아이_갈색_피부:','pt':':menina_pele_morena:','it':':bambina_carnagione_olivastra:','fa':':دختر_پوست_طلایی:','id':':anak_perempuan_warna_kulit_sedang:','zh':':女孩_中等肤色:','ru':':девочка_средний_тон_кожи:','tr':':kız_çocuk_orta_cilt_tonu:','ar':':فتاة_بشرة_بلون_معتدل:'},'\U0001F95B':{'en':':glass_of_milk:','status':fully_qualified,'E':3,'alias':[':milk_glass:'],'de':':glas_milch:','es':':vaso_de_leche:','fr':':verre_de_lait:','ja':':牛乳入りのコップ:','ko':':우유_한잔:','pt':':copo_de_leite:','it':':bicchiere_di_latte:','fa':':لیوان_شیر:','id':':gelas_susu:','zh':':一杯奶:','ru':':стакан_молока:','tr':':süt_bardağı:','ar':':كوب_حليب:'},'\U0001F453':{'en':':glasses:','status':fully_qualified,'E':0.6,'alias':[':eyeglasses:'],'variant':True,'de':':brille:','es':':gafas:','fr':':lunettes_de_vue:','ja':':メガネ:','ko':':안경:','pt':':óculos:','it':':occhiali_da_vista:','fa':':عینک_طبی:','id':':kacamata:','zh':':眼镜:','ru':':очки:','tr':':gözlük:','ar':':نظارة:'},'\U0001F30E':{'en':':globe_showing_Americas:','status':fully_qualified,'E':0.7,'alias':[':earth_americas:', ':globe_showing_americas:'],'variant':True,'de':':globus_mit_amerika:','es':':globo_terráqueo_mostrando_américa:','fr':':globe_tourné_sur_les_amériques:','ja':':地球(アメリカ大陸):','ko':':미대륙이_보이는_지구:','pt':':globo_mostrando_as_américas:','it':':america:','fa':':نقشهٔ_قارهٔ_امریکا_روی_کره:','id':':bola_dunia_amerika:','zh':':地球上的美洲:','ru':':Америка_на_земном_шаре:','tr':':yerkürede_amerika:','ar':':كرة_أرضية_تعرض_الأمريكتين:'},'\U0001F30F':{'en':':globe_showing_Asia-Australia:','status':fully_qualified,'E':0.6,'alias':[':earth_asia:', ':globe_showing_asia_australia:'],'variant':True,'de':':globus_mit_asien_und_australien:','es':':globo_terráqueo_mostrando_asia_y_australia:','fr':':globe_tourné_sur_l’asie_et_l’australie:','ja':':地球(アジアとオーストラリア):','ko':':아시아와_호주가_보이는_지구:','pt':':globo_mostrando_ásia_e_oceania:','it':':asia_e_australia:','fa':':نقشهٔ_آسیا-اقیانوسیه_روی_کره:','id':':bola_dunia_asia-australia:','zh':':地球上的亚洲澳洲:','ru':':Азия_и_Австралия_на_земном_шаре:','tr':':yerkürede_asya-avustralya:','ar':':كرة_أرضية_تعرض_آسيا_وأستراليا:'},'\U0001F30D':{'en':':globe_showing_Europe-Africa:','status':fully_qualified,'E':0.7,'alias':[':earth_africa:', ':globe_showing_europe_africa:'],'variant':True,'de':':globus_mit_europa_und_afrika:','es':':globo_terráqueo_mostrando_europa_y_áfrica:','fr':':globe_tourné_sur_l’afrique_et_l’europe:','ja':':地球(ヨーロッパとアフリカ):','ko':':유럽과_아프리카가_보이는_지구:','pt':':globo_mostrando_europa_e_áfrica:','it':':europa_e_africa:','fa':':نقشهٔ_اروپا-افریقا_روی_کره:','id':':bola_dunia_eropa-afrika:','zh':':地球上的欧洲非洲:','ru':':Европа_и_Африка_на_земном_шаре:','tr':':yerkürede_avrupa-afrika:','ar':':كرة_أرضية_تعرض_أوروبا_وإفريقيا:'},'\U0001F310':{'en':':globe_with_meridians:','status':fully_qualified,'E':1,'de':':globus_mit_meridianen:','es':':globo_terráqueo_con_meridianos:','fr':':globe_avec_méridiens:','ja':':子午線のある地球:','ko':':자오선_지구:','pt':':globo_com_meridianos:','it':':globo_con_meridiani:','fa':':کره_با_نصف_النهارها:','id':':bola_dunia_dengan_garis_meridian:','zh':':带经纬线的地球:','ru':':меридианы:','tr':':meridyenli_yerküre:','ar':':كرة_أرضية_بخطوط_طول:'},'\U0001F9E4':{'en':':gloves:','status':fully_qualified,'E':5,'de':':handschuhe:','es':':guantes:','fr':':gants:','ja':':手袋:','ko':':장갑:','pt':':luvas:','it':':guanti:','fa':':دستکش:','id':':sarung_tangan:','zh':':手套:','ru':':перчатки:','tr':':eldiven:','ar':':قفازان:'},'\U0001F31F':{'en':':glowing_star:','status':fully_qualified,'E':0.6,'alias':[':star2:'],'de':':funkelnder_stern:','es':':estrella_brillante:','fr':':étoile_brillante:','ja':':きらきら星:','ko':':반짝이는_별:','pt':':estrela_brilhante:','it':':stella_che_brilla:','fa':':ستاره_درخشان:','id':':bintang_bersinar:','zh':':闪亮的星星:','ru':':сияющая_звезда:','tr':':parıldayan_yıldız:','ar':':نجم_ساطع:'},'\U0001F945':{'en':':goal_net:','status':fully_qualified,'E':3,'de':':tor:','es':':portería:','fr':':cage:','ja':':ゴールネット:','ko':':골대:','pt':':gol:','it':':porta_da_calcio:','fa':':دروازه:','id':':gawang:','zh':':球门:','ru':':спортивные_ворота:','tr':':kale_ağı:','ar':':شبكة_المرمى:'},'\U0001F410':{'en':':goat:','status':fully_qualified,'E':1,'de':':ziege:','es':':cabra:','fr':':chèvre:','ja':':山羊:','ko':':염소:','pt':':cabra:','it':':capra:','fa':':بز:','id':':kambing:','zh':':山羊:','ru':':коза:','tr':':keçi:','ar':':معزة:'},'\U0001F47A':{'en':':goblin:','status':fully_qualified,'E':0.6,'alias':[':japanese_goblin:'],'de':':kobold:','es':':demonio_japonés_tengu:','fr':':monstre_japonais:','ja':':天狗:','ko':':코주부_도깨비_가면:','pt':':duende_japonês:','it':':goblin:','fa':':جن_کوتوله:','id':':topeng_hidung_panjang:','zh':':小妖精:','ru':':японский_гоблин:','tr':':goblin:','ar':':عفريت:'},'\U0001F97D':{'en':':goggles:','status':fully_qualified,'E':11,'de':':schutzbrille:','es':':gafas_de_protección:','fr':':lunettes:','ja':':ゴーグル:','ko':':고글:','pt':':óculos_de_proteção:','it':':maschera:','fa':':عینک_ایمنی:','id':':kacamata_renang:','zh':':护目镜:','ru':':защитные_очки:','tr':':koruyucu_gözlük:','ar':':نظارة_واقية:'},'\U0001FABF':{'en':':goose:','status':fully_qualified,'E':15,'de':':gans:','es':':oca:','fr':':oie:','ja':':ガチョウ:','ko':':거위:','pt':':ganso:','it':':oca:','fa':':غاز:','id':':angsa_leher_pendek:','zh':':鹅:','ru':':гусь:','tr':':kaz:','ar':':إوزة:'},'\U0001F98D':{'en':':gorilla:','status':fully_qualified,'E':3,'alias':[':harambe:'],'de':':gorilla:','es':':gorila:','fr':':gorille:','ja':':ゴリラ:','ko':':고릴라:','pt':':gorila:','it':':gorilla:','fa':':گوریل:','id':':gorila:','zh':':大猩猩:','ru':':горилла:','tr':':goril:','ar':':غوريلا:'},'\U0001F393':{'en':':graduation_cap:','status':fully_qualified,'E':0.6,'alias':[':mortar_board:'],'variant':True,'de':':doktorhut:','es':':birrete:','fr':':toque_universitaire:','ja':':角帽:','ko':':졸업_모자:','pt':':chapéu_de_formatura:','it':':cappello_da_laureato:','fa':':کلاه_فارغ_التحصیلی:','id':':topi_wisuda:','zh':':毕业帽:','ru':':шапочка_выпускника:','tr':':mezuniyet_kepi:','ar':':قبعة_تخرج:'},'\U0001F347':{'en':':grapes:','status':fully_qualified,'E':0.6,'de':':trauben:','es':':uvas:','fr':':raisin:','ja':':ぶどう:','ko':':포도:','pt':':uvas:','it':':uva:','fa':':انگور:','id':':anggur:','zh':':葡萄:','ru':':виноград:','tr':':üzüm:','ar':':عنب:'},'\U0001F34F':{'en':':green_apple:','status':fully_qualified,'E':0.6,'de':':grüner_apfel:','es':':manzana_verde:','fr':':pomme_verte:','ja':':青リンゴ:','ko':':초록_사과:','pt':':maçã_verde:','it':':mela_verde:','fa':':سیب_سبز:','id':':apel_hijau:','zh':':青苹果:','ru':':зеленое_яблоко:','tr':':yeşil_elma:','ar':':تفاح_أخضر:'},'\U0001F4D7':{'en':':green_book:','status':fully_qualified,'E':0.6,'de':':grünes_buch:','es':':libro_verde:','fr':':livre_vert:','ja':':緑の本:','ko':':초록색_책:','pt':':livro_verde:','it':':libro_verde:','fa':':کتاب_سبز:','id':':buku_hijau:','zh':':绿色书本:','ru':':зеленая_книга:','tr':':yeşil_kitap:','ar':':كتاب_أخضر:'},'\U0001F7E2':{'en':':green_circle:','status':fully_qualified,'E':12,'de':':grüner_punkt:','es':':círculo_verde:','fr':':disque_vert:','ja':':緑の丸:','ko':':녹색_원:','pt':':círculo_verde:','it':':cerchio_verde:','fa':':دایره_سبز:','id':':lingkaran_hijau:','zh':':绿色圆:','ru':':зеленый_круг:','tr':':yeşil_daire:','ar':':دائرة_خضراء:'},'\U0001F49A':{'en':':green_heart:','status':fully_qualified,'E':0.6,'de':':grünes_herz:','es':':corazón_verde:','fr':':cœur_vert:','ja':':緑のハート:','ko':':초록색_하트:','pt':':coração_verde:','it':':cuore_verde:','fa':':قلب_سبز:','id':':hati_hijau:','zh':':绿心:','ru':':зеленое_сердце:','tr':':yeşil_kalp:','ar':':قلب_أخضر:'},'\U0001F957':{'en':':green_salad:','status':fully_qualified,'E':3,'de':':salat:','es':':ensalada:','fr':':salade_verte:','ja':':グリーンサラダ:','ko':':야채샐러드:','pt':':salada_verde:','it':':insalata_verde:','fa':':سالاد_سبزیجات:','id':':salad_hijau:','zh':':绿色沙拉:','ru':':овощной_салат:','tr':':yeşil_salata:','ar':':سلطة_خضراء:'},'\U0001F7E9':{'en':':green_square:','status':fully_qualified,'E':12,'de':':grünes_quadrat:','es':':cuadrado_verde:','fr':':carré_vert:','ja':':緑の四角:','ko':':녹색_사각형:','pt':':quadrado_verde:','it':':quadrato_verde:','fa':':مربع_سبز:','id':':persegi_hijau:','zh':':绿色方块:','ru':':зеленый_квадрат:','tr':':yeşil_kare:','ar':':مربع_أخضر:'},'\U0001FA76':{'en':':grey_heart:','status':fully_qualified,'E':15,'de':':graues_herz:','es':':corazón_gris:','fr':':cœur_gris:','ja':':灰色のハート:','ko':':회색_하트:','pt':':coração_cinza:','it':':cuore_grigio:','fa':':قلب_خاکستری:','id':':hati_abu-abu:','zh':':灰心:','ru':':серое_сердце:','tr':':gri_kalp:','ar':':قلب_رمادي:'},'\U0001F62C':{'en':':grimacing_face:','status':fully_qualified,'E':1,'alias':[':grimacing:'],'de':':grimassen_schneidendes_gesicht:','es':':cara_haciendo_una_mueca:','fr':':visage_grimaçant:','ja':':しかめ面:','ko':':찌푸린_얼굴:','pt':':rosto_expressando_desagrado:','it':':faccina_con_smorfia:','fa':':دهن_کجی:','id':':wajah_menyeringai:','zh':':龇牙咧嘴:','ru':':гримаса:','tr':':dişlerini_gösteren_yüz:','ar':':وجه_متجهم:'},'\U0001F63A':{'en':':grinning_cat:','status':fully_qualified,'E':0.6,'alias':[':smiley_cat:'],'de':':grinsende_katze:','es':':gato_sonriendo:','fr':':chat_qui_sourit:','ja':':にっこり笑う猫:','ko':':활짝_웃는_고양이_얼굴:','pt':':rosto_de_gato_sorrindo:','it':':gatto_che_sorride:','fa':':گربهٔ_خندان:','id':':wajah_kucing_tersenyum_lebar:','zh':':大笑的猫:','ru':':улыбающийся_кот:','tr':':sırıtan_kedi:','ar':':وجه_قطة_مبتسم_بفم_مفتوح:'},'\U0001F638':{'en':':grinning_cat_with_smiling_eyes:','status':fully_qualified,'E':0.6,'alias':[':smile_cat:'],'de':':grinsende_katze_mit_lachenden_augen:','es':':gato_sonriendo_con_ojos_sonrientes:','fr':':chat_qui_sourit_avec_des_yeux_rieurs:','ja':':にやっと笑う猫:','ko':':미소_짓는_눈으로_활짝_웃는_고양이_얼굴:','pt':':rosto_de_gato_sorrindo_com_olhos_sorridentes:','it':':gatto_che_sogghigna:','fa':':گربه_با_چشم_های_خندان:','id':':wajah_kucing_gembira_dengan_mata_bahagia:','zh':':微笑的猫:','ru':':смеющийся_кот:','tr':':gülen_gözlerle_sırıtan_kedi:','ar':':وجه_قطة_مبتسم_بعينين_باسمتين:'},'\U0001F600':{'en':':grinning_face:','status':fully_qualified,'E':1,'alias':[':grinning:'],'de':':grinsendes_gesicht:','es':':cara_sonriendo:','fr':':visage_rieur:','ja':':にっこり笑う:','ko':':활짝_웃는_얼굴:','pt':':rosto_risonho:','it':':faccina_con_un_gran_sorriso:','fa':':خنده:','id':':wajah_gembira:','zh':':嘿嘿:','ru':':широко_улыбается:','tr':':sırıtan_yüz:','ar':':وجه_بابتسامة_عريضة:'},'\U0001F603':{'en':':grinning_face_with_big_eyes:','status':fully_qualified,'E':0.6,'alias':[':smiley:'],'de':':grinsendes_gesicht_mit_großen_augen:','es':':cara_sonriendo_con_ojos_grandes:','fr':':visage_souriant_avec_de_grands_yeux:','ja':':わーい:','ko':':눈을_크게_뜨고_웃는_얼굴:','pt':':rosto_risonho_com_olhos_bem_abertos:','it':':faccina_con_un_gran_sorriso_e_occhi_spalancati:','fa':':صورت_خندان:','id':':wajah_tersenyum_lebar:','zh':':哈哈:','ru':':смеется:','tr':':ağzı_açık_sırıtma:','ar':':وجه_مبتسم_بعينين_كبيرتين:'},'\U0001F604':{'en':':grinning_face_with_smiling_eyes:','status':fully_qualified,'E':0.6,'alias':[':smile:'],'de':':grinsendes_gesicht_mit_lachenden_augen:','es':':cara_sonriendo_con_ojos_sonrientes:','fr':':visage_très_souriant_aux_yeux_rieurs:','ja':':笑顔:','ko':':미소_짓는_눈으로_활짝_웃는_얼굴:','pt':':rosto_risonho_com_olhos_sorridentes:','it':':faccina_con_sorriso_e_occhi_sorridenti:','fa':':خندان:','id':':wajah_tersenyum_lebar_dan_mata_tertutup:','zh':':大笑:','ru':':смеется_с_закрытыми_глазами:','tr':':gülen_gözlerle_ağzı_açık_sırıtma:','ar':':وجه_مبتسم_بعينين_باسمتين:'},'\U0001F605':{'en':':grinning_face_with_sweat:','status':fully_qualified,'E':0.6,'alias':[':sweat_smile:'],'de':':grinsendes_gesicht_mit_schweißtropfen:','es':':cara_sonriendo_con_sudor_frío:','fr':':visage_souriant_avec_une_goutte_de_sueur:','ja':':冷や汗笑顔:','ko':':땀_흘리며_웃는_얼굴:','pt':':rosto_risonho_com_gota_de_suor:','it':':faccina_con_un_gran_sorriso_e_goccia_di_sudore:','fa':':صورت_خندان_و_عرق_کرده:','id':':wajah_tersenyum_lebar_dan_berkeringat:','zh':':苦笑:','ru':':нервно_смеется:','tr':':soğuk_terli_ağzı_açık_gülümseme:','ar':':وجه_مبتسم_بفم_مفتوح_وعرق_بارد:'},'\U0001F606':{'en':':grinning_squinting_face:','status':fully_qualified,'E':0.6,'alias':[':satisfied:', ':laughing:'],'de':':grinsegesicht_mit_zugekniffenen_augen:','es':':cara_sonriendo_con_los_ojos_cerrados:','fr':':visage_souriant_avec_yeux_plissés:','ja':':きゃー:','ko':':눈웃음짓는_얼굴:','pt':':rosto_risonho_com_olhos_semicerrados:','it':':sorriso_a_bocca_aperta_con_occhi_chiusi:','fa':':قاه_قاه:','id':':wajah_tersenyum_lebar_dan_memejamkan_mata:','zh':':斜眼笑:','ru':':смеется_с_прищуренными_глазами:','tr':':kapalı_gözlerle_ağzı_açık_sırıtma:','ar':':وجه_مبتسم_بعينين_مغلقتين:'},'\U0001F497':{'en':':growing_heart:','status':fully_qualified,'E':0.6,'alias':[':heartpulse:'],'de':':wachsendes_herz:','es':':corazón_creciente:','fr':':cœur_grandissant:','ja':':大きくなるハート:','ko':':커지는_하트:','pt':':coração_crescendo:','it':':cuore_che_cresce:','fa':':قلب_هیجان_زده:','id':':hati_membesar:','zh':':搏动的心:','ru':':растущее_сердце:','tr':':büyüyen_kalp:','ar':':قلب_ينمو:'},'\U0001F482':{'en':':guard:','status':fully_qualified,'E':0.6,'de':':wache:','es':':guardia:','fr':':garde:','ja':':衛兵:','ko':':근위병:','pt':':guarda:','it':':guardia:','fa':':گارد_نظامی:','id':':penjaga:','zh':':卫兵:','ru':':гвардеец:','tr':':muhafız:','ar':':حارس:'},'\U0001F482\U0001F3FF':{'en':':guard_dark_skin_tone:','status':fully_qualified,'E':1,'de':':wache_dunkle_hautfarbe:','es':':guardia_tono_de_piel_oscuro:','fr':':garde_peau_foncée:','ja':':衛兵_濃い肌色:','ko':':근위병_검은색_피부:','pt':':guarda_pele_escura:','it':':guardia_carnagione_scura:','fa':':گارد_نظامی_پوست_آبنوسی:','id':':penjaga_warna_kulit_gelap:','zh':':卫兵_较深肤色:','ru':':гвардеец_очень_темный_тон_кожи:','tr':':muhafız_koyu_cilt_tonu:','ar':':حارس_بشرة_بلون_غامق:'},'\U0001F482\U0001F3FB':{'en':':guard_light_skin_tone:','status':fully_qualified,'E':1,'de':':wache_helle_hautfarbe:','es':':guardia_tono_de_piel_claro:','fr':':garde_peau_claire:','ja':':衛兵_薄い肌色:','ko':':근위병_하얀_피부:','pt':':guarda_pele_clara:','it':':guardia_carnagione_chiara:','fa':':گارد_نظامی_پوست_سفید:','id':':penjaga_warna_kulit_cerah:','zh':':卫兵_较浅肤色:','ru':':гвардеец_очень_светлый_тон_кожи:','tr':':muhafız_açık_cilt_tonu:','ar':':حارس_بشرة_بلون_فاتح:'},'\U0001F482\U0001F3FE':{'en':':guard_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':wache_mitteldunkle_hautfarbe:','es':':guardia_tono_de_piel_oscuro_medio:','fr':':garde_peau_mate:','ja':':衛兵_やや濃い肌色:','ko':':근위병_진한_갈색_피부:','pt':':guarda_pele_morena_escura:','it':':guardia_carnagione_abbastanza_scura:','fa':':گارد_نظامی_پوست_گندمی:','id':':penjaga_warna_kulit_gelap-sedang:','zh':':卫兵_中等深肤色:','ru':':гвардеец_темный_тон_кожи:','tr':':muhafız_orta_koyu_cilt_tonu:','ar':':حارس_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F482\U0001F3FC':{'en':':guard_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':wache_mittelhelle_hautfarbe:','es':':guardia_tono_de_piel_claro_medio:','fr':':garde_peau_moyennement_claire:','ja':':衛兵_やや薄い肌色:','ko':':근위병_연한_갈색_피부:','pt':':guarda_pele_morena_clara:','it':':guardia_carnagione_abbastanza_chiara:','fa':':گارد_نظامی_پوست_روشن:','id':':penjaga_warna_kulit_cerah-sedang:','zh':':卫兵_中等浅肤色:','ru':':гвардеец_светлый_тон_кожи:','tr':':muhafız_orta_açık_cilt_tonu:','ar':':حارس_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F482\U0001F3FD':{'en':':guard_medium_skin_tone:','status':fully_qualified,'E':1,'de':':wache_mittlere_hautfarbe:','es':':guardia_tono_de_piel_medio:','fr':':garde_peau_légèrement_mate:','ja':':衛兵_中間の肌色:','ko':':근위병_갈색_피부:','pt':':guarda_pele_morena:','it':':guardia_carnagione_olivastra:','fa':':گارد_نظامی_پوست_طلایی:','id':':penjaga_warna_kulit_sedang:','zh':':卫兵_中等肤色:','ru':':гвардеец_средний_тон_кожи:','tr':':muhafız_orta_cilt_tonu:','ar':':حارس_بشرة_بلون_معتدل:'},'\U0001F9AE':{'en':':guide_dog:','status':fully_qualified,'E':12,'de':':blindenhund:','es':':perro_guía:','fr':':chien_guide:','ja':':盲導犬:','ko':':안내견:','pt':':cão-guia:','it':':cane_guida:','fa':':سگ_راهنما:','id':':anjing_pemandu:','zh':':导盲犬:','ru':':собака_поводырь:','tr':':rehber_köpek:','ar':':كلب_دليل:'},'\U0001F3B8':{'en':':guitar:','status':fully_qualified,'E':0.6,'de':':gitarre:','es':':guitarra:','fr':':guitare:','ja':':ギター:','ko':':기타:','pt':':guitarra:','it':':chitarra:','fa':':گیتار:','id':':gitar:','zh':':吉他:','ru':':гитара:','tr':':gitar:','ar':':غيتار:'},'\U0001FAAE':{'en':':hair_pick:','status':fully_qualified,'E':15,'de':':haarkamm:','es':':peineta:','fr':':peigne_afro:','ja':':アフロコーム:','ko':':머리_빗:','pt':':pente_de_cabelo:','it':':pettine_afro:','fa':':شانه_جداکننده:','id':':sisir_garpu:','zh':':发夹:','ru':':афро_гребень:','tr':':perma_tarağı:','ar':':مشط_عريض:'},'\U0001F354':{'en':':hamburger:','status':fully_qualified,'E':0.6,'de':':hamburger:','es':':hamburguesa:','fr':':hamburger:','ja':':ハンバーガー:','ko':':햄버거:','pt':':hambúrguer:','it':':hamburger:','fa':':همبرگر:','id':':hamburger:','zh':':汉堡:','ru':':гамбургер:','tr':':hamburger:','ar':':همبرغر:'},'\U0001F528':{'en':':hammer:','status':fully_qualified,'E':0.6,'de':':hammer:','es':':martillo:','fr':':marteau:','ja':':ハンマー:','ko':':망치:','pt':':martelo:','it':':martello:','fa':':چکش:','id':':palu:','zh':':锤子:','ru':':молоток:','tr':':çekiç:','ar':':مطرقة:'},'\U00002692\U0000FE0F':{'en':':hammer_and_pick:','status':fully_qualified,'E':1,'variant':True,'de':':hammer_und_pickel:','es':':martillo_y_pico:','fr':':marteau_et_pioche:','ja':':ハンマーとつるはし:','ko':':망치와_곡괭이:','pt':':martelo_e_picareta:','it':':piccone_e_martello:','fa':':چکش_و_کلنگ:','id':':palu_dan_beliung:','zh':':锤子与镐:','ru':':кирка_и_молот:','tr':':çekiç_ve_kazma:','ar':':مطرقة_ومثقاب:'},'\U00002692':{'en':':hammer_and_pick:','status':unqualified,'E':1,'variant':True,'de':':hammer_und_pickel:','es':':martillo_y_pico:','fr':':marteau_et_pioche:','ja':':ハンマーとつるはし:','ko':':망치와_곡괭이:','pt':':martelo_e_picareta:','it':':piccone_e_martello:','fa':':چکش_و_کلنگ:','id':':palu_dan_beliung:','zh':':锤子与镐:','ru':':кирка_и_молот:','tr':':çekiç_ve_kazma:','ar':':مطرقة_ومثقاب:'},'\U0001F6E0\U0000FE0F':{'en':':hammer_and_wrench:','status':fully_qualified,'E':0.7,'variant':True,'de':':hammer_und_schraubenschlüssel:','es':':martillo_y_llave_inglesa:','fr':':marteau_et_clé_à_molette:','ja':':ハンマーとレンチ:','ko':':망치와_렌치:','pt':':martelo_e_chave-inglesa:','it':':martello_e_chiave_inglese:','fa':':چکش_و_آچار:','id':':palu_dan_kunci_inggris:','zh':':锤子与扳手:','ru':':молот_и_гаечныи_ключ:','tr':':çekiç_ve_somun_anahtarı:','ar':':شاكوش_ومفتاح:'},'\U0001F6E0':{'en':':hammer_and_wrench:','status':unqualified,'E':0.7,'variant':True,'de':':hammer_und_schraubenschlüssel:','es':':martillo_y_llave_inglesa:','fr':':marteau_et_clé_à_molette:','ja':':ハンマーとレンチ:','ko':':망치와_렌치:','pt':':martelo_e_chave-inglesa:','it':':martello_e_chiave_inglese:','fa':':چکش_و_آچار:','id':':palu_dan_kunci_inggris:','zh':':锤子与扳手:','ru':':молот_и_гаечный_ключ:','tr':':çekiç_ve_somun_anahtarı:','ar':':شاكوش_ومفتاح:'},'\U0001FAAC':{'en':':hamsa:','status':fully_qualified,'E':14,'de':':hamsa:','es':':hamsa:','fr':':main_de_fatma:','ja':':ハムサ:','ko':':함사:','pt':':hamsá:','it':':mano_di_fatima:','fa':':خمسه:','id':':hamsa:','zh':':法蒂玛之手:','ru':':хамса:','tr':':fatıma’nın_eli:','ar':':خمسة:'},'\U0001F439':{'en':':hamster:','status':fully_qualified,'E':0.6,'de':':hamster:','es':':hámster:','fr':':hamster:','ja':':ハムスターの顔:','ko':':햄스터_얼굴:','pt':':rosto_de_hamster:','it':':criceto:','fa':':همستر:','id':':hamster:','zh':':仓鼠:','ru':':морда_хомяка:','tr':':hamster:','ar':':وجه_حيوان_الهمستر:'},'\U0001F590\U0000FE0F':{'en':':hand_with_fingers_splayed:','status':fully_qualified,'E':0.7,'alias':[':raised_hand_with_fingers_splayed:'],'variant':True,'de':':hand_mit_gespreizten_fingern:','es':':mano_abierta:','fr':':main_levée_doigts_écartés:','ja':':開いた手:','ko':':쫙_편_손바닥:','pt':':mão_aberta_com_os_dedos_separados:','it':':mano_aperta:','fa':':دست_با_انگشتان_باز:','id':':telapak_tangan_terbuka:','zh':':手掌:','ru':':раскрытая_ладонь:','tr':':parmaklar_açık_el_kaldırma:','ar':':يد_مرفوعة_بأصابع_متفرقة:'},'\U0001F590':{'en':':hand_with_fingers_splayed:','status':unqualified,'E':0.7,'alias':[':raised_hand_with_fingers_splayed:'],'variant':True,'de':':hand_mit_gespreizten_fingern:','es':':mano_abierta:','fr':':main_levée_doigts_écartés:','ja':':開いた手:','ko':':쫙_편_손바닥:','pt':':mão_aberta_com_os_dedos_separados:','it':':mano_aperta:','fa':':دست_با_انگشتان_باز:','id':':telapak_tangan_terbuka:','zh':':手掌:','ru':':раскрытая_ладонь:','tr':':parmaklar_açık_el_kaldırma:','ar':':يد_مرفوعة_بأصابع_متفرقة:'},'\U0001F590\U0001F3FF':{'en':':hand_with_fingers_splayed_dark_skin_tone:','status':fully_qualified,'E':1,'de':':hand_mit_gespreizten_fingern_dunkle_hautfarbe:','es':':mano_abierta_tono_de_piel_oscuro:','fr':':main_levée_doigts_écartés_peau_foncée:','ja':':開いた手_濃い肌色:','ko':':쫙_편_손바닥_검은색_피부:','pt':':mão_aberta_com_os_dedos_separados_pele_escura:','it':':mano_aperta_carnagione_scura:','fa':':دست_با_انگشتان_باز_پوست_آبنوسی:','id':':telapak_tangan_terbuka_warna_kulit_gelap:','zh':':手掌_较深肤色:','ru':':раскрытая_ладонь_очень_темный_тон_кожи:','tr':':parmaklar_açık_el_kaldırma_koyu_cilt_tonu:','ar':':يد_مرفوعة_بأصابع_متفرقة_بشرة_بلون_غامق:'},'\U0001F590\U0001F3FB':{'en':':hand_with_fingers_splayed_light_skin_tone:','status':fully_qualified,'E':1,'de':':hand_mit_gespreizten_fingern_helle_hautfarbe:','es':':mano_abierta_tono_de_piel_claro:','fr':':main_levée_doigts_écartés_peau_claire:','ja':':開いた手_薄い肌色:','ko':':쫙_편_손바닥_하얀_피부:','pt':':mão_aberta_com_os_dedos_separados_pele_clara:','it':':mano_aperta_carnagione_chiara:','fa':':دست_با_انگشتان_باز_پوست_سفید:','id':':telapak_tangan_terbuka_warna_kulit_cerah:','zh':':手掌_较浅肤色:','ru':':раскрытая_ладонь_очень_светлый_тон_кожи:','tr':':parmaklar_açık_el_kaldırma_açık_cilt_tonu:','ar':':يد_مرفوعة_بأصابع_متفرقة_بشرة_بلون_فاتح:'},'\U0001F590\U0001F3FE':{'en':':hand_with_fingers_splayed_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':hand_mit_gespreizten_fingern_mitteldunkle_hautfarbe:','es':':mano_abierta_tono_de_piel_oscuro_medio:','fr':':main_levée_doigts_écartés_peau_mate:','ja':':開いた手_やや濃い肌色:','ko':':쫙_편_손바닥_진한_갈색_피부:','pt':':mão_aberta_com_os_dedos_separados_pele_morena_escura:','it':':mano_aperta_carnagione_abbastanza_scura:','fa':':دست_با_انگشتان_باز_پوست_گندمی:','id':':telapak_tangan_terbuka_warna_kulit_gelap-sedang:','zh':':手掌_中等深肤色:','ru':':раскрытая_ладонь_темный_тон_кожи:','tr':':parmaklar_açık_el_kaldırma_orta_koyu_cilt_tonu:','ar':':يد_مرفوعة_بأصابع_متفرقة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F590\U0001F3FC':{'en':':hand_with_fingers_splayed_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':hand_mit_gespreizten_fingern_mittelhelle_hautfarbe:','es':':mano_abierta_tono_de_piel_claro_medio:','fr':':main_levée_doigts_écartés_peau_moyennement_claire:','ja':':開いた手_やや薄い肌色:','ko':':쫙_편_손바닥_연한_갈색_피부:','pt':':mão_aberta_com_os_dedos_separados_pele_morena_clara:','it':':mano_aperta_carnagione_abbastanza_chiara:','fa':':دست_با_انگشتان_باز_پوست_روشن:','id':':telapak_tangan_terbuka_warna_kulit_cerah-sedang:','zh':':手掌_中等浅肤色:','ru':':раскрытая_ладонь_светлый_тон_кожи:','tr':':parmaklar_açık_el_kaldırma_orta_açık_cilt_tonu:','ar':':يد_مرفوعة_بأصابع_متفرقة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F590\U0001F3FD':{'en':':hand_with_fingers_splayed_medium_skin_tone:','status':fully_qualified,'E':1,'de':':hand_mit_gespreizten_fingern_mittlere_hautfarbe:','es':':mano_abierta_tono_de_piel_medio:','fr':':main_levée_doigts_écartés_peau_légèrement_mate:','ja':':開いた手_中間の肌色:','ko':':쫙_편_손바닥_갈색_피부:','pt':':mão_aberta_com_os_dedos_separados_pele_morena:','it':':mano_aperta_carnagione_olivastra:','fa':':دست_با_انگشتان_باز_پوست_طلایی:','id':':telapak_tangan_terbuka_warna_kulit_sedang:','zh':':手掌_中等肤色:','ru':':раскрытая_ладонь_средний_тон_кожи:','tr':':parmaklar_açık_el_kaldırma_orta_cilt_tonu:','ar':':يد_مرفوعة_بأصابع_متفرقة_بشرة_بلون_معتدل:'},'\U0001FAF0':{'en':':hand_with_index_finger_and_thumb_crossed:','status':fully_qualified,'E':14,'de':':hand_mit_gekreuztem_zeigefinger_und_daumen:','es':':mano_con_dedo_índice_y_pulgar_cruzados:','fr':':main_avec_index_et_pouce_croisés:','ja':':親指と人差し指をクロス:','ko':':엄지와_중지를_교차한_손:','pt':':mão_com_dedo_indicador_e_polegar_cruzados:','it':':mano_con_indice_e_pollice_incrociati:','fa':':دست_با_انگشت_اشاره_و_شست_متقاطع:','id':':tangan_dengan_jari_telunjuk_dan_ibu_jari_menyilang:','zh':':食指与拇指交叉的手:','ru':':жест_«деньги»:','tr':':işaret_parmağı_ve_baş_parmak_birbirine_geçmiş_el:','ar':':يد_مع_تقاطع_السبابة_والإبهام:'},'\U0001FAF0\U0001F3FF':{'en':':hand_with_index_finger_and_thumb_crossed_dark_skin_tone:','status':fully_qualified,'E':14,'de':':hand_mit_gekreuztem_zeigefinger_und_daumen_dunkle_hautfarbe:','es':':mano_con_dedo_índice_y_pulgar_cruzados_tono_de_piel_oscuro:','fr':':main_avec_index_et_pouce_croisés_peau_foncée:','ja':':親指と人差し指をクロス_濃い肌色:','ko':':엄지와_중지를_교차한_손_검은색_피부:','pt':':mão_com_dedo_indicador_e_polegar_cruzados_pele_escura:','it':':mano_con_indice_e_pollice_incrociati_carnagione_scura:','fa':':دست_با_انگشت_اشاره_و_شست_متقاطع_پوست_آبنوسی:','id':':tangan_dengan_jari_telunjuk_dan_ibu_jari_menyilang_warna_kulit_gelap:','zh':':食指与拇指交叉的手_较深肤色:','ru':':жест_«деньги»_очень_темный_тон_кожи:','tr':':işaret_parmağı_ve_baş_parmak_birbirine_geçmiş_el_koyu_cilt_tonu:','ar':':يد_مع_تقاطع_السبابة_والإبهام_بشرة_بلون_غامق:'},'\U0001FAF0\U0001F3FB':{'en':':hand_with_index_finger_and_thumb_crossed_light_skin_tone:','status':fully_qualified,'E':14,'de':':hand_mit_gekreuztem_zeigefinger_und_daumen_helle_hautfarbe:','es':':mano_con_dedo_índice_y_pulgar_cruzados_tono_de_piel_claro:','fr':':main_avec_index_et_pouce_croisés_peau_claire:','ja':':親指と人差し指をクロス_薄い肌色:','ko':':엄지와_중지를_교차한_손_하얀_피부:','pt':':mão_com_dedo_indicador_e_polegar_cruzados_pele_clara:','it':':mano_con_indice_e_pollice_incrociati_carnagione_chiara:','fa':':دست_با_انگشت_اشاره_و_شست_متقاطع_پوست_سفید:','id':':tangan_dengan_jari_telunjuk_dan_ibu_jari_menyilang_warna_kulit_cerah:','zh':':食指与拇指交叉的手_较浅肤色:','ru':':жест_«деньги»_очень_светлый_тон_кожи:','tr':':işaret_parmağı_ve_baş_parmak_birbirine_geçmiş_el_açık_cilt_tonu:','ar':':يد_مع_تقاطع_السبابة_والإبهام_بشرة_بلون_فاتح:'},'\U0001FAF0\U0001F3FE':{'en':':hand_with_index_finger_and_thumb_crossed_medium-dark_skin_tone:','status':fully_qualified,'E':14,'de':':hand_mit_gekreuztem_zeigefinger_und_daumen_mitteldunkle_hautfarbe:','es':':mano_con_dedo_índice_y_pulgar_cruzados_tono_de_piel_oscuro_medio:','fr':':main_avec_index_et_pouce_croisés_peau_mate:','ja':':親指と人差し指をクロス_やや濃い肌色:','ko':':엄지와_중지를_교차한_손_진한_갈색_피부:','pt':':mão_com_dedo_indicador_e_polegar_cruzados_pele_morena_escura:','it':':mano_con_indice_e_pollice_incrociati_carnagione_abbastanza_scura:','fa':':دست_با_انگشت_اشاره_و_شست_متقاطع_پوست_گندمی:','id':':tangan_dengan_jari_telunjuk_dan_ibu_jari_menyilang_warna_kulit_gelap-sedang:','zh':':食指与拇指交叉的手_中等深肤色:','ru':':жест_«деньги»_темный_тон_кожи:','tr':':işaret_parmağı_ve_baş_parmak_birbirine_geçmiş_el_orta_koyu_cilt_tonu:','ar':':يد_مع_تقاطع_السبابة_والإبهام_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001FAF0\U0001F3FC':{'en':':hand_with_index_finger_and_thumb_crossed_medium-light_skin_tone:','status':fully_qualified,'E':14,'de':':hand_mit_gekreuztem_zeigefinger_und_daumen_mittelhelle_hautfarbe:','es':':mano_con_dedo_índice_y_pulgar_cruzados_tono_de_piel_claro_medio:','fr':':main_avec_index_et_pouce_croisés_peau_moyennement_claire:','ja':':親指と人差し指をクロス_やや薄い肌色:','ko':':엄지와_중지를_교차한_손_연한_갈색_피부:','pt':':mão_com_dedo_indicador_e_polegar_cruzados_pele_morena_clara:','it':':mano_con_indice_e_pollice_incrociati_carnagione_abbastanza_chiara:','fa':':دست_با_انگشت_اشاره_و_شست_متقاطع_پوست_روشن:','id':':tangan_dengan_jari_telunjuk_dan_ibu_jari_menyilang_warna_kulit_cerah-sedang:','zh':':食指与拇指交叉的手_中等浅肤色:','ru':':жест_«деньги»_светлый_тон_кожи:','tr':':işaret_parmağı_ve_baş_parmak_birbirine_geçmiş_el_orta_açık_cilt_tonu:','ar':':يد_مع_تقاطع_السبابة_والإبهام_بشرة_بلون_فاتح_ومعتدل:'},'\U0001FAF0\U0001F3FD':{'en':':hand_with_index_finger_and_thumb_crossed_medium_skin_tone:','status':fully_qualified,'E':14,'de':':hand_mit_gekreuztem_zeigefinger_und_daumen_mittlere_hautfarbe:','es':':mano_con_dedo_índice_y_pulgar_cruzados_tono_de_piel_medio:','fr':':main_avec_index_et_pouce_croisés_peau_légèrement_mate:','ja':':親指と人差し指をクロス_中間の肌色:','ko':':엄지와_중지를_교차한_손_갈색_피부:','pt':':mão_com_dedo_indicador_e_polegar_cruzados_pele_morena:','it':':mano_con_indice_e_pollice_incrociati_carnagione_olivastra:','fa':':دست_با_انگشت_اشاره_و_شست_متقاطع_پوست_طلایی:','id':':tangan_dengan_jari_telunjuk_dan_ibu_jari_menyilang_warna_kulit_sedang:','zh':':食指与拇指交叉的手_中等肤色:','ru':':жест_«деньги»_средний_тон_кожи:','tr':':işaret_parmağı_ve_baş_parmak_birbirine_geçmiş_el_orta_cilt_tonu:','ar':':يد_مع_تقاطع_السبابة_والإبهام_بشرة_بلون_معتدل:'},'\U0001F45C':{'en':':handbag:','status':fully_qualified,'E':0.6,'de':':handtasche:','es':':bolso:','fr':':sac_à_main:','ja':':ハンドバッグ:','ko':':핸드백:','pt':':bolsa_de_mão:','it':':borsa:','fa':':کیف_دوشی:','id':':tas_jinjing:','zh':':手提包:','ru':':сумка:','tr':':el_çantası:','ar':':شنطة_يد:'},'\U0001F91D':{'en':':handshake:','status':fully_qualified,'E':3,'de':':handschlag:','es':':apretón_de_manos:','fr':':poignée_de_main:','ja':':握手:','ko':':악수:','pt':':aperto_de_mãos:','it':':stretta_di_mano:','fa':':دست_دادن:','id':':jabat_tangan:','zh':':握手:','ru':':рукопожатие:','tr':':el_sıkışma:','ar':':مصافحة:'},'\U0001F91D\U0001F3FF':{'en':':handshake_dark_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_dunkle_hautfarbe:','es':':apretón_de_manos_tono_de_piel_oscuro:','fr':':poignée_de_main_peau_foncée:','ja':':握手_濃い肌色:','ko':':악수_검은색_피부:','pt':':aperto_de_mãos_pele_escura:','it':':stretta_di_mano_carnagione_scura:','fa':':دست_دادن_پوست_آبنوسی:','id':':jabat_tangan_warna_kulit_gelap:','zh':':握手_较深肤色:','ru':':рукопожатие_очень_темный_тон_кожи:','tr':':el_sıkışma_koyu_cilt_tonu:','ar':':مصافحة_بشرة_بلون_غامق:'},'\U0001FAF1\U0001F3FF\U0000200D\U0001FAF2\U0001F3FB':{'en':':handshake_dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_dunkle_hautfarbe_helle_hautfarbe:','es':':apretón_de_manos_tono_de_piel_oscuro_y_tono_de_piel_claro:','fr':':poignée_de_main_peau_foncée_et_peau_claire:','ja':':握手_濃い肌色_薄い肌色:','ko':':악수_검은색_피부_하얀_피부:','pt':':aperto_de_mãos_pele_escura_e_pele_clara:','it':':stretta_di_mano_carnagione_scura_e_carnagione_chiara:','fa':':دست_دادن_پوست_آبنوسی_پوست_سفید:','id':':jabat_tangan_warna_kulit_gelap_warna_kulit_cerah:','zh':':握手_较深肤色较浅肤色:','ru':':рукопожатие_очень_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':el_sıkışma_koyu_cilt_tonu_açık_cilt_tonu:','ar':':مصافحة_بشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001FAF1\U0001F3FF\U0000200D\U0001FAF2\U0001F3FE':{'en':':handshake_dark_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_dunkle_hautfarbe_mitteldunkle_hautfarbe:','es':':apretón_de_manos_tono_de_piel_oscuro_y_tono_de_piel_oscuro_medio:','fr':':poignée_de_main_peau_foncée_et_peau_mate:','ja':':握手_濃い肌色_やや濃い肌色:','ko':':악수_검은색_피부_진한_갈색_피부:','pt':':aperto_de_mãos_pele_escura_e_pele_morena_escura:','it':':stretta_di_mano_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':دست_دادن_پوست_آبنوسی_پوست_گندمی:','id':':jabat_tangan_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':握手_较深肤色中等深肤色:','ru':':рукопожатие_очень_темный_тон_кожи_темный_тон_кожи:','tr':':el_sıkışma_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':مصافحة_بشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001FAF1\U0001F3FF\U0000200D\U0001FAF2\U0001F3FC':{'en':':handshake_dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_dunkle_hautfarbe_mittelhelle_hautfarbe:','es':':apretón_de_manos_tono_de_piel_oscuro_y_tono_de_piel_claro_medio:','fr':':poignée_de_main_peau_foncée_et_peau_moyennement_claire:','ja':':握手_濃い肌色_やや薄い肌色:','ko':':악수_검은색_피부_연한_갈색_피부:','pt':':aperto_de_mãos_pele_escura_e_pele_morena_clara:','it':':stretta_di_mano_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':دست_دادن_پوست_آبنوسی_پوست_روشن:','id':':jabat_tangan_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':握手_较深肤色中等浅肤色:','ru':':рукопожатие_очень_темный_тон_кожи_светлый_тон_кожи:','tr':':el_sıkışma_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':مصافحة_بشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001FAF1\U0001F3FF\U0000200D\U0001FAF2\U0001F3FD':{'en':':handshake_dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_dunkle_hautfarbe_mittlere_hautfarbe:','es':':apretón_de_manos_tono_de_piel_oscuro_y_tono_de_piel_medio:','fr':':poignée_de_main_peau_foncée_et_peau_légèrement_mate:','ja':':握手_濃い肌色_中間の肌色:','ko':':악수_검은색_피부_갈색_피부:','pt':':aperto_de_mãos_pele_escura_e_pele_morena:','it':':stretta_di_mano_carnagione_scura_e_carnagione_olivastra:','fa':':دست_دادن_پوست_آبنوسی_پوست_طلایی:','id':':jabat_tangan_warna_kulit_gelap_warna_kulit_sedang:','zh':':握手_较深肤色中等肤色:','ru':':рукопожатие_очень_темный_тон_кожи_средний_тон_кожи:','tr':':el_sıkışma_koyu_cilt_tonu_orta_cilt_tonu:','ar':':مصافحة_بشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F91D\U0001F3FB':{'en':':handshake_light_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_helle_hautfarbe:','es':':apretón_de_manos_tono_de_piel_claro:','fr':':poignée_de_main_peau_claire:','ja':':握手_薄い肌色:','ko':':악수_하얀_피부:','pt':':aperto_de_mãos_pele_clara:','it':':stretta_di_mano_carnagione_chiara:','fa':':دست_دادن_پوست_سفید:','id':':jabat_tangan_warna_kulit_cerah:','zh':':握手_较浅肤色:','ru':':рукопожатие_очень_светлый_тон_кожи:','tr':':el_sıkışma_açık_cilt_tonu:','ar':':مصافحة_بشرة_بلون_فاتح:'},'\U0001FAF1\U0001F3FB\U0000200D\U0001FAF2\U0001F3FF':{'en':':handshake_light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_helle_hautfarbe_dunkle_hautfarbe:','es':':apretón_de_manos_tono_de_piel_claro_y_tono_de_piel_oscuro:','fr':':poignée_de_main_peau_claire_et_peau_foncée:','ja':':握手_薄い肌色_濃い肌色:','ko':':악수_하얀_피부_검은색_피부:','pt':':aperto_de_mãos_pele_clara_e_pele_escura:','it':':stretta_di_mano_carnagione_chiara_e_carnagione_scura:','fa':':دست_دادن_پوست_سفید_پوست_آبنوسی:','id':':jabat_tangan_warna_kulit_cerah_warna_kulit_gelap:','zh':':握手_较浅肤色较深肤色:','ru':':рукопожатие_очень_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':el_sıkışma_açık_cilt_tonu_koyu_cilt_tonu:','ar':':مصافحة_بشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001FAF1\U0001F3FB\U0000200D\U0001FAF2\U0001F3FE':{'en':':handshake_light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_helle_hautfarbe_mitteldunkle_hautfarbe:','es':':apretón_de_manos_tono_de_piel_claro_y_tono_de_piel_oscuro_medio:','fr':':poignée_de_main_peau_claire_et_peau_mate:','ja':':握手_薄い肌色_やや濃い肌色:','ko':':악수_하얀_피부_진한_갈색_피부:','pt':':aperto_de_mãos_pele_clara_e_pele_morena_escura:','it':':stretta_di_mano_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':دست_دادن_پوست_سفید_پوست_گندمی:','id':':jabat_tangan_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':握手_较浅肤色中等深肤色:','ru':':рукопожатие_очень_светлый_тон_кожи_темный_тон_кожи:','tr':':el_sıkışma_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':مصافحة_بشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001FAF1\U0001F3FB\U0000200D\U0001FAF2\U0001F3FC':{'en':':handshake_light_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_helle_hautfarbe_mittelhelle_hautfarbe:','es':':apretón_de_manos_tono_de_piel_claro_y_tono_de_piel_claro_medio:','fr':':poignée_de_main_peau_claire_et_peau_moyennement_claire:','ja':':握手_薄い肌色_やや薄い肌色:','ko':':악수_하얀_피부_연한_갈색_피부:','pt':':aperto_de_mãos_pele_clara_e_pele_morena_clara:','it':':stretta_di_mano_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':دست_دادن_پوست_سفید_پوست_روشن:','id':':jabat_tangan_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':握手_较浅肤色中等浅肤色:','ru':':рукопожатие_очень_светлый_тон_кожи_светлый_тон_кожи:','tr':':el_sıkışma_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':مصافحة_بشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001FAF1\U0001F3FB\U0000200D\U0001FAF2\U0001F3FD':{'en':':handshake_light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_helle_hautfarbe_mittlere_hautfarbe:','es':':apretón_de_manos_tono_de_piel_claro_y_tono_de_piel_medio:','fr':':poignée_de_main_peau_claire_et_peau_légèrement_mate:','ja':':握手_薄い肌色_中間の肌色:','ko':':악수_하얀_피부_갈색_피부:','pt':':aperto_de_mãos_pele_clara_e_pele_morena:','it':':stretta_di_mano_carnagione_chiara_e_carnagione_olivastra:','fa':':دست_دادن_پوست_سفید_پوست_طلایی:','id':':jabat_tangan_warna_kulit_cerah_warna_kulit_sedang:','zh':':握手_较浅肤色中等肤色:','ru':':рукопожатие_очень_светлый_тон_кожи_средний_тон_кожи:','tr':':el_sıkışma_açık_cilt_tonu_orta_cilt_tonu:','ar':':مصافحة_بشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F91D\U0001F3FE':{'en':':handshake_medium-dark_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_mitteldunkle_hautfarbe:','es':':apretón_de_manos_tono_de_piel_oscuro_medio:','fr':':poignée_de_main_peau_mate:','ja':':握手_やや濃い肌色:','ko':':악수_진한_갈색_피부:','pt':':aperto_de_mãos_pele_morena_escura:','it':':stretta_di_mano_carnagione_abbastanza_scura:','fa':':دست_دادن_پوست_گندمی:','id':':jabat_tangan_warna_kulit_gelap-sedang:','zh':':握手_中等深肤色:','ru':':рукопожатие_темный_тон_кожи:','tr':':el_sıkışma_orta_koyu_cilt_tonu:','ar':':مصافحة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001FAF1\U0001F3FE\U0000200D\U0001FAF2\U0001F3FF':{'en':':handshake_medium-dark_skin_tone_dark_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_mitteldunkle_hautfarbe_dunkle_hautfarbe:','es':':apretón_de_manos_tono_de_piel_oscuro_medio_y_tono_de_piel_oscuro:','fr':':poignée_de_main_peau_mate_et_peau_foncée:','ja':':握手_やや濃い肌色_濃い肌色:','ko':':악수_진한_갈색_피부_검은색_피부:','pt':':aperto_de_mãos_pele_morena_escura_e_pele_escura:','it':':stretta_di_mano_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':دست_دادن_پوست_گندمی_پوست_آبنوسی:','id':':jabat_tangan_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':握手_中等深肤色较深肤色:','ru':':рукопожатие_темный_тон_кожи_очень_темный_тон_кожи:','tr':':el_sıkışma_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':مصافحة_بشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001FAF1\U0001F3FE\U0000200D\U0001FAF2\U0001F3FB':{'en':':handshake_medium-dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_mitteldunkle_hautfarbe_helle_hautfarbe:','es':':apretón_de_manos_tono_de_piel_oscuro_medio_y_tono_de_piel_claro:','fr':':poignée_de_main_peau_mate_et_peau_claire:','ja':':握手_やや濃い肌色_薄い肌色:','ko':':악수_진한_갈색_피부_하얀_피부:','pt':':aperto_de_mãos_pele_morena_escura_e_pele_clara:','it':':stretta_di_mano_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':دست_دادن_پوست_گندمی_پوست_سفید:','id':':jabat_tangan_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':握手_中等深肤色较浅肤色:','ru':':рукопожатие_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':el_sıkışma_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':مصافحة_بشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001FAF1\U0001F3FE\U0000200D\U0001FAF2\U0001F3FC':{'en':':handshake_medium-dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_mitteldunkle_hautfarbe_mittelhelle_hautfarbe:','es':':apretón_de_manos_tono_de_piel_oscuro_medio_y_tono_de_piel_claro_medio:','fr':':poignée_de_main_peau_mate_et_peau_moyennement_claire:','ja':':握手_やや濃い肌色_やや薄い肌色:','ko':':악수_진한_갈색_피부_연한_갈색_피부:','pt':':aperto_de_mãos_pele_morena_escura_e_pele_morena_clara:','it':':stretta_di_mano_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':دست_دادن_پوست_گندمی_پوست_روشن:','id':':jabat_tangan_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':握手_中等深肤色中等浅肤色:','ru':':рукопожатие_темный_тон_кожи_светлый_тон_кожи:','tr':':el_sıkışma_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':مصافحة_بشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001FAF1\U0001F3FE\U0000200D\U0001FAF2\U0001F3FD':{'en':':handshake_medium-dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_mitteldunkle_hautfarbe_mittlere_hautfarbe:','es':':apretón_de_manos_tono_de_piel_oscuro_medio_y_tono_de_piel_medio:','fr':':poignée_de_main_peau_mate_et_peau_légèrement_mate:','ja':':握手_やや濃い肌色_中間の肌色:','ko':':악수_진한_갈색_피부_갈색_피부:','pt':':aperto_de_mãos_pele_morena_escura_e_pele_morena:','it':':stretta_di_mano_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':دست_دادن_پوست_گندمی_پوست_طلایی:','id':':jabat_tangan_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':握手_中等深肤色中等肤色:','ru':':рукопожатие_темный_тон_кожи_средний_тон_кожи:','tr':':el_sıkışma_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':مصافحة_بشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F91D\U0001F3FC':{'en':':handshake_medium-light_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_mittelhelle_hautfarbe:','es':':apretón_de_manos_tono_de_piel_claro_medio:','fr':':poignée_de_main_peau_moyennement_claire:','ja':':握手_やや薄い肌色:','ko':':악수_연한_갈색_피부:','pt':':aperto_de_mãos_pele_morena_clara:','it':':stretta_di_mano_carnagione_abbastanza_chiara:','fa':':دست_دادن_پوست_روشن:','id':':jabat_tangan_warna_kulit_cerah-sedang:','zh':':握手_中等浅肤色:','ru':':рукопожатие_светлый_тон_кожи:','tr':':el_sıkışma_orta_açık_cilt_tonu:','ar':':مصافحة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001FAF1\U0001F3FC\U0000200D\U0001FAF2\U0001F3FF':{'en':':handshake_medium-light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_mittelhelle_hautfarbe_dunkle_hautfarbe:','es':':apretón_de_manos_tono_de_piel_claro_medio_y_tono_de_piel_oscuro:','fr':':poignée_de_main_peau_moyennement_claire_et_peau_foncée:','ja':':握手_やや薄い肌色_濃い肌色:','ko':':악수_연한_갈색_피부_검은색_피부:','pt':':aperto_de_mãos_pele_morena_clara_e_pele_escura:','it':':stretta_di_mano_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':دست_دادن_پوست_روشن_پوست_آبنوسی:','id':':jabat_tangan_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':握手_中等浅肤色较深肤色:','ru':':рукопожатие_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':el_sıkışma_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':مصافحة_بشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001FAF1\U0001F3FC\U0000200D\U0001FAF2\U0001F3FB':{'en':':handshake_medium-light_skin_tone_light_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_mittelhelle_hautfarbe_helle_hautfarbe:','es':':apretón_de_manos_tono_de_piel_claro_medio_y_tono_de_piel_claro:','fr':':poignée_de_main_peau_moyennement_claire_et_peau_claire:','ja':':握手_やや薄い肌色_薄い肌色:','ko':':악수_연한_갈색_피부_하얀_피부:','pt':':aperto_de_mãos_pele_morena_clara_e_pele_clara:','it':':stretta_di_mano_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':دست_دادن_پوست_روشن_پوست_سفید:','id':':jabat_tangan_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':握手_中等浅肤色较浅肤色:','ru':':рукопожатие_светлый_тон_кожи_очень_светлый_тон_кожи:','tr':':el_sıkışma_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':مصافحة_بشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001FAF1\U0001F3FC\U0000200D\U0001FAF2\U0001F3FE':{'en':':handshake_medium-light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_mittelhelle_hautfarbe_mitteldunkle_hautfarbe:','es':':apretón_de_manos_tono_de_piel_claro_medio_y_tono_de_piel_oscuro_medio:','fr':':poignée_de_main_peau_moyennement_claire_et_peau_mate:','ja':':握手_やや薄い肌色_やや濃い肌色:','ko':':악수_연한_갈색_피부_진한_갈색_피부:','pt':':aperto_de_mãos_pele_morena_clara_e_pele_morena_escura:','it':':stretta_di_mano_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':دست_دادن_پوست_روشن_پوست_گندمی:','id':':jabat_tangan_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':握手_中等浅肤色中等深肤色:','ru':':рукопожатие_светлый_тон_кожи_темный_тон_кожи:','tr':':el_sıkışma_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':مصافحة_بشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001FAF1\U0001F3FC\U0000200D\U0001FAF2\U0001F3FD':{'en':':handshake_medium-light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_mittelhelle_hautfarbe_mittlere_hautfarbe:','es':':apretón_de_manos_tono_de_piel_claro_medio_y_tono_de_piel_medio:','fr':':poignée_de_main_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':握手_やや薄い肌色_中間の肌色:','ko':':악수_연한_갈색_피부_갈색_피부:','pt':':aperto_de_mãos_pele_morena_clara_e_pele_morena:','it':':stretta_di_mano_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':دست_دادن_پوست_روشن_پوست_طلایی:','id':':jabat_tangan_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':握手_中等浅肤色中等肤色:','ru':':рукопожатие_светлый_тон_кожи_средний_тон_кожи:','tr':':el_sıkışma_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':مصافحة_بشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F91D\U0001F3FD':{'en':':handshake_medium_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_mittlere_hautfarbe:','es':':apretón_de_manos_tono_de_piel_medio:','fr':':poignée_de_main_peau_légèrement_mate:','ja':':握手_中間の肌色:','ko':':악수_갈색_피부:','pt':':aperto_de_mãos_pele_morena:','it':':stretta_di_mano_carnagione_olivastra:','fa':':دست_دادن_پوست_طلایی:','id':':jabat_tangan_warna_kulit_sedang:','zh':':握手_中等肤色:','ru':':рукопожатие_средний_тон_кожи:','tr':':el_sıkışma_orta_cilt_tonu:','ar':':مصافحة_بشرة_بلون_معتدل:'},'\U0001FAF1\U0001F3FD\U0000200D\U0001FAF2\U0001F3FF':{'en':':handshake_medium_skin_tone_dark_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_mittlere_hautfarbe_dunkle_hautfarbe:','es':':apretón_de_manos_tono_de_piel_medio_y_tono_de_piel_oscuro:','fr':':poignée_de_main_peau_légèrement_mate_et_peau_foncée:','ja':':握手_中間の肌色_濃い肌色:','ko':':악수_갈색_피부_검은색_피부:','pt':':aperto_de_mãos_pele_morena_e_pele_escura:','it':':stretta_di_mano_carnagione_olivastra_e_carnagione_scura:','fa':':دست_دادن_پوست_طلایی_پوست_آبنوسی:','id':':jabat_tangan_warna_kulit_sedang_warna_kulit_gelap:','zh':':握手_中等肤色较深肤色:','ru':':рукопожатие_средний_тон_кожи_очень_темный_тон_кожи:','tr':':el_sıkışma_orta_cilt_tonu_koyu_cilt_tonu:','ar':':مصافحة_بشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001FAF1\U0001F3FD\U0000200D\U0001FAF2\U0001F3FB':{'en':':handshake_medium_skin_tone_light_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_mittlere_hautfarbe_helle_hautfarbe:','es':':apretón_de_manos_tono_de_piel_medio_y_tono_de_piel_claro:','fr':':poignée_de_main_peau_légèrement_mate_et_peau_claire:','ja':':握手_中間の肌色_薄い肌色:','ko':':악수_갈색_피부_하얀_피부:','pt':':aperto_de_mãos_pele_morena_e_pele_clara:','it':':stretta_di_mano_carnagione_olivastra_e_carnagione_chiara:','fa':':دست_دادن_پوست_طلایی_پوست_سفید:','id':':jabat_tangan_warna_kulit_sedang_warna_kulit_cerah:','zh':':握手_中等肤色较浅肤色:','ru':':рукопожатие_средний_тон_кожи_очень_светлый_тон_кожи:','tr':':el_sıkışma_orta_cilt_tonu_açık_cilt_tonu:','ar':':مصافحة_بشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001FAF1\U0001F3FD\U0000200D\U0001FAF2\U0001F3FE':{'en':':handshake_medium_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_mittlere_hautfarbe_mitteldunkle_hautfarbe:','es':':apretón_de_manos_tono_de_piel_medio_y_tono_de_piel_oscuro_medio:','fr':':poignée_de_main_peau_légèrement_mate_et_peau_mate:','ja':':握手_中間の肌色_やや濃い肌色:','ko':':악수_갈색_피부_진한_갈색_피부:','pt':':aperto_de_mãos_pele_morena_e_pele_morena_escura:','it':':stretta_di_mano_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':دست_دادن_پوست_طلایی_پوست_گندمی:','id':':jabat_tangan_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':握手_中等肤色中等深肤色:','ru':':рукопожатие_средний_тон_кожи_темный_тон_кожи:','tr':':el_sıkışma_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':مصافحة_بشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001FAF1\U0001F3FD\U0000200D\U0001FAF2\U0001F3FC':{'en':':handshake_medium_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':14,'de':':handschlag_mittlere_hautfarbe_mittelhelle_hautfarbe:','es':':apretón_de_manos_tono_de_piel_medio_y_tono_de_piel_claro_medio:','fr':':poignée_de_main_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':握手_中間の肌色_やや薄い肌色:','ko':':악수_갈색_피부_연한_갈색_피부:','pt':':aperto_de_mãos_pele_morena_e_pele_morena_clara:','it':':stretta_di_mano_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':دست_دادن_پوست_طلایی_پوست_روشن:','id':':jabat_tangan_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':握手_中等肤色中等浅肤色:','ru':':рукопожатие_средний_тон_кожи_светлый_тон_кожи:','tr':':el_sıkışma_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':مصافحة_بشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F423':{'en':':hatching_chick:','status':fully_qualified,'E':0.6,'de':':schlüpfendes_küken:','es':':pollito_rompiendo_el_cascarón:','fr':':poussin_qui_éclôt:','ja':':卵からかえったひよこ:','ko':':알에서_깬_병아리:','pt':':pintinho_chocando:','it':':pulcino_che_nasce:','fa':':جوجه_ازتخم_بیرون_آمده:','id':':itik_menetas:','zh':':小鸡破壳:','ru':':цыпленок_в_яйце:','tr':':yumurtadan_çıkan_civciv:','ar':':كتكوت_خارج_البيضة:'},'\U0001F642\U0000200D\U00002194\U0000FE0F':{'en':':head_shaking_horizontally:','status':fully_qualified,'E':15.1,'de':':kopfschütteln:','es':':cabeza_negando:','fr':':tête_secouée_horizontalement:','ja':':首を横に振る:','ko':':도리도리:','pt':':cabeça_virando_de_um_lado_para_o_outro:','it':':scuotere_la_testa_in_senso_orizzontale:','fa':':تکان_دادن_افقی_سر:','id':':kepala_bergeleng:','zh':':左右摇头:','ru':':отрицательно_качает_головой:','tr':':yatay_olarak_sallanan_yüz:','ar':':رأس_يهتز_أفقيا:'},'\U0001F642\U0000200D\U00002194':{'en':':head_shaking_horizontally:','status':minimally_qualified,'E':15.1,'de':':kopfschütteln:','es':':cabeza_negando:','fr':':tête_secouée_horizontalement:','ja':':首を横に振る:','ko':':도리도리:','pt':':cabeça_virando_de_um_lado_para_o_outro:','it':':scuotere_la_testa_in_senso_orizzontale:','fa':':تکان_دادن_افقی_سر:','id':':kepala_bergeleng:','zh':':左右摇头:','ru':':отрицательно_качает_головой:','tr':':yatay_olarak_sallanan_yüz:','ar':':رأس_يهتز_أفقيا:'},'\U0001F642\U0000200D\U00002195\U0000FE0F':{'en':':head_shaking_vertically:','status':fully_qualified,'E':15.1,'de':':kopfnicken:','es':':cabeza_asintiendo:','fr':':tête_secouée_verticalement:','ja':':首を縦に振る:','ko':':끄덕끄덕:','pt':':cabeça_balançando_na_vertical:','it':':scuotere_la_testa_in_senso_verticale:','fa':':تکان_دادن_عمودی_سر:','id':':kepala_mengangguk:','zh':':上下点头:','ru':':положительно_качает_головой:','tr':':dikey_olarak_sallanan_yüz:','ar':':رأس_يهتز_عموديا:'},'\U0001F642\U0000200D\U00002195':{'en':':head_shaking_vertically:','status':minimally_qualified,'E':15.1,'de':':kopfnicken:','es':':cabeza_asintiendo:','fr':':tête_secouée_verticalement:','ja':':首を縦に振る:','ko':':끄덕끄덕:','pt':':cabeça_balançando_na_vertical:','it':':scuotere_la_testa_in_senso_verticale:','fa':':تکان_دادن_عمودی_سر:','id':':kepala_mengangguk:','zh':':上下点头:','ru':':положительно_качает_головой:','tr':':dikey_olarak_sallanan_yüz:','ar':':رأس_يهتز_عموديا:'},'\U0001F3A7':{'en':':headphone:','status':fully_qualified,'E':0.6,'alias':[':headphones:'],'variant':True,'de':':kopfhörer:','es':':auricular:','fr':':casque:','ja':':ヘッドホン:','ko':':헤드폰:','pt':':fones_de_ouvido:','it':':cuffie:','fa':':هدفون:','id':':headphone:','zh':':耳机:','ru':':наушники:','tr':':kulaklık:','ar':':سماعة_رأس:'},'\U0001FAA6':{'en':':headstone:','status':fully_qualified,'E':13,'de':':grabstein:','es':':lápida:','fr':':pierre_tombale:','ja':':墓石:','ko':':묘비:','pt':':lápide:','it':':lapide:','fa':':سنگ_قبر:','id':':batu_nisan:','zh':':墓碑:','ru':':надгробие:','tr':':mezar_taşı:','ar':':حجر_مقبرة:'},'\U0001F9D1\U0000200D\U00002695\U0000FE0F':{'en':':health_worker:','status':fully_qualified,'E':12.1,'de':':arzt/ärztin:','es':':profesional_sanitario:','fr':':professionnel_de_la_santé_(tous_genres):','ja':':医者:','ko':':의료인:','pt':':profissional_de_saúde:','it':':persona_che_lavora_nella_sanità:','fa':':مأمور_بهداشت:','id':':petugas_kesehatan:','zh':':卫生工作者:','ru':':медицинскии_работник:','tr':':sağlık_çalışanı:','ar':':موظف_بقطاع_الصحة:'},'\U0001F9D1\U0000200D\U00002695':{'en':':health_worker:','status':minimally_qualified,'E':12.1,'de':':arzt/ärztin:','es':':profesional_sanitario:','fr':':professionnel_de_la_santé_(tous_genres):','ja':':医者:','ko':':의료인:','pt':':profissional_de_saúde:','it':':persona_che_lavora_nella_sanità:','fa':':مأمور_بهداشت:','id':':petugas_kesehatan:','zh':':卫生工作者:','ru':':медицинский_работник:','tr':':sağlık_çalışanı:','ar':':موظف_بقطاع_الصحة:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002695\U0000FE0F':{'en':':health_worker_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':arzt/ärztin_dunkle_hautfarbe:','es':':profesional_sanitario_tono_de_piel_oscuro:','fr':':professionnel_de_la_santé_(tous_genres)_peau_foncée:','ja':':医者_濃い肌色:','ko':':의료인_검은색_피부:','pt':':profissional_de_saúde_pele_escura:','it':':persona_che_lavora_nella_sanità_carnagione_scura:','fa':':مأمور_بهداشت_پوست_آبنوسی:','id':':petugas_kesehatan_warna_kulit_gelap:','zh':':卫生工作者_较深肤色:','ru':':медицинскии_работник_очень_темныи_тон_кожи:','tr':':sağlık_çalışanı_koyu_cilt_tonu:','ar':':موظف_بقطاع_الصحة_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002695':{'en':':health_worker_dark_skin_tone:','status':minimally_qualified,'E':12.1,'de':':arzt/ärztin_dunkle_hautfarbe:','es':':profesional_sanitario_tono_de_piel_oscuro:','fr':':professionnel_de_la_santé_(tous_genres)_peau_foncée:','ja':':医者_濃い肌色:','ko':':의료인_검은색_피부:','pt':':profissional_de_saúde_pele_escura:','it':':persona_che_lavora_nella_sanità_carnagione_scura:','fa':':مأمور_بهداشت_پوست_آبنوسی:','id':':petugas_kesehatan_warna_kulit_gelap:','zh':':卫生工作者_较深肤色:','ru':':медицинский_работник_очень_темный_тон_кожи:','tr':':sağlık_çalışanı_koyu_cilt_tonu:','ar':':موظف_بقطاع_الصحة_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002695\U0000FE0F':{'en':':health_worker_light_skin_tone:','status':fully_qualified,'E':12.1,'de':':arzt/ärztin_helle_hautfarbe:','es':':profesional_sanitario_tono_de_piel_claro:','fr':':professionnel_de_la_santé_(tous_genres)_peau_claire:','ja':':医者_薄い肌色:','ko':':의료인_하얀_피부:','pt':':profissional_de_saúde_pele_clara:','it':':persona_che_lavora_nella_sanità_carnagione_chiara:','fa':':مأمور_بهداشت_پوست_سفید:','id':':petugas_kesehatan_warna_kulit_cerah:','zh':':卫生工作者_较浅肤色:','ru':':медицинскии_работник_очень_светлыи_тон_кожи:','tr':':sağlık_çalışanı_açık_cilt_tonu:','ar':':موظف_بقطاع_الصحة_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002695':{'en':':health_worker_light_skin_tone:','status':minimally_qualified,'E':12.1,'de':':arzt/ärztin_helle_hautfarbe:','es':':profesional_sanitario_tono_de_piel_claro:','fr':':professionnel_de_la_santé_(tous_genres)_peau_claire:','ja':':医者_薄い肌色:','ko':':의료인_하얀_피부:','pt':':profissional_de_saúde_pele_clara:','it':':persona_che_lavora_nella_sanità_carnagione_chiara:','fa':':مأمور_بهداشت_پوست_سفید:','id':':petugas_kesehatan_warna_kulit_cerah:','zh':':卫生工作者_较浅肤色:','ru':':медицинский_работник_очень_светлый_тон_кожи:','tr':':sağlık_çalışanı_açık_cilt_tonu:','ar':':موظف_بقطاع_الصحة_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002695\U0000FE0F':{'en':':health_worker_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':arzt/ärztin_mitteldunkle_hautfarbe:','es':':profesional_sanitario_tono_de_piel_oscuro_medio:','fr':':professionnel_de_la_santé_(tous_genres)_peau_mate:','ja':':医者_やや濃い肌色:','ko':':의료인_진한_갈색_피부:','pt':':profissional_de_saúde_pele_morena_escura:','it':':persona_che_lavora_nella_sanità_carnagione_abbastanza_scura:','fa':':مأمور_بهداشت_پوست_گندمی:','id':':petugas_kesehatan_warna_kulit_gelap-sedang:','zh':':卫生工作者_中等深肤色:','ru':':медицинскии_работник_темныи_тон_кожи:','tr':':sağlık_çalışanı_orta_koyu_cilt_tonu:','ar':':موظف_بقطاع_الصحة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002695':{'en':':health_worker_medium-dark_skin_tone:','status':minimally_qualified,'E':12.1,'de':':arzt/ärztin_mitteldunkle_hautfarbe:','es':':profesional_sanitario_tono_de_piel_oscuro_medio:','fr':':professionnel_de_la_santé_(tous_genres)_peau_mate:','ja':':医者_やや濃い肌色:','ko':':의료인_진한_갈색_피부:','pt':':profissional_de_saúde_pele_morena_escura:','it':':persona_che_lavora_nella_sanità_carnagione_abbastanza_scura:','fa':':مأمور_بهداشت_پوست_گندمی:','id':':petugas_kesehatan_warna_kulit_gelap-sedang:','zh':':卫生工作者_中等深肤色:','ru':':медицинский_работник_темный_тон_кожи:','tr':':sağlık_çalışanı_orta_koyu_cilt_tonu:','ar':':موظف_بقطاع_الصحة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002695\U0000FE0F':{'en':':health_worker_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':arzt/ärztin_mittelhelle_hautfarbe:','es':':profesional_sanitario_tono_de_piel_claro_medio:','fr':':professionnel_de_la_santé_(tous_genres)_peau_moyennement_claire:','ja':':医者_やや薄い肌色:','ko':':의료인_연한_갈색_피부:','pt':':profissional_de_saúde_pele_morena_clara:','it':':persona_che_lavora_nella_sanità_carnagione_abbastanza_chiara:','fa':':مأمور_بهداشت_پوست_روشن:','id':':petugas_kesehatan_warna_kulit_cerah-sedang:','zh':':卫生工作者_中等浅肤色:','ru':':медицинскии_работник_светлыи_тон_кожи:','tr':':sağlık_çalışanı_orta_açık_cilt_tonu:','ar':':موظف_بقطاع_الصحة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002695':{'en':':health_worker_medium-light_skin_tone:','status':minimally_qualified,'E':12.1,'de':':arzt/ärztin_mittelhelle_hautfarbe:','es':':profesional_sanitario_tono_de_piel_claro_medio:','fr':':professionnel_de_la_santé_(tous_genres)_peau_moyennement_claire:','ja':':医者_やや薄い肌色:','ko':':의료인_연한_갈색_피부:','pt':':profissional_de_saúde_pele_morena_clara:','it':':persona_che_lavora_nella_sanità_carnagione_abbastanza_chiara:','fa':':مأمور_بهداشت_پوست_روشن:','id':':petugas_kesehatan_warna_kulit_cerah-sedang:','zh':':卫生工作者_中等浅肤色:','ru':':медицинский_работник_светлый_тон_кожи:','tr':':sağlık_çalışanı_orta_açık_cilt_tonu:','ar':':موظف_بقطاع_الصحة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002695\U0000FE0F':{'en':':health_worker_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':arzt/ärztin_mittlere_hautfarbe:','es':':profesional_sanitario_tono_de_piel_medio:','fr':':professionnel_de_la_santé_(tous_genres)_peau_légèrement_mate:','ja':':医者_中間の肌色:','ko':':의료인_갈색_피부:','pt':':profissional_de_saúde_pele_morena:','it':':persona_che_lavora_nella_sanità_carnagione_olivastra:','fa':':مأمور_بهداشت_پوست_طلایی:','id':':petugas_kesehatan_warna_kulit_sedang:','zh':':卫生工作者_中等肤色:','ru':':медицинскии_работник_среднии_тон_кожи:','tr':':sağlık_çalışanı_orta_cilt_tonu:','ar':':موظف_بقطاع_الصحة_بشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002695':{'en':':health_worker_medium_skin_tone:','status':minimally_qualified,'E':12.1,'de':':arzt/ärztin_mittlere_hautfarbe:','es':':profesional_sanitario_tono_de_piel_medio:','fr':':professionnel_de_la_santé_(tous_genres)_peau_légèrement_mate:','ja':':医者_中間の肌色:','ko':':의료인_갈색_피부:','pt':':profissional_de_saúde_pele_morena:','it':':persona_che_lavora_nella_sanità_carnagione_olivastra:','fa':':مأمور_بهداشت_پوست_طلایی:','id':':petugas_kesehatan_warna_kulit_sedang:','zh':':卫生工作者_中等肤色:','ru':':медицинский_работник_средний_тон_кожи:','tr':':sağlık_çalışanı_orta_cilt_tonu:','ar':':موظف_بقطاع_الصحة_بشرة_بلون_معتدل:'},'\U0001F649':{'en':':hear-no-evil_monkey:','status':fully_qualified,'E':0.6,'alias':[':hear_no_evil:', ':hear_no_evil_monkey:'],'de':':sich_die_ohren_zuhaltendes_affengesicht:','es':':mono_con_los_oídos_tapados:','fr':':singe_ne_rien_entendre:','ja':':聞かざる:','ko':':귀를_막고_있는_원숭이:','pt':':macaco_que_não_ouve_nada:','it':':non_sento:','fa':':میمون_شرنشنو:','id':':tidak_mendengar:','zh':':非礼勿听:','ru':':ничего_не_слышу:','tr':':duymadım:','ar':':قرد_يحجب_أذنيه:'},'\U0001F49F':{'en':':heart_decoration:','status':fully_qualified,'E':0.6,'de':':herzdekoration:','es':':adorno_de_corazón:','fr':':décoration_avec_cœur:','ja':':ハートデコ:','ko':':하트_장식:','pt':':coração_decorativo:','it':':decorazione_con_cuore:','fa':':قلب_تزئینی:','id':':dekorasi_hati:','zh':':心型装饰:','ru':':значок_«сердце»:','tr':':kalpli_dekorasyon:','ar':':زخرفة_قلب:'},'\U00002763\U0000FE0F':{'en':':heart_exclamation:','status':fully_qualified,'E':1,'alias':[':heavy_heart_exclamation:', ':heavy_heart_exclamation_mark_ornament:'],'variant':True,'de':':herz_als_ausrufezeichen:','es':':exclamación_de_corazón:','fr':':cœur_point_d’exclamation:','ja':':ハートのびっくり:','ko':':하트_모양_느낌표:','pt':':exclamação_de_coração:','it':':punto_esclamativo_a_cuore:','fa':':علامت_تعجب_قلبی:','id':':tanda_seru_berbentuk_hati:','zh':':心叹号:','ru':':восклицательныи_знак_в_виде_сердца:','tr':':kalp_şeklinde_ünlem_işareti:','ar':':قلب_على_شكل_علامة_تعجب:'},'\U00002763':{'en':':heart_exclamation:','status':unqualified,'E':1,'alias':[':heavy_heart_exclamation:', ':heavy_heart_exclamation_mark_ornament:'],'variant':True,'de':':herz_als_ausrufezeichen:','es':':exclamación_de_corazón:','fr':':cœur_point_d’exclamation:','ja':':ハートのびっくり:','ko':':하트_모양_느낌표:','pt':':exclamação_de_coração:','it':':punto_esclamativo_a_cuore:','fa':':علامت_تعجب_قلبی:','id':':tanda_seru_berbentuk_hati:','zh':':心叹号:','ru':':восклицательный_знак_в_виде_сердца:','tr':':kalp_şeklinde_ünlem_işareti:','ar':':قلب_على_شكل_علامة_تعجب:'},'\U0001FAF6':{'en':':heart_hands:','status':fully_qualified,'E':14,'de':':hände_die_herz_bilden:','es':':manos_formando_un_corazón:','fr':':mains_qui_forment_un_cœur:','ja':':ハート形の手:','ko':':손_하트:','pt':':mãos_de_coração:','it':':mani_a_cuore:','fa':':شکل_قلب_با_دست:','id':':tangan_membentuk_hati:','zh':':做成心形的双手:','ru':':жест_«сердце»:','tr':':kalp_oluşturan_eller:','ar':':أيدي_متشابكة_على_شكل_قلب:'},'\U0001FAF6\U0001F3FF':{'en':':heart_hands_dark_skin_tone:','status':fully_qualified,'E':14,'de':':hände_die_herz_bilden_dunkle_hautfarbe:','es':':manos_formando_un_corazón_tono_de_piel_oscuro:','fr':':mains_qui_forment_un_cœur_peau_foncée:','ja':':ハート形の手_濃い肌色:','ko':':손_하트_검은색_피부:','pt':':mãos_de_coração_pele_escura:','it':':mani_a_cuore_carnagione_scura:','fa':':شکل_قلب_با_دست_پوست_آبنوسی:','id':':tangan_membentuk_hati_warna_kulit_gelap:','zh':':做成心形的双手_较深肤色:','ru':':жест_«сердце»_очень_темный_тон_кожи:','tr':':kalp_oluşturan_eller_koyu_cilt_tonu:','ar':':أيدي_متشابكة_على_شكل_قلب_بشرة_بلون_غامق:'},'\U0001FAF6\U0001F3FB':{'en':':heart_hands_light_skin_tone:','status':fully_qualified,'E':14,'de':':hände_die_herz_bilden_helle_hautfarbe:','es':':manos_formando_un_corazón_tono_de_piel_claro:','fr':':mains_qui_forment_un_cœur_peau_claire:','ja':':ハート形の手_薄い肌色:','ko':':손_하트_하얀_피부:','pt':':mãos_de_coração_pele_clara:','it':':mani_a_cuore_carnagione_chiara:','fa':':شکل_قلب_با_دست_پوست_سفید:','id':':tangan_membentuk_hati_warna_kulit_cerah:','zh':':做成心形的双手_较浅肤色:','ru':':жест_«сердце»_очень_светлый_тон_кожи:','tr':':kalp_oluşturan_eller_açık_cilt_tonu:','ar':':أيدي_متشابكة_على_شكل_قلب_بشرة_بلون_فاتح:'},'\U0001FAF6\U0001F3FE':{'en':':heart_hands_medium-dark_skin_tone:','status':fully_qualified,'E':14,'de':':hände_die_herz_bilden_mitteldunkle_hautfarbe:','es':':manos_formando_un_corazón_tono_de_piel_oscuro_medio:','fr':':mains_qui_forment_un_cœur_peau_mate:','ja':':ハート形の手_やや濃い肌色:','ko':':손_하트_진한_갈색_피부:','pt':':mãos_de_coração_pele_morena_escura:','it':':mani_a_cuore_carnagione_abbastanza_scura:','fa':':شکل_قلب_با_دست_پوست_گندمی:','id':':tangan_membentuk_hati_warna_kulit_gelap-sedang:','zh':':做成心形的双手_中等深肤色:','ru':':жест_«сердце»_темный_тон_кожи:','tr':':kalp_oluşturan_eller_orta_koyu_cilt_tonu:','ar':':أيدي_متشابكة_على_شكل_قلب_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001FAF6\U0001F3FC':{'en':':heart_hands_medium-light_skin_tone:','status':fully_qualified,'E':14,'de':':hände_die_herz_bilden_mittelhelle_hautfarbe:','es':':manos_formando_un_corazón_tono_de_piel_claro_medio:','fr':':mains_qui_forment_un_cœur_peau_moyennement_claire:','ja':':ハート形の手_やや薄い肌色:','ko':':손_하트_연한_갈색_피부:','pt':':mãos_de_coração_pele_morena_clara:','it':':mani_a_cuore_carnagione_abbastanza_chiara:','fa':':شکل_قلب_با_دست_پوست_روشن:','id':':tangan_membentuk_hati_warna_kulit_cerah-sedang:','zh':':做成心形的双手_中等浅肤色:','ru':':жест_«сердце»_светлый_тон_кожи:','tr':':kalp_oluşturan_eller_orta_açık_cilt_tonu:','ar':':أيدي_متشابكة_على_شكل_قلب_بشرة_بلون_فاتح_ومعتدل:'},'\U0001FAF6\U0001F3FD':{'en':':heart_hands_medium_skin_tone:','status':fully_qualified,'E':14,'de':':hände_die_herz_bilden_mittlere_hautfarbe:','es':':manos_formando_un_corazón_tono_de_piel_medio:','fr':':mains_qui_forment_un_cœur_peau_légèrement_mate:','ja':':ハート形の手_中間の肌色:','ko':':손_하트_갈색_피부:','pt':':mãos_de_coração_pele_morena:','it':':mani_a_cuore_carnagione_olivastra:','fa':':شکل_قلب_با_دست_پوست_طلایی:','id':':tangan_membentuk_hati_warna_kulit_sedang:','zh':':做成心形的双手_中等肤色:','ru':':жест_«сердце»_средний_тон_кожи:','tr':':kalp_oluşturan_eller_orta_cilt_tonu:','ar':':أيدي_متشابكة_على_شكل_قلب_بشرة_بلون_معتدل:'},'\U00002764\U0000FE0F\U0000200D\U0001F525':{'en':':heart_on_fire:','status':fully_qualified,'E':13.1,'de':':herz_in_flammen:','es':':corazón_en_llamas:','fr':':cœur_enflammé:','ja':':燃えるハート:','ko':':불_난_하트:','pt':':coração_em_chamas:','it':':cuore_in_fiamme:','fa':':قلب_در_آتش:','id':':hati_berapi-api:','zh':':火上之心:','ru':':горящее_сердце:','tr':':ateşli_kalp:','ar':':قلب_يحترق:'},'\U00002764\U0000200D\U0001F525':{'en':':heart_on_fire:','status':unqualified,'E':13.1,'de':':herz_in_flammen:','es':':corazón_en_llamas:','fr':':cœur_enflammé:','ja':':燃えるハート:','ko':':불_난_하트:','pt':':coração_em_chamas:','it':':cuore_in_fiamme:','fa':':قلب_در_آتش:','id':':hati_berapi-api:','zh':':火上之心:','ru':':горящее_сердце:','tr':':ateşli_kalp:','ar':':قلب_يحترق:'},'\U00002665\U0000FE0F':{'en':':heart_suit:','status':fully_qualified,'E':0.6,'alias':[':hearts:'],'variant':True,'de':':herz:','es':':palo_de_corazones:','fr':':cœur_cartes:','ja':':ハート:','ko':':하트:','pt':':naipe_de_copas:','it':':cuori:','fa':':خال_دل:','id':':hati:','zh':':红桃:','ru':':червы:','tr':':kupa:','ar':':ورقة_قلب_أحمر:'},'\U00002665':{'en':':heart_suit:','status':unqualified,'E':0.6,'alias':[':hearts:'],'variant':True,'de':':herz:','es':':palo_de_corazones:','fr':':cœur_cartes:','ja':':ハート:','ko':':하트:','pt':':naipe_de_copas:','it':':cuori:','fa':':خال_دل:','id':':hati:','zh':':红桃:','ru':':червы:','tr':':kupa:','ar':':ورقة_قلب_أحمر:'},'\U0001F498':{'en':':heart_with_arrow:','status':fully_qualified,'E':0.6,'alias':[':cupid:'],'de':':herz_mit_pfeil:','es':':corazón_con_flecha:','fr':':cœur_et_flèche:','ja':':ハートに矢:','ko':':화살이_꽂힌_하트:','pt':':coração_com_flecha:','it':':cuore_con_freccia:','fa':':قلب_تیرخورده:','id':':hati_terpanah:','zh':':心中箭了:','ru':':сердце_пронзенное_стрелой:','tr':':ok_saplanmış_kalp:','ar':':قلب_وسهم:'},'\U0001F49D':{'en':':heart_with_ribbon:','status':fully_qualified,'E':0.6,'alias':[':gift_heart:'],'de':':herz_mit_schleife:','es':':corazón_con_lazo:','fr':':cœur_avec_ruban:','ja':':ハートにリボン:','ko':':리본_달린_하트:','pt':':coração_com_fita:','it':':cuore_con_fiocco:','fa':':قلب_با_روبان:','id':':hati_berpita:','zh':':系有缎带的心:','ru':':сердце_с_лентой:','tr':':kurdeleli_kalp:','ar':':قلب_عليه_شريط:'},'\U0001F4B2':{'en':':heavy_dollar_sign:','status':fully_qualified,'E':0.6,'de':':dollarzeichen_extrafett:','es':':símbolo_de_dólar:','fr':':symbole_dollar:','ja':':ドル記号:','ko':':미국_달러:','pt':':cifrão:','it':':dollaro:','fa':':نماد_دلار:','id':':tanda_dolar_tebal:','zh':':粗美元符号:','ru':':жирный_знак_доллара:','tr':':dolar_işareti:','ar':':علامة_دولار:'},'\U0001F7F0':{'en':':heavy_equals_sign:','status':fully_qualified,'E':14,'de':':gleichheitszeichen_extrafett:','es':':signo_igual_grueso:','fr':':signe_égal_gras:','ja':':太字の等号:','ko':':등호_기호:','pt':':sinal_de_igual:','it':':segno_di_uguaglianza:','fa':':علامت_تساوی_پررنگ:','id':':tanda_sama_dengan_tebal:','zh':':粗等号:','ru':':жирный_знак_равенства:','tr':':kalın_eşittir_işareti:','ar':':علامة_يساوي_قاتمة:'},'\U0001F994':{'en':':hedgehog:','status':fully_qualified,'E':5,'de':':igel:','es':':erizo:','fr':':hérisson:','ja':':ハリネズミ:','ko':':고슴도치:','pt':':porco-espinho:','it':':riccio:','fa':':جوجه_تیغی:','id':':landak:','zh':':刺猬:','ru':':еж:','tr':':kirpi:','ar':':قنفد:'},'\U0001F681':{'en':':helicopter:','status':fully_qualified,'E':1,'de':':hubschrauber:','es':':helicóptero:','fr':':hélicoptère:','ja':':ヘリコプター:','ko':':헬리콥터:','pt':':helicóptero:','it':':elicottero:','fa':':هلیکوپتر:','id':':helikopter:','zh':':直升机:','ru':':вертолет:','tr':':helikopter:','ar':':هليكوبتر:'},'\U0001F33F':{'en':':herb:','status':fully_qualified,'E':0.6,'de':':kräuter:','es':':hierba:','fr':':feuille:','ja':':ハーブ:','ko':':풀:','pt':':erva:','it':':pianta:','fa':':گیاه_دارویی:','id':':herba:','zh':':药草:','ru':':веточка:','tr':':ot:','ar':':عشب:'},'\U0001F33A':{'en':':hibiscus:','status':fully_qualified,'E':0.6,'de':':hibiskus:','es':':flor_de_hibisco:','fr':':hibiscus:','ja':':ハイビスカス:','ko':':무궁화:','pt':':hibisco:','it':':ibisco:','fa':':گل_ختمی:','id':':kembang_sepatu:','zh':':芙蓉:','ru':':гибискус:','tr':':çingülü:','ar':':كركديه:'},'\U0001F460':{'en':':high-heeled_shoe:','status':fully_qualified,'E':0.6,'alias':[':high_heel:', ':high_heeled_shoe:'],'de':':stöckelschuh:','es':':zapato_de_tacón:','fr':':chaussure_à_talon_haut:','ja':':ハイヒール:','ko':':하이힐:','pt':':sapato_de_salto_alto:','it':':scarpa_con_il_tacco:','fa':':کفش_پاشنه_بلند:','id':':sepatu_hak_tinggi:','zh':':高跟鞋:','ru':':туфли_на_каблуке:','tr':':yüksek_topuklu_ayakkabı:','ar':':حذاء_بكعب_عال:'},'\U0001F684':{'en':':high-speed_train:','status':fully_qualified,'E':0.6,'alias':[':bullettrain_side:', ':high_speed_train:'],'de':':hochgeschwindigkeitszug_mit_spitzer_nase:','es':':tren_de_alta_velocidad:','fr':':tgv:','ja':':新幹線:','ko':':고속열차:','pt':':trem_de_alta_velocidade:','it':':treno_alta_velocità:','fa':':قطار_سریع_السیر:','id':':kereta_ekspres:','zh':':高速列车:','ru':':скоростной_поезд:','tr':':yüksek_hızlı_tren:','ar':':قطار_سريع:'},'\U000026A1':{'en':':high_voltage:','status':fully_qualified,'E':0.6,'alias':[':zap:'],'variant':True,'de':':hochspannung:','es':':alto_voltaje:','fr':':haute_tension:','ja':':高電圧:','ko':':고압_주의:','pt':':alta_tensão:','it':':alta_tensione:','fa':':ولتاژ_بالا:','id':':tegangan_tinggi:','zh':':高压:','ru':':высокое_напряжение:','tr':':yüksek_voltaj:','ar':':جهد_كهربي_عال:'},'\U0001F97E':{'en':':hiking_boot:','status':fully_qualified,'E':11,'de':':wanderstiefel:','es':':bota_de_senderismo:','fr':':chaussure_de_randonnée:','ja':':ハイキングシューズ:','ko':':등산화:','pt':':bota_de_trekking:','it':':stivale_da_trekking:','fa':':چکمه_راه_پیمایی:','id':':sepatu_boot_untuk_mendaki:','zh':':登山鞋:','ru':':треккинговый_ботинок:','tr':':yürüyüş_botu:','ar':':حذاء_لرحلات_المشي:'},'\U0001F6D5':{'en':':hindu_temple:','status':fully_qualified,'E':12,'de':':hindutempel:','es':':templo_hindú:','fr':':temple_hindou:','ja':':ヒンドゥー教の寺院:','ko':':힌두교_사원:','pt':':templo_hindu:','it':':tempio_indù:','fa':':معبد_هندو:','id':':pura:','zh':':印度寺庙:','ru':':индуистский_храм:','tr':':hindu_tapınağı:','ar':':معبد_هندوسي:'},'\U0001F99B':{'en':':hippopotamus:','status':fully_qualified,'E':11,'de':':nilpferd:','es':':hipopótamo:','fr':':hippopotame:','ja':':カバ:','ko':':하마:','pt':':hipopótamo:','it':':ippopotamo:','fa':':اسب_آبی:','id':':kuda_nil:','zh':':河马:','ru':':гиппопотам:','tr':':hipopotam:','ar':':فرس_النهر:'},'\U0001F573\U0000FE0F':{'en':':hole:','status':fully_qualified,'E':0.7,'variant':True,'de':':loch:','es':':agujero:','fr':':trou:','ja':':穴:','ko':':구멍:','pt':':buraco:','it':':buco:','fa':':چاله:','id':':lubang:','zh':':洞:','ru':':дыра:','tr':':delik:','ar':':ثقب:'},'\U0001F573':{'en':':hole:','status':unqualified,'E':0.7,'variant':True,'de':':loch:','es':':agujero:','fr':':trou:','ja':':穴:','ko':':구멍:','pt':':buraco:','it':':buco:','fa':':چاله:','id':':lubang:','zh':':洞:','ru':':дыра:','tr':':delik:','ar':':ثقب:'},'\U00002B55':{'en':':hollow_red_circle:','status':fully_qualified,'E':0.6,'alias':[':o:'],'variant':True,'de':':hohler_roter_kreis:','es':':círculo_rojo_hueco:','fr':':cercle_rouge:','ja':':太い丸:','ko':':크고_진한_동그라미:','pt':':círculo_grande_oco:','it':':cerchio_rosso_vuoto:','fa':':دایره_قرمز_توخالی:','id':':lingkaran_besar_tebal:','zh':':红色空心圆圈:','ru':':красное_кольцо:','tr':':kalın_kırmızı_çember:','ar':':دائرة_كبيرة_سميكة:'},'\U0001F36F':{'en':':honey_pot:','status':fully_qualified,'E':0.6,'de':':honigtopf:','es':':tarro_de_miel:','fr':':pot_de_miel:','ja':':はちみつ:','ko':':꿀:','pt':':pote_de_mel:','it':':barattolo_di_miele:','fa':':ظرف_عسل:','id':':toples_madu:','zh':':蜂蜜:','ru':':мед:','tr':':bal_kabı:','ar':':جرة_عسل:'},'\U0001F41D':{'en':':honeybee:','status':fully_qualified,'E':0.6,'alias':[':bee:'],'de':':biene:','es':':abeja:','fr':':abeille:','ja':':ミツバチ:','ko':':꿀벌:','pt':':abelha:','it':':ape:','fa':':زنبور_عسل:','id':':lebah_madu:','zh':':蜜蜂:','ru':':пчела:','tr':':bal_arısı:','ar':':نحلة:'},'\U0001FA9D':{'en':':hook:','status':fully_qualified,'E':13,'de':':haken:','es':':gancho:','fr':':crochet:','ja':':フック:','ko':':갈고리:','pt':':gancho:','it':':gancio:','fa':':قلاب:','id':':kait:','zh':':挂钩:','ru':':крюк:','tr':':çengel:','ar':':خطاف:'},'\U0001F6A5':{'en':':horizontal_traffic_light:','status':fully_qualified,'E':0.6,'alias':[':traffic_light:'],'de':':horizontale_verkehrsampel:','es':':semáforo_horizontal:','fr':':feu_tricolore_horizontal:','ja':':信号横:','ko':':가로_신호등:','pt':':semáforo_horizontal:','it':':semaforo_orizzontale:','fa':':چراغ_راهنمایی_افقی:','id':':lampu_lalu_lintas_horizontal:','zh':':横向的红绿灯:','ru':':горизонтальный_светофор:','tr':':yatay_trafik_ışıkları:','ar':':إشارة_مرور_أفقية:'},'\U0001F40E':{'en':':horse:','status':fully_qualified,'E':0.6,'alias':[':racehorse:'],'de':':pferd:','es':':caballo:','fr':':cheval:','ja':':馬:','ko':':말:','pt':':cavalo:','it':':cavallo:','fa':':اسب:','id':':kuda:','zh':':马:','ru':':лошадь:','tr':':at:','ar':':حصان:'},'\U0001F434':{'en':':horse_face:','status':fully_qualified,'E':0.6,'alias':[':horse:'],'de':':pferdegesicht:','es':':cara_de_caballo:','fr':':tête_de_cheval:','ja':':馬の顔:','ko':':말_얼굴:','pt':':rosto_de_cavalo:','it':':muso_di_cavallo:','fa':':صورت_اسب:','id':':wajah_kuda:','zh':':马头:','ru':':морда_лошади:','tr':':at_yüzü:','ar':':وجه_حصان:'},'\U0001F3C7':{'en':':horse_racing:','status':fully_qualified,'E':1,'de':':pferderennen:','es':':carrera_de_caballos:','fr':':course_hippique:','ja':':競馬:','ko':':승마:','pt':':corrida_de_cavalos:','it':':ippica:','fa':':مسابقه_اسب_سواری:','id':':balap_kuda:','zh':':赛马:','ru':':скачки:','tr':':at_yarışı:','ar':':سباق_خيول:'},'\U0001F3C7\U0001F3FF':{'en':':horse_racing_dark_skin_tone:','status':fully_qualified,'E':1,'de':':pferderennen_dunkle_hautfarbe:','es':':carrera_de_caballos_tono_de_piel_oscuro:','fr':':course_hippique_peau_foncée:','ja':':競馬_濃い肌色:','ko':':승마_검은색_피부:','pt':':corrida_de_cavalos_pele_escura:','it':':ippica_carnagione_scura:','fa':':مسابقه_اسب_سواری_پوست_آبنوسی:','id':':balap_kuda_warna_kulit_gelap:','zh':':赛马_较深肤色:','ru':':скачки_очень_темный_тон_кожи:','tr':':at_yarışı_koyu_cilt_tonu:','ar':':سباق_خيول_بشرة_بلون_غامق:'},'\U0001F3C7\U0001F3FB':{'en':':horse_racing_light_skin_tone:','status':fully_qualified,'E':1,'de':':pferderennen_helle_hautfarbe:','es':':carrera_de_caballos_tono_de_piel_claro:','fr':':course_hippique_peau_claire:','ja':':競馬_薄い肌色:','ko':':승마_하얀_피부:','pt':':corrida_de_cavalos_pele_clara:','it':':ippica_carnagione_chiara:','fa':':مسابقه_اسب_سواری_پوست_سفید:','id':':balap_kuda_warna_kulit_cerah:','zh':':赛马_较浅肤色:','ru':':скачки_очень_светлый_тон_кожи:','tr':':at_yarışı_açık_cilt_tonu:','ar':':سباق_خيول_بشرة_بلون_فاتح:'},'\U0001F3C7\U0001F3FE':{'en':':horse_racing_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':pferderennen_mitteldunkle_hautfarbe:','es':':carrera_de_caballos_tono_de_piel_oscuro_medio:','fr':':course_hippique_peau_mate:','ja':':競馬_やや濃い肌色:','ko':':승마_진한_갈색_피부:','pt':':corrida_de_cavalos_pele_morena_escura:','it':':ippica_carnagione_abbastanza_scura:','fa':':مسابقه_اسب_سواری_پوست_گندمی:','id':':balap_kuda_warna_kulit_gelap-sedang:','zh':':赛马_中等深肤色:','ru':':скачки_темный_тон_кожи:','tr':':at_yarışı_orta_koyu_cilt_tonu:','ar':':سباق_خيول_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3C7\U0001F3FC':{'en':':horse_racing_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':pferderennen_mittelhelle_hautfarbe:','es':':carrera_de_caballos_tono_de_piel_claro_medio:','fr':':course_hippique_peau_moyennement_claire:','ja':':競馬_やや薄い肌色:','ko':':승마_연한_갈색_피부:','pt':':corrida_de_cavalos_pele_morena_clara:','it':':ippica_carnagione_abbastanza_chiara:','fa':':مسابقه_اسب_سواری_پوست_روشن:','id':':balap_kuda_warna_kulit_cerah-sedang:','zh':':赛马_中等浅肤色:','ru':':скачки_светлый_тон_кожи:','tr':':at_yarışı_orta_açık_cilt_tonu:','ar':':سباق_خيول_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3C7\U0001F3FD':{'en':':horse_racing_medium_skin_tone:','status':fully_qualified,'E':1,'de':':pferderennen_mittlere_hautfarbe:','es':':carrera_de_caballos_tono_de_piel_medio:','fr':':course_hippique_peau_légèrement_mate:','ja':':競馬_中間の肌色:','ko':':승마_갈색_피부:','pt':':corrida_de_cavalos_pele_morena:','it':':ippica_carnagione_olivastra:','fa':':مسابقه_اسب_سواری_پوست_طلایی:','id':':balap_kuda_warna_kulit_sedang:','zh':':赛马_中等肤色:','ru':':скачки_средний_тон_кожи:','tr':':at_yarışı_orta_cilt_tonu:','ar':':سباق_خيول_بشرة_بلون_معتدل:'},'\U0001F3E5':{'en':':hospital:','status':fully_qualified,'E':0.6,'de':':krankenhaus:','es':':hospital:','fr':':hôpital:','ja':':病院:','ko':':병원:','pt':':hospital:','it':':ospedale:','fa':':بیمارستان:','id':':rumah_sakit:','zh':':医院:','ru':':больница:','tr':':hastane:','ar':':مستشفى:'},'\U00002615':{'en':':hot_beverage:','status':fully_qualified,'E':0.6,'alias':[':coffee:'],'variant':True,'de':':heißgetränk:','es':':bebida_caliente:','fr':':boisson_chaude:','ja':':温かい飲み物:','ko':':뜨거운_음료:','pt':':café:','it':':bevanda_calda:','fa':':نوشیدنی_گرم:','id':':minuman_hangat:','zh':':热饮:','ru':':горячий_напиток:','tr':':sıcak_içecek:','ar':':مشروب_ساخن:'},'\U0001F32D':{'en':':hot_dog:','status':fully_qualified,'E':1,'alias':[':hotdog:'],'de':':hotdog:','es':':perrito_caliente:','fr':':hot_dog:','ja':':ホットドッグ:','ko':':핫도그:','pt':':cachorro-quente:','it':':hot_dog:','fa':':هات_داگ:','id':':hot_dog:','zh':':热狗:','ru':':хот_дог:','tr':':sosisli_sandviç:','ar':':هوت_دوغ:'},'\U0001F975':{'en':':hot_face:','status':fully_qualified,'E':11,'de':':schwitzendes_gesicht:','es':':cara_con_calor:','fr':':visage_rouge_et_chaud:','ja':':暑い顔:','ko':':더운_얼굴:','pt':':rosto_fervendo_de_calor:','it':':faccina_accaldata:','fa':':خیلی_گرم:','id':':wajah_kepanasan:','zh':':脸发烧:','ru':':жар:','tr':':kızarmış_yüz:','ar':':وجه_حار:'},'\U0001F336\U0000FE0F':{'en':':hot_pepper:','status':fully_qualified,'E':0.7,'variant':True,'de':':peperoni:','es':':chile_picante:','fr':':piment_rouge:','ja':':とうがらし:','ko':':홍고추:','pt':':pimenta:','it':':peperoncino:','fa':':فلفل_قرمز:','id':':cabai_pedas:','zh':':红辣椒:','ru':':острыи_перец:','tr':':acı_biber:','ar':':فلفل_حار:'},'\U0001F336':{'en':':hot_pepper:','status':unqualified,'E':0.7,'variant':True,'de':':peperoni:','es':':chile_picante:','fr':':piment_rouge:','ja':':とうがらし:','ko':':홍고추:','pt':':pimenta:','it':':peperoncino:','fa':':فلفل_قرمز:','id':':cabai_pedas:','zh':':红辣椒:','ru':':острый_перец:','tr':':acı_biber:','ar':':فلفل_حار:'},'\U00002668\U0000FE0F':{'en':':hot_springs:','status':fully_qualified,'E':0.6,'alias':[':hotsprings:'],'variant':True,'de':':heiße_quellen:','es':':aguas_termales:','fr':':sources_chaudes:','ja':':温泉マーク:','ko':':온천:','pt':':chamas:','it':':sorgenti_termali:','fa':':چشمه_آب_گرم:','id':':sumber_air_panas:','zh':':温泉:','ru':':горячие_источники:','tr':':kaplıca:','ar':':ينابيع_ساخنة:'},'\U00002668':{'en':':hot_springs:','status':unqualified,'E':0.6,'alias':[':hotsprings:'],'variant':True,'de':':heiße_quellen:','es':':aguas_termales:','fr':':sources_chaudes:','ja':':温泉マーク:','ko':':온천:','pt':':chamas:','it':':sorgenti_termali:','fa':':چشمهٔ_آب_گرم:','id':':sumber_air_panas:','zh':':温泉:','ru':':горячие_источники:','tr':':kaplıca:','ar':':ينابيع_ساخنة:'},'\U0001F3E8':{'en':':hotel:','status':fully_qualified,'E':0.6,'de':':hotel:','es':':hotel:','fr':':hôtel:','ja':':ホテル:','ko':':호텔:','pt':':hotel:','it':':hotel:','fa':':هتل:','id':':hotel:','zh':':酒店:','ru':':гостиница:','tr':':otel:','ar':':فندق:'},'\U0000231B':{'en':':hourglass_done:','status':fully_qualified,'E':0.6,'alias':[':hourglass:'],'variant':True,'de':':sanduhr:','es':':reloj_de_arena_sin_tiempo:','fr':':sablier:','ja':':砂時計:','ko':':시간이_다_된_모래시계:','pt':':ampulheta:','it':':clessidra:','fa':':ساعت_شنی:','id':':jam_pasir:','zh':':沙漏:','ru':':песочные_часы:','tr':':kum_saati:','ar':':مؤقت_رملي_مكتمل:'},'\U000023F3':{'en':':hourglass_not_done:','status':fully_qualified,'E':0.6,'alias':[':hourglass_flowing_sand:'],'variant':True,'de':':laufende_sanduhr:','es':':reloj_de_arena_con_tiempo:','fr':':sablier_avec_sable_qui_coule:','ja':':砂が落ちている砂時計:','ko':':시간이_남은_모래시계:','pt':':ampulheta_contando_o_tempo:','it':':clessidra_che_scorre:','fa':':ساعت_شنی_با_شن_روان:','id':':jam_pasir_yang_bergerak:','zh':':沙正往下流的沙漏:','ru':':время_пошло:','tr':':akan_kum_saati:','ar':':مؤقت_رملي_غير_مكتمل:'},'\U0001F3E0':{'en':':house:','status':fully_qualified,'E':0.6,'variant':True,'de':':haus:','es':':casa:','fr':':maison:','ja':':家:','ko':':집:','pt':':casa:','it':':casa:','fa':':خانه:','id':':rumah:','zh':':房子:','ru':':дом:','tr':':ev:','ar':':منزل:'},'\U0001F3E1':{'en':':house_with_garden:','status':fully_qualified,'E':0.6,'de':':haus_mit_garten:','es':':casa_con_jardín:','fr':':maison_avec_jardin:','ja':':庭付きの家:','ko':':정원이_있는_집:','pt':':casa_com_jardim:','it':':casa_con_giardino:','fa':':خانهٔ_حیاط_دار:','id':':rumah_dengan_taman:','zh':':别墅:','ru':':дом_с_садом:','tr':':bahçeli_ev:','ar':':منزل_بحديقة:'},'\U0001F3D8\U0000FE0F':{'en':':houses:','status':fully_qualified,'E':0.7,'alias':[':house_buildings:'],'variant':True,'de':':wohnhäuser:','es':':casas:','fr':':maisons:','ja':':住宅街:','ko':':주택_건물:','pt':':casas:','it':':case:','fa':':منازل:','id':':perumahan:','zh':':房屋建筑:','ru':':дома:','tr':':evler:','ar':':منازل:'},'\U0001F3D8':{'en':':houses:','status':unqualified,'E':0.7,'alias':[':house_buildings:'],'variant':True,'de':':wohnhäuser:','es':':casas:','fr':':maisons:','ja':':住宅街:','ko':':주택_건물:','pt':':casas:','it':':case:','fa':':منازل:','id':':perumahan:','zh':':房屋建筑:','ru':':дома:','tr':':evler:','ar':':منازل:'},'\U0001F4AF':{'en':':hundred_points:','status':fully_qualified,'E':0.6,'alias':[':100:'],'de':':100_punkte:','es':':cien_puntos:','fr':':cent_points:','ja':':100点満点:','ko':':백점:','pt':':cem_pontos:','it':':100_punti:','fa':':۱۰۰_امتیاز:','id':':nilai_seratus:','zh':':一百分:','ru':':сто_баллов:','tr':':yüz_puan:','ar':':100_درجة:'},'\U0001F62F':{'en':':hushed_face:','status':fully_qualified,'E':1,'alias':[':hushed:'],'de':':verdutztes_gesicht:','es':':cara_estupefacta:','fr':':visage_ébahi:','ja':':ぽかーん:','ko':':숨죽인_얼굴:','pt':':rosto_surpreso:','it':':faccina_sorpresa:','fa':':مات_و_مبهوت:','id':':wajah_hening:','zh':':缄默:','ru':':неприятный_сюрприз:','tr':':şaşkın_yüz:','ar':':وجه_مخفوت_الصوت:'},'\U0001F6D6':{'en':':hut:','status':fully_qualified,'E':13,'de':':hütte:','es':':cabaña:','fr':':hutte:','ja':':わらぶき小屋:','ko':':오두막:','pt':':cabana:','it':':capanna:','fa':':کلبه:','id':':pondok:','zh':':小屋:','ru':':хижина:','tr':':kulübe:','ar':':كوخ:'},'\U0001FABB':{'en':':hyacinth:','status':fully_qualified,'E':15,'de':':hyazinthe:','es':':campanilla:','fr':':jacinthe:','ja':':ヒヤシンス:','ko':':히아신스:','pt':':jacinto:','it':':giacinto:','fa':':سنبل:','id':':hyacinthus:','zh':':风信子:','ru':':гиацинт:','tr':':sümbül:','ar':':زهرة_الياقوتية:'},'\U0001F9CA':{'en':':ice:','status':fully_qualified,'E':12,'alias':[':ice_cube:'],'de':':eiswürfel:','es':':cubito_de_hielo:','fr':':glaçon:','ja':':角氷:','ko':':얼음:','pt':':cubo_de_gelo:','it':':cubetto_di_ghiaccio:','fa':':قالب_یخ:','id':':es_batu:','zh':':冰块:','ru':':кубик_льда:','tr':':buz_küpü:','ar':':مكعب_ثلج:'},'\U0001F368':{'en':':ice_cream:','status':fully_qualified,'E':0.6,'de':':eiscreme:','es':':helado:','fr':':glace:','ja':':アイスクリーム:','ko':':아이스크림:','pt':':sorvete:','it':':coppa_di_gelato:','fa':':بستنی:','id':':es_krim:','zh':':冰淇淋:','ru':':мороженое:','tr':':dondurma:','ar':':آيس_كريم:'},'\U0001F3D2':{'en':':ice_hockey:','status':fully_qualified,'E':1,'alias':[':ice_hockey_stick_and_puck:'],'de':':eishockey:','es':':hockey_sobre_hielo:','fr':':hockey_sur_glace:','ja':':アイスホッケー:','ko':':아이스_하키와_퍽:','pt':':hóquei_no_gelo:','it':':hockey_su_ghiaccio:','fa':':هاکی_روی_یخ:','id':':hoki_es:','zh':':冰球:','ru':':хоккей:','tr':':buz_hokeyi:','ar':':عصا_هوكي_جليد:'},'\U000026F8\U0000FE0F':{'en':':ice_skate:','status':fully_qualified,'E':0.7,'variant':True,'de':':schlittschuh:','es':':patín_de_hielo:','fr':':patin_à_glace:','ja':':アイススケート:','ko':':아이스_스케이트:','pt':':patins_de_gelo:','it':':pattinaggio_su_ghiaccio:','fa':':اسکی_روی_یخ:','id':':seluncur_es:','zh':':滑冰:','ru':':коньки:','tr':':buz_pateni:','ar':':حذاء_تزلج:'},'\U000026F8':{'en':':ice_skate:','status':unqualified,'E':0.7,'variant':True,'de':':schlittschuh:','es':':patín_de_hielo:','fr':':patin_à_glace:','ja':':アイススケート:','ko':':아이스_스케이트:','pt':':patins_de_gelo:','it':':pattinaggio_su_ghiaccio:','fa':':اسکی_روی_یخ:','id':':seluncur_es:','zh':':滑冰:','ru':':коньки:','tr':':buz_pateni:','ar':':حذاء_تزلج:'},'\U0001FAAA':{'en':':identification_card:','status':fully_qualified,'E':14,'de':':ausweis:','es':':carné_de_identidad:','fr':':carte_d’identité:','ja':':身分証:','ko':':신분증:','pt':':cartão_de_identificação:','it':':carta_d’identità:','fa':':کارت_شناسایی:','id':':kartu_identitas:','zh':':身份证:','ru':':удостоверение_личности:','tr':':kimlik_kartı:','ar':':بطاقة_الهوية:'},'\U0001F4E5':{'en':':inbox_tray:','status':fully_qualified,'E':0.6,'variant':True,'de':':posteingang:','es':':bandeja_de_entrada:','fr':':boîte_de_réception:','ja':':受信トレイ:','ko':':받은_편지함:','pt':':bandeja_de_entrada:','it':':posta_ricevuta:','fa':':صندوق_ورودی:','id':':baki_kotak_masuk:','zh':':收件箱:','ru':':входящие:','tr':':gelen_kutusu:','ar':':صندوق_وارد:'},'\U0001F4E8':{'en':':incoming_envelope:','status':fully_qualified,'E':0.6,'de':':eingehender_briefumschlag:','es':':sobre_entrante:','fr':':message_reçu:','ja':':メール受信中:','ko':':받은_편지:','pt':':envelope_chegando:','it':':posta_in_arrivo:','fa':':پاکت_نامهٔ_ورودی:','id':':amplop_datang:','zh':':来信:','ru':':входящее_письмо:','tr':':gelen_zarf:','ar':':بريد_وارد:'},'\U0001FAF5':{'en':':index_pointing_at_the_viewer:','status':fully_qualified,'E':14,'de':':auf_betrachter_zeigender_zeigefinger:','es':':dedo_índice_apuntándote_a_ti:','fr':':index_pointant_vers_l’utilisateur:','ja':':人を指差している手:','ko':':보는_사람을_가리키는_검지:','pt':':indicador_apontando_para_o_visualizador:','it':':indice_verso_l’osservatore:','fa':':اشاره_به_بیننده:','id':':jari_telunjuk_menunjuk_penonton:','zh':':指向观察者的食指:','ru':':указательный_палец_на_себя:','tr':':karşıyı_işaret_eden_işaret_parmağı:','ar':':إصبع_السبابة_يشير_إلى_الناظر:'},'\U0001FAF5\U0001F3FF':{'en':':index_pointing_at_the_viewer_dark_skin_tone:','status':fully_qualified,'E':14,'de':':auf_betrachter_zeigender_zeigefinger_dunkle_hautfarbe:','es':':dedo_índice_apuntándote_a_ti_tono_de_piel_oscuro:','fr':':index_pointant_vers_l’utilisateur_peau_foncée:','ja':':人を指差している手_濃い肌色:','ko':':보는_사람을_가리키는_검지_검은색_피부:','pt':':indicador_apontando_para_o_visualizador_pele_escura:','it':':indice_verso_l’osservatore_carnagione_scura:','fa':':اشاره_به_بیننده_پوست_آبنوسی:','id':':jari_telunjuk_menunjuk_penonton_warna_kulit_gelap:','zh':':指向观察者的食指_较深肤色:','ru':':указательный_палец_на_себя_очень_темный_тон_кожи:','tr':':karşıyı_işaret_eden_işaret_parmağı_koyu_cilt_tonu:','ar':':إصبع_السبابة_يشير_إلى_الناظر_بشرة_بلون_غامق:'},'\U0001FAF5\U0001F3FB':{'en':':index_pointing_at_the_viewer_light_skin_tone:','status':fully_qualified,'E':14,'de':':auf_betrachter_zeigender_zeigefinger_helle_hautfarbe:','es':':dedo_índice_apuntándote_a_ti_tono_de_piel_claro:','fr':':index_pointant_vers_l’utilisateur_peau_claire:','ja':':人を指差している手_薄い肌色:','ko':':보는_사람을_가리키는_검지_하얀_피부:','pt':':indicador_apontando_para_o_visualizador_pele_clara:','it':':indice_verso_l’osservatore_carnagione_chiara:','fa':':اشاره_به_بیننده_پوست_سفید:','id':':jari_telunjuk_menunjuk_penonton_warna_kulit_cerah:','zh':':指向观察者的食指_较浅肤色:','ru':':указательный_палец_на_себя_очень_светлый_тон_кожи:','tr':':karşıyı_işaret_eden_işaret_parmağı_açık_cilt_tonu:','ar':':إصبع_السبابة_يشير_إلى_الناظر_بشرة_بلون_فاتح:'},'\U0001FAF5\U0001F3FE':{'en':':index_pointing_at_the_viewer_medium-dark_skin_tone:','status':fully_qualified,'E':14,'de':':auf_betrachter_zeigender_zeigefinger_mitteldunkle_hautfarbe:','es':':dedo_índice_apuntándote_a_ti_tono_de_piel_oscuro_medio:','fr':':index_pointant_vers_l’utilisateur_peau_mate:','ja':':人を指差している手_やや濃い肌色:','ko':':보는_사람을_가리키는_검지_진한_갈색_피부:','pt':':indicador_apontando_para_o_visualizador_pele_morena_escura:','it':':indice_verso_l’osservatore_carnagione_abbastanza_scura:','fa':':اشاره_به_بیننده_پوست_گندمی:','id':':jari_telunjuk_menunjuk_penonton_warna_kulit_gelap-sedang:','zh':':指向观察者的食指_中等深肤色:','ru':':указательный_палец_на_себя_темный_тон_кожи:','tr':':karşıyı_işaret_eden_işaret_parmağı_orta_koyu_cilt_tonu:','ar':':إصبع_السبابة_يشير_إلى_الناظر_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001FAF5\U0001F3FC':{'en':':index_pointing_at_the_viewer_medium-light_skin_tone:','status':fully_qualified,'E':14,'de':':auf_betrachter_zeigender_zeigefinger_mittelhelle_hautfarbe:','es':':dedo_índice_apuntándote_a_ti_tono_de_piel_claro_medio:','fr':':index_pointant_vers_l’utilisateur_peau_moyennement_claire:','ja':':人を指差している手_やや薄い肌色:','ko':':보는_사람을_가리키는_검지_연한_갈색_피부:','pt':':indicador_apontando_para_o_visualizador_pele_morena_clara:','it':':indice_verso_l’osservatore_carnagione_abbastanza_chiara:','fa':':اشاره_به_بیننده_پوست_روشن:','id':':jari_telunjuk_menunjuk_penonton_warna_kulit_cerah-sedang:','zh':':指向观察者的食指_中等浅肤色:','ru':':указательный_палец_на_себя_светлый_тон_кожи:','tr':':karşıyı_işaret_eden_işaret_parmağı_orta_açık_cilt_tonu:','ar':':إصبع_السبابة_يشير_إلى_الناظر_بشرة_بلون_فاتح_ومعتدل:'},'\U0001FAF5\U0001F3FD':{'en':':index_pointing_at_the_viewer_medium_skin_tone:','status':fully_qualified,'E':14,'de':':auf_betrachter_zeigender_zeigefinger_mittlere_hautfarbe:','es':':dedo_índice_apuntándote_a_ti_tono_de_piel_medio:','fr':':index_pointant_vers_l’utilisateur_peau_légèrement_mate:','ja':':人を指差している手_中間の肌色:','ko':':보는_사람을_가리키는_검지_갈색_피부:','pt':':indicador_apontando_para_o_visualizador_pele_morena:','it':':indice_verso_l’osservatore_carnagione_olivastra:','fa':':اشاره_به_بیننده_پوست_طلایی:','id':':jari_telunjuk_menunjuk_penonton_warna_kulit_sedang:','zh':':指向观察者的食指_中等肤色:','ru':':указательный_палец_на_себя_средний_тон_кожи:','tr':':karşıyı_işaret_eden_işaret_parmağı_orta_cilt_tonu:','ar':':إصبع_السبابة_يشير_إلى_الناظر_بشرة_بلون_معتدل:'},'\U0000261D\U0000FE0F':{'en':':index_pointing_up:','status':fully_qualified,'E':0.6,'alias':[':point_up:'],'variant':True,'de':':nach_oben_weisender_zeigefinger_von_vorne:','es':':dedo_índice_hacia_arriba:','fr':':index_pointant_vers_le_haut:','ja':':上指差し:','ko':':위쪽을_가리키는_손:','pt':':indicador_apontando_para_cima:','it':':indice_verso_l’alto:','fa':':اشاره_به_بالا:','id':':menunjuk_ke_atas:','zh':':食指向上指:','ru':':указательныи_палец_вверх:','tr':':yukarıyı_gösteren_işaret_parmağı:','ar':':سبابة_يشير_لأعلى:'},'\U0000261D':{'en':':index_pointing_up:','status':unqualified,'E':0.6,'alias':[':point_up:'],'variant':True,'de':':nach_oben_weisender_zeigefinger_von_vorne:','es':':dedo_índice_hacia_arriba:','fr':':index_pointant_vers_le_haut:','ja':':上指差し:','ko':':위쪽을_가리키는_손:','pt':':indicador_apontando_para_cima:','it':':indice_verso_l’alto:','fa':':اشاره_به_بالا:','id':':menunjuk_ke_atas:','zh':':食指向上指:','ru':':указательный_палец_вверх:','tr':':yukarıyı_gösteren_işaret_parmağı:','ar':':سبابة_يشير_لأعلى:'},'\U0000261D\U0001F3FF':{'en':':index_pointing_up_dark_skin_tone:','status':fully_qualified,'E':1,'de':':nach_oben_weisender_zeigefinger_von_vorne_dunkle_hautfarbe:','es':':dedo_índice_hacia_arriba_tono_de_piel_oscuro:','fr':':index_pointant_vers_le_haut_peau_foncée:','ja':':上指差し_濃い肌色:','ko':':위쪽을_가리키는_손_검은색_피부:','pt':':indicador_apontando_para_cima_pele_escura:','it':':indice_verso_l’alto_carnagione_scura:','fa':':اشاره_به_بالا_پوست_آبنوسی:','id':':menunjuk_ke_atas_warna_kulit_gelap:','zh':':食指向上指_较深肤色:','ru':':указательный_палец_вверх_очень_темный_тон_кожи:','tr':':yukarıyı_gösteren_işaret_parmağı_koyu_cilt_tonu:','ar':':سبابة_يشير_لأعلى_بشرة_بلون_غامق:'},'\U0000261D\U0001F3FB':{'en':':index_pointing_up_light_skin_tone:','status':fully_qualified,'E':1,'de':':nach_oben_weisender_zeigefinger_von_vorne_helle_hautfarbe:','es':':dedo_índice_hacia_arriba_tono_de_piel_claro:','fr':':index_pointant_vers_le_haut_peau_claire:','ja':':上指差し_薄い肌色:','ko':':위쪽을_가리키는_손_하얀_피부:','pt':':indicador_apontando_para_cima_pele_clara:','it':':indice_verso_l’alto_carnagione_chiara:','fa':':اشاره_به_بالا_پوست_سفید:','id':':menunjuk_ke_atas_warna_kulit_cerah:','zh':':食指向上指_较浅肤色:','ru':':указательный_палец_вверх_очень_светлый_тон_кожи:','tr':':yukarıyı_gösteren_işaret_parmağı_açık_cilt_tonu:','ar':':سبابة_يشير_لأعلى_بشرة_بلون_فاتح:'},'\U0000261D\U0001F3FE':{'en':':index_pointing_up_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':nach_oben_weisender_zeigefinger_von_vorne_mitteldunkle_hautfarbe:','es':':dedo_índice_hacia_arriba_tono_de_piel_oscuro_medio:','fr':':index_pointant_vers_le_haut_peau_mate:','ja':':上指差し_やや濃い肌色:','ko':':위쪽을_가리키는_손_진한_갈색_피부:','pt':':indicador_apontando_para_cima_pele_morena_escura:','it':':indice_verso_l’alto_carnagione_abbastanza_scura:','fa':':اشاره_به_بالا_پوست_گندمی:','id':':menunjuk_ke_atas_warna_kulit_gelap-sedang:','zh':':食指向上指_中等深肤色:','ru':':указательный_палец_вверх_темный_тон_кожи:','tr':':yukarıyı_gösteren_işaret_parmağı_orta_koyu_cilt_tonu:','ar':':سبابة_يشير_لأعلى_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0000261D\U0001F3FC':{'en':':index_pointing_up_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':nach_oben_weisender_zeigefinger_von_vorne_mittelhelle_hautfarbe:','es':':dedo_índice_hacia_arriba_tono_de_piel_claro_medio:','fr':':index_pointant_vers_le_haut_peau_moyennement_claire:','ja':':上指差し_やや薄い肌色:','ko':':위쪽을_가리키는_손_연한_갈색_피부:','pt':':indicador_apontando_para_cima_pele_morena_clara:','it':':indice_verso_l’alto_carnagione_abbastanza_chiara:','fa':':اشاره_به_بالا_پوست_روشن:','id':':menunjuk_ke_atas_warna_kulit_cerah-sedang:','zh':':食指向上指_中等浅肤色:','ru':':указательный_палец_вверх_светлый_тон_кожи:','tr':':yukarıyı_gösteren_işaret_parmağı_orta_açık_cilt_tonu:','ar':':سبابة_يشير_لأعلى_بشرة_بلون_فاتح_ومعتدل:'},'\U0000261D\U0001F3FD':{'en':':index_pointing_up_medium_skin_tone:','status':fully_qualified,'E':1,'de':':nach_oben_weisender_zeigefinger_von_vorne_mittlere_hautfarbe:','es':':dedo_índice_hacia_arriba_tono_de_piel_medio:','fr':':index_pointant_vers_le_haut_peau_légèrement_mate:','ja':':上指差し_中間の肌色:','ko':':위쪽을_가리키는_손_갈색_피부:','pt':':indicador_apontando_para_cima_pele_morena:','it':':indice_verso_l’alto_carnagione_olivastra:','fa':':اشاره_به_بالا_پوست_طلایی:','id':':menunjuk_ke_atas_warna_kulit_sedang:','zh':':食指向上指_中等肤色:','ru':':указательный_палец_вверх_средний_тон_кожи:','tr':':yukarıyı_gösteren_işaret_parmağı_orta_cilt_tonu:','ar':':سبابة_يشير_لأعلى_بشرة_بلون_معتدل:'},'\U0000267E\U0000FE0F':{'en':':infinity:','status':fully_qualified,'E':11,'variant':True,'de':':unendlichkeit:','es':':infinito:','fr':':infini:','ja':':無限大:','ko':':무한:','pt':':infinito:','it':':simbolo_dell’infinito:','fa':':بی_نهایت:','id':':tak_terhingga:','zh':':无穷大:','ru':':бесконечность:','tr':':sonsuz:','ar':':اللا_نهاية:'},'\U0000267E':{'en':':infinity:','status':unqualified,'E':11,'variant':True,'de':':unendlichkeit:','es':':infinito:','fr':':infini:','ja':':無限大:','ko':':무한:','pt':':infinito:','it':':simbolo_dell’infinito:','fa':':بی_نهایت:','id':':tak_terhingga:','zh':':无穷大:','ru':':бесконечность:','tr':':sonsuz:','ar':':اللا_نهاية:'},'\U00002139\U0000FE0F':{'en':':information:','status':fully_qualified,'E':0.6,'alias':[':information_source:'],'variant':True,'de':':buchstabe_i_in_blauem_quadrat:','es':':información:','fr':':source_d’informations:','ja':':iマーク:','ko':':정보_출처:','pt':':informações:','it':':punto_informazioni:','fa':':اطلاعات:','id':':informasi:','zh':':信息:','ru':':значок_информации:','tr':':bilgi:','ar':':مصدر_معلومات:'},'\U00002139':{'en':':information:','status':unqualified,'E':0.6,'alias':[':information_source:'],'variant':True,'de':':buchstabe_i_in_blauem_quadrat:','es':':información:','fr':':source_d’informations:','ja':':iマーク:','ko':':정보_출처:','pt':':informações:','it':':punto_informazioni:','fa':':اطلاعات:','id':':informasi:','zh':':信息:','ru':':значок_информации:','tr':':bilgi:','ar':':مصدر_معلومات:'},'\U0001F524':{'en':':input_latin_letters:','status':fully_qualified,'E':0.6,'alias':[':abc:'],'de':':eingabesymbol_lateinische_buchstaben:','es':':alfabeto_latino:','fr':':alphabet_latin:','ja':':英字の入力:','ko':':라틴어_알파벳:','pt':':letras_latinas:','it':':pulsante_con_lettere_latine:','fa':':ورودی_حروف_لاتین:','id':':masukan_huruf_latin:','zh':':输入拉丁字母:','ru':':ввод_латиницей:','tr':':latin_alfabesi:','ar':':أبجدية_لاتينية:'},'\U0001F521':{'en':':input_latin_lowercase:','status':fully_qualified,'E':0.6,'alias':[':abcd:'],'de':':eingabesymbol_lateinische_kleinbuchstaben:','es':':letras_latinas_minúsculas:','fr':':minuscules:','ja':':英小文字の入力:','ko':':소문자:','pt':':letras_latinas_minúsculas:','it':':pulsante_con_lettere_latine_minuscole:','fa':':ورودی_حروف_کوچک:','id':':masukan_huruf_kecil_latin:','zh':':输入小写拉丁字母:','ru':':ввод_строчными:','tr':':küçük_harfler:','ar':':أحرف_صغيرة:'},'\U0001F520':{'en':':input_latin_uppercase:','status':fully_qualified,'E':0.6,'alias':[':capital_abcd:'],'de':':eingabesymbol_lateinische_großbuchstaben:','es':':letras_latinas_mayúsculas:','fr':':majuscules:','ja':':英大文字の入力:','ko':':대문자:','pt':':letras_latinas_maiúsculas:','it':':pulsante_con_lettere_latine_maiuscole:','fa':':ورودی_حروف_بزرگ:','id':':masukan_huruf_kapital_latin:','zh':':输入大写拉丁字母:','ru':':ввод_прописными:','tr':':büyük_harfler:','ar':':أحرف_كبيرة:'},'\U0001F522':{'en':':input_numbers:','status':fully_qualified,'E':0.6,'alias':[':1234:'],'de':':eingabesymbol_zahlen:','es':':números:','fr':':saisie_de_chiffres:','ja':':数字の入力:','ko':':숫자:','pt':':números:','it':':pulsante_con_numeri:','fa':':ورودی_عددی:','id':':masukan_angka:','zh':':输入数字:','ru':':ввод_цифр:','tr':':sayılar:','ar':':أرقام:'},'\U0001F523':{'en':':input_symbols:','status':fully_qualified,'E':0.6,'alias':[':symbols:'],'de':':eingabesymbol_sonderzeichen:','es':':símbolos:','fr':':saisie_de_symboles:','ja':':記号の入力:','ko':':기호:','pt':':símbolos:','it':':pulsante_con_simboli:','fa':':ورودی_نماد:','id':':masukan_simbol:','zh':':输入符号:','ru':':ввод_символов:','tr':':işaretler:','ar':':رموز:'},'\U0001F383':{'en':':jack-o-lantern:','status':fully_qualified,'E':0.6,'alias':[':jack_o_lantern:'],'de':':halloweenkürbis:','es':':calabaza_de_halloween:','fr':':citrouille:','ja':':ハロウィンかぼちゃ:','ko':':할로윈:','pt':':abóbora_de_halloween:','it':':zucca_di_halloween:','fa':':فانوس_کدو_تنبل:','id':':jack-o-lantern:','zh':':南瓜灯:','ru':':тыква_фонарь:','tr':':kabak_fener:','ar':':هالوين:'},'\U0001FAD9':{'en':':jar:','status':fully_qualified,'E':14,'de':':einmachglas:','es':':tarro:','fr':':bocal:','ja':':瓶:','ko':':작은_병:','pt':':jarro:','it':':barattolo:','fa':':شیشه_دهان_گشاد:','id':':toples:','zh':':罐:','ru':':банка:','tr':':kavanoz:','ar':':برطمان:'},'\U0001F456':{'en':':jeans:','status':fully_qualified,'E':0.6,'de':':jeans:','es':':vaqueros:','fr':':jean:','ja':':ジーンズ:','ko':':바지:','pt':':jeans:','it':':jeans:','fa':':شلوار_جین:','id':':celana_jeans:','zh':':牛仔裤:','ru':':джинсы:','tr':':kot:','ar':':بنطلون_جينز:'},'\U0001FABC':{'en':':jellyfish:','status':fully_qualified,'E':15,'de':':qualle:','es':':medusa:','fr':':méduse:','ja':':クラゲ:','ko':':해파리:','pt':':água-viva:','it':':medusa:','fa':':عروس_دریایی:','id':':ubur-ubur:','zh':':水母:','ru':':медуза:','tr':':denizanası:','ar':':قنديل_بحر:'},'\U0001F0CF':{'en':':joker:','status':fully_qualified,'E':0.6,'alias':[':black_joker:'],'de':':jokerkarte:','es':':comodín:','fr':':carte_joker:','ja':':ジョーカー:','ko':':조커:','pt':':curinga:','it':':jolly:','fa':':جوکر:','id':':joker:','zh':':大小王:','ru':':джокер:','tr':':joker:','ar':':جوكر:'},'\U0001F579\U0000FE0F':{'en':':joystick:','status':fully_qualified,'E':0.7,'variant':True,'de':':joystick:','es':':joystick:','fr':':manette_de_jeu:','ja':':ジョイスティック:','ko':':조이스틱:','pt':':joystick:','it':':joystick:','fa':':دسته_بازی:','id':':joystick:','zh':':游戏操控杆:','ru':':джоистик:','tr':':oyun_çubuğu:','ar':':ذراع_تحكم:'},'\U0001F579':{'en':':joystick:','status':unqualified,'E':0.7,'variant':True,'de':':joystick:','es':':joystick:','fr':':manette_de_jeu:','ja':':ジョイスティック:','ko':':조이스틱:','pt':':joystick:','it':':joystick:','fa':':دسته_بازی:','id':':joystick:','zh':':游戏操控杆:','ru':':джойстик:','tr':':oyun_çubuğu:','ar':':ذراع_تحكم:'},'\U0001F9D1\U0000200D\U00002696\U0000FE0F':{'en':':judge:','status':fully_qualified,'E':12.1,'de':':richter(in):','es':':fiscal:','fr':':juge:','ja':':裁判官:','ko':':판사:','pt':':juiz_no_tribunal:','it':':giudice:','fa':':قاضی:','id':':hakim:','zh':':法官:','ru':':судья:','tr':':hakim:','ar':':قاضي:'},'\U0001F9D1\U0000200D\U00002696':{'en':':judge:','status':minimally_qualified,'E':12.1,'de':':richter(in):','es':':fiscal:','fr':':juge:','ja':':裁判官:','ko':':판사:','pt':':juiz_no_tribunal:','it':':giudice:','fa':':قاضی:','id':':hakim:','zh':':法官:','ru':':судья:','tr':':hakim:','ar':':قاضي:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002696\U0000FE0F':{'en':':judge_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':richter(in)_dunkle_hautfarbe:','es':':fiscal_tono_de_piel_oscuro:','fr':':juge_peau_foncée:','ja':':裁判官_濃い肌色:','ko':':판사_검은색_피부:','pt':':juiz_no_tribunal_pele_escura:','it':':giudice_carnagione_scura:','fa':':قاضی_پوست_آبنوسی:','id':':hakim_warna_kulit_gelap:','zh':':法官_较深肤色:','ru':':судья_очень_темныи_тон_кожи:','tr':':hakim_koyu_cilt_tonu:','ar':':قاضي_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002696':{'en':':judge_dark_skin_tone:','status':minimally_qualified,'E':12.1,'de':':richter(in)_dunkle_hautfarbe:','es':':fiscal_tono_de_piel_oscuro:','fr':':juge_peau_foncée:','ja':':裁判官_濃い肌色:','ko':':판사_검은색_피부:','pt':':juiz_no_tribunal_pele_escura:','it':':giudice_carnagione_scura:','fa':':قاضی_پوست_آبنوسی:','id':':hakim_warna_kulit_gelap:','zh':':法官_较深肤色:','ru':':судья_очень_темный_тон_кожи:','tr':':hakim_koyu_cilt_tonu:','ar':':قاضي_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002696\U0000FE0F':{'en':':judge_light_skin_tone:','status':fully_qualified,'E':12.1,'de':':richter(in)_helle_hautfarbe:','es':':fiscal_tono_de_piel_claro:','fr':':juge_peau_claire:','ja':':裁判官_薄い肌色:','ko':':판사_하얀_피부:','pt':':juiz_no_tribunal_pele_clara:','it':':giudice_carnagione_chiara:','fa':':قاضی_پوست_سفید:','id':':hakim_warna_kulit_cerah:','zh':':法官_较浅肤色:','ru':':судья_очень_светлыи_тон_кожи:','tr':':hakim_açık_cilt_tonu:','ar':':قاضي_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002696':{'en':':judge_light_skin_tone:','status':minimally_qualified,'E':12.1,'de':':richter(in)_helle_hautfarbe:','es':':fiscal_tono_de_piel_claro:','fr':':juge_peau_claire:','ja':':裁判官_薄い肌色:','ko':':판사_하얀_피부:','pt':':juiz_no_tribunal_pele_clara:','it':':giudice_carnagione_chiara:','fa':':قاضی_پوست_سفید:','id':':hakim_warna_kulit_cerah:','zh':':法官_较浅肤色:','ru':':судья_очень_светлый_тон_кожи:','tr':':hakim_açık_cilt_tonu:','ar':':قاضي_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002696\U0000FE0F':{'en':':judge_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':richter(in)_mitteldunkle_hautfarbe:','es':':fiscal_tono_de_piel_oscuro_medio:','fr':':juge_peau_mate:','ja':':裁判官_やや濃い肌色:','ko':':판사_진한_갈색_피부:','pt':':juiz_no_tribunal_pele_morena_escura:','it':':giudice_carnagione_abbastanza_scura:','fa':':قاضی_پوست_گندمی:','id':':hakim_warna_kulit_gelap-sedang:','zh':':法官_中等深肤色:','ru':':судья_темныи_тон_кожи:','tr':':hakim_orta_koyu_cilt_tonu:','ar':':قاضي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002696':{'en':':judge_medium-dark_skin_tone:','status':minimally_qualified,'E':12.1,'de':':richter(in)_mitteldunkle_hautfarbe:','es':':fiscal_tono_de_piel_oscuro_medio:','fr':':juge_peau_mate:','ja':':裁判官_やや濃い肌色:','ko':':판사_진한_갈색_피부:','pt':':juiz_no_tribunal_pele_morena_escura:','it':':giudice_carnagione_abbastanza_scura:','fa':':قاضی_پوست_گندمی:','id':':hakim_warna_kulit_gelap-sedang:','zh':':法官_中等深肤色:','ru':':судья_темный_тон_кожи:','tr':':hakim_orta_koyu_cilt_tonu:','ar':':قاضي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002696\U0000FE0F':{'en':':judge_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':richter(in)_mittelhelle_hautfarbe:','es':':fiscal_tono_de_piel_claro_medio:','fr':':juge_peau_moyennement_claire:','ja':':裁判官_やや薄い肌色:','ko':':판사_연한_갈색_피부:','pt':':juiz_no_tribunal_pele_morena_clara:','it':':giudice_carnagione_abbastanza_chiara:','fa':':قاضی_پوست_روشن:','id':':hakim_warna_kulit_cerah-sedang:','zh':':法官_中等浅肤色:','ru':':судья_светлыи_тон_кожи:','tr':':hakim_orta_açık_cilt_tonu:','ar':':قاضي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002696':{'en':':judge_medium-light_skin_tone:','status':minimally_qualified,'E':12.1,'de':':richter(in)_mittelhelle_hautfarbe:','es':':fiscal_tono_de_piel_claro_medio:','fr':':juge_peau_moyennement_claire:','ja':':裁判官_やや薄い肌色:','ko':':판사_연한_갈색_피부:','pt':':juiz_no_tribunal_pele_morena_clara:','it':':giudice_carnagione_abbastanza_chiara:','fa':':قاضی_پوست_روشن:','id':':hakim_warna_kulit_cerah-sedang:','zh':':法官_中等浅肤色:','ru':':судья_светлый_тон_кожи:','tr':':hakim_orta_açık_cilt_tonu:','ar':':قاضي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002696\U0000FE0F':{'en':':judge_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':richter(in)_mittlere_hautfarbe:','es':':fiscal_tono_de_piel_medio:','fr':':juge_peau_légèrement_mate:','ja':':裁判官_中間の肌色:','ko':':판사_갈색_피부:','pt':':juiz_no_tribunal_pele_morena:','it':':giudice_carnagione_olivastra:','fa':':قاضی_پوست_طلایی:','id':':hakim_warna_kulit_sedang:','zh':':法官_中等肤色:','ru':':судья_среднии_тон_кожи:','tr':':hakim_orta_cilt_tonu:','ar':':قاضي_بشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002696':{'en':':judge_medium_skin_tone:','status':minimally_qualified,'E':12.1,'de':':richter(in)_mittlere_hautfarbe:','es':':fiscal_tono_de_piel_medio:','fr':':juge_peau_légèrement_mate:','ja':':裁判官_中間の肌色:','ko':':판사_갈색_피부:','pt':':juiz_no_tribunal_pele_morena:','it':':giudice_carnagione_olivastra:','fa':':قاضی_پوست_طلایی:','id':':hakim_warna_kulit_sedang:','zh':':法官_中等肤色:','ru':':судья_средний_тон_кожи:','tr':':hakim_orta_cilt_tonu:','ar':':قاضي_بشرة_بلون_معتدل:'},'\U0001F54B':{'en':':kaaba:','status':fully_qualified,'E':1,'de':':kaaba:','es':':kaaba:','fr':':kaaba:','ja':':カーバ:','ko':':카바:','pt':':caaba_islã:','it':':kaaba:','fa':':کعبه:','id':':ka’bah:','zh':':克尔白:','ru':':Кааба:','tr':':kâbe:','ar':':الكعبة:'},'\U0001F998':{'en':':kangaroo:','status':fully_qualified,'E':11,'de':':känguru:','es':':canguro:','fr':':kangourou:','ja':':カンガルー:','ko':':캥거루:','pt':':canguru:','it':':canguro:','fa':':کانگورو:','id':':kanguru:','zh':':袋鼠:','ru':':кенгуру:','tr':':kanguru:','ar':':كنغر:'},'\U0001F511':{'en':':key:','status':fully_qualified,'E':0.6,'de':':schlüssel:','es':':llave:','fr':':clé:','ja':':鍵:','ko':':열쇠:','pt':':chave:','it':':chiave:','fa':':کلید:','id':':kunci:','zh':':钥匙:','ru':':ключ:','tr':':anahtar:','ar':':مفتاح:'},'\U00002328\U0000FE0F':{'en':':keyboard:','status':fully_qualified,'E':1,'variant':True,'de':':tastatur:','es':':teclado:','fr':':clavier:','ja':':キーボード:','ko':':키보드:','pt':':teclado:','it':':tastiera:','fa':':صفحه_کلید:','id':':keyboard:','zh':':键盘:','ru':':клавиатура:','tr':':klavye:','ar':':لوحة_مفاتيح:'},'\U00002328':{'en':':keyboard:','status':unqualified,'E':1,'variant':True,'de':':tastatur:','es':':teclado:','fr':':clavier:','ja':':キーボード:','ko':':키보드:','pt':':teclado:','it':':tastiera:','fa':':صفحه_کلید:','id':':keyboard:','zh':':键盘:','ru':':клавиатура:','tr':':klavye:','ar':':لوحة_مفاتيح:'},'\U00000023\U0000FE0F\U000020E3':{'en':':keycap_#:','status':fully_qualified,'E':0.6,'alias':[':hash:'],'de':':taste_#:','es':':teclas_#:','fr':':touches_#:','ja':':囲み数字_#:','ko':':키_캡_#:','pt':':tecla_#:','it':':tasto_#:','fa':':جلد_کلید_#:','id':':keycap_#:','zh':':按键_#:','ru':':клавиши_#:','tr':':tuş_#:','ar':':مفتاح_#:'},'\U00000023\U000020E3':{'en':':keycap_#:','status':unqualified,'E':0.6,'alias':[':hash:'],'de':':taste_#:','es':':teclas_#:','fr':':touches_#:','ja':':囲み数字_#:','ko':':키_캡_#:','pt':':tecla_#:','it':':tasto_#:','fa':':جلد_کلید_#:','id':':keycap_#:','zh':':按键_#:','ru':':клавиши_#:','tr':':tuş_#:','ar':':مفتاح_#:'},'\U0000002A\U0000FE0F\U000020E3':{'en':':keycap_*:','status':fully_qualified,'E':2,'alias':[':asterisk:'],'de':':taste_*:','es':':teclas_*:','fr':':touches_*:','ja':':囲み数字_*:','ko':':키_캡_*:','pt':':tecla_*:','it':':tasto_*:','fa':':جلد_کلید_*:','id':':keycap_*:','zh':':按键_*:','ru':':клавиши_*:','tr':':tuş_*:','ar':':مفتاح_*:'},'\U0000002A\U000020E3':{'en':':keycap_*:','status':unqualified,'E':2,'alias':[':asterisk:'],'de':':taste_*:','es':':teclas_*:','fr':':touches_*:','ja':':囲み数字_*:','ko':':키_캡_*:','pt':':tecla_*:','it':':tasto_*:','fa':':جلد_کلید_*:','id':':keycap_*:','zh':':按键_*:','ru':':клавиши_*:','tr':':tuş_*:','ar':':مفتاح_*:'},'\U00000030\U0000FE0F\U000020E3':{'en':':keycap_0:','status':fully_qualified,'E':0.6,'alias':[':zero:'],'de':':taste_0:','es':':teclas_0:','fr':':touches_0:','ja':':囲み数字_0:','ko':':키_캡_0:','pt':':tecla_0:','it':':tasto_0:','fa':':جلد_کلید_0:','id':':keycap_0:','zh':':按键_0:','ru':':клавиши_0:','tr':':tuş_0:','ar':':مفتاح_0:'},'\U00000030\U000020E3':{'en':':keycap_0:','status':unqualified,'E':0.6,'alias':[':zero:'],'de':':taste_0:','es':':teclas_0:','fr':':touches_0:','ja':':囲み数字_0:','ko':':키_캡_0:','pt':':tecla_0:','it':':tasto_0:','fa':':جلد_کلید_0:','id':':keycap_0:','zh':':按键_0:','ru':':клавиши_0:','tr':':tuş_0:','ar':':مفتاح_0:'},'\U00000031\U0000FE0F\U000020E3':{'en':':keycap_1:','status':fully_qualified,'E':0.6,'alias':[':one:'],'de':':taste_1:','es':':teclas_1:','fr':':touches_1:','ja':':囲み数字_1:','ko':':키_캡_1:','pt':':tecla_1:','it':':tasto_1:','fa':':جلد_کلید_1:','id':':keycap_1:','zh':':按键_1:','ru':':клавиши_1:','tr':':tuş_1:','ar':':مفتاح_1:'},'\U00000031\U000020E3':{'en':':keycap_1:','status':unqualified,'E':0.6,'alias':[':one:'],'de':':taste_1:','es':':teclas_1:','fr':':touches_1:','ja':':囲み数字_1:','ko':':키_캡_1:','pt':':tecla_1:','it':':tasto_1:','fa':':جلد_کلید_1:','id':':keycap_1:','zh':':按键_1:','ru':':клавиши_1:','tr':':tuş_1:','ar':':مفتاح_1:'},'\U0001F51F':{'en':':keycap_10:','status':fully_qualified,'E':0.6,'alias':[':ten:', ':keycap_ten:'],'de':':taste_10:','es':':teclas_10:','fr':':touches_10:','ja':':囲み数字_10:','ko':':키_캡_10:','pt':':tecla_10:','it':':tasto_10:','fa':':جلد_کلید_10:','id':':keycap_10:','zh':':按键_10:','ru':':клавиши_10:','tr':':tuş_10:','ar':':مفتاح_10:'},'\U00000032\U0000FE0F\U000020E3':{'en':':keycap_2:','status':fully_qualified,'E':0.6,'alias':[':two:'],'de':':taste_2:','es':':teclas_2:','fr':':touches_2:','ja':':囲み数字_2:','ko':':키_캡_2:','pt':':tecla_2:','it':':tasto_2:','fa':':جلد_کلید_2:','id':':keycap_2:','zh':':按键_2:','ru':':клавиши_2:','tr':':tuş_2:','ar':':مفتاح_2:'},'\U00000032\U000020E3':{'en':':keycap_2:','status':unqualified,'E':0.6,'alias':[':two:'],'de':':taste_2:','es':':teclas_2:','fr':':touches_2:','ja':':囲み数字_2:','ko':':키_캡_2:','pt':':tecla_2:','it':':tasto_2:','fa':':جلد_کلید_2:','id':':keycap_2:','zh':':按键_2:','ru':':клавиши_2:','tr':':tuş_2:','ar':':مفتاح_2:'},'\U00000033\U0000FE0F\U000020E3':{'en':':keycap_3:','status':fully_qualified,'E':0.6,'alias':[':three:'],'de':':taste_3:','es':':teclas_3:','fr':':touches_3:','ja':':囲み数字_3:','ko':':키_캡_3:','pt':':tecla_3:','it':':tasto_3:','fa':':جلد_کلید_3:','id':':keycap_3:','zh':':按键_3:','ru':':клавиши_3:','tr':':tuş_3:','ar':':مفتاح_3:'},'\U00000033\U000020E3':{'en':':keycap_3:','status':unqualified,'E':0.6,'alias':[':three:'],'de':':taste_3:','es':':teclas_3:','fr':':touches_3:','ja':':囲み数字_3:','ko':':키_캡_3:','pt':':tecla_3:','it':':tasto_3:','fa':':جلد_کلید_3:','id':':keycap_3:','zh':':按键_3:','ru':':клавиши_3:','tr':':tuş_3:','ar':':مفتاح_3:'},'\U00000034\U0000FE0F\U000020E3':{'en':':keycap_4:','status':fully_qualified,'E':0.6,'alias':[':four:'],'de':':taste_4:','es':':teclas_4:','fr':':touches_4:','ja':':囲み数字_4:','ko':':키_캡_4:','pt':':tecla_4:','it':':tasto_4:','fa':':جلد_کلید_4:','id':':keycap_4:','zh':':按键_4:','ru':':клавиши_4:','tr':':tuş_4:','ar':':مفتاح_4:'},'\U00000034\U000020E3':{'en':':keycap_4:','status':unqualified,'E':0.6,'alias':[':four:'],'de':':taste_4:','es':':teclas_4:','fr':':touches_4:','ja':':囲み数字_4:','ko':':키_캡_4:','pt':':tecla_4:','it':':tasto_4:','fa':':جلد_کلید_4:','id':':keycap_4:','zh':':按键_4:','ru':':клавиши_4:','tr':':tuş_4:','ar':':مفتاح_4:'},'\U00000035\U0000FE0F\U000020E3':{'en':':keycap_5:','status':fully_qualified,'E':0.6,'alias':[':five:'],'de':':taste_5:','es':':teclas_5:','fr':':touches_5:','ja':':囲み数字_5:','ko':':키_캡_5:','pt':':tecla_5:','it':':tasto_5:','fa':':جلد_کلید_5:','id':':keycap_5:','zh':':按键_5:','ru':':клавиши_5:','tr':':tuş_5:','ar':':مفتاح_5:'},'\U00000035\U000020E3':{'en':':keycap_5:','status':unqualified,'E':0.6,'alias':[':five:'],'de':':taste_5:','es':':teclas_5:','fr':':touches_5:','ja':':囲み数字_5:','ko':':키_캡_5:','pt':':tecla_5:','it':':tasto_5:','fa':':جلد_کلید_5:','id':':keycap_5:','zh':':按键_5:','ru':':клавиши_5:','tr':':tuş_5:','ar':':مفتاح_5:'},'\U00000036\U0000FE0F\U000020E3':{'en':':keycap_6:','status':fully_qualified,'E':0.6,'alias':[':six:'],'de':':taste_6:','es':':teclas_6:','fr':':touches_6:','ja':':囲み数字_6:','ko':':키_캡_6:','pt':':tecla_6:','it':':tasto_6:','fa':':جلد_کلید_6:','id':':keycap_6:','zh':':按键_6:','ru':':клавиши_6:','tr':':tuş_6:','ar':':مفتاح_6:'},'\U00000036\U000020E3':{'en':':keycap_6:','status':unqualified,'E':0.6,'alias':[':six:'],'de':':taste_6:','es':':teclas_6:','fr':':touches_6:','ja':':囲み数字_6:','ko':':키_캡_6:','pt':':tecla_6:','it':':tasto_6:','fa':':جلد_کلید_6:','id':':keycap_6:','zh':':按键_6:','ru':':клавиши_6:','tr':':tuş_6:','ar':':مفتاح_6:'},'\U00000037\U0000FE0F\U000020E3':{'en':':keycap_7:','status':fully_qualified,'E':0.6,'alias':[':seven:'],'de':':taste_7:','es':':teclas_7:','fr':':touches_7:','ja':':囲み数字_7:','ko':':키_캡_7:','pt':':tecla_7:','it':':tasto_7:','fa':':جلد_کلید_7:','id':':keycap_7:','zh':':按键_7:','ru':':клавиши_7:','tr':':tuş_7:','ar':':مفتاح_7:'},'\U00000037\U000020E3':{'en':':keycap_7:','status':unqualified,'E':0.6,'alias':[':seven:'],'de':':taste_7:','es':':teclas_7:','fr':':touches_7:','ja':':囲み数字_7:','ko':':키_캡_7:','pt':':tecla_7:','it':':tasto_7:','fa':':جلد_کلید_7:','id':':keycap_7:','zh':':按键_7:','ru':':клавиши_7:','tr':':tuş_7:','ar':':مفتاح_7:'},'\U00000038\U0000FE0F\U000020E3':{'en':':keycap_8:','status':fully_qualified,'E':0.6,'alias':[':eight:'],'de':':taste_8:','es':':teclas_8:','fr':':touches_8:','ja':':囲み数字_8:','ko':':키_캡_8:','pt':':tecla_8:','it':':tasto_8:','fa':':جلد_کلید_8:','id':':keycap_8:','zh':':按键_8:','ru':':клавиши_8:','tr':':tuş_8:','ar':':مفتاح_8:'},'\U00000038\U000020E3':{'en':':keycap_8:','status':unqualified,'E':0.6,'alias':[':eight:'],'de':':taste_8:','es':':teclas_8:','fr':':touches_8:','ja':':囲み数字_8:','ko':':키_캡_8:','pt':':tecla_8:','it':':tasto_8:','fa':':جلد_کلید_8:','id':':keycap_8:','zh':':按键_8:','ru':':клавиши_8:','tr':':tuş_8:','ar':':مفتاح_8:'},'\U00000039\U0000FE0F\U000020E3':{'en':':keycap_9:','status':fully_qualified,'E':0.6,'alias':[':nine:'],'de':':taste_9:','es':':teclas_9:','fr':':touches_9:','ja':':囲み数字_9:','ko':':키_캡_9:','pt':':tecla_9:','it':':tasto_9:','fa':':جلد_کلید_9:','id':':keycap_9:','zh':':按键_9:','ru':':клавиши_9:','tr':':tuş_9:','ar':':مفتاح_9:'},'\U00000039\U000020E3':{'en':':keycap_9:','status':unqualified,'E':0.6,'alias':[':nine:'],'de':':taste_9:','es':':teclas_9:','fr':':touches_9:','ja':':囲み数字_9:','ko':':키_캡_9:','pt':':tecla_9:','it':':tasto_9:','fa':':جلد_کلید_9:','id':':keycap_9:','zh':':按键_9:','ru':':клавиши_9:','tr':':tuş_9:','ar':':مفتاح_9:'},'\U0001FAAF':{'en':':khanda:','status':fully_qualified,'E':15,'de':':khanda:','es':':khanda:','fr':':khanda:','ja':':カンダ:','ko':':칸다:','pt':':khanda:','it':':khanda:','fa':':خندا:','id':':khanda:','zh':':坎达:','ru':':кханда:','tr':':khanda:','ar':':خندا:'},'\U0001F6F4':{'en':':kick_scooter:','status':fully_qualified,'E':3,'de':':tretroller:','es':':patinete:','fr':':trottinette:','ja':':キックボード:','ko':':킥보드:','pt':':patinete:','it':':monopattino:','fa':':چرخ_پایی:','id':':otoped:','zh':':滑板车:','ru':':самокат:','tr':':ayakla_sürülen_skuter:','ar':':سكوتر_بدون_محرك:'},'\U0001F458':{'en':':kimono:','status':fully_qualified,'E':0.6,'de':':kimono:','es':':kimono:','fr':':kimono:','ja':':着物:','ko':':기모노:','pt':':quimono:','it':':kimono:','fa':':کیمونو:','id':':kimono:','zh':':和服:','ru':':кимоно:','tr':':kimono:','ar':':كيمونو_ياباني:'},'\U0001F48F':{'en':':kiss:','status':fully_qualified,'E':0.6,'alias':[':couplekiss:'],'de':':sich_küssendes_paar:','es':':beso:','fr':':bisou:','ja':':2人でキス:','ko':':키스:','pt':':beijo:','it':':bacio_tra_coppia:','fa':':بوسه:','id':':berciuman:','zh':':亲吻:','ru':':поцелуй:','tr':':öpücük:','ar':':قبلة:'},'\U0001F48F\U0001F3FF':{'en':':kiss_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_dunkle_hautfarbe:','es':':beso_tono_de_piel_oscuro:','fr':':bisou_peau_foncée:','ja':':2人でキス_濃い肌色:','ko':':키스_검은색_피부:','pt':':beijo_pele_escura:','it':':bacio_tra_coppia_carnagione_scura:','fa':':بوسه_پوست_آبنوسی:','id':':berciuman_warna_kulit_gelap:','zh':':亲吻_较深肤色:','ru':':поцелуй_очень_темный_тон_кожи:','tr':':öpücük_koyu_cilt_tonu:','ar':':قبلة_بشرة_بلون_غامق:'},'\U0001F48F\U0001F3FB':{'en':':kiss_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_helle_hautfarbe:','es':':beso_tono_de_piel_claro:','fr':':bisou_peau_claire:','ja':':2人でキス_薄い肌色:','ko':':키스_하얀_피부:','pt':':beijo_pele_clara:','it':':bacio_tra_coppia_carnagione_chiara:','fa':':بوسه_پوست_سفید:','id':':berciuman_warna_kulit_cerah:','zh':':亲吻_较浅肤色:','ru':':поцелуй_очень_светлый_тон_кожи:','tr':':öpücük_açık_cilt_tonu:','ar':':قبلة_بشرة_بلون_فاتح:'},'\U0001F468\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468':{'en':':kiss_man_man:','status':fully_qualified,'E':2,'alias':[':couplekiss_man_man:'],'de':':sich_küssendes_paar_mann,_mann:','es':':beso_hombre_y_hombre:','fr':':bisou_homme_et_homme:','ja':':2人でキス_男性_男性:','ko':':키스_남자_남자:','pt':':beijo_homem_e_homem:','it':':bacio_tra_coppia_uomo_e_uomo:','fa':':بوسه_مرد_مرد:','id':':berciuman_pria_pria:','zh':':亲吻_男人男人:','ru':':поцелуи_мужчина_мужчина:','tr':':öpücük_erkek_erkek:','ar':':قبلة_رجل_ورجل:'},'\U0001F468\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468':{'en':':kiss_man_man:','status':minimally_qualified,'E':2,'alias':[':couplekiss_man_man:'],'de':':sich_küssendes_paar_mann_mann:','es':':beso_hombre_y_hombre:','fr':':bisou_homme_et_homme:','ja':':2人でキス_男性_男性:','ko':':키스_남자_남자:','pt':':beijo_homem_e_homem:','it':':bacio_tra_coppia_uomo_e_uomo:','fa':':بوسه_مرد_مرد:','id':':berciuman_pria_pria:','zh':':亲吻_男人男人:','ru':':поцелуй_мужчина_мужчина:','tr':':öpücük_erkek_erkek:','ar':':قبلة_رجل_ورجل:'},'\U0001F468\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF':{'en':':kiss_man_man_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_und_dunkle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_oscuro:','fr':':bisou_homme_homme_et_peau_foncée:','ja':':2人でキス_男性_男性_濃い肌色:','ko':':키스_남자_남자_검은색_피부:','pt':':beijo_homem_homem_e_pele_escura:','it':':bacio_tra_coppia_uomo_uomo_e_carnagione_scura:','fa':':بوسه_مرد_مرد_و_پوست_آبنوسی:','id':':berciuman_pria_pria_warna_kulit_gelap:','zh':':亲吻_男人男人较深肤色:','ru':':поцелуи_мужчина_мужчина_очень_темныи_тон_кожи:','tr':':öpücük_erkek_erkek_koyu_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF':{'en':':kiss_man_man_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_und_dunkle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_oscuro:','fr':':bisou_homme_homme_et_peau_foncée:','ja':':2人でキス_男性_男性_濃い肌色:','ko':':키스_남자_남자_검은색_피부:','pt':':beijo_homem_homem_e_pele_escura:','it':':bacio_tra_coppia_uomo_uomo_e_carnagione_scura:','fa':':بوسه_مرد_مرد_و_پوست_آبنوسی:','id':':berciuman_pria_pria_warna_kulit_gelap:','zh':':亲吻_男人男人较深肤色:','ru':':поцелуй_мужчина_мужчина_очень_темный_тон_кожи:','tr':':öpücük_erkek_erkek_koyu_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB':{'en':':kiss_man_man_dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_dunkle_hautfarbe,helle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_oscuro_tono_de_piel_claro:','fr':':bisou_homme_homme_peau_foncée_et_peau_claire:','ja':':2人でキス_男性_男性_濃い肌色_薄い肌色:','ko':':키스_남자_남자_검은색_피부_하얀_피부:','pt':':beijo_homem_homem_pele_escura_e_pele_clara:','it':':bacio_tra_coppia_uomo_uomo_carnagione_scura_e_carnagione_chiara:','fa':':بوسه_مرد_مرد_پوست_آبنوسی_و_پوست_سفید:','id':':berciuman_pria_pria_warna_kulit_gelap_warna_kulit_cerah:','zh':':亲吻_男人男人较深肤色较浅肤色:','ru':':поцелуи_мужчина_мужчина_очень_темныи_тон_кожи_очень_светлыи_тон_кожи:','tr':':öpücük_erkek_erkek_koyu_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB':{'en':':kiss_man_man_dark_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_dunkle_hautfarbe,helle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_oscuro_tono_de_piel_claro:','fr':':bisou_homme_homme_peau_foncée_et_peau_claire:','ja':':2人でキス_男性_男性_濃い肌色_薄い肌色:','ko':':키스_남자_남자_검은색_피부_하얀_피부:','pt':':beijo_homem_homem_pele_escura_e_pele_clara:','it':':bacio_tra_coppia_uomo_uomo_carnagione_scura_e_carnagione_chiara:','fa':':بوسه_مرد_مرد_پوست_آبنوسی_و_پوست_سفید:','id':':berciuman_pria_pria_warna_kulit_gelap_warna_kulit_cerah:','zh':':亲吻_男人男人较深肤色较浅肤色:','ru':':поцелуй_мужчина_мужчина_очень_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':öpücük_erkek_erkek_koyu_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE':{'en':':kiss_man_man_dark_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_dunkle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_oscuro_tono_de_piel_oscuro_medio:','fr':':bisou_homme_homme_peau_foncée_et_peau_mate:','ja':':2人でキス_男性_男性_濃い肌色_やや濃い肌色:','ko':':키스_남자_남자_검은색_피부_진한_갈색_피부:','pt':':beijo_homem_homem_pele_escura_e_pele_morena_escura:','it':':bacio_tra_coppia_uomo_uomo_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':بوسه_مرد_مرد_پوست_آبنوسی_و_پوست_گندمی:','id':':berciuman_pria_pria_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':亲吻_男人男人较深肤色中等深肤色:','ru':':поцелуи_мужчина_мужчина_очень_темныи_тон_кожи_темныи_тон_кожи:','tr':':öpücük_erkek_erkek_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE':{'en':':kiss_man_man_dark_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_dunkle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_oscuro_tono_de_piel_oscuro_medio:','fr':':bisou_homme_homme_peau_foncée_et_peau_mate:','ja':':2人でキス_男性_男性_濃い肌色_やや濃い肌色:','ko':':키스_남자_남자_검은색_피부_진한_갈색_피부:','pt':':beijo_homem_homem_pele_escura_e_pele_morena_escura:','it':':bacio_tra_coppia_uomo_uomo_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':بوسه_مرد_مرد_پوست_آبنوسی_و_پوست_گندمی:','id':':berciuman_pria_pria_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':亲吻_男人男人较深肤色中等深肤色:','ru':':поцелуй_мужчина_мужчина_очень_темный_тон_кожи_темный_тон_кожи:','tr':':öpücük_erkek_erkek_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC':{'en':':kiss_man_man_dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_dunkle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_oscuro_tono_de_piel_claro_medio:','fr':':bisou_homme_homme_peau_foncée_et_peau_moyennement_claire:','ja':':2人でキス_男性_男性_濃い肌色_やや薄い肌色:','ko':':키스_남자_남자_검은색_피부_연한_갈색_피부:','pt':':beijo_homem_homem_pele_escura_e_pele_morena_clara:','it':':bacio_tra_coppia_uomo_uomo_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':بوسه_مرد_مرد_پوست_آبنوسی_و_پوست_روشن:','id':':berciuman_pria_pria_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':亲吻_男人男人较深肤色中等浅肤色:','ru':':поцелуи_мужчина_мужчина_очень_темныи_тон_кожи_светлыи_тон_кожи:','tr':':öpücük_erkek_erkek_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC':{'en':':kiss_man_man_dark_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_dunkle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_oscuro_tono_de_piel_claro_medio:','fr':':bisou_homme_homme_peau_foncée_et_peau_moyennement_claire:','ja':':2人でキス_男性_男性_濃い肌色_やや薄い肌色:','ko':':키스_남자_남자_검은색_피부_연한_갈색_피부:','pt':':beijo_homem_homem_pele_escura_e_pele_morena_clara:','it':':bacio_tra_coppia_uomo_uomo_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':بوسه_مرد_مرد_پوست_آبنوسی_و_پوست_روشن:','id':':berciuman_pria_pria_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':亲吻_男人男人较深肤色中等浅肤色:','ru':':поцелуй_мужчина_мужчина_очень_темный_тон_кожи_светлый_тон_кожи:','tr':':öpücük_erkek_erkek_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD':{'en':':kiss_man_man_dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_dunkle_hautfarbe,mittlere_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_oscuro_tono_de_piel_medio:','fr':':bisou_homme_homme_peau_foncée_et_peau_légèrement_mate:','ja':':2人でキス_男性_男性_濃い肌色_中間の肌色:','ko':':키스_남자_남자_검은색_피부_갈색_피부:','pt':':beijo_homem_homem_pele_escura_e_pele_morena:','it':':bacio_tra_coppia_uomo_uomo_carnagione_scura_e_carnagione_olivastra:','fa':':بوسه_مرد_مرد_پوست_آبنوسی_و_پوست_طلایی:','id':':berciuman_pria_pria_warna_kulit_gelap_warna_kulit_sedang:','zh':':亲吻_男人男人较深肤色中等肤色:','ru':':поцелуи_мужчина_мужчина_очень_темныи_тон_кожи_среднии_тон_кожи:','tr':':öpücük_erkek_erkek_koyu_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F468\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD':{'en':':kiss_man_man_dark_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_dunkle_hautfarbe,mittlere_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_oscuro_tono_de_piel_medio:','fr':':bisou_homme_homme_peau_foncée_et_peau_légèrement_mate:','ja':':2人でキス_男性_男性_濃い肌色_中間の肌色:','ko':':키스_남자_남자_검은색_피부_갈색_피부:','pt':':beijo_homem_homem_pele_escura_e_pele_morena:','it':':bacio_tra_coppia_uomo_uomo_carnagione_scura_e_carnagione_olivastra:','fa':':بوسه_مرد_مرد_پوست_آبنوسی_و_پوست_طلایی:','id':':berciuman_pria_pria_warna_kulit_gelap_warna_kulit_sedang:','zh':':亲吻_男人男人较深肤色中等肤色:','ru':':поцелуй_мужчина_мужчина_очень_темный_тон_кожи_средний_тон_кожи:','tr':':öpücük_erkek_erkek_koyu_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F468\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB':{'en':':kiss_man_man_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_und_helle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_claro:','fr':':bisou_homme_homme_et_peau_claire:','ja':':2人でキス_男性_男性_薄い肌色:','ko':':키스_남자_남자_하얀_피부:','pt':':beijo_homem_homem_e_pele_clara:','it':':bacio_tra_coppia_uomo_uomo_e_carnagione_chiara:','fa':':بوسه_مرد_مرد_و_پوست_سفید:','id':':berciuman_pria_pria_warna_kulit_cerah:','zh':':亲吻_男人男人较浅肤色:','ru':':поцелуи_мужчина_мужчина_очень_светлыи_тон_кожи:','tr':':öpücük_erkek_erkek_açık_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB':{'en':':kiss_man_man_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_und_helle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_claro:','fr':':bisou_homme_homme_et_peau_claire:','ja':':2人でキス_男性_男性_薄い肌色:','ko':':키스_남자_남자_하얀_피부:','pt':':beijo_homem_homem_e_pele_clara:','it':':bacio_tra_coppia_uomo_uomo_e_carnagione_chiara:','fa':':بوسه_مرد_مرد_و_پوست_سفید:','id':':berciuman_pria_pria_warna_kulit_cerah:','zh':':亲吻_男人男人较浅肤色:','ru':':поцелуй_мужчина_мужчина_очень_светлый_тон_кожи:','tr':':öpücük_erkek_erkek_açık_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF':{'en':':kiss_man_man_light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_helle_hautfarbe,dunkle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_claro_tono_de_piel_oscuro:','fr':':bisou_homme_homme_peau_claire_et_peau_foncée:','ja':':2人でキス_男性_男性_薄い肌色_濃い肌色:','ko':':키스_남자_남자_하얀_피부_검은색_피부:','pt':':beijo_homem_homem_pele_clara_e_pele_escura:','it':':bacio_tra_coppia_uomo_uomo_carnagione_chiara_e_carnagione_scura:','fa':':بوسه_مرد_مرد_پوست_سفید_و_پوست_آبنوسی:','id':':berciuman_pria_pria_warna_kulit_cerah_warna_kulit_gelap:','zh':':亲吻_男人男人较浅肤色较深肤色:','ru':':поцелуи_мужчина_мужчина_очень_светлыи_тон_кожи_очень_темныи_тон_кожи:','tr':':öpücük_erkek_erkek_açık_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF':{'en':':kiss_man_man_light_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_helle_hautfarbe,dunkle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_claro_tono_de_piel_oscuro:','fr':':bisou_homme_homme_peau_claire_et_peau_foncée:','ja':':2人でキス_男性_男性_薄い肌色_濃い肌色:','ko':':키스_남자_남자_하얀_피부_검은색_피부:','pt':':beijo_homem_homem_pele_clara_e_pele_escura:','it':':bacio_tra_coppia_uomo_uomo_carnagione_chiara_e_carnagione_scura:','fa':':بوسه_مرد_مرد_پوست_سفید_و_پوست_آبنوسی:','id':':berciuman_pria_pria_warna_kulit_cerah_warna_kulit_gelap:','zh':':亲吻_男人男人较浅肤色较深肤色:','ru':':поцелуй_мужчина_мужчина_очень_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':öpücük_erkek_erkek_açık_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE':{'en':':kiss_man_man_light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_helle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_claro_tono_de_piel_oscuro_medio:','fr':':bisou_homme_homme_peau_claire_et_peau_mate:','ja':':2人でキス_男性_男性_薄い肌色_やや濃い肌色:','ko':':키스_남자_남자_하얀_피부_진한_갈색_피부:','pt':':beijo_homem_homem_pele_clara_e_pele_morena_escura:','it':':bacio_tra_coppia_uomo_uomo_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':بوسه_مرد_مرد_پوست_سفید_و_پوست_گندمی:','id':':berciuman_pria_pria_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':亲吻_男人男人较浅肤色中等深肤色:','ru':':поцелуи_мужчина_мужчина_очень_светлыи_тон_кожи_темныи_тон_кожи:','tr':':öpücük_erkek_erkek_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE':{'en':':kiss_man_man_light_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_helle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_claro_tono_de_piel_oscuro_medio:','fr':':bisou_homme_homme_peau_claire_et_peau_mate:','ja':':2人でキス_男性_男性_薄い肌色_やや濃い肌色:','ko':':키스_남자_남자_하얀_피부_진한_갈색_피부:','pt':':beijo_homem_homem_pele_clara_e_pele_morena_escura:','it':':bacio_tra_coppia_uomo_uomo_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':بوسه_مرد_مرد_پوست_سفید_و_پوست_گندمی:','id':':berciuman_pria_pria_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':亲吻_男人男人较浅肤色中等深肤色:','ru':':поцелуй_мужчина_мужчина_очень_светлый_тон_кожи_темный_тон_кожи:','tr':':öpücük_erkek_erkek_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC':{'en':':kiss_man_man_light_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_helle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_claro_tono_de_piel_claro_medio:','fr':':bisou_homme_homme_peau_claire_et_peau_moyennement_claire:','ja':':2人でキス_男性_男性_薄い肌色_やや薄い肌色:','ko':':키스_남자_남자_하얀_피부_연한_갈색_피부:','pt':':beijo_homem_homem_pele_clara_e_pele_morena_clara:','it':':bacio_tra_coppia_uomo_uomo_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':بوسه_مرد_مرد_پوست_سفید_و_پوست_روشن:','id':':berciuman_pria_pria_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':亲吻_男人男人较浅肤色中等浅肤色:','ru':':поцелуи_мужчина_мужчина_очень_светлыи_тон_кожи_светлыи_тон_кожи:','tr':':öpücük_erkek_erkek_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC':{'en':':kiss_man_man_light_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_helle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_claro_tono_de_piel_claro_medio:','fr':':bisou_homme_homme_peau_claire_et_peau_moyennement_claire:','ja':':2人でキス_男性_男性_薄い肌色_やや薄い肌色:','ko':':키스_남자_남자_하얀_피부_연한_갈색_피부:','pt':':beijo_homem_homem_pele_clara_e_pele_morena_clara:','it':':bacio_tra_coppia_uomo_uomo_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':بوسه_مرد_مرد_پوست_سفید_و_پوست_روشن:','id':':berciuman_pria_pria_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':亲吻_男人男人较浅肤色中等浅肤色:','ru':':поцелуй_мужчина_мужчина_очень_светлый_тон_кожи_светлый_тон_кожи:','tr':':öpücük_erkek_erkek_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD':{'en':':kiss_man_man_light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_helle_hautfarbe,mittlere_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_claro_tono_de_piel_medio:','fr':':bisou_homme_homme_peau_claire_et_peau_légèrement_mate:','ja':':2人でキス_男性_男性_薄い肌色_中間の肌色:','ko':':키스_남자_남자_하얀_피부_갈색_피부:','pt':':beijo_homem_homem_pele_clara_e_pele_morena:','it':':bacio_tra_coppia_uomo_uomo_carnagione_chiara_e_carnagione_olivastra:','fa':':بوسه_مرد_مرد_پوست_سفید_و_پوست_طلایی:','id':':berciuman_pria_pria_warna_kulit_cerah_warna_kulit_sedang:','zh':':亲吻_男人男人较浅肤色中等肤色:','ru':':поцелуи_мужчина_мужчина_очень_светлыи_тон_кожи_среднии_тон_кожи:','tr':':öpücük_erkek_erkek_açık_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F468\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD':{'en':':kiss_man_man_light_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_helle_hautfarbe,mittlere_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_claro_tono_de_piel_medio:','fr':':bisou_homme_homme_peau_claire_et_peau_légèrement_mate:','ja':':2人でキス_男性_男性_薄い肌色_中間の肌色:','ko':':키스_남자_남자_하얀_피부_갈색_피부:','pt':':beijo_homem_homem_pele_clara_e_pele_morena:','it':':bacio_tra_coppia_uomo_uomo_carnagione_chiara_e_carnagione_olivastra:','fa':':بوسه_مرد_مرد_پوست_سفید_و_پوست_طلایی:','id':':berciuman_pria_pria_warna_kulit_cerah_warna_kulit_sedang:','zh':':亲吻_男人男人较浅肤色中等肤色:','ru':':поцелуй_мужчина_мужчина_очень_светлый_тон_кожи_средний_тон_кожи:','tr':':öpücük_erkek_erkek_açık_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F468\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE':{'en':':kiss_man_man_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_und_mitteldunkle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_oscuro_medio:','fr':':bisou_homme_homme_et_peau_mate:','ja':':2人でキス_男性_男性_やや濃い肌色:','ko':':키스_남자_남자_진한_갈색_피부:','pt':':beijo_homem_homem_e_pele_morena_escura:','it':':bacio_tra_coppia_uomo_uomo_e_carnagione_abbastanza_scura:','fa':':بوسه_مرد_مرد_و_پوست_گندمی:','id':':berciuman_pria_pria_warna_kulit_gelap-sedang:','zh':':亲吻_男人男人中等深肤色:','ru':':поцелуи_мужчина_мужчина_темныи_тон_кожи:','tr':':öpücük_erkek_erkek_orta_koyu_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE':{'en':':kiss_man_man_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_und_mitteldunkle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_oscuro_medio:','fr':':bisou_homme_homme_et_peau_mate:','ja':':2人でキス_男性_男性_やや濃い肌色:','ko':':키스_남자_남자_진한_갈색_피부:','pt':':beijo_homem_homem_e_pele_morena_escura:','it':':bacio_tra_coppia_uomo_uomo_e_carnagione_abbastanza_scura:','fa':':بوسه_مرد_مرد_و_پوست_گندمی:','id':':berciuman_pria_pria_warna_kulit_gelap-sedang:','zh':':亲吻_男人男人中等深肤色:','ru':':поцелуй_мужчина_мужчина_темный_тон_кожи:','tr':':öpücük_erkek_erkek_orta_koyu_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF':{'en':':kiss_man_man_medium-dark_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mitteldunkle_hautfarbe,dunkle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_oscuro_medio_tono_de_piel_oscuro:','fr':':bisou_homme_homme_peau_mate_et_peau_foncée:','ja':':2人でキス_男性_男性_やや濃い肌色_濃い肌色:','ko':':키스_남자_남자_진한_갈색_피부_검은색_피부:','pt':':beijo_homem_homem_pele_morena_escura_e_pele_escura:','it':':bacio_tra_coppia_uomo_uomo_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':بوسه_مرد_مرد_پوست_گندمی_و_پوست_آبنوسی:','id':':berciuman_pria_pria_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':亲吻_男人男人中等深肤色较深肤色:','ru':':поцелуи_мужчина_мужчина_темныи_тон_кожи_очень_темныи_тон_кожи:','tr':':öpücük_erkek_erkek_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF':{'en':':kiss_man_man_medium-dark_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mitteldunkle_hautfarbe,dunkle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_oscuro_medio_tono_de_piel_oscuro:','fr':':bisou_homme_homme_peau_mate_et_peau_foncée:','ja':':2人でキス_男性_男性_やや濃い肌色_濃い肌色:','ko':':키스_남자_남자_진한_갈색_피부_검은색_피부:','pt':':beijo_homem_homem_pele_morena_escura_e_pele_escura:','it':':bacio_tra_coppia_uomo_uomo_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':بوسه_مرد_مرد_پوست_گندمی_و_پوست_آبنوسی:','id':':berciuman_pria_pria_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':亲吻_男人男人中等深肤色较深肤色:','ru':':поцелуй_мужчина_мужчина_темный_тон_кожи_очень_темный_тон_кожи:','tr':':öpücük_erkek_erkek_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB':{'en':':kiss_man_man_medium-dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mitteldunkle_hautfarbe,helle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_oscuro_medio_tono_de_piel_claro:','fr':':bisou_homme_homme_peau_mate_et_peau_claire:','ja':':2人でキス_男性_男性_やや濃い肌色_薄い肌色:','ko':':키스_남자_남자_진한_갈색_피부_하얀_피부:','pt':':beijo_homem_homem_pele_morena_escura_e_pele_clara:','it':':bacio_tra_coppia_uomo_uomo_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':بوسه_مرد_مرد_پوست_گندمی_و_پوست_سفید:','id':':berciuman_pria_pria_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':亲吻_男人男人中等深肤色较浅肤色:','ru':':поцелуи_мужчина_мужчина_темныи_тон_кожи_очень_светлыи_тон_кожи:','tr':':öpücük_erkek_erkek_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB':{'en':':kiss_man_man_medium-dark_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mitteldunkle_hautfarbe,helle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_oscuro_medio_tono_de_piel_claro:','fr':':bisou_homme_homme_peau_mate_et_peau_claire:','ja':':2人でキス_男性_男性_やや濃い肌色_薄い肌色:','ko':':키스_남자_남자_진한_갈색_피부_하얀_피부:','pt':':beijo_homem_homem_pele_morena_escura_e_pele_clara:','it':':bacio_tra_coppia_uomo_uomo_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':بوسه_مرد_مرد_پوست_گندمی_و_پوست_سفید:','id':':berciuman_pria_pria_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':亲吻_男人男人中等深肤色较浅肤色:','ru':':поцелуй_мужчина_мужчина_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':öpücük_erkek_erkek_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC':{'en':':kiss_man_man_medium-dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mitteldunkle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_oscuro_medio_tono_de_piel_claro_medio:','fr':':bisou_homme_homme_peau_mate_et_peau_moyennement_claire:','ja':':2人でキス_男性_男性_やや濃い肌色_やや薄い肌色:','ko':':키스_남자_남자_진한_갈색_피부_연한_갈색_피부:','pt':':beijo_homem_homem_pele_morena_escura_e_pele_morena_clara:','it':':bacio_tra_coppia_uomo_uomo_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':بوسه_مرد_مرد_پوست_گندمی_و_پوست_روشن:','id':':berciuman_pria_pria_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':亲吻_男人男人中等深肤色中等浅肤色:','ru':':поцелуи_мужчина_мужчина_темныи_тон_кожи_светлыи_тон_кожи:','tr':':öpücük_erkek_erkek_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC':{'en':':kiss_man_man_medium-dark_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mitteldunkle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_oscuro_medio_tono_de_piel_claro_medio:','fr':':bisou_homme_homme_peau_mate_et_peau_moyennement_claire:','ja':':2人でキス_男性_男性_やや濃い肌色_やや薄い肌色:','ko':':키스_남자_남자_진한_갈색_피부_연한_갈색_피부:','pt':':beijo_homem_homem_pele_morena_escura_e_pele_morena_clara:','it':':bacio_tra_coppia_uomo_uomo_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':بوسه_مرد_مرد_پوست_گندمی_و_پوست_روشن:','id':':berciuman_pria_pria_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':亲吻_男人男人中等深肤色中等浅肤色:','ru':':поцелуй_мужчина_мужчина_темный_тон_кожи_светлый_тон_кожи:','tr':':öpücük_erkek_erkek_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD':{'en':':kiss_man_man_medium-dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mitteldunkle_hautfarbe,mittlere_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_oscuro_medio_tono_de_piel_medio:','fr':':bisou_homme_homme_peau_mate_et_peau_légèrement_mate:','ja':':2人でキス_男性_男性_やや濃い肌色_中間の肌色:','ko':':키스_남자_남자_진한_갈색_피부_갈색_피부:','pt':':beijo_homem_homem_pele_morena_escura_e_pele_morena:','it':':bacio_tra_coppia_uomo_uomo_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':بوسه_مرد_مرد_پوست_گندمی_و_پوست_طلایی:','id':':berciuman_pria_pria_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':亲吻_男人男人中等深肤色中等肤色:','ru':':поцелуи_мужчина_мужчина_темныи_тон_кожи_среднии_тон_кожи:','tr':':öpücük_erkek_erkek_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F468\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD':{'en':':kiss_man_man_medium-dark_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mitteldunkle_hautfarbe,mittlere_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_oscuro_medio_tono_de_piel_medio:','fr':':bisou_homme_homme_peau_mate_et_peau_légèrement_mate:','ja':':2人でキス_男性_男性_やや濃い肌色_中間の肌色:','ko':':키스_남자_남자_진한_갈색_피부_갈색_피부:','pt':':beijo_homem_homem_pele_morena_escura_e_pele_morena:','it':':bacio_tra_coppia_uomo_uomo_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':بوسه_مرد_مرد_پوست_گندمی_و_پوست_طلایی:','id':':berciuman_pria_pria_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':亲吻_男人男人中等深肤色中等肤色:','ru':':поцелуй_мужчина_мужчина_темный_тон_кожи_средний_тон_кожи:','tr':':öpücük_erkek_erkek_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F468\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC':{'en':':kiss_man_man_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_und_mittelhelle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_claro_medio:','fr':':bisou_homme_homme_et_peau_moyennement_claire:','ja':':2人でキス_男性_男性_やや薄い肌色:','ko':':키스_남자_남자_연한_갈색_피부:','pt':':beijo_homem_homem_e_pele_morena_clara:','it':':bacio_tra_coppia_uomo_uomo_e_carnagione_abbastanza_chiara:','fa':':بوسه_مرد_مرد_و_پوست_روشن:','id':':berciuman_pria_pria_warna_kulit_cerah-sedang:','zh':':亲吻_男人男人中等浅肤色:','ru':':поцелуи_мужчина_мужчина_светлыи_тон_кожи:','tr':':öpücük_erkek_erkek_orta_açık_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC':{'en':':kiss_man_man_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_und_mittelhelle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_claro_medio:','fr':':bisou_homme_homme_et_peau_moyennement_claire:','ja':':2人でキス_男性_男性_やや薄い肌色:','ko':':키스_남자_남자_연한_갈색_피부:','pt':':beijo_homem_homem_e_pele_morena_clara:','it':':bacio_tra_coppia_uomo_uomo_e_carnagione_abbastanza_chiara:','fa':':بوسه_مرد_مرد_و_پوست_روشن:','id':':berciuman_pria_pria_warna_kulit_cerah-sedang:','zh':':亲吻_男人男人中等浅肤色:','ru':':поцелуй_мужчина_мужчина_светлый_тон_кожи:','tr':':öpücük_erkek_erkek_orta_açık_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF':{'en':':kiss_man_man_medium-light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mittelhelle_hautfarbe,dunkle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_claro_medio_tono_de_piel_oscuro:','fr':':bisou_homme_homme_peau_moyennement_claire_et_peau_foncée:','ja':':2人でキス_男性_男性_やや薄い肌色_濃い肌色:','ko':':키스_남자_남자_연한_갈색_피부_검은색_피부:','pt':':beijo_homem_homem_pele_morena_clara_e_pele_escura:','it':':bacio_tra_coppia_uomo_uomo_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':بوسه_مرد_مرد_پوست_روشن_و_پوست_آبنوسی:','id':':berciuman_pria_pria_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':亲吻_男人男人中等浅肤色较深肤色:','ru':':поцелуи_мужчина_мужчина_светлыи_тон_кожи_очень_темныи_тон_кожи:','tr':':öpücük_erkek_erkek_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF':{'en':':kiss_man_man_medium-light_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mittelhelle_hautfarbe,dunkle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_claro_medio_tono_de_piel_oscuro:','fr':':bisou_homme_homme_peau_moyennement_claire_et_peau_foncée:','ja':':2人でキス_男性_男性_やや薄い肌色_濃い肌色:','ko':':키스_남자_남자_연한_갈색_피부_검은색_피부:','pt':':beijo_homem_homem_pele_morena_clara_e_pele_escura:','it':':bacio_tra_coppia_uomo_uomo_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':بوسه_مرد_مرد_پوست_روشن_و_پوست_آبنوسی:','id':':berciuman_pria_pria_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':亲吻_男人男人中等浅肤色较深肤色:','ru':':поцелуй_мужчина_мужчина_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':öpücük_erkek_erkek_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB':{'en':':kiss_man_man_medium-light_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mittelhelle_hautfarbe,helle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_claro_medio_tono_de_piel_claro:','fr':':bisou_homme_homme_peau_moyennement_claire_et_peau_claire:','ja':':2人でキス_男性_男性_やや薄い肌色_薄い肌色:','ko':':키스_남자_남자_연한_갈색_피부_하얀_피부:','pt':':beijo_homem_homem_pele_morena_clara_e_pele_clara:','it':':bacio_tra_coppia_uomo_uomo_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':بوسه_مرد_مرد_پوست_روشن_و_پوست_سفید:','id':':berciuman_pria_pria_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':亲吻_男人男人中等浅肤色较浅肤色:','ru':':поцелуи_мужчина_мужчина_светлыи_тон_кожи_очень_светлыи_тон_кожи:','tr':':öpücük_erkek_erkek_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB':{'en':':kiss_man_man_medium-light_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mittelhelle_hautfarbe,helle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_claro_medio_tono_de_piel_claro:','fr':':bisou_homme_homme_peau_moyennement_claire_et_peau_claire:','ja':':2人でキス_男性_男性_やや薄い肌色_薄い肌色:','ko':':키스_남자_남자_연한_갈색_피부_하얀_피부:','pt':':beijo_homem_homem_pele_morena_clara_e_pele_clara:','it':':bacio_tra_coppia_uomo_uomo_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':بوسه_مرد_مرد_پوست_روشن_و_پوست_سفید:','id':':berciuman_pria_pria_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':亲吻_男人男人中等浅肤色较浅肤色:','ru':':поцелуй_мужчина_мужчина_светлый_тон_кожи_очень_светлый_тон_кожи:','tr':':öpücük_erkek_erkek_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE':{'en':':kiss_man_man_medium-light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mittelhelle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_claro_medio_tono_de_piel_oscuro_medio:','fr':':bisou_homme_homme_peau_moyennement_claire_et_peau_mate:','ja':':2人でキス_男性_男性_やや薄い肌色_やや濃い肌色:','ko':':키스_남자_남자_연한_갈색_피부_진한_갈색_피부:','pt':':beijo_homem_homem_pele_morena_clara_e_pele_morena_escura:','it':':bacio_tra_coppia_uomo_uomo_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':بوسه_مرد_مرد_پوست_روشن_و_پوست_گندمی:','id':':berciuman_pria_pria_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':亲吻_男人男人中等浅肤色中等深肤色:','ru':':поцелуи_мужчина_мужчина_светлыи_тон_кожи_темныи_тон_кожи:','tr':':öpücük_erkek_erkek_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE':{'en':':kiss_man_man_medium-light_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mittelhelle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_claro_medio_tono_de_piel_oscuro_medio:','fr':':bisou_homme_homme_peau_moyennement_claire_et_peau_mate:','ja':':2人でキス_男性_男性_やや薄い肌色_やや濃い肌色:','ko':':키스_남자_남자_연한_갈색_피부_진한_갈색_피부:','pt':':beijo_homem_homem_pele_morena_clara_e_pele_morena_escura:','it':':bacio_tra_coppia_uomo_uomo_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':بوسه_مرد_مرد_پوست_روشن_و_پوست_گندمی:','id':':berciuman_pria_pria_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':亲吻_男人男人中等浅肤色中等深肤色:','ru':':поцелуй_мужчина_мужчина_светлый_тон_кожи_темный_тон_кожи:','tr':':öpücük_erkek_erkek_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD':{'en':':kiss_man_man_medium-light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mittelhelle_hautfarbe,mittlere_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_claro_medio_tono_de_piel_medio:','fr':':bisou_homme_homme_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':2人でキス_男性_男性_やや薄い肌色_中間の肌色:','ko':':키스_남자_남자_연한_갈색_피부_갈색_피부:','pt':':beijo_homem_homem_pele_morena_clara_e_pele_morena:','it':':bacio_tra_coppia_uomo_uomo_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':بوسه_مرد_مرد_پوست_روشن_و_پوست_طلایی:','id':':berciuman_pria_pria_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':亲吻_男人男人中等浅肤色中等肤色:','ru':':поцелуи_мужчина_мужчина_светлыи_тон_кожи_среднии_тон_кожи:','tr':':öpücük_erkek_erkek_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F468\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD':{'en':':kiss_man_man_medium-light_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mittelhelle_hautfarbe,mittlere_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_claro_medio_tono_de_piel_medio:','fr':':bisou_homme_homme_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':2人でキス_男性_男性_やや薄い肌色_中間の肌色:','ko':':키스_남자_남자_연한_갈색_피부_갈색_피부:','pt':':beijo_homem_homem_pele_morena_clara_e_pele_morena:','it':':bacio_tra_coppia_uomo_uomo_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':بوسه_مرد_مرد_پوست_روشن_و_پوست_طلایی:','id':':berciuman_pria_pria_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':亲吻_男人男人中等浅肤色中等肤色:','ru':':поцелуй_мужчина_мужчина_светлый_тон_кожи_средний_тон_кожи:','tr':':öpücük_erkek_erkek_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F468\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD':{'en':':kiss_man_man_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_und_mittlere_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_medio:','fr':':bisou_homme_homme_et_peau_légèrement_mate:','ja':':2人でキス_男性_男性_中間の肌色:','ko':':키스_남자_남자_갈색_피부:','pt':':beijo_homem_homem_e_pele_morena:','it':':bacio_tra_coppia_uomo_uomo_e_carnagione_olivastra:','fa':':بوسه_مرد_مرد_و_پوست_طلایی:','id':':berciuman_pria_pria_warna_kulit_sedang:','zh':':亲吻_男人男人中等肤色:','ru':':поцелуи_мужчина_мужчина_среднии_тон_кожи:','tr':':öpücük_erkek_erkek_orta_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_معتدل:'},'\U0001F468\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD':{'en':':kiss_man_man_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_und_mittlere_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_medio:','fr':':bisou_homme_homme_et_peau_légèrement_mate:','ja':':2人でキス_男性_男性_中間の肌色:','ko':':키스_남자_남자_갈색_피부:','pt':':beijo_homem_homem_e_pele_morena:','it':':bacio_tra_coppia_uomo_uomo_e_carnagione_olivastra:','fa':':بوسه_مرد_مرد_و_پوست_طلایی:','id':':berciuman_pria_pria_warna_kulit_sedang:','zh':':亲吻_男人男人中等肤色:','ru':':поцелуй_мужчина_мужчина_средний_тон_кожи:','tr':':öpücük_erkek_erkek_orta_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_معتدل:'},'\U0001F468\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF':{'en':':kiss_man_man_medium_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mittlere_hautfarbe,dunkle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_medio_tono_de_piel_oscuro:','fr':':bisou_homme_homme_peau_légèrement_mate_et_peau_foncée:','ja':':2人でキス_男性_男性_中間の肌色_濃い肌色:','ko':':키스_남자_남자_갈색_피부_검은색_피부:','pt':':beijo_homem_homem_pele_morena_e_pele_escura:','it':':bacio_tra_coppia_uomo_uomo_carnagione_olivastra_e_carnagione_scura:','fa':':بوسه_مرد_مرد_پوست_طلایی_و_پوست_آبنوسی:','id':':berciuman_pria_pria_warna_kulit_sedang_warna_kulit_gelap:','zh':':亲吻_男人男人中等肤色较深肤色:','ru':':поцелуи_мужчина_мужчина_среднии_тон_кожи_очень_темныи_тон_кожи:','tr':':öpücük_erkek_erkek_orta_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF':{'en':':kiss_man_man_medium_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mittlere_hautfarbe,dunkle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_medio_tono_de_piel_oscuro:','fr':':bisou_homme_homme_peau_légèrement_mate_et_peau_foncée:','ja':':2人でキス_男性_男性_中間の肌色_濃い肌色:','ko':':키스_남자_남자_갈색_피부_검은색_피부:','pt':':beijo_homem_homem_pele_morena_e_pele_escura:','it':':bacio_tra_coppia_uomo_uomo_carnagione_olivastra_e_carnagione_scura:','fa':':بوسه_مرد_مرد_پوست_طلایی_و_پوست_آبنوسی:','id':':berciuman_pria_pria_warna_kulit_sedang_warna_kulit_gelap:','zh':':亲吻_男人男人中等肤色较深肤色:','ru':':поцелуй_мужчина_мужчина_средний_тон_кожи_очень_темный_тон_кожи:','tr':':öpücük_erkek_erkek_orta_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB':{'en':':kiss_man_man_medium_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mittlere_hautfarbe,helle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_medio_tono_de_piel_claro:','fr':':bisou_homme_homme_peau_légèrement_mate_et_peau_claire:','ja':':2人でキス_男性_男性_中間の肌色_薄い肌色:','ko':':키스_남자_남자_갈색_피부_하얀_피부:','pt':':beijo_homem_homem_pele_morena_e_pele_clara:','it':':bacio_tra_coppia_uomo_uomo_carnagione_olivastra_e_carnagione_chiara:','fa':':بوسه_مرد_مرد_پوست_طلایی_و_پوست_سفید:','id':':berciuman_pria_pria_warna_kulit_sedang_warna_kulit_cerah:','zh':':亲吻_男人男人中等肤色较浅肤色:','ru':':поцелуи_мужчина_мужчина_среднии_тон_кожи_очень_светлыи_тон_кожи:','tr':':öpücük_erkek_erkek_orta_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB':{'en':':kiss_man_man_medium_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mittlere_hautfarbe,helle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_medio_tono_de_piel_claro:','fr':':bisou_homme_homme_peau_légèrement_mate_et_peau_claire:','ja':':2人でキス_男性_男性_中間の肌色_薄い肌色:','ko':':키스_남자_남자_갈색_피부_하얀_피부:','pt':':beijo_homem_homem_pele_morena_e_pele_clara:','it':':bacio_tra_coppia_uomo_uomo_carnagione_olivastra_e_carnagione_chiara:','fa':':بوسه_مرد_مرد_پوست_طلایی_و_پوست_سفید:','id':':berciuman_pria_pria_warna_kulit_sedang_warna_kulit_cerah:','zh':':亲吻_男人男人中等肤色较浅肤色:','ru':':поцелуй_мужчина_мужчина_средний_тон_кожи_очень_светлый_тон_кожи:','tr':':öpücük_erkek_erkek_orta_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE':{'en':':kiss_man_man_medium_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mittlere_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_medio_tono_de_piel_oscuro_medio:','fr':':bisou_homme_homme_peau_légèrement_mate_et_peau_mate:','ja':':2人でキス_男性_男性_中間の肌色_やや濃い肌色:','ko':':키스_남자_남자_갈색_피부_진한_갈색_피부:','pt':':beijo_homem_homem_pele_morena_e_pele_morena_escura:','it':':bacio_tra_coppia_uomo_uomo_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':بوسه_مرد_مرد_پوست_طلایی_و_پوست_گندمی:','id':':berciuman_pria_pria_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':亲吻_男人男人中等肤色中等深肤色:','ru':':поцелуи_мужчина_мужчина_среднии_тон_кожи_темныи_тон_кожи:','tr':':öpücük_erkek_erkek_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE':{'en':':kiss_man_man_medium_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mittlere_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_medio_tono_de_piel_oscuro_medio:','fr':':bisou_homme_homme_peau_légèrement_mate_et_peau_mate:','ja':':2人でキス_男性_男性_中間の肌色_やや濃い肌色:','ko':':키스_남자_남자_갈색_피부_진한_갈색_피부:','pt':':beijo_homem_homem_pele_morena_e_pele_morena_escura:','it':':bacio_tra_coppia_uomo_uomo_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':بوسه_مرد_مرد_پوست_طلایی_و_پوست_گندمی:','id':':berciuman_pria_pria_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':亲吻_男人男人中等肤色中等深肤色:','ru':':поцелуй_мужчина_мужчина_средний_тон_кожи_темный_тон_кожи:','tr':':öpücük_erkek_erkek_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC':{'en':':kiss_man_man_medium_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mittlere_hautfarbe,mittelhelle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_medio_tono_de_piel_claro_medio:','fr':':bisou_homme_homme_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':2人でキス_男性_男性_中間の肌色_やや薄い肌色:','ko':':키스_남자_남자_갈색_피부_연한_갈색_피부:','pt':':beijo_homem_homem_pele_morena_e_pele_morena_clara:','it':':bacio_tra_coppia_uomo_uomo_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':بوسه_مرد_مرد_پوست_طلایی_و_پوست_روشن:','id':':berciuman_pria_pria_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':亲吻_男人男人中等肤色中等浅肤色:','ru':':поцелуи_мужчина_мужчина_среднии_тон_кожи_светлыи_тон_кожи:','tr':':öpücük_erkek_erkek_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC':{'en':':kiss_man_man_medium_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_mann_mann_mittlere_hautfarbe,mittelhelle_hautfarbe:','es':':beso_hombre_hombre_tono_de_piel_medio_tono_de_piel_claro_medio:','fr':':bisou_homme_homme_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':2人でキス_男性_男性_中間の肌色_やや薄い肌色:','ko':':키스_남자_남자_갈색_피부_연한_갈색_피부:','pt':':beijo_homem_homem_pele_morena_e_pele_morena_clara:','it':':bacio_tra_coppia_uomo_uomo_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':بوسه_مرد_مرد_پوست_طلایی_و_پوست_روشن:','id':':berciuman_pria_pria_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':亲吻_男人男人中等肤色中等浅肤色:','ru':':поцелуй_мужчина_мужчина_средний_тон_кожи_светлый_тон_кожи:','tr':':öpücük_erkek_erkek_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_رجل_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F48B':{'en':':kiss_mark:','status':fully_qualified,'E':0.6,'alias':[':kiss:'],'de':':kussabdruck:','es':':marca_de_beso:','fr':':trace_de_rouge_à_lèvres:','ja':':キスマーク:','ko':':입술:','pt':':marca_de_beijo:','it':':impronta_della_bocca:','fa':':بوسه_ماتیکی:','id':':tanda_ciuman:','zh':':唇印:','ru':':след_от_поцелуя:','tr':':öpücük_izi:','ar':':علامة_قبلة:'},'\U0001F48F\U0001F3FE':{'en':':kiss_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mitteldunkle_hautfarbe:','es':':beso_tono_de_piel_oscuro_medio:','fr':':bisou_peau_mate:','ja':':2人でキス_やや濃い肌色:','ko':':키스_진한_갈색_피부:','pt':':beijo_pele_morena_escura:','it':':bacio_tra_coppia_carnagione_abbastanza_scura:','fa':':بوسه_پوست_گندمی:','id':':berciuman_warna_kulit_gelap-sedang:','zh':':亲吻_中等深肤色:','ru':':поцелуй_темный_тон_кожи:','tr':':öpücük_orta_koyu_cilt_tonu:','ar':':قبلة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F48F\U0001F3FC':{'en':':kiss_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mittelhelle_hautfarbe:','es':':beso_tono_de_piel_claro_medio:','fr':':bisou_peau_moyennement_claire:','ja':':2人でキス_やや薄い肌色:','ko':':키스_연한_갈색_피부:','pt':':beijo_pele_morena_clara:','it':':bacio_tra_coppia_carnagione_abbastanza_chiara:','fa':':بوسه_پوست_روشن:','id':':berciuman_warna_kulit_cerah-sedang:','zh':':亲吻_中等浅肤色:','ru':':поцелуй_светлый_тон_кожи:','tr':':öpücük_orta_açık_cilt_tonu:','ar':':قبلة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F48F\U0001F3FD':{'en':':kiss_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_mittlere_hautfarbe:','es':':beso_tono_de_piel_medio:','fr':':bisou_peau_légèrement_mate:','ja':':2人でキス_中間の肌色:','ko':':키스_갈색_피부:','pt':':beijo_pele_morena:','it':':bacio_tra_coppia_carnagione_olivastra:','fa':':بوسه_پوست_طلایی:','id':':berciuman_warna_kulit_sedang:','zh':':亲吻_中等肤色:','ru':':поцелуй_средний_тон_кожи:','tr':':öpücük_orta_cilt_tonu:','ar':':قبلة_بشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FB':{'en':':kiss_person_person_dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_erwachsener_erwachsener_dunkle_hautfarbe,helle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_oscuro_tono_de_piel_claro:','fr':':bisou_adulte_adulte_peau_foncée_et_peau_claire:','ja':':2人でキス_大人_大人_濃い肌色_薄い肌色:','ko':':키스_사람_사람_검은색_피부_하얀_피부:','pt':':beijo_pessoa_pessoa_pele_escura_e_pele_clara:','it':':bacio_tra_coppia_persona_persona_carnagione_scura_e_carnagione_chiara:','fa':':بوسه_بزرگسال_بزرگسال_پوست_آبنوسی_و_پوست_سفید:','id':':berciuman_dewasa_dewasa_warna_kulit_gelap_warna_kulit_cerah:','zh':':亲吻_成人成人较深肤色较浅肤色:','ru':':поцелуи_взрослыи_взрослыи_очень_темныи_тон_кожи_очень_светлыи_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_koyu_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FB':{'en':':kiss_person_person_dark_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_person_person_dunkle_hautfarbe,helle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_oscuro_tono_de_piel_claro:','fr':':bisou_adulte_adulte_peau_foncée_et_peau_claire:','ja':':2人でキス_大人_大人_濃い肌色_薄い肌色:','ko':':키스_사람_사람_검은색_피부_하얀_피부:','pt':':beijo_pessoa_pessoa_pele_escura_e_pele_clara:','it':':bacio_tra_coppia_persona_persona_carnagione_scura_e_carnagione_chiara:','fa':':بوسه_شخص_شخص_پوست_آبنوسی_و_پوست_سفید:','id':':berciuman_dewasa_dewasa_warna_kulit_gelap_warna_kulit_cerah:','zh':':亲吻_成人成人较深肤色较浅肤色:','ru':':поцелуй_взрослый_взрослый_очень_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_koyu_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FE':{'en':':kiss_person_person_dark_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_erwachsener_erwachsener_dunkle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_oscuro_tono_de_piel_oscuro_medio:','fr':':bisou_adulte_adulte_peau_foncée_et_peau_mate:','ja':':2人でキス_大人_大人_濃い肌色_やや濃い肌色:','ko':':키스_사람_사람_검은색_피부_진한_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_escura_e_pele_morena_escura:','it':':bacio_tra_coppia_persona_persona_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':بوسه_بزرگسال_بزرگسال_پوست_آبنوسی_و_پوست_گندمی:','id':':berciuman_dewasa_dewasa_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':亲吻_成人成人较深肤色中等深肤色:','ru':':поцелуи_взрослыи_взрослыи_очень_темныи_тон_кожи_темныи_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FE':{'en':':kiss_person_person_dark_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_person_person_dunkle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_oscuro_tono_de_piel_oscuro_medio:','fr':':bisou_adulte_adulte_peau_foncée_et_peau_mate:','ja':':2人でキス_大人_大人_濃い肌色_やや濃い肌色:','ko':':키스_사람_사람_검은색_피부_진한_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_escura_e_pele_morena_escura:','it':':bacio_tra_coppia_persona_persona_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':بوسه_شخص_شخص_پوست_آبنوسی_و_پوست_گندمی:','id':':berciuman_dewasa_dewasa_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':亲吻_成人成人较深肤色中等深肤色:','ru':':поцелуй_взрослый_взрослый_очень_темный_тон_кожи_темный_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FC':{'en':':kiss_person_person_dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_erwachsener_erwachsener_dunkle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_oscuro_tono_de_piel_claro_medio:','fr':':bisou_adulte_adulte_peau_foncée_et_peau_moyennement_claire:','ja':':2人でキス_大人_大人_濃い肌色_やや薄い肌色:','ko':':키스_사람_사람_검은색_피부_연한_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_escura_e_pele_morena_clara:','it':':bacio_tra_coppia_persona_persona_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':بوسه_بزرگسال_بزرگسال_پوست_آبنوسی_و_پوست_روشن:','id':':berciuman_dewasa_dewasa_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':亲吻_成人成人较深肤色中等浅肤色:','ru':':поцелуи_взрослыи_взрослыи_очень_темныи_тон_кожи_светлыи_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FC':{'en':':kiss_person_person_dark_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_person_person_dunkle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_oscuro_tono_de_piel_claro_medio:','fr':':bisou_adulte_adulte_peau_foncée_et_peau_moyennement_claire:','ja':':2人でキス_大人_大人_濃い肌色_やや薄い肌色:','ko':':키스_사람_사람_검은색_피부_연한_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_escura_e_pele_morena_clara:','it':':bacio_tra_coppia_persona_persona_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':بوسه_شخص_شخص_پوست_آبنوسی_و_پوست_روشن:','id':':berciuman_dewasa_dewasa_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':亲吻_成人成人较深肤色中等浅肤色:','ru':':поцелуй_взрослый_взрослый_очень_темный_тон_кожи_светлый_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FD':{'en':':kiss_person_person_dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_erwachsener_erwachsener_dunkle_hautfarbe,mittlere_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_oscuro_tono_de_piel_medio:','fr':':bisou_adulte_adulte_peau_foncée_et_peau_légèrement_mate:','ja':':2人でキス_大人_大人_濃い肌色_中間の肌色:','ko':':키스_사람_사람_검은색_피부_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_escura_e_pele_morena:','it':':bacio_tra_coppia_persona_persona_carnagione_scura_e_carnagione_olivastra:','fa':':بوسه_بزرگسال_بزرگسال_پوست_آبنوسی_و_پوست_طلایی:','id':':berciuman_dewasa_dewasa_warna_kulit_gelap_warna_kulit_sedang:','zh':':亲吻_成人成人较深肤色中等肤色:','ru':':поцелуи_взрослыи_взрослыи_очень_темныи_тон_кожи_среднии_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_koyu_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FD':{'en':':kiss_person_person_dark_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_person_person_dunkle_hautfarbe,mittlere_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_oscuro_tono_de_piel_medio:','fr':':bisou_adulte_adulte_peau_foncée_et_peau_légèrement_mate:','ja':':2人でキス_大人_大人_濃い肌色_中間の肌色:','ko':':키스_사람_사람_검은색_피부_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_escura_e_pele_morena:','it':':bacio_tra_coppia_persona_persona_carnagione_scura_e_carnagione_olivastra:','fa':':بوسه_شخص_شخص_پوست_آبنوسی_و_پوست_طلایی:','id':':berciuman_dewasa_dewasa_warna_kulit_gelap_warna_kulit_sedang:','zh':':亲吻_成人成人较深肤色中等肤色:','ru':':поцелуй_взрослый_взрослый_очень_темный_тон_кожи_средний_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_koyu_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FF':{'en':':kiss_person_person_light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_erwachsener_erwachsener_helle_hautfarbe,dunkle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_claro_tono_de_piel_oscuro:','fr':':bisou_adulte_adulte_peau_claire_et_peau_foncée:','ja':':2人でキス_大人_大人_薄い肌色_濃い肌色:','ko':':키스_사람_사람_하얀_피부_검은색_피부:','pt':':beijo_pessoa_pessoa_pele_clara_e_pele_escura:','it':':bacio_tra_coppia_persona_persona_carnagione_chiara_e_carnagione_scura:','fa':':بوسه_بزرگسال_بزرگسال_پوست_سفید_و_پوست_آبنوسی:','id':':berciuman_dewasa_dewasa_warna_kulit_cerah_warna_kulit_gelap:','zh':':亲吻_成人成人较浅肤色较深肤色:','ru':':поцелуи_взрослыи_взрослыи_очень_светлыи_тон_кожи_очень_темныи_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_açık_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FF':{'en':':kiss_person_person_light_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_person_person_helle_hautfarbe,dunkle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_claro_tono_de_piel_oscuro:','fr':':bisou_adulte_adulte_peau_claire_et_peau_foncée:','ja':':2人でキス_大人_大人_薄い肌色_濃い肌色:','ko':':키스_사람_사람_하얀_피부_검은색_피부:','pt':':beijo_pessoa_pessoa_pele_clara_e_pele_escura:','it':':bacio_tra_coppia_persona_persona_carnagione_chiara_e_carnagione_scura:','fa':':بوسه_شخص_شخص_پوست_سفید_و_پوست_آبنوسی:','id':':berciuman_dewasa_dewasa_warna_kulit_cerah_warna_kulit_gelap:','zh':':亲吻_成人成人较浅肤色较深肤色:','ru':':поцелуй_взрослый_взрослый_очень_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_açık_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FE':{'en':':kiss_person_person_light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_erwachsener_erwachsener_helle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_claro_tono_de_piel_oscuro_medio:','fr':':bisou_adulte_adulte_peau_claire_et_peau_mate:','ja':':2人でキス_大人_大人_薄い肌色_やや濃い肌色:','ko':':키스_사람_사람_하얀_피부_진한_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_clara_e_pele_morena_escura:','it':':bacio_tra_coppia_persona_persona_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':بوسه_بزرگسال_بزرگسال_پوست_سفید_و_پوست_گندمی:','id':':berciuman_dewasa_dewasa_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':亲吻_成人成人较浅肤色中等深肤色:','ru':':поцелуи_взрослыи_взрослыи_очень_светлыи_тон_кожи_темныи_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FE':{'en':':kiss_person_person_light_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_person_person_helle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_claro_tono_de_piel_oscuro_medio:','fr':':bisou_adulte_adulte_peau_claire_et_peau_mate:','ja':':2人でキス_大人_大人_薄い肌色_やや濃い肌色:','ko':':키스_사람_사람_하얀_피부_진한_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_clara_e_pele_morena_escura:','it':':bacio_tra_coppia_persona_persona_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':بوسه_شخص_شخص_پوست_سفید_و_پوست_گندمی:','id':':berciuman_dewasa_dewasa_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':亲吻_成人成人较浅肤色中等深肤色:','ru':':поцелуй_взрослый_взрослый_очень_светлый_тон_кожи_темный_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FC':{'en':':kiss_person_person_light_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_erwachsener_erwachsener_helle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_claro_tono_de_piel_claro_medio:','fr':':bisou_adulte_adulte_peau_claire_et_peau_moyennement_claire:','ja':':2人でキス_大人_大人_薄い肌色_やや薄い肌色:','ko':':키스_사람_사람_하얀_피부_연한_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_clara_e_pele_morena_clara:','it':':bacio_tra_coppia_persona_persona_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':بوسه_بزرگسال_بزرگسال_پوست_سفید_و_پوست_روشن:','id':':berciuman_dewasa_dewasa_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':亲吻_成人成人较浅肤色中等浅肤色:','ru':':поцелуи_взрослыи_взрослыи_очень_светлыи_тон_кожи_светлыи_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FC':{'en':':kiss_person_person_light_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_person_person_helle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_claro_tono_de_piel_claro_medio:','fr':':bisou_adulte_adulte_peau_claire_et_peau_moyennement_claire:','ja':':2人でキス_大人_大人_薄い肌色_やや薄い肌色:','ko':':키스_사람_사람_하얀_피부_연한_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_clara_e_pele_morena_clara:','it':':bacio_tra_coppia_persona_persona_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':بوسه_شخص_شخص_پوست_سفید_و_پوست_روشن:','id':':berciuman_dewasa_dewasa_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':亲吻_成人成人较浅肤色中等浅肤色:','ru':':поцелуй_взрослый_взрослый_очень_светлый_тон_кожи_светлый_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FD':{'en':':kiss_person_person_light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_erwachsener_erwachsener_helle_hautfarbe,mittlere_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_claro_tono_de_piel_medio:','fr':':bisou_adulte_adulte_peau_claire_et_peau_légèrement_mate:','ja':':2人でキス_大人_大人_薄い肌色_中間の肌色:','ko':':키스_사람_사람_하얀_피부_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_clara_e_pele_morena:','it':':bacio_tra_coppia_persona_persona_carnagione_chiara_e_carnagione_olivastra:','fa':':بوسه_بزرگسال_بزرگسال_پوست_سفید_و_پوست_طلایی:','id':':berciuman_dewasa_dewasa_warna_kulit_cerah_warna_kulit_sedang:','zh':':亲吻_成人成人较浅肤色中等肤色:','ru':':поцелуи_взрослыи_взрослыи_очень_светлыи_тон_кожи_среднии_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_açık_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FD':{'en':':kiss_person_person_light_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_person_person_helle_hautfarbe,mittlere_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_claro_tono_de_piel_medio:','fr':':bisou_adulte_adulte_peau_claire_et_peau_légèrement_mate:','ja':':2人でキス_大人_大人_薄い肌色_中間の肌色:','ko':':키스_사람_사람_하얀_피부_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_clara_e_pele_morena:','it':':bacio_tra_coppia_persona_persona_carnagione_chiara_e_carnagione_olivastra:','fa':':بوسه_شخص_شخص_پوست_سفید_و_پوست_طلایی:','id':':berciuman_dewasa_dewasa_warna_kulit_cerah_warna_kulit_sedang:','zh':':亲吻_成人成人较浅肤色中等肤色:','ru':':поцелуй_взрослый_взрослый_очень_светлый_тон_кожи_средний_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_açık_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FF':{'en':':kiss_person_person_medium-dark_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_erwachsener_erwachsener_mitteldunkle_hautfarbe,dunkle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_oscuro_medio_tono_de_piel_oscuro:','fr':':bisou_adulte_adulte_peau_mate_et_peau_foncée:','ja':':2人でキス_大人_大人_やや濃い肌色_濃い肌色:','ko':':키스_사람_사람_진한_갈색_피부_검은색_피부:','pt':':beijo_pessoa_pessoa_pele_morena_escura_e_pele_escura:','it':':bacio_tra_coppia_persona_persona_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':بوسه_بزرگسال_بزرگسال_پوست_گندمی_و_پوست_آبنوسی:','id':':berciuman_dewasa_dewasa_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':亲吻_成人成人中等深肤色较深肤色:','ru':':поцелуи_взрослыи_взрослыи_темныи_тон_кожи_очень_темныи_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FF':{'en':':kiss_person_person_medium-dark_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_person_person_mitteldunkle_hautfarbe,dunkle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_oscuro_medio_tono_de_piel_oscuro:','fr':':bisou_adulte_adulte_peau_mate_et_peau_foncée:','ja':':2人でキス_大人_大人_やや濃い肌色_濃い肌色:','ko':':키스_사람_사람_진한_갈색_피부_검은색_피부:','pt':':beijo_pessoa_pessoa_pele_morena_escura_e_pele_escura:','it':':bacio_tra_coppia_persona_persona_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':بوسه_شخص_شخص_پوست_گندمی_و_پوست_آبنوسی:','id':':berciuman_dewasa_dewasa_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':亲吻_成人成人中等深肤色较深肤色:','ru':':поцелуй_взрослый_взрослый_темный_тон_кожи_очень_темный_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FB':{'en':':kiss_person_person_medium-dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_erwachsener_erwachsener_mitteldunkle_hautfarbe,helle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_oscuro_medio_tono_de_piel_claro:','fr':':bisou_adulte_adulte_peau_mate_et_peau_claire:','ja':':2人でキス_大人_大人_やや濃い肌色_薄い肌色:','ko':':키스_사람_사람_진한_갈색_피부_하얀_피부:','pt':':beijo_pessoa_pessoa_pele_morena_escura_e_pele_clara:','it':':bacio_tra_coppia_persona_persona_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':بوسه_بزرگسال_بزرگسال_پوست_گندمی_و_پوست_سفید:','id':':berciuman_dewasa_dewasa_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':亲吻_成人成人中等深肤色较浅肤色:','ru':':поцелуи_взрослыи_взрослыи_темныи_тон_кожи_очень_светлыи_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FB':{'en':':kiss_person_person_medium-dark_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_person_person_mitteldunkle_hautfarbe,helle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_oscuro_medio_tono_de_piel_claro:','fr':':bisou_adulte_adulte_peau_mate_et_peau_claire:','ja':':2人でキス_大人_大人_やや濃い肌色_薄い肌色:','ko':':키스_사람_사람_진한_갈색_피부_하얀_피부:','pt':':beijo_pessoa_pessoa_pele_morena_escura_e_pele_clara:','it':':bacio_tra_coppia_persona_persona_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':بوسه_شخص_شخص_پوست_گندمی_و_پوست_سفید:','id':':berciuman_dewasa_dewasa_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':亲吻_成人成人中等深肤色较浅肤色:','ru':':поцелуй_взрослый_взрослый_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FC':{'en':':kiss_person_person_medium-dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_erwachsener_erwachsener_mitteldunkle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_oscuro_medio_tono_de_piel_claro_medio:','fr':':bisou_adulte_adulte_peau_mate_et_peau_moyennement_claire:','ja':':2人でキス_大人_大人_やや濃い肌色_やや薄い肌色:','ko':':키스_사람_사람_진한_갈색_피부_연한_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_morena_escura_e_pele_morena_clara:','it':':bacio_tra_coppia_persona_persona_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':بوسه_بزرگسال_بزرگسال_پوست_گندمی_و_پوست_روشن:','id':':berciuman_dewasa_dewasa_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':亲吻_成人成人中等深肤色中等浅肤色:','ru':':поцелуи_взрослыи_взрослыи_темныи_тон_кожи_светлыи_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FC':{'en':':kiss_person_person_medium-dark_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_person_person_mitteldunkle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_oscuro_medio_tono_de_piel_claro_medio:','fr':':bisou_adulte_adulte_peau_mate_et_peau_moyennement_claire:','ja':':2人でキス_大人_大人_やや濃い肌色_やや薄い肌色:','ko':':키스_사람_사람_진한_갈색_피부_연한_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_morena_escura_e_pele_morena_clara:','it':':bacio_tra_coppia_persona_persona_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':بوسه_شخص_شخص_پوست_گندمی_و_پوست_روشن:','id':':berciuman_dewasa_dewasa_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':亲吻_成人成人中等深肤色中等浅肤色:','ru':':поцелуй_взрослый_взрослый_темный_тон_кожи_светлый_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FD':{'en':':kiss_person_person_medium-dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_erwachsener_erwachsener_mitteldunkle_hautfarbe,mittlere_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_oscuro_medio_tono_de_piel_medio:','fr':':bisou_adulte_adulte_peau_mate_et_peau_légèrement_mate:','ja':':2人でキス_大人_大人_やや濃い肌色_中間の肌色:','ko':':키스_사람_사람_진한_갈색_피부_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_morena_escura_e_pele_morena:','it':':bacio_tra_coppia_persona_persona_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':بوسه_بزرگسال_بزرگسال_پوست_گندمی_و_پوست_طلایی:','id':':berciuman_dewasa_dewasa_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':亲吻_成人成人中等深肤色中等肤色:','ru':':поцелуи_взрослыи_взрослыи_темныи_тон_кожи_среднии_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FD':{'en':':kiss_person_person_medium-dark_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_person_person_mitteldunkle_hautfarbe,mittlere_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_oscuro_medio_tono_de_piel_medio:','fr':':bisou_adulte_adulte_peau_mate_et_peau_légèrement_mate:','ja':':2人でキス_大人_大人_やや濃い肌色_中間の肌色:','ko':':키스_사람_사람_진한_갈색_피부_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_morena_escura_e_pele_morena:','it':':bacio_tra_coppia_persona_persona_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':بوسه_شخص_شخص_پوست_گندمی_و_پوست_طلایی:','id':':berciuman_dewasa_dewasa_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':亲吻_成人成人中等深肤色中等肤色:','ru':':поцелуй_взрослый_взрослый_темный_тон_кожи_средний_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FF':{'en':':kiss_person_person_medium-light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_erwachsener_erwachsener_mittelhelle_hautfarbe,dunkle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_claro_medio_tono_de_piel_oscuro:','fr':':bisou_adulte_adulte_peau_moyennement_claire_et_peau_foncée:','ja':':2人でキス_大人_大人_やや薄い肌色_濃い肌色:','ko':':키스_사람_사람_연한_갈색_피부_검은색_피부:','pt':':beijo_pessoa_pessoa_pele_morena_clara_e_pele_escura:','it':':bacio_tra_coppia_persona_persona_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':بوسه_بزرگسال_بزرگسال_پوست_روشن_و_پوست_آبنوسی:','id':':berciuman_dewasa_dewasa_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':亲吻_成人成人中等浅肤色较深肤色:','ru':':поцелуи_взрослыи_взрослыи_светлыи_тон_кожи_очень_темныи_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FF':{'en':':kiss_person_person_medium-light_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_person_person_mittelhelle_hautfarbe,dunkle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_claro_medio_tono_de_piel_oscuro:','fr':':bisou_adulte_adulte_peau_moyennement_claire_et_peau_foncée:','ja':':2人でキス_大人_大人_やや薄い肌色_濃い肌色:','ko':':키스_사람_사람_연한_갈색_피부_검은색_피부:','pt':':beijo_pessoa_pessoa_pele_morena_clara_e_pele_escura:','it':':bacio_tra_coppia_persona_persona_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':بوسه_شخص_شخص_پوست_روشن_و_پوست_آبنوسی:','id':':berciuman_dewasa_dewasa_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':亲吻_成人成人中等浅肤色较深肤色:','ru':':поцелуй_взрослый_взрослый_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FB':{'en':':kiss_person_person_medium-light_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_erwachsener_erwachsener_mittelhelle_hautfarbe,helle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_claro_medio_tono_de_piel_claro:','fr':':bisou_adulte_adulte_peau_moyennement_claire_et_peau_claire:','ja':':2人でキス_大人_大人_やや薄い肌色_薄い肌色:','ko':':키스_사람_사람_연한_갈색_피부_하얀_피부:','pt':':beijo_pessoa_pessoa_pele_morena_clara_e_pele_clara:','it':':bacio_tra_coppia_persona_persona_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':بوسه_بزرگسال_بزرگسال_پوست_روشن_و_پوست_سفید:','id':':berciuman_dewasa_dewasa_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':亲吻_成人成人中等浅肤色较浅肤色:','ru':':поцелуи_взрослыи_взрослыи_светлыи_тон_кожи_очень_светлыи_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FB':{'en':':kiss_person_person_medium-light_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_person_person_mittelhelle_hautfarbe,helle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_claro_medio_tono_de_piel_claro:','fr':':bisou_adulte_adulte_peau_moyennement_claire_et_peau_claire:','ja':':2人でキス_大人_大人_やや薄い肌色_薄い肌色:','ko':':키스_사람_사람_연한_갈색_피부_하얀_피부:','pt':':beijo_pessoa_pessoa_pele_morena_clara_e_pele_clara:','it':':bacio_tra_coppia_persona_persona_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':بوسه_شخص_شخص_پوست_روشن_و_پوست_سفید:','id':':berciuman_dewasa_dewasa_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':亲吻_成人成人中等浅肤色较浅肤色:','ru':':поцелуй_взрослый_взрослый_светлый_тон_кожи_очень_светлый_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FE':{'en':':kiss_person_person_medium-light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_erwachsener_erwachsener_mittelhelle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_claro_medio_tono_de_piel_oscuro_medio:','fr':':bisou_adulte_adulte_peau_moyennement_claire_et_peau_mate:','ja':':2人でキス_大人_大人_やや薄い肌色_やや濃い肌色:','ko':':키스_사람_사람_연한_갈색_피부_진한_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_morena_clara_e_pele_morena_escura:','it':':bacio_tra_coppia_persona_persona_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':بوسه_بزرگسال_بزرگسال_پوست_روشن_و_پوست_گندمی:','id':':berciuman_dewasa_dewasa_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':亲吻_成人成人中等浅肤色中等深肤色:','ru':':поцелуи_взрослыи_взрослыи_светлыи_тон_кожи_темныи_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FE':{'en':':kiss_person_person_medium-light_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_person_person_mittelhelle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_claro_medio_tono_de_piel_oscuro_medio:','fr':':bisou_adulte_adulte_peau_moyennement_claire_et_peau_mate:','ja':':2人でキス_大人_大人_やや薄い肌色_やや濃い肌色:','ko':':키스_사람_사람_연한_갈색_피부_진한_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_morena_clara_e_pele_morena_escura:','it':':bacio_tra_coppia_persona_persona_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':بوسه_شخص_شخص_پوست_روشن_و_پوست_گندمی:','id':':berciuman_dewasa_dewasa_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':亲吻_成人成人中等浅肤色中等深肤色:','ru':':поцелуй_взрослый_взрослый_светлый_тон_кожи_темный_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FD':{'en':':kiss_person_person_medium-light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_erwachsener_erwachsener_mittelhelle_hautfarbe,mittlere_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_claro_medio_tono_de_piel_medio:','fr':':bisou_adulte_adulte_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':2人でキス_大人_大人_やや薄い肌色_中間の肌色:','ko':':키스_사람_사람_연한_갈색_피부_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_morena_clara_e_pele_morena:','it':':bacio_tra_coppia_persona_persona_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':بوسه_بزرگسال_بزرگسال_پوست_روشن_و_پوست_طلایی:','id':':berciuman_dewasa_dewasa_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':亲吻_成人成人中等浅肤色中等肤色:','ru':':поцелуи_взрослыи_взрослыи_светлыи_тон_кожи_среднии_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FD':{'en':':kiss_person_person_medium-light_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_person_person_mittelhelle_hautfarbe,mittlere_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_claro_medio_tono_de_piel_medio:','fr':':bisou_adulte_adulte_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':2人でキス_大人_大人_やや薄い肌色_中間の肌色:','ko':':키스_사람_사람_연한_갈색_피부_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_morena_clara_e_pele_morena:','it':':bacio_tra_coppia_persona_persona_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':بوسه_شخص_شخص_پوست_روشن_و_پوست_طلایی:','id':':berciuman_dewasa_dewasa_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':亲吻_成人成人中等浅肤色中等肤色:','ru':':поцелуй_взрослый_взрослый_светлый_тон_кожи_средний_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FF':{'en':':kiss_person_person_medium_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_erwachsener_erwachsener_mittlere_hautfarbe,dunkle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_medio_tono_de_piel_oscuro:','fr':':bisou_adulte_adulte_peau_légèrement_mate_et_peau_foncée:','ja':':2人でキス_大人_大人_中間の肌色_濃い肌色:','ko':':키스_사람_사람_갈색_피부_검은색_피부:','pt':':beijo_pessoa_pessoa_pele_morena_e_pele_escura:','it':':bacio_tra_coppia_persona_persona_carnagione_olivastra_e_carnagione_scura:','fa':':بوسه_بزرگسال_بزرگسال_پوست_طلایی_و_پوست_آبنوسی:','id':':berciuman_dewasa_dewasa_warna_kulit_sedang_warna_kulit_gelap:','zh':':亲吻_成人成人中等肤色较深肤色:','ru':':поцелуи_взрослыи_взрослыи_среднии_тон_кожи_очень_темныи_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FF':{'en':':kiss_person_person_medium_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_person_person_mittlere_hautfarbe,dunkle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_medio_tono_de_piel_oscuro:','fr':':bisou_adulte_adulte_peau_légèrement_mate_et_peau_foncée:','ja':':2人でキス_大人_大人_中間の肌色_濃い肌色:','ko':':키스_사람_사람_갈색_피부_검은색_피부:','pt':':beijo_pessoa_pessoa_pele_morena_e_pele_escura:','it':':bacio_tra_coppia_persona_persona_carnagione_olivastra_e_carnagione_scura:','fa':':بوسه_شخص_شخص_پوست_طلایی_و_پوست_آبنوسی:','id':':berciuman_dewasa_dewasa_warna_kulit_sedang_warna_kulit_gelap:','zh':':亲吻_成人成人中等肤色较深肤色:','ru':':поцелуй_взрослый_взрослый_средний_тон_кожи_очень_темный_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FB':{'en':':kiss_person_person_medium_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_erwachsener_erwachsener_mittlere_hautfarbe,helle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_medio_tono_de_piel_claro:','fr':':bisou_adulte_adulte_peau_légèrement_mate_et_peau_claire:','ja':':2人でキス_大人_大人_中間の肌色_薄い肌色:','ko':':키스_사람_사람_갈색_피부_하얀_피부:','pt':':beijo_pessoa_pessoa_pele_morena_e_pele_clara:','it':':bacio_tra_coppia_persona_persona_carnagione_olivastra_e_carnagione_chiara:','fa':':بوسه_بزرگسال_بزرگسال_پوست_طلایی_و_پوست_سفید:','id':':berciuman_dewasa_dewasa_warna_kulit_sedang_warna_kulit_cerah:','zh':':亲吻_成人成人中等肤色较浅肤色:','ru':':поцелуи_взрослыи_взрослыи_среднии_тон_кожи_очень_светлыи_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FB':{'en':':kiss_person_person_medium_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_person_person_mittlere_hautfarbe,helle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_medio_tono_de_piel_claro:','fr':':bisou_adulte_adulte_peau_légèrement_mate_et_peau_claire:','ja':':2人でキス_大人_大人_中間の肌色_薄い肌色:','ko':':키스_사람_사람_갈색_피부_하얀_피부:','pt':':beijo_pessoa_pessoa_pele_morena_e_pele_clara:','it':':bacio_tra_coppia_persona_persona_carnagione_olivastra_e_carnagione_chiara:','fa':':بوسه_شخص_شخص_پوست_طلایی_و_پوست_سفید:','id':':berciuman_dewasa_dewasa_warna_kulit_sedang_warna_kulit_cerah:','zh':':亲吻_成人成人中等肤色较浅肤色:','ru':':поцелуй_взрослый_взрослый_средний_тон_кожи_очень_светлый_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FE':{'en':':kiss_person_person_medium_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_erwachsener_erwachsener_mittlere_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_medio_tono_de_piel_oscuro_medio:','fr':':bisou_adulte_adulte_peau_légèrement_mate_et_peau_mate:','ja':':2人でキス_大人_大人_中間の肌色_やや濃い肌色:','ko':':키스_사람_사람_갈색_피부_진한_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_morena_e_pele_morena_escura:','it':':bacio_tra_coppia_persona_persona_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':بوسه_بزرگسال_بزرگسال_پوست_طلایی_و_پوست_گندمی:','id':':berciuman_dewasa_dewasa_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':亲吻_成人成人中等肤色中等深肤色:','ru':':поцелуи_взрослыи_взрослыи_среднии_тон_кожи_темныи_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FE':{'en':':kiss_person_person_medium_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_person_person_mittlere_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_medio_tono_de_piel_oscuro_medio:','fr':':bisou_adulte_adulte_peau_légèrement_mate_et_peau_mate:','ja':':2人でキス_大人_大人_中間の肌色_やや濃い肌色:','ko':':키스_사람_사람_갈색_피부_진한_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_morena_e_pele_morena_escura:','it':':bacio_tra_coppia_persona_persona_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':بوسه_شخص_شخص_پوست_طلایی_و_پوست_گندمی:','id':':berciuman_dewasa_dewasa_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':亲吻_成人成人中等肤色中等深肤色:','ru':':поцелуй_взрослый_взрослый_средний_тон_кожи_темный_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FC':{'en':':kiss_person_person_medium_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_erwachsener_erwachsener_mittlere_hautfarbe,mittelhelle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_medio_tono_de_piel_claro_medio:','fr':':bisou_adulte_adulte_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':2人でキス_大人_大人_中間の肌色_やや薄い肌色:','ko':':키스_사람_사람_갈색_피부_연한_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_morena_e_pele_morena_clara:','it':':bacio_tra_coppia_persona_persona_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':بوسه_بزرگسال_بزرگسال_پوست_طلایی_و_پوست_روشن:','id':':berciuman_dewasa_dewasa_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':亲吻_成人成人中等肤色中等浅肤色:','ru':':поцелуи_взрослыи_взрослыи_среднии_тон_кожи_светлыи_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F9D1\U0001F3FC':{'en':':kiss_person_person_medium_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_person_person_mittlere_hautfarbe,mittelhelle_hautfarbe:','es':':beso_persona_adulta_persona_adulta_tono_de_piel_medio_tono_de_piel_claro_medio:','fr':':bisou_adulte_adulte_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':2人でキス_大人_大人_中間の肌色_やや薄い肌色:','ko':':키스_사람_사람_갈색_피부_연한_갈색_피부:','pt':':beijo_pessoa_pessoa_pele_morena_e_pele_morena_clara:','it':':bacio_tra_coppia_persona_persona_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':بوسه_شخص_شخص_پوست_طلایی_و_پوست_روشن:','id':':berciuman_dewasa_dewasa_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':亲吻_成人成人中等肤色中等浅肤色:','ru':':поцелуй_взрослый_взрослый_средний_тон_кожи_светлый_тон_кожи:','tr':':öpücük_yetişkin_yetişkin_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_بالغ_وبالغ_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468':{'en':':kiss_woman_man:','status':fully_qualified,'E':2,'alias':[':couplekiss_man_woman:'],'de':':sich_küssendes_paar_frau,_mann:','es':':beso_mujer_y_hombre:','fr':':bisou_femme_et_homme:','ja':':2人でキス_女性_男性:','ko':':키스_여자_남자:','pt':':beijo_mulher_e_homem:','it':':bacio_tra_coppia_donna_e_uomo:','fa':':بوسه_زن_مرد:','id':':berciuman_wanita_pria:','zh':':亲吻_女人男人:','ru':':поцелуи_женщина_мужчина:','tr':':öpücük_kadın_erkek:','ar':':قبلة_سيدة_ورجل:'},'\U0001F469\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468':{'en':':kiss_woman_man:','status':minimally_qualified,'E':2,'alias':[':couplekiss_man_woman:'],'de':':sich_küssendes_paar_frau_mann:','es':':beso_mujer_y_hombre:','fr':':bisou_femme_et_homme:','ja':':2人でキス_女性_男性:','ko':':키스_여자_남자:','pt':':beijo_mulher_e_homem:','it':':bacio_tra_coppia_donna_e_uomo:','fa':':بوسه_زن_مرد:','id':':berciuman_wanita_pria:','zh':':亲吻_女人男人:','ru':':поцелуй_женщина_мужчина:','tr':':öpücük_kadın_erkek:','ar':':قبلة_سيدة_ورجل:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF':{'en':':kiss_woman_man_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_und_dunkle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_oscuro:','fr':':bisou_femme_homme_et_peau_foncée:','ja':':2人でキス_女性_男性_濃い肌色:','ko':':키스_여자_남자_검은색_피부:','pt':':beijo_mulher_homem_e_pele_escura:','it':':bacio_tra_coppia_donna_uomo_e_carnagione_scura:','fa':':بوسه_زن_مرد_و_پوست_آبنوسی:','id':':berciuman_wanita_pria_warna_kulit_gelap:','zh':':亲吻_女人男人较深肤色:','ru':':поцелуи_женщина_мужчина_очень_темныи_тон_кожи:','tr':':öpücük_kadın_erkek_koyu_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF':{'en':':kiss_woman_man_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_und_dunkle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_oscuro:','fr':':bisou_femme_homme_et_peau_foncée:','ja':':2人でキス_女性_男性_濃い肌色:','ko':':키스_여자_남자_검은색_피부:','pt':':beijo_mulher_homem_e_pele_escura:','it':':bacio_tra_coppia_donna_uomo_e_carnagione_scura:','fa':':بوسه_زن_مرد_و_پوست_آبنوسی:','id':':berciuman_wanita_pria_warna_kulit_gelap:','zh':':亲吻_女人男人较深肤色:','ru':':поцелуй_женщина_мужчина_очень_темный_тон_кожи:','tr':':öpücük_kadın_erkek_koyu_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB':{'en':':kiss_woman_man_dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_dunkle_hautfarbe,helle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_oscuro_tono_de_piel_claro:','fr':':bisou_femme_homme_peau_foncée_et_peau_claire:','ja':':2人でキス_女性_男性_濃い肌色_薄い肌色:','ko':':키스_여자_남자_검은색_피부_하얀_피부:','pt':':beijo_mulher_homem_pele_escura_e_pele_clara:','it':':bacio_tra_coppia_donna_uomo_carnagione_scura_e_carnagione_chiara:','fa':':بوسه_زن_مرد_پوست_آبنوسی_و_پوست_سفید:','id':':berciuman_wanita_pria_warna_kulit_gelap_warna_kulit_cerah:','zh':':亲吻_女人男人较深肤色较浅肤色:','ru':':поцелуи_женщина_мужчина_очень_темныи_тон_кожи_очень_светлыи_тон_кожи:','tr':':öpücük_kadın_erkek_koyu_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB':{'en':':kiss_woman_man_dark_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_dunkle_hautfarbe,helle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_oscuro_tono_de_piel_claro:','fr':':bisou_femme_homme_peau_foncée_et_peau_claire:','ja':':2人でキス_女性_男性_濃い肌色_薄い肌色:','ko':':키스_여자_남자_검은색_피부_하얀_피부:','pt':':beijo_mulher_homem_pele_escura_e_pele_clara:','it':':bacio_tra_coppia_donna_uomo_carnagione_scura_e_carnagione_chiara:','fa':':بوسه_زن_مرد_پوست_آبنوسی_و_پوست_سفید:','id':':berciuman_wanita_pria_warna_kulit_gelap_warna_kulit_cerah:','zh':':亲吻_女人男人较深肤色较浅肤色:','ru':':поцелуй_женщина_мужчина_очень_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':öpücük_kadın_erkek_koyu_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE':{'en':':kiss_woman_man_dark_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_dunkle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_oscuro_tono_de_piel_oscuro_medio:','fr':':bisou_femme_homme_peau_foncée_et_peau_mate:','ja':':2人でキス_女性_男性_濃い肌色_やや濃い肌色:','ko':':키스_여자_남자_검은색_피부_진한_갈색_피부:','pt':':beijo_mulher_homem_pele_escura_e_pele_morena_escura:','it':':bacio_tra_coppia_donna_uomo_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':بوسه_زن_مرد_پوست_آبنوسی_و_پوست_گندمی:','id':':berciuman_wanita_pria_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':亲吻_女人男人较深肤色中等深肤色:','ru':':поцелуи_женщина_мужчина_очень_темныи_тон_кожи_темныи_тон_кожи:','tr':':öpücük_kadın_erkek_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE':{'en':':kiss_woman_man_dark_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_dunkle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_oscuro_tono_de_piel_oscuro_medio:','fr':':bisou_femme_homme_peau_foncée_et_peau_mate:','ja':':2人でキス_女性_男性_濃い肌色_やや濃い肌色:','ko':':키스_여자_남자_검은색_피부_진한_갈색_피부:','pt':':beijo_mulher_homem_pele_escura_e_pele_morena_escura:','it':':bacio_tra_coppia_donna_uomo_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':بوسه_زن_مرد_پوست_آبنوسی_و_پوست_گندمی:','id':':berciuman_wanita_pria_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':亲吻_女人男人较深肤色中等深肤色:','ru':':поцелуй_женщина_мужчина_очень_темный_тон_кожи_темный_тон_кожи:','tr':':öpücük_kadın_erkek_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC':{'en':':kiss_woman_man_dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_dunkle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_oscuro_tono_de_piel_claro_medio:','fr':':bisou_femme_homme_peau_foncée_et_peau_moyennement_claire:','ja':':2人でキス_女性_男性_濃い肌色_やや薄い肌色:','ko':':키스_여자_남자_검은색_피부_연한_갈색_피부:','pt':':beijo_mulher_homem_pele_escura_e_pele_morena_clara:','it':':bacio_tra_coppia_donna_uomo_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':بوسه_زن_مرد_پوست_آبنوسی_و_پوست_روشن:','id':':berciuman_wanita_pria_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':亲吻_女人男人较深肤色中等浅肤色:','ru':':поцелуи_женщина_мужчина_очень_темныи_тон_кожи_светлыи_тон_кожи:','tr':':öpücük_kadın_erkek_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC':{'en':':kiss_woman_man_dark_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_dunkle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_oscuro_tono_de_piel_claro_medio:','fr':':bisou_femme_homme_peau_foncée_et_peau_moyennement_claire:','ja':':2人でキス_女性_男性_濃い肌色_やや薄い肌色:','ko':':키스_여자_남자_검은색_피부_연한_갈색_피부:','pt':':beijo_mulher_homem_pele_escura_e_pele_morena_clara:','it':':bacio_tra_coppia_donna_uomo_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':بوسه_زن_مرد_پوست_آبنوسی_و_پوست_روشن:','id':':berciuman_wanita_pria_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':亲吻_女人男人较深肤色中等浅肤色:','ru':':поцелуй_женщина_мужчина_очень_темный_тон_кожи_светлый_тон_кожи:','tr':':öpücük_kadın_erkek_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD':{'en':':kiss_woman_man_dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_dunkle_hautfarbe,mittlere_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_oscuro_tono_de_piel_medio:','fr':':bisou_femme_homme_peau_foncée_et_peau_légèrement_mate:','ja':':2人でキス_女性_男性_濃い肌色_中間の肌色:','ko':':키스_여자_남자_검은색_피부_갈색_피부:','pt':':beijo_mulher_homem_pele_escura_e_pele_morena:','it':':bacio_tra_coppia_donna_uomo_carnagione_scura_e_carnagione_olivastra:','fa':':بوسه_زن_مرد_پوست_آبنوسی_و_پوست_طلایی:','id':':berciuman_wanita_pria_warna_kulit_gelap_warna_kulit_sedang:','zh':':亲吻_女人男人较深肤色中等肤色:','ru':':поцелуи_женщина_мужчина_очень_темныи_тон_кожи_среднии_тон_кожи:','tr':':öpücük_kadın_erkek_koyu_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD':{'en':':kiss_woman_man_dark_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_dunkle_hautfarbe,mittlere_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_oscuro_tono_de_piel_medio:','fr':':bisou_femme_homme_peau_foncée_et_peau_légèrement_mate:','ja':':2人でキス_女性_男性_濃い肌色_中間の肌色:','ko':':키스_여자_남자_검은색_피부_갈색_피부:','pt':':beijo_mulher_homem_pele_escura_e_pele_morena:','it':':bacio_tra_coppia_donna_uomo_carnagione_scura_e_carnagione_olivastra:','fa':':بوسه_زن_مرد_پوست_آبنوسی_و_پوست_طلایی:','id':':berciuman_wanita_pria_warna_kulit_gelap_warna_kulit_sedang:','zh':':亲吻_女人男人较深肤色中等肤色:','ru':':поцелуй_женщина_мужчина_очень_темный_тон_кожи_средний_тон_кожи:','tr':':öpücük_kadın_erkek_koyu_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB':{'en':':kiss_woman_man_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_und_helle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_claro:','fr':':bisou_femme_homme_et_peau_claire:','ja':':2人でキス_女性_男性_薄い肌色:','ko':':키스_여자_남자_하얀_피부:','pt':':beijo_mulher_homem_e_pele_clara:','it':':bacio_tra_coppia_donna_uomo_e_carnagione_chiara:','fa':':بوسه_زن_مرد_و_پوست_سفید:','id':':berciuman_wanita_pria_warna_kulit_cerah:','zh':':亲吻_女人男人较浅肤色:','ru':':поцелуи_женщина_мужчина_очень_светлыи_тон_кожи:','tr':':öpücük_kadın_erkek_açık_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB':{'en':':kiss_woman_man_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_und_helle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_claro:','fr':':bisou_femme_homme_et_peau_claire:','ja':':2人でキス_女性_男性_薄い肌色:','ko':':키스_여자_남자_하얀_피부:','pt':':beijo_mulher_homem_e_pele_clara:','it':':bacio_tra_coppia_donna_uomo_e_carnagione_chiara:','fa':':بوسه_زن_مرد_و_پوست_سفید:','id':':berciuman_wanita_pria_warna_kulit_cerah:','zh':':亲吻_女人男人较浅肤色:','ru':':поцелуй_женщина_мужчина_очень_светлый_тон_кожи:','tr':':öpücük_kadın_erkek_açık_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF':{'en':':kiss_woman_man_light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_helle_hautfarbe,dunkle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_claro_tono_de_piel_oscuro:','fr':':bisou_femme_homme_peau_claire_et_peau_foncée:','ja':':2人でキス_女性_男性_薄い肌色_濃い肌色:','ko':':키스_여자_남자_하얀_피부_검은색_피부:','pt':':beijo_mulher_homem_pele_clara_e_pele_escura:','it':':bacio_tra_coppia_donna_uomo_carnagione_chiara_e_carnagione_scura:','fa':':بوسه_زن_مرد_پوست_سفید_و_پوست_آبنوسی:','id':':berciuman_wanita_pria_warna_kulit_cerah_warna_kulit_gelap:','zh':':亲吻_女人男人较浅肤色较深肤色:','ru':':поцелуи_женщина_мужчина_очень_светлыи_тон_кожи_очень_темныи_тон_кожи:','tr':':öpücük_kadın_erkek_açık_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF':{'en':':kiss_woman_man_light_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_helle_hautfarbe,dunkle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_claro_tono_de_piel_oscuro:','fr':':bisou_femme_homme_peau_claire_et_peau_foncée:','ja':':2人でキス_女性_男性_薄い肌色_濃い肌色:','ko':':키스_여자_남자_하얀_피부_검은색_피부:','pt':':beijo_mulher_homem_pele_clara_e_pele_escura:','it':':bacio_tra_coppia_donna_uomo_carnagione_chiara_e_carnagione_scura:','fa':':بوسه_زن_مرد_پوست_سفید_و_پوست_آبنوسی:','id':':berciuman_wanita_pria_warna_kulit_cerah_warna_kulit_gelap:','zh':':亲吻_女人男人较浅肤色较深肤色:','ru':':поцелуй_женщина_мужчина_очень_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':öpücük_kadın_erkek_açık_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE':{'en':':kiss_woman_man_light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_helle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_claro_tono_de_piel_oscuro_medio:','fr':':bisou_femme_homme_peau_claire_et_peau_mate:','ja':':2人でキス_女性_男性_薄い肌色_やや濃い肌色:','ko':':키스_여자_남자_하얀_피부_진한_갈색_피부:','pt':':beijo_mulher_homem_pele_clara_e_pele_morena_escura:','it':':bacio_tra_coppia_donna_uomo_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':بوسه_زن_مرد_پوست_سفید_و_پوست_گندمی:','id':':berciuman_wanita_pria_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':亲吻_女人男人较浅肤色中等深肤色:','ru':':поцелуи_женщина_мужчина_очень_светлыи_тон_кожи_темныи_тон_кожи:','tr':':öpücük_kadın_erkek_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE':{'en':':kiss_woman_man_light_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_helle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_claro_tono_de_piel_oscuro_medio:','fr':':bisou_femme_homme_peau_claire_et_peau_mate:','ja':':2人でキス_女性_男性_薄い肌色_やや濃い肌色:','ko':':키스_여자_남자_하얀_피부_진한_갈색_피부:','pt':':beijo_mulher_homem_pele_clara_e_pele_morena_escura:','it':':bacio_tra_coppia_donna_uomo_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':بوسه_زن_مرد_پوست_سفید_و_پوست_گندمی:','id':':berciuman_wanita_pria_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':亲吻_女人男人较浅肤色中等深肤色:','ru':':поцелуй_женщина_мужчина_очень_светлый_тон_кожи_темный_тон_кожи:','tr':':öpücük_kadın_erkek_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC':{'en':':kiss_woman_man_light_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_helle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_claro_tono_de_piel_claro_medio:','fr':':bisou_femme_homme_peau_claire_et_peau_moyennement_claire:','ja':':2人でキス_女性_男性_薄い肌色_やや薄い肌色:','ko':':키스_여자_남자_하얀_피부_연한_갈색_피부:','pt':':beijo_mulher_homem_pele_clara_e_pele_morena_clara:','it':':bacio_tra_coppia_donna_uomo_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':بوسه_زن_مرد_پوست_سفید_و_پوست_روشن:','id':':berciuman_wanita_pria_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':亲吻_女人男人较浅肤色中等浅肤色:','ru':':поцелуи_женщина_мужчина_очень_светлыи_тон_кожи_светлыи_тон_кожи:','tr':':öpücük_kadın_erkek_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC':{'en':':kiss_woman_man_light_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_helle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_claro_tono_de_piel_claro_medio:','fr':':bisou_femme_homme_peau_claire_et_peau_moyennement_claire:','ja':':2人でキス_女性_男性_薄い肌色_やや薄い肌色:','ko':':키스_여자_남자_하얀_피부_연한_갈색_피부:','pt':':beijo_mulher_homem_pele_clara_e_pele_morena_clara:','it':':bacio_tra_coppia_donna_uomo_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':بوسه_زن_مرد_پوست_سفید_و_پوست_روشن:','id':':berciuman_wanita_pria_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':亲吻_女人男人较浅肤色中等浅肤色:','ru':':поцелуй_женщина_мужчина_очень_светлый_тон_кожи_светлый_тон_кожи:','tr':':öpücük_kadın_erkek_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD':{'en':':kiss_woman_man_light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_helle_hautfarbe,mittlere_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_claro_tono_de_piel_medio:','fr':':bisou_femme_homme_peau_claire_et_peau_légèrement_mate:','ja':':2人でキス_女性_男性_薄い肌色_中間の肌色:','ko':':키스_여자_남자_하얀_피부_갈색_피부:','pt':':beijo_mulher_homem_pele_clara_e_pele_morena:','it':':bacio_tra_coppia_donna_uomo_carnagione_chiara_e_carnagione_olivastra:','fa':':بوسه_زن_مرد_پوست_سفید_و_پوست_طلایی:','id':':berciuman_wanita_pria_warna_kulit_cerah_warna_kulit_sedang:','zh':':亲吻_女人男人较浅肤色中等肤色:','ru':':поцелуи_женщина_мужчина_очень_светлыи_тон_кожи_среднии_тон_кожи:','tr':':öpücük_kadın_erkek_açık_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD':{'en':':kiss_woman_man_light_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_helle_hautfarbe,mittlere_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_claro_tono_de_piel_medio:','fr':':bisou_femme_homme_peau_claire_et_peau_légèrement_mate:','ja':':2人でキス_女性_男性_薄い肌色_中間の肌色:','ko':':키스_여자_남자_하얀_피부_갈색_피부:','pt':':beijo_mulher_homem_pele_clara_e_pele_morena:','it':':bacio_tra_coppia_donna_uomo_carnagione_chiara_e_carnagione_olivastra:','fa':':بوسه_زن_مرد_پوست_سفید_و_پوست_طلایی:','id':':berciuman_wanita_pria_warna_kulit_cerah_warna_kulit_sedang:','zh':':亲吻_女人男人较浅肤色中等肤色:','ru':':поцелуй_женщина_мужчина_очень_светлый_тон_кожи_средний_тон_кожи:','tr':':öpücük_kadın_erkek_açık_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE':{'en':':kiss_woman_man_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_und_mitteldunkle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_oscuro_medio:','fr':':bisou_femme_homme_et_peau_mate:','ja':':2人でキス_女性_男性_やや濃い肌色:','ko':':키스_여자_남자_진한_갈색_피부:','pt':':beijo_mulher_homem_e_pele_morena_escura:','it':':bacio_tra_coppia_donna_uomo_e_carnagione_abbastanza_scura:','fa':':بوسه_زن_مرد_و_پوست_گندمی:','id':':berciuman_wanita_pria_warna_kulit_gelap-sedang:','zh':':亲吻_女人男人中等深肤色:','ru':':поцелуи_женщина_мужчина_темныи_тон_кожи:','tr':':öpücük_kadın_erkek_orta_koyu_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE':{'en':':kiss_woman_man_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_und_mitteldunkle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_oscuro_medio:','fr':':bisou_femme_homme_et_peau_mate:','ja':':2人でキス_女性_男性_やや濃い肌色:','ko':':키스_여자_남자_진한_갈색_피부:','pt':':beijo_mulher_homem_e_pele_morena_escura:','it':':bacio_tra_coppia_donna_uomo_e_carnagione_abbastanza_scura:','fa':':بوسه_زن_مرد_و_پوست_گندمی:','id':':berciuman_wanita_pria_warna_kulit_gelap-sedang:','zh':':亲吻_女人男人中等深肤色:','ru':':поцелуй_женщина_мужчина_темный_тон_кожи:','tr':':öpücük_kadın_erkek_orta_koyu_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF':{'en':':kiss_woman_man_medium-dark_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mitteldunkle_hautfarbe,dunkle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_oscuro_medio_tono_de_piel_oscuro:','fr':':bisou_femme_homme_peau_mate_et_peau_foncée:','ja':':2人でキス_女性_男性_やや濃い肌色_濃い肌色:','ko':':키스_여자_남자_진한_갈색_피부_검은색_피부:','pt':':beijo_mulher_homem_pele_morena_escura_e_pele_escura:','it':':bacio_tra_coppia_donna_uomo_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':بوسه_زن_مرد_پوست_گندمی_و_پوست_آبنوسی:','id':':berciuman_wanita_pria_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':亲吻_女人男人中等深肤色较深肤色:','ru':':поцелуи_женщина_мужчина_темныи_тон_кожи_очень_темныи_тон_кожи:','tr':':öpücük_kadın_erkek_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF':{'en':':kiss_woman_man_medium-dark_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mitteldunkle_hautfarbe,dunkle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_oscuro_medio_tono_de_piel_oscuro:','fr':':bisou_femme_homme_peau_mate_et_peau_foncée:','ja':':2人でキス_女性_男性_やや濃い肌色_濃い肌色:','ko':':키스_여자_남자_진한_갈색_피부_검은색_피부:','pt':':beijo_mulher_homem_pele_morena_escura_e_pele_escura:','it':':bacio_tra_coppia_donna_uomo_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':بوسه_زن_مرد_پوست_گندمی_و_پوست_آبنوسی:','id':':berciuman_wanita_pria_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':亲吻_女人男人中等深肤色较深肤色:','ru':':поцелуй_женщина_мужчина_темный_тон_кожи_очень_темный_тон_кожи:','tr':':öpücük_kadın_erkek_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB':{'en':':kiss_woman_man_medium-dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mitteldunkle_hautfarbe,helle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_oscuro_medio_tono_de_piel_claro:','fr':':bisou_femme_homme_peau_mate_et_peau_claire:','ja':':2人でキス_女性_男性_やや濃い肌色_薄い肌色:','ko':':키스_여자_남자_진한_갈색_피부_하얀_피부:','pt':':beijo_mulher_homem_pele_morena_escura_e_pele_clara:','it':':bacio_tra_coppia_donna_uomo_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':بوسه_زن_مرد_پوست_گندمی_و_پوست_سفید:','id':':berciuman_wanita_pria_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':亲吻_女人男人中等深肤色较浅肤色:','ru':':поцелуи_женщина_мужчина_темныи_тон_кожи_очень_светлыи_тон_кожи:','tr':':öpücük_kadın_erkek_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB':{'en':':kiss_woman_man_medium-dark_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mitteldunkle_hautfarbe,helle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_oscuro_medio_tono_de_piel_claro:','fr':':bisou_femme_homme_peau_mate_et_peau_claire:','ja':':2人でキス_女性_男性_やや濃い肌色_薄い肌色:','ko':':키스_여자_남자_진한_갈색_피부_하얀_피부:','pt':':beijo_mulher_homem_pele_morena_escura_e_pele_clara:','it':':bacio_tra_coppia_donna_uomo_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':بوسه_زن_مرد_پوست_گندمی_و_پوست_سفید:','id':':berciuman_wanita_pria_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':亲吻_女人男人中等深肤色较浅肤色:','ru':':поцелуй_женщина_мужчина_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':öpücük_kadın_erkek_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC':{'en':':kiss_woman_man_medium-dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mitteldunkle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_oscuro_medio_tono_de_piel_claro_medio:','fr':':bisou_femme_homme_peau_mate_et_peau_moyennement_claire:','ja':':2人でキス_女性_男性_やや濃い肌色_やや薄い肌色:','ko':':키스_여자_남자_진한_갈색_피부_연한_갈색_피부:','pt':':beijo_mulher_homem_pele_morena_escura_e_pele_morena_clara:','it':':bacio_tra_coppia_donna_uomo_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':بوسه_زن_مرد_پوست_گندمی_و_پوست_روشن:','id':':berciuman_wanita_pria_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':亲吻_女人男人中等深肤色中等浅肤色:','ru':':поцелуи_женщина_мужчина_темныи_тон_кожи_светлыи_тон_кожи:','tr':':öpücük_kadın_erkek_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC':{'en':':kiss_woman_man_medium-dark_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mitteldunkle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_oscuro_medio_tono_de_piel_claro_medio:','fr':':bisou_femme_homme_peau_mate_et_peau_moyennement_claire:','ja':':2人でキス_女性_男性_やや濃い肌色_やや薄い肌色:','ko':':키스_여자_남자_진한_갈색_피부_연한_갈색_피부:','pt':':beijo_mulher_homem_pele_morena_escura_e_pele_morena_clara:','it':':bacio_tra_coppia_donna_uomo_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':بوسه_زن_مرد_پوست_گندمی_و_پوست_روشن:','id':':berciuman_wanita_pria_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':亲吻_女人男人中等深肤色中等浅肤色:','ru':':поцелуй_женщина_мужчина_темный_тон_кожи_светлый_тон_кожи:','tr':':öpücük_kadın_erkek_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD':{'en':':kiss_woman_man_medium-dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mitteldunkle_hautfarbe,mittlere_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_oscuro_medio_tono_de_piel_medio:','fr':':bisou_femme_homme_peau_mate_et_peau_légèrement_mate:','ja':':2人でキス_女性_男性_やや濃い肌色_中間の肌色:','ko':':키스_여자_남자_진한_갈색_피부_갈색_피부:','pt':':beijo_mulher_homem_pele_morena_escura_e_pele_morena:','it':':bacio_tra_coppia_donna_uomo_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':بوسه_زن_مرد_پوست_گندمی_و_پوست_طلایی:','id':':berciuman_wanita_pria_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':亲吻_女人男人中等深肤色中等肤色:','ru':':поцелуи_женщина_мужчина_темныи_тон_кожи_среднии_тон_кожи:','tr':':öpücük_kadın_erkek_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD':{'en':':kiss_woman_man_medium-dark_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mitteldunkle_hautfarbe,mittlere_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_oscuro_medio_tono_de_piel_medio:','fr':':bisou_femme_homme_peau_mate_et_peau_légèrement_mate:','ja':':2人でキス_女性_男性_やや濃い肌色_中間の肌色:','ko':':키스_여자_남자_진한_갈색_피부_갈색_피부:','pt':':beijo_mulher_homem_pele_morena_escura_e_pele_morena:','it':':bacio_tra_coppia_donna_uomo_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':بوسه_زن_مرد_پوست_گندمی_و_پوست_طلایی:','id':':berciuman_wanita_pria_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':亲吻_女人男人中等深肤色中等肤色:','ru':':поцелуй_женщина_мужчина_темный_тон_кожи_средний_тон_кожи:','tr':':öpücük_kadın_erkek_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC':{'en':':kiss_woman_man_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_und_mittelhelle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_claro_medio:','fr':':bisou_femme_homme_et_peau_moyennement_claire:','ja':':2人でキス_女性_男性_やや薄い肌色:','ko':':키스_여자_남자_연한_갈색_피부:','pt':':beijo_mulher_homem_e_pele_morena_clara:','it':':bacio_tra_coppia_donna_uomo_e_carnagione_abbastanza_chiara:','fa':':بوسه_زن_مرد_و_پوست_روشن:','id':':berciuman_wanita_pria_warna_kulit_cerah-sedang:','zh':':亲吻_女人男人中等浅肤色:','ru':':поцелуи_женщина_мужчина_светлыи_тон_кожи:','tr':':öpücük_kadın_erkek_orta_açık_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC':{'en':':kiss_woman_man_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_und_mittelhelle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_claro_medio:','fr':':bisou_femme_homme_et_peau_moyennement_claire:','ja':':2人でキス_女性_男性_やや薄い肌色:','ko':':키스_여자_남자_연한_갈색_피부:','pt':':beijo_mulher_homem_e_pele_morena_clara:','it':':bacio_tra_coppia_donna_uomo_e_carnagione_abbastanza_chiara:','fa':':بوسه_زن_مرد_و_پوست_روشن:','id':':berciuman_wanita_pria_warna_kulit_cerah-sedang:','zh':':亲吻_女人男人中等浅肤色:','ru':':поцелуй_женщина_мужчина_светлый_тон_кожи:','tr':':öpücük_kadın_erkek_orta_açık_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF':{'en':':kiss_woman_man_medium-light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mittelhelle_hautfarbe,dunkle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_claro_medio_tono_de_piel_oscuro:','fr':':bisou_femme_homme_peau_moyennement_claire_et_peau_foncée:','ja':':2人でキス_女性_男性_やや薄い肌色_濃い肌色:','ko':':키스_여자_남자_연한_갈색_피부_검은색_피부:','pt':':beijo_mulher_homem_pele_morena_clara_e_pele_escura:','it':':bacio_tra_coppia_donna_uomo_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':بوسه_زن_مرد_پوست_روشن_و_پوست_آبنوسی:','id':':berciuman_wanita_pria_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':亲吻_女人男人中等浅肤色较深肤色:','ru':':поцелуи_женщина_мужчина_светлыи_тон_кожи_очень_темныи_тон_кожи:','tr':':öpücük_kadın_erkek_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF':{'en':':kiss_woman_man_medium-light_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mittelhelle_hautfarbe,dunkle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_claro_medio_tono_de_piel_oscuro:','fr':':bisou_femme_homme_peau_moyennement_claire_et_peau_foncée:','ja':':2人でキス_女性_男性_やや薄い肌色_濃い肌色:','ko':':키스_여자_남자_연한_갈색_피부_검은색_피부:','pt':':beijo_mulher_homem_pele_morena_clara_e_pele_escura:','it':':bacio_tra_coppia_donna_uomo_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':بوسه_زن_مرد_پوست_روشن_و_پوست_آبنوسی:','id':':berciuman_wanita_pria_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':亲吻_女人男人中等浅肤色较深肤色:','ru':':поцелуй_женщина_мужчина_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':öpücük_kadın_erkek_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB':{'en':':kiss_woman_man_medium-light_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mittelhelle_hautfarbe,helle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_claro_medio_tono_de_piel_claro:','fr':':bisou_femme_homme_peau_moyennement_claire_et_peau_claire:','ja':':2人でキス_女性_男性_やや薄い肌色_薄い肌色:','ko':':키스_여자_남자_연한_갈색_피부_하얀_피부:','pt':':beijo_mulher_homem_pele_morena_clara_e_pele_clara:','it':':bacio_tra_coppia_donna_uomo_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':بوسه_زن_مرد_پوست_روشن_و_پوست_سفید:','id':':berciuman_wanita_pria_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':亲吻_女人男人中等浅肤色较浅肤色:','ru':':поцелуи_женщина_мужчина_светлыи_тон_кожи_очень_светлыи_тон_кожи:','tr':':öpücük_kadın_erkek_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB':{'en':':kiss_woman_man_medium-light_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mittelhelle_hautfarbe,helle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_claro_medio_tono_de_piel_claro:','fr':':bisou_femme_homme_peau_moyennement_claire_et_peau_claire:','ja':':2人でキス_女性_男性_やや薄い肌色_薄い肌色:','ko':':키스_여자_남자_연한_갈색_피부_하얀_피부:','pt':':beijo_mulher_homem_pele_morena_clara_e_pele_clara:','it':':bacio_tra_coppia_donna_uomo_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':بوسه_زن_مرد_پوست_روشن_و_پوست_سفید:','id':':berciuman_wanita_pria_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':亲吻_女人男人中等浅肤色较浅肤色:','ru':':поцелуй_женщина_мужчина_светлый_тон_кожи_очень_светлый_тон_кожи:','tr':':öpücük_kadın_erkek_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE':{'en':':kiss_woman_man_medium-light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mittelhelle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_claro_medio_tono_de_piel_oscuro_medio:','fr':':bisou_femme_homme_peau_moyennement_claire_et_peau_mate:','ja':':2人でキス_女性_男性_やや薄い肌色_やや濃い肌色:','ko':':키스_여자_남자_연한_갈색_피부_진한_갈색_피부:','pt':':beijo_mulher_homem_pele_morena_clara_e_pele_morena_escura:','it':':bacio_tra_coppia_donna_uomo_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':بوسه_زن_مرد_پوست_روشن_و_پوست_گندمی:','id':':berciuman_wanita_pria_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':亲吻_女人男人中等浅肤色中等深肤色:','ru':':поцелуи_женщина_мужчина_светлыи_тон_кожи_темныи_тон_кожи:','tr':':öpücük_kadın_erkek_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE':{'en':':kiss_woman_man_medium-light_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mittelhelle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_claro_medio_tono_de_piel_oscuro_medio:','fr':':bisou_femme_homme_peau_moyennement_claire_et_peau_mate:','ja':':2人でキス_女性_男性_やや薄い肌色_やや濃い肌色:','ko':':키스_여자_남자_연한_갈색_피부_진한_갈색_피부:','pt':':beijo_mulher_homem_pele_morena_clara_e_pele_morena_escura:','it':':bacio_tra_coppia_donna_uomo_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':بوسه_زن_مرد_پوست_روشن_و_پوست_گندمی:','id':':berciuman_wanita_pria_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':亲吻_女人男人中等浅肤色中等深肤色:','ru':':поцелуй_женщина_мужчина_светлый_тон_кожи_темный_тон_кожи:','tr':':öpücük_kadın_erkek_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD':{'en':':kiss_woman_man_medium-light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mittelhelle_hautfarbe,mittlere_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_claro_medio_tono_de_piel_medio:','fr':':bisou_femme_homme_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':2人でキス_女性_男性_やや薄い肌色_中間の肌色:','ko':':키스_여자_남자_연한_갈색_피부_갈색_피부:','pt':':beijo_mulher_homem_pele_morena_clara_e_pele_morena:','it':':bacio_tra_coppia_donna_uomo_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':بوسه_زن_مرد_پوست_روشن_و_پوست_طلایی:','id':':berciuman_wanita_pria_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':亲吻_女人男人中等浅肤色中等肤色:','ru':':поцелуи_женщина_мужчина_светлыи_тон_кожи_среднии_тон_кожи:','tr':':öpücük_kadın_erkek_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD':{'en':':kiss_woman_man_medium-light_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mittelhelle_hautfarbe,mittlere_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_claro_medio_tono_de_piel_medio:','fr':':bisou_femme_homme_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':2人でキス_女性_男性_やや薄い肌色_中間の肌色:','ko':':키스_여자_남자_연한_갈색_피부_갈색_피부:','pt':':beijo_mulher_homem_pele_morena_clara_e_pele_morena:','it':':bacio_tra_coppia_donna_uomo_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':بوسه_زن_مرد_پوست_روشن_و_پوست_طلایی:','id':':berciuman_wanita_pria_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':亲吻_女人男人中等浅肤色中等肤色:','ru':':поцелуй_женщина_мужчина_светлый_тон_кожи_средний_тон_кожи:','tr':':öpücük_kadın_erkek_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD':{'en':':kiss_woman_man_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_und_mittlere_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_medio:','fr':':bisou_femme_homme_et_peau_légèrement_mate:','ja':':2人でキス_女性_男性_中間の肌色:','ko':':키스_여자_남자_갈색_피부:','pt':':beijo_mulher_homem_e_pele_morena:','it':':bacio_tra_coppia_donna_uomo_e_carnagione_olivastra:','fa':':بوسه_زن_مرد_و_پوست_طلایی:','id':':berciuman_wanita_pria_warna_kulit_sedang:','zh':':亲吻_女人男人中等肤色:','ru':':поцелуи_женщина_мужчина_среднии_тон_кожи:','tr':':öpücük_kadın_erkek_orta_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FD':{'en':':kiss_woman_man_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_und_mittlere_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_medio:','fr':':bisou_femme_homme_et_peau_légèrement_mate:','ja':':2人でキス_女性_男性_中間の肌色:','ko':':키스_여자_남자_갈색_피부:','pt':':beijo_mulher_homem_e_pele_morena:','it':':bacio_tra_coppia_donna_uomo_e_carnagione_olivastra:','fa':':بوسه_زن_مرد_و_پوست_طلایی:','id':':berciuman_wanita_pria_warna_kulit_sedang:','zh':':亲吻_女人男人中等肤色:','ru':':поцелуй_женщина_мужчина_средний_тон_кожи:','tr':':öpücük_kadın_erkek_orta_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF':{'en':':kiss_woman_man_medium_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mittlere_hautfarbe,dunkle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_medio_tono_de_piel_oscuro:','fr':':bisou_femme_homme_peau_légèrement_mate_et_peau_foncée:','ja':':2人でキス_女性_男性_中間の肌色_濃い肌色:','ko':':키스_여자_남자_갈색_피부_검은색_피부:','pt':':beijo_mulher_homem_pele_morena_e_pele_escura:','it':':bacio_tra_coppia_donna_uomo_carnagione_olivastra_e_carnagione_scura:','fa':':بوسه_زن_مرد_پوست_طلایی_و_پوست_آبنوسی:','id':':berciuman_wanita_pria_warna_kulit_sedang_warna_kulit_gelap:','zh':':亲吻_女人男人中等肤色较深肤色:','ru':':поцелуи_женщина_мужчина_среднии_тон_кожи_очень_темныи_тон_кожи:','tr':':öpücük_kadın_erkek_orta_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FF':{'en':':kiss_woman_man_medium_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mittlere_hautfarbe,dunkle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_medio_tono_de_piel_oscuro:','fr':':bisou_femme_homme_peau_légèrement_mate_et_peau_foncée:','ja':':2人でキス_女性_男性_中間の肌色_濃い肌色:','ko':':키스_여자_남자_갈색_피부_검은색_피부:','pt':':beijo_mulher_homem_pele_morena_e_pele_escura:','it':':bacio_tra_coppia_donna_uomo_carnagione_olivastra_e_carnagione_scura:','fa':':بوسه_زن_مرد_پوست_طلایی_و_پوست_آبنوسی:','id':':berciuman_wanita_pria_warna_kulit_sedang_warna_kulit_gelap:','zh':':亲吻_女人男人中等肤色较深肤色:','ru':':поцелуй_женщина_мужчина_средний_тон_кожи_очень_темный_тон_кожи:','tr':':öpücük_kadın_erkek_orta_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB':{'en':':kiss_woman_man_medium_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mittlere_hautfarbe,helle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_medio_tono_de_piel_claro:','fr':':bisou_femme_homme_peau_légèrement_mate_et_peau_claire:','ja':':2人でキス_女性_男性_中間の肌色_薄い肌色:','ko':':키스_여자_남자_갈색_피부_하얀_피부:','pt':':beijo_mulher_homem_pele_morena_e_pele_clara:','it':':bacio_tra_coppia_donna_uomo_carnagione_olivastra_e_carnagione_chiara:','fa':':بوسه_زن_مرد_پوست_طلایی_و_پوست_سفید:','id':':berciuman_wanita_pria_warna_kulit_sedang_warna_kulit_cerah:','zh':':亲吻_女人男人中等肤色较浅肤色:','ru':':поцелуи_женщина_мужчина_среднии_тон_кожи_очень_светлыи_тон_кожи:','tr':':öpücük_kadın_erkek_orta_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FB':{'en':':kiss_woman_man_medium_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mittlere_hautfarbe,helle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_medio_tono_de_piel_claro:','fr':':bisou_femme_homme_peau_légèrement_mate_et_peau_claire:','ja':':2人でキス_女性_男性_中間の肌色_薄い肌色:','ko':':키스_여자_남자_갈색_피부_하얀_피부:','pt':':beijo_mulher_homem_pele_morena_e_pele_clara:','it':':bacio_tra_coppia_donna_uomo_carnagione_olivastra_e_carnagione_chiara:','fa':':بوسه_زن_مرد_پوست_طلایی_و_پوست_سفید:','id':':berciuman_wanita_pria_warna_kulit_sedang_warna_kulit_cerah:','zh':':亲吻_女人男人中等肤色较浅肤色:','ru':':поцелуй_женщина_мужчина_средний_тон_кожи_очень_светлый_тон_кожи:','tr':':öpücük_kadın_erkek_orta_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE':{'en':':kiss_woman_man_medium_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mittlere_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_medio_tono_de_piel_oscuro_medio:','fr':':bisou_femme_homme_peau_légèrement_mate_et_peau_mate:','ja':':2人でキス_女性_男性_中間の肌色_やや濃い肌色:','ko':':키스_여자_남자_갈색_피부_진한_갈색_피부:','pt':':beijo_mulher_homem_pele_morena_e_pele_morena_escura:','it':':bacio_tra_coppia_donna_uomo_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':بوسه_زن_مرد_پوست_طلایی_و_پوست_گندمی:','id':':berciuman_wanita_pria_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':亲吻_女人男人中等肤色中等深肤色:','ru':':поцелуи_женщина_мужчина_среднии_тон_кожи_темныи_тон_кожи:','tr':':öpücük_kadın_erkek_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FE':{'en':':kiss_woman_man_medium_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mittlere_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_medio_tono_de_piel_oscuro_medio:','fr':':bisou_femme_homme_peau_légèrement_mate_et_peau_mate:','ja':':2人でキス_女性_男性_中間の肌色_やや濃い肌色:','ko':':키스_여자_남자_갈색_피부_진한_갈색_피부:','pt':':beijo_mulher_homem_pele_morena_e_pele_morena_escura:','it':':bacio_tra_coppia_donna_uomo_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':بوسه_زن_مرد_پوست_طلایی_و_پوست_گندمی:','id':':berciuman_wanita_pria_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':亲吻_女人男人中等肤色中等深肤色:','ru':':поцелуй_женщина_мужчина_средний_тон_кожи_темный_тон_кожи:','tr':':öpücük_kadın_erkek_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC':{'en':':kiss_woman_man_medium_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mittlere_hautfarbe,mittelhelle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_medio_tono_de_piel_claro_medio:','fr':':bisou_femme_homme_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':2人でキス_女性_男性_中間の肌色_やや薄い肌色:','ko':':키스_여자_남자_갈색_피부_연한_갈색_피부:','pt':':beijo_mulher_homem_pele_morena_e_pele_morena_clara:','it':':bacio_tra_coppia_donna_uomo_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':بوسه_زن_مرد_پوست_طلایی_و_پوست_روشن:','id':':berciuman_wanita_pria_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':亲吻_女人男人中等肤色中等浅肤色:','ru':':поцелуи_женщина_мужчина_среднии_тон_кожи_светлыи_тон_кожи:','tr':':öpücük_kadın_erkek_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F468\U0001F3FC':{'en':':kiss_woman_man_medium_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_mann_mittlere_hautfarbe,mittelhelle_hautfarbe:','es':':beso_mujer_hombre_tono_de_piel_medio_tono_de_piel_claro_medio:','fr':':bisou_femme_homme_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':2人でキス_女性_男性_中間の肌色_やや薄い肌色:','ko':':키스_여자_남자_갈색_피부_연한_갈색_피부:','pt':':beijo_mulher_homem_pele_morena_e_pele_morena_clara:','it':':bacio_tra_coppia_donna_uomo_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':بوسه_زن_مرد_پوست_طلایی_و_پوست_روشن:','id':':berciuman_wanita_pria_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':亲吻_女人男人中等肤色中等浅肤色:','ru':':поцелуй_женщина_мужчина_средний_тон_кожи_светлый_тон_кожи:','tr':':öpücük_kadın_erkek_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_سيدة_ورجل_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469':{'en':':kiss_woman_woman:','status':fully_qualified,'E':2,'alias':[':couplekiss_woman_woman:'],'de':':sich_küssendes_paar_frau,_frau:','es':':beso_mujer_y_mujer:','fr':':bisou_femme_et_femme:','ja':':2人でキス_女性_女性:','ko':':키스_여자_여자:','pt':':beijo_mulher_e_mulher:','it':':bacio_tra_coppia_donna_e_donna:','fa':':بوسه_زن_زن:','id':':berciuman_wanita_wanita:','zh':':亲吻_女人女人:','ru':':поцелуи_женщина_женщина:','tr':':öpücük_kadın_kadın:','ar':':قبلة_سيدة_وسيدة:'},'\U0001F469\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469':{'en':':kiss_woman_woman:','status':minimally_qualified,'E':2,'alias':[':couplekiss_woman_woman:'],'de':':sich_küssendes_paar_frau_frau:','es':':beso_mujer_y_mujer:','fr':':bisou_femme_et_femme:','ja':':2人でキス_女性_女性:','ko':':키스_여자_여자:','pt':':beijo_mulher_e_mulher:','it':':bacio_tra_coppia_donna_e_donna:','fa':':بوسه_زن_زن:','id':':berciuman_wanita_wanita:','zh':':亲吻_女人女人:','ru':':поцелуй_женщина_женщина:','tr':':öpücük_kadın_kadın:','ar':':قبلة_سيدة_وسيدة:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FF':{'en':':kiss_woman_woman_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_und_dunkle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_oscuro:','fr':':bisou_femme_femme_et_peau_foncée:','ja':':2人でキス_女性_女性_濃い肌色:','ko':':키스_여자_여자_검은색_피부:','pt':':beijo_mulher_mulher_e_pele_escura:','it':':bacio_tra_coppia_donna_donna_e_carnagione_scura:','fa':':بوسه_زن_زن_و_پوست_آبنوسی:','id':':berciuman_wanita_wanita_warna_kulit_gelap:','zh':':亲吻_女人女人较深肤色:','ru':':поцелуи_женщина_женщина_очень_темныи_тон_кожи:','tr':':öpücük_kadın_kadın_koyu_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FF':{'en':':kiss_woman_woman_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_und_dunkle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_oscuro:','fr':':bisou_femme_femme_et_peau_foncée:','ja':':2人でキス_女性_女性_濃い肌色:','ko':':키스_여자_여자_검은색_피부:','pt':':beijo_mulher_mulher_e_pele_escura:','it':':bacio_tra_coppia_donna_donna_e_carnagione_scura:','fa':':بوسه_زن_زن_و_پوست_آبنوسی:','id':':berciuman_wanita_wanita_warna_kulit_gelap:','zh':':亲吻_女人女人较深肤色:','ru':':поцелуй_женщина_женщина_очень_темный_тон_кожи:','tr':':öpücük_kadın_kadın_koyu_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FB':{'en':':kiss_woman_woman_dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_dunkle_hautfarbe,helle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_oscuro_tono_de_piel_claro:','fr':':bisou_femme_femme_peau_foncée_et_peau_claire:','ja':':2人でキス_女性_女性_濃い肌色_薄い肌色:','ko':':키스_여자_여자_검은색_피부_하얀_피부:','pt':':beijo_mulher_mulher_pele_escura_e_pele_clara:','it':':bacio_tra_coppia_donna_donna_carnagione_scura_e_carnagione_chiara:','fa':':بوسه_زن_زن_پوست_آبنوسی_و_پوست_سفید:','id':':berciuman_wanita_wanita_warna_kulit_gelap_warna_kulit_cerah:','zh':':亲吻_女人女人较深肤色较浅肤色:','ru':':поцелуи_женщина_женщина_очень_темныи_тон_кожи_очень_светлыи_тон_кожи:','tr':':öpücük_kadın_kadın_koyu_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FB':{'en':':kiss_woman_woman_dark_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_dunkle_hautfarbe,helle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_oscuro_tono_de_piel_claro:','fr':':bisou_femme_femme_peau_foncée_et_peau_claire:','ja':':2人でキス_女性_女性_濃い肌色_薄い肌色:','ko':':키스_여자_여자_검은색_피부_하얀_피부:','pt':':beijo_mulher_mulher_pele_escura_e_pele_clara:','it':':bacio_tra_coppia_donna_donna_carnagione_scura_e_carnagione_chiara:','fa':':بوسه_زن_زن_پوست_آبنوسی_و_پوست_سفید:','id':':berciuman_wanita_wanita_warna_kulit_gelap_warna_kulit_cerah:','zh':':亲吻_女人女人较深肤色较浅肤色:','ru':':поцелуй_женщина_женщина_очень_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':öpücük_kadın_kadın_koyu_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FE':{'en':':kiss_woman_woman_dark_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_dunkle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_oscuro_tono_de_piel_oscuro_medio:','fr':':bisou_femme_femme_peau_foncée_et_peau_mate:','ja':':2人でキス_女性_女性_濃い肌色_やや濃い肌色:','ko':':키스_여자_여자_검은색_피부_진한_갈색_피부:','pt':':beijo_mulher_mulher_pele_escura_e_pele_morena_escura:','it':':bacio_tra_coppia_donna_donna_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':بوسه_زن_زن_پوست_آبنوسی_و_پوست_گندمی:','id':':berciuman_wanita_wanita_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':亲吻_女人女人较深肤色中等深肤色:','ru':':поцелуи_женщина_женщина_очень_темныи_тон_кожи_темныи_тон_кожи:','tr':':öpücük_kadın_kadın_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FE':{'en':':kiss_woman_woman_dark_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_dunkle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_oscuro_tono_de_piel_oscuro_medio:','fr':':bisou_femme_femme_peau_foncée_et_peau_mate:','ja':':2人でキス_女性_女性_濃い肌色_やや濃い肌色:','ko':':키스_여자_여자_검은색_피부_진한_갈색_피부:','pt':':beijo_mulher_mulher_pele_escura_e_pele_morena_escura:','it':':bacio_tra_coppia_donna_donna_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':بوسه_زن_زن_پوست_آبنوسی_و_پوست_گندمی:','id':':berciuman_wanita_wanita_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':亲吻_女人女人较深肤色中等深肤色:','ru':':поцелуй_женщина_женщина_очень_темный_тон_кожи_темный_тон_кожи:','tr':':öpücük_kadın_kadın_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FC':{'en':':kiss_woman_woman_dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_dunkle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_oscuro_tono_de_piel_claro_medio:','fr':':bisou_femme_femme_peau_foncée_et_peau_moyennement_claire:','ja':':2人でキス_女性_女性_濃い肌色_やや薄い肌色:','ko':':키스_여자_여자_검은색_피부_연한_갈색_피부:','pt':':beijo_mulher_mulher_pele_escura_e_pele_morena_clara:','it':':bacio_tra_coppia_donna_donna_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':بوسه_زن_زن_پوست_آبنوسی_و_پوست_روشن:','id':':berciuman_wanita_wanita_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':亲吻_女人女人较深肤色中等浅肤色:','ru':':поцелуи_женщина_женщина_очень_темныи_тон_кожи_светлыи_тон_кожи:','tr':':öpücük_kadın_kadın_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FC':{'en':':kiss_woman_woman_dark_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_dunkle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_oscuro_tono_de_piel_claro_medio:','fr':':bisou_femme_femme_peau_foncée_et_peau_moyennement_claire:','ja':':2人でキス_女性_女性_濃い肌色_やや薄い肌色:','ko':':키스_여자_여자_검은색_피부_연한_갈색_피부:','pt':':beijo_mulher_mulher_pele_escura_e_pele_morena_clara:','it':':bacio_tra_coppia_donna_donna_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':بوسه_زن_زن_پوست_آبنوسی_و_پوست_روشن:','id':':berciuman_wanita_wanita_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':亲吻_女人女人较深肤色中等浅肤色:','ru':':поцелуй_женщина_женщина_очень_темный_тон_кожи_светлый_тон_кожи:','tr':':öpücük_kadın_kadın_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FD':{'en':':kiss_woman_woman_dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_dunkle_hautfarbe,mittlere_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_oscuro_tono_de_piel_medio:','fr':':bisou_femme_femme_peau_foncée_et_peau_légèrement_mate:','ja':':2人でキス_女性_女性_濃い肌色_中間の肌色:','ko':':키스_여자_여자_검은색_피부_갈색_피부:','pt':':beijo_mulher_mulher_pele_escura_e_pele_morena:','it':':bacio_tra_coppia_donna_donna_carnagione_scura_e_carnagione_olivastra:','fa':':بوسه_زن_زن_پوست_آبنوسی_و_پوست_طلایی:','id':':berciuman_wanita_wanita_warna_kulit_gelap_warna_kulit_sedang:','zh':':亲吻_女人女人较深肤色中等肤色:','ru':':поцелуи_женщина_женщина_очень_темныи_тон_кожи_среднии_тон_кожи:','tr':':öpücük_kadın_kadın_koyu_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FF\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FD':{'en':':kiss_woman_woman_dark_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_dunkle_hautfarbe,mittlere_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_oscuro_tono_de_piel_medio:','fr':':bisou_femme_femme_peau_foncée_et_peau_légèrement_mate:','ja':':2人でキス_女性_女性_濃い肌色_中間の肌色:','ko':':키스_여자_여자_검은색_피부_갈색_피부:','pt':':beijo_mulher_mulher_pele_escura_e_pele_morena:','it':':bacio_tra_coppia_donna_donna_carnagione_scura_e_carnagione_olivastra:','fa':':بوسه_زن_زن_پوست_آبنوسی_و_پوست_طلایی:','id':':berciuman_wanita_wanita_warna_kulit_gelap_warna_kulit_sedang:','zh':':亲吻_女人女人较深肤色中等肤色:','ru':':поцелуй_женщина_женщина_очень_темный_тон_кожи_средний_тон_кожи:','tr':':öpücük_kadın_kadın_koyu_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FB':{'en':':kiss_woman_woman_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_und_helle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_claro:','fr':':bisou_femme_femme_et_peau_claire:','ja':':2人でキス_女性_女性_薄い肌色:','ko':':키스_여자_여자_하얀_피부:','pt':':beijo_mulher_mulher_e_pele_clara:','it':':bacio_tra_coppia_donna_donna_e_carnagione_chiara:','fa':':بوسه_زن_زن_و_پوست_سفید:','id':':berciuman_wanita_wanita_warna_kulit_cerah:','zh':':亲吻_女人女人较浅肤色:','ru':':поцелуи_женщина_женщина_очень_светлыи_тон_кожи:','tr':':öpücük_kadın_kadın_açık_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FB':{'en':':kiss_woman_woman_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_und_helle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_claro:','fr':':bisou_femme_femme_et_peau_claire:','ja':':2人でキス_女性_女性_薄い肌色:','ko':':키스_여자_여자_하얀_피부:','pt':':beijo_mulher_mulher_e_pele_clara:','it':':bacio_tra_coppia_donna_donna_e_carnagione_chiara:','fa':':بوسه_زن_زن_و_پوست_سفید:','id':':berciuman_wanita_wanita_warna_kulit_cerah:','zh':':亲吻_女人女人较浅肤色:','ru':':поцелуй_женщина_женщина_очень_светлый_тон_кожи:','tr':':öpücük_kadın_kadın_açık_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FF':{'en':':kiss_woman_woman_light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_helle_hautfarbe,dunkle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_claro_tono_de_piel_oscuro:','fr':':bisou_femme_femme_peau_claire_et_peau_foncée:','ja':':2人でキス_女性_女性_薄い肌色_濃い肌色:','ko':':키스_여자_여자_하얀_피부_검은색_피부:','pt':':beijo_mulher_mulher_pele_clara_e_pele_escura:','it':':bacio_tra_coppia_donna_donna_carnagione_chiara_e_carnagione_scura:','fa':':بوسه_زن_زن_پوست_سفید_و_پوست_آبنوسی:','id':':berciuman_wanita_wanita_warna_kulit_cerah_warna_kulit_gelap:','zh':':亲吻_女人女人较浅肤色较深肤色:','ru':':поцелуи_женщина_женщина_очень_светлыи_тон_кожи_очень_темныи_тон_кожи:','tr':':öpücük_kadın_kadın_açık_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FF':{'en':':kiss_woman_woman_light_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_helle_hautfarbe,dunkle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_claro_tono_de_piel_oscuro:','fr':':bisou_femme_femme_peau_claire_et_peau_foncée:','ja':':2人でキス_女性_女性_薄い肌色_濃い肌色:','ko':':키스_여자_여자_하얀_피부_검은색_피부:','pt':':beijo_mulher_mulher_pele_clara_e_pele_escura:','it':':bacio_tra_coppia_donna_donna_carnagione_chiara_e_carnagione_scura:','fa':':بوسه_زن_زن_پوست_سفید_و_پوست_آبنوسی:','id':':berciuman_wanita_wanita_warna_kulit_cerah_warna_kulit_gelap:','zh':':亲吻_女人女人较浅肤色较深肤色:','ru':':поцелуй_женщина_женщина_очень_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':öpücük_kadın_kadın_açık_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FE':{'en':':kiss_woman_woman_light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_helle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_claro_tono_de_piel_oscuro_medio:','fr':':bisou_femme_femme_peau_claire_et_peau_mate:','ja':':2人でキス_女性_女性_薄い肌色_やや濃い肌色:','ko':':키스_여자_여자_하얀_피부_진한_갈색_피부:','pt':':beijo_mulher_mulher_pele_clara_e_pele_morena_escura:','it':':bacio_tra_coppia_donna_donna_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':بوسه_زن_زن_پوست_سفید_و_پوست_گندمی:','id':':berciuman_wanita_wanita_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':亲吻_女人女人较浅肤色中等深肤色:','ru':':поцелуи_женщина_женщина_очень_светлыи_тон_кожи_темныи_тон_кожи:','tr':':öpücük_kadın_kadın_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FE':{'en':':kiss_woman_woman_light_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_helle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_claro_tono_de_piel_oscuro_medio:','fr':':bisou_femme_femme_peau_claire_et_peau_mate:','ja':':2人でキス_女性_女性_薄い肌色_やや濃い肌色:','ko':':키스_여자_여자_하얀_피부_진한_갈색_피부:','pt':':beijo_mulher_mulher_pele_clara_e_pele_morena_escura:','it':':bacio_tra_coppia_donna_donna_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':بوسه_زن_زن_پوست_سفید_و_پوست_گندمی:','id':':berciuman_wanita_wanita_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':亲吻_女人女人较浅肤色中等深肤色:','ru':':поцелуй_женщина_женщина_очень_светлый_тон_кожи_темный_тон_кожи:','tr':':öpücük_kadın_kadın_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FC':{'en':':kiss_woman_woman_light_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_helle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_claro_tono_de_piel_claro_medio:','fr':':bisou_femme_femme_peau_claire_et_peau_moyennement_claire:','ja':':2人でキス_女性_女性_薄い肌色_やや薄い肌色:','ko':':키스_여자_여자_하얀_피부_연한_갈색_피부:','pt':':beijo_mulher_mulher_pele_clara_e_pele_morena_clara:','it':':bacio_tra_coppia_donna_donna_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':بوسه_زن_زن_پوست_سفید_و_پوست_روشن:','id':':berciuman_wanita_wanita_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':亲吻_女人女人较浅肤色中等浅肤色:','ru':':поцелуи_женщина_женщина_очень_светлыи_тон_кожи_светлыи_тон_кожи:','tr':':öpücük_kadın_kadın_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FC':{'en':':kiss_woman_woman_light_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_helle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_claro_tono_de_piel_claro_medio:','fr':':bisou_femme_femme_peau_claire_et_peau_moyennement_claire:','ja':':2人でキス_女性_女性_薄い肌色_やや薄い肌色:','ko':':키스_여자_여자_하얀_피부_연한_갈색_피부:','pt':':beijo_mulher_mulher_pele_clara_e_pele_morena_clara:','it':':bacio_tra_coppia_donna_donna_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':بوسه_زن_زن_پوست_سفید_و_پوست_روشن:','id':':berciuman_wanita_wanita_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':亲吻_女人女人较浅肤色中等浅肤色:','ru':':поцелуй_женщина_женщина_очень_светлый_тон_кожи_светлый_тон_кожи:','tr':':öpücük_kadın_kadın_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FD':{'en':':kiss_woman_woman_light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_helle_hautfarbe,mittlere_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_claro_tono_de_piel_medio:','fr':':bisou_femme_femme_peau_claire_et_peau_légèrement_mate:','ja':':2人でキス_女性_女性_薄い肌色_中間の肌色:','ko':':키스_여자_여자_하얀_피부_갈색_피부:','pt':':beijo_mulher_mulher_pele_clara_e_pele_morena:','it':':bacio_tra_coppia_donna_donna_carnagione_chiara_e_carnagione_olivastra:','fa':':بوسه_زن_زن_پوست_سفید_و_پوست_طلایی:','id':':berciuman_wanita_wanita_warna_kulit_cerah_warna_kulit_sedang:','zh':':亲吻_女人女人较浅肤色中等肤色:','ru':':поцелуи_женщина_женщина_очень_светлыи_тон_кожи_среднии_тон_кожи:','tr':':öpücük_kadın_kadın_açık_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FB\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FD':{'en':':kiss_woman_woman_light_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_helle_hautfarbe,mittlere_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_claro_tono_de_piel_medio:','fr':':bisou_femme_femme_peau_claire_et_peau_légèrement_mate:','ja':':2人でキス_女性_女性_薄い肌色_中間の肌色:','ko':':키스_여자_여자_하얀_피부_갈색_피부:','pt':':beijo_mulher_mulher_pele_clara_e_pele_morena:','it':':bacio_tra_coppia_donna_donna_carnagione_chiara_e_carnagione_olivastra:','fa':':بوسه_زن_زن_پوست_سفید_و_پوست_طلایی:','id':':berciuman_wanita_wanita_warna_kulit_cerah_warna_kulit_sedang:','zh':':亲吻_女人女人较浅肤色中等肤色:','ru':':поцелуй_женщина_женщина_очень_светлый_тон_кожи_средний_тон_кожи:','tr':':öpücük_kadın_kadın_açık_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FE':{'en':':kiss_woman_woman_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_und_mitteldunkle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_oscuro_medio:','fr':':bisou_femme_femme_et_peau_mate:','ja':':2人でキス_女性_女性_やや濃い肌色:','ko':':키스_여자_여자_진한_갈색_피부:','pt':':beijo_mulher_mulher_e_pele_morena_escura:','it':':bacio_tra_coppia_donna_donna_e_carnagione_abbastanza_scura:','fa':':بوسه_زن_زن_و_پوست_گندمی:','id':':berciuman_wanita_wanita_warna_kulit_gelap-sedang:','zh':':亲吻_女人女人中等深肤色:','ru':':поцелуи_женщина_женщина_темныи_тон_кожи:','tr':':öpücük_kadın_kadın_orta_koyu_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FE':{'en':':kiss_woman_woman_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_und_mitteldunkle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_oscuro_medio:','fr':':bisou_femme_femme_et_peau_mate:','ja':':2人でキス_女性_女性_やや濃い肌色:','ko':':키스_여자_여자_진한_갈색_피부:','pt':':beijo_mulher_mulher_e_pele_morena_escura:','it':':bacio_tra_coppia_donna_donna_e_carnagione_abbastanza_scura:','fa':':بوسه_زن_زن_و_پوست_گندمی:','id':':berciuman_wanita_wanita_warna_kulit_gelap-sedang:','zh':':亲吻_女人女人中等深肤色:','ru':':поцелуй_женщина_женщина_темный_тон_кожи:','tr':':öpücük_kadın_kadın_orta_koyu_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FF':{'en':':kiss_woman_woman_medium-dark_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mitteldunkle_hautfarbe,dunkle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_oscuro_medio_tono_de_piel_oscuro:','fr':':bisou_femme_femme_peau_mate_et_peau_foncée:','ja':':2人でキス_女性_女性_やや濃い肌色_濃い肌色:','ko':':키스_여자_여자_진한_갈색_피부_검은색_피부:','pt':':beijo_mulher_mulher_pele_morena_escura_e_pele_escura:','it':':bacio_tra_coppia_donna_donna_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':بوسه_زن_زن_پوست_گندمی_و_پوست_آبنوسی:','id':':berciuman_wanita_wanita_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':亲吻_女人女人中等深肤色较深肤色:','ru':':поцелуи_женщина_женщина_темныи_тон_кожи_очень_темныи_тон_кожи:','tr':':öpücük_kadın_kadın_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FF':{'en':':kiss_woman_woman_medium-dark_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mitteldunkle_hautfarbe,dunkle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_oscuro_medio_tono_de_piel_oscuro:','fr':':bisou_femme_femme_peau_mate_et_peau_foncée:','ja':':2人でキス_女性_女性_やや濃い肌色_濃い肌色:','ko':':키스_여자_여자_진한_갈색_피부_검은색_피부:','pt':':beijo_mulher_mulher_pele_morena_escura_e_pele_escura:','it':':bacio_tra_coppia_donna_donna_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':بوسه_زن_زن_پوست_گندمی_و_پوست_آبنوسی:','id':':berciuman_wanita_wanita_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':亲吻_女人女人中等深肤色较深肤色:','ru':':поцелуй_женщина_женщина_темный_тон_кожи_очень_темный_тон_кожи:','tr':':öpücük_kadın_kadın_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FB':{'en':':kiss_woman_woman_medium-dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mitteldunkle_hautfarbe,helle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_oscuro_medio_tono_de_piel_claro:','fr':':bisou_femme_femme_peau_mate_et_peau_claire:','ja':':2人でキス_女性_女性_やや濃い肌色_薄い肌色:','ko':':키스_여자_여자_진한_갈색_피부_하얀_피부:','pt':':beijo_mulher_mulher_pele_morena_escura_e_pele_clara:','it':':bacio_tra_coppia_donna_donna_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':بوسه_زن_زن_پوست_گندمی_و_پوست_سفید:','id':':berciuman_wanita_wanita_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':亲吻_女人女人中等深肤色较浅肤色:','ru':':поцелуи_женщина_женщина_темныи_тон_кожи_очень_светлыи_тон_кожи:','tr':':öpücük_kadın_kadın_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FB':{'en':':kiss_woman_woman_medium-dark_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mitteldunkle_hautfarbe,helle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_oscuro_medio_tono_de_piel_claro:','fr':':bisou_femme_femme_peau_mate_et_peau_claire:','ja':':2人でキス_女性_女性_やや濃い肌色_薄い肌色:','ko':':키스_여자_여자_진한_갈색_피부_하얀_피부:','pt':':beijo_mulher_mulher_pele_morena_escura_e_pele_clara:','it':':bacio_tra_coppia_donna_donna_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':بوسه_زن_زن_پوست_گندمی_و_پوست_سفید:','id':':berciuman_wanita_wanita_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':亲吻_女人女人中等深肤色较浅肤色:','ru':':поцелуй_женщина_женщина_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':öpücük_kadın_kadın_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FC':{'en':':kiss_woman_woman_medium-dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mitteldunkle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_oscuro_medio_tono_de_piel_claro_medio:','fr':':bisou_femme_femme_peau_mate_et_peau_moyennement_claire:','ja':':2人でキス_女性_女性_やや濃い肌色_やや薄い肌色:','ko':':키스_여자_여자_진한_갈색_피부_연한_갈색_피부:','pt':':beijo_mulher_mulher_pele_morena_escura_e_pele_morena_clara:','it':':bacio_tra_coppia_donna_donna_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':بوسه_زن_زن_پوست_گندمی_و_پوست_روشن:','id':':berciuman_wanita_wanita_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':亲吻_女人女人中等深肤色中等浅肤色:','ru':':поцелуи_женщина_женщина_темныи_тон_кожи_светлыи_тон_кожи:','tr':':öpücük_kadın_kadın_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FC':{'en':':kiss_woman_woman_medium-dark_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mitteldunkle_hautfarbe,mittelhelle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_oscuro_medio_tono_de_piel_claro_medio:','fr':':bisou_femme_femme_peau_mate_et_peau_moyennement_claire:','ja':':2人でキス_女性_女性_やや濃い肌色_やや薄い肌色:','ko':':키스_여자_여자_진한_갈색_피부_연한_갈색_피부:','pt':':beijo_mulher_mulher_pele_morena_escura_e_pele_morena_clara:','it':':bacio_tra_coppia_donna_donna_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':بوسه_زن_زن_پوست_گندمی_و_پوست_روشن:','id':':berciuman_wanita_wanita_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':亲吻_女人女人中等深肤色中等浅肤色:','ru':':поцелуй_женщина_женщина_темный_тон_кожи_светлый_тон_кожи:','tr':':öpücük_kadın_kadın_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FD':{'en':':kiss_woman_woman_medium-dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mitteldunkle_hautfarbe,mittlere_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_oscuro_medio_tono_de_piel_medio:','fr':':bisou_femme_femme_peau_mate_et_peau_légèrement_mate:','ja':':2人でキス_女性_女性_やや濃い肌色_中間の肌色:','ko':':키스_여자_여자_진한_갈색_피부_갈색_피부:','pt':':beijo_mulher_mulher_pele_morena_escura_e_pele_morena:','it':':bacio_tra_coppia_donna_donna_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':بوسه_زن_زن_پوست_گندمی_و_پوست_طلایی:','id':':berciuman_wanita_wanita_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':亲吻_女人女人中等深肤色中等肤色:','ru':':поцелуи_женщина_женщина_темныи_тон_кожи_среднии_тон_кожи:','tr':':öpücük_kadın_kadın_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FE\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FD':{'en':':kiss_woman_woman_medium-dark_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mitteldunkle_hautfarbe,mittlere_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_oscuro_medio_tono_de_piel_medio:','fr':':bisou_femme_femme_peau_mate_et_peau_légèrement_mate:','ja':':2人でキス_女性_女性_やや濃い肌色_中間の肌色:','ko':':키스_여자_여자_진한_갈색_피부_갈색_피부:','pt':':beijo_mulher_mulher_pele_morena_escura_e_pele_morena:','it':':bacio_tra_coppia_donna_donna_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':بوسه_زن_زن_پوست_گندمی_و_پوست_طلایی:','id':':berciuman_wanita_wanita_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':亲吻_女人女人中等深肤色中等肤色:','ru':':поцелуй_женщина_женщина_темный_тон_кожи_средний_тон_кожи:','tr':':öpücük_kadın_kadın_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FC':{'en':':kiss_woman_woman_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_und_mittelhelle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_claro_medio:','fr':':bisou_femme_femme_et_peau_moyennement_claire:','ja':':2人でキス_女性_女性_やや薄い肌色:','ko':':키스_여자_여자_연한_갈색_피부:','pt':':beijo_mulher_mulher_e_pele_morena_clara:','it':':bacio_tra_coppia_donna_donna_e_carnagione_abbastanza_chiara:','fa':':بوسه_زن_زن_و_پوست_روشن:','id':':berciuman_wanita_wanita_warna_kulit_cerah-sedang:','zh':':亲吻_女人女人中等浅肤色:','ru':':поцелуи_женщина_женщина_светлыи_тон_кожи:','tr':':öpücük_kadın_kadın_orta_açık_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FC':{'en':':kiss_woman_woman_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_und_mittelhelle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_claro_medio:','fr':':bisou_femme_femme_et_peau_moyennement_claire:','ja':':2人でキス_女性_女性_やや薄い肌色:','ko':':키스_여자_여자_연한_갈색_피부:','pt':':beijo_mulher_mulher_e_pele_morena_clara:','it':':bacio_tra_coppia_donna_donna_e_carnagione_abbastanza_chiara:','fa':':بوسه_زن_زن_و_پوست_روشن:','id':':berciuman_wanita_wanita_warna_kulit_cerah-sedang:','zh':':亲吻_女人女人中等浅肤色:','ru':':поцелуй_женщина_женщина_светлый_тон_кожи:','tr':':öpücük_kadın_kadın_orta_açık_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FF':{'en':':kiss_woman_woman_medium-light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mittelhelle_hautfarbe,dunkle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_claro_medio_tono_de_piel_oscuro:','fr':':bisou_femme_femme_peau_moyennement_claire_et_peau_foncée:','ja':':2人でキス_女性_女性_やや薄い肌色_濃い肌色:','ko':':키스_여자_여자_연한_갈색_피부_검은색_피부:','pt':':beijo_mulher_mulher_pele_morena_clara_e_pele_escura:','it':':bacio_tra_coppia_donna_donna_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':بوسه_زن_زن_پوست_روشن_و_پوست_آبنوسی:','id':':berciuman_wanita_wanita_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':亲吻_女人女人中等浅肤色较深肤色:','ru':':поцелуи_женщина_женщина_светлыи_тон_кожи_очень_темныи_тон_кожи:','tr':':öpücük_kadın_kadın_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FF':{'en':':kiss_woman_woman_medium-light_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mittelhelle_hautfarbe,dunkle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_claro_medio_tono_de_piel_oscuro:','fr':':bisou_femme_femme_peau_moyennement_claire_et_peau_foncée:','ja':':2人でキス_女性_女性_やや薄い肌色_濃い肌色:','ko':':키스_여자_여자_연한_갈색_피부_검은색_피부:','pt':':beijo_mulher_mulher_pele_morena_clara_e_pele_escura:','it':':bacio_tra_coppia_donna_donna_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':بوسه_زن_زن_پوست_روشن_و_پوست_آبنوسی:','id':':berciuman_wanita_wanita_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':亲吻_女人女人中等浅肤色较深肤色:','ru':':поцелуй_женщина_женщина_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':öpücük_kadın_kadın_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FB':{'en':':kiss_woman_woman_medium-light_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mittelhelle_hautfarbe,helle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_claro_medio_tono_de_piel_claro:','fr':':bisou_femme_femme_peau_moyennement_claire_et_peau_claire:','ja':':2人でキス_女性_女性_やや薄い肌色_薄い肌色:','ko':':키스_여자_여자_연한_갈색_피부_하얀_피부:','pt':':beijo_mulher_mulher_pele_morena_clara_e_pele_clara:','it':':bacio_tra_coppia_donna_donna_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':بوسه_زن_زن_پوست_روشن_و_پوست_سفید:','id':':berciuman_wanita_wanita_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':亲吻_女人女人中等浅肤色较浅肤色:','ru':':поцелуи_женщина_женщина_светлыи_тон_кожи_очень_светлыи_тон_кожи:','tr':':öpücük_kadın_kadın_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FB':{'en':':kiss_woman_woman_medium-light_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mittelhelle_hautfarbe,helle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_claro_medio_tono_de_piel_claro:','fr':':bisou_femme_femme_peau_moyennement_claire_et_peau_claire:','ja':':2人でキス_女性_女性_やや薄い肌色_薄い肌色:','ko':':키스_여자_여자_연한_갈색_피부_하얀_피부:','pt':':beijo_mulher_mulher_pele_morena_clara_e_pele_clara:','it':':bacio_tra_coppia_donna_donna_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':بوسه_زن_زن_پوست_روشن_و_پوست_سفید:','id':':berciuman_wanita_wanita_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':亲吻_女人女人中等浅肤色较浅肤色:','ru':':поцелуй_женщина_женщина_светлый_тон_кожи_очень_светлый_тон_кожи:','tr':':öpücük_kadın_kadın_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FE':{'en':':kiss_woman_woman_medium-light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mittelhelle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_claro_medio_tono_de_piel_oscuro_medio:','fr':':bisou_femme_femme_peau_moyennement_claire_et_peau_mate:','ja':':2人でキス_女性_女性_やや薄い肌色_やや濃い肌色:','ko':':키스_여자_여자_연한_갈색_피부_진한_갈색_피부:','pt':':beijo_mulher_mulher_pele_morena_clara_e_pele_morena_escura:','it':':bacio_tra_coppia_donna_donna_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':بوسه_زن_زن_پوست_روشن_و_پوست_گندمی:','id':':berciuman_wanita_wanita_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':亲吻_女人女人中等浅肤色中等深肤色:','ru':':поцелуи_женщина_женщина_светлыи_тон_кожи_темныи_тон_кожи:','tr':':öpücük_kadın_kadın_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FE':{'en':':kiss_woman_woman_medium-light_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mittelhelle_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_claro_medio_tono_de_piel_oscuro_medio:','fr':':bisou_femme_femme_peau_moyennement_claire_et_peau_mate:','ja':':2人でキス_女性_女性_やや薄い肌色_やや濃い肌色:','ko':':키스_여자_여자_연한_갈색_피부_진한_갈색_피부:','pt':':beijo_mulher_mulher_pele_morena_clara_e_pele_morena_escura:','it':':bacio_tra_coppia_donna_donna_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':بوسه_زن_زن_پوست_روشن_و_پوست_گندمی:','id':':berciuman_wanita_wanita_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':亲吻_女人女人中等浅肤色中等深肤色:','ru':':поцелуй_женщина_женщина_светлый_тон_кожи_темный_тон_кожи:','tr':':öpücük_kadın_kadın_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FD':{'en':':kiss_woman_woman_medium-light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mittelhelle_hautfarbe,mittlere_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_claro_medio_tono_de_piel_medio:','fr':':bisou_femme_femme_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':2人でキス_女性_女性_やや薄い肌色_中間の肌色:','ko':':키스_여자_여자_연한_갈색_피부_갈색_피부:','pt':':beijo_mulher_mulher_pele_morena_clara_e_pele_morena:','it':':bacio_tra_coppia_donna_donna_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':بوسه_زن_زن_پوست_روشن_و_پوست_طلایی:','id':':berciuman_wanita_wanita_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':亲吻_女人女人中等浅肤色中等肤色:','ru':':поцелуи_женщина_женщина_светлыи_тон_кожи_среднии_тон_кожи:','tr':':öpücük_kadın_kadın_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FC\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FD':{'en':':kiss_woman_woman_medium-light_skin_tone_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mittelhelle_hautfarbe,mittlere_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_claro_medio_tono_de_piel_medio:','fr':':bisou_femme_femme_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':2人でキス_女性_女性_やや薄い肌色_中間の肌色:','ko':':키스_여자_여자_연한_갈색_피부_갈색_피부:','pt':':beijo_mulher_mulher_pele_morena_clara_e_pele_morena:','it':':bacio_tra_coppia_donna_donna_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':بوسه_زن_زن_پوست_روشن_و_پوست_طلایی:','id':':berciuman_wanita_wanita_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':亲吻_女人女人中等浅肤色中等肤色:','ru':':поцелуй_женщина_женщина_светлый_тон_кожи_средний_тон_кожи:','tr':':öpücük_kadın_kadın_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FD':{'en':':kiss_woman_woman_medium_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_und_mittlere_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_medio:','fr':':bisou_femme_femme_et_peau_légèrement_mate:','ja':':2人でキス_女性_女性_中間の肌色:','ko':':키스_여자_여자_갈색_피부:','pt':':beijo_mulher_mulher_e_pele_morena:','it':':bacio_tra_coppia_donna_donna_e_carnagione_olivastra:','fa':':بوسه_زن_زن_و_پوست_طلایی:','id':':berciuman_wanita_wanita_warna_kulit_sedang:','zh':':亲吻_女人女人中等肤色:','ru':':поцелуи_женщина_женщина_среднии_тон_кожи:','tr':':öpücük_kadın_kadın_orta_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FD':{'en':':kiss_woman_woman_medium_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_und_mittlere_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_medio:','fr':':bisou_femme_femme_et_peau_légèrement_mate:','ja':':2人でキス_女性_女性_中間の肌色:','ko':':키스_여자_여자_갈색_피부:','pt':':beijo_mulher_mulher_e_pele_morena:','it':':bacio_tra_coppia_donna_donna_e_carnagione_olivastra:','fa':':بوسه_زن_زن_و_پوست_طلایی:','id':':berciuman_wanita_wanita_warna_kulit_sedang:','zh':':亲吻_女人女人中等肤色:','ru':':поцелуй_женщина_женщина_средний_тон_кожи:','tr':':öpücük_kadın_kadın_orta_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FF':{'en':':kiss_woman_woman_medium_skin_tone_dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mittlere_hautfarbe,dunkle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_medio_tono_de_piel_oscuro:','fr':':bisou_femme_femme_peau_légèrement_mate_et_peau_foncée:','ja':':2人でキス_女性_女性_中間の肌色_濃い肌色:','ko':':키스_여자_여자_갈색_피부_검은색_피부:','pt':':beijo_mulher_mulher_pele_morena_e_pele_escura:','it':':bacio_tra_coppia_donna_donna_carnagione_olivastra_e_carnagione_scura:','fa':':بوسه_زن_زن_پوست_طلایی_و_پوست_آبنوسی:','id':':berciuman_wanita_wanita_warna_kulit_sedang_warna_kulit_gelap:','zh':':亲吻_女人女人中等肤色较深肤色:','ru':':поцелуи_женщина_женщина_среднии_тон_кожи_очень_темныи_тон_кожи:','tr':':öpücük_kadın_kadın_orta_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FF':{'en':':kiss_woman_woman_medium_skin_tone_dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mittlere_hautfarbe,dunkle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_medio_tono_de_piel_oscuro:','fr':':bisou_femme_femme_peau_légèrement_mate_et_peau_foncée:','ja':':2人でキス_女性_女性_中間の肌色_濃い肌色:','ko':':키스_여자_여자_갈색_피부_검은색_피부:','pt':':beijo_mulher_mulher_pele_morena_e_pele_escura:','it':':bacio_tra_coppia_donna_donna_carnagione_olivastra_e_carnagione_scura:','fa':':بوسه_زن_زن_پوست_طلایی_و_پوست_آبنوسی:','id':':berciuman_wanita_wanita_warna_kulit_sedang_warna_kulit_gelap:','zh':':亲吻_女人女人中等肤色较深肤色:','ru':':поцелуй_женщина_женщина_средний_тон_кожи_очень_темный_тон_кожи:','tr':':öpücük_kadın_kadın_orta_cilt_tonu_koyu_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FB':{'en':':kiss_woman_woman_medium_skin_tone_light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mittlere_hautfarbe,helle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_medio_tono_de_piel_claro:','fr':':bisou_femme_femme_peau_légèrement_mate_et_peau_claire:','ja':':2人でキス_女性_女性_中間の肌色_薄い肌色:','ko':':키스_여자_여자_갈색_피부_하얀_피부:','pt':':beijo_mulher_mulher_pele_morena_e_pele_clara:','it':':bacio_tra_coppia_donna_donna_carnagione_olivastra_e_carnagione_chiara:','fa':':بوسه_زن_زن_پوست_طلایی_و_پوست_سفید:','id':':berciuman_wanita_wanita_warna_kulit_sedang_warna_kulit_cerah:','zh':':亲吻_女人女人中等肤色较浅肤色:','ru':':поцелуи_женщина_женщина_среднии_тон_кожи_очень_светлыи_тон_кожи:','tr':':öpücük_kadın_kadın_orta_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FB':{'en':':kiss_woman_woman_medium_skin_tone_light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mittlere_hautfarbe,helle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_medio_tono_de_piel_claro:','fr':':bisou_femme_femme_peau_légèrement_mate_et_peau_claire:','ja':':2人でキス_女性_女性_中間の肌色_薄い肌色:','ko':':키스_여자_여자_갈색_피부_하얀_피부:','pt':':beijo_mulher_mulher_pele_morena_e_pele_clara:','it':':bacio_tra_coppia_donna_donna_carnagione_olivastra_e_carnagione_chiara:','fa':':بوسه_زن_زن_پوست_طلایی_و_پوست_سفید:','id':':berciuman_wanita_wanita_warna_kulit_sedang_warna_kulit_cerah:','zh':':亲吻_女人女人中等肤色较浅肤色:','ru':':поцелуй_женщина_женщина_средний_тон_кожи_очень_светлый_тон_кожи:','tr':':öpücük_kadın_kadın_orta_cilt_tonu_açık_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FE':{'en':':kiss_woman_woman_medium_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mittlere_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_medio_tono_de_piel_oscuro_medio:','fr':':bisou_femme_femme_peau_légèrement_mate_et_peau_mate:','ja':':2人でキス_女性_女性_中間の肌色_やや濃い肌色:','ko':':키스_여자_여자_갈색_피부_진한_갈색_피부:','pt':':beijo_mulher_mulher_pele_morena_e_pele_morena_escura:','it':':bacio_tra_coppia_donna_donna_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':بوسه_زن_زن_پوست_طلایی_و_پوست_گندمی:','id':':berciuman_wanita_wanita_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':亲吻_女人女人中等肤色中等深肤色:','ru':':поцелуи_женщина_женщина_среднии_тон_кожи_темныи_тон_кожи:','tr':':öpücük_kadın_kadın_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FE':{'en':':kiss_woman_woman_medium_skin_tone_medium-dark_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mittlere_hautfarbe,mitteldunkle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_medio_tono_de_piel_oscuro_medio:','fr':':bisou_femme_femme_peau_légèrement_mate_et_peau_mate:','ja':':2人でキス_女性_女性_中間の肌色_やや濃い肌色:','ko':':키스_여자_여자_갈색_피부_진한_갈색_피부:','pt':':beijo_mulher_mulher_pele_morena_e_pele_morena_escura:','it':':bacio_tra_coppia_donna_donna_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':بوسه_زن_زن_پوست_طلایی_و_پوست_گندمی:','id':':berciuman_wanita_wanita_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':亲吻_女人女人中等肤色中等深肤色:','ru':':поцелуй_женщина_женщина_средний_тон_кожи_темный_тон_кожи:','tr':':öpücük_kadın_kadın_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000FE0F\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FC':{'en':':kiss_woman_woman_medium_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mittlere_hautfarbe,mittelhelle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_medio_tono_de_piel_claro_medio:','fr':':bisou_femme_femme_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':2人でキス_女性_女性_中間の肌色_やや薄い肌色:','ko':':키스_여자_여자_갈색_피부_연한_갈색_피부:','pt':':beijo_mulher_mulher_pele_morena_e_pele_morena_clara:','it':':bacio_tra_coppia_donna_donna_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':بوسه_زن_زن_پوست_طلایی_و_پوست_روشن:','id':':berciuman_wanita_wanita_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':亲吻_女人女人中等肤色中等浅肤色:','ru':':поцелуи_женщина_женщина_среднии_тон_кожи_светлыи_тон_кожи:','tr':':öpücük_kadın_kadın_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002764\U0000200D\U0001F48B\U0000200D\U0001F469\U0001F3FC':{'en':':kiss_woman_woman_medium_skin_tone_medium-light_skin_tone:','status':minimally_qualified,'E':13.1,'de':':sich_küssendes_paar_frau_frau_mittlere_hautfarbe,mittelhelle_hautfarbe:','es':':beso_mujer_mujer_tono_de_piel_medio_tono_de_piel_claro_medio:','fr':':bisou_femme_femme_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':2人でキス_女性_女性_中間の肌色_やや薄い肌色:','ko':':키스_여자_여자_갈색_피부_연한_갈색_피부:','pt':':beijo_mulher_mulher_pele_morena_e_pele_morena_clara:','it':':bacio_tra_coppia_donna_donna_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':بوسه_زن_زن_پوست_طلایی_و_پوست_روشن:','id':':berciuman_wanita_wanita_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':亲吻_女人女人中等肤色中等浅肤色:','ru':':поцелуй_женщина_женщина_средний_тон_кожи_светлый_тон_кожи:','tr':':öpücük_kadın_kadın_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':قبلة_سيدة_وسيدة_وبشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F63D':{'en':':kissing_cat:','status':fully_qualified,'E':0.6,'de':':küssende_katze:','es':':gato_besando:','fr':':chat_qui_fait_un_bisou:','ja':':猫のキス:','ko':':키스하는_고양이_얼굴:','pt':':rosto_de_gato_mandando_um_beijo:','it':':gatto_che_manda_baci:','fa':':گربه_و_بوسهٔ_خجالتی:','id':':wajah_kucing_memberikan_ciuman_dengan_mata_tertutup:','zh':':亲亲猫:','ru':':целующий_кот:','tr':':öpen_kedi:','ar':':وجه_قطة_تقبل:'},'\U0001F617':{'en':':kissing_face:','status':fully_qualified,'E':1,'alias':[':kissing:'],'de':':küssendes_gesicht:','es':':cara_besando:','fr':':visage_faisant_un_bisou:','ja':':キス:','ko':':키스하는_얼굴:','pt':':rosto_beijando:','it':':faccina_che_bacia:','fa':':بوسیدن:','id':':wajah_memberikan_ciuman:','zh':':亲亲:','ru':':целует:','tr':':öpen_yüz:','ar':':وجه_يقبل:'},'\U0001F61A':{'en':':kissing_face_with_closed_eyes:','status':fully_qualified,'E':0.6,'alias':[':kissing_closed_eyes:'],'de':':küssendes_gesicht_mit_geschlossenen_augen:','es':':cara_besando_con_los_ojos_cerrados:','fr':':visage_faisant_un_bisou_avec_les_yeux_fermés:','ja':':ちゅっ:','ko':':눈을_감은_채로_키스하는_얼굴:','pt':':rosto_beijando_com_olhos_fechados:','it':':faccina_che_bacia_con_occhi_chiusi:','fa':':بوسهٔ_خجالتی:','id':':wajah_memberikan_ciuman_dengan_mata_tertutup:','zh':':羞涩亲亲:','ru':':целует_с_закрытыми_глазами:','tr':':kapalı_gözlerle_öpen_yüz:','ar':':وجه_يقبل_بعينين_مغلقتين:'},'\U0001F619':{'en':':kissing_face_with_smiling_eyes:','status':fully_qualified,'E':1,'alias':[':kissing_smiling_eyes:'],'de':':küssendes_gesicht_mit_lächelnden_augen:','es':':cara_besando_con_ojos_sonrientes:','fr':':visage_aux_yeux_rieurs_faisant_un_bisou:','ja':':にっこりキス:','ko':':미소_짓는_눈으로_키스하는_얼굴:','pt':':rosto_beijando_com_olhos_sorridentes:','it':':faccina_che_bacia_con_occhi_sorridenti:','fa':':بوسیدن_با_چشمان_خندان:','id':':wajah_memberikan_ciuman_dengan_mata_bahagia:','zh':':微笑亲亲:','ru':':счастливый_поцелуй:','tr':':gülen_gözlerle_öpen_yüz:','ar':':وجه_يقبل_بعينين_باسمتين:'},'\U0001F52A':{'en':':kitchen_knife:','status':fully_qualified,'E':0.6,'alias':[':hocho:', ':knife:'],'de':':küchenmesser:','es':':cuchillo_de_cocina:','fr':':couteau_de_cuisine:','ja':':包丁:','ko':':식칼:','pt':':faca_de_cozinha:','it':':coltello_da_cucina:','fa':':چاقوی_آشپزخانه:','id':':pisau_dapur:','zh':':菜刀:','ru':':нож:','tr':':mutfak_bıçağı:','ar':':سكين_طهي:'},'\U0001FA81':{'en':':kite:','status':fully_qualified,'E':12,'de':':drachen:','es':':cometa:','fr':':cerf-volant:','ja':':たこ:','ko':':연:','pt':':pipa:','it':':aquilone:','fa':':بادبادک:','id':':layang-layang:','zh':':风筝:','ru':':воздушный_змей:','tr':':uçurtma:','ar':':طائرة_ورقية:'},'\U0001F95D':{'en':':kiwi_fruit:','status':fully_qualified,'E':3,'de':':kiwi:','es':':kiwi:','fr':':kiwi:','ja':':キウイフルーツ:','ko':':키위:','pt':':kiwi:','it':':kiwi:','fa':':کیوی:','id':':buah_kiwi:','zh':':猕猴桃:','ru':':киви:','tr':':kivi:','ar':':كيوي:'},'\U0001FAA2':{'en':':knot:','status':fully_qualified,'E':13,'de':':knoten:','es':':nudo:','fr':':nœud:','ja':':結び目:','ko':':매듭:','pt':':nó:','it':':nodo:','fa':':گره:','id':':simpul:','zh':':结:','ru':':узел:','tr':':denizci_düğümü:','ar':':عُقدة:'},'\U0001F428':{'en':':koala:','status':fully_qualified,'E':0.6,'de':':koala:','es':':koala:','fr':':koala:','ja':':コアラ:','ko':':코알라:','pt':':coala:','it':':koala:','fa':':کوآلا:','id':':koala:','zh':':考拉:','ru':':морда_коалы:','tr':':koala:','ar':':دب_كوالا:'},'\U0001F97C':{'en':':lab_coat:','status':fully_qualified,'E':11,'de':':laborkittel:','es':':bata_de_laboratorio:','fr':':blouse_blanche:','ja':':白衣:','ko':':실험실_가운:','pt':':jaleco:','it':':camice:','fa':':روپوش_آزمایشگاه:','id':':jas_laboratorium:','zh':':白大褂:','ru':':лабораторный_халат:','tr':':laboratuvar_önlüğü:','ar':':معطف_مختبر:'},'\U0001F3F7\U0000FE0F':{'en':':label:','status':fully_qualified,'E':0.7,'variant':True,'de':':etikett:','es':':etiqueta:','fr':':étiquette:','ja':':荷札:','ko':':라벨:','pt':':etiqueta:','it':':etichetta:','fa':':برچسب:','id':':label:','zh':':标签:','ru':':бирка:','tr':':etiket:','ar':':ملصق:'},'\U0001F3F7':{'en':':label:','status':unqualified,'E':0.7,'variant':True,'de':':etikett:','es':':etiqueta:','fr':':étiquette:','ja':':荷札:','ko':':라벨:','pt':':etiqueta:','it':':etichetta:','fa':':برچسب:','id':':label:','zh':':标签:','ru':':бирка:','tr':':etiket:','ar':':ملصق:'},'\U0001F94D':{'en':':lacrosse:','status':fully_qualified,'E':11,'de':':lacrosse:','es':':lacrosse:','fr':':crosse:','ja':':ラクロス:','ko':':라크로스:','pt':':lacrosse:','it':':lacrosse:','fa':':لاکراس:','id':':lacrosse:','zh':':袋棍球:','ru':':лакросс:','tr':':lakros:','ar':':لاكروس:'},'\U0001FA9C':{'en':':ladder:','status':fully_qualified,'E':13,'de':':leiter:','es':':escalera:','fr':':échelle:','ja':':はしご:','ko':':사다리:','pt':':escada:','it':':scala:','fa':':نردبان:','id':':tangga:','zh':':梯子:','ru':':лестница:','tr':':merdiven:','ar':':سلم:'},'\U0001F41E':{'en':':lady_beetle:','status':fully_qualified,'E':0.6,'alias':[':beetle:'],'de':':marienkäfer:','es':':mariquita:','fr':':coccinelle:','ja':':テントウムシ:','ko':':무당벌레:','pt':':joaninha:','it':':coccinella:','fa':':کفشدوزک:','id':':kumbang_betina:','zh':':瓢虫:','ru':':божья_коровка:','tr':':uğur_böceği:','ar':':دعسوقة:'},'\U0001F4BB':{'en':':laptop:','status':fully_qualified,'E':0.6,'alias':[':computer:'],'variant':True,'de':':laptop:','es':':ordenador_portátil:','fr':':ordinateur_portable:','ja':':ノートパソコン:','ko':':노트북:','pt':':laptop:','it':':computer_portatile:','fa':':رایانهٔ_کیفی:','id':':komputer_laptop:','zh':':笔记本电脑:','ru':':ноутбук:','tr':':dizüstü_bilgisayar:','ar':':كمبيوتر_محمول:'},'\U0001F537':{'en':':large_blue_diamond:','status':fully_qualified,'E':0.6,'de':':große_blaue_raute:','es':':rombo_azul_grande:','fr':':grand_losange_bleu:','ja':':大きい青のひし形:','ko':':큰_파란색_마름모:','pt':':losango_azul_grande:','it':':rombo_blu_grande:','fa':':لوزی_آبی_بزرگ:','id':':wajik_biru_besar:','zh':':蓝色大菱形:','ru':':большой_голубой_ромб:','tr':':büyük_mavi_karo:','ar':':ماسة_زرقاء_كبيرة:'},'\U0001F536':{'en':':large_orange_diamond:','status':fully_qualified,'E':0.6,'de':':große_orangefarbene_raute:','es':':rombo_naranja_grande:','fr':':grand_losange_orange:','ja':':大きいオレンジのひし形:','ko':':큰_주황색_마름모:','pt':':losango_laranja_grande:','it':':rombo_arancione_grande:','fa':':لوزی_نارنجی_بزرگ:','id':':wajik_oranye_besar:','zh':':橙色大菱形:','ru':':большой_оранжевый_ромб:','tr':':büyük_turuncu_karo:','ar':':ماسة_برتقالي_كبيرة:'},'\U0001F317':{'en':':last_quarter_moon:','status':fully_qualified,'E':1,'de':':abnehmender_halbmond:','es':':luna_en_cuarto_menguante:','fr':':dernier_quartier_de_lune:','ja':':下弦の月:','ko':':왼쪽_반달:','pt':':quarto_minguante:','it':':ultimo_quarto_di_luna:','fa':':یک_چهارم_پایان:','id':':bulan_perbani_akhir:','zh':':下弦月:','ru':':луна_в_последней_четверти:','tr':':son_dördün:','ar':':تربيع_ثان:'},'\U0001F31C':{'en':':last_quarter_moon_face:','status':fully_qualified,'E':0.7,'alias':[':last_quarter_moon_with_face:'],'variant':True,'de':':mondsichel_mit_gesicht_rechts:','es':':luna_de_cuarto_menguante_con_cara:','fr':':dernier_quartier_de_lune_avec_visage:','ja':':顔のある下弦の月:','ko':':그믐달:','pt':':rosto_da_lua_de_quarto_minguante:','it':':faccina_ultimo_quarto_di_luna:','fa':':صورت_یک_چهارم_آخر:','id':':wajah_bulan_perbani_akhir:','zh':':微笑的下弦月:','ru':':луна_в_последней_четверти_с_лицом:','tr':':son_dördün_ay_yüzü:','ar':':وجه_تربيع_ثان:'},'\U000023EE\U0000FE0F':{'en':':last_track_button:','status':fully_qualified,'E':0.7,'alias':[':previous_track_button:', ':black_left_pointing_double_triangle_with_vertical_bar:'],'variant':True,'de':':vorheriger_titel:','es':':pista_anterior:','fr':':bouton_piste_précédente:','ja':':前の曲ボタン:','ko':':마지막_트랙_버튼:','pt':':botão_de_faixa_anterior:','it':':pulsante_traccia_precedente:','fa':':دکمه_آهنگ_قبلی:','id':':tombol_lagu_terakhir:','zh':':上一个音轨按钮:','ru':':предыдущии_трек:','tr':':son_parça_düğmesi:','ar':':زر_آخر_مقطع:'},'\U000023EE':{'en':':last_track_button:','status':unqualified,'E':0.7,'alias':[':previous_track_button:', ':black_left_pointing_double_triangle_with_vertical_bar:'],'variant':True,'de':':vorheriger_titel:','es':':pista_anterior:','fr':':bouton_piste_précédente:','ja':':前の曲ボタン:','ko':':마지막_트랙_버튼:','pt':':botão_de_faixa_anterior:','it':':pulsante_traccia_precedente:','fa':':دکمهٔ_آهنگ_قبلی:','id':':tombol_lagu_terakhir:','zh':':上一个音轨按钮:','ru':':предыдущий_трек:','tr':':son_parça_düğmesi:','ar':':زر_آخر_مقطع:'},'\U0000271D\U0000FE0F':{'en':':latin_cross:','status':fully_qualified,'E':0.7,'variant':True,'de':':römisches_kreuz:','es':':cruz_latina:','fr':':croix_latine:','ja':':十字架:','ko':':라틴_십자가:','pt':':cruz_latina:','it':':croce_latina:','fa':':صلیب:','id':':salib_latin:','zh':':十字架:','ru':':латинскии_крест:','tr':':latin_haçı:','ar':':صليب_لاتيني:'},'\U0000271D':{'en':':latin_cross:','status':unqualified,'E':0.7,'variant':True,'de':':lateinisches_kreuz:','es':':cruz_latina:','fr':':croix_latine:','ja':':十字架:','ko':':라틴_십자가:','pt':':cruz_latina:','it':':croce_latina:','fa':':صلیب:','id':':salib_latin:','zh':':十字架:','ru':':латинский_крест:','tr':':latin_haçı:','ar':':صليب_لاتيني:'},'\U0001F343':{'en':':leaf_fluttering_in_wind:','status':fully_qualified,'E':0.6,'alias':[':leaves:'],'de':':blätter_im_wind:','es':':hojas_revoloteando_al_viento:','fr':':feuille_virevoltante:','ja':':風に揺れる葉:','ko':':바람에_흔들리는_나뭇잎:','pt':':folha_ao_vento:','it':':foglia_al_vento:','fa':':رقص_برگ_در_باد:','id':':daun_tertiup_angin:','zh':':风吹叶落:','ru':':листья_на_ветру:','tr':':rüzgardaki_yaprak:','ar':':ورقة_متطايرة:'},'\U0001F96C':{'en':':leafy_green:','status':fully_qualified,'E':11,'de':':blattgemüse:','es':':verdura_de_hoja_verde:','fr':':légume_à_feuilles_vertes:','ja':':葉野菜:','ko':':녹색_채소:','pt':':verdura:','it':':verdure_a_foglia:','fa':':برگ_سبز:','id':':selada_hijau:','zh':':绿叶蔬菜:','ru':':зеленый_салат:','tr':':yeşil_yapraklı_sebze:','ar':':خضار_ورقي:'},'\U0001F4D2':{'en':':ledger:','status':fully_qualified,'E':0.6,'de':':spiralblock:','es':':libro_de_contabilidad:','fr':':carnet_de_compte:','ja':':リングノート:','ko':':노트:','pt':':livro_contábil:','it':':quaderno_ad_anelli:','fa':':دفتر_سیمی:','id':':buku_besar:','zh':':账本:','ru':':блокнот:','tr':':resmi_defter:','ar':':دفتر:'},'\U0001F91B':{'en':':left-facing_fist:','status':fully_qualified,'E':3,'alias':[':fist_left:', ':left_facing_fist:'],'de':':faust_nach_links:','es':':puño_hacia_la_izquierda:','fr':':poing_à_gauche:','ja':':左向きのこぶし:','ko':':왼쪽을_향하는_주먹:','pt':':punho_esquerdo:','it':':pugno_a_sinistra:','fa':':مشت_به_چپ:','id':':tinju_kiri:','zh':':朝左的拳头:','ru':':кулак_влево:','tr':':sola_bakan_yumruk:','ar':':قبضة_لليسار:'},'\U0001F91B\U0001F3FF':{'en':':left-facing_fist_dark_skin_tone:','status':fully_qualified,'E':3,'de':':faust_nach_links_dunkle_hautfarbe:','es':':puño_hacia_la_izquierda_tono_de_piel_oscuro:','fr':':poing_à_gauche_peau_foncée:','ja':':左向きのこぶし_濃い肌色:','ko':':왼쪽을_향하는_주먹_검은색_피부:','pt':':punho_esquerdo_pele_escura:','it':':pugno_a_sinistra_carnagione_scura:','fa':':مشت_به_چپ_پوست_آبنوسی:','id':':tinju_kiri_warna_kulit_gelap:','zh':':朝左的拳头_较深肤色:','ru':':кулак_влево_очень_темный_тон_кожи:','tr':':sola_bakan_yumruk_koyu_cilt_tonu:','ar':':قبضة_لليسار_بشرة_بلون_غامق:'},'\U0001F91B\U0001F3FB':{'en':':left-facing_fist_light_skin_tone:','status':fully_qualified,'E':3,'de':':faust_nach_links_helle_hautfarbe:','es':':puño_hacia_la_izquierda_tono_de_piel_claro:','fr':':poing_à_gauche_peau_claire:','ja':':左向きのこぶし_薄い肌色:','ko':':왼쪽을_향하는_주먹_하얀_피부:','pt':':punho_esquerdo_pele_clara:','it':':pugno_a_sinistra_carnagione_chiara:','fa':':مشت_به_چپ_پوست_سفید:','id':':tinju_kiri_warna_kulit_cerah:','zh':':朝左的拳头_较浅肤色:','ru':':кулак_влево_очень_светлый_тон_кожи:','tr':':sola_bakan_yumruk_açık_cilt_tonu:','ar':':قبضة_لليسار_بشرة_بلون_فاتح:'},'\U0001F91B\U0001F3FE':{'en':':left-facing_fist_medium-dark_skin_tone:','status':fully_qualified,'E':3,'de':':faust_nach_links_mitteldunkle_hautfarbe:','es':':puño_hacia_la_izquierda_tono_de_piel_oscuro_medio:','fr':':poing_à_gauche_peau_mate:','ja':':左向きのこぶし_やや濃い肌色:','ko':':왼쪽을_향하는_주먹_진한_갈색_피부:','pt':':punho_esquerdo_pele_morena_escura:','it':':pugno_a_sinistra_carnagione_abbastanza_scura:','fa':':مشت_به_چپ_پوست_گندمی:','id':':tinju_kiri_warna_kulit_gelap-sedang:','zh':':朝左的拳头_中等深肤色:','ru':':кулак_влево_темный_тон_кожи:','tr':':sola_bakan_yumruk_orta_koyu_cilt_tonu:','ar':':قبضة_لليسار_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F91B\U0001F3FC':{'en':':left-facing_fist_medium-light_skin_tone:','status':fully_qualified,'E':3,'de':':faust_nach_links_mittelhelle_hautfarbe:','es':':puño_hacia_la_izquierda_tono_de_piel_claro_medio:','fr':':poing_à_gauche_peau_moyennement_claire:','ja':':左向きのこぶし_やや薄い肌色:','ko':':왼쪽을_향하는_주먹_연한_갈색_피부:','pt':':punho_esquerdo_pele_morena_clara:','it':':pugno_a_sinistra_carnagione_abbastanza_chiara:','fa':':مشت_به_چپ_پوست_روشن:','id':':tinju_kiri_warna_kulit_cerah-sedang:','zh':':朝左的拳头_中等浅肤色:','ru':':кулак_влево_светлый_тон_кожи:','tr':':sola_bakan_yumruk_orta_açık_cilt_tonu:','ar':':قبضة_لليسار_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F91B\U0001F3FD':{'en':':left-facing_fist_medium_skin_tone:','status':fully_qualified,'E':3,'de':':faust_nach_links_mittlere_hautfarbe:','es':':puño_hacia_la_izquierda_tono_de_piel_medio:','fr':':poing_à_gauche_peau_légèrement_mate:','ja':':左向きのこぶし_中間の肌色:','ko':':왼쪽을_향하는_주먹_갈색_피부:','pt':':punho_esquerdo_pele_morena:','it':':pugno_a_sinistra_carnagione_olivastra:','fa':':مشت_به_چپ_پوست_طلایی:','id':':tinju_kiri_warna_kulit_sedang:','zh':':朝左的拳头_中等肤色:','ru':':кулак_влево_средний_тон_кожи:','tr':':sola_bakan_yumruk_orta_cilt_tonu:','ar':':قبضة_لليسار_بشرة_بلون_معتدل:'},'\U00002194\U0000FE0F':{'en':':left-right_arrow:','status':fully_qualified,'E':0.6,'alias':[':left_right_arrow:'],'variant':True,'de':':pfeil_nach_links_und_rechts:','es':':flecha_izquierda_y_derecha:','fr':':flèche_gauche_droite:','ja':':左右矢印:','ko':':좌우향_화살표:','pt':':seta_para_esquerda_e_direita:','it':':freccia_sinistra-destra:','fa':':پیکان_دوطرفه:','id':':tanda_panah_kiri_kanan:','zh':':左右箭头:','ru':':стрелка_влево-вправо:','tr':':sol_ve_sağ_ok:','ar':':سهم_لليسار_واليمين:'},'\U00002194':{'en':':left-right_arrow:','status':unqualified,'E':0.6,'alias':[':left_right_arrow:'],'variant':True,'de':':pfeil_nach_links_und_rechts:','es':':flecha_izquierda_y_derecha:','fr':':flèche_gauche_droite:','ja':':左右矢印:','ko':':좌우향_화살표:','pt':':seta_para_esquerda_e_direita:','it':':freccia_sinistra-destra:','fa':':پیکان_دوطرفه:','id':':tanda_panah_kiri_kanan:','zh':':左右箭头:','ru':':стрелка_влево_вправо:','tr':':sol_ve_sağ_ok:','ar':':سهم_لليسار_واليمين:'},'\U00002B05\U0000FE0F':{'en':':left_arrow:','status':fully_qualified,'E':0.6,'alias':[':arrow_left:'],'variant':True,'de':':pfeil_nach_links:','es':':flecha_hacia_la_izquierda:','fr':':flèche_gauche:','ja':':左矢印:','ko':':좌향_화살표:','pt':':seta_para_a_esquerda:','it':':freccia_rivolta_a_sinistra:','fa':':پیکان_چپ:','id':':tanda_panah_kiri:','zh':':向左箭头:','ru':':стрелка_влево:','tr':':sol_ok:','ar':':سهم_لليسار:'},'\U00002B05':{'en':':left_arrow:','status':unqualified,'E':0.6,'alias':[':arrow_left:'],'variant':True,'de':':pfeil_nach_links:','es':':flecha_hacia_la_izquierda:','fr':':flèche_gauche:','ja':':左矢印:','ko':':좌향_화살표:','pt':':seta_para_a_esquerda:','it':':freccia_rivolta_a_sinistra:','fa':':پیکان_چپ:','id':':tanda_panah_kiri:','zh':':向左箭头:','ru':':стрелка_влево:','tr':':sol_ok:','ar':':سهم_لليسار:'},'\U000021AA\U0000FE0F':{'en':':left_arrow_curving_right:','status':fully_qualified,'E':0.6,'alias':[':arrow_right_hook:'],'variant':True,'de':':geschwungener_pfeil_nach_rechts:','es':':flecha_izquierda_curvándose_a_la_derecha:','fr':':flèche_courbe_droite:','ja':':左カーブ矢印:','ko':':오른쪽으로_꺾어지는_좌향_화살표:','pt':':seta_curva_da_esquerda_para_a_direita:','it':':freccia_curva_a_destra:','fa':':پیکان_خمیده_چپ:','id':':tanda_panah_kanan_melengkung_ke_kanan:','zh':':左转弯箭头:','ru':':стрелка_влево_с_поворотом_вправо:','tr':':sağa_kıvrımlı_sol_ok:','ar':':سهم_أيسر_بانحناء_أيمن:'},'\U000021AA':{'en':':left_arrow_curving_right:','status':unqualified,'E':0.6,'alias':[':arrow_right_hook:'],'variant':True,'de':':geschwungener_pfeil_nach_rechts:','es':':flecha_izquierda_curvándose_a_la_derecha:','fr':':flèche_courbe_droite:','ja':':左カーブ矢印:','ko':':오른쪽으로_꺾어지는_좌향_화살표:','pt':':seta_curva_da_esquerda_para_a_direita:','it':':freccia_curva_a_destra:','fa':':پیکان_خمیده_چپ:','id':':tanda_panah_kanan_melengkung_ke_kanan:','zh':':左转弯箭头:','ru':':стрелка_влево_с_поворотом_вправо:','tr':':sağa_kıvrımlı_sol_ok:','ar':':سهم_أيسر_بانحناء_أيمن:'},'\U0001F6C5':{'en':':left_luggage:','status':fully_qualified,'E':1,'de':':gepäckaufbewahrung:','es':':consigna:','fr':':consigne:','ja':':手荷物預かり:','ko':':화물_서비스:','pt':':depósito_de_bagagem:','it':':simbolo_del_deposito_bagagli:','fa':':امانت_بار:','id':':koper_tertinggal:','zh':':寄存行李:','ru':':забытый_багаж:','tr':':eşya_emanet:','ar':':أمتعة:'},'\U0001F5E8\U0000FE0F':{'en':':left_speech_bubble:','status':fully_qualified,'E':2,'variant':True,'de':':sprechblase_links:','es':':bocadillo_de_diálogo_por_la_izquierda:','fr':':bulle_de_parole_gauche:','ja':':吹き出し左:','ko':':왼쪽_말풍선:','pt':':balão_de_diálogo_à_esquerda:','it':':nuvoletta_nera:','fa':':حباب_گفتار_چپ:','id':':gelembung_ucapan_kiri:','zh':':朝左的话语气泡:','ru':':пустое_облачко:','tr':':sol_konuşma_balonu:','ar':':فقاعة_كلام_يسرى:'},'\U0001F5E8':{'en':':left_speech_bubble:','status':unqualified,'E':2,'variant':True,'de':':sprechblase_links:','es':':bocadillo_de_diálogo_por_la_izquierda:','fr':':bulle_de_parole_gauche:','ja':':吹き出し左:','ko':':왼쪽_말풍선:','pt':':balão_de_diálogo_à_esquerda:','it':':nuvoletta_nera:','fa':':حباب_گفتار_چپ:','id':':gelembung_ucapan_kiri:','zh':':朝左的话语气泡:','ru':':пустое_облачко:','tr':':sol_konuşma_balonu:','ar':':فقاعة_كلام_يسرى:'},'\U0001FAF2':{'en':':leftwards_hand:','status':fully_qualified,'E':14,'de':':nach_links_weisende_hand:','es':':mano_hacia_la_izquierda:','fr':':main_vers_la_gauche:','ja':':左に向けた手:','ko':':왼쪽으로_향하는_손바닥:','pt':':mão_para_a_esquerda:','it':':mano_rivolta_a_sinistra:','fa':':دست_به_سمت_چپ:','id':':tangan_menghadap_kiri:','zh':':向左的手:','ru':':рука_влево:','tr':':sola_bakan_el:','ar':':يد_باتجاه_اليسار:'},'\U0001FAF2\U0001F3FF':{'en':':leftwards_hand_dark_skin_tone:','status':fully_qualified,'E':14,'de':':nach_links_weisende_hand_dunkle_hautfarbe:','es':':mano_hacia_la_izquierda_tono_de_piel_oscuro:','fr':':main_vers_la_gauche_peau_foncée:','ja':':左に向けた手_濃い肌色:','ko':':왼쪽으로_향하는_손바닥_검은색_피부:','pt':':mão_para_a_esquerda_pele_escura:','it':':mano_rivolta_a_sinistra_carnagione_scura:','fa':':دست_به_سمت_چپ_پوست_آبنوسی:','id':':tangan_menghadap_kiri_warna_kulit_gelap:','zh':':向左的手_较深肤色:','ru':':рука_влево_очень_темный_тон_кожи:','tr':':sola_bakan_el_koyu_cilt_tonu:','ar':':يد_باتجاه_اليسار_بشرة_بلون_غامق:'},'\U0001FAF2\U0001F3FB':{'en':':leftwards_hand_light_skin_tone:','status':fully_qualified,'E':14,'de':':nach_links_weisende_hand_helle_hautfarbe:','es':':mano_hacia_la_izquierda_tono_de_piel_claro:','fr':':main_vers_la_gauche_peau_claire:','ja':':左に向けた手_薄い肌色:','ko':':왼쪽으로_향하는_손바닥_하얀_피부:','pt':':mão_para_a_esquerda_pele_clara:','it':':mano_rivolta_a_sinistra_carnagione_chiara:','fa':':دست_به_سمت_چپ_پوست_سفید:','id':':tangan_menghadap_kiri_warna_kulit_cerah:','zh':':向左的手_较浅肤色:','ru':':рука_влево_очень_светлый_тон_кожи:','tr':':sola_bakan_el_açık_cilt_tonu:','ar':':يد_باتجاه_اليسار_بشرة_بلون_فاتح:'},'\U0001FAF2\U0001F3FE':{'en':':leftwards_hand_medium-dark_skin_tone:','status':fully_qualified,'E':14,'de':':nach_links_weisende_hand_mitteldunkle_hautfarbe:','es':':mano_hacia_la_izquierda_tono_de_piel_oscuro_medio:','fr':':main_vers_la_gauche_peau_mate:','ja':':左に向けた手_やや濃い肌色:','ko':':왼쪽으로_향하는_손바닥_진한_갈색_피부:','pt':':mão_para_a_esquerda_pele_morena_escura:','it':':mano_rivolta_a_sinistra_carnagione_abbastanza_scura:','fa':':دست_به_سمت_چپ_پوست_گندمی:','id':':tangan_menghadap_kiri_warna_kulit_gelap-sedang:','zh':':向左的手_中等深肤色:','ru':':рука_влево_темный_тон_кожи:','tr':':sola_bakan_el_orta_koyu_cilt_tonu:','ar':':يد_باتجاه_اليسار_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001FAF2\U0001F3FC':{'en':':leftwards_hand_medium-light_skin_tone:','status':fully_qualified,'E':14,'de':':nach_links_weisende_hand_mittelhelle_hautfarbe:','es':':mano_hacia_la_izquierda_tono_de_piel_claro_medio:','fr':':main_vers_la_gauche_peau_moyennement_claire:','ja':':左に向けた手_やや薄い肌色:','ko':':왼쪽으로_향하는_손바닥_연한_갈색_피부:','pt':':mão_para_a_esquerda_pele_morena_clara:','it':':mano_rivolta_a_sinistra_carnagione_abbastanza_chiara:','fa':':دست_به_سمت_چپ_پوست_روشن:','id':':tangan_menghadap_kiri_warna_kulit_cerah-sedang:','zh':':向左的手_中等浅肤色:','ru':':рука_влево_светлый_тон_кожи:','tr':':sola_bakan_el_orta_açık_cilt_tonu:','ar':':يد_باتجاه_اليسار_بشرة_بلون_فاتح_ومعتدل:'},'\U0001FAF2\U0001F3FD':{'en':':leftwards_hand_medium_skin_tone:','status':fully_qualified,'E':14,'de':':nach_links_weisende_hand_mittlere_hautfarbe:','es':':mano_hacia_la_izquierda_tono_de_piel_medio:','fr':':main_vers_la_gauche_peau_légèrement_mate:','ja':':左に向けた手_中間の肌色:','ko':':왼쪽으로_향하는_손바닥_갈색_피부:','pt':':mão_para_a_esquerda_pele_morena:','it':':mano_rivolta_a_sinistra_carnagione_olivastra:','fa':':دست_به_سمت_چپ_پوست_طلایی:','id':':tangan_menghadap_kiri_warna_kulit_sedang:','zh':':向左的手_中等肤色:','ru':':рука_влево_средний_тон_кожи:','tr':':sola_bakan_el_orta_cilt_tonu:','ar':':يد_باتجاه_اليسار_بشرة_بلون_معتدل:'},'\U0001FAF7':{'en':':leftwards_pushing_hand:','status':fully_qualified,'E':15,'de':':nach_links_schiebende_hand:','es':':mano_empujando_hacia_la_izquierda:','fr':':main_qui_pousse_vers_la_gauche:','ja':':左向きに押している手:','ko':':왼쪽으로_밀치는_손:','pt':':mão_empurrando_para_a_esquerda:','it':':mano_che_spinge_a_sinistra:','fa':':دست_فشاردهنده_به_سمت_چپ:','id':':tangan_mendorong_ke_kiri:','zh':':向左推:','ru':':рука_ладонью_влево:','tr':':sola_iten_el:','ar':':يد_تدفع_يسارا:'},'\U0001FAF7\U0001F3FF':{'en':':leftwards_pushing_hand_dark_skin_tone:','status':fully_qualified,'E':15,'de':':nach_links_schiebende_hand_dunkle_hautfarbe:','es':':mano_empujando_hacia_la_izquierda_tono_de_piel_oscuro:','fr':':main_qui_pousse_vers_la_gauche_peau_foncée:','ja':':左向きに押している手_濃い肌色:','ko':':왼쪽으로_밀치는_손_검은색_피부:','pt':':mão_empurrando_para_a_esquerda_pele_escura:','it':':mano_che_spinge_a_sinistra_carnagione_scura:','fa':':دست_فشاردهنده_به_سمت_چپ_پوست_آبنوسی:','id':':tangan_mendorong_ke_kiri_warna_kulit_gelap:','zh':':向左推_较深肤色:','ru':':рука_ладонью_влево_очень_темный_тон_кожи:','tr':':sola_iten_el_koyu_cilt_tonu:','ar':':يد_تدفع_يسارا_بشرة_بلون_غامق:'},'\U0001FAF7\U0001F3FB':{'en':':leftwards_pushing_hand_light_skin_tone:','status':fully_qualified,'E':15,'de':':nach_links_schiebende_hand_helle_hautfarbe:','es':':mano_empujando_hacia_la_izquierda_tono_de_piel_claro:','fr':':main_qui_pousse_vers_la_gauche_peau_claire:','ja':':左向きに押している手_薄い肌色:','ko':':왼쪽으로_밀치는_손_하얀_피부:','pt':':mão_empurrando_para_a_esquerda_pele_clara:','it':':mano_che_spinge_a_sinistra_carnagione_chiara:','fa':':دست_فشاردهنده_به_سمت_چپ_پوست_سفید:','id':':tangan_mendorong_ke_kiri_warna_kulit_cerah:','zh':':向左推_较浅肤色:','ru':':рука_ладонью_влево_очень_светлый_тон_кожи:','tr':':sola_iten_el_açık_cilt_tonu:','ar':':يد_تدفع_يسارا_بشرة_بلون_فاتح:'},'\U0001FAF7\U0001F3FE':{'en':':leftwards_pushing_hand_medium-dark_skin_tone:','status':fully_qualified,'E':15,'de':':nach_links_schiebende_hand_mitteldunkle_hautfarbe:','es':':mano_empujando_hacia_la_izquierda_tono_de_piel_oscuro_medio:','fr':':main_qui_pousse_vers_la_gauche_peau_mate:','ja':':左向きに押している手_やや濃い肌色:','ko':':왼쪽으로_밀치는_손_진한_갈색_피부:','pt':':mão_empurrando_para_a_esquerda_pele_morena_escura:','it':':mano_che_spinge_a_sinistra_carnagione_abbastanza_scura:','fa':':دست_فشاردهنده_به_سمت_چپ_پوست_گندمی:','id':':tangan_mendorong_ke_kiri_warna_kulit_gelap-sedang:','zh':':向左推_中等深肤色:','ru':':рука_ладонью_влево_темный_тон_кожи:','tr':':sola_iten_el_orta_koyu_cilt_tonu:','ar':':يد_تدفع_يسارا_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001FAF7\U0001F3FC':{'en':':leftwards_pushing_hand_medium-light_skin_tone:','status':fully_qualified,'E':15,'de':':nach_links_schiebende_hand_mittelhelle_hautfarbe:','es':':mano_empujando_hacia_la_izquierda_tono_de_piel_claro_medio:','fr':':main_qui_pousse_vers_la_gauche_peau_moyennement_claire:','ja':':左向きに押している手_やや薄い肌色:','ko':':왼쪽으로_밀치는_손_연한_갈색_피부:','pt':':mão_empurrando_para_a_esquerda_pele_morena_clara:','it':':mano_che_spinge_a_sinistra_carnagione_abbastanza_chiara:','fa':':دست_فشاردهنده_به_سمت_چپ_پوست_روشن:','id':':tangan_mendorong_ke_kiri_warna_kulit_cerah-sedang:','zh':':向左推_中等浅肤色:','ru':':рука_ладонью_влево_светлый_тон_кожи:','tr':':sola_iten_el_orta_açık_cilt_tonu:','ar':':يد_تدفع_يسارا_بشرة_بلون_فاتح_ومعتدل:'},'\U0001FAF7\U0001F3FD':{'en':':leftwards_pushing_hand_medium_skin_tone:','status':fully_qualified,'E':15,'de':':nach_links_schiebende_hand_mittlere_hautfarbe:','es':':mano_empujando_hacia_la_izquierda_tono_de_piel_medio:','fr':':main_qui_pousse_vers_la_gauche_peau_légèrement_mate:','ja':':左向きに押している手_中間の肌色:','ko':':왼쪽으로_밀치는_손_갈색_피부:','pt':':mão_empurrando_para_a_esquerda_pele_morena:','it':':mano_che_spinge_a_sinistra_carnagione_olivastra:','fa':':دست_فشاردهنده_به_سمت_چپ_پوست_طلایی:','id':':tangan_mendorong_ke_kiri_warna_kulit_sedang:','zh':':向左推_中等肤色:','ru':':рука_ладонью_влево_средний_тон_кожи:','tr':':sola_iten_el_orta_cilt_tonu:','ar':':يد_تدفع_يسارا_بشرة_بلون_معتدل:'},'\U0001F9B5':{'en':':leg:','status':fully_qualified,'E':11,'de':':bein:','es':':pierna:','fr':':jambe:','ja':':脚:','ko':':한쪽_다리:','pt':':perna:','it':':gamba:','fa':':لنگ:','id':':tungkai:','zh':':腿:','ru':':нога:','tr':':bacak:','ar':':ساق:'},'\U0001F9B5\U0001F3FF':{'en':':leg_dark_skin_tone:','status':fully_qualified,'E':11,'de':':bein_dunkle_hautfarbe:','es':':pierna_tono_de_piel_oscuro:','fr':':jambe_peau_foncée:','ja':':脚_濃い肌色:','ko':':한쪽_다리_검은색_피부:','pt':':perna_pele_escura:','it':':gamba_carnagione_scura:','fa':':لنگ_پوست_آبنوسی:','id':':tungkai_warna_kulit_gelap:','zh':':腿_较深肤色:','ru':':нога_очень_темный_тон_кожи:','tr':':bacak_koyu_cilt_tonu:','ar':':ساق_بشرة_بلون_غامق:'},'\U0001F9B5\U0001F3FB':{'en':':leg_light_skin_tone:','status':fully_qualified,'E':11,'de':':bein_helle_hautfarbe:','es':':pierna_tono_de_piel_claro:','fr':':jambe_peau_claire:','ja':':脚_薄い肌色:','ko':':한쪽_다리_하얀_피부:','pt':':perna_pele_clara:','it':':gamba_carnagione_chiara:','fa':':لنگ_پوست_سفید:','id':':tungkai_warna_kulit_cerah:','zh':':腿_较浅肤色:','ru':':нога_очень_светлый_тон_кожи:','tr':':bacak_açık_cilt_tonu:','ar':':ساق_بشرة_بلون_فاتح:'},'\U0001F9B5\U0001F3FE':{'en':':leg_medium-dark_skin_tone:','status':fully_qualified,'E':11,'de':':bein_mitteldunkle_hautfarbe:','es':':pierna_tono_de_piel_oscuro_medio:','fr':':jambe_peau_mate:','ja':':脚_やや濃い肌色:','ko':':한쪽_다리_진한_갈색_피부:','pt':':perna_pele_morena_escura:','it':':gamba_carnagione_abbastanza_scura:','fa':':لنگ_پوست_گندمی:','id':':tungkai_warna_kulit_gelap-sedang:','zh':':腿_中等深肤色:','ru':':нога_темный_тон_кожи:','tr':':bacak_orta_koyu_cilt_tonu:','ar':':ساق_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9B5\U0001F3FC':{'en':':leg_medium-light_skin_tone:','status':fully_qualified,'E':11,'de':':bein_mittelhelle_hautfarbe:','es':':pierna_tono_de_piel_claro_medio:','fr':':jambe_peau_moyennement_claire:','ja':':脚_やや薄い肌色:','ko':':한쪽_다리_연한_갈색_피부:','pt':':perna_pele_morena_clara:','it':':gamba_carnagione_abbastanza_chiara:','fa':':لنگ_پوست_روشن:','id':':tungkai_warna_kulit_cerah-sedang:','zh':':腿_中等浅肤色:','ru':':нога_светлый_тон_кожи:','tr':':bacak_orta_açık_cilt_tonu:','ar':':ساق_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9B5\U0001F3FD':{'en':':leg_medium_skin_tone:','status':fully_qualified,'E':11,'de':':bein_mittlere_hautfarbe:','es':':pierna_tono_de_piel_medio:','fr':':jambe_peau_légèrement_mate:','ja':':脚_中間の肌色:','ko':':한쪽_다리_갈색_피부:','pt':':perna_pele_morena:','it':':gamba_carnagione_olivastra:','fa':':لنگ_پوست_طلایی:','id':':tungkai_warna_kulit_sedang:','zh':':腿_中等肤色:','ru':':нога_средний_тон_кожи:','tr':':bacak_orta_cilt_tonu:','ar':':ساق_بشرة_بلون_معتدل:'},'\U0001F34B':{'en':':lemon:','status':fully_qualified,'E':1,'de':':zitrone:','es':':limón:','fr':':citron:','ja':':レモン:','ko':':레몬:','pt':':limão:','it':':limone:','fa':':لیمو_ترش:','id':':lemon:','zh':':柠檬:','ru':':лимон:','tr':':limon:','ar':':ليمون:'},'\U0001F406':{'en':':leopard:','status':fully_qualified,'E':1,'de':':leopard:','es':':leopardo:','fr':':léopard:','ja':':ヒョウ:','ko':':표범:','pt':':leopardo:','it':':leopardo:','fa':':پلنگ:','id':':macan_tutul:','zh':':豹子:','ru':':леопард:','tr':':leopar:','ar':':فهد:'},'\U0001F39A\U0000FE0F':{'en':':level_slider:','status':fully_qualified,'E':0.7,'variant':True,'de':':schieberegler:','es':':control_de_volumen:','fr':':curseur_de_niveau:','ja':':レベルスライダー:','ko':':레벨_슬라이더:','pt':':controle_de_volume:','it':':cursore_del_volume:','fa':':درجه_تنظیم_کننده:','id':':penggeser_tingkat:','zh':':电平滑块:','ru':':слаидер:','tr':':ayarlama_çubuğu:','ar':':شريط_منسوب:'},'\U0001F39A':{'en':':level_slider:','status':unqualified,'E':0.7,'variant':True,'de':':schieberegler:','es':':control_de_volumen:','fr':':curseur_de_niveau:','ja':':レベルスライダー:','ko':':레벨_슬라이더:','pt':':controle_de_volume:','it':':cursore_del_volume:','fa':':درجهٔ_تنظیم_کننده:','id':':penggeser_tingkat:','zh':':电平滑块:','ru':':слайдер:','tr':':ayarlama_çubuğu:','ar':':شريط_منسوب:'},'\U0001FA75':{'en':':light_blue_heart:','status':fully_qualified,'E':15,'de':':hellblaues_herz:','es':':corazón_azul_claro:','fr':':cœur_bleu_clair:','ja':':薄い青のハート:','ko':':밝은_파란색_하트:','pt':':coração_azul-claro:','it':':cuore_azzurro:','fa':':قلب_آبی_کم_رنگ:','id':':hati_biru_muda:','zh':':浅蓝色的心:','ru':':голубое_сердце:','tr':':açık_mavi_kalp:','ar':':قلب_أزرق_فاتح:'},'\U0001F4A1':{'en':':light_bulb:','status':fully_qualified,'E':0.6,'alias':[':bulb:'],'de':':glühbirne:','es':':bombilla:','fr':':ampoule:','ja':':電球:','ko':':전구:','pt':':lâmpada:','it':':lampadina:','fa':':لامپ:','id':':bohlam:','zh':':灯泡:','ru':':лампочка:','tr':':ampul:','ar':':مصباح_مضاء:'},'\U0001F688':{'en':':light_rail:','status':fully_qualified,'E':1,'de':':s-bahn:','es':':tren_ligero:','fr':':métro_léger:','ja':':ライトレール:','ko':':경전철:','pt':':trem_urbano:','it':':metrotranvia:','fa':':قطار_سبک_شهری:','id':':light_rail:','zh':':轻轨:','ru':':легкий_рельсовый_транспорт:','tr':':hafif_raylı:','ar':':نقل_خفيف:'},'\U0001F3FB':{'en':':light_skin_tone:','status':component,'E':1,'alias':[':emoji_modifier_fitzpatrick_type_1_2:'],'de':':helle_hautfarbe:','es':':tono_de_piel_claro:','fr':':peau_claire:','ja':':薄い肌色:','ko':':하얀_피부:','pt':':pele_clara:','it':':carnagione_chiara:','fa':':پوست_سفید:','id':':warna_kulit_cerah:','zh':':较浅肤色:','ru':':очень_светлый_тон_кожи:','tr':':açık_cilt_tonu:','ar':':بشرة_بلون_فاتح:'},'\U0001F34B\U0000200D\U0001F7E9':{'en':':lime:','status':fully_qualified,'E':15.1,'de':':limette:','es':':lima:','fr':':citron_vert:','ja':':ライム:','ko':':라임:','pt':':lima:','it':':lime:','fa':':لیمو:','id':':jeruk_limau:','zh':':青柠:','ru':':лайм:','tr':':misket_limonu:','ar':':الليم:'},'\U0001F517':{'en':':link:','status':fully_qualified,'E':0.6,'de':':linksymbol:','es':':eslabón:','fr':':chaînons:','ja':':リンクシンボル:','ko':':링크:','pt':':link:','it':':collegamento:','fa':':حلقهٔ_زنجیر:','id':':tautan:','zh':':链接:','ru':':гиперссылка:','tr':':bağlantı_işareti:','ar':':علامة_رابط:'},'\U0001F587\U0000FE0F':{'en':':linked_paperclips:','status':fully_qualified,'E':0.7,'alias':[':paperclips:'],'variant':True,'de':':verhakte_büroklammern:','es':':clips_unidos:','fr':':trombones:','ja':':つながったクリップ:','ko':':이어져_있는_클립:','pt':':clipes_de_papel_conectados:','it':':graffette_attaccate:','fa':':زنجیره_گیره:','id':':klip_kertas_tertaut:','zh':':连起来的两个回形针:','ru':':скрепки:','tr':':iç_içe_geçmiş_ataşlar:','ar':':مشبك_ورق_مقترن:'},'\U0001F587':{'en':':linked_paperclips:','status':unqualified,'E':0.7,'alias':[':paperclips:'],'variant':True,'de':':verhakte_büroklammern:','es':':clips_unidos:','fr':':trombones:','ja':':つながったクリップ:','ko':':이어져_있는_클립:','pt':':clipes_de_papel_conectados:','it':':graffette_attaccate:','fa':':زنجیرهٔ_گیره:','id':':klip_kertas_tertaut:','zh':':连起来的两个回形针:','ru':':скрепки:','tr':':iç_içe_geçmiş_ataşlar:','ar':':مشبك_ورق_مقترن:'},'\U0001F981':{'en':':lion:','status':fully_qualified,'E':1,'alias':[':lion_face:'],'de':':löwe:','es':':león:','fr':':tête_de_lion:','ja':':ライオンの顔:','ko':':사자_얼굴:','pt':':rosto_de_leão:','it':':leone:','fa':':شیر:','id':':singa:','zh':':狮子:','ru':':морда_льва:','tr':':aslan:','ar':':وجه_أسد:'},'\U0001F484':{'en':':lipstick:','status':fully_qualified,'E':0.6,'de':':lippenstift:','es':':pintalabios:','fr':':rouge_à_lèvres:','ja':':口紅:','ko':':립스틱:','pt':':batom:','it':':rossetto:','fa':':رژلب:','id':':lipstik:','zh':':唇膏:','ru':':помада:','tr':':ruj:','ar':':أحمر_شفاه:'},'\U0001F6AE':{'en':':litter_in_bin_sign:','status':fully_qualified,'E':1,'alias':[':put_litter_in_its_place:'],'de':':symbol_papierkorb:','es':':señal_de_usar_papelera:','fr':':icône_poubelle:','ja':':ゴミ箱:','ko':':쓰레기는_쓰레기통에:','pt':':símbolo_de_lixeira:','it':':simbolo_per_la_raccolta_dei_rifiuti:','fa':':علامت_ریختن_زباله_در_زباله_دان:','id':':tanda_membuang_sampah:','zh':':倒垃圾:','ru':':бросайте_мусор_в_урну:','tr':':çöp_kutusu_işareti:','ar':':إشارة_صندوق_النفايات:'},'\U0001F98E':{'en':':lizard:','status':fully_qualified,'E':3,'de':':eidechse:','es':':lagarto:','fr':':lézard:','ja':':トカゲ:','ko':':도마뱀:','pt':':lagartixa:','it':':lucertola:','fa':':مارمولک:','id':':kadal:','zh':':蜥蜴:','ru':':ящерица:','tr':':kertenkele:','ar':':سحلية:'},'\U0001F999':{'en':':llama:','status':fully_qualified,'E':11,'de':':lama:','es':':llama:','fr':':lama:','ja':':ラマ:','ko':':라마:','pt':':lhama:','it':':lama:','fa':':لاما:','id':':llama:','zh':':美洲鸵:','ru':':лама:','tr':':lama:','ar':':لاما:'},'\U0001F99E':{'en':':lobster:','status':fully_qualified,'E':11,'de':':hummer:','es':':bogavante:','fr':':homard:','ja':':ザリガニ:','ko':':바닷가재:','pt':':lagosta:','it':':aragosta:','fa':':لابستر:','id':':lobster:','zh':':龙虾:','ru':':омар:','tr':':ıstakoz:','ar':':كركند:'},'\U0001F512':{'en':':locked:','status':fully_qualified,'E':0.6,'alias':[':lock:'],'variant':True,'de':':geschlossenes_schloss:','es':':candado_cerrado:','fr':':cadenas_fermé:','ja':':閉じた錠:','ko':':자물쇠:','pt':':cadeado:','it':':lucchetto_chiuso:','fa':':قفل:','id':':gembok:','zh':':合上的锁:','ru':':закрытый_замок:','tr':':kilit:','ar':':قفل:'},'\U0001F510':{'en':':locked_with_key:','status':fully_qualified,'E':0.6,'alias':[':closed_lock_with_key:'],'de':':schloss_mit_schlüssel:','es':':candado_cerrado_y_llave:','fr':':cadenas_fermé_avec_clé:','ja':':閉じた錠と鍵:','ko':':자물쇠와_열쇠:','pt':':cadeado_fechado_com_chave:','it':':lucchetto_chiuso_con_chiave:','fa':':قفل_و_کلید:','id':':gembok_terkunci_dengan_kunci:','zh':':钥匙和锁:','ru':':закрытый_замок_с_ключом:','tr':':anahtarlı_kapalı_kilit:','ar':':قفل_مغلق_بمفتاح:'},'\U0001F50F':{'en':':locked_with_pen:','status':fully_qualified,'E':0.6,'alias':[':lock_with_ink_pen:'],'de':':schloss_mit_füller:','es':':candado_con_pluma_estilográfica:','fr':':cadenas_fermé_avec_stylo:','ja':':閉じた錠とペン:','ko':':자물쇠와_펜:','pt':':cadeado_com_caneta:','it':':lucchetto_con_penna_stilo:','fa':':قفل_با_قلم:','id':':gembok_terkunci_dengan_pena:','zh':':墨水笔和锁:','ru':':закрытый_замок_с_перьевой_ручкой:','tr':':kilit_ve_kalem:','ar':':قفل_مغلق_بقلم:'},'\U0001F682':{'en':':locomotive:','status':fully_qualified,'E':1,'alias':[':steam_locomotive:'],'de':':dampflokomotive:','es':':locomotora_de_vapor:','fr':':locomotive:','ja':':蒸気機関車:','ko':':기관차:','pt':':locomotiva:','it':':locomotiva:','fa':':لوکوموتیو:','id':':lokomotif:','zh':':蒸汽火车:','ru':':паровоз:','tr':':buharlı_lokomotif:','ar':':جرار_بخار:'},'\U0001F36D':{'en':':lollipop:','status':fully_qualified,'E':0.6,'de':':lutscher:','es':':piruleta:','fr':':sucette:','ja':':ぺろぺろキャンディ:','ko':':롤리팝:','pt':':pirulito:','it':':lecca_lecca:','fa':':آب_نبات_چوبی:','id':':permen_lolipop:','zh':':棒棒糖:','ru':':леденец:','tr':':lolipop:','ar':':مصاصة:'},'\U0001FA98':{'en':':long_drum:','status':fully_qualified,'E':13,'de':':afrikanische_trommel:','es':':tamboril:','fr':':djembé:','ja':':コンガ:','ko':':긴_북:','pt':':tambor_comprido:','it':':tamburo_lungo:','fa':':طبل_بلند:','id':':gendang:','zh':':长鼓:','ru':':конга:','tr':':uzun_davul:','ar':':طبل_طويل:'},'\U0001F9F4':{'en':':lotion_bottle:','status':fully_qualified,'E':11,'de':':creme:','es':':bote_de_crema:','fr':':bouteille_de_lotion:','ja':':ローション:','ko':':로션:','pt':':frasco_de_loção:','it':':flacone:','fa':':بطری_لوسیون:','id':':botol_losion:','zh':':乳液瓶:','ru':':бутылка_лосьона:','tr':':losyon_şişesi:','ar':':زجاجة_غسول:'},'\U0001FAB7':{'en':':lotus:','status':fully_qualified,'E':14,'de':':lotusblüte:','es':':loto:','fr':':lotus:','ja':':ハスの花:','ko':':연꽃:','pt':':lótus:','it':':loto:','fa':':نیلوفر:','id':':teratai:','zh':':莲花:','ru':':лотос:','tr':':nilüfer:','ar':':لوتس:'},'\U0001F62D':{'en':':loudly_crying_face:','status':fully_qualified,'E':0.6,'alias':[':sob:'],'de':':heulendes_gesicht:','es':':cara_llorando_fuerte:','fr':':visage_qui_pleure_à_chaudes_larmes:','ja':':大泣き:','ko':':엉엉_우는_얼굴:','pt':':rosto_chorando_aos_berros:','it':':faccina_disperata:','fa':':هق_هق_گریه:','id':':wajah_menangis_keras:','zh':':放声大哭:','ru':':слезы_рекой:','tr':':avaz_avaz_ağlayan_yüz:','ar':':وجه_يبكي_بكاء_مرتفعا:'},'\U0001F4E2':{'en':':loudspeaker:','status':fully_qualified,'E':0.6,'de':':lautsprecher:','es':':altavoz_de_mano:','fr':':haut-parleur:','ja':':拡声器:','ko':':확성기:','pt':':buzina:','it':':altoparlante:','fa':':بلندگو:','id':':pengeras_suara:','zh':':喇叭:','ru':':громкоговоритель:','tr':':hoparlör:','ar':':مكبر_صوت:'},'\U0001F91F':{'en':':love-you_gesture:','status':fully_qualified,'E':5,'alias':[':love_you_gesture:'],'de':':ich-liebe-dich-geste:','es':':gesto_de_te_quiero:','fr':':signe_je_t’aime:','ja':':アイラブユー:','ko':':사랑을_표시하는_손:','pt':':gesto_de_te_amo:','it':':gesto_ti_amo:','fa':':علامت_دوستت_دارم:','id':':gerakan_sayang_kamu:','zh':':爱你的手势:','ru':':я_тебя_люблю:','tr':':seni_seviyorum_işareti:','ar':':إشارة_أحبك:'},'\U0001F91F\U0001F3FF':{'en':':love-you_gesture_dark_skin_tone:','status':fully_qualified,'E':5,'de':':ich-liebe-dich-geste_dunkle_hautfarbe:','es':':gesto_de_te_quiero_tono_de_piel_oscuro:','fr':':signe_je_t’aime_peau_foncée:','ja':':アイラブユー_濃い肌色:','ko':':사랑을_표시하는_손_검은색_피부:','pt':':gesto_de_te_amo_pele_escura:','it':':gesto_ti_amo_carnagione_scura:','fa':':علامت_دوستت_دارم_پوست_آبنوسی:','id':':gerakan_sayang_kamu_warna_kulit_gelap:','zh':':爱你的手势_较深肤色:','ru':':я_тебя_люблю_очень_темный_тон_кожи:','tr':':seni_seviyorum_işareti_koyu_cilt_tonu:','ar':':إشارة_أحبك_بشرة_بلون_غامق:'},'\U0001F91F\U0001F3FB':{'en':':love-you_gesture_light_skin_tone:','status':fully_qualified,'E':5,'de':':ich-liebe-dich-geste_helle_hautfarbe:','es':':gesto_de_te_quiero_tono_de_piel_claro:','fr':':signe_je_t’aime_peau_claire:','ja':':アイラブユー_薄い肌色:','ko':':사랑을_표시하는_손_하얀_피부:','pt':':gesto_de_te_amo_pele_clara:','it':':gesto_ti_amo_carnagione_chiara:','fa':':علامت_دوستت_دارم_پوست_سفید:','id':':gerakan_sayang_kamu_warna_kulit_cerah:','zh':':爱你的手势_较浅肤色:','ru':':я_тебя_люблю_очень_светлый_тон_кожи:','tr':':seni_seviyorum_işareti_açık_cilt_tonu:','ar':':إشارة_أحبك_بشرة_بلون_فاتح:'},'\U0001F91F\U0001F3FE':{'en':':love-you_gesture_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':ich-liebe-dich-geste_mitteldunkle_hautfarbe:','es':':gesto_de_te_quiero_tono_de_piel_oscuro_medio:','fr':':signe_je_t’aime_peau_mate:','ja':':アイラブユー_やや濃い肌色:','ko':':사랑을_표시하는_손_진한_갈색_피부:','pt':':gesto_de_te_amo_pele_morena_escura:','it':':gesto_ti_amo_carnagione_abbastanza_scura:','fa':':علامت_دوستت_دارم_پوست_گندمی:','id':':gerakan_sayang_kamu_warna_kulit_gelap-sedang:','zh':':爱你的手势_中等深肤色:','ru':':я_тебя_люблю_темный_тон_кожи:','tr':':seni_seviyorum_işareti_orta_koyu_cilt_tonu:','ar':':إشارة_أحبك_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F91F\U0001F3FC':{'en':':love-you_gesture_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':ich-liebe-dich-geste_mittelhelle_hautfarbe:','es':':gesto_de_te_quiero_tono_de_piel_claro_medio:','fr':':signe_je_t’aime_peau_moyennement_claire:','ja':':アイラブユー_やや薄い肌色:','ko':':사랑을_표시하는_손_연한_갈색_피부:','pt':':gesto_de_te_amo_pele_morena_clara:','it':':gesto_ti_amo_carnagione_abbastanza_chiara:','fa':':علامت_دوستت_دارم_پوست_روشن:','id':':gerakan_sayang_kamu_warna_kulit_cerah-sedang:','zh':':爱你的手势_中等浅肤色:','ru':':я_тебя_люблю_светлый_тон_кожи:','tr':':seni_seviyorum_işareti_orta_açık_cilt_tonu:','ar':':إشارة_أحبك_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F91F\U0001F3FD':{'en':':love-you_gesture_medium_skin_tone:','status':fully_qualified,'E':5,'de':':ich-liebe-dich-geste_mittlere_hautfarbe:','es':':gesto_de_te_quiero_tono_de_piel_medio:','fr':':signe_je_t’aime_peau_légèrement_mate:','ja':':アイラブユー_中間の肌色:','ko':':사랑을_표시하는_손_갈색_피부:','pt':':gesto_de_te_amo_pele_morena:','it':':gesto_ti_amo_carnagione_olivastra:','fa':':علامت_دوستت_دارم_پوست_طلایی:','id':':gerakan_sayang_kamu_warna_kulit_sedang:','zh':':爱你的手势_中等肤色:','ru':':я_тебя_люблю_средний_тон_кожи:','tr':':seni_seviyorum_işareti_orta_cilt_tonu:','ar':':إشارة_أحبك_بشرة_بلون_معتدل:'},'\U0001F3E9':{'en':':love_hotel:','status':fully_qualified,'E':0.6,'de':':stundenhotel:','es':':hotel_del_amor:','fr':':love_hotel:','ja':':ラブホテル:','ko':':모텔:','pt':':motel:','it':':love_hotel:','fa':':هتل_عشق_ورزی:','id':':hotel_cinta:','zh':':情人酒店:','ru':':гостиница_для_свиданий:','tr':':aşk_oteli:','ar':':فندق_حب:'},'\U0001F48C':{'en':':love_letter:','status':fully_qualified,'E':0.6,'de':':liebesbrief:','es':':carta_de_amor:','fr':':lettre_d’amour:','ja':':ラブレター:','ko':':러브레터:','pt':':carta_de_amor:','it':':lettera_d’amore:','fa':':نامهٔ_عاشقانه:','id':':surat_cinta:','zh':':情书:','ru':':любовное_письмо:','tr':':aşk_mektubu:','ar':':خطاب_حب:'},'\U0001FAAB':{'en':':low_battery:','status':fully_qualified,'E':14,'de':':schwache_batterie:','es':':batería_baja:','fr':':pile_faible:','ja':':バッテリー残量低下:','ko':':배터리_부족:','pt':':pouca_bateria:','it':':batteria_scarica:','fa':':باتری_ضعیف:','id':':baterai_lemah:','zh':':电池电量不足:','ru':':низкий_уровень_заряда:','tr':':düşük_pil:','ar':':بطارية_ضعيفة:'},'\U0001F9F3':{'en':':luggage:','status':fully_qualified,'E':11,'de':':gepäck:','es':':equipaje:','fr':':bagage:','ja':':スーツケース:','ko':':여행용_가방:','pt':':mala:','it':':valigia:','fa':':چمدان:','id':':koper:','zh':':行李箱:','ru':':багаж:','tr':':bavul:','ar':':حقائب_السفر:'},'\U0001FAC1':{'en':':lungs:','status':fully_qualified,'E':13,'de':':lunge:','es':':pulmones:','fr':':poumons:','ja':':肺:','ko':':폐:','pt':':pulmões:','it':':polmoni:','fa':':ریه:','id':':paru-paru:','zh':':肺:','ru':':легкие:','tr':':akciğer:','ar':':رئتان:'},'\U0001F925':{'en':':lying_face:','status':fully_qualified,'E':3,'de':':lügendes_gesicht:','es':':cara_de_mentiroso:','fr':':visage_de_menteur:','ja':':うそつきの顔:','ko':':거짓말하는_얼굴:','pt':':rosto_de_mentiroso:','it':':faccina_bugiarda:','fa':':صورت_دروغگو:','id':':bohong:','zh':':说谎:','ru':':лжец:','tr':':yalan_söyleyen_yüz:','ar':':وجه_كاذب:'},'\U0001F9D9':{'en':':mage:','status':fully_qualified,'E':5,'de':':magier(in):','es':':persona_maga:','fr':':mage:','ja':':魔法使い:','ko':':마법사:','pt':':mago:','it':':mago:','fa':':ساحر:','id':':magi:','zh':':法师:','ru':':маг:','tr':':büyücü:','ar':':سحرة:'},'\U0001F9D9\U0001F3FF':{'en':':mage_dark_skin_tone:','status':fully_qualified,'E':5,'de':':magier(in)_dunkle_hautfarbe:','es':':persona_maga_tono_de_piel_oscuro:','fr':':mage_peau_foncée:','ja':':魔法使い_濃い肌色:','ko':':마법사_검은색_피부:','pt':':mago_pele_escura:','it':':mago_carnagione_scura:','fa':':ساحر_پوست_آبنوسی:','id':':magi_warna_kulit_gelap:','zh':':法师_较深肤色:','ru':':маг_очень_темный_тон_кожи:','tr':':büyücü_koyu_cilt_tonu:','ar':':سحرة_بشرة_بلون_غامق:'},'\U0001F9D9\U0001F3FB':{'en':':mage_light_skin_tone:','status':fully_qualified,'E':5,'de':':magier(in)_helle_hautfarbe:','es':':persona_maga_tono_de_piel_claro:','fr':':mage_peau_claire:','ja':':魔法使い_薄い肌色:','ko':':마법사_하얀_피부:','pt':':mago_pele_clara:','it':':mago_carnagione_chiara:','fa':':ساحر_پوست_سفید:','id':':magi_warna_kulit_cerah:','zh':':法师_较浅肤色:','ru':':маг_очень_светлый_тон_кожи:','tr':':büyücü_açık_cilt_tonu:','ar':':سحرة_بشرة_بلون_فاتح:'},'\U0001F9D9\U0001F3FE':{'en':':mage_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':magier(in)_mitteldunkle_hautfarbe:','es':':persona_maga_tono_de_piel_oscuro_medio:','fr':':mage_peau_mate:','ja':':魔法使い_やや濃い肌色:','ko':':마법사_진한_갈색_피부:','pt':':mago_pele_morena_escura:','it':':mago_carnagione_abbastanza_scura:','fa':':ساحر_پوست_گندمی:','id':':magi_warna_kulit_gelap-sedang:','zh':':法师_中等深肤色:','ru':':маг_темный_тон_кожи:','tr':':büyücü_orta_koyu_cilt_tonu:','ar':':سحرة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D9\U0001F3FC':{'en':':mage_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':magier(in)_mittelhelle_hautfarbe:','es':':persona_maga_tono_de_piel_claro_medio:','fr':':mage_peau_moyennement_claire:','ja':':魔法使い_やや薄い肌色:','ko':':마법사_연한_갈색_피부:','pt':':mago_pele_morena_clara:','it':':mago_carnagione_abbastanza_chiara:','fa':':ساحر_پوست_روشن:','id':':magi_warna_kulit_cerah-sedang:','zh':':法师_中等浅肤色:','ru':':маг_светлый_тон_кожи:','tr':':büyücü_orta_açık_cilt_tonu:','ar':':سحرة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D9\U0001F3FD':{'en':':mage_medium_skin_tone:','status':fully_qualified,'E':5,'de':':magier(in)_mittlere_hautfarbe:','es':':persona_maga_tono_de_piel_medio:','fr':':mage_peau_légèrement_mate:','ja':':魔法使い_中間の肌色:','ko':':마법사_갈색_피부:','pt':':mago_pele_morena:','it':':mago_carnagione_olivastra:','fa':':ساحر_پوست_طلایی:','id':':magi_warna_kulit_sedang:','zh':':法师_中等肤色:','ru':':маг_средний_тон_кожи:','tr':':büyücü_orta_cilt_tonu:','ar':':سحرة_بشرة_بلون_معتدل:'},'\U0001FA84':{'en':':magic_wand:','status':fully_qualified,'E':13,'de':':zauberstab:','es':':varita_mágica:','fr':':baguette_magique:','ja':':魔法の杖:','ko':':마술_지팡이:','pt':':varinha_mágica:','it':':bacchetta_magica:','fa':':عصای_سحرآمیز:','id':':tongkat_ajaib:','zh':':魔棒:','ru':':волшебная_палочка:','tr':':sihirli_asa:','ar':':عصا_سحرية:'},'\U0001F9F2':{'en':':magnet:','status':fully_qualified,'E':11,'de':':magnet:','es':':imán:','fr':':aimant:','ja':':u字型磁石:','ko':':자석:','pt':':ímã:','it':':calamita:','fa':':آهن_ربا:','id':':magnet:','zh':':磁铁:','ru':':магнит:','tr':':mıknatıs:','ar':':مغناطيس:'},'\U0001F50D':{'en':':magnifying_glass_tilted_left:','status':fully_qualified,'E':0.6,'alias':[':mag:'],'variant':True,'de':':lupe_nach_links:','es':':lupa_orientada_hacia_la_izquierda:','fr':':loupe_orientée_à_gauche:','ja':':虫眼鏡左:','ko':':왼쪽으로_향한_돋보기:','pt':':lupa_para_a_esquerda:','it':':lente_di_ingrandimento_rivolta_a_sinistra:','fa':':ذره_بین_روبه_چپ:','id':':kaca_pembesar_arah_kiri:','zh':':左斜的放大镜:','ru':':лупа_наклоненная_влево:','tr':':sola_dönük_büyüteç:','ar':':عدسة_مكبرة_اتجاه_اليسار:'},'\U0001F50E':{'en':':magnifying_glass_tilted_right:','status':fully_qualified,'E':0.6,'alias':[':mag_right:'],'de':':lupe_nach_rechts:','es':':lupa_orientada_hacia_la_derecha:','fr':':loupe_orientée_à_droite:','ja':':虫眼鏡右:','ko':':오른쪽으로_향한_돋보기:','pt':':lupa_para_a_direita:','it':':lente_di_ingrandimento_rivolta_a_destra:','fa':':ذره_بین_روبه_راست:','id':':kaca_pembesar_arah_kanan:','zh':':右斜的放大镜:','ru':':лупа_наклоненная_вправо:','tr':':sağa_dönük_büyüteç:','ar':':عدسة_مكبرة_اتجاه_اليمين:'},'\U0001F004':{'en':':mahjong_red_dragon:','status':fully_qualified,'E':0.6,'alias':[':mahjong:'],'variant':True,'de':':mahjong-stein:','es':':dragón_rojo_de_mahjong:','fr':':dragon_rouge_mahjong:','ja':':麻雀:','ko':':마작:','pt':':dragão_vermelho_de_mahjong:','it':':tessera_mahjong:','fa':':ماژونگ_اژدهای_قرمز:','id':':naga_merah_mahyong:','zh':':红中:','ru':':маджонг:','tr':':kızıl_ejder_mahjong_taşı:','ar':':لعبة_ورق_يابانية:'},'\U00002642\U0000FE0F':{'en':':male_sign:','status':fully_qualified,'E':4,'variant':True,'de':':männersymbol:','es':':signo_masculino:','fr':':symbole_de_l’homme:','ja':':男性のマーク:','ko':':남성_기호:','pt':':símbolo_de_masculino:','it':':simbolo_genere_maschile:','fa':':نشان_مریخ:','id':':simbol_pria:','zh':':男性符号:','ru':':мужскои_символ:','tr':':erkek_sembolü:','ar':':رمز_ذكر:'},'\U00002642':{'en':':male_sign:','status':unqualified,'E':4,'variant':True,'de':':männersymbol:','es':':signo_masculino:','fr':':symbole_de_l’homme:','ja':':男性のマーク:','ko':':남성_기호:','pt':':símbolo_de_masculino:','it':':simbolo_genere_maschile:','fa':':نشان_مریخ:','id':':simbol_pria:','zh':':男性符号:','ru':':мужской_символ:','tr':':erkek_sembolü:','ar':':رمز_ذكر:'},'\U0001F9A3':{'en':':mammoth:','status':fully_qualified,'E':13,'de':':mammut:','es':':mamut:','fr':':mammouth:','ja':':マンモス:','ko':':매머드:','pt':':mamute:','it':':mammut:','fa':':ماموت:','id':':mamut:','zh':':猛犸:','ru':':мамонт:','tr':':mamut:','ar':':ماموث:'},'\U0001F468':{'en':':man:','status':fully_qualified,'E':0.6,'de':':mann:','es':':hombre:','fr':':homme:','ja':':男性:','ko':':남자:','pt':':homem:','it':':uomo:','fa':':مرد:','id':':pria:','zh':':男人:','ru':':мужчина:','tr':':erkek:','ar':':رجل:'},'\U0001F468\U0000200D\U0001F3A8':{'en':':man_artist:','status':fully_qualified,'E':4,'de':':künstler:','es':':artista_hombre:','fr':':artiste_homme:','ja':':男性の芸術家:','ko':':남자_화가:','pt':':artista_plástico:','it':':artista_uomo:','fa':':هنرمند_مرد:','id':':seniman_pria:','zh':':男艺术家:','ru':':мужчина_художник:','tr':':erkek_ressam:','ar':':رسام:'},'\U0001F468\U0001F3FF\U0000200D\U0001F3A8':{'en':':man_artist_dark_skin_tone:','status':fully_qualified,'E':4,'de':':künstler_dunkle_hautfarbe:','es':':artista_hombre_tono_de_piel_oscuro:','fr':':artiste_homme_peau_foncée:','ja':':男性の芸術家_濃い肌色:','ko':':남자_화가_검은색_피부:','pt':':artista_plástico_pele_escura:','it':':artista_uomo_carnagione_scura:','fa':':هنرمند_مرد_پوست_آبنوسی:','id':':seniman_pria_warna_kulit_gelap:','zh':':男艺术家_较深肤色:','ru':':мужчина_художник_очень_темный_тон_кожи:','tr':':erkek_ressam_koyu_cilt_tonu:','ar':':رسام_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U0001F3A8':{'en':':man_artist_light_skin_tone:','status':fully_qualified,'E':4,'de':':künstler_helle_hautfarbe:','es':':artista_hombre_tono_de_piel_claro:','fr':':artiste_homme_peau_claire:','ja':':男性の芸術家_薄い肌色:','ko':':남자_화가_하얀_피부:','pt':':artista_plástico_pele_clara:','it':':artista_uomo_carnagione_chiara:','fa':':هنرمند_مرد_پوست_سفید:','id':':seniman_pria_warna_kulit_cerah:','zh':':男艺术家_较浅肤色:','ru':':мужчина_художник_очень_светлый_тон_кожи:','tr':':erkek_ressam_açık_cilt_tonu:','ar':':رسام_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F3A8':{'en':':man_artist_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':künstler_mitteldunkle_hautfarbe:','es':':artista_hombre_tono_de_piel_oscuro_medio:','fr':':artiste_homme_peau_mate:','ja':':男性の芸術家_やや濃い肌色:','ko':':남자_화가_진한_갈색_피부:','pt':':artista_plástico_pele_morena_escura:','it':':artista_uomo_carnagione_abbastanza_scura:','fa':':هنرمند_مرد_پوست_گندمی:','id':':seniman_pria_warna_kulit_gelap-sedang:','zh':':男艺术家_中等深肤色:','ru':':мужчина_художник_темный_тон_кожи:','tr':':erkek_ressam_orta_koyu_cilt_tonu:','ar':':رسام_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U0001F3A8':{'en':':man_artist_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':künstler_mittelhelle_hautfarbe:','es':':artista_hombre_tono_de_piel_claro_medio:','fr':':artiste_homme_peau_moyennement_claire:','ja':':男性の芸術家_やや薄い肌色:','ko':':남자_화가_연한_갈색_피부:','pt':':artista_plástico_pele_morena_clara:','it':':artista_uomo_carnagione_abbastanza_chiara:','fa':':هنرمند_مرد_پوست_روشن:','id':':seniman_pria_warna_kulit_cerah-sedang:','zh':':男艺术家_中等浅肤色:','ru':':мужчина_художник_светлый_тон_кожи:','tr':':erkek_ressam_orta_açık_cilt_tonu:','ar':':رسام_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F3A8':{'en':':man_artist_medium_skin_tone:','status':fully_qualified,'E':4,'de':':künstler_mittlere_hautfarbe:','es':':artista_hombre_tono_de_piel_medio:','fr':':artiste_homme_peau_légèrement_mate:','ja':':男性の芸術家_中間の肌色:','ko':':남자_화가_갈색_피부:','pt':':artista_plástico_pele_morena:','it':':artista_uomo_carnagione_olivastra:','fa':':هنرمند_مرد_پوست_طلایی:','id':':seniman_pria_warna_kulit_sedang:','zh':':男艺术家_中等肤色:','ru':':мужчина_художник_средний_тон_кожи:','tr':':erkek_ressam_orta_cilt_tonu:','ar':':رسام_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U0001F680':{'en':':man_astronaut:','status':fully_qualified,'E':4,'de':':astronaut:','es':':astronauta_hombre:','fr':':astronaute_homme:','ja':':男性宇宙飛行士:','ko':':남자_우주비행사:','pt':':astronauta_homem:','it':':astronauta_uomo:','fa':':فضانورد_مرد:','id':':antariksawan_pria:','zh':':男宇航员:','ru':':мужчина_космонавт:','tr':':erkek_astronot:','ar':':رائد_فضاء:'},'\U0001F468\U0001F3FF\U0000200D\U0001F680':{'en':':man_astronaut_dark_skin_tone:','status':fully_qualified,'E':4,'de':':astronaut_dunkle_hautfarbe:','es':':astronauta_hombre_tono_de_piel_oscuro:','fr':':astronaute_homme_peau_foncée:','ja':':男性宇宙飛行士_濃い肌色:','ko':':남자_우주비행사_검은색_피부:','pt':':astronauta_homem_pele_escura:','it':':astronauta_uomo_carnagione_scura:','fa':':فضانورد_مرد_پوست_آبنوسی:','id':':antariksawan_pria_warna_kulit_gelap:','zh':':男宇航员_较深肤色:','ru':':мужчина_космонавт_очень_темный_тон_кожи:','tr':':erkek_astronot_koyu_cilt_tonu:','ar':':رائد_فضاء_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U0001F680':{'en':':man_astronaut_light_skin_tone:','status':fully_qualified,'E':4,'de':':astronaut_helle_hautfarbe:','es':':astronauta_hombre_tono_de_piel_claro:','fr':':astronaute_homme_peau_claire:','ja':':男性宇宙飛行士_薄い肌色:','ko':':남자_우주비행사_하얀_피부:','pt':':astronauta_homem_pele_clara:','it':':astronauta_uomo_carnagione_chiara:','fa':':فضانورد_مرد_پوست_سفید:','id':':antariksawan_pria_warna_kulit_cerah:','zh':':男宇航员_较浅肤色:','ru':':мужчина_космонавт_очень_светлый_тон_кожи:','tr':':erkek_astronot_açık_cilt_tonu:','ar':':رائد_فضاء_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F680':{'en':':man_astronaut_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':astronaut_mitteldunkle_hautfarbe:','es':':astronauta_hombre_tono_de_piel_oscuro_medio:','fr':':astronaute_homme_peau_mate:','ja':':男性宇宙飛行士_やや濃い肌色:','ko':':남자_우주비행사_진한_갈색_피부:','pt':':astronauta_homem_pele_morena_escura:','it':':astronauta_uomo_carnagione_abbastanza_scura:','fa':':فضانورد_مرد_پوست_گندمی:','id':':antariksawan_pria_warna_kulit_gelap-sedang:','zh':':男宇航员_中等深肤色:','ru':':мужчина_космонавт_темный_тон_кожи:','tr':':erkek_astronot_orta_koyu_cilt_tonu:','ar':':رائد_فضاء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U0001F680':{'en':':man_astronaut_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':astronaut_mittelhelle_hautfarbe:','es':':astronauta_hombre_tono_de_piel_claro_medio:','fr':':astronaute_homme_peau_moyennement_claire:','ja':':男性宇宙飛行士_やや薄い肌色:','ko':':남자_우주비행사_연한_갈색_피부:','pt':':astronauta_homem_pele_morena_clara:','it':':astronauta_uomo_carnagione_abbastanza_chiara:','fa':':فضانورد_مرد_پوست_روشن:','id':':antariksawan_pria_warna_kulit_cerah-sedang:','zh':':男宇航员_中等浅肤色:','ru':':мужчина_космонавт_светлый_тон_кожи:','tr':':erkek_astronot_orta_açık_cilt_tonu:','ar':':رائد_فضاء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F680':{'en':':man_astronaut_medium_skin_tone:','status':fully_qualified,'E':4,'de':':astronaut_mittlere_hautfarbe:','es':':astronauta_hombre_tono_de_piel_medio:','fr':':astronaute_homme_peau_légèrement_mate:','ja':':男性宇宙飛行士_中間の肌色:','ko':':남자_우주비행사_갈색_피부:','pt':':astronauta_homem_pele_morena:','it':':astronauta_uomo_carnagione_olivastra:','fa':':فضانورد_مرد_پوست_طلایی:','id':':antariksawan_pria_warna_kulit_sedang:','zh':':男宇航员_中等肤色:','ru':':мужчина_космонавт_средний_тон_кожи:','tr':':erkek_astronot_orta_cilt_tonu:','ar':':رائد_فضاء_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U0001F9B2':{'en':':man_bald:','status':fully_qualified,'E':11,'alias':[':bald_man:'],'de':':mann_glatze:','es':':hombre_sin_pelo:','fr':':homme_chauve:','ja':':男性_はげ頭:','ko':':남자_대머리:','pt':':homem_careca:','it':':uomo_calvo:','fa':':مرد_تاس:','id':':pria_gundul:','zh':':男人_秃顶:','ru':':мужчина_без_волос:','tr':':erkek_kel:','ar':':رجل_أصلع:'},'\U0001F9D4\U0000200D\U00002642\U0000FE0F':{'en':':man_beard:','status':fully_qualified,'E':13.1,'de':':mann_bart:','es':':hombre_barba:','fr':':homme_barbu:','ja':':あごひげの男性:','ko':':수염_난_남자:','pt':':homem_barbudo:','it':':uomo_con_la_barba:','fa':':مرد_ریش:','id':':pria_janggut:','zh':':有络腮胡子的男人:','ru':':бородатыи_мужчина:','tr':':erkek_sakal:','ar':':رجل_بلحية:'},'\U0001F9D4\U0000200D\U00002642':{'en':':man_beard:','status':minimally_qualified,'E':13.1,'de':':mann_bart:','es':':hombre_barba:','fr':':homme_barbu:','ja':':あごひげの男性:','ko':':수염_난_남자:','pt':':homem_barbudo:','it':':uomo_con_la_barba:','fa':':مرد_ریش:','id':':pria_janggut:','zh':':有络腮胡子的男人:','ru':':бородатый_мужчина:','tr':':erkek_sakal:','ar':':رجل_بلحية:'},'\U0001F6B4\U0000200D\U00002642\U0000FE0F':{'en':':man_biking:','status':fully_qualified,'E':4,'alias':[':biking_man:'],'de':':radfahrer:','es':':hombre_en_bicicleta:','fr':':cycliste_homme:','ja':':自転車に乗る男:','ko':':자전거_타는_남자:','pt':':homem_ciclista:','it':':ciclista_uomo:','fa':':دوچرخه_سوار_مرد:','id':':pria_bersepeda:','zh':':男生骑自行车:','ru':':мужчина_на_велосипеде:','tr':':bisiklet_süren_erkek:','ar':':راكب_دراجة:'},'\U0001F6B4\U0000200D\U00002642':{'en':':man_biking:','status':minimally_qualified,'E':4,'alias':[':biking_man:'],'de':':radfahrer:','es':':hombre_en_bicicleta:','fr':':cycliste_homme:','ja':':自転車に乗る男:','ko':':자전거_타는_남자:','pt':':homem_ciclista:','it':':ciclista_uomo:','fa':':دوچرخه_سوار_مرد:','id':':pria_bersepeda:','zh':':男生骑自行车:','ru':':мужчина_на_велосипеде:','tr':':bisiklet_süren_erkek:','ar':':راكب_دراجة:'},'\U0001F6B4\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_biking_dark_skin_tone:','status':fully_qualified,'E':4,'de':':radfahrer_dunkle_hautfarbe:','es':':hombre_en_bicicleta_tono_de_piel_oscuro:','fr':':cycliste_homme_peau_foncée:','ja':':自転車に乗る男_濃い肌色:','ko':':자전거_타는_남자_검은색_피부:','pt':':homem_ciclista_pele_escura:','it':':ciclista_uomo_carnagione_scura:','fa':':دوچرخه_سوار_مرد_پوست_آبنوسی:','id':':pria_bersepeda_warna_kulit_gelap:','zh':':男生骑自行车_较深肤色:','ru':':мужчина_на_велосипеде_очень_темныи_тон_кожи:','tr':':bisiklet_süren_erkek_koyu_cilt_tonu:','ar':':راكب_دراجة_بشرة_بلون_غامق:'},'\U0001F6B4\U0001F3FF\U0000200D\U00002642':{'en':':man_biking_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':radfahrer_dunkle_hautfarbe:','es':':hombre_en_bicicleta_tono_de_piel_oscuro:','fr':':cycliste_homme_peau_foncée:','ja':':自転車に乗る男_濃い肌色:','ko':':자전거_타는_남자_검은색_피부:','pt':':homem_ciclista_pele_escura:','it':':ciclista_uomo_carnagione_scura:','fa':':دوچرخه_سوار_مرد_پوست_آبنوسی:','id':':pria_bersepeda_warna_kulit_gelap:','zh':':男生骑自行车_较深肤色:','ru':':мужчина_на_велосипеде_очень_темный_тон_кожи:','tr':':bisiklet_süren_erkek_koyu_cilt_tonu:','ar':':راكب_دراجة_بشرة_بلون_غامق:'},'\U0001F6B4\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_biking_light_skin_tone:','status':fully_qualified,'E':4,'de':':radfahrer_helle_hautfarbe:','es':':hombre_en_bicicleta_tono_de_piel_claro:','fr':':cycliste_homme_peau_claire:','ja':':自転車に乗る男_薄い肌色:','ko':':자전거_타는_남자_하얀_피부:','pt':':homem_ciclista_pele_clara:','it':':ciclista_uomo_carnagione_chiara:','fa':':دوچرخه_سوار_مرد_پوست_سفید:','id':':pria_bersepeda_warna_kulit_cerah:','zh':':男生骑自行车_较浅肤色:','ru':':мужчина_на_велосипеде_очень_светлыи_тон_кожи:','tr':':bisiklet_süren_erkek_açık_cilt_tonu:','ar':':راكب_دراجة_بشرة_بلون_فاتح:'},'\U0001F6B4\U0001F3FB\U0000200D\U00002642':{'en':':man_biking_light_skin_tone:','status':minimally_qualified,'E':4,'de':':radfahrer_helle_hautfarbe:','es':':hombre_en_bicicleta_tono_de_piel_claro:','fr':':cycliste_homme_peau_claire:','ja':':自転車に乗る男_薄い肌色:','ko':':자전거_타는_남자_하얀_피부:','pt':':homem_ciclista_pele_clara:','it':':ciclista_uomo_carnagione_chiara:','fa':':دوچرخه_سوار_مرد_پوست_سفید:','id':':pria_bersepeda_warna_kulit_cerah:','zh':':男生骑自行车_较浅肤色:','ru':':мужчина_на_велосипеде_очень_светлый_тон_кожи:','tr':':bisiklet_süren_erkek_açık_cilt_tonu:','ar':':راكب_دراجة_بشرة_بلون_فاتح:'},'\U0001F6B4\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_biking_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':radfahrer_mitteldunkle_hautfarbe:','es':':hombre_en_bicicleta_tono_de_piel_oscuro_medio:','fr':':cycliste_homme_peau_mate:','ja':':自転車に乗る男_やや濃い肌色:','ko':':자전거_타는_남자_진한_갈색_피부:','pt':':homem_ciclista_pele_morena_escura:','it':':ciclista_uomo_carnagione_abbastanza_scura:','fa':':دوچرخه_سوار_مرد_پوست_گندمی:','id':':pria_bersepeda_warna_kulit_gelap-sedang:','zh':':男生骑自行车_中等深肤色:','ru':':мужчина_на_велосипеде_темныи_тон_кожи:','tr':':bisiklet_süren_erkek_orta_koyu_cilt_tonu:','ar':':راكب_دراجة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6B4\U0001F3FE\U0000200D\U00002642':{'en':':man_biking_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':radfahrer_mitteldunkle_hautfarbe:','es':':hombre_en_bicicleta_tono_de_piel_oscuro_medio:','fr':':cycliste_homme_peau_mate:','ja':':自転車に乗る男_やや濃い肌色:','ko':':자전거_타는_남자_진한_갈색_피부:','pt':':homem_ciclista_pele_morena_escura:','it':':ciclista_uomo_carnagione_abbastanza_scura:','fa':':دوچرخه_سوار_مرد_پوست_گندمی:','id':':pria_bersepeda_warna_kulit_gelap-sedang:','zh':':男生骑自行车_中等深肤色:','ru':':мужчина_на_велосипеде_темный_тон_кожи:','tr':':bisiklet_süren_erkek_orta_koyu_cilt_tonu:','ar':':راكب_دراجة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6B4\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_biking_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':radfahrer_mittelhelle_hautfarbe:','es':':hombre_en_bicicleta_tono_de_piel_claro_medio:','fr':':cycliste_homme_peau_moyennement_claire:','ja':':自転車に乗る男_やや薄い肌色:','ko':':자전거_타는_남자_연한_갈색_피부:','pt':':homem_ciclista_pele_morena_clara:','it':':ciclista_uomo_carnagione_abbastanza_chiara:','fa':':دوچرخه_سوار_مرد_پوست_روشن:','id':':pria_bersepeda_warna_kulit_cerah-sedang:','zh':':男生骑自行车_中等浅肤色:','ru':':мужчина_на_велосипеде_светлыи_тон_кожи:','tr':':bisiklet_süren_erkek_orta_açık_cilt_tonu:','ar':':راكب_دراجة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6B4\U0001F3FC\U0000200D\U00002642':{'en':':man_biking_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':radfahrer_mittelhelle_hautfarbe:','es':':hombre_en_bicicleta_tono_de_piel_claro_medio:','fr':':cycliste_homme_peau_moyennement_claire:','ja':':自転車に乗る男_やや薄い肌色:','ko':':자전거_타는_남자_연한_갈색_피부:','pt':':homem_ciclista_pele_morena_clara:','it':':ciclista_uomo_carnagione_abbastanza_chiara:','fa':':دوچرخه_سوار_مرد_پوست_روشن:','id':':pria_bersepeda_warna_kulit_cerah-sedang:','zh':':男生骑自行车_中等浅肤色:','ru':':мужчина_на_велосипеде_светлый_тон_кожи:','tr':':bisiklet_süren_erkek_orta_açık_cilt_tonu:','ar':':راكب_دراجة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6B4\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_biking_medium_skin_tone:','status':fully_qualified,'E':4,'de':':radfahrer_mittlere_hautfarbe:','es':':hombre_en_bicicleta_tono_de_piel_medio:','fr':':cycliste_homme_peau_légèrement_mate:','ja':':自転車に乗る男_中間の肌色:','ko':':자전거_타는_남자_갈색_피부:','pt':':homem_ciclista_pele_morena:','it':':ciclista_uomo_carnagione_olivastra:','fa':':دوچرخه_سوار_مرد_پوست_طلایی:','id':':pria_bersepeda_warna_kulit_sedang:','zh':':男生骑自行车_中等肤色:','ru':':мужчина_на_велосипеде_среднии_тон_кожи:','tr':':bisiklet_süren_erkek_orta_cilt_tonu:','ar':':راكب_دراجة_بشرة_بلون_معتدل:'},'\U0001F6B4\U0001F3FD\U0000200D\U00002642':{'en':':man_biking_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':radfahrer_mittlere_hautfarbe:','es':':hombre_en_bicicleta_tono_de_piel_medio:','fr':':cycliste_homme_peau_légèrement_mate:','ja':':自転車に乗る男_中間の肌色:','ko':':자전거_타는_남자_갈색_피부:','pt':':homem_ciclista_pele_morena:','it':':ciclista_uomo_carnagione_olivastra:','fa':':دوچرخه_سوار_مرد_پوست_طلایی:','id':':pria_bersepeda_warna_kulit_sedang:','zh':':男生骑自行车_中等肤色:','ru':':мужчина_на_велосипеде_средний_тон_кожи:','tr':':bisiklet_süren_erkek_orta_cilt_tonu:','ar':':راكب_دراجة_بشرة_بلون_معتدل:'},'\U0001F471\U0000200D\U00002642\U0000FE0F':{'en':':man_blond_hair:','status':fully_qualified,'E':4,'alias':[':blond_haired_man:'],'de':':mann_blond:','es':':hombre_rubio:','fr':':homme_blond:','ja':':金髪の男性:','ko':':금발_남자:','pt':':homem_cabelo_loiro:','it':':uomo_biondo:','fa':':مرد_مو_بور:','id':':pria_berambut_pirang:','zh':':金发男:','ru':':блондин:','tr':':sarışın_erkek:','ar':':رجل_أشقر:'},'\U0001F471\U0000200D\U00002642':{'en':':man_blond_hair:','status':minimally_qualified,'E':4,'alias':[':blond_haired_man:'],'de':':mann_blond:','es':':hombre_rubio:','fr':':homme_blond:','ja':':金髪の男性:','ko':':금발_남자:','pt':':homem_cabelo_loiro:','it':':uomo_biondo:','fa':':مرد_مو_بور:','id':':pria_berambut_pirang:','zh':':金发男:','ru':':блондин:','tr':':sarışın_erkek:','ar':':رجل_أشقر:'},'\U000026F9\U0000FE0F\U0000200D\U00002642\U0000FE0F':{'en':':man_bouncing_ball:','status':fully_qualified,'E':4,'alias':[':basketball_man:', ':bouncing_ball_man:'],'de':':mann_mit_ball:','es':':hombre_botando_un_balón:','fr':':homme_avec_ballon:','ja':':バスケットボールをする男:','ko':':공_가진_남자:','pt':':homem_jogando_basquete:','it':':uomo_che_fa_rimbalzare_una_palla:','fa':':مرد_و_توپ:','id':':pria_memantulkan_bola:','zh':':男生玩球:','ru':':мужчина_с_мячом:','tr':':top_sektiren_erkek:','ar':':رجل_وكرة:'},'\U000026F9\U0000200D\U00002642\U0000FE0F':{'en':':man_bouncing_ball:','status':unqualified,'E':4,'alias':[':basketball_man:', ':bouncing_ball_man:'],'de':':mann_mit_ball:','es':':hombre_botando_un_balón:','fr':':homme_avec_ballon:','ja':':バスケットボールをする男:','ko':':공_가진_남자:','pt':':homem_jogando_basquete:','it':':uomo_che_fa_rimbalzare_una_palla:','fa':':مرد_و_توپ:','id':':pria_memantulkan_bola:','zh':':男生玩球:','ru':':мужчина_с_мячом:','tr':':top_sektiren_erkek:','ar':':رجل_وكرة:'},'\U000026F9\U0000FE0F\U0000200D\U00002642':{'en':':man_bouncing_ball:','status':minimally_qualified,'E':4,'alias':[':basketball_man:', ':bouncing_ball_man:'],'de':':mann_mit_ball:','es':':hombre_botando_un_balón:','fr':':homme_avec_ballon:','ja':':バスケットボールをする男:','ko':':공_가진_남자:','pt':':homem_jogando_basquete:','it':':uomo_che_fa_rimbalzare_una_palla:','fa':':مرد_و_توپ:','id':':pria_memantulkan_bola:','zh':':男生玩球:','ru':':мужчина_с_мячом:','tr':':top_sektiren_erkek:','ar':':رجل_وكرة:'},'\U000026F9\U0000200D\U00002642':{'en':':man_bouncing_ball:','status':unqualified,'E':4,'alias':[':basketball_man:', ':bouncing_ball_man:'],'de':':mann_mit_ball:','es':':hombre_botando_un_balón:','fr':':homme_avec_ballon:','ja':':バスケットボールをする男:','ko':':공_가진_남자:','pt':':homem_jogando_basquete:','it':':uomo_che_fa_rimbalzare_una_palla:','fa':':مرد_و_توپ:','id':':pria_memantulkan_bola:','zh':':男生玩球:','ru':':мужчина_с_мячом:','tr':':top_sektiren_erkek:','ar':':رجل_وكرة:'},'\U000026F9\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_bouncing_ball_dark_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_ball_dunkle_hautfarbe:','es':':hombre_botando_un_balón_tono_de_piel_oscuro:','fr':':homme_avec_ballon_peau_foncée:','ja':':バスケットボールをする男_濃い肌色:','ko':':공_가진_남자_검은색_피부:','pt':':homem_jogando_basquete_pele_escura:','it':':uomo_che_fa_rimbalzare_una_palla_carnagione_scura:','fa':':مرد_و_توپ_پوست_آبنوسی:','id':':pria_memantulkan_bola_warna_kulit_gelap:','zh':':男生玩球_较深肤色:','ru':':мужчина_с_мячом_очень_темныи_тон_кожи:','tr':':top_sektiren_erkek_koyu_cilt_tonu:','ar':':رجل_وكرة_بشرة_بلون_غامق:'},'\U000026F9\U0001F3FF\U0000200D\U00002642':{'en':':man_bouncing_ball_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_ball_dunkle_hautfarbe:','es':':hombre_botando_un_balón_tono_de_piel_oscuro:','fr':':homme_avec_ballon_peau_foncée:','ja':':バスケットボールをする男_濃い肌色:','ko':':공_가진_남자_검은색_피부:','pt':':homem_jogando_basquete_pele_escura:','it':':uomo_che_fa_rimbalzare_una_palla_carnagione_scura:','fa':':مرد_و_توپ_پوست_آبنوسی:','id':':pria_memantulkan_bola_warna_kulit_gelap:','zh':':男生玩球_较深肤色:','ru':':мужчина_с_мячом_очень_темный_тон_кожи:','tr':':top_sektiren_erkek_koyu_cilt_tonu:','ar':':رجل_وكرة_بشرة_بلون_غامق:'},'\U000026F9\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_bouncing_ball_light_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_ball_helle_hautfarbe:','es':':hombre_botando_un_balón_tono_de_piel_claro:','fr':':homme_avec_ballon_peau_claire:','ja':':バスケットボールをする男_薄い肌色:','ko':':공_가진_남자_하얀_피부:','pt':':homem_jogando_basquete_pele_clara:','it':':uomo_che_fa_rimbalzare_una_palla_carnagione_chiara:','fa':':مرد_و_توپ_پوست_سفید:','id':':pria_memantulkan_bola_warna_kulit_cerah:','zh':':男生玩球_较浅肤色:','ru':':мужчина_с_мячом_очень_светлыи_тон_кожи:','tr':':top_sektiren_erkek_açık_cilt_tonu:','ar':':رجل_وكرة_بشرة_بلون_فاتح:'},'\U000026F9\U0001F3FB\U0000200D\U00002642':{'en':':man_bouncing_ball_light_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_ball_helle_hautfarbe:','es':':hombre_botando_un_balón_tono_de_piel_claro:','fr':':homme_avec_ballon_peau_claire:','ja':':バスケットボールをする男_薄い肌色:','ko':':공_가진_남자_하얀_피부:','pt':':homem_jogando_basquete_pele_clara:','it':':uomo_che_fa_rimbalzare_una_palla_carnagione_chiara:','fa':':مرد_و_توپ_پوست_سفید:','id':':pria_memantulkan_bola_warna_kulit_cerah:','zh':':男生玩球_较浅肤色:','ru':':мужчина_с_мячом_очень_светлый_тон_кожи:','tr':':top_sektiren_erkek_açık_cilt_tonu:','ar':':رجل_وكرة_بشرة_بلون_فاتح:'},'\U000026F9\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_bouncing_ball_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_ball_mitteldunkle_hautfarbe:','es':':hombre_botando_un_balón_tono_de_piel_oscuro_medio:','fr':':homme_avec_ballon_peau_mate:','ja':':バスケットボールをする男_やや濃い肌色:','ko':':공_가진_남자_진한_갈색_피부:','pt':':homem_jogando_basquete_pele_morena_escura:','it':':uomo_che_fa_rimbalzare_una_palla_carnagione_abbastanza_scura:','fa':':مرد_و_توپ_پوست_گندمی:','id':':pria_memantulkan_bola_warna_kulit_gelap-sedang:','zh':':男生玩球_中等深肤色:','ru':':мужчина_с_мячом_темныи_тон_кожи:','tr':':top_sektiren_erkek_orta_koyu_cilt_tonu:','ar':':رجل_وكرة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U000026F9\U0001F3FE\U0000200D\U00002642':{'en':':man_bouncing_ball_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_ball_mitteldunkle_hautfarbe:','es':':hombre_botando_un_balón_tono_de_piel_oscuro_medio:','fr':':homme_avec_ballon_peau_mate:','ja':':バスケットボールをする男_やや濃い肌色:','ko':':공_가진_남자_진한_갈색_피부:','pt':':homem_jogando_basquete_pele_morena_escura:','it':':uomo_che_fa_rimbalzare_una_palla_carnagione_abbastanza_scura:','fa':':مرد_و_توپ_پوست_گندمی:','id':':pria_memantulkan_bola_warna_kulit_gelap-sedang:','zh':':男生玩球_中等深肤色:','ru':':мужчина_с_мячом_темный_тон_кожи:','tr':':top_sektiren_erkek_orta_koyu_cilt_tonu:','ar':':رجل_وكرة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U000026F9\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_bouncing_ball_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_ball_mittelhelle_hautfarbe:','es':':hombre_botando_un_balón_tono_de_piel_claro_medio:','fr':':homme_avec_ballon_peau_moyennement_claire:','ja':':バスケットボールをする男_やや薄い肌色:','ko':':공_가진_남자_연한_갈색_피부:','pt':':homem_jogando_basquete_pele_morena_clara:','it':':uomo_che_fa_rimbalzare_una_palla_carnagione_abbastanza_chiara:','fa':':مرد_و_توپ_پوست_روشن:','id':':pria_memantulkan_bola_warna_kulit_cerah-sedang:','zh':':男生玩球_中等浅肤色:','ru':':мужчина_с_мячом_светлыи_тон_кожи:','tr':':top_sektiren_erkek_orta_açık_cilt_tonu:','ar':':رجل_وكرة_بشرة_بلون_فاتح_ومعتدل:'},'\U000026F9\U0001F3FC\U0000200D\U00002642':{'en':':man_bouncing_ball_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_ball_mittelhelle_hautfarbe:','es':':hombre_botando_un_balón_tono_de_piel_claro_medio:','fr':':homme_avec_ballon_peau_moyennement_claire:','ja':':バスケットボールをする男_やや薄い肌色:','ko':':공_가진_남자_연한_갈색_피부:','pt':':homem_jogando_basquete_pele_morena_clara:','it':':uomo_che_fa_rimbalzare_una_palla_carnagione_abbastanza_chiara:','fa':':مرد_و_توپ_پوست_روشن:','id':':pria_memantulkan_bola_warna_kulit_cerah-sedang:','zh':':男生玩球_中等浅肤色:','ru':':мужчина_с_мячом_светлый_тон_кожи:','tr':':top_sektiren_erkek_orta_açık_cilt_tonu:','ar':':رجل_وكرة_بشرة_بلون_فاتح_ومعتدل:'},'\U000026F9\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_bouncing_ball_medium_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_ball_mittlere_hautfarbe:','es':':hombre_botando_un_balón_tono_de_piel_medio:','fr':':homme_avec_ballon_peau_légèrement_mate:','ja':':バスケットボールをする男_中間の肌色:','ko':':공_가진_남자_갈색_피부:','pt':':homem_jogando_basquete_pele_morena:','it':':uomo_che_fa_rimbalzare_una_palla_carnagione_olivastra:','fa':':مرد_و_توپ_پوست_طلایی:','id':':pria_memantulkan_bola_warna_kulit_sedang:','zh':':男生玩球_中等肤色:','ru':':мужчина_с_мячом_среднии_тон_кожи:','tr':':top_sektiren_erkek_orta_cilt_tonu:','ar':':رجل_وكرة_بشرة_بلون_معتدل:'},'\U000026F9\U0001F3FD\U0000200D\U00002642':{'en':':man_bouncing_ball_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_ball_mittlere_hautfarbe:','es':':hombre_botando_un_balón_tono_de_piel_medio:','fr':':homme_avec_ballon_peau_légèrement_mate:','ja':':バスケットボールをする男_中間の肌色:','ko':':공_가진_남자_갈색_피부:','pt':':homem_jogando_basquete_pele_morena:','it':':uomo_che_fa_rimbalzare_una_palla_carnagione_olivastra:','fa':':مرد_و_توپ_پوست_طلایی:','id':':pria_memantulkan_bola_warna_kulit_sedang:','zh':':男生玩球_中等肤色:','ru':':мужчина_с_мячом_средний_тон_кожи:','tr':':top_sektiren_erkek_orta_cilt_tonu:','ar':':رجل_وكرة_بشرة_بلون_معتدل:'},'\U0001F647\U0000200D\U00002642\U0000FE0F':{'en':':man_bowing:','status':fully_qualified,'E':4,'alias':[':bowing_man:'],'de':':sich_verbeugender_mann:','es':':hombre_haciendo_una_reverencia:','fr':':homme_qui_s’incline:','ja':':おじぎする男:','ko':':절하는_남자:','pt':':homem_fazendo_reverência:','it':':uomo_che_fa_inchino_profondo:','fa':':مرد_تعظیم_می_کند:','id':':pria_membungkuk:','zh':':男生鞠躬:','ru':':мужчина_кланяется:','tr':':başını_öne_eğen_erkek:','ar':':رجل_ينحني_اعتذارا:'},'\U0001F647\U0000200D\U00002642':{'en':':man_bowing:','status':minimally_qualified,'E':4,'alias':[':bowing_man:'],'de':':sich_verbeugender_mann:','es':':hombre_haciendo_una_reverencia:','fr':':homme_qui_s’incline:','ja':':おじぎする男:','ko':':절하는_남자:','pt':':homem_fazendo_reverência:','it':':uomo_che_fa_inchino_profondo:','fa':':مرد_تعظیم_می_کند:','id':':pria_membungkuk:','zh':':男生鞠躬:','ru':':мужчина_кланяется:','tr':':başını_öne_eğen_erkek:','ar':':رجل_ينحني_اعتذارا:'},'\U0001F647\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_bowing_dark_skin_tone:','status':fully_qualified,'E':4,'de':':sich_verbeugender_mann_dunkle_hautfarbe:','es':':hombre_haciendo_una_reverencia_tono_de_piel_oscuro:','fr':':homme_qui_s’incline_peau_foncée:','ja':':おじぎする男_濃い肌色:','ko':':절하는_남자_검은색_피부:','pt':':homem_fazendo_reverência_pele_escura:','it':':uomo_che_fa_inchino_profondo_carnagione_scura:','fa':':مرد_تعظیم_می_کند_پوست_آبنوسی:','id':':pria_membungkuk_warna_kulit_gelap:','zh':':男生鞠躬_较深肤色:','ru':':мужчина_кланяется_очень_темныи_тон_кожи:','tr':':başını_öne_eğen_erkek_koyu_cilt_tonu:','ar':':رجل_ينحني_اعتذارا_بشرة_بلون_غامق:'},'\U0001F647\U0001F3FF\U0000200D\U00002642':{'en':':man_bowing_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':sich_verbeugender_mann_dunkle_hautfarbe:','es':':hombre_haciendo_una_reverencia_tono_de_piel_oscuro:','fr':':homme_qui_s’incline_peau_foncée:','ja':':おじぎする男_濃い肌色:','ko':':절하는_남자_검은색_피부:','pt':':homem_fazendo_reverência_pele_escura:','it':':uomo_che_fa_inchino_profondo_carnagione_scura:','fa':':مرد_تعظیم_می_کند_پوست_آبنوسی:','id':':pria_membungkuk_warna_kulit_gelap:','zh':':男生鞠躬_较深肤色:','ru':':мужчина_кланяется_очень_темный_тон_кожи:','tr':':başını_öne_eğen_erkek_koyu_cilt_tonu:','ar':':رجل_ينحني_اعتذارا_بشرة_بلون_غامق:'},'\U0001F647\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_bowing_light_skin_tone:','status':fully_qualified,'E':4,'de':':sich_verbeugender_mann_helle_hautfarbe:','es':':hombre_haciendo_una_reverencia_tono_de_piel_claro:','fr':':homme_qui_s’incline_peau_claire:','ja':':おじぎする男_薄い肌色:','ko':':절하는_남자_하얀_피부:','pt':':homem_fazendo_reverência_pele_clara:','it':':uomo_che_fa_inchino_profondo_carnagione_chiara:','fa':':مرد_تعظیم_می_کند_پوست_سفید:','id':':pria_membungkuk_warna_kulit_cerah:','zh':':男生鞠躬_较浅肤色:','ru':':мужчина_кланяется_очень_светлыи_тон_кожи:','tr':':başını_öne_eğen_erkek_açık_cilt_tonu:','ar':':رجل_ينحني_اعتذارا_بشرة_بلون_فاتح:'},'\U0001F647\U0001F3FB\U0000200D\U00002642':{'en':':man_bowing_light_skin_tone:','status':minimally_qualified,'E':4,'de':':sich_verbeugender_mann_helle_hautfarbe:','es':':hombre_haciendo_una_reverencia_tono_de_piel_claro:','fr':':homme_qui_s’incline_peau_claire:','ja':':おじぎする男_薄い肌色:','ko':':절하는_남자_하얀_피부:','pt':':homem_fazendo_reverência_pele_clara:','it':':uomo_che_fa_inchino_profondo_carnagione_chiara:','fa':':مرد_تعظیم_می_کند_پوست_سفید:','id':':pria_membungkuk_warna_kulit_cerah:','zh':':男生鞠躬_较浅肤色:','ru':':мужчина_кланяется_очень_светлый_тон_кожи:','tr':':başını_öne_eğen_erkek_açık_cilt_tonu:','ar':':رجل_ينحني_اعتذارا_بشرة_بلون_فاتح:'},'\U0001F647\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_bowing_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':sich_verbeugender_mann_mitteldunkle_hautfarbe:','es':':hombre_haciendo_una_reverencia_tono_de_piel_oscuro_medio:','fr':':homme_qui_s’incline_peau_mate:','ja':':おじぎする男_やや濃い肌色:','ko':':절하는_남자_진한_갈색_피부:','pt':':homem_fazendo_reverência_pele_morena_escura:','it':':uomo_che_fa_inchino_profondo_carnagione_abbastanza_scura:','fa':':مرد_تعظیم_می_کند_پوست_گندمی:','id':':pria_membungkuk_warna_kulit_gelap-sedang:','zh':':男生鞠躬_中等深肤色:','ru':':мужчина_кланяется_темныи_тон_кожи:','tr':':başını_öne_eğen_erkek_orta_koyu_cilt_tonu:','ar':':رجل_ينحني_اعتذارا_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F647\U0001F3FE\U0000200D\U00002642':{'en':':man_bowing_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':sich_verbeugender_mann_mitteldunkle_hautfarbe:','es':':hombre_haciendo_una_reverencia_tono_de_piel_oscuro_medio:','fr':':homme_qui_s’incline_peau_mate:','ja':':おじぎする男_やや濃い肌色:','ko':':절하는_남자_진한_갈색_피부:','pt':':homem_fazendo_reverência_pele_morena_escura:','it':':uomo_che_fa_inchino_profondo_carnagione_abbastanza_scura:','fa':':مرد_تعظیم_می_کند_پوست_گندمی:','id':':pria_membungkuk_warna_kulit_gelap-sedang:','zh':':男生鞠躬_中等深肤色:','ru':':мужчина_кланяется_темный_тон_кожи:','tr':':başını_öne_eğen_erkek_orta_koyu_cilt_tonu:','ar':':رجل_ينحني_اعتذارا_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F647\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_bowing_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':sich_verbeugender_mann_mittelhelle_hautfarbe:','es':':hombre_haciendo_una_reverencia_tono_de_piel_claro_medio:','fr':':homme_qui_s’incline_peau_moyennement_claire:','ja':':おじぎする男_やや薄い肌色:','ko':':절하는_남자_연한_갈색_피부:','pt':':homem_fazendo_reverência_pele_morena_clara:','it':':uomo_che_fa_inchino_profondo_carnagione_abbastanza_chiara:','fa':':مرد_تعظیم_می_کند_پوست_روشن:','id':':pria_membungkuk_warna_kulit_cerah-sedang:','zh':':男生鞠躬_中等浅肤色:','ru':':мужчина_кланяется_светлыи_тон_кожи:','tr':':başını_öne_eğen_erkek_orta_açık_cilt_tonu:','ar':':رجل_ينحني_اعتذارا_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F647\U0001F3FC\U0000200D\U00002642':{'en':':man_bowing_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':sich_verbeugender_mann_mittelhelle_hautfarbe:','es':':hombre_haciendo_una_reverencia_tono_de_piel_claro_medio:','fr':':homme_qui_s’incline_peau_moyennement_claire:','ja':':おじぎする男_やや薄い肌色:','ko':':절하는_남자_연한_갈색_피부:','pt':':homem_fazendo_reverência_pele_morena_clara:','it':':uomo_che_fa_inchino_profondo_carnagione_abbastanza_chiara:','fa':':مرد_تعظیم_می_کند_پوست_روشن:','id':':pria_membungkuk_warna_kulit_cerah-sedang:','zh':':男生鞠躬_中等浅肤色:','ru':':мужчина_кланяется_светлый_тон_кожи:','tr':':başını_öne_eğen_erkek_orta_açık_cilt_tonu:','ar':':رجل_ينحني_اعتذارا_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F647\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_bowing_medium_skin_tone:','status':fully_qualified,'E':4,'de':':sich_verbeugender_mann_mittlere_hautfarbe:','es':':hombre_haciendo_una_reverencia_tono_de_piel_medio:','fr':':homme_qui_s’incline_peau_légèrement_mate:','ja':':おじぎする男_中間の肌色:','ko':':절하는_남자_갈색_피부:','pt':':homem_fazendo_reverência_pele_morena:','it':':uomo_che_fa_inchino_profondo_carnagione_olivastra:','fa':':مرد_تعظیم_می_کند_پوست_طلایی:','id':':pria_membungkuk_warna_kulit_sedang:','zh':':男生鞠躬_中等肤色:','ru':':мужчина_кланяется_среднии_тон_кожи:','tr':':başını_öne_eğen_erkek_orta_cilt_tonu:','ar':':رجل_ينحني_اعتذارا_بشرة_بلون_معتدل:'},'\U0001F647\U0001F3FD\U0000200D\U00002642':{'en':':man_bowing_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':sich_verbeugender_mann_mittlere_hautfarbe:','es':':hombre_haciendo_una_reverencia_tono_de_piel_medio:','fr':':homme_qui_s’incline_peau_légèrement_mate:','ja':':おじぎする男_中間の肌色:','ko':':절하는_남자_갈색_피부:','pt':':homem_fazendo_reverência_pele_morena:','it':':uomo_che_fa_inchino_profondo_carnagione_olivastra:','fa':':مرد_تعظیم_می_کند_پوست_طلایی:','id':':pria_membungkuk_warna_kulit_sedang:','zh':':男生鞠躬_中等肤色:','ru':':мужчина_кланяется_средний_тон_кожи:','tr':':başını_öne_eğen_erkek_orta_cilt_tonu:','ar':':رجل_ينحني_اعتذارا_بشرة_بلون_معتدل:'},'\U0001F938\U0000200D\U00002642\U0000FE0F':{'en':':man_cartwheeling:','status':fully_qualified,'E':4,'de':':rad_schlagender_mann:','es':':hombre_haciendo_una_voltereta_lateral:','fr':':homme_faisant_la_roue:','ja':':側転する男:','ko':':옆돌기하는_남자:','pt':':homem_fazendo_estrela:','it':':uomo_che_fa_la_ruota:','fa':':مردِ_چرخ_وفلک_زن:','id':':pria_melakukan_gerakan_meroda:','zh':':男生侧手翻:','ru':':акробат:','tr':':perende_atan_erkek:','ar':':رجل_يفعل_حركة_العجلة:'},'\U0001F938\U0000200D\U00002642':{'en':':man_cartwheeling:','status':minimally_qualified,'E':4,'de':':rad_schlagender_mann:','es':':hombre_dando_una_voltereta_lateral:','fr':':homme_faisant_la_roue:','ja':':側転する男:','ko':':옆돌기하는_남자:','pt':':homem_fazendo_estrela:','it':':uomo_che_fa_la_ruota:','fa':':مردِ_چرخ_وفلک_زن:','id':':pria_melakukan_gerakan_meroda:','zh':':男生侧手翻:','ru':':акробат:','tr':':perende_atan_erkek:','ar':':رجل_يفعل_حركة_العجلة:'},'\U0001F938\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_cartwheeling_dark_skin_tone:','status':fully_qualified,'E':4,'de':':rad_schlagender_mann_dunkle_hautfarbe:','es':':hombre_haciendo_una_voltereta_lateral_tono_de_piel_oscuro:','fr':':homme_faisant_la_roue_peau_foncée:','ja':':側転する男_濃い肌色:','ko':':옆돌기하는_남자_검은색_피부:','pt':':homem_fazendo_estrela_pele_escura:','it':':uomo_che_fa_la_ruota_carnagione_scura:','fa':':مردِ_چرخ_وفلک_زن_پوست_آبنوسی:','id':':pria_melakukan_gerakan_meroda_warna_kulit_gelap:','zh':':男生侧手翻_较深肤色:','ru':':акробат_очень_темныи_тон_кожи:','tr':':perende_atan_erkek_koyu_cilt_tonu:','ar':':رجل_يفعل_حركة_العجلة_بشرة_بلون_غامق:'},'\U0001F938\U0001F3FF\U0000200D\U00002642':{'en':':man_cartwheeling_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':rad_schlagender_mann_dunkle_hautfarbe:','es':':hombre_dando_una_voltereta_lateral_tono_de_piel_oscuro:','fr':':homme_faisant_la_roue_peau_foncée:','ja':':側転する男_濃い肌色:','ko':':옆돌기하는_남자_검은색_피부:','pt':':homem_fazendo_estrela_pele_escura:','it':':uomo_che_fa_la_ruota_carnagione_scura:','fa':':مردِ_چرخ_وفلک_زن_پوست_آبنوسی:','id':':pria_melakukan_gerakan_meroda_warna_kulit_gelap:','zh':':男生侧手翻_较深肤色:','ru':':акробат_очень_темный_тон_кожи:','tr':':perende_atan_erkek_koyu_cilt_tonu:','ar':':رجل_يفعل_حركة_العجلة_بشرة_بلون_غامق:'},'\U0001F938\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_cartwheeling_light_skin_tone:','status':fully_qualified,'E':4,'de':':rad_schlagender_mann_helle_hautfarbe:','es':':hombre_haciendo_una_voltereta_lateral_tono_de_piel_claro:','fr':':homme_faisant_la_roue_peau_claire:','ja':':側転する男_薄い肌色:','ko':':옆돌기하는_남자_하얀_피부:','pt':':homem_fazendo_estrela_pele_clara:','it':':uomo_che_fa_la_ruota_carnagione_chiara:','fa':':مردِ_چرخ_وفلک_زن_پوست_سفید:','id':':pria_melakukan_gerakan_meroda_warna_kulit_cerah:','zh':':男生侧手翻_较浅肤色:','ru':':акробат_очень_светлыи_тон_кожи:','tr':':perende_atan_erkek_açık_cilt_tonu:','ar':':رجل_يفعل_حركة_العجلة_بشرة_بلون_فاتح:'},'\U0001F938\U0001F3FB\U0000200D\U00002642':{'en':':man_cartwheeling_light_skin_tone:','status':minimally_qualified,'E':4,'de':':rad_schlagender_mann_helle_hautfarbe:','es':':hombre_dando_una_voltereta_lateral_tono_de_piel_claro:','fr':':homme_faisant_la_roue_peau_claire:','ja':':側転する男_薄い肌色:','ko':':옆돌기하는_남자_하얀_피부:','pt':':homem_fazendo_estrela_pele_clara:','it':':uomo_che_fa_la_ruota_carnagione_chiara:','fa':':مردِ_چرخ_وفلک_زن_پوست_سفید:','id':':pria_melakukan_gerakan_meroda_warna_kulit_cerah:','zh':':男生侧手翻_较浅肤色:','ru':':акробат_очень_светлый_тон_кожи:','tr':':perende_atan_erkek_açık_cilt_tonu:','ar':':رجل_يفعل_حركة_العجلة_بشرة_بلون_فاتح:'},'\U0001F938\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_cartwheeling_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':rad_schlagender_mann_mitteldunkle_hautfarbe:','es':':hombre_haciendo_una_voltereta_lateral_tono_de_piel_oscuro_medio:','fr':':homme_faisant_la_roue_peau_mate:','ja':':側転する男_やや濃い肌色:','ko':':옆돌기하는_남자_진한_갈색_피부:','pt':':homem_fazendo_estrela_pele_morena_escura:','it':':uomo_che_fa_la_ruota_carnagione_abbastanza_scura:','fa':':مردِ_چرخ_وفلک_زن_پوست_گندمی:','id':':pria_melakukan_gerakan_meroda_warna_kulit_gelap-sedang:','zh':':男生侧手翻_中等深肤色:','ru':':акробат_темныи_тон_кожи:','tr':':perende_atan_erkek_orta_koyu_cilt_tonu:','ar':':رجل_يفعل_حركة_العجلة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F938\U0001F3FE\U0000200D\U00002642':{'en':':man_cartwheeling_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':rad_schlagender_mann_mitteldunkle_hautfarbe:','es':':hombre_dando_una_voltereta_lateral_tono_de_piel_oscuro_medio:','fr':':homme_faisant_la_roue_peau_mate:','ja':':側転する男_やや濃い肌色:','ko':':옆돌기하는_남자_진한_갈색_피부:','pt':':homem_fazendo_estrela_pele_morena_escura:','it':':uomo_che_fa_la_ruota_carnagione_abbastanza_scura:','fa':':مردِ_چرخ_وفلک_زن_پوست_گندمی:','id':':pria_melakukan_gerakan_meroda_warna_kulit_gelap-sedang:','zh':':男生侧手翻_中等深肤色:','ru':':акробат_темный_тон_кожи:','tr':':perende_atan_erkek_orta_koyu_cilt_tonu:','ar':':رجل_يفعل_حركة_العجلة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F938\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_cartwheeling_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':rad_schlagender_mann_mittelhelle_hautfarbe:','es':':hombre_haciendo_una_voltereta_lateral_tono_de_piel_claro_medio:','fr':':homme_faisant_la_roue_peau_moyennement_claire:','ja':':側転する男_やや薄い肌色:','ko':':옆돌기하는_남자_연한_갈색_피부:','pt':':homem_fazendo_estrela_pele_morena_clara:','it':':uomo_che_fa_la_ruota_carnagione_abbastanza_chiara:','fa':':مردِ_چرخ_وفلک_زن_پوست_روشن:','id':':pria_melakukan_gerakan_meroda_warna_kulit_cerah-sedang:','zh':':男生侧手翻_中等浅肤色:','ru':':акробат_светлыи_тон_кожи:','tr':':perende_atan_erkek_orta_açık_cilt_tonu:','ar':':رجل_يفعل_حركة_العجلة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F938\U0001F3FC\U0000200D\U00002642':{'en':':man_cartwheeling_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':rad_schlagender_mann_mittelhelle_hautfarbe:','es':':hombre_dando_una_voltereta_lateral_tono_de_piel_claro_medio:','fr':':homme_faisant_la_roue_peau_moyennement_claire:','ja':':側転する男_やや薄い肌色:','ko':':옆돌기하는_남자_연한_갈색_피부:','pt':':homem_fazendo_estrela_pele_morena_clara:','it':':uomo_che_fa_la_ruota_carnagione_abbastanza_chiara:','fa':':مردِ_چرخ_وفلک_زن_پوست_روشن:','id':':pria_melakukan_gerakan_meroda_warna_kulit_cerah-sedang:','zh':':男生侧手翻_中等浅肤色:','ru':':акробат_светлый_тон_кожи:','tr':':perende_atan_erkek_orta_açık_cilt_tonu:','ar':':رجل_يفعل_حركة_العجلة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F938\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_cartwheeling_medium_skin_tone:','status':fully_qualified,'E':4,'de':':rad_schlagender_mann_mittlere_hautfarbe:','es':':hombre_haciendo_una_voltereta_lateral_tono_de_piel_medio:','fr':':homme_faisant_la_roue_peau_légèrement_mate:','ja':':側転する男_中間の肌色:','ko':':옆돌기하는_남자_갈색_피부:','pt':':homem_fazendo_estrela_pele_morena:','it':':uomo_che_fa_la_ruota_carnagione_olivastra:','fa':':مردِ_چرخ_وفلک_زن_پوست_طلایی:','id':':pria_melakukan_gerakan_meroda_warna_kulit_sedang:','zh':':男生侧手翻_中等肤色:','ru':':акробат_среднии_тон_кожи:','tr':':perende_atan_erkek_orta_cilt_tonu:','ar':':رجل_يفعل_حركة_العجلة_بشرة_بلون_معتدل:'},'\U0001F938\U0001F3FD\U0000200D\U00002642':{'en':':man_cartwheeling_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':rad_schlagender_mann_mittlere_hautfarbe:','es':':hombre_dando_una_voltereta_lateral_tono_de_piel_medio:','fr':':homme_faisant_la_roue_peau_légèrement_mate:','ja':':側転する男_中間の肌色:','ko':':옆돌기하는_남자_갈색_피부:','pt':':homem_fazendo_estrela_pele_morena:','it':':uomo_che_fa_la_ruota_carnagione_olivastra:','fa':':مردِ_چرخ_وفلک_زن_پوست_طلایی:','id':':pria_melakukan_gerakan_meroda_warna_kulit_sedang:','zh':':男生侧手翻_中等肤色:','ru':':акробат_средний_тон_кожи:','tr':':perende_atan_erkek_orta_cilt_tonu:','ar':':رجل_يفعل_حركة_العجلة_بشرة_بلون_معتدل:'},'\U0001F9D7\U0000200D\U00002642\U0000FE0F':{'en':':man_climbing:','status':fully_qualified,'E':5,'alias':[':climbing_man:'],'de':':bergsteiger:','es':':hombre_escalando:','fr':':homme_qui_grimpe:','ja':':山を登る男:','ko':':클라이밍하는_남자:','pt':':homem_escalando:','it':':scalatore:','fa':':مرد_کوه_نوردی_می_کند:','id':':pria_memanjat:','zh':':攀爬的男人:','ru':':мужчина_взбирается_по_скале:','tr':':dağcı_erkek:','ar':':رجل_يتسلق:'},'\U0001F9D7\U0000200D\U00002642':{'en':':man_climbing:','status':minimally_qualified,'E':5,'alias':[':climbing_man:'],'de':':bergsteiger:','es':':hombre_escalando:','fr':':homme_qui_grimpe:','ja':':山を登る男:','ko':':클라이밍하는_남자:','pt':':homem_escalando:','it':':scalatore:','fa':':مرد_کوه_نوردی_می_کند:','id':':pria_memanjat:','zh':':攀爬的男人:','ru':':мужчина_взбирается_по_скале:','tr':':dağcı_erkek:','ar':':رجل_يتسلق:'},'\U0001F9D7\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_climbing_dark_skin_tone:','status':fully_qualified,'E':5,'de':':bergsteiger_dunkle_hautfarbe:','es':':hombre_escalando_tono_de_piel_oscuro:','fr':':homme_qui_grimpe_peau_foncée:','ja':':山を登る男_濃い肌色:','ko':':클라이밍하는_남자_검은색_피부:','pt':':homem_escalando_pele_escura:','it':':scalatore_carnagione_scura:','fa':':مرد_کوه_نوردی_می_کند_پوست_آبنوسی:','id':':pria_memanjat_warna_kulit_gelap:','zh':':攀爬的男人_较深肤色:','ru':':мужчина_взбирается_по_скале_очень_темныи_тон_кожи:','tr':':dağcı_erkek_koyu_cilt_tonu:','ar':':رجل_يتسلق_بشرة_بلون_غامق:'},'\U0001F9D7\U0001F3FF\U0000200D\U00002642':{'en':':man_climbing_dark_skin_tone:','status':minimally_qualified,'E':5,'de':':bergsteiger_dunkle_hautfarbe:','es':':hombre_escalando_tono_de_piel_oscuro:','fr':':homme_qui_grimpe_peau_foncée:','ja':':山を登る男_濃い肌色:','ko':':클라이밍하는_남자_검은색_피부:','pt':':homem_escalando_pele_escura:','it':':scalatore_carnagione_scura:','fa':':مرد_کوه_نوردی_می_کند_پوست_آبنوسی:','id':':pria_memanjat_warna_kulit_gelap:','zh':':攀爬的男人_较深肤色:','ru':':мужчина_взбирается_по_скале_очень_темный_тон_кожи:','tr':':dağcı_erkek_koyu_cilt_tonu:','ar':':رجل_يتسلق_بشرة_بلون_غامق:'},'\U0001F9D7\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_climbing_light_skin_tone:','status':fully_qualified,'E':5,'de':':bergsteiger_helle_hautfarbe:','es':':hombre_escalando_tono_de_piel_claro:','fr':':homme_qui_grimpe_peau_claire:','ja':':山を登る男_薄い肌色:','ko':':클라이밍하는_남자_하얀_피부:','pt':':homem_escalando_pele_clara:','it':':scalatore_carnagione_chiara:','fa':':مرد_کوه_نوردی_می_کند_پوست_سفید:','id':':pria_memanjat_warna_kulit_cerah:','zh':':攀爬的男人_较浅肤色:','ru':':мужчина_взбирается_по_скале_очень_светлыи_тон_кожи:','tr':':dağcı_erkek_açık_cilt_tonu:','ar':':رجل_يتسلق_بشرة_بلون_فاتح:'},'\U0001F9D7\U0001F3FB\U0000200D\U00002642':{'en':':man_climbing_light_skin_tone:','status':minimally_qualified,'E':5,'de':':bergsteiger_helle_hautfarbe:','es':':hombre_escalando_tono_de_piel_claro:','fr':':homme_qui_grimpe_peau_claire:','ja':':山を登る男_薄い肌色:','ko':':클라이밍하는_남자_하얀_피부:','pt':':homem_escalando_pele_clara:','it':':scalatore_carnagione_chiara:','fa':':مرد_کوه_نوردی_می_کند_پوست_سفید:','id':':pria_memanjat_warna_kulit_cerah:','zh':':攀爬的男人_较浅肤色:','ru':':мужчина_взбирается_по_скале_очень_светлый_тон_кожи:','tr':':dağcı_erkek_açık_cilt_tonu:','ar':':رجل_يتسلق_بشرة_بلون_فاتح:'},'\U0001F9D7\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_climbing_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':bergsteiger_mitteldunkle_hautfarbe:','es':':hombre_escalando_tono_de_piel_oscuro_medio:','fr':':homme_qui_grimpe_peau_mate:','ja':':山を登る男_やや濃い肌色:','ko':':클라이밍하는_남자_진한_갈색_피부:','pt':':homem_escalando_pele_morena_escura:','it':':scalatore_carnagione_abbastanza_scura:','fa':':مرد_کوه_نوردی_می_کند_پوست_گندمی:','id':':pria_memanjat_warna_kulit_gelap-sedang:','zh':':攀爬的男人_中等深肤色:','ru':':мужчина_взбирается_по_скале_темныи_тон_кожи:','tr':':dağcı_erkek_orta_koyu_cilt_tonu:','ar':':رجل_يتسلق_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D7\U0001F3FE\U0000200D\U00002642':{'en':':man_climbing_medium-dark_skin_tone:','status':minimally_qualified,'E':5,'de':':bergsteiger_mitteldunkle_hautfarbe:','es':':hombre_escalando_tono_de_piel_oscuro_medio:','fr':':homme_qui_grimpe_peau_mate:','ja':':山を登る男_やや濃い肌色:','ko':':클라이밍하는_남자_진한_갈색_피부:','pt':':homem_escalando_pele_morena_escura:','it':':scalatore_carnagione_abbastanza_scura:','fa':':مرد_کوه_نوردی_می_کند_پوست_گندمی:','id':':pria_memanjat_warna_kulit_gelap-sedang:','zh':':攀爬的男人_中等深肤色:','ru':':мужчина_взбирается_по_скале_темный_тон_кожи:','tr':':dağcı_erkek_orta_koyu_cilt_tonu:','ar':':رجل_يتسلق_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D7\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_climbing_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':bergsteiger_mittelhelle_hautfarbe:','es':':hombre_escalando_tono_de_piel_claro_medio:','fr':':homme_qui_grimpe_peau_moyennement_claire:','ja':':山を登る男_やや薄い肌色:','ko':':클라이밍하는_남자_연한_갈색_피부:','pt':':homem_escalando_pele_morena_clara:','it':':scalatore_carnagione_abbastanza_chiara:','fa':':مرد_کوه_نوردی_می_کند_پوست_روشن:','id':':pria_memanjat_warna_kulit_cerah-sedang:','zh':':攀爬的男人_中等浅肤色:','ru':':мужчина_взбирается_по_скале_светлыи_тон_кожи:','tr':':dağcı_erkek_orta_açık_cilt_tonu:','ar':':رجل_يتسلق_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D7\U0001F3FC\U0000200D\U00002642':{'en':':man_climbing_medium-light_skin_tone:','status':minimally_qualified,'E':5,'de':':bergsteiger_mittelhelle_hautfarbe:','es':':hombre_escalando_tono_de_piel_claro_medio:','fr':':homme_qui_grimpe_peau_moyennement_claire:','ja':':山を登る男_やや薄い肌色:','ko':':클라이밍하는_남자_연한_갈색_피부:','pt':':homem_escalando_pele_morena_clara:','it':':scalatore_carnagione_abbastanza_chiara:','fa':':مرد_کوه_نوردی_می_کند_پوست_روشن:','id':':pria_memanjat_warna_kulit_cerah-sedang:','zh':':攀爬的男人_中等浅肤色:','ru':':мужчина_взбирается_по_скале_светлый_тон_кожи:','tr':':dağcı_erkek_orta_açık_cilt_tonu:','ar':':رجل_يتسلق_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D7\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_climbing_medium_skin_tone:','status':fully_qualified,'E':5,'de':':bergsteiger_mittlere_hautfarbe:','es':':hombre_escalando_tono_de_piel_medio:','fr':':homme_qui_grimpe_peau_légèrement_mate:','ja':':山を登る男_中間の肌色:','ko':':클라이밍하는_남자_갈색_피부:','pt':':homem_escalando_pele_morena:','it':':scalatore_carnagione_olivastra:','fa':':مرد_کوه_نوردی_می_کند_پوست_طلایی:','id':':pria_memanjat_warna_kulit_sedang:','zh':':攀爬的男人_中等肤色:','ru':':мужчина_взбирается_по_скале_среднии_тон_кожи:','tr':':dağcı_erkek_orta_cilt_tonu:','ar':':رجل_يتسلق_بشرة_بلون_معتدل:'},'\U0001F9D7\U0001F3FD\U0000200D\U00002642':{'en':':man_climbing_medium_skin_tone:','status':minimally_qualified,'E':5,'de':':bergsteiger_mittlere_hautfarbe:','es':':hombre_escalando_tono_de_piel_medio:','fr':':homme_qui_grimpe_peau_légèrement_mate:','ja':':山を登る男_中間の肌色:','ko':':클라이밍하는_남자_갈색_피부:','pt':':homem_escalando_pele_morena:','it':':scalatore_carnagione_olivastra:','fa':':مرد_کوه_نوردی_می_کند_پوست_طلایی:','id':':pria_memanjat_warna_kulit_sedang:','zh':':攀爬的男人_中等肤色:','ru':':мужчина_взбирается_по_скале_средний_тон_кожи:','tr':':dağcı_erkek_orta_cilt_tonu:','ar':':رجل_يتسلق_بشرة_بلون_معتدل:'},'\U0001F477\U0000200D\U00002642\U0000FE0F':{'en':':man_construction_worker:','status':fully_qualified,'E':4,'alias':[':construction_worker_man:'],'de':':bauarbeiter:','es':':profesional_de_la_construcción_hombre:','fr':':ouvrier_du_bâtiment:','ja':':男性の建設作業員:','ko':':건설_현장_남자_노동자:','pt':':pedreiro:','it':':operaio_edile_uomo:','fa':':کارگر_ساختمانی_مرد:','id':':pekerja_konstruksi_pria:','zh':':男建筑工人:','ru':':мужчина-строитель:','tr':':erkek_inşaat_işçisi:','ar':':عامل_بناء:'},'\U0001F477\U0000200D\U00002642':{'en':':man_construction_worker:','status':minimally_qualified,'E':4,'alias':[':construction_worker_man:'],'de':':bauarbeiter:','es':':profesional_de_la_construcción_hombre:','fr':':ouvrier_du_bâtiment:','ja':':男性の建設作業員:','ko':':건설_현장_남자_노동자:','pt':':pedreiro:','it':':operaio_edile_uomo:','fa':':کارگر_ساختمانی_مرد:','id':':pekerja_konstruksi_pria:','zh':':男建筑工人:','ru':':мужчина_строитель:','tr':':erkek_inşaat_işçisi:','ar':':عامل_بناء:'},'\U0001F477\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_construction_worker_dark_skin_tone:','status':fully_qualified,'E':4,'de':':bauarbeiter_dunkle_hautfarbe:','es':':profesional_de_la_construcción_hombre_tono_de_piel_oscuro:','fr':':ouvrier_du_bâtiment_peau_foncée:','ja':':男性の建設作業員_濃い肌色:','ko':':건설_현장_남자_노동자_검은색_피부:','pt':':pedreiro_pele_escura:','it':':operaio_edile_uomo_carnagione_scura:','fa':':کارگر_ساختمانی_مرد_پوست_آبنوسی:','id':':pekerja_konstruksi_pria_warna_kulit_gelap:','zh':':男建筑工人_较深肤色:','ru':':мужчина-строитель_очень_темныи_тон_кожи:','tr':':erkek_inşaat_işçisi_koyu_cilt_tonu:','ar':':عامل_بناء_بشرة_بلون_غامق:'},'\U0001F477\U0001F3FF\U0000200D\U00002642':{'en':':man_construction_worker_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':bauarbeiter_dunkle_hautfarbe:','es':':profesional_de_la_construcción_hombre_tono_de_piel_oscuro:','fr':':ouvrier_du_bâtiment_peau_foncée:','ja':':男性の建設作業員_濃い肌色:','ko':':건설_현장_남자_노동자_검은색_피부:','pt':':pedreiro_pele_escura:','it':':operaio_edile_uomo_carnagione_scura:','fa':':کارگر_ساختمانی_مرد_پوست_آبنوسی:','id':':pekerja_konstruksi_pria_warna_kulit_gelap:','zh':':男建筑工人_较深肤色:','ru':':мужчина_строитель_очень_темный_тон_кожи:','tr':':erkek_inşaat_işçisi_koyu_cilt_tonu:','ar':':عامل_بناء_بشرة_بلون_غامق:'},'\U0001F477\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_construction_worker_light_skin_tone:','status':fully_qualified,'E':4,'de':':bauarbeiter_helle_hautfarbe:','es':':profesional_de_la_construcción_hombre_tono_de_piel_claro:','fr':':ouvrier_du_bâtiment_peau_claire:','ja':':男性の建設作業員_薄い肌色:','ko':':건설_현장_남자_노동자_하얀_피부:','pt':':pedreiro_pele_clara:','it':':operaio_edile_uomo_carnagione_chiara:','fa':':کارگر_ساختمانی_مرد_پوست_سفید:','id':':pekerja_konstruksi_pria_warna_kulit_cerah:','zh':':男建筑工人_较浅肤色:','ru':':мужчина-строитель_очень_светлыи_тон_кожи:','tr':':erkek_inşaat_işçisi_açık_cilt_tonu:','ar':':عامل_بناء_بشرة_بلون_فاتح:'},'\U0001F477\U0001F3FB\U0000200D\U00002642':{'en':':man_construction_worker_light_skin_tone:','status':minimally_qualified,'E':4,'de':':bauarbeiter_helle_hautfarbe:','es':':profesional_de_la_construcción_hombre_tono_de_piel_claro:','fr':':ouvrier_du_bâtiment_peau_claire:','ja':':男性の建設作業員_薄い肌色:','ko':':건설_현장_남자_노동자_하얀_피부:','pt':':pedreiro_pele_clara:','it':':operaio_edile_uomo_carnagione_chiara:','fa':':کارگر_ساختمانی_مرد_پوست_سفید:','id':':pekerja_konstruksi_pria_warna_kulit_cerah:','zh':':男建筑工人_较浅肤色:','ru':':мужчина_строитель_очень_светлый_тон_кожи:','tr':':erkek_inşaat_işçisi_açık_cilt_tonu:','ar':':عامل_بناء_بشرة_بلون_فاتح:'},'\U0001F477\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_construction_worker_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':bauarbeiter_mitteldunkle_hautfarbe:','es':':profesional_de_la_construcción_hombre_tono_de_piel_oscuro_medio:','fr':':ouvrier_du_bâtiment_peau_mate:','ja':':男性の建設作業員_やや濃い肌色:','ko':':건설_현장_남자_노동자_진한_갈색_피부:','pt':':pedreiro_pele_morena_escura:','it':':operaio_edile_uomo_carnagione_abbastanza_scura:','fa':':کارگر_ساختمانی_مرد_پوست_گندمی:','id':':pekerja_konstruksi_pria_warna_kulit_gelap-sedang:','zh':':男建筑工人_中等深肤色:','ru':':мужчина-строитель_темныи_тон_кожи:','tr':':erkek_inşaat_işçisi_orta_koyu_cilt_tonu:','ar':':عامل_بناء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F477\U0001F3FE\U0000200D\U00002642':{'en':':man_construction_worker_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':bauarbeiter_mitteldunkle_hautfarbe:','es':':profesional_de_la_construcción_hombre_tono_de_piel_oscuro_medio:','fr':':ouvrier_du_bâtiment_peau_mate:','ja':':男性の建設作業員_やや濃い肌色:','ko':':건설_현장_남자_노동자_진한_갈색_피부:','pt':':pedreiro_pele_morena_escura:','it':':operaio_edile_uomo_carnagione_abbastanza_scura:','fa':':کارگر_ساختمانی_مرد_پوست_گندمی:','id':':pekerja_konstruksi_pria_warna_kulit_gelap-sedang:','zh':':男建筑工人_中等深肤色:','ru':':мужчина_строитель_темный_тон_кожи:','tr':':erkek_inşaat_işçisi_orta_koyu_cilt_tonu:','ar':':عامل_بناء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F477\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_construction_worker_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':bauarbeiter_mittelhelle_hautfarbe:','es':':profesional_de_la_construcción_hombre_tono_de_piel_claro_medio:','fr':':ouvrier_du_bâtiment_peau_moyennement_claire:','ja':':男性の建設作業員_やや薄い肌色:','ko':':건설_현장_남자_노동자_연한_갈색_피부:','pt':':pedreiro_pele_morena_clara:','it':':operaio_edile_uomo_carnagione_abbastanza_chiara:','fa':':کارگر_ساختمانی_مرد_پوست_روشن:','id':':pekerja_konstruksi_pria_warna_kulit_cerah-sedang:','zh':':男建筑工人_中等浅肤色:','ru':':мужчина-строитель_светлыи_тон_кожи:','tr':':erkek_inşaat_işçisi_orta_açık_cilt_tonu:','ar':':عامل_بناء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F477\U0001F3FC\U0000200D\U00002642':{'en':':man_construction_worker_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':bauarbeiter_mittelhelle_hautfarbe:','es':':profesional_de_la_construcción_hombre_tono_de_piel_claro_medio:','fr':':ouvrier_du_bâtiment_peau_moyennement_claire:','ja':':男性の建設作業員_やや薄い肌色:','ko':':건설_현장_남자_노동자_연한_갈색_피부:','pt':':pedreiro_pele_morena_clara:','it':':operaio_edile_uomo_carnagione_abbastanza_chiara:','fa':':کارگر_ساختمانی_مرد_پوست_روشن:','id':':pekerja_konstruksi_pria_warna_kulit_cerah-sedang:','zh':':男建筑工人_中等浅肤色:','ru':':мужчина_строитель_светлый_тон_кожи:','tr':':erkek_inşaat_işçisi_orta_açık_cilt_tonu:','ar':':عامل_بناء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F477\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_construction_worker_medium_skin_tone:','status':fully_qualified,'E':4,'de':':bauarbeiter_mittlere_hautfarbe:','es':':profesional_de_la_construcción_hombre_tono_de_piel_medio:','fr':':ouvrier_du_bâtiment_peau_légèrement_mate:','ja':':男性の建設作業員_中間の肌色:','ko':':건설_현장_남자_노동자_갈색_피부:','pt':':pedreiro_pele_morena:','it':':operaio_edile_uomo_carnagione_olivastra:','fa':':کارگر_ساختمانی_مرد_پوست_طلایی:','id':':pekerja_konstruksi_pria_warna_kulit_sedang:','zh':':男建筑工人_中等肤色:','ru':':мужчина-строитель_среднии_тон_кожи:','tr':':erkek_inşaat_işçisi_orta_cilt_tonu:','ar':':عامل_بناء_بشرة_بلون_معتدل:'},'\U0001F477\U0001F3FD\U0000200D\U00002642':{'en':':man_construction_worker_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':bauarbeiter_mittlere_hautfarbe:','es':':profesional_de_la_construcción_hombre_tono_de_piel_medio:','fr':':ouvrier_du_bâtiment_peau_légèrement_mate:','ja':':男性の建設作業員_中間の肌色:','ko':':건설_현장_남자_노동자_갈색_피부:','pt':':pedreiro_pele_morena:','it':':operaio_edile_uomo_carnagione_olivastra:','fa':':کارگر_ساختمانی_مرد_پوست_طلایی:','id':':pekerja_konstruksi_pria_warna_kulit_sedang:','zh':':男建筑工人_中等肤色:','ru':':мужчина_строитель_средний_тон_кожи:','tr':':erkek_inşaat_işçisi_orta_cilt_tonu:','ar':':عامل_بناء_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U0001F373':{'en':':man_cook:','status':fully_qualified,'E':4,'de':':koch:','es':':chef_hombre:','fr':':cuisinier:','ja':':男性のコック:','ko':':남자_요리사:','pt':':cozinheiro:','it':':cuoco:','fa':':آشپز_مرد:','id':':koki_pria:','zh':':男厨师:','ru':':мужчина_повар:','tr':':erkek_aşçı:','ar':':طباخ:'},'\U0001F468\U0001F3FF\U0000200D\U0001F373':{'en':':man_cook_dark_skin_tone:','status':fully_qualified,'E':4,'de':':koch_dunkle_hautfarbe:','es':':chef_hombre_tono_de_piel_oscuro:','fr':':cuisinier_peau_foncée:','ja':':男性のコック_濃い肌色:','ko':':남자_요리사_검은색_피부:','pt':':cozinheiro_pele_escura:','it':':cuoco_carnagione_scura:','fa':':آشپز_مرد_پوست_آبنوسی:','id':':koki_pria_warna_kulit_gelap:','zh':':男厨师_较深肤色:','ru':':мужчина_повар_очень_темный_тон_кожи:','tr':':erkek_aşçı_koyu_cilt_tonu:','ar':':طباخ_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U0001F373':{'en':':man_cook_light_skin_tone:','status':fully_qualified,'E':4,'de':':koch_helle_hautfarbe:','es':':chef_hombre_tono_de_piel_claro:','fr':':cuisinier_peau_claire:','ja':':男性のコック_薄い肌色:','ko':':남자_요리사_하얀_피부:','pt':':cozinheiro_pele_clara:','it':':cuoco_carnagione_chiara:','fa':':آشپز_مرد_پوست_سفید:','id':':koki_pria_warna_kulit_cerah:','zh':':男厨师_较浅肤色:','ru':':мужчина_повар_очень_светлый_тон_кожи:','tr':':erkek_aşçı_açık_cilt_tonu:','ar':':طباخ_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F373':{'en':':man_cook_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':koch_mitteldunkle_hautfarbe:','es':':chef_hombre_tono_de_piel_oscuro_medio:','fr':':cuisinier_peau_mate:','ja':':男性のコック_やや濃い肌色:','ko':':남자_요리사_진한_갈색_피부:','pt':':cozinheiro_pele_morena_escura:','it':':cuoco_carnagione_abbastanza_scura:','fa':':آشپز_مرد_پوست_گندمی:','id':':koki_pria_warna_kulit_gelap-sedang:','zh':':男厨师_中等深肤色:','ru':':мужчина_повар_темный_тон_кожи:','tr':':erkek_aşçı_orta_koyu_cilt_tonu:','ar':':طباخ_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U0001F373':{'en':':man_cook_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':koch_mittelhelle_hautfarbe:','es':':chef_hombre_tono_de_piel_claro_medio:','fr':':cuisinier_peau_moyennement_claire:','ja':':男性のコック_やや薄い肌色:','ko':':남자_요리사_연한_갈색_피부:','pt':':cozinheiro_pele_morena_clara:','it':':cuoco_carnagione_abbastanza_chiara:','fa':':آشپز_مرد_پوست_روشن:','id':':koki_pria_warna_kulit_cerah-sedang:','zh':':男厨师_中等浅肤色:','ru':':мужчина_повар_светлый_тон_кожи:','tr':':erkek_aşçı_orta_açık_cilt_tonu:','ar':':طباخ_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F373':{'en':':man_cook_medium_skin_tone:','status':fully_qualified,'E':4,'de':':koch_mittlere_hautfarbe:','es':':chef_hombre_tono_de_piel_medio:','fr':':cuisinier_peau_légèrement_mate:','ja':':男性のコック_中間の肌色:','ko':':남자_요리사_갈색_피부:','pt':':cozinheiro_pele_morena:','it':':cuoco_carnagione_olivastra:','fa':':آشپز_مرد_پوست_طلایی:','id':':koki_pria_warna_kulit_sedang:','zh':':男厨师_中等肤色:','ru':':мужчина_повар_средний_тон_кожи:','tr':':erkek_aşçı_orta_cilt_tonu:','ar':':طباخ_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U0001F9B1':{'en':':man_curly_hair:','status':fully_qualified,'E':11,'alias':[':curly_haired_man:'],'de':':mann_lockiges_haar:','es':':hombre_pelo_rizado:','fr':':homme_cheveux_bouclés:','ja':':男性_巻き毛:','ko':':남자_곱슬머리:','pt':':homem_cabelo_cacheado:','it':':uomo_capelli_ricci:','fa':':مرد_موفرفری:','id':':pria_rambut_keriting:','zh':':男人_卷发:','ru':':мужчина_кудрявые_волосы:','tr':':erkek_kıvırcık_saç:','ar':':رجل_شعر_مجعد:'},'\U0001F57A':{'en':':man_dancing:','status':fully_qualified,'E':3,'de':':tanzender_mann:','es':':hombre_bailando:','fr':':danseur:','ja':':踊る男:','ko':':남자_댄서:','pt':':homem_dançando:','it':':uomo_che_balla:','fa':':مرد_درحال_رقص:','id':':pria_menari:','zh':':跳舞的男人:','ru':':танцующий_мужчина:','tr':':dans_eden_adam:','ar':':رجل_يرقص:'},'\U0001F57A\U0001F3FF':{'en':':man_dancing_dark_skin_tone:','status':fully_qualified,'E':3,'de':':tanzender_mann_dunkle_hautfarbe:','es':':hombre_bailando_tono_de_piel_oscuro:','fr':':danseur_peau_foncée:','ja':':踊る男_濃い肌色:','ko':':남자_댄서_검은색_피부:','pt':':homem_dançando_pele_escura:','it':':uomo_che_balla_carnagione_scura:','fa':':مرد_درحال_رقص_پوست_آبنوسی:','id':':pria_menari_warna_kulit_gelap:','zh':':跳舞的男人_较深肤色:','ru':':танцующий_мужчина_очень_темный_тон_кожи:','tr':':dans_eden_adam_koyu_cilt_tonu:','ar':':رجل_يرقص_بشرة_بلون_غامق:'},'\U0001F57A\U0001F3FB':{'en':':man_dancing_light_skin_tone:','status':fully_qualified,'E':3,'de':':tanzender_mann_helle_hautfarbe:','es':':hombre_bailando_tono_de_piel_claro:','fr':':danseur_peau_claire:','ja':':踊る男_薄い肌色:','ko':':남자_댄서_하얀_피부:','pt':':homem_dançando_pele_clara:','it':':uomo_che_balla_carnagione_chiara:','fa':':مرد_درحال_رقص_پوست_سفید:','id':':pria_menari_warna_kulit_cerah:','zh':':跳舞的男人_较浅肤色:','ru':':танцующий_мужчина_очень_светлый_тон_кожи:','tr':':dans_eden_adam_açık_cilt_tonu:','ar':':رجل_يرقص_بشرة_بلون_فاتح:'},'\U0001F57A\U0001F3FE':{'en':':man_dancing_medium-dark_skin_tone:','status':fully_qualified,'E':3,'de':':tanzender_mann_mitteldunkle_hautfarbe:','es':':hombre_bailando_tono_de_piel_oscuro_medio:','fr':':danseur_peau_mate:','ja':':踊る男_やや濃い肌色:','ko':':남자_댄서_진한_갈색_피부:','pt':':homem_dançando_pele_morena_escura:','it':':uomo_che_balla_carnagione_abbastanza_scura:','fa':':مرد_درحال_رقص_پوست_گندمی:','id':':pria_menari_warna_kulit_gelap-sedang:','zh':':跳舞的男人_中等深肤色:','ru':':танцующий_мужчина_темный_тон_кожи:','tr':':dans_eden_adam_orta_koyu_cilt_tonu:','ar':':رجل_يرقص_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F57A\U0001F3FC':{'en':':man_dancing_medium-light_skin_tone:','status':fully_qualified,'E':3,'de':':tanzender_mann_mittelhelle_hautfarbe:','es':':hombre_bailando_tono_de_piel_claro_medio:','fr':':danseur_peau_moyennement_claire:','ja':':踊る男_やや薄い肌色:','ko':':남자_댄서_연한_갈색_피부:','pt':':homem_dançando_pele_morena_clara:','it':':uomo_che_balla_carnagione_abbastanza_chiara:','fa':':مرد_درحال_رقص_پوست_روشن:','id':':pria_menari_warna_kulit_cerah-sedang:','zh':':跳舞的男人_中等浅肤色:','ru':':танцующий_мужчина_светлый_тон_кожи:','tr':':dans_eden_adam_orta_açık_cilt_tonu:','ar':':رجل_يرقص_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F57A\U0001F3FD':{'en':':man_dancing_medium_skin_tone:','status':fully_qualified,'E':3,'de':':tanzender_mann_mittlere_hautfarbe:','es':':hombre_bailando_tono_de_piel_medio:','fr':':danseur_peau_légèrement_mate:','ja':':踊る男_中間の肌色:','ko':':남자_댄서_갈색_피부:','pt':':homem_dançando_pele_morena:','it':':uomo_che_balla_carnagione_olivastra:','fa':':مرد_درحال_رقص_پوست_طلایی:','id':':pria_menari_warna_kulit_sedang:','zh':':跳舞的男人_中等肤色:','ru':':танцующий_мужчина_средний_тон_кожи:','tr':':dans_eden_adam_orta_cilt_tonu:','ar':':رجل_يرقص_بشرة_بلون_معتدل:'},'\U0001F468\U0001F3FF':{'en':':man_dark_skin_tone:','status':fully_qualified,'E':1,'de':':mann_dunkle_hautfarbe:','es':':hombre_tono_de_piel_oscuro:','fr':':homme_peau_foncée:','ja':':男性_濃い肌色:','ko':':남자_검은색_피부:','pt':':homem_pele_escura:','it':':uomo_carnagione_scura:','fa':':مرد_پوست_آبنوسی:','id':':pria_warna_kulit_gelap:','zh':':男人_较深肤色:','ru':':мужчина_очень_темный_тон_кожи:','tr':':erkek_koyu_cilt_tonu:','ar':':رجل_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FF\U0000200D\U0001F9B2':{'en':':man_dark_skin_tone_bald:','status':fully_qualified,'E':11,'de':':mann_dunkle_hautfarbe_glatze:','es':':hombre_tono_de_piel_oscuro_y_sin_pelo:','fr':':homme_peau_foncée_et_chauve:','ja':':男性_濃い肌色_はげ頭:','ko':':남자_검은색_피부_대머리:','pt':':homem_pele_escura_e_careca:','it':':uomo_carnagione_scura_e_calvo:','fa':':مرد_پوست_آبنوسی_تاس:','id':':pria_warna_kulit_gelap_gundul:','zh':':男人_较深肤色秃顶:','ru':':мужчина_очень_темный_тон_кожи_без_волос:','tr':':erkek_koyu_cilt_tonu_kel:','ar':':رجل_بشرة_بلون_غامق_وأصلع:'},'\U0001F9D4\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_dark_skin_tone_beard:','status':fully_qualified,'E':13.1,'de':':mann_dunkle_hautfarbe_bart:','es':':hombre_tono_de_piel_oscuro_y_barba:','fr':':homme_barbu_peau_foncée:','ja':':あごひげの男性_濃い肌色:','ko':':수염_난_남자_검은색_피부:','pt':':homem_pele_escura_e_barbudo:','it':':uomo_con_la_barba_carnagione_scura:','fa':':مرد_پوست_آبنوسی_ریش:','id':':pria_warna_kulit_gelap_janggut:','zh':':有络腮胡子的男人_较深肤色:','ru':':бородатыи_мужчина_очень_темныи_тон_кожи:','tr':':erkek_koyu_cilt_tonu_sakal:','ar':':رجل_بلحية_بشرة_بلون_غامق:'},'\U0001F9D4\U0001F3FF\U0000200D\U00002642':{'en':':man_dark_skin_tone_beard:','status':minimally_qualified,'E':13.1,'de':':mann_dunkle_hautfarbe_bart:','es':':hombre_tono_de_piel_oscuro_y_barba:','fr':':homme_barbu_peau_foncée:','ja':':あごひげの男性_濃い肌色:','ko':':수염_난_남자_검은색_피부:','pt':':homem_pele_escura_e_barbudo:','it':':uomo_con_la_barba_carnagione_scura:','fa':':مرد_پوست_آبنوسی_ریش:','id':':pria_warna_kulit_gelap_janggut:','zh':':有络腮胡子的男人_较深肤色:','ru':':бородатый_мужчина_очень_темный_тон_кожи:','tr':':erkek_koyu_cilt_tonu_sakal:','ar':':رجل_بلحية_بشرة_بلون_غامق:'},'\U0001F471\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_dark_skin_tone_blond_hair:','status':fully_qualified,'E':4,'de':':mann_dunkle_hautfarbe,_blond:','es':':hombre_rubio_tono_de_piel_oscuro:','fr':':homme_blond_peau_foncée:','ja':':金髪の男性_濃い肌色:','ko':':금발_남자_검은색_피부:','pt':':homem_pele_escura_e_cabelo_loiro:','it':':uomo_biondo_carnagione_scura:','fa':':مرد_پوست_آبنوسی_مو_بور:','id':':pria_berambut_pirang_warna_kulit_gelap:','zh':':金发男_较深肤色:','ru':':блондин_очень_темныи_тон_кожи:','tr':':sarışın_erkek_koyu_cilt_tonu:','ar':':رجل_أشقر_بشرة_بلون_غامق:'},'\U0001F471\U0001F3FF\U0000200D\U00002642':{'en':':man_dark_skin_tone_blond_hair:','status':minimally_qualified,'E':4,'de':':mann_dunkle_hautfarbe_blond:','es':':hombre_rubio_tono_de_piel_oscuro:','fr':':homme_blond_peau_foncée:','ja':':金髪の男性_濃い肌色:','ko':':금발_남자_검은색_피부:','pt':':homem_pele_escura_e_cabelo_loiro:','it':':uomo_biondo_carnagione_scura:','fa':':مرد_پوست_آبنوسی_مو_بور:','id':':pria_berambut_pirang_warna_kulit_gelap:','zh':':金发男_较深肤色:','ru':':блондин_очень_темный_тон_кожи:','tr':':sarışın_erkek_koyu_cilt_tonu:','ar':':رجل_أشقر_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FF\U0000200D\U0001F9B1':{'en':':man_dark_skin_tone_curly_hair:','status':fully_qualified,'E':11,'de':':mann_dunkle_hautfarbe_lockiges_haar:','es':':hombre_tono_de_piel_oscuro_y_pelo_rizado:','fr':':homme_peau_foncée_et_cheveux_bouclés:','ja':':男性_濃い肌色_巻き毛:','ko':':남자_검은색_피부_곱슬머리:','pt':':homem_pele_escura_e_cabelo_cacheado:','it':':uomo_carnagione_scura_e_capelli_ricci:','fa':':مرد_پوست_آبنوسی_موفرفری:','id':':pria_warna_kulit_gelap_rambut_keriting:','zh':':男人_较深肤色卷发:','ru':':мужчина_очень_темный_тон_кожи_кудрявые_волосы:','tr':':erkek_koyu_cilt_tonu_kıvırcık_saç:','ar':':رجل_بشرة_بلون_غامق_وشعر_مجعد:'},'\U0001F468\U0001F3FF\U0000200D\U0001F9B0':{'en':':man_dark_skin_tone_red_hair:','status':fully_qualified,'E':11,'de':':mann_dunkle_hautfarbe_rotes_haar:','es':':hombre_tono_de_piel_oscuro_y_pelo_pelirrojo:','fr':':homme_peau_foncée_et_cheveux_roux:','ja':':男性_濃い肌色_赤毛:','ko':':남자_검은색_피부_빨간_머리:','pt':':homem_pele_escura_e_cabelo_vermelho:','it':':uomo_carnagione_scura_e_capelli_rossi:','fa':':مرد_پوست_آبنوسی_موقرمز:','id':':pria_warna_kulit_gelap_rambut_merah:','zh':':男人_较深肤色红发:','ru':':мужчина_очень_темный_тон_кожи_рыжие_волосы:','tr':':erkek_koyu_cilt_tonu_kızıl_saç:','ar':':رجل_بشرة_بلون_غامق_وشعر_أحمر:'},'\U0001F468\U0001F3FF\U0000200D\U0001F9B3':{'en':':man_dark_skin_tone_white_hair:','status':fully_qualified,'E':11,'de':':mann_dunkle_hautfarbe_weißes_haar:','es':':hombre_tono_de_piel_oscuro_y_pelo_blanco:','fr':':homme_peau_foncée_et_cheveux_blancs:','ja':':男性_濃い肌色_白髪:','ko':':남자_검은색_피부_백발:','pt':':homem_pele_escura_e_cabelo_branco:','it':':uomo_carnagione_scura_e_capelli_bianchi:','fa':':مرد_پوست_آبنوسی_موسفید:','id':':pria_warna_kulit_gelap_rambut_putih:','zh':':男人_较深肤色白发:','ru':':мужчина_очень_темный_тон_кожи_седые_волосы:','tr':':erkek_koyu_cilt_tonu_beyaz_saç:','ar':':رجل_بشرة_بلون_غامق_وشعر_أبيض:'},'\U0001F575\U0000FE0F\U0000200D\U00002642\U0000FE0F':{'en':':man_detective:','status':fully_qualified,'E':4,'alias':[':male_detective:'],'de':':detektiv:','es':':detective_hombre:','fr':':détective_homme:','ja':':男性の探偵:','ko':':남자_탐정:','pt':':detetive_homem:','it':':investigatore:','fa':':کارآگاه_مرد:','id':':detektif_pria:','zh':':男侦探:','ru':':мужчина-детектив:','tr':':erkek_dedektif:','ar':':محقق:'},'\U0001F575\U0000200D\U00002642\U0000FE0F':{'en':':man_detective:','status':unqualified,'E':4,'alias':[':male_detective:'],'de':':detektiv:','es':':detective_hombre:','fr':':détective_homme:','ja':':男性の探偵:','ko':':남자_탐정:','pt':':detetive_homem:','it':':investigatore:','fa':':کارآگاه_مرد:','id':':detektif_pria:','zh':':男侦探:','ru':':мужчина-детектив:','tr':':erkek_dedektif:','ar':':محقق:'},'\U0001F575\U0000FE0F\U0000200D\U00002642':{'en':':man_detective:','status':minimally_qualified,'E':4,'alias':[':male_detective:'],'de':':detektiv:','es':':detective_hombre:','fr':':détective_homme:','ja':':男性の探偵:','ko':':남자_탐정:','pt':':detetive_homem:','it':':investigatore:','fa':':کارآگاه_مرد:','id':':detektif_pria:','zh':':男侦探:','ru':':мужчина-детектив:','tr':':erkek_dedektif:','ar':':محقق:'},'\U0001F575\U0000200D\U00002642':{'en':':man_detective:','status':unqualified,'E':4,'alias':[':male_detective:'],'de':':detektiv:','es':':detective_hombre:','fr':':détective_homme:','ja':':男性の探偵:','ko':':남자_탐정:','pt':':detetive_homem:','it':':investigatore:','fa':':کارآگاه_مرد:','id':':detektif_pria:','zh':':男侦探:','ru':':мужчина_детектив:','tr':':erkek_dedektif:','ar':':محقق:'},'\U0001F575\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_detective_dark_skin_tone:','status':fully_qualified,'E':4,'de':':detektiv_dunkle_hautfarbe:','es':':detective_hombre_tono_de_piel_oscuro:','fr':':détective_homme_peau_foncée:','ja':':男性の探偵_濃い肌色:','ko':':남자_탐정_검은색_피부:','pt':':detetive_homem_pele_escura:','it':':investigatore_carnagione_scura:','fa':':کارآگاه_مرد_پوست_آبنوسی:','id':':detektif_pria_warna_kulit_gelap:','zh':':男侦探_较深肤色:','ru':':мужчина-детектив_очень_темныи_тон_кожи:','tr':':erkek_dedektif_koyu_cilt_tonu:','ar':':محقق_بشرة_بلون_غامق:'},'\U0001F575\U0001F3FF\U0000200D\U00002642':{'en':':man_detective_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':detektiv_dunkle_hautfarbe:','es':':detective_hombre_tono_de_piel_oscuro:','fr':':détective_homme_peau_foncée:','ja':':男性の探偵_濃い肌色:','ko':':남자_탐정_검은색_피부:','pt':':detetive_homem_pele_escura:','it':':investigatore_carnagione_scura:','fa':':کارآگاه_مرد_پوست_آبنوسی:','id':':detektif_pria_warna_kulit_gelap:','zh':':男侦探_较深肤色:','ru':':мужчина_детектив_очень_темный_тон_кожи:','tr':':erkek_dedektif_koyu_cilt_tonu:','ar':':محقق_بشرة_بلون_غامق:'},'\U0001F575\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_detective_light_skin_tone:','status':fully_qualified,'E':4,'de':':detektiv_helle_hautfarbe:','es':':detective_hombre_tono_de_piel_claro:','fr':':détective_homme_peau_claire:','ja':':男性の探偵_薄い肌色:','ko':':남자_탐정_하얀_피부:','pt':':detetive_homem_pele_clara:','it':':investigatore_carnagione_chiara:','fa':':کارآگاه_مرد_پوست_سفید:','id':':detektif_pria_warna_kulit_cerah:','zh':':男侦探_较浅肤色:','ru':':мужчина-детектив_очень_светлыи_тон_кожи:','tr':':erkek_dedektif_açık_cilt_tonu:','ar':':محقق_بشرة_بلون_فاتح:'},'\U0001F575\U0001F3FB\U0000200D\U00002642':{'en':':man_detective_light_skin_tone:','status':minimally_qualified,'E':4,'de':':detektiv_helle_hautfarbe:','es':':detective_hombre_tono_de_piel_claro:','fr':':détective_homme_peau_claire:','ja':':男性の探偵_薄い肌色:','ko':':남자_탐정_하얀_피부:','pt':':detetive_homem_pele_clara:','it':':investigatore_carnagione_chiara:','fa':':کارآگاه_مرد_پوست_سفید:','id':':detektif_pria_warna_kulit_cerah:','zh':':男侦探_较浅肤色:','ru':':мужчина_детектив_очень_светлый_тон_кожи:','tr':':erkek_dedektif_açık_cilt_tonu:','ar':':محقق_بشرة_بلون_فاتح:'},'\U0001F575\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_detective_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':detektiv_mitteldunkle_hautfarbe:','es':':detective_hombre_tono_de_piel_oscuro_medio:','fr':':détective_homme_peau_mate:','ja':':男性の探偵_やや濃い肌色:','ko':':남자_탐정_진한_갈색_피부:','pt':':detetive_homem_pele_morena_escura:','it':':investigatore_carnagione_abbastanza_scura:','fa':':کارآگاه_مرد_پوست_گندمی:','id':':detektif_pria_warna_kulit_gelap-sedang:','zh':':男侦探_中等深肤色:','ru':':мужчина-детектив_темныи_тон_кожи:','tr':':erkek_dedektif_orta_koyu_cilt_tonu:','ar':':محقق_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F575\U0001F3FE\U0000200D\U00002642':{'en':':man_detective_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':detektiv_mitteldunkle_hautfarbe:','es':':detective_hombre_tono_de_piel_oscuro_medio:','fr':':détective_homme_peau_mate:','ja':':男性の探偵_やや濃い肌色:','ko':':남자_탐정_진한_갈색_피부:','pt':':detetive_homem_pele_morena_escura:','it':':investigatore_carnagione_abbastanza_scura:','fa':':کارآگاه_مرد_پوست_گندمی:','id':':detektif_pria_warna_kulit_gelap-sedang:','zh':':男侦探_中等深肤色:','ru':':мужчина_детектив_темный_тон_кожи:','tr':':erkek_dedektif_orta_koyu_cilt_tonu:','ar':':محقق_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F575\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_detective_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':detektiv_mittelhelle_hautfarbe:','es':':detective_hombre_tono_de_piel_claro_medio:','fr':':détective_homme_peau_moyennement_claire:','ja':':男性の探偵_やや薄い肌色:','ko':':남자_탐정_연한_갈색_피부:','pt':':detetive_homem_pele_morena_clara:','it':':investigatore_carnagione_abbastanza_chiara:','fa':':کارآگاه_مرد_پوست_روشن:','id':':detektif_pria_warna_kulit_cerah-sedang:','zh':':男侦探_中等浅肤色:','ru':':мужчина-детектив_светлыи_тон_кожи:','tr':':erkek_dedektif_orta_açık_cilt_tonu:','ar':':محقق_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F575\U0001F3FC\U0000200D\U00002642':{'en':':man_detective_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':detektiv_mittelhelle_hautfarbe:','es':':detective_hombre_tono_de_piel_claro_medio:','fr':':détective_homme_peau_moyennement_claire:','ja':':男性の探偵_やや薄い肌色:','ko':':남자_탐정_연한_갈색_피부:','pt':':detetive_homem_pele_morena_clara:','it':':investigatore_carnagione_abbastanza_chiara:','fa':':کارآگاه_مرد_پوست_روشن:','id':':detektif_pria_warna_kulit_cerah-sedang:','zh':':男侦探_中等浅肤色:','ru':':мужчина_детектив_светлый_тон_кожи:','tr':':erkek_dedektif_orta_açık_cilt_tonu:','ar':':محقق_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F575\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_detective_medium_skin_tone:','status':fully_qualified,'E':4,'de':':detektiv_mittlere_hautfarbe:','es':':detective_hombre_tono_de_piel_medio:','fr':':détective_homme_peau_légèrement_mate:','ja':':男性の探偵_中間の肌色:','ko':':남자_탐정_갈색_피부:','pt':':detetive_homem_pele_morena:','it':':investigatore_carnagione_olivastra:','fa':':کارآگاه_مرد_پوست_طلایی:','id':':detektif_pria_warna_kulit_sedang:','zh':':男侦探_中等肤色:','ru':':мужчина-детектив_среднии_тон_кожи:','tr':':erkek_dedektif_orta_cilt_tonu:','ar':':محقق_بشرة_بلون_معتدل:'},'\U0001F575\U0001F3FD\U0000200D\U00002642':{'en':':man_detective_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':detektiv_mittlere_hautfarbe:','es':':detective_hombre_tono_de_piel_medio:','fr':':détective_homme_peau_légèrement_mate:','ja':':男性の探偵_中間の肌色:','ko':':남자_탐정_갈색_피부:','pt':':detetive_homem_pele_morena:','it':':investigatore_carnagione_olivastra:','fa':':کارآگاه_مرد_پوست_طلایی:','id':':detektif_pria_warna_kulit_sedang:','zh':':男侦探_中等肤色:','ru':':мужчина_детектив_средний_тон_кожи:','tr':':erkek_dedektif_orta_cilt_tonu:','ar':':محقق_بشرة_بلون_معتدل:'},'\U0001F9DD\U0000200D\U00002642\U0000FE0F':{'en':':man_elf:','status':fully_qualified,'E':5,'alias':[':elf_man:'],'de':':elf:','es':':elfo_hombre:','fr':':elfe_homme:','ja':':男のエルフ:','ko':':남자_엘프:','pt':':elfo_homem:','it':':elfo_uomo:','fa':':اِلف_مرد:','id':':kurcaci_pria:','zh':':男小精灵:','ru':':мужчина-эльф:','tr':':erkek_elf:','ar':':جني_صغير_ذكر:'},'\U0001F9DD\U0000200D\U00002642':{'en':':man_elf:','status':minimally_qualified,'E':5,'alias':[':elf_man:'],'de':':elf:','es':':elfo_hombre:','fr':':elfe_homme:','ja':':男のエルフ:','ko':':남자_엘프:','pt':':elfo_homem:','it':':elfo_uomo:','fa':':اِلف_مرد:','id':':kurcaci_pria:','zh':':男小精灵:','ru':':мужчина_эльф:','tr':':erkek_elf:','ar':':جني_صغير_ذكر:'},'\U0001F9DD\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_elf_dark_skin_tone:','status':fully_qualified,'E':5,'de':':elf_dunkle_hautfarbe:','es':':elfo_hombre_tono_de_piel_oscuro:','fr':':elfe_homme_peau_foncée:','ja':':男のエルフ_濃い肌色:','ko':':남자_엘프_검은색_피부:','pt':':elfo_homem_pele_escura:','it':':elfo_uomo_carnagione_scura:','fa':':اِلف_مرد_پوست_آبنوسی:','id':':kurcaci_pria_warna_kulit_gelap:','zh':':男小精灵_较深肤色:','ru':':мужчина-эльф_очень_темныи_тон_кожи:','tr':':erkek_elf_koyu_cilt_tonu:','ar':':جني_صغير_ذكر_بشرة_بلون_غامق:'},'\U0001F9DD\U0001F3FF\U0000200D\U00002642':{'en':':man_elf_dark_skin_tone:','status':minimally_qualified,'E':5,'de':':elf_dunkle_hautfarbe:','es':':elfo_hombre_tono_de_piel_oscuro:','fr':':elfe_homme_peau_foncée:','ja':':男のエルフ_濃い肌色:','ko':':남자_엘프_검은색_피부:','pt':':elfo_homem_pele_escura:','it':':elfo_uomo_carnagione_scura:','fa':':اِلف_مرد_پوست_آبنوسی:','id':':kurcaci_pria_warna_kulit_gelap:','zh':':男小精灵_较深肤色:','ru':':мужчина_эльф_очень_темный_тон_кожи:','tr':':erkek_elf_koyu_cilt_tonu:','ar':':جني_صغير_ذكر_بشرة_بلون_غامق:'},'\U0001F9DD\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_elf_light_skin_tone:','status':fully_qualified,'E':5,'de':':elf_helle_hautfarbe:','es':':elfo_hombre_tono_de_piel_claro:','fr':':elfe_homme_peau_claire:','ja':':男のエルフ_薄い肌色:','ko':':남자_엘프_하얀_피부:','pt':':elfo_homem_pele_clara:','it':':elfo_uomo_carnagione_chiara:','fa':':اِلف_مرد_پوست_سفید:','id':':kurcaci_pria_warna_kulit_cerah:','zh':':男小精灵_较浅肤色:','ru':':мужчина-эльф_очень_светлыи_тон_кожи:','tr':':erkek_elf_açık_cilt_tonu:','ar':':جني_صغير_ذكر_بشرة_بلون_فاتح:'},'\U0001F9DD\U0001F3FB\U0000200D\U00002642':{'en':':man_elf_light_skin_tone:','status':minimally_qualified,'E':5,'de':':elf_helle_hautfarbe:','es':':elfo_hombre_tono_de_piel_claro:','fr':':elfe_homme_peau_claire:','ja':':男のエルフ_薄い肌色:','ko':':남자_엘프_하얀_피부:','pt':':elfo_homem_pele_clara:','it':':elfo_uomo_carnagione_chiara:','fa':':اِلف_مرد_پوست_سفید:','id':':kurcaci_pria_warna_kulit_cerah:','zh':':男小精灵_较浅肤色:','ru':':мужчина_эльф_очень_светлый_тон_кожи:','tr':':erkek_elf_açık_cilt_tonu:','ar':':جني_صغير_ذكر_بشرة_بلون_فاتح:'},'\U0001F9DD\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_elf_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':elf_mitteldunkle_hautfarbe:','es':':elfo_hombre_tono_de_piel_oscuro_medio:','fr':':elfe_homme_peau_mate:','ja':':男のエルフ_やや濃い肌色:','ko':':남자_엘프_진한_갈색_피부:','pt':':elfo_homem_pele_morena_escura:','it':':elfo_uomo_carnagione_abbastanza_scura:','fa':':اِلف_مرد_پوست_گندمی:','id':':kurcaci_pria_warna_kulit_gelap-sedang:','zh':':男小精灵_中等深肤色:','ru':':мужчина-эльф_темныи_тон_кожи:','tr':':erkek_elf_orta_koyu_cilt_tonu:','ar':':جني_صغير_ذكر_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9DD\U0001F3FE\U0000200D\U00002642':{'en':':man_elf_medium-dark_skin_tone:','status':minimally_qualified,'E':5,'de':':elf_mitteldunkle_hautfarbe:','es':':elfo_hombre_tono_de_piel_oscuro_medio:','fr':':elfe_homme_peau_mate:','ja':':男のエルフ_やや濃い肌色:','ko':':남자_엘프_진한_갈색_피부:','pt':':elfo_homem_pele_morena_escura:','it':':elfo_uomo_carnagione_abbastanza_scura:','fa':':اِلف_مرد_پوست_گندمی:','id':':kurcaci_pria_warna_kulit_gelap-sedang:','zh':':男小精灵_中等深肤色:','ru':':мужчина_эльф_темный_тон_кожи:','tr':':erkek_elf_orta_koyu_cilt_tonu:','ar':':جني_صغير_ذكر_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9DD\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_elf_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':elf_mittelhelle_hautfarbe:','es':':elfo_hombre_tono_de_piel_claro_medio:','fr':':elfe_homme_peau_moyennement_claire:','ja':':男のエルフ_やや薄い肌色:','ko':':남자_엘프_연한_갈색_피부:','pt':':elfo_homem_pele_morena_clara:','it':':elfo_uomo_carnagione_abbastanza_chiara:','fa':':اِلف_مرد_پوست_روشن:','id':':kurcaci_pria_warna_kulit_cerah-sedang:','zh':':男小精灵_中等浅肤色:','ru':':мужчина-эльф_светлыи_тон_кожи:','tr':':erkek_elf_orta_açık_cilt_tonu:','ar':':جني_صغير_ذكر_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9DD\U0001F3FC\U0000200D\U00002642':{'en':':man_elf_medium-light_skin_tone:','status':minimally_qualified,'E':5,'de':':elf_mittelhelle_hautfarbe:','es':':elfo_hombre_tono_de_piel_claro_medio:','fr':':elfe_homme_peau_moyennement_claire:','ja':':男のエルフ_やや薄い肌色:','ko':':남자_엘프_연한_갈색_피부:','pt':':elfo_homem_pele_morena_clara:','it':':elfo_uomo_carnagione_abbastanza_chiara:','fa':':اِلف_مرد_پوست_روشن:','id':':kurcaci_pria_warna_kulit_cerah-sedang:','zh':':男小精灵_中等浅肤色:','ru':':мужчина_эльф_светлый_тон_кожи:','tr':':erkek_elf_orta_açık_cilt_tonu:','ar':':جني_صغير_ذكر_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9DD\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_elf_medium_skin_tone:','status':fully_qualified,'E':5,'de':':elf_mittlere_hautfarbe:','es':':elfo_hombre_tono_de_piel_medio:','fr':':elfe_homme_peau_légèrement_mate:','ja':':男のエルフ_中間の肌色:','ko':':남자_엘프_갈색_피부:','pt':':elfo_homem_pele_morena:','it':':elfo_uomo_carnagione_olivastra:','fa':':اِلف_مرد_پوست_طلایی:','id':':kurcaci_pria_warna_kulit_sedang:','zh':':男小精灵_中等肤色:','ru':':мужчина-эльф_среднии_тон_кожи:','tr':':erkek_elf_orta_cilt_tonu:','ar':':جني_صغير_ذكر_بشرة_بلون_معتدل:'},'\U0001F9DD\U0001F3FD\U0000200D\U00002642':{'en':':man_elf_medium_skin_tone:','status':minimally_qualified,'E':5,'de':':elf_mittlere_hautfarbe:','es':':elfo_hombre_tono_de_piel_medio:','fr':':elfe_homme_peau_légèrement_mate:','ja':':男のエルフ_中間の肌色:','ko':':남자_엘프_갈색_피부:','pt':':elfo_homem_pele_morena:','it':':elfo_uomo_carnagione_olivastra:','fa':':اِلف_مرد_پوست_طلایی:','id':':kurcaci_pria_warna_kulit_sedang:','zh':':男小精灵_中等肤色:','ru':':мужчина_эльф_средний_тон_кожи:','tr':':erkek_elf_orta_cilt_tonu:','ar':':جني_صغير_ذكر_بشرة_بلون_معتدل:'},'\U0001F926\U0000200D\U00002642\U0000FE0F':{'en':':man_facepalming:','status':fully_qualified,'E':4,'de':':sich_an_den_kopf_fassender_mann:','es':':hombre_con_la_mano_en_la_frente:','fr':':homme_avec_la_paume_sur_le_visage:','ja':':ひたいに手をあてる男:','ko':':골치_아파하는_남자:','pt':':homem_decepcionado:','it':':uomo_esasperato:','fa':':مرد_با_کف_دست_به_پیشانی_می_زند:','id':':pria_tepuk_jidat:','zh':':男生捂脸:','ru':':мужчина_потерял_дар_речи:','tr':':eliyle_yüzünü_kapatan_erkek:','ar':':رجل_يشعر_بإحباط:'},'\U0001F926\U0000200D\U00002642':{'en':':man_facepalming:','status':minimally_qualified,'E':4,'de':':sich_an_den_kopf_fassender_mann:','es':':hombre_con_la_mano_en_la_frente:','fr':':homme_avec_la_paume_sur_le_visage:','ja':':ひたいに手をあてる男:','ko':':골치_아파하는_남자:','pt':':homem_decepcionado:','it':':uomo_esasperato:','fa':':مرد_با_کف_دست_به_پیشانی_می_زند:','id':':pria_tepuk_jidat:','zh':':男生捂脸:','ru':':мужчина_потерял_дар_речи:','tr':':eliyle_yüzünü_kapatan_erkek:','ar':':رجل_يشعر_بإحباط:'},'\U0001F926\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_facepalming_dark_skin_tone:','status':fully_qualified,'E':4,'de':':sich_an_den_kopf_fassender_mann_dunkle_hautfarbe:','es':':hombre_con_la_mano_en_la_frente_tono_de_piel_oscuro:','fr':':homme_avec_la_paume_sur_le_visage_peau_foncée:','ja':':ひたいに手をあてる男_濃い肌色:','ko':':골치_아파하는_남자_검은색_피부:','pt':':homem_decepcionado_pele_escura:','it':':uomo_esasperato_carnagione_scura:','fa':':مرد_با_کف_دست_به_پیشانی_می_زند_پوست_آبنوسی:','id':':pria_tepuk_jidat_warna_kulit_gelap:','zh':':男生捂脸_较深肤色:','ru':':мужчина_потерял_дар_речи_очень_темныи_тон_кожи:','tr':':eliyle_yüzünü_kapatan_erkek_koyu_cilt_tonu:','ar':':رجل_يشعر_بإحباط_بشرة_بلون_غامق:'},'\U0001F926\U0001F3FF\U0000200D\U00002642':{'en':':man_facepalming_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':sich_an_den_kopf_fassender_mann_dunkle_hautfarbe:','es':':hombre_con_la_mano_en_la_frente_tono_de_piel_oscuro:','fr':':homme_avec_la_paume_sur_le_visage_peau_foncée:','ja':':ひたいに手をあてる男_濃い肌色:','ko':':골치_아파하는_남자_검은색_피부:','pt':':homem_decepcionado_pele_escura:','it':':uomo_esasperato_carnagione_scura:','fa':':مرد_با_کف_دست_به_پیشانی_می_زند_پوست_آبنوسی:','id':':pria_tepuk_jidat_warna_kulit_gelap:','zh':':男生捂脸_较深肤色:','ru':':мужчина_потерял_дар_речи_очень_темный_тон_кожи:','tr':':eliyle_yüzünü_kapatan_erkek_koyu_cilt_tonu:','ar':':رجل_يشعر_بإحباط_بشرة_بلون_غامق:'},'\U0001F926\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_facepalming_light_skin_tone:','status':fully_qualified,'E':4,'de':':sich_an_den_kopf_fassender_mann_helle_hautfarbe:','es':':hombre_con_la_mano_en_la_frente_tono_de_piel_claro:','fr':':homme_avec_la_paume_sur_le_visage_peau_claire:','ja':':ひたいに手をあてる男_薄い肌色:','ko':':골치_아파하는_남자_하얀_피부:','pt':':homem_decepcionado_pele_clara:','it':':uomo_esasperato_carnagione_chiara:','fa':':مرد_با_کف_دست_به_پیشانی_می_زند_پوست_سفید:','id':':pria_tepuk_jidat_warna_kulit_cerah:','zh':':男生捂脸_较浅肤色:','ru':':мужчина_потерял_дар_речи_очень_светлыи_тон_кожи:','tr':':eliyle_yüzünü_kapatan_erkek_açık_cilt_tonu:','ar':':رجل_يشعر_بإحباط_بشرة_بلون_فاتح:'},'\U0001F926\U0001F3FB\U0000200D\U00002642':{'en':':man_facepalming_light_skin_tone:','status':minimally_qualified,'E':4,'de':':sich_an_den_kopf_fassender_mann_helle_hautfarbe:','es':':hombre_con_la_mano_en_la_frente_tono_de_piel_claro:','fr':':homme_avec_la_paume_sur_le_visage_peau_claire:','ja':':ひたいに手をあてる男_薄い肌色:','ko':':골치_아파하는_남자_하얀_피부:','pt':':homem_decepcionado_pele_clara:','it':':uomo_esasperato_carnagione_chiara:','fa':':مرد_با_کف_دست_به_پیشانی_می_زند_پوست_سفید:','id':':pria_tepuk_jidat_warna_kulit_cerah:','zh':':男生捂脸_较浅肤色:','ru':':мужчина_потерял_дар_речи_очень_светлый_тон_кожи:','tr':':eliyle_yüzünü_kapatan_erkek_açık_cilt_tonu:','ar':':رجل_يشعر_بإحباط_بشرة_بلون_فاتح:'},'\U0001F926\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_facepalming_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':sich_an_den_kopf_fassender_mann_mitteldunkle_hautfarbe:','es':':hombre_con_la_mano_en_la_frente_tono_de_piel_oscuro_medio:','fr':':homme_avec_la_paume_sur_le_visage_peau_mate:','ja':':ひたいに手をあてる男_やや濃い肌色:','ko':':골치_아파하는_남자_진한_갈색_피부:','pt':':homem_decepcionado_pele_morena_escura:','it':':uomo_esasperato_carnagione_abbastanza_scura:','fa':':مرد_با_کف_دست_به_پیشانی_می_زند_پوست_گندمی:','id':':pria_tepuk_jidat_warna_kulit_gelap-sedang:','zh':':男生捂脸_中等深肤色:','ru':':мужчина_потерял_дар_речи_темныи_тон_кожи:','tr':':eliyle_yüzünü_kapatan_erkek_orta_koyu_cilt_tonu:','ar':':رجل_يشعر_بإحباط_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F926\U0001F3FE\U0000200D\U00002642':{'en':':man_facepalming_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':sich_an_den_kopf_fassender_mann_mitteldunkle_hautfarbe:','es':':hombre_con_la_mano_en_la_frente_tono_de_piel_oscuro_medio:','fr':':homme_avec_la_paume_sur_le_visage_peau_mate:','ja':':ひたいに手をあてる男_やや濃い肌色:','ko':':골치_아파하는_남자_진한_갈색_피부:','pt':':homem_decepcionado_pele_morena_escura:','it':':uomo_esasperato_carnagione_abbastanza_scura:','fa':':مرد_با_کف_دست_به_پیشانی_می_زند_پوست_گندمی:','id':':pria_tepuk_jidat_warna_kulit_gelap-sedang:','zh':':男生捂脸_中等深肤色:','ru':':мужчина_потерял_дар_речи_темный_тон_кожи:','tr':':eliyle_yüzünü_kapatan_erkek_orta_koyu_cilt_tonu:','ar':':رجل_يشعر_بإحباط_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F926\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_facepalming_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':sich_an_den_kopf_fassender_mann_mittelhelle_hautfarbe:','es':':hombre_con_la_mano_en_la_frente_tono_de_piel_claro_medio:','fr':':homme_avec_la_paume_sur_le_visage_peau_moyennement_claire:','ja':':ひたいに手をあてる男_やや薄い肌色:','ko':':골치_아파하는_남자_연한_갈색_피부:','pt':':homem_decepcionado_pele_morena_clara:','it':':uomo_esasperato_carnagione_abbastanza_chiara:','fa':':مرد_با_کف_دست_به_پیشانی_می_زند_پوست_روشن:','id':':pria_tepuk_jidat_warna_kulit_cerah-sedang:','zh':':男生捂脸_中等浅肤色:','ru':':мужчина_потерял_дар_речи_светлыи_тон_кожи:','tr':':eliyle_yüzünü_kapatan_erkek_orta_açık_cilt_tonu:','ar':':رجل_يشعر_بإحباط_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F926\U0001F3FC\U0000200D\U00002642':{'en':':man_facepalming_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':sich_an_den_kopf_fassender_mann_mittelhelle_hautfarbe:','es':':hombre_con_la_mano_en_la_frente_tono_de_piel_claro_medio:','fr':':homme_avec_la_paume_sur_le_visage_peau_moyennement_claire:','ja':':ひたいに手をあてる男_やや薄い肌色:','ko':':골치_아파하는_남자_연한_갈색_피부:','pt':':homem_decepcionado_pele_morena_clara:','it':':uomo_esasperato_carnagione_abbastanza_chiara:','fa':':مرد_با_کف_دست_به_پیشانی_می_زند_پوست_روشن:','id':':pria_tepuk_jidat_warna_kulit_cerah-sedang:','zh':':男生捂脸_中等浅肤色:','ru':':мужчина_потерял_дар_речи_светлый_тон_кожи:','tr':':eliyle_yüzünü_kapatan_erkek_orta_açık_cilt_tonu:','ar':':رجل_يشعر_بإحباط_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F926\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_facepalming_medium_skin_tone:','status':fully_qualified,'E':4,'de':':sich_an_den_kopf_fassender_mann_mittlere_hautfarbe:','es':':hombre_con_la_mano_en_la_frente_tono_de_piel_medio:','fr':':homme_avec_la_paume_sur_le_visage_peau_légèrement_mate:','ja':':ひたいに手をあてる男_中間の肌色:','ko':':골치_아파하는_남자_갈색_피부:','pt':':homem_decepcionado_pele_morena:','it':':uomo_esasperato_carnagione_olivastra:','fa':':مرد_با_کف_دست_به_پیشانی_می_زند_پوست_طلایی:','id':':pria_tepuk_jidat_warna_kulit_sedang:','zh':':男生捂脸_中等肤色:','ru':':мужчина_потерял_дар_речи_среднии_тон_кожи:','tr':':eliyle_yüzünü_kapatan_erkek_orta_cilt_tonu:','ar':':رجل_يشعر_بإحباط_بشرة_بلون_معتدل:'},'\U0001F926\U0001F3FD\U0000200D\U00002642':{'en':':man_facepalming_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':sich_an_den_kopf_fassender_mann_mittlere_hautfarbe:','es':':hombre_con_la_mano_en_la_frente_tono_de_piel_medio:','fr':':homme_avec_la_paume_sur_le_visage_peau_légèrement_mate:','ja':':ひたいに手をあてる男_中間の肌色:','ko':':골치_아파하는_남자_갈색_피부:','pt':':homem_decepcionado_pele_morena:','it':':uomo_esasperato_carnagione_olivastra:','fa':':مرد_با_کف_دست_به_پیشانی_می_زند_پوست_طلایی:','id':':pria_tepuk_jidat_warna_kulit_sedang:','zh':':男生捂脸_中等肤色:','ru':':мужчина_потерял_дар_речи_средний_тон_кожи:','tr':':eliyle_yüzünü_kapatan_erkek_orta_cilt_tonu:','ar':':رجل_يشعر_بإحباط_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U0001F3ED':{'en':':man_factory_worker:','status':fully_qualified,'E':4,'de':':fabrikarbeiter:','es':':profesional_industrial_hombre:','fr':':ouvrier:','ja':':男性の溶接工:','ko':':공장_남자_직원:','pt':':operário:','it':':operaio:','fa':':کارگر_کارخانه_مرد:','id':':pekerja_pabrik_pria:','zh':':男工人:','ru':':мужчина_рабочий:','tr':':erkek_fabrika_işçisi:','ar':':عامل_مصنع:'},'\U0001F468\U0001F3FF\U0000200D\U0001F3ED':{'en':':man_factory_worker_dark_skin_tone:','status':fully_qualified,'E':4,'de':':fabrikarbeiter_dunkle_hautfarbe:','es':':profesional_industrial_hombre_tono_de_piel_oscuro:','fr':':ouvrier_peau_foncée:','ja':':男性の溶接工_濃い肌色:','ko':':공장_남자_직원_검은색_피부:','pt':':operário_pele_escura:','it':':operaio_carnagione_scura:','fa':':کارگر_کارخانه_مرد_پوست_آبنوسی:','id':':pekerja_pabrik_pria_warna_kulit_gelap:','zh':':男工人_较深肤色:','ru':':мужчина_рабочий_очень_темный_тон_кожи:','tr':':erkek_fabrika_işçisi_koyu_cilt_tonu:','ar':':عامل_مصنع_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U0001F3ED':{'en':':man_factory_worker_light_skin_tone:','status':fully_qualified,'E':4,'de':':fabrikarbeiter_helle_hautfarbe:','es':':profesional_industrial_hombre_tono_de_piel_claro:','fr':':ouvrier_peau_claire:','ja':':男性の溶接工_薄い肌色:','ko':':공장_남자_직원_하얀_피부:','pt':':operário_pele_clara:','it':':operaio_carnagione_chiara:','fa':':کارگر_کارخانه_مرد_پوست_سفید:','id':':pekerja_pabrik_pria_warna_kulit_cerah:','zh':':男工人_较浅肤色:','ru':':мужчина_рабочий_очень_светлый_тон_кожи:','tr':':erkek_fabrika_işçisi_açık_cilt_tonu:','ar':':عامل_مصنع_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F3ED':{'en':':man_factory_worker_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':fabrikarbeiter_mitteldunkle_hautfarbe:','es':':profesional_industrial_hombre_tono_de_piel_oscuro_medio:','fr':':ouvrier_peau_mate:','ja':':男性の溶接工_やや濃い肌色:','ko':':공장_남자_직원_진한_갈색_피부:','pt':':operário_pele_morena_escura:','it':':operaio_carnagione_abbastanza_scura:','fa':':کارگر_کارخانه_مرد_پوست_گندمی:','id':':pekerja_pabrik_pria_warna_kulit_gelap-sedang:','zh':':男工人_中等深肤色:','ru':':мужчина_рабочий_темный_тон_кожи:','tr':':erkek_fabrika_işçisi_orta_koyu_cilt_tonu:','ar':':عامل_مصنع_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U0001F3ED':{'en':':man_factory_worker_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':fabrikarbeiter_mittelhelle_hautfarbe:','es':':profesional_industrial_hombre_tono_de_piel_claro_medio:','fr':':ouvrier_peau_moyennement_claire:','ja':':男性の溶接工_やや薄い肌色:','ko':':공장_남자_직원_연한_갈색_피부:','pt':':operário_pele_morena_clara:','it':':operaio_carnagione_abbastanza_chiara:','fa':':کارگر_کارخانه_مرد_پوست_روشن:','id':':pekerja_pabrik_pria_warna_kulit_cerah-sedang:','zh':':男工人_中等浅肤色:','ru':':мужчина_рабочий_светлый_тон_кожи:','tr':':erkek_fabrika_işçisi_orta_açık_cilt_tonu:','ar':':عامل_مصنع_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F3ED':{'en':':man_factory_worker_medium_skin_tone:','status':fully_qualified,'E':4,'de':':fabrikarbeiter_mittlere_hautfarbe:','es':':profesional_industrial_hombre_tono_de_piel_medio:','fr':':ouvrier_peau_légèrement_mate:','ja':':男性の溶接工_中間の肌色:','ko':':공장_남자_직원_갈색_피부:','pt':':operário_pele_morena:','it':':operaio_carnagione_olivastra:','fa':':کارگر_کارخانه_مرد_پوست_طلایی:','id':':pekerja_pabrik_pria_warna_kulit_sedang:','zh':':男工人_中等肤色:','ru':':мужчина_рабочий_средний_тон_кожи:','tr':':erkek_fabrika_işçisi_orta_cilt_tonu:','ar':':عامل_مصنع_بشرة_بلون_معتدل:'},'\U0001F9DA\U0000200D\U00002642\U0000FE0F':{'en':':man_fairy:','status':fully_qualified,'E':5,'alias':[':fairy_man:'],'de':':männliche_fee:','es':':hada_hombre:','fr':':féetaud:','ja':':男の妖精:','ko':':남자_요정:','pt':':homem_fada:','it':':folletto_alato:','fa':':پری_مرد:','id':':peri_pria:','zh':':仙人:','ru':':мужчина-фея:','tr':':erkek_peri:','ar':':جني:'},'\U0001F9DA\U0000200D\U00002642':{'en':':man_fairy:','status':minimally_qualified,'E':5,'alias':[':fairy_man:'],'de':':männliche_fee:','es':':hada_hombre:','fr':':féetaud:','ja':':男の妖精:','ko':':남자_요정:','pt':':homem_fada:','it':':folletto_alato:','fa':':پری_مرد:','id':':peri_pria:','zh':':仙人:','ru':':мужчина_фея:','tr':':erkek_peri:','ar':':جني:'},'\U0001F9DA\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_fairy_dark_skin_tone:','status':fully_qualified,'E':5,'de':':männliche_fee_dunkle_hautfarbe:','es':':hada_hombre_tono_de_piel_oscuro:','fr':':féetaud_peau_foncée:','ja':':男の妖精_濃い肌色:','ko':':남자_요정_검은색_피부:','pt':':homem_fada_pele_escura:','it':':folletto_alato_carnagione_scura:','fa':':پری_مرد_پوست_آبنوسی:','id':':peri_pria_warna_kulit_gelap:','zh':':仙人_较深肤色:','ru':':мужчина-фея_очень_темныи_тон_кожи:','tr':':erkek_peri_koyu_cilt_tonu:','ar':':جني_بشرة_بلون_غامق:'},'\U0001F9DA\U0001F3FF\U0000200D\U00002642':{'en':':man_fairy_dark_skin_tone:','status':minimally_qualified,'E':5,'de':':männliche_fee_dunkle_hautfarbe:','es':':hada_hombre_tono_de_piel_oscuro:','fr':':féetaud_peau_foncée:','ja':':男の妖精_濃い肌色:','ko':':남자_요정_검은색_피부:','pt':':homem_fada_pele_escura:','it':':folletto_alato_carnagione_scura:','fa':':پری_مرد_پوست_آبنوسی:','id':':peri_pria_warna_kulit_gelap:','zh':':仙人_较深肤色:','ru':':мужчина_фея_очень_темный_тон_кожи:','tr':':erkek_peri_koyu_cilt_tonu:','ar':':جني_بشرة_بلون_غامق:'},'\U0001F9DA\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_fairy_light_skin_tone:','status':fully_qualified,'E':5,'de':':männliche_fee_helle_hautfarbe:','es':':hada_hombre_tono_de_piel_claro:','fr':':féetaud_peau_claire:','ja':':男の妖精_薄い肌色:','ko':':남자_요정_하얀_피부:','pt':':homem_fada_pele_clara:','it':':folletto_alato_carnagione_chiara:','fa':':پری_مرد_پوست_سفید:','id':':peri_pria_warna_kulit_cerah:','zh':':仙人_较浅肤色:','ru':':мужчина-фея_очень_светлыи_тон_кожи:','tr':':erkek_peri_açık_cilt_tonu:','ar':':جني_بشرة_بلون_فاتح:'},'\U0001F9DA\U0001F3FB\U0000200D\U00002642':{'en':':man_fairy_light_skin_tone:','status':minimally_qualified,'E':5,'de':':männliche_fee_helle_hautfarbe:','es':':hada_hombre_tono_de_piel_claro:','fr':':féetaud_peau_claire:','ja':':男の妖精_薄い肌色:','ko':':남자_요정_하얀_피부:','pt':':homem_fada_pele_clara:','it':':folletto_alato_carnagione_chiara:','fa':':پری_مرد_پوست_سفید:','id':':peri_pria_warna_kulit_cerah:','zh':':仙人_较浅肤色:','ru':':мужчина_фея_очень_светлый_тон_кожи:','tr':':erkek_peri_açık_cilt_tonu:','ar':':جني_بشرة_بلون_فاتح:'},'\U0001F9DA\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_fairy_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':männliche_fee_mitteldunkle_hautfarbe:','es':':hada_hombre_tono_de_piel_oscuro_medio:','fr':':féetaud_peau_mate:','ja':':男の妖精_やや濃い肌色:','ko':':남자_요정_진한_갈색_피부:','pt':':homem_fada_pele_morena_escura:','it':':folletto_alato_carnagione_abbastanza_scura:','fa':':پری_مرد_پوست_گندمی:','id':':peri_pria_warna_kulit_gelap-sedang:','zh':':仙人_中等深肤色:','ru':':мужчина-фея_темныи_тон_кожи:','tr':':erkek_peri_orta_koyu_cilt_tonu:','ar':':جني_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9DA\U0001F3FE\U0000200D\U00002642':{'en':':man_fairy_medium-dark_skin_tone:','status':minimally_qualified,'E':5,'de':':männliche_fee_mitteldunkle_hautfarbe:','es':':hada_hombre_tono_de_piel_oscuro_medio:','fr':':féetaud_peau_mate:','ja':':男の妖精_やや濃い肌色:','ko':':남자_요정_진한_갈색_피부:','pt':':homem_fada_pele_morena_escura:','it':':folletto_alato_carnagione_abbastanza_scura:','fa':':پری_مرد_پوست_گندمی:','id':':peri_pria_warna_kulit_gelap-sedang:','zh':':仙人_中等深肤色:','ru':':мужчина_фея_темный_тон_кожи:','tr':':erkek_peri_orta_koyu_cilt_tonu:','ar':':جني_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9DA\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_fairy_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':männliche_fee_mittelhelle_hautfarbe:','es':':hada_hombre_tono_de_piel_claro_medio:','fr':':féetaud_peau_moyennement_claire:','ja':':男の妖精_やや薄い肌色:','ko':':남자_요정_연한_갈색_피부:','pt':':homem_fada_pele_morena_clara:','it':':folletto_alato_carnagione_abbastanza_chiara:','fa':':پری_مرد_پوست_روشن:','id':':peri_pria_warna_kulit_cerah-sedang:','zh':':仙人_中等浅肤色:','ru':':мужчина-фея_светлыи_тон_кожи:','tr':':erkek_peri_orta_açık_cilt_tonu:','ar':':جني_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9DA\U0001F3FC\U0000200D\U00002642':{'en':':man_fairy_medium-light_skin_tone:','status':minimally_qualified,'E':5,'de':':männliche_fee_mittelhelle_hautfarbe:','es':':hada_hombre_tono_de_piel_claro_medio:','fr':':féetaud_peau_moyennement_claire:','ja':':男の妖精_やや薄い肌色:','ko':':남자_요정_연한_갈색_피부:','pt':':homem_fada_pele_morena_clara:','it':':folletto_alato_carnagione_abbastanza_chiara:','fa':':پری_مرد_پوست_روشن:','id':':peri_pria_warna_kulit_cerah-sedang:','zh':':仙人_中等浅肤色:','ru':':мужчина_фея_светлый_тон_кожи:','tr':':erkek_peri_orta_açık_cilt_tonu:','ar':':جني_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9DA\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_fairy_medium_skin_tone:','status':fully_qualified,'E':5,'de':':männliche_fee_mittlere_hautfarbe:','es':':hada_hombre_tono_de_piel_medio:','fr':':féetaud_peau_légèrement_mate:','ja':':男の妖精_中間の肌色:','ko':':남자_요정_갈색_피부:','pt':':homem_fada_pele_morena:','it':':folletto_alato_carnagione_olivastra:','fa':':پری_مرد_پوست_طلایی:','id':':peri_pria_warna_kulit_sedang:','zh':':仙人_中等肤色:','ru':':мужчина-фея_среднии_тон_кожи:','tr':':erkek_peri_orta_cilt_tonu:','ar':':جني_بشرة_بلون_معتدل:'},'\U0001F9DA\U0001F3FD\U0000200D\U00002642':{'en':':man_fairy_medium_skin_tone:','status':minimally_qualified,'E':5,'de':':männliche_fee_mittlere_hautfarbe:','es':':hada_hombre_tono_de_piel_medio:','fr':':féetaud_peau_légèrement_mate:','ja':':男の妖精_中間の肌色:','ko':':남자_요정_갈색_피부:','pt':':homem_fada_pele_morena:','it':':folletto_alato_carnagione_olivastra:','fa':':پری_مرد_پوست_طلایی:','id':':peri_pria_warna_kulit_sedang:','zh':':仙人_中等肤色:','ru':':мужчина_фея_средний_тон_кожи:','tr':':erkek_peri_orta_cilt_tonu:','ar':':جني_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U0001F33E':{'en':':man_farmer:','status':fully_qualified,'E':4,'de':':bauer:','es':':profesional_de_la_agricultura_hombre:','fr':':fermier:','ja':':農家の男性:','ko':':남자_농부:','pt':':fazendeiro:','it':':contadino:','fa':':کشاورز_مرد:','id':':petani_pria:','zh':':农夫:','ru':':мужчина_фермер:','tr':':erkek_çiftçi:','ar':':مزارع:'},'\U0001F468\U0001F3FF\U0000200D\U0001F33E':{'en':':man_farmer_dark_skin_tone:','status':fully_qualified,'E':4,'de':':bauer_dunkle_hautfarbe:','es':':profesional_de_la_agricultura_hombre_tono_de_piel_oscuro:','fr':':fermier_peau_foncée:','ja':':農家の男性_濃い肌色:','ko':':남자_농부_검은색_피부:','pt':':fazendeiro_pele_escura:','it':':contadino_carnagione_scura:','fa':':کشاورز_مرد_پوست_آبنوسی:','id':':petani_pria_warna_kulit_gelap:','zh':':农夫_较深肤色:','ru':':мужчина_фермер_очень_темный_тон_кожи:','tr':':erkek_çiftçi_koyu_cilt_tonu:','ar':':مزارع_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U0001F33E':{'en':':man_farmer_light_skin_tone:','status':fully_qualified,'E':4,'de':':bauer_helle_hautfarbe:','es':':profesional_de_la_agricultura_hombre_tono_de_piel_claro:','fr':':fermier_peau_claire:','ja':':農家の男性_薄い肌色:','ko':':남자_농부_하얀_피부:','pt':':fazendeiro_pele_clara:','it':':contadino_carnagione_chiara:','fa':':کشاورز_مرد_پوست_سفید:','id':':petani_pria_warna_kulit_cerah:','zh':':农夫_较浅肤色:','ru':':мужчина_фермер_очень_светлый_тон_кожи:','tr':':erkek_çiftçi_açık_cilt_tonu:','ar':':مزارع_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F33E':{'en':':man_farmer_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':bauer_mitteldunkle_hautfarbe:','es':':profesional_de_la_agricultura_hombre_tono_de_piel_oscuro_medio:','fr':':fermier_peau_mate:','ja':':農家の男性_やや濃い肌色:','ko':':남자_농부_진한_갈색_피부:','pt':':fazendeiro_pele_morena_escura:','it':':contadino_carnagione_abbastanza_scura:','fa':':کشاورز_مرد_پوست_گندمی:','id':':petani_pria_warna_kulit_gelap-sedang:','zh':':农夫_中等深肤色:','ru':':мужчина_фермер_темный_тон_кожи:','tr':':erkek_çiftçi_orta_koyu_cilt_tonu:','ar':':مزارع_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U0001F33E':{'en':':man_farmer_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':bauer_mittelhelle_hautfarbe:','es':':profesional_de_la_agricultura_hombre_tono_de_piel_claro_medio:','fr':':fermier_peau_moyennement_claire:','ja':':農家の男性_やや薄い肌色:','ko':':남자_농부_연한_갈색_피부:','pt':':fazendeiro_pele_morena_clara:','it':':contadino_carnagione_abbastanza_chiara:','fa':':کشاورز_مرد_پوست_روشن:','id':':petani_pria_warna_kulit_cerah-sedang:','zh':':农夫_中等浅肤色:','ru':':мужчина_фермер_светлый_тон_кожи:','tr':':erkek_çiftçi_orta_açık_cilt_tonu:','ar':':مزارع_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F33E':{'en':':man_farmer_medium_skin_tone:','status':fully_qualified,'E':4,'de':':bauer_mittlere_hautfarbe:','es':':profesional_de_la_agricultura_hombre_tono_de_piel_medio:','fr':':fermier_peau_légèrement_mate:','ja':':農家の男性_中間の肌色:','ko':':남자_농부_갈색_피부:','pt':':fazendeiro_pele_morena:','it':':contadino_carnagione_olivastra:','fa':':کشاورز_مرد_پوست_طلایی:','id':':petani_pria_warna_kulit_sedang:','zh':':农夫_中等肤色:','ru':':мужчина_фермер_средний_тон_кожи:','tr':':erkek_çiftçi_orta_cilt_tonu:','ar':':مزارع_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U0001F37C':{'en':':man_feeding_baby:','status':fully_qualified,'E':13,'de':':mann_der_baby_die_flasche_gibt:','es':':hombre_alimentando_a_bebé:','fr':':homme_allaitant_un_bébé:','ja':':授乳する男性:','ko':':수유_중인_남자:','pt':':homem_alimentando_bebê:','it':':uomo_che_allatta:','fa':':مردی_که_به_بچه_شیر_می_دهد:','id':':pria_menyusui_bayi:','zh':':哺乳的男人:','ru':':кормящий_мужчина:','tr':':bebek_emziren_erkek:','ar':':رجل_يُرضع_طفلا:'},'\U0001F468\U0001F3FF\U0000200D\U0001F37C':{'en':':man_feeding_baby_dark_skin_tone:','status':fully_qualified,'E':13,'de':':mann_der_baby_die_flasche_gibt_dunkle_hautfarbe:','es':':hombre_alimentando_a_bebé_tono_de_piel_oscuro:','fr':':homme_allaitant_un_bébé_peau_foncée:','ja':':授乳する男性_濃い肌色:','ko':':수유_중인_남자_검은색_피부:','pt':':homem_alimentando_bebê_pele_escura:','it':':uomo_che_allatta_carnagione_scura:','fa':':مردی_که_به_بچه_شیر_می_دهد_پوست_آبنوسی:','id':':pria_menyusui_bayi_warna_kulit_gelap:','zh':':哺乳的男人_较深肤色:','ru':':кормящий_мужчина_очень_темный_тон_кожи:','tr':':bebek_emziren_erkek_koyu_cilt_tonu:','ar':':رجل_يُرضع_طفلا_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U0001F37C':{'en':':man_feeding_baby_light_skin_tone:','status':fully_qualified,'E':13,'de':':mann_der_baby_die_flasche_gibt_helle_hautfarbe:','es':':hombre_alimentando_a_bebé_tono_de_piel_claro:','fr':':homme_allaitant_un_bébé_peau_claire:','ja':':授乳する男性_薄い肌色:','ko':':수유_중인_남자_하얀_피부:','pt':':homem_alimentando_bebê_pele_clara:','it':':uomo_che_allatta_carnagione_chiara:','fa':':مردی_که_به_بچه_شیر_می_دهد_پوست_سفید:','id':':pria_menyusui_bayi_warna_kulit_cerah:','zh':':哺乳的男人_较浅肤色:','ru':':кормящий_мужчина_очень_светлый_тон_кожи:','tr':':bebek_emziren_erkek_açık_cilt_tonu:','ar':':رجل_يُرضع_طفلا_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F37C':{'en':':man_feeding_baby_medium-dark_skin_tone:','status':fully_qualified,'E':13,'de':':mann_der_baby_die_flasche_gibt_mitteldunkle_hautfarbe:','es':':hombre_alimentando_a_bebé_tono_de_piel_oscuro_medio:','fr':':homme_allaitant_un_bébé_peau_mate:','ja':':授乳する男性_やや濃い肌色:','ko':':수유_중인_남자_진한_갈색_피부:','pt':':homem_alimentando_bebê_pele_morena_escura:','it':':uomo_che_allatta_carnagione_abbastanza_scura:','fa':':مردی_که_به_بچه_شیر_می_دهد_پوست_گندمی:','id':':pria_menyusui_bayi_warna_kulit_gelap-sedang:','zh':':哺乳的男人_中等深肤色:','ru':':кормящий_мужчина_темный_тон_кожи:','tr':':bebek_emziren_erkek_orta_koyu_cilt_tonu:','ar':':رجل_يُرضع_طفلا_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U0001F37C':{'en':':man_feeding_baby_medium-light_skin_tone:','status':fully_qualified,'E':13,'de':':mann_der_baby_die_flasche_gibt_mittelhelle_hautfarbe:','es':':hombre_alimentando_a_bebé_tono_de_piel_claro_medio:','fr':':homme_allaitant_un_bébé_peau_moyennement_claire:','ja':':授乳する男性_やや薄い肌色:','ko':':수유_중인_남자_연한_갈색_피부:','pt':':homem_alimentando_bebê_pele_morena_clara:','it':':uomo_che_allatta_carnagione_abbastanza_chiara:','fa':':مردی_که_به_بچه_شیر_می_دهد_پوست_روشن:','id':':pria_menyusui_bayi_warna_kulit_cerah-sedang:','zh':':哺乳的男人_中等浅肤色:','ru':':кормящий_мужчина_светлый_тон_кожи:','tr':':bebek_emziren_erkek_orta_açık_cilt_tonu:','ar':':رجل_يُرضع_طفلا_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F37C':{'en':':man_feeding_baby_medium_skin_tone:','status':fully_qualified,'E':13,'de':':mann_der_baby_die_flasche_gibt_mittlere_hautfarbe:','es':':hombre_alimentando_a_bebé_tono_de_piel_medio:','fr':':homme_allaitant_un_bébé_peau_légèrement_mate:','ja':':授乳する男性_中間の肌色:','ko':':수유_중인_남자_갈색_피부:','pt':':homem_alimentando_bebê_pele_morena:','it':':uomo_che_allatta_carnagione_olivastra:','fa':':مردی_که_به_بچه_شیر_می_دهد_پوست_طلایی:','id':':pria_menyusui_bayi_warna_kulit_sedang:','zh':':哺乳的男人_中等肤色:','ru':':кормящий_мужчина_средний_тон_кожи:','tr':':bebek_emziren_erkek_orta_cilt_tonu:','ar':':رجل_يُرضع_طفلا_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U0001F692':{'en':':man_firefighter:','status':fully_qualified,'E':4,'de':':feuerwehrmann:','es':':bombero_hombre:','fr':':pompier_homme:','ja':':男性消防士:','ko':':남자_소방관:','pt':':bombeiro_homem:','it':':pompiere_uomo:','fa':':آتش_نشان_مرد:','id':':pemadam_kebakaran_pria:','zh':':男消防员:','ru':':мужчина_пожарный:','tr':':erkek_itfaiyeci:','ar':':رجل_إطفاء:'},'\U0001F468\U0001F3FF\U0000200D\U0001F692':{'en':':man_firefighter_dark_skin_tone:','status':fully_qualified,'E':4,'de':':feuerwehrmann_dunkle_hautfarbe:','es':':bombero_hombre_tono_de_piel_oscuro:','fr':':pompier_homme_peau_foncée:','ja':':男性消防士_濃い肌色:','ko':':남자_소방관_검은색_피부:','pt':':bombeiro_homem_pele_escura:','it':':pompiere_uomo_carnagione_scura:','fa':':آتش_نشان_مرد_پوست_آبنوسی:','id':':pemadam_kebakaran_pria_warna_kulit_gelap:','zh':':男消防员_较深肤色:','ru':':мужчина_пожарный_очень_темный_тон_кожи:','tr':':erkek_itfaiyeci_koyu_cilt_tonu:','ar':':رجل_إطفاء_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U0001F692':{'en':':man_firefighter_light_skin_tone:','status':fully_qualified,'E':4,'de':':feuerwehrmann_helle_hautfarbe:','es':':bombero_hombre_tono_de_piel_claro:','fr':':pompier_homme_peau_claire:','ja':':男性消防士_薄い肌色:','ko':':남자_소방관_하얀_피부:','pt':':bombeiro_homem_pele_clara:','it':':pompiere_uomo_carnagione_chiara:','fa':':آتش_نشان_مرد_پوست_سفید:','id':':pemadam_kebakaran_pria_warna_kulit_cerah:','zh':':男消防员_较浅肤色:','ru':':мужчина_пожарный_очень_светлый_тон_кожи:','tr':':erkek_itfaiyeci_açık_cilt_tonu:','ar':':رجل_إطفاء_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F692':{'en':':man_firefighter_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':feuerwehrmann_mitteldunkle_hautfarbe:','es':':bombero_hombre_tono_de_piel_oscuro_medio:','fr':':pompier_homme_peau_mate:','ja':':男性消防士_やや濃い肌色:','ko':':남자_소방관_진한_갈색_피부:','pt':':bombeiro_homem_pele_morena_escura:','it':':pompiere_uomo_carnagione_abbastanza_scura:','fa':':آتش_نشان_مرد_پوست_گندمی:','id':':pemadam_kebakaran_pria_warna_kulit_gelap-sedang:','zh':':男消防员_中等深肤色:','ru':':мужчина_пожарный_темный_тон_кожи:','tr':':erkek_itfaiyeci_orta_koyu_cilt_tonu:','ar':':رجل_إطفاء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U0001F692':{'en':':man_firefighter_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':feuerwehrmann_mittelhelle_hautfarbe:','es':':bombero_hombre_tono_de_piel_claro_medio:','fr':':pompier_homme_peau_moyennement_claire:','ja':':男性消防士_やや薄い肌色:','ko':':남자_소방관_연한_갈색_피부:','pt':':bombeiro_homem_pele_morena_clara:','it':':pompiere_uomo_carnagione_abbastanza_chiara:','fa':':آتش_نشان_مرد_پوست_روشن:','id':':pemadam_kebakaran_pria_warna_kulit_cerah-sedang:','zh':':男消防员_中等浅肤色:','ru':':мужчина_пожарный_светлый_тон_кожи:','tr':':erkek_itfaiyeci_orta_açık_cilt_tonu:','ar':':رجل_إطفاء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F692':{'en':':man_firefighter_medium_skin_tone:','status':fully_qualified,'E':4,'de':':feuerwehrmann_mittlere_hautfarbe:','es':':bombero_hombre_tono_de_piel_medio:','fr':':pompier_homme_peau_légèrement_mate:','ja':':男性消防士_中間の肌色:','ko':':남자_소방관_갈색_피부:','pt':':bombeiro_homem_pele_morena:','it':':pompiere_uomo_carnagione_olivastra:','fa':':آتش_نشان_مرد_پوست_طلایی:','id':':pemadam_kebakaran_pria_warna_kulit_sedang:','zh':':男消防员_中等肤色:','ru':':мужчина_пожарный_средний_тон_кожи:','tr':':erkek_itfaiyeci_orta_cilt_tonu:','ar':':رجل_إطفاء_بشرة_بلون_معتدل:'},'\U0001F64D\U0000200D\U00002642\U0000FE0F':{'en':':man_frowning:','status':fully_qualified,'E':4,'alias':[':frowning_man:'],'de':':missmutiger_mann:','es':':hombre_frunciendo_el_ceño:','fr':':homme_fronçant_les_sourcils:','ja':':しかめ面の男:','ko':':찌푸린_남자:','pt':':homem_franzindo_a_sobrancelha:','it':':uomo_corrucciato:','fa':':مرد_اخم_می_کند:','id':':pria_muka_tidak_senang:','zh':':皱眉男:','ru':':нахмурившиися_мужчина:','tr':':somurtan_erkek:','ar':':متجهم:'},'\U0001F64D\U0000200D\U00002642':{'en':':man_frowning:','status':minimally_qualified,'E':4,'alias':[':frowning_man:'],'de':':missmutiger_mann:','es':':hombre_frunciendo_el_ceño:','fr':':homme_fronçant_les_sourcils:','ja':':しかめ面の男:','ko':':찌푸린_남자:','pt':':homem_franzindo_a_sobrancelha:','it':':uomo_corrucciato:','fa':':مرد_اخم_می_کند:','id':':pria_muka_tidak_senang:','zh':':皱眉男:','ru':':нахмурившийся_мужчина:','tr':':somurtan_erkek:','ar':':متجهم:'},'\U0001F64D\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_frowning_dark_skin_tone:','status':fully_qualified,'E':4,'de':':missmutiger_mann_dunkle_hautfarbe:','es':':hombre_frunciendo_el_ceño_tono_de_piel_oscuro:','fr':':homme_fronçant_les_sourcils_peau_foncée:','ja':':しかめ面の男_濃い肌色:','ko':':찌푸린_남자_검은색_피부:','pt':':homem_franzindo_a_sobrancelha_pele_escura:','it':':uomo_corrucciato_carnagione_scura:','fa':':مرد_اخم_می_کند_پوست_آبنوسی:','id':':pria_muka_tidak_senang_warna_kulit_gelap:','zh':':皱眉男_较深肤色:','ru':':нахмурившиися_мужчина_очень_темныи_тон_кожи:','tr':':somurtan_erkek_koyu_cilt_tonu:','ar':':متجهم_بشرة_بلون_غامق:'},'\U0001F64D\U0001F3FF\U0000200D\U00002642':{'en':':man_frowning_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':missmutiger_mann_dunkle_hautfarbe:','es':':hombre_frunciendo_el_ceño_tono_de_piel_oscuro:','fr':':homme_fronçant_les_sourcils_peau_foncée:','ja':':しかめ面の男_濃い肌色:','ko':':찌푸린_남자_검은색_피부:','pt':':homem_franzindo_a_sobrancelha_pele_escura:','it':':uomo_corrucciato_carnagione_scura:','fa':':مرد_اخم_می_کند_پوست_آبنوسی:','id':':pria_muka_tidak_senang_warna_kulit_gelap:','zh':':皱眉男_较深肤色:','ru':':нахмурившийся_мужчина_очень_темный_тон_кожи:','tr':':somurtan_erkek_koyu_cilt_tonu:','ar':':متجهم_بشرة_بلون_غامق:'},'\U0001F64D\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_frowning_light_skin_tone:','status':fully_qualified,'E':4,'de':':missmutiger_mann_helle_hautfarbe:','es':':hombre_frunciendo_el_ceño_tono_de_piel_claro:','fr':':homme_fronçant_les_sourcils_peau_claire:','ja':':しかめ面の男_薄い肌色:','ko':':찌푸린_남자_하얀_피부:','pt':':homem_franzindo_a_sobrancelha_pele_clara:','it':':uomo_corrucciato_carnagione_chiara:','fa':':مرد_اخم_می_کند_پوست_سفید:','id':':pria_muka_tidak_senang_warna_kulit_cerah:','zh':':皱眉男_较浅肤色:','ru':':нахмурившиися_мужчина_очень_светлыи_тон_кожи:','tr':':somurtan_erkek_açık_cilt_tonu:','ar':':متجهم_بشرة_بلون_فاتح:'},'\U0001F64D\U0001F3FB\U0000200D\U00002642':{'en':':man_frowning_light_skin_tone:','status':minimally_qualified,'E':4,'de':':missmutiger_mann_helle_hautfarbe:','es':':hombre_frunciendo_el_ceño_tono_de_piel_claro:','fr':':homme_fronçant_les_sourcils_peau_claire:','ja':':しかめ面の男_薄い肌色:','ko':':찌푸린_남자_하얀_피부:','pt':':homem_franzindo_a_sobrancelha_pele_clara:','it':':uomo_corrucciato_carnagione_chiara:','fa':':مرد_اخم_می_کند_پوست_سفید:','id':':pria_muka_tidak_senang_warna_kulit_cerah:','zh':':皱眉男_较浅肤色:','ru':':нахмурившийся_мужчина_очень_светлый_тон_кожи:','tr':':somurtan_erkek_açık_cilt_tonu:','ar':':متجهم_بشرة_بلون_فاتح:'},'\U0001F64D\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_frowning_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':missmutiger_mann_mitteldunkle_hautfarbe:','es':':hombre_frunciendo_el_ceño_tono_de_piel_oscuro_medio:','fr':':homme_fronçant_les_sourcils_peau_mate:','ja':':しかめ面の男_やや濃い肌色:','ko':':찌푸린_남자_진한_갈색_피부:','pt':':homem_franzindo_a_sobrancelha_pele_morena_escura:','it':':uomo_corrucciato_carnagione_abbastanza_scura:','fa':':مرد_اخم_می_کند_پوست_گندمی:','id':':pria_muka_tidak_senang_warna_kulit_gelap-sedang:','zh':':皱眉男_中等深肤色:','ru':':нахмурившиися_мужчина_темныи_тон_кожи:','tr':':somurtan_erkek_orta_koyu_cilt_tonu:','ar':':متجهم_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F64D\U0001F3FE\U0000200D\U00002642':{'en':':man_frowning_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':missmutiger_mann_mitteldunkle_hautfarbe:','es':':hombre_frunciendo_el_ceño_tono_de_piel_oscuro_medio:','fr':':homme_fronçant_les_sourcils_peau_mate:','ja':':しかめ面の男_やや濃い肌色:','ko':':찌푸린_남자_진한_갈색_피부:','pt':':homem_franzindo_a_sobrancelha_pele_morena_escura:','it':':uomo_corrucciato_carnagione_abbastanza_scura:','fa':':مرد_اخم_می_کند_پوست_گندمی:','id':':pria_muka_tidak_senang_warna_kulit_gelap-sedang:','zh':':皱眉男_中等深肤色:','ru':':нахмурившийся_мужчина_темный_тон_кожи:','tr':':somurtan_erkek_orta_koyu_cilt_tonu:','ar':':متجهم_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F64D\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_frowning_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':missmutiger_mann_mittelhelle_hautfarbe:','es':':hombre_frunciendo_el_ceño_tono_de_piel_claro_medio:','fr':':homme_fronçant_les_sourcils_peau_moyennement_claire:','ja':':しかめ面の男_やや薄い肌色:','ko':':찌푸린_남자_연한_갈색_피부:','pt':':homem_franzindo_a_sobrancelha_pele_morena_clara:','it':':uomo_corrucciato_carnagione_abbastanza_chiara:','fa':':مرد_اخم_می_کند_پوست_روشن:','id':':pria_muka_tidak_senang_warna_kulit_cerah-sedang:','zh':':皱眉男_中等浅肤色:','ru':':нахмурившиися_мужчина_светлыи_тон_кожи:','tr':':somurtan_erkek_orta_açık_cilt_tonu:','ar':':متجهم_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F64D\U0001F3FC\U0000200D\U00002642':{'en':':man_frowning_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':missmutiger_mann_mittelhelle_hautfarbe:','es':':hombre_frunciendo_el_ceño_tono_de_piel_claro_medio:','fr':':homme_fronçant_les_sourcils_peau_moyennement_claire:','ja':':しかめ面の男_やや薄い肌色:','ko':':찌푸린_남자_연한_갈색_피부:','pt':':homem_franzindo_a_sobrancelha_pele_morena_clara:','it':':uomo_corrucciato_carnagione_abbastanza_chiara:','fa':':مرد_اخم_می_کند_پوست_روشن:','id':':pria_muka_tidak_senang_warna_kulit_cerah-sedang:','zh':':皱眉男_中等浅肤色:','ru':':нахмурившийся_мужчина_светлый_тон_кожи:','tr':':somurtan_erkek_orta_açık_cilt_tonu:','ar':':متجهم_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F64D\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_frowning_medium_skin_tone:','status':fully_qualified,'E':4,'de':':missmutiger_mann_mittlere_hautfarbe:','es':':hombre_frunciendo_el_ceño_tono_de_piel_medio:','fr':':homme_fronçant_les_sourcils_peau_légèrement_mate:','ja':':しかめ面の男_中間の肌色:','ko':':찌푸린_남자_갈색_피부:','pt':':homem_franzindo_a_sobrancelha_pele_morena:','it':':uomo_corrucciato_carnagione_olivastra:','fa':':مرد_اخم_می_کند_پوست_طلایی:','id':':pria_muka_tidak_senang_warna_kulit_sedang:','zh':':皱眉男_中等肤色:','ru':':нахмурившиися_мужчина_среднии_тон_кожи:','tr':':somurtan_erkek_orta_cilt_tonu:','ar':':متجهم_بشرة_بلون_معتدل:'},'\U0001F64D\U0001F3FD\U0000200D\U00002642':{'en':':man_frowning_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':missmutiger_mann_mittlere_hautfarbe:','es':':hombre_frunciendo_el_ceño_tono_de_piel_medio:','fr':':homme_fronçant_les_sourcils_peau_légèrement_mate:','ja':':しかめ面の男_中間の肌色:','ko':':찌푸린_남자_갈색_피부:','pt':':homem_franzindo_a_sobrancelha_pele_morena:','it':':uomo_corrucciato_carnagione_olivastra:','fa':':مرد_اخم_می_کند_پوست_طلایی:','id':':pria_muka_tidak_senang_warna_kulit_sedang:','zh':':皱眉男_中等肤色:','ru':':нахмурившийся_мужчина_средний_тон_кожи:','tr':':somurtan_erkek_orta_cilt_tonu:','ar':':متجهم_بشرة_بلون_معتدل:'},'\U0001F9DE\U0000200D\U00002642\U0000FE0F':{'en':':man_genie:','status':fully_qualified,'E':5,'alias':[':genie_man:'],'de':':männlicher_flaschengeist:','es':':genio_hombre:','fr':':génie_homme:','ja':':男の精霊:','ko':':남자_지니:','pt':':homem_gênio:','it':':genio_uomo:','fa':':مرد_جن:','id':':jin_pria:','zh':':男妖怪:','ru':':мужчина-джинн:','tr':':erkek_cin:','ar':':مارد:'},'\U0001F9DE\U0000200D\U00002642':{'en':':man_genie:','status':minimally_qualified,'E':5,'alias':[':genie_man:'],'de':':männlicher_flaschengeist:','es':':genio_hombre:','fr':':génie_homme:','ja':':男の精霊:','ko':':남자_지니:','pt':':homem_gênio:','it':':genio_uomo:','fa':':مرد_جن:','id':':jin_pria:','zh':':男妖怪:','ru':':мужчина_джинн:','tr':':erkek_cin:','ar':':مارد:'},'\U0001F645\U0000200D\U00002642\U0000FE0F':{'en':':man_gesturing_NO:','status':fully_qualified,'E':4,'alias':[':no_good_man:', ':ng_man:', ':man_gesturing_no:'],'de':':mann_mit_überkreuzten_armen:','es':':hombre_haciendo_el_gesto_de_no:','fr':':homme_faisant_un_geste_d’interdiction:','ja':':ダメのポーズをする男:','ko':':안_된다는_제스처를_하는_남자:','pt':':homem_fazendo_gesto_de_“não”:','it':':uomo_con_gesto_di_rifiuto:','fa':':مرد_نمی_پذیرد:','id':':pria_mengisyaratkan_tidak:','zh':':禁止手势男:','ru':':мужчина_против:','tr':':hayır_hareketi_yapan_erkek:','ar':':غير_موافق:'},'\U0001F645\U0000200D\U00002642':{'en':':man_gesturing_NO:','status':minimally_qualified,'E':4,'alias':[':no_good_man:', ':ng_man:', ':man_gesturing_no:'],'de':':mann_mit_überkreuzten_armen:','es':':hombre_haciendo_el_gesto_de_no:','fr':':homme_faisant_un_geste_d’interdiction:','ja':':ダメのポーズをする男:','ko':':안_된다는_제스처를_하는_남자:','pt':':homem_fazendo_gesto_de_“não”:','it':':uomo_con_gesto_di_rifiuto:','fa':':مرد_نمی_پذیرد:','id':':pria_mengisyaratkan_tidak:','zh':':禁止手势男:','ru':':мужчина_против:','tr':':hayır_hareketi_yapan_erkek:','ar':':غير_موافق:'},'\U0001F645\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_gesturing_NO_dark_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_überkreuzten_armen_dunkle_hautfarbe:','es':':hombre_haciendo_el_gesto_de_no_tono_de_piel_oscuro:','fr':':homme_faisant_un_geste_d’interdiction_peau_foncée:','ja':':ダメのポーズをする男_濃い肌色:','ko':':안_된다는_제스처를_하는_남자_검은색_피부:','pt':':homem_fazendo_gesto_de_“não”_pele_escura:','it':':uomo_con_gesto_di_rifiuto_carnagione_scura:','fa':':مرد_نمی_پذیرد_پوست_آبنوسی:','id':':pria_mengisyaratkan_tidak_warna_kulit_gelap:','zh':':禁止手势男_较深肤色:','ru':':мужчина_против_очень_темныи_тон_кожи:','tr':':hayır_hareketi_yapan_erkek_koyu_cilt_tonu:','ar':':غير_موافق_بشرة_بلون_غامق:'},'\U0001F645\U0001F3FF\U0000200D\U00002642':{'en':':man_gesturing_NO_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_überkreuzten_armen_dunkle_hautfarbe:','es':':hombre_haciendo_el_gesto_de_no_tono_de_piel_oscuro:','fr':':homme_faisant_un_geste_d’interdiction_peau_foncée:','ja':':ダメのポーズをする男_濃い肌色:','ko':':안_된다는_제스처를_하는_남자_검은색_피부:','pt':':homem_fazendo_gesto_de_“não”_pele_escura:','it':':uomo_con_gesto_di_rifiuto_carnagione_scura:','fa':':مرد_نمی_پذیرد_پوست_آبنوسی:','id':':pria_mengisyaratkan_tidak_warna_kulit_gelap:','zh':':禁止手势男_较深肤色:','ru':':мужчина_против_очень_темный_тон_кожи:','tr':':hayır_hareketi_yapan_erkek_koyu_cilt_tonu:','ar':':غير_موافق_بشرة_بلون_غامق:'},'\U0001F645\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_gesturing_NO_light_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_überkreuzten_armen_helle_hautfarbe:','es':':hombre_haciendo_el_gesto_de_no_tono_de_piel_claro:','fr':':homme_faisant_un_geste_d’interdiction_peau_claire:','ja':':ダメのポーズをする男_薄い肌色:','ko':':안_된다는_제스처를_하는_남자_하얀_피부:','pt':':homem_fazendo_gesto_de_“não”_pele_clara:','it':':uomo_con_gesto_di_rifiuto_carnagione_chiara:','fa':':مرد_نمی_پذیرد_پوست_سفید:','id':':pria_mengisyaratkan_tidak_warna_kulit_cerah:','zh':':禁止手势男_较浅肤色:','ru':':мужчина_против_очень_светлыи_тон_кожи:','tr':':hayır_hareketi_yapan_erkek_açık_cilt_tonu:','ar':':غير_موافق_بشرة_بلون_فاتح:'},'\U0001F645\U0001F3FB\U0000200D\U00002642':{'en':':man_gesturing_NO_light_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_überkreuzten_armen_helle_hautfarbe:','es':':hombre_haciendo_el_gesto_de_no_tono_de_piel_claro:','fr':':homme_faisant_un_geste_d’interdiction_peau_claire:','ja':':ダメのポーズをする男_薄い肌色:','ko':':안_된다는_제스처를_하는_남자_하얀_피부:','pt':':homem_fazendo_gesto_de_“não”_pele_clara:','it':':uomo_con_gesto_di_rifiuto_carnagione_chiara:','fa':':مرد_نمی_پذیرد_پوست_سفید:','id':':pria_mengisyaratkan_tidak_warna_kulit_cerah:','zh':':禁止手势男_较浅肤色:','ru':':мужчина_против_очень_светлый_тон_кожи:','tr':':hayır_hareketi_yapan_erkek_açık_cilt_tonu:','ar':':غير_موافق_بشرة_بلون_فاتح:'},'\U0001F645\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_gesturing_NO_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_überkreuzten_armen_mitteldunkle_hautfarbe:','es':':hombre_haciendo_el_gesto_de_no_tono_de_piel_oscuro_medio:','fr':':homme_faisant_un_geste_d’interdiction_peau_mate:','ja':':ダメのポーズをする男_やや濃い肌色:','ko':':안_된다는_제스처를_하는_남자_진한_갈색_피부:','pt':':homem_fazendo_gesto_de_“não”_pele_morena_escura:','it':':uomo_con_gesto_di_rifiuto_carnagione_abbastanza_scura:','fa':':مرد_نمی_پذیرد_پوست_گندمی:','id':':pria_mengisyaratkan_tidak_warna_kulit_gelap-sedang:','zh':':禁止手势男_中等深肤色:','ru':':мужчина_против_темныи_тон_кожи:','tr':':hayır_hareketi_yapan_erkek_orta_koyu_cilt_tonu:','ar':':غير_موافق_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F645\U0001F3FE\U0000200D\U00002642':{'en':':man_gesturing_NO_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_überkreuzten_armen_mitteldunkle_hautfarbe:','es':':hombre_haciendo_el_gesto_de_no_tono_de_piel_oscuro_medio:','fr':':homme_faisant_un_geste_d’interdiction_peau_mate:','ja':':ダメのポーズをする男_やや濃い肌色:','ko':':안_된다는_제스처를_하는_남자_진한_갈색_피부:','pt':':homem_fazendo_gesto_de_“não”_pele_morena_escura:','it':':uomo_con_gesto_di_rifiuto_carnagione_abbastanza_scura:','fa':':مرد_نمی_پذیرد_پوست_گندمی:','id':':pria_mengisyaratkan_tidak_warna_kulit_gelap-sedang:','zh':':禁止手势男_中等深肤色:','ru':':мужчина_против_темный_тон_кожи:','tr':':hayır_hareketi_yapan_erkek_orta_koyu_cilt_tonu:','ar':':غير_موافق_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F645\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_gesturing_NO_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_überkreuzten_armen_mittelhelle_hautfarbe:','es':':hombre_haciendo_el_gesto_de_no_tono_de_piel_claro_medio:','fr':':homme_faisant_un_geste_d’interdiction_peau_moyennement_claire:','ja':':ダメのポーズをする男_やや薄い肌色:','ko':':안_된다는_제스처를_하는_남자_연한_갈색_피부:','pt':':homem_fazendo_gesto_de_“não”_pele_morena_clara:','it':':uomo_con_gesto_di_rifiuto_carnagione_abbastanza_chiara:','fa':':مرد_نمی_پذیرد_پوست_روشن:','id':':pria_mengisyaratkan_tidak_warna_kulit_cerah-sedang:','zh':':禁止手势男_中等浅肤色:','ru':':мужчина_против_светлыи_тон_кожи:','tr':':hayır_hareketi_yapan_erkek_orta_açık_cilt_tonu:','ar':':غير_موافق_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F645\U0001F3FC\U0000200D\U00002642':{'en':':man_gesturing_NO_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_überkreuzten_armen_mittelhelle_hautfarbe:','es':':hombre_haciendo_el_gesto_de_no_tono_de_piel_claro_medio:','fr':':homme_faisant_un_geste_d’interdiction_peau_moyennement_claire:','ja':':ダメのポーズをする男_やや薄い肌色:','ko':':안_된다는_제스처를_하는_남자_연한_갈색_피부:','pt':':homem_fazendo_gesto_de_“não”_pele_morena_clara:','it':':uomo_con_gesto_di_rifiuto_carnagione_abbastanza_chiara:','fa':':مرد_نمی_پذیرد_پوست_روشن:','id':':pria_mengisyaratkan_tidak_warna_kulit_cerah-sedang:','zh':':禁止手势男_中等浅肤色:','ru':':мужчина_против_светлый_тон_кожи:','tr':':hayır_hareketi_yapan_erkek_orta_açık_cilt_tonu:','ar':':غير_موافق_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F645\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_gesturing_NO_medium_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_überkreuzten_armen_mittlere_hautfarbe:','es':':hombre_haciendo_el_gesto_de_no_tono_de_piel_medio:','fr':':homme_faisant_un_geste_d’interdiction_peau_légèrement_mate:','ja':':ダメのポーズをする男_中間の肌色:','ko':':안_된다는_제스처를_하는_남자_갈색_피부:','pt':':homem_fazendo_gesto_de_“não”_pele_morena:','it':':uomo_con_gesto_di_rifiuto_carnagione_olivastra:','fa':':مرد_نمی_پذیرد_پوست_طلایی:','id':':pria_mengisyaratkan_tidak_warna_kulit_sedang:','zh':':禁止手势男_中等肤色:','ru':':мужчина_против_среднии_тон_кожи:','tr':':hayır_hareketi_yapan_erkek_orta_cilt_tonu:','ar':':غير_موافق_بشرة_بلون_معتدل:'},'\U0001F645\U0001F3FD\U0000200D\U00002642':{'en':':man_gesturing_NO_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_überkreuzten_armen_mittlere_hautfarbe:','es':':hombre_haciendo_el_gesto_de_no_tono_de_piel_medio:','fr':':homme_faisant_un_geste_d’interdiction_peau_légèrement_mate:','ja':':ダメのポーズをする男_中間の肌色:','ko':':안_된다는_제스처를_하는_남자_갈색_피부:','pt':':homem_fazendo_gesto_de_“não”_pele_morena:','it':':uomo_con_gesto_di_rifiuto_carnagione_olivastra:','fa':':مرد_نمی_پذیرد_پوست_طلایی:','id':':pria_mengisyaratkan_tidak_warna_kulit_sedang:','zh':':禁止手势男_中等肤色:','ru':':мужчина_против_средний_тон_кожи:','tr':':hayır_hareketi_yapan_erkek_orta_cilt_tonu:','ar':':غير_موافق_بشرة_بلون_معتدل:'},'\U0001F646\U0000200D\U00002642\U0000FE0F':{'en':':man_gesturing_OK:','status':fully_qualified,'E':4,'alias':[':ok_man:', ':man_gesturing_ok:'],'de':':mann_mit_händen_auf_dem_kopf:','es':':hombre_haciendo_el_gesto_de_de_acuerdo:','fr':':homme_faisant_un_geste_d’acceptation:','ja':':okのポーズをする男:','ko':':오케이라는_제스처를_하는_남자:','pt':':homem_fazendo_gesto_de_“ok”:','it':':uomo_con_gesto_ok:','fa':':مرد_می_پذیرد:','id':':pria_mengisyaratkan_oke:','zh':':OK手势男:','ru':':мужчина_за:','tr':':tamam_hareketi_yapan_erkek:','ar':':موافق:'},'\U0001F646\U0000200D\U00002642':{'en':':man_gesturing_OK:','status':minimally_qualified,'E':4,'alias':[':ok_man:', ':man_gesturing_ok:'],'de':':mann_mit_händen_auf_dem_kopf:','es':':hombre_haciendo_el_gesto_de_de_acuerdo:','fr':':homme_faisant_un_geste_d’acceptation:','ja':':okのポーズをする男:','ko':':오케이라는_제스처를_하는_남자:','pt':':homem_fazendo_gesto_de_“ok”:','it':':uomo_con_gesto_ok:','fa':':مرد_می_پذیرد:','id':':pria_mengisyaratkan_oke:','zh':':OK手势男:','ru':':мужчина_за:','tr':':tamam_hareketi_yapan_erkek:','ar':':موافق:'},'\U0001F646\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_gesturing_OK_dark_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_händen_auf_dem_kopf_dunkle_hautfarbe:','es':':hombre_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_oscuro:','fr':':homme_faisant_un_geste_d’acceptation_peau_foncée:','ja':':okのポーズをする男_濃い肌色:','ko':':오케이라는_제스처를_하는_남자_검은색_피부:','pt':':homem_fazendo_gesto_de_“ok”_pele_escura:','it':':uomo_con_gesto_ok_carnagione_scura:','fa':':مرد_می_پذیرد_پوست_آبنوسی:','id':':pria_mengisyaratkan_oke_warna_kulit_gelap:','zh':':OK手势男_较深肤色:','ru':':мужчина_за_очень_темныи_тон_кожи:','tr':':tamam_hareketi_yapan_erkek_koyu_cilt_tonu:','ar':':موافق_بشرة_بلون_غامق:'},'\U0001F646\U0001F3FF\U0000200D\U00002642':{'en':':man_gesturing_OK_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_händen_auf_dem_kopf_dunkle_hautfarbe:','es':':hombre_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_oscuro:','fr':':homme_faisant_un_geste_d’acceptation_peau_foncée:','ja':':okのポーズをする男_濃い肌色:','ko':':오케이라는_제스처를_하는_남자_검은색_피부:','pt':':homem_fazendo_gesto_de_“ok”_pele_escura:','it':':uomo_con_gesto_ok_carnagione_scura:','fa':':مرد_می_پذیرد_پوست_آبنوسی:','id':':pria_mengisyaratkan_oke_warna_kulit_gelap:','zh':':OK手势男_较深肤色:','ru':':мужчина_за_очень_темный_тон_кожи:','tr':':tamam_hareketi_yapan_erkek_koyu_cilt_tonu:','ar':':موافق_بشرة_بلون_غامق:'},'\U0001F646\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_gesturing_OK_light_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_händen_auf_dem_kopf_helle_hautfarbe:','es':':hombre_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_claro:','fr':':homme_faisant_un_geste_d’acceptation_peau_claire:','ja':':okのポーズをする男_薄い肌色:','ko':':오케이라는_제스처를_하는_남자_하얀_피부:','pt':':homem_fazendo_gesto_de_“ok”_pele_clara:','it':':uomo_con_gesto_ok_carnagione_chiara:','fa':':مرد_می_پذیرد_پوست_سفید:','id':':pria_mengisyaratkan_oke_warna_kulit_cerah:','zh':':OK手势男_较浅肤色:','ru':':мужчина_за_очень_светлыи_тон_кожи:','tr':':tamam_hareketi_yapan_erkek_açık_cilt_tonu:','ar':':موافق_بشرة_بلون_فاتح:'},'\U0001F646\U0001F3FB\U0000200D\U00002642':{'en':':man_gesturing_OK_light_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_händen_auf_dem_kopf_helle_hautfarbe:','es':':hombre_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_claro:','fr':':homme_faisant_un_geste_d’acceptation_peau_claire:','ja':':okのポーズをする男_薄い肌色:','ko':':오케이라는_제스처를_하는_남자_하얀_피부:','pt':':homem_fazendo_gesto_de_“ok”_pele_clara:','it':':uomo_con_gesto_ok_carnagione_chiara:','fa':':مرد_می_پذیرد_پوست_سفید:','id':':pria_mengisyaratkan_oke_warna_kulit_cerah:','zh':':OK手势男_较浅肤色:','ru':':мужчина_за_очень_светлый_тон_кожи:','tr':':tamam_hareketi_yapan_erkek_açık_cilt_tonu:','ar':':موافق_بشرة_بلون_فاتح:'},'\U0001F646\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_gesturing_OK_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_händen_auf_dem_kopf_mitteldunkle_hautfarbe:','es':':hombre_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_oscuro_medio:','fr':':homme_faisant_un_geste_d’acceptation_peau_mate:','ja':':okのポーズをする男_やや濃い肌色:','ko':':오케이라는_제스처를_하는_남자_진한_갈색_피부:','pt':':homem_fazendo_gesto_de_“ok”_pele_morena_escura:','it':':uomo_con_gesto_ok_carnagione_abbastanza_scura:','fa':':مرد_می_پذیرد_پوست_گندمی:','id':':pria_mengisyaratkan_oke_warna_kulit_gelap-sedang:','zh':':OK手势男_中等深肤色:','ru':':мужчина_за_темныи_тон_кожи:','tr':':tamam_hareketi_yapan_erkek_orta_koyu_cilt_tonu:','ar':':موافق_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F646\U0001F3FE\U0000200D\U00002642':{'en':':man_gesturing_OK_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_händen_auf_dem_kopf_mitteldunkle_hautfarbe:','es':':hombre_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_oscuro_medio:','fr':':homme_faisant_un_geste_d’acceptation_peau_mate:','ja':':okのポーズをする男_やや濃い肌色:','ko':':오케이라는_제스처를_하는_남자_진한_갈색_피부:','pt':':homem_fazendo_gesto_de_“ok”_pele_morena_escura:','it':':uomo_con_gesto_ok_carnagione_abbastanza_scura:','fa':':مرد_می_پذیرد_پوست_گندمی:','id':':pria_mengisyaratkan_oke_warna_kulit_gelap-sedang:','zh':':OK手势男_中等深肤色:','ru':':мужчина_за_темный_тон_кожи:','tr':':tamam_hareketi_yapan_erkek_orta_koyu_cilt_tonu:','ar':':موافق_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F646\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_gesturing_OK_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_händen_auf_dem_kopf_mittelhelle_hautfarbe:','es':':hombre_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_claro_medio:','fr':':homme_faisant_un_geste_d’acceptation_peau_moyennement_claire:','ja':':okのポーズをする男_やや薄い肌色:','ko':':오케이라는_제스처를_하는_남자_연한_갈색_피부:','pt':':homem_fazendo_gesto_de_“ok”_pele_morena_clara:','it':':uomo_con_gesto_ok_carnagione_abbastanza_chiara:','fa':':مرد_می_پذیرد_پوست_روشن:','id':':pria_mengisyaratkan_oke_warna_kulit_cerah-sedang:','zh':':OK手势男_中等浅肤色:','ru':':мужчина_за_светлыи_тон_кожи:','tr':':tamam_hareketi_yapan_erkek_orta_açık_cilt_tonu:','ar':':موافق_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F646\U0001F3FC\U0000200D\U00002642':{'en':':man_gesturing_OK_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_händen_auf_dem_kopf_mittelhelle_hautfarbe:','es':':hombre_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_claro_medio:','fr':':homme_faisant_un_geste_d’acceptation_peau_moyennement_claire:','ja':':okのポーズをする男_やや薄い肌色:','ko':':오케이라는_제스처를_하는_남자_연한_갈색_피부:','pt':':homem_fazendo_gesto_de_“ok”_pele_morena_clara:','it':':uomo_con_gesto_ok_carnagione_abbastanza_chiara:','fa':':مرد_می_پذیرد_پوست_روشن:','id':':pria_mengisyaratkan_oke_warna_kulit_cerah-sedang:','zh':':OK手势男_中等浅肤色:','ru':':мужчина_за_светлый_тон_кожи:','tr':':tamam_hareketi_yapan_erkek_orta_açık_cilt_tonu:','ar':':موافق_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F646\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_gesturing_OK_medium_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_händen_auf_dem_kopf_mittlere_hautfarbe:','es':':hombre_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_medio:','fr':':homme_faisant_un_geste_d’acceptation_peau_légèrement_mate:','ja':':okのポーズをする男_中間の肌色:','ko':':오케이라는_제스처를_하는_남자_갈색_피부:','pt':':homem_fazendo_gesto_de_“ok”_pele_morena:','it':':uomo_con_gesto_ok_carnagione_olivastra:','fa':':مرد_می_پذیرد_پوست_طلایی:','id':':pria_mengisyaratkan_oke_warna_kulit_sedang:','zh':':OK手势男_中等肤色:','ru':':мужчина_за_среднии_тон_кожи:','tr':':tamam_hareketi_yapan_erkek_orta_cilt_tonu:','ar':':موافق_بشرة_بلون_معتدل:'},'\U0001F646\U0001F3FD\U0000200D\U00002642':{'en':':man_gesturing_OK_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_händen_auf_dem_kopf_mittlere_hautfarbe:','es':':hombre_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_medio:','fr':':homme_faisant_un_geste_d’acceptation_peau_légèrement_mate:','ja':':okのポーズをする男_中間の肌色:','ko':':오케이라는_제스처를_하는_남자_갈색_피부:','pt':':homem_fazendo_gesto_de_“ok”_pele_morena:','it':':uomo_con_gesto_ok_carnagione_olivastra:','fa':':مرد_می_پذیرد_پوست_طلایی:','id':':pria_mengisyaratkan_oke_warna_kulit_sedang:','zh':':OK手势男_中等肤色:','ru':':мужчина_за_средний_тон_кожи:','tr':':tamam_hareketi_yapan_erkek_orta_cilt_tonu:','ar':':موافق_بشرة_بلون_معتدل:'},'\U0001F487\U0000200D\U00002642\U0000FE0F':{'en':':man_getting_haircut:','status':fully_qualified,'E':4,'alias':[':haircut_man:'],'de':':mann_beim_haareschneiden:','es':':hombre_cortándose_el_pelo:','fr':':homme_qui_se_fait_couper_les_cheveux:','ja':':散髪される男:','ko':':남자_헤어컷:','pt':':homem_cortando_o_cabelo:','it':':taglio_di_capelli_per_uomo:','fa':':مرد_مو_کوتاه_می_کند:','id':':pria_potong_rambut:','zh':':男生理发:','ru':':мужчина_у_парикмахера:','tr':':saç_kestiren_erkek:','ar':':يقص_شعره:'},'\U0001F487\U0000200D\U00002642':{'en':':man_getting_haircut:','status':minimally_qualified,'E':4,'alias':[':haircut_man:'],'de':':mann_beim_haareschneiden:','es':':hombre_cortándose_el_pelo:','fr':':homme_qui_se_fait_couper_les_cheveux:','ja':':散髪される男:','ko':':남자_헤어컷:','pt':':homem_cortando_o_cabelo:','it':':taglio_di_capelli_per_uomo:','fa':':مرد_مو_کوتاه_می_کند:','id':':pria_potong_rambut:','zh':':男生理发:','ru':':мужчина_у_парикмахера:','tr':':saç_kestiren_erkek:','ar':':يقص_شعره:'},'\U0001F487\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_getting_haircut_dark_skin_tone:','status':fully_qualified,'E':4,'de':':mann_beim_haareschneiden_dunkle_hautfarbe:','es':':hombre_cortándose_el_pelo_tono_de_piel_oscuro:','fr':':homme_qui_se_fait_couper_les_cheveux_peau_foncée:','ja':':散髪される男_濃い肌色:','ko':':남자_헤어컷_검은색_피부:','pt':':homem_cortando_o_cabelo_pele_escura:','it':':taglio_di_capelli_per_uomo_carnagione_scura:','fa':':مرد_مو_کوتاه_می_کند_پوست_آبنوسی:','id':':pria_potong_rambut_warna_kulit_gelap:','zh':':男生理发_较深肤色:','ru':':мужчина_у_парикмахера_очень_темныи_тон_кожи:','tr':':saç_kestiren_erkek_koyu_cilt_tonu:','ar':':يقص_شعره_بشرة_بلون_غامق:'},'\U0001F487\U0001F3FF\U0000200D\U00002642':{'en':':man_getting_haircut_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_beim_haareschneiden_dunkle_hautfarbe:','es':':hombre_cortándose_el_pelo_tono_de_piel_oscuro:','fr':':homme_qui_se_fait_couper_les_cheveux_peau_foncée:','ja':':散髪される男_濃い肌色:','ko':':남자_헤어컷_검은색_피부:','pt':':homem_cortando_o_cabelo_pele_escura:','it':':taglio_di_capelli_per_uomo_carnagione_scura:','fa':':مرد_مو_کوتاه_می_کند_پوست_آبنوسی:','id':':pria_potong_rambut_warna_kulit_gelap:','zh':':男生理发_较深肤色:','ru':':мужчина_у_парикмахера_очень_темный_тон_кожи:','tr':':saç_kestiren_erkek_koyu_cilt_tonu:','ar':':يقص_شعره_بشرة_بلون_غامق:'},'\U0001F487\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_getting_haircut_light_skin_tone:','status':fully_qualified,'E':4,'de':':mann_beim_haareschneiden_helle_hautfarbe:','es':':hombre_cortándose_el_pelo_tono_de_piel_claro:','fr':':homme_qui_se_fait_couper_les_cheveux_peau_claire:','ja':':散髪される男_薄い肌色:','ko':':남자_헤어컷_하얀_피부:','pt':':homem_cortando_o_cabelo_pele_clara:','it':':taglio_di_capelli_per_uomo_carnagione_chiara:','fa':':مرد_مو_کوتاه_می_کند_پوست_سفید:','id':':pria_potong_rambut_warna_kulit_cerah:','zh':':男生理发_较浅肤色:','ru':':мужчина_у_парикмахера_очень_светлыи_тон_кожи:','tr':':saç_kestiren_erkek_açık_cilt_tonu:','ar':':يقص_شعره_بشرة_بلون_فاتح:'},'\U0001F487\U0001F3FB\U0000200D\U00002642':{'en':':man_getting_haircut_light_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_beim_haareschneiden_helle_hautfarbe:','es':':hombre_cortándose_el_pelo_tono_de_piel_claro:','fr':':homme_qui_se_fait_couper_les_cheveux_peau_claire:','ja':':散髪される男_薄い肌色:','ko':':남자_헤어컷_하얀_피부:','pt':':homem_cortando_o_cabelo_pele_clara:','it':':taglio_di_capelli_per_uomo_carnagione_chiara:','fa':':مرد_مو_کوتاه_می_کند_پوست_سفید:','id':':pria_potong_rambut_warna_kulit_cerah:','zh':':男生理发_较浅肤色:','ru':':мужчина_у_парикмахера_очень_светлый_тон_кожи:','tr':':saç_kestiren_erkek_açık_cilt_tonu:','ar':':يقص_شعره_بشرة_بلون_فاتح:'},'\U0001F487\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_getting_haircut_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':mann_beim_haareschneiden_mitteldunkle_hautfarbe:','es':':hombre_cortándose_el_pelo_tono_de_piel_oscuro_medio:','fr':':homme_qui_se_fait_couper_les_cheveux_peau_mate:','ja':':散髪される男_やや濃い肌色:','ko':':남자_헤어컷_진한_갈색_피부:','pt':':homem_cortando_o_cabelo_pele_morena_escura:','it':':taglio_di_capelli_per_uomo_carnagione_abbastanza_scura:','fa':':مرد_مو_کوتاه_می_کند_پوست_گندمی:','id':':pria_potong_rambut_warna_kulit_gelap-sedang:','zh':':男生理发_中等深肤色:','ru':':мужчина_у_парикмахера_темныи_тон_кожи:','tr':':saç_kestiren_erkek_orta_koyu_cilt_tonu:','ar':':يقص_شعره_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F487\U0001F3FE\U0000200D\U00002642':{'en':':man_getting_haircut_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_beim_haareschneiden_mitteldunkle_hautfarbe:','es':':hombre_cortándose_el_pelo_tono_de_piel_oscuro_medio:','fr':':homme_qui_se_fait_couper_les_cheveux_peau_mate:','ja':':散髪される男_やや濃い肌色:','ko':':남자_헤어컷_진한_갈색_피부:','pt':':homem_cortando_o_cabelo_pele_morena_escura:','it':':taglio_di_capelli_per_uomo_carnagione_abbastanza_scura:','fa':':مرد_مو_کوتاه_می_کند_پوست_گندمی:','id':':pria_potong_rambut_warna_kulit_gelap-sedang:','zh':':男生理发_中等深肤色:','ru':':мужчина_у_парикмахера_темный_тон_кожи:','tr':':saç_kestiren_erkek_orta_koyu_cilt_tonu:','ar':':يقص_شعره_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F487\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_getting_haircut_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':mann_beim_haareschneiden_mittelhelle_hautfarbe:','es':':hombre_cortándose_el_pelo_tono_de_piel_claro_medio:','fr':':homme_qui_se_fait_couper_les_cheveux_peau_moyennement_claire:','ja':':散髪される男_やや薄い肌色:','ko':':남자_헤어컷_연한_갈색_피부:','pt':':homem_cortando_o_cabelo_pele_morena_clara:','it':':taglio_di_capelli_per_uomo_carnagione_abbastanza_chiara:','fa':':مرد_مو_کوتاه_می_کند_پوست_روشن:','id':':pria_potong_rambut_warna_kulit_cerah-sedang:','zh':':男生理发_中等浅肤色:','ru':':мужчина_у_парикмахера_светлыи_тон_кожи:','tr':':saç_kestiren_erkek_orta_açık_cilt_tonu:','ar':':يقص_شعره_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F487\U0001F3FC\U0000200D\U00002642':{'en':':man_getting_haircut_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_beim_haareschneiden_mittelhelle_hautfarbe:','es':':hombre_cortándose_el_pelo_tono_de_piel_claro_medio:','fr':':homme_qui_se_fait_couper_les_cheveux_peau_moyennement_claire:','ja':':散髪される男_やや薄い肌色:','ko':':남자_헤어컷_연한_갈색_피부:','pt':':homem_cortando_o_cabelo_pele_morena_clara:','it':':taglio_di_capelli_per_uomo_carnagione_abbastanza_chiara:','fa':':مرد_مو_کوتاه_می_کند_پوست_روشن:','id':':pria_potong_rambut_warna_kulit_cerah-sedang:','zh':':男生理发_中等浅肤色:','ru':':мужчина_у_парикмахера_светлый_тон_кожи:','tr':':saç_kestiren_erkek_orta_açık_cilt_tonu:','ar':':يقص_شعره_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F487\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_getting_haircut_medium_skin_tone:','status':fully_qualified,'E':4,'de':':mann_beim_haareschneiden_mittlere_hautfarbe:','es':':hombre_cortándose_el_pelo_tono_de_piel_medio:','fr':':homme_qui_se_fait_couper_les_cheveux_peau_légèrement_mate:','ja':':散髪される男_中間の肌色:','ko':':남자_헤어컷_갈색_피부:','pt':':homem_cortando_o_cabelo_pele_morena:','it':':taglio_di_capelli_per_uomo_carnagione_olivastra:','fa':':مرد_مو_کوتاه_می_کند_پوست_طلایی:','id':':pria_potong_rambut_warna_kulit_sedang:','zh':':男生理发_中等肤色:','ru':':мужчина_у_парикмахера_среднии_тон_кожи:','tr':':saç_kestiren_erkek_orta_cilt_tonu:','ar':':يقص_شعره_بشرة_بلون_معتدل:'},'\U0001F487\U0001F3FD\U0000200D\U00002642':{'en':':man_getting_haircut_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_beim_haareschneiden_mittlere_hautfarbe:','es':':hombre_cortándose_el_pelo_tono_de_piel_medio:','fr':':homme_qui_se_fait_couper_les_cheveux_peau_légèrement_mate:','ja':':散髪される男_中間の肌色:','ko':':남자_헤어컷_갈색_피부:','pt':':homem_cortando_o_cabelo_pele_morena:','it':':taglio_di_capelli_per_uomo_carnagione_olivastra:','fa':':مرد_مو_کوتاه_می_کند_پوست_طلایی:','id':':pria_potong_rambut_warna_kulit_sedang:','zh':':男生理发_中等肤色:','ru':':мужчина_у_парикмахера_средний_тон_кожи:','tr':':saç_kestiren_erkek_orta_cilt_tonu:','ar':':يقص_شعره_بشرة_بلون_معتدل:'},'\U0001F486\U0000200D\U00002642\U0000FE0F':{'en':':man_getting_massage:','status':fully_qualified,'E':4,'alias':[':massage_man:'],'de':':mann,_der_eine_kopfmassage_bekommt:','es':':hombre_recibiendo_masaje:','fr':':homme_qui_se_fait_masser:','ja':':フェイスマッサージ中の男:','ko':':마사지_받는_남자:','pt':':homem_recebendo_massagem_facial:','it':':uomo_che_riceve_un_massaggio:','fa':':ماساژ_صورت_مرد:','id':':pria_dipijat_kepalanya:','zh':':男生按摩:','ru':':мужчине_массируют_лицо:','tr':':yüz_masajı_yaptıran_erkek:','ar':':تدليك_لرجل:'},'\U0001F486\U0000200D\U00002642':{'en':':man_getting_massage:','status':minimally_qualified,'E':4,'alias':[':massage_man:'],'de':':mann_der_eine_kopfmassage_bekommt:','es':':hombre_recibiendo_masaje:','fr':':homme_qui_se_fait_masser:','ja':':フェイスマッサージ中の男:','ko':':마사지_받는_남자:','pt':':homem_recebendo_massagem_facial:','it':':uomo_che_riceve_un_massaggio:','fa':':ماساژ_صورت_مرد:','id':':pria_dipijat_kepalanya:','zh':':男生按摩:','ru':':мужчине_массируют_лицо:','tr':':yüz_masajı_yaptıran_erkek:','ar':':تدليك_لرجل:'},'\U0001F486\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_getting_massage_dark_skin_tone:','status':fully_qualified,'E':4,'de':':mann,_der_eine_kopfmassage_bekommt_dunkle_hautfarbe:','es':':hombre_recibiendo_masaje_tono_de_piel_oscuro:','fr':':homme_qui_se_fait_masser_peau_foncée:','ja':':フェイスマッサージ中の男_濃い肌色:','ko':':마사지_받는_남자_검은색_피부:','pt':':homem_recebendo_massagem_facial_pele_escura:','it':':uomo_che_riceve_un_massaggio_carnagione_scura:','fa':':ماساژ_صورت_مرد_پوست_آبنوسی:','id':':pria_dipijat_kepalanya_warna_kulit_gelap:','zh':':男生按摩_较深肤色:','ru':':мужчине_массируют_лицо_очень_темныи_тон_кожи:','tr':':yüz_masajı_yaptıran_erkek_koyu_cilt_tonu:','ar':':تدليك_لرجل_بشرة_بلون_غامق:'},'\U0001F486\U0001F3FF\U0000200D\U00002642':{'en':':man_getting_massage_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_der_eine_kopfmassage_bekommt_dunkle_hautfarbe:','es':':hombre_recibiendo_masaje_tono_de_piel_oscuro:','fr':':homme_qui_se_fait_masser_peau_foncée:','ja':':フェイスマッサージ中の男_濃い肌色:','ko':':마사지_받는_남자_검은색_피부:','pt':':homem_recebendo_massagem_facial_pele_escura:','it':':uomo_che_riceve_un_massaggio_carnagione_scura:','fa':':ماساژ_صورت_مرد_پوست_آبنوسی:','id':':pria_dipijat_kepalanya_warna_kulit_gelap:','zh':':男生按摩_较深肤色:','ru':':мужчине_массируют_лицо_очень_темный_тон_кожи:','tr':':yüz_masajı_yaptıran_erkek_koyu_cilt_tonu:','ar':':تدليك_لرجل_بشرة_بلون_غامق:'},'\U0001F486\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_getting_massage_light_skin_tone:','status':fully_qualified,'E':4,'de':':mann,_der_eine_kopfmassage_bekommt_helle_hautfarbe:','es':':hombre_recibiendo_masaje_tono_de_piel_claro:','fr':':homme_qui_se_fait_masser_peau_claire:','ja':':フェイスマッサージ中の男_薄い肌色:','ko':':마사지_받는_남자_하얀_피부:','pt':':homem_recebendo_massagem_facial_pele_clara:','it':':uomo_che_riceve_un_massaggio_carnagione_chiara:','fa':':ماساژ_صورت_مرد_پوست_سفید:','id':':pria_dipijat_kepalanya_warna_kulit_cerah:','zh':':男生按摩_较浅肤色:','ru':':мужчине_массируют_лицо_очень_светлыи_тон_кожи:','tr':':yüz_masajı_yaptıran_erkek_açık_cilt_tonu:','ar':':تدليك_لرجل_بشرة_بلون_فاتح:'},'\U0001F486\U0001F3FB\U0000200D\U00002642':{'en':':man_getting_massage_light_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_der_eine_kopfmassage_bekommt_helle_hautfarbe:','es':':hombre_recibiendo_masaje_tono_de_piel_claro:','fr':':homme_qui_se_fait_masser_peau_claire:','ja':':フェイスマッサージ中の男_薄い肌色:','ko':':마사지_받는_남자_하얀_피부:','pt':':homem_recebendo_massagem_facial_pele_clara:','it':':uomo_che_riceve_un_massaggio_carnagione_chiara:','fa':':ماساژ_صورت_مرد_پوست_سفید:','id':':pria_dipijat_kepalanya_warna_kulit_cerah:','zh':':男生按摩_较浅肤色:','ru':':мужчине_массируют_лицо_очень_светлый_тон_кожи:','tr':':yüz_masajı_yaptıran_erkek_açık_cilt_tonu:','ar':':تدليك_لرجل_بشرة_بلون_فاتح:'},'\U0001F486\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_getting_massage_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':mann,_der_eine_kopfmassage_bekommt_mitteldunkle_hautfarbe:','es':':hombre_recibiendo_masaje_tono_de_piel_oscuro_medio:','fr':':homme_qui_se_fait_masser_peau_mate:','ja':':フェイスマッサージ中の男_やや濃い肌色:','ko':':마사지_받는_남자_진한_갈색_피부:','pt':':homem_recebendo_massagem_facial_pele_morena_escura:','it':':uomo_che_riceve_un_massaggio_carnagione_abbastanza_scura:','fa':':ماساژ_صورت_مرد_پوست_گندمی:','id':':pria_dipijat_kepalanya_warna_kulit_gelap-sedang:','zh':':男生按摩_中等深肤色:','ru':':мужчине_массируют_лицо_темныи_тон_кожи:','tr':':yüz_masajı_yaptıran_erkek_orta_koyu_cilt_tonu:','ar':':تدليك_لرجل_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F486\U0001F3FE\U0000200D\U00002642':{'en':':man_getting_massage_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_der_eine_kopfmassage_bekommt_mitteldunkle_hautfarbe:','es':':hombre_recibiendo_masaje_tono_de_piel_oscuro_medio:','fr':':homme_qui_se_fait_masser_peau_mate:','ja':':フェイスマッサージ中の男_やや濃い肌色:','ko':':마사지_받는_남자_진한_갈색_피부:','pt':':homem_recebendo_massagem_facial_pele_morena_escura:','it':':uomo_che_riceve_un_massaggio_carnagione_abbastanza_scura:','fa':':ماساژ_صورت_مرد_پوست_گندمی:','id':':pria_dipijat_kepalanya_warna_kulit_gelap-sedang:','zh':':男生按摩_中等深肤色:','ru':':мужчине_массируют_лицо_темный_тон_кожи:','tr':':yüz_masajı_yaptıran_erkek_orta_koyu_cilt_tonu:','ar':':تدليك_لرجل_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F486\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_getting_massage_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':mann,_der_eine_kopfmassage_bekommt_mittelhelle_hautfarbe:','es':':hombre_recibiendo_masaje_tono_de_piel_claro_medio:','fr':':homme_qui_se_fait_masser_peau_moyennement_claire:','ja':':フェイスマッサージ中の男_やや薄い肌色:','ko':':마사지_받는_남자_연한_갈색_피부:','pt':':homem_recebendo_massagem_facial_pele_morena_clara:','it':':uomo_che_riceve_un_massaggio_carnagione_abbastanza_chiara:','fa':':ماساژ_صورت_مرد_پوست_روشن:','id':':pria_dipijat_kepalanya_warna_kulit_cerah-sedang:','zh':':男生按摩_中等浅肤色:','ru':':мужчине_массируют_лицо_светлыи_тон_кожи:','tr':':yüz_masajı_yaptıran_erkek_orta_açık_cilt_tonu:','ar':':تدليك_لرجل_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F486\U0001F3FC\U0000200D\U00002642':{'en':':man_getting_massage_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_der_eine_kopfmassage_bekommt_mittelhelle_hautfarbe:','es':':hombre_recibiendo_masaje_tono_de_piel_claro_medio:','fr':':homme_qui_se_fait_masser_peau_moyennement_claire:','ja':':フェイスマッサージ中の男_やや薄い肌色:','ko':':마사지_받는_남자_연한_갈색_피부:','pt':':homem_recebendo_massagem_facial_pele_morena_clara:','it':':uomo_che_riceve_un_massaggio_carnagione_abbastanza_chiara:','fa':':ماساژ_صورت_مرد_پوست_روشن:','id':':pria_dipijat_kepalanya_warna_kulit_cerah-sedang:','zh':':男生按摩_中等浅肤色:','ru':':мужчине_массируют_лицо_светлый_тон_кожи:','tr':':yüz_masajı_yaptıran_erkek_orta_açık_cilt_tonu:','ar':':تدليك_لرجل_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F486\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_getting_massage_medium_skin_tone:','status':fully_qualified,'E':4,'de':':mann,_der_eine_kopfmassage_bekommt_mittlere_hautfarbe:','es':':hombre_recibiendo_masaje_tono_de_piel_medio:','fr':':homme_qui_se_fait_masser_peau_légèrement_mate:','ja':':フェイスマッサージ中の男_中間の肌色:','ko':':마사지_받는_남자_갈색_피부:','pt':':homem_recebendo_massagem_facial_pele_morena:','it':':uomo_che_riceve_un_massaggio_carnagione_olivastra:','fa':':ماساژ_صورت_مرد_پوست_طلایی:','id':':pria_dipijat_kepalanya_warna_kulit_sedang:','zh':':男生按摩_中等肤色:','ru':':мужчине_массируют_лицо_среднии_тон_кожи:','tr':':yüz_masajı_yaptıran_erkek_orta_cilt_tonu:','ar':':تدليك_لرجل_بشرة_بلون_معتدل:'},'\U0001F486\U0001F3FD\U0000200D\U00002642':{'en':':man_getting_massage_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_der_eine_kopfmassage_bekommt_mittlere_hautfarbe:','es':':hombre_recibiendo_masaje_tono_de_piel_medio:','fr':':homme_qui_se_fait_masser_peau_légèrement_mate:','ja':':フェイスマッサージ中の男_中間の肌色:','ko':':마사지_받는_남자_갈색_피부:','pt':':homem_recebendo_massagem_facial_pele_morena:','it':':uomo_che_riceve_un_massaggio_carnagione_olivastra:','fa':':ماساژ_صورت_مرد_پوست_طلایی:','id':':pria_dipijat_kepalanya_warna_kulit_sedang:','zh':':男生按摩_中等肤色:','ru':':мужчине_массируют_лицо_средний_тон_кожи:','tr':':yüz_masajı_yaptıran_erkek_orta_cilt_tonu:','ar':':تدليك_لرجل_بشرة_بلون_معتدل:'},'\U0001F3CC\U0000FE0F\U0000200D\U00002642\U0000FE0F':{'en':':man_golfing:','status':fully_qualified,'E':4,'alias':[':golfing_man:'],'de':':golfer:','es':':hombre_jugando_al_golf:','fr':':golfeur:','ja':':ゴルフをする男:','ko':':골프치는_남자:','pt':':homem_golfista:','it':':golfista_uomo:','fa':':گلف_باز_مرد:','id':':pria_bermain_golf:','zh':':男生打高尔夫:','ru':':мужчина_играет_в_гольф:','tr':':golf_oynayan_erkek:','ar':':لاعب_غولف:'},'\U0001F3CC\U0000200D\U00002642\U0000FE0F':{'en':':man_golfing:','status':unqualified,'E':4,'alias':[':golfing_man:'],'de':':golfer:','es':':hombre_jugando_al_golf:','fr':':golfeur:','ja':':ゴルフをする男:','ko':':골프치는_남자:','pt':':homem_golfista:','it':':golfista_uomo:','fa':':گلف_باز_مرد:','id':':pria_bermain_golf:','zh':':男生打高尔夫:','ru':':мужчина_играет_в_гольф:','tr':':golf_oynayan_erkek:','ar':':لاعب_غولف:'},'\U0001F3CC\U0000FE0F\U0000200D\U00002642':{'en':':man_golfing:','status':minimally_qualified,'E':4,'alias':[':golfing_man:'],'de':':golfer:','es':':hombre_jugando_al_golf:','fr':':golfeur:','ja':':ゴルフをする男:','ko':':골프치는_남자:','pt':':homem_golfista:','it':':golfista_uomo:','fa':':گلف_باز_مرد:','id':':pria_bermain_golf:','zh':':男生打高尔夫:','ru':':мужчина_играет_в_гольф:','tr':':golf_oynayan_erkek:','ar':':لاعب_غولف:'},'\U0001F3CC\U0000200D\U00002642':{'en':':man_golfing:','status':unqualified,'E':4,'alias':[':golfing_man:'],'de':':golfer:','es':':hombre_jugando_al_golf:','fr':':golfeur:','ja':':ゴルフをする男:','ko':':골프치는_남자:','pt':':homem_golfista:','it':':golfista_uomo:','fa':':گلف_باز_مرد:','id':':pria_bermain_golf:','zh':':男生打高尔夫:','ru':':мужчина_играет_в_гольф:','tr':':golf_oynayan_erkek:','ar':':لاعب_غولف:'},'\U0001F3CC\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_golfing_dark_skin_tone:','status':fully_qualified,'E':4,'de':':golfer_dunkle_hautfarbe:','es':':hombre_jugando_al_golf_tono_de_piel_oscuro:','fr':':golfeur_peau_foncée:','ja':':ゴルフをする男_濃い肌色:','ko':':골프치는_남자_검은색_피부:','pt':':homem_golfista_pele_escura:','it':':golfista_uomo_carnagione_scura:','fa':':گلف_باز_مرد_پوست_آبنوسی:','id':':pria_bermain_golf_warna_kulit_gelap:','zh':':男生打高尔夫_较深肤色:','ru':':мужчина_играет_в_гольф_очень_темныи_тон_кожи:','tr':':golf_oynayan_erkek_koyu_cilt_tonu:','ar':':لاعب_غولف_بشرة_بلون_غامق:'},'\U0001F3CC\U0001F3FF\U0000200D\U00002642':{'en':':man_golfing_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':golfer_dunkle_hautfarbe:','es':':hombre_jugando_al_golf_tono_de_piel_oscuro:','fr':':golfeur_peau_foncée:','ja':':ゴルフをする男_濃い肌色:','ko':':골프치는_남자_검은색_피부:','pt':':homem_golfista_pele_escura:','it':':golfista_uomo_carnagione_scura:','fa':':گلف_باز_مرد_پوست_آبنوسی:','id':':pria_bermain_golf_warna_kulit_gelap:','zh':':男生打高尔夫_较深肤色:','ru':':мужчина_играет_в_гольф_очень_темный_тон_кожи:','tr':':golf_oynayan_erkek_koyu_cilt_tonu:','ar':':لاعب_غولف_بشرة_بلون_غامق:'},'\U0001F3CC\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_golfing_light_skin_tone:','status':fully_qualified,'E':4,'de':':golfer_helle_hautfarbe:','es':':hombre_jugando_al_golf_tono_de_piel_claro:','fr':':golfeur_peau_claire:','ja':':ゴルフをする男_薄い肌色:','ko':':골프치는_남자_하얀_피부:','pt':':homem_golfista_pele_clara:','it':':golfista_uomo_carnagione_chiara:','fa':':گلف_باز_مرد_پوست_سفید:','id':':pria_bermain_golf_warna_kulit_cerah:','zh':':男生打高尔夫_较浅肤色:','ru':':мужчина_играет_в_гольф_очень_светлыи_тон_кожи:','tr':':golf_oynayan_erkek_açık_cilt_tonu:','ar':':لاعب_غولف_بشرة_بلون_فاتح:'},'\U0001F3CC\U0001F3FB\U0000200D\U00002642':{'en':':man_golfing_light_skin_tone:','status':minimally_qualified,'E':4,'de':':golfer_helle_hautfarbe:','es':':hombre_jugando_al_golf_tono_de_piel_claro:','fr':':golfeur_peau_claire:','ja':':ゴルフをする男_薄い肌色:','ko':':골프치는_남자_하얀_피부:','pt':':homem_golfista_pele_clara:','it':':golfista_uomo_carnagione_chiara:','fa':':گلف_باز_مرد_پوست_سفید:','id':':pria_bermain_golf_warna_kulit_cerah:','zh':':男生打高尔夫_较浅肤色:','ru':':мужчина_играет_в_гольф_очень_светлый_тон_кожи:','tr':':golf_oynayan_erkek_açık_cilt_tonu:','ar':':لاعب_غولف_بشرة_بلون_فاتح:'},'\U0001F3CC\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_golfing_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':golfer_mitteldunkle_hautfarbe:','es':':hombre_jugando_al_golf_tono_de_piel_oscuro_medio:','fr':':golfeur_peau_mate:','ja':':ゴルフをする男_やや濃い肌色:','ko':':골프치는_남자_진한_갈색_피부:','pt':':homem_golfista_pele_morena_escura:','it':':golfista_uomo_carnagione_abbastanza_scura:','fa':':گلف_باز_مرد_پوست_گندمی:','id':':pria_bermain_golf_warna_kulit_gelap-sedang:','zh':':男生打高尔夫_中等深肤色:','ru':':мужчина_играет_в_гольф_темныи_тон_кожи:','tr':':golf_oynayan_erkek_orta_koyu_cilt_tonu:','ar':':لاعب_غولف_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3CC\U0001F3FE\U0000200D\U00002642':{'en':':man_golfing_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':golfer_mitteldunkle_hautfarbe:','es':':hombre_jugando_al_golf_tono_de_piel_oscuro_medio:','fr':':golfeur_peau_mate:','ja':':ゴルフをする男_やや濃い肌色:','ko':':골프치는_남자_진한_갈색_피부:','pt':':homem_golfista_pele_morena_escura:','it':':golfista_uomo_carnagione_abbastanza_scura:','fa':':گلف_باز_مرد_پوست_گندمی:','id':':pria_bermain_golf_warna_kulit_gelap-sedang:','zh':':男生打高尔夫_中等深肤色:','ru':':мужчина_играет_в_гольф_темный_тон_кожи:','tr':':golf_oynayan_erkek_orta_koyu_cilt_tonu:','ar':':لاعب_غولف_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3CC\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_golfing_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':golfer_mittelhelle_hautfarbe:','es':':hombre_jugando_al_golf_tono_de_piel_claro_medio:','fr':':golfeur_peau_moyennement_claire:','ja':':ゴルフをする男_やや薄い肌色:','ko':':골프치는_남자_연한_갈색_피부:','pt':':homem_golfista_pele_morena_clara:','it':':golfista_uomo_carnagione_abbastanza_chiara:','fa':':گلف_باز_مرد_پوست_روشن:','id':':pria_bermain_golf_warna_kulit_cerah-sedang:','zh':':男生打高尔夫_中等浅肤色:','ru':':мужчина_играет_в_гольф_светлыи_тон_кожи:','tr':':golf_oynayan_erkek_orta_açık_cilt_tonu:','ar':':لاعب_غولف_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3CC\U0001F3FC\U0000200D\U00002642':{'en':':man_golfing_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':golfer_mittelhelle_hautfarbe:','es':':hombre_jugando_al_golf_tono_de_piel_claro_medio:','fr':':golfeur_peau_moyennement_claire:','ja':':ゴルフをする男_やや薄い肌色:','ko':':골프치는_남자_연한_갈색_피부:','pt':':homem_golfista_pele_morena_clara:','it':':golfista_uomo_carnagione_abbastanza_chiara:','fa':':گلف_باز_مرد_پوست_روشن:','id':':pria_bermain_golf_warna_kulit_cerah-sedang:','zh':':男生打高尔夫_中等浅肤色:','ru':':мужчина_играет_в_гольф_светлый_тон_кожи:','tr':':golf_oynayan_erkek_orta_açık_cilt_tonu:','ar':':لاعب_غولف_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3CC\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_golfing_medium_skin_tone:','status':fully_qualified,'E':4,'de':':golfer_mittlere_hautfarbe:','es':':hombre_jugando_al_golf_tono_de_piel_medio:','fr':':golfeur_peau_légèrement_mate:','ja':':ゴルフをする男_中間の肌色:','ko':':골프치는_남자_갈색_피부:','pt':':homem_golfista_pele_morena:','it':':golfista_uomo_carnagione_olivastra:','fa':':گلف_باز_مرد_پوست_طلایی:','id':':pria_bermain_golf_warna_kulit_sedang:','zh':':男生打高尔夫_中等肤色:','ru':':мужчина_играет_в_гольф_среднии_тон_кожи:','tr':':golf_oynayan_erkek_orta_cilt_tonu:','ar':':لاعب_غولف_بشرة_بلون_معتدل:'},'\U0001F3CC\U0001F3FD\U0000200D\U00002642':{'en':':man_golfing_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':golfer_mittlere_hautfarbe:','es':':hombre_jugando_al_golf_tono_de_piel_medio:','fr':':golfeur_peau_légèrement_mate:','ja':':ゴルフをする男_中間の肌色:','ko':':골프치는_남자_갈색_피부:','pt':':homem_golfista_pele_morena:','it':':golfista_uomo_carnagione_olivastra:','fa':':گلف_باز_مرد_پوست_طلایی:','id':':pria_bermain_golf_warna_kulit_sedang:','zh':':男生打高尔夫_中等肤色:','ru':':мужчина_играет_в_гольф_средний_тон_кожи:','tr':':golf_oynayan_erkek_orta_cilt_tonu:','ar':':لاعب_غولف_بشرة_بلون_معتدل:'},'\U0001F482\U0000200D\U00002642\U0000FE0F':{'en':':man_guard:','status':fully_qualified,'E':4,'alias':[':guardsman:'],'de':':wachmann:','es':':guardia_hombre:','fr':':garde_homme:','ja':':男性の衛兵:','ko':':남자_근위병:','pt':':guarda_homem:','it':':guardia_uomo:','fa':':گارد_مرد:','id':':penjaga_pria:','zh':':男卫兵:','ru':':мужчина-гвардеец:','tr':':erkek_muhafız:','ar':':رجل_حراسة:'},'\U0001F482\U0000200D\U00002642':{'en':':man_guard:','status':minimally_qualified,'E':4,'alias':[':guardsman:'],'de':':wachsoldat:','es':':guardia_hombre:','fr':':garde_homme:','ja':':男性の衛兵:','ko':':남자_근위병:','pt':':guarda_homem:','it':':guardia_uomo:','fa':':گارد_مرد:','id':':penjaga_pria:','zh':':男卫兵:','ru':':мужчина_гвардеец:','tr':':erkek_muhafız:','ar':':رجل_حراسة:'},'\U0001F482\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_guard_dark_skin_tone:','status':fully_qualified,'E':4,'de':':wachmann_dunkle_hautfarbe:','es':':guardia_hombre_tono_de_piel_oscuro:','fr':':garde_homme_peau_foncée:','ja':':男性の衛兵_濃い肌色:','ko':':남자_근위병_검은색_피부:','pt':':guarda_homem_pele_escura:','it':':guardia_uomo_carnagione_scura:','fa':':گارد_مرد_پوست_آبنوسی:','id':':penjaga_pria_warna_kulit_gelap:','zh':':男卫兵_较深肤色:','ru':':мужчина-гвардеец_очень_темныи_тон_кожи:','tr':':erkek_muhafız_koyu_cilt_tonu:','ar':':رجل_حراسة_بشرة_بلون_غامق:'},'\U0001F482\U0001F3FF\U0000200D\U00002642':{'en':':man_guard_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':wachsoldat_dunkle_hautfarbe:','es':':guardia_hombre_tono_de_piel_oscuro:','fr':':garde_homme_peau_foncée:','ja':':男性の衛兵_濃い肌色:','ko':':남자_근위병_검은색_피부:','pt':':guarda_homem_pele_escura:','it':':guardia_uomo_carnagione_scura:','fa':':گارد_مرد_پوست_آبنوسی:','id':':penjaga_pria_warna_kulit_gelap:','zh':':男卫兵_较深肤色:','ru':':мужчина_гвардеец_очень_темный_тон_кожи:','tr':':erkek_muhafız_koyu_cilt_tonu:','ar':':رجل_حراسة_بشرة_بلون_غامق:'},'\U0001F482\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_guard_light_skin_tone:','status':fully_qualified,'E':4,'de':':wachmann_helle_hautfarbe:','es':':guardia_hombre_tono_de_piel_claro:','fr':':garde_homme_peau_claire:','ja':':男性の衛兵_薄い肌色:','ko':':남자_근위병_하얀_피부:','pt':':guarda_homem_pele_clara:','it':':guardia_uomo_carnagione_chiara:','fa':':گارد_مرد_پوست_سفید:','id':':penjaga_pria_warna_kulit_cerah:','zh':':男卫兵_较浅肤色:','ru':':мужчина-гвардеец_очень_светлыи_тон_кожи:','tr':':erkek_muhafız_açık_cilt_tonu:','ar':':رجل_حراسة_بشرة_بلون_فاتح:'},'\U0001F482\U0001F3FB\U0000200D\U00002642':{'en':':man_guard_light_skin_tone:','status':minimally_qualified,'E':4,'de':':wachsoldat_helle_hautfarbe:','es':':guardia_hombre_tono_de_piel_claro:','fr':':garde_homme_peau_claire:','ja':':男性の衛兵_薄い肌色:','ko':':남자_근위병_하얀_피부:','pt':':guarda_homem_pele_clara:','it':':guardia_uomo_carnagione_chiara:','fa':':گارد_مرد_پوست_سفید:','id':':penjaga_pria_warna_kulit_cerah:','zh':':男卫兵_较浅肤色:','ru':':мужчина_гвардеец_очень_светлый_тон_кожи:','tr':':erkek_muhafız_açık_cilt_tonu:','ar':':رجل_حراسة_بشرة_بلون_فاتح:'},'\U0001F482\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_guard_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':wachmann_mitteldunkle_hautfarbe:','es':':guardia_hombre_tono_de_piel_oscuro_medio:','fr':':garde_homme_peau_mate:','ja':':男性の衛兵_やや濃い肌色:','ko':':남자_근위병_진한_갈색_피부:','pt':':guarda_homem_pele_morena_escura:','it':':guardia_uomo_carnagione_abbastanza_scura:','fa':':گارد_مرد_پوست_گندمی:','id':':penjaga_pria_warna_kulit_gelap-sedang:','zh':':男卫兵_中等深肤色:','ru':':мужчина-гвардеец_темныи_тон_кожи:','tr':':erkek_muhafız_orta_koyu_cilt_tonu:','ar':':رجل_حراسة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F482\U0001F3FE\U0000200D\U00002642':{'en':':man_guard_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':wachsoldat_mitteldunkle_hautfarbe:','es':':guardia_hombre_tono_de_piel_oscuro_medio:','fr':':garde_homme_peau_mate:','ja':':男性の衛兵_やや濃い肌色:','ko':':남자_근위병_진한_갈색_피부:','pt':':guarda_homem_pele_morena_escura:','it':':guardia_uomo_carnagione_abbastanza_scura:','fa':':گارد_مرد_پوست_گندمی:','id':':penjaga_pria_warna_kulit_gelap-sedang:','zh':':男卫兵_中等深肤色:','ru':':мужчина_гвардеец_темный_тон_кожи:','tr':':erkek_muhafız_orta_koyu_cilt_tonu:','ar':':رجل_حراسة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F482\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_guard_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':wachmann_mittelhelle_hautfarbe:','es':':guardia_hombre_tono_de_piel_claro_medio:','fr':':garde_homme_peau_moyennement_claire:','ja':':男性の衛兵_やや薄い肌色:','ko':':남자_근위병_연한_갈색_피부:','pt':':guarda_homem_pele_morena_clara:','it':':guardia_uomo_carnagione_abbastanza_chiara:','fa':':گارد_مرد_پوست_روشن:','id':':penjaga_pria_warna_kulit_cerah-sedang:','zh':':男卫兵_中等浅肤色:','ru':':мужчина-гвардеец_светлыи_тон_кожи:','tr':':erkek_muhafız_orta_açık_cilt_tonu:','ar':':رجل_حراسة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F482\U0001F3FC\U0000200D\U00002642':{'en':':man_guard_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':wachsoldat_mittelhelle_hautfarbe:','es':':guardia_hombre_tono_de_piel_claro_medio:','fr':':garde_homme_peau_moyennement_claire:','ja':':男性の衛兵_やや薄い肌色:','ko':':남자_근위병_연한_갈색_피부:','pt':':guarda_homem_pele_morena_clara:','it':':guardia_uomo_carnagione_abbastanza_chiara:','fa':':گارد_مرد_پوست_روشن:','id':':penjaga_pria_warna_kulit_cerah-sedang:','zh':':男卫兵_中等浅肤色:','ru':':мужчина_гвардеец_светлый_тон_кожи:','tr':':erkek_muhafız_orta_açık_cilt_tonu:','ar':':رجل_حراسة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F482\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_guard_medium_skin_tone:','status':fully_qualified,'E':4,'de':':wachmann_mittlere_hautfarbe:','es':':guardia_hombre_tono_de_piel_medio:','fr':':garde_homme_peau_légèrement_mate:','ja':':男性の衛兵_中間の肌色:','ko':':남자_근위병_갈색_피부:','pt':':guarda_homem_pele_morena:','it':':guardia_uomo_carnagione_olivastra:','fa':':گارد_مرد_پوست_طلایی:','id':':penjaga_pria_warna_kulit_sedang:','zh':':男卫兵_中等肤色:','ru':':мужчина-гвардеец_среднии_тон_кожи:','tr':':erkek_muhafız_orta_cilt_tonu:','ar':':رجل_حراسة_بشرة_بلون_معتدل:'},'\U0001F482\U0001F3FD\U0000200D\U00002642':{'en':':man_guard_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':wachsoldat_mittlere_hautfarbe:','es':':guardia_hombre_tono_de_piel_medio:','fr':':garde_homme_peau_légèrement_mate:','ja':':男性の衛兵_中間の肌色:','ko':':남자_근위병_갈색_피부:','pt':':guarda_homem_pele_morena:','it':':guardia_uomo_carnagione_olivastra:','fa':':گارد_مرد_پوست_طلایی:','id':':penjaga_pria_warna_kulit_sedang:','zh':':男卫兵_中等肤色:','ru':':мужчина_гвардеец_средний_тон_кожи:','tr':':erkek_muhafız_orta_cilt_tonu:','ar':':رجل_حراسة_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U00002695\U0000FE0F':{'en':':man_health_worker:','status':fully_qualified,'E':4,'de':':arzt:','es':':profesional_sanitario_hombre:','fr':':professionnel_de_la_santé:','ja':':男性の医者:','ko':':남자_의료인:','pt':':homem_profissional_da_saúde:','it':':operatore_sanitario:','fa':':مددکار_بهداشت_مرد:','id':':pekerja_kesehatan_pria:','zh':':男医生:','ru':':врач:','tr':':erkek_sağlık_çalışanı:','ar':':عامل_بقطاع_الصحة:'},'\U0001F468\U0000200D\U00002695':{'en':':man_health_worker:','status':minimally_qualified,'E':4,'de':':arzt:','es':':profesional_sanitario_hombre:','fr':':professionnel_de_la_santé:','ja':':男性の医者:','ko':':남자_의료인:','pt':':homem_profissional_da_saúde:','it':':operatore_sanitario:','fa':':مددکار_بهداشت_مرد:','id':':pekerja_kesehatan_pria:','zh':':男医生:','ru':':врач:','tr':':erkek_sağlık_çalışanı:','ar':':عامل_بقطاع_الصحة:'},'\U0001F468\U0001F3FF\U0000200D\U00002695\U0000FE0F':{'en':':man_health_worker_dark_skin_tone:','status':fully_qualified,'E':4,'de':':arzt_dunkle_hautfarbe:','es':':profesional_sanitario_hombre_tono_de_piel_oscuro:','fr':':professionnel_de_la_santé_peau_foncée:','ja':':男性の医者_濃い肌色:','ko':':남자_의료인_검은색_피부:','pt':':homem_profissional_da_saúde_pele_escura:','it':':operatore_sanitario_carnagione_scura:','fa':':مددکار_بهداشت_مرد_پوست_آبنوسی:','id':':pekerja_kesehatan_pria_warna_kulit_gelap:','zh':':男医生_较深肤色:','ru':':врач_очень_темныи_тон_кожи:','tr':':erkek_sağlık_çalışanı_koyu_cilt_tonu:','ar':':عامل_بقطاع_الصحة_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FF\U0000200D\U00002695':{'en':':man_health_worker_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':arzt_dunkle_hautfarbe:','es':':profesional_sanitario_hombre_tono_de_piel_oscuro:','fr':':professionnel_de_la_santé_peau_foncée:','ja':':男性の医者_濃い肌色:','ko':':남자_의료인_검은색_피부:','pt':':homem_profissional_da_saúde_pele_escura:','it':':operatore_sanitario_carnagione_scura:','fa':':مددکار_بهداشت_مرد_پوست_آبنوسی:','id':':pekerja_kesehatan_pria_warna_kulit_gelap:','zh':':男医生_较深肤色:','ru':':врач_очень_темный_тон_кожи:','tr':':erkek_sağlık_çalışanı_koyu_cilt_tonu:','ar':':عامل_بقطاع_الصحة_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U00002695\U0000FE0F':{'en':':man_health_worker_light_skin_tone:','status':fully_qualified,'E':4,'de':':arzt_helle_hautfarbe:','es':':profesional_sanitario_hombre_tono_de_piel_claro:','fr':':professionnel_de_la_santé_peau_claire:','ja':':男性の医者_薄い肌色:','ko':':남자_의료인_하얀_피부:','pt':':homem_profissional_da_saúde_pele_clara:','it':':operatore_sanitario_carnagione_chiara:','fa':':مددکار_بهداشت_مرد_پوست_سفید:','id':':pekerja_kesehatan_pria_warna_kulit_cerah:','zh':':男医生_较浅肤色:','ru':':врач_очень_светлыи_тон_кожи:','tr':':erkek_sağlık_çalışanı_açık_cilt_tonu:','ar':':عامل_بقطاع_الصحة_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FB\U0000200D\U00002695':{'en':':man_health_worker_light_skin_tone:','status':minimally_qualified,'E':4,'de':':arzt_helle_hautfarbe:','es':':profesional_sanitario_hombre_tono_de_piel_claro:','fr':':professionnel_de_la_santé_peau_claire:','ja':':男性の医者_薄い肌色:','ko':':남자_의료인_하얀_피부:','pt':':homem_profissional_da_saúde_pele_clara:','it':':operatore_sanitario_carnagione_chiara:','fa':':مددکار_بهداشت_مرد_پوست_سفید:','id':':pekerja_kesehatan_pria_warna_kulit_cerah:','zh':':男医生_较浅肤色:','ru':':врач_очень_светлый_тон_кожи:','tr':':erkek_sağlık_çalışanı_açık_cilt_tonu:','ar':':عامل_بقطاع_الصحة_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U00002695\U0000FE0F':{'en':':man_health_worker_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':arzt_mitteldunkle_hautfarbe:','es':':profesional_sanitario_hombre_tono_de_piel_oscuro_medio:','fr':':professionnel_de_la_santé_peau_mate:','ja':':男性の医者_やや濃い肌色:','ko':':남자_의료인_진한_갈색_피부:','pt':':homem_profissional_da_saúde_pele_morena_escura:','it':':operatore_sanitario_carnagione_abbastanza_scura:','fa':':مددکار_بهداشت_مرد_پوست_گندمی:','id':':pekerja_kesehatan_pria_warna_kulit_gelap-sedang:','zh':':男医生_中等深肤色:','ru':':врач_темныи_тон_кожи:','tr':':erkek_sağlık_çalışanı_orta_koyu_cilt_tonu:','ar':':عامل_بقطاع_الصحة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FE\U0000200D\U00002695':{'en':':man_health_worker_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':arzt_mitteldunkle_hautfarbe:','es':':profesional_sanitario_hombre_tono_de_piel_oscuro_medio:','fr':':professionnel_de_la_santé_peau_mate:','ja':':男性の医者_やや濃い肌色:','ko':':남자_의료인_진한_갈색_피부:','pt':':homem_profissional_da_saúde_pele_morena_escura:','it':':operatore_sanitario_carnagione_abbastanza_scura:','fa':':مددکار_بهداشت_مرد_پوست_گندمی:','id':':pekerja_kesehatan_pria_warna_kulit_gelap-sedang:','zh':':男医生_中等深肤色:','ru':':врач_темный_тон_кожи:','tr':':erkek_sağlık_çalışanı_orta_koyu_cilt_tonu:','ar':':عامل_بقطاع_الصحة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U00002695\U0000FE0F':{'en':':man_health_worker_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':arzt_mittelhelle_hautfarbe:','es':':profesional_sanitario_hombre_tono_de_piel_claro_medio:','fr':':professionnel_de_la_santé_peau_moyennement_claire:','ja':':男性の医者_やや薄い肌色:','ko':':남자_의료인_연한_갈색_피부:','pt':':homem_profissional_da_saúde_pele_morena_clara:','it':':operatore_sanitario_carnagione_abbastanza_chiara:','fa':':مددکار_بهداشت_مرد_پوست_روشن:','id':':pekerja_kesehatan_pria_warna_kulit_cerah-sedang:','zh':':男医生_中等浅肤色:','ru':':врач_светлыи_тон_кожи:','tr':':erkek_sağlık_çalışanı_orta_açık_cilt_tonu:','ar':':عامل_بقطاع_الصحة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FC\U0000200D\U00002695':{'en':':man_health_worker_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':arzt_mittelhelle_hautfarbe:','es':':profesional_sanitario_hombre_tono_de_piel_claro_medio:','fr':':professionnel_de_la_santé_peau_moyennement_claire:','ja':':男性の医者_やや薄い肌色:','ko':':남자_의료인_연한_갈색_피부:','pt':':homem_profissional_da_saúde_pele_morena_clara:','it':':operatore_sanitario_carnagione_abbastanza_chiara:','fa':':مددکار_بهداشت_مرد_پوست_روشن:','id':':pekerja_kesehatan_pria_warna_kulit_cerah-sedang:','zh':':男医生_中等浅肤色:','ru':':врач_светлый_тон_кожи:','tr':':erkek_sağlık_çalışanı_orta_açık_cilt_tonu:','ar':':عامل_بقطاع_الصحة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U00002695\U0000FE0F':{'en':':man_health_worker_medium_skin_tone:','status':fully_qualified,'E':4,'de':':arzt_mittlere_hautfarbe:','es':':profesional_sanitario_hombre_tono_de_piel_medio:','fr':':professionnel_de_la_santé_peau_légèrement_mate:','ja':':男性の医者_中間の肌色:','ko':':남자_의료인_갈색_피부:','pt':':homem_profissional_da_saúde_pele_morena:','it':':operatore_sanitario_carnagione_olivastra:','fa':':مددکار_بهداشت_مرد_پوست_طلایی:','id':':pekerja_kesehatan_pria_warna_kulit_sedang:','zh':':男医生_中等肤色:','ru':':врач_среднии_тон_кожи:','tr':':erkek_sağlık_çalışanı_orta_cilt_tonu:','ar':':عامل_بقطاع_الصحة_بشرة_بلون_معتدل:'},'\U0001F468\U0001F3FD\U0000200D\U00002695':{'en':':man_health_worker_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':arzt_mittlere_hautfarbe:','es':':profesional_sanitario_hombre_tono_de_piel_medio:','fr':':professionnel_de_la_santé_peau_légèrement_mate:','ja':':男性の医者_中間の肌色:','ko':':남자_의료인_갈색_피부:','pt':':homem_profissional_da_saúde_pele_morena:','it':':operatore_sanitario_carnagione_olivastra:','fa':':مددکار_بهداشت_مرد_پوست_طلایی:','id':':pekerja_kesehatan_pria_warna_kulit_sedang:','zh':':男医生_中等肤色:','ru':':врач_средний_тон_кожи:','tr':':erkek_sağlık_çalışanı_orta_cilt_tonu:','ar':':عامل_بقطاع_الصحة_بشرة_بلون_معتدل:'},'\U0001F9D8\U0000200D\U00002642\U0000FE0F':{'en':':man_in_lotus_position:','status':fully_qualified,'E':5,'alias':[':lotus_position_man:'],'de':':mann_im_lotossitz:','es':':hombre_en_posición_de_loto:','fr':':homme_dans_la_posture_du_lotus:','ja':':ヨガのポーズをする男:','ko':':가부좌한_남자:','pt':':homem_na_posição_de_lótus:','it':':uomo_in_posizione_del_loto:','fa':':مرد_در_حالت_خلسه:','id':':pria_dalam_posisi_teratai:','zh':':盘腿的男人:','ru':':мужчина_в_позе_лотоса:','tr':':lotus_pozisyonundaki_erkek:','ar':':رجل_متربع:'},'\U0001F9D8\U0000200D\U00002642':{'en':':man_in_lotus_position:','status':minimally_qualified,'E':5,'alias':[':lotus_position_man:'],'de':':mann_im_lotossitz:','es':':hombre_en_posición_de_loto:','fr':':homme_dans_la_posture_du_lotus:','ja':':ヨガのポーズをする男:','ko':':가부좌한_남자:','pt':':homem_na_posição_de_lótus:','it':':uomo_in_posizione_del_loto:','fa':':مرد_در_حالت_خلسه:','id':':pria_dalam_posisi_teratai:','zh':':盘腿的男人:','ru':':мужчина_в_позе_лотоса:','tr':':lotus_pozisyonundaki_erkek:','ar':':رجل_متربع:'},'\U0001F9D8\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_in_lotus_position_dark_skin_tone:','status':fully_qualified,'E':5,'de':':mann_im_lotossitz_dunkle_hautfarbe:','es':':hombre_en_posición_de_loto_tono_de_piel_oscuro:','fr':':homme_dans_la_posture_du_lotus_peau_foncée:','ja':':ヨガのポーズをする男_濃い肌色:','ko':':가부좌한_남자_검은색_피부:','pt':':homem_na_posição_de_lótus_pele_escura:','it':':uomo_in_posizione_del_loto_carnagione_scura:','fa':':مرد_در_حالت_خلسه_پوست_آبنوسی:','id':':pria_dalam_posisi_teratai_warna_kulit_gelap:','zh':':盘腿的男人_较深肤色:','ru':':мужчина_в_позе_лотоса_очень_темныи_тон_кожи:','tr':':lotus_pozisyonundaki_erkek_koyu_cilt_tonu:','ar':':رجل_متربع_بشرة_بلون_غامق:'},'\U0001F9D8\U0001F3FF\U0000200D\U00002642':{'en':':man_in_lotus_position_dark_skin_tone:','status':minimally_qualified,'E':5,'de':':mann_im_lotossitz_dunkle_hautfarbe:','es':':hombre_en_posición_de_loto_tono_de_piel_oscuro:','fr':':homme_dans_la_posture_du_lotus_peau_foncée:','ja':':ヨガのポーズをする男_濃い肌色:','ko':':가부좌한_남자_검은색_피부:','pt':':homem_na_posição_de_lótus_pele_escura:','it':':uomo_in_posizione_del_loto_carnagione_scura:','fa':':مرد_در_حالت_خلسه_پوست_آبنوسی:','id':':pria_dalam_posisi_teratai_warna_kulit_gelap:','zh':':盘腿的男人_较深肤色:','ru':':мужчина_в_позе_лотоса_очень_темный_тон_кожи:','tr':':lotus_pozisyonundaki_erkek_koyu_cilt_tonu:','ar':':رجل_متربع_بشرة_بلون_غامق:'},'\U0001F9D8\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_in_lotus_position_light_skin_tone:','status':fully_qualified,'E':5,'de':':mann_im_lotossitz_helle_hautfarbe:','es':':hombre_en_posición_de_loto_tono_de_piel_claro:','fr':':homme_dans_la_posture_du_lotus_peau_claire:','ja':':ヨガのポーズをする男_薄い肌色:','ko':':가부좌한_남자_하얀_피부:','pt':':homem_na_posição_de_lótus_pele_clara:','it':':uomo_in_posizione_del_loto_carnagione_chiara:','fa':':مرد_در_حالت_خلسه_پوست_سفید:','id':':pria_dalam_posisi_teratai_warna_kulit_cerah:','zh':':盘腿的男人_较浅肤色:','ru':':мужчина_в_позе_лотоса_очень_светлыи_тон_кожи:','tr':':lotus_pozisyonundaki_erkek_açık_cilt_tonu:','ar':':رجل_متربع_بشرة_بلون_فاتح:'},'\U0001F9D8\U0001F3FB\U0000200D\U00002642':{'en':':man_in_lotus_position_light_skin_tone:','status':minimally_qualified,'E':5,'de':':mann_im_lotossitz_helle_hautfarbe:','es':':hombre_en_posición_de_loto_tono_de_piel_claro:','fr':':homme_dans_la_posture_du_lotus_peau_claire:','ja':':ヨガのポーズをする男_薄い肌色:','ko':':가부좌한_남자_하얀_피부:','pt':':homem_na_posição_de_lótus_pele_clara:','it':':uomo_in_posizione_del_loto_carnagione_chiara:','fa':':مرد_در_حالت_خلسه_پوست_سفید:','id':':pria_dalam_posisi_teratai_warna_kulit_cerah:','zh':':盘腿的男人_较浅肤色:','ru':':мужчина_в_позе_лотоса_очень_светлый_тон_кожи:','tr':':lotus_pozisyonundaki_erkek_açık_cilt_tonu:','ar':':رجل_متربع_بشرة_بلون_فاتح:'},'\U0001F9D8\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_in_lotus_position_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':mann_im_lotossitz_mitteldunkle_hautfarbe:','es':':hombre_en_posición_de_loto_tono_de_piel_oscuro_medio:','fr':':homme_dans_la_posture_du_lotus_peau_mate:','ja':':ヨガのポーズをする男_やや濃い肌色:','ko':':가부좌한_남자_진한_갈색_피부:','pt':':homem_na_posição_de_lótus_pele_morena_escura:','it':':uomo_in_posizione_del_loto_carnagione_abbastanza_scura:','fa':':مرد_در_حالت_خلسه_پوست_گندمی:','id':':pria_dalam_posisi_teratai_warna_kulit_gelap-sedang:','zh':':盘腿的男人_中等深肤色:','ru':':мужчина_в_позе_лотоса_темныи_тон_кожи:','tr':':lotus_pozisyonundaki_erkek_orta_koyu_cilt_tonu:','ar':':رجل_متربع_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D8\U0001F3FE\U0000200D\U00002642':{'en':':man_in_lotus_position_medium-dark_skin_tone:','status':minimally_qualified,'E':5,'de':':mann_im_lotossitz_mitteldunkle_hautfarbe:','es':':hombre_en_posición_de_loto_tono_de_piel_oscuro_medio:','fr':':homme_dans_la_posture_du_lotus_peau_mate:','ja':':ヨガのポーズをする男_やや濃い肌色:','ko':':가부좌한_남자_진한_갈색_피부:','pt':':homem_na_posição_de_lótus_pele_morena_escura:','it':':uomo_in_posizione_del_loto_carnagione_abbastanza_scura:','fa':':مرد_در_حالت_خلسه_پوست_گندمی:','id':':pria_dalam_posisi_teratai_warna_kulit_gelap-sedang:','zh':':盘腿的男人_中等深肤色:','ru':':мужчина_в_позе_лотоса_темный_тон_кожи:','tr':':lotus_pozisyonundaki_erkek_orta_koyu_cilt_tonu:','ar':':رجل_متربع_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D8\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_in_lotus_position_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':mann_im_lotossitz_mittelhelle_hautfarbe:','es':':hombre_en_posición_de_loto_tono_de_piel_claro_medio:','fr':':homme_dans_la_posture_du_lotus_peau_moyennement_claire:','ja':':ヨガのポーズをする男_やや薄い肌色:','ko':':가부좌한_남자_연한_갈색_피부:','pt':':homem_na_posição_de_lótus_pele_morena_clara:','it':':uomo_in_posizione_del_loto_carnagione_abbastanza_chiara:','fa':':مرد_در_حالت_خلسه_پوست_روشن:','id':':pria_dalam_posisi_teratai_warna_kulit_cerah-sedang:','zh':':盘腿的男人_中等浅肤色:','ru':':мужчина_в_позе_лотоса_светлыи_тон_кожи:','tr':':lotus_pozisyonundaki_erkek_orta_açık_cilt_tonu:','ar':':رجل_متربع_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D8\U0001F3FC\U0000200D\U00002642':{'en':':man_in_lotus_position_medium-light_skin_tone:','status':minimally_qualified,'E':5,'de':':mann_im_lotossitz_mittelhelle_hautfarbe:','es':':hombre_en_posición_de_loto_tono_de_piel_claro_medio:','fr':':homme_dans_la_posture_du_lotus_peau_moyennement_claire:','ja':':ヨガのポーズをする男_やや薄い肌色:','ko':':가부좌한_남자_연한_갈색_피부:','pt':':homem_na_posição_de_lótus_pele_morena_clara:','it':':uomo_in_posizione_del_loto_carnagione_abbastanza_chiara:','fa':':مرد_در_حالت_خلسه_پوست_روشن:','id':':pria_dalam_posisi_teratai_warna_kulit_cerah-sedang:','zh':':盘腿的男人_中等浅肤色:','ru':':мужчина_в_позе_лотоса_светлый_тон_кожи:','tr':':lotus_pozisyonundaki_erkek_orta_açık_cilt_tonu:','ar':':رجل_متربع_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D8\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_in_lotus_position_medium_skin_tone:','status':fully_qualified,'E':5,'de':':mann_im_lotossitz_mittlere_hautfarbe:','es':':hombre_en_posición_de_loto_tono_de_piel_medio:','fr':':homme_dans_la_posture_du_lotus_peau_légèrement_mate:','ja':':ヨガのポーズをする男_中間の肌色:','ko':':가부좌한_남자_갈색_피부:','pt':':homem_na_posição_de_lótus_pele_morena:','it':':uomo_in_posizione_del_loto_carnagione_olivastra:','fa':':مرد_در_حالت_خلسه_پوست_طلایی:','id':':pria_dalam_posisi_teratai_warna_kulit_sedang:','zh':':盘腿的男人_中等肤色:','ru':':мужчина_в_позе_лотоса_среднии_тон_кожи:','tr':':lotus_pozisyonundaki_erkek_orta_cilt_tonu:','ar':':رجل_متربع_بشرة_بلون_معتدل:'},'\U0001F9D8\U0001F3FD\U0000200D\U00002642':{'en':':man_in_lotus_position_medium_skin_tone:','status':minimally_qualified,'E':5,'de':':mann_im_lotossitz_mittlere_hautfarbe:','es':':hombre_en_posición_de_loto_tono_de_piel_medio:','fr':':homme_dans_la_posture_du_lotus_peau_légèrement_mate:','ja':':ヨガのポーズをする男_中間の肌色:','ko':':가부좌한_남자_갈색_피부:','pt':':homem_na_posição_de_lótus_pele_morena:','it':':uomo_in_posizione_del_loto_carnagione_olivastra:','fa':':مرد_در_حالت_خلسه_پوست_طلایی:','id':':pria_dalam_posisi_teratai_warna_kulit_sedang:','zh':':盘腿的男人_中等肤色:','ru':':мужчина_в_позе_лотоса_средний_тон_кожи:','tr':':lotus_pozisyonundaki_erkek_orta_cilt_tonu:','ar':':رجل_متربع_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U0001F9BD':{'en':':man_in_manual_wheelchair:','status':fully_qualified,'E':12,'de':':mann_in_manuellem_rollstuhl:','es':':hombre_en_silla_de_ruedas_manual:','fr':':homme_en_fauteuil_roulant_manuel:','ja':':手動式車椅子の男性:','ko':':수동_휠체어를_탄_남자:','pt':':homem_em_cadeira_de_rodas_manual:','it':':uomo_su_sedia_a_rotelle_manuale:','fa':':مرد_در_صندلی_چرخ_دار_دستی:','id':':pria_dengan_kursi_roda_manual:','zh':':坐手动轮椅的男人:','ru':':мужчина_в_ручном_кресле_коляске:','tr':':tekerlekli_sandalyede_erkek:','ar':':رجل_على_كرسي_متحرك_يدوي:'},'\U0001F468\U0001F3FF\U0000200D\U0001F9BD':{'en':':man_in_manual_wheelchair_dark_skin_tone:','status':fully_qualified,'E':12,'de':':mann_in_manuellem_rollstuhl_dunkle_hautfarbe:','es':':hombre_en_silla_de_ruedas_manual_tono_de_piel_oscuro:','fr':':homme_en_fauteuil_roulant_manuel_peau_foncée:','ja':':手動式車椅子の男性_濃い肌色:','ko':':수동_휠체어를_탄_남자_검은색_피부:','pt':':homem_em_cadeira_de_rodas_manual_pele_escura:','it':':uomo_su_sedia_a_rotelle_manuale_carnagione_scura:','fa':':مرد_در_صندلی_چرخ_دار_دستی_پوست_آبنوسی:','id':':pria_dengan_kursi_roda_manual_warna_kulit_gelap:','zh':':坐手动轮椅的男人_较深肤色:','ru':':мужчина_в_ручном_кресле_коляске_очень_темный_тон_кожи:','tr':':tekerlekli_sandalyede_erkek_koyu_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_يدوي_بشرة_بلون_غامق:'},'\U0001F468\U0000200D\U0001F9BD\U0000200D\U000027A1\U0000FE0F':{'en':':man_in_manual_wheelchair_facing_right:','status':fully_qualified,'E':15.1,'de':':mann_in_manuellem_rollstuhl_nach_rechts:','es':':hombre_en_silla_de_ruedas_manual_hacia_la_derecha:','fr':':homme_en_fauteuil_roulant_manuel_vers_la_droite:','ja':':手動式車椅子の男性_右向き:','ko':':오른쪽을_향한_수동_휠체어를_탄_남자:','pt':':homem_em_cadeira_de_rodas_manual_de_frente_para_a_direita:','it':':uomo_su_sedia_a_rotelle_manuale_a_destra:','fa':':مرد_در_صندلی_چرخ_دار_دستی_به_سمت_راست:','id':':pria_dengan_kursi_roda_manual_menghadap_kanan:','zh':':坐手动轮椅的男人面向右边:','ru':':мужчина_в_ручном_кресле_коляске_направленные_вправо:','tr':':sağa_dönük_tekerlekli_sandalyede_erkek:','ar':':رجل_على_كرسي_متحرك_يدوي_متجه_لليمين:'},'\U0001F468\U0000200D\U0001F9BD\U0000200D\U000027A1':{'en':':man_in_manual_wheelchair_facing_right:','status':minimally_qualified,'E':15.1,'de':':mann_in_manuellem_rollstuhl_nach_rechts:','es':':hombre_en_silla_de_ruedas_manual_hacia_la_derecha:','fr':':homme_en_fauteuil_roulant_manuel_vers_la_droite:','ja':':手動式車椅子の男性_右向き:','ko':':오른쪽을_향한_수동_휠체어를_탄_남자:','pt':':homem_em_cadeira_de_rodas_manual_de_frente_para_a_direita:','it':':uomo_su_sedia_a_rotelle_manuale_a_destra:','fa':':مرد_در_صندلی_چرخ_دار_دستی_به_سمت_راست:','id':':pria_dengan_kursi_roda_manual_menghadap_kanan:','zh':':坐手动轮椅的男人面向右边:','ru':':мужчина_в_ручном_кресле_коляске_направленные_вправо:','tr':':sağa_dönük_tekerlekli_sandalyede_erkek:','ar':':رجل_على_كرسي_متحرك_يدوي_متجه_لليمين:'},'\U0001F468\U0001F3FF\U0000200D\U0001F9BD\U0000200D\U000027A1\U0000FE0F':{'en':':man_in_manual_wheelchair_facing_right_dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':mann_in_manuellem_rollstuhl_nach_rechts_dunkle_hautfarbe:','es':':hombre_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_oscuro:','fr':':homme_en_fauteuil_roulant_manuel_vers_la_droite_peau_foncée:','ja':':手動式車椅子の男性_右向き_濃い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_남자_검은색_피부:','pt':':homem_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_escura:','it':':uomo_su_sedia_a_rotelle_manuale_a_destra_carnagione_scura:','fa':':مرد_در_صندلی_چرخ_دار_دستی_به_سمت_راستپوست_آبنوسی:','id':':pria_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_gelap:','zh':':坐手动轮椅的男人面向右边_较深肤色:','ru':':мужчина_в_ручном_кресле_коляске_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_erkek_koyu_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_يدوي_متجه_لليمين_بلون_غامق:'},'\U0001F468\U0001F3FF\U0000200D\U0001F9BD\U0000200D\U000027A1':{'en':':man_in_manual_wheelchair_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':mann_in_manuellem_rollstuhl_nach_rechts_dunkle_hautfarbe:','es':':hombre_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_oscuro:','fr':':homme_en_fauteuil_roulant_manuel_vers_la_droite_peau_foncée:','ja':':手動式車椅子の男性_右向き_濃い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_남자_검은색_피부:','pt':':homem_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_escura:','it':':uomo_su_sedia_a_rotelle_manuale_a_destra_carnagione_scura:','fa':':مرد_در_صندلی_چرخ_دار_دستی_به_سمت_راستپوست_آبنوسی:','id':':pria_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_gelap:','zh':':坐手动轮椅的男人面向右边_较深肤色:','ru':':мужчина_в_ручном_кресле_коляске_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_erkek_koyu_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_يدوي_متجه_لليمين_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U0001F9BD\U0000200D\U000027A1\U0000FE0F':{'en':':man_in_manual_wheelchair_facing_right_light_skin_tone:','status':fully_qualified,'E':15.1,'de':':mann_in_manuellem_rollstuhl_nach_rechts_helle_hautfarbe:','es':':hombre_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_claro:','fr':':homme_en_fauteuil_roulant_manuel_vers_la_droite_peau_claire:','ja':':手動式車椅子の男性_右向き_薄い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_남자_하얀_피부:','pt':':homem_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_clara:','it':':uomo_su_sedia_a_rotelle_manuale_a_destra_carnagione_chiara:','fa':':مرد_در_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_سفید:','id':':pria_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_cerah:','zh':':坐手动轮椅的男人面向右边_较浅肤色:','ru':':мужчина_в_ручном_кресле_коляске_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_erkek_açık_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_يدوي_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FB\U0000200D\U0001F9BD\U0000200D\U000027A1':{'en':':man_in_manual_wheelchair_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':mann_in_manuellem_rollstuhl_nach_rechts_helle_hautfarbe:','es':':hombre_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_claro:','fr':':homme_en_fauteuil_roulant_manuel_vers_la_droite_peau_claire:','ja':':手動式車椅子の男性_右向き_薄い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_남자_하얀_피부:','pt':':homem_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_clara:','it':':uomo_su_sedia_a_rotelle_manuale_a_destra_carnagione_chiara:','fa':':مرد_در_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_سفید:','id':':pria_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_cerah:','zh':':坐手动轮椅的男人面向右边_较浅肤色:','ru':':мужчина_в_ручном_кресле_коляске_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_erkek_açık_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_يدوي_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F9BD\U0000200D\U000027A1\U0000FE0F':{'en':':man_in_manual_wheelchair_facing_right_medium-dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':mann_in_manuellem_rollstuhl_nach_rechts_mitteldunkle_hautfarbe:','es':':hombre_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':homme_en_fauteuil_roulant_manuel_vers_la_droite_peau_mate:','ja':':手動式車椅子の男性_右向き_やや濃い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_남자_진한_갈색_피부:','pt':':homem_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_morena_escura:','it':':uomo_su_sedia_a_rotelle_manuale_a_destra_carnagione_abbastanza_scura:','fa':':مرد_در_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_گندمی:','id':':pria_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':坐手动轮椅的男人面向右边_中等深肤色:','ru':':мужчина_в_ручном_кресле_коляске_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_erkek_orta_koyu_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_يدوي_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F468\U0001F3FE\U0000200D\U0001F9BD\U0000200D\U000027A1':{'en':':man_in_manual_wheelchair_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':mann_in_manuellem_rollstuhl_nach_rechts_mitteldunkle_hautfarbe:','es':':hombre_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':homme_en_fauteuil_roulant_manuel_vers_la_droite_peau_mate:','ja':':手動式車椅子の男性_右向き_やや濃い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_남자_진한_갈색_피부:','pt':':homem_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_morena_escura:','it':':uomo_su_sedia_a_rotelle_manuale_a_destra_carnagione_abbastanza_scura:','fa':':مرد_در_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_گندمی:','id':':pria_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':坐手动轮椅的男人面向右边_中等深肤色:','ru':':мужчина_в_ручном_кресле_коляске_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_erkek_orta_koyu_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_يدوي_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F468\U0001F3FC\U0000200D\U0001F9BD\U0000200D\U000027A1\U0000FE0F':{'en':':man_in_manual_wheelchair_facing_right_medium-light_skin_tone:','status':fully_qualified,'E':15.1,'de':':mann_in_manuellem_rollstuhl_nach_rechts_mittelhelle_hautfarbe:','es':':hombre_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':homme_en_fauteuil_roulant_manuel_vers_la_droite_peau_moyennement_claire:','ja':':手動式車椅子の男性_右向き_やや薄い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_남자_연한_갈색_피부:','pt':':homem_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_morena_clara:','it':':uomo_su_sedia_a_rotelle_manuale_a_destra_carnagione_abbastanza_chiara:','fa':':مرد_در_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_روشن:','id':':pria_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':坐手动轮椅的男人面向右边_中等浅肤色:','ru':':мужчина_в_ручном_кресле_коляске_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_tekerlekli_sandalyede_erkek_orta_açık_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_يدوي_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FC\U0000200D\U0001F9BD\U0000200D\U000027A1':{'en':':man_in_manual_wheelchair_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':mann_in_manuellem_rollstuhl_nach_rechts_mittelhelle_hautfarbe:','es':':hombre_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':homme_en_fauteuil_roulant_manuel_vers_la_droite_peau_moyennement_claire:','ja':':手動式車椅子の男性_右向き_やや薄い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_남자_연한_갈색_피부:','pt':':homem_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_morena_clara:','it':':uomo_su_sedia_a_rotelle_manuale_a_destra_carnagione_abbastanza_chiara:','fa':':مرد_در_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_روشن:','id':':pria_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':坐手动轮椅的男人面向右边_中等浅肤色:','ru':':мужчина_в_ручном_кресле_коляске_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_tekerlekli_sandalyede_erkek_orta_açık_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_يدوي_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F9BD\U0000200D\U000027A1\U0000FE0F':{'en':':man_in_manual_wheelchair_facing_right_medium_skin_tone:','status':fully_qualified,'E':15.1,'de':':mann_in_manuellem_rollstuhl_nach_rechts_mittlere_hautfarbe:','es':':hombre_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_medio:','fr':':homme_en_fauteuil_roulant_manuel_vers_la_droite_peau_légèrement_mate:','ja':':手動式車椅子の男性_右向き_中間の肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_남자_갈색_피부:','pt':':homem_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_morena:','it':':uomo_su_sedia_a_rotelle_manuale_a_destra_carnagione_olivastra:','fa':':مرد_در_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_طلایی:','id':':pria_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_sedang:','zh':':坐手动轮椅的男人面向右边_中等肤色:','ru':':мужчина_в_ручном_кресле_коляске_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_erkek_orta_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_يدوي_متجه_لليمين_بلون_معتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F9BD\U0000200D\U000027A1':{'en':':man_in_manual_wheelchair_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':mann_in_manuellem_rollstuhl_nach_rechts_mittlere_hautfarbe:','es':':hombre_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_medio:','fr':':homme_en_fauteuil_roulant_manuel_vers_la_droite_peau_légèrement_mate:','ja':':手動式車椅子の男性_右向き_中間の肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_남자_갈색_피부:','pt':':homem_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_morena:','it':':uomo_su_sedia_a_rotelle_manuale_a_destra_carnagione_olivastra:','fa':':مرد_در_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_طلایی:','id':':pria_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_sedang:','zh':':坐手动轮椅的男人面向右边_中等肤色:','ru':':мужчина_в_ручном_кресле_коляске_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_erkek_orta_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_يدوي_متجه_لليمين_بلون_معتدل:'},'\U0001F468\U0001F3FB\U0000200D\U0001F9BD':{'en':':man_in_manual_wheelchair_light_skin_tone:','status':fully_qualified,'E':12,'de':':mann_in_manuellem_rollstuhl_helle_hautfarbe:','es':':hombre_en_silla_de_ruedas_manual_tono_de_piel_claro:','fr':':homme_en_fauteuil_roulant_manuel_peau_claire:','ja':':手動式車椅子の男性_薄い肌色:','ko':':수동_휠체어를_탄_남자_하얀_피부:','pt':':homem_em_cadeira_de_rodas_manual_pele_clara:','it':':uomo_su_sedia_a_rotelle_manuale_carnagione_chiara:','fa':':مرد_در_صندلی_چرخ_دار_دستی_پوست_سفید:','id':':pria_dengan_kursi_roda_manual_warna_kulit_cerah:','zh':':坐手动轮椅的男人_较浅肤色:','ru':':мужчина_в_ручном_кресле_коляске_очень_светлый_тон_кожи:','tr':':tekerlekli_sandalyede_erkek_açık_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_يدوي_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F9BD':{'en':':man_in_manual_wheelchair_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':mann_in_manuellem_rollstuhl_mitteldunkle_hautfarbe:','es':':hombre_en_silla_de_ruedas_manual_tono_de_piel_oscuro_medio:','fr':':homme_en_fauteuil_roulant_manuel_peau_mate:','ja':':手動式車椅子の男性_やや濃い肌色:','ko':':수동_휠체어를_탄_남자_진한_갈색_피부:','pt':':homem_em_cadeira_de_rodas_manual_pele_morena_escura:','it':':uomo_su_sedia_a_rotelle_manuale_carnagione_abbastanza_scura:','fa':':مرد_در_صندلی_چرخ_دار_دستی_پوست_گندمی:','id':':pria_dengan_kursi_roda_manual_warna_kulit_gelap-sedang:','zh':':坐手动轮椅的男人_中等深肤色:','ru':':мужчина_в_ручном_кресле_коляске_темный_тон_кожи:','tr':':tekerlekli_sandalyede_erkek_orta_koyu_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_يدوي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U0001F9BD':{'en':':man_in_manual_wheelchair_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':mann_in_manuellem_rollstuhl_mittelhelle_hautfarbe:','es':':hombre_en_silla_de_ruedas_manual_tono_de_piel_claro_medio:','fr':':homme_en_fauteuil_roulant_manuel_peau_moyennement_claire:','ja':':手動式車椅子の男性_やや薄い肌色:','ko':':수동_휠체어를_탄_남자_연한_갈색_피부:','pt':':homem_em_cadeira_de_rodas_manual_pele_morena_clara:','it':':uomo_su_sedia_a_rotelle_manuale_carnagione_abbastanza_chiara:','fa':':مرد_در_صندلی_چرخ_دار_دستی_پوست_روشن:','id':':pria_dengan_kursi_roda_manual_warna_kulit_cerah-sedang:','zh':':坐手动轮椅的男人_中等浅肤色:','ru':':мужчина_в_ручном_кресле_коляске_светлый_тон_кожи:','tr':':tekerlekli_sandalyede_erkek_orta_açık_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_يدوي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F9BD':{'en':':man_in_manual_wheelchair_medium_skin_tone:','status':fully_qualified,'E':12,'de':':mann_in_manuellem_rollstuhl_mittlere_hautfarbe:','es':':hombre_en_silla_de_ruedas_manual_tono_de_piel_medio:','fr':':homme_en_fauteuil_roulant_manuel_peau_légèrement_mate:','ja':':手動式車椅子の男性_中間の肌色:','ko':':수동_휠체어를_탄_남자_갈색_피부:','pt':':homem_em_cadeira_de_rodas_manual_pele_morena:','it':':uomo_su_sedia_a_rotelle_manuale_carnagione_olivastra:','fa':':مرد_در_صندلی_چرخ_دار_دستی_پوست_طلایی:','id':':pria_dengan_kursi_roda_manual_warna_kulit_sedang:','zh':':坐手动轮椅的男人_中等肤色:','ru':':мужчина_в_ручном_кресле_коляске_средний_тон_кожи:','tr':':tekerlekli_sandalyede_erkek_orta_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_يدوي_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U0001F9BC':{'en':':man_in_motorized_wheelchair:','status':fully_qualified,'E':12,'de':':mann_in_elektrischem_rollstuhl:','es':':hombre_en_silla_de_ruedas_eléctrica:','fr':':homme_en_fauteuil_motorisé:','ja':':電動車椅子の男性:','ko':':전동_휠체어를_탄_남자:','pt':':homem_em_cadeira_de_rodas_motorizada:','it':':uomo_su_sedia_a_rotelle_motorizzata:','fa':':مرد_در_صندلی_چرخ_دار_برقی:','id':':pria_dengan_kursi_roda_bermotor:','zh':':坐电动轮椅的男人:','ru':':мужчина_в_электрическом_кресле_коляске:','tr':':elektrikli_tekerlekli_sandalyede_erkek:','ar':':رجل_على_كرسي_متحرك_كهربائي:'},'\U0001F468\U0001F3FF\U0000200D\U0001F9BC':{'en':':man_in_motorized_wheelchair_dark_skin_tone:','status':fully_qualified,'E':12,'de':':mann_in_elektrischem_rollstuhl_dunkle_hautfarbe:','es':':hombre_en_silla_de_ruedas_eléctrica_tono_de_piel_oscuro:','fr':':homme_en_fauteuil_motorisé_peau_foncée:','ja':':電動車椅子の男性_濃い肌色:','ko':':전동_휠체어를_탄_남자_검은색_피부:','pt':':homem_em_cadeira_de_rodas_motorizada_pele_escura:','it':':uomo_su_sedia_a_rotelle_motorizzata_carnagione_scura:','fa':':مرد_در_صندلی_چرخ_دار_برقی_پوست_آبنوسی:','id':':pria_dengan_kursi_roda_bermotor_warna_kulit_gelap:','zh':':坐电动轮椅的男人_较深肤色:','ru':':мужчина_в_электрическом_кресле_коляске_очень_темный_тон_кожи:','tr':':elektrikli_tekerlekli_sandalyede_erkek_koyu_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_كهربائي_بشرة_بلون_غامق:'},'\U0001F468\U0000200D\U0001F9BC\U0000200D\U000027A1\U0000FE0F':{'en':':man_in_motorized_wheelchair_facing_right:','status':fully_qualified,'E':15.1,'de':':mann_in_elektrischem_rollstuhl_nach_rechts:','es':':hombre_en_silla_de_ruedas_eléctrica_hacia_la_derecha:','fr':':homme_en_fauteuil_motorisé_vers_la_droite:','ja':':電動車椅子の男性_右向き:','ko':':오른쪽을_향한_전동_휠체어를_탄_남자:','pt':':homem_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita:','it':':uomo_su_sedia_a_rotelle_motorizzata_a_destra:','fa':':مرد_در_صندلی_چرخ_دار_برقی_به_سمت_راست:','id':':pria_dengan_kursi_roda_bermotor_menghadap_kanan:','zh':':坐电动轮椅的男人面向右边:','ru':':мужчина_в_электрическом_кресле_коляске_направленные_вправо:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_erkek:','ar':':رجل_على_كرسي_متحرك_كهربائي_متجه_لليمين:'},'\U0001F468\U0000200D\U0001F9BC\U0000200D\U000027A1':{'en':':man_in_motorized_wheelchair_facing_right:','status':minimally_qualified,'E':15.1,'de':':mann_in_elektrischem_rollstuhl_nach_rechts:','es':':hombre_en_silla_de_ruedas_eléctrica_hacia_la_derecha:','fr':':homme_en_fauteuil_motorisé_vers_la_droite:','ja':':電動車椅子の男性_右向き:','ko':':오른쪽을_향한_전동_휠체어를_탄_남자:','pt':':homem_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita:','it':':uomo_su_sedia_a_rotelle_motorizzata_a_destra:','fa':':مرد_در_صندلی_چرخ_دار_برقی_به_سمت_راست:','id':':pria_dengan_kursi_roda_bermotor_menghadap_kanan:','zh':':坐电动轮椅的男人面向右边:','ru':':мужчина_в_электрическом_кресле_коляске_направленные_вправо:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_erkek:','ar':':رجل_على_كرسي_متحرك_كهربائي_متجه_لليمين:'},'\U0001F468\U0001F3FF\U0000200D\U0001F9BC\U0000200D\U000027A1\U0000FE0F':{'en':':man_in_motorized_wheelchair_facing_right_dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':mann_in_elektrischem_rollstuhl_nach_rechts_dunkle_hautfarbe:','es':':hombre_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_oscuro:','fr':':homme_en_fauteuil_motorisé_vers_la_droite_peau_foncée:','ja':':電動車椅子の男性_右向き_濃い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_남자_검은색_피부:','pt':':homem_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_escura:','it':':uomo_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_scura:','fa':':مرد_در_صندلی_چرخ_دار_برقی_به_سمت_راستپوست_آبنوسی:','id':':pria_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_gelap:','zh':':坐电动轮椅的男人面向右边_较深肤色:','ru':':мужчина_в_электрическом_кресле_коляске_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_erkek_koyu_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_كهربائي_متجه_لليمين_بلون_غامق:'},'\U0001F468\U0001F3FF\U0000200D\U0001F9BC\U0000200D\U000027A1':{'en':':man_in_motorized_wheelchair_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':mann_in_elektrischem_rollstuhl_nach_rechts_dunkle_hautfarbe:','es':':hombre_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_oscuro:','fr':':homme_en_fauteuil_motorisé_vers_la_droite_peau_foncée:','ja':':電動車椅子の男性_右向き_濃い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_남자_검은색_피부:','pt':':homem_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_escura:','it':':uomo_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_scura:','fa':':مرد_در_صندلی_چرخ_دار_برقی_به_سمت_راستپوست_آبنوسی:','id':':pria_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_gelap:','zh':':坐电动轮椅的男人面向右边_较深肤色:','ru':':мужчина_в_электрическом_кресле_коляске_направленные_очень_темныи_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_erkek_koyu_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_كهربائي_متجه_لليمين_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U0001F9BC\U0000200D\U000027A1\U0000FE0F':{'en':':man_in_motorized_wheelchair_facing_right_light_skin_tone:','status':fully_qualified,'E':15.1,'de':':mann_in_elektrischem_rollstuhl_nach_rechts_helle_hautfarbe:','es':':hombre_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_claro:','fr':':homme_en_fauteuil_motorisé_vers_la_droite_peau_claire:','ja':':電動車椅子の男性_右向き_薄い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_남자_하얀_피부:','pt':':homem_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_clara:','it':':uomo_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_chiara:','fa':':مرد_در_صندلی_چرخ_دار_برقی_به_سمت_راست_پوست_سفید:','id':':pria_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_cerah:','zh':':坐电动轮椅的男人面向右边_较浅肤色:','ru':':мужчина_в_электрическом_кресле_коляске_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_erkek_açık_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_كهربائي_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FB\U0000200D\U0001F9BC\U0000200D\U000027A1':{'en':':man_in_motorized_wheelchair_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':mann_in_elektrischem_rollstuhl_nach_rechts_helle_hautfarbe:','es':':hombre_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_claro:','fr':':homme_en_fauteuil_motorisé_vers_la_droite_peau_claire:','ja':':電動車椅子の男性_右向き_薄い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_남자_하얀_피부:','pt':':homem_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_clara:','it':':uomo_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_chiara:','fa':':مرد_در_صندلی_چرخ_دار_برقی_به_سمت_راست_پوست_سفید:','id':':pria_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_cerah:','zh':':坐电动轮椅的男人面向右边_较浅肤色:','ru':':мужчина_в_электрическом_кресле_коляске_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_erkek_açık_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_كهربائي_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F9BC\U0000200D\U000027A1\U0000FE0F':{'en':':man_in_motorized_wheelchair_facing_right_medium-dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':mann_in_elektrischem_rollstuhl_nach_rechts_mitteldunkle_hautfarbe:','es':':hombre_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':homme_en_fauteuil_motorisé_vers_la_droite_peau_mate:','ja':':電動車椅子の男性_右向き_やや濃い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_남자_진한_갈색_피부:','pt':':homem_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_morena_escura:','it':':uomo_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_abbastanza_scura:','fa':':مرد_در_صندلی_چرخ_دار_برقی_به_سمت_راست_پوست_گندمی:','id':':pria_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':坐电动轮椅的男人面向右边_中等深肤色:','ru':':мужчина_в_электрическом_кресле_коляске_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_erkek_orta_koyu_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_كهربائي_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F468\U0001F3FE\U0000200D\U0001F9BC\U0000200D\U000027A1':{'en':':man_in_motorized_wheelchair_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':mann_in_elektrischem_rollstuhl_nach_rechts_mitteldunkle_hautfarbe:','es':':hombre_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':homme_en_fauteuil_motorisé_vers_la_droite_peau_mate:','ja':':電動車椅子の男性_右向き_やや濃い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_남자_진한_갈색_피부:','pt':':homem_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_morena_escura:','it':':uomo_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_abbastanza_scura:','fa':':مرد_در_صندلی_چرخ_دار_برقی_به_سمت_راست_پوست_گندمی:','id':':pria_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':坐电动轮椅的男人面向右边_中等深肤色:','ru':':мужчина_в_электрическом_кресле_коляске_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_erkek_orta_koyu_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_كهربائي_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F468\U0001F3FC\U0000200D\U0001F9BC\U0000200D\U000027A1\U0000FE0F':{'en':':man_in_motorized_wheelchair_facing_right_medium-light_skin_tone:','status':fully_qualified,'E':15.1,'de':':mann_in_elektrischem_rollstuhl_nach_rechts_mittelhelle_hautfarbe:','es':':hombre_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':homme_en_fauteuil_motorisé_vers_la_droite_peau_moyennement_claire:','ja':':電動車椅子の男性_右向き_やや薄い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_남자_연한_갈색_피부:','pt':':homem_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_morena_clara:','it':':uomo_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_abbastanza_chiara:','fa':':مرد_در_صندلی_چرخ_دار_برقی_به_سمت_راست_پوست_روشن:','id':':pria_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':坐电动轮椅的男人面向右边_中等浅肤色:','ru':':мужчина_в_электрическом_кресле_коляске_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_erkek_orta_açık_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_كهربائي_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FC\U0000200D\U0001F9BC\U0000200D\U000027A1':{'en':':man_in_motorized_wheelchair_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':mann_in_elektrischem_rollstuhl_nach_rechts_mittelhelle_hautfarbe:','es':':hombre_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':homme_en_fauteuil_motorisé_vers_la_droite_peau_moyennement_claire:','ja':':電動車椅子の男性_右向き_やや薄い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_남자_연한_갈색_피부:','pt':':homem_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_morena_clara:','it':':uomo_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_abbastanza_chiara:','fa':':مرد_در_صندلی_چرخ_دار_برقی_به_سمت_راست_پوست_روشن:','id':':pria_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':坐电动轮椅的男人面向右边_中等浅肤色:','ru':':мужчина_в_электрическом_кресле_коляске_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_erkek_orta_açık_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_كهربائي_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F9BC\U0000200D\U000027A1\U0000FE0F':{'en':':man_in_motorized_wheelchair_facing_right_medium_skin_tone:','status':fully_qualified,'E':15.1,'de':':mann_in_elektrischem_rollstuhl_nach_rechts_mittlere_hautfarbe:','es':':hombre_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_medio:','fr':':homme_en_fauteuil_motorisé_vers_la_droite_peau_légèrement_mate:','ja':':電動車椅子の男性_右向き_中間の肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_남자_갈색_피부:','pt':':homem_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_morena:','it':':uomo_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_olivastra:','fa':':مرد_در_صندلی_چرخ_دار_برقی_به_سمت_راست_پوست_طلایی:','id':':pria_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_sedang:','zh':':坐电动轮椅的男人面向右边_中等肤色:','ru':':мужчина_в_электрическом_кресле_коляске_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_erkek_orta_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_كهربائي_متجه_لليمين_بلون_معتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F9BC\U0000200D\U000027A1':{'en':':man_in_motorized_wheelchair_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':mann_in_elektrischem_rollstuhl_nach_rechts_mittlere_hautfarbe:','es':':hombre_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_medio:','fr':':homme_en_fauteuil_motorisé_vers_la_droite_peau_légèrement_mate:','ja':':電動車椅子の男性_右向き_中間の肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_남자_갈색_피부:','pt':':homem_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_morena:','it':':uomo_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_olivastra:','fa':':مرد_در_صندلی_چرخ_دار_برقی_به_سمت_راست_پوست_طلایی:','id':':pria_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_sedang:','zh':':坐电动轮椅的男人面向右边_中等肤色:','ru':':мужчина_в_электрическом_кресле_коляске_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_erkek_orta_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_كهربائي_متجه_لليمين_بلون_معتدل:'},'\U0001F468\U0001F3FB\U0000200D\U0001F9BC':{'en':':man_in_motorized_wheelchair_light_skin_tone:','status':fully_qualified,'E':12,'de':':mann_in_elektrischem_rollstuhl_helle_hautfarbe:','es':':hombre_en_silla_de_ruedas_eléctrica_tono_de_piel_claro:','fr':':homme_en_fauteuil_motorisé_peau_claire:','ja':':電動車椅子の男性_薄い肌色:','ko':':전동_휠체어를_탄_남자_하얀_피부:','pt':':homem_em_cadeira_de_rodas_motorizada_pele_clara:','it':':uomo_su_sedia_a_rotelle_motorizzata_carnagione_chiara:','fa':':مرد_در_صندلی_چرخ_دار_برقی_پوست_سفید:','id':':pria_dengan_kursi_roda_bermotor_warna_kulit_cerah:','zh':':坐电动轮椅的男人_较浅肤色:','ru':':мужчина_в_электрическом_кресле_коляске_очень_светлый_тон_кожи:','tr':':elektrikli_tekerlekli_sandalyede_erkek_açık_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_كهربائي_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F9BC':{'en':':man_in_motorized_wheelchair_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':mann_in_elektrischem_rollstuhl_mitteldunkle_hautfarbe:','es':':hombre_en_silla_de_ruedas_eléctrica_tono_de_piel_oscuro_medio:','fr':':homme_en_fauteuil_motorisé_peau_mate:','ja':':電動車椅子の男性_やや濃い肌色:','ko':':전동_휠체어를_탄_남자_진한_갈색_피부:','pt':':homem_em_cadeira_de_rodas_motorizada_pele_morena_escura:','it':':uomo_su_sedia_a_rotelle_motorizzata_carnagione_abbastanza_scura:','fa':':مرد_در_صندلی_چرخ_دار_برقی_پوست_گندمی:','id':':pria_dengan_kursi_roda_bermotor_warna_kulit_gelap-sedang:','zh':':坐电动轮椅的男人_中等深肤色:','ru':':мужчина_в_электрическом_кресле_коляске_темный_тон_кожи:','tr':':elektrikli_tekerlekli_sandalyede_erkek_orta_koyu_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_كهربائي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U0001F9BC':{'en':':man_in_motorized_wheelchair_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':mann_in_elektrischem_rollstuhl_mittelhelle_hautfarbe:','es':':hombre_en_silla_de_ruedas_eléctrica_tono_de_piel_claro_medio:','fr':':homme_en_fauteuil_motorisé_peau_moyennement_claire:','ja':':電動車椅子の男性_やや薄い肌色:','ko':':전동_휠체어를_탄_남자_연한_갈색_피부:','pt':':homem_em_cadeira_de_rodas_motorizada_pele_morena_clara:','it':':uomo_su_sedia_a_rotelle_motorizzata_carnagione_abbastanza_chiara:','fa':':مرد_در_صندلی_چرخ_دار_برقی_پوست_روشن:','id':':pria_dengan_kursi_roda_bermotor_warna_kulit_cerah-sedang:','zh':':坐电动轮椅的男人_中等浅肤色:','ru':':мужчина_в_электрическом_кресле_коляске_светлый_тон_кожи:','tr':':elektrikli_tekerlekli_sandalyede_erkek_orta_açık_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_كهربائي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F9BC':{'en':':man_in_motorized_wheelchair_medium_skin_tone:','status':fully_qualified,'E':12,'de':':mann_in_elektrischem_rollstuhl_mittlere_hautfarbe:','es':':hombre_en_silla_de_ruedas_eléctrica_tono_de_piel_medio:','fr':':homme_en_fauteuil_motorisé_peau_légèrement_mate:','ja':':電動車椅子の男性_中間の肌色:','ko':':전동_휠체어를_탄_남자_갈색_피부:','pt':':homem_em_cadeira_de_rodas_motorizada_pele_morena:','it':':uomo_su_sedia_a_rotelle_motorizzata_carnagione_olivastra:','fa':':مرد_در_صندلی_چرخ_دار_برقی_پوست_طلایی:','id':':pria_dengan_kursi_roda_bermotor_warna_kulit_sedang:','zh':':坐电动轮椅的男人_中等肤色:','ru':':мужчина_в_электрическом_кресле_коляске_средний_тон_кожи:','tr':':elektrikli_tekerlekli_sandalyede_erkek_orta_cilt_tonu:','ar':':رجل_على_كرسي_متحرك_كهربائي_بشرة_بلون_معتدل:'},'\U0001F9D6\U0000200D\U00002642\U0000FE0F':{'en':':man_in_steamy_room:','status':fully_qualified,'E':5,'alias':[':sauna_man:'],'de':':mann_in_dampfsauna:','es':':hombre_en_una_sauna:','fr':':homme_au_hammam:','ja':':サウナに入る男:','ko':':사우나_하는_남자:','pt':':homem_na_sauna:','it':':uomo_in_sauna:','fa':':مرد_در_حمام_بخار:','id':':pria_di_ruangan_beruap:','zh':':蒸房里的男人:','ru':':мужчина_в_бане:','tr':':buhar_odasındaki_erkek:','ar':':رجل_في_حمام_بخار:'},'\U0001F9D6\U0000200D\U00002642':{'en':':man_in_steamy_room:','status':minimally_qualified,'E':5,'alias':[':sauna_man:'],'de':':mann_in_dampfsauna:','es':':hombre_en_una_sauna:','fr':':homme_au_hammam:','ja':':サウナに入る男:','ko':':사우나_하는_남자:','pt':':homem_na_sauna:','it':':uomo_in_sauna:','fa':':مرد_در_حمام_بخار:','id':':pria_di_ruangan_beruap:','zh':':蒸房里的男人:','ru':':мужчина_в_бане:','tr':':buhar_odasındaki_erkek:','ar':':رجل_في_حمام_بخار:'},'\U0001F9D6\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_in_steamy_room_dark_skin_tone:','status':fully_qualified,'E':5,'de':':mann_in_dampfsauna_dunkle_hautfarbe:','es':':hombre_en_una_sauna_tono_de_piel_oscuro:','fr':':homme_au_hammam_peau_foncée:','ja':':サウナに入る男_濃い肌色:','ko':':사우나_하는_남자_검은색_피부:','pt':':homem_na_sauna_pele_escura:','it':':uomo_in_sauna_carnagione_scura:','fa':':مرد_در_حمام_بخار_پوست_آبنوسی:','id':':pria_di_ruangan_beruap_warna_kulit_gelap:','zh':':蒸房里的男人_较深肤色:','ru':':мужчина_в_бане_очень_темныи_тон_кожи:','tr':':buhar_odasındaki_erkek_koyu_cilt_tonu:','ar':':رجل_في_حمام_بخار_بشرة_بلون_غامق:'},'\U0001F9D6\U0001F3FF\U0000200D\U00002642':{'en':':man_in_steamy_room_dark_skin_tone:','status':minimally_qualified,'E':5,'de':':mann_in_dampfsauna_dunkle_hautfarbe:','es':':hombre_en_una_sauna_tono_de_piel_oscuro:','fr':':homme_au_hammam_peau_foncée:','ja':':サウナに入る男_濃い肌色:','ko':':사우나_하는_남자_검은색_피부:','pt':':homem_na_sauna_pele_escura:','it':':uomo_in_sauna_carnagione_scura:','fa':':مرد_در_حمام_بخار_پوست_آبنوسی:','id':':pria_di_ruangan_beruap_warna_kulit_gelap:','zh':':蒸房里的男人_较深肤色:','ru':':мужчина_в_бане_очень_темный_тон_кожи:','tr':':buhar_odasındaki_erkek_koyu_cilt_tonu:','ar':':رجل_في_حمام_بخار_بشرة_بلون_غامق:'},'\U0001F9D6\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_in_steamy_room_light_skin_tone:','status':fully_qualified,'E':5,'de':':mann_in_dampfsauna_helle_hautfarbe:','es':':hombre_en_una_sauna_tono_de_piel_claro:','fr':':homme_au_hammam_peau_claire:','ja':':サウナに入る男_薄い肌色:','ko':':사우나_하는_남자_하얀_피부:','pt':':homem_na_sauna_pele_clara:','it':':uomo_in_sauna_carnagione_chiara:','fa':':مرد_در_حمام_بخار_پوست_سفید:','id':':pria_di_ruangan_beruap_warna_kulit_cerah:','zh':':蒸房里的男人_较浅肤色:','ru':':мужчина_в_бане_очень_светлыи_тон_кожи:','tr':':buhar_odasındaki_erkek_açık_cilt_tonu:','ar':':رجل_في_حمام_بخار_بشرة_بلون_فاتح:'},'\U0001F9D6\U0001F3FB\U0000200D\U00002642':{'en':':man_in_steamy_room_light_skin_tone:','status':minimally_qualified,'E':5,'de':':mann_in_dampfsauna_helle_hautfarbe:','es':':hombre_en_una_sauna_tono_de_piel_claro:','fr':':homme_au_hammam_peau_claire:','ja':':サウナに入る男_薄い肌色:','ko':':사우나_하는_남자_하얀_피부:','pt':':homem_na_sauna_pele_clara:','it':':uomo_in_sauna_carnagione_chiara:','fa':':مرد_در_حمام_بخار_پوست_سفید:','id':':pria_di_ruangan_beruap_warna_kulit_cerah:','zh':':蒸房里的男人_较浅肤色:','ru':':мужчина_в_бане_очень_светлый_тон_кожи:','tr':':buhar_odasındaki_erkek_açık_cilt_tonu:','ar':':رجل_في_حمام_بخار_بشرة_بلون_فاتح:'},'\U0001F9D6\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_in_steamy_room_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':mann_in_dampfsauna_mitteldunkle_hautfarbe:','es':':hombre_en_una_sauna_tono_de_piel_oscuro_medio:','fr':':homme_au_hammam_peau_mate:','ja':':サウナに入る男_やや濃い肌色:','ko':':사우나_하는_남자_진한_갈색_피부:','pt':':homem_na_sauna_pele_morena_escura:','it':':uomo_in_sauna_carnagione_abbastanza_scura:','fa':':مرد_در_حمام_بخار_پوست_گندمی:','id':':pria_di_ruangan_beruap_warna_kulit_gelap-sedang:','zh':':蒸房里的男人_中等深肤色:','ru':':мужчина_в_бане_темныи_тон_кожи:','tr':':buhar_odasındaki_erkek_orta_koyu_cilt_tonu:','ar':':رجل_في_حمام_بخار_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D6\U0001F3FE\U0000200D\U00002642':{'en':':man_in_steamy_room_medium-dark_skin_tone:','status':minimally_qualified,'E':5,'de':':mann_in_dampfsauna_mitteldunkle_hautfarbe:','es':':hombre_en_una_sauna_tono_de_piel_oscuro_medio:','fr':':homme_au_hammam_peau_mate:','ja':':サウナに入る男_やや濃い肌色:','ko':':사우나_하는_남자_진한_갈색_피부:','pt':':homem_na_sauna_pele_morena_escura:','it':':uomo_in_sauna_carnagione_abbastanza_scura:','fa':':مرد_در_حمام_بخار_پوست_گندمی:','id':':pria_di_ruangan_beruap_warna_kulit_gelap-sedang:','zh':':蒸房里的男人_中等深肤色:','ru':':мужчина_в_бане_темный_тон_кожи:','tr':':buhar_odasındaki_erkek_orta_koyu_cilt_tonu:','ar':':رجل_في_حمام_بخار_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D6\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_in_steamy_room_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':mann_in_dampfsauna_mittelhelle_hautfarbe:','es':':hombre_en_una_sauna_tono_de_piel_claro_medio:','fr':':homme_au_hammam_peau_moyennement_claire:','ja':':サウナに入る男_やや薄い肌色:','ko':':사우나_하는_남자_연한_갈색_피부:','pt':':homem_na_sauna_pele_morena_clara:','it':':uomo_in_sauna_carnagione_abbastanza_chiara:','fa':':مرد_در_حمام_بخار_پوست_روشن:','id':':pria_di_ruangan_beruap_warna_kulit_cerah-sedang:','zh':':蒸房里的男人_中等浅肤色:','ru':':мужчина_в_бане_светлыи_тон_кожи:','tr':':buhar_odasındaki_erkek_orta_açık_cilt_tonu:','ar':':رجل_في_حمام_بخار_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D6\U0001F3FC\U0000200D\U00002642':{'en':':man_in_steamy_room_medium-light_skin_tone:','status':minimally_qualified,'E':5,'de':':mann_in_dampfsauna_mittelhelle_hautfarbe:','es':':hombre_en_una_sauna_tono_de_piel_claro_medio:','fr':':homme_au_hammam_peau_moyennement_claire:','ja':':サウナに入る男_やや薄い肌色:','ko':':사우나_하는_남자_연한_갈색_피부:','pt':':homem_na_sauna_pele_morena_clara:','it':':uomo_in_sauna_carnagione_abbastanza_chiara:','fa':':مرد_در_حمام_بخار_پوست_روشن:','id':':pria_di_ruangan_beruap_warna_kulit_cerah-sedang:','zh':':蒸房里的男人_中等浅肤色:','ru':':мужчина_в_бане_светлый_тон_кожи:','tr':':buhar_odasındaki_erkek_orta_açık_cilt_tonu:','ar':':رجل_في_حمام_بخار_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D6\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_in_steamy_room_medium_skin_tone:','status':fully_qualified,'E':5,'de':':mann_in_dampfsauna_mittlere_hautfarbe:','es':':hombre_en_una_sauna_tono_de_piel_medio:','fr':':homme_au_hammam_peau_légèrement_mate:','ja':':サウナに入る男_中間の肌色:','ko':':사우나_하는_남자_갈색_피부:','pt':':homem_na_sauna_pele_morena:','it':':uomo_in_sauna_carnagione_olivastra:','fa':':مرد_در_حمام_بخار_پوست_طلایی:','id':':pria_di_ruangan_beruap_warna_kulit_sedang:','zh':':蒸房里的男人_中等肤色:','ru':':мужчина_в_бане_среднии_тон_кожи:','tr':':buhar_odasındaki_erkek_orta_cilt_tonu:','ar':':رجل_في_حمام_بخار_بشرة_بلون_معتدل:'},'\U0001F9D6\U0001F3FD\U0000200D\U00002642':{'en':':man_in_steamy_room_medium_skin_tone:','status':minimally_qualified,'E':5,'de':':mann_in_dampfsauna_mittlere_hautfarbe:','es':':hombre_en_una_sauna_tono_de_piel_medio:','fr':':homme_au_hammam_peau_légèrement_mate:','ja':':サウナに入る男_中間の肌色:','ko':':사우나_하는_남자_갈색_피부:','pt':':homem_na_sauna_pele_morena:','it':':uomo_in_sauna_carnagione_olivastra:','fa':':مرد_در_حمام_بخار_پوست_طلایی:','id':':pria_di_ruangan_beruap_warna_kulit_sedang:','zh':':蒸房里的男人_中等肤色:','ru':':мужчина_в_бане_средний_тон_кожи:','tr':':buhar_odasındaki_erkek_orta_cilt_tonu:','ar':':رجل_في_حمام_بخار_بشرة_بلون_معتدل:'},'\U0001F935\U0000200D\U00002642\U0000FE0F':{'en':':man_in_tuxedo:','status':fully_qualified,'E':13,'de':':mann_im_smoking:','es':':hombre_con_esmoquin:','fr':':homme_en_smoking:','ja':':タキシードの男性:','ko':':턱시도를_입은_남자:','pt':':homem_de_smoking:','it':':uomo_in_smoking:','fa':':مرد_با_کت_و_شلوار_و_پاپیون:','id':':pria_dewasa_bertuksedo:','zh':':穿礼服的男人:','ru':':мужчина_в_смокинге:','tr':':smokinli_erkek:','ar':':رجل_يرتدي_بدلة_رسمية:'},'\U0001F935\U0000200D\U00002642':{'en':':man_in_tuxedo:','status':minimally_qualified,'E':13,'de':':mann_im_smoking:','es':':hombre_con_esmoquin:','fr':':homme_en_smoking:','ja':':タキシードの男性:','ko':':턱시도를_입은_남자:','pt':':homem_de_smoking:','it':':uomo_in_smoking:','fa':':مرد_با_کت_و_شلوار_و_پاپیون:','id':':pria_dewasa_bertuksedo:','zh':':穿礼服的男人:','ru':':мужчина_в_смокинге:','tr':':smokinli_erkek:','ar':':رجل_يرتدي_بدلة_رسمية:'},'\U0001F935\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_in_tuxedo_dark_skin_tone:','status':fully_qualified,'E':13,'de':':mann_im_smoking_dunkle_hautfarbe:','es':':hombre_con_esmoquin_tono_de_piel_oscuro:','fr':':homme_en_smoking_peau_foncée:','ja':':タキシードの男性_濃い肌色:','ko':':턱시도를_입은_남자_검은색_피부:','pt':':homem_de_smoking_pele_escura:','it':':uomo_in_smoking_carnagione_scura:','fa':':مرد_با_کت_و_شلوار_و_پاپیون_پوست_آبنوسی:','id':':pria_dewasa_bertuksedo_warna_kulit_gelap:','zh':':穿礼服的男人_较深肤色:','ru':':мужчина_в_смокинге_очень_темныи_тон_кожи:','tr':':smokinli_erkek_koyu_cilt_tonu:','ar':':رجل_يرتدي_بدلة_رسمية_بشرة_بلون_غامق:'},'\U0001F935\U0001F3FF\U0000200D\U00002642':{'en':':man_in_tuxedo_dark_skin_tone:','status':minimally_qualified,'E':13,'de':':mann_im_smoking_dunkle_hautfarbe:','es':':hombre_con_esmoquin_tono_de_piel_oscuro:','fr':':homme_en_smoking_peau_foncée:','ja':':タキシードの男性_濃い肌色:','ko':':턱시도를_입은_남자_검은색_피부:','pt':':homem_de_smoking_pele_escura:','it':':uomo_in_smoking_carnagione_scura:','fa':':مرد_با_کت_و_شلوار_و_پاپیون_پوست_آبنوسی:','id':':pria_dewasa_bertuksedo_warna_kulit_gelap:','zh':':穿礼服的男人_较深肤色:','ru':':мужчина_в_смокинге_очень_темный_тон_кожи:','tr':':smokinli_erkek_koyu_cilt_tonu:','ar':':رجل_يرتدي_بدلة_رسمية_بشرة_بلون_غامق:'},'\U0001F935\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_in_tuxedo_light_skin_tone:','status':fully_qualified,'E':13,'de':':mann_im_smoking_helle_hautfarbe:','es':':hombre_con_esmoquin_tono_de_piel_claro:','fr':':homme_en_smoking_peau_claire:','ja':':タキシードの男性_薄い肌色:','ko':':턱시도를_입은_남자_하얀_피부:','pt':':homem_de_smoking_pele_clara:','it':':uomo_in_smoking_carnagione_chiara:','fa':':مرد_با_کت_و_شلوار_و_پاپیون_پوست_سفید:','id':':pria_dewasa_bertuksedo_warna_kulit_cerah:','zh':':穿礼服的男人_较浅肤色:','ru':':мужчина_в_смокинге_очень_светлыи_тон_кожи:','tr':':smokinli_erkek_açık_cilt_tonu:','ar':':رجل_يرتدي_بدلة_رسمية_بشرة_بلون_فاتح:'},'\U0001F935\U0001F3FB\U0000200D\U00002642':{'en':':man_in_tuxedo_light_skin_tone:','status':minimally_qualified,'E':13,'de':':mann_im_smoking_helle_hautfarbe:','es':':hombre_con_esmoquin_tono_de_piel_claro:','fr':':homme_en_smoking_peau_claire:','ja':':タキシードの男性_薄い肌色:','ko':':턱시도를_입은_남자_하얀_피부:','pt':':homem_de_smoking_pele_clara:','it':':uomo_in_smoking_carnagione_chiara:','fa':':مرد_با_کت_و_شلوار_و_پاپیون_پوست_سفید:','id':':pria_dewasa_bertuksedo_warna_kulit_cerah:','zh':':穿礼服的男人_较浅肤色:','ru':':мужчина_в_смокинге_очень_светлый_тон_кожи:','tr':':smokinli_erkek_açık_cilt_tonu:','ar':':رجل_يرتدي_بدلة_رسمية_بشرة_بلون_فاتح:'},'\U0001F935\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_in_tuxedo_medium-dark_skin_tone:','status':fully_qualified,'E':13,'de':':mann_im_smoking_mitteldunkle_hautfarbe:','es':':hombre_con_esmoquin_tono_de_piel_oscuro_medio:','fr':':homme_en_smoking_peau_mate:','ja':':タキシードの男性_やや濃い肌色:','ko':':턱시도를_입은_남자_진한_갈색_피부:','pt':':homem_de_smoking_pele_morena_escura:','it':':uomo_in_smoking_carnagione_abbastanza_scura:','fa':':مرد_با_کت_و_شلوار_و_پاپیون_پوست_گندمی:','id':':pria_dewasa_bertuksedo_warna_kulit_gelap-sedang:','zh':':穿礼服的男人_中等深肤色:','ru':':мужчина_в_смокинге_темныи_тон_кожи:','tr':':smokinli_erkek_orta_koyu_cilt_tonu:','ar':':رجل_يرتدي_بدلة_رسمية_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F935\U0001F3FE\U0000200D\U00002642':{'en':':man_in_tuxedo_medium-dark_skin_tone:','status':minimally_qualified,'E':13,'de':':mann_im_smoking_mitteldunkle_hautfarbe:','es':':hombre_con_esmoquin_tono_de_piel_oscuro_medio:','fr':':homme_en_smoking_peau_mate:','ja':':タキシードの男性_やや濃い肌色:','ko':':턱시도를_입은_남자_진한_갈색_피부:','pt':':homem_de_smoking_pele_morena_escura:','it':':uomo_in_smoking_carnagione_abbastanza_scura:','fa':':مرد_با_کت_و_شلوار_و_پاپیون_پوست_گندمی:','id':':pria_dewasa_bertuksedo_warna_kulit_gelap-sedang:','zh':':穿礼服的男人_中等深肤色:','ru':':мужчина_в_смокинге_темный_тон_кожи:','tr':':smokinli_erkek_orta_koyu_cilt_tonu:','ar':':رجل_يرتدي_بدلة_رسمية_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F935\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_in_tuxedo_medium-light_skin_tone:','status':fully_qualified,'E':13,'de':':mann_im_smoking_mittelhelle_hautfarbe:','es':':hombre_con_esmoquin_tono_de_piel_claro_medio:','fr':':homme_en_smoking_peau_moyennement_claire:','ja':':タキシードの男性_やや薄い肌色:','ko':':턱시도를_입은_남자_연한_갈색_피부:','pt':':homem_de_smoking_pele_morena_clara:','it':':uomo_in_smoking_carnagione_abbastanza_chiara:','fa':':مرد_با_کت_و_شلوار_و_پاپیون_پوست_روشن:','id':':pria_dewasa_bertuksedo_warna_kulit_cerah-sedang:','zh':':穿礼服的男人_中等浅肤色:','ru':':мужчина_в_смокинге_светлыи_тон_кожи:','tr':':smokinli_erkek_orta_açık_cilt_tonu:','ar':':رجل_يرتدي_بدلة_رسمية_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F935\U0001F3FC\U0000200D\U00002642':{'en':':man_in_tuxedo_medium-light_skin_tone:','status':minimally_qualified,'E':13,'de':':mann_im_smoking_mittelhelle_hautfarbe:','es':':hombre_con_esmoquin_tono_de_piel_claro_medio:','fr':':homme_en_smoking_peau_moyennement_claire:','ja':':タキシードの男性_やや薄い肌色:','ko':':턱시도를_입은_남자_연한_갈색_피부:','pt':':homem_de_smoking_pele_morena_clara:','it':':uomo_in_smoking_carnagione_abbastanza_chiara:','fa':':مرد_با_کت_و_شلوار_و_پاپیون_پوست_روشن:','id':':pria_dewasa_bertuksedo_warna_kulit_cerah-sedang:','zh':':穿礼服的男人_中等浅肤色:','ru':':мужчина_в_смокинге_светлый_тон_кожи:','tr':':smokinli_erkek_orta_açık_cilt_tonu:','ar':':رجل_يرتدي_بدلة_رسمية_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F935\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_in_tuxedo_medium_skin_tone:','status':fully_qualified,'E':13,'de':':mann_im_smoking_mittlere_hautfarbe:','es':':hombre_con_esmoquin_tono_de_piel_medio:','fr':':homme_en_smoking_peau_légèrement_mate:','ja':':タキシードの男性_中間の肌色:','ko':':턱시도를_입은_남자_갈색_피부:','pt':':homem_de_smoking_pele_morena:','it':':uomo_in_smoking_carnagione_olivastra:','fa':':مرد_با_کت_و_شلوار_و_پاپیون_پوست_طلایی:','id':':pria_dewasa_bertuksedo_warna_kulit_sedang:','zh':':穿礼服的男人_中等肤色:','ru':':мужчина_в_смокинге_среднии_тон_кожи:','tr':':smokinli_erkek_orta_cilt_tonu:','ar':':رجل_يرتدي_بدلة_رسمية_بشرة_بلون_معتدل:'},'\U0001F935\U0001F3FD\U0000200D\U00002642':{'en':':man_in_tuxedo_medium_skin_tone:','status':minimally_qualified,'E':13,'de':':mann_im_smoking_mittlere_hautfarbe:','es':':hombre_con_esmoquin_tono_de_piel_medio:','fr':':homme_en_smoking_peau_légèrement_mate:','ja':':タキシードの男性_中間の肌色:','ko':':턱시도를_입은_남자_갈색_피부:','pt':':homem_de_smoking_pele_morena:','it':':uomo_in_smoking_carnagione_olivastra:','fa':':مرد_با_کت_و_شلوار_و_پاپیون_پوست_طلایی:','id':':pria_dewasa_bertuksedo_warna_kulit_sedang:','zh':':穿礼服的男人_中等肤色:','ru':':мужчина_в_смокинге_средний_тон_кожи:','tr':':smokinli_erkek_orta_cilt_tonu:','ar':':رجل_يرتدي_بدلة_رسمية_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U00002696\U0000FE0F':{'en':':man_judge:','status':fully_qualified,'E':4,'de':':richter:','es':':fiscal_hombre:','fr':':juge_homme:','ja':':男性の裁判官:','ko':':남자_판사:','pt':':juiz:','it':':giudice_uomo:','fa':':قاضی_مرد:','id':':hakim_pria:','zh':':男法官:','ru':':мужчина-судья:','tr':':erkek_hakim:','ar':':قاض:'},'\U0001F468\U0000200D\U00002696':{'en':':man_judge:','status':minimally_qualified,'E':4,'de':':richter:','es':':fiscal_hombre:','fr':':juge_homme:','ja':':男性の裁判官:','ko':':남자_판사:','pt':':juiz:','it':':giudice_uomo:','fa':':قاضی_مرد:','id':':hakim_pria:','zh':':男法官:','ru':':мужчина_судья:','tr':':erkek_hakim:','ar':':قاض:'},'\U0001F468\U0001F3FF\U0000200D\U00002696\U0000FE0F':{'en':':man_judge_dark_skin_tone:','status':fully_qualified,'E':4,'de':':richter_dunkle_hautfarbe:','es':':fiscal_hombre_tono_de_piel_oscuro:','fr':':juge_homme_peau_foncée:','ja':':男性の裁判官_濃い肌色:','ko':':남자_판사_검은색_피부:','pt':':juiz_pele_escura:','it':':giudice_uomo_carnagione_scura:','fa':':قاضی_مرد_پوست_آبنوسی:','id':':hakim_pria_warna_kulit_gelap:','zh':':男法官_较深肤色:','ru':':мужчина-судья_очень_темныи_тон_кожи:','tr':':erkek_hakim_koyu_cilt_tonu:','ar':':قاض_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FF\U0000200D\U00002696':{'en':':man_judge_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':richter_dunkle_hautfarbe:','es':':fiscal_hombre_tono_de_piel_oscuro:','fr':':juge_homme_peau_foncée:','ja':':男性の裁判官_濃い肌色:','ko':':남자_판사_검은색_피부:','pt':':juiz_pele_escura:','it':':giudice_uomo_carnagione_scura:','fa':':قاضی_مرد_پوست_آبنوسی:','id':':hakim_pria_warna_kulit_gelap:','zh':':男法官_较深肤色:','ru':':мужчина_судья_очень_темный_тон_кожи:','tr':':erkek_hakim_koyu_cilt_tonu:','ar':':قاض_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U00002696\U0000FE0F':{'en':':man_judge_light_skin_tone:','status':fully_qualified,'E':4,'de':':richter_helle_hautfarbe:','es':':fiscal_hombre_tono_de_piel_claro:','fr':':juge_homme_peau_claire:','ja':':男性の裁判官_薄い肌色:','ko':':남자_판사_하얀_피부:','pt':':juiz_pele_clara:','it':':giudice_uomo_carnagione_chiara:','fa':':قاضی_مرد_پوست_سفید:','id':':hakim_pria_warna_kulit_cerah:','zh':':男法官_较浅肤色:','ru':':мужчина-судья_очень_светлыи_тон_кожи:','tr':':erkek_hakim_açık_cilt_tonu:','ar':':قاض_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FB\U0000200D\U00002696':{'en':':man_judge_light_skin_tone:','status':minimally_qualified,'E':4,'de':':richter_helle_hautfarbe:','es':':fiscal_hombre_tono_de_piel_claro:','fr':':juge_homme_peau_claire:','ja':':男性の裁判官_薄い肌色:','ko':':남자_판사_하얀_피부:','pt':':juiz_pele_clara:','it':':giudice_uomo_carnagione_chiara:','fa':':قاضی_مرد_پوست_سفید:','id':':hakim_pria_warna_kulit_cerah:','zh':':男法官_较浅肤色:','ru':':мужчина_судья_очень_светлый_тон_кожи:','tr':':erkek_hakim_açık_cilt_tonu:','ar':':قاض_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U00002696\U0000FE0F':{'en':':man_judge_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':richter_mitteldunkle_hautfarbe:','es':':fiscal_hombre_tono_de_piel_oscuro_medio:','fr':':juge_homme_peau_mate:','ja':':男性の裁判官_やや濃い肌色:','ko':':남자_판사_진한_갈색_피부:','pt':':juiz_pele_morena_escura:','it':':giudice_uomo_carnagione_abbastanza_scura:','fa':':قاضی_مرد_پوست_گندمی:','id':':hakim_pria_warna_kulit_gelap-sedang:','zh':':男法官_中等深肤色:','ru':':мужчина-судья_темныи_тон_кожи:','tr':':erkek_hakim_orta_koyu_cilt_tonu:','ar':':قاض_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FE\U0000200D\U00002696':{'en':':man_judge_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':richter_mitteldunkle_hautfarbe:','es':':fiscal_hombre_tono_de_piel_oscuro_medio:','fr':':juge_homme_peau_mate:','ja':':男性の裁判官_やや濃い肌色:','ko':':남자_판사_진한_갈색_피부:','pt':':juiz_pele_morena_escura:','it':':giudice_uomo_carnagione_abbastanza_scura:','fa':':قاضی_مرد_پوست_گندمی:','id':':hakim_pria_warna_kulit_gelap-sedang:','zh':':男法官_中等深肤色:','ru':':мужчина_судья_темный_тон_кожи:','tr':':erkek_hakim_orta_koyu_cilt_tonu:','ar':':قاض_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U00002696\U0000FE0F':{'en':':man_judge_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':richter_mittelhelle_hautfarbe:','es':':fiscal_hombre_tono_de_piel_claro_medio:','fr':':juge_homme_peau_moyennement_claire:','ja':':男性の裁判官_やや薄い肌色:','ko':':남자_판사_연한_갈색_피부:','pt':':juiz_pele_morena_clara:','it':':giudice_uomo_carnagione_abbastanza_chiara:','fa':':قاضی_مرد_پوست_روشن:','id':':hakim_pria_warna_kulit_cerah-sedang:','zh':':男法官_中等浅肤色:','ru':':мужчина-судья_светлыи_тон_кожи:','tr':':erkek_hakim_orta_açık_cilt_tonu:','ar':':قاض_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FC\U0000200D\U00002696':{'en':':man_judge_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':richter_mittelhelle_hautfarbe:','es':':fiscal_hombre_tono_de_piel_claro_medio:','fr':':juge_homme_peau_moyennement_claire:','ja':':男性の裁判官_やや薄い肌色:','ko':':남자_판사_연한_갈색_피부:','pt':':juiz_pele_morena_clara:','it':':giudice_uomo_carnagione_abbastanza_chiara:','fa':':قاضی_مرد_پوست_روشن:','id':':hakim_pria_warna_kulit_cerah-sedang:','zh':':男法官_中等浅肤色:','ru':':мужчина_судья_светлый_тон_кожи:','tr':':erkek_hakim_orta_açık_cilt_tonu:','ar':':قاض_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U00002696\U0000FE0F':{'en':':man_judge_medium_skin_tone:','status':fully_qualified,'E':4,'de':':richter_mittlere_hautfarbe:','es':':fiscal_hombre_tono_de_piel_medio:','fr':':juge_homme_peau_légèrement_mate:','ja':':男性の裁判官_中間の肌色:','ko':':남자_판사_갈색_피부:','pt':':juiz_pele_morena:','it':':giudice_uomo_carnagione_olivastra:','fa':':قاضی_مرد_پوست_طلایی:','id':':hakim_pria_warna_kulit_sedang:','zh':':男法官_中等肤色:','ru':':мужчина-судья_среднии_тон_кожи:','tr':':erkek_hakim_orta_cilt_tonu:','ar':':قاض_بشرة_بلون_معتدل:'},'\U0001F468\U0001F3FD\U0000200D\U00002696':{'en':':man_judge_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':richter_mittlere_hautfarbe:','es':':fiscal_hombre_tono_de_piel_medio:','fr':':juge_homme_peau_légèrement_mate:','ja':':男性の裁判官_中間の肌色:','ko':':남자_판사_갈색_피부:','pt':':juiz_pele_morena:','it':':giudice_uomo_carnagione_olivastra:','fa':':قاضی_مرد_پوست_طلایی:','id':':hakim_pria_warna_kulit_sedang:','zh':':男法官_中等肤色:','ru':':мужчина_судья_средний_тон_кожи:','tr':':erkek_hakim_orta_cilt_tonu:','ar':':قاض_بشرة_بلون_معتدل:'},'\U0001F939\U0000200D\U00002642\U0000FE0F':{'en':':man_juggling:','status':fully_qualified,'E':4,'de':':jongleur:','es':':hombre_haciendo_malabares:','fr':':jongleur:','ja':':ジャグリングをする男:','ko':':저글링하는_남자:','pt':':homem_malabarista:','it':':giocoliere_uomo:','fa':':مرد_ژانگولربازی_می_کند:','id':':pria_melempar:','zh':':男生抛接杂耍:','ru':':мужчина-жонглер:','tr':':jonglör_erkek:','ar':':لاعب_قذف_الكرات:'},'\U0001F939\U0000200D\U00002642':{'en':':man_juggling:','status':minimally_qualified,'E':4,'de':':jongleur:','es':':hombre_haciendo_malabares:','fr':':jongleur:','ja':':ジャグリングをする男:','ko':':저글링하는_남자:','pt':':homem_malabarista:','it':':giocoliere_uomo:','fa':':مرد_ژانگولربازی_می_کند:','id':':pria_melempar:','zh':':男生抛接杂耍:','ru':':мужчина_жонглер:','tr':':jonglör_erkek:','ar':':لاعب_قذف_الكرات:'},'\U0001F939\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_juggling_dark_skin_tone:','status':fully_qualified,'E':4,'de':':jongleur_dunkle_hautfarbe:','es':':hombre_haciendo_malabares_tono_de_piel_oscuro:','fr':':jongleur_peau_foncée:','ja':':ジャグリングをする男_濃い肌色:','ko':':저글링하는_남자_검은색_피부:','pt':':homem_malabarista_pele_escura:','it':':giocoliere_uomo_carnagione_scura:','fa':':مرد_ژانگولربازی_می_کند_پوست_آبنوسی:','id':':pria_melempar_warna_kulit_gelap:','zh':':男生抛接杂耍_较深肤色:','ru':':мужчина-жонглер_очень_темныи_тон_кожи:','tr':':jonglör_erkek_koyu_cilt_tonu:','ar':':لاعب_قذف_الكرات_بشرة_بلون_غامق:'},'\U0001F939\U0001F3FF\U0000200D\U00002642':{'en':':man_juggling_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':jongleur_dunkle_hautfarbe:','es':':hombre_haciendo_malabares_tono_de_piel_oscuro:','fr':':jongleur_peau_foncée:','ja':':ジャグリングをする男_濃い肌色:','ko':':저글링하는_남자_검은색_피부:','pt':':homem_malabarista_pele_escura:','it':':giocoliere_uomo_carnagione_scura:','fa':':مرد_ژانگولربازی_می_کند_پوست_آبنوسی:','id':':pria_melempar_warna_kulit_gelap:','zh':':男生抛接杂耍_较深肤色:','ru':':мужчина_жонглер_очень_темный_тон_кожи:','tr':':jonglör_erkek_koyu_cilt_tonu:','ar':':لاعب_قذف_الكرات_بشرة_بلون_غامق:'},'\U0001F939\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_juggling_light_skin_tone:','status':fully_qualified,'E':4,'de':':jongleur_helle_hautfarbe:','es':':hombre_haciendo_malabares_tono_de_piel_claro:','fr':':jongleur_peau_claire:','ja':':ジャグリングをする男_薄い肌色:','ko':':저글링하는_남자_하얀_피부:','pt':':homem_malabarista_pele_clara:','it':':giocoliere_uomo_carnagione_chiara:','fa':':مرد_ژانگولربازی_می_کند_پوست_سفید:','id':':pria_melempar_warna_kulit_cerah:','zh':':男生抛接杂耍_较浅肤色:','ru':':мужчина-жонглер_очень_светлыи_тон_кожи:','tr':':jonglör_erkek_açık_cilt_tonu:','ar':':لاعب_قذف_الكرات_بشرة_بلون_فاتح:'},'\U0001F939\U0001F3FB\U0000200D\U00002642':{'en':':man_juggling_light_skin_tone:','status':minimally_qualified,'E':4,'de':':jongleur_helle_hautfarbe:','es':':hombre_haciendo_malabares_tono_de_piel_claro:','fr':':jongleur_peau_claire:','ja':':ジャグリングをする男_薄い肌色:','ko':':저글링하는_남자_하얀_피부:','pt':':homem_malabarista_pele_clara:','it':':giocoliere_uomo_carnagione_chiara:','fa':':مرد_ژانگولربازی_می_کند_پوست_سفید:','id':':pria_melempar_warna_kulit_cerah:','zh':':男生抛接杂耍_较浅肤色:','ru':':мужчина_жонглер_очень_светлый_тон_кожи:','tr':':jonglör_erkek_açık_cilt_tonu:','ar':':لاعب_قذف_الكرات_بشرة_بلون_فاتح:'},'\U0001F939\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_juggling_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':jongleur_mitteldunkle_hautfarbe:','es':':hombre_haciendo_malabares_tono_de_piel_oscuro_medio:','fr':':jongleur_peau_mate:','ja':':ジャグリングをする男_やや濃い肌色:','ko':':저글링하는_남자_진한_갈색_피부:','pt':':homem_malabarista_pele_morena_escura:','it':':giocoliere_uomo_carnagione_abbastanza_scura:','fa':':مرد_ژانگولربازی_می_کند_پوست_گندمی:','id':':pria_melempar_warna_kulit_gelap-sedang:','zh':':男生抛接杂耍_中等深肤色:','ru':':мужчина-жонглер_темныи_тон_кожи:','tr':':jonglör_erkek_orta_koyu_cilt_tonu:','ar':':لاعب_قذف_الكرات_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F939\U0001F3FE\U0000200D\U00002642':{'en':':man_juggling_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':jongleur_mitteldunkle_hautfarbe:','es':':hombre_haciendo_malabares_tono_de_piel_oscuro_medio:','fr':':jongleur_peau_mate:','ja':':ジャグリングをする男_やや濃い肌色:','ko':':저글링하는_남자_진한_갈색_피부:','pt':':homem_malabarista_pele_morena_escura:','it':':giocoliere_uomo_carnagione_abbastanza_scura:','fa':':مرد_ژانگولربازی_می_کند_پوست_گندمی:','id':':pria_melempar_warna_kulit_gelap-sedang:','zh':':男生抛接杂耍_中等深肤色:','ru':':мужчина_жонглер_темный_тон_кожи:','tr':':jonglör_erkek_orta_koyu_cilt_tonu:','ar':':لاعب_قذف_الكرات_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F939\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_juggling_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':jongleur_mittelhelle_hautfarbe:','es':':hombre_haciendo_malabares_tono_de_piel_claro_medio:','fr':':jongleur_peau_moyennement_claire:','ja':':ジャグリングをする男_やや薄い肌色:','ko':':저글링하는_남자_연한_갈색_피부:','pt':':homem_malabarista_pele_morena_clara:','it':':giocoliere_uomo_carnagione_abbastanza_chiara:','fa':':مرد_ژانگولربازی_می_کند_پوست_روشن:','id':':pria_melempar_warna_kulit_cerah-sedang:','zh':':男生抛接杂耍_中等浅肤色:','ru':':мужчина-жонглер_светлыи_тон_кожи:','tr':':jonglör_erkek_orta_açık_cilt_tonu:','ar':':لاعب_قذف_الكرات_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F939\U0001F3FC\U0000200D\U00002642':{'en':':man_juggling_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':jongleur_mittelhelle_hautfarbe:','es':':hombre_haciendo_malabares_tono_de_piel_claro_medio:','fr':':jongleur_peau_moyennement_claire:','ja':':ジャグリングをする男_やや薄い肌色:','ko':':저글링하는_남자_연한_갈색_피부:','pt':':homem_malabarista_pele_morena_clara:','it':':giocoliere_uomo_carnagione_abbastanza_chiara:','fa':':مرد_ژانگولربازی_می_کند_پوست_روشن:','id':':pria_melempar_warna_kulit_cerah-sedang:','zh':':男生抛接杂耍_中等浅肤色:','ru':':мужчина_жонглер_светлый_тон_кожи:','tr':':jonglör_erkek_orta_açık_cilt_tonu:','ar':':لاعب_قذف_الكرات_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F939\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_juggling_medium_skin_tone:','status':fully_qualified,'E':4,'de':':jongleur_mittlere_hautfarbe:','es':':hombre_haciendo_malabares_tono_de_piel_medio:','fr':':jongleur_peau_légèrement_mate:','ja':':ジャグリングをする男_中間の肌色:','ko':':저글링하는_남자_갈색_피부:','pt':':homem_malabarista_pele_morena:','it':':giocoliere_uomo_carnagione_olivastra:','fa':':مرد_ژانگولربازی_می_کند_پوست_طلایی:','id':':pria_melempar_warna_kulit_sedang:','zh':':男生抛接杂耍_中等肤色:','ru':':мужчина-жонглер_среднии_тон_кожи:','tr':':jonglör_erkek_orta_cilt_tonu:','ar':':لاعب_قذف_الكرات_بشرة_بلون_معتدل:'},'\U0001F939\U0001F3FD\U0000200D\U00002642':{'en':':man_juggling_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':jongleur_mittlere_hautfarbe:','es':':hombre_haciendo_malabares_tono_de_piel_medio:','fr':':jongleur_peau_légèrement_mate:','ja':':ジャグリングをする男_中間の肌色:','ko':':저글링하는_남자_갈색_피부:','pt':':homem_malabarista_pele_morena:','it':':giocoliere_uomo_carnagione_olivastra:','fa':':مرد_ژانگولربازی_می_کند_پوست_طلایی:','id':':pria_melempar_warna_kulit_sedang:','zh':':男生抛接杂耍_中等肤色:','ru':':мужчина_жонглер_средний_тон_кожи:','tr':':jonglör_erkek_orta_cilt_tonu:','ar':':لاعب_قذف_الكرات_بشرة_بلون_معتدل:'},'\U0001F9CE\U0000200D\U00002642\U0000FE0F':{'en':':man_kneeling:','status':fully_qualified,'E':12,'alias':[':kneeling_man:'],'de':':kniender_mann:','es':':hombre_de_rodillas:','fr':':homme_à_genoux:','ja':':正座する男性:','ko':':무릎을_꿇은_남자:','pt':':homem_ajoelhando:','it':':uomo_inginocchiato:','fa':':مرد_زانوزده:','id':':pria_berlutut:','zh':':跪下的男人:','ru':':мужчина_на_коленях:','tr':':diz_çöken_erkek:','ar':':رجل_جالس_على_ركبتيه:'},'\U0001F9CE\U0000200D\U00002642':{'en':':man_kneeling:','status':minimally_qualified,'E':12,'alias':[':kneeling_man:'],'de':':kniender_mann:','es':':hombre_de_rodillas:','fr':':homme_à_genoux:','ja':':正座する男性:','ko':':무릎을_꿇은_남자:','pt':':homem_ajoelhando:','it':':uomo_inginocchiato:','fa':':مرد_زانوزده:','id':':pria_berlutut:','zh':':跪下的男人:','ru':':мужчина_на_коленях:','tr':':diz_çöken_erkek:','ar':':رجل_جالس_على_ركبتيه:'},'\U0001F9CE\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_kneeling_dark_skin_tone:','status':fully_qualified,'E':12,'de':':kniender_mann_dunkle_hautfarbe:','es':':hombre_de_rodillas_tono_de_piel_oscuro:','fr':':homme_à_genoux_peau_foncée:','ja':':正座する男性_濃い肌色:','ko':':무릎을_꿇은_남자_검은색_피부:','pt':':homem_ajoelhando_pele_escura:','it':':uomo_inginocchiato_carnagione_scura:','fa':':مرد_زانوزده_پوست_آبنوسی:','id':':pria_berlutut_warna_kulit_gelap:','zh':':跪下的男人_较深肤色:','ru':':мужчина_на_коленях_очень_темныи_тон_кожи:','tr':':diz_çöken_erkek_koyu_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_بشرة_بلون_غامق:'},'\U0001F9CE\U0001F3FF\U0000200D\U00002642':{'en':':man_kneeling_dark_skin_tone:','status':minimally_qualified,'E':12,'de':':kniender_mann_dunkle_hautfarbe:','es':':hombre_de_rodillas_tono_de_piel_oscuro:','fr':':homme_à_genoux_peau_foncée:','ja':':正座する男性_濃い肌色:','ko':':무릎을_꿇은_남자_검은색_피부:','pt':':homem_ajoelhando_pele_escura:','it':':uomo_inginocchiato_carnagione_scura:','fa':':مرد_زانوزده_پوست_آبنوسی:','id':':pria_berlutut_warna_kulit_gelap:','zh':':跪下的男人_较深肤色:','ru':':мужчина_на_коленях_очень_темный_тон_кожи:','tr':':diz_çöken_erkek_koyu_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_بشرة_بلون_غامق:'},'\U0001F9CE\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':man_kneeling_facing_right:','status':fully_qualified,'E':15.1,'de':':kniender_mann_nach_rechts:','es':':hombre_de_rodillas_hacia_la_derecha:','fr':':homme_à_genoux_vers_la_droite:','ja':':正座する男性_右向き:','ko':':오른쪽을_향한_무릎을_꿇은_남자:','pt':':homem_ajoelhando_de_frente_para_a_direita:','it':':uomo_inginocchiato_a_destra:','fa':':مرد_زانوزده_به_سمت_راست:','id':':pria_berlutut_menghadap_kanan:','zh':':跪下的男人面向右边:','ru':':мужчина_на_коленях_направленные_вправо:','tr':':sağa_dönük_diz_çöken_erkek:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين:'},'\U0001F9CE\U0000200D\U00002642\U0000200D\U000027A1\U0000FE0F':{'en':':man_kneeling_facing_right:','status':minimally_qualified,'E':15.1,'de':':kniender_mann_nach_rechts:','es':':hombre_de_rodillas_hacia_la_derecha:','fr':':homme_à_genoux_vers_la_droite:','ja':':正座する男性_右向き:','ko':':오른쪽을_향한_무릎을_꿇은_남자:','pt':':homem_ajoelhando_de_frente_para_a_direita:','it':':uomo_inginocchiato_a_destra:','fa':':مرد_زانوزده_به_سمت_راست:','id':':pria_berlutut_menghadap_kanan:','zh':':跪下的男人面向右边:','ru':':мужчина_на_коленях_направленные_вправо:','tr':':sağa_dönük_diz_çöken_erkek:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين:'},'\U0001F9CE\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1':{'en':':man_kneeling_facing_right:','status':minimally_qualified,'E':15.1,'de':':kniender_mann_nach_rechts:','es':':hombre_de_rodillas_hacia_la_derecha:','fr':':homme_à_genoux_vers_la_droite:','ja':':正座する男性_右向き:','ko':':오른쪽을_향한_무릎을_꿇은_남자:','pt':':homem_ajoelhando_de_frente_para_a_direita:','it':':uomo_inginocchiato_a_destra:','fa':':مرد_زانوزده_به_سمت_راست:','id':':pria_berlutut_menghadap_kanan:','zh':':跪下的男人面向右边:','ru':':мужчина_на_коленях_направленные_вправо:','tr':':sağa_dönük_diz_çöken_erkek:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين:'},'\U0001F9CE\U0000200D\U00002642\U0000200D\U000027A1':{'en':':man_kneeling_facing_right:','status':minimally_qualified,'E':15.1,'de':':kniender_mann_nach_rechts:','es':':hombre_de_rodillas_hacia_la_derecha:','fr':':homme_à_genoux_vers_la_droite:','ja':':正座する男性_右向き:','ko':':오른쪽을_향한_무릎을_꿇은_남자:','pt':':homem_ajoelhando_de_frente_para_a_direita:','it':':uomo_inginocchiato_a_destra:','fa':':مرد_زانوزده_به_سمت_راست:','id':':pria_berlutut_menghadap_kanan:','zh':':跪下的男人面向右边:','ru':':мужчина_на_коленях_направленные_вправо:','tr':':sağa_dönük_diz_çöken_erkek:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين:'},'\U0001F9CE\U0001F3FF\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':man_kneeling_facing_right_dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':kniender_mann_nach_rechts_dunkle_hautfarbe:','es':':hombre_de_rodillas_hacia_la_derecha_tono_de_piel_oscuro:','fr':':homme_à_genoux_vers_la_droite_peau_foncée:','ja':':正座する男性_右向き_濃い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_남자_검은색_피부:','pt':':homem_ajoelhando_de_frente_para_a_direita_pele_escura:','it':':uomo_inginocchiato_a_destra_carnagione_scura:','fa':':مرد_زانوزده_به_سمت_راستپوست_آبنوسی:','id':':pria_berlutut_menghadap_kanan_warna_kulit_gelap:','zh':':跪下的男人面向右边_较深肤色:','ru':':мужчина_на_коленях_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_diz_çöken_erkek_koyu_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين_بلون_غامق:'},'\U0001F9CE\U0001F3FF\U0000200D\U00002642\U0000200D\U000027A1\U0000FE0F':{'en':':man_kneeling_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniender_mann_nach_rechts_dunkle_hautfarbe:','es':':hombre_de_rodillas_hacia_la_derecha_tono_de_piel_oscuro:','fr':':homme_à_genoux_vers_la_droite_peau_foncée:','ja':':正座する男性_右向き_濃い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_남자_검은색_피부:','pt':':homem_ajoelhando_de_frente_para_a_direita_pele_escura:','it':':uomo_inginocchiato_a_destra_carnagione_scura:','fa':':مرد_زانوزده_به_سمت_راستپوست_آبنوسی:','id':':pria_berlutut_menghadap_kanan_warna_kulit_gelap:','zh':':跪下的男人面向右边_较深肤色:','ru':':мужчина_на_коленях_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_diz_çöken_erkek_koyu_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين_بلون_غامق:'},'\U0001F9CE\U0001F3FF\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1':{'en':':man_kneeling_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniender_mann_nach_rechts_dunkle_hautfarbe:','es':':hombre_de_rodillas_hacia_la_derecha_tono_de_piel_oscuro:','fr':':homme_à_genoux_vers_la_droite_peau_foncée:','ja':':正座する男性_右向き_濃い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_남자_검은색_피부:','pt':':homem_ajoelhando_de_frente_para_a_direita_pele_escura:','it':':uomo_inginocchiato_a_destra_carnagione_scura:','fa':':مرد_زانوزده_به_سمت_راستپوست_آبنوسی:','id':':pria_berlutut_menghadap_kanan_warna_kulit_gelap:','zh':':跪下的男人面向右边_较深肤色:','ru':':мужчина_на_коленях_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_diz_çöken_erkek_koyu_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين_بلون_غامق:'},'\U0001F9CE\U0001F3FF\U0000200D\U00002642\U0000200D\U000027A1':{'en':':man_kneeling_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniender_mann_nach_rechts_dunkle_hautfarbe:','es':':hombre_de_rodillas_hacia_la_derecha_tono_de_piel_oscuro:','fr':':homme_à_genoux_vers_la_droite_peau_foncée:','ja':':正座する男性_右向き_濃い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_남자_검은색_피부:','pt':':homem_ajoelhando_de_frente_para_a_direita_pele_escura:','it':':uomo_inginocchiato_a_destra_carnagione_scura:','fa':':مرد_زانوزده_به_سمت_راستپوست_آبنوسی:','id':':pria_berlutut_menghadap_kanan_warna_kulit_gelap:','zh':':跪下的男人面向右边_较深肤色:','ru':':мужчина_на_коленях_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_diz_çöken_erkek_koyu_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين_بلون_غامق:'},'\U0001F9CE\U0001F3FB\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':man_kneeling_facing_right_light_skin_tone:','status':fully_qualified,'E':15.1,'de':':kniender_mann_nach_rechts_helle_hautfarbe:','es':':hombre_de_rodillas_hacia_la_derecha_tono_de_piel_claro:','fr':':homme_à_genoux_vers_la_droite_peau_claire:','ja':':正座する男性_右向き_薄い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_남자_하얀_피부:','pt':':homem_ajoelhando_de_frente_para_a_direita_pele_clara:','it':':uomo_inginocchiato_a_destra_carnagione_chiara:','fa':':مرد_زانوزده_به_سمت_راست_پوست_سفید:','id':':pria_berlutut_menghadap_kanan_warna_kulit_cerah:','zh':':跪下的男人面向右边_较浅肤色:','ru':':мужчина_на_коленях_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_diz_çöken_erkek_açık_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F9CE\U0001F3FB\U0000200D\U00002642\U0000200D\U000027A1\U0000FE0F':{'en':':man_kneeling_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniender_mann_nach_rechts_helle_hautfarbe:','es':':hombre_de_rodillas_hacia_la_derecha_tono_de_piel_claro:','fr':':homme_à_genoux_vers_la_droite_peau_claire:','ja':':正座する男性_右向き_薄い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_남자_하얀_피부:','pt':':homem_ajoelhando_de_frente_para_a_direita_pele_clara:','it':':uomo_inginocchiato_a_destra_carnagione_chiara:','fa':':مرد_زانوزده_به_سمت_راست_پوست_سفید:','id':':pria_berlutut_menghadap_kanan_warna_kulit_cerah:','zh':':跪下的男人面向右边_较浅肤色:','ru':':мужчина_на_коленях_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_diz_çöken_erkek_açık_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F9CE\U0001F3FB\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1':{'en':':man_kneeling_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniender_mann_nach_rechts_helle_hautfarbe:','es':':hombre_de_rodillas_hacia_la_derecha_tono_de_piel_claro:','fr':':homme_à_genoux_vers_la_droite_peau_claire:','ja':':正座する男性_右向き_薄い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_남자_하얀_피부:','pt':':homem_ajoelhando_de_frente_para_a_direita_pele_clara:','it':':uomo_inginocchiato_a_destra_carnagione_chiara:','fa':':مرد_زانوزده_به_سمت_راست_پوست_سفید:','id':':pria_berlutut_menghadap_kanan_warna_kulit_cerah:','zh':':跪下的男人面向右边_较浅肤色:','ru':':мужчина_на_коленях_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_diz_çöken_erkek_açık_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F9CE\U0001F3FB\U0000200D\U00002642\U0000200D\U000027A1':{'en':':man_kneeling_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniender_mann_nach_rechts_helle_hautfarbe:','es':':hombre_de_rodillas_hacia_la_derecha_tono_de_piel_claro:','fr':':homme_à_genoux_vers_la_droite_peau_claire:','ja':':正座する男性_右向き_薄い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_남자_하얀_피부:','pt':':homem_ajoelhando_de_frente_para_a_direita_pele_clara:','it':':uomo_inginocchiato_a_destra_carnagione_chiara:','fa':':مرد_زانوزده_به_سمت_راست_پوست_سفید:','id':':pria_berlutut_menghadap_kanan_warna_kulit_cerah:','zh':':跪下的男人面向右边_较浅肤色:','ru':':мужчина_на_коленях_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_diz_çöken_erkek_açık_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F9CE\U0001F3FE\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':man_kneeling_facing_right_medium-dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':kniender_mann_nach_rechts_mitteldunkle_hautfarbe:','es':':hombre_de_rodillas_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':homme_à_genoux_vers_la_droite_peau_mate:','ja':':正座する男性_右向き_やや濃い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_남자_진한_갈색_피부:','pt':':homem_ajoelhando_de_frente_para_a_direita_pele_morena_escura:','it':':uomo_inginocchiato_a_destra_carnagione_abbastanza_scura:','fa':':مرد_زانوزده_به_سمت_راست_پوست_گندمی:','id':':pria_berlutut_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':跪下的男人面向右边_中等深肤色:','ru':':мужчина_на_коленях_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_diz_çöken_erkek_orta_koyu_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F9CE\U0001F3FE\U0000200D\U00002642\U0000200D\U000027A1\U0000FE0F':{'en':':man_kneeling_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniender_mann_nach_rechts_mitteldunkle_hautfarbe:','es':':hombre_de_rodillas_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':homme_à_genoux_vers_la_droite_peau_mate:','ja':':正座する男性_右向き_やや濃い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_남자_진한_갈색_피부:','pt':':homem_ajoelhando_de_frente_para_a_direita_pele_morena_escura:','it':':uomo_inginocchiato_a_destra_carnagione_abbastanza_scura:','fa':':مرد_زانوزده_به_سمت_راست_پوست_گندمی:','id':':pria_berlutut_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':跪下的男人面向右边_中等深肤色:','ru':':мужчина_на_коленях_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_diz_çöken_erkek_orta_koyu_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F9CE\U0001F3FE\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1':{'en':':man_kneeling_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniender_mann_nach_rechts_mitteldunkle_hautfarbe:','es':':hombre_de_rodillas_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':homme_à_genoux_vers_la_droite_peau_mate:','ja':':正座する男性_右向き_やや濃い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_남자_진한_갈색_피부:','pt':':homem_ajoelhando_de_frente_para_a_direita_pele_morena_escura:','it':':uomo_inginocchiato_a_destra_carnagione_abbastanza_scura:','fa':':مرد_زانوزده_به_سمت_راست_پوست_گندمی:','id':':pria_berlutut_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':跪下的男人面向右边_中等深肤色:','ru':':мужчина_на_коленях_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_diz_çöken_erkek_orta_koyu_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F9CE\U0001F3FE\U0000200D\U00002642\U0000200D\U000027A1':{'en':':man_kneeling_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniender_mann_nach_rechts_mitteldunkle_hautfarbe:','es':':hombre_de_rodillas_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':homme_à_genoux_vers_la_droite_peau_mate:','ja':':正座する男性_右向き_やや濃い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_남자_진한_갈색_피부:','pt':':homem_ajoelhando_de_frente_para_a_direita_pele_morena_escura:','it':':uomo_inginocchiato_a_destra_carnagione_abbastanza_scura:','fa':':مرد_زانوزده_به_سمت_راست_پوست_گندمی:','id':':pria_berlutut_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':跪下的男人面向右边_中等深肤色:','ru':':мужчина_на_коленях_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_diz_çöken_erkek_orta_koyu_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F9CE\U0001F3FC\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':man_kneeling_facing_right_medium-light_skin_tone:','status':fully_qualified,'E':15.1,'de':':kniender_mann_nach_rechts_mittelhelle_hautfarbe:','es':':hombre_de_rodillas_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':homme_à_genoux_vers_la_droite_peau_moyennement_claire:','ja':':正座する男性_右向き_やや薄い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_남자_연한_갈색_피부:','pt':':homem_ajoelhando_de_frente_para_a_direita_pele_morena_clara:','it':':uomo_inginocchiato_a_destra_carnagione_abbastanza_chiara:','fa':':مرد_زانوزده_به_سمت_راست_پوست_روشن:','id':':pria_berlutut_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':跪下的男人面向右边_中等浅肤色:','ru':':мужчина_на_коленях_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_diz_çöken_erkek_orta_açık_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F9CE\U0001F3FC\U0000200D\U00002642\U0000200D\U000027A1\U0000FE0F':{'en':':man_kneeling_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniender_mann_nach_rechts_mittelhelle_hautfarbe:','es':':hombre_de_rodillas_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':homme_à_genoux_vers_la_droite_peau_moyennement_claire:','ja':':正座する男性_右向き_やや薄い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_남자_연한_갈색_피부:','pt':':homem_ajoelhando_de_frente_para_a_direita_pele_morena_clara:','it':':uomo_inginocchiato_a_destra_carnagione_abbastanza_chiara:','fa':':مرد_زانوزده_به_سمت_راست_پوست_روشن:','id':':pria_berlutut_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':跪下的男人面向右边_中等浅肤色:','ru':':мужчина_на_коленях_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_diz_çöken_erkek_orta_açık_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F9CE\U0001F3FC\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1':{'en':':man_kneeling_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniender_mann_nach_rechts_mittelhelle_hautfarbe:','es':':hombre_de_rodillas_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':homme_à_genoux_vers_la_droite_peau_moyennement_claire:','ja':':正座する男性_右向き_やや薄い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_남자_연한_갈색_피부:','pt':':homem_ajoelhando_de_frente_para_a_direita_pele_morena_clara:','it':':uomo_inginocchiato_a_destra_carnagione_abbastanza_chiara:','fa':':مرد_زانوزده_به_سمت_راست_پوست_روشن:','id':':pria_berlutut_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':跪下的男人面向右边_中等浅肤色:','ru':':мужчина_на_коленях_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_diz_çöken_erkek_orta_açık_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F9CE\U0001F3FC\U0000200D\U00002642\U0000200D\U000027A1':{'en':':man_kneeling_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniender_mann_nach_rechts_mittelhelle_hautfarbe:','es':':hombre_de_rodillas_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':homme_à_genoux_vers_la_droite_peau_moyennement_claire:','ja':':正座する男性_右向き_やや薄い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_남자_연한_갈색_피부:','pt':':homem_ajoelhando_de_frente_para_a_direita_pele_morena_clara:','it':':uomo_inginocchiato_a_destra_carnagione_abbastanza_chiara:','fa':':مرد_زانوزده_به_سمت_راست_پوست_روشن:','id':':pria_berlutut_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':跪下的男人面向右边_中等浅肤色:','ru':':мужчина_на_коленях_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_diz_çöken_erkek_orta_açık_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F9CE\U0001F3FD\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':man_kneeling_facing_right_medium_skin_tone:','status':fully_qualified,'E':15.1,'de':':kniender_mann_nach_rechts_mittlere_hautfarbe:','es':':hombre_de_rodillas_hacia_la_derecha_tono_de_piel_medio:','fr':':homme_à_genoux_vers_la_droite_peau_légèrement_mate:','ja':':正座する男性_右向き_中間の肌色:','ko':':오른쪽을_향한_무릎을_꿇은_남자_갈색_피부:','pt':':homem_ajoelhando_de_frente_para_a_direita_pele_morena:','it':':uomo_inginocchiato_a_destra_carnagione_olivastra:','fa':':مرد_زانوزده_به_سمت_راست_پوست_طلایی:','id':':pria_berlutut_menghadap_kanan_warna_kulit_sedang:','zh':':跪下的男人面向右边_中等肤色:','ru':':мужчина_на_коленях_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_diz_çöken_erkek_orta_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين_بلون_معتدل:'},'\U0001F9CE\U0001F3FD\U0000200D\U00002642\U0000200D\U000027A1\U0000FE0F':{'en':':man_kneeling_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniender_mann_nach_rechts_mittlere_hautfarbe:','es':':hombre_de_rodillas_hacia_la_derecha_tono_de_piel_medio:','fr':':homme_à_genoux_vers_la_droite_peau_légèrement_mate:','ja':':正座する男性_右向き_中間の肌色:','ko':':오른쪽을_향한_무릎을_꿇은_남자_갈색_피부:','pt':':homem_ajoelhando_de_frente_para_a_direita_pele_morena:','it':':uomo_inginocchiato_a_destra_carnagione_olivastra:','fa':':مرد_زانوزده_به_سمت_راست_پوست_طلایی:','id':':pria_berlutut_menghadap_kanan_warna_kulit_sedang:','zh':':跪下的男人面向右边_中等肤色:','ru':':мужчина_на_коленях_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_diz_çöken_erkek_orta_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين_بلون_معتدل:'},'\U0001F9CE\U0001F3FD\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1':{'en':':man_kneeling_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniender_mann_nach_rechts_mittlere_hautfarbe:','es':':hombre_de_rodillas_hacia_la_derecha_tono_de_piel_medio:','fr':':homme_à_genoux_vers_la_droite_peau_légèrement_mate:','ja':':正座する男性_右向き_中間の肌色:','ko':':오른쪽을_향한_무릎을_꿇은_남자_갈색_피부:','pt':':homem_ajoelhando_de_frente_para_a_direita_pele_morena:','it':':uomo_inginocchiato_a_destra_carnagione_olivastra:','fa':':مرد_زانوزده_به_سمت_راست_پوست_طلایی:','id':':pria_berlutut_menghadap_kanan_warna_kulit_sedang:','zh':':跪下的男人面向右边_中等肤色:','ru':':мужчина_на_коленях_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_diz_çöken_erkek_orta_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين_بلون_معتدل:'},'\U0001F9CE\U0001F3FD\U0000200D\U00002642\U0000200D\U000027A1':{'en':':man_kneeling_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniender_mann_nach_rechts_mittlere_hautfarbe:','es':':hombre_de_rodillas_hacia_la_derecha_tono_de_piel_medio:','fr':':homme_à_genoux_vers_la_droite_peau_légèrement_mate:','ja':':正座する男性_右向き_中間の肌色:','ko':':오른쪽을_향한_무릎을_꿇은_남자_갈색_피부:','pt':':homem_ajoelhando_de_frente_para_a_direita_pele_morena:','it':':uomo_inginocchiato_a_destra_carnagione_olivastra:','fa':':مرد_زانوزده_به_سمت_راست_پوست_طلایی:','id':':pria_berlutut_menghadap_kanan_warna_kulit_sedang:','zh':':跪下的男人面向右边_中等肤色:','ru':':мужчина_на_коленях_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_diz_çöken_erkek_orta_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_متجه_لليمين_بلون_معتدل:'},'\U0001F9CE\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_kneeling_light_skin_tone:','status':fully_qualified,'E':12,'de':':kniender_mann_helle_hautfarbe:','es':':hombre_de_rodillas_tono_de_piel_claro:','fr':':homme_à_genoux_peau_claire:','ja':':正座する男性_薄い肌色:','ko':':무릎을_꿇은_남자_하얀_피부:','pt':':homem_ajoelhando_pele_clara:','it':':uomo_inginocchiato_carnagione_chiara:','fa':':مرد_زانوزده_پوست_سفید:','id':':pria_berlutut_warna_kulit_cerah:','zh':':跪下的男人_较浅肤色:','ru':':мужчина_на_коленях_очень_светлыи_тон_кожи:','tr':':diz_çöken_erkek_açık_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_بشرة_بلون_فاتح:'},'\U0001F9CE\U0001F3FB\U0000200D\U00002642':{'en':':man_kneeling_light_skin_tone:','status':minimally_qualified,'E':12,'de':':kniender_mann_helle_hautfarbe:','es':':hombre_de_rodillas_tono_de_piel_claro:','fr':':homme_à_genoux_peau_claire:','ja':':正座する男性_薄い肌色:','ko':':무릎을_꿇은_남자_하얀_피부:','pt':':homem_ajoelhando_pele_clara:','it':':uomo_inginocchiato_carnagione_chiara:','fa':':مرد_زانوزده_پوست_سفید:','id':':pria_berlutut_warna_kulit_cerah:','zh':':跪下的男人_较浅肤色:','ru':':мужчина_на_коленях_очень_светлый_тон_кожи:','tr':':diz_çöken_erkek_açık_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_بشرة_بلون_فاتح:'},'\U0001F9CE\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_kneeling_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':kniender_mann_mitteldunkle_hautfarbe:','es':':hombre_de_rodillas_tono_de_piel_oscuro_medio:','fr':':homme_à_genoux_peau_mate:','ja':':正座する男性_やや濃い肌色:','ko':':무릎을_꿇은_남자_진한_갈색_피부:','pt':':homem_ajoelhando_pele_morena_escura:','it':':uomo_inginocchiato_carnagione_abbastanza_scura:','fa':':مرد_زانوزده_پوست_گندمی:','id':':pria_berlutut_warna_kulit_gelap-sedang:','zh':':跪下的男人_中等深肤色:','ru':':мужчина_на_коленях_темныи_тон_кожи:','tr':':diz_çöken_erkek_orta_koyu_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9CE\U0001F3FE\U0000200D\U00002642':{'en':':man_kneeling_medium-dark_skin_tone:','status':minimally_qualified,'E':12,'de':':kniender_mann_mitteldunkle_hautfarbe:','es':':hombre_de_rodillas_tono_de_piel_oscuro_medio:','fr':':homme_à_genoux_peau_mate:','ja':':正座する男性_やや濃い肌色:','ko':':무릎을_꿇은_남자_진한_갈색_피부:','pt':':homem_ajoelhando_pele_morena_escura:','it':':uomo_inginocchiato_carnagione_abbastanza_scura:','fa':':مرد_زانوزده_پوست_گندمی:','id':':pria_berlutut_warna_kulit_gelap-sedang:','zh':':跪下的男人_中等深肤色:','ru':':мужчина_на_коленях_темный_тон_кожи:','tr':':diz_çöken_erkek_orta_koyu_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9CE\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_kneeling_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':kniender_mann_mittelhelle_hautfarbe:','es':':hombre_de_rodillas_tono_de_piel_claro_medio:','fr':':homme_à_genoux_peau_moyennement_claire:','ja':':正座する男性_やや薄い肌色:','ko':':무릎을_꿇은_남자_연한_갈색_피부:','pt':':homem_ajoelhando_pele_morena_clara:','it':':uomo_inginocchiato_carnagione_abbastanza_chiara:','fa':':مرد_زانوزده_پوست_روشن:','id':':pria_berlutut_warna_kulit_cerah-sedang:','zh':':跪下的男人_中等浅肤色:','ru':':мужчина_на_коленях_светлыи_тон_кожи:','tr':':diz_çöken_erkek_orta_açık_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9CE\U0001F3FC\U0000200D\U00002642':{'en':':man_kneeling_medium-light_skin_tone:','status':minimally_qualified,'E':12,'de':':kniender_mann_mittelhelle_hautfarbe:','es':':hombre_de_rodillas_tono_de_piel_claro_medio:','fr':':homme_à_genoux_peau_moyennement_claire:','ja':':正座する男性_やや薄い肌色:','ko':':무릎을_꿇은_남자_연한_갈색_피부:','pt':':homem_ajoelhando_pele_morena_clara:','it':':uomo_inginocchiato_carnagione_abbastanza_chiara:','fa':':مرد_زانوزده_پوست_روشن:','id':':pria_berlutut_warna_kulit_cerah-sedang:','zh':':跪下的男人_中等浅肤色:','ru':':мужчина_на_коленях_светлый_тон_кожи:','tr':':diz_çöken_erkek_orta_açık_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9CE\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_kneeling_medium_skin_tone:','status':fully_qualified,'E':12,'de':':kniender_mann_mittlere_hautfarbe:','es':':hombre_de_rodillas_tono_de_piel_medio:','fr':':homme_à_genoux_peau_légèrement_mate:','ja':':正座する男性_中間の肌色:','ko':':무릎을_꿇은_남자_갈색_피부:','pt':':homem_ajoelhando_pele_morena:','it':':uomo_inginocchiato_carnagione_olivastra:','fa':':مرد_زانوزده_پوست_طلایی:','id':':pria_berlutut_warna_kulit_sedang:','zh':':跪下的男人_中等肤色:','ru':':мужчина_на_коленях_среднии_тон_кожи:','tr':':diz_çöken_erkek_orta_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_بشرة_بلون_معتدل:'},'\U0001F9CE\U0001F3FD\U0000200D\U00002642':{'en':':man_kneeling_medium_skin_tone:','status':minimally_qualified,'E':12,'de':':kniender_mann_mittlere_hautfarbe:','es':':hombre_de_rodillas_tono_de_piel_medio:','fr':':homme_à_genoux_peau_légèrement_mate:','ja':':正座する男性_中間の肌色:','ko':':무릎을_꿇은_남자_갈색_피부:','pt':':homem_ajoelhando_pele_morena:','it':':uomo_inginocchiato_carnagione_olivastra:','fa':':مرد_زانوزده_پوست_طلایی:','id':':pria_berlutut_warna_kulit_sedang:','zh':':跪下的男人_中等肤色:','ru':':мужчина_на_коленях_средний_тон_кожи:','tr':':diz_çöken_erkek_orta_cilt_tonu:','ar':':رجل_جالس_على_ركبتيه_بشرة_بلون_معتدل:'},'\U0001F3CB\U0000FE0F\U0000200D\U00002642\U0000FE0F':{'en':':man_lifting_weights:','status':fully_qualified,'E':4,'alias':[':weight_lifting_man:'],'de':':gewichtheber:','es':':hombre_levantando_pesas:','fr':':homme_haltérophile:','ja':':重量挙げをする男:','ko':':남자_역도_선수:','pt':':homem_levantando_peso:','it':':uomo_che_solleva_pesi:','fa':':وزنه_بردار_مرد:','id':':pria_mengangkat_beban:','zh':':男生举重:','ru':':мужчина_со_штангои:','tr':':ağırlık_kaldıran_erkek:','ar':':رافع_أثقال:'},'\U0001F3CB\U0000200D\U00002642\U0000FE0F':{'en':':man_lifting_weights:','status':unqualified,'E':4,'alias':[':weight_lifting_man:'],'de':':gewichtheber:','es':':hombre_levantando_pesas:','fr':':homme_haltérophile:','ja':':重量挙げをする男:','ko':':남자_역도_선수:','pt':':homem_levantando_peso:','it':':uomo_che_solleva_pesi:','fa':':وزنه_بردار_مرد:','id':':pria_mengangkat_beban:','zh':':男生举重:','ru':':мужчина_со_штангои:','tr':':ağırlık_kaldıran_erkek:','ar':':رافع_أثقال:'},'\U0001F3CB\U0000FE0F\U0000200D\U00002642':{'en':':man_lifting_weights:','status':minimally_qualified,'E':4,'alias':[':weight_lifting_man:'],'de':':gewichtheber:','es':':hombre_levantando_pesas:','fr':':homme_haltérophile:','ja':':重量挙げをする男:','ko':':남자_역도_선수:','pt':':homem_levantando_peso:','it':':uomo_che_solleva_pesi:','fa':':وزنه_بردار_مرد:','id':':pria_mengangkat_beban:','zh':':男生举重:','ru':':мужчина_со_штангои:','tr':':ağırlık_kaldıran_erkek:','ar':':رافع_أثقال:'},'\U0001F3CB\U0000200D\U00002642':{'en':':man_lifting_weights:','status':unqualified,'E':4,'alias':[':weight_lifting_man:'],'de':':gewichtheber:','es':':hombre_levantando_pesas:','fr':':homme_haltérophile:','ja':':重量挙げをする男:','ko':':남자_역도_선수:','pt':':homem_levantando_peso:','it':':uomo_che_solleva_pesi:','fa':':وزنه_بردار_مرد:','id':':pria_mengangkat_beban:','zh':':男生举重:','ru':':мужчина_со_штангой:','tr':':ağırlık_kaldıran_erkek:','ar':':رافع_أثقال:'},'\U0001F3CB\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_lifting_weights_dark_skin_tone:','status':fully_qualified,'E':4,'de':':gewichtheber_dunkle_hautfarbe:','es':':hombre_levantando_pesas_tono_de_piel_oscuro:','fr':':homme_haltérophile_peau_foncée:','ja':':重量挙げをする男_濃い肌色:','ko':':남자_역도_선수_검은색_피부:','pt':':homem_levantando_peso_pele_escura:','it':':uomo_che_solleva_pesi_carnagione_scura:','fa':':وزنه_بردار_مرد_پوست_آبنوسی:','id':':pria_mengangkat_beban_warna_kulit_gelap:','zh':':男生举重_较深肤色:','ru':':мужчина_со_штангои_очень_темныи_тон_кожи:','tr':':ağırlık_kaldıran_erkek_koyu_cilt_tonu:','ar':':رافع_أثقال_بشرة_بلون_غامق:'},'\U0001F3CB\U0001F3FF\U0000200D\U00002642':{'en':':man_lifting_weights_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':gewichtheber_dunkle_hautfarbe:','es':':hombre_levantando_pesas_tono_de_piel_oscuro:','fr':':homme_haltérophile_peau_foncée:','ja':':重量挙げをする男_濃い肌色:','ko':':남자_역도_선수_검은색_피부:','pt':':homem_levantando_peso_pele_escura:','it':':uomo_che_solleva_pesi_carnagione_scura:','fa':':وزنه_بردار_مرد_پوست_آبنوسی:','id':':pria_mengangkat_beban_warna_kulit_gelap:','zh':':男生举重_较深肤色:','ru':':мужчина_со_штангой_очень_темный_тон_кожи:','tr':':ağırlık_kaldıran_erkek_koyu_cilt_tonu:','ar':':رافع_أثقال_بشرة_بلون_غامق:'},'\U0001F3CB\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_lifting_weights_light_skin_tone:','status':fully_qualified,'E':4,'de':':gewichtheber_helle_hautfarbe:','es':':hombre_levantando_pesas_tono_de_piel_claro:','fr':':homme_haltérophile_peau_claire:','ja':':重量挙げをする男_薄い肌色:','ko':':남자_역도_선수_하얀_피부:','pt':':homem_levantando_peso_pele_clara:','it':':uomo_che_solleva_pesi_carnagione_chiara:','fa':':وزنه_بردار_مرد_پوست_سفید:','id':':pria_mengangkat_beban_warna_kulit_cerah:','zh':':男生举重_较浅肤色:','ru':':мужчина_со_штангои_очень_светлыи_тон_кожи:','tr':':ağırlık_kaldıran_erkek_açık_cilt_tonu:','ar':':رافع_أثقال_بشرة_بلون_فاتح:'},'\U0001F3CB\U0001F3FB\U0000200D\U00002642':{'en':':man_lifting_weights_light_skin_tone:','status':minimally_qualified,'E':4,'de':':gewichtheber_helle_hautfarbe:','es':':hombre_levantando_pesas_tono_de_piel_claro:','fr':':homme_haltérophile_peau_claire:','ja':':重量挙げをする男_薄い肌色:','ko':':남자_역도_선수_하얀_피부:','pt':':homem_levantando_peso_pele_clara:','it':':uomo_che_solleva_pesi_carnagione_chiara:','fa':':وزنه_بردار_مرد_پوست_سفید:','id':':pria_mengangkat_beban_warna_kulit_cerah:','zh':':男生举重_较浅肤色:','ru':':мужчина_со_штангой_очень_светлый_тон_кожи:','tr':':ağırlık_kaldıran_erkek_açık_cilt_tonu:','ar':':رافع_أثقال_بشرة_بلون_فاتح:'},'\U0001F3CB\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_lifting_weights_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':gewichtheber_mitteldunkle_hautfarbe:','es':':hombre_levantando_pesas_tono_de_piel_oscuro_medio:','fr':':homme_haltérophile_peau_mate:','ja':':重量挙げをする男_やや濃い肌色:','ko':':남자_역도_선수_진한_갈색_피부:','pt':':homem_levantando_peso_pele_morena_escura:','it':':uomo_che_solleva_pesi_carnagione_abbastanza_scura:','fa':':وزنه_بردار_مرد_پوست_گندمی:','id':':pria_mengangkat_beban_warna_kulit_gelap-sedang:','zh':':男生举重_中等深肤色:','ru':':мужчина_со_штангои_темныи_тон_кожи:','tr':':ağırlık_kaldıran_erkek_orta_koyu_cilt_tonu:','ar':':رافع_أثقال_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3CB\U0001F3FE\U0000200D\U00002642':{'en':':man_lifting_weights_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':gewichtheber_mitteldunkle_hautfarbe:','es':':hombre_levantando_pesas_tono_de_piel_oscuro_medio:','fr':':homme_haltérophile_peau_mate:','ja':':重量挙げをする男_やや濃い肌色:','ko':':남자_역도_선수_진한_갈색_피부:','pt':':homem_levantando_peso_pele_morena_escura:','it':':uomo_che_solleva_pesi_carnagione_abbastanza_scura:','fa':':وزنه_بردار_مرد_پوست_گندمی:','id':':pria_mengangkat_beban_warna_kulit_gelap-sedang:','zh':':男生举重_中等深肤色:','ru':':мужчина_со_штангой_темный_тон_кожи:','tr':':ağırlık_kaldıran_erkek_orta_koyu_cilt_tonu:','ar':':رافع_أثقال_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3CB\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_lifting_weights_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':gewichtheber_mittelhelle_hautfarbe:','es':':hombre_levantando_pesas_tono_de_piel_claro_medio:','fr':':homme_haltérophile_peau_moyennement_claire:','ja':':重量挙げをする男_やや薄い肌色:','ko':':남자_역도_선수_연한_갈색_피부:','pt':':homem_levantando_peso_pele_morena_clara:','it':':uomo_che_solleva_pesi_carnagione_abbastanza_chiara:','fa':':وزنه_بردار_مرد_پوست_روشن:','id':':pria_mengangkat_beban_warna_kulit_cerah-sedang:','zh':':男生举重_中等浅肤色:','ru':':мужчина_со_штангои_светлыи_тон_кожи:','tr':':ağırlık_kaldıran_erkek_orta_açık_cilt_tonu:','ar':':رافع_أثقال_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3CB\U0001F3FC\U0000200D\U00002642':{'en':':man_lifting_weights_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':gewichtheber_mittelhelle_hautfarbe:','es':':hombre_levantando_pesas_tono_de_piel_claro_medio:','fr':':homme_haltérophile_peau_moyennement_claire:','ja':':重量挙げをする男_やや薄い肌色:','ko':':남자_역도_선수_연한_갈색_피부:','pt':':homem_levantando_peso_pele_morena_clara:','it':':uomo_che_solleva_pesi_carnagione_abbastanza_chiara:','fa':':وزنه_بردار_مرد_پوست_روشن:','id':':pria_mengangkat_beban_warna_kulit_cerah-sedang:','zh':':男生举重_中等浅肤色:','ru':':мужчина_со_штангой_светлый_тон_кожи:','tr':':ağırlık_kaldıran_erkek_orta_açık_cilt_tonu:','ar':':رافع_أثقال_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3CB\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_lifting_weights_medium_skin_tone:','status':fully_qualified,'E':4,'de':':gewichtheber_mittlere_hautfarbe:','es':':hombre_levantando_pesas_tono_de_piel_medio:','fr':':homme_haltérophile_peau_légèrement_mate:','ja':':重量挙げをする男_中間の肌色:','ko':':남자_역도_선수_갈색_피부:','pt':':homem_levantando_peso_pele_morena:','it':':uomo_che_solleva_pesi_carnagione_olivastra:','fa':':وزنه_بردار_مرد_پوست_طلایی:','id':':pria_mengangkat_beban_warna_kulit_sedang:','zh':':男生举重_中等肤色:','ru':':мужчина_со_штангои_среднии_тон_кожи:','tr':':ağırlık_kaldıran_erkek_orta_cilt_tonu:','ar':':رافع_أثقال_بشرة_بلون_معتدل:'},'\U0001F3CB\U0001F3FD\U0000200D\U00002642':{'en':':man_lifting_weights_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':gewichtheber_mittlere_hautfarbe:','es':':hombre_levantando_pesas_tono_de_piel_medio:','fr':':homme_haltérophile_peau_légèrement_mate:','ja':':重量挙げをする男_中間の肌色:','ko':':남자_역도_선수_갈색_피부:','pt':':homem_levantando_peso_pele_morena:','it':':uomo_che_solleva_pesi_carnagione_olivastra:','fa':':وزنه_بردار_مرد_پوست_طلایی:','id':':pria_mengangkat_beban_warna_kulit_sedang:','zh':':男生举重_中等肤色:','ru':':мужчина_со_штангой_средний_тон_кожи:','tr':':ağırlık_kaldıran_erkek_orta_cilt_tonu:','ar':':رافع_أثقال_بشرة_بلون_معتدل:'},'\U0001F468\U0001F3FB':{'en':':man_light_skin_tone:','status':fully_qualified,'E':1,'de':':mann_helle_hautfarbe:','es':':hombre_tono_de_piel_claro:','fr':':homme_peau_claire:','ja':':男性_薄い肌色:','ko':':남자_하얀_피부:','pt':':homem_pele_clara:','it':':uomo_carnagione_chiara:','fa':':مرد_پوست_سفید:','id':':pria_warna_kulit_cerah:','zh':':男人_较浅肤色:','ru':':мужчина_очень_светлый_тон_кожи:','tr':':erkek_açık_cilt_tonu:','ar':':رجل_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FB\U0000200D\U0001F9B2':{'en':':man_light_skin_tone_bald:','status':fully_qualified,'E':11,'de':':mann_helle_hautfarbe_glatze:','es':':hombre_tono_de_piel_claro_y_sin_pelo:','fr':':homme_peau_claire_et_chauve:','ja':':男性_薄い肌色_はげ頭:','ko':':남자_하얀_피부_대머리:','pt':':homem_pele_clara_e_careca:','it':':uomo_carnagione_chiara_e_calvo:','fa':':مرد_پوست_سفید_تاس:','id':':pria_warna_kulit_cerah_gundul:','zh':':男人_较浅肤色秃顶:','ru':':мужчина_очень_светлый_тон_кожи_без_волос:','tr':':erkek_açık_cilt_tonu_kel:','ar':':رجل_بشرة_بلون_فاتح_وأصلع:'},'\U0001F9D4\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_light_skin_tone_beard:','status':fully_qualified,'E':13.1,'de':':mann_helle_hautfarbe_bart:','es':':hombre_tono_de_piel_claro_y_barba:','fr':':homme_barbu_peau_claire:','ja':':あごひげの男性_薄い肌色:','ko':':수염_난_남자_하얀_피부:','pt':':homem_pele_clara_e_barbudo:','it':':uomo_con_la_barba_carnagione_chiara:','fa':':مرد_پوست_سفید_ریش:','id':':pria_warna_kulit_cerah_janggut:','zh':':有络腮胡子的男人_较浅肤色:','ru':':бородатыи_мужчина_очень_светлыи_тон_кожи:','tr':':erkek_açık_cilt_tonu_sakal:','ar':':رجل_بلحية_بشرة_بلون_فاتح:'},'\U0001F9D4\U0001F3FB\U0000200D\U00002642':{'en':':man_light_skin_tone_beard:','status':minimally_qualified,'E':13.1,'de':':mann_helle_hautfarbe_bart:','es':':hombre_tono_de_piel_claro_y_barba:','fr':':homme_barbu_peau_claire:','ja':':あごひげの男性_薄い肌色:','ko':':수염_난_남자_하얀_피부:','pt':':homem_pele_clara_e_barbudo:','it':':uomo_con_la_barba_carnagione_chiara:','fa':':مرد_پوست_سفید_ریش:','id':':pria_warna_kulit_cerah_janggut:','zh':':有络腮胡子的男人_较浅肤色:','ru':':бородатый_мужчина_очень_светлый_тон_кожи:','tr':':erkek_açık_cilt_tonu_sakal:','ar':':رجل_بلحية_بشرة_بلون_فاتح:'},'\U0001F471\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_light_skin_tone_blond_hair:','status':fully_qualified,'E':4,'de':':mann_helle_hautfarbe,_blond:','es':':hombre_rubio_tono_de_piel_claro:','fr':':homme_blond_peau_claire:','ja':':金髪の男性_薄い肌色:','ko':':금발_남자_하얀_피부:','pt':':homem_pele_clara_e_cabelo_loiro:','it':':uomo_biondo_carnagione_chiara:','fa':':مرد_پوست_سفید_مو_بور:','id':':pria_berambut_pirang_warna_kulit_cerah:','zh':':金发男_较浅肤色:','ru':':блондин_очень_светлыи_тон_кожи:','tr':':sarışın_erkek_açık_cilt_tonu:','ar':':رجل_أشقر_بشرة_بلون_فاتح:'},'\U0001F471\U0001F3FB\U0000200D\U00002642':{'en':':man_light_skin_tone_blond_hair:','status':minimally_qualified,'E':4,'de':':mann_helle_hautfarbe_blond:','es':':hombre_rubio_tono_de_piel_claro:','fr':':homme_blond_peau_claire:','ja':':金髪の男性_薄い肌色:','ko':':금발_남자_하얀_피부:','pt':':homem_pele_clara_e_cabelo_loiro:','it':':uomo_biondo_carnagione_chiara:','fa':':مرد_پوست_سفید_مو_بور:','id':':pria_berambut_pirang_warna_kulit_cerah:','zh':':金发男_较浅肤色:','ru':':блондин_очень_светлый_тон_кожи:','tr':':sarışın_erkek_açık_cilt_tonu:','ar':':رجل_أشقر_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FB\U0000200D\U0001F9B1':{'en':':man_light_skin_tone_curly_hair:','status':fully_qualified,'E':11,'de':':mann_helle_hautfarbe_lockiges_haar:','es':':hombre_tono_de_piel_claro_y_pelo_rizado:','fr':':homme_peau_claire_et_cheveux_bouclés:','ja':':男性_薄い肌色_巻き毛:','ko':':남자_하얀_피부_곱슬머리:','pt':':homem_pele_clara_e_cabelo_cacheado:','it':':uomo_carnagione_chiara_e_capelli_ricci:','fa':':مرد_پوست_سفید_موفرفری:','id':':pria_warna_kulit_cerah_rambut_keriting:','zh':':男人_较浅肤色卷发:','ru':':мужчина_очень_светлый_тон_кожи_кудрявые_волосы:','tr':':erkek_açık_cilt_tonu_kıvırcık_saç:','ar':':رجل_بشرة_بلون_فاتح_وشعر_مجعد:'},'\U0001F468\U0001F3FB\U0000200D\U0001F9B0':{'en':':man_light_skin_tone_red_hair:','status':fully_qualified,'E':11,'de':':mann_helle_hautfarbe_rotes_haar:','es':':hombre_tono_de_piel_claro_y_pelo_pelirrojo:','fr':':homme_peau_claire_et_cheveux_roux:','ja':':男性_薄い肌色_赤毛:','ko':':남자_하얀_피부_빨간_머리:','pt':':homem_pele_clara_e_cabelo_vermelho:','it':':uomo_carnagione_chiara_e_capelli_rossi:','fa':':مرد_پوست_سفید_موقرمز:','id':':pria_warna_kulit_cerah_rambut_merah:','zh':':男人_较浅肤色红发:','ru':':мужчина_очень_светлый_тон_кожи_рыжие_волосы:','tr':':erkek_açık_cilt_tonu_kızıl_saç:','ar':':رجل_بشرة_بلون_فاتح_وشعر_أحمر:'},'\U0001F468\U0001F3FB\U0000200D\U0001F9B3':{'en':':man_light_skin_tone_white_hair:','status':fully_qualified,'E':11,'de':':mann_helle_hautfarbe_weißes_haar:','es':':hombre_tono_de_piel_claro_y_pelo_blanco:','fr':':homme_peau_claire_et_cheveux_blancs:','ja':':男性_薄い肌色_白髪:','ko':':남자_하얀_피부_백발:','pt':':homem_pele_clara_e_cabelo_branco:','it':':uomo_carnagione_chiara_e_capelli_bianchi:','fa':':مرد_پوست_سفید_موسفید:','id':':pria_warna_kulit_cerah_rambut_putih:','zh':':男人_较浅肤色白发:','ru':':мужчина_очень_светлый_тон_кожи_седые_волосы:','tr':':erkek_açık_cilt_tonu_beyaz_saç:','ar':':رجل_بشرة_بلون_فاتح_وشعر_أبيض:'},'\U0001F9D9\U0000200D\U00002642\U0000FE0F':{'en':':man_mage:','status':fully_qualified,'E':5,'alias':[':mage_man:'],'de':':magier:','es':':mago:','fr':':mage_homme:','ja':':男の魔法使い:','ko':':남자_마법사:','pt':':homem_mago:','it':':mago_uomo:','fa':':ساحر_مرد:','id':':magi_pria:','zh':':男法师:','ru':':волшебник:','tr':':erkek_büyücü:','ar':':ساحر:'},'\U0001F9D9\U0000200D\U00002642':{'en':':man_mage:','status':minimally_qualified,'E':5,'alias':[':mage_man:'],'de':':magier:','es':':mago:','fr':':mage_homme:','ja':':男の魔法使い:','ko':':남자_마법사:','pt':':homem_mago:','it':':mago_uomo:','fa':':ساحر_مرد:','id':':magi_pria:','zh':':男法师:','ru':':волшебник:','tr':':erkek_büyücü:','ar':':ساحر:'},'\U0001F9D9\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_mage_dark_skin_tone:','status':fully_qualified,'E':5,'de':':magier_dunkle_hautfarbe:','es':':mago_tono_de_piel_oscuro:','fr':':mage_homme_peau_foncée:','ja':':男の魔法使い_濃い肌色:','ko':':남자_마법사_검은색_피부:','pt':':homem_mago_pele_escura:','it':':mago_uomo_carnagione_scura:','fa':':ساحر_مرد_پوست_آبنوسی:','id':':magi_pria_warna_kulit_gelap:','zh':':男法师_较深肤色:','ru':':волшебник_очень_темныи_тон_кожи:','tr':':erkek_büyücü_koyu_cilt_tonu:','ar':':ساحر_بشرة_بلون_غامق:'},'\U0001F9D9\U0001F3FF\U0000200D\U00002642':{'en':':man_mage_dark_skin_tone:','status':minimally_qualified,'E':5,'de':':magier_dunkle_hautfarbe:','es':':mago_tono_de_piel_oscuro:','fr':':mage_homme_peau_foncée:','ja':':男の魔法使い_濃い肌色:','ko':':남자_마법사_검은색_피부:','pt':':homem_mago_pele_escura:','it':':mago_uomo_carnagione_scura:','fa':':ساحر_مرد_پوست_آبنوسی:','id':':magi_pria_warna_kulit_gelap:','zh':':男法师_较深肤色:','ru':':волшебник_очень_темный_тон_кожи:','tr':':erkek_büyücü_koyu_cilt_tonu:','ar':':ساحر_بشرة_بلون_غامق:'},'\U0001F9D9\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_mage_light_skin_tone:','status':fully_qualified,'E':5,'de':':magier_helle_hautfarbe:','es':':mago_tono_de_piel_claro:','fr':':mage_homme_peau_claire:','ja':':男の魔法使い_薄い肌色:','ko':':남자_마법사_하얀_피부:','pt':':homem_mago_pele_clara:','it':':mago_uomo_carnagione_chiara:','fa':':ساحر_مرد_پوست_سفید:','id':':magi_pria_warna_kulit_cerah:','zh':':男法师_较浅肤色:','ru':':волшебник_очень_светлыи_тон_кожи:','tr':':erkek_büyücü_açık_cilt_tonu:','ar':':ساحر_بشرة_بلون_فاتح:'},'\U0001F9D9\U0001F3FB\U0000200D\U00002642':{'en':':man_mage_light_skin_tone:','status':minimally_qualified,'E':5,'de':':magier_helle_hautfarbe:','es':':mago_tono_de_piel_claro:','fr':':mage_homme_peau_claire:','ja':':男の魔法使い_薄い肌色:','ko':':남자_마법사_하얀_피부:','pt':':homem_mago_pele_clara:','it':':mago_uomo_carnagione_chiara:','fa':':ساحر_مرد_پوست_سفید:','id':':magi_pria_warna_kulit_cerah:','zh':':男法师_较浅肤色:','ru':':волшебник_очень_светлый_тон_кожи:','tr':':erkek_büyücü_açık_cilt_tonu:','ar':':ساحر_بشرة_بلون_فاتح:'},'\U0001F9D9\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_mage_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':magier_mitteldunkle_hautfarbe:','es':':mago_tono_de_piel_oscuro_medio:','fr':':mage_homme_peau_mate:','ja':':男の魔法使い_やや濃い肌色:','ko':':남자_마법사_진한_갈색_피부:','pt':':homem_mago_pele_morena_escura:','it':':mago_uomo_carnagione_abbastanza_scura:','fa':':ساحر_مرد_پوست_گندمی:','id':':magi_pria_warna_kulit_gelap-sedang:','zh':':男法师_中等深肤色:','ru':':волшебник_темныи_тон_кожи:','tr':':erkek_büyücü_orta_koyu_cilt_tonu:','ar':':ساحر_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D9\U0001F3FE\U0000200D\U00002642':{'en':':man_mage_medium-dark_skin_tone:','status':minimally_qualified,'E':5,'de':':magier_mitteldunkle_hautfarbe:','es':':mago_tono_de_piel_oscuro_medio:','fr':':mage_homme_peau_mate:','ja':':男の魔法使い_やや濃い肌色:','ko':':남자_마법사_진한_갈색_피부:','pt':':homem_mago_pele_morena_escura:','it':':mago_uomo_carnagione_abbastanza_scura:','fa':':ساحر_مرد_پوست_گندمی:','id':':magi_pria_warna_kulit_gelap-sedang:','zh':':男法师_中等深肤色:','ru':':волшебник_темный_тон_кожи:','tr':':erkek_büyücü_orta_koyu_cilt_tonu:','ar':':ساحر_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D9\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_mage_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':magier_mittelhelle_hautfarbe:','es':':mago_tono_de_piel_claro_medio:','fr':':mage_homme_peau_moyennement_claire:','ja':':男の魔法使い_やや薄い肌色:','ko':':남자_마법사_연한_갈색_피부:','pt':':homem_mago_pele_morena_clara:','it':':mago_uomo_carnagione_abbastanza_chiara:','fa':':ساحر_مرد_پوست_روشن:','id':':magi_pria_warna_kulit_cerah-sedang:','zh':':男法师_中等浅肤色:','ru':':волшебник_светлыи_тон_кожи:','tr':':erkek_büyücü_orta_açık_cilt_tonu:','ar':':ساحر_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D9\U0001F3FC\U0000200D\U00002642':{'en':':man_mage_medium-light_skin_tone:','status':minimally_qualified,'E':5,'de':':magier_mittelhelle_hautfarbe:','es':':mago_tono_de_piel_claro_medio:','fr':':mage_homme_peau_moyennement_claire:','ja':':男の魔法使い_やや薄い肌色:','ko':':남자_마법사_연한_갈색_피부:','pt':':homem_mago_pele_morena_clara:','it':':mago_uomo_carnagione_abbastanza_chiara:','fa':':ساحر_مرد_پوست_روشن:','id':':magi_pria_warna_kulit_cerah-sedang:','zh':':男法师_中等浅肤色:','ru':':волшебник_светлый_тон_кожи:','tr':':erkek_büyücü_orta_açık_cilt_tonu:','ar':':ساحر_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D9\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_mage_medium_skin_tone:','status':fully_qualified,'E':5,'de':':magier_mittlere_hautfarbe:','es':':mago_tono_de_piel_medio:','fr':':mage_homme_peau_légèrement_mate:','ja':':男の魔法使い_中間の肌色:','ko':':남자_마법사_갈색_피부:','pt':':homem_mago_pele_morena:','it':':mago_uomo_carnagione_olivastra:','fa':':ساحر_مرد_پوست_طلایی:','id':':magi_pria_warna_kulit_sedang:','zh':':男法师_中等肤色:','ru':':волшебник_среднии_тон_кожи:','tr':':erkek_büyücü_orta_cilt_tonu:','ar':':ساحر_بشرة_بلون_معتدل:'},'\U0001F9D9\U0001F3FD\U0000200D\U00002642':{'en':':man_mage_medium_skin_tone:','status':minimally_qualified,'E':5,'de':':magier_mittlere_hautfarbe:','es':':mago_tono_de_piel_medio:','fr':':mage_homme_peau_légèrement_mate:','ja':':男の魔法使い_中間の肌色:','ko':':남자_마법사_갈색_피부:','pt':':homem_mago_pele_morena:','it':':mago_uomo_carnagione_olivastra:','fa':':ساحر_مرد_پوست_طلایی:','id':':magi_pria_warna_kulit_sedang:','zh':':男法师_中等肤色:','ru':':волшебник_средний_тон_кожи:','tr':':erkek_büyücü_orta_cilt_tonu:','ar':':ساحر_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U0001F527':{'en':':man_mechanic:','status':fully_qualified,'E':4,'de':':mechaniker:','es':':profesional_de_la_mecánica_hombre:','fr':':mécanicien:','ja':':男性の整備士:','ko':':남자_정비공:','pt':':mecânico:','it':':meccanico_uomo:','fa':':تعمیرکار_مرد:','id':':montir_pria:','zh':':男技工:','ru':':мужчина_механик:','tr':':erkek_tamirci:','ar':':ميكانيكي:'},'\U0001F468\U0001F3FF\U0000200D\U0001F527':{'en':':man_mechanic_dark_skin_tone:','status':fully_qualified,'E':4,'de':':mechaniker_dunkle_hautfarbe:','es':':profesional_de_la_mecánica_hombre_tono_de_piel_oscuro:','fr':':mécanicien_peau_foncée:','ja':':男性の整備士_濃い肌色:','ko':':남자_정비공_검은색_피부:','pt':':mecânico_pele_escura:','it':':meccanico_uomo_carnagione_scura:','fa':':تعمیرکار_مرد_پوست_آبنوسی:','id':':montir_pria_warna_kulit_gelap:','zh':':男技工_较深肤色:','ru':':мужчина_механик_очень_темный_тон_кожи:','tr':':erkek_tamirci_koyu_cilt_tonu:','ar':':ميكانيكي_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U0001F527':{'en':':man_mechanic_light_skin_tone:','status':fully_qualified,'E':4,'de':':mechaniker_helle_hautfarbe:','es':':profesional_de_la_mecánica_hombre_tono_de_piel_claro:','fr':':mécanicien_peau_claire:','ja':':男性の整備士_薄い肌色:','ko':':남자_정비공_하얀_피부:','pt':':mecânico_pele_clara:','it':':meccanico_uomo_carnagione_chiara:','fa':':تعمیرکار_مرد_پوست_سفید:','id':':montir_pria_warna_kulit_cerah:','zh':':男技工_较浅肤色:','ru':':мужчина_механик_очень_светлый_тон_кожи:','tr':':erkek_tamirci_açık_cilt_tonu:','ar':':ميكانيكي_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F527':{'en':':man_mechanic_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':mechaniker_mitteldunkle_hautfarbe:','es':':profesional_de_la_mecánica_hombre_tono_de_piel_oscuro_medio:','fr':':mécanicien_peau_mate:','ja':':男性の整備士_やや濃い肌色:','ko':':남자_정비공_진한_갈색_피부:','pt':':mecânico_pele_morena_escura:','it':':meccanico_uomo_carnagione_abbastanza_scura:','fa':':تعمیرکار_مرد_پوست_گندمی:','id':':montir_pria_warna_kulit_gelap-sedang:','zh':':男技工_中等深肤色:','ru':':мужчина_механик_темный_тон_кожи:','tr':':erkek_tamirci_orta_koyu_cilt_tonu:','ar':':ميكانيكي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U0001F527':{'en':':man_mechanic_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':mechaniker_mittelhelle_hautfarbe:','es':':profesional_de_la_mecánica_hombre_tono_de_piel_claro_medio:','fr':':mécanicien_peau_moyennement_claire:','ja':':男性の整備士_やや薄い肌色:','ko':':남자_정비공_연한_갈색_피부:','pt':':mecânico_pele_morena_clara:','it':':meccanico_uomo_carnagione_abbastanza_chiara:','fa':':تعمیرکار_مرد_پوست_روشن:','id':':montir_pria_warna_kulit_cerah-sedang:','zh':':男技工_中等浅肤色:','ru':':мужчина_механик_светлый_тон_кожи:','tr':':erkek_tamirci_orta_açık_cilt_tonu:','ar':':ميكانيكي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F527':{'en':':man_mechanic_medium_skin_tone:','status':fully_qualified,'E':4,'de':':mechaniker_mittlere_hautfarbe:','es':':profesional_de_la_mecánica_hombre_tono_de_piel_medio:','fr':':mécanicien_peau_légèrement_mate:','ja':':男性の整備士_中間の肌色:','ko':':남자_정비공_갈색_피부:','pt':':mecânico_pele_morena:','it':':meccanico_uomo_carnagione_olivastra:','fa':':تعمیرکار_مرد_پوست_طلایی:','id':':montir_pria_warna_kulit_sedang:','zh':':男技工_中等肤色:','ru':':мужчина_механик_средний_тон_кожи:','tr':':erkek_tamirci_orta_cilt_tonu:','ar':':ميكانيكي_بشرة_بلون_معتدل:'},'\U0001F468\U0001F3FE':{'en':':man_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':mann_mitteldunkle_hautfarbe:','es':':hombre_tono_de_piel_oscuro_medio:','fr':':homme_peau_mate:','ja':':男性_やや濃い肌色:','ko':':남자_진한_갈색_피부:','pt':':homem_pele_morena_escura:','it':':uomo_carnagione_abbastanza_scura:','fa':':مرد_پوست_گندمی:','id':':pria_warna_kulit_gelap-sedang:','zh':':男人_中等深肤色:','ru':':мужчина_темный_тон_кожи:','tr':':erkek_orta_koyu_cilt_tonu:','ar':':رجل_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FE\U0000200D\U0001F9B2':{'en':':man_medium-dark_skin_tone_bald:','status':fully_qualified,'E':11,'de':':mann_mitteldunkle_hautfarbe_glatze:','es':':hombre_tono_de_piel_oscuro_medio_y_sin_pelo:','fr':':homme_peau_mate_et_chauve:','ja':':男性_やや濃い肌色_はげ頭:','ko':':남자_진한_갈색_피부_대머리:','pt':':homem_pele_morena_escura_e_careca:','it':':uomo_carnagione_abbastanza_scura_e_calvo:','fa':':مرد_پوست_گندمی_تاس:','id':':pria_warna_kulit_gelap-sedang_gundul:','zh':':男人_中等深肤色秃顶:','ru':':мужчина_темный_тон_кожи_без_волос:','tr':':erkek_orta_koyu_cilt_tonu_kel:','ar':':رجل_بشرة_بلون_معتدل_مائل_للغامق_وأصلع:'},'\U0001F9D4\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_medium-dark_skin_tone_beard:','status':fully_qualified,'E':13.1,'de':':mann_mitteldunkle_hautfarbe_bart:','es':':hombre_tono_de_piel_oscuro_medio_y_barba:','fr':':homme_barbu_peau_mate:','ja':':あごひげの男性_やや濃い肌色:','ko':':수염_난_남자_진한_갈색_피부:','pt':':homem_pele_morena_escura_e_barbudo:','it':':uomo_con_la_barba_carnagione_abbastanza_scura:','fa':':مرد_پوست_گندمی_ریش:','id':':pria_warna_kulit_gelap-sedang_janggut:','zh':':有络腮胡子的男人_中等深肤色:','ru':':бородатыи_мужчина_темныи_тон_кожи:','tr':':erkek_orta_koyu_cilt_tonu_sakal:','ar':':رجل_بلحية_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D4\U0001F3FE\U0000200D\U00002642':{'en':':man_medium-dark_skin_tone_beard:','status':minimally_qualified,'E':13.1,'de':':mann_mitteldunkle_hautfarbe_bart:','es':':hombre_tono_de_piel_oscuro_medio_y_barba:','fr':':homme_barbu_peau_mate:','ja':':あごひげの男性_やや濃い肌色:','ko':':수염_난_남자_진한_갈색_피부:','pt':':homem_pele_morena_escura_e_barbudo:','it':':uomo_con_la_barba_carnagione_abbastanza_scura:','fa':':مرد_پوست_گندمی_ریش:','id':':pria_warna_kulit_gelap-sedang_janggut:','zh':':有络腮胡子的男人_中等深肤色:','ru':':бородатый_мужчина_темный_тон_кожи:','tr':':erkek_orta_koyu_cilt_tonu_sakal:','ar':':رجل_بلحية_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F471\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_medium-dark_skin_tone_blond_hair:','status':fully_qualified,'E':4,'de':':mann_mitteldunkle_hautfarbe,_blond:','es':':hombre_rubio_tono_de_piel_oscuro_medio:','fr':':homme_blond_peau_mate:','ja':':金髪の男性_やや濃い肌色:','ko':':금발_남자_진한_갈색_피부:','pt':':homem_pele_morena_escura_e_cabelo_loiro:','it':':uomo_biondo_carnagione_abbastanza_scura:','fa':':مرد_پوست_گندمی_مو_بور:','id':':pria_berambut_pirang_warna_kulit_gelap-sedang:','zh':':金发男_中等深肤色:','ru':':блондин_темныи_тон_кожи:','tr':':sarışın_erkek_orta_koyu_cilt_tonu:','ar':':رجل_أشقر_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F471\U0001F3FE\U0000200D\U00002642':{'en':':man_medium-dark_skin_tone_blond_hair:','status':minimally_qualified,'E':4,'de':':mann_mitteldunkle_hautfarbe_blond:','es':':hombre_rubio_tono_de_piel_oscuro_medio:','fr':':homme_blond_peau_mate:','ja':':金髪の男性_やや濃い肌色:','ko':':금발_남자_진한_갈색_피부:','pt':':homem_pele_morena_escura_e_cabelo_loiro:','it':':uomo_biondo_carnagione_abbastanza_scura:','fa':':مرد_پوست_گندمی_مو_بور:','id':':pria_berambut_pirang_warna_kulit_gelap-sedang:','zh':':金发男_中等深肤色:','ru':':блондин_темный_тон_кожи:','tr':':sarışın_erkek_orta_koyu_cilt_tonu:','ar':':رجل_أشقر_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FE\U0000200D\U0001F9B1':{'en':':man_medium-dark_skin_tone_curly_hair:','status':fully_qualified,'E':11,'de':':mann_mitteldunkle_hautfarbe_lockiges_haar:','es':':hombre_tono_de_piel_oscuro_medio_y_pelo_rizado:','fr':':homme_peau_mate_et_cheveux_bouclés:','ja':':男性_やや濃い肌色_巻き毛:','ko':':남자_진한_갈색_피부_곱슬머리:','pt':':homem_pele_morena_escura_e_cabelo_cacheado:','it':':uomo_carnagione_abbastanza_scura_e_capelli_ricci:','fa':':مرد_پوست_گندمی_موفرفری:','id':':pria_warna_kulit_gelap-sedang_rambut_keriting:','zh':':男人_中等深肤色卷发:','ru':':мужчина_темный_тон_кожи_кудрявые_волосы:','tr':':erkek_orta_koyu_cilt_tonu_kıvırcık_saç:','ar':':رجل_بشرة_بلون_معتدل_مائل_للغامق_وشعر_مجعد:'},'\U0001F468\U0001F3FE\U0000200D\U0001F9B0':{'en':':man_medium-dark_skin_tone_red_hair:','status':fully_qualified,'E':11,'de':':mann_mitteldunkle_hautfarbe_rotes_haar:','es':':hombre_tono_de_piel_oscuro_medio_y_pelo_pelirrojo:','fr':':homme_peau_mate_et_cheveux_roux:','ja':':男性_やや濃い肌色_赤毛:','ko':':남자_진한_갈색_피부_빨간_머리:','pt':':homem_pele_morena_escura_e_cabelo_vermelho:','it':':uomo_carnagione_abbastanza_scura_e_capelli_rossi:','fa':':مرد_پوست_گندمی_موقرمز:','id':':pria_warna_kulit_gelap-sedang_rambut_merah:','zh':':男人_中等深肤色红发:','ru':':мужчина_темный_тон_кожи_рыжие_волосы:','tr':':erkek_orta_koyu_cilt_tonu_kızıl_saç:','ar':':رجل_بشرة_بلون_معتدل_مائل_للغامق_وشعر_أحمر:'},'\U0001F468\U0001F3FE\U0000200D\U0001F9B3':{'en':':man_medium-dark_skin_tone_white_hair:','status':fully_qualified,'E':11,'de':':mann_mitteldunkle_hautfarbe_weißes_haar:','es':':hombre_tono_de_piel_oscuro_medio_y_pelo_blanco:','fr':':homme_peau_mate_et_cheveux_blancs:','ja':':男性_やや濃い肌色_白髪:','ko':':남자_진한_갈색_피부_백발:','pt':':homem_pele_morena_escura_e_cabelo_branco:','it':':uomo_carnagione_abbastanza_scura_e_capelli_bianchi:','fa':':مرد_پوست_گندمی_موسفید:','id':':pria_warna_kulit_gelap-sedang_rambut_putih:','zh':':男人_中等深肤色白发:','ru':':мужчина_темный_тон_кожи_седые_волосы:','tr':':erkek_orta_koyu_cilt_tonu_beyaz_saç:','ar':':رجل_بشرة_بلون_معتدل_مائل_للغامق_وشعر_أبيض:'},'\U0001F468\U0001F3FC':{'en':':man_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':mann_mittelhelle_hautfarbe:','es':':hombre_tono_de_piel_claro_medio:','fr':':homme_peau_moyennement_claire:','ja':':男性_やや薄い肌色:','ko':':남자_연한_갈색_피부:','pt':':homem_pele_morena_clara:','it':':uomo_carnagione_abbastanza_chiara:','fa':':مرد_پوست_روشن:','id':':pria_warna_kulit_cerah-sedang:','zh':':男人_中等浅肤色:','ru':':мужчина_светлый_тон_кожи:','tr':':erkek_orta_açık_cilt_tonu:','ar':':رجل_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FC\U0000200D\U0001F9B2':{'en':':man_medium-light_skin_tone_bald:','status':fully_qualified,'E':11,'de':':mann_mittelhelle_hautfarbe_glatze:','es':':hombre_tono_de_piel_claro_medio_y_sin_pelo:','fr':':homme_peau_moyennement_claire_et_chauve:','ja':':男性_やや薄い肌色_はげ頭:','ko':':남자_연한_갈색_피부_대머리:','pt':':homem_pele_morena_clara_e_careca:','it':':uomo_carnagione_abbastanza_chiara_e_calvo:','fa':':مرد_پوست_روشن_تاس:','id':':pria_warna_kulit_cerah-sedang_gundul:','zh':':男人_中等浅肤色秃顶:','ru':':мужчина_светлый_тон_кожи_без_волос:','tr':':erkek_orta_açık_cilt_tonu_kel:','ar':':رجل_بشرة_بلون_فاتح_ومعتدل_وأصلع:'},'\U0001F9D4\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_medium-light_skin_tone_beard:','status':fully_qualified,'E':13.1,'de':':mann_mittelhelle_hautfarbe_bart:','es':':hombre_tono_de_piel_claro_medio_y_barba:','fr':':homme_barbu_peau_moyennement_claire:','ja':':あごひげの男性_やや薄い肌色:','ko':':수염_난_남자_연한_갈색_피부:','pt':':homem_pele_morena_clara_e_barbudo:','it':':uomo_con_la_barba_carnagione_abbastanza_chiara:','fa':':مرد_پوست_روشن_ریش:','id':':pria_warna_kulit_cerah-sedang_janggut:','zh':':有络腮胡子的男人_中等浅肤色:','ru':':бородатыи_мужчина_светлыи_тон_кожи:','tr':':erkek_orta_açık_cilt_tonu_sakal:','ar':':رجل_بلحية_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D4\U0001F3FC\U0000200D\U00002642':{'en':':man_medium-light_skin_tone_beard:','status':minimally_qualified,'E':13.1,'de':':mann_mittelhelle_hautfarbe_bart:','es':':hombre_tono_de_piel_claro_medio_y_barba:','fr':':homme_barbu_peau_moyennement_claire:','ja':':あごひげの男性_やや薄い肌色:','ko':':수염_난_남자_연한_갈색_피부:','pt':':homem_pele_morena_clara_e_barbudo:','it':':uomo_con_la_barba_carnagione_abbastanza_chiara:','fa':':مرد_پوست_روشن_ریش:','id':':pria_warna_kulit_cerah-sedang_janggut:','zh':':有络腮胡子的男人_中等浅肤色:','ru':':бородатый_мужчина_светлый_тон_кожи:','tr':':erkek_orta_açık_cilt_tonu_sakal:','ar':':رجل_بلحية_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F471\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_medium-light_skin_tone_blond_hair:','status':fully_qualified,'E':4,'de':':mann_mittelhelle_hautfarbe,_blond:','es':':hombre_rubio_tono_de_piel_claro_medio:','fr':':homme_blond_peau_moyennement_claire:','ja':':金髪の男性_やや薄い肌色:','ko':':금발_남자_연한_갈색_피부:','pt':':homem_pele_morena_clara_e_cabelo_loiro:','it':':uomo_biondo_carnagione_abbastanza_chiara:','fa':':مرد_پوست_روشن_مو_بور:','id':':pria_berambut_pirang_warna_kulit_cerah-sedang:','zh':':金发男_中等浅肤色:','ru':':блондин_светлыи_тон_кожи:','tr':':sarışın_erkek_orta_açık_cilt_tonu:','ar':':رجل_أشقر_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F471\U0001F3FC\U0000200D\U00002642':{'en':':man_medium-light_skin_tone_blond_hair:','status':minimally_qualified,'E':4,'de':':mann_mittelhelle_hautfarbe_blond:','es':':hombre_rubio_tono_de_piel_claro_medio:','fr':':homme_blond_peau_moyennement_claire:','ja':':金髪の男性_やや薄い肌色:','ko':':금발_남자_연한_갈색_피부:','pt':':homem_pele_morena_clara_e_cabelo_loiro:','it':':uomo_biondo_carnagione_abbastanza_chiara:','fa':':مرد_پوست_روشن_مو_بور:','id':':pria_berambut_pirang_warna_kulit_cerah-sedang:','zh':':金发男_中等浅肤色:','ru':':блондин_светлый_тон_кожи:','tr':':sarışın_erkek_orta_açık_cilt_tonu:','ar':':رجل_أشقر_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FC\U0000200D\U0001F9B1':{'en':':man_medium-light_skin_tone_curly_hair:','status':fully_qualified,'E':11,'de':':mann_mittelhelle_hautfarbe_lockiges_haar:','es':':hombre_tono_de_piel_claro_medio_y_pelo_rizado:','fr':':homme_peau_moyennement_claire_et_cheveux_bouclés:','ja':':男性_やや薄い肌色_巻き毛:','ko':':남자_연한_갈색_피부_곱슬머리:','pt':':homem_pele_morena_clara_e_cabelo_cacheado:','it':':uomo_carnagione_abbastanza_chiara_e_capelli_ricci:','fa':':مرد_پوست_روشن_موفرفری:','id':':pria_warna_kulit_cerah-sedang_rambut_keriting:','zh':':男人_中等浅肤色卷发:','ru':':мужчина_светлый_тон_кожи_кудрявые_волосы:','tr':':erkek_orta_açık_cilt_tonu_kıvırcık_saç:','ar':':رجل_بشرة_بلون_فاتح_ومعتدل_وشعر_مجعد:'},'\U0001F468\U0001F3FC\U0000200D\U0001F9B0':{'en':':man_medium-light_skin_tone_red_hair:','status':fully_qualified,'E':11,'de':':mann_mittelhelle_hautfarbe_rotes_haar:','es':':hombre_tono_de_piel_claro_medio_y_pelo_pelirrojo:','fr':':homme_peau_moyennement_claire_et_cheveux_roux:','ja':':男性_やや薄い肌色_赤毛:','ko':':남자_연한_갈색_피부_빨간_머리:','pt':':homem_pele_morena_clara_e_cabelo_vermelho:','it':':uomo_carnagione_abbastanza_chiara_e_capelli_rossi:','fa':':مرد_پوست_روشن_موقرمز:','id':':pria_warna_kulit_cerah-sedang_rambut_merah:','zh':':男人_中等浅肤色红发:','ru':':мужчина_светлый_тон_кожи_рыжие_волосы:','tr':':erkek_orta_açık_cilt_tonu_kızıl_saç:','ar':':رجل_بشرة_بلون_فاتح_ومعتدل_وشعر_أحمر:'},'\U0001F468\U0001F3FC\U0000200D\U0001F9B3':{'en':':man_medium-light_skin_tone_white_hair:','status':fully_qualified,'E':11,'de':':mann_mittelhelle_hautfarbe_weißes_haar:','es':':hombre_tono_de_piel_claro_medio_y_pelo_blanco:','fr':':homme_peau_moyennement_claire_et_cheveux_blancs:','ja':':男性_やや薄い肌色_白髪:','ko':':남자_연한_갈색_피부_백발:','pt':':homem_pele_morena_clara_e_cabelo_branco:','it':':uomo_carnagione_abbastanza_chiara_e_capelli_bianchi:','fa':':مرد_پوست_روشن_موسفید:','id':':pria_warna_kulit_cerah-sedang_rambut_putih:','zh':':男人_中等浅肤色白发:','ru':':мужчина_светлый_тон_кожи_седые_волосы:','tr':':erkek_orta_açık_cilt_tonu_beyaz_saç:','ar':':رجل_بشرة_بلون_فاتح_ومعتدل_وشعر_أبيض:'},'\U0001F468\U0001F3FD':{'en':':man_medium_skin_tone:','status':fully_qualified,'E':1,'de':':mann_mittlere_hautfarbe:','es':':hombre_tono_de_piel_medio:','fr':':homme_peau_légèrement_mate:','ja':':男性_中間の肌色:','ko':':남자_갈색_피부:','pt':':homem_pele_morena:','it':':uomo_carnagione_olivastra:','fa':':مرد_پوست_طلایی:','id':':pria_warna_kulit_sedang:','zh':':男人_中等肤色:','ru':':мужчина_средний_тон_кожи:','tr':':erkek_orta_cilt_tonu:','ar':':رجل_بشرة_بلون_معتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F9B2':{'en':':man_medium_skin_tone_bald:','status':fully_qualified,'E':11,'de':':mann_mittlere_hautfarbe_glatze:','es':':hombre_tono_de_piel_medio_y_sin_pelo:','fr':':homme_peau_légèrement_mate_et_chauve:','ja':':男性_中間の肌色_はげ頭:','ko':':남자_갈색_피부_대머리:','pt':':homem_pele_morena_e_careca:','it':':uomo_carnagione_olivastra_e_calvo:','fa':':مرد_پوست_طلایی_تاس:','id':':pria_warna_kulit_sedang_gundul:','zh':':男人_中等肤色秃顶:','ru':':мужчина_средний_тон_кожи_без_волос:','tr':':erkek_orta_cilt_tonu_kel:','ar':':رجل_بشرة_بلون_معتدل_وأصلع:'},'\U0001F9D4\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_medium_skin_tone_beard:','status':fully_qualified,'E':13.1,'de':':mann_mittlere_hautfarbe_bart:','es':':hombre_tono_de_piel_medio_y_barba:','fr':':homme_barbu_peau_légèrement_mate:','ja':':あごひげの男性_中間の肌色:','ko':':수염_난_남자_갈색_피부:','pt':':homem_pele_morena_e_barbudo:','it':':uomo_con_la_barba_carnagione_olivastra:','fa':':مرد_پوست_طلایی_ریش:','id':':pria_warna_kulit_sedang_janggut:','zh':':有络腮胡子的男人_中等肤色:','ru':':бородатыи_мужчина_среднии_тон_кожи:','tr':':erkek_orta_cilt_tonu_sakal:','ar':':رجل_بلحية_بشرة_بلون_معتدل:'},'\U0001F9D4\U0001F3FD\U0000200D\U00002642':{'en':':man_medium_skin_tone_beard:','status':minimally_qualified,'E':13.1,'de':':mann_mittlere_hautfarbe_bart:','es':':hombre_tono_de_piel_medio_y_barba:','fr':':homme_barbu_peau_légèrement_mate:','ja':':あごひげの男性_中間の肌色:','ko':':수염_난_남자_갈색_피부:','pt':':homem_pele_morena_e_barbudo:','it':':uomo_con_la_barba_carnagione_olivastra:','fa':':مرد_پوست_طلایی_ریش:','id':':pria_warna_kulit_sedang_janggut:','zh':':有络腮胡子的男人_中等肤色:','ru':':бородатый_мужчина_средний_тон_кожи:','tr':':erkek_orta_cilt_tonu_sakal:','ar':':رجل_بلحية_بشرة_بلون_معتدل:'},'\U0001F471\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_medium_skin_tone_blond_hair:','status':fully_qualified,'E':4,'de':':mann_mittlere_hautfarbe,_blond:','es':':hombre_rubio_tono_de_piel_medio:','fr':':homme_blond_peau_légèrement_mate:','ja':':金髪の男性_中間の肌色:','ko':':금발_남자_갈색_피부:','pt':':homem_pele_morena_e_cabelo_loiro:','it':':uomo_biondo_carnagione_olivastra:','fa':':مرد_پوست_طلایی_مو_بور:','id':':pria_berambut_pirang_warna_kulit_sedang:','zh':':金发男_中等肤色:','ru':':блондин_среднии_тон_кожи:','tr':':sarışın_erkek_orta_cilt_tonu:','ar':':رجل_أشقر_بشرة_بلون_معتدل:'},'\U0001F471\U0001F3FD\U0000200D\U00002642':{'en':':man_medium_skin_tone_blond_hair:','status':minimally_qualified,'E':4,'de':':mann_mittlere_hautfarbe_blond:','es':':hombre_rubio_tono_de_piel_medio:','fr':':homme_blond_peau_légèrement_mate:','ja':':金髪の男性_中間の肌色:','ko':':금발_남자_갈색_피부:','pt':':homem_pele_morena_e_cabelo_loiro:','it':':uomo_biondo_carnagione_olivastra:','fa':':مرد_پوست_طلایی_مو_بور:','id':':pria_berambut_pirang_warna_kulit_sedang:','zh':':金发男_中等肤色:','ru':':блондин_средний_тон_кожи:','tr':':sarışın_erkek_orta_cilt_tonu:','ar':':رجل_أشقر_بشرة_بلون_معتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F9B1':{'en':':man_medium_skin_tone_curly_hair:','status':fully_qualified,'E':11,'de':':mann_mittlere_hautfarbe_lockiges_haar:','es':':hombre_tono_de_piel_medio_y_pelo_rizado:','fr':':homme_peau_légèrement_mate_et_cheveux_bouclés:','ja':':男性_中間の肌色_巻き毛:','ko':':남자_갈색_피부_곱슬머리:','pt':':homem_pele_morena_e_cabelo_cacheado:','it':':uomo_carnagione_olivastra_e_capelli_ricci:','fa':':مرد_پوست_طلایی_موفرفری:','id':':pria_warna_kulit_sedang_rambut_keriting:','zh':':男人_中等肤色卷发:','ru':':мужчина_средний_тон_кожи_кудрявые_волосы:','tr':':erkek_orta_cilt_tonu_kıvırcık_saç:','ar':':رجل_بشرة_بلون_معتدل_وشعر_مجعد:'},'\U0001F468\U0001F3FD\U0000200D\U0001F9B0':{'en':':man_medium_skin_tone_red_hair:','status':fully_qualified,'E':11,'de':':mann_mittlere_hautfarbe_rotes_haar:','es':':hombre_tono_de_piel_medio_y_pelo_pelirrojo:','fr':':homme_peau_légèrement_mate_et_cheveux_roux:','ja':':男性_中間の肌色_赤毛:','ko':':남자_갈색_피부_빨간_머리:','pt':':homem_pele_morena_e_cabelo_vermelho:','it':':uomo_carnagione_olivastra_e_capelli_rossi:','fa':':مرد_پوست_طلایی_موقرمز:','id':':pria_warna_kulit_sedang_rambut_merah:','zh':':男人_中等肤色红发:','ru':':мужчина_средний_тон_кожи_рыжие_волосы:','tr':':erkek_orta_cilt_tonu_kızıl_saç:','ar':':رجل_بشرة_بلون_معتدل_وشعر_أحمر:'},'\U0001F468\U0001F3FD\U0000200D\U0001F9B3':{'en':':man_medium_skin_tone_white_hair:','status':fully_qualified,'E':11,'de':':mann_mittlere_hautfarbe_weißes_haar:','es':':hombre_tono_de_piel_medio_y_pelo_blanco:','fr':':homme_peau_légèrement_mate_et_cheveux_blancs:','ja':':男性_中間の肌色_白髪:','ko':':남자_갈색_피부_백발:','pt':':homem_pele_morena_e_cabelo_branco:','it':':uomo_carnagione_olivastra_e_capelli_bianchi:','fa':':مرد_پوست_طلایی_موسفید:','id':':pria_warna_kulit_sedang_rambut_putih:','zh':':男人_中等肤色白发:','ru':':мужчина_средний_тон_кожи_седые_волосы:','tr':':erkek_orta_cilt_tonu_beyaz_saç:','ar':':رجل_بشرة_بلون_معتدل_وشعر_أبيض:'},'\U0001F6B5\U0000200D\U00002642\U0000FE0F':{'en':':man_mountain_biking:','status':fully_qualified,'E':4,'alias':[':mountain_biking_man:'],'de':':mountainbiker:','es':':hombre_en_bicicleta_de_montaña:','fr':':homme_en_vtt:','ja':':マウンテンバイクに乗る男:','ko':':산악_자전거_타는_남자:','pt':':homem_fazendo_mountain_bike:','it':':ciclista_uomo_di_mountain_bike:','fa':':مرد_و_دوچرخه_سواری_کوهستان:','id':':pria_bersepeda_gunung:','zh':':男生骑山地车:','ru':':мужчина_на_горном_велосипеде:','tr':':dağ_bisikleti_süren_erkek:','ar':':متسلق_جبال_بدراجة:'},'\U0001F6B5\U0000200D\U00002642':{'en':':man_mountain_biking:','status':minimally_qualified,'E':4,'alias':[':mountain_biking_man:'],'de':':mountainbiker:','es':':hombre_en_bicicleta_de_montaña:','fr':':homme_en_vtt:','ja':':マウンテンバイクに乗る男:','ko':':산악_자전거_타는_남자:','pt':':homem_fazendo_mountain_bike:','it':':ciclista_uomo_di_mountain_bike:','fa':':مرد_و_دوچرخه_سواری_کوهستان:','id':':pria_bersepeda_gunung:','zh':':男生骑山地车:','ru':':мужчина_на_горном_велосипеде:','tr':':dağ_bisikleti_süren_erkek:','ar':':متسلق_جبال_بدراجة:'},'\U0001F6B5\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_mountain_biking_dark_skin_tone:','status':fully_qualified,'E':4,'de':':mountainbiker_dunkle_hautfarbe:','es':':hombre_en_bicicleta_de_montaña_tono_de_piel_oscuro:','fr':':homme_en_vtt_peau_foncée:','ja':':マウンテンバイクに乗る男_濃い肌色:','ko':':산악_자전거_타는_남자_검은색_피부:','pt':':homem_fazendo_mountain_bike_pele_escura:','it':':ciclista_uomo_di_mountain_bike_carnagione_scura:','fa':':مرد_و_دوچرخه_سواری_کوهستان_پوست_آبنوسی:','id':':pria_bersepeda_gunung_warna_kulit_gelap:','zh':':男生骑山地车_较深肤色:','ru':':мужчина_на_горном_велосипеде_очень_темныи_тон_кожи:','tr':':dağ_bisikleti_süren_erkek_koyu_cilt_tonu:','ar':':متسلق_جبال_بدراجة_بشرة_بلون_غامق:'},'\U0001F6B5\U0001F3FF\U0000200D\U00002642':{'en':':man_mountain_biking_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':mountainbiker_dunkle_hautfarbe:','es':':hombre_en_bicicleta_de_montaña_tono_de_piel_oscuro:','fr':':homme_en_vtt_peau_foncée:','ja':':マウンテンバイクに乗る男_濃い肌色:','ko':':산악_자전거_타는_남자_검은색_피부:','pt':':homem_fazendo_mountain_bike_pele_escura:','it':':ciclista_uomo_di_mountain_bike_carnagione_scura:','fa':':مرد_و_دوچرخه_سواری_کوهستان_پوست_آبنوسی:','id':':pria_bersepeda_gunung_warna_kulit_gelap:','zh':':男生骑山地车_较深肤色:','ru':':мужчина_на_горном_велосипеде_очень_темный_тон_кожи:','tr':':dağ_bisikleti_süren_erkek_koyu_cilt_tonu:','ar':':متسلق_جبال_بدراجة_بشرة_بلون_غامق:'},'\U0001F6B5\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_mountain_biking_light_skin_tone:','status':fully_qualified,'E':4,'de':':mountainbiker_helle_hautfarbe:','es':':hombre_en_bicicleta_de_montaña_tono_de_piel_claro:','fr':':homme_en_vtt_peau_claire:','ja':':マウンテンバイクに乗る男_薄い肌色:','ko':':산악_자전거_타는_남자_하얀_피부:','pt':':homem_fazendo_mountain_bike_pele_clara:','it':':ciclista_uomo_di_mountain_bike_carnagione_chiara:','fa':':مرد_و_دوچرخه_سواری_کوهستان_پوست_سفید:','id':':pria_bersepeda_gunung_warna_kulit_cerah:','zh':':男生骑山地车_较浅肤色:','ru':':мужчина_на_горном_велосипеде_очень_светлыи_тон_кожи:','tr':':dağ_bisikleti_süren_erkek_açık_cilt_tonu:','ar':':متسلق_جبال_بدراجة_بشرة_بلون_فاتح:'},'\U0001F6B5\U0001F3FB\U0000200D\U00002642':{'en':':man_mountain_biking_light_skin_tone:','status':minimally_qualified,'E':4,'de':':mountainbiker_helle_hautfarbe:','es':':hombre_en_bicicleta_de_montaña_tono_de_piel_claro:','fr':':homme_en_vtt_peau_claire:','ja':':マウンテンバイクに乗る男_薄い肌色:','ko':':산악_자전거_타는_남자_하얀_피부:','pt':':homem_fazendo_mountain_bike_pele_clara:','it':':ciclista_uomo_di_mountain_bike_carnagione_chiara:','fa':':مرد_و_دوچرخه_سواری_کوهستان_پوست_سفید:','id':':pria_bersepeda_gunung_warna_kulit_cerah:','zh':':男生骑山地车_较浅肤色:','ru':':мужчина_на_горном_велосипеде_очень_светлый_тон_кожи:','tr':':dağ_bisikleti_süren_erkek_açık_cilt_tonu:','ar':':متسلق_جبال_بدراجة_بشرة_بلون_فاتح:'},'\U0001F6B5\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_mountain_biking_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':mountainbiker_mitteldunkle_hautfarbe:','es':':hombre_en_bicicleta_de_montaña_tono_de_piel_oscuro_medio:','fr':':homme_en_vtt_peau_mate:','ja':':マウンテンバイクに乗る男_やや濃い肌色:','ko':':산악_자전거_타는_남자_진한_갈색_피부:','pt':':homem_fazendo_mountain_bike_pele_morena_escura:','it':':ciclista_uomo_di_mountain_bike_carnagione_abbastanza_scura:','fa':':مرد_و_دوچرخه_سواری_کوهستان_پوست_گندمی:','id':':pria_bersepeda_gunung_warna_kulit_gelap-sedang:','zh':':男生骑山地车_中等深肤色:','ru':':мужчина_на_горном_велосипеде_темныи_тон_кожи:','tr':':dağ_bisikleti_süren_erkek_orta_koyu_cilt_tonu:','ar':':متسلق_جبال_بدراجة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6B5\U0001F3FE\U0000200D\U00002642':{'en':':man_mountain_biking_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':mountainbiker_mitteldunkle_hautfarbe:','es':':hombre_en_bicicleta_de_montaña_tono_de_piel_oscuro_medio:','fr':':homme_en_vtt_peau_mate:','ja':':マウンテンバイクに乗る男_やや濃い肌色:','ko':':산악_자전거_타는_남자_진한_갈색_피부:','pt':':homem_fazendo_mountain_bike_pele_morena_escura:','it':':ciclista_uomo_di_mountain_bike_carnagione_abbastanza_scura:','fa':':مرد_و_دوچرخه_سواری_کوهستان_پوست_گندمی:','id':':pria_bersepeda_gunung_warna_kulit_gelap-sedang:','zh':':男生骑山地车_中等深肤色:','ru':':мужчина_на_горном_велосипеде_темный_тон_кожи:','tr':':dağ_bisikleti_süren_erkek_orta_koyu_cilt_tonu:','ar':':متسلق_جبال_بدراجة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6B5\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_mountain_biking_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':mountainbiker_mittelhelle_hautfarbe:','es':':hombre_en_bicicleta_de_montaña_tono_de_piel_claro_medio:','fr':':homme_en_vtt_peau_moyennement_claire:','ja':':マウンテンバイクに乗る男_やや薄い肌色:','ko':':산악_자전거_타는_남자_연한_갈색_피부:','pt':':homem_fazendo_mountain_bike_pele_morena_clara:','it':':ciclista_uomo_di_mountain_bike_carnagione_abbastanza_chiara:','fa':':مرد_و_دوچرخه_سواری_کوهستان_پوست_روشن:','id':':pria_bersepeda_gunung_warna_kulit_cerah-sedang:','zh':':男生骑山地车_中等浅肤色:','ru':':мужчина_на_горном_велосипеде_светлыи_тон_кожи:','tr':':dağ_bisikleti_süren_erkek_orta_açık_cilt_tonu:','ar':':متسلق_جبال_بدراجة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6B5\U0001F3FC\U0000200D\U00002642':{'en':':man_mountain_biking_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':mountainbiker_mittelhelle_hautfarbe:','es':':hombre_en_bicicleta_de_montaña_tono_de_piel_claro_medio:','fr':':homme_en_vtt_peau_moyennement_claire:','ja':':マウンテンバイクに乗る男_やや薄い肌色:','ko':':산악_자전거_타는_남자_연한_갈색_피부:','pt':':homem_fazendo_mountain_bike_pele_morena_clara:','it':':ciclista_uomo_di_mountain_bike_carnagione_abbastanza_chiara:','fa':':مرد_و_دوچرخه_سواری_کوهستان_پوست_روشن:','id':':pria_bersepeda_gunung_warna_kulit_cerah-sedang:','zh':':男生骑山地车_中等浅肤色:','ru':':мужчина_на_горном_велосипеде_светлый_тон_кожи:','tr':':dağ_bisikleti_süren_erkek_orta_açık_cilt_tonu:','ar':':متسلق_جبال_بدراجة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6B5\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_mountain_biking_medium_skin_tone:','status':fully_qualified,'E':4,'de':':mountainbiker_mittlere_hautfarbe:','es':':hombre_en_bicicleta_de_montaña_tono_de_piel_medio:','fr':':homme_en_vtt_peau_légèrement_mate:','ja':':マウンテンバイクに乗る男_中間の肌色:','ko':':산악_자전거_타는_남자_갈색_피부:','pt':':homem_fazendo_mountain_bike_pele_morena:','it':':ciclista_uomo_di_mountain_bike_carnagione_olivastra:','fa':':مرد_و_دوچرخه_سواری_کوهستان_پوست_طلایی:','id':':pria_bersepeda_gunung_warna_kulit_sedang:','zh':':男生骑山地车_中等肤色:','ru':':мужчина_на_горном_велосипеде_среднии_тон_кожи:','tr':':dağ_bisikleti_süren_erkek_orta_cilt_tonu:','ar':':متسلق_جبال_بدراجة_بشرة_بلون_معتدل:'},'\U0001F6B5\U0001F3FD\U0000200D\U00002642':{'en':':man_mountain_biking_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':mountainbiker_mittlere_hautfarbe:','es':':hombre_en_bicicleta_de_montaña_tono_de_piel_medio:','fr':':homme_en_vtt_peau_légèrement_mate:','ja':':マウンテンバイクに乗る男_中間の肌色:','ko':':산악_자전거_타는_남자_갈색_피부:','pt':':homem_fazendo_mountain_bike_pele_morena:','it':':ciclista_uomo_di_mountain_bike_carnagione_olivastra:','fa':':مرد_و_دوچرخه_سواری_کوهستان_پوست_طلایی:','id':':pria_bersepeda_gunung_warna_kulit_sedang:','zh':':男生骑山地车_中等肤色:','ru':':мужчина_на_горном_велосипеде_средний_тон_кожи:','tr':':dağ_bisikleti_süren_erkek_orta_cilt_tonu:','ar':':متسلق_جبال_بدراجة_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U0001F4BC':{'en':':man_office_worker:','status':fully_qualified,'E':4,'de':':büroangestellter:','es':':oficinista_hombre:','fr':':employé_de_bureau:','ja':':男性会社員:','ko':':남자_회사원:','pt':':funcionário_de_escritório:','it':':impiegato:','fa':':کارمند_مرد:','id':':pekerja_kantor_pria:','zh':':男白领:','ru':':офисный_работник_мужчина:','tr':':erkek_ofis_çalışanı:','ar':':موظف:'},'\U0001F468\U0001F3FF\U0000200D\U0001F4BC':{'en':':man_office_worker_dark_skin_tone:','status':fully_qualified,'E':4,'de':':büroangestellter_dunkle_hautfarbe:','es':':oficinista_hombre_tono_de_piel_oscuro:','fr':':employé_de_bureau_peau_foncée:','ja':':男性会社員_濃い肌色:','ko':':남자_회사원_검은색_피부:','pt':':funcionário_de_escritório_pele_escura:','it':':impiegato_carnagione_scura:','fa':':کارمند_مرد_پوست_آبنوسی:','id':':pekerja_kantor_pria_warna_kulit_gelap:','zh':':男白领_较深肤色:','ru':':офисный_работник_мужчина_очень_темный_тон_кожи:','tr':':erkek_ofis_çalışanı_koyu_cilt_tonu:','ar':':موظف_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U0001F4BC':{'en':':man_office_worker_light_skin_tone:','status':fully_qualified,'E':4,'de':':büroangestellter_helle_hautfarbe:','es':':oficinista_hombre_tono_de_piel_claro:','fr':':employé_de_bureau_peau_claire:','ja':':男性会社員_薄い肌色:','ko':':남자_회사원_하얀_피부:','pt':':funcionário_de_escritório_pele_clara:','it':':impiegato_carnagione_chiara:','fa':':کارمند_مرد_پوست_سفید:','id':':pekerja_kantor_pria_warna_kulit_cerah:','zh':':男白领_较浅肤色:','ru':':офисный_работник_мужчина_очень_светлый_тон_кожи:','tr':':erkek_ofis_çalışanı_açık_cilt_tonu:','ar':':موظف_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F4BC':{'en':':man_office_worker_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':büroangestellter_mitteldunkle_hautfarbe:','es':':oficinista_hombre_tono_de_piel_oscuro_medio:','fr':':employé_de_bureau_peau_mate:','ja':':男性会社員_やや濃い肌色:','ko':':남자_회사원_진한_갈색_피부:','pt':':funcionário_de_escritório_pele_morena_escura:','it':':impiegato_carnagione_abbastanza_scura:','fa':':کارمند_مرد_پوست_گندمی:','id':':pekerja_kantor_pria_warna_kulit_gelap-sedang:','zh':':男白领_中等深肤色:','ru':':офисный_работник_мужчина_темный_тон_кожи:','tr':':erkek_ofis_çalışanı_orta_koyu_cilt_tonu:','ar':':موظف_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U0001F4BC':{'en':':man_office_worker_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':büroangestellter_mittelhelle_hautfarbe:','es':':oficinista_hombre_tono_de_piel_claro_medio:','fr':':employé_de_bureau_peau_moyennement_claire:','ja':':男性会社員_やや薄い肌色:','ko':':남자_회사원_연한_갈색_피부:','pt':':funcionário_de_escritório_pele_morena_clara:','it':':impiegato_carnagione_abbastanza_chiara:','fa':':کارمند_مرد_پوست_روشن:','id':':pekerja_kantor_pria_warna_kulit_cerah-sedang:','zh':':男白领_中等浅肤色:','ru':':офисный_работник_мужчина_светлый_тон_кожи:','tr':':erkek_ofis_çalışanı_orta_açık_cilt_tonu:','ar':':موظف_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F4BC':{'en':':man_office_worker_medium_skin_tone:','status':fully_qualified,'E':4,'de':':büroangestellter_mittlere_hautfarbe:','es':':oficinista_hombre_tono_de_piel_medio:','fr':':employé_de_bureau_peau_légèrement_mate:','ja':':男性会社員_中間の肌色:','ko':':남자_회사원_갈색_피부:','pt':':funcionário_de_escritório_pele_morena:','it':':impiegato_carnagione_olivastra:','fa':':کارمند_مرد_پوست_طلایی:','id':':pekerja_kantor_pria_warna_kulit_sedang:','zh':':男白领_中等肤色:','ru':':офисный_работник_мужчина_средний_тон_кожи:','tr':':erkek_ofis_çalışanı_orta_cilt_tonu:','ar':':موظف_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U00002708\U0000FE0F':{'en':':man_pilot:','status':fully_qualified,'E':4,'de':':pilot:','es':':piloto_hombre:','fr':':pilote_homme:','ja':':男性パイロット:','ko':':남자_기장:','pt':':piloto_de_avião_homem:','it':':pilota_uomo:','fa':':خلبان_مرد:','id':':pilot_pria:','zh':':男飞行员:','ru':':мужчина-пилот:','tr':':erkek_pilot:','ar':':طيار:'},'\U0001F468\U0000200D\U00002708':{'en':':man_pilot:','status':minimally_qualified,'E':4,'de':':pilot:','es':':piloto_hombre:','fr':':pilote_homme:','ja':':男性パイロット:','ko':':남자_기장:','pt':':piloto_de_avião_homem:','it':':pilota_uomo:','fa':':خلبان_مرد:','id':':pilot_pria:','zh':':男飞行员:','ru':':мужчина_пилот:','tr':':erkek_pilot:','ar':':طيار:'},'\U0001F468\U0001F3FF\U0000200D\U00002708\U0000FE0F':{'en':':man_pilot_dark_skin_tone:','status':fully_qualified,'E':4,'de':':pilot_dunkle_hautfarbe:','es':':piloto_hombre_tono_de_piel_oscuro:','fr':':pilote_homme_peau_foncée:','ja':':男性パイロット_濃い肌色:','ko':':남자_기장_검은색_피부:','pt':':piloto_de_avião_homem_pele_escura:','it':':pilota_uomo_carnagione_scura:','fa':':خلبان_مرد_پوست_آبنوسی:','id':':pilot_pria_warna_kulit_gelap:','zh':':男飞行员_较深肤色:','ru':':мужчина-пилот_очень_темныи_тон_кожи:','tr':':erkek_pilot_koyu_cilt_tonu:','ar':':طيار_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FF\U0000200D\U00002708':{'en':':man_pilot_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':pilot_dunkle_hautfarbe:','es':':piloto_hombre_tono_de_piel_oscuro:','fr':':pilote_homme_peau_foncée:','ja':':男性パイロット_濃い肌色:','ko':':남자_기장_검은색_피부:','pt':':piloto_de_avião_homem_pele_escura:','it':':pilota_uomo_carnagione_scura:','fa':':خلبان_مرد_پوست_آبنوسی:','id':':pilot_pria_warna_kulit_gelap:','zh':':男飞行员_较深肤色:','ru':':мужчина_пилот_очень_темный_тон_кожи:','tr':':erkek_pilot_koyu_cilt_tonu:','ar':':طيار_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U00002708\U0000FE0F':{'en':':man_pilot_light_skin_tone:','status':fully_qualified,'E':4,'de':':pilot_helle_hautfarbe:','es':':piloto_hombre_tono_de_piel_claro:','fr':':pilote_homme_peau_claire:','ja':':男性パイロット_薄い肌色:','ko':':남자_기장_하얀_피부:','pt':':piloto_de_avião_homem_pele_clara:','it':':pilota_uomo_carnagione_chiara:','fa':':خلبان_مرد_پوست_سفید:','id':':pilot_pria_warna_kulit_cerah:','zh':':男飞行员_较浅肤色:','ru':':мужчина-пилот_очень_светлыи_тон_кожи:','tr':':erkek_pilot_açık_cilt_tonu:','ar':':طيار_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FB\U0000200D\U00002708':{'en':':man_pilot_light_skin_tone:','status':minimally_qualified,'E':4,'de':':pilot_helle_hautfarbe:','es':':piloto_hombre_tono_de_piel_claro:','fr':':pilote_homme_peau_claire:','ja':':男性パイロット_薄い肌色:','ko':':남자_기장_하얀_피부:','pt':':piloto_de_avião_homem_pele_clara:','it':':pilota_uomo_carnagione_chiara:','fa':':خلبان_مرد_پوست_سفید:','id':':pilot_pria_warna_kulit_cerah:','zh':':男飞行员_较浅肤色:','ru':':мужчина_пилот_очень_светлый_тон_кожи:','tr':':erkek_pilot_açık_cilt_tonu:','ar':':طيار_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U00002708\U0000FE0F':{'en':':man_pilot_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':pilot_mitteldunkle_hautfarbe:','es':':piloto_hombre_tono_de_piel_oscuro_medio:','fr':':pilote_homme_peau_mate:','ja':':男性パイロット_やや濃い肌色:','ko':':남자_기장_진한_갈색_피부:','pt':':piloto_de_avião_homem_pele_morena_escura:','it':':pilota_uomo_carnagione_abbastanza_scura:','fa':':خلبان_مرد_پوست_گندمی:','id':':pilot_pria_warna_kulit_gelap-sedang:','zh':':男飞行员_中等深肤色:','ru':':мужчина-пилот_темныи_тон_кожи:','tr':':erkek_pilot_orta_koyu_cilt_tonu:','ar':':طيار_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FE\U0000200D\U00002708':{'en':':man_pilot_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':pilot_mitteldunkle_hautfarbe:','es':':piloto_hombre_tono_de_piel_oscuro_medio:','fr':':pilote_homme_peau_mate:','ja':':男性パイロット_やや濃い肌色:','ko':':남자_기장_진한_갈색_피부:','pt':':piloto_de_avião_homem_pele_morena_escura:','it':':pilota_uomo_carnagione_abbastanza_scura:','fa':':خلبان_مرد_پوست_گندمی:','id':':pilot_pria_warna_kulit_gelap-sedang:','zh':':男飞行员_中等深肤色:','ru':':мужчина_пилот_темный_тон_кожи:','tr':':erkek_pilot_orta_koyu_cilt_tonu:','ar':':طيار_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U00002708\U0000FE0F':{'en':':man_pilot_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':pilot_mittelhelle_hautfarbe:','es':':piloto_hombre_tono_de_piel_claro_medio:','fr':':pilote_homme_peau_moyennement_claire:','ja':':男性パイロット_やや薄い肌色:','ko':':남자_기장_연한_갈색_피부:','pt':':piloto_de_avião_homem_pele_morena_clara:','it':':pilota_uomo_carnagione_abbastanza_chiara:','fa':':خلبان_مرد_پوست_روشن:','id':':pilot_pria_warna_kulit_cerah-sedang:','zh':':男飞行员_中等浅肤色:','ru':':мужчина-пилот_светлыи_тон_кожи:','tr':':erkek_pilot_orta_açık_cilt_tonu:','ar':':طيار_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FC\U0000200D\U00002708':{'en':':man_pilot_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':pilot_mittelhelle_hautfarbe:','es':':piloto_hombre_tono_de_piel_claro_medio:','fr':':pilote_homme_peau_moyennement_claire:','ja':':男性パイロット_やや薄い肌色:','ko':':남자_기장_연한_갈색_피부:','pt':':piloto_de_avião_homem_pele_morena_clara:','it':':pilota_uomo_carnagione_abbastanza_chiara:','fa':':خلبان_مرد_پوست_روشن:','id':':pilot_pria_warna_kulit_cerah-sedang:','zh':':男飞行员_中等浅肤色:','ru':':мужчина_пилот_светлый_тон_кожи:','tr':':erkek_pilot_orta_açık_cilt_tonu:','ar':':طيار_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U00002708\U0000FE0F':{'en':':man_pilot_medium_skin_tone:','status':fully_qualified,'E':4,'de':':pilot_mittlere_hautfarbe:','es':':piloto_hombre_tono_de_piel_medio:','fr':':pilote_homme_peau_légèrement_mate:','ja':':男性パイロット_中間の肌色:','ko':':남자_기장_갈색_피부:','pt':':piloto_de_avião_homem_pele_morena:','it':':pilota_uomo_carnagione_olivastra:','fa':':خلبان_مرد_پوست_طلایی:','id':':pilot_pria_warna_kulit_sedang:','zh':':男飞行员_中等肤色:','ru':':мужчина-пилот_среднии_тон_кожи:','tr':':erkek_pilot_orta_cilt_tonu:','ar':':طيار_بشرة_بلون_معتدل:'},'\U0001F468\U0001F3FD\U0000200D\U00002708':{'en':':man_pilot_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':pilot_mittlere_hautfarbe:','es':':piloto_hombre_tono_de_piel_medio:','fr':':pilote_homme_peau_légèrement_mate:','ja':':男性パイロット_中間の肌色:','ko':':남자_기장_갈색_피부:','pt':':piloto_de_avião_homem_pele_morena:','it':':pilota_uomo_carnagione_olivastra:','fa':':خلبان_مرد_پوست_طلایی:','id':':pilot_pria_warna_kulit_sedang:','zh':':男飞行员_中等肤色:','ru':':мужчина_пилот_средний_тон_кожи:','tr':':erkek_pilot_orta_cilt_tonu:','ar':':طيار_بشرة_بلون_معتدل:'},'\U0001F93E\U0000200D\U00002642\U0000FE0F':{'en':':man_playing_handball:','status':fully_qualified,'E':4,'de':':handballspieler:','es':':hombre_jugando_al_balonmano:','fr':':handballeur:','ja':':ハンドボールをする男:','ko':':핸드볼하는_남자:','pt':':jogador_de_handebol:','it':':pallamanista_uomo:','fa':':هندبالیست_مرد:','id':':pria_bermain_bola_tangan:','zh':':男生玩手球:','ru':':гандболист:','tr':':hentbol_oynayan_erkek:','ar':':لاعب_كرة_يد:'},'\U0001F93E\U0000200D\U00002642':{'en':':man_playing_handball:','status':minimally_qualified,'E':4,'de':':handballspieler:','es':':hombre_jugando_al_balonmano:','fr':':handballeur:','ja':':ハンドボールをする男:','ko':':핸드볼하는_남자:','pt':':jogador_de_handebol:','it':':pallamanista_uomo:','fa':':هندبالیست_مرد:','id':':pria_bermain_bola_tangan:','zh':':男生玩手球:','ru':':гандболист:','tr':':hentbol_oynayan_erkek:','ar':':لاعب_كرة_يد:'},'\U0001F93E\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_playing_handball_dark_skin_tone:','status':fully_qualified,'E':4,'de':':handballspieler_dunkle_hautfarbe:','es':':hombre_jugando_al_balonmano_tono_de_piel_oscuro:','fr':':handballeur_peau_foncée:','ja':':ハンドボールをする男_濃い肌色:','ko':':핸드볼하는_남자_검은색_피부:','pt':':jogador_de_handebol_pele_escura:','it':':pallamanista_uomo_carnagione_scura:','fa':':هندبالیست_مرد_پوست_آبنوسی:','id':':pria_bermain_bola_tangan_warna_kulit_gelap:','zh':':男生玩手球_较深肤色:','ru':':гандболист_очень_темныи_тон_кожи:','tr':':hentbol_oynayan_erkek_koyu_cilt_tonu:','ar':':لاعب_كرة_يد_بشرة_بلون_غامق:'},'\U0001F93E\U0001F3FF\U0000200D\U00002642':{'en':':man_playing_handball_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':handballspieler_dunkle_hautfarbe:','es':':hombre_jugando_al_balonmano_tono_de_piel_oscuro:','fr':':handballeur_peau_foncée:','ja':':ハンドボールをする男_濃い肌色:','ko':':핸드볼하는_남자_검은색_피부:','pt':':jogador_de_handebol_pele_escura:','it':':pallamanista_uomo_carnagione_scura:','fa':':هندبالیست_مرد_پوست_آبنوسی:','id':':pria_bermain_bola_tangan_warna_kulit_gelap:','zh':':男生玩手球_较深肤色:','ru':':гандболист_очень_темный_тон_кожи:','tr':':hentbol_oynayan_erkek_koyu_cilt_tonu:','ar':':لاعب_كرة_يد_بشرة_بلون_غامق:'},'\U0001F93E\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_playing_handball_light_skin_tone:','status':fully_qualified,'E':4,'de':':handballspieler_helle_hautfarbe:','es':':hombre_jugando_al_balonmano_tono_de_piel_claro:','fr':':handballeur_peau_claire:','ja':':ハンドボールをする男_薄い肌色:','ko':':핸드볼하는_남자_하얀_피부:','pt':':jogador_de_handebol_pele_clara:','it':':pallamanista_uomo_carnagione_chiara:','fa':':هندبالیست_مرد_پوست_سفید:','id':':pria_bermain_bola_tangan_warna_kulit_cerah:','zh':':男生玩手球_较浅肤色:','ru':':гандболист_очень_светлыи_тон_кожи:','tr':':hentbol_oynayan_erkek_açık_cilt_tonu:','ar':':لاعب_كرة_يد_بشرة_بلون_فاتح:'},'\U0001F93E\U0001F3FB\U0000200D\U00002642':{'en':':man_playing_handball_light_skin_tone:','status':minimally_qualified,'E':4,'de':':handballspieler_helle_hautfarbe:','es':':hombre_jugando_al_balonmano_tono_de_piel_claro:','fr':':handballeur_peau_claire:','ja':':ハンドボールをする男_薄い肌色:','ko':':핸드볼하는_남자_하얀_피부:','pt':':jogador_de_handebol_pele_clara:','it':':pallamanista_uomo_carnagione_chiara:','fa':':هندبالیست_مرد_پوست_سفید:','id':':pria_bermain_bola_tangan_warna_kulit_cerah:','zh':':男生玩手球_较浅肤色:','ru':':гандболист_очень_светлый_тон_кожи:','tr':':hentbol_oynayan_erkek_açık_cilt_tonu:','ar':':لاعب_كرة_يد_بشرة_بلون_فاتح:'},'\U0001F93E\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_playing_handball_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':handballspieler_mitteldunkle_hautfarbe:','es':':hombre_jugando_al_balonmano_tono_de_piel_oscuro_medio:','fr':':handballeur_peau_mate:','ja':':ハンドボールをする男_やや濃い肌色:','ko':':핸드볼하는_남자_진한_갈색_피부:','pt':':jogador_de_handebol_pele_morena_escura:','it':':pallamanista_uomo_carnagione_abbastanza_scura:','fa':':هندبالیست_مرد_پوست_گندمی:','id':':pria_bermain_bola_tangan_warna_kulit_gelap-sedang:','zh':':男生玩手球_中等深肤色:','ru':':гандболист_темныи_тон_кожи:','tr':':hentbol_oynayan_erkek_orta_koyu_cilt_tonu:','ar':':لاعب_كرة_يد_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F93E\U0001F3FE\U0000200D\U00002642':{'en':':man_playing_handball_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':handballspieler_mitteldunkle_hautfarbe:','es':':hombre_jugando_al_balonmano_tono_de_piel_oscuro_medio:','fr':':handballeur_peau_mate:','ja':':ハンドボールをする男_やや濃い肌色:','ko':':핸드볼하는_남자_진한_갈색_피부:','pt':':jogador_de_handebol_pele_morena_escura:','it':':pallamanista_uomo_carnagione_abbastanza_scura:','fa':':هندبالیست_مرد_پوست_گندمی:','id':':pria_bermain_bola_tangan_warna_kulit_gelap-sedang:','zh':':男生玩手球_中等深肤色:','ru':':гандболист_темный_тон_кожи:','tr':':hentbol_oynayan_erkek_orta_koyu_cilt_tonu:','ar':':لاعب_كرة_يد_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F93E\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_playing_handball_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':handballspieler_mittelhelle_hautfarbe:','es':':hombre_jugando_al_balonmano_tono_de_piel_claro_medio:','fr':':handballeur_peau_moyennement_claire:','ja':':ハンドボールをする男_やや薄い肌色:','ko':':핸드볼하는_남자_연한_갈색_피부:','pt':':jogador_de_handebol_pele_morena_clara:','it':':pallamanista_uomo_carnagione_abbastanza_chiara:','fa':':هندبالیست_مرد_پوست_روشن:','id':':pria_bermain_bola_tangan_warna_kulit_cerah-sedang:','zh':':男生玩手球_中等浅肤色:','ru':':гандболист_светлыи_тон_кожи:','tr':':hentbol_oynayan_erkek_orta_açık_cilt_tonu:','ar':':لاعب_كرة_يد_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F93E\U0001F3FC\U0000200D\U00002642':{'en':':man_playing_handball_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':handballspieler_mittelhelle_hautfarbe:','es':':hombre_jugando_al_balonmano_tono_de_piel_claro_medio:','fr':':handballeur_peau_moyennement_claire:','ja':':ハンドボールをする男_やや薄い肌色:','ko':':핸드볼하는_남자_연한_갈색_피부:','pt':':jogador_de_handebol_pele_morena_clara:','it':':pallamanista_uomo_carnagione_abbastanza_chiara:','fa':':هندبالیست_مرد_پوست_روشن:','id':':pria_bermain_bola_tangan_warna_kulit_cerah-sedang:','zh':':男生玩手球_中等浅肤色:','ru':':гандболист_светлый_тон_кожи:','tr':':hentbol_oynayan_erkek_orta_açık_cilt_tonu:','ar':':لاعب_كرة_يد_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F93E\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_playing_handball_medium_skin_tone:','status':fully_qualified,'E':4,'de':':handballspieler_mittlere_hautfarbe:','es':':hombre_jugando_al_balonmano_tono_de_piel_medio:','fr':':handballeur_peau_légèrement_mate:','ja':':ハンドボールをする男_中間の肌色:','ko':':핸드볼하는_남자_갈색_피부:','pt':':jogador_de_handebol_pele_morena:','it':':pallamanista_uomo_carnagione_olivastra:','fa':':هندبالیست_مرد_پوست_طلایی:','id':':pria_bermain_bola_tangan_warna_kulit_sedang:','zh':':男生玩手球_中等肤色:','ru':':гандболист_среднии_тон_кожи:','tr':':hentbol_oynayan_erkek_orta_cilt_tonu:','ar':':لاعب_كرة_يد_بشرة_بلون_معتدل:'},'\U0001F93E\U0001F3FD\U0000200D\U00002642':{'en':':man_playing_handball_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':handballspieler_mittlere_hautfarbe:','es':':hombre_jugando_al_balonmano_tono_de_piel_medio:','fr':':handballeur_peau_légèrement_mate:','ja':':ハンドボールをする男_中間の肌色:','ko':':핸드볼하는_남자_갈색_피부:','pt':':jogador_de_handebol_pele_morena:','it':':pallamanista_uomo_carnagione_olivastra:','fa':':هندبالیست_مرد_پوست_طلایی:','id':':pria_bermain_bola_tangan_warna_kulit_sedang:','zh':':男生玩手球_中等肤色:','ru':':гандболист_средний_тон_кожи:','tr':':hentbol_oynayan_erkek_orta_cilt_tonu:','ar':':لاعب_كرة_يد_بشرة_بلون_معتدل:'},'\U0001F93D\U0000200D\U00002642\U0000FE0F':{'en':':man_playing_water_polo:','status':fully_qualified,'E':4,'de':':wasserballspieler:','es':':hombre_jugando_al_waterpolo:','fr':':joueur_de_water-polo:','ja':':水球をする男:','ko':':수구하는_남자:','pt':':homem_jogando_polo_aquático:','it':':pallanuotista_uomo:','fa':':واترپلو_مردان:','id':':pria_bermain_polo_air:','zh':':男生玩水球:','ru':':мужчина_играет_в_водное_поло:','tr':':su_topu_oynayan_erkek:','ar':':لاعب_كرة_ماء:'},'\U0001F93D\U0000200D\U00002642':{'en':':man_playing_water_polo:','status':minimally_qualified,'E':4,'de':':wasserballspieler:','es':':hombre_jugando_al_waterpolo:','fr':':joueur_de_water-polo:','ja':':水球をする男:','ko':':수구하는_남자:','pt':':homem_jogando_polo_aquático:','it':':pallanuotista_uomo:','fa':':واترپلو_مردان:','id':':pria_bermain_polo_air:','zh':':男生玩水球:','ru':':мужчина_играет_в_водное_поло:','tr':':su_topu_oynayan_erkek:','ar':':لاعب_كرة_ماء:'},'\U0001F93D\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_playing_water_polo_dark_skin_tone:','status':fully_qualified,'E':4,'de':':wasserballspieler_dunkle_hautfarbe:','es':':hombre_jugando_al_waterpolo_tono_de_piel_oscuro:','fr':':joueur_de_water-polo_peau_foncée:','ja':':水球をする男_濃い肌色:','ko':':수구하는_남자_검은색_피부:','pt':':homem_jogando_polo_aquático_pele_escura:','it':':pallanuotista_uomo_carnagione_scura:','fa':':واترپلو_مردان_پوست_آبنوسی:','id':':pria_bermain_polo_air_warna_kulit_gelap:','zh':':男生玩水球_较深肤色:','ru':':мужчина_играет_в_водное_поло_очень_темныи_тон_кожи:','tr':':su_topu_oynayan_erkek_koyu_cilt_tonu:','ar':':لاعب_كرة_ماء_بشرة_بلون_غامق:'},'\U0001F93D\U0001F3FF\U0000200D\U00002642':{'en':':man_playing_water_polo_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':wasserballspieler_dunkle_hautfarbe:','es':':hombre_jugando_al_waterpolo_tono_de_piel_oscuro:','fr':':joueur_de_water-polo_peau_foncée:','ja':':水球をする男_濃い肌色:','ko':':수구하는_남자_검은색_피부:','pt':':homem_jogando_polo_aquático_pele_escura:','it':':pallanuotista_uomo_carnagione_scura:','fa':':واترپلو_مردان_پوست_آبنوسی:','id':':pria_bermain_polo_air_warna_kulit_gelap:','zh':':男生玩水球_较深肤色:','ru':':мужчина_играет_в_водное_поло_очень_темный_тон_кожи:','tr':':su_topu_oynayan_erkek_koyu_cilt_tonu:','ar':':لاعب_كرة_ماء_بشرة_بلون_غامق:'},'\U0001F93D\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_playing_water_polo_light_skin_tone:','status':fully_qualified,'E':4,'de':':wasserballspieler_helle_hautfarbe:','es':':hombre_jugando_al_waterpolo_tono_de_piel_claro:','fr':':joueur_de_water-polo_peau_claire:','ja':':水球をする男_薄い肌色:','ko':':수구하는_남자_하얀_피부:','pt':':homem_jogando_polo_aquático_pele_clara:','it':':pallanuotista_uomo_carnagione_chiara:','fa':':واترپلو_مردان_پوست_سفید:','id':':pria_bermain_polo_air_warna_kulit_cerah:','zh':':男生玩水球_较浅肤色:','ru':':мужчина_играет_в_водное_поло_очень_светлыи_тон_кожи:','tr':':su_topu_oynayan_erkek_açık_cilt_tonu:','ar':':لاعب_كرة_ماء_بشرة_بلون_فاتح:'},'\U0001F93D\U0001F3FB\U0000200D\U00002642':{'en':':man_playing_water_polo_light_skin_tone:','status':minimally_qualified,'E':4,'de':':wasserballspieler_helle_hautfarbe:','es':':hombre_jugando_al_waterpolo_tono_de_piel_claro:','fr':':joueur_de_water-polo_peau_claire:','ja':':水球をする男_薄い肌色:','ko':':수구하는_남자_하얀_피부:','pt':':homem_jogando_polo_aquático_pele_clara:','it':':pallanuotista_uomo_carnagione_chiara:','fa':':واترپلو_مردان_پوست_سفید:','id':':pria_bermain_polo_air_warna_kulit_cerah:','zh':':男生玩水球_较浅肤色:','ru':':мужчина_играет_в_водное_поло_очень_светлый_тон_кожи:','tr':':su_topu_oynayan_erkek_açık_cilt_tonu:','ar':':لاعب_كرة_ماء_بشرة_بلون_فاتح:'},'\U0001F93D\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_playing_water_polo_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':wasserballspieler_mitteldunkle_hautfarbe:','es':':hombre_jugando_al_waterpolo_tono_de_piel_oscuro_medio:','fr':':joueur_de_water-polo_peau_mate:','ja':':水球をする男_やや濃い肌色:','ko':':수구하는_남자_진한_갈색_피부:','pt':':homem_jogando_polo_aquático_pele_morena_escura:','it':':pallanuotista_uomo_carnagione_abbastanza_scura:','fa':':واترپلو_مردان_پوست_گندمی:','id':':pria_bermain_polo_air_warna_kulit_gelap-sedang:','zh':':男生玩水球_中等深肤色:','ru':':мужчина_играет_в_водное_поло_темныи_тон_кожи:','tr':':su_topu_oynayan_erkek_orta_koyu_cilt_tonu:','ar':':لاعب_كرة_ماء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F93D\U0001F3FE\U0000200D\U00002642':{'en':':man_playing_water_polo_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':wasserballspieler_mitteldunkle_hautfarbe:','es':':hombre_jugando_al_waterpolo_tono_de_piel_oscuro_medio:','fr':':joueur_de_water-polo_peau_mate:','ja':':水球をする男_やや濃い肌色:','ko':':수구하는_남자_진한_갈색_피부:','pt':':homem_jogando_polo_aquático_pele_morena_escura:','it':':pallanuotista_uomo_carnagione_abbastanza_scura:','fa':':واترپلو_مردان_پوست_گندمی:','id':':pria_bermain_polo_air_warna_kulit_gelap-sedang:','zh':':男生玩水球_中等深肤色:','ru':':мужчина_играет_в_водное_поло_темный_тон_кожи:','tr':':su_topu_oynayan_erkek_orta_koyu_cilt_tonu:','ar':':لاعب_كرة_ماء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F93D\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_playing_water_polo_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':wasserballspieler_mittelhelle_hautfarbe:','es':':hombre_jugando_al_waterpolo_tono_de_piel_claro_medio:','fr':':joueur_de_water-polo_peau_moyennement_claire:','ja':':水球をする男_やや薄い肌色:','ko':':수구하는_남자_연한_갈색_피부:','pt':':homem_jogando_polo_aquático_pele_morena_clara:','it':':pallanuotista_uomo_carnagione_abbastanza_chiara:','fa':':واترپلو_مردان_پوست_روشن:','id':':pria_bermain_polo_air_warna_kulit_cerah-sedang:','zh':':男生玩水球_中等浅肤色:','ru':':мужчина_играет_в_водное_поло_светлыи_тон_кожи:','tr':':su_topu_oynayan_erkek_orta_açık_cilt_tonu:','ar':':لاعب_كرة_ماء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F93D\U0001F3FC\U0000200D\U00002642':{'en':':man_playing_water_polo_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':wasserballspieler_mittelhelle_hautfarbe:','es':':hombre_jugando_al_waterpolo_tono_de_piel_claro_medio:','fr':':joueur_de_water-polo_peau_moyennement_claire:','ja':':水球をする男_やや薄い肌色:','ko':':수구하는_남자_연한_갈색_피부:','pt':':homem_jogando_polo_aquático_pele_morena_clara:','it':':pallanuotista_uomo_carnagione_abbastanza_chiara:','fa':':واترپلو_مردان_پوست_روشن:','id':':pria_bermain_polo_air_warna_kulit_cerah-sedang:','zh':':男生玩水球_中等浅肤色:','ru':':мужчина_играет_в_водное_поло_светлый_тон_кожи:','tr':':su_topu_oynayan_erkek_orta_açık_cilt_tonu:','ar':':لاعب_كرة_ماء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F93D\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_playing_water_polo_medium_skin_tone:','status':fully_qualified,'E':4,'de':':wasserballspieler_mittlere_hautfarbe:','es':':hombre_jugando_al_waterpolo_tono_de_piel_medio:','fr':':joueur_de_water-polo_peau_légèrement_mate:','ja':':水球をする男_中間の肌色:','ko':':수구하는_남자_갈색_피부:','pt':':homem_jogando_polo_aquático_pele_morena:','it':':pallanuotista_uomo_carnagione_olivastra:','fa':':واترپلو_مردان_پوست_طلایی:','id':':pria_bermain_polo_air_warna_kulit_sedang:','zh':':男生玩水球_中等肤色:','ru':':мужчина_играет_в_водное_поло_среднии_тон_кожи:','tr':':su_topu_oynayan_erkek_orta_cilt_tonu:','ar':':لاعب_كرة_ماء_بشرة_بلون_معتدل:'},'\U0001F93D\U0001F3FD\U0000200D\U00002642':{'en':':man_playing_water_polo_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':wasserballspieler_mittlere_hautfarbe:','es':':hombre_jugando_al_waterpolo_tono_de_piel_medio:','fr':':joueur_de_water-polo_peau_légèrement_mate:','ja':':水球をする男_中間の肌色:','ko':':수구하는_남자_갈색_피부:','pt':':homem_jogando_polo_aquático_pele_morena:','it':':pallanuotista_uomo_carnagione_olivastra:','fa':':واترپلو_مردان_پوست_طلایی:','id':':pria_bermain_polo_air_warna_kulit_sedang:','zh':':男生玩水球_中等肤色:','ru':':мужчина_играет_в_водное_поло_средний_тон_кожи:','tr':':su_topu_oynayan_erkek_orta_cilt_tonu:','ar':':لاعب_كرة_ماء_بشرة_بلون_معتدل:'},'\U0001F46E\U0000200D\U00002642\U0000FE0F':{'en':':man_police_officer:','status':fully_qualified,'E':4,'alias':[':policeman:'],'de':':polizist:','es':':agente_de_policía_hombre:','fr':':policier:','ja':':男性警察官:','ko':':남자_경찰관:','pt':':policial_homem:','it':':poliziotto_uomo:','fa':':پلیس_مرد:','id':':polisi_pria:','zh':':男警察:','ru':':мужчина-полицеискии:','tr':':erkek_polis_memuru:','ar':':شرطي:'},'\U0001F46E\U0000200D\U00002642':{'en':':man_police_officer:','status':minimally_qualified,'E':4,'alias':[':policeman:'],'de':':polizist:','es':':agente_de_policía_hombre:','fr':':policier:','ja':':男性警察官:','ko':':남자_경찰관:','pt':':policial_homem:','it':':poliziotto_uomo:','fa':':پلیس_مرد:','id':':polisi_pria:','zh':':男警察:','ru':':мужчина_полицейский:','tr':':erkek_polis_memuru:','ar':':شرطي:'},'\U0001F46E\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_police_officer_dark_skin_tone:','status':fully_qualified,'E':4,'de':':polizist_dunkle_hautfarbe:','es':':agente_de_policía_hombre_tono_de_piel_oscuro:','fr':':policier_peau_foncée:','ja':':男性警察官_濃い肌色:','ko':':남자_경찰관_검은색_피부:','pt':':policial_homem_pele_escura:','it':':poliziotto_uomo_carnagione_scura:','fa':':پلیس_مرد_پوست_آبنوسی:','id':':polisi_pria_warna_kulit_gelap:','zh':':男警察_较深肤色:','ru':':мужчина-полицеискии_очень_темныи_тон_кожи:','tr':':erkek_polis_memuru_koyu_cilt_tonu:','ar':':شرطي_بشرة_بلون_غامق:'},'\U0001F46E\U0001F3FF\U0000200D\U00002642':{'en':':man_police_officer_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':polizist_dunkle_hautfarbe:','es':':agente_de_policía_hombre_tono_de_piel_oscuro:','fr':':policier_peau_foncée:','ja':':男性警察官_濃い肌色:','ko':':남자_경찰관_검은색_피부:','pt':':policial_homem_pele_escura:','it':':poliziotto_uomo_carnagione_scura:','fa':':پلیس_مرد_پوست_آبنوسی:','id':':polisi_pria_warna_kulit_gelap:','zh':':男警察_较深肤色:','ru':':мужчина_полицейский_очень_темный_тон_кожи:','tr':':erkek_polis_memuru_koyu_cilt_tonu:','ar':':شرطي_بشرة_بلون_غامق:'},'\U0001F46E\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_police_officer_light_skin_tone:','status':fully_qualified,'E':4,'de':':polizist_helle_hautfarbe:','es':':agente_de_policía_hombre_tono_de_piel_claro:','fr':':policier_peau_claire:','ja':':男性警察官_薄い肌色:','ko':':남자_경찰관_하얀_피부:','pt':':policial_homem_pele_clara:','it':':poliziotto_uomo_carnagione_chiara:','fa':':پلیس_مرد_پوست_سفید:','id':':polisi_pria_warna_kulit_cerah:','zh':':男警察_较浅肤色:','ru':':мужчина-полицеискии_очень_светлыи_тон_кожи:','tr':':erkek_polis_memuru_açık_cilt_tonu:','ar':':شرطي_بشرة_بلون_فاتح:'},'\U0001F46E\U0001F3FB\U0000200D\U00002642':{'en':':man_police_officer_light_skin_tone:','status':minimally_qualified,'E':4,'de':':polizist_helle_hautfarbe:','es':':agente_de_policía_hombre_tono_de_piel_claro:','fr':':policier_peau_claire:','ja':':男性警察官_薄い肌色:','ko':':남자_경찰관_하얀_피부:','pt':':policial_homem_pele_clara:','it':':poliziotto_uomo_carnagione_chiara:','fa':':پلیس_مرد_پوست_سفید:','id':':polisi_pria_warna_kulit_cerah:','zh':':男警察_较浅肤色:','ru':':мужчина_полицейский_очень_светлый_тон_кожи:','tr':':erkek_polis_memuru_açık_cilt_tonu:','ar':':شرطي_بشرة_بلون_فاتح:'},'\U0001F46E\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_police_officer_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':polizist_mitteldunkle_hautfarbe:','es':':agente_de_policía_hombre_tono_de_piel_oscuro_medio:','fr':':policier_peau_mate:','ja':':男性警察官_やや濃い肌色:','ko':':남자_경찰관_진한_갈색_피부:','pt':':policial_homem_pele_morena_escura:','it':':poliziotto_uomo_carnagione_abbastanza_scura:','fa':':پلیس_مرد_پوست_گندمی:','id':':polisi_pria_warna_kulit_gelap-sedang:','zh':':男警察_中等深肤色:','ru':':мужчина-полицеискии_темныи_тон_кожи:','tr':':erkek_polis_memuru_orta_koyu_cilt_tonu:','ar':':شرطي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F46E\U0001F3FE\U0000200D\U00002642':{'en':':man_police_officer_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':polizist_mitteldunkle_hautfarbe:','es':':agente_de_policía_hombre_tono_de_piel_oscuro_medio:','fr':':policier_peau_mate:','ja':':男性警察官_やや濃い肌色:','ko':':남자_경찰관_진한_갈색_피부:','pt':':policial_homem_pele_morena_escura:','it':':poliziotto_uomo_carnagione_abbastanza_scura:','fa':':پلیس_مرد_پوست_گندمی:','id':':polisi_pria_warna_kulit_gelap-sedang:','zh':':男警察_中等深肤色:','ru':':мужчина_полицейский_темный_тон_кожи:','tr':':erkek_polis_memuru_orta_koyu_cilt_tonu:','ar':':شرطي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F46E\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_police_officer_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':polizist_mittelhelle_hautfarbe:','es':':agente_de_policía_hombre_tono_de_piel_claro_medio:','fr':':policier_peau_moyennement_claire:','ja':':男性警察官_やや薄い肌色:','ko':':남자_경찰관_연한_갈색_피부:','pt':':policial_homem_pele_morena_clara:','it':':poliziotto_uomo_carnagione_abbastanza_chiara:','fa':':پلیس_مرد_پوست_روشن:','id':':polisi_pria_warna_kulit_cerah-sedang:','zh':':男警察_中等浅肤色:','ru':':мужчина-полицеискии_светлыи_тон_кожи:','tr':':erkek_polis_memuru_orta_açık_cilt_tonu:','ar':':شرطي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F46E\U0001F3FC\U0000200D\U00002642':{'en':':man_police_officer_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':polizist_mittelhelle_hautfarbe:','es':':agente_de_policía_hombre_tono_de_piel_claro_medio:','fr':':policier_peau_moyennement_claire:','ja':':男性警察官_やや薄い肌色:','ko':':남자_경찰관_연한_갈색_피부:','pt':':policial_homem_pele_morena_clara:','it':':poliziotto_uomo_carnagione_abbastanza_chiara:','fa':':پلیس_مرد_پوست_روشن:','id':':polisi_pria_warna_kulit_cerah-sedang:','zh':':男警察_中等浅肤色:','ru':':мужчина_полицейский_светлый_тон_кожи:','tr':':erkek_polis_memuru_orta_açık_cilt_tonu:','ar':':شرطي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F46E\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_police_officer_medium_skin_tone:','status':fully_qualified,'E':4,'de':':polizist_mittlere_hautfarbe:','es':':agente_de_policía_hombre_tono_de_piel_medio:','fr':':policier_peau_légèrement_mate:','ja':':男性警察官_中間の肌色:','ko':':남자_경찰관_갈색_피부:','pt':':policial_homem_pele_morena:','it':':poliziotto_uomo_carnagione_olivastra:','fa':':پلیس_مرد_پوست_طلایی:','id':':polisi_pria_warna_kulit_sedang:','zh':':男警察_中等肤色:','ru':':мужчина-полицеискии_среднии_тон_кожи:','tr':':erkek_polis_memuru_orta_cilt_tonu:','ar':':شرطي_بشرة_بلون_معتدل:'},'\U0001F46E\U0001F3FD\U0000200D\U00002642':{'en':':man_police_officer_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':polizist_mittlere_hautfarbe:','es':':agente_de_policía_hombre_tono_de_piel_medio:','fr':':policier_peau_légèrement_mate:','ja':':男性警察官_中間の肌色:','ko':':남자_경찰관_갈색_피부:','pt':':policial_homem_pele_morena:','it':':poliziotto_uomo_carnagione_olivastra:','fa':':پلیس_مرد_پوست_طلایی:','id':':polisi_pria_warna_kulit_sedang:','zh':':男警察_中等肤色:','ru':':мужчина_полицейский_средний_тон_кожи:','tr':':erkek_polis_memuru_orta_cilt_tonu:','ar':':شرطي_بشرة_بلون_معتدل:'},'\U0001F64E\U0000200D\U00002642\U0000FE0F':{'en':':man_pouting:','status':fully_qualified,'E':4,'alias':[':pouting_man:'],'de':':schmollender_mann:','es':':hombre_haciendo_pucheros:','fr':':homme_qui_boude:','ja':':不機嫌な男:','ko':':뾰로통한_남자:','pt':':homem_fazendo_bico:','it':':uomo_imbronciato:','fa':':مرد_اخم_وتخم_می_کند:','id':':pria_tidak_senang:','zh':':撅嘴男:','ru':':обиженныи_мужчина:','tr':':darılan_erkek:','ar':':رجل_عابس:'},'\U0001F64E\U0000200D\U00002642':{'en':':man_pouting:','status':minimally_qualified,'E':4,'alias':[':pouting_man:'],'de':':schmollender_mann:','es':':hombre_haciendo_pucheros:','fr':':homme_qui_boude:','ja':':不機嫌な男:','ko':':뾰로통한_남자:','pt':':homem_fazendo_bico:','it':':uomo_imbronciato:','fa':':مرد_اخم_وتخم_می_کند:','id':':pria_tidak_senang:','zh':':撅嘴男:','ru':':обиженный_мужчина:','tr':':darılan_erkek:','ar':':رجل_عابس:'},'\U0001F64E\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_pouting_dark_skin_tone:','status':fully_qualified,'E':4,'de':':schmollender_mann_dunkle_hautfarbe:','es':':hombre_haciendo_pucheros_tono_de_piel_oscuro:','fr':':homme_qui_boude_peau_foncée:','ja':':不機嫌な男_濃い肌色:','ko':':뾰로통한_남자_검은색_피부:','pt':':homem_fazendo_bico_pele_escura:','it':':uomo_imbronciato_carnagione_scura:','fa':':مرد_اخم_وتخم_می_کند_پوست_آبنوسی:','id':':pria_tidak_senang_warna_kulit_gelap:','zh':':撅嘴男_较深肤色:','ru':':обиженныи_мужчина_очень_темныи_тон_кожи:','tr':':darılan_erkek_koyu_cilt_tonu:','ar':':رجل_عابس_بشرة_بلون_غامق:'},'\U0001F64E\U0001F3FF\U0000200D\U00002642':{'en':':man_pouting_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':schmollender_mann_dunkle_hautfarbe:','es':':hombre_haciendo_pucheros_tono_de_piel_oscuro:','fr':':homme_qui_boude_peau_foncée:','ja':':不機嫌な男_濃い肌色:','ko':':뾰로통한_남자_검은색_피부:','pt':':homem_fazendo_bico_pele_escura:','it':':uomo_imbronciato_carnagione_scura:','fa':':مرد_اخم_وتخم_می_کند_پوست_آبنوسی:','id':':pria_tidak_senang_warna_kulit_gelap:','zh':':撅嘴男_较深肤色:','ru':':обиженный_мужчина_очень_темный_тон_кожи:','tr':':darılan_erkek_koyu_cilt_tonu:','ar':':رجل_عابس_بشرة_بلون_غامق:'},'\U0001F64E\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_pouting_light_skin_tone:','status':fully_qualified,'E':4,'de':':schmollender_mann_helle_hautfarbe:','es':':hombre_haciendo_pucheros_tono_de_piel_claro:','fr':':homme_qui_boude_peau_claire:','ja':':不機嫌な男_薄い肌色:','ko':':뾰로통한_남자_하얀_피부:','pt':':homem_fazendo_bico_pele_clara:','it':':uomo_imbronciato_carnagione_chiara:','fa':':مرد_اخم_وتخم_می_کند_پوست_سفید:','id':':pria_tidak_senang_warna_kulit_cerah:','zh':':撅嘴男_较浅肤色:','ru':':обиженныи_мужчина_очень_светлыи_тон_кожи:','tr':':darılan_erkek_açık_cilt_tonu:','ar':':رجل_عابس_بشرة_بلون_فاتح:'},'\U0001F64E\U0001F3FB\U0000200D\U00002642':{'en':':man_pouting_light_skin_tone:','status':minimally_qualified,'E':4,'de':':schmollender_mann_helle_hautfarbe:','es':':hombre_haciendo_pucheros_tono_de_piel_claro:','fr':':homme_qui_boude_peau_claire:','ja':':不機嫌な男_薄い肌色:','ko':':뾰로통한_남자_하얀_피부:','pt':':homem_fazendo_bico_pele_clara:','it':':uomo_imbronciato_carnagione_chiara:','fa':':مرد_اخم_وتخم_می_کند_پوست_سفید:','id':':pria_tidak_senang_warna_kulit_cerah:','zh':':撅嘴男_较浅肤色:','ru':':обиженный_мужчина_очень_светлый_тон_кожи:','tr':':darılan_erkek_açık_cilt_tonu:','ar':':رجل_عابس_بشرة_بلون_فاتح:'},'\U0001F64E\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_pouting_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':schmollender_mann_mitteldunkle_hautfarbe:','es':':hombre_haciendo_pucheros_tono_de_piel_oscuro_medio:','fr':':homme_qui_boude_peau_mate:','ja':':不機嫌な男_やや濃い肌色:','ko':':뾰로통한_남자_진한_갈색_피부:','pt':':homem_fazendo_bico_pele_morena_escura:','it':':uomo_imbronciato_carnagione_abbastanza_scura:','fa':':مرد_اخم_وتخم_می_کند_پوست_گندمی:','id':':pria_tidak_senang_warna_kulit_gelap-sedang:','zh':':撅嘴男_中等深肤色:','ru':':обиженныи_мужчина_темныи_тон_кожи:','tr':':darılan_erkek_orta_koyu_cilt_tonu:','ar':':رجل_عابس_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F64E\U0001F3FE\U0000200D\U00002642':{'en':':man_pouting_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':schmollender_mann_mitteldunkle_hautfarbe:','es':':hombre_haciendo_pucheros_tono_de_piel_oscuro_medio:','fr':':homme_qui_boude_peau_mate:','ja':':不機嫌な男_やや濃い肌色:','ko':':뾰로통한_남자_진한_갈색_피부:','pt':':homem_fazendo_bico_pele_morena_escura:','it':':uomo_imbronciato_carnagione_abbastanza_scura:','fa':':مرد_اخم_وتخم_می_کند_پوست_گندمی:','id':':pria_tidak_senang_warna_kulit_gelap-sedang:','zh':':撅嘴男_中等深肤色:','ru':':обиженный_мужчина_темный_тон_кожи:','tr':':darılan_erkek_orta_koyu_cilt_tonu:','ar':':رجل_عابس_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F64E\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_pouting_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':schmollender_mann_mittelhelle_hautfarbe:','es':':hombre_haciendo_pucheros_tono_de_piel_claro_medio:','fr':':homme_qui_boude_peau_moyennement_claire:','ja':':不機嫌な男_やや薄い肌色:','ko':':뾰로통한_남자_연한_갈색_피부:','pt':':homem_fazendo_bico_pele_morena_clara:','it':':uomo_imbronciato_carnagione_abbastanza_chiara:','fa':':مرد_اخم_وتخم_می_کند_پوست_روشن:','id':':pria_tidak_senang_warna_kulit_cerah-sedang:','zh':':撅嘴男_中等浅肤色:','ru':':обиженныи_мужчина_светлыи_тон_кожи:','tr':':darılan_erkek_orta_açık_cilt_tonu:','ar':':رجل_عابس_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F64E\U0001F3FC\U0000200D\U00002642':{'en':':man_pouting_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':schmollender_mann_mittelhelle_hautfarbe:','es':':hombre_haciendo_pucheros_tono_de_piel_claro_medio:','fr':':homme_qui_boude_peau_moyennement_claire:','ja':':不機嫌な男_やや薄い肌色:','ko':':뾰로통한_남자_연한_갈색_피부:','pt':':homem_fazendo_bico_pele_morena_clara:','it':':uomo_imbronciato_carnagione_abbastanza_chiara:','fa':':مرد_اخم_وتخم_می_کند_پوست_روشن:','id':':pria_tidak_senang_warna_kulit_cerah-sedang:','zh':':撅嘴男_中等浅肤色:','ru':':обиженный_мужчина_светлый_тон_кожи:','tr':':darılan_erkek_orta_açık_cilt_tonu:','ar':':رجل_عابس_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F64E\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_pouting_medium_skin_tone:','status':fully_qualified,'E':4,'de':':schmollender_mann_mittlere_hautfarbe:','es':':hombre_haciendo_pucheros_tono_de_piel_medio:','fr':':homme_qui_boude_peau_légèrement_mate:','ja':':不機嫌な男_中間の肌色:','ko':':뾰로통한_남자_갈색_피부:','pt':':homem_fazendo_bico_pele_morena:','it':':uomo_imbronciato_carnagione_olivastra:','fa':':مرد_اخم_وتخم_می_کند_پوست_طلایی:','id':':pria_tidak_senang_warna_kulit_sedang:','zh':':撅嘴男_中等肤色:','ru':':обиженныи_мужчина_среднии_тон_кожи:','tr':':darılan_erkek_orta_cilt_tonu:','ar':':رجل_عابس_بشرة_بلون_معتدل:'},'\U0001F64E\U0001F3FD\U0000200D\U00002642':{'en':':man_pouting_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':schmollender_mann_mittlere_hautfarbe:','es':':hombre_haciendo_pucheros_tono_de_piel_medio:','fr':':homme_qui_boude_peau_légèrement_mate:','ja':':不機嫌な男_中間の肌色:','ko':':뾰로통한_남자_갈색_피부:','pt':':homem_fazendo_bico_pele_morena:','it':':uomo_imbronciato_carnagione_olivastra:','fa':':مرد_اخم_وتخم_می_کند_پوست_طلایی:','id':':pria_tidak_senang_warna_kulit_sedang:','zh':':撅嘴男_中等肤色:','ru':':обиженный_мужчина_средний_тон_кожи:','tr':':darılan_erkek_orta_cilt_tonu:','ar':':رجل_عابس_بشرة_بلون_معتدل:'},'\U0001F64B\U0000200D\U00002642\U0000FE0F':{'en':':man_raising_hand:','status':fully_qualified,'E':4,'alias':[':raising_hand_man:'],'de':':mann_mit_erhobenem_arm:','es':':hombre_con_la_mano_levantada:','fr':':homme_qui_lève_la_main:','ja':':手を挙げる男:','ko':':한_손_든_남자:','pt':':homem_levantando_a_mão:','it':':uomo_con_mano_alzata:','fa':':مرد_دستش_را_بالا_می_برد:','id':':pria_mengangkat_tangan:','zh':':男生举手:','ru':':мужчина_с_поднятои_рукои:','tr':':elini_kaldıran_erkek:','ar':':رجل_يرفع_يده:'},'\U0001F64B\U0000200D\U00002642':{'en':':man_raising_hand:','status':minimally_qualified,'E':4,'alias':[':raising_hand_man:'],'de':':mann_mit_erhobenem_arm:','es':':hombre_con_la_mano_levantada:','fr':':homme_qui_lève_la_main:','ja':':手を挙げる男:','ko':':한_손_든_남자:','pt':':homem_levantando_a_mão:','it':':uomo_con_mano_alzata:','fa':':مرد_دستش_را_بالا_می_برد:','id':':pria_mengangkat_tangan:','zh':':男生举手:','ru':':мужчина_с_поднятой_рукой:','tr':':elini_kaldıran_erkek:','ar':':رجل_يرفع_يده:'},'\U0001F64B\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_raising_hand_dark_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_erhobenem_arm_dunkle_hautfarbe:','es':':hombre_con_la_mano_levantada_tono_de_piel_oscuro:','fr':':homme_qui_lève_la_main_peau_foncée:','ja':':手を挙げる男_濃い肌色:','ko':':한_손_든_남자_검은색_피부:','pt':':homem_levantando_a_mão_pele_escura:','it':':uomo_con_mano_alzata_carnagione_scura:','fa':':مرد_دستش_را_بالا_می_برد_پوست_آبنوسی:','id':':pria_mengangkat_tangan_warna_kulit_gelap:','zh':':男生举手_较深肤色:','ru':':мужчина_с_поднятои_рукои_очень_темныи_тон_кожи:','tr':':elini_kaldıran_erkek_koyu_cilt_tonu:','ar':':رجل_يرفع_يده_بشرة_بلون_غامق:'},'\U0001F64B\U0001F3FF\U0000200D\U00002642':{'en':':man_raising_hand_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_erhobenem_arm_dunkle_hautfarbe:','es':':hombre_con_la_mano_levantada_tono_de_piel_oscuro:','fr':':homme_qui_lève_la_main_peau_foncée:','ja':':手を挙げる男_濃い肌色:','ko':':한_손_든_남자_검은색_피부:','pt':':homem_levantando_a_mão_pele_escura:','it':':uomo_con_mano_alzata_carnagione_scura:','fa':':مرد_دستش_را_بالا_می_برد_پوست_آبنوسی:','id':':pria_mengangkat_tangan_warna_kulit_gelap:','zh':':男生举手_较深肤色:','ru':':мужчина_с_поднятой_рукой_очень_темный_тон_кожи:','tr':':elini_kaldıran_erkek_koyu_cilt_tonu:','ar':':رجل_يرفع_يده_بشرة_بلون_غامق:'},'\U0001F64B\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_raising_hand_light_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_erhobenem_arm_helle_hautfarbe:','es':':hombre_con_la_mano_levantada_tono_de_piel_claro:','fr':':homme_qui_lève_la_main_peau_claire:','ja':':手を挙げる男_薄い肌色:','ko':':한_손_든_남자_하얀_피부:','pt':':homem_levantando_a_mão_pele_clara:','it':':uomo_con_mano_alzata_carnagione_chiara:','fa':':مرد_دستش_را_بالا_می_برد_پوست_سفید:','id':':pria_mengangkat_tangan_warna_kulit_cerah:','zh':':男生举手_较浅肤色:','ru':':мужчина_с_поднятои_рукои_очень_светлыи_тон_кожи:','tr':':elini_kaldıran_erkek_açık_cilt_tonu:','ar':':رجل_يرفع_يده_بشرة_بلون_فاتح:'},'\U0001F64B\U0001F3FB\U0000200D\U00002642':{'en':':man_raising_hand_light_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_erhobenem_arm_helle_hautfarbe:','es':':hombre_con_la_mano_levantada_tono_de_piel_claro:','fr':':homme_qui_lève_la_main_peau_claire:','ja':':手を挙げる男_薄い肌色:','ko':':한_손_든_남자_하얀_피부:','pt':':homem_levantando_a_mão_pele_clara:','it':':uomo_con_mano_alzata_carnagione_chiara:','fa':':مرد_دستش_را_بالا_می_برد_پوست_سفید:','id':':pria_mengangkat_tangan_warna_kulit_cerah:','zh':':男生举手_较浅肤色:','ru':':мужчина_с_поднятой_рукой_очень_светлый_тон_кожи:','tr':':elini_kaldıran_erkek_açık_cilt_tonu:','ar':':رجل_يرفع_يده_بشرة_بلون_فاتح:'},'\U0001F64B\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_raising_hand_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_erhobenem_arm_mitteldunkle_hautfarbe:','es':':hombre_con_la_mano_levantada_tono_de_piel_oscuro_medio:','fr':':homme_qui_lève_la_main_peau_mate:','ja':':手を挙げる男_やや濃い肌色:','ko':':한_손_든_남자_진한_갈색_피부:','pt':':homem_levantando_a_mão_pele_morena_escura:','it':':uomo_con_mano_alzata_carnagione_abbastanza_scura:','fa':':مرد_دستش_را_بالا_می_برد_پوست_گندمی:','id':':pria_mengangkat_tangan_warna_kulit_gelap-sedang:','zh':':男生举手_中等深肤色:','ru':':мужчина_с_поднятои_рукои_темныи_тон_кожи:','tr':':elini_kaldıran_erkek_orta_koyu_cilt_tonu:','ar':':رجل_يرفع_يده_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F64B\U0001F3FE\U0000200D\U00002642':{'en':':man_raising_hand_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_erhobenem_arm_mitteldunkle_hautfarbe:','es':':hombre_con_la_mano_levantada_tono_de_piel_oscuro_medio:','fr':':homme_qui_lève_la_main_peau_mate:','ja':':手を挙げる男_やや濃い肌色:','ko':':한_손_든_남자_진한_갈색_피부:','pt':':homem_levantando_a_mão_pele_morena_escura:','it':':uomo_con_mano_alzata_carnagione_abbastanza_scura:','fa':':مرد_دستش_را_بالا_می_برد_پوست_گندمی:','id':':pria_mengangkat_tangan_warna_kulit_gelap-sedang:','zh':':男生举手_中等深肤色:','ru':':мужчина_с_поднятой_рукой_темный_тон_кожи:','tr':':elini_kaldıran_erkek_orta_koyu_cilt_tonu:','ar':':رجل_يرفع_يده_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F64B\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_raising_hand_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_erhobenem_arm_mittelhelle_hautfarbe:','es':':hombre_con_la_mano_levantada_tono_de_piel_claro_medio:','fr':':homme_qui_lève_la_main_peau_moyennement_claire:','ja':':手を挙げる男_やや薄い肌色:','ko':':한_손_든_남자_연한_갈색_피부:','pt':':homem_levantando_a_mão_pele_morena_clara:','it':':uomo_con_mano_alzata_carnagione_abbastanza_chiara:','fa':':مرد_دستش_را_بالا_می_برد_پوست_روشن:','id':':pria_mengangkat_tangan_warna_kulit_cerah-sedang:','zh':':男生举手_中等浅肤色:','ru':':мужчина_с_поднятои_рукои_светлыи_тон_кожи:','tr':':elini_kaldıran_erkek_orta_açık_cilt_tonu:','ar':':رجل_يرفع_يده_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F64B\U0001F3FC\U0000200D\U00002642':{'en':':man_raising_hand_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_erhobenem_arm_mittelhelle_hautfarbe:','es':':hombre_con_la_mano_levantada_tono_de_piel_claro_medio:','fr':':homme_qui_lève_la_main_peau_moyennement_claire:','ja':':手を挙げる男_やや薄い肌色:','ko':':한_손_든_남자_연한_갈색_피부:','pt':':homem_levantando_a_mão_pele_morena_clara:','it':':uomo_con_mano_alzata_carnagione_abbastanza_chiara:','fa':':مرد_دستش_را_بالا_می_برد_پوست_روشن:','id':':pria_mengangkat_tangan_warna_kulit_cerah-sedang:','zh':':男生举手_中等浅肤色:','ru':':мужчина_с_поднятой_рукой_светлый_тон_кожи:','tr':':elini_kaldıran_erkek_orta_açık_cilt_tonu:','ar':':رجل_يرفع_يده_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F64B\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_raising_hand_medium_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_erhobenem_arm_mittlere_hautfarbe:','es':':hombre_con_la_mano_levantada_tono_de_piel_medio:','fr':':homme_qui_lève_la_main_peau_légèrement_mate:','ja':':手を挙げる男_中間の肌色:','ko':':한_손_든_남자_갈색_피부:','pt':':homem_levantando_a_mão_pele_morena:','it':':uomo_con_mano_alzata_carnagione_olivastra:','fa':':مرد_دستش_را_بالا_می_برد_پوست_طلایی:','id':':pria_mengangkat_tangan_warna_kulit_sedang:','zh':':男生举手_中等肤色:','ru':':мужчина_с_поднятои_рукои_среднии_тон_кожи:','tr':':elini_kaldıran_erkek_orta_cilt_tonu:','ar':':رجل_يرفع_يده_بشرة_بلون_معتدل:'},'\U0001F64B\U0001F3FD\U0000200D\U00002642':{'en':':man_raising_hand_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_erhobenem_arm_mittlere_hautfarbe:','es':':hombre_con_la_mano_levantada_tono_de_piel_medio:','fr':':homme_qui_lève_la_main_peau_légèrement_mate:','ja':':手を挙げる男_中間の肌色:','ko':':한_손_든_남자_갈색_피부:','pt':':homem_levantando_a_mão_pele_morena:','it':':uomo_con_mano_alzata_carnagione_olivastra:','fa':':مرد_دستش_را_بالا_می_برد_پوست_طلایی:','id':':pria_mengangkat_tangan_warna_kulit_sedang:','zh':':男生举手_中等肤色:','ru':':мужчина_с_поднятой_рукой_средний_тон_кожи:','tr':':elini_kaldıran_erkek_orta_cilt_tonu:','ar':':رجل_يرفع_يده_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U0001F9B0':{'en':':man_red_hair:','status':fully_qualified,'E':11,'alias':[':red_haired_man:'],'de':':mann_rotes_haar:','es':':hombre_pelo_pelirrojo:','fr':':homme_cheveux_roux:','ja':':男性_赤毛:','ko':':남자_빨간_머리:','pt':':homem_cabelo_vermelho:','it':':uomo_capelli_rossi:','fa':':مرد_موقرمز:','id':':pria_rambut_merah:','zh':':男人_红发:','ru':':мужчина_рыжие_волосы:','tr':':erkek_kızıl_saç:','ar':':رجل_شعر_أحمر:'},'\U0001F6A3\U0000200D\U00002642\U0000FE0F':{'en':':man_rowing_boat:','status':fully_qualified,'E':4,'alias':[':rowing_man:'],'de':':mann_im_ruderboot:','es':':hombre_remando_en_un_bote:','fr':':rameur_dans_une_barque:','ja':':ボートをこぐ男:','ko':':노젓는_남자:','pt':':homem_remando:','it':':uomo_in_barca_a_remi:','fa':':پاروزن_مرد:','id':':pria_mendayung_perahu:','zh':':男生划船:','ru':':гребец:','tr':':sandalda_kürek_çeken_erkek:','ar':':رجل_يجدف_بقارب:'},'\U0001F6A3\U0000200D\U00002642':{'en':':man_rowing_boat:','status':minimally_qualified,'E':4,'alias':[':rowing_man:'],'de':':mann_im_ruderboot:','es':':hombre_remando_en_un_bote:','fr':':rameur_dans_une_barque:','ja':':ボートをこぐ男:','ko':':노젓는_남자:','pt':':homem_remando:','it':':uomo_in_barca_a_remi:','fa':':پاروزن_مرد:','id':':pria_mendayung_perahu:','zh':':男生划船:','ru':':гребец:','tr':':sandalda_kürek_çeken_erkek:','ar':':رجل_يجدف_بقارب:'},'\U0001F6A3\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_rowing_boat_dark_skin_tone:','status':fully_qualified,'E':4,'de':':mann_im_ruderboot_dunkle_hautfarbe:','es':':hombre_remando_en_un_bote_tono_de_piel_oscuro:','fr':':rameur_dans_une_barque_peau_foncée:','ja':':ボートをこぐ男_濃い肌色:','ko':':노젓는_남자_검은색_피부:','pt':':homem_remando_pele_escura:','it':':uomo_in_barca_a_remi_carnagione_scura:','fa':':پاروزن_مرد_پوست_آبنوسی:','id':':pria_mendayung_perahu_warna_kulit_gelap:','zh':':男生划船_较深肤色:','ru':':гребец_очень_темныи_тон_кожи:','tr':':sandalda_kürek_çeken_erkek_koyu_cilt_tonu:','ar':':رجل_يجدف_بقارب_بشرة_بلون_غامق:'},'\U0001F6A3\U0001F3FF\U0000200D\U00002642':{'en':':man_rowing_boat_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_im_ruderboot_dunkle_hautfarbe:','es':':hombre_remando_en_un_bote_tono_de_piel_oscuro:','fr':':rameur_dans_une_barque_peau_foncée:','ja':':ボートをこぐ男_濃い肌色:','ko':':노젓는_남자_검은색_피부:','pt':':homem_remando_pele_escura:','it':':uomo_in_barca_a_remi_carnagione_scura:','fa':':پاروزن_مرد_پوست_آبنوسی:','id':':pria_mendayung_perahu_warna_kulit_gelap:','zh':':男生划船_较深肤色:','ru':':гребец_очень_темный_тон_кожи:','tr':':sandalda_kürek_çeken_erkek_koyu_cilt_tonu:','ar':':رجل_يجدف_بقارب_بشرة_بلون_غامق:'},'\U0001F6A3\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_rowing_boat_light_skin_tone:','status':fully_qualified,'E':4,'de':':mann_im_ruderboot_helle_hautfarbe:','es':':hombre_remando_en_un_bote_tono_de_piel_claro:','fr':':rameur_dans_une_barque_peau_claire:','ja':':ボートをこぐ男_薄い肌色:','ko':':노젓는_남자_하얀_피부:','pt':':homem_remando_pele_clara:','it':':uomo_in_barca_a_remi_carnagione_chiara:','fa':':پاروزن_مرد_پوست_سفید:','id':':pria_mendayung_perahu_warna_kulit_cerah:','zh':':男生划船_较浅肤色:','ru':':гребец_очень_светлыи_тон_кожи:','tr':':sandalda_kürek_çeken_erkek_açık_cilt_tonu:','ar':':رجل_يجدف_بقارب_بشرة_بلون_فاتح:'},'\U0001F6A3\U0001F3FB\U0000200D\U00002642':{'en':':man_rowing_boat_light_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_im_ruderboot_helle_hautfarbe:','es':':hombre_remando_en_un_bote_tono_de_piel_claro:','fr':':rameur_dans_une_barque_peau_claire:','ja':':ボートをこぐ男_薄い肌色:','ko':':노젓는_남자_하얀_피부:','pt':':homem_remando_pele_clara:','it':':uomo_in_barca_a_remi_carnagione_chiara:','fa':':پاروزن_مرد_پوست_سفید:','id':':pria_mendayung_perahu_warna_kulit_cerah:','zh':':男生划船_较浅肤色:','ru':':гребец_очень_светлый_тон_кожи:','tr':':sandalda_kürek_çeken_erkek_açık_cilt_tonu:','ar':':رجل_يجدف_بقارب_بشرة_بلون_فاتح:'},'\U0001F6A3\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_rowing_boat_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':mann_im_ruderboot_mitteldunkle_hautfarbe:','es':':hombre_remando_en_un_bote_tono_de_piel_oscuro_medio:','fr':':rameur_dans_une_barque_peau_mate:','ja':':ボートをこぐ男_やや濃い肌色:','ko':':노젓는_남자_진한_갈색_피부:','pt':':homem_remando_pele_morena_escura:','it':':uomo_in_barca_a_remi_carnagione_abbastanza_scura:','fa':':پاروزن_مرد_پوست_گندمی:','id':':pria_mendayung_perahu_warna_kulit_gelap-sedang:','zh':':男生划船_中等深肤色:','ru':':гребец_темныи_тон_кожи:','tr':':sandalda_kürek_çeken_erkek_orta_koyu_cilt_tonu:','ar':':رجل_يجدف_بقارب_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6A3\U0001F3FE\U0000200D\U00002642':{'en':':man_rowing_boat_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_im_ruderboot_mitteldunkle_hautfarbe:','es':':hombre_remando_en_un_bote_tono_de_piel_oscuro_medio:','fr':':rameur_dans_une_barque_peau_mate:','ja':':ボートをこぐ男_やや濃い肌色:','ko':':노젓는_남자_진한_갈색_피부:','pt':':homem_remando_pele_morena_escura:','it':':uomo_in_barca_a_remi_carnagione_abbastanza_scura:','fa':':پاروزن_مرد_پوست_گندمی:','id':':pria_mendayung_perahu_warna_kulit_gelap-sedang:','zh':':男生划船_中等深肤色:','ru':':гребец_темный_тон_кожи:','tr':':sandalda_kürek_çeken_erkek_orta_koyu_cilt_tonu:','ar':':رجل_يجدف_بقارب_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6A3\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_rowing_boat_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':mann_im_ruderboot_mittelhelle_hautfarbe:','es':':hombre_remando_en_un_bote_tono_de_piel_claro_medio:','fr':':rameur_dans_une_barque_peau_moyennement_claire:','ja':':ボートをこぐ男_やや薄い肌色:','ko':':노젓는_남자_연한_갈색_피부:','pt':':homem_remando_pele_morena_clara:','it':':uomo_in_barca_a_remi_carnagione_abbastanza_chiara:','fa':':پاروزن_مرد_پوست_روشن:','id':':pria_mendayung_perahu_warna_kulit_cerah-sedang:','zh':':男生划船_中等浅肤色:','ru':':гребец_светлыи_тон_кожи:','tr':':sandalda_kürek_çeken_erkek_orta_açık_cilt_tonu:','ar':':رجل_يجدف_بقارب_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6A3\U0001F3FC\U0000200D\U00002642':{'en':':man_rowing_boat_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_im_ruderboot_mittelhelle_hautfarbe:','es':':hombre_remando_en_un_bote_tono_de_piel_claro_medio:','fr':':rameur_dans_une_barque_peau_moyennement_claire:','ja':':ボートをこぐ男_やや薄い肌色:','ko':':노젓는_남자_연한_갈색_피부:','pt':':homem_remando_pele_morena_clara:','it':':uomo_in_barca_a_remi_carnagione_abbastanza_chiara:','fa':':پاروزن_مرد_پوست_روشن:','id':':pria_mendayung_perahu_warna_kulit_cerah-sedang:','zh':':男生划船_中等浅肤色:','ru':':гребец_светлый_тон_кожи:','tr':':sandalda_kürek_çeken_erkek_orta_açık_cilt_tonu:','ar':':رجل_يجدف_بقارب_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6A3\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_rowing_boat_medium_skin_tone:','status':fully_qualified,'E':4,'de':':mann_im_ruderboot_mittlere_hautfarbe:','es':':hombre_remando_en_un_bote_tono_de_piel_medio:','fr':':rameur_dans_une_barque_peau_légèrement_mate:','ja':':ボートをこぐ男_中間の肌色:','ko':':노젓는_남자_갈색_피부:','pt':':homem_remando_pele_morena:','it':':uomo_in_barca_a_remi_carnagione_olivastra:','fa':':پاروزن_مرد_پوست_طلایی:','id':':pria_mendayung_perahu_warna_kulit_sedang:','zh':':男生划船_中等肤色:','ru':':гребец_среднии_тон_кожи:','tr':':sandalda_kürek_çeken_erkek_orta_cilt_tonu:','ar':':رجل_يجدف_بقارب_بشرة_بلون_معتدل:'},'\U0001F6A3\U0001F3FD\U0000200D\U00002642':{'en':':man_rowing_boat_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_im_ruderboot_mittlere_hautfarbe:','es':':hombre_remando_en_un_bote_tono_de_piel_medio:','fr':':rameur_dans_une_barque_peau_légèrement_mate:','ja':':ボートをこぐ男_中間の肌色:','ko':':노젓는_남자_갈색_피부:','pt':':homem_remando_pele_morena:','it':':uomo_in_barca_a_remi_carnagione_olivastra:','fa':':پاروزن_مرد_پوست_طلایی:','id':':pria_mendayung_perahu_warna_kulit_sedang:','zh':':男生划船_中等肤色:','ru':':гребец_средний_тон_кожи:','tr':':sandalda_kürek_çeken_erkek_orta_cilt_tonu:','ar':':رجل_يجدف_بقارب_بشرة_بلون_معتدل:'},'\U0001F3C3\U0000200D\U00002642\U0000FE0F':{'en':':man_running:','status':fully_qualified,'E':4,'alias':[':running_man:'],'de':':laufender_mann:','es':':hombre_corriendo:','fr':':homme_qui_court:','ja':':走る男:','ko':':뛰는_남자:','pt':':homem_correndo:','it':':uomo_che_corre:','fa':':مرد_دونده:','id':':pria_berlari:','zh':':男生跑步:','ru':':мужчина_бежит:','tr':':koşan_erkek:','ar':':راكض:'},'\U0001F3C3\U0000200D\U00002642':{'en':':man_running:','status':minimally_qualified,'E':4,'alias':[':running_man:'],'de':':laufender_mann:','es':':hombre_corriendo:','fr':':homme_qui_court:','ja':':走る男:','ko':':뛰는_남자:','pt':':homem_correndo:','it':':uomo_che_corre:','fa':':مرد_دونده:','id':':pria_berlari:','zh':':男生跑步:','ru':':мужчина_бежит:','tr':':koşan_erkek:','ar':':راكض:'},'\U0001F3C3\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_running_dark_skin_tone:','status':fully_qualified,'E':4,'de':':laufender_mann_dunkle_hautfarbe:','es':':hombre_corriendo_tono_de_piel_oscuro:','fr':':homme_qui_court_peau_foncée:','ja':':走る男_濃い肌色:','ko':':뛰는_남자_검은색_피부:','pt':':homem_correndo_pele_escura:','it':':uomo_che_corre_carnagione_scura:','fa':':مرد_دونده_پوست_آبنوسی:','id':':pria_berlari_warna_kulit_gelap:','zh':':男生跑步_较深肤色:','ru':':мужчина_бежит_очень_темныи_тон_кожи:','tr':':koşan_erkek_koyu_cilt_tonu:','ar':':راكض_بشرة_بلون_غامق:'},'\U0001F3C3\U0001F3FF\U0000200D\U00002642':{'en':':man_running_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':laufender_mann_dunkle_hautfarbe:','es':':hombre_corriendo_tono_de_piel_oscuro:','fr':':homme_qui_court_peau_foncée:','ja':':走る男_濃い肌色:','ko':':뛰는_남자_검은색_피부:','pt':':homem_correndo_pele_escura:','it':':uomo_che_corre_carnagione_scura:','fa':':مرد_دونده_پوست_آبنوسی:','id':':pria_berlari_warna_kulit_gelap:','zh':':男生跑步_较深肤色:','ru':':мужчина_бежит_очень_темный_тон_кожи:','tr':':koşan_erkek_koyu_cilt_tonu:','ar':':راكض_بشرة_بلون_غامق:'},'\U0001F3C3\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':man_running_facing_right:','status':fully_qualified,'E':15.1,'de':':laufender_mann_nach_rechts:','es':':hombre_corriendo_hacia_la_derecha:','fr':':homme_qui_court_vers_la_droite:','ja':':走る男_右向き:','ko':':오른쪽을_향한_뛰는_남자:','pt':':homem_correndo_de_frente_para_a_direita:','it':':uomo_che_corre_a_destra:','fa':':مرد_دونده_به_سمت_راست:','id':':pria_berlari_menghadap_kanan:','zh':':男生跑步面向右边:','ru':':мужчина_бежит_направленные_вправо:','tr':':sağa_dönük_koşan_erkek:','ar':':راكض_متجه_لليمين:'},'\U0001F3C3\U0000200D\U00002642\U0000200D\U000027A1\U0000FE0F':{'en':':man_running_facing_right:','status':minimally_qualified,'E':15.1,'de':':laufender_mann_nach_rechts:','es':':hombre_corriendo_hacia_la_derecha:','fr':':homme_qui_court_vers_la_droite:','ja':':走る男_右向き:','ko':':오른쪽을_향한_뛰는_남자:','pt':':homem_correndo_de_frente_para_a_direita:','it':':uomo_che_corre_a_destra:','fa':':مرد_دونده_به_سمت_راست:','id':':pria_berlari_menghadap_kanan:','zh':':男生跑步面向右边:','ru':':мужчина_бежит_направленные_вправо:','tr':':sağa_dönük_koşan_erkek:','ar':':راكض_متجه_لليمين:'},'\U0001F3C3\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1':{'en':':man_running_facing_right:','status':minimally_qualified,'E':15.1,'de':':laufender_mann_nach_rechts:','es':':hombre_corriendo_hacia_la_derecha:','fr':':homme_qui_court_vers_la_droite:','ja':':走る男_右向き:','ko':':오른쪽을_향한_뛰는_남자:','pt':':homem_correndo_de_frente_para_a_direita:','it':':uomo_che_corre_a_destra:','fa':':مرد_دونده_به_سمت_راست:','id':':pria_berlari_menghadap_kanan:','zh':':男生跑步面向右边:','ru':':мужчина_бежит_направленные_вправо:','tr':':sağa_dönük_koşan_erkek:','ar':':راكض_متجه_لليمين:'},'\U0001F3C3\U0000200D\U00002642\U0000200D\U000027A1':{'en':':man_running_facing_right:','status':minimally_qualified,'E':15.1,'de':':laufender_mann_nach_rechts:','es':':hombre_corriendo_hacia_la_derecha:','fr':':homme_qui_court_vers_la_droite:','ja':':走る男_右向き:','ko':':오른쪽을_향한_뛰는_남자:','pt':':homem_correndo_de_frente_para_a_direita:','it':':uomo_che_corre_a_destra:','fa':':مرد_دونده_به_سمت_راست:','id':':pria_berlari_menghadap_kanan:','zh':':男生跑步面向右边:','ru':':мужчина_бежит_направленные_вправо:','tr':':sağa_dönük_koşan_erkek:','ar':':راكض_متجه_لليمين:'},'\U0001F3C3\U0001F3FF\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':man_running_facing_right_dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':laufender_mann_nach_rechts_dunkle_hautfarbe:','es':':hombre_corriendo_hacia_la_derecha_tono_de_piel_oscuro:','fr':':homme_qui_court_vers_la_droite_peau_foncée:','ja':':走る男_右向き_濃い肌色:','ko':':오른쪽을_향한_뛰는_남자_검은색_피부:','pt':':homem_correndo_de_frente_para_a_direita_pele_escura:','it':':uomo_che_corre_a_destra_carnagione_scura:','fa':':مرد_دونده_به_سمت_راستپوست_آبنوسی:','id':':pria_berlari_menghadap_kanan_warna_kulit_gelap:','zh':':男生跑步面向右边_较深肤色:','ru':':мужчина_бежит_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_koşan_erkek_koyu_cilt_tonu:','ar':':راكض_متجه_لليمين_بلون_غامق:'},'\U0001F3C3\U0001F3FF\U0000200D\U00002642\U0000200D\U000027A1\U0000FE0F':{'en':':man_running_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufender_mann_nach_rechts_dunkle_hautfarbe:','es':':hombre_corriendo_hacia_la_derecha_tono_de_piel_oscuro:','fr':':homme_qui_court_vers_la_droite_peau_foncée:','ja':':走る男_右向き_濃い肌色:','ko':':오른쪽을_향한_뛰는_남자_검은색_피부:','pt':':homem_correndo_de_frente_para_a_direita_pele_escura:','it':':uomo_che_corre_a_destra_carnagione_scura:','fa':':مرد_دونده_به_سمت_راستپوست_آبنوسی:','id':':pria_berlari_menghadap_kanan_warna_kulit_gelap:','zh':':男生跑步面向右边_较深肤色:','ru':':мужчина_бежит_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_koşan_erkek_koyu_cilt_tonu:','ar':':راكض_متجه_لليمين_بلون_غامق:'},'\U0001F3C3\U0001F3FF\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1':{'en':':man_running_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufender_mann_nach_rechts_dunkle_hautfarbe:','es':':hombre_corriendo_hacia_la_derecha_tono_de_piel_oscuro:','fr':':homme_qui_court_vers_la_droite_peau_foncée:','ja':':走る男_右向き_濃い肌色:','ko':':오른쪽을_향한_뛰는_남자_검은색_피부:','pt':':homem_correndo_de_frente_para_a_direita_pele_escura:','it':':uomo_che_corre_a_destra_carnagione_scura:','fa':':مرد_دونده_به_سمت_راستپوست_آبنوسی:','id':':pria_berlari_menghadap_kanan_warna_kulit_gelap:','zh':':男生跑步面向右边_较深肤色:','ru':':мужчина_бежит_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_koşan_erkek_koyu_cilt_tonu:','ar':':راكض_متجه_لليمين_بلون_غامق:'},'\U0001F3C3\U0001F3FF\U0000200D\U00002642\U0000200D\U000027A1':{'en':':man_running_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufender_mann_nach_rechts_dunkle_hautfarbe:','es':':hombre_corriendo_hacia_la_derecha_tono_de_piel_oscuro:','fr':':homme_qui_court_vers_la_droite_peau_foncée:','ja':':走る男_右向き_濃い肌色:','ko':':오른쪽을_향한_뛰는_남자_검은색_피부:','pt':':homem_correndo_de_frente_para_a_direita_pele_escura:','it':':uomo_che_corre_a_destra_carnagione_scura:','fa':':مرد_دونده_به_سمت_راستپوست_آبنوسی:','id':':pria_berlari_menghadap_kanan_warna_kulit_gelap:','zh':':男生跑步面向右边_较深肤色:','ru':':мужчина_бежит_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_koşan_erkek_koyu_cilt_tonu:','ar':':راكض_متجه_لليمين_بلون_غامق:'},'\U0001F3C3\U0001F3FB\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':man_running_facing_right_light_skin_tone:','status':fully_qualified,'E':15.1,'de':':laufender_mann_nach_rechts_helle_hautfarbe:','es':':hombre_corriendo_hacia_la_derecha_tono_de_piel_claro:','fr':':homme_qui_court_vers_la_droite_peau_claire:','ja':':走る男_右向き_薄い肌色:','ko':':오른쪽을_향한_뛰는_남자_하얀_피부:','pt':':homem_correndo_de_frente_para_a_direita_pele_clara:','it':':uomo_che_corre_a_destra_carnagione_chiara:','fa':':مرد_دونده_به_سمت_راست_پوست_سفید:','id':':pria_berlari_menghadap_kanan_warna_kulit_cerah:','zh':':男生跑步面向右边_较浅肤色:','ru':':мужчина_бежит_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_koşan_erkek_açık_cilt_tonu:','ar':':راكض_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F3C3\U0001F3FB\U0000200D\U00002642\U0000200D\U000027A1\U0000FE0F':{'en':':man_running_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufender_mann_nach_rechts_helle_hautfarbe:','es':':hombre_corriendo_hacia_la_derecha_tono_de_piel_claro:','fr':':homme_qui_court_vers_la_droite_peau_claire:','ja':':走る男_右向き_薄い肌色:','ko':':오른쪽을_향한_뛰는_남자_하얀_피부:','pt':':homem_correndo_de_frente_para_a_direita_pele_clara:','it':':uomo_che_corre_a_destra_carnagione_chiara:','fa':':مرد_دونده_به_سمت_راست_پوست_سفید:','id':':pria_berlari_menghadap_kanan_warna_kulit_cerah:','zh':':男生跑步面向右边_较浅肤色:','ru':':мужчина_бежит_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_koşan_erkek_açık_cilt_tonu:','ar':':راكض_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F3C3\U0001F3FB\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1':{'en':':man_running_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufender_mann_nach_rechts_helle_hautfarbe:','es':':hombre_corriendo_hacia_la_derecha_tono_de_piel_claro:','fr':':homme_qui_court_vers_la_droite_peau_claire:','ja':':走る男_右向き_薄い肌色:','ko':':오른쪽을_향한_뛰는_남자_하얀_피부:','pt':':homem_correndo_de_frente_para_a_direita_pele_clara:','it':':uomo_che_corre_a_destra_carnagione_chiara:','fa':':مرد_دونده_به_سمت_راست_پوست_سفید:','id':':pria_berlari_menghadap_kanan_warna_kulit_cerah:','zh':':男生跑步面向右边_较浅肤色:','ru':':мужчина_бежит_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_koşan_erkek_açık_cilt_tonu:','ar':':راكض_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F3C3\U0001F3FB\U0000200D\U00002642\U0000200D\U000027A1':{'en':':man_running_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufender_mann_nach_rechts_helle_hautfarbe:','es':':hombre_corriendo_hacia_la_derecha_tono_de_piel_claro:','fr':':homme_qui_court_vers_la_droite_peau_claire:','ja':':走る男_右向き_薄い肌色:','ko':':오른쪽을_향한_뛰는_남자_하얀_피부:','pt':':homem_correndo_de_frente_para_a_direita_pele_clara:','it':':uomo_che_corre_a_destra_carnagione_chiara:','fa':':مرد_دونده_به_سمت_راست_پوست_سفید:','id':':pria_berlari_menghadap_kanan_warna_kulit_cerah:','zh':':男生跑步面向右边_较浅肤色:','ru':':мужчина_бежит_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_koşan_erkek_açık_cilt_tonu:','ar':':راكض_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F3C3\U0001F3FE\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':man_running_facing_right_medium-dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':laufender_mann_nach_rechts_mitteldunkle_hautfarbe:','es':':hombre_corriendo_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':homme_qui_court_vers_la_droite_peau_mate:','ja':':走る男_右向き_やや濃い肌色:','ko':':오른쪽을_향한_뛰는_남자_진한_갈색_피부:','pt':':homem_correndo_de_frente_para_a_direita_pele_morena_escura:','it':':uomo_che_corre_a_destra_carnagione_abbastanza_scura:','fa':':مرد_دونده_به_سمت_راست_پوست_گندمی:','id':':pria_berlari_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':男生跑步面向右边_中等深肤色:','ru':':мужчина_бежит_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_koşan_erkek_orta_koyu_cilt_tonu:','ar':':راكض_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F3C3\U0001F3FE\U0000200D\U00002642\U0000200D\U000027A1\U0000FE0F':{'en':':man_running_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufender_mann_nach_rechts_mitteldunkle_hautfarbe:','es':':hombre_corriendo_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':homme_qui_court_vers_la_droite_peau_mate:','ja':':走る男_右向き_やや濃い肌色:','ko':':오른쪽을_향한_뛰는_남자_진한_갈색_피부:','pt':':homem_correndo_de_frente_para_a_direita_pele_morena_escura:','it':':uomo_che_corre_a_destra_carnagione_abbastanza_scura:','fa':':مرد_دونده_به_سمت_راست_پوست_گندمی:','id':':pria_berlari_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':男生跑步面向右边_中等深肤色:','ru':':мужчина_бежит_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_koşan_erkek_orta_koyu_cilt_tonu:','ar':':راكض_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F3C3\U0001F3FE\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1':{'en':':man_running_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufender_mann_nach_rechts_mitteldunkle_hautfarbe:','es':':hombre_corriendo_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':homme_qui_court_vers_la_droite_peau_mate:','ja':':走る男_右向き_やや濃い肌色:','ko':':오른쪽을_향한_뛰는_남자_진한_갈색_피부:','pt':':homem_correndo_de_frente_para_a_direita_pele_morena_escura:','it':':uomo_che_corre_a_destra_carnagione_abbastanza_scura:','fa':':مرد_دونده_به_سمت_راست_پوست_گندمی:','id':':pria_berlari_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':男生跑步面向右边_中等深肤色:','ru':':мужчина_бежит_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_koşan_erkek_orta_koyu_cilt_tonu:','ar':':راكض_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F3C3\U0001F3FE\U0000200D\U00002642\U0000200D\U000027A1':{'en':':man_running_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufender_mann_nach_rechts_mitteldunkle_hautfarbe:','es':':hombre_corriendo_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':homme_qui_court_vers_la_droite_peau_mate:','ja':':走る男_右向き_やや濃い肌色:','ko':':오른쪽을_향한_뛰는_남자_진한_갈색_피부:','pt':':homem_correndo_de_frente_para_a_direita_pele_morena_escura:','it':':uomo_che_corre_a_destra_carnagione_abbastanza_scura:','fa':':مرد_دونده_به_سمت_راست_پوست_گندمی:','id':':pria_berlari_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':男生跑步面向右边_中等深肤色:','ru':':мужчина_бежит_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_koşan_erkek_orta_koyu_cilt_tonu:','ar':':راكض_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F3C3\U0001F3FC\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':man_running_facing_right_medium-light_skin_tone:','status':fully_qualified,'E':15.1,'de':':laufender_mann_nach_rechts_mittelhelle_hautfarbe:','es':':hombre_corriendo_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':homme_qui_court_vers_la_droite_peau_moyennement_claire:','ja':':走る男_右向き_やや薄い肌色:','ko':':오른쪽을_향한_뛰는_남자_연한_갈색_피부:','pt':':homem_correndo_de_frente_para_a_direita_pele_morena_clara:','it':':uomo_che_corre_a_destra_carnagione_abbastanza_chiara:','fa':':مرد_دونده_به_سمت_راست_پوست_روشن:','id':':pria_berlari_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':男生跑步面向右边_中等浅肤色:','ru':':мужчина_бежит_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_koşan_erkek_orta_açık_cilt_tonu:','ar':':راكض_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F3C3\U0001F3FC\U0000200D\U00002642\U0000200D\U000027A1\U0000FE0F':{'en':':man_running_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufender_mann_nach_rechts_mittelhelle_hautfarbe:','es':':hombre_corriendo_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':homme_qui_court_vers_la_droite_peau_moyennement_claire:','ja':':走る男_右向き_やや薄い肌色:','ko':':오른쪽을_향한_뛰는_남자_연한_갈색_피부:','pt':':homem_correndo_de_frente_para_a_direita_pele_morena_clara:','it':':uomo_che_corre_a_destra_carnagione_abbastanza_chiara:','fa':':مرد_دونده_به_سمت_راست_پوست_روشن:','id':':pria_berlari_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':男生跑步面向右边_中等浅肤色:','ru':':мужчина_бежит_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_koşan_erkek_orta_açık_cilt_tonu:','ar':':راكض_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F3C3\U0001F3FC\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1':{'en':':man_running_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufender_mann_nach_rechts_mittelhelle_hautfarbe:','es':':hombre_corriendo_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':homme_qui_court_vers_la_droite_peau_moyennement_claire:','ja':':走る男_右向き_やや薄い肌色:','ko':':오른쪽을_향한_뛰는_남자_연한_갈색_피부:','pt':':homem_correndo_de_frente_para_a_direita_pele_morena_clara:','it':':uomo_che_corre_a_destra_carnagione_abbastanza_chiara:','fa':':مرد_دونده_به_سمت_راست_پوست_روشن:','id':':pria_berlari_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':男生跑步面向右边_中等浅肤色:','ru':':мужчина_бежит_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_koşan_erkek_orta_açık_cilt_tonu:','ar':':راكض_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F3C3\U0001F3FC\U0000200D\U00002642\U0000200D\U000027A1':{'en':':man_running_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufender_mann_nach_rechts_mittelhelle_hautfarbe:','es':':hombre_corriendo_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':homme_qui_court_vers_la_droite_peau_moyennement_claire:','ja':':走る男_右向き_やや薄い肌色:','ko':':오른쪽을_향한_뛰는_남자_연한_갈색_피부:','pt':':homem_correndo_de_frente_para_a_direita_pele_morena_clara:','it':':uomo_che_corre_a_destra_carnagione_abbastanza_chiara:','fa':':مرد_دونده_به_سمت_راست_پوست_روشن:','id':':pria_berlari_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':男生跑步面向右边_中等浅肤色:','ru':':мужчина_бежит_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_koşan_erkek_orta_açık_cilt_tonu:','ar':':راكض_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F3C3\U0001F3FD\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':man_running_facing_right_medium_skin_tone:','status':fully_qualified,'E':15.1,'de':':laufender_mann_nach_rechts_mittlere_hautfarbe:','es':':hombre_corriendo_hacia_la_derecha_tono_de_piel_medio:','fr':':homme_qui_court_vers_la_droite_peau_légèrement_mate:','ja':':走る男_右向き_中間の肌色:','ko':':오른쪽을_향한_뛰는_남자_갈색_피부:','pt':':homem_correndo_de_frente_para_a_direita_pele_morena:','it':':uomo_che_corre_a_destra_carnagione_olivastra:','fa':':مرد_دونده_به_سمت_راست_پوست_طلایی:','id':':pria_berlari_menghadap_kanan_warna_kulit_sedang:','zh':':男生跑步面向右边_中等肤色:','ru':':мужчина_бежит_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_koşan_erkek_orta_cilt_tonu:','ar':':راكض_متجه_لليمين_بلون_معتدل:'},'\U0001F3C3\U0001F3FD\U0000200D\U00002642\U0000200D\U000027A1\U0000FE0F':{'en':':man_running_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufender_mann_nach_rechts_mittlere_hautfarbe:','es':':hombre_corriendo_hacia_la_derecha_tono_de_piel_medio:','fr':':homme_qui_court_vers_la_droite_peau_légèrement_mate:','ja':':走る男_右向き_中間の肌色:','ko':':오른쪽을_향한_뛰는_남자_갈색_피부:','pt':':homem_correndo_de_frente_para_a_direita_pele_morena:','it':':uomo_che_corre_a_destra_carnagione_olivastra:','fa':':مرد_دونده_به_سمت_راست_پوست_طلایی:','id':':pria_berlari_menghadap_kanan_warna_kulit_sedang:','zh':':男生跑步面向右边_中等肤色:','ru':':мужчина_бежит_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_koşan_erkek_orta_cilt_tonu:','ar':':راكض_متجه_لليمين_بلون_معتدل:'},'\U0001F3C3\U0001F3FD\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1':{'en':':man_running_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufender_mann_nach_rechts_mittlere_hautfarbe:','es':':hombre_corriendo_hacia_la_derecha_tono_de_piel_medio:','fr':':homme_qui_court_vers_la_droite_peau_légèrement_mate:','ja':':走る男_右向き_中間の肌色:','ko':':오른쪽을_향한_뛰는_남자_갈색_피부:','pt':':homem_correndo_de_frente_para_a_direita_pele_morena:','it':':uomo_che_corre_a_destra_carnagione_olivastra:','fa':':مرد_دونده_به_سمت_راست_پوست_طلایی:','id':':pria_berlari_menghadap_kanan_warna_kulit_sedang:','zh':':男生跑步面向右边_中等肤色:','ru':':мужчина_бежит_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_koşan_erkek_orta_cilt_tonu:','ar':':راكض_متجه_لليمين_بلون_معتدل:'},'\U0001F3C3\U0001F3FD\U0000200D\U00002642\U0000200D\U000027A1':{'en':':man_running_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufender_mann_nach_rechts_mittlere_hautfarbe:','es':':hombre_corriendo_hacia_la_derecha_tono_de_piel_medio:','fr':':homme_qui_court_vers_la_droite_peau_légèrement_mate:','ja':':走る男_右向き_中間の肌色:','ko':':오른쪽을_향한_뛰는_남자_갈색_피부:','pt':':homem_correndo_de_frente_para_a_direita_pele_morena:','it':':uomo_che_corre_a_destra_carnagione_olivastra:','fa':':مرد_دونده_به_سمت_راست_پوست_طلایی:','id':':pria_berlari_menghadap_kanan_warna_kulit_sedang:','zh':':男生跑步面向右边_中等肤色:','ru':':мужчина_бежит_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_koşan_erkek_orta_cilt_tonu:','ar':':راكض_متجه_لليمين_بلون_معتدل:'},'\U0001F3C3\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_running_light_skin_tone:','status':fully_qualified,'E':4,'de':':laufender_mann_helle_hautfarbe:','es':':hombre_corriendo_tono_de_piel_claro:','fr':':homme_qui_court_peau_claire:','ja':':走る男_薄い肌色:','ko':':뛰는_남자_하얀_피부:','pt':':homem_correndo_pele_clara:','it':':uomo_che_corre_carnagione_chiara:','fa':':مرد_دونده_پوست_سفید:','id':':pria_berlari_warna_kulit_cerah:','zh':':男生跑步_较浅肤色:','ru':':мужчина_бежит_очень_светлыи_тон_кожи:','tr':':koşan_erkek_açık_cilt_tonu:','ar':':راكض_بشرة_بلون_فاتح:'},'\U0001F3C3\U0001F3FB\U0000200D\U00002642':{'en':':man_running_light_skin_tone:','status':minimally_qualified,'E':4,'de':':laufender_mann_helle_hautfarbe:','es':':hombre_corriendo_tono_de_piel_claro:','fr':':homme_qui_court_peau_claire:','ja':':走る男_薄い肌色:','ko':':뛰는_남자_하얀_피부:','pt':':homem_correndo_pele_clara:','it':':uomo_che_corre_carnagione_chiara:','fa':':مرد_دونده_پوست_سفید:','id':':pria_berlari_warna_kulit_cerah:','zh':':男生跑步_较浅肤色:','ru':':мужчина_бежит_очень_светлый_тон_кожи:','tr':':koşan_erkek_açık_cilt_tonu:','ar':':راكض_بشرة_بلون_فاتح:'},'\U0001F3C3\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_running_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':laufender_mann_mitteldunkle_hautfarbe:','es':':hombre_corriendo_tono_de_piel_oscuro_medio:','fr':':homme_qui_court_peau_mate:','ja':':走る男_やや濃い肌色:','ko':':뛰는_남자_진한_갈색_피부:','pt':':homem_correndo_pele_morena_escura:','it':':uomo_che_corre_carnagione_abbastanza_scura:','fa':':مرد_دونده_پوست_گندمی:','id':':pria_berlari_warna_kulit_gelap-sedang:','zh':':男生跑步_中等深肤色:','ru':':мужчина_бежит_темныи_тон_кожи:','tr':':koşan_erkek_orta_koyu_cilt_tonu:','ar':':راكض_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3C3\U0001F3FE\U0000200D\U00002642':{'en':':man_running_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':laufender_mann_mitteldunkle_hautfarbe:','es':':hombre_corriendo_tono_de_piel_oscuro_medio:','fr':':homme_qui_court_peau_mate:','ja':':走る男_やや濃い肌色:','ko':':뛰는_남자_진한_갈색_피부:','pt':':homem_correndo_pele_morena_escura:','it':':uomo_che_corre_carnagione_abbastanza_scura:','fa':':مرد_دونده_پوست_گندمی:','id':':pria_berlari_warna_kulit_gelap-sedang:','zh':':男生跑步_中等深肤色:','ru':':мужчина_бежит_темный_тон_кожи:','tr':':koşan_erkek_orta_koyu_cilt_tonu:','ar':':راكض_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3C3\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_running_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':laufender_mann_mittelhelle_hautfarbe:','es':':hombre_corriendo_tono_de_piel_claro_medio:','fr':':homme_qui_court_peau_moyennement_claire:','ja':':走る男_やや薄い肌色:','ko':':뛰는_남자_연한_갈색_피부:','pt':':homem_correndo_pele_morena_clara:','it':':uomo_che_corre_carnagione_abbastanza_chiara:','fa':':مرد_دونده_پوست_روشن:','id':':pria_berlari_warna_kulit_cerah-sedang:','zh':':男生跑步_中等浅肤色:','ru':':мужчина_бежит_светлыи_тон_кожи:','tr':':koşan_erkek_orta_açık_cilt_tonu:','ar':':راكض_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3C3\U0001F3FC\U0000200D\U00002642':{'en':':man_running_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':laufender_mann_mittelhelle_hautfarbe:','es':':hombre_corriendo_tono_de_piel_claro_medio:','fr':':homme_qui_court_peau_moyennement_claire:','ja':':走る男_やや薄い肌色:','ko':':뛰는_남자_연한_갈색_피부:','pt':':homem_correndo_pele_morena_clara:','it':':uomo_che_corre_carnagione_abbastanza_chiara:','fa':':مرد_دونده_پوست_روشن:','id':':pria_berlari_warna_kulit_cerah-sedang:','zh':':男生跑步_中等浅肤色:','ru':':мужчина_бежит_светлый_тон_кожи:','tr':':koşan_erkek_orta_açık_cilt_tonu:','ar':':راكض_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3C3\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_running_medium_skin_tone:','status':fully_qualified,'E':4,'de':':laufender_mann_mittlere_hautfarbe:','es':':hombre_corriendo_tono_de_piel_medio:','fr':':homme_qui_court_peau_légèrement_mate:','ja':':走る男_中間の肌色:','ko':':뛰는_남자_갈색_피부:','pt':':homem_correndo_pele_morena:','it':':uomo_che_corre_carnagione_olivastra:','fa':':مرد_دونده_پوست_طلایی:','id':':pria_berlari_warna_kulit_sedang:','zh':':男生跑步_中等肤色:','ru':':мужчина_бежит_среднии_тон_кожи:','tr':':koşan_erkek_orta_cilt_tonu:','ar':':راكض_بشرة_بلون_معتدل:'},'\U0001F3C3\U0001F3FD\U0000200D\U00002642':{'en':':man_running_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':laufender_mann_mittlere_hautfarbe:','es':':hombre_corriendo_tono_de_piel_medio:','fr':':homme_qui_court_peau_légèrement_mate:','ja':':走る男_中間の肌色:','ko':':뛰는_남자_갈색_피부:','pt':':homem_correndo_pele_morena:','it':':uomo_che_corre_carnagione_olivastra:','fa':':مرد_دونده_پوست_طلایی:','id':':pria_berlari_warna_kulit_sedang:','zh':':男生跑步_中等肤色:','ru':':мужчина_бежит_средний_тон_кожи:','tr':':koşan_erkek_orta_cilt_tonu:','ar':':راكض_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U0001F52C':{'en':':man_scientist:','status':fully_qualified,'E':4,'de':':wissenschaftler:','es':':profesional_de_la_ciencia_hombre:','fr':':scientifique_homme:','ja':':男性科学者:','ko':':남자_과학자:','pt':':cientista_homem:','it':':scienziato:','fa':':دانشمند_مرد:','id':':ilmuwan_pria:','zh':':男科学家:','ru':':мужчина_ученый:','tr':':erkek_bilim_insanı:','ar':':عالم:'},'\U0001F468\U0001F3FF\U0000200D\U0001F52C':{'en':':man_scientist_dark_skin_tone:','status':fully_qualified,'E':4,'de':':wissenschaftler_dunkle_hautfarbe:','es':':profesional_de_la_ciencia_hombre_tono_de_piel_oscuro:','fr':':scientifique_homme_peau_foncée:','ja':':男性科学者_濃い肌色:','ko':':남자_과학자_검은색_피부:','pt':':cientista_homem_pele_escura:','it':':scienziato_carnagione_scura:','fa':':دانشمند_مرد_پوست_آبنوسی:','id':':ilmuwan_pria_warna_kulit_gelap:','zh':':男科学家_较深肤色:','ru':':мужчина_ученый_очень_темный_тон_кожи:','tr':':erkek_bilim_insanı_koyu_cilt_tonu:','ar':':عالم_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U0001F52C':{'en':':man_scientist_light_skin_tone:','status':fully_qualified,'E':4,'de':':wissenschaftler_helle_hautfarbe:','es':':profesional_de_la_ciencia_hombre_tono_de_piel_claro:','fr':':scientifique_homme_peau_claire:','ja':':男性科学者_薄い肌色:','ko':':남자_과학자_하얀_피부:','pt':':cientista_homem_pele_clara:','it':':scienziato_carnagione_chiara:','fa':':دانشمند_مرد_پوست_سفید:','id':':ilmuwan_pria_warna_kulit_cerah:','zh':':男科学家_较浅肤色:','ru':':мужчина_ученый_очень_светлый_тон_кожи:','tr':':erkek_bilim_insanı_açık_cilt_tonu:','ar':':عالم_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F52C':{'en':':man_scientist_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':wissenschaftler_mitteldunkle_hautfarbe:','es':':profesional_de_la_ciencia_hombre_tono_de_piel_oscuro_medio:','fr':':scientifique_homme_peau_mate:','ja':':男性科学者_やや濃い肌色:','ko':':남자_과학자_진한_갈색_피부:','pt':':cientista_homem_pele_morena_escura:','it':':scienziato_carnagione_abbastanza_scura:','fa':':دانشمند_مرد_پوست_گندمی:','id':':ilmuwan_pria_warna_kulit_gelap-sedang:','zh':':男科学家_中等深肤色:','ru':':мужчина_ученый_темный_тон_кожи:','tr':':erkek_bilim_insanı_orta_koyu_cilt_tonu:','ar':':عالم_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U0001F52C':{'en':':man_scientist_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':wissenschaftler_mittelhelle_hautfarbe:','es':':profesional_de_la_ciencia_hombre_tono_de_piel_claro_medio:','fr':':scientifique_homme_peau_moyennement_claire:','ja':':男性科学者_やや薄い肌色:','ko':':남자_과학자_연한_갈색_피부:','pt':':cientista_homem_pele_morena_clara:','it':':scienziato_carnagione_abbastanza_chiara:','fa':':دانشمند_مرد_پوست_روشن:','id':':ilmuwan_pria_warna_kulit_cerah-sedang:','zh':':男科学家_中等浅肤色:','ru':':мужчина_ученый_светлый_тон_кожи:','tr':':erkek_bilim_insanı_orta_açık_cilt_tonu:','ar':':عالم_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F52C':{'en':':man_scientist_medium_skin_tone:','status':fully_qualified,'E':4,'de':':wissenschaftler_mittlere_hautfarbe:','es':':profesional_de_la_ciencia_hombre_tono_de_piel_medio:','fr':':scientifique_homme_peau_légèrement_mate:','ja':':男性科学者_中間の肌色:','ko':':남자_과학자_갈색_피부:','pt':':cientista_homem_pele_morena:','it':':scienziato_carnagione_olivastra:','fa':':دانشمند_مرد_پوست_طلایی:','id':':ilmuwan_pria_warna_kulit_sedang:','zh':':男科学家_中等肤色:','ru':':мужчина_ученый_средний_тон_кожи:','tr':':erkek_bilim_insanı_orta_cilt_tonu:','ar':':عالم_بشرة_بلون_معتدل:'},'\U0001F937\U0000200D\U00002642\U0000FE0F':{'en':':man_shrugging:','status':fully_qualified,'E':4,'de':':schulterzuckender_mann:','es':':hombre_encogido_de_hombros:','fr':':homme_qui_hausse_les_épaules:','ja':':お手上げする男:','ko':':어깨를_으쓱하는_남자:','pt':':homem_dando_de_ombros:','it':':uomo_che_scrolla_le_spalle:','fa':':مرد_شانه_بالا_می_اندازد:','id':':pria_mengangkat_bahu:','zh':':男生耸肩:','ru':':мужчина_пожимает_плечами:','tr':':omuz_silken_erkek:','ar':':رجل_لا_يبالي:'},'\U0001F937\U0000200D\U00002642':{'en':':man_shrugging:','status':minimally_qualified,'E':4,'de':':schulterzuckender_mann:','es':':hombre_encogido_de_hombros:','fr':':homme_qui_hausse_les_épaules:','ja':':お手上げする男:','ko':':어깨를_으쓱하는_남자:','pt':':homem_dando_de_ombros:','it':':uomo_che_scrolla_le_spalle:','fa':':مرد_شانه_بالا_می_اندازد:','id':':pria_mengangkat_bahu:','zh':':男生耸肩:','ru':':мужчина_пожимает_плечами:','tr':':omuz_silken_erkek:','ar':':رجل_لا_يبالي:'},'\U0001F937\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_shrugging_dark_skin_tone:','status':fully_qualified,'E':4,'de':':schulterzuckender_mann_dunkle_hautfarbe:','es':':hombre_encogido_de_hombros_tono_de_piel_oscuro:','fr':':homme_qui_hausse_les_épaules_peau_foncée:','ja':':お手上げする男_濃い肌色:','ko':':어깨를_으쓱하는_남자_검은색_피부:','pt':':homem_dando_de_ombros_pele_escura:','it':':uomo_che_scrolla_le_spalle_carnagione_scura:','fa':':مرد_شانه_بالا_می_اندازد_پوست_آبنوسی:','id':':pria_mengangkat_bahu_warna_kulit_gelap:','zh':':男生耸肩_较深肤色:','ru':':мужчина_пожимает_плечами_очень_темныи_тон_кожи:','tr':':omuz_silken_erkek_koyu_cilt_tonu:','ar':':رجل_لا_يبالي_بشرة_بلون_غامق:'},'\U0001F937\U0001F3FF\U0000200D\U00002642':{'en':':man_shrugging_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':schulterzuckender_mann_dunkle_hautfarbe:','es':':hombre_encogido_de_hombros_tono_de_piel_oscuro:','fr':':homme_qui_hausse_les_épaules_peau_foncée:','ja':':お手上げする男_濃い肌色:','ko':':어깨를_으쓱하는_남자_검은색_피부:','pt':':homem_dando_de_ombros_pele_escura:','it':':uomo_che_scrolla_le_spalle_carnagione_scura:','fa':':مرد_شانه_بالا_می_اندازد_پوست_آبنوسی:','id':':pria_mengangkat_bahu_warna_kulit_gelap:','zh':':男生耸肩_较深肤色:','ru':':мужчина_пожимает_плечами_очень_темный_тон_кожи:','tr':':omuz_silken_erkek_koyu_cilt_tonu:','ar':':رجل_لا_يبالي_بشرة_بلون_غامق:'},'\U0001F937\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_shrugging_light_skin_tone:','status':fully_qualified,'E':4,'de':':schulterzuckender_mann_helle_hautfarbe:','es':':hombre_encogido_de_hombros_tono_de_piel_claro:','fr':':homme_qui_hausse_les_épaules_peau_claire:','ja':':お手上げする男_薄い肌色:','ko':':어깨를_으쓱하는_남자_하얀_피부:','pt':':homem_dando_de_ombros_pele_clara:','it':':uomo_che_scrolla_le_spalle_carnagione_chiara:','fa':':مرد_شانه_بالا_می_اندازد_پوست_سفید:','id':':pria_mengangkat_bahu_warna_kulit_cerah:','zh':':男生耸肩_较浅肤色:','ru':':мужчина_пожимает_плечами_очень_светлыи_тон_кожи:','tr':':omuz_silken_erkek_açık_cilt_tonu:','ar':':رجل_لا_يبالي_بشرة_بلون_فاتح:'},'\U0001F937\U0001F3FB\U0000200D\U00002642':{'en':':man_shrugging_light_skin_tone:','status':minimally_qualified,'E':4,'de':':schulterzuckender_mann_helle_hautfarbe:','es':':hombre_encogido_de_hombros_tono_de_piel_claro:','fr':':homme_qui_hausse_les_épaules_peau_claire:','ja':':お手上げする男_薄い肌色:','ko':':어깨를_으쓱하는_남자_하얀_피부:','pt':':homem_dando_de_ombros_pele_clara:','it':':uomo_che_scrolla_le_spalle_carnagione_chiara:','fa':':مرد_شانه_بالا_می_اندازد_پوست_سفید:','id':':pria_mengangkat_bahu_warna_kulit_cerah:','zh':':男生耸肩_较浅肤色:','ru':':мужчина_пожимает_плечами_очень_светлый_тон_кожи:','tr':':omuz_silken_erkek_açık_cilt_tonu:','ar':':رجل_لا_يبالي_بشرة_بلون_فاتح:'},'\U0001F937\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_shrugging_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':schulterzuckender_mann_mitteldunkle_hautfarbe:','es':':hombre_encogido_de_hombros_tono_de_piel_oscuro_medio:','fr':':homme_qui_hausse_les_épaules_peau_mate:','ja':':お手上げする男_やや濃い肌色:','ko':':어깨를_으쓱하는_남자_진한_갈색_피부:','pt':':homem_dando_de_ombros_pele_morena_escura:','it':':uomo_che_scrolla_le_spalle_carnagione_abbastanza_scura:','fa':':مرد_شانه_بالا_می_اندازد_پوست_گندمی:','id':':pria_mengangkat_bahu_warna_kulit_gelap-sedang:','zh':':男生耸肩_中等深肤色:','ru':':мужчина_пожимает_плечами_темныи_тон_кожи:','tr':':omuz_silken_erkek_orta_koyu_cilt_tonu:','ar':':رجل_لا_يبالي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F937\U0001F3FE\U0000200D\U00002642':{'en':':man_shrugging_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':schulterzuckender_mann_mitteldunkle_hautfarbe:','es':':hombre_encogido_de_hombros_tono_de_piel_oscuro_medio:','fr':':homme_qui_hausse_les_épaules_peau_mate:','ja':':お手上げする男_やや濃い肌色:','ko':':어깨를_으쓱하는_남자_진한_갈색_피부:','pt':':homem_dando_de_ombros_pele_morena_escura:','it':':uomo_che_scrolla_le_spalle_carnagione_abbastanza_scura:','fa':':مرد_شانه_بالا_می_اندازد_پوست_گندمی:','id':':pria_mengangkat_bahu_warna_kulit_gelap-sedang:','zh':':男生耸肩_中等深肤色:','ru':':мужчина_пожимает_плечами_темный_тон_кожи:','tr':':omuz_silken_erkek_orta_koyu_cilt_tonu:','ar':':رجل_لا_يبالي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F937\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_shrugging_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':schulterzuckender_mann_mittelhelle_hautfarbe:','es':':hombre_encogido_de_hombros_tono_de_piel_claro_medio:','fr':':homme_qui_hausse_les_épaules_peau_moyennement_claire:','ja':':お手上げする男_やや薄い肌色:','ko':':어깨를_으쓱하는_남자_연한_갈색_피부:','pt':':homem_dando_de_ombros_pele_morena_clara:','it':':uomo_che_scrolla_le_spalle_carnagione_abbastanza_chiara:','fa':':مرد_شانه_بالا_می_اندازد_پوست_روشن:','id':':pria_mengangkat_bahu_warna_kulit_cerah-sedang:','zh':':男生耸肩_中等浅肤色:','ru':':мужчина_пожимает_плечами_светлыи_тон_кожи:','tr':':omuz_silken_erkek_orta_açık_cilt_tonu:','ar':':رجل_لا_يبالي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F937\U0001F3FC\U0000200D\U00002642':{'en':':man_shrugging_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':schulterzuckender_mann_mittelhelle_hautfarbe:','es':':hombre_encogido_de_hombros_tono_de_piel_claro_medio:','fr':':homme_qui_hausse_les_épaules_peau_moyennement_claire:','ja':':お手上げする男_やや薄い肌色:','ko':':어깨를_으쓱하는_남자_연한_갈색_피부:','pt':':homem_dando_de_ombros_pele_morena_clara:','it':':uomo_che_scrolla_le_spalle_carnagione_abbastanza_chiara:','fa':':مرد_شانه_بالا_می_اندازد_پوست_روشن:','id':':pria_mengangkat_bahu_warna_kulit_cerah-sedang:','zh':':男生耸肩_中等浅肤色:','ru':':мужчина_пожимает_плечами_светлый_тон_кожи:','tr':':omuz_silken_erkek_orta_açık_cilt_tonu:','ar':':رجل_لا_يبالي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F937\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_shrugging_medium_skin_tone:','status':fully_qualified,'E':4,'de':':schulterzuckender_mann_mittlere_hautfarbe:','es':':hombre_encogido_de_hombros_tono_de_piel_medio:','fr':':homme_qui_hausse_les_épaules_peau_légèrement_mate:','ja':':お手上げする男_中間の肌色:','ko':':어깨를_으쓱하는_남자_갈색_피부:','pt':':homem_dando_de_ombros_pele_morena:','it':':uomo_che_scrolla_le_spalle_carnagione_olivastra:','fa':':مرد_شانه_بالا_می_اندازد_پوست_طلایی:','id':':pria_mengangkat_bahu_warna_kulit_sedang:','zh':':男生耸肩_中等肤色:','ru':':мужчина_пожимает_плечами_среднии_тон_кожи:','tr':':omuz_silken_erkek_orta_cilt_tonu:','ar':':رجل_لا_يبالي_بشرة_بلون_معتدل:'},'\U0001F937\U0001F3FD\U0000200D\U00002642':{'en':':man_shrugging_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':schulterzuckender_mann_mittlere_hautfarbe:','es':':hombre_encogido_de_hombros_tono_de_piel_medio:','fr':':homme_qui_hausse_les_épaules_peau_légèrement_mate:','ja':':お手上げする男_中間の肌色:','ko':':어깨를_으쓱하는_남자_갈색_피부:','pt':':homem_dando_de_ombros_pele_morena:','it':':uomo_che_scrolla_le_spalle_carnagione_olivastra:','fa':':مرد_شانه_بالا_می_اندازد_پوست_طلایی:','id':':pria_mengangkat_bahu_warna_kulit_sedang:','zh':':男生耸肩_中等肤色:','ru':':мужчина_пожимает_плечами_средний_тон_кожи:','tr':':omuz_silken_erkek_orta_cilt_tonu:','ar':':رجل_لا_يبالي_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U0001F3A4':{'en':':man_singer:','status':fully_qualified,'E':4,'de':':sänger:','es':':cantante_hombre:','fr':':chanteur:','ja':':男性歌手:','ko':':남자_가수:','pt':':cantor_homem:','it':':cantante_uomo:','fa':':خواننده_مرد:','id':':penyanyi_pria:','zh':':男歌手:','ru':':мужчина_певец:','tr':':erkek_şarkıcı:','ar':':مطرب:'},'\U0001F468\U0001F3FF\U0000200D\U0001F3A4':{'en':':man_singer_dark_skin_tone:','status':fully_qualified,'E':4,'de':':sänger_dunkle_hautfarbe:','es':':cantante_hombre_tono_de_piel_oscuro:','fr':':chanteur_peau_foncée:','ja':':男性歌手_濃い肌色:','ko':':남자_가수_검은색_피부:','pt':':cantor_homem_pele_escura:','it':':cantante_uomo_carnagione_scura:','fa':':خواننده_مرد_پوست_آبنوسی:','id':':penyanyi_pria_warna_kulit_gelap:','zh':':男歌手_较深肤色:','ru':':мужчина_певец_очень_темный_тон_кожи:','tr':':erkek_şarkıcı_koyu_cilt_tonu:','ar':':مطرب_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U0001F3A4':{'en':':man_singer_light_skin_tone:','status':fully_qualified,'E':4,'de':':sänger_helle_hautfarbe:','es':':cantante_hombre_tono_de_piel_claro:','fr':':chanteur_peau_claire:','ja':':男性歌手_薄い肌色:','ko':':남자_가수_하얀_피부:','pt':':cantor_homem_pele_clara:','it':':cantante_uomo_carnagione_chiara:','fa':':خواننده_مرد_پوست_سفید:','id':':penyanyi_pria_warna_kulit_cerah:','zh':':男歌手_较浅肤色:','ru':':мужчина_певец_очень_светлый_тон_кожи:','tr':':erkek_şarkıcı_açık_cilt_tonu:','ar':':مطرب_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F3A4':{'en':':man_singer_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':sänger_mitteldunkle_hautfarbe:','es':':cantante_hombre_tono_de_piel_oscuro_medio:','fr':':chanteur_peau_mate:','ja':':男性歌手_やや濃い肌色:','ko':':남자_가수_진한_갈색_피부:','pt':':cantor_homem_pele_morena_escura:','it':':cantante_uomo_carnagione_abbastanza_scura:','fa':':خواننده_مرد_پوست_گندمی:','id':':penyanyi_pria_warna_kulit_gelap-sedang:','zh':':男歌手_中等深肤色:','ru':':мужчина_певец_темный_тон_кожи:','tr':':erkek_şarkıcı_orta_koyu_cilt_tonu:','ar':':مطرب_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U0001F3A4':{'en':':man_singer_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':sänger_mittelhelle_hautfarbe:','es':':cantante_hombre_tono_de_piel_claro_medio:','fr':':chanteur_peau_moyennement_claire:','ja':':男性歌手_やや薄い肌色:','ko':':남자_가수_연한_갈색_피부:','pt':':cantor_homem_pele_morena_clara:','it':':cantante_uomo_carnagione_abbastanza_chiara:','fa':':خواننده_مرد_پوست_روشن:','id':':penyanyi_pria_warna_kulit_cerah-sedang:','zh':':男歌手_中等浅肤色:','ru':':мужчина_певец_светлый_тон_кожи:','tr':':erkek_şarkıcı_orta_açık_cilt_tonu:','ar':':مطرب_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F3A4':{'en':':man_singer_medium_skin_tone:','status':fully_qualified,'E':4,'de':':sänger_mittlere_hautfarbe:','es':':cantante_hombre_tono_de_piel_medio:','fr':':chanteur_peau_légèrement_mate:','ja':':男性歌手_中間の肌色:','ko':':남자_가수_갈색_피부:','pt':':cantor_homem_pele_morena:','it':':cantante_uomo_carnagione_olivastra:','fa':':خواننده_مرد_پوست_طلایی:','id':':penyanyi_pria_warna_kulit_sedang:','zh':':男歌手_中等肤色:','ru':':мужчина_певец_средний_тон_кожи:','tr':':erkek_şarkıcı_orta_cilt_tonu:','ar':':مطرب_بشرة_بلون_معتدل:'},'\U0001F9CD\U0000200D\U00002642\U0000FE0F':{'en':':man_standing:','status':fully_qualified,'E':12,'alias':[':standing_man:'],'de':':stehender_mann:','es':':hombre_de_pie:','fr':':homme_debout:','ja':':立つ男:','ko':':서_있는_남자:','pt':':homem_em_pé:','it':':uomo_in_piedi:','fa':':مرد_ایستاده:','id':':pria_berdiri:','zh':':站立的男人:','ru':':мужчина_стоит:','tr':':ayakta_duran_erkek:','ar':':رجل_واقف:'},'\U0001F9CD\U0000200D\U00002642':{'en':':man_standing:','status':minimally_qualified,'E':12,'alias':[':standing_man:'],'de':':stehender_mann:','es':':hombre_de_pie:','fr':':homme_debout:','ja':':立つ男:','ko':':서_있는_남자:','pt':':homem_em_pé:','it':':uomo_in_piedi:','fa':':مرد_ایستاده:','id':':pria_berdiri:','zh':':站立的男人:','ru':':мужчина_стоит:','tr':':ayakta_duran_erkek:','ar':':رجل_واقف:'},'\U0001F9CD\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_standing_dark_skin_tone:','status':fully_qualified,'E':12,'de':':stehender_mann_dunkle_hautfarbe:','es':':hombre_de_pie_tono_de_piel_oscuro:','fr':':homme_debout_peau_foncée:','ja':':立つ男_濃い肌色:','ko':':서_있는_남자_검은색_피부:','pt':':homem_em_pé_pele_escura:','it':':uomo_in_piedi_carnagione_scura:','fa':':مرد_ایستاده_پوست_آبنوسی:','id':':pria_berdiri_warna_kulit_gelap:','zh':':站立的男人_较深肤色:','ru':':мужчина_стоит_очень_темныи_тон_кожи:','tr':':ayakta_duran_erkek_koyu_cilt_tonu:','ar':':رجل_واقف_بشرة_بلون_غامق:'},'\U0001F9CD\U0001F3FF\U0000200D\U00002642':{'en':':man_standing_dark_skin_tone:','status':minimally_qualified,'E':12,'de':':stehender_mann_dunkle_hautfarbe:','es':':hombre_de_pie_tono_de_piel_oscuro:','fr':':homme_debout_peau_foncée:','ja':':立つ男_濃い肌色:','ko':':서_있는_남자_검은색_피부:','pt':':homem_em_pé_pele_escura:','it':':uomo_in_piedi_carnagione_scura:','fa':':مرد_ایستاده_پوست_آبنوسی:','id':':pria_berdiri_warna_kulit_gelap:','zh':':站立的男人_较深肤色:','ru':':мужчина_стоит_очень_темный_тон_кожи:','tr':':ayakta_duran_erkek_koyu_cilt_tonu:','ar':':رجل_واقف_بشرة_بلون_غامق:'},'\U0001F9CD\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_standing_light_skin_tone:','status':fully_qualified,'E':12,'de':':stehender_mann_helle_hautfarbe:','es':':hombre_de_pie_tono_de_piel_claro:','fr':':homme_debout_peau_claire:','ja':':立つ男_薄い肌色:','ko':':서_있는_남자_하얀_피부:','pt':':homem_em_pé_pele_clara:','it':':uomo_in_piedi_carnagione_chiara:','fa':':مرد_ایستاده_پوست_سفید:','id':':pria_berdiri_warna_kulit_cerah:','zh':':站立的男人_较浅肤色:','ru':':мужчина_стоит_очень_светлыи_тон_кожи:','tr':':ayakta_duran_erkek_açık_cilt_tonu:','ar':':رجل_واقف_بشرة_بلون_فاتح:'},'\U0001F9CD\U0001F3FB\U0000200D\U00002642':{'en':':man_standing_light_skin_tone:','status':minimally_qualified,'E':12,'de':':stehender_mann_helle_hautfarbe:','es':':hombre_de_pie_tono_de_piel_claro:','fr':':homme_debout_peau_claire:','ja':':立つ男_薄い肌色:','ko':':서_있는_남자_하얀_피부:','pt':':homem_em_pé_pele_clara:','it':':uomo_in_piedi_carnagione_chiara:','fa':':مرد_ایستاده_پوست_سفید:','id':':pria_berdiri_warna_kulit_cerah:','zh':':站立的男人_较浅肤色:','ru':':мужчина_стоит_очень_светлый_тон_кожи:','tr':':ayakta_duran_erkek_açık_cilt_tonu:','ar':':رجل_واقف_بشرة_بلون_فاتح:'},'\U0001F9CD\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_standing_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':stehender_mann_mitteldunkle_hautfarbe:','es':':hombre_de_pie_tono_de_piel_oscuro_medio:','fr':':homme_debout_peau_mate:','ja':':立つ男_やや濃い肌色:','ko':':서_있는_남자_진한_갈색_피부:','pt':':homem_em_pé_pele_morena_escura:','it':':uomo_in_piedi_carnagione_abbastanza_scura:','fa':':مرد_ایستاده_پوست_گندمی:','id':':pria_berdiri_warna_kulit_gelap-sedang:','zh':':站立的男人_中等深肤色:','ru':':мужчина_стоит_темныи_тон_кожи:','tr':':ayakta_duran_erkek_orta_koyu_cilt_tonu:','ar':':رجل_واقف_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9CD\U0001F3FE\U0000200D\U00002642':{'en':':man_standing_medium-dark_skin_tone:','status':minimally_qualified,'E':12,'de':':stehender_mann_mitteldunkle_hautfarbe:','es':':hombre_de_pie_tono_de_piel_oscuro_medio:','fr':':homme_debout_peau_mate:','ja':':立つ男_やや濃い肌色:','ko':':서_있는_남자_진한_갈색_피부:','pt':':homem_em_pé_pele_morena_escura:','it':':uomo_in_piedi_carnagione_abbastanza_scura:','fa':':مرد_ایستاده_پوست_گندمی:','id':':pria_berdiri_warna_kulit_gelap-sedang:','zh':':站立的男人_中等深肤色:','ru':':мужчина_стоит_темный_тон_кожи:','tr':':ayakta_duran_erkek_orta_koyu_cilt_tonu:','ar':':رجل_واقف_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9CD\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_standing_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':stehender_mann_mittelhelle_hautfarbe:','es':':hombre_de_pie_tono_de_piel_claro_medio:','fr':':homme_debout_peau_moyennement_claire:','ja':':立つ男_やや薄い肌色:','ko':':서_있는_남자_연한_갈색_피부:','pt':':homem_em_pé_pele_morena_clara:','it':':uomo_in_piedi_carnagione_abbastanza_chiara:','fa':':مرد_ایستاده_پوست_روشن:','id':':pria_berdiri_warna_kulit_cerah-sedang:','zh':':站立的男人_中等浅肤色:','ru':':мужчина_стоит_светлыи_тон_кожи:','tr':':ayakta_duran_erkek_orta_açık_cilt_tonu:','ar':':رجل_واقف_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9CD\U0001F3FC\U0000200D\U00002642':{'en':':man_standing_medium-light_skin_tone:','status':minimally_qualified,'E':12,'de':':stehender_mann_mittelhelle_hautfarbe:','es':':hombre_de_pie_tono_de_piel_claro_medio:','fr':':homme_debout_peau_moyennement_claire:','ja':':立つ男_やや薄い肌色:','ko':':서_있는_남자_연한_갈색_피부:','pt':':homem_em_pé_pele_morena_clara:','it':':uomo_in_piedi_carnagione_abbastanza_chiara:','fa':':مرد_ایستاده_پوست_روشن:','id':':pria_berdiri_warna_kulit_cerah-sedang:','zh':':站立的男人_中等浅肤色:','ru':':мужчина_стоит_светлый_тон_кожи:','tr':':ayakta_duran_erkek_orta_açık_cilt_tonu:','ar':':رجل_واقف_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9CD\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_standing_medium_skin_tone:','status':fully_qualified,'E':12,'de':':stehender_mann_mittlere_hautfarbe:','es':':hombre_de_pie_tono_de_piel_medio:','fr':':homme_debout_peau_légèrement_mate:','ja':':立つ男_中間の肌色:','ko':':서_있는_남자_갈색_피부:','pt':':homem_em_pé_pele_morena:','it':':uomo_in_piedi_carnagione_olivastra:','fa':':مرد_ایستاده_پوست_طلایی:','id':':pria_berdiri_warna_kulit_sedang:','zh':':站立的男人_中等肤色:','ru':':мужчина_стоит_среднии_тон_кожи:','tr':':ayakta_duran_erkek_orta_cilt_tonu:','ar':':رجل_واقف_بشرة_بلون_معتدل:'},'\U0001F9CD\U0001F3FD\U0000200D\U00002642':{'en':':man_standing_medium_skin_tone:','status':minimally_qualified,'E':12,'de':':stehender_mann_mittlere_hautfarbe:','es':':hombre_de_pie_tono_de_piel_medio:','fr':':homme_debout_peau_légèrement_mate:','ja':':立つ男_中間の肌色:','ko':':서_있는_남자_갈색_피부:','pt':':homem_em_pé_pele_morena:','it':':uomo_in_piedi_carnagione_olivastra:','fa':':مرد_ایستاده_پوست_طلایی:','id':':pria_berdiri_warna_kulit_sedang:','zh':':站立的男人_中等肤色:','ru':':мужчина_стоит_средний_тон_кожи:','tr':':ayakta_duran_erkek_orta_cilt_tonu:','ar':':رجل_واقف_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U0001F393':{'en':':man_student:','status':fully_qualified,'E':4,'de':':student:','es':':estudiante_hombre:','fr':':étudiant:','ja':':男子学生:','ko':':남학생:','pt':':estudante:','it':':studente_maschio:','fa':':دانشجوی_مرد:','id':':sarjana_pria:','zh':':男学生:','ru':':студент:','tr':':erkek_öğrenci:','ar':':طالب:'},'\U0001F468\U0001F3FF\U0000200D\U0001F393':{'en':':man_student_dark_skin_tone:','status':fully_qualified,'E':4,'de':':student_dunkle_hautfarbe:','es':':estudiante_hombre_tono_de_piel_oscuro:','fr':':étudiant_peau_foncée:','ja':':男子学生_濃い肌色:','ko':':남학생_검은색_피부:','pt':':estudante_pele_escura:','it':':studente_maschio_carnagione_scura:','fa':':دانشجوی_مرد_پوست_آبنوسی:','id':':sarjana_pria_warna_kulit_gelap:','zh':':男学生_较深肤色:','ru':':студент_очень_темный_тон_кожи:','tr':':erkek_öğrenci_koyu_cilt_tonu:','ar':':طالب_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U0001F393':{'en':':man_student_light_skin_tone:','status':fully_qualified,'E':4,'de':':student_helle_hautfarbe:','es':':estudiante_hombre_tono_de_piel_claro:','fr':':étudiant_peau_claire:','ja':':男子学生_薄い肌色:','ko':':남학생_하얀_피부:','pt':':estudante_pele_clara:','it':':studente_maschio_carnagione_chiara:','fa':':دانشجوی_مرد_پوست_سفید:','id':':sarjana_pria_warna_kulit_cerah:','zh':':男学生_较浅肤色:','ru':':студент_очень_светлый_тон_кожи:','tr':':erkek_öğrenci_açık_cilt_tonu:','ar':':طالب_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F393':{'en':':man_student_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':student_mitteldunkle_hautfarbe:','es':':estudiante_hombre_tono_de_piel_oscuro_medio:','fr':':étudiant_peau_mate:','ja':':男子学生_やや濃い肌色:','ko':':남학생_진한_갈색_피부:','pt':':estudante_pele_morena_escura:','it':':studente_maschio_carnagione_abbastanza_scura:','fa':':دانشجوی_مرد_پوست_گندمی:','id':':sarjana_pria_warna_kulit_gelap-sedang:','zh':':男学生_中等深肤色:','ru':':студент_темный_тон_кожи:','tr':':erkek_öğrenci_orta_koyu_cilt_tonu:','ar':':طالب_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U0001F393':{'en':':man_student_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':student_mittelhelle_hautfarbe:','es':':estudiante_hombre_tono_de_piel_claro_medio:','fr':':étudiant_peau_moyennement_claire:','ja':':男子学生_やや薄い肌色:','ko':':남학생_연한_갈색_피부:','pt':':estudante_pele_morena_clara:','it':':studente_maschio_carnagione_abbastanza_chiara:','fa':':دانشجوی_مرد_پوست_روشن:','id':':sarjana_pria_warna_kulit_cerah-sedang:','zh':':男学生_中等浅肤色:','ru':':студент_светлый_тон_кожи:','tr':':erkek_öğrenci_orta_açık_cilt_tonu:','ar':':طالب_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F393':{'en':':man_student_medium_skin_tone:','status':fully_qualified,'E':4,'de':':student_mittlere_hautfarbe:','es':':estudiante_hombre_tono_de_piel_medio:','fr':':étudiant_peau_légèrement_mate:','ja':':男子学生_中間の肌色:','ko':':남학생_갈색_피부:','pt':':estudante_pele_morena:','it':':studente_maschio_carnagione_olivastra:','fa':':دانشجوی_مرد_پوست_طلایی:','id':':sarjana_pria_warna_kulit_sedang:','zh':':男学生_中等肤色:','ru':':студент_средний_тон_кожи:','tr':':erkek_öğrenci_orta_cilt_tonu:','ar':':طالب_بشرة_بلون_معتدل:'},'\U0001F9B8\U0000200D\U00002642\U0000FE0F':{'en':':man_superhero:','status':fully_qualified,'E':11,'alias':[':superhero_man:'],'de':':superheld:','es':':superhéroe:','fr':':super-héros_homme:','ja':':男性のスーパーヒーロー:','ko':':남자_히어로:','pt':':homem_super-herói:','it':':supereroe_uomo:','fa':':ابرقهرمان_مرد:','id':':pahlawan_super_pria:','zh':':男超级英雄:','ru':':мужчина-супергерои:','tr':':erkek_süper_kahraman:','ar':':بطل_خارق_رجل:'},'\U0001F9B8\U0000200D\U00002642':{'en':':man_superhero:','status':minimally_qualified,'E':11,'alias':[':superhero_man:'],'de':':superheld:','es':':superhéroe:','fr':':super-héros_homme:','ja':':男性のスーパーヒーロー:','ko':':남자_히어로:','pt':':homem_super-herói:','it':':supereroe_uomo:','fa':':ابرقهرمان_مرد:','id':':pahlawan_super_pria:','zh':':男超级英雄:','ru':':мужчина_супергерой:','tr':':erkek_süper_kahraman:','ar':':بطل_خارق_رجل:'},'\U0001F9B8\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_superhero_dark_skin_tone:','status':fully_qualified,'E':11,'de':':superheld_dunkle_hautfarbe:','es':':superhéroe_tono_de_piel_oscuro:','fr':':super-héros_homme_peau_foncée:','ja':':男性のスーパーヒーロー_濃い肌色:','ko':':남자_히어로_검은색_피부:','pt':':homem_super-herói_pele_escura:','it':':supereroe_uomo_carnagione_scura:','fa':':ابرقهرمان_مرد_پوست_آبنوسی:','id':':pahlawan_super_pria_warna_kulit_gelap:','zh':':男超级英雄_较深肤色:','ru':':мужчина-супергерои_очень_темныи_тон_кожи:','tr':':erkek_süper_kahraman_koyu_cilt_tonu:','ar':':بطل_خارق_رجل_بشرة_بلون_غامق:'},'\U0001F9B8\U0001F3FF\U0000200D\U00002642':{'en':':man_superhero_dark_skin_tone:','status':minimally_qualified,'E':11,'de':':superheld_dunkle_hautfarbe:','es':':superhéroe_tono_de_piel_oscuro:','fr':':super-héros_homme_peau_foncée:','ja':':男性のスーパーヒーロー_濃い肌色:','ko':':남자_히어로_검은색_피부:','pt':':homem_super-herói_pele_escura:','it':':supereroe_uomo_carnagione_scura:','fa':':ابرقهرمان_مرد_پوست_آبنوسی:','id':':pahlawan_super_pria_warna_kulit_gelap:','zh':':男超级英雄_较深肤色:','ru':':мужчина_супергерой_очень_темный_тон_кожи:','tr':':erkek_süper_kahraman_koyu_cilt_tonu:','ar':':بطل_خارق_رجل_بشرة_بلون_غامق:'},'\U0001F9B8\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_superhero_light_skin_tone:','status':fully_qualified,'E':11,'de':':superheld_helle_hautfarbe:','es':':superhéroe_tono_de_piel_claro:','fr':':super-héros_homme_peau_claire:','ja':':男性のスーパーヒーロー_薄い肌色:','ko':':남자_히어로_하얀_피부:','pt':':homem_super-herói_pele_clara:','it':':supereroe_uomo_carnagione_chiara:','fa':':ابرقهرمان_مرد_پوست_سفید:','id':':pahlawan_super_pria_warna_kulit_cerah:','zh':':男超级英雄_较浅肤色:','ru':':мужчина-супергерои_очень_светлыи_тон_кожи:','tr':':erkek_süper_kahraman_açık_cilt_tonu:','ar':':بطل_خارق_رجل_بشرة_بلون_فاتح:'},'\U0001F9B8\U0001F3FB\U0000200D\U00002642':{'en':':man_superhero_light_skin_tone:','status':minimally_qualified,'E':11,'de':':superheld_helle_hautfarbe:','es':':superhéroe_tono_de_piel_claro:','fr':':super-héros_homme_peau_claire:','ja':':男性のスーパーヒーロー_薄い肌色:','ko':':남자_히어로_하얀_피부:','pt':':homem_super-herói_pele_clara:','it':':supereroe_uomo_carnagione_chiara:','fa':':ابرقهرمان_مرد_پوست_سفید:','id':':pahlawan_super_pria_warna_kulit_cerah:','zh':':男超级英雄_较浅肤色:','ru':':мужчина_супергерой_очень_светлый_тон_кожи:','tr':':erkek_süper_kahraman_açık_cilt_tonu:','ar':':بطل_خارق_رجل_بشرة_بلون_فاتح:'},'\U0001F9B8\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_superhero_medium-dark_skin_tone:','status':fully_qualified,'E':11,'de':':superheld_mitteldunkle_hautfarbe:','es':':superhéroe_tono_de_piel_oscuro_medio:','fr':':super-héros_homme_peau_mate:','ja':':男性のスーパーヒーロー_やや濃い肌色:','ko':':남자_히어로_진한_갈색_피부:','pt':':homem_super-herói_pele_morena_escura:','it':':supereroe_uomo_carnagione_abbastanza_scura:','fa':':ابرقهرمان_مرد_پوست_گندمی:','id':':pahlawan_super_pria_warna_kulit_gelap-sedang:','zh':':男超级英雄_中等深肤色:','ru':':мужчина-супергерои_темныи_тон_кожи:','tr':':erkek_süper_kahraman_orta_koyu_cilt_tonu:','ar':':بطل_خارق_رجل_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9B8\U0001F3FE\U0000200D\U00002642':{'en':':man_superhero_medium-dark_skin_tone:','status':minimally_qualified,'E':11,'de':':superheld_mitteldunkle_hautfarbe:','es':':superhéroe_tono_de_piel_oscuro_medio:','fr':':super-héros_homme_peau_mate:','ja':':男性のスーパーヒーロー_やや濃い肌色:','ko':':남자_히어로_진한_갈색_피부:','pt':':homem_super-herói_pele_morena_escura:','it':':supereroe_uomo_carnagione_abbastanza_scura:','fa':':ابرقهرمان_مرد_پوست_گندمی:','id':':pahlawan_super_pria_warna_kulit_gelap-sedang:','zh':':男超级英雄_中等深肤色:','ru':':мужчина_супергерой_темный_тон_кожи:','tr':':erkek_süper_kahraman_orta_koyu_cilt_tonu:','ar':':بطل_خارق_رجل_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9B8\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_superhero_medium-light_skin_tone:','status':fully_qualified,'E':11,'de':':superheld_mittelhelle_hautfarbe:','es':':superhéroe_tono_de_piel_claro_medio:','fr':':super-héros_homme_peau_moyennement_claire:','ja':':男性のスーパーヒーロー_やや薄い肌色:','ko':':남자_히어로_연한_갈색_피부:','pt':':homem_super-herói_pele_morena_clara:','it':':supereroe_uomo_carnagione_abbastanza_chiara:','fa':':ابرقهرمان_مرد_پوست_روشن:','id':':pahlawan_super_pria_warna_kulit_cerah-sedang:','zh':':男超级英雄_中等浅肤色:','ru':':мужчина-супергерои_светлыи_тон_кожи:','tr':':erkek_süper_kahraman_orta_açık_cilt_tonu:','ar':':بطل_خارق_رجل_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9B8\U0001F3FC\U0000200D\U00002642':{'en':':man_superhero_medium-light_skin_tone:','status':minimally_qualified,'E':11,'de':':superheld_mittelhelle_hautfarbe:','es':':superhéroe_tono_de_piel_claro_medio:','fr':':super-héros_homme_peau_moyennement_claire:','ja':':男性のスーパーヒーロー_やや薄い肌色:','ko':':남자_히어로_연한_갈색_피부:','pt':':homem_super-herói_pele_morena_clara:','it':':supereroe_uomo_carnagione_abbastanza_chiara:','fa':':ابرقهرمان_مرد_پوست_روشن:','id':':pahlawan_super_pria_warna_kulit_cerah-sedang:','zh':':男超级英雄_中等浅肤色:','ru':':мужчина_супергерой_светлый_тон_кожи:','tr':':erkek_süper_kahraman_orta_açık_cilt_tonu:','ar':':بطل_خارق_رجل_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9B8\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_superhero_medium_skin_tone:','status':fully_qualified,'E':11,'de':':superheld_mittlere_hautfarbe:','es':':superhéroe_tono_de_piel_medio:','fr':':super-héros_homme_peau_légèrement_mate:','ja':':男性のスーパーヒーロー_中間の肌色:','ko':':남자_히어로_갈색_피부:','pt':':homem_super-herói_pele_morena:','it':':supereroe_uomo_carnagione_olivastra:','fa':':ابرقهرمان_مرد_پوست_طلایی:','id':':pahlawan_super_pria_warna_kulit_sedang:','zh':':男超级英雄_中等肤色:','ru':':мужчина-супергерои_среднии_тон_кожи:','tr':':erkek_süper_kahraman_orta_cilt_tonu:','ar':':بطل_خارق_رجل_بشرة_بلون_معتدل:'},'\U0001F9B8\U0001F3FD\U0000200D\U00002642':{'en':':man_superhero_medium_skin_tone:','status':minimally_qualified,'E':11,'de':':superheld_mittlere_hautfarbe:','es':':superhéroe_tono_de_piel_medio:','fr':':super-héros_homme_peau_légèrement_mate:','ja':':男性のスーパーヒーロー_中間の肌色:','ko':':남자_히어로_갈색_피부:','pt':':homem_super-herói_pele_morena:','it':':supereroe_uomo_carnagione_olivastra:','fa':':ابرقهرمان_مرد_پوست_طلایی:','id':':pahlawan_super_pria_warna_kulit_sedang:','zh':':男超级英雄_中等肤色:','ru':':мужчина_супергерой_средний_тон_кожи:','tr':':erkek_süper_kahraman_orta_cilt_tonu:','ar':':بطل_خارق_رجل_بشرة_بلون_معتدل:'},'\U0001F9B9\U0000200D\U00002642\U0000FE0F':{'en':':man_supervillain:','status':fully_qualified,'E':11,'alias':[':supervillain_man:'],'de':':männlicher_bösewicht:','es':':supervillano:','fr':':super-vilain_homme:','ja':':男性の悪役:','ko':':남자_슈퍼_악당:','pt':':homem_supervilão:','it':':supercattivo_uomo:','fa':':ابرشرور_مرد:','id':':penjahat_super_pria:','zh':':男超级大坏蛋:','ru':':мужчина-суперзлодеи:','tr':':erkek_süper_kötü:','ar':':شرير_خارق_رجل:'},'\U0001F9B9\U0000200D\U00002642':{'en':':man_supervillain:','status':minimally_qualified,'E':11,'alias':[':supervillain_man:'],'de':':männlicher_bösewicht:','es':':supervillano:','fr':':super-vilain_homme:','ja':':男性の悪役:','ko':':남자_슈퍼_악당:','pt':':homem_supervilão:','it':':supercattivo_uomo:','fa':':ابرشرور_مرد:','id':':penjahat_super_pria:','zh':':男超级大坏蛋:','ru':':мужчина_суперзлодей:','tr':':erkek_süper_kötü:','ar':':شرير_خارق_رجل:'},'\U0001F9B9\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_supervillain_dark_skin_tone:','status':fully_qualified,'E':11,'de':':männlicher_bösewicht_dunkle_hautfarbe:','es':':supervillano_tono_de_piel_oscuro:','fr':':super-vilain_homme_peau_foncée:','ja':':男性の悪役_濃い肌色:','ko':':남자_슈퍼_악당_검은색_피부:','pt':':homem_supervilão_pele_escura:','it':':supercattivo_uomo_carnagione_scura:','fa':':ابرشرور_مرد_پوست_آبنوسی:','id':':penjahat_super_pria_warna_kulit_gelap:','zh':':男超级大坏蛋_较深肤色:','ru':':мужчина-суперзлодеи_очень_темныи_тон_кожи:','tr':':erkek_süper_kötü_koyu_cilt_tonu:','ar':':شرير_خارق_رجل_بشرة_بلون_غامق:'},'\U0001F9B9\U0001F3FF\U0000200D\U00002642':{'en':':man_supervillain_dark_skin_tone:','status':minimally_qualified,'E':11,'de':':männlicher_bösewicht_dunkle_hautfarbe:','es':':supervillano_tono_de_piel_oscuro:','fr':':super-vilain_homme_peau_foncée:','ja':':男性の悪役_濃い肌色:','ko':':남자_슈퍼_악당_검은색_피부:','pt':':homem_supervilão_pele_escura:','it':':supercattivo_uomo_carnagione_scura:','fa':':ابرشرور_مرد_پوست_آبنوسی:','id':':penjahat_super_pria_warna_kulit_gelap:','zh':':男超级大坏蛋_较深肤色:','ru':':мужчина_суперзлодей_очень_темный_тон_кожи:','tr':':erkek_süper_kötü_koyu_cilt_tonu:','ar':':شرير_خارق_رجل_بشرة_بلون_غامق:'},'\U0001F9B9\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_supervillain_light_skin_tone:','status':fully_qualified,'E':11,'de':':männlicher_bösewicht_helle_hautfarbe:','es':':supervillano_tono_de_piel_claro:','fr':':super-vilain_homme_peau_claire:','ja':':男性の悪役_薄い肌色:','ko':':남자_슈퍼_악당_하얀_피부:','pt':':homem_supervilão_pele_clara:','it':':supercattivo_uomo_carnagione_chiara:','fa':':ابرشرور_مرد_پوست_سفید:','id':':penjahat_super_pria_warna_kulit_cerah:','zh':':男超级大坏蛋_较浅肤色:','ru':':мужчина-суперзлодеи_очень_светлыи_тон_кожи:','tr':':erkek_süper_kötü_açık_cilt_tonu:','ar':':شرير_خارق_رجل_بشرة_بلون_فاتح:'},'\U0001F9B9\U0001F3FB\U0000200D\U00002642':{'en':':man_supervillain_light_skin_tone:','status':minimally_qualified,'E':11,'de':':männlicher_bösewicht_helle_hautfarbe:','es':':supervillano_tono_de_piel_claro:','fr':':super-vilain_homme_peau_claire:','ja':':男性の悪役_薄い肌色:','ko':':남자_슈퍼_악당_하얀_피부:','pt':':homem_supervilão_pele_clara:','it':':supercattivo_uomo_carnagione_chiara:','fa':':ابرشرور_مرد_پوست_سفید:','id':':penjahat_super_pria_warna_kulit_cerah:','zh':':男超级大坏蛋_较浅肤色:','ru':':мужчина_суперзлодей_очень_светлый_тон_кожи:','tr':':erkek_süper_kötü_açık_cilt_tonu:','ar':':شرير_خارق_رجل_بشرة_بلون_فاتح:'},'\U0001F9B9\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_supervillain_medium-dark_skin_tone:','status':fully_qualified,'E':11,'de':':männlicher_bösewicht_mitteldunkle_hautfarbe:','es':':supervillano_tono_de_piel_oscuro_medio:','fr':':super-vilain_homme_peau_mate:','ja':':男性の悪役_やや濃い肌色:','ko':':남자_슈퍼_악당_진한_갈색_피부:','pt':':homem_supervilão_pele_morena_escura:','it':':supercattivo_uomo_carnagione_abbastanza_scura:','fa':':ابرشرور_مرد_پوست_گندمی:','id':':penjahat_super_pria_warna_kulit_gelap-sedang:','zh':':男超级大坏蛋_中等深肤色:','ru':':мужчина-суперзлодеи_темныи_тон_кожи:','tr':':erkek_süper_kötü_orta_koyu_cilt_tonu:','ar':':شرير_خارق_رجل_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9B9\U0001F3FE\U0000200D\U00002642':{'en':':man_supervillain_medium-dark_skin_tone:','status':minimally_qualified,'E':11,'de':':männlicher_bösewicht_mitteldunkle_hautfarbe:','es':':supervillano_tono_de_piel_oscuro_medio:','fr':':super-vilain_homme_peau_mate:','ja':':男性の悪役_やや濃い肌色:','ko':':남자_슈퍼_악당_진한_갈색_피부:','pt':':homem_supervilão_pele_morena_escura:','it':':supercattivo_uomo_carnagione_abbastanza_scura:','fa':':ابرشرور_مرد_پوست_گندمی:','id':':penjahat_super_pria_warna_kulit_gelap-sedang:','zh':':男超级大坏蛋_中等深肤色:','ru':':мужчина_суперзлодей_темный_тон_кожи:','tr':':erkek_süper_kötü_orta_koyu_cilt_tonu:','ar':':شرير_خارق_رجل_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9B9\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_supervillain_medium-light_skin_tone:','status':fully_qualified,'E':11,'de':':männlicher_bösewicht_mittelhelle_hautfarbe:','es':':supervillano_tono_de_piel_claro_medio:','fr':':super-vilain_homme_peau_moyennement_claire:','ja':':男性の悪役_やや薄い肌色:','ko':':남자_슈퍼_악당_연한_갈색_피부:','pt':':homem_supervilão_pele_morena_clara:','it':':supercattivo_uomo_carnagione_abbastanza_chiara:','fa':':ابرشرور_مرد_پوست_روشن:','id':':penjahat_super_pria_warna_kulit_cerah-sedang:','zh':':男超级大坏蛋_中等浅肤色:','ru':':мужчина-суперзлодеи_светлыи_тон_кожи:','tr':':erkek_süper_kötü_orta_açık_cilt_tonu:','ar':':شرير_خارق_رجل_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9B9\U0001F3FC\U0000200D\U00002642':{'en':':man_supervillain_medium-light_skin_tone:','status':minimally_qualified,'E':11,'de':':männlicher_bösewicht_mittelhelle_hautfarbe:','es':':supervillano_tono_de_piel_claro_medio:','fr':':super-vilain_homme_peau_moyennement_claire:','ja':':男性の悪役_やや薄い肌色:','ko':':남자_슈퍼_악당_연한_갈색_피부:','pt':':homem_supervilão_pele_morena_clara:','it':':supercattivo_uomo_carnagione_abbastanza_chiara:','fa':':ابرشرور_مرد_پوست_روشن:','id':':penjahat_super_pria_warna_kulit_cerah-sedang:','zh':':男超级大坏蛋_中等浅肤色:','ru':':мужчина_суперзлодей_светлый_тон_кожи:','tr':':erkek_süper_kötü_orta_açık_cilt_tonu:','ar':':شرير_خارق_رجل_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9B9\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_supervillain_medium_skin_tone:','status':fully_qualified,'E':11,'de':':männlicher_bösewicht_mittlere_hautfarbe:','es':':supervillano_tono_de_piel_medio:','fr':':super-vilain_homme_peau_légèrement_mate:','ja':':男性の悪役_中間の肌色:','ko':':남자_슈퍼_악당_갈색_피부:','pt':':homem_supervilão_pele_morena:','it':':supercattivo_uomo_carnagione_olivastra:','fa':':ابرشرور_مرد_پوست_طلایی:','id':':penjahat_super_pria_warna_kulit_sedang:','zh':':男超级大坏蛋_中等肤色:','ru':':мужчина-суперзлодеи_среднии_тон_кожи:','tr':':erkek_süper_kötü_orta_cilt_tonu:','ar':':شرير_خارق_رجل_بشرة_بلون_معتدل:'},'\U0001F9B9\U0001F3FD\U0000200D\U00002642':{'en':':man_supervillain_medium_skin_tone:','status':minimally_qualified,'E':11,'de':':männlicher_bösewicht_mittlere_hautfarbe:','es':':supervillano_tono_de_piel_medio:','fr':':super-vilain_homme_peau_légèrement_mate:','ja':':男性の悪役_中間の肌色:','ko':':남자_슈퍼_악당_갈색_피부:','pt':':homem_supervilão_pele_morena:','it':':supercattivo_uomo_carnagione_olivastra:','fa':':ابرشرور_مرد_پوست_طلایی:','id':':penjahat_super_pria_warna_kulit_sedang:','zh':':男超级大坏蛋_中等肤色:','ru':':мужчина_суперзлодей_средний_тон_кожи:','tr':':erkek_süper_kötü_orta_cilt_tonu:','ar':':شرير_خارق_رجل_بشرة_بلون_معتدل:'},'\U0001F3C4\U0000200D\U00002642\U0000FE0F':{'en':':man_surfing:','status':fully_qualified,'E':4,'alias':[':surfing_man:'],'de':':surfer:','es':':hombre_haciendo_surf:','fr':':surfeur:','ja':':サーフィンする男:','ko':':서핑하는_남자:','pt':':homem_surfista:','it':':surfista_uomo:','fa':':موج_سوار_مرد:','id':':pria_berselancar:','zh':':男生冲浪:','ru':':серфингист:','tr':':sörf_yapan_erkek:','ar':':راكب_أمواج:'},'\U0001F3C4\U0000200D\U00002642':{'en':':man_surfing:','status':minimally_qualified,'E':4,'alias':[':surfing_man:'],'de':':surfer:','es':':hombre_haciendo_surf:','fr':':surfeur:','ja':':サーフィンする男:','ko':':서핑하는_남자:','pt':':homem_surfista:','it':':surfista_uomo:','fa':':موج_سوار_مرد:','id':':pria_berselancar:','zh':':男生冲浪:','ru':':серфингист:','tr':':sörf_yapan_erkek:','ar':':راكب_أمواج:'},'\U0001F3C4\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_surfing_dark_skin_tone:','status':fully_qualified,'E':4,'de':':surfer_dunkle_hautfarbe:','es':':hombre_haciendo_surf_tono_de_piel_oscuro:','fr':':surfeur_peau_foncée:','ja':':サーフィンする男_濃い肌色:','ko':':서핑하는_남자_검은색_피부:','pt':':homem_surfista_pele_escura:','it':':surfista_uomo_carnagione_scura:','fa':':موج_سوار_مرد_پوست_آبنوسی:','id':':pria_berselancar_warna_kulit_gelap:','zh':':男生冲浪_较深肤色:','ru':':серфингист_очень_темныи_тон_кожи:','tr':':sörf_yapan_erkek_koyu_cilt_tonu:','ar':':راكب_أمواج_بشرة_بلون_غامق:'},'\U0001F3C4\U0001F3FF\U0000200D\U00002642':{'en':':man_surfing_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':surfer_dunkle_hautfarbe:','es':':hombre_haciendo_surf_tono_de_piel_oscuro:','fr':':surfeur_peau_foncée:','ja':':サーフィンする男_濃い肌色:','ko':':서핑하는_남자_검은색_피부:','pt':':homem_surfista_pele_escura:','it':':surfista_uomo_carnagione_scura:','fa':':موج_سوار_مرد_پوست_آبنوسی:','id':':pria_berselancar_warna_kulit_gelap:','zh':':男生冲浪_较深肤色:','ru':':серфингист_очень_темный_тон_кожи:','tr':':sörf_yapan_erkek_koyu_cilt_tonu:','ar':':راكب_أمواج_بشرة_بلون_غامق:'},'\U0001F3C4\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_surfing_light_skin_tone:','status':fully_qualified,'E':4,'de':':surfer_helle_hautfarbe:','es':':hombre_haciendo_surf_tono_de_piel_claro:','fr':':surfeur_peau_claire:','ja':':サーフィンする男_薄い肌色:','ko':':서핑하는_남자_하얀_피부:','pt':':homem_surfista_pele_clara:','it':':surfista_uomo_carnagione_chiara:','fa':':موج_سوار_مرد_پوست_سفید:','id':':pria_berselancar_warna_kulit_cerah:','zh':':男生冲浪_较浅肤色:','ru':':серфингист_очень_светлыи_тон_кожи:','tr':':sörf_yapan_erkek_açık_cilt_tonu:','ar':':راكب_أمواج_بشرة_بلون_فاتح:'},'\U0001F3C4\U0001F3FB\U0000200D\U00002642':{'en':':man_surfing_light_skin_tone:','status':minimally_qualified,'E':4,'de':':surfer_helle_hautfarbe:','es':':hombre_haciendo_surf_tono_de_piel_claro:','fr':':surfeur_peau_claire:','ja':':サーフィンする男_薄い肌色:','ko':':서핑하는_남자_하얀_피부:','pt':':homem_surfista_pele_clara:','it':':surfista_uomo_carnagione_chiara:','fa':':موج_سوار_مرد_پوست_سفید:','id':':pria_berselancar_warna_kulit_cerah:','zh':':男生冲浪_较浅肤色:','ru':':серфингист_очень_светлый_тон_кожи:','tr':':sörf_yapan_erkek_açık_cilt_tonu:','ar':':راكب_أمواج_بشرة_بلون_فاتح:'},'\U0001F3C4\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_surfing_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':surfer_mitteldunkle_hautfarbe:','es':':hombre_haciendo_surf_tono_de_piel_oscuro_medio:','fr':':surfeur_peau_mate:','ja':':サーフィンする男_やや濃い肌色:','ko':':서핑하는_남자_진한_갈색_피부:','pt':':homem_surfista_pele_morena_escura:','it':':surfista_uomo_carnagione_abbastanza_scura:','fa':':موج_سوار_مرد_پوست_گندمی:','id':':pria_berselancar_warna_kulit_gelap-sedang:','zh':':男生冲浪_中等深肤色:','ru':':серфингист_темныи_тон_кожи:','tr':':sörf_yapan_erkek_orta_koyu_cilt_tonu:','ar':':راكب_أمواج_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3C4\U0001F3FE\U0000200D\U00002642':{'en':':man_surfing_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':surfer_mitteldunkle_hautfarbe:','es':':hombre_haciendo_surf_tono_de_piel_oscuro_medio:','fr':':surfeur_peau_mate:','ja':':サーフィンする男_やや濃い肌色:','ko':':서핑하는_남자_진한_갈색_피부:','pt':':homem_surfista_pele_morena_escura:','it':':surfista_uomo_carnagione_abbastanza_scura:','fa':':موج_سوار_مرد_پوست_گندمی:','id':':pria_berselancar_warna_kulit_gelap-sedang:','zh':':男生冲浪_中等深肤色:','ru':':серфингист_темный_тон_кожи:','tr':':sörf_yapan_erkek_orta_koyu_cilt_tonu:','ar':':راكب_أمواج_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3C4\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_surfing_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':surfer_mittelhelle_hautfarbe:','es':':hombre_haciendo_surf_tono_de_piel_claro_medio:','fr':':surfeur_peau_moyennement_claire:','ja':':サーフィンする男_やや薄い肌色:','ko':':서핑하는_남자_연한_갈색_피부:','pt':':homem_surfista_pele_morena_clara:','it':':surfista_uomo_carnagione_abbastanza_chiara:','fa':':موج_سوار_مرد_پوست_روشن:','id':':pria_berselancar_warna_kulit_cerah-sedang:','zh':':男生冲浪_中等浅肤色:','ru':':серфингист_светлыи_тон_кожи:','tr':':sörf_yapan_erkek_orta_açık_cilt_tonu:','ar':':راكب_أمواج_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3C4\U0001F3FC\U0000200D\U00002642':{'en':':man_surfing_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':surfer_mittelhelle_hautfarbe:','es':':hombre_haciendo_surf_tono_de_piel_claro_medio:','fr':':surfeur_peau_moyennement_claire:','ja':':サーフィンする男_やや薄い肌色:','ko':':서핑하는_남자_연한_갈색_피부:','pt':':homem_surfista_pele_morena_clara:','it':':surfista_uomo_carnagione_abbastanza_chiara:','fa':':موج_سوار_مرد_پوست_روشن:','id':':pria_berselancar_warna_kulit_cerah-sedang:','zh':':男生冲浪_中等浅肤色:','ru':':серфингист_светлый_тон_кожи:','tr':':sörf_yapan_erkek_orta_açık_cilt_tonu:','ar':':راكب_أمواج_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3C4\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_surfing_medium_skin_tone:','status':fully_qualified,'E':4,'de':':surfer_mittlere_hautfarbe:','es':':hombre_haciendo_surf_tono_de_piel_medio:','fr':':surfeur_peau_légèrement_mate:','ja':':サーフィンする男_中間の肌色:','ko':':서핑하는_남자_갈색_피부:','pt':':homem_surfista_pele_morena:','it':':surfista_uomo_carnagione_olivastra:','fa':':موج_سوار_مرد_پوست_طلایی:','id':':pria_berselancar_warna_kulit_sedang:','zh':':男生冲浪_中等肤色:','ru':':серфингист_среднии_тон_кожи:','tr':':sörf_yapan_erkek_orta_cilt_tonu:','ar':':راكب_أمواج_بشرة_بلون_معتدل:'},'\U0001F3C4\U0001F3FD\U0000200D\U00002642':{'en':':man_surfing_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':surfer_mittlere_hautfarbe:','es':':hombre_haciendo_surf_tono_de_piel_medio:','fr':':surfeur_peau_légèrement_mate:','ja':':サーフィンする男_中間の肌色:','ko':':서핑하는_남자_갈색_피부:','pt':':homem_surfista_pele_morena:','it':':surfista_uomo_carnagione_olivastra:','fa':':موج_سوار_مرد_پوست_طلایی:','id':':pria_berselancar_warna_kulit_sedang:','zh':':男生冲浪_中等肤色:','ru':':серфингист_средний_тон_кожи:','tr':':sörf_yapan_erkek_orta_cilt_tonu:','ar':':راكب_أمواج_بشرة_بلون_معتدل:'},'\U0001F3CA\U0000200D\U00002642\U0000FE0F':{'en':':man_swimming:','status':fully_qualified,'E':4,'alias':[':swimming_man:'],'de':':schwimmer:','es':':hombre_nadando:','fr':':nageur:','ja':':泳ぐ男:','ko':':수영하는_남자:','pt':':homem_nadando:','it':':nuotatore:','fa':':شناگر_مرد:','id':':pria_berenang:','zh':':男生游泳:','ru':':пловец:','tr':':yüzen_erkek:','ar':':سباح:'},'\U0001F3CA\U0000200D\U00002642':{'en':':man_swimming:','status':minimally_qualified,'E':4,'alias':[':swimming_man:'],'de':':schwimmer:','es':':hombre_nadando:','fr':':nageur:','ja':':泳ぐ男:','ko':':수영하는_남자:','pt':':homem_nadando:','it':':nuotatore:','fa':':شناگر_مرد:','id':':pria_berenang:','zh':':男生游泳:','ru':':пловец:','tr':':yüzen_erkek:','ar':':سباح:'},'\U0001F3CA\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_swimming_dark_skin_tone:','status':fully_qualified,'E':4,'de':':schwimmer_dunkle_hautfarbe:','es':':hombre_nadando_tono_de_piel_oscuro:','fr':':nageur_peau_foncée:','ja':':泳ぐ男_濃い肌色:','ko':':수영하는_남자_검은색_피부:','pt':':homem_nadando_pele_escura:','it':':nuotatore_carnagione_scura:','fa':':شناگر_مرد_پوست_آبنوسی:','id':':pria_berenang_warna_kulit_gelap:','zh':':男生游泳_较深肤色:','ru':':пловец_очень_темныи_тон_кожи:','tr':':yüzen_erkek_koyu_cilt_tonu:','ar':':سباح_بشرة_بلون_غامق:'},'\U0001F3CA\U0001F3FF\U0000200D\U00002642':{'en':':man_swimming_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':schwimmer_dunkle_hautfarbe:','es':':hombre_nadando_tono_de_piel_oscuro:','fr':':nageur_peau_foncée:','ja':':泳ぐ男_濃い肌色:','ko':':수영하는_남자_검은색_피부:','pt':':homem_nadando_pele_escura:','it':':nuotatore_carnagione_scura:','fa':':شناگر_مرد_پوست_آبنوسی:','id':':pria_berenang_warna_kulit_gelap:','zh':':男生游泳_较深肤色:','ru':':пловец_очень_темный_тон_кожи:','tr':':yüzen_erkek_koyu_cilt_tonu:','ar':':سباح_بشرة_بلون_غامق:'},'\U0001F3CA\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_swimming_light_skin_tone:','status':fully_qualified,'E':4,'de':':schwimmer_helle_hautfarbe:','es':':hombre_nadando_tono_de_piel_claro:','fr':':nageur_peau_claire:','ja':':泳ぐ男_薄い肌色:','ko':':수영하는_남자_하얀_피부:','pt':':homem_nadando_pele_clara:','it':':nuotatore_carnagione_chiara:','fa':':شناگر_مرد_پوست_سفید:','id':':pria_berenang_warna_kulit_cerah:','zh':':男生游泳_较浅肤色:','ru':':пловец_очень_светлыи_тон_кожи:','tr':':yüzen_erkek_açık_cilt_tonu:','ar':':سباح_بشرة_بلون_فاتح:'},'\U0001F3CA\U0001F3FB\U0000200D\U00002642':{'en':':man_swimming_light_skin_tone:','status':minimally_qualified,'E':4,'de':':schwimmer_helle_hautfarbe:','es':':hombre_nadando_tono_de_piel_claro:','fr':':nageur_peau_claire:','ja':':泳ぐ男_薄い肌色:','ko':':수영하는_남자_하얀_피부:','pt':':homem_nadando_pele_clara:','it':':nuotatore_carnagione_chiara:','fa':':شناگر_مرد_پوست_سفید:','id':':pria_berenang_warna_kulit_cerah:','zh':':男生游泳_较浅肤色:','ru':':пловец_очень_светлый_тон_кожи:','tr':':yüzen_erkek_açık_cilt_tonu:','ar':':سباح_بشرة_بلون_فاتح:'},'\U0001F3CA\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_swimming_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':schwimmer_mitteldunkle_hautfarbe:','es':':hombre_nadando_tono_de_piel_oscuro_medio:','fr':':nageur_peau_mate:','ja':':泳ぐ男_やや濃い肌色:','ko':':수영하는_남자_진한_갈색_피부:','pt':':homem_nadando_pele_morena_escura:','it':':nuotatore_carnagione_abbastanza_scura:','fa':':شناگر_مرد_پوست_گندمی:','id':':pria_berenang_warna_kulit_gelap-sedang:','zh':':男生游泳_中等深肤色:','ru':':пловец_темныи_тон_кожи:','tr':':yüzen_erkek_orta_koyu_cilt_tonu:','ar':':سباح_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3CA\U0001F3FE\U0000200D\U00002642':{'en':':man_swimming_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':schwimmer_mitteldunkle_hautfarbe:','es':':hombre_nadando_tono_de_piel_oscuro_medio:','fr':':nageur_peau_mate:','ja':':泳ぐ男_やや濃い肌色:','ko':':수영하는_남자_진한_갈색_피부:','pt':':homem_nadando_pele_morena_escura:','it':':nuotatore_carnagione_abbastanza_scura:','fa':':شناگر_مرد_پوست_گندمی:','id':':pria_berenang_warna_kulit_gelap-sedang:','zh':':男生游泳_中等深肤色:','ru':':пловец_темный_тон_кожи:','tr':':yüzen_erkek_orta_koyu_cilt_tonu:','ar':':سباح_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3CA\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_swimming_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':schwimmer_mittelhelle_hautfarbe:','es':':hombre_nadando_tono_de_piel_claro_medio:','fr':':nageur_peau_moyennement_claire:','ja':':泳ぐ男_やや薄い肌色:','ko':':수영하는_남자_연한_갈색_피부:','pt':':homem_nadando_pele_morena_clara:','it':':nuotatore_carnagione_abbastanza_chiara:','fa':':شناگر_مرد_پوست_روشن:','id':':pria_berenang_warna_kulit_cerah-sedang:','zh':':男生游泳_中等浅肤色:','ru':':пловец_светлыи_тон_кожи:','tr':':yüzen_erkek_orta_açık_cilt_tonu:','ar':':سباح_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3CA\U0001F3FC\U0000200D\U00002642':{'en':':man_swimming_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':schwimmer_mittelhelle_hautfarbe:','es':':hombre_nadando_tono_de_piel_claro_medio:','fr':':nageur_peau_moyennement_claire:','ja':':泳ぐ男_やや薄い肌色:','ko':':수영하는_남자_연한_갈색_피부:','pt':':homem_nadando_pele_morena_clara:','it':':nuotatore_carnagione_abbastanza_chiara:','fa':':شناگر_مرد_پوست_روشن:','id':':pria_berenang_warna_kulit_cerah-sedang:','zh':':男生游泳_中等浅肤色:','ru':':пловец_светлый_тон_кожи:','tr':':yüzen_erkek_orta_açık_cilt_tonu:','ar':':سباح_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3CA\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_swimming_medium_skin_tone:','status':fully_qualified,'E':4,'de':':schwimmer_mittlere_hautfarbe:','es':':hombre_nadando_tono_de_piel_medio:','fr':':nageur_peau_légèrement_mate:','ja':':泳ぐ男_中間の肌色:','ko':':수영하는_남자_갈색_피부:','pt':':homem_nadando_pele_morena:','it':':nuotatore_carnagione_olivastra:','fa':':شناگر_مرد_پوست_طلایی:','id':':pria_berenang_warna_kulit_sedang:','zh':':男生游泳_中等肤色:','ru':':пловец_среднии_тон_кожи:','tr':':yüzen_erkek_orta_cilt_tonu:','ar':':سباح_بشرة_بلون_معتدل:'},'\U0001F3CA\U0001F3FD\U0000200D\U00002642':{'en':':man_swimming_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':schwimmer_mittlere_hautfarbe:','es':':hombre_nadando_tono_de_piel_medio:','fr':':nageur_peau_légèrement_mate:','ja':':泳ぐ男_中間の肌色:','ko':':수영하는_남자_갈색_피부:','pt':':homem_nadando_pele_morena:','it':':nuotatore_carnagione_olivastra:','fa':':شناگر_مرد_پوست_طلایی:','id':':pria_berenang_warna_kulit_sedang:','zh':':男生游泳_中等肤色:','ru':':пловец_средний_тон_кожи:','tr':':yüzen_erkek_orta_cilt_tonu:','ar':':سباح_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U0001F3EB':{'en':':man_teacher:','status':fully_qualified,'E':4,'de':':lehrer:','es':':docente_hombre:','fr':':enseignant:','ja':':男性の教師:','ko':':남교사:','pt':':professor:','it':':professore:','fa':':آموزگار_مرد:','id':':pengajar_pria:','zh':':男老师:','ru':':учитель:','tr':':erkek_öğretmen:','ar':':معلم:'},'\U0001F468\U0001F3FF\U0000200D\U0001F3EB':{'en':':man_teacher_dark_skin_tone:','status':fully_qualified,'E':4,'de':':lehrer_dunkle_hautfarbe:','es':':docente_hombre_tono_de_piel_oscuro:','fr':':enseignant_peau_foncée:','ja':':男性の教師_濃い肌色:','ko':':남교사_검은색_피부:','pt':':professor_pele_escura:','it':':professore_carnagione_scura:','fa':':آموزگار_مرد_پوست_آبنوسی:','id':':pengajar_pria_warna_kulit_gelap:','zh':':男老师_较深肤色:','ru':':учитель_очень_темный_тон_кожи:','tr':':erkek_öğretmen_koyu_cilt_tonu:','ar':':معلم_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U0001F3EB':{'en':':man_teacher_light_skin_tone:','status':fully_qualified,'E':4,'de':':lehrer_helle_hautfarbe:','es':':docente_hombre_tono_de_piel_claro:','fr':':enseignant_peau_claire:','ja':':男性の教師_薄い肌色:','ko':':남교사_하얀_피부:','pt':':professor_pele_clara:','it':':professore_carnagione_chiara:','fa':':آموزگار_مرد_پوست_سفید:','id':':pengajar_pria_warna_kulit_cerah:','zh':':男老师_较浅肤色:','ru':':учитель_очень_светлый_тон_кожи:','tr':':erkek_öğretmen_açık_cilt_tonu:','ar':':معلم_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F3EB':{'en':':man_teacher_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':lehrer_mitteldunkle_hautfarbe:','es':':docente_hombre_tono_de_piel_oscuro_medio:','fr':':enseignant_peau_mate:','ja':':男性の教師_やや濃い肌色:','ko':':남교사_진한_갈색_피부:','pt':':professor_pele_morena_escura:','it':':professore_carnagione_abbastanza_scura:','fa':':آموزگار_مرد_پوست_گندمی:','id':':pengajar_pria_warna_kulit_gelap-sedang:','zh':':男老师_中等深肤色:','ru':':учитель_темный_тон_кожи:','tr':':erkek_öğretmen_orta_koyu_cilt_tonu:','ar':':معلم_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U0001F3EB':{'en':':man_teacher_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':lehrer_mittelhelle_hautfarbe:','es':':docente_hombre_tono_de_piel_claro_medio:','fr':':enseignant_peau_moyennement_claire:','ja':':男性の教師_やや薄い肌色:','ko':':남교사_연한_갈색_피부:','pt':':professor_pele_morena_clara:','it':':professore_carnagione_abbastanza_chiara:','fa':':آموزگار_مرد_پوست_روشن:','id':':pengajar_pria_warna_kulit_cerah-sedang:','zh':':男老师_中等浅肤色:','ru':':учитель_светлый_тон_кожи:','tr':':erkek_öğretmen_orta_açık_cilt_tonu:','ar':':معلم_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F3EB':{'en':':man_teacher_medium_skin_tone:','status':fully_qualified,'E':4,'de':':lehrer_mittlere_hautfarbe:','es':':docente_hombre_tono_de_piel_medio:','fr':':enseignant_peau_légèrement_mate:','ja':':男性の教師_中間の肌色:','ko':':남교사_갈색_피부:','pt':':professor_pele_morena:','it':':professore_carnagione_olivastra:','fa':':آموزگار_مرد_پوست_طلایی:','id':':pengajar_pria_warna_kulit_sedang:','zh':':男老师_中等肤色:','ru':':учитель_средний_тон_кожи:','tr':':erkek_öğretmen_orta_cilt_tonu:','ar':':معلم_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U0001F4BB':{'en':':man_technologist:','status':fully_qualified,'E':4,'de':':it-experte:','es':':profesional_de_la_tecnología_hombre:','fr':':informaticien:','ja':':男性技術者:','ko':':남자_기술_전문가:','pt':':tecnólogo:','it':':tecnologo:','fa':':کارشناس_فناوری_مرد:','id':':ahli_teknologi_pria:','zh':':男程序员:','ru':':мужчина_инженер:','tr':':erkek_teknoloji_uzmanı:','ar':':عالم_تكنولوجيا:'},'\U0001F468\U0001F3FF\U0000200D\U0001F4BB':{'en':':man_technologist_dark_skin_tone:','status':fully_qualified,'E':4,'de':':it-experte_dunkle_hautfarbe:','es':':profesional_de_la_tecnología_hombre_tono_de_piel_oscuro:','fr':':informaticien_peau_foncée:','ja':':男性技術者_濃い肌色:','ko':':남자_기술_전문가_검은색_피부:','pt':':tecnólogo_pele_escura:','it':':tecnologo_carnagione_scura:','fa':':کارشناس_فناوری_مرد_پوست_آبنوسی:','id':':ahli_teknologi_pria_warna_kulit_gelap:','zh':':男程序员_较深肤色:','ru':':мужчина_инженер_очень_темный_тон_кожи:','tr':':erkek_teknoloji_uzmanı_koyu_cilt_tonu:','ar':':عالم_تكنولوجيا_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U0001F4BB':{'en':':man_technologist_light_skin_tone:','status':fully_qualified,'E':4,'de':':it-experte_helle_hautfarbe:','es':':profesional_de_la_tecnología_hombre_tono_de_piel_claro:','fr':':informaticien_peau_claire:','ja':':男性技術者_薄い肌色:','ko':':남자_기술_전문가_하얀_피부:','pt':':tecnólogo_pele_clara:','it':':tecnologo_carnagione_chiara:','fa':':کارشناس_فناوری_مرد_پوست_سفید:','id':':ahli_teknologi_pria_warna_kulit_cerah:','zh':':男程序员_较浅肤色:','ru':':мужчина_инженер_очень_светлый_тон_кожи:','tr':':erkek_teknoloji_uzmanı_açık_cilt_tonu:','ar':':عالم_تكنولوجيا_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F4BB':{'en':':man_technologist_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':it-experte_mitteldunkle_hautfarbe:','es':':profesional_de_la_tecnología_hombre_tono_de_piel_oscuro_medio:','fr':':informaticien_peau_mate:','ja':':男性技術者_やや濃い肌色:','ko':':남자_기술_전문가_진한_갈색_피부:','pt':':tecnólogo_pele_morena_escura:','it':':tecnologo_carnagione_abbastanza_scura:','fa':':کارشناس_فناوری_مرد_پوست_گندمی:','id':':ahli_teknologi_pria_warna_kulit_gelap-sedang:','zh':':男程序员_中等深肤色:','ru':':мужчина_инженер_темный_тон_кожи:','tr':':erkek_teknoloji_uzmanı_orta_koyu_cilt_tonu:','ar':':عالم_تكنولوجيا_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U0001F4BB':{'en':':man_technologist_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':it-experte_mittelhelle_hautfarbe:','es':':profesional_de_la_tecnología_hombre_tono_de_piel_claro_medio:','fr':':informaticien_peau_moyennement_claire:','ja':':男性技術者_やや薄い肌色:','ko':':남자_기술_전문가_연한_갈색_피부:','pt':':tecnólogo_pele_morena_clara:','it':':tecnologo_carnagione_abbastanza_chiara:','fa':':کارشناس_فناوری_مرد_پوست_روشن:','id':':ahli_teknologi_pria_warna_kulit_cerah-sedang:','zh':':男程序员_中等浅肤色:','ru':':мужчина_инженер_светлый_тон_кожи:','tr':':erkek_teknoloji_uzmanı_orta_açık_cilt_tonu:','ar':':عالم_تكنولوجيا_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F4BB':{'en':':man_technologist_medium_skin_tone:','status':fully_qualified,'E':4,'de':':it-experte_mittlere_hautfarbe:','es':':profesional_de_la_tecnología_hombre_tono_de_piel_medio:','fr':':informaticien_peau_légèrement_mate:','ja':':男性技術者_中間の肌色:','ko':':남자_기술_전문가_갈색_피부:','pt':':tecnólogo_pele_morena:','it':':tecnologo_carnagione_olivastra:','fa':':کارشناس_فناوری_مرد_پوست_طلایی:','id':':ahli_teknologi_pria_warna_kulit_sedang:','zh':':男程序员_中等肤色:','ru':':мужчина_инженер_средний_тон_кожи:','tr':':erkek_teknoloji_uzmanı_orta_cilt_tonu:','ar':':عالم_تكنولوجيا_بشرة_بلون_معتدل:'},'\U0001F481\U0000200D\U00002642\U0000FE0F':{'en':':man_tipping_hand:','status':fully_qualified,'E':4,'alias':[':sassy_man:', ':tipping_hand_man:'],'de':':infoschalter-mitarbeiter:','es':':empleado_de_mostrador_de_información:','fr':':homme_paume_vers_le_haut:','ja':':案内する男:','ko':':손바닥_꺾은_남자:','pt':':homem_com_a_palma_virada_para_cima:','it':':uomo_con_suggerimento:','fa':':مرد_کمک_می_کند:','id':':pria_menengadahkan_tangan:','zh':':前台男:','ru':':мужчина_тянет_руку:','tr':':elini_yana_yatıran_erkek:','ar':':رجل_بيد_مائلة:'},'\U0001F481\U0000200D\U00002642':{'en':':man_tipping_hand:','status':minimally_qualified,'E':4,'alias':[':sassy_man:', ':tipping_hand_man:'],'de':':infoschalter-mitarbeiter:','es':':empleado_de_mostrador_de_información:','fr':':homme_paume_vers_le_haut:','ja':':案内する男:','ko':':손바닥_꺾은_남자:','pt':':homem_com_a_palma_virada_para_cima:','it':':uomo_al_punto_informazioni:','fa':':مرد_کمک_می_کند:','id':':pria_menengadahkan_tangan:','zh':':前台男:','ru':':мужчина_тянет_руку:','tr':':elini_yana_yatıran_erkek:','ar':':رجل_بيد_مائلة:'},'\U0001F481\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_tipping_hand_dark_skin_tone:','status':fully_qualified,'E':4,'de':':infoschalter-mitarbeiter_dunkle_hautfarbe:','es':':empleado_de_mostrador_de_información_tono_de_piel_oscuro:','fr':':homme_paume_vers_le_haut_peau_foncée:','ja':':案内する男_濃い肌色:','ko':':손바닥_꺾은_남자_검은색_피부:','pt':':homem_com_a_palma_virada_para_cima_pele_escura:','it':':uomo_con_suggerimento_carnagione_scura:','fa':':مرد_کمک_می_کند_پوست_آبنوسی:','id':':pria_menengadahkan_tangan_warna_kulit_gelap:','zh':':前台男_较深肤色:','ru':':мужчина_тянет_руку_очень_темныи_тон_кожи:','tr':':elini_yana_yatıran_erkek_koyu_cilt_tonu:','ar':':رجل_بيد_مائلة_بشرة_بلون_غامق:'},'\U0001F481\U0001F3FF\U0000200D\U00002642':{'en':':man_tipping_hand_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':infoschalter-mitarbeiter_dunkle_hautfarbe:','es':':empleado_de_mostrador_de_información_tono_de_piel_oscuro:','fr':':homme_paume_vers_le_haut_peau_foncée:','ja':':案内する男_濃い肌色:','ko':':손바닥_꺾은_남자_검은색_피부:','pt':':homem_com_a_palma_virada_para_cima_pele_escura:','it':':uomo_al_punto_informazioni_carnagione_scura:','fa':':مرد_کمک_می_کند_پوست_آبنوسی:','id':':pria_menengadahkan_tangan_warna_kulit_gelap:','zh':':前台男_较深肤色:','ru':':мужчина_тянет_руку_очень_темный_тон_кожи:','tr':':elini_yana_yatıran_erkek_koyu_cilt_tonu:','ar':':رجل_بيد_مائلة_بشرة_بلون_غامق:'},'\U0001F481\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_tipping_hand_light_skin_tone:','status':fully_qualified,'E':4,'de':':infoschalter-mitarbeiter_helle_hautfarbe:','es':':empleado_de_mostrador_de_información_tono_de_piel_claro:','fr':':homme_paume_vers_le_haut_peau_claire:','ja':':案内する男_薄い肌色:','ko':':손바닥_꺾은_남자_하얀_피부:','pt':':homem_com_a_palma_virada_para_cima_pele_clara:','it':':uomo_con_suggerimento_carnagione_chiara:','fa':':مرد_کمک_می_کند_پوست_سفید:','id':':pria_menengadahkan_tangan_warna_kulit_cerah:','zh':':前台男_较浅肤色:','ru':':мужчина_тянет_руку_очень_светлыи_тон_кожи:','tr':':elini_yana_yatıran_erkek_açık_cilt_tonu:','ar':':رجل_بيد_مائلة_بشرة_بلون_فاتح:'},'\U0001F481\U0001F3FB\U0000200D\U00002642':{'en':':man_tipping_hand_light_skin_tone:','status':minimally_qualified,'E':4,'de':':infoschalter-mitarbeiter_helle_hautfarbe:','es':':empleado_de_mostrador_de_información_tono_de_piel_claro:','fr':':homme_paume_vers_le_haut_peau_claire:','ja':':案内する男_薄い肌色:','ko':':손바닥_꺾은_남자_하얀_피부:','pt':':homem_com_a_palma_virada_para_cima_pele_clara:','it':':uomo_al_punto_informazioni_carnagione_chiara:','fa':':مرد_کمک_می_کند_پوست_سفید:','id':':pria_menengadahkan_tangan_warna_kulit_cerah:','zh':':前台男_较浅肤色:','ru':':мужчина_тянет_руку_очень_светлый_тон_кожи:','tr':':elini_yana_yatıran_erkek_açık_cilt_tonu:','ar':':رجل_بيد_مائلة_بشرة_بلون_فاتح:'},'\U0001F481\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_tipping_hand_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':infoschalter-mitarbeiter_mitteldunkle_hautfarbe:','es':':empleado_de_mostrador_de_información_tono_de_piel_oscuro_medio:','fr':':homme_paume_vers_le_haut_peau_mate:','ja':':案内する男_やや濃い肌色:','ko':':손바닥_꺾은_남자_진한_갈색_피부:','pt':':homem_com_a_palma_virada_para_cima_pele_morena_escura:','it':':uomo_con_suggerimento_carnagione_abbastanza_scura:','fa':':مرد_کمک_می_کند_پوست_گندمی:','id':':pria_menengadahkan_tangan_warna_kulit_gelap-sedang:','zh':':前台男_中等深肤色:','ru':':мужчина_тянет_руку_темныи_тон_кожи:','tr':':elini_yana_yatıran_erkek_orta_koyu_cilt_tonu:','ar':':رجل_بيد_مائلة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F481\U0001F3FE\U0000200D\U00002642':{'en':':man_tipping_hand_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':infoschalter-mitarbeiter_mitteldunkle_hautfarbe:','es':':empleado_de_mostrador_de_información_tono_de_piel_oscuro_medio:','fr':':homme_paume_vers_le_haut_peau_mate:','ja':':案内する男_やや濃い肌色:','ko':':손바닥_꺾은_남자_진한_갈색_피부:','pt':':homem_com_a_palma_virada_para_cima_pele_morena_escura:','it':':uomo_al_punto_informazioni_carnagione_abbastanza_scura:','fa':':مرد_کمک_می_کند_پوست_گندمی:','id':':pria_menengadahkan_tangan_warna_kulit_gelap-sedang:','zh':':前台男_中等深肤色:','ru':':мужчина_тянет_руку_темный_тон_кожи:','tr':':elini_yana_yatıran_erkek_orta_koyu_cilt_tonu:','ar':':رجل_بيد_مائلة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F481\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_tipping_hand_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':infoschalter-mitarbeiter_mittelhelle_hautfarbe:','es':':empleado_de_mostrador_de_información_tono_de_piel_claro_medio:','fr':':homme_paume_vers_le_haut_peau_moyennement_claire:','ja':':案内する男_やや薄い肌色:','ko':':손바닥_꺾은_남자_연한_갈색_피부:','pt':':homem_com_a_palma_virada_para_cima_pele_morena_clara:','it':':uomo_con_suggerimento_carnagione_abbastanza_chiara:','fa':':مرد_کمک_می_کند_پوست_روشن:','id':':pria_menengadahkan_tangan_warna_kulit_cerah-sedang:','zh':':前台男_中等浅肤色:','ru':':мужчина_тянет_руку_светлыи_тон_кожи:','tr':':elini_yana_yatıran_erkek_orta_açık_cilt_tonu:','ar':':رجل_بيد_مائلة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F481\U0001F3FC\U0000200D\U00002642':{'en':':man_tipping_hand_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':infoschalter-mitarbeiter_mittelhelle_hautfarbe:','es':':empleado_de_mostrador_de_información_tono_de_piel_claro_medio:','fr':':homme_paume_vers_le_haut_peau_moyennement_claire:','ja':':案内する男_やや薄い肌色:','ko':':손바닥_꺾은_남자_연한_갈색_피부:','pt':':homem_com_a_palma_virada_para_cima_pele_morena_clara:','it':':uomo_al_punto_informazioni_carnagione_abbastanza_chiara:','fa':':مرد_کمک_می_کند_پوست_روشن:','id':':pria_menengadahkan_tangan_warna_kulit_cerah-sedang:','zh':':前台男_中等浅肤色:','ru':':мужчина_тянет_руку_светлый_тон_кожи:','tr':':elini_yana_yatıran_erkek_orta_açık_cilt_tonu:','ar':':رجل_بيد_مائلة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F481\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_tipping_hand_medium_skin_tone:','status':fully_qualified,'E':4,'de':':infoschalter-mitarbeiter_mittlere_hautfarbe:','es':':empleado_de_mostrador_de_información_tono_de_piel_medio:','fr':':homme_paume_vers_le_haut_peau_légèrement_mate:','ja':':案内する男_中間の肌色:','ko':':손바닥_꺾은_남자_갈색_피부:','pt':':homem_com_a_palma_virada_para_cima_pele_morena:','it':':uomo_con_suggerimento_carnagione_olivastra:','fa':':مرد_کمک_می_کند_پوست_طلایی:','id':':pria_menengadahkan_tangan_warna_kulit_sedang:','zh':':前台男_中等肤色:','ru':':мужчина_тянет_руку_среднии_тон_кожи:','tr':':elini_yana_yatıran_erkek_orta_cilt_tonu:','ar':':رجل_بيد_مائلة_بشرة_بلون_معتدل:'},'\U0001F481\U0001F3FD\U0000200D\U00002642':{'en':':man_tipping_hand_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':infoschalter-mitarbeiter_mittlere_hautfarbe:','es':':empleado_de_mostrador_de_información_tono_de_piel_medio:','fr':':homme_paume_vers_le_haut_peau_légèrement_mate:','ja':':案内する男_中間の肌色:','ko':':손바닥_꺾은_남자_갈색_피부:','pt':':homem_com_a_palma_virada_para_cima_pele_morena:','it':':uomo_al_punto_informazioni_carnagione_olivastra:','fa':':مرد_کمک_می_کند_پوست_طلایی:','id':':pria_menengadahkan_tangan_warna_kulit_sedang:','zh':':前台男_中等肤色:','ru':':мужчина_тянет_руку_средний_тон_кожи:','tr':':elini_yana_yatıran_erkek_orta_cilt_tonu:','ar':':رجل_بيد_مائلة_بشرة_بلون_معتدل:'},'\U0001F9DB\U0000200D\U00002642\U0000FE0F':{'en':':man_vampire:','status':fully_qualified,'E':5,'alias':[':vampire_man:'],'de':':männlicher_vampir:','es':':vampiro_hombre:','fr':':vampire_homme:','ja':':男の吸血鬼:','ko':':남자_뱀파이어:','pt':':homem_vampiro:','it':':vampiro_uomo:','fa':':خون_آشام_مرد:','id':':vampir_pria:','zh':':男吸血鬼:','ru':':мужчина-вампир:','tr':':erkek_vampir:','ar':':مصاص_دماء_رجل:'},'\U0001F9DB\U0000200D\U00002642':{'en':':man_vampire:','status':minimally_qualified,'E':5,'alias':[':vampire_man:'],'de':':männlicher_vampir:','es':':vampiro_hombre:','fr':':vampire_homme:','ja':':男の吸血鬼:','ko':':남자_뱀파이어:','pt':':homem_vampiro:','it':':vampiro_uomo:','fa':':خون_آشام_مرد:','id':':vampir_pria:','zh':':男吸血鬼:','ru':':мужчина_вампир:','tr':':erkek_vampir:','ar':':مصاص_دماء:'},'\U0001F9DB\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_vampire_dark_skin_tone:','status':fully_qualified,'E':5,'de':':männlicher_vampir_dunkle_hautfarbe:','es':':vampiro_hombre_tono_de_piel_oscuro:','fr':':vampire_homme_peau_foncée:','ja':':男の吸血鬼_濃い肌色:','ko':':남자_뱀파이어_검은색_피부:','pt':':homem_vampiro_pele_escura:','it':':vampiro_uomo_carnagione_scura:','fa':':خون_آشام_مرد_پوست_آبنوسی:','id':':vampir_pria_warna_kulit_gelap:','zh':':男吸血鬼_较深肤色:','ru':':мужчина-вампир_очень_темныи_тон_кожи:','tr':':erkek_vampir_koyu_cilt_tonu:','ar':':مصاص_دماء_بشرة_بلون_غامق:'},'\U0001F9DB\U0001F3FF\U0000200D\U00002642':{'en':':man_vampire_dark_skin_tone:','status':minimally_qualified,'E':5,'de':':männlicher_vampir_dunkle_hautfarbe:','es':':vampiro_hombre_tono_de_piel_oscuro:','fr':':vampire_homme_peau_foncée:','ja':':男の吸血鬼_濃い肌色:','ko':':남자_뱀파이어_검은색_피부:','pt':':homem_vampiro_pele_escura:','it':':vampiro_uomo_carnagione_scura:','fa':':خون_آشام_مرد_پوست_آبنوسی:','id':':vampir_pria_warna_kulit_gelap:','zh':':男吸血鬼_较深肤色:','ru':':мужчина_вампир_очень_темный_тон_кожи:','tr':':erkek_vampir_koyu_cilt_tonu:','ar':':مصاص_دماء_بشرة_بلون_غامق:'},'\U0001F9DB\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_vampire_light_skin_tone:','status':fully_qualified,'E':5,'de':':männlicher_vampir_helle_hautfarbe:','es':':vampiro_hombre_tono_de_piel_claro:','fr':':vampire_homme_peau_claire:','ja':':男の吸血鬼_薄い肌色:','ko':':남자_뱀파이어_하얀_피부:','pt':':homem_vampiro_pele_clara:','it':':vampiro_uomo_carnagione_chiara:','fa':':خون_آشام_مرد_پوست_سفید:','id':':vampir_pria_warna_kulit_cerah:','zh':':男吸血鬼_较浅肤色:','ru':':мужчина-вампир_очень_светлыи_тон_кожи:','tr':':erkek_vampir_açık_cilt_tonu:','ar':':مصاص_دماء_بشرة_بلون_فاتح:'},'\U0001F9DB\U0001F3FB\U0000200D\U00002642':{'en':':man_vampire_light_skin_tone:','status':minimally_qualified,'E':5,'de':':männlicher_vampir_helle_hautfarbe:','es':':vampiro_hombre_tono_de_piel_claro:','fr':':vampire_homme_peau_claire:','ja':':男の吸血鬼_薄い肌色:','ko':':남자_뱀파이어_하얀_피부:','pt':':homem_vampiro_pele_clara:','it':':vampiro_uomo_carnagione_chiara:','fa':':خون_آشام_مرد_پوست_سفید:','id':':vampir_pria_warna_kulit_cerah:','zh':':男吸血鬼_较浅肤色:','ru':':мужчина_вампир_очень_светлый_тон_кожи:','tr':':erkek_vampir_açık_cilt_tonu:','ar':':مصاص_دماء_بشرة_بلون_فاتح:'},'\U0001F9DB\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_vampire_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':männlicher_vampir_mitteldunkle_hautfarbe:','es':':vampiro_hombre_tono_de_piel_oscuro_medio:','fr':':vampire_homme_peau_mate:','ja':':男の吸血鬼_やや濃い肌色:','ko':':남자_뱀파이어_진한_갈색_피부:','pt':':homem_vampiro_pele_morena_escura:','it':':vampiro_uomo_carnagione_abbastanza_scura:','fa':':خون_آشام_مرد_پوست_گندمی:','id':':vampir_pria_warna_kulit_gelap-sedang:','zh':':男吸血鬼_中等深肤色:','ru':':мужчина-вампир_темныи_тон_кожи:','tr':':erkek_vampir_orta_koyu_cilt_tonu:','ar':':مصاص_دماء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9DB\U0001F3FE\U0000200D\U00002642':{'en':':man_vampire_medium-dark_skin_tone:','status':minimally_qualified,'E':5,'de':':männlicher_vampir_mitteldunkle_hautfarbe:','es':':vampiro_hombre_tono_de_piel_oscuro_medio:','fr':':vampire_homme_peau_mate:','ja':':男の吸血鬼_やや濃い肌色:','ko':':남자_뱀파이어_진한_갈색_피부:','pt':':homem_vampiro_pele_morena_escura:','it':':vampiro_uomo_carnagione_abbastanza_scura:','fa':':خون_آشام_مرد_پوست_گندمی:','id':':vampir_pria_warna_kulit_gelap-sedang:','zh':':男吸血鬼_中等深肤色:','ru':':мужчина_вампир_темный_тон_кожи:','tr':':erkek_vampir_orta_koyu_cilt_tonu:','ar':':مصاص_دماء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9DB\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_vampire_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':männlicher_vampir_mittelhelle_hautfarbe:','es':':vampiro_hombre_tono_de_piel_claro_medio:','fr':':vampire_homme_peau_moyennement_claire:','ja':':男の吸血鬼_やや薄い肌色:','ko':':남자_뱀파이어_연한_갈색_피부:','pt':':homem_vampiro_pele_morena_clara:','it':':vampiro_uomo_carnagione_abbastanza_chiara:','fa':':خون_آشام_مرد_پوست_روشن:','id':':vampir_pria_warna_kulit_cerah-sedang:','zh':':男吸血鬼_中等浅肤色:','ru':':мужчина-вампир_светлыи_тон_кожи:','tr':':erkek_vampir_orta_açık_cilt_tonu:','ar':':مصاص_دماء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9DB\U0001F3FC\U0000200D\U00002642':{'en':':man_vampire_medium-light_skin_tone:','status':minimally_qualified,'E':5,'de':':männlicher_vampir_mittelhelle_hautfarbe:','es':':vampiro_hombre_tono_de_piel_claro_medio:','fr':':vampire_homme_peau_moyennement_claire:','ja':':男の吸血鬼_やや薄い肌色:','ko':':남자_뱀파이어_연한_갈색_피부:','pt':':homem_vampiro_pele_morena_clara:','it':':vampiro_uomo_carnagione_abbastanza_chiara:','fa':':خون_آشام_مرد_پوست_روشن:','id':':vampir_pria_warna_kulit_cerah-sedang:','zh':':男吸血鬼_中等浅肤色:','ru':':мужчина_вампир_светлый_тон_кожи:','tr':':erkek_vampir_orta_açık_cilt_tonu:','ar':':مصاص_دماء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9DB\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_vampire_medium_skin_tone:','status':fully_qualified,'E':5,'de':':männlicher_vampir_mittlere_hautfarbe:','es':':vampiro_hombre_tono_de_piel_medio:','fr':':vampire_homme_peau_légèrement_mate:','ja':':男の吸血鬼_中間の肌色:','ko':':남자_뱀파이어_갈색_피부:','pt':':homem_vampiro_pele_morena:','it':':vampiro_uomo_carnagione_olivastra:','fa':':خون_آشام_مرد_پوست_طلایی:','id':':vampir_pria_warna_kulit_sedang:','zh':':男吸血鬼_中等肤色:','ru':':мужчина-вампир_среднии_тон_кожи:','tr':':erkek_vampir_orta_cilt_tonu:','ar':':مصاص_دماء_بشرة_بلون_معتدل:'},'\U0001F9DB\U0001F3FD\U0000200D\U00002642':{'en':':man_vampire_medium_skin_tone:','status':minimally_qualified,'E':5,'de':':männlicher_vampir_mittlere_hautfarbe:','es':':vampiro_hombre_tono_de_piel_medio:','fr':':vampire_homme_peau_légèrement_mate:','ja':':男の吸血鬼_中間の肌色:','ko':':남자_뱀파이어_갈색_피부:','pt':':homem_vampiro_pele_morena:','it':':vampiro_uomo_carnagione_olivastra:','fa':':خون_آشام_مرد_پوست_طلایی:','id':':vampir_pria_warna_kulit_sedang:','zh':':男吸血鬼_中等肤色:','ru':':мужчина_вампир_средний_тон_кожи:','tr':':erkek_vampir_orta_cilt_tonu:','ar':':مصاص_دماء_بشرة_بلون_معتدل:'},'\U0001F6B6\U0000200D\U00002642\U0000FE0F':{'en':':man_walking:','status':fully_qualified,'E':4,'alias':[':walking_man:'],'de':':fußgänger:','es':':hombre_caminando:','fr':':homme_qui_marche:','ja':':歩く男:','ko':':걷는_남자:','pt':':homem_andando:','it':':uomo_che_cammina:','fa':':مرد_پیاده_روی_می_کند:','id':':pria_berjalan_kaki:','zh':':男行人:','ru':':мужчина_идет:','tr':':yürüyen_erkek:','ar':':سائر:'},'\U0001F6B6\U0000200D\U00002642':{'en':':man_walking:','status':minimally_qualified,'E':4,'alias':[':walking_man:'],'de':':fußgänger:','es':':hombre_caminando:','fr':':homme_qui_marche:','ja':':歩く男:','ko':':걷는_남자:','pt':':homem_andando:','it':':uomo_che_cammina:','fa':':مرد_پیاده_روی_می_کند:','id':':pria_berjalan_kaki:','zh':':男行人:','ru':':мужчина_идет:','tr':':yürüyen_erkek:','ar':':سائر:'},'\U0001F6B6\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_walking_dark_skin_tone:','status':fully_qualified,'E':4,'de':':fußgänger_dunkle_hautfarbe:','es':':hombre_caminando_tono_de_piel_oscuro:','fr':':homme_qui_marche_peau_foncée:','ja':':歩く男_濃い肌色:','ko':':걷는_남자_검은색_피부:','pt':':homem_andando_pele_escura:','it':':uomo_che_cammina_carnagione_scura:','fa':':مرد_پیاده_روی_می_کند_پوست_آبنوسی:','id':':pria_berjalan_kaki_warna_kulit_gelap:','zh':':男行人_较深肤色:','ru':':мужчина_идет_очень_темныи_тон_кожи:','tr':':yürüyen_erkek_koyu_cilt_tonu:','ar':':سائر_بشرة_بلون_غامق:'},'\U0001F6B6\U0001F3FF\U0000200D\U00002642':{'en':':man_walking_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':fußgänger_dunkle_hautfarbe:','es':':hombre_caminando_tono_de_piel_oscuro:','fr':':homme_qui_marche_peau_foncée:','ja':':歩く男_濃い肌色:','ko':':걷는_남자_검은색_피부:','pt':':homem_andando_pele_escura:','it':':uomo_che_cammina_carnagione_scura:','fa':':مرد_پیاده_روی_می_کند_پوست_آبنوسی:','id':':pria_berjalan_kaki_warna_kulit_gelap:','zh':':男行人_较深肤色:','ru':':мужчина_идет_очень_темный_тон_кожи:','tr':':yürüyen_erkek_koyu_cilt_tonu:','ar':':سائر_بشرة_بلون_غامق:'},'\U0001F6B6\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':man_walking_facing_right:','status':fully_qualified,'E':15.1,'de':':fußgänger_nach_rechts:','es':':hombre_caminando_hacia_la_derecha:','fr':':homme_qui_marche_vers_la_droite:','ja':':歩く男_右向き:','ko':':오른쪽을_향한_걷는_남자:','pt':':homem_andando_de_frente_para_a_direita:','it':':uomo_che_cammina_a_destra:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راست:','id':':pria_berjalan_kaki_menghadap_kanan:','zh':':男行人面向右边:','ru':':мужчина_идет_направленные_вправо:','tr':':sağa_dönük_yürüyen_erkek:','ar':':سائر_متجه_لليمين:'},'\U0001F6B6\U0000200D\U00002642\U0000200D\U000027A1\U0000FE0F':{'en':':man_walking_facing_right:','status':minimally_qualified,'E':15.1,'de':':fußgänger_nach_rechts:','es':':hombre_caminando_hacia_la_derecha:','fr':':homme_qui_marche_vers_la_droite:','ja':':歩く男_右向き:','ko':':오른쪽을_향한_걷는_남자:','pt':':homem_andando_de_frente_para_a_direita:','it':':uomo_che_cammina_a_destra:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راست:','id':':pria_berjalan_kaki_menghadap_kanan:','zh':':男行人面向右边:','ru':':мужчина_идет_направленные_вправо:','tr':':sağa_dönük_yürüyen_erkek:','ar':':سائر_متجه_لليمين:'},'\U0001F6B6\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1':{'en':':man_walking_facing_right:','status':minimally_qualified,'E':15.1,'de':':fußgänger_nach_rechts:','es':':hombre_caminando_hacia_la_derecha:','fr':':homme_qui_marche_vers_la_droite:','ja':':歩く男_右向き:','ko':':오른쪽을_향한_걷는_남자:','pt':':homem_andando_de_frente_para_a_direita:','it':':uomo_che_cammina_a_destra:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راست:','id':':pria_berjalan_kaki_menghadap_kanan:','zh':':男行人面向右边:','ru':':мужчина_идет_направленные_вправо:','tr':':sağa_dönük_yürüyen_erkek:','ar':':سائر_متجه_لليمين:'},'\U0001F6B6\U0000200D\U00002642\U0000200D\U000027A1':{'en':':man_walking_facing_right:','status':minimally_qualified,'E':15.1,'de':':fußgänger_nach_rechts:','es':':hombre_caminando_hacia_la_derecha:','fr':':homme_qui_marche_vers_la_droite:','ja':':歩く男_右向き:','ko':':오른쪽을_향한_걷는_남자:','pt':':homem_andando_de_frente_para_a_direita:','it':':uomo_che_cammina_a_destra:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راست:','id':':pria_berjalan_kaki_menghadap_kanan:','zh':':男行人面向右边:','ru':':мужчина_идет_направленные_вправо:','tr':':sağa_dönük_yürüyen_erkek:','ar':':سائر_متجه_لليمين:'},'\U0001F6B6\U0001F3FF\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':man_walking_facing_right_dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':fußgänger_nach_rechts_dunkle_hautfarbe:','es':':hombre_caminando_hacia_la_derecha_tono_de_piel_oscuro:','fr':':homme_qui_marche_vers_la_droite_peau_foncée:','ja':':歩く男_右向き_濃い肌色:','ko':':오른쪽을_향한_걷는_남자_검은색_피부:','pt':':homem_andando_de_frente_para_a_direita_pele_escura:','it':':uomo_che_cammina_a_destra_carnagione_scura:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راستپوست_آبنوسی:','id':':pria_berjalan_kaki_menghadap_kanan_warna_kulit_gelap:','zh':':男行人面向右边_较深肤色:','ru':':мужчина_идет_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_yürüyen_erkek_koyu_cilt_tonu:','ar':':سائر_متجه_لليمين_بلون_غامق:'},'\U0001F6B6\U0001F3FF\U0000200D\U00002642\U0000200D\U000027A1\U0000FE0F':{'en':':man_walking_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgänger_nach_rechts_dunkle_hautfarbe:','es':':hombre_caminando_hacia_la_derecha_tono_de_piel_oscuro:','fr':':homme_qui_marche_vers_la_droite_peau_foncée:','ja':':歩く男_右向き_濃い肌色:','ko':':오른쪽을_향한_걷는_남자_검은색_피부:','pt':':homem_andando_de_frente_para_a_direita_pele_escura:','it':':uomo_che_cammina_a_destra_carnagione_scura:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راستپوست_آبنوسی:','id':':pria_berjalan_kaki_menghadap_kanan_warna_kulit_gelap:','zh':':男行人面向右边_较深肤色:','ru':':мужчина_идет_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_yürüyen_erkek_koyu_cilt_tonu:','ar':':سائر_متجه_لليمين_بلون_غامق:'},'\U0001F6B6\U0001F3FF\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1':{'en':':man_walking_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgänger_nach_rechts_dunkle_hautfarbe:','es':':hombre_caminando_hacia_la_derecha_tono_de_piel_oscuro:','fr':':homme_qui_marche_vers_la_droite_peau_foncée:','ja':':歩く男_右向き_濃い肌色:','ko':':오른쪽을_향한_걷는_남자_검은색_피부:','pt':':homem_andando_de_frente_para_a_direita_pele_escura:','it':':uomo_che_cammina_a_destra_carnagione_scura:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راستپوست_آبنوسی:','id':':pria_berjalan_kaki_menghadap_kanan_warna_kulit_gelap:','zh':':男行人面向右边_较深肤色:','ru':':мужчина_идет_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_yürüyen_erkek_koyu_cilt_tonu:','ar':':سائر_متجه_لليمين_بلون_غامق:'},'\U0001F6B6\U0001F3FF\U0000200D\U00002642\U0000200D\U000027A1':{'en':':man_walking_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgänger_nach_rechts_dunkle_hautfarbe:','es':':hombre_caminando_hacia_la_derecha_tono_de_piel_oscuro:','fr':':homme_qui_marche_vers_la_droite_peau_foncée:','ja':':歩く男_右向き_濃い肌色:','ko':':오른쪽을_향한_걷는_남자_검은색_피부:','pt':':homem_andando_de_frente_para_a_direita_pele_escura:','it':':uomo_che_cammina_a_destra_carnagione_scura:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راستپوست_آبنوسی:','id':':pria_berjalan_kaki_menghadap_kanan_warna_kulit_gelap:','zh':':男行人面向右边_较深肤色:','ru':':мужчина_идет_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_yürüyen_erkek_koyu_cilt_tonu:','ar':':سائر_متجه_لليمين_بلون_غامق:'},'\U0001F6B6\U0001F3FB\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':man_walking_facing_right_light_skin_tone:','status':fully_qualified,'E':15.1,'de':':fußgänger_nach_rechts_helle_hautfarbe:','es':':hombre_caminando_hacia_la_derecha_tono_de_piel_claro:','fr':':homme_qui_marche_vers_la_droite_peau_claire:','ja':':歩く男_右向き_薄い肌色:','ko':':오른쪽을_향한_걷는_남자_하얀_피부:','pt':':homem_andando_de_frente_para_a_direita_pele_clara:','it':':uomo_che_cammina_a_destra_carnagione_chiara:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راست_پوست_سفید:','id':':pria_berjalan_kaki_menghadap_kanan_warna_kulit_cerah:','zh':':男行人面向右边_较浅肤色:','ru':':мужчина_идет_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_yürüyen_erkek_açık_cilt_tonu:','ar':':سائر_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F6B6\U0001F3FB\U0000200D\U00002642\U0000200D\U000027A1\U0000FE0F':{'en':':man_walking_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgänger_nach_rechts_helle_hautfarbe:','es':':hombre_caminando_hacia_la_derecha_tono_de_piel_claro:','fr':':homme_qui_marche_vers_la_droite_peau_claire:','ja':':歩く男_右向き_薄い肌色:','ko':':오른쪽을_향한_걷는_남자_하얀_피부:','pt':':homem_andando_de_frente_para_a_direita_pele_clara:','it':':uomo_che_cammina_a_destra_carnagione_chiara:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راست_پوست_سفید:','id':':pria_berjalan_kaki_menghadap_kanan_warna_kulit_cerah:','zh':':男行人面向右边_较浅肤色:','ru':':мужчина_идет_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_yürüyen_erkek_açık_cilt_tonu:','ar':':سائر_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F6B6\U0001F3FB\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1':{'en':':man_walking_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgänger_nach_rechts_helle_hautfarbe:','es':':hombre_caminando_hacia_la_derecha_tono_de_piel_claro:','fr':':homme_qui_marche_vers_la_droite_peau_claire:','ja':':歩く男_右向き_薄い肌色:','ko':':오른쪽을_향한_걷는_남자_하얀_피부:','pt':':homem_andando_de_frente_para_a_direita_pele_clara:','it':':uomo_che_cammina_a_destra_carnagione_chiara:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راست_پوست_سفید:','id':':pria_berjalan_kaki_menghadap_kanan_warna_kulit_cerah:','zh':':男行人面向右边_较浅肤色:','ru':':мужчина_идет_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_yürüyen_erkek_açık_cilt_tonu:','ar':':سائر_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F6B6\U0001F3FB\U0000200D\U00002642\U0000200D\U000027A1':{'en':':man_walking_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgänger_nach_rechts_helle_hautfarbe:','es':':hombre_caminando_hacia_la_derecha_tono_de_piel_claro:','fr':':homme_qui_marche_vers_la_droite_peau_claire:','ja':':歩く男_右向き_薄い肌色:','ko':':오른쪽을_향한_걷는_남자_하얀_피부:','pt':':homem_andando_de_frente_para_a_direita_pele_clara:','it':':uomo_che_cammina_a_destra_carnagione_chiara:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راست_پوست_سفید:','id':':pria_berjalan_kaki_menghadap_kanan_warna_kulit_cerah:','zh':':男行人面向右边_较浅肤色:','ru':':мужчина_идет_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_yürüyen_erkek_açık_cilt_tonu:','ar':':سائر_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F6B6\U0001F3FE\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':man_walking_facing_right_medium-dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':fußgänger_nach_rechts_mitteldunkle_hautfarbe:','es':':hombre_caminando_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':homme_qui_marche_vers_la_droite_peau_mate:','ja':':歩く男_右向き_やや濃い肌色:','ko':':오른쪽을_향한_걷는_남자_진한_갈색_피부:','pt':':homem_andando_de_frente_para_a_direita_pele_morena_escura:','it':':uomo_che_cammina_a_destra_carnagione_abbastanza_scura:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راست_پوست_گندمی:','id':':pria_berjalan_kaki_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':男行人面向右边_中等深肤色:','ru':':мужчина_идет_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_yürüyen_erkek_orta_koyu_cilt_tonu:','ar':':سائر_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F6B6\U0001F3FE\U0000200D\U00002642\U0000200D\U000027A1\U0000FE0F':{'en':':man_walking_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgänger_nach_rechts_mitteldunkle_hautfarbe:','es':':hombre_caminando_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':homme_qui_marche_vers_la_droite_peau_mate:','ja':':歩く男_右向き_やや濃い肌色:','ko':':오른쪽을_향한_걷는_남자_진한_갈색_피부:','pt':':homem_andando_de_frente_para_a_direita_pele_morena_escura:','it':':uomo_che_cammina_a_destra_carnagione_abbastanza_scura:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راست_پوست_گندمی:','id':':pria_berjalan_kaki_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':男行人面向右边_中等深肤色:','ru':':мужчина_идет_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_yürüyen_erkek_orta_koyu_cilt_tonu:','ar':':سائر_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F6B6\U0001F3FE\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1':{'en':':man_walking_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgänger_nach_rechts_mitteldunkle_hautfarbe:','es':':hombre_caminando_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':homme_qui_marche_vers_la_droite_peau_mate:','ja':':歩く男_右向き_やや濃い肌色:','ko':':오른쪽을_향한_걷는_남자_진한_갈색_피부:','pt':':homem_andando_de_frente_para_a_direita_pele_morena_escura:','it':':uomo_che_cammina_a_destra_carnagione_abbastanza_scura:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راست_پوست_گندمی:','id':':pria_berjalan_kaki_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':男行人面向右边_中等深肤色:','ru':':мужчина_идет_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_yürüyen_erkek_orta_koyu_cilt_tonu:','ar':':سائر_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F6B6\U0001F3FE\U0000200D\U00002642\U0000200D\U000027A1':{'en':':man_walking_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgänger_nach_rechts_mitteldunkle_hautfarbe:','es':':hombre_caminando_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':homme_qui_marche_vers_la_droite_peau_mate:','ja':':歩く男_右向き_やや濃い肌色:','ko':':오른쪽을_향한_걷는_남자_진한_갈색_피부:','pt':':homem_andando_de_frente_para_a_direita_pele_morena_escura:','it':':uomo_che_cammina_a_destra_carnagione_abbastanza_scura:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راست_پوست_گندمی:','id':':pria_berjalan_kaki_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':男行人面向右边_中等深肤色:','ru':':мужчина_идет_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_yürüyen_erkek_orta_koyu_cilt_tonu:','ar':':سائر_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F6B6\U0001F3FC\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':man_walking_facing_right_medium-light_skin_tone:','status':fully_qualified,'E':15.1,'de':':fußgänger_nach_rechts_mittelhelle_hautfarbe:','es':':hombre_caminando_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':homme_qui_marche_vers_la_droite_peau_moyennement_claire:','ja':':歩く男_右向き_やや薄い肌色:','ko':':오른쪽을_향한_걷는_남자_연한_갈색_피부:','pt':':homem_andando_de_frente_para_a_direita_pele_morena_clara:','it':':uomo_che_cammina_a_destra_carnagione_abbastanza_chiara:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راست_پوست_روشن:','id':':pria_berjalan_kaki_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':男行人面向右边_中等浅肤色:','ru':':мужчина_идет_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_yürüyen_erkek_orta_açık_cilt_tonu:','ar':':سائر_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F6B6\U0001F3FC\U0000200D\U00002642\U0000200D\U000027A1\U0000FE0F':{'en':':man_walking_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgänger_nach_rechts_mittelhelle_hautfarbe:','es':':hombre_caminando_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':homme_qui_marche_vers_la_droite_peau_moyennement_claire:','ja':':歩く男_右向き_やや薄い肌色:','ko':':오른쪽을_향한_걷는_남자_연한_갈색_피부:','pt':':homem_andando_de_frente_para_a_direita_pele_morena_clara:','it':':uomo_che_cammina_a_destra_carnagione_abbastanza_chiara:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راست_پوست_روشن:','id':':pria_berjalan_kaki_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':男行人面向右边_中等浅肤色:','ru':':мужчина_идет_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_yürüyen_erkek_orta_açık_cilt_tonu:','ar':':سائر_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F6B6\U0001F3FC\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1':{'en':':man_walking_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgänger_nach_rechts_mittelhelle_hautfarbe:','es':':hombre_caminando_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':homme_qui_marche_vers_la_droite_peau_moyennement_claire:','ja':':歩く男_右向き_やや薄い肌色:','ko':':오른쪽을_향한_걷는_남자_연한_갈색_피부:','pt':':homem_andando_de_frente_para_a_direita_pele_morena_clara:','it':':uomo_che_cammina_a_destra_carnagione_abbastanza_chiara:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راست_پوست_روشن:','id':':pria_berjalan_kaki_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':男行人面向右边_中等浅肤色:','ru':':мужчина_идет_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_yürüyen_erkek_orta_açık_cilt_tonu:','ar':':سائر_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F6B6\U0001F3FC\U0000200D\U00002642\U0000200D\U000027A1':{'en':':man_walking_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgänger_nach_rechts_mittelhelle_hautfarbe:','es':':hombre_caminando_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':homme_qui_marche_vers_la_droite_peau_moyennement_claire:','ja':':歩く男_右向き_やや薄い肌色:','ko':':오른쪽을_향한_걷는_남자_연한_갈색_피부:','pt':':homem_andando_de_frente_para_a_direita_pele_morena_clara:','it':':uomo_che_cammina_a_destra_carnagione_abbastanza_chiara:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راست_پوست_روشن:','id':':pria_berjalan_kaki_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':男行人面向右边_中等浅肤色:','ru':':мужчина_идет_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_yürüyen_erkek_orta_açık_cilt_tonu:','ar':':سائر_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F6B6\U0001F3FD\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':man_walking_facing_right_medium_skin_tone:','status':fully_qualified,'E':15.1,'de':':fußgänger_nach_rechts_mittlere_hautfarbe:','es':':hombre_caminando_hacia_la_derecha_tono_de_piel_medio:','fr':':homme_qui_marche_vers_la_droite_peau_légèrement_mate:','ja':':歩く男_右向き_中間の肌色:','ko':':오른쪽을_향한_걷는_남자_갈색_피부:','pt':':homem_andando_de_frente_para_a_direita_pele_morena:','it':':uomo_che_cammina_a_destra_carnagione_olivastra:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راست_پوست_طلایی:','id':':pria_berjalan_kaki_menghadap_kanan_warna_kulit_sedang:','zh':':男行人面向右边_中等肤色:','ru':':мужчина_идет_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_yürüyen_erkek_orta_cilt_tonu:','ar':':سائر_متجه_لليمين_بلون_معتدل:'},'\U0001F6B6\U0001F3FD\U0000200D\U00002642\U0000200D\U000027A1\U0000FE0F':{'en':':man_walking_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgänger_nach_rechts_mittlere_hautfarbe:','es':':hombre_caminando_hacia_la_derecha_tono_de_piel_medio:','fr':':homme_qui_marche_vers_la_droite_peau_légèrement_mate:','ja':':歩く男_右向き_中間の肌色:','ko':':오른쪽을_향한_걷는_남자_갈색_피부:','pt':':homem_andando_de_frente_para_a_direita_pele_morena:','it':':uomo_che_cammina_a_destra_carnagione_olivastra:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راست_پوست_طلایی:','id':':pria_berjalan_kaki_menghadap_kanan_warna_kulit_sedang:','zh':':男行人面向右边_中等肤色:','ru':':мужчина_идет_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_yürüyen_erkek_orta_cilt_tonu:','ar':':سائر_متجه_لليمين_بلون_معتدل:'},'\U0001F6B6\U0001F3FD\U0000200D\U00002642\U0000FE0F\U0000200D\U000027A1':{'en':':man_walking_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgänger_nach_rechts_mittlere_hautfarbe:','es':':hombre_caminando_hacia_la_derecha_tono_de_piel_medio:','fr':':homme_qui_marche_vers_la_droite_peau_légèrement_mate:','ja':':歩く男_右向き_中間の肌色:','ko':':오른쪽을_향한_걷는_남자_갈색_피부:','pt':':homem_andando_de_frente_para_a_direita_pele_morena:','it':':uomo_che_cammina_a_destra_carnagione_olivastra:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راست_پوست_طلایی:','id':':pria_berjalan_kaki_menghadap_kanan_warna_kulit_sedang:','zh':':男行人面向右边_中等肤色:','ru':':мужчина_идет_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_yürüyen_erkek_orta_cilt_tonu:','ar':':سائر_متجه_لليمين_بلون_معتدل:'},'\U0001F6B6\U0001F3FD\U0000200D\U00002642\U0000200D\U000027A1':{'en':':man_walking_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgänger_nach_rechts_mittlere_hautfarbe:','es':':hombre_caminando_hacia_la_derecha_tono_de_piel_medio:','fr':':homme_qui_marche_vers_la_droite_peau_légèrement_mate:','ja':':歩く男_右向き_中間の肌色:','ko':':오른쪽을_향한_걷는_남자_갈색_피부:','pt':':homem_andando_de_frente_para_a_direita_pele_morena:','it':':uomo_che_cammina_a_destra_carnagione_olivastra:','fa':':مرد_پیاده_روی_می_کند_به_سمت_راست_پوست_طلایی:','id':':pria_berjalan_kaki_menghadap_kanan_warna_kulit_sedang:','zh':':男行人面向右边_中等肤色:','ru':':мужчина_идет_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_yürüyen_erkek_orta_cilt_tonu:','ar':':سائر_متجه_لليمين_بلون_معتدل:'},'\U0001F6B6\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_walking_light_skin_tone:','status':fully_qualified,'E':4,'de':':fußgänger_helle_hautfarbe:','es':':hombre_caminando_tono_de_piel_claro:','fr':':homme_qui_marche_peau_claire:','ja':':歩く男_薄い肌色:','ko':':걷는_남자_하얀_피부:','pt':':homem_andando_pele_clara:','it':':uomo_che_cammina_carnagione_chiara:','fa':':مرد_پیاده_روی_می_کند_پوست_سفید:','id':':pria_berjalan_kaki_warna_kulit_cerah:','zh':':男行人_较浅肤色:','ru':':мужчина_идет_очень_светлыи_тон_кожи:','tr':':yürüyen_erkek_açık_cilt_tonu:','ar':':سائر_بشرة_بلون_فاتح:'},'\U0001F6B6\U0001F3FB\U0000200D\U00002642':{'en':':man_walking_light_skin_tone:','status':minimally_qualified,'E':4,'de':':fußgänger_helle_hautfarbe:','es':':hombre_caminando_tono_de_piel_claro:','fr':':homme_qui_marche_peau_claire:','ja':':歩く男_薄い肌色:','ko':':걷는_남자_하얀_피부:','pt':':homem_andando_pele_clara:','it':':uomo_che_cammina_carnagione_chiara:','fa':':مرد_پیاده_روی_می_کند_پوست_سفید:','id':':pria_berjalan_kaki_warna_kulit_cerah:','zh':':男行人_较浅肤色:','ru':':мужчина_идет_очень_светлый_тон_кожи:','tr':':yürüyen_erkek_açık_cilt_tonu:','ar':':سائر_بشرة_بلون_فاتح:'},'\U0001F6B6\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_walking_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':fußgänger_mitteldunkle_hautfarbe:','es':':hombre_caminando_tono_de_piel_oscuro_medio:','fr':':homme_qui_marche_peau_mate:','ja':':歩く男_やや濃い肌色:','ko':':걷는_남자_진한_갈색_피부:','pt':':homem_andando_pele_morena_escura:','it':':uomo_che_cammina_carnagione_abbastanza_scura:','fa':':مرد_پیاده_روی_می_کند_پوست_گندمی:','id':':pria_berjalan_kaki_warna_kulit_gelap-sedang:','zh':':男行人_中等深肤色:','ru':':мужчина_идет_темныи_тон_кожи:','tr':':yürüyen_erkek_orta_koyu_cilt_tonu:','ar':':سائر_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6B6\U0001F3FE\U0000200D\U00002642':{'en':':man_walking_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':fußgänger_mitteldunkle_hautfarbe:','es':':hombre_caminando_tono_de_piel_oscuro_medio:','fr':':homme_qui_marche_peau_mate:','ja':':歩く男_やや濃い肌色:','ko':':걷는_남자_진한_갈색_피부:','pt':':homem_andando_pele_morena_escura:','it':':uomo_che_cammina_carnagione_abbastanza_scura:','fa':':مرد_پیاده_روی_می_کند_پوست_گندمی:','id':':pria_berjalan_kaki_warna_kulit_gelap-sedang:','zh':':男行人_中等深肤色:','ru':':мужчина_идет_темный_тон_кожи:','tr':':yürüyen_erkek_orta_koyu_cilt_tonu:','ar':':سائر_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6B6\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_walking_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':fußgänger_mittelhelle_hautfarbe:','es':':hombre_caminando_tono_de_piel_claro_medio:','fr':':homme_qui_marche_peau_moyennement_claire:','ja':':歩く男_やや薄い肌色:','ko':':걷는_남자_연한_갈색_피부:','pt':':homem_andando_pele_morena_clara:','it':':uomo_che_cammina_carnagione_abbastanza_chiara:','fa':':مرد_پیاده_روی_می_کند_پوست_روشن:','id':':pria_berjalan_kaki_warna_kulit_cerah-sedang:','zh':':男行人_中等浅肤色:','ru':':мужчина_идет_светлыи_тон_кожи:','tr':':yürüyen_erkek_orta_açık_cilt_tonu:','ar':':سائر_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6B6\U0001F3FC\U0000200D\U00002642':{'en':':man_walking_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':fußgänger_mittelhelle_hautfarbe:','es':':hombre_caminando_tono_de_piel_claro_medio:','fr':':homme_qui_marche_peau_moyennement_claire:','ja':':歩く男_やや薄い肌色:','ko':':걷는_남자_연한_갈색_피부:','pt':':homem_andando_pele_morena_clara:','it':':uomo_che_cammina_carnagione_abbastanza_chiara:','fa':':مرد_پیاده_روی_می_کند_پوست_روشن:','id':':pria_berjalan_kaki_warna_kulit_cerah-sedang:','zh':':男行人_中等浅肤色:','ru':':мужчина_идет_светлый_тон_кожи:','tr':':yürüyen_erkek_orta_açık_cilt_tonu:','ar':':سائر_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6B6\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_walking_medium_skin_tone:','status':fully_qualified,'E':4,'de':':fußgänger_mittlere_hautfarbe:','es':':hombre_caminando_tono_de_piel_medio:','fr':':homme_qui_marche_peau_légèrement_mate:','ja':':歩く男_中間の肌色:','ko':':걷는_남자_갈색_피부:','pt':':homem_andando_pele_morena:','it':':uomo_che_cammina_carnagione_olivastra:','fa':':مرد_پیاده_روی_می_کند_پوست_طلایی:','id':':pria_berjalan_kaki_warna_kulit_sedang:','zh':':男行人_中等肤色:','ru':':мужчина_идет_среднии_тон_кожи:','tr':':yürüyen_erkek_orta_cilt_tonu:','ar':':سائر_بشرة_بلون_معتدل:'},'\U0001F6B6\U0001F3FD\U0000200D\U00002642':{'en':':man_walking_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':fußgänger_mittlere_hautfarbe:','es':':hombre_caminando_tono_de_piel_medio:','fr':':homme_qui_marche_peau_légèrement_mate:','ja':':歩く男_中間の肌色:','ko':':걷는_남자_갈색_피부:','pt':':homem_andando_pele_morena:','it':':uomo_che_cammina_carnagione_olivastra:','fa':':مرد_پیاده_روی_می_کند_پوست_طلایی:','id':':pria_berjalan_kaki_warna_kulit_sedang:','zh':':男行人_中等肤色:','ru':':мужчина_идет_средний_тон_кожи:','tr':':yürüyen_erkek_orta_cilt_tonu:','ar':':سائر_بشرة_بلون_معتدل:'},'\U0001F473\U0000200D\U00002642\U0000FE0F':{'en':':man_wearing_turban:','status':fully_qualified,'E':4,'alias':[':man_with_turban:'],'de':':mann_mit_turban:','es':':hombre_con_turbante:','fr':':homme_en_turban:','ja':':ターバンの男性:','ko':':터번을_쓰고_있는_남자:','pt':':homem_com_turbante:','it':':uomo_con_turbante:','fa':':مرد_با_دستار:','id':':pria_mengenakan_serban:','zh':':戴头巾的男人:','ru':':мужчина_в_тюрбане:','tr':':sarıklı_erkek:','ar':':رجل_يرتدي_عمامة:'},'\U0001F473\U0000200D\U00002642':{'en':':man_wearing_turban:','status':minimally_qualified,'E':4,'alias':[':man_with_turban:'],'de':':mann_mit_turban:','es':':hombre_con_turbante:','fr':':homme_en_turban:','ja':':ターバンの男性:','ko':':터번을_쓰고_있는_남자:','pt':':homem_com_turbante:','it':':uomo_con_turbante:','fa':':مرد_با_دستار:','id':':pria_mengenakan_serban:','zh':':戴头巾的男人:','ru':':мужчина_в_тюрбане:','tr':':sarıklı_erkek:','ar':':رجل_يرتدي_عمامة:'},'\U0001F473\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_wearing_turban_dark_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_turban_dunkle_hautfarbe:','es':':hombre_con_turbante_tono_de_piel_oscuro:','fr':':homme_en_turban_peau_foncée:','ja':':ターバンの男性_濃い肌色:','ko':':터번을_쓰고_있는_남자_검은색_피부:','pt':':homem_com_turbante_pele_escura:','it':':uomo_con_turbante_carnagione_scura:','fa':':مرد_با_دستار_پوست_آبنوسی:','id':':pria_mengenakan_serban_warna_kulit_gelap:','zh':':戴头巾的男人_较深肤色:','ru':':мужчина_в_тюрбане_очень_темныи_тон_кожи:','tr':':sarıklı_erkek_koyu_cilt_tonu:','ar':':رجل_يرتدي_عمامة_بشرة_بلون_غامق:'},'\U0001F473\U0001F3FF\U0000200D\U00002642':{'en':':man_wearing_turban_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_turban_dunkle_hautfarbe:','es':':hombre_con_turbante_tono_de_piel_oscuro:','fr':':homme_en_turban_peau_foncée:','ja':':ターバンの男性_濃い肌色:','ko':':터번을_쓰고_있는_남자_검은색_피부:','pt':':homem_com_turbante_pele_escura:','it':':uomo_con_turbante_carnagione_scura:','fa':':مرد_با_دستار_پوست_آبنوسی:','id':':pria_mengenakan_serban_warna_kulit_gelap:','zh':':戴头巾的男人_较深肤色:','ru':':мужчина_в_тюрбане_очень_темный_тон_кожи:','tr':':sarıklı_erkek_koyu_cilt_tonu:','ar':':رجل_يرتدي_عمامة_بشرة_بلون_غامق:'},'\U0001F473\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_wearing_turban_light_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_turban_helle_hautfarbe:','es':':hombre_con_turbante_tono_de_piel_claro:','fr':':homme_en_turban_peau_claire:','ja':':ターバンの男性_薄い肌色:','ko':':터번을_쓰고_있는_남자_하얀_피부:','pt':':homem_com_turbante_pele_clara:','it':':uomo_con_turbante_carnagione_chiara:','fa':':مرد_با_دستار_پوست_سفید:','id':':pria_mengenakan_serban_warna_kulit_cerah:','zh':':戴头巾的男人_较浅肤色:','ru':':мужчина_в_тюрбане_очень_светлыи_тон_кожи:','tr':':sarıklı_erkek_açık_cilt_tonu:','ar':':رجل_يرتدي_عمامة_بشرة_بلون_فاتح:'},'\U0001F473\U0001F3FB\U0000200D\U00002642':{'en':':man_wearing_turban_light_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_turban_helle_hautfarbe:','es':':hombre_con_turbante_tono_de_piel_claro:','fr':':homme_en_turban_peau_claire:','ja':':ターバンの男性_薄い肌色:','ko':':터번을_쓰고_있는_남자_하얀_피부:','pt':':homem_com_turbante_pele_clara:','it':':uomo_con_turbante_carnagione_chiara:','fa':':مرد_با_دستار_پوست_سفید:','id':':pria_mengenakan_serban_warna_kulit_cerah:','zh':':戴头巾的男人_较浅肤色:','ru':':мужчина_в_тюрбане_очень_светлый_тон_кожи:','tr':':sarıklı_erkek_açık_cilt_tonu:','ar':':رجل_يرتدي_عمامة_بشرة_بلون_فاتح:'},'\U0001F473\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_wearing_turban_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_turban_mitteldunkle_hautfarbe:','es':':hombre_con_turbante_tono_de_piel_oscuro_medio:','fr':':homme_en_turban_peau_mate:','ja':':ターバンの男性_やや濃い肌色:','ko':':터번을_쓰고_있는_남자_진한_갈색_피부:','pt':':homem_com_turbante_pele_morena_escura:','it':':uomo_con_turbante_carnagione_abbastanza_scura:','fa':':مرد_با_دستار_پوست_گندمی:','id':':pria_mengenakan_serban_warna_kulit_gelap-sedang:','zh':':戴头巾的男人_中等深肤色:','ru':':мужчина_в_тюрбане_темныи_тон_кожи:','tr':':sarıklı_erkek_orta_koyu_cilt_tonu:','ar':':رجل_يرتدي_عمامة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F473\U0001F3FE\U0000200D\U00002642':{'en':':man_wearing_turban_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_turban_mitteldunkle_hautfarbe:','es':':hombre_con_turbante_tono_de_piel_oscuro_medio:','fr':':homme_en_turban_peau_mate:','ja':':ターバンの男性_やや濃い肌色:','ko':':터번을_쓰고_있는_남자_진한_갈색_피부:','pt':':homem_com_turbante_pele_morena_escura:','it':':uomo_con_turbante_carnagione_abbastanza_scura:','fa':':مرد_با_دستار_پوست_گندمی:','id':':pria_mengenakan_serban_warna_kulit_gelap-sedang:','zh':':戴头巾的男人_中等深肤色:','ru':':мужчина_в_тюрбане_темный_тон_кожи:','tr':':sarıklı_erkek_orta_koyu_cilt_tonu:','ar':':رجل_يرتدي_عمامة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F473\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_wearing_turban_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_turban_mittelhelle_hautfarbe:','es':':hombre_con_turbante_tono_de_piel_claro_medio:','fr':':homme_en_turban_peau_moyennement_claire:','ja':':ターバンの男性_やや薄い肌色:','ko':':터번을_쓰고_있는_남자_연한_갈색_피부:','pt':':homem_com_turbante_pele_morena_clara:','it':':uomo_con_turbante_carnagione_abbastanza_chiara:','fa':':مرد_با_دستار_پوست_روشن:','id':':pria_mengenakan_serban_warna_kulit_cerah-sedang:','zh':':戴头巾的男人_中等浅肤色:','ru':':мужчина_в_тюрбане_светлыи_тон_кожи:','tr':':sarıklı_erkek_orta_açık_cilt_tonu:','ar':':رجل_يرتدي_عمامة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F473\U0001F3FC\U0000200D\U00002642':{'en':':man_wearing_turban_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_turban_mittelhelle_hautfarbe:','es':':hombre_con_turbante_tono_de_piel_claro_medio:','fr':':homme_en_turban_peau_moyennement_claire:','ja':':ターバンの男性_やや薄い肌色:','ko':':터번을_쓰고_있는_남자_연한_갈색_피부:','pt':':homem_com_turbante_pele_morena_clara:','it':':uomo_con_turbante_carnagione_abbastanza_chiara:','fa':':مرد_با_دستار_پوست_روشن:','id':':pria_mengenakan_serban_warna_kulit_cerah-sedang:','zh':':戴头巾的男人_中等浅肤色:','ru':':мужчина_в_тюрбане_светлый_тон_кожи:','tr':':sarıklı_erkek_orta_açık_cilt_tonu:','ar':':رجل_يرتدي_عمامة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F473\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_wearing_turban_medium_skin_tone:','status':fully_qualified,'E':4,'de':':mann_mit_turban_mittlere_hautfarbe:','es':':hombre_con_turbante_tono_de_piel_medio:','fr':':homme_en_turban_peau_légèrement_mate:','ja':':ターバンの男性_中間の肌色:','ko':':터번을_쓰고_있는_남자_갈색_피부:','pt':':homem_com_turbante_pele_morena:','it':':uomo_con_turbante_carnagione_olivastra:','fa':':مرد_با_دستار_پوست_طلایی:','id':':pria_mengenakan_serban_warna_kulit_sedang:','zh':':戴头巾的男人_中等肤色:','ru':':мужчина_в_тюрбане_среднии_тон_кожи:','tr':':sarıklı_erkek_orta_cilt_tonu:','ar':':رجل_يرتدي_عمامة_بشرة_بلون_معتدل:'},'\U0001F473\U0001F3FD\U0000200D\U00002642':{'en':':man_wearing_turban_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':mann_mit_turban_mittlere_hautfarbe:','es':':hombre_con_turbante_tono_de_piel_medio:','fr':':homme_en_turban_peau_légèrement_mate:','ja':':ターバンの男性_中間の肌色:','ko':':터번을_쓰고_있는_남자_갈색_피부:','pt':':homem_com_turbante_pele_morena:','it':':uomo_con_turbante_carnagione_olivastra:','fa':':مرد_با_دستار_پوست_طلایی:','id':':pria_mengenakan_serban_warna_kulit_sedang:','zh':':戴头巾的男人_中等肤色:','ru':':мужчина_в_тюрбане_средний_тон_кожи:','tr':':sarıklı_erkek_orta_cilt_tonu:','ar':':رجل_يرتدي_عمامة_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U0001F9B3':{'en':':man_white_hair:','status':fully_qualified,'E':11,'alias':[':white_haired_man:'],'de':':mann_weißes_haar:','es':':hombre_pelo_blanco:','fr':':homme_cheveux_blancs:','ja':':男性_白髪:','ko':':남자_백발:','pt':':homem_cabelo_branco:','it':':uomo_capelli_bianchi:','fa':':مرد_موسفید:','id':':pria_rambut_putih:','zh':':男人_白发:','ru':':мужчина_седые_волосы:','tr':':erkek_beyaz_saç:','ar':':رجل_شعر_أبيض:'},'\U0001F470\U0000200D\U00002642\U0000FE0F':{'en':':man_with_veil:','status':fully_qualified,'E':13,'de':':mann_mit_schleier:','es':':hombre_con_velo:','fr':':homme_avec_voile:','ja':':ベールの男性:','ko':':면사포를_쓴_남자:','pt':':noivo_com_véu:','it':':sposo_con_velo:','fa':':مرد_با_تور_عروس:','id':':pria_berpenutup_kepala:','zh':':戴头纱的男人:','ru':':мужчина_в_фате:','tr':':duvaklı_erkek:','ar':':رجل_بطرحة:'},'\U0001F470\U0000200D\U00002642':{'en':':man_with_veil:','status':minimally_qualified,'E':13,'de':':mann_mit_schleier:','es':':hombre_con_velo:','fr':':homme_avec_voile:','ja':':ベールの男性:','ko':':면사포를_쓴_남자:','pt':':noivo_com_véu:','it':':sposo_con_velo:','fa':':مرد_با_تور_عروس:','id':':pria_berpenutup_kepala:','zh':':戴头纱的男人:','ru':':мужчина_в_фате:','tr':':duvaklı_erkek:','ar':':رجل_بطرحة:'},'\U0001F470\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':man_with_veil_dark_skin_tone:','status':fully_qualified,'E':13,'de':':mann_mit_schleier_dunkle_hautfarbe:','es':':hombre_con_velo_tono_de_piel_oscuro:','fr':':homme_avec_voile_peau_foncée:','ja':':ベールの男性_濃い肌色:','ko':':면사포를_쓴_남자_검은색_피부:','pt':':noivo_com_véu_pele_escura:','it':':sposo_con_velo_carnagione_scura:','fa':':مرد_با_تور_عروس_پوست_آبنوسی:','id':':pria_berpenutup_kepala_warna_kulit_gelap:','zh':':戴头纱的男人_较深肤色:','ru':':мужчина_в_фате_очень_темныи_тон_кожи:','tr':':duvaklı_erkek_koyu_cilt_tonu:','ar':':رجل_بطرحة_بشرة_بلون_غامق:'},'\U0001F470\U0001F3FF\U0000200D\U00002642':{'en':':man_with_veil_dark_skin_tone:','status':minimally_qualified,'E':13,'de':':mann_mit_schleier_dunkle_hautfarbe:','es':':hombre_con_velo_tono_de_piel_oscuro:','fr':':homme_avec_voile_peau_foncée:','ja':':ベールの男性_濃い肌色:','ko':':면사포를_쓴_남자_검은색_피부:','pt':':noivo_com_véu_pele_escura:','it':':sposo_con_velo_carnagione_scura:','fa':':مرد_با_تور_عروس_پوست_آبنوسی:','id':':pria_berpenutup_kepala_warna_kulit_gelap:','zh':':戴头纱的男人_较深肤色:','ru':':мужчина_в_фате_очень_темный_тон_кожи:','tr':':duvaklı_erkek_koyu_cilt_tonu:','ar':':رجل_بطرحة_بشرة_بلون_غامق:'},'\U0001F470\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':man_with_veil_light_skin_tone:','status':fully_qualified,'E':13,'de':':mann_mit_schleier_helle_hautfarbe:','es':':hombre_con_velo_tono_de_piel_claro:','fr':':homme_avec_voile_peau_claire:','ja':':ベールの男性_薄い肌色:','ko':':면사포를_쓴_남자_하얀_피부:','pt':':noivo_com_véu_pele_clara:','it':':sposo_con_velo_carnagione_chiara:','fa':':مرد_با_تور_عروس_پوست_سفید:','id':':pria_berpenutup_kepala_warna_kulit_cerah:','zh':':戴头纱的男人_较浅肤色:','ru':':мужчина_в_фате_очень_светлыи_тон_кожи:','tr':':duvaklı_erkek_açık_cilt_tonu:','ar':':رجل_بطرحة_بشرة_بلون_فاتح:'},'\U0001F470\U0001F3FB\U0000200D\U00002642':{'en':':man_with_veil_light_skin_tone:','status':minimally_qualified,'E':13,'de':':mann_mit_schleier_helle_hautfarbe:','es':':hombre_con_velo_tono_de_piel_claro:','fr':':homme_avec_voile_peau_claire:','ja':':ベールの男性_薄い肌色:','ko':':면사포를_쓴_남자_하얀_피부:','pt':':noivo_com_véu_pele_clara:','it':':sposo_con_velo_carnagione_chiara:','fa':':مرد_با_تور_عروس_پوست_سفید:','id':':pria_berpenutup_kepala_warna_kulit_cerah:','zh':':戴头纱的男人_较浅肤色:','ru':':мужчина_в_фате_очень_светлый_тон_кожи:','tr':':duvaklı_erkek_açık_cilt_tonu:','ar':':رجل_بطرحة_بشرة_بلون_فاتح:'},'\U0001F470\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':man_with_veil_medium-dark_skin_tone:','status':fully_qualified,'E':13,'de':':mann_mit_schleier_mitteldunkle_hautfarbe:','es':':hombre_con_velo_tono_de_piel_oscuro_medio:','fr':':homme_avec_voile_peau_mate:','ja':':ベールの男性_やや濃い肌色:','ko':':면사포를_쓴_남자_진한_갈색_피부:','pt':':noivo_com_véu_pele_morena_escura:','it':':sposo_con_velo_carnagione_abbastanza_scura:','fa':':مرد_با_تور_عروس_پوست_گندمی:','id':':pria_berpenutup_kepala_warna_kulit_gelap-sedang:','zh':':戴头纱的男人_中等深肤色:','ru':':мужчина_в_фате_темныи_тон_кожи:','tr':':duvaklı_erkek_orta_koyu_cilt_tonu:','ar':':رجل_بطرحة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F470\U0001F3FE\U0000200D\U00002642':{'en':':man_with_veil_medium-dark_skin_tone:','status':minimally_qualified,'E':13,'de':':mann_mit_schleier_mitteldunkle_hautfarbe:','es':':hombre_con_velo_tono_de_piel_oscuro_medio:','fr':':homme_avec_voile_peau_mate:','ja':':ベールの男性_やや濃い肌色:','ko':':면사포를_쓴_남자_진한_갈색_피부:','pt':':noivo_com_véu_pele_morena_escura:','it':':sposo_con_velo_carnagione_abbastanza_scura:','fa':':مرد_با_تور_عروس_پوست_گندمی:','id':':pria_berpenutup_kepala_warna_kulit_gelap-sedang:','zh':':戴头纱的男人_中等深肤色:','ru':':мужчина_в_фате_темный_тон_кожи:','tr':':duvaklı_erkek_orta_koyu_cilt_tonu:','ar':':رجل_بطرحة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F470\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':man_with_veil_medium-light_skin_tone:','status':fully_qualified,'E':13,'de':':mann_mit_schleier_mittelhelle_hautfarbe:','es':':hombre_con_velo_tono_de_piel_claro_medio:','fr':':homme_avec_voile_peau_moyennement_claire:','ja':':ベールの男性_やや薄い肌色:','ko':':면사포를_쓴_남자_연한_갈색_피부:','pt':':noivo_com_véu_pele_morena_clara:','it':':sposo_con_velo_carnagione_abbastanza_chiara:','fa':':مرد_با_تور_عروس_پوست_روشن:','id':':pria_berpenutup_kepala_warna_kulit_cerah-sedang:','zh':':戴头纱的男人_中等浅肤色:','ru':':мужчина_в_фате_светлыи_тон_кожи:','tr':':duvaklı_erkek_orta_açık_cilt_tonu:','ar':':رجل_بطرحة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F470\U0001F3FC\U0000200D\U00002642':{'en':':man_with_veil_medium-light_skin_tone:','status':minimally_qualified,'E':13,'de':':mann_mit_schleier_mittelhelle_hautfarbe:','es':':hombre_con_velo_tono_de_piel_claro_medio:','fr':':homme_avec_voile_peau_moyennement_claire:','ja':':ベールの男性_やや薄い肌色:','ko':':면사포를_쓴_남자_연한_갈색_피부:','pt':':noivo_com_véu_pele_morena_clara:','it':':sposo_con_velo_carnagione_abbastanza_chiara:','fa':':مرد_با_تور_عروس_پوست_روشن:','id':':pria_berpenutup_kepala_warna_kulit_cerah-sedang:','zh':':戴头纱的男人_中等浅肤色:','ru':':мужчина_в_фате_светлый_тон_кожи:','tr':':duvaklı_erkek_orta_açık_cilt_tonu:','ar':':رجل_بطرحة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F470\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':man_with_veil_medium_skin_tone:','status':fully_qualified,'E':13,'de':':mann_mit_schleier_mittlere_hautfarbe:','es':':hombre_con_velo_tono_de_piel_medio:','fr':':homme_avec_voile_peau_légèrement_mate:','ja':':ベールの男性_中間の肌色:','ko':':면사포를_쓴_남자_갈색_피부:','pt':':noivo_com_véu_pele_morena:','it':':sposo_con_velo_carnagione_olivastra:','fa':':مرد_با_تور_عروس_پوست_طلایی:','id':':pria_berpenutup_kepala_warna_kulit_sedang:','zh':':戴头纱的男人_中等肤色:','ru':':мужчина_в_фате_среднии_тон_кожи:','tr':':duvaklı_erkek_orta_cilt_tonu:','ar':':رجل_بطرحة_بشرة_بلون_معتدل:'},'\U0001F470\U0001F3FD\U0000200D\U00002642':{'en':':man_with_veil_medium_skin_tone:','status':minimally_qualified,'E':13,'de':':mann_mit_schleier_mittlere_hautfarbe:','es':':hombre_con_velo_tono_de_piel_medio:','fr':':homme_avec_voile_peau_légèrement_mate:','ja':':ベールの男性_中間の肌色:','ko':':면사포를_쓴_남자_갈색_피부:','pt':':noivo_com_véu_pele_morena:','it':':sposo_con_velo_carnagione_olivastra:','fa':':مرد_با_تور_عروس_پوست_طلایی:','id':':pria_berpenutup_kepala_warna_kulit_sedang:','zh':':戴头纱的男人_中等肤色:','ru':':мужчина_в_фате_средний_тон_кожи:','tr':':duvaklı_erkek_orta_cilt_tonu:','ar':':رجل_بطرحة_بشرة_بلون_معتدل:'},'\U0001F468\U0000200D\U0001F9AF':{'en':':man_with_white_cane:','status':fully_qualified,'E':12,'alias':[':man_with_probing_cane:'],'de':':mann_mit_langstock:','es':':hombre_con_bastón:','fr':':homme_avec_canne_blanche:','ja':':杖をついた男性:','ko':':지팡이를_든_남자:','pt':':homem_com_bengala_para_cego:','it':':uomo_con_bastone_bianco_di_orientamento:','fa':':مرد_با_عصای_سفید:','id':':pria_dengan_tongkat:','zh':':拄盲杖的男人:','ru':':мужчина_с_тростью_для_ходьбы:','tr':':görme_engelli_bastonlu_erkek:','ar':':رجل_بعصا_سير:'},'\U0001F468\U0001F3FF\U0000200D\U0001F9AF':{'en':':man_with_white_cane_dark_skin_tone:','status':fully_qualified,'E':12,'de':':mann_mit_langstock_dunkle_hautfarbe:','es':':hombre_con_bastón_tono_de_piel_oscuro:','fr':':homme_avec_canne_blanche_peau_foncée:','ja':':杖をついた男性_濃い肌色:','ko':':지팡이를_든_남자_검은색_피부:','pt':':homem_com_bengala_para_cego_pele_escura:','it':':uomo_con_bastone_bianco_di_orientamento_carnagione_scura:','fa':':مرد_با_عصای_سفید_پوست_آبنوسی:','id':':pria_dengan_tongkat_warna_kulit_gelap:','zh':':拄盲杖的男人_较深肤色:','ru':':мужчина_с_тростью_для_ходьбы_очень_темный_тон_кожи:','tr':':görme_engelli_bastonlu_erkek_koyu_cilt_tonu:','ar':':رجل_بعصا_سير_بشرة_بلون_غامق:'},'\U0001F468\U0000200D\U0001F9AF\U0000200D\U000027A1\U0000FE0F':{'en':':man_with_white_cane_facing_right:','status':fully_qualified,'E':15.1,'de':':mann_mit_langstock_nach_rechts:','es':':hombre_con_bastón_hacia_la_derecha:','fr':':homme_avec_canne_blanche_vers_la_droite:','ja':':杖をついた男性_右向き:','ko':':오른쪽을_향한_지팡이를_든_남자:','pt':':homem_com_bengala_para_cego_de_frente_para_a_direita:','it':':uomo_con_bastone_bianco_di_orientamento_a_destra:','fa':':مرد_با_عصای_سفید_به_سمت_راست:','id':':pria_dengan_tongkat_menghadap_kanan:','zh':':拄盲杖的男人面向右边:','ru':':мужчина_с_тростью_для_ходьбы_направленные_вправо:','tr':':sağa_dönük_görme_engelli_bastonlu_erkek:','ar':':رجل_بعصا_سير_متجه_لليمين:'},'\U0001F468\U0000200D\U0001F9AF\U0000200D\U000027A1':{'en':':man_with_white_cane_facing_right:','status':minimally_qualified,'E':15.1,'de':':mann_mit_langstock_nach_rechts:','es':':hombre_con_bastón_hacia_la_derecha:','fr':':homme_avec_canne_blanche_vers_la_droite:','ja':':杖をついた男性_右向き:','ko':':오른쪽을_향한_지팡이를_든_남자:','pt':':homem_com_bengala_para_cego_de_frente_para_a_direita:','it':':uomo_con_bastone_bianco_di_orientamento_a_destra:','fa':':مرد_با_عصای_سفید_به_سمت_راست:','id':':pria_dengan_tongkat_menghadap_kanan:','zh':':拄盲杖的男人面向右边:','ru':':мужчина_с_тростью_для_ходьбы_направленные_вправо:','tr':':sağa_dönük_görme_engelli_bastonlu_erkek:','ar':':رجل_بعصا_سير_متجه_لليمين:'},'\U0001F468\U0001F3FF\U0000200D\U0001F9AF\U0000200D\U000027A1\U0000FE0F':{'en':':man_with_white_cane_facing_right_dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':mann_mit_langstock_nach_rechts_dunkle_hautfarbe:','es':':hombre_con_bastón_hacia_la_derecha_tono_de_piel_oscuro:','fr':':homme_avec_canne_blanche_vers_la_droite_peau_foncée:','ja':':杖をついた男性_右向き_濃い肌色:','ko':':오른쪽을_향한_지팡이를_든_남자_검은색_피부:','pt':':homem_com_bengala_para_cego_de_frente_para_a_direita_pele_escura:','it':':uomo_con_bastone_bianco_di_orientamento_a_destra_carnagione_scura:','fa':':مرد_با_عصای_سفید_به_سمت_راستپوست_آبنوسی:','id':':pria_dengan_tongkat_menghadap_kanan_warna_kulit_gelap:','zh':':拄盲杖的男人面向右边_较深肤色:','ru':':мужчина_с_тростью_для_ходьбы_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_erkek_koyu_cilt_tonu:','ar':':رجل_بعصا_سير_متجه_لليمين_بلون_غامق:'},'\U0001F468\U0001F3FF\U0000200D\U0001F9AF\U0000200D\U000027A1':{'en':':man_with_white_cane_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':mann_mit_langstock_nach_rechts_dunkle_hautfarbe:','es':':hombre_con_bastón_hacia_la_derecha_tono_de_piel_oscuro:','fr':':homme_avec_canne_blanche_vers_la_droite_peau_foncée:','ja':':杖をついた男性_右向き_濃い肌色:','ko':':오른쪽을_향한_지팡이를_든_남자_검은색_피부:','pt':':homem_com_bengala_para_cego_de_frente_para_a_direita_pele_escura:','it':':uomo_con_bastone_bianco_di_orientamento_a_destra_carnagione_scura:','fa':':مرد_با_عصای_سفید_به_سمت_راستپوست_آبنوسی:','id':':pria_dengan_tongkat_menghadap_kanan_warna_kulit_gelap:','zh':':拄盲杖的男人面向右边_较深肤色:','ru':':мужчина_с_тростью_для_ходьбы_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_erkek_koyu_cilt_tonu:','ar':':رجل_بعصا_سير_متجه_لليمين_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U0001F9AF\U0000200D\U000027A1\U0000FE0F':{'en':':man_with_white_cane_facing_right_light_skin_tone:','status':fully_qualified,'E':15.1,'de':':mann_mit_langstock_nach_rechts_helle_hautfarbe:','es':':hombre_con_bastón_hacia_la_derecha_tono_de_piel_claro:','fr':':homme_avec_canne_blanche_vers_la_droite_peau_claire:','ja':':杖をついた男性_右向き_薄い肌色:','ko':':오른쪽을_향한_지팡이를_든_남자_하얀_피부:','pt':':homem_com_bengala_para_cego_de_frente_para_a_direita_pele_clara:','it':':uomo_con_bastone_bianco_di_orientamento_a_destra_carnagione_chiara:','fa':':مرد_با_عصای_سفید_به_سمت_راست_پوست_سفید:','id':':pria_dengan_tongkat_menghadap_kanan_warna_kulit_cerah:','zh':':拄盲杖的男人面向右边_较浅肤色:','ru':':мужчина_с_тростью_для_ходьбы_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_erkek_açık_cilt_tonu:','ar':':رجل_بعصا_سير_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FB\U0000200D\U0001F9AF\U0000200D\U000027A1':{'en':':man_with_white_cane_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':mann_mit_langstock_nach_rechts_helle_hautfarbe:','es':':hombre_con_bastón_hacia_la_derecha_tono_de_piel_claro:','fr':':homme_avec_canne_blanche_vers_la_droite_peau_claire:','ja':':杖をついた男性_右向き_薄い肌色:','ko':':오른쪽을_향한_지팡이를_든_남자_하얀_피부:','pt':':homem_com_bengala_para_cego_de_frente_para_a_direita_pele_clara:','it':':uomo_con_bastone_bianco_di_orientamento_a_destra_carnagione_chiara:','fa':':مرد_با_عصای_سفید_به_سمت_راست_پوست_سفید:','id':':pria_dengan_tongkat_menghadap_kanan_warna_kulit_cerah:','zh':':拄盲杖的男人面向右边_较浅肤色:','ru':':мужчина_с_тростью_для_ходьбы_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_erkek_açık_cilt_tonu:','ar':':رجل_بعصا_سير_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F9AF\U0000200D\U000027A1\U0000FE0F':{'en':':man_with_white_cane_facing_right_medium-dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':mann_mit_langstock_nach_rechts_mitteldunkle_hautfarbe:','es':':hombre_con_bastón_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':homme_avec_canne_blanche_vers_la_droite_peau_mate:','ja':':杖をついた男性_右向き_やや濃い肌色:','ko':':오른쪽을_향한_지팡이를_든_남자_진한_갈색_피부:','pt':':homem_com_bengala_para_cego_de_frente_para_a_direita_pele_morena_escura:','it':':uomo_con_bastone_bianco_di_orientamento_a_destra_carnagione_abbastanza_scura:','fa':':مرد_با_عصای_سفید_به_سمت_راست_پوست_گندمی:','id':':pria_dengan_tongkat_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':拄盲杖的男人面向右边_中等深肤色:','ru':':мужчина_с_тростью_для_ходьбы_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_erkek_orta_koyu_cilt_tonu:','ar':':رجل_بعصا_سير_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F468\U0001F3FE\U0000200D\U0001F9AF\U0000200D\U000027A1':{'en':':man_with_white_cane_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':mann_mit_langstock_nach_rechts_mitteldunkle_hautfarbe:','es':':hombre_con_bastón_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':homme_avec_canne_blanche_vers_la_droite_peau_mate:','ja':':杖をついた男性_右向き_やや濃い肌色:','ko':':오른쪽을_향한_지팡이를_든_남자_진한_갈색_피부:','pt':':homem_com_bengala_para_cego_de_frente_para_a_direita_pele_morena_escura:','it':':uomo_con_bastone_bianco_di_orientamento_a_destra_carnagione_abbastanza_scura:','fa':':مرد_با_عصای_سفید_به_سمت_راست_پوست_گندمی:','id':':pria_dengan_tongkat_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':拄盲杖的男人面向右边_中等深肤色:','ru':':мужчина_с_тростью_для_ходьбы_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_erkek_orta_koyu_cilt_tonu:','ar':':رجل_بعصا_سير_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F468\U0001F3FC\U0000200D\U0001F9AF\U0000200D\U000027A1\U0000FE0F':{'en':':man_with_white_cane_facing_right_medium-light_skin_tone:','status':fully_qualified,'E':15.1,'de':':mann_mit_langstock_nach_rechts_mittelhelle_hautfarbe:','es':':hombre_con_bastón_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':homme_avec_canne_blanche_vers_la_droite_peau_moyennement_claire:','ja':':杖をついた男性_右向き_やや薄い肌色:','ko':':오른쪽을_향한_지팡이를_든_남자_연한_갈색_피부:','pt':':homem_com_bengala_para_cego_de_frente_para_a_direita_pele_morena_clara:','it':':uomo_con_bastone_bianco_di_orientamento_a_destra_carnagione_abbastanza_chiara:','fa':':مرد_با_عصای_سفید_به_سمت_راست_پوست_روشن:','id':':pria_dengan_tongkat_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':拄盲杖的男人面向右边_中等浅肤色:','ru':':мужчина_с_тростью_для_ходьбы_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_görme_engelli_bastonlu_erkek_orta_açık_cilt_tonu:','ar':':رجل_بعصا_سير_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FC\U0000200D\U0001F9AF\U0000200D\U000027A1':{'en':':man_with_white_cane_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':mann_mit_langstock_nach_rechts_mittelhelle_hautfarbe:','es':':hombre_con_bastón_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':homme_avec_canne_blanche_vers_la_droite_peau_moyennement_claire:','ja':':杖をついた男性_右向き_やや薄い肌色:','ko':':오른쪽을_향한_지팡이를_든_남자_연한_갈색_피부:','pt':':homem_com_bengala_para_cego_de_frente_para_a_direita_pele_morena_clara:','it':':uomo_con_bastone_bianco_di_orientamento_a_destra_carnagione_abbastanza_chiara:','fa':':مرد_با_عصای_سفید_به_سمت_راست_پوست_روشن:','id':':pria_dengan_tongkat_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':拄盲杖的男人面向右边_中等浅肤色:','ru':':мужчина_с_тростью_для_ходьбы_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_görme_engelli_bastonlu_erkek_orta_açık_cilt_tonu:','ar':':رجل_بعصا_سير_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F9AF\U0000200D\U000027A1\U0000FE0F':{'en':':man_with_white_cane_facing_right_medium_skin_tone:','status':fully_qualified,'E':15.1,'de':':mann_mit_langstock_nach_rechts_mittlere_hautfarbe:','es':':hombre_con_bastón_hacia_la_derecha_tono_de_piel_medio:','fr':':homme_avec_canne_blanche_vers_la_droite_peau_légèrement_mate:','ja':':杖をついた男性_右向き_中間の肌色:','ko':':오른쪽을_향한_지팡이를_든_남자_갈색_피부:','pt':':homem_com_bengala_para_cego_de_frente_para_a_direita_pele_morena:','it':':uomo_con_bastone_bianco_di_orientamento_a_destra_carnagione_olivastra:','fa':':مرد_با_عصای_سفید_به_سمت_راست_پوست_طلایی:','id':':pria_dengan_tongkat_menghadap_kanan_warna_kulit_sedang:','zh':':拄盲杖的男人面向右边_中等肤色:','ru':':мужчина_с_тростью_для_ходьбы_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_erkek_orta_cilt_tonu:','ar':':رجل_بعصا_سير_متجه_لليمين_بلون_معتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F9AF\U0000200D\U000027A1':{'en':':man_with_white_cane_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':mann_mit_langstock_nach_rechts_mittlere_hautfarbe:','es':':hombre_con_bastón_hacia_la_derecha_tono_de_piel_medio:','fr':':homme_avec_canne_blanche_vers_la_droite_peau_légèrement_mate:','ja':':杖をついた男性_右向き_中間の肌色:','ko':':오른쪽을_향한_지팡이를_든_남자_갈색_피부:','pt':':homem_com_bengala_para_cego_de_frente_para_a_direita_pele_morena:','it':':uomo_con_bastone_bianco_di_orientamento_a_destra_carnagione_olivastra:','fa':':مرد_با_عصای_سفید_به_سمت_راست_پوست_طلایی:','id':':pria_dengan_tongkat_menghadap_kanan_warna_kulit_sedang:','zh':':拄盲杖的男人面向右边_中等肤色:','ru':':мужчина_с_тростью_для_ходьбы_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_erkek_orta_cilt_tonu:','ar':':رجل_بعصا_سير_متجه_لليمين_بلون_معتدل:'},'\U0001F468\U0001F3FB\U0000200D\U0001F9AF':{'en':':man_with_white_cane_light_skin_tone:','status':fully_qualified,'E':12,'de':':mann_mit_langstock_helle_hautfarbe:','es':':hombre_con_bastón_tono_de_piel_claro:','fr':':homme_avec_canne_blanche_peau_claire:','ja':':杖をついた男性_薄い肌色:','ko':':지팡이를_든_남자_하얀_피부:','pt':':homem_com_bengala_para_cego_pele_clara:','it':':uomo_con_bastone_bianco_di_orientamento_carnagione_chiara:','fa':':مرد_با_عصای_سفید_پوست_سفید:','id':':pria_dengan_tongkat_warna_kulit_cerah:','zh':':拄盲杖的男人_较浅肤色:','ru':':мужчина_с_тростью_для_ходьбы_очень_светлый_тон_кожи:','tr':':görme_engelli_bastonlu_erkek_açık_cilt_tonu:','ar':':رجل_بعصا_سير_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F9AF':{'en':':man_with_white_cane_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':mann_mit_langstock_mitteldunkle_hautfarbe:','es':':hombre_con_bastón_tono_de_piel_oscuro_medio:','fr':':homme_avec_canne_blanche_peau_mate:','ja':':杖をついた男性_やや濃い肌色:','ko':':지팡이를_든_남자_진한_갈색_피부:','pt':':homem_com_bengala_para_cego_pele_morena_escura:','it':':uomo_con_bastone_bianco_di_orientamento_carnagione_abbastanza_scura:','fa':':مرد_با_عصای_سفید_پوست_گندمی:','id':':pria_dengan_tongkat_warna_kulit_gelap-sedang:','zh':':拄盲杖的男人_中等深肤色:','ru':':мужчина_с_тростью_для_ходьбы_темный_тон_кожи:','tr':':görme_engelli_bastonlu_erkek_orta_koyu_cilt_tonu:','ar':':رجل_بعصا_سير_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U0001F9AF':{'en':':man_with_white_cane_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':mann_mit_langstock_mittelhelle_hautfarbe:','es':':hombre_con_bastón_tono_de_piel_claro_medio:','fr':':homme_avec_canne_blanche_peau_moyennement_claire:','ja':':杖をついた男性_やや薄い肌色:','ko':':지팡이를_든_남자_연한_갈색_피부:','pt':':homem_com_bengala_para_cego_pele_morena_clara:','it':':uomo_con_bastone_bianco_di_orientamento_carnagione_abbastanza_chiara:','fa':':مرد_با_عصای_سفید_پوست_روشن:','id':':pria_dengan_tongkat_warna_kulit_cerah-sedang:','zh':':拄盲杖的男人_中等浅肤色:','ru':':мужчина_с_тростью_для_ходьбы_светлый_тон_кожи:','tr':':görme_engelli_bastonlu_erkek_orta_açık_cilt_tonu:','ar':':رجل_بعصا_سير_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F9AF':{'en':':man_with_white_cane_medium_skin_tone:','status':fully_qualified,'E':12,'de':':mann_mit_langstock_mittlere_hautfarbe:','es':':hombre_con_bastón_tono_de_piel_medio:','fr':':homme_avec_canne_blanche_peau_légèrement_mate:','ja':':杖をついた男性_中間の肌色:','ko':':지팡이를_든_남자_갈색_피부:','pt':':homem_com_bengala_para_cego_pele_morena:','it':':uomo_con_bastone_bianco_di_orientamento_carnagione_olivastra:','fa':':مرد_با_عصای_سفید_پوست_طلایی:','id':':pria_dengan_tongkat_warna_kulit_sedang:','zh':':拄盲杖的男人_中等肤色:','ru':':мужчина_с_тростью_для_ходьбы_средний_тон_кожи:','tr':':görme_engelli_bastonlu_erkek_orta_cilt_tonu:','ar':':رجل_بعصا_سير_بشرة_بلون_معتدل:'},'\U0001F9DF\U0000200D\U00002642\U0000FE0F':{'en':':man_zombie:','status':fully_qualified,'E':5,'alias':[':zombie_man:'],'de':':männlicher_zombie:','es':':zombi_hombre:','fr':':zombie_homme:','ja':':男のゾンビ:','ko':':남자_좀비:','pt':':homem_zumbi:','it':':zombie_uomo:','fa':':مرده_متحرک_مرد:','id':':zombi_pria:','zh':':男僵尸:','ru':':мужчина-зомби:','tr':':erkek_zombi:','ar':':رجل_زومبي:'},'\U0001F9DF\U0000200D\U00002642':{'en':':man_zombie:','status':minimally_qualified,'E':5,'alias':[':zombie_man:'],'de':':männlicher_zombie:','es':':zombi_hombre:','fr':':zombie_homme:','ja':':男のゾンビ:','ko':':남자_좀비:','pt':':homem_zumbi:','it':':zombie_uomo:','fa':':مرده_متحرک_مرد:','id':':zombi_pria:','zh':':男僵尸:','ru':':мужчина_зомби:','tr':':erkek_zombi:','ar':':رجل_زومبي:'},'\U0001F96D':{'en':':mango:','status':fully_qualified,'E':11,'de':':mango:','es':':mango:','fr':':mangue:','ja':':マンゴー:','ko':':망고:','pt':':manga:','it':':mango:','fa':':انبه:','id':':mangga:','zh':':芒果:','ru':':манго:','tr':':mango:','ar':':مانجو:'},'\U0001F570\U0000FE0F':{'en':':mantelpiece_clock:','status':fully_qualified,'E':0.7,'variant':True,'de':':kaminuhr:','es':':reloj_de_sobremesa:','fr':':pendule:','ja':':置時計:','ko':':벽난로_선반_시계:','pt':':relógio_de_mesa:','it':':orologio_da_mensola:','fa':':ساعت_طاقچه_ای:','id':':jam_perapian:','zh':':座钟:','ru':':каминные_часы:','tr':':raf_saati:','ar':':ساعة_موقد:'},'\U0001F570':{'en':':mantelpiece_clock:','status':unqualified,'E':0.7,'variant':True,'de':':kaminuhr:','es':':reloj_de_sobremesa:','fr':':pendule:','ja':':置時計:','ko':':벽난로_선반_시계:','pt':':relógio_de_mesa:','it':':orologio_da_mensola:','fa':':ساعت_طاقچه_ای:','id':':jam_perapian:','zh':':座钟:','ru':':каминные_часы:','tr':':raf_saati:','ar':':ساعة_موقد:'},'\U0001F9BD':{'en':':manual_wheelchair:','status':fully_qualified,'E':12,'de':':manueller_rollstuhl:','es':':silla_de_ruedas_manual:','fr':':fauteuil_roulant_manuel:','ja':':手動式車椅子:','ko':':수동_휠체어:','pt':':cadeira_de_rodas_manual:','it':':sedia_a_rotelle_manuale:','fa':':صندلی_چرخ_دار_دستی:','id':':kursi_roda_manual:','zh':':手动轮椅:','ru':':кресло_коляска_с_ручным_приводом:','tr':':tekerlekli_sandalye:','ar':':كرسي_متحرك_يدوي:'},'\U0001F45E':{'en':':man’s_shoe:','status':fully_qualified,'E':0.6,'alias':[':mans_shoe:', ':shoe:'],'de':':herrenschuh:','es':':zapato_de_hombre:','fr':':chaussure_d’homme:','ja':':紳士靴:','ko':':남성용_구두:','pt':':sapato_masculino:','it':':scarpa_da_uomo:','fa':':کفش_مردانه:','id':':sepatu_pria:','zh':':男鞋:','ru':':ботинок:','tr':':erkek_ayakkabısı:','ar':':حذاء_رجالي:'},'\U0001F5FE':{'en':':map_of_Japan:','status':fully_qualified,'E':0.6,'alias':[':japan:', ':map_of_japan:'],'de':':umriss_von_japan:','es':':mapa_de_japón:','fr':':carte_du_japon:','ja':':日本地図:','ko':':일본_지도:','pt':':mapa_do_japão:','it':':mappa_del_giappone:','fa':':نقشهٔ_ژاپن:','id':':peta_jepang:','zh':':日本地图:','ru':':карта_Японии:','tr':':japonya_haritası:','ar':':خريطة_اليابان:'},'\U0001F341':{'en':':maple_leaf:','status':fully_qualified,'E':0.6,'de':':ahornblatt:','es':':hoja_de_arce:','fr':':feuille_d’érable:','ja':':かえで:','ko':':단풍잎:','pt':':folha_de_bordo:','it':':foglia_d’acero:','fa':':برگ_افرا:','id':':daun_maple:','zh':':枫叶:','ru':':кленовый_лист:','tr':':akçaağaç_yaprağı:','ar':':ورقة_شجر_القيقب:'},'\U0001FA87':{'en':':maracas:','status':fully_qualified,'E':15,'de':':maracas:','es':':maracas:','fr':':maracas:','ja':':マラカス:','ko':':마라카스:','pt':':maracás:','it':':maracas:','fa':':ساز_جغجغه_ای:','id':':marakas:','zh':':沙球:','ru':':маракасы:','tr':':marakas:','ar':':مركاس:'},'\U0001F94B':{'en':':martial_arts_uniform:','status':fully_qualified,'E':3,'de':':kampfsportanzug:','es':':uniforme_de_artes_marciales:','fr':':tenue_d’arts_martiaux:','ja':':武道:','ko':':도복:','pt':':quimono_de_artes_marciais:','it':':kimono_per_arti_marziali:','fa':':لباس_هنرهای_رزمی:','id':':seragam_beladiri:','zh':':练武服:','ru':':спортивное_кимоно:','tr':':dövüş_sporu_forması:','ar':':زي_فنون_قتالية:'},'\U0001F9C9':{'en':':mate:','status':fully_qualified,'E':12,'de':':mate-tee:','es':':mate:','fr':':maté:','ja':':マテ茶:','ko':':마테차:','pt':':mate:','it':':mate:','fa':':ماته:','id':':teh_mate:','zh':':马黛茶:','ru':':мате:','tr':':mate_çayı:','ar':':متة:'},'\U0001F356':{'en':':meat_on_bone:','status':fully_qualified,'E':0.6,'de':':fleischhachse:','es':':carne_con_hueso:','fr':':viande_sur_un_os:','ja':':骨付き肉:','ko':':고기:','pt':':carne:','it':':carne:','fa':':گوشت_با_استخوان:','id':':daging_pada_tulang:','zh':':排骨:','ru':':мясо_на_косточке:','tr':':kemikli_et:','ar':':لحم_بعظم:'},'\U0001F9D1\U0000200D\U0001F527':{'en':':mechanic:','status':fully_qualified,'E':12.1,'de':':mechaniker(in):','es':':profesional_de_la_mecánica:','fr':':mécanicien_(tous_genres):','ja':':整備士:','ko':':정비공:','pt':':mecânico_trabalhando:','it':':meccanico:','fa':':مکانیک:','id':':mekanik:','zh':':技工:','ru':':механик:','tr':':tamirci:','ar':':خبير_ميكانيكي:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F527':{'en':':mechanic_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':mechaniker(in)_dunkle_hautfarbe:','es':':profesional_de_la_mecánica_tono_de_piel_oscuro:','fr':':mécanicien_(tous_genres)_peau_foncée:','ja':':整備士_濃い肌色:','ko':':정비공_검은색_피부:','pt':':mecânico_trabalhando_pele_escura:','it':':meccanico_carnagione_scura:','fa':':مکانیک_پوست_آبنوسی:','id':':mekanik_warna_kulit_gelap:','zh':':技工_较深肤色:','ru':':механик_очень_темный_тон_кожи:','tr':':tamirci_koyu_cilt_tonu:','ar':':خبير_ميكانيكي_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F527':{'en':':mechanic_light_skin_tone:','status':fully_qualified,'E':12.1,'de':':mechaniker(in)_helle_hautfarbe:','es':':profesional_de_la_mecánica_tono_de_piel_claro:','fr':':mécanicien_(tous_genres)_peau_claire:','ja':':整備士_薄い肌色:','ko':':정비공_하얀_피부:','pt':':mecânico_trabalhando_pele_clara:','it':':meccanico_carnagione_chiara:','fa':':مکانیک_پوست_سفید:','id':':mekanik_warna_kulit_cerah:','zh':':技工_较浅肤色:','ru':':механик_очень_светлый_тон_кожи:','tr':':tamirci_açık_cilt_tonu:','ar':':خبير_ميكانيكي_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F527':{'en':':mechanic_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':mechaniker(in)_mitteldunkle_hautfarbe:','es':':profesional_de_la_mecánica_tono_de_piel_oscuro_medio:','fr':':mécanicien_(tous_genres)_peau_mate:','ja':':整備士_やや濃い肌色:','ko':':정비공_진한_갈색_피부:','pt':':mecânico_trabalhando_pele_morena_escura:','it':':meccanico_carnagione_abbastanza_scura:','fa':':مکانیک_پوست_گندمی:','id':':mekanik_warna_kulit_gelap-sedang:','zh':':技工_中等深肤色:','ru':':механик_темный_тон_кожи:','tr':':tamirci_orta_koyu_cilt_tonu:','ar':':خبير_ميكانيكي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F527':{'en':':mechanic_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':mechaniker(in)_mittelhelle_hautfarbe:','es':':profesional_de_la_mecánica_tono_de_piel_claro_medio:','fr':':mécanicien_(tous_genres)_peau_moyennement_claire:','ja':':整備士_やや薄い肌色:','ko':':정비공_연한_갈색_피부:','pt':':mecânico_trabalhando_pele_morena_clara:','it':':meccanico_carnagione_abbastanza_chiara:','fa':':مکانیک_پوست_روشن:','id':':mekanik_warna_kulit_cerah-sedang:','zh':':技工_中等浅肤色:','ru':':механик_светлый_тон_кожи:','tr':':tamirci_orta_açık_cilt_tonu:','ar':':خبير_ميكانيكي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F527':{'en':':mechanic_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':mechaniker(in)_mittlere_hautfarbe:','es':':profesional_de_la_mecánica_tono_de_piel_medio:','fr':':mécanicien_(tous_genres)_peau_légèrement_mate:','ja':':整備士_中間の肌色:','ko':':정비공_갈색_피부:','pt':':mecânico_trabalhando_pele_morena:','it':':meccanico_carnagione_olivastra:','fa':':مکانیک_پوست_طلایی:','id':':mekanik_warna_kulit_sedang:','zh':':技工_中等肤色:','ru':':механик_средний_тон_кожи:','tr':':tamirci_orta_cilt_tonu:','ar':':خبير_ميكانيكي_بشرة_بلون_معتدل:'},'\U0001F9BE':{'en':':mechanical_arm:','status':fully_qualified,'E':12,'de':':armprothese:','es':':brazo_mecánico:','fr':':bras_mécanique:','ja':':義手:','ko':':기계_팔:','pt':':braço_mecânico:','it':':protesi_robotica_per_il_braccio:','fa':':بازوی_مکانیکی:','id':':lengan_robot:','zh':':机械手臂:','ru':':механическая_рука:','tr':':mekanik_kol:','ar':':ذراع_اصطناعية:'},'\U0001F9BF':{'en':':mechanical_leg:','status':fully_qualified,'E':12,'de':':beinprothese:','es':':pierna_mecánica:','fr':':jambe_mécanique:','ja':':義足:','ko':':인공_다리:','pt':':perna_mecânica:','it':':protesi_robotica_per_la_gamba:','fa':':پای_مکانیکی:','id':':tungkai_robot:','zh':':机械腿:','ru':':механическая_нога:','tr':':mekanik_bacak:','ar':':ساق_اصطناعية:'},'\U00002695\U0000FE0F':{'en':':medical_symbol:','status':fully_qualified,'E':4,'variant':True,'de':':äskulapstab:','es':':símbolo_de_medicina:','fr':':caducée:','ja':':医療のシンボル:','ko':':의학_기호:','pt':':símbolo_da_medicina:','it':':simbolo_della_medicina:','fa':':نماد_پزشکی:','id':':simbol_kedokteran:','zh':':医疗标志:','ru':':медицинскии_символ:','tr':':tıp_sembolü:','ar':':رمز_طبي:'},'\U00002695':{'en':':medical_symbol:','status':unqualified,'E':4,'variant':True,'de':':äskulapstab:','es':':símbolo_de_medicina:','fr':':caducée:','ja':':医療のシンボル:','ko':':의학_기호:','pt':':símbolo_da_medicina:','it':':simbolo_della_medicina:','fa':':نماد_پزشکی:','id':':simbol_kedokteran:','zh':':医疗标志:','ru':':медицинский_символ:','tr':':tıp_sembolü:','ar':':رمز_طبي:'},'\U0001F3FE':{'en':':medium-dark_skin_tone:','status':component,'E':1,'alias':[':emoji_modifier_fitzpatrick_type_5:'],'de':':mitteldunkle_hautfarbe:','es':':tono_de_piel_oscuro_medio:','fr':':peau_mate:','ja':':やや濃い肌色:','ko':':진한_갈색_피부:','pt':':pele_morena_escura:','it':':carnagione_abbastanza_scura:','fa':':پوست_گندمی:','id':':warna_kulit_gelap-sedang:','zh':':中等深肤色:','ru':':темный_тон_кожи:','tr':':orta_koyu_cilt_tonu:','ar':':بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3FC':{'en':':medium-light_skin_tone:','status':component,'E':1,'alias':[':emoji_modifier_fitzpatrick_type_3:'],'de':':mittelhelle_hautfarbe:','es':':tono_de_piel_claro_medio:','fr':':peau_moyennement_claire:','ja':':やや薄い肌色:','ko':':연한_갈색_피부:','pt':':pele_morena_clara:','it':':carnagione_abbastanza_chiara:','fa':':پوست_روشن:','id':':warna_kulit_cerah-sedang:','zh':':中等浅肤色:','ru':':светлый_тон_кожи:','tr':':orta_açık_cilt_tonu:','ar':':بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3FD':{'en':':medium_skin_tone:','status':component,'E':1,'alias':[':emoji_modifier_fitzpatrick_type_4:'],'de':':mittlere_hautfarbe:','es':':tono_de_piel_medio:','fr':':peau_légèrement_mate:','ja':':中間の肌色:','ko':':갈색_피부:','pt':':pele_morena:','it':':carnagione_olivastra:','fa':':پوست_طلایی:','id':':warna_kulit_sedang:','zh':':中等肤色:','ru':':средний_тон_кожи:','tr':':orta_cilt_tonu:','ar':':بشرة_بلون_معتدل:'},'\U0001F4E3':{'en':':megaphone:','status':fully_qualified,'E':0.6,'alias':[':mega:'],'de':':megafon:','es':':megáfono:','fr':':porte-voix:','ja':':メガホン:','ko':':메가폰:','pt':':megafone:','it':':megafono:','fa':':بلندگوی_دستی:','id':':megafon:','zh':':扩音器:','ru':':мегафон:','tr':':megafon:','ar':':مضخم_الصوت:'},'\U0001F348':{'en':':melon:','status':fully_qualified,'E':0.6,'de':':honigmelone:','es':':melón:','fr':':melon:','ja':':メロン:','ko':':멜론:','pt':':melão:','it':':melone:','fa':':طالبی:','id':':melon:','zh':':甜瓜:','ru':':дыня:','tr':':kavun:','ar':':شمام:'},'\U0001FAE0':{'en':':melting_face:','status':fully_qualified,'E':14,'de':':schmelzendes_gesicht:','es':':cara_derritiéndose:','fr':':visage_qui_fond:','ja':':溶けている顔:','ko':':녹아_내리는_얼굴:','pt':':rosto_derretendo:','it':':faccina_che_si_scioglie:','fa':':چهره_درحال_ذوب_شدن:','id':':wajah_meleleh:','zh':':融化:','ru':':тает:','tr':':eriyen_yüz:','ar':':وجه_يذوب:'},'\U0001F4DD':{'en':':memo:','status':fully_qualified,'E':0.6,'alias':[':pencil:'],'de':':papier_und_bleistift:','es':':cuaderno_de_notas:','fr':':mémo:','ja':':鉛筆とメモ:','ko':':메모:','pt':':memorando:','it':':memo:','fa':':یادداشت:','id':':memo:','zh':':备忘录:','ru':':заметка:','tr':':not:','ar':':مذكرة:'},'\U0001F46C':{'en':':men_holding_hands:','status':fully_qualified,'E':1,'alias':[':two_men_holding_hands:'],'de':':händchen_haltende_männer:','es':':hombres_de_la_mano:','fr':':deux_hommes_se_tenant_la_main:','ja':':手をつなぐ男性:','ko':':손을_잡고_있는_두_명의_남자:','pt':':dois_homens_de_mãos_dadas:','it':':due_uomini_che_si_tengono_per_mano:','fa':':دو_مرد_دست_در_دست:','id':':dua_pria_bergandengan:','zh':':手拉手的两个男人:','ru':':двое_мужчин:','tr':':el_ele_tutuşan_erkekler:','ar':':رجلان_يمسكان_بيد_بعضهما:'},'\U0001F46C\U0001F3FF':{'en':':men_holding_hands_dark_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_männer_dunkle_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_oscuro:','fr':':deux_hommes_se_tenant_la_main_peau_foncée:','ja':':手をつなぐ男性_濃い肌色:','ko':':손을_잡고_있는_두_명의_남자_검은색_피부:','pt':':dois_homens_de_mãos_dadas_pele_escura:','it':':due_uomini_che_si_tengono_per_mano_carnagione_scura:','fa':':دو_مرد_دست_در_دست_پوست_آبنوسی:','id':':dua_pria_bergandengan_warna_kulit_gelap:','zh':':手拉手的两个男人_较深肤色:','ru':':двое_мужчин_очень_темный_тон_кожи:','tr':':el_ele_tutuşan_erkekler_koyu_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_غامق:'},'\U0001F468\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FB':{'en':':men_holding_hands_dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_männer_dunkle_hautfarbe_helle_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_oscuro_y_tono_de_piel_claro:','fr':':deux_hommes_se_tenant_la_main_peau_foncée_et_peau_claire:','ja':':手をつなぐ男性_濃い肌色_薄い肌色:','ko':':손을_잡고_있는_두_명의_남자_검은색_피부_하얀_피부:','pt':':dois_homens_de_mãos_dadas_pele_escura_e_pele_clara:','it':':due_uomini_che_si_tengono_per_mano_carnagione_scura_e_carnagione_chiara:','fa':':دو_مرد_دست_در_دست_پوست_آبنوسی_پوست_سفید:','id':':dua_pria_bergandengan_warna_kulit_gelap_warna_kulit_cerah:','zh':':手拉手的两个男人_较深肤色较浅肤色:','ru':':двое_мужчин_очень_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':el_ele_tutuşan_erkekler_koyu_cilt_tonu_açık_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FE':{'en':':men_holding_hands_dark_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_männer_dunkle_hautfarbe_mitteldunkle_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_oscuro_y_tono_de_piel_oscuro_medio:','fr':':deux_hommes_se_tenant_la_main_peau_foncée_et_peau_mate:','ja':':手をつなぐ男性_濃い肌色_やや濃い肌色:','ko':':손을_잡고_있는_두_명의_남자_검은색_피부_진한_갈색_피부:','pt':':dois_homens_de_mãos_dadas_pele_escura_e_pele_morena_escura:','it':':due_uomini_che_si_tengono_per_mano_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':دو_مرد_دست_در_دست_پوست_آبنوسی_پوست_گندمی:','id':':dua_pria_bergandengan_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':手拉手的两个男人_较深肤色中等深肤色:','ru':':двое_мужчин_очень_темный_тон_кожи_темный_тон_кожи:','tr':':el_ele_tutuşan_erkekler_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FC':{'en':':men_holding_hands_dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_männer_dunkle_hautfarbe_mittelhelle_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_oscuro_y_tono_de_piel_claro_medio:','fr':':deux_hommes_se_tenant_la_main_peau_foncée_et_peau_moyennement_claire:','ja':':手をつなぐ男性_濃い肌色_やや薄い肌色:','ko':':손을_잡고_있는_두_명의_남자_검은색_피부_연한_갈색_피부:','pt':':dois_homens_de_mãos_dadas_pele_escura_e_pele_morena_clara:','it':':due_uomini_che_si_tengono_per_mano_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':دو_مرد_دست_در_دست_پوست_آبنوسی_پوست_روشن:','id':':dua_pria_bergandengan_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':手拉手的两个男人_较深肤色中等浅肤色:','ru':':двое_мужчин_очень_темный_тон_кожи_светлый_тон_кожи:','tr':':el_ele_tutuşan_erkekler_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FD':{'en':':men_holding_hands_dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_männer_dunkle_hautfarbe_mittlere_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_oscuro_y_tono_de_piel_medio:','fr':':deux_hommes_se_tenant_la_main_peau_foncée_et_peau_légèrement_mate:','ja':':手をつなぐ男性_濃い肌色_中間の肌色:','ko':':손을_잡고_있는_두_명의_남자_검은색_피부_갈색_피부:','pt':':dois_homens_de_mãos_dadas_pele_escura_e_pele_morena:','it':':due_uomini_che_si_tengono_per_mano_carnagione_scura_e_carnagione_olivastra:','fa':':دو_مرد_دست_در_دست_پوست_آبنوسی_پوست_طلایی:','id':':dua_pria_bergandengan_warna_kulit_gelap_warna_kulit_sedang:','zh':':手拉手的两个男人_较深肤色中等肤色:','ru':':двое_мужчин_очень_темный_тон_кожи_средний_тон_кожи:','tr':':el_ele_tutuşan_erkekler_koyu_cilt_tonu_orta_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F46C\U0001F3FB':{'en':':men_holding_hands_light_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_männer_helle_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_claro:','fr':':deux_hommes_se_tenant_la_main_peau_claire:','ja':':手をつなぐ男性_薄い肌色:','ko':':손을_잡고_있는_두_명의_남자_하얀_피부:','pt':':dois_homens_de_mãos_dadas_pele_clara:','it':':due_uomini_che_si_tengono_per_mano_carnagione_chiara:','fa':':دو_مرد_دست_در_دست_پوست_سفید:','id':':dua_pria_bergandengan_warna_kulit_cerah:','zh':':手拉手的两个男人_较浅肤色:','ru':':двое_мужчин_очень_светлый_тон_кожи:','tr':':el_ele_tutuşan_erkekler_açık_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_فاتح:'},'\U0001F468\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FF':{'en':':men_holding_hands_light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':händchen_haltende_männer_helle_hautfarbe_dunkle_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_claro_y_tono_de_piel_oscuro:','fr':':deux_hommes_se_tenant_la_main_peau_claire_et_peau_foncée:','ja':':手をつなぐ男性_薄い肌色_濃い肌色:','ko':':손을_잡고_있는_두_명의_남자_하얀_피부_검은색_피부:','pt':':dois_homens_de_mãos_dadas_pele_clara_e_pele_escura:','it':':due_uomini_che_si_tengono_per_mano_carnagione_chiara_e_carnagione_scura:','fa':':دو_مرد_دست_در_دست_پوست_سفید_پوست_آبنوسی:','id':':dua_pria_bergandengan_warna_kulit_cerah_warna_kulit_gelap:','zh':':手拉手的两个男人_较浅肤色较深肤色:','ru':':двое_мужчин_очень_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':el_ele_tutuşan_erkekler_açık_cilt_tonu_koyu_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FE':{'en':':men_holding_hands_light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':händchen_haltende_männer_helle_hautfarbe_mitteldunkle_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_claro_y_tono_de_piel_oscuro_medio:','fr':':deux_hommes_se_tenant_la_main_peau_claire_et_peau_mate:','ja':':手をつなぐ男性_薄い肌色_やや濃い肌色:','ko':':손을_잡고_있는_두_명의_남자_하얀_피부_진한_갈색_피부:','pt':':dois_homens_de_mãos_dadas_pele_clara_e_pele_morena_escura:','it':':due_uomini_che_si_tengono_per_mano_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':دو_مرد_دست_در_دست_پوست_سفید_پوست_گندمی:','id':':dua_pria_bergandengan_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':手拉手的两个男人_较浅肤色中等深肤色:','ru':':двое_мужчин_очень_светлый_тон_кожи_темный_тон_кожи:','tr':':el_ele_tutuşan_erkekler_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FC':{'en':':men_holding_hands_light_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':händchen_haltende_männer_helle_hautfarbe_mittelhelle_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_claro_y_tono_de_piel_claro_medio:','fr':':deux_hommes_se_tenant_la_main_peau_claire_et_peau_moyennement_claire:','ja':':手をつなぐ男性_薄い肌色_やや薄い肌色:','ko':':손을_잡고_있는_두_명의_남자_하얀_피부_연한_갈색_피부:','pt':':dois_homens_de_mãos_dadas_pele_clara_e_pele_morena_clara:','it':':due_uomini_che_si_tengono_per_mano_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':دو_مرد_دست_در_دست_پوست_سفید_پوست_روشن:','id':':dua_pria_bergandengan_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':手拉手的两个男人_较浅肤色中等浅肤色:','ru':':двое_мужчин_очень_светлый_тон_кожи_светлый_тон_кожи:','tr':':el_ele_tutuşan_erkekler_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FD':{'en':':men_holding_hands_light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':händchen_haltende_männer_helle_hautfarbe_mittlere_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_claro_y_tono_de_piel_medio:','fr':':deux_hommes_se_tenant_la_main_peau_claire_et_peau_légèrement_mate:','ja':':手をつなぐ男性_薄い肌色_中間の肌色:','ko':':손을_잡고_있는_두_명의_남자_하얀_피부_갈색_피부:','pt':':dois_homens_de_mãos_dadas_pele_clara_e_pele_morena:','it':':due_uomini_che_si_tengono_per_mano_carnagione_chiara_e_carnagione_olivastra:','fa':':دو_مرد_دست_در_دست_پوست_سفید_پوست_طلایی:','id':':dua_pria_bergandengan_warna_kulit_cerah_warna_kulit_sedang:','zh':':手拉手的两个男人_较浅肤色中等肤色:','ru':':двое_мужчин_очень_светлый_тон_кожи_средний_тон_кожи:','tr':':el_ele_tutuşan_erkekler_açık_cilt_tonu_orta_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F46C\U0001F3FE':{'en':':men_holding_hands_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_männer_mitteldunkle_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_oscuro_medio:','fr':':deux_hommes_se_tenant_la_main_peau_mate:','ja':':手をつなぐ男性_やや濃い肌色:','ko':':손을_잡고_있는_두_명의_남자_진한_갈색_피부:','pt':':dois_homens_de_mãos_dadas_pele_morena_escura:','it':':due_uomini_che_si_tengono_per_mano_carnagione_abbastanza_scura:','fa':':دو_مرد_دست_در_دست_پوست_گندمی:','id':':dua_pria_bergandengan_warna_kulit_gelap-sedang:','zh':':手拉手的两个男人_中等深肤色:','ru':':двое_мужчин_темный_тон_кожи:','tr':':el_ele_tutuşan_erkekler_orta_koyu_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FF':{'en':':men_holding_hands_medium-dark_skin_tone_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':händchen_haltende_männer_mitteldunkle_hautfarbe_dunkle_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_oscuro_medio_y_tono_de_piel_oscuro:','fr':':deux_hommes_se_tenant_la_main_peau_mate_et_peau_foncée:','ja':':手をつなぐ男性_やや濃い肌色_濃い肌色:','ko':':손을_잡고_있는_두_명의_남자_진한_갈색_피부_검은색_피부:','pt':':dois_homens_de_mãos_dadas_pele_morena_escura_e_pele_escura:','it':':due_uomini_che_si_tengono_per_mano_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':دو_مرد_دست_در_دست_پوست_گندمی_پوست_آبنوسی:','id':':dua_pria_bergandengan_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':手拉手的两个男人_中等深肤色较深肤色:','ru':':двое_мужчин_темный_тон_кожи_очень_темный_тон_кожи:','tr':':el_ele_tutuşan_erkekler_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FB':{'en':':men_holding_hands_medium-dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_männer_mitteldunkle_hautfarbe_helle_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_oscuro_medio_y_tono_de_piel_claro:','fr':':deux_hommes_se_tenant_la_main_peau_mate_et_peau_claire:','ja':':手をつなぐ男性_やや濃い肌色_薄い肌色:','ko':':손을_잡고_있는_두_명의_남자_진한_갈색_피부_하얀_피부:','pt':':dois_homens_de_mãos_dadas_pele_morena_escura_e_pele_clara:','it':':due_uomini_che_si_tengono_per_mano_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':دو_مرد_دست_در_دست_پوست_گندمی_پوست_سفید:','id':':dua_pria_bergandengan_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':手拉手的两个男人_中等深肤色较浅肤色:','ru':':двое_мужчин_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':el_ele_tutuşan_erkekler_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FC':{'en':':men_holding_hands_medium-dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_männer_mitteldunkle_hautfarbe_mittelhelle_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_oscuro_medio_y_tono_de_piel_claro_medio:','fr':':deux_hommes_se_tenant_la_main_peau_mate_et_peau_moyennement_claire:','ja':':手をつなぐ男性_やや濃い肌色_やや薄い肌色:','ko':':손을_잡고_있는_두_명의_남자_진한_갈색_피부_연한_갈색_피부:','pt':':dois_homens_de_mãos_dadas_pele_morena_escura_e_pele_morena_clara:','it':':due_uomini_che_si_tengono_per_mano_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':دو_مرد_دست_در_دست_پوست_گندمی_پوست_روشن:','id':':dua_pria_bergandengan_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':手拉手的两个男人_中等深肤色中等浅肤色:','ru':':двое_мужчин_темный_тон_кожи_светлый_тон_кожи:','tr':':el_ele_tutuşan_erkekler_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FD':{'en':':men_holding_hands_medium-dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_männer_mitteldunkle_hautfarbe_mittlere_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_oscuro_medio_y_tono_de_piel_medio:','fr':':deux_hommes_se_tenant_la_main_peau_mate_et_peau_légèrement_mate:','ja':':手をつなぐ男性_やや濃い肌色_中間の肌色:','ko':':손을_잡고_있는_두_명의_남자_진한_갈색_피부_갈색_피부:','pt':':dois_homens_de_mãos_dadas_pele_morena_escura_e_pele_morena:','it':':due_uomini_che_si_tengono_per_mano_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':دو_مرد_دست_در_دست_پوست_گندمی_پوست_طلایی:','id':':dua_pria_bergandengan_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':手拉手的两个男人_中等深肤色中等肤色:','ru':':двое_мужчин_темный_тон_кожи_средний_тон_кожи:','tr':':el_ele_tutuşan_erkekler_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F46C\U0001F3FC':{'en':':men_holding_hands_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_männer_mittelhelle_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_claro_medio:','fr':':deux_hommes_se_tenant_la_main_peau_moyennement_claire:','ja':':手をつなぐ男性_やや薄い肌色:','ko':':손을_잡고_있는_두_명의_남자_연한_갈색_피부:','pt':':dois_homens_de_mãos_dadas_pele_morena_clara:','it':':due_uomini_che_si_tengono_per_mano_carnagione_abbastanza_chiara:','fa':':دو_مرد_دست_در_دست_پوست_روشن:','id':':dua_pria_bergandengan_warna_kulit_cerah-sedang:','zh':':手拉手的两个男人_中等浅肤色:','ru':':двое_мужчин_светлый_тон_кожи:','tr':':el_ele_tutuşan_erkekler_orta_açık_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F468\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FF':{'en':':men_holding_hands_medium-light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':händchen_haltende_männer_mittelhelle_hautfarbe_dunkle_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_claro_medio_y_tono_de_piel_oscuro:','fr':':deux_hommes_se_tenant_la_main_peau_moyennement_claire_et_peau_foncée:','ja':':手をつなぐ男性_やや薄い肌色_濃い肌色:','ko':':손을_잡고_있는_두_명의_남자_연한_갈색_피부_검은색_피부:','pt':':dois_homens_de_mãos_dadas_pele_morena_clara_e_pele_escura:','it':':due_uomini_che_si_tengono_per_mano_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':دو_مرد_دست_در_دست_پوست_روشن_پوست_آبنوسی:','id':':dua_pria_bergandengan_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':手拉手的两个男人_中等浅肤色较深肤色:','ru':':двое_мужчин_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':el_ele_tutuşan_erkekler_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FB':{'en':':men_holding_hands_medium-light_skin_tone_light_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_männer_mittelhelle_hautfarbe_helle_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_claro_medio_y_tono_de_piel_claro:','fr':':deux_hommes_se_tenant_la_main_peau_moyennement_claire_et_peau_claire:','ja':':手をつなぐ男性_やや薄い肌色_薄い肌色:','ko':':손을_잡고_있는_두_명의_남자_연한_갈색_피부_하얀_피부:','pt':':dois_homens_de_mãos_dadas_pele_morena_clara_e_pele_clara:','it':':due_uomini_che_si_tengono_per_mano_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':دو_مرد_دست_در_دست_پوست_روشن_پوست_سفید:','id':':dua_pria_bergandengan_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':手拉手的两个男人_中等浅肤色较浅肤色:','ru':':двое_мужчин_светлый_тон_кожи_очень_светлый_тон_кожи:','tr':':el_ele_tutuşan_erkekler_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FE':{'en':':men_holding_hands_medium-light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':händchen_haltende_männer_mittelhelle_hautfarbe_mitteldunkle_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_claro_medio_y_tono_de_piel_oscuro_medio:','fr':':deux_hommes_se_tenant_la_main_peau_moyennement_claire_et_peau_mate:','ja':':手をつなぐ男性_やや薄い肌色_やや濃い肌色:','ko':':손을_잡고_있는_두_명의_남자_연한_갈색_피부_진한_갈색_피부:','pt':':dois_homens_de_mãos_dadas_pele_morena_clara_e_pele_morena_escura:','it':':due_uomini_che_si_tengono_per_mano_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':دو_مرد_دست_در_دست_پوست_روشن_پوست_گندمی:','id':':dua_pria_bergandengan_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':手拉手的两个男人_中等浅肤色中等深肤色:','ru':':двое_мужчин_светлый_тон_кожи_темный_тон_кожи:','tr':':el_ele_tutuşan_erkekler_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FD':{'en':':men_holding_hands_medium-light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':händchen_haltende_männer_mittelhelle_hautfarbe_mittlere_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_claro_medio_y_tono_de_piel_medio:','fr':':deux_hommes_se_tenant_la_main_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':手をつなぐ男性_やや薄い肌色_中間の肌色:','ko':':손을_잡고_있는_두_명의_남자_연한_갈색_피부_갈색_피부:','pt':':dois_homens_de_mãos_dadas_pele_morena_clara_e_pele_morena:','it':':due_uomini_che_si_tengono_per_mano_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':دو_مرد_دست_در_دست_پوست_روشن_پوست_طلایی:','id':':dua_pria_bergandengan_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':手拉手的两个男人_中等浅肤色中等肤色:','ru':':двое_мужчин_светлый_тон_кожи_средний_тон_кожи:','tr':':el_ele_tutuşan_erkekler_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F46C\U0001F3FD':{'en':':men_holding_hands_medium_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_männer_mittlere_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_medio:','fr':':deux_hommes_se_tenant_la_main_peau_légèrement_mate:','ja':':手をつなぐ男性_中間の肌色:','ko':':손을_잡고_있는_두_명의_남자_갈색_피부:','pt':':dois_homens_de_mãos_dadas_pele_morena:','it':':due_uomini_che_si_tengono_per_mano_carnagione_olivastra:','fa':':دو_مرد_دست_در_دست_پوست_طلایی:','id':':dua_pria_bergandengan_warna_kulit_sedang:','zh':':手拉手的两个男人_中等肤色:','ru':':двое_мужчин_средний_тон_кожи:','tr':':el_ele_tutuşan_erkekler_orta_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_معتدل:'},'\U0001F468\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FF':{'en':':men_holding_hands_medium_skin_tone_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':händchen_haltende_männer_mittlere_hautfarbe_dunkle_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_medio_y_tono_de_piel_oscuro:','fr':':deux_hommes_se_tenant_la_main_peau_légèrement_mate_et_peau_foncée:','ja':':手をつなぐ男性_中間の肌色_濃い肌色:','ko':':손을_잡고_있는_두_명의_남자_갈색_피부_검은색_피부:','pt':':dois_homens_de_mãos_dadas_pele_morena_e_pele_escura:','it':':due_uomini_che_si_tengono_per_mano_carnagione_olivastra_e_carnagione_scura:','fa':':دو_مرد_دست_در_دست_پوست_طلایی_پوست_آبنوسی:','id':':dua_pria_bergandengan_warna_kulit_sedang_warna_kulit_gelap:','zh':':手拉手的两个男人_中等肤色较深肤色:','ru':':двое_мужчин_средний_тон_кожи_очень_темный_тон_кожи:','tr':':el_ele_tutuşan_erkekler_orta_cilt_tonu_koyu_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001F468\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FB':{'en':':men_holding_hands_medium_skin_tone_light_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_männer_mittlere_hautfarbe_helle_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_medio_y_tono_de_piel_claro:','fr':':deux_hommes_se_tenant_la_main_peau_légèrement_mate_et_peau_claire:','ja':':手をつなぐ男性_中間の肌色_薄い肌色:','ko':':손을_잡고_있는_두_명의_남자_갈색_피부_하얀_피부:','pt':':dois_homens_de_mãos_dadas_pele_morena_e_pele_clara:','it':':due_uomini_che_si_tengono_per_mano_carnagione_olivastra_e_carnagione_chiara:','fa':':دو_مرد_دست_در_دست_پوست_طلایی_پوست_سفید:','id':':dua_pria_bergandengan_warna_kulit_sedang_warna_kulit_cerah:','zh':':手拉手的两个男人_中等肤色较浅肤色:','ru':':двое_мужчин_средний_тон_кожи_очень_светлый_тон_кожи:','tr':':el_ele_tutuşan_erkekler_orta_cilt_tonu_açık_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001F468\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FE':{'en':':men_holding_hands_medium_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':händchen_haltende_männer_mittlere_hautfarbe_mitteldunkle_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_medio_y_tono_de_piel_oscuro_medio:','fr':':deux_hommes_se_tenant_la_main_peau_légèrement_mate_et_peau_mate:','ja':':手をつなぐ男性_中間の肌色_やや濃い肌色:','ko':':손을_잡고_있는_두_명의_남자_갈색_피부_진한_갈색_피부:','pt':':dois_homens_de_mãos_dadas_pele_morena_e_pele_morena_escura:','it':':due_uomini_che_si_tengono_per_mano_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':دو_مرد_دست_در_دست_پوست_طلایی_پوست_گندمی:','id':':dua_pria_bergandengan_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':手拉手的两个男人_中等肤色中等深肤色:','ru':':двое_мужчин_средний_тон_кожи_темный_тон_кожи:','tr':':el_ele_tutuşan_erkekler_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F468\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FC':{'en':':men_holding_hands_medium_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_männer_mittlere_hautfarbe_mittelhelle_hautfarbe:','es':':hombres_de_la_mano_tono_de_piel_medio_y_tono_de_piel_claro_medio:','fr':':deux_hommes_se_tenant_la_main_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':手をつなぐ男性_中間の肌色_やや薄い肌色:','ko':':손을_잡고_있는_두_명의_남자_갈색_피부_연한_갈색_피부:','pt':':dois_homens_de_mãos_dadas_pele_morena_e_pele_morena_clara:','it':':due_uomini_che_si_tengono_per_mano_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':دو_مرد_دست_در_دست_پوست_طلایی_پوست_روشن:','id':':dua_pria_bergandengan_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':手拉手的两个男人_中等肤色中等浅肤色:','ru':':двое_мужчин_средний_тон_кожи_светлый_тон_кожи:','tr':':el_ele_tutuşan_erkekler_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':رجلان_يمسكان_بيد_بعضهما_بشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F46F\U0000200D\U00002642\U0000FE0F':{'en':':men_with_bunny_ears:','status':fully_qualified,'E':4,'alias':[':dancing_men:'],'de':':männer_mit_hasenohren:','es':':hombres_con_orejas_de_conejo:','fr':':hommes_avec_des_oreilles_de_lapin:','ja':':バニーボーイ:','ko':':토끼_귀를_쓰고_춤추는_남자들:','pt':':homens_com_orelhas_de_coelho:','it':':uomini_con_orecchie_da_coniglio:','fa':':مردان_در_پارتی:','id':':pria_dengan_telinga_kelinci:','zh':':兔先生:','ru':':мужчины_с_ушами_кролика:','tr':':tavşan_kulaklı_erkekler:','ar':':رجلان_يرقصان:'},'\U0001F46F\U0000200D\U00002642':{'en':':men_with_bunny_ears:','status':minimally_qualified,'E':4,'alias':[':dancing_men:'],'de':':männer_mit_hasenohren:','es':':hombres_con_orejas_de_conejo:','fr':':hommes_avec_des_oreilles_de_lapin:','ja':':バニーボーイ:','ko':':토끼_귀를_쓰고_춤추는_남자들:','pt':':homens_com_orelhas_de_coelho:','it':':uomini_con_orecchie_da_coniglio:','fa':':مردان_در_پارتی:','id':':pria_dengan_telinga_kelinci:','zh':':兔先生:','ru':':мужчины_с_ушами_кролика:','tr':':tavşan_kulaklı_erkekler:','ar':':رجلان_يرقصان:'},'\U0001F93C\U0000200D\U00002642\U0000FE0F':{'en':':men_wrestling:','status':fully_qualified,'E':4,'de':':ringende_männer:','es':':hombres_luchando:','fr':':lutteurs:','ja':':レスリングする男:','ko':':레슬링하는_남자:','pt':':homens_lutando:','it':':lottatori:','fa':':کُشتی_گیر_مرد:','id':':pria_bergulat:','zh':':男生摔跤:','ru':':мужчины-борцы:','tr':':güreş_yapan_erkek:','ar':':مصارعة_رجال:'},'\U0001F93C\U0000200D\U00002642':{'en':':men_wrestling:','status':minimally_qualified,'E':4,'de':':ringende_männer:','es':':hombres_luchando:','fr':':lutteurs:','ja':':レスリングする男:','ko':':레슬링하는_남자:','pt':':homens_lutando:','it':':lottatori:','fa':':کُشتی_گیر_مرد:','id':':pria_bergulat:','zh':':男生摔跤:','ru':':мужчины_борцы:','tr':':güreş_yapan_erkek:','ar':':مصارعة_رجال:'},'\U00002764\U0000FE0F\U0000200D\U0001FA79':{'en':':mending_heart:','status':fully_qualified,'E':13.1,'de':':herz_mit_verband:','es':':corazón_curándose:','fr':':cœur_avec_bandages:','ja':':包帯を巻いたハート:','ko':':낫고_있는_하트:','pt':':coração_remendado:','it':':cuore_guarito:','fa':':قلب_التیام_یافته:','id':':mengobati_hati_terluka:','zh':':修复受伤的心灵:','ru':':заживающее_сердце:','tr':':iyileşen_kalp:','ar':':قلب_يتعافى:'},'\U00002764\U0000200D\U0001FA79':{'en':':mending_heart:','status':unqualified,'E':13.1,'de':':herz_mit_verband:','es':':corazón_vendado:','fr':':cœur_avec_bandages:','ja':':包帯を巻いたハート:','ko':':낫고_있는_하트:','pt':':coração_remendado:','it':':cuore_guarito:','fa':':قلب_التیام_یافته:','id':':mengobati_hati_terluka:','zh':':修复受伤的心灵:','ru':':заживающее_сердце:','tr':':iyileşen_kalp:','ar':':قلب_يتعافى:'},'\U0001F54E':{'en':':menorah:','status':fully_qualified,'E':1,'alias':[':menorah_with_nine_branches:'],'de':':menora:','es':':menorá:','fr':':chandelier_à_sept_branches:','ja':':メノーラー:','ko':':메노라:','pt':':menorá:','it':':menorah:','fa':':منورا:','id':':menorah:','zh':':烛台:','ru':':менора:','tr':':yahudi_şamdanı:','ar':':شمعدان_شعار_اليهودية:'},'\U0001F6B9':{'en':':men’s_room:','status':fully_qualified,'E':0.6,'alias':[':mens:', ':mens_room:'],'variant':True,'de':':herren:','es':':aseo_para_hombres:','fr':':symbole_toilettes_hommes:','ja':':男子トイレ:','ko':':남자_화장실:','pt':':banheiro_masculino:','it':':simbolo_del_bagno_degli_uomini:','fa':':توالت_مردانه:','id':':toilet_pria:','zh':':男厕:','ru':':мужской_туалет:','tr':':erkekler_tuvaleti:','ar':':للرجال:'},'\U0001F9DC\U0000200D\U00002640\U0000FE0F':{'en':':mermaid:','status':fully_qualified,'E':5,'de':':meerjungfrau:','es':':sirena:','fr':':sirène:','ja':':マーメイド:','ko':':여자_인어:','pt':':sereia:','it':':sirena_donna:','fa':':پری_دریایی:','id':':putri_duyung:','zh':':美人鱼:','ru':':женщина-русалка:','tr':':denizkızı:','ar':':حورية_بحر:'},'\U0001F9DC\U0000200D\U00002640':{'en':':mermaid:','status':minimally_qualified,'E':5,'de':':meerjungfrau:','es':':sirena:','fr':':sirène:','ja':':マーメイド:','ko':':여자_인어:','pt':':sereia:','it':':sirena_donna:','fa':':پری_دریایی:','id':':putri_duyung:','zh':':美人鱼:','ru':':женщина_русалка:','tr':':denizkızı:','ar':':حورية_بحر:'},'\U0001F9DC\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':mermaid_dark_skin_tone:','status':fully_qualified,'E':5,'de':':meerjungfrau_dunkle_hautfarbe:','es':':sirena_tono_de_piel_oscuro:','fr':':sirène_peau_foncée:','ja':':マーメイド_濃い肌色:','ko':':여자_인어_검은색_피부:','pt':':sereia_pele_escura:','it':':sirena_donna_carnagione_scura:','fa':':پری_دریایی_پوست_آبنوسی:','id':':putri_duyung_warna_kulit_gelap:','zh':':美人鱼_较深肤色:','ru':':женщина-русалка_очень_темныи_тон_кожи:','tr':':denizkızı_koyu_cilt_tonu:','ar':':حورية_بحر_بشرة_بلون_غامق:'},'\U0001F9DC\U0001F3FF\U0000200D\U00002640':{'en':':mermaid_dark_skin_tone:','status':minimally_qualified,'E':5,'de':':meerjungfrau_dunkle_hautfarbe:','es':':sirena_tono_de_piel_oscuro:','fr':':sirène_peau_foncée:','ja':':マーメイド_濃い肌色:','ko':':여자_인어_검은색_피부:','pt':':sereia_pele_escura:','it':':sirena_donna_carnagione_scura:','fa':':پری_دریایی_پوست_آبنوسی:','id':':putri_duyung_warna_kulit_gelap:','zh':':美人鱼_较深肤色:','ru':':женщина_русалка_очень_темный_тон_кожи:','tr':':denizkızı_koyu_cilt_tonu:','ar':':حورية_بحر_بشرة_بلون_غامق:'},'\U0001F9DC\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':mermaid_light_skin_tone:','status':fully_qualified,'E':5,'de':':meerjungfrau_helle_hautfarbe:','es':':sirena_tono_de_piel_claro:','fr':':sirène_peau_claire:','ja':':マーメイド_薄い肌色:','ko':':여자_인어_하얀_피부:','pt':':sereia_pele_clara:','it':':sirena_donna_carnagione_chiara:','fa':':پری_دریایی_پوست_سفید:','id':':putri_duyung_warna_kulit_cerah:','zh':':美人鱼_较浅肤色:','ru':':женщина-русалка_очень_светлыи_тон_кожи:','tr':':denizkızı_açık_cilt_tonu:','ar':':حورية_بحر_بشرة_بلون_فاتح:'},'\U0001F9DC\U0001F3FB\U0000200D\U00002640':{'en':':mermaid_light_skin_tone:','status':minimally_qualified,'E':5,'de':':meerjungfrau_helle_hautfarbe:','es':':sirena_tono_de_piel_claro:','fr':':sirène_peau_claire:','ja':':マーメイド_薄い肌色:','ko':':여자_인어_하얀_피부:','pt':':sereia_pele_clara:','it':':sirena_donna_carnagione_chiara:','fa':':پری_دریایی_پوست_سفید:','id':':putri_duyung_warna_kulit_cerah:','zh':':美人鱼_较浅肤色:','ru':':женщина_русалка_очень_светлый_тон_кожи:','tr':':denizkızı_açık_cilt_tonu:','ar':':حورية_بحر_بشرة_بلون_فاتح:'},'\U0001F9DC\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':mermaid_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':meerjungfrau_mitteldunkle_hautfarbe:','es':':sirena_tono_de_piel_oscuro_medio:','fr':':sirène_peau_mate:','ja':':マーメイド_やや濃い肌色:','ko':':여자_인어_진한_갈색_피부:','pt':':sereia_pele_morena_escura:','it':':sirena_donna_carnagione_abbastanza_scura:','fa':':پری_دریایی_پوست_گندمی:','id':':putri_duyung_warna_kulit_gelap-sedang:','zh':':美人鱼_中等深肤色:','ru':':женщина-русалка_темныи_тон_кожи:','tr':':denizkızı_orta_koyu_cilt_tonu:','ar':':حورية_بحر_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9DC\U0001F3FE\U0000200D\U00002640':{'en':':mermaid_medium-dark_skin_tone:','status':minimally_qualified,'E':5,'de':':meerjungfrau_mitteldunkle_hautfarbe:','es':':sirena_tono_de_piel_oscuro_medio:','fr':':sirène_peau_mate:','ja':':マーメイド_やや濃い肌色:','ko':':여자_인어_진한_갈색_피부:','pt':':sereia_pele_morena_escura:','it':':sirena_donna_carnagione_abbastanza_scura:','fa':':پری_دریایی_پوست_گندمی:','id':':putri_duyung_warna_kulit_gelap-sedang:','zh':':美人鱼_中等深肤色:','ru':':женщина_русалка_темный_тон_кожи:','tr':':denizkızı_orta_koyu_cilt_tonu:','ar':':حورية_بحر_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9DC\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':mermaid_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':meerjungfrau_mittelhelle_hautfarbe:','es':':sirena_tono_de_piel_claro_medio:','fr':':sirène_peau_moyennement_claire:','ja':':マーメイド_やや薄い肌色:','ko':':여자_인어_연한_갈색_피부:','pt':':sereia_pele_morena_clara:','it':':sirena_donna_carnagione_abbastanza_chiara:','fa':':پری_دریایی_پوست_روشن:','id':':putri_duyung_warna_kulit_cerah-sedang:','zh':':美人鱼_中等浅肤色:','ru':':женщина-русалка_светлыи_тон_кожи:','tr':':denizkızı_orta_açık_cilt_tonu:','ar':':حورية_بحر_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9DC\U0001F3FC\U0000200D\U00002640':{'en':':mermaid_medium-light_skin_tone:','status':minimally_qualified,'E':5,'de':':meerjungfrau_mittelhelle_hautfarbe:','es':':sirena_tono_de_piel_claro_medio:','fr':':sirène_peau_moyennement_claire:','ja':':マーメイド_やや薄い肌色:','ko':':여자_인어_연한_갈색_피부:','pt':':sereia_pele_morena_clara:','it':':sirena_donna_carnagione_abbastanza_chiara:','fa':':پری_دریایی_پوست_روشن:','id':':putri_duyung_warna_kulit_cerah-sedang:','zh':':美人鱼_中等浅肤色:','ru':':женщина_русалка_светлый_тон_кожи:','tr':':denizkızı_orta_açık_cilt_tonu:','ar':':حورية_بحر_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9DC\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':mermaid_medium_skin_tone:','status':fully_qualified,'E':5,'de':':meerjungfrau_mittlere_hautfarbe:','es':':sirena_tono_de_piel_medio:','fr':':sirène_peau_légèrement_mate:','ja':':マーメイド_中間の肌色:','ko':':여자_인어_갈색_피부:','pt':':sereia_pele_morena:','it':':sirena_donna_carnagione_olivastra:','fa':':پری_دریایی_پوست_طلایی:','id':':putri_duyung_warna_kulit_sedang:','zh':':美人鱼_中等肤色:','ru':':женщина-русалка_среднии_тон_кожи:','tr':':denizkızı_orta_cilt_tonu:','ar':':حورية_بحر_بشرة_بلون_معتدل:'},'\U0001F9DC\U0001F3FD\U0000200D\U00002640':{'en':':mermaid_medium_skin_tone:','status':minimally_qualified,'E':5,'de':':meerjungfrau_mittlere_hautfarbe:','es':':sirena_tono_de_piel_medio:','fr':':sirène_peau_légèrement_mate:','ja':':マーメイド_中間の肌色:','ko':':여자_인어_갈색_피부:','pt':':sereia_pele_morena:','it':':sirena_donna_carnagione_olivastra:','fa':':پری_دریایی_پوست_طلایی:','id':':putri_duyung_warna_kulit_sedang:','zh':':美人鱼_中等肤色:','ru':':женщина_русалка_средний_тон_кожи:','tr':':denizkızı_orta_cilt_tonu:','ar':':حورية_بحر_بشرة_بلون_معتدل:'},'\U0001F9DC\U0000200D\U00002642\U0000FE0F':{'en':':merman:','status':fully_qualified,'E':5,'de':':wassermann:','es':':sirena_hombre:','fr':':triton:','ja':':マーマン:','ko':':남자_인어:','pt':':sereio:','it':':tritone:','fa':':دریامرد:','id':':pria_duyung:','zh':':男人鱼:','ru':':тритон:','tr':':balık_adam:','ar':':مخلوق_له_جسد_رجل_وذيل_سمكة:'},'\U0001F9DC\U0000200D\U00002642':{'en':':merman:','status':minimally_qualified,'E':5,'de':':wassermann:','es':':sirena_hombre:','fr':':triton:','ja':':マーマン:','ko':':남자_인어:','pt':':sereio:','it':':tritone:','fa':':دریامرد:','id':':pria_duyung:','zh':':男人鱼:','ru':':тритон:','tr':':balık_adam:','ar':':مخلوق_له_جسد_رجل_وذيل_سمكة:'},'\U0001F9DC\U0001F3FF\U0000200D\U00002642\U0000FE0F':{'en':':merman_dark_skin_tone:','status':fully_qualified,'E':5,'de':':wassermann_dunkle_hautfarbe:','es':':sirena_hombre_tono_de_piel_oscuro:','fr':':triton_peau_foncée:','ja':':マーマン_濃い肌色:','ko':':남자_인어_검은색_피부:','pt':':sereio_pele_escura:','it':':tritone_carnagione_scura:','fa':':دریامرد_پوست_آبنوسی:','id':':pria_duyung_warna_kulit_gelap:','zh':':男人鱼_较深肤色:','ru':':тритон_очень_темныи_тон_кожи:','tr':':balık_adam_koyu_cilt_tonu:','ar':':مخلوق_له_جسد_رجل_وذيل_سمكة_بشرة_بلون_غامق:'},'\U0001F9DC\U0001F3FF\U0000200D\U00002642':{'en':':merman_dark_skin_tone:','status':minimally_qualified,'E':5,'de':':wassermann_dunkle_hautfarbe:','es':':sirena_hombre_tono_de_piel_oscuro:','fr':':triton_peau_foncée:','ja':':マーマン_濃い肌色:','ko':':남자_인어_검은색_피부:','pt':':sereio_pele_escura:','it':':tritone_carnagione_scura:','fa':':دریامرد_پوست_آبنوسی:','id':':pria_duyung_warna_kulit_gelap:','zh':':男人鱼_较深肤色:','ru':':тритон_очень_темный_тон_кожи:','tr':':balık_adam_koyu_cilt_tonu:','ar':':مخلوق_له_جسد_رجل_وذيل_سمكة_بشرة_بلون_غامق:'},'\U0001F9DC\U0001F3FB\U0000200D\U00002642\U0000FE0F':{'en':':merman_light_skin_tone:','status':fully_qualified,'E':5,'de':':wassermann_helle_hautfarbe:','es':':sirena_hombre_tono_de_piel_claro:','fr':':triton_peau_claire:','ja':':マーマン_薄い肌色:','ko':':남자_인어_하얀_피부:','pt':':sereio_pele_clara:','it':':tritone_carnagione_chiara:','fa':':دریامرد_پوست_سفید:','id':':pria_duyung_warna_kulit_cerah:','zh':':男人鱼_较浅肤色:','ru':':тритон_очень_светлыи_тон_кожи:','tr':':balık_adam_açık_cilt_tonu:','ar':':مخلوق_له_جسد_رجل_وذيل_سمكة_بشرة_بلون_فاتح:'},'\U0001F9DC\U0001F3FB\U0000200D\U00002642':{'en':':merman_light_skin_tone:','status':minimally_qualified,'E':5,'de':':wassermann_helle_hautfarbe:','es':':sirena_hombre_tono_de_piel_claro:','fr':':triton_peau_claire:','ja':':マーマン_薄い肌色:','ko':':남자_인어_하얀_피부:','pt':':sereio_pele_clara:','it':':tritone_carnagione_chiara:','fa':':دریامرد_پوست_سفید:','id':':pria_duyung_warna_kulit_cerah:','zh':':男人鱼_较浅肤色:','ru':':тритон_очень_светлый_тон_кожи:','tr':':balık_adam_açık_cilt_tonu:','ar':':مخلوق_له_جسد_رجل_وذيل_سمكة_بشرة_بلون_فاتح:'},'\U0001F9DC\U0001F3FE\U0000200D\U00002642\U0000FE0F':{'en':':merman_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':wassermann_mitteldunkle_hautfarbe:','es':':sirena_hombre_tono_de_piel_oscuro_medio:','fr':':triton_peau_mate:','ja':':マーマン_やや濃い肌色:','ko':':남자_인어_진한_갈색_피부:','pt':':sereio_pele_morena_escura:','it':':tritone_carnagione_abbastanza_scura:','fa':':دریامرد_پوست_گندمی:','id':':pria_duyung_warna_kulit_gelap-sedang:','zh':':男人鱼_中等深肤色:','ru':':тритон_темныи_тон_кожи:','tr':':balık_adam_orta_koyu_cilt_tonu:','ar':':مخلوق_له_جسد_رجل_وذيل_سمكة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9DC\U0001F3FE\U0000200D\U00002642':{'en':':merman_medium-dark_skin_tone:','status':minimally_qualified,'E':5,'de':':wassermann_mitteldunkle_hautfarbe:','es':':sirena_hombre_tono_de_piel_oscuro_medio:','fr':':triton_peau_mate:','ja':':マーマン_やや濃い肌色:','ko':':남자_인어_진한_갈색_피부:','pt':':sereio_pele_morena_escura:','it':':tritone_carnagione_abbastanza_scura:','fa':':دریامرد_پوست_گندمی:','id':':pria_duyung_warna_kulit_gelap-sedang:','zh':':男人鱼_中等深肤色:','ru':':тритон_темный_тон_кожи:','tr':':balık_adam_orta_koyu_cilt_tonu:','ar':':مخلوق_له_جسد_رجل_وذيل_سمكة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9DC\U0001F3FC\U0000200D\U00002642\U0000FE0F':{'en':':merman_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':wassermann_mittelhelle_hautfarbe:','es':':sirena_hombre_tono_de_piel_claro_medio:','fr':':triton_peau_moyennement_claire:','ja':':マーマン_やや薄い肌色:','ko':':남자_인어_연한_갈색_피부:','pt':':sereio_pele_morena_clara:','it':':tritone_carnagione_abbastanza_chiara:','fa':':دریامرد_پوست_روشن:','id':':pria_duyung_warna_kulit_cerah-sedang:','zh':':男人鱼_中等浅肤色:','ru':':тритон_светлыи_тон_кожи:','tr':':balık_adam_orta_açık_cilt_tonu:','ar':':مخلوق_له_جسد_رجل_وذيل_سمكة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9DC\U0001F3FC\U0000200D\U00002642':{'en':':merman_medium-light_skin_tone:','status':minimally_qualified,'E':5,'de':':wassermann_mittelhelle_hautfarbe:','es':':sirena_hombre_tono_de_piel_claro_medio:','fr':':triton_peau_moyennement_claire:','ja':':マーマン_やや薄い肌色:','ko':':남자_인어_연한_갈색_피부:','pt':':sereio_pele_morena_clara:','it':':tritone_carnagione_abbastanza_chiara:','fa':':دریامرد_پوست_روشن:','id':':pria_duyung_warna_kulit_cerah-sedang:','zh':':男人鱼_中等浅肤色:','ru':':тритон_светлый_тон_кожи:','tr':':balık_adam_orta_açık_cilt_tonu:','ar':':مخلوق_له_جسد_رجل_وذيل_سمكة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9DC\U0001F3FD\U0000200D\U00002642\U0000FE0F':{'en':':merman_medium_skin_tone:','status':fully_qualified,'E':5,'de':':wassermann_mittlere_hautfarbe:','es':':sirena_hombre_tono_de_piel_medio:','fr':':triton_peau_légèrement_mate:','ja':':マーマン_中間の肌色:','ko':':남자_인어_갈색_피부:','pt':':sereio_pele_morena:','it':':tritone_carnagione_olivastra:','fa':':دریامرد_پوست_طلایی:','id':':pria_duyung_warna_kulit_sedang:','zh':':男人鱼_中等肤色:','ru':':тритон_среднии_тон_кожи:','tr':':balık_adam_orta_cilt_tonu:','ar':':مخلوق_له_جسد_رجل_وذيل_سمكة_بشرة_بلون_معتدل:'},'\U0001F9DC\U0001F3FD\U0000200D\U00002642':{'en':':merman_medium_skin_tone:','status':minimally_qualified,'E':5,'de':':wassermann_mittlere_hautfarbe:','es':':sirena_hombre_tono_de_piel_medio:','fr':':triton_peau_légèrement_mate:','ja':':マーマン_中間の肌色:','ko':':남자_인어_갈색_피부:','pt':':sereio_pele_morena:','it':':tritone_carnagione_olivastra:','fa':':دریامرد_پوست_طلایی:','id':':pria_duyung_warna_kulit_sedang:','zh':':男人鱼_中等肤色:','ru':':тритон_средний_тон_кожи:','tr':':balık_adam_orta_cilt_tonu:','ar':':مخلوق_له_جسد_رجل_وذيل_سمكة_بشرة_بلون_معتدل:'},'\U0001F9DC':{'en':':merperson:','status':fully_qualified,'E':5,'de':':wassermensch:','es':':persona_sirena:','fr':':créature_aquatique:','ja':':人魚:','ko':':인어:','pt':':pessoa_sereia:','it':':sirena:','fa':':دریامردمان:','id':':orang_duyung:','zh':':人鱼:','ru':':русалка:','tr':':balık_insan:','ar':':مخلوق_بحري_بجسد_بشري_وذيل_سمكة:'},'\U0001F9DC\U0001F3FF':{'en':':merperson_dark_skin_tone:','status':fully_qualified,'E':5,'de':':wassermensch_dunkle_hautfarbe:','es':':persona_sirena_tono_de_piel_oscuro:','fr':':créature_aquatique_peau_foncée:','ja':':人魚_濃い肌色:','ko':':인어_검은색_피부:','pt':':pessoa_sereia_pele_escura:','it':':sirena_carnagione_scura:','fa':':دریامردمان_پوست_آبنوسی:','id':':orang_duyung_warna_kulit_gelap:','zh':':人鱼_较深肤色:','ru':':русалка_очень_темный_тон_кожи:','tr':':balık_insan_koyu_cilt_tonu:','ar':':مخلوق_بحري_بجسد_بشري_وذيل_سمكة_بشرة_بلون_غامق:'},'\U0001F9DC\U0001F3FB':{'en':':merperson_light_skin_tone:','status':fully_qualified,'E':5,'de':':wassermensch_helle_hautfarbe:','es':':persona_sirena_tono_de_piel_claro:','fr':':créature_aquatique_peau_claire:','ja':':人魚_薄い肌色:','ko':':인어_하얀_피부:','pt':':pessoa_sereia_pele_clara:','it':':sirena_carnagione_chiara:','fa':':دریامردمان_پوست_سفید:','id':':orang_duyung_warna_kulit_cerah:','zh':':人鱼_较浅肤色:','ru':':русалка_очень_светлый_тон_кожи:','tr':':balık_insan_açık_cilt_tonu:','ar':':مخلوق_بحري_بجسد_بشري_وذيل_سمكة_بشرة_بلون_فاتح:'},'\U0001F9DC\U0001F3FE':{'en':':merperson_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':wassermensch_mitteldunkle_hautfarbe:','es':':persona_sirena_tono_de_piel_oscuro_medio:','fr':':créature_aquatique_peau_mate:','ja':':人魚_やや濃い肌色:','ko':':인어_진한_갈색_피부:','pt':':pessoa_sereia_pele_morena_escura:','it':':sirena_carnagione_abbastanza_scura:','fa':':دریامردمان_پوست_گندمی:','id':':orang_duyung_warna_kulit_gelap-sedang:','zh':':人鱼_中等深肤色:','ru':':русалка_темный_тон_кожи:','tr':':balık_insan_orta_koyu_cilt_tonu:','ar':':مخلوق_بحري_بجسد_بشري_وذيل_سمكة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9DC\U0001F3FC':{'en':':merperson_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':wassermensch_mittelhelle_hautfarbe:','es':':persona_sirena_tono_de_piel_claro_medio:','fr':':créature_aquatique_peau_moyennement_claire:','ja':':人魚_やや薄い肌色:','ko':':인어_연한_갈색_피부:','pt':':pessoa_sereia_pele_morena_clara:','it':':sirena_carnagione_abbastanza_chiara:','fa':':دریامردمان_پوست_روشن:','id':':orang_duyung_warna_kulit_cerah-sedang:','zh':':人鱼_中等浅肤色:','ru':':русалка_светлый_тон_кожи:','tr':':balık_insan_orta_açık_cilt_tonu:','ar':':مخلوق_بحري_بجسد_بشري_وذيل_سمكة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9DC\U0001F3FD':{'en':':merperson_medium_skin_tone:','status':fully_qualified,'E':5,'de':':wassermensch_mittlere_hautfarbe:','es':':persona_sirena_tono_de_piel_medio:','fr':':créature_aquatique_peau_légèrement_mate:','ja':':人魚_中間の肌色:','ko':':인어_갈색_피부:','pt':':pessoa_sereia_pele_morena:','it':':sirena_carnagione_olivastra:','fa':':دریامردمان_پوست_طلایی:','id':':orang_duyung_warna_kulit_sedang:','zh':':人鱼_中等肤色:','ru':':русалка_средний_тон_кожи:','tr':':balık_insan_orta_cilt_tonu:','ar':':مخلوق_بحري_بجسد_بشري_وذيل_سمكة_بشرة_بلون_معتدل:'},'\U0001F687':{'en':':metro:','status':fully_qualified,'E':0.6,'variant':True,'de':':u-bahn:','es':':metro:','fr':':métro:','ja':':地下鉄:','ko':':지하철:','pt':':metrô:','it':':metropolitana:','fa':':قطار_شهری:','id':':kereta_bawah_tanah:','zh':':地铁:','ru':':метро:','tr':':metro:','ar':':مترو:'},'\U0001F9A0':{'en':':microbe:','status':fully_qualified,'E':11,'de':':mikrobe:','es':':microbio:','fr':':microbe:','ja':':微生物:','ko':':미생물:','pt':':micróbio:','it':':microbo:','fa':':میکروب:','id':':mikroba:','zh':':细菌:','ru':':микроб:','tr':':mikrop:','ar':':جرثومة:'},'\U0001F3A4':{'en':':microphone:','status':fully_qualified,'E':0.6,'de':':mikrofon:','es':':micrófono:','fr':':micro:','ja':':マイク:','ko':':마이크:','pt':':microfone:','it':':microfono:','fa':':میکروفون:','id':':mikrofon:','zh':':麦克风:','ru':':микрофон:','tr':':mikrofon:','ar':':ميكروفون:'},'\U0001F52C':{'en':':microscope:','status':fully_qualified,'E':1,'de':':mikroskop:','es':':microscopio:','fr':':microscope:','ja':':顕微鏡:','ko':':현미경:','pt':':microscópio:','it':':microscopio:','fa':':میکروسکوپ:','id':':mikroskop:','zh':':显微镜:','ru':':микроскоп:','tr':':mikroskop:','ar':':ميكروسكوب:'},'\U0001F595':{'en':':middle_finger:','status':fully_qualified,'E':1,'alias':[':fu:', ':reversed_hand_with_middle_finger_extended:'],'de':':mittelfinger:','es':':dedo_corazón_hacia_arriba:','fr':':doigt_d’honneur:','ja':':立てた中指:','ko':':중지:','pt':':dedo_do_meio:','it':':dito_medio:','fa':':انگشت_میانه:','id':':jari_tengah:','zh':':竖中指:','ru':':средний_палец:','tr':':orta_parmak:','ar':':إصبع_الوسطى:'},'\U0001F595\U0001F3FF':{'en':':middle_finger_dark_skin_tone:','status':fully_qualified,'E':1,'de':':mittelfinger_dunkle_hautfarbe:','es':':dedo_corazón_hacia_arriba_tono_de_piel_oscuro:','fr':':doigt_d’honneur_peau_foncée:','ja':':立てた中指_濃い肌色:','ko':':중지_검은색_피부:','pt':':dedo_do_meio_pele_escura:','it':':dito_medio_carnagione_scura:','fa':':انگشت_میانه_پوست_آبنوسی:','id':':jari_tengah_warna_kulit_gelap:','zh':':竖中指_较深肤色:','ru':':средний_палец_очень_темный_тон_кожи:','tr':':orta_parmak_koyu_cilt_tonu:','ar':':إصبع_الوسطى_بشرة_بلون_غامق:'},'\U0001F595\U0001F3FB':{'en':':middle_finger_light_skin_tone:','status':fully_qualified,'E':1,'de':':mittelfinger_helle_hautfarbe:','es':':dedo_corazón_hacia_arriba_tono_de_piel_claro:','fr':':doigt_d’honneur_peau_claire:','ja':':立てた中指_薄い肌色:','ko':':중지_하얀_피부:','pt':':dedo_do_meio_pele_clara:','it':':dito_medio_carnagione_chiara:','fa':':انگشت_میانه_پوست_سفید:','id':':jari_tengah_warna_kulit_cerah:','zh':':竖中指_较浅肤色:','ru':':средний_палец_очень_светлый_тон_кожи:','tr':':orta_parmak_açık_cilt_tonu:','ar':':إصبع_الوسطى_بشرة_بلون_فاتح:'},'\U0001F595\U0001F3FE':{'en':':middle_finger_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':mittelfinger_mitteldunkle_hautfarbe:','es':':dedo_corazón_hacia_arriba_tono_de_piel_oscuro_medio:','fr':':doigt_d’honneur_peau_mate:','ja':':立てた中指_やや濃い肌色:','ko':':중지_진한_갈색_피부:','pt':':dedo_do_meio_pele_morena_escura:','it':':dito_medio_carnagione_abbastanza_scura:','fa':':انگشت_میانه_پوست_گندمی:','id':':jari_tengah_warna_kulit_gelap-sedang:','zh':':竖中指_中等深肤色:','ru':':средний_палец_темный_тон_кожи:','tr':':orta_parmak_orta_koyu_cilt_tonu:','ar':':إصبع_الوسطى_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F595\U0001F3FC':{'en':':middle_finger_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':mittelfinger_mittelhelle_hautfarbe:','es':':dedo_corazón_hacia_arriba_tono_de_piel_claro_medio:','fr':':doigt_d’honneur_peau_moyennement_claire:','ja':':立てた中指_やや薄い肌色:','ko':':중지_연한_갈색_피부:','pt':':dedo_do_meio_pele_morena_clara:','it':':dito_medio_carnagione_abbastanza_chiara:','fa':':انگشت_میانه_پوست_روشن:','id':':jari_tengah_warna_kulit_cerah-sedang:','zh':':竖中指_中等浅肤色:','ru':':средний_палец_светлый_тон_кожи:','tr':':orta_parmak_orta_açık_cilt_tonu:','ar':':إصبع_الوسطى_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F595\U0001F3FD':{'en':':middle_finger_medium_skin_tone:','status':fully_qualified,'E':1,'de':':mittelfinger_mittlere_hautfarbe:','es':':dedo_corazón_hacia_arriba_tono_de_piel_medio:','fr':':doigt_d’honneur_peau_légèrement_mate:','ja':':立てた中指_中間の肌色:','ko':':중지_갈색_피부:','pt':':dedo_do_meio_pele_morena:','it':':dito_medio_carnagione_olivastra:','fa':':انگشت_میانه_پوست_طلایی:','id':':jari_tengah_warna_kulit_sedang:','zh':':竖中指_中等肤色:','ru':':средний_палец_средний_тон_кожи:','tr':':orta_parmak_orta_cilt_tonu:','ar':':إصبع_الوسطى_بشرة_بلون_معتدل:'},'\U0001FA96':{'en':':military_helmet:','status':fully_qualified,'E':13,'de':':militärhelm:','es':':casco_militar:','fr':':casque_militaire:','ja':':軍用ヘルメット:','ko':':군용_헬멧:','pt':':capacete_militar:','it':':elmetto_militare:','fa':':کلاه_خود:','id':':helm_tentara:','zh':':军用头盔:','ru':':армейская_каска:','tr':':asker_kaskı:','ar':':خوذة_عسكرية:'},'\U0001F396\U0000FE0F':{'en':':military_medal:','status':fully_qualified,'E':0.7,'alias':[':medal_military:'],'variant':True,'de':':militärorden:','es':':medalla_militar:','fr':':médaille_militaire:','ja':':勲章:','ko':':무공_훈장:','pt':':medalha_militar:','it':':medaglia_militare:','fa':':مدال_ارتشی:','id':':medali_militer:','zh':':军功章:','ru':':военная_медаль:','tr':':askeri_madalya:','ar':':ميدالية_عسكرية:'},'\U0001F396':{'en':':military_medal:','status':unqualified,'E':0.7,'alias':[':medal_military:'],'variant':True,'de':':militärorden:','es':':medalla_militar:','fr':':médaille_militaire:','ja':':勲章:','ko':':무공_훈장:','pt':':medalha_militar:','it':':medaglia_militare:','fa':':مدال_ارتشی:','id':':medali_militer:','zh':':军功章:','ru':':военная_медаль:','tr':':askeri_madalya:','ar':':ميدالية_عسكرية:'},'\U0001F30C':{'en':':milky_way:','status':fully_qualified,'E':0.6,'de':':milchstraße:','es':':vía_láctea:','fr':':voie_lactée:','ja':':天の川:','ko':':은하수:','pt':':via_láctea:','it':':via_lattea:','fa':':راه_شیری:','id':':bima_sakti:','zh':':银河:','ru':':Млечный_Путь:','tr':':samanyolu:','ar':':درب_التبانة:'},'\U0001F690':{'en':':minibus:','status':fully_qualified,'E':1,'de':':kleinbus:','es':':minibús:','fr':':minibus:','ja':':マイクロバス:','ko':':미니버스:','pt':':van:','it':':pulmino:','fa':':مینی_بوس:','id':':minibus:','zh':':小巴:','ru':':микроавтобус:','tr':':minibüs:','ar':':حافلة_صغيرة:'},'\U00002796':{'en':':minus:','status':fully_qualified,'E':0.6,'alias':[':heavy_minus_sign:'],'variant':True,'de':':minus:','es':':menos:','fr':':moins:','ja':':マイナス:','ko':':빼기:','pt':':símbolo_de_subtração:','it':':sottrazione:','fa':':تفریق:','id':':kurang:','zh':':减:','ru':':знак_вычитания:','tr':':eksi:','ar':':علامة_طرح:'},'\U0001FA9E':{'en':':mirror:','status':fully_qualified,'E':13,'de':':spiegel:','es':':espejo:','fr':':miroir:','ja':':鏡:','ko':':거울:','pt':':espelho:','it':':specchio:','fa':':آینه:','id':':cermin:','zh':':镜子:','ru':':зеркало:','tr':':ayna:','ar':':مرآة:'},'\U0001FAA9':{'en':':mirror_ball:','status':fully_qualified,'E':14,'de':':discokugel:','es':':bola_de_espejos:','fr':':boule_à_facettes:','ja':':ミラーボール:','ko':':미러볼:','pt':':globo_de_espelhos:','it':':palla_da_discoteca:','fa':':توپ_آینه_ای:','id':':bola_disko:','zh':':镜球:','ru':':диско_шар:','tr':':aynalı_küre:','ar':':الكرة_المضيئة:'},'\U0001F5FF':{'en':':moai:','status':fully_qualified,'E':0.6,'alias':[':moyai:'],'de':':statue:','es':':estatua_moái:','fr':':moai:','ja':':モアイ:','ko':':모아이:','pt':':moai:','it':':moai:','fa':':سردیس_موآی:','id':':moai:','zh':':摩埃:','ru':':истукан:','tr':':moyai_heykeli:','ar':':تمثال:'},'\U0001F4F1':{'en':':mobile_phone:','status':fully_qualified,'E':0.6,'alias':[':iphone:'],'de':':mobiltelefon:','es':':teléfono_móvil:','fr':':téléphone_portable:','ja':':携帯電話:','ko':':휴대전화:','pt':':telefone_celular:','it':':telefono_cellulare:','fa':':تلفن_همراه:','id':':ponsel:','zh':':手机:','ru':':мобильный_телефон:','tr':':cep_telefonu:','ar':':هاتف_جوال:'},'\U0001F4F4':{'en':':mobile_phone_off:','status':fully_qualified,'E':0.6,'de':':mobiltelefon_aus:','es':':teléfono_móvil_apagado:','fr':':téléphone_éteint:','ja':':携帯電話電源オフ:','ko':':휴대전화_끄기:','pt':':telefone_celular_desligado:','it':':cellulare_spento:','fa':':تلفن_همراه_خاموش:','id':':ponsel_nonaktif:','zh':':手机关机:','ru':':выключенный_телефон:','tr':':cep_telefonu_kapalı:','ar':':إغلاق_الجوال:'},'\U0001F4F2':{'en':':mobile_phone_with_arrow:','status':fully_qualified,'E':0.6,'alias':[':calling:'],'de':':mobiltelefon_mit_pfeil:','es':':móvil_con_una_flecha:','fr':':appel_entrant:','ja':':着信中:','ko':':왼쪽에_화살표가_있는_휴대전화:','pt':':telefone_celular_com_seta:','it':':telefono_cellulare_con_freccia:','fa':':تلفن_همراه_با_پیکان:','id':':telepon_masuk:','zh':':带有箭头的手机:','ru':':входящий_вызов:','tr':':ok_işaretli_cep_telefonu:','ar':':هاتف_جوال_مع_سهم:'},'\U0001F911':{'en':':money-mouth_face:','status':fully_qualified,'E':1,'alias':[':money_mouth_face:'],'de':':gesicht_mit_dollarzeichen:','es':':cara_con_lengua_de_dinero:','fr':':argent_dans_les_yeux_et_la_bouche:','ja':':お金の顔:','ko':':돈_모양의_입이_있는_얼굴:','pt':':rosto_com_cifrões:','it':':faccina_avida_di_denaro:','fa':':صورت_پول_دوست:','id':':wajah_materialistis:','zh':':发财:','ru':':деньги_в_глазах:','tr':':para_ağızlı_yüz:','ar':':وجه_بلسان_من_مال:'},'\U0001F4B0':{'en':':money_bag:','status':fully_qualified,'E':0.6,'alias':[':moneybag:'],'variant':True,'de':':geldsack:','es':':bolsa_de_dinero:','fr':':sac_plein_d’argent:','ja':':ドル袋:','ko':':돈주머니:','pt':':saco_de_dinheiro:','it':':sacco_di_soldi:','fa':':کیسهٔ_پول:','id':':kantung_uang:','zh':':钱袋:','ru':':мешок_денег:','tr':':para_torbası:','ar':':حقيبة_نقود:'},'\U0001F4B8':{'en':':money_with_wings:','status':fully_qualified,'E':0.6,'de':':geldschein_mit_flügeln:','es':':billete_con_alas:','fr':':billet_avec_des_ailes:','ja':':羽が生えたお金:','ko':':날개_달린_돈:','pt':':dinheiro_voando:','it':':soldi_con_le_ali:','fa':':اسکناس_بال_دار:','id':':uang_bersayap:','zh':':长翅膀的钱:','ru':':банкнота_с_крылышками:','tr':':kanatlı_para:','ar':':أموال_تطير:'},'\U0001F412':{'en':':monkey:','status':fully_qualified,'E':0.6,'de':':affe:','es':':mono:','fr':':singe:','ja':':サル:','ko':':원숭이:','pt':':macaco:','it':':scimmia:','fa':':میمون:','id':':monyet:','zh':':猴子:','ru':':обезьяна:','tr':':maymun:','ar':':قرد:'},'\U0001F435':{'en':':monkey_face:','status':fully_qualified,'E':0.6,'de':':affengesicht:','es':':cara_de_mono:','fr':':tête_de_singe:','ja':':サルの顔:','ko':':원숭이_얼굴:','pt':':rosto_de_macaco:','it':':muso_di_scimmia:','fa':':صورت_میمون:','id':':wajah_monyet:','zh':':猴头:','ru':':морда_обезьяны:','tr':':maymun_yüzü:','ar':':وجه_قرد:'},'\U0001F69D':{'en':':monorail:','status':fully_qualified,'E':1,'de':':einschienenbahn:','es':':monorraíl:','fr':':monorail:','ja':':モノレール:','ko':':모노레일:','pt':':monotrilho:','it':':monorotaia:','fa':':تک_ریل:','id':':monorel:','zh':':单轨:','ru':':монорельс:','tr':':monoray:','ar':':خط_سكة_حديد_أحادي:'},'\U0001F96E':{'en':':moon_cake:','status':fully_qualified,'E':11,'de':':mondkuchen:','es':':pastel_de_luna:','fr':':gâteau_de_lune:','ja':':月餅:','ko':':월병:','pt':':bolo_lunar:','it':':torta_della_luna:','fa':':کیک_ماه:','id':':kue_bulan:','zh':':月饼:','ru':':лунный_пирог:','tr':':ay_pastası:','ar':':كعكة_القمر:'},'\U0001F391':{'en':':moon_viewing_ceremony:','status':fully_qualified,'E':0.6,'alias':[':rice_scene:'],'de':':traditionelles_mondfest:','es':':ceremonia_de_contemplación_de_la_luna:','fr':':cérémonie_de_la_lune:','ja':':月見:','ko':':달맞이:','pt':':contemplação_da_lua:','it':':contemplazione_della_luna:','fa':':آئین_مشاهدهٔ_ماه_کامل:','id':':perayaan_bulan:','zh':':赏月:','ru':':праздник_Луны:','tr':':ay_izleme:','ar':':احتفال_برؤية_القمر:'},'\U0001FACE':{'en':':moose:','status':fully_qualified,'E':15,'de':':elch:','es':':alce:','fr':':élan:','ja':':ヘラジカ:','ko':':무스:','pt':':alce:','it':':alce:','fa':':موس:','id':':rusa_besar:','zh':':驼鹿:','ru':':лось:','tr':':sığın:','ar':':أيل:'},'\U0001F54C':{'en':':mosque:','status':fully_qualified,'E':1,'de':':moschee:','es':':mezquita:','fr':':mosquée:','ja':':モスク:','ko':':모스크:','pt':':mesquita:','it':':moschea:','fa':':مسجد:','id':':masjid:','zh':':清真寺:','ru':':мечеть:','tr':':cami:','ar':':مسجد:'},'\U0001F99F':{'en':':mosquito:','status':fully_qualified,'E':11,'de':':mücke:','es':':mosquito:','fr':':moustique:','ja':':蚊:','ko':':모기:','pt':':mosquito:','it':':zanzara:','fa':':پشه:','id':':nyamuk:','zh':':蚊子:','ru':':комар:','tr':':sivrisinek:','ar':':بعوضة:'},'\U0001F6E5\U0000FE0F':{'en':':motor_boat:','status':fully_qualified,'E':0.7,'variant':True,'de':':motorboot:','es':':barco_a_motor:','fr':':bateau_à_moteur:','ja':':モーターボート:','ko':':모터보트:','pt':':barco:','it':':barca_a_motore:','fa':':قایق_موتوری:','id':':perahu_bermotor:','zh':':摩托艇:','ru':':моторная_лодка:','tr':':motorlu_tekne:','ar':':زورق_بخاري:'},'\U0001F6E5':{'en':':motor_boat:','status':unqualified,'E':0.7,'variant':True,'de':':motorboot:','es':':barco_a_motor:','fr':':bateau_à_moteur:','ja':':モーターボート:','ko':':모터보트:','pt':':barco:','it':':barca_a_motore:','fa':':قایق_موتوری:','id':':perahu_bermotor:','zh':':摩托艇:','ru':':моторная_лодка:','tr':':motorlu_tekne:','ar':':زورق_بخاري:'},'\U0001F6F5':{'en':':motor_scooter:','status':fully_qualified,'E':3,'de':':motorroller:','es':':scooter:','fr':':scooter:','ja':':スクーター:','ko':':스쿠터:','pt':':scooter:','it':':scooter:','fa':':اسکوتر:','id':':motor_skuter:','zh':':小型摩托车:','ru':':скутер:','tr':':motorlu_skuter:','ar':':سكوتر_بمحرك:'},'\U0001F3CD\U0000FE0F':{'en':':motorcycle:','status':fully_qualified,'E':0.7,'alias':[':racing_motorcycle:'],'variant':True,'de':':motorrad:','es':':moto:','fr':':moto:','ja':':オートバイ:','ko':':오토바이:','pt':':motocicleta:','it':':motocicletta:','fa':':موتورسیکلت:','id':':sepeda_motor:','zh':':摩托车:','ru':':мотоцикл:','tr':':motosiklet:','ar':':دراجة_بخارية:'},'\U0001F3CD':{'en':':motorcycle:','status':unqualified,'E':0.7,'alias':[':racing_motorcycle:'],'variant':True,'de':':motorrad:','es':':moto:','fr':':moto:','ja':':オートバイ:','ko':':오토바이:','pt':':motocicleta:','it':':motocicletta:','fa':':موتورسیکلت:','id':':sepeda_motor:','zh':':摩托车:','ru':':мотоцикл:','tr':':motosiklet:','ar':':دراجة_بخارية:'},'\U0001F9BC':{'en':':motorized_wheelchair:','status':fully_qualified,'E':12,'de':':elektrischer_rollstuhl:','es':':silla_de_ruedas_eléctrica:','fr':':fauteuil_motorisé:','ja':':電動車椅子:','ko':':전동_휠체어:','pt':':cadeira_de_rodas_motorizada:','it':':sedia_a_rotelle_motorizzata:','fa':':صندلی_چرخ_دار_برقی:','id':':kursi_roda_bermotor:','zh':':电动轮椅:','ru':':кресло_коляска_с_мотором:','tr':':elektrikli_tekerlekli_sandalye:','ar':':كرسي_متحرك_كهربائي:'},'\U0001F6E3\U0000FE0F':{'en':':motorway:','status':fully_qualified,'E':0.7,'variant':True,'de':':autobahn:','es':':autopista:','fr':':autoroute:','ja':':高速道路:','ko':':고속도로:','pt':':estrada:','it':':autostrada:','fa':':بزرگراه:','id':':jalan_raya:','zh':':高速公路:','ru':':шоссе:','tr':':otoban:','ar':':طريق_سريع:'},'\U0001F6E3':{'en':':motorway:','status':unqualified,'E':0.7,'variant':True,'de':':autobahn:','es':':autopista:','fr':':autoroute:','ja':':高速道路:','ko':':고속도로:','pt':':estrada:','it':':autostrada:','fa':':بزرگراه:','id':':jalan_raya:','zh':':高速公路:','ru':':шоссе:','tr':':otoban:','ar':':طريق_سريع:'},'\U0001F5FB':{'en':':mount_fuji:','status':fully_qualified,'E':0.6,'de':':fuji:','es':':monte_fuji:','fr':':mont_fuji:','ja':':富士山:','ko':':후지산:','pt':':monte_fuji:','it':':monte_fuji:','fa':':کوه_فوجی:','id':':gunung_fuji:','zh':':富士山:','ru':':Фудзияма:','tr':':fuji_dağı:','ar':':جبل_فوجي:'},'\U000026F0\U0000FE0F':{'en':':mountain:','status':fully_qualified,'E':0.7,'variant':True,'de':':berg:','es':':montaña:','fr':':montagne:','ja':':山:','ko':':산:','pt':':montanha:','it':':montagna:','fa':':کوه:','id':':gunung:','zh':':山:','ru':':гора:','tr':':dağ:','ar':':جبل:'},'\U000026F0':{'en':':mountain:','status':unqualified,'E':0.7,'variant':True,'de':':berg:','es':':montaña:','fr':':montagne:','ja':':山:','ko':':산:','pt':':montanha:','it':':montagna:','fa':':کوه:','id':':gunung:','zh':':山:','ru':':гора:','tr':':dağ:','ar':':جبل:'},'\U0001F6A0':{'en':':mountain_cableway:','status':fully_qualified,'E':1,'de':':bergschwebebahn:','es':':teleférico_de_montaña:','fr':':téléphérique:','ja':':ケーブルカー:','ko':':산악_케이블카:','pt':':cabo_suspenso_usado_em_teleféricos_nas_montanhas:','it':':funivia:','fa':':تله_کابین_کوهستانی:','id':':kereta_gantung:','zh':':缆车:','ru':':вагон_фуникулера:','tr':':dağ_teleferik_hattı:','ar':':قطار_جبلي:'},'\U0001F69E':{'en':':mountain_railway:','status':fully_qualified,'E':1,'de':':bergbahn:','es':':ferrocarril_de_montaña:','fr':':train_de_montagne:','ja':':登山鉄道:','ko':':기차_터널:','pt':':estrada_de_ferro_na_montanha:','it':':ferrovia_di_montagna:','fa':':ریل_کوهستانی:','id':':jalur_kereta_pegunungan:','zh':':山区铁路:','ru':':поезд_в_горах:','tr':':dağ_demiryolu:','ar':':سكة_حديد_جبلية:'},'\U0001F401':{'en':':mouse:','status':fully_qualified,'E':1,'alias':[':mouse2:'],'de':':maus:','es':':ratón:','fr':':souris:','ja':':ハツカネズミ:','ko':':생쥐:','pt':':camundongo:','it':':topo:','fa':':موش:','id':':tikus_putih:','zh':':老鼠:','ru':':мышь:','tr':':fare:','ar':':فأر:'},'\U0001F42D':{'en':':mouse_face:','status':fully_qualified,'E':0.6,'alias':[':mouse:'],'de':':mäusegesicht:','es':':cara_de_ratón:','fr':':tête_de_souris:','ja':':ネズミの顔:','ko':':쥐_얼굴:','pt':':rosto_de_camundongo:','it':':muso_di_topo:','fa':':صورت_موش:','id':':wajah_tikus:','zh':':老鼠头:','ru':':морда_мыши:','tr':':fare_yüzü:','ar':':وجه_فأر:'},'\U0001FAA4':{'en':':mouse_trap:','status':fully_qualified,'E':13,'de':':mausefalle:','es':':trampa_de_ratones:','fr':':tapette_à_souris:','ja':':ねずみ捕り:','ko':':쥐덫:','pt':':ratoeira:','it':':trappola_per_topi:','fa':':تله_موش:','id':':perangkap_tikus:','zh':':捕鼠器:','ru':':мышеловка:','tr':':fare_kapanı:','ar':':مصيدة_فئران:'},'\U0001F444':{'en':':mouth:','status':fully_qualified,'E':0.6,'alias':[':lips:'],'de':':mund:','es':':boca:','fr':':bouche:','ja':':口:','ko':':입:','pt':':boca:','it':':bocca:','fa':':لب:','id':':mulut:','zh':':嘴:','ru':':рот:','tr':':ağız:','ar':':فم:'},'\U0001F3A5':{'en':':movie_camera:','status':fully_qualified,'E':0.6,'de':':filmkamera:','es':':cámara_de_cine:','fr':':caméra:','ja':':映画カメラ:','ko':':영화_카메라:','pt':':câmera_de_cinema:','it':':cinepresa:','fa':':دوربین_سینمایی:','id':':kamera_film:','zh':':电影摄影机:','ru':':кинокамера:','tr':':film_kamerası:','ar':':كاميرا_أفلام:'},'\U00002716\U0000FE0F':{'en':':multiply:','status':fully_qualified,'E':0.6,'alias':[':heavy_multiplication_x:'],'variant':True,'de':':multiplikationszeichen:','es':':multiplicación:','fr':':signe_multiplier:','ja':':掛け算:','ko':':곱하기:','pt':':sinal_de_multiplicação:','it':':segno_moltiplicazione:','fa':':ضرب_کردن:','id':':kali:','zh':':乘:','ru':':знак_умножения:','tr':':çarpım_işareti:','ar':':ضرب:'},'\U00002716':{'en':':multiply:','status':unqualified,'E':0.6,'alias':[':heavy_multiplication_x:'],'variant':True,'de':':mal:','es':':multiplicación:','fr':':signe_multiplier:','ja':':掛け算:','ko':':곱하기:','pt':':sinal_de_multiplicação:','it':':moltiplicazione:','fa':':ضرب_کردن:','id':':kali:','zh':':乘:','ru':':знак_умножения:','tr':':çarpım_işareti:','ar':':ضرب:'},'\U0001F344':{'en':':mushroom:','status':fully_qualified,'E':0.6,'de':':fliegenpilz:','es':':champiñón:','fr':':champignon:','ja':':キノコ:','ko':':버섯:','pt':':cogumelo:','it':':fungo:','fa':':قارچ:','id':':jamur:','zh':':蘑菇:','ru':':гриб:','tr':':mantar:','ar':':عيش_الغراب:'},'\U0001F3B9':{'en':':musical_keyboard:','status':fully_qualified,'E':0.6,'de':':klaviatur:','es':':teclado_musical:','fr':':piano:','ja':':鍵盤:','ko':':피아노:','pt':':teclado_musical:','it':':piano:','fa':':کیبورد_موسیقی:','id':':keyboard_musik:','zh':':音乐键盘:','ru':':клавишные:','tr':':müzik_klavyesi:','ar':':بيانو:'},'\U0001F3B5':{'en':':musical_note:','status':fully_qualified,'E':0.6,'de':':musiknote:','es':':nota_musical:','fr':':note_de_musique:','ja':':音符:','ko':':음표:','pt':':nota_musical:','it':':nota_musicale:','fa':':نت_موسیقی:','id':':not_musik:','zh':':音符:','ru':':нота:','tr':':müzik_notası:','ar':':نوتة_موسيقية:'},'\U0001F3B6':{'en':':musical_notes:','status':fully_qualified,'E':0.6,'alias':[':notes:'],'de':':musiknoten:','es':':notas_musicales:','fr':':notes_de_musique:','ja':':複数の音符:','ko':':노래_음표:','pt':':notas_musicais:','it':':note_musicali:','fa':':نت_های_موسیقی:','id':':not-not_musik:','zh':':多个音符:','ru':':ноты:','tr':':müzik_notaları:','ar':':نوتات_موسيقية:'},'\U0001F3BC':{'en':':musical_score:','status':fully_qualified,'E':0.6,'de':':notenschlüssel:','es':':pentagrama:','fr':':partition:','ja':':楽譜:','ko':':높은음자리표:','pt':':partitura_musical:','it':':pentagramma:','fa':':پارتیتور:','id':':naskah_musik:','zh':':乐谱:','ru':':нотный_стан:','tr':':sol_anahtarı:','ar':':علامة_موسيقية:'},'\U0001F507':{'en':':muted_speaker:','status':fully_qualified,'E':1,'alias':[':mute:'],'de':':durchgestrichener_lautsprecher:','es':':altavoz_silenciado:','fr':':muet:','ja':':スピーカー消音:','ko':':무음:','pt':':alto-falante_silenciado:','it':':altoparlante_disattivato:','fa':':بلندگوی_خاموش:','id':':speaker_mati:','zh':':已静音的扬声器:','ru':':без_звука:','tr':':ses_kapalı:','ar':':ممنوع_الحديث:'},'\U0001F9D1\U0000200D\U0001F384':{'en':':mx_claus:','status':fully_qualified,'E':13,'de':':weihnachtsperson:','es':':noel:','fr':':santa:','ja':':サンタさん:','ko':':산타:','pt':':noel:','it':':santa_claus:','fa':':نوئل:','id':':manusia_sinterklas:','zh':':圣诞人:','ru':':Санта:','tr':':cinsiyet_belirtmeyen_noel_karakteri:','ar':':كلوز:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F384':{'en':':mx_claus_dark_skin_tone:','status':fully_qualified,'E':13,'de':':weihnachtsperson_dunkle_hautfarbe:','es':':noel_tono_de_piel_oscuro:','fr':':santa_peau_foncée:','ja':':サンタさん_濃い肌色:','ko':':산타_검은색_피부:','pt':':noel_pele_escura:','it':':santa_claus_carnagione_scura:','fa':':نوئل_پوست_آبنوسی:','id':':manusia_sinterklas_warna_kulit_gelap:','zh':':圣诞人_较深肤色:','ru':':Санта_очень_темный_тон_кожи:','tr':':cinsiyet_belirtmeyen_noel_karakteri_koyu_cilt_tonu:','ar':':كلوز_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F384':{'en':':mx_claus_light_skin_tone:','status':fully_qualified,'E':13,'de':':weihnachtsperson_helle_hautfarbe:','es':':noel_tono_de_piel_claro:','fr':':santa_peau_claire:','ja':':サンタさん_薄い肌色:','ko':':산타_하얀_피부:','pt':':noel_pele_clara:','it':':santa_claus_carnagione_chiara:','fa':':نوئل_پوست_سفید:','id':':manusia_sinterklas_warna_kulit_cerah:','zh':':圣诞人_较浅肤色:','ru':':Санта_очень_светлый_тон_кожи:','tr':':cinsiyet_belirtmeyen_noel_karakteri_açık_cilt_tonu:','ar':':كلوز_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F384':{'en':':mx_claus_medium-dark_skin_tone:','status':fully_qualified,'E':13,'de':':weihnachtsperson_mitteldunkle_hautfarbe:','es':':noel_tono_de_piel_oscuro_medio:','fr':':santa_peau_mate:','ja':':サンタさん_やや濃い肌色:','ko':':산타_진한_갈색_피부:','pt':':noel_pele_morena_escura:','it':':santa_claus_carnagione_abbastanza_scura:','fa':':نوئل_پوست_گندمی:','id':':manusia_sinterklas_warna_kulit_gelap-sedang:','zh':':圣诞人_中等深肤色:','ru':':Санта_темный_тон_кожи:','tr':':cinsiyet_belirtmeyen_noel_karakteri_orta_koyu_cilt_tonu:','ar':':كلوز_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F384':{'en':':mx_claus_medium-light_skin_tone:','status':fully_qualified,'E':13,'de':':weihnachtsperson_mittelhelle_hautfarbe:','es':':noel_tono_de_piel_claro_medio:','fr':':santa_peau_moyennement_claire:','ja':':サンタさん_やや薄い肌色:','ko':':산타_연한_갈색_피부:','pt':':noel_pele_morena_clara:','it':':santa_claus_carnagione_abbastanza_chiara:','fa':':نوئل_پوست_روشن:','id':':manusia_sinterklas_warna_kulit_cerah-sedang:','zh':':圣诞人_中等浅肤色:','ru':':Санта_светлый_тон_кожи:','tr':':cinsiyet_belirtmeyen_noel_karakteri_orta_açık_cilt_tonu:','ar':':كلوز_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F384':{'en':':mx_claus_medium_skin_tone:','status':fully_qualified,'E':13,'de':':weihnachtsperson_mittlere_hautfarbe:','es':':noel_tono_de_piel_medio:','fr':':santa_peau_légèrement_mate:','ja':':サンタさん_中間の肌色:','ko':':산타_갈색_피부:','pt':':noel_pele_morena:','it':':santa_claus_carnagione_olivastra:','fa':':نوئل_پوست_طلایی:','id':':manusia_sinterklas_warna_kulit_sedang:','zh':':圣诞人_中等肤色:','ru':':Санта_средний_тон_кожи:','tr':':cinsiyet_belirtmeyen_noel_karakteri_orta_cilt_tonu:','ar':':كلوز_بشرة_بلون_معتدل:'},'\U0001F485':{'en':':nail_polish:','status':fully_qualified,'E':0.6,'alias':[':nail_care:'],'de':':nagellack:','es':':pintarse_las_uñas:','fr':':vernis_à_ongles:','ja':':マニキュアを塗る手:','ko':':매니큐어:','pt':':esmalte_de_unha:','it':':smalto_per_unghie:','fa':':لاک_زدن:','id':':poles_kuku:','zh':':涂指甲油:','ru':':маникюр:','tr':':oje_sürme:','ar':':طلاء_أظافر:'},'\U0001F485\U0001F3FF':{'en':':nail_polish_dark_skin_tone:','status':fully_qualified,'E':1,'de':':nagellack_dunkle_hautfarbe:','es':':pintarse_las_uñas_tono_de_piel_oscuro:','fr':':vernis_à_ongles_peau_foncée:','ja':':マニキュアを塗る手_濃い肌色:','ko':':매니큐어_검은색_피부:','pt':':esmalte_de_unha_pele_escura:','it':':smalto_per_unghie_carnagione_scura:','fa':':لاک_زدن_پوست_آبنوسی:','id':':poles_kuku_warna_kulit_gelap:','zh':':涂指甲油_较深肤色:','ru':':маникюр_очень_темный_тон_кожи:','tr':':oje_sürme_koyu_cilt_tonu:','ar':':طلاء_أظافر_بشرة_بلون_غامق:'},'\U0001F485\U0001F3FB':{'en':':nail_polish_light_skin_tone:','status':fully_qualified,'E':1,'de':':nagellack_helle_hautfarbe:','es':':pintarse_las_uñas_tono_de_piel_claro:','fr':':vernis_à_ongles_peau_claire:','ja':':マニキュアを塗る手_薄い肌色:','ko':':매니큐어_하얀_피부:','pt':':esmalte_de_unha_pele_clara:','it':':smalto_per_unghie_carnagione_chiara:','fa':':لاک_زدن_پوست_سفید:','id':':poles_kuku_warna_kulit_cerah:','zh':':涂指甲油_较浅肤色:','ru':':маникюр_очень_светлый_тон_кожи:','tr':':oje_sürme_açık_cilt_tonu:','ar':':طلاء_أظافر_بشرة_بلون_فاتح:'},'\U0001F485\U0001F3FE':{'en':':nail_polish_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':nagellack_mitteldunkle_hautfarbe:','es':':pintarse_las_uñas_tono_de_piel_oscuro_medio:','fr':':vernis_à_ongles_peau_mate:','ja':':マニキュアを塗る手_やや濃い肌色:','ko':':매니큐어_진한_갈색_피부:','pt':':esmalte_de_unha_pele_morena_escura:','it':':smalto_per_unghie_carnagione_abbastanza_scura:','fa':':لاک_زدن_پوست_گندمی:','id':':poles_kuku_warna_kulit_gelap-sedang:','zh':':涂指甲油_中等深肤色:','ru':':маникюр_темный_тон_кожи:','tr':':oje_sürme_orta_koyu_cilt_tonu:','ar':':طلاء_أظافر_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F485\U0001F3FC':{'en':':nail_polish_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':nagellack_mittelhelle_hautfarbe:','es':':pintarse_las_uñas_tono_de_piel_claro_medio:','fr':':vernis_à_ongles_peau_moyennement_claire:','ja':':マニキュアを塗る手_やや薄い肌色:','ko':':매니큐어_연한_갈색_피부:','pt':':esmalte_de_unha_pele_morena_clara:','it':':smalto_per_unghie_carnagione_abbastanza_chiara:','fa':':لاک_زدن_پوست_روشن:','id':':poles_kuku_warna_kulit_cerah-sedang:','zh':':涂指甲油_中等浅肤色:','ru':':маникюр_светлый_тон_кожи:','tr':':oje_sürme_orta_açık_cilt_tonu:','ar':':طلاء_أظافر_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F485\U0001F3FD':{'en':':nail_polish_medium_skin_tone:','status':fully_qualified,'E':1,'de':':nagellack_mittlere_hautfarbe:','es':':pintarse_las_uñas_tono_de_piel_medio:','fr':':vernis_à_ongles_peau_légèrement_mate:','ja':':マニキュアを塗る手_中間の肌色:','ko':':매니큐어_갈색_피부:','pt':':esmalte_de_unha_pele_morena:','it':':smalto_per_unghie_carnagione_olivastra:','fa':':لاک_زدن_پوست_طلایی:','id':':poles_kuku_warna_kulit_sedang:','zh':':涂指甲油_中等肤色:','ru':':маникюр_средний_тон_кожи:','tr':':oje_sürme_orta_cilt_tonu:','ar':':طلاء_أظافر_بشرة_بلون_معتدل:'},'\U0001F4DB':{'en':':name_badge:','status':fully_qualified,'E':0.6,'de':':namensschild:','es':':etiqueta_identificativa:','fr':':badge_nominatif:','ja':':名札:','ko':':이름표:','pt':':crachá:','it':':tesserino_per_nome:','fa':':برچسب_نام:','id':':lencana_nama:','zh':':姓名牌:','ru':':бейдж:','tr':':ad_rozeti:','ar':':ملصق_اسم:'},'\U0001F3DE\U0000FE0F':{'en':':national_park:','status':fully_qualified,'E':0.7,'variant':True,'de':':nationalpark:','es':':parque_nacional:','fr':':parc_national:','ja':':国立公園:','ko':':국립공원:','pt':':parque_nacional:','it':':parco_nazionale:','fa':':پارک_ملی:','id':':taman_nasional:','zh':':国家公园:','ru':':национальныи_парк:','tr':':milli_park:','ar':':حديقة_وطنية:'},'\U0001F3DE':{'en':':national_park:','status':unqualified,'E':0.7,'variant':True,'de':':nationalpark:','es':':parque_nacional:','fr':':parc_national:','ja':':国立公園:','ko':':국립공원:','pt':':parque_nacional:','it':':parco_nazionale:','fa':':پارک_ملی:','id':':taman_nasional:','zh':':国家公园:','ru':':национальный_парк:','tr':':milli_park:','ar':':حديقة_وطنية:'},'\U0001F922':{'en':':nauseated_face:','status':fully_qualified,'E':3,'de':':würgendes_gesicht:','es':':cara_de_náuseas:','fr':':visage_nauséeux:','ja':':吐き気を催している顔:','ko':':구역질을_하는_얼굴:','pt':':rosto_nauseado:','it':':faccina_nauseata:','fa':':حالت_تهوع:','id':':mual:','zh':':恶心:','ru':':тошнит:','tr':':kusmak_üzere_olan_yüz:','ar':':وجه_مشمئز:'},'\U0001F9FF':{'en':':nazar_amulet:','status':fully_qualified,'E':11,'de':':nazar-amulett:','es':':ojo_turco:','fr':':mauvais_œil:','ja':':ナザール・ボンジュウ:','ko':':악마의_눈:','pt':':olho_grego:','it':':occhio_di_allah:','fa':':نظرقربانی:','id':':jimat_nazar:','zh':':纳扎尔护身符:','ru':':назар_бонджук:','tr':':nazarlık:','ar':':الخرزة_الزرقاء:'},'\U0001F454':{'en':':necktie:','status':fully_qualified,'E':0.6,'de':':hemd_mit_krawatte:','es':':corbata:','fr':':cravate:','ja':':ネクタイ:','ko':':넥타이:','pt':':gravata:','it':':cravatta:','fa':':کراوات:','id':':dasi:','zh':':领带:','ru':':галстук:','tr':':kravat:','ar':':رابطة_عنق:'},'\U0001F913':{'en':':nerd_face:','status':fully_qualified,'E':1,'de':':strebergesicht:','es':':cara_de_empollón:','fr':':visage_de_geek:','ja':':オタク:','ko':':모범생_얼굴:','pt':':rosto_de_nerd:','it':':faccina_nerd:','fa':':خوره:','id':':wajah_kutu_buku:','zh':':书呆子脸:','ru':':очкарик:','tr':':ukala_yüz:','ar':':وجه_عبقري:'},'\U0001FABA':{'en':':nest_with_eggs:','status':fully_qualified,'E':14,'de':':nest_mit_eiern:','es':':nido_con_huevos:','fr':':nid_avec_œufs:','ja':':鳥の卵と巣:','ko':':알이_들어_있는_둥지:','pt':':ninho_com_ovos:','it':':nido_con_uova:','fa':':آشیانه_با_تخم:','id':':sarang_dengan_telur:','zh':':有蛋的巢:','ru':':гнездо_с_яйцами:','tr':':yumurta_bulunan_yuva:','ar':':عش_به_بيض:'},'\U0001FA86':{'en':':nesting_dolls:','status':fully_qualified,'E':13,'de':':matroschka:','es':':muñeca_rusa:','fr':':poupées_russes:','ja':':マトリョーシカ:','ko':':네스팅_인형:','pt':':boneca_russa:','it':':matrioska:','fa':':عروسک_تودرتو:','id':':boneka_bersarang:','zh':':套娃:','ru':':матрешка:','tr':':matruşka:','ar':':دمية_روسية:'},'\U0001F610':{'en':':neutral_face:','status':fully_qualified,'E':0.7,'variant':True,'de':':neutrales_gesicht:','es':':cara_neutral:','fr':':visage_neutre:','ja':':ポーカーフェイス:','ko':':덤덤한_얼굴:','pt':':rosto_neutro:','it':':faccina_neutra:','fa':':خنثی:','id':':wajah_netral:','zh':':冷漠:','ru':':все_равно:','tr':':duygusuz_yüz:','ar':':وجه_محايد:'},'\U0001F311':{'en':':new_moon:','status':fully_qualified,'E':0.6,'de':':neumond:','es':':luna_nueva:','fr':':nouvelle_lune:','ja':':新月:','ko':':삭:','pt':':lua_nova:','it':':luna_nuova:','fa':':ماه_نو:','id':':bulan_baru:','zh':':朔月:','ru':':новолуние:','tr':':yeni_ay:','ar':':محاق:'},'\U0001F31A':{'en':':new_moon_face:','status':fully_qualified,'E':1,'alias':[':new_moon_with_face:'],'de':':neumond_mit_gesicht:','es':':luna_nueva_con_cara:','fr':':nouvelle_lune_avec_visage:','ja':':顔のある新月:','ko':':얼굴이_있는_삭:','pt':':rosto_da_lua_nova:','it':':faccina_luna_nuova:','fa':':صورت_ماه_نو:','id':':wajah_bulan_baru:','zh':':微笑的朔月:','ru':':новая_луна_с_лицом:','tr':':yeni_ay_yüzü:','ar':':وجه_محاق:'},'\U0001F4F0':{'en':':newspaper:','status':fully_qualified,'E':0.6,'de':':zeitung:','es':':periódico:','fr':':journal:','ja':':新聞:','ko':':신문:','pt':':jornal:','it':':quotidiano:','fa':':روزنامه:','id':':koran:','zh':':报纸:','ru':':газета:','tr':':gazete:','ar':':جريدة:'},'\U000023ED\U0000FE0F':{'en':':next_track_button:','status':fully_qualified,'E':0.7,'alias':[':black_right_pointing_double_triangle_with_vertical_bar:'],'variant':True,'de':':nächster_titel:','es':':pista_siguiente:','fr':':bouton_piste_suivante:','ja':':次の曲ボタン:','ko':':다음_트랙_버튼:','pt':':botão_de_próxima_faixa:','it':':traccia_successiva:','fa':':دکمه_آهنگ_بعدی:','id':':tombol_lagu_berikutnya:','zh':':下一个音轨按钮:','ru':':следующии_трек:','tr':':sonraki_parça_düğmesi:','ar':':زر_التالي:'},'\U000023ED':{'en':':next_track_button:','status':unqualified,'E':0.7,'alias':[':black_right_pointing_double_triangle_with_vertical_bar:'],'variant':True,'de':':nächster_titel:','es':':pista_siguiente:','fr':':bouton_piste_suivante:','ja':':次の曲ボタン:','ko':':다음_트랙_버튼:','pt':':botão_de_próxima_faixa:','it':':traccia_successiva:','fa':':دکمهٔ_آهنگ_بعدی:','id':':tombol_lagu_berikutnya:','zh':':下一个音轨按钮:','ru':':следующий_трек:','tr':':sonraki_parça_düğmesi:','ar':':زر_التالي:'},'\U0001F303':{'en':':night_with_stars:','status':fully_qualified,'E':0.6,'de':':sternenhimmel:','es':':noche_estrellada:','fr':':nuit_étoilée:','ja':':夜の都会:','ko':':밤하늘_별:','pt':':noite_estrelada:','it':':notte_stellata:','fa':':شب_پرستاره:','id':':malam_berbintang:','zh':':夜晚:','ru':':звездная_ночь:','tr':':yıldızlı_gece:','ar':':ليل_مع_نجوم:'},'\U0001F564':{'en':':nine-thirty:','status':fully_qualified,'E':0.7,'alias':[':clock930:', ':nine_thirty:'],'variant':True,'de':':9.30_uhr:','es':':nueve_y_media:','fr':':neuf_heures_et_demie:','ja':':9時半:','ko':':아홉_시_반:','pt':':nove_e_meia:','it':':ore_nove_e_mezza:','fa':':ساعت_نه_و_نیم:','id':':jam_setengah_sepuluh:','zh':':九点半:','ru':':полдесятого:','tr':':dokuz_buçuk:','ar':':التاسعة_والنصف:'},'\U0001F558':{'en':':nine_o’clock:','status':fully_qualified,'E':0.6,'alias':[':clock9:', ':nine_oclock:'],'variant':True,'de':':9.00_uhr:','es':':9_en_punto:','fr':':neuf_heures:','ja':':9時:','ko':':아홉_시:','pt':':9_horas:','it':':ore_nove:','fa':':ساعت_نه:','id':':jam_sembilan_tepat:','zh':':九点:','ru':':девять_часов:','tr':':saat_9:','ar':':الساعة_التاسعة:'},'\U0001F977':{'en':':ninja:','status':fully_qualified,'E':13,'de':':ninja:','es':':ninja:','fr':':ninja:','ja':':忍者:','ko':':닌자:','pt':':ninja:','it':':ninja:','fa':':نینجا:','id':':ninja:','zh':':忍者:','ru':':ниндзя:','tr':':ninja:','ar':':نينجا:'},'\U0001F977\U0001F3FF':{'en':':ninja_dark_skin_tone:','status':fully_qualified,'E':13,'de':':ninja_dunkle_hautfarbe:','es':':ninja_tono_de_piel_oscuro:','fr':':ninja_peau_foncée:','ja':':忍者_濃い肌色:','ko':':닌자_검은색_피부:','pt':':ninja_pele_escura:','it':':ninja_carnagione_scura:','fa':':نینجا_پوست_آبنوسی:','id':':ninja_warna_kulit_gelap:','zh':':忍者_较深肤色:','ru':':ниндзя_очень_темный_тон_кожи:','tr':':ninja_koyu_cilt_tonu:','ar':':نينجا_بشرة_بلون_غامق:'},'\U0001F977\U0001F3FB':{'en':':ninja_light_skin_tone:','status':fully_qualified,'E':13,'de':':ninja_helle_hautfarbe:','es':':ninja_tono_de_piel_claro:','fr':':ninja_peau_claire:','ja':':忍者_薄い肌色:','ko':':닌자_하얀_피부:','pt':':ninja_pele_clara:','it':':ninja_carnagione_chiara:','fa':':نینجا_پوست_سفید:','id':':ninja_warna_kulit_cerah:','zh':':忍者_较浅肤色:','ru':':ниндзя_очень_светлый_тон_кожи:','tr':':ninja_açık_cilt_tonu:','ar':':نينجا_بشرة_بلون_فاتح:'},'\U0001F977\U0001F3FE':{'en':':ninja_medium-dark_skin_tone:','status':fully_qualified,'E':13,'de':':ninja_mitteldunkle_hautfarbe:','es':':ninja_tono_de_piel_oscuro_medio:','fr':':ninja_peau_mate:','ja':':忍者_やや濃い肌色:','ko':':닌자_진한_갈색_피부:','pt':':ninja_pele_morena_escura:','it':':ninja_carnagione_abbastanza_scura:','fa':':نینجا_پوست_گندمی:','id':':ninja_warna_kulit_gelap-sedang:','zh':':忍者_中等深肤色:','ru':':ниндзя_темный_тон_кожи:','tr':':ninja_orta_koyu_cilt_tonu:','ar':':نينجا_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F977\U0001F3FC':{'en':':ninja_medium-light_skin_tone:','status':fully_qualified,'E':13,'de':':ninja_mittelhelle_hautfarbe:','es':':ninja_tono_de_piel_claro_medio:','fr':':ninja_peau_moyennement_claire:','ja':':忍者_やや薄い肌色:','ko':':닌자_연한_갈색_피부:','pt':':ninja_pele_morena_clara:','it':':ninja_carnagione_abbastanza_chiara:','fa':':نینجا_پوست_روشن:','id':':ninja_warna_kulit_cerah-sedang:','zh':':忍者_中等浅肤色:','ru':':ниндзя_светлый_тон_кожи:','tr':':ninja_orta_açık_cilt_tonu:','ar':':نينجا_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F977\U0001F3FD':{'en':':ninja_medium_skin_tone:','status':fully_qualified,'E':13,'de':':ninja_mittlere_hautfarbe:','es':':ninja_tono_de_piel_medio:','fr':':ninja_peau_légèrement_mate:','ja':':忍者_中間の肌色:','ko':':닌자_갈색_피부:','pt':':ninja_pele_morena:','it':':ninja_carnagione_olivastra:','fa':':نینجا_پوست_طلایی:','id':':ninja_warna_kulit_sedang:','zh':':忍者_中等肤色:','ru':':ниндзя_средний_тон_кожи:','tr':':ninja_orta_cilt_tonu:','ar':':نينجا_بشرة_بلون_معتدل:'},'\U0001F6B3':{'en':':no_bicycles:','status':fully_qualified,'E':1,'de':':fahrräder_verboten:','es':':bicicletas_prohibidas:','fr':':vélos_interdits:','ja':':自転車禁止:','ko':':자전거_금지:','pt':':proibido_andar_de_bicicleta:','it':':segnale_di_divieto_di_transito_delle_biciclette:','fa':':دوچرخه_ممنوع:','id':':dilarang_bersepeda:','zh':':禁止自行车:','ru':':перечеркнутый_велосипед:','tr':':bisiklet_yasak:','ar':':ممنوع_ركوب_الدراجات:'},'\U000026D4':{'en':':no_entry:','status':fully_qualified,'E':0.6,'variant':True,'de':':zutritt_verboten:','es':':dirección_prohibida:','fr':':sens_interdit:','ja':':車両進入禁止:','ko':':진입_금지:','pt':':entrada_proibida:','it':':segnale_di_divieto_di_accesso:','fa':':ورود_ممنوع:','id':':dilarang_masuk:','zh':':禁止通行:','ru':':въезд_запрещен:','tr':':girilmez:','ar':':ممنوع_الدخول:'},'\U0001F6AF':{'en':':no_littering:','status':fully_qualified,'E':1,'alias':[':do_not_litter:'],'de':':abfall_verboten:','es':':prohibido_tirar_basura:','fr':':dépôt_d’ordures_interdit:','ja':':ポイ捨て禁止:','ko':':쓰레기_투기_금지:','pt':':proibido_jogar_lixo_no_chão:','it':':simbolo_del_divieto_di_gettare_rifiuti:','fa':':ریختن_زباله_ممنوع:','id':':dilarang_membuang_sampah_sembarangan:','zh':':禁止乱扔垃圾:','ru':':не_мусорьте:','tr':':çöp_atılmaz:','ar':':ممنوع_وضع_النفايات:'},'\U0001F4F5':{'en':':no_mobile_phones:','status':fully_qualified,'E':1,'de':':mobiltelefone_verboten:','es':':prohibido_el_uso_de_móviles:','fr':':téléphones_portables_interdits:','ja':':携帯電話禁止:','ko':':휴대전화_사용금지:','pt':':proibido_o_uso_de_telefone_celular:','it':':simbolo_che_vieta_l’uso_dei_cellulari:','fa':':تلفن_همراه_ممنوع:','id':':dilarang_menggunakan_ponsel:','zh':':禁止使用手机:','ru':':мобильные_телефоны_запрещены:','tr':':cep_telefonu_yasak:','ar':':ممنوع_استخدام_الهاتف_الجوال:'},'\U0001F51E':{'en':':no_one_under_eighteen:','status':fully_qualified,'E':0.6,'alias':[':underage:'],'de':':minderjährige_verboten:','es':':prohibido_para_menos_de_18_años:','fr':':18_ans_et_plus:','ja':':18歳未満禁止:','ko':':성인용:','pt':':proibido_para_menores_de_18_anos:','it':':simbolo_di_divieto_ai_minorenni:','fa':':زیر_۱۸_سال_ممنوع:','id':':di_bawah_18_tahun_dilarang:','zh':':18禁:','ru':':для_людей_старше_восемнадцати_лет:','tr':':18_yaş_altı_yasak:','ar':':ممنوع_لأقل_من_18_عاما:'},'\U0001F6B7':{'en':':no_pedestrians:','status':fully_qualified,'E':1,'de':':fußgänger_verboten:','es':':prohibido_el_paso_de_peatones:','fr':':interdit_aux_piétons:','ja':':歩行者立入禁止:','ko':':보행자_금지:','pt':':proibida_a_passagem_de_pedestres:','it':':simbolo_del_divieto_di_transito_pedoni:','fa':':عابر_پیاده_ممنوع:','id':':pejalan_kaki_dilarang_masuk:','zh':':禁止行人通行:','ru':':переход_запрещен:','tr':':yayalara_yasak:','ar':':ممنوع_سير_المشاة:'},'\U0001F6AD':{'en':':no_smoking:','status':fully_qualified,'E':0.6,'variant':True,'de':':rauchverbot:','es':':prohibido_fumar:','fr':':interdiction_de_fumer:','ja':':禁煙:','ko':':금연_신호:','pt':':proibido_fumar:','it':':segnale_di_vietato_fumare:','fa':':سیگار_کشیدن_ممنوع:','id':':dilarang_merokok:','zh':':禁止吸烟:','ru':':курение_запрещено:','tr':':sigara_içilmez:','ar':':ممنوع_التدخين:'},'\U0001F6B1':{'en':':non-potable_water:','status':fully_qualified,'E':1,'alias':[':non_potable_water:'],'de':':kein_trinkwasser:','es':':agua_no_potable:','fr':':eau_non_potable:','ja':':飲用不可:','ko':':마실_수_없는_물:','pt':':água_não_potável:','it':':simbolo_di_acqua_non_potabile:','fa':':آب_غیرآشامیدنی:','id':':air_tidak_dapat_diminum:','zh':':非饮用水:','ru':':непитьевая_вода:','tr':':içilmeyen_su:','ar':':مياه_غير_صالحة_للشرب:'},'\U0001F443':{'en':':nose:','status':fully_qualified,'E':0.6,'de':':nase:','es':':nariz:','fr':':nez:','ja':':鼻:','ko':':코:','pt':':nariz:','it':':naso:','fa':':بینی:','id':':hidung:','zh':':鼻子:','ru':':нос:','tr':':burun:','ar':':أنف:'},'\U0001F443\U0001F3FF':{'en':':nose_dark_skin_tone:','status':fully_qualified,'E':1,'de':':nase_dunkle_hautfarbe:','es':':nariz_tono_de_piel_oscuro:','fr':':nez_peau_foncée:','ja':':鼻_濃い肌色:','ko':':코_검은색_피부:','pt':':nariz_pele_escura:','it':':naso_carnagione_scura:','fa':':بینی_پوست_آبنوسی:','id':':hidung_warna_kulit_gelap:','zh':':鼻子_较深肤色:','ru':':нос_очень_темный_тон_кожи:','tr':':burun_koyu_cilt_tonu:','ar':':أنف_بشرة_بلون_غامق:'},'\U0001F443\U0001F3FB':{'en':':nose_light_skin_tone:','status':fully_qualified,'E':1,'de':':nase_helle_hautfarbe:','es':':nariz_tono_de_piel_claro:','fr':':nez_peau_claire:','ja':':鼻_薄い肌色:','ko':':코_하얀_피부:','pt':':nariz_pele_clara:','it':':naso_carnagione_chiara:','fa':':بینی_پوست_سفید:','id':':hidung_warna_kulit_cerah:','zh':':鼻子_较浅肤色:','ru':':нос_очень_светлый_тон_кожи:','tr':':burun_açık_cilt_tonu:','ar':':أنف_بشرة_بلون_فاتح:'},'\U0001F443\U0001F3FE':{'en':':nose_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':nase_mitteldunkle_hautfarbe:','es':':nariz_tono_de_piel_oscuro_medio:','fr':':nez_peau_mate:','ja':':鼻_やや濃い肌色:','ko':':코_진한_갈색_피부:','pt':':nariz_pele_morena_escura:','it':':naso_carnagione_abbastanza_scura:','fa':':بینی_پوست_گندمی:','id':':hidung_warna_kulit_gelap-sedang:','zh':':鼻子_中等深肤色:','ru':':нос_темный_тон_кожи:','tr':':burun_orta_koyu_cilt_tonu:','ar':':أنف_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F443\U0001F3FC':{'en':':nose_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':nase_mittelhelle_hautfarbe:','es':':nariz_tono_de_piel_claro_medio:','fr':':nez_peau_moyennement_claire:','ja':':鼻_やや薄い肌色:','ko':':코_연한_갈색_피부:','pt':':nariz_pele_morena_clara:','it':':naso_carnagione_abbastanza_chiara:','fa':':بینی_پوست_روشن:','id':':hidung_warna_kulit_cerah-sedang:','zh':':鼻子_中等浅肤色:','ru':':нос_светлый_тон_кожи:','tr':':burun_orta_açık_cilt_tonu:','ar':':أنف_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F443\U0001F3FD':{'en':':nose_medium_skin_tone:','status':fully_qualified,'E':1,'de':':nase_mittlere_hautfarbe:','es':':nariz_tono_de_piel_medio:','fr':':nez_peau_légèrement_mate:','ja':':鼻_中間の肌色:','ko':':코_갈색_피부:','pt':':nariz_pele_morena:','it':':naso_carnagione_olivastra:','fa':':بینی_پوست_طلایی:','id':':hidung_warna_kulit_sedang:','zh':':鼻子_中等肤色:','ru':':нос_средний_тон_кожи:','tr':':burun_orta_cilt_tonu:','ar':':أنف_بشرة_بلون_معتدل:'},'\U0001F4D3':{'en':':notebook:','status':fully_qualified,'E':0.6,'de':':notizbuch:','es':':cuaderno:','fr':':carnet:','ja':':ノート:','ko':':공책:','pt':':caderno:','it':':quaderno:','fa':':دفتر_یادداشت:','id':':buku_catatan:','zh':':笔记本:','ru':':тетрадь:','tr':':not_defteri:','ar':':دفتر_ملاحظات:'},'\U0001F4D4':{'en':':notebook_with_decorative_cover:','status':fully_qualified,'E':0.6,'de':':notizbuch_mit_dekorativem_einband:','es':':cuaderno_con_tapa_decorativa:','fr':':carnet_avec_couverture:','ja':':表紙付きノート:','ko':':표지가_있는_노트:','pt':':caderno_decorado:','it':':quaderno_con_copertina_decorata:','fa':':دفتر_با_جلد_سخت:','id':':buku_catatan_bersampul:','zh':':精装笔记本:','ru':':записная_книжка:','tr':':süslü_not_defteri:','ar':':دفتر_مزين:'},'\U0001F529':{'en':':nut_and_bolt:','status':fully_qualified,'E':0.6,'de':':mutter_und_schraube:','es':':tornillo_y_tuerca:','fr':':vis_et_écrou:','ja':':ボルトとナット:','ko':':볼트와_너트:','pt':':porca_e_parafuso:','it':':vite_e_bullone:','fa':':پیچ_و_مهره:','id':':mur_dan_baut:','zh':':螺母与螺栓:','ru':':болт_и_гайка:','tr':':somun_ve_vida:','ar':':صامولة_ومسمار:'},'\U0001F419':{'en':':octopus:','status':fully_qualified,'E':0.6,'de':':oktopus:','es':':pulpo:','fr':':pieuvre:','ja':':タコ:','ko':':문어:','pt':':polvo:','it':':polpo:','fa':':هشت_پا:','id':':gurita:','zh':':章鱼:','ru':':осьминог:','tr':':ahtapot:','ar':':أخطبوط:'},'\U0001F362':{'en':':oden:','status':fully_qualified,'E':0.6,'de':':oden:','es':':brocheta:','fr':':brochette_de_poisson:','ja':':おでん:','ko':':오뎅:','pt':':oden:','it':':oden_giapponese:','fa':':اودن:','id':':oden:','zh':':关东煮:','ru':':одэн:','tr':':oden:','ar':':طعام_مشكل:'},'\U0001F3E2':{'en':':office_building:','status':fully_qualified,'E':0.6,'alias':[':office:'],'de':':bürogebäude:','es':':edificio_de_oficinas:','fr':':immeuble_de_bureaux:','ja':':オフィスビル:','ko':':빌딩:','pt':':edifício_comercial:','it':':edificio:','fa':':ساختمان_اداری:','id':':gedung_perkantoran:','zh':':办公楼:','ru':':офисное_здание:','tr':':ofis_binası:','ar':':مبنى_إداري:'},'\U0001F9D1\U0000200D\U0001F4BC':{'en':':office_worker:','status':fully_qualified,'E':12.1,'de':':büroangestellte(r):','es':':oficinista:','fr':':employé_de_bureau_(tous_genres):','ja':':会社員:','ko':':회사원:','pt':':trabalhador_de_escritório:','it':':persona_che_fa_un_lavoro_d’ufficio:','fa':':کارمند:','id':':pekerja_kantoran:','zh':':白领:','ru':':офисный_работник:','tr':':ofis_çalışanı:','ar':':موظف_مكتب:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F4BC':{'en':':office_worker_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':büroangestellte(r)_dunkle_hautfarbe:','es':':oficinista_tono_de_piel_oscuro:','fr':':employé_de_bureau_(tous_genres)_peau_foncée:','ja':':会社員_濃い肌色:','ko':':회사원_검은색_피부:','pt':':trabalhador_de_escritório_pele_escura:','it':':persona_che_fa_un_lavoro_d’ufficio_carnagione_scura:','fa':':کارمند_پوست_آبنوسی:','id':':pekerja_kantoran_warna_kulit_gelap:','zh':':白领_较深肤色:','ru':':офисный_работник_очень_темный_тон_кожи:','tr':':ofis_çalışanı_koyu_cilt_tonu:','ar':':موظف_مكتب_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F4BC':{'en':':office_worker_light_skin_tone:','status':fully_qualified,'E':12.1,'de':':büroangestellte(r)_helle_hautfarbe:','es':':oficinista_tono_de_piel_claro:','fr':':employé_de_bureau_(tous_genres)_peau_claire:','ja':':会社員_薄い肌色:','ko':':회사원_하얀_피부:','pt':':trabalhador_de_escritório_pele_clara:','it':':persona_che_fa_un_lavoro_d’ufficio_carnagione_chiara:','fa':':کارمند_پوست_سفید:','id':':pekerja_kantoran_warna_kulit_cerah:','zh':':白领_较浅肤色:','ru':':офисный_работник_очень_светлый_тон_кожи:','tr':':ofis_çalışanı_açık_cilt_tonu:','ar':':موظف_مكتب_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F4BC':{'en':':office_worker_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':büroangestellte(r)_mitteldunkle_hautfarbe:','es':':oficinista_tono_de_piel_oscuro_medio:','fr':':employé_de_bureau_(tous_genres)_peau_mate:','ja':':会社員_やや濃い肌色:','ko':':회사원_진한_갈색_피부:','pt':':trabalhador_de_escritório_pele_morena_escura:','it':':persona_che_fa_un_lavoro_d’ufficio_carnagione_abbastanza_scura:','fa':':کارمند_پوست_گندمی:','id':':pekerja_kantoran_warna_kulit_gelap-sedang:','zh':':白领_中等深肤色:','ru':':офисный_работник_темный_тон_кожи:','tr':':ofis_çalışanı_orta_koyu_cilt_tonu:','ar':':موظف_مكتب_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F4BC':{'en':':office_worker_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':büroangestellte(r)_mittelhelle_hautfarbe:','es':':oficinista_tono_de_piel_claro_medio:','fr':':employé_de_bureau_(tous_genres)_peau_moyennement_claire:','ja':':会社員_やや薄い肌色:','ko':':회사원_연한_갈색_피부:','pt':':trabalhador_de_escritório_pele_morena_clara:','it':':persona_che_fa_un_lavoro_d’ufficio_carnagione_abbastanza_chiara:','fa':':کارمند_پوست_روشن:','id':':pekerja_kantoran_warna_kulit_cerah-sedang:','zh':':白领_中等浅肤色:','ru':':офисный_работник_светлый_тон_кожи:','tr':':ofis_çalışanı_orta_açık_cilt_tonu:','ar':':موظف_مكتب_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F4BC':{'en':':office_worker_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':büroangestellte(r)_mittlere_hautfarbe:','es':':oficinista_tono_de_piel_medio:','fr':':employé_de_bureau_(tous_genres)_peau_légèrement_mate:','ja':':会社員_中間の肌色:','ko':':회사원_갈색_피부:','pt':':trabalhador_de_escritório_pele_morena:','it':':persona_che_fa_un_lavoro_d’ufficio_carnagione_olivastra:','fa':':کارمند_پوست_طلایی:','id':':pekerja_kantoran_warna_kulit_sedang:','zh':':白领_中等肤色:','ru':':офисный_работник_средний_тон_кожи:','tr':':ofis_çalışanı_orta_cilt_tonu:','ar':':موظف_مكتب_بشرة_بلون_معتدل:'},'\U0001F479':{'en':':ogre:','status':fully_qualified,'E':0.6,'alias':[':japanese_ogre:'],'de':':ungeheuer:','es':':demonio_japonés_oni:','fr':':ogre:','ja':':鬼:','ko':':도깨비_가면:','pt':':ogro:','it':':orco:','fa':':دیو:','id':':wajah_monster:','zh':':食人魔:','ru':':чудовище:','tr':':dev:','ar':':غول:'},'\U0001F6E2\U0000FE0F':{'en':':oil_drum:','status':fully_qualified,'E':0.7,'variant':True,'de':':ölfass:','es':':barril_de_petróleo:','fr':':baril_de_pétrole:','ja':':ドラム缶:','ko':':기름통:','pt':':barril_de_óleo:','it':':barile_di_petrolio:','fa':':بشکه_نفت:','id':':drum_minyak:','zh':':石油桶:','ru':':бочка:','tr':':petrol_varili:','ar':':برميل_نفط:'},'\U0001F6E2':{'en':':oil_drum:','status':unqualified,'E':0.7,'variant':True,'de':':ölfass:','es':':barril_de_petróleo:','fr':':baril_de_pétrole:','ja':':ドラム缶:','ko':':기름통:','pt':':barril_de_óleo:','it':':barile_di_petrolio:','fa':':بشکه_نفت:','id':':drum_minyak:','zh':':石油桶:','ru':':бочка:','tr':':petrol_varili:','ar':':برميل_نفط:'},'\U0001F5DD\U0000FE0F':{'en':':old_key:','status':fully_qualified,'E':0.7,'variant':True,'de':':alter_schlüssel:','es':':llave_antigua:','fr':':clé_ancienne:','ja':':古い鍵:','ko':':오래된_열쇠:','pt':':chave_antiga:','it':':chiave_antica:','fa':':کلید_قدیمی:','id':':kunci_antik:','zh':':老式钥匙:','ru':':старинныи_ключ:','tr':':eski_anahtar:','ar':':مفتاح_قديم:'},'\U0001F5DD':{'en':':old_key:','status':unqualified,'E':0.7,'variant':True,'de':':alter_schlüssel:','es':':llave_antigua:','fr':':clé_ancienne:','ja':':古い鍵:','ko':':오래된_열쇠:','pt':':chave_antiga:','it':':chiave_antica:','fa':':کلید_قدیمی:','id':':kunci_antik:','zh':':老式钥匙:','ru':':старинный_ключ:','tr':':eski_anahtar:','ar':':مفتاح_قديم:'},'\U0001F474':{'en':':old_man:','status':fully_qualified,'E':0.6,'alias':[':older_man:'],'de':':älterer_mann:','es':':anciano:','fr':':homme_âgé:','ja':':おじいさん:','ko':':할아버지:','pt':':homem_idoso:','it':':uomo_anziano:','fa':':پیرمرد:','id':':pria_tua:','zh':':老爷爷:','ru':':пожилой_мужчина:','tr':':yaşlı_erkek:','ar':':رجل_عجوز:'},'\U0001F474\U0001F3FF':{'en':':old_man_dark_skin_tone:','status':fully_qualified,'E':1,'de':':älterer_mann_dunkle_hautfarbe:','es':':anciano_tono_de_piel_oscuro:','fr':':homme_âgé_peau_foncée:','ja':':おじいさん_濃い肌色:','ko':':할아버지_검은색_피부:','pt':':homem_idoso_pele_escura:','it':':uomo_anziano_carnagione_scura:','fa':':پیرمرد_پوست_آبنوسی:','id':':pria_tua_warna_kulit_gelap:','zh':':老爷爷_较深肤色:','ru':':пожилой_мужчина_очень_темный_тон_кожи:','tr':':yaşlı_erkek_koyu_cilt_tonu:','ar':':رجل_عجوز_بشرة_بلون_غامق:'},'\U0001F474\U0001F3FB':{'en':':old_man_light_skin_tone:','status':fully_qualified,'E':1,'de':':älterer_mann_helle_hautfarbe:','es':':anciano_tono_de_piel_claro:','fr':':homme_âgé_peau_claire:','ja':':おじいさん_薄い肌色:','ko':':할아버지_하얀_피부:','pt':':homem_idoso_pele_clara:','it':':uomo_anziano_carnagione_chiara:','fa':':پیرمرد_پوست_سفید:','id':':pria_tua_warna_kulit_cerah:','zh':':老爷爷_较浅肤色:','ru':':пожилой_мужчина_очень_светлый_тон_кожи:','tr':':yaşlı_erkek_açık_cilt_tonu:','ar':':رجل_عجوز_بشرة_بلون_فاتح:'},'\U0001F474\U0001F3FE':{'en':':old_man_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':älterer_mann_mitteldunkle_hautfarbe:','es':':anciano_tono_de_piel_oscuro_medio:','fr':':homme_âgé_peau_mate:','ja':':おじいさん_やや濃い肌色:','ko':':할아버지_진한_갈색_피부:','pt':':homem_idoso_pele_morena_escura:','it':':uomo_anziano_carnagione_abbastanza_scura:','fa':':پیرمرد_پوست_گندمی:','id':':pria_tua_warna_kulit_gelap-sedang:','zh':':老爷爷_中等深肤色:','ru':':пожилой_мужчина_темный_тон_кожи:','tr':':yaşlı_erkek_orta_koyu_cilt_tonu:','ar':':رجل_عجوز_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F474\U0001F3FC':{'en':':old_man_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':älterer_mann_mittelhelle_hautfarbe:','es':':anciano_tono_de_piel_claro_medio:','fr':':homme_âgé_peau_moyennement_claire:','ja':':おじいさん_やや薄い肌色:','ko':':할아버지_연한_갈색_피부:','pt':':homem_idoso_pele_morena_clara:','it':':uomo_anziano_carnagione_abbastanza_chiara:','fa':':پیرمرد_پوست_روشن:','id':':pria_tua_warna_kulit_cerah-sedang:','zh':':老爷爷_中等浅肤色:','ru':':пожилой_мужчина_светлый_тон_кожи:','tr':':yaşlı_erkek_orta_açık_cilt_tonu:','ar':':رجل_عجوز_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F474\U0001F3FD':{'en':':old_man_medium_skin_tone:','status':fully_qualified,'E':1,'de':':älterer_mann_mittlere_hautfarbe:','es':':anciano_tono_de_piel_medio:','fr':':homme_âgé_peau_légèrement_mate:','ja':':おじいさん_中間の肌色:','ko':':할아버지_갈색_피부:','pt':':homem_idoso_pele_morena:','it':':uomo_anziano_carnagione_olivastra:','fa':':پیرمرد_پوست_طلایی:','id':':pria_tua_warna_kulit_sedang:','zh':':老爷爷_中等肤色:','ru':':пожилой_мужчина_средний_тон_кожи:','tr':':yaşlı_erkek_orta_cilt_tonu:','ar':':رجل_عجوز_بشرة_بلون_معتدل:'},'\U0001F475':{'en':':old_woman:','status':fully_qualified,'E':0.6,'alias':[':older_woman:'],'de':':ältere_frau:','es':':anciana:','fr':':femme_âgée:','ja':':おばあさん:','ko':':할머니:','pt':':idosa:','it':':donna_anziana:','fa':':پیرزن:','id':':wanita_tua:','zh':':老奶奶:','ru':':пожилая_женщина:','tr':':yaşlı_kadın:','ar':':سيدة_عجوز:'},'\U0001F475\U0001F3FF':{'en':':old_woman_dark_skin_tone:','status':fully_qualified,'E':1,'de':':ältere_frau_dunkle_hautfarbe:','es':':anciana_tono_de_piel_oscuro:','fr':':femme_âgée_peau_foncée:','ja':':おばあさん_濃い肌色:','ko':':할머니_검은색_피부:','pt':':idosa_pele_escura:','it':':donna_anziana_carnagione_scura:','fa':':پیرزن_پوست_آبنوسی:','id':':wanita_tua_warna_kulit_gelap:','zh':':老奶奶_较深肤色:','ru':':пожилая_женщина_очень_темный_тон_кожи:','tr':':yaşlı_kadın_koyu_cilt_tonu:','ar':':سيدة_عجوز_بشرة_بلون_غامق:'},'\U0001F475\U0001F3FB':{'en':':old_woman_light_skin_tone:','status':fully_qualified,'E':1,'de':':ältere_frau_helle_hautfarbe:','es':':anciana_tono_de_piel_claro:','fr':':femme_âgée_peau_claire:','ja':':おばあさん_薄い肌色:','ko':':할머니_하얀_피부:','pt':':idosa_pele_clara:','it':':donna_anziana_carnagione_chiara:','fa':':پیرزن_پوست_سفید:','id':':wanita_tua_warna_kulit_cerah:','zh':':老奶奶_较浅肤色:','ru':':пожилая_женщина_очень_светлый_тон_кожи:','tr':':yaşlı_kadın_açık_cilt_tonu:','ar':':سيدة_عجوز_بشرة_بلون_فاتح:'},'\U0001F475\U0001F3FE':{'en':':old_woman_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':ältere_frau_mitteldunkle_hautfarbe:','es':':anciana_tono_de_piel_oscuro_medio:','fr':':femme_âgée_peau_mate:','ja':':おばあさん_やや濃い肌色:','ko':':할머니_진한_갈색_피부:','pt':':idosa_pele_morena_escura:','it':':donna_anziana_carnagione_abbastanza_scura:','fa':':پیرزن_پوست_گندمی:','id':':wanita_tua_warna_kulit_gelap-sedang:','zh':':老奶奶_中等深肤色:','ru':':пожилая_женщина_темный_тон_кожи:','tr':':yaşlı_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_عجوز_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F475\U0001F3FC':{'en':':old_woman_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':ältere_frau_mittelhelle_hautfarbe:','es':':anciana_tono_de_piel_claro_medio:','fr':':femme_âgée_peau_moyennement_claire:','ja':':おばあさん_やや薄い肌色:','ko':':할머니_연한_갈색_피부:','pt':':idosa_pele_morena_clara:','it':':donna_anziana_carnagione_abbastanza_chiara:','fa':':پیرزن_پوست_روشن:','id':':wanita_tua_warna_kulit_cerah-sedang:','zh':':老奶奶_中等浅肤色:','ru':':пожилая_женщина_светлый_тон_кожи:','tr':':yaşlı_kadın_orta_açık_cilt_tonu:','ar':':سيدة_عجوز_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F475\U0001F3FD':{'en':':old_woman_medium_skin_tone:','status':fully_qualified,'E':1,'de':':ältere_frau_mittlere_hautfarbe:','es':':anciana_tono_de_piel_medio:','fr':':femme_âgée_peau_légèrement_mate:','ja':':おばあさん_中間の肌色:','ko':':할머니_갈색_피부:','pt':':idosa_pele_morena:','it':':donna_anziana_carnagione_olivastra:','fa':':پیرزن_پوست_طلایی:','id':':wanita_tua_warna_kulit_sedang:','zh':':老奶奶_中等肤色:','ru':':пожилая_женщина_средний_тон_кожи:','tr':':yaşlı_kadın_orta_cilt_tonu:','ar':':سيدة_عجوز_بشرة_بلون_معتدل:'},'\U0001F9D3':{'en':':older_person:','status':fully_qualified,'E':5,'alias':[':older_adult:'],'de':':ältere_person:','es':':persona_mayor:','fr':':personne_âgée:','ja':':お年寄り:','ko':':노인:','pt':':idoso:','it':':persona_anziana:','fa':':میان_سال:','id':':lanjut_usia:','zh':':老年人:','ru':':пожилой_человек:','tr':':yaşlı_yetişkin:','ar':':بالغ_أكبر_في_السن:'},'\U0001F9D3\U0001F3FF':{'en':':older_person_dark_skin_tone:','status':fully_qualified,'E':5,'de':':ältere_person_dunkle_hautfarbe:','es':':persona_mayor_tono_de_piel_oscuro:','fr':':personne_âgée_peau_foncée:','ja':':お年寄り_濃い肌色:','ko':':노인_검은색_피부:','pt':':idoso_pele_escura:','it':':persona_anziana_carnagione_scura:','fa':':میان_سال_پوست_آبنوسی:','id':':lanjut_usia_warna_kulit_gelap:','zh':':老年人_较深肤色:','ru':':пожилой_человек_очень_темный_тон_кожи:','tr':':yaşlı_yetişkin_koyu_cilt_tonu:','ar':':بالغ_أكبر_في_السن_بشرة_بلون_غامق:'},'\U0001F9D3\U0001F3FB':{'en':':older_person_light_skin_tone:','status':fully_qualified,'E':5,'de':':ältere_person_helle_hautfarbe:','es':':persona_mayor_tono_de_piel_claro:','fr':':personne_âgée_peau_claire:','ja':':お年寄り_薄い肌色:','ko':':노인_하얀_피부:','pt':':idoso_pele_clara:','it':':persona_anziana_carnagione_chiara:','fa':':میان_سال_پوست_سفید:','id':':lanjut_usia_warna_kulit_cerah:','zh':':老年人_较浅肤色:','ru':':пожилой_человек_очень_светлый_тон_кожи:','tr':':yaşlı_yetişkin_açık_cilt_tonu:','ar':':بالغ_أكبر_في_السن_بشرة_بلون_فاتح:'},'\U0001F9D3\U0001F3FE':{'en':':older_person_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':ältere_person_mitteldunkle_hautfarbe:','es':':persona_mayor_tono_de_piel_oscuro_medio:','fr':':personne_âgée_peau_mate:','ja':':お年寄り_やや濃い肌色:','ko':':노인_진한_갈색_피부:','pt':':idoso_pele_morena_escura:','it':':persona_anziana_carnagione_abbastanza_scura:','fa':':میان_سال_پوست_گندمی:','id':':lanjut_usia_warna_kulit_gelap-sedang:','zh':':老年人_中等深肤色:','ru':':пожилой_человек_темный_тон_кожи:','tr':':yaşlı_yetişkin_orta_koyu_cilt_tonu:','ar':':بالغ_أكبر_في_السن_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D3\U0001F3FC':{'en':':older_person_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':ältere_person_mittelhelle_hautfarbe:','es':':persona_mayor_tono_de_piel_claro_medio:','fr':':personne_âgée_peau_moyennement_claire:','ja':':お年寄り_やや薄い肌色:','ko':':노인_연한_갈색_피부:','pt':':idoso_pele_morena_clara:','it':':persona_anziana_carnagione_abbastanza_chiara:','fa':':میان_سال_پوست_روشن:','id':':lanjut_usia_warna_kulit_cerah-sedang:','zh':':老年人_中等浅肤色:','ru':':пожилой_человек_светлый_тон_кожи:','tr':':yaşlı_yetişkin_orta_açık_cilt_tonu:','ar':':بالغ_أكبر_في_السن_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D3\U0001F3FD':{'en':':older_person_medium_skin_tone:','status':fully_qualified,'E':5,'de':':ältere_person_mittlere_hautfarbe:','es':':persona_mayor_tono_de_piel_medio:','fr':':personne_âgée_peau_légèrement_mate:','ja':':お年寄り_中間の肌色:','ko':':노인_갈색_피부:','pt':':idoso_pele_morena:','it':':persona_anziana_carnagione_olivastra:','fa':':میان_سال_پوست_طلایی:','id':':lanjut_usia_warna_kulit_sedang:','zh':':老年人_中等肤色:','ru':':пожилой_человек_средний_тон_кожи:','tr':':yaşlı_yetişkin_orta_cilt_tonu:','ar':':بالغ_أكبر_في_السن_بشرة_بلون_معتدل:'},'\U0001FAD2':{'en':':olive:','status':fully_qualified,'E':13,'de':':olive:','es':':aceituna:','fr':':olive:','ja':':オリーブ:','ko':':올리브:','pt':':azeitona:','it':':oliva:','fa':':زیتون:','id':':zaitun:','zh':':橄榄:','ru':':оливка:','tr':':zeytin:','ar':':زيتون:'},'\U0001F549\U0000FE0F':{'en':':om:','status':fully_qualified,'E':0.7,'alias':[':om_symbol:'],'variant':True,'de':':om:','es':':om:','fr':':om:','ja':':オーム:','ko':':옴:','pt':':om:','it':':om:','fa':':ام:','id':':om:','zh':':奥姆:','ru':':ом:','tr':':om:','ar':':دين_هندوسي:'},'\U0001F549':{'en':':om:','status':unqualified,'E':0.7,'alias':[':om_symbol:'],'variant':True,'de':':om:','es':':om:','fr':':om:','ja':':オーム:','ko':':옴:','pt':':om:','it':':om:','fa':':اُم:','id':':om:','zh':':奥姆:','ru':':ом:','tr':':om:','ar':':دين_هندوسي:'},'\U0001F698':{'en':':oncoming_automobile:','status':fully_qualified,'E':0.7,'variant':True,'de':':auto_von_vorne:','es':':coche_próximo:','fr':':voiture_de_face:','ja':':自動車正面:','ko':':오고_있는_자동차:','pt':':carro_se_aproximando:','it':':auto_in_arrivo:','fa':':خودرویی_که_نزدیک_می_شود:','id':':mobil_datang:','zh':':迎面驶来的汽车:','ru':':автомобиль_спереди:','tr':':yaklaşan_araba:','ar':':سيارة_مقتربة:'},'\U0001F68D':{'en':':oncoming_bus:','status':fully_qualified,'E':0.7,'variant':True,'de':':bus_von_vorne:','es':':autobús_próximo:','fr':':bus_de_face:','ja':':バス正面:','ko':':오고_있는_버스:','pt':':ônibus_se_aproximando:','it':':bus_in_arrivo:','fa':':اتوبوس_روبه_جلو:','id':':bus_datang:','zh':':迎面驶来的公交车:','ru':':автобус_спереди:','tr':':yaklaşan_otobüs:','ar':':حافلة_مقتربة:'},'\U0001F44A':{'en':':oncoming_fist:','status':fully_qualified,'E':0.6,'alias':[':fist_oncoming:', ':punch:', ':facepunch:'],'de':':geballte_faust:','es':':puño_cerrado:','fr':':poing_de_face:','ja':':グー:','ko':':주먹_전면:','pt':':soco:','it':':pugno_chiuso:','fa':':مشت_گره_کرده:','id':':tangan_terkepal_ke_depan:','zh':':出拳:','ru':':кулак:','tr':':yaklaşan_yumruk:','ar':':قبضة_يد:'},'\U0001F44A\U0001F3FF':{'en':':oncoming_fist_dark_skin_tone:','status':fully_qualified,'E':1,'de':':geballte_faust_dunkle_hautfarbe:','es':':puño_cerrado_tono_de_piel_oscuro:','fr':':poing_de_face_peau_foncée:','ja':':グー_濃い肌色:','ko':':주먹_전면_검은색_피부:','pt':':soco_pele_escura:','it':':pugno_chiuso_carnagione_scura:','fa':':مشت_گره_کرده_پوست_آبنوسی:','id':':tangan_terkepal_ke_depan_warna_kulit_gelap:','zh':':出拳_较深肤色:','ru':':кулак_очень_темный_тон_кожи:','tr':':yaklaşan_yumruk_koyu_cilt_tonu:','ar':':قبضة_يد_بشرة_بلون_غامق:'},'\U0001F44A\U0001F3FB':{'en':':oncoming_fist_light_skin_tone:','status':fully_qualified,'E':1,'de':':geballte_faust_helle_hautfarbe:','es':':puño_cerrado_tono_de_piel_claro:','fr':':poing_de_face_peau_claire:','ja':':グー_薄い肌色:','ko':':주먹_전면_하얀_피부:','pt':':soco_pele_clara:','it':':pugno_chiuso_carnagione_chiara:','fa':':مشت_گره_کرده_پوست_سفید:','id':':tangan_terkepal_ke_depan_warna_kulit_cerah:','zh':':出拳_较浅肤色:','ru':':кулак_очень_светлый_тон_кожи:','tr':':yaklaşan_yumruk_açık_cilt_tonu:','ar':':قبضة_يد_بشرة_بلون_فاتح:'},'\U0001F44A\U0001F3FE':{'en':':oncoming_fist_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':geballte_faust_mitteldunkle_hautfarbe:','es':':puño_cerrado_tono_de_piel_oscuro_medio:','fr':':poing_de_face_peau_mate:','ja':':グー_やや濃い肌色:','ko':':주먹_전면_진한_갈색_피부:','pt':':soco_pele_morena_escura:','it':':pugno_chiuso_carnagione_abbastanza_scura:','fa':':مشت_گره_کرده_پوست_گندمی:','id':':tangan_terkepal_ke_depan_warna_kulit_gelap-sedang:','zh':':出拳_中等深肤色:','ru':':кулак_темный_тон_кожи:','tr':':yaklaşan_yumruk_orta_koyu_cilt_tonu:','ar':':قبضة_يد_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F44A\U0001F3FC':{'en':':oncoming_fist_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':geballte_faust_mittelhelle_hautfarbe:','es':':puño_cerrado_tono_de_piel_claro_medio:','fr':':poing_de_face_peau_moyennement_claire:','ja':':グー_やや薄い肌色:','ko':':주먹_전면_연한_갈색_피부:','pt':':soco_pele_morena_clara:','it':':pugno_chiuso_carnagione_abbastanza_chiara:','fa':':مشت_گره_کرده_پوست_روشن:','id':':tangan_terkepal_ke_depan_warna_kulit_cerah-sedang:','zh':':出拳_中等浅肤色:','ru':':кулак_светлый_тон_кожи:','tr':':yaklaşan_yumruk_orta_açık_cilt_tonu:','ar':':قبضة_يد_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F44A\U0001F3FD':{'en':':oncoming_fist_medium_skin_tone:','status':fully_qualified,'E':1,'de':':geballte_faust_mittlere_hautfarbe:','es':':puño_cerrado_tono_de_piel_medio:','fr':':poing_de_face_peau_légèrement_mate:','ja':':グー_中間の肌色:','ko':':주먹_전면_갈색_피부:','pt':':soco_pele_morena:','it':':pugno_chiuso_carnagione_olivastra:','fa':':مشت_گره_کرده_پوست_طلایی:','id':':tangan_terkepal_ke_depan_warna_kulit_sedang:','zh':':出拳_中等肤色:','ru':':кулак_средний_тон_кожи:','tr':':yaklaşan_yumruk_orta_cilt_tonu:','ar':':قبضة_يد_بشرة_بلون_معتدل:'},'\U0001F694':{'en':':oncoming_police_car:','status':fully_qualified,'E':0.7,'variant':True,'de':':polizeiwagen_von_vorne:','es':':coche_de_policía_próximo:','fr':':voiture_de_police_de_face:','ja':':パトカー正面:','ko':':오고_있는_경찰차:','pt':':viatura_policial_se_aproximando:','it':':macchina_della_polizia_in_arrivo:','fa':':ماشین_پلیسی_که_نزدیک_می_شود:','id':':mobil_polisi_datang:','zh':':迎面驶来的警车:','ru':':полицейская_машина_спереди:','tr':':yaklaşan_polis_arabası:','ar':':سيارة_شرطة_مقتربة:'},'\U0001F696':{'en':':oncoming_taxi:','status':fully_qualified,'E':1,'de':':taxi_von_vorne:','es':':taxi_próximo:','fr':':taxi_de_face:','ja':':タクシー正面:','ko':':오고_있는_택시:','pt':':táxi_se_aproximando:','it':':taxi_in_arrivo:','fa':':تاکسی_ای_که_نزدیک_می_شود:','id':':taksi_datang:','zh':':迎面驶来的出租车:','ru':':такси_спереди:','tr':':yaklaşan_taksi:','ar':':تاكسي_مقترب:'},'\U0001FA71':{'en':':one-piece_swimsuit:','status':fully_qualified,'E':12,'alias':[':one_piece_swimsuit:'],'de':':einteiliger_badeanzug:','es':':traje_de_baño_de_una_pieza:','fr':':maillot_de_bain_une_pièce:','ja':':ワンピースの水着:','ko':':원피스_수영복:','pt':':maiô:','it':':costume_da_bagno_intero:','fa':':مایو_یک_تکه:','id':':pakaian_renang_one-piece:','zh':':连体泳衣:','ru':':цельный_купальник:','tr':':tek_parça_mayo:','ar':':مايوه_من_قطعة_واحدة:'},'\U0001F55C':{'en':':one-thirty:','status':fully_qualified,'E':0.7,'alias':[':clock130:', ':one_thirty:'],'variant':True,'de':':1.30_uhr:','es':':una_y_media:','fr':':une_heure_et_demie:','ja':':1時半:','ko':':한_시_반:','pt':':uma_e_meia:','it':':ore_una_e_mezza:','fa':':ساعت_یک_و_نیم:','id':':jam_setengah_dua:','zh':':一点半:','ru':':полвторого:','tr':':bir_buçuk:','ar':':الواحدة_والنصف:'},'\U0001F550':{'en':':one_o’clock:','status':fully_qualified,'E':0.6,'alias':[':clock1:', ':one_oclock:'],'variant':True,'de':':1.00_uhr:','es':':1_en_punto:','fr':':une_heure:','ja':':1時:','ko':':한_시:','pt':':1_hora:','it':':ore_una:','fa':':ساعت_یک:','id':':jam_satu_tepat:','zh':':一点:','ru':':час:','tr':':saat_1:','ar':':الساعة_الواحدة:'},'\U0001F9C5':{'en':':onion:','status':fully_qualified,'E':12,'de':':zwiebel:','es':':cebolla:','fr':':oignon:','ja':':タマネギ:','ko':':양파:','pt':':cebola:','it':':cipolla:','fa':':پیاز:','id':':bawang_bombay:','zh':':洋葱:','ru':':лук:','tr':':soğan:','ar':':بصل:'},'\U0001F4D6':{'en':':open_book:','status':fully_qualified,'E':0.6,'alias':[':book:'],'de':':offenes_buch:','es':':libro_abierto:','fr':':livre_ouvert:','ja':':開いた本:','ko':':펼쳐진_책:','pt':':livro_aberto:','it':':libro_aperto:','fa':':کتاب_باز:','id':':buku_terbuka:','zh':':打开的书本:','ru':':открытая_книга:','tr':':açık_kitap:','ar':':كتاب_مفتوح:'},'\U0001F4C2':{'en':':open_file_folder:','status':fully_qualified,'E':0.6,'de':':geöffneter_ordner:','es':':carpeta_de_archivos_abierta:','fr':':dossier_ouvert:','ja':':開いたフォルダー:','ko':':폴더_열기:','pt':':pasta_de_arquivos_aberta:','it':':cartella_aperta:','fa':':پوشهٔ_باز:','id':':folder_file_terbuka:','zh':':打开的文件夹:','ru':':открытая_папка:','tr':':açık_dosya_klasörü:','ar':':مجلد_مفتوح:'},'\U0001F450':{'en':':open_hands:','status':fully_qualified,'E':0.6,'de':':offene_hände:','es':':manos_abiertas:','fr':':mains_ouvertes:','ja':':両手のひら:','ko':':양손을_모아서_벌린_모양:','pt':':mãos_abertas:','it':':mani_aperte:','fa':':دست_های_باز:','id':':tangan_terbuka:','zh':':张开双手:','ru':':ладони_в_стороны:','tr':':eller_açık:','ar':':يدان_مفتوحتان:'},'\U0001F450\U0001F3FF':{'en':':open_hands_dark_skin_tone:','status':fully_qualified,'E':1,'de':':offene_hände_dunkle_hautfarbe:','es':':manos_abiertas_tono_de_piel_oscuro:','fr':':mains_ouvertes_peau_foncée:','ja':':両手のひら_濃い肌色:','ko':':양손을_모아서_벌린_모양_검은색_피부:','pt':':mãos_abertas_pele_escura:','it':':mani_aperte_carnagione_scura:','fa':':دست_های_باز_پوست_آبنوسی:','id':':tangan_terbuka_warna_kulit_gelap:','zh':':张开双手_较深肤色:','ru':':ладони_в_стороны_очень_темный_тон_кожи:','tr':':eller_açık_koyu_cilt_tonu:','ar':':يدان_مفتوحتان_بشرة_بلون_غامق:'},'\U0001F450\U0001F3FB':{'en':':open_hands_light_skin_tone:','status':fully_qualified,'E':1,'de':':offene_hände_helle_hautfarbe:','es':':manos_abiertas_tono_de_piel_claro:','fr':':mains_ouvertes_peau_claire:','ja':':両手のひら_薄い肌色:','ko':':양손을_모아서_벌린_모양_하얀_피부:','pt':':mãos_abertas_pele_clara:','it':':mani_aperte_carnagione_chiara:','fa':':دست_های_باز_پوست_سفید:','id':':tangan_terbuka_warna_kulit_cerah:','zh':':张开双手_较浅肤色:','ru':':ладони_в_стороны_очень_светлый_тон_кожи:','tr':':eller_açık_açık_cilt_tonu:','ar':':يدان_مفتوحتان_بشرة_بلون_فاتح:'},'\U0001F450\U0001F3FE':{'en':':open_hands_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':offene_hände_mitteldunkle_hautfarbe:','es':':manos_abiertas_tono_de_piel_oscuro_medio:','fr':':mains_ouvertes_peau_mate:','ja':':両手のひら_やや濃い肌色:','ko':':양손을_모아서_벌린_모양_진한_갈색_피부:','pt':':mãos_abertas_pele_morena_escura:','it':':mani_aperte_carnagione_abbastanza_scura:','fa':':دست_های_باز_پوست_گندمی:','id':':tangan_terbuka_warna_kulit_gelap-sedang:','zh':':张开双手_中等深肤色:','ru':':ладони_в_стороны_темный_тон_кожи:','tr':':eller_açık_orta_koyu_cilt_tonu:','ar':':يدان_مفتوحتان_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F450\U0001F3FC':{'en':':open_hands_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':offene_hände_mittelhelle_hautfarbe:','es':':manos_abiertas_tono_de_piel_claro_medio:','fr':':mains_ouvertes_peau_moyennement_claire:','ja':':両手のひら_やや薄い肌色:','ko':':양손을_모아서_벌린_모양_연한_갈색_피부:','pt':':mãos_abertas_pele_morena_clara:','it':':mani_aperte_carnagione_abbastanza_chiara:','fa':':دست_های_باز_پوست_روشن:','id':':tangan_terbuka_warna_kulit_cerah-sedang:','zh':':张开双手_中等浅肤色:','ru':':ладони_в_стороны_светлый_тон_кожи:','tr':':eller_açık_orta_açık_cilt_tonu:','ar':':يدان_مفتوحتان_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F450\U0001F3FD':{'en':':open_hands_medium_skin_tone:','status':fully_qualified,'E':1,'de':':offene_hände_mittlere_hautfarbe:','es':':manos_abiertas_tono_de_piel_medio:','fr':':mains_ouvertes_peau_légèrement_mate:','ja':':両手のひら_中間の肌色:','ko':':양손을_모아서_벌린_모양_갈색_피부:','pt':':mãos_abertas_pele_morena:','it':':mani_aperte_carnagione_olivastra:','fa':':دست_های_باز_پوست_طلایی:','id':':tangan_terbuka_warna_kulit_sedang:','zh':':张开双手_中等肤色:','ru':':ладони_в_стороны_средний_тон_кожи:','tr':':eller_açık_orta_cilt_tonu:','ar':':يدان_مفتوحتان_بشرة_بلون_معتدل:'},'\U0001F4ED':{'en':':open_mailbox_with_lowered_flag:','status':fully_qualified,'E':0.7,'alias':[':mailbox_with_no_mail:'],'variant':True,'de':':offener_briefkasten_ohne_post:','es':':buzón_abierto_con_la_bandera_bajada:','fr':':boîte_aux_lettres_ouverte_drapeau_baissé:','ja':':開いた郵便受け(手紙なし):','ko':':표지가_내려간_열린_우편함:','pt':':caixa_de_correio_aberta_com_bandeira_abaixada:','it':':cassetta_postale_aperta_bandierina_abbassata:','fa':':صندوق_پستی_باز_با_پرچم_پایین:','id':':kotak_surat_terbuka_dengan_bendera_turun:','zh':':无新信件:','ru':':открытый_почтовый_ящик_с_опущенным_флажком:','tr':':işareti_aşağıda_açık_posta_kutusu:','ar':':صندوق_بريد_مفتوح_بإشارة_لأسفل:'},'\U0001F4EC':{'en':':open_mailbox_with_raised_flag:','status':fully_qualified,'E':0.7,'alias':[':mailbox_with_mail:'],'variant':True,'de':':offener_briefkasten_mit_post:','es':':buzón_abierto_con_la_bandera_levantada:','fr':':boîte_aux_lettres_ouverte_drapeau_levé:','ja':':開いた郵便受け(手紙あり):','ko':':표지가_올라간_열린_우편함:','pt':':caixa_de_correio_aberta_com_bandeira_levantada:','it':':cassetta_postale_aperta_bandierina_alzata:','fa':':صندوق_پستی_باز_با_پرچم_بالا:','id':':kotak_surat_terbuka_dengan_bendera_terangkat:','zh':':有新信件:','ru':':открытый_почтовый_ящик_с_поднятым_флажком:','tr':':işareti_yukarıda_açık_posta_kutusu:','ar':':صندوق_بريد_مفتوح_بإشارة_مرفوعة:'},'\U0001F4BF':{'en':':optical_disk:','status':fully_qualified,'E':0.6,'alias':[':cd:'],'variant':True,'de':':cd:','es':':disco_óptico:','fr':':cd:','ja':':cd:','ko':':씨디:','pt':':cd:','it':':disco_ottico:','fa':':سی_دی:','id':':disk_optik:','zh':':光盘:','ru':':CD:','tr':':optik_disk:','ar':':سي_دي:'},'\U0001F4D9':{'en':':orange_book:','status':fully_qualified,'E':0.6,'de':':orangefarbenes_buch:','es':':libro_naranja:','fr':':livre_orange:','ja':':オレンジの本:','ko':':주황색_책:','pt':':livro_laranja:','it':':libro_arancione:','fa':':کتاب_نارنجی:','id':':buku_oranye:','zh':':橙色书本:','ru':':оранжевая_книга:','tr':':turuncu_kitap:','ar':':كتاب_برتقالي:'},'\U0001F7E0':{'en':':orange_circle:','status':fully_qualified,'E':12,'de':':oranger_punkt:','es':':círculo_naranja:','fr':':disque_orange:','ja':':オレンジの丸:','ko':':오렌지색_원:','pt':':círculo_laranja:','it':':cerchio_arancione:','fa':':دایره_نارنجی:','id':':lingkaran_oranye:','zh':':橙色圆:','ru':':оранжевый_круг:','tr':':turuncu_daire:','ar':':دائرة_برتقالية:'},'\U0001F9E1':{'en':':orange_heart:','status':fully_qualified,'E':5,'de':':oranges_herz:','es':':corazón_naranja:','fr':':cœur_orange:','ja':':オレンジのハート:','ko':':주황색_하트:','pt':':coração_laranja:','it':':cuore_arancione:','fa':':قلب_نارنجی:','id':':hati_oranye:','zh':':橙心:','ru':':оранжевое_сердце:','tr':':turuncu_kalp:','ar':':قلب_برتقالي:'},'\U0001F7E7':{'en':':orange_square:','status':fully_qualified,'E':12,'de':':oranges_quadrat:','es':':cuadrado_naranja:','fr':':carré_orange:','ja':':オレンジの四角:','ko':':오렌지색_사각형:','pt':':quadrado_laranja:','it':':quadrato_arancione:','fa':':مربع_نارنجی:','id':':persegi_oranye:','zh':':橙色方块:','ru':':оранжевый_квадрат:','tr':':turuncu_kare:','ar':':مربع_برتقالي:'},'\U0001F9A7':{'en':':orangutan:','status':fully_qualified,'E':12,'de':':orang-utan:','es':':orangután:','fr':':orang-outan:','ja':':オランウータン:','ko':':오랑우탄:','pt':':orangotango:','it':':orangotango:','fa':':اورانگوتان:','id':':orangutan:','zh':':红毛猩猩:','ru':':орангутан:','tr':':orangutan:','ar':':إنسان_الغاب:'},'\U00002626\U0000FE0F':{'en':':orthodox_cross:','status':fully_qualified,'E':1,'variant':True,'de':':orthodoxes_kreuz:','es':':cruz_ortodoxa:','fr':':croix_orthodoxe:','ja':':八端十字架:','ko':':전통적인_십자가:','pt':':cruz_ortodoxa:','it':':croce_ortodossa:','fa':':صلیب_ارتدوکسی:','id':':salib_ortodoks:','zh':':东正教十字架:','ru':':православныи_крест:','tr':':ortodoks_haçı:','ar':':صليب_أرثوزوكس:'},'\U00002626':{'en':':orthodox_cross:','status':unqualified,'E':1,'variant':True,'de':':orthodoxes_kreuz:','es':':cruz_ortodoxa:','fr':':croix_orthodoxe:','ja':':八端十字架:','ko':':전통적인_십자가:','pt':':cruz_ortodoxa:','it':':croce_ortodossa:','fa':':صلیب_ارتدوکسی:','id':':salib_ortodoks:','zh':':东正教十字架:','ru':':православный_крест:','tr':':ortodoks_haçı:','ar':':صليب_أرثوزوكس:'},'\U0001F9A6':{'en':':otter:','status':fully_qualified,'E':12,'de':':otter:','es':':nutria:','fr':':loutre:','ja':':カワウソ:','ko':':수달:','pt':':lontra:','it':':lontra:','fa':':سمور_آبی:','id':':berang-berang:','zh':':水獭:','ru':':выдра:','tr':':su_samuru:','ar':':قندس:'},'\U0001F4E4':{'en':':outbox_tray:','status':fully_qualified,'E':0.6,'variant':True,'de':':postausgang:','es':':bandeja_de_salida:','fr':':boîte_d’envoi:','ja':':送信トレイ:','ko':':보낸_편지함:','pt':':bandeja_de_saída:','it':':posta_inviata:','fa':':صندوق_خروجی:','id':':baki_kotak_keluar:','zh':':发件箱:','ru':':исходящие:','tr':':giden_kutusu:','ar':':صندوق_صادر:'},'\U0001F989':{'en':':owl:','status':fully_qualified,'E':3,'de':':eule:','es':':búho:','fr':':chouette:','ja':':フクロウ:','ko':':부엉이:','pt':':coruja:','it':':gufo:','fa':':جغد:','id':':burung_hantu:','zh':':猫头鹰:','ru':':сова:','tr':':baykuş:','ar':':بومة:'},'\U0001F402':{'en':':ox:','status':fully_qualified,'E':1,'de':':ochse:','es':':buey:','fr':':bœuf:','ja':':牡牛:','ko':':소:','pt':':boi:','it':':bue:','fa':':گاو_نر:','id':':lembu:','zh':':公牛:','ru':':бык:','tr':':öküz:','ar':':ثور:'},'\U0001F9AA':{'en':':oyster:','status':fully_qualified,'E':12,'de':':auster:','es':':ostra:','fr':':huître:','ja':':牡蠣:','ko':':굴:','pt':':ostra:','it':':ostrica:','fa':':صدف_خوراکی:','id':':tiram:','zh':':牡蛎:','ru':':устрица:','tr':':istiridye:','ar':':محار:'},'\U0001F4E6':{'en':':package:','status':fully_qualified,'E':0.6,'variant':True,'de':':paket:','es':':paquete:','fr':':colis:','ja':':荷物:','ko':':소포:','pt':':pacote:','it':':pacco:','fa':':بسته:','id':':paket:','zh':':包裹:','ru':':посылка:','tr':':paket:','ar':':طرد:'},'\U0001F4C4':{'en':':page_facing_up:','status':fully_qualified,'E':0.6,'de':':vorderseite_eines_blattes:','es':':página_hacia_arriba:','fr':':page:','ja':':ページ:','ko':':오른쪽_모서리가_접힌_문서:','pt':':página_voltada_para_cima:','it':':pagina:','fa':':صفحهٔ_روبه_بالا:','id':':halaman_tampak_depan:','zh':':文件:','ru':':страница:','tr':':yüzü_yukarıda_sayfa:','ar':':مستند:'},'\U0001F4C3':{'en':':page_with_curl:','status':fully_qualified,'E':0.6,'de':':teilweise_eingerolltes_blatt:','es':':página_doblada:','fr':':page_enroulée:','ja':':巻きページ:','ko':':안으로_말린_문서:','pt':':página_dobrada:','it':':pagina_con_piega:','fa':':صفحهٔ_تاشده:','id':':halaman_tergulung:','zh':':带卷边的页面:','ru':':страница_с_загибом:','tr':':kıvrık_sayfa:','ar':':صفحة_ملفوفة:'},'\U0001F4DF':{'en':':pager:','status':fully_qualified,'E':0.6,'variant':True,'de':':pager:','es':':busca:','fr':':bipeur:','ja':':ポケベル:','ko':':삐삐:','pt':':pager:','it':':cercapersone:','fa':':پیجر:','id':':pager:','zh':':寻呼机:','ru':':пейджер:','tr':':çağrı_cihazı:','ar':':جهاز_نداء_آلي:'},'\U0001F58C\U0000FE0F':{'en':':paintbrush:','status':fully_qualified,'E':0.7,'alias':[':lower_left_paintbrush:'],'variant':True,'de':':pinsel:','es':':pincel:','fr':':pinceau:','ja':':絵筆:','ko':':붓:','pt':':pincel:','it':':pennello:','fa':':قلم_مو:','id':':kuas:','zh':':画笔:','ru':':кисточка:','tr':':boya_fırçası:','ar':':فرشاة_ألوان:'},'\U0001F58C':{'en':':paintbrush:','status':unqualified,'E':0.7,'alias':[':lower_left_paintbrush:'],'variant':True,'de':':pinsel:','es':':pincel:','fr':':pinceau:','ja':':絵筆:','ko':':붓:','pt':':pincel:','it':':pennello:','fa':':قلم_مو:','id':':kuas:','zh':':画笔:','ru':':кисточка:','tr':':boya_fırçası:','ar':':فرشاة_ألوان:'},'\U0001FAF3':{'en':':palm_down_hand:','status':fully_qualified,'E':14,'de':':hand_mit_handfläche_nach_unten:','es':':mano_con_la_palma_hacia_abajo:','fr':':main_paume_vers_le_bas:','ja':':下に向けた手:','ko':':손바닥을_아래로_향한_손:','pt':':mão_com_a_palma_para_baixo:','it':':mano_con_il_palmo_verso_il_basso:','fa':':کف_دست_به_پایین:','id':':telapak_tangan_menghadap_ke_bawah:','zh':':掌心向下的手:','ru':':рука_ладонью_вниз:','tr':':avuç_içi_aşağı_bakan_el:','ar':':راحة_يد_للأسفل:'},'\U0001FAF3\U0001F3FF':{'en':':palm_down_hand_dark_skin_tone:','status':fully_qualified,'E':14,'de':':hand_mit_handfläche_nach_unten_dunkle_hautfarbe:','es':':mano_con_la_palma_hacia_abajo_tono_de_piel_oscuro:','fr':':main_paume_vers_le_bas_peau_foncée:','ja':':下に向けた手_濃い肌色:','ko':':손바닥을_아래로_향한_손_검은색_피부:','pt':':mão_com_a_palma_para_baixo_pele_escura:','it':':mano_con_il_palmo_verso_il_basso_carnagione_scura:','fa':':کف_دست_به_پایین_پوست_آبنوسی:','id':':telapak_tangan_menghadap_ke_bawah_warna_kulit_gelap:','zh':':掌心向下的手_较深肤色:','ru':':рука_ладонью_вниз_очень_темный_тон_кожи:','tr':':avuç_içi_aşağı_bakan_el_koyu_cilt_tonu:','ar':':راحة_يد_للأسفل_بشرة_بلون_غامق:'},'\U0001FAF3\U0001F3FB':{'en':':palm_down_hand_light_skin_tone:','status':fully_qualified,'E':14,'de':':hand_mit_handfläche_nach_unten_helle_hautfarbe:','es':':mano_con_la_palma_hacia_abajo_tono_de_piel_claro:','fr':':main_paume_vers_le_bas_peau_claire:','ja':':下に向けた手_薄い肌色:','ko':':손바닥을_아래로_향한_손_하얀_피부:','pt':':mão_com_a_palma_para_baixo_pele_clara:','it':':mano_con_il_palmo_verso_il_basso_carnagione_chiara:','fa':':کف_دست_به_پایین_پوست_سفید:','id':':telapak_tangan_menghadap_ke_bawah_warna_kulit_cerah:','zh':':掌心向下的手_较浅肤色:','ru':':рука_ладонью_вниз_очень_светлый_тон_кожи:','tr':':avuç_içi_aşağı_bakan_el_açık_cilt_tonu:','ar':':راحة_يد_للأسفل_بشرة_بلون_فاتح:'},'\U0001FAF3\U0001F3FE':{'en':':palm_down_hand_medium-dark_skin_tone:','status':fully_qualified,'E':14,'de':':hand_mit_handfläche_nach_unten_mitteldunkle_hautfarbe:','es':':mano_con_la_palma_hacia_abajo_tono_de_piel_oscuro_medio:','fr':':main_paume_vers_le_bas_peau_mate:','ja':':下に向けた手_やや濃い肌色:','ko':':손바닥을_아래로_향한_손_진한_갈색_피부:','pt':':mão_com_a_palma_para_baixo_pele_morena_escura:','it':':mano_con_il_palmo_verso_il_basso_carnagione_abbastanza_scura:','fa':':کف_دست_به_پایین_پوست_گندمی:','id':':telapak_tangan_menghadap_ke_bawah_warna_kulit_gelap-sedang:','zh':':掌心向下的手_中等深肤色:','ru':':рука_ладонью_вниз_темный_тон_кожи:','tr':':avuç_içi_aşağı_bakan_el_orta_koyu_cilt_tonu:','ar':':راحة_يد_للأسفل_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001FAF3\U0001F3FC':{'en':':palm_down_hand_medium-light_skin_tone:','status':fully_qualified,'E':14,'de':':hand_mit_handfläche_nach_unten_mittelhelle_hautfarbe:','es':':mano_con_la_palma_hacia_abajo_tono_de_piel_claro_medio:','fr':':main_paume_vers_le_bas_peau_moyennement_claire:','ja':':下に向けた手_やや薄い肌色:','ko':':손바닥을_아래로_향한_손_연한_갈색_피부:','pt':':mão_com_a_palma_para_baixo_pele_morena_clara:','it':':mano_con_il_palmo_verso_il_basso_carnagione_abbastanza_chiara:','fa':':کف_دست_به_پایین_پوست_روشن:','id':':telapak_tangan_menghadap_ke_bawah_warna_kulit_cerah-sedang:','zh':':掌心向下的手_中等浅肤色:','ru':':рука_ладонью_вниз_светлый_тон_кожи:','tr':':avuç_içi_aşağı_bakan_el_orta_açık_cilt_tonu:','ar':':راحة_يد_للأسفل_بشرة_بلون_فاتح_ومعتدل:'},'\U0001FAF3\U0001F3FD':{'en':':palm_down_hand_medium_skin_tone:','status':fully_qualified,'E':14,'de':':hand_mit_handfläche_nach_unten_mittlere_hautfarbe:','es':':mano_con_la_palma_hacia_abajo_tono_de_piel_medio:','fr':':main_paume_vers_le_bas_peau_légèrement_mate:','ja':':下に向けた手_中間の肌色:','ko':':손바닥을_아래로_향한_손_갈색_피부:','pt':':mão_com_a_palma_para_baixo_pele_morena:','it':':mano_con_il_palmo_verso_il_basso_carnagione_olivastra:','fa':':کف_دست_به_پایین_پوست_طلایی:','id':':telapak_tangan_menghadap_ke_bawah_warna_kulit_sedang:','zh':':掌心向下的手_中等肤色:','ru':':рука_ладонью_вниз_средний_тон_кожи:','tr':':avuç_içi_aşağı_bakan_el_orta_cilt_tonu:','ar':':راحة_يد_للأسفل_بشرة_بلون_معتدل:'},'\U0001F334':{'en':':palm_tree:','status':fully_qualified,'E':0.6,'de':':palme:','es':':palmera:','fr':':palmier:','ja':':ヤシの木:','ko':':야자수:','pt':':palmeira:','it':':palma:','fa':':نخل:','id':':pohon_palem:','zh':':棕榈树:','ru':':пальма:','tr':':palmiye_ağacı:','ar':':نخلة:'},'\U0001FAF4':{'en':':palm_up_hand:','status':fully_qualified,'E':14,'de':':hand_mit_handfläche_nach_oben:','es':':mano_con_la_palma_hacia_arriba:','fr':':main_paume_vers_le_haut:','ja':':上に向けた手:','ko':':손바닥을_위로_향한_손:','pt':':mão_com_a_palma_para_cima:','it':':mano_con_il_palmo_verso_l’alto:','fa':':کف_دست_به_بالا:','id':':telapak_tangan_menghadap_ke_atas:','zh':':掌心向上的手:','ru':':рука_ладонью_вверх:','tr':':avuç_içi_yukarı_bakan_el:','ar':':راحة_يد_للأعلى:'},'\U0001FAF4\U0001F3FF':{'en':':palm_up_hand_dark_skin_tone:','status':fully_qualified,'E':14,'de':':hand_mit_handfläche_nach_oben_dunkle_hautfarbe:','es':':mano_con_la_palma_hacia_arriba_tono_de_piel_oscuro:','fr':':main_paume_vers_le_haut_peau_foncée:','ja':':上に向けた手_濃い肌色:','ko':':손바닥을_위로_향한_손_검은색_피부:','pt':':mão_com_a_palma_para_cima_pele_escura:','it':':mano_con_il_palmo_verso_l’alto_carnagione_scura:','fa':':کف_دست_به_بالا_پوست_آبنوسی:','id':':telapak_tangan_menghadap_ke_atas_warna_kulit_gelap:','zh':':掌心向上的手_较深肤色:','ru':':рука_ладонью_вверх_очень_темный_тон_кожи:','tr':':avuç_içi_yukarı_bakan_el_koyu_cilt_tonu:','ar':':راحة_يد_للأعلى_بشرة_بلون_غامق:'},'\U0001FAF4\U0001F3FB':{'en':':palm_up_hand_light_skin_tone:','status':fully_qualified,'E':14,'de':':hand_mit_handfläche_nach_oben_helle_hautfarbe:','es':':mano_con_la_palma_hacia_arriba_tono_de_piel_claro:','fr':':main_paume_vers_le_haut_peau_claire:','ja':':上に向けた手_薄い肌色:','ko':':손바닥을_위로_향한_손_하얀_피부:','pt':':mão_com_a_palma_para_cima_pele_clara:','it':':mano_con_il_palmo_verso_l’alto_carnagione_chiara:','fa':':کف_دست_به_بالا_پوست_سفید:','id':':telapak_tangan_menghadap_ke_atas_warna_kulit_cerah:','zh':':掌心向上的手_较浅肤色:','ru':':рука_ладонью_вверх_очень_светлый_тон_кожи:','tr':':avuç_içi_yukarı_bakan_el_açık_cilt_tonu:','ar':':راحة_يد_للأعلى_بشرة_بلون_فاتح:'},'\U0001FAF4\U0001F3FE':{'en':':palm_up_hand_medium-dark_skin_tone:','status':fully_qualified,'E':14,'de':':hand_mit_handfläche_nach_oben_mitteldunkle_hautfarbe:','es':':mano_con_la_palma_hacia_arriba_tono_de_piel_oscuro_medio:','fr':':main_paume_vers_le_haut_peau_mate:','ja':':上に向けた手_やや濃い肌色:','ko':':손바닥을_위로_향한_손_진한_갈색_피부:','pt':':mão_com_a_palma_para_cima_pele_morena_escura:','it':':mano_con_il_palmo_verso_l’alto_carnagione_abbastanza_scura:','fa':':کف_دست_به_بالا_پوست_گندمی:','id':':telapak_tangan_menghadap_ke_atas_warna_kulit_gelap-sedang:','zh':':掌心向上的手_中等深肤色:','ru':':рука_ладонью_вверх_темный_тон_кожи:','tr':':avuç_içi_yukarı_bakan_el_orta_koyu_cilt_tonu:','ar':':راحة_يد_للأعلى_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001FAF4\U0001F3FC':{'en':':palm_up_hand_medium-light_skin_tone:','status':fully_qualified,'E':14,'de':':hand_mit_handfläche_nach_oben_mittelhelle_hautfarbe:','es':':mano_con_la_palma_hacia_arriba_tono_de_piel_claro_medio:','fr':':main_paume_vers_le_haut_peau_moyennement_claire:','ja':':上に向けた手_やや薄い肌色:','ko':':손바닥을_위로_향한_손_연한_갈색_피부:','pt':':mão_com_a_palma_para_cima_pele_morena_clara:','it':':mano_con_il_palmo_verso_l’alto_carnagione_abbastanza_chiara:','fa':':کف_دست_به_بالا_پوست_روشن:','id':':telapak_tangan_menghadap_ke_atas_warna_kulit_cerah-sedang:','zh':':掌心向上的手_中等浅肤色:','ru':':рука_ладонью_вверх_светлый_тон_кожи:','tr':':avuç_içi_yukarı_bakan_el_orta_açık_cilt_tonu:','ar':':راحة_يد_للأعلى_بشرة_بلون_فاتح_ومعتدل:'},'\U0001FAF4\U0001F3FD':{'en':':palm_up_hand_medium_skin_tone:','status':fully_qualified,'E':14,'de':':hand_mit_handfläche_nach_oben_mittlere_hautfarbe:','es':':mano_con_la_palma_hacia_arriba_tono_de_piel_medio:','fr':':main_paume_vers_le_haut_peau_légèrement_mate:','ja':':上に向けた手_中間の肌色:','ko':':손바닥을_위로_향한_손_갈색_피부:','pt':':mão_com_a_palma_para_cima_pele_morena:','it':':mano_con_il_palmo_verso_l’alto_carnagione_olivastra:','fa':':کف_دست_به_بالا_پوست_طلایی:','id':':telapak_tangan_menghadap_ke_atas_warna_kulit_sedang:','zh':':掌心向上的手_中等肤色:','ru':':рука_ладонью_вверх_средний_тон_кожи:','tr':':avuç_içi_yukarı_bakan_el_orta_cilt_tonu:','ar':':راحة_يد_للأعلى_بشرة_بلون_معتدل:'},'\U0001F932':{'en':':palms_up_together:','status':fully_qualified,'E':5,'de':':handflächen_nach_oben:','es':':palmas_hacia_arriba_juntas:','fr':':paume_contre_paume_doigts_vers_le_haut:','ja':':手のひらを揃えた両手:','ko':':내민_두_손:','pt':':palmas_unidas_para_cima:','it':':mani_unite_in_alto:','fa':':دست_دعا:','id':':tangan_diangkat_dan_dirapatkan_bersama:','zh':':掌心向上托起:','ru':':ладони_лодочкой:','tr':':iki_avuç_açık:','ar':':راحتان_مفتوحتان:'},'\U0001F932\U0001F3FF':{'en':':palms_up_together_dark_skin_tone:','status':fully_qualified,'E':5,'de':':handflächen_nach_oben_dunkle_hautfarbe:','es':':palmas_hacia_arriba_juntas_tono_de_piel_oscuro:','fr':':paume_contre_paume_doigts_vers_le_haut_peau_foncée:','ja':':手のひらを揃えた両手_濃い肌色:','ko':':내민_두_손_검은색_피부:','pt':':palmas_unidas_para_cima_pele_escura:','it':':mani_unite_in_alto_carnagione_scura:','fa':':دست_دعا_پوست_آبنوسی:','id':':tangan_diangkat_dan_dirapatkan_bersama_warna_kulit_gelap:','zh':':掌心向上托起_较深肤色:','ru':':ладони_лодочкой_очень_темный_тон_кожи:','tr':':iki_avuç_açık_koyu_cilt_tonu:','ar':':راحتان_مفتوحتان_بشرة_بلون_غامق:'},'\U0001F932\U0001F3FB':{'en':':palms_up_together_light_skin_tone:','status':fully_qualified,'E':5,'de':':handflächen_nach_oben_helle_hautfarbe:','es':':palmas_hacia_arriba_juntas_tono_de_piel_claro:','fr':':paume_contre_paume_doigts_vers_le_haut_peau_claire:','ja':':手のひらを揃えた両手_薄い肌色:','ko':':내민_두_손_하얀_피부:','pt':':palmas_unidas_para_cima_pele_clara:','it':':mani_unite_in_alto_carnagione_chiara:','fa':':دست_دعا_پوست_سفید:','id':':tangan_diangkat_dan_dirapatkan_bersama_warna_kulit_cerah:','zh':':掌心向上托起_较浅肤色:','ru':':ладони_лодочкой_очень_светлый_тон_кожи:','tr':':iki_avuç_açık_açık_cilt_tonu:','ar':':راحتان_مفتوحتان_بشرة_بلون_فاتح:'},'\U0001F932\U0001F3FE':{'en':':palms_up_together_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':handflächen_nach_oben_mitteldunkle_hautfarbe:','es':':palmas_hacia_arriba_juntas_tono_de_piel_oscuro_medio:','fr':':paume_contre_paume_doigts_vers_le_haut_peau_mate:','ja':':手のひらを揃えた両手_やや濃い肌色:','ko':':내민_두_손_진한_갈색_피부:','pt':':palmas_unidas_para_cima_pele_morena_escura:','it':':mani_unite_in_alto_carnagione_abbastanza_scura:','fa':':دست_دعا_پوست_گندمی:','id':':tangan_diangkat_dan_dirapatkan_bersama_warna_kulit_gelap-sedang:','zh':':掌心向上托起_中等深肤色:','ru':':ладони_лодочкой_темный_тон_кожи:','tr':':iki_avuç_açık_orta_koyu_cilt_tonu:','ar':':راحتان_مفتوحتان_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F932\U0001F3FC':{'en':':palms_up_together_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':handflächen_nach_oben_mittelhelle_hautfarbe:','es':':palmas_hacia_arriba_juntas_tono_de_piel_claro_medio:','fr':':paume_contre_paume_doigts_vers_le_haut_peau_moyennement_claire:','ja':':手のひらを揃えた両手_やや薄い肌色:','ko':':내민_두_손_연한_갈색_피부:','pt':':palmas_unidas_para_cima_pele_morena_clara:','it':':mani_unite_in_alto_carnagione_abbastanza_chiara:','fa':':دست_دعا_پوست_روشن:','id':':tangan_diangkat_dan_dirapatkan_bersama_warna_kulit_cerah-sedang:','zh':':掌心向上托起_中等浅肤色:','ru':':ладони_лодочкой_светлый_тон_кожи:','tr':':iki_avuç_açık_orta_açık_cilt_tonu:','ar':':راحتان_مفتوحتان_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F932\U0001F3FD':{'en':':palms_up_together_medium_skin_tone:','status':fully_qualified,'E':5,'de':':handflächen_nach_oben_mittlere_hautfarbe:','es':':palmas_hacia_arriba_juntas_tono_de_piel_medio:','fr':':paume_contre_paume_doigts_vers_le_haut_peau_légèrement_mate:','ja':':手のひらを揃えた両手_中間の肌色:','ko':':내민_두_손_갈색_피부:','pt':':palmas_unidas_para_cima_pele_morena:','it':':mani_unite_in_alto_carnagione_olivastra:','fa':':دست_دعا_پوست_طلایی:','id':':tangan_diangkat_dan_dirapatkan_bersama_warna_kulit_sedang:','zh':':掌心向上托起_中等肤色:','ru':':ладони_лодочкой_средний_тон_кожи:','tr':':iki_avuç_açık_orta_cilt_tonu:','ar':':راحتان_مفتوحتان_بشرة_بلون_معتدل:'},'\U0001F95E':{'en':':pancakes:','status':fully_qualified,'E':3,'de':':pfannkuchen:','es':':tortitas:','fr':':pancakes:','ja':':パンケーキ:','ko':':팬케이크:','pt':':panquecas:','it':':pancake:','fa':':پنکیک:','id':':panekuk:','zh':':烙饼:','ru':':блины:','tr':':krep:','ar':':فطائر:'},'\U0001F43C':{'en':':panda:','status':fully_qualified,'E':0.6,'alias':[':panda_face:'],'de':':panda:','es':':panda:','fr':':panda:','ja':':パンダの顔:','ko':':판다_얼굴:','pt':':rosto_de_panda:','it':':panda:','fa':':پاندا:','id':':panda:','zh':':熊猫:','ru':':морда_панды:','tr':':panda:','ar':':وجه_باندا:'},'\U0001F4CE':{'en':':paperclip:','status':fully_qualified,'E':0.6,'de':':büroklammer:','es':':clip:','fr':':trombone:','ja':':クリップ:','ko':':클립:','pt':':clipe_de_papel:','it':':graffetta:','fa':':گیرهٔ_کاغذ:','id':':klip_kertas:','zh':':回形针:','ru':':скрепка:','tr':':ataş:','ar':':مشبك_ورق:'},'\U0001FA82':{'en':':parachute:','status':fully_qualified,'E':12,'de':':fallschirm:','es':':paracaídas:','fr':':parachute:','ja':':パラシュート:','ko':':낙하산:','pt':':paraquedas:','it':':paracadute:','fa':':چتر_نجات:','id':':parasut:','zh':':降落伞:','ru':':парашют:','tr':':paraşüt:','ar':':باراشوت:'},'\U0001F99C':{'en':':parrot:','status':fully_qualified,'E':11,'de':':papagei:','es':':loro:','fr':':perroquet:','ja':':オウム:','ko':':앵무새:','pt':':papagaio:','it':':pappagallo:','fa':':طوطی:','id':':nuri:','zh':':鹦鹉:','ru':':попугай:','tr':':papağan:','ar':':ببغاء:'},'\U0000303D\U0000FE0F':{'en':':part_alternation_mark:','status':fully_qualified,'E':0.6,'variant':True,'de':':teilalternationszeichen:','es':':marca_de_alternancia:','fr':':alternance:','ja':':いおりてん:','ko':':부분_교체_기호:','pt':':sinal_japonês_indicando_início_de_música:','it':':simbolo_di_alternanza_delle_parti:','fa':':علامت_تناوب_بخش:','id':':tanda_alternasi_bagian:','zh':':庵点:','ru':':вступает_другои_голос:','tr':':parça_değiştirme_işareti:','ar':':بديل:'},'\U0000303D':{'en':':part_alternation_mark:','status':unqualified,'E':0.6,'variant':True,'de':':teilalternationszeichen:','es':':marca_de_alternancia:','fr':':alternance:','ja':':いおりてん:','ko':':부분_교체_기호:','pt':':sinal_japonês_indicando_início_de_música:','it':':simbolo_di_alternanza_delle_parti:','fa':':علامت_تناوب_بخش:','id':':tanda_alternasi_bagian:','zh':':庵点:','ru':':вступает_другой_голос:','tr':':parça_değiştirme_işareti:','ar':':بديل:'},'\U0001F389':{'en':':party_popper:','status':fully_qualified,'E':0.6,'alias':[':tada:'],'de':':konfettibombe:','es':':cañón_de_confeti:','fr':':cotillons:','ja':':クラッカー:','ko':':파티:','pt':':cone_de_festa:','it':':spara_coriandoli:','fa':':بمب_کاغذ_رنگی:','id':':party_popper:','zh':':拉炮彩带:','ru':':хлопушка:','tr':':parti_konfetisi:','ar':':فرقعة_حفلة:'},'\U0001F973':{'en':':partying_face:','status':fully_qualified,'E':11,'de':':partygesicht:','es':':cara_de_fiesta:','fr':':visage_festif:','ja':':パーティーの顔:','ko':':파티하는_얼굴:','pt':':rosto_festivo:','it':':faccina_che_festeggia:','fa':':جشن_و_سور:','id':':wajah_pesta:','zh':':聚会笑脸:','ru':':на_вечеринке:','tr':':partide_eğlenen_yüz:','ar':':وجه_محتفل:'},'\U0001F6F3\U0000FE0F':{'en':':passenger_ship:','status':fully_qualified,'E':0.7,'variant':True,'de':':passagierschiff:','es':':barco_de_pasajeros:','fr':':paquebot:','ja':':客船:','ko':':여객선:','pt':':cruzeiro:','it':':nave_da_crociera:','fa':':کشتی_مسافربری:','id':':kapal_penumpang:','zh':':客轮:','ru':':пассажирскии_лаинер:','tr':':yolcu_gemisi:','ar':':سفينة_ركاب:'},'\U0001F6F3':{'en':':passenger_ship:','status':unqualified,'E':0.7,'variant':True,'de':':passagierschiff:','es':':barco_de_pasajeros:','fr':':paquebot:','ja':':客船:','ko':':여객선:','pt':':cruzeiro:','it':':nave_da_crociera:','fa':':کشتی_مسافربری:','id':':kapal_penumpang:','zh':':客轮:','ru':':пассажирский_лайнер:','tr':':yolcu_gemisi:','ar':':سفينة_ركاب:'},'\U0001F6C2':{'en':':passport_control:','status':fully_qualified,'E':1,'de':':passkontrolle:','es':':control_de_pasaportes:','fr':':contrôle_des_passeports:','ja':':出入国審査:','ko':':출입국_심사:','pt':':controle_de_passaportes:','it':':simbolo_del_controllo_passaporti:','fa':':کنترل_گذرنامه:','id':':kontrol_paspor:','zh':':护照检查:','ru':':паспортный_контроль:','tr':':pasaport_kontrolü:','ar':':مراقبة_جوازات_السفر:'},'\U000023F8\U0000FE0F':{'en':':pause_button:','status':fully_qualified,'E':0.7,'alias':[':double_vertical_bar:'],'variant':True,'de':':pause:','es':':pausa:','fr':':bouton_pause:','ja':':一時停止ボタン:','ko':':일시_정지_버튼:','pt':':botão_pausar:','it':':pulsante_pausa:','fa':':دکمه_مکث:','id':':tombol_jeda:','zh':':暂停按钮:','ru':':пауза:','tr':':duraklatma_düğmesi:','ar':':زر_إيقاف_مؤقت:'},'\U000023F8':{'en':':pause_button:','status':unqualified,'E':0.7,'alias':[':double_vertical_bar:'],'variant':True,'de':':pause:','es':':pausa:','fr':':bouton_pause:','ja':':一時停止ボタン:','ko':':일시_정지_버튼:','pt':':botão_pausar:','it':':pulsante_pausa:','fa':':دکمهٔ_مکث:','id':':tombol_jeda:','zh':':暂停按钮:','ru':':пауза:','tr':':duraklatma_düğmesi:','ar':':زر_إيقاف_مؤقت:'},'\U0001F43E':{'en':':paw_prints:','status':fully_qualified,'E':0.6,'alias':[':feet:'],'de':':tatzenabdrücke:','es':':huellas_de_pezuñas:','fr':':empreintes_d’animaux:','ja':':肉球:','ko':':곰발바닥:','pt':':patas:','it':':impronte_di_zampe:','fa':':رد_پنجه:','id':':jejak_kaki_hewan:','zh':':爪印:','ru':':следы_животного:','tr':':pençe_izi:','ar':':آثار_حوافر:'},'\U0001FADB':{'en':':pea_pod:','status':fully_qualified,'E':15,'de':':erbsenschote:','es':':vaina:','fr':':cosse_de_petits_pois:','ja':':エンドウ豆:','ko':':완두콩_꼬투리:','pt':':vagem:','it':':baccello_di_piselli:','fa':':نخودفرنگی:','id':':kacang_polong:','zh':':豌豆荚:','ru':':стручок_гороха:','tr':':kabuklu_bezelye:','ar':':قرن_بازلاء:'},'\U0000262E\U0000FE0F':{'en':':peace_symbol:','status':fully_qualified,'E':1,'variant':True,'de':':friedenszeichen:','es':':símbolo_de_la_paz:','fr':':symbole_de_paix:','ja':':ピースマーク:','ko':':평화_기호:','pt':':símbolo_da_paz:','it':':simbolo_della_pace:','fa':':نماد_صلح:','id':':simbol_damai:','zh':':和平符号:','ru':':пацифик:','tr':':barış_sembolü:','ar':':رمز_سلام:'},'\U0000262E':{'en':':peace_symbol:','status':unqualified,'E':1,'variant':True,'de':':friedenszeichen:','es':':símbolo_de_la_paz:','fr':':symbole_de_paix:','ja':':ピースマーク:','ko':':평화_기호:','pt':':símbolo_da_paz:','it':':simbolo_della_pace:','fa':':نماد_صلح:','id':':simbol_damai:','zh':':和平符号:','ru':':пацифик:','tr':':barış_sembolü:','ar':':رمز_سلام:'},'\U0001F351':{'en':':peach:','status':fully_qualified,'E':0.6,'de':':pfirsich:','es':':melocotón:','fr':':pêche:','ja':':桃:','ko':':복숭아:','pt':':pêssego:','it':':pesca:','fa':':هلو:','id':':persik:','zh':':桃:','ru':':персик:','tr':':şeftali:','ar':':خوخ:'},'\U0001F99A':{'en':':peacock:','status':fully_qualified,'E':11,'de':':pfau:','es':':pavo_real:','fr':':paon:','ja':':クジャク:','ko':':공작:','pt':':pavão:','it':':pavone:','fa':':طاووس:','id':':merak:','zh':':孔雀:','ru':':павлин:','tr':':tavus_kuşu:','ar':':طاووس:'},'\U0001F95C':{'en':':peanuts:','status':fully_qualified,'E':3,'de':':erdnuss:','es':':cacahuetes:','fr':':cacahuètes:','ja':':ピーナッツ:','ko':':땅콩:','pt':':amendoim:','it':':arachidi:','fa':':بادام_زمینی:','id':':kacang_tanah:','zh':':花生:','ru':':арахис:','tr':':yer_fıstığı:','ar':':فول_سوداني:'},'\U0001F350':{'en':':pear:','status':fully_qualified,'E':1,'de':':birne:','es':':pera:','fr':':poire:','ja':':洋ナシ:','ko':':배:','pt':':pera:','it':':pera:','fa':':گلابی:','id':':pir:','zh':':梨:','ru':':груша:','tr':':armut:','ar':':كمثرى:'},'\U0001F58A\U0000FE0F':{'en':':pen:','status':fully_qualified,'E':0.7,'alias':[':lower_left_ballpoint_pen:'],'variant':True,'de':':kugelschreiber:','es':':bolígrafo:','fr':':stylo:','ja':':ペン:','ko':':펜:','pt':':caneta:','it':':penna_a_sfera:','fa':':خودکار:','id':':pena:','zh':':笔:','ru':':шариковая_ручка:','tr':':kalem:','ar':':قلم:'},'\U0001F58A':{'en':':pen:','status':unqualified,'E':0.7,'alias':[':lower_left_ballpoint_pen:'],'variant':True,'de':':kugelschreiber:','es':':bolígrafo:','fr':':stylo:','ja':':ペン:','ko':':펜:','pt':':caneta:','it':':penna_a_sfera:','fa':':خودکار:','id':':pena:','zh':':笔:','ru':':шариковая_ручка:','tr':':kalem:','ar':':قلم:'},'\U0000270F\U0000FE0F':{'en':':pencil:','status':fully_qualified,'E':0.6,'alias':[':pencil2:'],'variant':True,'de':':bleistift:','es':':lápiz:','fr':':crayon:','ja':':鉛筆:','ko':':연필:','pt':':lápis:','it':':matita:','fa':':مداد:','id':':pensil:','zh':':铅笔:','ru':':карандаш:','tr':':kurşun_kalem:','ar':':قلم_رصاص:'},'\U0000270F':{'en':':pencil:','status':unqualified,'E':0.6,'alias':[':pencil2:'],'variant':True,'de':':bleistift:','es':':lápiz:','fr':':crayon:','ja':':鉛筆:','ko':':연필:','pt':':lápis:','it':':matita:','fa':':مداد:','id':':pensil:','zh':':铅笔:','ru':':карандаш:','tr':':kurşun_kalem:','ar':':قلم_رصاص:'},'\U0001F427':{'en':':penguin:','status':fully_qualified,'E':0.6,'de':':pinguin:','es':':pingüino:','fr':':pingouin:','ja':':ペンギン:','ko':':펭귄:','pt':':pinguim:','it':':pinguino:','fa':':پنگوئن:','id':':pinguin:','zh':':企鹅:','ru':':пингвин:','tr':':penguen:','ar':':بطريق:'},'\U0001F614':{'en':':pensive_face:','status':fully_qualified,'E':0.6,'alias':[':pensive:'],'de':':nachdenkliches_gesicht:','es':':cara_desanimada:','fr':':visage_pensif:','ja':':しょぼーん:','ko':':수심_어린_얼굴:','pt':':rosto_deprimido:','it':':faccina_pensierosa:','fa':':چهره_غمگین:','id':':wajah_merenung:','zh':':沉思:','ru':':размышляет:','tr':':düşünceli_yüz:','ar':':وجه_متأمل:'},'\U0001F9D1\U0000200D\U0001F91D\U0000200D\U0001F9D1':{'en':':people_holding_hands:','status':fully_qualified,'E':12,'de':':sich_an_den_händen_haltende_personen:','es':':dos_personas_de_la_mano:','fr':':deux_personnes_se_tenant_la_main:','ja':':手をつなぐ2人:','ko':':손을_잡고_있는_두_명의_사람:','pt':':pessoas_de_mãos_dadas:','it':':persone_che_si_tengono_per_mano:','fa':':اشخاصی_دست_در_دست_هم:','id':':orang_bergandengan:','zh':':手拉手的两个人:','ru':':люди_держащиеся_за_руки:','tr':':el_ele_tutuşan_insanlar:','ar':':أشخاص_يمسكون_بأيدي_بعضهم:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FF':{'en':':people_holding_hands_dark_skin_tone:','status':fully_qualified,'E':12,'de':':sich_an_den_händen_haltende_personen_dunkle_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_oscuro:','fr':':deux_personnes_se_tenant_la_main_peau_foncée:','ja':':手をつなぐ2人_濃い肌色:','ko':':손을_잡고_있는_두_명의_사람_검은색_피부:','pt':':pessoas_de_mãos_dadas_pele_escura:','it':':persone_che_si_tengono_per_mano_carnagione_scura:','fa':':اشخاصی_دست_در_دست_هم_پوست_آبنوسی:','id':':orang_bergandengan_warna_kulit_gelap:','zh':':手拉手的两个人_较深肤色:','ru':':люди_держащиеся_за_руки_очень_темный_тон_кожи:','tr':':el_ele_tutuşan_insanlar_koyu_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FB':{'en':':people_holding_hands_dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':12,'de':':sich_an_den_händen_haltende_personen_dunkle_hautfarbe_helle_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_oscuro_y_tono_de_piel_claro:','fr':':deux_personnes_se_tenant_la_main_peau_foncée_et_peau_claire:','ja':':手をつなぐ2人_濃い肌色_薄い肌色:','ko':':손을_잡고_있는_두_명의_사람_검은색_피부_하얀_피부:','pt':':pessoas_de_mãos_dadas_pele_escura_e_pele_clara:','it':':persone_che_si_tengono_per_mano_carnagione_scura_e_carnagione_chiara:','fa':':اشخاصی_دست_در_دست_هم_پوست_آبنوسی_پوست_سفید:','id':':orang_bergandengan_warna_kulit_gelap_warna_kulit_cerah:','zh':':手拉手的两个人_较深肤色较浅肤色:','ru':':люди_держащиеся_за_руки_очень_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':el_ele_tutuşan_insanlar_koyu_cilt_tonu_açık_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FE':{'en':':people_holding_hands_dark_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':sich_an_den_händen_haltende_personen_dunkle_hautfarbe_mitteldunkle_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_oscuro_y_tono_de_piel_oscuro_medio:','fr':':deux_personnes_se_tenant_la_main_peau_foncée_et_peau_mate:','ja':':手をつなぐ2人_濃い肌色_やや濃い肌色:','ko':':손을_잡고_있는_두_명의_사람_검은색_피부_진한_갈색_피부:','pt':':pessoas_de_mãos_dadas_pele_escura_e_pele_morena_escura:','it':':persone_che_si_tengono_per_mano_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':اشخاصی_دست_در_دست_هم_پوست_آبنوسی_پوست_گندمی:','id':':orang_bergandengan_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':手拉手的两个人_较深肤色中等深肤色:','ru':':люди_держащиеся_за_руки_очень_темный_тон_кожи_темный_тон_кожи:','tr':':el_ele_tutuşan_insanlar_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FC':{'en':':people_holding_hands_dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':sich_an_den_händen_haltende_personen_dunkle_hautfarbe_mittelhelle_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_oscuro_y_tono_de_piel_claro_medio:','fr':':deux_personnes_se_tenant_la_main_peau_foncée_et_peau_moyennement_claire:','ja':':手をつなぐ2人_濃い肌色_やや薄い肌色:','ko':':손을_잡고_있는_두_명의_사람_검은색_피부_연한_갈색_피부:','pt':':pessoas_de_mãos_dadas_pele_escura_e_pele_morena_clara:','it':':persone_che_si_tengono_per_mano_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':اشخاصی_دست_در_دست_هم_پوست_آبنوسی_پوست_روشن:','id':':orang_bergandengan_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':手拉手的两个人_较深肤色中等浅肤色:','ru':':люди_держащиеся_за_руки_очень_темный_тон_кожи_светлый_тон_кожи:','tr':':el_ele_tutuşan_insanlar_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FD':{'en':':people_holding_hands_dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':12,'de':':sich_an_den_händen_haltende_personen_dunkle_hautfarbe_mittlere_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_oscuro_y_tono_de_piel_medio:','fr':':deux_personnes_se_tenant_la_main_peau_foncée_et_peau_légèrement_mate:','ja':':手をつなぐ2人_濃い肌色_中間の肌色:','ko':':손을_잡고_있는_두_명의_사람_검은색_피부_갈색_피부:','pt':':pessoas_de_mãos_dadas_pele_escura_e_pele_morena:','it':':persone_che_si_tengono_per_mano_carnagione_scura_e_carnagione_olivastra:','fa':':اشخاصی_دست_در_دست_هم_پوست_آبنوسی_پوست_طلایی:','id':':orang_bergandengan_warna_kulit_gelap_warna_kulit_sedang:','zh':':手拉手的两个人_较深肤色中等肤色:','ru':':люди_держащиеся_за_руки_очень_темный_тон_кожи_средний_тон_кожи:','tr':':el_ele_tutuşan_insanlar_koyu_cilt_tonu_orta_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FB':{'en':':people_holding_hands_light_skin_tone:','status':fully_qualified,'E':12,'de':':sich_an_den_händen_haltende_personen_helle_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_claro:','fr':':deux_personnes_se_tenant_la_main_peau_claire:','ja':':手をつなぐ2人_薄い肌色:','ko':':손을_잡고_있는_두_명의_사람_하얀_피부:','pt':':pessoas_de_mãos_dadas_pele_clara:','it':':persone_che_si_tengono_per_mano_carnagione_chiara:','fa':':اشخاصی_دست_در_دست_هم_پوست_سفید:','id':':orang_bergandengan_warna_kulit_cerah:','zh':':手拉手的两个人_较浅肤色:','ru':':люди_держащиеся_за_руки_очень_светлый_тон_кожи:','tr':':el_ele_tutuşan_insanlar_açık_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FF':{'en':':people_holding_hands_light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':sich_an_den_händen_haltende_personen_helle_hautfarbe_dunkle_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_claro_y_tono_de_piel_oscuro:','fr':':deux_personnes_se_tenant_la_main_peau_claire_et_peau_foncée:','ja':':手をつなぐ2人_薄い肌色_濃い肌色:','ko':':손을_잡고_있는_두_명의_사람_하얀_피부_검은색_피부:','pt':':pessoas_de_mãos_dadas_pele_clara_e_pele_escura:','it':':persone_che_si_tengono_per_mano_carnagione_chiara_e_carnagione_scura:','fa':':اشخاصی_دست_در_دست_هم_پوست_سفید_پوست_آبنوسی:','id':':orang_bergandengan_warna_kulit_cerah_warna_kulit_gelap:','zh':':手拉手的两个人_较浅肤色较深肤色:','ru':':люди_держащиеся_за_руки_очень_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':el_ele_tutuşan_insanlar_açık_cilt_tonu_koyu_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FE':{'en':':people_holding_hands_light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':sich_an_den_händen_haltende_personen_helle_hautfarbe_mitteldunkle_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_claro_y_tono_de_piel_oscuro_medio:','fr':':deux_personnes_se_tenant_la_main_peau_claire_et_peau_mate:','ja':':手をつなぐ2人_薄い肌色_やや濃い肌色:','ko':':손을_잡고_있는_두_명의_사람_하얀_피부_진한_갈색_피부:','pt':':pessoas_de_mãos_dadas_pele_clara_e_pele_morena_escura:','it':':persone_che_si_tengono_per_mano_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':اشخاصی_دست_در_دست_هم_پوست_سفید_پوست_گندمی:','id':':orang_bergandengan_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':手拉手的两个人_较浅肤色中等深肤色:','ru':':люди_держащиеся_за_руки_очень_светлый_тон_кожи_темный_тон_кожи:','tr':':el_ele_tutuşan_insanlar_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FC':{'en':':people_holding_hands_light_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':sich_an_den_händen_haltende_personen_helle_hautfarbe_mittelhelle_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_claro_y_tono_de_piel_claro_medio:','fr':':deux_personnes_se_tenant_la_main_peau_claire_et_peau_moyennement_claire:','ja':':手をつなぐ2人_薄い肌色_やや薄い肌色:','ko':':손을_잡고_있는_두_명의_사람_하얀_피부_연한_갈색_피부:','pt':':pessoas_de_mãos_dadas_pele_clara_e_pele_morena_clara:','it':':persone_che_si_tengono_per_mano_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':اشخاصی_دست_در_دست_هم_پوست_سفید_پوست_روشن:','id':':orang_bergandengan_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':手拉手的两个人_较浅肤色中等浅肤色:','ru':':люди_держащиеся_за_руки_очень_светлый_тон_кожи_светлый_тон_кожи:','tr':':el_ele_tutuşan_insanlar_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FD':{'en':':people_holding_hands_light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':sich_an_den_händen_haltende_personen_helle_hautfarbe_mittlere_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_claro_y_tono_de_piel_medio:','fr':':deux_personnes_se_tenant_la_main_peau_claire_et_peau_légèrement_mate:','ja':':手をつなぐ2人_薄い肌色_中間の肌色:','ko':':손을_잡고_있는_두_명의_사람_하얀_피부_갈색_피부:','pt':':pessoas_de_mãos_dadas_pele_clara_e_pele_morena:','it':':persone_che_si_tengono_per_mano_carnagione_chiara_e_carnagione_olivastra:','fa':':اشخاصی_دست_در_دست_هم_پوست_سفید_پوست_طلایی:','id':':orang_bergandengan_warna_kulit_cerah_warna_kulit_sedang:','zh':':手拉手的两个人_较浅肤色中等肤色:','ru':':люди_держащиеся_за_руки_очень_светлый_тон_кожи_средний_тон_кожи:','tr':':el_ele_tutuşan_insanlar_açık_cilt_tonu_orta_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FE':{'en':':people_holding_hands_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':sich_an_den_händen_haltende_personen_mitteldunkle_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_oscuro_medio:','fr':':deux_personnes_se_tenant_la_main_peau_mate:','ja':':手をつなぐ2人_やや濃い肌色:','ko':':손을_잡고_있는_두_명의_사람_진한_갈색_피부:','pt':':pessoas_de_mãos_dadas_pele_morena_escura:','it':':persone_che_si_tengono_per_mano_carnagione_abbastanza_scura:','fa':':اشخاصی_دست_در_دست_هم_پوست_گندمی:','id':':orang_bergandengan_warna_kulit_gelap-sedang:','zh':':手拉手的两个人_中等深肤色:','ru':':люди_держащиеся_за_руки_темный_тон_кожи:','tr':':el_ele_tutuşan_insanlar_orta_koyu_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FF':{'en':':people_holding_hands_medium-dark_skin_tone_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':sich_an_den_händen_haltende_personen_mitteldunkle_hautfarbe_dunkle_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_oscuro_medio_y_tono_de_piel_oscuro:','fr':':deux_personnes_se_tenant_la_main_peau_mate_et_peau_foncée:','ja':':手をつなぐ2人_やや濃い肌色_濃い肌色:','ko':':손을_잡고_있는_두_명의_사람_진한_갈색_피부_검은색_피부:','pt':':pessoas_de_mãos_dadas_pele_morena_escura_e_pele_escura:','it':':persone_che_si_tengono_per_mano_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':اشخاصی_دست_در_دست_هم_پوست_گندمی_پوست_آبنوسی:','id':':orang_bergandengan_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':手拉手的两个人_中等深肤色较深肤色:','ru':':люди_держащиеся_за_руки_темный_тон_кожи_очень_темный_тон_кожи:','tr':':el_ele_tutuşan_insanlar_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FB':{'en':':people_holding_hands_medium-dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':12,'de':':sich_an_den_händen_haltende_personen_mitteldunkle_hautfarbe_helle_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_oscuro_medio_y_tono_de_piel_claro:','fr':':deux_personnes_se_tenant_la_main_peau_mate_et_peau_claire:','ja':':手をつなぐ2人_やや濃い肌色_薄い肌色:','ko':':손을_잡고_있는_두_명의_사람_진한_갈색_피부_하얀_피부:','pt':':pessoas_de_mãos_dadas_pele_morena_escura_e_pele_clara:','it':':persone_che_si_tengono_per_mano_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':اشخاصی_دست_در_دست_هم_پوست_گندمی_پوست_سفید:','id':':orang_bergandengan_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':手拉手的两个人_中等深肤色较浅肤色:','ru':':люди_держащиеся_за_руки_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':el_ele_tutuşan_insanlar_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FC':{'en':':people_holding_hands_medium-dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':sich_an_den_händen_haltende_personen_mitteldunkle_hautfarbe_mittelhelle_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_oscuro_medio_y_tono_de_piel_claro_medio:','fr':':deux_personnes_se_tenant_la_main_peau_mate_et_peau_moyennement_claire:','ja':':手をつなぐ2人_やや濃い肌色_やや薄い肌色:','ko':':손을_잡고_있는_두_명의_사람_진한_갈색_피부_연한_갈색_피부:','pt':':pessoas_de_mãos_dadas_pele_morena_escura_e_pele_morena_clara:','it':':persone_che_si_tengono_per_mano_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':اشخاصی_دست_در_دست_هم_پوست_گندمی_پوست_روشن:','id':':orang_bergandengan_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':手拉手的两个人_中等深肤色中等浅肤色:','ru':':люди_держащиеся_за_руки_темный_тон_кожи_светлый_тон_кожи:','tr':':el_ele_tutuşan_insanlar_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FD':{'en':':people_holding_hands_medium-dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':12,'de':':sich_an_den_händen_haltende_personen_mitteldunkle_hautfarbe_mittlere_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_oscuro_medio_y_tono_de_piel_medio:','fr':':deux_personnes_se_tenant_la_main_peau_mate_et_peau_légèrement_mate:','ja':':手をつなぐ2人_やや濃い肌色_中間の肌色:','ko':':손을_잡고_있는_두_명의_사람_진한_갈색_피부_갈색_피부:','pt':':pessoas_de_mãos_dadas_pele_morena_escura_e_pele_morena:','it':':persone_che_si_tengono_per_mano_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':اشخاصی_دست_در_دست_هم_پوست_گندمی_پوست_طلایی:','id':':orang_bergandengan_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':手拉手的两个人_中等深肤色中等肤色:','ru':':люди_держащиеся_за_руки_темный_тон_кожи_средний_тон_кожи:','tr':':el_ele_tutuşan_insanlar_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FC':{'en':':people_holding_hands_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':sich_an_den_händen_haltende_personen_mittelhelle_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_claro_medio:','fr':':deux_personnes_se_tenant_la_main_peau_moyennement_claire:','ja':':手をつなぐ2人_やや薄い肌色:','ko':':손을_잡고_있는_두_명의_사람_연한_갈색_피부:','pt':':pessoas_de_mãos_dadas_pele_morena_clara:','it':':persone_che_si_tengono_per_mano_carnagione_abbastanza_chiara:','fa':':اشخاصی_دست_در_دست_هم_پوست_روشن:','id':':orang_bergandengan_warna_kulit_cerah-sedang:','zh':':手拉手的两个人_中等浅肤色:','ru':':люди_держащиеся_за_руки_светлый_тон_кожи:','tr':':el_ele_tutuşan_insanlar_orta_açık_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FF':{'en':':people_holding_hands_medium-light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':sich_an_den_händen_haltende_personen_mittelhelle_hautfarbe_dunkle_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_claro_medio_y_tono_de_piel_oscuro:','fr':':deux_personnes_se_tenant_la_main_peau_moyennement_claire_et_peau_foncée:','ja':':手をつなぐ2人_やや薄い肌色_濃い肌色:','ko':':손을_잡고_있는_두_명의_사람_연한_갈색_피부_검은색_피부:','pt':':pessoas_de_mãos_dadas_pele_morena_clara_e_pele_escura:','it':':persone_che_si_tengono_per_mano_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':اشخاصی_دست_در_دست_هم_پوست_روشن_پوست_آبنوسی:','id':':orang_bergandengan_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':手拉手的两个人_中等浅肤色较深肤色:','ru':':люди_держащиеся_за_руки_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':el_ele_tutuşan_insanlar_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FB':{'en':':people_holding_hands_medium-light_skin_tone_light_skin_tone:','status':fully_qualified,'E':12,'de':':sich_an_den_händen_haltende_personen_mittelhelle_hautfarbe_helle_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_claro_medio_y_tono_de_piel_claro:','fr':':deux_personnes_se_tenant_la_main_peau_moyennement_claire_et_peau_claire:','ja':':手をつなぐ2人_やや薄い肌色_薄い肌色:','ko':':손을_잡고_있는_두_명의_사람_연한_갈색_피부_하얀_피부:','pt':':pessoas_de_mãos_dadas_pele_morena_clara_e_pele_clara:','it':':persone_che_si_tengono_per_mano_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':اشخاصی_دست_در_دست_هم_پوست_روشن_پوست_سفید:','id':':orang_bergandengan_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':手拉手的两个人_中等浅肤色较浅肤色:','ru':':люди_держащиеся_за_руки_светлый_тон_кожи_очень_светлый_тон_кожи:','tr':':el_ele_tutuşan_insanlar_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FE':{'en':':people_holding_hands_medium-light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':sich_an_den_händen_haltende_personen_mittelhelle_hautfarbe_mitteldunkle_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_claro_medio_y_tono_de_piel_oscuro_medio:','fr':':deux_personnes_se_tenant_la_main_peau_moyennement_claire_et_peau_mate:','ja':':手をつなぐ2人_やや薄い肌色_やや濃い肌色:','ko':':손을_잡고_있는_두_명의_사람_연한_갈색_피부_진한_갈색_피부:','pt':':pessoas_de_mãos_dadas_pele_morena_clara_e_pele_morena_escura:','it':':persone_che_si_tengono_per_mano_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':اشخاصی_دست_در_دست_هم_پوست_روشن_پوست_گندمی:','id':':orang_bergandengan_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':手拉手的两个人_中等浅肤色中等深肤色:','ru':':люди_держащиеся_за_руки_светлый_тон_кожи_темный_тон_кожи:','tr':':el_ele_tutuşan_insanlar_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FD':{'en':':people_holding_hands_medium-light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':sich_an_den_händen_haltende_personen_mittelhelle_hautfarbe_mittlere_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_claro_medio_y_tono_de_piel_medio:','fr':':deux_personnes_se_tenant_la_main_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':手をつなぐ2人_やや薄い肌色_中間の肌色:','ko':':손을_잡고_있는_두_명의_사람_연한_갈색_피부_갈색_피부:','pt':':pessoas_de_mãos_dadas_pele_morena_clara_e_pele_morena:','it':':persone_che_si_tengono_per_mano_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':اشخاصی_دست_در_دست_هم_پوست_روشن_پوست_طلایی:','id':':orang_bergandengan_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':手拉手的两个人_中等浅肤色中等肤色:','ru':':люди_держащиеся_за_руки_светлый_тон_кожи_средний_тон_кожи:','tr':':el_ele_tutuşan_insanlar_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FD':{'en':':people_holding_hands_medium_skin_tone:','status':fully_qualified,'E':12,'de':':sich_an_den_händen_haltende_personen_mittlere_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_medio:','fr':':deux_personnes_se_tenant_la_main_peau_légèrement_mate:','ja':':手をつなぐ2人_中間の肌色:','ko':':손을_잡고_있는_두_명의_사람_갈색_피부:','pt':':pessoas_de_mãos_dadas_pele_morena:','it':':persone_che_si_tengono_per_mano_carnagione_olivastra:','fa':':اشخاصی_دست_در_دست_هم_پوست_طلایی:','id':':orang_bergandengan_warna_kulit_sedang:','zh':':手拉手的两个人_中等肤色:','ru':':люди_держащиеся_за_руки_средний_тон_кожи:','tr':':el_ele_tutuşan_insanlar_orta_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FF':{'en':':people_holding_hands_medium_skin_tone_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':sich_an_den_händen_haltende_personen_mittlere_hautfarbe_dunkle_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_medio_y_tono_de_piel_oscuro:','fr':':deux_personnes_se_tenant_la_main_peau_légèrement_mate_et_peau_foncée:','ja':':手をつなぐ2人_中間の肌色_濃い肌色:','ko':':손을_잡고_있는_두_명의_사람_갈색_피부_검은색_피부:','pt':':pessoas_de_mãos_dadas_pele_morena_e_pele_escura:','it':':persone_che_si_tengono_per_mano_carnagione_olivastra_e_carnagione_scura:','fa':':اشخاصی_دست_در_دست_هم_پوست_طلایی_پوست_آبنوسی:','id':':orang_bergandengan_warna_kulit_sedang_warna_kulit_gelap:','zh':':手拉手的两个人_中等肤色较深肤色:','ru':':люди_держащиеся_за_руки_средний_тон_кожи_очень_темный_тон_кожи:','tr':':el_ele_tutuşan_insanlar_orta_cilt_tonu_koyu_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FB':{'en':':people_holding_hands_medium_skin_tone_light_skin_tone:','status':fully_qualified,'E':12,'de':':sich_an_den_händen_haltende_personen_mittlere_hautfarbe_helle_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_medio_y_tono_de_piel_claro:','fr':':deux_personnes_se_tenant_la_main_peau_légèrement_mate_et_peau_claire:','ja':':手をつなぐ2人_中間の肌色_薄い肌色:','ko':':손을_잡고_있는_두_명의_사람_갈색_피부_하얀_피부:','pt':':pessoas_de_mãos_dadas_pele_morena_e_pele_clara:','it':':persone_che_si_tengono_per_mano_carnagione_olivastra_e_carnagione_chiara:','fa':':اشخاصی_دست_در_دست_هم_پوست_طلایی_پوست_سفید:','id':':orang_bergandengan_warna_kulit_sedang_warna_kulit_cerah:','zh':':手拉手的两个人_中等肤色较浅肤色:','ru':':люди_держащиеся_за_руки_средний_тон_кожи_очень_светлый_тон_кожи:','tr':':el_ele_tutuşan_insanlar_orta_cilt_tonu_açık_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FE':{'en':':people_holding_hands_medium_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':sich_an_den_händen_haltende_personen_mittlere_hautfarbe_mitteldunkle_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_medio_y_tono_de_piel_oscuro_medio:','fr':':deux_personnes_se_tenant_la_main_peau_légèrement_mate_et_peau_mate:','ja':':手をつなぐ2人_中間の肌色_やや濃い肌色:','ko':':손을_잡고_있는_두_명의_사람_갈색_피부_진한_갈색_피부:','pt':':pessoas_de_mãos_dadas_pele_morena_e_pele_morena_escura:','it':':persone_che_si_tengono_per_mano_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':اشخاصی_دست_در_دست_هم_پوست_طلایی_پوست_گندمی:','id':':orang_bergandengan_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':手拉手的两个人_中等肤色中等深肤色:','ru':':люди_держащиеся_за_руки_средний_тон_кожи_темный_тон_кожи:','tr':':el_ele_tutuşan_insanlar_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F9D1\U0001F3FC':{'en':':people_holding_hands_medium_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':sich_an_den_händen_haltende_personen_mittlere_hautfarbe_mittelhelle_hautfarbe:','es':':dos_personas_de_la_mano_tono_de_piel_medio_y_tono_de_piel_claro_medio:','fr':':deux_personnes_se_tenant_la_main_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':手をつなぐ2人_中間の肌色_やや薄い肌色:','ko':':손을_잡고_있는_두_명의_사람_갈색_피부_연한_갈색_피부:','pt':':pessoas_de_mãos_dadas_pele_morena_e_pele_morena_clara:','it':':persone_che_si_tengono_per_mano_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':اشخاصی_دست_در_دست_هم_پوست_طلایی_پوست_روشن:','id':':orang_bergandengan_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':手拉手的两个人_中等肤色中等浅肤色:','ru':':люди_держащиеся_за_руки_средний_тон_кожи_светлый_тон_кожи:','tr':':el_ele_tutuşan_insanlar_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':أشخاص_يمسكون_بأيدي_بعضهم_بشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001FAC2':{'en':':people_hugging:','status':fully_qualified,'E':13,'de':':sich_umarmende_personen:','es':':personas_abrazándose:','fr':':étreinte:','ja':':ハグする人:','ko':':포옹하는_사람:','pt':':pessoas_se_abraçando:','it':':persone_che_si_abbracciano:','fa':':درآغوش_گرفتن:','id':':orang_berpelukan:','zh':':人的拥抱:','ru':':обнимающиеся_люди:','tr':':kucaklaşan_insanlar:','ar':':شخصان_يتعانقان:'},'\U0001F46F':{'en':':people_with_bunny_ears:','status':fully_qualified,'E':0.6,'alias':[':dancers:'],'de':':personen_mit_hasenohren:','es':':personas_con_orejas_de_conejo:','fr':':personnes_avec_des_oreilles_de_lapin:','ja':':バニー:','ko':':토끼_귀를_쓰고_춤추는_사람들:','pt':':pessoas_com_orelhas_de_coelho:','it':':persone_con_orecchie_da_coniglio:','fa':':رقص:','id':':orang_dengan_telinga_kelinci:','zh':':戴兔耳朵的人:','ru':':люди_с_ушами_кролика:','tr':':tavşan_kulaklı_kişiler:','ar':':شخصان_يرقصان:'},'\U0001F93C':{'en':':people_wrestling:','status':fully_qualified,'E':3,'alias':[':wrestling:'],'de':':ringer(in):','es':':personas_luchando:','fr':':personnes_faisant_de_la_lutte:','ja':':レスリングする人:','ko':':레슬링하는_사람:','pt':':pessoas_lutando:','it':':persone_che_fanno_la_lotta:','fa':':کُشتی_گیران:','id':':orang_bergulat:','zh':':摔跤选手:','ru':':борцы:','tr':':güreşen_kişiler:','ar':':مصارعون:'},'\U0001F3AD':{'en':':performing_arts:','status':fully_qualified,'E':0.6,'variant':True,'de':':masken:','es':':máscaras_de_teatro:','fr':':spectacle_vivant:','ja':':舞台芸術:','ko':':가면:','pt':':máscara:','it':':maschere:','fa':':هنرهای_نمایشی:','id':':seni_drama:','zh':':表演艺术:','ru':':маски:','tr':':sahne_sanatları:','ar':':قناع_وجه:'},'\U0001F623':{'en':':persevering_face:','status':fully_qualified,'E':0.6,'alias':[':persevere:'],'de':':entschlossenes_gesicht:','es':':cara_desesperada:','fr':':visage_persévérant:','ja':':我慢:','ko':':참아내는_얼굴:','pt':':rosto_perseverante:','it':':faccina_perseverante:','fa':':پافشاری:','id':':wajah_tidak_setuju:','zh':':痛苦:','ru':':упорство:','tr':':kararlı_yüz:','ar':':وجه_جاد:'},'\U0001F9D1':{'en':':person:','status':fully_qualified,'E':5,'alias':[':adult:'],'de':':person:','es':':persona_adulta:','fr':':adulte:','ja':':大人:','ko':':사람:','pt':':pessoa:','it':':persona:','fa':':شخص:','id':':dewasa:','zh':':成人:','ru':':взрослый:','tr':':yetişkin:','ar':':بالغ:'},'\U0001F9D1\U0000200D\U0001F9B2':{'en':':person_bald:','status':fully_qualified,'E':12.1,'de':':person_glatze:','es':':persona_adulta_sin_pelo:','fr':':adulte_chauve:','ja':':大人_はげ頭:','ko':':사람_대머리:','pt':':pessoa_careca:','it':':persona_calvo:','fa':':شخص_تاس:','id':':dewasa_gundul:','zh':':成人_秃顶:','ru':':взрослый_без_волос:','tr':':yetişkin_kel:','ar':':بالغ_أصلع:'},'\U0001F9D4':{'en':':person_beard:','status':fully_qualified,'E':5,'alias':[':bearded_person:'],'de':':person_bart:','es':':persona_con_barba:','fr':':personne_barbue:','ja':':あごひげの人:','ko':':수염_난_사람:','pt':':homem_barba:','it':':persona_con_la_barba:','fa':':مرد_ریش_دار:','id':':orang_janggut:','zh':':有胡子的人:','ru':':бородатый_человек:','tr':':sakallı_kişi:','ar':':شخص_بلحية:'},'\U0001F6B4':{'en':':person_biking:','status':fully_qualified,'E':1,'alias':[':bicyclist:'],'de':':radfahrer(in):','es':':persona_en_bicicleta:','fr':':cycliste:','ja':':自転車に乗る人:','ko':':자전거_타는_사람:','pt':':ciclista:','it':':ciclista:','fa':':دوچرخه_سوار:','id':':orang_bersepeda:','zh':':骑自行车:','ru':':велосипедист:','tr':':bisiklet_süren_kişi:','ar':':شخص_يركب_دراجة:'},'\U0001F6B4\U0001F3FF':{'en':':person_biking_dark_skin_tone:','status':fully_qualified,'E':1,'de':':radfahrer(in)_dunkle_hautfarbe:','es':':persona_en_bicicleta_tono_de_piel_oscuro:','fr':':cycliste_peau_foncée:','ja':':自転車に乗る人_濃い肌色:','ko':':자전거_타는_사람_검은색_피부:','pt':':ciclista_pele_escura:','it':':ciclista_carnagione_scura:','fa':':دوچرخه_سوار_پوست_آبنوسی:','id':':orang_bersepeda_warna_kulit_gelap:','zh':':骑自行车_较深肤色:','ru':':велосипедист_очень_темный_тон_кожи:','tr':':bisiklet_süren_kişi_koyu_cilt_tonu:','ar':':شخص_يركب_دراجة_بشرة_بلون_غامق:'},'\U0001F6B4\U0001F3FB':{'en':':person_biking_light_skin_tone:','status':fully_qualified,'E':1,'de':':radfahrer(in)_helle_hautfarbe:','es':':persona_en_bicicleta_tono_de_piel_claro:','fr':':cycliste_peau_claire:','ja':':自転車に乗る人_薄い肌色:','ko':':자전거_타는_사람_하얀_피부:','pt':':ciclista_pele_clara:','it':':ciclista_carnagione_chiara:','fa':':دوچرخه_سوار_پوست_سفید:','id':':orang_bersepeda_warna_kulit_cerah:','zh':':骑自行车_较浅肤色:','ru':':велосипедист_очень_светлый_тон_кожи:','tr':':bisiklet_süren_kişi_açık_cilt_tonu:','ar':':شخص_يركب_دراجة_بشرة_بلون_فاتح:'},'\U0001F6B4\U0001F3FE':{'en':':person_biking_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':radfahrer(in)_mitteldunkle_hautfarbe:','es':':persona_en_bicicleta_tono_de_piel_oscuro_medio:','fr':':cycliste_peau_mate:','ja':':自転車に乗る人_やや濃い肌色:','ko':':자전거_타는_사람_진한_갈색_피부:','pt':':ciclista_pele_morena_escura:','it':':ciclista_carnagione_abbastanza_scura:','fa':':دوچرخه_سوار_پوست_گندمی:','id':':orang_bersepeda_warna_kulit_gelap-sedang:','zh':':骑自行车_中等深肤色:','ru':':велосипедист_темный_тон_кожи:','tr':':bisiklet_süren_kişi_orta_koyu_cilt_tonu:','ar':':شخص_يركب_دراجة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6B4\U0001F3FC':{'en':':person_biking_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':radfahrer(in)_mittelhelle_hautfarbe:','es':':persona_en_bicicleta_tono_de_piel_claro_medio:','fr':':cycliste_peau_moyennement_claire:','ja':':自転車に乗る人_やや薄い肌色:','ko':':자전거_타는_사람_연한_갈색_피부:','pt':':ciclista_pele_morena_clara:','it':':ciclista_carnagione_abbastanza_chiara:','fa':':دوچرخه_سوار_پوست_روشن:','id':':orang_bersepeda_warna_kulit_cerah-sedang:','zh':':骑自行车_中等浅肤色:','ru':':велосипедист_светлый_тон_кожи:','tr':':bisiklet_süren_kişi_orta_açık_cilt_tonu:','ar':':شخص_يركب_دراجة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6B4\U0001F3FD':{'en':':person_biking_medium_skin_tone:','status':fully_qualified,'E':1,'de':':radfahrer(in)_mittlere_hautfarbe:','es':':persona_en_bicicleta_tono_de_piel_medio:','fr':':cycliste_peau_légèrement_mate:','ja':':自転車に乗る人_中間の肌色:','ko':':자전거_타는_사람_갈색_피부:','pt':':ciclista_pele_morena:','it':':ciclista_carnagione_olivastra:','fa':':دوچرخه_سوار_پوست_طلایی:','id':':orang_bersepeda_warna_kulit_sedang:','zh':':骑自行车_中等肤色:','ru':':велосипедист_средний_тон_кожи:','tr':':bisiklet_süren_kişi_orta_cilt_tonu:','ar':':شخص_يركب_دراجة_بشرة_بلون_معتدل:'},'\U0001F471':{'en':':person_blond_hair:','status':fully_qualified,'E':0.6,'alias':[':blond_haired_person:', ':person_with_blond_hair:'],'de':':person_blondes_haar:','es':':persona_adulta_rubia:','fr':':personne_blonde:','ja':':金髪の人:','ko':':금발인_사람:','pt':':pessoa_cabelo_louro:','it':':persona_bionda:','fa':':آدم_موبلوند:','id':':orang_rambut_pirang:','zh':':金色头发的人:','ru':':человек_со_светлыми_волосами:','tr':':sarışın_kişi:','ar':':شخص_أشقر:'},'\U000026F9\U0000FE0F':{'en':':person_bouncing_ball:','status':fully_qualified,'E':0.7,'alias':[':bouncing_ball_person:', ':person_with_ball:'],'variant':True,'de':':person_mit_ball:','es':':persona_botando_un_balón:','fr':':personne_avec_ballon:','ja':':バスケットボールをする人:','ko':':공을_갖고_있는_사람:','pt':':pessoa_jogando_basquete:','it':':persona_che_fa_rimbalzare_una_palla:','fa':':شخص_با_توپ:','id':':orang_memantulkan_bola:','zh':':玩球:','ru':':баскетболист:','tr':':top_sektiren_kişi:','ar':':شخص_وكرة:'},'\U000026F9':{'en':':person_bouncing_ball:','status':unqualified,'E':0.7,'alias':[':bouncing_ball_person:', ':person_with_ball:'],'variant':True,'de':':person_mit_ball:','es':':persona_botando_un_balón:','fr':':personne_avec_ballon:','ja':':バスケットボールをする人:','ko':':공을_갖고_있는_사람:','pt':':pessoa_jogando_basquete:','it':':persona_che_fa_rimbalzare_una_palla:','fa':':شخص_با_توپ:','id':':orang_memantulkan_bola:','zh':':玩球:','ru':':баскетболист:','tr':':top_sektiren_kişi:','ar':':شخص_وكرة:'},'\U000026F9\U0001F3FF':{'en':':person_bouncing_ball_dark_skin_tone:','status':fully_qualified,'E':2,'de':':person_mit_ball_dunkle_hautfarbe:','es':':persona_botando_un_balón_tono_de_piel_oscuro:','fr':':personne_avec_ballon_peau_foncée:','ja':':バスケットボールをする人_濃い肌色:','ko':':공을_갖고_있는_사람_검은색_피부:','pt':':pessoa_jogando_basquete_pele_escura:','it':':persona_che_fa_rimbalzare_una_palla_carnagione_scura:','fa':':شخص_با_توپ_پوست_آبنوسی:','id':':orang_memantulkan_bola_warna_kulit_gelap:','zh':':玩球_较深肤色:','ru':':баскетболист_очень_темный_тон_кожи:','tr':':top_sektiren_kişi_koyu_cilt_tonu:','ar':':شخص_وكرة_بشرة_بلون_غامق:'},'\U000026F9\U0001F3FB':{'en':':person_bouncing_ball_light_skin_tone:','status':fully_qualified,'E':2,'de':':person_mit_ball_helle_hautfarbe:','es':':persona_botando_un_balón_tono_de_piel_claro:','fr':':personne_avec_ballon_peau_claire:','ja':':バスケットボールをする人_薄い肌色:','ko':':공을_갖고_있는_사람_하얀_피부:','pt':':pessoa_jogando_basquete_pele_clara:','it':':persona_che_fa_rimbalzare_una_palla_carnagione_chiara:','fa':':شخص_با_توپ_پوست_سفید:','id':':orang_memantulkan_bola_warna_kulit_cerah:','zh':':玩球_较浅肤色:','ru':':баскетболист_очень_светлый_тон_кожи:','tr':':top_sektiren_kişi_açık_cilt_tonu:','ar':':شخص_وكرة_بشرة_بلون_فاتح:'},'\U000026F9\U0001F3FE':{'en':':person_bouncing_ball_medium-dark_skin_tone:','status':fully_qualified,'E':2,'de':':person_mit_ball_mitteldunkle_hautfarbe:','es':':persona_botando_un_balón_tono_de_piel_oscuro_medio:','fr':':personne_avec_ballon_peau_mate:','ja':':バスケットボールをする人_やや濃い肌色:','ko':':공을_갖고_있는_사람_진한_갈색_피부:','pt':':pessoa_jogando_basquete_pele_morena_escura:','it':':persona_che_fa_rimbalzare_una_palla_carnagione_abbastanza_scura:','fa':':شخص_با_توپ_پوست_گندمی:','id':':orang_memantulkan_bola_warna_kulit_gelap-sedang:','zh':':玩球_中等深肤色:','ru':':баскетболист_темный_тон_кожи:','tr':':top_sektiren_kişi_orta_koyu_cilt_tonu:','ar':':شخص_وكرة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U000026F9\U0001F3FC':{'en':':person_bouncing_ball_medium-light_skin_tone:','status':fully_qualified,'E':2,'de':':person_mit_ball_mittelhelle_hautfarbe:','es':':persona_botando_un_balón_tono_de_piel_claro_medio:','fr':':personne_avec_ballon_peau_moyennement_claire:','ja':':バスケットボールをする人_やや薄い肌色:','ko':':공을_갖고_있는_사람_연한_갈색_피부:','pt':':pessoa_jogando_basquete_pele_morena_clara:','it':':persona_che_fa_rimbalzare_una_palla_carnagione_abbastanza_chiara:','fa':':شخص_با_توپ_پوست_روشن:','id':':orang_memantulkan_bola_warna_kulit_cerah-sedang:','zh':':玩球_中等浅肤色:','ru':':баскетболист_светлый_тон_кожи:','tr':':top_sektiren_kişi_orta_açık_cilt_tonu:','ar':':شخص_وكرة_بشرة_بلون_فاتح_ومعتدل:'},'\U000026F9\U0001F3FD':{'en':':person_bouncing_ball_medium_skin_tone:','status':fully_qualified,'E':2,'de':':person_mit_ball_mittlere_hautfarbe:','es':':persona_botando_un_balón_tono_de_piel_medio:','fr':':personne_avec_ballon_peau_légèrement_mate:','ja':':バスケットボールをする人_中間の肌色:','ko':':공을_갖고_있는_사람_갈색_피부:','pt':':pessoa_jogando_basquete_pele_morena:','it':':persona_che_fa_rimbalzare_una_palla_carnagione_olivastra:','fa':':شخص_با_توپ_پوست_طلایی:','id':':orang_memantulkan_bola_warna_kulit_sedang:','zh':':玩球_中等肤色:','ru':':баскетболист_средний_тон_кожи:','tr':':top_sektiren_kişi_orta_cilt_tonu:','ar':':شخص_وكرة_بشرة_بلون_معتدل:'},'\U0001F647':{'en':':person_bowing:','status':fully_qualified,'E':0.6,'alias':[':bow:'],'de':':sich_verbeugende_person:','es':':persona_haciendo_una_reverencia:','fr':':personne_qui_s’incline:','ja':':おじぎする人:','ko':':절하는_사람:','pt':':pessoa_fazendo_reverência:','it':':persona_che_fa_un_inchino_profondo:','fa':':عذرخواهی:','id':':orang_membungkuk:','zh':':鞠躬:','ru':':человек_кланяется:','tr':':başını_öne_eğme:','ar':':شخص_ينحني_اعتذارا:'},'\U0001F647\U0001F3FF':{'en':':person_bowing_dark_skin_tone:','status':fully_qualified,'E':1,'de':':sich_verbeugende_person_dunkle_hautfarbe:','es':':persona_haciendo_una_reverencia_tono_de_piel_oscuro:','fr':':personne_qui_s’incline_peau_foncée:','ja':':おじぎする人_濃い肌色:','ko':':절하는_사람_검은색_피부:','pt':':pessoa_fazendo_reverência_pele_escura:','it':':persona_che_fa_un_inchino_profondo_carnagione_scura:','fa':':عذرخواهی_پوست_آبنوسی:','id':':orang_membungkuk_warna_kulit_gelap:','zh':':鞠躬_较深肤色:','ru':':человек_кланяется_очень_темный_тон_кожи:','tr':':başını_öne_eğme_koyu_cilt_tonu:','ar':':شخص_ينحني_اعتذارا_بشرة_بلون_غامق:'},'\U0001F647\U0001F3FB':{'en':':person_bowing_light_skin_tone:','status':fully_qualified,'E':1,'de':':sich_verbeugende_person_helle_hautfarbe:','es':':persona_haciendo_una_reverencia_tono_de_piel_claro:','fr':':personne_qui_s’incline_peau_claire:','ja':':おじぎする人_薄い肌色:','ko':':절하는_사람_하얀_피부:','pt':':pessoa_fazendo_reverência_pele_clara:','it':':persona_che_fa_un_inchino_profondo_carnagione_chiara:','fa':':عذرخواهی_پوست_سفید:','id':':orang_membungkuk_warna_kulit_cerah:','zh':':鞠躬_较浅肤色:','ru':':человек_кланяется_очень_светлый_тон_кожи:','tr':':başını_öne_eğme_açık_cilt_tonu:','ar':':شخص_ينحني_اعتذارا_بشرة_بلون_فاتح:'},'\U0001F647\U0001F3FE':{'en':':person_bowing_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':sich_verbeugende_person_mitteldunkle_hautfarbe:','es':':persona_haciendo_una_reverencia_tono_de_piel_oscuro_medio:','fr':':personne_qui_s’incline_peau_mate:','ja':':おじぎする人_やや濃い肌色:','ko':':절하는_사람_진한_갈색_피부:','pt':':pessoa_fazendo_reverência_pele_morena_escura:','it':':persona_che_fa_un_inchino_profondo_carnagione_abbastanza_scura:','fa':':عذرخواهی_پوست_گندمی:','id':':orang_membungkuk_warna_kulit_gelap-sedang:','zh':':鞠躬_中等深肤色:','ru':':человек_кланяется_темный_тон_кожи:','tr':':başını_öne_eğme_orta_koyu_cilt_tonu:','ar':':شخص_ينحني_اعتذارا_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F647\U0001F3FC':{'en':':person_bowing_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':sich_verbeugende_person_mittelhelle_hautfarbe:','es':':persona_haciendo_una_reverencia_tono_de_piel_claro_medio:','fr':':personne_qui_s’incline_peau_moyennement_claire:','ja':':おじぎする人_やや薄い肌色:','ko':':절하는_사람_연한_갈색_피부:','pt':':pessoa_fazendo_reverência_pele_morena_clara:','it':':persona_che_fa_un_inchino_profondo_carnagione_abbastanza_chiara:','fa':':عذرخواهی_پوست_روشن:','id':':orang_membungkuk_warna_kulit_cerah-sedang:','zh':':鞠躬_中等浅肤色:','ru':':человек_кланяется_светлый_тон_кожи:','tr':':başını_öne_eğme_orta_açık_cilt_tonu:','ar':':شخص_ينحني_اعتذارا_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F647\U0001F3FD':{'en':':person_bowing_medium_skin_tone:','status':fully_qualified,'E':1,'de':':sich_verbeugende_person_mittlere_hautfarbe:','es':':persona_haciendo_una_reverencia_tono_de_piel_medio:','fr':':personne_qui_s’incline_peau_légèrement_mate:','ja':':おじぎする人_中間の肌色:','ko':':절하는_사람_갈색_피부:','pt':':pessoa_fazendo_reverência_pele_morena:','it':':persona_che_fa_un_inchino_profondo_carnagione_olivastra:','fa':':عذرخواهی_پوست_طلایی:','id':':orang_membungkuk_warna_kulit_sedang:','zh':':鞠躬_中等肤色:','ru':':человек_кланяется_средний_тон_кожи:','tr':':başını_öne_eğme_orta_cilt_tonu:','ar':':شخص_ينحني_اعتذارا_بشرة_بلون_معتدل:'},'\U0001F938':{'en':':person_cartwheeling:','status':fully_qualified,'E':3,'alias':[':cartwheeling:'],'de':':rad_schlagende_person:','es':':persona_haciendo_voltereta_lateral:','fr':':personne_faisant_la_roue:','ja':':側転する人:','ko':':옆돌기하는_사람:','pt':':pessoa_fazendo_estrela:','it':':persona_che_fa_la_ruota:','fa':':حرکت_چرخ_و_فلک:','id':':orang_melakukan_gerakan_meroda:','zh':':侧手翻:','ru':':акробатический_трюк:','tr':':perende_atan_kişi:','ar':':شخص_يؤدي_حركة_العجلة:'},'\U0001F938\U0001F3FF':{'en':':person_cartwheeling_dark_skin_tone:','status':fully_qualified,'E':3,'de':':rad_schlagende_person_dunkle_hautfarbe:','es':':persona_haciendo_voltereta_lateral_tono_de_piel_oscuro:','fr':':personne_faisant_la_roue_peau_foncée:','ja':':側転する人_濃い肌色:','ko':':옆돌기하는_사람_검은색_피부:','pt':':pessoa_fazendo_estrela_pele_escura:','it':':persona_che_fa_la_ruota_carnagione_scura:','fa':':حرکت_چرخ_و_فلک_پوست_آبنوسی:','id':':orang_melakukan_gerakan_meroda_warna_kulit_gelap:','zh':':侧手翻_较深肤色:','ru':':акробатический_трюк_очень_темный_тон_кожи:','tr':':perende_atan_kişi_koyu_cilt_tonu:','ar':':شخص_يؤدي_حركة_العجلة_بشرة_بلون_غامق:'},'\U0001F938\U0001F3FB':{'en':':person_cartwheeling_light_skin_tone:','status':fully_qualified,'E':3,'de':':rad_schlagende_person_helle_hautfarbe:','es':':persona_haciendo_voltereta_lateral_tono_de_piel_claro:','fr':':personne_faisant_la_roue_peau_claire:','ja':':側転する人_薄い肌色:','ko':':옆돌기하는_사람_하얀_피부:','pt':':pessoa_fazendo_estrela_pele_clara:','it':':persona_che_fa_la_ruota_carnagione_chiara:','fa':':حرکت_چرخ_و_فلک_پوست_سفید:','id':':orang_melakukan_gerakan_meroda_warna_kulit_cerah:','zh':':侧手翻_较浅肤色:','ru':':акробатический_трюк_очень_светлый_тон_кожи:','tr':':perende_atan_kişi_açık_cilt_tonu:','ar':':شخص_يؤدي_حركة_العجلة_بشرة_بلون_فاتح:'},'\U0001F938\U0001F3FE':{'en':':person_cartwheeling_medium-dark_skin_tone:','status':fully_qualified,'E':3,'de':':rad_schlagende_person_mitteldunkle_hautfarbe:','es':':persona_haciendo_voltereta_lateral_tono_de_piel_oscuro_medio:','fr':':personne_faisant_la_roue_peau_mate:','ja':':側転する人_やや濃い肌色:','ko':':옆돌기하는_사람_진한_갈색_피부:','pt':':pessoa_fazendo_estrela_pele_morena_escura:','it':':persona_che_fa_la_ruota_carnagione_abbastanza_scura:','fa':':حرکت_چرخ_و_فلک_پوست_گندمی:','id':':orang_melakukan_gerakan_meroda_warna_kulit_gelap-sedang:','zh':':侧手翻_中等深肤色:','ru':':акробатический_трюк_темный_тон_кожи:','tr':':perende_atan_kişi_orta_koyu_cilt_tonu:','ar':':شخص_يؤدي_حركة_العجلة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F938\U0001F3FC':{'en':':person_cartwheeling_medium-light_skin_tone:','status':fully_qualified,'E':3,'de':':rad_schlagende_person_mittelhelle_hautfarbe:','es':':persona_haciendo_voltereta_lateral_tono_de_piel_claro_medio:','fr':':personne_faisant_la_roue_peau_moyennement_claire:','ja':':側転する人_やや薄い肌色:','ko':':옆돌기하는_사람_연한_갈색_피부:','pt':':pessoa_fazendo_estrela_pele_morena_clara:','it':':persona_che_fa_la_ruota_carnagione_abbastanza_chiara:','fa':':حرکت_چرخ_و_فلک_پوست_روشن:','id':':orang_melakukan_gerakan_meroda_warna_kulit_cerah-sedang:','zh':':侧手翻_中等浅肤色:','ru':':акробатический_трюк_светлый_тон_кожи:','tr':':perende_atan_kişi_orta_açık_cilt_tonu:','ar':':شخص_يؤدي_حركة_العجلة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F938\U0001F3FD':{'en':':person_cartwheeling_medium_skin_tone:','status':fully_qualified,'E':3,'de':':rad_schlagende_person_mittlere_hautfarbe:','es':':persona_haciendo_voltereta_lateral_tono_de_piel_medio:','fr':':personne_faisant_la_roue_peau_légèrement_mate:','ja':':側転する人_中間の肌色:','ko':':옆돌기하는_사람_갈색_피부:','pt':':pessoa_fazendo_estrela_pele_morena:','it':':persona_che_fa_la_ruota_carnagione_olivastra:','fa':':حرکت_چرخ_و_فلک_پوست_طلایی:','id':':orang_melakukan_gerakan_meroda_warna_kulit_sedang:','zh':':侧手翻_中等肤色:','ru':':акробатический_трюк_средний_тон_кожи:','tr':':perende_atan_kişi_orta_cilt_tonu:','ar':':شخص_يؤدي_حركة_العجلة_بشرة_بلون_معتدل:'},'\U0001F9D7':{'en':':person_climbing:','status':fully_qualified,'E':5,'alias':[':climbing:'],'de':':bergsteiger(in):','es':':persona_escalando:','fr':':personne_qui_grimpe:','ja':':山を登る人:','ko':':클라이밍하는_사람:','pt':':pessoa_escalando:','it':':persona_che_scala:','fa':':فردی_کوه_نوردی_می_کند:','id':':orang_memanjat:','zh':':攀爬的人:','ru':':человек_взбирается_по_скале:','tr':':dağcı_kişi:','ar':':شخص_يتسلق:'},'\U0001F9D7\U0001F3FF':{'en':':person_climbing_dark_skin_tone:','status':fully_qualified,'E':5,'de':':bergsteiger(in)_dunkle_hautfarbe:','es':':persona_escalando_tono_de_piel_oscuro:','fr':':personne_qui_grimpe_peau_foncée:','ja':':山を登る人_濃い肌色:','ko':':클라이밍하는_사람_검은색_피부:','pt':':pessoa_escalando_pele_escura:','it':':persona_che_scala_carnagione_scura:','fa':':فردی_کوه_نوردی_می_کند_پوست_آبنوسی:','id':':orang_memanjat_warna_kulit_gelap:','zh':':攀爬的人_较深肤色:','ru':':человек_взбирается_по_скале_очень_темный_тон_кожи:','tr':':dağcı_kişi_koyu_cilt_tonu:','ar':':شخص_يتسلق_بشرة_بلون_غامق:'},'\U0001F9D7\U0001F3FB':{'en':':person_climbing_light_skin_tone:','status':fully_qualified,'E':5,'de':':bergsteiger(in)_helle_hautfarbe:','es':':persona_escalando_tono_de_piel_claro:','fr':':personne_qui_grimpe_peau_claire:','ja':':山を登る人_薄い肌色:','ko':':클라이밍하는_사람_하얀_피부:','pt':':pessoa_escalando_pele_clara:','it':':persona_che_scala_carnagione_chiara:','fa':':فردی_کوه_نوردی_می_کند_پوست_سفید:','id':':orang_memanjat_warna_kulit_cerah:','zh':':攀爬的人_较浅肤色:','ru':':человек_взбирается_по_скале_очень_светлый_тон_кожи:','tr':':dağcı_kişi_açık_cilt_tonu:','ar':':شخص_يتسلق_بشرة_بلون_فاتح:'},'\U0001F9D7\U0001F3FE':{'en':':person_climbing_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':bergsteiger(in)_mitteldunkle_hautfarbe:','es':':persona_escalando_tono_de_piel_oscuro_medio:','fr':':personne_qui_grimpe_peau_mate:','ja':':山を登る人_やや濃い肌色:','ko':':클라이밍하는_사람_진한_갈색_피부:','pt':':pessoa_escalando_pele_morena_escura:','it':':persona_che_scala_carnagione_abbastanza_scura:','fa':':فردی_کوه_نوردی_می_کند_پوست_گندمی:','id':':orang_memanjat_warna_kulit_gelap-sedang:','zh':':攀爬的人_中等深肤色:','ru':':человек_взбирается_по_скале_темный_тон_кожи:','tr':':dağcı_kişi_orta_koyu_cilt_tonu:','ar':':شخص_يتسلق_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D7\U0001F3FC':{'en':':person_climbing_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':bergsteiger(in)_mittelhelle_hautfarbe:','es':':persona_escalando_tono_de_piel_claro_medio:','fr':':personne_qui_grimpe_peau_moyennement_claire:','ja':':山を登る人_やや薄い肌色:','ko':':클라이밍하는_사람_연한_갈색_피부:','pt':':pessoa_escalando_pele_morena_clara:','it':':persona_che_scala_carnagione_abbastanza_chiara:','fa':':فردی_کوه_نوردی_می_کند_پوست_روشن:','id':':orang_memanjat_warna_kulit_cerah-sedang:','zh':':攀爬的人_中等浅肤色:','ru':':человек_взбирается_по_скале_светлый_тон_кожи:','tr':':dağcı_kişi_orta_açık_cilt_tonu:','ar':':شخص_يتسلق_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D7\U0001F3FD':{'en':':person_climbing_medium_skin_tone:','status':fully_qualified,'E':5,'de':':bergsteiger(in)_mittlere_hautfarbe:','es':':persona_escalando_tono_de_piel_medio:','fr':':personne_qui_grimpe_peau_légèrement_mate:','ja':':山を登る人_中間の肌色:','ko':':클라이밍하는_사람_갈색_피부:','pt':':pessoa_escalando_pele_morena:','it':':persona_che_scala_carnagione_olivastra:','fa':':فردی_کوه_نوردی_می_کند_پوست_طلایی:','id':':orang_memanjat_warna_kulit_sedang:','zh':':攀爬的人_中等肤色:','ru':':человек_взбирается_по_скале_средний_тон_кожи:','tr':':dağcı_kişi_orta_cilt_tonu:','ar':':شخص_يتسلق_بشرة_بلون_معتدل:'},'\U0001F9D1\U0000200D\U0001F9B1':{'en':':person_curly_hair:','status':fully_qualified,'E':12.1,'de':':person_lockiges_haar:','es':':persona_adulta_pelo_rizado:','fr':':adulte_cheveux_bouclés:','ja':':大人_巻き毛:','ko':':사람_곱슬머리:','pt':':pessoa_cabelo_cacheado:','it':':persona_capelli_ricci:','fa':':شخص_موفرفری:','id':':dewasa_rambut_keriting:','zh':':成人_卷发:','ru':':взрослый_кудрявые_волосы:','tr':':yetişkin_kıvırcık_saç:','ar':':بالغ_شعر_مجعد:'},'\U0001F9D1\U0001F3FF':{'en':':person_dark_skin_tone:','status':fully_qualified,'E':5,'de':':person_dunkle_hautfarbe:','es':':persona_adulta_tono_de_piel_oscuro:','fr':':adulte_peau_foncée:','ja':':大人_濃い肌色:','ko':':사람_검은색_피부:','pt':':pessoa_pele_escura:','it':':persona_carnagione_scura:','fa':':شخص_پوست_آبنوسی:','id':':dewasa_warna_kulit_gelap:','zh':':成人_较深肤色:','ru':':взрослый_очень_темный_тон_кожи:','tr':':yetişkin_koyu_cilt_tonu:','ar':':بالغ_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F9B2':{'en':':person_dark_skin_tone_bald:','status':fully_qualified,'E':12.1,'de':':person_dunkle_hautfarbe_glatze:','es':':persona_adulta_tono_de_piel_oscuro_y_sin_pelo:','fr':':adulte_peau_foncée_et_chauve:','ja':':大人_濃い肌色_はげ頭:','ko':':사람_검은색_피부_대머리:','pt':':pessoa_pele_escura_e_careca:','it':':persona_carnagione_scura_e_calvo:','fa':':شخص_پوست_آبنوسی_تاس:','id':':dewasa_warna_kulit_gelap_gundul:','zh':':成人_较深肤色秃顶:','ru':':взрослый_очень_темный_тон_кожи_без_волос:','tr':':yetişkin_koyu_cilt_tonu_kel:','ar':':بالغ_بشرة_بلون_غامق_وأصلع:'},'\U0001F9D4\U0001F3FF':{'en':':person_dark_skin_tone_beard:','status':fully_qualified,'E':5,'de':':person_dunkle_hautfarbe_bart:','es':':persona_con_barba_tono_de_piel_oscuro:','fr':':personne_barbue_peau_foncée:','ja':':あごひげの人_濃い肌色:','ko':':수염_난_사람_검은색_피부:','pt':':homem_pele_escura_e_barba:','it':':persona_con_la_barba_carnagione_scura:','fa':':مرد_پوست_آبنوسی_ریش_دار:','id':':orang_warna_kulit_gelap_janggut:','zh':':有胡子的人_较深肤色:','ru':':бородатый_человек_очень_темный_тон_кожи:','tr':':sakallı_kişi_koyu_cilt_tonu:','ar':':شخص_بلحية_بشرة_بلون_غامق:'},'\U0001F471\U0001F3FF':{'en':':person_dark_skin_tone_blond_hair:','status':fully_qualified,'E':1,'de':':person_dunkle_hautfarbe_blondes_haar:','es':':persona_adulta_rubia_tono_de_piel_oscuro:','fr':':personne_blonde_peau_foncée:','ja':':金髪の人_濃い肌色:','ko':':금발인_사람_검은색_피부:','pt':':pessoa_pele_escura_e_cabelo_louro:','it':':persona_bionda_carnagione_scura:','fa':':آدم_پوست_آبنوسی_موبلوند:','id':':orang_warna_kulit_gelap_rambut_pirang:','zh':':金色头发的人_较深肤色:','ru':':человек_со_светлыми_волосами_очень_темный_тон_кожи:','tr':':sarışın_kişi_koyu_cilt_tonu:','ar':':شخص_أشقر_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F9B1':{'en':':person_dark_skin_tone_curly_hair:','status':fully_qualified,'E':12.1,'de':':person_dunkle_hautfarbe_lockiges_haar:','es':':persona_adulta_tono_de_piel_oscuro_y_pelo_rizado:','fr':':adulte_peau_foncée_et_cheveux_bouclés:','ja':':大人_濃い肌色_巻き毛:','ko':':사람_검은색_피부_곱슬머리:','pt':':pessoa_pele_escura_e_cabelo_cacheado:','it':':persona_carnagione_scura_e_capelli_ricci:','fa':':شخص_پوست_آبنوسی_موفرفری:','id':':dewasa_warna_kulit_gelap_rambut_keriting:','zh':':成人_较深肤色卷发:','ru':':взрослый_очень_темный_тон_кожи_кудрявые_волосы:','tr':':yetişkin_koyu_cilt_tonu_kıvırcık_saç:','ar':':بالغ_بشرة_بلون_غامق_وشعر_مجعد:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F9B0':{'en':':person_dark_skin_tone_red_hair:','status':fully_qualified,'E':12.1,'de':':person_dunkle_hautfarbe_rotes_haar:','es':':persona_adulta_tono_de_piel_oscuro_y_pelo_pelirrojo:','fr':':adulte_peau_foncée_et_cheveux_roux:','ja':':大人_濃い肌色_赤毛:','ko':':사람_검은색_피부_빨간_머리:','pt':':pessoa_pele_escura_e_cabelo_vermelho:','it':':persona_carnagione_scura_e_capelli_rossi:','fa':':شخص_پوست_آبنوسی_موقرمز:','id':':dewasa_warna_kulit_gelap_rambut_merah:','zh':':成人_较深肤色红发:','ru':':взрослый_очень_темный_тон_кожи_рыжие_волосы:','tr':':yetişkin_koyu_cilt_tonu_kızıl_saç:','ar':':بالغ_بشرة_بلون_غامق_وشعر_أحمر:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F9B3':{'en':':person_dark_skin_tone_white_hair:','status':fully_qualified,'E':12.1,'de':':person_dunkle_hautfarbe_weißes_haar:','es':':persona_adulta_tono_de_piel_oscuro_y_pelo_blanco:','fr':':adulte_peau_foncée_et_cheveux_blancs:','ja':':大人_濃い肌色_白髪:','ko':':사람_검은색_피부_백발:','pt':':pessoa_pele_escura_e_cabelo_branco:','it':':persona_carnagione_scura_e_capelli_bianchi:','fa':':شخص_پوست_آبنوسی_موسفید:','id':':dewasa_warna_kulit_gelap_rambut_putih:','zh':':成人_较深肤色白发:','ru':':взрослый_очень_темный_тон_кожи_седые_волосы:','tr':':yetişkin_koyu_cilt_tonu_beyaz_saç:','ar':':بالغ_بشرة_بلون_غامق_وشعر_أبيض:'},'\U0001F926':{'en':':person_facepalming:','status':fully_qualified,'E':3,'alias':[':facepalm:'],'de':':sich_an_den_kopf_fassende_person:','es':':persona_con_la_mano_en_la_frente:','fr':':personne_avec_la_paume_sur_le_visage:','ja':':ひたいに手をあてる人:','ko':':골치_아파하는_사람:','pt':':pessoa_decepcionada:','it':':persona_esasperata:','fa':':با_کف_دست_به_پیشانی_زدن:','id':':orang_tepuk_jidat:','zh':':捂脸:','ru':':человек_потерял_дар_речи:','tr':':yüzünü_eliyle_kapama:','ar':':شخص_محبط:'},'\U0001F926\U0001F3FF':{'en':':person_facepalming_dark_skin_tone:','status':fully_qualified,'E':3,'de':':sich_an_den_kopf_fassende_person_dunkle_hautfarbe:','es':':persona_con_la_mano_en_la_frente_tono_de_piel_oscuro:','fr':':personne_avec_la_paume_sur_le_visage_peau_foncée:','ja':':ひたいに手をあてる人_濃い肌色:','ko':':골치_아파하는_사람_검은색_피부:','pt':':pessoa_decepcionada_pele_escura:','it':':persona_esasperata_carnagione_scura:','fa':':با_کف_دست_به_پیشانی_زدن_پوست_آبنوسی:','id':':orang_tepuk_jidat_warna_kulit_gelap:','zh':':捂脸_较深肤色:','ru':':человек_потерял_дар_речи_очень_темный_тон_кожи:','tr':':yüzünü_eliyle_kapama_koyu_cilt_tonu:','ar':':شخص_محبط_بشرة_بلون_غامق:'},'\U0001F926\U0001F3FB':{'en':':person_facepalming_light_skin_tone:','status':fully_qualified,'E':3,'de':':sich_an_den_kopf_fassende_person_helle_hautfarbe:','es':':persona_con_la_mano_en_la_frente_tono_de_piel_claro:','fr':':personne_avec_la_paume_sur_le_visage_peau_claire:','ja':':ひたいに手をあてる人_薄い肌色:','ko':':골치_아파하는_사람_하얀_피부:','pt':':pessoa_decepcionada_pele_clara:','it':':persona_esasperata_carnagione_chiara:','fa':':با_کف_دست_به_پیشانی_زدن_پوست_سفید:','id':':orang_tepuk_jidat_warna_kulit_cerah:','zh':':捂脸_较浅肤色:','ru':':человек_потерял_дар_речи_очень_светлый_тон_кожи:','tr':':yüzünü_eliyle_kapama_açık_cilt_tonu:','ar':':شخص_محبط_بشرة_بلون_فاتح:'},'\U0001F926\U0001F3FE':{'en':':person_facepalming_medium-dark_skin_tone:','status':fully_qualified,'E':3,'de':':sich_an_den_kopf_fassende_person_mitteldunkle_hautfarbe:','es':':persona_con_la_mano_en_la_frente_tono_de_piel_oscuro_medio:','fr':':personne_avec_la_paume_sur_le_visage_peau_mate:','ja':':ひたいに手をあてる人_やや濃い肌色:','ko':':골치_아파하는_사람_진한_갈색_피부:','pt':':pessoa_decepcionada_pele_morena_escura:','it':':persona_esasperata_carnagione_abbastanza_scura:','fa':':با_کف_دست_به_پیشانی_زدن_پوست_گندمی:','id':':orang_tepuk_jidat_warna_kulit_gelap-sedang:','zh':':捂脸_中等深肤色:','ru':':человек_потерял_дар_речи_темный_тон_кожи:','tr':':yüzünü_eliyle_kapama_orta_koyu_cilt_tonu:','ar':':شخص_محبط_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F926\U0001F3FC':{'en':':person_facepalming_medium-light_skin_tone:','status':fully_qualified,'E':3,'de':':sich_an_den_kopf_fassende_person_mittelhelle_hautfarbe:','es':':persona_con_la_mano_en_la_frente_tono_de_piel_claro_medio:','fr':':personne_avec_la_paume_sur_le_visage_peau_moyennement_claire:','ja':':ひたいに手をあてる人_やや薄い肌色:','ko':':골치_아파하는_사람_연한_갈색_피부:','pt':':pessoa_decepcionada_pele_morena_clara:','it':':persona_esasperata_carnagione_abbastanza_chiara:','fa':':با_کف_دست_به_پیشانی_زدن_پوست_روشن:','id':':orang_tepuk_jidat_warna_kulit_cerah-sedang:','zh':':捂脸_中等浅肤色:','ru':':человек_потерял_дар_речи_светлый_тон_кожи:','tr':':yüzünü_eliyle_kapama_orta_açık_cilt_tonu:','ar':':شخص_محبط_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F926\U0001F3FD':{'en':':person_facepalming_medium_skin_tone:','status':fully_qualified,'E':3,'de':':sich_an_den_kopf_fassende_person_mittlere_hautfarbe:','es':':persona_con_la_mano_en_la_frente_tono_de_piel_medio:','fr':':personne_avec_la_paume_sur_le_visage_peau_légèrement_mate:','ja':':ひたいに手をあてる人_中間の肌色:','ko':':골치_아파하는_사람_갈색_피부:','pt':':pessoa_decepcionada_pele_morena:','it':':persona_esasperata_carnagione_olivastra:','fa':':با_کف_دست_به_پیشانی_زدن_پوست_طلایی:','id':':orang_tepuk_jidat_warna_kulit_sedang:','zh':':捂脸_中等肤色:','ru':':человек_потерял_дар_речи_средний_тон_кожи:','tr':':yüzünü_eliyle_kapama_orta_cilt_tonu:','ar':':شخص_محبط_بشرة_بلون_معتدل:'},'\U0001F9D1\U0000200D\U0001F37C':{'en':':person_feeding_baby:','status':fully_qualified,'E':13,'de':':person_die_baby_die_flasche_gibt:','es':':persona_alimentando_a_bebé:','fr':':personne_allaitant_un_bébé:','ja':':授乳する人:','ko':':수유_중인_사람:','pt':':pessoa_alimentando_bebê:','it':':persona_che_allatta:','fa':':شخصی_که_به_بچه_شیر_می_دهد:','id':':orang_menyusui_bayi:','zh':':哺乳的人:','ru':':кормящий_человек:','tr':':bebek_emziren_kişi:','ar':':شخص_يُرضع_طفلا:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F37C':{'en':':person_feeding_baby_dark_skin_tone:','status':fully_qualified,'E':13,'de':':person_die_baby_die_flasche_gibt_dunkle_hautfarbe:','es':':persona_alimentando_a_bebé_tono_de_piel_oscuro:','fr':':personne_allaitant_un_bébé_peau_foncée:','ja':':授乳する人_濃い肌色:','ko':':수유_중인_사람_검은색_피부:','pt':':pessoa_alimentando_bebê_pele_escura:','it':':persona_che_allatta_carnagione_scura:','fa':':شخصی_که_به_بچه_شیر_می_دهد_پوست_آبنوسی:','id':':orang_menyusui_bayi_warna_kulit_gelap:','zh':':哺乳的人_较深肤色:','ru':':кормящий_человек_очень_темный_тон_кожи:','tr':':bebek_emziren_kişi_koyu_cilt_tonu:','ar':':شخص_يُرضع_طفلا_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F37C':{'en':':person_feeding_baby_light_skin_tone:','status':fully_qualified,'E':13,'de':':person_die_baby_die_flasche_gibt_helle_hautfarbe:','es':':persona_alimentando_a_bebé_tono_de_piel_claro:','fr':':personne_allaitant_un_bébé_peau_claire:','ja':':授乳する人_薄い肌色:','ko':':수유_중인_사람_하얀_피부:','pt':':pessoa_alimentando_bebê_pele_clara:','it':':persona_che_allatta_carnagione_chiara:','fa':':شخصی_که_به_بچه_شیر_می_دهد_پوست_سفید:','id':':orang_menyusui_bayi_warna_kulit_cerah:','zh':':哺乳的人_较浅肤色:','ru':':кормящий_человек_очень_светлый_тон_кожи:','tr':':bebek_emziren_kişi_açık_cilt_tonu:','ar':':شخص_يُرضع_طفلا_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F37C':{'en':':person_feeding_baby_medium-dark_skin_tone:','status':fully_qualified,'E':13,'de':':person_die_baby_die_flasche_gibt_mitteldunkle_hautfarbe:','es':':persona_alimentando_a_bebé_tono_de_piel_oscuro_medio:','fr':':personne_allaitant_un_bébé_peau_mate:','ja':':授乳する人_やや濃い肌色:','ko':':수유_중인_사람_진한_갈색_피부:','pt':':pessoa_alimentando_bebê_pele_morena_escura:','it':':persona_che_allatta_carnagione_abbastanza_scura:','fa':':شخصی_که_به_بچه_شیر_می_دهد_پوست_گندمی:','id':':orang_menyusui_bayi_warna_kulit_gelap-sedang:','zh':':哺乳的人_中等深肤色:','ru':':кормящий_человек_темный_тон_кожи:','tr':':bebek_emziren_kişi_orta_koyu_cilt_tonu:','ar':':شخص_يُرضع_طفلا_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F37C':{'en':':person_feeding_baby_medium-light_skin_tone:','status':fully_qualified,'E':13,'de':':person_die_baby_die_flasche_gibt_mittelhelle_hautfarbe:','es':':persona_alimentando_a_bebé_tono_de_piel_claro_medio:','fr':':personne_allaitant_un_bébé_peau_moyennement_claire:','ja':':授乳する人_やや薄い肌色:','ko':':수유_중인_사람_연한_갈색_피부:','pt':':pessoa_alimentando_bebê_pele_morena_clara:','it':':persona_che_allatta_carnagione_abbastanza_chiara:','fa':':شخصی_که_به_بچه_شیر_می_دهد_پوست_روشن:','id':':orang_menyusui_bayi_warna_kulit_cerah-sedang:','zh':':哺乳的人_中等浅肤色:','ru':':кормящий_человек_светлый_тон_кожи:','tr':':bebek_emziren_kişi_orta_açık_cilt_tonu:','ar':':شخص_يُرضع_طفلا_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F37C':{'en':':person_feeding_baby_medium_skin_tone:','status':fully_qualified,'E':13,'de':':person_die_baby_die_flasche_gibt_mittlere_hautfarbe:','es':':persona_alimentando_a_bebé_tono_de_piel_medio:','fr':':personne_allaitant_un_bébé_peau_légèrement_mate:','ja':':授乳する人_中間の肌色:','ko':':수유_중인_사람_갈색_피부:','pt':':pessoa_alimentando_bebê_pele_morena:','it':':persona_che_allatta_carnagione_olivastra:','fa':':شخصی_که_به_بچه_شیر_می_دهد_پوست_طلایی:','id':':orang_menyusui_bayi_warna_kulit_sedang:','zh':':哺乳的人_中等肤色:','ru':':кормящий_человек_средний_тон_кожи:','tr':':bebek_emziren_kişi_orta_cilt_tonu:','ar':':شخص_يُرضع_طفلا_بشرة_بلون_معتدل:'},'\U0001F93A':{'en':':person_fencing:','status':fully_qualified,'E':3,'de':':fechter(in):','es':':persona_haciendo_esgrima:','fr':':escrimeur:','ja':':フェンシングをする人:','ko':':펜싱하는_사람:','pt':':esgrimista:','it':':schermidore:','fa':':شمشیربازی:','id':':orang_berolahraga_anggar:','zh':':击剑选手:','ru':':фехтовальщик:','tr':':eskrimci:','ar':':مبارز:'},'\U0001F64D':{'en':':person_frowning:','status':fully_qualified,'E':0.6,'alias':[':frowning_person:'],'de':':missmutige_person:','es':':persona_frunciendo_el_ceño:','fr':':personne_fronçant_les_sourcils:','ja':':しかめ面の人:','ko':':찌푸린_사람:','pt':':franzindo_a_sobrancelha:','it':':persona_corrucciata:','fa':':آدم_اخمو:','id':':orang_marah:','zh':':皱眉:','ru':':нахмурившийся_человек:','tr':':somurtma:','ar':':شخص_عابس:'},'\U0001F64D\U0001F3FF':{'en':':person_frowning_dark_skin_tone:','status':fully_qualified,'E':1,'de':':missmutige_person_dunkle_hautfarbe:','es':':persona_frunciendo_el_ceño_tono_de_piel_oscuro:','fr':':personne_fronçant_les_sourcils_peau_foncée:','ja':':しかめ面の人_濃い肌色:','ko':':찌푸린_사람_검은색_피부:','pt':':franzindo_a_sobrancelha_pele_escura:','it':':persona_corrucciata_carnagione_scura:','fa':':آدم_اخمو_پوست_آبنوسی:','id':':orang_marah_warna_kulit_gelap:','zh':':皱眉_较深肤色:','ru':':нахмурившийся_человек_очень_темный_тон_кожи:','tr':':somurtma_koyu_cilt_tonu:','ar':':شخص_عابس_بشرة_بلون_غامق:'},'\U0001F64D\U0001F3FB':{'en':':person_frowning_light_skin_tone:','status':fully_qualified,'E':1,'de':':missmutige_person_helle_hautfarbe:','es':':persona_frunciendo_el_ceño_tono_de_piel_claro:','fr':':personne_fronçant_les_sourcils_peau_claire:','ja':':しかめ面の人_薄い肌色:','ko':':찌푸린_사람_하얀_피부:','pt':':franzindo_a_sobrancelha_pele_clara:','it':':persona_corrucciata_carnagione_chiara:','fa':':آدم_اخمو_پوست_سفید:','id':':orang_marah_warna_kulit_cerah:','zh':':皱眉_较浅肤色:','ru':':нахмурившийся_человек_очень_светлый_тон_кожи:','tr':':somurtma_açık_cilt_tonu:','ar':':شخص_عابس_بشرة_بلون_فاتح:'},'\U0001F64D\U0001F3FE':{'en':':person_frowning_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':missmutige_person_mitteldunkle_hautfarbe:','es':':persona_frunciendo_el_ceño_tono_de_piel_oscuro_medio:','fr':':personne_fronçant_les_sourcils_peau_mate:','ja':':しかめ面の人_やや濃い肌色:','ko':':찌푸린_사람_진한_갈색_피부:','pt':':franzindo_a_sobrancelha_pele_morena_escura:','it':':persona_corrucciata_carnagione_abbastanza_scura:','fa':':آدم_اخمو_پوست_گندمی:','id':':orang_marah_warna_kulit_gelap-sedang:','zh':':皱眉_中等深肤色:','ru':':нахмурившийся_человек_темный_тон_кожи:','tr':':somurtma_orta_koyu_cilt_tonu:','ar':':شخص_عابس_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F64D\U0001F3FC':{'en':':person_frowning_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':missmutige_person_mittelhelle_hautfarbe:','es':':persona_frunciendo_el_ceño_tono_de_piel_claro_medio:','fr':':personne_fronçant_les_sourcils_peau_moyennement_claire:','ja':':しかめ面の人_やや薄い肌色:','ko':':찌푸린_사람_연한_갈색_피부:','pt':':franzindo_a_sobrancelha_pele_morena_clara:','it':':persona_corrucciata_carnagione_abbastanza_chiara:','fa':':آدم_اخمو_پوست_روشن:','id':':orang_marah_warna_kulit_cerah-sedang:','zh':':皱眉_中等浅肤色:','ru':':нахмурившийся_человек_светлый_тон_кожи:','tr':':somurtma_orta_açık_cilt_tonu:','ar':':شخص_عابس_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F64D\U0001F3FD':{'en':':person_frowning_medium_skin_tone:','status':fully_qualified,'E':1,'de':':missmutige_person_mittlere_hautfarbe:','es':':persona_frunciendo_el_ceño_tono_de_piel_medio:','fr':':personne_fronçant_les_sourcils_peau_légèrement_mate:','ja':':しかめ面の人_中間の肌色:','ko':':찌푸린_사람_갈색_피부:','pt':':franzindo_a_sobrancelha_pele_morena:','it':':persona_corrucciata_carnagione_olivastra:','fa':':آدم_اخمو_پوست_طلایی:','id':':orang_marah_warna_kulit_sedang:','zh':':皱眉_中等肤色:','ru':':нахмурившийся_человек_средний_тон_кожи:','tr':':somurtma_orta_cilt_tonu:','ar':':شخص_عابس_بشرة_بلون_معتدل:'},'\U0001F645':{'en':':person_gesturing_NO:','status':fully_qualified,'E':0.6,'alias':[':no_good:', ':person_gesturing_no:'],'de':':person_mit_überkreuzten_armen:','es':':persona_haciendo_el_gesto_de_no:','fr':':personne_faisant_un_geste_d’interdiction:','ja':':ダメのポーズをする人:','ko':':안_된다는_제스처를_하는_사람:','pt':':pessoa_fazendo_gesto_de_“não”:','it':':persona_che_fa_segno_di_no:','fa':':ابداً_نه:','id':':orang_mengisyaratkan_tidak:','zh':':禁止手势:','ru':':человек_против:','tr':':hayır_hareketi:','ar':':إشارة_رفض:'},'\U0001F645\U0001F3FF':{'en':':person_gesturing_NO_dark_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_überkreuzten_armen_dunkle_hautfarbe:','es':':persona_haciendo_el_gesto_de_no_tono_de_piel_oscuro:','fr':':personne_faisant_un_geste_d’interdiction_peau_foncée:','ja':':ダメのポーズをする人_濃い肌色:','ko':':안_된다는_제스처를_하는_사람_검은색_피부:','pt':':pessoa_fazendo_gesto_de_“não”_pele_escura:','it':':persona_che_fa_segno_di_no_carnagione_scura:','fa':':ابداً_نه_پوست_آبنوسی:','id':':orang_mengisyaratkan_tidak_warna_kulit_gelap:','zh':':禁止手势_较深肤色:','ru':':человек_против_очень_темный_тон_кожи:','tr':':hayır_hareketi_koyu_cilt_tonu:','ar':':إشارة_رفض_بشرة_بلون_غامق:'},'\U0001F645\U0001F3FB':{'en':':person_gesturing_NO_light_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_überkreuzten_armen_helle_hautfarbe:','es':':persona_haciendo_el_gesto_de_no_tono_de_piel_claro:','fr':':personne_faisant_un_geste_d’interdiction_peau_claire:','ja':':ダメのポーズをする人_薄い肌色:','ko':':안_된다는_제스처를_하는_사람_하얀_피부:','pt':':pessoa_fazendo_gesto_de_“não”_pele_clara:','it':':persona_che_fa_segno_di_no_carnagione_chiara:','fa':':ابداً_نه_پوست_سفید:','id':':orang_mengisyaratkan_tidak_warna_kulit_cerah:','zh':':禁止手势_较浅肤色:','ru':':человек_против_очень_светлый_тон_кожи:','tr':':hayır_hareketi_açık_cilt_tonu:','ar':':إشارة_رفض_بشرة_بلون_فاتح:'},'\U0001F645\U0001F3FE':{'en':':person_gesturing_NO_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_überkreuzten_armen_mitteldunkle_hautfarbe:','es':':persona_haciendo_el_gesto_de_no_tono_de_piel_oscuro_medio:','fr':':personne_faisant_un_geste_d’interdiction_peau_mate:','ja':':ダメのポーズをする人_やや濃い肌色:','ko':':안_된다는_제스처를_하는_사람_진한_갈색_피부:','pt':':pessoa_fazendo_gesto_de_“não”_pele_morena_escura:','it':':persona_che_fa_segno_di_no_carnagione_abbastanza_scura:','fa':':ابداً_نه_پوست_گندمی:','id':':orang_mengisyaratkan_tidak_warna_kulit_gelap-sedang:','zh':':禁止手势_中等深肤色:','ru':':человек_против_темный_тон_кожи:','tr':':hayır_hareketi_orta_koyu_cilt_tonu:','ar':':إشارة_رفض_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F645\U0001F3FC':{'en':':person_gesturing_NO_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_überkreuzten_armen_mittelhelle_hautfarbe:','es':':persona_haciendo_el_gesto_de_no_tono_de_piel_claro_medio:','fr':':personne_faisant_un_geste_d’interdiction_peau_moyennement_claire:','ja':':ダメのポーズをする人_やや薄い肌色:','ko':':안_된다는_제스처를_하는_사람_연한_갈색_피부:','pt':':pessoa_fazendo_gesto_de_“não”_pele_morena_clara:','it':':persona_che_fa_segno_di_no_carnagione_abbastanza_chiara:','fa':':ابداً_نه_پوست_روشن:','id':':orang_mengisyaratkan_tidak_warna_kulit_cerah-sedang:','zh':':禁止手势_中等浅肤色:','ru':':человек_против_светлый_тон_кожи:','tr':':hayır_hareketi_orta_açık_cilt_tonu:','ar':':إشارة_رفض_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F645\U0001F3FD':{'en':':person_gesturing_NO_medium_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_überkreuzten_armen_mittlere_hautfarbe:','es':':persona_haciendo_el_gesto_de_no_tono_de_piel_medio:','fr':':personne_faisant_un_geste_d’interdiction_peau_légèrement_mate:','ja':':ダメのポーズをする人_中間の肌色:','ko':':안_된다는_제스처를_하는_사람_갈색_피부:','pt':':pessoa_fazendo_gesto_de_“não”_pele_morena:','it':':persona_che_fa_segno_di_no_carnagione_olivastra:','fa':':ابداً_نه_پوست_طلایی:','id':':orang_mengisyaratkan_tidak_warna_kulit_sedang:','zh':':禁止手势_中等肤色:','ru':':человек_против_средний_тон_кожи:','tr':':hayır_hareketi_orta_cilt_tonu:','ar':':إشارة_رفض_بشرة_بلون_معتدل:'},'\U0001F646':{'en':':person_gesturing_OK:','status':fully_qualified,'E':0.6,'alias':[':ok_person:', ':person_gesturing_ok:'],'de':':person_mit_händen_auf_dem_kopf:','es':':persona_haciendo_el_gesto_de_de_acuerdo:','fr':':personne_faisant_un_geste_d’acceptation:','ja':':okのポーズをする人:','ko':':오케이라는_제스처를_하는_사람:','pt':':pessoa_fazendo_gesto_de_“ok”:','it':':persona_con_gesto_ok:','fa':':باشه:','id':':orang_mengisyaratkan_oke:','zh':':OK手势:','ru':':человек_за:','tr':':tamam_hareketi:','ar':':إشارة_موافقة:'},'\U0001F646\U0001F3FF':{'en':':person_gesturing_OK_dark_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_händen_auf_dem_kopf_dunkle_hautfarbe:','es':':persona_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_oscuro:','fr':':personne_faisant_un_geste_d’acceptation_peau_foncée:','ja':':okのポーズをする人_濃い肌色:','ko':':오케이라는_제스처를_하는_사람_검은색_피부:','pt':':pessoa_fazendo_gesto_de_“ok”_pele_escura:','it':':persona_con_gesto_ok_carnagione_scura:','fa':':باشه_پوست_آبنوسی:','id':':orang_mengisyaratkan_oke_warna_kulit_gelap:','zh':':OK手势_较深肤色:','ru':':человек_за_очень_темный_тон_кожи:','tr':':tamam_hareketi_koyu_cilt_tonu:','ar':':إشارة_موافقة_بشرة_بلون_غامق:'},'\U0001F646\U0001F3FB':{'en':':person_gesturing_OK_light_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_händen_auf_dem_kopf_helle_hautfarbe:','es':':persona_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_claro:','fr':':personne_faisant_un_geste_d’acceptation_peau_claire:','ja':':okのポーズをする人_薄い肌色:','ko':':오케이라는_제스처를_하는_사람_하얀_피부:','pt':':pessoa_fazendo_gesto_de_“ok”_pele_clara:','it':':persona_con_gesto_ok_carnagione_chiara:','fa':':باشه_پوست_سفید:','id':':orang_mengisyaratkan_oke_warna_kulit_cerah:','zh':':OK手势_较浅肤色:','ru':':человек_за_очень_светлый_тон_кожи:','tr':':tamam_hareketi_açık_cilt_tonu:','ar':':إشارة_موافقة_بشرة_بلون_فاتح:'},'\U0001F646\U0001F3FE':{'en':':person_gesturing_OK_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_händen_auf_dem_kopf_mitteldunkle_hautfarbe:','es':':persona_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_oscuro_medio:','fr':':personne_faisant_un_geste_d’acceptation_peau_mate:','ja':':okのポーズをする人_やや濃い肌色:','ko':':오케이라는_제스처를_하는_사람_진한_갈색_피부:','pt':':pessoa_fazendo_gesto_de_“ok”_pele_morena_escura:','it':':persona_con_gesto_ok_carnagione_abbastanza_scura:','fa':':باشه_پوست_گندمی:','id':':orang_mengisyaratkan_oke_warna_kulit_gelap-sedang:','zh':':OK手势_中等深肤色:','ru':':человек_за_темный_тон_кожи:','tr':':tamam_hareketi_orta_koyu_cilt_tonu:','ar':':إشارة_موافقة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F646\U0001F3FC':{'en':':person_gesturing_OK_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_händen_auf_dem_kopf_mittelhelle_hautfarbe:','es':':persona_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_claro_medio:','fr':':personne_faisant_un_geste_d’acceptation_peau_moyennement_claire:','ja':':okのポーズをする人_やや薄い肌色:','ko':':오케이라는_제스처를_하는_사람_연한_갈색_피부:','pt':':pessoa_fazendo_gesto_de_“ok”_pele_morena_clara:','it':':persona_con_gesto_ok_carnagione_abbastanza_chiara:','fa':':باشه_پوست_روشن:','id':':orang_mengisyaratkan_oke_warna_kulit_cerah-sedang:','zh':':OK手势_中等浅肤色:','ru':':человек_за_светлый_тон_кожи:','tr':':tamam_hareketi_orta_açık_cilt_tonu:','ar':':إشارة_موافقة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F646\U0001F3FD':{'en':':person_gesturing_OK_medium_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_händen_auf_dem_kopf_mittlere_hautfarbe:','es':':persona_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_medio:','fr':':personne_faisant_un_geste_d’acceptation_peau_légèrement_mate:','ja':':okのポーズをする人_中間の肌色:','ko':':오케이라는_제스처를_하는_사람_갈색_피부:','pt':':pessoa_fazendo_gesto_de_“ok”_pele_morena:','it':':persona_con_gesto_ok_carnagione_olivastra:','fa':':باشه_پوست_طلایی:','id':':orang_mengisyaratkan_oke_warna_kulit_sedang:','zh':':OK手势_中等肤色:','ru':':человек_за_средний_тон_кожи:','tr':':tamam_hareketi_orta_cilt_tonu:','ar':':إشارة_موافقة_بشرة_بلون_معتدل:'},'\U0001F487':{'en':':person_getting_haircut:','status':fully_qualified,'E':0.6,'alias':[':haircut:'],'de':':person_beim_haareschneiden:','es':':persona_cortándose_el_pelo:','fr':':personne_qui_se_fait_couper_les_cheveux:','ja':':散髪される人:','ko':':헤어컷:','pt':':pessoa_cortando_o_cabelo:','it':':taglio_di_capelli:','fa':':آرایشگاه:','id':':orang_potong_rambut:','zh':':理发:','ru':':человек_у_парикмахера:','tr':':saç_kesimi:','ar':':شخص_يُقص_شعره:'},'\U0001F487\U0001F3FF':{'en':':person_getting_haircut_dark_skin_tone:','status':fully_qualified,'E':1,'de':':person_beim_haareschneiden_dunkle_hautfarbe:','es':':persona_cortándose_el_pelo_tono_de_piel_oscuro:','fr':':personne_qui_se_fait_couper_les_cheveux_peau_foncée:','ja':':散髪される人_濃い肌色:','ko':':헤어컷_검은색_피부:','pt':':pessoa_cortando_o_cabelo_pele_escura:','it':':taglio_di_capelli_carnagione_scura:','fa':':آرایشگاه_پوست_آبنوسی:','id':':orang_potong_rambut_warna_kulit_gelap:','zh':':理发_较深肤色:','ru':':человек_у_парикмахера_очень_темный_тон_кожи:','tr':':saç_kesimi_koyu_cilt_tonu:','ar':':شخص_يُقص_شعره_بشرة_بلون_غامق:'},'\U0001F487\U0001F3FB':{'en':':person_getting_haircut_light_skin_tone:','status':fully_qualified,'E':1,'de':':person_beim_haareschneiden_helle_hautfarbe:','es':':persona_cortándose_el_pelo_tono_de_piel_claro:','fr':':personne_qui_se_fait_couper_les_cheveux_peau_claire:','ja':':散髪される人_薄い肌色:','ko':':헤어컷_하얀_피부:','pt':':pessoa_cortando_o_cabelo_pele_clara:','it':':taglio_di_capelli_carnagione_chiara:','fa':':آرایشگاه_پوست_سفید:','id':':orang_potong_rambut_warna_kulit_cerah:','zh':':理发_较浅肤色:','ru':':человек_у_парикмахера_очень_светлый_тон_кожи:','tr':':saç_kesimi_açık_cilt_tonu:','ar':':شخص_يُقص_شعره_بشرة_بلون_فاتح:'},'\U0001F487\U0001F3FE':{'en':':person_getting_haircut_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':person_beim_haareschneiden_mitteldunkle_hautfarbe:','es':':persona_cortándose_el_pelo_tono_de_piel_oscuro_medio:','fr':':personne_qui_se_fait_couper_les_cheveux_peau_mate:','ja':':散髪される人_やや濃い肌色:','ko':':헤어컷_진한_갈색_피부:','pt':':pessoa_cortando_o_cabelo_pele_morena_escura:','it':':taglio_di_capelli_carnagione_abbastanza_scura:','fa':':آرایشگاه_پوست_گندمی:','id':':orang_potong_rambut_warna_kulit_gelap-sedang:','zh':':理发_中等深肤色:','ru':':человек_у_парикмахера_темный_тон_кожи:','tr':':saç_kesimi_orta_koyu_cilt_tonu:','ar':':شخص_يُقص_شعره_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F487\U0001F3FC':{'en':':person_getting_haircut_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':person_beim_haareschneiden_mittelhelle_hautfarbe:','es':':persona_cortándose_el_pelo_tono_de_piel_claro_medio:','fr':':personne_qui_se_fait_couper_les_cheveux_peau_moyennement_claire:','ja':':散髪される人_やや薄い肌色:','ko':':헤어컷_연한_갈색_피부:','pt':':pessoa_cortando_o_cabelo_pele_morena_clara:','it':':taglio_di_capelli_carnagione_abbastanza_chiara:','fa':':آرایشگاه_پوست_روشن:','id':':orang_potong_rambut_warna_kulit_cerah-sedang:','zh':':理发_中等浅肤色:','ru':':человек_у_парикмахера_светлый_тон_кожи:','tr':':saç_kesimi_orta_açık_cilt_tonu:','ar':':شخص_يُقص_شعره_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F487\U0001F3FD':{'en':':person_getting_haircut_medium_skin_tone:','status':fully_qualified,'E':1,'de':':person_beim_haareschneiden_mittlere_hautfarbe:','es':':persona_cortándose_el_pelo_tono_de_piel_medio:','fr':':personne_qui_se_fait_couper_les_cheveux_peau_légèrement_mate:','ja':':散髪される人_中間の肌色:','ko':':헤어컷_갈색_피부:','pt':':pessoa_cortando_o_cabelo_pele_morena:','it':':taglio_di_capelli_carnagione_olivastra:','fa':':آرایشگاه_پوست_طلایی:','id':':orang_potong_rambut_warna_kulit_sedang:','zh':':理发_中等肤色:','ru':':человек_у_парикмахера_средний_тон_кожи:','tr':':saç_kesimi_orta_cilt_tonu:','ar':':شخص_يُقص_شعره_بشرة_بلون_معتدل:'},'\U0001F486':{'en':':person_getting_massage:','status':fully_qualified,'E':0.6,'alias':[':massage:'],'de':':person_die_eine_kopfmassage_bekommt:','es':':persona_recibiendo_masaje:','fr':':personne_qui_se_fait_masser:','ja':':フェイスマッサージ中の人:','ko':':마사지_받는_사람:','pt':':pessoa_recebendo_massagem_facial:','it':':persona_che_riceve_un_massaggio:','fa':':ماساژ_صورت:','id':':pria_dipijat:','zh':':按摩:','ru':':человеку_массируют_лицо:','tr':':yüz_masajı:','ar':':شخص_يحصل_على_مساج:'},'\U0001F486\U0001F3FF':{'en':':person_getting_massage_dark_skin_tone:','status':fully_qualified,'E':1,'de':':person_die_eine_kopfmassage_bekommt_dunkle_hautfarbe:','es':':persona_recibiendo_masaje_tono_de_piel_oscuro:','fr':':personne_qui_se_fait_masser_peau_foncée:','ja':':フェイスマッサージ中の人_濃い肌色:','ko':':마사지_받는_사람_검은색_피부:','pt':':pessoa_recebendo_massagem_facial_pele_escura:','it':':persona_che_riceve_un_massaggio_carnagione_scura:','fa':':ماساژ_صورت_پوست_آبنوسی:','id':':pria_dipijat_warna_kulit_gelap:','zh':':按摩_较深肤色:','ru':':человеку_массируют_лицо_очень_темный_тон_кожи:','tr':':yüz_masajı_koyu_cilt_tonu:','ar':':شخص_يحصل_على_مساج_بشرة_بلون_غامق:'},'\U0001F486\U0001F3FB':{'en':':person_getting_massage_light_skin_tone:','status':fully_qualified,'E':1,'de':':person_die_eine_kopfmassage_bekommt_helle_hautfarbe:','es':':persona_recibiendo_masaje_tono_de_piel_claro:','fr':':personne_qui_se_fait_masser_peau_claire:','ja':':フェイスマッサージ中の人_薄い肌色:','ko':':마사지_받는_사람_하얀_피부:','pt':':pessoa_recebendo_massagem_facial_pele_clara:','it':':persona_che_riceve_un_massaggio_carnagione_chiara:','fa':':ماساژ_صورت_پوست_سفید:','id':':pria_dipijat_warna_kulit_cerah:','zh':':按摩_较浅肤色:','ru':':человеку_массируют_лицо_очень_светлый_тон_кожи:','tr':':yüz_masajı_açık_cilt_tonu:','ar':':شخص_يحصل_على_مساج_بشرة_بلون_فاتح:'},'\U0001F486\U0001F3FE':{'en':':person_getting_massage_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':person_die_eine_kopfmassage_bekommt_mitteldunkle_hautfarbe:','es':':persona_recibiendo_masaje_tono_de_piel_oscuro_medio:','fr':':personne_qui_se_fait_masser_peau_mate:','ja':':フェイスマッサージ中の人_やや濃い肌色:','ko':':마사지_받는_사람_진한_갈색_피부:','pt':':pessoa_recebendo_massagem_facial_pele_morena_escura:','it':':persona_che_riceve_un_massaggio_carnagione_abbastanza_scura:','fa':':ماساژ_صورت_پوست_گندمی:','id':':pria_dipijat_warna_kulit_gelap-sedang:','zh':':按摩_中等深肤色:','ru':':человеку_массируют_лицо_темный_тон_кожи:','tr':':yüz_masajı_orta_koyu_cilt_tonu:','ar':':شخص_يحصل_على_مساج_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F486\U0001F3FC':{'en':':person_getting_massage_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':person_die_eine_kopfmassage_bekommt_mittelhelle_hautfarbe:','es':':persona_recibiendo_masaje_tono_de_piel_claro_medio:','fr':':personne_qui_se_fait_masser_peau_moyennement_claire:','ja':':フェイスマッサージ中の人_やや薄い肌色:','ko':':마사지_받는_사람_연한_갈색_피부:','pt':':pessoa_recebendo_massagem_facial_pele_morena_clara:','it':':persona_che_riceve_un_massaggio_carnagione_abbastanza_chiara:','fa':':ماساژ_صورت_پوست_روشن:','id':':pria_dipijat_warna_kulit_cerah-sedang:','zh':':按摩_中等浅肤色:','ru':':человеку_массируют_лицо_светлый_тон_кожи:','tr':':yüz_masajı_orta_açık_cilt_tonu:','ar':':شخص_يحصل_على_مساج_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F486\U0001F3FD':{'en':':person_getting_massage_medium_skin_tone:','status':fully_qualified,'E':1,'de':':person_die_eine_kopfmassage_bekommt_mittlere_hautfarbe:','es':':persona_recibiendo_masaje_tono_de_piel_medio:','fr':':personne_qui_se_fait_masser_peau_légèrement_mate:','ja':':フェイスマッサージ中の人_中間の肌色:','ko':':마사지_받는_사람_갈색_피부:','pt':':pessoa_recebendo_massagem_facial_pele_morena:','it':':persona_che_riceve_un_massaggio_carnagione_olivastra:','fa':':ماساژ_صورت_پوست_طلایی:','id':':pria_dipijat_warna_kulit_sedang:','zh':':按摩_中等肤色:','ru':':человеку_массируют_лицо_средний_тон_кожи:','tr':':yüz_masajı_orta_cilt_tonu:','ar':':شخص_يحصل_على_مساج_بشرة_بلون_معتدل:'},'\U0001F3CC\U0000FE0F':{'en':':person_golfing:','status':fully_qualified,'E':0.7,'alias':[':golfing:', ':golfer:'],'variant':True,'de':':golfer(in):','es':':golfista:','fr':':joueur_de_golf:','ja':':ゴルフをする人:','ko':':골프치는_사람:','pt':':golfista:','it':':persona_che_gioca_a_golf:','fa':':گلف_باز:','id':':orang_bermain_golf:','zh':':打高尔夫的人:','ru':':гольфист:','tr':':golf_oynayan_kişi:','ar':':شخص_يلعب_غولف:'},'\U0001F3CC':{'en':':person_golfing:','status':unqualified,'E':0.7,'alias':[':golfing:', ':golfer:'],'variant':True,'de':':golfer(in):','es':':persona_jugando_al_golf:','fr':':joueur_de_golf:','ja':':ゴルフをする人:','ko':':골프치는_사람:','pt':':golfista:','it':':persona_che_gioca_a_golf:','fa':':گلف_باز:','id':':orang_bermain_golf:','zh':':打高尔夫的人:','ru':':гольфист:','tr':':golf_oynayan_kişi:','ar':':شخص_يلعب_غولف:'},'\U0001F3CC\U0001F3FF':{'en':':person_golfing_dark_skin_tone:','status':fully_qualified,'E':4,'de':':golfer(in)_dunkle_hautfarbe:','es':':persona_jugando_al_golf_tono_de_piel_oscuro:','fr':':joueur_de_golf_peau_foncée:','ja':':ゴルフをする人_濃い肌色:','ko':':골프치는_사람_검은색_피부:','pt':':golfista_pele_escura:','it':':persona_che_gioca_a_golf_carnagione_scura:','fa':':گلف_باز_پوست_آبنوسی:','id':':orang_bermain_golf_warna_kulit_gelap:','zh':':打高尔夫的人_较深肤色:','ru':':гольфист_очень_темный_тон_кожи:','tr':':golf_oynayan_kişi_koyu_cilt_tonu:','ar':':شخص_يلعب_غولف_بشرة_بلون_غامق:'},'\U0001F3CC\U0001F3FB':{'en':':person_golfing_light_skin_tone:','status':fully_qualified,'E':4,'de':':golfer(in)_helle_hautfarbe:','es':':persona_jugando_al_golf_tono_de_piel_claro:','fr':':joueur_de_golf_peau_claire:','ja':':ゴルフをする人_薄い肌色:','ko':':골프치는_사람_하얀_피부:','pt':':golfista_pele_clara:','it':':persona_che_gioca_a_golf_carnagione_chiara:','fa':':گلف_باز_پوست_سفید:','id':':orang_bermain_golf_warna_kulit_cerah:','zh':':打高尔夫的人_较浅肤色:','ru':':гольфист_очень_светлый_тон_кожи:','tr':':golf_oynayan_kişi_açık_cilt_tonu:','ar':':شخص_يلعب_غولف_بشرة_بلون_فاتح:'},'\U0001F3CC\U0001F3FE':{'en':':person_golfing_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':golfer(in)_mitteldunkle_hautfarbe:','es':':persona_jugando_al_golf_tono_de_piel_oscuro_medio:','fr':':joueur_de_golf_peau_mate:','ja':':ゴルフをする人_やや濃い肌色:','ko':':골프치는_사람_진한_갈색_피부:','pt':':golfista_pele_morena_escura:','it':':persona_che_gioca_a_golf_carnagione_abbastanza_scura:','fa':':گلف_باز_پوست_گندمی:','id':':orang_bermain_golf_warna_kulit_gelap-sedang:','zh':':打高尔夫的人_中等深肤色:','ru':':гольфист_темный_тон_кожи:','tr':':golf_oynayan_kişi_orta_koyu_cilt_tonu:','ar':':شخص_يلعب_غولف_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3CC\U0001F3FC':{'en':':person_golfing_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':golfer(in)_mittelhelle_hautfarbe:','es':':persona_jugando_al_golf_tono_de_piel_claro_medio:','fr':':joueur_de_golf_peau_moyennement_claire:','ja':':ゴルフをする人_やや薄い肌色:','ko':':골프치는_사람_연한_갈색_피부:','pt':':golfista_pele_morena_clara:','it':':persona_che_gioca_a_golf_carnagione_abbastanza_chiara:','fa':':گلف_باز_پوست_روشن:','id':':orang_bermain_golf_warna_kulit_cerah-sedang:','zh':':打高尔夫的人_中等浅肤色:','ru':':гольфист_светлый_тон_кожи:','tr':':golf_oynayan_kişi_orta_açık_cilt_tonu:','ar':':شخص_يلعب_غولف_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3CC\U0001F3FD':{'en':':person_golfing_medium_skin_tone:','status':fully_qualified,'E':4,'de':':golfer(in)_mittlere_hautfarbe:','es':':persona_jugando_al_golf_tono_de_piel_medio:','fr':':joueur_de_golf_peau_légèrement_mate:','ja':':ゴルフをする人_中間の肌色:','ko':':골프치는_사람_갈색_피부:','pt':':golfista_pele_morena:','it':':persona_che_gioca_a_golf_carnagione_olivastra:','fa':':گلف_باز_پوست_طلایی:','id':':orang_bermain_golf_warna_kulit_sedang:','zh':':打高尔夫的人_中等肤色:','ru':':гольфист_средний_тон_кожи:','tr':':golf_oynayan_kişi_orta_cilt_tonu:','ar':':شخص_يلعب_غولف_بشرة_بلون_معتدل:'},'\U0001F6CC':{'en':':person_in_bed:','status':fully_qualified,'E':1,'alias':[':sleeping_bed:', ':sleeping_accommodation:'],'de':':im_bett_liegende_person:','es':':persona_en_la_cama:','fr':':personne_au_lit:','ja':':ベッドに寝る人:','ko':':침대에_누운_사람:','pt':':pessoa_deitada_na_cama:','it':':persona_a_letto:','fa':':فرد_در_تختخواب:','id':':orang_di_tempat_tidur:','zh':':躺在床上的人:','ru':':человек_в_кровати:','tr':':yatakta_yatan_kişi:','ar':':شخص_على_سرير:'},'\U0001F6CC\U0001F3FF':{'en':':person_in_bed_dark_skin_tone:','status':fully_qualified,'E':4,'de':':im_bett_liegende_person_dunkle_hautfarbe:','es':':persona_en_la_cama_tono_de_piel_oscuro:','fr':':personne_au_lit_peau_foncée:','ja':':ベッドに寝る人_濃い肌色:','ko':':침대에_누운_사람_검은색_피부:','pt':':pessoa_deitada_na_cama_pele_escura:','it':':persona_a_letto_carnagione_scura:','fa':':فرد_در_تختخواب_پوست_آبنوسی:','id':':orang_di_tempat_tidur_warna_kulit_gelap:','zh':':躺在床上的人_较深肤色:','ru':':человек_в_кровати_очень_темный_тон_кожи:','tr':':yatakta_yatan_kişi_koyu_cilt_tonu:','ar':':شخص_على_سرير_بشرة_بلون_غامق:'},'\U0001F6CC\U0001F3FB':{'en':':person_in_bed_light_skin_tone:','status':fully_qualified,'E':4,'de':':im_bett_liegende_person_helle_hautfarbe:','es':':persona_en_la_cama_tono_de_piel_claro:','fr':':personne_au_lit_peau_claire:','ja':':ベッドに寝る人_薄い肌色:','ko':':침대에_누운_사람_하얀_피부:','pt':':pessoa_deitada_na_cama_pele_clara:','it':':persona_a_letto_carnagione_chiara:','fa':':فرد_در_تختخواب_پوست_سفید:','id':':orang_di_tempat_tidur_warna_kulit_cerah:','zh':':躺在床上的人_较浅肤色:','ru':':человек_в_кровати_очень_светлый_тон_кожи:','tr':':yatakta_yatan_kişi_açık_cilt_tonu:','ar':':شخص_على_سرير_بشرة_بلون_فاتح:'},'\U0001F6CC\U0001F3FE':{'en':':person_in_bed_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':im_bett_liegende_person_mitteldunkle_hautfarbe:','es':':persona_en_la_cama_tono_de_piel_oscuro_medio:','fr':':personne_au_lit_peau_mate:','ja':':ベッドに寝る人_やや濃い肌色:','ko':':침대에_누운_사람_진한_갈색_피부:','pt':':pessoa_deitada_na_cama_pele_morena_escura:','it':':persona_a_letto_carnagione_abbastanza_scura:','fa':':فرد_در_تختخواب_پوست_گندمی:','id':':orang_di_tempat_tidur_warna_kulit_gelap-sedang:','zh':':躺在床上的人_中等深肤色:','ru':':человек_в_кровати_темный_тон_кожи:','tr':':yatakta_yatan_kişi_orta_koyu_cilt_tonu:','ar':':شخص_على_سرير_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6CC\U0001F3FC':{'en':':person_in_bed_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':im_bett_liegende_person_mittelhelle_hautfarbe:','es':':persona_en_la_cama_tono_de_piel_claro_medio:','fr':':personne_au_lit_peau_moyennement_claire:','ja':':ベッドに寝る人_やや薄い肌色:','ko':':침대에_누운_사람_연한_갈색_피부:','pt':':pessoa_deitada_na_cama_pele_morena_clara:','it':':persona_a_letto_carnagione_abbastanza_chiara:','fa':':فرد_در_تختخواب_پوست_روشن:','id':':orang_di_tempat_tidur_warna_kulit_cerah-sedang:','zh':':躺在床上的人_中等浅肤色:','ru':':человек_в_кровати_светлый_тон_кожи:','tr':':yatakta_yatan_kişi_orta_açık_cilt_tonu:','ar':':شخص_على_سرير_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6CC\U0001F3FD':{'en':':person_in_bed_medium_skin_tone:','status':fully_qualified,'E':4,'de':':im_bett_liegende_person_mittlere_hautfarbe:','es':':persona_en_la_cama_tono_de_piel_medio:','fr':':personne_au_lit_peau_légèrement_mate:','ja':':ベッドに寝る人_中間の肌色:','ko':':침대에_누운_사람_갈색_피부:','pt':':pessoa_deitada_na_cama_pele_morena:','it':':persona_a_letto_carnagione_olivastra:','fa':':فرد_در_تختخواب_پوست_طلایی:','id':':orang_di_tempat_tidur_warna_kulit_sedang:','zh':':躺在床上的人_中等肤色:','ru':':человек_в_кровати_средний_тон_кожи:','tr':':yatakta_yatan_kişi_orta_cilt_tonu:','ar':':شخص_على_سرير_بشرة_بلون_معتدل:'},'\U0001F9D8':{'en':':person_in_lotus_position:','status':fully_qualified,'E':5,'alias':[':lotus_position:'],'de':':person_im_lotossitz:','es':':persona_en_posición_de_loto:','fr':':personne_dans_la_posture_du_lotus:','ja':':ヨガのポーズをする人:','ko':':가부좌한_사람:','pt':':pessoa_na_posição_de_lótus:','it':':persona_in_posizione_del_loto:','fa':':فرد_در_حالت_خلسه:','id':':orang_dalam_posisi_teratai:','zh':':盘腿的人:','ru':':человек_в_позе_лотоса:','tr':':lotus_pozisyonundaki_kişi:','ar':':شخص_متربع:'},'\U0001F9D8\U0001F3FF':{'en':':person_in_lotus_position_dark_skin_tone:','status':fully_qualified,'E':5,'de':':person_im_lotossitz_dunkle_hautfarbe:','es':':persona_en_posición_de_loto_tono_de_piel_oscuro:','fr':':personne_dans_la_posture_du_lotus_peau_foncée:','ja':':ヨガのポーズをする人_濃い肌色:','ko':':가부좌한_사람_검은색_피부:','pt':':pessoa_na_posição_de_lótus_pele_escura:','it':':persona_in_posizione_del_loto_carnagione_scura:','fa':':فرد_در_حالت_خلسه_پوست_آبنوسی:','id':':orang_dalam_posisi_teratai_warna_kulit_gelap:','zh':':盘腿的人_较深肤色:','ru':':человек_в_позе_лотоса_очень_темный_тон_кожи:','tr':':lotus_pozisyonundaki_kişi_koyu_cilt_tonu:','ar':':شخص_متربع_بشرة_بلون_غامق:'},'\U0001F9D8\U0001F3FB':{'en':':person_in_lotus_position_light_skin_tone:','status':fully_qualified,'E':5,'de':':person_im_lotossitz_helle_hautfarbe:','es':':persona_en_posición_de_loto_tono_de_piel_claro:','fr':':personne_dans_la_posture_du_lotus_peau_claire:','ja':':ヨガのポーズをする人_薄い肌色:','ko':':가부좌한_사람_하얀_피부:','pt':':pessoa_na_posição_de_lótus_pele_clara:','it':':persona_in_posizione_del_loto_carnagione_chiara:','fa':':فرد_در_حالت_خلسه_پوست_سفید:','id':':orang_dalam_posisi_teratai_warna_kulit_cerah:','zh':':盘腿的人_较浅肤色:','ru':':человек_в_позе_лотоса_очень_светлый_тон_кожи:','tr':':lotus_pozisyonundaki_kişi_açık_cilt_tonu:','ar':':شخص_متربع_بشرة_بلون_فاتح:'},'\U0001F9D8\U0001F3FE':{'en':':person_in_lotus_position_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':person_im_lotossitz_mitteldunkle_hautfarbe:','es':':persona_en_posición_de_loto_tono_de_piel_oscuro_medio:','fr':':personne_dans_la_posture_du_lotus_peau_mate:','ja':':ヨガのポーズをする人_やや濃い肌色:','ko':':가부좌한_사람_진한_갈색_피부:','pt':':pessoa_na_posição_de_lótus_pele_morena_escura:','it':':persona_in_posizione_del_loto_carnagione_abbastanza_scura:','fa':':فرد_در_حالت_خلسه_پوست_گندمی:','id':':orang_dalam_posisi_teratai_warna_kulit_gelap-sedang:','zh':':盘腿的人_中等深肤色:','ru':':человек_в_позе_лотоса_темный_тон_кожи:','tr':':lotus_pozisyonundaki_kişi_orta_koyu_cilt_tonu:','ar':':شخص_متربع_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D8\U0001F3FC':{'en':':person_in_lotus_position_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':person_im_lotossitz_mittelhelle_hautfarbe:','es':':persona_en_posición_de_loto_tono_de_piel_claro_medio:','fr':':personne_dans_la_posture_du_lotus_peau_moyennement_claire:','ja':':ヨガのポーズをする人_やや薄い肌色:','ko':':가부좌한_사람_연한_갈색_피부:','pt':':pessoa_na_posição_de_lótus_pele_morena_clara:','it':':persona_in_posizione_del_loto_carnagione_abbastanza_chiara:','fa':':فرد_در_حالت_خلسه_پوست_روشن:','id':':orang_dalam_posisi_teratai_warna_kulit_cerah-sedang:','zh':':盘腿的人_中等浅肤色:','ru':':человек_в_позе_лотоса_светлый_тон_кожи:','tr':':lotus_pozisyonundaki_kişi_orta_açık_cilt_tonu:','ar':':شخص_متربع_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D8\U0001F3FD':{'en':':person_in_lotus_position_medium_skin_tone:','status':fully_qualified,'E':5,'de':':person_im_lotossitz_mittlere_hautfarbe:','es':':persona_en_posición_de_loto_tono_de_piel_medio:','fr':':personne_dans_la_posture_du_lotus_peau_légèrement_mate:','ja':':ヨガのポーズをする人_中間の肌色:','ko':':가부좌한_사람_갈색_피부:','pt':':pessoa_na_posição_de_lótus_pele_morena:','it':':persona_in_posizione_del_loto_carnagione_olivastra:','fa':':فرد_در_حالت_خلسه_پوست_طلایی:','id':':orang_dalam_posisi_teratai_warna_kulit_sedang:','zh':':盘腿的人_中等肤色:','ru':':человек_в_позе_лотоса_средний_тон_кожи:','tr':':lotus_pozisyonundaki_kişi_orta_cilt_tonu:','ar':':شخص_متربع_بشرة_بلون_معتدل:'},'\U0001F9D1\U0000200D\U0001F9BD':{'en':':person_in_manual_wheelchair:','status':fully_qualified,'E':12.1,'de':':person_in_manuellem_rollstuhl:','es':':persona_en_silla_de_ruedas_manual:','fr':':personne_en_fauteuil_roulant_manuel:','ja':':手動式車椅子の人:','ko':':수동_휠체어를_탄_사람:','pt':':pessoa_em_cadeira_de_rodas_manual:','it':':persona_su_sedia_a_rotelle_manuale:','fa':':شخصی_با_صندلی_چرخ_دار_دستی:','id':':orang_dengan_kursi_roda_manual:','zh':':坐手动轮椅的人:','ru':':человек_в_кресле_коляске_с_ручным_приводом:','tr':':tekerlekli_sandalyede_kişi:','ar':':شخص_على_كرسي_متحرك_يدوي:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F9BD':{'en':':person_in_manual_wheelchair_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':person_in_manuellem_rollstuhl_dunkle_hautfarbe:','es':':persona_en_silla_de_ruedas_manual_tono_de_piel_oscuro:','fr':':personne_en_fauteuil_roulant_manuel_peau_foncée:','ja':':手動式車椅子の人_濃い肌色:','ko':':수동_휠체어를_탄_사람_검은색_피부:','pt':':pessoa_em_cadeira_de_rodas_manual_pele_escura:','it':':persona_su_sedia_a_rotelle_manuale_carnagione_scura:','fa':':شخصی_با_صندلی_چرخ_دار_دستی_پوست_آبنوسی:','id':':orang_dengan_kursi_roda_manual_warna_kulit_gelap:','zh':':坐手动轮椅的人_较深肤色:','ru':':человек_в_кресле_коляске_с_ручным_приводом_очень_темный_тон_кожи:','tr':':tekerlekli_sandalyede_kişi_koyu_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_يدوي_بشرة_بلون_غامق:'},'\U0001F9D1\U0000200D\U0001F9BD\U0000200D\U000027A1\U0000FE0F':{'en':':person_in_manual_wheelchair_facing_right:','status':fully_qualified,'E':15.1,'de':':person_in_manuellem_rollstuhl_nach_rechts:','es':':persona_en_silla_de_ruedas_manual_hacia_la_derecha:','fr':':personne_en_fauteuil_roulant_manuel_vers_la_droite:','ja':':手動式車椅子の人_右向き:','ko':':오른쪽을_향한_수동_휠체어를_탄_사람:','pt':':pessoa_em_cadeira_de_rodas_manual_de_frente_para_a_direita:','it':':persona_su_sedia_a_rotelle_manuale_a_destra:','fa':':شخصی_با_صندلی_چرخ_دار_دستی_به_سمت_راست:','id':':orang_dengan_kursi_roda_manual_menghadap_kanan:','zh':':坐手动轮椅的人面向右边:','ru':':человек_в_кресле_коляске_с_ручным_приводом_направленные_вправо:','tr':':sağa_dönük_tekerlekli_sandalyede_kişi:','ar':':شخص_على_كرسي_متحرك_يدوي_متجه_لليمين:'},'\U0001F9D1\U0000200D\U0001F9BD\U0000200D\U000027A1':{'en':':person_in_manual_wheelchair_facing_right:','status':minimally_qualified,'E':15.1,'de':':person_in_manuellem_rollstuhl_nach_rechts:','es':':persona_en_silla_de_ruedas_manual_hacia_la_derecha:','fr':':personne_en_fauteuil_roulant_manuel_vers_la_droite:','ja':':手動式車椅子の人_右向き:','ko':':오른쪽을_향한_수동_휠체어를_탄_사람:','pt':':pessoa_em_cadeira_de_rodas_manual_de_frente_para_a_direita:','it':':persona_su_sedia_a_rotelle_manuale_a_destra:','fa':':شخصی_با_صندلی_چرخ_دار_دستی_به_سمت_راست:','id':':orang_dengan_kursi_roda_manual_menghadap_kanan:','zh':':坐手动轮椅的人面向右边:','ru':':человек_в_кресле_коляске_с_ручным_приводом_направленные_вправо:','tr':':sağa_dönük_tekerlekli_sandalyede_kişi:','ar':':شخص_على_كرسي_متحرك_يدوي_متجه_لليمين:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F9BD\U0000200D\U000027A1\U0000FE0F':{'en':':person_in_manual_wheelchair_facing_right_dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':person_in_manuellem_rollstuhl_nach_rechts_dunkle_hautfarbe:','es':':persona_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_oscuro:','fr':':personne_en_fauteuil_roulant_manuel_vers_la_droite_peau_foncée:','ja':':手動式車椅子の人_右向き_濃い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_사람_검은색_피부:','pt':':pessoa_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_escura:','it':':persona_su_sedia_a_rotelle_manuale_a_destra_carnagione_scura:','fa':':شخصی_با_صندلی_چرخ_دار_دستی_به_سمت_راستپوست_آبنوسی:','id':':orang_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_gelap:','zh':':坐手动轮椅的人面向右边_较深肤色:','ru':':человек_в_кресле_коляске_с_ручным_приводом_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_kişi_koyu_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_يدوي_متجه_لليمين_بلون_غامق:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F9BD\U0000200D\U000027A1':{'en':':person_in_manual_wheelchair_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':person_in_manuellem_rollstuhl_nach_rechts_dunkle_hautfarbe:','es':':persona_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_oscuro:','fr':':personne_en_fauteuil_roulant_manuel_vers_la_droite_peau_foncée:','ja':':手動式車椅子の人_右向き_濃い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_사람_검은색_피부:','pt':':pessoa_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_escura:','it':':persona_su_sedia_a_rotelle_manuale_a_destra_carnagione_scura:','fa':':شخصی_با_صندلی_چرخ_دار_دستی_به_سمت_راستپوست_آبنوسی:','id':':orang_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_gelap:','zh':':坐手动轮椅的人面向右边_较深肤色:','ru':':человек_в_кресле_коляске_с_ручным_приводом_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_kişi_koyu_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_يدوي_متجه_لليمين_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F9BD\U0000200D\U000027A1\U0000FE0F':{'en':':person_in_manual_wheelchair_facing_right_light_skin_tone:','status':fully_qualified,'E':15.1,'de':':person_in_manuellem_rollstuhl_nach_rechts_helle_hautfarbe:','es':':persona_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_claro:','fr':':personne_en_fauteuil_roulant_manuel_vers_la_droite_peau_claire:','ja':':手動式車椅子の人_右向き_薄い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_사람_하얀_피부:','pt':':pessoa_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_clara:','it':':persona_su_sedia_a_rotelle_manuale_a_destra_carnagione_chiara:','fa':':شخصی_با_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_سفید:','id':':orang_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_cerah:','zh':':坐手动轮椅的人面向右边_较浅肤色:','ru':':человек_в_кресле_коляске_с_ручным_приводом_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_kişi_açık_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_يدوي_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F9BD\U0000200D\U000027A1':{'en':':person_in_manual_wheelchair_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':person_in_manuellem_rollstuhl_nach_rechts_helle_hautfarbe:','es':':persona_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_claro:','fr':':personne_en_fauteuil_roulant_manuel_vers_la_droite_peau_claire:','ja':':手動式車椅子の人_右向き_薄い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_사람_하얀_피부:','pt':':pessoa_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_clara:','it':':persona_su_sedia_a_rotelle_manuale_a_destra_carnagione_chiara:','fa':':شخصی_با_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_سفید:','id':':orang_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_cerah:','zh':':坐手动轮椅的人面向右边_较浅肤色:','ru':':человек_в_кресле_коляске_с_ручным_приводом_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_kişi_açık_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_يدوي_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F9BD\U0000200D\U000027A1\U0000FE0F':{'en':':person_in_manual_wheelchair_facing_right_medium-dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':person_in_manuellem_rollstuhl_nach_rechts_mitteldunkle_hautfarbe:','es':':persona_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':personne_en_fauteuil_roulant_manuel_vers_la_droite_peau_mate:','ja':':手動式車椅子の人_右向き_やや濃い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_사람_진한_갈색_피부:','pt':':pessoa_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_morena_escura:','it':':persona_su_sedia_a_rotelle_manuale_a_destra_carnagione_abbastanza_scura:','fa':':شخصی_با_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_گندمی:','id':':orang_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':坐手动轮椅的人面向右边_中等深肤色:','ru':':человек_в_кресле_коляске_с_ручным_приводом_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_kişi_orta_koyu_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_يدوي_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F9BD\U0000200D\U000027A1':{'en':':person_in_manual_wheelchair_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':person_in_manuellem_rollstuhl_nach_rechts_mitteldunkle_hautfarbe:','es':':persona_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':personne_en_fauteuil_roulant_manuel_vers_la_droite_peau_mate:','ja':':手動式車椅子の人_右向き_やや濃い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_사람_진한_갈색_피부:','pt':':pessoa_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_morena_escura:','it':':persona_su_sedia_a_rotelle_manuale_a_destra_carnagione_abbastanza_scura:','fa':':شخصی_با_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_گندمی:','id':':orang_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':坐手动轮椅的人面向右边_中等深肤色:','ru':':человек_в_кресле_коляске_с_ручным_приводом_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_kişi_orta_koyu_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_يدوي_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F9BD\U0000200D\U000027A1\U0000FE0F':{'en':':person_in_manual_wheelchair_facing_right_medium-light_skin_tone:','status':fully_qualified,'E':15.1,'de':':person_in_manuellem_rollstuhl_nach_rechts_mittelhelle_hautfarbe:','es':':persona_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':personne_en_fauteuil_roulant_manuel_vers_la_droite_peau_moyennement_claire:','ja':':手動式車椅子の人_右向き_やや薄い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_사람_연한_갈색_피부:','pt':':pessoa_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_morena_clara:','it':':persona_su_sedia_a_rotelle_manuale_a_destra_carnagione_abbastanza_chiara:','fa':':شخصی_با_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_روشن:','id':':orang_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':坐手动轮椅的人面向右边_中等浅肤色:','ru':':человек_в_кресле_коляске_с_ручным_приводом_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_tekerlekli_sandalyede_kişi_orta_açık_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_يدوي_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F9BD\U0000200D\U000027A1':{'en':':person_in_manual_wheelchair_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':person_in_manuellem_rollstuhl_nach_rechts_mittelhelle_hautfarbe:','es':':persona_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':personne_en_fauteuil_roulant_manuel_vers_la_droite_peau_moyennement_claire:','ja':':手動式車椅子の人_右向き_やや薄い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_사람_연한_갈색_피부:','pt':':pessoa_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_morena_clara:','it':':persona_su_sedia_a_rotelle_manuale_a_destra_carnagione_abbastanza_chiara:','fa':':شخصی_با_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_روشن:','id':':orang_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':坐手动轮椅的人面向右边_中等浅肤色:','ru':':человек_в_кресле_коляске_с_ручным_приводом_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_tekerlekli_sandalyede_kişi_orta_açık_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_يدوي_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F9BD\U0000200D\U000027A1\U0000FE0F':{'en':':person_in_manual_wheelchair_facing_right_medium_skin_tone:','status':fully_qualified,'E':15.1,'de':':person_in_manuellem_rollstuhl_nach_rechts_mittlere_hautfarbe:','es':':persona_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_medio:','fr':':personne_en_fauteuil_roulant_manuel_vers_la_droite_peau_légèrement_mate:','ja':':手動式車椅子の人_右向き_中間の肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_사람_갈색_피부:','pt':':pessoa_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_morena:','it':':persona_su_sedia_a_rotelle_manuale_a_destra_carnagione_olivastra:','fa':':شخصی_با_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_طلایی:','id':':orang_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_sedang:','zh':':坐手动轮椅的人面向右边_中等肤色:','ru':':человек_в_кресле_коляске_с_ручным_приводом_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_kişi_orta_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_يدوي_متجه_لليمين_بلون_معتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F9BD\U0000200D\U000027A1':{'en':':person_in_manual_wheelchair_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':person_in_manuellem_rollstuhl_nach_rechts_mittlere_hautfarbe:','es':':persona_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_medio:','fr':':personne_en_fauteuil_roulant_manuel_vers_la_droite_peau_légèrement_mate:','ja':':手動式車椅子の人_右向き_中間の肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_사람_갈색_피부:','pt':':pessoa_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_morena:','it':':persona_su_sedia_a_rotelle_manuale_a_destra_carnagione_olivastra:','fa':':شخصی_با_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_طلایی:','id':':orang_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_sedang:','zh':':坐手动轮椅的人面向右边_中等肤色:','ru':':человек_в_кресле_коляске_с_ручным_приводом_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_kişi_orta_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_يدوي_متجه_لليمين_بلون_معتدل:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F9BD':{'en':':person_in_manual_wheelchair_light_skin_tone:','status':fully_qualified,'E':12.1,'de':':person_in_manuellem_rollstuhl_helle_hautfarbe:','es':':persona_en_silla_de_ruedas_manual_tono_de_piel_claro:','fr':':personne_en_fauteuil_roulant_manuel_peau_claire:','ja':':手動式車椅子の人_薄い肌色:','ko':':수동_휠체어를_탄_사람_하얀_피부:','pt':':pessoa_em_cadeira_de_rodas_manual_pele_clara:','it':':persona_su_sedia_a_rotelle_manuale_carnagione_chiara:','fa':':شخصی_با_صندلی_چرخ_دار_دستی_پوست_سفید:','id':':orang_dengan_kursi_roda_manual_warna_kulit_cerah:','zh':':坐手动轮椅的人_较浅肤色:','ru':':человек_в_кресле_коляске_с_ручным_приводом_очень_светлый_тон_кожи:','tr':':tekerlekli_sandalyede_kişi_açık_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_يدوي_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F9BD':{'en':':person_in_manual_wheelchair_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':person_in_manuellem_rollstuhl_mitteldunkle_hautfarbe:','es':':persona_en_silla_de_ruedas_manual_tono_de_piel_oscuro_medio:','fr':':personne_en_fauteuil_roulant_manuel_peau_mate:','ja':':手動式車椅子の人_やや濃い肌色:','ko':':수동_휠체어를_탄_사람_진한_갈색_피부:','pt':':pessoa_em_cadeira_de_rodas_manual_pele_morena_escura:','it':':persona_su_sedia_a_rotelle_manuale_carnagione_abbastanza_scura:','fa':':شخصی_با_صندلی_چرخ_دار_دستی_پوست_گندمی:','id':':orang_dengan_kursi_roda_manual_warna_kulit_gelap-sedang:','zh':':坐手动轮椅的人_中等深肤色:','ru':':человек_в_кресле_коляске_с_ручным_приводом_темный_тон_кожи:','tr':':tekerlekli_sandalyede_kişi_orta_koyu_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_يدوي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F9BD':{'en':':person_in_manual_wheelchair_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':person_in_manuellem_rollstuhl_mittelhelle_hautfarbe:','es':':persona_en_silla_de_ruedas_manual_tono_de_piel_claro_medio:','fr':':personne_en_fauteuil_roulant_manuel_peau_moyennement_claire:','ja':':手動式車椅子の人_やや薄い肌色:','ko':':수동_휠체어를_탄_사람_연한_갈색_피부:','pt':':pessoa_em_cadeira_de_rodas_manual_pele_morena_clara:','it':':persona_su_sedia_a_rotelle_manuale_carnagione_abbastanza_chiara:','fa':':شخصی_با_صندلی_چرخ_دار_دستی_پوست_روشن:','id':':orang_dengan_kursi_roda_manual_warna_kulit_cerah-sedang:','zh':':坐手动轮椅的人_中等浅肤色:','ru':':человек_в_кресле_коляске_с_ручным_приводом_светлый_тон_кожи:','tr':':tekerlekli_sandalyede_kişi_orta_açık_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_يدوي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F9BD':{'en':':person_in_manual_wheelchair_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':person_in_manuellem_rollstuhl_mittlere_hautfarbe:','es':':persona_en_silla_de_ruedas_manual_tono_de_piel_medio:','fr':':personne_en_fauteuil_roulant_manuel_peau_légèrement_mate:','ja':':手動式車椅子の人_中間の肌色:','ko':':수동_휠체어를_탄_사람_갈색_피부:','pt':':pessoa_em_cadeira_de_rodas_manual_pele_morena:','it':':persona_su_sedia_a_rotelle_manuale_carnagione_olivastra:','fa':':شخصی_با_صندلی_چرخ_دار_دستی_پوست_طلایی:','id':':orang_dengan_kursi_roda_manual_warna_kulit_sedang:','zh':':坐手动轮椅的人_中等肤色:','ru':':человек_в_кресле_коляске_с_ручным_приводом_средний_тон_кожи:','tr':':tekerlekli_sandalyede_kişi_orta_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_يدوي_بشرة_بلون_معتدل:'},'\U0001F9D1\U0000200D\U0001F9BC':{'en':':person_in_motorized_wheelchair:','status':fully_qualified,'E':12.1,'de':':person_in_elektrischem_rollstuhl:','es':':persona_en_silla_de_ruedas_eléctrica:','fr':':personne_en_fauteuil_motorisé:','ja':':電動車椅子の人:','ko':':전동_휠체어를_탄_사람:','pt':':pessoa_em_cadeira_de_rodas_motorizada:','it':':persona_su_sedia_a_rotelle_motorizzata:','fa':':شخصی_با_صندلی_چرخ_دار_موتوری:','id':':orang_dengan_kursi_roda_bermotor:','zh':':坐电动轮椅的人:','ru':':человек_в_кресле_коляске_с_мотором:','tr':':elektrikli_tekerlekli_sandalyede_kişi:','ar':':شخص_على_كرسي_متحرك_كهربائي:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F9BC':{'en':':person_in_motorized_wheelchair_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':person_in_elektrischem_rollstuhl_dunkle_hautfarbe:','es':':persona_en_silla_de_ruedas_eléctrica_tono_de_piel_oscuro:','fr':':personne_en_fauteuil_motorisé_peau_foncée:','ja':':電動車椅子の人_濃い肌色:','ko':':전동_휠체어를_탄_사람_검은색_피부:','pt':':pessoa_em_cadeira_de_rodas_motorizada_pele_escura:','it':':persona_su_sedia_a_rotelle_motorizzata_carnagione_scura:','fa':':شخصی_با_صندلی_چرخ_دار_موتوری_پوست_آبنوسی:','id':':orang_dengan_kursi_roda_bermotor_warna_kulit_gelap:','zh':':坐电动轮椅的人_较深肤色:','ru':':человек_в_кресле_коляске_с_мотором_очень_темный_тон_кожи:','tr':':elektrikli_tekerlekli_sandalyede_kişi_koyu_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_كهربائي_بشرة_بلون_غامق:'},'\U0001F9D1\U0000200D\U0001F9BC\U0000200D\U000027A1\U0000FE0F':{'en':':person_in_motorized_wheelchair_facing_right:','status':fully_qualified,'E':15.1,'de':':person_in_elektrischem_rollstuhl_nach_rechts:','es':':persona_en_silla_de_ruedas_eléctrica_hacia_la_derecha:','fr':':personne_en_fauteuil_motorisé_vers_la_droite:','ja':':電動車椅子の人_右向き:','ko':':오른쪽을_향한_전동_휠체어를_탄_사람:','pt':':pessoa_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita:','it':':persona_su_sedia_a_rotelle_motorizzata_a_destra:','fa':':شخصی_با_صندلی_چرخ_دار_موتوری_به_سمت_راست:','id':':orang_dengan_kursi_roda_bermotor_menghadap_kanan:','zh':':坐电动轮椅的人面向右边:','ru':':человек_в_кресле_коляске_с_мотором_направленные_вправо:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kişi:','ar':':شخص_على_كرسي_متحرك_كهربائي_متجه_لليمين:'},'\U0001F9D1\U0000200D\U0001F9BC\U0000200D\U000027A1':{'en':':person_in_motorized_wheelchair_facing_right:','status':minimally_qualified,'E':15.1,'de':':person_in_elektrischem_rollstuhl_nach_rechts:','es':':persona_en_silla_de_ruedas_eléctrica_hacia_la_derecha:','fr':':personne_en_fauteuil_motorisé_vers_la_droite:','ja':':電動車椅子の人_右向き:','ko':':오른쪽을_향한_전동_휠체어를_탄_사람:','pt':':pessoa_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita:','it':':persona_su_sedia_a_rotelle_motorizzata_a_destra:','fa':':شخصی_با_صندلی_چرخ_دار_موتوری_به_سمت_راست:','id':':orang_dengan_kursi_roda_bermotor_menghadap_kanan:','zh':':坐电动轮椅的人面向右边:','ru':':человек_в_кресле_коляске_с_мотором_направленные_вправо:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kişi:','ar':':شخص_على_كرسي_متحرك_كهربائي_متجه_لليمين:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F9BC\U0000200D\U000027A1\U0000FE0F':{'en':':person_in_motorized_wheelchair_facing_right_dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':person_in_elektrischem_rollstuhl_nach_rechts_dunkle_hautfarbe:','es':':persona_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_oscuro:','fr':':personne_en_fauteuil_motorisé_vers_la_droite_peau_foncée:','ja':':電動車椅子の人_右向き_濃い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_사람_검은색_피부:','pt':':pessoa_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_escura:','it':':persona_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_scura:','fa':':شخصی_با_صندلی_چرخ_دار_موتوری_به_سمت_راستپوست_آبنوسی:','id':':orang_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_gelap:','zh':':坐电动轮椅的人面向右边_较深肤色:','ru':':человек_в_кресле_коляске_с_мотором_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kişi_koyu_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_كهربائي_متجه_لليمين_بلون_غامق:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F9BC\U0000200D\U000027A1':{'en':':person_in_motorized_wheelchair_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':person_in_elektrischem_rollstuhl_nach_rechts_dunkle_hautfarbe:','es':':persona_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_oscuro:','fr':':personne_en_fauteuil_motorisé_vers_la_droite_peau_foncée:','ja':':電動車椅子の人_右向き_濃い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_사람_검은색_피부:','pt':':pessoa_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_escura:','it':':persona_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_scura:','fa':':شخصی_با_صندلی_چرخ_دار_موتوری_به_سمت_راستپوست_آبنوسی:','id':':orang_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_gelap:','zh':':坐电动轮椅的人面向右边_较深肤色:','ru':':человек_в_кресле_коляске_с_мотором_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kişi_koyu_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_كهربائي_متجه_لليمين_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F9BC\U0000200D\U000027A1\U0000FE0F':{'en':':person_in_motorized_wheelchair_facing_right_light_skin_tone:','status':fully_qualified,'E':15.1,'de':':person_in_elektrischem_rollstuhl_nach_rechts_helle_hautfarbe:','es':':persona_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_claro:','fr':':personne_en_fauteuil_motorisé_vers_la_droite_peau_claire:','ja':':電動車椅子の人_右向き_薄い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_사람_하얀_피부:','pt':':pessoa_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_clara:','it':':persona_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_chiara:','fa':':شخصی_با_صندلی_چرخ_دار_موتوری_به_سمت_راست_پوست_سفید:','id':':orang_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_cerah:','zh':':坐电动轮椅的人面向右边_较浅肤色:','ru':':человек_в_кресле_коляске_с_мотором_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kişi_açık_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_كهربائي_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F9BC\U0000200D\U000027A1':{'en':':person_in_motorized_wheelchair_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':person_in_elektrischem_rollstuhl_nach_rechts_helle_hautfarbe:','es':':persona_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_claro:','fr':':personne_en_fauteuil_motorisé_vers_la_droite_peau_claire:','ja':':電動車椅子の人_右向き_薄い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_사람_하얀_피부:','pt':':pessoa_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_clara:','it':':persona_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_chiara:','fa':':شخصی_با_صندلی_چرخ_دار_موتوری_به_سمت_راست_پوست_سفید:','id':':orang_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_cerah:','zh':':坐电动轮椅的人面向右边_较浅肤色:','ru':':человек_в_кресле_коляске_с_мотором_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kişi_açık_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_كهربائي_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F9BC\U0000200D\U000027A1\U0000FE0F':{'en':':person_in_motorized_wheelchair_facing_right_medium-dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':person_in_elektrischem_rollstuhl_nach_rechts_mitteldunkle_hautfarbe:','es':':persona_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':personne_en_fauteuil_motorisé_vers_la_droite_peau_mate:','ja':':電動車椅子の人_右向き_やや濃い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_사람_진한_갈색_피부:','pt':':pessoa_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_morena_escura:','it':':persona_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_abbastanza_scura:','fa':':شخصی_با_صندلی_چرخ_دار_موتوری_به_سمت_راست_پوست_گندمی:','id':':orang_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':坐电动轮椅的人面向右边_中等深肤色:','ru':':человек_в_кресле_коляске_с_мотором_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kişi_orta_koyu_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_كهربائي_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F9BC\U0000200D\U000027A1':{'en':':person_in_motorized_wheelchair_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':person_in_elektrischem_rollstuhl_nach_rechts_mitteldunkle_hautfarbe:','es':':persona_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':personne_en_fauteuil_motorisé_vers_la_droite_peau_mate:','ja':':電動車椅子の人_右向き_やや濃い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_사람_진한_갈색_피부:','pt':':pessoa_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_morena_escura:','it':':persona_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_abbastanza_scura:','fa':':شخصی_با_صندلی_چرخ_دار_موتوری_به_سمت_راست_پوست_گندمی:','id':':orang_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':坐电动轮椅的人面向右边_中等深肤色:','ru':':человек_в_кресле_коляске_с_мотором_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kişi_orta_koyu_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_كهربائي_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F9BC\U0000200D\U000027A1\U0000FE0F':{'en':':person_in_motorized_wheelchair_facing_right_medium-light_skin_tone:','status':fully_qualified,'E':15.1,'de':':person_in_elektrischem_rollstuhl_nach_rechts_mittelhelle_hautfarbe:','es':':persona_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':personne_en_fauteuil_motorisé_vers_la_droite_peau_moyennement_claire:','ja':':電動車椅子の人_右向き_やや薄い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_사람_연한_갈색_피부:','pt':':pessoa_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_morena_clara:','it':':persona_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_abbastanza_chiara:','fa':':شخصی_با_صندلی_چرخ_دار_موتوری_به_سمت_راست_پوست_روشن:','id':':orang_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':坐电动轮椅的人面向右边_中等浅肤色:','ru':':человек_в_кресле_коляске_с_мотором_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kişi_orta_açık_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_كهربائي_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F9BC\U0000200D\U000027A1':{'en':':person_in_motorized_wheelchair_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':person_in_elektrischem_rollstuhl_nach_rechts_mittelhelle_hautfarbe:','es':':persona_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':personne_en_fauteuil_motorisé_vers_la_droite_peau_moyennement_claire:','ja':':電動車椅子の人_右向き_やや薄い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_사람_연한_갈색_피부:','pt':':pessoa_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_morena_clara:','it':':persona_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_abbastanza_chiara:','fa':':شخصی_با_صندلی_چرخ_دار_موتوری_به_سمت_راست_پوست_روشن:','id':':orang_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':坐电动轮椅的人面向右边_中等浅肤色:','ru':':человек_в_кресле_коляске_с_мотором_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kişi_orta_açık_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_كهربائي_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F9BC\U0000200D\U000027A1\U0000FE0F':{'en':':person_in_motorized_wheelchair_facing_right_medium_skin_tone:','status':fully_qualified,'E':15.1,'de':':person_in_elektrischem_rollstuhl_nach_rechts_mittlere_hautfarbe:','es':':persona_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_medio:','fr':':personne_en_fauteuil_motorisé_vers_la_droite_peau_légèrement_mate:','ja':':電動車椅子の人_右向き_中間の肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_사람_갈색_피부:','pt':':pessoa_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_morena:','it':':persona_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_olivastra:','fa':':شخصی_با_صندلی_چرخ_دار_موتوری_به_سمت_راست_پوست_طلایی:','id':':orang_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_sedang:','zh':':坐电动轮椅的人面向右边_中等肤色:','ru':':человек_в_кресле_коляске_с_мотором_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kişi_orta_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_كهربائي_متجه_لليمين_بلون_معتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F9BC\U0000200D\U000027A1':{'en':':person_in_motorized_wheelchair_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':person_in_elektrischem_rollstuhl_nach_rechts_mittlere_hautfarbe:','es':':persona_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_medio:','fr':':personne_en_fauteuil_motorisé_vers_la_droite_peau_légèrement_mate:','ja':':電動車椅子の人_右向き_中間の肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_사람_갈색_피부:','pt':':pessoa_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_morena:','it':':persona_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_olivastra:','fa':':شخصی_با_صندلی_چرخ_دار_موتوری_به_سمت_راست_پوست_طلایی:','id':':orang_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_sedang:','zh':':坐电动轮椅的人面向右边_中等肤色:','ru':':человек_в_кресле_коляске_с_мотором_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kişi_orta_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_كهربائي_متجه_لليمين_بلون_معتدل:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F9BC':{'en':':person_in_motorized_wheelchair_light_skin_tone:','status':fully_qualified,'E':12.1,'de':':person_in_elektrischem_rollstuhl_helle_hautfarbe:','es':':persona_en_silla_de_ruedas_eléctrica_tono_de_piel_claro:','fr':':personne_en_fauteuil_motorisé_peau_claire:','ja':':電動車椅子の人_薄い肌色:','ko':':전동_휠체어를_탄_사람_하얀_피부:','pt':':pessoa_em_cadeira_de_rodas_motorizada_pele_clara:','it':':persona_su_sedia_a_rotelle_motorizzata_carnagione_chiara:','fa':':شخصی_با_صندلی_چرخ_دار_موتوری_پوست_سفید:','id':':orang_dengan_kursi_roda_bermotor_warna_kulit_cerah:','zh':':坐电动轮椅的人_较浅肤色:','ru':':человек_в_кресле_коляске_с_мотором_очень_светлый_тон_кожи:','tr':':elektrikli_tekerlekli_sandalyede_kişi_açık_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_كهربائي_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F9BC':{'en':':person_in_motorized_wheelchair_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':person_in_elektrischem_rollstuhl_mitteldunkle_hautfarbe:','es':':persona_en_silla_de_ruedas_eléctrica_tono_de_piel_oscuro_medio:','fr':':personne_en_fauteuil_motorisé_peau_mate:','ja':':電動車椅子の人_やや濃い肌色:','ko':':전동_휠체어를_탄_사람_진한_갈색_피부:','pt':':pessoa_em_cadeira_de_rodas_motorizada_pele_morena_escura:','it':':persona_su_sedia_a_rotelle_motorizzata_carnagione_abbastanza_scura:','fa':':شخصی_با_صندلی_چرخ_دار_موتوری_پوست_گندمی:','id':':orang_dengan_kursi_roda_bermotor_warna_kulit_gelap-sedang:','zh':':坐电动轮椅的人_中等深肤色:','ru':':человек_в_кресле_коляске_с_мотором_темный_тон_кожи:','tr':':elektrikli_tekerlekli_sandalyede_kişi_orta_koyu_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_كهربائي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F9BC':{'en':':person_in_motorized_wheelchair_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':person_in_elektrischem_rollstuhl_mittelhelle_hautfarbe:','es':':persona_en_silla_de_ruedas_eléctrica_tono_de_piel_claro_medio:','fr':':personne_en_fauteuil_motorisé_peau_moyennement_claire:','ja':':電動車椅子の人_やや薄い肌色:','ko':':전동_휠체어를_탄_사람_연한_갈색_피부:','pt':':pessoa_em_cadeira_de_rodas_motorizada_pele_morena_clara:','it':':persona_su_sedia_a_rotelle_motorizzata_carnagione_abbastanza_chiara:','fa':':شخصی_با_صندلی_چرخ_دار_موتوری_پوست_روشن:','id':':orang_dengan_kursi_roda_bermotor_warna_kulit_cerah-sedang:','zh':':坐电动轮椅的人_中等浅肤色:','ru':':человек_в_кресле_коляске_с_мотором_светлый_тон_кожи:','tr':':elektrikli_tekerlekli_sandalyede_kişi_orta_açık_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_كهربائي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F9BC':{'en':':person_in_motorized_wheelchair_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':person_in_elektrischem_rollstuhl_mittlere_hautfarbe:','es':':persona_en_silla_de_ruedas_eléctrica_tono_de_piel_medio:','fr':':personne_en_fauteuil_motorisé_peau_légèrement_mate:','ja':':電動車椅子の人_中間の肌色:','ko':':전동_휠체어를_탄_사람_갈색_피부:','pt':':pessoa_em_cadeira_de_rodas_motorizada_pele_morena:','it':':persona_su_sedia_a_rotelle_motorizzata_carnagione_olivastra:','fa':':شخصی_با_صندلی_چرخ_دار_موتوری_پوست_طلایی:','id':':orang_dengan_kursi_roda_bermotor_warna_kulit_sedang:','zh':':坐电动轮椅的人_中等肤色:','ru':':человек_в_кресле_коляске_с_мотором_средний_тон_кожи:','tr':':elektrikli_tekerlekli_sandalyede_kişi_orta_cilt_tonu:','ar':':شخص_على_كرسي_متحرك_كهربائي_بشرة_بلون_معتدل:'},'\U0001F9D6':{'en':':person_in_steamy_room:','status':fully_qualified,'E':5,'alias':[':sauna_person:'],'de':':person_in_dampfsauna:','es':':persona_en_una_sauna:','fr':':personne_au_hammam:','ja':':サウナに入る人:','ko':':사우나_하는_사람:','pt':':pessoa_na_sauna:','it':':persona_in_sauna:','fa':':فردی_در_حمام_بخار:','id':':orang_di_ruangan_beruap:','zh':':蒸房里的人:','ru':':человек_в_бане:','tr':':buhar_odasındaki_kişi:','ar':':شخص_في_حمام_بخار:'},'\U0001F9D6\U0001F3FF':{'en':':person_in_steamy_room_dark_skin_tone:','status':fully_qualified,'E':5,'de':':person_in_dampfsauna_dunkle_hautfarbe:','es':':persona_en_una_sauna_tono_de_piel_oscuro:','fr':':personne_au_hammam_peau_foncée:','ja':':サウナに入る人_濃い肌色:','ko':':사우나_하는_사람_검은색_피부:','pt':':pessoa_na_sauna_pele_escura:','it':':persona_in_sauna_carnagione_scura:','fa':':فردی_در_حمام_بخار_پوست_آبنوسی:','id':':orang_di_ruangan_beruap_warna_kulit_gelap:','zh':':蒸房里的人_较深肤色:','ru':':человек_в_бане_очень_темный_тон_кожи:','tr':':buhar_odasındaki_kişi_koyu_cilt_tonu:','ar':':شخص_في_حمام_بخار_بشرة_بلون_غامق:'},'\U0001F9D6\U0001F3FB':{'en':':person_in_steamy_room_light_skin_tone:','status':fully_qualified,'E':5,'de':':person_in_dampfsauna_helle_hautfarbe:','es':':persona_en_una_sauna_tono_de_piel_claro:','fr':':personne_au_hammam_peau_claire:','ja':':サウナに入る人_薄い肌色:','ko':':사우나_하는_사람_하얀_피부:','pt':':pessoa_na_sauna_pele_clara:','it':':persona_in_sauna_carnagione_chiara:','fa':':فردی_در_حمام_بخار_پوست_سفید:','id':':orang_di_ruangan_beruap_warna_kulit_cerah:','zh':':蒸房里的人_较浅肤色:','ru':':человек_в_бане_очень_светлый_тон_кожи:','tr':':buhar_odasındaki_kişi_açık_cilt_tonu:','ar':':شخص_في_حمام_بخار_بشرة_بلون_فاتح:'},'\U0001F9D6\U0001F3FE':{'en':':person_in_steamy_room_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':person_in_dampfsauna_mitteldunkle_hautfarbe:','es':':persona_en_una_sauna_tono_de_piel_oscuro_medio:','fr':':personne_au_hammam_peau_mate:','ja':':サウナに入る人_やや濃い肌色:','ko':':사우나_하는_사람_진한_갈색_피부:','pt':':pessoa_na_sauna_pele_morena_escura:','it':':persona_in_sauna_carnagione_abbastanza_scura:','fa':':فردی_در_حمام_بخار_پوست_گندمی:','id':':orang_di_ruangan_beruap_warna_kulit_gelap-sedang:','zh':':蒸房里的人_中等深肤色:','ru':':человек_в_бане_темный_тон_кожи:','tr':':buhar_odasındaki_kişi_orta_koyu_cilt_tonu:','ar':':شخص_في_حمام_بخار_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D6\U0001F3FC':{'en':':person_in_steamy_room_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':person_in_dampfsauna_mittelhelle_hautfarbe:','es':':persona_en_una_sauna_tono_de_piel_claro_medio:','fr':':personne_au_hammam_peau_moyennement_claire:','ja':':サウナに入る人_やや薄い肌色:','ko':':사우나_하는_사람_연한_갈색_피부:','pt':':pessoa_na_sauna_pele_morena_clara:','it':':persona_in_sauna_carnagione_abbastanza_chiara:','fa':':فردی_در_حمام_بخار_پوست_روشن:','id':':orang_di_ruangan_beruap_warna_kulit_cerah-sedang:','zh':':蒸房里的人_中等浅肤色:','ru':':человек_в_бане_светлый_тон_кожи:','tr':':buhar_odasındaki_kişi_orta_açık_cilt_tonu:','ar':':شخص_في_حمام_بخار_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D6\U0001F3FD':{'en':':person_in_steamy_room_medium_skin_tone:','status':fully_qualified,'E':5,'de':':person_in_dampfsauna_mittlere_hautfarbe:','es':':persona_en_una_sauna_tono_de_piel_medio:','fr':':personne_au_hammam_peau_légèrement_mate:','ja':':サウナに入る人_中間の肌色:','ko':':사우나_하는_사람_갈색_피부:','pt':':pessoa_na_sauna_pele_morena:','it':':persona_in_sauna_carnagione_olivastra:','fa':':فردی_در_حمام_بخار_پوست_طلایی:','id':':orang_di_ruangan_beruap_warna_kulit_sedang:','zh':':蒸房里的人_中等肤色:','ru':':человек_в_бане_средний_тон_кожи:','tr':':buhar_odasındaki_kişi_orta_cilt_tonu:','ar':':شخص_في_حمام_بخار_بشرة_بلون_معتدل:'},'\U0001F574\U0000FE0F':{'en':':person_in_suit_levitating:','status':fully_qualified,'E':0.7,'alias':[':business_suit_levitating:', ':man_in_business_suit_levitating:'],'variant':True,'de':':schwebender_mann_im_anzug:','es':':hombre_trajeado_levitando:','fr':':homme_d’affaires_en_lévitation:','ja':':浮いてるビジネスマン:','ko':':공중에_떠_있는_정장_입은_남자:','pt':':homem_de_terno_levitando:','it':':uomo_con_completo_che_levita:','fa':':پرواز_مرد_با_لباس_رسمی:','id':':pria_melayang_mengenakan_jas:','zh':':西装革履的人:','ru':':бизнесмен_в_воздухе:','tr':':havalanan_takım_elbiseli_adam:','ar':':رجل_أعمال_يرتقي:'},'\U0001F574':{'en':':person_in_suit_levitating:','status':unqualified,'E':0.7,'alias':[':business_suit_levitating:', ':man_in_business_suit_levitating:'],'variant':True,'de':':schwebender_mann_im_anzug:','es':':persona_trajeada_levitando:','fr':':homme_d’affaires_en_lévitation:','ja':':浮いてるビジネスマン:','ko':':공중에_떠_있는_정장_입은_남자:','pt':':homem_de_terno_levitando:','it':':uomo_con_completo_che_levita:','fa':':پرواز_مرد_با_لباس_رسمی:','id':':pria_melayang_mengenakan_jas:','zh':':西装革履的人:','ru':':бизнесмен_в_воздухе:','tr':':havalanan_takım_elbiseli_adam:','ar':':رجل_أعمال_يرتقي:'},'\U0001F574\U0001F3FF':{'en':':person_in_suit_levitating_dark_skin_tone:','status':fully_qualified,'E':4,'de':':schwebender_mann_im_anzug_dunkle_hautfarbe:','es':':persona_trajeada_levitando_tono_de_piel_oscuro:','fr':':homme_d’affaires_en_lévitation_peau_foncée:','ja':':浮いてるビジネスマン_濃い肌色:','ko':':공중에_떠_있는_정장_입은_남자_검은색_피부:','pt':':homem_de_terno_levitando_pele_escura:','it':':uomo_con_completo_che_levita_carnagione_scura:','fa':':پرواز_مرد_با_لباس_رسمی_پوست_آبنوسی:','id':':pria_melayang_mengenakan_jas_warna_kulit_gelap:','zh':':西装革履的人_较深肤色:','ru':':бизнесмен_в_воздухе_очень_темный_тон_кожи:','tr':':havalanan_takım_elbiseli_adam_koyu_cilt_tonu:','ar':':رجل_أعمال_يرتقي_بشرة_بلون_غامق:'},'\U0001F574\U0001F3FB':{'en':':person_in_suit_levitating_light_skin_tone:','status':fully_qualified,'E':4,'de':':schwebender_mann_im_anzug_helle_hautfarbe:','es':':persona_trajeada_levitando_tono_de_piel_claro:','fr':':homme_d’affaires_en_lévitation_peau_claire:','ja':':浮いてるビジネスマン_薄い肌色:','ko':':공중에_떠_있는_정장_입은_남자_하얀_피부:','pt':':homem_de_terno_levitando_pele_clara:','it':':uomo_con_completo_che_levita_carnagione_chiara:','fa':':پرواز_مرد_با_لباس_رسمی_پوست_سفید:','id':':pria_melayang_mengenakan_jas_warna_kulit_cerah:','zh':':西装革履的人_较浅肤色:','ru':':бизнесмен_в_воздухе_очень_светлый_тон_кожи:','tr':':havalanan_takım_elbiseli_adam_açık_cilt_tonu:','ar':':رجل_أعمال_يرتقي_بشرة_بلون_فاتح:'},'\U0001F574\U0001F3FE':{'en':':person_in_suit_levitating_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':schwebender_mann_im_anzug_mitteldunkle_hautfarbe:','es':':persona_trajeada_levitando_tono_de_piel_oscuro_medio:','fr':':homme_d’affaires_en_lévitation_peau_mate:','ja':':浮いてるビジネスマン_やや濃い肌色:','ko':':공중에_떠_있는_정장_입은_남자_진한_갈색_피부:','pt':':homem_de_terno_levitando_pele_morena_escura:','it':':uomo_con_completo_che_levita_carnagione_abbastanza_scura:','fa':':پرواز_مرد_با_لباس_رسمی_پوست_گندمی:','id':':pria_melayang_mengenakan_jas_warna_kulit_gelap-sedang:','zh':':西装革履的人_中等深肤色:','ru':':бизнесмен_в_воздухе_темный_тон_кожи:','tr':':havalanan_takım_elbiseli_adam_orta_koyu_cilt_tonu:','ar':':رجل_أعمال_يرتقي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F574\U0001F3FC':{'en':':person_in_suit_levitating_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':schwebender_mann_im_anzug_mittelhelle_hautfarbe:','es':':persona_trajeada_levitando_tono_de_piel_claro_medio:','fr':':homme_d’affaires_en_lévitation_peau_moyennement_claire:','ja':':浮いてるビジネスマン_やや薄い肌色:','ko':':공중에_떠_있는_정장_입은_남자_연한_갈색_피부:','pt':':homem_de_terno_levitando_pele_morena_clara:','it':':uomo_con_completo_che_levita_carnagione_abbastanza_chiara:','fa':':پرواز_مرد_با_لباس_رسمی_پوست_روشن:','id':':pria_melayang_mengenakan_jas_warna_kulit_cerah-sedang:','zh':':西装革履的人_中等浅肤色:','ru':':бизнесмен_в_воздухе_светлый_тон_кожи:','tr':':havalanan_takım_elbiseli_adam_orta_açık_cilt_tonu:','ar':':رجل_أعمال_يرتقي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F574\U0001F3FD':{'en':':person_in_suit_levitating_medium_skin_tone:','status':fully_qualified,'E':4,'de':':schwebender_mann_im_anzug_mittlere_hautfarbe:','es':':persona_trajeada_levitando_tono_de_piel_medio:','fr':':homme_d’affaires_en_lévitation_peau_légèrement_mate:','ja':':浮いてるビジネスマン_中間の肌色:','ko':':공중에_떠_있는_정장_입은_남자_갈색_피부:','pt':':homem_de_terno_levitando_pele_morena:','it':':uomo_con_completo_che_levita_carnagione_olivastra:','fa':':پرواز_مرد_با_لباس_رسمی_پوست_طلایی:','id':':pria_melayang_mengenakan_jas_warna_kulit_sedang:','zh':':西装革履的人_中等肤色:','ru':':бизнесмен_в_воздухе_средний_тон_кожи:','tr':':havalanan_takım_elbiseli_adam_orta_cilt_tonu:','ar':':رجل_أعمال_يرتقي_بشرة_بلون_معتدل:'},'\U0001F935':{'en':':person_in_tuxedo:','status':fully_qualified,'E':3,'de':':person_im_smoking:','es':':persona_con_esmoquin:','fr':':personne_en_smoking:','ja':':タキシードの人:','ko':':턱시도를_입은_사람:','pt':':pessoa_de_smoking:','it':':persona_in_smoking:','fa':':شخص_با_کت_و_شلوار_و_پاپیون:','id':':orang_bertuksedo:','zh':':穿燕尾服的人:','ru':':жених:','tr':':smokinli_kişi:','ar':':شخص_ببدلة_رسمية:'},'\U0001F935\U0001F3FF':{'en':':person_in_tuxedo_dark_skin_tone:','status':fully_qualified,'E':3,'de':':person_im_smoking_dunkle_hautfarbe:','es':':persona_con_esmoquin_tono_de_piel_oscuro:','fr':':personne_en_smoking_peau_foncée:','ja':':タキシードの人_濃い肌色:','ko':':턱시도를_입은_사람_검은색_피부:','pt':':pessoa_de_smoking_pele_escura:','it':':persona_in_smoking_carnagione_scura:','fa':':شخص_با_کت_و_شلوار_و_پاپیون_پوست_آبنوسی:','id':':orang_bertuksedo_warna_kulit_gelap:','zh':':穿燕尾服的人_较深肤色:','ru':':жених_очень_темный_тон_кожи:','tr':':smokinli_kişi_koyu_cilt_tonu:','ar':':شخص_ببدلة_رسمية_بشرة_بلون_غامق:'},'\U0001F935\U0001F3FB':{'en':':person_in_tuxedo_light_skin_tone:','status':fully_qualified,'E':3,'de':':person_im_smoking_helle_hautfarbe:','es':':persona_con_esmoquin_tono_de_piel_claro:','fr':':personne_en_smoking_peau_claire:','ja':':タキシードの人_薄い肌色:','ko':':턱시도를_입은_사람_하얀_피부:','pt':':pessoa_de_smoking_pele_clara:','it':':persona_in_smoking_carnagione_chiara:','fa':':شخص_با_کت_و_شلوار_و_پاپیون_پوست_سفید:','id':':orang_bertuksedo_warna_kulit_cerah:','zh':':穿燕尾服的人_较浅肤色:','ru':':жених_очень_светлый_тон_кожи:','tr':':smokinli_kişi_açık_cilt_tonu:','ar':':شخص_ببدلة_رسمية_بشرة_بلون_فاتح:'},'\U0001F935\U0001F3FE':{'en':':person_in_tuxedo_medium-dark_skin_tone:','status':fully_qualified,'E':3,'de':':person_im_smoking_mitteldunkle_hautfarbe:','es':':persona_con_esmoquin_tono_de_piel_oscuro_medio:','fr':':personne_en_smoking_peau_mate:','ja':':タキシードの人_やや濃い肌色:','ko':':턱시도를_입은_사람_진한_갈색_피부:','pt':':pessoa_de_smoking_pele_morena_escura:','it':':persona_in_smoking_carnagione_abbastanza_scura:','fa':':شخص_با_کت_و_شلوار_و_پاپیون_پوست_گندمی:','id':':orang_bertuksedo_warna_kulit_gelap-sedang:','zh':':穿燕尾服的人_中等深肤色:','ru':':жених_темный_тон_кожи:','tr':':smokinli_kişi_orta_koyu_cilt_tonu:','ar':':شخص_ببدلة_رسمية_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F935\U0001F3FC':{'en':':person_in_tuxedo_medium-light_skin_tone:','status':fully_qualified,'E':3,'de':':person_im_smoking_mittelhelle_hautfarbe:','es':':persona_con_esmoquin_tono_de_piel_claro_medio:','fr':':personne_en_smoking_peau_moyennement_claire:','ja':':タキシードの人_やや薄い肌色:','ko':':턱시도를_입은_사람_연한_갈색_피부:','pt':':pessoa_de_smoking_pele_morena_clara:','it':':persona_in_smoking_carnagione_abbastanza_chiara:','fa':':شخص_با_کت_و_شلوار_و_پاپیون_پوست_روشن:','id':':orang_bertuksedo_warna_kulit_cerah-sedang:','zh':':穿燕尾服的人_中等浅肤色:','ru':':жених_светлый_тон_кожи:','tr':':smokinli_kişi_orta_açık_cilt_tonu:','ar':':شخص_ببدلة_رسمية_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F935\U0001F3FD':{'en':':person_in_tuxedo_medium_skin_tone:','status':fully_qualified,'E':3,'de':':person_im_smoking_mittlere_hautfarbe:','es':':persona_con_esmoquin_tono_de_piel_medio:','fr':':personne_en_smoking_peau_légèrement_mate:','ja':':タキシードの人_中間の肌色:','ko':':턱시도를_입은_사람_갈색_피부:','pt':':pessoa_de_smoking_pele_morena:','it':':persona_in_smoking_carnagione_olivastra:','fa':':شخص_با_کت_و_شلوار_و_پاپیون_پوست_طلایی:','id':':orang_bertuksedo_warna_kulit_sedang:','zh':':穿燕尾服的人_中等肤色:','ru':':жених_средний_тон_кожи:','tr':':smokinli_kişi_orta_cilt_tonu:','ar':':شخص_ببدلة_رسمية_بشرة_بلون_معتدل:'},'\U0001F939':{'en':':person_juggling:','status':fully_qualified,'E':3,'alias':[':juggling_person:'],'de':':jongleur(in):','es':':persona_haciendo_malabares:','fr':':personne_qui_jongle:','ja':':ジャグリングをする人:','ko':':저글링하는_사람:','pt':':malabarista:','it':':persona_che_fa_giocoleria:','fa':':ژانگولربازی:','id':':orang_melempar:','zh':':抛接杂耍:','ru':':жонглер:','tr':':jonglör:','ar':':شخص_يؤدي_ألعابا_بهلوانية:'},'\U0001F939\U0001F3FF':{'en':':person_juggling_dark_skin_tone:','status':fully_qualified,'E':3,'de':':jongleur(in)_dunkle_hautfarbe:','es':':persona_haciendo_malabares_tono_de_piel_oscuro:','fr':':personne_qui_jongle_peau_foncée:','ja':':ジャグリングをする人_濃い肌色:','ko':':저글링하는_사람_검은색_피부:','pt':':malabarista_pele_escura:','it':':persona_che_fa_giocoleria_carnagione_scura:','fa':':ژانگولربازی_پوست_آبنوسی:','id':':orang_melempar_warna_kulit_gelap:','zh':':抛接杂耍_较深肤色:','ru':':жонглер_очень_темный_тон_кожи:','tr':':jonglör_koyu_cilt_tonu:','ar':':شخص_يؤدي_ألعابا_بهلوانية_بشرة_بلون_غامق:'},'\U0001F939\U0001F3FB':{'en':':person_juggling_light_skin_tone:','status':fully_qualified,'E':3,'de':':jongleur(in)_helle_hautfarbe:','es':':persona_haciendo_malabares_tono_de_piel_claro:','fr':':personne_qui_jongle_peau_claire:','ja':':ジャグリングをする人_薄い肌色:','ko':':저글링하는_사람_하얀_피부:','pt':':malabarista_pele_clara:','it':':persona_che_fa_giocoleria_carnagione_chiara:','fa':':ژانگولربازی_پوست_سفید:','id':':orang_melempar_warna_kulit_cerah:','zh':':抛接杂耍_较浅肤色:','ru':':жонглер_очень_светлый_тон_кожи:','tr':':jonglör_açık_cilt_tonu:','ar':':شخص_يؤدي_ألعابا_بهلوانية_بشرة_بلون_فاتح:'},'\U0001F939\U0001F3FE':{'en':':person_juggling_medium-dark_skin_tone:','status':fully_qualified,'E':3,'de':':jongleur(in)_mitteldunkle_hautfarbe:','es':':persona_haciendo_malabares_tono_de_piel_oscuro_medio:','fr':':personne_qui_jongle_peau_mate:','ja':':ジャグリングをする人_やや濃い肌色:','ko':':저글링하는_사람_진한_갈색_피부:','pt':':malabarista_pele_morena_escura:','it':':persona_che_fa_giocoleria_carnagione_abbastanza_scura:','fa':':ژانگولربازی_پوست_گندمی:','id':':orang_melempar_warna_kulit_gelap-sedang:','zh':':抛接杂耍_中等深肤色:','ru':':жонглер_темный_тон_кожи:','tr':':jonglör_orta_koyu_cilt_tonu:','ar':':شخص_يؤدي_ألعابا_بهلوانية_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F939\U0001F3FC':{'en':':person_juggling_medium-light_skin_tone:','status':fully_qualified,'E':3,'de':':jongleur(in)_mittelhelle_hautfarbe:','es':':persona_haciendo_malabares_tono_de_piel_claro_medio:','fr':':personne_qui_jongle_peau_moyennement_claire:','ja':':ジャグリングをする人_やや薄い肌色:','ko':':저글링하는_사람_연한_갈색_피부:','pt':':malabarista_pele_morena_clara:','it':':persona_che_fa_giocoleria_carnagione_abbastanza_chiara:','fa':':ژانگولربازی_پوست_روشن:','id':':orang_melempar_warna_kulit_cerah-sedang:','zh':':抛接杂耍_中等浅肤色:','ru':':жонглер_светлый_тон_кожи:','tr':':jonglör_orta_açık_cilt_tonu:','ar':':شخص_يؤدي_ألعابا_بهلوانية_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F939\U0001F3FD':{'en':':person_juggling_medium_skin_tone:','status':fully_qualified,'E':3,'de':':jongleur(in)_mittlere_hautfarbe:','es':':persona_haciendo_malabares_tono_de_piel_medio:','fr':':personne_qui_jongle_peau_légèrement_mate:','ja':':ジャグリングをする人_中間の肌色:','ko':':저글링하는_사람_갈색_피부:','pt':':malabarista_pele_morena:','it':':persona_che_fa_giocoleria_carnagione_olivastra:','fa':':ژانگولربازی_پوست_طلایی:','id':':orang_melempar_warna_kulit_sedang:','zh':':抛接杂耍_中等肤色:','ru':':жонглер_средний_тон_кожи:','tr':':jonglör_orta_cilt_tonu:','ar':':شخص_يؤدي_ألعابا_بهلوانية_بشرة_بلون_معتدل:'},'\U0001F9CE':{'en':':person_kneeling:','status':fully_qualified,'E':12,'alias':[':kneeling_person:'],'de':':kniende_person:','es':':persona_de_rodillas:','fr':':personne_à_genoux:','ja':':正座する人:','ko':':무릎을_꿇은_사람:','pt':':pessoa_ajoelhando:','it':':persona_inginocchiata:','fa':':آدم_زانوزده:','id':':orang_berlutut:','zh':':下跪者:','ru':':человек_на_коленях:','tr':':diz_çöken_kişi:','ar':':شخص_جالس_على_ركبتيه:'},'\U0001F9CE\U0001F3FF':{'en':':person_kneeling_dark_skin_tone:','status':fully_qualified,'E':12,'de':':kniende_person_dunkle_hautfarbe:','es':':persona_de_rodillas_tono_de_piel_oscuro:','fr':':personne_à_genoux_peau_foncée:','ja':':正座する人_濃い肌色:','ko':':무릎을_꿇은_사람_검은색_피부:','pt':':pessoa_ajoelhando_pele_escura:','it':':persona_inginocchiata_carnagione_scura:','fa':':آدم_زانوزده_پوست_آبنوسی:','id':':orang_berlutut_warna_kulit_gelap:','zh':':下跪者_较深肤色:','ru':':человек_на_коленях_очень_темный_тон_кожи:','tr':':diz_çöken_kişi_koyu_cilt_tonu:','ar':':شخص_جالس_على_ركبتيه_بشرة_بلون_غامق:'},'\U0001F9CE\U0000200D\U000027A1\U0000FE0F':{'en':':person_kneeling_facing_right:','status':fully_qualified,'E':15.1,'de':':kniende_person_nach_rechts:','es':':persona_de_rodillas_hacia_la_derecha:','fr':':personne_à_genoux_vers_la_droite:','ja':':正座する人_右向き:','ko':':오른쪽을_향한_무릎을_꿇은_사람:','pt':':pessoa_ajoelhando_de_frente_para_a_direita:','it':':persona_inginocchiata_a_destra:','fa':':آدم_زانوزده_به_سمت_راست:','id':':orang_berlutut_menghadap_kanan:','zh':':下跪者面向右边:','ru':':человек_на_коленях_направленные_вправо:','tr':':sağa_dönük_diz_çöken_kişi:','ar':':شخص_جالس_على_ركبتيه_متجه_لليمين:'},'\U0001F9CE\U0000200D\U000027A1':{'en':':person_kneeling_facing_right:','status':minimally_qualified,'E':15.1,'de':':kniende_person_nach_rechts:','es':':persona_de_rodillas_hacia_la_derecha:','fr':':personne_à_genoux_vers_la_droite:','ja':':正座する人_右向き:','ko':':오른쪽을_향한_무릎을_꿇은_사람:','pt':':pessoa_ajoelhando_de_frente_para_a_direita:','it':':persona_inginocchiata_a_destra:','fa':':آدم_زانوزده_به_سمت_راست:','id':':orang_berlutut_menghadap_kanan:','zh':':下跪者面向右边:','ru':':человек_на_коленях_направленные_вправо:','tr':':sağa_dönük_diz_çöken_kişi:','ar':':شخص_جالس_على_ركبتيه_متجه_لليمين:'},'\U0001F9CE\U0001F3FF\U0000200D\U000027A1\U0000FE0F':{'en':':person_kneeling_facing_right_dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':kniende_person_nach_rechts_dunkle_hautfarbe:','es':':persona_de_rodillas_hacia_la_derecha_tono_de_piel_oscuro:','fr':':personne_à_genoux_vers_la_droite_peau_foncée:','ja':':正座する人_右向き_濃い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_사람_검은색_피부:','pt':':pessoa_ajoelhando_de_frente_para_a_direita_pele_escura:','it':':persona_inginocchiata_a_destra_carnagione_scura:','fa':':آدم_زانوزده_به_سمت_راستپوست_آبنوسی:','id':':orang_berlutut_menghadap_kanan_warna_kulit_gelap:','zh':':下跪者面向右边_较深肤色:','ru':':человек_на_коленях_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_diz_çöken_kişi_koyu_cilt_tonu:','ar':':شخص_جالس_على_ركبتيه_متجه_لليمين_بلون_غامق:'},'\U0001F9CE\U0001F3FF\U0000200D\U000027A1':{'en':':person_kneeling_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniende_person_nach_rechts_dunkle_hautfarbe:','es':':persona_de_rodillas_hacia_la_derecha_tono_de_piel_oscuro:','fr':':personne_à_genoux_vers_la_droite_peau_foncée:','ja':':正座する人_右向き_濃い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_사람_검은색_피부:','pt':':pessoa_ajoelhando_de_frente_para_a_direita_pele_escura:','it':':persona_inginocchiata_a_destra_carnagione_scura:','fa':':آدم_زانوزده_به_سمت_راستپوست_آبنوسی:','id':':orang_berlutut_menghadap_kanan_warna_kulit_gelap:','zh':':下跪者面向右边_较深肤色:','ru':':человек_на_коленях_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_diz_çöken_kişi_koyu_cilt_tonu:','ar':':شخص_جالس_على_ركبتيه_متجه_لليمين_بلون_غامق:'},'\U0001F9CE\U0001F3FB\U0000200D\U000027A1\U0000FE0F':{'en':':person_kneeling_facing_right_light_skin_tone:','status':fully_qualified,'E':15.1,'de':':kniende_person_nach_rechts_helle_hautfarbe:','es':':persona_de_rodillas_hacia_la_derecha_tono_de_piel_claro:','fr':':personne_à_genoux_vers_la_droite_peau_claire:','ja':':正座する人_右向き_薄い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_사람_하얀_피부:','pt':':pessoa_ajoelhando_de_frente_para_a_direita_pele_clara:','it':':persona_inginocchiata_a_destra_carnagione_chiara:','fa':':آدم_زانوزده_به_سمت_راست_پوست_سفید:','id':':orang_berlutut_menghadap_kanan_warna_kulit_cerah:','zh':':下跪者面向右边_较浅肤色:','ru':':человек_на_коленях_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_diz_çöken_kişi_açık_cilt_tonu:','ar':':شخص_جالس_على_ركبتيه_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F9CE\U0001F3FB\U0000200D\U000027A1':{'en':':person_kneeling_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniende_person_nach_rechts_helle_hautfarbe:','es':':persona_de_rodillas_hacia_la_derecha_tono_de_piel_claro:','fr':':personne_à_genoux_vers_la_droite_peau_claire:','ja':':正座する人_右向き_薄い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_사람_하얀_피부:','pt':':pessoa_ajoelhando_de_frente_para_a_direita_pele_clara:','it':':persona_inginocchiata_a_destra_carnagione_chiara:','fa':':آدم_زانوزده_به_سمت_راست_پوست_سفید:','id':':orang_berlutut_menghadap_kanan_warna_kulit_cerah:','zh':':下跪者面向右边_较浅肤色:','ru':':человек_на_коленях_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_diz_çöken_kişi_açık_cilt_tonu:','ar':':شخص_جالس_على_ركبتيه_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F9CE\U0001F3FE\U0000200D\U000027A1\U0000FE0F':{'en':':person_kneeling_facing_right_medium-dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':kniende_person_nach_rechts_mitteldunkle_hautfarbe:','es':':persona_de_rodillas_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':personne_à_genoux_vers_la_droite_peau_mate:','ja':':正座する人_右向き_やや濃い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_사람_진한_갈색_피부:','pt':':pessoa_ajoelhando_de_frente_para_a_direita_pele_morena_escura:','it':':persona_inginocchiata_a_destra_carnagione_abbastanza_scura:','fa':':آدم_زانوزده_به_سمت_راست_پوست_گندمی:','id':':orang_berlutut_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':下跪者面向右边_中等深肤色:','ru':':человек_на_коленях_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_diz_çöken_kişi_orta_koyu_cilt_tonu:','ar':':شخص_جالس_على_ركبتيه_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F9CE\U0001F3FE\U0000200D\U000027A1':{'en':':person_kneeling_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniende_person_nach_rechts_mitteldunkle_hautfarbe:','es':':persona_de_rodillas_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':personne_à_genoux_vers_la_droite_peau_mate:','ja':':正座する人_右向き_やや濃い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_사람_진한_갈색_피부:','pt':':pessoa_ajoelhando_de_frente_para_a_direita_pele_morena_escura:','it':':persona_inginocchiata_a_destra_carnagione_abbastanza_scura:','fa':':آدم_زانوزده_به_سمت_راست_پوست_گندمی:','id':':orang_berlutut_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':下跪者面向右边_中等深肤色:','ru':':человек_на_коленях_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_diz_çöken_kişi_orta_koyu_cilt_tonu:','ar':':شخص_جالس_على_ركبتيه_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F9CE\U0001F3FC\U0000200D\U000027A1\U0000FE0F':{'en':':person_kneeling_facing_right_medium-light_skin_tone:','status':fully_qualified,'E':15.1,'de':':kniende_person_nach_rechts_mittelhelle_hautfarbe:','es':':persona_de_rodillas_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':personne_à_genoux_vers_la_droite_peau_moyennement_claire:','ja':':正座する人_右向き_やや薄い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_사람_연한_갈색_피부:','pt':':pessoa_ajoelhando_de_frente_para_a_direita_pele_morena_clara:','it':':persona_inginocchiata_a_destra_carnagione_abbastanza_chiara:','fa':':آدم_زانوزده_به_سمت_راست_پوست_روشن:','id':':orang_berlutut_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':下跪者面向右边_中等浅肤色:','ru':':человек_на_коленях_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_diz_çöken_kişi_orta_açık_cilt_tonu:','ar':':شخص_جالس_على_ركبتيه_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F9CE\U0001F3FC\U0000200D\U000027A1':{'en':':person_kneeling_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniende_person_nach_rechts_mittelhelle_hautfarbe:','es':':persona_de_rodillas_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':personne_à_genoux_vers_la_droite_peau_moyennement_claire:','ja':':正座する人_右向き_やや薄い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_사람_연한_갈색_피부:','pt':':pessoa_ajoelhando_de_frente_para_a_direita_pele_morena_clara:','it':':persona_inginocchiata_a_destra_carnagione_abbastanza_chiara:','fa':':آدم_زانوزده_به_سمت_راست_پوست_روشن:','id':':orang_berlutut_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':下跪者面向右边_中等浅肤色:','ru':':человек_на_коленях_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_diz_çöken_kişi_orta_açık_cilt_tonu:','ar':':شخص_جالس_على_ركبتيه_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F9CE\U0001F3FD\U0000200D\U000027A1\U0000FE0F':{'en':':person_kneeling_facing_right_medium_skin_tone:','status':fully_qualified,'E':15.1,'de':':kniende_person_nach_rechts_mittlere_hautfarbe:','es':':persona_de_rodillas_hacia_la_derecha_tono_de_piel_medio:','fr':':personne_à_genoux_vers_la_droite_peau_légèrement_mate:','ja':':正座する人_右向き_中間の肌色:','ko':':오른쪽을_향한_무릎을_꿇은_사람_갈색_피부:','pt':':pessoa_ajoelhando_de_frente_para_a_direita_pele_morena:','it':':persona_inginocchiata_a_destra_carnagione_olivastra:','fa':':آدم_زانوزده_به_سمت_راست_پوست_طلایی:','id':':orang_berlutut_menghadap_kanan_warna_kulit_sedang:','zh':':下跪者面向右边_中等肤色:','ru':':человек_на_коленях_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_diz_çöken_kişi_orta_cilt_tonu:','ar':':شخص_جالس_على_ركبتيه_متجه_لليمين_بلون_معتدل:'},'\U0001F9CE\U0001F3FD\U0000200D\U000027A1':{'en':':person_kneeling_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniende_person_nach_rechts_mittlere_hautfarbe:','es':':persona_de_rodillas_hacia_la_derecha_tono_de_piel_medio:','fr':':personne_à_genoux_vers_la_droite_peau_légèrement_mate:','ja':':正座する人_右向き_中間の肌色:','ko':':오른쪽을_향한_무릎을_꿇은_사람_갈색_피부:','pt':':pessoa_ajoelhando_de_frente_para_a_direita_pele_morena:','it':':persona_inginocchiata_a_destra_carnagione_olivastra:','fa':':آدم_زانوزده_به_سمت_راست_پوست_طلایی:','id':':orang_berlutut_menghadap_kanan_warna_kulit_sedang:','zh':':下跪者面向右边_中等肤色:','ru':':человек_на_коленях_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_diz_çöken_kişi_orta_cilt_tonu:','ar':':شخص_جالس_على_ركبتيه_متجه_لليمين_بلون_معتدل:'},'\U0001F9CE\U0001F3FB':{'en':':person_kneeling_light_skin_tone:','status':fully_qualified,'E':12,'de':':kniende_person_helle_hautfarbe:','es':':persona_de_rodillas_tono_de_piel_claro:','fr':':personne_à_genoux_peau_claire:','ja':':正座する人_薄い肌色:','ko':':무릎을_꿇은_사람_하얀_피부:','pt':':pessoa_ajoelhando_pele_clara:','it':':persona_inginocchiata_carnagione_chiara:','fa':':آدم_زانوزده_پوست_سفید:','id':':orang_berlutut_warna_kulit_cerah:','zh':':下跪者_较浅肤色:','ru':':человек_на_коленях_очень_светлый_тон_кожи:','tr':':diz_çöken_kişi_açık_cilt_tonu:','ar':':شخص_جالس_على_ركبتيه_بشرة_بلون_فاتح:'},'\U0001F9CE\U0001F3FE':{'en':':person_kneeling_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':kniende_person_mitteldunkle_hautfarbe:','es':':persona_de_rodillas_tono_de_piel_oscuro_medio:','fr':':personne_à_genoux_peau_mate:','ja':':正座する人_やや濃い肌色:','ko':':무릎을_꿇은_사람_진한_갈색_피부:','pt':':pessoa_ajoelhando_pele_morena_escura:','it':':persona_inginocchiata_carnagione_abbastanza_scura:','fa':':آدم_زانوزده_پوست_گندمی:','id':':orang_berlutut_warna_kulit_gelap-sedang:','zh':':下跪者_中等深肤色:','ru':':человек_на_коленях_темный_тон_кожи:','tr':':diz_çöken_kişi_orta_koyu_cilt_tonu:','ar':':شخص_جالس_على_ركبتيه_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9CE\U0001F3FC':{'en':':person_kneeling_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':kniende_person_mittelhelle_hautfarbe:','es':':persona_de_rodillas_tono_de_piel_claro_medio:','fr':':personne_à_genoux_peau_moyennement_claire:','ja':':正座する人_やや薄い肌色:','ko':':무릎을_꿇은_사람_연한_갈색_피부:','pt':':pessoa_ajoelhando_pele_morena_clara:','it':':persona_inginocchiata_carnagione_abbastanza_chiara:','fa':':آدم_زانوزده_پوست_روشن:','id':':orang_berlutut_warna_kulit_cerah-sedang:','zh':':下跪者_中等浅肤色:','ru':':человек_на_коленях_светлый_тон_кожи:','tr':':diz_çöken_kişi_orta_açık_cilt_tonu:','ar':':شخص_جالس_على_ركبتيه_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9CE\U0001F3FD':{'en':':person_kneeling_medium_skin_tone:','status':fully_qualified,'E':12,'de':':kniende_person_mittlere_hautfarbe:','es':':persona_de_rodillas_tono_de_piel_medio:','fr':':personne_à_genoux_peau_légèrement_mate:','ja':':正座する人_中間の肌色:','ko':':무릎을_꿇은_사람_갈색_피부:','pt':':pessoa_ajoelhando_pele_morena:','it':':persona_inginocchiata_carnagione_olivastra:','fa':':آدم_زانوزده_پوست_طلایی:','id':':orang_berlutut_warna_kulit_sedang:','zh':':下跪者_中等肤色:','ru':':человек_на_коленях_средний_тон_кожи:','tr':':diz_çöken_kişi_orta_cilt_tonu:','ar':':شخص_جالس_على_ركبتيه_بشرة_بلون_معتدل:'},'\U0001F3CB\U0000FE0F':{'en':':person_lifting_weights:','status':fully_qualified,'E':0.7,'alias':[':weight_lifting:', ':weight_lifter:'],'variant':True,'de':':gewichtheber(in):','es':':persona_levantando_pesas:','fr':':haltérophile:','ja':':重量挙げをする人:','ko':':역도_선수:','pt':':pessoa_levantando_peso:','it':':persona_che_solleva_pesi:','fa':':وزنه_بردار:','id':':orang_mengangkat_beban:','zh':':举重:','ru':':тяжелоатлет:','tr':':ağırlık_kaldıran_kişi:','ar':':شخص_يرفع_أثقال:'},'\U0001F3CB':{'en':':person_lifting_weights:','status':unqualified,'E':0.7,'alias':[':weight_lifting:', ':weight_lifter:'],'variant':True,'de':':gewichtheber(in):','es':':persona_levantando_pesas:','fr':':haltérophile:','ja':':重量挙げをする人:','ko':':역도_선수:','pt':':pessoa_levantando_peso:','it':':persona_che_solleva_pesi:','fa':':وزنه_بردار:','id':':orang_mengangkat_beban:','zh':':举重:','ru':':тяжелоатлет:','tr':':ağırlık_kaldıran_kişi:','ar':':شخص_يرفع_أثقال:'},'\U0001F3CB\U0001F3FF':{'en':':person_lifting_weights_dark_skin_tone:','status':fully_qualified,'E':2,'de':':gewichtheber(in)_dunkle_hautfarbe:','es':':persona_levantando_pesas_tono_de_piel_oscuro:','fr':':haltérophile_peau_foncée:','ja':':重量挙げをする人_濃い肌色:','ko':':역도_선수_검은색_피부:','pt':':pessoa_levantando_peso_pele_escura:','it':':persona_che_solleva_pesi_carnagione_scura:','fa':':وزنه_بردار_پوست_آبنوسی:','id':':orang_mengangkat_beban_warna_kulit_gelap:','zh':':举重_较深肤色:','ru':':тяжелоатлет_очень_темный_тон_кожи:','tr':':ağırlık_kaldıran_kişi_koyu_cilt_tonu:','ar':':شخص_يرفع_أثقال_بشرة_بلون_غامق:'},'\U0001F3CB\U0001F3FB':{'en':':person_lifting_weights_light_skin_tone:','status':fully_qualified,'E':2,'de':':gewichtheber(in)_helle_hautfarbe:','es':':persona_levantando_pesas_tono_de_piel_claro:','fr':':haltérophile_peau_claire:','ja':':重量挙げをする人_薄い肌色:','ko':':역도_선수_하얀_피부:','pt':':pessoa_levantando_peso_pele_clara:','it':':persona_che_solleva_pesi_carnagione_chiara:','fa':':وزنه_بردار_پوست_سفید:','id':':orang_mengangkat_beban_warna_kulit_cerah:','zh':':举重_较浅肤色:','ru':':тяжелоатлет_очень_светлый_тон_кожи:','tr':':ağırlık_kaldıran_kişi_açık_cilt_tonu:','ar':':شخص_يرفع_أثقال_بشرة_بلون_فاتح:'},'\U0001F3CB\U0001F3FE':{'en':':person_lifting_weights_medium-dark_skin_tone:','status':fully_qualified,'E':2,'de':':gewichtheber(in)_mitteldunkle_hautfarbe:','es':':persona_levantando_pesas_tono_de_piel_oscuro_medio:','fr':':haltérophile_peau_mate:','ja':':重量挙げをする人_やや濃い肌色:','ko':':역도_선수_진한_갈색_피부:','pt':':pessoa_levantando_peso_pele_morena_escura:','it':':persona_che_solleva_pesi_carnagione_abbastanza_scura:','fa':':وزنه_بردار_پوست_گندمی:','id':':orang_mengangkat_beban_warna_kulit_gelap-sedang:','zh':':举重_中等深肤色:','ru':':тяжелоатлет_темный_тон_кожи:','tr':':ağırlık_kaldıran_kişi_orta_koyu_cilt_tonu:','ar':':شخص_يرفع_أثقال_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3CB\U0001F3FC':{'en':':person_lifting_weights_medium-light_skin_tone:','status':fully_qualified,'E':2,'de':':gewichtheber(in)_mittelhelle_hautfarbe:','es':':persona_levantando_pesas_tono_de_piel_claro_medio:','fr':':haltérophile_peau_moyennement_claire:','ja':':重量挙げをする人_やや薄い肌色:','ko':':역도_선수_연한_갈색_피부:','pt':':pessoa_levantando_peso_pele_morena_clara:','it':':persona_che_solleva_pesi_carnagione_abbastanza_chiara:','fa':':وزنه_بردار_پوست_روشن:','id':':orang_mengangkat_beban_warna_kulit_cerah-sedang:','zh':':举重_中等浅肤色:','ru':':тяжелоатлет_светлый_тон_кожи:','tr':':ağırlık_kaldıran_kişi_orta_açık_cilt_tonu:','ar':':شخص_يرفع_أثقال_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3CB\U0001F3FD':{'en':':person_lifting_weights_medium_skin_tone:','status':fully_qualified,'E':2,'de':':gewichtheber(in)_mittlere_hautfarbe:','es':':persona_levantando_pesas_tono_de_piel_medio:','fr':':haltérophile_peau_légèrement_mate:','ja':':重量挙げをする人_中間の肌色:','ko':':역도_선수_갈색_피부:','pt':':pessoa_levantando_peso_pele_morena:','it':':persona_che_solleva_pesi_carnagione_olivastra:','fa':':وزنه_بردار_پوست_طلایی:','id':':orang_mengangkat_beban_warna_kulit_sedang:','zh':':举重_中等肤色:','ru':':тяжелоатлет_средний_тон_кожи:','tr':':ağırlık_kaldıran_kişi_orta_cilt_tonu:','ar':':شخص_يرفع_أثقال_بشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FB':{'en':':person_light_skin_tone:','status':fully_qualified,'E':5,'de':':person_helle_hautfarbe:','es':':persona_adulta_tono_de_piel_claro:','fr':':adulte_peau_claire:','ja':':大人_薄い肌色:','ko':':사람_하얀_피부:','pt':':pessoa_pele_clara:','it':':persona_carnagione_chiara:','fa':':شخص_پوست_سفید:','id':':dewasa_warna_kulit_cerah:','zh':':成人_较浅肤色:','ru':':взрослый_очень_светлый_тон_кожи:','tr':':yetişkin_açık_cilt_tonu:','ar':':بالغ_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F9B2':{'en':':person_light_skin_tone_bald:','status':fully_qualified,'E':12.1,'de':':person_helle_hautfarbe_glatze:','es':':persona_adulta_tono_de_piel_claro_y_sin_pelo:','fr':':adulte_peau_claire_et_chauve:','ja':':大人_薄い肌色_はげ頭:','ko':':사람_하얀_피부_대머리:','pt':':pessoa_pele_clara_e_careca:','it':':persona_carnagione_chiara_e_calvo:','fa':':شخص_پوست_سفید_تاس:','id':':dewasa_warna_kulit_cerah_gundul:','zh':':成人_较浅肤色秃顶:','ru':':взрослый_очень_светлый_тон_кожи_без_волос:','tr':':yetişkin_açık_cilt_tonu_kel:','ar':':بالغ_بشرة_بلون_فاتح_وأصلع:'},'\U0001F9D4\U0001F3FB':{'en':':person_light_skin_tone_beard:','status':fully_qualified,'E':5,'de':':person_helle_hautfarbe_bart:','es':':persona_con_barba_tono_de_piel_claro:','fr':':personne_barbue_peau_claire:','ja':':あごひげの人_薄い肌色:','ko':':수염_난_사람_하얀_피부:','pt':':homem_pele_clara_e_barba:','it':':persona_con_la_barba_carnagione_chiara:','fa':':مرد_پوست_سفید_ریش_دار:','id':':orang_warna_kulit_cerah_janggut:','zh':':有胡子的人_较浅肤色:','ru':':бородатый_человек_очень_светлый_тон_кожи:','tr':':sakallı_kişi_açık_cilt_tonu:','ar':':شخص_بلحية_بشرة_بلون_فاتح:'},'\U0001F471\U0001F3FB':{'en':':person_light_skin_tone_blond_hair:','status':fully_qualified,'E':1,'de':':person_helle_hautfarbe_blondes_haar:','es':':persona_adulta_rubia_tono_de_piel_claro:','fr':':personne_blonde_peau_claire:','ja':':金髪の人_薄い肌色:','ko':':금발인_사람_하얀_피부:','pt':':pessoa_pele_clara_e_cabelo_louro:','it':':persona_bionda_carnagione_chiara:','fa':':آدم_پوست_سفید_موبلوند:','id':':orang_warna_kulit_cerah_rambut_pirang:','zh':':金色头发的人_较浅肤色:','ru':':человек_со_светлыми_волосами_очень_светлый_тон_кожи:','tr':':sarışın_kişi_açık_cilt_tonu:','ar':':شخص_أشقر_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F9B1':{'en':':person_light_skin_tone_curly_hair:','status':fully_qualified,'E':12.1,'de':':person_helle_hautfarbe_lockiges_haar:','es':':persona_adulta_tono_de_piel_claro_y_pelo_rizado:','fr':':adulte_peau_claire_et_cheveux_bouclés:','ja':':大人_薄い肌色_巻き毛:','ko':':사람_하얀_피부_곱슬머리:','pt':':pessoa_pele_clara_e_cabelo_cacheado:','it':':persona_carnagione_chiara_e_capelli_ricci:','fa':':شخص_پوست_سفید_موفرفری:','id':':dewasa_warna_kulit_cerah_rambut_keriting:','zh':':成人_较浅肤色卷发:','ru':':взрослый_очень_светлый_тон_кожи_кудрявые_волосы:','tr':':yetişkin_açık_cilt_tonu_kıvırcık_saç:','ar':':بالغ_بشرة_بلون_فاتح_وشعر_مجعد:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F9B0':{'en':':person_light_skin_tone_red_hair:','status':fully_qualified,'E':12.1,'de':':person_helle_hautfarbe_rotes_haar:','es':':persona_adulta_tono_de_piel_claro_y_pelo_pelirrojo:','fr':':adulte_peau_claire_et_cheveux_roux:','ja':':大人_薄い肌色_赤毛:','ko':':사람_하얀_피부_빨간_머리:','pt':':pessoa_pele_clara_e_cabelo_vermelho:','it':':persona_carnagione_chiara_e_capelli_rossi:','fa':':شخص_پوست_سفید_موقرمز:','id':':dewasa_warna_kulit_cerah_rambut_merah:','zh':':成人_较浅肤色红发:','ru':':взрослый_очень_светлый_тон_кожи_рыжие_волосы:','tr':':yetişkin_açık_cilt_tonu_kızıl_saç:','ar':':بالغ_بشرة_بلون_فاتح_وشعر_أحمر:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F9B3':{'en':':person_light_skin_tone_white_hair:','status':fully_qualified,'E':12.1,'de':':person_helle_hautfarbe_weißes_haar:','es':':persona_adulta_tono_de_piel_claro_y_pelo_blanco:','fr':':adulte_peau_claire_et_cheveux_blancs:','ja':':大人_薄い肌色_白髪:','ko':':사람_하얀_피부_백발:','pt':':pessoa_pele_clara_e_cabelo_branco:','it':':persona_carnagione_chiara_e_capelli_bianchi:','fa':':شخص_پوست_سفید_موسفید:','id':':dewasa_warna_kulit_cerah_rambut_putih:','zh':':成人_较浅肤色白发:','ru':':взрослый_очень_светлый_тон_кожи_седые_волосы:','tr':':yetişkin_açık_cilt_tonu_beyaz_saç:','ar':':بالغ_بشرة_بلون_فاتح_وشعر_أبيض:'},'\U0001F9D1\U0001F3FE':{'en':':person_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':person_mitteldunkle_hautfarbe:','es':':persona_adulta_tono_de_piel_oscuro_medio:','fr':':adulte_peau_mate:','ja':':大人_やや濃い肌色:','ko':':사람_진한_갈색_피부:','pt':':pessoa_pele_morena_escura:','it':':persona_carnagione_abbastanza_scura:','fa':':شخص_پوست_گندمی:','id':':dewasa_warna_kulit_gelap-sedang:','zh':':成人_中等深肤色:','ru':':взрослый_темный_тон_кожи:','tr':':yetişkin_orta_koyu_cilt_tonu:','ar':':بالغ_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F9B2':{'en':':person_medium-dark_skin_tone_bald:','status':fully_qualified,'E':12.1,'de':':person_mitteldunkle_hautfarbe_glatze:','es':':persona_adulta_tono_de_piel_oscuro_medio_y_sin_pelo:','fr':':adulte_peau_mate_et_chauve:','ja':':大人_やや濃い肌色_はげ頭:','ko':':사람_진한_갈색_피부_대머리:','pt':':pessoa_pele_morena_escura_e_careca:','it':':persona_carnagione_abbastanza_scura_e_calvo:','fa':':شخص_پوست_گندمی_تاس:','id':':dewasa_warna_kulit_gelap-sedang_gundul:','zh':':成人_中等深肤色秃顶:','ru':':взрослый_темный_тон_кожи_без_волос:','tr':':yetişkin_orta_koyu_cilt_tonu_kel:','ar':':بالغ_بشرة_بلون_معتدل_مائل_للغامق_وأصلع:'},'\U0001F9D4\U0001F3FE':{'en':':person_medium-dark_skin_tone_beard:','status':fully_qualified,'E':5,'de':':person_mitteldunkle_hautfarbe_bart:','es':':persona_con_barba_tono_de_piel_oscuro_medio:','fr':':personne_barbue_peau_mate:','ja':':あごひげの人_やや濃い肌色:','ko':':수염_난_사람_진한_갈색_피부:','pt':':homem_pele_morena_escura_e_barba:','it':':persona_con_la_barba_carnagione_abbastanza_scura:','fa':':مرد_پوست_گندمی_ریش_دار:','id':':orang_warna_kulit_gelap-sedang_janggut:','zh':':有胡子的人_中等深肤色:','ru':':бородатый_человек_темный_тон_кожи:','tr':':sakallı_kişi_orta_koyu_cilt_tonu:','ar':':شخص_بلحية_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F471\U0001F3FE':{'en':':person_medium-dark_skin_tone_blond_hair:','status':fully_qualified,'E':1,'de':':person_mitteldunkle_hautfarbe_blondes_haar:','es':':persona_adulta_rubia_tono_de_piel_oscuro_medio:','fr':':personne_blonde_peau_mate:','ja':':金髪の人_やや濃い肌色:','ko':':금발인_사람_진한_갈색_피부:','pt':':pessoa_pele_morena_escura_e_cabelo_louro:','it':':persona_bionda_carnagione_abbastanza_scura:','fa':':آدم_پوست_گندمی_موبلوند:','id':':orang_warna_kulit_gelap-sedang_rambut_pirang:','zh':':金色头发的人_中等深肤色:','ru':':человек_со_светлыми_волосами_темный_тон_кожи:','tr':':sarışın_kişi_orta_koyu_cilt_tonu:','ar':':شخص_أشقر_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F9B1':{'en':':person_medium-dark_skin_tone_curly_hair:','status':fully_qualified,'E':12.1,'de':':person_mitteldunkle_hautfarbe_lockiges_haar:','es':':persona_adulta_tono_de_piel_oscuro_medio_y_pelo_rizado:','fr':':adulte_peau_mate_et_cheveux_bouclés:','ja':':大人_やや濃い肌色_巻き毛:','ko':':사람_진한_갈색_피부_곱슬머리:','pt':':pessoa_pele_morena_escura_e_cabelo_cacheado:','it':':persona_carnagione_abbastanza_scura_e_capelli_ricci:','fa':':شخص_پوست_گندمی_موفرفری:','id':':dewasa_warna_kulit_gelap-sedang_rambut_keriting:','zh':':成人_中等深肤色卷发:','ru':':взрослый_темный_тон_кожи_кудрявые_волосы:','tr':':yetişkin_orta_koyu_cilt_tonu_kıvırcık_saç:','ar':':بالغ_بشرة_بلون_معتدل_مائل_للغامق_وشعر_مجعد:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F9B0':{'en':':person_medium-dark_skin_tone_red_hair:','status':fully_qualified,'E':12.1,'de':':person_mitteldunkle_hautfarbe_rotes_haar:','es':':persona_adulta_tono_de_piel_oscuro_medio_y_pelo_pelirrojo:','fr':':adulte_peau_mate_et_cheveux_roux:','ja':':大人_やや濃い肌色_赤毛:','ko':':사람_진한_갈색_피부_빨간_머리:','pt':':pessoa_pele_morena_escura_e_cabelo_vermelho:','it':':persona_carnagione_abbastanza_scura_e_capelli_rossi:','fa':':شخص_پوست_گندمی_موقرمز:','id':':dewasa_warna_kulit_gelap-sedang_rambut_merah:','zh':':成人_中等深肤色红发:','ru':':взрослый_темный_тон_кожи_рыжие_волосы:','tr':':yetişkin_orta_koyu_cilt_tonu_kızıl_saç:','ar':':بالغ_بشرة_بلون_معتدل_مائل_للغامق_وشعر_أحمر:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F9B3':{'en':':person_medium-dark_skin_tone_white_hair:','status':fully_qualified,'E':12.1,'de':':person_mitteldunkle_hautfarbe_weißes_haar:','es':':persona_adulta_tono_de_piel_oscuro_medio_y_pelo_blanco:','fr':':adulte_peau_mate_et_cheveux_blancs:','ja':':大人_やや濃い肌色_白髪:','ko':':사람_진한_갈색_피부_백발:','pt':':pessoa_pele_morena_escura_e_cabelo_branco:','it':':persona_carnagione_abbastanza_scura_e_capelli_bianchi:','fa':':شخص_پوست_گندمی_موسفید:','id':':dewasa_warna_kulit_gelap-sedang_rambut_putih:','zh':':成人_中等深肤色白发:','ru':':взрослый_темный_тон_кожи_седые_волосы:','tr':':yetişkin_orta_koyu_cilt_tonu_beyaz_saç:','ar':':بالغ_بشرة_بلون_معتدل_مائل_للغامق_وشعر_أبيض:'},'\U0001F9D1\U0001F3FC':{'en':':person_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':person_mittelhelle_hautfarbe:','es':':persona_adulta_tono_de_piel_claro_medio:','fr':':adulte_peau_moyennement_claire:','ja':':大人_やや薄い肌色:','ko':':사람_연한_갈색_피부:','pt':':pessoa_pele_morena_clara:','it':':persona_carnagione_abbastanza_chiara:','fa':':شخص_پوست_روشن:','id':':dewasa_warna_kulit_cerah-sedang:','zh':':成人_中等浅肤色:','ru':':взрослый_светлый_тон_кожи:','tr':':yetişkin_orta_açık_cilt_tonu:','ar':':بالغ_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F9B2':{'en':':person_medium-light_skin_tone_bald:','status':fully_qualified,'E':12.1,'de':':person_mittelhelle_hautfarbe_glatze:','es':':persona_adulta_tono_de_piel_claro_medio_y_sin_pelo:','fr':':adulte_peau_moyennement_claire_et_chauve:','ja':':大人_やや薄い肌色_はげ頭:','ko':':사람_연한_갈색_피부_대머리:','pt':':pessoa_pele_morena_clara_e_careca:','it':':persona_carnagione_abbastanza_chiara_e_calvo:','fa':':شخص_پوست_روشن_تاس:','id':':dewasa_warna_kulit_cerah-sedang_gundul:','zh':':成人_中等浅肤色秃顶:','ru':':взрослый_светлый_тон_кожи_без_волос:','tr':':yetişkin_orta_açık_cilt_tonu_kel:','ar':':بالغ_بشرة_بلون_فاتح_ومعتدل_وأصلع:'},'\U0001F9D4\U0001F3FC':{'en':':person_medium-light_skin_tone_beard:','status':fully_qualified,'E':5,'de':':person_mittelhelle_hautfarbe_bart:','es':':persona_con_barba_tono_de_piel_claro_medio:','fr':':personne_barbue_peau_moyennement_claire:','ja':':あごひげの人_やや薄い肌色:','ko':':수염_난_사람_연한_갈색_피부:','pt':':homem_pele_morena_clara_e_barba:','it':':persona_con_la_barba_carnagione_abbastanza_chiara:','fa':':مرد_پوست_روشن_ریش_دار:','id':':orang_warna_kulit_cerah-sedang_janggut:','zh':':有胡子的人_中等浅肤色:','ru':':бородатый_человек_светлый_тон_кожи:','tr':':sakallı_kişi_orta_açık_cilt_tonu:','ar':':شخص_بلحية_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F471\U0001F3FC':{'en':':person_medium-light_skin_tone_blond_hair:','status':fully_qualified,'E':1,'de':':person_mittelhelle_hautfarbe_blondes_haar:','es':':persona_adulta_rubia_tono_de_piel_claro_medio:','fr':':personne_blonde_peau_moyennement_claire:','ja':':金髪の人_やや薄い肌色:','ko':':금발인_사람_연한_갈색_피부:','pt':':pessoa_pele_morena_clara_e_cabelo_louro:','it':':persona_bionda_carnagione_abbastanza_chiara:','fa':':آدم_پوست_روشن_موبلوند:','id':':orang_warna_kulit_cerah-sedang_rambut_pirang:','zh':':金色头发的人_中等浅肤色:','ru':':человек_со_светлыми_волосами_светлый_тон_кожи:','tr':':sarışın_kişi_orta_açık_cilt_tonu:','ar':':شخص_أشقر_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F9B1':{'en':':person_medium-light_skin_tone_curly_hair:','status':fully_qualified,'E':12.1,'de':':person_mittelhelle_hautfarbe_lockiges_haar:','es':':persona_adulta_tono_de_piel_claro_medio_y_pelo_rizado:','fr':':adulte_peau_moyennement_claire_et_cheveux_bouclés:','ja':':大人_やや薄い肌色_巻き毛:','ko':':사람_연한_갈색_피부_곱슬머리:','pt':':pessoa_pele_morena_clara_e_cabelo_cacheado:','it':':persona_carnagione_abbastanza_chiara_e_capelli_ricci:','fa':':شخص_پوست_روشن_موفرفری:','id':':dewasa_warna_kulit_cerah-sedang_rambut_keriting:','zh':':成人_中等浅肤色卷发:','ru':':взрослый_светлый_тон_кожи_кудрявые_волосы:','tr':':yetişkin_orta_açık_cilt_tonu_kıvırcık_saç:','ar':':بالغ_بشرة_بلون_فاتح_ومعتدل_وشعر_مجعد:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F9B0':{'en':':person_medium-light_skin_tone_red_hair:','status':fully_qualified,'E':12.1,'de':':person_mittelhelle_hautfarbe_rotes_haar:','es':':persona_adulta_tono_de_piel_claro_medio_y_pelo_pelirrojo:','fr':':adulte_peau_moyennement_claire_et_cheveux_roux:','ja':':大人_やや薄い肌色_赤毛:','ko':':사람_연한_갈색_피부_빨간_머리:','pt':':pessoa_pele_morena_clara_e_cabelo_vermelho:','it':':persona_carnagione_abbastanza_chiara_e_capelli_rossi:','fa':':شخص_پوست_روشن_موقرمز:','id':':dewasa_warna_kulit_cerah-sedang_rambut_merah:','zh':':成人_中等浅肤色红发:','ru':':взрослый_светлый_тон_кожи_рыжие_волосы:','tr':':yetişkin_orta_açık_cilt_tonu_kızıl_saç:','ar':':بالغ_بشرة_بلون_فاتح_ومعتدل_وشعر_أحمر:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F9B3':{'en':':person_medium-light_skin_tone_white_hair:','status':fully_qualified,'E':12.1,'de':':person_mittelhelle_hautfarbe_weißes_haar:','es':':persona_adulta_tono_de_piel_claro_medio_y_pelo_blanco:','fr':':adulte_peau_moyennement_claire_et_cheveux_blancs:','ja':':大人_やや薄い肌色_白髪:','ko':':사람_연한_갈색_피부_백발:','pt':':pessoa_pele_morena_clara_e_cabelo_branco:','it':':persona_carnagione_abbastanza_chiara_e_capelli_bianchi:','fa':':شخص_پوست_روشن_موسفید:','id':':dewasa_warna_kulit_cerah-sedang_rambut_putih:','zh':':成人_中等浅肤色白发:','ru':':взрослый_светлый_тон_кожи_седые_волосы:','tr':':yetişkin_orta_açık_cilt_tonu_beyaz_saç:','ar':':بالغ_بشرة_بلون_فاتح_ومعتدل_وشعر_أبيض:'},'\U0001F9D1\U0001F3FD':{'en':':person_medium_skin_tone:','status':fully_qualified,'E':5,'de':':person_mittlere_hautfarbe:','es':':persona_adulta_tono_de_piel_medio:','fr':':adulte_peau_légèrement_mate:','ja':':大人_中間の肌色:','ko':':사람_갈색_피부:','pt':':pessoa_pele_morena:','it':':persona_carnagione_olivastra:','fa':':شخص_پوست_طلایی:','id':':dewasa_warna_kulit_sedang:','zh':':成人_中等肤色:','ru':':взрослый_средний_тон_кожи:','tr':':yetişkin_orta_cilt_tonu:','ar':':بالغ_بشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F9B2':{'en':':person_medium_skin_tone_bald:','status':fully_qualified,'E':12.1,'de':':person_mittlere_hautfarbe_glatze:','es':':persona_adulta_tono_de_piel_medio_y_sin_pelo:','fr':':adulte_peau_légèrement_mate_et_chauve:','ja':':大人_中間の肌色_はげ頭:','ko':':사람_갈색_피부_대머리:','pt':':pessoa_pele_morena_e_careca:','it':':persona_carnagione_olivastra_e_calvo:','fa':':شخص_پوست_طلایی_تاس:','id':':dewasa_warna_kulit_sedang_gundul:','zh':':成人_中等肤色秃顶:','ru':':взрослый_средний_тон_кожи_без_волос:','tr':':yetişkin_orta_cilt_tonu_kel:','ar':':بالغ_بشرة_بلون_معتدل_وأصلع:'},'\U0001F9D4\U0001F3FD':{'en':':person_medium_skin_tone_beard:','status':fully_qualified,'E':5,'de':':person_mittlere_hautfarbe_bart:','es':':persona_con_barba_tono_de_piel_medio:','fr':':personne_barbue_peau_légèrement_mate:','ja':':あごひげの人_中間の肌色:','ko':':수염_난_사람_갈색_피부:','pt':':homem_pele_morena_e_barba:','it':':persona_con_la_barba_carnagione_olivastra:','fa':':مرد_پوست_طلایی_ریش_دار:','id':':orang_warna_kulit_sedang_janggut:','zh':':有胡子的人_中等肤色:','ru':':бородатый_человек_средний_тон_кожи:','tr':':sakallı_kişi_orta_cilt_tonu:','ar':':شخص_بلحية_بشرة_بلون_معتدل:'},'\U0001F471\U0001F3FD':{'en':':person_medium_skin_tone_blond_hair:','status':fully_qualified,'E':1,'de':':person_mittlere_hautfarbe_blondes_haar:','es':':persona_adulta_rubia_tono_de_piel_medio:','fr':':personne_blonde_peau_légèrement_mate:','ja':':金髪の人_中間の肌色:','ko':':금발인_사람_갈색_피부:','pt':':pessoa_pele_morena_e_cabelo_louro:','it':':persona_bionda_carnagione_olivastra:','fa':':آدم_پوست_طلایی_موبلوند:','id':':orang_warna_kulit_sedang_rambut_pirang:','zh':':金色头发的人_中等肤色:','ru':':человек_со_светлыми_волосами_средний_тон_кожи:','tr':':sarışın_kişi_orta_cilt_tonu:','ar':':شخص_أشقر_بشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F9B1':{'en':':person_medium_skin_tone_curly_hair:','status':fully_qualified,'E':12.1,'de':':person_mittlere_hautfarbe_lockiges_haar:','es':':persona_adulta_tono_de_piel_medio_y_pelo_rizado:','fr':':adulte_peau_légèrement_mate_et_cheveux_bouclés:','ja':':大人_中間の肌色_巻き毛:','ko':':사람_갈색_피부_곱슬머리:','pt':':pessoa_pele_morena_e_cabelo_cacheado:','it':':persona_carnagione_olivastra_e_capelli_ricci:','fa':':شخص_پوست_طلایی_موفرفری:','id':':dewasa_warna_kulit_sedang_rambut_keriting:','zh':':成人_中等肤色卷发:','ru':':взрослый_средний_тон_кожи_кудрявые_волосы:','tr':':yetişkin_orta_cilt_tonu_kıvırcık_saç:','ar':':بالغ_بشرة_بلون_معتدل_وشعر_مجعد:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F9B0':{'en':':person_medium_skin_tone_red_hair:','status':fully_qualified,'E':12.1,'de':':person_mittlere_hautfarbe_rotes_haar:','es':':persona_adulta_tono_de_piel_medio_y_pelo_pelirrojo:','fr':':adulte_peau_légèrement_mate_et_cheveux_roux:','ja':':大人_中間の肌色_赤毛:','ko':':사람_갈색_피부_빨간_머리:','pt':':pessoa_pele_morena_e_cabelo_vermelho:','it':':persona_carnagione_olivastra_e_capelli_rossi:','fa':':شخص_پوست_طلایی_موقرمز:','id':':dewasa_warna_kulit_sedang_rambut_merah:','zh':':成人_中等肤色红发:','ru':':взрослый_средний_тон_кожи_рыжие_волосы:','tr':':yetişkin_orta_cilt_tonu_kızıl_saç:','ar':':بالغ_بشرة_بلون_معتدل_وشعر_أحمر:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F9B3':{'en':':person_medium_skin_tone_white_hair:','status':fully_qualified,'E':12.1,'de':':person_mittlere_hautfarbe_weißes_haar:','es':':persona_adulta_tono_de_piel_medio_y_pelo_blanco:','fr':':adulte_peau_légèrement_mate_et_cheveux_blancs:','ja':':大人_中間の肌色_白髪:','ko':':사람_갈색_피부_백발:','pt':':pessoa_pele_morena_e_cabelo_branco:','it':':persona_carnagione_olivastra_e_capelli_bianchi:','fa':':شخص_پوست_طلایی_موسفید:','id':':dewasa_warna_kulit_sedang_rambut_putih:','zh':':成人_中等肤色白发:','ru':':взрослый_средний_тон_кожи_седые_волосы:','tr':':yetişkin_orta_cilt_tonu_beyaz_saç:','ar':':بالغ_بشرة_بلون_معتدل_وشعر_أبيض:'},'\U0001F6B5':{'en':':person_mountain_biking:','status':fully_qualified,'E':1,'alias':[':mountain_bicyclist:'],'de':':mountainbiker(in):','es':':persona_en_bicicleta_de_montaña:','fr':':personne_en_vtt:','ja':':マウンテンバイクに乗る人:','ko':':산악_자전거_타는_사람:','pt':':pessoa_fazendo_mountain_bike:','it':':ciclista_di_mountain_bike:','fa':':دوچرخه_کوهستان:','id':':orang_bersepeda_gunung:','zh':':骑山地车:','ru':':человек_на_горном_велосипеде:','tr':':dağ_bisikletçisi:','ar':':شخص_يركب_دراجة_في_الجبال:'},'\U0001F6B5\U0001F3FF':{'en':':person_mountain_biking_dark_skin_tone:','status':fully_qualified,'E':1,'de':':mountainbiker(in)_dunkle_hautfarbe:','es':':persona_en_bicicleta_de_montaña_tono_de_piel_oscuro:','fr':':personne_en_vtt_peau_foncée:','ja':':マウンテンバイクに乗る人_濃い肌色:','ko':':산악_자전거_타는_사람_검은색_피부:','pt':':pessoa_fazendo_mountain_bike_pele_escura:','it':':ciclista_di_mountain_bike_carnagione_scura:','fa':':دوچرخه_کوهستان_پوست_آبنوسی:','id':':orang_bersepeda_gunung_warna_kulit_gelap:','zh':':骑山地车_较深肤色:','ru':':человек_на_горном_велосипеде_очень_темный_тон_кожи:','tr':':dağ_bisikletçisi_koyu_cilt_tonu:','ar':':شخص_يركب_دراجة_في_الجبال_بشرة_بلون_غامق:'},'\U0001F6B5\U0001F3FB':{'en':':person_mountain_biking_light_skin_tone:','status':fully_qualified,'E':1,'de':':mountainbiker(in)_helle_hautfarbe:','es':':persona_en_bicicleta_de_montaña_tono_de_piel_claro:','fr':':personne_en_vtt_peau_claire:','ja':':マウンテンバイクに乗る人_薄い肌色:','ko':':산악_자전거_타는_사람_하얀_피부:','pt':':pessoa_fazendo_mountain_bike_pele_clara:','it':':ciclista_di_mountain_bike_carnagione_chiara:','fa':':دوچرخه_کوهستان_پوست_سفید:','id':':orang_bersepeda_gunung_warna_kulit_cerah:','zh':':骑山地车_较浅肤色:','ru':':человек_на_горном_велосипеде_очень_светлый_тон_кожи:','tr':':dağ_bisikletçisi_açık_cilt_tonu:','ar':':شخص_يركب_دراجة_في_الجبال_بشرة_بلون_فاتح:'},'\U0001F6B5\U0001F3FE':{'en':':person_mountain_biking_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':mountainbiker(in)_mitteldunkle_hautfarbe:','es':':persona_en_bicicleta_de_montaña_tono_de_piel_oscuro_medio:','fr':':personne_en_vtt_peau_mate:','ja':':マウンテンバイクに乗る人_やや濃い肌色:','ko':':산악_자전거_타는_사람_진한_갈색_피부:','pt':':pessoa_fazendo_mountain_bike_pele_morena_escura:','it':':ciclista_di_mountain_bike_carnagione_abbastanza_scura:','fa':':دوچرخه_کوهستان_پوست_گندمی:','id':':orang_bersepeda_gunung_warna_kulit_gelap-sedang:','zh':':骑山地车_中等深肤色:','ru':':человек_на_горном_велосипеде_темный_тон_кожи:','tr':':dağ_bisikletçisi_orta_koyu_cilt_tonu:','ar':':شخص_يركب_دراجة_في_الجبال_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6B5\U0001F3FC':{'en':':person_mountain_biking_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':mountainbiker(in)_mittelhelle_hautfarbe:','es':':persona_en_bicicleta_de_montaña_tono_de_piel_claro_medio:','fr':':personne_en_vtt_peau_moyennement_claire:','ja':':マウンテンバイクに乗る人_やや薄い肌色:','ko':':산악_자전거_타는_사람_연한_갈색_피부:','pt':':pessoa_fazendo_mountain_bike_pele_morena_clara:','it':':ciclista_di_mountain_bike_carnagione_abbastanza_chiara:','fa':':دوچرخه_کوهستان_پوست_روشن:','id':':orang_bersepeda_gunung_warna_kulit_cerah-sedang:','zh':':骑山地车_中等浅肤色:','ru':':человек_на_горном_велосипеде_светлый_тон_кожи:','tr':':dağ_bisikletçisi_orta_açık_cilt_tonu:','ar':':شخص_يركب_دراجة_في_الجبال_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6B5\U0001F3FD':{'en':':person_mountain_biking_medium_skin_tone:','status':fully_qualified,'E':1,'de':':mountainbiker(in)_mittlere_hautfarbe:','es':':persona_en_bicicleta_de_montaña_tono_de_piel_medio:','fr':':personne_en_vtt_peau_légèrement_mate:','ja':':マウンテンバイクに乗る人_中間の肌色:','ko':':산악_자전거_타는_사람_갈색_피부:','pt':':pessoa_fazendo_mountain_bike_pele_morena:','it':':ciclista_di_mountain_bike_carnagione_olivastra:','fa':':دوچرخه_کوهستان_پوست_طلایی:','id':':orang_bersepeda_gunung_warna_kulit_sedang:','zh':':骑山地车_中等肤色:','ru':':человек_на_горном_велосипеде_средний_тон_кожи:','tr':':dağ_bisikletçisi_orta_cilt_tonu:','ar':':شخص_يركب_دراجة_في_الجبال_بشرة_بلون_معتدل:'},'\U0001F93E':{'en':':person_playing_handball:','status':fully_qualified,'E':3,'alias':[':handball_person:'],'de':':handballspieler(in):','es':':persona_jugando_al_balonmano:','fr':':personne_jouant_au_handball:','ja':':ハンドボールをする人:','ko':':핸드볼하는_사람:','pt':':handebol:','it':':persona_che_gioca_a_pallamano:','fa':':هندبال:','id':':orang_bermain_bola_tangan:','zh':':手球:','ru':':гандбол:','tr':':hentbol:','ar':':شخص_يلعب_كرة_يد:'},'\U0001F93E\U0001F3FF':{'en':':person_playing_handball_dark_skin_tone:','status':fully_qualified,'E':3,'de':':handballspieler(in)_dunkle_hautfarbe:','es':':persona_jugando_al_balonmano_tono_de_piel_oscuro:','fr':':personne_jouant_au_handball_peau_foncée:','ja':':ハンドボールをする人_濃い肌色:','ko':':핸드볼하는_사람_검은색_피부:','pt':':handebol_pele_escura:','it':':persona_che_gioca_a_pallamano_carnagione_scura:','fa':':هندبال_پوست_آبنوسی:','id':':orang_bermain_bola_tangan_warna_kulit_gelap:','zh':':手球_较深肤色:','ru':':гандбол_очень_темный_тон_кожи:','tr':':hentbol_koyu_cilt_tonu:','ar':':شخص_يلعب_كرة_يد_بشرة_بلون_غامق:'},'\U0001F93E\U0001F3FB':{'en':':person_playing_handball_light_skin_tone:','status':fully_qualified,'E':3,'de':':handballspieler(in)_helle_hautfarbe:','es':':persona_jugando_al_balonmano_tono_de_piel_claro:','fr':':personne_jouant_au_handball_peau_claire:','ja':':ハンドボールをする人_薄い肌色:','ko':':핸드볼하는_사람_하얀_피부:','pt':':handebol_pele_clara:','it':':persona_che_gioca_a_pallamano_carnagione_chiara:','fa':':هندبال_پوست_سفید:','id':':orang_bermain_bola_tangan_warna_kulit_cerah:','zh':':手球_较浅肤色:','ru':':гандбол_очень_светлый_тон_кожи:','tr':':hentbol_açık_cilt_tonu:','ar':':شخص_يلعب_كرة_يد_بشرة_بلون_فاتح:'},'\U0001F93E\U0001F3FE':{'en':':person_playing_handball_medium-dark_skin_tone:','status':fully_qualified,'E':3,'de':':handballspieler(in)_mitteldunkle_hautfarbe:','es':':persona_jugando_al_balonmano_tono_de_piel_oscuro_medio:','fr':':personne_jouant_au_handball_peau_mate:','ja':':ハンドボールをする人_やや濃い肌色:','ko':':핸드볼하는_사람_진한_갈색_피부:','pt':':handebol_pele_morena_escura:','it':':persona_che_gioca_a_pallamano_carnagione_abbastanza_scura:','fa':':هندبال_پوست_گندمی:','id':':orang_bermain_bola_tangan_warna_kulit_gelap-sedang:','zh':':手球_中等深肤色:','ru':':гандбол_темный_тон_кожи:','tr':':hentbol_orta_koyu_cilt_tonu:','ar':':شخص_يلعب_كرة_يد_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F93E\U0001F3FC':{'en':':person_playing_handball_medium-light_skin_tone:','status':fully_qualified,'E':3,'de':':handballspieler(in)_mittelhelle_hautfarbe:','es':':persona_jugando_al_balonmano_tono_de_piel_claro_medio:','fr':':personne_jouant_au_handball_peau_moyennement_claire:','ja':':ハンドボールをする人_やや薄い肌色:','ko':':핸드볼하는_사람_연한_갈색_피부:','pt':':handebol_pele_morena_clara:','it':':persona_che_gioca_a_pallamano_carnagione_abbastanza_chiara:','fa':':هندبال_پوست_روشن:','id':':orang_bermain_bola_tangan_warna_kulit_cerah-sedang:','zh':':手球_中等浅肤色:','ru':':гандбол_светлый_тон_кожи:','tr':':hentbol_orta_açık_cilt_tonu:','ar':':شخص_يلعب_كرة_يد_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F93E\U0001F3FD':{'en':':person_playing_handball_medium_skin_tone:','status':fully_qualified,'E':3,'de':':handballspieler(in)_mittlere_hautfarbe:','es':':persona_jugando_al_balonmano_tono_de_piel_medio:','fr':':personne_jouant_au_handball_peau_légèrement_mate:','ja':':ハンドボールをする人_中間の肌色:','ko':':핸드볼하는_사람_갈색_피부:','pt':':handebol_pele_morena:','it':':persona_che_gioca_a_pallamano_carnagione_olivastra:','fa':':هندبال_پوست_طلایی:','id':':orang_bermain_bola_tangan_warna_kulit_sedang:','zh':':手球_中等肤色:','ru':':гандбол_средний_тон_кожи:','tr':':hentbol_orta_cilt_tonu:','ar':':شخص_يلعب_كرة_يد_بشرة_بلون_معتدل:'},'\U0001F93D':{'en':':person_playing_water_polo:','status':fully_qualified,'E':3,'alias':[':water_polo:'],'de':':wasserballspieler(in):','es':':persona_jugando_al_waterpolo:','fr':':personne_jouant_au_water-polo:','ja':':水球をする人:','ko':':수구하는_사람:','pt':':pessoa_jogando_polo_aquático:','it':':persona_che_gioca_a_pallanuoto:','fa':':واترپلو:','id':':orang_bermain_polo_air:','zh':':水球:','ru':':водное_поло:','tr':':su_topu:','ar':':شخص_يلعب_كرة_ماء:'},'\U0001F93D\U0001F3FF':{'en':':person_playing_water_polo_dark_skin_tone:','status':fully_qualified,'E':3,'de':':wasserballspieler(in)_dunkle_hautfarbe:','es':':persona_jugando_al_waterpolo_tono_de_piel_oscuro:','fr':':personne_jouant_au_water-polo_peau_foncée:','ja':':水球をする人_濃い肌色:','ko':':수구하는_사람_검은색_피부:','pt':':pessoa_jogando_polo_aquático_pele_escura:','it':':persona_che_gioca_a_pallanuoto_carnagione_scura:','fa':':واترپلو_پوست_آبنوسی:','id':':orang_bermain_polo_air_warna_kulit_gelap:','zh':':水球_较深肤色:','ru':':водное_поло_очень_темный_тон_кожи:','tr':':su_topu_koyu_cilt_tonu:','ar':':شخص_يلعب_كرة_ماء_بشرة_بلون_غامق:'},'\U0001F93D\U0001F3FB':{'en':':person_playing_water_polo_light_skin_tone:','status':fully_qualified,'E':3,'de':':wasserballspieler(in)_helle_hautfarbe:','es':':persona_jugando_al_waterpolo_tono_de_piel_claro:','fr':':personne_jouant_au_water-polo_peau_claire:','ja':':水球をする人_薄い肌色:','ko':':수구하는_사람_하얀_피부:','pt':':pessoa_jogando_polo_aquático_pele_clara:','it':':persona_che_gioca_a_pallanuoto_carnagione_chiara:','fa':':واترپلو_پوست_سفید:','id':':orang_bermain_polo_air_warna_kulit_cerah:','zh':':水球_较浅肤色:','ru':':водное_поло_очень_светлый_тон_кожи:','tr':':su_topu_açık_cilt_tonu:','ar':':شخص_يلعب_كرة_ماء_بشرة_بلون_فاتح:'},'\U0001F93D\U0001F3FE':{'en':':person_playing_water_polo_medium-dark_skin_tone:','status':fully_qualified,'E':3,'de':':wasserballspieler(in)_mitteldunkle_hautfarbe:','es':':persona_jugando_al_waterpolo_tono_de_piel_oscuro_medio:','fr':':personne_jouant_au_water-polo_peau_mate:','ja':':水球をする人_やや濃い肌色:','ko':':수구하는_사람_진한_갈색_피부:','pt':':pessoa_jogando_polo_aquático_pele_morena_escura:','it':':persona_che_gioca_a_pallanuoto_carnagione_abbastanza_scura:','fa':':واترپلو_پوست_گندمی:','id':':orang_bermain_polo_air_warna_kulit_gelap-sedang:','zh':':水球_中等深肤色:','ru':':водное_поло_темный_тон_кожи:','tr':':su_topu_orta_koyu_cilt_tonu:','ar':':شخص_يلعب_كرة_ماء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F93D\U0001F3FC':{'en':':person_playing_water_polo_medium-light_skin_tone:','status':fully_qualified,'E':3,'de':':wasserballspieler(in)_mittelhelle_hautfarbe:','es':':persona_jugando_al_waterpolo_tono_de_piel_claro_medio:','fr':':personne_jouant_au_water-polo_peau_moyennement_claire:','ja':':水球をする人_やや薄い肌色:','ko':':수구하는_사람_연한_갈색_피부:','pt':':pessoa_jogando_polo_aquático_pele_morena_clara:','it':':persona_che_gioca_a_pallanuoto_carnagione_abbastanza_chiara:','fa':':واترپلو_پوست_روشن:','id':':orang_bermain_polo_air_warna_kulit_cerah-sedang:','zh':':水球_中等浅肤色:','ru':':водное_поло_светлый_тон_кожи:','tr':':su_topu_orta_açık_cilt_tonu:','ar':':شخص_يلعب_كرة_ماء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F93D\U0001F3FD':{'en':':person_playing_water_polo_medium_skin_tone:','status':fully_qualified,'E':3,'de':':wasserballspieler(in)_mittlere_hautfarbe:','es':':persona_jugando_al_waterpolo_tono_de_piel_medio:','fr':':personne_jouant_au_water-polo_peau_légèrement_mate:','ja':':水球をする人_中間の肌色:','ko':':수구하는_사람_갈색_피부:','pt':':pessoa_jogando_polo_aquático_pele_morena:','it':':persona_che_gioca_a_pallanuoto_carnagione_olivastra:','fa':':واترپلو_پوست_طلایی:','id':':orang_bermain_polo_air_warna_kulit_sedang:','zh':':水球_中等肤色:','ru':':водное_поло_средний_тон_кожи:','tr':':su_topu_orta_cilt_tonu:','ar':':شخص_يلعب_كرة_ماء_بشرة_بلون_معتدل:'},'\U0001F64E':{'en':':person_pouting:','status':fully_qualified,'E':0.6,'alias':[':pouting_face:', ':person_with_pouting_face:'],'de':':schmollende_person:','es':':persona_haciendo_pucheros:','fr':':personne_qui_boude:','ja':':不機嫌な人:','ko':':뾰로통한_사람:','pt':':pessoa_fazendo_bico:','it':':persona_imbronciata:','fa':':آدم_ناراضی:','id':':orang_tidak_senang:','zh':':撅嘴:','ru':':обиженный_человек:','tr':':darılma:','ar':':شخص_متجهم:'},'\U0001F64E\U0001F3FF':{'en':':person_pouting_dark_skin_tone:','status':fully_qualified,'E':1,'de':':schmollende_person_dunkle_hautfarbe:','es':':persona_haciendo_pucheros_tono_de_piel_oscuro:','fr':':personne_qui_boude_peau_foncée:','ja':':不機嫌な人_濃い肌色:','ko':':뾰로통한_사람_검은색_피부:','pt':':pessoa_fazendo_bico_pele_escura:','it':':persona_imbronciata_carnagione_scura:','fa':':آدم_ناراضی_پوست_آبنوسی:','id':':orang_tidak_senang_warna_kulit_gelap:','zh':':撅嘴_较深肤色:','ru':':обиженный_человек_очень_темный_тон_кожи:','tr':':darılma_koyu_cilt_tonu:','ar':':شخص_متجهم_بشرة_بلون_غامق:'},'\U0001F64E\U0001F3FB':{'en':':person_pouting_light_skin_tone:','status':fully_qualified,'E':1,'de':':schmollende_person_helle_hautfarbe:','es':':persona_haciendo_pucheros_tono_de_piel_claro:','fr':':personne_qui_boude_peau_claire:','ja':':不機嫌な人_薄い肌色:','ko':':뾰로통한_사람_하얀_피부:','pt':':pessoa_fazendo_bico_pele_clara:','it':':persona_imbronciata_carnagione_chiara:','fa':':آدم_ناراضی_پوست_سفید:','id':':orang_tidak_senang_warna_kulit_cerah:','zh':':撅嘴_较浅肤色:','ru':':обиженный_человек_очень_светлый_тон_кожи:','tr':':darılma_açık_cilt_tonu:','ar':':شخص_متجهم_بشرة_بلون_فاتح:'},'\U0001F64E\U0001F3FE':{'en':':person_pouting_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':schmollende_person_mitteldunkle_hautfarbe:','es':':persona_haciendo_pucheros_tono_de_piel_oscuro_medio:','fr':':personne_qui_boude_peau_mate:','ja':':不機嫌な人_やや濃い肌色:','ko':':뾰로통한_사람_진한_갈색_피부:','pt':':pessoa_fazendo_bico_pele_morena_escura:','it':':persona_imbronciata_carnagione_abbastanza_scura:','fa':':آدم_ناراضی_پوست_گندمی:','id':':orang_tidak_senang_warna_kulit_gelap-sedang:','zh':':撅嘴_中等深肤色:','ru':':обиженный_человек_темный_тон_кожи:','tr':':darılma_orta_koyu_cilt_tonu:','ar':':شخص_متجهم_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F64E\U0001F3FC':{'en':':person_pouting_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':schmollende_person_mittelhelle_hautfarbe:','es':':persona_haciendo_pucheros_tono_de_piel_claro_medio:','fr':':personne_qui_boude_peau_moyennement_claire:','ja':':不機嫌な人_やや薄い肌色:','ko':':뾰로통한_사람_연한_갈색_피부:','pt':':pessoa_fazendo_bico_pele_morena_clara:','it':':persona_imbronciata_carnagione_abbastanza_chiara:','fa':':آدم_ناراضی_پوست_روشن:','id':':orang_tidak_senang_warna_kulit_cerah-sedang:','zh':':撅嘴_中等浅肤色:','ru':':обиженный_человек_светлый_тон_кожи:','tr':':darılma_orta_açık_cilt_tonu:','ar':':شخص_متجهم_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F64E\U0001F3FD':{'en':':person_pouting_medium_skin_tone:','status':fully_qualified,'E':1,'de':':schmollende_person_mittlere_hautfarbe:','es':':persona_haciendo_pucheros_tono_de_piel_medio:','fr':':personne_qui_boude_peau_légèrement_mate:','ja':':不機嫌な人_中間の肌色:','ko':':뾰로통한_사람_갈색_피부:','pt':':pessoa_fazendo_bico_pele_morena:','it':':persona_imbronciata_carnagione_olivastra:','fa':':آدم_ناراضی_پوست_طلایی:','id':':orang_tidak_senang_warna_kulit_sedang:','zh':':撅嘴_中等肤色:','ru':':обиженный_человек_средний_тон_кожи:','tr':':darılma_orta_cilt_tonu:','ar':':شخص_متجهم_بشرة_بلون_معتدل:'},'\U0001F64B':{'en':':person_raising_hand:','status':fully_qualified,'E':0.6,'alias':[':raising_hand:'],'de':':person_mit_erhobenem_arm:','es':':persona_con_la_mano_levantada:','fr':':personne_qui_lève_la_main:','ja':':手を挙げる人:','ko':':한_손_든_사람:','pt':':pessoa_levantando_a_mão:','it':':persona_con_mano_alzata:','fa':':ای_ول:','id':':orang_mengangkat_tangan:','zh':':举手:','ru':':человек_с_поднятой_рукой:','tr':':elini_kaldırma:','ar':':شخص_يرفع_يده:'},'\U0001F64B\U0001F3FF':{'en':':person_raising_hand_dark_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_erhobenem_arm_dunkle_hautfarbe:','es':':persona_con_la_mano_levantada_tono_de_piel_oscuro:','fr':':personne_qui_lève_la_main_peau_foncée:','ja':':手を挙げる人_濃い肌色:','ko':':한_손_든_사람_검은색_피부:','pt':':pessoa_levantando_a_mão_pele_escura:','it':':persona_con_mano_alzata_carnagione_scura:','fa':':ای_ول_پوست_آبنوسی:','id':':orang_mengangkat_tangan_warna_kulit_gelap:','zh':':举手_较深肤色:','ru':':человек_с_поднятой_рукой_очень_темный_тон_кожи:','tr':':elini_kaldırma_koyu_cilt_tonu:','ar':':شخص_يرفع_يده_بشرة_بلون_غامق:'},'\U0001F64B\U0001F3FB':{'en':':person_raising_hand_light_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_erhobenem_arm_helle_hautfarbe:','es':':persona_con_la_mano_levantada_tono_de_piel_claro:','fr':':personne_qui_lève_la_main_peau_claire:','ja':':手を挙げる人_薄い肌色:','ko':':한_손_든_사람_하얀_피부:','pt':':pessoa_levantando_a_mão_pele_clara:','it':':persona_con_mano_alzata_carnagione_chiara:','fa':':ای_ول_پوست_سفید:','id':':orang_mengangkat_tangan_warna_kulit_cerah:','zh':':举手_较浅肤色:','ru':':человек_с_поднятой_рукой_очень_светлый_тон_кожи:','tr':':elini_kaldırma_açık_cilt_tonu:','ar':':شخص_يرفع_يده_بشرة_بلون_فاتح:'},'\U0001F64B\U0001F3FE':{'en':':person_raising_hand_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_erhobenem_arm_mitteldunkle_hautfarbe:','es':':persona_con_la_mano_levantada_tono_de_piel_oscuro_medio:','fr':':personne_qui_lève_la_main_peau_mate:','ja':':手を挙げる人_やや濃い肌色:','ko':':한_손_든_사람_진한_갈색_피부:','pt':':pessoa_levantando_a_mão_pele_morena_escura:','it':':persona_con_mano_alzata_carnagione_abbastanza_scura:','fa':':ای_ول_پوست_گندمی:','id':':orang_mengangkat_tangan_warna_kulit_gelap-sedang:','zh':':举手_中等深肤色:','ru':':человек_с_поднятой_рукой_темный_тон_кожи:','tr':':elini_kaldırma_orta_koyu_cilt_tonu:','ar':':شخص_يرفع_يده_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F64B\U0001F3FC':{'en':':person_raising_hand_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_erhobenem_arm_mittelhelle_hautfarbe:','es':':persona_con_la_mano_levantada_tono_de_piel_claro_medio:','fr':':personne_qui_lève_la_main_peau_moyennement_claire:','ja':':手を挙げる人_やや薄い肌色:','ko':':한_손_든_사람_연한_갈색_피부:','pt':':pessoa_levantando_a_mão_pele_morena_clara:','it':':persona_con_mano_alzata_carnagione_abbastanza_chiara:','fa':':ای_ول_پوست_روشن:','id':':orang_mengangkat_tangan_warna_kulit_cerah-sedang:','zh':':举手_中等浅肤色:','ru':':человек_с_поднятой_рукой_светлый_тон_кожи:','tr':':elini_kaldırma_orta_açık_cilt_tonu:','ar':':شخص_يرفع_يده_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F64B\U0001F3FD':{'en':':person_raising_hand_medium_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_erhobenem_arm_mittlere_hautfarbe:','es':':persona_con_la_mano_levantada_tono_de_piel_medio:','fr':':personne_qui_lève_la_main_peau_légèrement_mate:','ja':':手を挙げる人_中間の肌色:','ko':':한_손_든_사람_갈색_피부:','pt':':pessoa_levantando_a_mão_pele_morena:','it':':persona_con_mano_alzata_carnagione_olivastra:','fa':':ای_ول_پوست_طلایی:','id':':orang_mengangkat_tangan_warna_kulit_sedang:','zh':':举手_中等肤色:','ru':':человек_с_поднятой_рукой_средний_тон_кожи:','tr':':elini_kaldırma_orta_cilt_tonu:','ar':':شخص_يرفع_يده_بشرة_بلون_معتدل:'},'\U0001F9D1\U0000200D\U0001F9B0':{'en':':person_red_hair:','status':fully_qualified,'E':12.1,'de':':person_rotes_haar:','es':':persona_adulta_pelo_pelirrojo:','fr':':adulte_cheveux_roux:','ja':':大人_赤毛:','ko':':사람_빨간_머리:','pt':':pessoa_cabelo_vermelho:','it':':persona_capelli_rossi:','fa':':شخص_موقرمز:','id':':dewasa_rambut_merah:','zh':':成人_红发:','ru':':взрослый_рыжие_волосы:','tr':':yetişkin_kızıl_saç:','ar':':بالغ_شعر_أحمر:'},'\U0001F6A3':{'en':':person_rowing_boat:','status':fully_qualified,'E':1,'alias':[':rowboat:'],'de':':person_im_ruderboot:','es':':persona_remando_en_un_bote:','fr':':personne_ramant_dans_une_barque:','ja':':ボートをこぐ人:','ko':':노젓는_사람:','pt':':pessoa_remando:','it':':persona_in_barca_a_remi:','fa':':قایق_پارویی:','id':':orang_mendayung_perahu:','zh':':划艇:','ru':':гребля:','tr':':sandalda_kürek_çeken_kişi:','ar':':شخص_يجدف_بزورق:'},'\U0001F6A3\U0001F3FF':{'en':':person_rowing_boat_dark_skin_tone:','status':fully_qualified,'E':1,'de':':person_im_ruderboot_dunkle_hautfarbe:','es':':persona_remando_en_un_bote_tono_de_piel_oscuro:','fr':':personne_ramant_dans_une_barque_peau_foncée:','ja':':ボートをこぐ人_濃い肌色:','ko':':노젓는_사람_검은색_피부:','pt':':pessoa_remando_pele_escura:','it':':persona_in_barca_a_remi_carnagione_scura:','fa':':قایق_پارویی_پوست_آبنوسی:','id':':orang_mendayung_perahu_warna_kulit_gelap:','zh':':划艇_较深肤色:','ru':':гребля_очень_темный_тон_кожи:','tr':':sandalda_kürek_çeken_kişi_koyu_cilt_tonu:','ar':':شخص_يجدف_بزورق_بشرة_بلون_غامق:'},'\U0001F6A3\U0001F3FB':{'en':':person_rowing_boat_light_skin_tone:','status':fully_qualified,'E':1,'de':':person_im_ruderboot_helle_hautfarbe:','es':':persona_remando_en_un_bote_tono_de_piel_claro:','fr':':personne_ramant_dans_une_barque_peau_claire:','ja':':ボートをこぐ人_薄い肌色:','ko':':노젓는_사람_하얀_피부:','pt':':pessoa_remando_pele_clara:','it':':persona_in_barca_a_remi_carnagione_chiara:','fa':':قایق_پارویی_پوست_سفید:','id':':orang_mendayung_perahu_warna_kulit_cerah:','zh':':划艇_较浅肤色:','ru':':гребля_очень_светлый_тон_кожи:','tr':':sandalda_kürek_çeken_kişi_açık_cilt_tonu:','ar':':شخص_يجدف_بزورق_بشرة_بلون_فاتح:'},'\U0001F6A3\U0001F3FE':{'en':':person_rowing_boat_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':person_im_ruderboot_mitteldunkle_hautfarbe:','es':':persona_remando_en_un_bote_tono_de_piel_oscuro_medio:','fr':':personne_ramant_dans_une_barque_peau_mate:','ja':':ボートをこぐ人_やや濃い肌色:','ko':':노젓는_사람_진한_갈색_피부:','pt':':pessoa_remando_pele_morena_escura:','it':':persona_in_barca_a_remi_carnagione_abbastanza_scura:','fa':':قایق_پارویی_پوست_گندمی:','id':':orang_mendayung_perahu_warna_kulit_gelap-sedang:','zh':':划艇_中等深肤色:','ru':':гребля_темный_тон_кожи:','tr':':sandalda_kürek_çeken_kişi_orta_koyu_cilt_tonu:','ar':':شخص_يجدف_بزورق_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6A3\U0001F3FC':{'en':':person_rowing_boat_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':person_im_ruderboot_mittelhelle_hautfarbe:','es':':persona_remando_en_un_bote_tono_de_piel_claro_medio:','fr':':personne_ramant_dans_une_barque_peau_moyennement_claire:','ja':':ボートをこぐ人_やや薄い肌色:','ko':':노젓는_사람_연한_갈색_피부:','pt':':pessoa_remando_pele_morena_clara:','it':':persona_in_barca_a_remi_carnagione_abbastanza_chiara:','fa':':قایق_پارویی_پوست_روشن:','id':':orang_mendayung_perahu_warna_kulit_cerah-sedang:','zh':':划艇_中等浅肤色:','ru':':гребля_светлый_тон_кожи:','tr':':sandalda_kürek_çeken_kişi_orta_açık_cilt_tonu:','ar':':شخص_يجدف_بزورق_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6A3\U0001F3FD':{'en':':person_rowing_boat_medium_skin_tone:','status':fully_qualified,'E':1,'de':':person_im_ruderboot_mittlere_hautfarbe:','es':':persona_remando_en_un_bote_tono_de_piel_medio:','fr':':personne_ramant_dans_une_barque_peau_légèrement_mate:','ja':':ボートをこぐ人_中間の肌色:','ko':':노젓는_사람_갈색_피부:','pt':':pessoa_remando_pele_morena:','it':':persona_in_barca_a_remi_carnagione_olivastra:','fa':':قایق_پارویی_پوست_طلایی:','id':':orang_mendayung_perahu_warna_kulit_sedang:','zh':':划艇_中等肤色:','ru':':гребля_средний_тон_кожи:','tr':':sandalda_kürek_çeken_kişi_orta_cilt_tonu:','ar':':شخص_يجدف_بزورق_بشرة_بلون_معتدل:'},'\U0001F3C3':{'en':':person_running:','status':fully_qualified,'E':0.6,'alias':[':runner:', ':running:'],'de':':laufende_person:','es':':persona_corriendo:','fr':':personne_qui_court:','ja':':走る人:','ko':':뛰는_사람:','pt':':pessoa_correndo:','it':':persona_che_corre:','fa':':دونده:','id':':orang_berlari:','zh':':跑步者:','ru':':человек_бежит:','tr':':koşan_kişi:','ar':':شخص_يجري:'},'\U0001F3C3\U0001F3FF':{'en':':person_running_dark_skin_tone:','status':fully_qualified,'E':1,'de':':laufende_person_dunkle_hautfarbe:','es':':persona_corriendo_tono_de_piel_oscuro:','fr':':personne_qui_court_peau_foncée:','ja':':走る人_濃い肌色:','ko':':뛰는_사람_검은색_피부:','pt':':pessoa_correndo_pele_escura:','it':':persona_che_corre_carnagione_scura:','fa':':دونده_پوست_آبنوسی:','id':':orang_berlari_warna_kulit_gelap:','zh':':跑步者_较深肤色:','ru':':человек_бежит_очень_темный_тон_кожи:','tr':':koşan_kişi_koyu_cilt_tonu:','ar':':شخص_يجري_بشرة_بلون_غامق:'},'\U0001F3C3\U0000200D\U000027A1\U0000FE0F':{'en':':person_running_facing_right:','status':fully_qualified,'E':15.1,'de':':laufende_person_nach_rechts:','es':':persona_corriendo_hacia_la_derecha:','fr':':personne_qui_court_vers_la_droite:','ja':':走る人_右向き:','ko':':오른쪽을_향한_뛰는_사람:','pt':':pessoa_correndo_de_frente_para_a_direita:','it':':persona_che_corre_a_destra:','fa':':دونده_به_سمت_راست:','id':':orang_berlari_menghadap_kanan:','zh':':跑步者面向右边:','ru':':человек_бежит_направленные_вправо:','tr':':sağa_dönük_koşan_kişi:','ar':':شخص_يجري_متجه_لليمين:'},'\U0001F3C3\U0000200D\U000027A1':{'en':':person_running_facing_right:','status':minimally_qualified,'E':15.1,'de':':laufende_person_nach_rechts:','es':':persona_corriendo_hacia_la_derecha:','fr':':personne_qui_court_vers_la_droite:','ja':':走る人_右向き:','ko':':오른쪽을_향한_뛰는_사람:','pt':':pessoa_correndo_de_frente_para_a_direita:','it':':persona_che_corre_a_destra:','fa':':دونده_به_سمت_راست:','id':':orang_berlari_menghadap_kanan:','zh':':跑步者面向右边:','ru':':человек_бежит_направленные_вправо:','tr':':sağa_dönük_koşan_kişi:','ar':':شخص_يجري_متجه_لليمين:'},'\U0001F3C3\U0001F3FF\U0000200D\U000027A1\U0000FE0F':{'en':':person_running_facing_right_dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':laufende_person_nach_rechts_dunkle_hautfarbe:','es':':persona_corriendo_hacia_la_derecha_tono_de_piel_oscuro:','fr':':personne_qui_court_vers_la_droite_peau_foncée:','ja':':走る人_右向き_濃い肌色:','ko':':오른쪽을_향한_뛰는_사람_검은색_피부:','pt':':pessoa_correndo_de_frente_para_a_direita_pele_escura:','it':':persona_che_corre_a_destra_carnagione_scura:','fa':':دونده_به_سمت_راستپوست_آبنوسی:','id':':orang_berlari_menghadap_kanan_warna_kulit_gelap:','zh':':跑步者面向右边_较深肤色:','ru':':человек_бежит_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_koşan_kişi_koyu_cilt_tonu:','ar':':شخص_يجري_متجه_لليمين_بلون_غامق:'},'\U0001F3C3\U0001F3FF\U0000200D\U000027A1':{'en':':person_running_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufende_person_nach_rechts_dunkle_hautfarbe:','es':':persona_corriendo_hacia_la_derecha_tono_de_piel_oscuro:','fr':':personne_qui_court_vers_la_droite_peau_foncée:','ja':':走る人_右向き_濃い肌色:','ko':':오른쪽을_향한_뛰는_사람_검은색_피부:','pt':':pessoa_correndo_de_frente_para_a_direita_pele_escura:','it':':persona_che_corre_a_destra_carnagione_scura:','fa':':دونده_به_سمت_راستپوست_آبنوسی:','id':':orang_berlari_menghadap_kanan_warna_kulit_gelap:','zh':':跑步者面向右边_较深肤色:','ru':':человек_бежит_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_koşan_kişi_koyu_cilt_tonu:','ar':':شخص_يجري_متجه_لليمين_بلون_غامق:'},'\U0001F3C3\U0001F3FB\U0000200D\U000027A1\U0000FE0F':{'en':':person_running_facing_right_light_skin_tone:','status':fully_qualified,'E':15.1,'de':':laufende_person_nach_rechts_helle_hautfarbe:','es':':persona_corriendo_hacia_la_derecha_tono_de_piel_claro:','fr':':personne_qui_court_vers_la_droite_peau_claire:','ja':':走る人_右向き_薄い肌色:','ko':':오른쪽을_향한_뛰는_사람_하얀_피부:','pt':':pessoa_correndo_de_frente_para_a_direita_pele_clara:','it':':persona_che_corre_a_destra_carnagione_chiara:','fa':':دونده_به_سمت_راست_پوست_سفید:','id':':orang_berlari_menghadap_kanan_warna_kulit_cerah:','zh':':跑步者面向右边_较浅肤色:','ru':':человек_бежит_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_koşan_kişi_açık_cilt_tonu:','ar':':شخص_يجري_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F3C3\U0001F3FB\U0000200D\U000027A1':{'en':':person_running_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufende_person_nach_rechts_helle_hautfarbe:','es':':persona_corriendo_hacia_la_derecha_tono_de_piel_claro:','fr':':personne_qui_court_vers_la_droite_peau_claire:','ja':':走る人_右向き_薄い肌色:','ko':':오른쪽을_향한_뛰는_사람_하얀_피부:','pt':':pessoa_correndo_de_frente_para_a_direita_pele_clara:','it':':persona_che_corre_a_destra_carnagione_chiara:','fa':':دونده_به_سمت_راست_پوست_سفید:','id':':orang_berlari_menghadap_kanan_warna_kulit_cerah:','zh':':跑步者面向右边_较浅肤色:','ru':':человек_бежит_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_koşan_kişi_açık_cilt_tonu:','ar':':شخص_يجري_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F3C3\U0001F3FE\U0000200D\U000027A1\U0000FE0F':{'en':':person_running_facing_right_medium-dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':laufende_person_nach_rechts_mitteldunkle_hautfarbe:','es':':persona_corriendo_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':personne_qui_court_vers_la_droite_peau_mate:','ja':':走る人_右向き_やや濃い肌色:','ko':':오른쪽을_향한_뛰는_사람_진한_갈색_피부:','pt':':pessoa_correndo_de_frente_para_a_direita_pele_morena_escura:','it':':persona_che_corre_a_destra_carnagione_abbastanza_scura:','fa':':دونده_به_سمت_راست_پوست_گندمی:','id':':orang_berlari_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':跑步者面向右边_中等深肤色:','ru':':человек_бежит_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_koşan_kişi_orta_koyu_cilt_tonu:','ar':':شخص_يجري_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F3C3\U0001F3FE\U0000200D\U000027A1':{'en':':person_running_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufende_person_nach_rechts_mitteldunkle_hautfarbe:','es':':persona_corriendo_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':personne_qui_court_vers_la_droite_peau_mate:','ja':':走る人_右向き_やや濃い肌色:','ko':':오른쪽을_향한_뛰는_사람_진한_갈색_피부:','pt':':pessoa_correndo_de_frente_para_a_direita_pele_morena_escura:','it':':persona_che_corre_a_destra_carnagione_abbastanza_scura:','fa':':دونده_به_سمت_راست_پوست_گندمی:','id':':orang_berlari_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':跑步者面向右边_中等深肤色:','ru':':человек_бежит_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_koşan_kişi_orta_koyu_cilt_tonu:','ar':':شخص_يجري_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F3C3\U0001F3FC\U0000200D\U000027A1\U0000FE0F':{'en':':person_running_facing_right_medium-light_skin_tone:','status':fully_qualified,'E':15.1,'de':':laufende_person_nach_rechts_mittelhelle_hautfarbe:','es':':persona_corriendo_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':personne_qui_court_vers_la_droite_peau_moyennement_claire:','ja':':走る人_右向き_やや薄い肌色:','ko':':오른쪽을_향한_뛰는_사람_연한_갈색_피부:','pt':':pessoa_correndo_de_frente_para_a_direita_pele_morena_clara:','it':':persona_che_corre_a_destra_carnagione_abbastanza_chiara:','fa':':دونده_به_سمت_راست_پوست_روشن:','id':':orang_berlari_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':跑步者面向右边_中等浅肤色:','ru':':человек_бежит_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_koşan_kişi_orta_açık_cilt_tonu:','ar':':شخص_يجري_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F3C3\U0001F3FC\U0000200D\U000027A1':{'en':':person_running_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufende_person_nach_rechts_mittelhelle_hautfarbe:','es':':persona_corriendo_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':personne_qui_court_vers_la_droite_peau_moyennement_claire:','ja':':走る人_右向き_やや薄い肌色:','ko':':오른쪽을_향한_뛰는_사람_연한_갈색_피부:','pt':':pessoa_correndo_de_frente_para_a_direita_pele_morena_clara:','it':':persona_che_corre_a_destra_carnagione_abbastanza_chiara:','fa':':دونده_به_سمت_راست_پوست_روشن:','id':':orang_berlari_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':跑步者面向右边_中等浅肤色:','ru':':человек_бежит_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_koşan_kişi_orta_açık_cilt_tonu:','ar':':شخص_يجري_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F3C3\U0001F3FD\U0000200D\U000027A1\U0000FE0F':{'en':':person_running_facing_right_medium_skin_tone:','status':fully_qualified,'E':15.1,'de':':laufende_person_nach_rechts_mittlere_hautfarbe:','es':':persona_corriendo_hacia_la_derecha_tono_de_piel_medio:','fr':':personne_qui_court_vers_la_droite_peau_légèrement_mate:','ja':':走る人_右向き_中間の肌色:','ko':':오른쪽을_향한_뛰는_사람_갈색_피부:','pt':':pessoa_correndo_de_frente_para_a_direita_pele_morena:','it':':persona_che_corre_a_destra_carnagione_olivastra:','fa':':دونده_به_سمت_راست_پوست_طلایی:','id':':orang_berlari_menghadap_kanan_warna_kulit_sedang:','zh':':跑步者面向右边_中等肤色:','ru':':человек_бежит_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_koşan_kişi_orta_cilt_tonu:','ar':':شخص_يجري_متجه_لليمين_بلون_معتدل:'},'\U0001F3C3\U0001F3FD\U0000200D\U000027A1':{'en':':person_running_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufende_person_nach_rechts_mittlere_hautfarbe:','es':':persona_corriendo_hacia_la_derecha_tono_de_piel_medio:','fr':':personne_qui_court_vers_la_droite_peau_légèrement_mate:','ja':':走る人_右向き_中間の肌色:','ko':':오른쪽을_향한_뛰는_사람_갈색_피부:','pt':':pessoa_correndo_de_frente_para_a_direita_pele_morena:','it':':persona_che_corre_a_destra_carnagione_olivastra:','fa':':دونده_به_سمت_راست_پوست_طلایی:','id':':orang_berlari_menghadap_kanan_warna_kulit_sedang:','zh':':跑步者面向右边_中等肤色:','ru':':человек_бежит_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_koşan_kişi_orta_cilt_tonu:','ar':':شخص_يجري_متجه_لليمين_بلون_معتدل:'},'\U0001F3C3\U0001F3FB':{'en':':person_running_light_skin_tone:','status':fully_qualified,'E':1,'de':':laufende_person_helle_hautfarbe:','es':':persona_corriendo_tono_de_piel_claro:','fr':':personne_qui_court_peau_claire:','ja':':走る人_薄い肌色:','ko':':뛰는_사람_하얀_피부:','pt':':pessoa_correndo_pele_clara:','it':':persona_che_corre_carnagione_chiara:','fa':':دونده_پوست_سفید:','id':':orang_berlari_warna_kulit_cerah:','zh':':跑步者_较浅肤色:','ru':':человек_бежит_очень_светлый_тон_кожи:','tr':':koşan_kişi_açık_cilt_tonu:','ar':':شخص_يجري_بشرة_بلون_فاتح:'},'\U0001F3C3\U0001F3FE':{'en':':person_running_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':laufende_person_mitteldunkle_hautfarbe:','es':':persona_corriendo_tono_de_piel_oscuro_medio:','fr':':personne_qui_court_peau_mate:','ja':':走る人_やや濃い肌色:','ko':':뛰는_사람_진한_갈색_피부:','pt':':pessoa_correndo_pele_morena_escura:','it':':persona_che_corre_carnagione_abbastanza_scura:','fa':':دونده_پوست_گندمی:','id':':orang_berlari_warna_kulit_gelap-sedang:','zh':':跑步者_中等深肤色:','ru':':человек_бежит_темный_тон_кожи:','tr':':koşan_kişi_orta_koyu_cilt_tonu:','ar':':شخص_يجري_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3C3\U0001F3FC':{'en':':person_running_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':laufende_person_mittelhelle_hautfarbe:','es':':persona_corriendo_tono_de_piel_claro_medio:','fr':':personne_qui_court_peau_moyennement_claire:','ja':':走る人_やや薄い肌色:','ko':':뛰는_사람_연한_갈색_피부:','pt':':pessoa_correndo_pele_morena_clara:','it':':persona_che_corre_carnagione_abbastanza_chiara:','fa':':دونده_پوست_روشن:','id':':orang_berlari_warna_kulit_cerah-sedang:','zh':':跑步者_中等浅肤色:','ru':':человек_бежит_светлый_тон_кожи:','tr':':koşan_kişi_orta_açık_cilt_tonu:','ar':':شخص_يجري_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3C3\U0001F3FD':{'en':':person_running_medium_skin_tone:','status':fully_qualified,'E':1,'de':':laufende_person_mittlere_hautfarbe:','es':':persona_corriendo_tono_de_piel_medio:','fr':':personne_qui_court_peau_légèrement_mate:','ja':':走る人_中間の肌色:','ko':':뛰는_사람_갈색_피부:','pt':':pessoa_correndo_pele_morena:','it':':persona_che_corre_carnagione_olivastra:','fa':':دونده_پوست_طلایی:','id':':orang_berlari_warna_kulit_sedang:','zh':':跑步者_中等肤色:','ru':':человек_бежит_средний_тон_кожи:','tr':':koşan_kişi_orta_cilt_tonu:','ar':':شخص_يجري_بشرة_بلون_معتدل:'},'\U0001F937':{'en':':person_shrugging:','status':fully_qualified,'E':3,'alias':[':shrug:'],'de':':schulterzuckende_person:','es':':persona_encogida_de_hombros:','fr':':personne_qui_hausse_les_épaules:','ja':':お手上げする人:','ko':':어깨를_으쓱하는_사람:','pt':':pessoa_dando_de_ombros:','it':':persona_che_scrolla_le_spalle:','fa':':شانه_بالا_انداختن:','id':':orang_mengangkat_bahu:','zh':':耸肩:','ru':':человек_пожимает_плечами:','tr':':omuz_silkme:','ar':':لا_مبالاة:'},'\U0001F937\U0001F3FF':{'en':':person_shrugging_dark_skin_tone:','status':fully_qualified,'E':3,'de':':schulterzuckende_person_dunkle_hautfarbe:','es':':persona_encogida_de_hombros_tono_de_piel_oscuro:','fr':':personne_qui_hausse_les_épaules_peau_foncée:','ja':':お手上げする人_濃い肌色:','ko':':어깨를_으쓱하는_사람_검은색_피부:','pt':':pessoa_dando_de_ombros_pele_escura:','it':':persona_che_scrolla_le_spalle_carnagione_scura:','fa':':شانه_بالا_انداختن_پوست_آبنوسی:','id':':orang_mengangkat_bahu_warna_kulit_gelap:','zh':':耸肩_较深肤色:','ru':':человек_пожимает_плечами_очень_темный_тон_кожи:','tr':':omuz_silkme_koyu_cilt_tonu:','ar':':لا_مبالاة_بشرة_بلون_غامق:'},'\U0001F937\U0001F3FB':{'en':':person_shrugging_light_skin_tone:','status':fully_qualified,'E':3,'de':':schulterzuckende_person_helle_hautfarbe:','es':':persona_encogida_de_hombros_tono_de_piel_claro:','fr':':personne_qui_hausse_les_épaules_peau_claire:','ja':':お手上げする人_薄い肌色:','ko':':어깨를_으쓱하는_사람_하얀_피부:','pt':':pessoa_dando_de_ombros_pele_clara:','it':':persona_che_scrolla_le_spalle_carnagione_chiara:','fa':':شانه_بالا_انداختن_پوست_سفید:','id':':orang_mengangkat_bahu_warna_kulit_cerah:','zh':':耸肩_较浅肤色:','ru':':человек_пожимает_плечами_очень_светлый_тон_кожи:','tr':':omuz_silkme_açık_cilt_tonu:','ar':':لا_مبالاة_بشرة_بلون_فاتح:'},'\U0001F937\U0001F3FE':{'en':':person_shrugging_medium-dark_skin_tone:','status':fully_qualified,'E':3,'de':':schulterzuckende_person_mitteldunkle_hautfarbe:','es':':persona_encogida_de_hombros_tono_de_piel_oscuro_medio:','fr':':personne_qui_hausse_les_épaules_peau_mate:','ja':':お手上げする人_やや濃い肌色:','ko':':어깨를_으쓱하는_사람_진한_갈색_피부:','pt':':pessoa_dando_de_ombros_pele_morena_escura:','it':':persona_che_scrolla_le_spalle_carnagione_abbastanza_scura:','fa':':شانه_بالا_انداختن_پوست_گندمی:','id':':orang_mengangkat_bahu_warna_kulit_gelap-sedang:','zh':':耸肩_中等深肤色:','ru':':человек_пожимает_плечами_темный_тон_кожи:','tr':':omuz_silkme_orta_koyu_cilt_tonu:','ar':':لا_مبالاة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F937\U0001F3FC':{'en':':person_shrugging_medium-light_skin_tone:','status':fully_qualified,'E':3,'de':':schulterzuckende_person_mittelhelle_hautfarbe:','es':':persona_encogida_de_hombros_tono_de_piel_claro_medio:','fr':':personne_qui_hausse_les_épaules_peau_moyennement_claire:','ja':':お手上げする人_やや薄い肌色:','ko':':어깨를_으쓱하는_사람_연한_갈색_피부:','pt':':pessoa_dando_de_ombros_pele_morena_clara:','it':':persona_che_scrolla_le_spalle_carnagione_abbastanza_chiara:','fa':':شانه_بالا_انداختن_پوست_روشن:','id':':orang_mengangkat_bahu_warna_kulit_cerah-sedang:','zh':':耸肩_中等浅肤色:','ru':':человек_пожимает_плечами_светлый_тон_кожи:','tr':':omuz_silkme_orta_açık_cilt_tonu:','ar':':لا_مبالاة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F937\U0001F3FD':{'en':':person_shrugging_medium_skin_tone:','status':fully_qualified,'E':3,'de':':schulterzuckende_person_mittlere_hautfarbe:','es':':persona_encogida_de_hombros_tono_de_piel_medio:','fr':':personne_qui_hausse_les_épaules_peau_légèrement_mate:','ja':':お手上げする人_中間の肌色:','ko':':어깨를_으쓱하는_사람_갈색_피부:','pt':':pessoa_dando_de_ombros_pele_morena:','it':':persona_che_scrolla_le_spalle_carnagione_olivastra:','fa':':شانه_بالا_انداختن_پوست_طلایی:','id':':orang_mengangkat_bahu_warna_kulit_sedang:','zh':':耸肩_中等肤色:','ru':':человек_пожимает_плечами_средний_тон_кожи:','tr':':omuz_silkme_orta_cilt_tonu:','ar':':لا_مبالاة_بشرة_بلون_معتدل:'},'\U0001F9CD':{'en':':person_standing:','status':fully_qualified,'E':12,'alias':[':standing_person:'],'de':':stehende_person:','es':':persona_de_pie:','fr':':personne_debout:','ja':':立つ人:','ko':':서_있는_사람:','pt':':pessoa_em_pé:','it':':persona_in_piedi:','fa':':آدم_ایستاده:','id':':orang_berdiri:','zh':':站立者:','ru':':человек_стоит:','tr':':ayakta_duran_kişi:','ar':':شخص_واقف:'},'\U0001F9CD\U0001F3FF':{'en':':person_standing_dark_skin_tone:','status':fully_qualified,'E':12,'de':':stehende_person_dunkle_hautfarbe:','es':':persona_de_pie_tono_de_piel_oscuro:','fr':':personne_debout_peau_foncée:','ja':':立つ人_濃い肌色:','ko':':서_있는_사람_검은색_피부:','pt':':pessoa_em_pé_pele_escura:','it':':persona_in_piedi_carnagione_scura:','fa':':آدم_ایستاده_پوست_آبنوسی:','id':':orang_berdiri_warna_kulit_gelap:','zh':':站立者_较深肤色:','ru':':человек_стоит_очень_темный_тон_кожи:','tr':':ayakta_duran_kişi_koyu_cilt_tonu:','ar':':شخص_واقف_بشرة_بلون_غامق:'},'\U0001F9CD\U0001F3FB':{'en':':person_standing_light_skin_tone:','status':fully_qualified,'E':12,'de':':stehende_person_helle_hautfarbe:','es':':persona_de_pie_tono_de_piel_claro:','fr':':personne_debout_peau_claire:','ja':':立つ人_薄い肌色:','ko':':서_있는_사람_하얀_피부:','pt':':pessoa_em_pé_pele_clara:','it':':persona_in_piedi_carnagione_chiara:','fa':':آدم_ایستاده_پوست_سفید:','id':':orang_berdiri_warna_kulit_cerah:','zh':':站立者_较浅肤色:','ru':':человек_стоит_очень_светлый_тон_кожи:','tr':':ayakta_duran_kişi_açık_cilt_tonu:','ar':':شخص_واقف_بشرة_بلون_فاتح:'},'\U0001F9CD\U0001F3FE':{'en':':person_standing_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':stehende_person_mitteldunkle_hautfarbe:','es':':persona_de_pie_tono_de_piel_oscuro_medio:','fr':':personne_debout_peau_mate:','ja':':立つ人_やや濃い肌色:','ko':':서_있는_사람_진한_갈색_피부:','pt':':pessoa_em_pé_pele_morena_escura:','it':':persona_in_piedi_carnagione_abbastanza_scura:','fa':':آدم_ایستاده_پوست_گندمی:','id':':orang_berdiri_warna_kulit_gelap-sedang:','zh':':站立者_中等深肤色:','ru':':человек_стоит_темный_тон_кожи:','tr':':ayakta_duran_kişi_orta_koyu_cilt_tonu:','ar':':شخص_واقف_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9CD\U0001F3FC':{'en':':person_standing_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':stehende_person_mittelhelle_hautfarbe:','es':':persona_de_pie_tono_de_piel_claro_medio:','fr':':personne_debout_peau_moyennement_claire:','ja':':立つ人_やや薄い肌色:','ko':':서_있는_사람_연한_갈색_피부:','pt':':pessoa_em_pé_pele_morena_clara:','it':':persona_in_piedi_carnagione_abbastanza_chiara:','fa':':آدم_ایستاده_پوست_روشن:','id':':orang_berdiri_warna_kulit_cerah-sedang:','zh':':站立者_中等浅肤色:','ru':':человек_стоит_светлый_тон_кожи:','tr':':ayakta_duran_kişi_orta_açık_cilt_tonu:','ar':':شخص_واقف_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9CD\U0001F3FD':{'en':':person_standing_medium_skin_tone:','status':fully_qualified,'E':12,'de':':stehende_person_mittlere_hautfarbe:','es':':persona_de_pie_tono_de_piel_medio:','fr':':personne_debout_peau_légèrement_mate:','ja':':立つ人_中間の肌色:','ko':':서_있는_사람_갈색_피부:','pt':':pessoa_em_pé_pele_morena:','it':':persona_in_piedi_carnagione_olivastra:','fa':':آدم_ایستاده_پوست_طلایی:','id':':orang_berdiri_warna_kulit_sedang:','zh':':站立者_中等肤色:','ru':':человек_стоит_средний_тон_кожи:','tr':':ayakta_duran_kişi_orta_cilt_tonu:','ar':':شخص_واقف_بشرة_بلون_معتدل:'},'\U0001F3C4':{'en':':person_surfing:','status':fully_qualified,'E':0.6,'alias':[':surfer:'],'variant':True,'de':':surfer(in):','es':':persona_haciendo_surf:','fr':':personne_faisant_du_surf:','ja':':サーフィンする人:','ko':':서핑하는_사람:','pt':':surfista:','it':':persona_che_fa_surf:','fa':':موج_سوار:','id':':orang_berselancar:','zh':':冲浪:','ru':':серфинг:','tr':':sörf_yapan_kişi:','ar':':شخص_يركب_الأمواج:'},'\U0001F3C4\U0001F3FF':{'en':':person_surfing_dark_skin_tone:','status':fully_qualified,'E':1,'de':':surfer(in)_dunkle_hautfarbe:','es':':persona_haciendo_surf_tono_de_piel_oscuro:','fr':':personne_faisant_du_surf_peau_foncée:','ja':':サーフィンする人_濃い肌色:','ko':':서핑하는_사람_검은색_피부:','pt':':surfista_pele_escura:','it':':persona_che_fa_surf_carnagione_scura:','fa':':موج_سوار_پوست_آبنوسی:','id':':orang_berselancar_warna_kulit_gelap:','zh':':冲浪_较深肤色:','ru':':серфинг_очень_темный_тон_кожи:','tr':':sörf_yapan_kişi_koyu_cilt_tonu:','ar':':شخص_يركب_الأمواج_بشرة_بلون_غامق:'},'\U0001F3C4\U0001F3FB':{'en':':person_surfing_light_skin_tone:','status':fully_qualified,'E':1,'de':':surfer(in)_helle_hautfarbe:','es':':persona_haciendo_surf_tono_de_piel_claro:','fr':':personne_faisant_du_surf_peau_claire:','ja':':サーフィンする人_薄い肌色:','ko':':서핑하는_사람_하얀_피부:','pt':':surfista_pele_clara:','it':':persona_che_fa_surf_carnagione_chiara:','fa':':موج_سوار_پوست_سفید:','id':':orang_berselancar_warna_kulit_cerah:','zh':':冲浪_较浅肤色:','ru':':серфинг_очень_светлый_тон_кожи:','tr':':sörf_yapan_kişi_açık_cilt_tonu:','ar':':شخص_يركب_الأمواج_بشرة_بلون_فاتح:'},'\U0001F3C4\U0001F3FE':{'en':':person_surfing_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':surfer(in)_mitteldunkle_hautfarbe:','es':':persona_haciendo_surf_tono_de_piel_oscuro_medio:','fr':':personne_faisant_du_surf_peau_mate:','ja':':サーフィンする人_やや濃い肌色:','ko':':서핑하는_사람_진한_갈색_피부:','pt':':surfista_pele_morena_escura:','it':':persona_che_fa_surf_carnagione_abbastanza_scura:','fa':':موج_سوار_پوست_گندمی:','id':':orang_berselancar_warna_kulit_gelap-sedang:','zh':':冲浪_中等深肤色:','ru':':серфинг_темный_тон_кожи:','tr':':sörf_yapan_kişi_orta_koyu_cilt_tonu:','ar':':شخص_يركب_الأمواج_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3C4\U0001F3FC':{'en':':person_surfing_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':surfer(in)_mittelhelle_hautfarbe:','es':':persona_haciendo_surf_tono_de_piel_claro_medio:','fr':':personne_faisant_du_surf_peau_moyennement_claire:','ja':':サーフィンする人_やや薄い肌色:','ko':':서핑하는_사람_연한_갈색_피부:','pt':':surfista_pele_morena_clara:','it':':persona_che_fa_surf_carnagione_abbastanza_chiara:','fa':':موج_سوار_پوست_روشن:','id':':orang_berselancar_warna_kulit_cerah-sedang:','zh':':冲浪_中等浅肤色:','ru':':серфинг_светлый_тон_кожи:','tr':':sörf_yapan_kişi_orta_açık_cilt_tonu:','ar':':شخص_يركب_الأمواج_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3C4\U0001F3FD':{'en':':person_surfing_medium_skin_tone:','status':fully_qualified,'E':1,'de':':surfer(in)_mittlere_hautfarbe:','es':':persona_haciendo_surf_tono_de_piel_medio:','fr':':personne_faisant_du_surf_peau_légèrement_mate:','ja':':サーフィンする人_中間の肌色:','ko':':서핑하는_사람_갈색_피부:','pt':':surfista_pele_morena:','it':':persona_che_fa_surf_carnagione_olivastra:','fa':':موج_سوار_پوست_طلایی:','id':':orang_berselancar_warna_kulit_sedang:','zh':':冲浪_中等肤色:','ru':':серфинг_средний_тон_кожи:','tr':':sörf_yapan_kişi_orta_cilt_tonu:','ar':':شخص_يركب_الأمواج_بشرة_بلون_معتدل:'},'\U0001F3CA':{'en':':person_swimming:','status':fully_qualified,'E':0.6,'alias':[':swimmer:'],'variant':True,'de':':schwimmer(in):','es':':persona_nadando:','fr':':personne_nageant:','ja':':泳ぐ人:','ko':':수영하는_사람:','pt':':pessoa_nadando:','it':':persona_che_nuota:','fa':':شناگر:','id':':orang_berenang:','zh':':游泳:','ru':':плавание:','tr':':yüzen_kişi:','ar':':شخص_يسبح:'},'\U0001F3CA\U0001F3FF':{'en':':person_swimming_dark_skin_tone:','status':fully_qualified,'E':1,'de':':schwimmer(in)_dunkle_hautfarbe:','es':':persona_nadando_tono_de_piel_oscuro:','fr':':personne_nageant_peau_foncée:','ja':':泳ぐ人_濃い肌色:','ko':':수영하는_사람_검은색_피부:','pt':':pessoa_nadando_pele_escura:','it':':persona_che_nuota_carnagione_scura:','fa':':شناگر_پوست_آبنوسی:','id':':orang_berenang_warna_kulit_gelap:','zh':':游泳_较深肤色:','ru':':плавание_очень_темный_тон_кожи:','tr':':yüzen_kişi_koyu_cilt_tonu:','ar':':شخص_يسبح_بشرة_بلون_غامق:'},'\U0001F3CA\U0001F3FB':{'en':':person_swimming_light_skin_tone:','status':fully_qualified,'E':1,'de':':schwimmer(in)_helle_hautfarbe:','es':':persona_nadando_tono_de_piel_claro:','fr':':personne_nageant_peau_claire:','ja':':泳ぐ人_薄い肌色:','ko':':수영하는_사람_하얀_피부:','pt':':pessoa_nadando_pele_clara:','it':':persona_che_nuota_carnagione_chiara:','fa':':شناگر_پوست_سفید:','id':':orang_berenang_warna_kulit_cerah:','zh':':游泳_较浅肤色:','ru':':плавание_очень_светлый_тон_кожи:','tr':':yüzen_kişi_açık_cilt_tonu:','ar':':شخص_يسبح_بشرة_بلون_فاتح:'},'\U0001F3CA\U0001F3FE':{'en':':person_swimming_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':schwimmer(in)_mitteldunkle_hautfarbe:','es':':persona_nadando_tono_de_piel_oscuro_medio:','fr':':personne_nageant_peau_mate:','ja':':泳ぐ人_やや濃い肌色:','ko':':수영하는_사람_진한_갈색_피부:','pt':':pessoa_nadando_pele_morena_escura:','it':':persona_che_nuota_carnagione_abbastanza_scura:','fa':':شناگر_پوست_گندمی:','id':':orang_berenang_warna_kulit_gelap-sedang:','zh':':游泳_中等深肤色:','ru':':плавание_темный_тон_кожи:','tr':':yüzen_kişi_orta_koyu_cilt_tonu:','ar':':شخص_يسبح_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3CA\U0001F3FC':{'en':':person_swimming_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':schwimmer(in)_mittelhelle_hautfarbe:','es':':persona_nadando_tono_de_piel_claro_medio:','fr':':personne_nageant_peau_moyennement_claire:','ja':':泳ぐ人_やや薄い肌色:','ko':':수영하는_사람_연한_갈색_피부:','pt':':pessoa_nadando_pele_morena_clara:','it':':persona_che_nuota_carnagione_abbastanza_chiara:','fa':':شناگر_پوست_روشن:','id':':orang_berenang_warna_kulit_cerah-sedang:','zh':':游泳_中等浅肤色:','ru':':плавание_светлый_тон_кожи:','tr':':yüzen_kişi_orta_açık_cilt_tonu:','ar':':شخص_يسبح_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3CA\U0001F3FD':{'en':':person_swimming_medium_skin_tone:','status':fully_qualified,'E':1,'de':':schwimmer(in)_mittlere_hautfarbe:','es':':persona_nadando_tono_de_piel_medio:','fr':':personne_nageant_peau_légèrement_mate:','ja':':泳ぐ人_中間の肌色:','ko':':수영하는_사람_갈색_피부:','pt':':pessoa_nadando_pele_morena:','it':':persona_che_nuota_carnagione_olivastra:','fa':':شناگر_پوست_طلایی:','id':':orang_berenang_warna_kulit_sedang:','zh':':游泳_中等肤色:','ru':':плавание_средний_тон_кожи:','tr':':yüzen_kişi_orta_cilt_tonu:','ar':':شخص_يسبح_بشرة_بلون_معتدل:'},'\U0001F6C0':{'en':':person_taking_bath:','status':fully_qualified,'E':0.6,'alias':[':bath:'],'de':':badende_person:','es':':persona_en_la_bañera:','fr':':personne_prenant_un_bain:','ja':':風呂に入る人:','ko':':목욕하는_사람:','pt':':pessoa_tomando_banho:','it':':persona_che_fa_il_bagno:','fa':':فرد_در_وان:','id':':orang_di_bak_mandi:','zh':':洗澡的人:','ru':':человек_в_ванне:','tr':':banyo_yapan_kişi:','ar':':شخص_يستحم:'},'\U0001F6C0\U0001F3FF':{'en':':person_taking_bath_dark_skin_tone:','status':fully_qualified,'E':1,'de':':badende_person_dunkle_hautfarbe:','es':':persona_en_la_bañera_tono_de_piel_oscuro:','fr':':personne_prenant_un_bain_peau_foncée:','ja':':風呂に入る人_濃い肌色:','ko':':목욕하는_사람_검은색_피부:','pt':':pessoa_tomando_banho_pele_escura:','it':':persona_che_fa_il_bagno_carnagione_scura:','fa':':فرد_در_وان_پوست_آبنوسی:','id':':orang_di_bak_mandi_warna_kulit_gelap:','zh':':洗澡的人_较深肤色:','ru':':человек_в_ванне_очень_темный_тон_кожи:','tr':':banyo_yapan_kişi_koyu_cilt_tonu:','ar':':شخص_يستحم_بشرة_بلون_غامق:'},'\U0001F6C0\U0001F3FB':{'en':':person_taking_bath_light_skin_tone:','status':fully_qualified,'E':1,'de':':badende_person_helle_hautfarbe:','es':':persona_en_la_bañera_tono_de_piel_claro:','fr':':personne_prenant_un_bain_peau_claire:','ja':':風呂に入る人_薄い肌色:','ko':':목욕하는_사람_하얀_피부:','pt':':pessoa_tomando_banho_pele_clara:','it':':persona_che_fa_il_bagno_carnagione_chiara:','fa':':فرد_در_وان_پوست_سفید:','id':':orang_di_bak_mandi_warna_kulit_cerah:','zh':':洗澡的人_较浅肤色:','ru':':человек_в_ванне_очень_светлый_тон_кожи:','tr':':banyo_yapan_kişi_açık_cilt_tonu:','ar':':شخص_يستحم_بشرة_بلون_فاتح:'},'\U0001F6C0\U0001F3FE':{'en':':person_taking_bath_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':badende_person_mitteldunkle_hautfarbe:','es':':persona_en_la_bañera_tono_de_piel_oscuro_medio:','fr':':personne_prenant_un_bain_peau_mate:','ja':':風呂に入る人_やや濃い肌色:','ko':':목욕하는_사람_진한_갈색_피부:','pt':':pessoa_tomando_banho_pele_morena_escura:','it':':persona_che_fa_il_bagno_carnagione_abbastanza_scura:','fa':':فرد_در_وان_پوست_گندمی:','id':':orang_di_bak_mandi_warna_kulit_gelap-sedang:','zh':':洗澡的人_中等深肤色:','ru':':человек_в_ванне_темный_тон_кожи:','tr':':banyo_yapan_kişi_orta_koyu_cilt_tonu:','ar':':شخص_يستحم_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6C0\U0001F3FC':{'en':':person_taking_bath_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':badende_person_mittelhelle_hautfarbe:','es':':persona_en_la_bañera_tono_de_piel_claro_medio:','fr':':personne_prenant_un_bain_peau_moyennement_claire:','ja':':風呂に入る人_やや薄い肌色:','ko':':목욕하는_사람_연한_갈색_피부:','pt':':pessoa_tomando_banho_pele_morena_clara:','it':':persona_che_fa_il_bagno_carnagione_abbastanza_chiara:','fa':':فرد_در_وان_پوست_روشن:','id':':orang_di_bak_mandi_warna_kulit_cerah-sedang:','zh':':洗澡的人_中等浅肤色:','ru':':человек_в_ванне_светлый_тон_кожи:','tr':':banyo_yapan_kişi_orta_açık_cilt_tonu:','ar':':شخص_يستحم_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6C0\U0001F3FD':{'en':':person_taking_bath_medium_skin_tone:','status':fully_qualified,'E':1,'de':':badende_person_mittlere_hautfarbe:','es':':persona_en_la_bañera_tono_de_piel_medio:','fr':':personne_prenant_un_bain_peau_légèrement_mate:','ja':':風呂に入る人_中間の肌色:','ko':':목욕하는_사람_갈색_피부:','pt':':pessoa_tomando_banho_pele_morena:','it':':persona_che_fa_il_bagno_carnagione_olivastra:','fa':':فرد_در_وان_پوست_طلایی:','id':':orang_di_bak_mandi_warna_kulit_sedang:','zh':':洗澡的人_中等肤色:','ru':':человек_в_ванне_средний_тон_кожи:','tr':':banyo_yapan_kişi_orta_cilt_tonu:','ar':':شخص_يستحم_بشرة_بلون_معتدل:'},'\U0001F481':{'en':':person_tipping_hand:','status':fully_qualified,'E':0.6,'alias':[':tipping_hand_person:', ':information_desk_person:'],'de':':infoschalter-mitarbeiter(in):','es':':persona_de_mostrador_de_información:','fr':':personne_paume_vers_le_haut:','ja':':案内する人:','ko':':손바닥_꺾은_사람:','pt':':pessoa_com_a_palma_virada_para_cima:','it':':persona_al_punto_informazioni:','fa':':آماده_کمک:','id':':orang_menengadahkan_tangan:','zh':':前台:','ru':':человек_тянет_руку:','tr':':elini_yana_yatırma:','ar':':شخص_بيد_مائلة:'},'\U0001F481\U0001F3FF':{'en':':person_tipping_hand_dark_skin_tone:','status':fully_qualified,'E':1,'de':':infoschalter-mitarbeiter(in)_dunkle_hautfarbe:','es':':persona_de_mostrador_de_información_tono_de_piel_oscuro:','fr':':personne_paume_vers_le_haut_peau_foncée:','ja':':案内する人_濃い肌色:','ko':':손바닥_꺾은_사람_검은색_피부:','pt':':pessoa_com_a_palma_virada_para_cima_pele_escura:','it':':persona_al_punto_informazioni_carnagione_scura:','fa':':آماده_کمک_پوست_آبنوسی:','id':':orang_menengadahkan_tangan_warna_kulit_gelap:','zh':':前台_较深肤色:','ru':':человек_тянет_руку_очень_темный_тон_кожи:','tr':':elini_yana_yatırma_koyu_cilt_tonu:','ar':':شخص_بيد_مائلة_بشرة_بلون_غامق:'},'\U0001F481\U0001F3FB':{'en':':person_tipping_hand_light_skin_tone:','status':fully_qualified,'E':1,'de':':infoschalter-mitarbeiter(in)_helle_hautfarbe:','es':':persona_de_mostrador_de_información_tono_de_piel_claro:','fr':':personne_paume_vers_le_haut_peau_claire:','ja':':案内する人_薄い肌色:','ko':':손바닥_꺾은_사람_하얀_피부:','pt':':pessoa_com_a_palma_virada_para_cima_pele_clara:','it':':persona_al_punto_informazioni_carnagione_chiara:','fa':':آماده_کمک_پوست_سفید:','id':':orang_menengadahkan_tangan_warna_kulit_cerah:','zh':':前台_较浅肤色:','ru':':человек_тянет_руку_очень_светлый_тон_кожи:','tr':':elini_yana_yatırma_açık_cilt_tonu:','ar':':شخص_بيد_مائلة_بشرة_بلون_فاتح:'},'\U0001F481\U0001F3FE':{'en':':person_tipping_hand_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':infoschalter-mitarbeiter(in)_mitteldunkle_hautfarbe:','es':':persona_de_mostrador_de_información_tono_de_piel_oscuro_medio:','fr':':personne_paume_vers_le_haut_peau_mate:','ja':':案内する人_やや濃い肌色:','ko':':손바닥_꺾은_사람_진한_갈색_피부:','pt':':pessoa_com_a_palma_virada_para_cima_pele_morena_escura:','it':':persona_al_punto_informazioni_carnagione_abbastanza_scura:','fa':':آماده_کمک_پوست_گندمی:','id':':orang_menengadahkan_tangan_warna_kulit_gelap-sedang:','zh':':前台_中等深肤色:','ru':':человек_тянет_руку_темный_тон_кожи:','tr':':elini_yana_yatırma_orta_koyu_cilt_tonu:','ar':':شخص_بيد_مائلة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F481\U0001F3FC':{'en':':person_tipping_hand_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':infoschalter-mitarbeiter(in)_mittelhelle_hautfarbe:','es':':persona_de_mostrador_de_información_tono_de_piel_claro_medio:','fr':':personne_paume_vers_le_haut_peau_moyennement_claire:','ja':':案内する人_やや薄い肌色:','ko':':손바닥_꺾은_사람_연한_갈색_피부:','pt':':pessoa_com_a_palma_virada_para_cima_pele_morena_clara:','it':':persona_al_punto_informazioni_carnagione_abbastanza_chiara:','fa':':آماده_کمک_پوست_روشن:','id':':orang_menengadahkan_tangan_warna_kulit_cerah-sedang:','zh':':前台_中等浅肤色:','ru':':человек_тянет_руку_светлый_тон_кожи:','tr':':elini_yana_yatırma_orta_açık_cilt_tonu:','ar':':شخص_بيد_مائلة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F481\U0001F3FD':{'en':':person_tipping_hand_medium_skin_tone:','status':fully_qualified,'E':1,'de':':infoschalter-mitarbeiter(in)_mittlere_hautfarbe:','es':':persona_de_mostrador_de_información_tono_de_piel_medio:','fr':':personne_paume_vers_le_haut_peau_légèrement_mate:','ja':':案内する人_中間の肌色:','ko':':손바닥_꺾은_사람_갈색_피부:','pt':':pessoa_com_a_palma_virada_para_cima_pele_morena:','it':':persona_al_punto_informazioni_carnagione_olivastra:','fa':':آماده_کمک_پوست_طلایی:','id':':orang_menengadahkan_tangan_warna_kulit_sedang:','zh':':前台_中等肤色:','ru':':человек_тянет_руку_средний_тон_кожи:','tr':':elini_yana_yatırma_orta_cilt_tonu:','ar':':شخص_بيد_مائلة_بشرة_بلون_معتدل:'},'\U0001F6B6':{'en':':person_walking:','status':fully_qualified,'E':0.6,'alias':[':walking:'],'de':':fußgänger(in):','es':':persona_caminando:','fr':':personne_qui_marche:','ja':':歩く人:','ko':':보행자:','pt':':pessoa_andando:','it':':pedone:','fa':':عابر_پیاده:','id':':orang_berjalan_kaki:','zh':':行人:','ru':':человек_идет:','tr':':yürüyen_kişi:','ar':':شخص_يسير:'},'\U0001F6B6\U0001F3FF':{'en':':person_walking_dark_skin_tone:','status':fully_qualified,'E':1,'de':':fußgänger(in)_dunkle_hautfarbe:','es':':persona_caminando_tono_de_piel_oscuro:','fr':':personne_qui_marche_peau_foncée:','ja':':歩く人_濃い肌色:','ko':':보행자_검은색_피부:','pt':':pessoa_andando_pele_escura:','it':':pedone_carnagione_scura:','fa':':عابر_پیاده_پوست_آبنوسی:','id':':orang_berjalan_kaki_warna_kulit_gelap:','zh':':行人_较深肤色:','ru':':человек_идет_очень_темный_тон_кожи:','tr':':yürüyen_kişi_koyu_cilt_tonu:','ar':':شخص_يسير_بشرة_بلون_غامق:'},'\U0001F6B6\U0000200D\U000027A1\U0000FE0F':{'en':':person_walking_facing_right:','status':fully_qualified,'E':15.1,'de':':fußgänger(in)_nach_rechts:','es':':persona_caminando_hacia_la_derecha:','fr':':personne_qui_marche_vers_la_droite:','ja':':歩く人_右向き:','ko':':오른쪽을_향한_보행자:','pt':':pessoa_andando_de_frente_para_a_direita:','it':':pedone_a_destra:','fa':':عابر_پیاده_به_سمت_راست:','id':':orang_berjalan_kaki_menghadap_kanan:','zh':':行人面向右边:','ru':':человек_идет_направленные_вправо:','tr':':sağa_dönük_yürüyen_kişi:','ar':':شخص_يسير_متجه_لليمين:'},'\U0001F6B6\U0000200D\U000027A1':{'en':':person_walking_facing_right:','status':minimally_qualified,'E':15.1,'de':':fußgänger(in)_nach_rechts:','es':':persona_caminando_hacia_la_derecha:','fr':':personne_qui_marche_vers_la_droite:','ja':':歩く人_右向き:','ko':':오른쪽을_향한_보행자:','pt':':pessoa_andando_de_frente_para_a_direita:','it':':pedone_a_destra:','fa':':عابر_پیاده_به_سمت_راست:','id':':orang_berjalan_kaki_menghadap_kanan:','zh':':行人面向右边:','ru':':человек_идет_направленные_вправо:','tr':':sağa_dönük_yürüyen_kişi:','ar':':شخص_يسير_متجه_لليمين:'},'\U0001F6B6\U0001F3FF\U0000200D\U000027A1\U0000FE0F':{'en':':person_walking_facing_right_dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':fußgänger(in)_nach_rechts_dunkle_hautfarbe:','es':':persona_caminando_hacia_la_derecha_tono_de_piel_oscuro:','fr':':personne_qui_marche_vers_la_droite_peau_foncée:','ja':':歩く人_右向き_濃い肌色:','ko':':오른쪽을_향한_보행자_검은색_피부:','pt':':pessoa_andando_de_frente_para_a_direita_pele_escura:','it':':pedone_a_destra_carnagione_scura:','fa':':عابر_پیاده_به_سمت_راستپوست_آبنوسی:','id':':orang_berjalan_kaki_menghadap_kanan_warna_kulit_gelap:','zh':':行人面向右边_较深肤色:','ru':':человек_идет_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_yürüyen_kişi_koyu_cilt_tonu:','ar':':شخص_يسير_متجه_لليمين_بلون_غامق:'},'\U0001F6B6\U0001F3FF\U0000200D\U000027A1':{'en':':person_walking_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgänger(in)_nach_rechts_dunkle_hautfarbe:','es':':persona_caminando_hacia_la_derecha_tono_de_piel_oscuro:','fr':':personne_qui_marche_vers_la_droite_peau_foncée:','ja':':歩く人_右向き_濃い肌色:','ko':':오른쪽을_향한_보행자_검은색_피부:','pt':':pessoa_andando_de_frente_para_a_direita_pele_escura:','it':':pedone_a_destra_carnagione_scura:','fa':':عابر_پیاده_به_سمت_راستپوست_آبنوسی:','id':':orang_berjalan_kaki_menghadap_kanan_warna_kulit_gelap:','zh':':行人面向右边_较深肤色:','ru':':человек_идет_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_yürüyen_kişi_koyu_cilt_tonu:','ar':':شخص_يسير_متجه_لليمين_بلون_غامق:'},'\U0001F6B6\U0001F3FB\U0000200D\U000027A1\U0000FE0F':{'en':':person_walking_facing_right_light_skin_tone:','status':fully_qualified,'E':15.1,'de':':fußgänger(in)_nach_rechts_helle_hautfarbe:','es':':persona_caminando_hacia_la_derecha_tono_de_piel_claro:','fr':':personne_qui_marche_vers_la_droite_peau_claire:','ja':':歩く人_右向き_薄い肌色:','ko':':오른쪽을_향한_보행자_하얀_피부:','pt':':pessoa_andando_de_frente_para_a_direita_pele_clara:','it':':pedone_a_destra_carnagione_chiara:','fa':':عابر_پیاده_به_سمت_راست_پوست_سفید:','id':':orang_berjalan_kaki_menghadap_kanan_warna_kulit_cerah:','zh':':行人面向右边_较浅肤色:','ru':':человек_идет_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_yürüyen_kişi_açık_cilt_tonu:','ar':':شخص_يسير_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F6B6\U0001F3FB\U0000200D\U000027A1':{'en':':person_walking_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgänger(in)_nach_rechts_helle_hautfarbe:','es':':persona_caminando_hacia_la_derecha_tono_de_piel_claro:','fr':':personne_qui_marche_vers_la_droite_peau_claire:','ja':':歩く人_右向き_薄い肌色:','ko':':오른쪽을_향한_보행자_하얀_피부:','pt':':pessoa_andando_de_frente_para_a_direita_pele_clara:','it':':pedone_a_destra_carnagione_chiara:','fa':':عابر_پیاده_به_سمت_راست_پوست_سفید:','id':':orang_berjalan_kaki_menghadap_kanan_warna_kulit_cerah:','zh':':行人面向右边_较浅肤色:','ru':':человек_идет_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_yürüyen_kişi_açık_cilt_tonu:','ar':':شخص_يسير_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F6B6\U0001F3FE\U0000200D\U000027A1\U0000FE0F':{'en':':person_walking_facing_right_medium-dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':fußgänger(in)_nach_rechts_mitteldunkle_hautfarbe:','es':':persona_caminando_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':personne_qui_marche_vers_la_droite_peau_mate:','ja':':歩く人_右向き_やや濃い肌色:','ko':':오른쪽을_향한_보행자_진한_갈색_피부:','pt':':pessoa_andando_de_frente_para_a_direita_pele_morena_escura:','it':':pedone_a_destra_carnagione_abbastanza_scura:','fa':':عابر_پیاده_به_سمت_راست_پوست_گندمی:','id':':orang_berjalan_kaki_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':行人面向右边_中等深肤色:','ru':':человек_идет_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_yürüyen_kişi_orta_koyu_cilt_tonu:','ar':':شخص_يسير_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F6B6\U0001F3FE\U0000200D\U000027A1':{'en':':person_walking_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgänger(in)_nach_rechts_mitteldunkle_hautfarbe:','es':':persona_caminando_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':personne_qui_marche_vers_la_droite_peau_mate:','ja':':歩く人_右向き_やや濃い肌色:','ko':':오른쪽을_향한_보행자_진한_갈색_피부:','pt':':pessoa_andando_de_frente_para_a_direita_pele_morena_escura:','it':':pedone_a_destra_carnagione_abbastanza_scura:','fa':':عابر_پیاده_به_سمت_راست_پوست_گندمی:','id':':orang_berjalan_kaki_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':行人面向右边_中等深肤色:','ru':':человек_идет_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_yürüyen_kişi_orta_koyu_cilt_tonu:','ar':':شخص_يسير_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F6B6\U0001F3FC\U0000200D\U000027A1\U0000FE0F':{'en':':person_walking_facing_right_medium-light_skin_tone:','status':fully_qualified,'E':15.1,'de':':fußgänger(in)_nach_rechts_mittelhelle_hautfarbe:','es':':persona_caminando_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':personne_qui_marche_vers_la_droite_peau_moyennement_claire:','ja':':歩く人_右向き_やや薄い肌色:','ko':':오른쪽을_향한_보행자_연한_갈색_피부:','pt':':pessoa_andando_de_frente_para_a_direita_pele_morena_clara:','it':':pedone_a_destra_carnagione_abbastanza_chiara:','fa':':عابر_پیاده_به_سمت_راست_پوست_روشن:','id':':orang_berjalan_kaki_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':行人面向右边_中等浅肤色:','ru':':человек_идет_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_yürüyen_kişi_orta_açık_cilt_tonu:','ar':':شخص_يسير_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F6B6\U0001F3FC\U0000200D\U000027A1':{'en':':person_walking_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgänger(in)_nach_rechts_mittelhelle_hautfarbe:','es':':persona_caminando_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':personne_qui_marche_vers_la_droite_peau_moyennement_claire:','ja':':歩く人_右向き_やや薄い肌色:','ko':':오른쪽을_향한_보행자_연한_갈색_피부:','pt':':pessoa_andando_de_frente_para_a_direita_pele_morena_clara:','it':':pedone_a_destra_carnagione_abbastanza_chiara:','fa':':عابر_پیاده_به_سمت_راست_پوست_روشن:','id':':orang_berjalan_kaki_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':行人面向右边_中等浅肤色:','ru':':человек_идет_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_yürüyen_kişi_orta_açık_cilt_tonu:','ar':':شخص_يسير_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F6B6\U0001F3FD\U0000200D\U000027A1\U0000FE0F':{'en':':person_walking_facing_right_medium_skin_tone:','status':fully_qualified,'E':15.1,'de':':fußgänger(in)_nach_rechts_mittlere_hautfarbe:','es':':persona_caminando_hacia_la_derecha_tono_de_piel_medio:','fr':':personne_qui_marche_vers_la_droite_peau_légèrement_mate:','ja':':歩く人_右向き_中間の肌色:','ko':':오른쪽을_향한_보행자_갈색_피부:','pt':':pessoa_andando_de_frente_para_a_direita_pele_morena:','it':':pedone_a_destra_carnagione_olivastra:','fa':':عابر_پیاده_به_سمت_راست_پوست_طلایی:','id':':orang_berjalan_kaki_menghadap_kanan_warna_kulit_sedang:','zh':':行人面向右边_中等肤色:','ru':':человек_идет_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_yürüyen_kişi_orta_cilt_tonu:','ar':':شخص_يسير_متجه_لليمين_بلون_معتدل:'},'\U0001F6B6\U0001F3FD\U0000200D\U000027A1':{'en':':person_walking_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgänger(in)_nach_rechts_mittlere_hautfarbe:','es':':persona_caminando_hacia_la_derecha_tono_de_piel_medio:','fr':':personne_qui_marche_vers_la_droite_peau_légèrement_mate:','ja':':歩く人_右向き_中間の肌色:','ko':':오른쪽을_향한_보행자_갈색_피부:','pt':':pessoa_andando_de_frente_para_a_direita_pele_morena:','it':':pedone_a_destra_carnagione_olivastra:','fa':':عابر_پیاده_به_سمت_راست_پوست_طلایی:','id':':orang_berjalan_kaki_menghadap_kanan_warna_kulit_sedang:','zh':':行人面向右边_中等肤色:','ru':':человек_идет_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_yürüyen_kişi_orta_cilt_tonu:','ar':':شخص_يسير_متجه_لليمين_بلون_معتدل:'},'\U0001F6B6\U0001F3FB':{'en':':person_walking_light_skin_tone:','status':fully_qualified,'E':1,'de':':fußgänger(in)_helle_hautfarbe:','es':':persona_caminando_tono_de_piel_claro:','fr':':personne_qui_marche_peau_claire:','ja':':歩く人_薄い肌色:','ko':':보행자_하얀_피부:','pt':':pessoa_andando_pele_clara:','it':':pedone_carnagione_chiara:','fa':':عابر_پیاده_پوست_سفید:','id':':orang_berjalan_kaki_warna_kulit_cerah:','zh':':行人_较浅肤色:','ru':':человек_идет_очень_светлый_тон_кожи:','tr':':yürüyen_kişi_açık_cilt_tonu:','ar':':شخص_يسير_بشرة_بلون_فاتح:'},'\U0001F6B6\U0001F3FE':{'en':':person_walking_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':fußgänger(in)_mitteldunkle_hautfarbe:','es':':persona_caminando_tono_de_piel_oscuro_medio:','fr':':personne_qui_marche_peau_mate:','ja':':歩く人_やや濃い肌色:','ko':':보행자_진한_갈색_피부:','pt':':pessoa_andando_pele_morena_escura:','it':':pedone_carnagione_abbastanza_scura:','fa':':عابر_پیاده_پوست_گندمی:','id':':orang_berjalan_kaki_warna_kulit_gelap-sedang:','zh':':行人_中等深肤色:','ru':':человек_идет_темный_тон_кожи:','tr':':yürüyen_kişi_orta_koyu_cilt_tonu:','ar':':شخص_يسير_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6B6\U0001F3FC':{'en':':person_walking_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':fußgänger(in)_mittelhelle_hautfarbe:','es':':persona_caminando_tono_de_piel_claro_medio:','fr':':personne_qui_marche_peau_moyennement_claire:','ja':':歩く人_やや薄い肌色:','ko':':보행자_연한_갈색_피부:','pt':':pessoa_andando_pele_morena_clara:','it':':pedone_carnagione_abbastanza_chiara:','fa':':عابر_پیاده_پوست_روشن:','id':':orang_berjalan_kaki_warna_kulit_cerah-sedang:','zh':':行人_中等浅肤色:','ru':':человек_идет_светлый_тон_кожи:','tr':':yürüyen_kişi_orta_açık_cilt_tonu:','ar':':شخص_يسير_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6B6\U0001F3FD':{'en':':person_walking_medium_skin_tone:','status':fully_qualified,'E':1,'de':':fußgänger(in)_mittlere_hautfarbe:','es':':persona_caminando_tono_de_piel_medio:','fr':':personne_qui_marche_peau_légèrement_mate:','ja':':歩く人_中間の肌色:','ko':':보행자_갈색_피부:','pt':':pessoa_andando_pele_morena:','it':':pedone_carnagione_olivastra:','fa':':عابر_پیاده_پوست_طلایی:','id':':orang_berjalan_kaki_warna_kulit_sedang:','zh':':行人_中等肤色:','ru':':человек_идет_средний_тон_кожи:','tr':':yürüyen_kişi_orta_cilt_tonu:','ar':':شخص_يسير_بشرة_بلون_معتدل:'},'\U0001F473':{'en':':person_wearing_turban:','status':fully_qualified,'E':0.6,'alias':[':person_with_turban:'],'de':':person_mit_turban:','es':':persona_con_turbante:','fr':':personne_en_turban:','ja':':ターバンの人:','ko':':터번을_쓰고_있는_사람:','pt':':pessoa_com_turbante:','it':':persona_con_turbante:','fa':':مرد_عمامه_دار:','id':':orang_mengenakan_serban:','zh':':戴头巾的人:','ru':':человек_в_тюрбане:','tr':':sarıklı_kişi:','ar':':رجل_بعمامة:'},'\U0001F473\U0001F3FF':{'en':':person_wearing_turban_dark_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_turban_dunkle_hautfarbe:','es':':persona_con_turbante_tono_de_piel_oscuro:','fr':':personne_en_turban_peau_foncée:','ja':':ターバンの人_濃い肌色:','ko':':터번을_쓰고_있는_사람_검은색_피부:','pt':':pessoa_com_turbante_pele_escura:','it':':persona_con_turbante_carnagione_scura:','fa':':مرد_عمامه_دار_پوست_آبنوسی:','id':':orang_mengenakan_serban_warna_kulit_gelap:','zh':':戴头巾的人_较深肤色:','ru':':человек_в_тюрбане_очень_темный_тон_кожи:','tr':':sarıklı_kişi_koyu_cilt_tonu:','ar':':رجل_بعمامة_بشرة_بلون_غامق:'},'\U0001F473\U0001F3FB':{'en':':person_wearing_turban_light_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_turban_helle_hautfarbe:','es':':persona_con_turbante_tono_de_piel_claro:','fr':':personne_en_turban_peau_claire:','ja':':ターバンの人_薄い肌色:','ko':':터번을_쓰고_있는_사람_하얀_피부:','pt':':pessoa_com_turbante_pele_clara:','it':':persona_con_turbante_carnagione_chiara:','fa':':مرد_عمامه_دار_پوست_سفید:','id':':orang_mengenakan_serban_warna_kulit_cerah:','zh':':戴头巾的人_较浅肤色:','ru':':человек_в_тюрбане_очень_светлый_тон_кожи:','tr':':sarıklı_kişi_açık_cilt_tonu:','ar':':رجل_بعمامة_بشرة_بلون_فاتح:'},'\U0001F473\U0001F3FE':{'en':':person_wearing_turban_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_turban_mitteldunkle_hautfarbe:','es':':persona_con_turbante_tono_de_piel_oscuro_medio:','fr':':personne_en_turban_peau_mate:','ja':':ターバンの人_やや濃い肌色:','ko':':터번을_쓰고_있는_사람_진한_갈색_피부:','pt':':pessoa_com_turbante_pele_morena_escura:','it':':persona_con_turbante_carnagione_abbastanza_scura:','fa':':مرد_عمامه_دار_پوست_گندمی:','id':':orang_mengenakan_serban_warna_kulit_gelap-sedang:','zh':':戴头巾的人_中等深肤色:','ru':':человек_в_тюрбане_темный_тон_кожи:','tr':':sarıklı_kişi_orta_koyu_cilt_tonu:','ar':':رجل_بعمامة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F473\U0001F3FC':{'en':':person_wearing_turban_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_turban_mittelhelle_hautfarbe:','es':':persona_con_turbante_tono_de_piel_claro_medio:','fr':':personne_en_turban_peau_moyennement_claire:','ja':':ターバンの人_やや薄い肌色:','ko':':터번을_쓰고_있는_사람_연한_갈색_피부:','pt':':pessoa_com_turbante_pele_morena_clara:','it':':persona_con_turbante_carnagione_abbastanza_chiara:','fa':':مرد_عمامه_دار_پوست_روشن:','id':':orang_mengenakan_serban_warna_kulit_cerah-sedang:','zh':':戴头巾的人_中等浅肤色:','ru':':человек_в_тюрбане_светлый_тон_кожи:','tr':':sarıklı_kişi_orta_açık_cilt_tonu:','ar':':رجل_بعمامة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F473\U0001F3FD':{'en':':person_wearing_turban_medium_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_turban_mittlere_hautfarbe:','es':':persona_con_turbante_tono_de_piel_medio:','fr':':personne_en_turban_peau_légèrement_mate:','ja':':ターバンの人_中間の肌色:','ko':':터번을_쓰고_있는_사람_갈색_피부:','pt':':pessoa_com_turbante_pele_morena:','it':':persona_con_turbante_carnagione_olivastra:','fa':':مرد_عمامه_دار_پوست_طلایی:','id':':orang_mengenakan_serban_warna_kulit_sedang:','zh':':戴头巾的人_中等肤色:','ru':':человек_в_тюрбане_средний_тон_кожи:','tr':':sarıklı_kişi_orta_cilt_tonu:','ar':':رجل_بعمامة_بشرة_بلون_معتدل:'},'\U0001F9D1\U0000200D\U0001F9B3':{'en':':person_white_hair:','status':fully_qualified,'E':12.1,'de':':person_weißes_haar:','es':':persona_adulta_pelo_blanco:','fr':':adulte_cheveux_blancs:','ja':':大人_白髪:','ko':':사람_백발:','pt':':pessoa_cabelo_branco:','it':':persona_capelli_bianchi:','fa':':شخص_موسفید:','id':':dewasa_rambut_putih:','zh':':成人_白发:','ru':':взрослый_седые_волосы:','tr':':yetişkin_beyaz_saç:','ar':':بالغ_شعر_أبيض:'},'\U0001FAC5':{'en':':person_with_crown:','status':fully_qualified,'E':14,'de':':person_mit_krone:','es':':persona_con_corona:','fr':':personne_avec_une_couronne:','ja':':王冠をかぶった人:','ko':':왕관을_쓴_사람:','pt':':pessoa_com_coroa:','it':':persona_con_corona:','fa':':آدم_تاج_دار:','id':':orang_mengenakan_mahkota:','zh':':戴王冠的人:','ru':':человек_с_короной:','tr':':taç_takmış_insan:','ar':':شخص_يرتدي_تاجا:'},'\U0001FAC5\U0001F3FF':{'en':':person_with_crown_dark_skin_tone:','status':fully_qualified,'E':14,'de':':person_mit_krone_dunkle_hautfarbe:','es':':persona_con_corona_tono_de_piel_oscuro:','fr':':personne_avec_une_couronne_peau_foncée:','ja':':王冠をかぶった人_濃い肌色:','ko':':왕관을_쓴_사람_검은색_피부:','pt':':pessoa_com_coroa_pele_escura:','it':':persona_con_corona_carnagione_scura:','fa':':آدم_تاج_دار_پوست_آبنوسی:','id':':orang_mengenakan_mahkota_warna_kulit_gelap:','zh':':戴王冠的人_较深肤色:','ru':':человек_с_короной_очень_темный_тон_кожи:','tr':':taç_takmış_insan_koyu_cilt_tonu:','ar':':شخص_يرتدي_تاجا_بشرة_بلون_غامق:'},'\U0001FAC5\U0001F3FB':{'en':':person_with_crown_light_skin_tone:','status':fully_qualified,'E':14,'de':':person_mit_krone_helle_hautfarbe:','es':':persona_con_corona_tono_de_piel_claro:','fr':':personne_avec_une_couronne_peau_claire:','ja':':王冠をかぶった人_薄い肌色:','ko':':왕관을_쓴_사람_하얀_피부:','pt':':pessoa_com_coroa_pele_clara:','it':':persona_con_corona_carnagione_chiara:','fa':':آدم_تاج_دار_پوست_سفید:','id':':orang_mengenakan_mahkota_warna_kulit_cerah:','zh':':戴王冠的人_较浅肤色:','ru':':человек_с_короной_очень_светлый_тон_кожи:','tr':':taç_takmış_insan_açık_cilt_tonu:','ar':':شخص_يرتدي_تاجا_بشرة_بلون_فاتح:'},'\U0001FAC5\U0001F3FE':{'en':':person_with_crown_medium-dark_skin_tone:','status':fully_qualified,'E':14,'de':':person_mit_krone_mitteldunkle_hautfarbe:','es':':persona_con_corona_tono_de_piel_oscuro_medio:','fr':':personne_avec_une_couronne_peau_mate:','ja':':王冠をかぶった人_やや濃い肌色:','ko':':왕관을_쓴_사람_진한_갈색_피부:','pt':':pessoa_com_coroa_pele_morena_escura:','it':':persona_con_corona_carnagione_abbastanza_scura:','fa':':آدم_تاج_دار_پوست_گندمی:','id':':orang_mengenakan_mahkota_warna_kulit_gelap-sedang:','zh':':戴王冠的人_中等深肤色:','ru':':человек_с_короной_темный_тон_кожи:','tr':':taç_takmış_insan_orta_koyu_cilt_tonu:','ar':':شخص_يرتدي_تاجا_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001FAC5\U0001F3FC':{'en':':person_with_crown_medium-light_skin_tone:','status':fully_qualified,'E':14,'de':':person_mit_krone_mittelhelle_hautfarbe:','es':':persona_con_corona_tono_de_piel_claro_medio:','fr':':personne_avec_une_couronne_peau_moyennement_claire:','ja':':王冠をかぶった人_やや薄い肌色:','ko':':왕관을_쓴_사람_연한_갈색_피부:','pt':':pessoa_com_coroa_pele_morena_clara:','it':':persona_con_corona_carnagione_abbastanza_chiara:','fa':':آدم_تاج_دار_پوست_روشن:','id':':orang_mengenakan_mahkota_warna_kulit_cerah-sedang:','zh':':戴王冠的人_中等浅肤色:','ru':':человек_с_короной_светлый_тон_кожи:','tr':':taç_takmış_insan_orta_açık_cilt_tonu:','ar':':شخص_يرتدي_تاجا_بشرة_بلون_فاتح_ومعتدل:'},'\U0001FAC5\U0001F3FD':{'en':':person_with_crown_medium_skin_tone:','status':fully_qualified,'E':14,'de':':person_mit_krone_mittlere_hautfarbe:','es':':persona_con_corona_tono_de_piel_medio:','fr':':personne_avec_une_couronne_peau_légèrement_mate:','ja':':王冠をかぶった人_中間の肌色:','ko':':왕관을_쓴_사람_갈색_피부:','pt':':pessoa_com_coroa_pele_morena:','it':':persona_con_corona_carnagione_olivastra:','fa':':آدم_تاج_دار_پوست_طلایی:','id':':orang_mengenakan_mahkota_warna_kulit_sedang:','zh':':戴王冠的人_中等肤色:','ru':':человек_с_короной_средний_тон_кожи:','tr':':taç_takmış_insan_orta_cilt_tonu:','ar':':شخص_يرتدي_تاجا_بشرة_بلون_معتدل:'},'\U0001F472':{'en':':person_with_skullcap:','status':fully_qualified,'E':0.6,'alias':[':man_with_gua_pi_mao:'],'de':':mann_mit_chinesischem_hut:','es':':persona_con_gorro_chino:','fr':':homme_avec_casquette_chinoise:','ja':':中華帽の男性:','ko':':중국_전통_모자를_쓰고_있는_남자:','pt':':homem_de_boné:','it':':uomo_con_zucchetto_cinese:','fa':':مرد_با_کلاه_چینی:','id':':orang_dengan_topi_tionghoa:','zh':':戴瓜皮帽的人:','ru':':мужчина_в_китайской_шапке:','tr':':çin_şapkalı_erkek:','ar':':رجل_بعمامة_صينية:'},'\U0001F472\U0001F3FF':{'en':':person_with_skullcap_dark_skin_tone:','status':fully_qualified,'E':1,'de':':mann_mit_chinesischem_hut_dunkle_hautfarbe:','es':':persona_con_gorro_chino_tono_de_piel_oscuro:','fr':':homme_avec_casquette_chinoise_peau_foncée:','ja':':中華帽の男性_濃い肌色:','ko':':중국_전통_모자를_쓰고_있는_남자_검은색_피부:','pt':':homem_de_boné_pele_escura:','it':':uomo_con_zucchetto_cinese_carnagione_scura:','fa':':مرد_با_کلاه_چینی_پوست_آبنوسی:','id':':orang_dengan_topi_tionghoa_warna_kulit_gelap:','zh':':戴瓜皮帽的人_较深肤色:','ru':':мужчина_в_китайской_шапке_очень_темный_тон_кожи:','tr':':çin_şapkalı_erkek_koyu_cilt_tonu:','ar':':رجل_بعمامة_صينية_بشرة_بلون_غامق:'},'\U0001F472\U0001F3FB':{'en':':person_with_skullcap_light_skin_tone:','status':fully_qualified,'E':1,'de':':mann_mit_chinesischem_hut_helle_hautfarbe:','es':':persona_con_gorro_chino_tono_de_piel_claro:','fr':':homme_avec_casquette_chinoise_peau_claire:','ja':':中華帽の男性_薄い肌色:','ko':':중국_전통_모자를_쓰고_있는_남자_하얀_피부:','pt':':homem_de_boné_pele_clara:','it':':uomo_con_zucchetto_cinese_carnagione_chiara:','fa':':مرد_با_کلاه_چینی_پوست_سفید:','id':':orang_dengan_topi_tionghoa_warna_kulit_cerah:','zh':':戴瓜皮帽的人_较浅肤色:','ru':':мужчина_в_китайской_шапке_очень_светлый_тон_кожи:','tr':':çin_şapkalı_erkek_açık_cilt_tonu:','ar':':رجل_بعمامة_صينية_بشرة_بلون_فاتح:'},'\U0001F472\U0001F3FE':{'en':':person_with_skullcap_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':mann_mit_chinesischem_hut_mitteldunkle_hautfarbe:','es':':persona_con_gorro_chino_tono_de_piel_oscuro_medio:','fr':':homme_avec_casquette_chinoise_peau_mate:','ja':':中華帽の男性_やや濃い肌色:','ko':':중국_전통_모자를_쓰고_있는_남자_진한_갈색_피부:','pt':':homem_de_boné_pele_morena_escura:','it':':uomo_con_zucchetto_cinese_carnagione_abbastanza_scura:','fa':':مرد_با_کلاه_چینی_پوست_گندمی:','id':':orang_dengan_topi_tionghoa_warna_kulit_gelap-sedang:','zh':':戴瓜皮帽的人_中等深肤色:','ru':':мужчина_в_китайской_шапке_темный_тон_кожи:','tr':':çin_şapkalı_erkek_orta_koyu_cilt_tonu:','ar':':رجل_بعمامة_صينية_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F472\U0001F3FC':{'en':':person_with_skullcap_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':mann_mit_chinesischem_hut_mittelhelle_hautfarbe:','es':':persona_con_gorro_chino_tono_de_piel_claro_medio:','fr':':homme_avec_casquette_chinoise_peau_moyennement_claire:','ja':':中華帽の男性_やや薄い肌色:','ko':':중국_전통_모자를_쓰고_있는_남자_연한_갈색_피부:','pt':':homem_de_boné_pele_morena_clara:','it':':uomo_con_zucchetto_cinese_carnagione_abbastanza_chiara:','fa':':مرد_با_کلاه_چینی_پوست_روشن:','id':':orang_dengan_topi_tionghoa_warna_kulit_cerah-sedang:','zh':':戴瓜皮帽的人_中等浅肤色:','ru':':мужчина_в_китайской_шапке_светлый_тон_кожи:','tr':':çin_şapkalı_erkek_orta_açık_cilt_tonu:','ar':':رجل_بعمامة_صينية_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F472\U0001F3FD':{'en':':person_with_skullcap_medium_skin_tone:','status':fully_qualified,'E':1,'de':':mann_mit_chinesischem_hut_mittlere_hautfarbe:','es':':persona_con_gorro_chino_tono_de_piel_medio:','fr':':homme_avec_casquette_chinoise_peau_légèrement_mate:','ja':':中華帽の男性_中間の肌色:','ko':':중국_전통_모자를_쓰고_있는_남자_갈색_피부:','pt':':homem_de_boné_pele_morena:','it':':uomo_con_zucchetto_cinese_carnagione_olivastra:','fa':':مرد_با_کلاه_چینی_پوست_طلایی:','id':':orang_dengan_topi_tionghoa_warna_kulit_sedang:','zh':':戴瓜皮帽的人_中等肤色:','ru':':мужчина_в_китайской_шапке_средний_тон_кожи:','tr':':çin_şapkalı_erkek_orta_cilt_tonu:','ar':':رجل_بعمامة_صينية_بشرة_بلون_معتدل:'},'\U0001F470':{'en':':person_with_veil:','status':fully_qualified,'E':0.6,'de':':person_mit_schleier:','es':':persona_con_velo:','fr':':personne_mariée_avec_voile:','ja':':ベールの人:','ko':':면사포를_쓴_사람:','pt':':pessoa_com_véu:','it':':persona_con_velo:','fa':':شخص_با_تور_عروس:','id':':orang_berpenutup_kepala:','zh':':戴头纱的人:','ru':':невеста:','tr':':duvaklı_gelin:','ar':':عروس_بطرحة:'},'\U0001F470\U0001F3FF':{'en':':person_with_veil_dark_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_schleier_dunkle_hautfarbe:','es':':persona_con_velo_tono_de_piel_oscuro:','fr':':personne_mariée_avec_voile_peau_foncée:','ja':':ベールの人_濃い肌色:','ko':':면사포를_쓴_사람_검은색_피부:','pt':':pessoa_com_véu_pele_escura:','it':':persona_con_velo_carnagione_scura:','fa':':شخص_با_تور_عروس_پوست_آبنوسی:','id':':orang_berpenutup_kepala_warna_kulit_gelap:','zh':':戴头纱的人_较深肤色:','ru':':невеста_очень_темный_тон_кожи:','tr':':duvaklı_gelin_koyu_cilt_tonu:','ar':':عروس_بطرحة_بشرة_بلون_غامق:'},'\U0001F470\U0001F3FB':{'en':':person_with_veil_light_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_schleier_helle_hautfarbe:','es':':persona_con_velo_tono_de_piel_claro:','fr':':personne_mariée_avec_voile_peau_claire:','ja':':ベールの人_薄い肌色:','ko':':면사포를_쓴_사람_하얀_피부:','pt':':pessoa_com_véu_pele_clara:','it':':persona_con_velo_carnagione_chiara:','fa':':شخص_با_تور_عروس_پوست_سفید:','id':':orang_berpenutup_kepala_warna_kulit_cerah:','zh':':戴头纱的人_较浅肤色:','ru':':невеста_очень_светлый_тон_кожи:','tr':':duvaklı_gelin_açık_cilt_tonu:','ar':':عروس_بطرحة_بشرة_بلون_فاتح:'},'\U0001F470\U0001F3FE':{'en':':person_with_veil_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_schleier_mitteldunkle_hautfarbe:','es':':persona_con_velo_tono_de_piel_oscuro_medio:','fr':':personne_mariée_avec_voile_peau_mate:','ja':':ベールの人_やや濃い肌色:','ko':':면사포를_쓴_사람_진한_갈색_피부:','pt':':pessoa_com_véu_pele_morena_escura:','it':':persona_con_velo_carnagione_abbastanza_scura:','fa':':شخص_با_تور_عروس_پوست_گندمی:','id':':orang_berpenutup_kepala_warna_kulit_gelap-sedang:','zh':':戴头纱的人_中等深肤色:','ru':':невеста_темный_тон_кожи:','tr':':duvaklı_gelin_orta_koyu_cilt_tonu:','ar':':عروس_بطرحة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F470\U0001F3FC':{'en':':person_with_veil_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_schleier_mittelhelle_hautfarbe:','es':':persona_con_velo_tono_de_piel_claro_medio:','fr':':personne_mariée_avec_voile_peau_moyennement_claire:','ja':':ベールの人_やや薄い肌色:','ko':':면사포를_쓴_사람_연한_갈색_피부:','pt':':pessoa_com_véu_pele_morena_clara:','it':':persona_con_velo_carnagione_abbastanza_chiara:','fa':':شخص_با_تور_عروس_پوست_روشن:','id':':orang_berpenutup_kepala_warna_kulit_cerah-sedang:','zh':':戴头纱的人_中等浅肤色:','ru':':невеста_светлый_тон_кожи:','tr':':duvaklı_gelin_orta_açık_cilt_tonu:','ar':':عروس_بطرحة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F470\U0001F3FD':{'en':':person_with_veil_medium_skin_tone:','status':fully_qualified,'E':1,'de':':person_mit_schleier_mittlere_hautfarbe:','es':':persona_con_velo_tono_de_piel_medio:','fr':':personne_mariée_avec_voile_peau_légèrement_mate:','ja':':ベールの人_中間の肌色:','ko':':면사포를_쓴_사람_갈색_피부:','pt':':pessoa_com_véu_pele_morena:','it':':persona_con_velo_carnagione_olivastra:','fa':':شخص_با_تور_عروس_پوست_طلایی:','id':':orang_berpenutup_kepala_warna_kulit_sedang:','zh':':戴头纱的人_中等肤色:','ru':':невеста_средний_тон_кожи:','tr':':duvaklı_gelin_orta_cilt_tonu:','ar':':عروس_بطرحة_بشرة_بلون_معتدل:'},'\U0001F9D1\U0000200D\U0001F9AF':{'en':':person_with_white_cane:','status':fully_qualified,'E':12.1,'alias':[':person_with_probing_cane:'],'de':':person_mit_langstock:','es':':persona_con_bastón:','fr':':personne_avec_une_canne_blanche:','ja':':杖をついた人:','ko':':지팡이를_든_사람:','pt':':pessoa_com_bengala_para_cego:','it':':persona_con_bastone_di_orientamento:','fa':':شخصی_با_عصای_راهنما:','id':':orang_dengan_tongkat:','zh':':拄盲杖的人:','ru':':человек_с_белой_тростью:','tr':':görme_engelli_bastonlu_kişi:','ar':':شخص_مع_عصا_سير:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F9AF':{'en':':person_with_white_cane_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':person_mit_langstock_dunkle_hautfarbe:','es':':persona_con_bastón_tono_de_piel_oscuro:','fr':':personne_avec_une_canne_blanche_peau_foncée:','ja':':杖をついた人_濃い肌色:','ko':':지팡이를_든_사람_검은색_피부:','pt':':pessoa_com_bengala_para_cego_pele_escura:','it':':persona_con_bastone_di_orientamento_carnagione_scura:','fa':':شخصی_با_عصای_راهنما_پوست_آبنوسی:','id':':orang_dengan_tongkat_warna_kulit_gelap:','zh':':拄盲杖的人_较深肤色:','ru':':человек_с_белой_тростью_очень_темный_тон_кожи:','tr':':görme_engelli_bastonlu_kişi_koyu_cilt_tonu:','ar':':شخص_مع_عصا_سير_بشرة_بلون_غامق:'},'\U0001F9D1\U0000200D\U0001F9AF\U0000200D\U000027A1\U0000FE0F':{'en':':person_with_white_cane_facing_right:','status':fully_qualified,'E':15.1,'de':':person_mit_langstock_nach_rechts:','es':':persona_con_bastón_hacia_la_derecha:','fr':':personne_avec_une_canne_blanche_vers_la_droite:','ja':':杖をついた人_右向き:','ko':':오른쪽을_향한_지팡이를_든_사람:','pt':':pessoa_com_bengala_para_cego_de_frente_para_a_direita:','it':':persona_con_bastone_di_orientamento_a_destra:','fa':':شخصی_با_عصای_راهنما_به_سمت_راست:','id':':orang_dengan_tongkat_menghadap_kanan:','zh':':拄盲杖的人面向右边:','ru':':человек_с_белой_тростью_направленные_вправо:','tr':':sağa_dönük_görme_engelli_bastonlu_kişi:','ar':':شخص_مع_عصا_سير_متجه_لليمين:'},'\U0001F9D1\U0000200D\U0001F9AF\U0000200D\U000027A1':{'en':':person_with_white_cane_facing_right:','status':minimally_qualified,'E':15.1,'de':':person_mit_langstock_nach_rechts:','es':':persona_con_bastón_hacia_la_derecha:','fr':':personne_avec_une_canne_blanche_vers_la_droite:','ja':':杖をついた人_右向き:','ko':':오른쪽을_향한_지팡이를_든_사람:','pt':':pessoa_com_bengala_para_cego_de_frente_para_a_direita:','it':':persona_con_bastone_di_orientamento_a_destra:','fa':':شخصی_با_عصای_راهنما_به_سمت_راست:','id':':orang_dengan_tongkat_menghadap_kanan:','zh':':拄盲杖的人面向右边:','ru':':человек_с_белой_тростью_направленные_вправо:','tr':':sağa_dönük_görme_engelli_bastonlu_kişi:','ar':':شخص_مع_عصا_سير_متجه_لليمين:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F9AF\U0000200D\U000027A1\U0000FE0F':{'en':':person_with_white_cane_facing_right_dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':person_mit_langstock_nach_rechts_dunkle_hautfarbe:','es':':persona_con_bastón_hacia_la_derecha_tono_de_piel_oscuro:','fr':':personne_avec_une_canne_blanche_vers_la_droite_peau_foncée:','ja':':杖をついた人_右向き_濃い肌色:','ko':':오른쪽을_향한_지팡이를_든_사람_검은색_피부:','pt':':pessoa_com_bengala_para_cego_de_frente_para_a_direita_pele_escura:','it':':persona_con_bastone_di_orientamento_a_destra_carnagione_scura:','fa':':شخصی_با_عصای_راهنما_به_سمت_راستپوست_آبنوسی:','id':':orang_dengan_tongkat_menghadap_kanan_warna_kulit_gelap:','zh':':拄盲杖的人面向右边_较深肤色:','ru':':человек_с_белой_тростью_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_kişi_koyu_cilt_tonu:','ar':':شخص_مع_عصا_سير_متجه_لليمين_بلون_غامق:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F9AF\U0000200D\U000027A1':{'en':':person_with_white_cane_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':person_mit_langstock_nach_rechts_dunkle_hautfarbe:','es':':persona_con_bastón_hacia_la_derecha_tono_de_piel_oscuro:','fr':':personne_avec_une_canne_blanche_vers_la_droite_peau_foncée:','ja':':杖をついた人_右向き_濃い肌色:','ko':':오른쪽을_향한_지팡이를_든_사람_검은색_피부:','pt':':pessoa_com_bengala_para_cego_de_frente_para_a_direita_pele_escura:','it':':persona_con_bastone_di_orientamento_a_destra_carnagione_scura:','fa':':شخصی_با_عصای_راهنما_به_سمت_راستپوست_آبنوسی:','id':':orang_dengan_tongkat_menghadap_kanan_warna_kulit_gelap:','zh':':拄盲杖的人面向右边_较深肤色:','ru':':человек_с_белой_тростью_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_kişi_koyu_cilt_tonu:','ar':':شخص_مع_عصا_سير_متجه_لليمين_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F9AF\U0000200D\U000027A1\U0000FE0F':{'en':':person_with_white_cane_facing_right_light_skin_tone:','status':fully_qualified,'E':15.1,'de':':person_mit_langstock_nach_rechts_helle_hautfarbe:','es':':persona_con_bastón_hacia_la_derecha_tono_de_piel_claro:','fr':':personne_avec_une_canne_blanche_vers_la_droite_peau_claire:','ja':':杖をついた人_右向き_薄い肌色:','ko':':오른쪽을_향한_지팡이를_든_사람_하얀_피부:','pt':':pessoa_com_bengala_para_cego_de_frente_para_a_direita_pele_clara:','it':':persona_con_bastone_di_orientamento_a_destra_carnagione_chiara:','fa':':شخصی_با_عصای_راهنما_به_سمت_راست_پوست_سفید:','id':':orang_dengan_tongkat_menghadap_kanan_warna_kulit_cerah:','zh':':拄盲杖的人面向右边_较浅肤色:','ru':':человек_с_белой_тростью_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_kişi_açık_cilt_tonu:','ar':':شخص_مع_عصا_سير_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F9AF\U0000200D\U000027A1':{'en':':person_with_white_cane_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':person_mit_langstock_nach_rechts_helle_hautfarbe:','es':':persona_con_bastón_hacia_la_derecha_tono_de_piel_claro:','fr':':personne_avec_une_canne_blanche_vers_la_droite_peau_claire:','ja':':杖をついた人_右向き_薄い肌色:','ko':':오른쪽을_향한_지팡이를_든_사람_하얀_피부:','pt':':pessoa_com_bengala_para_cego_de_frente_para_a_direita_pele_clara:','it':':persona_con_bastone_di_orientamento_a_destra_carnagione_chiara:','fa':':شخصی_با_عصای_راهنما_به_سمت_راست_پوست_سفید:','id':':orang_dengan_tongkat_menghadap_kanan_warna_kulit_cerah:','zh':':拄盲杖的人面向右边_较浅肤色:','ru':':человек_с_белой_тростью_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_kişi_açık_cilt_tonu:','ar':':شخص_مع_عصا_سير_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F9AF\U0000200D\U000027A1\U0000FE0F':{'en':':person_with_white_cane_facing_right_medium-dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':person_mit_langstock_nach_rechts_mitteldunkle_hautfarbe:','es':':persona_con_bastón_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':personne_avec_une_canne_blanche_vers_la_droite_peau_mate:','ja':':杖をついた人_右向き_やや濃い肌色:','ko':':오른쪽을_향한_지팡이를_든_사람_진한_갈색_피부:','pt':':pessoa_com_bengala_para_cego_de_frente_para_a_direita_pele_morena_escura:','it':':persona_con_bastone_di_orientamento_a_destra_carnagione_abbastanza_scura:','fa':':شخصی_با_عصای_راهنما_به_سمت_راست_پوست_گندمی:','id':':orang_dengan_tongkat_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':拄盲杖的人面向右边_中等深肤色:','ru':':человек_с_белой_тростью_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_kişi_orta_koyu_cilt_tonu:','ar':':شخص_مع_عصا_سير_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F9AF\U0000200D\U000027A1':{'en':':person_with_white_cane_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':person_mit_langstock_nach_rechts_mitteldunkle_hautfarbe:','es':':persona_con_bastón_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':personne_avec_une_canne_blanche_vers_la_droite_peau_mate:','ja':':杖をついた人_右向き_やや濃い肌色:','ko':':오른쪽을_향한_지팡이를_든_사람_진한_갈색_피부:','pt':':pessoa_com_bengala_para_cego_de_frente_para_a_direita_pele_morena_escura:','it':':persona_con_bastone_di_orientamento_a_destra_carnagione_abbastanza_scura:','fa':':شخصی_با_عصای_راهنما_به_سمت_راست_پوست_گندمی:','id':':orang_dengan_tongkat_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':拄盲杖的人面向右边_中等深肤色:','ru':':человек_с_белой_тростью_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_kişi_orta_koyu_cilt_tonu:','ar':':شخص_مع_عصا_سير_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F9AF\U0000200D\U000027A1\U0000FE0F':{'en':':person_with_white_cane_facing_right_medium-light_skin_tone:','status':fully_qualified,'E':15.1,'de':':person_mit_langstock_nach_rechts_mittelhelle_hautfarbe:','es':':persona_con_bastón_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':personne_avec_une_canne_blanche_vers_la_droite_peau_moyennement_claire:','ja':':杖をついた人_右向き_やや薄い肌色:','ko':':오른쪽을_향한_지팡이를_든_사람_연한_갈색_피부:','pt':':pessoa_com_bengala_para_cego_de_frente_para_a_direita_pele_morena_clara:','it':':persona_con_bastone_di_orientamento_a_destra_carnagione_abbastanza_chiara:','fa':':شخصی_با_عصای_راهنما_به_سمت_راست_پوست_روشن:','id':':orang_dengan_tongkat_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':拄盲杖的人面向右边_中等浅肤色:','ru':':человек_с_белой_тростью_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_görme_engelli_bastonlu_kişi_orta_açık_cilt_tonu:','ar':':شخص_مع_عصا_سير_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F9AF\U0000200D\U000027A1':{'en':':person_with_white_cane_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':person_mit_langstock_nach_rechts_mittelhelle_hautfarbe:','es':':persona_con_bastón_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':personne_avec_une_canne_blanche_vers_la_droite_peau_moyennement_claire:','ja':':杖をついた人_右向き_やや薄い肌色:','ko':':오른쪽을_향한_지팡이를_든_사람_연한_갈색_피부:','pt':':pessoa_com_bengala_para_cego_de_frente_para_a_direita_pele_morena_clara:','it':':persona_con_bastone_di_orientamento_a_destra_carnagione_abbastanza_chiara:','fa':':شخصی_با_عصای_راهنما_به_سمت_راست_پوست_روشن:','id':':orang_dengan_tongkat_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':拄盲杖的人面向右边_中等浅肤色:','ru':':человек_с_белой_тростью_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_görme_engelli_bastonlu_kişi_orta_açık_cilt_tonu:','ar':':شخص_مع_عصا_سير_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F9AF\U0000200D\U000027A1\U0000FE0F':{'en':':person_with_white_cane_facing_right_medium_skin_tone:','status':fully_qualified,'E':15.1,'de':':person_mit_langstock_nach_rechts_mittlere_hautfarbe:','es':':persona_con_bastón_hacia_la_derecha_tono_de_piel_medio:','fr':':personne_avec_une_canne_blanche_vers_la_droite_peau_légèrement_mate:','ja':':杖をついた人_右向き_中間の肌色:','ko':':오른쪽을_향한_지팡이를_든_사람_갈색_피부:','pt':':pessoa_com_bengala_para_cego_de_frente_para_a_direita_pele_morena:','it':':persona_con_bastone_di_orientamento_a_destra_carnagione_olivastra:','fa':':شخصی_با_عصای_راهنما_به_سمت_راست_پوست_طلایی:','id':':orang_dengan_tongkat_menghadap_kanan_warna_kulit_sedang:','zh':':拄盲杖的人面向右边_中等肤色:','ru':':человек_с_белой_тростью_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_kişi_orta_cilt_tonu:','ar':':شخص_مع_عصا_سير_متجه_لليمين_بلون_معتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F9AF\U0000200D\U000027A1':{'en':':person_with_white_cane_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':person_mit_langstock_nach_rechts_mittlere_hautfarbe:','es':':persona_con_bastón_hacia_la_derecha_tono_de_piel_medio:','fr':':personne_avec_une_canne_blanche_vers_la_droite_peau_légèrement_mate:','ja':':杖をついた人_右向き_中間の肌色:','ko':':오른쪽을_향한_지팡이를_든_사람_갈색_피부:','pt':':pessoa_com_bengala_para_cego_de_frente_para_a_direita_pele_morena:','it':':persona_con_bastone_di_orientamento_a_destra_carnagione_olivastra:','fa':':شخصی_با_عصای_راهنما_به_سمت_راست_پوست_طلایی:','id':':orang_dengan_tongkat_menghadap_kanan_warna_kulit_sedang:','zh':':拄盲杖的人面向右边_中等肤色:','ru':':человек_с_белой_тростью_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_kişi_orta_cilt_tonu:','ar':':شخص_مع_عصا_سير_متجه_لليمين_بلون_معتدل:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F9AF':{'en':':person_with_white_cane_light_skin_tone:','status':fully_qualified,'E':12.1,'de':':person_mit_langstock_helle_hautfarbe:','es':':persona_con_bastón_tono_de_piel_claro:','fr':':personne_avec_une_canne_blanche_peau_claire:','ja':':杖をついた人_薄い肌色:','ko':':지팡이를_든_사람_하얀_피부:','pt':':pessoa_com_bengala_para_cego_pele_clara:','it':':persona_con_bastone_di_orientamento_carnagione_chiara:','fa':':شخصی_با_عصای_راهنما_پوست_سفید:','id':':orang_dengan_tongkat_warna_kulit_cerah:','zh':':拄盲杖的人_较浅肤色:','ru':':человек_с_белой_тростью_очень_светлый_тон_кожи:','tr':':görme_engelli_bastonlu_kişi_açık_cilt_tonu:','ar':':شخص_مع_عصا_سير_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F9AF':{'en':':person_with_white_cane_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':person_mit_langstock_mitteldunkle_hautfarbe:','es':':persona_con_bastón_tono_de_piel_oscuro_medio:','fr':':personne_avec_une_canne_blanche_peau_mate:','ja':':杖をついた人_やや濃い肌色:','ko':':지팡이를_든_사람_진한_갈색_피부:','pt':':pessoa_com_bengala_para_cego_pele_morena_escura:','it':':persona_con_bastone_di_orientamento_carnagione_abbastanza_scura:','fa':':شخصی_با_عصای_راهنما_پوست_گندمی:','id':':orang_dengan_tongkat_warna_kulit_gelap-sedang:','zh':':拄盲杖的人_中等深肤色:','ru':':человек_с_белой_тростью_темный_тон_кожи:','tr':':görme_engelli_bastonlu_kişi_orta_koyu_cilt_tonu:','ar':':شخص_مع_عصا_سير_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F9AF':{'en':':person_with_white_cane_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':person_mit_langstock_mittelhelle_hautfarbe:','es':':persona_con_bastón_tono_de_piel_claro_medio:','fr':':personne_avec_une_canne_blanche_peau_moyennement_claire:','ja':':杖をついた人_やや薄い肌色:','ko':':지팡이를_든_사람_연한_갈색_피부:','pt':':pessoa_com_bengala_para_cego_pele_morena_clara:','it':':persona_con_bastone_di_orientamento_carnagione_abbastanza_chiara:','fa':':شخصی_با_عصای_راهنما_پوست_روشن:','id':':orang_dengan_tongkat_warna_kulit_cerah-sedang:','zh':':拄盲杖的人_中等浅肤色:','ru':':человек_с_белой_тростью_светлый_тон_кожи:','tr':':görme_engelli_bastonlu_kişi_orta_açık_cilt_tonu:','ar':':شخص_مع_عصا_سير_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F9AF':{'en':':person_with_white_cane_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':person_mit_langstock_mittlere_hautfarbe:','es':':persona_con_bastón_tono_de_piel_medio:','fr':':personne_avec_une_canne_blanche_peau_légèrement_mate:','ja':':杖をついた人_中間の肌色:','ko':':지팡이를_든_사람_갈색_피부:','pt':':pessoa_com_bengala_para_cego_pele_morena:','it':':persona_con_bastone_di_orientamento_carnagione_olivastra:','fa':':شخصی_با_عصای_راهنما_پوست_طلایی:','id':':orang_dengan_tongkat_warna_kulit_sedang:','zh':':拄盲杖的人_中等肤色:','ru':':человек_с_белой_тростью_средний_тон_кожи:','tr':':görme_engelli_bastonlu_kişi_orta_cilt_tonu:','ar':':شخص_مع_عصا_سير_بشرة_بلون_معتدل:'},'\U0001F9EB':{'en':':petri_dish:','status':fully_qualified,'E':11,'de':':petrischale:','es':':placa_de_petri:','fr':':boîte_de_pétri:','ja':':ペトリ皿:','ko':':페트리_접시:','pt':':placa_de_petri:','it':':piastra_di_petri:','fa':':ظرف_کشت:','id':':cawan_petri:','zh':':培养皿:','ru':':чашка_Петри:','tr':':petri_kabı:','ar':':صحفة_بتري:'},'\U0001F426\U0000200D\U0001F525':{'en':':phoenix:','status':fully_qualified,'E':15.1,'de':':phönix:','es':':fénix:','fr':':phénix:','ja':':フェニックス:','ko':':불사조:','pt':':fênix:','it':':fenice:','fa':':ققنوس:','id':':phoenix:','zh':':凤凰:','ru':':феникс:','tr':':anka:','ar':':العنقاء:'},'\U000026CF\U0000FE0F':{'en':':pick:','status':fully_qualified,'E':0.7,'variant':True,'de':':pickel:','es':':pico:','fr':':pioche:','ja':':つるはし:','ko':':곡괭이:','pt':':picareta:','it':':piccone:','fa':':کلنگ:','id':':beliung:','zh':':铁镐:','ru':':кирка:','tr':':kazma:','ar':':فأس:'},'\U000026CF':{'en':':pick:','status':unqualified,'E':0.7,'variant':True,'de':':pickel:','es':':pico:','fr':':pioche:','ja':':つるはし:','ko':':곡괭이:','pt':':picareta:','it':':piccone:','fa':':کلنگ:','id':':beliung:','zh':':铁镐:','ru':':кирка:','tr':':kazma:','ar':':فأس:'},'\U0001F6FB':{'en':':pickup_truck:','status':fully_qualified,'E':13,'de':':pick-up:','es':':camioneta:','fr':':pick-up:','ja':':軽トラック:','ko':':픽업트럭:','pt':':caminhonete:','it':':pickup:','fa':':وانت:','id':':truk_pikap:','zh':':敞蓬小型载货卡车:','ru':':пикап:','tr':':pikap:','ar':':شاحنة_خفيفة:'},'\U0001F967':{'en':':pie:','status':fully_qualified,'E':5,'de':':kuchen:','es':':pastel:','fr':':tarte:','ja':':パイ:','ko':':파이:','pt':':torta:','it':':torta:','fa':':پای:','id':':pai:','zh':':派:','ru':':пирог:','tr':':tart:','ar':':فطيرة:'},'\U0001F416':{'en':':pig:','status':fully_qualified,'E':1,'alias':[':pig2:'],'de':':schwein:','es':':cerdo:','fr':':cochon:','ja':':ブタ:','ko':':돼지:','pt':':porco:','it':':maiale:','fa':':خوک:','id':':babi:','zh':':猪:','ru':':свинья:','tr':':domuz:','ar':':خنزير:'},'\U0001F437':{'en':':pig_face:','status':fully_qualified,'E':0.6,'alias':[':pig:'],'de':':schweinegesicht:','es':':cara_de_cerdo:','fr':':tête_de_cochon:','ja':':ブタの顔:','ko':':돼지_얼굴:','pt':':rosto_de_porco:','it':':muso_di_maiale:','fa':':صورت_خوک:','id':':wajah_babi:','zh':':猪头:','ru':':морда_свиньи:','tr':':domuz_yüzü:','ar':':وجه_خنزير:'},'\U0001F43D':{'en':':pig_nose:','status':fully_qualified,'E':0.6,'de':':schweinerüssel:','es':':nariz_de_cerdo:','fr':':groin:','ja':':ブタ鼻:','ko':':돼지코:','pt':':nariz_de_porco:','it':':naso_da_maiale:','fa':':دماغ_خوک:','id':':hidung_babi:','zh':':猪鼻子:','ru':':свиной_пятачок:','tr':':domuz_burnu:','ar':':أنف_خنزير:'},'\U0001F4A9':{'en':':pile_of_poo:','status':fully_qualified,'E':0.6,'alias':[':poop:', ':hankey:', ':shit:'],'de':':kothaufen:','es':':caca_con_ojos:','fr':':tas_de_crotte:','ja':':うんち:','ko':':똥:','pt':':cocô:','it':':cacca:','fa':':یه_تپهٔ_گه:','id':':kotoran:','zh':':大便:','ru':':какашка:','tr':':kaka:','ar':':فضلات:'},'\U0001F48A':{'en':':pill:','status':fully_qualified,'E':0.6,'de':':kapsel:','es':':píldora:','fr':':pilule:','ja':':薬:','ko':':알약:','pt':':comprimido:','it':':pillola:','fa':':قرص:','id':':pil:','zh':':药丸:','ru':':пилюля:','tr':':hap:','ar':':حبوب:'},'\U0001F9D1\U0000200D\U00002708\U0000FE0F':{'en':':pilot:','status':fully_qualified,'E':12.1,'de':':pilot(in):','es':':piloto:','fr':':pilote:','ja':':パイロット:','ko':':기장:','pt':':piloto:','it':':pilota:','fa':':خلبان:','id':':pilot:','zh':':飞行员:','ru':':пилот:','tr':':pilot:','ar':':ربان_طائرة:'},'\U0001F9D1\U0000200D\U00002708':{'en':':pilot:','status':minimally_qualified,'E':12.1,'de':':pilot(in):','es':':piloto:','fr':':pilote:','ja':':パイロット:','ko':':기장:','pt':':piloto:','it':':pilota:','fa':':خلبان:','id':':pilot:','zh':':飞行员:','ru':':пилот:','tr':':pilot:','ar':':ربان_طائرة:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002708\U0000FE0F':{'en':':pilot_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':pilot(in)_dunkle_hautfarbe:','es':':piloto_tono_de_piel_oscuro:','fr':':pilote_peau_foncée:','ja':':パイロット_濃い肌色:','ko':':기장_검은색_피부:','pt':':piloto_pele_escura:','it':':pilota_carnagione_scura:','fa':':خلبان_پوست_آبنوسی:','id':':pilot_warna_kulit_gelap:','zh':':飞行员_较深肤色:','ru':':пилот_очень_темныи_тон_кожи:','tr':':pilot_koyu_cilt_tonu:','ar':':ربان_طائرة_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FF\U0000200D\U00002708':{'en':':pilot_dark_skin_tone:','status':minimally_qualified,'E':12.1,'de':':pilot(in)_dunkle_hautfarbe:','es':':piloto_tono_de_piel_oscuro:','fr':':pilote_peau_foncée:','ja':':パイロット_濃い肌色:','ko':':기장_검은색_피부:','pt':':piloto_pele_escura:','it':':pilota_carnagione_scura:','fa':':خلبان_پوست_آبنوسی:','id':':pilot_warna_kulit_gelap:','zh':':飞行员_较深肤色:','ru':':пилот_очень_темный_тон_кожи:','tr':':pilot_koyu_cilt_tonu:','ar':':ربان_طائرة_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002708\U0000FE0F':{'en':':pilot_light_skin_tone:','status':fully_qualified,'E':12.1,'de':':pilot(in)_helle_hautfarbe:','es':':piloto_tono_de_piel_claro:','fr':':pilote_peau_claire:','ja':':パイロット_薄い肌色:','ko':':기장_하얀_피부:','pt':':piloto_pele_clara:','it':':pilota_carnagione_chiara:','fa':':خلبان_پوست_سفید:','id':':pilot_warna_kulit_cerah:','zh':':飞行员_较浅肤色:','ru':':пилот_очень_светлыи_тон_кожи:','tr':':pilot_açık_cilt_tonu:','ar':':ربان_طائرة_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FB\U0000200D\U00002708':{'en':':pilot_light_skin_tone:','status':minimally_qualified,'E':12.1,'de':':pilot(in)_helle_hautfarbe:','es':':piloto_tono_de_piel_claro:','fr':':pilote_peau_claire:','ja':':パイロット_薄い肌色:','ko':':기장_하얀_피부:','pt':':piloto_pele_clara:','it':':pilota_carnagione_chiara:','fa':':خلبان_پوست_سفید:','id':':pilot_warna_kulit_cerah:','zh':':飞行员_较浅肤色:','ru':':пилот_очень_светлый_тон_кожи:','tr':':pilot_açık_cilt_tonu:','ar':':ربان_طائرة_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002708\U0000FE0F':{'en':':pilot_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':pilot(in)_mitteldunkle_hautfarbe:','es':':piloto_tono_de_piel_oscuro_medio:','fr':':pilote_peau_mate:','ja':':パイロット_やや濃い肌色:','ko':':기장_진한_갈색_피부:','pt':':piloto_pele_morena_escura:','it':':pilota_carnagione_abbastanza_scura:','fa':':خلبان_پوست_گندمی:','id':':pilot_warna_kulit_gelap-sedang:','zh':':飞行员_中等深肤色:','ru':':пилот_темныи_тон_кожи:','tr':':pilot_orta_koyu_cilt_tonu:','ar':':ربان_طائرة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FE\U0000200D\U00002708':{'en':':pilot_medium-dark_skin_tone:','status':minimally_qualified,'E':12.1,'de':':pilot(in)_mitteldunkle_hautfarbe:','es':':piloto_tono_de_piel_oscuro_medio:','fr':':pilote_peau_mate:','ja':':パイロット_やや濃い肌色:','ko':':기장_진한_갈색_피부:','pt':':piloto_pele_morena_escura:','it':':pilota_carnagione_abbastanza_scura:','fa':':خلبان_پوست_گندمی:','id':':pilot_warna_kulit_gelap-sedang:','zh':':飞行员_中等深肤色:','ru':':пилот_темный_тон_кожи:','tr':':pilot_orta_koyu_cilt_tonu:','ar':':ربان_طائرة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002708\U0000FE0F':{'en':':pilot_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':pilot(in)_mittelhelle_hautfarbe:','es':':piloto_tono_de_piel_claro_medio:','fr':':pilote_peau_moyennement_claire:','ja':':パイロット_やや薄い肌色:','ko':':기장_연한_갈색_피부:','pt':':piloto_pele_morena_clara:','it':':pilota_carnagione_abbastanza_chiara:','fa':':خلبان_پوست_روشن:','id':':pilot_warna_kulit_cerah-sedang:','zh':':飞行员_中等浅肤色:','ru':':пилот_светлыи_тон_кожи:','tr':':pilot_orta_açık_cilt_tonu:','ar':':ربان_طائرة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FC\U0000200D\U00002708':{'en':':pilot_medium-light_skin_tone:','status':minimally_qualified,'E':12.1,'de':':pilot(in)_mittelhelle_hautfarbe:','es':':piloto_tono_de_piel_claro_medio:','fr':':pilote_peau_moyennement_claire:','ja':':パイロット_やや薄い肌色:','ko':':기장_연한_갈색_피부:','pt':':piloto_pele_morena_clara:','it':':pilota_carnagione_abbastanza_chiara:','fa':':خلبان_پوست_روشن:','id':':pilot_warna_kulit_cerah-sedang:','zh':':飞行员_中等浅肤色:','ru':':пилот_светлый_тон_кожи:','tr':':pilot_orta_açık_cilt_tonu:','ar':':ربان_طائرة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002708\U0000FE0F':{'en':':pilot_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':pilot(in)_mittlere_hautfarbe:','es':':piloto_tono_de_piel_medio:','fr':':pilote_peau_légèrement_mate:','ja':':パイロット_中間の肌色:','ko':':기장_갈색_피부:','pt':':piloto_pele_morena:','it':':pilota_carnagione_olivastra:','fa':':خلبان_پوست_طلایی:','id':':pilot_warna_kulit_sedang:','zh':':飞行员_中等肤色:','ru':':пилот_среднии_тон_кожи:','tr':':pilot_orta_cilt_tonu:','ar':':ربان_طائرة_بشرة_بلون_معتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U00002708':{'en':':pilot_medium_skin_tone:','status':minimally_qualified,'E':12.1,'de':':pilot(in)_mittlere_hautfarbe:','es':':piloto_tono_de_piel_medio:','fr':':pilote_peau_légèrement_mate:','ja':':パイロット_中間の肌色:','ko':':기장_갈색_피부:','pt':':piloto_pele_morena:','it':':pilota_carnagione_olivastra:','fa':':خلبان_پوست_طلایی:','id':':pilot_warna_kulit_sedang:','zh':':飞行员_中等肤色:','ru':':пилот_средний_тон_кожи:','tr':':pilot_orta_cilt_tonu:','ar':':ربان_طائرة_بشرة_بلون_معتدل:'},'\U0001F90C':{'en':':pinched_fingers:','status':fully_qualified,'E':13,'de':':zusammengedrückte_finger:','es':':dedos_juntos_apuntando_hacia_arriba:','fr':':bout_des_doigts_joints:','ja':':上向きにすぼめた手:','ko':':꼬집는_손가락:','pt':':dedos_comprimidos:','it':':mano_a_pigna:','fa':':انگشتان_جمع_شده:','id':':jari_terkuncup:','zh':':捏手指:','ru':':щепоть:','tr':':kıstırma_hareketi_yapan_parmaklar:','ar':':أصابع_مضمومة:'},'\U0001F90C\U0001F3FF':{'en':':pinched_fingers_dark_skin_tone:','status':fully_qualified,'E':13,'de':':zusammengedrückte_finger_dunkle_hautfarbe:','es':':dedos_juntos_apuntando_hacia_arriba_tono_de_piel_oscuro:','fr':':bout_des_doigts_joints_peau_foncée:','ja':':上向きにすぼめた手_濃い肌色:','ko':':꼬집는_손가락_검은색_피부:','pt':':dedos_comprimidos_pele_escura:','it':':mano_a_pigna_carnagione_scura:','fa':':انگشتان_جمع_شده_پوست_آبنوسی:','id':':jari_terkuncup_warna_kulit_gelap:','zh':':捏手指_较深肤色:','ru':':щепоть_очень_темный_тон_кожи:','tr':':kıstırma_hareketi_yapan_parmaklar_koyu_cilt_tonu:','ar':':أصابع_مضمومة_بشرة_بلون_غامق:'},'\U0001F90C\U0001F3FB':{'en':':pinched_fingers_light_skin_tone:','status':fully_qualified,'E':13,'de':':zusammengedrückte_finger_helle_hautfarbe:','es':':dedos_juntos_apuntando_hacia_arriba_tono_de_piel_claro:','fr':':bout_des_doigts_joints_peau_claire:','ja':':上向きにすぼめた手_薄い肌色:','ko':':꼬집는_손가락_하얀_피부:','pt':':dedos_comprimidos_pele_clara:','it':':mano_a_pigna_carnagione_chiara:','fa':':انگشتان_جمع_شده_پوست_سفید:','id':':jari_terkuncup_warna_kulit_cerah:','zh':':捏手指_较浅肤色:','ru':':щепоть_очень_светлый_тон_кожи:','tr':':kıstırma_hareketi_yapan_parmaklar_açık_cilt_tonu:','ar':':أصابع_مضمومة_بشرة_بلون_فاتح:'},'\U0001F90C\U0001F3FE':{'en':':pinched_fingers_medium-dark_skin_tone:','status':fully_qualified,'E':13,'de':':zusammengedrückte_finger_mitteldunkle_hautfarbe:','es':':dedos_juntos_apuntando_hacia_arriba_tono_de_piel_oscuro_medio:','fr':':bout_des_doigts_joints_peau_mate:','ja':':上向きにすぼめた手_やや濃い肌色:','ko':':꼬집는_손가락_진한_갈색_피부:','pt':':dedos_comprimidos_pele_morena_escura:','it':':mano_a_pigna_carnagione_abbastanza_scura:','fa':':انگشتان_جمع_شده_پوست_گندمی:','id':':jari_terkuncup_warna_kulit_gelap-sedang:','zh':':捏手指_中等深肤色:','ru':':щепоть_темный_тон_кожи:','tr':':kıstırma_hareketi_yapan_parmaklar_orta_koyu_cilt_tonu:','ar':':أصابع_مضمومة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F90C\U0001F3FC':{'en':':pinched_fingers_medium-light_skin_tone:','status':fully_qualified,'E':13,'de':':zusammengedrückte_finger_mittelhelle_hautfarbe:','es':':dedos_juntos_apuntando_hacia_arriba_tono_de_piel_claro_medio:','fr':':bout_des_doigts_joints_peau_moyennement_claire:','ja':':上向きにすぼめた手_やや薄い肌色:','ko':':꼬집는_손가락_연한_갈색_피부:','pt':':dedos_comprimidos_pele_morena_clara:','it':':mano_a_pigna_carnagione_abbastanza_chiara:','fa':':انگشتان_جمع_شده_پوست_روشن:','id':':jari_terkuncup_warna_kulit_cerah-sedang:','zh':':捏手指_中等浅肤色:','ru':':щепоть_светлый_тон_кожи:','tr':':kıstırma_hareketi_yapan_parmaklar_orta_açık_cilt_tonu:','ar':':أصابع_مضمومة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F90C\U0001F3FD':{'en':':pinched_fingers_medium_skin_tone:','status':fully_qualified,'E':13,'de':':zusammengedrückte_finger_mittlere_hautfarbe:','es':':dedos_juntos_apuntando_hacia_arriba_tono_de_piel_medio:','fr':':bout_des_doigts_joints_peau_légèrement_mate:','ja':':上向きにすぼめた手_中間の肌色:','ko':':꼬집는_손가락_갈색_피부:','pt':':dedos_comprimidos_pele_morena:','it':':mano_a_pigna_carnagione_olivastra:','fa':':انگشتان_جمع_شده_پوست_طلایی:','id':':jari_terkuncup_warna_kulit_sedang:','zh':':捏手指_中等肤色:','ru':':щепоть_средний_тон_кожи:','tr':':kıstırma_hareketi_yapan_parmaklar_orta_cilt_tonu:','ar':':أصابع_مضمومة_بشرة_بلون_معتدل:'},'\U0001F90F':{'en':':pinching_hand:','status':fully_qualified,'E':12,'de':':wenig-geste:','es':':mano_pellizcando:','fr':':pouce_et_index_rapprochés:','ja':':つまんでいる指:','ko':':꼬집는_손:','pt':':mão_beliscando:','it':':mano_che_fa_il_gesto_del_pizzico:','fa':':دست_در_حال_نیشگون_گرفتن:','id':':tangan_menjepit:','zh':':捏合的手势:','ru':':жест_«мало»:','tr':':kıstırma_hareketi_yapan_el:','ar':':يد_تشير_بمقدار_ضئيل:'},'\U0001F90F\U0001F3FF':{'en':':pinching_hand_dark_skin_tone:','status':fully_qualified,'E':12,'de':':wenig-geste_dunkle_hautfarbe:','es':':mano_pellizcando_tono_de_piel_oscuro:','fr':':pouce_et_index_rapprochés_peau_foncée:','ja':':つまんでいる指_濃い肌色:','ko':':꼬집는_손_검은색_피부:','pt':':mão_beliscando_pele_escura:','it':':mano_che_fa_il_gesto_del_pizzico_carnagione_scura:','fa':':دست_در_حال_نیشگون_گرفتن_پوست_آبنوسی:','id':':tangan_menjepit_warna_kulit_gelap:','zh':':捏合的手势_较深肤色:','ru':':жест_«мало»_очень_темный_тон_кожи:','tr':':kıstırma_hareketi_yapan_el_koyu_cilt_tonu:','ar':':يد_تشير_بمقدار_ضئيل_بشرة_بلون_غامق:'},'\U0001F90F\U0001F3FB':{'en':':pinching_hand_light_skin_tone:','status':fully_qualified,'E':12,'de':':wenig-geste_helle_hautfarbe:','es':':mano_pellizcando_tono_de_piel_claro:','fr':':pouce_et_index_rapprochés_peau_claire:','ja':':つまんでいる指_薄い肌色:','ko':':꼬집는_손_하얀_피부:','pt':':mão_beliscando_pele_clara:','it':':mano_che_fa_il_gesto_del_pizzico_carnagione_chiara:','fa':':دست_در_حال_نیشگون_گرفتن_پوست_سفید:','id':':tangan_menjepit_warna_kulit_cerah:','zh':':捏合的手势_较浅肤色:','ru':':жест_«мало»_очень_светлый_тон_кожи:','tr':':kıstırma_hareketi_yapan_el_açık_cilt_tonu:','ar':':يد_تشير_بمقدار_ضئيل_بشرة_بلون_فاتح:'},'\U0001F90F\U0001F3FE':{'en':':pinching_hand_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':wenig-geste_mitteldunkle_hautfarbe:','es':':mano_pellizcando_tono_de_piel_oscuro_medio:','fr':':pouce_et_index_rapprochés_peau_mate:','ja':':つまんでいる指_やや濃い肌色:','ko':':꼬집는_손_진한_갈색_피부:','pt':':mão_beliscando_pele_morena_escura:','it':':mano_che_fa_il_gesto_del_pizzico_carnagione_abbastanza_scura:','fa':':دست_در_حال_نیشگون_گرفتن_پوست_گندمی:','id':':tangan_menjepit_warna_kulit_gelap-sedang:','zh':':捏合的手势_中等深肤色:','ru':':жест_«мало»_темный_тон_кожи:','tr':':kıstırma_hareketi_yapan_el_orta_koyu_cilt_tonu:','ar':':يد_تشير_بمقدار_ضئيل_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F90F\U0001F3FC':{'en':':pinching_hand_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':wenig-geste_mittelhelle_hautfarbe:','es':':mano_pellizcando_tono_de_piel_claro_medio:','fr':':pouce_et_index_rapprochés_peau_moyennement_claire:','ja':':つまんでいる指_やや薄い肌色:','ko':':꼬집는_손_연한_갈색_피부:','pt':':mão_beliscando_pele_morena_clara:','it':':mano_che_fa_il_gesto_del_pizzico_carnagione_abbastanza_chiara:','fa':':دست_در_حال_نیشگون_گرفتن_پوست_روشن:','id':':tangan_menjepit_warna_kulit_cerah-sedang:','zh':':捏合的手势_中等浅肤色:','ru':':жест_«мало»_светлый_тон_кожи:','tr':':kıstırma_hareketi_yapan_el_orta_açık_cilt_tonu:','ar':':يد_تشير_بمقدار_ضئيل_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F90F\U0001F3FD':{'en':':pinching_hand_medium_skin_tone:','status':fully_qualified,'E':12,'de':':wenig-geste_mittlere_hautfarbe:','es':':mano_pellizcando_tono_de_piel_medio:','fr':':pouce_et_index_rapprochés_peau_légèrement_mate:','ja':':つまんでいる指_中間の肌色:','ko':':꼬집는_손_갈색_피부:','pt':':mão_beliscando_pele_morena:','it':':mano_che_fa_il_gesto_del_pizzico_carnagione_olivastra:','fa':':دست_در_حال_نیشگون_گرفتن_پوست_طلایی:','id':':tangan_menjepit_warna_kulit_sedang:','zh':':捏合的手势_中等肤色:','ru':':жест_«мало»_средний_тон_кожи:','tr':':kıstırma_hareketi_yapan_el_orta_cilt_tonu:','ar':':يد_تشير_بمقدار_ضئيل_بشرة_بلون_معتدل:'},'\U0001F38D':{'en':':pine_decoration:','status':fully_qualified,'E':0.6,'alias':[':bamboo:'],'de':':piniendekoration:','es':':decoración_de_pino:','fr':':bambou_décoratif:','ja':':門松:','ko':':일본_장식:','pt':':decoração_de_pinhos:','it':':bambù_decorato:','fa':':تزئین_کاج:','id':':dekorasi_pinus:','zh':':门松:','ru':':украшенный_бамбук:','tr':':çam_süsleme:','ar':':ديكور_صنوبر:'},'\U0001F34D':{'en':':pineapple:','status':fully_qualified,'E':0.6,'de':':ananas:','es':':piña:','fr':':ananas:','ja':':パイナップル:','ko':':파인애플:','pt':':abacaxi:','it':':ananas:','fa':':آناناس:','id':':nanas:','zh':':菠萝:','ru':':ананас:','tr':':ananas:','ar':':أناناس:'},'\U0001F3D3':{'en':':ping_pong:','status':fully_qualified,'E':1,'alias':[':table_tennis_paddle_and_ball:'],'de':':tischtennis:','es':':tenis_de_mesa:','fr':':ping-pong:','ja':':卓球:','ko':':탁구:','pt':':pingue-pongue:','it':':ping_pong:','fa':':پینگ_پونگ:','id':':ping_pong:','zh':':乒乓球:','ru':':настольный_теннис:','tr':':masa_tenisi:','ar':':تنس_الطاولة:'},'\U0001FA77':{'en':':pink_heart:','status':fully_qualified,'E':15,'de':':pinkes_herz:','es':':corazón_rosa:','fr':':cœur_rose:','ja':':ピンクのハート:','ko':':분홍색_하트:','pt':':coração_rosa:','it':':cuore_rosa:','fa':':قلب_صورتی:','id':':hati_merah_muda:','zh':':粉红色的心:','ru':':розовое_сердце:','tr':':pembe_kalp:','ar':':قلب_وردي:'},'\U0001F3F4\U0000200D\U00002620\U0000FE0F':{'en':':pirate_flag:','status':fully_qualified,'E':11,'de':':piratenflagge:','es':':bandera_pirata:','fr':':drapeau_de_pirate:','ja':':海賊旗:','ko':':해적_깃발:','pt':':bandeira_de_pirata:','it':':bandiera_dei_pirati:','fa':':پرچم_دزدان_دریایی:','id':':bendera_bajak_laut:','zh':':海盗旗:','ru':':пиратскии_флаг:','tr':':korsan_bayrağı:','ar':':علم_قراصنة:'},'\U0001F3F4\U0000200D\U00002620':{'en':':pirate_flag:','status':minimally_qualified,'E':11,'de':':piratenflagge:','es':':bandera_pirata:','fr':':drapeau_de_pirate:','ja':':海賊旗:','ko':':해적_깃발:','pt':':bandeira_de_pirata:','it':':bandiera_dei_pirati:','fa':':پرچم_دزدان_دریایی:','id':':bendera_bajak_laut:','zh':':海盗旗:','ru':':пиратский_флаг:','tr':':korsan_bayrağı:','ar':':علم_قراصنة:'},'\U0001F355':{'en':':pizza:','status':fully_qualified,'E':0.6,'de':':pizza:','es':':pizza:','fr':':pizza:','ja':':ピザ:','ko':':피자:','pt':':pizza:','it':':pizza:','fa':':پیتزا:','id':':pizza:','zh':':披萨:','ru':':пицца:','tr':':pizza_dilimi:','ar':':بيتزا:'},'\U0001FA85':{'en':':piñata:','status':fully_qualified,'E':13,'alias':[':pinata:'],'de':':piñata:','es':':piñata:','fr':':piñata:','ja':':ピニャータ:','ko':':피냐타:','pt':':pinhata:','it':':pentolaccia:','fa':':پینیاتا:','id':':pinata:','zh':':彩罐:','ru':':пиньята:','tr':':pinyata:','ar':':بنياتا:'},'\U0001FAA7':{'en':':placard:','status':fully_qualified,'E':13,'de':':protestschild:','es':':letrero:','fr':':pancarte:','ja':':プラカード:','ko':':플래카드:','pt':':placa:','it':':cartello:','fa':':پلاکارد:','id':':plakat:','zh':':标语牌:','ru':':плакат:','tr':':tabela:','ar':':يافطة:'},'\U0001F6D0':{'en':':place_of_worship:','status':fully_qualified,'E':1,'de':':religiöse_stätte:','es':':lugar_de_culto:','fr':':lieu_de_culte:','ja':':礼拝所:','ko':':예배공간:','pt':':local_de_culto:','it':':luogo_di_culto:','fa':':محل_عبادت:','id':':tempat_ibadah:','zh':':宗教场所:','ru':':религия:','tr':':ibadethane:','ar':':مكان_عبادة:'},'\U000025B6\U0000FE0F':{'en':':play_button:','status':fully_qualified,'E':0.6,'alias':[':arrow_forward:'],'variant':True,'de':':wiedergabe:','es':':reproducir:','fr':':bouton_lecture:','ja':':再生ボタン:','ko':':재생_버튼:','pt':':botão_reproduzir:','it':':pulsante_di_riproduzione:','fa':':دکمه_پخش:','id':':tombol_putar:','zh':':播放按钮:','ru':':воспроизведение:','tr':':oynatma_düğmesi:','ar':':زر_تشغيل:'},'\U000025B6':{'en':':play_button:','status':unqualified,'E':0.6,'alias':[':arrow_forward:'],'variant':True,'de':':wiedergabe:','es':':reproducir:','fr':':bouton_lecture:','ja':':再生ボタン:','ko':':재생_버튼:','pt':':botão_reproduzir:','it':':pulsante_di_riproduzione:','fa':':دکمهٔ_پخش:','id':':tombol_putar:','zh':':播放按钮:','ru':':воспроизведение:','tr':':oynatma_düğmesi:','ar':':زر_تشغيل:'},'\U000023EF\U0000FE0F':{'en':':play_or_pause_button:','status':fully_qualified,'E':1,'alias':[':black_right_pointing_triangle_with_double_vertical_bar:'],'variant':True,'de':':wiedergabe_oder_pause:','es':':reproducir_o_pausa:','fr':':bouton_lecture/pause:','ja':':再生/一時停止ボタン:','ko':':재생_또는_일시_정지_버튼:','pt':':botão_de_reproduzir_ou_pausar:','it':':pulsante_riproduci_pausa:','fa':':دکمه_پخش_یا_مکث:','id':':tombol_putar_atau_jeda:','zh':':播放或暂停按钮:','ru':':воспроизведение/пауза:','tr':':oynatma_veya_duraklatma_düğmesi:','ar':':زر_تشغيل_أو_إيقاف_مؤقت:'},'\U000023EF':{'en':':play_or_pause_button:','status':unqualified,'E':1,'alias':[':black_right_pointing_triangle_with_double_vertical_bar:'],'variant':True,'de':':wiedergabe_oder_pause:','es':':reproducir_o_pausa:','fr':':bouton_lecture/pause:','ja':':再生/一時停止ボタン:','ko':':재생_또는_일시_정지_버튼:','pt':':botão_de_reproduzir_ou_pausar:','it':':pulsante_riproduci/pausa:','fa':':دکمهٔ_پخش_یا_مکث:','id':':tombol_putar_atau_jeda:','zh':':播放或暂停按钮:','ru':':воспроизведение/пауза:','tr':':oynatma_veya_duraklatma_düğmesi:','ar':':زر_تشغيل_أو_إيقاف_مؤقت:'},'\U0001F6DD':{'en':':playground_slide:','status':fully_qualified,'E':14,'de':':spielplatzrutsche:','es':':tobogán:','fr':':toboggan:','ja':':すべり台:','ko':':미끄럼틀:','pt':':escorregador:','it':':scivolo:','fa':':سرسره:','id':':perosotan:','zh':':游乐场滑梯:','ru':':детская_горка:','tr':':kaydırak:','ar':':منزلق_اللعب:'},'\U0001F97A':{'en':':pleading_face:','status':fully_qualified,'E':11,'de':':bettelndes_gesicht:','es':':cara_suplicante:','fr':':visage_implorant:','ja':':訴えるような顔:','ko':':애원하는_얼굴:','pt':':rosto_implorando:','it':':faccina_supplichevole:','fa':':التماس_کنان:','id':':wajah_memelas:','zh':':恳求的脸:','ru':':умоляет:','tr':':yalvaran_yüz:','ar':':وجه_ملتمس:'},'\U0001FAA0':{'en':':plunger:','status':fully_qualified,'E':13,'de':':saugglocke:','es':':desatascador:','fr':':ventouse:','ja':':ラバーカップ:','ko':':뚫어뻥:','pt':':desentupidor:','it':':sturalavandini:','fa':':لوله_بازکن:','id':':penyedot_wc:','zh':':活塞:','ru':':вантуз:','tr':':lavabo_pompası:','ar':':مكبس_غطاس:'},'\U00002795':{'en':':plus:','status':fully_qualified,'E':0.6,'alias':[':heavy_plus_sign:'],'variant':True,'de':':plus:','es':':más:','fr':':plus:','ja':':プラス:','ko':':더하기:','pt':':símbolo_de_adição:','it':':addizione:','fa':':بعلاوه:','id':':tambah:','zh':':加:','ru':':знак_сложения:','tr':':artı:','ar':':علامة_جمع:'},'\U0001F43B\U0000200D\U00002744\U0000FE0F':{'en':':polar_bear:','status':fully_qualified,'E':13,'de':':eisbär:','es':':oso_polar:','fr':':ours_polaire:','ja':':シロクマ:','ko':':북극곰:','pt':':urso_polar:','it':':orso_polare:','fa':':خرس_قطبی:','id':':beruang_kutub:','zh':':北极熊:','ru':':морда_белого_медведя:','tr':':kutup_ayısı:','ar':':دب_قطبي:'},'\U0001F43B\U0000200D\U00002744':{'en':':polar_bear:','status':minimally_qualified,'E':13,'de':':eisbär:','es':':oso_polar:','fr':':ours_polaire:','ja':':シロクマ:','ko':':북극곰:','pt':':urso_polar:','it':':orso_polare:','fa':':خرس_قطبی:','id':':beruang_kutub:','zh':':北极熊:','ru':':морда_белого_медведя:','tr':':kutup_ayısı:','ar':':دب_قطبي:'},'\U0001F693':{'en':':police_car:','status':fully_qualified,'E':0.6,'de':':polizeiwagen:','es':':coche_de_policía:','fr':':voiture_de_police:','ja':':パトカー:','ko':':경찰차:','pt':':viatura_policial:','it':':macchina_della_polizia:','fa':':ماشین_پلیس:','id':':mobil_polisi:','zh':':警车:','ru':':полицейская_машина:','tr':':polis_arabası:','ar':':سيارة_شرطة:'},'\U0001F6A8':{'en':':police_car_light:','status':fully_qualified,'E':0.6,'alias':[':rotating_light:'],'de':':polizeilicht:','es':':luces_de_policía:','fr':':gyrophare:','ja':':パトランプ:','ko':':사이렌:','pt':':sirene:','it':':lampeggiante:','fa':':چراغ_ماشین_پلیس:','id':':lampu_mobil_polisi:','zh':':警车灯:','ru':':мигалка:','tr':':polis_lambası:','ar':':إضاءة_سيارة_شرطة_دوارة:'},'\U0001F46E':{'en':':police_officer:','status':fully_qualified,'E':0.6,'alias':[':cop:'],'de':':polizist(in):','es':':agente_de_policía:','fr':':officier_de_police:','ja':':警察官:','ko':':경찰관:','pt':':policial:','it':':agente_di_polizia:','fa':':پلیس:','id':':polisi:','zh':':警察:','ru':':полицейский:','tr':':polis_memuru:','ar':':ضابط_شرطة:'},'\U0001F46E\U0001F3FF':{'en':':police_officer_dark_skin_tone:','status':fully_qualified,'E':1,'de':':polizist(in)_dunkle_hautfarbe:','es':':agente_de_policía_tono_de_piel_oscuro:','fr':':officier_de_police_peau_foncée:','ja':':警察官_濃い肌色:','ko':':경찰관_검은색_피부:','pt':':policial_pele_escura:','it':':agente_di_polizia_carnagione_scura:','fa':':پلیس_پوست_آبنوسی:','id':':polisi_warna_kulit_gelap:','zh':':警察_较深肤色:','ru':':полицейский_очень_темный_тон_кожи:','tr':':polis_memuru_koyu_cilt_tonu:','ar':':ضابط_شرطة_بشرة_بلون_غامق:'},'\U0001F46E\U0001F3FB':{'en':':police_officer_light_skin_tone:','status':fully_qualified,'E':1,'de':':polizist(in)_helle_hautfarbe:','es':':agente_de_policía_tono_de_piel_claro:','fr':':officier_de_police_peau_claire:','ja':':警察官_薄い肌色:','ko':':경찰관_하얀_피부:','pt':':policial_pele_clara:','it':':agente_di_polizia_carnagione_chiara:','fa':':پلیس_پوست_سفید:','id':':polisi_warna_kulit_cerah:','zh':':警察_较浅肤色:','ru':':полицейский_очень_светлый_тон_кожи:','tr':':polis_memuru_açık_cilt_tonu:','ar':':ضابط_شرطة_بشرة_بلون_فاتح:'},'\U0001F46E\U0001F3FE':{'en':':police_officer_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':polizist(in)_mitteldunkle_hautfarbe:','es':':agente_de_policía_tono_de_piel_oscuro_medio:','fr':':officier_de_police_peau_mate:','ja':':警察官_やや濃い肌色:','ko':':경찰관_진한_갈색_피부:','pt':':policial_pele_morena_escura:','it':':agente_di_polizia_carnagione_abbastanza_scura:','fa':':پلیس_پوست_گندمی:','id':':polisi_warna_kulit_gelap-sedang:','zh':':警察_中等深肤色:','ru':':полицейский_темный_тон_кожи:','tr':':polis_memuru_orta_koyu_cilt_tonu:','ar':':ضابط_شرطة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F46E\U0001F3FC':{'en':':police_officer_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':polizist(in)_mittelhelle_hautfarbe:','es':':agente_de_policía_tono_de_piel_claro_medio:','fr':':officier_de_police_peau_moyennement_claire:','ja':':警察官_やや薄い肌色:','ko':':경찰관_연한_갈색_피부:','pt':':policial_pele_morena_clara:','it':':agente_di_polizia_carnagione_abbastanza_chiara:','fa':':پلیس_پوست_روشن:','id':':polisi_warna_kulit_cerah-sedang:','zh':':警察_中等浅肤色:','ru':':полицейский_светлый_тон_кожи:','tr':':polis_memuru_orta_açık_cilt_tonu:','ar':':ضابط_شرطة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F46E\U0001F3FD':{'en':':police_officer_medium_skin_tone:','status':fully_qualified,'E':1,'de':':polizist(in)_mittlere_hautfarbe:','es':':agente_de_policía_tono_de_piel_medio:','fr':':officier_de_police_peau_légèrement_mate:','ja':':警察官_中間の肌色:','ko':':경찰관_갈색_피부:','pt':':policial_pele_morena:','it':':agente_di_polizia_carnagione_olivastra:','fa':':پلیس_پوست_طلایی:','id':':polisi_warna_kulit_sedang:','zh':':警察_中等肤色:','ru':':полицейский_средний_тон_кожи:','tr':':polis_memuru_orta_cilt_tonu:','ar':':ضابط_شرطة_بشرة_بلون_معتدل:'},'\U0001F429':{'en':':poodle:','status':fully_qualified,'E':0.6,'de':':pudel:','es':':caniche:','fr':':caniche:','ja':':プードル:','ko':':푸들:','pt':':poodle:','it':':barboncino:','fa':':پودل:','id':':pudel:','zh':':贵宾犬:','ru':':пудель:','tr':':kaniş:','ar':':بودل:'},'\U0001F3B1':{'en':':pool_8_ball:','status':fully_qualified,'E':0.6,'alias':[':8ball:'],'de':':billardkugel:','es':':bola_negra_de_billar:','fr':':boule_de_billard:','ja':':ビリヤード:','ko':':당구:','pt':':bilhar:','it':':palla_da_biliardo:','fa':':بیلیارد:','id':':biliar_8_bola:','zh':':台球:','ru':':бильярд:','tr':':bilardo:','ar':':بلياردو:'},'\U0001F37F':{'en':':popcorn:','status':fully_qualified,'E':1,'de':':popcorn:','es':':palomitas:','fr':':pop-corn:','ja':':ポップコーン:','ko':':팝콘:','pt':':pipoca:','it':':popcorn:','fa':':ذرت_بوداده:','id':':popcorn:','zh':':爆米花:','ru':':попкорн:','tr':':patlamış_mısır:','ar':':فشار:'},'\U0001F3E4':{'en':':post_office:','status':fully_qualified,'E':1,'alias':[':european_post_office:'],'de':':postgebäude:','es':':oficina_de_correos_europea:','fr':':bureau_de_poste:','ja':':西洋の郵便局:','ko':':우체국:','pt':':correio:','it':':ufficio_postale:','fa':':ادارهٔ_پست:','id':':kantor_pos:','zh':':邮局:','ru':':почта:','tr':':postane:','ar':':مكتب_بريد:'},'\U0001F4EF':{'en':':postal_horn:','status':fully_qualified,'E':1,'de':':posthorn:','es':':corneta_de_posta:','fr':':cor_postal:','ja':':郵便ラッパ:','ko':':호른:','pt':':corneta_postal:','it':':corno_postale:','fa':':شیپور_پست:','id':':trompet_pos:','zh':':邮号:','ru':':почтовый_рожок:','tr':':posta_klaksonu:','ar':':جرس_بريد:'},'\U0001F4EE':{'en':':postbox:','status':fully_qualified,'E':0.6,'de':':briefkasten:','es':':buzón:','fr':':boîte_aux_lettres:','ja':':郵便ポスト:','ko':':우편함:','pt':':caixa_de_correio:','it':':cassetta_delle_lettere:','fa':':صندوق_پست:','id':':kotak_pos:','zh':':邮筒:','ru':':почтовый_ящик:','tr':':posta_kutusu:','ar':':صندوق_بريد:'},'\U0001F372':{'en':':pot_of_food:','status':fully_qualified,'E':0.6,'alias':[':stew:'],'de':':topf_mit_essen:','es':':olla_de_comida:','fr':':marmite:','ja':':なべ:','ko':':국:','pt':':panela:','it':':pentola_di_cibo:','fa':':ظرف_غذا:','id':':panci_makanan:','zh':':一锅食物:','ru':':кастрюля_с_горячей_едой:','tr':':yemek_tenceresi:','ar':':وعاء_طعام:'},'\U0001F6B0':{'en':':potable_water:','status':fully_qualified,'E':1,'de':':trinkwasser:','es':':agua_potable:','fr':':eau_potable:','ja':':飲料水:','ko':':음수대:','pt':':água_potável:','it':':acqua_potabile:','fa':':آب_آشامیدنی:','id':':air_dapat_diminum:','zh':':饮用水:','ru':':питьевая_вода:','tr':':içilebilir_su:','ar':':مياه_صالحة_للشرب:'},'\U0001F954':{'en':':potato:','status':fully_qualified,'E':3,'de':':kartoffel:','es':':patata:','fr':':pomme_de_terre:','ja':':ジャガイモ:','ko':':감자:','pt':':batata:','it':':patata:','fa':':سیب_زمینی:','id':':kentang:','zh':':土豆:','ru':':картофель:','tr':':patates:','ar':':بطاطس:'},'\U0001FAB4':{'en':':potted_plant:','status':fully_qualified,'E':13,'de':':topfpflanze:','es':':planta_de_maceta:','fr':':plante_en_pot:','ja':':鉢植え:','ko':':분재:','pt':':vaso_com_planta:','it':':pianta_in_vaso:','fa':':گیاه_گلدانی:','id':':tanaman_pot:','zh':':盆栽植物:','ru':':растение_в_горшке:','tr':':saksı_bitkisi:','ar':':نبتة_في_وعاء:'},'\U0001F357':{'en':':poultry_leg:','status':fully_qualified,'E':0.6,'de':':hähnchenschenkel:','es':':muslo_de_pollo:','fr':':cuisse_de_poulet:','ja':':鶏もも肉:','ko':':닭다리:','pt':':coxa_de_frango:','it':':coscia_di_pollo:','fa':':ران_ماکیان:','id':':kaki_ayam:','zh':':家禽的腿:','ru':':куриная_ножка:','tr':':tavuk_butu:','ar':':ساق_دجاجة:'},'\U0001F4B7':{'en':':pound_banknote:','status':fully_qualified,'E':1,'alias':[':pound:'],'de':':pfund-banknote:','es':':billete_de_libra:','fr':':billet_en_livres:','ja':':ポンド札:','ko':':파운드:','pt':':nota_de_libra:','it':':banconota_sterlina:','fa':':اسکناس_پوند:','id':':uang_kertas_pound:','zh':':英镑:','ru':':банкнота_фунта:','tr':':sterlin_banknot:','ar':':ورقة_جنيه:'},'\U0001FAD7':{'en':':pouring_liquid:','status':fully_qualified,'E':14,'de':':flüssigkeit_ausgießen:','es':':líquido_derramándose:','fr':':verser_un_liquide:','ja':':コップから注ぐ:','ko':':쏟아져_나오는_액체:','pt':':derramando_líquido:','it':':liquido_versato:','fa':':ریختن_مایع:','id':':menuang_cairan:','zh':':倾倒液体:','ru':':стакан_с_жидкостью:','tr':':dökülen_sıvı:','ar':':سكب_السائل:'},'\U0001F63E':{'en':':pouting_cat:','status':fully_qualified,'E':0.6,'de':':schmollende_katze:','es':':gato_enfadado:','fr':':chat_qui_boude:','ja':':不機嫌な猫:','ko':':뾰로통한_고양이_얼굴:','pt':':rosto_de_gato_mal-humorado:','it':':gatto_imbronciato:','fa':':گربهٔ_اخمو:','id':':wajah_kucing_marah:','zh':':生气的猫:','ru':':злой_кот:','tr':':somurtan_kedi:','ar':':وجه_قطة_عابس:'},'\U0001F4FF':{'en':':prayer_beads:','status':fully_qualified,'E':1,'de':':gebetskette:','es':':rosario:','fr':':chapelet:','ja':':数珠:','ko':':묵주:','pt':':rosário_de_oração:','it':':rosario:','fa':':تسبیح:','id':':tasbih:','zh':':念珠:','ru':':четки:','tr':':tespih:','ar':':مسبحة_صلاة:'},'\U0001FAC3':{'en':':pregnant_man:','status':fully_qualified,'E':14,'de':':schwangerer_mann:','es':':hombre_embarazado:','fr':':homme_enceint:','ja':':妊夫:','ko':':임신한_남자:','pt':':homem_grávido:','it':':uomo_incinto:','fa':':مرد_باردار:','id':':pria_hamil:','zh':':怀孕的男人:','ru':':беременный_мужчина:','tr':':hamile_adam:','ar':':رجل_حامل:'},'\U0001FAC3\U0001F3FF':{'en':':pregnant_man_dark_skin_tone:','status':fully_qualified,'E':14,'de':':schwangerer_mann_dunkle_hautfarbe:','es':':hombre_embarazado_tono_de_piel_oscuro:','fr':':homme_enceint_peau_foncée:','ja':':妊夫_濃い肌色:','ko':':임신한_남자_검은색_피부:','pt':':homem_grávido_pele_escura:','it':':uomo_incinto_carnagione_scura:','fa':':مرد_باردار_پوست_آبنوسی:','id':':pria_hamil_warna_kulit_gelap:','zh':':怀孕的男人_较深肤色:','ru':':беременный_мужчина_очень_темный_тон_кожи:','tr':':hamile_adam_koyu_cilt_tonu:','ar':':رجل_حامل_بشرة_بلون_غامق:'},'\U0001FAC3\U0001F3FB':{'en':':pregnant_man_light_skin_tone:','status':fully_qualified,'E':14,'de':':schwangerer_mann_helle_hautfarbe:','es':':hombre_embarazado_tono_de_piel_claro:','fr':':homme_enceint_peau_claire:','ja':':妊夫_薄い肌色:','ko':':임신한_남자_하얀_피부:','pt':':homem_grávido_pele_clara:','it':':uomo_incinto_carnagione_chiara:','fa':':مرد_باردار_پوست_سفید:','id':':pria_hamil_warna_kulit_cerah:','zh':':怀孕的男人_较浅肤色:','ru':':беременный_мужчина_очень_светлый_тон_кожи:','tr':':hamile_adam_açık_cilt_tonu:','ar':':رجل_حامل_بشرة_بلون_فاتح:'},'\U0001FAC3\U0001F3FE':{'en':':pregnant_man_medium-dark_skin_tone:','status':fully_qualified,'E':14,'de':':schwangerer_mann_mitteldunkle_hautfarbe:','es':':hombre_embarazado_tono_de_piel_oscuro_medio:','fr':':homme_enceint_peau_mate:','ja':':妊夫_やや濃い肌色:','ko':':임신한_남자_진한_갈색_피부:','pt':':homem_grávido_pele_morena_escura:','it':':uomo_incinto_carnagione_abbastanza_scura:','fa':':مرد_باردار_پوست_گندمی:','id':':pria_hamil_warna_kulit_gelap-sedang:','zh':':怀孕的男人_中等深肤色:','ru':':беременный_мужчина_темный_тон_кожи:','tr':':hamile_adam_orta_koyu_cilt_tonu:','ar':':رجل_حامل_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001FAC3\U0001F3FC':{'en':':pregnant_man_medium-light_skin_tone:','status':fully_qualified,'E':14,'de':':schwangerer_mann_mittelhelle_hautfarbe:','es':':hombre_embarazado_tono_de_piel_claro_medio:','fr':':homme_enceint_peau_moyennement_claire:','ja':':妊夫_やや薄い肌色:','ko':':임신한_남자_연한_갈색_피부:','pt':':homem_grávido_pele_morena_clara:','it':':uomo_incinto_carnagione_abbastanza_chiara:','fa':':مرد_باردار_پوست_روشن:','id':':pria_hamil_warna_kulit_cerah-sedang:','zh':':怀孕的男人_中等浅肤色:','ru':':беременный_мужчина_светлый_тон_кожи:','tr':':hamile_adam_orta_açık_cilt_tonu:','ar':':رجل_حامل_بشرة_بلون_فاتح_ومعتدل:'},'\U0001FAC3\U0001F3FD':{'en':':pregnant_man_medium_skin_tone:','status':fully_qualified,'E':14,'de':':schwangerer_mann_mittlere_hautfarbe:','es':':hombre_embarazado_tono_de_piel_medio:','fr':':homme_enceint_peau_légèrement_mate:','ja':':妊夫_中間の肌色:','ko':':임신한_남자_갈색_피부:','pt':':homem_grávido_pele_morena:','it':':uomo_incinto_carnagione_olivastra:','fa':':مرد_باردار_پوست_طلایی:','id':':pria_hamil_warna_kulit_sedang:','zh':':怀孕的男人_中等肤色:','ru':':беременный_мужчина_средний_тон_кожи:','tr':':hamile_adam_orta_cilt_tonu:','ar':':رجل_حامل_بشرة_بلون_معتدل:'},'\U0001FAC4':{'en':':pregnant_person:','status':fully_qualified,'E':14,'de':':schwangere_person:','es':':persona_embarazada:','fr':':personne_enceinte:','ja':':妊娠した人:','ko':':임신한_사람:','pt':':pessoa_grávida:','it':':persona_incinta:','fa':':آدم_باردار:','id':':orang_hamil:','zh':':怀孕的人:','ru':':беременный_человек:','tr':':hamile_kişi:','ar':':شخص_حامل:'},'\U0001FAC4\U0001F3FF':{'en':':pregnant_person_dark_skin_tone:','status':fully_qualified,'E':14,'de':':schwangere_person_dunkle_hautfarbe:','es':':persona_embarazada_tono_de_piel_oscuro:','fr':':personne_enceinte_peau_foncée:','ja':':妊娠した人_濃い肌色:','ko':':임신한_사람_검은색_피부:','pt':':pessoa_grávida_pele_escura:','it':':persona_incinta_carnagione_scura:','fa':':آدم_باردار_پوست_آبنوسی:','id':':orang_hamil_warna_kulit_gelap:','zh':':怀孕的人_较深肤色:','ru':':беременный_человек_очень_темный_тон_кожи:','tr':':hamile_kişi_koyu_cilt_tonu:','ar':':شخص_حامل_بشرة_بلون_غامق:'},'\U0001FAC4\U0001F3FB':{'en':':pregnant_person_light_skin_tone:','status':fully_qualified,'E':14,'de':':schwangere_person_helle_hautfarbe:','es':':persona_embarazada_tono_de_piel_claro:','fr':':personne_enceinte_peau_claire:','ja':':妊娠した人_薄い肌色:','ko':':임신한_사람_하얀_피부:','pt':':pessoa_grávida_pele_clara:','it':':persona_incinta_carnagione_chiara:','fa':':آدم_باردار_پوست_سفید:','id':':orang_hamil_warna_kulit_cerah:','zh':':怀孕的人_较浅肤色:','ru':':беременный_человек_очень_светлый_тон_кожи:','tr':':hamile_kişi_açık_cilt_tonu:','ar':':شخص_حامل_بشرة_بلون_فاتح:'},'\U0001FAC4\U0001F3FE':{'en':':pregnant_person_medium-dark_skin_tone:','status':fully_qualified,'E':14,'de':':schwangere_person_mitteldunkle_hautfarbe:','es':':persona_embarazada_tono_de_piel_oscuro_medio:','fr':':personne_enceinte_peau_mate:','ja':':妊娠した人_やや濃い肌色:','ko':':임신한_사람_진한_갈색_피부:','pt':':pessoa_grávida_pele_morena_escura:','it':':persona_incinta_carnagione_abbastanza_scura:','fa':':آدم_باردار_پوست_گندمی:','id':':orang_hamil_warna_kulit_gelap-sedang:','zh':':怀孕的人_中等深肤色:','ru':':беременный_человек_темный_тон_кожи:','tr':':hamile_kişi_orta_koyu_cilt_tonu:','ar':':شخص_حامل_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001FAC4\U0001F3FC':{'en':':pregnant_person_medium-light_skin_tone:','status':fully_qualified,'E':14,'de':':schwangere_person_mittelhelle_hautfarbe:','es':':persona_embarazada_tono_de_piel_claro_medio:','fr':':personne_enceinte_peau_moyennement_claire:','ja':':妊娠した人_やや薄い肌色:','ko':':임신한_사람_연한_갈색_피부:','pt':':pessoa_grávida_pele_morena_clara:','it':':persona_incinta_carnagione_abbastanza_chiara:','fa':':آدم_باردار_پوست_روشن:','id':':orang_hamil_warna_kulit_cerah-sedang:','zh':':怀孕的人_中等浅肤色:','ru':':беременный_человек_светлый_тон_кожи:','tr':':hamile_kişi_orta_açık_cilt_tonu:','ar':':شخص_حامل_بشرة_بلون_فاتح_ومعتدل:'},'\U0001FAC4\U0001F3FD':{'en':':pregnant_person_medium_skin_tone:','status':fully_qualified,'E':14,'de':':schwangere_person_mittlere_hautfarbe:','es':':persona_embarazada_tono_de_piel_medio:','fr':':personne_enceinte_peau_légèrement_mate:','ja':':妊娠した人_中間の肌色:','ko':':임신한_사람_갈색_피부:','pt':':pessoa_grávida_pele_morena:','it':':persona_incinta_carnagione_olivastra:','fa':':آدم_باردار_پوست_طلایی:','id':':orang_hamil_warna_kulit_sedang:','zh':':怀孕的人_中等肤色:','ru':':беременный_человек_средний_тон_кожи:','tr':':hamile_kişi_orta_cilt_tonu:','ar':':شخص_حامل_بشرة_بلون_معتدل:'},'\U0001F930':{'en':':pregnant_woman:','status':fully_qualified,'E':3,'de':':schwangere_frau:','es':':mujer_embarazada:','fr':':femme_enceinte:','ja':':妊婦:','ko':':임산부:','pt':':grávida:','it':':donna_incinta:','fa':':زن_باردار:','id':':wanita_hamil:','zh':':孕妇:','ru':':беременная_женщина:','tr':':hamile_kadın:','ar':':امرأة_حامل:'},'\U0001F930\U0001F3FF':{'en':':pregnant_woman_dark_skin_tone:','status':fully_qualified,'E':3,'de':':schwangere_frau_dunkle_hautfarbe:','es':':mujer_embarazada_tono_de_piel_oscuro:','fr':':femme_enceinte_peau_foncée:','ja':':妊婦_濃い肌色:','ko':':임산부_검은색_피부:','pt':':grávida_pele_escura:','it':':donna_incinta_carnagione_scura:','fa':':زن_باردار_پوست_آبنوسی:','id':':wanita_hamil_warna_kulit_gelap:','zh':':孕妇_较深肤色:','ru':':беременная_женщина_очень_темный_тон_кожи:','tr':':hamile_kadın_koyu_cilt_tonu:','ar':':امرأة_حامل_بشرة_بلون_غامق:'},'\U0001F930\U0001F3FB':{'en':':pregnant_woman_light_skin_tone:','status':fully_qualified,'E':3,'de':':schwangere_frau_helle_hautfarbe:','es':':mujer_embarazada_tono_de_piel_claro:','fr':':femme_enceinte_peau_claire:','ja':':妊婦_薄い肌色:','ko':':임산부_하얀_피부:','pt':':grávida_pele_clara:','it':':donna_incinta_carnagione_chiara:','fa':':زن_باردار_پوست_سفید:','id':':wanita_hamil_warna_kulit_cerah:','zh':':孕妇_较浅肤色:','ru':':беременная_женщина_очень_светлый_тон_кожи:','tr':':hamile_kadın_açık_cilt_tonu:','ar':':امرأة_حامل_بشرة_بلون_فاتح:'},'\U0001F930\U0001F3FE':{'en':':pregnant_woman_medium-dark_skin_tone:','status':fully_qualified,'E':3,'de':':schwangere_frau_mitteldunkle_hautfarbe:','es':':mujer_embarazada_tono_de_piel_oscuro_medio:','fr':':femme_enceinte_peau_mate:','ja':':妊婦_やや濃い肌色:','ko':':임산부_진한_갈색_피부:','pt':':grávida_pele_morena_escura:','it':':donna_incinta_carnagione_abbastanza_scura:','fa':':زن_باردار_پوست_گندمی:','id':':wanita_hamil_warna_kulit_gelap-sedang:','zh':':孕妇_中等深肤色:','ru':':беременная_женщина_темный_тон_кожи:','tr':':hamile_kadın_orta_koyu_cilt_tonu:','ar':':امرأة_حامل_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F930\U0001F3FC':{'en':':pregnant_woman_medium-light_skin_tone:','status':fully_qualified,'E':3,'de':':schwangere_frau_mittelhelle_hautfarbe:','es':':mujer_embarazada_tono_de_piel_claro_medio:','fr':':femme_enceinte_peau_moyennement_claire:','ja':':妊婦_やや薄い肌色:','ko':':임산부_연한_갈색_피부:','pt':':grávida_pele_morena_clara:','it':':donna_incinta_carnagione_abbastanza_chiara:','fa':':زن_باردار_پوست_روشن:','id':':wanita_hamil_warna_kulit_cerah-sedang:','zh':':孕妇_中等浅肤色:','ru':':беременная_женщина_светлый_тон_кожи:','tr':':hamile_kadın_orta_açık_cilt_tonu:','ar':':امرأة_حامل_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F930\U0001F3FD':{'en':':pregnant_woman_medium_skin_tone:','status':fully_qualified,'E':3,'de':':schwangere_frau_mittlere_hautfarbe:','es':':mujer_embarazada_tono_de_piel_medio:','fr':':femme_enceinte_peau_légèrement_mate:','ja':':妊婦_中間の肌色:','ko':':임산부_갈색_피부:','pt':':grávida_pele_morena:','it':':donna_incinta_carnagione_olivastra:','fa':':زن_باردار_پوست_طلایی:','id':':wanita_hamil_warna_kulit_sedang:','zh':':孕妇_中等肤色:','ru':':беременная_женщина_средний_тон_кожи:','tr':':hamile_kadın_orta_cilt_tonu:','ar':':امرأة_حامل_بشرة_بلون_معتدل:'},'\U0001F968':{'en':':pretzel:','status':fully_qualified,'E':5,'de':':brezel:','es':':bretzel:','fr':':bretzel:','ja':':プレッツェル:','ko':':프레첼:','pt':':pretzel:','it':':pretzel:','fa':':چوب_شور:','id':':pretzel:','zh':':椒盐卷饼:','ru':':крендель:','tr':':pretzel:','ar':':بريتزل:'},'\U0001F934':{'en':':prince:','status':fully_qualified,'E':3,'de':':prinz:','es':':príncipe:','fr':':prince:','ja':':プリンス:','ko':':왕자:','pt':':príncipe:','it':':principe:','fa':':شاهزاده:','id':':pangeran:','zh':':王子:','ru':':принц:','tr':':prens:','ar':':أمير:'},'\U0001F934\U0001F3FF':{'en':':prince_dark_skin_tone:','status':fully_qualified,'E':3,'de':':prinz_dunkle_hautfarbe:','es':':príncipe_tono_de_piel_oscuro:','fr':':prince_peau_foncée:','ja':':プリンス_濃い肌色:','ko':':왕자_검은색_피부:','pt':':príncipe_pele_escura:','it':':principe_carnagione_scura:','fa':':شاهزاده_پوست_آبنوسی:','id':':pangeran_warna_kulit_gelap:','zh':':王子_较深肤色:','ru':':принц_очень_темный_тон_кожи:','tr':':prens_koyu_cilt_tonu:','ar':':أمير_بشرة_بلون_غامق:'},'\U0001F934\U0001F3FB':{'en':':prince_light_skin_tone:','status':fully_qualified,'E':3,'de':':prinz_helle_hautfarbe:','es':':príncipe_tono_de_piel_claro:','fr':':prince_peau_claire:','ja':':プリンス_薄い肌色:','ko':':왕자_하얀_피부:','pt':':príncipe_pele_clara:','it':':principe_carnagione_chiara:','fa':':شاهزاده_پوست_سفید:','id':':pangeran_warna_kulit_cerah:','zh':':王子_较浅肤色:','ru':':принц_очень_светлый_тон_кожи:','tr':':prens_açık_cilt_tonu:','ar':':أمير_بشرة_بلون_فاتح:'},'\U0001F934\U0001F3FE':{'en':':prince_medium-dark_skin_tone:','status':fully_qualified,'E':3,'de':':prinz_mitteldunkle_hautfarbe:','es':':príncipe_tono_de_piel_oscuro_medio:','fr':':prince_peau_mate:','ja':':プリンス_やや濃い肌色:','ko':':왕자_진한_갈색_피부:','pt':':príncipe_pele_morena_escura:','it':':principe_carnagione_abbastanza_scura:','fa':':شاهزاده_پوست_گندمی:','id':':pangeran_warna_kulit_gelap-sedang:','zh':':王子_中等深肤色:','ru':':принц_темный_тон_кожи:','tr':':prens_orta_koyu_cilt_tonu:','ar':':أمير_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F934\U0001F3FC':{'en':':prince_medium-light_skin_tone:','status':fully_qualified,'E':3,'de':':prinz_mittelhelle_hautfarbe:','es':':príncipe_tono_de_piel_claro_medio:','fr':':prince_peau_moyennement_claire:','ja':':プリンス_やや薄い肌色:','ko':':왕자_연한_갈색_피부:','pt':':príncipe_pele_morena_clara:','it':':principe_carnagione_abbastanza_chiara:','fa':':شاهزاده_پوست_روشن:','id':':pangeran_warna_kulit_cerah-sedang:','zh':':王子_中等浅肤色:','ru':':принц_светлый_тон_кожи:','tr':':prens_orta_açık_cilt_tonu:','ar':':أمير_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F934\U0001F3FD':{'en':':prince_medium_skin_tone:','status':fully_qualified,'E':3,'de':':prinz_mittlere_hautfarbe:','es':':príncipe_tono_de_piel_medio:','fr':':prince_peau_légèrement_mate:','ja':':プリンス_中間の肌色:','ko':':왕자_갈색_피부:','pt':':príncipe_pele_morena:','it':':principe_carnagione_olivastra:','fa':':شاهزاده_پوست_طلایی:','id':':pangeran_warna_kulit_sedang:','zh':':王子_中等肤色:','ru':':принц_средний_тон_кожи:','tr':':prens_orta_cilt_tonu:','ar':':أمير_بشرة_بلون_معتدل:'},'\U0001F478':{'en':':princess:','status':fully_qualified,'E':0.6,'de':':prinzessin:','es':':princesa:','fr':':princesse:','ja':':プリンセス:','ko':':공주:','pt':':princesa:','it':':principessa:','fa':':شاهزاده_خانم:','id':':putri:','zh':':公主:','ru':':принцесса:','tr':':prenses:','ar':':أميرة:'},'\U0001F478\U0001F3FF':{'en':':princess_dark_skin_tone:','status':fully_qualified,'E':1,'de':':prinzessin_dunkle_hautfarbe:','es':':princesa_tono_de_piel_oscuro:','fr':':princesse_peau_foncée:','ja':':プリンセス_濃い肌色:','ko':':공주_검은색_피부:','pt':':princesa_pele_escura:','it':':principessa_carnagione_scura:','fa':':شاهزاده_خانم_پوست_آبنوسی:','id':':putri_warna_kulit_gelap:','zh':':公主_较深肤色:','ru':':принцесса_очень_темный_тон_кожи:','tr':':prenses_koyu_cilt_tonu:','ar':':أميرة_بشرة_بلون_غامق:'},'\U0001F478\U0001F3FB':{'en':':princess_light_skin_tone:','status':fully_qualified,'E':1,'de':':prinzessin_helle_hautfarbe:','es':':princesa_tono_de_piel_claro:','fr':':princesse_peau_claire:','ja':':プリンセス_薄い肌色:','ko':':공주_하얀_피부:','pt':':princesa_pele_clara:','it':':principessa_carnagione_chiara:','fa':':شاهزاده_خانم_پوست_سفید:','id':':putri_warna_kulit_cerah:','zh':':公主_较浅肤色:','ru':':принцесса_очень_светлый_тон_кожи:','tr':':prenses_açık_cilt_tonu:','ar':':أميرة_بشرة_بلون_فاتح:'},'\U0001F478\U0001F3FE':{'en':':princess_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':prinzessin_mitteldunkle_hautfarbe:','es':':princesa_tono_de_piel_oscuro_medio:','fr':':princesse_peau_mate:','ja':':プリンセス_やや濃い肌色:','ko':':공주_진한_갈색_피부:','pt':':princesa_pele_morena_escura:','it':':principessa_carnagione_abbastanza_scura:','fa':':شاهزاده_خانم_پوست_گندمی:','id':':putri_warna_kulit_gelap-sedang:','zh':':公主_中等深肤色:','ru':':принцесса_темный_тон_кожи:','tr':':prenses_orta_koyu_cilt_tonu:','ar':':أميرة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F478\U0001F3FC':{'en':':princess_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':prinzessin_mittelhelle_hautfarbe:','es':':princesa_tono_de_piel_claro_medio:','fr':':princesse_peau_moyennement_claire:','ja':':プリンセス_やや薄い肌色:','ko':':공주_연한_갈색_피부:','pt':':princesa_pele_morena_clara:','it':':principessa_carnagione_abbastanza_chiara:','fa':':شاهزاده_خانم_پوست_روشن:','id':':putri_warna_kulit_cerah-sedang:','zh':':公主_中等浅肤色:','ru':':принцесса_светлый_тон_кожи:','tr':':prenses_orta_açık_cilt_tonu:','ar':':أميرة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F478\U0001F3FD':{'en':':princess_medium_skin_tone:','status':fully_qualified,'E':1,'de':':prinzessin_mittlere_hautfarbe:','es':':princesa_tono_de_piel_medio:','fr':':princesse_peau_légèrement_mate:','ja':':プリンセス_中間の肌色:','ko':':공주_갈색_피부:','pt':':princesa_pele_morena:','it':':principessa_carnagione_olivastra:','fa':':شاهزاده_خانم_پوست_طلایی:','id':':putri_warna_kulit_sedang:','zh':':公主_中等肤色:','ru':':принцесса_средний_тон_кожи:','tr':':prenses_orta_cilt_tonu:','ar':':أميرة_بشرة_بلون_معتدل:'},'\U0001F5A8\U0000FE0F':{'en':':printer:','status':fully_qualified,'E':0.7,'variant':True,'de':':drucker:','es':':impresora:','fr':':imprimante:','ja':':プリンタ:','ko':':프린터:','pt':':impressora:','it':':stampante:','fa':':چاپگر:','id':':printer:','zh':':打印机:','ru':':принтер:','tr':':yazıcı:','ar':':طابعة:'},'\U0001F5A8':{'en':':printer:','status':unqualified,'E':0.7,'variant':True,'de':':drucker:','es':':impresora:','fr':':imprimante:','ja':':プリンタ:','ko':':프린터:','pt':':impressora:','it':':stampante:','fa':':چاپگر:','id':':printer:','zh':':打印机:','ru':':принтер:','tr':':yazıcı:','ar':':طابعة:'},'\U0001F6AB':{'en':':prohibited:','status':fully_qualified,'E':0.6,'alias':[':no_entry_sign:'],'de':':verboten:','es':':prohibido:','fr':':symbole_d’interdiction:','ja':':車両通行止め:','ko':':금지:','pt':':proibido:','it':':segnale_di_divieto:','fa':':ممنوع:','id':':dilarang:','zh':':禁止:','ru':':запрещено:','tr':':girmek_yasak:','ar':':ممنوع:'},'\U0001F7E3':{'en':':purple_circle:','status':fully_qualified,'E':12,'de':':lila_punkt:','es':':círculo_morado:','fr':':disque_violet:','ja':':紫の丸:','ko':':보라색_원:','pt':':círculo_roxo:','it':':cerchio_viola:','fa':':دایره_بنفش:','id':':lingkaran_ungu:','zh':':紫色圆:','ru':':фиолетовый_круг:','tr':':mor_daire:','ar':':دائرة_بنفسجية:'},'\U0001F49C':{'en':':purple_heart:','status':fully_qualified,'E':0.6,'de':':lila_herz:','es':':corazón_morado:','fr':':cœur_violet:','ja':':紫のハート:','ko':':보라색_하트:','pt':':coração_roxo:','it':':cuore_viola:','fa':':قلب_بنفش:','id':':hati_ungu:','zh':':紫心:','ru':':фиолетовое_сердце:','tr':':mor_kalp:','ar':':قلب_أرجواني:'},'\U0001F7EA':{'en':':purple_square:','status':fully_qualified,'E':12,'de':':lila_quadrat:','es':':cuadrado_morado:','fr':':carré_violet:','ja':':紫の四角:','ko':':보라색_사각형:','pt':':quadrado_roxo:','it':':quadrato_viola:','fa':':مربع_بنفش:','id':':persegi_ungu:','zh':':紫色方块:','ru':':фиолетовый_квадрат:','tr':':mor_kare:','ar':':مربع_بنفسجي:'},'\U0001F45B':{'en':':purse:','status':fully_qualified,'E':0.6,'de':':geldbörse:','es':':monedero:','fr':':porte-monnaie:','ja':':がま口:','ko':':지갑:','pt':':bolsinha:','it':':borsellino:','fa':':کیف_دستی:','id':':dompet:','zh':':钱包:','ru':':кошелек:','tr':':cüzdan:','ar':':كيس_نقود:'},'\U0001F4CC':{'en':':pushpin:','status':fully_qualified,'E':0.6,'de':':reißzwecke:','es':':chincheta:','fr':':punaise:','ja':':押しピン:','ko':':압정:','pt':':tacha:','it':':puntina:','fa':':سنجاق:','id':':paku_penanda:','zh':':图钉:','ru':':канцелярская_кнопка:','tr':':raptiye:','ar':':دبوس_ضغط:'},'\U0001F9E9':{'en':':puzzle_piece:','status':fully_qualified,'E':11,'alias':[':jigsaw:'],'de':':puzzleteil:','es':':pieza_de_puzle:','fr':':pièce_de_puzzle:','ja':':ジグソーパズル:','ko':':퍼즐:','pt':':quebra-cabeça:','it':':pezzo_di_puzzle:','fa':':قطعه_پازل:','id':':keping_puzzle:','zh':':拼图:','ru':':пазл:','tr':':yapboz_parçası:','ar':':أحجية_الصور_المقطوعة:'},'\U0001F407':{'en':':rabbit:','status':fully_qualified,'E':1,'alias':[':rabbit2:'],'de':':hase:','es':':conejo:','fr':':lapin:','ja':':ウサギ:','ko':':토끼:','pt':':coelho:','it':':coniglio:','fa':':خرگوش:','id':':kelinci:','zh':':兔子:','ru':':кролик:','tr':':tavşan:','ar':':أرنب:'},'\U0001F430':{'en':':rabbit_face:','status':fully_qualified,'E':0.6,'alias':[':rabbit:'],'de':':hasengesicht:','es':':cara_de_conejo:','fr':':tête_de_lapin:','ja':':ウサギの顔:','ko':':토끼_얼굴:','pt':':rosto_de_coelho:','it':':muso_di_coniglio:','fa':':صورت_خرگوش:','id':':wajah_kelinci:','zh':':兔子头:','ru':':морда_кролика:','tr':':tavşan_yüzü:','ar':':وجه_أرنب:'},'\U0001F99D':{'en':':raccoon:','status':fully_qualified,'E':11,'de':':waschbär:','es':':mapache:','fr':':raton_laveur:','ja':':アライグマ:','ko':':너구리:','pt':':guaxinim:','it':':procione:','fa':':راکون:','id':':rakun:','zh':':浣熊:','ru':':морда_енота:','tr':':rakun:','ar':':راكون:'},'\U0001F3CE\U0000FE0F':{'en':':racing_car:','status':fully_qualified,'E':0.7,'variant':True,'de':':rennauto:','es':':coche_de_carreras:','fr':':voiture_de_course:','ja':':レーシングカー:','ko':':경주용_자동차:','pt':':carro_de_corrida:','it':':auto_da_corsa:','fa':':ماشین_مسابقه:','id':':mobil_balap:','zh':':赛车:','ru':':гоночныи_автомобиль:','tr':':yarış_arabası:','ar':':سيارة_سباق:'},'\U0001F3CE':{'en':':racing_car:','status':unqualified,'E':0.7,'variant':True,'de':':rennauto:','es':':coche_de_carreras:','fr':':voiture_de_course:','ja':':レーシングカー:','ko':':경주용_자동차:','pt':':carro_de_corrida:','it':':auto_da_corsa:','fa':':ماشین_مسابقه:','id':':mobil_balap:','zh':':赛车:','ru':':гоночный_автомобиль:','tr':':yarış_arabası:','ar':':سيارة_سباق:'},'\U0001F4FB':{'en':':radio:','status':fully_qualified,'E':0.6,'variant':True,'de':':radio:','es':':radio:','fr':':radio:','ja':':ラジオ:','ko':':라디오:','pt':':rádio:','it':':radio:','fa':':رادیو:','id':':radio:','zh':':收音机:','ru':':радио:','tr':':radyo:','ar':':راديو:'},'\U0001F518':{'en':':radio_button:','status':fully_qualified,'E':0.6,'de':':optionsfeld:','es':':botón_de_opción:','fr':':bouton_radio:','ja':':ラジオボタン:','ko':':라디오_버튼:','pt':':botão_de_opção:','it':':pulsante_rotondo:','fa':':دکمهٔ_رادیویی:','id':':tombol_radio:','zh':':单选按钮:','ru':':круглая_кнопка:','tr':':radyo_düğmesi:','ar':':زر_تبادلي:'},'\U00002622\U0000FE0F':{'en':':radioactive:','status':fully_qualified,'E':1,'alias':[':radioactive_sign:'],'variant':True,'de':':radioaktiv:','es':':radiactivo:','fr':':radioactif:','ja':':放射能:','ko':':방사능:','pt':':radioativo:','it':':simbolo_della_radioattività:','fa':':رادیواکتیو:','id':':radioaktif:','zh':':辐射:','ru':':радиация:','tr':':radyoaktif:','ar':':إشعاع:'},'\U00002622':{'en':':radioactive:','status':unqualified,'E':1,'alias':[':radioactive_sign:'],'variant':True,'de':':radioaktiv:','es':':radiactivo:','fr':':radioactif:','ja':':放射能:','ko':':방사능:','pt':':radioativo:','it':':simbolo_della_radioattività:','fa':':رادیواکتیو:','id':':radioaktif:','zh':':辐射:','ru':':радиация:','tr':':radyoaktif:','ar':':إشعاع:'},'\U0001F683':{'en':':railway_car:','status':fully_qualified,'E':0.6,'de':':eisenbahnwagen:','es':':vagón:','fr':':wagon:','ja':':電車:','ko':':전철:','pt':':vagão_de_trem:','it':':vagone:','fa':':واگن_ریلی:','id':':gerbong:','zh':':轨道车:','ru':':вагон:','tr':':vagon:','ar':':عربة_قطار:'},'\U0001F6E4\U0000FE0F':{'en':':railway_track:','status':fully_qualified,'E':0.7,'variant':True,'de':':bahngleis:','es':':vía_de_tren:','fr':':voie_ferrée:','ja':':線路:','ko':':철도_선로:','pt':':trilhos:','it':':binari:','fa':':ریل_قطار:','id':':rel_kereta:','zh':':铁轨:','ru':':железная_дорога:','tr':':tren_yolu:','ar':':سكة_حديد:'},'\U0001F6E4':{'en':':railway_track:','status':unqualified,'E':0.7,'variant':True,'de':':bahngleis:','es':':vía_de_tren:','fr':':voie_ferrée:','ja':':線路:','ko':':철도_선로:','pt':':trilhos:','it':':binari:','fa':':ریل_قطار:','id':':rel_kereta:','zh':':铁轨:','ru':':железная_дорога:','tr':':tren_yolu:','ar':':سكة_حديد:'},'\U0001F308':{'en':':rainbow:','status':fully_qualified,'E':0.6,'de':':regenbogen:','es':':arcoíris:','fr':':arc-en-ciel:','ja':':虹:','ko':':무지개:','pt':':arco-íris:','it':':arcobaleno:','fa':':رنگین_کمان:','id':':pelangi:','zh':':彩虹:','ru':':радуга:','tr':':gökkuşağı:','ar':':ألوان_الطيف:'},'\U0001F3F3\U0000FE0F\U0000200D\U0001F308':{'en':':rainbow_flag:','status':fully_qualified,'E':4,'de':':regenbogenflagge:','es':':bandera_del_arcoíris:','fr':':drapeau_arc-en-ciel:','ja':':レインボーフラッグ:','ko':':무지개_깃발:','pt':':bandeira_do_arco-íris:','it':':bandiera_arcobaleno:','fa':':پرچم_رنگین_کمان:','id':':bendera_pelangi:','zh':':彩虹旗:','ru':':радужныи_флаг:','tr':':gökkuşağı_bayrağı:','ar':':علم_ألوان_طيف:'},'\U0001F3F3\U0000200D\U0001F308':{'en':':rainbow_flag:','status':unqualified,'E':4,'de':':regenbogenflagge:','es':':bandera_del_arcoíris:','fr':':drapeau_arc-en-ciel:','ja':':レインボーフラッグ:','ko':':무지개_깃발:','pt':':bandeira_do_arco-íris:','it':':bandiera_arcobaleno:','fa':':پرچم_رنگین_کمان:','id':':bendera_pelangi:','zh':':彩虹旗:','ru':':радужный_флаг:','tr':':gökkuşağı_bayrağı:','ar':':علم_ألوان_طيف:'},'\U0001F91A':{'en':':raised_back_of_hand:','status':fully_qualified,'E':3,'de':':erhobene_hand_von_hinten:','es':':dorso_de_la_mano:','fr':':dos_de_main_levée:','ja':':手の甲:','ko':':손등:','pt':':dorso_da_mão_levantado:','it':':dorso_mano_alzata:','fa':':پشت_دست_روبه_بالا:','id':':angkat_tangan:','zh':':立起的手背:','ru':':поднятая_рука:','tr':':elinin_tersini_kaldırma:','ar':':ظهر_يد_مرفوعة:'},'\U0001F91A\U0001F3FF':{'en':':raised_back_of_hand_dark_skin_tone:','status':fully_qualified,'E':3,'de':':erhobene_hand_von_hinten_dunkle_hautfarbe:','es':':dorso_de_la_mano_tono_de_piel_oscuro:','fr':':dos_de_main_levée_peau_foncée:','ja':':手の甲_濃い肌色:','ko':':손등_검은색_피부:','pt':':dorso_da_mão_levantado_pele_escura:','it':':dorso_mano_alzata_carnagione_scura:','fa':':پشت_دست_روبه_بالا_پوست_آبنوسی:','id':':angkat_tangan_warna_kulit_gelap:','zh':':立起的手背_较深肤色:','ru':':поднятая_рука_очень_темный_тон_кожи:','tr':':elinin_tersini_kaldırma_koyu_cilt_tonu:','ar':':ظهر_يد_مرفوعة_بشرة_بلون_غامق:'},'\U0001F91A\U0001F3FB':{'en':':raised_back_of_hand_light_skin_tone:','status':fully_qualified,'E':3,'de':':erhobene_hand_von_hinten_helle_hautfarbe:','es':':dorso_de_la_mano_tono_de_piel_claro:','fr':':dos_de_main_levée_peau_claire:','ja':':手の甲_薄い肌色:','ko':':손등_하얀_피부:','pt':':dorso_da_mão_levantado_pele_clara:','it':':dorso_mano_alzata_carnagione_chiara:','fa':':پشت_دست_روبه_بالا_پوست_سفید:','id':':angkat_tangan_warna_kulit_cerah:','zh':':立起的手背_较浅肤色:','ru':':поднятая_рука_очень_светлый_тон_кожи:','tr':':elinin_tersini_kaldırma_açık_cilt_tonu:','ar':':ظهر_يد_مرفوعة_بشرة_بلون_فاتح:'},'\U0001F91A\U0001F3FE':{'en':':raised_back_of_hand_medium-dark_skin_tone:','status':fully_qualified,'E':3,'de':':erhobene_hand_von_hinten_mitteldunkle_hautfarbe:','es':':dorso_de_la_mano_tono_de_piel_oscuro_medio:','fr':':dos_de_main_levée_peau_mate:','ja':':手の甲_やや濃い肌色:','ko':':손등_진한_갈색_피부:','pt':':dorso_da_mão_levantado_pele_morena_escura:','it':':dorso_mano_alzata_carnagione_abbastanza_scura:','fa':':پشت_دست_روبه_بالا_پوست_گندمی:','id':':angkat_tangan_warna_kulit_gelap-sedang:','zh':':立起的手背_中等深肤色:','ru':':поднятая_рука_темный_тон_кожи:','tr':':elinin_tersini_kaldırma_orta_koyu_cilt_tonu:','ar':':ظهر_يد_مرفوعة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F91A\U0001F3FC':{'en':':raised_back_of_hand_medium-light_skin_tone:','status':fully_qualified,'E':3,'de':':erhobene_hand_von_hinten_mittelhelle_hautfarbe:','es':':dorso_de_la_mano_tono_de_piel_claro_medio:','fr':':dos_de_main_levée_peau_moyennement_claire:','ja':':手の甲_やや薄い肌色:','ko':':손등_연한_갈색_피부:','pt':':dorso_da_mão_levantado_pele_morena_clara:','it':':dorso_mano_alzata_carnagione_abbastanza_chiara:','fa':':پشت_دست_روبه_بالا_پوست_روشن:','id':':angkat_tangan_warna_kulit_cerah-sedang:','zh':':立起的手背_中等浅肤色:','ru':':поднятая_рука_светлый_тон_кожи:','tr':':elinin_tersini_kaldırma_orta_açık_cilt_tonu:','ar':':ظهر_يد_مرفوعة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F91A\U0001F3FD':{'en':':raised_back_of_hand_medium_skin_tone:','status':fully_qualified,'E':3,'de':':erhobene_hand_von_hinten_mittlere_hautfarbe:','es':':dorso_de_la_mano_tono_de_piel_medio:','fr':':dos_de_main_levée_peau_légèrement_mate:','ja':':手の甲_中間の肌色:','ko':':손등_갈색_피부:','pt':':dorso_da_mão_levantado_pele_morena:','it':':dorso_mano_alzata_carnagione_olivastra:','fa':':پشت_دست_روبه_بالا_پوست_طلایی:','id':':angkat_tangan_warna_kulit_sedang:','zh':':立起的手背_中等肤色:','ru':':поднятая_рука_средний_тон_кожи:','tr':':elinin_tersini_kaldırma_orta_cilt_tonu:','ar':':ظهر_يد_مرفوعة_بشرة_بلون_معتدل:'},'\U0000270A':{'en':':raised_fist:','status':fully_qualified,'E':0.6,'alias':[':fist:', ':fist_raised:'],'variant':True,'de':':erhobene_faust:','es':':puño_en_alto:','fr':':poing_levé:','ja':':げんこつ:','ko':':주먹:','pt':':punho_levantado:','it':':pugno:','fa':':مشت_کردن:','id':':tangan_terkepal_ke_atas:','zh':':举起拳头:','ru':':поднятый_кулак:','tr':':yumruk_kaldırma:','ar':':قبضة_يد_مرفوعة:'},'\U0000270A\U0001F3FF':{'en':':raised_fist_dark_skin_tone:','status':fully_qualified,'E':1,'de':':erhobene_faust_dunkle_hautfarbe:','es':':puño_en_alto_tono_de_piel_oscuro:','fr':':poing_levé_peau_foncée:','ja':':げんこつ_濃い肌色:','ko':':주먹_검은색_피부:','pt':':punho_levantado_pele_escura:','it':':pugno_carnagione_scura:','fa':':مشت_کردن_پوست_آبنوسی:','id':':tangan_terkepal_ke_atas_warna_kulit_gelap:','zh':':举起拳头_较深肤色:','ru':':поднятый_кулак_очень_темный_тон_кожи:','tr':':yumruk_kaldırma_koyu_cilt_tonu:','ar':':قبضة_يد_مرفوعة_بشرة_بلون_غامق:'},'\U0000270A\U0001F3FB':{'en':':raised_fist_light_skin_tone:','status':fully_qualified,'E':1,'de':':erhobene_faust_helle_hautfarbe:','es':':puño_en_alto_tono_de_piel_claro:','fr':':poing_levé_peau_claire:','ja':':げんこつ_薄い肌色:','ko':':주먹_하얀_피부:','pt':':punho_levantado_pele_clara:','it':':pugno_carnagione_chiara:','fa':':مشت_کردن_پوست_سفید:','id':':tangan_terkepal_ke_atas_warna_kulit_cerah:','zh':':举起拳头_较浅肤色:','ru':':поднятый_кулак_очень_светлый_тон_кожи:','tr':':yumruk_kaldırma_açık_cilt_tonu:','ar':':قبضة_يد_مرفوعة_بشرة_بلون_فاتح:'},'\U0000270A\U0001F3FE':{'en':':raised_fist_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':erhobene_faust_mitteldunkle_hautfarbe:','es':':puño_en_alto_tono_de_piel_oscuro_medio:','fr':':poing_levé_peau_mate:','ja':':げんこつ_やや濃い肌色:','ko':':주먹_진한_갈색_피부:','pt':':punho_levantado_pele_morena_escura:','it':':pugno_carnagione_abbastanza_scura:','fa':':مشت_کردن_پوست_گندمی:','id':':tangan_terkepal_ke_atas_warna_kulit_gelap-sedang:','zh':':举起拳头_中等深肤色:','ru':':поднятый_кулак_темный_тон_кожи:','tr':':yumruk_kaldırma_orta_koyu_cilt_tonu:','ar':':قبضة_يد_مرفوعة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0000270A\U0001F3FC':{'en':':raised_fist_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':erhobene_faust_mittelhelle_hautfarbe:','es':':puño_en_alto_tono_de_piel_claro_medio:','fr':':poing_levé_peau_moyennement_claire:','ja':':げんこつ_やや薄い肌色:','ko':':주먹_연한_갈색_피부:','pt':':punho_levantado_pele_morena_clara:','it':':pugno_carnagione_abbastanza_chiara:','fa':':مشت_کردن_پوست_روشن:','id':':tangan_terkepal_ke_atas_warna_kulit_cerah-sedang:','zh':':举起拳头_中等浅肤色:','ru':':поднятый_кулак_светлый_тон_кожи:','tr':':yumruk_kaldırma_orta_açık_cilt_tonu:','ar':':قبضة_يد_مرفوعة_بشرة_بلون_فاتح_ومعتدل:'},'\U0000270A\U0001F3FD':{'en':':raised_fist_medium_skin_tone:','status':fully_qualified,'E':1,'de':':erhobene_faust_mittlere_hautfarbe:','es':':puño_en_alto_tono_de_piel_medio:','fr':':poing_levé_peau_légèrement_mate:','ja':':げんこつ_中間の肌色:','ko':':주먹_갈색_피부:','pt':':punho_levantado_pele_morena:','it':':pugno_carnagione_olivastra:','fa':':مشت_کردن_پوست_طلایی:','id':':tangan_terkepal_ke_atas_warna_kulit_sedang:','zh':':举起拳头_中等肤色:','ru':':поднятый_кулак_средний_тон_кожи:','tr':':yumruk_kaldırma_orta_cilt_tonu:','ar':':قبضة_يد_مرفوعة_بشرة_بلون_معتدل:'},'\U0000270B':{'en':':raised_hand:','status':fully_qualified,'E':0.6,'alias':[':hand:'],'variant':True,'de':':erhobene_hand:','es':':mano_levantada:','fr':':main_levée:','ja':':挙手:','ko':':손바닥:','pt':':mão_levantada:','it':':mano_alzata:','fa':':دست_بالابرده:','id':':telapak_tangan_terbuka_dengan_jari_rapat:','zh':':举起手:','ru':':поднятая_ладонь:','tr':':el_kaldırma:','ar':':يد_مرفوعة:'},'\U0000270B\U0001F3FF':{'en':':raised_hand_dark_skin_tone:','status':fully_qualified,'E':1,'de':':erhobene_hand_dunkle_hautfarbe:','es':':mano_levantada_tono_de_piel_oscuro:','fr':':main_levée_peau_foncée:','ja':':挙手_濃い肌色:','ko':':손바닥_검은색_피부:','pt':':mão_levantada_pele_escura:','it':':mano_alzata_carnagione_scura:','fa':':دست_بالابرده_پوست_آبنوسی:','id':':telapak_tangan_terbuka_dengan_jari_rapat_warna_kulit_gelap:','zh':':举起手_较深肤色:','ru':':поднятая_ладонь_очень_темный_тон_кожи:','tr':':el_kaldırma_koyu_cilt_tonu:','ar':':يد_مرفوعة_بشرة_بلون_غامق:'},'\U0000270B\U0001F3FB':{'en':':raised_hand_light_skin_tone:','status':fully_qualified,'E':1,'de':':erhobene_hand_helle_hautfarbe:','es':':mano_levantada_tono_de_piel_claro:','fr':':main_levée_peau_claire:','ja':':挙手_薄い肌色:','ko':':손바닥_하얀_피부:','pt':':mão_levantada_pele_clara:','it':':mano_alzata_carnagione_chiara:','fa':':دست_بالابرده_پوست_سفید:','id':':telapak_tangan_terbuka_dengan_jari_rapat_warna_kulit_cerah:','zh':':举起手_较浅肤色:','ru':':поднятая_ладонь_очень_светлый_тон_кожи:','tr':':el_kaldırma_açık_cilt_tonu:','ar':':يد_مرفوعة_بشرة_بلون_فاتح:'},'\U0000270B\U0001F3FE':{'en':':raised_hand_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':erhobene_hand_mitteldunkle_hautfarbe:','es':':mano_levantada_tono_de_piel_oscuro_medio:','fr':':main_levée_peau_mate:','ja':':挙手_やや濃い肌色:','ko':':손바닥_진한_갈색_피부:','pt':':mão_levantada_pele_morena_escura:','it':':mano_alzata_carnagione_abbastanza_scura:','fa':':دست_بالابرده_پوست_گندمی:','id':':telapak_tangan_terbuka_dengan_jari_rapat_warna_kulit_gelap-sedang:','zh':':举起手_中等深肤色:','ru':':поднятая_ладонь_темный_тон_кожи:','tr':':el_kaldırma_orta_koyu_cilt_tonu:','ar':':يد_مرفوعة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0000270B\U0001F3FC':{'en':':raised_hand_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':erhobene_hand_mittelhelle_hautfarbe:','es':':mano_levantada_tono_de_piel_claro_medio:','fr':':main_levée_peau_moyennement_claire:','ja':':挙手_やや薄い肌色:','ko':':손바닥_연한_갈색_피부:','pt':':mão_levantada_pele_morena_clara:','it':':mano_alzata_carnagione_abbastanza_chiara:','fa':':دست_بالابرده_پوست_روشن:','id':':telapak_tangan_terbuka_dengan_jari_rapat_warna_kulit_cerah-sedang:','zh':':举起手_中等浅肤色:','ru':':поднятая_ладонь_светлый_тон_кожи:','tr':':el_kaldırma_orta_açık_cilt_tonu:','ar':':يد_مرفوعة_بشرة_بلون_فاتح_ومعتدل:'},'\U0000270B\U0001F3FD':{'en':':raised_hand_medium_skin_tone:','status':fully_qualified,'E':1,'de':':erhobene_hand_mittlere_hautfarbe:','es':':mano_levantada_tono_de_piel_medio:','fr':':main_levée_peau_légèrement_mate:','ja':':挙手_中間の肌色:','ko':':손바닥_갈색_피부:','pt':':mão_levantada_pele_morena:','it':':mano_alzata_carnagione_olivastra:','fa':':دست_بالابرده_پوست_طلایی:','id':':telapak_tangan_terbuka_dengan_jari_rapat_warna_kulit_sedang:','zh':':举起手_中等肤色:','ru':':поднятая_ладонь_средний_тон_кожи:','tr':':el_kaldırma_orta_cilt_tonu:','ar':':يد_مرفوعة_بشرة_بلون_معتدل:'},'\U0001F64C':{'en':':raising_hands:','status':fully_qualified,'E':0.6,'alias':[':raised_hands:'],'de':':zwei_erhobene_handflächen:','es':':manos_levantadas_celebrando:','fr':':mains_levées:','ja':':バンザイ:','ko':':만세:','pt':':mãos_para_cima:','it':':mani_alzate:','fa':':دست_های_بالا_آورده:','id':':mengangkat_tangan:','zh':':举双手:','ru':':ура:','tr':':eller_havaya:','ar':':تشجيع:'},'\U0001F64C\U0001F3FF':{'en':':raising_hands_dark_skin_tone:','status':fully_qualified,'E':1,'de':':zwei_erhobene_handflächen_dunkle_hautfarbe:','es':':manos_levantadas_celebrando_tono_de_piel_oscuro:','fr':':mains_levées_peau_foncée:','ja':':バンザイ_濃い肌色:','ko':':만세_검은색_피부:','pt':':mãos_para_cima_pele_escura:','it':':mani_alzate_carnagione_scura:','fa':':دست_های_بالا_آورده_پوست_آبنوسی:','id':':mengangkat_tangan_warna_kulit_gelap:','zh':':举双手_较深肤色:','ru':':ура_очень_темный_тон_кожи:','tr':':eller_havaya_koyu_cilt_tonu:','ar':':تشجيع_بشرة_بلون_غامق:'},'\U0001F64C\U0001F3FB':{'en':':raising_hands_light_skin_tone:','status':fully_qualified,'E':1,'de':':zwei_erhobene_handflächen_helle_hautfarbe:','es':':manos_levantadas_celebrando_tono_de_piel_claro:','fr':':mains_levées_peau_claire:','ja':':バンザイ_薄い肌色:','ko':':만세_하얀_피부:','pt':':mãos_para_cima_pele_clara:','it':':mani_alzate_carnagione_chiara:','fa':':دست_های_بالا_آورده_پوست_سفید:','id':':mengangkat_tangan_warna_kulit_cerah:','zh':':举双手_较浅肤色:','ru':':ура_очень_светлый_тон_кожи:','tr':':eller_havaya_açık_cilt_tonu:','ar':':تشجيع_بشرة_بلون_فاتح:'},'\U0001F64C\U0001F3FE':{'en':':raising_hands_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':zwei_erhobene_handflächen_mitteldunkle_hautfarbe:','es':':manos_levantadas_celebrando_tono_de_piel_oscuro_medio:','fr':':mains_levées_peau_mate:','ja':':バンザイ_やや濃い肌色:','ko':':만세_진한_갈색_피부:','pt':':mãos_para_cima_pele_morena_escura:','it':':mani_alzate_carnagione_abbastanza_scura:','fa':':دست_های_بالا_آورده_پوست_گندمی:','id':':mengangkat_tangan_warna_kulit_gelap-sedang:','zh':':举双手_中等深肤色:','ru':':ура_темный_тон_кожи:','tr':':eller_havaya_orta_koyu_cilt_tonu:','ar':':تشجيع_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F64C\U0001F3FC':{'en':':raising_hands_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':zwei_erhobene_handflächen_mittelhelle_hautfarbe:','es':':manos_levantadas_celebrando_tono_de_piel_claro_medio:','fr':':mains_levées_peau_moyennement_claire:','ja':':バンザイ_やや薄い肌色:','ko':':만세_연한_갈색_피부:','pt':':mãos_para_cima_pele_morena_clara:','it':':mani_alzate_carnagione_abbastanza_chiara:','fa':':دست_های_بالا_آورده_پوست_روشن:','id':':mengangkat_tangan_warna_kulit_cerah-sedang:','zh':':举双手_中等浅肤色:','ru':':ура_светлый_тон_кожи:','tr':':eller_havaya_orta_açık_cilt_tonu:','ar':':تشجيع_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F64C\U0001F3FD':{'en':':raising_hands_medium_skin_tone:','status':fully_qualified,'E':1,'de':':zwei_erhobene_handflächen_mittlere_hautfarbe:','es':':manos_levantadas_celebrando_tono_de_piel_medio:','fr':':mains_levées_peau_légèrement_mate:','ja':':バンザイ_中間の肌色:','ko':':만세_갈색_피부:','pt':':mãos_para_cima_pele_morena:','it':':mani_alzate_carnagione_olivastra:','fa':':دست_های_بالا_آورده_پوست_طلایی:','id':':mengangkat_tangan_warna_kulit_sedang:','zh':':举双手_中等肤色:','ru':':ура_средний_тон_кожи:','tr':':eller_havaya_orta_cilt_tonu:','ar':':تشجيع_بشرة_بلون_معتدل:'},'\U0001F40F':{'en':':ram:','status':fully_qualified,'E':1,'de':':widder:','es':':carnero:','fr':':bélier:','ja':':牡羊:','ko':':숫양:','pt':':carneiro:','it':':montone:','fa':':قوچ:','id':':domba_jantan:','zh':':公羊:','ru':':баран:','tr':':koç:','ar':':كبش:'},'\U0001F400':{'en':':rat:','status':fully_qualified,'E':1,'de':':ratte:','es':':rata:','fr':':rat:','ja':':ネズミ:','ko':':쥐:','pt':':rato:','it':':ratto:','fa':':موش_صحرایی:','id':':tikus:','zh':':耗子:','ru':':крыса:','tr':':sıçan:','ar':':جرذ:'},'\U0001FA92':{'en':':razor:','status':fully_qualified,'E':12,'de':':rasierer:','es':':cuchilla_de_afeitar:','fr':':rasoir:','ja':':剃刀:','ko':':면도칼:','pt':':lâmina:','it':':rasoio:','fa':':ریش_تراش:','id':':pisau_cukur:','zh':':剃须刀:','ru':':бритва:','tr':':ustura:','ar':':موس_حلاقة:'},'\U0001F9FE':{'en':':receipt:','status':fully_qualified,'E':11,'de':':beleg:','es':':recibo:','fr':':reçu:','ja':':レシート:','ko':':영수증:','pt':':recibo:','it':':scontrino:','fa':':رسید:','id':':kuitansi:','zh':':收据:','ru':':чек:','tr':':makbuz:','ar':':إيصال:'},'\U000023FA\U0000FE0F':{'en':':record_button:','status':fully_qualified,'E':0.7,'alias':[':black_circle_for_record:'],'variant':True,'de':':aufnehmen:','es':':grabar:','fr':':bouton_enregistrer:','ja':':録音録画ボタン:','ko':':녹음_버튼:','pt':':botão_gravar:','it':':pulsante_di_registrazione:','fa':':دکمه_ضبط:','id':':tombol_rekam:','zh':':录制按钮:','ru':':запись:','tr':':kayıt_düğmesi:','ar':':زر_تسجيل:'},'\U000023FA':{'en':':record_button:','status':unqualified,'E':0.7,'alias':[':black_circle_for_record:'],'variant':True,'de':':aufnehmen:','es':':grabar:','fr':':bouton_enregistrer:','ja':':録音録画ボタン:','ko':':녹음_버튼:','pt':':botão_gravar:','it':':pulsante_di_registrazione:','fa':':دکمهٔ_ضبط:','id':':tombol_rekam:','zh':':录制按钮:','ru':':запись:','tr':':kayıt_düğmesi:','ar':':زر_تسجيل:'},'\U0000267B\U0000FE0F':{'en':':recycling_symbol:','status':fully_qualified,'E':0.6,'alias':[':recycle:'],'variant':True,'de':':recycling-symbol:','es':':símbolo_de_reciclaje:','fr':':symbole_recyclage:','ja':':リサイクルマーク:','ko':':재활용_표시:','pt':':símbolo_de_reciclagem:','it':':simbolo_del_riciclaggio:','fa':':نماد_بازیافت:','id':':simbol_daur_ulang:','zh':':回收标志:','ru':':утилизация:','tr':':geri_dönüşüm_simgesi:','ar':':رمز_تدوير:'},'\U0000267B':{'en':':recycling_symbol:','status':unqualified,'E':0.6,'alias':[':recycle:'],'variant':True,'de':':recycling-symbol:','es':':símbolo_de_reciclaje:','fr':':symbole_recyclage:','ja':':リサイクルマーク:','ko':':재활용_표시:','pt':':símbolo_de_reciclagem:','it':':simbolo_del_riciclaggio:','fa':':نماد_بازیافت:','id':':simbol_daur_ulang:','zh':':回收标志:','ru':':утилизация:','tr':':geri_dönüşüm_simgesi:','ar':':رمز_تدوير:'},'\U0001F34E':{'en':':red_apple:','status':fully_qualified,'E':0.6,'alias':[':apple:'],'de':':roter_apfel:','es':':manzana_roja:','fr':':pomme_rouge:','ja':':赤リンゴ:','ko':':빨간_사과:','pt':':maçã_vermelha:','it':':mela_rossa:','fa':':سیب_سرخ:','id':':apel_merah:','zh':':红苹果:','ru':':красное_яблоко:','tr':':kırmızı_elma:','ar':':تفاح_أحمر:'},'\U0001F534':{'en':':red_circle:','status':fully_qualified,'E':0.6,'de':':roter_punkt:','es':':círculo_rojo:','fr':':disque_rouge:','ja':':赤い丸:','ko':':빨간_원:','pt':':círculo_vermelho:','it':':cerchio_rosso:','fa':':دایره_قرمز:','id':':lingkaran_merah:','zh':':红色圆:','ru':':красный_круг:','tr':':kırmızı_daire:','ar':':دائرة_حمراء:'},'\U0001F9E7':{'en':':red_envelope:','status':fully_qualified,'E':11,'de':':roter_umschlag:','es':':sobre_rojo:','fr':':enveloppe_rouge:','ja':':赤い封筒:','ko':':세뱃돈:','pt':':envelope_vermelho:','it':':busta_hongbao:','fa':':پاکت_سرخ:','id':':amplop_merah:','zh':':红包:','ru':':красный_конверт:','tr':':kırmızı_zarf:','ar':':ظرف_أحمر_بكتابة_صينية:'},'\U00002757':{'en':':red_exclamation_mark:','status':fully_qualified,'E':0.6,'alias':[':heavy_exclamation_mark:', ':exclamation:'],'variant':True,'de':':rotes_ausrufezeichen:','es':':exclamación_roja:','fr':':point_d’exclamation_rouge:','ja':':赤い感嘆符:','ko':':빨간색_느낌표:','pt':':ponto_de_exclamação_vermelho:','it':':punto_esclamativo_rosso:','fa':':علامت_تعجب_قرمز:','id':':tanda_seru_merah:','zh':':红色感叹号:','ru':':красный_восклицательный_знак:','tr':':kırmızı_ünlem_işareti:','ar':':علامة_تعجب:'},'\U0001F9B0':{'en':':red_hair:','status':component,'E':11,'de':':rotes_haar:','es':':pelo_pelirrojo:','fr':':cheveux_roux:','ja':':赤毛:','ko':':빨간_머리:','pt':':cabelo_vermelho:','it':':capelli_rossi:','fa':':موقرمز:','id':':rambut_merah:','zh':':红发:','ru':':рыжие_волосы:','tr':':kızıl_saç:','ar':':شعر_أحمر:'},'\U00002764\U0000FE0F':{'en':':red_heart:','status':fully_qualified,'E':0.6,'alias':[':heart:'],'variant':True,'de':':rotes_herz:','es':':corazón_rojo:','fr':':cœur_rouge:','ja':':赤いハート:','ko':':빨간색_하트:','pt':':coração_vermelho:','it':':cuore_rosso:','fa':':قلب_قرمز:','id':':hati_merah:','zh':':红心:','ru':':алое_сердце:','tr':':kırmızı_kalp:','ar':':قلب_أحمر:'},'\U00002764':{'en':':red_heart:','status':unqualified,'E':0.6,'alias':[':heart:'],'variant':True,'de':':rotes_herz:','es':':corazón_rojo:','fr':':cœur_rouge:','ja':':赤いハート:','ko':':빨간색_하트:','pt':':coração_vermelho:','it':':cuore_rosso:','fa':':قلب_قرمز:','id':':hati_merah:','zh':':红心:','ru':':алое_сердце:','tr':':kırmızı_kalp:','ar':':قلب_أحمر:'},'\U0001F3EE':{'en':':red_paper_lantern:','status':fully_qualified,'E':0.6,'alias':[':izakaya_lantern:', ':lantern:'],'de':':rote_papierlaterne:','es':':lámpara_japonesa:','fr':':lampion_rouge:','ja':':赤ちょうちん:','ko':':일본식_등:','pt':':lanterna_vermelha_de_papel:','it':':lanterna_rossa:','fa':':فانوس_کاغذی_قرمز:','id':':lampion_merah:','zh':':红灯笼:','ru':':красный_бумажный_фонарик:','tr':':kırmızı_kağıt_fener:','ar':':فانوس_بورق_أحمر:'},'\U00002753':{'en':':red_question_mark:','status':fully_qualified,'E':0.6,'alias':[':question:'],'variant':True,'de':':rotes_fragezeichen:','es':':interrogación_roja:','fr':':point_d’interrogation_rouge:','ja':':赤い疑問符:','ko':':빨간색_물음표:','pt':':ponto_de_interrogação_vermelho:','it':':punto_interrogativo_rosso:','fa':':علامت_سؤال_قرمز:','id':':tanda_tanya_merah:','zh':':红色问号:','ru':':красный_вопросительный_знак:','tr':':kırmızı_soru_işareti:','ar':':علامة_استفهام_حمراء:'},'\U0001F7E5':{'en':':red_square:','status':fully_qualified,'E':12,'de':':rotes_quadrat:','es':':cuadrado_rojo:','fr':':carré_rouge:','ja':':赤い四角:','ko':':빨간색_사각형:','pt':':quadrado_vermelho:','it':':quadrato_rosso:','fa':':مربع_سرخ:','id':':persegi_merah:','zh':':红色方块:','ru':':красный_квадрат:','tr':':kırmızı_kare:','ar':':مربع_أحمر:'},'\U0001F53B':{'en':':red_triangle_pointed_down:','status':fully_qualified,'E':0.6,'alias':[':small_red_triangle_down:'],'de':':rotes_dreieck_mit_der_spitze_nach_unten:','es':':triángulo_rojo_hacia_abajo:','fr':':triangle_rouge_pointant_vers_le_bas:','ja':':赤い下三角:','ko':':빨간색_아래쪽_삼각형:','pt':':triângulo_vermelho_para_baixo:','it':':triangolo_rosso_con_punta_verso_il_basso:','fa':':مثلث_با_رأس_پایین:','id':':segitiga_merah_mengarah_ke_bawah:','zh':':红色倒三角:','ru':':треугольник_вершиной_вниз:','tr':':aşağı_bakan_kırmızı_üçgen:','ar':':مثلث_لأسفل_أحمر:'},'\U0001F53A':{'en':':red_triangle_pointed_up:','status':fully_qualified,'E':0.6,'alias':[':small_red_triangle:'],'de':':rotes_dreieck_mit_der_spitze_nach_oben:','es':':triángulo_rojo_hacia_arriba:','fr':':triangle_rouge_pointant_vers_le_haut:','ja':':赤い上三角:','ko':':빨간색_위쪽_삼각형:','pt':':triângulo_vermelho_para_cima:','it':':triangolo_rosso_con_punta_verso_l’alto:','fa':':مثلث_با_رأس_بالا:','id':':segitiga_merah_mengarah_ke_atas:','zh':':红色正三角:','ru':':треугольник_вершиной_вверх:','tr':':yukarı_bakan_kırmızı_üçgen:','ar':':مثلث_لأعلى_أحمر:'},'\U000000AE\U0000FE0F':{'en':':registered:','status':fully_qualified,'E':0.6,'variant':True,'de':':registered-trademark:','es':':marca_registrada:','fr':':marque_déposée:','ja':':登録商標マーク:','ko':':등록상표:','pt':':símbolo_de_registrado:','it':':marchio_registrato:','fa':':نشان_تجاری_ثبت_شده:','id':':terdaftar:','zh':':注册:','ru':':зарегистрированныи_товарныи_знак:','tr':':tescil_işareti:','ar':':مسجَل:'},'\U000000AE':{'en':':registered:','status':unqualified,'E':0.6,'variant':True,'de':':registered-trademark:','es':':marca_registrada:','fr':':marque_déposée:','ja':':登録商標マーク:','ko':':등록상표:','pt':':símbolo_de_registrado:','it':':marchio_registrato:','fa':':نشان_تجاری_ثبت_شده:','id':':terdaftar:','zh':':注册:','ru':':зарегистрированный_товарный_знак:','tr':':tescil_işareti:','ar':':مسجَل:'},'\U0001F60C':{'en':':relieved_face:','status':fully_qualified,'E':0.6,'alias':[':relieved:'],'de':':erleichtertes_gesicht:','es':':cara_de_alivio:','fr':':visage_soulagé:','ja':':ほっとした顔:','ko':':안심한_얼굴:','pt':':rosto_aliviado:','it':':faccina_sollevata:','fa':':آسوده_خاطر:','id':':wajah_lega:','zh':':松了口气:','ru':':испытывает_облегчение:','tr':':rahatlamış_yüz:','ar':':وجه_مرتاح:'},'\U0001F397\U0000FE0F':{'en':':reminder_ribbon:','status':fully_qualified,'E':0.7,'variant':True,'de':':gedenkschleife:','es':':lazo_conmemorativo:','fr':':ruban_de_mémoire:','ja':':リマインダーリボン:','ko':':추모_리본:','pt':':fita_de_lembrete:','it':':nastro:','fa':':روبان_یادآوری:','id':':pita_pengingat:','zh':':提示丝带:','ru':':памятная_лента:','tr':':anma_kurdelesi:','ar':':شريطة_تذكير:'},'\U0001F397':{'en':':reminder_ribbon:','status':unqualified,'E':0.7,'variant':True,'de':':gedenkschleife:','es':':lazo_conmemorativo:','fr':':ruban_de_mémoire:','ja':':リマインダーリボン:','ko':':추모_리본:','pt':':fita_de_lembrete:','it':':nastro:','fa':':روبان_یادآوری:','id':':pita_pengingat:','zh':':提示丝带:','ru':':памятная_лента:','tr':':anma_kurdelesi:','ar':':شريطة_تذكير:'},'\U0001F501':{'en':':repeat_button:','status':fully_qualified,'E':1,'alias':[':repeat:'],'de':':wiederholen:','es':':repetir:','fr':':bouton_répétition:','ja':':リピートボタン:','ko':':전곡_반복_재생:','pt':':botão_de_repetir:','it':':pulsante_di_ripetizione_della_riproduzione:','fa':':دکمهٔ_تکرار:','id':':tombol_ulangi:','zh':':重复按钮:','ru':':повтор:','tr':':tekrarlama_düğmesi:','ar':':زر_تكرار:'},'\U0001F502':{'en':':repeat_single_button:','status':fully_qualified,'E':1,'alias':[':repeat_one:'],'de':':titel_wiederholen:','es':':repetir_una_vez:','fr':':bouton_répétition_de_la_piste:','ja':':1曲リピートボタン:','ko':':한_곡_반복_재생:','pt':':botão_de_repetir_uma_única_faixa:','it':':pulsante_di_ripetizione_della_riproduzione_di_una_traccia:','fa':':دکمهٔ_یک_بار_تکرار:','id':':tombol_ulangi_satu_lagu:','zh':':重复一次按钮:','ru':':повтор_одной_песни:','tr':':tek_parça_tekrarlama_düğmesi:','ar':':زر_تكرار_مرة_واحدة:'},'\U000026D1\U0000FE0F':{'en':':rescue_worker’s_helmet:','status':fully_qualified,'E':0.7,'alias':[':helmet_with_white_cross:', ':rescue_worker_helmet:', ':rescue_workers_helmet:'],'variant':True,'de':':rettungshelm:','es':':casco_con_una_cruz_blanca:','fr':':casque_de_secouriste:','ja':':白十字ヘルメット:','ko':':흰_십자가가_있는_헬멧:','pt':':capacacete_de_socorrista:','it':':elmetto_con_croce_bianca:','fa':':کلاه_صلیب_سرخ:','id':':helm_dengan_palang_putih:','zh':':白十字头盔:','ru':':каска_с_белым_крестом:','tr':':yardım_ekibi_kaskı:','ar':':خوذة_رجل_إنقاذ:'},'\U000026D1':{'en':':rescue_worker’s_helmet:','status':unqualified,'E':0.7,'alias':[':helmet_with_white_cross:', ':rescue_worker_helmet:', ':rescue_workers_helmet:'],'variant':True,'de':':rettungshelm:','es':':casco_con_una_cruz_blanca:','fr':':casque_de_secouriste:','ja':':白十字ヘルメット:','ko':':흰_십자가가_있는_헬멧:','pt':':capacacete_de_socorrista:','it':':elmetto_con_croce_bianca:','fa':':کلاه_صلیب_سرخ:','id':':helm_dengan_palang_putih:','zh':':白十字头盔:','ru':':каска_с_белым_крестом:','tr':':yardım_ekibi_kaskı:','ar':':خوذة_رجل_إنقاذ:'},'\U0001F6BB':{'en':':restroom:','status':fully_qualified,'E':0.6,'de':':toiletten:','es':':señal_de_aseos:','fr':':panneau_toilettes:','ja':':トイレ:','ko':':화장실:','pt':':banheiro:','it':':simbolo_dei_servizi_igienici:','fa':':توالت_عمومی:','id':':kamar_kecil:','zh':':卫生间:','ru':':уборная:','tr':':tuvalet:','ar':':دورة_مياه:'},'\U000025C0\U0000FE0F':{'en':':reverse_button:','status':fully_qualified,'E':0.6,'alias':[':arrow_backward:'],'variant':True,'de':':pfeil_zurück:','es':':retroceso:','fr':':bouton_retour:','ja':':逆再生ボタン:','ko':':반대로_버튼:','pt':':botão_de_voltar:','it':':pulsante_di_riavvolgimento:','fa':':دکمه_برگرداندن:','id':':tombol_mundur:','zh':':倒退按钮:','ru':':реверс:','tr':':geri_düğmesi:','ar':':زر_ترجيع:'},'\U000025C0':{'en':':reverse_button:','status':unqualified,'E':0.6,'alias':[':arrow_backward:'],'variant':True,'de':':pfeil_zurück:','es':':retroceso:','fr':':bouton_retour:','ja':':逆再生ボタン:','ko':':반대로_버튼:','pt':':botão_de_voltar:','it':':pulsante_di_riavvolgimento:','fa':':دکمهٔ_برگرداندن:','id':':tombol_mundur:','zh':':倒退按钮:','ru':':реверс:','tr':':geri_düğmesi:','ar':':زر_ترجيع:'},'\U0001F49E':{'en':':revolving_hearts:','status':fully_qualified,'E':0.6,'de':':kreisende_herzen:','es':':corazones_giratorios:','fr':':cœurs_qui_tournent:','ja':':回転するハート:','ko':':회전하는_하트:','pt':':corações_girando:','it':':cuori_che_girano:','fa':':قلب_چرخان:','id':':hati_berputar:','zh':':舞动的心:','ru':':вращающиеся_сердца:','tr':':dönen_kalpler:','ar':':قلوب_تدور_حول_بعضها:'},'\U0001F98F':{'en':':rhinoceros:','status':fully_qualified,'E':3,'de':':nashorn:','es':':rinoceronte:','fr':':rhinocéros:','ja':':サイ:','ko':':코뿔소:','pt':':rinoceronte:','it':':rinoceronte:','fa':':کرگدن:','id':':badak:','zh':':犀牛:','ru':':носорог:','tr':':gergedan:','ar':':وحيد_القرن:'},'\U0001F380':{'en':':ribbon:','status':fully_qualified,'E':0.6,'de':':pinke_schleife:','es':':lazo:','fr':':ruban:','ja':':リボン:','ko':':리본:','pt':':laço_de_fita:','it':':fiocco:','fa':':روبان:','id':':pita:','zh':':蝴蝶结:','ru':':бантик:','tr':':kurdele:','ar':':شريط:'},'\U0001F359':{'en':':rice_ball:','status':fully_qualified,'E':0.6,'de':':reisbällchen:','es':':bola_de_arroz:','fr':':boulette_de_riz:','ja':':おにぎり:','ko':':삼각_김밥:','pt':':arroz_japonês:','it':':onigiri:','fa':':کوفته_برنجی:','id':':nasi_kepal:','zh':':饭团:','ru':':онигири:','tr':':pirinç_topu:','ar':':كرة_أرز:'},'\U0001F358':{'en':':rice_cracker:','status':fully_qualified,'E':0.6,'de':':reiscracker:','es':':galleta_de_arroz:','fr':':galette_de_riz:','ja':':せんべい:','ko':':주먹밥:','pt':':biscoito_de_arroz:','it':':galletta_di_riso:','fa':':تردک_برنجی:','id':':kerupuk_beras:','zh':':米饼:','ru':':рисовый_крекер:','tr':':pirinç_gevreği:','ar':':مقرمشات_أرز:'},'\U0001F91C':{'en':':right-facing_fist:','status':fully_qualified,'E':3,'alias':[':fist_right:', ':right_facing_fist:'],'de':':faust_nach_rechts:','es':':puño_hacia_la_derecha:','fr':':poing_à_droite:','ja':':右向きのこぶし:','ko':':오른쪽을_향하는_주먹:','pt':':punho_direito:','it':':pugno_a_destra:','fa':':مشت_به_راست:','id':':tinju_kanan:','zh':':朝右的拳头:','ru':':кулак_вправо:','tr':':sağa_bakan_yumruk:','ar':':قبضة_لليمين:'},'\U0001F91C\U0001F3FF':{'en':':right-facing_fist_dark_skin_tone:','status':fully_qualified,'E':3,'de':':faust_nach_rechts_dunkle_hautfarbe:','es':':puño_hacia_la_derecha_tono_de_piel_oscuro:','fr':':poing_à_droite_peau_foncée:','ja':':右向きのこぶし_濃い肌色:','ko':':오른쪽을_향하는_주먹_검은색_피부:','pt':':punho_direito_pele_escura:','it':':pugno_a_destra_carnagione_scura:','fa':':مشت_به_راست_پوست_آبنوسی:','id':':tinju_kanan_warna_kulit_gelap:','zh':':朝右的拳头_较深肤色:','ru':':кулак_вправо_очень_темный_тон_кожи:','tr':':sağa_bakan_yumruk_koyu_cilt_tonu:','ar':':قبضة_لليمين_بشرة_بلون_غامق:'},'\U0001F91C\U0001F3FB':{'en':':right-facing_fist_light_skin_tone:','status':fully_qualified,'E':3,'de':':faust_nach_rechts_helle_hautfarbe:','es':':puño_hacia_la_derecha_tono_de_piel_claro:','fr':':poing_à_droite_peau_claire:','ja':':右向きのこぶし_薄い肌色:','ko':':오른쪽을_향하는_주먹_하얀_피부:','pt':':punho_direito_pele_clara:','it':':pugno_a_destra_carnagione_chiara:','fa':':مشت_به_راست_پوست_سفید:','id':':tinju_kanan_warna_kulit_cerah:','zh':':朝右的拳头_较浅肤色:','ru':':кулак_вправо_очень_светлый_тон_кожи:','tr':':sağa_bakan_yumruk_açık_cilt_tonu:','ar':':قبضة_لليمين_بشرة_بلون_فاتح:'},'\U0001F91C\U0001F3FE':{'en':':right-facing_fist_medium-dark_skin_tone:','status':fully_qualified,'E':3,'de':':faust_nach_rechts_mitteldunkle_hautfarbe:','es':':puño_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':poing_à_droite_peau_mate:','ja':':右向きのこぶし_やや濃い肌色:','ko':':오른쪽을_향하는_주먹_진한_갈색_피부:','pt':':punho_direito_pele_morena_escura:','it':':pugno_a_destra_carnagione_abbastanza_scura:','fa':':مشت_به_راست_پوست_گندمی:','id':':tinju_kanan_warna_kulit_gelap-sedang:','zh':':朝右的拳头_中等深肤色:','ru':':кулак_вправо_темный_тон_кожи:','tr':':sağa_bakan_yumruk_orta_koyu_cilt_tonu:','ar':':قبضة_لليمين_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F91C\U0001F3FC':{'en':':right-facing_fist_medium-light_skin_tone:','status':fully_qualified,'E':3,'de':':faust_nach_rechts_mittelhelle_hautfarbe:','es':':puño_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':poing_à_droite_peau_moyennement_claire:','ja':':右向きのこぶし_やや薄い肌色:','ko':':오른쪽을_향하는_주먹_연한_갈색_피부:','pt':':punho_direito_pele_morena_clara:','it':':pugno_a_destra_carnagione_abbastanza_chiara:','fa':':مشت_به_راست_پوست_روشن:','id':':tinju_kanan_warna_kulit_cerah-sedang:','zh':':朝右的拳头_中等浅肤色:','ru':':кулак_вправо_светлый_тон_кожи:','tr':':sağa_bakan_yumruk_orta_açık_cilt_tonu:','ar':':قبضة_لليمين_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F91C\U0001F3FD':{'en':':right-facing_fist_medium_skin_tone:','status':fully_qualified,'E':3,'de':':faust_nach_rechts_mittlere_hautfarbe:','es':':puño_hacia_la_derecha_tono_de_piel_medio:','fr':':poing_à_droite_peau_légèrement_mate:','ja':':右向きのこぶし_中間の肌色:','ko':':오른쪽을_향하는_주먹_갈색_피부:','pt':':punho_direito_pele_morena:','it':':pugno_a_destra_carnagione_olivastra:','fa':':مشت_به_راست_پوست_طلایی:','id':':tinju_kanan_warna_kulit_sedang:','zh':':朝右的拳头_中等肤色:','ru':':кулак_вправо_средний_тон_кожи:','tr':':sağa_bakan_yumruk_orta_cilt_tonu:','ar':':قبضة_لليمين_بشرة_بلون_معتدل:'},'\U0001F5EF\U0000FE0F':{'en':':right_anger_bubble:','status':fully_qualified,'E':0.7,'variant':True,'de':':sprechblase_für_wütende_aussage_rechts:','es':':bocadillo_de_enfado_por_la_derecha:','fr':':bulle_colère_droite:','ja':':怒りの吹き出し:','ko':':화난_표현의_오른쪽_말풍선:','pt':':balão_de_raiva_à_direita:','it':':nuvoletta_rabbia:','fa':':حباب_خشم_راست:','id':':gelembung_marah_kanan:','zh':':愤怒话语气泡:','ru':':гневное_облачко:','tr':':sağ_öfke_balonu:','ar':':فقاعة_غضب_يمنى:'},'\U0001F5EF':{'en':':right_anger_bubble:','status':unqualified,'E':0.7,'variant':True,'de':':sprechblase_für_wütende_aussage_rechts:','es':':bocadillo_de_enfado_por_la_derecha:','fr':':bulle_colère_droite:','ja':':怒りの吹き出し:','ko':':화난_표현의_오른쪽_말풍선:','pt':':balão_de_raiva_à_direita:','it':':nuvoletta_rabbia:','fa':':حباب_خشم_راست:','id':':gelembung_marah_kanan:','zh':':愤怒话语气泡:','ru':':гневное_облачко:','tr':':sağ_öfke_balonu:','ar':':فقاعة_غضب_يمنى:'},'\U000027A1\U0000FE0F':{'en':':right_arrow:','status':fully_qualified,'E':0.6,'alias':[':arrow_right:'],'variant':True,'de':':pfeil_nach_rechts:','es':':flecha_hacia_la_derecha:','fr':':flèche_droite:','ja':':右矢印:','ko':':우향_화살표:','pt':':seta_para_a_direita:','it':':freccia_rivolta_verso_destra:','fa':':پیکان_راست:','id':':tanda_panah_kanan:','zh':':向右箭头:','ru':':стрелка_вправо:','tr':':sağ_ok:','ar':':سهم_لليمين:'},'\U000027A1':{'en':':right_arrow:','status':unqualified,'E':0.6,'alias':[':arrow_right:'],'variant':True,'de':':pfeil_nach_rechts:','es':':flecha_hacia_la_derecha:','fr':':flèche_droite:','ja':':右矢印:','ko':':우향_화살표:','pt':':seta_para_a_direita:','it':':freccia_rivolta_verso_destra:','fa':':پیکان_راست:','id':':tanda_panah_kanan:','zh':':向右箭头:','ru':':стрелка_вправо:','tr':':sağ_ok:','ar':':سهم_لليمين:'},'\U00002935\U0000FE0F':{'en':':right_arrow_curving_down:','status':fully_qualified,'E':0.6,'alias':[':arrow_heading_down:'],'variant':True,'de':':geschwungener_pfeil_nach_unten:','es':':flecha_derecha_curvándose_hacia_abajo:','fr':':flèche_courbe_bas:','ja':':下カーブ矢印:','ko':':아래쪽으로_꺾어지는_우향_화살표:','pt':':seta_para_a_direita_curvada_para_baixo:','it':':freccia_curva_in_basso:','fa':':پیکان_خمیده_پایین_راست:','id':':tanda_panah_kanan_melengkung_ke_bawah:','zh':':右下弯箭头:','ru':':стрелка_вправо_с_поворотом_вниз:','tr':':aşağı_kıvrımlı_sağ_ok:','ar':':سهم_لأسفل_من_اليسار:'},'\U00002935':{'en':':right_arrow_curving_down:','status':unqualified,'E':0.6,'alias':[':arrow_heading_down:'],'variant':True,'de':':geschwungener_pfeil_nach_unten:','es':':flecha_derecha_curvándose_hacia_abajo:','fr':':flèche_courbe_bas:','ja':':下カーブ矢印:','ko':':아래쪽으로_꺾어지는_우향_화살표:','pt':':seta_para_a_direita_curvada_para_baixo:','it':':freccia_curva_in_basso:','fa':':پیکان_خمیده_پایین_راست:','id':':tanda_panah_kanan_melengkung_ke_bawah:','zh':':右下弯箭头:','ru':':стрелка_вправо_с_поворотом_вниз:','tr':':aşağı_kıvrımlı_sağ_ok:','ar':':سهم_لأسفل_من_اليسار:'},'\U000021A9\U0000FE0F':{'en':':right_arrow_curving_left:','status':fully_qualified,'E':0.6,'alias':[':leftwards_arrow_with_hook:'],'variant':True,'de':':geschwungener_pfeil_nach_links:','es':':flecha_derecha_curvándose_a_la_izquierda:','fr':':flèche_courbe_gauche:','ja':':右カーブ矢印:','ko':':왼쪽으로_꺾어지는_우향_화살표:','pt':':seta_curva_da_direita_para_a_esquerda:','it':':freccia_curva_a_sinistra:','fa':':پیکان_خمیده_راست:','id':':tanda_panah_kanan_melengkung_ke_kiri:','zh':':右转弯箭头:','ru':':стрелка_вправо_с_поворотом_влево:','tr':':sola_kıvrımlı_sağ_ok:','ar':':سهم_أيمن_بانحناء_أيسر:'},'\U000021A9':{'en':':right_arrow_curving_left:','status':unqualified,'E':0.6,'alias':[':leftwards_arrow_with_hook:'],'variant':True,'de':':geschwungener_pfeil_nach_links:','es':':flecha_derecha_curvándose_a_la_izquierda:','fr':':flèche_courbe_gauche:','ja':':右カーブ矢印:','ko':':왼쪽으로_꺾어지는_우향_화살표:','pt':':seta_curva_da_direita_para_a_esquerda:','it':':freccia_curva_a_sinistra:','fa':':پیکان_خمیده_راست:','id':':tanda_panah_kanan_melengkung_ke_kiri:','zh':':右转弯箭头:','ru':':стрелка_вправо_с_поворотом_влево:','tr':':sola_kıvrımlı_sağ_ok:','ar':':سهم_أيمن_بانحناء_أيسر:'},'\U00002934\U0000FE0F':{'en':':right_arrow_curving_up:','status':fully_qualified,'E':0.6,'alias':[':arrow_heading_up:'],'variant':True,'de':':geschwungener_pfeil_nach_oben:','es':':flecha_derecha_curvándose_hacia_arriba:','fr':':flèche_courbe_haut:','ja':':上カーブ矢印:','ko':':위쪽으로_꺾어지는_우향_화살표:','pt':':seta_para_a_direita_curvada_para_cima:','it':':freccia_curva_in_alto:','fa':':پیکان_خمیده_بالا_راست:','id':':tanda_panah_kanan_melengkung_ke_atas:','zh':':右上弯箭头:','ru':':стрелка_вправо_с_поворотом_вверх:','tr':':yukarı_kıvrımlı_sağ_ok:','ar':':سهم_لأعلى_من_اليسار:'},'\U00002934':{'en':':right_arrow_curving_up:','status':unqualified,'E':0.6,'alias':[':arrow_heading_up:'],'variant':True,'de':':geschwungener_pfeil_nach_oben:','es':':flecha_derecha_curvándose_hacia_arriba:','fr':':flèche_courbe_haut:','ja':':上カーブ矢印:','ko':':위쪽으로_꺾어지는_우향_화살표:','pt':':seta_para_a_direita_curvada_para_cima:','it':':freccia_curva_in_alto:','fa':':پیکان_خمیده_بالا_راست:','id':':tanda_panah_kanan_melengkung_ke_atas:','zh':':右上弯箭头:','ru':':стрелка_вправо_с_поворотом_вверх:','tr':':yukarı_kıvrımlı_sağ_ok:','ar':':سهم_لأعلى_من_اليسار:'},'\U0001FAF1':{'en':':rightwards_hand:','status':fully_qualified,'E':14,'de':':nach_rechts_weisende_hand:','es':':mano_hacia_la_derecha:','fr':':main_vers_la_droite:','ja':':右に向けた手:','ko':':오른쪽으로_향하는_손등:','pt':':mão_para_a_direita:','it':':mano_rivolta_a_destra:','fa':':دست_به_سمت_راست:','id':':tangan_menghadap_kanan:','zh':':向右的手:','ru':':рука_вправо:','tr':':sağa_bakan_el:','ar':':يد_باتجاه_اليمين:'},'\U0001FAF1\U0001F3FF':{'en':':rightwards_hand_dark_skin_tone:','status':fully_qualified,'E':14,'de':':nach_rechts_weisende_hand_dunkle_hautfarbe:','es':':mano_hacia_la_derecha_tono_de_piel_oscuro:','fr':':main_vers_la_droite_peau_foncée:','ja':':右に向けた手_濃い肌色:','ko':':오른쪽으로_향하는_손등_검은색_피부:','pt':':mão_para_a_direita_pele_escura:','it':':mano_rivolta_a_destra_carnagione_scura:','fa':':دست_به_سمت_راست_پوست_آبنوسی:','id':':tangan_menghadap_kanan_warna_kulit_gelap:','zh':':向右的手_较深肤色:','ru':':рука_вправо_очень_темный_тон_кожи:','tr':':sağa_bakan_el_koyu_cilt_tonu:','ar':':يد_باتجاه_اليمين_بشرة_بلون_غامق:'},'\U0001FAF1\U0001F3FB':{'en':':rightwards_hand_light_skin_tone:','status':fully_qualified,'E':14,'de':':nach_rechts_weisende_hand_helle_hautfarbe:','es':':mano_hacia_la_derecha_tono_de_piel_claro:','fr':':main_vers_la_droite_peau_claire:','ja':':右に向けた手_薄い肌色:','ko':':오른쪽으로_향하는_손등_하얀_피부:','pt':':mão_para_a_direita_pele_clara:','it':':mano_rivolta_a_destra_carnagione_chiara:','fa':':دست_به_سمت_راست_پوست_سفید:','id':':tangan_menghadap_kanan_warna_kulit_cerah:','zh':':向右的手_较浅肤色:','ru':':рука_вправо_очень_светлый_тон_кожи:','tr':':sağa_bakan_el_açık_cilt_tonu:','ar':':يد_باتجاه_اليمين_بشرة_بلون_فاتح:'},'\U0001FAF1\U0001F3FE':{'en':':rightwards_hand_medium-dark_skin_tone:','status':fully_qualified,'E':14,'de':':nach_rechts_weisende_hand_mitteldunkle_hautfarbe:','es':':mano_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':main_vers_la_droite_peau_mate:','ja':':右に向けた手_やや濃い肌色:','ko':':오른쪽으로_향하는_손등_진한_갈색_피부:','pt':':mão_para_a_direita_pele_morena_escura:','it':':mano_rivolta_a_destra_carnagione_abbastanza_scura:','fa':':دست_به_سمت_راست_پوست_گندمی:','id':':tangan_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':向右的手_中等深肤色:','ru':':рука_вправо_темный_тон_кожи:','tr':':sağa_bakan_el_orta_koyu_cilt_tonu:','ar':':يد_باتجاه_اليمين_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001FAF1\U0001F3FC':{'en':':rightwards_hand_medium-light_skin_tone:','status':fully_qualified,'E':14,'de':':nach_rechts_weisende_hand_mittelhelle_hautfarbe:','es':':mano_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':main_vers_la_droite_peau_moyennement_claire:','ja':':右に向けた手_やや薄い肌色:','ko':':오른쪽으로_향하는_손등_연한_갈색_피부:','pt':':mão_para_a_direita_pele_morena_clara:','it':':mano_rivolta_a_destra_carnagione_abbastanza_chiara:','fa':':دست_به_سمت_راست_پوست_روشن:','id':':tangan_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':向右的手_中等浅肤色:','ru':':рука_вправо_светлый_тон_кожи:','tr':':sağa_bakan_el_orta_açık_cilt_tonu:','ar':':يد_باتجاه_اليمين_بشرة_بلون_فاتح_ومعتدل:'},'\U0001FAF1\U0001F3FD':{'en':':rightwards_hand_medium_skin_tone:','status':fully_qualified,'E':14,'de':':nach_rechts_weisende_hand_mittlere_hautfarbe:','es':':mano_hacia_la_derecha_tono_de_piel_medio:','fr':':main_vers_la_droite_peau_légèrement_mate:','ja':':右に向けた手_中間の肌色:','ko':':오른쪽으로_향하는_손등_갈색_피부:','pt':':mão_para_a_direita_pele_morena:','it':':mano_rivolta_a_destra_carnagione_olivastra:','fa':':دست_به_سمت_راست_پوست_طلایی:','id':':tangan_menghadap_kanan_warna_kulit_sedang:','zh':':向右的手_中等肤色:','ru':':рука_вправо_средний_тон_кожи:','tr':':sağa_bakan_el_orta_cilt_tonu:','ar':':يد_باتجاه_اليمين_بشرة_بلون_معتدل:'},'\U0001FAF8':{'en':':rightwards_pushing_hand:','status':fully_qualified,'E':15,'de':':nach_rechts_schiebende_hand:','es':':mano_empujando_hacia_la_derecha:','fr':':main_qui_pousse_vers_la_droite:','ja':':右向きに押している手:','ko':':오른쪽으로_밀치는_손:','pt':':mão_empurrando_para_a_direita:','it':':mano_che_spinge_a_destra:','fa':':دست_فشاردهنده_به_سمت_راست:','id':':tangan_mendorong_ke_kanan:','zh':':向右推:','ru':':рука_ладонью_вправо:','tr':':sağa_iten_el:','ar':':يد_تدفع_يمينا:'},'\U0001FAF8\U0001F3FF':{'en':':rightwards_pushing_hand_dark_skin_tone:','status':fully_qualified,'E':15,'de':':nach_rechts_schiebende_hand_dunkle_hautfarbe:','es':':mano_empujando_hacia_la_derecha_tono_de_piel_oscuro:','fr':':main_qui_pousse_vers_la_droite_peau_foncée:','ja':':右向きに押している手_濃い肌色:','ko':':오른쪽으로_밀치는_손_검은색_피부:','pt':':mão_empurrando_para_a_direita_pele_escura:','it':':mano_che_spinge_a_destra_carnagione_scura:','fa':':دست_فشاردهنده_به_سمت_راست_پوست_آبنوسی:','id':':tangan_mendorong_ke_kanan_warna_kulit_gelap:','zh':':向右推_较深肤色:','ru':':рука_ладонью_вправо_очень_темный_тон_кожи:','tr':':sağa_iten_el_koyu_cilt_tonu:','ar':':يد_تدفع_يمينا_بشرة_بلون_غامق:'},'\U0001FAF8\U0001F3FB':{'en':':rightwards_pushing_hand_light_skin_tone:','status':fully_qualified,'E':15,'de':':nach_rechts_schiebende_hand_helle_hautfarbe:','es':':mano_empujando_hacia_la_derecha_tono_de_piel_claro:','fr':':main_qui_pousse_vers_la_droite_peau_claire:','ja':':右向きに押している手_薄い肌色:','ko':':오른쪽으로_밀치는_손_하얀_피부:','pt':':mão_empurrando_para_a_direita_pele_clara:','it':':mano_che_spinge_a_destra_carnagione_chiara:','fa':':دست_فشاردهنده_به_سمت_راست_پوست_سفید:','id':':tangan_mendorong_ke_kanan_warna_kulit_cerah:','zh':':向右推_较浅肤色:','ru':':рука_ладонью_вправо_очень_светлый_тон_кожи:','tr':':sağa_iten_el_açık_cilt_tonu:','ar':':يد_تدفع_يمينا_بشرة_بلون_فاتح:'},'\U0001FAF8\U0001F3FE':{'en':':rightwards_pushing_hand_medium-dark_skin_tone:','status':fully_qualified,'E':15,'de':':nach_rechts_schiebende_hand_mitteldunkle_hautfarbe:','es':':mano_empujando_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':main_qui_pousse_vers_la_droite_peau_mate:','ja':':右向きに押している手_やや濃い肌色:','ko':':오른쪽으로_밀치는_손_진한_갈색_피부:','pt':':mão_empurrando_para_a_direita_pele_morena_escura:','it':':mano_che_spinge_a_destra_carnagione_abbastanza_scura:','fa':':دست_فشاردهنده_به_سمت_راست_پوست_گندمی:','id':':tangan_mendorong_ke_kanan_warna_kulit_gelap-sedang:','zh':':向右推_中等深肤色:','ru':':рука_ладонью_вправо_темный_тон_кожи:','tr':':sağa_iten_el_orta_koyu_cilt_tonu:','ar':':يد_تدفع_يمينا_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001FAF8\U0001F3FC':{'en':':rightwards_pushing_hand_medium-light_skin_tone:','status':fully_qualified,'E':15,'de':':nach_rechts_schiebende_hand_mittelhelle_hautfarbe:','es':':mano_empujando_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':main_qui_pousse_vers_la_droite_peau_moyennement_claire:','ja':':右向きに押している手_やや薄い肌色:','ko':':오른쪽으로_밀치는_손_연한_갈색_피부:','pt':':mão_empurrando_para_a_direita_pele_morena_clara:','it':':mano_che_spinge_a_destra_carnagione_abbastanza_chiara:','fa':':دست_فشاردهنده_به_سمت_راست_پوست_روشن:','id':':tangan_mendorong_ke_kanan_warna_kulit_cerah-sedang:','zh':':向右推_中等浅肤色:','ru':':рука_ладонью_вправо_светлый_тон_кожи:','tr':':sağa_iten_el_orta_açık_cilt_tonu:','ar':':يد_تدفع_يمينا_بشرة_بلون_فاتح_ومعتدل:'},'\U0001FAF8\U0001F3FD':{'en':':rightwards_pushing_hand_medium_skin_tone:','status':fully_qualified,'E':15,'de':':nach_rechts_schiebende_hand_mittlere_hautfarbe:','es':':mano_empujando_hacia_la_derecha_tono_de_piel_medio:','fr':':main_qui_pousse_vers_la_droite_peau_légèrement_mate:','ja':':右向きに押している手_中間の肌色:','ko':':오른쪽으로_밀치는_손_갈색_피부:','pt':':mão_empurrando_para_a_direita_pele_morena:','it':':mano_che_spinge_a_destra_carnagione_olivastra:','fa':':دست_فشاردهنده_به_سمت_راست_پوست_طلایی:','id':':tangan_mendorong_ke_kanan_warna_kulit_sedang:','zh':':向右推_中等肤色:','ru':':рука_ладонью_вправо_средний_тон_кожи:','tr':':sağa_iten_el_orta_cilt_tonu:','ar':':يد_تدفع_يمينا_بشرة_بلون_معتدل:'},'\U0001F48D':{'en':':ring:','status':fully_qualified,'E':0.6,'de':':ring:','es':':anillo:','fr':':bague:','ja':':指輪:','ko':':반지:','pt':':anel:','it':':anello:','fa':':حلقه:','id':':cincin:','zh':':戒指:','ru':':кольцо:','tr':':yüzük:','ar':':خاتم:'},'\U0001F6DF':{'en':':ring_buoy:','status':fully_qualified,'E':14,'de':':rettungsring:','es':':salvavidas:','fr':':bouée_de_sauvetage:','ja':':救命浮環:','ko':':구명부환:','pt':':boia:','it':':salvagente:','fa':':حلقه_نجات:','id':':ban_pelampung:','zh':':救生圈:','ru':':спасательный_круг:','tr':':cankurtaran_simidi:','ar':':عوامة:'},'\U0001FA90':{'en':':ringed_planet:','status':fully_qualified,'E':12,'de':':ringplanet:','es':':planeta_con_anillos:','fr':':planète_à_anneaux:','ja':':環のある惑星:','ko':':고리가_있는_행성:','pt':':planeta_com_anéis:','it':':pianeta_con_satellite:','fa':':سیاره_حلقه_دار:','id':':planet_bercincin:','zh':':有环行星:','ru':':планета_с_кольцом:','tr':':halkalı_gezegen:','ar':':كوكب_داخل_حلقة:'},'\U0001F360':{'en':':roasted_sweet_potato:','status':fully_qualified,'E':0.6,'alias':[':sweet_potato:'],'de':':geröstete_süßkartoffel:','es':':patata_asada:','fr':':patate_douce:','ja':':焼き芋:','ko':':고구마:','pt':':batata_assada:','it':':patata_dolce_arrosto:','fa':':سیب_زمینی_شیرین_کباب_شده:','id':':ubi_panggang:','zh':':烤红薯:','ru':':печеный_сладкий_картофель:','tr':':fırınlanmış_patates:','ar':':بطاطا_حلوة_مشوية:'},'\U0001F916':{'en':':robot:','status':fully_qualified,'E':1,'alias':[':robot_face:'],'de':':roboter:','es':':robot:','fr':':robot:','ja':':ロボット:','ko':':로봇_얼굴:','pt':':rosto_de_robô:','it':':faccina_di_robot:','fa':':روبات:','id':':robot:','zh':':机器人:','ru':':робот:','tr':':robot:','ar':':وجه_روبوت:'},'\U0001FAA8':{'en':':rock:','status':fully_qualified,'E':13,'de':':felsen:','es':':piedra:','fr':':rocher:','ja':':岩石:','ko':':바위:','pt':':pedra:','it':':roccia:','fa':':صخره:','id':':batu:','zh':':岩石:','ru':':камень:','tr':':kaya:','ar':':صخرة:'},'\U0001F680':{'en':':rocket:','status':fully_qualified,'E':0.6,'de':':rakete:','es':':cohete:','fr':':fusée:','ja':':ロケット:','ko':':로켓:','pt':':foguete:','it':':razzo:','fa':':موشک:','id':':roket:','zh':':火箭:','ru':':ракета:','tr':':roket:','ar':':صاروخ:'},'\U0001F9FB':{'en':':roll_of_paper:','status':fully_qualified,'E':11,'de':':küchenrolle:','es':':rollo_de_papel:','fr':':rouleau_de_papier:','ja':':トイレットペーパー:','ko':':두루마리:','pt':':rolo_de_papel:','it':':rotolo_di_carta_igienica:','fa':':رول_کاغذ:','id':':gulungan_kertas:','zh':':卷纸:','ru':':рулон_бумаги:','tr':':kağıt_rulosu:','ar':':لفة_ورق:'},'\U0001F5DE\U0000FE0F':{'en':':rolled-up_newspaper:','status':fully_qualified,'E':0.7,'alias':[':rolled_up_newspaper:', ':newspaper_roll:'],'variant':True,'de':':zusammengerollte_zeitung:','es':':periódico_enrollado:','fr':':journal_roulé:','ja':':丸めた新聞:','ko':':돌돌_말린_신문:','pt':':jornal_enrolado:','it':':quotidiano_arrotolato:','fa':':رول_روزنامه:','id':':gulungan_koran:','zh':':报纸卷:','ru':':свернутая_газета:','tr':':rulo_gazete:','ar':':صحيفة_مطوية:'},'\U0001F5DE':{'en':':rolled-up_newspaper:','status':unqualified,'E':0.7,'alias':[':rolled_up_newspaper:', ':newspaper_roll:'],'variant':True,'de':':zusammengerollte_zeitung:','es':':periódico_enrollado:','fr':':journal_roulé:','ja':':丸めた新聞:','ko':':돌돌_말린_신문:','pt':':jornal_enrolado:','it':':quotidiano_arrotolato:','fa':':رول_روزنامه:','id':':gulungan_koran:','zh':':报纸卷:','ru':':свернутая_газета:','tr':':rulo_gazete:','ar':':صحيفة_مطوية:'},'\U0001F3A2':{'en':':roller_coaster:','status':fully_qualified,'E':0.6,'de':':achterbahn:','es':':montaña_rusa:','fr':':montagnes_russes:','ja':':ジェットコースター:','ko':':롤러코스터:','pt':':montanha_russa:','it':':montagne_russe:','fa':':ترن_هوایی:','id':':roller_coaster:','zh':':过山车:','ru':':американские_горки:','tr':':hız_treni:','ar':':قطار_ملاهي:'},'\U0001F6FC':{'en':':roller_skate:','status':fully_qualified,'E':13,'de':':rollschuh:','es':':patines:','fr':':patin_à_roulettes:','ja':':ローラースケート:','ko':':롤러스케이트:','pt':':patins_de_rodas:','it':':pattini_a_rotelle:','fa':':کفش_اسکیت:','id':':sepatu_roda:','zh':':四轮滑冰鞋:','ru':':роликовые_коньки:','tr':':paten:','ar':':حذاء_مع_عجلات:'},'\U0001F923':{'en':':rolling_on_the_floor_laughing:','status':fully_qualified,'E':3,'alias':[':rofl:'],'de':':sich_vor_lachen_auf_dem_boden_wälzen:','es':':cara_revolviéndose_de_la_risa:','fr':':se_rouler_par_terre_de_rire:','ja':':笑い転げる:','ko':':바닥을_구르며_웃는_얼굴:','pt':':rolando_no_chão_de_rir:','it':':ridere_a_crepapelle:','fa':':از_خنده_غش_کردن:','id':':tertawa_terbahak-bahak:','zh':':笑得满地打滚:','ru':':катается_со_смеху:','tr':':gülmekten_yerlere_yatma:','ar':':وجه_ضاحك_يتدحرج_على_الأرض:'},'\U0001F413':{'en':':rooster:','status':fully_qualified,'E':1,'de':':hahn:','es':':gallo:','fr':':coq:','ja':':おんどり:','ko':':수탉:','pt':':galo:','it':':gallo:','fa':':خروس:','id':':ayam_jago:','zh':':公鸡:','ru':':петух:','tr':':horoz:','ar':':ديك:'},'\U0001F339':{'en':':rose:','status':fully_qualified,'E':0.6,'de':':rose:','es':':rosa:','fr':':rose:','ja':':バラ:','ko':':장미꽃:','pt':':rosa:','it':':rosa:','fa':':رز:','id':':mawar:','zh':':玫瑰:','ru':':роза:','tr':':gül:','ar':':وردة:'},'\U0001F3F5\U0000FE0F':{'en':':rosette:','status':fully_qualified,'E':0.7,'variant':True,'de':':rosette:','es':':roseta:','fr':':rosette:','ja':':花飾り:','ko':':장미:','pt':':roseta:','it':':rosetta:','fa':':گل_رزت:','id':':hiasan_mawar:','zh':':圆形花饰:','ru':':розетка:','tr':':çiçek_rozet:','ar':':وردية:'},'\U0001F3F5':{'en':':rosette:','status':unqualified,'E':0.7,'variant':True,'de':':rosette:','es':':roseta:','fr':':rosette:','ja':':花飾り:','ko':':장미:','pt':':roseta:','it':':rosetta:','fa':':گل_رزت:','id':':hiasan_mawar:','zh':':圆形花饰:','ru':':розетка:','tr':':çiçek_rozet:','ar':':وردية:'},'\U0001F4CD':{'en':':round_pushpin:','status':fully_qualified,'E':0.6,'de':':stecknadel:','es':':chincheta_redonda:','fr':':épingle:','ja':':丸い押しピン:','ko':':둥근_머리_핀:','pt':':tacha_redonda:','it':':puntina_arrotondata:','fa':':سنجاق_ته_گرد:','id':':paku_penanda_bulat:','zh':':圆图钉:','ru':':канцелярская_кнопка_с_шариком:','tr':':yuvarlak_raptiye:','ar':':دبوس_مستدير:'},'\U0001F3C9':{'en':':rugby_football:','status':fully_qualified,'E':1,'de':':rugbyball:','es':':balón_de_rugby:','fr':':rugby:','ja':':ラグビー:','ko':':럭비공:','pt':':bola_de_rugby:','it':':pallone_da_rugby:','fa':':راگبی:','id':':sepak_bola_rugbi:','zh':':英式橄榄球:','ru':':регби:','tr':':ragbi:','ar':':كرة_رجبي:'},'\U0001F3BD':{'en':':running_shirt:','status':fully_qualified,'E':0.6,'alias':[':running_shirt_with_sash:'],'de':':laufshirt:','es':':camiseta_sin_mangas:','fr':':maillot_de_course:','ja':':長距離走:','ko':':러닝_셔츠:','pt':':camiseta_de_corrida:','it':':maglia_da_corsa:','fa':':لباس_دو:','id':':kaos_lari:','zh':':运动背心:','ru':':футболка_для_бега:','tr':':koşu_tişörtü:','ar':':قميص_رياضي:'},'\U0001F45F':{'en':':running_shoe:','status':fully_qualified,'E':0.6,'alias':[':athletic_shoe:'],'de':':sportschuh:','es':':zapatilla_deportiva:','fr':':chaussure_de_sport:','ja':':スニーカー:','ko':':운동화:','pt':':tênis_de_corrida:','it':':scarpa_sportiva:','fa':':کفش_دویدن:','id':':sepatu_lari:','zh':':跑鞋:','ru':':кроссовки:','tr':':koşu_ayakkabısı:','ar':':حذاء_رياضي:'},'\U0001F625':{'en':':sad_but_relieved_face:','status':fully_qualified,'E':0.6,'alias':[':disappointed_relieved:'],'de':':trauriges_aber_erleichtertes_gesicht:','es':':cara_triste_pero_aliviada:','fr':':visage_triste_mais_soulagé:','ja':':どうしよう:','ko':':실망했지만_안도하는_얼굴:','pt':':rosto_triste_mas_aliviado:','it':':faccina_delusa_ma_sollevata:','fa':':ناامید_اما_تسکین_یافته:','id':':wajah_kecewa_namun_lega:','zh':':失望但如释重负:','ru':':обошлось:','tr':':üzgün_ama_rahatlamış_yüz:','ar':':وجه_خيبة_أمل:'},'\U0001F9F7':{'en':':safety_pin:','status':fully_qualified,'E':11,'de':':sicherheitsnadel:','es':':imperdible:','fr':':épingle_de_sûreté:','ja':':安全ピン:','ko':':옷핀:','pt':':alfinete_de_segurança:','it':':spilla_da_balia:','fa':':سنجاق_قفلی:','id':':peniti:','zh':':安全别针:','ru':':английская_булавка:','tr':':çengelli_iğne:','ar':':دبوس_أمان:'},'\U0001F9BA':{'en':':safety_vest:','status':fully_qualified,'E':12,'de':':sicherheitsweste:','es':':chaleco_de_seguridad:','fr':':gilet_de_sécurité:','ja':':安全ベスト:','ko':':구명조끼:','pt':':colete_salva-vidas:','it':':gilet_di_sicurezza:','fa':':جلیقه_ایمنی:','id':':rompi_pengaman:','zh':':救生衣:','ru':':спасательный_жилет:','tr':':güvenlik_yeleği:','ar':':سترة_سلامة:'},'\U000026F5':{'en':':sailboat:','status':fully_qualified,'E':0.6,'alias':[':boat:'],'variant':True,'de':':segelboot:','es':':velero:','fr':':voilier:','ja':':ヨット:','ko':':돛단배:','pt':':barco_a_vela:','it':':barca_a_vela:','fa':':قایق_بادبانی:','id':':perahu_layar:','zh':':帆船:','ru':':парусник:','tr':':yelkenli_tekne:','ar':':مركب_شراعي:'},'\U0001F376':{'en':':sake:','status':fully_qualified,'E':0.6,'de':':sake-flasche_mit_tasse:','es':':sake:','fr':':saké:','ja':':徳利:','ko':':사케:','pt':':saquê:','it':':sakè:','fa':':ساکی:','id':':sake:','zh':':清酒:','ru':':бутылка_саке:','tr':':sake:','ar':':زجاجة_وفنجان:'},'\U0001F9C2':{'en':':salt:','status':fully_qualified,'E':11,'de':':salz:','es':':sal:','fr':':sel:','ja':':塩:','ko':':소금:','pt':':sal:','it':':sale:','fa':':نمک:','id':':garam:','zh':':盐:','ru':':солонка:','tr':':tuz:','ar':':ملح:'},'\U0001FAE1':{'en':':saluting_face:','status':fully_qualified,'E':14,'de':':salutierendes_gesicht:','es':':cara_saludando:','fr':':visage_qui_fait_un_salut_militaire:','ja':':敬礼する顔:','ko':':경례하는_얼굴:','pt':':rosto_saudando:','it':':faccina_che_fa_il_saluto:','fa':':چهره_سلام_کنان:','id':':wajah_memberi_hormat:','zh':':致敬:','ru':':воинское_приветствие:','tr':':selamlayan_yüz:','ar':':وجه_يلقي_التحية:'},'\U0001F96A':{'en':':sandwich:','status':fully_qualified,'E':5,'de':':sandwich:','es':':sándwich:','fr':':sandwich:','ja':':サンドイッチ:','ko':':샌드위치:','pt':':sanduíche:','it':':sandwich:','fa':':ساندویچ:','id':':roti_lapis:','zh':':三明治:','ru':':сэндвич:','tr':':sandviç:','ar':':ساندوتش:'},'\U0001F97B':{'en':':sari:','status':fully_qualified,'E':12,'de':':sari:','es':':sari:','fr':':sari:','ja':':サリー:','ko':':사리:','pt':':sari:','it':':sari:','fa':':ساری:','id':':sari:','zh':':纱丽:','ru':':сари:','tr':':sari:','ar':':ساري:'},'\U0001F6F0\U0000FE0F':{'en':':satellite:','status':fully_qualified,'E':0.7,'alias':[':artificial_satellite:'],'variant':True,'de':':satellit:','es':':satélite:','fr':':satellite:','ja':':人工衛星:','ko':':인공위성:','pt':':satélite:','it':':satellite:','fa':':ماهواره:','id':':satelit:','zh':':卫星:','ru':':спутник:','tr':':uydu:','ar':':قمر_صناعي:'},'\U0001F6F0':{'en':':satellite:','status':unqualified,'E':0.7,'alias':[':artificial_satellite:'],'variant':True,'de':':satellit:','es':':satélite:','fr':':satellite:','ja':':人工衛星:','ko':':인공위성:','pt':':satélite:','it':':satellite:','fa':':ماهواره:','id':':satelit:','zh':':卫星:','ru':':спутник:','tr':':uydu:','ar':':قمر_صناعي:'},'\U0001F4E1':{'en':':satellite_antenna:','status':fully_qualified,'E':0.6,'alias':[':satellite:'],'de':':satellitenschüssel:','es':':antena_de_satélite:','fr':':antenne_satellite:','ja':':パラボラアンテナ:','ko':':위성_안테나:','pt':':antena_parabólica:','it':':antenna_satellitare:','fa':':آنتن_ماهواره:','id':':antena_satelit:','zh':':卫星天线:','ru':':спутниковая_антенна:','tr':':uydu_anteni:','ar':':طبق_قمر_صناعي:'},'\U0001F995':{'en':':sauropod:','status':fully_qualified,'E':5,'de':':sauropode:','es':':saurópodo:','fr':':sauropode:','ja':':草食恐竜:','ko':':초식_공룡:','pt':':saurópode:','it':':sauropode:','fa':':سوسمار:','id':':sauropod:','zh':':蜥蜴类:','ru':':зауропод:','tr':':soropod:','ar':':ديناصور_صوروبودا:'},'\U0001F3B7':{'en':':saxophone:','status':fully_qualified,'E':0.6,'de':':saxofon:','es':':saxofón:','fr':':saxophone:','ja':':サックス:','ko':':색소폰:','pt':':saxofone:','it':':sassofono:','fa':':ساکسیفون:','id':':saksofon:','zh':':萨克斯管:','ru':':саксофон:','tr':':saksafon:','ar':':ساكسفون:'},'\U0001F9E3':{'en':':scarf:','status':fully_qualified,'E':5,'de':':schal:','es':':bufanda:','fr':':foulard:','ja':':マフラー:','ko':':스카프:','pt':':cachecol:','it':':sciarpa:','fa':':روسری:','id':':syal:','zh':':围巾:','ru':':шарф:','tr':':atkı:','ar':':وشاح:'},'\U0001F3EB':{'en':':school:','status':fully_qualified,'E':0.6,'de':':schule:','es':':colegio:','fr':':école:','ja':':学校:','ko':':학교:','pt':':escola:','it':':scuola:','fa':':مدرسه:','id':':sekolah:','zh':':学校:','ru':':школа:','tr':':okul:','ar':':مدرسة:'},'\U0001F9D1\U0000200D\U0001F52C':{'en':':scientist:','status':fully_qualified,'E':12.1,'de':':wissenschaftler(in):','es':':profesional_de_la_ciencia:','fr':':scientifique:','ja':':科学者:','ko':':과학자:','pt':':cientista:','it':':persona_che_lavora_in_campo_scientifico:','fa':':دانشمند:','id':':ilmuwan:','zh':':科学家:','ru':':ученый:','tr':':bilim_insanı:','ar':':عالِم:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F52C':{'en':':scientist_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':wissenschaftler(in)_dunkle_hautfarbe:','es':':profesional_de_la_ciencia_tono_de_piel_oscuro:','fr':':scientifique_peau_foncée:','ja':':科学者_濃い肌色:','ko':':과학자_검은색_피부:','pt':':cientista_pele_escura:','it':':persona_che_lavora_in_campo_scientifico_carnagione_scura:','fa':':دانشمند_پوست_آبنوسی:','id':':ilmuwan_warna_kulit_gelap:','zh':':科学家_较深肤色:','ru':':ученый_очень_темный_тон_кожи:','tr':':bilim_insanı_koyu_cilt_tonu:','ar':':عالِم_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F52C':{'en':':scientist_light_skin_tone:','status':fully_qualified,'E':12.1,'de':':wissenschaftler(in)_helle_hautfarbe:','es':':profesional_de_la_ciencia_tono_de_piel_claro:','fr':':scientifique_peau_claire:','ja':':科学者_薄い肌色:','ko':':과학자_하얀_피부:','pt':':cientista_pele_clara:','it':':persona_che_lavora_in_campo_scientifico_carnagione_chiara:','fa':':دانشمند_پوست_سفید:','id':':ilmuwan_warna_kulit_cerah:','zh':':科学家_较浅肤色:','ru':':ученый_очень_светлый_тон_кожи:','tr':':bilim_insanı_açık_cilt_tonu:','ar':':عالِم_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F52C':{'en':':scientist_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':wissenschaftler(in)_mitteldunkle_hautfarbe:','es':':profesional_de_la_ciencia_tono_de_piel_oscuro_medio:','fr':':scientifique_peau_mate:','ja':':科学者_やや濃い肌色:','ko':':과학자_진한_갈색_피부:','pt':':cientista_pele_morena_escura:','it':':persona_che_lavora_in_campo_scientifico_carnagione_abbastanza_scura:','fa':':دانشمند_پوست_گندمی:','id':':ilmuwan_warna_kulit_gelap-sedang:','zh':':科学家_中等深肤色:','ru':':ученый_темный_тон_кожи:','tr':':bilim_insanı_orta_koyu_cilt_tonu:','ar':':عالِم_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F52C':{'en':':scientist_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':wissenschaftler(in)_mittelhelle_hautfarbe:','es':':profesional_de_la_ciencia_tono_de_piel_claro_medio:','fr':':scientifique_peau_moyennement_claire:','ja':':科学者_やや薄い肌色:','ko':':과학자_연한_갈색_피부:','pt':':cientista_pele_morena_clara:','it':':persona_che_lavora_in_campo_scientifico_carnagione_abbastanza_chiara:','fa':':دانشمند_پوست_روشن:','id':':ilmuwan_warna_kulit_cerah-sedang:','zh':':科学家_中等浅肤色:','ru':':ученый_светлый_тон_кожи:','tr':':bilim_insanı_orta_açık_cilt_tonu:','ar':':عالِم_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F52C':{'en':':scientist_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':wissenschaftler(in)_mittlere_hautfarbe:','es':':profesional_de_la_ciencia_tono_de_piel_medio:','fr':':scientifique_peau_légèrement_mate:','ja':':科学者_中間の肌色:','ko':':과학자_갈색_피부:','pt':':cientista_pele_morena:','it':':persona_che_lavora_in_campo_scientifico_carnagione_olivastra:','fa':':دانشمند_پوست_طلایی:','id':':ilmuwan_warna_kulit_sedang:','zh':':科学家_中等肤色:','ru':':ученый_средний_тон_кожи:','tr':':bilim_insanı_orta_cilt_tonu:','ar':':عالِم_بشرة_بلون_معتدل:'},'\U00002702\U0000FE0F':{'en':':scissors:','status':fully_qualified,'E':0.6,'variant':True,'de':':schere:','es':':tijeras:','fr':':ciseaux:','ja':':はさみ:','ko':':가위:','pt':':tesoura:','it':':forbici:','fa':':قیچی:','id':':gunting:','zh':':剪刀:','ru':':ножницы:','tr':':makas:','ar':':مقص:'},'\U00002702':{'en':':scissors:','status':unqualified,'E':0.6,'variant':True,'de':':schere:','es':':tijeras:','fr':':ciseaux:','ja':':はさみ:','ko':':가위:','pt':':tesoura:','it':':forbici:','fa':':قیچی:','id':':gunting:','zh':':剪刀:','ru':':ножницы:','tr':':makas:','ar':':مقص:'},'\U0001F982':{'en':':scorpion:','status':fully_qualified,'E':1,'de':':skorpion:','es':':escorpión:','fr':':scorpion:','ja':':サソリ:','ko':':전갈:','pt':':escorpião:','it':':scorpione:','fa':':عقرب:','id':':kalajengking:','zh':':蝎子:','ru':':скорпион:','tr':':akrep:','ar':':عقرب:'},'\U0001FA9B':{'en':':screwdriver:','status':fully_qualified,'E':13,'de':':schraubenzieher:','es':':destornillador:','fr':':tournevis:','ja':':ねじ回し:','ko':':드라이버:','pt':':chave_de_fenda:','it':':cacciavite:','fa':':پیچ_گوشتی:','id':':obeng:','zh':':螺丝刀:','ru':':отвертка:','tr':':tornavida:','ar':':مفك_براغي:'},'\U0001F4DC':{'en':':scroll:','status':fully_qualified,'E':0.6,'de':':schriftrolle:','es':':pergamino:','fr':':parchemin:','ja':':巻き物:','ko':':문서:','pt':':pergaminho:','it':':pergamena:','fa':':طومار:','id':':gulungan:','zh':':卷轴:','ru':':свиток:','tr':':parşömen:','ar':':ورقة_ملفوفة:'},'\U0001F9AD':{'en':':seal:','status':fully_qualified,'E':13,'de':':seehund:','es':':foca:','fr':':phoque:','ja':':アザラシ:','ko':':물개:','pt':':foca:','it':':foca:','fa':':فُک:','id':':anjing_laut:','zh':':海豹:','ru':':тюлень:','tr':':fok:','ar':':فقمة:'},'\U0001F4BA':{'en':':seat:','status':fully_qualified,'E':0.6,'de':':sitzplatz:','es':':asiento_de_transporte:','fr':':siège:','ja':':座席:','ko':':좌석:','pt':':assento:','it':':sedile:','fa':':جایگاه:','id':':kursi:','zh':':座位:','ru':':кресло:','tr':':koltuk:','ar':':مقعد:'},'\U0001F648':{'en':':see-no-evil_monkey:','status':fully_qualified,'E':0.6,'alias':[':see_no_evil:', ':see_no_evil_monkey:'],'de':':sich_die_augen_zuhaltendes_affengesicht:','es':':mono_con_los_ojos_tapados:','fr':':singe_ne_rien_voir:','ja':':見ざる:','ko':':눈을_가리고_있는_원숭이:','pt':':macaco_que_não_vê_nada:','it':':non_vedo:','fa':':میمون_شرنبین:','id':':tidak_melihat:','zh':':非礼勿视:','ru':':ничего_не_вижу:','tr':':görmedim:','ar':':قرد_يحجب_عينيه:'},'\U0001F331':{'en':':seedling:','status':fully_qualified,'E':0.6,'de':':spross:','es':':planta_joven:','fr':':jeune_pousse:','ja':':新芽:','ko':':새싹:','pt':':muda_de_planta:','it':':germoglio:','fa':':جوانه:','id':':semai:','zh':':幼苗:','ru':':росток:','tr':':filizlenme:','ar':':شتلة:'},'\U0001F933':{'en':':selfie:','status':fully_qualified,'E':3,'de':':selfie:','es':':selfi:','fr':':selfie:','ja':':セルフィー:','ko':':셀카:','pt':':selfie:','it':':selfie:','fa':':خودگرفت:','id':':selfie:','zh':':自拍:','ru':':селфи:','tr':':selfie:','ar':':صورة_سيلفي:'},'\U0001F933\U0001F3FF':{'en':':selfie_dark_skin_tone:','status':fully_qualified,'E':3,'de':':selfie_dunkle_hautfarbe:','es':':selfi_tono_de_piel_oscuro:','fr':':selfie_peau_foncée:','ja':':セルフィー_濃い肌色:','ko':':셀카_검은색_피부:','pt':':selfie_pele_escura:','it':':selfie_carnagione_scura:','fa':':خودگرفت_پوست_آبنوسی:','id':':selfie_warna_kulit_gelap:','zh':':自拍_较深肤色:','ru':':селфи_очень_темный_тон_кожи:','tr':':selfie_koyu_cilt_tonu:','ar':':صورة_سيلفي_بشرة_بلون_غامق:'},'\U0001F933\U0001F3FB':{'en':':selfie_light_skin_tone:','status':fully_qualified,'E':3,'de':':selfie_helle_hautfarbe:','es':':selfi_tono_de_piel_claro:','fr':':selfie_peau_claire:','ja':':セルフィー_薄い肌色:','ko':':셀카_하얀_피부:','pt':':selfie_pele_clara:','it':':selfie_carnagione_chiara:','fa':':خودگرفت_پوست_سفید:','id':':selfie_warna_kulit_cerah:','zh':':自拍_较浅肤色:','ru':':селфи_очень_светлый_тон_кожи:','tr':':selfie_açık_cilt_tonu:','ar':':صورة_سيلفي_بشرة_بلون_فاتح:'},'\U0001F933\U0001F3FE':{'en':':selfie_medium-dark_skin_tone:','status':fully_qualified,'E':3,'de':':selfie_mitteldunkle_hautfarbe:','es':':selfi_tono_de_piel_oscuro_medio:','fr':':selfie_peau_mate:','ja':':セルフィー_やや濃い肌色:','ko':':셀카_진한_갈색_피부:','pt':':selfie_pele_morena_escura:','it':':selfie_carnagione_abbastanza_scura:','fa':':خودگرفت_پوست_گندمی:','id':':selfie_warna_kulit_gelap-sedang:','zh':':自拍_中等深肤色:','ru':':селфи_темный_тон_кожи:','tr':':selfie_orta_koyu_cilt_tonu:','ar':':صورة_سيلفي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F933\U0001F3FC':{'en':':selfie_medium-light_skin_tone:','status':fully_qualified,'E':3,'de':':selfie_mittelhelle_hautfarbe:','es':':selfi_tono_de_piel_claro_medio:','fr':':selfie_peau_moyennement_claire:','ja':':セルフィー_やや薄い肌色:','ko':':셀카_연한_갈색_피부:','pt':':selfie_pele_morena_clara:','it':':selfie_carnagione_abbastanza_chiara:','fa':':خودگرفت_پوست_روشن:','id':':selfie_warna_kulit_cerah-sedang:','zh':':自拍_中等浅肤色:','ru':':селфи_светлый_тон_кожи:','tr':':selfie_orta_açık_cilt_tonu:','ar':':صورة_سيلفي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F933\U0001F3FD':{'en':':selfie_medium_skin_tone:','status':fully_qualified,'E':3,'de':':selfie_mittlere_hautfarbe:','es':':selfi_tono_de_piel_medio:','fr':':selfie_peau_légèrement_mate:','ja':':セルフィー_中間の肌色:','ko':':셀카_갈색_피부:','pt':':selfie_pele_morena:','it':':selfie_carnagione_olivastra:','fa':':خودگرفت_پوست_طلایی:','id':':selfie_warna_kulit_sedang:','zh':':自拍_中等肤色:','ru':':селфи_средний_тон_кожи:','tr':':selfie_orta_cilt_tonu:','ar':':صورة_سيلفي_بشرة_بلون_معتدل:'},'\U0001F415\U0000200D\U0001F9BA':{'en':':service_dog:','status':fully_qualified,'E':12,'de':':assistenzhund:','es':':perro_de_servicio:','fr':':chien_d’assistance:','ja':':介助犬:','ko':':보조견:','pt':':cão_de_serviço:','it':':cane_da_assistenza:','fa':':سگ_کمکی:','id':':anjing_pembimbing:','zh':':服务犬:','ru':':служебная_собака:','tr':':yardımcı_köpek:','ar':':كلب_مساعدة:'},'\U0001F562':{'en':':seven-thirty:','status':fully_qualified,'E':0.7,'alias':[':clock730:', ':seven_thirty:'],'variant':True,'de':':7.30_uhr:','es':':siete_y_media:','fr':':sept_heures_et_demie:','ja':':7時半:','ko':':일곱_시_반:','pt':':sete_e_meia:','it':':ore_sette_e_mezza:','fa':':ساعت_هفت_و_نیم:','id':':jam_setengah_delapan:','zh':':七点半:','ru':':полвосьмого:','tr':':yedi_buçuk:','ar':':السابعة_والنصف:'},'\U0001F556':{'en':':seven_o’clock:','status':fully_qualified,'E':0.6,'alias':[':clock7:', ':seven_oclock:'],'variant':True,'de':':7.00_uhr:','es':':7_en_punto:','fr':':sept_heures:','ja':':7時:','ko':':일곱_시:','pt':':7_horas:','it':':ore_sette:','fa':':ساعت_هفت:','id':':jam_tujuh_tepat:','zh':':七点:','ru':':семь_часов:','tr':':saat_7:','ar':':الساعة_السابعة:'},'\U0001FAA1':{'en':':sewing_needle:','status':fully_qualified,'E':13,'de':':nähnadel:','es':':aguja_de_coser:','fr':':aiguille_à_coudre:','ja':':縫い針:','ko':':바늘:','pt':':agulha_de_costura:','it':':ago_da_cucito:','fa':':سوزن_خیاطی:','id':':jarum_jahit:','zh':':缝合针:','ru':':швейная_игла:','tr':':dikiş_iğnesi:','ar':':إبرة_خياطة:'},'\U0001FAE8':{'en':':shaking_face:','status':fully_qualified,'E':15,'de':':zitterndes_doppelgesicht:','es':':cara_temblorosa:','fr':':visage_secoué:','ja':':震えている顔:','ko':':요동치는_얼굴:','pt':':rosto_tremendo:','it':':faccina_tremante:','fa':':چهره_لرزان:','id':':wajah_gemetar:','zh':':颤抖:','ru':':трясущееся_лицо:','tr':':sallanan_yüz:','ar':':وجه_يهتز:'},'\U0001F958':{'en':':shallow_pan_of_food:','status':fully_qualified,'E':3,'de':':pfannengericht:','es':':paella:','fr':':plat_mitonné:','ja':':パエリア:','ko':':납작한_냄비:','pt':':caçarola_com_comida:','it':':padella_con_cibo:','fa':':ماهیتابه_غذا:','id':':wajan_dangkal:','zh':':装有食物的浅底锅:','ru':':рагу_в_сковороде:','tr':':sahanda_yemek:','ar':':مقلاة_مسطحة_بها_طعام:'},'\U00002618\U0000FE0F':{'en':':shamrock:','status':fully_qualified,'E':1,'variant':True,'de':':kleeblatt:','es':':trébol:','fr':':trèfle:','ja':':クローバー:','ko':':토끼풀:','pt':':trevo:','it':':trifoglio:','fa':':برگ_شبدر:','id':':shamrock:','zh':':三叶草:','ru':':трилистник:','tr':':yonca:','ar':':نبات_النفل:'},'\U00002618':{'en':':shamrock:','status':unqualified,'E':1,'variant':True,'de':':kleeblatt:','es':':trébol:','fr':':trèfle:','ja':':クローバー:','ko':':토끼풀:','pt':':trevo:','it':':trifoglio:','fa':':برگ_شبدر:','id':':shamrock:','zh':':三叶草:','ru':':трилистник:','tr':':yonca:','ar':':نبات_النفل:'},'\U0001F988':{'en':':shark:','status':fully_qualified,'E':3,'de':':hai:','es':':tiburón:','fr':':requin:','ja':':サメ:','ko':':상어:','pt':':tubarão:','it':':squalo:','fa':':کوسه:','id':':hiu:','zh':':鲨鱼:','ru':':акула:','tr':':köpek_balığı:','ar':':سمكة_قرش:'},'\U0001F367':{'en':':shaved_ice:','status':fully_qualified,'E':0.6,'de':':wassereis:','es':':granizado_hawaiano:','fr':':granité:','ja':':かき氷:','ko':':샤베트_아이스크림:','pt':':raspadinha_de_gelo:','it':':granita:','fa':':یخ_در_بهشت:','id':':es_serut:','zh':':刨冰:','ru':':мороженое_в_креманке:','tr':':meyveli_buz:','ar':':آيس_كريم_مجروش:'},'\U0001F33E':{'en':':sheaf_of_rice:','status':fully_qualified,'E':0.6,'alias':[':ear_of_rice:'],'de':':reisähre:','es':':espiga_de_arroz:','fr':':plant_de_riz:','ja':':稲:','ko':':벼:','pt':':planta_de_arroz:','it':':pianta_di_riso:','fa':':خوشه_برنج:','id':':seikat_padi:','zh':':稻子:','ru':':колос:','tr':':pirinç_bitkisi:','ar':':حزمة_من_الأرز:'},'\U0001F6E1\U0000FE0F':{'en':':shield:','status':fully_qualified,'E':0.7,'variant':True,'de':':schutzschild:','es':':escudo:','fr':':bouclier:','ja':':盾:','ko':':방패:','pt':':escudo:','it':':scudo:','fa':':سپر:','id':':perisai:','zh':':盾牌:','ru':':щит:','tr':':kalkan:','ar':':درع:'},'\U0001F6E1':{'en':':shield:','status':unqualified,'E':0.7,'variant':True,'de':':schutzschild:','es':':escudo:','fr':':bouclier:','ja':':盾:','ko':':방패:','pt':':escudo:','it':':scudo:','fa':':سپر:','id':':perisai:','zh':':盾牌:','ru':':щит:','tr':':kalkan:','ar':':درع:'},'\U000026E9\U0000FE0F':{'en':':shinto_shrine:','status':fully_qualified,'E':0.7,'variant':True,'de':':shinto-schrein:','es':':santuario_sintoísta:','fr':':sanctuaire_shinto:','ja':':鳥居:','ko':':신토_신사:','pt':':santuário_japonês:','it':':santuario_shintoista:','fa':':معبد_شینتو:','id':':kuil_shinto:','zh':':神社:','ru':':синтоистскии_храм:','tr':':şinto_tapınağı:','ar':':ضريح:'},'\U000026E9':{'en':':shinto_shrine:','status':unqualified,'E':0.7,'variant':True,'de':':shinto-schrein:','es':':santuario_sintoísta:','fr':':sanctuaire_shinto:','ja':':鳥居:','ko':':신토_신사:','pt':':santuário_japonês:','it':':santuario_shintoista:','fa':':معبد_شینتو:','id':':kuil_shinto:','zh':':神社:','ru':':синтоистский_храм:','tr':':şinto_tapınağı:','ar':':ضريح:'},'\U0001F6A2':{'en':':ship:','status':fully_qualified,'E':0.6,'de':':schiff:','es':':barco:','fr':':navire:','ja':':船:','ko':':선박:','pt':':navio:','it':':nave:','fa':':کشتی:','id':':kapal:','zh':':船:','ru':':корабль:','tr':':gemi:','ar':':سفينة:'},'\U0001F320':{'en':':shooting_star:','status':fully_qualified,'E':0.6,'alias':[':stars:'],'de':':sternschnuppe:','es':':estrella_fugaz:','fr':':étoile_filante:','ja':':流れ星:','ko':':별똥별:','pt':':estrela_cadente:','it':':stella_cadente:','fa':':ستاره_ثاقب:','id':':bintang_jatuh:','zh':':流星:','ru':':падающая_звезда:','tr':':yıldız_kayması:','ar':':شهاب:'},'\U0001F6CD\U0000FE0F':{'en':':shopping_bags:','status':fully_qualified,'E':0.7,'alias':[':shopping:'],'variant':True,'de':':einkaufstüten:','es':':bolsas_de_compras:','fr':':sacs_de_shopping:','ja':':紙袋:','ko':':쇼핑백:','pt':':sacolas_de_compras:','it':':borse_della_spesa:','fa':':کیف_خرید:','id':':tas_belanja:','zh':':购物袋:','ru':':пакеты:','tr':':alışveriş_çantası:','ar':':حقائب_تسوق:'},'\U0001F6CD':{'en':':shopping_bags:','status':unqualified,'E':0.7,'alias':[':shopping:'],'variant':True,'de':':einkaufstüten:','es':':bolsas_de_compras:','fr':':sacs_de_shopping:','ja':':紙袋:','ko':':쇼핑백:','pt':':sacolas_de_compras:','it':':borse_della_spesa:','fa':':کیف_خرید:','id':':tas_belanja:','zh':':购物袋:','ru':':пакеты:','tr':':alışveriş_çantası:','ar':':حقائب_تسوق:'},'\U0001F6D2':{'en':':shopping_cart:','status':fully_qualified,'E':3,'de':':einkaufswagen:','es':':carrito_de_la_compra:','fr':':chariot:','ja':':ショッピングカート:','ko':':쇼핑카트:','pt':':carrinho_de_compras:','it':':carrello:','fa':':گاری_خرید:','id':':troli_belanja:','zh':':购物车:','ru':':тележка_для_покупок:','tr':':alışveriş_arabası:','ar':':عربة_التسوق:'},'\U0001F370':{'en':':shortcake:','status':fully_qualified,'E':0.6,'alias':[':cake:'],'de':':torte:','es':':trozo_de_tarta:','fr':':gâteau_sablé:','ja':':ショートケーキ:','ko':':조각_케익:','pt':':pão_de_ló_de_morango:','it':':fetta_di_torta:','fa':':شُرت_کیک:','id':':kue:','zh':':水果蛋糕:','ru':':кусочек_торта:','tr':':pasta_dilimi:','ar':':قطعة_من_الكعك:'},'\U0001FA73':{'en':':shorts:','status':fully_qualified,'E':12,'de':':shorts:','es':':pantalones_cortos:','fr':':short:','ja':':ショーツ:','ko':':반바지:','pt':':shorts:','it':':pantaloncini:','fa':':شلوارک:','id':':celana_pendek:','zh':':短裤:','ru':':шорты:','tr':':şort:','ar':':شورت:'},'\U0001F6BF':{'en':':shower:','status':fully_qualified,'E':1,'de':':dusche:','es':':ducha:','fr':':douche:','ja':':シャワー:','ko':':샤워:','pt':':chuveiro:','it':':doccia:','fa':':دوش:','id':':shower:','zh':':淋浴:','ru':':душ:','tr':':duş:','ar':':دوش:'},'\U0001F990':{'en':':shrimp:','status':fully_qualified,'E':3,'de':':garnele:','es':':gamba:','fr':':crevette:','ja':':エビ:','ko':':새우:','pt':':camarão:','it':':gamberetto:','fa':':میگو:','id':':udang:','zh':':虾:','ru':':креветка:','tr':':karides:','ar':':جمبري:'},'\U0001F500':{'en':':shuffle_tracks_button:','status':fully_qualified,'E':1,'alias':[':twisted_rightwards_arrows:'],'de':':zufallsmodus:','es':':reproducción_aleatoria:','fr':':bouton_lecture_aléatoire:','ja':':シャッフルボタン:','ko':':랜덤_재생:','pt':':botão_de_músicas_aleatórias:','it':':pulsante_di_riproduzione_casuale:','fa':':دکمهٔ_پخش_تصادفی:','id':':tombol_acak_lagu:','zh':':随机播放音轨按钮:','ru':':случайный_выбор:','tr':':parça_karıştırma_düğmesi:','ar':':زر_أسهم_متداخلة:'},'\U0001F92B':{'en':':shushing_face:','status':fully_qualified,'E':5,'de':':ermahnendes_gesicht:','es':':cara_pidiendo_silencio:','fr':':visage_avec_un_doigt_sur_la_bouche:','ja':':しーっ:','ko':':쉿_하는_얼굴:','pt':':rosto_fazendo_sinal_de_silêncio:','it':':faccina_che_zittisce:','fa':':هیس:','id':':wajah_menyuruh_diam:','zh':':安静的脸:','ru':':тс_с_с:','tr':':sus_işareti_yapan_yüz:','ar':':وجه_يطلب_الصمت:'},'\U0001F918':{'en':':sign_of_the_horns:','status':fully_qualified,'E':1,'alias':[':metal:'],'de':':teufelsgruß:','es':':mano_haciendo_el_signo_de_cuernos:','fr':':cornes_avec_les_doigts:','ja':':角の指サイン:','ko':':뿔_모양_손짓:','pt':':saudação_do_rock:','it':':segno_delle_corna:','fa':':نماد_شاخ:','id':':lambang_tanduk:','zh':':摇滚:','ru':':рокерский_жест:','tr':':boynuz_işareti:','ar':':إشارة_قرون:'},'\U0001F918\U0001F3FF':{'en':':sign_of_the_horns_dark_skin_tone:','status':fully_qualified,'E':1,'de':':teufelsgruß_dunkle_hautfarbe:','es':':mano_haciendo_el_signo_de_cuernos_tono_de_piel_oscuro:','fr':':cornes_avec_les_doigts_peau_foncée:','ja':':角の指サイン_濃い肌色:','ko':':뿔_모양_손짓_검은색_피부:','pt':':saudação_do_rock_pele_escura:','it':':segno_delle_corna_carnagione_scura:','fa':':نماد_شاخ_پوست_آبنوسی:','id':':lambang_tanduk_warna_kulit_gelap:','zh':':摇滚_较深肤色:','ru':':рокерский_жест_очень_темный_тон_кожи:','tr':':boynuz_işareti_koyu_cilt_tonu:','ar':':إشارة_قرون_بشرة_بلون_غامق:'},'\U0001F918\U0001F3FB':{'en':':sign_of_the_horns_light_skin_tone:','status':fully_qualified,'E':1,'de':':teufelsgruß_helle_hautfarbe:','es':':mano_haciendo_el_signo_de_cuernos_tono_de_piel_claro:','fr':':cornes_avec_les_doigts_peau_claire:','ja':':角の指サイン_薄い肌色:','ko':':뿔_모양_손짓_하얀_피부:','pt':':saudação_do_rock_pele_clara:','it':':segno_delle_corna_carnagione_chiara:','fa':':نماد_شاخ_پوست_سفید:','id':':lambang_tanduk_warna_kulit_cerah:','zh':':摇滚_较浅肤色:','ru':':рокерский_жест_очень_светлый_тон_кожи:','tr':':boynuz_işareti_açık_cilt_tonu:','ar':':إشارة_قرون_بشرة_بلون_فاتح:'},'\U0001F918\U0001F3FE':{'en':':sign_of_the_horns_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':teufelsgruß_mitteldunkle_hautfarbe:','es':':mano_haciendo_el_signo_de_cuernos_tono_de_piel_oscuro_medio:','fr':':cornes_avec_les_doigts_peau_mate:','ja':':角の指サイン_やや濃い肌色:','ko':':뿔_모양_손짓_진한_갈색_피부:','pt':':saudação_do_rock_pele_morena_escura:','it':':segno_delle_corna_carnagione_abbastanza_scura:','fa':':نماد_شاخ_پوست_گندمی:','id':':lambang_tanduk_warna_kulit_gelap-sedang:','zh':':摇滚_中等深肤色:','ru':':рокерский_жест_темный_тон_кожи:','tr':':boynuz_işareti_orta_koyu_cilt_tonu:','ar':':إشارة_قرون_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F918\U0001F3FC':{'en':':sign_of_the_horns_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':teufelsgruß_mittelhelle_hautfarbe:','es':':mano_haciendo_el_signo_de_cuernos_tono_de_piel_claro_medio:','fr':':cornes_avec_les_doigts_peau_moyennement_claire:','ja':':角の指サイン_やや薄い肌色:','ko':':뿔_모양_손짓_연한_갈색_피부:','pt':':saudação_do_rock_pele_morena_clara:','it':':segno_delle_corna_carnagione_abbastanza_chiara:','fa':':نماد_شاخ_پوست_روشن:','id':':lambang_tanduk_warna_kulit_cerah-sedang:','zh':':摇滚_中等浅肤色:','ru':':рокерский_жест_светлый_тон_кожи:','tr':':boynuz_işareti_orta_açık_cilt_tonu:','ar':':إشارة_قرون_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F918\U0001F3FD':{'en':':sign_of_the_horns_medium_skin_tone:','status':fully_qualified,'E':1,'de':':teufelsgruß_mittlere_hautfarbe:','es':':mano_haciendo_el_signo_de_cuernos_tono_de_piel_medio:','fr':':cornes_avec_les_doigts_peau_légèrement_mate:','ja':':角の指サイン_中間の肌色:','ko':':뿔_모양_손짓_갈색_피부:','pt':':saudação_do_rock_pele_morena:','it':':segno_delle_corna_carnagione_olivastra:','fa':':نماد_شاخ_پوست_طلایی:','id':':lambang_tanduk_warna_kulit_sedang:','zh':':摇滚_中等肤色:','ru':':рокерский_жест_средний_тон_кожи:','tr':':boynuz_işareti_orta_cilt_tonu:','ar':':إشارة_قرون_بشرة_بلون_معتدل:'},'\U0001F9D1\U0000200D\U0001F3A4':{'en':':singer:','status':fully_qualified,'E':12.1,'de':':sänger(in):','es':':cantante:','fr':':chanteur_(tous_genres):','ja':':歌手:','ko':':가수:','pt':':cantor:','it':':cantante:','fa':':خواننده:','id':':penyanyi:','zh':':歌手:','ru':':певец:','tr':':şarkıcı:','ar':':مغني:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F3A4':{'en':':singer_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':sänger(in)_dunkle_hautfarbe:','es':':cantante_tono_de_piel_oscuro:','fr':':chanteur_(tous_genres)_peau_foncée:','ja':':歌手_濃い肌色:','ko':':가수_검은색_피부:','pt':':cantor_pele_escura:','it':':cantante_carnagione_scura:','fa':':خواننده_پوست_آبنوسی:','id':':penyanyi_warna_kulit_gelap:','zh':':歌手_较深肤色:','ru':':певец_очень_темный_тон_кожи:','tr':':şarkıcı_koyu_cilt_tonu:','ar':':مغني_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F3A4':{'en':':singer_light_skin_tone:','status':fully_qualified,'E':12.1,'de':':sänger(in)_helle_hautfarbe:','es':':cantante_tono_de_piel_claro:','fr':':chanteur_(tous_genres)_peau_claire:','ja':':歌手_薄い肌色:','ko':':가수_하얀_피부:','pt':':cantor_pele_clara:','it':':cantante_carnagione_chiara:','fa':':خواننده_پوست_سفید:','id':':penyanyi_warna_kulit_cerah:','zh':':歌手_较浅肤色:','ru':':певец_очень_светлый_тон_кожи:','tr':':şarkıcı_açık_cilt_tonu:','ar':':مغني_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F3A4':{'en':':singer_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':sänger(in)_mitteldunkle_hautfarbe:','es':':cantante_tono_de_piel_oscuro_medio:','fr':':chanteur_(tous_genres)_peau_mate:','ja':':歌手_やや濃い肌色:','ko':':가수_진한_갈색_피부:','pt':':cantor_pele_morena_escura:','it':':cantante_carnagione_abbastanza_scura:','fa':':خواننده_پوست_گندمی:','id':':penyanyi_warna_kulit_gelap-sedang:','zh':':歌手_中等深肤色:','ru':':певец_темный_тон_кожи:','tr':':şarkıcı_orta_koyu_cilt_tonu:','ar':':مغني_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F3A4':{'en':':singer_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':sänger(in)_mittelhelle_hautfarbe:','es':':cantante_tono_de_piel_claro_medio:','fr':':chanteur_(tous_genres)_peau_moyennement_claire:','ja':':歌手_やや薄い肌色:','ko':':가수_연한_갈색_피부:','pt':':cantor_pele_morena_clara:','it':':cantante_carnagione_abbastanza_chiara:','fa':':خواننده_پوست_روشن:','id':':penyanyi_warna_kulit_cerah-sedang:','zh':':歌手_中等浅肤色:','ru':':певец_светлый_тон_кожи:','tr':':şarkıcı_orta_açık_cilt_tonu:','ar':':مغني_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F3A4':{'en':':singer_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':sänger(in)_mittlere_hautfarbe:','es':':cantante_tono_de_piel_medio:','fr':':chanteur_(tous_genres)_peau_légèrement_mate:','ja':':歌手_中間の肌色:','ko':':가수_갈색_피부:','pt':':cantor_pele_morena:','it':':cantante_carnagione_olivastra:','fa':':خواننده_پوست_طلایی:','id':':penyanyi_warna_kulit_sedang:','zh':':歌手_中等肤色:','ru':':певец_средний_тон_кожи:','tr':':şarkıcı_orta_cilt_tonu:','ar':':مغني_بشرة_بلون_معتدل:'},'\U0001F561':{'en':':six-thirty:','status':fully_qualified,'E':0.7,'alias':[':clock630:', ':six_thirty:'],'variant':True,'de':':6.30_uhr:','es':':seis_y_media:','fr':':six_heures_et_demie:','ja':':6時半:','ko':':여섯_시_반:','pt':':seis_e_meia:','it':':ore_sei_e_mezza:','fa':':ساعت_شش_و_نیم:','id':':jam_setengah_tujuh:','zh':':六点半:','ru':':полседьмого:','tr':':altı_buçuk:','ar':':السادسة_والنصف:'},'\U0001F555':{'en':':six_o’clock:','status':fully_qualified,'E':0.6,'alias':[':clock6:', ':six_oclock:'],'variant':True,'de':':6.00_uhr:','es':':6_en_punto:','fr':':six_heures:','ja':':6時:','ko':':여섯_시:','pt':':6_horas:','it':':ore_sei:','fa':':ساعت_شش:','id':':jam_enam_tepat:','zh':':六点:','ru':':шесть_часов:','tr':':saat_6:','ar':':الساعة_السادسة:'},'\U0001F6F9':{'en':':skateboard:','status':fully_qualified,'E':11,'de':':skateboard:','es':':monopatín:','fr':':planche_à_roulettes:','ja':':スケートボード:','ko':':스케이트보드:','pt':':skate:','it':':skateboard:','fa':':تختهٔ_اسکیت:','id':':papan_luncur:','zh':':滑板:','ru':':скейтборд:','tr':':kaykay:','ar':':لوح_التزلج:'},'\U000026F7\U0000FE0F':{'en':':skier:','status':fully_qualified,'E':0.7,'variant':True,'de':':skifahrer(in):','es':':esquiador:','fr':':skieur:','ja':':スキーヤー:','ko':':스키_타는_사람:','pt':':esquiador:','it':':sciatore:','fa':':اسکی_سوار:','id':':pemain_ski:','zh':':滑雪的人:','ru':':горные_лыжи:','tr':':kayakçı:','ar':':متزلج_على_الثلج:'},'\U000026F7':{'en':':skier:','status':unqualified,'E':0.7,'variant':True,'de':':skifahrer(in):','es':':persona_esquiando:','fr':':skieur:','ja':':スキーヤー:','ko':':스키_타는_사람:','pt':':esquiador:','it':':sciatore:','fa':':اسکی_سوار:','id':':pemain_ski:','zh':':滑雪的人:','ru':':горные_лыжи:','tr':':kayakçı:','ar':':متزلج_على_الثلج:'},'\U0001F3BF':{'en':':skis:','status':fully_qualified,'E':0.6,'alias':[':ski:'],'de':':ski:','es':':esquís:','fr':':ski:','ja':':スキー:','ko':':스키:','pt':':esqui:','it':':sci:','fa':':چوب_اسکی:','id':':ski:','zh':':滑雪:','ru':':лыжи:','tr':':kayak:','ar':':تزلج:'},'\U0001F480':{'en':':skull:','status':fully_qualified,'E':0.6,'de':':totenkopf:','es':':calavera:','fr':':crâne:','ja':':ドクロ:','ko':':해골:','pt':':caveira:','it':':teschio:','fa':':جمجمه:','id':':tengkorak:','zh':':头骨:','ru':':череп:','tr':':kafatası:','ar':':جمجمة:'},'\U00002620\U0000FE0F':{'en':':skull_and_crossbones:','status':fully_qualified,'E':1,'variant':True,'de':':totenkopf_mit_gekreuzten_knochen:','es':':calavera_y_huesos_cruzados:','fr':':tête_de_mort:','ja':':ドクロと骨:','ko':':해골과_뼈다귀:','pt':':caveira_e_ossos_cruzados:','it':':teschio_con_ossa_incrociate:','fa':':جمجمه_و_استخوان_صلیبی:','id':':tengkorak_dan_tulang:','zh':':骷髅:','ru':':череп_и_кости:','tr':':kuru_kafa:','ar':':جمجمة_بعظمتين_متصالبتين:'},'\U00002620':{'en':':skull_and_crossbones:','status':unqualified,'E':1,'variant':True,'de':':totenkopf_mit_gekreuzten_knochen:','es':':calavera_y_huesos_cruzados:','fr':':tête_de_mort:','ja':':ドクロと骨:','ko':':해골과_뼈다귀:','pt':':caveira_e_ossos_cruzados:','it':':teschio_con_ossa_incrociate:','fa':':جمجمه_و_استخوان_صلیبی:','id':':tengkorak_dan_tulang:','zh':':骷髅:','ru':':череп_и_кости:','tr':':kuru_kafa:','ar':':جمجمة_بعظمتين_متصالبتين:'},'\U0001F9A8':{'en':':skunk:','status':fully_qualified,'E':12,'de':':stinktier:','es':':mofeta:','fr':':mouffette:','ja':':スカンク:','ko':':스컹크:','pt':':gambá:','it':':puzzola:','fa':':راسو:','id':':sigung:','zh':':臭鼬:','ru':':скунс:','tr':':kokarca:','ar':':ظربان:'},'\U0001F6F7':{'en':':sled:','status':fully_qualified,'E':5,'de':':schlitten:','es':':trineo:','fr':':luge:','ja':':そり競技:','ko':':썰매:','pt':':trenó:','it':':slitta:','fa':':سورتمه:','id':':kereta_luncur:','zh':':雪橇:','ru':':санки:','tr':':kızak:','ar':':مزلجة:'},'\U0001F634':{'en':':sleeping_face:','status':fully_qualified,'E':1,'alias':[':sleeping:'],'de':':schlafendes_gesicht:','es':':cara_durmiendo:','fr':':visage_somnolent:','ja':':寝る:','ko':':졸고_있는_얼굴:','pt':':rosto_dormindo:','it':':faccina_che_dorme:','fa':':خوابیده:','id':':wajah_tidur:','zh':':睡着了:','ru':':спит:','tr':':uyuyan_yüz:','ar':':وجه_نائم:'},'\U0001F62A':{'en':':sleepy_face:','status':fully_qualified,'E':0.6,'alias':[':sleepy:'],'de':':schläfriges_gesicht:','es':':cara_de_sueño:','fr':':visage_endormi:','ja':':眠い:','ko':':졸린_얼굴:','pt':':rosto_sonolento:','it':':faccina_assonnata:','fa':':خواب_آلود:','id':':wajah_mengantuk:','zh':':困:','ru':':хочет_спать:','tr':':uykulu_yüz:','ar':':وجه_ناعس:'},'\U0001F641':{'en':':slightly_frowning_face:','status':fully_qualified,'E':1,'de':':betrübtes_gesicht:','es':':cara_con_el_ceño_ligeramente_fruncido:','fr':':visage_légèrement_mécontent:','ja':':少し困った顔:','ko':':살짝_찡그린_얼굴:','pt':':rosto_meio_triste:','it':':faccina_leggermente_imbronciata:','fa':':دلخور:','id':':wajah_agak_cemberut:','zh':':微微不满:','ru':':немного_грустит:','tr':':biraz_asık_surat:','ar':':وجه_عابس_قليلا:'},'\U0001F642':{'en':':slightly_smiling_face:','status':fully_qualified,'E':1,'de':':leicht_lächelndes_gesicht:','es':':cara_sonriendo_ligeramente:','fr':':visage_avec_un_léger_sourire:','ja':':微笑む:','ko':':살짝_미소_짓는_얼굴:','pt':':rosto_levemente_sorridente:','it':':faccina_con_sorriso_accennato:','fa':':لبخند_ملایم:','id':':wajah_agak_tersenyum:','zh':':呵呵:','ru':':слегка_улыбается:','tr':':hafifçe_gülümseyen_yüz:','ar':':وجه_بابتسامة_بسيطة:'},'\U0001F3B0':{'en':':slot_machine:','status':fully_qualified,'E':0.6,'de':':spielautomat:','es':':máquina_tragaperras:','fr':':machine_à_sous:','ja':':スロットマシン:','ko':':슬롯_머신:','pt':':caça-níquel:','it':':slot_machine:','fa':':ماشین_پولی:','id':':mesin_slot:','zh':':老虎机:','ru':':игровой_автомат:','tr':':kumar_makinesi:','ar':':جهاز_قمار:'},'\U0001F9A5':{'en':':sloth:','status':fully_qualified,'E':12,'de':':faultier:','es':':perezoso:','fr':':paresseux:','ja':':ナマケモノ:','ko':':나무늘보:','pt':':preguiça:','it':':bradipo:','fa':':تنبل:','id':':kungkang:','zh':':树懒:','ru':':ленивец:','tr':':tembel_hayvan:','ar':':كسلان:'},'\U0001F6E9\U0000FE0F':{'en':':small_airplane:','status':fully_qualified,'E':0.7,'variant':True,'de':':kleines_flugzeug:','es':':avioneta:','fr':':petit_avion:','ja':':小型飛行機:','ko':':경비행기:','pt':':avião_pequeno:','it':':aereo_turistico:','fa':':هواپیمای_کوچک:','id':':pesawat_kecil:','zh':':小型飞机:','ru':':маленькии_самолет:','tr':':küçük_uçak:','ar':':طائرة_صغيرة:'},'\U0001F6E9':{'en':':small_airplane:','status':unqualified,'E':0.7,'variant':True,'de':':kleines_flugzeug:','es':':avioneta:','fr':':petit_avion:','ja':':小型飛行機:','ko':':경비행기:','pt':':avião_pequeno:','it':':aereo_turistico:','fa':':هواپیمای_کوچک:','id':':pesawat_kecil:','zh':':小型飞机:','ru':':маленький_самолет:','tr':':küçük_uçak:','ar':':طائرة_صغيرة:'},'\U0001F539':{'en':':small_blue_diamond:','status':fully_qualified,'E':0.6,'de':':kleine_blaue_raute:','es':':rombo_azul_pequeño:','fr':':petit_losange_bleu:','ja':':小さい青のひし形:','ko':':작은_파란색_마름모:','pt':':losango_azul_pequeno:','it':':rombo_blu_piccolo:','fa':':لوزی_آبی_کوچک:','id':':wajik_biru_kecil:','zh':':蓝色小菱形:','ru':':небольшой_голубой_ромб:','tr':':küçük_mavi_karo:','ar':':ماسة_زرقاء_صغيرة:'},'\U0001F538':{'en':':small_orange_diamond:','status':fully_qualified,'E':0.6,'de':':kleine_orangefarbene_raute:','es':':rombo_naranja_pequeño:','fr':':petit_losange_orange:','ja':':小さいオレンジのひし形:','ko':':작은_주황색_마름모:','pt':':losango_laranja_pequeno:','it':':rombo_arancione_piccolo:','fa':':لوزی_نارنجی_کوچک:','id':':wajik_oranye_kecil:','zh':':橙色小菱形:','ru':':небольшой_оранжевый_ромб:','tr':':küçük_turuncu_karo:','ar':':ماسة_برتقالي_صغيرة:'},'\U0001F63B':{'en':':smiling_cat_with_heart-eyes:','status':fully_qualified,'E':0.6,'alias':[':heart_eyes_cat:', ':smiling_cat_with_heart_eyes:'],'de':':lachende_katze_mit_herzen_als_augen:','es':':gato_sonriendo_con_ojos_de_corazón:','fr':':chat_souriant_aux_yeux_en_cœurs:','ja':':目がハートの猫:','ko':':하트_눈_고양이_얼굴:','pt':':rosto_de_gato_sorridente_com_olhos_de_coração:','it':':gatto_innamorato:','fa':':گربهٔ_خندان_عاشق:','id':':wajah_kucing_gembira_bermata_hati:','zh':':花痴的猫:','ru':':влюбленный_кот:','tr':':kalp_gözlü_gülümseyen_kedi:','ar':':وجه_قطة_مبتسم_بعينين_على_شكل_قلب:'},'\U0000263A\U0000FE0F':{'en':':smiling_face:','status':fully_qualified,'E':0.6,'alias':[':relaxed:'],'variant':True,'de':':lächelndes_gesicht:','es':':cara_sonriente:','fr':':visage_souriant:','ja':':スマイリー:','ko':':웃는_얼굴:','pt':':rosto_sorridente:','it':':faccina_sorridente:','fa':':لبخند:','id':':wajah_tersenyum:','zh':':微笑:','ru':':улыбается:','tr':':gülen_yüz:','ar':':وجه_مبتسم:'},'\U0000263A':{'en':':smiling_face:','status':unqualified,'E':0.6,'alias':[':relaxed:'],'variant':True,'de':':lächelndes_gesicht:','es':':cara_sonriente:','fr':':visage_souriant:','ja':':スマイリー:','ko':':웃는_얼굴:','pt':':rosto_sorridente:','it':':faccina_sorridente:','fa':':لبخند:','id':':wajah_tersenyum:','zh':':微笑:','ru':':улыбается:','tr':':gülen_yüz:','ar':':وجه_مبتسم:'},'\U0001F607':{'en':':smiling_face_with_halo:','status':fully_qualified,'E':1,'alias':[':innocent:'],'de':':lächelndes_gesicht_mit_heiligenschein:','es':':cara_sonriendo_con_aureola:','fr':':visage_souriant_avec_auréole:','ja':':天使の輪がついた笑顔:','ko':':후광이_비치는_웃는_얼굴:','pt':':rosto_sorridente_com_auréola:','it':':faccina_con_sorriso_e_aureola:','fa':':هاله_دار:','id':':wajah_malaikat:','zh':':微笑天使:','ru':':с_нимбом:','tr':':ışık_halkalı_gülümseme:','ar':':وجه_مبتسم_بهالة:'},'\U0001F60D':{'en':':smiling_face_with_heart-eyes:','status':fully_qualified,'E':0.6,'alias':[':heart_eyes:', ':smiling_face_with_heart_eyes:'],'de':':lächelndes_gesicht_mit_herzförmigen_augen:','es':':cara_sonriendo_con_ojos_de_corazón:','fr':':visage_souriant_avec_yeux_en_forme_de_cœur:','ja':':目がハートの笑顔:','ko':':하트_눈_얼굴:','pt':':rosto_sorridente_com_olhos_de_coração:','it':':faccina_con_sorriso_e_occhi_a_cuore:','fa':':چشم_های_عاشق:','id':':wajah_tersenyum_lebar_bermata_hati:','zh':':花痴:','ru':':влюбленное_лицо:','tr':':kalp_gözlü_gülümseme:','ar':':وجه_مبتسم_مع_عينين_على_شكل_قلب:'},'\U0001F970':{'en':':smiling_face_with_hearts:','status':fully_qualified,'E':11,'alias':[':smiling_face_with_three_hearts:'],'de':':lächelndes_gesicht_mit_herzen:','es':':cara_sonriendo_con_corazones:','fr':':visage_souriant_avec_cœurs:','ja':':ハートの笑顔:','ko':':하트와_함께_웃는_얼굴:','pt':':rosto_sorridente_com_3_corações:','it':':faccina_con_cuoricini:','fa':':صورت_خندان_و_عاشق:','id':':wajah_tersenyum_dengan_3_hati:','zh':':喜笑颜开:','ru':':улыбающееся_лицо_с_сердечками:','tr':':3_kalpli_gülümseyen_yüz:','ar':':وجه_مبتسم_مع_ثلاثة_قلوب:'},'\U0001F608':{'en':':smiling_face_with_horns:','status':fully_qualified,'E':1,'alias':[':smiling_imp:'],'de':':grinsendes_gesicht_mit_hörnern:','es':':cara_sonriendo_con_cuernos:','fr':':visage_souriant_avec_des_cornes:','ja':':笑った悪魔:','ko':':머리에_뿔이_달린_웃는_얼굴:','pt':':rosto_sorridente_com_chifres:','it':':faccina_con_sorriso_e_corna:','fa':':صورت_لبخندزنان_با_دوشاخ:','id':':wajah_tersenyum_bertanduk:','zh':':恶魔微笑:','ru':':улыбающийся_чертенок:','tr':':boynuzlu_gülen_yüz:','ar':':وجه_مبتسم_بقرنين:'},'\U0001F917':{'en':':smiling_face_with_open_hands:','status':fully_qualified,'E':1,'alias':[':hugging_face:', ':hugs:'],'de':':gesicht_mit_umarmenden_händen:','es':':cara_con_manos_abrazando:','fr':':visage_qui_fait_un_câlin:','ja':':ハグ:','ko':':포옹하고_있는_얼굴:','pt':':rosto_abraçando:','it':':faccina_che_abbraccia:','fa':':بغل_کردن:','id':':wajah_memeluk:','zh':':抱抱:','ru':':обнимает:','tr':':kucaklayarak_gülümseme:','ar':':وجه_يعانق_بابتسامة:'},'\U0001F60A':{'en':':smiling_face_with_smiling_eyes:','status':fully_qualified,'E':0.6,'alias':[':blush:'],'de':':lächelndes_gesicht_mit_lachenden_augen:','es':':cara_feliz_con_ojos_sonrientes:','fr':':visage_souriant_avec_yeux_rieurs:','ja':':にこにこ:','ko':':미소_짓는_눈으로_살짝_웃는_얼굴:','pt':':rosto_sorridente_com_olhos_sorridentes:','it':':faccina_con_occhi_sorridenti:','fa':':لبخند_خجالتی:','id':':wajah_tersenyum_dengan_mata_bahagia:','zh':':羞涩微笑:','ru':':довольно_улыбается:','tr':':ağzı_açık_gülme:','ar':':وجه_باسم_بعينين_باسمتين:'},'\U0001F60E':{'en':':smiling_face_with_sunglasses:','status':fully_qualified,'E':1,'alias':[':sunglasses:'],'de':':lächelndes_gesicht_mit_sonnenbrille:','es':':cara_sonriendo_con_gafas_de_sol:','fr':':visage_avec_lunettes_de_soleil:','ja':':サングラスで笑顔:','ko':':선글라스_낀_얼굴:','pt':':rosto_sorridente_com_óculos_escuros:','it':':faccina_con_sorriso_e_occhiali_da_sole:','fa':':باحال:','id':':wajah_berkacamata_hitam:','zh':':墨镜笑脸:','ru':':лицо_в_темных_очках:','tr':':güneş_gözlüklü_gülümseme:','ar':':وجه_مبتسم_بنظارة_شمس:'},'\U0001F972':{'en':':smiling_face_with_tear:','status':fully_qualified,'E':13,'de':':lachendes_gesicht_mit_träne:','es':':cara_sonriente_con_lágrima:','fr':':visage_souriant_avec_une_larme:','ja':':嬉し涙の顔:','ko':':눈물_흘리며_웃는_얼굴:','pt':':rosto_sorridente_com_lágrima:','it':':faccina_sorridente_con_lacrima:','fa':':چهرهٔ_خندان_با_اشک:','id':':wajah_tersenyum_dengan_air_mata:','zh':':含泪的笑脸:','ru':':улыбающееся_лицо_со_слезой:','tr':':gözü_yaşlı_gülümseyen_yüz:','ar':':وجه_مبتسم_مع_دمعة:'},'\U0001F60F':{'en':':smirking_face:','status':fully_qualified,'E':0.6,'alias':[':smirk:'],'de':':süffisant_lächelndes_gesicht:','es':':cara_sonriendo_con_superioridad:','fr':':visage_avec_un_sourire_malin:','ja':':薄笑いをする顔:','ko':':히죽거리는_얼굴:','pt':':rosto_com_sorriso_maroto:','it':':faccina_con_sorrisetto:','fa':':پوزخند:','id':':wajah_tersenyum_sombong:','zh':':得意:','ru':':ухмыляется:','tr':':imalı_gülümseme:','ar':':وجه_بابتسامة_متكلفة:'},'\U0001F40C':{'en':':snail:','status':fully_qualified,'E':0.6,'de':':schnecke:','es':':caracol:','fr':':escargot:','ja':':かたつむり:','ko':':달팽이:','pt':':caracol:','it':':lumaca:','fa':':حلزون:','id':':siput:','zh':':蜗牛:','ru':':улитка:','tr':':salyangoz:','ar':':حلزون:'},'\U0001F40D':{'en':':snake:','status':fully_qualified,'E':0.6,'de':':schlange:','es':':serpiente:','fr':':serpent:','ja':':ヘビ:','ko':':뱀:','pt':':cobra:','it':':serpente:','fa':':مار:','id':':ular:','zh':':蛇:','ru':':змея:','tr':':yılan:','ar':':ثعبان:'},'\U0001F927':{'en':':sneezing_face:','status':fully_qualified,'E':3,'de':':niesendes_gesicht:','es':':cara_estornudando:','fr':':visage_qui_éternue:','ja':':くしゃみする顔:','ko':':재채기하는_얼굴:','pt':':rosto_espirrando:','it':':faccina_che_starnutisce:','fa':':عطسه:','id':':bersin:','zh':':打喷嚏:','ru':':чихает:','tr':':hapşıran_yüz:','ar':':وجه_يعطس:'},'\U0001F3D4\U0000FE0F':{'en':':snow-capped_mountain:','status':fully_qualified,'E':0.7,'alias':[':mountain_snow:', ':snow_capped_mountain:'],'variant':True,'de':':schneebedeckter_berg:','es':':montaña_con_nieve:','fr':':montagne_enneigée:','ja':':雪山:','ko':':눈_덮인_산:','pt':':montanha_com_neve:','it':':montagna_innevata:','fa':':قله_برفی:','id':':gunung_dengan_puncak_bersalju:','zh':':雪山:','ru':':гора_со_снежнои_шапкои:','tr':':karla_kaplı_dağ:','ar':':جبل_بقمة_ثلجية:'},'\U0001F3D4':{'en':':snow-capped_mountain:','status':unqualified,'E':0.7,'alias':[':mountain_snow:', ':snow_capped_mountain:'],'variant':True,'de':':schneebedeckter_berg:','es':':montaña_con_nieve:','fr':':montagne_enneigée:','ja':':雪山:','ko':':눈_덮인_산:','pt':':montanha_com_neve:','it':':montagna_innevata:','fa':':قلهٔ_برفی:','id':':gunung_dengan_puncak_bersalju:','zh':':雪山:','ru':':гора_со_снежной_шапкой:','tr':':karla_kaplı_dağ:','ar':':جبل_بقمة_ثلجية:'},'\U0001F3C2':{'en':':snowboarder:','status':fully_qualified,'E':0.6,'variant':True,'de':':snowboarder(in):','es':':practicante_de_snowboard:','fr':':snowboardeur:','ja':':スノーボーダー:','ko':':스노보드_타는_사람:','pt':':praticante_de_snowboard:','it':':persona_sullo_snowboard:','fa':':اسنوبرد:','id':':peselancar_salju:','zh':':滑雪板:','ru':':сноубордист:','tr':':snowbordçu:','ar':':متزلج:'},'\U0001F3C2\U0001F3FF':{'en':':snowboarder_dark_skin_tone:','status':fully_qualified,'E':1,'de':':snowboarder(in)_dunkle_hautfarbe:','es':':practicante_de_snowboard_tono_de_piel_oscuro:','fr':':snowboardeur_peau_foncée:','ja':':スノーボーダー_濃い肌色:','ko':':스노보드_타는_사람_검은색_피부:','pt':':praticante_de_snowboard_pele_escura:','it':':persona_sullo_snowboard_carnagione_scura:','fa':':اسنوبرد_پوست_آبنوسی:','id':':peselancar_salju_warna_kulit_gelap:','zh':':滑雪板_较深肤色:','ru':':сноубордист_очень_темный_тон_кожи:','tr':':snowbordçu_koyu_cilt_tonu:','ar':':متزلج_بشرة_بلون_غامق:'},'\U0001F3C2\U0001F3FB':{'en':':snowboarder_light_skin_tone:','status':fully_qualified,'E':1,'de':':snowboarder(in)_helle_hautfarbe:','es':':practicante_de_snowboard_tono_de_piel_claro:','fr':':snowboardeur_peau_claire:','ja':':スノーボーダー_薄い肌色:','ko':':스노보드_타는_사람_하얀_피부:','pt':':praticante_de_snowboard_pele_clara:','it':':persona_sullo_snowboard_carnagione_chiara:','fa':':اسنوبرد_پوست_سفید:','id':':peselancar_salju_warna_kulit_cerah:','zh':':滑雪板_较浅肤色:','ru':':сноубордист_очень_светлый_тон_кожи:','tr':':snowbordçu_açık_cilt_tonu:','ar':':متزلج_بشرة_بلون_فاتح:'},'\U0001F3C2\U0001F3FE':{'en':':snowboarder_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':snowboarder(in)_mitteldunkle_hautfarbe:','es':':practicante_de_snowboard_tono_de_piel_oscuro_medio:','fr':':snowboardeur_peau_mate:','ja':':スノーボーダー_やや濃い肌色:','ko':':스노보드_타는_사람_진한_갈색_피부:','pt':':praticante_de_snowboard_pele_morena_escura:','it':':persona_sullo_snowboard_carnagione_abbastanza_scura:','fa':':اسنوبرد_پوست_گندمی:','id':':peselancar_salju_warna_kulit_gelap-sedang:','zh':':滑雪板_中等深肤色:','ru':':сноубордист_темный_тон_кожи:','tr':':snowbordçu_orta_koyu_cilt_tonu:','ar':':متزلج_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3C2\U0001F3FC':{'en':':snowboarder_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':snowboarder(in)_mittelhelle_hautfarbe:','es':':practicante_de_snowboard_tono_de_piel_claro_medio:','fr':':snowboardeur_peau_moyennement_claire:','ja':':スノーボーダー_やや薄い肌色:','ko':':스노보드_타는_사람_연한_갈색_피부:','pt':':praticante_de_snowboard_pele_morena_clara:','it':':persona_sullo_snowboard_carnagione_abbastanza_chiara:','fa':':اسنوبرد_پوست_روشن:','id':':peselancar_salju_warna_kulit_cerah-sedang:','zh':':滑雪板_中等浅肤色:','ru':':сноубордист_светлый_тон_кожи:','tr':':snowbordçu_orta_açık_cilt_tonu:','ar':':متزلج_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3C2\U0001F3FD':{'en':':snowboarder_medium_skin_tone:','status':fully_qualified,'E':1,'de':':snowboarder(in)_mittlere_hautfarbe:','es':':practicante_de_snowboard_tono_de_piel_medio:','fr':':snowboardeur_peau_légèrement_mate:','ja':':スノーボーダー_中間の肌色:','ko':':스노보드_타는_사람_갈색_피부:','pt':':praticante_de_snowboard_pele_morena:','it':':persona_sullo_snowboard_carnagione_olivastra:','fa':':اسنوبرد_پوست_طلایی:','id':':peselancar_salju_warna_kulit_sedang:','zh':':滑雪板_中等肤色:','ru':':сноубордист_средний_тон_кожи:','tr':':snowbordçu_orta_cilt_tonu:','ar':':متزلج_بشرة_بلون_معتدل:'},'\U00002744\U0000FE0F':{'en':':snowflake:','status':fully_qualified,'E':0.6,'variant':True,'de':':schneeflocke:','es':':copo_de_nieve:','fr':':flocon:','ja':':雪の結晶:','ko':':눈송이:','pt':':floco_de_neve:','it':':fiocco_di_neve:','fa':':برف_دانه:','id':':serpihan_salju:','zh':':雪花:','ru':':снежинка:','tr':':kar_tanesi:','ar':':كتلة_ثلج:'},'\U00002744':{'en':':snowflake:','status':unqualified,'E':0.6,'variant':True,'de':':schneeflocke:','es':':copo_de_nieve:','fr':':flocon:','ja':':雪の結晶:','ko':':눈송이:','pt':':floco_de_neve:','it':':fiocco_di_neve:','fa':':برف_دانه:','id':':serpihan_salju:','zh':':雪花:','ru':':снежинка:','tr':':kar_tanesi:','ar':':كتلة_ثلج:'},'\U00002603\U0000FE0F':{'en':':snowman:','status':fully_qualified,'E':0.7,'alias':[':snowman_with_snow:'],'variant':True,'de':':schneemann_im_schnee:','es':':muñeco_de_nieve_con_nieve:','fr':':bonhomme_de_neige:','ja':':雪だるまと雪:','ko':':눈사람:','pt':':boneco_de_neve:','it':':pupazzo_di_neve:','fa':':آدم_برفی:','id':':boneka_salju_dengan_hujan_salju:','zh':':雪与雪人:','ru':':снеговик_под_снегом:','tr':':kardan_adam:','ar':':رجل_ثلج:'},'\U00002603':{'en':':snowman:','status':unqualified,'E':0.7,'alias':[':snowman_with_snow:'],'variant':True,'de':':schneemann_im_schnee:','es':':muñeco_de_nieve_con_nieve:','fr':':bonhomme_de_neige:','ja':':雪だるまと雪:','ko':':눈사람:','pt':':boneco_de_neve:','it':':pupazzo_di_neve:','fa':':آدم_برفی:','id':':boneka_salju_dengan_hujan_salju:','zh':':雪与雪人:','ru':':снеговик_под_снегом:','tr':':kardan_adam:','ar':':رجل_ثلج:'},'\U000026C4':{'en':':snowman_without_snow:','status':fully_qualified,'E':0.6,'alias':[':snowman:'],'variant':True,'de':':schneemann_ohne_schneeflocken:','es':':muñeco_de_nieve:','fr':':bonhomme_de_neige_sans_neige:','ja':':雪だるま:','ko':':스노우맨:','pt':':boneco_de_neve_sem_neve:','it':':pupazzo_di_neve_senza_neve:','fa':':آدم_برفی_بدون_برف:','id':':boneka_salju_tanpa_salju:','zh':':雪人:','ru':':снеговик:','tr':':kar_yağmadan_kardan_adam:','ar':':رجل_ثلج_بدون_ثلج_متساقط:'},'\U0001F9FC':{'en':':soap:','status':fully_qualified,'E':11,'de':':seife:','es':':jabón:','fr':':savon:','ja':':石けん:','ko':':비누:','pt':':sabonete:','it':':sapone:','fa':':صابون:','id':':sabun:','zh':':皂:','ru':':мыло:','tr':':sabun:','ar':':صابون:'},'\U000026BD':{'en':':soccer_ball:','status':fully_qualified,'E':0.6,'alias':[':soccer:'],'variant':True,'de':':fußball:','es':':balón_de_fútbol:','fr':':ballon_de_football:','ja':':サッカー:','ko':':축구공:','pt':':bola_de_futebol:','it':':pallone_da_calcio:','fa':':توپ_فوتبال:','id':':bola_sepak:','zh':':足球:','ru':':футбол:','tr':':futbol_topu:','ar':':كرة_قدم:'},'\U0001F9E6':{'en':':socks:','status':fully_qualified,'E':5,'de':':socken:','es':':calcetines:','fr':':chaussettes:','ja':':ソックス:','ko':':양말:','pt':':meias:','it':':calzini:','fa':':جوراب:','id':':kaus_kaki:','zh':':袜子:','ru':':носки:','tr':':çorap:','ar':':جورب:'},'\U0001F366':{'en':':soft_ice_cream:','status':fully_qualified,'E':0.6,'alias':[':icecream:'],'de':':softeis:','es':':cucurucho_de_helado:','fr':':glace_italienne:','ja':':ソフトクリーム:','ko':':소프트_아이스크림:','pt':':sorvete_italiano:','it':':cono_gelato:','fa':':بستنی_قیفی:','id':':es_krim_lembut:','zh':':圆筒冰激凌:','ru':':мороженое_в_стаканчике:','tr':':yumuşak_dondurma:','ar':':آيس_كريم_مخفف:'},'\U0001F94E':{'en':':softball:','status':fully_qualified,'E':11,'de':':softball:','es':':pelota_de_softball:','fr':':softball:','ja':':ソフトボール:','ko':':소프트볼:','pt':':softbol:','it':':palla_da_softball:','fa':':سافت_بال:','id':':sofbol:','zh':':垒球:','ru':':софтбол:','tr':':softball:','ar':':كرة_لينة:'},'\U00002660\U0000FE0F':{'en':':spade_suit:','status':fully_qualified,'E':0.6,'alias':[':spades:'],'variant':True,'de':':pik:','es':':palo_de_picas:','fr':':pique:','ja':':スペード:','ko':':스페이드:','pt':':naipe_de_espadas:','it':':picche:','fa':':خال_پیک:','id':':sekop:','zh':':黑桃:','ru':':пики:','tr':':maça:','ar':':ورقة_البستوني:'},'\U00002660':{'en':':spade_suit:','status':unqualified,'E':0.6,'alias':[':spades:'],'variant':True,'de':':pik:','es':':palo_de_picas:','fr':':pique:','ja':':スペード:','ko':':스페이드:','pt':':naipe_de_espadas:','it':':picche:','fa':':خال_پیک:','id':':sekop:','zh':':黑桃:','ru':':пики:','tr':':maça:','ar':':ورقة_البستوني:'},'\U0001F35D':{'en':':spaghetti:','status':fully_qualified,'E':0.6,'de':':spaghetti:','es':':espagueti:','fr':':spaghetti:','ja':':スパゲッティ:','ko':':스파게티:','pt':':espaguete:','it':':spaghetti:','fa':':اسپاگتی:','id':':spageti:','zh':':意粉:','ru':':спагетти:','tr':':spagetti:','ar':':سباغتي:'},'\U00002747\U0000FE0F':{'en':':sparkle:','status':fully_qualified,'E':0.6,'variant':True,'de':':funkeln:','es':':chispa:','fr':':éclat:','ja':':スパークル:','ko':':반짝임:','pt':':faísca:','it':':scintilla_stilizzata:','fa':':جرقه:','id':':kilauan:','zh':':火花:','ru':':искра:','tr':':parlama:','ar':':ومضة:'},'\U00002747':{'en':':sparkle:','status':unqualified,'E':0.6,'variant':True,'de':':funkeln:','es':':chispa:','fr':':éclat:','ja':':スパークル:','ko':':반짝임:','pt':':faísca:','it':':scintilla_stilizzata:','fa':':جرقه:','id':':kilauan:','zh':':火花:','ru':':искра:','tr':':parlama:','ar':':ومضة:'},'\U0001F387':{'en':':sparkler:','status':fully_qualified,'E':0.6,'de':':wunderkerze:','es':':bengala:','fr':':cierge_magique:','ja':':線香花火:','ko':':불꽃:','pt':':vela_estrela:','it':':stellina_scintillante:','fa':':فشفشه:','id':':kembang_api_batang:','zh':':烟花:','ru':':бенгальский_огонь:','tr':':fişek:','ar':':ماسة_نارية:'},'\U00002728':{'en':':sparkles:','status':fully_qualified,'E':0.6,'variant':True,'de':':funkelnde_sterne:','es':':chispas:','fr':':étincelles:','ja':':きらきら:','ko':':블링블링:','pt':':brilhos:','it':':stelline:','fa':':درخشش:','id':':berkilau:','zh':':闪亮:','ru':':блестки:','tr':':parıltılar:','ar':':ومضات:'},'\U0001F496':{'en':':sparkling_heart:','status':fully_qualified,'E':0.6,'de':':funkelndes_herz:','es':':corazón_brillante:','fr':':cœur_étincelant:','ja':':きらきらハート:','ko':':빛나는_하트:','pt':':coração_brilhante:','it':':cuore_che_luccica:','fa':':قلب_درخشان:','id':':hati_bersinar:','zh':':闪亮的心:','ru':':сверкающее_сердце:','tr':':pırıldayan_kalp:','ar':':قلب_لامع:'},'\U0001F64A':{'en':':speak-no-evil_monkey:','status':fully_qualified,'E':0.6,'alias':[':speak_no_evil:', ':speak_no_evil_monkey:'],'de':':sich_den_mund_zuhaltendes_affengesicht:','es':':mono_con_la_boca_tapada:','fr':':singe_ne_rien_dire:','ja':':言わざる:','ko':':입을_막고_있는_원숭이:','pt':':macaco_que_não_fala_nada:','it':':non_parlo:','fa':':میمون_شرنگو:','id':':tidak_bicara:','zh':':非礼勿言:','ru':':ничего_никому_не_скажу:','tr':':bilmiyorum:','ar':':قرد_يحجب_الكلام:'},'\U0001F50A':{'en':':speaker_high_volume:','status':fully_qualified,'E':0.6,'alias':[':loud_sound:'],'de':':lautsprecher_mit_hoher_lautstärke:','es':':altavoz_a_volumen_alto:','fr':':volume_des_enceintes_élevé:','ja':':スピーカー音量大:','ko':':스피커_소리_크게:','pt':':alto-falante_com_volume_alto:','it':':altoparlante_a_volume_alto:','fa':':بلندگو_با_صدای_بلند:','id':':volume_speaker_tinggi:','zh':':高音量的扬声器:','ru':':высокая_громкость:','tr':':yüksek_ses:','ar':':صوت_سماعة_مرتفع:'},'\U0001F508':{'en':':speaker_low_volume:','status':fully_qualified,'E':0.7,'alias':[':speaker:'],'variant':True,'de':':lautsprecher_mit_geringer_lautstärke:','es':':altavoz_a_volumen_bajo:','fr':':volume_des_enceintes_faible:','ja':':スピーカー:','ko':':스피커:','pt':':alto-falante_com_volume_baixo:','it':':altoparlante_a_volume_basso:','fa':':بلندگو_با_صدای_کم:','id':':volume_speaker_rendah:','zh':':低音量的扬声器:','ru':':низкая_громкость:','tr':':kısık_ses:','ar':':صوت_سماعة_منخفض:'},'\U0001F509':{'en':':speaker_medium_volume:','status':fully_qualified,'E':1,'alias':[':sound:'],'de':':lautsprecher_mit_mittlerer_lautstärke:','es':':altavoz_a_volumen_medio:','fr':':volume_des_enceintes_moyen:','ja':':スピーカー音量小:','ko':':스피커_소리_작게:','pt':':alto-falante_com_volume_médio:','it':':altoparlante_a_volume_intermedio:','fa':':بلندگو_با_صدای_متعادل:','id':':volume_speaker_medium:','zh':':中等音量的扬声器:','ru':':средняя_громкость:','tr':':ses_açık:','ar':':صوت_معتدل:'},'\U0001F5E3\U0000FE0F':{'en':':speaking_head:','status':fully_qualified,'E':0.7,'alias':[':speaking_head_in_silhouette:'],'variant':True,'de':':sprechender_kopf:','es':':cabeza_parlante:','fr':':tête_qui_parle:','ja':':話す人のシルエット:','ko':':말하고_있는_옆얼굴:','pt':':silhueta_falando:','it':':persona_che_parla:','fa':':درحال_صحبت_کردن:','id':':kepala_berbicara:','zh':':说话:','ru':':говорящая_голова:','tr':':konuşan_kişi:','ar':':رأس_تتحدث:'},'\U0001F5E3':{'en':':speaking_head:','status':unqualified,'E':0.7,'alias':[':speaking_head_in_silhouette:'],'variant':True,'de':':sprechender_kopf:','es':':cabeza_parlante:','fr':':tête_qui_parle:','ja':':話す人のシルエット:','ko':':말하고_있는_옆얼굴:','pt':':silhueta_falando:','it':':persona_che_parla:','fa':':درحال_صحبت_کردن:','id':':kepala_berbicara:','zh':':说话:','ru':':говорящая_голова:','tr':':konuşan_kişi:','ar':':رأس_تتحدث:'},'\U0001F4AC':{'en':':speech_balloon:','status':fully_qualified,'E':0.6,'de':':sprechblase_mit_drei_punkten:','es':':bocadillo_de_diálogo:','fr':':bulle_de_parole:','ja':':会話の吹き出し:','ko':':말풍선:','pt':':balão_de_diálogo:','it':':fumetto:','fa':':بالن_گفتار:','id':':balon_ucapan:','zh':':话语气泡:','ru':':облачко_с_многоточием:','tr':':konuşma_balonu:','ar':':فقاعة_كلام:'},'\U0001F6A4':{'en':':speedboat:','status':fully_qualified,'E':0.6,'de':':schnellboot:','es':':lancha_motora:','fr':':hors-bord:','ja':':スピードボート:','ko':':쾌속정:','pt':':lancha:','it':':motoscafo:','fa':':قایق_تندرو:','id':':perahu_cepat:','zh':':快艇:','ru':':катер:','tr':':sürat_teknesi:','ar':':قارب_سريع:'},'\U0001F577\U0000FE0F':{'en':':spider:','status':fully_qualified,'E':0.7,'variant':True,'de':':spinne:','es':':araña:','fr':':araignée:','ja':':クモ:','ko':':거미:','pt':':aranha:','it':':ragno:','fa':':عنکبوت:','id':':laba-laba:','zh':':蜘蛛:','ru':':паук:','tr':':örümcek:','ar':':عنكبوت:'},'\U0001F577':{'en':':spider:','status':unqualified,'E':0.7,'variant':True,'de':':spinne:','es':':araña:','fr':':araignée:','ja':':クモ:','ko':':거미:','pt':':aranha:','it':':ragno:','fa':':عنکبوت:','id':':laba-laba:','zh':':蜘蛛:','ru':':паук:','tr':':örümcek:','ar':':عنكبوت:'},'\U0001F578\U0000FE0F':{'en':':spider_web:','status':fully_qualified,'E':0.7,'variant':True,'de':':spinnennetz:','es':':tela_de_araña:','fr':':toile_d’araignée:','ja':':クモの巣:','ko':':거미줄:','pt':':teia_de_aranha:','it':':ragnatela:','fa':':تار_عنکبوت:','id':':jaring_laba-laba:','zh':':蜘蛛网:','ru':':паутина:','tr':':örümcek_ağı:','ar':':شبكة_عنكبوت:'},'\U0001F578':{'en':':spider_web:','status':unqualified,'E':0.7,'variant':True,'de':':spinnennetz:','es':':tela_de_araña:','fr':':toile_d’araignée:','ja':':クモの巣:','ko':':거미줄:','pt':':teia_de_aranha:','it':':ragnatela:','fa':':تار_عنکبوت:','id':':jaring_laba-laba:','zh':':蜘蛛网:','ru':':паутина:','tr':':örümcek_ağı:','ar':':شبكة_عنكبوت:'},'\U0001F5D3\U0000FE0F':{'en':':spiral_calendar:','status':fully_qualified,'E':0.7,'alias':[':spiral_calendar_pad:'],'variant':True,'de':':spiralkalender:','es':':calendario_de_espiral:','fr':':calendrier_à_spirale:','ja':':月めくりカレンダー:','ko':':스프링_달력:','pt':':calendário_espiral:','it':':calendario_a_spirale:','fa':':تقویم_سیمی:','id':':kalender_spiral:','zh':':线圈日历:','ru':':перекиднои_календарь:','tr':':spiral_takvim:','ar':':تقويم_حلزوني:'},'\U0001F5D3':{'en':':spiral_calendar:','status':unqualified,'E':0.7,'alias':[':spiral_calendar_pad:'],'variant':True,'de':':spiralkalender:','es':':calendario_de_espiral:','fr':':calendrier_à_spirale:','ja':':月めくりカレンダー:','ko':':스프링_달력:','pt':':calendário_espiral:','it':':calendario_a_spirale:','fa':':تقویم_سیمی:','id':':kalender_spiral:','zh':':线圈日历:','ru':':перекидной_календарь:','tr':':spiral_takvim:','ar':':تقويم_حلزوني:'},'\U0001F5D2\U0000FE0F':{'en':':spiral_notepad:','status':fully_qualified,'E':0.7,'alias':[':spiral_note_pad:'],'variant':True,'de':':notizblock:','es':':bloc_de_notas_de_espiral:','fr':':bloc-notes_à_spirale:','ja':':メモ帳:','ko':':스프링_노트:','pt':':bloco_espiral:','it':':blocco_per_appunti_con_spirale:','fa':':دفترچه_یادداشت_سیمی:','id':':catatan_spiral:','zh':':线圈本:','ru':':перекиднои_блокнот:','tr':':spiral_not_defteri:','ar':':ورقة_ملاحظات:'},'\U0001F5D2':{'en':':spiral_notepad:','status':unqualified,'E':0.7,'alias':[':spiral_note_pad:'],'variant':True,'de':':notizblock:','es':':bloc_de_notas_de_espiral:','fr':':bloc-notes_à_spirale:','ja':':メモ帳:','ko':':스프링_노트:','pt':':bloco_espiral:','it':':blocco_per_appunti_con_spirale:','fa':':دفترچهٔ_یادداشت_سیمی:','id':':catatan_spiral:','zh':':线圈本:','ru':':перекидной_блокнот:','tr':':spiral_not_defteri:','ar':':ورقة_ملاحظات:'},'\U0001F41A':{'en':':spiral_shell:','status':fully_qualified,'E':0.6,'alias':[':shell:'],'de':':schneckenhaus:','es':':caracola:','fr':':coquille_en_spirale:','ja':':巻き貝:','ko':':달팽이집:','pt':':caramujo:','it':':conchiglia:','fa':':صدف_مارپیچی:','id':':cangkang_spiral:','zh':':海螺:','ru':':раковина:','tr':':deniz_kabuğu:','ar':':صدفة_حلزونية:'},'\U0001F9FD':{'en':':sponge:','status':fully_qualified,'E':11,'de':':schwamm:','es':':esponja:','fr':':éponge:','ja':':スポンジ:','ko':':스펀지:','pt':':esponja:','it':':spugna:','fa':':لیف_حمام:','id':':spons:','zh':':海绵:','ru':':губка:','tr':':sünger:','ar':':اسفنجة:'},'\U0001F944':{'en':':spoon:','status':fully_qualified,'E':3,'de':':löffel:','es':':cuchara:','fr':':cuillère:','ja':':スプーン:','ko':':숟가락:','pt':':colher:','it':':cucchiaio:','fa':':قاشق:','id':':sendok:','zh':':匙:','ru':':ложка:','tr':':kaşık:','ar':':ملعقة:'},'\U0001F699':{'en':':sport_utility_vehicle:','status':fully_qualified,'E':0.6,'alias':[':blue_car:'],'de':':wohnmobil:','es':':vehículo_deportivo_utilitario:','fr':':véhicule_utilitaire_sport:','ja':':アールブイ車:','ko':':지프:','pt':':suv:','it':':suv:','fa':':خودروی_شاسی_بلند:','id':':kendaraan_rekreasi:','zh':':运动型多用途车:','ru':':джип:','tr':':cip:','ar':':مركبة_رياضية:'},'\U0001F3C5':{'en':':sports_medal:','status':fully_qualified,'E':1,'alias':[':medal_sports:'],'de':':sportmedaille:','es':':medalla_deportiva:','fr':':médaille_sportive:','ja':':メダル:','ko':':스포츠_메달:','pt':':medalha_esportiva:','it':':medaglia_sportiva:','fa':':مدال_ورزشی:','id':':medali_olahraga:','zh':':奖牌:','ru':':спортивная_медаль:','tr':':spor_madalyası:','ar':':ميدالية_رياضية:'},'\U0001F433':{'en':':spouting_whale:','status':fully_qualified,'E':0.6,'alias':[':whale:'],'de':':blasender_wal:','es':':ballena_soltando_un_chorro:','fr':':baleine_soufflant_par_son_évent:','ja':':潮吹きクジラ:','ko':':물_뿜는_고래:','pt':':baleia_esguichando_água:','it':':balena_che_spruzza_acqua:','fa':':نهنگ_در_حال_آب_فشانی:','id':':ikan_paus_menyembur:','zh':':喷水的鲸:','ru':':кит_с_фонтанчиком:','tr':':su_püskürten_balina:','ar':':حوت_بنافورة:'},'\U0001F991':{'en':':squid:','status':fully_qualified,'E':3,'de':':tintenfisch:','es':':calamar:','fr':':calamar:','ja':':イカ:','ko':':오징어:','pt':':lula:','it':':calamaro:','fa':':ماهی_مرکب:','id':':cumi-cumi:','zh':':乌贼:','ru':':кальмар:','tr':':mürekkep_balığı:','ar':':حبار:'},'\U0001F61D':{'en':':squinting_face_with_tongue:','status':fully_qualified,'E':0.6,'alias':[':stuck_out_tongue_closed_eyes:'],'de':':gesicht_mit_herausgestreckter_zunge_und_zusammengekniffenen_augen:','es':':cara_con_ojos_cerrados_y_lengua_fuera:','fr':':visage_qui_tire_la_langue_les_yeux_plissés:','ja':':目を閉じてべー:','ko':':눈을_감고_메롱하는_얼굴:','pt':':rosto_com_olhos_semicerrados_e_língua_para_fora:','it':':faccina_che_strizza_gli_occhi_e_mostra_la_lingua:','fa':':زبونک_با_چشم_بسته:','id':':wajah_menjulurkan_lidah_dan_memejamkan_mata:','zh':':眯眼吐舌:','ru':':морщится_и_показывает_язык:','tr':':dil_dışarıda_ve_gözler_kapalı:','ar':':وجه_بعينين_مغلقتين_ولسان_متدل:'},'\U0001F3DF\U0000FE0F':{'en':':stadium:','status':fully_qualified,'E':0.7,'variant':True,'de':':stadion:','es':':estadio:','fr':':stade:','ja':':競技場:','ko':':경기장:','pt':':estádio:','it':':stadio:','fa':':ورزشگاه:','id':':stadion:','zh':':体育馆:','ru':':стадион:','tr':':stadyum:','ar':':استاد:'},'\U0001F3DF':{'en':':stadium:','status':unqualified,'E':0.7,'variant':True,'de':':stadion:','es':':estadio:','fr':':stade:','ja':':競技場:','ko':':경기장:','pt':':estádio:','it':':stadio:','fa':':ورزشگاه:','id':':stadion:','zh':':体育馆:','ru':':стадион:','tr':':stadyum:','ar':':استاد:'},'\U00002B50':{'en':':star:','status':fully_qualified,'E':0.6,'variant':True,'de':':weißer_mittelgroßer_stern:','es':':estrella:','fr':':étoile:','ja':':スター:','ko':':별:','pt':':estrela_branca_média:','it':':stella:','fa':':ستاره_سفید:','id':':bintang_medium_putih:','zh':':星星:','ru':':желтая_звезда:','tr':':yıldız:','ar':':نجم_متوسط_أبيض:'},'\U0001F929':{'en':':star-struck:','status':fully_qualified,'E':5,'alias':[':star_struck:'],'de':':überwältigt:','es':':cara_sonriendo_con_estrellas:','fr':':visage_avec_des_étoiles_à_la_place_des_yeux:','ja':':目が星の笑顔:','ko':':반한_얼굴:','pt':':rosto_com_olhar_maravilhado:','it':':colpo_di_fulmine:','fa':':فوق_العاده:','id':':wajah_dengan_mata_berbentuk_bintang:','zh':':好崇拜哦:','ru':':звезды_в_глазах:','tr':':yıldız_çarpmış:','ar':':وجه_بعينين_على_شكل_نجمة:'},'\U0000262A\U0000FE0F':{'en':':star_and_crescent:','status':fully_qualified,'E':0.7,'variant':True,'de':':hilal_und_stern:','es':':media_luna_y_estrella:','fr':':lune_et_étoile:','ja':':星と三日月:','ko':':초승달과_별:','pt':':estrela_e_lua_crescente:','it':':mezzaluna_e_stella:','fa':':هلال_و_ستاره:','id':':bintang_dan_bulan:','zh':':星月:','ru':':звезда_и_полумесяц:','tr':':ay_yıldız:','ar':':هلال_ونجمة:'},'\U0000262A':{'en':':star_and_crescent:','status':unqualified,'E':0.7,'variant':True,'de':':hilal_und_stern:','es':':media_luna_y_estrella:','fr':':lune_et_étoile:','ja':':星と三日月:','ko':':초승달과_별:','pt':':estrela_e_lua_crescente:','it':':mezzaluna_e_stella:','fa':':هلال_و_ستاره:','id':':bintang_dan_bulan:','zh':':星月:','ru':':звезда_и_полумесяц:','tr':':ay_yıldız:','ar':':هلال_ونجمة:'},'\U00002721\U0000FE0F':{'en':':star_of_David:','status':fully_qualified,'E':0.7,'alias':[':star_of_david:'],'variant':True,'de':':davidstern:','es':':estrella_de_david:','fr':':étoile_de_david:','ja':':ダビデの星:','ko':':다비드별:','pt':':estrela_de_davi:','it':':stella_di_david:','fa':':ستاره_داود:','id':':bintang_daud:','zh':':六芒星:','ru':':звезда_давида:','tr':':davut_yıldızı:','ar':':نجمة_داوود:'},'\U00002721':{'en':':star_of_David:','status':unqualified,'E':0.7,'alias':[':star_of_david:'],'variant':True,'de':':davidstern:','es':':estrella_de_david:','fr':':étoile_de_david:','ja':':ダビデの星:','ko':':다비드별:','pt':':estrela_de_davi:','it':':stella_di_david:','fa':':ستارهٔ_داود:','id':':bintang_daud:','zh':':六芒星:','ru':':звезда_Давида:','tr':':davut_yıldızı:','ar':':نجمة_داوود:'},'\U0001F689':{'en':':station:','status':fully_qualified,'E':0.6,'de':':bahnhof:','es':':estación_de_tren:','fr':':gare:','ja':':駅:','ko':':기차역:','pt':':estação:','it':':stazione:','fa':':ایستگاه_قطار:','id':':stasiun:','zh':':车站:','ru':':железнодорожный_вокзал:','tr':':tren_istasyonu:','ar':':محطة_قطار:'},'\U0001F35C':{'en':':steaming_bowl:','status':fully_qualified,'E':0.6,'alias':[':ramen:'],'de':':schüssel_und_essstäbchen:','es':':tazón_de_fideos:','fr':':bol_fumant:','ja':':ラーメン:','ko':':국수:','pt':':lámen:','it':':ciotola_fumante:','fa':':کاسه_نودل:','id':':mangkuk_mengepul:','zh':':面条:','ru':':лапша:','tr':':erişte:','ar':':سلطانية_البخار:'},'\U0001FA7A':{'en':':stethoscope:','status':fully_qualified,'E':12,'de':':stethoskop:','es':':estetoscopio:','fr':':stéthoscope:','ja':':聴診器:','ko':':청진기:','pt':':estetoscópio:','it':':stetoscopio:','fa':':گوشی_پزشکی:','id':':stetoskop:','zh':':听诊器:','ru':':стетоскоп:','tr':':stetoskop:','ar':':سماعة_طبيب:'},'\U000023F9\U0000FE0F':{'en':':stop_button:','status':fully_qualified,'E':0.7,'alias':[':black_square_for_stop:'],'variant':True,'de':':stopp:','es':':detener:','fr':':bouton_stop:','ja':':停止ボタン:','ko':':정지_버튼:','pt':':botão_parar:','it':':pulsante_stop:','fa':':دکمه_توقف:','id':':tombol_berhenti:','zh':':停止按钮:','ru':':стоп:','tr':':durdurma_düğmesi:','ar':':زر_إيقاف:'},'\U000023F9':{'en':':stop_button:','status':unqualified,'E':0.7,'alias':[':black_square_for_stop:'],'variant':True,'de':':stopp:','es':':detener:','fr':':bouton_stop:','ja':':停止ボタン:','ko':':정지_버튼:','pt':':botão_parar:','it':':pulsante_stop:','fa':':دکمهٔ_توقف:','id':':tombol_berhenti:','zh':':停止按钮:','ru':':стоп:','tr':':durdurma_düğmesi:','ar':':زر_إيقاف:'},'\U0001F6D1':{'en':':stop_sign:','status':fully_qualified,'E':3,'de':':stoppschild:','es':':señal_de_stop:','fr':':stop:','ja':':止まれの標識:','ko':':멈춤_표시:','pt':':sinal_de_pare:','it':':segnale_di_stop:','fa':':علامت_ایست:','id':':rambu_stop:','zh':':停止标志:','ru':':знак_«стоп»:','tr':':dur_işareti:','ar':':علامة_توقف:'},'\U000023F1\U0000FE0F':{'en':':stopwatch:','status':fully_qualified,'E':1,'variant':True,'de':':stoppuhr:','es':':cronómetro:','fr':':chronomètre:','ja':':ストップウォッチ:','ko':':스톱워치:','pt':':cronômetro:','it':':cronometro:','fa':':کرنومتر:','id':':stopwatch:','zh':':秒表:','ru':':секундомер:','tr':':kronometre:','ar':':ساعة_توقف:'},'\U000023F1':{'en':':stopwatch:','status':unqualified,'E':1,'variant':True,'de':':stoppuhr:','es':':cronómetro:','fr':':chronomètre:','ja':':ストップウォッチ:','ko':':스톱워치:','pt':':cronômetro:','it':':cronometro:','fa':':کرنومتر:','id':':stopwatch:','zh':':秒表:','ru':':секундомер:','tr':':kronometre:','ar':':ساعة_توقف:'},'\U0001F4CF':{'en':':straight_ruler:','status':fully_qualified,'E':0.6,'de':':lineal:','es':':regla:','fr':':règle:','ja':':定規:','ko':':자:','pt':':régua_reta:','it':':righello:','fa':':خط_کش:','id':':penggaris_lurus:','zh':':直尺:','ru':':линейка:','tr':':düz_cetvel:','ar':':مسطرة:'},'\U0001F353':{'en':':strawberry:','status':fully_qualified,'E':0.6,'de':':erdbeere:','es':':fresa:','fr':':fraise:','ja':':いちご:','ko':':딸기:','pt':':morango:','it':':fragola:','fa':':توت_فرنگی:','id':':stroberi:','zh':':草莓:','ru':':клубника:','tr':':çilek:','ar':':فراولة:'},'\U0001F9D1\U0000200D\U0001F393':{'en':':student:','status':fully_qualified,'E':12.1,'de':':student(in):','es':':estudiante:','fr':':étudiant_(tous_genres):','ja':':学生:','ko':':학생:','pt':':aluno:','it':':studente:','fa':':دانش_آموز:','id':':murid:','zh':':学生:','ru':':учащийся:','tr':':öğrenci:','ar':':طالِب:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F393':{'en':':student_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':student(in)_dunkle_hautfarbe:','es':':estudiante_tono_de_piel_oscuro:','fr':':étudiant_(tous_genres)_peau_foncée:','ja':':学生_濃い肌色:','ko':':학생_검은색_피부:','pt':':aluno_pele_escura:','it':':studente_carnagione_scura:','fa':':دانش_آموز_پوست_آبنوسی:','id':':murid_warna_kulit_gelap:','zh':':学生_较深肤色:','ru':':учащийся_очень_темный_тон_кожи:','tr':':öğrenci_koyu_cilt_tonu:','ar':':طالِب_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F393':{'en':':student_light_skin_tone:','status':fully_qualified,'E':12.1,'de':':student(in)_helle_hautfarbe:','es':':estudiante_tono_de_piel_claro:','fr':':étudiant_(tous_genres)_peau_claire:','ja':':学生_薄い肌色:','ko':':학생_하얀_피부:','pt':':aluno_pele_clara:','it':':studente_carnagione_chiara:','fa':':دانش_آموز_پوست_سفید:','id':':murid_warna_kulit_cerah:','zh':':学生_较浅肤色:','ru':':учащийся_очень_светлый_тон_кожи:','tr':':öğrenci_açık_cilt_tonu:','ar':':طالِب_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F393':{'en':':student_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':student(in)_mitteldunkle_hautfarbe:','es':':estudiante_tono_de_piel_oscuro_medio:','fr':':étudiant_(tous_genres)_peau_mate:','ja':':学生_やや濃い肌色:','ko':':학생_진한_갈색_피부:','pt':':aluno_pele_morena_escura:','it':':studente_carnagione_abbastanza_scura:','fa':':دانش_آموز_پوست_گندمی:','id':':murid_warna_kulit_gelap-sedang:','zh':':学生_中等深肤色:','ru':':учащийся_темный_тон_кожи:','tr':':öğrenci_orta_koyu_cilt_tonu:','ar':':طالِب_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F393':{'en':':student_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':student(in)_mittelhelle_hautfarbe:','es':':estudiante_tono_de_piel_claro_medio:','fr':':étudiant_(tous_genres)_peau_moyennement_claire:','ja':':学生_やや薄い肌色:','ko':':학생_연한_갈색_피부:','pt':':aluno_pele_morena_clara:','it':':studente_carnagione_abbastanza_chiara:','fa':':دانش_آموز_پوست_روشن:','id':':murid_warna_kulit_cerah-sedang:','zh':':学生_中等浅肤色:','ru':':учащийся_светлый_тон_кожи:','tr':':öğrenci_orta_açık_cilt_tonu:','ar':':طالِب_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F393':{'en':':student_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':student(in)_mittlere_hautfarbe:','es':':estudiante_tono_de_piel_medio:','fr':':étudiant_(tous_genres)_peau_légèrement_mate:','ja':':学生_中間の肌色:','ko':':학생_갈색_피부:','pt':':aluno_pele_morena:','it':':studente_carnagione_olivastra:','fa':':دانش_آموز_پوست_طلایی:','id':':murid_warna_kulit_sedang:','zh':':学生_中等肤色:','ru':':учащийся_средний_тон_кожи:','tr':':öğrenci_orta_cilt_tonu:','ar':':طالِب_بشرة_بلون_معتدل:'},'\U0001F399\U0000FE0F':{'en':':studio_microphone:','status':fully_qualified,'E':0.7,'variant':True,'de':':studiomikrofon:','es':':micrófono_de_estudio:','fr':':micro_de_studio:','ja':':スタジオマイク:','ko':':스튜디오_마이크:','pt':':microfone_de_estúdio:','it':':microfono_radiofonico:','fa':':میکروفون_استودیویی:','id':':mikrofon_studio:','zh':':录音室麦克风:','ru':':студииныи_микрофон:','tr':':stüdyo_mikrofonu:','ar':':ميكروفون_استوديو:'},'\U0001F399':{'en':':studio_microphone:','status':unqualified,'E':0.7,'variant':True,'de':':studiomikrofon:','es':':micrófono_de_estudio:','fr':':micro_de_studio:','ja':':スタジオマイク:','ko':':스튜디오_마이크:','pt':':microfone_de_estúdio:','it':':microfono_radiofonico:','fa':':میکروفون_استودیویی:','id':':mikrofon_studio:','zh':':录音室麦克风:','ru':':студийный_микрофон:','tr':':stüdyo_mikrofonu:','ar':':ميكروفون_استوديو:'},'\U0001F959':{'en':':stuffed_flatbread:','status':fully_qualified,'E':3,'de':':döner:','es':':pan_relleno:','fr':':kebab:','ja':':ケバブサンド:','ko':':밀전병에_싼_요리:','pt':':pão_recheado:','it':':pita_farcita:','fa':':لقمه:','id':':roti_tawar_isi:','zh':':夹心饼:','ru':':шаурма:','tr':':dürüm:','ar':':خبز_مفرود_محشو:'},'\U00002600\U0000FE0F':{'en':':sun:','status':fully_qualified,'E':0.6,'alias':[':sunny:'],'variant':True,'de':':sonne:','es':':sol:','fr':':soleil:','ja':':太陽:','ko':':태양:','pt':':sol:','it':':sole:','fa':':خورشید:','id':':matahari:','zh':':太阳:','ru':':солнце:','tr':':güneş:','ar':':شمس:'},'\U00002600':{'en':':sun:','status':unqualified,'E':0.6,'alias':[':sunny:'],'variant':True,'de':':sonne:','es':':sol:','fr':':soleil:','ja':':太陽:','ko':':태양:','pt':':sol:','it':':sole:','fa':':خورشید:','id':':matahari:','zh':':太阳:','ru':':солнце:','tr':':güneş:','ar':':شمس:'},'\U000026C5':{'en':':sun_behind_cloud:','status':fully_qualified,'E':0.6,'alias':[':partly_sunny:'],'variant':True,'de':':sonne_hinter_wolke:','es':':sol_detrás_de_una_nube:','fr':':soleil_derrière_les_nuages:','ja':':曇り時々晴れ:','ko':':구름_뒤의_해:','pt':':sol_por_trás_das_nuvens:','it':':sole_coperto:','fa':':خورشید_پشت_ابر:','id':':matahari_di_balik_awan:','zh':':阴:','ru':':солнце_за_облаками:','tr':':bulutlar_ardındaki_güneş:','ar':':شمس_خلف_السحب:'},'\U0001F325\U0000FE0F':{'en':':sun_behind_large_cloud:','status':fully_qualified,'E':0.7,'alias':[':white_sun_behind_cloud:'],'variant':True,'de':':sonne_hinter_großer_wolke:','es':':sol_detrás_de_una_nube_grande:','fr':':soleil_derrière_un_gros_nuage:','ja':':曇り一時晴れ:','ko':':큰_구름_뒤의_태양:','pt':':nublado:','it':':molto_nuvoloso:','fa':':خورشید_پشت_ابر_بزرگ:','id':':matahari_di_balik_awan_besar:','zh':':多云:','ru':':солнце_за_облаком:','tr':':bulutta_kaybolan_güneş:','ar':':شمس_خلف_سحابة_كبيرة:'},'\U0001F325':{'en':':sun_behind_large_cloud:','status':unqualified,'E':0.7,'alias':[':white_sun_behind_cloud:'],'variant':True,'de':':sonne_hinter_großer_wolke:','es':':sol_detrás_de_una_nube_grande:','fr':':soleil_derrière_un_gros_nuage:','ja':':曇り一時晴れ:','ko':':큰_구름_뒤의_태양:','pt':':nublado:','it':':molto_nuvoloso:','fa':':خورشید_پشت_ابر_بزرگ:','id':':matahari_di_balik_awan_besar:','zh':':多云:','ru':':солнце_за_облаком:','tr':':bulutta_kaybolan_güneş:','ar':':شمس_خلف_سحابة_كبيرة:'},'\U0001F326\U0000FE0F':{'en':':sun_behind_rain_cloud:','status':fully_qualified,'E':0.7,'alias':[':white_sun_behind_cloud_with_rain:'],'variant':True,'de':':sonne_hinter_regenwolke:','es':':sol_detrás_de_una_nube_con_lluvia:','fr':':soleil_derrière_un_nuage_de_pluie:','ja':':雨時々晴れ:','ko':':비구름_뒤의_태양:','pt':':sol_com_chuva:','it':':sole_coperto_e_pioggia:','fa':':خورشید_پشت_ابر_بارانی:','id':':matahari_di_balik_awan_hujan:','zh':':晴转雨:','ru':':слепои_дождь:','tr':':yağmur_bulutunun_arkasındaki_güneş:','ar':':شمس_خلف_سحابة_ومطر:'},'\U0001F326':{'en':':sun_behind_rain_cloud:','status':unqualified,'E':0.7,'alias':[':white_sun_behind_cloud_with_rain:'],'variant':True,'de':':sonne_hinter_regenwolke:','es':':sol_detrás_de_una_nube_con_lluvia:','fr':':soleil_derrière_un_nuage_de_pluie:','ja':':雨時々晴れ:','ko':':비구름_뒤의_태양:','pt':':sol_com_chuva:','it':':sole_coperto_e_pioggia:','fa':':خورشید_پشت_ابر_بارانی:','id':':matahari_di_balik_awan_hujan:','zh':':晴转雨:','ru':':слепой_дождь:','tr':':yağmur_bulutunun_arkasındaki_güneş:','ar':':شمس_خلف_سحابة_ومطر:'},'\U0001F324\U0000FE0F':{'en':':sun_behind_small_cloud:','status':fully_qualified,'E':0.7,'alias':[':white_sun_with_small_cloud:'],'variant':True,'de':':sonne_hinter_kleiner_wolke:','es':':sol_detrás_de_una_nube_pequeña:','fr':':soleil_derrière_un_petit_nuage:','ja':':晴れ時々曇り:','ko':':작은_구름_뒤의_태양:','pt':':sol_com_nuvens:','it':':parzialmente_nuvoloso:','fa':':خورشید_پشت_ابر_کوچک:','id':':matahari_di_balik_awan_kecil:','zh':':晴偶有云:','ru':':солнце_за_облачком:','tr':':bulutun_arkasındaki_güneş:','ar':':شمس_خلف_سحابة_صغيرة:'},'\U0001F324':{'en':':sun_behind_small_cloud:','status':unqualified,'E':0.7,'alias':[':white_sun_with_small_cloud:'],'variant':True,'de':':sonne_hinter_kleiner_wolke:','es':':sol_detrás_de_una_nube_pequeña:','fr':':soleil_derrière_un_petit_nuage:','ja':':晴れ時々曇り:','ko':':작은_구름_뒤의_태양:','pt':':sol_com_nuvens:','it':':parzialmente_nuvoloso:','fa':':خورشید_پشت_ابر_کوچک:','id':':matahari_di_balik_awan_kecil:','zh':':晴偶有云:','ru':':солнце_за_облачком:','tr':':bulutun_arkasındaki_güneş:','ar':':شمس_خلف_سحابة_صغيرة:'},'\U0001F31E':{'en':':sun_with_face:','status':fully_qualified,'E':1,'de':':sonne_mit_gesicht:','es':':sol_con_cara:','fr':':soleil_avec_visage:','ja':':顔のある太陽:','ko':':얼굴이_있는_해:','pt':':rosto_do_sol:','it':':faccina_sole:','fa':':صورت_خورشید:','id':':wajah_matahari:','zh':':微笑的太阳:','ru':':солнце_с_лицом:','tr':':güneş_yüz:','ar':':وجه_الشمس:'},'\U0001F33B':{'en':':sunflower:','status':fully_qualified,'E':0.6,'de':':sonnenblume:','es':':girasol:','fr':':tournesol:','ja':':ヒマワリ:','ko':':해바라기:','pt':':girassol:','it':':girasole:','fa':':گل_آفتابگردان:','id':':bunga_matahari:','zh':':向日葵:','ru':':подсолнух:','tr':':ayçiçeği:','ar':':دوار_الشمس:'},'\U0001F576\U0000FE0F':{'en':':sunglasses:','status':fully_qualified,'E':0.7,'alias':[':dark_sunglasses:'],'variant':True,'de':':sonnenbrille:','es':':gafas_de_sol:','fr':':lunettes_de_soleil:','ja':':サングラス:','ko':':선글라스:','pt':':óculos_escuros:','it':':occhiali_da_sole:','fa':':عینک_آفتابی:','id':':kacamata_hitam:','zh':':墨镜:','ru':':солнечные_очки:','tr':':güneş_gözlüğü:','ar':':نظارة_شمس:'},'\U0001F576':{'en':':sunglasses:','status':unqualified,'E':0.7,'alias':[':dark_sunglasses:'],'variant':True,'de':':sonnenbrille:','es':':gafas_de_sol:','fr':':lunettes_de_soleil:','ja':':サングラス:','ko':':선글라스:','pt':':óculos_escuros:','it':':occhiali_da_sole:','fa':':عینک_آفتابی:','id':':kacamata_hitam:','zh':':墨镜:','ru':':солнечные_очки:','tr':':güneş_gözlüğü:','ar':':نظارة_شمس:'},'\U0001F305':{'en':':sunrise:','status':fully_qualified,'E':0.6,'de':':sonnenaufgang_über_dem_meer:','es':':amanecer:','fr':':soleil_levant:','ja':':日の出:','ko':':일출:','pt':':aurora_sobre_água:','it':':alba:','fa':':طلوع:','id':':matahari_terbit:','zh':':日出:','ru':':восход:','tr':':güneşin_doğuşu:','ar':':شروق:'},'\U0001F304':{'en':':sunrise_over_mountains:','status':fully_qualified,'E':0.6,'de':':sonnenaufgang_über_bergen:','es':':amanecer_sobre_montañas:','fr':':soleil_levant_derrière_les_montagnes:','ja':':山から日の出:','ko':':산에서_떠오르는_해:','pt':':aurora_sobre_montanhas:','it':':alba_sulle_montagne:','fa':':طلوع_در_کوهستان:','id':':matahari_terbit_di_antara_gunung:','zh':':山顶日出:','ru':':восход_солнца_над_горами:','tr':':dağların_arasından_güneşin_doğuşu:','ar':':شروق_على_جبل:'},'\U0001F307':{'en':':sunset:','status':fully_qualified,'E':0.6,'alias':[':city_sunrise:'],'de':':sonnenuntergang_in_der_stadt:','es':':puesta_del_sol:','fr':':coucher_de_soleil:','ja':':夕日:','ko':':일몰:','pt':':pôr_do_sol:','it':':tramonto:','fa':':غروب:','id':':matahari_terbenam:','zh':':日落:','ru':':закат:','tr':':gün_batımı:','ar':':غسق:'},'\U0001F9B8':{'en':':superhero:','status':fully_qualified,'E':11,'de':':superheld(in):','es':':personaje_de_superhéroe:','fr':':super-héros:','ja':':スーパーヒーロー:','ko':':히로인:','pt':':super-herói:','it':':supereroe:','fa':':ابرقهرمان:','id':':pahlawan_super:','zh':':超级英雄:','ru':':супергерой:','tr':':süper_kahraman:','ar':':بطل_خارق:'},'\U0001F9B8\U0001F3FF':{'en':':superhero_dark_skin_tone:','status':fully_qualified,'E':11,'de':':superheld(in)_dunkle_hautfarbe:','es':':personaje_de_superhéroe_tono_de_piel_oscuro:','fr':':super-héros_peau_foncée:','ja':':スーパーヒーロー_濃い肌色:','ko':':히로인_검은색_피부:','pt':':super-herói_pele_escura:','it':':supereroe_carnagione_scura:','fa':':ابرقهرمان_پوست_آبنوسی:','id':':pahlawan_super_warna_kulit_gelap:','zh':':超级英雄_较深肤色:','ru':':супергерой_очень_темный_тон_кожи:','tr':':süper_kahraman_koyu_cilt_tonu:','ar':':بطل_خارق_بشرة_بلون_غامق:'},'\U0001F9B8\U0001F3FB':{'en':':superhero_light_skin_tone:','status':fully_qualified,'E':11,'de':':superheld(in)_helle_hautfarbe:','es':':personaje_de_superhéroe_tono_de_piel_claro:','fr':':super-héros_peau_claire:','ja':':スーパーヒーロー_薄い肌色:','ko':':히로인_하얀_피부:','pt':':super-herói_pele_clara:','it':':supereroe_carnagione_chiara:','fa':':ابرقهرمان_پوست_سفید:','id':':pahlawan_super_warna_kulit_cerah:','zh':':超级英雄_较浅肤色:','ru':':супергерой_очень_светлый_тон_кожи:','tr':':süper_kahraman_açık_cilt_tonu:','ar':':بطل_خارق_بشرة_بلون_فاتح:'},'\U0001F9B8\U0001F3FE':{'en':':superhero_medium-dark_skin_tone:','status':fully_qualified,'E':11,'de':':superheld(in)_mitteldunkle_hautfarbe:','es':':personaje_de_superhéroe_tono_de_piel_oscuro_medio:','fr':':super-héros_peau_mate:','ja':':スーパーヒーロー_やや濃い肌色:','ko':':히로인_진한_갈색_피부:','pt':':super-herói_pele_morena_escura:','it':':supereroe_carnagione_abbastanza_scura:','fa':':ابرقهرمان_پوست_گندمی:','id':':pahlawan_super_warna_kulit_gelap-sedang:','zh':':超级英雄_中等深肤色:','ru':':супергерой_темный_тон_кожи:','tr':':süper_kahraman_orta_koyu_cilt_tonu:','ar':':بطل_خارق_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9B8\U0001F3FC':{'en':':superhero_medium-light_skin_tone:','status':fully_qualified,'E':11,'de':':superheld(in)_mittelhelle_hautfarbe:','es':':personaje_de_superhéroe_tono_de_piel_claro_medio:','fr':':super-héros_peau_moyennement_claire:','ja':':スーパーヒーロー_やや薄い肌色:','ko':':히로인_연한_갈색_피부:','pt':':super-herói_pele_morena_clara:','it':':supereroe_carnagione_abbastanza_chiara:','fa':':ابرقهرمان_پوست_روشن:','id':':pahlawan_super_warna_kulit_cerah-sedang:','zh':':超级英雄_中等浅肤色:','ru':':супергерой_светлый_тон_кожи:','tr':':süper_kahraman_orta_açık_cilt_tonu:','ar':':بطل_خارق_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9B8\U0001F3FD':{'en':':superhero_medium_skin_tone:','status':fully_qualified,'E':11,'de':':superheld(in)_mittlere_hautfarbe:','es':':personaje_de_superhéroe_tono_de_piel_medio:','fr':':super-héros_peau_légèrement_mate:','ja':':スーパーヒーロー_中間の肌色:','ko':':히로인_갈색_피부:','pt':':super-herói_pele_morena:','it':':supereroe_carnagione_olivastra:','fa':':ابرقهرمان_پوست_طلایی:','id':':pahlawan_super_warna_kulit_sedang:','zh':':超级英雄_中等肤色:','ru':':супергерой_средний_тон_кожи:','tr':':süper_kahraman_orta_cilt_tonu:','ar':':بطل_خارق_بشرة_بلون_معتدل:'},'\U0001F9B9':{'en':':supervillain:','status':fully_qualified,'E':11,'de':':bösewicht:','es':':personaje_de_supervillano:','fr':':super-vilain:','ja':':悪役:','ko':':슈퍼_악당:','pt':':supervilão:','it':':supercattivo:','fa':':ابرشرور:','id':':penjahat_super:','zh':':超级大坏蛋:','ru':':суперзлодей:','tr':':süper_kötü:','ar':':شرير_خارق:'},'\U0001F9B9\U0001F3FF':{'en':':supervillain_dark_skin_tone:','status':fully_qualified,'E':11,'de':':bösewicht_dunkle_hautfarbe:','es':':personaje_de_supervillano_tono_de_piel_oscuro:','fr':':super-vilain_peau_foncée:','ja':':悪役_濃い肌色:','ko':':슈퍼_악당_검은색_피부:','pt':':supervilão_pele_escura:','it':':supercattivo_carnagione_scura:','fa':':ابرشرور_پوست_آبنوسی:','id':':penjahat_super_warna_kulit_gelap:','zh':':超级大坏蛋_较深肤色:','ru':':суперзлодей_очень_темный_тон_кожи:','tr':':süper_kötü_koyu_cilt_tonu:','ar':':شرير_خارق_بشرة_بلون_غامق:'},'\U0001F9B9\U0001F3FB':{'en':':supervillain_light_skin_tone:','status':fully_qualified,'E':11,'de':':bösewicht_helle_hautfarbe:','es':':personaje_de_supervillano_tono_de_piel_claro:','fr':':super-vilain_peau_claire:','ja':':悪役_薄い肌色:','ko':':슈퍼_악당_하얀_피부:','pt':':supervilão_pele_clara:','it':':supercattivo_carnagione_chiara:','fa':':ابرشرور_پوست_سفید:','id':':penjahat_super_warna_kulit_cerah:','zh':':超级大坏蛋_较浅肤色:','ru':':суперзлодей_очень_светлый_тон_кожи:','tr':':süper_kötü_açık_cilt_tonu:','ar':':شرير_خارق_بشرة_بلون_فاتح:'},'\U0001F9B9\U0001F3FE':{'en':':supervillain_medium-dark_skin_tone:','status':fully_qualified,'E':11,'de':':bösewicht_mitteldunkle_hautfarbe:','es':':personaje_de_supervillano_tono_de_piel_oscuro_medio:','fr':':super-vilain_peau_mate:','ja':':悪役_やや濃い肌色:','ko':':슈퍼_악당_진한_갈색_피부:','pt':':supervilão_pele_morena_escura:','it':':supercattivo_carnagione_abbastanza_scura:','fa':':ابرشرور_پوست_گندمی:','id':':penjahat_super_warna_kulit_gelap-sedang:','zh':':超级大坏蛋_中等深肤色:','ru':':суперзлодей_темный_тон_кожи:','tr':':süper_kötü_orta_koyu_cilt_tonu:','ar':':شرير_خارق_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9B9\U0001F3FC':{'en':':supervillain_medium-light_skin_tone:','status':fully_qualified,'E':11,'de':':bösewicht_mittelhelle_hautfarbe:','es':':personaje_de_supervillano_tono_de_piel_claro_medio:','fr':':super-vilain_peau_moyennement_claire:','ja':':悪役_やや薄い肌色:','ko':':슈퍼_악당_연한_갈색_피부:','pt':':supervilão_pele_morena_clara:','it':':supercattivo_carnagione_abbastanza_chiara:','fa':':ابرشرور_پوست_روشن:','id':':penjahat_super_warna_kulit_cerah-sedang:','zh':':超级大坏蛋_中等浅肤色:','ru':':суперзлодей_светлый_тон_кожи:','tr':':süper_kötü_orta_açık_cilt_tonu:','ar':':شرير_خارق_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9B9\U0001F3FD':{'en':':supervillain_medium_skin_tone:','status':fully_qualified,'E':11,'de':':bösewicht_mittlere_hautfarbe:','es':':personaje_de_supervillano_tono_de_piel_medio:','fr':':super-vilain_peau_légèrement_mate:','ja':':悪役_中間の肌色:','ko':':슈퍼_악당_갈색_피부:','pt':':supervilão_pele_morena:','it':':supercattivo_carnagione_olivastra:','fa':':ابرشرور_پوست_طلایی:','id':':penjahat_super_warna_kulit_sedang:','zh':':超级大坏蛋_中等肤色:','ru':':суперзлодей_средний_тон_кожи:','tr':':süper_kötü_orta_cilt_tonu:','ar':':شرير_خارق_بشرة_بلون_معتدل:'},'\U0001F363':{'en':':sushi:','status':fully_qualified,'E':0.6,'de':':sushi:','es':':sushi:','fr':':sushi:','ja':':すし:','ko':':초밥:','pt':':sushi:','it':':sushi:','fa':':سوشی:','id':':sushi:','zh':':寿司:','ru':':суши:','tr':':suşi:','ar':':سوشي:'},'\U0001F69F':{'en':':suspension_railway:','status':fully_qualified,'E':1,'de':':schwebebahn:','es':':ferrocarril_de_suspensión:','fr':':train_suspendu:','ja':':懸垂式モノレール:','ko':':매달린_케이블카:','pt':':estrada_de_ferro_suspensa:','it':':ferrovia_sospesa:','fa':':قطار_معلق:','id':':kereta_suspensi:','zh':':空轨:','ru':':подвесная_железная_дорога:','tr':':asma_demiryolu:','ar':':سكة_حديد_معلقة:'},'\U0001F9A2':{'en':':swan:','status':fully_qualified,'E':11,'de':':schwan:','es':':cisne:','fr':':cygne:','ja':':白鳥:','ko':':백조:','pt':':cisne:','it':':cigno:','fa':':قو:','id':':angsa_leher_panjang:','zh':':天鹅:','ru':':лебедь:','tr':':kuğu:','ar':':إوَزة:'},'\U0001F4A6':{'en':':sweat_droplets:','status':fully_qualified,'E':0.6,'alias':[':sweat_drops:'],'de':':schweißtropfen:','es':':gotas_de_sudor:','fr':':gouttes_de_sueur:','ja':':あせあせ:','ko':':땀:','pt':':pingos_de_suor:','it':':gocce_di_sudore:','fa':':قطره_های_عرق:','id':':keringat:','zh':':汗滴:','ru':':капли_пота:','tr':':ter_damlası:','ar':':عرق_يتصبب:'},'\U0001F54D':{'en':':synagogue:','status':fully_qualified,'E':1,'de':':synagoge:','es':':sinagoga:','fr':':synagogue:','ja':':シナゴーグ:','ko':':시나고그:','pt':':sinagoga:','it':':sinagoga:','fa':':کنیسه:','id':':sinagog:','zh':':犹太教堂:','ru':':синагога:','tr':':sinagog:','ar':':معبد_يهودي:'},'\U0001F489':{'en':':syringe:','status':fully_qualified,'E':0.6,'de':':spritze:','es':':jeringuilla:','fr':':seringue:','ja':':注射器:','ko':':주사기:','pt':':seringa:','it':':siringa:','fa':':سرنگ:','id':':suntikan:','zh':':注射器:','ru':':шприц:','tr':':şırınga:','ar':':إبرة_حقن:'},'\U0001F455':{'en':':t-shirt:','status':fully_qualified,'E':0.6,'alias':[':tshirt:', ':shirt:', ':t_shirt:'],'de':':t-shirt:','es':':camiseta:','fr':':t-shirt:','ja':':tシャツ:','ko':':티셔츠:','pt':':camiseta:','it':':t-shirt:','fa':':تی_شرت:','id':':kaos:','zh':':T恤:','ru':':футболка:','tr':':tişört:','ar':':تي_شيرت:'},'\U0001F32E':{'en':':taco:','status':fully_qualified,'E':1,'de':':taco:','es':':taco:','fr':':taco:','ja':':タコス:','ko':':타코:','pt':':taco:','it':':taco:','fa':':تاکو:','id':':taco:','zh':':墨西哥卷饼:','ru':':тако:','tr':':tako:','ar':':تاكو:'},'\U0001F961':{'en':':takeout_box:','status':fully_qualified,'E':5,'de':':takeaway-schachtel:','es':':caja_para_llevar:','fr':':boîte_à_emporter:','ja':':テイクアウト弁当:','ko':':테이크아웃_상자:','pt':':caixa_para_viagem:','it':':confezione_da_asporto:','fa':':جعبه_بردنی:','id':':kotak_makanan:','zh':':外卖盒:','ru':':коробка_для_еды_на_вынос:','tr':':yiyecek_paketi:','ar':':علبة_طعام_للخارج:'},'\U0001FAD4':{'en':':tamale:','status':fully_qualified,'E':13,'de':':tamale:','es':':tamal:','fr':':tamal:','ja':':タマル:','ko':':타말레:','pt':':tamale:','it':':tamale:','fa':':تامال:','id':':tamale:','zh':':墨西哥粽子:','ru':':тамале:','tr':':tamale:','ar':':تامال:'},'\U0001F38B':{'en':':tanabata_tree:','status':fully_qualified,'E':0.6,'de':':tanabata-baum:','es':':árbol_de_tanabata:','fr':':arbre_à_vœux:','ja':':七夕:','ko':':소원을_건_나무:','pt':':árvore_de_tanabata:','it':':albero_di_tanabata:','fa':':درخت_تاناباتا:','id':':pohon_tanabata:','zh':':七夕树:','ru':':бамбук_с_бумажными_лентами:','tr':':tanabata_ağacı:','ar':':شجرة_احتفال:'},'\U0001F34A':{'en':':tangerine:','status':fully_qualified,'E':0.6,'alias':[':orange:', ':mandarin:'],'de':':mandarine:','es':':mandarina:','fr':':mandarine:','ja':':みかん:','ko':':귤:','pt':':tangerina:','it':':mandarino:','fa':':نارنگی:','id':':buah_jeruk:','zh':':橘子:','ru':':мандарин:','tr':':mandalina:','ar':':يوسفي:'},'\U0001F695':{'en':':taxi:','status':fully_qualified,'E':0.6,'de':':taxi:','es':':taxi:','fr':':taxi:','ja':':タクシー:','ko':':택시:','pt':':táxi:','it':':taxi:','fa':':تاکسی:','id':':taksi:','zh':':出租车:','ru':':такси:','tr':':taksi:','ar':':تاكسي:'},'\U0001F9D1\U0000200D\U0001F3EB':{'en':':teacher:','status':fully_qualified,'E':12.1,'de':':lehrer(in):','es':':docente:','fr':':personnel_enseignant:','ja':':教師:','ko':':교사:','pt':':professora_na_escola:','it':':insegnante:','fa':':معلم:','id':':guru:','zh':':老师:','ru':':педагог:','tr':':öğretmen:','ar':':أستاذ:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F3EB':{'en':':teacher_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':lehrer(in)_dunkle_hautfarbe:','es':':docente_tono_de_piel_oscuro:','fr':':personnel_enseignant_peau_foncée:','ja':':教師_濃い肌色:','ko':':교사_검은색_피부:','pt':':professora_na_escola_pele_escura:','it':':insegnante_carnagione_scura:','fa':':معلم_پوست_آبنوسی:','id':':guru_warna_kulit_gelap:','zh':':老师_较深肤色:','ru':':педагог_очень_темный_тон_кожи:','tr':':öğretmen_koyu_cilt_tonu:','ar':':أستاذ_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F3EB':{'en':':teacher_light_skin_tone:','status':fully_qualified,'E':12.1,'de':':lehrer(in)_helle_hautfarbe:','es':':docente_tono_de_piel_claro:','fr':':personnel_enseignant_peau_claire:','ja':':教師_薄い肌色:','ko':':교사_하얀_피부:','pt':':professora_na_escola_pele_clara:','it':':insegnante_carnagione_chiara:','fa':':معلم_پوست_سفید:','id':':guru_warna_kulit_cerah:','zh':':老师_较浅肤色:','ru':':педагог_очень_светлый_тон_кожи:','tr':':öğretmen_açık_cilt_tonu:','ar':':أستاذ_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F3EB':{'en':':teacher_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':lehrer(in)_mitteldunkle_hautfarbe:','es':':docente_tono_de_piel_oscuro_medio:','fr':':personnel_enseignant_peau_mate:','ja':':教師_やや濃い肌色:','ko':':교사_진한_갈색_피부:','pt':':professora_na_escola_pele_morena_escura:','it':':insegnante_carnagione_abbastanza_scura:','fa':':معلم_پوست_گندمی:','id':':guru_warna_kulit_gelap-sedang:','zh':':老师_中等深肤色:','ru':':педагог_темный_тон_кожи:','tr':':öğretmen_orta_koyu_cilt_tonu:','ar':':أستاذ_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F3EB':{'en':':teacher_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':lehrer(in)_mittelhelle_hautfarbe:','es':':docente_tono_de_piel_claro_medio:','fr':':personnel_enseignant_peau_moyennement_claire:','ja':':教師_やや薄い肌色:','ko':':교사_연한_갈색_피부:','pt':':professora_na_escola_pele_morena_clara:','it':':insegnante_carnagione_abbastanza_chiara:','fa':':معلم_پوست_روشن:','id':':guru_warna_kulit_cerah-sedang:','zh':':老师_中等浅肤色:','ru':':педагог_светлый_тон_кожи:','tr':':öğretmen_orta_açık_cilt_tonu:','ar':':أستاذ_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F3EB':{'en':':teacher_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':lehrer(in)_mittlere_hautfarbe:','es':':docente_tono_de_piel_medio:','fr':':personnel_enseignant_peau_légèrement_mate:','ja':':教師_中間の肌色:','ko':':교사_갈색_피부:','pt':':professora_na_escola_pele_morena:','it':':insegnante_carnagione_olivastra:','fa':':معلم_پوست_طلایی:','id':':guru_warna_kulit_sedang:','zh':':老师_中等肤色:','ru':':педагог_средний_тон_кожи:','tr':':öğretmen_orta_cilt_tonu:','ar':':أستاذ_بشرة_بلون_معتدل:'},'\U0001F375':{'en':':teacup_without_handle:','status':fully_qualified,'E':0.6,'alias':[':tea:'],'de':':teetasse_ohne_henkel:','es':':tazón_de_té:','fr':':tasse:','ja':':湯飲み:','ko':':차:','pt':':xícara_de_chá_sem_alça:','it':':tazza_da_tè_senza_manico:','fa':':فنجان_چای_بدون_دسته:','id':':cangkir_teh_tanpa_gagang:','zh':':热茶:','ru':':чашка_без_ручки:','tr':':çay_fincanı:','ar':':فنجان_شاي_بدون_مقبض:'},'\U0001FAD6':{'en':':teapot:','status':fully_qualified,'E':13,'de':':teekanne:','es':':tetera:','fr':':théière:','ja':':ティーポット:','ko':':찻주전자:','pt':':bule:','it':':teiera:','fa':':قوری:','id':':teko:','zh':':茶壶:','ru':':чайник:','tr':':demlik:','ar':':إبريق_شاي:'},'\U0001F4C6':{'en':':tear-off_calendar:','status':fully_qualified,'E':0.6,'alias':[':calendar:', ':tear_off_calendar:'],'de':':abreißkalender:','es':':calendario_recortable:','fr':':éphéméride:','ja':':日めくりカレンダー:','ko':':뜯어진_달력:','pt':':calendário_com_folhas_destacáveis:','it':':calendario_a_strappo:','fa':':تقویم_رومیزی:','id':':kalender_sobek:','zh':':手撕日历:','ru':':отрывной_календарь:','tr':':koparmalı_takvim:','ar':':تقويم_بأوراق_للقطع:'},'\U0001F9D1\U0000200D\U0001F4BB':{'en':':technologist:','status':fully_qualified,'E':12.1,'de':':it-experte/it-expertin:','es':':profesional_de_la_tecnología:','fr':':informaticien_(tous_genres):','ja':':技術者:','ko':':기술_전문가:','pt':':programador:','it':':persona_esperta_di_tecnologia:','fa':':متخصص:','id':':ahli_teknologi:','zh':':程序员:','ru':':инженер:','tr':':teknoloji_uzmanı:','ar':':خبير_تكنولوجيا:'},'\U0001F9D1\U0001F3FF\U0000200D\U0001F4BB':{'en':':technologist_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':it-experte/it-expertin_dunkle_hautfarbe:','es':':profesional_de_la_tecnología_tono_de_piel_oscuro:','fr':':informaticien_(tous_genres)_peau_foncée:','ja':':技術者_濃い肌色:','ko':':기술_전문가_검은색_피부:','pt':':programador_pele_escura:','it':':persona_esperta_di_tecnologia_carnagione_scura:','fa':':متخصص_پوست_آبنوسی:','id':':ahli_teknologi_warna_kulit_gelap:','zh':':程序员_较深肤色:','ru':':инженер_очень_темный_тон_кожи:','tr':':teknoloji_uzmanı_koyu_cilt_tonu:','ar':':خبير_تكنولوجيا_بشرة_بلون_غامق:'},'\U0001F9D1\U0001F3FB\U0000200D\U0001F4BB':{'en':':technologist_light_skin_tone:','status':fully_qualified,'E':12.1,'de':':it-experte/it-expertin_helle_hautfarbe:','es':':profesional_de_la_tecnología_tono_de_piel_claro:','fr':':informaticien_(tous_genres)_peau_claire:','ja':':技術者_薄い肌色:','ko':':기술_전문가_하얀_피부:','pt':':programador_pele_clara:','it':':persona_esperta_di_tecnologia_carnagione_chiara:','fa':':متخصص_پوست_سفید:','id':':ahli_teknologi_warna_kulit_cerah:','zh':':程序员_较浅肤色:','ru':':инженер_очень_светлый_тон_кожи:','tr':':teknoloji_uzmanı_açık_cilt_tonu:','ar':':خبير_تكنولوجيا_بشرة_بلون_فاتح:'},'\U0001F9D1\U0001F3FE\U0000200D\U0001F4BB':{'en':':technologist_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':it-experte/it-expertin_mitteldunkle_hautfarbe:','es':':profesional_de_la_tecnología_tono_de_piel_oscuro_medio:','fr':':informaticien_(tous_genres)_peau_mate:','ja':':技術者_やや濃い肌色:','ko':':기술_전문가_진한_갈색_피부:','pt':':programador_pele_morena_escura:','it':':persona_esperta_di_tecnologia_carnagione_abbastanza_scura:','fa':':متخصص_پوست_گندمی:','id':':ahli_teknologi_warna_kulit_gelap-sedang:','zh':':程序员_中等深肤色:','ru':':инженер_темный_тон_кожи:','tr':':teknoloji_uzmanı_orta_koyu_cilt_tonu:','ar':':خبير_تكنولوجيا_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D1\U0001F3FC\U0000200D\U0001F4BB':{'en':':technologist_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':it-experte/it-expertin_mittelhelle_hautfarbe:','es':':profesional_de_la_tecnología_tono_de_piel_claro_medio:','fr':':informaticien_(tous_genres)_peau_moyennement_claire:','ja':':技術者_やや薄い肌色:','ko':':기술_전문가_연한_갈색_피부:','pt':':programador_pele_morena_clara:','it':':persona_esperta_di_tecnologia_carnagione_abbastanza_chiara:','fa':':متخصص_پوست_روشن:','id':':ahli_teknologi_warna_kulit_cerah-sedang:','zh':':程序员_中等浅肤色:','ru':':инженер_светлый_тон_кожи:','tr':':teknoloji_uzmanı_orta_açık_cilt_tonu:','ar':':خبير_تكنولوجيا_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D1\U0001F3FD\U0000200D\U0001F4BB':{'en':':technologist_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':it-experte/it-expertin_mittlere_hautfarbe:','es':':profesional_de_la_tecnología_tono_de_piel_medio:','fr':':informaticien_(tous_genres)_peau_légèrement_mate:','ja':':技術者_中間の肌色:','ko':':기술_전문가_갈색_피부:','pt':':programador_pele_morena:','it':':persona_esperta_di_tecnologia_carnagione_olivastra:','fa':':متخصص_پوست_طلایی:','id':':ahli_teknologi_warna_kulit_sedang:','zh':':程序员_中等肤色:','ru':':инженер_средний_тон_кожи:','tr':':teknoloji_uzmanı_orta_cilt_tonu:','ar':':خبير_تكنولوجيا_بشرة_بلون_معتدل:'},'\U0001F9F8':{'en':':teddy_bear:','status':fully_qualified,'E':11,'de':':teddybär:','es':':osito_de_peluche:','fr':':ours_en_peluche:','ja':':テディベア:','ko':':테디_베어:','pt':':ursinho_de_pelúcia:','it':':orsetto:','fa':':تدی_خرسه:','id':':boneka_beruang:','zh':':泰迪熊:','ru':':плюшевый_мишка:','tr':':oyuncak_ayı:','ar':':لعبة_محشوة_بشكل_دب:'},'\U0000260E\U0000FE0F':{'en':':telephone:','status':fully_qualified,'E':0.6,'alias':[':phone:'],'variant':True,'de':':telefon:','es':':teléfono:','fr':':téléphone:','ja':':固定電話:','ko':':전화기:','pt':':telefone_no_gancho:','it':':telefono_fisso:','fa':':تلفن:','id':':telepon:','zh':':电话:','ru':':телефон:','tr':':telefon:','ar':':هاتف_أرضي:'},'\U0000260E':{'en':':telephone:','status':unqualified,'E':0.6,'alias':[':phone:'],'variant':True,'de':':telefon:','es':':teléfono:','fr':':téléphone:','ja':':固定電話:','ko':':전화기:','pt':':telefone_no_gancho:','it':':telefono_fisso:','fa':':تلفن:','id':':telepon:','zh':':电话:','ru':':телефон:','tr':':telefon:','ar':':هاتف_أرضي:'},'\U0001F4DE':{'en':':telephone_receiver:','status':fully_qualified,'E':0.6,'de':':telefonhörer:','es':':auricular_de_teléfono:','fr':':combiné_téléphonique:','ja':':受話器:','ko':':수화기:','pt':':telefone:','it':':cornetta:','fa':':گوشی_تلفن:','id':':gagang_telepon:','zh':':电话听筒:','ru':':телефонная_трубка:','tr':':ahize:','ar':':سماعة_هاتف:'},'\U0001F52D':{'en':':telescope:','status':fully_qualified,'E':1,'de':':teleskop:','es':':telescopio:','fr':':télescope:','ja':':望遠鏡:','ko':':망원경:','pt':':telescópio:','it':':telescopio:','fa':':تلسکوپ:','id':':teleskop:','zh':':望远镜:','ru':':телескоп:','tr':':teleskop:','ar':':تليسكوب:'},'\U0001F4FA':{'en':':television:','status':fully_qualified,'E':0.6,'alias':[':tv:'],'variant':True,'de':':fernseher:','es':':televisión:','fr':':téléviseur:','ja':':テレビ:','ko':':텔레비전:','pt':':televisão:','it':':televisore:','fa':':تلویزیون:','id':':televisi:','zh':':电视机:','ru':':телевизор:','tr':':televizyon:','ar':':تليفزيون:'},'\U0001F565':{'en':':ten-thirty:','status':fully_qualified,'E':0.7,'alias':[':clock1030:', ':ten_thirty:'],'variant':True,'de':':10.30_uhr:','es':':diez_y_media:','fr':':dix_heures_et_demie:','ja':':10時半:','ko':':열_시_반:','pt':':dez_e_meia:','it':':ore_dieci_e_mezza:','fa':':ساعت_ده_و_نیم:','id':':jam_setengah_sebelas:','zh':':十点半:','ru':':пол_одиннадцатого:','tr':':on_buçuk:','ar':':العاشرة_والنصف:'},'\U0001F559':{'en':':ten_o’clock:','status':fully_qualified,'E':0.6,'alias':[':clock10:', ':ten_oclock:'],'variant':True,'de':':10.00_uhr:','es':':10_en_punto:','fr':':dix_heures:','ja':':10時:','ko':':열_시:','pt':':10_horas:','it':':ore_dieci:','fa':':ساعت_ده:','id':':jam_sepuluh_tepat:','zh':':十点:','ru':':десять_часов:','tr':':saat_10:','ar':':الساعة_العاشرة:'},'\U0001F3BE':{'en':':tennis:','status':fully_qualified,'E':0.6,'de':':tennisball:','es':':pelota_de_tenis:','fr':':tennis:','ja':':テニス:','ko':':테니스:','pt':':tênis:','it':':tennis:','fa':':تنیس:','id':':tenis:','zh':':网球:','ru':':теннис:','tr':':tenis:','ar':':تنس:'},'\U000026FA':{'en':':tent:','status':fully_qualified,'E':0.6,'variant':True,'de':':zelt:','es':':tienda_de_campaña:','fr':':tente:','ja':':テント:','ko':':텐트:','pt':':barraca:','it':':tenda:','fa':':چادر:','id':':tenda:','zh':':帐篷:','ru':':палатка:','tr':':çadır:','ar':':خيمة:'},'\U0001F9EA':{'en':':test_tube:','status':fully_qualified,'E':11,'de':':reagenzglas:','es':':tubo_de_ensayo:','fr':':tube_à_essai:','ja':':試験管:','ko':':시험관:','pt':':tubo_de_ensaio:','it':':provetta:','fa':':لولهٔ_آزمایشگاه:','id':':tabung_uji:','zh':':试管:','ru':':пробирка:','tr':':deney_tüpü:','ar':':أنبوب_اختبار:'},'\U0001F321\U0000FE0F':{'en':':thermometer:','status':fully_qualified,'E':0.7,'variant':True,'de':':thermometer:','es':':termómetro:','fr':':thermomètre:','ja':':温度計:','ko':':온도계:','pt':':termômetro:','it':':termometro:','fa':':دماسنج:','id':':termometer:','zh':':温度计:','ru':':термометр:','tr':':termometre:','ar':':ترمومتر:'},'\U0001F321':{'en':':thermometer:','status':unqualified,'E':0.7,'variant':True,'de':':thermometer:','es':':termómetro:','fr':':thermomètre:','ja':':温度計:','ko':':온도계:','pt':':termômetro:','it':':termometro:','fa':':دماسنج:','id':':termometer:','zh':':温度计:','ru':':термометр:','tr':':termometre:','ar':':ترمومتر:'},'\U0001F914':{'en':':thinking_face:','status':fully_qualified,'E':1,'alias':[':thinking:'],'de':':nachdenkendes_gesicht:','es':':cara_pensativa:','fr':':visage_en_pleine_réflexion:','ja':':考える顔:','ko':':생각하는_얼굴:','pt':':rosto_pensativo:','it':':faccina_concentrata:','fa':':درحال_تفکر:','id':':wajah_berpikir:','zh':':想一想:','ru':':думает:','tr':':düşünen_yüz:','ar':':وجه_يفكر:'},'\U0001FA74':{'en':':thong_sandal:','status':fully_qualified,'E':13,'de':':zehensandale:','es':':chancla:','fr':':tong:','ja':':ビーチサンダル:','ko':':가락신:','pt':':chinelo:','it':':infradito:','fa':':دمپایی_لاانگشتی:','id':':sandal_jepit:','zh':':夹趾凉鞋:','ru':':вьетнамки:','tr':':parmak_arası_terlik:','ar':':صندل_إصبع:'},'\U0001F4AD':{'en':':thought_balloon:','status':fully_qualified,'E':1,'de':':gedankenblase:','es':':bocadillo_de_pensamiento:','fr':':bulle_de_pensée:','ja':':雲形の吹き出し:','ko':':생각_풍선:','pt':':balão_de_pensamento:','it':':nuvoletta:','fa':':بالن_افکار:','id':':gelembung_pikiran:','zh':':内心活动气泡:','ru':':облачко_с_мыслями:','tr':':düşünce_balonu:','ar':':فقاعة_أفكار:'},'\U0001F9F5':{'en':':thread:','status':fully_qualified,'E':11,'de':':faden:','es':':hilo:','fr':':bobine_de_fil:','ja':':糸:','ko':':실타래:','pt':':carretel:','it':':filo:','fa':':نخ:','id':':benang:','zh':':线:','ru':':катушка_ниток:','tr':':iplik:','ar':':بكرة_خيط:'},'\U0001F55E':{'en':':three-thirty:','status':fully_qualified,'E':0.7,'alias':[':clock330:', ':three_thirty:'],'variant':True,'de':':3.30_uhr:','es':':tres_y_media:','fr':':trois_heures_et_demie:','ja':':3時半:','ko':':세_시_반:','pt':':três_e_meia:','it':':ore_tre_e_mezza:','fa':':ساعت_سه_و_نیم:','id':':jam_setengah_empat:','zh':':三点半:','ru':':полчетвертого:','tr':':üç_buçuk:','ar':':الثالثة_والنصف:'},'\U0001F552':{'en':':three_o’clock:','status':fully_qualified,'E':0.6,'alias':[':clock3:', ':three_oclock:'],'variant':True,'de':':3.00_uhr:','es':':3_en_punto:','fr':':trois_heures:','ja':':3時:','ko':':세_시:','pt':':3_horas:','it':':ore_tre:','fa':':ساعت_سه:','id':':jam_tiga_tepat:','zh':':三点:','ru':':три_часа:','tr':':saat_3:','ar':':الساعة_الثالثة:'},'\U0001F44E':{'en':':thumbs_down:','status':fully_qualified,'E':0.6,'alias':[':thumbsdown:', ':_1:', ':-1:'],'variant':True,'de':':daumen_runter:','es':':pulgar_hacia_abajo:','fr':':pouce_vers_le_bas:','ja':':サムズダウン:','ko':':내린_엄지:','pt':':polegar_para_baixo:','it':':pollice_verso:','fa':':نپسندیدن:','id':':jempol_ke_bawah:','zh':':拇指向下:','ru':':большой_палец_вниз:','tr':':baş_parmak_aşağıda:','ar':':رفض:'},'\U0001F44E\U0001F3FF':{'en':':thumbs_down_dark_skin_tone:','status':fully_qualified,'E':1,'de':':daumen_runter_dunkle_hautfarbe:','es':':pulgar_hacia_abajo_tono_de_piel_oscuro:','fr':':pouce_vers_le_bas_peau_foncée:','ja':':サムズダウン_濃い肌色:','ko':':내린_엄지_검은색_피부:','pt':':polegar_para_baixo_pele_escura:','it':':pollice_verso_carnagione_scura:','fa':':نپسندیدن_پوست_آبنوسی:','id':':jempol_ke_bawah_warna_kulit_gelap:','zh':':拇指向下_较深肤色:','ru':':большой_палец_вниз_очень_темный_тон_кожи:','tr':':baş_parmak_aşağıda_koyu_cilt_tonu:','ar':':رفض_بشرة_بلون_غامق:'},'\U0001F44E\U0001F3FB':{'en':':thumbs_down_light_skin_tone:','status':fully_qualified,'E':1,'de':':daumen_runter_helle_hautfarbe:','es':':pulgar_hacia_abajo_tono_de_piel_claro:','fr':':pouce_vers_le_bas_peau_claire:','ja':':サムズダウン_薄い肌色:','ko':':내린_엄지_하얀_피부:','pt':':polegar_para_baixo_pele_clara:','it':':pollice_verso_carnagione_chiara:','fa':':نپسندیدن_پوست_سفید:','id':':jempol_ke_bawah_warna_kulit_cerah:','zh':':拇指向下_较浅肤色:','ru':':большой_палец_вниз_очень_светлый_тон_кожи:','tr':':baş_parmak_aşağıda_açık_cilt_tonu:','ar':':رفض_بشرة_بلون_فاتح:'},'\U0001F44E\U0001F3FE':{'en':':thumbs_down_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':daumen_runter_mitteldunkle_hautfarbe:','es':':pulgar_hacia_abajo_tono_de_piel_oscuro_medio:','fr':':pouce_vers_le_bas_peau_mate:','ja':':サムズダウン_やや濃い肌色:','ko':':내린_엄지_진한_갈색_피부:','pt':':polegar_para_baixo_pele_morena_escura:','it':':pollice_verso_carnagione_abbastanza_scura:','fa':':نپسندیدن_پوست_گندمی:','id':':jempol_ke_bawah_warna_kulit_gelap-sedang:','zh':':拇指向下_中等深肤色:','ru':':большой_палец_вниз_темный_тон_кожи:','tr':':baş_parmak_aşağıda_orta_koyu_cilt_tonu:','ar':':رفض_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F44E\U0001F3FC':{'en':':thumbs_down_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':daumen_runter_mittelhelle_hautfarbe:','es':':pulgar_hacia_abajo_tono_de_piel_claro_medio:','fr':':pouce_vers_le_bas_peau_moyennement_claire:','ja':':サムズダウン_やや薄い肌色:','ko':':내린_엄지_연한_갈색_피부:','pt':':polegar_para_baixo_pele_morena_clara:','it':':pollice_verso_carnagione_abbastanza_chiara:','fa':':نپسندیدن_پوست_روشن:','id':':jempol_ke_bawah_warna_kulit_cerah-sedang:','zh':':拇指向下_中等浅肤色:','ru':':большой_палец_вниз_светлый_тон_кожи:','tr':':baş_parmak_aşağıda_orta_açık_cilt_tonu:','ar':':رفض_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F44E\U0001F3FD':{'en':':thumbs_down_medium_skin_tone:','status':fully_qualified,'E':1,'de':':daumen_runter_mittlere_hautfarbe:','es':':pulgar_hacia_abajo_tono_de_piel_medio:','fr':':pouce_vers_le_bas_peau_légèrement_mate:','ja':':サムズダウン_中間の肌色:','ko':':내린_엄지_갈색_피부:','pt':':polegar_para_baixo_pele_morena:','it':':pollice_verso_carnagione_olivastra:','fa':':نپسندیدن_پوست_طلایی:','id':':jempol_ke_bawah_warna_kulit_sedang:','zh':':拇指向下_中等肤色:','ru':':большой_палец_вниз_средний_тон_кожи:','tr':':baş_parmak_aşağıda_orta_cilt_tonu:','ar':':رفض_بشرة_بلون_معتدل:'},'\U0001F44D':{'en':':thumbs_up:','status':fully_qualified,'E':0.6,'alias':[':thumbsup:', ':+1:'],'variant':True,'de':':daumen_hoch:','es':':pulgar_hacia_arriba:','fr':':pouce_vers_le_haut:','ja':':サムズアップ:','ko':':올린_엄지:','pt':':polegar_para_cima:','it':':pollice_in_su:','fa':':پسندیدن:','id':':jempol_ke_atas:','zh':':拇指向上:','ru':':большой_палец_вверх:','tr':':baş_parmak_yukarıda:','ar':':قبول:'},'\U0001F44D\U0001F3FF':{'en':':thumbs_up_dark_skin_tone:','status':fully_qualified,'E':1,'de':':daumen_hoch_dunkle_hautfarbe:','es':':pulgar_hacia_arriba_tono_de_piel_oscuro:','fr':':pouce_vers_le_haut_peau_foncée:','ja':':サムズアップ_濃い肌色:','ko':':올린_엄지_검은색_피부:','pt':':polegar_para_cima_pele_escura:','it':':pollice_in_su_carnagione_scura:','fa':':پسندیدن_پوست_آبنوسی:','id':':jempol_ke_atas_warna_kulit_gelap:','zh':':拇指向上_较深肤色:','ru':':большой_палец_вверх_очень_темный_тон_кожи:','tr':':baş_parmak_yukarıda_koyu_cilt_tonu:','ar':':قبول_بشرة_بلون_غامق:'},'\U0001F44D\U0001F3FB':{'en':':thumbs_up_light_skin_tone:','status':fully_qualified,'E':1,'de':':daumen_hoch_helle_hautfarbe:','es':':pulgar_hacia_arriba_tono_de_piel_claro:','fr':':pouce_vers_le_haut_peau_claire:','ja':':サムズアップ_薄い肌色:','ko':':올린_엄지_하얀_피부:','pt':':polegar_para_cima_pele_clara:','it':':pollice_in_su_carnagione_chiara:','fa':':پسندیدن_پوست_سفید:','id':':jempol_ke_atas_warna_kulit_cerah:','zh':':拇指向上_较浅肤色:','ru':':большой_палец_вверх_очень_светлый_тон_кожи:','tr':':baş_parmak_yukarıda_açık_cilt_tonu:','ar':':قبول_بشرة_بلون_فاتح:'},'\U0001F44D\U0001F3FE':{'en':':thumbs_up_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':daumen_hoch_mitteldunkle_hautfarbe:','es':':pulgar_hacia_arriba_tono_de_piel_oscuro_medio:','fr':':pouce_vers_le_haut_peau_mate:','ja':':サムズアップ_やや濃い肌色:','ko':':올린_엄지_진한_갈색_피부:','pt':':polegar_para_cima_pele_morena_escura:','it':':pollice_in_su_carnagione_abbastanza_scura:','fa':':پسندیدن_پوست_گندمی:','id':':jempol_ke_atas_warna_kulit_gelap-sedang:','zh':':拇指向上_中等深肤色:','ru':':большой_палец_вверх_темный_тон_кожи:','tr':':baş_parmak_yukarıda_orta_koyu_cilt_tonu:','ar':':قبول_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F44D\U0001F3FC':{'en':':thumbs_up_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':daumen_hoch_mittelhelle_hautfarbe:','es':':pulgar_hacia_arriba_tono_de_piel_claro_medio:','fr':':pouce_vers_le_haut_peau_moyennement_claire:','ja':':サムズアップ_やや薄い肌色:','ko':':올린_엄지_연한_갈색_피부:','pt':':polegar_para_cima_pele_morena_clara:','it':':pollice_in_su_carnagione_abbastanza_chiara:','fa':':پسندیدن_پوست_روشن:','id':':jempol_ke_atas_warna_kulit_cerah-sedang:','zh':':拇指向上_中等浅肤色:','ru':':большой_палец_вверх_светлый_тон_кожи:','tr':':baş_parmak_yukarıda_orta_açık_cilt_tonu:','ar':':قبول_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F44D\U0001F3FD':{'en':':thumbs_up_medium_skin_tone:','status':fully_qualified,'E':1,'de':':daumen_hoch_mittlere_hautfarbe:','es':':pulgar_hacia_arriba_tono_de_piel_medio:','fr':':pouce_vers_le_haut_peau_légèrement_mate:','ja':':サムズアップ_中間の肌色:','ko':':올린_엄지_갈색_피부:','pt':':polegar_para_cima_pele_morena:','it':':pollice_in_su_carnagione_olivastra:','fa':':پسندیدن_پوست_طلایی:','id':':jempol_ke_atas_warna_kulit_sedang:','zh':':拇指向上_中等肤色:','ru':':большой_палец_вверх_средний_тон_кожи:','tr':':baş_parmak_yukarıda_orta_cilt_tonu:','ar':':قبول_بشرة_بلون_معتدل:'},'\U0001F3AB':{'en':':ticket:','status':fully_qualified,'E':0.6,'de':':ticket:','es':':tique:','fr':':billet:','ja':':チケット:','ko':':티켓:','pt':':ingresso:','it':':biglietto:','fa':':بلیت:','id':':tiket:','zh':':票:','ru':':билет:','tr':':bilet:','ar':':تذكرة:'},'\U0001F405':{'en':':tiger:','status':fully_qualified,'E':1,'alias':[':tiger2:'],'de':':tiger:','es':':tigre:','fr':':tigre:','ja':':トラ:','ko':':호랑이:','pt':':tigre:','it':':tigre:','fa':':ببر:','id':':harimau:','zh':':老虎:','ru':':тигр:','tr':':kaplan:','ar':':نمر:'},'\U0001F42F':{'en':':tiger_face:','status':fully_qualified,'E':0.6,'alias':[':tiger:'],'de':':tigergesicht:','es':':cara_de_tigre:','fr':':tête_de_tigre:','ja':':トラの顔:','ko':':호랑이_얼굴:','pt':':rosto_de_tigre:','it':':muso_di_tigre:','fa':':صورت_ببر:','id':':wajah_harimau:','zh':':老虎头:','ru':':морда_тигра:','tr':':kaplan_yüzü:','ar':':وجه_نمر:'},'\U000023F2\U0000FE0F':{'en':':timer_clock:','status':fully_qualified,'E':1,'variant':True,'de':':zeitschaltuhr:','es':':temporizador:','fr':':horloge:','ja':':タイマー:','ko':':타이머_시계:','pt':':relógio_temporizador:','it':':timer:','fa':':تایمر:','id':':penghitung_waktu:','zh':':定时器:','ru':':таимер:','tr':':alarm:','ar':':ساعة_مؤقت:'},'\U000023F2':{'en':':timer_clock:','status':unqualified,'E':1,'variant':True,'de':':zeitschaltuhr:','es':':temporizador:','fr':':horloge:','ja':':タイマー:','ko':':타이머_시계:','pt':':relógio_temporizador:','it':':timer:','fa':':تایمر:','id':':penghitung_waktu:','zh':':定时器:','ru':':таймер:','tr':':alarm:','ar':':ساعة_مؤقت:'},'\U0001F62B':{'en':':tired_face:','status':fully_qualified,'E':0.6,'de':':müdes_gesicht:','es':':cara_cansada:','fr':':visage_fatigué:','ja':':疲れた:','ko':':피곤한_얼굴:','pt':':rosto_cansado:','it':':faccina_stanca:','fa':':صورت_خسته:','id':':wajah_kelelahan:','zh':':累:','ru':':сильная_усталость:','tr':':yorgun_yüz:','ar':':وجه_متعب:'},'\U0001F6BD':{'en':':toilet:','status':fully_qualified,'E':0.6,'de':':toilette:','es':':inodoro:','fr':':toilettes:','ja':':便器:','ko':':변기:','pt':':vaso_sanitário:','it':':toilette:','fa':':توالت:','id':':toilet:','zh':':马桶:','ru':':унитаз:','tr':':klozet:','ar':':مرحاض:'},'\U0001F345':{'en':':tomato:','status':fully_qualified,'E':0.6,'de':':tomate:','es':':tomate:','fr':':tomate:','ja':':トマト:','ko':':토마토:','pt':':tomate:','it':':pomodoro:','fa':':گوجه_فرنگی:','id':':tomat:','zh':':西红柿:','ru':':помидор:','tr':':domates:','ar':':طماطم:'},'\U0001F445':{'en':':tongue:','status':fully_qualified,'E':0.6,'de':':zunge:','es':':lengua:','fr':':langue:','ja':':舌:','ko':':혀:','pt':':língua:','it':':lingua:','fa':':زبان:','id':':lidah:','zh':':舌头:','ru':':язык:','tr':':dil:','ar':':لسان:'},'\U0001F9F0':{'en':':toolbox:','status':fully_qualified,'E':11,'de':':werkzeugkasten:','es':':caja_de_herramientas:','fr':':boîte_à_outils:','ja':':工具箱:','ko':':공구통:','pt':':caixa_de_ferramentas:','it':':cassetta_degli_attrezzi:','fa':':جعبه_ابزار:','id':':kotak_perkakas:','zh':':工具箱:','ru':':ящик_для_инструментов:','tr':':alet_çantası:','ar':':صندوق_أدوات:'},'\U0001F9B7':{'en':':tooth:','status':fully_qualified,'E':11,'de':':zahn:','es':':diente:','fr':':dent:','ja':':歯:','ko':':이빨:','pt':':dente:','it':':dente:','fa':':دندان:','id':':gigi:','zh':':牙齿:','ru':':зуб:','tr':':diş:','ar':':سن:'},'\U0001FAA5':{'en':':toothbrush:','status':fully_qualified,'E':13,'de':':zahnbürste:','es':':cepillo_de_dientes:','fr':':brosse_à_dents:','ja':':歯ブラシ:','ko':':칫솔:','pt':':escova_de_dentes:','it':':spazzolino_da_denti:','fa':':مسواک:','id':':sikat_gigi:','zh':':牙刷:','ru':':зубная_щетка:','tr':':diş_fırçası:','ar':':فرشاة_أسنان:'},'\U0001F3A9':{'en':':top_hat:','status':fully_qualified,'E':0.6,'alias':[':tophat:'],'de':':zylinder:','es':':sombrero_de_copa:','fr':':haut_de_forme:','ja':':シルクハット:','ko':':마술사_모자:','pt':':cartola:','it':':cilindro:','fa':':کلاه_مردانه_استوانه_ای:','id':':topi_tinggi:','zh':':礼帽:','ru':':цилиндр:','tr':':silindir_şapka:','ar':':قبعة_رسمية:'},'\U0001F32A\U0000FE0F':{'en':':tornado:','status':fully_qualified,'E':0.7,'alias':[':cloud_with_tornado:'],'variant':True,'de':':wirbelsturm:','es':':tornado:','fr':':tornade:','ja':':竜巻:','ko':':토네이도:','pt':':tornado:','it':':tornado:','fa':':گردباد:','id':':tornado:','zh':':龙卷风:','ru':':торнадо:','tr':':hortum:','ar':':إعصار:'},'\U0001F32A':{'en':':tornado:','status':unqualified,'E':0.7,'alias':[':cloud_with_tornado:'],'variant':True,'de':':wirbelsturm:','es':':tornado:','fr':':tornade:','ja':':竜巻:','ko':':토네이도:','pt':':tornado:','it':':tornado:','fa':':گردباد:','id':':tornado:','zh':':龙卷风:','ru':':торнадо:','tr':':hortum:','ar':':إعصار:'},'\U0001F5B2\U0000FE0F':{'en':':trackball:','status':fully_qualified,'E':0.7,'variant':True,'de':':trackball:','es':':bola_de_desplazamiento:','fr':':boule_de_commande:','ja':':トラックボール:','ko':':트랙볼:','pt':':trackball:','it':':trackball:','fa':':توپک:','id':':trackball:','zh':':轨迹球:','ru':':трекбол:','tr':':iz_topu:','ar':':كرة_مسار:'},'\U0001F5B2':{'en':':trackball:','status':unqualified,'E':0.7,'variant':True,'de':':trackball:','es':':bola_de_desplazamiento:','fr':':boule_de_commande:','ja':':トラックボール:','ko':':트랙볼:','pt':':trackball:','it':':trackball:','fa':':توپک:','id':':trackball:','zh':':轨迹球:','ru':':трекбол:','tr':':iz_topu:','ar':':كرة_مسار:'},'\U0001F69C':{'en':':tractor:','status':fully_qualified,'E':1,'de':':traktor:','es':':tractor:','fr':':tracteur:','ja':':トラクター:','ko':':트랙터:','pt':':trator:','it':':trattore:','fa':':تراکتور:','id':':traktor:','zh':':拖拉机:','ru':':трактор:','tr':':traktör:','ar':':جرار:'},'\U00002122\U0000FE0F':{'en':':trade_mark:','status':fully_qualified,'E':0.6,'alias':[':tm:'],'variant':True,'de':':markenzeichen:','es':':símbolo_de_marca_comercial:','fr':':marque_commerciale_déposée:','ja':':商標マーク:','ko':':상표:','pt':':símbolo_de_marca_registrada:','it':':marchio:','fa':':نماد_بازرگانی:','id':':merek_dagang:','zh':':商标:','ru':':товарныи_знак:','tr':':ticari_marka_işareti:','ar':':إشارة_علامة_تجارية:'},'\U00002122':{'en':':trade_mark:','status':unqualified,'E':0.6,'alias':[':tm:'],'variant':True,'de':':markenzeichen:','es':':símbolo_de_marca_comercial:','fr':':marque_commerciale_déposée:','ja':':商標マーク:','ko':':상표:','pt':':símbolo_de_marca_registrada:','it':':marchio:','fa':':نماد_بازرگانی:','id':':merek_dagang:','zh':':商标:','ru':':товарный_знак:','tr':':ticari_marka_işareti:','ar':':إشارة_علامة_تجارية:'},'\U0001F686':{'en':':train:','status':fully_qualified,'E':1,'alias':[':train2:'],'de':':zug:','es':':tren:','fr':':train:','ja':':電車正面:','ko':':기차:','pt':':trem:','it':':treno:','fa':':قطار:','id':':kereta:','zh':':火车:','ru':':поезд:','tr':':tren:','ar':':قطار:'},'\U0001F68A':{'en':':tram:','status':fully_qualified,'E':1,'de':':straßenbahn:','es':':tranvía:','fr':':tramway:','ja':':路面電車正面:','ko':':트램:','pt':':bonde_elétrico:','it':':tram:','fa':':تراموا:','id':':trem:','zh':':路面电车:','ru':':трамвай:','tr':':tramvay:','ar':':ترام:'},'\U0001F68B':{'en':':tram_car:','status':fully_qualified,'E':1,'alias':[':train:'],'de':':straßenbahnwagen:','es':':vagón_de_tranvía:','fr':':wagon_de_tramway:','ja':':路面電車:','ko':':트램_차량:','pt':':bonde:','it':':vagone_del_tram:','fa':':قطار_خیابانی:','id':':gerbong_trem:','zh':':有轨电车:','ru':':трамвайный_вагон:','tr':':tramvay_vagonu:','ar':':سيارة_ترام:'},'\U0001F3F3\U0000FE0F\U0000200D\U000026A7\U0000FE0F':{'en':':transgender_flag:','status':fully_qualified,'E':13,'de':':transgender-flagge:','es':':bandera_transgénero:','fr':':drapeau_transgenre:','ja':':トランスジェンダーフラッグ:','ko':':트랜스젠더_깃발:','pt':':bandeira_transgênero:','it':':bandiera_transgender:','fa':':پرچم_تراجنسیتی:','id':':bendera_transgender:','zh':':跨性别旗:','ru':':флаг_трансгендерного_сообщества:','tr':':transgender_bayrağı:','ar':':علم_المتحولين_جنسيا:'},'\U0001F3F3\U0000200D\U000026A7\U0000FE0F':{'en':':transgender_flag:','status':unqualified,'E':13,'de':':transgender-flagge:','es':':bandera_transgénero:','fr':':drapeau_transgenre:','ja':':トランスジェンダーフラッグ:','ko':':트랜스젠더_깃발:','pt':':bandeira_transgênero:','it':':bandiera_transgender:','fa':':پرچم_تراجنسیتی:','id':':bendera_transgender:','zh':':跨性别旗:','ru':':флаг_трансгендерного_сообщества:','tr':':transgender_bayrağı:','ar':':علم_المتحولين_جنسيا:'},'\U0001F3F3\U0000FE0F\U0000200D\U000026A7':{'en':':transgender_flag:','status':minimally_qualified,'E':13,'de':':transgender-flagge:','es':':bandera_transgénero:','fr':':drapeau_transgenre:','ja':':トランスジェンダーフラッグ:','ko':':트랜스젠더_깃발:','pt':':bandeira_transgênero:','it':':bandiera_transgender:','fa':':پرچم_تراجنسیتی:','id':':bendera_transgender:','zh':':跨性别旗:','ru':':флаг_трансгендерного_сообщества:','tr':':transgender_bayrağı:','ar':':علم_المتحولين_جنسيا:'},'\U0001F3F3\U0000200D\U000026A7':{'en':':transgender_flag:','status':unqualified,'E':13,'de':':transgender-flagge:','es':':bandera_transgénero:','fr':':drapeau_transgenre:','ja':':トランスジェンダーフラッグ:','ko':':트랜스젠더_깃발:','pt':':bandeira_transgênero:','it':':bandiera_transgender:','fa':':پرچم_تراجنسیتی:','id':':bendera_transgender:','zh':':跨性别旗:','ru':':флаг_трансгендерного_сообщества:','tr':':transgender_bayrağı:','ar':':علم_المتحولين_جنسيا:'},'\U000026A7\U0000FE0F':{'en':':transgender_symbol:','status':fully_qualified,'E':13,'variant':True,'de':':symbol_für_transgender:','es':':símbolo_de_transgénero:','fr':':symbole_de_la_communauté_transgenre:','ja':':トランスジェンダーのマーク:','ko':':트랜스젠더_기호:','pt':':símbolo_transgênero:','it':':simbolo_transgender:','fa':':علامت_تراجنسیتی:','id':':simbol_transgender:','zh':':跨性别符号:','ru':':трансгендер:','tr':':transgender_sembolü:','ar':':رمز_المتحولين_جنسيا:'},'\U000026A7':{'en':':transgender_symbol:','status':unqualified,'E':13,'variant':True,'de':':transgender-symbol:','es':':símbolo_de_transgénero:','fr':':symbole_de_la_communauté_transgenre:','ja':':トランスジェンダーのマーク:','ko':':트랜스젠더_기호:','pt':':símbolo_transgênero:','it':':simbolo_transgender:','fa':':علامت_تراجنسیتی:','id':':simbol_transgender:','zh':':跨性别符号:','ru':':трансгендер:','tr':':transgender_sembolü:','ar':':رمز_المتحولين_جنسيا:'},'\U0001F6A9':{'en':':triangular_flag:','status':fully_qualified,'E':0.6,'alias':[':triangular_flag_on_post:'],'de':':wimpel:','es':':bandera_triangular:','fr':':drapeau_triangulaire:','ja':':三角の旗:','ko':':세모_깃발:','pt':':bandeira_triangular:','it':':bandierina_rossa:','fa':':پرچم_مثلثی:','id':':bendera_segitiga:','zh':':三角旗:','ru':':треугольный_флаг:','tr':':üçgen_bayrak:','ar':':علم_مثلث:'},'\U0001F4D0':{'en':':triangular_ruler:','status':fully_qualified,'E':0.6,'de':':dreieckiges_lineal:','es':':escuadra:','fr':':équerre:','ja':':三角定規:','ko':':삼각자:','pt':':régua_triangular:','it':':squadra:','fa':':گونیا:','id':':penggaris_segitiga:','zh':':三角尺:','ru':':линейка_треугольник:','tr':':gönye:','ar':':مسطرة_مثلثة:'},'\U0001F531':{'en':':trident_emblem:','status':fully_qualified,'E':0.6,'alias':[':trident:'],'de':':dreizack:','es':':emblema_de_tridente:','fr':':trident:','ja':':トライデント:','ko':':삼지창:','pt':':emblema_de_tridente:','it':':tridente:','fa':':نیزهٔ_سه_شاخه:','id':':emblem_trisula:','zh':':三叉戟徽章:','ru':':трезубец:','tr':':üçlü_çatal_işareti:','ar':':رمز_ثلاثي_الشعب:'},'\U0001F9CC':{'en':':troll:','status':fully_qualified,'E':14,'de':':troll:','es':':trol:','fr':':troll:','ja':':トロール:','ko':':트롤:','pt':':troll:','it':':troll:','fa':':غول:','id':':raksasa:','zh':':穴居巨怪:','ru':':тролль:','tr':':trol:','ar':':الترول:'},'\U0001F68E':{'en':':trolleybus:','status':fully_qualified,'E':1,'de':':oberleitungsbus:','es':':trolebús:','fr':':trolleybus:','ja':':トロリーバス:','ko':':트롤리_버스:','pt':':trólebus:','it':':filobus:','fa':':اتوبوس_برقی:','id':':bus_listrik:','zh':':无轨电车:','ru':':троллейбус:','tr':':troleybüs:','ar':':أتوبيس_ترولي:'},'\U0001F3C6':{'en':':trophy:','status':fully_qualified,'E':0.6,'variant':True,'de':':pokal:','es':':trofeo:','fr':':trophée:','ja':':トロフィー:','ko':':트로피:','pt':':troféu:','it':':coppa:','fa':':جام_ورزشی:','id':':piala:','zh':':奖杯:','ru':':кубок:','tr':':zafer_kupası:','ar':':كأس:'},'\U0001F379':{'en':':tropical_drink:','status':fully_qualified,'E':0.6,'de':':cocktail:','es':':bebida_tropical:','fr':':cocktail_tropical:','ja':':トロピカルドリンク:','ko':':음료수:','pt':':bebida_tropical:','it':':cocktail_tropicale:','fa':':نوشیدنی_استوایی:','id':':minuman_tropis:','zh':':热带水果饮料:','ru':':тропический_коктейль:','tr':':tropik_içecek:','ar':':مشروب_استوائي:'},'\U0001F420':{'en':':tropical_fish:','status':fully_qualified,'E':0.6,'de':':tropenfisch:','es':':pez_tropical:','fr':':poisson_tropical:','ja':':熱帯魚:','ko':':열대어:','pt':':peixe_tropical:','it':':pesce_tropicale:','fa':':ماهی_استوایی:','id':':ikan_tropis:','zh':':热带鱼:','ru':':тропическая_рыба:','tr':':tropikal_balık:','ar':':سمكة_استوائية:'},'\U0001F3BA':{'en':':trumpet:','status':fully_qualified,'E':0.6,'de':':trompete:','es':':trompeta:','fr':':trompette:','ja':':トランペット:','ko':':트럼펫:','pt':':trompete:','it':':tromba:','fa':':ترومپت:','id':':trompet:','zh':':小号:','ru':':труба:','tr':':trompet:','ar':':آلة_نفخ:'},'\U0001F337':{'en':':tulip:','status':fully_qualified,'E':0.6,'de':':tulpe:','es':':tulipán:','fr':':tulipe:','ja':':チューリップ:','ko':':튤립:','pt':':tulipa:','it':':tulipano:','fa':':لاله:','id':':tulip:','zh':':郁金香:','ru':':тюльпан:','tr':':lale:','ar':':زهرة_التوليب:'},'\U0001F943':{'en':':tumbler_glass:','status':fully_qualified,'E':3,'de':':trinkglas:','es':':vaso_de_whisky:','fr':':verre_tumbler:','ja':':タンブラーグラス:','ko':':양주잔:','pt':':copo:','it':':bicchiere_tumbler:','fa':':لیوان_ته_گنبدی:','id':':gelas_tumbler:','zh':':平底杯:','ru':':стакан_с_виски:','tr':':kısa_ve_geniş_bardak:','ar':':كوب_زجاجي:'},'\U0001F983':{'en':':turkey:','status':fully_qualified,'E':1,'de':':truthahn:','es':':pavo:','fr':':dindon:','ja':':七面鳥:','ko':':칠면조:','pt':':peru:','it':':tacchino:','fa':':بوقلمون:','id':':kalkun:','zh':':火鸡:','ru':':индейка:','tr':':hindi:','ar':':ديك_رومي:'},'\U0001F422':{'en':':turtle:','status':fully_qualified,'E':0.6,'de':':schildkröte:','es':':tortuga:','fr':':tortue:','ja':':カメ:','ko':':거북이:','pt':':tartaruga:','it':':tartaruga:','fa':':لاک_پشت:','id':':kura-kura:','zh':':龟:','ru':':черепаха:','tr':':kaplumbağa:','ar':':سلحفاة:'},'\U0001F567':{'en':':twelve-thirty:','status':fully_qualified,'E':0.7,'alias':[':clock1230:', ':twelve_thirty:'],'variant':True,'de':':12.30_uhr:','es':':doce_y_media:','fr':':midi/minuit_et_demie:','ja':':12時半:','ko':':열두_시_반:','pt':':doze_e_meia:','it':':ore_dodici_e_mezza:','fa':':ساعت_دوازده_و_نیم:','id':':jam_setengah_satu:','zh':':十二点半:','ru':':полпервого:','tr':':on_iki_buçuk:','ar':':الثانية_عشرة_والنصف:'},'\U0001F55B':{'en':':twelve_o’clock:','status':fully_qualified,'E':0.6,'alias':[':clock12:', ':twelve_oclock:'],'variant':True,'de':':ziffernblatt_12.00_uhr:','es':':12_en_punto:','fr':':midi/minuit:','ja':':12時:','ko':':열두_시:','pt':':12_horas:','it':':ore_dodici:','fa':':ساعت_دوازده:','id':':jam_dua_belas_tepat:','zh':':十二点:','ru':':двенадцать_часов:','tr':':saat_12:','ar':':الساعة_الثانية_عشرة:'},'\U0001F42B':{'en':':two-hump_camel:','status':fully_qualified,'E':0.6,'alias':[':camel:', ':two_hump_camel:'],'de':':kamel:','es':':camello:','fr':':chameau:','ja':':フタコブラクダ:','ko':':쌍봉_낙타:','pt':':camelo_com_duas_corcovas:','it':':cammello:','fa':':شتر_دو_کوهان:','id':':unta_berpunuk_dua:','zh':':双峰骆驼:','ru':':двугорбый_верблюд:','tr':':iki_hörgüçlü_deve:','ar':':جمل_ثنائي_السنام:'},'\U0001F55D':{'en':':two-thirty:','status':fully_qualified,'E':0.7,'alias':[':clock230:', ':two_thirty:'],'variant':True,'de':':2.30_uhr:','es':':dos_y_media:','fr':':deux_heures_et_demie:','ja':':2時半:','ko':':두_시_반:','pt':':duas_e_meia:','it':':ore_due_e_mezza:','fa':':ساعت_دو_و_نیم:','id':':jam_setengah_tiga:','zh':':两点半:','ru':':полтретьего:','tr':':iki_buçuk:','ar':':الثانية_والنصف:'},'\U0001F495':{'en':':two_hearts:','status':fully_qualified,'E':0.6,'de':':zwei_herzen:','es':':dos_corazones:','fr':':deux_cœurs:','ja':':2つのハート:','ko':':하트_두_개:','pt':':dois_corações:','it':':due_cuori:','fa':':دو_قلب:','id':':dua_hati:','zh':':两颗心:','ru':':два_сердца:','tr':':iki_kalp:','ar':':قلبان:'},'\U0001F551':{'en':':two_o’clock:','status':fully_qualified,'E':0.6,'alias':[':clock2:', ':two_oclock:'],'variant':True,'de':':2.00_uhr:','es':':2_en_punto:','fr':':deux_heures:','ja':':2時:','ko':':두_시:','pt':':2_horas:','it':':ore_due:','fa':':ساعت_دو:','id':':jam_dua_tepat:','zh':':两点:','ru':':два_часа:','tr':':saat_2:','ar':':الساعة_الثانية:'},'\U00002602\U0000FE0F':{'en':':umbrella:','status':fully_qualified,'E':0.7,'alias':[':open_umbrella:'],'variant':True,'de':':regenschirm:','es':':paraguas:','fr':':parapluie_ouvert:','ja':':傘:','ko':':우산:','pt':':guarda-chuva:','it':':ombrello:','fa':':چتر:','id':':payung:','zh':':伞:','ru':':зонт:','tr':':şemsiye:','ar':':شمسية:'},'\U00002602':{'en':':umbrella:','status':unqualified,'E':0.7,'alias':[':open_umbrella:'],'variant':True,'de':':regenschirm:','es':':paraguas:','fr':':parapluie_ouvert:','ja':':傘:','ko':':우산:','pt':':guarda-chuva:','it':':ombrello:','fa':':چتر:','id':':payung:','zh':':伞:','ru':':зонт:','tr':':şemsiye:','ar':':شمسية:'},'\U000026F1\U0000FE0F':{'en':':umbrella_on_ground:','status':fully_qualified,'E':0.7,'alias':[':parasol_on_ground:'],'variant':True,'de':':aufgestellter_sonnenschirm:','es':':sombrilla_en_la_arena:','fr':':parasol_sur_le_sol:','ja':':パラソル:','ko':':땅에_꽂힌_파라솔:','pt':':guarda-sol:','it':':ombrellone:','fa':':چتر_روی_زمین:','id':':payung_di_tanah:','zh':':阳伞:','ru':':пляжныи_зонт:','tr':':sabit_şemsiye:','ar':':شمسية_على_الأرض:'},'\U000026F1':{'en':':umbrella_on_ground:','status':unqualified,'E':0.7,'alias':[':parasol_on_ground:'],'variant':True,'de':':aufgestellter_sonnenschirm:','es':':sombrilla_en_la_arena:','fr':':parasol_sur_le_sol:','ja':':パラソル:','ko':':땅에_꽂힌_파라솔:','pt':':guarda-sol:','it':':ombrellone:','fa':':چتر_روی_زمین:','id':':payung_di_tanah:','zh':':阳伞:','ru':':пляжный_зонт:','tr':':sabit_şemsiye:','ar':':شمسية_على_الأرض:'},'\U00002614':{'en':':umbrella_with_rain_drops:','status':fully_qualified,'E':0.6,'alias':[':umbrella:'],'variant':True,'de':':regenschirm_im_regen:','es':':paraguas_con_gotas_de_lluvia:','fr':':parapluie_avec_gouttes_de_pluie:','ja':':傘と雨:','ko':':빗방울이_있는_우산:','pt':':sombrinha_na_chuva:','it':':ombrello_con_gocce_di_pioggia:','fa':':چتر_خیس:','id':':payung_dengan_tetesan_hujan:','zh':':雨伞:','ru':':зонт_под_дождем:','tr':':yağmur_damlalı_şemsiye:','ar':':شمسية_وقطرات_مطر:'},'\U0001F612':{'en':':unamused_face:','status':fully_qualified,'E':0.6,'alias':[':unamused:'],'de':':verstimmtes_gesicht:','es':':cara_de_desaprobación:','fr':':visage_blasé:','ja':':しらけた:','ko':':지루해하는_얼굴:','pt':':rosto_aborrecido:','it':':faccina_contrariata:','fa':':کسل:','id':':wajah_bosan:','zh':':不高兴:','ru':':скучает:','tr':':keyifsiz_yüz:','ar':':وجه_غير_مستمتع:'},'\U0001F984':{'en':':unicorn:','status':fully_qualified,'E':1,'alias':[':unicorn_face:'],'de':':einhorn:','es':':unicornio:','fr':':licorne:','ja':':ユニコーンの顔:','ko':':유니콘_얼굴:','pt':':rosto_de_unicórnio:','it':':unicorno:','fa':':تک_شاخ:','id':':unicorn:','zh':':独角兽:','ru':':голова_единорога:','tr':':tek_boynuzlu_at:','ar':':وجه_أحادي_قرن:'},'\U0001F513':{'en':':unlocked:','status':fully_qualified,'E':0.6,'alias':[':unlock:'],'variant':True,'de':':offenes_schloss:','es':':candado_abierto:','fr':':cadenas_ouvert:','ja':':開いた錠:','ko':':열린_자물쇠:','pt':':cadeado_aberto:','it':':lucchetto_aperto:','fa':':قفل_باز:','id':':gembok_terbuka:','zh':':打开的锁:','ru':':открытый_замок:','tr':':açık_kilit:','ar':':قفل_مفتوح:'},'\U00002195\U0000FE0F':{'en':':up-down_arrow:','status':fully_qualified,'E':0.6,'alias':[':arrow_up_down:', ':up_down_arrow:'],'variant':True,'de':':pfeil_nach_oben_und_unten:','es':':flecha_arriba_y_abajo:','fr':':flèche_haut_bas:','ja':':上下矢印:','ko':':상하향_화살표:','pt':':seta_para_cima_e_para_baixo:','it':':freccia_su-giù:','fa':':پیکان_بالا_و_پایین:','id':':tanda_panah_atas_bawah:','zh':':上下箭头:','ru':':стрелка_вверх-вниз:','tr':':yukarı_ve_aşağı_ok:','ar':':سهم_لأعلى_وأسفل:'},'\U00002195':{'en':':up-down_arrow:','status':unqualified,'E':0.6,'alias':[':arrow_up_down:', ':up_down_arrow:'],'variant':True,'de':':pfeil_nach_oben_und_unten:','es':':flecha_arriba_y_abajo:','fr':':flèche_haut_bas:','ja':':上下矢印:','ko':':상하향_화살표:','pt':':seta_para_cima_e_para_baixo:','it':':freccia_su-giù:','fa':':پیکان_بالا_و_پایین:','id':':tanda_panah_atas_bawah:','zh':':上下箭头:','ru':':стрелка_вверх_вниз:','tr':':yukarı_ve_aşağı_ok:','ar':':سهم_لأعلى_وأسفل:'},'\U00002196\U0000FE0F':{'en':':up-left_arrow:','status':fully_qualified,'E':0.6,'alias':[':arrow_upper_left:', ':up_left_arrow:'],'variant':True,'de':':pfeil_nach_links_oben:','es':':flecha_hacia_la_esquina_superior_izquierda:','fr':':flèche_haut_gauche:','ja':':左上矢印:','ko':':좌상향_화살표:','pt':':seta_para_cima_e_para_a_esquerda:','it':':freccia_in_alto_a_sinistra:','fa':':پیکان_بالا_چپ:','id':':tanda_panah_kiri_atas:','zh':':左上箭头:','ru':':стрелка_влево-вверх:','tr':':sol_yukarı_ok:','ar':':سهم_لأعلى_اليسار:'},'\U00002196':{'en':':up-left_arrow:','status':unqualified,'E':0.6,'alias':[':arrow_upper_left:', ':up_left_arrow:'],'variant':True,'de':':pfeil_nach_links_oben:','es':':flecha_hacia_la_esquina_superior_izquierda:','fr':':flèche_haut_gauche:','ja':':左上矢印:','ko':':좌상향_화살표:','pt':':seta_para_cima_e_para_a_esquerda:','it':':freccia_in_alto_a_sinistra:','fa':':پیکان_بالا_چپ:','id':':tanda_panah_kiri_atas:','zh':':左上箭头:','ru':':стрелка_влево_вверх:','tr':':sol_yukarı_ok:','ar':':سهم_لأعلى_اليسار:'},'\U00002197\U0000FE0F':{'en':':up-right_arrow:','status':fully_qualified,'E':0.6,'alias':[':arrow_upper_right:', ':up_right_arrow:'],'variant':True,'de':':pfeil_nach_rechts_oben:','es':':flecha_hacia_la_esquina_superior_derecha:','fr':':flèche_haut_droite:','ja':':右上矢印:','ko':':우상향_화살표:','pt':':seta_para_cima_e_para_a_direita:','it':':freccia_rivolta_verso_destra_che_punta_in_alto:','fa':':پیکان_بالا_راست:','id':':tanda_panah_kanan_atas:','zh':':右上箭头:','ru':':стрелка_вправо-вверх:','tr':':sağ_yukarı_ok:','ar':':سهم_لأعلى_اليمين:'},'\U00002197':{'en':':up-right_arrow:','status':unqualified,'E':0.6,'alias':[':arrow_upper_right:', ':up_right_arrow:'],'variant':True,'de':':pfeil_nach_rechts_oben:','es':':flecha_hacia_la_esquina_superior_derecha:','fr':':flèche_haut_droite:','ja':':右上矢印:','ko':':우상향_화살표:','pt':':seta_para_cima_e_para_a_direita:','it':':freccia_rivolta_verso_destra_che_punta_in_alto:','fa':':پیکان_بالا_راست:','id':':tanda_panah_kanan_atas:','zh':':右上箭头:','ru':':стрелка_вправо_вверх:','tr':':sağ_yukarı_ok:','ar':':سهم_لأعلى_اليمين:'},'\U00002B06\U0000FE0F':{'en':':up_arrow:','status':fully_qualified,'E':0.6,'alias':[':arrow_up:'],'variant':True,'de':':pfeil_nach_oben:','es':':flecha_hacia_arriba:','fr':':flèche_haut:','ja':':上矢印:','ko':':상향_화살표:','pt':':seta_para_cima:','it':':freccia_rivolta_verso_l’alto:','fa':':پیکان_بالا:','id':':tanda_panah_atas:','zh':':向上箭头:','ru':':стрелка_вверх:','tr':':yukarı_ok:','ar':':سهم_لأعلى:'},'\U00002B06':{'en':':up_arrow:','status':unqualified,'E':0.6,'alias':[':arrow_up:'],'variant':True,'de':':pfeil_nach_oben:','es':':flecha_hacia_arriba:','fr':':flèche_haut:','ja':':上矢印:','ko':':상향_화살표:','pt':':seta_para_cima:','it':':freccia_rivolta_verso_l’alto:','fa':':پیکان_بالا:','id':':tanda_panah_atas:','zh':':向上箭头:','ru':':стрелка_вверх:','tr':':yukarı_ok:','ar':':سهم_لأعلى:'},'\U0001F643':{'en':':upside-down_face:','status':fully_qualified,'E':1,'alias':[':upside_down_face:'],'de':':umgekehrtes_gesicht:','es':':cara_al_revés:','fr':':tête_à_l’envers:','ja':':さかさまの顔:','ko':':거꾸로_된_얼굴:','pt':':rosto_de_cabeça_para_baixo:','it':':faccina_sottosopra:','fa':':صورت_وارونه:','id':':wajah_terbalik:','zh':':倒脸:','ru':':вверх_тормашками:','tr':':ters_yüz:','ar':':وجه_مقلوب:'},'\U0001F53C':{'en':':upwards_button:','status':fully_qualified,'E':0.6,'alias':[':arrow_up_small:'],'de':':aufwärts-schaltfläche:','es':':triángulo_hacia_arriba:','fr':':petit_triangle_haut:','ja':':上ボタン:','ko':':위쪽_삼각형:','pt':':botão_apontando_para_cima:','it':':pulsante_a_triangolo_rivolto_verso_l’alto:','fa':':دکمهٔ_بالا:','id':':tombol_naik:','zh':':向上三角形按钮:','ru':':треугольник_острием_вверх:','tr':':yukarı_düğmesi:','ar':':زر_لأعلى:'},'\U0001F9DB':{'en':':vampire:','status':fully_qualified,'E':5,'de':':vampir:','es':':vampiro:','fr':':vampire:','ja':':吸血鬼:','ko':':뱀파이어:','pt':':vampiro:','it':':vampiro:','fa':':خون_آشام:','id':':vampir:','zh':':吸血鬼:','ru':':вампир:','tr':':vampir:','ar':':مصاص_دماء:'},'\U0001F9DB\U0001F3FF':{'en':':vampire_dark_skin_tone:','status':fully_qualified,'E':5,'de':':vampir_dunkle_hautfarbe:','es':':vampiro_tono_de_piel_oscuro:','fr':':vampire_peau_foncée:','ja':':吸血鬼_濃い肌色:','ko':':뱀파이어_검은색_피부:','pt':':vampiro_pele_escura:','it':':vampiro_carnagione_scura:','fa':':خون_آشام_پوست_آبنوسی:','id':':vampir_warna_kulit_gelap:','zh':':吸血鬼_较深肤色:','ru':':вампир_очень_темный_тон_кожи:','tr':':vampir_koyu_cilt_tonu:','ar':':مصاص_دماء_رجل_بشرة_بلون_غامق:'},'\U0001F9DB\U0001F3FB':{'en':':vampire_light_skin_tone:','status':fully_qualified,'E':5,'de':':vampir_helle_hautfarbe:','es':':vampiro_tono_de_piel_claro:','fr':':vampire_peau_claire:','ja':':吸血鬼_薄い肌色:','ko':':뱀파이어_하얀_피부:','pt':':vampiro_pele_clara:','it':':vampiro_carnagione_chiara:','fa':':خون_آشام_پوست_سفید:','id':':vampir_warna_kulit_cerah:','zh':':吸血鬼_较浅肤色:','ru':':вампир_очень_светлый_тон_кожи:','tr':':vampir_açık_cilt_tonu:','ar':':مصاص_دماء_رجل_بشرة_بلون_فاتح:'},'\U0001F9DB\U0001F3FE':{'en':':vampire_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':vampir_mitteldunkle_hautfarbe:','es':':vampiro_tono_de_piel_oscuro_medio:','fr':':vampire_peau_mate:','ja':':吸血鬼_やや濃い肌色:','ko':':뱀파이어_진한_갈색_피부:','pt':':vampiro_pele_morena_escura:','it':':vampiro_carnagione_abbastanza_scura:','fa':':خون_آشام_پوست_گندمی:','id':':vampir_warna_kulit_gelap-sedang:','zh':':吸血鬼_中等深肤色:','ru':':вампир_темный_тон_кожи:','tr':':vampir_orta_koyu_cilt_tonu:','ar':':مصاص_دماء_رجل_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9DB\U0001F3FC':{'en':':vampire_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':vampir_mittelhelle_hautfarbe:','es':':vampiro_tono_de_piel_claro_medio:','fr':':vampire_peau_moyennement_claire:','ja':':吸血鬼_やや薄い肌色:','ko':':뱀파이어_연한_갈색_피부:','pt':':vampiro_pele_morena_clara:','it':':vampiro_carnagione_abbastanza_chiara:','fa':':خون_آشام_پوست_روشن:','id':':vampir_warna_kulit_cerah-sedang:','zh':':吸血鬼_中等浅肤色:','ru':':вампир_светлый_тон_кожи:','tr':':vampir_orta_açık_cilt_tonu:','ar':':مصاص_دماء_رجل_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9DB\U0001F3FD':{'en':':vampire_medium_skin_tone:','status':fully_qualified,'E':5,'de':':vampir_mittlere_hautfarbe:','es':':vampiro_tono_de_piel_medio:','fr':':vampire_peau_légèrement_mate:','ja':':吸血鬼_中間の肌色:','ko':':뱀파이어_갈색_피부:','pt':':vampiro_pele_morena:','it':':vampiro_carnagione_olivastra:','fa':':خون_آشام_پوست_طلایی:','id':':vampir_warna_kulit_sedang:','zh':':吸血鬼_中等肤色:','ru':':вампир_средний_тон_кожи:','tr':':vampir_orta_cilt_tonu:','ar':':مصاص_دماء_رجل_بشرة_بلون_معتدل:'},'\U0001F6A6':{'en':':vertical_traffic_light:','status':fully_qualified,'E':1,'de':':vertikale_verkehrsampel:','es':':semáforo:','fr':':feu_tricolore_vertical:','ja':':信号縦:','ko':':세로_신호등:','pt':':semáforo_vertical:','it':':semaforo_verticale:','fa':':چراغ_راهنمایی_عمودی:','id':':lampu_lalu_lintas_vertikal:','zh':':纵向的红绿灯:','ru':':вертикальный_светофор:','tr':':dikey_trafik_ışıkları:','ar':':إشارة_مرور_رأسية:'},'\U0001F4F3':{'en':':vibration_mode:','status':fully_qualified,'E':0.6,'de':':vibrationsmodus:','es':':modo_vibración:','fr':':mode_vibreur:','ja':':マナーモード:','ko':':진동_모드:','pt':':modo_vibratório:','it':':modalità_vibrazione:','fa':':حالت_لرزش:','id':':mode_getar:','zh':':振动模式:','ru':':режим_вибрации:','tr':':titreşim_modu:','ar':':وضع_اهتزاز:'},'\U0000270C\U0000FE0F':{'en':':victory_hand:','status':fully_qualified,'E':0.6,'alias':[':v:'],'variant':True,'de':':victory-geste:','es':':mano_con_señal_de_victoria:','fr':':v_de_la_victoire:','ja':':vサイン:','ko':':브이_표시:','pt':':mão_em_v_de_vitória:','it':':vittoria:','fa':':علامت_پیروزی:','id':':isyarat_kemenangan:','zh':':胜利手势:','ru':':жест_v:','tr':':zafer_işareti:','ar':':علامة_النصر:'},'\U0000270C':{'en':':victory_hand:','status':unqualified,'E':0.6,'alias':[':v:'],'variant':True,'de':':victory-geste:','es':':mano_con_señal_de_victoria:','fr':':v_de_la_victoire:','ja':':vサイン:','ko':':브이_표시:','pt':':mão_em_v_de_vitória:','it':':vittoria:','fa':':علامت_پیروزی:','id':':isyarat_kemenangan:','zh':':胜利手势:','ru':':жест_V:','tr':':zafer_işareti:','ar':':علامة_النصر:'},'\U0000270C\U0001F3FF':{'en':':victory_hand_dark_skin_tone:','status':fully_qualified,'E':1,'de':':victory-geste_dunkle_hautfarbe:','es':':mano_con_señal_de_victoria_tono_de_piel_oscuro:','fr':':v_de_la_victoire_peau_foncée:','ja':':vサイン_濃い肌色:','ko':':브이_표시_검은색_피부:','pt':':mão_em_v_de_vitória_pele_escura:','it':':vittoria_carnagione_scura:','fa':':علامت_پیروزی_پوست_آبنوسی:','id':':isyarat_kemenangan_warna_kulit_gelap:','zh':':胜利手势_较深肤色:','ru':':жест_V_очень_темный_тон_кожи:','tr':':zafer_işareti_koyu_cilt_tonu:','ar':':علامة_النصر_بشرة_بلون_غامق:'},'\U0000270C\U0001F3FB':{'en':':victory_hand_light_skin_tone:','status':fully_qualified,'E':1,'de':':victory-geste_helle_hautfarbe:','es':':mano_con_señal_de_victoria_tono_de_piel_claro:','fr':':v_de_la_victoire_peau_claire:','ja':':vサイン_薄い肌色:','ko':':브이_표시_하얀_피부:','pt':':mão_em_v_de_vitória_pele_clara:','it':':vittoria_carnagione_chiara:','fa':':علامت_پیروزی_پوست_سفید:','id':':isyarat_kemenangan_warna_kulit_cerah:','zh':':胜利手势_较浅肤色:','ru':':жест_V_очень_светлый_тон_кожи:','tr':':zafer_işareti_açık_cilt_tonu:','ar':':علامة_النصر_بشرة_بلون_فاتح:'},'\U0000270C\U0001F3FE':{'en':':victory_hand_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':victory-geste_mitteldunkle_hautfarbe:','es':':mano_con_señal_de_victoria_tono_de_piel_oscuro_medio:','fr':':v_de_la_victoire_peau_mate:','ja':':vサイン_やや濃い肌色:','ko':':브이_표시_진한_갈색_피부:','pt':':mão_em_v_de_vitória_pele_morena_escura:','it':':vittoria_carnagione_abbastanza_scura:','fa':':علامت_پیروزی_پوست_گندمی:','id':':isyarat_kemenangan_warna_kulit_gelap-sedang:','zh':':胜利手势_中等深肤色:','ru':':жест_V_темный_тон_кожи:','tr':':zafer_işareti_orta_koyu_cilt_tonu:','ar':':علامة_النصر_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0000270C\U0001F3FC':{'en':':victory_hand_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':victory-geste_mittelhelle_hautfarbe:','es':':mano_con_señal_de_victoria_tono_de_piel_claro_medio:','fr':':v_de_la_victoire_peau_moyennement_claire:','ja':':vサイン_やや薄い肌色:','ko':':브이_표시_연한_갈색_피부:','pt':':mão_em_v_de_vitória_pele_morena_clara:','it':':vittoria_carnagione_abbastanza_chiara:','fa':':علامت_پیروزی_پوست_روشن:','id':':isyarat_kemenangan_warna_kulit_cerah-sedang:','zh':':胜利手势_中等浅肤色:','ru':':жест_V_светлый_тон_кожи:','tr':':zafer_işareti_orta_açık_cilt_tonu:','ar':':علامة_النصر_بشرة_بلون_فاتح_ومعتدل:'},'\U0000270C\U0001F3FD':{'en':':victory_hand_medium_skin_tone:','status':fully_qualified,'E':1,'de':':victory-geste_mittlere_hautfarbe:','es':':mano_con_señal_de_victoria_tono_de_piel_medio:','fr':':v_de_la_victoire_peau_légèrement_mate:','ja':':vサイン_中間の肌色:','ko':':브이_표시_갈색_피부:','pt':':mão_em_v_de_vitória_pele_morena:','it':':vittoria_carnagione_olivastra:','fa':':علامت_پیروزی_پوست_طلایی:','id':':isyarat_kemenangan_warna_kulit_sedang:','zh':':胜利手势_中等肤色:','ru':':жест_V_средний_тон_кожи:','tr':':zafer_işareti_orta_cilt_tonu:','ar':':علامة_النصر_بشرة_بلون_معتدل:'},'\U0001F4F9':{'en':':video_camera:','status':fully_qualified,'E':0.6,'variant':True,'de':':videokamera:','es':':videocámara:','fr':':caméscope:','ja':':ビデオカメラ:','ko':':캠코더:','pt':':câmera_de_vídeo:','it':':videocamera:','fa':':دوربین_فیلم_برداری:','id':':kamera_video:','zh':':摄像机:','ru':':видеокамера:','tr':':video_kamera:','ar':':كاميرا_فيديو:'},'\U0001F3AE':{'en':':video_game:','status':fully_qualified,'E':0.6,'variant':True,'de':':gamepad:','es':':mando_de_videoconsola:','fr':':jeu_vidéo:','ja':':テレビゲーム:','ko':':비디오_게임:','pt':':videogame:','it':':gamepad:','fa':':بازی_ویدیویی:','id':':video_game:','zh':':游戏手柄:','ru':':видеоигры:','tr':':video_oyunu:','ar':':لعبة_فيديو:'},'\U0001F4FC':{'en':':videocassette:','status':fully_qualified,'E':0.6,'alias':[':vhs:'],'de':':videokassette:','es':':cinta_de_vídeo:','fr':':cassette_vidéo:','ja':':ビデオテープ:','ko':':비디오테이프:','pt':':videocassete:','it':':videocassetta:','fa':':کاست_ویدیویی:','id':':kaset_video:','zh':':录像带:','ru':':видеокассета:','tr':':videokaset:','ar':':شريط_فيديو:'},'\U0001F3BB':{'en':':violin:','status':fully_qualified,'E':0.6,'de':':geige:','es':':violín:','fr':':violon:','ja':':バイオリン:','ko':':바이올린:','pt':':violino:','it':':violino:','fa':':ویولن:','id':':biola:','zh':':小提琴:','ru':':скрипка:','tr':':keman:','ar':':كمان:'},'\U0001F30B':{'en':':volcano:','status':fully_qualified,'E':0.6,'de':':vulkan:','es':':volcán:','fr':':volcan:','ja':':火山:','ko':':화산:','pt':':vulcão:','it':':vulcano:','fa':':آتشفشان:','id':':gunung_berapi:','zh':':火山:','ru':':вулкан:','tr':':yanardağ:','ar':':بركان:'},'\U0001F3D0':{'en':':volleyball:','status':fully_qualified,'E':1,'de':':volleyball:','es':':pelota_de_voleibol:','fr':':volley-ball:','ja':':バレーボール:','ko':':배구공:','pt':':bola_de_vôlei:','it':':palla_da_pallavolo:','fa':':والیبال:','id':':bola_voli:','zh':':排球:','ru':':волейбол:','tr':':voleybol:','ar':':كرة_طائرة:'},'\U0001F596':{'en':':vulcan_salute:','status':fully_qualified,'E':1,'alias':[':raised_hand_with_part_between_middle_and_ring_fingers:'],'de':':vulkanischer_gruß:','es':':saludo_vulcano:','fr':':salut_vulcain:','ja':':バルカンの挨拶:','ko':':벌칸식_인사:','pt':':saudação_vulcana:','it':':saluto_vulcaniano:','fa':':سلام_ولکان:','id':':salam_vulcan:','zh':':瓦肯举手礼:','ru':':вулканский_салют:','tr':':vulkan_selamı:','ar':':تحية_فولكان:'},'\U0001F596\U0001F3FF':{'en':':vulcan_salute_dark_skin_tone:','status':fully_qualified,'E':1,'de':':vulkanischer_gruß_dunkle_hautfarbe:','es':':saludo_vulcano_tono_de_piel_oscuro:','fr':':salut_vulcain_peau_foncée:','ja':':バルカンの挨拶_濃い肌色:','ko':':벌칸식_인사_검은색_피부:','pt':':saudação_vulcana_pele_escura:','it':':saluto_vulcaniano_carnagione_scura:','fa':':سلام_ولکان_پوست_آبنوسی:','id':':salam_vulcan_warna_kulit_gelap:','zh':':瓦肯举手礼_较深肤色:','ru':':вулканский_салют_очень_темный_тон_кожи:','tr':':vulkan_selamı_koyu_cilt_tonu:','ar':':تحية_فولكان_بشرة_بلون_غامق:'},'\U0001F596\U0001F3FB':{'en':':vulcan_salute_light_skin_tone:','status':fully_qualified,'E':1,'de':':vulkanischer_gruß_helle_hautfarbe:','es':':saludo_vulcano_tono_de_piel_claro:','fr':':salut_vulcain_peau_claire:','ja':':バルカンの挨拶_薄い肌色:','ko':':벌칸식_인사_하얀_피부:','pt':':saudação_vulcana_pele_clara:','it':':saluto_vulcaniano_carnagione_chiara:','fa':':سلام_ولکان_پوست_سفید:','id':':salam_vulcan_warna_kulit_cerah:','zh':':瓦肯举手礼_较浅肤色:','ru':':вулканский_салют_очень_светлый_тон_кожи:','tr':':vulkan_selamı_açık_cilt_tonu:','ar':':تحية_فولكان_بشرة_بلون_فاتح:'},'\U0001F596\U0001F3FE':{'en':':vulcan_salute_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':vulkanischer_gruß_mitteldunkle_hautfarbe:','es':':saludo_vulcano_tono_de_piel_oscuro_medio:','fr':':salut_vulcain_peau_mate:','ja':':バルカンの挨拶_やや濃い肌色:','ko':':벌칸식_인사_진한_갈색_피부:','pt':':saudação_vulcana_pele_morena_escura:','it':':saluto_vulcaniano_carnagione_abbastanza_scura:','fa':':سلام_ولکان_پوست_گندمی:','id':':salam_vulcan_warna_kulit_gelap-sedang:','zh':':瓦肯举手礼_中等深肤色:','ru':':вулканский_салют_темный_тон_кожи:','tr':':vulkan_selamı_orta_koyu_cilt_tonu:','ar':':تحية_فولكان_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F596\U0001F3FC':{'en':':vulcan_salute_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':vulkanischer_gruß_mittelhelle_hautfarbe:','es':':saludo_vulcano_tono_de_piel_claro_medio:','fr':':salut_vulcain_peau_moyennement_claire:','ja':':バルカンの挨拶_やや薄い肌色:','ko':':벌칸식_인사_연한_갈색_피부:','pt':':saudação_vulcana_pele_morena_clara:','it':':saluto_vulcaniano_carnagione_abbastanza_chiara:','fa':':سلام_ولکان_پوست_روشن:','id':':salam_vulcan_warna_kulit_cerah-sedang:','zh':':瓦肯举手礼_中等浅肤色:','ru':':вулканский_салют_светлый_тон_кожи:','tr':':vulkan_selamı_orta_açık_cilt_tonu:','ar':':تحية_فولكان_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F596\U0001F3FD':{'en':':vulcan_salute_medium_skin_tone:','status':fully_qualified,'E':1,'de':':vulkanischer_gruß_mittlere_hautfarbe:','es':':saludo_vulcano_tono_de_piel_medio:','fr':':salut_vulcain_peau_légèrement_mate:','ja':':バルカンの挨拶_中間の肌色:','ko':':벌칸식_인사_갈색_피부:','pt':':saudação_vulcana_pele_morena:','it':':saluto_vulcaniano_carnagione_olivastra:','fa':':سلام_ولکان_پوست_طلایی:','id':':salam_vulcan_warna_kulit_sedang:','zh':':瓦肯举手礼_中等肤色:','ru':':вулканский_салют_средний_тон_кожи:','tr':':vulkan_selamı_orta_cilt_tonu:','ar':':تحية_فولكان_بشرة_بلون_معتدل:'},'\U0001F9C7':{'en':':waffle:','status':fully_qualified,'E':12,'de':':waffel:','es':':gofre:','fr':':gaufre:','ja':':ワッフル:','ko':':와플:','pt':':waffle:','it':':waffle:','fa':':وافل:','id':':wafel:','zh':':华夫饼:','ru':':вафля:','tr':':waffle:','ar':':وافل:'},'\U0001F318':{'en':':waning_crescent_moon:','status':fully_qualified,'E':1,'de':':letztes_mondviertel:','es':':luna_menguante:','fr':':lune_décroissante:','ja':':有明月:','ko':':그믐달_하현:','pt':':lua_minguante_côncava:','it':':luna_calante:','fa':':هلال_کاهنده_ماه:','id':':bulan_sabit_akhir:','zh':':残月:','ru':':старая_луна:','tr':':ince_ay:','ar':':هلال_آخر_الشهر:'},'\U0001F316':{'en':':waning_gibbous_moon:','status':fully_qualified,'E':1,'de':':drittes_mondviertel:','es':':luna_gibosa_menguante:','fr':':lune_gibbeuse_décroissante:','ja':':寝待月:','ko':':하현달:','pt':':lua_minguante_convexa:','it':':luna_gibbosa_calante:','fa':':کوژماه_کاهنده:','id':':bulan_cembung_akhir:','zh':':亏凸月:','ru':':убывающая_луна:','tr':':küçülen_ay:','ar':':قمر_أحدب_متناقص:'},'\U000026A0\U0000FE0F':{'en':':warning:','status':fully_qualified,'E':0.6,'variant':True,'de':':warnung:','es':':advertencia:','fr':':symbole_d’avertissement:','ja':':警告:','ko':':주의:','pt':':aviso:','it':':simbolo_di_avviso:','fa':':هشدار:','id':':peringatan:','zh':':警告:','ru':':значок_предупреждения:','tr':':uyarı_işareti:','ar':':إشارة_تحذير:'},'\U000026A0':{'en':':warning:','status':unqualified,'E':0.6,'variant':True,'de':':warnung:','es':':advertencia:','fr':':symbole_d’avertissement:','ja':':警告:','ko':':주의:','pt':':aviso:','it':':simbolo_di_avviso:','fa':':هشدار:','id':':peringatan:','zh':':警告:','ru':':значок_предупреждения:','tr':':uyarı_işareti:','ar':':إشارة_تحذير:'},'\U0001F5D1\U0000FE0F':{'en':':wastebasket:','status':fully_qualified,'E':0.7,'variant':True,'de':':papierkorb:','es':':papelera:','fr':':corbeille_à_papiers:','ja':':ごみ箱:','ko':':쓰레기통:','pt':':lixeira:','it':':cestino:','fa':':سطل_کاغذ_باطله:','id':':keranjang_sampah:','zh':':垃圾桶:','ru':':корзина_для_мусора:','tr':':çöp_sepeti:','ar':':سلة_نفايات:'},'\U0001F5D1':{'en':':wastebasket:','status':unqualified,'E':0.7,'variant':True,'de':':papierkorb:','es':':papelera:','fr':':corbeille_à_papiers:','ja':':ごみ箱:','ko':':쓰레기통:','pt':':lixeira:','it':':cestino:','fa':':سطل_کاغذ_باطله:','id':':keranjang_sampah:','zh':':垃圾桶:','ru':':корзина_для_мусора:','tr':':çöp_sepeti:','ar':':سلة_نفايات:'},'\U0000231A':{'en':':watch:','status':fully_qualified,'E':0.6,'variant':True,'de':':armbanduhr:','es':':reloj:','fr':':montre:','ja':':腕時計:','ko':':시계:','pt':':relógio_de_pulso:','it':':orologio:','fa':':ساعت_مچی:','id':':jam_tangan:','zh':':手表:','ru':':наручные_часы:','tr':':kol_saati:','ar':':ساعة:'},'\U0001F403':{'en':':water_buffalo:','status':fully_qualified,'E':1,'de':':wasserbüffel:','es':':búfalo_de_agua:','fr':':buffle:','ja':':水牛:','ko':':물소:','pt':':búfalo-asiático:','it':':bufalo_d’acqua:','fa':':بوفالوی_آبی:','id':':kerbau:','zh':':水牛:','ru':':буйвол:','tr':':manda:','ar':':جاموس_مائي:'},'\U0001F6BE':{'en':':water_closet:','status':fully_qualified,'E':0.6,'alias':[':wc:'],'de':':wc:','es':':aseos:','fr':':wc:','ja':':wc:','ko':':더블유씨:','pt':':wc:','it':':simbolo_del_wc:','fa':':سرویس_بهداشتی:','id':':kloset_air:','zh':':厕所:','ru':':туалет:','tr':':wc:','ar':':حمام:'},'\U0001F52B':{'en':':water_pistol:','status':fully_qualified,'E':0.6,'alias':[':gun:'],'de':':wasserpistole:','es':':pistola_de_agua:','fr':':pistolet_à_eau:','ja':':水鉄砲:','ko':':물총:','pt':':pistola_d’água:','it':':pistola_ad_acqua:','fa':':تفنگ_آب_پاش:','id':':pistol_air:','zh':':水枪:','ru':':водяной_пистолет:','tr':':su_tabancası:','ar':':مسدس_ماء:'},'\U0001F30A':{'en':':water_wave:','status':fully_qualified,'E':0.6,'alias':[':ocean:'],'de':':welle:','es':':ola_de_mar:','fr':':vague:','ja':':波:','ko':':파도:','pt':':onda:','it':':onda:','fa':':موج_آب:','id':':ombak:','zh':':浪花:','ru':':волна:','tr':':dalga:','ar':':موجة:'},'\U0001F349':{'en':':watermelon:','status':fully_qualified,'E':0.6,'de':':wassermelone:','es':':sandía:','fr':':pastèque:','ja':':スイカ:','ko':':수박:','pt':':melancia:','it':':anguria:','fa':':هندوانه:','id':':semangka:','zh':':西瓜:','ru':':арбуз:','tr':':karpuz:','ar':':بطيخ:'},'\U0001F44B':{'en':':waving_hand:','status':fully_qualified,'E':0.6,'alias':[':wave:'],'de':':winkende_hand:','es':':mano_saludando:','fr':':signe_de_la_main:','ja':':手を振る:','ko':':흔드는_손:','pt':':mão_acenando:','it':':mano_che_saluta:','fa':':تکان_دادن_دست:','id':':tangan_melambai:','zh':':挥手:','ru':':машет_рукой:','tr':':el_sallama:','ar':':تلويح_باليد:'},'\U0001F44B\U0001F3FF':{'en':':waving_hand_dark_skin_tone:','status':fully_qualified,'E':1,'de':':winkende_hand_dunkle_hautfarbe:','es':':mano_saludando_tono_de_piel_oscuro:','fr':':signe_de_la_main_peau_foncée:','ja':':手を振る_濃い肌色:','ko':':흔드는_손_검은색_피부:','pt':':mão_acenando_pele_escura:','it':':mano_che_saluta_carnagione_scura:','fa':':تکان_دادن_دست_پوست_آبنوسی:','id':':tangan_melambai_warna_kulit_gelap:','zh':':挥手_较深肤色:','ru':':машет_рукой_очень_темный_тон_кожи:','tr':':el_sallama_koyu_cilt_tonu:','ar':':تلويح_باليد_بشرة_بلون_غامق:'},'\U0001F44B\U0001F3FB':{'en':':waving_hand_light_skin_tone:','status':fully_qualified,'E':1,'de':':winkende_hand_helle_hautfarbe:','es':':mano_saludando_tono_de_piel_claro:','fr':':signe_de_la_main_peau_claire:','ja':':手を振る_薄い肌色:','ko':':흔드는_손_하얀_피부:','pt':':mão_acenando_pele_clara:','it':':mano_che_saluta_carnagione_chiara:','fa':':تکان_دادن_دست_پوست_سفید:','id':':tangan_melambai_warna_kulit_cerah:','zh':':挥手_较浅肤色:','ru':':машет_рукой_очень_светлый_тон_кожи:','tr':':el_sallama_açık_cilt_tonu:','ar':':تلويح_باليد_بشرة_بلون_فاتح:'},'\U0001F44B\U0001F3FE':{'en':':waving_hand_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':winkende_hand_mitteldunkle_hautfarbe:','es':':mano_saludando_tono_de_piel_oscuro_medio:','fr':':signe_de_la_main_peau_mate:','ja':':手を振る_やや濃い肌色:','ko':':흔드는_손_진한_갈색_피부:','pt':':mão_acenando_pele_morena_escura:','it':':mano_che_saluta_carnagione_abbastanza_scura:','fa':':تکان_دادن_دست_پوست_گندمی:','id':':tangan_melambai_warna_kulit_gelap-sedang:','zh':':挥手_中等深肤色:','ru':':машет_рукой_темный_тон_кожи:','tr':':el_sallama_orta_koyu_cilt_tonu:','ar':':تلويح_باليد_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F44B\U0001F3FC':{'en':':waving_hand_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':winkende_hand_mittelhelle_hautfarbe:','es':':mano_saludando_tono_de_piel_claro_medio:','fr':':signe_de_la_main_peau_moyennement_claire:','ja':':手を振る_やや薄い肌色:','ko':':흔드는_손_연한_갈색_피부:','pt':':mão_acenando_pele_morena_clara:','it':':mano_che_saluta_carnagione_abbastanza_chiara:','fa':':تکان_دادن_دست_پوست_روشن:','id':':tangan_melambai_warna_kulit_cerah-sedang:','zh':':挥手_中等浅肤色:','ru':':машет_рукой_светлый_тон_кожи:','tr':':el_sallama_orta_açık_cilt_tonu:','ar':':تلويح_باليد_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F44B\U0001F3FD':{'en':':waving_hand_medium_skin_tone:','status':fully_qualified,'E':1,'de':':winkende_hand_mittlere_hautfarbe:','es':':mano_saludando_tono_de_piel_medio:','fr':':signe_de_la_main_peau_légèrement_mate:','ja':':手を振る_中間の肌色:','ko':':흔드는_손_갈색_피부:','pt':':mão_acenando_pele_morena:','it':':mano_che_saluta_carnagione_olivastra:','fa':':تکان_دادن_دست_پوست_طلایی:','id':':tangan_melambai_warna_kulit_sedang:','zh':':挥手_中等肤色:','ru':':машет_рукой_средний_тон_кожи:','tr':':el_sallama_orta_cilt_tonu:','ar':':تلويح_باليد_بشرة_بلون_معتدل:'},'\U00003030\U0000FE0F':{'en':':wavy_dash:','status':fully_qualified,'E':0.6,'variant':True,'de':':wellenlinie:','es':':guion_ondulado:','fr':':ligne_ondulée:','ja':':波線:','ko':':물결_모양_대시:','pt':':travessão_ondulado:','it':':trattino_ondulato:','fa':':خط_موج_دار:','id':':garis_bergelombang:','zh':':波浪型破折号:','ru':':волнистая_линия:','tr':':dalgalı_çizgi:','ar':':شرطة_مموجة:'},'\U00003030':{'en':':wavy_dash:','status':unqualified,'E':0.6,'variant':True,'de':':wellenlinie:','es':':guion_ondulado:','fr':':ligne_ondulée:','ja':':波線:','ko':':물결_모양_대시:','pt':':travessão_ondulado:','it':':trattino_ondulato:','fa':':خط_موج_دار:','id':':garis_bergelombang:','zh':':波浪型破折号:','ru':':волнистая_линия:','tr':':dalgalı_çizgi:','ar':':شرطة_مموجة:'},'\U0001F312':{'en':':waxing_crescent_moon:','status':fully_qualified,'E':1,'de':':erstes_mondviertel:','es':':luna_creciente:','fr':':lune_croissante:','ja':':初月:','ko':':초승달_상현:','pt':':lua_crescente_côncava:','it':':luna_crescente:','fa':':هلال_فزاینده_ماه:','id':':bulan_sabit_awal:','zh':':娥眉月:','ru':':молодая_луна:','tr':':büyüyen_ay:','ar':':هلال_بداية_الشهر:'},'\U0001F314':{'en':':waxing_gibbous_moon:','status':fully_qualified,'E':0.6,'alias':[':moon:'],'de':':zweites_mondviertel:','es':':luna_gibosa_creciente:','fr':':lune_gibbeuse_croissante:','ja':':十三夜月:','ko':':상현달:','pt':':lua_crescente_convexa:','it':':luna_gibbosa_crescente:','fa':':کوژماه_افزاینده:','id':':bulan_cembung_awal:','zh':':盈凸月:','ru':':прибывающая_луна:','tr':':geniş_ay:','ar':':قمر_أحدب_متزايد:'},'\U0001F640':{'en':':weary_cat:','status':fully_qualified,'E':0.6,'alias':[':scream_cat:'],'de':':erschöpfte_katze:','es':':gato_asustado:','fr':':chat_fatigué:','ja':':絶望する猫:','ko':':절규하는_고양이_얼굴:','pt':':rosto_de_gato_desolado:','it':':gatto_esterrefatto:','fa':':گربهٔ_بهت_زده:','id':':wajah_kucing_ketakutan:','zh':':疲倦的猫:','ru':':кот_в_шоке:','tr':':korkuyla_bağıran_kedi:','ar':':وجه_قطة_مجهد:'},'\U0001F629':{'en':':weary_face:','status':fully_qualified,'E':0.6,'alias':[':weary:'],'de':':erschöpftes_gesicht:','es':':cara_agotada:','fr':':visage_épuisé:','ja':':あきらめ:','ko':':지친_얼굴:','pt':':rosto_desolado:','it':':faccina_esausta:','fa':':ازپاافتاده:','id':':wajah_terlalu_lelah:','zh':':累死了:','ru':':усталость:','tr':':çok_yorgun:','ar':':وجه_مجهد:'},'\U0001F492':{'en':':wedding:','status':fully_qualified,'E':0.6,'de':':hochzeit:','es':':iglesia_celebrando_boda:','fr':':mariage:','ja':':結婚式:','ko':':결혼식:','pt':':capela_de_casamento:','it':':chiesa_per_matrimonio:','fa':':ازدواج:','id':':pernikahan:','zh':':婚礼:','ru':':свадьба:','tr':':düğün:','ar':':كنيسة_زفاف:'},'\U0001F40B':{'en':':whale:','status':fully_qualified,'E':1,'alias':[':whale2:'],'de':':wal:','es':':ballena:','fr':':baleine:','ja':':クジラ:','ko':':고래:','pt':':baleia:','it':':balena:','fa':':نهنگ:','id':':ikan_paus:','zh':':鲸鱼:','ru':':кит:','tr':':balina:','ar':':حوت:'},'\U0001F6DE':{'en':':wheel:','status':fully_qualified,'E':14,'de':':autorad:','es':':rueda:','fr':':roue:','ja':':ホイール:','ko':':휠:','pt':':roda:','it':':ruota:','fa':':چرخ:','id':':roda:','zh':':车轮:','ru':':автомобильное_колесо:','tr':':tekerlek:','ar':':عجلة:'},'\U00002638\U0000FE0F':{'en':':wheel_of_dharma:','status':fully_qualified,'E':0.7,'variant':True,'de':':dharma-rad:','es':':rueda_del_dharma:','fr':':roue_du_dharma:','ja':':法輪:','ko':':진리의_수레바퀴:','pt':':roda_do_dharma:','it':':ruota_del_dharma:','fa':':چرخ_دارما:','id':':roda_dharma:','zh':':法轮:','ru':':колесо_дхармы:','tr':':darma_çemberi:','ar':':عجلة_دارما:'},'\U00002638':{'en':':wheel_of_dharma:','status':unqualified,'E':0.7,'variant':True,'de':':dharma-rad:','es':':rueda_del_dharma:','fr':':roue_du_dharma:','ja':':法輪:','ko':':진리의_수레바퀴:','pt':':roda_do_dharma:','it':':ruota_del_dharma:','fa':':چرخ_دارما:','id':':roda_dharma:','zh':':法轮:','ru':':колесо_дхармы:','tr':':darma_çemberi:','ar':':عجلة_دارما:'},'\U0000267F':{'en':':wheelchair_symbol:','status':fully_qualified,'E':0.6,'alias':[':wheelchair:'],'variant':True,'de':':symbol_rollstuhl:','es':':símbolo_de_silla_de_ruedas:','fr':':symbole_accès_handicapés:','ja':':車椅子:','ko':':휠체어:','pt':':símbolo_de_cadeira_de_rodas:','it':':simbolo_della_disabilità:','fa':':علامت_صندلی_چرخ_دار:','id':':simbol_kursi_roda:','zh':':轮椅标识:','ru':':значок_«для_инвалидов»:','tr':':tekerlekli_sandalye_sembolü:','ar':':كرسي_متحرك:'},'\U0001F9AF':{'en':':white_cane:','status':fully_qualified,'E':12,'alias':[':probing_cane:'],'de':':blindenstock:','es':':bastón:','fr':':canne_blanche:','ja':':白杖:','ko':':시각장애인_지팡이:','pt':':bengala_para_cegos:','it':':bastone_per_non_vedenti:','fa':':عصای_سفید:','id':':tongkat_tunanetra:','zh':':盲杖:','ru':':белая_трость:','tr':':görme_engelli_bastonu:','ar':':عصا_سير:'},'\U000026AA':{'en':':white_circle:','status':fully_qualified,'E':0.6,'variant':True,'de':':weißer_punkt:','es':':círculo_blanco:','fr':':disque_blanc:','ja':':白い丸:','ko':':흰색_원:','pt':':círculo_branco:','it':':cerchio_bianco:','fa':':دایره_سفید:','id':':lingkaran_putih:','zh':':白色圆:','ru':':белый_круг:','tr':':beyaz_daire:','ar':':دائرة_بيضاء:'},'\U00002755':{'en':':white_exclamation_mark:','status':fully_qualified,'E':0.6,'alias':[':grey_exclamation:'],'variant':True,'de':':weißes_ausrufezeichen:','es':':exclamación_blanca:','fr':':point_d’exclamation_blanc:','ja':':白い感嘆符:','ko':':하얀색_느낌표:','pt':':ponto_de_exclamação_branco:','it':':punto_esclamativo_bianco:','fa':':علامت_تعجب_سفید:','id':':tanda_seru_putih:','zh':':白色感叹号:','ru':':белый_восклицательный_знак:','tr':':beyaz_ünlem_işareti:','ar':':علامة_تعجب_بيضاء:'},'\U0001F3F3\U0000FE0F':{'en':':white_flag:','status':fully_qualified,'E':0.7,'alias':[':waving_white_flag:'],'variant':True,'de':':weiße_flagge:','es':':bandera_blanca:','fr':':drapeau_blanc:','ja':':白旗:','ko':':펄럭이는_흰색_깃발:','pt':':bandeira_branca:','it':':bandiera_bianca:','fa':':پرچم_سفید:','id':':bendera_putih:','zh':':举白旗:','ru':':белыи_флаг:','tr':':beyaz_bayrak:','ar':':علم_أبيض_يرفرف:'},'\U0001F3F3':{'en':':white_flag:','status':unqualified,'E':0.7,'alias':[':waving_white_flag:'],'variant':True,'de':':weiße_flagge:','es':':bandera_blanca:','fr':':drapeau_blanc:','ja':':白旗:','ko':':펄럭이는_흰색_깃발:','pt':':bandeira_branca:','it':':bandiera_bianca:','fa':':پرچم_سفید:','id':':bendera_putih:','zh':':白旗:','ru':':белый_флаг:','tr':':beyaz_bayrak:','ar':':علم_أبيض_يرفرف:'},'\U0001F4AE':{'en':':white_flower:','status':fully_qualified,'E':0.6,'de':':blumenstempel:','es':':flor_blanca:','fr':':fleur_blanche:','ja':':大変よくできました:','ko':':흰_꽃:','pt':':flor_branca:','it':':fiore_bianco:','fa':':گل_سفید:','id':':bunga_putih:','zh':':白花:','ru':':белый_цветок:','tr':':beyaz_çiçek:','ar':':وردة_بيضاء:'},'\U0001F9B3':{'en':':white_hair:','status':component,'E':11,'de':':weißes_haar:','es':':pelo_blanco:','fr':':cheveux_blancs:','ja':':白髪:','ko':':백발:','pt':':cabelo_branco:','it':':capelli_bianchi:','fa':':موسفید:','id':':rambut_putih:','zh':':白发:','ru':':седые_волосы:','tr':':beyaz_saç:','ar':':شعر_أبيض:'},'\U0001F90D':{'en':':white_heart:','status':fully_qualified,'E':12,'de':':weißes_herz:','es':':corazón_blanco:','fr':':cœur_blanc:','ja':':白いハート:','ko':':흰색_하트:','pt':':coração_branco:','it':':cuore_bianco:','fa':':قلب_سفید:','id':':hati_putih:','zh':':白心:','ru':':белое_сердце:','tr':':beyaz_kalp:','ar':':قلب_أبيض:'},'\U00002B1C':{'en':':white_large_square:','status':fully_qualified,'E':0.6,'variant':True,'de':':großes_weißes_quadrat:','es':':cuadrado_blanco_grande:','fr':':grand_carré_blanc:','ja':':白四角特大:','ko':':큰_흰색_사각형:','pt':':quadrado_branco_grande:','it':':quadrato_bianco_grande:','fa':':مربع_سفید_بزرگ:','id':':persegi_besar_putih:','zh':':白线大方框:','ru':':большой_белый_квадрат:','tr':':büyük_beyaz_kare:','ar':':مربع_كبير_أبيض:'},'\U000025FD':{'en':':white_medium-small_square:','status':fully_qualified,'E':0.6,'alias':[':white_medium_small_square:'],'variant':True,'de':':mittelkleines_weißes_quadrat:','es':':cuadrado_blanco_mediano-pequeño:','fr':':carré_petit_moyen_blanc:','ja':':白四角中:','ko':':중간_작은_흰색_사각형:','pt':':quadrado_branco_médio_menor:','it':':quadrato_bianco_medio-piccolo:','fa':':مربع_سفید_نسبتاً_کوچک:','id':':persegi_kecil_medium_putih:','zh':':白色中小方块:','ru':':небольшой_белый_квадрат:','tr':':küçük-orta_beyaz_kare:','ar':':مربع_صغير_متوسط_أبيض:'},'\U000025FB\U0000FE0F':{'en':':white_medium_square:','status':fully_qualified,'E':0.6,'variant':True,'de':':mittelgroßes_weißes_quadrat:','es':':cuadrado_blanco_mediano:','fr':':carré_moyen_blanc:','ja':':白四角大:','ko':':중간_흰색_사각형:','pt':':quadrado_branco_médio:','it':':quadrato_bianco_medio:','fa':':مربع_سفید_متوسط:','id':':persegi_medium_putih:','zh':':白色中方块:','ru':':белыи_квадрат:','tr':':orta_boy_beyaz_kare:','ar':':مربع_متوسط_أبيض:'},'\U000025FB':{'en':':white_medium_square:','status':unqualified,'E':0.6,'variant':True,'de':':mittelgroßes_weißes_quadrat:','es':':cuadrado_blanco_mediano:','fr':':carré_moyen_blanc:','ja':':白四角大:','ko':':중간_흰색_사각형:','pt':':quadrado_branco_médio:','it':':quadrato_bianco_medio:','fa':':مربع_سفید_متوسط:','id':':persegi_medium_putih:','zh':':白色中方块:','ru':':белый_квадрат:','tr':':orta_boy_beyaz_kare:','ar':':مربع_متوسط_أبيض:'},'\U00002754':{'en':':white_question_mark:','status':fully_qualified,'E':0.6,'alias':[':grey_question:'],'variant':True,'de':':weißes_fragezeichen:','es':':interrogación_blanca:','fr':':point_d’interrogation_blanc:','ja':':白い疑問符:','ko':':하얀색_물음표:','pt':':ponto_de_interrogação_branco:','it':':punto_interrogativo_bianco:','fa':':علامت_سؤال_سفید:','id':':tanda_tanya_putih:','zh':':白色问号:','ru':':белый_вопросительный_знак:','tr':':beyaz_soru_işareti:','ar':':علامة_استفهام_بيضاء:'},'\U000025AB\U0000FE0F':{'en':':white_small_square:','status':fully_qualified,'E':0.6,'variant':True,'de':':kleines_weißes_quadrat:','es':':cuadrado_blanco_pequeño:','fr':':petit_carré_blanc:','ja':':白四角小:','ko':':작은_흰색_사각형:','pt':':quadrado_branco_pequeno:','it':':quadrato_bianco_piccolo:','fa':':مربع_سفید_کوچک:','id':':persegi_kecil_putih:','zh':':白色小方块:','ru':':белыи_квадратик:','tr':':küçük_beyaz_kare:','ar':':مربع_صغير_أبيض:'},'\U000025AB':{'en':':white_small_square:','status':unqualified,'E':0.6,'variant':True,'de':':kleines_weißes_quadrat:','es':':cuadrado_blanco_pequeño:','fr':':petit_carré_blanc:','ja':':白四角小:','ko':':작은_흰색_사각형:','pt':':quadrado_branco_pequeno:','it':':quadrato_bianco_piccolo:','fa':':مربع_سفید_کوچک:','id':':persegi_kecil_putih:','zh':':白色小方块:','ru':':белый_квадратик:','tr':':küçük_beyaz_kare:','ar':':مربع_صغير_أبيض:'},'\U0001F533':{'en':':white_square_button:','status':fully_qualified,'E':0.6,'de':':weiße_quadratische_schaltfläche:','es':':botón_cuadrado_con_borde_blanco:','fr':':carré_blanc:','ja':':白枠四角:','ko':':테두리가_하얀_사각_버튼:','pt':':botão_quadrado_branco:','it':':tasto_quadrato_nero_con_bordo_bianco:','fa':':دکمۀ_مربع_سفید:','id':':tombol_persegi_putih:','zh':':白色方形按钮:','ru':':кнопка_с_белой_рамкой:','tr':':beyaz_kare_düğme:','ar':':زر_مربع_أبيض:'},'\U0001F940':{'en':':wilted_flower:','status':fully_qualified,'E':3,'de':':welke_blume:','es':':flor_marchita:','fr':':fleur_fanée:','ja':':しおれた花:','ko':':시든_꽃:','pt':':flor_murcha:','it':':fiore_appassito:','fa':':گل_پژمرده:','id':':bunga_layu:','zh':':枯萎的花:','ru':':увядший_цветок:','tr':':solmuş_çiçek:','ar':':وردة_ذابلة:'},'\U0001F390':{'en':':wind_chime:','status':fully_qualified,'E':0.6,'de':':japanisches_windspiel:','es':':campanilla_de_viento:','fr':':carillon_éolien:','ja':':風鈴:','ko':':풍경:','pt':':carrilhão_de_vento:','it':':campana_al_vento:','fa':':باد_سِنج:','id':':lonceng_angin:','zh':':风铃:','ru':':музыка_ветра:','tr':':rüzgar_çanı:','ar':':جرس_الريح:'},'\U0001F32C\U0000FE0F':{'en':':wind_face:','status':fully_qualified,'E':0.7,'alias':[':wind_blowing_face:'],'variant':True,'de':':wind:','es':':cara_de_viento:','fr':':vent_avec_visage:','ja':':顔のある風:','ko':':얼굴이_있는_바람:','pt':':rosto_de_vento:','it':':vento:','fa':':صورت_باد:','id':':wajah_tertiup_angin:','zh':':大风:','ru':':дует_ветер:','tr':':rüzgar:','ar':':رياح:'},'\U0001F32C':{'en':':wind_face:','status':unqualified,'E':0.7,'alias':[':wind_blowing_face:'],'variant':True,'de':':wind:','es':':cara_de_viento:','fr':':vent_avec_visage:','ja':':顔のある風:','ko':':얼굴이_있는_바람:','pt':':rosto_de_vento:','it':':vento:','fa':':صورت_باد:','id':':wajah_tertiup_angin:','zh':':大风:','ru':':дует_ветер:','tr':':rüzgar:','ar':':رياح:'},'\U0001FA9F':{'en':':window:','status':fully_qualified,'E':13,'de':':fenster:','es':':ventana:','fr':':fenêtre:','ja':':窓:','ko':':창문:','pt':':janela:','it':':finestra:','fa':':پنجره:','id':':jendela:','zh':':窗户:','ru':':окно:','tr':':pencere:','ar':':نافذة:'},'\U0001F377':{'en':':wine_glass:','status':fully_qualified,'E':0.6,'de':':weinglas:','es':':copa_de_vino:','fr':':verre_de_vin:','ja':':ワイングラス:','ko':':와인잔:','pt':':vinho:','it':':bicchiere_di_vino:','fa':':جام_شراب:','id':':gelas_anggur:','zh':':葡萄酒:','ru':':бокал_вина:','tr':':şarap_kadehi:','ar':':كأس_شراب:'},'\U0001FABD':{'en':':wing:','status':fully_qualified,'E':15,'de':':flügel:','es':':ala:','fr':':aile:','ja':':翼:','ko':':날개:','pt':':asa:','it':':ala:','fa':':بال:','id':':sayap:','zh':':翅膀:','ru':':крыло:','tr':':kanat:','ar':':جناح:'},'\U0001F609':{'en':':winking_face:','status':fully_qualified,'E':0.6,'alias':[':wink:'],'de':':zwinkerndes_gesicht:','es':':cara_guiñando_el_ojo:','fr':':visage_faisant_un_clin_d’œil:','ja':':ウインク:','ko':':윙크하는_얼굴:','pt':':rosto_com_olho_piscando:','it':':faccina_che_fa_l’occhiolino:','fa':':چشمک:','id':':wajah_berkedip:','zh':':眨眼:','ru':':подмигивает:','tr':':göz_kırpan_yüz:','ar':':وجه_يغمز:'},'\U0001F61C':{'en':':winking_face_with_tongue:','status':fully_qualified,'E':0.6,'alias':[':stuck_out_tongue_winking_eye:'],'de':':zwinkerndes_gesicht_mit_herausgestreckter_zunge:','es':':cara_sacando_la_lengua_y_guiñando_un_ojo:','fr':':visage_qui_tire_la_langue_et_fait_un_clin_d’œil:','ja':':あっかんべー:','ko':':윙크하면서_혀를_내민_얼굴:','pt':':rosto_piscando_e_com_língua_para_fora:','it':':faccina_che_fa_l’occhiolino_e_mostra_la_lingua:','fa':':صورت_با_چشمک_و_زبان_بیرون_آمده:','id':':wajah_menjulurkan_lidah_dan_mengedipkan_mata:','zh':':单眼吐舌:','ru':':показывает_язык_и_подмигивает:','tr':':dili_dışarda_göz_kırpma:','ar':':وجه_بلسان_متدل_وعين_غامزة:'},'\U0001F6DC':{'en':':wireless:','status':fully_qualified,'E':15,'de':':wlan:','es':':wifi:','fr':':sans_fil:','ja':':wi-fi:','ko':':무선:','pt':':sem_fio:','it':':wireless:','fa':':بی_سیم:','id':':nirkabel:','zh':':无线:','ru':':беспроводная_сеть:','tr':':kablosuz:','ar':':اتصال_لاسلكي:'},'\U0001F43A':{'en':':wolf:','status':fully_qualified,'E':0.6,'de':':wolf:','es':':lobo:','fr':':loup:','ja':':オオカミの顔:','ko':':늑대_얼굴:','pt':':rosto_de_lobo:','it':':lupo:','fa':':گرگ:','id':':serigala:','zh':':狼:','ru':':морда_волка:','tr':':kurt:','ar':':وجه_ذئب:'},'\U0001F469':{'en':':woman:','status':fully_qualified,'E':0.6,'de':':frau:','es':':mujer:','fr':':femme:','ja':':女性:','ko':':여자:','pt':':mulher:','it':':donna:','fa':':زن:','id':':wanita:','zh':':女人:','ru':':женщина:','tr':':kadın:','ar':':سيدة:'},'\U0001F46B':{'en':':woman_and_man_holding_hands:','status':fully_qualified,'E':0.6,'alias':[':couple:'],'de':':mann_und_frau_halten_hände:','es':':mujer_y_hombre_de_la_mano:','fr':':femme_et_homme_se_tenant_la_main:','ja':':手をつなぐ男女:','ko':':손을_잡고_있는_남녀:','pt':':homem_e_mulher_de_mãos_dadas:','it':':uomo_e_donna_che_si_tengono_per_mano:','fa':':زن_و_مرد_دست_در_دست:','id':':pria_dan_wanita_bergandengan:','zh':':手拉手的一男一女:','ru':':мужчина_и_женщина:','tr':':el_ele_tutuşan_erkek_ve_kadın:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما:'},'\U0001F46B\U0001F3FF':{'en':':woman_and_man_holding_hands_dark_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_dunkle_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_oscuro:','fr':':femme_et_homme_se_tenant_la_main_peau_foncée:','ja':':手をつなぐ男女_濃い肌色:','ko':':손을_잡고_있는_남녀_검은색_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_escura:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_scura:','fa':':زن_و_مرد_دست_در_دست_پوست_آبنوسی:','id':':pria_dan_wanita_bergandengan_warna_kulit_gelap:','zh':':手拉手的一男一女_较深肤色:','ru':':мужчина_и_женщина_очень_темный_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_koyu_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FB':{'en':':woman_and_man_holding_hands_dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_dunkle_hautfarbe_helle_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_oscuro_y_tono_de_piel_claro:','fr':':femme_et_homme_se_tenant_la_main_peau_foncée_et_peau_claire:','ja':':手をつなぐ男女_濃い肌色_薄い肌色:','ko':':손을_잡고_있는_남녀_검은색_피부_하얀_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_escura_e_pele_clara:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_scura_e_carnagione_chiara:','fa':':زن_و_مرد_دست_در_دست_پوست_آبنوسی_پوست_سفید:','id':':pria_dan_wanita_bergandengan_warna_kulit_gelap_warna_kulit_cerah:','zh':':手拉手的一男一女_较深肤色较浅肤色:','ru':':мужчина_и_женщина_очень_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_koyu_cilt_tonu_açık_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FE':{'en':':woman_and_man_holding_hands_dark_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_dunkle_hautfarbe_mitteldunkle_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_oscuro_y_tono_de_piel_oscuro_medio:','fr':':femme_et_homme_se_tenant_la_main_peau_foncée_et_peau_mate:','ja':':手をつなぐ男女_濃い肌色_やや濃い肌色:','ko':':손을_잡고_있는_남녀_검은색_피부_진한_갈색_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_escura_e_pele_morena_escura:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':زن_و_مرد_دست_در_دست_پوست_آبنوسی_پوست_گندمی:','id':':pria_dan_wanita_bergandengan_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':手拉手的一男一女_较深肤色中等深肤色:','ru':':мужчина_и_женщина_очень_темный_тон_кожи_темный_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FC':{'en':':woman_and_man_holding_hands_dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_dunkle_hautfarbe_mittelhelle_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_oscuro_y_tono_de_piel_claro_medio:','fr':':femme_et_homme_se_tenant_la_main_peau_foncée_et_peau_moyennement_claire:','ja':':手をつなぐ男女_濃い肌色_やや薄い肌色:','ko':':손을_잡고_있는_남녀_검은색_피부_연한_갈색_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_escura_e_pele_morena_clara:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':زن_و_مرد_دست_در_دست_پوست_آبنوسی_پوست_روشن:','id':':pria_dan_wanita_bergandengan_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':手拉手的一男一女_较深肤色中等浅肤色:','ru':':мужчина_и_женщина_очень_темный_тон_кожи_светлый_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FD':{'en':':woman_and_man_holding_hands_dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_dunkle_hautfarbe_mittlere_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_oscuro_y_tono_de_piel_medio:','fr':':femme_et_homme_se_tenant_la_main_peau_foncée_et_peau_légèrement_mate:','ja':':手をつなぐ男女_濃い肌色_中間の肌色:','ko':':손을_잡고_있는_남녀_검은색_피부_갈색_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_escura_e_pele_morena:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_scura_e_carnagione_olivastra:','fa':':زن_و_مرد_دست_در_دست_پوست_آبنوسی_پوست_طلایی:','id':':pria_dan_wanita_bergandengan_warna_kulit_gelap_warna_kulit_sedang:','zh':':手拉手的一男一女_较深肤色中等肤色:','ru':':мужчина_и_женщина_очень_темный_тон_кожи_средний_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_koyu_cilt_tonu_orta_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F46B\U0001F3FB':{'en':':woman_and_man_holding_hands_light_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_helle_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_claro:','fr':':femme_et_homme_se_tenant_la_main_peau_claire:','ja':':手をつなぐ男女_薄い肌色:','ko':':손을_잡고_있는_남녀_하얀_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_clara:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_chiara:','fa':':زن_و_مرد_دست_در_دست_پوست_سفید:','id':':pria_dan_wanita_bergandengan_warna_kulit_cerah:','zh':':手拉手的一男一女_较浅肤色:','ru':':мужчина_и_женщина_очень_светлый_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_açık_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FF':{'en':':woman_and_man_holding_hands_light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_helle_hautfarbe_dunkle_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_claro_y_tono_de_piel_oscuro:','fr':':femme_et_homme_se_tenant_la_main_peau_claire_et_peau_foncée:','ja':':手をつなぐ男女_薄い肌色_濃い肌色:','ko':':손을_잡고_있는_남녀_하얀_피부_검은색_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_clara_e_pele_escura:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_chiara_e_carnagione_scura:','fa':':زن_و_مرد_دست_در_دست_پوست_سفید_پوست_آبنوسی:','id':':pria_dan_wanita_bergandengan_warna_kulit_cerah_warna_kulit_gelap:','zh':':手拉手的一男一女_较浅肤色较深肤色:','ru':':мужчина_и_женщина_очень_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_açık_cilt_tonu_koyu_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FE':{'en':':woman_and_man_holding_hands_light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_helle_hautfarbe_mitteldunkle_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_claro_y_tono_de_piel_oscuro_medio:','fr':':femme_et_homme_se_tenant_la_main_peau_claire_et_peau_mate:','ja':':手をつなぐ男女_薄い肌色_やや濃い肌色:','ko':':손을_잡고_있는_남녀_하얀_피부_진한_갈색_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_clara_e_pele_morena_escura:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':زن_و_مرد_دست_در_دست_پوست_سفید_پوست_گندمی:','id':':pria_dan_wanita_bergandengan_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':手拉手的一男一女_较浅肤色中等深肤色:','ru':':мужчина_и_женщина_очень_светлый_тон_кожи_темный_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FC':{'en':':woman_and_man_holding_hands_light_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_helle_hautfarbe_mittelhelle_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_claro_y_tono_de_piel_claro_medio:','fr':':femme_et_homme_se_tenant_la_main_peau_claire_et_peau_moyennement_claire:','ja':':手をつなぐ男女_薄い肌色_やや薄い肌色:','ko':':손을_잡고_있는_남녀_하얀_피부_연한_갈색_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_clara_e_pele_morena_clara:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':زن_و_مرد_دست_در_دست_پوست_سفید_پوست_روشن:','id':':pria_dan_wanita_bergandengan_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':手拉手的一男一女_较浅肤色中等浅肤色:','ru':':мужчина_и_женщина_очень_светлый_тон_кожи_светлый_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FD':{'en':':woman_and_man_holding_hands_light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_helle_hautfarbe_mittlere_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_claro_y_tono_de_piel_medio:','fr':':femme_et_homme_se_tenant_la_main_peau_claire_et_peau_légèrement_mate:','ja':':手をつなぐ男女_薄い肌色_中間の肌色:','ko':':손을_잡고_있는_남녀_하얀_피부_갈색_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_clara_e_pele_morena:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_chiara_e_carnagione_olivastra:','fa':':زن_و_مرد_دست_در_دست_پوست_سفید_پوست_طلایی:','id':':pria_dan_wanita_bergandengan_warna_kulit_cerah_warna_kulit_sedang:','zh':':手拉手的一男一女_较浅肤色中等肤色:','ru':':мужчина_и_женщина_очень_светлый_тон_кожи_средний_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_açık_cilt_tonu_orta_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F46B\U0001F3FE':{'en':':woman_and_man_holding_hands_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_mitteldunkle_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_oscuro_medio:','fr':':femme_et_homme_se_tenant_la_main_peau_mate:','ja':':手をつなぐ男女_やや濃い肌色:','ko':':손을_잡고_있는_남녀_진한_갈색_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_morena_escura:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_abbastanza_scura:','fa':':زن_و_مرد_دست_در_دست_پوست_گندمی:','id':':pria_dan_wanita_bergandengan_warna_kulit_gelap-sedang:','zh':':手拉手的一男一女_中等深肤色:','ru':':мужчина_и_женщина_темный_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_orta_koyu_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FF':{'en':':woman_and_man_holding_hands_medium-dark_skin_tone_dark_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_mitteldunkle_hautfarbe_dunkle_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_oscuro_medio_y_tono_de_piel_oscuro:','fr':':femme_et_homme_se_tenant_la_main_peau_mate_et_peau_foncée:','ja':':手をつなぐ男女_やや濃い肌色_濃い肌色:','ko':':손을_잡고_있는_남녀_진한_갈색_피부_검은색_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_morena_escura_e_pele_escura:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':زن_و_مرد_دست_در_دست_پوست_گندمی_پوست_آبنوسی:','id':':pria_dan_wanita_bergandengan_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':手拉手的一男一女_中等深肤色较深肤色:','ru':':мужчина_и_женщина_темный_тон_кожи_очень_темный_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FB':{'en':':woman_and_man_holding_hands_medium-dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_mitteldunkle_hautfarbe_helle_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_oscuro_medio_y_tono_de_piel_claro:','fr':':femme_et_homme_se_tenant_la_main_peau_mate_et_peau_claire:','ja':':手をつなぐ男女_やや濃い肌色_薄い肌色:','ko':':손을_잡고_있는_남녀_진한_갈색_피부_하얀_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_morena_escura_e_pele_clara:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':زن_و_مرد_دست_در_دست_پوست_گندمی_پوست_سفید:','id':':pria_dan_wanita_bergandengan_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':手拉手的一男一女_中等深肤色较浅肤色:','ru':':мужчина_и_женщина_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FC':{'en':':woman_and_man_holding_hands_medium-dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_mitteldunkle_hautfarbe_mittelhelle_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_oscuro_medio_y_tono_de_piel_claro_medio:','fr':':femme_et_homme_se_tenant_la_main_peau_mate_et_peau_moyennement_claire:','ja':':手をつなぐ男女_やや濃い肌色_やや薄い肌色:','ko':':손을_잡고_있는_남녀_진한_갈색_피부_연한_갈색_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_morena_escura_e_pele_morena_clara:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':زن_و_مرد_دست_در_دست_پوست_گندمی_پوست_روشن:','id':':pria_dan_wanita_bergandengan_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':手拉手的一男一女_中等深肤色中等浅肤色:','ru':':мужчина_и_женщина_темный_тон_кожи_светлый_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FD':{'en':':woman_and_man_holding_hands_medium-dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_mitteldunkle_hautfarbe_mittlere_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_oscuro_medio_y_tono_de_piel_medio:','fr':':femme_et_homme_se_tenant_la_main_peau_mate_et_peau_légèrement_mate:','ja':':手をつなぐ男女_やや濃い肌色_中間の肌色:','ko':':손을_잡고_있는_남녀_진한_갈색_피부_갈색_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_morena_escura_e_pele_morena:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':زن_و_مرد_دست_در_دست_پوست_گندمی_پوست_طلایی:','id':':pria_dan_wanita_bergandengan_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':手拉手的一男一女_中等深肤色中等肤色:','ru':':мужчина_и_женщина_темный_тон_кожи_средний_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F46B\U0001F3FC':{'en':':woman_and_man_holding_hands_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_mittelhelle_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_claro_medio:','fr':':femme_et_homme_se_tenant_la_main_peau_moyennement_claire:','ja':':手をつなぐ男女_やや薄い肌色:','ko':':손을_잡고_있는_남녀_연한_갈색_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_morena_clara:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_abbastanza_chiara:','fa':':زن_و_مرد_دست_در_دست_پوست_روشن:','id':':pria_dan_wanita_bergandengan_warna_kulit_cerah-sedang:','zh':':手拉手的一男一女_中等浅肤色:','ru':':мужчина_и_женщина_светлый_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_orta_açık_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FF':{'en':':woman_and_man_holding_hands_medium-light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_mittelhelle_hautfarbe_dunkle_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_claro_medio_y_tono_de_piel_oscuro:','fr':':femme_et_homme_se_tenant_la_main_peau_moyennement_claire_et_peau_foncée:','ja':':手をつなぐ男女_やや薄い肌色_濃い肌色:','ko':':손을_잡고_있는_남녀_연한_갈색_피부_검은색_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_morena_clara_e_pele_escura:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':زن_و_مرد_دست_در_دست_پوست_روشن_پوست_آبنوسی:','id':':pria_dan_wanita_bergandengan_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':手拉手的一男一女_中等浅肤色较深肤色:','ru':':мужчина_и_женщина_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FB':{'en':':woman_and_man_holding_hands_medium-light_skin_tone_light_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_mittelhelle_hautfarbe_helle_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_claro_medio_y_tono_de_piel_claro:','fr':':femme_et_homme_se_tenant_la_main_peau_moyennement_claire_et_peau_claire:','ja':':手をつなぐ男女_やや薄い肌色_薄い肌色:','ko':':손을_잡고_있는_남녀_연한_갈색_피부_하얀_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_morena_clara_e_pele_clara:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':زن_و_مرد_دست_در_دست_پوست_روشن_پوست_سفید:','id':':pria_dan_wanita_bergandengan_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':手拉手的一男一女_中等浅肤色较浅肤色:','ru':':мужчина_и_женщина_светлый_тон_кожи_очень_светлый_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FE':{'en':':woman_and_man_holding_hands_medium-light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_mittelhelle_hautfarbe_mitteldunkle_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_claro_medio_y_tono_de_piel_oscuro_medio:','fr':':femme_et_homme_se_tenant_la_main_peau_moyennement_claire_et_peau_mate:','ja':':手をつなぐ男女_やや薄い肌色_やや濃い肌色:','ko':':손을_잡고_있는_남녀_연한_갈색_피부_진한_갈색_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_morena_clara_e_pele_morena_escura:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':زن_و_مرد_دست_در_دست_پوست_روشن_پوست_گندمی:','id':':pria_dan_wanita_bergandengan_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':手拉手的一男一女_中等浅肤色中等深肤色:','ru':':мужчина_и_женщина_светлый_тон_кожи_темный_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FD':{'en':':woman_and_man_holding_hands_medium-light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_mittelhelle_hautfarbe_mittlere_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_claro_medio_y_tono_de_piel_medio:','fr':':femme_et_homme_se_tenant_la_main_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':手をつなぐ男女_やや薄い肌色_中間の肌色:','ko':':손을_잡고_있는_남녀_연한_갈색_피부_갈색_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_morena_clara_e_pele_morena:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':زن_و_مرد_دست_در_دست_پوست_روشن_پوست_طلایی:','id':':pria_dan_wanita_bergandengan_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':手拉手的一男一女_中等浅肤色中等肤色:','ru':':мужчина_и_женщина_светлый_тон_кожи_средний_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F46B\U0001F3FD':{'en':':woman_and_man_holding_hands_medium_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_mittlere_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_medio:','fr':':femme_et_homme_se_tenant_la_main_peau_légèrement_mate:','ja':':手をつなぐ男女_中間の肌色:','ko':':손을_잡고_있는_남녀_갈색_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_morena:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_olivastra:','fa':':زن_و_مرد_دست_در_دست_پوست_طلایی:','id':':pria_dan_wanita_bergandengan_warna_kulit_sedang:','zh':':手拉手的一男一女_中等肤色:','ru':':мужчина_и_женщина_средний_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_orta_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FF':{'en':':woman_and_man_holding_hands_medium_skin_tone_dark_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_mittlere_hautfarbe_dunkle_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_medio_y_tono_de_piel_oscuro:','fr':':femme_et_homme_se_tenant_la_main_peau_légèrement_mate_et_peau_foncée:','ja':':手をつなぐ男女_中間の肌色_濃い肌色:','ko':':손을_잡고_있는_남녀_갈색_피부_검은색_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_morena_e_pele_escura:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_olivastra_e_carnagione_scura:','fa':':زن_و_مرد_دست_در_دست_پوست_طلایی_پوست_آبنوسی:','id':':pria_dan_wanita_bergandengan_warna_kulit_sedang_warna_kulit_gelap:','zh':':手拉手的一男一女_中等肤色较深肤色:','ru':':мужчина_и_женщина_средний_тон_кожи_очень_темный_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_orta_cilt_tonu_koyu_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FB':{'en':':woman_and_man_holding_hands_medium_skin_tone_light_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_mittlere_hautfarbe_helle_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_medio_y_tono_de_piel_claro:','fr':':femme_et_homme_se_tenant_la_main_peau_légèrement_mate_et_peau_claire:','ja':':手をつなぐ男女_中間の肌色_薄い肌色:','ko':':손을_잡고_있는_남녀_갈색_피부_하얀_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_morena_e_pele_clara:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_olivastra_e_carnagione_chiara:','fa':':زن_و_مرد_دست_در_دست_پوست_طلایی_پوست_سفید:','id':':pria_dan_wanita_bergandengan_warna_kulit_sedang_warna_kulit_cerah:','zh':':手拉手的一男一女_中等肤色较浅肤色:','ru':':мужчина_и_женщина_средний_тон_кожи_очень_светлый_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_orta_cilt_tonu_açık_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FE':{'en':':woman_and_man_holding_hands_medium_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_mittlere_hautfarbe_mitteldunkle_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_medio_y_tono_de_piel_oscuro_medio:','fr':':femme_et_homme_se_tenant_la_main_peau_légèrement_mate_et_peau_mate:','ja':':手をつなぐ男女_中間の肌色_やや濃い肌色:','ko':':손을_잡고_있는_남녀_갈색_피부_진한_갈색_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_morena_e_pele_morena_escura:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':زن_و_مرد_دست_در_دست_پوست_طلایی_پوست_گندمی:','id':':pria_dan_wanita_bergandengan_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':手拉手的一男一女_中等肤色中等深肤色:','ru':':мужчина_и_женщина_средний_тон_кожи_темный_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F468\U0001F3FC':{'en':':woman_and_man_holding_hands_medium_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':mann_und_frau_halten_hände_mittlere_hautfarbe_mittelhelle_hautfarbe:','es':':mujer_y_hombre_de_la_mano_tono_de_piel_medio_y_tono_de_piel_claro_medio:','fr':':femme_et_homme_se_tenant_la_main_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':手をつなぐ男女_中間の肌色_やや薄い肌色:','ko':':손을_잡고_있는_남녀_갈색_피부_연한_갈색_피부:','pt':':homem_e_mulher_de_mãos_dadas_pele_morena_e_pele_morena_clara:','it':':uomo_e_donna_che_si_tengono_per_mano_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':زن_و_مرد_دست_در_دست_پوست_طلایی_پوست_روشن:','id':':pria_dan_wanita_bergandengan_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':手拉手的一男一女_中等肤色中等浅肤色:','ru':':мужчина_и_женщина_средний_тон_кожи_светлый_тон_кожи:','tr':':el_ele_tutuşan_erkek_ve_kadın_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':رجل_وامرأة_يمسكان_بيد_بعضهما_بشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0000200D\U0001F3A8':{'en':':woman_artist:','status':fully_qualified,'E':4,'de':':künstlerin:','es':':artista_mujer:','fr':':artiste_femme:','ja':':女性の芸術家:','ko':':여자_화가:','pt':':artista_plástica:','it':':artista_donna:','fa':':هنرمند_زن:','id':':seniman_wanita:','zh':':女艺术家:','ru':':художница:','tr':':kadın_ressam:','ar':':رسامة:'},'\U0001F469\U0001F3FF\U0000200D\U0001F3A8':{'en':':woman_artist_dark_skin_tone:','status':fully_qualified,'E':4,'de':':künstlerin_dunkle_hautfarbe:','es':':artista_mujer_tono_de_piel_oscuro:','fr':':artiste_femme_peau_foncée:','ja':':女性の芸術家_濃い肌色:','ko':':여자_화가_검은색_피부:','pt':':artista_plástica_pele_escura:','it':':artista_donna_carnagione_scura:','fa':':هنرمند_زن_پوست_آبنوسی:','id':':seniman_wanita_warna_kulit_gelap:','zh':':女艺术家_较深肤色:','ru':':художница_очень_темный_тон_кожи:','tr':':kadın_ressam_koyu_cilt_tonu:','ar':':رسامة_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F3A8':{'en':':woman_artist_light_skin_tone:','status':fully_qualified,'E':4,'de':':künstlerin_helle_hautfarbe:','es':':artista_mujer_tono_de_piel_claro:','fr':':artiste_femme_peau_claire:','ja':':女性の芸術家_薄い肌色:','ko':':여자_화가_하얀_피부:','pt':':artista_plástica_pele_clara:','it':':artista_donna_carnagione_chiara:','fa':':هنرمند_زن_پوست_سفید:','id':':seniman_wanita_warna_kulit_cerah:','zh':':女艺术家_较浅肤色:','ru':':художница_очень_светлый_тон_кожи:','tr':':kadın_ressam_açık_cilt_tonu:','ar':':رسامة_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F3A8':{'en':':woman_artist_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':künstlerin_mitteldunkle_hautfarbe:','es':':artista_mujer_tono_de_piel_oscuro_medio:','fr':':artiste_femme_peau_mate:','ja':':女性の芸術家_やや濃い肌色:','ko':':여자_화가_진한_갈색_피부:','pt':':artista_plástica_pele_morena_escura:','it':':artista_donna_carnagione_abbastanza_scura:','fa':':هنرمند_زن_پوست_گندمی:','id':':seniman_wanita_warna_kulit_gelap-sedang:','zh':':女艺术家_中等深肤色:','ru':':художница_темный_тон_кожи:','tr':':kadın_ressam_orta_koyu_cilt_tonu:','ar':':رسامة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F3A8':{'en':':woman_artist_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':künstlerin_mittelhelle_hautfarbe:','es':':artista_mujer_tono_de_piel_claro_medio:','fr':':artiste_femme_peau_moyennement_claire:','ja':':女性の芸術家_やや薄い肌色:','ko':':여자_화가_연한_갈색_피부:','pt':':artista_plástica_pele_morena_clara:','it':':artista_donna_carnagione_abbastanza_chiara:','fa':':هنرمند_زن_پوست_روشن:','id':':seniman_wanita_warna_kulit_cerah-sedang:','zh':':女艺术家_中等浅肤色:','ru':':художница_светлый_тон_кожи:','tr':':kadın_ressam_orta_açık_cilt_tonu:','ar':':رسامة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F3A8':{'en':':woman_artist_medium_skin_tone:','status':fully_qualified,'E':4,'de':':künstlerin_mittlere_hautfarbe:','es':':artista_mujer_tono_de_piel_medio:','fr':':artiste_femme_peau_légèrement_mate:','ja':':女性の芸術家_中間の肌色:','ko':':여자_화가_갈색_피부:','pt':':artista_plástica_pele_morena:','it':':artista_donna_carnagione_olivastra:','fa':':هنرمند_زن_پوست_طلایی:','id':':seniman_wanita_warna_kulit_sedang:','zh':':女艺术家_中等肤色:','ru':':художница_средний_тон_кожи:','tr':':kadın_ressam_orta_cilt_tonu:','ar':':رسامة_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U0001F680':{'en':':woman_astronaut:','status':fully_qualified,'E':4,'de':':astronautin:','es':':astronauta_mujer:','fr':':astronaute_femme:','ja':':女性宇宙飛行士:','ko':':여자_우주비행사:','pt':':astronauta_mulher:','it':':astronauta_donna:','fa':':فضانورد_زن:','id':':antariksawan_wanita:','zh':':女宇航员:','ru':':женщина_космонавт:','tr':':kadın_astronot:','ar':':رائدة_فضاء:'},'\U0001F469\U0001F3FF\U0000200D\U0001F680':{'en':':woman_astronaut_dark_skin_tone:','status':fully_qualified,'E':4,'de':':astronautin_dunkle_hautfarbe:','es':':astronauta_mujer_tono_de_piel_oscuro:','fr':':astronaute_femme_peau_foncée:','ja':':女性宇宙飛行士_濃い肌色:','ko':':여자_우주비행사_검은색_피부:','pt':':astronauta_mulher_pele_escura:','it':':astronauta_donna_carnagione_scura:','fa':':فضانورد_زن_پوست_آبنوسی:','id':':antariksawan_wanita_warna_kulit_gelap:','zh':':女宇航员_较深肤色:','ru':':женщина_космонавт_очень_темный_тон_кожи:','tr':':kadın_astronot_koyu_cilt_tonu:','ar':':رائدة_فضاء_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F680':{'en':':woman_astronaut_light_skin_tone:','status':fully_qualified,'E':4,'de':':astronautin_helle_hautfarbe:','es':':astronauta_mujer_tono_de_piel_claro:','fr':':astronaute_femme_peau_claire:','ja':':女性宇宙飛行士_薄い肌色:','ko':':여자_우주비행사_하얀_피부:','pt':':astronauta_mulher_pele_clara:','it':':astronauta_donna_carnagione_chiara:','fa':':فضانورد_زن_پوست_سفید:','id':':antariksawan_wanita_warna_kulit_cerah:','zh':':女宇航员_较浅肤色:','ru':':женщина_космонавт_очень_светлый_тон_кожи:','tr':':kadın_astronot_açık_cilt_tonu:','ar':':رائدة_فضاء_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F680':{'en':':woman_astronaut_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':astronautin_mitteldunkle_hautfarbe:','es':':astronauta_mujer_tono_de_piel_oscuro_medio:','fr':':astronaute_femme_peau_mate:','ja':':女性宇宙飛行士_やや濃い肌色:','ko':':여자_우주비행사_진한_갈색_피부:','pt':':astronauta_mulher_pele_morena_escura:','it':':astronauta_donna_carnagione_abbastanza_scura:','fa':':فضانورد_زن_پوست_گندمی:','id':':antariksawan_wanita_warna_kulit_gelap-sedang:','zh':':女宇航员_中等深肤色:','ru':':женщина_космонавт_темный_тон_кожи:','tr':':kadın_astronot_orta_koyu_cilt_tonu:','ar':':رائدة_فضاء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F680':{'en':':woman_astronaut_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':astronautin_mittelhelle_hautfarbe:','es':':astronauta_mujer_tono_de_piel_claro_medio:','fr':':astronaute_femme_peau_moyennement_claire:','ja':':女性宇宙飛行士_やや薄い肌色:','ko':':여자_우주비행사_연한_갈색_피부:','pt':':astronauta_mulher_pele_morena_clara:','it':':astronauta_donna_carnagione_abbastanza_chiara:','fa':':فضانورد_زن_پوست_روشن:','id':':antariksawan_wanita_warna_kulit_cerah-sedang:','zh':':女宇航员_中等浅肤色:','ru':':женщина_космонавт_светлый_тон_кожи:','tr':':kadın_astronot_orta_açık_cilt_tonu:','ar':':رائدة_فضاء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F680':{'en':':woman_astronaut_medium_skin_tone:','status':fully_qualified,'E':4,'de':':astronautin_mittlere_hautfarbe:','es':':astronauta_mujer_tono_de_piel_medio:','fr':':astronaute_femme_peau_légèrement_mate:','ja':':女性宇宙飛行士_中間の肌色:','ko':':여자_우주비행사_갈색_피부:','pt':':astronauta_mulher_pele_morena:','it':':astronauta_donna_carnagione_olivastra:','fa':':فضانورد_زن_پوست_طلایی:','id':':antariksawan_wanita_warna_kulit_sedang:','zh':':女宇航员_中等肤色:','ru':':женщина_космонавт_средний_тон_кожи:','tr':':kadın_astronot_orta_cilt_tonu:','ar':':رائدة_فضاء_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U0001F9B2':{'en':':woman_bald:','status':fully_qualified,'E':11,'alias':[':bald_woman:'],'de':':frau_glatze:','es':':mujer_sin_pelo:','fr':':femme_chauve:','ja':':女性_はげ頭:','ko':':여자_대머리:','pt':':mulher_careca:','it':':donna_calvo:','fa':':زن_تاس:','id':':wanita_gundul:','zh':':女人_秃顶:','ru':':женщина_без_волос:','tr':':kadın_kel:','ar':':سيدة_أصلع:'},'\U0001F9D4\U0000200D\U00002640\U0000FE0F':{'en':':woman_beard:','status':fully_qualified,'E':13.1,'de':':frau_bart:','es':':mujer_barba:','fr':':femme_barbue:','ja':':あごひげの女性:','ko':':수염_난_여자:','pt':':mulher_barba:','it':':donna_con_la_barba:','fa':':زن_ریش:','id':':wanita_janggut:','zh':':有络腮胡子的女人:','ru':':бородатая_женщина:','tr':':kadın_sakal:','ar':':امرأة_بلحية:'},'\U0001F9D4\U0000200D\U00002640':{'en':':woman_beard:','status':minimally_qualified,'E':13.1,'de':':frau_bart:','es':':mujer_barba:','fr':':femme_barbue:','ja':':あごひげの女性:','ko':':수염_난_여자:','pt':':mulher_barba:','it':':donna_con_la_barba:','fa':':زن_ریش:','id':':wanita_janggut:','zh':':有络腮胡子的女人:','ru':':бородатая_женщина:','tr':':kadın_sakal:','ar':':امرأة_بلحية:'},'\U0001F6B4\U0000200D\U00002640\U0000FE0F':{'en':':woman_biking:','status':fully_qualified,'E':4,'alias':[':biking_woman:'],'de':':radfahrerin:','es':':mujer_en_bicicleta:','fr':':cycliste_femme:','ja':':自転車に乗る女:','ko':':자전거_타는_여자:','pt':':mulher_ciclista:','it':':ciclista_donna:','fa':':دوچرخه_سوار_زن:','id':':wanita_bersepeda:','zh':':女生骑自行车:','ru':':женщина_на_велосипеде:','tr':':bisiklet_süren_kadın:','ar':':راكبة_دراجة:'},'\U0001F6B4\U0000200D\U00002640':{'en':':woman_biking:','status':minimally_qualified,'E':4,'alias':[':biking_woman:'],'de':':radfahrerin:','es':':mujer_en_bicicleta:','fr':':cycliste_femme:','ja':':自転車に乗る女:','ko':':자전거_타는_여자:','pt':':mulher_ciclista:','it':':ciclista_donna:','fa':':دوچرخه_سوار_زن:','id':':wanita_bersepeda:','zh':':女生骑自行车:','ru':':женщина_на_велосипеде:','tr':':bisiklet_süren_kadın:','ar':':راكبة_دراجة:'},'\U0001F6B4\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_biking_dark_skin_tone:','status':fully_qualified,'E':4,'de':':radfahrerin_dunkle_hautfarbe:','es':':mujer_en_bicicleta_tono_de_piel_oscuro:','fr':':cycliste_femme_peau_foncée:','ja':':自転車に乗る女_濃い肌色:','ko':':자전거_타는_여자_검은색_피부:','pt':':mulher_ciclista_pele_escura:','it':':ciclista_donna_carnagione_scura:','fa':':دوچرخه_سوار_زن_پوست_آبنوسی:','id':':wanita_bersepeda_warna_kulit_gelap:','zh':':女生骑自行车_较深肤色:','ru':':женщина_на_велосипеде_очень_темныи_тон_кожи:','tr':':bisiklet_süren_kadın_koyu_cilt_tonu:','ar':':راكبة_دراجة_بشرة_بلون_غامق:'},'\U0001F6B4\U0001F3FF\U0000200D\U00002640':{'en':':woman_biking_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':radfahrerin_dunkle_hautfarbe:','es':':mujer_en_bicicleta_tono_de_piel_oscuro:','fr':':cycliste_femme_peau_foncée:','ja':':自転車に乗る女_濃い肌色:','ko':':자전거_타는_여자_검은색_피부:','pt':':mulher_ciclista_pele_escura:','it':':ciclista_donna_carnagione_scura:','fa':':دوچرخه_سوار_زن_پوست_آبنوسی:','id':':wanita_bersepeda_warna_kulit_gelap:','zh':':女生骑自行车_较深肤色:','ru':':женщина_на_велосипеде_очень_темный_тон_кожи:','tr':':bisiklet_süren_kadın_koyu_cilt_tonu:','ar':':راكبة_دراجة_بشرة_بلون_غامق:'},'\U0001F6B4\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_biking_light_skin_tone:','status':fully_qualified,'E':4,'de':':radfahrerin_helle_hautfarbe:','es':':mujer_en_bicicleta_tono_de_piel_claro:','fr':':cycliste_femme_peau_claire:','ja':':自転車に乗る女_薄い肌色:','ko':':자전거_타는_여자_하얀_피부:','pt':':mulher_ciclista_pele_clara:','it':':ciclista_donna_carnagione_chiara:','fa':':دوچرخه_سوار_زن_پوست_سفید:','id':':wanita_bersepeda_warna_kulit_cerah:','zh':':女生骑自行车_较浅肤色:','ru':':женщина_на_велосипеде_очень_светлыи_тон_кожи:','tr':':bisiklet_süren_kadın_açık_cilt_tonu:','ar':':راكبة_دراجة_بشرة_بلون_فاتح:'},'\U0001F6B4\U0001F3FB\U0000200D\U00002640':{'en':':woman_biking_light_skin_tone:','status':minimally_qualified,'E':4,'de':':radfahrerin_helle_hautfarbe:','es':':mujer_en_bicicleta_tono_de_piel_claro:','fr':':cycliste_femme_peau_claire:','ja':':自転車に乗る女_薄い肌色:','ko':':자전거_타는_여자_하얀_피부:','pt':':mulher_ciclista_pele_clara:','it':':ciclista_donna_carnagione_chiara:','fa':':دوچرخه_سوار_زن_پوست_سفید:','id':':wanita_bersepeda_warna_kulit_cerah:','zh':':女生骑自行车_较浅肤色:','ru':':женщина_на_велосипеде_очень_светлый_тон_кожи:','tr':':bisiklet_süren_kadın_açık_cilt_tonu:','ar':':راكبة_دراجة_بشرة_بلون_فاتح:'},'\U0001F6B4\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_biking_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':radfahrerin_mitteldunkle_hautfarbe:','es':':mujer_en_bicicleta_tono_de_piel_oscuro_medio:','fr':':cycliste_femme_peau_mate:','ja':':自転車に乗る女_やや濃い肌色:','ko':':자전거_타는_여자_진한_갈색_피부:','pt':':mulher_ciclista_pele_morena_escura:','it':':ciclista_donna_carnagione_abbastanza_scura:','fa':':دوچرخه_سوار_زن_پوست_گندمی:','id':':wanita_bersepeda_warna_kulit_gelap-sedang:','zh':':女生骑自行车_中等深肤色:','ru':':женщина_на_велосипеде_темныи_тон_кожи:','tr':':bisiklet_süren_kadın_orta_koyu_cilt_tonu:','ar':':راكبة_دراجة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6B4\U0001F3FE\U0000200D\U00002640':{'en':':woman_biking_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':radfahrerin_mitteldunkle_hautfarbe:','es':':mujer_en_bicicleta_tono_de_piel_oscuro_medio:','fr':':cycliste_femme_peau_mate:','ja':':自転車に乗る女_やや濃い肌色:','ko':':자전거_타는_여자_진한_갈색_피부:','pt':':mulher_ciclista_pele_morena_escura:','it':':ciclista_donna_carnagione_abbastanza_scura:','fa':':دوچرخه_سوار_زن_پوست_گندمی:','id':':wanita_bersepeda_warna_kulit_gelap-sedang:','zh':':女生骑自行车_中等深肤色:','ru':':женщина_на_велосипеде_темный_тон_кожи:','tr':':bisiklet_süren_kadın_orta_koyu_cilt_tonu:','ar':':راكبة_دراجة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6B4\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_biking_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':radfahrerin_mittelhelle_hautfarbe:','es':':mujer_en_bicicleta_tono_de_piel_claro_medio:','fr':':cycliste_femme_peau_moyennement_claire:','ja':':自転車に乗る女_やや薄い肌色:','ko':':자전거_타는_여자_연한_갈색_피부:','pt':':mulher_ciclista_pele_morena_clara:','it':':ciclista_donna_carnagione_abbastanza_chiara:','fa':':دوچرخه_سوار_زن_پوست_روشن:','id':':wanita_bersepeda_warna_kulit_cerah-sedang:','zh':':女生骑自行车_中等浅肤色:','ru':':женщина_на_велосипеде_светлыи_тон_кожи:','tr':':bisiklet_süren_kadın_orta_açık_cilt_tonu:','ar':':راكبة_دراجة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6B4\U0001F3FC\U0000200D\U00002640':{'en':':woman_biking_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':radfahrerin_mittelhelle_hautfarbe:','es':':mujer_en_bicicleta_tono_de_piel_claro_medio:','fr':':cycliste_femme_peau_moyennement_claire:','ja':':自転車に乗る女_やや薄い肌色:','ko':':자전거_타는_여자_연한_갈색_피부:','pt':':mulher_ciclista_pele_morena_clara:','it':':ciclista_donna_carnagione_abbastanza_chiara:','fa':':دوچرخه_سوار_زن_پوست_روشن:','id':':wanita_bersepeda_warna_kulit_cerah-sedang:','zh':':女生骑自行车_中等浅肤色:','ru':':женщина_на_велосипеде_светлый_тон_кожи:','tr':':bisiklet_süren_kadın_orta_açık_cilt_tonu:','ar':':راكبة_دراجة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6B4\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_biking_medium_skin_tone:','status':fully_qualified,'E':4,'de':':radfahrerin_mittlere_hautfarbe:','es':':mujer_en_bicicleta_tono_de_piel_medio:','fr':':cycliste_femme_peau_légèrement_mate:','ja':':自転車に乗る女_中間の肌色:','ko':':자전거_타는_여자_갈색_피부:','pt':':mulher_ciclista_pele_morena:','it':':ciclista_donna_carnagione_olivastra:','fa':':دوچرخه_سوار_زن_پوست_طلایی:','id':':wanita_bersepeda_warna_kulit_sedang:','zh':':女生骑自行车_中等肤色:','ru':':женщина_на_велосипеде_среднии_тон_кожи:','tr':':bisiklet_süren_kadın_orta_cilt_tonu:','ar':':راكبة_دراجة_بشرة_بلون_معتدل:'},'\U0001F6B4\U0001F3FD\U0000200D\U00002640':{'en':':woman_biking_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':radfahrerin_mittlere_hautfarbe:','es':':mujer_en_bicicleta_tono_de_piel_medio:','fr':':cycliste_femme_peau_légèrement_mate:','ja':':自転車に乗る女_中間の肌色:','ko':':자전거_타는_여자_갈색_피부:','pt':':mulher_ciclista_pele_morena:','it':':ciclista_donna_carnagione_olivastra:','fa':':دوچرخه_سوار_زن_پوست_طلایی:','id':':wanita_bersepeda_warna_kulit_sedang:','zh':':女生骑自行车_中等肤色:','ru':':женщина_на_велосипеде_средний_тон_кожи:','tr':':bisiklet_süren_kadın_orta_cilt_tonu:','ar':':راكبة_دراجة_بشرة_بلون_معتدل:'},'\U0001F471\U0000200D\U00002640\U0000FE0F':{'en':':woman_blond_hair:','status':fully_qualified,'E':4,'alias':[':blonde_woman:', ':blond_haired_woman:'],'de':':frau_blond:','es':':mujer_rubia:','fr':':femme_blonde:','ja':':金髪の女性:','ko':':금발_여자:','pt':':mulher_cabelo_loiro:','it':':donna_bionda:','fa':':زن_موبور:','id':':wanita_berambut_pirang:','zh':':金发女:','ru':':блондинка:','tr':':sarışın_kadın:','ar':':امرأة_شقراء:'},'\U0001F471\U0000200D\U00002640':{'en':':woman_blond_hair:','status':minimally_qualified,'E':4,'alias':[':blonde_woman:', ':blond_haired_woman:'],'de':':frau_blond:','es':':mujer_rubia:','fr':':femme_blonde:','ja':':金髪の女性:','ko':':금발_여자:','pt':':mulher_cabelo_loiro:','it':':donna_bionda:','fa':':زن_موبور:','id':':wanita_berambut_pirang:','zh':':金发女:','ru':':блондинка:','tr':':sarışın_kadın:','ar':':امرأة_شقراء:'},'\U000026F9\U0000FE0F\U0000200D\U00002640\U0000FE0F':{'en':':woman_bouncing_ball:','status':fully_qualified,'E':4,'alias':[':basketball_woman:', ':bouncing_ball_woman:'],'de':':frau_mit_ball:','es':':mujer_botando_un_balón:','fr':':femme_avec_ballon:','ja':':バスケットボールをする女:','ko':':공_가진_여자:','pt':':mulher_jogando_basquete:','it':':donna_che_fa_rimbalzare_una_palla:','fa':':زن_و_توپ:','id':':wanita_memantulkan_bola:','zh':':女生玩球:','ru':':женщина_с_мячом:','tr':':top_sektiren_kadın:','ar':':سيدة_وكرة:'},'\U000026F9\U0000200D\U00002640\U0000FE0F':{'en':':woman_bouncing_ball:','status':unqualified,'E':4,'alias':[':basketball_woman:', ':bouncing_ball_woman:'],'de':':frau_mit_ball:','es':':mujer_botando_un_balón:','fr':':femme_avec_ballon:','ja':':バスケットボールをする女:','ko':':공_가진_여자:','pt':':mulher_jogando_basquete:','it':':donna_che_fa_rimbalzare_una_palla:','fa':':زن_و_توپ:','id':':wanita_memantulkan_bola:','zh':':女生玩球:','ru':':женщина_с_мячом:','tr':':top_sektiren_kadın:','ar':':سيدة_وكرة:'},'\U000026F9\U0000FE0F\U0000200D\U00002640':{'en':':woman_bouncing_ball:','status':minimally_qualified,'E':4,'alias':[':basketball_woman:', ':bouncing_ball_woman:'],'de':':frau_mit_ball:','es':':mujer_botando_un_balón:','fr':':femme_avec_ballon:','ja':':バスケットボールをする女:','ko':':공_가진_여자:','pt':':mulher_jogando_basquete:','it':':donna_che_fa_rimbalzare_una_palla:','fa':':زن_و_توپ:','id':':wanita_memantulkan_bola:','zh':':女生玩球:','ru':':женщина_с_мячом:','tr':':top_sektiren_kadın:','ar':':سيدة_وكرة:'},'\U000026F9\U0000200D\U00002640':{'en':':woman_bouncing_ball:','status':unqualified,'E':4,'alias':[':basketball_woman:', ':bouncing_ball_woman:'],'de':':frau_mit_ball:','es':':mujer_botando_un_balón:','fr':':femme_avec_ballon:','ja':':バスケットボールをする女:','ko':':공_가진_여자:','pt':':mulher_jogando_basquete:','it':':donna_che_fa_rimbalzare_una_palla:','fa':':زن_و_توپ:','id':':wanita_memantulkan_bola:','zh':':女生玩球:','ru':':женщина_с_мячом:','tr':':top_sektiren_kadın:','ar':':سيدة_وكرة:'},'\U000026F9\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_bouncing_ball_dark_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_ball_dunkle_hautfarbe:','es':':mujer_botando_un_balón_tono_de_piel_oscuro:','fr':':femme_avec_ballon_peau_foncée:','ja':':バスケットボールをする女_濃い肌色:','ko':':공_가진_여자_검은색_피부:','pt':':mulher_jogando_basquete_pele_escura:','it':':donna_che_fa_rimbalzare_una_palla_carnagione_scura:','fa':':زن_و_توپ_پوست_آبنوسی:','id':':wanita_memantulkan_bola_warna_kulit_gelap:','zh':':女生玩球_较深肤色:','ru':':женщина_с_мячом_очень_темныи_тон_кожи:','tr':':top_sektiren_kadın_koyu_cilt_tonu:','ar':':سيدة_وكرة_بشرة_بلون_غامق:'},'\U000026F9\U0001F3FF\U0000200D\U00002640':{'en':':woman_bouncing_ball_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_ball_dunkle_hautfarbe:','es':':mujer_botando_un_balón_tono_de_piel_oscuro:','fr':':femme_avec_ballon_peau_foncée:','ja':':バスケットボールをする女_濃い肌色:','ko':':공_가진_여자_검은색_피부:','pt':':mulher_jogando_basquete_pele_escura:','it':':donna_che_fa_rimbalzare_una_palla_carnagione_scura:','fa':':زن_و_توپ_پوست_آبنوسی:','id':':wanita_memantulkan_bola_warna_kulit_gelap:','zh':':女生玩球_较深肤色:','ru':':женщина_с_мячом_очень_темный_тон_кожи:','tr':':top_sektiren_kadın_koyu_cilt_tonu:','ar':':سيدة_وكرة_بشرة_بلون_غامق:'},'\U000026F9\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_bouncing_ball_light_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_ball_helle_hautfarbe:','es':':mujer_botando_un_balón_tono_de_piel_claro:','fr':':femme_avec_ballon_peau_claire:','ja':':バスケットボールをする女_薄い肌色:','ko':':공_가진_여자_하얀_피부:','pt':':mulher_jogando_basquete_pele_clara:','it':':donna_che_fa_rimbalzare_una_palla_carnagione_chiara:','fa':':زن_و_توپ_پوست_سفید:','id':':wanita_memantulkan_bola_warna_kulit_cerah:','zh':':女生玩球_较浅肤色:','ru':':женщина_с_мячом_очень_светлыи_тон_кожи:','tr':':top_sektiren_kadın_açık_cilt_tonu:','ar':':سيدة_وكرة_بشرة_بلون_فاتح:'},'\U000026F9\U0001F3FB\U0000200D\U00002640':{'en':':woman_bouncing_ball_light_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_ball_helle_hautfarbe:','es':':mujer_botando_un_balón_tono_de_piel_claro:','fr':':femme_avec_ballon_peau_claire:','ja':':バスケットボールをする女_薄い肌色:','ko':':공_가진_여자_하얀_피부:','pt':':mulher_jogando_basquete_pele_clara:','it':':donna_che_fa_rimbalzare_una_palla_carnagione_chiara:','fa':':زن_و_توپ_پوست_سفید:','id':':wanita_memantulkan_bola_warna_kulit_cerah:','zh':':女生玩球_较浅肤色:','ru':':женщина_с_мячом_очень_светлый_тон_кожи:','tr':':top_sektiren_kadın_açık_cilt_tonu:','ar':':سيدة_وكرة_بشرة_بلون_فاتح:'},'\U000026F9\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_bouncing_ball_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_ball_mitteldunkle_hautfarbe:','es':':mujer_botando_un_balón_tono_de_piel_oscuro_medio:','fr':':femme_avec_ballon_peau_mate:','ja':':バスケットボールをする女_やや濃い肌色:','ko':':공_가진_여자_진한_갈색_피부:','pt':':mulher_jogando_basquete_pele_morena_escura:','it':':donna_che_fa_rimbalzare_una_palla_carnagione_abbastanza_scura:','fa':':زن_و_توپ_پوست_گندمی:','id':':wanita_memantulkan_bola_warna_kulit_gelap-sedang:','zh':':女生玩球_中等深肤色:','ru':':женщина_с_мячом_темныи_тон_кожи:','tr':':top_sektiren_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_وكرة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U000026F9\U0001F3FE\U0000200D\U00002640':{'en':':woman_bouncing_ball_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_ball_mitteldunkle_hautfarbe:','es':':mujer_botando_un_balón_tono_de_piel_oscuro_medio:','fr':':femme_avec_ballon_peau_mate:','ja':':バスケットボールをする女_やや濃い肌色:','ko':':공_가진_여자_진한_갈색_피부:','pt':':mulher_jogando_basquete_pele_morena_escura:','it':':donna_che_fa_rimbalzare_una_palla_carnagione_abbastanza_scura:','fa':':زن_و_توپ_پوست_گندمی:','id':':wanita_memantulkan_bola_warna_kulit_gelap-sedang:','zh':':女生玩球_中等深肤色:','ru':':женщина_с_мячом_темный_тон_кожи:','tr':':top_sektiren_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_وكرة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U000026F9\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_bouncing_ball_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_ball_mittelhelle_hautfarbe:','es':':mujer_botando_un_balón_tono_de_piel_claro_medio:','fr':':femme_avec_ballon_peau_moyennement_claire:','ja':':バスケットボールをする女_やや薄い肌色:','ko':':공_가진_여자_연한_갈색_피부:','pt':':mulher_jogando_basquete_pele_morena_clara:','it':':donna_che_fa_rimbalzare_una_palla_carnagione_abbastanza_chiara:','fa':':زن_و_توپ_پوست_روشن:','id':':wanita_memantulkan_bola_warna_kulit_cerah-sedang:','zh':':女生玩球_中等浅肤色:','ru':':женщина_с_мячом_светлыи_тон_кожи:','tr':':top_sektiren_kadın_orta_açık_cilt_tonu:','ar':':سيدة_وكرة_بشرة_بلون_فاتح_ومعتدل:'},'\U000026F9\U0001F3FC\U0000200D\U00002640':{'en':':woman_bouncing_ball_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_ball_mittelhelle_hautfarbe:','es':':mujer_botando_un_balón_tono_de_piel_claro_medio:','fr':':femme_avec_ballon_peau_moyennement_claire:','ja':':バスケットボールをする女_やや薄い肌色:','ko':':공_가진_여자_연한_갈색_피부:','pt':':mulher_jogando_basquete_pele_morena_clara:','it':':donna_che_fa_rimbalzare_una_palla_carnagione_abbastanza_chiara:','fa':':زن_و_توپ_پوست_روشن:','id':':wanita_memantulkan_bola_warna_kulit_cerah-sedang:','zh':':女生玩球_中等浅肤色:','ru':':женщина_с_мячом_светлый_тон_кожи:','tr':':top_sektiren_kadın_orta_açık_cilt_tonu:','ar':':سيدة_وكرة_بشرة_بلون_فاتح_ومعتدل:'},'\U000026F9\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_bouncing_ball_medium_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_ball_mittlere_hautfarbe:','es':':mujer_botando_un_balón_tono_de_piel_medio:','fr':':femme_avec_ballon_peau_légèrement_mate:','ja':':バスケットボールをする女_中間の肌色:','ko':':공_가진_여자_갈색_피부:','pt':':mulher_jogando_basquete_pele_morena:','it':':donna_che_fa_rimbalzare_una_palla_carnagione_olivastra:','fa':':زن_و_توپ_پوست_طلایی:','id':':wanita_memantulkan_bola_warna_kulit_sedang:','zh':':女生玩球_中等肤色:','ru':':женщина_с_мячом_среднии_тон_кожи:','tr':':top_sektiren_kadın_orta_cilt_tonu:','ar':':سيدة_وكرة_بشرة_بلون_معتدل:'},'\U000026F9\U0001F3FD\U0000200D\U00002640':{'en':':woman_bouncing_ball_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_ball_mittlere_hautfarbe:','es':':mujer_botando_un_balón_tono_de_piel_medio:','fr':':femme_avec_ballon_peau_légèrement_mate:','ja':':バスケットボールをする女_中間の肌色:','ko':':공_가진_여자_갈색_피부:','pt':':mulher_jogando_basquete_pele_morena:','it':':donna_che_fa_rimbalzare_una_palla_carnagione_olivastra:','fa':':زن_و_توپ_پوست_طلایی:','id':':wanita_memantulkan_bola_warna_kulit_sedang:','zh':':女生玩球_中等肤色:','ru':':женщина_с_мячом_средний_тон_кожи:','tr':':top_sektiren_kadın_orta_cilt_tonu:','ar':':سيدة_وكرة_بشرة_بلون_معتدل:'},'\U0001F647\U0000200D\U00002640\U0000FE0F':{'en':':woman_bowing:','status':fully_qualified,'E':4,'alias':[':bowing_woman:'],'de':':sich_verbeugende_frau:','es':':mujer_haciendo_una_reverencia:','fr':':femme_qui_s’incline:','ja':':おじぎする女:','ko':':절하는_여자:','pt':':mulher_fazendo_reverência:','it':':donna_che_fa_inchino_profondo:','fa':':زن_تعظیم_می_کند:','id':':wanita_membungkuk:','zh':':女生鞠躬:','ru':':женщина_кланяется:','tr':':başını_öne_eğen_kadın:','ar':':سيدة_تنحني_اعتذارا:'},'\U0001F647\U0000200D\U00002640':{'en':':woman_bowing:','status':minimally_qualified,'E':4,'alias':[':bowing_woman:'],'de':':sich_verbeugende_frau:','es':':mujer_haciendo_una_reverencia:','fr':':femme_qui_s’incline:','ja':':おじぎする女:','ko':':절하는_여자:','pt':':mulher_fazendo_reverência:','it':':donna_che_fa_inchino_profondo:','fa':':زن_تعظیم_می_کند:','id':':wanita_membungkuk:','zh':':女生鞠躬:','ru':':женщина_кланяется:','tr':':başını_öne_eğen_kadın:','ar':':سيدة_تنحني_اعتذارا:'},'\U0001F647\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_bowing_dark_skin_tone:','status':fully_qualified,'E':4,'de':':sich_verbeugende_frau_dunkle_hautfarbe:','es':':mujer_haciendo_una_reverencia_tono_de_piel_oscuro:','fr':':femme_qui_s’incline_peau_foncée:','ja':':おじぎする女_濃い肌色:','ko':':절하는_여자_검은색_피부:','pt':':mulher_fazendo_reverência_pele_escura:','it':':donna_che_fa_inchino_profondo_carnagione_scura:','fa':':زن_تعظیم_می_کند_پوست_آبنوسی:','id':':wanita_membungkuk_warna_kulit_gelap:','zh':':女生鞠躬_较深肤色:','ru':':женщина_кланяется_очень_темныи_тон_кожи:','tr':':başını_öne_eğen_kadın_koyu_cilt_tonu:','ar':':سيدة_تنحني_اعتذارا_بشرة_بلون_غامق:'},'\U0001F647\U0001F3FF\U0000200D\U00002640':{'en':':woman_bowing_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':sich_verbeugende_frau_dunkle_hautfarbe:','es':':mujer_haciendo_una_reverencia_tono_de_piel_oscuro:','fr':':femme_qui_s’incline_peau_foncée:','ja':':おじぎする女_濃い肌色:','ko':':절하는_여자_검은색_피부:','pt':':mulher_fazendo_reverência_pele_escura:','it':':donna_che_fa_inchino_profondo_carnagione_scura:','fa':':زن_تعظیم_می_کند_پوست_آبنوسی:','id':':wanita_membungkuk_warna_kulit_gelap:','zh':':女生鞠躬_较深肤色:','ru':':женщина_кланяется_очень_темный_тон_кожи:','tr':':başını_öne_eğen_kadın_koyu_cilt_tonu:','ar':':سيدة_تنحني_اعتذارا_بشرة_بلون_غامق:'},'\U0001F647\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_bowing_light_skin_tone:','status':fully_qualified,'E':4,'de':':sich_verbeugende_frau_helle_hautfarbe:','es':':mujer_haciendo_una_reverencia_tono_de_piel_claro:','fr':':femme_qui_s’incline_peau_claire:','ja':':おじぎする女_薄い肌色:','ko':':절하는_여자_하얀_피부:','pt':':mulher_fazendo_reverência_pele_clara:','it':':donna_che_fa_inchino_profondo_carnagione_chiara:','fa':':زن_تعظیم_می_کند_پوست_سفید:','id':':wanita_membungkuk_warna_kulit_cerah:','zh':':女生鞠躬_较浅肤色:','ru':':женщина_кланяется_очень_светлыи_тон_кожи:','tr':':başını_öne_eğen_kadın_açık_cilt_tonu:','ar':':سيدة_تنحني_اعتذارا_بشرة_بلون_فاتح:'},'\U0001F647\U0001F3FB\U0000200D\U00002640':{'en':':woman_bowing_light_skin_tone:','status':minimally_qualified,'E':4,'de':':sich_verbeugende_frau_helle_hautfarbe:','es':':mujer_haciendo_una_reverencia_tono_de_piel_claro:','fr':':femme_qui_s’incline_peau_claire:','ja':':おじぎする女_薄い肌色:','ko':':절하는_여자_하얀_피부:','pt':':mulher_fazendo_reverência_pele_clara:','it':':donna_che_fa_inchino_profondo_carnagione_chiara:','fa':':زن_تعظیم_می_کند_پوست_سفید:','id':':wanita_membungkuk_warna_kulit_cerah:','zh':':女生鞠躬_较浅肤色:','ru':':женщина_кланяется_очень_светлый_тон_кожи:','tr':':başını_öne_eğen_kadın_açık_cilt_tonu:','ar':':سيدة_تنحني_اعتذارا_بشرة_بلون_فاتح:'},'\U0001F647\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_bowing_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':sich_verbeugende_frau_mitteldunkle_hautfarbe:','es':':mujer_haciendo_una_reverencia_tono_de_piel_oscuro_medio:','fr':':femme_qui_s’incline_peau_mate:','ja':':おじぎする女_やや濃い肌色:','ko':':절하는_여자_진한_갈색_피부:','pt':':mulher_fazendo_reverência_pele_morena_escura:','it':':donna_che_fa_inchino_profondo_carnagione_abbastanza_scura:','fa':':زن_تعظیم_می_کند_پوست_گندمی:','id':':wanita_membungkuk_warna_kulit_gelap-sedang:','zh':':女生鞠躬_中等深肤色:','ru':':женщина_кланяется_темныи_тон_кожи:','tr':':başını_öne_eğen_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_تنحني_اعتذارا_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F647\U0001F3FE\U0000200D\U00002640':{'en':':woman_bowing_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':sich_verbeugende_frau_mitteldunkle_hautfarbe:','es':':mujer_haciendo_una_reverencia_tono_de_piel_oscuro_medio:','fr':':femme_qui_s’incline_peau_mate:','ja':':おじぎする女_やや濃い肌色:','ko':':절하는_여자_진한_갈색_피부:','pt':':mulher_fazendo_reverência_pele_morena_escura:','it':':donna_che_fa_inchino_profondo_carnagione_abbastanza_scura:','fa':':زن_تعظیم_می_کند_پوست_گندمی:','id':':wanita_membungkuk_warna_kulit_gelap-sedang:','zh':':女生鞠躬_中等深肤色:','ru':':женщина_кланяется_темный_тон_кожи:','tr':':başını_öne_eğen_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_تنحني_اعتذارا_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F647\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_bowing_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':sich_verbeugende_frau_mittelhelle_hautfarbe:','es':':mujer_haciendo_una_reverencia_tono_de_piel_claro_medio:','fr':':femme_qui_s’incline_peau_moyennement_claire:','ja':':おじぎする女_やや薄い肌色:','ko':':절하는_여자_연한_갈색_피부:','pt':':mulher_fazendo_reverência_pele_morena_clara:','it':':donna_che_fa_inchino_profondo_carnagione_abbastanza_chiara:','fa':':زن_تعظیم_می_کند_پوست_روشن:','id':':wanita_membungkuk_warna_kulit_cerah-sedang:','zh':':女生鞠躬_中等浅肤色:','ru':':женщина_кланяется_светлыи_тон_кожи:','tr':':başını_öne_eğen_kadın_orta_açık_cilt_tonu:','ar':':سيدة_تنحني_اعتذارا_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F647\U0001F3FC\U0000200D\U00002640':{'en':':woman_bowing_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':sich_verbeugende_frau_mittelhelle_hautfarbe:','es':':mujer_haciendo_una_reverencia_tono_de_piel_claro_medio:','fr':':femme_qui_s’incline_peau_moyennement_claire:','ja':':おじぎする女_やや薄い肌色:','ko':':절하는_여자_연한_갈색_피부:','pt':':mulher_fazendo_reverência_pele_morena_clara:','it':':donna_che_fa_inchino_profondo_carnagione_abbastanza_chiara:','fa':':زن_تعظیم_می_کند_پوست_روشن:','id':':wanita_membungkuk_warna_kulit_cerah-sedang:','zh':':女生鞠躬_中等浅肤色:','ru':':женщина_кланяется_светлый_тон_кожи:','tr':':başını_öne_eğen_kadın_orta_açık_cilt_tonu:','ar':':سيدة_تنحني_اعتذارا_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F647\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_bowing_medium_skin_tone:','status':fully_qualified,'E':4,'de':':sich_verbeugende_frau_mittlere_hautfarbe:','es':':mujer_haciendo_una_reverencia_tono_de_piel_medio:','fr':':femme_qui_s’incline_peau_légèrement_mate:','ja':':おじぎする女_中間の肌色:','ko':':절하는_여자_갈색_피부:','pt':':mulher_fazendo_reverência_pele_morena:','it':':donna_che_fa_inchino_profondo_carnagione_olivastra:','fa':':زن_تعظیم_می_کند_پوست_طلایی:','id':':wanita_membungkuk_warna_kulit_sedang:','zh':':女生鞠躬_中等肤色:','ru':':женщина_кланяется_среднии_тон_кожи:','tr':':başını_öne_eğen_kadın_orta_cilt_tonu:','ar':':سيدة_تنحني_اعتذارا_بشرة_بلون_معتدل:'},'\U0001F647\U0001F3FD\U0000200D\U00002640':{'en':':woman_bowing_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':sich_verbeugende_frau_mittlere_hautfarbe:','es':':mujer_haciendo_una_reverencia_tono_de_piel_medio:','fr':':femme_qui_s’incline_peau_légèrement_mate:','ja':':おじぎする女_中間の肌色:','ko':':절하는_여자_갈색_피부:','pt':':mulher_fazendo_reverência_pele_morena:','it':':donna_che_fa_inchino_profondo_carnagione_olivastra:','fa':':زن_تعظیم_می_کند_پوست_طلایی:','id':':wanita_membungkuk_warna_kulit_sedang:','zh':':女生鞠躬_中等肤色:','ru':':женщина_кланяется_средний_тон_кожи:','tr':':başını_öne_eğen_kadın_orta_cilt_tonu:','ar':':سيدة_تنحني_اعتذارا_بشرة_بلون_معتدل:'},'\U0001F938\U0000200D\U00002640\U0000FE0F':{'en':':woman_cartwheeling:','status':fully_qualified,'E':4,'de':':rad_schlagende_frau:','es':':mujer_haciendo_voltereta_lateral:','fr':':femme_faisant_la_roue:','ja':':側転する女:','ko':':옆돌기하는_여자:','pt':':mulher_fazendo_estrela:','it':':donna_che_fa_la_ruota:','fa':':زنِ_چرخ_وفلک_زن:','id':':wanita_melakukan_gerakan_meroda:','zh':':女生侧手翻:','ru':':женщина-акробат:','tr':':perende_atan_kadın:','ar':':سيدة_تفعل_حركة_العجلة:'},'\U0001F938\U0000200D\U00002640':{'en':':woman_cartwheeling:','status':minimally_qualified,'E':4,'de':':rad_schlagende_frau:','es':':mujer_dando_una_voltereta_lateral:','fr':':femme_faisant_la_roue:','ja':':側転する女:','ko':':옆돌기하는_여자:','pt':':mulher_fazendo_estrela:','it':':donna_che_fa_la_ruota:','fa':':زنِ_چرخ_وفلک_زن:','id':':wanita_melakukan_gerakan_meroda:','zh':':女生侧手翻:','ru':':женщина_акробат:','tr':':perende_atan_kadın:','ar':':سيدة_تفعل_حركة_العجلة:'},'\U0001F938\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_cartwheeling_dark_skin_tone:','status':fully_qualified,'E':4,'de':':rad_schlagende_frau_dunkle_hautfarbe:','es':':mujer_haciendo_voltereta_lateral_tono_de_piel_oscuro:','fr':':femme_faisant_la_roue_peau_foncée:','ja':':側転する女_濃い肌色:','ko':':옆돌기하는_여자_검은색_피부:','pt':':mulher_fazendo_estrela_pele_escura:','it':':donna_che_fa_la_ruota_carnagione_scura:','fa':':زنِ_چرخ_وفلک_زن_پوست_آبنوسی:','id':':wanita_melakukan_gerakan_meroda_warna_kulit_gelap:','zh':':女生侧手翻_较深肤色:','ru':':женщина-акробат_очень_темныи_тон_кожи:','tr':':perende_atan_kadın_koyu_cilt_tonu:','ar':':سيدة_تفعل_حركة_العجلة_بشرة_بلون_غامق:'},'\U0001F938\U0001F3FF\U0000200D\U00002640':{'en':':woman_cartwheeling_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':rad_schlagende_frau_dunkle_hautfarbe:','es':':mujer_dando_una_voltereta_lateral_tono_de_piel_oscuro:','fr':':femme_faisant_la_roue_peau_foncée:','ja':':側転する女_濃い肌色:','ko':':옆돌기하는_여자_검은색_피부:','pt':':mulher_fazendo_estrela_pele_escura:','it':':donna_che_fa_la_ruota_carnagione_scura:','fa':':زنِ_چرخ_وفلک_زن_پوست_آبنوسی:','id':':wanita_melakukan_gerakan_meroda_warna_kulit_gelap:','zh':':女生侧手翻_较深肤色:','ru':':женщина_акробат_очень_темный_тон_кожи:','tr':':perende_atan_kadın_koyu_cilt_tonu:','ar':':سيدة_تفعل_حركة_العجلة_بشرة_بلون_غامق:'},'\U0001F938\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_cartwheeling_light_skin_tone:','status':fully_qualified,'E':4,'de':':rad_schlagende_frau_helle_hautfarbe:','es':':mujer_haciendo_voltereta_lateral_tono_de_piel_claro:','fr':':femme_faisant_la_roue_peau_claire:','ja':':側転する女_薄い肌色:','ko':':옆돌기하는_여자_하얀_피부:','pt':':mulher_fazendo_estrela_pele_clara:','it':':donna_che_fa_la_ruota_carnagione_chiara:','fa':':زنِ_چرخ_وفلک_زن_پوست_سفید:','id':':wanita_melakukan_gerakan_meroda_warna_kulit_cerah:','zh':':女生侧手翻_较浅肤色:','ru':':женщина-акробат_очень_светлыи_тон_кожи:','tr':':perende_atan_kadın_açık_cilt_tonu:','ar':':سيدة_تفعل_حركة_العجلة_بشرة_بلون_فاتح:'},'\U0001F938\U0001F3FB\U0000200D\U00002640':{'en':':woman_cartwheeling_light_skin_tone:','status':minimally_qualified,'E':4,'de':':rad_schlagende_frau_helle_hautfarbe:','es':':mujer_dando_una_voltereta_lateral_tono_de_piel_claro:','fr':':femme_faisant_la_roue_peau_claire:','ja':':側転する女_薄い肌色:','ko':':옆돌기하는_여자_하얀_피부:','pt':':mulher_fazendo_estrela_pele_clara:','it':':donna_che_fa_la_ruota_carnagione_chiara:','fa':':زنِ_چرخ_وفلک_زن_پوست_سفید:','id':':wanita_melakukan_gerakan_meroda_warna_kulit_cerah:','zh':':女生侧手翻_较浅肤色:','ru':':женщина_акробат_очень_светлый_тон_кожи:','tr':':perende_atan_kadın_açık_cilt_tonu:','ar':':سيدة_تفعل_حركة_العجلة_بشرة_بلون_فاتح:'},'\U0001F938\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_cartwheeling_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':rad_schlagende_frau_mitteldunkle_hautfarbe:','es':':mujer_haciendo_voltereta_lateral_tono_de_piel_oscuro_medio:','fr':':femme_faisant_la_roue_peau_mate:','ja':':側転する女_やや濃い肌色:','ko':':옆돌기하는_여자_진한_갈색_피부:','pt':':mulher_fazendo_estrela_pele_morena_escura:','it':':donna_che_fa_la_ruota_carnagione_abbastanza_scura:','fa':':زنِ_چرخ_وفلک_زن_پوست_گندمی:','id':':wanita_melakukan_gerakan_meroda_warna_kulit_gelap-sedang:','zh':':女生侧手翻_中等深肤色:','ru':':женщина-акробат_темныи_тон_кожи:','tr':':perende_atan_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_تفعل_حركة_العجلة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F938\U0001F3FE\U0000200D\U00002640':{'en':':woman_cartwheeling_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':rad_schlagende_frau_mitteldunkle_hautfarbe:','es':':mujer_dando_una_voltereta_lateral_tono_de_piel_oscuro_medio:','fr':':femme_faisant_la_roue_peau_mate:','ja':':側転する女_やや濃い肌色:','ko':':옆돌기하는_여자_진한_갈색_피부:','pt':':mulher_fazendo_estrela_pele_morena_escura:','it':':donna_che_fa_la_ruota_carnagione_abbastanza_scura:','fa':':زنِ_چرخ_وفلک_زن_پوست_گندمی:','id':':wanita_melakukan_gerakan_meroda_warna_kulit_gelap-sedang:','zh':':女生侧手翻_中等深肤色:','ru':':женщина_акробат_темный_тон_кожи:','tr':':perende_atan_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_تفعل_حركة_العجلة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F938\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_cartwheeling_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':rad_schlagende_frau_mittelhelle_hautfarbe:','es':':mujer_haciendo_voltereta_lateral_tono_de_piel_claro_medio:','fr':':femme_faisant_la_roue_peau_moyennement_claire:','ja':':側転する女_やや薄い肌色:','ko':':옆돌기하는_여자_연한_갈색_피부:','pt':':mulher_fazendo_estrela_pele_morena_clara:','it':':donna_che_fa_la_ruota_carnagione_abbastanza_chiara:','fa':':زنِ_چرخ_وفلک_زن_پوست_روشن:','id':':wanita_melakukan_gerakan_meroda_warna_kulit_cerah-sedang:','zh':':女生侧手翻_中等浅肤色:','ru':':женщина-акробат_светлыи_тон_кожи:','tr':':perende_atan_kadın_orta_açık_cilt_tonu:','ar':':سيدة_تفعل_حركة_العجلة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F938\U0001F3FC\U0000200D\U00002640':{'en':':woman_cartwheeling_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':rad_schlagende_frau_mittelhelle_hautfarbe:','es':':mujer_dando_una_voltereta_lateral_tono_de_piel_claro_medio:','fr':':femme_faisant_la_roue_peau_moyennement_claire:','ja':':側転する女_やや薄い肌色:','ko':':옆돌기하는_여자_연한_갈색_피부:','pt':':mulher_fazendo_estrela_pele_morena_clara:','it':':donna_che_fa_la_ruota_carnagione_abbastanza_chiara:','fa':':زنِ_چرخ_وفلک_زن_پوست_روشن:','id':':wanita_melakukan_gerakan_meroda_warna_kulit_cerah-sedang:','zh':':女生侧手翻_中等浅肤色:','ru':':женщина_акробат_светлый_тон_кожи:','tr':':perende_atan_kadın_orta_açık_cilt_tonu:','ar':':سيدة_تفعل_حركة_العجلة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F938\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_cartwheeling_medium_skin_tone:','status':fully_qualified,'E':4,'de':':rad_schlagende_frau_mittlere_hautfarbe:','es':':mujer_haciendo_voltereta_lateral_tono_de_piel_medio:','fr':':femme_faisant_la_roue_peau_légèrement_mate:','ja':':側転する女_中間の肌色:','ko':':옆돌기하는_여자_갈색_피부:','pt':':mulher_fazendo_estrela_pele_morena:','it':':donna_che_fa_la_ruota_carnagione_olivastra:','fa':':زنِ_چرخ_وفلک_زن_پوست_طلایی:','id':':wanita_melakukan_gerakan_meroda_warna_kulit_sedang:','zh':':女生侧手翻_中等肤色:','ru':':женщина-акробат_среднии_тон_кожи:','tr':':perende_atan_kadın_orta_cilt_tonu:','ar':':سيدة_تفعل_حركة_العجلة_بشرة_بلون_معتدل:'},'\U0001F938\U0001F3FD\U0000200D\U00002640':{'en':':woman_cartwheeling_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':rad_schlagende_frau_mittlere_hautfarbe:','es':':mujer_dando_una_voltereta_lateral_tono_de_piel_medio:','fr':':femme_faisant_la_roue_peau_légèrement_mate:','ja':':側転する女_中間の肌色:','ko':':옆돌기하는_여자_갈색_피부:','pt':':mulher_fazendo_estrela_pele_morena:','it':':donna_che_fa_la_ruota_carnagione_olivastra:','fa':':زنِ_چرخ_وفلک_زن_پوست_طلایی:','id':':wanita_melakukan_gerakan_meroda_warna_kulit_sedang:','zh':':女生侧手翻_中等肤色:','ru':':женщина_акробат_средний_тон_кожи:','tr':':perende_atan_kadın_orta_cilt_tonu:','ar':':سيدة_تفعل_حركة_العجلة_بشرة_بلون_معتدل:'},'\U0001F9D7\U0000200D\U00002640\U0000FE0F':{'en':':woman_climbing:','status':fully_qualified,'E':5,'alias':[':climbing_woman:'],'de':':bergsteigerin:','es':':mujer_escalando:','fr':':femme_qui_grimpe:','ja':':山を登る女:','ko':':클라이밍하는_여자:','pt':':mulher_escalando:','it':':scalatrice:','fa':':زن_کوه_نوردی_می_کند:','id':':wanita_memanjat:','zh':':攀爬的女人:','ru':':женщина_взбирается_по_скале:','tr':':dağcı_kadın:','ar':':امرأة_تتسلق:'},'\U0001F9D7\U0000200D\U00002640':{'en':':woman_climbing:','status':minimally_qualified,'E':5,'alias':[':climbing_woman:'],'de':':bergsteigerin:','es':':mujer_escalando:','fr':':femme_qui_grimpe:','ja':':山を登る女:','ko':':클라이밍하는_여자:','pt':':mulher_escalando:','it':':scalatrice:','fa':':زن_کوه_نوردی_می_کند:','id':':wanita_memanjat:','zh':':攀爬的女人:','ru':':женщина_взбирается_по_скале:','tr':':dağcı_kadın:','ar':':امرأة_تتسلق:'},'\U0001F9D7\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_climbing_dark_skin_tone:','status':fully_qualified,'E':5,'de':':bergsteigerin_dunkle_hautfarbe:','es':':mujer_escalando_tono_de_piel_oscuro:','fr':':femme_qui_grimpe_peau_foncée:','ja':':山を登る女_濃い肌色:','ko':':클라이밍하는_여자_검은색_피부:','pt':':mulher_escalando_pele_escura:','it':':scalatrice_carnagione_scura:','fa':':زن_کوه_نوردی_می_کند_پوست_آبنوسی:','id':':wanita_memanjat_warna_kulit_gelap:','zh':':攀爬的女人_较深肤色:','ru':':женщина_взбирается_по_скале_очень_темныи_тон_кожи:','tr':':dağcı_kadın_koyu_cilt_tonu:','ar':':امرأة_تتسلق_بشرة_بلون_غامق:'},'\U0001F9D7\U0001F3FF\U0000200D\U00002640':{'en':':woman_climbing_dark_skin_tone:','status':minimally_qualified,'E':5,'de':':bergsteigerin_dunkle_hautfarbe:','es':':mujer_escalando_tono_de_piel_oscuro:','fr':':femme_qui_grimpe_peau_foncée:','ja':':山を登る女_濃い肌色:','ko':':클라이밍하는_여자_검은색_피부:','pt':':mulher_escalando_pele_escura:','it':':scalatrice_carnagione_scura:','fa':':زن_کوه_نوردی_می_کند_پوست_آبنوسی:','id':':wanita_memanjat_warna_kulit_gelap:','zh':':攀爬的女人_较深肤色:','ru':':женщина_взбирается_по_скале_очень_темный_тон_кожи:','tr':':dağcı_kadın_koyu_cilt_tonu:','ar':':امرأة_تتسلق_بشرة_بلون_غامق:'},'\U0001F9D7\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_climbing_light_skin_tone:','status':fully_qualified,'E':5,'de':':bergsteigerin_helle_hautfarbe:','es':':mujer_escalando_tono_de_piel_claro:','fr':':femme_qui_grimpe_peau_claire:','ja':':山を登る女_薄い肌色:','ko':':클라이밍하는_여자_하얀_피부:','pt':':mulher_escalando_pele_clara:','it':':scalatrice_carnagione_chiara:','fa':':زن_کوه_نوردی_می_کند_پوست_سفید:','id':':wanita_memanjat_warna_kulit_cerah:','zh':':攀爬的女人_较浅肤色:','ru':':женщина_взбирается_по_скале_очень_светлыи_тон_кожи:','tr':':dağcı_kadın_açık_cilt_tonu:','ar':':امرأة_تتسلق_بشرة_بلون_فاتح:'},'\U0001F9D7\U0001F3FB\U0000200D\U00002640':{'en':':woman_climbing_light_skin_tone:','status':minimally_qualified,'E':5,'de':':bergsteigerin_helle_hautfarbe:','es':':mujer_escalando_tono_de_piel_claro:','fr':':femme_qui_grimpe_peau_claire:','ja':':山を登る女_薄い肌色:','ko':':클라이밍하는_여자_하얀_피부:','pt':':mulher_escalando_pele_clara:','it':':scalatrice_carnagione_chiara:','fa':':زن_کوه_نوردی_می_کند_پوست_سفید:','id':':wanita_memanjat_warna_kulit_cerah:','zh':':攀爬的女人_较浅肤色:','ru':':женщина_взбирается_по_скале_очень_светлый_тон_кожи:','tr':':dağcı_kadın_açık_cilt_tonu:','ar':':امرأة_تتسلق_بشرة_بلون_فاتح:'},'\U0001F9D7\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_climbing_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':bergsteigerin_mitteldunkle_hautfarbe:','es':':mujer_escalando_tono_de_piel_oscuro_medio:','fr':':femme_qui_grimpe_peau_mate:','ja':':山を登る女_やや濃い肌色:','ko':':클라이밍하는_여자_진한_갈색_피부:','pt':':mulher_escalando_pele_morena_escura:','it':':scalatrice_carnagione_abbastanza_scura:','fa':':زن_کوه_نوردی_می_کند_پوست_گندمی:','id':':wanita_memanjat_warna_kulit_gelap-sedang:','zh':':攀爬的女人_中等深肤色:','ru':':женщина_взбирается_по_скале_темныи_тон_кожи:','tr':':dağcı_kadın_orta_koyu_cilt_tonu:','ar':':امرأة_تتسلق_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D7\U0001F3FE\U0000200D\U00002640':{'en':':woman_climbing_medium-dark_skin_tone:','status':minimally_qualified,'E':5,'de':':bergsteigerin_mitteldunkle_hautfarbe:','es':':mujer_escalando_tono_de_piel_oscuro_medio:','fr':':femme_qui_grimpe_peau_mate:','ja':':山を登る女_やや濃い肌色:','ko':':클라이밍하는_여자_진한_갈색_피부:','pt':':mulher_escalando_pele_morena_escura:','it':':scalatrice_carnagione_abbastanza_scura:','fa':':زن_کوه_نوردی_می_کند_پوست_گندمی:','id':':wanita_memanjat_warna_kulit_gelap-sedang:','zh':':攀爬的女人_中等深肤色:','ru':':женщина_взбирается_по_скале_темный_тон_кожи:','tr':':dağcı_kadın_orta_koyu_cilt_tonu:','ar':':امرأة_تتسلق_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D7\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_climbing_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':bergsteigerin_mittelhelle_hautfarbe:','es':':mujer_escalando_tono_de_piel_claro_medio:','fr':':femme_qui_grimpe_peau_moyennement_claire:','ja':':山を登る女_やや薄い肌色:','ko':':클라이밍하는_여자_연한_갈색_피부:','pt':':mulher_escalando_pele_morena_clara:','it':':scalatrice_carnagione_abbastanza_chiara:','fa':':زن_کوه_نوردی_می_کند_پوست_روشن:','id':':wanita_memanjat_warna_kulit_cerah-sedang:','zh':':攀爬的女人_中等浅肤色:','ru':':женщина_взбирается_по_скале_светлыи_тон_кожи:','tr':':dağcı_kadın_orta_açık_cilt_tonu:','ar':':امرأة_تتسلق_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D7\U0001F3FC\U0000200D\U00002640':{'en':':woman_climbing_medium-light_skin_tone:','status':minimally_qualified,'E':5,'de':':bergsteigerin_mittelhelle_hautfarbe:','es':':mujer_escalando_tono_de_piel_claro_medio:','fr':':femme_qui_grimpe_peau_moyennement_claire:','ja':':山を登る女_やや薄い肌色:','ko':':클라이밍하는_여자_연한_갈색_피부:','pt':':mulher_escalando_pele_morena_clara:','it':':scalatrice_carnagione_abbastanza_chiara:','fa':':زن_کوه_نوردی_می_کند_پوست_روشن:','id':':wanita_memanjat_warna_kulit_cerah-sedang:','zh':':攀爬的女人_中等浅肤色:','ru':':женщина_взбирается_по_скале_светлый_тон_кожи:','tr':':dağcı_kadın_orta_açık_cilt_tonu:','ar':':امرأة_تتسلق_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D7\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_climbing_medium_skin_tone:','status':fully_qualified,'E':5,'de':':bergsteigerin_mittlere_hautfarbe:','es':':mujer_escalando_tono_de_piel_medio:','fr':':femme_qui_grimpe_peau_légèrement_mate:','ja':':山を登る女_中間の肌色:','ko':':클라이밍하는_여자_갈색_피부:','pt':':mulher_escalando_pele_morena:','it':':scalatrice_carnagione_olivastra:','fa':':زن_کوه_نوردی_می_کند_پوست_طلایی:','id':':wanita_memanjat_warna_kulit_sedang:','zh':':攀爬的女人_中等肤色:','ru':':женщина_взбирается_по_скале_среднии_тон_кожи:','tr':':dağcı_kadın_orta_cilt_tonu:','ar':':امرأة_تتسلق_بشرة_بلون_معتدل:'},'\U0001F9D7\U0001F3FD\U0000200D\U00002640':{'en':':woman_climbing_medium_skin_tone:','status':minimally_qualified,'E':5,'de':':bergsteigerin_mittlere_hautfarbe:','es':':mujer_escalando_tono_de_piel_medio:','fr':':femme_qui_grimpe_peau_légèrement_mate:','ja':':山を登る女_中間の肌色:','ko':':클라이밍하는_여자_갈색_피부:','pt':':mulher_escalando_pele_morena:','it':':scalatrice_carnagione_olivastra:','fa':':زن_کوه_نوردی_می_کند_پوست_طلایی:','id':':wanita_memanjat_warna_kulit_sedang:','zh':':攀爬的女人_中等肤色:','ru':':женщина_взбирается_по_скале_средний_тон_кожи:','tr':':dağcı_kadın_orta_cilt_tonu:','ar':':امرأة_تتسلق_بشرة_بلون_معتدل:'},'\U0001F477\U0000200D\U00002640\U0000FE0F':{'en':':woman_construction_worker:','status':fully_qualified,'E':4,'alias':[':construction_worker_woman:'],'de':':bauarbeiterin:','es':':profesional_de_la_construcción_mujer:','fr':':ouvrière_du_bâtiment:','ja':':女性の建設作業員:','ko':':건설_현장_여자_노동자:','pt':':pedreira:','it':':operaia_edile:','fa':':کارگر_ساختمانی_زن:','id':':pekerja_konstruksi_wanita:','zh':':女建筑工人:','ru':':женщина-строитель:','tr':':kadın_inşaat_işçisi:','ar':':عاملة_بناء:'},'\U0001F477\U0000200D\U00002640':{'en':':woman_construction_worker:','status':minimally_qualified,'E':4,'alias':[':construction_worker_woman:'],'de':':bauarbeiterin:','es':':profesional_de_la_construcción_mujer:','fr':':ouvrière_du_bâtiment:','ja':':女性の建設作業員:','ko':':건설_현장_여자_노동자:','pt':':pedreira:','it':':operaia_edile:','fa':':کارگر_ساختمانی_زن:','id':':pekerja_konstruksi_wanita:','zh':':女建筑工人:','ru':':женщина_строитель:','tr':':kadın_inşaat_işçisi:','ar':':عاملة_بناء:'},'\U0001F477\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_construction_worker_dark_skin_tone:','status':fully_qualified,'E':4,'de':':bauarbeiterin_dunkle_hautfarbe:','es':':profesional_de_la_construcción_mujer_tono_de_piel_oscuro:','fr':':ouvrière_du_bâtiment_peau_foncée:','ja':':女性の建設作業員_濃い肌色:','ko':':건설_현장_여자_노동자_검은색_피부:','pt':':pedreira_pele_escura:','it':':operaia_edile_carnagione_scura:','fa':':کارگر_ساختمانی_زن_پوست_آبنوسی:','id':':pekerja_konstruksi_wanita_warna_kulit_gelap:','zh':':女建筑工人_较深肤色:','ru':':женщина-строитель_очень_темныи_тон_кожи:','tr':':kadın_inşaat_işçisi_koyu_cilt_tonu:','ar':':عاملة_بناء_بشرة_بلون_غامق:'},'\U0001F477\U0001F3FF\U0000200D\U00002640':{'en':':woman_construction_worker_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':bauarbeiterin_dunkle_hautfarbe:','es':':profesional_de_la_construcción_mujer_tono_de_piel_oscuro:','fr':':ouvrière_du_bâtiment_peau_foncée:','ja':':女性の建設作業員_濃い肌色:','ko':':건설_현장_여자_노동자_검은색_피부:','pt':':pedreira_pele_escura:','it':':operaia_edile_carnagione_scura:','fa':':کارگر_ساختمانی_زن_پوست_آبنوسی:','id':':pekerja_konstruksi_wanita_warna_kulit_gelap:','zh':':女建筑工人_较深肤色:','ru':':женщина_строитель_очень_темный_тон_кожи:','tr':':kadın_inşaat_işçisi_koyu_cilt_tonu:','ar':':عاملة_بناء_بشرة_بلون_غامق:'},'\U0001F477\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_construction_worker_light_skin_tone:','status':fully_qualified,'E':4,'de':':bauarbeiterin_helle_hautfarbe:','es':':profesional_de_la_construcción_mujer_tono_de_piel_claro:','fr':':ouvrière_du_bâtiment_peau_claire:','ja':':女性の建設作業員_薄い肌色:','ko':':건설_현장_여자_노동자_하얀_피부:','pt':':pedreira_pele_clara:','it':':operaia_edile_carnagione_chiara:','fa':':کارگر_ساختمانی_زن_پوست_سفید:','id':':pekerja_konstruksi_wanita_warna_kulit_cerah:','zh':':女建筑工人_较浅肤色:','ru':':женщина-строитель_очень_светлыи_тон_кожи:','tr':':kadın_inşaat_işçisi_açık_cilt_tonu:','ar':':عاملة_بناء_بشرة_بلون_فاتح:'},'\U0001F477\U0001F3FB\U0000200D\U00002640':{'en':':woman_construction_worker_light_skin_tone:','status':minimally_qualified,'E':4,'de':':bauarbeiterin_helle_hautfarbe:','es':':profesional_de_la_construcción_mujer_tono_de_piel_claro:','fr':':ouvrière_du_bâtiment_peau_claire:','ja':':女性の建設作業員_薄い肌色:','ko':':건설_현장_여자_노동자_하얀_피부:','pt':':pedreira_pele_clara:','it':':operaia_edile_carnagione_chiara:','fa':':کارگر_ساختمانی_زن_پوست_سفید:','id':':pekerja_konstruksi_wanita_warna_kulit_cerah:','zh':':女建筑工人_较浅肤色:','ru':':женщина_строитель_очень_светлый_тон_кожи:','tr':':kadın_inşaat_işçisi_açık_cilt_tonu:','ar':':عاملة_بناء_بشرة_بلون_فاتح:'},'\U0001F477\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_construction_worker_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':bauarbeiterin_mitteldunkle_hautfarbe:','es':':profesional_de_la_construcción_mujer_tono_de_piel_oscuro_medio:','fr':':ouvrière_du_bâtiment_peau_mate:','ja':':女性の建設作業員_やや濃い肌色:','ko':':건설_현장_여자_노동자_진한_갈색_피부:','pt':':pedreira_pele_morena_escura:','it':':operaia_edile_carnagione_abbastanza_scura:','fa':':کارگر_ساختمانی_زن_پوست_گندمی:','id':':pekerja_konstruksi_wanita_warna_kulit_gelap-sedang:','zh':':女建筑工人_中等深肤色:','ru':':женщина-строитель_темныи_тон_кожи:','tr':':kadın_inşaat_işçisi_orta_koyu_cilt_tonu:','ar':':عاملة_بناء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F477\U0001F3FE\U0000200D\U00002640':{'en':':woman_construction_worker_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':bauarbeiterin_mitteldunkle_hautfarbe:','es':':profesional_de_la_construcción_mujer_tono_de_piel_oscuro_medio:','fr':':ouvrière_du_bâtiment_peau_mate:','ja':':女性の建設作業員_やや濃い肌色:','ko':':건설_현장_여자_노동자_진한_갈색_피부:','pt':':pedreira_pele_morena_escura:','it':':operaia_edile_carnagione_abbastanza_scura:','fa':':کارگر_ساختمانی_زن_پوست_گندمی:','id':':pekerja_konstruksi_wanita_warna_kulit_gelap-sedang:','zh':':女建筑工人_中等深肤色:','ru':':женщина_строитель_темный_тон_кожи:','tr':':kadın_inşaat_işçisi_orta_koyu_cilt_tonu:','ar':':عاملة_بناء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F477\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_construction_worker_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':bauarbeiterin_mittelhelle_hautfarbe:','es':':profesional_de_la_construcción_mujer_tono_de_piel_claro_medio:','fr':':ouvrière_du_bâtiment_peau_moyennement_claire:','ja':':女性の建設作業員_やや薄い肌色:','ko':':건설_현장_여자_노동자_연한_갈색_피부:','pt':':pedreira_pele_morena_clara:','it':':operaia_edile_carnagione_abbastanza_chiara:','fa':':کارگر_ساختمانی_زن_پوست_روشن:','id':':pekerja_konstruksi_wanita_warna_kulit_cerah-sedang:','zh':':女建筑工人_中等浅肤色:','ru':':женщина-строитель_светлыи_тон_кожи:','tr':':kadın_inşaat_işçisi_orta_açık_cilt_tonu:','ar':':عاملة_بناء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F477\U0001F3FC\U0000200D\U00002640':{'en':':woman_construction_worker_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':bauarbeiterin_mittelhelle_hautfarbe:','es':':profesional_de_la_construcción_mujer_tono_de_piel_claro_medio:','fr':':ouvrière_du_bâtiment_peau_moyennement_claire:','ja':':女性の建設作業員_やや薄い肌色:','ko':':건설_현장_여자_노동자_연한_갈색_피부:','pt':':pedreira_pele_morena_clara:','it':':operaia_edile_carnagione_abbastanza_chiara:','fa':':کارگر_ساختمانی_زن_پوست_روشن:','id':':pekerja_konstruksi_wanita_warna_kulit_cerah-sedang:','zh':':女建筑工人_中等浅肤色:','ru':':женщина_строитель_светлый_тон_кожи:','tr':':kadın_inşaat_işçisi_orta_açık_cilt_tonu:','ar':':عاملة_بناء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F477\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_construction_worker_medium_skin_tone:','status':fully_qualified,'E':4,'de':':bauarbeiterin_mittlere_hautfarbe:','es':':profesional_de_la_construcción_mujer_tono_de_piel_medio:','fr':':ouvrière_du_bâtiment_peau_légèrement_mate:','ja':':女性の建設作業員_中間の肌色:','ko':':건설_현장_여자_노동자_갈색_피부:','pt':':pedreira_pele_morena:','it':':operaia_edile_carnagione_olivastra:','fa':':کارگر_ساختمانی_زن_پوست_طلایی:','id':':pekerja_konstruksi_wanita_warna_kulit_sedang:','zh':':女建筑工人_中等肤色:','ru':':женщина-строитель_среднии_тон_кожи:','tr':':kadın_inşaat_işçisi_orta_cilt_tonu:','ar':':عاملة_بناء_بشرة_بلون_معتدل:'},'\U0001F477\U0001F3FD\U0000200D\U00002640':{'en':':woman_construction_worker_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':bauarbeiterin_mittlere_hautfarbe:','es':':profesional_de_la_construcción_mujer_tono_de_piel_medio:','fr':':ouvrière_du_bâtiment_peau_légèrement_mate:','ja':':女性の建設作業員_中間の肌色:','ko':':건설_현장_여자_노동자_갈색_피부:','pt':':pedreira_pele_morena:','it':':operaia_edile_carnagione_olivastra:','fa':':کارگر_ساختمانی_زن_پوست_طلایی:','id':':pekerja_konstruksi_wanita_warna_kulit_sedang:','zh':':女建筑工人_中等肤色:','ru':':женщина_строитель_средний_тон_кожи:','tr':':kadın_inşaat_işçisi_orta_cilt_tonu:','ar':':عاملة_بناء_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U0001F373':{'en':':woman_cook:','status':fully_qualified,'E':4,'de':':köchin:','es':':chef_mujer:','fr':':cuisinière:','ja':':女性のコック:','ko':':여자_요리사:','pt':':cozinheira:','it':':cuoca:','fa':':آشپز_زن:','id':':koki_wanita:','zh':':女厨师:','ru':':повариха:','tr':':kadın_aşçı:','ar':':طباخة:'},'\U0001F469\U0001F3FF\U0000200D\U0001F373':{'en':':woman_cook_dark_skin_tone:','status':fully_qualified,'E':4,'de':':köchin_dunkle_hautfarbe:','es':':chef_mujer_tono_de_piel_oscuro:','fr':':cuisinière_peau_foncée:','ja':':女性のコック_濃い肌色:','ko':':여자_요리사_검은색_피부:','pt':':cozinheira_pele_escura:','it':':cuoca_carnagione_scura:','fa':':آشپز_زن_پوست_آبنوسی:','id':':koki_wanita_warna_kulit_gelap:','zh':':女厨师_较深肤色:','ru':':повариха_очень_темный_тон_кожи:','tr':':kadın_aşçı_koyu_cilt_tonu:','ar':':طباخة_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F373':{'en':':woman_cook_light_skin_tone:','status':fully_qualified,'E':4,'de':':köchin_helle_hautfarbe:','es':':chef_mujer_tono_de_piel_claro:','fr':':cuisinière_peau_claire:','ja':':女性のコック_薄い肌色:','ko':':여자_요리사_하얀_피부:','pt':':cozinheira_pele_clara:','it':':cuoca_carnagione_chiara:','fa':':آشپز_زن_پوست_سفید:','id':':koki_wanita_warna_kulit_cerah:','zh':':女厨师_较浅肤色:','ru':':повариха_очень_светлый_тон_кожи:','tr':':kadın_aşçı_açık_cilt_tonu:','ar':':طباخة_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F373':{'en':':woman_cook_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':köchin_mitteldunkle_hautfarbe:','es':':chef_mujer_tono_de_piel_oscuro_medio:','fr':':cuisinière_peau_mate:','ja':':女性のコック_やや濃い肌色:','ko':':여자_요리사_진한_갈색_피부:','pt':':cozinheira_pele_morena_escura:','it':':cuoca_carnagione_abbastanza_scura:','fa':':آشپز_زن_پوست_گندمی:','id':':koki_wanita_warna_kulit_gelap-sedang:','zh':':女厨师_中等深肤色:','ru':':повариха_темный_тон_кожи:','tr':':kadın_aşçı_orta_koyu_cilt_tonu:','ar':':طباخة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F373':{'en':':woman_cook_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':köchin_mittelhelle_hautfarbe:','es':':chef_mujer_tono_de_piel_claro_medio:','fr':':cuisinière_peau_moyennement_claire:','ja':':女性のコック_やや薄い肌色:','ko':':여자_요리사_연한_갈색_피부:','pt':':cozinheira_pele_morena_clara:','it':':cuoca_carnagione_abbastanza_chiara:','fa':':آشپز_زن_پوست_روشن:','id':':koki_wanita_warna_kulit_cerah-sedang:','zh':':女厨师_中等浅肤色:','ru':':повариха_светлый_тон_кожи:','tr':':kadın_aşçı_orta_açık_cilt_tonu:','ar':':طباخة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F373':{'en':':woman_cook_medium_skin_tone:','status':fully_qualified,'E':4,'de':':köchin_mittlere_hautfarbe:','es':':chef_mujer_tono_de_piel_medio:','fr':':cuisinière_peau_légèrement_mate:','ja':':女性のコック_中間の肌色:','ko':':여자_요리사_갈색_피부:','pt':':cozinheira_pele_morena:','it':':cuoca_carnagione_olivastra:','fa':':آشپز_زن_پوست_طلایی:','id':':koki_wanita_warna_kulit_sedang:','zh':':女厨师_中等肤色:','ru':':повариха_средний_тон_кожи:','tr':':kadın_aşçı_orta_cilt_tonu:','ar':':طباخة_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U0001F9B1':{'en':':woman_curly_hair:','status':fully_qualified,'E':11,'alias':[':curly_haired_woman:'],'de':':frau_lockiges_haar:','es':':mujer_pelo_rizado:','fr':':femme_cheveux_bouclés:','ja':':女性_巻き毛:','ko':':여자_곱슬머리:','pt':':mulher_cabelo_cacheado:','it':':donna_capelli_ricci:','fa':':زن_موفرفری:','id':':wanita_rambut_keriting:','zh':':女人_卷发:','ru':':женщина_кудрявые_волосы:','tr':':kadın_kıvırcık_saç:','ar':':سيدة_شعر_مجعد:'},'\U0001F483':{'en':':woman_dancing:','status':fully_qualified,'E':0.6,'alias':[':dancer:'],'de':':tanzende_frau:','es':':mujer_bailando:','fr':':danseuse:','ja':':踊る女:','ko':':여자_댄서:','pt':':mulher_dançando:','it':':donna_che_balla:','fa':':رقاص:','id':':wanita_menari:','zh':':跳舞的女人:','ru':':танцующая_женщина:','tr':':dans_eden_kadın:','ar':':راقصة:'},'\U0001F483\U0001F3FF':{'en':':woman_dancing_dark_skin_tone:','status':fully_qualified,'E':1,'de':':tanzende_frau_dunkle_hautfarbe:','es':':mujer_bailando_tono_de_piel_oscuro:','fr':':danseuse_peau_foncée:','ja':':踊る女_濃い肌色:','ko':':여자_댄서_검은색_피부:','pt':':mulher_dançando_pele_escura:','it':':donna_che_balla_carnagione_scura:','fa':':رقاص_پوست_آبنوسی:','id':':wanita_menari_warna_kulit_gelap:','zh':':跳舞的女人_较深肤色:','ru':':танцующая_женщина_очень_темный_тон_кожи:','tr':':dans_eden_kadın_koyu_cilt_tonu:','ar':':راقصة_بشرة_بلون_غامق:'},'\U0001F483\U0001F3FB':{'en':':woman_dancing_light_skin_tone:','status':fully_qualified,'E':1,'de':':tanzende_frau_helle_hautfarbe:','es':':mujer_bailando_tono_de_piel_claro:','fr':':danseuse_peau_claire:','ja':':踊る女_薄い肌色:','ko':':여자_댄서_하얀_피부:','pt':':mulher_dançando_pele_clara:','it':':donna_che_balla_carnagione_chiara:','fa':':رقاص_پوست_سفید:','id':':wanita_menari_warna_kulit_cerah:','zh':':跳舞的女人_较浅肤色:','ru':':танцующая_женщина_очень_светлый_тон_кожи:','tr':':dans_eden_kadın_açık_cilt_tonu:','ar':':راقصة_بشرة_بلون_فاتح:'},'\U0001F483\U0001F3FE':{'en':':woman_dancing_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':tanzende_frau_mitteldunkle_hautfarbe:','es':':mujer_bailando_tono_de_piel_oscuro_medio:','fr':':danseuse_peau_mate:','ja':':踊る女_やや濃い肌色:','ko':':여자_댄서_진한_갈색_피부:','pt':':mulher_dançando_pele_morena_escura:','it':':donna_che_balla_carnagione_abbastanza_scura:','fa':':رقاص_پوست_گندمی:','id':':wanita_menari_warna_kulit_gelap-sedang:','zh':':跳舞的女人_中等深肤色:','ru':':танцующая_женщина_темный_тон_кожи:','tr':':dans_eden_kadın_orta_koyu_cilt_tonu:','ar':':راقصة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F483\U0001F3FC':{'en':':woman_dancing_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':tanzende_frau_mittelhelle_hautfarbe:','es':':mujer_bailando_tono_de_piel_claro_medio:','fr':':danseuse_peau_moyennement_claire:','ja':':踊る女_やや薄い肌色:','ko':':여자_댄서_연한_갈색_피부:','pt':':mulher_dançando_pele_morena_clara:','it':':donna_che_balla_carnagione_abbastanza_chiara:','fa':':رقاص_پوست_روشن:','id':':wanita_menari_warna_kulit_cerah-sedang:','zh':':跳舞的女人_中等浅肤色:','ru':':танцующая_женщина_светлый_тон_кожи:','tr':':dans_eden_kadın_orta_açık_cilt_tonu:','ar':':راقصة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F483\U0001F3FD':{'en':':woman_dancing_medium_skin_tone:','status':fully_qualified,'E':1,'de':':tanzende_frau_mittlere_hautfarbe:','es':':mujer_bailando_tono_de_piel_medio:','fr':':danseuse_peau_légèrement_mate:','ja':':踊る女_中間の肌色:','ko':':여자_댄서_갈색_피부:','pt':':mulher_dançando_pele_morena:','it':':donna_che_balla_carnagione_olivastra:','fa':':رقاص_پوست_طلایی:','id':':wanita_menari_warna_kulit_sedang:','zh':':跳舞的女人_中等肤色:','ru':':танцующая_женщина_средний_тон_кожи:','tr':':dans_eden_kadın_orta_cilt_tonu:','ar':':راقصة_بشرة_بلون_معتدل:'},'\U0001F469\U0001F3FF':{'en':':woman_dark_skin_tone:','status':fully_qualified,'E':1,'de':':frau_dunkle_hautfarbe:','es':':mujer_tono_de_piel_oscuro:','fr':':femme_peau_foncée:','ja':':女性_濃い肌色:','ko':':여자_검은색_피부:','pt':':mulher_pele_escura:','it':':donna_carnagione_scura:','fa':':زن_پوست_آبنوسی:','id':':wanita_warna_kulit_gelap:','zh':':女人_较深肤色:','ru':':женщина_очень_темный_тон_кожи:','tr':':kadın_koyu_cilt_tonu:','ar':':سيدة_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FF\U0000200D\U0001F9B2':{'en':':woman_dark_skin_tone_bald:','status':fully_qualified,'E':11,'de':':frau_dunkle_hautfarbe_glatze:','es':':mujer_tono_de_piel_oscuro_y_sin_pelo:','fr':':femme_peau_foncée_et_chauve:','ja':':女性_濃い肌色_はげ頭:','ko':':여자_검은색_피부_대머리:','pt':':mulher_pele_escura_e_careca:','it':':donna_carnagione_scura_e_calvo:','fa':':زن_پوست_آبنوسی_تاس:','id':':wanita_warna_kulit_gelap_gundul:','zh':':女人_较深肤色秃顶:','ru':':женщина_очень_темный_тон_кожи_без_волос:','tr':':kadın_koyu_cilt_tonu_kel:','ar':':سيدة_بشرة_بلون_غامق_وأصلع:'},'\U0001F9D4\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_dark_skin_tone_beard:','status':fully_qualified,'E':13.1,'de':':frau_dunkle_hautfarbe_bart:','es':':mujer_tono_de_piel_oscuro_y_barba:','fr':':femme_barbue_peau_foncée:','ja':':あごひげの女性_濃い肌色:','ko':':수염_난_여자_검은색_피부:','pt':':mulher_pele_escura_e_barba:','it':':donna_con_la_barba_carnagione_scura:','fa':':زن_پوست_آبنوسی_ریش:','id':':wanita_warna_kulit_gelap_janggut:','zh':':有络腮胡子的女人_较深肤色:','ru':':бородатая_женщина_очень_темныи_тон_кожи:','tr':':kadın_koyu_cilt_tonu_sakal:','ar':':امرأة_بلحية_بشرة_بلون_غامق:'},'\U0001F9D4\U0001F3FF\U0000200D\U00002640':{'en':':woman_dark_skin_tone_beard:','status':minimally_qualified,'E':13.1,'de':':frau_dunkle_hautfarbe_bart:','es':':mujer_tono_de_piel_oscuro_y_barba:','fr':':femme_barbue_peau_foncée:','ja':':あごひげの女性_濃い肌色:','ko':':수염_난_여자_검은색_피부:','pt':':mulher_pele_escura_e_barba:','it':':donna_con_la_barba_carnagione_scura:','fa':':زن_پوست_آبنوسی_ریش:','id':':wanita_warna_kulit_gelap_janggut:','zh':':有络腮胡子的女人_较深肤色:','ru':':бородатая_женщина_очень_темный_тон_кожи:','tr':':kadın_koyu_cilt_tonu_sakal:','ar':':امرأة_بلحية_بشرة_بلون_غامق:'},'\U0001F471\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_dark_skin_tone_blond_hair:','status':fully_qualified,'E':4,'de':':frau_dunkle_hautfarbe,_blond:','es':':mujer_rubia_tono_de_piel_oscuro:','fr':':femme_blonde_peau_foncée:','ja':':金髪の女性_濃い肌色:','ko':':금발_여자_검은색_피부:','pt':':mulher_pele_escura_e_cabelo_loiro:','it':':donna_bionda_carnagione_scura:','fa':':زن_پوست_آبنوسی_موبور:','id':':wanita_berambut_pirang_warna_kulit_gelap:','zh':':金发女_较深肤色:','ru':':блондинка_очень_темныи_тон_кожи:','tr':':sarışın_kadın_koyu_cilt_tonu:','ar':':امرأة_شقراء_بشرة_بلون_غامق:'},'\U0001F471\U0001F3FF\U0000200D\U00002640':{'en':':woman_dark_skin_tone_blond_hair:','status':minimally_qualified,'E':4,'de':':frau_dunkle_hautfarbe_blond:','es':':mujer_rubia_tono_de_piel_oscuro:','fr':':femme_blonde_peau_foncée:','ja':':金髪の女性_濃い肌色:','ko':':금발_여자_검은색_피부:','pt':':mulher_pele_escura_e_cabelo_loiro:','it':':donna_bionda_carnagione_scura:','fa':':زن_پوست_آبنوسی_موبور:','id':':wanita_berambut_pirang_warna_kulit_gelap:','zh':':金发女_较深肤色:','ru':':блондинка_очень_темный_тон_кожи:','tr':':sarışın_kadın_koyu_cilt_tonu:','ar':':امرأة_شقراء_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FF\U0000200D\U0001F9B1':{'en':':woman_dark_skin_tone_curly_hair:','status':fully_qualified,'E':11,'de':':frau_dunkle_hautfarbe_lockiges_haar:','es':':mujer_tono_de_piel_oscuro_y_pelo_rizado:','fr':':femme_peau_foncée_et_cheveux_bouclés:','ja':':女性_濃い肌色_巻き毛:','ko':':여자_검은색_피부_곱슬머리:','pt':':mulher_pele_escura_e_cabelo_cacheado:','it':':donna_carnagione_scura_e_capelli_ricci:','fa':':زن_پوست_آبنوسی_موفرفری:','id':':wanita_warna_kulit_gelap_rambut_keriting:','zh':':女人_较深肤色卷发:','ru':':женщина_очень_темный_тон_кожи_кудрявые_волосы:','tr':':kadın_koyu_cilt_tonu_kıvırcık_saç:','ar':':سيدة_بشرة_بلون_غامق_وشعر_مجعد:'},'\U0001F469\U0001F3FF\U0000200D\U0001F9B0':{'en':':woman_dark_skin_tone_red_hair:','status':fully_qualified,'E':11,'de':':frau_dunkle_hautfarbe_rotes_haar:','es':':mujer_tono_de_piel_oscuro_y_pelo_pelirrojo:','fr':':femme_peau_foncée_et_cheveux_roux:','ja':':女性_濃い肌色_赤毛:','ko':':여자_검은색_피부_빨간_머리:','pt':':mulher_pele_escura_e_cabelo_vermelho:','it':':donna_carnagione_scura_e_capelli_rossi:','fa':':زن_پوست_آبنوسی_موقرمز:','id':':wanita_warna_kulit_gelap_rambut_merah:','zh':':女人_较深肤色红发:','ru':':женщина_очень_темный_тон_кожи_рыжие_волосы:','tr':':kadın_koyu_cilt_tonu_kızıl_saç:','ar':':سيدة_بشرة_بلون_غامق_وشعر_أحمر:'},'\U0001F469\U0001F3FF\U0000200D\U0001F9B3':{'en':':woman_dark_skin_tone_white_hair:','status':fully_qualified,'E':11,'de':':frau_dunkle_hautfarbe_weißes_haar:','es':':mujer_tono_de_piel_oscuro_y_pelo_blanco:','fr':':femme_peau_foncée_et_cheveux_blancs:','ja':':女性_濃い肌色_白髪:','ko':':여자_검은색_피부_백발:','pt':':mulher_pele_escura_e_cabelo_branco:','it':':donna_carnagione_scura_e_capelli_bianchi:','fa':':زن_پوست_آبنوسی_موسفید:','id':':wanita_warna_kulit_gelap_rambut_putih:','zh':':女人_较深肤色白发:','ru':':женщина_очень_темный_тон_кожи_седые_волосы:','tr':':kadın_koyu_cilt_tonu_beyaz_saç:','ar':':سيدة_بشرة_بلون_غامق_وشعر_أبيض:'},'\U0001F575\U0000FE0F\U0000200D\U00002640\U0000FE0F':{'en':':woman_detective:','status':fully_qualified,'E':4,'alias':[':female_detective:'],'de':':detektivin:','es':':detective_mujer:','fr':':détective_femme:','ja':':女性の探偵:','ko':':여자_탐정:','pt':':detetive_mulher:','it':':investigatrice:','fa':':کارآگاه_زن:','id':':detektif_wanita:','zh':':女侦探:','ru':':женщина-детектив:','tr':':kadın_dedektif:','ar':':محققة:'},'\U0001F575\U0000200D\U00002640\U0000FE0F':{'en':':woman_detective:','status':unqualified,'E':4,'alias':[':female_detective:'],'de':':detektivin:','es':':detective_mujer:','fr':':détective_femme:','ja':':女性の探偵:','ko':':여자_탐정:','pt':':detetive_mulher:','it':':investigatrice:','fa':':کارآگاه_زن:','id':':detektif_wanita:','zh':':女侦探:','ru':':женщина-детектив:','tr':':kadın_dedektif:','ar':':محققة:'},'\U0001F575\U0000FE0F\U0000200D\U00002640':{'en':':woman_detective:','status':minimally_qualified,'E':4,'alias':[':female_detective:'],'de':':detektivin:','es':':detective_mujer:','fr':':détective_femme:','ja':':女性の探偵:','ko':':여자_탐정:','pt':':detetive_mulher:','it':':investigatrice:','fa':':کارآگاه_زن:','id':':detektif_wanita:','zh':':女侦探:','ru':':женщина-детектив:','tr':':kadın_dedektif:','ar':':محققة:'},'\U0001F575\U0000200D\U00002640':{'en':':woman_detective:','status':unqualified,'E':4,'alias':[':female_detective:'],'de':':detektivin:','es':':detective_mujer:','fr':':détective_femme:','ja':':女性の探偵:','ko':':여자_탐정:','pt':':detetive_mulher:','it':':investigatrice:','fa':':کارآگاه_زن:','id':':detektif_wanita:','zh':':女侦探:','ru':':женщина_детектив:','tr':':kadın_dedektif:','ar':':محققة:'},'\U0001F575\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_detective_dark_skin_tone:','status':fully_qualified,'E':4,'de':':detektivin_dunkle_hautfarbe:','es':':detective_mujer_tono_de_piel_oscuro:','fr':':détective_femme_peau_foncée:','ja':':女性の探偵_濃い肌色:','ko':':여자_탐정_검은색_피부:','pt':':detetive_mulher_pele_escura:','it':':investigatrice_carnagione_scura:','fa':':کارآگاه_زن_پوست_آبنوسی:','id':':detektif_wanita_warna_kulit_gelap:','zh':':女侦探_较深肤色:','ru':':женщина-детектив_очень_темныи_тон_кожи:','tr':':kadın_dedektif_koyu_cilt_tonu:','ar':':محققة_بشرة_بلون_غامق:'},'\U0001F575\U0001F3FF\U0000200D\U00002640':{'en':':woman_detective_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':detektivin_dunkle_hautfarbe:','es':':detective_mujer_tono_de_piel_oscuro:','fr':':détective_femme_peau_foncée:','ja':':女性の探偵_濃い肌色:','ko':':여자_탐정_검은색_피부:','pt':':detetive_mulher_pele_escura:','it':':investigatrice_carnagione_scura:','fa':':کارآگاه_زن_پوست_آبنوسی:','id':':detektif_wanita_warna_kulit_gelap:','zh':':女侦探_较深肤色:','ru':':женщина_детектив_очень_темный_тон_кожи:','tr':':kadın_dedektif_koyu_cilt_tonu:','ar':':محققة_بشرة_بلون_غامق:'},'\U0001F575\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_detective_light_skin_tone:','status':fully_qualified,'E':4,'de':':detektivin_helle_hautfarbe:','es':':detective_mujer_tono_de_piel_claro:','fr':':détective_femme_peau_claire:','ja':':女性の探偵_薄い肌色:','ko':':여자_탐정_하얀_피부:','pt':':detetive_mulher_pele_clara:','it':':investigatrice_carnagione_chiara:','fa':':کارآگاه_زن_پوست_سفید:','id':':detektif_wanita_warna_kulit_cerah:','zh':':女侦探_较浅肤色:','ru':':женщина-детектив_очень_светлыи_тон_кожи:','tr':':kadın_dedektif_açık_cilt_tonu:','ar':':محققة_بشرة_بلون_فاتح:'},'\U0001F575\U0001F3FB\U0000200D\U00002640':{'en':':woman_detective_light_skin_tone:','status':minimally_qualified,'E':4,'de':':detektivin_helle_hautfarbe:','es':':detective_mujer_tono_de_piel_claro:','fr':':détective_femme_peau_claire:','ja':':女性の探偵_薄い肌色:','ko':':여자_탐정_하얀_피부:','pt':':detetive_mulher_pele_clara:','it':':investigatrice_carnagione_chiara:','fa':':کارآگاه_زن_پوست_سفید:','id':':detektif_wanita_warna_kulit_cerah:','zh':':女侦探_较浅肤色:','ru':':женщина_детектив_очень_светлый_тон_кожи:','tr':':kadın_dedektif_açık_cilt_tonu:','ar':':محققة_بشرة_بلون_فاتح:'},'\U0001F575\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_detective_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':detektivin_mitteldunkle_hautfarbe:','es':':detective_mujer_tono_de_piel_oscuro_medio:','fr':':détective_femme_peau_mate:','ja':':女性の探偵_やや濃い肌色:','ko':':여자_탐정_진한_갈색_피부:','pt':':detetive_mulher_pele_morena_escura:','it':':investigatrice_carnagione_abbastanza_scura:','fa':':کارآگاه_زن_پوست_گندمی:','id':':detektif_wanita_warna_kulit_gelap-sedang:','zh':':女侦探_中等深肤色:','ru':':женщина-детектив_темныи_тон_кожи:','tr':':kadın_dedektif_orta_koyu_cilt_tonu:','ar':':محققة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F575\U0001F3FE\U0000200D\U00002640':{'en':':woman_detective_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':detektivin_mitteldunkle_hautfarbe:','es':':detective_mujer_tono_de_piel_oscuro_medio:','fr':':détective_femme_peau_mate:','ja':':女性の探偵_やや濃い肌色:','ko':':여자_탐정_진한_갈색_피부:','pt':':detetive_mulher_pele_morena_escura:','it':':investigatrice_carnagione_abbastanza_scura:','fa':':کارآگاه_زن_پوست_گندمی:','id':':detektif_wanita_warna_kulit_gelap-sedang:','zh':':女侦探_中等深肤色:','ru':':женщина_детектив_темный_тон_кожи:','tr':':kadın_dedektif_orta_koyu_cilt_tonu:','ar':':محققة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F575\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_detective_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':detektivin_mittelhelle_hautfarbe:','es':':detective_mujer_tono_de_piel_claro_medio:','fr':':détective_femme_peau_moyennement_claire:','ja':':女性の探偵_やや薄い肌色:','ko':':여자_탐정_연한_갈색_피부:','pt':':detetive_mulher_pele_morena_clara:','it':':investigatrice_carnagione_abbastanza_chiara:','fa':':کارآگاه_زن_پوست_روشن:','id':':detektif_wanita_warna_kulit_cerah-sedang:','zh':':女侦探_中等浅肤色:','ru':':женщина-детектив_светлыи_тон_кожи:','tr':':kadın_dedektif_orta_açık_cilt_tonu:','ar':':محققة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F575\U0001F3FC\U0000200D\U00002640':{'en':':woman_detective_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':detektivin_mittelhelle_hautfarbe:','es':':detective_mujer_tono_de_piel_claro_medio:','fr':':détective_femme_peau_moyennement_claire:','ja':':女性の探偵_やや薄い肌色:','ko':':여자_탐정_연한_갈색_피부:','pt':':detetive_mulher_pele_morena_clara:','it':':investigatrice_carnagione_abbastanza_chiara:','fa':':کارآگاه_زن_پوست_روشن:','id':':detektif_wanita_warna_kulit_cerah-sedang:','zh':':女侦探_中等浅肤色:','ru':':женщина_детектив_светлый_тон_кожи:','tr':':kadın_dedektif_orta_açık_cilt_tonu:','ar':':محققة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F575\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_detective_medium_skin_tone:','status':fully_qualified,'E':4,'de':':detektivin_mittlere_hautfarbe:','es':':detective_mujer_tono_de_piel_medio:','fr':':détective_femme_peau_légèrement_mate:','ja':':女性の探偵_中間の肌色:','ko':':여자_탐정_갈색_피부:','pt':':detetive_mulher_pele_morena:','it':':investigatrice_carnagione_olivastra:','fa':':کارآگاه_زن_پوست_طلایی:','id':':detektif_wanita_warna_kulit_sedang:','zh':':女侦探_中等肤色:','ru':':женщина-детектив_среднии_тон_кожи:','tr':':kadın_dedektif_orta_cilt_tonu:','ar':':محققة_بشرة_بلون_معتدل:'},'\U0001F575\U0001F3FD\U0000200D\U00002640':{'en':':woman_detective_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':detektivin_mittlere_hautfarbe:','es':':detective_mujer_tono_de_piel_medio:','fr':':détective_femme_peau_légèrement_mate:','ja':':女性の探偵_中間の肌色:','ko':':여자_탐정_갈색_피부:','pt':':detetive_mulher_pele_morena:','it':':investigatrice_carnagione_olivastra:','fa':':کارآگاه_زن_پوست_طلایی:','id':':detektif_wanita_warna_kulit_sedang:','zh':':女侦探_中等肤色:','ru':':женщина_детектив_средний_тон_кожи:','tr':':kadın_dedektif_orta_cilt_tonu:','ar':':محققة_بشرة_بلون_معتدل:'},'\U0001F9DD\U0000200D\U00002640\U0000FE0F':{'en':':woman_elf:','status':fully_qualified,'E':5,'alias':[':elf_woman:'],'de':':elfe:','es':':elfa:','fr':':elfe_femme:','ja':':女のエルフ:','ko':':여자_엘프:','pt':':elfa:','it':':elfo_donna:','fa':':اِلف_زن:','id':':kurcaci_wanita:','zh':':女小精灵:','ru':':женщина-эльф:','tr':':kadın_elf:','ar':':جنية_صغيرة:'},'\U0001F9DD\U0000200D\U00002640':{'en':':woman_elf:','status':minimally_qualified,'E':5,'alias':[':elf_woman:'],'de':':elfe:','es':':elfa:','fr':':elfe_femme:','ja':':女のエルフ:','ko':':여자_엘프:','pt':':elfa:','it':':elfo_donna:','fa':':اِلف_زن:','id':':kurcaci_wanita:','zh':':女小精灵:','ru':':женщина_эльф:','tr':':kadın_elf:','ar':':جنية_صغيرة:'},'\U0001F9DD\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_elf_dark_skin_tone:','status':fully_qualified,'E':5,'de':':elfe_dunkle_hautfarbe:','es':':elfa_tono_de_piel_oscuro:','fr':':elfe_femme_peau_foncée:','ja':':女のエルフ_濃い肌色:','ko':':여자_엘프_검은색_피부:','pt':':elfa_pele_escura:','it':':elfo_donna_carnagione_scura:','fa':':اِلف_زن_پوست_آبنوسی:','id':':kurcaci_wanita_warna_kulit_gelap:','zh':':女小精灵_较深肤色:','ru':':женщина-эльф_очень_темныи_тон_кожи:','tr':':kadın_elf_koyu_cilt_tonu:','ar':':جنية_صغيرة_بشرة_بلون_غامق:'},'\U0001F9DD\U0001F3FF\U0000200D\U00002640':{'en':':woman_elf_dark_skin_tone:','status':minimally_qualified,'E':5,'de':':elfe_dunkle_hautfarbe:','es':':elfa_tono_de_piel_oscuro:','fr':':elfe_femme_peau_foncée:','ja':':女のエルフ_濃い肌色:','ko':':여자_엘프_검은색_피부:','pt':':elfa_pele_escura:','it':':elfo_donna_carnagione_scura:','fa':':اِلف_زن_پوست_آبنوسی:','id':':kurcaci_wanita_warna_kulit_gelap:','zh':':女小精灵_较深肤色:','ru':':женщина_эльф_очень_темный_тон_кожи:','tr':':kadın_elf_koyu_cilt_tonu:','ar':':جنية_صغيرة_بشرة_بلون_غامق:'},'\U0001F9DD\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_elf_light_skin_tone:','status':fully_qualified,'E':5,'de':':elfe_helle_hautfarbe:','es':':elfa_tono_de_piel_claro:','fr':':elfe_femme_peau_claire:','ja':':女のエルフ_薄い肌色:','ko':':여자_엘프_하얀_피부:','pt':':elfa_pele_clara:','it':':elfo_donna_carnagione_chiara:','fa':':اِلف_زن_پوست_سفید:','id':':kurcaci_wanita_warna_kulit_cerah:','zh':':女小精灵_较浅肤色:','ru':':женщина-эльф_очень_светлыи_тон_кожи:','tr':':kadın_elf_açık_cilt_tonu:','ar':':جنية_صغيرة_بشرة_بلون_فاتح:'},'\U0001F9DD\U0001F3FB\U0000200D\U00002640':{'en':':woman_elf_light_skin_tone:','status':minimally_qualified,'E':5,'de':':elfe_helle_hautfarbe:','es':':elfa_tono_de_piel_claro:','fr':':elfe_femme_peau_claire:','ja':':女のエルフ_薄い肌色:','ko':':여자_엘프_하얀_피부:','pt':':elfa_pele_clara:','it':':elfo_donna_carnagione_chiara:','fa':':اِلف_زن_پوست_سفید:','id':':kurcaci_wanita_warna_kulit_cerah:','zh':':女小精灵_较浅肤色:','ru':':женщина_эльф_очень_светлый_тон_кожи:','tr':':kadın_elf_açık_cilt_tonu:','ar':':جنية_صغيرة_بشرة_بلون_فاتح:'},'\U0001F9DD\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_elf_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':elfe_mitteldunkle_hautfarbe:','es':':elfa_tono_de_piel_oscuro_medio:','fr':':elfe_femme_peau_mate:','ja':':女のエルフ_やや濃い肌色:','ko':':여자_엘프_진한_갈색_피부:','pt':':elfa_pele_morena_escura:','it':':elfo_donna_carnagione_abbastanza_scura:','fa':':اِلف_زن_پوست_گندمی:','id':':kurcaci_wanita_warna_kulit_gelap-sedang:','zh':':女小精灵_中等深肤色:','ru':':женщина-эльф_темныи_тон_кожи:','tr':':kadın_elf_orta_koyu_cilt_tonu:','ar':':جنية_صغيرة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9DD\U0001F3FE\U0000200D\U00002640':{'en':':woman_elf_medium-dark_skin_tone:','status':minimally_qualified,'E':5,'de':':elfe_mitteldunkle_hautfarbe:','es':':elfa_tono_de_piel_oscuro_medio:','fr':':elfe_femme_peau_mate:','ja':':女のエルフ_やや濃い肌色:','ko':':여자_엘프_진한_갈색_피부:','pt':':elfa_pele_morena_escura:','it':':elfo_donna_carnagione_abbastanza_scura:','fa':':اِلف_زن_پوست_گندمی:','id':':kurcaci_wanita_warna_kulit_gelap-sedang:','zh':':女小精灵_中等深肤色:','ru':':женщина_эльф_темный_тон_кожи:','tr':':kadın_elf_orta_koyu_cilt_tonu:','ar':':جنية_صغيرة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9DD\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_elf_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':elfe_mittelhelle_hautfarbe:','es':':elfa_tono_de_piel_claro_medio:','fr':':elfe_femme_peau_moyennement_claire:','ja':':女のエルフ_やや薄い肌色:','ko':':여자_엘프_연한_갈색_피부:','pt':':elfa_pele_morena_clara:','it':':elfo_donna_carnagione_abbastanza_chiara:','fa':':اِلف_زن_پوست_روشن:','id':':kurcaci_wanita_warna_kulit_cerah-sedang:','zh':':女小精灵_中等浅肤色:','ru':':женщина-эльф_светлыи_тон_кожи:','tr':':kadın_elf_orta_açık_cilt_tonu:','ar':':جنية_صغيرة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9DD\U0001F3FC\U0000200D\U00002640':{'en':':woman_elf_medium-light_skin_tone:','status':minimally_qualified,'E':5,'de':':elfe_mittelhelle_hautfarbe:','es':':elfa_tono_de_piel_claro_medio:','fr':':elfe_femme_peau_moyennement_claire:','ja':':女のエルフ_やや薄い肌色:','ko':':여자_엘프_연한_갈색_피부:','pt':':elfa_pele_morena_clara:','it':':elfo_donna_carnagione_abbastanza_chiara:','fa':':اِلف_زن_پوست_روشن:','id':':kurcaci_wanita_warna_kulit_cerah-sedang:','zh':':女小精灵_中等浅肤色:','ru':':женщина_эльф_светлый_тон_кожи:','tr':':kadın_elf_orta_açık_cilt_tonu:','ar':':جنية_صغيرة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9DD\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_elf_medium_skin_tone:','status':fully_qualified,'E':5,'de':':elfe_mittlere_hautfarbe:','es':':elfa_tono_de_piel_medio:','fr':':elfe_femme_peau_légèrement_mate:','ja':':女のエルフ_中間の肌色:','ko':':여자_엘프_갈색_피부:','pt':':elfa_pele_morena:','it':':elfo_donna_carnagione_olivastra:','fa':':اِلف_زن_پوست_طلایی:','id':':kurcaci_wanita_warna_kulit_sedang:','zh':':女小精灵_中等肤色:','ru':':женщина-эльф_среднии_тон_кожи:','tr':':kadın_elf_orta_cilt_tonu:','ar':':جنية_صغيرة_بشرة_بلون_معتدل:'},'\U0001F9DD\U0001F3FD\U0000200D\U00002640':{'en':':woman_elf_medium_skin_tone:','status':minimally_qualified,'E':5,'de':':elfe_mittlere_hautfarbe:','es':':elfa_tono_de_piel_medio:','fr':':elfe_femme_peau_légèrement_mate:','ja':':女のエルフ_中間の肌色:','ko':':여자_엘프_갈색_피부:','pt':':elfa_pele_morena:','it':':elfo_donna_carnagione_olivastra:','fa':':اِلف_زن_پوست_طلایی:','id':':kurcaci_wanita_warna_kulit_sedang:','zh':':女小精灵_中等肤色:','ru':':женщина_эльф_средний_тон_кожи:','tr':':kadın_elf_orta_cilt_tonu:','ar':':جنية_صغيرة_بشرة_بلون_معتدل:'},'\U0001F926\U0000200D\U00002640\U0000FE0F':{'en':':woman_facepalming:','status':fully_qualified,'E':4,'de':':sich_an_den_kopf_fassende_frau:','es':':mujer_con_la_mano_en_la_frente:','fr':':femme_avec_la_paume_sur_le_visage:','ja':':ひたいに手をあてる女:','ko':':골치_아파하는_여자:','pt':':mulher_decepcionada:','it':':donna_esasperata:','fa':':زن_با_کف_دست_به_پیشانی_می_زند:','id':':wanita_tepuk_jidat:','zh':':女生捂脸:','ru':':женщина_потеряла_дар_речи:','tr':':eliyle_yüzünü_kapatan_kadın:','ar':':سيدة_تشعر_بإحباط:'},'\U0001F926\U0000200D\U00002640':{'en':':woman_facepalming:','status':minimally_qualified,'E':4,'de':':sich_an_den_kopf_fassende_frau:','es':':mujer_con_la_mano_en_la_frente:','fr':':femme_avec_la_paume_sur_le_visage:','ja':':ひたいに手をあてる女:','ko':':골치_아파하는_여자:','pt':':mulher_decepcionada:','it':':donna_esasperata:','fa':':زن_با_کف_دست_به_پیشانی_می_زند:','id':':wanita_tepuk_jidat:','zh':':女生捂脸:','ru':':женщина_потеряла_дар_речи:','tr':':eliyle_yüzünü_kapatan_kadın:','ar':':سيدة_تشعر_بإحباط:'},'\U0001F926\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_facepalming_dark_skin_tone:','status':fully_qualified,'E':4,'de':':sich_an_den_kopf_fassende_frau_dunkle_hautfarbe:','es':':mujer_con_la_mano_en_la_frente_tono_de_piel_oscuro:','fr':':femme_avec_la_paume_sur_le_visage_peau_foncée:','ja':':ひたいに手をあてる女_濃い肌色:','ko':':골치_아파하는_여자_검은색_피부:','pt':':mulher_decepcionada_pele_escura:','it':':donna_esasperata_carnagione_scura:','fa':':زن_با_کف_دست_به_پیشانی_می_زند_پوست_آبنوسی:','id':':wanita_tepuk_jidat_warna_kulit_gelap:','zh':':女生捂脸_较深肤色:','ru':':женщина_потеряла_дар_речи_очень_темныи_тон_кожи:','tr':':eliyle_yüzünü_kapatan_kadın_koyu_cilt_tonu:','ar':':سيدة_تشعر_بإحباط_بشرة_بلون_غامق:'},'\U0001F926\U0001F3FF\U0000200D\U00002640':{'en':':woman_facepalming_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':sich_an_den_kopf_fassende_frau_dunkle_hautfarbe:','es':':mujer_con_la_mano_en_la_frente_tono_de_piel_oscuro:','fr':':femme_avec_la_paume_sur_le_visage_peau_foncée:','ja':':ひたいに手をあてる女_濃い肌色:','ko':':골치_아파하는_여자_검은색_피부:','pt':':mulher_decepcionada_pele_escura:','it':':donna_esasperata_carnagione_scura:','fa':':زن_با_کف_دست_به_پیشانی_می_زند_پوست_آبنوسی:','id':':wanita_tepuk_jidat_warna_kulit_gelap:','zh':':女生捂脸_较深肤色:','ru':':женщина_потеряла_дар_речи_очень_темный_тон_кожи:','tr':':eliyle_yüzünü_kapatan_kadın_koyu_cilt_tonu:','ar':':سيدة_تشعر_بإحباط_بشرة_بلون_غامق:'},'\U0001F926\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_facepalming_light_skin_tone:','status':fully_qualified,'E':4,'de':':sich_an_den_kopf_fassende_frau_helle_hautfarbe:','es':':mujer_con_la_mano_en_la_frente_tono_de_piel_claro:','fr':':femme_avec_la_paume_sur_le_visage_peau_claire:','ja':':ひたいに手をあてる女_薄い肌色:','ko':':골치_아파하는_여자_하얀_피부:','pt':':mulher_decepcionada_pele_clara:','it':':donna_esasperata_carnagione_chiara:','fa':':زن_با_کف_دست_به_پیشانی_می_زند_پوست_سفید:','id':':wanita_tepuk_jidat_warna_kulit_cerah:','zh':':女生捂脸_较浅肤色:','ru':':женщина_потеряла_дар_речи_очень_светлыи_тон_кожи:','tr':':eliyle_yüzünü_kapatan_kadın_açık_cilt_tonu:','ar':':سيدة_تشعر_بإحباط_بشرة_بلون_فاتح:'},'\U0001F926\U0001F3FB\U0000200D\U00002640':{'en':':woman_facepalming_light_skin_tone:','status':minimally_qualified,'E':4,'de':':sich_an_den_kopf_fassende_frau_helle_hautfarbe:','es':':mujer_con_la_mano_en_la_frente_tono_de_piel_claro:','fr':':femme_avec_la_paume_sur_le_visage_peau_claire:','ja':':ひたいに手をあてる女_薄い肌色:','ko':':골치_아파하는_여자_하얀_피부:','pt':':mulher_decepcionada_pele_clara:','it':':donna_esasperata_carnagione_chiara:','fa':':زن_با_کف_دست_به_پیشانی_می_زند_پوست_سفید:','id':':wanita_tepuk_jidat_warna_kulit_cerah:','zh':':女生捂脸_较浅肤色:','ru':':женщина_потеряла_дар_речи_очень_светлый_тон_кожи:','tr':':eliyle_yüzünü_kapatan_kadın_açık_cilt_tonu:','ar':':سيدة_تشعر_بإحباط_بشرة_بلون_فاتح:'},'\U0001F926\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_facepalming_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':sich_an_den_kopf_fassende_frau_mitteldunkle_hautfarbe:','es':':mujer_con_la_mano_en_la_frente_tono_de_piel_oscuro_medio:','fr':':femme_avec_la_paume_sur_le_visage_peau_mate:','ja':':ひたいに手をあてる女_やや濃い肌色:','ko':':골치_아파하는_여자_진한_갈색_피부:','pt':':mulher_decepcionada_pele_morena_escura:','it':':donna_esasperata_carnagione_abbastanza_scura:','fa':':زن_با_کف_دست_به_پیشانی_می_زند_پوست_گندمی:','id':':wanita_tepuk_jidat_warna_kulit_gelap-sedang:','zh':':女生捂脸_中等深肤色:','ru':':женщина_потеряла_дар_речи_темныи_тон_кожи:','tr':':eliyle_yüzünü_kapatan_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_تشعر_بإحباط_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F926\U0001F3FE\U0000200D\U00002640':{'en':':woman_facepalming_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':sich_an_den_kopf_fassende_frau_mitteldunkle_hautfarbe:','es':':mujer_con_la_mano_en_la_frente_tono_de_piel_oscuro_medio:','fr':':femme_avec_la_paume_sur_le_visage_peau_mate:','ja':':ひたいに手をあてる女_やや濃い肌色:','ko':':골치_아파하는_여자_진한_갈색_피부:','pt':':mulher_decepcionada_pele_morena_escura:','it':':donna_esasperata_carnagione_abbastanza_scura:','fa':':زن_با_کف_دست_به_پیشانی_می_زند_پوست_گندمی:','id':':wanita_tepuk_jidat_warna_kulit_gelap-sedang:','zh':':女生捂脸_中等深肤色:','ru':':женщина_потеряла_дар_речи_темный_тон_кожи:','tr':':eliyle_yüzünü_kapatan_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_تشعر_بإحباط_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F926\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_facepalming_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':sich_an_den_kopf_fassende_frau_mittelhelle_hautfarbe:','es':':mujer_con_la_mano_en_la_frente_tono_de_piel_claro_medio:','fr':':femme_avec_la_paume_sur_le_visage_peau_moyennement_claire:','ja':':ひたいに手をあてる女_やや薄い肌色:','ko':':골치_아파하는_여자_연한_갈색_피부:','pt':':mulher_decepcionada_pele_morena_clara:','it':':donna_esasperata_carnagione_abbastanza_chiara:','fa':':زن_با_کف_دست_به_پیشانی_می_زند_پوست_روشن:','id':':wanita_tepuk_jidat_warna_kulit_cerah-sedang:','zh':':女生捂脸_中等浅肤色:','ru':':женщина_потеряла_дар_речи_светлыи_тон_кожи:','tr':':eliyle_yüzünü_kapatan_kadın_orta_açık_cilt_tonu:','ar':':سيدة_تشعر_بإحباط_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F926\U0001F3FC\U0000200D\U00002640':{'en':':woman_facepalming_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':sich_an_den_kopf_fassende_frau_mittelhelle_hautfarbe:','es':':mujer_con_la_mano_en_la_frente_tono_de_piel_claro_medio:','fr':':femme_avec_la_paume_sur_le_visage_peau_moyennement_claire:','ja':':ひたいに手をあてる女_やや薄い肌色:','ko':':골치_아파하는_여자_연한_갈색_피부:','pt':':mulher_decepcionada_pele_morena_clara:','it':':donna_esasperata_carnagione_abbastanza_chiara:','fa':':زن_با_کف_دست_به_پیشانی_می_زند_پوست_روشن:','id':':wanita_tepuk_jidat_warna_kulit_cerah-sedang:','zh':':女生捂脸_中等浅肤色:','ru':':женщина_потеряла_дар_речи_светлый_тон_кожи:','tr':':eliyle_yüzünü_kapatan_kadın_orta_açık_cilt_tonu:','ar':':سيدة_تشعر_بإحباط_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F926\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_facepalming_medium_skin_tone:','status':fully_qualified,'E':4,'de':':sich_an_den_kopf_fassende_frau_mittlere_hautfarbe:','es':':mujer_con_la_mano_en_la_frente_tono_de_piel_medio:','fr':':femme_avec_la_paume_sur_le_visage_peau_légèrement_mate:','ja':':ひたいに手をあてる女_中間の肌色:','ko':':골치_아파하는_여자_갈색_피부:','pt':':mulher_decepcionada_pele_morena:','it':':donna_esasperata_carnagione_olivastra:','fa':':زن_با_کف_دست_به_پیشانی_می_زند_پوست_طلایی:','id':':wanita_tepuk_jidat_warna_kulit_sedang:','zh':':女生捂脸_中等肤色:','ru':':женщина_потеряла_дар_речи_среднии_тон_кожи:','tr':':eliyle_yüzünü_kapatan_kadın_orta_cilt_tonu:','ar':':سيدة_تشعر_بإحباط_بشرة_بلون_معتدل:'},'\U0001F926\U0001F3FD\U0000200D\U00002640':{'en':':woman_facepalming_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':sich_an_den_kopf_fassende_frau_mittlere_hautfarbe:','es':':mujer_con_la_mano_en_la_frente_tono_de_piel_medio:','fr':':femme_avec_la_paume_sur_le_visage_peau_légèrement_mate:','ja':':ひたいに手をあてる女_中間の肌色:','ko':':골치_아파하는_여자_갈색_피부:','pt':':mulher_decepcionada_pele_morena:','it':':donna_esasperata_carnagione_olivastra:','fa':':زن_با_کف_دست_به_پیشانی_می_زند_پوست_طلایی:','id':':wanita_tepuk_jidat_warna_kulit_sedang:','zh':':女生捂脸_中等肤色:','ru':':женщина_потеряла_дар_речи_средний_тон_кожи:','tr':':eliyle_yüzünü_kapatan_kadın_orta_cilt_tonu:','ar':':سيدة_تشعر_بإحباط_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U0001F3ED':{'en':':woman_factory_worker:','status':fully_qualified,'E':4,'de':':fabrikarbeiterin:','es':':profesional_industrial_mujer:','fr':':ouvrière:','ja':':女性の溶接工:','ko':':공장_여자_직원:','pt':':operária:','it':':operaia:','fa':':کارگر_کارخانه_زن:','id':':pekerja_pabrik_wanita:','zh':':女工人:','ru':':работница:','tr':':kadın_fabrika_işçisi:','ar':':عاملة_مصنع:'},'\U0001F469\U0001F3FF\U0000200D\U0001F3ED':{'en':':woman_factory_worker_dark_skin_tone:','status':fully_qualified,'E':4,'de':':fabrikarbeiterin_dunkle_hautfarbe:','es':':profesional_industrial_mujer_tono_de_piel_oscuro:','fr':':ouvrière_peau_foncée:','ja':':女性の溶接工_濃い肌色:','ko':':공장_여자_직원_검은색_피부:','pt':':operária_pele_escura:','it':':operaia_carnagione_scura:','fa':':کارگر_کارخانه_زن_پوست_آبنوسی:','id':':pekerja_pabrik_wanita_warna_kulit_gelap:','zh':':女工人_较深肤色:','ru':':работница_очень_темный_тон_кожи:','tr':':kadın_fabrika_işçisi_koyu_cilt_tonu:','ar':':عاملة_مصنع_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F3ED':{'en':':woman_factory_worker_light_skin_tone:','status':fully_qualified,'E':4,'de':':fabrikarbeiterin_helle_hautfarbe:','es':':profesional_industrial_mujer_tono_de_piel_claro:','fr':':ouvrière_peau_claire:','ja':':女性の溶接工_薄い肌色:','ko':':공장_여자_직원_하얀_피부:','pt':':operária_pele_clara:','it':':operaia_carnagione_chiara:','fa':':کارگر_کارخانه_زن_پوست_سفید:','id':':pekerja_pabrik_wanita_warna_kulit_cerah:','zh':':女工人_较浅肤色:','ru':':работница_очень_светлый_тон_кожи:','tr':':kadın_fabrika_işçisi_açık_cilt_tonu:','ar':':عاملة_مصنع_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F3ED':{'en':':woman_factory_worker_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':fabrikarbeiterin_mitteldunkle_hautfarbe:','es':':profesional_industrial_mujer_tono_de_piel_oscuro_medio:','fr':':ouvrière_peau_mate:','ja':':女性の溶接工_やや濃い肌色:','ko':':공장_여자_직원_진한_갈색_피부:','pt':':operária_pele_morena_escura:','it':':operaia_carnagione_abbastanza_scura:','fa':':کارگر_کارخانه_زن_پوست_گندمی:','id':':pekerja_pabrik_wanita_warna_kulit_gelap-sedang:','zh':':女工人_中等深肤色:','ru':':работница_темный_тон_кожи:','tr':':kadın_fabrika_işçisi_orta_koyu_cilt_tonu:','ar':':عاملة_مصنع_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F3ED':{'en':':woman_factory_worker_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':fabrikarbeiterin_mittelhelle_hautfarbe:','es':':profesional_industrial_mujer_tono_de_piel_claro_medio:','fr':':ouvrière_peau_moyennement_claire:','ja':':女性の溶接工_やや薄い肌色:','ko':':공장_여자_직원_연한_갈색_피부:','pt':':operária_pele_morena_clara:','it':':operaia_carnagione_abbastanza_chiara:','fa':':کارگر_کارخانه_زن_پوست_روشن:','id':':pekerja_pabrik_wanita_warna_kulit_cerah-sedang:','zh':':女工人_中等浅肤色:','ru':':работница_светлый_тон_кожи:','tr':':kadın_fabrika_işçisi_orta_açık_cilt_tonu:','ar':':عاملة_مصنع_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F3ED':{'en':':woman_factory_worker_medium_skin_tone:','status':fully_qualified,'E':4,'de':':fabrikarbeiterin_mittlere_hautfarbe:','es':':profesional_industrial_mujer_tono_de_piel_medio:','fr':':ouvrière_peau_légèrement_mate:','ja':':女性の溶接工_中間の肌色:','ko':':공장_여자_직원_갈색_피부:','pt':':operária_pele_morena:','it':':operaia_carnagione_olivastra:','fa':':کارگر_کارخانه_زن_پوست_طلایی:','id':':pekerja_pabrik_wanita_warna_kulit_sedang:','zh':':女工人_中等肤色:','ru':':работница_средний_тон_кожи:','tr':':kadın_fabrika_işçisi_orta_cilt_tonu:','ar':':عاملة_مصنع_بشرة_بلون_معتدل:'},'\U0001F9DA\U0000200D\U00002640\U0000FE0F':{'en':':woman_fairy:','status':fully_qualified,'E':5,'alias':[':fairy_woman:'],'de':':fee:','es':':hada_mujer:','fr':':fée:','ja':':女の妖精:','ko':':여자_요정:','pt':':mulher_fada:','it':':fata_donna:','fa':':پری_زن:','id':':peri_wanita:','zh':':仙女:','ru':':женщина-фея:','tr':':kadın_peri:','ar':':جنية:'},'\U0001F9DA\U0000200D\U00002640':{'en':':woman_fairy:','status':minimally_qualified,'E':5,'alias':[':fairy_woman:'],'de':':fee:','es':':hada_mujer:','fr':':fée:','ja':':女の妖精:','ko':':여자_요정:','pt':':mulher_fada:','it':':fata_donna:','fa':':پری_زن:','id':':peri_wanita:','zh':':仙女:','ru':':женщина_фея:','tr':':kadın_peri:','ar':':جنية:'},'\U0001F9DA\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_fairy_dark_skin_tone:','status':fully_qualified,'E':5,'de':':fee_dunkle_hautfarbe:','es':':hada_mujer_tono_de_piel_oscuro:','fr':':fée_peau_foncée:','ja':':女の妖精_濃い肌色:','ko':':여자_요정_검은색_피부:','pt':':mulher_fada_pele_escura:','it':':fata_donna_carnagione_scura:','fa':':پری_زن_پوست_آبنوسی:','id':':peri_wanita_warna_kulit_gelap:','zh':':仙女_较深肤色:','ru':':женщина-фея_очень_темныи_тон_кожи:','tr':':kadın_peri_koyu_cilt_tonu:','ar':':جنية_بشرة_بلون_غامق:'},'\U0001F9DA\U0001F3FF\U0000200D\U00002640':{'en':':woman_fairy_dark_skin_tone:','status':minimally_qualified,'E':5,'de':':fee_dunkle_hautfarbe:','es':':hada_mujer_tono_de_piel_oscuro:','fr':':fée_peau_foncée:','ja':':女の妖精_濃い肌色:','ko':':여자_요정_검은색_피부:','pt':':mulher_fada_pele_escura:','it':':fata_donna_carnagione_scura:','fa':':پری_زن_پوست_آبنوسی:','id':':peri_wanita_warna_kulit_gelap:','zh':':仙女_较深肤色:','ru':':женщина_фея_очень_темный_тон_кожи:','tr':':kadın_peri_koyu_cilt_tonu:','ar':':جنية_بشرة_بلون_غامق:'},'\U0001F9DA\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_fairy_light_skin_tone:','status':fully_qualified,'E':5,'de':':fee_helle_hautfarbe:','es':':hada_mujer_tono_de_piel_claro:','fr':':fée_peau_claire:','ja':':女の妖精_薄い肌色:','ko':':여자_요정_하얀_피부:','pt':':mulher_fada_pele_clara:','it':':fata_donna_carnagione_chiara:','fa':':پری_زن_پوست_سفید:','id':':peri_wanita_warna_kulit_cerah:','zh':':仙女_较浅肤色:','ru':':женщина-фея_очень_светлыи_тон_кожи:','tr':':kadın_peri_açık_cilt_tonu:','ar':':جنية_بشرة_بلون_فاتح:'},'\U0001F9DA\U0001F3FB\U0000200D\U00002640':{'en':':woman_fairy_light_skin_tone:','status':minimally_qualified,'E':5,'de':':fee_helle_hautfarbe:','es':':hada_mujer_tono_de_piel_claro:','fr':':fée_peau_claire:','ja':':女の妖精_薄い肌色:','ko':':여자_요정_하얀_피부:','pt':':mulher_fada_pele_clara:','it':':fata_donna_carnagione_chiara:','fa':':پری_زن_پوست_سفید:','id':':peri_wanita_warna_kulit_cerah:','zh':':仙女_较浅肤色:','ru':':женщина_фея_очень_светлый_тон_кожи:','tr':':kadın_peri_açık_cilt_tonu:','ar':':جنية_بشرة_بلون_فاتح:'},'\U0001F9DA\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_fairy_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':fee_mitteldunkle_hautfarbe:','es':':hada_mujer_tono_de_piel_oscuro_medio:','fr':':fée_peau_mate:','ja':':女の妖精_やや濃い肌色:','ko':':여자_요정_진한_갈색_피부:','pt':':mulher_fada_pele_morena_escura:','it':':fata_donna_carnagione_abbastanza_scura:','fa':':پری_زن_پوست_گندمی:','id':':peri_wanita_warna_kulit_gelap-sedang:','zh':':仙女_中等深肤色:','ru':':женщина-фея_темныи_тон_кожи:','tr':':kadın_peri_orta_koyu_cilt_tonu:','ar':':جنية_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9DA\U0001F3FE\U0000200D\U00002640':{'en':':woman_fairy_medium-dark_skin_tone:','status':minimally_qualified,'E':5,'de':':fee_mitteldunkle_hautfarbe:','es':':hada_mujer_tono_de_piel_oscuro_medio:','fr':':fée_peau_mate:','ja':':女の妖精_やや濃い肌色:','ko':':여자_요정_진한_갈색_피부:','pt':':mulher_fada_pele_morena_escura:','it':':fata_donna_carnagione_abbastanza_scura:','fa':':پری_زن_پوست_گندمی:','id':':peri_wanita_warna_kulit_gelap-sedang:','zh':':仙女_中等深肤色:','ru':':женщина_фея_темный_тон_кожи:','tr':':kadın_peri_orta_koyu_cilt_tonu:','ar':':جنية_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9DA\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_fairy_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':fee_mittelhelle_hautfarbe:','es':':hada_mujer_tono_de_piel_claro_medio:','fr':':fée_peau_moyennement_claire:','ja':':女の妖精_やや薄い肌色:','ko':':여자_요정_연한_갈색_피부:','pt':':mulher_fada_pele_morena_clara:','it':':fata_donna_carnagione_abbastanza_chiara:','fa':':پری_زن_پوست_روشن:','id':':peri_wanita_warna_kulit_cerah-sedang:','zh':':仙女_中等浅肤色:','ru':':женщина-фея_светлыи_тон_кожи:','tr':':kadın_peri_orta_açık_cilt_tonu:','ar':':جنية_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9DA\U0001F3FC\U0000200D\U00002640':{'en':':woman_fairy_medium-light_skin_tone:','status':minimally_qualified,'E':5,'de':':fee_mittelhelle_hautfarbe:','es':':hada_mujer_tono_de_piel_claro_medio:','fr':':fée_peau_moyennement_claire:','ja':':女の妖精_やや薄い肌色:','ko':':여자_요정_연한_갈색_피부:','pt':':mulher_fada_pele_morena_clara:','it':':fata_donna_carnagione_abbastanza_chiara:','fa':':پری_زن_پوست_روشن:','id':':peri_wanita_warna_kulit_cerah-sedang:','zh':':仙女_中等浅肤色:','ru':':женщина_фея_светлый_тон_кожи:','tr':':kadın_peri_orta_açık_cilt_tonu:','ar':':جنية_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9DA\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_fairy_medium_skin_tone:','status':fully_qualified,'E':5,'de':':fee_mittlere_hautfarbe:','es':':hada_mujer_tono_de_piel_medio:','fr':':fée_peau_légèrement_mate:','ja':':女の妖精_中間の肌色:','ko':':여자_요정_갈색_피부:','pt':':mulher_fada_pele_morena:','it':':fata_donna_carnagione_olivastra:','fa':':پری_زن_پوست_طلایی:','id':':peri_wanita_warna_kulit_sedang:','zh':':仙女_中等肤色:','ru':':женщина-фея_среднии_тон_кожи:','tr':':kadın_peri_orta_cilt_tonu:','ar':':جنية_بشرة_بلون_معتدل:'},'\U0001F9DA\U0001F3FD\U0000200D\U00002640':{'en':':woman_fairy_medium_skin_tone:','status':minimally_qualified,'E':5,'de':':fee_mittlere_hautfarbe:','es':':hada_mujer_tono_de_piel_medio:','fr':':fée_peau_légèrement_mate:','ja':':女の妖精_中間の肌色:','ko':':여자_요정_갈색_피부:','pt':':mulher_fada_pele_morena:','it':':fata_donna_carnagione_olivastra:','fa':':پری_زن_پوست_طلایی:','id':':peri_wanita_warna_kulit_sedang:','zh':':仙女_中等肤色:','ru':':женщина_фея_средний_тон_кожи:','tr':':kadın_peri_orta_cilt_tonu:','ar':':جنية_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U0001F33E':{'en':':woman_farmer:','status':fully_qualified,'E':4,'de':':bäuerin:','es':':profesional_de_la_agricultura_mujer:','fr':':fermière:','ja':':農家の女性:','ko':':여자_농부:','pt':':fazendeira:','it':':contadina:','fa':':کشاورز_زن:','id':':petani_wanita:','zh':':农妇:','ru':':женщина_фермер:','tr':':kadın_çiftçi:','ar':':مزارعة:'},'\U0001F469\U0001F3FF\U0000200D\U0001F33E':{'en':':woman_farmer_dark_skin_tone:','status':fully_qualified,'E':4,'de':':bäuerin_dunkle_hautfarbe:','es':':profesional_de_la_agricultura_mujer_tono_de_piel_oscuro:','fr':':fermière_peau_foncée:','ja':':農家の女性_濃い肌色:','ko':':여자_농부_검은색_피부:','pt':':fazendeira_pele_escura:','it':':contadina_carnagione_scura:','fa':':کشاورز_زن_پوست_آبنوسی:','id':':petani_wanita_warna_kulit_gelap:','zh':':农妇_较深肤色:','ru':':женщина_фермер_очень_темный_тон_кожи:','tr':':kadın_çiftçi_koyu_cilt_tonu:','ar':':مزارعة_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F33E':{'en':':woman_farmer_light_skin_tone:','status':fully_qualified,'E':4,'de':':bäuerin_helle_hautfarbe:','es':':profesional_de_la_agricultura_mujer_tono_de_piel_claro:','fr':':fermière_peau_claire:','ja':':農家の女性_薄い肌色:','ko':':여자_농부_하얀_피부:','pt':':fazendeira_pele_clara:','it':':contadina_carnagione_chiara:','fa':':کشاورز_زن_پوست_سفید:','id':':petani_wanita_warna_kulit_cerah:','zh':':农妇_较浅肤色:','ru':':женщина_фермер_очень_светлый_тон_кожи:','tr':':kadın_çiftçi_açık_cilt_tonu:','ar':':مزارعة_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F33E':{'en':':woman_farmer_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':bäuerin_mitteldunkle_hautfarbe:','es':':profesional_de_la_agricultura_mujer_tono_de_piel_oscuro_medio:','fr':':fermière_peau_mate:','ja':':農家の女性_やや濃い肌色:','ko':':여자_농부_진한_갈색_피부:','pt':':fazendeira_pele_morena_escura:','it':':contadina_carnagione_abbastanza_scura:','fa':':کشاورز_زن_پوست_گندمی:','id':':petani_wanita_warna_kulit_gelap-sedang:','zh':':农妇_中等深肤色:','ru':':женщина_фермер_темный_тон_кожи:','tr':':kadın_çiftçi_orta_koyu_cilt_tonu:','ar':':مزارعة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F33E':{'en':':woman_farmer_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':bäuerin_mittelhelle_hautfarbe:','es':':profesional_de_la_agricultura_mujer_tono_de_piel_claro_medio:','fr':':fermière_peau_moyennement_claire:','ja':':農家の女性_やや薄い肌色:','ko':':여자_농부_연한_갈색_피부:','pt':':fazendeira_pele_morena_clara:','it':':contadina_carnagione_abbastanza_chiara:','fa':':کشاورز_زن_پوست_روشن:','id':':petani_wanita_warna_kulit_cerah-sedang:','zh':':农妇_中等浅肤色:','ru':':женщина_фермер_светлый_тон_кожи:','tr':':kadın_çiftçi_orta_açık_cilt_tonu:','ar':':مزارعة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F33E':{'en':':woman_farmer_medium_skin_tone:','status':fully_qualified,'E':4,'de':':bäuerin_mittlere_hautfarbe:','es':':profesional_de_la_agricultura_mujer_tono_de_piel_medio:','fr':':fermière_peau_légèrement_mate:','ja':':農家の女性_中間の肌色:','ko':':여자_농부_갈색_피부:','pt':':fazendeira_pele_morena:','it':':contadina_carnagione_olivastra:','fa':':کشاورز_زن_پوست_طلایی:','id':':petani_wanita_warna_kulit_sedang:','zh':':农妇_中等肤色:','ru':':женщина_фермер_средний_тон_кожи:','tr':':kadın_çiftçi_orta_cilt_tonu:','ar':':مزارعة_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U0001F37C':{'en':':woman_feeding_baby:','status':fully_qualified,'E':13,'de':':flasche_gebende_frau:','es':':mujer_alimentando_a_bebé:','fr':':femme_allaitant_un_bébé:','ja':':授乳する女性:','ko':':수유_중인_여자:','pt':':mulher_alimentando_bebê:','it':':donna_che_allatta:','fa':':زنی_که_به_بچه_شیر_می_دهد:','id':':wanita_menyusui_bayi:','zh':':哺乳的女人:','ru':':кормящая_женщина:','tr':':bebek_emziren_kadın:','ar':':امرأة_تُرضع_طفلا:'},'\U0001F469\U0001F3FF\U0000200D\U0001F37C':{'en':':woman_feeding_baby_dark_skin_tone:','status':fully_qualified,'E':13,'de':':flasche_gebende_frau_dunkle_hautfarbe:','es':':mujer_alimentando_a_bebé_tono_de_piel_oscuro:','fr':':femme_allaitant_un_bébé_peau_foncée:','ja':':授乳する女性_濃い肌色:','ko':':수유_중인_여자_검은색_피부:','pt':':mulher_alimentando_bebê_pele_escura:','it':':donna_che_allatta_carnagione_scura:','fa':':زنی_که_به_بچه_شیر_می_دهد_پوست_آبنوسی:','id':':wanita_menyusui_bayi_warna_kulit_gelap:','zh':':哺乳的女人_较深肤色:','ru':':кормящая_женщина_очень_темный_тон_кожи:','tr':':bebek_emziren_kadın_koyu_cilt_tonu:','ar':':امرأة_تُرضع_طفلا_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F37C':{'en':':woman_feeding_baby_light_skin_tone:','status':fully_qualified,'E':13,'de':':flasche_gebende_frau_helle_hautfarbe:','es':':mujer_alimentando_a_bebé_tono_de_piel_claro:','fr':':femme_allaitant_un_bébé_peau_claire:','ja':':授乳する女性_薄い肌色:','ko':':수유_중인_여자_하얀_피부:','pt':':mulher_alimentando_bebê_pele_clara:','it':':donna_che_allatta_carnagione_chiara:','fa':':زنی_که_به_بچه_شیر_می_دهد_پوست_سفید:','id':':wanita_menyusui_bayi_warna_kulit_cerah:','zh':':哺乳的女人_较浅肤色:','ru':':кормящая_женщина_очень_светлый_тон_кожи:','tr':':bebek_emziren_kadın_açık_cilt_tonu:','ar':':امرأة_تُرضع_طفلا_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F37C':{'en':':woman_feeding_baby_medium-dark_skin_tone:','status':fully_qualified,'E':13,'de':':flasche_gebende_frau_mitteldunkle_hautfarbe:','es':':mujer_alimentando_a_bebé_tono_de_piel_oscuro_medio:','fr':':femme_allaitant_un_bébé_peau_mate:','ja':':授乳する女性_やや濃い肌色:','ko':':수유_중인_여자_진한_갈색_피부:','pt':':mulher_alimentando_bebê_pele_morena_escura:','it':':donna_che_allatta_carnagione_abbastanza_scura:','fa':':زنی_که_به_بچه_شیر_می_دهد_پوست_گندمی:','id':':wanita_menyusui_bayi_warna_kulit_gelap-sedang:','zh':':哺乳的女人_中等深肤色:','ru':':кормящая_женщина_темный_тон_кожи:','tr':':bebek_emziren_kadın_orta_koyu_cilt_tonu:','ar':':امرأة_تُرضع_طفلا_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F37C':{'en':':woman_feeding_baby_medium-light_skin_tone:','status':fully_qualified,'E':13,'de':':flasche_gebende_frau_mittelhelle_hautfarbe:','es':':mujer_alimentando_a_bebé_tono_de_piel_claro_medio:','fr':':femme_allaitant_un_bébé_peau_moyennement_claire:','ja':':授乳する女性_やや薄い肌色:','ko':':수유_중인_여자_연한_갈색_피부:','pt':':mulher_alimentando_bebê_pele_morena_clara:','it':':donna_che_allatta_carnagione_abbastanza_chiara:','fa':':زنی_که_به_بچه_شیر_می_دهد_پوست_روشن:','id':':wanita_menyusui_bayi_warna_kulit_cerah-sedang:','zh':':哺乳的女人_中等浅肤色:','ru':':кормящая_женщина_светлый_тон_кожи:','tr':':bebek_emziren_kadın_orta_açık_cilt_tonu:','ar':':امرأة_تُرضع_طفلا_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F37C':{'en':':woman_feeding_baby_medium_skin_tone:','status':fully_qualified,'E':13,'de':':flasche_gebende_frau_mittlere_hautfarbe:','es':':mujer_alimentando_a_bebé_tono_de_piel_medio:','fr':':femme_allaitant_un_bébé_peau_légèrement_mate:','ja':':授乳する女性_中間の肌色:','ko':':수유_중인_여자_갈색_피부:','pt':':mulher_alimentando_bebê_pele_morena:','it':':donna_che_allatta_carnagione_olivastra:','fa':':زنی_که_به_بچه_شیر_می_دهد_پوست_طلایی:','id':':wanita_menyusui_bayi_warna_kulit_sedang:','zh':':哺乳的女人_中等肤色:','ru':':кормящая_женщина_средний_тон_кожи:','tr':':bebek_emziren_kadın_orta_cilt_tonu:','ar':':امرأة_تُرضع_طفلا_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U0001F692':{'en':':woman_firefighter:','status':fully_qualified,'E':4,'de':':feuerwehrfrau:','es':':bombera:','fr':':pompier_femme:','ja':':女性消防士:','ko':':여자_소방관:','pt':':bombeira:','it':':pompiere_donna:','fa':':آتش_نشان_زن:','id':':pemadam_kebakaran_wanita:','zh':':女消防员:','ru':':женщина_пожарный:','tr':':kadın_itfaiyeci:','ar':':سيدة_إطفاء:'},'\U0001F469\U0001F3FF\U0000200D\U0001F692':{'en':':woman_firefighter_dark_skin_tone:','status':fully_qualified,'E':4,'de':':feuerwehrfrau_dunkle_hautfarbe:','es':':bombera_tono_de_piel_oscuro:','fr':':pompier_femme_peau_foncée:','ja':':女性消防士_濃い肌色:','ko':':여자_소방관_검은색_피부:','pt':':bombeira_pele_escura:','it':':pompiere_donna_carnagione_scura:','fa':':آتش_نشان_زن_پوست_آبنوسی:','id':':pemadam_kebakaran_wanita_warna_kulit_gelap:','zh':':女消防员_较深肤色:','ru':':женщина_пожарный_очень_темный_тон_кожи:','tr':':kadın_itfaiyeci_koyu_cilt_tonu:','ar':':سيدة_إطفاء_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F692':{'en':':woman_firefighter_light_skin_tone:','status':fully_qualified,'E':4,'de':':feuerwehrfrau_helle_hautfarbe:','es':':bombera_tono_de_piel_claro:','fr':':pompier_femme_peau_claire:','ja':':女性消防士_薄い肌色:','ko':':여자_소방관_하얀_피부:','pt':':bombeira_pele_clara:','it':':pompiere_donna_carnagione_chiara:','fa':':آتش_نشان_زن_پوست_سفید:','id':':pemadam_kebakaran_wanita_warna_kulit_cerah:','zh':':女消防员_较浅肤色:','ru':':женщина_пожарный_очень_светлый_тон_кожи:','tr':':kadın_itfaiyeci_açık_cilt_tonu:','ar':':سيدة_إطفاء_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F692':{'en':':woman_firefighter_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':feuerwehrfrau_mitteldunkle_hautfarbe:','es':':bombera_tono_de_piel_oscuro_medio:','fr':':pompier_femme_peau_mate:','ja':':女性消防士_やや濃い肌色:','ko':':여자_소방관_진한_갈색_피부:','pt':':bombeira_pele_morena_escura:','it':':pompiere_donna_carnagione_abbastanza_scura:','fa':':آتش_نشان_زن_پوست_گندمی:','id':':pemadam_kebakaran_wanita_warna_kulit_gelap-sedang:','zh':':女消防员_中等深肤色:','ru':':женщина_пожарный_темный_тон_кожи:','tr':':kadın_itfaiyeci_orta_koyu_cilt_tonu:','ar':':سيدة_إطفاء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F692':{'en':':woman_firefighter_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':feuerwehrfrau_mittelhelle_hautfarbe:','es':':bombera_tono_de_piel_claro_medio:','fr':':pompier_femme_peau_moyennement_claire:','ja':':女性消防士_やや薄い肌色:','ko':':여자_소방관_연한_갈색_피부:','pt':':bombeira_pele_morena_clara:','it':':pompiere_donna_carnagione_abbastanza_chiara:','fa':':آتش_نشان_زن_پوست_روشن:','id':':pemadam_kebakaran_wanita_warna_kulit_cerah-sedang:','zh':':女消防员_中等浅肤色:','ru':':женщина_пожарный_светлый_тон_кожи:','tr':':kadın_itfaiyeci_orta_açık_cilt_tonu:','ar':':سيدة_إطفاء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F692':{'en':':woman_firefighter_medium_skin_tone:','status':fully_qualified,'E':4,'de':':feuerwehrfrau_mittlere_hautfarbe:','es':':bombera_tono_de_piel_medio:','fr':':pompier_femme_peau_légèrement_mate:','ja':':女性消防士_中間の肌色:','ko':':여자_소방관_갈색_피부:','pt':':bombeira_pele_morena:','it':':pompiere_donna_carnagione_olivastra:','fa':':آتش_نشان_زن_پوست_طلایی:','id':':pemadam_kebakaran_wanita_warna_kulit_sedang:','zh':':女消防员_中等肤色:','ru':':женщина_пожарный_средний_тон_кожи:','tr':':kadın_itfaiyeci_orta_cilt_tonu:','ar':':سيدة_إطفاء_بشرة_بلون_معتدل:'},'\U0001F64D\U0000200D\U00002640\U0000FE0F':{'en':':woman_frowning:','status':fully_qualified,'E':4,'alias':[':frowning_woman:'],'de':':missmutige_frau:','es':':mujer_frunciendo_el_ceño:','fr':':femme_fronçant_les_sourcils:','ja':':しかめ面の女:','ko':':찌푸린_여자:','pt':':mulher_franzindo_a_sobrancelha:','it':':donna_corrucciata:','fa':':زن_اخم_می_کند:','id':':wanita_muka_tidak_senang:','zh':':皱眉女:','ru':':нахмурившаяся_женщина:','tr':':somurtan_kadın:','ar':':متجهمة:'},'\U0001F64D\U0000200D\U00002640':{'en':':woman_frowning:','status':minimally_qualified,'E':4,'alias':[':frowning_woman:'],'de':':missmutige_frau:','es':':mujer_frunciendo_el_ceño:','fr':':femme_fronçant_les_sourcils:','ja':':しかめ面の女:','ko':':찌푸린_여자:','pt':':mulher_franzindo_a_sobrancelha:','it':':donna_corrucciata:','fa':':زن_اخم_می_کند:','id':':wanita_muka_tidak_senang:','zh':':皱眉女:','ru':':нахмурившаяся_женщина:','tr':':somurtan_kadın:','ar':':متجهمة:'},'\U0001F64D\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_frowning_dark_skin_tone:','status':fully_qualified,'E':4,'de':':missmutige_frau_dunkle_hautfarbe:','es':':mujer_frunciendo_el_ceño_tono_de_piel_oscuro:','fr':':femme_fronçant_les_sourcils_peau_foncée:','ja':':しかめ面の女_濃い肌色:','ko':':찌푸린_여자_검은색_피부:','pt':':mulher_franzindo_a_sobrancelha_pele_escura:','it':':donna_corrucciata_carnagione_scura:','fa':':زن_اخم_می_کند_پوست_آبنوسی:','id':':wanita_muka_tidak_senang_warna_kulit_gelap:','zh':':皱眉女_较深肤色:','ru':':нахмурившаяся_женщина_очень_темныи_тон_кожи:','tr':':somurtan_kadın_koyu_cilt_tonu:','ar':':متجهمة_بشرة_بلون_غامق:'},'\U0001F64D\U0001F3FF\U0000200D\U00002640':{'en':':woman_frowning_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':missmutige_frau_dunkle_hautfarbe:','es':':mujer_frunciendo_el_ceño_tono_de_piel_oscuro:','fr':':femme_fronçant_les_sourcils_peau_foncée:','ja':':しかめ面の女_濃い肌色:','ko':':찌푸린_여자_검은색_피부:','pt':':mulher_franzindo_a_sobrancelha_pele_escura:','it':':donna_corrucciata_carnagione_scura:','fa':':زن_اخم_می_کند_پوست_آبنوسی:','id':':wanita_muka_tidak_senang_warna_kulit_gelap:','zh':':皱眉女_较深肤色:','ru':':нахмурившаяся_женщина_очень_темный_тон_кожи:','tr':':somurtan_kadın_koyu_cilt_tonu:','ar':':متجهمة_بشرة_بلون_غامق:'},'\U0001F64D\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_frowning_light_skin_tone:','status':fully_qualified,'E':4,'de':':missmutige_frau_helle_hautfarbe:','es':':mujer_frunciendo_el_ceño_tono_de_piel_claro:','fr':':femme_fronçant_les_sourcils_peau_claire:','ja':':しかめ面の女_薄い肌色:','ko':':찌푸린_여자_하얀_피부:','pt':':mulher_franzindo_a_sobrancelha_pele_clara:','it':':donna_corrucciata_carnagione_chiara:','fa':':زن_اخم_می_کند_پوست_سفید:','id':':wanita_muka_tidak_senang_warna_kulit_cerah:','zh':':皱眉女_较浅肤色:','ru':':нахмурившаяся_женщина_очень_светлыи_тон_кожи:','tr':':somurtan_kadın_açık_cilt_tonu:','ar':':متجهمة_بشرة_بلون_فاتح:'},'\U0001F64D\U0001F3FB\U0000200D\U00002640':{'en':':woman_frowning_light_skin_tone:','status':minimally_qualified,'E':4,'de':':missmutige_frau_helle_hautfarbe:','es':':mujer_frunciendo_el_ceño_tono_de_piel_claro:','fr':':femme_fronçant_les_sourcils_peau_claire:','ja':':しかめ面の女_薄い肌色:','ko':':찌푸린_여자_하얀_피부:','pt':':mulher_franzindo_a_sobrancelha_pele_clara:','it':':donna_corrucciata_carnagione_chiara:','fa':':زن_اخم_می_کند_پوست_سفید:','id':':wanita_muka_tidak_senang_warna_kulit_cerah:','zh':':皱眉女_较浅肤色:','ru':':нахмурившаяся_женщина_очень_светлый_тон_кожи:','tr':':somurtan_kadın_açık_cilt_tonu:','ar':':متجهمة_بشرة_بلون_فاتح:'},'\U0001F64D\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_frowning_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':missmutige_frau_mitteldunkle_hautfarbe:','es':':mujer_frunciendo_el_ceño_tono_de_piel_oscuro_medio:','fr':':femme_fronçant_les_sourcils_peau_mate:','ja':':しかめ面の女_やや濃い肌色:','ko':':찌푸린_여자_진한_갈색_피부:','pt':':mulher_franzindo_a_sobrancelha_pele_morena_escura:','it':':donna_corrucciata_carnagione_abbastanza_scura:','fa':':زن_اخم_می_کند_پوست_گندمی:','id':':wanita_muka_tidak_senang_warna_kulit_gelap-sedang:','zh':':皱眉女_中等深肤色:','ru':':нахмурившаяся_женщина_темныи_тон_кожи:','tr':':somurtan_kadın_orta_koyu_cilt_tonu:','ar':':متجهمة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F64D\U0001F3FE\U0000200D\U00002640':{'en':':woman_frowning_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':missmutige_frau_mitteldunkle_hautfarbe:','es':':mujer_frunciendo_el_ceño_tono_de_piel_oscuro_medio:','fr':':femme_fronçant_les_sourcils_peau_mate:','ja':':しかめ面の女_やや濃い肌色:','ko':':찌푸린_여자_진한_갈색_피부:','pt':':mulher_franzindo_a_sobrancelha_pele_morena_escura:','it':':donna_corrucciata_carnagione_abbastanza_scura:','fa':':زن_اخم_می_کند_پوست_گندمی:','id':':wanita_muka_tidak_senang_warna_kulit_gelap-sedang:','zh':':皱眉女_中等深肤色:','ru':':нахмурившаяся_женщина_темный_тон_кожи:','tr':':somurtan_kadın_orta_koyu_cilt_tonu:','ar':':متجهمة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F64D\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_frowning_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':missmutige_frau_mittelhelle_hautfarbe:','es':':mujer_frunciendo_el_ceño_tono_de_piel_claro_medio:','fr':':femme_fronçant_les_sourcils_peau_moyennement_claire:','ja':':しかめ面の女_やや薄い肌色:','ko':':찌푸린_여자_연한_갈색_피부:','pt':':mulher_franzindo_a_sobrancelha_pele_morena_clara:','it':':donna_corrucciata_carnagione_abbastanza_chiara:','fa':':زن_اخم_می_کند_پوست_روشن:','id':':wanita_muka_tidak_senang_warna_kulit_cerah-sedang:','zh':':皱眉女_中等浅肤色:','ru':':нахмурившаяся_женщина_светлыи_тон_кожи:','tr':':somurtan_kadın_orta_açık_cilt_tonu:','ar':':متجهمة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F64D\U0001F3FC\U0000200D\U00002640':{'en':':woman_frowning_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':missmutige_frau_mittelhelle_hautfarbe:','es':':mujer_frunciendo_el_ceño_tono_de_piel_claro_medio:','fr':':femme_fronçant_les_sourcils_peau_moyennement_claire:','ja':':しかめ面の女_やや薄い肌色:','ko':':찌푸린_여자_연한_갈색_피부:','pt':':mulher_franzindo_a_sobrancelha_pele_morena_clara:','it':':donna_corrucciata_carnagione_abbastanza_chiara:','fa':':زن_اخم_می_کند_پوست_روشن:','id':':wanita_muka_tidak_senang_warna_kulit_cerah-sedang:','zh':':皱眉女_中等浅肤色:','ru':':нахмурившаяся_женщина_светлый_тон_кожи:','tr':':somurtan_kadın_orta_açık_cilt_tonu:','ar':':متجهمة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F64D\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_frowning_medium_skin_tone:','status':fully_qualified,'E':4,'de':':missmutige_frau_mittlere_hautfarbe:','es':':mujer_frunciendo_el_ceño_tono_de_piel_medio:','fr':':femme_fronçant_les_sourcils_peau_légèrement_mate:','ja':':しかめ面の女_中間の肌色:','ko':':찌푸린_여자_갈색_피부:','pt':':mulher_franzindo_a_sobrancelha_pele_morena:','it':':donna_corrucciata_carnagione_olivastra:','fa':':زن_اخم_می_کند_پوست_طلایی:','id':':wanita_muka_tidak_senang_warna_kulit_sedang:','zh':':皱眉女_中等肤色:','ru':':нахмурившаяся_женщина_среднии_тон_кожи:','tr':':somurtan_kadın_orta_cilt_tonu:','ar':':متجهمة_بشرة_بلون_معتدل:'},'\U0001F64D\U0001F3FD\U0000200D\U00002640':{'en':':woman_frowning_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':missmutige_frau_mittlere_hautfarbe:','es':':mujer_frunciendo_el_ceño_tono_de_piel_medio:','fr':':femme_fronçant_les_sourcils_peau_légèrement_mate:','ja':':しかめ面の女_中間の肌色:','ko':':찌푸린_여자_갈색_피부:','pt':':mulher_franzindo_a_sobrancelha_pele_morena:','it':':donna_corrucciata_carnagione_olivastra:','fa':':زن_اخم_می_کند_پوست_طلایی:','id':':wanita_muka_tidak_senang_warna_kulit_sedang:','zh':':皱眉女_中等肤色:','ru':':нахмурившаяся_женщина_средний_тон_кожи:','tr':':somurtan_kadın_orta_cilt_tonu:','ar':':متجهمة_بشرة_بلون_معتدل:'},'\U0001F9DE\U0000200D\U00002640\U0000FE0F':{'en':':woman_genie:','status':fully_qualified,'E':5,'alias':[':genie_woman:'],'de':':weiblicher_flaschengeist:','es':':genio_mujer:','fr':':génie_femme:','ja':':女の精霊:','ko':':여자_지니:','pt':':mulher_gênio:','it':':genio_donna:','fa':':زن_جن:','id':':jin_wanita:','zh':':女妖怪:','ru':':женщина-джинн:','tr':':kadın_cin:','ar':':ماردة:'},'\U0001F9DE\U0000200D\U00002640':{'en':':woman_genie:','status':minimally_qualified,'E':5,'alias':[':genie_woman:'],'de':':weiblicher_flaschengeist:','es':':genio_mujer:','fr':':génie_femme:','ja':':女の精霊:','ko':':여자_지니:','pt':':mulher_gênio:','it':':genio_donna:','fa':':زن_جن:','id':':jin_wanita:','zh':':女妖怪:','ru':':женщина_джинн:','tr':':kadın_cin:','ar':':ماردة:'},'\U0001F645\U0000200D\U00002640\U0000FE0F':{'en':':woman_gesturing_NO:','status':fully_qualified,'E':4,'alias':[':ng_woman:', ':no_good_woman:', ':woman_gesturing_no:'],'de':':frau_mit_überkreuzten_armen:','es':':mujer_haciendo_el_gesto_de_no:','fr':':femme_faisant_un_geste_d’interdiction:','ja':':ダメのポーズをする女:','ko':':안_된다는_제스처를_하는_여자:','pt':':mulher_fazendo_gesto_de_“não”:','it':':donna_con_gesto_di_rifiuto:','fa':':زن_نمی_پذیرد:','id':':wanita_mengisyaratkan_tidak:','zh':':禁止手势女:','ru':':женщина_против:','tr':':hayır_hareketi_yapan_kadın:','ar':':غير_موافقة:'},'\U0001F645\U0000200D\U00002640':{'en':':woman_gesturing_NO:','status':minimally_qualified,'E':4,'alias':[':ng_woman:', ':no_good_woman:', ':woman_gesturing_no:'],'de':':frau_mit_überkreuzten_armen:','es':':mujer_haciendo_el_gesto_de_no:','fr':':femme_faisant_un_geste_d’interdiction:','ja':':ダメのポーズをする女:','ko':':안_된다는_제스처를_하는_여자:','pt':':mulher_fazendo_gesto_de_“não”:','it':':donna_con_gesto_di_rifiuto:','fa':':زن_نمی_پذیرد:','id':':wanita_mengisyaratkan_tidak:','zh':':禁止手势女:','ru':':женщина_против:','tr':':hayır_hareketi_yapan_kadın:','ar':':غير_موافقة:'},'\U0001F645\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_gesturing_NO_dark_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_überkreuzten_armen_dunkle_hautfarbe:','es':':mujer_haciendo_el_gesto_de_no_tono_de_piel_oscuro:','fr':':femme_faisant_un_geste_d’interdiction_peau_foncée:','ja':':ダメのポーズをする女_濃い肌色:','ko':':안_된다는_제스처를_하는_여자_검은색_피부:','pt':':mulher_fazendo_gesto_de_“não”_pele_escura:','it':':donna_con_gesto_di_rifiuto_carnagione_scura:','fa':':زن_نمی_پذیرد_پوست_آبنوسی:','id':':wanita_mengisyaratkan_tidak_warna_kulit_gelap:','zh':':禁止手势女_较深肤色:','ru':':женщина_против_очень_темныи_тон_кожи:','tr':':hayır_hareketi_yapan_kadın_koyu_cilt_tonu:','ar':':غير_موافقة_بشرة_بلون_غامق:'},'\U0001F645\U0001F3FF\U0000200D\U00002640':{'en':':woman_gesturing_NO_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_überkreuzten_armen_dunkle_hautfarbe:','es':':mujer_haciendo_el_gesto_de_no_tono_de_piel_oscuro:','fr':':femme_faisant_un_geste_d’interdiction_peau_foncée:','ja':':ダメのポーズをする女_濃い肌色:','ko':':안_된다는_제스처를_하는_여자_검은색_피부:','pt':':mulher_fazendo_gesto_de_“não”_pele_escura:','it':':donna_con_gesto_di_rifiuto_carnagione_scura:','fa':':زن_نمی_پذیرد_پوست_آبنوسی:','id':':wanita_mengisyaratkan_tidak_warna_kulit_gelap:','zh':':禁止手势女_较深肤色:','ru':':женщина_против_очень_темный_тон_кожи:','tr':':hayır_hareketi_yapan_kadın_koyu_cilt_tonu:','ar':':غير_موافقة_بشرة_بلون_غامق:'},'\U0001F645\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_gesturing_NO_light_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_überkreuzten_armen_helle_hautfarbe:','es':':mujer_haciendo_el_gesto_de_no_tono_de_piel_claro:','fr':':femme_faisant_un_geste_d’interdiction_peau_claire:','ja':':ダメのポーズをする女_薄い肌色:','ko':':안_된다는_제스처를_하는_여자_하얀_피부:','pt':':mulher_fazendo_gesto_de_“não”_pele_clara:','it':':donna_con_gesto_di_rifiuto_carnagione_chiara:','fa':':زن_نمی_پذیرد_پوست_سفید:','id':':wanita_mengisyaratkan_tidak_warna_kulit_cerah:','zh':':禁止手势女_较浅肤色:','ru':':женщина_против_очень_светлыи_тон_кожи:','tr':':hayır_hareketi_yapan_kadın_açık_cilt_tonu:','ar':':غير_موافقة_بشرة_بلون_فاتح:'},'\U0001F645\U0001F3FB\U0000200D\U00002640':{'en':':woman_gesturing_NO_light_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_überkreuzten_armen_helle_hautfarbe:','es':':mujer_haciendo_el_gesto_de_no_tono_de_piel_claro:','fr':':femme_faisant_un_geste_d’interdiction_peau_claire:','ja':':ダメのポーズをする女_薄い肌色:','ko':':안_된다는_제스처를_하는_여자_하얀_피부:','pt':':mulher_fazendo_gesto_de_“não”_pele_clara:','it':':donna_con_gesto_di_rifiuto_carnagione_chiara:','fa':':زن_نمی_پذیرد_پوست_سفید:','id':':wanita_mengisyaratkan_tidak_warna_kulit_cerah:','zh':':禁止手势女_较浅肤色:','ru':':женщина_против_очень_светлый_тон_кожи:','tr':':hayır_hareketi_yapan_kadın_açık_cilt_tonu:','ar':':غير_موافقة_بشرة_بلون_فاتح:'},'\U0001F645\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_gesturing_NO_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_überkreuzten_armen_mitteldunkle_hautfarbe:','es':':mujer_haciendo_el_gesto_de_no_tono_de_piel_oscuro_medio:','fr':':femme_faisant_un_geste_d’interdiction_peau_mate:','ja':':ダメのポーズをする女_やや濃い肌色:','ko':':안_된다는_제스처를_하는_여자_진한_갈색_피부:','pt':':mulher_fazendo_gesto_de_“não”_pele_morena_escura:','it':':donna_con_gesto_di_rifiuto_carnagione_abbastanza_scura:','fa':':زن_نمی_پذیرد_پوست_گندمی:','id':':wanita_mengisyaratkan_tidak_warna_kulit_gelap-sedang:','zh':':禁止手势女_中等深肤色:','ru':':женщина_против_темныи_тон_кожи:','tr':':hayır_hareketi_yapan_kadın_orta_koyu_cilt_tonu:','ar':':غير_موافقة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F645\U0001F3FE\U0000200D\U00002640':{'en':':woman_gesturing_NO_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_überkreuzten_armen_mitteldunkle_hautfarbe:','es':':mujer_haciendo_el_gesto_de_no_tono_de_piel_oscuro_medio:','fr':':femme_faisant_un_geste_d’interdiction_peau_mate:','ja':':ダメのポーズをする女_やや濃い肌色:','ko':':안_된다는_제스처를_하는_여자_진한_갈색_피부:','pt':':mulher_fazendo_gesto_de_“não”_pele_morena_escura:','it':':donna_con_gesto_di_rifiuto_carnagione_abbastanza_scura:','fa':':زن_نمی_پذیرد_پوست_گندمی:','id':':wanita_mengisyaratkan_tidak_warna_kulit_gelap-sedang:','zh':':禁止手势女_中等深肤色:','ru':':женщина_против_темный_тон_кожи:','tr':':hayır_hareketi_yapan_kadın_orta_koyu_cilt_tonu:','ar':':غير_موافقة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F645\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_gesturing_NO_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_überkreuzten_armen_mittelhelle_hautfarbe:','es':':mujer_haciendo_el_gesto_de_no_tono_de_piel_claro_medio:','fr':':femme_faisant_un_geste_d’interdiction_peau_moyennement_claire:','ja':':ダメのポーズをする女_やや薄い肌色:','ko':':안_된다는_제스처를_하는_여자_연한_갈색_피부:','pt':':mulher_fazendo_gesto_de_“não”_pele_morena_clara:','it':':donna_con_gesto_di_rifiuto_carnagione_abbastanza_chiara:','fa':':زن_نمی_پذیرد_پوست_روشن:','id':':wanita_mengisyaratkan_tidak_warna_kulit_cerah-sedang:','zh':':禁止手势女_中等浅肤色:','ru':':женщина_против_светлыи_тон_кожи:','tr':':hayır_hareketi_yapan_kadın_orta_açık_cilt_tonu:','ar':':غير_موافقة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F645\U0001F3FC\U0000200D\U00002640':{'en':':woman_gesturing_NO_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_überkreuzten_armen_mittelhelle_hautfarbe:','es':':mujer_haciendo_el_gesto_de_no_tono_de_piel_claro_medio:','fr':':femme_faisant_un_geste_d’interdiction_peau_moyennement_claire:','ja':':ダメのポーズをする女_やや薄い肌色:','ko':':안_된다는_제스처를_하는_여자_연한_갈색_피부:','pt':':mulher_fazendo_gesto_de_“não”_pele_morena_clara:','it':':donna_con_gesto_di_rifiuto_carnagione_abbastanza_chiara:','fa':':زن_نمی_پذیرد_پوست_روشن:','id':':wanita_mengisyaratkan_tidak_warna_kulit_cerah-sedang:','zh':':禁止手势女_中等浅肤色:','ru':':женщина_против_светлый_тон_кожи:','tr':':hayır_hareketi_yapan_kadın_orta_açık_cilt_tonu:','ar':':غير_موافقة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F645\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_gesturing_NO_medium_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_überkreuzten_armen_mittlere_hautfarbe:','es':':mujer_haciendo_el_gesto_de_no_tono_de_piel_medio:','fr':':femme_faisant_un_geste_d’interdiction_peau_légèrement_mate:','ja':':ダメのポーズをする女_中間の肌色:','ko':':안_된다는_제스처를_하는_여자_갈색_피부:','pt':':mulher_fazendo_gesto_de_“não”_pele_morena:','it':':donna_con_gesto_di_rifiuto_carnagione_olivastra:','fa':':زن_نمی_پذیرد_پوست_طلایی:','id':':wanita_mengisyaratkan_tidak_warna_kulit_sedang:','zh':':禁止手势女_中等肤色:','ru':':женщина_против_среднии_тон_кожи:','tr':':hayır_hareketi_yapan_kadın_orta_cilt_tonu:','ar':':غير_موافقة_بشرة_بلون_معتدل:'},'\U0001F645\U0001F3FD\U0000200D\U00002640':{'en':':woman_gesturing_NO_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_überkreuzten_armen_mittlere_hautfarbe:','es':':mujer_haciendo_el_gesto_de_no_tono_de_piel_medio:','fr':':femme_faisant_un_geste_d’interdiction_peau_légèrement_mate:','ja':':ダメのポーズをする女_中間の肌色:','ko':':안_된다는_제스처를_하는_여자_갈색_피부:','pt':':mulher_fazendo_gesto_de_“não”_pele_morena:','it':':donna_con_gesto_di_rifiuto_carnagione_olivastra:','fa':':زن_نمی_پذیرد_پوست_طلایی:','id':':wanita_mengisyaratkan_tidak_warna_kulit_sedang:','zh':':禁止手势女_中等肤色:','ru':':женщина_против_средний_тон_кожи:','tr':':hayır_hareketi_yapan_kadın_orta_cilt_tonu:','ar':':غير_موافقة_بشرة_بلون_معتدل:'},'\U0001F646\U0000200D\U00002640\U0000FE0F':{'en':':woman_gesturing_OK:','status':fully_qualified,'E':4,'alias':[':ok_woman:', ':woman_gesturing_ok:'],'de':':frau_mit_händen_auf_dem_kopf:','es':':mujer_haciendo_el_gesto_de_de_acuerdo:','fr':':femme_faisant_un_geste_d’acceptation:','ja':':okのポーズをする女:','ko':':오케이라는_제스처를_하는_여자:','pt':':mulher_fazendo_gesto_de_“ok”:','it':':donna_con_gesto_ok:','fa':':زن_می_پذیرد:','id':':wanita_mengisyaratkan_oke:','zh':':OK手势女:','ru':':женщина_за:','tr':':tamam_hareketi_yapan_kadın:','ar':':موافقة:'},'\U0001F646\U0000200D\U00002640':{'en':':woman_gesturing_OK:','status':minimally_qualified,'E':4,'alias':[':ok_woman:', ':woman_gesturing_ok:'],'de':':frau_mit_händen_auf_dem_kopf:','es':':mujer_haciendo_el_gesto_de_de_acuerdo:','fr':':femme_faisant_un_geste_d’acceptation:','ja':':okのポーズをする女:','ko':':오케이라는_제스처를_하는_여자:','pt':':mulher_fazendo_gesto_de_“ok”:','it':':donna_con_gesto_ok:','fa':':زن_می_پذیرد:','id':':wanita_mengisyaratkan_oke:','zh':':OK手势女:','ru':':женщина_за:','tr':':tamam_hareketi_yapan_kadın:','ar':':موافقة:'},'\U0001F646\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_gesturing_OK_dark_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_händen_auf_dem_kopf_dunkle_hautfarbe:','es':':mujer_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_oscuro:','fr':':femme_faisant_un_geste_d’acceptation_peau_foncée:','ja':':okのポーズをする女_濃い肌色:','ko':':오케이라는_제스처를_하는_여자_검은색_피부:','pt':':mulher_fazendo_gesto_de_“ok”_pele_escura:','it':':donna_con_gesto_ok_carnagione_scura:','fa':':زن_می_پذیرد_پوست_آبنوسی:','id':':wanita_mengisyaratkan_oke_warna_kulit_gelap:','zh':':OK手势女_较深肤色:','ru':':женщина_за_очень_темныи_тон_кожи:','tr':':tamam_hareketi_yapan_kadın_koyu_cilt_tonu:','ar':':موافقة_بشرة_بلون_غامق:'},'\U0001F646\U0001F3FF\U0000200D\U00002640':{'en':':woman_gesturing_OK_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_händen_auf_dem_kopf_dunkle_hautfarbe:','es':':mujer_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_oscuro:','fr':':femme_faisant_un_geste_d’acceptation_peau_foncée:','ja':':okのポーズをする女_濃い肌色:','ko':':오케이라는_제스처를_하는_여자_검은색_피부:','pt':':mulher_fazendo_gesto_de_“ok”_pele_escura:','it':':donna_con_gesto_ok_carnagione_scura:','fa':':زن_می_پذیرد_پوست_آبنوسی:','id':':wanita_mengisyaratkan_oke_warna_kulit_gelap:','zh':':OK手势女_较深肤色:','ru':':женщина_за_очень_темный_тон_кожи:','tr':':tamam_hareketi_yapan_kadın_koyu_cilt_tonu:','ar':':موافقة_بشرة_بلون_غامق:'},'\U0001F646\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_gesturing_OK_light_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_händen_auf_dem_kopf_helle_hautfarbe:','es':':mujer_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_claro:','fr':':femme_faisant_un_geste_d’acceptation_peau_claire:','ja':':okのポーズをする女_薄い肌色:','ko':':오케이라는_제스처를_하는_여자_하얀_피부:','pt':':mulher_fazendo_gesto_de_“ok”_pele_clara:','it':':donna_con_gesto_ok_carnagione_chiara:','fa':':زن_می_پذیرد_پوست_سفید:','id':':wanita_mengisyaratkan_oke_warna_kulit_cerah:','zh':':OK手势女_较浅肤色:','ru':':женщина_за_очень_светлыи_тон_кожи:','tr':':tamam_hareketi_yapan_kadın_açık_cilt_tonu:','ar':':موافقة_بشرة_بلون_فاتح:'},'\U0001F646\U0001F3FB\U0000200D\U00002640':{'en':':woman_gesturing_OK_light_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_händen_auf_dem_kopf_helle_hautfarbe:','es':':mujer_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_claro:','fr':':femme_faisant_un_geste_d’acceptation_peau_claire:','ja':':okのポーズをする女_薄い肌色:','ko':':오케이라는_제스처를_하는_여자_하얀_피부:','pt':':mulher_fazendo_gesto_de_“ok”_pele_clara:','it':':donna_con_gesto_ok_carnagione_chiara:','fa':':زن_می_پذیرد_پوست_سفید:','id':':wanita_mengisyaratkan_oke_warna_kulit_cerah:','zh':':OK手势女_较浅肤色:','ru':':женщина_за_очень_светлый_тон_кожи:','tr':':tamam_hareketi_yapan_kadın_açık_cilt_tonu:','ar':':موافقة_بشرة_بلون_فاتح:'},'\U0001F646\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_gesturing_OK_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_händen_auf_dem_kopf_mitteldunkle_hautfarbe:','es':':mujer_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_oscuro_medio:','fr':':femme_faisant_un_geste_d’acceptation_peau_mate:','ja':':okのポーズをする女_やや濃い肌色:','ko':':오케이라는_제스처를_하는_여자_진한_갈색_피부:','pt':':mulher_fazendo_gesto_de_“ok”_pele_morena_escura:','it':':donna_con_gesto_ok_carnagione_abbastanza_scura:','fa':':زن_می_پذیرد_پوست_گندمی:','id':':wanita_mengisyaratkan_oke_warna_kulit_gelap-sedang:','zh':':OK手势女_中等深肤色:','ru':':женщина_за_темныи_тон_кожи:','tr':':tamam_hareketi_yapan_kadın_orta_koyu_cilt_tonu:','ar':':موافقة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F646\U0001F3FE\U0000200D\U00002640':{'en':':woman_gesturing_OK_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_händen_auf_dem_kopf_mitteldunkle_hautfarbe:','es':':mujer_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_oscuro_medio:','fr':':femme_faisant_un_geste_d’acceptation_peau_mate:','ja':':okのポーズをする女_やや濃い肌色:','ko':':오케이라는_제스처를_하는_여자_진한_갈색_피부:','pt':':mulher_fazendo_gesto_de_“ok”_pele_morena_escura:','it':':donna_con_gesto_ok_carnagione_abbastanza_scura:','fa':':زن_می_پذیرد_پوست_گندمی:','id':':wanita_mengisyaratkan_oke_warna_kulit_gelap-sedang:','zh':':OK手势女_中等深肤色:','ru':':женщина_за_темный_тон_кожи:','tr':':tamam_hareketi_yapan_kadın_orta_koyu_cilt_tonu:','ar':':موافقة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F646\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_gesturing_OK_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_händen_auf_dem_kopf_mittelhelle_hautfarbe:','es':':mujer_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_claro_medio:','fr':':femme_faisant_un_geste_d’acceptation_peau_moyennement_claire:','ja':':okのポーズをする女_やや薄い肌色:','ko':':오케이라는_제스처를_하는_여자_연한_갈색_피부:','pt':':mulher_fazendo_gesto_de_“ok”_pele_morena_clara:','it':':donna_con_gesto_ok_carnagione_abbastanza_chiara:','fa':':زن_می_پذیرد_پوست_روشن:','id':':wanita_mengisyaratkan_oke_warna_kulit_cerah-sedang:','zh':':OK手势女_中等浅肤色:','ru':':женщина_за_светлыи_тон_кожи:','tr':':tamam_hareketi_yapan_kadın_orta_açık_cilt_tonu:','ar':':موافقة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F646\U0001F3FC\U0000200D\U00002640':{'en':':woman_gesturing_OK_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_händen_auf_dem_kopf_mittelhelle_hautfarbe:','es':':mujer_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_claro_medio:','fr':':femme_faisant_un_geste_d’acceptation_peau_moyennement_claire:','ja':':okのポーズをする女_やや薄い肌色:','ko':':오케이라는_제스처를_하는_여자_연한_갈색_피부:','pt':':mulher_fazendo_gesto_de_“ok”_pele_morena_clara:','it':':donna_con_gesto_ok_carnagione_abbastanza_chiara:','fa':':زن_می_پذیرد_پوست_روشن:','id':':wanita_mengisyaratkan_oke_warna_kulit_cerah-sedang:','zh':':OK手势女_中等浅肤色:','ru':':женщина_за_светлый_тон_кожи:','tr':':tamam_hareketi_yapan_kadın_orta_açık_cilt_tonu:','ar':':موافقة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F646\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_gesturing_OK_medium_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_händen_auf_dem_kopf_mittlere_hautfarbe:','es':':mujer_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_medio:','fr':':femme_faisant_un_geste_d’acceptation_peau_légèrement_mate:','ja':':okのポーズをする女_中間の肌色:','ko':':오케이라는_제스처를_하는_여자_갈색_피부:','pt':':mulher_fazendo_gesto_de_“ok”_pele_morena:','it':':donna_con_gesto_ok_carnagione_olivastra:','fa':':زن_می_پذیرد_پوست_طلایی:','id':':wanita_mengisyaratkan_oke_warna_kulit_sedang:','zh':':OK手势女_中等肤色:','ru':':женщина_за_среднии_тон_кожи:','tr':':tamam_hareketi_yapan_kadın_orta_cilt_tonu:','ar':':موافقة_بشرة_بلون_معتدل:'},'\U0001F646\U0001F3FD\U0000200D\U00002640':{'en':':woman_gesturing_OK_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_händen_auf_dem_kopf_mittlere_hautfarbe:','es':':mujer_haciendo_el_gesto_de_de_acuerdo_tono_de_piel_medio:','fr':':femme_faisant_un_geste_d’acceptation_peau_légèrement_mate:','ja':':okのポーズをする女_中間の肌色:','ko':':오케이라는_제스처를_하는_여자_갈색_피부:','pt':':mulher_fazendo_gesto_de_“ok”_pele_morena:','it':':donna_con_gesto_ok_carnagione_olivastra:','fa':':زن_می_پذیرد_پوست_طلایی:','id':':wanita_mengisyaratkan_oke_warna_kulit_sedang:','zh':':OK手势女_中等肤色:','ru':':женщина_за_средний_тон_кожи:','tr':':tamam_hareketi_yapan_kadın_orta_cilt_tonu:','ar':':موافقة_بشرة_بلون_معتدل:'},'\U0001F487\U0000200D\U00002640\U0000FE0F':{'en':':woman_getting_haircut:','status':fully_qualified,'E':4,'alias':[':haircut_woman:'],'de':':frau_beim_haareschneiden:','es':':mujer_cortándose_el_pelo:','fr':':femme_qui_se_fait_couper_les_cheveux:','ja':':散髪される女:','ko':':여자_헤어컷:','pt':':mulher_cortando_o_cabelo:','it':':taglio_di_capelli_per_donna:','fa':':زن_مو_کوتاه_می_کند:','id':':wanita_potong_rambut:','zh':':女生理发:','ru':':женщина_у_парикмахера:','tr':':saç_kestiren_kadın:','ar':':تقص_شعرها:'},'\U0001F487\U0000200D\U00002640':{'en':':woman_getting_haircut:','status':minimally_qualified,'E':4,'alias':[':haircut_woman:'],'de':':frau_beim_haareschneiden:','es':':mujer_cortándose_el_pelo:','fr':':femme_qui_se_fait_couper_les_cheveux:','ja':':散髪される女:','ko':':여자_헤어컷:','pt':':mulher_cortando_o_cabelo:','it':':taglio_di_capelli_per_donna:','fa':':زن_مو_کوتاه_می_کند:','id':':wanita_potong_rambut:','zh':':女生理发:','ru':':женщина_у_парикмахера:','tr':':saç_kestiren_kadın:','ar':':تقص_شعرها:'},'\U0001F487\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_getting_haircut_dark_skin_tone:','status':fully_qualified,'E':4,'de':':frau_beim_haareschneiden_dunkle_hautfarbe:','es':':mujer_cortándose_el_pelo_tono_de_piel_oscuro:','fr':':femme_qui_se_fait_couper_les_cheveux_peau_foncée:','ja':':散髪される女_濃い肌色:','ko':':여자_헤어컷_검은색_피부:','pt':':mulher_cortando_o_cabelo_pele_escura:','it':':taglio_di_capelli_per_donna_carnagione_scura:','fa':':زن_مو_کوتاه_می_کند_پوست_آبنوسی:','id':':wanita_potong_rambut_warna_kulit_gelap:','zh':':女生理发_较深肤色:','ru':':женщина_у_парикмахера_очень_темныи_тон_кожи:','tr':':saç_kestiren_kadın_koyu_cilt_tonu:','ar':':تقص_شعرها_بشرة_بلون_غامق:'},'\U0001F487\U0001F3FF\U0000200D\U00002640':{'en':':woman_getting_haircut_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_beim_haareschneiden_dunkle_hautfarbe:','es':':mujer_cortándose_el_pelo_tono_de_piel_oscuro:','fr':':femme_qui_se_fait_couper_les_cheveux_peau_foncée:','ja':':散髪される女_濃い肌色:','ko':':여자_헤어컷_검은색_피부:','pt':':mulher_cortando_o_cabelo_pele_escura:','it':':taglio_di_capelli_per_donna_carnagione_scura:','fa':':زن_مو_کوتاه_می_کند_پوست_آبنوسی:','id':':wanita_potong_rambut_warna_kulit_gelap:','zh':':女生理发_较深肤色:','ru':':женщина_у_парикмахера_очень_темный_тон_кожи:','tr':':saç_kestiren_kadın_koyu_cilt_tonu:','ar':':تقص_شعرها_بشرة_بلون_غامق:'},'\U0001F487\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_getting_haircut_light_skin_tone:','status':fully_qualified,'E':4,'de':':frau_beim_haareschneiden_helle_hautfarbe:','es':':mujer_cortándose_el_pelo_tono_de_piel_claro:','fr':':femme_qui_se_fait_couper_les_cheveux_peau_claire:','ja':':散髪される女_薄い肌色:','ko':':여자_헤어컷_하얀_피부:','pt':':mulher_cortando_o_cabelo_pele_clara:','it':':taglio_di_capelli_per_donna_carnagione_chiara:','fa':':زن_مو_کوتاه_می_کند_پوست_سفید:','id':':wanita_potong_rambut_warna_kulit_cerah:','zh':':女生理发_较浅肤色:','ru':':женщина_у_парикмахера_очень_светлыи_тон_кожи:','tr':':saç_kestiren_kadın_açık_cilt_tonu:','ar':':تقص_شعرها_بشرة_بلون_فاتح:'},'\U0001F487\U0001F3FB\U0000200D\U00002640':{'en':':woman_getting_haircut_light_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_beim_haareschneiden_helle_hautfarbe:','es':':mujer_cortándose_el_pelo_tono_de_piel_claro:','fr':':femme_qui_se_fait_couper_les_cheveux_peau_claire:','ja':':散髪される女_薄い肌色:','ko':':여자_헤어컷_하얀_피부:','pt':':mulher_cortando_o_cabelo_pele_clara:','it':':taglio_di_capelli_per_donna_carnagione_chiara:','fa':':زن_مو_کوتاه_می_کند_پوست_سفید:','id':':wanita_potong_rambut_warna_kulit_cerah:','zh':':女生理发_较浅肤色:','ru':':женщина_у_парикмахера_очень_светлый_тон_кожи:','tr':':saç_kestiren_kadın_açık_cilt_tonu:','ar':':تقص_شعرها_بشرة_بلون_فاتح:'},'\U0001F487\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_getting_haircut_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':frau_beim_haareschneiden_mitteldunkle_hautfarbe:','es':':mujer_cortándose_el_pelo_tono_de_piel_oscuro_medio:','fr':':femme_qui_se_fait_couper_les_cheveux_peau_mate:','ja':':散髪される女_やや濃い肌色:','ko':':여자_헤어컷_진한_갈색_피부:','pt':':mulher_cortando_o_cabelo_pele_morena_escura:','it':':taglio_di_capelli_per_donna_carnagione_abbastanza_scura:','fa':':زن_مو_کوتاه_می_کند_پوست_گندمی:','id':':wanita_potong_rambut_warna_kulit_gelap-sedang:','zh':':女生理发_中等深肤色:','ru':':женщина_у_парикмахера_темныи_тон_кожи:','tr':':saç_kestiren_kadın_orta_koyu_cilt_tonu:','ar':':تقص_شعرها_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F487\U0001F3FE\U0000200D\U00002640':{'en':':woman_getting_haircut_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_beim_haareschneiden_mitteldunkle_hautfarbe:','es':':mujer_cortándose_el_pelo_tono_de_piel_oscuro_medio:','fr':':femme_qui_se_fait_couper_les_cheveux_peau_mate:','ja':':散髪される女_やや濃い肌色:','ko':':여자_헤어컷_진한_갈색_피부:','pt':':mulher_cortando_o_cabelo_pele_morena_escura:','it':':taglio_di_capelli_per_donna_carnagione_abbastanza_scura:','fa':':زن_مو_کوتاه_می_کند_پوست_گندمی:','id':':wanita_potong_rambut_warna_kulit_gelap-sedang:','zh':':女生理发_中等深肤色:','ru':':женщина_у_парикмахера_темный_тон_кожи:','tr':':saç_kestiren_kadın_orta_koyu_cilt_tonu:','ar':':تقص_شعرها_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F487\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_getting_haircut_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':frau_beim_haareschneiden_mittelhelle_hautfarbe:','es':':mujer_cortándose_el_pelo_tono_de_piel_claro_medio:','fr':':femme_qui_se_fait_couper_les_cheveux_peau_moyennement_claire:','ja':':散髪される女_やや薄い肌色:','ko':':여자_헤어컷_연한_갈색_피부:','pt':':mulher_cortando_o_cabelo_pele_morena_clara:','it':':taglio_di_capelli_per_donna_carnagione_abbastanza_chiara:','fa':':زن_مو_کوتاه_می_کند_پوست_روشن:','id':':wanita_potong_rambut_warna_kulit_cerah-sedang:','zh':':女生理发_中等浅肤色:','ru':':женщина_у_парикмахера_светлыи_тон_кожи:','tr':':saç_kestiren_kadın_orta_açık_cilt_tonu:','ar':':تقص_شعرها_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F487\U0001F3FC\U0000200D\U00002640':{'en':':woman_getting_haircut_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_beim_haareschneiden_mittelhelle_hautfarbe:','es':':mujer_cortándose_el_pelo_tono_de_piel_claro_medio:','fr':':femme_qui_se_fait_couper_les_cheveux_peau_moyennement_claire:','ja':':散髪される女_やや薄い肌色:','ko':':여자_헤어컷_연한_갈색_피부:','pt':':mulher_cortando_o_cabelo_pele_morena_clara:','it':':taglio_di_capelli_per_donna_carnagione_abbastanza_chiara:','fa':':زن_مو_کوتاه_می_کند_پوست_روشن:','id':':wanita_potong_rambut_warna_kulit_cerah-sedang:','zh':':女生理发_中等浅肤色:','ru':':женщина_у_парикмахера_светлый_тон_кожи:','tr':':saç_kestiren_kadın_orta_açık_cilt_tonu:','ar':':تقص_شعرها_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F487\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_getting_haircut_medium_skin_tone:','status':fully_qualified,'E':4,'de':':frau_beim_haareschneiden_mittlere_hautfarbe:','es':':mujer_cortándose_el_pelo_tono_de_piel_medio:','fr':':femme_qui_se_fait_couper_les_cheveux_peau_légèrement_mate:','ja':':散髪される女_中間の肌色:','ko':':여자_헤어컷_갈색_피부:','pt':':mulher_cortando_o_cabelo_pele_morena:','it':':taglio_di_capelli_per_donna_carnagione_olivastra:','fa':':زن_مو_کوتاه_می_کند_پوست_طلایی:','id':':wanita_potong_rambut_warna_kulit_sedang:','zh':':女生理发_中等肤色:','ru':':женщина_у_парикмахера_среднии_тон_кожи:','tr':':saç_kestiren_kadın_orta_cilt_tonu:','ar':':تقص_شعرها_بشرة_بلون_معتدل:'},'\U0001F487\U0001F3FD\U0000200D\U00002640':{'en':':woman_getting_haircut_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_beim_haareschneiden_mittlere_hautfarbe:','es':':mujer_cortándose_el_pelo_tono_de_piel_medio:','fr':':femme_qui_se_fait_couper_les_cheveux_peau_légèrement_mate:','ja':':散髪される女_中間の肌色:','ko':':여자_헤어컷_갈색_피부:','pt':':mulher_cortando_o_cabelo_pele_morena:','it':':taglio_di_capelli_per_donna_carnagione_olivastra:','fa':':زن_مو_کوتاه_می_کند_پوست_طلایی:','id':':wanita_potong_rambut_warna_kulit_sedang:','zh':':女生理发_中等肤色:','ru':':женщина_у_парикмахера_средний_тон_кожи:','tr':':saç_kestiren_kadın_orta_cilt_tonu:','ar':':تقص_شعرها_بشرة_بلون_معتدل:'},'\U0001F486\U0000200D\U00002640\U0000FE0F':{'en':':woman_getting_massage:','status':fully_qualified,'E':4,'alias':[':massage_woman:'],'de':':frau,_die_eine_kopfmassage_bekommt:','es':':mujer_recibiendo_masaje:','fr':':femme_qui_se_fait_masser:','ja':':フェイスマッサージ中の女:','ko':':마사지_받는_여자:','pt':':mulher_recebendo_massagem_facial:','it':':donna_che_riceve_un_massaggio:','fa':':ماساژ_صورت_زن:','id':':wanita_dipijat:','zh':':女生按摩:','ru':':женщине_массируют_лицо:','tr':':yüz_masajı_yaptıran_kadın:','ar':':تدليك_لسيدة:'},'\U0001F486\U0000200D\U00002640':{'en':':woman_getting_massage:','status':minimally_qualified,'E':4,'alias':[':massage_woman:'],'de':':frau_die_eine_kopfmassage_bekommt:','es':':mujer_recibiendo_masaje:','fr':':femme_qui_se_fait_masser:','ja':':フェイスマッサージ中の女:','ko':':마사지_받는_여자:','pt':':mulher_recebendo_massagem_facial:','it':':donna_che_riceve_un_massaggio:','fa':':ماساژ_صورت_زن:','id':':wanita_dipijat:','zh':':女生按摩:','ru':':женщине_массируют_лицо:','tr':':yüz_masajı_yaptıran_kadın:','ar':':تدليك_لسيدة:'},'\U0001F486\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_getting_massage_dark_skin_tone:','status':fully_qualified,'E':4,'de':':frau,_die_eine_kopfmassage_bekommt_dunkle_hautfarbe:','es':':mujer_recibiendo_masaje_tono_de_piel_oscuro:','fr':':femme_qui_se_fait_masser_peau_foncée:','ja':':フェイスマッサージ中の女_濃い肌色:','ko':':마사지_받는_여자_검은색_피부:','pt':':mulher_recebendo_massagem_facial_pele_escura:','it':':donna_che_riceve_un_massaggio_carnagione_scura:','fa':':ماساژ_صورت_زن_پوست_آبنوسی:','id':':wanita_dipijat_warna_kulit_gelap:','zh':':女生按摩_较深肤色:','ru':':женщине_массируют_лицо_очень_темныи_тон_кожи:','tr':':yüz_masajı_yaptıran_kadın_koyu_cilt_tonu:','ar':':تدليك_لسيدة_بشرة_بلون_غامق:'},'\U0001F486\U0001F3FF\U0000200D\U00002640':{'en':':woman_getting_massage_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_die_eine_kopfmassage_bekommt_dunkle_hautfarbe:','es':':mujer_recibiendo_masaje_tono_de_piel_oscuro:','fr':':femme_qui_se_fait_masser_peau_foncée:','ja':':フェイスマッサージ中の女_濃い肌色:','ko':':마사지_받는_여자_검은색_피부:','pt':':mulher_recebendo_massagem_facial_pele_escura:','it':':donna_che_riceve_un_massaggio_carnagione_scura:','fa':':ماساژ_صورت_زن_پوست_آبنوسی:','id':':wanita_dipijat_warna_kulit_gelap:','zh':':女生按摩_较深肤色:','ru':':женщине_массируют_лицо_очень_темный_тон_кожи:','tr':':yüz_masajı_yaptıran_kadın_koyu_cilt_tonu:','ar':':تدليك_لسيدة_بشرة_بلون_غامق:'},'\U0001F486\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_getting_massage_light_skin_tone:','status':fully_qualified,'E':4,'de':':frau,_die_eine_kopfmassage_bekommt_helle_hautfarbe:','es':':mujer_recibiendo_masaje_tono_de_piel_claro:','fr':':femme_qui_se_fait_masser_peau_claire:','ja':':フェイスマッサージ中の女_薄い肌色:','ko':':마사지_받는_여자_하얀_피부:','pt':':mulher_recebendo_massagem_facial_pele_clara:','it':':donna_che_riceve_un_massaggio_carnagione_chiara:','fa':':ماساژ_صورت_زن_پوست_سفید:','id':':wanita_dipijat_warna_kulit_cerah:','zh':':女生按摩_较浅肤色:','ru':':женщине_массируют_лицо_очень_светлыи_тон_кожи:','tr':':yüz_masajı_yaptıran_kadın_açık_cilt_tonu:','ar':':تدليك_لسيدة_بشرة_بلون_فاتح:'},'\U0001F486\U0001F3FB\U0000200D\U00002640':{'en':':woman_getting_massage_light_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_die_eine_kopfmassage_bekommt_helle_hautfarbe:','es':':mujer_recibiendo_masaje_tono_de_piel_claro:','fr':':femme_qui_se_fait_masser_peau_claire:','ja':':フェイスマッサージ中の女_薄い肌色:','ko':':마사지_받는_여자_하얀_피부:','pt':':mulher_recebendo_massagem_facial_pele_clara:','it':':donna_che_riceve_un_massaggio_carnagione_chiara:','fa':':ماساژ_صورت_زن_پوست_سفید:','id':':wanita_dipijat_warna_kulit_cerah:','zh':':女生按摩_较浅肤色:','ru':':женщине_массируют_лицо_очень_светлый_тон_кожи:','tr':':yüz_masajı_yaptıran_kadın_açık_cilt_tonu:','ar':':تدليك_لسيدة_بشرة_بلون_فاتح:'},'\U0001F486\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_getting_massage_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':frau,_die_eine_kopfmassage_bekommt_mitteldunkle_hautfarbe:','es':':mujer_recibiendo_masaje_tono_de_piel_oscuro_medio:','fr':':femme_qui_se_fait_masser_peau_mate:','ja':':フェイスマッサージ中の女_やや濃い肌色:','ko':':마사지_받는_여자_진한_갈색_피부:','pt':':mulher_recebendo_massagem_facial_pele_morena_escura:','it':':donna_che_riceve_un_massaggio_carnagione_abbastanza_scura:','fa':':ماساژ_صورت_زن_پوست_گندمی:','id':':wanita_dipijat_warna_kulit_gelap-sedang:','zh':':女生按摩_中等深肤色:','ru':':женщине_массируют_лицо_темныи_тон_кожи:','tr':':yüz_masajı_yaptıran_kadın_orta_koyu_cilt_tonu:','ar':':تدليك_لسيدة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F486\U0001F3FE\U0000200D\U00002640':{'en':':woman_getting_massage_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_die_eine_kopfmassage_bekommt_mitteldunkle_hautfarbe:','es':':mujer_recibiendo_masaje_tono_de_piel_oscuro_medio:','fr':':femme_qui_se_fait_masser_peau_mate:','ja':':フェイスマッサージ中の女_やや濃い肌色:','ko':':마사지_받는_여자_진한_갈색_피부:','pt':':mulher_recebendo_massagem_facial_pele_morena_escura:','it':':donna_che_riceve_un_massaggio_carnagione_abbastanza_scura:','fa':':ماساژ_صورت_زن_پوست_گندمی:','id':':wanita_dipijat_warna_kulit_gelap-sedang:','zh':':女生按摩_中等深肤色:','ru':':женщине_массируют_лицо_темный_тон_кожи:','tr':':yüz_masajı_yaptıran_kadın_orta_koyu_cilt_tonu:','ar':':تدليك_لسيدة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F486\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_getting_massage_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':frau,_die_eine_kopfmassage_bekommt_mittelhelle_hautfarbe:','es':':mujer_recibiendo_masaje_tono_de_piel_claro_medio:','fr':':femme_qui_se_fait_masser_peau_moyennement_claire:','ja':':フェイスマッサージ中の女_やや薄い肌色:','ko':':마사지_받는_여자_연한_갈색_피부:','pt':':mulher_recebendo_massagem_facial_pele_morena_clara:','it':':donna_che_riceve_un_massaggio_carnagione_abbastanza_chiara:','fa':':ماساژ_صورت_زن_پوست_روشن:','id':':wanita_dipijat_warna_kulit_cerah-sedang:','zh':':女生按摩_中等浅肤色:','ru':':женщине_массируют_лицо_светлыи_тон_кожи:','tr':':yüz_masajı_yaptıran_kadın_orta_açık_cilt_tonu:','ar':':تدليك_لسيدة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F486\U0001F3FC\U0000200D\U00002640':{'en':':woman_getting_massage_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_die_eine_kopfmassage_bekommt_mittelhelle_hautfarbe:','es':':mujer_recibiendo_masaje_tono_de_piel_claro_medio:','fr':':femme_qui_se_fait_masser_peau_moyennement_claire:','ja':':フェイスマッサージ中の女_やや薄い肌色:','ko':':마사지_받는_여자_연한_갈색_피부:','pt':':mulher_recebendo_massagem_facial_pele_morena_clara:','it':':donna_che_riceve_un_massaggio_carnagione_abbastanza_chiara:','fa':':ماساژ_صورت_زن_پوست_روشن:','id':':wanita_dipijat_warna_kulit_cerah-sedang:','zh':':女生按摩_中等浅肤色:','ru':':женщине_массируют_лицо_светлый_тон_кожи:','tr':':yüz_masajı_yaptıran_kadın_orta_açık_cilt_tonu:','ar':':تدليك_لسيدة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F486\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_getting_massage_medium_skin_tone:','status':fully_qualified,'E':4,'de':':frau,_die_eine_kopfmassage_bekommt_mittlere_hautfarbe:','es':':mujer_recibiendo_masaje_tono_de_piel_medio:','fr':':femme_qui_se_fait_masser_peau_légèrement_mate:','ja':':フェイスマッサージ中の女_中間の肌色:','ko':':마사지_받는_여자_갈색_피부:','pt':':mulher_recebendo_massagem_facial_pele_morena:','it':':donna_che_riceve_un_massaggio_carnagione_olivastra:','fa':':ماساژ_صورت_زن_پوست_طلایی:','id':':wanita_dipijat_warna_kulit_sedang:','zh':':女生按摩_中等肤色:','ru':':женщине_массируют_лицо_среднии_тон_кожи:','tr':':yüz_masajı_yaptıran_kadın_orta_cilt_tonu:','ar':':تدليك_لسيدة_بشرة_بلون_معتدل:'},'\U0001F486\U0001F3FD\U0000200D\U00002640':{'en':':woman_getting_massage_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_die_eine_kopfmassage_bekommt_mittlere_hautfarbe:','es':':mujer_recibiendo_masaje_tono_de_piel_medio:','fr':':femme_qui_se_fait_masser_peau_légèrement_mate:','ja':':フェイスマッサージ中の女_中間の肌色:','ko':':마사지_받는_여자_갈색_피부:','pt':':mulher_recebendo_massagem_facial_pele_morena:','it':':donna_che_riceve_un_massaggio_carnagione_olivastra:','fa':':ماساژ_صورت_زن_پوست_طلایی:','id':':wanita_dipijat_warna_kulit_sedang:','zh':':女生按摩_中等肤色:','ru':':женщине_массируют_лицо_средний_тон_кожи:','tr':':yüz_masajı_yaptıran_kadın_orta_cilt_tonu:','ar':':تدليك_لسيدة_بشرة_بلون_معتدل:'},'\U0001F3CC\U0000FE0F\U0000200D\U00002640\U0000FE0F':{'en':':woman_golfing:','status':fully_qualified,'E':4,'alias':[':golfing_woman:'],'de':':golferin:','es':':mujer_jugando_al_golf:','fr':':golfeuse:','ja':':ゴルフをする女:','ko':':골프치는_여자:','pt':':mulher_golfista:','it':':golfista_donna:','fa':':گلف_باز_زن:','id':':wanita_bermain_golf:','zh':':女生打高尔夫:','ru':':женщина_играет_в_гольф:','tr':':golf_oynayan_kadın:','ar':':لاعبة_غولف:'},'\U0001F3CC\U0000200D\U00002640\U0000FE0F':{'en':':woman_golfing:','status':unqualified,'E':4,'alias':[':golfing_woman:'],'de':':golferin:','es':':mujer_jugando_al_golf:','fr':':golfeuse:','ja':':ゴルフをする女:','ko':':골프치는_여자:','pt':':mulher_golfista:','it':':golfista_donna:','fa':':گلف_باز_زن:','id':':wanita_bermain_golf:','zh':':女生打高尔夫:','ru':':женщина_играет_в_гольф:','tr':':golf_oynayan_kadın:','ar':':لاعبة_غولف:'},'\U0001F3CC\U0000FE0F\U0000200D\U00002640':{'en':':woman_golfing:','status':minimally_qualified,'E':4,'alias':[':golfing_woman:'],'de':':golferin:','es':':mujer_jugando_al_golf:','fr':':golfeuse:','ja':':ゴルフをする女:','ko':':골프치는_여자:','pt':':mulher_golfista:','it':':golfista_donna:','fa':':گلف_باز_زن:','id':':wanita_bermain_golf:','zh':':女生打高尔夫:','ru':':женщина_играет_в_гольф:','tr':':golf_oynayan_kadın:','ar':':لاعبة_غولف:'},'\U0001F3CC\U0000200D\U00002640':{'en':':woman_golfing:','status':unqualified,'E':4,'alias':[':golfing_woman:'],'de':':golferin:','es':':mujer_jugando_al_golf:','fr':':golfeuse:','ja':':ゴルフをする女:','ko':':골프치는_여자:','pt':':mulher_golfista:','it':':golfista_donna:','fa':':گلف_باز_زن:','id':':wanita_bermain_golf:','zh':':女生打高尔夫:','ru':':женщина_играет_в_гольф:','tr':':golf_oynayan_kadın:','ar':':لاعبة_غولف:'},'\U0001F3CC\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_golfing_dark_skin_tone:','status':fully_qualified,'E':4,'de':':golferin_dunkle_hautfarbe:','es':':mujer_jugando_al_golf_tono_de_piel_oscuro:','fr':':golfeuse_peau_foncée:','ja':':ゴルフをする女_濃い肌色:','ko':':골프치는_여자_검은색_피부:','pt':':mulher_golfista_pele_escura:','it':':golfista_donna_carnagione_scura:','fa':':گلف_باز_زن_پوست_آبنوسی:','id':':wanita_bermain_golf_warna_kulit_gelap:','zh':':女生打高尔夫_较深肤色:','ru':':женщина_играет_в_гольф_очень_темныи_тон_кожи:','tr':':golf_oynayan_kadın_koyu_cilt_tonu:','ar':':لاعبة_غولف_بشرة_بلون_غامق:'},'\U0001F3CC\U0001F3FF\U0000200D\U00002640':{'en':':woman_golfing_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':golferin_dunkle_hautfarbe:','es':':mujer_jugando_al_golf_tono_de_piel_oscuro:','fr':':golfeuse_peau_foncée:','ja':':ゴルフをする女_濃い肌色:','ko':':골프치는_여자_검은색_피부:','pt':':mulher_golfista_pele_escura:','it':':golfista_donna_carnagione_scura:','fa':':گلف_باز_زن_پوست_آبنوسی:','id':':wanita_bermain_golf_warna_kulit_gelap:','zh':':女生打高尔夫_较深肤色:','ru':':женщина_играет_в_гольф_очень_темный_тон_кожи:','tr':':golf_oynayan_kadın_koyu_cilt_tonu:','ar':':لاعبة_غولف_بشرة_بلون_غامق:'},'\U0001F3CC\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_golfing_light_skin_tone:','status':fully_qualified,'E':4,'de':':golferin_helle_hautfarbe:','es':':mujer_jugando_al_golf_tono_de_piel_claro:','fr':':golfeuse_peau_claire:','ja':':ゴルフをする女_薄い肌色:','ko':':골프치는_여자_하얀_피부:','pt':':mulher_golfista_pele_clara:','it':':golfista_donna_carnagione_chiara:','fa':':گلف_باز_زن_پوست_سفید:','id':':wanita_bermain_golf_warna_kulit_cerah:','zh':':女生打高尔夫_较浅肤色:','ru':':женщина_играет_в_гольф_очень_светлыи_тон_кожи:','tr':':golf_oynayan_kadın_açık_cilt_tonu:','ar':':لاعبة_غولف_بشرة_بلون_فاتح:'},'\U0001F3CC\U0001F3FB\U0000200D\U00002640':{'en':':woman_golfing_light_skin_tone:','status':minimally_qualified,'E':4,'de':':golferin_helle_hautfarbe:','es':':mujer_jugando_al_golf_tono_de_piel_claro:','fr':':golfeuse_peau_claire:','ja':':ゴルフをする女_薄い肌色:','ko':':골프치는_여자_하얀_피부:','pt':':mulher_golfista_pele_clara:','it':':golfista_donna_carnagione_chiara:','fa':':گلف_باز_زن_پوست_سفید:','id':':wanita_bermain_golf_warna_kulit_cerah:','zh':':女生打高尔夫_较浅肤色:','ru':':женщина_играет_в_гольф_очень_светлый_тон_кожи:','tr':':golf_oynayan_kadın_açık_cilt_tonu:','ar':':لاعبة_غولف_بشرة_بلون_فاتح:'},'\U0001F3CC\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_golfing_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':golferin_mitteldunkle_hautfarbe:','es':':mujer_jugando_al_golf_tono_de_piel_oscuro_medio:','fr':':golfeuse_peau_mate:','ja':':ゴルフをする女_やや濃い肌色:','ko':':골프치는_여자_진한_갈색_피부:','pt':':mulher_golfista_pele_morena_escura:','it':':golfista_donna_carnagione_abbastanza_scura:','fa':':گلف_باز_زن_پوست_گندمی:','id':':wanita_bermain_golf_warna_kulit_gelap-sedang:','zh':':女生打高尔夫_中等深肤色:','ru':':женщина_играет_в_гольф_темныи_тон_кожи:','tr':':golf_oynayan_kadın_orta_koyu_cilt_tonu:','ar':':لاعبة_غولف_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3CC\U0001F3FE\U0000200D\U00002640':{'en':':woman_golfing_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':golferin_mitteldunkle_hautfarbe:','es':':mujer_jugando_al_golf_tono_de_piel_oscuro_medio:','fr':':golfeuse_peau_mate:','ja':':ゴルフをする女_やや濃い肌色:','ko':':골프치는_여자_진한_갈색_피부:','pt':':mulher_golfista_pele_morena_escura:','it':':golfista_donna_carnagione_abbastanza_scura:','fa':':گلف_باز_زن_پوست_گندمی:','id':':wanita_bermain_golf_warna_kulit_gelap-sedang:','zh':':女生打高尔夫_中等深肤色:','ru':':женщина_играет_в_гольф_темный_тон_кожи:','tr':':golf_oynayan_kadın_orta_koyu_cilt_tonu:','ar':':لاعبة_غولف_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3CC\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_golfing_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':golferin_mittelhelle_hautfarbe:','es':':mujer_jugando_al_golf_tono_de_piel_claro_medio:','fr':':golfeuse_peau_moyennement_claire:','ja':':ゴルフをする女_やや薄い肌色:','ko':':골프치는_여자_연한_갈색_피부:','pt':':mulher_golfista_pele_morena_clara:','it':':golfista_donna_carnagione_abbastanza_chiara:','fa':':گلف_باز_زن_پوست_روشن:','id':':wanita_bermain_golf_warna_kulit_cerah-sedang:','zh':':女生打高尔夫_中等浅肤色:','ru':':женщина_играет_в_гольф_светлыи_тон_кожи:','tr':':golf_oynayan_kadın_orta_açık_cilt_tonu:','ar':':لاعبة_غولف_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3CC\U0001F3FC\U0000200D\U00002640':{'en':':woman_golfing_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':golferin_mittelhelle_hautfarbe:','es':':mujer_jugando_al_golf_tono_de_piel_claro_medio:','fr':':golfeuse_peau_moyennement_claire:','ja':':ゴルフをする女_やや薄い肌色:','ko':':골프치는_여자_연한_갈색_피부:','pt':':mulher_golfista_pele_morena_clara:','it':':golfista_donna_carnagione_abbastanza_chiara:','fa':':گلف_باز_زن_پوست_روشن:','id':':wanita_bermain_golf_warna_kulit_cerah-sedang:','zh':':女生打高尔夫_中等浅肤色:','ru':':женщина_играет_в_гольф_светлый_тон_кожи:','tr':':golf_oynayan_kadın_orta_açık_cilt_tonu:','ar':':لاعبة_غولف_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3CC\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_golfing_medium_skin_tone:','status':fully_qualified,'E':4,'de':':golferin_mittlere_hautfarbe:','es':':mujer_jugando_al_golf_tono_de_piel_medio:','fr':':golfeuse_peau_légèrement_mate:','ja':':ゴルフをする女_中間の肌色:','ko':':골프치는_여자_갈색_피부:','pt':':mulher_golfista_pele_morena:','it':':golfista_donna_carnagione_olivastra:','fa':':گلف_باز_زن_پوست_طلایی:','id':':wanita_bermain_golf_warna_kulit_sedang:','zh':':女生打高尔夫_中等肤色:','ru':':женщина_играет_в_гольф_среднии_тон_кожи:','tr':':golf_oynayan_kadın_orta_cilt_tonu:','ar':':لاعبة_غولف_بشرة_بلون_معتدل:'},'\U0001F3CC\U0001F3FD\U0000200D\U00002640':{'en':':woman_golfing_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':golferin_mittlere_hautfarbe:','es':':mujer_jugando_al_golf_tono_de_piel_medio:','fr':':golfeuse_peau_légèrement_mate:','ja':':ゴルフをする女_中間の肌色:','ko':':골프치는_여자_갈색_피부:','pt':':mulher_golfista_pele_morena:','it':':golfista_donna_carnagione_olivastra:','fa':':گلف_باز_زن_پوست_طلایی:','id':':wanita_bermain_golf_warna_kulit_sedang:','zh':':女生打高尔夫_中等肤色:','ru':':женщина_играет_в_гольф_средний_тон_кожи:','tr':':golf_oynayan_kadın_orta_cilt_tonu:','ar':':لاعبة_غولف_بشرة_بلون_معتدل:'},'\U0001F482\U0000200D\U00002640\U0000FE0F':{'en':':woman_guard:','status':fully_qualified,'E':4,'alias':[':guardswoman:'],'de':':wachfrau:','es':':guardia_mujer:','fr':':garde_femme:','ja':':女性の衛兵:','ko':':여자_근위병:','pt':':guarda_mulher:','it':':guardia_donna:','fa':':گارد_زن:','id':':penjaga_wanita:','zh':':女卫兵:','ru':':женщина-гвардеец:','tr':':kadın_muhafız:','ar':':حارسة:'},'\U0001F482\U0000200D\U00002640':{'en':':woman_guard:','status':minimally_qualified,'E':4,'alias':[':guardswoman:'],'de':':wachsoldatin:','es':':guardia_mujer:','fr':':garde_femme:','ja':':女性の衛兵:','ko':':여자_근위병:','pt':':guarda_mulher:','it':':guardia_donna:','fa':':گارد_زن:','id':':penjaga_wanita:','zh':':女卫兵:','ru':':женщина_гвардеец:','tr':':kadın_muhafız:','ar':':حارسة:'},'\U0001F482\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_guard_dark_skin_tone:','status':fully_qualified,'E':4,'de':':wachfrau_dunkle_hautfarbe:','es':':guardia_mujer_tono_de_piel_oscuro:','fr':':garde_femme_peau_foncée:','ja':':女性の衛兵_濃い肌色:','ko':':여자_근위병_검은색_피부:','pt':':guarda_mulher_pele_escura:','it':':guardia_donna_carnagione_scura:','fa':':گارد_زن_پوست_آبنوسی:','id':':penjaga_wanita_warna_kulit_gelap:','zh':':女卫兵_较深肤色:','ru':':женщина-гвардеец_очень_темныи_тон_кожи:','tr':':kadın_muhafız_koyu_cilt_tonu:','ar':':حارسة_بشرة_بلون_غامق:'},'\U0001F482\U0001F3FF\U0000200D\U00002640':{'en':':woman_guard_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':wachsoldatin_dunkle_hautfarbe:','es':':guardia_mujer_tono_de_piel_oscuro:','fr':':garde_femme_peau_foncée:','ja':':女性の衛兵_濃い肌色:','ko':':여자_근위병_검은색_피부:','pt':':guarda_mulher_pele_escura:','it':':guardia_donna_carnagione_scura:','fa':':گارد_زن_پوست_آبنوسی:','id':':penjaga_wanita_warna_kulit_gelap:','zh':':女卫兵_较深肤色:','ru':':женщина_гвардеец_очень_темный_тон_кожи:','tr':':kadın_muhafız_koyu_cilt_tonu:','ar':':حارسة_بشرة_بلون_غامق:'},'\U0001F482\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_guard_light_skin_tone:','status':fully_qualified,'E':4,'de':':wachfrau_helle_hautfarbe:','es':':guardia_mujer_tono_de_piel_claro:','fr':':garde_femme_peau_claire:','ja':':女性の衛兵_薄い肌色:','ko':':여자_근위병_하얀_피부:','pt':':guarda_mulher_pele_clara:','it':':guardia_donna_carnagione_chiara:','fa':':گارد_زن_پوست_سفید:','id':':penjaga_wanita_warna_kulit_cerah:','zh':':女卫兵_较浅肤色:','ru':':женщина-гвардеец_очень_светлыи_тон_кожи:','tr':':kadın_muhafız_açık_cilt_tonu:','ar':':حارسة_بشرة_بلون_فاتح:'},'\U0001F482\U0001F3FB\U0000200D\U00002640':{'en':':woman_guard_light_skin_tone:','status':minimally_qualified,'E':4,'de':':wachsoldatin_helle_hautfarbe:','es':':guardia_mujer_tono_de_piel_claro:','fr':':garde_femme_peau_claire:','ja':':女性の衛兵_薄い肌色:','ko':':여자_근위병_하얀_피부:','pt':':guarda_mulher_pele_clara:','it':':guardia_donna_carnagione_chiara:','fa':':گارد_زن_پوست_سفید:','id':':penjaga_wanita_warna_kulit_cerah:','zh':':女卫兵_较浅肤色:','ru':':женщина_гвардеец_очень_светлый_тон_кожи:','tr':':kadın_muhafız_açık_cilt_tonu:','ar':':حارسة_بشرة_بلون_فاتح:'},'\U0001F482\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_guard_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':wachfrau_mitteldunkle_hautfarbe:','es':':guardia_mujer_tono_de_piel_oscuro_medio:','fr':':garde_femme_peau_mate:','ja':':女性の衛兵_やや濃い肌色:','ko':':여자_근위병_진한_갈색_피부:','pt':':guarda_mulher_pele_morena_escura:','it':':guardia_donna_carnagione_abbastanza_scura:','fa':':گارد_زن_پوست_گندمی:','id':':penjaga_wanita_warna_kulit_gelap-sedang:','zh':':女卫兵_中等深肤色:','ru':':женщина-гвардеец_темныи_тон_кожи:','tr':':kadın_muhafız_orta_koyu_cilt_tonu:','ar':':حارسة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F482\U0001F3FE\U0000200D\U00002640':{'en':':woman_guard_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':wachsoldatin_mitteldunkle_hautfarbe:','es':':guardia_mujer_tono_de_piel_oscuro_medio:','fr':':garde_femme_peau_mate:','ja':':女性の衛兵_やや濃い肌色:','ko':':여자_근위병_진한_갈색_피부:','pt':':guarda_mulher_pele_morena_escura:','it':':guardia_donna_carnagione_abbastanza_scura:','fa':':گارد_زن_پوست_گندمی:','id':':penjaga_wanita_warna_kulit_gelap-sedang:','zh':':女卫兵_中等深肤色:','ru':':женщина_гвардеец_темный_тон_кожи:','tr':':kadın_muhafız_orta_koyu_cilt_tonu:','ar':':حارسة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F482\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_guard_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':wachfrau_mittelhelle_hautfarbe:','es':':guardia_mujer_tono_de_piel_claro_medio:','fr':':garde_femme_peau_moyennement_claire:','ja':':女性の衛兵_やや薄い肌色:','ko':':여자_근위병_연한_갈색_피부:','pt':':guarda_mulher_pele_morena_clara:','it':':guardia_donna_carnagione_abbastanza_chiara:','fa':':گارد_زن_پوست_روشن:','id':':penjaga_wanita_warna_kulit_cerah-sedang:','zh':':女卫兵_中等浅肤色:','ru':':женщина-гвардеец_светлыи_тон_кожи:','tr':':kadın_muhafız_orta_açık_cilt_tonu:','ar':':حارسة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F482\U0001F3FC\U0000200D\U00002640':{'en':':woman_guard_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':wachsoldatin_mittelhelle_hautfarbe:','es':':guardia_mujer_tono_de_piel_claro_medio:','fr':':garde_femme_peau_moyennement_claire:','ja':':女性の衛兵_やや薄い肌色:','ko':':여자_근위병_연한_갈색_피부:','pt':':guarda_mulher_pele_morena_clara:','it':':guardia_donna_carnagione_abbastanza_chiara:','fa':':گارد_زن_پوست_روشن:','id':':penjaga_wanita_warna_kulit_cerah-sedang:','zh':':女卫兵_中等浅肤色:','ru':':женщина_гвардеец_светлый_тон_кожи:','tr':':kadın_muhafız_orta_açık_cilt_tonu:','ar':':حارسة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F482\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_guard_medium_skin_tone:','status':fully_qualified,'E':4,'de':':wachfrau_mittlere_hautfarbe:','es':':guardia_mujer_tono_de_piel_medio:','fr':':garde_femme_peau_légèrement_mate:','ja':':女性の衛兵_中間の肌色:','ko':':여자_근위병_갈색_피부:','pt':':guarda_mulher_pele_morena:','it':':guardia_donna_carnagione_olivastra:','fa':':گارد_زن_پوست_طلایی:','id':':penjaga_wanita_warna_kulit_sedang:','zh':':女卫兵_中等肤色:','ru':':женщина-гвардеец_среднии_тон_кожи:','tr':':kadın_muhafız_orta_cilt_tonu:','ar':':حارسة_بشرة_بلون_معتدل:'},'\U0001F482\U0001F3FD\U0000200D\U00002640':{'en':':woman_guard_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':wachsoldatin_mittlere_hautfarbe:','es':':guardia_mujer_tono_de_piel_medio:','fr':':garde_femme_peau_légèrement_mate:','ja':':女性の衛兵_中間の肌色:','ko':':여자_근위병_갈색_피부:','pt':':guarda_mulher_pele_morena:','it':':guardia_donna_carnagione_olivastra:','fa':':گارد_زن_پوست_طلایی:','id':':penjaga_wanita_warna_kulit_sedang:','zh':':女卫兵_中等肤色:','ru':':женщина_гвардеец_средний_тон_кожи:','tr':':kadın_muhafız_orta_cilt_tonu:','ar':':حارسة_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U00002695\U0000FE0F':{'en':':woman_health_worker:','status':fully_qualified,'E':4,'de':':ärztin:','es':':profesional_sanitario_mujer:','fr':':professionnelle_de_la_santé:','ja':':女性の医者:','ko':':여자_의료인:','pt':':mulher_profissional_da_saúde:','it':':operatrice_sanitaria:','fa':':مددکار_بهداشت_زن:','id':':pekerja_kesehatan_wanita:','zh':':女医生:','ru':':женщина-врач:','tr':':kadın_sağlık_çalışanı:','ar':':عاملة_بقطاع_الصحة:'},'\U0001F469\U0000200D\U00002695':{'en':':woman_health_worker:','status':minimally_qualified,'E':4,'de':':ärztin:','es':':profesional_sanitario_mujer:','fr':':professionnelle_de_la_santé:','ja':':女性の医者:','ko':':여자_의료인:','pt':':mulher_profissional_da_saúde:','it':':operatrice_sanitaria:','fa':':مددکار_بهداشت_زن:','id':':pekerja_kesehatan_wanita:','zh':':女医生:','ru':':женщина_врач:','tr':':kadın_sağlık_çalışanı:','ar':':عاملة_بقطاع_الصحة:'},'\U0001F469\U0001F3FF\U0000200D\U00002695\U0000FE0F':{'en':':woman_health_worker_dark_skin_tone:','status':fully_qualified,'E':4,'de':':ärztin_dunkle_hautfarbe:','es':':profesional_sanitario_mujer_tono_de_piel_oscuro:','fr':':professionnelle_de_la_santé_peau_foncée:','ja':':女性の医者_濃い肌色:','ko':':여자_의료인_검은색_피부:','pt':':mulher_profissional_da_saúde_pele_escura:','it':':operatrice_sanitaria_carnagione_scura:','fa':':مددکار_بهداشت_زن_پوست_آبنوسی:','id':':pekerja_kesehatan_wanita_warna_kulit_gelap:','zh':':女医生_较深肤色:','ru':':женщина-врач_очень_темныи_тон_кожи:','tr':':kadın_sağlık_çalışanı_koyu_cilt_tonu:','ar':':عاملة_بقطاع_الصحة_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FF\U0000200D\U00002695':{'en':':woman_health_worker_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':ärztin_dunkle_hautfarbe:','es':':profesional_sanitario_mujer_tono_de_piel_oscuro:','fr':':professionnelle_de_la_santé_peau_foncée:','ja':':女性の医者_濃い肌色:','ko':':여자_의료인_검은색_피부:','pt':':mulher_profissional_da_saúde_pele_escura:','it':':operatrice_sanitaria_carnagione_scura:','fa':':مددکار_بهداشت_زن_پوست_آبنوسی:','id':':pekerja_kesehatan_wanita_warna_kulit_gelap:','zh':':女医生_较深肤色:','ru':':женщина_врач_очень_темный_тон_кожи:','tr':':kadın_sağlık_çalışanı_koyu_cilt_tonu:','ar':':عاملة_بقطاع_الصحة_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U00002695\U0000FE0F':{'en':':woman_health_worker_light_skin_tone:','status':fully_qualified,'E':4,'de':':ärztin_helle_hautfarbe:','es':':profesional_sanitario_mujer_tono_de_piel_claro:','fr':':professionnelle_de_la_santé_peau_claire:','ja':':女性の医者_薄い肌色:','ko':':여자_의료인_하얀_피부:','pt':':mulher_profissional_da_saúde_pele_clara:','it':':operatrice_sanitaria_carnagione_chiara:','fa':':مددکار_بهداشت_زن_پوست_سفید:','id':':pekerja_kesehatan_wanita_warna_kulit_cerah:','zh':':女医生_较浅肤色:','ru':':женщина-врач_очень_светлыи_тон_кожи:','tr':':kadın_sağlık_çalışanı_açık_cilt_tonu:','ar':':عاملة_بقطاع_الصحة_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FB\U0000200D\U00002695':{'en':':woman_health_worker_light_skin_tone:','status':minimally_qualified,'E':4,'de':':ärztin_helle_hautfarbe:','es':':profesional_sanitario_mujer_tono_de_piel_claro:','fr':':professionnelle_de_la_santé_peau_claire:','ja':':女性の医者_薄い肌色:','ko':':여자_의료인_하얀_피부:','pt':':mulher_profissional_da_saúde_pele_clara:','it':':operatrice_sanitaria_carnagione_chiara:','fa':':مددکار_بهداشت_زن_پوست_سفید:','id':':pekerja_kesehatan_wanita_warna_kulit_cerah:','zh':':女医生_较浅肤色:','ru':':женщина_врач_очень_светлый_тон_кожи:','tr':':kadın_sağlık_çalışanı_açık_cilt_tonu:','ar':':عاملة_بقطاع_الصحة_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U00002695\U0000FE0F':{'en':':woman_health_worker_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':ärztin_mitteldunkle_hautfarbe:','es':':profesional_sanitario_mujer_tono_de_piel_oscuro_medio:','fr':':professionnelle_de_la_santé_peau_mate:','ja':':女性の医者_やや濃い肌色:','ko':':여자_의료인_진한_갈색_피부:','pt':':mulher_profissional_da_saúde_pele_morena_escura:','it':':operatrice_sanitaria_carnagione_abbastanza_scura:','fa':':مددکار_بهداشت_زن_پوست_گندمی:','id':':pekerja_kesehatan_wanita_warna_kulit_gelap-sedang:','zh':':女医生_中等深肤色:','ru':':женщина-врач_темныи_тон_кожи:','tr':':kadın_sağlık_çalışanı_orta_koyu_cilt_tonu:','ar':':عاملة_بقطاع_الصحة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FE\U0000200D\U00002695':{'en':':woman_health_worker_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':ärztin_mitteldunkle_hautfarbe:','es':':profesional_sanitario_mujer_tono_de_piel_oscuro_medio:','fr':':professionnelle_de_la_santé_peau_mate:','ja':':女性の医者_やや濃い肌色:','ko':':여자_의료인_진한_갈색_피부:','pt':':mulher_profissional_da_saúde_pele_morena_escura:','it':':operatrice_sanitaria_carnagione_abbastanza_scura:','fa':':مددکار_بهداشت_زن_پوست_گندمی:','id':':pekerja_kesehatan_wanita_warna_kulit_gelap-sedang:','zh':':女医生_中等深肤色:','ru':':женщина_врач_темный_тон_кожи:','tr':':kadın_sağlık_çalışanı_orta_koyu_cilt_tonu:','ar':':عاملة_بقطاع_الصحة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U00002695\U0000FE0F':{'en':':woman_health_worker_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':ärztin_mittelhelle_hautfarbe:','es':':profesional_sanitario_mujer_tono_de_piel_claro_medio:','fr':':professionnelle_de_la_santé_peau_moyennement_claire:','ja':':女性の医者_やや薄い肌色:','ko':':여자_의료인_연한_갈색_피부:','pt':':mulher_profissional_da_saúde_pele_morena_clara:','it':':operatrice_sanitaria_carnagione_abbastanza_chiara:','fa':':مددکار_بهداشت_زن_پوست_روشن:','id':':pekerja_kesehatan_wanita_warna_kulit_cerah-sedang:','zh':':女医生_中等浅肤色:','ru':':женщина-врач_светлыи_тон_кожи:','tr':':kadın_sağlık_çalışanı_orta_açık_cilt_tonu:','ar':':عاملة_بقطاع_الصحة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FC\U0000200D\U00002695':{'en':':woman_health_worker_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':ärztin_mittelhelle_hautfarbe:','es':':profesional_sanitario_mujer_tono_de_piel_claro_medio:','fr':':professionnelle_de_la_santé_peau_moyennement_claire:','ja':':女性の医者_やや薄い肌色:','ko':':여자_의료인_연한_갈색_피부:','pt':':mulher_profissional_da_saúde_pele_morena_clara:','it':':operatrice_sanitaria_carnagione_abbastanza_chiara:','fa':':مددکار_بهداشت_زن_پوست_روشن:','id':':pekerja_kesehatan_wanita_warna_kulit_cerah-sedang:','zh':':女医生_中等浅肤色:','ru':':женщина_врач_светлый_тон_кожи:','tr':':kadın_sağlık_çalışanı_orta_açık_cilt_tonu:','ar':':عاملة_بقطاع_الصحة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002695\U0000FE0F':{'en':':woman_health_worker_medium_skin_tone:','status':fully_qualified,'E':4,'de':':ärztin_mittlere_hautfarbe:','es':':profesional_sanitario_mujer_tono_de_piel_medio:','fr':':professionnelle_de_la_santé_peau_légèrement_mate:','ja':':女性の医者_中間の肌色:','ko':':여자_의료인_갈색_피부:','pt':':mulher_profissional_da_saúde_pele_morena:','it':':operatrice_sanitaria_carnagione_olivastra:','fa':':مددکار_بهداشت_زن_پوست_طلایی:','id':':pekerja_kesehatan_wanita_warna_kulit_sedang:','zh':':女医生_中等肤色:','ru':':женщина-врач_среднии_тон_кожи:','tr':':kadın_sağlık_çalışanı_orta_cilt_tonu:','ar':':عاملة_بقطاع_الصحة_بشرة_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002695':{'en':':woman_health_worker_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':ärztin_mittlere_hautfarbe:','es':':profesional_sanitario_mujer_tono_de_piel_medio:','fr':':professionnelle_de_la_santé_peau_légèrement_mate:','ja':':女性の医者_中間の肌色:','ko':':여자_의료인_갈색_피부:','pt':':mulher_profissional_da_saúde_pele_morena:','it':':operatrice_sanitaria_carnagione_olivastra:','fa':':مددکار_بهداشت_زن_پوست_طلایی:','id':':pekerja_kesehatan_wanita_warna_kulit_sedang:','zh':':女医生_中等肤色:','ru':':женщина_врач_средний_тон_кожи:','tr':':kadın_sağlık_çalışanı_orta_cilt_tonu:','ar':':عاملة_بقطاع_الصحة_بشرة_بلون_معتدل:'},'\U0001F9D8\U0000200D\U00002640\U0000FE0F':{'en':':woman_in_lotus_position:','status':fully_qualified,'E':5,'alias':[':lotus_position_woman:'],'de':':frau_im_lotossitz:','es':':mujer_en_posición_de_loto:','fr':':femme_dans_la_posture_du_lotus:','ja':':ヨガのポーズをする女:','ko':':가부좌한_여자:','pt':':mulher_na_posição_de_lótus:','it':':donna_in_posizione_del_loto:','fa':':زن_در_حالت_خلسه:','id':':wanita_dalam_posisi_teratai:','zh':':盘腿的女人:','ru':':женщина_в_позе_лотоса:','tr':':lotus_pozisyonundaki_kadın:','ar':':امرأة_متربعة:'},'\U0001F9D8\U0000200D\U00002640':{'en':':woman_in_lotus_position:','status':minimally_qualified,'E':5,'alias':[':lotus_position_woman:'],'de':':frau_im_lotossitz:','es':':mujer_en_posición_de_loto:','fr':':femme_dans_la_posture_du_lotus:','ja':':ヨガのポーズをする女:','ko':':가부좌한_여자:','pt':':mulher_na_posição_de_lótus:','it':':donna_in_posizione_del_loto:','fa':':زن_در_حالت_خلسه:','id':':wanita_dalam_posisi_teratai:','zh':':盘腿的女人:','ru':':женщина_в_позе_лотоса:','tr':':lotus_pozisyonundaki_kadın:','ar':':امرأة_متربعة:'},'\U0001F9D8\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_in_lotus_position_dark_skin_tone:','status':fully_qualified,'E':5,'de':':frau_im_lotossitz_dunkle_hautfarbe:','es':':mujer_en_posición_de_loto_tono_de_piel_oscuro:','fr':':femme_dans_la_posture_du_lotus_peau_foncée:','ja':':ヨガのポーズをする女_濃い肌色:','ko':':가부좌한_여자_검은색_피부:','pt':':mulher_na_posição_de_lótus_pele_escura:','it':':donna_in_posizione_del_loto_carnagione_scura:','fa':':زن_در_حالت_خلسه_پوست_آبنوسی:','id':':wanita_dalam_posisi_teratai_warna_kulit_gelap:','zh':':盘腿的女人_较深肤色:','ru':':женщина_в_позе_лотоса_очень_темныи_тон_кожи:','tr':':lotus_pozisyonundaki_kadın_koyu_cilt_tonu:','ar':':امرأة_متربعة_بشرة_بلون_غامق:'},'\U0001F9D8\U0001F3FF\U0000200D\U00002640':{'en':':woman_in_lotus_position_dark_skin_tone:','status':minimally_qualified,'E':5,'de':':frau_im_lotossitz_dunkle_hautfarbe:','es':':mujer_en_posición_de_loto_tono_de_piel_oscuro:','fr':':femme_dans_la_posture_du_lotus_peau_foncée:','ja':':ヨガのポーズをする女_濃い肌色:','ko':':가부좌한_여자_검은색_피부:','pt':':mulher_na_posição_de_lótus_pele_escura:','it':':donna_in_posizione_del_loto_carnagione_scura:','fa':':زن_در_حالت_خلسه_پوست_آبنوسی:','id':':wanita_dalam_posisi_teratai_warna_kulit_gelap:','zh':':盘腿的女人_较深肤色:','ru':':женщина_в_позе_лотоса_очень_темный_тон_кожи:','tr':':lotus_pozisyonundaki_kadın_koyu_cilt_tonu:','ar':':امرأة_متربعة_بشرة_بلون_غامق:'},'\U0001F9D8\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_in_lotus_position_light_skin_tone:','status':fully_qualified,'E':5,'de':':frau_im_lotossitz_helle_hautfarbe:','es':':mujer_en_posición_de_loto_tono_de_piel_claro:','fr':':femme_dans_la_posture_du_lotus_peau_claire:','ja':':ヨガのポーズをする女_薄い肌色:','ko':':가부좌한_여자_하얀_피부:','pt':':mulher_na_posição_de_lótus_pele_clara:','it':':donna_in_posizione_del_loto_carnagione_chiara:','fa':':زن_در_حالت_خلسه_پوست_سفید:','id':':wanita_dalam_posisi_teratai_warna_kulit_cerah:','zh':':盘腿的女人_较浅肤色:','ru':':женщина_в_позе_лотоса_очень_светлыи_тон_кожи:','tr':':lotus_pozisyonundaki_kadın_açık_cilt_tonu:','ar':':امرأة_متربعة_بشرة_بلون_فاتح:'},'\U0001F9D8\U0001F3FB\U0000200D\U00002640':{'en':':woman_in_lotus_position_light_skin_tone:','status':minimally_qualified,'E':5,'de':':frau_im_lotossitz_helle_hautfarbe:','es':':mujer_en_posición_de_loto_tono_de_piel_claro:','fr':':femme_dans_la_posture_du_lotus_peau_claire:','ja':':ヨガのポーズをする女_薄い肌色:','ko':':가부좌한_여자_하얀_피부:','pt':':mulher_na_posição_de_lótus_pele_clara:','it':':donna_in_posizione_del_loto_carnagione_chiara:','fa':':زن_در_حالت_خلسه_پوست_سفید:','id':':wanita_dalam_posisi_teratai_warna_kulit_cerah:','zh':':盘腿的女人_较浅肤色:','ru':':женщина_в_позе_лотоса_очень_светлый_тон_кожи:','tr':':lotus_pozisyonundaki_kadın_açık_cilt_tonu:','ar':':امرأة_متربعة_بشرة_بلون_فاتح:'},'\U0001F9D8\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_in_lotus_position_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':frau_im_lotossitz_mitteldunkle_hautfarbe:','es':':mujer_en_posición_de_loto_tono_de_piel_oscuro_medio:','fr':':femme_dans_la_posture_du_lotus_peau_mate:','ja':':ヨガのポーズをする女_やや濃い肌色:','ko':':가부좌한_여자_진한_갈색_피부:','pt':':mulher_na_posição_de_lótus_pele_morena_escura:','it':':donna_in_posizione_del_loto_carnagione_abbastanza_scura:','fa':':زن_در_حالت_خلسه_پوست_گندمی:','id':':wanita_dalam_posisi_teratai_warna_kulit_gelap-sedang:','zh':':盘腿的女人_中等深肤色:','ru':':женщина_в_позе_лотоса_темныи_тон_кожи:','tr':':lotus_pozisyonundaki_kadın_orta_koyu_cilt_tonu:','ar':':امرأة_متربعة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D8\U0001F3FE\U0000200D\U00002640':{'en':':woman_in_lotus_position_medium-dark_skin_tone:','status':minimally_qualified,'E':5,'de':':frau_im_lotossitz_mitteldunkle_hautfarbe:','es':':mujer_en_posición_de_loto_tono_de_piel_oscuro_medio:','fr':':femme_dans_la_posture_du_lotus_peau_mate:','ja':':ヨガのポーズをする女_やや濃い肌色:','ko':':가부좌한_여자_진한_갈색_피부:','pt':':mulher_na_posição_de_lótus_pele_morena_escura:','it':':donna_in_posizione_del_loto_carnagione_abbastanza_scura:','fa':':زن_در_حالت_خلسه_پوست_گندمی:','id':':wanita_dalam_posisi_teratai_warna_kulit_gelap-sedang:','zh':':盘腿的女人_中等深肤色:','ru':':женщина_в_позе_лотоса_темный_тон_кожи:','tr':':lotus_pozisyonundaki_kadın_orta_koyu_cilt_tonu:','ar':':امرأة_متربعة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D8\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_in_lotus_position_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':frau_im_lotossitz_mittelhelle_hautfarbe:','es':':mujer_en_posición_de_loto_tono_de_piel_claro_medio:','fr':':femme_dans_la_posture_du_lotus_peau_moyennement_claire:','ja':':ヨガのポーズをする女_やや薄い肌色:','ko':':가부좌한_여자_연한_갈색_피부:','pt':':mulher_na_posição_de_lótus_pele_morena_clara:','it':':donna_in_posizione_del_loto_carnagione_abbastanza_chiara:','fa':':زن_در_حالت_خلسه_پوست_روشن:','id':':wanita_dalam_posisi_teratai_warna_kulit_cerah-sedang:','zh':':盘腿的女人_中等浅肤色:','ru':':женщина_в_позе_лотоса_светлыи_тон_кожи:','tr':':lotus_pozisyonundaki_kadın_orta_açık_cilt_tonu:','ar':':امرأة_متربعة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D8\U0001F3FC\U0000200D\U00002640':{'en':':woman_in_lotus_position_medium-light_skin_tone:','status':minimally_qualified,'E':5,'de':':frau_im_lotossitz_mittelhelle_hautfarbe:','es':':mujer_en_posición_de_loto_tono_de_piel_claro_medio:','fr':':femme_dans_la_posture_du_lotus_peau_moyennement_claire:','ja':':ヨガのポーズをする女_やや薄い肌色:','ko':':가부좌한_여자_연한_갈색_피부:','pt':':mulher_na_posição_de_lótus_pele_morena_clara:','it':':donna_in_posizione_del_loto_carnagione_abbastanza_chiara:','fa':':زن_در_حالت_خلسه_پوست_روشن:','id':':wanita_dalam_posisi_teratai_warna_kulit_cerah-sedang:','zh':':盘腿的女人_中等浅肤色:','ru':':женщина_в_позе_лотоса_светлый_тон_кожи:','tr':':lotus_pozisyonundaki_kadın_orta_açık_cilt_tonu:','ar':':امرأة_متربعة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D8\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_in_lotus_position_medium_skin_tone:','status':fully_qualified,'E':5,'de':':frau_im_lotossitz_mittlere_hautfarbe:','es':':mujer_en_posición_de_loto_tono_de_piel_medio:','fr':':femme_dans_la_posture_du_lotus_peau_légèrement_mate:','ja':':ヨガのポーズをする女_中間の肌色:','ko':':가부좌한_여자_갈색_피부:','pt':':mulher_na_posição_de_lótus_pele_morena:','it':':donna_in_posizione_del_loto_carnagione_olivastra:','fa':':زن_در_حالت_خلسه_پوست_طلایی:','id':':wanita_dalam_posisi_teratai_warna_kulit_sedang:','zh':':盘腿的女人_中等肤色:','ru':':женщина_в_позе_лотоса_среднии_тон_кожи:','tr':':lotus_pozisyonundaki_kadın_orta_cilt_tonu:','ar':':امرأة_متربعة_بشرة_بلون_معتدل:'},'\U0001F9D8\U0001F3FD\U0000200D\U00002640':{'en':':woman_in_lotus_position_medium_skin_tone:','status':minimally_qualified,'E':5,'de':':frau_im_lotossitz_mittlere_hautfarbe:','es':':mujer_en_posición_de_loto_tono_de_piel_medio:','fr':':femme_dans_la_posture_du_lotus_peau_légèrement_mate:','ja':':ヨガのポーズをする女_中間の肌色:','ko':':가부좌한_여자_갈색_피부:','pt':':mulher_na_posição_de_lótus_pele_morena:','it':':donna_in_posizione_del_loto_carnagione_olivastra:','fa':':زن_در_حالت_خلسه_پوست_طلایی:','id':':wanita_dalam_posisi_teratai_warna_kulit_sedang:','zh':':盘腿的女人_中等肤色:','ru':':женщина_в_позе_лотоса_средний_тон_кожи:','tr':':lotus_pozisyonundaki_kadın_orta_cilt_tonu:','ar':':امرأة_متربعة_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U0001F9BD':{'en':':woman_in_manual_wheelchair:','status':fully_qualified,'E':12,'de':':frau_in_manuellem_rollstuhl:','es':':mujer_en_silla_de_ruedas_manual:','fr':':femme_en_fauteuil_roulant_manuel:','ja':':手動式車椅子の女性:','ko':':수동_휠체어를_탄_여자:','pt':':mulher_em_cadeira_de_rodas_manual:','it':':donna_su_sedia_a_rotelle_manuale:','fa':':زن_در_صندلی_چرخ_دار_دستی:','id':':wanita_dengan_kursi_roda_manual:','zh':':坐手动轮椅的女人:','ru':':женщина_в_ручном_кресле_коляске:','tr':':tekerlekli_sandalyede_kadın:','ar':':سيدة_على_كرسي_متحرك_يدوي:'},'\U0001F469\U0001F3FF\U0000200D\U0001F9BD':{'en':':woman_in_manual_wheelchair_dark_skin_tone:','status':fully_qualified,'E':12,'de':':frau_in_manuellem_rollstuhl_dunkle_hautfarbe:','es':':mujer_en_silla_de_ruedas_manual_tono_de_piel_oscuro:','fr':':femme_en_fauteuil_roulant_manuel_peau_foncée:','ja':':手動式車椅子の女性_濃い肌色:','ko':':수동_휠체어를_탄_여자_검은색_피부:','pt':':mulher_em_cadeira_de_rodas_manual_pele_escura:','it':':donna_su_sedia_a_rotelle_manuale_carnagione_scura:','fa':':زن_در_صندلی_چرخ_دار_دستی_پوست_آبنوسی:','id':':wanita_dengan_kursi_roda_manual_warna_kulit_gelap:','zh':':坐手动轮椅的女人_较深肤色:','ru':':женщина_в_ручном_кресле_коляске_очень_темный_тон_кожи:','tr':':tekerlekli_sandalyede_kadın_koyu_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_يدوي_بشرة_بلون_غامق:'},'\U0001F469\U0000200D\U0001F9BD\U0000200D\U000027A1\U0000FE0F':{'en':':woman_in_manual_wheelchair_facing_right:','status':fully_qualified,'E':15.1,'de':':frau_in_manuellem_rollstuhl_nach_rechts:','es':':mujer_en_silla_de_ruedas_manual_hacia_la_derecha:','fr':':femme_en_fauteuil_roulant_manuel_vers_la_droite:','ja':':手動式車椅子の女性_右向き:','ko':':오른쪽을_향한_수동_휠체어를_탄_여자:','pt':':mulher_em_cadeira_de_rodas_manual_de_frente_para_a_direita:','it':':donna_su_sedia_a_rotelle_manuale_a_destra:','fa':':زن_در_صندلی_چرخ_دار_دستی_به_سمت_راست:','id':':wanita_dengan_kursi_roda_manual_menghadap_kanan:','zh':':坐手动轮椅的女人面向右边:','ru':':женщина_в_ручном_кресле_коляске_направленные_вправо:','tr':':sağa_dönük_tekerlekli_sandalyede_kadın:','ar':':سيدة_على_كرسي_متحرك_يدوي_متجه_لليمين:'},'\U0001F469\U0000200D\U0001F9BD\U0000200D\U000027A1':{'en':':woman_in_manual_wheelchair_facing_right:','status':minimally_qualified,'E':15.1,'de':':frau_in_manuellem_rollstuhl_nach_rechts:','es':':mujer_en_silla_de_ruedas_manual_hacia_la_derecha:','fr':':femme_en_fauteuil_roulant_manuel_vers_la_droite:','ja':':手動式車椅子の女性_右向き:','ko':':오른쪽을_향한_수동_휠체어를_탄_여자:','pt':':mulher_em_cadeira_de_rodas_manual_de_frente_para_a_direita:','it':':donna_su_sedia_a_rotelle_manuale_a_destra:','fa':':زن_در_صندلی_چرخ_دار_دستی_به_سمت_راست:','id':':wanita_dengan_kursi_roda_manual_menghadap_kanan:','zh':':坐手动轮椅的女人面向右边:','ru':':женщина_в_ручном_кресле_коляске_направленные_вправо:','tr':':sağa_dönük_tekerlekli_sandalyede_kadın:','ar':':سيدة_على_كرسي_متحرك_يدوي_متجه_لليمين:'},'\U0001F469\U0001F3FF\U0000200D\U0001F9BD\U0000200D\U000027A1\U0000FE0F':{'en':':woman_in_manual_wheelchair_facing_right_dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':frau_in_manuellem_rollstuhl_nach_rechts_dunkle_hautfarbe:','es':':mujer_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_oscuro:','fr':':femme_en_fauteuil_roulant_manuel_vers_la_droite_peau_foncée:','ja':':手動式車椅子の女性_右向き_濃い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_여자_검은색_피부:','pt':':mulher_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_escura:','it':':donna_su_sedia_a_rotelle_manuale_a_destra_carnagione_scura:','fa':':زن_در_صندلی_چرخ_دار_دستی_به_سمت_راستپوست_آبنوسی:','id':':wanita_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_gelap:','zh':':坐手动轮椅的女人面向右边_较深肤色:','ru':':женщина_в_ручном_кресле_коляске_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_kadın_koyu_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_يدوي_متجه_لليمين_بلون_غامق:'},'\U0001F469\U0001F3FF\U0000200D\U0001F9BD\U0000200D\U000027A1':{'en':':woman_in_manual_wheelchair_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':frau_in_manuellem_rollstuhl_nach_rechts_dunkle_hautfarbe:','es':':mujer_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_oscuro:','fr':':femme_en_fauteuil_roulant_manuel_vers_la_droite_peau_foncée:','ja':':手動式車椅子の女性_右向き_濃い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_여자_검은색_피부:','pt':':mulher_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_escura:','it':':donna_su_sedia_a_rotelle_manuale_a_destra_carnagione_scura:','fa':':زن_در_صندلی_چرخ_دار_دستی_به_سمت_راستپوست_آبنوسی:','id':':wanita_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_gelap:','zh':':坐手动轮椅的女人面向右边_较深肤色:','ru':':женщина_в_ручном_кресле_коляске_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_kadın_koyu_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_يدوي_متجه_لليمين_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F9BD\U0000200D\U000027A1\U0000FE0F':{'en':':woman_in_manual_wheelchair_facing_right_light_skin_tone:','status':fully_qualified,'E':15.1,'de':':frau_in_manuellem_rollstuhl_nach_rechts_helle_hautfarbe:','es':':mujer_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_claro:','fr':':femme_en_fauteuil_roulant_manuel_vers_la_droite_peau_claire:','ja':':手動式車椅子の女性_右向き_薄い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_여자_하얀_피부:','pt':':mulher_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_clara:','it':':donna_su_sedia_a_rotelle_manuale_a_destra_carnagione_chiara:','fa':':زن_در_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_سفید:','id':':wanita_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_cerah:','zh':':坐手动轮椅的女人面向右边_较浅肤色:','ru':':женщина_в_ручном_кресле_коляске_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_kadın_açık_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_يدوي_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FB\U0000200D\U0001F9BD\U0000200D\U000027A1':{'en':':woman_in_manual_wheelchair_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':frau_in_manuellem_rollstuhl_nach_rechts_helle_hautfarbe:','es':':mujer_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_claro:','fr':':femme_en_fauteuil_roulant_manuel_vers_la_droite_peau_claire:','ja':':手動式車椅子の女性_右向き_薄い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_여자_하얀_피부:','pt':':mulher_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_clara:','it':':donna_su_sedia_a_rotelle_manuale_a_destra_carnagione_chiara:','fa':':زن_در_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_سفید:','id':':wanita_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_cerah:','zh':':坐手动轮椅的女人面向右边_较浅肤色:','ru':':женщина_в_ручном_кресле_коляске_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_kadın_açık_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_يدوي_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F9BD\U0000200D\U000027A1\U0000FE0F':{'en':':woman_in_manual_wheelchair_facing_right_medium-dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':frau_in_manuellem_rollstuhl_nach_rechts_mitteldunkle_hautfarbe:','es':':mujer_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':femme_en_fauteuil_roulant_manuel_vers_la_droite_peau_mate:','ja':':手動式車椅子の女性_右向き_やや濃い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_여자_진한_갈색_피부:','pt':':mulher_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_morena_escura:','it':':donna_su_sedia_a_rotelle_manuale_a_destra_carnagione_abbastanza_scura:','fa':':زن_در_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_گندمی:','id':':wanita_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':坐手动轮椅的女人面向右边_中等深肤色:','ru':':женщина_в_ручном_кресле_коляске_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_يدوي_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F469\U0001F3FE\U0000200D\U0001F9BD\U0000200D\U000027A1':{'en':':woman_in_manual_wheelchair_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':frau_in_manuellem_rollstuhl_nach_rechts_mitteldunkle_hautfarbe:','es':':mujer_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':femme_en_fauteuil_roulant_manuel_vers_la_droite_peau_mate:','ja':':手動式車椅子の女性_右向き_やや濃い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_여자_진한_갈색_피부:','pt':':mulher_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_morena_escura:','it':':donna_su_sedia_a_rotelle_manuale_a_destra_carnagione_abbastanza_scura:','fa':':زن_در_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_گندمی:','id':':wanita_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':坐手动轮椅的女人面向右边_中等深肤色:','ru':':женщина_в_ручном_кресле_коляске_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_يدوي_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F469\U0001F3FC\U0000200D\U0001F9BD\U0000200D\U000027A1\U0000FE0F':{'en':':woman_in_manual_wheelchair_facing_right_medium-light_skin_tone:','status':fully_qualified,'E':15.1,'de':':frau_in_manuellem_rollstuhl_nach_rechts_mittelhelle_hautfarbe:','es':':mujer_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':femme_en_fauteuil_roulant_manuel_vers_la_droite_peau_moyennement_claire:','ja':':手動式車椅子の女性_右向き_やや薄い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_여자_연한_갈색_피부:','pt':':mulher_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_morena_clara:','it':':donna_su_sedia_a_rotelle_manuale_a_destra_carnagione_abbastanza_chiara:','fa':':زن_در_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_روشن:','id':':wanita_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':坐手动轮椅的女人面向右边_中等浅肤色:','ru':':женщина_в_ручном_кресле_коляске_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_tekerlekli_sandalyede_kadın_orta_açık_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_يدوي_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FC\U0000200D\U0001F9BD\U0000200D\U000027A1':{'en':':woman_in_manual_wheelchair_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':frau_in_manuellem_rollstuhl_nach_rechts_mittelhelle_hautfarbe:','es':':mujer_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':femme_en_fauteuil_roulant_manuel_vers_la_droite_peau_moyennement_claire:','ja':':手動式車椅子の女性_右向き_やや薄い肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_여자_연한_갈색_피부:','pt':':mulher_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_morena_clara:','it':':donna_su_sedia_a_rotelle_manuale_a_destra_carnagione_abbastanza_chiara:','fa':':زن_در_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_روشن:','id':':wanita_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':坐手动轮椅的女人面向右边_中等浅肤色:','ru':':женщина_в_ручном_кресле_коляске_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_tekerlekli_sandalyede_kadın_orta_açık_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_يدوي_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F9BD\U0000200D\U000027A1\U0000FE0F':{'en':':woman_in_manual_wheelchair_facing_right_medium_skin_tone:','status':fully_qualified,'E':15.1,'de':':frau_in_manuellem_rollstuhl_nach_rechts_mittlere_hautfarbe:','es':':mujer_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_medio:','fr':':femme_en_fauteuil_roulant_manuel_vers_la_droite_peau_légèrement_mate:','ja':':手動式車椅子の女性_右向き_中間の肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_여자_갈색_피부:','pt':':mulher_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_morena:','it':':donna_su_sedia_a_rotelle_manuale_a_destra_carnagione_olivastra:','fa':':زن_در_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_طلایی:','id':':wanita_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_sedang:','zh':':坐手动轮椅的女人面向右边_中等肤色:','ru':':женщина_в_ручном_кресле_коляске_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_kadın_orta_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_يدوي_متجه_لليمين_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F9BD\U0000200D\U000027A1':{'en':':woman_in_manual_wheelchair_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':frau_in_manuellem_rollstuhl_nach_rechts_mittlere_hautfarbe:','es':':mujer_en_silla_de_ruedas_manual_hacia_la_derecha_tono_de_piel_medio:','fr':':femme_en_fauteuil_roulant_manuel_vers_la_droite_peau_légèrement_mate:','ja':':手動式車椅子の女性_右向き_中間の肌色:','ko':':오른쪽을_향한_수동_휠체어를_탄_여자_갈색_피부:','pt':':mulher_em_cadeira_de_rodas_manual_de_frente_para_a_direita_pele_morena:','it':':donna_su_sedia_a_rotelle_manuale_a_destra_carnagione_olivastra:','fa':':زن_در_صندلی_چرخ_دار_دستی_به_سمت_راست_پوست_طلایی:','id':':wanita_dengan_kursi_roda_manual_menghadap_kanan_warna_kulit_sedang:','zh':':坐手动轮椅的女人面向右边_中等肤色:','ru':':женщина_в_ручном_кресле_коляске_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_tekerlekli_sandalyede_kadın_orta_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_يدوي_متجه_لليمين_بلون_معتدل:'},'\U0001F469\U0001F3FB\U0000200D\U0001F9BD':{'en':':woman_in_manual_wheelchair_light_skin_tone:','status':fully_qualified,'E':12,'de':':frau_in_manuellem_rollstuhl_helle_hautfarbe:','es':':mujer_en_silla_de_ruedas_manual_tono_de_piel_claro:','fr':':femme_en_fauteuil_roulant_manuel_peau_claire:','ja':':手動式車椅子の女性_薄い肌色:','ko':':수동_휠체어를_탄_여자_하얀_피부:','pt':':mulher_em_cadeira_de_rodas_manual_pele_clara:','it':':donna_su_sedia_a_rotelle_manuale_carnagione_chiara:','fa':':زن_در_صندلی_چرخ_دار_دستی_پوست_سفید:','id':':wanita_dengan_kursi_roda_manual_warna_kulit_cerah:','zh':':坐手动轮椅的女人_较浅肤色:','ru':':женщина_в_ручном_кресле_коляске_очень_светлый_тон_кожи:','tr':':tekerlekli_sandalyede_kadın_açık_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_يدوي_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F9BD':{'en':':woman_in_manual_wheelchair_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':frau_in_manuellem_rollstuhl_mitteldunkle_hautfarbe:','es':':mujer_en_silla_de_ruedas_manual_tono_de_piel_oscuro_medio:','fr':':femme_en_fauteuil_roulant_manuel_peau_mate:','ja':':手動式車椅子の女性_やや濃い肌色:','ko':':수동_휠체어를_탄_여자_진한_갈색_피부:','pt':':mulher_em_cadeira_de_rodas_manual_pele_morena_escura:','it':':donna_su_sedia_a_rotelle_manuale_carnagione_abbastanza_scura:','fa':':زن_در_صندلی_چرخ_دار_دستی_پوست_گندمی:','id':':wanita_dengan_kursi_roda_manual_warna_kulit_gelap-sedang:','zh':':坐手动轮椅的女人_中等深肤色:','ru':':женщина_в_ручном_кресле_коляске_темный_тон_кожи:','tr':':tekerlekli_sandalyede_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_يدوي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F9BD':{'en':':woman_in_manual_wheelchair_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':frau_in_manuellem_rollstuhl_mittelhelle_hautfarbe:','es':':mujer_en_silla_de_ruedas_manual_tono_de_piel_claro_medio:','fr':':femme_en_fauteuil_roulant_manuel_peau_moyennement_claire:','ja':':手動式車椅子の女性_やや薄い肌色:','ko':':수동_휠체어를_탄_여자_연한_갈색_피부:','pt':':mulher_em_cadeira_de_rodas_manual_pele_morena_clara:','it':':donna_su_sedia_a_rotelle_manuale_carnagione_abbastanza_chiara:','fa':':زن_در_صندلی_چرخ_دار_دستی_پوست_روشن:','id':':wanita_dengan_kursi_roda_manual_warna_kulit_cerah-sedang:','zh':':坐手动轮椅的女人_中等浅肤色:','ru':':женщина_в_ручном_кресле_коляске_светлый_тон_кожи:','tr':':tekerlekli_sandalyede_kadın_orta_açık_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_يدوي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F9BD':{'en':':woman_in_manual_wheelchair_medium_skin_tone:','status':fully_qualified,'E':12,'de':':frau_in_manuellem_rollstuhl_mittlere_hautfarbe:','es':':mujer_en_silla_de_ruedas_manual_tono_de_piel_medio:','fr':':femme_en_fauteuil_roulant_manuel_peau_légèrement_mate:','ja':':手動式車椅子の女性_中間の肌色:','ko':':수동_휠체어를_탄_여자_갈색_피부:','pt':':mulher_em_cadeira_de_rodas_manual_pele_morena:','it':':donna_su_sedia_a_rotelle_manuale_carnagione_olivastra:','fa':':زن_در_صندلی_چرخ_دار_دستی_پوست_طلایی:','id':':wanita_dengan_kursi_roda_manual_warna_kulit_sedang:','zh':':坐手动轮椅的女人_中等肤色:','ru':':женщина_в_ручном_кресле_коляске_средний_тон_кожи:','tr':':tekerlekli_sandalyede_kadın_orta_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_يدوي_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U0001F9BC':{'en':':woman_in_motorized_wheelchair:','status':fully_qualified,'E':12,'de':':frau_in_elektrischem_rollstuhl:','es':':mujer_en_silla_de_ruedas_eléctrica:','fr':':femme_en_fauteuil_motorisé:','ja':':電動車椅子の女性:','ko':':전동_휠체어를_탄_여자:','pt':':mulher_em_cadeira_de_rodas_motorizada:','it':':donna_su_sedia_a_rotelle_motorizzata:','fa':':زن_در_صندلی_چرخ_دار_برقی:','id':':wanita_dengan_kursi_roda_bermotor:','zh':':坐电动轮椅的女人:','ru':':женщина_в_электрическом_кресле_коляске:','tr':':elektrikli_tekerlekli_sandalyede_kadın:','ar':':سيدة_على_كرسي_متحرك_كهربائي:'},'\U0001F469\U0001F3FF\U0000200D\U0001F9BC':{'en':':woman_in_motorized_wheelchair_dark_skin_tone:','status':fully_qualified,'E':12,'de':':frau_in_elektrischem_rollstuhl_dunkle_hautfarbe:','es':':mujer_en_silla_de_ruedas_eléctrica_tono_de_piel_oscuro:','fr':':femme_en_fauteuil_motorisé_peau_foncée:','ja':':電動車椅子の女性_濃い肌色:','ko':':전동_휠체어를_탄_여자_검은색_피부:','pt':':mulher_em_cadeira_de_rodas_motorizada_pele_escura:','it':':donna_su_sedia_a_rotelle_motorizzata_carnagione_scura:','fa':':زن_در_صندلی_چرخ_دار_برقی_پوست_آبنوسی:','id':':wanita_dengan_kursi_roda_bermotor_warna_kulit_gelap:','zh':':坐电动轮椅的女人_较深肤色:','ru':':женщина_в_электрическом_кресле_коляске_очень_темный_тон_кожи:','tr':':elektrikli_tekerlekli_sandalyede_kadın_koyu_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_كهربائي_بشرة_بلون_غامق:'},'\U0001F469\U0000200D\U0001F9BC\U0000200D\U000027A1\U0000FE0F':{'en':':woman_in_motorized_wheelchair_facing_right:','status':fully_qualified,'E':15.1,'de':':frau_in_elektrischem_rollstuhl_nach_rechts:','es':':mujer_en_silla_de_ruedas_eléctrica_hacia_la_derecha:','fr':':femme_en_fauteuil_motorisé_vers_la_droite:','ja':':電動車椅子の女性_右向き:','ko':':오른쪽을_향한_전동_휠체어를_탄_여자:','pt':':mulher_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita:','it':':donna_su_sedia_a_rotelle_motorizzata_a_destra:','fa':':زن_در_صندلی_چرخ_دار_برقی_به_سمت_راست:','id':':wanita_dengan_kursi_roda_bermotor_menghadap_kanan:','zh':':坐电动轮椅的女人面向右边:','ru':':женщина_в_электрическом_кресле_коляске_направленные_вправо:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kadın:','ar':':سيدة_على_كرسي_متحرك_كهربائي_متجه_لليمين:'},'\U0001F469\U0000200D\U0001F9BC\U0000200D\U000027A1':{'en':':woman_in_motorized_wheelchair_facing_right:','status':minimally_qualified,'E':15.1,'de':':frau_in_elektrischem_rollstuhl_nach_rechts:','es':':mujer_en_silla_de_ruedas_eléctrica_hacia_la_derecha:','fr':':femme_en_fauteuil_motorisé_vers_la_droite:','ja':':電動車椅子の女性_右向き:','ko':':오른쪽을_향한_전동_휠체어를_탄_여자:','pt':':mulher_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita:','it':':donna_su_sedia_a_rotelle_motorizzata_a_destra:','fa':':زن_در_صندلی_چرخ_دار_برقی_به_سمت_راست:','id':':wanita_dengan_kursi_roda_bermotor_menghadap_kanan:','zh':':坐电动轮椅的女人面向右边:','ru':':женщина_в_электрическом_кресле_коляске_направленные_вправо:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kadın:','ar':':سيدة_على_كرسي_متحرك_كهربائي_متجه_لليمين:'},'\U0001F469\U0001F3FF\U0000200D\U0001F9BC\U0000200D\U000027A1\U0000FE0F':{'en':':woman_in_motorized_wheelchair_facing_right_dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':frau_in_elektrischem_rollstuhl_nach_rechts_dunkle_hautfarbe:','es':':mujer_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_oscuro:','fr':':femme_en_fauteuil_motorisé_vers_la_droite_peau_foncée:','ja':':電動車椅子の女性_右向き_濃い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_여자_검은색_피부:','pt':':mulher_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_escura:','it':':donna_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_scura:','fa':':زن_در_صندلی_چرخ_دار_برقی_به_سمت_راستپوست_آبنوسی:','id':':wanita_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_gelap:','zh':':坐电动轮椅的女人面向右边_较深肤色:','ru':':женщина_в_электрическом_кресле_коляске_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kadın_koyu_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_كهربائي_متجه_لليمين_بلون_غامق:'},'\U0001F469\U0001F3FF\U0000200D\U0001F9BC\U0000200D\U000027A1':{'en':':woman_in_motorized_wheelchair_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':frau_in_elektrischem_rollstuhl_nach_rechts_dunkle_hautfarbe:','es':':mujer_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_oscuro:','fr':':femme_en_fauteuil_motorisé_vers_la_droite_peau_foncée:','ja':':電動車椅子の女性_右向き_濃い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_여자_검은색_피부:','pt':':mulher_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_escura:','it':':donna_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_scura:','fa':':زن_در_صندلی_چرخ_دار_برقی_به_سمت_راستپوست_آبنوسی:','id':':wanita_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_gelap:','zh':':坐电动轮椅的女人面向右边_较深肤色:','ru':':женщина_в_электрическом_кресле_коляске_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kadın_koyu_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_كهربائي_متجه_لليمين_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F9BC\U0000200D\U000027A1\U0000FE0F':{'en':':woman_in_motorized_wheelchair_facing_right_light_skin_tone:','status':fully_qualified,'E':15.1,'de':':frau_in_elektrischem_rollstuhl_nach_rechts_helle_hautfarbe:','es':':mujer_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_claro:','fr':':femme_en_fauteuil_motorisé_vers_la_droite_peau_claire:','ja':':電動車椅子の女性_右向き_薄い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_여자_하얀_피부:','pt':':mulher_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_clara:','it':':donna_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_chiara:','fa':':زن_در_صندلی_چرخ_دار_برقی_به_سمت_راست_پوست_سفید:','id':':wanita_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_cerah:','zh':':坐电动轮椅的女人面向右边_较浅肤色:','ru':':женщина_в_электрическом_кресле_коляске_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kadın_açık_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_كهربائي_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FB\U0000200D\U0001F9BC\U0000200D\U000027A1':{'en':':woman_in_motorized_wheelchair_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':frau_in_elektrischem_rollstuhl_nach_rechts_helle_hautfarbe:','es':':mujer_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_claro:','fr':':femme_en_fauteuil_motorisé_vers_la_droite_peau_claire:','ja':':電動車椅子の女性_右向き_薄い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_여자_하얀_피부:','pt':':mulher_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_clara:','it':':donna_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_chiara:','fa':':زن_در_صندلی_چرخ_دار_برقی_به_سمت_راست_پوست_سفید:','id':':wanita_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_cerah:','zh':':坐电动轮椅的女人面向右边_较浅肤色:','ru':':женщина_в_электрическом_кресле_коляске_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kadın_açık_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_كهربائي_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F9BC\U0000200D\U000027A1\U0000FE0F':{'en':':woman_in_motorized_wheelchair_facing_right_medium-dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':frau_in_elektrischem_rollstuhl_nach_rechts_mitteldunkle_hautfarbe:','es':':mujer_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':femme_en_fauteuil_motorisé_vers_la_droite_peau_mate:','ja':':電動車椅子の女性_右向き_やや濃い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_여자_진한_갈색_피부:','pt':':mulher_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_morena_escura:','it':':donna_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_abbastanza_scura:','fa':':زن_در_صندلی_چرخ_دار_برقی_به_سمت_راست_پوست_گندمی:','id':':wanita_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':坐电动轮椅的女人面向右边_中等深肤色:','ru':':женщина_в_электрическом_кресле_коляске_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_كهربائي_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F469\U0001F3FE\U0000200D\U0001F9BC\U0000200D\U000027A1':{'en':':woman_in_motorized_wheelchair_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':frau_in_elektrischem_rollstuhl_nach_rechts_mitteldunkle_hautfarbe:','es':':mujer_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':femme_en_fauteuil_motorisé_vers_la_droite_peau_mate:','ja':':電動車椅子の女性_右向き_やや濃い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_여자_진한_갈색_피부:','pt':':mulher_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_morena_escura:','it':':donna_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_abbastanza_scura:','fa':':زن_در_صندلی_چرخ_دار_برقی_به_سمت_راست_پوست_گندمی:','id':':wanita_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':坐电动轮椅的女人面向右边_中等深肤色:','ru':':женщина_в_электрическом_кресле_коляске_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_كهربائي_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F469\U0001F3FC\U0000200D\U0001F9BC\U0000200D\U000027A1\U0000FE0F':{'en':':woman_in_motorized_wheelchair_facing_right_medium-light_skin_tone:','status':fully_qualified,'E':15.1,'de':':frau_in_elektrischem_rollstuhl_nach_rechts_mittelhelle_hautfarbe:','es':':mujer_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':femme_en_fauteuil_motorisé_vers_la_droite_peau_moyennement_claire:','ja':':電動車椅子の女性_右向き_やや薄い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_여자_연한_갈색_피부:','pt':':mulher_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_morena_clara:','it':':donna_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_abbastanza_chiara:','fa':':زن_در_صندلی_چرخ_دار_برقی_به_سمت_راست_پوست_روشن:','id':':wanita_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':坐电动轮椅的女人面向右边_中等浅肤色:','ru':':женщина_в_электрическом_кресле_коляске_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kadın_orta_açık_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_كهربائي_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FC\U0000200D\U0001F9BC\U0000200D\U000027A1':{'en':':woman_in_motorized_wheelchair_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':frau_in_elektrischem_rollstuhl_nach_rechts_mittelhelle_hautfarbe:','es':':mujer_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':femme_en_fauteuil_motorisé_vers_la_droite_peau_moyennement_claire:','ja':':電動車椅子の女性_右向き_やや薄い肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_여자_연한_갈색_피부:','pt':':mulher_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_morena_clara:','it':':donna_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_abbastanza_chiara:','fa':':زن_در_صندلی_چرخ_دار_برقی_به_سمت_راست_پوست_روشن:','id':':wanita_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':坐电动轮椅的女人面向右边_中等浅肤色:','ru':':женщина_в_электрическом_кресле_коляске_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kadın_orta_açık_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_كهربائي_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F9BC\U0000200D\U000027A1\U0000FE0F':{'en':':woman_in_motorized_wheelchair_facing_right_medium_skin_tone:','status':fully_qualified,'E':15.1,'de':':frau_in_elektrischem_rollstuhl_nach_rechts_mittlere_hautfarbe:','es':':mujer_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_medio:','fr':':femme_en_fauteuil_motorisé_vers_la_droite_peau_légèrement_mate:','ja':':電動車椅子の女性_右向き_中間の肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_여자_갈색_피부:','pt':':mulher_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_morena:','it':':donna_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_olivastra:','fa':':زن_در_صندلی_چرخ_دار_برقی_به_سمت_راست_پوست_طلایی:','id':':wanita_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_sedang:','zh':':坐电动轮椅的女人面向右边_中等肤色:','ru':':женщина_в_электрическом_кресле_коляске_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kadın_orta_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_كهربائي_متجه_لليمين_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F9BC\U0000200D\U000027A1':{'en':':woman_in_motorized_wheelchair_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':frau_in_elektrischem_rollstuhl_nach_rechts_mittlere_hautfarbe:','es':':mujer_en_silla_de_ruedas_eléctrica_hacia_la_derecha_tono_de_piel_medio:','fr':':femme_en_fauteuil_motorisé_vers_la_droite_peau_légèrement_mate:','ja':':電動車椅子の女性_右向き_中間の肌色:','ko':':오른쪽을_향한_전동_휠체어를_탄_여자_갈색_피부:','pt':':mulher_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_pele_morena:','it':':donna_su_sedia_a_rotelle_motorizzata_a_destra_carnagione_olivastra:','fa':':زن_در_صندلی_چرخ_دار_برقی_به_سمت_راست_پوست_طلایی:','id':':wanita_dengan_kursi_roda_bermotor_menghadap_kanan_warna_kulit_sedang:','zh':':坐电动轮椅的女人面向右边_中等肤色:','ru':':женщина_в_электрическом_кресле_коляске_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_elektrikli_tekerlekli_sandalyede_kadın_orta_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_كهربائي_متجه_لليمين_بلون_معتدل:'},'\U0001F469\U0001F3FB\U0000200D\U0001F9BC':{'en':':woman_in_motorized_wheelchair_light_skin_tone:','status':fully_qualified,'E':12,'de':':frau_in_elektrischem_rollstuhl_helle_hautfarbe:','es':':mujer_en_silla_de_ruedas_eléctrica_tono_de_piel_claro:','fr':':femme_en_fauteuil_motorisé_peau_claire:','ja':':電動車椅子の女性_薄い肌色:','ko':':전동_휠체어를_탄_여자_하얀_피부:','pt':':mulher_em_cadeira_de_rodas_motorizada_pele_clara:','it':':donna_su_sedia_a_rotelle_motorizzata_carnagione_chiara:','fa':':زن_در_صندلی_چرخ_دار_برقی_پوست_سفید:','id':':wanita_dengan_kursi_roda_bermotor_warna_kulit_cerah:','zh':':坐电动轮椅的女人_较浅肤色:','ru':':женщина_в_электрическом_кресле_коляске_очень_светлый_тон_кожи:','tr':':elektrikli_tekerlekli_sandalyede_kadın_açık_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_كهربائي_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F9BC':{'en':':woman_in_motorized_wheelchair_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':frau_in_elektrischem_rollstuhl_mitteldunkle_hautfarbe:','es':':mujer_en_silla_de_ruedas_eléctrica_tono_de_piel_oscuro_medio:','fr':':femme_en_fauteuil_motorisé_peau_mate:','ja':':電動車椅子の女性_やや濃い肌色:','ko':':전동_휠체어를_탄_여자_진한_갈색_피부:','pt':':mulher_em_cadeira_de_rodas_motorizada_pele_morena_escura:','it':':donna_su_sedia_a_rotelle_motorizzata_carnagione_abbastanza_scura:','fa':':زن_در_صندلی_چرخ_دار_برقی_پوست_گندمی:','id':':wanita_dengan_kursi_roda_bermotor_warna_kulit_gelap-sedang:','zh':':坐电动轮椅的女人_中等深肤色:','ru':':женщина_в_электрическом_кресле_коляске_темный_тон_кожи:','tr':':elektrikli_tekerlekli_sandalyede_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_كهربائي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F9BC':{'en':':woman_in_motorized_wheelchair_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':frau_in_elektrischem_rollstuhl_mittelhelle_hautfarbe:','es':':mujer_en_silla_de_ruedas_eléctrica_tono_de_piel_claro_medio:','fr':':femme_en_fauteuil_motorisé_peau_moyennement_claire:','ja':':電動車椅子の女性_やや薄い肌色:','ko':':전동_휠체어를_탄_여자_연한_갈색_피부:','pt':':mulher_em_cadeira_de_rodas_motorizada_pele_morena_clara:','it':':donna_su_sedia_a_rotelle_motorizzata_carnagione_abbastanza_chiara:','fa':':زن_در_صندلی_چرخ_دار_برقی_پوست_روشن:','id':':wanita_dengan_kursi_roda_bermotor_warna_kulit_cerah-sedang:','zh':':坐电动轮椅的女人_中等浅肤色:','ru':':женщина_в_электрическом_кресле_коляске_светлый_тон_кожи:','tr':':elektrikli_tekerlekli_sandalyede_kadın_orta_açık_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_كهربائي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F9BC':{'en':':woman_in_motorized_wheelchair_medium_skin_tone:','status':fully_qualified,'E':12,'de':':frau_in_elektrischem_rollstuhl_mittlere_hautfarbe:','es':':mujer_en_silla_de_ruedas_eléctrica_tono_de_piel_medio:','fr':':femme_en_fauteuil_motorisé_peau_légèrement_mate:','ja':':電動車椅子の女性_中間の肌色:','ko':':전동_휠체어를_탄_여자_갈색_피부:','pt':':mulher_em_cadeira_de_rodas_motorizada_pele_morena:','it':':donna_su_sedia_a_rotelle_motorizzata_carnagione_olivastra:','fa':':زن_در_صندلی_چرخ_دار_برقی_پوست_طلایی:','id':':wanita_dengan_kursi_roda_bermotor_warna_kulit_sedang:','zh':':坐电动轮椅的女人_中等肤色:','ru':':женщина_в_электрическом_кресле_коляске_средний_тон_кожи:','tr':':elektrikli_tekerlekli_sandalyede_kadın_orta_cilt_tonu:','ar':':سيدة_على_كرسي_متحرك_كهربائي_بشرة_بلون_معتدل:'},'\U0001F9D6\U0000200D\U00002640\U0000FE0F':{'en':':woman_in_steamy_room:','status':fully_qualified,'E':5,'alias':[':sauna_woman:'],'de':':frau_in_dampfsauna:','es':':mujer_en_una_sauna:','fr':':femme_au_hammam:','ja':':サウナに入る女:','ko':':사우나_하는_여자:','pt':':mulher_na_sauna:','it':':donna_in_sauna:','fa':':زن_در_حمام_بخار:','id':':wanita_di_ruangan_beruap:','zh':':蒸房里的女人:','ru':':женщина_в_бане:','tr':':buhar_odasındaki_kadın:','ar':':امرأة_في_حمام_بخار:'},'\U0001F9D6\U0000200D\U00002640':{'en':':woman_in_steamy_room:','status':minimally_qualified,'E':5,'alias':[':sauna_woman:'],'de':':frau_in_dampfsauna:','es':':mujer_en_una_sauna:','fr':':femme_au_hammam:','ja':':サウナに入る女:','ko':':사우나_하는_여자:','pt':':mulher_na_sauna:','it':':donna_in_sauna:','fa':':زن_در_حمام_بخار:','id':':wanita_di_ruangan_beruap:','zh':':蒸房里的女人:','ru':':женщина_в_бане:','tr':':buhar_odasındaki_kadın:','ar':':امرأة_في_حمام_بخار:'},'\U0001F9D6\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_in_steamy_room_dark_skin_tone:','status':fully_qualified,'E':5,'de':':frau_in_dampfsauna_dunkle_hautfarbe:','es':':mujer_en_una_sauna_tono_de_piel_oscuro:','fr':':femme_au_hammam_peau_foncée:','ja':':サウナに入る女_濃い肌色:','ko':':사우나_하는_여자_검은색_피부:','pt':':mulher_na_sauna_pele_escura:','it':':donna_in_sauna_carnagione_scura:','fa':':زن_در_حمام_بخار_پوست_آبنوسی:','id':':wanita_di_ruangan_beruap_warna_kulit_gelap:','zh':':蒸房里的女人_较深肤色:','ru':':женщина_в_бане_очень_темныи_тон_кожи:','tr':':buhar_odasındaki_kadın_koyu_cilt_tonu:','ar':':امرأة_في_حمام_بخار_بشرة_بلون_غامق:'},'\U0001F9D6\U0001F3FF\U0000200D\U00002640':{'en':':woman_in_steamy_room_dark_skin_tone:','status':minimally_qualified,'E':5,'de':':frau_in_dampfsauna_dunkle_hautfarbe:','es':':mujer_en_una_sauna_tono_de_piel_oscuro:','fr':':femme_au_hammam_peau_foncée:','ja':':サウナに入る女_濃い肌色:','ko':':사우나_하는_여자_검은색_피부:','pt':':mulher_na_sauna_pele_escura:','it':':donna_in_sauna_carnagione_scura:','fa':':زن_در_حمام_بخار_پوست_آبنوسی:','id':':wanita_di_ruangan_beruap_warna_kulit_gelap:','zh':':蒸房里的女人_较深肤色:','ru':':женщина_в_бане_очень_темный_тон_кожи:','tr':':buhar_odasındaki_kadın_koyu_cilt_tonu:','ar':':امرأة_في_حمام_بخار_بشرة_بلون_غامق:'},'\U0001F9D6\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_in_steamy_room_light_skin_tone:','status':fully_qualified,'E':5,'de':':frau_in_dampfsauna_helle_hautfarbe:','es':':mujer_en_una_sauna_tono_de_piel_claro:','fr':':femme_au_hammam_peau_claire:','ja':':サウナに入る女_薄い肌色:','ko':':사우나_하는_여자_하얀_피부:','pt':':mulher_na_sauna_pele_clara:','it':':donna_in_sauna_carnagione_chiara:','fa':':زن_در_حمام_بخار_پوست_سفید:','id':':wanita_di_ruangan_beruap_warna_kulit_cerah:','zh':':蒸房里的女人_较浅肤色:','ru':':женщина_в_бане_очень_светлыи_тон_кожи:','tr':':buhar_odasındaki_kadın_açık_cilt_tonu:','ar':':امرأة_في_حمام_بخار_بشرة_بلون_فاتح:'},'\U0001F9D6\U0001F3FB\U0000200D\U00002640':{'en':':woman_in_steamy_room_light_skin_tone:','status':minimally_qualified,'E':5,'de':':frau_in_dampfsauna_helle_hautfarbe:','es':':mujer_en_una_sauna_tono_de_piel_claro:','fr':':femme_au_hammam_peau_claire:','ja':':サウナに入る女_薄い肌色:','ko':':사우나_하는_여자_하얀_피부:','pt':':mulher_na_sauna_pele_clara:','it':':donna_in_sauna_carnagione_chiara:','fa':':زن_در_حمام_بخار_پوست_سفید:','id':':wanita_di_ruangan_beruap_warna_kulit_cerah:','zh':':蒸房里的女人_较浅肤色:','ru':':женщина_в_бане_очень_светлый_тон_кожи:','tr':':buhar_odasındaki_kadın_açık_cilt_tonu:','ar':':امرأة_في_حمام_بخار_بشرة_بلون_فاتح:'},'\U0001F9D6\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_in_steamy_room_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':frau_in_dampfsauna_mitteldunkle_hautfarbe:','es':':mujer_en_una_sauna_tono_de_piel_oscuro_medio:','fr':':femme_au_hammam_peau_mate:','ja':':サウナに入る女_やや濃い肌色:','ko':':사우나_하는_여자_진한_갈색_피부:','pt':':mulher_na_sauna_pele_morena_escura:','it':':donna_in_sauna_carnagione_abbastanza_scura:','fa':':زن_در_حمام_بخار_پوست_گندمی:','id':':wanita_di_ruangan_beruap_warna_kulit_gelap-sedang:','zh':':蒸房里的女人_中等深肤色:','ru':':женщина_в_бане_темныи_тон_кожи:','tr':':buhar_odasındaki_kadın_orta_koyu_cilt_tonu:','ar':':امرأة_في_حمام_بخار_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D6\U0001F3FE\U0000200D\U00002640':{'en':':woman_in_steamy_room_medium-dark_skin_tone:','status':minimally_qualified,'E':5,'de':':frau_in_dampfsauna_mitteldunkle_hautfarbe:','es':':mujer_en_una_sauna_tono_de_piel_oscuro_medio:','fr':':femme_au_hammam_peau_mate:','ja':':サウナに入る女_やや濃い肌色:','ko':':사우나_하는_여자_진한_갈색_피부:','pt':':mulher_na_sauna_pele_morena_escura:','it':':donna_in_sauna_carnagione_abbastanza_scura:','fa':':زن_در_حمام_بخار_پوست_گندمی:','id':':wanita_di_ruangan_beruap_warna_kulit_gelap-sedang:','zh':':蒸房里的女人_中等深肤色:','ru':':женщина_в_бане_темный_тон_кожи:','tr':':buhar_odasındaki_kadın_orta_koyu_cilt_tonu:','ar':':امرأة_في_حمام_بخار_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D6\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_in_steamy_room_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':frau_in_dampfsauna_mittelhelle_hautfarbe:','es':':mujer_en_una_sauna_tono_de_piel_claro_medio:','fr':':femme_au_hammam_peau_moyennement_claire:','ja':':サウナに入る女_やや薄い肌色:','ko':':사우나_하는_여자_연한_갈색_피부:','pt':':mulher_na_sauna_pele_morena_clara:','it':':donna_in_sauna_carnagione_abbastanza_chiara:','fa':':زن_در_حمام_بخار_پوست_روشن:','id':':wanita_di_ruangan_beruap_warna_kulit_cerah-sedang:','zh':':蒸房里的女人_中等浅肤色:','ru':':женщина_в_бане_светлыи_тон_кожи:','tr':':buhar_odasındaki_kadın_orta_açık_cilt_tonu:','ar':':امرأة_في_حمام_بخار_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D6\U0001F3FC\U0000200D\U00002640':{'en':':woman_in_steamy_room_medium-light_skin_tone:','status':minimally_qualified,'E':5,'de':':frau_in_dampfsauna_mittelhelle_hautfarbe:','es':':mujer_en_una_sauna_tono_de_piel_claro_medio:','fr':':femme_au_hammam_peau_moyennement_claire:','ja':':サウナに入る女_やや薄い肌色:','ko':':사우나_하는_여자_연한_갈색_피부:','pt':':mulher_na_sauna_pele_morena_clara:','it':':donna_in_sauna_carnagione_abbastanza_chiara:','fa':':زن_در_حمام_بخار_پوست_روشن:','id':':wanita_di_ruangan_beruap_warna_kulit_cerah-sedang:','zh':':蒸房里的女人_中等浅肤色:','ru':':женщина_в_бане_светлый_тон_кожи:','tr':':buhar_odasındaki_kadın_orta_açık_cilt_tonu:','ar':':امرأة_في_حمام_بخار_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D6\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_in_steamy_room_medium_skin_tone:','status':fully_qualified,'E':5,'de':':frau_in_dampfsauna_mittlere_hautfarbe:','es':':mujer_en_una_sauna_tono_de_piel_medio:','fr':':femme_au_hammam_peau_légèrement_mate:','ja':':サウナに入る女_中間の肌色:','ko':':사우나_하는_여자_갈색_피부:','pt':':mulher_na_sauna_pele_morena:','it':':donna_in_sauna_carnagione_olivastra:','fa':':زن_در_حمام_بخار_پوست_طلایی:','id':':wanita_di_ruangan_beruap_warna_kulit_sedang:','zh':':蒸房里的女人_中等肤色:','ru':':женщина_в_бане_среднии_тон_кожи:','tr':':buhar_odasındaki_kadın_orta_cilt_tonu:','ar':':امرأة_في_حمام_بخار_بشرة_بلون_معتدل:'},'\U0001F9D6\U0001F3FD\U0000200D\U00002640':{'en':':woman_in_steamy_room_medium_skin_tone:','status':minimally_qualified,'E':5,'de':':frau_in_dampfsauna_mittlere_hautfarbe:','es':':mujer_en_una_sauna_tono_de_piel_medio:','fr':':femme_au_hammam_peau_légèrement_mate:','ja':':サウナに入る女_中間の肌色:','ko':':사우나_하는_여자_갈색_피부:','pt':':mulher_na_sauna_pele_morena:','it':':donna_in_sauna_carnagione_olivastra:','fa':':زن_در_حمام_بخار_پوست_طلایی:','id':':wanita_di_ruangan_beruap_warna_kulit_sedang:','zh':':蒸房里的女人_中等肤色:','ru':':женщина_в_бане_средний_тон_кожи:','tr':':buhar_odasındaki_kadın_orta_cilt_tonu:','ar':':امرأة_في_حمام_بخار_بشرة_بلون_معتدل:'},'\U0001F935\U0000200D\U00002640\U0000FE0F':{'en':':woman_in_tuxedo:','status':fully_qualified,'E':13,'de':':frau_im_smoking:','es':':mujer_con_esmoquin:','fr':':femme_en_smoking:','ja':':タキシードの女性:','ko':':턱시도를_입은_여자:','pt':':mulher_de_smoking:','it':':donna_in_smoking:','fa':':زن_با_کت_و_شلوار_و_پاپیون:','id':':wanita_bertuksedo:','zh':':穿礼服的女人:','ru':':женщина_в_смокинге:','tr':':smokinli_kadın:','ar':':امرأة_ببدلة_رسمية:'},'\U0001F935\U0000200D\U00002640':{'en':':woman_in_tuxedo:','status':minimally_qualified,'E':13,'de':':frau_im_smoking:','es':':mujer_con_esmoquin:','fr':':femme_en_smoking:','ja':':タキシードの女性:','ko':':턱시도를_입은_여자:','pt':':mulher_de_smoking:','it':':donna_in_smoking:','fa':':زن_با_کت_و_شلوار_و_پاپیون:','id':':wanita_bertuksedo:','zh':':穿礼服的女人:','ru':':женщина_в_смокинге:','tr':':smokinli_kadın:','ar':':امرأة_ببدلة_رسمية:'},'\U0001F935\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_in_tuxedo_dark_skin_tone:','status':fully_qualified,'E':13,'de':':frau_im_smoking_dunkle_hautfarbe:','es':':mujer_con_esmoquin_tono_de_piel_oscuro:','fr':':femme_en_smoking_peau_foncée:','ja':':タキシードの女性_濃い肌色:','ko':':턱시도를_입은_여자_검은색_피부:','pt':':mulher_de_smoking_pele_escura:','it':':donna_in_smoking_carnagione_scura:','fa':':زن_با_کت_و_شلوار_و_پاپیون_پوست_آبنوسی:','id':':wanita_bertuksedo_warna_kulit_gelap:','zh':':穿礼服的女人_较深肤色:','ru':':женщина_в_смокинге_очень_темныи_тон_кожи:','tr':':smokinli_kadın_koyu_cilt_tonu:','ar':':امرأة_ببدلة_رسمية_بشرة_بلون_غامق:'},'\U0001F935\U0001F3FF\U0000200D\U00002640':{'en':':woman_in_tuxedo_dark_skin_tone:','status':minimally_qualified,'E':13,'de':':frau_im_smoking_dunkle_hautfarbe:','es':':mujer_con_esmoquin_tono_de_piel_oscuro:','fr':':femme_en_smoking_peau_foncée:','ja':':タキシードの女性_濃い肌色:','ko':':턱시도를_입은_여자_검은색_피부:','pt':':mulher_de_smoking_pele_escura:','it':':donna_in_smoking_carnagione_scura:','fa':':زن_با_کت_و_شلوار_و_پاپیون_پوست_آبنوسی:','id':':wanita_bertuksedo_warna_kulit_gelap:','zh':':穿礼服的女人_较深肤色:','ru':':женщина_в_смокинге_очень_темный_тон_кожи:','tr':':smokinli_kadın_koyu_cilt_tonu:','ar':':امرأة_ببدلة_رسمية_بشرة_بلون_غامق:'},'\U0001F935\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_in_tuxedo_light_skin_tone:','status':fully_qualified,'E':13,'de':':frau_im_smoking_helle_hautfarbe:','es':':mujer_con_esmoquin_tono_de_piel_claro:','fr':':femme_en_smoking_peau_claire:','ja':':タキシードの女性_薄い肌色:','ko':':턱시도를_입은_여자_하얀_피부:','pt':':mulher_de_smoking_pele_clara:','it':':donna_in_smoking_carnagione_chiara:','fa':':زن_با_کت_و_شلوار_و_پاپیون_پوست_سفید:','id':':wanita_bertuksedo_warna_kulit_cerah:','zh':':穿礼服的女人_较浅肤色:','ru':':женщина_в_смокинге_очень_светлыи_тон_кожи:','tr':':smokinli_kadın_açık_cilt_tonu:','ar':':امرأة_ببدلة_رسمية_بشرة_بلون_فاتح:'},'\U0001F935\U0001F3FB\U0000200D\U00002640':{'en':':woman_in_tuxedo_light_skin_tone:','status':minimally_qualified,'E':13,'de':':frau_im_smoking_helle_hautfarbe:','es':':mujer_con_esmoquin_tono_de_piel_claro:','fr':':femme_en_smoking_peau_claire:','ja':':タキシードの女性_薄い肌色:','ko':':턱시도를_입은_여자_하얀_피부:','pt':':mulher_de_smoking_pele_clara:','it':':donna_in_smoking_carnagione_chiara:','fa':':زن_با_کت_و_شلوار_و_پاپیون_پوست_سفید:','id':':wanita_bertuksedo_warna_kulit_cerah:','zh':':穿礼服的女人_较浅肤色:','ru':':женщина_в_смокинге_очень_светлый_тон_кожи:','tr':':smokinli_kadın_açık_cilt_tonu:','ar':':امرأة_ببدلة_رسمية_بشرة_بلون_فاتح:'},'\U0001F935\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_in_tuxedo_medium-dark_skin_tone:','status':fully_qualified,'E':13,'de':':frau_im_smoking_mitteldunkle_hautfarbe:','es':':mujer_con_esmoquin_tono_de_piel_oscuro_medio:','fr':':femme_en_smoking_peau_mate:','ja':':タキシードの女性_やや濃い肌色:','ko':':턱시도를_입은_여자_진한_갈색_피부:','pt':':mulher_de_smoking_pele_morena_escura:','it':':donna_in_smoking_carnagione_abbastanza_scura:','fa':':زن_با_کت_و_شلوار_و_پاپیون_پوست_گندمی:','id':':wanita_bertuksedo_warna_kulit_gelap-sedang:','zh':':穿礼服的女人_中等深肤色:','ru':':женщина_в_смокинге_темныи_тон_кожи:','tr':':smokinli_kadın_orta_koyu_cilt_tonu:','ar':':امرأة_ببدلة_رسمية_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F935\U0001F3FE\U0000200D\U00002640':{'en':':woman_in_tuxedo_medium-dark_skin_tone:','status':minimally_qualified,'E':13,'de':':frau_im_smoking_mitteldunkle_hautfarbe:','es':':mujer_con_esmoquin_tono_de_piel_oscuro_medio:','fr':':femme_en_smoking_peau_mate:','ja':':タキシードの女性_やや濃い肌色:','ko':':턱시도를_입은_여자_진한_갈색_피부:','pt':':mulher_de_smoking_pele_morena_escura:','it':':donna_in_smoking_carnagione_abbastanza_scura:','fa':':زن_با_کت_و_شلوار_و_پاپیون_پوست_گندمی:','id':':wanita_bertuksedo_warna_kulit_gelap-sedang:','zh':':穿礼服的女人_中等深肤色:','ru':':женщина_в_смокинге_темный_тон_кожи:','tr':':smokinli_kadın_orta_koyu_cilt_tonu:','ar':':امرأة_ببدلة_رسمية_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F935\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_in_tuxedo_medium-light_skin_tone:','status':fully_qualified,'E':13,'de':':frau_im_smoking_mittelhelle_hautfarbe:','es':':mujer_con_esmoquin_tono_de_piel_claro_medio:','fr':':femme_en_smoking_peau_moyennement_claire:','ja':':タキシードの女性_やや薄い肌色:','ko':':턱시도를_입은_여자_연한_갈색_피부:','pt':':mulher_de_smoking_pele_morena_clara:','it':':donna_in_smoking_carnagione_abbastanza_chiara:','fa':':زن_با_کت_و_شلوار_و_پاپیون_پوست_روشن:','id':':wanita_bertuksedo_warna_kulit_cerah-sedang:','zh':':穿礼服的女人_中等浅肤色:','ru':':женщина_в_смокинге_светлыи_тон_кожи:','tr':':smokinli_kadın_orta_açık_cilt_tonu:','ar':':امرأة_ببدلة_رسمية_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F935\U0001F3FC\U0000200D\U00002640':{'en':':woman_in_tuxedo_medium-light_skin_tone:','status':minimally_qualified,'E':13,'de':':frau_im_smoking_mittelhelle_hautfarbe:','es':':mujer_con_esmoquin_tono_de_piel_claro_medio:','fr':':femme_en_smoking_peau_moyennement_claire:','ja':':タキシードの女性_やや薄い肌色:','ko':':턱시도를_입은_여자_연한_갈색_피부:','pt':':mulher_de_smoking_pele_morena_clara:','it':':donna_in_smoking_carnagione_abbastanza_chiara:','fa':':زن_با_کت_و_شلوار_و_پاپیون_پوست_روشن:','id':':wanita_bertuksedo_warna_kulit_cerah-sedang:','zh':':穿礼服的女人_中等浅肤色:','ru':':женщина_в_смокинге_светлый_тон_кожи:','tr':':smokinli_kadın_orta_açık_cilt_tonu:','ar':':امرأة_ببدلة_رسمية_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F935\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_in_tuxedo_medium_skin_tone:','status':fully_qualified,'E':13,'de':':frau_im_smoking_mittlere_hautfarbe:','es':':mujer_con_esmoquin_tono_de_piel_medio:','fr':':femme_en_smoking_peau_légèrement_mate:','ja':':タキシードの女性_中間の肌色:','ko':':턱시도를_입은_여자_갈색_피부:','pt':':mulher_de_smoking_pele_morena:','it':':donna_in_smoking_carnagione_olivastra:','fa':':زن_با_کت_و_شلوار_و_پاپیون_پوست_طلایی:','id':':wanita_bertuksedo_warna_kulit_sedang:','zh':':穿礼服的女人_中等肤色:','ru':':женщина_в_смокинге_среднии_тон_кожи:','tr':':smokinli_kadın_orta_cilt_tonu:','ar':':امرأة_ببدلة_رسمية_بشرة_بلون_معتدل:'},'\U0001F935\U0001F3FD\U0000200D\U00002640':{'en':':woman_in_tuxedo_medium_skin_tone:','status':minimally_qualified,'E':13,'de':':frau_im_smoking_mittlere_hautfarbe:','es':':mujer_con_esmoquin_tono_de_piel_medio:','fr':':femme_en_smoking_peau_légèrement_mate:','ja':':タキシードの女性_中間の肌色:','ko':':턱시도를_입은_여자_갈색_피부:','pt':':mulher_de_smoking_pele_morena:','it':':donna_in_smoking_carnagione_olivastra:','fa':':زن_با_کت_و_شلوار_و_پاپیون_پوست_طلایی:','id':':wanita_bertuksedo_warna_kulit_sedang:','zh':':穿礼服的女人_中等肤色:','ru':':женщина_в_смокинге_средний_тон_кожи:','tr':':smokinli_kadın_orta_cilt_tonu:','ar':':امرأة_ببدلة_رسمية_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U00002696\U0000FE0F':{'en':':woman_judge:','status':fully_qualified,'E':4,'de':':richterin:','es':':fiscal_mujer:','fr':':juge_femme:','ja':':女性の裁判官:','ko':':여자_판사:','pt':':juíza:','it':':giudice_donna:','fa':':قاضی_زن:','id':':hakim_wanita:','zh':':女法官:','ru':':женщина-судья:','tr':':kadın_hakim:','ar':':قاضية:'},'\U0001F469\U0000200D\U00002696':{'en':':woman_judge:','status':minimally_qualified,'E':4,'de':':richterin:','es':':fiscal_mujer:','fr':':juge_femme:','ja':':女性の裁判官:','ko':':여자_판사:','pt':':juíza:','it':':giudice_donna:','fa':':قاضی_زن:','id':':hakim_wanita:','zh':':女法官:','ru':':женщина_судья:','tr':':kadın_hakim:','ar':':قاضية:'},'\U0001F469\U0001F3FF\U0000200D\U00002696\U0000FE0F':{'en':':woman_judge_dark_skin_tone:','status':fully_qualified,'E':4,'de':':richterin_dunkle_hautfarbe:','es':':fiscal_mujer_tono_de_piel_oscuro:','fr':':juge_femme_peau_foncée:','ja':':女性の裁判官_濃い肌色:','ko':':여자_판사_검은색_피부:','pt':':juíza_pele_escura:','it':':giudice_donna_carnagione_scura:','fa':':قاضی_زن_پوست_آبنوسی:','id':':hakim_wanita_warna_kulit_gelap:','zh':':女法官_较深肤色:','ru':':женщина-судья_очень_темныи_тон_кожи:','tr':':kadın_hakim_koyu_cilt_tonu:','ar':':قاضية_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FF\U0000200D\U00002696':{'en':':woman_judge_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':richterin_dunkle_hautfarbe:','es':':fiscal_mujer_tono_de_piel_oscuro:','fr':':juge_femme_peau_foncée:','ja':':女性の裁判官_濃い肌色:','ko':':여자_판사_검은색_피부:','pt':':juíza_pele_escura:','it':':giudice_donna_carnagione_scura:','fa':':قاضی_زن_پوست_آبنوسی:','id':':hakim_wanita_warna_kulit_gelap:','zh':':女法官_较深肤色:','ru':':женщина_судья_очень_темный_тон_кожи:','tr':':kadın_hakim_koyu_cilt_tonu:','ar':':قاضية_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U00002696\U0000FE0F':{'en':':woman_judge_light_skin_tone:','status':fully_qualified,'E':4,'de':':richterin_helle_hautfarbe:','es':':fiscal_mujer_tono_de_piel_claro:','fr':':juge_femme_peau_claire:','ja':':女性の裁判官_薄い肌色:','ko':':여자_판사_하얀_피부:','pt':':juíza_pele_clara:','it':':giudice_donna_carnagione_chiara:','fa':':قاضی_زن_پوست_سفید:','id':':hakim_wanita_warna_kulit_cerah:','zh':':女法官_较浅肤色:','ru':':женщина-судья_очень_светлыи_тон_кожи:','tr':':kadın_hakim_açık_cilt_tonu:','ar':':قاضية_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FB\U0000200D\U00002696':{'en':':woman_judge_light_skin_tone:','status':minimally_qualified,'E':4,'de':':richterin_helle_hautfarbe:','es':':fiscal_mujer_tono_de_piel_claro:','fr':':juge_femme_peau_claire:','ja':':女性の裁判官_薄い肌色:','ko':':여자_판사_하얀_피부:','pt':':juíza_pele_clara:','it':':giudice_donna_carnagione_chiara:','fa':':قاضی_زن_پوست_سفید:','id':':hakim_wanita_warna_kulit_cerah:','zh':':女法官_较浅肤色:','ru':':женщина_судья_очень_светлый_тон_кожи:','tr':':kadın_hakim_açık_cilt_tonu:','ar':':قاضية_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U00002696\U0000FE0F':{'en':':woman_judge_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':richterin_mitteldunkle_hautfarbe:','es':':fiscal_mujer_tono_de_piel_oscuro_medio:','fr':':juge_femme_peau_mate:','ja':':女性の裁判官_やや濃い肌色:','ko':':여자_판사_진한_갈색_피부:','pt':':juíza_pele_morena_escura:','it':':giudice_donna_carnagione_abbastanza_scura:','fa':':قاضی_زن_پوست_گندمی:','id':':hakim_wanita_warna_kulit_gelap-sedang:','zh':':女法官_中等深肤色:','ru':':женщина-судья_темныи_тон_кожи:','tr':':kadın_hakim_orta_koyu_cilt_tonu:','ar':':قاضية_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FE\U0000200D\U00002696':{'en':':woman_judge_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':richterin_mitteldunkle_hautfarbe:','es':':fiscal_mujer_tono_de_piel_oscuro_medio:','fr':':juge_femme_peau_mate:','ja':':女性の裁判官_やや濃い肌色:','ko':':여자_판사_진한_갈색_피부:','pt':':juíza_pele_morena_escura:','it':':giudice_donna_carnagione_abbastanza_scura:','fa':':قاضی_زن_پوست_گندمی:','id':':hakim_wanita_warna_kulit_gelap-sedang:','zh':':女法官_中等深肤色:','ru':':женщина_судья_темный_тон_кожи:','tr':':kadın_hakim_orta_koyu_cilt_tonu:','ar':':قاضية_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U00002696\U0000FE0F':{'en':':woman_judge_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':richterin_mittelhelle_hautfarbe:','es':':fiscal_mujer_tono_de_piel_claro_medio:','fr':':juge_femme_peau_moyennement_claire:','ja':':女性の裁判官_やや薄い肌色:','ko':':여자_판사_연한_갈색_피부:','pt':':juíza_pele_morena_clara:','it':':giudice_donna_carnagione_abbastanza_chiara:','fa':':قاضی_زن_پوست_روشن:','id':':hakim_wanita_warna_kulit_cerah-sedang:','zh':':女法官_中等浅肤色:','ru':':женщина-судья_светлыи_тон_кожи:','tr':':kadın_hakim_orta_açık_cilt_tonu:','ar':':قاضية_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FC\U0000200D\U00002696':{'en':':woman_judge_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':richterin_mittelhelle_hautfarbe:','es':':fiscal_mujer_tono_de_piel_claro_medio:','fr':':juge_femme_peau_moyennement_claire:','ja':':女性の裁判官_やや薄い肌色:','ko':':여자_판사_연한_갈색_피부:','pt':':juíza_pele_morena_clara:','it':':giudice_donna_carnagione_abbastanza_chiara:','fa':':قاضی_زن_پوست_روشن:','id':':hakim_wanita_warna_kulit_cerah-sedang:','zh':':女法官_中等浅肤色:','ru':':женщина_судья_светлый_тон_кожи:','tr':':kadın_hakim_orta_açık_cilt_tonu:','ar':':قاضية_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002696\U0000FE0F':{'en':':woman_judge_medium_skin_tone:','status':fully_qualified,'E':4,'de':':richterin_mittlere_hautfarbe:','es':':fiscal_mujer_tono_de_piel_medio:','fr':':juge_femme_peau_légèrement_mate:','ja':':女性の裁判官_中間の肌色:','ko':':여자_판사_갈색_피부:','pt':':juíza_pele_morena:','it':':giudice_donna_carnagione_olivastra:','fa':':قاضی_زن_پوست_طلایی:','id':':hakim_wanita_warna_kulit_sedang:','zh':':女法官_中等肤色:','ru':':женщина-судья_среднии_тон_кожи:','tr':':kadın_hakim_orta_cilt_tonu:','ar':':قاضية_بشرة_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002696':{'en':':woman_judge_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':richterin_mittlere_hautfarbe:','es':':fiscal_mujer_tono_de_piel_medio:','fr':':juge_femme_peau_légèrement_mate:','ja':':女性の裁判官_中間の肌色:','ko':':여자_판사_갈색_피부:','pt':':juíza_pele_morena:','it':':giudice_donna_carnagione_olivastra:','fa':':قاضی_زن_پوست_طلایی:','id':':hakim_wanita_warna_kulit_sedang:','zh':':女法官_中等肤色:','ru':':женщина_судья_средний_тон_кожи:','tr':':kadın_hakim_orta_cilt_tonu:','ar':':قاضية_بشرة_بلون_معتدل:'},'\U0001F939\U0000200D\U00002640\U0000FE0F':{'en':':woman_juggling:','status':fully_qualified,'E':4,'de':':jongleurin:','es':':mujer_haciendo_malabares:','fr':':jongleuse:','ja':':ジャグリングをする女:','ko':':저글링하는_여자:','pt':':mulher_malabarista:','it':':giocoliere_donna:','fa':':زن_ژانگولربازی_می_کند:','id':':wanita_melempar:','zh':':女生抛接杂耍:','ru':':женщина-жонглер:','tr':':jonglör_kadın:','ar':':لاعبة_قذف_كرات:'},'\U0001F939\U0000200D\U00002640':{'en':':woman_juggling:','status':minimally_qualified,'E':4,'de':':jongleurin:','es':':mujer_haciendo_malabares:','fr':':jongleuse:','ja':':ジャグリングをする女:','ko':':저글링하는_여자:','pt':':mulher_malabarista:','it':':giocoliere_donna:','fa':':زن_ژانگولربازی_می_کند:','id':':wanita_melempar:','zh':':女生抛接杂耍:','ru':':женщина_жонглер:','tr':':jonglör_kadın:','ar':':لاعبة_قذف_كرات:'},'\U0001F939\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_juggling_dark_skin_tone:','status':fully_qualified,'E':4,'de':':jongleurin_dunkle_hautfarbe:','es':':mujer_haciendo_malabares_tono_de_piel_oscuro:','fr':':jongleuse_peau_foncée:','ja':':ジャグリングをする女_濃い肌色:','ko':':저글링하는_여자_검은색_피부:','pt':':mulher_malabarista_pele_escura:','it':':giocoliere_donna_carnagione_scura:','fa':':زن_ژانگولربازی_می_کند_پوست_آبنوسی:','id':':wanita_melempar_warna_kulit_gelap:','zh':':女生抛接杂耍_较深肤色:','ru':':женщина-жонглер_очень_темныи_тон_кожи:','tr':':jonglör_kadın_koyu_cilt_tonu:','ar':':لاعبة_قذف_كرات_بشرة_بلون_غامق:'},'\U0001F939\U0001F3FF\U0000200D\U00002640':{'en':':woman_juggling_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':jongleurin_dunkle_hautfarbe:','es':':mujer_haciendo_malabares_tono_de_piel_oscuro:','fr':':jongleuse_peau_foncée:','ja':':ジャグリングをする女_濃い肌色:','ko':':저글링하는_여자_검은색_피부:','pt':':mulher_malabarista_pele_escura:','it':':giocoliere_donna_carnagione_scura:','fa':':زن_ژانگولربازی_می_کند_پوست_آبنوسی:','id':':wanita_melempar_warna_kulit_gelap:','zh':':女生抛接杂耍_较深肤色:','ru':':женщина_жонглер_очень_темный_тон_кожи:','tr':':jonglör_kadın_koyu_cilt_tonu:','ar':':لاعبة_قذف_كرات_بشرة_بلون_غامق:'},'\U0001F939\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_juggling_light_skin_tone:','status':fully_qualified,'E':4,'de':':jongleurin_helle_hautfarbe:','es':':mujer_haciendo_malabares_tono_de_piel_claro:','fr':':jongleuse_peau_claire:','ja':':ジャグリングをする女_薄い肌色:','ko':':저글링하는_여자_하얀_피부:','pt':':mulher_malabarista_pele_clara:','it':':giocoliere_donna_carnagione_chiara:','fa':':زن_ژانگولربازی_می_کند_پوست_سفید:','id':':wanita_melempar_warna_kulit_cerah:','zh':':女生抛接杂耍_较浅肤色:','ru':':женщина-жонглер_очень_светлыи_тон_кожи:','tr':':jonglör_kadın_açık_cilt_tonu:','ar':':لاعبة_قذف_كرات_بشرة_بلون_فاتح:'},'\U0001F939\U0001F3FB\U0000200D\U00002640':{'en':':woman_juggling_light_skin_tone:','status':minimally_qualified,'E':4,'de':':jongleurin_helle_hautfarbe:','es':':mujer_haciendo_malabares_tono_de_piel_claro:','fr':':jongleuse_peau_claire:','ja':':ジャグリングをする女_薄い肌色:','ko':':저글링하는_여자_하얀_피부:','pt':':mulher_malabarista_pele_clara:','it':':giocoliere_donna_carnagione_chiara:','fa':':زن_ژانگولربازی_می_کند_پوست_سفید:','id':':wanita_melempar_warna_kulit_cerah:','zh':':女生抛接杂耍_较浅肤色:','ru':':женщина_жонглер_очень_светлый_тон_кожи:','tr':':jonglör_kadın_açık_cilt_tonu:','ar':':لاعبة_قذف_كرات_بشرة_بلون_فاتح:'},'\U0001F939\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_juggling_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':jongleurin_mitteldunkle_hautfarbe:','es':':mujer_haciendo_malabares_tono_de_piel_oscuro_medio:','fr':':jongleuse_peau_mate:','ja':':ジャグリングをする女_やや濃い肌色:','ko':':저글링하는_여자_진한_갈색_피부:','pt':':mulher_malabarista_pele_morena_escura:','it':':giocoliere_donna_carnagione_abbastanza_scura:','fa':':زن_ژانگولربازی_می_کند_پوست_گندمی:','id':':wanita_melempar_warna_kulit_gelap-sedang:','zh':':女生抛接杂耍_中等深肤色:','ru':':женщина-жонглер_темныи_тон_кожи:','tr':':jonglör_kadın_orta_koyu_cilt_tonu:','ar':':لاعبة_قذف_كرات_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F939\U0001F3FE\U0000200D\U00002640':{'en':':woman_juggling_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':jongleurin_mitteldunkle_hautfarbe:','es':':mujer_haciendo_malabares_tono_de_piel_oscuro_medio:','fr':':jongleuse_peau_mate:','ja':':ジャグリングをする女_やや濃い肌色:','ko':':저글링하는_여자_진한_갈색_피부:','pt':':mulher_malabarista_pele_morena_escura:','it':':giocoliere_donna_carnagione_abbastanza_scura:','fa':':زن_ژانگولربازی_می_کند_پوست_گندمی:','id':':wanita_melempar_warna_kulit_gelap-sedang:','zh':':女生抛接杂耍_中等深肤色:','ru':':женщина_жонглер_темный_тон_кожи:','tr':':jonglör_kadın_orta_koyu_cilt_tonu:','ar':':لاعبة_قذف_كرات_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F939\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_juggling_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':jongleurin_mittelhelle_hautfarbe:','es':':mujer_haciendo_malabares_tono_de_piel_claro_medio:','fr':':jongleuse_peau_moyennement_claire:','ja':':ジャグリングをする女_やや薄い肌色:','ko':':저글링하는_여자_연한_갈색_피부:','pt':':mulher_malabarista_pele_morena_clara:','it':':giocoliere_donna_carnagione_abbastanza_chiara:','fa':':زن_ژانگولربازی_می_کند_پوست_روشن:','id':':wanita_melempar_warna_kulit_cerah-sedang:','zh':':女生抛接杂耍_中等浅肤色:','ru':':женщина-жонглер_светлыи_тон_кожи:','tr':':jonglör_kadın_orta_açık_cilt_tonu:','ar':':لاعبة_قذف_كرات_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F939\U0001F3FC\U0000200D\U00002640':{'en':':woman_juggling_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':jongleurin_mittelhelle_hautfarbe:','es':':mujer_haciendo_malabares_tono_de_piel_claro_medio:','fr':':jongleuse_peau_moyennement_claire:','ja':':ジャグリングをする女_やや薄い肌色:','ko':':저글링하는_여자_연한_갈색_피부:','pt':':mulher_malabarista_pele_morena_clara:','it':':giocoliere_donna_carnagione_abbastanza_chiara:','fa':':زن_ژانگولربازی_می_کند_پوست_روشن:','id':':wanita_melempar_warna_kulit_cerah-sedang:','zh':':女生抛接杂耍_中等浅肤色:','ru':':женщина_жонглер_светлый_тон_кожи:','tr':':jonglör_kadın_orta_açık_cilt_tonu:','ar':':لاعبة_قذف_كرات_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F939\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_juggling_medium_skin_tone:','status':fully_qualified,'E':4,'de':':jongleurin_mittlere_hautfarbe:','es':':mujer_haciendo_malabares_tono_de_piel_medio:','fr':':jongleuse_peau_légèrement_mate:','ja':':ジャグリングをする女_中間の肌色:','ko':':저글링하는_여자_갈색_피부:','pt':':mulher_malabarista_pele_morena:','it':':giocoliere_donna_carnagione_olivastra:','fa':':زن_ژانگولربازی_می_کند_پوست_طلایی:','id':':wanita_melempar_warna_kulit_sedang:','zh':':女生抛接杂耍_中等肤色:','ru':':женщина-жонглер_среднии_тон_кожи:','tr':':jonglör_kadın_orta_cilt_tonu:','ar':':لاعبة_قذف_كرات_بشرة_بلون_معتدل:'},'\U0001F939\U0001F3FD\U0000200D\U00002640':{'en':':woman_juggling_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':jongleurin_mittlere_hautfarbe:','es':':mujer_haciendo_malabares_tono_de_piel_medio:','fr':':jongleuse_peau_légèrement_mate:','ja':':ジャグリングをする女_中間の肌色:','ko':':저글링하는_여자_갈색_피부:','pt':':mulher_malabarista_pele_morena:','it':':giocoliere_donna_carnagione_olivastra:','fa':':زن_ژانگولربازی_می_کند_پوست_طلایی:','id':':wanita_melempar_warna_kulit_sedang:','zh':':女生抛接杂耍_中等肤色:','ru':':женщина_жонглер_средний_тон_кожи:','tr':':jonglör_kadın_orta_cilt_tonu:','ar':':لاعبة_قذف_كرات_بشرة_بلون_معتدل:'},'\U0001F9CE\U0000200D\U00002640\U0000FE0F':{'en':':woman_kneeling:','status':fully_qualified,'E':12,'alias':[':kneeling_woman:'],'de':':kniende_frau:','es':':mujer_de_rodillas:','fr':':femme_à_genoux:','ja':':正座する女性:','ko':':무릎을_꿇은_여자:','pt':':mulher_ajoelhando:','it':':donna_inginocchiata:','fa':':زن_زانوزده:','id':':wanita_berlutut:','zh':':跪下的女人:','ru':':женщина_на_коленях:','tr':':diz_çöken_kadın:','ar':':سيدة_جالسة_على_ركبتيها:'},'\U0001F9CE\U0000200D\U00002640':{'en':':woman_kneeling:','status':minimally_qualified,'E':12,'alias':[':kneeling_woman:'],'de':':kniende_frau:','es':':mujer_de_rodillas:','fr':':femme_à_genoux:','ja':':正座する女性:','ko':':무릎을_꿇은_여자:','pt':':mulher_ajoelhando:','it':':donna_inginocchiata:','fa':':زن_زانوزده:','id':':wanita_berlutut:','zh':':跪下的女人:','ru':':женщина_на_коленях:','tr':':diz_çöken_kadın:','ar':':سيدة_جالسة_على_ركبتيها:'},'\U0001F9CE\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_kneeling_dark_skin_tone:','status':fully_qualified,'E':12,'de':':kniende_frau_dunkle_hautfarbe:','es':':mujer_de_rodillas_tono_de_piel_oscuro:','fr':':femme_à_genoux_peau_foncée:','ja':':正座する女性_濃い肌色:','ko':':무릎을_꿇은_여자_검은색_피부:','pt':':mulher_ajoelhando_pele_escura:','it':':donna_inginocchiata_carnagione_scura:','fa':':زن_زانوزده_پوست_آبنوسی:','id':':wanita_berlutut_warna_kulit_gelap:','zh':':跪下的女人_较深肤色:','ru':':женщина_на_коленях_очень_темныи_тон_кожи:','tr':':diz_çöken_kadın_koyu_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_بشرة_بلون_غامق:'},'\U0001F9CE\U0001F3FF\U0000200D\U00002640':{'en':':woman_kneeling_dark_skin_tone:','status':minimally_qualified,'E':12,'de':':kniende_frau_dunkle_hautfarbe:','es':':mujer_de_rodillas_tono_de_piel_oscuro:','fr':':femme_à_genoux_peau_foncée:','ja':':正座する女性_濃い肌色:','ko':':무릎을_꿇은_여자_검은색_피부:','pt':':mulher_ajoelhando_pele_escura:','it':':donna_inginocchiata_carnagione_scura:','fa':':زن_زانوزده_پوست_آبنوسی:','id':':wanita_berlutut_warna_kulit_gelap:','zh':':跪下的女人_较深肤色:','ru':':женщина_на_коленях_очень_темный_тон_кожи:','tr':':diz_çöken_kadın_koyu_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_بشرة_بلون_غامق:'},'\U0001F9CE\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':woman_kneeling_facing_right:','status':fully_qualified,'E':15.1,'de':':kniende_frau_nach_rechts:','es':':mujer_de_rodillas_hacia_la_derecha:','fr':':femme_à_genoux_vers_la_droite:','ja':':正座する女性_右向き:','ko':':오른쪽을_향한_무릎을_꿇은_여자:','pt':':mulher_ajoelhando_de_frente_para_a_direita:','it':':donna_inginocchiata_a_destra:','fa':':زن_زانوزده_به_سمت_راست:','id':':wanita_berlutut_menghadap_kanan:','zh':':跪下的女人面向右边:','ru':':женщина_на_коленях_направленные_вправо:','tr':':sağa_dönük_diz_çöken_kadın:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين:'},'\U0001F9CE\U0000200D\U00002640\U0000200D\U000027A1\U0000FE0F':{'en':':woman_kneeling_facing_right:','status':minimally_qualified,'E':15.1,'de':':kniende_frau_nach_rechts:','es':':mujer_de_rodillas_hacia_la_derecha:','fr':':femme_à_genoux_vers_la_droite:','ja':':正座する女性_右向き:','ko':':오른쪽을_향한_무릎을_꿇은_여자:','pt':':mulher_ajoelhando_de_frente_para_a_direita:','it':':donna_inginocchiata_a_destra:','fa':':زن_زانوزده_به_سمت_راست:','id':':wanita_berlutut_menghadap_kanan:','zh':':跪下的女人面向右边:','ru':':женщина_на_коленях_направленные_вправо:','tr':':sağa_dönük_diz_çöken_kadın:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين:'},'\U0001F9CE\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1':{'en':':woman_kneeling_facing_right:','status':minimally_qualified,'E':15.1,'de':':kniende_frau_nach_rechts:','es':':mujer_de_rodillas_hacia_la_derecha:','fr':':femme_à_genoux_vers_la_droite:','ja':':正座する女性_右向き:','ko':':오른쪽을_향한_무릎을_꿇은_여자:','pt':':mulher_ajoelhando_de_frente_para_a_direita:','it':':donna_inginocchiata_a_destra:','fa':':زن_زانوزده_به_سمت_راست:','id':':wanita_berlutut_menghadap_kanan:','zh':':跪下的女人面向右边:','ru':':женщина_на_коленях_направленные_вправо:','tr':':sağa_dönük_diz_çöken_kadın:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين:'},'\U0001F9CE\U0000200D\U00002640\U0000200D\U000027A1':{'en':':woman_kneeling_facing_right:','status':minimally_qualified,'E':15.1,'de':':kniende_frau_nach_rechts:','es':':mujer_de_rodillas_hacia_la_derecha:','fr':':femme_à_genoux_vers_la_droite:','ja':':正座する女性_右向き:','ko':':오른쪽을_향한_무릎을_꿇은_여자:','pt':':mulher_ajoelhando_de_frente_para_a_direita:','it':':donna_inginocchiata_a_destra:','fa':':زن_زانوزده_به_سمت_راست:','id':':wanita_berlutut_menghadap_kanan:','zh':':跪下的女人面向右边:','ru':':женщина_на_коленях_направленные_вправо:','tr':':sağa_dönük_diz_çöken_kadın:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين:'},'\U0001F9CE\U0001F3FF\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':woman_kneeling_facing_right_dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':kniende_frau_nach_rechts_dunkle_hautfarbe:','es':':mujer_de_rodillas_hacia_la_derecha_tono_de_piel_oscuro:','fr':':femme_à_genoux_vers_la_droite_peau_foncée:','ja':':正座する女性_右向き_濃い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_여자_검은색_피부:','pt':':mulher_ajoelhando_de_frente_para_a_direita_pele_escura:','it':':donna_inginocchiata_a_destra_carnagione_scura:','fa':':زن_زانوزده_به_سمت_راستپوست_آبنوسی:','id':':wanita_berlutut_menghadap_kanan_warna_kulit_gelap:','zh':':跪下的女人面向右边_较深肤色:','ru':':женщина_на_коленях_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_diz_çöken_kadın_koyu_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين_بلون_غامق:'},'\U0001F9CE\U0001F3FF\U0000200D\U00002640\U0000200D\U000027A1\U0000FE0F':{'en':':woman_kneeling_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniende_frau_nach_rechts_dunkle_hautfarbe:','es':':mujer_de_rodillas_hacia_la_derecha_tono_de_piel_oscuro:','fr':':femme_à_genoux_vers_la_droite_peau_foncée:','ja':':正座する女性_右向き_濃い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_여자_검은색_피부:','pt':':mulher_ajoelhando_de_frente_para_a_direita_pele_escura:','it':':donna_inginocchiata_a_destra_carnagione_scura:','fa':':زن_زانوزده_به_سمت_راستپوست_آبنوسی:','id':':wanita_berlutut_menghadap_kanan_warna_kulit_gelap:','zh':':跪下的女人面向右边_较深肤色:','ru':':женщина_на_коленях_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_diz_çöken_kadın_koyu_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين_بلون_غامق:'},'\U0001F9CE\U0001F3FF\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1':{'en':':woman_kneeling_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniende_frau_nach_rechts_dunkle_hautfarbe:','es':':mujer_de_rodillas_hacia_la_derecha_tono_de_piel_oscuro:','fr':':femme_à_genoux_vers_la_droite_peau_foncée:','ja':':正座する女性_右向き_濃い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_여자_검은색_피부:','pt':':mulher_ajoelhando_de_frente_para_a_direita_pele_escura:','it':':donna_inginocchiata_a_destra_carnagione_scura:','fa':':زن_زانوزده_به_سمت_راستپوست_آبنوسی:','id':':wanita_berlutut_menghadap_kanan_warna_kulit_gelap:','zh':':跪下的女人面向右边_较深肤色:','ru':':женщина_на_коленях_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_diz_çöken_kadın_koyu_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين_بلون_غامق:'},'\U0001F9CE\U0001F3FF\U0000200D\U00002640\U0000200D\U000027A1':{'en':':woman_kneeling_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniende_frau_nach_rechts_dunkle_hautfarbe:','es':':mujer_de_rodillas_hacia_la_derecha_tono_de_piel_oscuro:','fr':':femme_à_genoux_vers_la_droite_peau_foncée:','ja':':正座する女性_右向き_濃い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_여자_검은색_피부:','pt':':mulher_ajoelhando_de_frente_para_a_direita_pele_escura:','it':':donna_inginocchiata_a_destra_carnagione_scura:','fa':':زن_زانوزده_به_سمت_راستپوست_آبنوسی:','id':':wanita_berlutut_menghadap_kanan_warna_kulit_gelap:','zh':':跪下的女人面向右边_较深肤色:','ru':':женщина_на_коленях_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_diz_çöken_kadın_koyu_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين_بلون_غامق:'},'\U0001F9CE\U0001F3FB\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':woman_kneeling_facing_right_light_skin_tone:','status':fully_qualified,'E':15.1,'de':':kniende_frau_nach_rechts_helle_hautfarbe:','es':':mujer_de_rodillas_hacia_la_derecha_tono_de_piel_claro:','fr':':femme_à_genoux_vers_la_droite_peau_claire:','ja':':正座する女性_右向き_薄い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_여자_하얀_피부:','pt':':mulher_ajoelhando_de_frente_para_a_direita_pele_clara:','it':':donna_inginocchiata_a_destra_carnagione_chiara:','fa':':زن_زانوزده_به_سمت_راست_پوست_سفید:','id':':wanita_berlutut_menghadap_kanan_warna_kulit_cerah:','zh':':跪下的女人面向右边_较浅肤色:','ru':':женщина_на_коленях_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_diz_çöken_kadın_açık_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F9CE\U0001F3FB\U0000200D\U00002640\U0000200D\U000027A1\U0000FE0F':{'en':':woman_kneeling_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniende_frau_nach_rechts_helle_hautfarbe:','es':':mujer_de_rodillas_hacia_la_derecha_tono_de_piel_claro:','fr':':femme_à_genoux_vers_la_droite_peau_claire:','ja':':正座する女性_右向き_薄い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_여자_하얀_피부:','pt':':mulher_ajoelhando_de_frente_para_a_direita_pele_clara:','it':':donna_inginocchiata_a_destra_carnagione_chiara:','fa':':زن_زانوزده_به_سمت_راست_پوست_سفید:','id':':wanita_berlutut_menghadap_kanan_warna_kulit_cerah:','zh':':跪下的女人面向右边_较浅肤色:','ru':':женщина_на_коленях_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_diz_çöken_kadın_açık_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F9CE\U0001F3FB\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1':{'en':':woman_kneeling_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniende_frau_nach_rechts_helle_hautfarbe:','es':':mujer_de_rodillas_hacia_la_derecha_tono_de_piel_claro:','fr':':femme_à_genoux_vers_la_droite_peau_claire:','ja':':正座する女性_右向き_薄い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_여자_하얀_피부:','pt':':mulher_ajoelhando_de_frente_para_a_direita_pele_clara:','it':':donna_inginocchiata_a_destra_carnagione_chiara:','fa':':زن_زانوزده_به_سمت_راست_پوست_سفید:','id':':wanita_berlutut_menghadap_kanan_warna_kulit_cerah:','zh':':跪下的女人面向右边_较浅肤色:','ru':':женщина_на_коленях_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_diz_çöken_kadın_açık_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F9CE\U0001F3FB\U0000200D\U00002640\U0000200D\U000027A1':{'en':':woman_kneeling_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniende_frau_nach_rechts_helle_hautfarbe:','es':':mujer_de_rodillas_hacia_la_derecha_tono_de_piel_claro:','fr':':femme_à_genoux_vers_la_droite_peau_claire:','ja':':正座する女性_右向き_薄い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_여자_하얀_피부:','pt':':mulher_ajoelhando_de_frente_para_a_direita_pele_clara:','it':':donna_inginocchiata_a_destra_carnagione_chiara:','fa':':زن_زانوزده_به_سمت_راست_پوست_سفید:','id':':wanita_berlutut_menghadap_kanan_warna_kulit_cerah:','zh':':跪下的女人面向右边_较浅肤色:','ru':':женщина_на_коленях_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_diz_çöken_kadın_açık_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F9CE\U0001F3FE\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':woman_kneeling_facing_right_medium-dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':kniende_frau_nach_rechts_mitteldunkle_hautfarbe:','es':':mujer_de_rodillas_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':femme_à_genoux_vers_la_droite_peau_mate:','ja':':正座する女性_右向き_やや濃い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_여자_진한_갈색_피부:','pt':':mulher_ajoelhando_de_frente_para_a_direita_pele_morena_escura:','it':':donna_inginocchiata_a_destra_carnagione_abbastanza_scura:','fa':':زن_زانوزده_به_سمت_راست_پوست_گندمی:','id':':wanita_berlutut_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':跪下的女人面向右边_中等深肤色:','ru':':женщина_на_коленях_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_diz_çöken_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F9CE\U0001F3FE\U0000200D\U00002640\U0000200D\U000027A1\U0000FE0F':{'en':':woman_kneeling_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniende_frau_nach_rechts_mitteldunkle_hautfarbe:','es':':mujer_de_rodillas_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':femme_à_genoux_vers_la_droite_peau_mate:','ja':':正座する女性_右向き_やや濃い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_여자_진한_갈색_피부:','pt':':mulher_ajoelhando_de_frente_para_a_direita_pele_morena_escura:','it':':donna_inginocchiata_a_destra_carnagione_abbastanza_scura:','fa':':زن_زانوزده_به_سمت_راست_پوست_گندمی:','id':':wanita_berlutut_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':跪下的女人面向右边_中等深肤色:','ru':':женщина_на_коленях_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_diz_çöken_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F9CE\U0001F3FE\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1':{'en':':woman_kneeling_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniende_frau_nach_rechts_mitteldunkle_hautfarbe:','es':':mujer_de_rodillas_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':femme_à_genoux_vers_la_droite_peau_mate:','ja':':正座する女性_右向き_やや濃い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_여자_진한_갈색_피부:','pt':':mulher_ajoelhando_de_frente_para_a_direita_pele_morena_escura:','it':':donna_inginocchiata_a_destra_carnagione_abbastanza_scura:','fa':':زن_زانوزده_به_سمت_راست_پوست_گندمی:','id':':wanita_berlutut_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':跪下的女人面向右边_中等深肤色:','ru':':женщина_на_коленях_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_diz_çöken_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F9CE\U0001F3FE\U0000200D\U00002640\U0000200D\U000027A1':{'en':':woman_kneeling_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniende_frau_nach_rechts_mitteldunkle_hautfarbe:','es':':mujer_de_rodillas_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':femme_à_genoux_vers_la_droite_peau_mate:','ja':':正座する女性_右向き_やや濃い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_여자_진한_갈색_피부:','pt':':mulher_ajoelhando_de_frente_para_a_direita_pele_morena_escura:','it':':donna_inginocchiata_a_destra_carnagione_abbastanza_scura:','fa':':زن_زانوزده_به_سمت_راست_پوست_گندمی:','id':':wanita_berlutut_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':跪下的女人面向右边_中等深肤色:','ru':':женщина_на_коленях_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_diz_çöken_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F9CE\U0001F3FC\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':woman_kneeling_facing_right_medium-light_skin_tone:','status':fully_qualified,'E':15.1,'de':':kniende_frau_nach_rechts_mittelhelle_hautfarbe:','es':':mujer_de_rodillas_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':femme_à_genoux_vers_la_droite_peau_moyennement_claire:','ja':':正座する女性_右向き_やや薄い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_여자_연한_갈색_피부:','pt':':mulher_ajoelhando_de_frente_para_a_direita_pele_morena_clara:','it':':donna_inginocchiata_a_destra_carnagione_abbastanza_chiara:','fa':':زن_زانوزده_به_سمت_راست_پوست_روشن:','id':':wanita_berlutut_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':跪下的女人面向右边_中等浅肤色:','ru':':женщина_на_коленях_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_diz_çöken_kadın_orta_açık_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F9CE\U0001F3FC\U0000200D\U00002640\U0000200D\U000027A1\U0000FE0F':{'en':':woman_kneeling_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniende_frau_nach_rechts_mittelhelle_hautfarbe:','es':':mujer_de_rodillas_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':femme_à_genoux_vers_la_droite_peau_moyennement_claire:','ja':':正座する女性_右向き_やや薄い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_여자_연한_갈색_피부:','pt':':mulher_ajoelhando_de_frente_para_a_direita_pele_morena_clara:','it':':donna_inginocchiata_a_destra_carnagione_abbastanza_chiara:','fa':':زن_زانوزده_به_سمت_راست_پوست_روشن:','id':':wanita_berlutut_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':跪下的女人面向右边_中等浅肤色:','ru':':женщина_на_коленях_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_diz_çöken_kadın_orta_açık_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F9CE\U0001F3FC\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1':{'en':':woman_kneeling_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniende_frau_nach_rechts_mittelhelle_hautfarbe:','es':':mujer_de_rodillas_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':femme_à_genoux_vers_la_droite_peau_moyennement_claire:','ja':':正座する女性_右向き_やや薄い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_여자_연한_갈색_피부:','pt':':mulher_ajoelhando_de_frente_para_a_direita_pele_morena_clara:','it':':donna_inginocchiata_a_destra_carnagione_abbastanza_chiara:','fa':':زن_زانوزده_به_سمت_راست_پوست_روشن:','id':':wanita_berlutut_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':跪下的女人面向右边_中等浅肤色:','ru':':женщина_на_коленях_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_diz_çöken_kadın_orta_açık_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F9CE\U0001F3FC\U0000200D\U00002640\U0000200D\U000027A1':{'en':':woman_kneeling_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniende_frau_nach_rechts_mittelhelle_hautfarbe:','es':':mujer_de_rodillas_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':femme_à_genoux_vers_la_droite_peau_moyennement_claire:','ja':':正座する女性_右向き_やや薄い肌色:','ko':':오른쪽을_향한_무릎을_꿇은_여자_연한_갈색_피부:','pt':':mulher_ajoelhando_de_frente_para_a_direita_pele_morena_clara:','it':':donna_inginocchiata_a_destra_carnagione_abbastanza_chiara:','fa':':زن_زانوزده_به_سمت_راست_پوست_روشن:','id':':wanita_berlutut_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':跪下的女人面向右边_中等浅肤色:','ru':':женщина_на_коленях_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_diz_çöken_kadın_orta_açık_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F9CE\U0001F3FD\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':woman_kneeling_facing_right_medium_skin_tone:','status':fully_qualified,'E':15.1,'de':':kniende_frau_nach_rechts_mittlere_hautfarbe:','es':':mujer_de_rodillas_hacia_la_derecha_tono_de_piel_medio:','fr':':femme_à_genoux_vers_la_droite_peau_légèrement_mate:','ja':':正座する女性_右向き_中間の肌色:','ko':':오른쪽을_향한_무릎을_꿇은_여자_갈색_피부:','pt':':mulher_ajoelhando_de_frente_para_a_direita_pele_morena:','it':':donna_inginocchiata_a_destra_carnagione_olivastra:','fa':':زن_زانوزده_به_سمت_راست_پوست_طلایی:','id':':wanita_berlutut_menghadap_kanan_warna_kulit_sedang:','zh':':跪下的女人面向右边_中等肤色:','ru':':женщина_на_коленях_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_diz_çöken_kadın_orta_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين_بلون_معتدل:'},'\U0001F9CE\U0001F3FD\U0000200D\U00002640\U0000200D\U000027A1\U0000FE0F':{'en':':woman_kneeling_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniende_frau_nach_rechts_mittlere_hautfarbe:','es':':mujer_de_rodillas_hacia_la_derecha_tono_de_piel_medio:','fr':':femme_à_genoux_vers_la_droite_peau_légèrement_mate:','ja':':正座する女性_右向き_中間の肌色:','ko':':오른쪽을_향한_무릎을_꿇은_여자_갈색_피부:','pt':':mulher_ajoelhando_de_frente_para_a_direita_pele_morena:','it':':donna_inginocchiata_a_destra_carnagione_olivastra:','fa':':زن_زانوزده_به_سمت_راست_پوست_طلایی:','id':':wanita_berlutut_menghadap_kanan_warna_kulit_sedang:','zh':':跪下的女人面向右边_中等肤色:','ru':':женщина_на_коленях_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_diz_çöken_kadın_orta_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين_بلون_معتدل:'},'\U0001F9CE\U0001F3FD\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1':{'en':':woman_kneeling_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniende_frau_nach_rechts_mittlere_hautfarbe:','es':':mujer_de_rodillas_hacia_la_derecha_tono_de_piel_medio:','fr':':femme_à_genoux_vers_la_droite_peau_légèrement_mate:','ja':':正座する女性_右向き_中間の肌色:','ko':':오른쪽을_향한_무릎을_꿇은_여자_갈색_피부:','pt':':mulher_ajoelhando_de_frente_para_a_direita_pele_morena:','it':':donna_inginocchiata_a_destra_carnagione_olivastra:','fa':':زن_زانوزده_به_سمت_راست_پوست_طلایی:','id':':wanita_berlutut_menghadap_kanan_warna_kulit_sedang:','zh':':跪下的女人面向右边_中等肤色:','ru':':женщина_на_коленях_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_diz_çöken_kadın_orta_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين_بلون_معتدل:'},'\U0001F9CE\U0001F3FD\U0000200D\U00002640\U0000200D\U000027A1':{'en':':woman_kneeling_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':kniende_frau_nach_rechts_mittlere_hautfarbe:','es':':mujer_de_rodillas_hacia_la_derecha_tono_de_piel_medio:','fr':':femme_à_genoux_vers_la_droite_peau_légèrement_mate:','ja':':正座する女性_右向き_中間の肌色:','ko':':오른쪽을_향한_무릎을_꿇은_여자_갈색_피부:','pt':':mulher_ajoelhando_de_frente_para_a_direita_pele_morena:','it':':donna_inginocchiata_a_destra_carnagione_olivastra:','fa':':زن_زانوزده_به_سمت_راست_پوست_طلایی:','id':':wanita_berlutut_menghadap_kanan_warna_kulit_sedang:','zh':':跪下的女人面向右边_中等肤色:','ru':':женщина_на_коленях_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_diz_çöken_kadın_orta_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_متجه_لليمين_بلون_معتدل:'},'\U0001F9CE\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_kneeling_light_skin_tone:','status':fully_qualified,'E':12,'de':':kniende_frau_helle_hautfarbe:','es':':mujer_de_rodillas_tono_de_piel_claro:','fr':':femme_à_genoux_peau_claire:','ja':':正座する女性_薄い肌色:','ko':':무릎을_꿇은_여자_하얀_피부:','pt':':mulher_ajoelhando_pele_clara:','it':':donna_inginocchiata_carnagione_chiara:','fa':':زن_زانوزده_پوست_سفید:','id':':wanita_berlutut_warna_kulit_cerah:','zh':':跪下的女人_较浅肤色:','ru':':женщина_на_коленях_очень_светлыи_тон_кожи:','tr':':diz_çöken_kadın_açık_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_بشرة_بلون_فاتح:'},'\U0001F9CE\U0001F3FB\U0000200D\U00002640':{'en':':woman_kneeling_light_skin_tone:','status':minimally_qualified,'E':12,'de':':kniende_frau_helle_hautfarbe:','es':':mujer_de_rodillas_tono_de_piel_claro:','fr':':femme_à_genoux_peau_claire:','ja':':正座する女性_薄い肌色:','ko':':무릎을_꿇은_여자_하얀_피부:','pt':':mulher_ajoelhando_pele_clara:','it':':donna_inginocchiata_carnagione_chiara:','fa':':زن_زانوزده_پوست_سفید:','id':':wanita_berlutut_warna_kulit_cerah:','zh':':跪下的女人_较浅肤色:','ru':':женщина_на_коленях_очень_светлый_тон_кожи:','tr':':diz_çöken_kadın_açık_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_بشرة_بلون_فاتح:'},'\U0001F9CE\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_kneeling_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':kniende_frau_mitteldunkle_hautfarbe:','es':':mujer_de_rodillas_tono_de_piel_oscuro_medio:','fr':':femme_à_genoux_peau_mate:','ja':':正座する女性_やや濃い肌色:','ko':':무릎을_꿇은_여자_진한_갈색_피부:','pt':':mulher_ajoelhando_pele_morena_escura:','it':':donna_inginocchiata_carnagione_abbastanza_scura:','fa':':زن_زانوزده_پوست_گندمی:','id':':wanita_berlutut_warna_kulit_gelap-sedang:','zh':':跪下的女人_中等深肤色:','ru':':женщина_на_коленях_темныи_тон_кожи:','tr':':diz_çöken_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9CE\U0001F3FE\U0000200D\U00002640':{'en':':woman_kneeling_medium-dark_skin_tone:','status':minimally_qualified,'E':12,'de':':kniende_frau_mitteldunkle_hautfarbe:','es':':mujer_de_rodillas_tono_de_piel_oscuro_medio:','fr':':femme_à_genoux_peau_mate:','ja':':正座する女性_やや濃い肌色:','ko':':무릎을_꿇은_여자_진한_갈색_피부:','pt':':mulher_ajoelhando_pele_morena_escura:','it':':donna_inginocchiata_carnagione_abbastanza_scura:','fa':':زن_زانوزده_پوست_گندمی:','id':':wanita_berlutut_warna_kulit_gelap-sedang:','zh':':跪下的女人_中等深肤色:','ru':':женщина_на_коленях_темный_тон_кожи:','tr':':diz_çöken_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9CE\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_kneeling_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':kniende_frau_mittelhelle_hautfarbe:','es':':mujer_de_rodillas_tono_de_piel_claro_medio:','fr':':femme_à_genoux_peau_moyennement_claire:','ja':':正座する女性_やや薄い肌色:','ko':':무릎을_꿇은_여자_연한_갈색_피부:','pt':':mulher_ajoelhando_pele_morena_clara:','it':':donna_inginocchiata_carnagione_abbastanza_chiara:','fa':':زن_زانوزده_پوست_روشن:','id':':wanita_berlutut_warna_kulit_cerah-sedang:','zh':':跪下的女人_中等浅肤色:','ru':':женщина_на_коленях_светлыи_тон_кожи:','tr':':diz_çöken_kadın_orta_açık_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9CE\U0001F3FC\U0000200D\U00002640':{'en':':woman_kneeling_medium-light_skin_tone:','status':minimally_qualified,'E':12,'de':':kniende_frau_mittelhelle_hautfarbe:','es':':mujer_de_rodillas_tono_de_piel_claro_medio:','fr':':femme_à_genoux_peau_moyennement_claire:','ja':':正座する女性_やや薄い肌色:','ko':':무릎을_꿇은_여자_연한_갈색_피부:','pt':':mulher_ajoelhando_pele_morena_clara:','it':':donna_inginocchiata_carnagione_abbastanza_chiara:','fa':':زن_زانوزده_پوست_روشن:','id':':wanita_berlutut_warna_kulit_cerah-sedang:','zh':':跪下的女人_中等浅肤色:','ru':':женщина_на_коленях_светлый_тон_кожи:','tr':':diz_çöken_kadın_orta_açık_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9CE\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_kneeling_medium_skin_tone:','status':fully_qualified,'E':12,'de':':kniende_frau_mittlere_hautfarbe:','es':':mujer_de_rodillas_tono_de_piel_medio:','fr':':femme_à_genoux_peau_légèrement_mate:','ja':':正座する女性_中間の肌色:','ko':':무릎을_꿇은_여자_갈색_피부:','pt':':mulher_ajoelhando_pele_morena:','it':':donna_inginocchiata_carnagione_olivastra:','fa':':زن_زانوزده_پوست_طلایی:','id':':wanita_berlutut_warna_kulit_sedang:','zh':':跪下的女人_中等肤色:','ru':':женщина_на_коленях_среднии_тон_кожи:','tr':':diz_çöken_kadın_orta_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_بشرة_بلون_معتدل:'},'\U0001F9CE\U0001F3FD\U0000200D\U00002640':{'en':':woman_kneeling_medium_skin_tone:','status':minimally_qualified,'E':12,'de':':kniende_frau_mittlere_hautfarbe:','es':':mujer_de_rodillas_tono_de_piel_medio:','fr':':femme_à_genoux_peau_légèrement_mate:','ja':':正座する女性_中間の肌色:','ko':':무릎을_꿇은_여자_갈색_피부:','pt':':mulher_ajoelhando_pele_morena:','it':':donna_inginocchiata_carnagione_olivastra:','fa':':زن_زانوزده_پوست_طلایی:','id':':wanita_berlutut_warna_kulit_sedang:','zh':':跪下的女人_中等肤色:','ru':':женщина_на_коленях_средний_тон_кожи:','tr':':diz_çöken_kadın_orta_cilt_tonu:','ar':':سيدة_جالسة_على_ركبتيها_بشرة_بلون_معتدل:'},'\U0001F3CB\U0000FE0F\U0000200D\U00002640\U0000FE0F':{'en':':woman_lifting_weights:','status':fully_qualified,'E':4,'alias':[':weight_lifting_woman:'],'de':':gewichtheberin:','es':':mujer_levantando_pesas:','fr':':femme_haltérophile:','ja':':重量挙げをする女:','ko':':여자_역도_선수:','pt':':mulher_levantando_peso:','it':':donna_che_solleva_pesi:','fa':':وزنه_بردار_زن:','id':':wanita_mengangkat_beban:','zh':':女生举重:','ru':':женщина_со_штангои:','tr':':ağırlık_kaldıran_kadın:','ar':':رافعة_أثقال:'},'\U0001F3CB\U0000200D\U00002640\U0000FE0F':{'en':':woman_lifting_weights:','status':unqualified,'E':4,'alias':[':weight_lifting_woman:'],'de':':gewichtheberin:','es':':mujer_levantando_pesas:','fr':':femme_haltérophile:','ja':':重量挙げをする女:','ko':':여자_역도_선수:','pt':':mulher_levantando_peso:','it':':donna_che_solleva_pesi:','fa':':وزنه_بردار_زن:','id':':wanita_mengangkat_beban:','zh':':女生举重:','ru':':женщина_со_штангои:','tr':':ağırlık_kaldıran_kadın:','ar':':رافعة_أثقال:'},'\U0001F3CB\U0000FE0F\U0000200D\U00002640':{'en':':woman_lifting_weights:','status':minimally_qualified,'E':4,'alias':[':weight_lifting_woman:'],'de':':gewichtheberin:','es':':mujer_levantando_pesas:','fr':':femme_haltérophile:','ja':':重量挙げをする女:','ko':':여자_역도_선수:','pt':':mulher_levantando_peso:','it':':donna_che_solleva_pesi:','fa':':وزنه_بردار_زن:','id':':wanita_mengangkat_beban:','zh':':女生举重:','ru':':женщина_со_штангои:','tr':':ağırlık_kaldıran_kadın:','ar':':رافعة_أثقال:'},'\U0001F3CB\U0000200D\U00002640':{'en':':woman_lifting_weights:','status':unqualified,'E':4,'alias':[':weight_lifting_woman:'],'de':':gewichtheberin:','es':':mujer_levantando_pesas:','fr':':femme_haltérophile:','ja':':重量挙げをする女:','ko':':여자_역도_선수:','pt':':mulher_levantando_peso:','it':':donna_che_solleva_pesi:','fa':':وزنه_بردار_زن:','id':':wanita_mengangkat_beban:','zh':':女生举重:','ru':':женщина_со_штангой:','tr':':ağırlık_kaldıran_kadın:','ar':':رافعة_أثقال:'},'\U0001F3CB\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_lifting_weights_dark_skin_tone:','status':fully_qualified,'E':4,'de':':gewichtheberin_dunkle_hautfarbe:','es':':mujer_levantando_pesas_tono_de_piel_oscuro:','fr':':femme_haltérophile_peau_foncée:','ja':':重量挙げをする女_濃い肌色:','ko':':여자_역도_선수_검은색_피부:','pt':':mulher_levantando_peso_pele_escura:','it':':donna_che_solleva_pesi_carnagione_scura:','fa':':وزنه_بردار_زن_پوست_آبنوسی:','id':':wanita_mengangkat_beban_warna_kulit_gelap:','zh':':女生举重_较深肤色:','ru':':женщина_со_штангои_очень_темныи_тон_кожи:','tr':':ağırlık_kaldıran_kadın_koyu_cilt_tonu:','ar':':رافعة_أثقال_بشرة_بلون_غامق:'},'\U0001F3CB\U0001F3FF\U0000200D\U00002640':{'en':':woman_lifting_weights_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':gewichtheberin_dunkle_hautfarbe:','es':':mujer_levantando_pesas_tono_de_piel_oscuro:','fr':':femme_haltérophile_peau_foncée:','ja':':重量挙げをする女_濃い肌色:','ko':':여자_역도_선수_검은색_피부:','pt':':mulher_levantando_peso_pele_escura:','it':':donna_che_solleva_pesi_carnagione_scura:','fa':':وزنه_بردار_زن_پوست_آبنوسی:','id':':wanita_mengangkat_beban_warna_kulit_gelap:','zh':':女生举重_较深肤色:','ru':':женщина_со_штангой_очень_темный_тон_кожи:','tr':':ağırlık_kaldıran_kadın_koyu_cilt_tonu:','ar':':رافعة_أثقال_بشرة_بلون_غامق:'},'\U0001F3CB\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_lifting_weights_light_skin_tone:','status':fully_qualified,'E':4,'de':':gewichtheberin_helle_hautfarbe:','es':':mujer_levantando_pesas_tono_de_piel_claro:','fr':':femme_haltérophile_peau_claire:','ja':':重量挙げをする女_薄い肌色:','ko':':여자_역도_선수_하얀_피부:','pt':':mulher_levantando_peso_pele_clara:','it':':donna_che_solleva_pesi_carnagione_chiara:','fa':':وزنه_بردار_زن_پوست_سفید:','id':':wanita_mengangkat_beban_warna_kulit_cerah:','zh':':女生举重_较浅肤色:','ru':':женщина_со_штангои_очень_светлыи_тон_кожи:','tr':':ağırlık_kaldıran_kadın_açık_cilt_tonu:','ar':':رافعة_أثقال_بشرة_بلون_فاتح:'},'\U0001F3CB\U0001F3FB\U0000200D\U00002640':{'en':':woman_lifting_weights_light_skin_tone:','status':minimally_qualified,'E':4,'de':':gewichtheberin_helle_hautfarbe:','es':':mujer_levantando_pesas_tono_de_piel_claro:','fr':':femme_haltérophile_peau_claire:','ja':':重量挙げをする女_薄い肌色:','ko':':여자_역도_선수_하얀_피부:','pt':':mulher_levantando_peso_pele_clara:','it':':donna_che_solleva_pesi_carnagione_chiara:','fa':':وزنه_بردار_زن_پوست_سفید:','id':':wanita_mengangkat_beban_warna_kulit_cerah:','zh':':女生举重_较浅肤色:','ru':':женщина_со_штангой_очень_светлый_тон_кожи:','tr':':ağırlık_kaldıran_kadın_açık_cilt_tonu:','ar':':رافعة_أثقال_بشرة_بلون_فاتح:'},'\U0001F3CB\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_lifting_weights_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':gewichtheberin_mitteldunkle_hautfarbe:','es':':mujer_levantando_pesas_tono_de_piel_oscuro_medio:','fr':':femme_haltérophile_peau_mate:','ja':':重量挙げをする女_やや濃い肌色:','ko':':여자_역도_선수_진한_갈색_피부:','pt':':mulher_levantando_peso_pele_morena_escura:','it':':donna_che_solleva_pesi_carnagione_abbastanza_scura:','fa':':وزنه_بردار_زن_پوست_گندمی:','id':':wanita_mengangkat_beban_warna_kulit_gelap-sedang:','zh':':女生举重_中等深肤色:','ru':':женщина_со_штангои_темныи_тон_кожи:','tr':':ağırlık_kaldıran_kadın_orta_koyu_cilt_tonu:','ar':':رافعة_أثقال_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3CB\U0001F3FE\U0000200D\U00002640':{'en':':woman_lifting_weights_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':gewichtheberin_mitteldunkle_hautfarbe:','es':':mujer_levantando_pesas_tono_de_piel_oscuro_medio:','fr':':femme_haltérophile_peau_mate:','ja':':重量挙げをする女_やや濃い肌色:','ko':':여자_역도_선수_진한_갈색_피부:','pt':':mulher_levantando_peso_pele_morena_escura:','it':':donna_che_solleva_pesi_carnagione_abbastanza_scura:','fa':':وزنه_بردار_زن_پوست_گندمی:','id':':wanita_mengangkat_beban_warna_kulit_gelap-sedang:','zh':':女生举重_中等深肤色:','ru':':женщина_со_штангой_темный_тон_кожи:','tr':':ağırlık_kaldıran_kadın_orta_koyu_cilt_tonu:','ar':':رافعة_أثقال_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3CB\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_lifting_weights_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':gewichtheberin_mittelhelle_hautfarbe:','es':':mujer_levantando_pesas_tono_de_piel_claro_medio:','fr':':femme_haltérophile_peau_moyennement_claire:','ja':':重量挙げをする女_やや薄い肌色:','ko':':여자_역도_선수_연한_갈색_피부:','pt':':mulher_levantando_peso_pele_morena_clara:','it':':donna_che_solleva_pesi_carnagione_abbastanza_chiara:','fa':':وزنه_بردار_زن_پوست_روشن:','id':':wanita_mengangkat_beban_warna_kulit_cerah-sedang:','zh':':女生举重_中等浅肤色:','ru':':женщина_со_штангои_светлыи_тон_кожи:','tr':':ağırlık_kaldıran_kadın_orta_açık_cilt_tonu:','ar':':رافعة_أثقال_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3CB\U0001F3FC\U0000200D\U00002640':{'en':':woman_lifting_weights_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':gewichtheberin_mittelhelle_hautfarbe:','es':':mujer_levantando_pesas_tono_de_piel_claro_medio:','fr':':femme_haltérophile_peau_moyennement_claire:','ja':':重量挙げをする女_やや薄い肌色:','ko':':여자_역도_선수_연한_갈색_피부:','pt':':mulher_levantando_peso_pele_morena_clara:','it':':donna_che_solleva_pesi_carnagione_abbastanza_chiara:','fa':':وزنه_بردار_زن_پوست_روشن:','id':':wanita_mengangkat_beban_warna_kulit_cerah-sedang:','zh':':女生举重_中等浅肤色:','ru':':женщина_со_штангой_светлый_тон_кожи:','tr':':ağırlık_kaldıran_kadın_orta_açık_cilt_tonu:','ar':':رافعة_أثقال_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3CB\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_lifting_weights_medium_skin_tone:','status':fully_qualified,'E':4,'de':':gewichtheberin_mittlere_hautfarbe:','es':':mujer_levantando_pesas_tono_de_piel_medio:','fr':':femme_haltérophile_peau_légèrement_mate:','ja':':重量挙げをする女_中間の肌色:','ko':':여자_역도_선수_갈색_피부:','pt':':mulher_levantando_peso_pele_morena:','it':':donna_che_solleva_pesi_carnagione_olivastra:','fa':':وزنه_بردار_زن_پوست_طلایی:','id':':wanita_mengangkat_beban_warna_kulit_sedang:','zh':':女生举重_中等肤色:','ru':':женщина_со_штангои_среднии_тон_кожи:','tr':':ağırlık_kaldıran_kadın_orta_cilt_tonu:','ar':':رافعة_أثقال_بشرة_بلون_معتدل:'},'\U0001F3CB\U0001F3FD\U0000200D\U00002640':{'en':':woman_lifting_weights_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':gewichtheberin_mittlere_hautfarbe:','es':':mujer_levantando_pesas_tono_de_piel_medio:','fr':':femme_haltérophile_peau_légèrement_mate:','ja':':重量挙げをする女_中間の肌色:','ko':':여자_역도_선수_갈색_피부:','pt':':mulher_levantando_peso_pele_morena:','it':':donna_che_solleva_pesi_carnagione_olivastra:','fa':':وزنه_بردار_زن_پوست_طلایی:','id':':wanita_mengangkat_beban_warna_kulit_sedang:','zh':':女生举重_中等肤色:','ru':':женщина_со_штангой_средний_тон_кожи:','tr':':ağırlık_kaldıran_kadın_orta_cilt_tonu:','ar':':رافعة_أثقال_بشرة_بلون_معتدل:'},'\U0001F469\U0001F3FB':{'en':':woman_light_skin_tone:','status':fully_qualified,'E':1,'de':':frau_helle_hautfarbe:','es':':mujer_tono_de_piel_claro:','fr':':femme_peau_claire:','ja':':女性_薄い肌色:','ko':':여자_하얀_피부:','pt':':mulher_pele_clara:','it':':donna_carnagione_chiara:','fa':':زن_پوست_سفید:','id':':wanita_warna_kulit_cerah:','zh':':女人_较浅肤色:','ru':':женщина_очень_светлый_тон_кожи:','tr':':kadın_açık_cilt_tonu:','ar':':سيدة_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FB\U0000200D\U0001F9B2':{'en':':woman_light_skin_tone_bald:','status':fully_qualified,'E':11,'de':':frau_helle_hautfarbe_glatze:','es':':mujer_tono_de_piel_claro_y_sin_pelo:','fr':':femme_peau_claire_et_chauve:','ja':':女性_薄い肌色_はげ頭:','ko':':여자_하얀_피부_대머리:','pt':':mulher_pele_clara_e_careca:','it':':donna_carnagione_chiara_e_calvo:','fa':':زن_پوست_سفید_تاس:','id':':wanita_warna_kulit_cerah_gundul:','zh':':女人_较浅肤色秃顶:','ru':':женщина_очень_светлый_тон_кожи_без_волос:','tr':':kadın_açık_cilt_tonu_kel:','ar':':سيدة_بشرة_بلون_فاتح_وأصلع:'},'\U0001F9D4\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_light_skin_tone_beard:','status':fully_qualified,'E':13.1,'de':':frau_helle_hautfarbe_bart:','es':':mujer_tono_de_piel_claro_y_barba:','fr':':femme_barbue_peau_claire:','ja':':あごひげの女性_薄い肌色:','ko':':수염_난_여자_하얀_피부:','pt':':mulher_pele_clara_e_barba:','it':':donna_con_la_barba_carnagione_chiara:','fa':':زن_پوست_سفید_ریش:','id':':wanita_warna_kulit_cerah_janggut:','zh':':有络腮胡子的女人_较浅肤色:','ru':':бородатая_женщина_очень_светлыи_тон_кожи:','tr':':kadın_açık_cilt_tonu_sakal:','ar':':امرأة_بلحية_بشرة_بلون_فاتح:'},'\U0001F9D4\U0001F3FB\U0000200D\U00002640':{'en':':woman_light_skin_tone_beard:','status':minimally_qualified,'E':13.1,'de':':frau_helle_hautfarbe_bart:','es':':mujer_tono_de_piel_claro_y_barba:','fr':':femme_barbue_peau_claire:','ja':':あごひげの女性_薄い肌色:','ko':':수염_난_여자_하얀_피부:','pt':':mulher_pele_clara_e_barba:','it':':donna_con_la_barba_carnagione_chiara:','fa':':زن_پوست_سفید_ریش:','id':':wanita_warna_kulit_cerah_janggut:','zh':':有络腮胡子的女人_较浅肤色:','ru':':бородатая_женщина_очень_светлый_тон_кожи:','tr':':kadın_açık_cilt_tonu_sakal:','ar':':امرأة_بلحية_بشرة_بلون_فاتح:'},'\U0001F471\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_light_skin_tone_blond_hair:','status':fully_qualified,'E':4,'de':':frau_helle_hautfarbe,_blond:','es':':mujer_rubia_tono_de_piel_claro:','fr':':femme_blonde_peau_claire:','ja':':金髪の女性_薄い肌色:','ko':':금발_여자_하얀_피부:','pt':':mulher_pele_clara_e_cabelo_loiro:','it':':donna_bionda_carnagione_chiara:','fa':':زن_پوست_سفید_موبور:','id':':wanita_berambut_pirang_warna_kulit_cerah:','zh':':金发女_较浅肤色:','ru':':блондинка_очень_светлыи_тон_кожи:','tr':':sarışın_kadın_açık_cilt_tonu:','ar':':امرأة_شقراء_بشرة_بلون_فاتح:'},'\U0001F471\U0001F3FB\U0000200D\U00002640':{'en':':woman_light_skin_tone_blond_hair:','status':minimally_qualified,'E':4,'de':':frau_helle_hautfarbe_blond:','es':':mujer_rubia_tono_de_piel_claro:','fr':':femme_blonde_peau_claire:','ja':':金髪の女性_薄い肌色:','ko':':금발_여자_하얀_피부:','pt':':mulher_pele_clara_e_cabelo_loiro:','it':':donna_bionda_carnagione_chiara:','fa':':زن_پوست_سفید_موبور:','id':':wanita_berambut_pirang_warna_kulit_cerah:','zh':':金发女_较浅肤色:','ru':':блондинка_очень_светлый_тон_кожи:','tr':':sarışın_kadın_açık_cilt_tonu:','ar':':امرأة_شقراء_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FB\U0000200D\U0001F9B1':{'en':':woman_light_skin_tone_curly_hair:','status':fully_qualified,'E':11,'de':':frau_helle_hautfarbe_lockiges_haar:','es':':mujer_tono_de_piel_claro_y_pelo_rizado:','fr':':femme_peau_claire_et_cheveux_bouclés:','ja':':女性_薄い肌色_巻き毛:','ko':':여자_하얀_피부_곱슬머리:','pt':':mulher_pele_clara_e_cabelo_cacheado:','it':':donna_carnagione_chiara_e_capelli_ricci:','fa':':زن_پوست_سفید_موفرفری:','id':':wanita_warna_kulit_cerah_rambut_keriting:','zh':':女人_较浅肤色卷发:','ru':':женщина_очень_светлый_тон_кожи_кудрявые_волосы:','tr':':kadın_açık_cilt_tonu_kıvırcık_saç:','ar':':سيدة_بشرة_بلون_فاتح_وشعر_مجعد:'},'\U0001F469\U0001F3FB\U0000200D\U0001F9B0':{'en':':woman_light_skin_tone_red_hair:','status':fully_qualified,'E':11,'de':':frau_helle_hautfarbe_rotes_haar:','es':':mujer_tono_de_piel_claro_y_pelo_pelirrojo:','fr':':femme_peau_claire_et_cheveux_roux:','ja':':女性_薄い肌色_赤毛:','ko':':여자_하얀_피부_빨간_머리:','pt':':mulher_pele_clara_e_cabelo_vermelho:','it':':donna_carnagione_chiara_e_capelli_rossi:','fa':':زن_پوست_سفید_موقرمز:','id':':wanita_warna_kulit_cerah_rambut_merah:','zh':':女人_较浅肤色红发:','ru':':женщина_очень_светлый_тон_кожи_рыжие_волосы:','tr':':kadın_açık_cilt_tonu_kızıl_saç:','ar':':سيدة_بشرة_بلون_فاتح_وشعر_أحمر:'},'\U0001F469\U0001F3FB\U0000200D\U0001F9B3':{'en':':woman_light_skin_tone_white_hair:','status':fully_qualified,'E':11,'de':':frau_helle_hautfarbe_weißes_haar:','es':':mujer_tono_de_piel_claro_y_pelo_blanco:','fr':':femme_peau_claire_et_cheveux_blancs:','ja':':女性_薄い肌色_白髪:','ko':':여자_하얀_피부_백발:','pt':':mulher_pele_clara_e_cabelo_branco:','it':':donna_carnagione_chiara_e_capelli_bianchi:','fa':':زن_پوست_سفید_موسفید:','id':':wanita_warna_kulit_cerah_rambut_putih:','zh':':女人_较浅肤色白发:','ru':':женщина_очень_светлый_тон_кожи_седые_волосы:','tr':':kadın_açık_cilt_tonu_beyaz_saç:','ar':':سيدة_بشرة_بلون_فاتح_وشعر_أبيض:'},'\U0001F9D9\U0000200D\U00002640\U0000FE0F':{'en':':woman_mage:','status':fully_qualified,'E':5,'alias':[':mage_woman:'],'de':':magierin:','es':':maga:','fr':':mage_femme:','ja':':女の魔法使い:','ko':':여자_마법사:','pt':':maga:','it':':maga:','fa':':ساحر_زن:','id':':magi_wanita:','zh':':女法师:','ru':':волшебница:','tr':':kadın_büyücü:','ar':':ساحرة:'},'\U0001F9D9\U0000200D\U00002640':{'en':':woman_mage:','status':minimally_qualified,'E':5,'alias':[':mage_woman:'],'de':':magierin:','es':':maga:','fr':':mage_femme:','ja':':女の魔法使い:','ko':':여자_마법사:','pt':':maga:','it':':maga:','fa':':ساحر_زن:','id':':magi_wanita:','zh':':女法师:','ru':':волшебница:','tr':':kadın_büyücü:','ar':':ساحرة:'},'\U0001F9D9\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_mage_dark_skin_tone:','status':fully_qualified,'E':5,'de':':magierin_dunkle_hautfarbe:','es':':maga_tono_de_piel_oscuro:','fr':':mage_femme_peau_foncée:','ja':':女の魔法使い_濃い肌色:','ko':':여자_마법사_검은색_피부:','pt':':maga_pele_escura:','it':':maga_carnagione_scura:','fa':':ساحر_زن_پوست_آبنوسی:','id':':magi_wanita_warna_kulit_gelap:','zh':':女法师_较深肤色:','ru':':волшебница_очень_темныи_тон_кожи:','tr':':kadın_büyücü_koyu_cilt_tonu:','ar':':ساحرة_بشرة_بلون_غامق:'},'\U0001F9D9\U0001F3FF\U0000200D\U00002640':{'en':':woman_mage_dark_skin_tone:','status':minimally_qualified,'E':5,'de':':magierin_dunkle_hautfarbe:','es':':maga_tono_de_piel_oscuro:','fr':':mage_femme_peau_foncée:','ja':':女の魔法使い_濃い肌色:','ko':':여자_마법사_검은색_피부:','pt':':maga_pele_escura:','it':':maga_carnagione_scura:','fa':':ساحر_زن_پوست_آبنوسی:','id':':magi_wanita_warna_kulit_gelap:','zh':':女法师_较深肤色:','ru':':волшебница_очень_темный_тон_кожи:','tr':':kadın_büyücü_koyu_cilt_tonu:','ar':':ساحرة_بشرة_بلون_غامق:'},'\U0001F9D9\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_mage_light_skin_tone:','status':fully_qualified,'E':5,'de':':magierin_helle_hautfarbe:','es':':maga_tono_de_piel_claro:','fr':':mage_femme_peau_claire:','ja':':女の魔法使い_薄い肌色:','ko':':여자_마법사_하얀_피부:','pt':':maga_pele_clara:','it':':maga_carnagione_chiara:','fa':':ساحر_زن_پوست_سفید:','id':':magi_wanita_warna_kulit_cerah:','zh':':女法师_较浅肤色:','ru':':волшебница_очень_светлыи_тон_кожи:','tr':':kadın_büyücü_açık_cilt_tonu:','ar':':ساحرة_بشرة_بلون_فاتح:'},'\U0001F9D9\U0001F3FB\U0000200D\U00002640':{'en':':woman_mage_light_skin_tone:','status':minimally_qualified,'E':5,'de':':magierin_helle_hautfarbe:','es':':maga_tono_de_piel_claro:','fr':':mage_femme_peau_claire:','ja':':女の魔法使い_薄い肌色:','ko':':여자_마법사_하얀_피부:','pt':':maga_pele_clara:','it':':maga_carnagione_chiara:','fa':':ساحر_زن_پوست_سفید:','id':':magi_wanita_warna_kulit_cerah:','zh':':女法师_较浅肤色:','ru':':волшебница_очень_светлый_тон_кожи:','tr':':kadın_büyücü_açık_cilt_tonu:','ar':':ساحرة_بشرة_بلون_فاتح:'},'\U0001F9D9\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_mage_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':magierin_mitteldunkle_hautfarbe:','es':':maga_tono_de_piel_oscuro_medio:','fr':':mage_femme_peau_mate:','ja':':女の魔法使い_やや濃い肌色:','ko':':여자_마법사_진한_갈색_피부:','pt':':maga_pele_morena_escura:','it':':maga_carnagione_abbastanza_scura:','fa':':ساحر_زن_پوست_گندمی:','id':':magi_wanita_warna_kulit_gelap-sedang:','zh':':女法师_中等深肤色:','ru':':волшебница_темныи_тон_кожи:','tr':':kadın_büyücü_orta_koyu_cilt_tonu:','ar':':ساحرة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D9\U0001F3FE\U0000200D\U00002640':{'en':':woman_mage_medium-dark_skin_tone:','status':minimally_qualified,'E':5,'de':':magierin_mitteldunkle_hautfarbe:','es':':maga_tono_de_piel_oscuro_medio:','fr':':mage_femme_peau_mate:','ja':':女の魔法使い_やや濃い肌色:','ko':':여자_마법사_진한_갈색_피부:','pt':':maga_pele_morena_escura:','it':':maga_carnagione_abbastanza_scura:','fa':':ساحر_زن_پوست_گندمی:','id':':magi_wanita_warna_kulit_gelap-sedang:','zh':':女法师_中等深肤色:','ru':':волшебница_темный_тон_кожи:','tr':':kadın_büyücü_orta_koyu_cilt_tonu:','ar':':ساحرة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D9\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_mage_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':magierin_mittelhelle_hautfarbe:','es':':maga_tono_de_piel_claro_medio:','fr':':mage_femme_peau_moyennement_claire:','ja':':女の魔法使い_やや薄い肌色:','ko':':여자_마법사_연한_갈색_피부:','pt':':maga_pele_morena_clara:','it':':maga_carnagione_abbastanza_chiara:','fa':':ساحر_زن_پوست_روشن:','id':':magi_wanita_warna_kulit_cerah-sedang:','zh':':女法师_中等浅肤色:','ru':':волшебница_светлыи_тон_кожи:','tr':':kadın_büyücü_orta_açık_cilt_tonu:','ar':':ساحرة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D9\U0001F3FC\U0000200D\U00002640':{'en':':woman_mage_medium-light_skin_tone:','status':minimally_qualified,'E':5,'de':':magierin_mittelhelle_hautfarbe:','es':':maga_tono_de_piel_claro_medio:','fr':':mage_femme_peau_moyennement_claire:','ja':':女の魔法使い_やや薄い肌色:','ko':':여자_마법사_연한_갈색_피부:','pt':':maga_pele_morena_clara:','it':':maga_carnagione_abbastanza_chiara:','fa':':ساحر_زن_پوست_روشن:','id':':magi_wanita_warna_kulit_cerah-sedang:','zh':':女法师_中等浅肤色:','ru':':волшебница_светлый_тон_кожи:','tr':':kadın_büyücü_orta_açık_cilt_tonu:','ar':':ساحرة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D9\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_mage_medium_skin_tone:','status':fully_qualified,'E':5,'de':':magierin_mittlere_hautfarbe:','es':':maga_tono_de_piel_medio:','fr':':mage_femme_peau_légèrement_mate:','ja':':女の魔法使い_中間の肌色:','ko':':여자_마법사_갈색_피부:','pt':':maga_pele_morena:','it':':maga_carnagione_olivastra:','fa':':ساحر_زن_پوست_طلایی:','id':':magi_wanita_warna_kulit_sedang:','zh':':女法师_中等肤色:','ru':':волшебница_среднии_тон_кожи:','tr':':kadın_büyücü_orta_cilt_tonu:','ar':':ساحرة_بشرة_بلون_معتدل:'},'\U0001F9D9\U0001F3FD\U0000200D\U00002640':{'en':':woman_mage_medium_skin_tone:','status':minimally_qualified,'E':5,'de':':magierin_mittlere_hautfarbe:','es':':maga_tono_de_piel_medio:','fr':':mage_femme_peau_légèrement_mate:','ja':':女の魔法使い_中間の肌色:','ko':':여자_마법사_갈색_피부:','pt':':maga_pele_morena:','it':':maga_carnagione_olivastra:','fa':':ساحر_زن_پوست_طلایی:','id':':magi_wanita_warna_kulit_sedang:','zh':':女法师_中等肤色:','ru':':волшебница_средний_тон_кожи:','tr':':kadın_büyücü_orta_cilt_tonu:','ar':':ساحرة_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U0001F527':{'en':':woman_mechanic:','status':fully_qualified,'E':4,'de':':mechanikerin:','es':':profesional_de_la_mecánica_mujer:','fr':':mécanicienne:','ja':':女性の整備士:','ko':':여자_정비공:','pt':':mecânica:','it':':meccanico_donna:','fa':':تعمیرکار_زن:','id':':montir_wanita:','zh':':女技工:','ru':':женщина_механик:','tr':':kadın_tamirci:','ar':':ميكانيكية:'},'\U0001F469\U0001F3FF\U0000200D\U0001F527':{'en':':woman_mechanic_dark_skin_tone:','status':fully_qualified,'E':4,'de':':mechanikerin_dunkle_hautfarbe:','es':':profesional_de_la_mecánica_mujer_tono_de_piel_oscuro:','fr':':mécanicienne_peau_foncée:','ja':':女性の整備士_濃い肌色:','ko':':여자_정비공_검은색_피부:','pt':':mecânica_pele_escura:','it':':meccanico_donna_carnagione_scura:','fa':':تعمیرکار_زن_پوست_آبنوسی:','id':':montir_wanita_warna_kulit_gelap:','zh':':女技工_较深肤色:','ru':':женщина_механик_очень_темный_тон_кожи:','tr':':kadın_tamirci_koyu_cilt_tonu:','ar':':ميكانيكية_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F527':{'en':':woman_mechanic_light_skin_tone:','status':fully_qualified,'E':4,'de':':mechanikerin_helle_hautfarbe:','es':':profesional_de_la_mecánica_mujer_tono_de_piel_claro:','fr':':mécanicienne_peau_claire:','ja':':女性の整備士_薄い肌色:','ko':':여자_정비공_하얀_피부:','pt':':mecânica_pele_clara:','it':':meccanico_donna_carnagione_chiara:','fa':':تعمیرکار_زن_پوست_سفید:','id':':montir_wanita_warna_kulit_cerah:','zh':':女技工_较浅肤色:','ru':':женщина_механик_очень_светлый_тон_кожи:','tr':':kadın_tamirci_açık_cilt_tonu:','ar':':ميكانيكية_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F527':{'en':':woman_mechanic_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':mechanikerin_mitteldunkle_hautfarbe:','es':':profesional_de_la_mecánica_mujer_tono_de_piel_oscuro_medio:','fr':':mécanicienne_peau_mate:','ja':':女性の整備士_やや濃い肌色:','ko':':여자_정비공_진한_갈색_피부:','pt':':mecânica_pele_morena_escura:','it':':meccanico_donna_carnagione_abbastanza_scura:','fa':':تعمیرکار_زن_پوست_گندمی:','id':':montir_wanita_warna_kulit_gelap-sedang:','zh':':女技工_中等深肤色:','ru':':женщина_механик_темный_тон_кожи:','tr':':kadın_tamirci_orta_koyu_cilt_tonu:','ar':':ميكانيكية_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F527':{'en':':woman_mechanic_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':mechanikerin_mittelhelle_hautfarbe:','es':':profesional_de_la_mecánica_mujer_tono_de_piel_claro_medio:','fr':':mécanicienne_peau_moyennement_claire:','ja':':女性の整備士_やや薄い肌色:','ko':':여자_정비공_연한_갈색_피부:','pt':':mecânica_pele_morena_clara:','it':':meccanico_donna_carnagione_abbastanza_chiara:','fa':':تعمیرکار_زن_پوست_روشن:','id':':montir_wanita_warna_kulit_cerah-sedang:','zh':':女技工_中等浅肤色:','ru':':женщина_механик_светлый_тон_кожи:','tr':':kadın_tamirci_orta_açık_cilt_tonu:','ar':':ميكانيكية_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F527':{'en':':woman_mechanic_medium_skin_tone:','status':fully_qualified,'E':4,'de':':mechanikerin_mittlere_hautfarbe:','es':':profesional_de_la_mecánica_mujer_tono_de_piel_medio:','fr':':mécanicienne_peau_légèrement_mate:','ja':':女性の整備士_中間の肌色:','ko':':여자_정비공_갈색_피부:','pt':':mecânica_pele_morena:','it':':meccanico_donna_carnagione_olivastra:','fa':':تعمیرکار_زن_پوست_طلایی:','id':':montir_wanita_warna_kulit_sedang:','zh':':女技工_中等肤色:','ru':':женщина_механик_средний_тон_кожи:','tr':':kadın_tamirci_orta_cilt_tonu:','ar':':ميكانيكية_بشرة_بلون_معتدل:'},'\U0001F469\U0001F3FE':{'en':':woman_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':frau_mitteldunkle_hautfarbe:','es':':mujer_tono_de_piel_oscuro_medio:','fr':':femme_peau_mate:','ja':':女性_やや濃い肌色:','ko':':여자_진한_갈색_피부:','pt':':mulher_pele_morena_escura:','it':':donna_carnagione_abbastanza_scura:','fa':':زن_پوست_گندمی:','id':':wanita_warna_kulit_gelap-sedang:','zh':':女人_中等深肤色:','ru':':женщина_темный_тон_кожи:','tr':':kadın_orta_koyu_cilt_tonu:','ar':':سيدة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FE\U0000200D\U0001F9B2':{'en':':woman_medium-dark_skin_tone_bald:','status':fully_qualified,'E':11,'de':':frau_mitteldunkle_hautfarbe_glatze:','es':':mujer_tono_de_piel_oscuro_medio_y_sin_pelo:','fr':':femme_peau_mate_et_chauve:','ja':':女性_やや濃い肌色_はげ頭:','ko':':여자_진한_갈색_피부_대머리:','pt':':mulher_pele_morena_escura_e_careca:','it':':donna_carnagione_abbastanza_scura_e_calvo:','fa':':زن_پوست_گندمی_تاس:','id':':wanita_warna_kulit_gelap-sedang_gundul:','zh':':女人_中等深肤色秃顶:','ru':':женщина_темный_тон_кожи_без_волос:','tr':':kadın_orta_koyu_cilt_tonu_kel:','ar':':سيدة_بشرة_بلون_معتدل_مائل_للغامق_وأصلع:'},'\U0001F9D4\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_medium-dark_skin_tone_beard:','status':fully_qualified,'E':13.1,'de':':frau_mitteldunkle_hautfarbe_bart:','es':':mujer_tono_de_piel_oscuro_medio_y_barba:','fr':':femme_barbue_peau_mate:','ja':':あごひげの女性_やや濃い肌色:','ko':':수염_난_여자_진한_갈색_피부:','pt':':mulher_pele_morena_escura_e_barba:','it':':donna_con_la_barba_carnagione_abbastanza_scura:','fa':':زن_پوست_گندمی_ریش:','id':':wanita_warna_kulit_gelap-sedang_janggut:','zh':':有络腮胡子的女人_中等深肤色:','ru':':бородатая_женщина_темныи_тон_кожи:','tr':':kadın_orta_koyu_cilt_tonu_sakal:','ar':':امرأة_بلحية_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D4\U0001F3FE\U0000200D\U00002640':{'en':':woman_medium-dark_skin_tone_beard:','status':minimally_qualified,'E':13.1,'de':':frau_mitteldunkle_hautfarbe_bart:','es':':mujer_tono_de_piel_oscuro_medio_y_barba:','fr':':femme_barbue_peau_mate:','ja':':あごひげの女性_やや濃い肌色:','ko':':수염_난_여자_진한_갈색_피부:','pt':':mulher_pele_morena_escura_e_barba:','it':':donna_con_la_barba_carnagione_abbastanza_scura:','fa':':زن_پوست_گندمی_ریش:','id':':wanita_warna_kulit_gelap-sedang_janggut:','zh':':有络腮胡子的女人_中等深肤色:','ru':':бородатая_женщина_темный_тон_кожи:','tr':':kadın_orta_koyu_cilt_tonu_sakal:','ar':':امرأة_بلحية_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F471\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_medium-dark_skin_tone_blond_hair:','status':fully_qualified,'E':4,'de':':frau_mitteldunkle_hautfarbe,_blond:','es':':mujer_rubia_tono_de_piel_oscuro_medio:','fr':':femme_blonde_peau_mate:','ja':':金髪の女性_やや濃い肌色:','ko':':금발_여자_진한_갈색_피부:','pt':':mulher_pele_morena_escura_e_cabelo_loiro:','it':':donna_bionda_carnagione_abbastanza_scura:','fa':':زن_پوست_گندمی_موبور:','id':':wanita_berambut_pirang_warna_kulit_gelap-sedang:','zh':':金发女_中等深肤色:','ru':':блондинка_темныи_тон_кожи:','tr':':sarışın_kadın_orta_koyu_cilt_tonu:','ar':':امرأة_شقراء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F471\U0001F3FE\U0000200D\U00002640':{'en':':woman_medium-dark_skin_tone_blond_hair:','status':minimally_qualified,'E':4,'de':':frau_mitteldunkle_hautfarbe_blond:','es':':mujer_rubia_tono_de_piel_oscuro_medio:','fr':':femme_blonde_peau_mate:','ja':':金髪の女性_やや濃い肌色:','ko':':금발_여자_진한_갈색_피부:','pt':':mulher_pele_morena_escura_e_cabelo_loiro:','it':':donna_bionda_carnagione_abbastanza_scura:','fa':':زن_پوست_گندمی_موبور:','id':':wanita_berambut_pirang_warna_kulit_gelap-sedang:','zh':':金发女_中等深肤色:','ru':':блондинка_темный_тон_кожи:','tr':':sarışın_kadın_orta_koyu_cilt_tonu:','ar':':امرأة_شقراء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FE\U0000200D\U0001F9B1':{'en':':woman_medium-dark_skin_tone_curly_hair:','status':fully_qualified,'E':11,'de':':frau_mitteldunkle_hautfarbe_lockiges_haar:','es':':mujer_tono_de_piel_oscuro_medio_y_pelo_rizado:','fr':':femme_peau_mate_et_cheveux_bouclés:','ja':':女性_やや濃い肌色_巻き毛:','ko':':여자_진한_갈색_피부_곱슬머리:','pt':':mulher_pele_morena_escura_e_cabelo_cacheado:','it':':donna_carnagione_abbastanza_scura_e_capelli_ricci:','fa':':زن_پوست_گندمی_موفرفری:','id':':wanita_warna_kulit_gelap-sedang_rambut_keriting:','zh':':女人_中等深肤色卷发:','ru':':женщина_темный_тон_кожи_кудрявые_волосы:','tr':':kadın_orta_koyu_cilt_tonu_kıvırcık_saç:','ar':':سيدة_بشرة_بلون_معتدل_مائل_للغامق_وشعر_مجعد:'},'\U0001F469\U0001F3FE\U0000200D\U0001F9B0':{'en':':woman_medium-dark_skin_tone_red_hair:','status':fully_qualified,'E':11,'de':':frau_mitteldunkle_hautfarbe_rotes_haar:','es':':mujer_tono_de_piel_oscuro_medio_y_pelo_pelirrojo:','fr':':femme_peau_mate_et_cheveux_roux:','ja':':女性_やや濃い肌色_赤毛:','ko':':여자_진한_갈색_피부_빨간_머리:','pt':':mulher_pele_morena_escura_e_cabelo_vermelho:','it':':donna_carnagione_abbastanza_scura_e_capelli_rossi:','fa':':زن_پوست_گندمی_موقرمز:','id':':wanita_warna_kulit_gelap-sedang_rambut_merah:','zh':':女人_中等深肤色红发:','ru':':женщина_темный_тон_кожи_рыжие_волосы:','tr':':kadın_orta_koyu_cilt_tonu_kızıl_saç:','ar':':سيدة_بشرة_بلون_معتدل_مائل_للغامق_وشعر_أحمر:'},'\U0001F469\U0001F3FE\U0000200D\U0001F9B3':{'en':':woman_medium-dark_skin_tone_white_hair:','status':fully_qualified,'E':11,'de':':frau_mitteldunkle_hautfarbe_weißes_haar:','es':':mujer_tono_de_piel_oscuro_medio_y_pelo_blanco:','fr':':femme_peau_mate_et_cheveux_blancs:','ja':':女性_やや濃い肌色_白髪:','ko':':여자_진한_갈색_피부_백발:','pt':':mulher_pele_morena_escura_e_cabelo_branco:','it':':donna_carnagione_abbastanza_scura_e_capelli_bianchi:','fa':':زن_پوست_گندمی_موسفید:','id':':wanita_warna_kulit_gelap-sedang_rambut_putih:','zh':':女人_中等深肤色白发:','ru':':женщина_темный_тон_кожи_седые_волосы:','tr':':kadın_orta_koyu_cilt_tonu_beyaz_saç:','ar':':سيدة_بشرة_بلون_معتدل_مائل_للغامق_وشعر_أبيض:'},'\U0001F469\U0001F3FC':{'en':':woman_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':frau_mittelhelle_hautfarbe:','es':':mujer_tono_de_piel_claro_medio:','fr':':femme_peau_moyennement_claire:','ja':':女性_やや薄い肌色:','ko':':여자_연한_갈색_피부:','pt':':mulher_pele_morena_clara:','it':':donna_carnagione_abbastanza_chiara:','fa':':زن_پوست_روشن:','id':':wanita_warna_kulit_cerah-sedang:','zh':':女人_中等浅肤色:','ru':':женщина_светлый_тон_кожи:','tr':':kadın_orta_açık_cilt_tonu:','ar':':سيدة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FC\U0000200D\U0001F9B2':{'en':':woman_medium-light_skin_tone_bald:','status':fully_qualified,'E':11,'de':':frau_mittelhelle_hautfarbe_glatze:','es':':mujer_tono_de_piel_claro_medio_y_sin_pelo:','fr':':femme_peau_moyennement_claire_et_chauve:','ja':':女性_やや薄い肌色_はげ頭:','ko':':여자_연한_갈색_피부_대머리:','pt':':mulher_pele_morena_clara_e_careca:','it':':donna_carnagione_abbastanza_chiara_e_calvo:','fa':':زن_پوست_روشن_تاس:','id':':wanita_warna_kulit_cerah-sedang_gundul:','zh':':女人_中等浅肤色秃顶:','ru':':женщина_светлый_тон_кожи_без_волос:','tr':':kadın_orta_açık_cilt_tonu_kel:','ar':':سيدة_بشرة_بلون_فاتح_ومعتدل_وأصلع:'},'\U0001F9D4\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_medium-light_skin_tone_beard:','status':fully_qualified,'E':13.1,'de':':frau_mittelhelle_hautfarbe_bart:','es':':mujer_tono_de_piel_claro_medio_y_barba:','fr':':femme_barbue_peau_moyennement_claire:','ja':':あごひげの女性_やや薄い肌色:','ko':':수염_난_여자_연한_갈색_피부:','pt':':mulher_pele_morena_clara_e_barba:','it':':donna_con_la_barba_carnagione_abbastanza_chiara:','fa':':زن_پوست_روشن_ریش:','id':':wanita_warna_kulit_cerah-sedang_janggut:','zh':':有络腮胡子的女人_中等浅肤色:','ru':':бородатая_женщина_светлыи_тон_кожи:','tr':':kadın_orta_açık_cilt_tonu_sakal:','ar':':امرأة_بلحية_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D4\U0001F3FC\U0000200D\U00002640':{'en':':woman_medium-light_skin_tone_beard:','status':minimally_qualified,'E':13.1,'de':':frau_mittelhelle_hautfarbe_bart:','es':':mujer_tono_de_piel_claro_medio_y_barba:','fr':':femme_barbue_peau_moyennement_claire:','ja':':あごひげの女性_やや薄い肌色:','ko':':수염_난_여자_연한_갈색_피부:','pt':':mulher_pele_morena_clara_e_barba:','it':':donna_con_la_barba_carnagione_abbastanza_chiara:','fa':':زن_پوست_روشن_ریش:','id':':wanita_warna_kulit_cerah-sedang_janggut:','zh':':有络腮胡子的女人_中等浅肤色:','ru':':бородатая_женщина_светлый_тон_кожи:','tr':':kadın_orta_açık_cilt_tonu_sakal:','ar':':امرأة_بلحية_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F471\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_medium-light_skin_tone_blond_hair:','status':fully_qualified,'E':4,'de':':frau_mittelhelle_hautfarbe,_blond:','es':':mujer_rubia_tono_de_piel_claro_medio:','fr':':femme_blonde_peau_moyennement_claire:','ja':':金髪の女性_やや薄い肌色:','ko':':금발_여자_연한_갈색_피부:','pt':':mulher_pele_morena_clara_e_cabelo_loiro:','it':':donna_bionda_carnagione_abbastanza_chiara:','fa':':زن_پوست_روشن_موبور:','id':':wanita_berambut_pirang_warna_kulit_cerah-sedang:','zh':':金发女_中等浅肤色:','ru':':блондинка_светлыи_тон_кожи:','tr':':sarışın_kadın_orta_açık_cilt_tonu:','ar':':امرأة_شقراء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F471\U0001F3FC\U0000200D\U00002640':{'en':':woman_medium-light_skin_tone_blond_hair:','status':minimally_qualified,'E':4,'de':':frau_mittelhelle_hautfarbe_blond:','es':':mujer_rubia_tono_de_piel_claro_medio:','fr':':femme_blonde_peau_moyennement_claire:','ja':':金髪の女性_やや薄い肌色:','ko':':금발_여자_연한_갈색_피부:','pt':':mulher_pele_morena_clara_e_cabelo_loiro:','it':':donna_bionda_carnagione_abbastanza_chiara:','fa':':زن_پوست_روشن_موبور:','id':':wanita_berambut_pirang_warna_kulit_cerah-sedang:','zh':':金发女_中等浅肤色:','ru':':блондинка_светлый_тон_кожи:','tr':':sarışın_kadın_orta_açık_cilt_tonu:','ar':':امرأة_شقراء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FC\U0000200D\U0001F9B1':{'en':':woman_medium-light_skin_tone_curly_hair:','status':fully_qualified,'E':11,'de':':frau_mittelhelle_hautfarbe_lockiges_haar:','es':':mujer_tono_de_piel_claro_medio_y_pelo_rizado:','fr':':femme_peau_moyennement_claire_et_cheveux_bouclés:','ja':':女性_やや薄い肌色_巻き毛:','ko':':여자_연한_갈색_피부_곱슬머리:','pt':':mulher_pele_morena_clara_e_cabelo_cacheado:','it':':donna_carnagione_abbastanza_chiara_e_capelli_ricci:','fa':':زن_پوست_روشن_موفرفری:','id':':wanita_warna_kulit_cerah-sedang_rambut_keriting:','zh':':女人_中等浅肤色卷发:','ru':':женщина_светлый_тон_кожи_кудрявые_волосы:','tr':':kadın_orta_açık_cilt_tonu_kıvırcık_saç:','ar':':سيدة_بشرة_بلون_فاتح_ومعتدل_وشعر_مجعد:'},'\U0001F469\U0001F3FC\U0000200D\U0001F9B0':{'en':':woman_medium-light_skin_tone_red_hair:','status':fully_qualified,'E':11,'de':':frau_mittelhelle_hautfarbe_rotes_haar:','es':':mujer_tono_de_piel_claro_medio_y_pelo_pelirrojo:','fr':':femme_peau_moyennement_claire_et_cheveux_roux:','ja':':女性_やや薄い肌色_赤毛:','ko':':여자_연한_갈색_피부_빨간_머리:','pt':':mulher_pele_morena_clara_e_cabelo_vermelho:','it':':donna_carnagione_abbastanza_chiara_e_capelli_rossi:','fa':':زن_پوست_روشن_موقرمز:','id':':wanita_warna_kulit_cerah-sedang_rambut_merah:','zh':':女人_中等浅肤色红发:','ru':':женщина_светлый_тон_кожи_рыжие_волосы:','tr':':kadın_orta_açık_cilt_tonu_kızıl_saç:','ar':':سيدة_بشرة_بلون_فاتح_ومعتدل_وشعر_أحمر:'},'\U0001F469\U0001F3FC\U0000200D\U0001F9B3':{'en':':woman_medium-light_skin_tone_white_hair:','status':fully_qualified,'E':11,'de':':frau_mittelhelle_hautfarbe_weißes_haar:','es':':mujer_tono_de_piel_claro_medio_y_pelo_blanco:','fr':':femme_peau_moyennement_claire_et_cheveux_blancs:','ja':':女性_やや薄い肌色_白髪:','ko':':여자_연한_갈색_피부_백발:','pt':':mulher_pele_morena_clara_e_cabelo_branco:','it':':donna_carnagione_abbastanza_chiara_e_capelli_bianchi:','fa':':زن_پوست_روشن_موسفید:','id':':wanita_warna_kulit_cerah-sedang_rambut_putih:','zh':':女人_中等浅肤色白发:','ru':':женщина_светлый_тон_кожи_седые_волосы:','tr':':kadın_orta_açık_cilt_tonu_beyaz_saç:','ar':':سيدة_بشرة_بلون_فاتح_ومعتدل_وشعر_أبيض:'},'\U0001F469\U0001F3FD':{'en':':woman_medium_skin_tone:','status':fully_qualified,'E':1,'de':':frau_mittlere_hautfarbe:','es':':mujer_tono_de_piel_medio:','fr':':femme_peau_légèrement_mate:','ja':':女性_中間の肌色:','ko':':여자_갈색_피부:','pt':':mulher_pele_morena:','it':':donna_carnagione_olivastra:','fa':':زن_پوست_طلایی:','id':':wanita_warna_kulit_sedang:','zh':':女人_中等肤色:','ru':':женщина_средний_тон_кожи:','tr':':kadın_orta_cilt_tonu:','ar':':سيدة_بشرة_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F9B2':{'en':':woman_medium_skin_tone_bald:','status':fully_qualified,'E':11,'de':':frau_mittlere_hautfarbe_glatze:','es':':mujer_tono_de_piel_medio_y_sin_pelo:','fr':':femme_peau_légèrement_mate_et_chauve:','ja':':女性_中間の肌色_はげ頭:','ko':':여자_갈색_피부_대머리:','pt':':mulher_pele_morena_e_careca:','it':':donna_carnagione_olivastra_e_calvo:','fa':':زن_پوست_طلایی_تاس:','id':':wanita_warna_kulit_sedang_gundul:','zh':':女人_中等肤色秃顶:','ru':':женщина_средний_тон_кожи_без_волос:','tr':':kadın_orta_cilt_tonu_kel:','ar':':سيدة_بشرة_بلون_معتدل_وأصلع:'},'\U0001F9D4\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_medium_skin_tone_beard:','status':fully_qualified,'E':13.1,'de':':frau_mittlere_hautfarbe_bart:','es':':mujer_tono_de_piel_medio_y_barba:','fr':':femme_barbue_peau_légèrement_mate:','ja':':あごひげの女性_中間の肌色:','ko':':수염_난_여자_갈색_피부:','pt':':mulher_pele_morena_e_barba:','it':':donna_con_la_barba_carnagione_olivastra:','fa':':زن_پوست_طلایی_ریش:','id':':wanita_warna_kulit_sedang_janggut:','zh':':有络腮胡子的女人_中等肤色:','ru':':бородатая_женщина_среднии_тон_кожи:','tr':':kadın_orta_cilt_tonu_sakal:','ar':':امرأة_بلحية_بشرة_بلون_معتدل:'},'\U0001F9D4\U0001F3FD\U0000200D\U00002640':{'en':':woman_medium_skin_tone_beard:','status':minimally_qualified,'E':13.1,'de':':frau_mittlere_hautfarbe_bart:','es':':mujer_tono_de_piel_medio_y_barba:','fr':':femme_barbue_peau_légèrement_mate:','ja':':あごひげの女性_中間の肌色:','ko':':수염_난_여자_갈색_피부:','pt':':mulher_pele_morena_e_barba:','it':':donna_con_la_barba_carnagione_olivastra:','fa':':زن_پوست_طلایی_ریش:','id':':wanita_warna_kulit_sedang_janggut:','zh':':有络腮胡子的女人_中等肤色:','ru':':бородатая_женщина_средний_тон_кожи:','tr':':kadın_orta_cilt_tonu_sakal:','ar':':امرأة_بلحية_بشرة_بلون_معتدل:'},'\U0001F471\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_medium_skin_tone_blond_hair:','status':fully_qualified,'E':4,'de':':frau_mittlere_hautfarbe,_blond:','es':':mujer_rubia_tono_de_piel_medio:','fr':':femme_blonde_peau_légèrement_mate:','ja':':金髪の女性_中間の肌色:','ko':':금발_여자_갈색_피부:','pt':':mulher_pele_morena_e_cabelo_loiro:','it':':donna_bionda_carnagione_olivastra:','fa':':زن_پوست_طلایی_موبور:','id':':wanita_berambut_pirang_warna_kulit_sedang:','zh':':金发女_中等肤色:','ru':':блондинка_среднии_тон_кожи:','tr':':sarışın_kadın_orta_cilt_tonu:','ar':':امرأة_شقراء_بشرة_بلون_معتدل:'},'\U0001F471\U0001F3FD\U0000200D\U00002640':{'en':':woman_medium_skin_tone_blond_hair:','status':minimally_qualified,'E':4,'de':':frau_mittlere_hautfarbe_blond:','es':':mujer_rubia_tono_de_piel_medio:','fr':':femme_blonde_peau_légèrement_mate:','ja':':金髪の女性_中間の肌色:','ko':':금발_여자_갈색_피부:','pt':':mulher_pele_morena_e_cabelo_loiro:','it':':donna_bionda_carnagione_olivastra:','fa':':زن_پوست_طلایی_موبور:','id':':wanita_berambut_pirang_warna_kulit_sedang:','zh':':金发女_中等肤色:','ru':':блондинка_средний_тон_кожи:','tr':':sarışın_kadın_orta_cilt_tonu:','ar':':امرأة_شقراء_بشرة_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F9B1':{'en':':woman_medium_skin_tone_curly_hair:','status':fully_qualified,'E':11,'de':':frau_mittlere_hautfarbe_lockiges_haar:','es':':mujer_tono_de_piel_medio_y_pelo_rizado:','fr':':femme_peau_légèrement_mate_et_cheveux_bouclés:','ja':':女性_中間の肌色_巻き毛:','ko':':여자_갈색_피부_곱슬머리:','pt':':mulher_pele_morena_e_cabelo_cacheado:','it':':donna_carnagione_olivastra_e_capelli_ricci:','fa':':زن_پوست_طلایی_موفرفری:','id':':wanita_warna_kulit_sedang_rambut_keriting:','zh':':女人_中等肤色卷发:','ru':':женщина_средний_тон_кожи_кудрявые_волосы:','tr':':kadın_orta_cilt_tonu_kıvırcık_saç:','ar':':سيدة_بشرة_بلون_معتدل_وشعر_مجعد:'},'\U0001F469\U0001F3FD\U0000200D\U0001F9B0':{'en':':woman_medium_skin_tone_red_hair:','status':fully_qualified,'E':11,'de':':frau_mittlere_hautfarbe_rotes_haar:','es':':mujer_tono_de_piel_medio_y_pelo_pelirrojo:','fr':':femme_peau_légèrement_mate_et_cheveux_roux:','ja':':女性_中間の肌色_赤毛:','ko':':여자_갈색_피부_빨간_머리:','pt':':mulher_pele_morena_e_cabelo_vermelho:','it':':donna_carnagione_olivastra_e_capelli_rossi:','fa':':زن_پوست_طلایی_موقرمز:','id':':wanita_warna_kulit_sedang_rambut_merah:','zh':':女人_中等肤色红发:','ru':':женщина_средний_тон_кожи_рыжие_волосы:','tr':':kadın_orta_cilt_tonu_kızıl_saç:','ar':':سيدة_بشرة_بلون_معتدل_وشعر_أحمر:'},'\U0001F469\U0001F3FD\U0000200D\U0001F9B3':{'en':':woman_medium_skin_tone_white_hair:','status':fully_qualified,'E':11,'de':':frau_mittlere_hautfarbe_weißes_haar:','es':':mujer_tono_de_piel_medio_y_pelo_blanco:','fr':':femme_peau_légèrement_mate_et_cheveux_blancs:','ja':':女性_中間の肌色_白髪:','ko':':여자_갈색_피부_백발:','pt':':mulher_pele_morena_e_cabelo_branco:','it':':donna_carnagione_olivastra_e_capelli_bianchi:','fa':':زن_پوست_طلایی_موسفید:','id':':wanita_warna_kulit_sedang_rambut_putih:','zh':':女人_中等肤色白发:','ru':':женщина_средний_тон_кожи_седые_волосы:','tr':':kadın_orta_cilt_tonu_beyaz_saç:','ar':':سيدة_بشرة_بلون_معتدل_وشعر_أبيض:'},'\U0001F6B5\U0000200D\U00002640\U0000FE0F':{'en':':woman_mountain_biking:','status':fully_qualified,'E':4,'alias':[':mountain_biking_woman:'],'de':':mountainbikerin:','es':':mujer_en_bicicleta_de_montaña:','fr':':femme_en_vtt:','ja':':マウンテンバイクに乗る女:','ko':':산악_자전거_타는_여자:','pt':':mulher_fazendo_mountain_bike:','it':':ciclista_donna_di_mountain_bike:','fa':':زن_و_دوچرخه_سواری_کوهستان:','id':':wanita_bersepeda_gunung:','zh':':女生骑山地车:','ru':':женщина_на_горном_велосипеде:','tr':':dağ_bisikleti_süren_kadın:','ar':':متسلقة_جبال_بدراجة:'},'\U0001F6B5\U0000200D\U00002640':{'en':':woman_mountain_biking:','status':minimally_qualified,'E':4,'alias':[':mountain_biking_woman:'],'de':':mountainbikerin:','es':':mujer_en_bicicleta_de_montaña:','fr':':femme_en_vtt:','ja':':マウンテンバイクに乗る女:','ko':':산악_자전거_타는_여자:','pt':':mulher_fazendo_mountain_bike:','it':':ciclista_donna_di_mountain_bike:','fa':':زن_و_دوچرخه_سواری_کوهستان:','id':':wanita_bersepeda_gunung:','zh':':女生骑山地车:','ru':':женщина_на_горном_велосипеде:','tr':':dağ_bisikleti_süren_kadın:','ar':':متسلقة_جبال_بدراجة:'},'\U0001F6B5\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_mountain_biking_dark_skin_tone:','status':fully_qualified,'E':4,'de':':mountainbikerin_dunkle_hautfarbe:','es':':mujer_en_bicicleta_de_montaña_tono_de_piel_oscuro:','fr':':femme_en_vtt_peau_foncée:','ja':':マウンテンバイクに乗る女_濃い肌色:','ko':':산악_자전거_타는_여자_검은색_피부:','pt':':mulher_fazendo_mountain_bike_pele_escura:','it':':ciclista_donna_di_mountain_bike_carnagione_scura:','fa':':زن_و_دوچرخه_سواری_کوهستان_پوست_آبنوسی:','id':':wanita_bersepeda_gunung_warna_kulit_gelap:','zh':':女生骑山地车_较深肤色:','ru':':женщина_на_горном_велосипеде_очень_темныи_тон_кожи:','tr':':dağ_bisikleti_süren_kadın_koyu_cilt_tonu:','ar':':متسلقة_جبال_بدراجة_بشرة_بلون_غامق:'},'\U0001F6B5\U0001F3FF\U0000200D\U00002640':{'en':':woman_mountain_biking_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':mountainbikerin_dunkle_hautfarbe:','es':':mujer_en_bicicleta_de_montaña_tono_de_piel_oscuro:','fr':':femme_en_vtt_peau_foncée:','ja':':マウンテンバイクに乗る女_濃い肌色:','ko':':산악_자전거_타는_여자_검은색_피부:','pt':':mulher_fazendo_mountain_bike_pele_escura:','it':':ciclista_donna_di_mountain_bike_carnagione_scura:','fa':':زن_و_دوچرخه_سواری_کوهستان_پوست_آبنوسی:','id':':wanita_bersepeda_gunung_warna_kulit_gelap:','zh':':女生骑山地车_较深肤色:','ru':':женщина_на_горном_велосипеде_очень_темный_тон_кожи:','tr':':dağ_bisikleti_süren_kadın_koyu_cilt_tonu:','ar':':متسلقة_جبال_بدراجة_بشرة_بلون_غامق:'},'\U0001F6B5\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_mountain_biking_light_skin_tone:','status':fully_qualified,'E':4,'de':':mountainbikerin_helle_hautfarbe:','es':':mujer_en_bicicleta_de_montaña_tono_de_piel_claro:','fr':':femme_en_vtt_peau_claire:','ja':':マウンテンバイクに乗る女_薄い肌色:','ko':':산악_자전거_타는_여자_하얀_피부:','pt':':mulher_fazendo_mountain_bike_pele_clara:','it':':ciclista_donna_di_mountain_bike_carnagione_chiara:','fa':':زن_و_دوچرخه_سواری_کوهستان_پوست_سفید:','id':':wanita_bersepeda_gunung_warna_kulit_cerah:','zh':':女生骑山地车_较浅肤色:','ru':':женщина_на_горном_велосипеде_очень_светлыи_тон_кожи:','tr':':dağ_bisikleti_süren_kadın_açık_cilt_tonu:','ar':':متسلقة_جبال_بدراجة_بشرة_بلون_فاتح:'},'\U0001F6B5\U0001F3FB\U0000200D\U00002640':{'en':':woman_mountain_biking_light_skin_tone:','status':minimally_qualified,'E':4,'de':':mountainbikerin_helle_hautfarbe:','es':':mujer_en_bicicleta_de_montaña_tono_de_piel_claro:','fr':':femme_en_vtt_peau_claire:','ja':':マウンテンバイクに乗る女_薄い肌色:','ko':':산악_자전거_타는_여자_하얀_피부:','pt':':mulher_fazendo_mountain_bike_pele_clara:','it':':ciclista_donna_di_mountain_bike_carnagione_chiara:','fa':':زن_و_دوچرخه_سواری_کوهستان_پوست_سفید:','id':':wanita_bersepeda_gunung_warna_kulit_cerah:','zh':':女生骑山地车_较浅肤色:','ru':':женщина_на_горном_велосипеде_очень_светлый_тон_кожи:','tr':':dağ_bisikleti_süren_kadın_açık_cilt_tonu:','ar':':متسلقة_جبال_بدراجة_بشرة_بلون_فاتح:'},'\U0001F6B5\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_mountain_biking_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':mountainbikerin_mitteldunkle_hautfarbe:','es':':mujer_en_bicicleta_de_montaña_tono_de_piel_oscuro_medio:','fr':':femme_en_vtt_peau_mate:','ja':':マウンテンバイクに乗る女_やや濃い肌色:','ko':':산악_자전거_타는_여자_진한_갈색_피부:','pt':':mulher_fazendo_mountain_bike_pele_morena_escura:','it':':ciclista_donna_di_mountain_bike_carnagione_abbastanza_scura:','fa':':زن_و_دوچرخه_سواری_کوهستان_پوست_گندمی:','id':':wanita_bersepeda_gunung_warna_kulit_gelap-sedang:','zh':':女生骑山地车_中等深肤色:','ru':':женщина_на_горном_велосипеде_темныи_тон_кожи:','tr':':dağ_bisikleti_süren_kadın_orta_koyu_cilt_tonu:','ar':':متسلقة_جبال_بدراجة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6B5\U0001F3FE\U0000200D\U00002640':{'en':':woman_mountain_biking_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':mountainbikerin_mitteldunkle_hautfarbe:','es':':mujer_en_bicicleta_de_montaña_tono_de_piel_oscuro_medio:','fr':':femme_en_vtt_peau_mate:','ja':':マウンテンバイクに乗る女_やや濃い肌色:','ko':':산악_자전거_타는_여자_진한_갈색_피부:','pt':':mulher_fazendo_mountain_bike_pele_morena_escura:','it':':ciclista_donna_di_mountain_bike_carnagione_abbastanza_scura:','fa':':زن_و_دوچرخه_سواری_کوهستان_پوست_گندمی:','id':':wanita_bersepeda_gunung_warna_kulit_gelap-sedang:','zh':':女生骑山地车_中等深肤色:','ru':':женщина_на_горном_велосипеде_темный_тон_кожи:','tr':':dağ_bisikleti_süren_kadın_orta_koyu_cilt_tonu:','ar':':متسلقة_جبال_بدراجة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6B5\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_mountain_biking_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':mountainbikerin_mittelhelle_hautfarbe:','es':':mujer_en_bicicleta_de_montaña_tono_de_piel_claro_medio:','fr':':femme_en_vtt_peau_moyennement_claire:','ja':':マウンテンバイクに乗る女_やや薄い肌色:','ko':':산악_자전거_타는_여자_연한_갈색_피부:','pt':':mulher_fazendo_mountain_bike_pele_morena_clara:','it':':ciclista_donna_di_mountain_bike_carnagione_abbastanza_chiara:','fa':':زن_و_دوچرخه_سواری_کوهستان_پوست_روشن:','id':':wanita_bersepeda_gunung_warna_kulit_cerah-sedang:','zh':':女生骑山地车_中等浅肤色:','ru':':женщина_на_горном_велосипеде_светлыи_тон_кожи:','tr':':dağ_bisikleti_süren_kadın_orta_açık_cilt_tonu:','ar':':متسلقة_جبال_بدراجة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6B5\U0001F3FC\U0000200D\U00002640':{'en':':woman_mountain_biking_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':mountainbikerin_mittelhelle_hautfarbe:','es':':mujer_en_bicicleta_de_montaña_tono_de_piel_claro_medio:','fr':':femme_en_vtt_peau_moyennement_claire:','ja':':マウンテンバイクに乗る女_やや薄い肌色:','ko':':산악_자전거_타는_여자_연한_갈색_피부:','pt':':mulher_fazendo_mountain_bike_pele_morena_clara:','it':':ciclista_donna_di_mountain_bike_carnagione_abbastanza_chiara:','fa':':زن_و_دوچرخه_سواری_کوهستان_پوست_روشن:','id':':wanita_bersepeda_gunung_warna_kulit_cerah-sedang:','zh':':女生骑山地车_中等浅肤色:','ru':':женщина_на_горном_велосипеде_светлый_тон_кожи:','tr':':dağ_bisikleti_süren_kadın_orta_açık_cilt_tonu:','ar':':متسلقة_جبال_بدراجة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6B5\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_mountain_biking_medium_skin_tone:','status':fully_qualified,'E':4,'de':':mountainbikerin_mittlere_hautfarbe:','es':':mujer_en_bicicleta_de_montaña_tono_de_piel_medio:','fr':':femme_en_vtt_peau_légèrement_mate:','ja':':マウンテンバイクに乗る女_中間の肌色:','ko':':산악_자전거_타는_여자_갈색_피부:','pt':':mulher_fazendo_mountain_bike_pele_morena:','it':':ciclista_donna_di_mountain_bike_carnagione_olivastra:','fa':':زن_و_دوچرخه_سواری_کوهستان_پوست_طلایی:','id':':wanita_bersepeda_gunung_warna_kulit_sedang:','zh':':女生骑山地车_中等肤色:','ru':':женщина_на_горном_велосипеде_среднии_тон_кожи:','tr':':dağ_bisikleti_süren_kadın_orta_cilt_tonu:','ar':':متسلقة_جبال_بدراجة_بشرة_بلون_معتدل:'},'\U0001F6B5\U0001F3FD\U0000200D\U00002640':{'en':':woman_mountain_biking_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':mountainbikerin_mittlere_hautfarbe:','es':':mujer_en_bicicleta_de_montaña_tono_de_piel_medio:','fr':':femme_en_vtt_peau_légèrement_mate:','ja':':マウンテンバイクに乗る女_中間の肌色:','ko':':산악_자전거_타는_여자_갈색_피부:','pt':':mulher_fazendo_mountain_bike_pele_morena:','it':':ciclista_donna_di_mountain_bike_carnagione_olivastra:','fa':':زن_و_دوچرخه_سواری_کوهستان_پوست_طلایی:','id':':wanita_bersepeda_gunung_warna_kulit_sedang:','zh':':女生骑山地车_中等肤色:','ru':':женщина_на_горном_велосипеде_средний_тон_кожи:','tr':':dağ_bisikleti_süren_kadın_orta_cilt_tonu:','ar':':متسلقة_جبال_بدراجة_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U0001F4BC':{'en':':woman_office_worker:','status':fully_qualified,'E':4,'de':':büroangestellte:','es':':oficinista_mujer:','fr':':employée_de_bureau:','ja':':女性会社員:','ko':':여자_회사원:','pt':':funcionária_de_escritório:','it':':impiegata:','fa':':کارمند_زن:','id':':pekerja_kantor_wanita:','zh':':女白领:','ru':':офисная_работница:','tr':':kadın_ofis_çalışanı:','ar':':موظفة:'},'\U0001F469\U0001F3FF\U0000200D\U0001F4BC':{'en':':woman_office_worker_dark_skin_tone:','status':fully_qualified,'E':4,'de':':büroangestellte_dunkle_hautfarbe:','es':':oficinista_mujer_tono_de_piel_oscuro:','fr':':employée_de_bureau_peau_foncée:','ja':':女性会社員_濃い肌色:','ko':':여자_회사원_검은색_피부:','pt':':funcionária_de_escritório_pele_escura:','it':':impiegata_carnagione_scura:','fa':':کارمند_زن_پوست_آبنوسی:','id':':pekerja_kantor_wanita_warna_kulit_gelap:','zh':':女白领_较深肤色:','ru':':офисная_работница_очень_темный_тон_кожи:','tr':':kadın_ofis_çalışanı_koyu_cilt_tonu:','ar':':موظفة_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F4BC':{'en':':woman_office_worker_light_skin_tone:','status':fully_qualified,'E':4,'de':':büroangestellte_helle_hautfarbe:','es':':oficinista_mujer_tono_de_piel_claro:','fr':':employée_de_bureau_peau_claire:','ja':':女性会社員_薄い肌色:','ko':':여자_회사원_하얀_피부:','pt':':funcionária_de_escritório_pele_clara:','it':':impiegata_carnagione_chiara:','fa':':کارمند_زن_پوست_سفید:','id':':pekerja_kantor_wanita_warna_kulit_cerah:','zh':':女白领_较浅肤色:','ru':':офисная_работница_очень_светлый_тон_кожи:','tr':':kadın_ofis_çalışanı_açık_cilt_tonu:','ar':':موظفة_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F4BC':{'en':':woman_office_worker_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':büroangestellte_mitteldunkle_hautfarbe:','es':':oficinista_mujer_tono_de_piel_oscuro_medio:','fr':':employée_de_bureau_peau_mate:','ja':':女性会社員_やや濃い肌色:','ko':':여자_회사원_진한_갈색_피부:','pt':':funcionária_de_escritório_pele_morena_escura:','it':':impiegata_carnagione_abbastanza_scura:','fa':':کارمند_زن_پوست_گندمی:','id':':pekerja_kantor_wanita_warna_kulit_gelap-sedang:','zh':':女白领_中等深肤色:','ru':':офисная_работница_темный_тон_кожи:','tr':':kadın_ofis_çalışanı_orta_koyu_cilt_tonu:','ar':':موظفة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F4BC':{'en':':woman_office_worker_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':büroangestellte_mittelhelle_hautfarbe:','es':':oficinista_mujer_tono_de_piel_claro_medio:','fr':':employée_de_bureau_peau_moyennement_claire:','ja':':女性会社員_やや薄い肌色:','ko':':여자_회사원_연한_갈색_피부:','pt':':funcionária_de_escritório_pele_morena_clara:','it':':impiegata_carnagione_abbastanza_chiara:','fa':':کارمند_زن_پوست_روشن:','id':':pekerja_kantor_wanita_warna_kulit_cerah-sedang:','zh':':女白领_中等浅肤色:','ru':':офисная_работница_светлый_тон_кожи:','tr':':kadın_ofis_çalışanı_orta_açık_cilt_tonu:','ar':':موظفة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F4BC':{'en':':woman_office_worker_medium_skin_tone:','status':fully_qualified,'E':4,'de':':büroangestellte_mittlere_hautfarbe:','es':':oficinista_mujer_tono_de_piel_medio:','fr':':employée_de_bureau_peau_légèrement_mate:','ja':':女性会社員_中間の肌色:','ko':':여자_회사원_갈색_피부:','pt':':funcionária_de_escritório_pele_morena:','it':':impiegata_carnagione_olivastra:','fa':':کارمند_زن_پوست_طلایی:','id':':pekerja_kantor_wanita_warna_kulit_sedang:','zh':':女白领_中等肤色:','ru':':офисная_работница_средний_тон_кожи:','tr':':kadın_ofis_çalışanı_orta_cilt_tonu:','ar':':موظفة_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U00002708\U0000FE0F':{'en':':woman_pilot:','status':fully_qualified,'E':4,'de':':pilotin:','es':':piloto_mujer:','fr':':pilote_femme:','ja':':女性パイロット:','ko':':여자_기장:','pt':':piloto_de_avião_mulher:','it':':pilota_donna:','fa':':خلبان_زن:','id':':pilot_wanita:','zh':':女飞行员:','ru':':женщина-пилот:','tr':':kadın_pilot:','ar':':ربانة_طائرة:'},'\U0001F469\U0000200D\U00002708':{'en':':woman_pilot:','status':minimally_qualified,'E':4,'de':':pilotin:','es':':piloto_mujer:','fr':':pilote_femme:','ja':':女性パイロット:','ko':':여자_기장:','pt':':piloto_de_avião_mulher:','it':':pilota_donna:','fa':':خلبان_زن:','id':':pilot_wanita:','zh':':女飞行员:','ru':':женщина_пилот:','tr':':kadın_pilot:','ar':':ربانة_طائرة:'},'\U0001F469\U0001F3FF\U0000200D\U00002708\U0000FE0F':{'en':':woman_pilot_dark_skin_tone:','status':fully_qualified,'E':4,'de':':pilotin_dunkle_hautfarbe:','es':':piloto_mujer_tono_de_piel_oscuro:','fr':':pilote_femme_peau_foncée:','ja':':女性パイロット_濃い肌色:','ko':':여자_기장_검은색_피부:','pt':':piloto_de_avião_mulher_pele_escura:','it':':pilota_donna_carnagione_scura:','fa':':خلبان_زن_پوست_آبنوسی:','id':':pilot_wanita_warna_kulit_gelap:','zh':':女飞行员_较深肤色:','ru':':женщина-пилот_очень_темныи_тон_кожи:','tr':':kadın_pilot_koyu_cilt_tonu:','ar':':ربانة_طائرة_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FF\U0000200D\U00002708':{'en':':woman_pilot_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':pilotin_dunkle_hautfarbe:','es':':piloto_mujer_tono_de_piel_oscuro:','fr':':pilote_femme_peau_foncée:','ja':':女性パイロット_濃い肌色:','ko':':여자_기장_검은색_피부:','pt':':piloto_de_avião_mulher_pele_escura:','it':':pilota_donna_carnagione_scura:','fa':':خلبان_زن_پوست_آبنوسی:','id':':pilot_wanita_warna_kulit_gelap:','zh':':女飞行员_较深肤色:','ru':':женщина_пилот_очень_темный_тон_кожи:','tr':':kadın_pilot_koyu_cilt_tonu:','ar':':ربانة_طائرة_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U00002708\U0000FE0F':{'en':':woman_pilot_light_skin_tone:','status':fully_qualified,'E':4,'de':':pilotin_helle_hautfarbe:','es':':piloto_mujer_tono_de_piel_claro:','fr':':pilote_femme_peau_claire:','ja':':女性パイロット_薄い肌色:','ko':':여자_기장_하얀_피부:','pt':':piloto_de_avião_mulher_pele_clara:','it':':pilota_donna_carnagione_chiara:','fa':':خلبان_زن_پوست_سفید:','id':':pilot_wanita_warna_kulit_cerah:','zh':':女飞行员_较浅肤色:','ru':':женщина-пилот_очень_светлыи_тон_кожи:','tr':':kadın_pilot_açık_cilt_tonu:','ar':':ربانة_طائرة_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FB\U0000200D\U00002708':{'en':':woman_pilot_light_skin_tone:','status':minimally_qualified,'E':4,'de':':pilotin_helle_hautfarbe:','es':':piloto_mujer_tono_de_piel_claro:','fr':':pilote_femme_peau_claire:','ja':':女性パイロット_薄い肌色:','ko':':여자_기장_하얀_피부:','pt':':piloto_de_avião_mulher_pele_clara:','it':':pilota_donna_carnagione_chiara:','fa':':خلبان_زن_پوست_سفید:','id':':pilot_wanita_warna_kulit_cerah:','zh':':女飞行员_较浅肤色:','ru':':женщина_пилот_очень_светлый_тон_кожи:','tr':':kadın_pilot_açık_cilt_tonu:','ar':':ربانة_طائرة_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U00002708\U0000FE0F':{'en':':woman_pilot_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':pilotin_mitteldunkle_hautfarbe:','es':':piloto_mujer_tono_de_piel_oscuro_medio:','fr':':pilote_femme_peau_mate:','ja':':女性パイロット_やや濃い肌色:','ko':':여자_기장_진한_갈색_피부:','pt':':piloto_de_avião_mulher_pele_morena_escura:','it':':pilota_donna_carnagione_abbastanza_scura:','fa':':خلبان_زن_پوست_گندمی:','id':':pilot_wanita_warna_kulit_gelap-sedang:','zh':':女飞行员_中等深肤色:','ru':':женщина-пилот_темныи_тон_кожи:','tr':':kadın_pilot_orta_koyu_cilt_tonu:','ar':':ربانة_طائرة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FE\U0000200D\U00002708':{'en':':woman_pilot_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':pilotin_mitteldunkle_hautfarbe:','es':':piloto_mujer_tono_de_piel_oscuro_medio:','fr':':pilote_femme_peau_mate:','ja':':女性パイロット_やや濃い肌色:','ko':':여자_기장_진한_갈색_피부:','pt':':piloto_de_avião_mulher_pele_morena_escura:','it':':pilota_donna_carnagione_abbastanza_scura:','fa':':خلبان_زن_پوست_گندمی:','id':':pilot_wanita_warna_kulit_gelap-sedang:','zh':':女飞行员_中等深肤色:','ru':':женщина_пилот_темный_тон_кожи:','tr':':kadın_pilot_orta_koyu_cilt_tonu:','ar':':ربانة_طائرة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U00002708\U0000FE0F':{'en':':woman_pilot_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':pilotin_mittelhelle_hautfarbe:','es':':piloto_mujer_tono_de_piel_claro_medio:','fr':':pilote_femme_peau_moyennement_claire:','ja':':女性パイロット_やや薄い肌色:','ko':':여자_기장_연한_갈색_피부:','pt':':piloto_de_avião_mulher_pele_morena_clara:','it':':pilota_donna_carnagione_abbastanza_chiara:','fa':':خلبان_زن_پوست_روشن:','id':':pilot_wanita_warna_kulit_cerah-sedang:','zh':':女飞行员_中等浅肤色:','ru':':женщина-пилот_светлыи_тон_кожи:','tr':':kadın_pilot_orta_açık_cilt_tonu:','ar':':ربانة_طائرة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FC\U0000200D\U00002708':{'en':':woman_pilot_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':pilotin_mittelhelle_hautfarbe:','es':':piloto_mujer_tono_de_piel_claro_medio:','fr':':pilote_femme_peau_moyennement_claire:','ja':':女性パイロット_やや薄い肌色:','ko':':여자_기장_연한_갈색_피부:','pt':':piloto_de_avião_mulher_pele_morena_clara:','it':':pilota_donna_carnagione_abbastanza_chiara:','fa':':خلبان_زن_پوست_روشن:','id':':pilot_wanita_warna_kulit_cerah-sedang:','zh':':女飞行员_中等浅肤色:','ru':':женщина_пилот_светлый_тон_кожи:','tr':':kadın_pilot_orta_açık_cilt_tonu:','ar':':ربانة_طائرة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002708\U0000FE0F':{'en':':woman_pilot_medium_skin_tone:','status':fully_qualified,'E':4,'de':':pilotin_mittlere_hautfarbe:','es':':piloto_mujer_tono_de_piel_medio:','fr':':pilote_femme_peau_légèrement_mate:','ja':':女性パイロット_中間の肌色:','ko':':여자_기장_갈색_피부:','pt':':piloto_de_avião_mulher_pele_morena:','it':':pilota_donna_carnagione_olivastra:','fa':':خلبان_زن_پوست_طلایی:','id':':pilot_wanita_warna_kulit_sedang:','zh':':女飞行员_中等肤色:','ru':':женщина-пилот_среднии_тон_кожи:','tr':':kadın_pilot_orta_cilt_tonu:','ar':':ربانة_طائرة_بشرة_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U00002708':{'en':':woman_pilot_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':pilotin_mittlere_hautfarbe:','es':':piloto_mujer_tono_de_piel_medio:','fr':':pilote_femme_peau_légèrement_mate:','ja':':女性パイロット_中間の肌色:','ko':':여자_기장_갈색_피부:','pt':':piloto_de_avião_mulher_pele_morena:','it':':pilota_donna_carnagione_olivastra:','fa':':خلبان_زن_پوست_طلایی:','id':':pilot_wanita_warna_kulit_sedang:','zh':':女飞行员_中等肤色:','ru':':женщина_пилот_средний_тон_кожи:','tr':':kadın_pilot_orta_cilt_tonu:','ar':':ربانة_طائرة_بشرة_بلون_معتدل:'},'\U0001F93E\U0000200D\U00002640\U0000FE0F':{'en':':woman_playing_handball:','status':fully_qualified,'E':4,'de':':handballspielerin:','es':':mujer_jugando_al_balonmano:','fr':':handballeuse:','ja':':ハンドボールをする女:','ko':':핸드볼하는_여자:','pt':':jogadora_de_handebol:','it':':pallamanista_donna:','fa':':هندبالیست_زن:','id':':wanita_bermain_bola_tangan:','zh':':女生玩手球:','ru':':гандболистка:','tr':':hentbol_oynayan_kadın:','ar':':لاعبة_كرة_يد:'},'\U0001F93E\U0000200D\U00002640':{'en':':woman_playing_handball:','status':minimally_qualified,'E':4,'de':':handballspielerin:','es':':mujer_jugando_al_balonmano:','fr':':handballeuse:','ja':':ハンドボールをする女:','ko':':핸드볼하는_여자:','pt':':jogadora_de_handebol:','it':':pallamanista_donna:','fa':':هندبالیست_زن:','id':':wanita_bermain_bola_tangan:','zh':':女生玩手球:','ru':':гандболистка:','tr':':hentbol_oynayan_kadın:','ar':':لاعبة_كرة_يد:'},'\U0001F93E\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_playing_handball_dark_skin_tone:','status':fully_qualified,'E':4,'de':':handballspielerin_dunkle_hautfarbe:','es':':mujer_jugando_al_balonmano_tono_de_piel_oscuro:','fr':':handballeuse_peau_foncée:','ja':':ハンドボールをする女_濃い肌色:','ko':':핸드볼하는_여자_검은색_피부:','pt':':jogadora_de_handebol_pele_escura:','it':':pallamanista_donna_carnagione_scura:','fa':':هندبالیست_زن_پوست_آبنوسی:','id':':wanita_bermain_bola_tangan_warna_kulit_gelap:','zh':':女生玩手球_较深肤色:','ru':':гандболистка_очень_темныи_тон_кожи:','tr':':hentbol_oynayan_kadın_koyu_cilt_tonu:','ar':':لاعبة_كرة_يد_بشرة_بلون_غامق:'},'\U0001F93E\U0001F3FF\U0000200D\U00002640':{'en':':woman_playing_handball_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':handballspielerin_dunkle_hautfarbe:','es':':mujer_jugando_al_balonmano_tono_de_piel_oscuro:','fr':':handballeuse_peau_foncée:','ja':':ハンドボールをする女_濃い肌色:','ko':':핸드볼하는_여자_검은색_피부:','pt':':jogadora_de_handebol_pele_escura:','it':':pallamanista_donna_carnagione_scura:','fa':':هندبالیست_زن_پوست_آبنوسی:','id':':wanita_bermain_bola_tangan_warna_kulit_gelap:','zh':':女生玩手球_较深肤色:','ru':':гандболистка_очень_темный_тон_кожи:','tr':':hentbol_oynayan_kadın_koyu_cilt_tonu:','ar':':لاعبة_كرة_يد_بشرة_بلون_غامق:'},'\U0001F93E\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_playing_handball_light_skin_tone:','status':fully_qualified,'E':4,'de':':handballspielerin_helle_hautfarbe:','es':':mujer_jugando_al_balonmano_tono_de_piel_claro:','fr':':handballeuse_peau_claire:','ja':':ハンドボールをする女_薄い肌色:','ko':':핸드볼하는_여자_하얀_피부:','pt':':jogadora_de_handebol_pele_clara:','it':':pallamanista_donna_carnagione_chiara:','fa':':هندبالیست_زن_پوست_سفید:','id':':wanita_bermain_bola_tangan_warna_kulit_cerah:','zh':':女生玩手球_较浅肤色:','ru':':гандболистка_очень_светлыи_тон_кожи:','tr':':hentbol_oynayan_kadın_açık_cilt_tonu:','ar':':لاعبة_كرة_يد_بشرة_بلون_فاتح:'},'\U0001F93E\U0001F3FB\U0000200D\U00002640':{'en':':woman_playing_handball_light_skin_tone:','status':minimally_qualified,'E':4,'de':':handballspielerin_helle_hautfarbe:','es':':mujer_jugando_al_balonmano_tono_de_piel_claro:','fr':':handballeuse_peau_claire:','ja':':ハンドボールをする女_薄い肌色:','ko':':핸드볼하는_여자_하얀_피부:','pt':':jogadora_de_handebol_pele_clara:','it':':pallamanista_donna_carnagione_chiara:','fa':':هندبالیست_زن_پوست_سفید:','id':':wanita_bermain_bola_tangan_warna_kulit_cerah:','zh':':女生玩手球_较浅肤色:','ru':':гандболистка_очень_светлый_тон_кожи:','tr':':hentbol_oynayan_kadın_açık_cilt_tonu:','ar':':لاعبة_كرة_يد_بشرة_بلون_فاتح:'},'\U0001F93E\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_playing_handball_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':handballspielerin_mitteldunkle_hautfarbe:','es':':mujer_jugando_al_balonmano_tono_de_piel_oscuro_medio:','fr':':handballeuse_peau_mate:','ja':':ハンドボールをする女_やや濃い肌色:','ko':':핸드볼하는_여자_진한_갈색_피부:','pt':':jogadora_de_handebol_pele_morena_escura:','it':':pallamanista_donna_carnagione_abbastanza_scura:','fa':':هندبالیست_زن_پوست_گندمی:','id':':wanita_bermain_bola_tangan_warna_kulit_gelap-sedang:','zh':':女生玩手球_中等深肤色:','ru':':гандболистка_темныи_тон_кожи:','tr':':hentbol_oynayan_kadın_orta_koyu_cilt_tonu:','ar':':لاعبة_كرة_يد_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F93E\U0001F3FE\U0000200D\U00002640':{'en':':woman_playing_handball_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':handballspielerin_mitteldunkle_hautfarbe:','es':':mujer_jugando_al_balonmano_tono_de_piel_oscuro_medio:','fr':':handballeuse_peau_mate:','ja':':ハンドボールをする女_やや濃い肌色:','ko':':핸드볼하는_여자_진한_갈색_피부:','pt':':jogadora_de_handebol_pele_morena_escura:','it':':pallamanista_donna_carnagione_abbastanza_scura:','fa':':هندبالیست_زن_پوست_گندمی:','id':':wanita_bermain_bola_tangan_warna_kulit_gelap-sedang:','zh':':女生玩手球_中等深肤色:','ru':':гандболистка_темный_тон_кожи:','tr':':hentbol_oynayan_kadın_orta_koyu_cilt_tonu:','ar':':لاعبة_كرة_يد_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F93E\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_playing_handball_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':handballspielerin_mittelhelle_hautfarbe:','es':':mujer_jugando_al_balonmano_tono_de_piel_claro_medio:','fr':':handballeuse_peau_moyennement_claire:','ja':':ハンドボールをする女_やや薄い肌色:','ko':':핸드볼하는_여자_연한_갈색_피부:','pt':':jogadora_de_handebol_pele_morena_clara:','it':':pallamanista_donna_carnagione_abbastanza_chiara:','fa':':هندبالیست_زن_پوست_روشن:','id':':wanita_bermain_bola_tangan_warna_kulit_cerah-sedang:','zh':':女生玩手球_中等浅肤色:','ru':':гандболистка_светлыи_тон_кожи:','tr':':hentbol_oynayan_kadın_orta_açık_cilt_tonu:','ar':':لاعبة_كرة_يد_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F93E\U0001F3FC\U0000200D\U00002640':{'en':':woman_playing_handball_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':handballspielerin_mittelhelle_hautfarbe:','es':':mujer_jugando_al_balonmano_tono_de_piel_claro_medio:','fr':':handballeuse_peau_moyennement_claire:','ja':':ハンドボールをする女_やや薄い肌色:','ko':':핸드볼하는_여자_연한_갈색_피부:','pt':':jogadora_de_handebol_pele_morena_clara:','it':':pallamanista_donna_carnagione_abbastanza_chiara:','fa':':هندبالیست_زن_پوست_روشن:','id':':wanita_bermain_bola_tangan_warna_kulit_cerah-sedang:','zh':':女生玩手球_中等浅肤色:','ru':':гандболистка_светлый_тон_кожи:','tr':':hentbol_oynayan_kadın_orta_açık_cilt_tonu:','ar':':لاعبة_كرة_يد_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F93E\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_playing_handball_medium_skin_tone:','status':fully_qualified,'E':4,'de':':handballspielerin_mittlere_hautfarbe:','es':':mujer_jugando_al_balonmano_tono_de_piel_medio:','fr':':handballeuse_peau_légèrement_mate:','ja':':ハンドボールをする女_中間の肌色:','ko':':핸드볼하는_여자_갈색_피부:','pt':':jogadora_de_handebol_pele_morena:','it':':pallamanista_donna_carnagione_olivastra:','fa':':هندبالیست_زن_پوست_طلایی:','id':':wanita_bermain_bola_tangan_warna_kulit_sedang:','zh':':女生玩手球_中等肤色:','ru':':гандболистка_среднии_тон_кожи:','tr':':hentbol_oynayan_kadın_orta_cilt_tonu:','ar':':لاعبة_كرة_يد_بشرة_بلون_معتدل:'},'\U0001F93E\U0001F3FD\U0000200D\U00002640':{'en':':woman_playing_handball_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':handballspielerin_mittlere_hautfarbe:','es':':mujer_jugando_al_balonmano_tono_de_piel_medio:','fr':':handballeuse_peau_légèrement_mate:','ja':':ハンドボールをする女_中間の肌色:','ko':':핸드볼하는_여자_갈색_피부:','pt':':jogadora_de_handebol_pele_morena:','it':':pallamanista_donna_carnagione_olivastra:','fa':':هندبالیست_زن_پوست_طلایی:','id':':wanita_bermain_bola_tangan_warna_kulit_sedang:','zh':':女生玩手球_中等肤色:','ru':':гандболистка_средний_тон_кожи:','tr':':hentbol_oynayan_kadın_orta_cilt_tonu:','ar':':لاعبة_كرة_يد_بشرة_بلون_معتدل:'},'\U0001F93D\U0000200D\U00002640\U0000FE0F':{'en':':woman_playing_water_polo:','status':fully_qualified,'E':4,'de':':wasserballspielerin:','es':':mujer_jugando_al_waterpolo:','fr':':joueuse_de_water-polo:','ja':':水球をする女:','ko':':수구하는_여자:','pt':':mulher_jogando_polo_aquático:','it':':pallanuotista_donna:','fa':':واترپلو_زنان:','id':':wanita_bermain_polo_air:','zh':':女生玩水球:','ru':':женщина_играет_в_водное_поло:','tr':':su_topu_oynayan_kadın:','ar':':لاعبة_كرة_ماء:'},'\U0001F93D\U0000200D\U00002640':{'en':':woman_playing_water_polo:','status':minimally_qualified,'E':4,'de':':wasserballspielerin:','es':':mujer_jugando_al_waterpolo:','fr':':joueuse_de_water-polo:','ja':':水球をする女:','ko':':수구하는_여자:','pt':':mulher_jogando_polo_aquático:','it':':pallanuotista_donna:','fa':':واترپلو_زنان:','id':':wanita_bermain_polo_air:','zh':':女生玩水球:','ru':':женщина_играет_в_водное_поло:','tr':':su_topu_oynayan_kadın:','ar':':لاعبة_كرة_ماء:'},'\U0001F93D\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_playing_water_polo_dark_skin_tone:','status':fully_qualified,'E':4,'de':':wasserballspielerin_dunkle_hautfarbe:','es':':mujer_jugando_al_waterpolo_tono_de_piel_oscuro:','fr':':joueuse_de_water-polo_peau_foncée:','ja':':水球をする女_濃い肌色:','ko':':수구하는_여자_검은색_피부:','pt':':mulher_jogando_polo_aquático_pele_escura:','it':':pallanuotista_donna_carnagione_scura:','fa':':واترپلو_زنان_پوست_آبنوسی:','id':':wanita_bermain_polo_air_warna_kulit_gelap:','zh':':女生玩水球_较深肤色:','ru':':женщина_играет_в_водное_поло_очень_темныи_тон_кожи:','tr':':su_topu_oynayan_kadın_koyu_cilt_tonu:','ar':':لاعبة_كرة_ماء_بشرة_بلون_غامق:'},'\U0001F93D\U0001F3FF\U0000200D\U00002640':{'en':':woman_playing_water_polo_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':wasserballspielerin_dunkle_hautfarbe:','es':':mujer_jugando_al_waterpolo_tono_de_piel_oscuro:','fr':':joueuse_de_water-polo_peau_foncée:','ja':':水球をする女_濃い肌色:','ko':':수구하는_여자_검은색_피부:','pt':':mulher_jogando_polo_aquático_pele_escura:','it':':pallanuotista_donna_carnagione_scura:','fa':':واترپلو_زنان_پوست_آبنوسی:','id':':wanita_bermain_polo_air_warna_kulit_gelap:','zh':':女生玩水球_较深肤色:','ru':':женщина_играет_в_водное_поло_очень_темный_тон_кожи:','tr':':su_topu_oynayan_kadın_koyu_cilt_tonu:','ar':':لاعبة_كرة_ماء_بشرة_بلون_غامق:'},'\U0001F93D\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_playing_water_polo_light_skin_tone:','status':fully_qualified,'E':4,'de':':wasserballspielerin_helle_hautfarbe:','es':':mujer_jugando_al_waterpolo_tono_de_piel_claro:','fr':':joueuse_de_water-polo_peau_claire:','ja':':水球をする女_薄い肌色:','ko':':수구하는_여자_하얀_피부:','pt':':mulher_jogando_polo_aquático_pele_clara:','it':':pallanuotista_donna_carnagione_chiara:','fa':':واترپلو_زنان_پوست_سفید:','id':':wanita_bermain_polo_air_warna_kulit_cerah:','zh':':女生玩水球_较浅肤色:','ru':':женщина_играет_в_водное_поло_очень_светлыи_тон_кожи:','tr':':su_topu_oynayan_kadın_açık_cilt_tonu:','ar':':لاعبة_كرة_ماء_بشرة_بلون_فاتح:'},'\U0001F93D\U0001F3FB\U0000200D\U00002640':{'en':':woman_playing_water_polo_light_skin_tone:','status':minimally_qualified,'E':4,'de':':wasserballspielerin_helle_hautfarbe:','es':':mujer_jugando_al_waterpolo_tono_de_piel_claro:','fr':':joueuse_de_water-polo_peau_claire:','ja':':水球をする女_薄い肌色:','ko':':수구하는_여자_하얀_피부:','pt':':mulher_jogando_polo_aquático_pele_clara:','it':':pallanuotista_donna_carnagione_chiara:','fa':':واترپلو_زنان_پوست_سفید:','id':':wanita_bermain_polo_air_warna_kulit_cerah:','zh':':女生玩水球_较浅肤色:','ru':':женщина_играет_в_водное_поло_очень_светлый_тон_кожи:','tr':':su_topu_oynayan_kadın_açık_cilt_tonu:','ar':':لاعبة_كرة_ماء_بشرة_بلون_فاتح:'},'\U0001F93D\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_playing_water_polo_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':wasserballspielerin_mitteldunkle_hautfarbe:','es':':mujer_jugando_al_waterpolo_tono_de_piel_oscuro_medio:','fr':':joueuse_de_water-polo_peau_mate:','ja':':水球をする女_やや濃い肌色:','ko':':수구하는_여자_진한_갈색_피부:','pt':':mulher_jogando_polo_aquático_pele_morena_escura:','it':':pallanuotista_donna_carnagione_abbastanza_scura:','fa':':واترپلو_زنان_پوست_گندمی:','id':':wanita_bermain_polo_air_warna_kulit_gelap-sedang:','zh':':女生玩水球_中等深肤色:','ru':':женщина_играет_в_водное_поло_темныи_тон_кожи:','tr':':su_topu_oynayan_kadın_orta_koyu_cilt_tonu:','ar':':لاعبة_كرة_ماء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F93D\U0001F3FE\U0000200D\U00002640':{'en':':woman_playing_water_polo_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':wasserballspielerin_mitteldunkle_hautfarbe:','es':':mujer_jugando_al_waterpolo_tono_de_piel_oscuro_medio:','fr':':joueuse_de_water-polo_peau_mate:','ja':':水球をする女_やや濃い肌色:','ko':':수구하는_여자_진한_갈색_피부:','pt':':mulher_jogando_polo_aquático_pele_morena_escura:','it':':pallanuotista_donna_carnagione_abbastanza_scura:','fa':':واترپلو_زنان_پوست_گندمی:','id':':wanita_bermain_polo_air_warna_kulit_gelap-sedang:','zh':':女生玩水球_中等深肤色:','ru':':женщина_играет_в_водное_поло_темный_тон_кожи:','tr':':su_topu_oynayan_kadın_orta_koyu_cilt_tonu:','ar':':لاعبة_كرة_ماء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F93D\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_playing_water_polo_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':wasserballspielerin_mittelhelle_hautfarbe:','es':':mujer_jugando_al_waterpolo_tono_de_piel_claro_medio:','fr':':joueuse_de_water-polo_peau_moyennement_claire:','ja':':水球をする女_やや薄い肌色:','ko':':수구하는_여자_연한_갈색_피부:','pt':':mulher_jogando_polo_aquático_pele_morena_clara:','it':':pallanuotista_donna_carnagione_abbastanza_chiara:','fa':':واترپلو_زنان_پوست_روشن:','id':':wanita_bermain_polo_air_warna_kulit_cerah-sedang:','zh':':女生玩水球_中等浅肤色:','ru':':женщина_играет_в_водное_поло_светлыи_тон_кожи:','tr':':su_topu_oynayan_kadın_orta_açık_cilt_tonu:','ar':':لاعبة_كرة_ماء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F93D\U0001F3FC\U0000200D\U00002640':{'en':':woman_playing_water_polo_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':wasserballspielerin_mittelhelle_hautfarbe:','es':':mujer_jugando_al_waterpolo_tono_de_piel_claro_medio:','fr':':joueuse_de_water-polo_peau_moyennement_claire:','ja':':水球をする女_やや薄い肌色:','ko':':수구하는_여자_연한_갈색_피부:','pt':':mulher_jogando_polo_aquático_pele_morena_clara:','it':':pallanuotista_donna_carnagione_abbastanza_chiara:','fa':':واترپلو_زنان_پوست_روشن:','id':':wanita_bermain_polo_air_warna_kulit_cerah-sedang:','zh':':女生玩水球_中等浅肤色:','ru':':женщина_играет_в_водное_поло_светлый_тон_кожи:','tr':':su_topu_oynayan_kadın_orta_açık_cilt_tonu:','ar':':لاعبة_كرة_ماء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F93D\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_playing_water_polo_medium_skin_tone:','status':fully_qualified,'E':4,'de':':wasserballspielerin_mittlere_hautfarbe:','es':':mujer_jugando_al_waterpolo_tono_de_piel_medio:','fr':':joueuse_de_water-polo_peau_légèrement_mate:','ja':':水球をする女_中間の肌色:','ko':':수구하는_여자_갈색_피부:','pt':':mulher_jogando_polo_aquático_pele_morena:','it':':pallanuotista_donna_carnagione_olivastra:','fa':':واترپلو_زنان_پوست_طلایی:','id':':wanita_bermain_polo_air_warna_kulit_sedang:','zh':':女生玩水球_中等肤色:','ru':':женщина_играет_в_водное_поло_среднии_тон_кожи:','tr':':su_topu_oynayan_kadın_orta_cilt_tonu:','ar':':لاعبة_كرة_ماء_بشرة_بلون_معتدل:'},'\U0001F93D\U0001F3FD\U0000200D\U00002640':{'en':':woman_playing_water_polo_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':wasserballspielerin_mittlere_hautfarbe:','es':':mujer_jugando_al_waterpolo_tono_de_piel_medio:','fr':':joueuse_de_water-polo_peau_légèrement_mate:','ja':':水球をする女_中間の肌色:','ko':':수구하는_여자_갈색_피부:','pt':':mulher_jogando_polo_aquático_pele_morena:','it':':pallanuotista_donna_carnagione_olivastra:','fa':':واترپلو_زنان_پوست_طلایی:','id':':wanita_bermain_polo_air_warna_kulit_sedang:','zh':':女生玩水球_中等肤色:','ru':':женщина_играет_в_водное_поло_средний_тон_кожи:','tr':':su_topu_oynayan_kadın_orta_cilt_tonu:','ar':':لاعبة_كرة_ماء_بشرة_بلون_معتدل:'},'\U0001F46E\U0000200D\U00002640\U0000FE0F':{'en':':woman_police_officer:','status':fully_qualified,'E':4,'alias':[':policewoman:'],'de':':polizistin:','es':':agente_de_policía_mujer:','fr':':policière:','ja':':女性警察官:','ko':':여자_경찰관:','pt':':policial_mulher:','it':':poliziotta:','fa':':پلیس_زن:','id':':polisi_wanita:','zh':':女警察:','ru':':женщина-полицеискии:','tr':':kadın_polis_memuru:','ar':':شرطية:'},'\U0001F46E\U0000200D\U00002640':{'en':':woman_police_officer:','status':minimally_qualified,'E':4,'alias':[':policewoman:'],'de':':polizistin:','es':':agente_de_policía_mujer:','fr':':policière:','ja':':女性警察官:','ko':':여자_경찰관:','pt':':policial_mulher:','it':':poliziotta:','fa':':پلیس_زن:','id':':polisi_wanita:','zh':':女警察:','ru':':женщина_полицейский:','tr':':kadın_polis_memuru:','ar':':شرطية:'},'\U0001F46E\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_police_officer_dark_skin_tone:','status':fully_qualified,'E':4,'de':':polizistin_dunkle_hautfarbe:','es':':agente_de_policía_mujer_tono_de_piel_oscuro:','fr':':policière_peau_foncée:','ja':':女性警察官_濃い肌色:','ko':':여자_경찰관_검은색_피부:','pt':':policial_mulher_pele_escura:','it':':poliziotta_carnagione_scura:','fa':':پلیس_زن_پوست_آبنوسی:','id':':polisi_wanita_warna_kulit_gelap:','zh':':女警察_较深肤色:','ru':':женщина-полицеискии_очень_темныи_тон_кожи:','tr':':kadın_polis_memuru_koyu_cilt_tonu:','ar':':شرطية_بشرة_بلون_غامق:'},'\U0001F46E\U0001F3FF\U0000200D\U00002640':{'en':':woman_police_officer_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':polizistin_dunkle_hautfarbe:','es':':agente_de_policía_mujer_tono_de_piel_oscuro:','fr':':policière_peau_foncée:','ja':':女性警察官_濃い肌色:','ko':':여자_경찰관_검은색_피부:','pt':':policial_mulher_pele_escura:','it':':poliziotta_carnagione_scura:','fa':':پلیس_زن_پوست_آبنوسی:','id':':polisi_wanita_warna_kulit_gelap:','zh':':女警察_较深肤色:','ru':':женщина_полицейский_очень_темный_тон_кожи:','tr':':kadın_polis_memuru_koyu_cilt_tonu:','ar':':شرطية_بشرة_بلون_غامق:'},'\U0001F46E\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_police_officer_light_skin_tone:','status':fully_qualified,'E':4,'de':':polizistin_helle_hautfarbe:','es':':agente_de_policía_mujer_tono_de_piel_claro:','fr':':policière_peau_claire:','ja':':女性警察官_薄い肌色:','ko':':여자_경찰관_하얀_피부:','pt':':policial_mulher_pele_clara:','it':':poliziotta_carnagione_chiara:','fa':':پلیس_زن_پوست_سفید:','id':':polisi_wanita_warna_kulit_cerah:','zh':':女警察_较浅肤色:','ru':':женщина-полицеискии_очень_светлыи_тон_кожи:','tr':':kadın_polis_memuru_açık_cilt_tonu:','ar':':شرطية_بشرة_بلون_فاتح:'},'\U0001F46E\U0001F3FB\U0000200D\U00002640':{'en':':woman_police_officer_light_skin_tone:','status':minimally_qualified,'E':4,'de':':polizistin_helle_hautfarbe:','es':':agente_de_policía_mujer_tono_de_piel_claro:','fr':':policière_peau_claire:','ja':':女性警察官_薄い肌色:','ko':':여자_경찰관_하얀_피부:','pt':':policial_mulher_pele_clara:','it':':poliziotta_carnagione_chiara:','fa':':پلیس_زن_پوست_سفید:','id':':polisi_wanita_warna_kulit_cerah:','zh':':女警察_较浅肤色:','ru':':женщина_полицейский_очень_светлый_тон_кожи:','tr':':kadın_polis_memuru_açık_cilt_tonu:','ar':':شرطية_بشرة_بلون_فاتح:'},'\U0001F46E\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_police_officer_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':polizistin_mitteldunkle_hautfarbe:','es':':agente_de_policía_mujer_tono_de_piel_oscuro_medio:','fr':':policière_peau_mate:','ja':':女性警察官_やや濃い肌色:','ko':':여자_경찰관_진한_갈색_피부:','pt':':policial_mulher_pele_morena_escura:','it':':poliziotta_carnagione_abbastanza_scura:','fa':':پلیس_زن_پوست_گندمی:','id':':polisi_wanita_warna_kulit_gelap-sedang:','zh':':女警察_中等深肤色:','ru':':женщина-полицеискии_темныи_тон_кожи:','tr':':kadın_polis_memuru_orta_koyu_cilt_tonu:','ar':':شرطية_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F46E\U0001F3FE\U0000200D\U00002640':{'en':':woman_police_officer_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':polizistin_mitteldunkle_hautfarbe:','es':':agente_de_policía_mujer_tono_de_piel_oscuro_medio:','fr':':policière_peau_mate:','ja':':女性警察官_やや濃い肌色:','ko':':여자_경찰관_진한_갈색_피부:','pt':':policial_mulher_pele_morena_escura:','it':':poliziotta_carnagione_abbastanza_scura:','fa':':پلیس_زن_پوست_گندمی:','id':':polisi_wanita_warna_kulit_gelap-sedang:','zh':':女警察_中等深肤色:','ru':':женщина_полицейский_темный_тон_кожи:','tr':':kadın_polis_memuru_orta_koyu_cilt_tonu:','ar':':شرطية_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F46E\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_police_officer_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':polizistin_mittelhelle_hautfarbe:','es':':agente_de_policía_mujer_tono_de_piel_claro_medio:','fr':':policière_peau_moyennement_claire:','ja':':女性警察官_やや薄い肌色:','ko':':여자_경찰관_연한_갈색_피부:','pt':':policial_mulher_pele_morena_clara:','it':':poliziotta_carnagione_abbastanza_chiara:','fa':':پلیس_زن_پوست_روشن:','id':':polisi_wanita_warna_kulit_cerah-sedang:','zh':':女警察_中等浅肤色:','ru':':женщина-полицеискии_светлыи_тон_кожи:','tr':':kadın_polis_memuru_orta_açık_cilt_tonu:','ar':':شرطية_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F46E\U0001F3FC\U0000200D\U00002640':{'en':':woman_police_officer_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':polizistin_mittelhelle_hautfarbe:','es':':agente_de_policía_mujer_tono_de_piel_claro_medio:','fr':':policière_peau_moyennement_claire:','ja':':女性警察官_やや薄い肌色:','ko':':여자_경찰관_연한_갈색_피부:','pt':':policial_mulher_pele_morena_clara:','it':':poliziotta_carnagione_abbastanza_chiara:','fa':':پلیس_زن_پوست_روشن:','id':':polisi_wanita_warna_kulit_cerah-sedang:','zh':':女警察_中等浅肤色:','ru':':женщина_полицейский_светлый_тон_кожи:','tr':':kadın_polis_memuru_orta_açık_cilt_tonu:','ar':':شرطية_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F46E\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_police_officer_medium_skin_tone:','status':fully_qualified,'E':4,'de':':polizistin_mittlere_hautfarbe:','es':':agente_de_policía_mujer_tono_de_piel_medio:','fr':':policière_peau_légèrement_mate:','ja':':女性警察官_中間の肌色:','ko':':여자_경찰관_갈색_피부:','pt':':policial_mulher_pele_morena:','it':':poliziotta_carnagione_olivastra:','fa':':پلیس_زن_پوست_طلایی:','id':':polisi_wanita_warna_kulit_sedang:','zh':':女警察_中等肤色:','ru':':женщина-полицеискии_среднии_тон_кожи:','tr':':kadın_polis_memuru_orta_cilt_tonu:','ar':':شرطية_بشرة_بلون_معتدل:'},'\U0001F46E\U0001F3FD\U0000200D\U00002640':{'en':':woman_police_officer_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':polizistin_mittlere_hautfarbe:','es':':agente_de_policía_mujer_tono_de_piel_medio:','fr':':policière_peau_légèrement_mate:','ja':':女性警察官_中間の肌色:','ko':':여자_경찰관_갈색_피부:','pt':':policial_mulher_pele_morena:','it':':poliziotta_carnagione_olivastra:','fa':':پلیس_زن_پوست_طلایی:','id':':polisi_wanita_warna_kulit_sedang:','zh':':女警察_中等肤色:','ru':':женщина_полицейский_средний_тон_кожи:','tr':':kadın_polis_memuru_orta_cilt_tonu:','ar':':شرطية_بشرة_بلون_معتدل:'},'\U0001F64E\U0000200D\U00002640\U0000FE0F':{'en':':woman_pouting:','status':fully_qualified,'E':4,'alias':[':pouting_woman:'],'de':':schmollende_frau:','es':':mujer_haciendo_pucheros:','fr':':femme_qui_boude:','ja':':不機嫌な女:','ko':':뾰로통한_여자:','pt':':mulher_fazendo_bico:','it':':donna_imbronciata:','fa':':زن_اخم_وتخم_می_کند:','id':':wanita_tidak_senang:','zh':':撅嘴女:','ru':':обиженная_женщина:','tr':':darılan_kadın:','ar':':سيدة_عابسة:'},'\U0001F64E\U0000200D\U00002640':{'en':':woman_pouting:','status':minimally_qualified,'E':4,'alias':[':pouting_woman:'],'de':':schmollende_frau:','es':':mujer_haciendo_pucheros:','fr':':femme_qui_boude:','ja':':不機嫌な女:','ko':':뾰로통한_여자:','pt':':mulher_fazendo_bico:','it':':donna_imbronciata:','fa':':زن_اخم_وتخم_می_کند:','id':':wanita_tidak_senang:','zh':':撅嘴女:','ru':':обиженная_женщина:','tr':':darılan_kadın:','ar':':سيدة_عابسة:'},'\U0001F64E\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_pouting_dark_skin_tone:','status':fully_qualified,'E':4,'de':':schmollende_frau_dunkle_hautfarbe:','es':':mujer_haciendo_pucheros_tono_de_piel_oscuro:','fr':':femme_qui_boude_peau_foncée:','ja':':不機嫌な女_濃い肌色:','ko':':뾰로통한_여자_검은색_피부:','pt':':mulher_fazendo_bico_pele_escura:','it':':donna_imbronciata_carnagione_scura:','fa':':زن_اخم_وتخم_می_کند_پوست_آبنوسی:','id':':wanita_tidak_senang_warna_kulit_gelap:','zh':':撅嘴女_较深肤色:','ru':':обиженная_женщина_очень_темныи_тон_кожи:','tr':':darılan_kadın_koyu_cilt_tonu:','ar':':سيدة_عابسة_بشرة_بلون_غامق:'},'\U0001F64E\U0001F3FF\U0000200D\U00002640':{'en':':woman_pouting_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':schmollende_frau_dunkle_hautfarbe:','es':':mujer_haciendo_pucheros_tono_de_piel_oscuro:','fr':':femme_qui_boude_peau_foncée:','ja':':不機嫌な女_濃い肌色:','ko':':뾰로통한_여자_검은색_피부:','pt':':mulher_fazendo_bico_pele_escura:','it':':donna_imbronciata_carnagione_scura:','fa':':زن_اخم_وتخم_می_کند_پوست_آبنوسی:','id':':wanita_tidak_senang_warna_kulit_gelap:','zh':':撅嘴女_较深肤色:','ru':':обиженная_женщина_очень_темный_тон_кожи:','tr':':darılan_kadın_koyu_cilt_tonu:','ar':':سيدة_عابسة_بشرة_بلون_غامق:'},'\U0001F64E\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_pouting_light_skin_tone:','status':fully_qualified,'E':4,'de':':schmollende_frau_helle_hautfarbe:','es':':mujer_haciendo_pucheros_tono_de_piel_claro:','fr':':femme_qui_boude_peau_claire:','ja':':不機嫌な女_薄い肌色:','ko':':뾰로통한_여자_하얀_피부:','pt':':mulher_fazendo_bico_pele_clara:','it':':donna_imbronciata_carnagione_chiara:','fa':':زن_اخم_وتخم_می_کند_پوست_سفید:','id':':wanita_tidak_senang_warna_kulit_cerah:','zh':':撅嘴女_较浅肤色:','ru':':обиженная_женщина_очень_светлыи_тон_кожи:','tr':':darılan_kadın_açık_cilt_tonu:','ar':':سيدة_عابسة_بشرة_بلون_فاتح:'},'\U0001F64E\U0001F3FB\U0000200D\U00002640':{'en':':woman_pouting_light_skin_tone:','status':minimally_qualified,'E':4,'de':':schmollende_frau_helle_hautfarbe:','es':':mujer_haciendo_pucheros_tono_de_piel_claro:','fr':':femme_qui_boude_peau_claire:','ja':':不機嫌な女_薄い肌色:','ko':':뾰로통한_여자_하얀_피부:','pt':':mulher_fazendo_bico_pele_clara:','it':':donna_imbronciata_carnagione_chiara:','fa':':زن_اخم_وتخم_می_کند_پوست_سفید:','id':':wanita_tidak_senang_warna_kulit_cerah:','zh':':撅嘴女_较浅肤色:','ru':':обиженная_женщина_очень_светлый_тон_кожи:','tr':':darılan_kadın_açık_cilt_tonu:','ar':':سيدة_عابسة_بشرة_بلون_فاتح:'},'\U0001F64E\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_pouting_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':schmollende_frau_mitteldunkle_hautfarbe:','es':':mujer_haciendo_pucheros_tono_de_piel_oscuro_medio:','fr':':femme_qui_boude_peau_mate:','ja':':不機嫌な女_やや濃い肌色:','ko':':뾰로통한_여자_진한_갈색_피부:','pt':':mulher_fazendo_bico_pele_morena_escura:','it':':donna_imbronciata_carnagione_abbastanza_scura:','fa':':زن_اخم_وتخم_می_کند_پوست_گندمی:','id':':wanita_tidak_senang_warna_kulit_gelap-sedang:','zh':':撅嘴女_中等深肤色:','ru':':обиженная_женщина_темныи_тон_кожи:','tr':':darılan_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_عابسة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F64E\U0001F3FE\U0000200D\U00002640':{'en':':woman_pouting_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':schmollende_frau_mitteldunkle_hautfarbe:','es':':mujer_haciendo_pucheros_tono_de_piel_oscuro_medio:','fr':':femme_qui_boude_peau_mate:','ja':':不機嫌な女_やや濃い肌色:','ko':':뾰로통한_여자_진한_갈색_피부:','pt':':mulher_fazendo_bico_pele_morena_escura:','it':':donna_imbronciata_carnagione_abbastanza_scura:','fa':':زن_اخم_وتخم_می_کند_پوست_گندمی:','id':':wanita_tidak_senang_warna_kulit_gelap-sedang:','zh':':撅嘴女_中等深肤色:','ru':':обиженная_женщина_темный_тон_кожи:','tr':':darılan_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_عابسة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F64E\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_pouting_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':schmollende_frau_mittelhelle_hautfarbe:','es':':mujer_haciendo_pucheros_tono_de_piel_claro_medio:','fr':':femme_qui_boude_peau_moyennement_claire:','ja':':不機嫌な女_やや薄い肌色:','ko':':뾰로통한_여자_연한_갈색_피부:','pt':':mulher_fazendo_bico_pele_morena_clara:','it':':donna_imbronciata_carnagione_abbastanza_chiara:','fa':':زن_اخم_وتخم_می_کند_پوست_روشن:','id':':wanita_tidak_senang_warna_kulit_cerah-sedang:','zh':':撅嘴女_中等浅肤色:','ru':':обиженная_женщина_светлыи_тон_кожи:','tr':':darılan_kadın_orta_açık_cilt_tonu:','ar':':سيدة_عابسة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F64E\U0001F3FC\U0000200D\U00002640':{'en':':woman_pouting_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':schmollende_frau_mittelhelle_hautfarbe:','es':':mujer_haciendo_pucheros_tono_de_piel_claro_medio:','fr':':femme_qui_boude_peau_moyennement_claire:','ja':':不機嫌な女_やや薄い肌色:','ko':':뾰로통한_여자_연한_갈색_피부:','pt':':mulher_fazendo_bico_pele_morena_clara:','it':':donna_imbronciata_carnagione_abbastanza_chiara:','fa':':زن_اخم_وتخم_می_کند_پوست_روشن:','id':':wanita_tidak_senang_warna_kulit_cerah-sedang:','zh':':撅嘴女_中等浅肤色:','ru':':обиженная_женщина_светлый_тон_кожи:','tr':':darılan_kadın_orta_açık_cilt_tonu:','ar':':سيدة_عابسة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F64E\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_pouting_medium_skin_tone:','status':fully_qualified,'E':4,'de':':schmollende_frau_mittlere_hautfarbe:','es':':mujer_haciendo_pucheros_tono_de_piel_medio:','fr':':femme_qui_boude_peau_légèrement_mate:','ja':':不機嫌な女_中間の肌色:','ko':':뾰로통한_여자_갈색_피부:','pt':':mulher_fazendo_bico_pele_morena:','it':':donna_imbronciata_carnagione_olivastra:','fa':':زن_اخم_وتخم_می_کند_پوست_طلایی:','id':':wanita_tidak_senang_warna_kulit_sedang:','zh':':撅嘴女_中等肤色:','ru':':обиженная_женщина_среднии_тон_кожи:','tr':':darılan_kadın_orta_cilt_tonu:','ar':':سيدة_عابسة_بشرة_بلون_معتدل:'},'\U0001F64E\U0001F3FD\U0000200D\U00002640':{'en':':woman_pouting_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':schmollende_frau_mittlere_hautfarbe:','es':':mujer_haciendo_pucheros_tono_de_piel_medio:','fr':':femme_qui_boude_peau_légèrement_mate:','ja':':不機嫌な女_中間の肌色:','ko':':뾰로통한_여자_갈색_피부:','pt':':mulher_fazendo_bico_pele_morena:','it':':donna_imbronciata_carnagione_olivastra:','fa':':زن_اخم_وتخم_می_کند_پوست_طلایی:','id':':wanita_tidak_senang_warna_kulit_sedang:','zh':':撅嘴女_中等肤色:','ru':':обиженная_женщина_средний_тон_кожи:','tr':':darılan_kadın_orta_cilt_tonu:','ar':':سيدة_عابسة_بشرة_بلون_معتدل:'},'\U0001F64B\U0000200D\U00002640\U0000FE0F':{'en':':woman_raising_hand:','status':fully_qualified,'E':4,'alias':[':raising_hand_woman:'],'de':':frau_mit_erhobenem_arm:','es':':mujer_con_la_mano_levantada:','fr':':femme_qui_lève_la_main:','ja':':手を挙げる女:','ko':':한_손_든_여자:','pt':':mulher_levantando_a_mão:','it':':donna_con_mano_alzata:','fa':':زن_دستش_را_بالا_می_برد:','id':':wanita_mengangkat_tangan:','zh':':女生举手:','ru':':женщина_с_поднятои_рукои:','tr':':elini_kaldıran_kadın:','ar':':سيدة_ترفع_يدها:'},'\U0001F64B\U0000200D\U00002640':{'en':':woman_raising_hand:','status':minimally_qualified,'E':4,'alias':[':raising_hand_woman:'],'de':':frau_mit_erhobenem_arm:','es':':mujer_con_la_mano_levantada:','fr':':femme_qui_lève_la_main:','ja':':手を挙げる女:','ko':':한_손_든_여자:','pt':':mulher_levantando_a_mão:','it':':donna_con_mano_alzata:','fa':':زن_دستش_را_بالا_می_برد:','id':':wanita_mengangkat_tangan:','zh':':女生举手:','ru':':женщина_с_поднятой_рукой:','tr':':elini_kaldıran_kadın:','ar':':سيدة_ترفع_يدها:'},'\U0001F64B\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_raising_hand_dark_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_erhobenem_arm_dunkle_hautfarbe:','es':':mujer_con_la_mano_levantada_tono_de_piel_oscuro:','fr':':femme_qui_lève_la_main_peau_foncée:','ja':':手を挙げる女_濃い肌色:','ko':':한_손_든_여자_검은색_피부:','pt':':mulher_levantando_a_mão_pele_escura:','it':':donna_con_mano_alzata_carnagione_scura:','fa':':زن_دستش_را_بالا_می_برد_پوست_آبنوسی:','id':':wanita_mengangkat_tangan_warna_kulit_gelap:','zh':':女生举手_较深肤色:','ru':':женщина_с_поднятои_рукои_очень_темныи_тон_кожи:','tr':':elini_kaldıran_kadın_koyu_cilt_tonu:','ar':':سيدة_ترفع_يدها_بشرة_بلون_غامق:'},'\U0001F64B\U0001F3FF\U0000200D\U00002640':{'en':':woman_raising_hand_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_erhobenem_arm_dunkle_hautfarbe:','es':':mujer_con_la_mano_levantada_tono_de_piel_oscuro:','fr':':femme_qui_lève_la_main_peau_foncée:','ja':':手を挙げる女_濃い肌色:','ko':':한_손_든_여자_검은색_피부:','pt':':mulher_levantando_a_mão_pele_escura:','it':':donna_con_mano_alzata_carnagione_scura:','fa':':زن_دستش_را_بالا_می_برد_پوست_آبنوسی:','id':':wanita_mengangkat_tangan_warna_kulit_gelap:','zh':':女生举手_较深肤色:','ru':':женщина_с_поднятой_рукой_очень_темный_тон_кожи:','tr':':elini_kaldıran_kadın_koyu_cilt_tonu:','ar':':سيدة_ترفع_يدها_بشرة_بلون_غامق:'},'\U0001F64B\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_raising_hand_light_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_erhobenem_arm_helle_hautfarbe:','es':':mujer_con_la_mano_levantada_tono_de_piel_claro:','fr':':femme_qui_lève_la_main_peau_claire:','ja':':手を挙げる女_薄い肌色:','ko':':한_손_든_여자_하얀_피부:','pt':':mulher_levantando_a_mão_pele_clara:','it':':donna_con_mano_alzata_carnagione_chiara:','fa':':زن_دستش_را_بالا_می_برد_پوست_سفید:','id':':wanita_mengangkat_tangan_warna_kulit_cerah:','zh':':女生举手_较浅肤色:','ru':':женщина_с_поднятои_рукои_очень_светлыи_тон_кожи:','tr':':elini_kaldıran_kadın_açık_cilt_tonu:','ar':':سيدة_ترفع_يدها_بشرة_بلون_فاتح:'},'\U0001F64B\U0001F3FB\U0000200D\U00002640':{'en':':woman_raising_hand_light_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_erhobenem_arm_helle_hautfarbe:','es':':mujer_con_la_mano_levantada_tono_de_piel_claro:','fr':':femme_qui_lève_la_main_peau_claire:','ja':':手を挙げる女_薄い肌色:','ko':':한_손_든_여자_하얀_피부:','pt':':mulher_levantando_a_mão_pele_clara:','it':':donna_con_mano_alzata_carnagione_chiara:','fa':':زن_دستش_را_بالا_می_برد_پوست_سفید:','id':':wanita_mengangkat_tangan_warna_kulit_cerah:','zh':':女生举手_较浅肤色:','ru':':женщина_с_поднятой_рукой_очень_светлый_тон_кожи:','tr':':elini_kaldıran_kadın_açık_cilt_tonu:','ar':':سيدة_ترفع_يدها_بشرة_بلون_فاتح:'},'\U0001F64B\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_raising_hand_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_erhobenem_arm_mitteldunkle_hautfarbe:','es':':mujer_con_la_mano_levantada_tono_de_piel_oscuro_medio:','fr':':femme_qui_lève_la_main_peau_mate:','ja':':手を挙げる女_やや濃い肌色:','ko':':한_손_든_여자_진한_갈색_피부:','pt':':mulher_levantando_a_mão_pele_morena_escura:','it':':donna_con_mano_alzata_carnagione_abbastanza_scura:','fa':':زن_دستش_را_بالا_می_برد_پوست_گندمی:','id':':wanita_mengangkat_tangan_warna_kulit_gelap-sedang:','zh':':女生举手_中等深肤色:','ru':':женщина_с_поднятои_рукои_темныи_тон_кожи:','tr':':elini_kaldıran_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_ترفع_يدها_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F64B\U0001F3FE\U0000200D\U00002640':{'en':':woman_raising_hand_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_erhobenem_arm_mitteldunkle_hautfarbe:','es':':mujer_con_la_mano_levantada_tono_de_piel_oscuro_medio:','fr':':femme_qui_lève_la_main_peau_mate:','ja':':手を挙げる女_やや濃い肌色:','ko':':한_손_든_여자_진한_갈색_피부:','pt':':mulher_levantando_a_mão_pele_morena_escura:','it':':donna_con_mano_alzata_carnagione_abbastanza_scura:','fa':':زن_دستش_را_بالا_می_برد_پوست_گندمی:','id':':wanita_mengangkat_tangan_warna_kulit_gelap-sedang:','zh':':女生举手_中等深肤色:','ru':':женщина_с_поднятой_рукой_темный_тон_кожи:','tr':':elini_kaldıran_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_ترفع_يدها_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F64B\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_raising_hand_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_erhobenem_arm_mittelhelle_hautfarbe:','es':':mujer_con_la_mano_levantada_tono_de_piel_claro_medio:','fr':':femme_qui_lève_la_main_peau_moyennement_claire:','ja':':手を挙げる女_やや薄い肌色:','ko':':한_손_든_여자_연한_갈색_피부:','pt':':mulher_levantando_a_mão_pele_morena_clara:','it':':donna_con_mano_alzata_carnagione_abbastanza_chiara:','fa':':زن_دستش_را_بالا_می_برد_پوست_روشن:','id':':wanita_mengangkat_tangan_warna_kulit_cerah-sedang:','zh':':女生举手_中等浅肤色:','ru':':женщина_с_поднятои_рукои_светлыи_тон_кожи:','tr':':elini_kaldıran_kadın_orta_açık_cilt_tonu:','ar':':سيدة_ترفع_يدها_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F64B\U0001F3FC\U0000200D\U00002640':{'en':':woman_raising_hand_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_erhobenem_arm_mittelhelle_hautfarbe:','es':':mujer_con_la_mano_levantada_tono_de_piel_claro_medio:','fr':':femme_qui_lève_la_main_peau_moyennement_claire:','ja':':手を挙げる女_やや薄い肌色:','ko':':한_손_든_여자_연한_갈색_피부:','pt':':mulher_levantando_a_mão_pele_morena_clara:','it':':donna_con_mano_alzata_carnagione_abbastanza_chiara:','fa':':زن_دستش_را_بالا_می_برد_پوست_روشن:','id':':wanita_mengangkat_tangan_warna_kulit_cerah-sedang:','zh':':女生举手_中等浅肤色:','ru':':женщина_с_поднятой_рукой_светлый_тон_кожи:','tr':':elini_kaldıran_kadın_orta_açık_cilt_tonu:','ar':':سيدة_ترفع_يدها_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F64B\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_raising_hand_medium_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_erhobenem_arm_mittlere_hautfarbe:','es':':mujer_con_la_mano_levantada_tono_de_piel_medio:','fr':':femme_qui_lève_la_main_peau_légèrement_mate:','ja':':手を挙げる女_中間の肌色:','ko':':한_손_든_여자_갈색_피부:','pt':':mulher_levantando_a_mão_pele_morena:','it':':donna_con_mano_alzata_carnagione_olivastra:','fa':':زن_دستش_را_بالا_می_برد_پوست_طلایی:','id':':wanita_mengangkat_tangan_warna_kulit_sedang:','zh':':女生举手_中等肤色:','ru':':женщина_с_поднятои_рукои_среднии_тон_кожи:','tr':':elini_kaldıran_kadın_orta_cilt_tonu:','ar':':سيدة_ترفع_يدها_بشرة_بلون_معتدل:'},'\U0001F64B\U0001F3FD\U0000200D\U00002640':{'en':':woman_raising_hand_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_erhobenem_arm_mittlere_hautfarbe:','es':':mujer_con_la_mano_levantada_tono_de_piel_medio:','fr':':femme_qui_lève_la_main_peau_légèrement_mate:','ja':':手を挙げる女_中間の肌色:','ko':':한_손_든_여자_갈색_피부:','pt':':mulher_levantando_a_mão_pele_morena:','it':':donna_con_mano_alzata_carnagione_olivastra:','fa':':زن_دستش_را_بالا_می_برد_پوست_طلایی:','id':':wanita_mengangkat_tangan_warna_kulit_sedang:','zh':':女生举手_中等肤色:','ru':':женщина_с_поднятой_рукой_средний_тон_кожи:','tr':':elini_kaldıran_kadın_orta_cilt_tonu:','ar':':سيدة_ترفع_يدها_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U0001F9B0':{'en':':woman_red_hair:','status':fully_qualified,'E':11,'alias':[':red_haired_woman:'],'de':':frau_rotes_haar:','es':':mujer_pelo_pelirrojo:','fr':':femme_cheveux_roux:','ja':':女性_赤毛:','ko':':여자_빨간_머리:','pt':':mulher_cabelo_vermelho:','it':':donna_capelli_rossi:','fa':':زن_موقرمز:','id':':wanita_rambut_merah:','zh':':女人_红发:','ru':':женщина_рыжие_волосы:','tr':':kadın_kızıl_saç:','ar':':سيدة_شعر_أحمر:'},'\U0001F6A3\U0000200D\U00002640\U0000FE0F':{'en':':woman_rowing_boat:','status':fully_qualified,'E':4,'alias':[':rowing_woman:'],'de':':frau_im_ruderboot:','es':':mujer_remando_en_un_bote:','fr':':rameuse_dans_une_barque:','ja':':ボートをこぐ女:','ko':':노젓는_여자:','pt':':mulher_remando:','it':':donna_in_barca_a_remi:','fa':':پاروزن_زن:','id':':wanita_mendayung_perahu:','zh':':女生划船:','ru':':гребчиха:','tr':':sandalda_kürek_çeken_kadın:','ar':':سيدة_تجدف_بقارب:'},'\U0001F6A3\U0000200D\U00002640':{'en':':woman_rowing_boat:','status':minimally_qualified,'E':4,'alias':[':rowing_woman:'],'de':':frau_im_ruderboot:','es':':mujer_remando_en_un_bote:','fr':':rameuse_dans_une_barque:','ja':':ボートをこぐ女:','ko':':노젓는_여자:','pt':':mulher_remando:','it':':donna_in_barca_a_remi:','fa':':پاروزن_زن:','id':':wanita_mendayung_perahu:','zh':':女生划船:','ru':':гребчиха:','tr':':sandalda_kürek_çeken_kadın:','ar':':سيدة_تجدف_بقارب:'},'\U0001F6A3\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_rowing_boat_dark_skin_tone:','status':fully_qualified,'E':4,'de':':frau_im_ruderboot_dunkle_hautfarbe:','es':':mujer_remando_en_un_bote_tono_de_piel_oscuro:','fr':':rameuse_dans_une_barque_peau_foncée:','ja':':ボートをこぐ女_濃い肌色:','ko':':노젓는_여자_검은색_피부:','pt':':mulher_remando_pele_escura:','it':':donna_in_barca_a_remi_carnagione_scura:','fa':':پاروزن_زن_پوست_آبنوسی:','id':':wanita_mendayung_perahu_warna_kulit_gelap:','zh':':女生划船_较深肤色:','ru':':гребчиха_очень_темныи_тон_кожи:','tr':':sandalda_kürek_çeken_kadın_koyu_cilt_tonu:','ar':':سيدة_تجدف_بقارب_بشرة_بلون_غامق:'},'\U0001F6A3\U0001F3FF\U0000200D\U00002640':{'en':':woman_rowing_boat_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_im_ruderboot_dunkle_hautfarbe:','es':':mujer_remando_en_un_bote_tono_de_piel_oscuro:','fr':':rameuse_dans_une_barque_peau_foncée:','ja':':ボートをこぐ女_濃い肌色:','ko':':노젓는_여자_검은색_피부:','pt':':mulher_remando_pele_escura:','it':':donna_in_barca_a_remi_carnagione_scura:','fa':':پاروزن_زن_پوست_آبنوسی:','id':':wanita_mendayung_perahu_warna_kulit_gelap:','zh':':女生划船_较深肤色:','ru':':гребчиха_очень_темный_тон_кожи:','tr':':sandalda_kürek_çeken_kadın_koyu_cilt_tonu:','ar':':سيدة_تجدف_بقارب_بشرة_بلون_غامق:'},'\U0001F6A3\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_rowing_boat_light_skin_tone:','status':fully_qualified,'E':4,'de':':frau_im_ruderboot_helle_hautfarbe:','es':':mujer_remando_en_un_bote_tono_de_piel_claro:','fr':':rameuse_dans_une_barque_peau_claire:','ja':':ボートをこぐ女_薄い肌色:','ko':':노젓는_여자_하얀_피부:','pt':':mulher_remando_pele_clara:','it':':donna_in_barca_a_remi_carnagione_chiara:','fa':':پاروزن_زن_پوست_سفید:','id':':wanita_mendayung_perahu_warna_kulit_cerah:','zh':':女生划船_较浅肤色:','ru':':гребчиха_очень_светлыи_тон_кожи:','tr':':sandalda_kürek_çeken_kadın_açık_cilt_tonu:','ar':':سيدة_تجدف_بقارب_بشرة_بلون_فاتح:'},'\U0001F6A3\U0001F3FB\U0000200D\U00002640':{'en':':woman_rowing_boat_light_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_im_ruderboot_helle_hautfarbe:','es':':mujer_remando_en_un_bote_tono_de_piel_claro:','fr':':rameuse_dans_une_barque_peau_claire:','ja':':ボートをこぐ女_薄い肌色:','ko':':노젓는_여자_하얀_피부:','pt':':mulher_remando_pele_clara:','it':':donna_in_barca_a_remi_carnagione_chiara:','fa':':پاروزن_زن_پوست_سفید:','id':':wanita_mendayung_perahu_warna_kulit_cerah:','zh':':女生划船_较浅肤色:','ru':':гребчиха_очень_светлый_тон_кожи:','tr':':sandalda_kürek_çeken_kadın_açık_cilt_tonu:','ar':':سيدة_تجدف_بقارب_بشرة_بلون_فاتح:'},'\U0001F6A3\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_rowing_boat_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':frau_im_ruderboot_mitteldunkle_hautfarbe:','es':':mujer_remando_en_un_bote_tono_de_piel_oscuro_medio:','fr':':rameuse_dans_une_barque_peau_mate:','ja':':ボートをこぐ女_やや濃い肌色:','ko':':노젓는_여자_진한_갈색_피부:','pt':':mulher_remando_pele_morena_escura:','it':':donna_in_barca_a_remi_carnagione_abbastanza_scura:','fa':':پاروزن_زن_پوست_گندمی:','id':':wanita_mendayung_perahu_warna_kulit_gelap-sedang:','zh':':女生划船_中等深肤色:','ru':':гребчиха_темныи_тон_кожи:','tr':':sandalda_kürek_çeken_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_تجدف_بقارب_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6A3\U0001F3FE\U0000200D\U00002640':{'en':':woman_rowing_boat_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_im_ruderboot_mitteldunkle_hautfarbe:','es':':mujer_remando_en_un_bote_tono_de_piel_oscuro_medio:','fr':':rameuse_dans_une_barque_peau_mate:','ja':':ボートをこぐ女_やや濃い肌色:','ko':':노젓는_여자_진한_갈색_피부:','pt':':mulher_remando_pele_morena_escura:','it':':donna_in_barca_a_remi_carnagione_abbastanza_scura:','fa':':پاروزن_زن_پوست_گندمی:','id':':wanita_mendayung_perahu_warna_kulit_gelap-sedang:','zh':':女生划船_中等深肤色:','ru':':гребчиха_темный_тон_кожи:','tr':':sandalda_kürek_çeken_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_تجدف_بقارب_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6A3\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_rowing_boat_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':frau_im_ruderboot_mittelhelle_hautfarbe:','es':':mujer_remando_en_un_bote_tono_de_piel_claro_medio:','fr':':rameuse_dans_une_barque_peau_moyennement_claire:','ja':':ボートをこぐ女_やや薄い肌色:','ko':':노젓는_여자_연한_갈색_피부:','pt':':mulher_remando_pele_morena_clara:','it':':donna_in_barca_a_remi_carnagione_abbastanza_chiara:','fa':':پاروزن_زن_پوست_روشن:','id':':wanita_mendayung_perahu_warna_kulit_cerah-sedang:','zh':':女生划船_中等浅肤色:','ru':':гребчиха_светлыи_тон_кожи:','tr':':sandalda_kürek_çeken_kadın_orta_açık_cilt_tonu:','ar':':سيدة_تجدف_بقارب_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6A3\U0001F3FC\U0000200D\U00002640':{'en':':woman_rowing_boat_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_im_ruderboot_mittelhelle_hautfarbe:','es':':mujer_remando_en_un_bote_tono_de_piel_claro_medio:','fr':':rameuse_dans_une_barque_peau_moyennement_claire:','ja':':ボートをこぐ女_やや薄い肌色:','ko':':노젓는_여자_연한_갈색_피부:','pt':':mulher_remando_pele_morena_clara:','it':':donna_in_barca_a_remi_carnagione_abbastanza_chiara:','fa':':پاروزن_زن_پوست_روشن:','id':':wanita_mendayung_perahu_warna_kulit_cerah-sedang:','zh':':女生划船_中等浅肤色:','ru':':гребчиха_светлый_тон_кожи:','tr':':sandalda_kürek_çeken_kadın_orta_açık_cilt_tonu:','ar':':سيدة_تجدف_بقارب_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6A3\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_rowing_boat_medium_skin_tone:','status':fully_qualified,'E':4,'de':':frau_im_ruderboot_mittlere_hautfarbe:','es':':mujer_remando_en_un_bote_tono_de_piel_medio:','fr':':rameuse_dans_une_barque_peau_légèrement_mate:','ja':':ボートをこぐ女_中間の肌色:','ko':':노젓는_여자_갈색_피부:','pt':':mulher_remando_pele_morena:','it':':donna_in_barca_a_remi_carnagione_olivastra:','fa':':پاروزن_زن_پوست_طلایی:','id':':wanita_mendayung_perahu_warna_kulit_sedang:','zh':':女生划船_中等肤色:','ru':':гребчиха_среднии_тон_кожи:','tr':':sandalda_kürek_çeken_kadın_orta_cilt_tonu:','ar':':سيدة_تجدف_بقارب_بشرة_بلون_معتدل:'},'\U0001F6A3\U0001F3FD\U0000200D\U00002640':{'en':':woman_rowing_boat_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_im_ruderboot_mittlere_hautfarbe:','es':':mujer_remando_en_un_bote_tono_de_piel_medio:','fr':':rameuse_dans_une_barque_peau_légèrement_mate:','ja':':ボートをこぐ女_中間の肌色:','ko':':노젓는_여자_갈색_피부:','pt':':mulher_remando_pele_morena:','it':':donna_in_barca_a_remi_carnagione_olivastra:','fa':':پاروزن_زن_پوست_طلایی:','id':':wanita_mendayung_perahu_warna_kulit_sedang:','zh':':女生划船_中等肤色:','ru':':гребчиха_средний_тон_кожи:','tr':':sandalda_kürek_çeken_kadın_orta_cilt_tonu:','ar':':سيدة_تجدف_بقارب_بشرة_بلون_معتدل:'},'\U0001F3C3\U0000200D\U00002640\U0000FE0F':{'en':':woman_running:','status':fully_qualified,'E':4,'alias':[':running_woman:'],'de':':laufende_frau:','es':':mujer_corriendo:','fr':':femme_qui_court:','ja':':走る女:','ko':':뛰는_여자:','pt':':mulher_correndo:','it':':donna_che_corre:','fa':':زن_دونده:','id':':wanita_berlari:','zh':':女生跑步:','ru':':женщина_бежит:','tr':':koşan_kadın:','ar':':راكضة:'},'\U0001F3C3\U0000200D\U00002640':{'en':':woman_running:','status':minimally_qualified,'E':4,'alias':[':running_woman:'],'de':':laufende_frau:','es':':mujer_corriendo:','fr':':femme_qui_court:','ja':':走る女:','ko':':뛰는_여자:','pt':':mulher_correndo:','it':':donna_che_corre:','fa':':زن_دونده:','id':':wanita_berlari:','zh':':女生跑步:','ru':':женщина_бежит:','tr':':koşan_kadın:','ar':':راكضة:'},'\U0001F3C3\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_running_dark_skin_tone:','status':fully_qualified,'E':4,'de':':laufende_frau_dunkle_hautfarbe:','es':':mujer_corriendo_tono_de_piel_oscuro:','fr':':femme_qui_court_peau_foncée:','ja':':走る女_濃い肌色:','ko':':뛰는_여자_검은색_피부:','pt':':mulher_correndo_pele_escura:','it':':donna_che_corre_carnagione_scura:','fa':':زن_دونده_پوست_آبنوسی:','id':':wanita_berlari_warna_kulit_gelap:','zh':':女生跑步_较深肤色:','ru':':женщина_бежит_очень_темныи_тон_кожи:','tr':':koşan_kadın_koyu_cilt_tonu:','ar':':راكضة_بشرة_بلون_غامق:'},'\U0001F3C3\U0001F3FF\U0000200D\U00002640':{'en':':woman_running_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':laufende_frau_dunkle_hautfarbe:','es':':mujer_corriendo_tono_de_piel_oscuro:','fr':':femme_qui_court_peau_foncée:','ja':':走る女_濃い肌色:','ko':':뛰는_여자_검은색_피부:','pt':':mulher_correndo_pele_escura:','it':':donna_che_corre_carnagione_scura:','fa':':زن_دونده_پوست_آبنوسی:','id':':wanita_berlari_warna_kulit_gelap:','zh':':女生跑步_较深肤色:','ru':':женщина_бежит_очень_темный_тон_кожи:','tr':':koşan_kadın_koyu_cilt_tonu:','ar':':راكضة_بشرة_بلون_غامق:'},'\U0001F3C3\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':woman_running_facing_right:','status':fully_qualified,'E':15.1,'de':':laufende_frau_nach_rechts:','es':':mujer_corriendo_hacia_la_derecha:','fr':':femme_qui_court_vers_la_droite:','ja':':走る女_右向き:','ko':':오른쪽을_향한_뛰는_여자:','pt':':mulher_correndo_de_frente_para_a_direita:','it':':donna_che_corre_a_destra:','fa':':زن_دونده_به_سمت_راست:','id':':wanita_berlari_menghadap_kanan:','zh':':女生跑步面向右边:','ru':':женщина_бежит_направленные_вправо:','tr':':sağa_dönük_koşan_kadın:','ar':':راكضة_متجه_لليمين:'},'\U0001F3C3\U0000200D\U00002640\U0000200D\U000027A1\U0000FE0F':{'en':':woman_running_facing_right:','status':minimally_qualified,'E':15.1,'de':':laufende_frau_nach_rechts:','es':':mujer_corriendo_hacia_la_derecha:','fr':':femme_qui_court_vers_la_droite:','ja':':走る女_右向き:','ko':':오른쪽을_향한_뛰는_여자:','pt':':mulher_correndo_de_frente_para_a_direita:','it':':donna_che_corre_a_destra:','fa':':زن_دونده_به_سمت_راست:','id':':wanita_berlari_menghadap_kanan:','zh':':女生跑步面向右边:','ru':':женщина_бежит_направленные_вправо:','tr':':sağa_dönük_koşan_kadın:','ar':':راكضة_متجه_لليمين:'},'\U0001F3C3\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1':{'en':':woman_running_facing_right:','status':minimally_qualified,'E':15.1,'de':':laufende_frau_nach_rechts:','es':':mujer_corriendo_hacia_la_derecha:','fr':':femme_qui_court_vers_la_droite:','ja':':走る女_右向き:','ko':':오른쪽을_향한_뛰는_여자:','pt':':mulher_correndo_de_frente_para_a_direita:','it':':donna_che_corre_a_destra:','fa':':زن_دونده_به_سمت_راست:','id':':wanita_berlari_menghadap_kanan:','zh':':女生跑步面向右边:','ru':':женщина_бежит_направленные_вправо:','tr':':sağa_dönük_koşan_kadın:','ar':':راكضة_متجه_لليمين:'},'\U0001F3C3\U0000200D\U00002640\U0000200D\U000027A1':{'en':':woman_running_facing_right:','status':minimally_qualified,'E':15.1,'de':':laufende_frau_nach_rechts:','es':':mujer_corriendo_hacia_la_derecha:','fr':':femme_qui_court_vers_la_droite:','ja':':走る女_右向き:','ko':':오른쪽을_향한_뛰는_여자:','pt':':mulher_correndo_de_frente_para_a_direita:','it':':donna_che_corre_a_destra:','fa':':زن_دونده_به_سمت_راست:','id':':wanita_berlari_menghadap_kanan:','zh':':女生跑步面向右边:','ru':':женщина_бежит_направленные_вправо:','tr':':sağa_dönük_koşan_kadın:','ar':':راكضة_متجه_لليمين:'},'\U0001F3C3\U0001F3FF\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':woman_running_facing_right_dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':laufende_frau_nach_rechts_dunkle_hautfarbe:','es':':mujer_corriendo_hacia_la_derecha_tono_de_piel_oscuro:','fr':':femme_qui_court_vers_la_droite_peau_foncée:','ja':':走る女_右向き_濃い肌色:','ko':':오른쪽을_향한_뛰는_여자_검은색_피부:','pt':':mulher_correndo_de_frente_para_a_direita_pele_escura:','it':':donna_che_corre_a_destra_carnagione_scura:','fa':':زن_دونده_به_سمت_راستپوست_آبنوسی:','id':':wanita_berlari_menghadap_kanan_warna_kulit_gelap:','zh':':女生跑步面向右边_较深肤色:','ru':':женщина_бежит_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_koşan_kadın_koyu_cilt_tonu:','ar':':راكضة_متجه_لليمين_بلون_غامق:'},'\U0001F3C3\U0001F3FF\U0000200D\U00002640\U0000200D\U000027A1\U0000FE0F':{'en':':woman_running_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufende_frau_nach_rechts_dunkle_hautfarbe:','es':':mujer_corriendo_hacia_la_derecha_tono_de_piel_oscuro:','fr':':femme_qui_court_vers_la_droite_peau_foncée:','ja':':走る女_右向き_濃い肌色:','ko':':오른쪽을_향한_뛰는_여자_검은색_피부:','pt':':mulher_correndo_de_frente_para_a_direita_pele_escura:','it':':donna_che_corre_a_destra_carnagione_scura:','fa':':زن_دونده_به_سمت_راستپوست_آبنوسی:','id':':wanita_berlari_menghadap_kanan_warna_kulit_gelap:','zh':':女生跑步面向右边_较深肤色:','ru':':женщина_бежит_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_koşan_kadın_koyu_cilt_tonu:','ar':':راكضة_متجه_لليمين_بلون_غامق:'},'\U0001F3C3\U0001F3FF\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1':{'en':':woman_running_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufende_frau_nach_rechts_dunkle_hautfarbe:','es':':mujer_corriendo_hacia_la_derecha_tono_de_piel_oscuro:','fr':':femme_qui_court_vers_la_droite_peau_foncée:','ja':':走る女_右向き_濃い肌色:','ko':':오른쪽을_향한_뛰는_여자_검은색_피부:','pt':':mulher_correndo_de_frente_para_a_direita_pele_escura:','it':':donna_che_corre_a_destra_carnagione_scura:','fa':':زن_دونده_به_سمت_راستپوست_آبنوسی:','id':':wanita_berlari_menghadap_kanan_warna_kulit_gelap:','zh':':女生跑步面向右边_较深肤色:','ru':':женщина_бежит_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_koşan_kadın_koyu_cilt_tonu:','ar':':راكضة_متجه_لليمين_بلون_غامق:'},'\U0001F3C3\U0001F3FF\U0000200D\U00002640\U0000200D\U000027A1':{'en':':woman_running_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufende_frau_nach_rechts_dunkle_hautfarbe:','es':':mujer_corriendo_hacia_la_derecha_tono_de_piel_oscuro:','fr':':femme_qui_court_vers_la_droite_peau_foncée:','ja':':走る女_右向き_濃い肌色:','ko':':오른쪽을_향한_뛰는_여자_검은색_피부:','pt':':mulher_correndo_de_frente_para_a_direita_pele_escura:','it':':donna_che_corre_a_destra_carnagione_scura:','fa':':زن_دونده_به_سمت_راستپوست_آبنوسی:','id':':wanita_berlari_menghadap_kanan_warna_kulit_gelap:','zh':':女生跑步面向右边_较深肤色:','ru':':женщина_бежит_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_koşan_kadın_koyu_cilt_tonu:','ar':':راكضة_متجه_لليمين_بلون_غامق:'},'\U0001F3C3\U0001F3FB\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':woman_running_facing_right_light_skin_tone:','status':fully_qualified,'E':15.1,'de':':laufende_frau_nach_rechts_helle_hautfarbe:','es':':mujer_corriendo_hacia_la_derecha_tono_de_piel_claro:','fr':':femme_qui_court_vers_la_droite_peau_claire:','ja':':走る女_右向き_薄い肌色:','ko':':오른쪽을_향한_뛰는_여자_하얀_피부:','pt':':mulher_correndo_de_frente_para_a_direita_pele_clara:','it':':donna_che_corre_a_destra_carnagione_chiara:','fa':':زن_دونده_به_سمت_راست_پوست_سفید:','id':':wanita_berlari_menghadap_kanan_warna_kulit_cerah:','zh':':女生跑步面向右边_较浅肤色:','ru':':женщина_бежит_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_koşan_kadın_açık_cilt_tonu:','ar':':راكضة_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F3C3\U0001F3FB\U0000200D\U00002640\U0000200D\U000027A1\U0000FE0F':{'en':':woman_running_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufende_frau_nach_rechts_helle_hautfarbe:','es':':mujer_corriendo_hacia_la_derecha_tono_de_piel_claro:','fr':':femme_qui_court_vers_la_droite_peau_claire:','ja':':走る女_右向き_薄い肌色:','ko':':오른쪽을_향한_뛰는_여자_하얀_피부:','pt':':mulher_correndo_de_frente_para_a_direita_pele_clara:','it':':donna_che_corre_a_destra_carnagione_chiara:','fa':':زن_دونده_به_سمت_راست_پوست_سفید:','id':':wanita_berlari_menghadap_kanan_warna_kulit_cerah:','zh':':女生跑步面向右边_较浅肤色:','ru':':женщина_бежит_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_koşan_kadın_açık_cilt_tonu:','ar':':راكضة_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F3C3\U0001F3FB\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1':{'en':':woman_running_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufende_frau_nach_rechts_helle_hautfarbe:','es':':mujer_corriendo_hacia_la_derecha_tono_de_piel_claro:','fr':':femme_qui_court_vers_la_droite_peau_claire:','ja':':走る女_右向き_薄い肌色:','ko':':오른쪽을_향한_뛰는_여자_하얀_피부:','pt':':mulher_correndo_de_frente_para_a_direita_pele_clara:','it':':donna_che_corre_a_destra_carnagione_chiara:','fa':':زن_دونده_به_سمت_راست_پوست_سفید:','id':':wanita_berlari_menghadap_kanan_warna_kulit_cerah:','zh':':女生跑步面向右边_较浅肤色:','ru':':женщина_бежит_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_koşan_kadın_açık_cilt_tonu:','ar':':راكضة_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F3C3\U0001F3FB\U0000200D\U00002640\U0000200D\U000027A1':{'en':':woman_running_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufende_frau_nach_rechts_helle_hautfarbe:','es':':mujer_corriendo_hacia_la_derecha_tono_de_piel_claro:','fr':':femme_qui_court_vers_la_droite_peau_claire:','ja':':走る女_右向き_薄い肌色:','ko':':오른쪽을_향한_뛰는_여자_하얀_피부:','pt':':mulher_correndo_de_frente_para_a_direita_pele_clara:','it':':donna_che_corre_a_destra_carnagione_chiara:','fa':':زن_دونده_به_سمت_راست_پوست_سفید:','id':':wanita_berlari_menghadap_kanan_warna_kulit_cerah:','zh':':女生跑步面向右边_较浅肤色:','ru':':женщина_бежит_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_koşan_kadın_açık_cilt_tonu:','ar':':راكضة_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F3C3\U0001F3FE\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':woman_running_facing_right_medium-dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':laufende_frau_nach_rechts_mitteldunkle_hautfarbe:','es':':mujer_corriendo_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':femme_qui_court_vers_la_droite_peau_mate:','ja':':走る女_右向き_やや濃い肌色:','ko':':오른쪽을_향한_뛰는_여자_진한_갈색_피부:','pt':':mulher_correndo_de_frente_para_a_direita_pele_morena_escura:','it':':donna_che_corre_a_destra_carnagione_abbastanza_scura:','fa':':زن_دونده_به_سمت_راست_پوست_گندمی:','id':':wanita_berlari_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':女生跑步面向右边_中等深肤色:','ru':':женщина_бежит_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_koşan_kadın_orta_koyu_cilt_tonu:','ar':':راكضة_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F3C3\U0001F3FE\U0000200D\U00002640\U0000200D\U000027A1\U0000FE0F':{'en':':woman_running_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufende_frau_nach_rechts_mitteldunkle_hautfarbe:','es':':mujer_corriendo_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':femme_qui_court_vers_la_droite_peau_mate:','ja':':走る女_右向き_やや濃い肌色:','ko':':오른쪽을_향한_뛰는_여자_진한_갈색_피부:','pt':':mulher_correndo_de_frente_para_a_direita_pele_morena_escura:','it':':donna_che_corre_a_destra_carnagione_abbastanza_scura:','fa':':زن_دونده_به_سمت_راست_پوست_گندمی:','id':':wanita_berlari_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':女生跑步面向右边_中等深肤色:','ru':':женщина_бежит_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_koşan_kadın_orta_koyu_cilt_tonu:','ar':':راكضة_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F3C3\U0001F3FE\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1':{'en':':woman_running_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufende_frau_nach_rechts_mitteldunkle_hautfarbe:','es':':mujer_corriendo_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':femme_qui_court_vers_la_droite_peau_mate:','ja':':走る女_右向き_やや濃い肌色:','ko':':오른쪽을_향한_뛰는_여자_진한_갈색_피부:','pt':':mulher_correndo_de_frente_para_a_direita_pele_morena_escura:','it':':donna_che_corre_a_destra_carnagione_abbastanza_scura:','fa':':زن_دونده_به_سمت_راست_پوست_گندمی:','id':':wanita_berlari_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':女生跑步面向右边_中等深肤色:','ru':':женщина_бежит_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_koşan_kadın_orta_koyu_cilt_tonu:','ar':':راكضة_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F3C3\U0001F3FE\U0000200D\U00002640\U0000200D\U000027A1':{'en':':woman_running_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufende_frau_nach_rechts_mitteldunkle_hautfarbe:','es':':mujer_corriendo_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':femme_qui_court_vers_la_droite_peau_mate:','ja':':走る女_右向き_やや濃い肌色:','ko':':오른쪽을_향한_뛰는_여자_진한_갈색_피부:','pt':':mulher_correndo_de_frente_para_a_direita_pele_morena_escura:','it':':donna_che_corre_a_destra_carnagione_abbastanza_scura:','fa':':زن_دونده_به_سمت_راست_پوست_گندمی:','id':':wanita_berlari_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':女生跑步面向右边_中等深肤色:','ru':':женщина_бежит_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_koşan_kadın_orta_koyu_cilt_tonu:','ar':':راكضة_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F3C3\U0001F3FC\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':woman_running_facing_right_medium-light_skin_tone:','status':fully_qualified,'E':15.1,'de':':laufende_frau_nach_rechts_mittelhelle_hautfarbe:','es':':mujer_corriendo_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':femme_qui_court_vers_la_droite_peau_moyennement_claire:','ja':':走る女_右向き_やや薄い肌色:','ko':':오른쪽을_향한_뛰는_여자_연한_갈색_피부:','pt':':mulher_correndo_de_frente_para_a_direita_pele_morena_clara:','it':':donna_che_corre_a_destra_carnagione_abbastanza_chiara:','fa':':زن_دونده_به_سمت_راست_پوست_روشن:','id':':wanita_berlari_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':女生跑步面向右边_中等浅肤色:','ru':':женщина_бежит_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_koşan_kadın_orta_açık_cilt_tonu:','ar':':راكضة_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F3C3\U0001F3FC\U0000200D\U00002640\U0000200D\U000027A1\U0000FE0F':{'en':':woman_running_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufende_frau_nach_rechts_mittelhelle_hautfarbe:','es':':mujer_corriendo_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':femme_qui_court_vers_la_droite_peau_moyennement_claire:','ja':':走る女_右向き_やや薄い肌色:','ko':':오른쪽을_향한_뛰는_여자_연한_갈색_피부:','pt':':mulher_correndo_de_frente_para_a_direita_pele_morena_clara:','it':':donna_che_corre_a_destra_carnagione_abbastanza_chiara:','fa':':زن_دونده_به_سمت_راست_پوست_روشن:','id':':wanita_berlari_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':女生跑步面向右边_中等浅肤色:','ru':':женщина_бежит_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_koşan_kadın_orta_açık_cilt_tonu:','ar':':راكضة_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F3C3\U0001F3FC\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1':{'en':':woman_running_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufende_frau_nach_rechts_mittelhelle_hautfarbe:','es':':mujer_corriendo_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':femme_qui_court_vers_la_droite_peau_moyennement_claire:','ja':':走る女_右向き_やや薄い肌色:','ko':':오른쪽을_향한_뛰는_여자_연한_갈색_피부:','pt':':mulher_correndo_de_frente_para_a_direita_pele_morena_clara:','it':':donna_che_corre_a_destra_carnagione_abbastanza_chiara:','fa':':زن_دونده_به_سمت_راست_پوست_روشن:','id':':wanita_berlari_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':女生跑步面向右边_中等浅肤色:','ru':':женщина_бежит_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_koşan_kadın_orta_açık_cilt_tonu:','ar':':راكضة_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F3C3\U0001F3FC\U0000200D\U00002640\U0000200D\U000027A1':{'en':':woman_running_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufende_frau_nach_rechts_mittelhelle_hautfarbe:','es':':mujer_corriendo_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':femme_qui_court_vers_la_droite_peau_moyennement_claire:','ja':':走る女_右向き_やや薄い肌色:','ko':':오른쪽을_향한_뛰는_여자_연한_갈색_피부:','pt':':mulher_correndo_de_frente_para_a_direita_pele_morena_clara:','it':':donna_che_corre_a_destra_carnagione_abbastanza_chiara:','fa':':زن_دونده_به_سمت_راست_پوست_روشن:','id':':wanita_berlari_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':女生跑步面向右边_中等浅肤色:','ru':':женщина_бежит_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_koşan_kadın_orta_açık_cilt_tonu:','ar':':راكضة_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F3C3\U0001F3FD\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':woman_running_facing_right_medium_skin_tone:','status':fully_qualified,'E':15.1,'de':':laufende_frau_nach_rechts_mittlere_hautfarbe:','es':':mujer_corriendo_hacia_la_derecha_tono_de_piel_medio:','fr':':femme_qui_court_vers_la_droite_peau_légèrement_mate:','ja':':走る女_右向き_中間の肌色:','ko':':오른쪽을_향한_뛰는_여자_갈색_피부:','pt':':mulher_correndo_de_frente_para_a_direita_pele_morena:','it':':donna_che_corre_a_destra_carnagione_olivastra:','fa':':زن_دونده_به_سمت_راست_پوست_طلایی:','id':':wanita_berlari_menghadap_kanan_warna_kulit_sedang:','zh':':女生跑步面向右边_中等肤色:','ru':':женщина_бежит_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_koşan_kadın_orta_cilt_tonu:','ar':':راكضة_متجه_لليمين_بلون_معتدل:'},'\U0001F3C3\U0001F3FD\U0000200D\U00002640\U0000200D\U000027A1\U0000FE0F':{'en':':woman_running_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufende_frau_nach_rechts_mittlere_hautfarbe:','es':':mujer_corriendo_hacia_la_derecha_tono_de_piel_medio:','fr':':femme_qui_court_vers_la_droite_peau_légèrement_mate:','ja':':走る女_右向き_中間の肌色:','ko':':오른쪽을_향한_뛰는_여자_갈색_피부:','pt':':mulher_correndo_de_frente_para_a_direita_pele_morena:','it':':donna_che_corre_a_destra_carnagione_olivastra:','fa':':زن_دونده_به_سمت_راست_پوست_طلایی:','id':':wanita_berlari_menghadap_kanan_warna_kulit_sedang:','zh':':女生跑步面向右边_中等肤色:','ru':':женщина_бежит_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_koşan_kadın_orta_cilt_tonu:','ar':':راكضة_متجه_لليمين_بلون_معتدل:'},'\U0001F3C3\U0001F3FD\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1':{'en':':woman_running_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufende_frau_nach_rechts_mittlere_hautfarbe:','es':':mujer_corriendo_hacia_la_derecha_tono_de_piel_medio:','fr':':femme_qui_court_vers_la_droite_peau_légèrement_mate:','ja':':走る女_右向き_中間の肌色:','ko':':오른쪽을_향한_뛰는_여자_갈색_피부:','pt':':mulher_correndo_de_frente_para_a_direita_pele_morena:','it':':donna_che_corre_a_destra_carnagione_olivastra:','fa':':زن_دونده_به_سمت_راست_پوست_طلایی:','id':':wanita_berlari_menghadap_kanan_warna_kulit_sedang:','zh':':女生跑步面向右边_中等肤色:','ru':':женщина_бежит_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_koşan_kadın_orta_cilt_tonu:','ar':':راكضة_متجه_لليمين_بلون_معتدل:'},'\U0001F3C3\U0001F3FD\U0000200D\U00002640\U0000200D\U000027A1':{'en':':woman_running_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':laufende_frau_nach_rechts_mittlere_hautfarbe:','es':':mujer_corriendo_hacia_la_derecha_tono_de_piel_medio:','fr':':femme_qui_court_vers_la_droite_peau_légèrement_mate:','ja':':走る女_右向き_中間の肌色:','ko':':오른쪽을_향한_뛰는_여자_갈색_피부:','pt':':mulher_correndo_de_frente_para_a_direita_pele_morena:','it':':donna_che_corre_a_destra_carnagione_olivastra:','fa':':زن_دونده_به_سمت_راست_پوست_طلایی:','id':':wanita_berlari_menghadap_kanan_warna_kulit_sedang:','zh':':女生跑步面向右边_中等肤色:','ru':':женщина_бежит_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_koşan_kadın_orta_cilt_tonu:','ar':':راكضة_متجه_لليمين_بلون_معتدل:'},'\U0001F3C3\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_running_light_skin_tone:','status':fully_qualified,'E':4,'de':':laufende_frau_helle_hautfarbe:','es':':mujer_corriendo_tono_de_piel_claro:','fr':':femme_qui_court_peau_claire:','ja':':走る女_薄い肌色:','ko':':뛰는_여자_하얀_피부:','pt':':mulher_correndo_pele_clara:','it':':donna_che_corre_carnagione_chiara:','fa':':زن_دونده_پوست_سفید:','id':':wanita_berlari_warna_kulit_cerah:','zh':':女生跑步_较浅肤色:','ru':':женщина_бежит_очень_светлыи_тон_кожи:','tr':':koşan_kadın_açık_cilt_tonu:','ar':':راكضة_بشرة_بلون_فاتح:'},'\U0001F3C3\U0001F3FB\U0000200D\U00002640':{'en':':woman_running_light_skin_tone:','status':minimally_qualified,'E':4,'de':':laufende_frau_helle_hautfarbe:','es':':mujer_corriendo_tono_de_piel_claro:','fr':':femme_qui_court_peau_claire:','ja':':走る女_薄い肌色:','ko':':뛰는_여자_하얀_피부:','pt':':mulher_correndo_pele_clara:','it':':donna_che_corre_carnagione_chiara:','fa':':زن_دونده_پوست_سفید:','id':':wanita_berlari_warna_kulit_cerah:','zh':':女生跑步_较浅肤色:','ru':':женщина_бежит_очень_светлый_тон_кожи:','tr':':koşan_kadın_açık_cilt_tonu:','ar':':راكضة_بشرة_بلون_فاتح:'},'\U0001F3C3\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_running_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':laufende_frau_mitteldunkle_hautfarbe:','es':':mujer_corriendo_tono_de_piel_oscuro_medio:','fr':':femme_qui_court_peau_mate:','ja':':走る女_やや濃い肌色:','ko':':뛰는_여자_진한_갈색_피부:','pt':':mulher_correndo_pele_morena_escura:','it':':donna_che_corre_carnagione_abbastanza_scura:','fa':':زن_دونده_پوست_گندمی:','id':':wanita_berlari_warna_kulit_gelap-sedang:','zh':':女生跑步_中等深肤色:','ru':':женщина_бежит_темныи_тон_кожи:','tr':':koşan_kadın_orta_koyu_cilt_tonu:','ar':':راكضة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3C3\U0001F3FE\U0000200D\U00002640':{'en':':woman_running_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':laufende_frau_mitteldunkle_hautfarbe:','es':':mujer_corriendo_tono_de_piel_oscuro_medio:','fr':':femme_qui_court_peau_mate:','ja':':走る女_やや濃い肌色:','ko':':뛰는_여자_진한_갈색_피부:','pt':':mulher_correndo_pele_morena_escura:','it':':donna_che_corre_carnagione_abbastanza_scura:','fa':':زن_دونده_پوست_گندمی:','id':':wanita_berlari_warna_kulit_gelap-sedang:','zh':':女生跑步_中等深肤色:','ru':':женщина_бежит_темный_тон_кожи:','tr':':koşan_kadın_orta_koyu_cilt_tonu:','ar':':راكضة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3C3\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_running_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':laufende_frau_mittelhelle_hautfarbe:','es':':mujer_corriendo_tono_de_piel_claro_medio:','fr':':femme_qui_court_peau_moyennement_claire:','ja':':走る女_やや薄い肌色:','ko':':뛰는_여자_연한_갈색_피부:','pt':':mulher_correndo_pele_morena_clara:','it':':donna_che_corre_carnagione_abbastanza_chiara:','fa':':زن_دونده_پوست_روشن:','id':':wanita_berlari_warna_kulit_cerah-sedang:','zh':':女生跑步_中等浅肤色:','ru':':женщина_бежит_светлыи_тон_кожи:','tr':':koşan_kadın_orta_açık_cilt_tonu:','ar':':راكضة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3C3\U0001F3FC\U0000200D\U00002640':{'en':':woman_running_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':laufende_frau_mittelhelle_hautfarbe:','es':':mujer_corriendo_tono_de_piel_claro_medio:','fr':':femme_qui_court_peau_moyennement_claire:','ja':':走る女_やや薄い肌色:','ko':':뛰는_여자_연한_갈색_피부:','pt':':mulher_correndo_pele_morena_clara:','it':':donna_che_corre_carnagione_abbastanza_chiara:','fa':':زن_دونده_پوست_روشن:','id':':wanita_berlari_warna_kulit_cerah-sedang:','zh':':女生跑步_中等浅肤色:','ru':':женщина_бежит_светлый_тон_кожи:','tr':':koşan_kadın_orta_açık_cilt_tonu:','ar':':راكضة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3C3\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_running_medium_skin_tone:','status':fully_qualified,'E':4,'de':':laufende_frau_mittlere_hautfarbe:','es':':mujer_corriendo_tono_de_piel_medio:','fr':':femme_qui_court_peau_légèrement_mate:','ja':':走る女_中間の肌色:','ko':':뛰는_여자_갈색_피부:','pt':':mulher_correndo_pele_morena:','it':':donna_che_corre_carnagione_olivastra:','fa':':زن_دونده_پوست_طلایی:','id':':wanita_berlari_warna_kulit_sedang:','zh':':女生跑步_中等肤色:','ru':':женщина_бежит_среднии_тон_кожи:','tr':':koşan_kadın_orta_cilt_tonu:','ar':':راكضة_بشرة_بلون_معتدل:'},'\U0001F3C3\U0001F3FD\U0000200D\U00002640':{'en':':woman_running_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':laufende_frau_mittlere_hautfarbe:','es':':mujer_corriendo_tono_de_piel_medio:','fr':':femme_qui_court_peau_légèrement_mate:','ja':':走る女_中間の肌色:','ko':':뛰는_여자_갈색_피부:','pt':':mulher_correndo_pele_morena:','it':':donna_che_corre_carnagione_olivastra:','fa':':زن_دونده_پوست_طلایی:','id':':wanita_berlari_warna_kulit_sedang:','zh':':女生跑步_中等肤色:','ru':':женщина_бежит_средний_тон_кожи:','tr':':koşan_kadın_orta_cilt_tonu:','ar':':راكضة_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U0001F52C':{'en':':woman_scientist:','status':fully_qualified,'E':4,'de':':wissenschaftlerin:','es':':profesional_de_la_ciencia_mujer:','fr':':scientifique_femme:','ja':':女性科学者:','ko':':여자_과학자:','pt':':cientista_mulher:','it':':scienziata:','fa':':دانشمند_زن:','id':':ilmuwan_wanita:','zh':':女科学家:','ru':':ученая:','tr':':kadın_bilim_insanı:','ar':':عالِمة:'},'\U0001F469\U0001F3FF\U0000200D\U0001F52C':{'en':':woman_scientist_dark_skin_tone:','status':fully_qualified,'E':4,'de':':wissenschaftlerin_dunkle_hautfarbe:','es':':profesional_de_la_ciencia_mujer_tono_de_piel_oscuro:','fr':':scientifique_femme_peau_foncée:','ja':':女性科学者_濃い肌色:','ko':':여자_과학자_검은색_피부:','pt':':cientista_mulher_pele_escura:','it':':scienziata_carnagione_scura:','fa':':دانشمند_زن_پوست_آبنوسی:','id':':ilmuwan_wanita_warna_kulit_gelap:','zh':':女科学家_较深肤色:','ru':':ученая_очень_темный_тон_кожи:','tr':':kadın_bilim_insanı_koyu_cilt_tonu:','ar':':عالِمة_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F52C':{'en':':woman_scientist_light_skin_tone:','status':fully_qualified,'E':4,'de':':wissenschaftlerin_helle_hautfarbe:','es':':profesional_de_la_ciencia_mujer_tono_de_piel_claro:','fr':':scientifique_femme_peau_claire:','ja':':女性科学者_薄い肌色:','ko':':여자_과학자_하얀_피부:','pt':':cientista_mulher_pele_clara:','it':':scienziata_carnagione_chiara:','fa':':دانشمند_زن_پوست_سفید:','id':':ilmuwan_wanita_warna_kulit_cerah:','zh':':女科学家_较浅肤色:','ru':':ученая_очень_светлый_тон_кожи:','tr':':kadın_bilim_insanı_açık_cilt_tonu:','ar':':عالِمة_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F52C':{'en':':woman_scientist_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':wissenschaftlerin_mitteldunkle_hautfarbe:','es':':profesional_de_la_ciencia_mujer_tono_de_piel_oscuro_medio:','fr':':scientifique_femme_peau_mate:','ja':':女性科学者_やや濃い肌色:','ko':':여자_과학자_진한_갈색_피부:','pt':':cientista_mulher_pele_morena_escura:','it':':scienziata_carnagione_abbastanza_scura:','fa':':دانشمند_زن_پوست_گندمی:','id':':ilmuwan_wanita_warna_kulit_gelap-sedang:','zh':':女科学家_中等深肤色:','ru':':ученая_темный_тон_кожи:','tr':':kadın_bilim_insanı_orta_koyu_cilt_tonu:','ar':':عالِمة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F52C':{'en':':woman_scientist_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':wissenschaftlerin_mittelhelle_hautfarbe:','es':':profesional_de_la_ciencia_mujer_tono_de_piel_claro_medio:','fr':':scientifique_femme_peau_moyennement_claire:','ja':':女性科学者_やや薄い肌色:','ko':':여자_과학자_연한_갈색_피부:','pt':':cientista_mulher_pele_morena_clara:','it':':scienziata_carnagione_abbastanza_chiara:','fa':':دانشمند_زن_پوست_روشن:','id':':ilmuwan_wanita_warna_kulit_cerah-sedang:','zh':':女科学家_中等浅肤色:','ru':':ученая_светлый_тон_кожи:','tr':':kadın_bilim_insanı_orta_açık_cilt_tonu:','ar':':عالِمة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F52C':{'en':':woman_scientist_medium_skin_tone:','status':fully_qualified,'E':4,'de':':wissenschaftlerin_mittlere_hautfarbe:','es':':profesional_de_la_ciencia_mujer_tono_de_piel_medio:','fr':':scientifique_femme_peau_légèrement_mate:','ja':':女性科学者_中間の肌色:','ko':':여자_과학자_갈색_피부:','pt':':cientista_mulher_pele_morena:','it':':scienziata_carnagione_olivastra:','fa':':دانشمند_زن_پوست_طلایی:','id':':ilmuwan_wanita_warna_kulit_sedang:','zh':':女科学家_中等肤色:','ru':':ученая_средний_тон_кожи:','tr':':kadın_bilim_insanı_orta_cilt_tonu:','ar':':عالِمة_بشرة_بلون_معتدل:'},'\U0001F937\U0000200D\U00002640\U0000FE0F':{'en':':woman_shrugging:','status':fully_qualified,'E':4,'de':':schulterzuckende_frau:','es':':mujer_encogida_de_hombros:','fr':':femme_qui_hausse_les_épaules:','ja':':お手上げする女:','ko':':어깨를_으쓱하는_여자:','pt':':mulher_dando_de_ombros:','it':':donna_che_scrolla_le_spalle:','fa':':زن_شانه_بالا_می_اندازد:','id':':wanita_mengangkat_bahu:','zh':':女生耸肩:','ru':':женщина_пожимает_плечами:','tr':':omuz_silken_kadın:','ar':':سيدة_لا_تبالي:'},'\U0001F937\U0000200D\U00002640':{'en':':woman_shrugging:','status':minimally_qualified,'E':4,'de':':schulterzuckende_frau:','es':':mujer_encogida_de_hombros:','fr':':femme_qui_hausse_les_épaules:','ja':':お手上げする女:','ko':':어깨를_으쓱하는_여자:','pt':':mulher_dando_de_ombros:','it':':donna_che_scrolla_le_spalle:','fa':':زن_شانه_بالا_می_اندازد:','id':':wanita_mengangkat_bahu:','zh':':女生耸肩:','ru':':женщина_пожимает_плечами:','tr':':omuz_silken_kadın:','ar':':سيدة_لا_تبالي:'},'\U0001F937\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_shrugging_dark_skin_tone:','status':fully_qualified,'E':4,'de':':schulterzuckende_frau_dunkle_hautfarbe:','es':':mujer_encogida_de_hombros_tono_de_piel_oscuro:','fr':':femme_qui_hausse_les_épaules_peau_foncée:','ja':':お手上げする女_濃い肌色:','ko':':어깨를_으쓱하는_여자_검은색_피부:','pt':':mulher_dando_de_ombros_pele_escura:','it':':donna_che_scrolla_le_spalle_carnagione_scura:','fa':':زن_شانه_بالا_می_اندازد_پوست_آبنوسی:','id':':wanita_mengangkat_bahu_warna_kulit_gelap:','zh':':女生耸肩_较深肤色:','ru':':женщина_пожимает_плечами_очень_темныи_тон_кожи:','tr':':omuz_silken_kadın_koyu_cilt_tonu:','ar':':سيدة_لا_تبالي_بشرة_بلون_غامق:'},'\U0001F937\U0001F3FF\U0000200D\U00002640':{'en':':woman_shrugging_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':schulterzuckende_frau_dunkle_hautfarbe:','es':':mujer_encogida_de_hombros_tono_de_piel_oscuro:','fr':':femme_qui_hausse_les_épaules_peau_foncée:','ja':':お手上げする女_濃い肌色:','ko':':어깨를_으쓱하는_여자_검은색_피부:','pt':':mulher_dando_de_ombros_pele_escura:','it':':donna_che_scrolla_le_spalle_carnagione_scura:','fa':':زن_شانه_بالا_می_اندازد_پوست_آبنوسی:','id':':wanita_mengangkat_bahu_warna_kulit_gelap:','zh':':女生耸肩_较深肤色:','ru':':женщина_пожимает_плечами_очень_темный_тон_кожи:','tr':':omuz_silken_kadın_koyu_cilt_tonu:','ar':':سيدة_لا_تبالي_بشرة_بلون_غامق:'},'\U0001F937\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_shrugging_light_skin_tone:','status':fully_qualified,'E':4,'de':':schulterzuckende_frau_helle_hautfarbe:','es':':mujer_encogida_de_hombros_tono_de_piel_claro:','fr':':femme_qui_hausse_les_épaules_peau_claire:','ja':':お手上げする女_薄い肌色:','ko':':어깨를_으쓱하는_여자_하얀_피부:','pt':':mulher_dando_de_ombros_pele_clara:','it':':donna_che_scrolla_le_spalle_carnagione_chiara:','fa':':زن_شانه_بالا_می_اندازد_پوست_سفید:','id':':wanita_mengangkat_bahu_warna_kulit_cerah:','zh':':女生耸肩_较浅肤色:','ru':':женщина_пожимает_плечами_очень_светлыи_тон_кожи:','tr':':omuz_silken_kadın_açık_cilt_tonu:','ar':':سيدة_لا_تبالي_بشرة_بلون_فاتح:'},'\U0001F937\U0001F3FB\U0000200D\U00002640':{'en':':woman_shrugging_light_skin_tone:','status':minimally_qualified,'E':4,'de':':schulterzuckende_frau_helle_hautfarbe:','es':':mujer_encogida_de_hombros_tono_de_piel_claro:','fr':':femme_qui_hausse_les_épaules_peau_claire:','ja':':お手上げする女_薄い肌色:','ko':':어깨를_으쓱하는_여자_하얀_피부:','pt':':mulher_dando_de_ombros_pele_clara:','it':':donna_che_scrolla_le_spalle_carnagione_chiara:','fa':':زن_شانه_بالا_می_اندازد_پوست_سفید:','id':':wanita_mengangkat_bahu_warna_kulit_cerah:','zh':':女生耸肩_较浅肤色:','ru':':женщина_пожимает_плечами_очень_светлый_тон_кожи:','tr':':omuz_silken_kadın_açık_cilt_tonu:','ar':':سيدة_لا_تبالي_بشرة_بلون_فاتح:'},'\U0001F937\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_shrugging_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':schulterzuckende_frau_mitteldunkle_hautfarbe:','es':':mujer_encogida_de_hombros_tono_de_piel_oscuro_medio:','fr':':femme_qui_hausse_les_épaules_peau_mate:','ja':':お手上げする女_やや濃い肌色:','ko':':어깨를_으쓱하는_여자_진한_갈색_피부:','pt':':mulher_dando_de_ombros_pele_morena_escura:','it':':donna_che_scrolla_le_spalle_carnagione_abbastanza_scura:','fa':':زن_شانه_بالا_می_اندازد_پوست_گندمی:','id':':wanita_mengangkat_bahu_warna_kulit_gelap-sedang:','zh':':女生耸肩_中等深肤色:','ru':':женщина_пожимает_плечами_темныи_тон_кожи:','tr':':omuz_silken_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_لا_تبالي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F937\U0001F3FE\U0000200D\U00002640':{'en':':woman_shrugging_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':schulterzuckende_frau_mitteldunkle_hautfarbe:','es':':mujer_encogida_de_hombros_tono_de_piel_oscuro_medio:','fr':':femme_qui_hausse_les_épaules_peau_mate:','ja':':お手上げする女_やや濃い肌色:','ko':':어깨를_으쓱하는_여자_진한_갈색_피부:','pt':':mulher_dando_de_ombros_pele_morena_escura:','it':':donna_che_scrolla_le_spalle_carnagione_abbastanza_scura:','fa':':زن_شانه_بالا_می_اندازد_پوست_گندمی:','id':':wanita_mengangkat_bahu_warna_kulit_gelap-sedang:','zh':':女生耸肩_中等深肤色:','ru':':женщина_пожимает_плечами_темный_тон_кожи:','tr':':omuz_silken_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_لا_تبالي_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F937\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_shrugging_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':schulterzuckende_frau_mittelhelle_hautfarbe:','es':':mujer_encogida_de_hombros_tono_de_piel_claro_medio:','fr':':femme_qui_hausse_les_épaules_peau_moyennement_claire:','ja':':お手上げする女_やや薄い肌色:','ko':':어깨를_으쓱하는_여자_연한_갈색_피부:','pt':':mulher_dando_de_ombros_pele_morena_clara:','it':':donna_che_scrolla_le_spalle_carnagione_abbastanza_chiara:','fa':':زن_شانه_بالا_می_اندازد_پوست_روشن:','id':':wanita_mengangkat_bahu_warna_kulit_cerah-sedang:','zh':':女生耸肩_中等浅肤色:','ru':':женщина_пожимает_плечами_светлыи_тон_кожи:','tr':':omuz_silken_kadın_orta_açık_cilt_tonu:','ar':':سيدة_لا_تبالي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F937\U0001F3FC\U0000200D\U00002640':{'en':':woman_shrugging_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':schulterzuckende_frau_mittelhelle_hautfarbe:','es':':mujer_encogida_de_hombros_tono_de_piel_claro_medio:','fr':':femme_qui_hausse_les_épaules_peau_moyennement_claire:','ja':':お手上げする女_やや薄い肌色:','ko':':어깨를_으쓱하는_여자_연한_갈색_피부:','pt':':mulher_dando_de_ombros_pele_morena_clara:','it':':donna_che_scrolla_le_spalle_carnagione_abbastanza_chiara:','fa':':زن_شانه_بالا_می_اندازد_پوست_روشن:','id':':wanita_mengangkat_bahu_warna_kulit_cerah-sedang:','zh':':女生耸肩_中等浅肤色:','ru':':женщина_пожимает_плечами_светлый_тон_кожи:','tr':':omuz_silken_kadın_orta_açık_cilt_tonu:','ar':':سيدة_لا_تبالي_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F937\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_shrugging_medium_skin_tone:','status':fully_qualified,'E':4,'de':':schulterzuckende_frau_mittlere_hautfarbe:','es':':mujer_encogida_de_hombros_tono_de_piel_medio:','fr':':femme_qui_hausse_les_épaules_peau_légèrement_mate:','ja':':お手上げする女_中間の肌色:','ko':':어깨를_으쓱하는_여자_갈색_피부:','pt':':mulher_dando_de_ombros_pele_morena:','it':':donna_che_scrolla_le_spalle_carnagione_olivastra:','fa':':زن_شانه_بالا_می_اندازد_پوست_طلایی:','id':':wanita_mengangkat_bahu_warna_kulit_sedang:','zh':':女生耸肩_中等肤色:','ru':':женщина_пожимает_плечами_среднии_тон_кожи:','tr':':omuz_silken_kadın_orta_cilt_tonu:','ar':':سيدة_لا_تبالي_بشرة_بلون_معتدل:'},'\U0001F937\U0001F3FD\U0000200D\U00002640':{'en':':woman_shrugging_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':schulterzuckende_frau_mittlere_hautfarbe:','es':':mujer_encogida_de_hombros_tono_de_piel_medio:','fr':':femme_qui_hausse_les_épaules_peau_légèrement_mate:','ja':':お手上げする女_中間の肌色:','ko':':어깨를_으쓱하는_여자_갈색_피부:','pt':':mulher_dando_de_ombros_pele_morena:','it':':donna_che_scrolla_le_spalle_carnagione_olivastra:','fa':':زن_شانه_بالا_می_اندازد_پوست_طلایی:','id':':wanita_mengangkat_bahu_warna_kulit_sedang:','zh':':女生耸肩_中等肤色:','ru':':женщина_пожимает_плечами_средний_тон_кожи:','tr':':omuz_silken_kadın_orta_cilt_tonu:','ar':':سيدة_لا_تبالي_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U0001F3A4':{'en':':woman_singer:','status':fully_qualified,'E':4,'de':':sängerin:','es':':cantante_mujer:','fr':':chanteuse:','ja':':女性歌手:','ko':':여자_가수:','pt':':cantora:','it':':cantante_donna:','fa':':خواننده_زن:','id':':penyanyi_wanita:','zh':':女歌手:','ru':':певица:','tr':':kadın_şarkıcı:','ar':':مطربة:'},'\U0001F469\U0001F3FF\U0000200D\U0001F3A4':{'en':':woman_singer_dark_skin_tone:','status':fully_qualified,'E':4,'de':':sängerin_dunkle_hautfarbe:','es':':cantante_mujer_tono_de_piel_oscuro:','fr':':chanteuse_peau_foncée:','ja':':女性歌手_濃い肌色:','ko':':여자_가수_검은색_피부:','pt':':cantora_pele_escura:','it':':cantante_donna_carnagione_scura:','fa':':خواننده_زن_پوست_آبنوسی:','id':':penyanyi_wanita_warna_kulit_gelap:','zh':':女歌手_较深肤色:','ru':':певица_очень_темный_тон_кожи:','tr':':kadın_şarkıcı_koyu_cilt_tonu:','ar':':مطربة_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F3A4':{'en':':woman_singer_light_skin_tone:','status':fully_qualified,'E':4,'de':':sängerin_helle_hautfarbe:','es':':cantante_mujer_tono_de_piel_claro:','fr':':chanteuse_peau_claire:','ja':':女性歌手_薄い肌色:','ko':':여자_가수_하얀_피부:','pt':':cantora_pele_clara:','it':':cantante_donna_carnagione_chiara:','fa':':خواننده_زن_پوست_سفید:','id':':penyanyi_wanita_warna_kulit_cerah:','zh':':女歌手_较浅肤色:','ru':':певица_очень_светлый_тон_кожи:','tr':':kadın_şarkıcı_açık_cilt_tonu:','ar':':مطربة_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F3A4':{'en':':woman_singer_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':sängerin_mitteldunkle_hautfarbe:','es':':cantante_mujer_tono_de_piel_oscuro_medio:','fr':':chanteuse_peau_mate:','ja':':女性歌手_やや濃い肌色:','ko':':여자_가수_진한_갈색_피부:','pt':':cantora_pele_morena_escura:','it':':cantante_donna_carnagione_abbastanza_scura:','fa':':خواننده_زن_پوست_گندمی:','id':':penyanyi_wanita_warna_kulit_gelap-sedang:','zh':':女歌手_中等深肤色:','ru':':певица_темный_тон_кожи:','tr':':kadın_şarkıcı_orta_koyu_cilt_tonu:','ar':':مطربة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F3A4':{'en':':woman_singer_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':sängerin_mittelhelle_hautfarbe:','es':':cantante_mujer_tono_de_piel_claro_medio:','fr':':chanteuse_peau_moyennement_claire:','ja':':女性歌手_やや薄い肌色:','ko':':여자_가수_연한_갈색_피부:','pt':':cantora_pele_morena_clara:','it':':cantante_donna_carnagione_abbastanza_chiara:','fa':':خواننده_زن_پوست_روشن:','id':':penyanyi_wanita_warna_kulit_cerah-sedang:','zh':':女歌手_中等浅肤色:','ru':':певица_светлый_тон_кожи:','tr':':kadın_şarkıcı_orta_açık_cilt_tonu:','ar':':مطربة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F3A4':{'en':':woman_singer_medium_skin_tone:','status':fully_qualified,'E':4,'de':':sängerin_mittlere_hautfarbe:','es':':cantante_mujer_tono_de_piel_medio:','fr':':chanteuse_peau_légèrement_mate:','ja':':女性歌手_中間の肌色:','ko':':여자_가수_갈색_피부:','pt':':cantora_pele_morena:','it':':cantante_donna_carnagione_olivastra:','fa':':خواننده_زن_پوست_طلایی:','id':':penyanyi_wanita_warna_kulit_sedang:','zh':':女歌手_中等肤色:','ru':':певица_средний_тон_кожи:','tr':':kadın_şarkıcı_orta_cilt_tonu:','ar':':مطربة_بشرة_بلون_معتدل:'},'\U0001F9CD\U0000200D\U00002640\U0000FE0F':{'en':':woman_standing:','status':fully_qualified,'E':12,'alias':[':standing_woman:'],'de':':stehende_frau:','es':':mujer_de_pie:','fr':':femme_debout:','ja':':立つ女:','ko':':서_있는_여자:','pt':':mulher_em_pé:','it':':donna_in_piedi:','fa':':زن_ایستاده:','id':':wanita_berdiri:','zh':':站立的女人:','ru':':женщина_стоит:','tr':':ayakta_duran_kadın:','ar':':سيدة_واقفة:'},'\U0001F9CD\U0000200D\U00002640':{'en':':woman_standing:','status':minimally_qualified,'E':12,'alias':[':standing_woman:'],'de':':stehende_frau:','es':':mujer_de_pie:','fr':':femme_debout:','ja':':立つ女:','ko':':서_있는_여자:','pt':':mulher_em_pé:','it':':donna_in_piedi:','fa':':زن_ایستاده:','id':':wanita_berdiri:','zh':':站立的女人:','ru':':женщина_стоит:','tr':':ayakta_duran_kadın:','ar':':سيدة_واقفة:'},'\U0001F9CD\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_standing_dark_skin_tone:','status':fully_qualified,'E':12,'de':':stehende_frau_dunkle_hautfarbe:','es':':mujer_de_pie_tono_de_piel_oscuro:','fr':':femme_debout_peau_foncée:','ja':':立つ女_濃い肌色:','ko':':서_있는_여자_검은색_피부:','pt':':mulher_em_pé_pele_escura:','it':':donna_in_piedi_carnagione_scura:','fa':':زن_ایستاده_پوست_آبنوسی:','id':':wanita_berdiri_warna_kulit_gelap:','zh':':站立的女人_较深肤色:','ru':':женщина_стоит_очень_темныи_тон_кожи:','tr':':ayakta_duran_kadın_koyu_cilt_tonu:','ar':':سيدة_واقفة_بشرة_بلون_غامق:'},'\U0001F9CD\U0001F3FF\U0000200D\U00002640':{'en':':woman_standing_dark_skin_tone:','status':minimally_qualified,'E':12,'de':':stehende_frau_dunkle_hautfarbe:','es':':mujer_de_pie_tono_de_piel_oscuro:','fr':':femme_debout_peau_foncée:','ja':':立つ女_濃い肌色:','ko':':서_있는_여자_검은색_피부:','pt':':mulher_em_pé_pele_escura:','it':':donna_in_piedi_carnagione_scura:','fa':':زن_ایستاده_پوست_آبنوسی:','id':':wanita_berdiri_warna_kulit_gelap:','zh':':站立的女人_较深肤色:','ru':':женщина_стоит_очень_темный_тон_кожи:','tr':':ayakta_duran_kadın_koyu_cilt_tonu:','ar':':سيدة_واقفة_بشرة_بلون_غامق:'},'\U0001F9CD\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_standing_light_skin_tone:','status':fully_qualified,'E':12,'de':':stehende_frau_helle_hautfarbe:','es':':mujer_de_pie_tono_de_piel_claro:','fr':':femme_debout_peau_claire:','ja':':立つ女_薄い肌色:','ko':':서_있는_여자_하얀_피부:','pt':':mulher_em_pé_pele_clara:','it':':donna_in_piedi_carnagione_chiara:','fa':':زن_ایستاده_پوست_سفید:','id':':wanita_berdiri_warna_kulit_cerah:','zh':':站立的女人_较浅肤色:','ru':':женщина_стоит_очень_светлыи_тон_кожи:','tr':':ayakta_duran_kadın_açık_cilt_tonu:','ar':':سيدة_واقفة_بشرة_بلون_فاتح:'},'\U0001F9CD\U0001F3FB\U0000200D\U00002640':{'en':':woman_standing_light_skin_tone:','status':minimally_qualified,'E':12,'de':':stehende_frau_helle_hautfarbe:','es':':mujer_de_pie_tono_de_piel_claro:','fr':':femme_debout_peau_claire:','ja':':立つ女_薄い肌色:','ko':':서_있는_여자_하얀_피부:','pt':':mulher_em_pé_pele_clara:','it':':donna_in_piedi_carnagione_chiara:','fa':':زن_ایستاده_پوست_سفید:','id':':wanita_berdiri_warna_kulit_cerah:','zh':':站立的女人_较浅肤色:','ru':':женщина_стоит_очень_светлый_тон_кожи:','tr':':ayakta_duran_kadın_açık_cilt_tonu:','ar':':سيدة_واقفة_بشرة_بلون_فاتح:'},'\U0001F9CD\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_standing_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':stehende_frau_mitteldunkle_hautfarbe:','es':':mujer_de_pie_tono_de_piel_oscuro_medio:','fr':':femme_debout_peau_mate:','ja':':立つ女_やや濃い肌色:','ko':':서_있는_여자_진한_갈색_피부:','pt':':mulher_em_pé_pele_morena_escura:','it':':donna_in_piedi_carnagione_abbastanza_scura:','fa':':زن_ایستاده_پوست_گندمی:','id':':wanita_berdiri_warna_kulit_gelap-sedang:','zh':':站立的女人_中等深肤色:','ru':':женщина_стоит_темныи_тон_кожи:','tr':':ayakta_duran_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_واقفة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9CD\U0001F3FE\U0000200D\U00002640':{'en':':woman_standing_medium-dark_skin_tone:','status':minimally_qualified,'E':12,'de':':stehende_frau_mitteldunkle_hautfarbe:','es':':mujer_de_pie_tono_de_piel_oscuro_medio:','fr':':femme_debout_peau_mate:','ja':':立つ女_やや濃い肌色:','ko':':서_있는_여자_진한_갈색_피부:','pt':':mulher_em_pé_pele_morena_escura:','it':':donna_in_piedi_carnagione_abbastanza_scura:','fa':':زن_ایستاده_پوست_گندمی:','id':':wanita_berdiri_warna_kulit_gelap-sedang:','zh':':站立的女人_中等深肤色:','ru':':женщина_стоит_темный_тон_кожи:','tr':':ayakta_duran_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_واقفة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9CD\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_standing_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':stehende_frau_mittelhelle_hautfarbe:','es':':mujer_de_pie_tono_de_piel_claro_medio:','fr':':femme_debout_peau_moyennement_claire:','ja':':立つ女_やや薄い肌色:','ko':':서_있는_여자_연한_갈색_피부:','pt':':mulher_em_pé_pele_morena_clara:','it':':donna_in_piedi_carnagione_abbastanza_chiara:','fa':':زن_ایستاده_پوست_روشن:','id':':wanita_berdiri_warna_kulit_cerah-sedang:','zh':':站立的女人_中等浅肤色:','ru':':женщина_стоит_светлыи_тон_кожи:','tr':':ayakta_duran_kadın_orta_açık_cilt_tonu:','ar':':سيدة_واقفة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9CD\U0001F3FC\U0000200D\U00002640':{'en':':woman_standing_medium-light_skin_tone:','status':minimally_qualified,'E':12,'de':':stehende_frau_mittelhelle_hautfarbe:','es':':mujer_de_pie_tono_de_piel_claro_medio:','fr':':femme_debout_peau_moyennement_claire:','ja':':立つ女_やや薄い肌色:','ko':':서_있는_여자_연한_갈색_피부:','pt':':mulher_em_pé_pele_morena_clara:','it':':donna_in_piedi_carnagione_abbastanza_chiara:','fa':':زن_ایستاده_پوست_روشن:','id':':wanita_berdiri_warna_kulit_cerah-sedang:','zh':':站立的女人_中等浅肤色:','ru':':женщина_стоит_светлый_тон_кожи:','tr':':ayakta_duran_kadın_orta_açık_cilt_tonu:','ar':':سيدة_واقفة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9CD\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_standing_medium_skin_tone:','status':fully_qualified,'E':12,'de':':stehende_frau_mittlere_hautfarbe:','es':':mujer_de_pie_tono_de_piel_medio:','fr':':femme_debout_peau_légèrement_mate:','ja':':立つ女_中間の肌色:','ko':':서_있는_여자_갈색_피부:','pt':':mulher_em_pé_pele_morena:','it':':donna_in_piedi_carnagione_olivastra:','fa':':زن_ایستاده_پوست_طلایی:','id':':wanita_berdiri_warna_kulit_sedang:','zh':':站立的女人_中等肤色:','ru':':женщина_стоит_среднии_тон_кожи:','tr':':ayakta_duran_kadın_orta_cilt_tonu:','ar':':سيدة_واقفة_بشرة_بلون_معتدل:'},'\U0001F9CD\U0001F3FD\U0000200D\U00002640':{'en':':woman_standing_medium_skin_tone:','status':minimally_qualified,'E':12,'de':':stehende_frau_mittlere_hautfarbe:','es':':mujer_de_pie_tono_de_piel_medio:','fr':':femme_debout_peau_légèrement_mate:','ja':':立つ女_中間の肌色:','ko':':서_있는_여자_갈색_피부:','pt':':mulher_em_pé_pele_morena:','it':':donna_in_piedi_carnagione_olivastra:','fa':':زن_ایستاده_پوست_طلایی:','id':':wanita_berdiri_warna_kulit_sedang:','zh':':站立的女人_中等肤色:','ru':':женщина_стоит_средний_тон_кожи:','tr':':ayakta_duran_kadın_orta_cilt_tonu:','ar':':سيدة_واقفة_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U0001F393':{'en':':woman_student:','status':fully_qualified,'E':4,'de':':studentin:','es':':estudiante_mujer:','fr':':étudiante:','ja':':女子学生:','ko':':여학생:','pt':':aluna:','it':':studentessa:','fa':':دانشجوی_زن:','id':':sarjana_wanita:','zh':':女学生:','ru':':студентка:','tr':':kız_öğrenci:','ar':':طالبة:'},'\U0001F469\U0001F3FF\U0000200D\U0001F393':{'en':':woman_student_dark_skin_tone:','status':fully_qualified,'E':4,'de':':studentin_dunkle_hautfarbe:','es':':estudiante_mujer_tono_de_piel_oscuro:','fr':':étudiante_peau_foncée:','ja':':女子学生_濃い肌色:','ko':':여학생_검은색_피부:','pt':':aluna_pele_escura:','it':':studentessa_carnagione_scura:','fa':':دانشجوی_زن_پوست_آبنوسی:','id':':sarjana_wanita_warna_kulit_gelap:','zh':':女学生_较深肤色:','ru':':студентка_очень_темный_тон_кожи:','tr':':kız_öğrenci_koyu_cilt_tonu:','ar':':طالبة_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F393':{'en':':woman_student_light_skin_tone:','status':fully_qualified,'E':4,'de':':studentin_helle_hautfarbe:','es':':estudiante_mujer_tono_de_piel_claro:','fr':':étudiante_peau_claire:','ja':':女子学生_薄い肌色:','ko':':여학생_하얀_피부:','pt':':aluna_pele_clara:','it':':studentessa_carnagione_chiara:','fa':':دانشجوی_زن_پوست_سفید:','id':':sarjana_wanita_warna_kulit_cerah:','zh':':女学生_较浅肤色:','ru':':студентка_очень_светлый_тон_кожи:','tr':':kız_öğrenci_açık_cilt_tonu:','ar':':طالبة_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F393':{'en':':woman_student_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':studentin_mitteldunkle_hautfarbe:','es':':estudiante_mujer_tono_de_piel_oscuro_medio:','fr':':étudiante_peau_mate:','ja':':女子学生_やや濃い肌色:','ko':':여학생_진한_갈색_피부:','pt':':aluna_pele_morena_escura:','it':':studentessa_carnagione_abbastanza_scura:','fa':':دانشجوی_زن_پوست_گندمی:','id':':sarjana_wanita_warna_kulit_gelap-sedang:','zh':':女学生_中等深肤色:','ru':':студентка_темный_тон_кожи:','tr':':kız_öğrenci_orta_koyu_cilt_tonu:','ar':':طالبة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F393':{'en':':woman_student_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':studentin_mittelhelle_hautfarbe:','es':':estudiante_mujer_tono_de_piel_claro_medio:','fr':':étudiante_peau_moyennement_claire:','ja':':女子学生_やや薄い肌色:','ko':':여학생_연한_갈색_피부:','pt':':aluna_pele_morena_clara:','it':':studentessa_carnagione_abbastanza_chiara:','fa':':دانشجوی_زن_پوست_روشن:','id':':sarjana_wanita_warna_kulit_cerah-sedang:','zh':':女学生_中等浅肤色:','ru':':студентка_светлый_тон_кожи:','tr':':kız_öğrenci_orta_açık_cilt_tonu:','ar':':طالبة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F393':{'en':':woman_student_medium_skin_tone:','status':fully_qualified,'E':4,'de':':studentin_mittlere_hautfarbe:','es':':estudiante_mujer_tono_de_piel_medio:','fr':':étudiante_peau_légèrement_mate:','ja':':女子学生_中間の肌色:','ko':':여학생_갈색_피부:','pt':':aluna_pele_morena:','it':':studentessa_carnagione_olivastra:','fa':':دانشجوی_زن_پوست_طلایی:','id':':sarjana_wanita_warna_kulit_sedang:','zh':':女学生_中等肤色:','ru':':студентка_средний_тон_кожи:','tr':':kız_öğrenci_orta_cilt_tonu:','ar':':طالبة_بشرة_بلون_معتدل:'},'\U0001F9B8\U0000200D\U00002640\U0000FE0F':{'en':':woman_superhero:','status':fully_qualified,'E':11,'alias':[':superhero_woman:'],'de':':heldin:','es':':superheroína:','fr':':super-héroïne:','ja':':女性のスーパーヒーロー:','ko':':여자_히어로:','pt':':super-heroína:','it':':supereroina:','fa':':ابرقهرمان_زن:','id':':pahlawan_super_wanita:','zh':':女超级英雄:','ru':':женщина-супергерои:','tr':':kadın_süper_kahraman:','ar':':بطلة_خارقة:'},'\U0001F9B8\U0000200D\U00002640':{'en':':woman_superhero:','status':minimally_qualified,'E':11,'alias':[':superhero_woman:'],'de':':heldin:','es':':superheroína:','fr':':super-héroïne:','ja':':女性のスーパーヒーロー:','ko':':여자_히어로:','pt':':super-heroína:','it':':supereroina:','fa':':ابرقهرمان_زن:','id':':pahlawan_super_wanita:','zh':':女超级英雄:','ru':':женщина_супергерой:','tr':':kadın_süper_kahraman:','ar':':بطلة_خارقة:'},'\U0001F9B8\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_superhero_dark_skin_tone:','status':fully_qualified,'E':11,'de':':heldin_dunkle_hautfarbe:','es':':superheroína_tono_de_piel_oscuro:','fr':':super-héroïne_peau_foncée:','ja':':女性のスーパーヒーロー_濃い肌色:','ko':':여자_히어로_검은색_피부:','pt':':super-heroína_pele_escura:','it':':supereroina_carnagione_scura:','fa':':ابرقهرمان_زن_پوست_آبنوسی:','id':':pahlawan_super_wanita_warna_kulit_gelap:','zh':':女超级英雄_较深肤色:','ru':':женщина-супергерои_очень_темныи_тон_кожи:','tr':':kadın_süper_kahraman_koyu_cilt_tonu:','ar':':بطلة_خارقة_بشرة_بلون_غامق:'},'\U0001F9B8\U0001F3FF\U0000200D\U00002640':{'en':':woman_superhero_dark_skin_tone:','status':minimally_qualified,'E':11,'de':':heldin_dunkle_hautfarbe:','es':':superheroína_tono_de_piel_oscuro:','fr':':super-héroïne_peau_foncée:','ja':':女性のスーパーヒーロー_濃い肌色:','ko':':여자_히어로_검은색_피부:','pt':':super-heroína_pele_escura:','it':':supereroina_carnagione_scura:','fa':':ابرقهرمان_زن_پوست_آبنوسی:','id':':pahlawan_super_wanita_warna_kulit_gelap:','zh':':女超级英雄_较深肤色:','ru':':женщина_супергерой_очень_темный_тон_кожи:','tr':':kadın_süper_kahraman_koyu_cilt_tonu:','ar':':بطلة_خارقة_بشرة_بلون_غامق:'},'\U0001F9B8\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_superhero_light_skin_tone:','status':fully_qualified,'E':11,'de':':heldin_helle_hautfarbe:','es':':superheroína_tono_de_piel_claro:','fr':':super-héroïne_peau_claire:','ja':':女性のスーパーヒーロー_薄い肌色:','ko':':여자_히어로_하얀_피부:','pt':':super-heroína_pele_clara:','it':':supereroina_carnagione_chiara:','fa':':ابرقهرمان_زن_پوست_سفید:','id':':pahlawan_super_wanita_warna_kulit_cerah:','zh':':女超级英雄_较浅肤色:','ru':':женщина-супергерои_очень_светлыи_тон_кожи:','tr':':kadın_süper_kahraman_açık_cilt_tonu:','ar':':بطلة_خارقة_بشرة_بلون_فاتح:'},'\U0001F9B8\U0001F3FB\U0000200D\U00002640':{'en':':woman_superhero_light_skin_tone:','status':minimally_qualified,'E':11,'de':':heldin_helle_hautfarbe:','es':':superheroína_tono_de_piel_claro:','fr':':super-héroïne_peau_claire:','ja':':女性のスーパーヒーロー_薄い肌色:','ko':':여자_히어로_하얀_피부:','pt':':super-heroína_pele_clara:','it':':supereroina_carnagione_chiara:','fa':':ابرقهرمان_زن_پوست_سفید:','id':':pahlawan_super_wanita_warna_kulit_cerah:','zh':':女超级英雄_较浅肤色:','ru':':женщина_супергерой_очень_светлый_тон_кожи:','tr':':kadın_süper_kahraman_açık_cilt_tonu:','ar':':بطلة_خارقة_بشرة_بلون_فاتح:'},'\U0001F9B8\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_superhero_medium-dark_skin_tone:','status':fully_qualified,'E':11,'de':':heldin_mitteldunkle_hautfarbe:','es':':superheroína_tono_de_piel_oscuro_medio:','fr':':super-héroïne_peau_mate:','ja':':女性のスーパーヒーロー_やや濃い肌色:','ko':':여자_히어로_진한_갈색_피부:','pt':':super-heroína_pele_morena_escura:','it':':supereroina_carnagione_abbastanza_scura:','fa':':ابرقهرمان_زن_پوست_گندمی:','id':':pahlawan_super_wanita_warna_kulit_gelap-sedang:','zh':':女超级英雄_中等深肤色:','ru':':женщина-супергерои_темныи_тон_кожи:','tr':':kadın_süper_kahraman_orta_koyu_cilt_tonu:','ar':':بطلة_خارقة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9B8\U0001F3FE\U0000200D\U00002640':{'en':':woman_superhero_medium-dark_skin_tone:','status':minimally_qualified,'E':11,'de':':heldin_mitteldunkle_hautfarbe:','es':':superheroína_tono_de_piel_oscuro_medio:','fr':':super-héroïne_peau_mate:','ja':':女性のスーパーヒーロー_やや濃い肌色:','ko':':여자_히어로_진한_갈색_피부:','pt':':super-heroína_pele_morena_escura:','it':':supereroina_carnagione_abbastanza_scura:','fa':':ابرقهرمان_زن_پوست_گندمی:','id':':pahlawan_super_wanita_warna_kulit_gelap-sedang:','zh':':女超级英雄_中等深肤色:','ru':':женщина_супергерой_темный_тон_кожи:','tr':':kadın_süper_kahraman_orta_koyu_cilt_tonu:','ar':':بطلة_خارقة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9B8\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_superhero_medium-light_skin_tone:','status':fully_qualified,'E':11,'de':':heldin_mittelhelle_hautfarbe:','es':':superheroína_tono_de_piel_claro_medio:','fr':':super-héroïne_peau_moyennement_claire:','ja':':女性のスーパーヒーロー_やや薄い肌色:','ko':':여자_히어로_연한_갈색_피부:','pt':':super-heroína_pele_morena_clara:','it':':supereroina_carnagione_abbastanza_chiara:','fa':':ابرقهرمان_زن_پوست_روشن:','id':':pahlawan_super_wanita_warna_kulit_cerah-sedang:','zh':':女超级英雄_中等浅肤色:','ru':':женщина-супергерои_светлыи_тон_кожи:','tr':':kadın_süper_kahraman_orta_açık_cilt_tonu:','ar':':بطلة_خارقة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9B8\U0001F3FC\U0000200D\U00002640':{'en':':woman_superhero_medium-light_skin_tone:','status':minimally_qualified,'E':11,'de':':heldin_mittelhelle_hautfarbe:','es':':superheroína_tono_de_piel_claro_medio:','fr':':super-héroïne_peau_moyennement_claire:','ja':':女性のスーパーヒーロー_やや薄い肌色:','ko':':여자_히어로_연한_갈색_피부:','pt':':super-heroína_pele_morena_clara:','it':':supereroina_carnagione_abbastanza_chiara:','fa':':ابرقهرمان_زن_پوست_روشن:','id':':pahlawan_super_wanita_warna_kulit_cerah-sedang:','zh':':女超级英雄_中等浅肤色:','ru':':женщина_супергерой_светлый_тон_кожи:','tr':':kadın_süper_kahraman_orta_açık_cilt_tonu:','ar':':بطلة_خارقة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9B8\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_superhero_medium_skin_tone:','status':fully_qualified,'E':11,'de':':heldin_mittlere_hautfarbe:','es':':superheroína_tono_de_piel_medio:','fr':':super-héroïne_peau_légèrement_mate:','ja':':女性のスーパーヒーロー_中間の肌色:','ko':':여자_히어로_갈색_피부:','pt':':super-heroína_pele_morena:','it':':supereroina_carnagione_olivastra:','fa':':ابرقهرمان_زن_پوست_طلایی:','id':':pahlawan_super_wanita_warna_kulit_sedang:','zh':':女超级英雄_中等肤色:','ru':':женщина-супергерои_среднии_тон_кожи:','tr':':kadın_süper_kahraman_orta_cilt_tonu:','ar':':بطلة_خارقة_بشرة_بلون_معتدل:'},'\U0001F9B8\U0001F3FD\U0000200D\U00002640':{'en':':woman_superhero_medium_skin_tone:','status':minimally_qualified,'E':11,'de':':heldin_mittlere_hautfarbe:','es':':superheroína_tono_de_piel_medio:','fr':':super-héroïne_peau_légèrement_mate:','ja':':女性のスーパーヒーロー_中間の肌色:','ko':':여자_히어로_갈색_피부:','pt':':super-heroína_pele_morena:','it':':supereroina_carnagione_olivastra:','fa':':ابرقهرمان_زن_پوست_طلایی:','id':':pahlawan_super_wanita_warna_kulit_sedang:','zh':':女超级英雄_中等肤色:','ru':':женщина_супергерой_средний_тон_кожи:','tr':':kadın_süper_kahraman_orta_cilt_tonu:','ar':':بطلة_خارقة_بشرة_بلون_معتدل:'},'\U0001F9B9\U0000200D\U00002640\U0000FE0F':{'en':':woman_supervillain:','status':fully_qualified,'E':11,'alias':[':supervillain_woman:'],'de':':weiblicher_bösewicht:','es':':supervillana:','fr':':super-vilain_femme:','ja':':女性の悪役:','ko':':여자_슈퍼_악당:','pt':':supervilã:','it':':supercattiva:','fa':':ابرشرور_زن:','id':':penjahat_super_wanita:','zh':':女超级大坏蛋:','ru':':женщина-суперзлодеи:','tr':':kadın_süper_kötü:','ar':':شريرة_خارقة:'},'\U0001F9B9\U0000200D\U00002640':{'en':':woman_supervillain:','status':minimally_qualified,'E':11,'alias':[':supervillain_woman:'],'de':':weiblicher_bösewicht:','es':':supervillana:','fr':':super-vilain_femme:','ja':':女性の悪役:','ko':':여자_슈퍼_악당:','pt':':supervilã:','it':':supercattiva:','fa':':ابرشرور_زن:','id':':penjahat_super_wanita:','zh':':女超级大坏蛋:','ru':':женщина_суперзлодей:','tr':':kadın_süper_kötü:','ar':':شريرة_خارقة:'},'\U0001F9B9\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_supervillain_dark_skin_tone:','status':fully_qualified,'E':11,'de':':weiblicher_bösewicht_dunkle_hautfarbe:','es':':supervillana_tono_de_piel_oscuro:','fr':':super-vilain_femme_peau_foncée:','ja':':女性の悪役_濃い肌色:','ko':':여자_슈퍼_악당_검은색_피부:','pt':':supervilã_pele_escura:','it':':supercattiva_carnagione_scura:','fa':':ابرشرور_زن_پوست_آبنوسی:','id':':penjahat_super_wanita_warna_kulit_gelap:','zh':':女超级大坏蛋_较深肤色:','ru':':женщина-суперзлодеи_очень_темныи_тон_кожи:','tr':':kadın_süper_kötü_koyu_cilt_tonu:','ar':':شريرة_خارقة_بشرة_بلون_غامق:'},'\U0001F9B9\U0001F3FF\U0000200D\U00002640':{'en':':woman_supervillain_dark_skin_tone:','status':minimally_qualified,'E':11,'de':':weiblicher_bösewicht_dunkle_hautfarbe:','es':':supervillana_tono_de_piel_oscuro:','fr':':super-vilain_femme_peau_foncée:','ja':':女性の悪役_濃い肌色:','ko':':여자_슈퍼_악당_검은색_피부:','pt':':supervilã_pele_escura:','it':':supercattiva_carnagione_scura:','fa':':ابرشرور_زن_پوست_آبنوسی:','id':':penjahat_super_wanita_warna_kulit_gelap:','zh':':女超级大坏蛋_较深肤色:','ru':':женщина_суперзлодей_очень_темный_тон_кожи:','tr':':kadın_süper_kötü_koyu_cilt_tonu:','ar':':شريرة_خارقة_بشرة_بلون_غامق:'},'\U0001F9B9\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_supervillain_light_skin_tone:','status':fully_qualified,'E':11,'de':':weiblicher_bösewicht_helle_hautfarbe:','es':':supervillana_tono_de_piel_claro:','fr':':super-vilain_femme_peau_claire:','ja':':女性の悪役_薄い肌色:','ko':':여자_슈퍼_악당_하얀_피부:','pt':':supervilã_pele_clara:','it':':supercattiva_carnagione_chiara:','fa':':ابرشرور_زن_پوست_سفید:','id':':penjahat_super_wanita_warna_kulit_cerah:','zh':':女超级大坏蛋_较浅肤色:','ru':':женщина-суперзлодеи_очень_светлыи_тон_кожи:','tr':':kadın_süper_kötü_açık_cilt_tonu:','ar':':شريرة_خارقة_بشرة_بلون_فاتح:'},'\U0001F9B9\U0001F3FB\U0000200D\U00002640':{'en':':woman_supervillain_light_skin_tone:','status':minimally_qualified,'E':11,'de':':weiblicher_bösewicht_helle_hautfarbe:','es':':supervillana_tono_de_piel_claro:','fr':':super-vilain_femme_peau_claire:','ja':':女性の悪役_薄い肌色:','ko':':여자_슈퍼_악당_하얀_피부:','pt':':supervilã_pele_clara:','it':':supercattiva_carnagione_chiara:','fa':':ابرشرور_زن_پوست_سفید:','id':':penjahat_super_wanita_warna_kulit_cerah:','zh':':女超级大坏蛋_较浅肤色:','ru':':женщина_суперзлодей_очень_светлый_тон_кожи:','tr':':kadın_süper_kötü_açık_cilt_tonu:','ar':':شريرة_خارقة_بشرة_بلون_فاتح:'},'\U0001F9B9\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_supervillain_medium-dark_skin_tone:','status':fully_qualified,'E':11,'de':':weiblicher_bösewicht_mitteldunkle_hautfarbe:','es':':supervillana_tono_de_piel_oscuro_medio:','fr':':super-vilain_femme_peau_mate:','ja':':女性の悪役_やや濃い肌色:','ko':':여자_슈퍼_악당_진한_갈색_피부:','pt':':supervilã_pele_morena_escura:','it':':supercattiva_carnagione_abbastanza_scura:','fa':':ابرشرور_زن_پوست_گندمی:','id':':penjahat_super_wanita_warna_kulit_gelap-sedang:','zh':':女超级大坏蛋_中等深肤色:','ru':':женщина-суперзлодеи_темныи_тон_кожи:','tr':':kadın_süper_kötü_orta_koyu_cilt_tonu:','ar':':شريرة_خارقة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9B9\U0001F3FE\U0000200D\U00002640':{'en':':woman_supervillain_medium-dark_skin_tone:','status':minimally_qualified,'E':11,'de':':weiblicher_bösewicht_mitteldunkle_hautfarbe:','es':':supervillana_tono_de_piel_oscuro_medio:','fr':':super-vilain_femme_peau_mate:','ja':':女性の悪役_やや濃い肌色:','ko':':여자_슈퍼_악당_진한_갈색_피부:','pt':':supervilã_pele_morena_escura:','it':':supercattiva_carnagione_abbastanza_scura:','fa':':ابرشرور_زن_پوست_گندمی:','id':':penjahat_super_wanita_warna_kulit_gelap-sedang:','zh':':女超级大坏蛋_中等深肤色:','ru':':женщина_суперзлодей_темный_тон_кожи:','tr':':kadın_süper_kötü_orta_koyu_cilt_tonu:','ar':':شريرة_خارقة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9B9\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_supervillain_medium-light_skin_tone:','status':fully_qualified,'E':11,'de':':weiblicher_bösewicht_mittelhelle_hautfarbe:','es':':supervillana_tono_de_piel_claro_medio:','fr':':super-vilain_femme_peau_moyennement_claire:','ja':':女性の悪役_やや薄い肌色:','ko':':여자_슈퍼_악당_연한_갈색_피부:','pt':':supervilã_pele_morena_clara:','it':':supercattiva_carnagione_abbastanza_chiara:','fa':':ابرشرور_زن_پوست_روشن:','id':':penjahat_super_wanita_warna_kulit_cerah-sedang:','zh':':女超级大坏蛋_中等浅肤色:','ru':':женщина-суперзлодеи_светлыи_тон_кожи:','tr':':kadın_süper_kötü_orta_açık_cilt_tonu:','ar':':شريرة_خارقة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9B9\U0001F3FC\U0000200D\U00002640':{'en':':woman_supervillain_medium-light_skin_tone:','status':minimally_qualified,'E':11,'de':':weiblicher_bösewicht_mittelhelle_hautfarbe:','es':':supervillana_tono_de_piel_claro_medio:','fr':':super-vilain_femme_peau_moyennement_claire:','ja':':女性の悪役_やや薄い肌色:','ko':':여자_슈퍼_악당_연한_갈색_피부:','pt':':supervilã_pele_morena_clara:','it':':supercattiva_carnagione_abbastanza_chiara:','fa':':ابرشرور_زن_پوست_روشن:','id':':penjahat_super_wanita_warna_kulit_cerah-sedang:','zh':':女超级大坏蛋_中等浅肤色:','ru':':женщина_суперзлодей_светлый_тон_кожи:','tr':':kadın_süper_kötü_orta_açık_cilt_tonu:','ar':':شريرة_خارقة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9B9\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_supervillain_medium_skin_tone:','status':fully_qualified,'E':11,'de':':weiblicher_bösewicht_mittlere_hautfarbe:','es':':supervillana_tono_de_piel_medio:','fr':':super-vilain_femme_peau_légèrement_mate:','ja':':女性の悪役_中間の肌色:','ko':':여자_슈퍼_악당_갈색_피부:','pt':':supervilã_pele_morena:','it':':supercattiva_carnagione_olivastra:','fa':':ابرشرور_زن_پوست_طلایی:','id':':penjahat_super_wanita_warna_kulit_sedang:','zh':':女超级大坏蛋_中等肤色:','ru':':женщина-суперзлодеи_среднии_тон_кожи:','tr':':kadın_süper_kötü_orta_cilt_tonu:','ar':':شريرة_خارقة_بشرة_بلون_معتدل:'},'\U0001F9B9\U0001F3FD\U0000200D\U00002640':{'en':':woman_supervillain_medium_skin_tone:','status':minimally_qualified,'E':11,'de':':weiblicher_bösewicht_mittlere_hautfarbe:','es':':supervillana_tono_de_piel_medio:','fr':':super-vilain_femme_peau_légèrement_mate:','ja':':女性の悪役_中間の肌色:','ko':':여자_슈퍼_악당_갈색_피부:','pt':':supervilã_pele_morena:','it':':supercattiva_carnagione_olivastra:','fa':':ابرشرور_زن_پوست_طلایی:','id':':penjahat_super_wanita_warna_kulit_sedang:','zh':':女超级大坏蛋_中等肤色:','ru':':женщина_суперзлодей_средний_тон_кожи:','tr':':kadın_süper_kötü_orta_cilt_tonu:','ar':':شريرة_خارقة_بشرة_بلون_معتدل:'},'\U0001F3C4\U0000200D\U00002640\U0000FE0F':{'en':':woman_surfing:','status':fully_qualified,'E':4,'alias':[':surfing_woman:'],'de':':surferin:','es':':mujer_haciendo_surf:','fr':':surfeuse:','ja':':サーフィンする女:','ko':':서핑하는_여자:','pt':':mulher_surfista:','it':':surfista_donna:','fa':':موج_سوار_زن:','id':':wanita_berselancar:','zh':':女生冲浪:','ru':':серфингистка:','tr':':sörf_yapan_kadın:','ar':':راكبة_أمواج:'},'\U0001F3C4\U0000200D\U00002640':{'en':':woman_surfing:','status':minimally_qualified,'E':4,'alias':[':surfing_woman:'],'de':':surferin:','es':':mujer_haciendo_surf:','fr':':surfeuse:','ja':':サーフィンする女:','ko':':서핑하는_여자:','pt':':mulher_surfista:','it':':surfista_donna:','fa':':موج_سوار_زن:','id':':wanita_berselancar:','zh':':女生冲浪:','ru':':серфингистка:','tr':':sörf_yapan_kadın:','ar':':راكبة_أمواج:'},'\U0001F3C4\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_surfing_dark_skin_tone:','status':fully_qualified,'E':4,'de':':surferin_dunkle_hautfarbe:','es':':mujer_haciendo_surf_tono_de_piel_oscuro:','fr':':surfeuse_peau_foncée:','ja':':サーフィンする女_濃い肌色:','ko':':서핑하는_여자_검은색_피부:','pt':':mulher_surfista_pele_escura:','it':':surfista_donna_carnagione_scura:','fa':':موج_سوار_زن_پوست_آبنوسی:','id':':wanita_berselancar_warna_kulit_gelap:','zh':':女生冲浪_较深肤色:','ru':':серфингистка_очень_темныи_тон_кожи:','tr':':sörf_yapan_kadın_koyu_cilt_tonu:','ar':':راكبة_أمواج_بشرة_بلون_غامق:'},'\U0001F3C4\U0001F3FF\U0000200D\U00002640':{'en':':woman_surfing_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':surferin_dunkle_hautfarbe:','es':':mujer_haciendo_surf_tono_de_piel_oscuro:','fr':':surfeuse_peau_foncée:','ja':':サーフィンする女_濃い肌色:','ko':':서핑하는_여자_검은색_피부:','pt':':mulher_surfista_pele_escura:','it':':surfista_donna_carnagione_scura:','fa':':موج_سوار_زن_پوست_آبنوسی:','id':':wanita_berselancar_warna_kulit_gelap:','zh':':女生冲浪_较深肤色:','ru':':серфингистка_очень_темный_тон_кожи:','tr':':sörf_yapan_kadın_koyu_cilt_tonu:','ar':':راكبة_أمواج_بشرة_بلون_غامق:'},'\U0001F3C4\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_surfing_light_skin_tone:','status':fully_qualified,'E':4,'de':':surferin_helle_hautfarbe:','es':':mujer_haciendo_surf_tono_de_piel_claro:','fr':':surfeuse_peau_claire:','ja':':サーフィンする女_薄い肌色:','ko':':서핑하는_여자_하얀_피부:','pt':':mulher_surfista_pele_clara:','it':':surfista_donna_carnagione_chiara:','fa':':موج_سوار_زن_پوست_سفید:','id':':wanita_berselancar_warna_kulit_cerah:','zh':':女生冲浪_较浅肤色:','ru':':серфингистка_очень_светлыи_тон_кожи:','tr':':sörf_yapan_kadın_açık_cilt_tonu:','ar':':راكبة_أمواج_بشرة_بلون_فاتح:'},'\U0001F3C4\U0001F3FB\U0000200D\U00002640':{'en':':woman_surfing_light_skin_tone:','status':minimally_qualified,'E':4,'de':':surferin_helle_hautfarbe:','es':':mujer_haciendo_surf_tono_de_piel_claro:','fr':':surfeuse_peau_claire:','ja':':サーフィンする女_薄い肌色:','ko':':서핑하는_여자_하얀_피부:','pt':':mulher_surfista_pele_clara:','it':':surfista_donna_carnagione_chiara:','fa':':موج_سوار_زن_پوست_سفید:','id':':wanita_berselancar_warna_kulit_cerah:','zh':':女生冲浪_较浅肤色:','ru':':серфингистка_очень_светлый_тон_кожи:','tr':':sörf_yapan_kadın_açık_cilt_tonu:','ar':':راكبة_أمواج_بشرة_بلون_فاتح:'},'\U0001F3C4\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_surfing_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':surferin_mitteldunkle_hautfarbe:','es':':mujer_haciendo_surf_tono_de_piel_oscuro_medio:','fr':':surfeuse_peau_mate:','ja':':サーフィンする女_やや濃い肌色:','ko':':서핑하는_여자_진한_갈색_피부:','pt':':mulher_surfista_pele_morena_escura:','it':':surfista_donna_carnagione_abbastanza_scura:','fa':':موج_سوار_زن_پوست_گندمی:','id':':wanita_berselancar_warna_kulit_gelap-sedang:','zh':':女生冲浪_中等深肤色:','ru':':серфингистка_темныи_тон_кожи:','tr':':sörf_yapan_kadın_orta_koyu_cilt_tonu:','ar':':راكبة_أمواج_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3C4\U0001F3FE\U0000200D\U00002640':{'en':':woman_surfing_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':surferin_mitteldunkle_hautfarbe:','es':':mujer_haciendo_surf_tono_de_piel_oscuro_medio:','fr':':surfeuse_peau_mate:','ja':':サーフィンする女_やや濃い肌色:','ko':':서핑하는_여자_진한_갈색_피부:','pt':':mulher_surfista_pele_morena_escura:','it':':surfista_donna_carnagione_abbastanza_scura:','fa':':موج_سوار_زن_پوست_گندمی:','id':':wanita_berselancar_warna_kulit_gelap-sedang:','zh':':女生冲浪_中等深肤色:','ru':':серфингистка_темный_тон_кожи:','tr':':sörf_yapan_kadın_orta_koyu_cilt_tonu:','ar':':راكبة_أمواج_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3C4\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_surfing_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':surferin_mittelhelle_hautfarbe:','es':':mujer_haciendo_surf_tono_de_piel_claro_medio:','fr':':surfeuse_peau_moyennement_claire:','ja':':サーフィンする女_やや薄い肌色:','ko':':서핑하는_여자_연한_갈색_피부:','pt':':mulher_surfista_pele_morena_clara:','it':':surfista_donna_carnagione_abbastanza_chiara:','fa':':موج_سوار_زن_پوست_روشن:','id':':wanita_berselancar_warna_kulit_cerah-sedang:','zh':':女生冲浪_中等浅肤色:','ru':':серфингистка_светлыи_тон_кожи:','tr':':sörf_yapan_kadın_orta_açık_cilt_tonu:','ar':':راكبة_أمواج_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3C4\U0001F3FC\U0000200D\U00002640':{'en':':woman_surfing_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':surferin_mittelhelle_hautfarbe:','es':':mujer_haciendo_surf_tono_de_piel_claro_medio:','fr':':surfeuse_peau_moyennement_claire:','ja':':サーフィンする女_やや薄い肌色:','ko':':서핑하는_여자_연한_갈색_피부:','pt':':mulher_surfista_pele_morena_clara:','it':':surfista_donna_carnagione_abbastanza_chiara:','fa':':موج_سوار_زن_پوست_روشن:','id':':wanita_berselancar_warna_kulit_cerah-sedang:','zh':':女生冲浪_中等浅肤色:','ru':':серфингистка_светлый_тон_кожи:','tr':':sörf_yapan_kadın_orta_açık_cilt_tonu:','ar':':راكبة_أمواج_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3C4\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_surfing_medium_skin_tone:','status':fully_qualified,'E':4,'de':':surferin_mittlere_hautfarbe:','es':':mujer_haciendo_surf_tono_de_piel_medio:','fr':':surfeuse_peau_légèrement_mate:','ja':':サーフィンする女_中間の肌色:','ko':':서핑하는_여자_갈색_피부:','pt':':mulher_surfista_pele_morena:','it':':surfista_donna_carnagione_olivastra:','fa':':موج_سوار_زن_پوست_طلایی:','id':':wanita_berselancar_warna_kulit_sedang:','zh':':女生冲浪_中等肤色:','ru':':серфингистка_среднии_тон_кожи:','tr':':sörf_yapan_kadın_orta_cilt_tonu:','ar':':راكبة_أمواج_بشرة_بلون_معتدل:'},'\U0001F3C4\U0001F3FD\U0000200D\U00002640':{'en':':woman_surfing_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':surferin_mittlere_hautfarbe:','es':':mujer_haciendo_surf_tono_de_piel_medio:','fr':':surfeuse_peau_légèrement_mate:','ja':':サーフィンする女_中間の肌色:','ko':':서핑하는_여자_갈색_피부:','pt':':mulher_surfista_pele_morena:','it':':surfista_donna_carnagione_olivastra:','fa':':موج_سوار_زن_پوست_طلایی:','id':':wanita_berselancar_warna_kulit_sedang:','zh':':女生冲浪_中等肤色:','ru':':серфингистка_средний_тон_кожи:','tr':':sörf_yapan_kadın_orta_cilt_tonu:','ar':':راكبة_أمواج_بشرة_بلون_معتدل:'},'\U0001F3CA\U0000200D\U00002640\U0000FE0F':{'en':':woman_swimming:','status':fully_qualified,'E':4,'alias':[':swimming_woman:'],'de':':schwimmerin:','es':':mujer_nadando:','fr':':nageuse:','ja':':泳ぐ女:','ko':':수영하는_여자:','pt':':mulher_nadando:','it':':nuotatrice:','fa':':شناگر_زن:','id':':wanita_berenang:','zh':':女生游泳:','ru':':пловчиха:','tr':':yüzen_kadın:','ar':':سبَاحة:'},'\U0001F3CA\U0000200D\U00002640':{'en':':woman_swimming:','status':minimally_qualified,'E':4,'alias':[':swimming_woman:'],'de':':schwimmerin:','es':':mujer_nadando:','fr':':nageuse:','ja':':泳ぐ女:','ko':':수영하는_여자:','pt':':mulher_nadando:','it':':nuotatrice:','fa':':شناگر_زن:','id':':wanita_berenang:','zh':':女生游泳:','ru':':пловчиха:','tr':':yüzen_kadın:','ar':':سبَاحة:'},'\U0001F3CA\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_swimming_dark_skin_tone:','status':fully_qualified,'E':4,'de':':schwimmerin_dunkle_hautfarbe:','es':':mujer_nadando_tono_de_piel_oscuro:','fr':':nageuse_peau_foncée:','ja':':泳ぐ女_濃い肌色:','ko':':수영하는_여자_검은색_피부:','pt':':mulher_nadando_pele_escura:','it':':nuotatrice_carnagione_scura:','fa':':شناگر_زن_پوست_آبنوسی:','id':':wanita_berenang_warna_kulit_gelap:','zh':':女生游泳_较深肤色:','ru':':пловчиха_очень_темныи_тон_кожи:','tr':':yüzen_kadın_koyu_cilt_tonu:','ar':':سبَاحة_بشرة_بلون_غامق:'},'\U0001F3CA\U0001F3FF\U0000200D\U00002640':{'en':':woman_swimming_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':schwimmerin_dunkle_hautfarbe:','es':':mujer_nadando_tono_de_piel_oscuro:','fr':':nageuse_peau_foncée:','ja':':泳ぐ女_濃い肌色:','ko':':수영하는_여자_검은색_피부:','pt':':mulher_nadando_pele_escura:','it':':nuotatrice_carnagione_scura:','fa':':شناگر_زن_پوست_آبنوسی:','id':':wanita_berenang_warna_kulit_gelap:','zh':':女生游泳_较深肤色:','ru':':пловчиха_очень_темный_тон_кожи:','tr':':yüzen_kadın_koyu_cilt_tonu:','ar':':سبَاحة_بشرة_بلون_غامق:'},'\U0001F3CA\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_swimming_light_skin_tone:','status':fully_qualified,'E':4,'de':':schwimmerin_helle_hautfarbe:','es':':mujer_nadando_tono_de_piel_claro:','fr':':nageuse_peau_claire:','ja':':泳ぐ女_薄い肌色:','ko':':수영하는_여자_하얀_피부:','pt':':mulher_nadando_pele_clara:','it':':nuotatrice_carnagione_chiara:','fa':':شناگر_زن_پوست_سفید:','id':':wanita_berenang_warna_kulit_cerah:','zh':':女生游泳_较浅肤色:','ru':':пловчиха_очень_светлыи_тон_кожи:','tr':':yüzen_kadın_açık_cilt_tonu:','ar':':سبَاحة_بشرة_بلون_فاتح:'},'\U0001F3CA\U0001F3FB\U0000200D\U00002640':{'en':':woman_swimming_light_skin_tone:','status':minimally_qualified,'E':4,'de':':schwimmerin_helle_hautfarbe:','es':':mujer_nadando_tono_de_piel_claro:','fr':':nageuse_peau_claire:','ja':':泳ぐ女_薄い肌色:','ko':':수영하는_여자_하얀_피부:','pt':':mulher_nadando_pele_clara:','it':':nuotatrice_carnagione_chiara:','fa':':شناگر_زن_پوست_سفید:','id':':wanita_berenang_warna_kulit_cerah:','zh':':女生游泳_较浅肤色:','ru':':пловчиха_очень_светлый_тон_кожи:','tr':':yüzen_kadın_açık_cilt_tonu:','ar':':سبَاحة_بشرة_بلون_فاتح:'},'\U0001F3CA\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_swimming_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':schwimmerin_mitteldunkle_hautfarbe:','es':':mujer_nadando_tono_de_piel_oscuro_medio:','fr':':nageuse_peau_mate:','ja':':泳ぐ女_やや濃い肌色:','ko':':수영하는_여자_진한_갈색_피부:','pt':':mulher_nadando_pele_morena_escura:','it':':nuotatrice_carnagione_abbastanza_scura:','fa':':شناگر_زن_پوست_گندمی:','id':':wanita_berenang_warna_kulit_gelap-sedang:','zh':':女生游泳_中等深肤色:','ru':':пловчиха_темныи_тон_кожи:','tr':':yüzen_kadın_orta_koyu_cilt_tonu:','ar':':سبَاحة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3CA\U0001F3FE\U0000200D\U00002640':{'en':':woman_swimming_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':schwimmerin_mitteldunkle_hautfarbe:','es':':mujer_nadando_tono_de_piel_oscuro_medio:','fr':':nageuse_peau_mate:','ja':':泳ぐ女_やや濃い肌色:','ko':':수영하는_여자_진한_갈색_피부:','pt':':mulher_nadando_pele_morena_escura:','it':':nuotatrice_carnagione_abbastanza_scura:','fa':':شناگر_زن_پوست_گندمی:','id':':wanita_berenang_warna_kulit_gelap-sedang:','zh':':女生游泳_中等深肤色:','ru':':пловчиха_темный_тон_кожи:','tr':':yüzen_kadın_orta_koyu_cilt_tonu:','ar':':سبَاحة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F3CA\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_swimming_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':schwimmerin_mittelhelle_hautfarbe:','es':':mujer_nadando_tono_de_piel_claro_medio:','fr':':nageuse_peau_moyennement_claire:','ja':':泳ぐ女_やや薄い肌色:','ko':':수영하는_여자_연한_갈색_피부:','pt':':mulher_nadando_pele_morena_clara:','it':':nuotatrice_carnagione_abbastanza_chiara:','fa':':شناگر_زن_پوست_روشن:','id':':wanita_berenang_warna_kulit_cerah-sedang:','zh':':女生游泳_中等浅肤色:','ru':':пловчиха_светлыи_тон_кожи:','tr':':yüzen_kadın_orta_açık_cilt_tonu:','ar':':سبَاحة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3CA\U0001F3FC\U0000200D\U00002640':{'en':':woman_swimming_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':schwimmerin_mittelhelle_hautfarbe:','es':':mujer_nadando_tono_de_piel_claro_medio:','fr':':nageuse_peau_moyennement_claire:','ja':':泳ぐ女_やや薄い肌色:','ko':':수영하는_여자_연한_갈색_피부:','pt':':mulher_nadando_pele_morena_clara:','it':':nuotatrice_carnagione_abbastanza_chiara:','fa':':شناگر_زن_پوست_روشن:','id':':wanita_berenang_warna_kulit_cerah-sedang:','zh':':女生游泳_中等浅肤色:','ru':':пловчиха_светлый_тон_кожи:','tr':':yüzen_kadın_orta_açık_cilt_tonu:','ar':':سبَاحة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F3CA\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_swimming_medium_skin_tone:','status':fully_qualified,'E':4,'de':':schwimmerin_mittlere_hautfarbe:','es':':mujer_nadando_tono_de_piel_medio:','fr':':nageuse_peau_légèrement_mate:','ja':':泳ぐ女_中間の肌色:','ko':':수영하는_여자_갈색_피부:','pt':':mulher_nadando_pele_morena:','it':':nuotatrice_carnagione_olivastra:','fa':':شناگر_زن_پوست_طلایی:','id':':wanita_berenang_warna_kulit_sedang:','zh':':女生游泳_中等肤色:','ru':':пловчиха_среднии_тон_кожи:','tr':':yüzen_kadın_orta_cilt_tonu:','ar':':سبَاحة_بشرة_بلون_معتدل:'},'\U0001F3CA\U0001F3FD\U0000200D\U00002640':{'en':':woman_swimming_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':schwimmerin_mittlere_hautfarbe:','es':':mujer_nadando_tono_de_piel_medio:','fr':':nageuse_peau_légèrement_mate:','ja':':泳ぐ女_中間の肌色:','ko':':수영하는_여자_갈색_피부:','pt':':mulher_nadando_pele_morena:','it':':nuotatrice_carnagione_olivastra:','fa':':شناگر_زن_پوست_طلایی:','id':':wanita_berenang_warna_kulit_sedang:','zh':':女生游泳_中等肤色:','ru':':пловчиха_средний_тон_кожи:','tr':':yüzen_kadın_orta_cilt_tonu:','ar':':سبَاحة_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U0001F3EB':{'en':':woman_teacher:','status':fully_qualified,'E':4,'de':':lehrerin:','es':':docente_mujer:','fr':':enseignante:','ja':':女性の教師:','ko':':여교사:','pt':':professora:','it':':professoressa:','fa':':آموزگار_زن:','id':':pengajar_wanita:','zh':':女老师:','ru':':учительница:','tr':':kadın_öğretmen:','ar':':معلمة:'},'\U0001F469\U0001F3FF\U0000200D\U0001F3EB':{'en':':woman_teacher_dark_skin_tone:','status':fully_qualified,'E':4,'de':':lehrerin_dunkle_hautfarbe:','es':':docente_mujer_tono_de_piel_oscuro:','fr':':enseignante_peau_foncée:','ja':':女性の教師_濃い肌色:','ko':':여교사_검은색_피부:','pt':':professora_pele_escura:','it':':professoressa_carnagione_scura:','fa':':آموزگار_زن_پوست_آبنوسی:','id':':pengajar_wanita_warna_kulit_gelap:','zh':':女老师_较深肤色:','ru':':учительница_очень_темный_тон_кожи:','tr':':kadın_öğretmen_koyu_cilt_tonu:','ar':':معلمة_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F3EB':{'en':':woman_teacher_light_skin_tone:','status':fully_qualified,'E':4,'de':':lehrerin_helle_hautfarbe:','es':':docente_mujer_tono_de_piel_claro:','fr':':enseignante_peau_claire:','ja':':女性の教師_薄い肌色:','ko':':여교사_하얀_피부:','pt':':professora_pele_clara:','it':':professoressa_carnagione_chiara:','fa':':آموزگار_زن_پوست_سفید:','id':':pengajar_wanita_warna_kulit_cerah:','zh':':女老师_较浅肤色:','ru':':учительница_очень_светлый_тон_кожи:','tr':':kadın_öğretmen_açık_cilt_tonu:','ar':':معلمة_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F3EB':{'en':':woman_teacher_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':lehrerin_mitteldunkle_hautfarbe:','es':':docente_mujer_tono_de_piel_oscuro_medio:','fr':':enseignante_peau_mate:','ja':':女性の教師_やや濃い肌色:','ko':':여교사_진한_갈색_피부:','pt':':professora_pele_morena_escura:','it':':professoressa_carnagione_abbastanza_scura:','fa':':آموزگار_زن_پوست_گندمی:','id':':pengajar_wanita_warna_kulit_gelap-sedang:','zh':':女老师_中等深肤色:','ru':':учительница_темный_тон_кожи:','tr':':kadın_öğretmen_orta_koyu_cilt_tonu:','ar':':معلمة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F3EB':{'en':':woman_teacher_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':lehrerin_mittelhelle_hautfarbe:','es':':docente_mujer_tono_de_piel_claro_medio:','fr':':enseignante_peau_moyennement_claire:','ja':':女性の教師_やや薄い肌色:','ko':':여교사_연한_갈색_피부:','pt':':professora_pele_morena_clara:','it':':professoressa_carnagione_abbastanza_chiara:','fa':':آموزگار_زن_پوست_روشن:','id':':pengajar_wanita_warna_kulit_cerah-sedang:','zh':':女老师_中等浅肤色:','ru':':учительница_светлый_тон_кожи:','tr':':kadın_öğretmen_orta_açık_cilt_tonu:','ar':':معلمة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F3EB':{'en':':woman_teacher_medium_skin_tone:','status':fully_qualified,'E':4,'de':':lehrerin_mittlere_hautfarbe:','es':':docente_mujer_tono_de_piel_medio:','fr':':enseignante_peau_légèrement_mate:','ja':':女性の教師_中間の肌色:','ko':':여교사_갈색_피부:','pt':':professora_pele_morena:','it':':professoressa_carnagione_olivastra:','fa':':آموزگار_زن_پوست_طلایی:','id':':pengajar_wanita_warna_kulit_sedang:','zh':':女老师_中等肤色:','ru':':учительница_средний_тон_кожи:','tr':':kadın_öğretmen_orta_cilt_tonu:','ar':':معلمة_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U0001F4BB':{'en':':woman_technologist:','status':fully_qualified,'E':4,'de':':it-expertin:','es':':profesional_de_la_tecnología_mujer:','fr':':informaticienne:','ja':':女性技術者:','ko':':여자_기술_전문가:','pt':':tecnóloga:','it':':tecnologa:','fa':':کارشناس_فناوری_زن:','id':':ahli_teknologi_wanita:','zh':':女程序员:','ru':':женщина_инженер:','tr':':kadın_teknoloji_uzmanı:','ar':':عالِمة_تكنولوجيا:'},'\U0001F469\U0001F3FF\U0000200D\U0001F4BB':{'en':':woman_technologist_dark_skin_tone:','status':fully_qualified,'E':4,'de':':it-expertin_dunkle_hautfarbe:','es':':profesional_de_la_tecnología_mujer_tono_de_piel_oscuro:','fr':':informaticienne_peau_foncée:','ja':':女性技術者_濃い肌色:','ko':':여자_기술_전문가_검은색_피부:','pt':':tecnóloga_pele_escura:','it':':tecnologa_carnagione_scura:','fa':':کارشناس_فناوری_زن_پوست_آبنوسی:','id':':ahli_teknologi_wanita_warna_kulit_gelap:','zh':':女程序员_较深肤色:','ru':':женщина_инженер_очень_темный_тон_кожи:','tr':':kadın_teknoloji_uzmanı_koyu_cilt_tonu:','ar':':عالِمة_تكنولوجيا_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F4BB':{'en':':woman_technologist_light_skin_tone:','status':fully_qualified,'E':4,'de':':it-expertin_helle_hautfarbe:','es':':profesional_de_la_tecnología_mujer_tono_de_piel_claro:','fr':':informaticienne_peau_claire:','ja':':女性技術者_薄い肌色:','ko':':여자_기술_전문가_하얀_피부:','pt':':tecnóloga_pele_clara:','it':':tecnologa_carnagione_chiara:','fa':':کارشناس_فناوری_زن_پوست_سفید:','id':':ahli_teknologi_wanita_warna_kulit_cerah:','zh':':女程序员_较浅肤色:','ru':':женщина_инженер_очень_светлый_тон_кожи:','tr':':kadın_teknoloji_uzmanı_açık_cilt_tonu:','ar':':عالِمة_تكنولوجيا_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F4BB':{'en':':woman_technologist_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':it-expertin_mitteldunkle_hautfarbe:','es':':profesional_de_la_tecnología_mujer_tono_de_piel_oscuro_medio:','fr':':informaticienne_peau_mate:','ja':':女性技術者_やや濃い肌色:','ko':':여자_기술_전문가_진한_갈색_피부:','pt':':tecnóloga_pele_morena_escura:','it':':tecnologa_carnagione_abbastanza_scura:','fa':':کارشناس_فناوری_زن_پوست_گندمی:','id':':ahli_teknologi_wanita_warna_kulit_gelap-sedang:','zh':':女程序员_中等深肤色:','ru':':женщина_инженер_темный_тон_кожи:','tr':':kadın_teknoloji_uzmanı_orta_koyu_cilt_tonu:','ar':':عالِمة_تكنولوجيا_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F4BB':{'en':':woman_technologist_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':it-expertin_mittelhelle_hautfarbe:','es':':profesional_de_la_tecnología_mujer_tono_de_piel_claro_medio:','fr':':informaticienne_peau_moyennement_claire:','ja':':女性技術者_やや薄い肌色:','ko':':여자_기술_전문가_연한_갈색_피부:','pt':':tecnóloga_pele_morena_clara:','it':':tecnologa_carnagione_abbastanza_chiara:','fa':':کارشناس_فناوری_زن_پوست_روشن:','id':':ahli_teknologi_wanita_warna_kulit_cerah-sedang:','zh':':女程序员_中等浅肤色:','ru':':женщина_инженер_светлый_тон_кожи:','tr':':kadın_teknoloji_uzmanı_orta_açık_cilt_tonu:','ar':':عالِمة_تكنولوجيا_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F4BB':{'en':':woman_technologist_medium_skin_tone:','status':fully_qualified,'E':4,'de':':it-expertin_mittlere_hautfarbe:','es':':profesional_de_la_tecnología_mujer_tono_de_piel_medio:','fr':':informaticienne_peau_légèrement_mate:','ja':':女性技術者_中間の肌色:','ko':':여자_기술_전문가_갈색_피부:','pt':':tecnóloga_pele_morena:','it':':tecnologa_carnagione_olivastra:','fa':':کارشناس_فناوری_زن_پوست_طلایی:','id':':ahli_teknologi_wanita_warna_kulit_sedang:','zh':':女程序员_中等肤色:','ru':':женщина_инженер_средний_тон_кожи:','tr':':kadın_teknoloji_uzmanı_orta_cilt_tonu:','ar':':عالِمة_تكنولوجيا_بشرة_بلون_معتدل:'},'\U0001F481\U0000200D\U00002640\U0000FE0F':{'en':':woman_tipping_hand:','status':fully_qualified,'E':4,'alias':[':sassy_woman:', ':tipping_hand_woman:'],'de':':infoschalter-mitarbeiterin:','es':':empleada_de_mostrador_de_información:','fr':':femme_paume_vers_le_haut:','ja':':案内する女:','ko':':손바닥_꺾은_여자:','pt':':mulher_com_a_palma_virada_para_cima:','it':':donna_con_suggerimento:','fa':':زن_کمک_می_کند:','id':':wanita_menengadahkan_tangan:','zh':':前台女:','ru':':женщина_тянет_руку:','tr':':elini_yana_yatıran_kadın:','ar':':سيدة_بيد_مائلة:'},'\U0001F481\U0000200D\U00002640':{'en':':woman_tipping_hand:','status':minimally_qualified,'E':4,'alias':[':sassy_woman:', ':tipping_hand_woman:'],'de':':infoschalter-mitarbeiterin:','es':':empleada_de_mostrador_de_información:','fr':':femme_paume_vers_le_haut:','ja':':案内する女:','ko':':손바닥_꺾은_여자:','pt':':mulher_com_a_palma_virada_para_cima:','it':':donna_al_punto_informazioni:','fa':':زن_کمک_می_کند:','id':':wanita_menengadahkan_tangan:','zh':':前台女:','ru':':женщина_тянет_руку:','tr':':elini_yana_yatıran_kadın:','ar':':سيدة_بيد_مائلة:'},'\U0001F481\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_tipping_hand_dark_skin_tone:','status':fully_qualified,'E':4,'de':':infoschalter-mitarbeiterin_dunkle_hautfarbe:','es':':empleada_de_mostrador_de_información_tono_de_piel_oscuro:','fr':':femme_paume_vers_le_haut_peau_foncée:','ja':':案内する女_濃い肌色:','ko':':손바닥_꺾은_여자_검은색_피부:','pt':':mulher_com_a_palma_virada_para_cima_pele_escura:','it':':donna_con_suggerimento_carnagione_scura:','fa':':زن_کمک_می_کند_پوست_آبنوسی:','id':':wanita_menengadahkan_tangan_warna_kulit_gelap:','zh':':前台女_较深肤色:','ru':':женщина_тянет_руку_очень_темныи_тон_кожи:','tr':':elini_yana_yatıran_kadın_koyu_cilt_tonu:','ar':':سيدة_بيد_مائلة_بشرة_بلون_غامق:'},'\U0001F481\U0001F3FF\U0000200D\U00002640':{'en':':woman_tipping_hand_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':infoschalter-mitarbeiterin_dunkle_hautfarbe:','es':':empleada_de_mostrador_de_información_tono_de_piel_oscuro:','fr':':femme_paume_vers_le_haut_peau_foncée:','ja':':案内する女_濃い肌色:','ko':':손바닥_꺾은_여자_검은색_피부:','pt':':mulher_com_a_palma_virada_para_cima_pele_escura:','it':':donna_al_punto_informazioni_carnagione_scura:','fa':':زن_کمک_می_کند_پوست_آبنوسی:','id':':wanita_menengadahkan_tangan_warna_kulit_gelap:','zh':':前台女_较深肤色:','ru':':женщина_тянет_руку_очень_темный_тон_кожи:','tr':':elini_yana_yatıran_kadın_koyu_cilt_tonu:','ar':':سيدة_بيد_مائلة_بشرة_بلون_غامق:'},'\U0001F481\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_tipping_hand_light_skin_tone:','status':fully_qualified,'E':4,'de':':infoschalter-mitarbeiterin_helle_hautfarbe:','es':':empleada_de_mostrador_de_información_tono_de_piel_claro:','fr':':femme_paume_vers_le_haut_peau_claire:','ja':':案内する女_薄い肌色:','ko':':손바닥_꺾은_여자_하얀_피부:','pt':':mulher_com_a_palma_virada_para_cima_pele_clara:','it':':donna_con_suggerimento_carnagione_chiara:','fa':':زن_کمک_می_کند_پوست_سفید:','id':':wanita_menengadahkan_tangan_warna_kulit_cerah:','zh':':前台女_较浅肤色:','ru':':женщина_тянет_руку_очень_светлыи_тон_кожи:','tr':':elini_yana_yatıran_kadın_açık_cilt_tonu:','ar':':سيدة_بيد_مائلة_بشرة_بلون_فاتح:'},'\U0001F481\U0001F3FB\U0000200D\U00002640':{'en':':woman_tipping_hand_light_skin_tone:','status':minimally_qualified,'E':4,'de':':infoschalter-mitarbeiterin_helle_hautfarbe:','es':':empleada_de_mostrador_de_información_tono_de_piel_claro:','fr':':femme_paume_vers_le_haut_peau_claire:','ja':':案内する女_薄い肌色:','ko':':손바닥_꺾은_여자_하얀_피부:','pt':':mulher_com_a_palma_virada_para_cima_pele_clara:','it':':donna_al_punto_informazioni_carnagione_chiara:','fa':':زن_کمک_می_کند_پوست_سفید:','id':':wanita_menengadahkan_tangan_warna_kulit_cerah:','zh':':前台女_较浅肤色:','ru':':женщина_тянет_руку_очень_светлый_тон_кожи:','tr':':elini_yana_yatıran_kadın_açık_cilt_tonu:','ar':':سيدة_بيد_مائلة_بشرة_بلون_فاتح:'},'\U0001F481\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_tipping_hand_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':infoschalter-mitarbeiterin_mitteldunkle_hautfarbe:','es':':empleada_de_mostrador_de_información_tono_de_piel_oscuro_medio:','fr':':femme_paume_vers_le_haut_peau_mate:','ja':':案内する女_やや濃い肌色:','ko':':손바닥_꺾은_여자_진한_갈색_피부:','pt':':mulher_com_a_palma_virada_para_cima_pele_morena_escura:','it':':donna_con_suggerimento_carnagione_abbastanza_scura:','fa':':زن_کمک_می_کند_پوست_گندمی:','id':':wanita_menengadahkan_tangan_warna_kulit_gelap-sedang:','zh':':前台女_中等深肤色:','ru':':женщина_тянет_руку_темныи_тон_кожи:','tr':':elini_yana_yatıran_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_بيد_مائلة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F481\U0001F3FE\U0000200D\U00002640':{'en':':woman_tipping_hand_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':infoschalter-mitarbeiterin_mitteldunkle_hautfarbe:','es':':empleada_de_mostrador_de_información_tono_de_piel_oscuro_medio:','fr':':femme_paume_vers_le_haut_peau_mate:','ja':':案内する女_やや濃い肌色:','ko':':손바닥_꺾은_여자_진한_갈색_피부:','pt':':mulher_com_a_palma_virada_para_cima_pele_morena_escura:','it':':donna_al_punto_informazioni_carnagione_abbastanza_scura:','fa':':زن_کمک_می_کند_پوست_گندمی:','id':':wanita_menengadahkan_tangan_warna_kulit_gelap-sedang:','zh':':前台女_中等深肤色:','ru':':женщина_тянет_руку_темный_тон_кожи:','tr':':elini_yana_yatıran_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_بيد_مائلة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F481\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_tipping_hand_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':infoschalter-mitarbeiterin_mittelhelle_hautfarbe:','es':':empleada_de_mostrador_de_información_tono_de_piel_claro_medio:','fr':':femme_paume_vers_le_haut_peau_moyennement_claire:','ja':':案内する女_やや薄い肌色:','ko':':손바닥_꺾은_여자_연한_갈색_피부:','pt':':mulher_com_a_palma_virada_para_cima_pele_morena_clara:','it':':donna_con_suggerimento_carnagione_abbastanza_chiara:','fa':':زن_کمک_می_کند_پوست_روشن:','id':':wanita_menengadahkan_tangan_warna_kulit_cerah-sedang:','zh':':前台女_中等浅肤色:','ru':':женщина_тянет_руку_светлыи_тон_кожи:','tr':':elini_yana_yatıran_kadın_orta_açık_cilt_tonu:','ar':':سيدة_بيد_مائلة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F481\U0001F3FC\U0000200D\U00002640':{'en':':woman_tipping_hand_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':infoschalter-mitarbeiterin_mittelhelle_hautfarbe:','es':':empleada_de_mostrador_de_información_tono_de_piel_claro_medio:','fr':':femme_paume_vers_le_haut_peau_moyennement_claire:','ja':':案内する女_やや薄い肌色:','ko':':손바닥_꺾은_여자_연한_갈색_피부:','pt':':mulher_com_a_palma_virada_para_cima_pele_morena_clara:','it':':donna_al_punto_informazioni_carnagione_abbastanza_chiara:','fa':':زن_کمک_می_کند_پوست_روشن:','id':':wanita_menengadahkan_tangan_warna_kulit_cerah-sedang:','zh':':前台女_中等浅肤色:','ru':':женщина_тянет_руку_светлый_тон_кожи:','tr':':elini_yana_yatıran_kadın_orta_açık_cilt_tonu:','ar':':سيدة_بيد_مائلة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F481\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_tipping_hand_medium_skin_tone:','status':fully_qualified,'E':4,'de':':infoschalter-mitarbeiterin_mittlere_hautfarbe:','es':':empleada_de_mostrador_de_información_tono_de_piel_medio:','fr':':femme_paume_vers_le_haut_peau_légèrement_mate:','ja':':案内する女_中間の肌色:','ko':':손바닥_꺾은_여자_갈색_피부:','pt':':mulher_com_a_palma_virada_para_cima_pele_morena:','it':':donna_con_suggerimento_carnagione_olivastra:','fa':':زن_کمک_می_کند_پوست_طلایی:','id':':wanita_menengadahkan_tangan_warna_kulit_sedang:','zh':':前台女_中等肤色:','ru':':женщина_тянет_руку_среднии_тон_кожи:','tr':':elini_yana_yatıran_kadın_orta_cilt_tonu:','ar':':سيدة_بيد_مائلة_بشرة_بلون_معتدل:'},'\U0001F481\U0001F3FD\U0000200D\U00002640':{'en':':woman_tipping_hand_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':infoschalter-mitarbeiterin_mittlere_hautfarbe:','es':':empleada_de_mostrador_de_información_tono_de_piel_medio:','fr':':femme_paume_vers_le_haut_peau_légèrement_mate:','ja':':案内する女_中間の肌色:','ko':':손바닥_꺾은_여자_갈색_피부:','pt':':mulher_com_a_palma_virada_para_cima_pele_morena:','it':':donna_al_punto_informazioni_carnagione_olivastra:','fa':':زن_کمک_می_کند_پوست_طلایی:','id':':wanita_menengadahkan_tangan_warna_kulit_sedang:','zh':':前台女_中等肤色:','ru':':женщина_тянет_руку_средний_тон_кожи:','tr':':elini_yana_yatıran_kadın_orta_cilt_tonu:','ar':':سيدة_بيد_مائلة_بشرة_بلون_معتدل:'},'\U0001F9DB\U0000200D\U00002640\U0000FE0F':{'en':':woman_vampire:','status':fully_qualified,'E':5,'alias':[':vampire_woman:'],'de':':weiblicher_vampir:','es':':vampiresa:','fr':':vampire_femme:','ja':':女の吸血鬼:','ko':':여자_뱀파이어:','pt':':mulher_vampira:','it':':vampira:','fa':':دراکولای_زن:','id':':vampir_wanita:','zh':':女吸血鬼:','ru':':женщина-вампир:','tr':':kadın_vampir:','ar':':مصاصة_دماء:'},'\U0001F9DB\U0000200D\U00002640':{'en':':woman_vampire:','status':minimally_qualified,'E':5,'alias':[':vampire_woman:'],'de':':weiblicher_vampir:','es':':vampiresa:','fr':':vampire_femme:','ja':':女の吸血鬼:','ko':':여자_뱀파이어:','pt':':mulher_vampira:','it':':vampira:','fa':':دراکولای_زن:','id':':vampir_wanita:','zh':':女吸血鬼:','ru':':женщина_вампир:','tr':':kadın_vampir:','ar':':مصاصة_دماء:'},'\U0001F9DB\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_vampire_dark_skin_tone:','status':fully_qualified,'E':5,'de':':weiblicher_vampir_dunkle_hautfarbe:','es':':vampiresa_tono_de_piel_oscuro:','fr':':vampire_femme_peau_foncée:','ja':':女の吸血鬼_濃い肌色:','ko':':여자_뱀파이어_검은색_피부:','pt':':mulher_vampira_pele_escura:','it':':vampira_carnagione_scura:','fa':':دراکولای_زن_پوست_آبنوسی:','id':':vampir_wanita_warna_kulit_gelap:','zh':':女吸血鬼_较深肤色:','ru':':женщина-вампир_очень_темныи_тон_кожи:','tr':':kadın_vampir_koyu_cilt_tonu:','ar':':مصاصة_دماء_بشرة_بلون_غامق:'},'\U0001F9DB\U0001F3FF\U0000200D\U00002640':{'en':':woman_vampire_dark_skin_tone:','status':minimally_qualified,'E':5,'de':':weiblicher_vampir_dunkle_hautfarbe:','es':':vampiresa_tono_de_piel_oscuro:','fr':':vampire_femme_peau_foncée:','ja':':女の吸血鬼_濃い肌色:','ko':':여자_뱀파이어_검은색_피부:','pt':':mulher_vampira_pele_escura:','it':':vampira_carnagione_scura:','fa':':دراکولای_زن_پوست_آبنوسی:','id':':vampir_wanita_warna_kulit_gelap:','zh':':女吸血鬼_较深肤色:','ru':':женщина_вампир_очень_темный_тон_кожи:','tr':':kadın_vampir_koyu_cilt_tonu:','ar':':مصاصة_دماء_بشرة_بلون_غامق:'},'\U0001F9DB\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_vampire_light_skin_tone:','status':fully_qualified,'E':5,'de':':weiblicher_vampir_helle_hautfarbe:','es':':vampiresa_tono_de_piel_claro:','fr':':vampire_femme_peau_claire:','ja':':女の吸血鬼_薄い肌色:','ko':':여자_뱀파이어_하얀_피부:','pt':':mulher_vampira_pele_clara:','it':':vampira_carnagione_chiara:','fa':':دراکولای_زن_پوست_سفید:','id':':vampir_wanita_warna_kulit_cerah:','zh':':女吸血鬼_较浅肤色:','ru':':женщина-вампир_очень_светлыи_тон_кожи:','tr':':kadın_vampir_açık_cilt_tonu:','ar':':مصاصة_دماء_بشرة_بلون_فاتح:'},'\U0001F9DB\U0001F3FB\U0000200D\U00002640':{'en':':woman_vampire_light_skin_tone:','status':minimally_qualified,'E':5,'de':':weiblicher_vampir_helle_hautfarbe:','es':':vampiresa_tono_de_piel_claro:','fr':':vampire_femme_peau_claire:','ja':':女の吸血鬼_薄い肌色:','ko':':여자_뱀파이어_하얀_피부:','pt':':mulher_vampira_pele_clara:','it':':vampira_carnagione_chiara:','fa':':دراکولای_زن_پوست_سفید:','id':':vampir_wanita_warna_kulit_cerah:','zh':':女吸血鬼_较浅肤色:','ru':':женщина_вампир_очень_светлый_тон_кожи:','tr':':kadın_vampir_açık_cilt_tonu:','ar':':مصاصة_دماء_بشرة_بلون_فاتح:'},'\U0001F9DB\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_vampire_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':weiblicher_vampir_mitteldunkle_hautfarbe:','es':':vampiresa_tono_de_piel_oscuro_medio:','fr':':vampire_femme_peau_mate:','ja':':女の吸血鬼_やや濃い肌色:','ko':':여자_뱀파이어_진한_갈색_피부:','pt':':mulher_vampira_pele_morena_escura:','it':':vampira_carnagione_abbastanza_scura:','fa':':دراکولای_زن_پوست_گندمی:','id':':vampir_wanita_warna_kulit_gelap-sedang:','zh':':女吸血鬼_中等深肤色:','ru':':женщина-вампир_темныи_тон_кожи:','tr':':kadın_vampir_orta_koyu_cilt_tonu:','ar':':مصاصة_دماء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9DB\U0001F3FE\U0000200D\U00002640':{'en':':woman_vampire_medium-dark_skin_tone:','status':minimally_qualified,'E':5,'de':':weiblicher_vampir_mitteldunkle_hautfarbe:','es':':vampiresa_tono_de_piel_oscuro_medio:','fr':':vampire_femme_peau_mate:','ja':':女の吸血鬼_やや濃い肌色:','ko':':여자_뱀파이어_진한_갈색_피부:','pt':':mulher_vampira_pele_morena_escura:','it':':vampira_carnagione_abbastanza_scura:','fa':':دراکولای_زن_پوست_گندمی:','id':':vampir_wanita_warna_kulit_gelap-sedang:','zh':':女吸血鬼_中等深肤色:','ru':':женщина_вампир_темный_тон_кожи:','tr':':kadın_vampir_orta_koyu_cilt_tonu:','ar':':مصاصة_دماء_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9DB\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_vampire_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':weiblicher_vampir_mittelhelle_hautfarbe:','es':':vampiresa_tono_de_piel_claro_medio:','fr':':vampire_femme_peau_moyennement_claire:','ja':':女の吸血鬼_やや薄い肌色:','ko':':여자_뱀파이어_연한_갈색_피부:','pt':':mulher_vampira_pele_morena_clara:','it':':vampira_carnagione_abbastanza_chiara:','fa':':دراکولای_زن_پوست_روشن:','id':':vampir_wanita_warna_kulit_cerah-sedang:','zh':':女吸血鬼_中等浅肤色:','ru':':женщина-вампир_светлыи_тон_кожи:','tr':':kadın_vampir_orta_açık_cilt_tonu:','ar':':مصاصة_دماء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9DB\U0001F3FC\U0000200D\U00002640':{'en':':woman_vampire_medium-light_skin_tone:','status':minimally_qualified,'E':5,'de':':weiblicher_vampir_mittelhelle_hautfarbe:','es':':vampiresa_tono_de_piel_claro_medio:','fr':':vampire_femme_peau_moyennement_claire:','ja':':女の吸血鬼_やや薄い肌色:','ko':':여자_뱀파이어_연한_갈색_피부:','pt':':mulher_vampira_pele_morena_clara:','it':':vampira_carnagione_abbastanza_chiara:','fa':':دراکولای_زن_پوست_روشن:','id':':vampir_wanita_warna_kulit_cerah-sedang:','zh':':女吸血鬼_中等浅肤色:','ru':':женщина_вампир_светлый_тон_кожи:','tr':':kadın_vampir_orta_açık_cilt_tonu:','ar':':مصاصة_دماء_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9DB\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_vampire_medium_skin_tone:','status':fully_qualified,'E':5,'de':':weiblicher_vampir_mittlere_hautfarbe:','es':':vampiresa_tono_de_piel_medio:','fr':':vampire_femme_peau_légèrement_mate:','ja':':女の吸血鬼_中間の肌色:','ko':':여자_뱀파이어_갈색_피부:','pt':':mulher_vampira_pele_morena:','it':':vampira_carnagione_olivastra:','fa':':دراکولای_زن_پوست_طلایی:','id':':vampir_wanita_warna_kulit_sedang:','zh':':女吸血鬼_中等肤色:','ru':':женщина-вампир_среднии_тон_кожи:','tr':':kadın_vampir_orta_cilt_tonu:','ar':':مصاصة_دماء_بشرة_بلون_معتدل:'},'\U0001F9DB\U0001F3FD\U0000200D\U00002640':{'en':':woman_vampire_medium_skin_tone:','status':minimally_qualified,'E':5,'de':':weiblicher_vampir_mittlere_hautfarbe:','es':':vampiresa_tono_de_piel_medio:','fr':':vampire_femme_peau_légèrement_mate:','ja':':女の吸血鬼_中間の肌色:','ko':':여자_뱀파이어_갈색_피부:','pt':':mulher_vampira_pele_morena:','it':':vampira_carnagione_olivastra:','fa':':دراکولای_زن_پوست_طلایی:','id':':vampir_wanita_warna_kulit_sedang:','zh':':女吸血鬼_中等肤色:','ru':':женщина_вампир_средний_тон_кожи:','tr':':kadın_vampir_orta_cilt_tonu:','ar':':مصاصة_دماء_بشرة_بلون_معتدل:'},'\U0001F6B6\U0000200D\U00002640\U0000FE0F':{'en':':woman_walking:','status':fully_qualified,'E':4,'alias':[':walking_woman:'],'de':':fußgängerin:','es':':mujer_caminando:','fr':':femme_qui_marche:','ja':':歩く女:','ko':':걷는_여자:','pt':':mulher_andando:','it':':donna_che_cammina:','fa':':زن_پیاده_روی_می_کند:','id':':wanita_berjalan_kaki:','zh':':女行人:','ru':':женщина_идет:','tr':':yürüyen_kadın:','ar':':سائرة:'},'\U0001F6B6\U0000200D\U00002640':{'en':':woman_walking:','status':minimally_qualified,'E':4,'alias':[':walking_woman:'],'de':':fußgängerin:','es':':mujer_caminando:','fr':':femme_qui_marche:','ja':':歩く女:','ko':':걷는_여자:','pt':':mulher_andando:','it':':donna_che_cammina:','fa':':زن_پیاده_روی_می_کند:','id':':wanita_berjalan_kaki:','zh':':女行人:','ru':':женщина_идет:','tr':':yürüyen_kadın:','ar':':سائرة:'},'\U0001F6B6\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_walking_dark_skin_tone:','status':fully_qualified,'E':4,'de':':fußgängerin_dunkle_hautfarbe:','es':':mujer_caminando_tono_de_piel_oscuro:','fr':':femme_qui_marche_peau_foncée:','ja':':歩く女_濃い肌色:','ko':':걷는_여자_검은색_피부:','pt':':mulher_andando_pele_escura:','it':':donna_che_cammina_carnagione_scura:','fa':':زن_پیاده_روی_می_کند_پوست_آبنوسی:','id':':wanita_berjalan_kaki_warna_kulit_gelap:','zh':':女行人_较深肤色:','ru':':женщина_идет_очень_темныи_тон_кожи:','tr':':yürüyen_kadın_koyu_cilt_tonu:','ar':':سائرة_بشرة_بلون_غامق:'},'\U0001F6B6\U0001F3FF\U0000200D\U00002640':{'en':':woman_walking_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':fußgängerin_dunkle_hautfarbe:','es':':mujer_caminando_tono_de_piel_oscuro:','fr':':femme_qui_marche_peau_foncée:','ja':':歩く女_濃い肌色:','ko':':걷는_여자_검은색_피부:','pt':':mulher_andando_pele_escura:','it':':donna_che_cammina_carnagione_scura:','fa':':زن_پیاده_روی_می_کند_پوست_آبنوسی:','id':':wanita_berjalan_kaki_warna_kulit_gelap:','zh':':女行人_较深肤色:','ru':':женщина_идет_очень_темный_тон_кожи:','tr':':yürüyen_kadın_koyu_cilt_tonu:','ar':':سائرة_بشرة_بلون_غامق:'},'\U0001F6B6\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':woman_walking_facing_right:','status':fully_qualified,'E':15.1,'de':':fußgängerin_nach_rechts:','es':':mujer_caminando_hacia_la_derecha:','fr':':femme_qui_marche_vers_la_droite:','ja':':歩く女_右向き:','ko':':오른쪽을_향한_걷는_여자:','pt':':mulher_andando_de_frente_para_a_direita:','it':':donna_che_cammina_a_destra:','fa':':زن_پیاده_روی_می_کند_به_سمت_راست:','id':':wanita_berjalan_kaki_menghadap_kanan:','zh':':女行人面向右边:','ru':':женщина_идет_направленные_вправо:','tr':':sağa_dönük_yürüyen_kadın:','ar':':سائرة_متجه_لليمين:'},'\U0001F6B6\U0000200D\U00002640\U0000200D\U000027A1\U0000FE0F':{'en':':woman_walking_facing_right:','status':minimally_qualified,'E':15.1,'de':':fußgängerin_nach_rechts:','es':':mujer_caminando_hacia_la_derecha:','fr':':femme_qui_marche_vers_la_droite:','ja':':歩く女_右向き:','ko':':오른쪽을_향한_걷는_여자:','pt':':mulher_andando_de_frente_para_a_direita:','it':':donna_che_cammina_a_destra:','fa':':زن_پیاده_روی_می_کند_به_سمت_راست:','id':':wanita_berjalan_kaki_menghadap_kanan:','zh':':女行人面向右边:','ru':':женщина_идет_направленные_вправо:','tr':':sağa_dönük_yürüyen_kadın:','ar':':سائرة_متجه_لليمين:'},'\U0001F6B6\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1':{'en':':woman_walking_facing_right:','status':minimally_qualified,'E':15.1,'de':':fußgängerin_nach_rechts:','es':':mujer_caminando_hacia_la_derecha:','fr':':femme_qui_marche_vers_la_droite:','ja':':歩く女_右向き:','ko':':오른쪽을_향한_걷는_여자:','pt':':mulher_andando_de_frente_para_a_direita:','it':':donna_che_cammina_a_destra:','fa':':زن_پیاده_روی_می_کند_به_سمت_راست:','id':':wanita_berjalan_kaki_menghadap_kanan:','zh':':女行人面向右边:','ru':':женщина_идет_направленные_вправо:','tr':':sağa_dönük_yürüyen_kadın:','ar':':سائرة_متجه_لليمين:'},'\U0001F6B6\U0000200D\U00002640\U0000200D\U000027A1':{'en':':woman_walking_facing_right:','status':minimally_qualified,'E':15.1,'de':':fußgängerin_nach_rechts:','es':':mujer_caminando_hacia_la_derecha:','fr':':femme_qui_marche_vers_la_droite:','ja':':歩く女_右向き:','ko':':오른쪽을_향한_걷는_여자:','pt':':mulher_andando_de_frente_para_a_direita:','it':':donna_che_cammina_a_destra:','fa':':زن_پیاده_روی_می_کند_به_سمت_راست:','id':':wanita_berjalan_kaki_menghadap_kanan:','zh':':女行人面向右边:','ru':':женщина_идет_направленные_вправо:','tr':':sağa_dönük_yürüyen_kadın:','ar':':سائرة_متجه_لليمين:'},'\U0001F6B6\U0001F3FF\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':woman_walking_facing_right_dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':fußgängerin_nach_rechts_dunkle_hautfarbe:','es':':mujer_caminando_hacia_la_derecha_tono_de_piel_oscuro:','fr':':femme_qui_marche_vers_la_droite_peau_foncée:','ja':':歩く女_右向き_濃い肌色:','ko':':오른쪽을_향한_걷는_여자_검은색_피부:','pt':':mulher_andando_de_frente_para_a_direita_pele_escura:','it':':donna_che_cammina_a_destra_carnagione_scura:','fa':':زن_پیاده_روی_می_کند_به_سمت_راستپوست_آبنوسی:','id':':wanita_berjalan_kaki_menghadap_kanan_warna_kulit_gelap:','zh':':女行人面向右边_较深肤色:','ru':':женщина_идет_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_yürüyen_kadın_koyu_cilt_tonu:','ar':':سائرة_متجه_لليمين_بلون_غامق:'},'\U0001F6B6\U0001F3FF\U0000200D\U00002640\U0000200D\U000027A1\U0000FE0F':{'en':':woman_walking_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgängerin_nach_rechts_dunkle_hautfarbe:','es':':mujer_caminando_hacia_la_derecha_tono_de_piel_oscuro:','fr':':femme_qui_marche_vers_la_droite_peau_foncée:','ja':':歩く女_右向き_濃い肌色:','ko':':오른쪽을_향한_걷는_여자_검은색_피부:','pt':':mulher_andando_de_frente_para_a_direita_pele_escura:','it':':donna_che_cammina_a_destra_carnagione_scura:','fa':':زن_پیاده_روی_می_کند_به_سمت_راستپوست_آبنوسی:','id':':wanita_berjalan_kaki_menghadap_kanan_warna_kulit_gelap:','zh':':女行人面向右边_较深肤色:','ru':':женщина_идет_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_yürüyen_kadın_koyu_cilt_tonu:','ar':':سائرة_متجه_لليمين_بلون_غامق:'},'\U0001F6B6\U0001F3FF\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1':{'en':':woman_walking_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgängerin_nach_rechts_dunkle_hautfarbe:','es':':mujer_caminando_hacia_la_derecha_tono_de_piel_oscuro:','fr':':femme_qui_marche_vers_la_droite_peau_foncée:','ja':':歩く女_右向き_濃い肌色:','ko':':오른쪽을_향한_걷는_여자_검은색_피부:','pt':':mulher_andando_de_frente_para_a_direita_pele_escura:','it':':donna_che_cammina_a_destra_carnagione_scura:','fa':':زن_پیاده_روی_می_کند_به_سمت_راستپوست_آبنوسی:','id':':wanita_berjalan_kaki_menghadap_kanan_warna_kulit_gelap:','zh':':女行人面向右边_较深肤色:','ru':':женщина_идет_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_yürüyen_kadın_koyu_cilt_tonu:','ar':':سائرة_متجه_لليمين_بلون_غامق:'},'\U0001F6B6\U0001F3FF\U0000200D\U00002640\U0000200D\U000027A1':{'en':':woman_walking_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgängerin_nach_rechts_dunkle_hautfarbe:','es':':mujer_caminando_hacia_la_derecha_tono_de_piel_oscuro:','fr':':femme_qui_marche_vers_la_droite_peau_foncée:','ja':':歩く女_右向き_濃い肌色:','ko':':오른쪽을_향한_걷는_여자_검은색_피부:','pt':':mulher_andando_de_frente_para_a_direita_pele_escura:','it':':donna_che_cammina_a_destra_carnagione_scura:','fa':':زن_پیاده_روی_می_کند_به_سمت_راستپوست_آبنوسی:','id':':wanita_berjalan_kaki_menghadap_kanan_warna_kulit_gelap:','zh':':女行人面向右边_较深肤色:','ru':':женщина_идет_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_yürüyen_kadın_koyu_cilt_tonu:','ar':':سائرة_متجه_لليمين_بلون_غامق:'},'\U0001F6B6\U0001F3FB\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':woman_walking_facing_right_light_skin_tone:','status':fully_qualified,'E':15.1,'de':':fußgängerin_nach_rechts_helle_hautfarbe:','es':':mujer_caminando_hacia_la_derecha_tono_de_piel_claro:','fr':':femme_qui_marche_vers_la_droite_peau_claire:','ja':':歩く女_右向き_薄い肌色:','ko':':오른쪽을_향한_걷는_여자_하얀_피부:','pt':':mulher_andando_de_frente_para_a_direita_pele_clara:','it':':donna_che_cammina_a_destra_carnagione_chiara:','fa':':زن_پیاده_روی_می_کند_به_سمت_راست_پوست_سفید:','id':':wanita_berjalan_kaki_menghadap_kanan_warna_kulit_cerah:','zh':':女行人面向右边_较浅肤色:','ru':':женщина_идет_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_yürüyen_kadın_açık_cilt_tonu:','ar':':سائرة_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F6B6\U0001F3FB\U0000200D\U00002640\U0000200D\U000027A1\U0000FE0F':{'en':':woman_walking_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgängerin_nach_rechts_helle_hautfarbe:','es':':mujer_caminando_hacia_la_derecha_tono_de_piel_claro:','fr':':femme_qui_marche_vers_la_droite_peau_claire:','ja':':歩く女_右向き_薄い肌色:','ko':':오른쪽을_향한_걷는_여자_하얀_피부:','pt':':mulher_andando_de_frente_para_a_direita_pele_clara:','it':':donna_che_cammina_a_destra_carnagione_chiara:','fa':':زن_پیاده_روی_می_کند_به_سمت_راست_پوست_سفید:','id':':wanita_berjalan_kaki_menghadap_kanan_warna_kulit_cerah:','zh':':女行人面向右边_较浅肤色:','ru':':женщина_идет_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_yürüyen_kadın_açık_cilt_tonu:','ar':':سائرة_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F6B6\U0001F3FB\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1':{'en':':woman_walking_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgängerin_nach_rechts_helle_hautfarbe:','es':':mujer_caminando_hacia_la_derecha_tono_de_piel_claro:','fr':':femme_qui_marche_vers_la_droite_peau_claire:','ja':':歩く女_右向き_薄い肌色:','ko':':오른쪽을_향한_걷는_여자_하얀_피부:','pt':':mulher_andando_de_frente_para_a_direita_pele_clara:','it':':donna_che_cammina_a_destra_carnagione_chiara:','fa':':زن_پیاده_روی_می_کند_به_سمت_راست_پوست_سفید:','id':':wanita_berjalan_kaki_menghadap_kanan_warna_kulit_cerah:','zh':':女行人面向右边_较浅肤色:','ru':':женщина_идет_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_yürüyen_kadın_açık_cilt_tonu:','ar':':سائرة_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F6B6\U0001F3FB\U0000200D\U00002640\U0000200D\U000027A1':{'en':':woman_walking_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgängerin_nach_rechts_helle_hautfarbe:','es':':mujer_caminando_hacia_la_derecha_tono_de_piel_claro:','fr':':femme_qui_marche_vers_la_droite_peau_claire:','ja':':歩く女_右向き_薄い肌色:','ko':':오른쪽을_향한_걷는_여자_하얀_피부:','pt':':mulher_andando_de_frente_para_a_direita_pele_clara:','it':':donna_che_cammina_a_destra_carnagione_chiara:','fa':':زن_پیاده_روی_می_کند_به_سمت_راست_پوست_سفید:','id':':wanita_berjalan_kaki_menghadap_kanan_warna_kulit_cerah:','zh':':女行人面向右边_较浅肤色:','ru':':женщина_идет_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_yürüyen_kadın_açık_cilt_tonu:','ar':':سائرة_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F6B6\U0001F3FE\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':woman_walking_facing_right_medium-dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':fußgängerin_nach_rechts_mitteldunkle_hautfarbe:','es':':mujer_caminando_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':femme_qui_marche_vers_la_droite_peau_mate:','ja':':歩く女_右向き_やや濃い肌色:','ko':':오른쪽을_향한_걷는_여자_진한_갈색_피부:','pt':':mulher_andando_de_frente_para_a_direita_pele_morena_escura:','it':':donna_che_cammina_a_destra_carnagione_abbastanza_scura:','fa':':زن_پیاده_روی_می_کند_به_سمت_راست_پوست_گندمی:','id':':wanita_berjalan_kaki_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':女行人面向右边_中等深肤色:','ru':':женщина_идет_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_yürüyen_kadın_orta_koyu_cilt_tonu:','ar':':سائرة_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F6B6\U0001F3FE\U0000200D\U00002640\U0000200D\U000027A1\U0000FE0F':{'en':':woman_walking_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgängerin_nach_rechts_mitteldunkle_hautfarbe:','es':':mujer_caminando_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':femme_qui_marche_vers_la_droite_peau_mate:','ja':':歩く女_右向き_やや濃い肌色:','ko':':오른쪽을_향한_걷는_여자_진한_갈색_피부:','pt':':mulher_andando_de_frente_para_a_direita_pele_morena_escura:','it':':donna_che_cammina_a_destra_carnagione_abbastanza_scura:','fa':':زن_پیاده_روی_می_کند_به_سمت_راست_پوست_گندمی:','id':':wanita_berjalan_kaki_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':女行人面向右边_中等深肤色:','ru':':женщина_идет_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_yürüyen_kadın_orta_koyu_cilt_tonu:','ar':':سائرة_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F6B6\U0001F3FE\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1':{'en':':woman_walking_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgängerin_nach_rechts_mitteldunkle_hautfarbe:','es':':mujer_caminando_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':femme_qui_marche_vers_la_droite_peau_mate:','ja':':歩く女_右向き_やや濃い肌色:','ko':':오른쪽을_향한_걷는_여자_진한_갈색_피부:','pt':':mulher_andando_de_frente_para_a_direita_pele_morena_escura:','it':':donna_che_cammina_a_destra_carnagione_abbastanza_scura:','fa':':زن_پیاده_روی_می_کند_به_سمت_راست_پوست_گندمی:','id':':wanita_berjalan_kaki_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':女行人面向右边_中等深肤色:','ru':':женщина_идет_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_yürüyen_kadın_orta_koyu_cilt_tonu:','ar':':سائرة_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F6B6\U0001F3FE\U0000200D\U00002640\U0000200D\U000027A1':{'en':':woman_walking_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgängerin_nach_rechts_mitteldunkle_hautfarbe:','es':':mujer_caminando_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':femme_qui_marche_vers_la_droite_peau_mate:','ja':':歩く女_右向き_やや濃い肌色:','ko':':오른쪽을_향한_걷는_여자_진한_갈색_피부:','pt':':mulher_andando_de_frente_para_a_direita_pele_morena_escura:','it':':donna_che_cammina_a_destra_carnagione_abbastanza_scura:','fa':':زن_پیاده_روی_می_کند_به_سمت_راست_پوست_گندمی:','id':':wanita_berjalan_kaki_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':女行人面向右边_中等深肤色:','ru':':женщина_идет_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_yürüyen_kadın_orta_koyu_cilt_tonu:','ar':':سائرة_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F6B6\U0001F3FC\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':woman_walking_facing_right_medium-light_skin_tone:','status':fully_qualified,'E':15.1,'de':':fußgängerin_nach_rechts_mittelhelle_hautfarbe:','es':':mujer_caminando_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':femme_qui_marche_vers_la_droite_peau_moyennement_claire:','ja':':歩く女_右向き_やや薄い肌色:','ko':':오른쪽을_향한_걷는_여자_연한_갈색_피부:','pt':':mulher_andando_de_frente_para_a_direita_pele_morena_clara:','it':':donna_che_cammina_a_destra_carnagione_abbastanza_chiara:','fa':':زن_پیاده_روی_می_کند_به_سمت_راست_پوست_روشن:','id':':wanita_berjalan_kaki_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':女行人面向右边_中等浅肤色:','ru':':женщина_идет_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_yürüyen_kadın_orta_açık_cilt_tonu:','ar':':سائرة_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F6B6\U0001F3FC\U0000200D\U00002640\U0000200D\U000027A1\U0000FE0F':{'en':':woman_walking_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgängerin_nach_rechts_mittelhelle_hautfarbe:','es':':mujer_caminando_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':femme_qui_marche_vers_la_droite_peau_moyennement_claire:','ja':':歩く女_右向き_やや薄い肌色:','ko':':오른쪽을_향한_걷는_여자_연한_갈색_피부:','pt':':mulher_andando_de_frente_para_a_direita_pele_morena_clara:','it':':donna_che_cammina_a_destra_carnagione_abbastanza_chiara:','fa':':زن_پیاده_روی_می_کند_به_سمت_راست_پوست_روشن:','id':':wanita_berjalan_kaki_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':女行人面向右边_中等浅肤色:','ru':':женщина_идет_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_yürüyen_kadın_orta_açık_cilt_tonu:','ar':':سائرة_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F6B6\U0001F3FC\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1':{'en':':woman_walking_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgängerin_nach_rechts_mittelhelle_hautfarbe:','es':':mujer_caminando_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':femme_qui_marche_vers_la_droite_peau_moyennement_claire:','ja':':歩く女_右向き_やや薄い肌色:','ko':':오른쪽을_향한_걷는_여자_연한_갈색_피부:','pt':':mulher_andando_de_frente_para_a_direita_pele_morena_clara:','it':':donna_che_cammina_a_destra_carnagione_abbastanza_chiara:','fa':':زن_پیاده_روی_می_کند_به_سمت_راست_پوست_روشن:','id':':wanita_berjalan_kaki_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':女行人面向右边_中等浅肤色:','ru':':женщина_идет_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_yürüyen_kadın_orta_açık_cilt_tonu:','ar':':سائرة_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F6B6\U0001F3FC\U0000200D\U00002640\U0000200D\U000027A1':{'en':':woman_walking_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgängerin_nach_rechts_mittelhelle_hautfarbe:','es':':mujer_caminando_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':femme_qui_marche_vers_la_droite_peau_moyennement_claire:','ja':':歩く女_右向き_やや薄い肌色:','ko':':오른쪽을_향한_걷는_여자_연한_갈색_피부:','pt':':mulher_andando_de_frente_para_a_direita_pele_morena_clara:','it':':donna_che_cammina_a_destra_carnagione_abbastanza_chiara:','fa':':زن_پیاده_روی_می_کند_به_سمت_راست_پوست_روشن:','id':':wanita_berjalan_kaki_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':女行人面向右边_中等浅肤色:','ru':':женщина_идет_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_yürüyen_kadın_orta_açık_cilt_tonu:','ar':':سائرة_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F6B6\U0001F3FD\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1\U0000FE0F':{'en':':woman_walking_facing_right_medium_skin_tone:','status':fully_qualified,'E':15.1,'de':':fußgängerin_nach_rechts_mittlere_hautfarbe:','es':':mujer_caminando_hacia_la_derecha_tono_de_piel_medio:','fr':':femme_qui_marche_vers_la_droite_peau_légèrement_mate:','ja':':歩く女_右向き_中間の肌色:','ko':':오른쪽을_향한_걷는_여자_갈색_피부:','pt':':mulher_andando_de_frente_para_a_direita_pele_morena:','it':':donna_che_cammina_a_destra_carnagione_olivastra:','fa':':زن_پیاده_روی_می_کند_به_سمت_راست_پوست_طلایی:','id':':wanita_berjalan_kaki_menghadap_kanan_warna_kulit_sedang:','zh':':女行人面向右边_中等肤色:','ru':':женщина_идет_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_yürüyen_kadın_orta_cilt_tonu:','ar':':سائرة_متجه_لليمين_بلون_معتدل:'},'\U0001F6B6\U0001F3FD\U0000200D\U00002640\U0000200D\U000027A1\U0000FE0F':{'en':':woman_walking_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgängerin_nach_rechts_mittlere_hautfarbe:','es':':mujer_caminando_hacia_la_derecha_tono_de_piel_medio:','fr':':femme_qui_marche_vers_la_droite_peau_légèrement_mate:','ja':':歩く女_右向き_中間の肌色:','ko':':오른쪽을_향한_걷는_여자_갈색_피부:','pt':':mulher_andando_de_frente_para_a_direita_pele_morena:','it':':donna_che_cammina_a_destra_carnagione_olivastra:','fa':':زن_پیاده_روی_می_کند_به_سمت_راست_پوست_طلایی:','id':':wanita_berjalan_kaki_menghadap_kanan_warna_kulit_sedang:','zh':':女行人面向右边_中等肤色:','ru':':женщина_идет_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_yürüyen_kadın_orta_cilt_tonu:','ar':':سائرة_متجه_لليمين_بلون_معتدل:'},'\U0001F6B6\U0001F3FD\U0000200D\U00002640\U0000FE0F\U0000200D\U000027A1':{'en':':woman_walking_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgängerin_nach_rechts_mittlere_hautfarbe:','es':':mujer_caminando_hacia_la_derecha_tono_de_piel_medio:','fr':':femme_qui_marche_vers_la_droite_peau_légèrement_mate:','ja':':歩く女_右向き_中間の肌色:','ko':':오른쪽을_향한_걷는_여자_갈색_피부:','pt':':mulher_andando_de_frente_para_a_direita_pele_morena:','it':':donna_che_cammina_a_destra_carnagione_olivastra:','fa':':زن_پیاده_روی_می_کند_به_سمت_راست_پوست_طلایی:','id':':wanita_berjalan_kaki_menghadap_kanan_warna_kulit_sedang:','zh':':女行人面向右边_中等肤色:','ru':':женщина_идет_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_yürüyen_kadın_orta_cilt_tonu:','ar':':سائرة_متجه_لليمين_بلون_معتدل:'},'\U0001F6B6\U0001F3FD\U0000200D\U00002640\U0000200D\U000027A1':{'en':':woman_walking_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':fußgängerin_nach_rechts_mittlere_hautfarbe:','es':':mujer_caminando_hacia_la_derecha_tono_de_piel_medio:','fr':':femme_qui_marche_vers_la_droite_peau_légèrement_mate:','ja':':歩く女_右向き_中間の肌色:','ko':':오른쪽을_향한_걷는_여자_갈색_피부:','pt':':mulher_andando_de_frente_para_a_direita_pele_morena:','it':':donna_che_cammina_a_destra_carnagione_olivastra:','fa':':زن_پیاده_روی_می_کند_به_سمت_راست_پوست_طلایی:','id':':wanita_berjalan_kaki_menghadap_kanan_warna_kulit_sedang:','zh':':女行人面向右边_中等肤色:','ru':':женщина_идет_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_yürüyen_kadın_orta_cilt_tonu:','ar':':سائرة_متجه_لليمين_بلون_معتدل:'},'\U0001F6B6\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_walking_light_skin_tone:','status':fully_qualified,'E':4,'de':':fußgängerin_helle_hautfarbe:','es':':mujer_caminando_tono_de_piel_claro:','fr':':femme_qui_marche_peau_claire:','ja':':歩く女_薄い肌色:','ko':':걷는_여자_하얀_피부:','pt':':mulher_andando_pele_clara:','it':':donna_che_cammina_carnagione_chiara:','fa':':زن_پیاده_روی_می_کند_پوست_سفید:','id':':wanita_berjalan_kaki_warna_kulit_cerah:','zh':':女行人_较浅肤色:','ru':':женщина_идет_очень_светлыи_тон_кожи:','tr':':yürüyen_kadın_açık_cilt_tonu:','ar':':سائرة_بشرة_بلون_فاتح:'},'\U0001F6B6\U0001F3FB\U0000200D\U00002640':{'en':':woman_walking_light_skin_tone:','status':minimally_qualified,'E':4,'de':':fußgängerin_helle_hautfarbe:','es':':mujer_caminando_tono_de_piel_claro:','fr':':femme_qui_marche_peau_claire:','ja':':歩く女_薄い肌色:','ko':':걷는_여자_하얀_피부:','pt':':mulher_andando_pele_clara:','it':':donna_che_cammina_carnagione_chiara:','fa':':زن_پیاده_روی_می_کند_پوست_سفید:','id':':wanita_berjalan_kaki_warna_kulit_cerah:','zh':':女行人_较浅肤色:','ru':':женщина_идет_очень_светлый_тон_кожи:','tr':':yürüyen_kadın_açık_cilt_tonu:','ar':':سائرة_بشرة_بلون_فاتح:'},'\U0001F6B6\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_walking_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':fußgängerin_mitteldunkle_hautfarbe:','es':':mujer_caminando_tono_de_piel_oscuro_medio:','fr':':femme_qui_marche_peau_mate:','ja':':歩く女_やや濃い肌色:','ko':':걷는_여자_진한_갈색_피부:','pt':':mulher_andando_pele_morena_escura:','it':':donna_che_cammina_carnagione_abbastanza_scura:','fa':':زن_پیاده_روی_می_کند_پوست_گندمی:','id':':wanita_berjalan_kaki_warna_kulit_gelap-sedang:','zh':':女行人_中等深肤色:','ru':':женщина_идет_темныи_тон_кожи:','tr':':yürüyen_kadın_orta_koyu_cilt_tonu:','ar':':سائرة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6B6\U0001F3FE\U0000200D\U00002640':{'en':':woman_walking_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':fußgängerin_mitteldunkle_hautfarbe:','es':':mujer_caminando_tono_de_piel_oscuro_medio:','fr':':femme_qui_marche_peau_mate:','ja':':歩く女_やや濃い肌色:','ko':':걷는_여자_진한_갈색_피부:','pt':':mulher_andando_pele_morena_escura:','it':':donna_che_cammina_carnagione_abbastanza_scura:','fa':':زن_پیاده_روی_می_کند_پوست_گندمی:','id':':wanita_berjalan_kaki_warna_kulit_gelap-sedang:','zh':':女行人_中等深肤色:','ru':':женщина_идет_темный_тон_кожи:','tr':':yürüyen_kadın_orta_koyu_cilt_tonu:','ar':':سائرة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F6B6\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_walking_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':fußgängerin_mittelhelle_hautfarbe:','es':':mujer_caminando_tono_de_piel_claro_medio:','fr':':femme_qui_marche_peau_moyennement_claire:','ja':':歩く女_やや薄い肌色:','ko':':걷는_여자_연한_갈색_피부:','pt':':mulher_andando_pele_morena_clara:','it':':donna_che_cammina_carnagione_abbastanza_chiara:','fa':':زن_پیاده_روی_می_کند_پوست_روشن:','id':':wanita_berjalan_kaki_warna_kulit_cerah-sedang:','zh':':女行人_中等浅肤色:','ru':':женщина_идет_светлыи_тон_кожи:','tr':':yürüyen_kadın_orta_açık_cilt_tonu:','ar':':سائرة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6B6\U0001F3FC\U0000200D\U00002640':{'en':':woman_walking_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':fußgängerin_mittelhelle_hautfarbe:','es':':mujer_caminando_tono_de_piel_claro_medio:','fr':':femme_qui_marche_peau_moyennement_claire:','ja':':歩く女_やや薄い肌色:','ko':':걷는_여자_연한_갈색_피부:','pt':':mulher_andando_pele_morena_clara:','it':':donna_che_cammina_carnagione_abbastanza_chiara:','fa':':زن_پیاده_روی_می_کند_پوست_روشن:','id':':wanita_berjalan_kaki_warna_kulit_cerah-sedang:','zh':':女行人_中等浅肤色:','ru':':женщина_идет_светлый_тон_кожи:','tr':':yürüyen_kadın_orta_açık_cilt_tonu:','ar':':سائرة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F6B6\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_walking_medium_skin_tone:','status':fully_qualified,'E':4,'de':':fußgängerin_mittlere_hautfarbe:','es':':mujer_caminando_tono_de_piel_medio:','fr':':femme_qui_marche_peau_légèrement_mate:','ja':':歩く女_中間の肌色:','ko':':걷는_여자_갈색_피부:','pt':':mulher_andando_pele_morena:','it':':donna_che_cammina_carnagione_olivastra:','fa':':زن_پیاده_روی_می_کند_پوست_طلایی:','id':':wanita_berjalan_kaki_warna_kulit_sedang:','zh':':女行人_中等肤色:','ru':':женщина_идет_среднии_тон_кожи:','tr':':yürüyen_kadın_orta_cilt_tonu:','ar':':سائرة_بشرة_بلون_معتدل:'},'\U0001F6B6\U0001F3FD\U0000200D\U00002640':{'en':':woman_walking_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':fußgängerin_mittlere_hautfarbe:','es':':mujer_caminando_tono_de_piel_medio:','fr':':femme_qui_marche_peau_légèrement_mate:','ja':':歩く女_中間の肌色:','ko':':걷는_여자_갈색_피부:','pt':':mulher_andando_pele_morena:','it':':donna_che_cammina_carnagione_olivastra:','fa':':زن_پیاده_روی_می_کند_پوست_طلایی:','id':':wanita_berjalan_kaki_warna_kulit_sedang:','zh':':女行人_中等肤色:','ru':':женщина_идет_средний_тон_кожи:','tr':':yürüyen_kadın_orta_cilt_tonu:','ar':':سائرة_بشرة_بلون_معتدل:'},'\U0001F473\U0000200D\U00002640\U0000FE0F':{'en':':woman_wearing_turban:','status':fully_qualified,'E':4,'alias':[':woman_with_turban:'],'de':':frau_mit_turban:','es':':mujer_con_turbante:','fr':':femme_en_turban:','ja':':ターバンの女性:','ko':':터번을_쓰고_있는_여자:','pt':':mulher_com_turbante:','it':':donna_con_turbante:','fa':':زن_با_دستار:','id':':wanita_mengenakan_serban:','zh':':戴头巾的女人:','ru':':женщина_в_тюрбане:','tr':':sarıklı_kadın:','ar':':سيدة_ترتدي_عمامة:'},'\U0001F473\U0000200D\U00002640':{'en':':woman_wearing_turban:','status':minimally_qualified,'E':4,'alias':[':woman_with_turban:'],'de':':frau_mit_turban:','es':':mujer_con_turbante:','fr':':femme_en_turban:','ja':':ターバンの女性:','ko':':터번을_쓰고_있는_여자:','pt':':mulher_com_turbante:','it':':donna_con_turbante:','fa':':زن_با_دستار:','id':':wanita_mengenakan_serban:','zh':':戴头巾的女人:','ru':':женщина_в_тюрбане:','tr':':sarıklı_kadın:','ar':':سيدة_ترتدي_عمامة:'},'\U0001F473\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_wearing_turban_dark_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_turban_dunkle_hautfarbe:','es':':mujer_con_turbante_tono_de_piel_oscuro:','fr':':femme_en_turban_peau_foncée:','ja':':ターバンの女性_濃い肌色:','ko':':터번을_쓰고_있는_여자_검은색_피부:','pt':':mulher_com_turbante_pele_escura:','it':':donna_con_turbante_carnagione_scura:','fa':':زن_با_دستار_پوست_آبنوسی:','id':':wanita_mengenakan_serban_warna_kulit_gelap:','zh':':戴头巾的女人_较深肤色:','ru':':женщина_в_тюрбане_очень_темныи_тон_кожи:','tr':':sarıklı_kadın_koyu_cilt_tonu:','ar':':سيدة_ترتدي_عمامة_بشرة_بلون_غامق:'},'\U0001F473\U0001F3FF\U0000200D\U00002640':{'en':':woman_wearing_turban_dark_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_turban_dunkle_hautfarbe:','es':':mujer_con_turbante_tono_de_piel_oscuro:','fr':':femme_en_turban_peau_foncée:','ja':':ターバンの女性_濃い肌色:','ko':':터번을_쓰고_있는_여자_검은색_피부:','pt':':mulher_com_turbante_pele_escura:','it':':donna_con_turbante_carnagione_scura:','fa':':زن_با_دستار_پوست_آبنوسی:','id':':wanita_mengenakan_serban_warna_kulit_gelap:','zh':':戴头巾的女人_较深肤色:','ru':':женщина_в_тюрбане_очень_темный_тон_кожи:','tr':':sarıklı_kadın_koyu_cilt_tonu:','ar':':سيدة_ترتدي_عمامة_بشرة_بلون_غامق:'},'\U0001F473\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_wearing_turban_light_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_turban_helle_hautfarbe:','es':':mujer_con_turbante_tono_de_piel_claro:','fr':':femme_en_turban_peau_claire:','ja':':ターバンの女性_薄い肌色:','ko':':터번을_쓰고_있는_여자_하얀_피부:','pt':':mulher_com_turbante_pele_clara:','it':':donna_con_turbante_carnagione_chiara:','fa':':زن_با_دستار_پوست_سفید:','id':':wanita_mengenakan_serban_warna_kulit_cerah:','zh':':戴头巾的女人_较浅肤色:','ru':':женщина_в_тюрбане_очень_светлыи_тон_кожи:','tr':':sarıklı_kadın_açık_cilt_tonu:','ar':':سيدة_ترتدي_عمامة_بشرة_بلون_فاتح:'},'\U0001F473\U0001F3FB\U0000200D\U00002640':{'en':':woman_wearing_turban_light_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_turban_helle_hautfarbe:','es':':mujer_con_turbante_tono_de_piel_claro:','fr':':femme_en_turban_peau_claire:','ja':':ターバンの女性_薄い肌色:','ko':':터번을_쓰고_있는_여자_하얀_피부:','pt':':mulher_com_turbante_pele_clara:','it':':donna_con_turbante_carnagione_chiara:','fa':':زن_با_دستار_پوست_سفید:','id':':wanita_mengenakan_serban_warna_kulit_cerah:','zh':':戴头巾的女人_较浅肤色:','ru':':женщина_в_тюрбане_очень_светлый_тон_кожи:','tr':':sarıklı_kadın_açık_cilt_tonu:','ar':':سيدة_ترتدي_عمامة_بشرة_بلون_فاتح:'},'\U0001F473\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_wearing_turban_medium-dark_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_turban_mitteldunkle_hautfarbe:','es':':mujer_con_turbante_tono_de_piel_oscuro_medio:','fr':':femme_en_turban_peau_mate:','ja':':ターバンの女性_やや濃い肌色:','ko':':터번을_쓰고_있는_여자_진한_갈색_피부:','pt':':mulher_com_turbante_pele_morena_escura:','it':':donna_con_turbante_carnagione_abbastanza_scura:','fa':':زن_با_دستار_پوست_گندمی:','id':':wanita_mengenakan_serban_warna_kulit_gelap-sedang:','zh':':戴头巾的女人_中等深肤色:','ru':':женщина_в_тюрбане_темныи_тон_кожи:','tr':':sarıklı_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_ترتدي_عمامة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F473\U0001F3FE\U0000200D\U00002640':{'en':':woman_wearing_turban_medium-dark_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_turban_mitteldunkle_hautfarbe:','es':':mujer_con_turbante_tono_de_piel_oscuro_medio:','fr':':femme_en_turban_peau_mate:','ja':':ターバンの女性_やや濃い肌色:','ko':':터번을_쓰고_있는_여자_진한_갈색_피부:','pt':':mulher_com_turbante_pele_morena_escura:','it':':donna_con_turbante_carnagione_abbastanza_scura:','fa':':زن_با_دستار_پوست_گندمی:','id':':wanita_mengenakan_serban_warna_kulit_gelap-sedang:','zh':':戴头巾的女人_中等深肤色:','ru':':женщина_в_тюрбане_темный_тон_кожи:','tr':':sarıklı_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_ترتدي_عمامة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F473\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_wearing_turban_medium-light_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_turban_mittelhelle_hautfarbe:','es':':mujer_con_turbante_tono_de_piel_claro_medio:','fr':':femme_en_turban_peau_moyennement_claire:','ja':':ターバンの女性_やや薄い肌色:','ko':':터번을_쓰고_있는_여자_연한_갈색_피부:','pt':':mulher_com_turbante_pele_morena_clara:','it':':donna_con_turbante_carnagione_abbastanza_chiara:','fa':':زن_با_دستار_پوست_روشن:','id':':wanita_mengenakan_serban_warna_kulit_cerah-sedang:','zh':':戴头巾的女人_中等浅肤色:','ru':':женщина_в_тюрбане_светлыи_тон_кожи:','tr':':sarıklı_kadın_orta_açık_cilt_tonu:','ar':':سيدة_ترتدي_عمامة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F473\U0001F3FC\U0000200D\U00002640':{'en':':woman_wearing_turban_medium-light_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_turban_mittelhelle_hautfarbe:','es':':mujer_con_turbante_tono_de_piel_claro_medio:','fr':':femme_en_turban_peau_moyennement_claire:','ja':':ターバンの女性_やや薄い肌色:','ko':':터번을_쓰고_있는_여자_연한_갈색_피부:','pt':':mulher_com_turbante_pele_morena_clara:','it':':donna_con_turbante_carnagione_abbastanza_chiara:','fa':':زن_با_دستار_پوست_روشن:','id':':wanita_mengenakan_serban_warna_kulit_cerah-sedang:','zh':':戴头巾的女人_中等浅肤色:','ru':':женщина_в_тюрбане_светлый_тон_кожи:','tr':':sarıklı_kadın_orta_açık_cilt_tonu:','ar':':سيدة_ترتدي_عمامة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F473\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_wearing_turban_medium_skin_tone:','status':fully_qualified,'E':4,'de':':frau_mit_turban_mittlere_hautfarbe:','es':':mujer_con_turbante_tono_de_piel_medio:','fr':':femme_en_turban_peau_légèrement_mate:','ja':':ターバンの女性_中間の肌色:','ko':':터번을_쓰고_있는_여자_갈색_피부:','pt':':mulher_com_turbante_pele_morena:','it':':donna_con_turbante_carnagione_olivastra:','fa':':زن_با_دستار_پوست_طلایی:','id':':wanita_mengenakan_serban_warna_kulit_sedang:','zh':':戴头巾的女人_中等肤色:','ru':':женщина_в_тюрбане_среднии_тон_кожи:','tr':':sarıklı_kadın_orta_cilt_tonu:','ar':':سيدة_ترتدي_عمامة_بشرة_بلون_معتدل:'},'\U0001F473\U0001F3FD\U0000200D\U00002640':{'en':':woman_wearing_turban_medium_skin_tone:','status':minimally_qualified,'E':4,'de':':frau_mit_turban_mittlere_hautfarbe:','es':':mujer_con_turbante_tono_de_piel_medio:','fr':':femme_en_turban_peau_légèrement_mate:','ja':':ターバンの女性_中間の肌色:','ko':':터번을_쓰고_있는_여자_갈색_피부:','pt':':mulher_com_turbante_pele_morena:','it':':donna_con_turbante_carnagione_olivastra:','fa':':زن_با_دستار_پوست_طلایی:','id':':wanita_mengenakan_serban_warna_kulit_sedang:','zh':':戴头巾的女人_中等肤色:','ru':':женщина_в_тюрбане_средний_тон_кожи:','tr':':sarıklı_kadın_orta_cilt_tonu:','ar':':سيدة_ترتدي_عمامة_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U0001F9B3':{'en':':woman_white_hair:','status':fully_qualified,'E':11,'alias':[':white_haired_woman:'],'de':':frau_weißes_haar:','es':':mujer_pelo_blanco:','fr':':femme_cheveux_blancs:','ja':':女性_白髪:','ko':':여자_백발:','pt':':mulher_cabelo_branco:','it':':donna_capelli_bianchi:','fa':':زن_موسفید:','id':':wanita_rambut_putih:','zh':':女人_白发:','ru':':женщина_седые_волосы:','tr':':kadın_beyaz_saç:','ar':':سيدة_شعر_أبيض:'},'\U0001F9D5':{'en':':woman_with_headscarf:','status':fully_qualified,'E':5,'de':':frau_mit_kopftuch:','es':':mujer_con_hiyab:','fr':':femme_avec_foulard:','ja':':スカーフの女性:','ko':':머리에_스카프를_두른_여자:','pt':':mulher_com_véu:','it':':donna_con_velo:','fa':':زن_با_روسری:','id':':wanita_dengan_kerudung:','zh':':带头饰的女人:','ru':':женщина_в_платке:','tr':':başörtülü_kadın:','ar':':امرأة_محجبة:'},'\U0001F9D5\U0001F3FF':{'en':':woman_with_headscarf_dark_skin_tone:','status':fully_qualified,'E':5,'de':':frau_mit_kopftuch_dunkle_hautfarbe:','es':':mujer_con_hiyab_tono_de_piel_oscuro:','fr':':femme_avec_foulard_peau_foncée:','ja':':スカーフの女性_濃い肌色:','ko':':머리에_스카프를_두른_여자_검은색_피부:','pt':':mulher_com_véu_pele_escura:','it':':donna_con_velo_carnagione_scura:','fa':':زن_با_روسری_پوست_آبنوسی:','id':':wanita_dengan_kerudung_warna_kulit_gelap:','zh':':带头饰的女人_较深肤色:','ru':':женщина_в_платке_очень_темный_тон_кожи:','tr':':başörtülü_kadın_koyu_cilt_tonu:','ar':':امرأة_محجبة_بشرة_بلون_غامق:'},'\U0001F9D5\U0001F3FB':{'en':':woman_with_headscarf_light_skin_tone:','status':fully_qualified,'E':5,'de':':frau_mit_kopftuch_helle_hautfarbe:','es':':mujer_con_hiyab_tono_de_piel_claro:','fr':':femme_avec_foulard_peau_claire:','ja':':スカーフの女性_薄い肌色:','ko':':머리에_스카프를_두른_여자_하얀_피부:','pt':':mulher_com_véu_pele_clara:','it':':donna_con_velo_carnagione_chiara:','fa':':زن_با_روسری_پوست_سفید:','id':':wanita_dengan_kerudung_warna_kulit_cerah:','zh':':带头饰的女人_较浅肤色:','ru':':женщина_в_платке_очень_светлый_тон_кожи:','tr':':başörtülü_kadın_açık_cilt_tonu:','ar':':امرأة_محجبة_بشرة_بلون_فاتح:'},'\U0001F9D5\U0001F3FE':{'en':':woman_with_headscarf_medium-dark_skin_tone:','status':fully_qualified,'E':5,'de':':frau_mit_kopftuch_mitteldunkle_hautfarbe:','es':':mujer_con_hiyab_tono_de_piel_oscuro_medio:','fr':':femme_avec_foulard_peau_mate:','ja':':スカーフの女性_やや濃い肌色:','ko':':머리에_스카프를_두른_여자_진한_갈색_피부:','pt':':mulher_com_véu_pele_morena_escura:','it':':donna_con_velo_carnagione_abbastanza_scura:','fa':':زن_با_روسری_پوست_گندمی:','id':':wanita_dengan_kerudung_warna_kulit_gelap-sedang:','zh':':带头饰的女人_中等深肤色:','ru':':женщина_в_платке_темный_тон_кожи:','tr':':başörtülü_kadın_orta_koyu_cilt_tonu:','ar':':امرأة_محجبة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F9D5\U0001F3FC':{'en':':woman_with_headscarf_medium-light_skin_tone:','status':fully_qualified,'E':5,'de':':frau_mit_kopftuch_mittelhelle_hautfarbe:','es':':mujer_con_hiyab_tono_de_piel_claro_medio:','fr':':femme_avec_foulard_peau_moyennement_claire:','ja':':スカーフの女性_やや薄い肌色:','ko':':머리에_스카프를_두른_여자_연한_갈색_피부:','pt':':mulher_com_véu_pele_morena_clara:','it':':donna_con_velo_carnagione_abbastanza_chiara:','fa':':زن_با_روسری_پوست_روشن:','id':':wanita_dengan_kerudung_warna_kulit_cerah-sedang:','zh':':带头饰的女人_中等浅肤色:','ru':':женщина_в_платке_светлый_тон_кожи:','tr':':başörtülü_kadın_orta_açık_cilt_tonu:','ar':':امرأة_محجبة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F9D5\U0001F3FD':{'en':':woman_with_headscarf_medium_skin_tone:','status':fully_qualified,'E':5,'de':':frau_mit_kopftuch_mittlere_hautfarbe:','es':':mujer_con_hiyab_tono_de_piel_medio:','fr':':femme_avec_foulard_peau_légèrement_mate:','ja':':スカーフの女性_中間の肌色:','ko':':머리에_스카프를_두른_여자_갈색_피부:','pt':':mulher_com_véu_pele_morena:','it':':donna_con_velo_carnagione_olivastra:','fa':':زن_با_روسری_پوست_طلایی:','id':':wanita_dengan_kerudung_warna_kulit_sedang:','zh':':带头饰的女人_中等肤色:','ru':':женщина_в_платке_средний_тон_кожи:','tr':':başörtülü_kadın_orta_cilt_tonu:','ar':':امرأة_محجبة_بشرة_بلون_معتدل:'},'\U0001F470\U0000200D\U00002640\U0000FE0F':{'en':':woman_with_veil:','status':fully_qualified,'E':13,'alias':[':bride_with_veil:'],'de':':frau_mit_schleier:','es':':mujer_con_velo:','fr':':femme_avec_voile:','ja':':ベールの女性:','ko':':면사포를_쓴_여자:','pt':':noiva_com_véu:','it':':sposa_con_velo:','fa':':زن_با_تور_عروس:','id':':wanita_berpenutup_kepala:','zh':':戴头纱的女人:','ru':':женщина_в_фате:','tr':':duvaklı_kadın:','ar':':امرأة_بطرحة:'},'\U0001F470\U0000200D\U00002640':{'en':':woman_with_veil:','status':minimally_qualified,'E':13,'alias':[':bride_with_veil:'],'de':':frau_mit_schleier:','es':':mujer_con_velo:','fr':':femme_avec_voile:','ja':':ベールの女性:','ko':':면사포를_쓴_여자:','pt':':noiva_com_véu:','it':':sposa_con_velo:','fa':':زن_با_تور_عروس:','id':':wanita_berpenutup_kepala:','zh':':戴头纱的女人:','ru':':женщина_в_фате:','tr':':duvaklı_kadın:','ar':':امرأة_بطرحة:'},'\U0001F470\U0001F3FF\U0000200D\U00002640\U0000FE0F':{'en':':woman_with_veil_dark_skin_tone:','status':fully_qualified,'E':13,'de':':frau_mit_schleier_dunkle_hautfarbe:','es':':mujer_con_velo_tono_de_piel_oscuro:','fr':':femme_avec_voile_peau_foncée:','ja':':ベールの女性_濃い肌色:','ko':':면사포를_쓴_여자_검은색_피부:','pt':':noiva_com_véu_pele_escura:','it':':sposa_con_velo_carnagione_scura:','fa':':زن_با_تور_عروس_پوست_آبنوسی:','id':':wanita_berpenutup_kepala_warna_kulit_gelap:','zh':':戴头纱的女人_较深肤色:','ru':':женщина_в_фате_очень_темныи_тон_кожи:','tr':':duvaklı_kadın_koyu_cilt_tonu:','ar':':امرأة_بطرحة_بشرة_بلون_غامق:'},'\U0001F470\U0001F3FF\U0000200D\U00002640':{'en':':woman_with_veil_dark_skin_tone:','status':minimally_qualified,'E':13,'de':':frau_mit_schleier_dunkle_hautfarbe:','es':':mujer_con_velo_tono_de_piel_oscuro:','fr':':femme_avec_voile_peau_foncée:','ja':':ベールの女性_濃い肌色:','ko':':면사포를_쓴_여자_검은색_피부:','pt':':noiva_com_véu_pele_escura:','it':':sposa_con_velo_carnagione_scura:','fa':':زن_با_تور_عروس_پوست_آبنوسی:','id':':wanita_berpenutup_kepala_warna_kulit_gelap:','zh':':戴头纱的女人_较深肤色:','ru':':женщина_в_фате_очень_темный_тон_кожи:','tr':':duvaklı_kadın_koyu_cilt_tonu:','ar':':امرأة_بطرحة_بشرة_بلون_غامق:'},'\U0001F470\U0001F3FB\U0000200D\U00002640\U0000FE0F':{'en':':woman_with_veil_light_skin_tone:','status':fully_qualified,'E':13,'de':':frau_mit_schleier_helle_hautfarbe:','es':':mujer_con_velo_tono_de_piel_claro:','fr':':femme_avec_voile_peau_claire:','ja':':ベールの女性_薄い肌色:','ko':':면사포를_쓴_여자_하얀_피부:','pt':':noiva_com_véu_pele_clara:','it':':sposa_con_velo_carnagione_chiara:','fa':':زن_با_تور_عروس_پوست_سفید:','id':':wanita_berpenutup_kepala_warna_kulit_cerah:','zh':':戴头纱的女人_较浅肤色:','ru':':женщина_в_фате_очень_светлыи_тон_кожи:','tr':':duvaklı_kadın_açık_cilt_tonu:','ar':':امرأة_بطرحة_بشرة_بلون_فاتح:'},'\U0001F470\U0001F3FB\U0000200D\U00002640':{'en':':woman_with_veil_light_skin_tone:','status':minimally_qualified,'E':13,'de':':frau_mit_schleier_helle_hautfarbe:','es':':mujer_con_velo_tono_de_piel_claro:','fr':':femme_avec_voile_peau_claire:','ja':':ベールの女性_薄い肌色:','ko':':면사포를_쓴_여자_하얀_피부:','pt':':noiva_com_véu_pele_clara:','it':':sposa_con_velo_carnagione_chiara:','fa':':زن_با_تور_عروس_پوست_سفید:','id':':wanita_berpenutup_kepala_warna_kulit_cerah:','zh':':戴头纱的女人_较浅肤色:','ru':':женщина_в_фате_очень_светлый_тон_кожи:','tr':':duvaklı_kadın_açık_cilt_tonu:','ar':':امرأة_بطرحة_بشرة_بلون_فاتح:'},'\U0001F470\U0001F3FE\U0000200D\U00002640\U0000FE0F':{'en':':woman_with_veil_medium-dark_skin_tone:','status':fully_qualified,'E':13,'de':':frau_mit_schleier_mitteldunkle_hautfarbe:','es':':mujer_con_velo_tono_de_piel_oscuro_medio:','fr':':femme_avec_voile_peau_mate:','ja':':ベールの女性_やや濃い肌色:','ko':':면사포를_쓴_여자_진한_갈색_피부:','pt':':noiva_com_véu_pele_morena_escura:','it':':sposa_con_velo_carnagione_abbastanza_scura:','fa':':زن_با_تور_عروس_پوست_گندمی:','id':':wanita_berpenutup_kepala_warna_kulit_gelap-sedang:','zh':':戴头纱的女人_中等深肤色:','ru':':женщина_в_фате_темныи_тон_кожи:','tr':':duvaklı_kadın_orta_koyu_cilt_tonu:','ar':':امرأة_بطرحة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F470\U0001F3FE\U0000200D\U00002640':{'en':':woman_with_veil_medium-dark_skin_tone:','status':minimally_qualified,'E':13,'de':':frau_mit_schleier_mitteldunkle_hautfarbe:','es':':mujer_con_velo_tono_de_piel_oscuro_medio:','fr':':femme_avec_voile_peau_mate:','ja':':ベールの女性_やや濃い肌色:','ko':':면사포를_쓴_여자_진한_갈색_피부:','pt':':noiva_com_véu_pele_morena_escura:','it':':sposa_con_velo_carnagione_abbastanza_scura:','fa':':زن_با_تور_عروس_پوست_گندمی:','id':':wanita_berpenutup_kepala_warna_kulit_gelap-sedang:','zh':':戴头纱的女人_中等深肤色:','ru':':женщина_в_фате_темный_тон_кожи:','tr':':duvaklı_kadın_orta_koyu_cilt_tonu:','ar':':امرأة_بطرحة_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F470\U0001F3FC\U0000200D\U00002640\U0000FE0F':{'en':':woman_with_veil_medium-light_skin_tone:','status':fully_qualified,'E':13,'de':':frau_mit_schleier_mittelhelle_hautfarbe:','es':':mujer_con_velo_tono_de_piel_claro_medio:','fr':':femme_avec_voile_peau_moyennement_claire:','ja':':ベールの女性_やや薄い肌色:','ko':':면사포를_쓴_여자_연한_갈색_피부:','pt':':noiva_com_véu_pele_morena_clara:','it':':sposa_con_velo_carnagione_abbastanza_chiara:','fa':':زن_با_تور_عروس_پوست_روشن:','id':':wanita_berpenutup_kepala_warna_kulit_cerah-sedang:','zh':':戴头纱的女人_中等浅肤色:','ru':':женщина_в_фате_светлыи_тон_кожи:','tr':':duvaklı_kadın_orta_açık_cilt_tonu:','ar':':امرأة_بطرحة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F470\U0001F3FC\U0000200D\U00002640':{'en':':woman_with_veil_medium-light_skin_tone:','status':minimally_qualified,'E':13,'de':':frau_mit_schleier_mittelhelle_hautfarbe:','es':':mujer_con_velo_tono_de_piel_claro_medio:','fr':':femme_avec_voile_peau_moyennement_claire:','ja':':ベールの女性_やや薄い肌色:','ko':':면사포를_쓴_여자_연한_갈색_피부:','pt':':noiva_com_véu_pele_morena_clara:','it':':sposa_con_velo_carnagione_abbastanza_chiara:','fa':':زن_با_تور_عروس_پوست_روشن:','id':':wanita_berpenutup_kepala_warna_kulit_cerah-sedang:','zh':':戴头纱的女人_中等浅肤色:','ru':':женщина_в_фате_светлый_тон_кожи:','tr':':duvaklı_kadın_orta_açık_cilt_tonu:','ar':':امرأة_بطرحة_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F470\U0001F3FD\U0000200D\U00002640\U0000FE0F':{'en':':woman_with_veil_medium_skin_tone:','status':fully_qualified,'E':13,'de':':frau_mit_schleier_mittlere_hautfarbe:','es':':mujer_con_velo_tono_de_piel_medio:','fr':':femme_avec_voile_peau_légèrement_mate:','ja':':ベールの女性_中間の肌色:','ko':':면사포를_쓴_여자_갈색_피부:','pt':':noiva_com_véu_pele_morena:','it':':sposa_con_velo_carnagione_olivastra:','fa':':زن_با_تور_عروس_پوست_طلایی:','id':':wanita_berpenutup_kepala_warna_kulit_sedang:','zh':':戴头纱的女人_中等肤色:','ru':':женщина_в_фате_среднии_тон_кожи:','tr':':duvaklı_kadın_orta_cilt_tonu:','ar':':امرأة_بطرحة_بشرة_بلون_معتدل:'},'\U0001F470\U0001F3FD\U0000200D\U00002640':{'en':':woman_with_veil_medium_skin_tone:','status':minimally_qualified,'E':13,'de':':frau_mit_schleier_mittlere_hautfarbe:','es':':mujer_con_velo_tono_de_piel_medio:','fr':':femme_avec_voile_peau_légèrement_mate:','ja':':ベールの女性_中間の肌色:','ko':':면사포를_쓴_여자_갈색_피부:','pt':':noiva_com_véu_pele_morena:','it':':sposa_con_velo_carnagione_olivastra:','fa':':زن_با_تور_عروس_پوست_طلایی:','id':':wanita_berpenutup_kepala_warna_kulit_sedang:','zh':':戴头纱的女人_中等肤色:','ru':':женщина_в_фате_средний_тон_кожи:','tr':':duvaklı_kadın_orta_cilt_tonu:','ar':':امرأة_بطرحة_بشرة_بلون_معتدل:'},'\U0001F469\U0000200D\U0001F9AF':{'en':':woman_with_white_cane:','status':fully_qualified,'E':12,'alias':[':woman_with_probing_cane:'],'de':':frau_mit_langstock:','es':':mujer_con_bastón:','fr':':femme_avec_canne_blanche:','ja':':杖をついた女性:','ko':':지팡이를_든_여자:','pt':':mulher_com_bengala_para_cego:','it':':donna_con_bastone_bianco_di_orientamento:','fa':':زن_با_عصای_سفید:','id':':wanita_dengan_tongkat:','zh':':拄盲杖的女人:','ru':':женщина_с_тростью_для_ходьбы:','tr':':görme_engelli_bastonlu_kadın:','ar':':سيدة_بعصا_سير:'},'\U0001F469\U0001F3FF\U0000200D\U0001F9AF':{'en':':woman_with_white_cane_dark_skin_tone:','status':fully_qualified,'E':12,'de':':frau_mit_langstock_dunkle_hautfarbe:','es':':mujer_con_bastón_tono_de_piel_oscuro:','fr':':femme_avec_canne_blanche_peau_foncée:','ja':':杖をついた女性_濃い肌色:','ko':':지팡이를_든_여자_검은색_피부:','pt':':mulher_com_bengala_para_cego_pele_escura:','it':':donna_con_bastone_bianco_di_orientamento_carnagione_scura:','fa':':زن_با_عصای_سفید_پوست_آبنوسی:','id':':wanita_dengan_tongkat_warna_kulit_gelap:','zh':':拄盲杖的女人_较深肤色:','ru':':женщина_с_тростью_для_ходьбы_очень_темный_тон_кожи:','tr':':görme_engelli_bastonlu_kadın_koyu_cilt_tonu:','ar':':سيدة_بعصا_سير_بشرة_بلون_غامق:'},'\U0001F469\U0000200D\U0001F9AF\U0000200D\U000027A1\U0000FE0F':{'en':':woman_with_white_cane_facing_right:','status':fully_qualified,'E':15.1,'de':':frau_mit_langstock_nach_rechts:','es':':mujer_con_bastón_hacia_la_derecha:','fr':':femme_avec_canne_blanche_vers_la_droite:','ja':':杖をついた女性_右向き:','ko':':오른쪽을_향한_지팡이를_든_여자:','pt':':mulher_com_bengala_para_cego_de_frente_para_a_direita:','it':':donna_con_bastone_bianco_di_orientamento_a_destra:','fa':':زن_با_عصای_سفید_به_سمت_راست:','id':':wanita_dengan_tongkat_menghadap_kanan:','zh':':拄盲杖的女人面向右边:','ru':':женщина_с_тростью_для_ходьбы_направленные_вправо:','tr':':sağa_dönük_görme_engelli_bastonlu_kadın:','ar':':سيدة_بعصا_سير_متجه_لليمين:'},'\U0001F469\U0000200D\U0001F9AF\U0000200D\U000027A1':{'en':':woman_with_white_cane_facing_right:','status':minimally_qualified,'E':15.1,'de':':frau_mit_langstock_nach_rechts:','es':':mujer_con_bastón_hacia_la_derecha:','fr':':femme_avec_canne_blanche_vers_la_droite:','ja':':杖をついた女性_右向き:','ko':':오른쪽을_향한_지팡이를_든_여자:','pt':':mulher_com_bengala_para_cego_de_frente_para_a_direita:','it':':donna_con_bastone_bianco_di_orientamento_a_destra:','fa':':زن_با_عصای_سفید_به_سمت_راست:','id':':wanita_dengan_tongkat_menghadap_kanan:','zh':':拄盲杖的女人面向右边:','ru':':женщина_с_тростью_для_ходьбы_направленные_вправо:','tr':':sağa_dönük_görme_engelli_bastonlu_kadın:','ar':':سيدة_بعصا_سير_متجه_لليمين:'},'\U0001F469\U0001F3FF\U0000200D\U0001F9AF\U0000200D\U000027A1\U0000FE0F':{'en':':woman_with_white_cane_facing_right_dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':frau_mit_langstock_nach_rechts_dunkle_hautfarbe:','es':':mujer_con_bastón_hacia_la_derecha_tono_de_piel_oscuro:','fr':':femme_avec_canne_blanche_vers_la_droite_peau_foncée:','ja':':杖をついた女性_右向き_濃い肌色:','ko':':오른쪽을_향한_지팡이를_든_여자_검은색_피부:','pt':':mulher_com_bengala_para_cego_de_frente_para_a_direita_pele_escura:','it':':donna_con_bastone_bianco_di_orientamento_a_destra_carnagione_scura:','fa':':زن_با_عصای_سفید_به_سمت_راستپوست_آبنوسی:','id':':wanita_dengan_tongkat_menghadap_kanan_warna_kulit_gelap:','zh':':拄盲杖的女人面向右边_较深肤色:','ru':':женщина_с_тростью_для_ходьбы_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_kadın_koyu_cilt_tonu:','ar':':سيدة_بعصا_سير_متجه_لليمين_بلون_غامق:'},'\U0001F469\U0001F3FF\U0000200D\U0001F9AF\U0000200D\U000027A1':{'en':':woman_with_white_cane_facing_right_dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':frau_mit_langstock_nach_rechts_dunkle_hautfarbe:','es':':mujer_con_bastón_hacia_la_derecha_tono_de_piel_oscuro:','fr':':femme_avec_canne_blanche_vers_la_droite_peau_foncée:','ja':':杖をついた女性_右向き_濃い肌色:','ko':':오른쪽을_향한_지팡이를_든_여자_검은색_피부:','pt':':mulher_com_bengala_para_cego_de_frente_para_a_direita_pele_escura:','it':':donna_con_bastone_bianco_di_orientamento_a_destra_carnagione_scura:','fa':':زن_با_عصای_سفید_به_سمت_راستپوست_آبنوسی:','id':':wanita_dengan_tongkat_menghadap_kanan_warna_kulit_gelap:','zh':':拄盲杖的女人面向右边_较深肤色:','ru':':женщина_с_тростью_для_ходьбы_направленные_вправо_очень_темныи_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_kadın_koyu_cilt_tonu:','ar':':سيدة_بعصا_سير_متجه_لليمين_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F9AF\U0000200D\U000027A1\U0000FE0F':{'en':':woman_with_white_cane_facing_right_light_skin_tone:','status':fully_qualified,'E':15.1,'de':':frau_mit_langstock_nach_rechts_helle_hautfarbe:','es':':mujer_con_bastón_hacia_la_derecha_tono_de_piel_claro:','fr':':femme_avec_canne_blanche_vers_la_droite_peau_claire:','ja':':杖をついた女性_右向き_薄い肌色:','ko':':오른쪽을_향한_지팡이를_든_여자_하얀_피부:','pt':':mulher_com_bengala_para_cego_de_frente_para_a_direita_pele_clara:','it':':donna_con_bastone_bianco_di_orientamento_a_destra_carnagione_chiara:','fa':':زن_با_عصای_سفید_به_سمت_راست_پوست_سفید:','id':':wanita_dengan_tongkat_menghadap_kanan_warna_kulit_cerah:','zh':':拄盲杖的女人面向右边_较浅肤色:','ru':':женщина_с_тростью_для_ходьбы_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_kadın_açık_cilt_tonu:','ar':':سيدة_بعصا_سير_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FB\U0000200D\U0001F9AF\U0000200D\U000027A1':{'en':':woman_with_white_cane_facing_right_light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':frau_mit_langstock_nach_rechts_helle_hautfarbe:','es':':mujer_con_bastón_hacia_la_derecha_tono_de_piel_claro:','fr':':femme_avec_canne_blanche_vers_la_droite_peau_claire:','ja':':杖をついた女性_右向き_薄い肌色:','ko':':오른쪽을_향한_지팡이를_든_여자_하얀_피부:','pt':':mulher_com_bengala_para_cego_de_frente_para_a_direita_pele_clara:','it':':donna_con_bastone_bianco_di_orientamento_a_destra_carnagione_chiara:','fa':':زن_با_عصای_سفید_به_سمت_راست_پوست_سفید:','id':':wanita_dengan_tongkat_menghadap_kanan_warna_kulit_cerah:','zh':':拄盲杖的女人面向右边_较浅肤色:','ru':':женщина_с_тростью_для_ходьбы_направленные_вправо_светлый_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_kadın_açık_cilt_tonu:','ar':':سيدة_بعصا_سير_متجه_لليمين_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F9AF\U0000200D\U000027A1\U0000FE0F':{'en':':woman_with_white_cane_facing_right_medium-dark_skin_tone:','status':fully_qualified,'E':15.1,'de':':frau_mit_langstock_nach_rechts_mitteldunkle_hautfarbe:','es':':mujer_con_bastón_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':femme_avec_canne_blanche_vers_la_droite_peau_mate:','ja':':杖をついた女性_右向き_やや濃い肌色:','ko':':오른쪽을_향한_지팡이를_든_여자_진한_갈색_피부:','pt':':mulher_com_bengala_para_cego_de_frente_para_a_direita_pele_morena_escura:','it':':donna_con_bastone_bianco_di_orientamento_a_destra_carnagione_abbastanza_scura:','fa':':زن_با_عصای_سفید_به_سمت_راست_پوست_گندمی:','id':':wanita_dengan_tongkat_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':拄盲杖的女人面向右边_中等深肤色:','ru':':женщина_с_тростью_для_ходьбы_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_بعصا_سير_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F469\U0001F3FE\U0000200D\U0001F9AF\U0000200D\U000027A1':{'en':':woman_with_white_cane_facing_right_medium-dark_skin_tone:','status':minimally_qualified,'E':15.1,'de':':frau_mit_langstock_nach_rechts_mitteldunkle_hautfarbe:','es':':mujer_con_bastón_hacia_la_derecha_tono_de_piel_oscuro_medio:','fr':':femme_avec_canne_blanche_vers_la_droite_peau_mate:','ja':':杖をついた女性_右向き_やや濃い肌色:','ko':':오른쪽을_향한_지팡이를_든_여자_진한_갈색_피부:','pt':':mulher_com_bengala_para_cego_de_frente_para_a_direita_pele_morena_escura:','it':':donna_con_bastone_bianco_di_orientamento_a_destra_carnagione_abbastanza_scura:','fa':':زن_با_عصای_سفید_به_سمت_راست_پوست_گندمی:','id':':wanita_dengan_tongkat_menghadap_kanan_warna_kulit_gelap-sedang:','zh':':拄盲杖的女人面向右边_中等深肤色:','ru':':женщина_с_тростью_для_ходьбы_направленные_вправо_темный_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_بعصا_سير_متجه_لليمين_بشرة_بلون_معتدل:'},'\U0001F469\U0001F3FC\U0000200D\U0001F9AF\U0000200D\U000027A1\U0000FE0F':{'en':':woman_with_white_cane_facing_right_medium-light_skin_tone:','status':fully_qualified,'E':15.1,'de':':frau_mit_langstock_nach_rechts_mittelhelle_hautfarbe:','es':':mujer_con_bastón_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':femme_avec_canne_blanche_vers_la_droite_peau_moyennement_claire:','ja':':杖をついた女性_右向き_やや薄い肌色:','ko':':오른쪽을_향한_지팡이를_든_여자_연한_갈색_피부:','pt':':mulher_com_bengala_para_cego_de_frente_para_a_direita_pele_morena_clara:','it':':donna_con_bastone_bianco_di_orientamento_a_destra_carnagione_abbastanza_chiara:','fa':':زن_با_عصای_سفید_به_سمت_راست_پوست_روشن:','id':':wanita_dengan_tongkat_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':拄盲杖的女人面向右边_中等浅肤色:','ru':':женщина_с_тростью_для_ходьбы_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_görme_engelli_bastonlu_kadın_orta_açık_cilt_tonu:','ar':':سيدة_بعصا_سير_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FC\U0000200D\U0001F9AF\U0000200D\U000027A1':{'en':':woman_with_white_cane_facing_right_medium-light_skin_tone:','status':minimally_qualified,'E':15.1,'de':':frau_mit_langstock_nach_rechts_mittelhelle_hautfarbe:','es':':mujer_con_bastón_hacia_la_derecha_tono_de_piel_claro_medio:','fr':':femme_avec_canne_blanche_vers_la_droite_peau_moyennement_claire:','ja':':杖をついた女性_右向き_やや薄い肌色:','ko':':오른쪽을_향한_지팡이를_든_여자_연한_갈색_피부:','pt':':mulher_com_bengala_para_cego_de_frente_para_a_direita_pele_morena_clara:','it':':donna_con_bastone_bianco_di_orientamento_a_destra_carnagione_abbastanza_chiara:','fa':':زن_با_عصای_سفید_به_سمت_راست_پوست_روشن:','id':':wanita_dengan_tongkat_menghadap_kanan_warna_kulit_cerah-sedang:','zh':':拄盲杖的女人面向右边_中等浅肤色:','ru':':женщина_с_тростью_для_ходьбы_направленные_вправо_светлыи_тон_кож:','tr':':sağa_dönük_görme_engelli_bastonlu_kadın_orta_açık_cilt_tonu:','ar':':سيدة_بعصا_سير_متجه_لليمين_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F9AF\U0000200D\U000027A1\U0000FE0F':{'en':':woman_with_white_cane_facing_right_medium_skin_tone:','status':fully_qualified,'E':15.1,'de':':frau_mit_langstock_nach_rechts_mittlere_hautfarbe:','es':':mujer_con_bastón_hacia_la_derecha_tono_de_piel_medio:','fr':':femme_avec_canne_blanche_vers_la_droite_peau_légèrement_mate:','ja':':杖をついた女性_右向き_中間の肌色:','ko':':오른쪽을_향한_지팡이를_든_여자_갈색_피부:','pt':':mulher_com_bengala_para_cego_de_frente_para_a_direita_pele_morena:','it':':donna_con_bastone_bianco_di_orientamento_a_destra_carnagione_olivastra:','fa':':زن_با_عصای_سفید_به_سمت_راست_پوست_طلایی:','id':':wanita_dengan_tongkat_menghadap_kanan_warna_kulit_sedang:','zh':':拄盲杖的女人面向右边_中等肤色:','ru':':женщина_с_тростью_для_ходьбы_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_kadın_orta_cilt_tonu:','ar':':سيدة_بعصا_سير_متجه_لليمين_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F9AF\U0000200D\U000027A1':{'en':':woman_with_white_cane_facing_right_medium_skin_tone:','status':minimally_qualified,'E':15.1,'de':':frau_mit_langstock_nach_rechts_mittlere_hautfarbe:','es':':mujer_con_bastón_hacia_la_derecha_tono_de_piel_medio:','fr':':femme_avec_canne_blanche_vers_la_droite_peau_légèrement_mate:','ja':':杖をついた女性_右向き_中間の肌色:','ko':':오른쪽을_향한_지팡이를_든_여자_갈색_피부:','pt':':mulher_com_bengala_para_cego_de_frente_para_a_direita_pele_morena:','it':':donna_con_bastone_bianco_di_orientamento_a_destra_carnagione_olivastra:','fa':':زن_با_عصای_سفید_به_سمت_راست_پوست_طلایی:','id':':wanita_dengan_tongkat_menghadap_kanan_warna_kulit_sedang:','zh':':拄盲杖的女人面向右边_中等肤色:','ru':':женщина_с_тростью_для_ходьбы_направленные_вправо_средний_тон_кожи:','tr':':sağa_dönük_görme_engelli_bastonlu_kadın_orta_cilt_tonu:','ar':':سيدة_بعصا_سير_متجه_لليمين_بلون_معتدل:'},'\U0001F469\U0001F3FB\U0000200D\U0001F9AF':{'en':':woman_with_white_cane_light_skin_tone:','status':fully_qualified,'E':12,'de':':frau_mit_langstock_helle_hautfarbe:','es':':mujer_con_bastón_tono_de_piel_claro:','fr':':femme_avec_canne_blanche_peau_claire:','ja':':杖をついた女性_薄い肌色:','ko':':지팡이를_든_여자_하얀_피부:','pt':':mulher_com_bengala_para_cego_pele_clara:','it':':donna_con_bastone_bianco_di_orientamento_carnagione_chiara:','fa':':زن_با_عصای_سفید_پوست_سفید:','id':':wanita_dengan_tongkat_warna_kulit_cerah:','zh':':拄盲杖的女人_较浅肤色:','ru':':женщина_с_тростью_для_ходьбы_очень_светлый_тон_кожи:','tr':':görme_engelli_bastonlu_kadın_açık_cilt_tonu:','ar':':سيدة_بعصا_سير_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F9AF':{'en':':woman_with_white_cane_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':frau_mit_langstock_mitteldunkle_hautfarbe:','es':':mujer_con_bastón_tono_de_piel_oscuro_medio:','fr':':femme_avec_canne_blanche_peau_mate:','ja':':杖をついた女性_やや濃い肌色:','ko':':지팡이를_든_여자_진한_갈색_피부:','pt':':mulher_com_bengala_para_cego_pele_morena_escura:','it':':donna_con_bastone_bianco_di_orientamento_carnagione_abbastanza_scura:','fa':':زن_با_عصای_سفید_پوست_گندمی:','id':':wanita_dengan_tongkat_warna_kulit_gelap-sedang:','zh':':拄盲杖的女人_中等深肤色:','ru':':женщина_с_тростью_для_ходьбы_темный_тон_кожи:','tr':':görme_engelli_bastonlu_kadın_orta_koyu_cilt_tonu:','ar':':سيدة_بعصا_سير_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F9AF':{'en':':woman_with_white_cane_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':frau_mit_langstock_mittelhelle_hautfarbe:','es':':mujer_con_bastón_tono_de_piel_claro_medio:','fr':':femme_avec_canne_blanche_peau_moyennement_claire:','ja':':杖をついた女性_やや薄い肌色:','ko':':지팡이를_든_여자_연한_갈색_피부:','pt':':mulher_com_bengala_para_cego_pele_morena_clara:','it':':donna_con_bastone_bianco_di_orientamento_carnagione_abbastanza_chiara:','fa':':زن_با_عصای_سفید_پوست_روشن:','id':':wanita_dengan_tongkat_warna_kulit_cerah-sedang:','zh':':拄盲杖的女人_中等浅肤色:','ru':':женщина_с_тростью_для_ходьбы_светлый_тон_кожи:','tr':':görme_engelli_bastonlu_kadın_orta_açık_cilt_tonu:','ar':':سيدة_بعصا_سير_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F9AF':{'en':':woman_with_white_cane_medium_skin_tone:','status':fully_qualified,'E':12,'de':':frau_mit_langstock_mittlere_hautfarbe:','es':':mujer_con_bastón_tono_de_piel_medio:','fr':':femme_avec_canne_blanche_peau_légèrement_mate:','ja':':杖をついた女性_中間の肌色:','ko':':지팡이를_든_여자_갈색_피부:','pt':':mulher_com_bengala_para_cego_pele_morena:','it':':donna_con_bastone_bianco_di_orientamento_carnagione_olivastra:','fa':':زن_با_عصای_سفید_پوست_طلایی:','id':':wanita_dengan_tongkat_warna_kulit_sedang:','zh':':拄盲杖的女人_中等肤色:','ru':':женщина_с_тростью_для_ходьбы_средний_тон_кожи:','tr':':görme_engelli_bastonlu_kadın_orta_cilt_tonu:','ar':':سيدة_بعصا_سير_بشرة_بلون_معتدل:'},'\U0001F9DF\U0000200D\U00002640\U0000FE0F':{'en':':woman_zombie:','status':fully_qualified,'E':5,'alias':[':zombie_woman:'],'de':':weiblicher_zombie:','es':':zombi_mujer:','fr':':zombie_femme:','ja':':女のゾンビ:','ko':':여자_좀비:','pt':':mulher_zumbi:','it':':zombie_donna:','fa':':مرده_متحرک_زن:','id':':zombi_wanita:','zh':':女僵尸:','ru':':женщина-зомби:','tr':':kadın_zombi:','ar':':امرأة_زومبي:'},'\U0001F9DF\U0000200D\U00002640':{'en':':woman_zombie:','status':minimally_qualified,'E':5,'alias':[':zombie_woman:'],'de':':weiblicher_zombie:','es':':zombi_mujer:','fr':':zombie_femme:','ja':':女のゾンビ:','ko':':여자_좀비:','pt':':mulher_zumbi:','it':':zombie_donna:','fa':':مرده_متحرک_زن:','id':':zombi_wanita:','zh':':女僵尸:','ru':':женщина_зомби:','tr':':kadın_zombi:','ar':':امرأة_زومبي:'},'\U0001F462':{'en':':woman’s_boot:','status':fully_qualified,'E':0.6,'alias':[':boot:', ':womans_boot:'],'de':':damenstiefel:','es':':bota_de_mujer:','fr':':botte_de_femme:','ja':':ブーツ:','ko':':부츠:','pt':':bota_feminina:','it':':stivale_da_donna:','fa':':پوتین_زنانه:','id':':sepatu_bot_wanita:','zh':':女靴:','ru':':сапог:','tr':':kadın_botu:','ar':':حذاء_طويل_الساق_حريمي:'},'\U0001F45A':{'en':':woman’s_clothes:','status':fully_qualified,'E':0.6,'alias':[':womans_clothes:'],'de':':bluse:','es':':ropa_de_mujer:','fr':':vêtements_de_femme:','ja':':婦人服:','ko':':여성복:','pt':':roupas_femininas:','it':':maglietta_da_donna:','fa':':لباس_زنانه:','id':':pakaian_wanita:','zh':':女装:','ru':':блузка:','tr':':kadın_giysisi:','ar':':ملابس_حريمي:'},'\U0001F452':{'en':':woman’s_hat:','status':fully_qualified,'E':0.6,'alias':[':womans_hat:'],'de':':damenhut:','es':':sombrero_de_mujer:','fr':':chapeau_de_femme:','ja':':婦人帽子:','ko':':여성용_모자:','pt':':chapéu_feminino:','it':':cappello_da_donna:','fa':':کلاه_زنانه:','id':':topi_wanita:','zh':':女帽:','ru':':шляпка:','tr':':kadın_şapkası:','ar':':قبعة_حريمي:'},'\U0001F461':{'en':':woman’s_sandal:','status':fully_qualified,'E':0.6,'alias':[':sandal:', ':womans_sandal:'],'de':':damensandale:','es':':sandalia_de_mujer:','fr':':sandale_de_femme:','ja':':サンダル:','ko':':샌들:','pt':':sandália_feminina:','it':':sandalo_da_donna:','fa':':صندل_زنانه:','id':':sandal_wanita:','zh':':女式凉鞋:','ru':':босоножки:','tr':':kadın_sandaleti:','ar':':صندل_حريمي:'},'\U0001F46D':{'en':':women_holding_hands:','status':fully_qualified,'E':1,'alias':[':two_women_holding_hands:'],'de':':händchen_haltende_frauen:','es':':mujeres_de_la_mano:','fr':':deux_femmes_se_tenant_la_main:','ja':':手をつなぐ女性:','ko':':손을_잡고_있는_두_명의_여자:','pt':':duas_mulheres_de_mãos_dadas:','it':':due_donne_che_si_tengono_per_mano:','fa':':دو_زن_دست_در_دست:','id':':dua_wanita_bergandengan:','zh':':手拉手的两个女人:','ru':':две_женщины:','tr':':el_ele_tutuşan_kadınlar:','ar':':امرأتان_تمسكان_بيد_بعضهما:'},'\U0001F46D\U0001F3FF':{'en':':women_holding_hands_dark_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_frauen_dunkle_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_oscuro:','fr':':deux_femmes_se_tenant_la_main_peau_foncée:','ja':':手をつなぐ女性_濃い肌色:','ko':':손을_잡고_있는_두_명의_여자_검은색_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_escura:','it':':due_donne_che_si_tengono_per_mano_carnagione_scura:','fa':':دو_زن_دست_در_دست_پوست_آبنوسی:','id':':dua_wanita_bergandengan_warna_kulit_gelap:','zh':':手拉手的两个女人_较深肤色:','ru':':две_женщины_очень_темный_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_koyu_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_غامق:'},'\U0001F469\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FB':{'en':':women_holding_hands_dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_frauen_dunkle_hautfarbe_helle_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_oscuro_y_tono_de_piel_claro:','fr':':deux_femmes_se_tenant_la_main_peau_foncée_et_peau_claire:','ja':':手をつなぐ女性_濃い肌色_薄い肌色:','ko':':손을_잡고_있는_두_명의_여자_검은색_피부_하얀_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_escura_e_pele_clara:','it':':due_donne_che_si_tengono_per_mano_carnagione_scura_e_carnagione_chiara:','fa':':دو_زن_دست_در_دست_پوست_آبنوسی_پوست_سفید:','id':':dua_wanita_bergandengan_warna_kulit_gelap_warna_kulit_cerah:','zh':':手拉手的两个女人_较深肤色较浅肤色:','ru':':две_женщины_очень_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_koyu_cilt_tonu_açık_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_غامق_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FE':{'en':':women_holding_hands_dark_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_frauen_dunkle_hautfarbe_mitteldunkle_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_oscuro_y_tono_de_piel_oscuro_medio:','fr':':deux_femmes_se_tenant_la_main_peau_foncée_et_peau_mate:','ja':':手をつなぐ女性_濃い肌色_やや濃い肌色:','ko':':손을_잡고_있는_두_명의_여자_검은색_피부_진한_갈색_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_escura_e_pele_morena_escura:','it':':due_donne_che_si_tengono_per_mano_carnagione_scura_e_carnagione_abbastanza_scura:','fa':':دو_زن_دست_در_دست_پوست_آبنوسی_پوست_گندمی:','id':':dua_wanita_bergandengan_warna_kulit_gelap_warna_kulit_gelap-sedang:','zh':':手拉手的两个女人_较深肤色中等深肤色:','ru':':две_женщины_очень_темный_тон_кожи_темный_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_koyu_cilt_tonu_orta_koyu_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_غامق_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FC':{'en':':women_holding_hands_dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_frauen_dunkle_hautfarbe_mittelhelle_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_oscuro_y_tono_de_piel_claro_medio:','fr':':deux_femmes_se_tenant_la_main_peau_foncée_et_peau_moyennement_claire:','ja':':手をつなぐ女性_濃い肌色_やや薄い肌色:','ko':':손을_잡고_있는_두_명의_여자_검은색_피부_연한_갈색_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_escura_e_pele_morena_clara:','it':':due_donne_che_si_tengono_per_mano_carnagione_scura_e_carnagione_abbastanza_chiara:','fa':':دو_زن_دست_در_دست_پوست_آبنوسی_پوست_روشن:','id':':dua_wanita_bergandengan_warna_kulit_gelap_warna_kulit_cerah-sedang:','zh':':手拉手的两个女人_较深肤色中等浅肤色:','ru':':две_женщины_очень_темный_тон_кожи_светлый_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_غامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FF\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FD':{'en':':women_holding_hands_dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_frauen_dunkle_hautfarbe_mittlere_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_oscuro_y_tono_de_piel_medio:','fr':':deux_femmes_se_tenant_la_main_peau_foncée_et_peau_légèrement_mate:','ja':':手をつなぐ女性_濃い肌色_中間の肌色:','ko':':손을_잡고_있는_두_명의_여자_검은색_피부_갈색_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_escura_e_pele_morena:','it':':due_donne_che_si_tengono_per_mano_carnagione_scura_e_carnagione_olivastra:','fa':':دو_زن_دست_در_دست_پوست_آبنوسی_پوست_طلایی:','id':':dua_wanita_bergandengan_warna_kulit_gelap_warna_kulit_sedang:','zh':':手拉手的两个女人_较深肤色中等肤色:','ru':':две_женщины_очень_темный_тон_кожи_средний_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_koyu_cilt_tonu_orta_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_غامق_وبشرة_بلون_معتدل:'},'\U0001F46D\U0001F3FB':{'en':':women_holding_hands_light_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_frauen_helle_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_claro:','fr':':deux_femmes_se_tenant_la_main_peau_claire:','ja':':手をつなぐ女性_薄い肌色:','ko':':손을_잡고_있는_두_명의_여자_하얀_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_clara:','it':':due_donne_che_si_tengono_per_mano_carnagione_chiara:','fa':':دو_زن_دست_در_دست_پوست_سفید:','id':':dua_wanita_bergandengan_warna_kulit_cerah:','zh':':手拉手的两个女人_较浅肤色:','ru':':две_женщины_очень_светлый_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_açık_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_فاتح:'},'\U0001F469\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FF':{'en':':women_holding_hands_light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':händchen_haltende_frauen_helle_hautfarbe_dunkle_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_claro_y_tono_de_piel_oscuro:','fr':':deux_femmes_se_tenant_la_main_peau_claire_et_peau_foncée:','ja':':手をつなぐ女性_薄い肌色_濃い肌色:','ko':':손을_잡고_있는_두_명의_여자_하얀_피부_검은색_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_clara_e_pele_escura:','it':':due_donne_che_si_tengono_per_mano_carnagione_chiara_e_carnagione_scura:','fa':':دو_زن_دست_در_دست_پوست_سفید_پوست_آبنوسی:','id':':dua_wanita_bergandengan_warna_kulit_cerah_warna_kulit_gelap:','zh':':手拉手的两个女人_较浅肤色较深肤色:','ru':':две_женщины_очень_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_açık_cilt_tonu_koyu_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_فاتح_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FE':{'en':':women_holding_hands_light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':händchen_haltende_frauen_helle_hautfarbe_mitteldunkle_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_claro_y_tono_de_piel_oscuro_medio:','fr':':deux_femmes_se_tenant_la_main_peau_claire_et_peau_mate:','ja':':手をつなぐ女性_薄い肌色_やや濃い肌色:','ko':':손을_잡고_있는_두_명의_여자_하얀_피부_진한_갈색_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_clara_e_pele_morena_escura:','it':':due_donne_che_si_tengono_per_mano_carnagione_chiara_e_carnagione_abbastanza_scura:','fa':':دو_زن_دست_در_دست_پوست_سفید_پوست_گندمی:','id':':dua_wanita_bergandengan_warna_kulit_cerah_warna_kulit_gelap-sedang:','zh':':手拉手的两个女人_较浅肤色中等深肤色:','ru':':две_женщины_очень_светлый_тон_кожи_темный_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_فاتح_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FC':{'en':':women_holding_hands_light_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':12.1,'de':':händchen_haltende_frauen_helle_hautfarbe_mittelhelle_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_claro_y_tono_de_piel_claro_medio:','fr':':deux_femmes_se_tenant_la_main_peau_claire_et_peau_moyennement_claire:','ja':':手をつなぐ女性_薄い肌色_やや薄い肌色:','ko':':손을_잡고_있는_두_명의_여자_하얀_피부_연한_갈색_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_clara_e_pele_morena_clara:','it':':due_donne_che_si_tengono_per_mano_carnagione_chiara_e_carnagione_abbastanza_chiara:','fa':':دو_زن_دست_در_دست_پوست_سفید_پوست_روشن:','id':':dua_wanita_bergandengan_warna_kulit_cerah_warna_kulit_cerah-sedang:','zh':':手拉手的两个女人_较浅肤色中等浅肤色:','ru':':две_женщины_очень_светлый_тон_кожи_светлый_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_açık_cilt_tonu_orta_açık_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_فاتح_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FB\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FD':{'en':':women_holding_hands_light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':händchen_haltende_frauen_helle_hautfarbe_mittlere_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_claro_y_tono_de_piel_medio:','fr':':deux_femmes_se_tenant_la_main_peau_claire_et_peau_légèrement_mate:','ja':':手をつなぐ女性_薄い肌色_中間の肌色:','ko':':손을_잡고_있는_두_명의_여자_하얀_피부_갈색_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_clara_e_pele_morena:','it':':due_donne_che_si_tengono_per_mano_carnagione_chiara_e_carnagione_olivastra:','fa':':دو_زن_دست_در_دست_پوست_سفید_پوست_طلایی:','id':':dua_wanita_bergandengan_warna_kulit_cerah_warna_kulit_sedang:','zh':':手拉手的两个女人_较浅肤色中等肤色:','ru':':две_женщины_очень_светлый_тон_кожи_средний_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_açık_cilt_tonu_orta_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_فاتح_وبشرة_بلون_معتدل:'},'\U0001F46D\U0001F3FE':{'en':':women_holding_hands_medium-dark_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_frauen_mitteldunkle_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_oscuro_medio:','fr':':deux_femmes_se_tenant_la_main_peau_mate:','ja':':手をつなぐ女性_やや濃い肌色:','ko':':손을_잡고_있는_두_명의_여자_진한_갈색_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_morena_escura:','it':':due_donne_che_si_tengono_per_mano_carnagione_abbastanza_scura:','fa':':دو_زن_دست_در_دست_پوست_گندمی:','id':':dua_wanita_bergandengan_warna_kulit_gelap-sedang:','zh':':手拉手的两个女人_中等深肤色:','ru':':две_женщины_темный_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_orta_koyu_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FF':{'en':':women_holding_hands_medium-dark_skin_tone_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':händchen_haltende_frauen_mitteldunkle_hautfarbe_dunkle_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_oscuro_medio_y_tono_de_piel_oscuro:','fr':':deux_femmes_se_tenant_la_main_peau_mate_et_peau_foncée:','ja':':手をつなぐ女性_やや濃い肌色_濃い肌色:','ko':':손을_잡고_있는_두_명의_여자_진한_갈색_피부_검은색_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_morena_escura_e_pele_escura:','it':':due_donne_che_si_tengono_per_mano_carnagione_abbastanza_scura_e_carnagione_scura:','fa':':دو_زن_دست_در_دست_پوست_گندمی_پوست_آبنوسی:','id':':dua_wanita_bergandengan_warna_kulit_gelap-sedang_warna_kulit_gelap:','zh':':手拉手的两个女人_中等深肤色较深肤色:','ru':':две_женщины_темный_тон_кожи_очень_темный_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_orta_koyu_cilt_tonu_koyu_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FB':{'en':':women_holding_hands_medium-dark_skin_tone_light_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_frauen_mitteldunkle_hautfarbe_helle_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_oscuro_medio_y_tono_de_piel_claro:','fr':':deux_femmes_se_tenant_la_main_peau_mate_et_peau_claire:','ja':':手をつなぐ女性_やや濃い肌色_薄い肌色:','ko':':손을_잡고_있는_두_명의_여자_진한_갈색_피부_하얀_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_morena_escura_e_pele_clara:','it':':due_donne_che_si_tengono_per_mano_carnagione_abbastanza_scura_e_carnagione_chiara:','fa':':دو_زن_دست_در_دست_پوست_گندمی_پوست_سفید:','id':':dua_wanita_bergandengan_warna_kulit_gelap-sedang_warna_kulit_cerah:','zh':':手拉手的两个女人_中等深肤色较浅肤色:','ru':':две_женщины_темный_тон_кожи_очень_светлый_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_orta_koyu_cilt_tonu_açık_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FC':{'en':':women_holding_hands_medium-dark_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_frauen_mitteldunkle_hautfarbe_mittelhelle_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_oscuro_medio_y_tono_de_piel_claro_medio:','fr':':deux_femmes_se_tenant_la_main_peau_mate_et_peau_moyennement_claire:','ja':':手をつなぐ女性_やや濃い肌色_やや薄い肌色:','ko':':손을_잡고_있는_두_명의_여자_진한_갈색_피부_연한_갈색_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_morena_escura_e_pele_morena_clara:','it':':due_donne_che_si_tengono_per_mano_carnagione_abbastanza_scura_e_carnagione_abbastanza_chiara:','fa':':دو_زن_دست_در_دست_پوست_گندمی_پوست_روشن:','id':':dua_wanita_bergandengan_warna_kulit_gelap-sedang_warna_kulit_cerah-sedang:','zh':':手拉手的两个女人_中等深肤色中等浅肤色:','ru':':две_женщины_темный_тон_кожи_светлый_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_orta_koyu_cilt_tonu_orta_açık_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FE\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FD':{'en':':women_holding_hands_medium-dark_skin_tone_medium_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_frauen_mitteldunkle_hautfarbe_mittlere_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_oscuro_medio_y_tono_de_piel_medio:','fr':':deux_femmes_se_tenant_la_main_peau_mate_et_peau_légèrement_mate:','ja':':手をつなぐ女性_やや濃い肌色_中間の肌色:','ko':':손을_잡고_있는_두_명의_여자_진한_갈색_피부_갈색_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_morena_escura_e_pele_morena:','it':':due_donne_che_si_tengono_per_mano_carnagione_abbastanza_scura_e_carnagione_olivastra:','fa':':دو_زن_دست_در_دست_پوست_گندمی_پوست_طلایی:','id':':dua_wanita_bergandengan_warna_kulit_gelap-sedang_warna_kulit_sedang:','zh':':手拉手的两个女人_中等深肤色中等肤色:','ru':':две_женщины_темный_тон_кожи_средний_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_orta_koyu_cilt_tonu_orta_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_معتدل_مائل_للغامق_وبشرة_بلون_معتدل:'},'\U0001F46D\U0001F3FC':{'en':':women_holding_hands_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_frauen_mittelhelle_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_claro_medio:','fr':':deux_femmes_se_tenant_la_main_peau_moyennement_claire:','ja':':手をつなぐ女性_やや薄い肌色:','ko':':손을_잡고_있는_두_명의_여자_연한_갈색_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_morena_clara:','it':':due_donne_che_si_tengono_per_mano_carnagione_abbastanza_chiara:','fa':':دو_زن_دست_در_دست_پوست_روشن:','id':':dua_wanita_bergandengan_warna_kulit_cerah-sedang:','zh':':手拉手的两个女人_中等浅肤色:','ru':':две_женщины_светлый_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_orta_açık_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_فاتح_ومعتدل:'},'\U0001F469\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FF':{'en':':women_holding_hands_medium-light_skin_tone_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':händchen_haltende_frauen_mittelhelle_hautfarbe_dunkle_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_claro_medio_y_tono_de_piel_oscuro:','fr':':deux_femmes_se_tenant_la_main_peau_moyennement_claire_et_peau_foncée:','ja':':手をつなぐ女性_やや薄い肌色_濃い肌色:','ko':':손을_잡고_있는_두_명의_여자_연한_갈색_피부_검은색_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_morena_clara_e_pele_escura:','it':':due_donne_che_si_tengono_per_mano_carnagione_abbastanza_chiara_e_carnagione_scura:','fa':':دو_زن_دست_در_دست_پوست_روشن_پوست_آبنوسی:','id':':dua_wanita_bergandengan_warna_kulit_cerah-sedang_warna_kulit_gelap:','zh':':手拉手的两个女人_中等浅肤色较深肤色:','ru':':две_женщины_светлый_тон_кожи_очень_темный_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_orta_açık_cilt_tonu_koyu_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FB':{'en':':women_holding_hands_medium-light_skin_tone_light_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_frauen_mittelhelle_hautfarbe_helle_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_claro_medio_y_tono_de_piel_claro:','fr':':deux_femmes_se_tenant_la_main_peau_moyennement_claire_et_peau_claire:','ja':':手をつなぐ女性_やや薄い肌色_薄い肌色:','ko':':손을_잡고_있는_두_명의_여자_연한_갈색_피부_하얀_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_morena_clara_e_pele_clara:','it':':due_donne_che_si_tengono_per_mano_carnagione_abbastanza_chiara_e_carnagione_chiara:','fa':':دو_زن_دست_در_دست_پوست_روشن_پوست_سفید:','id':':dua_wanita_bergandengan_warna_kulit_cerah-sedang_warna_kulit_cerah:','zh':':手拉手的两个女人_中等浅肤色较浅肤色:','ru':':две_женщины_светлый_тон_кожи_очень_светлый_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_orta_açık_cilt_tonu_açık_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FE':{'en':':women_holding_hands_medium-light_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':händchen_haltende_frauen_mittelhelle_hautfarbe_mitteldunkle_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_claro_medio_y_tono_de_piel_oscuro_medio:','fr':':deux_femmes_se_tenant_la_main_peau_moyennement_claire_et_peau_mate:','ja':':手をつなぐ女性_やや薄い肌色_やや濃い肌色:','ko':':손을_잡고_있는_두_명의_여자_연한_갈색_피부_진한_갈색_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_morena_clara_e_pele_morena_escura:','it':':due_donne_che_si_tengono_per_mano_carnagione_abbastanza_chiara_e_carnagione_abbastanza_scura:','fa':':دو_زن_دست_در_دست_پوست_روشن_پوست_گندمی:','id':':dua_wanita_bergandengan_warna_kulit_cerah-sedang_warna_kulit_gelap-sedang:','zh':':手拉手的两个女人_中等浅肤色中等深肤色:','ru':':две_женщины_светлый_тон_кожи_темный_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_orta_açık_cilt_tonu_orta_koyu_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FC\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FD':{'en':':women_holding_hands_medium-light_skin_tone_medium_skin_tone:','status':fully_qualified,'E':12.1,'de':':händchen_haltende_frauen_mittelhelle_hautfarbe_mittlere_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_claro_medio_y_tono_de_piel_medio:','fr':':deux_femmes_se_tenant_la_main_peau_moyennement_claire_et_peau_légèrement_mate:','ja':':手をつなぐ女性_やや薄い肌色_中間の肌色:','ko':':손을_잡고_있는_두_명의_여자_연한_갈색_피부_갈색_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_morena_clara_e_pele_morena:','it':':due_donne_che_si_tengono_per_mano_carnagione_abbastanza_chiara_e_carnagione_olivastra:','fa':':دو_زن_دست_در_دست_پوست_روشن_پوست_طلایی:','id':':dua_wanita_bergandengan_warna_kulit_cerah-sedang_warna_kulit_sedang:','zh':':手拉手的两个女人_中等浅肤色中等肤色:','ru':':две_женщины_светлый_тон_кожи_средний_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_orta_açık_cilt_tonu_orta_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_فاتح_ومعتدل_وبشرة_بلون_معتدل:'},'\U0001F46D\U0001F3FD':{'en':':women_holding_hands_medium_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_frauen_mittlere_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_medio:','fr':':deux_femmes_se_tenant_la_main_peau_légèrement_mate:','ja':':手をつなぐ女性_中間の肌色:','ko':':손을_잡고_있는_두_명의_여자_갈색_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_morena:','it':':due_donne_che_si_tengono_per_mano_carnagione_olivastra:','fa':':دو_زن_دست_در_دست_پوست_طلایی:','id':':dua_wanita_bergandengan_warna_kulit_sedang:','zh':':手拉手的两个女人_中等肤色:','ru':':две_женщины_средний_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_orta_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_معتدل:'},'\U0001F469\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FF':{'en':':women_holding_hands_medium_skin_tone_dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':händchen_haltende_frauen_mittlere_hautfarbe_dunkle_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_medio_y_tono_de_piel_oscuro:','fr':':deux_femmes_se_tenant_la_main_peau_légèrement_mate_et_peau_foncée:','ja':':手をつなぐ女性_中間の肌色_濃い肌色:','ko':':손을_잡고_있는_두_명의_여자_갈색_피부_검은색_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_morena_e_pele_escura:','it':':due_donne_che_si_tengono_per_mano_carnagione_olivastra_e_carnagione_scura:','fa':':دو_زن_دست_در_دست_پوست_طلایی_پوست_آبنوسی:','id':':dua_wanita_bergandengan_warna_kulit_sedang_warna_kulit_gelap:','zh':':手拉手的两个女人_中等肤色较深肤色:','ru':':две_женщины_средний_тон_кожи_очень_темный_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_orta_cilt_tonu_koyu_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_معتدل_وبشرة_بلون_غامق:'},'\U0001F469\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FB':{'en':':women_holding_hands_medium_skin_tone_light_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_frauen_mittlere_hautfarbe_helle_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_medio_y_tono_de_piel_claro:','fr':':deux_femmes_se_tenant_la_main_peau_légèrement_mate_et_peau_claire:','ja':':手をつなぐ女性_中間の肌色_薄い肌色:','ko':':손을_잡고_있는_두_명의_여자_갈색_피부_하얀_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_morena_e_pele_clara:','it':':due_donne_che_si_tengono_per_mano_carnagione_olivastra_e_carnagione_chiara:','fa':':دو_زن_دست_در_دست_پوست_طلایی_پوست_سفید:','id':':dua_wanita_bergandengan_warna_kulit_sedang_warna_kulit_cerah:','zh':':手拉手的两个女人_中等肤色较浅肤色:','ru':':две_женщины_средний_тон_кожи_очень_светлый_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_orta_cilt_tonu_açık_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_معتدل_وبشرة_بلون_فاتح:'},'\U0001F469\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FE':{'en':':women_holding_hands_medium_skin_tone_medium-dark_skin_tone:','status':fully_qualified,'E':12.1,'de':':händchen_haltende_frauen_mittlere_hautfarbe_mitteldunkle_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_medio_y_tono_de_piel_oscuro_medio:','fr':':deux_femmes_se_tenant_la_main_peau_légèrement_mate_et_peau_mate:','ja':':手をつなぐ女性_中間の肌色_やや濃い肌色:','ko':':손을_잡고_있는_두_명의_여자_갈색_피부_진한_갈색_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_morena_e_pele_morena_escura:','it':':due_donne_che_si_tengono_per_mano_carnagione_olivastra_e_carnagione_abbastanza_scura:','fa':':دو_زن_دست_در_دست_پوست_طلایی_پوست_گندمی:','id':':dua_wanita_bergandengan_warna_kulit_sedang_warna_kulit_gelap-sedang:','zh':':手拉手的两个女人_中等肤色中等深肤色:','ru':':две_женщины_средний_тон_кожи_темный_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_orta_cilt_tonu_orta_koyu_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_معتدل_وبشرة_بلون_معتدل_مائل_للغامق:'},'\U0001F469\U0001F3FD\U0000200D\U0001F91D\U0000200D\U0001F469\U0001F3FC':{'en':':women_holding_hands_medium_skin_tone_medium-light_skin_tone:','status':fully_qualified,'E':12,'de':':händchen_haltende_frauen_mittlere_hautfarbe_mittelhelle_hautfarbe:','es':':mujeres_de_la_mano_tono_de_piel_medio_y_tono_de_piel_claro_medio:','fr':':deux_femmes_se_tenant_la_main_peau_légèrement_mate_et_peau_moyennement_claire:','ja':':手をつなぐ女性_中間の肌色_やや薄い肌色:','ko':':손을_잡고_있는_두_명의_여자_갈색_피부_연한_갈색_피부:','pt':':duas_mulheres_de_mãos_dadas_pele_morena_e_pele_morena_clara:','it':':due_donne_che_si_tengono_per_mano_carnagione_olivastra_e_carnagione_abbastanza_chiara:','fa':':دو_زن_دست_در_دست_پوست_طلایی_پوست_روشن:','id':':dua_wanita_bergandengan_warna_kulit_sedang_warna_kulit_cerah-sedang:','zh':':手拉手的两个女人_中等肤色中等浅肤色:','ru':':две_женщины_средний_тон_кожи_светлый_тон_кожи:','tr':':el_ele_tutuşan_kadınlar_orta_cilt_tonu_orta_açık_cilt_tonu:','ar':':امرأتان_تمسكان_بيد_بعضهما_بشرة_بلون_معتدل_وبشرة_بلون_فاتح_ومعتدل:'},'\U0001F46F\U0000200D\U00002640\U0000FE0F':{'en':':women_with_bunny_ears:','status':fully_qualified,'E':4,'alias':[':dancing_women:'],'de':':frauen_mit_hasenohren:','es':':mujeres_con_orejas_de_conejo:','fr':':femmes_avec_des_oreilles_de_lapin:','ja':':バニーガール:','ko':':토끼_귀를_쓰고_춤추는_여자들:','pt':':mulheres_com_orelhas_de_coelho:','it':':donne_con_orecchie_da_coniglio:','fa':':زنان_در_پارتی:','id':':wanita_dengan_telinga_kelinci:','zh':':兔女郎:','ru':':женщины_с_ушами_кролика:','tr':':tavşan_kulaklı_kadınlar:','ar':':امرأتان_ترقصان:'},'\U0001F46F\U0000200D\U00002640':{'en':':women_with_bunny_ears:','status':minimally_qualified,'E':4,'alias':[':dancing_women:'],'de':':frauen_mit_hasenohren:','es':':mujeres_con_orejas_de_conejo:','fr':':femmes_avec_des_oreilles_de_lapin:','ja':':バニーガール:','ko':':토끼_귀를_쓰고_춤추는_여자들:','pt':':mulheres_com_orelhas_de_coelho:','it':':donne_con_orecchie_da_coniglio:','fa':':زنان_در_پارتی:','id':':wanita_dengan_telinga_kelinci:','zh':':兔女郎:','ru':':женщины_с_ушами_кролика:','tr':':tavşan_kulaklı_kadınlar:','ar':':امرأتان_ترقصان:'},'\U0001F93C\U0000200D\U00002640\U0000FE0F':{'en':':women_wrestling:','status':fully_qualified,'E':4,'de':':ringende_frauen:','es':':mujeres_luchando:','fr':':lutteuses:','ja':':レスリングする女:','ko':':레슬링하는_여자:','pt':':mulheres_lutando:','it':':lottatrici:','fa':':کُشتی_گیر_زن:','id':':wanita_bergulat:','zh':':女生摔跤:','ru':':женщины-борцы:','tr':':güreş_yapan_kadın:','ar':':مصارعة_سيدات:'},'\U0001F93C\U0000200D\U00002640':{'en':':women_wrestling:','status':minimally_qualified,'E':4,'de':':ringende_frauen:','es':':mujeres_luchando:','fr':':lutteuses:','ja':':レスリングする女:','ko':':레슬링하는_여자:','pt':':mulheres_lutando:','it':':lottatrici:','fa':':کُشتی_گیر_زن:','id':':wanita_bergulat:','zh':':女生摔跤:','ru':':женщины_борцы:','tr':':güreş_yapan_kadın:','ar':':مصارعة_سيدات:'},'\U0001F6BA':{'en':':women’s_room:','status':fully_qualified,'E':0.6,'alias':[':womens:', ':womens_room:'],'variant':True,'de':':damen:','es':':señal_de_aseo_para_mujeres:','fr':':symbole_toilettes_femmes:','ja':':女子トイレ:','ko':':여자_화장실:','pt':':banheiro_feminino:','it':':simbolo_del_bagno_delle_donne:','fa':':توالت_زنانه:','id':':toilet_wanita:','zh':':女厕:','ru':':женский_туалет:','tr':':kadınlar_tuvaleti:','ar':':للسيدات:'},'\U0001FAB5':{'en':':wood:','status':fully_qualified,'E':13,'de':':holz:','es':':madera:','fr':':bois:','ja':':丸太:','ko':':목재:','pt':':madeira:','it':':legna:','fa':':چوب:','id':':kayu:','zh':':木头:','ru':':древесина:','tr':':kütük:','ar':':خشب:'},'\U0001F974':{'en':':woozy_face:','status':fully_qualified,'E':11,'de':':schwindeliges_gesicht:','es':':cara_de_grogui:','fr':':visage_éméché:','ja':':ふらふらの顔:','ko':':헤롱헤롱_얼굴:','pt':':rosto_embriagado:','it':':faccina_stordita:','fa':':نشئه:','id':':wajah_pusing_karena_mabuk:','zh':':头昏眼花:','ru':':чувствует_головокружение:','tr':':sarhoş_yüz:','ar':':وجه_مصاب_بدوار:'},'\U0001F5FA\U0000FE0F':{'en':':world_map:','status':fully_qualified,'E':0.7,'variant':True,'de':':weltkarte:','es':':mapa_mundial:','fr':':carte_du_monde:','ja':':世界地図:','ko':':세계_지도:','pt':':mapa-múndi:','it':':mappa_mondiale:','fa':':نقشه_جهان:','id':':peta_dunia:','zh':':世界地图:','ru':':карта_мира:','tr':':dünya_haritası:','ar':':خريطة_العالم:'},'\U0001F5FA':{'en':':world_map:','status':unqualified,'E':0.7,'variant':True,'de':':weltkarte:','es':':mapa_mundial:','fr':':carte_du_monde:','ja':':世界地図:','ko':':세계_지도:','pt':':mapa-múndi:','it':':mappa_mondiale:','fa':':نقشهٔ_جهان:','id':':peta_dunia:','zh':':世界地图:','ru':':карта_мира:','tr':':dünya_haritası:','ar':':خريطة_العالم:'},'\U0001FAB1':{'en':':worm:','status':fully_qualified,'E':13,'de':':wurm:','es':':gusano:','fr':':lombric:','ja':':ミミズ:','ko':':지렁이:','pt':':minhoca:','it':':verme:','fa':':کرم:','id':':cacing:','zh':':蠕虫:','ru':':червяк:','tr':':solucan:','ar':':دودة:'},'\U0001F61F':{'en':':worried_face:','status':fully_qualified,'E':1,'alias':[':worried:'],'de':':besorgtes_gesicht:','es':':cara_preocupada:','fr':':visage_inquiet:','ja':':悩む顔:','ko':':걱정스러운_얼굴:','pt':':rosto_preocupado:','it':':faccina_preoccupata:','fa':':چهره_نگران:','id':':wajah_cemas:','zh':':担心:','ru':':беспокоится:','tr':':endişeli_yüz:','ar':':وجه_قلق:'},'\U0001F381':{'en':':wrapped_gift:','status':fully_qualified,'E':0.6,'alias':[':gift:'],'de':':geschenk:','es':':regalo:','fr':':cadeau:','ja':':プレゼント:','ko':':선물:','pt':':presente:','it':':regalo:','fa':':هدیهٔ_کادوشده:','id':':kado:','zh':':礼物:','ru':':подарок:','tr':':paketlenmiş_hediye:','ar':':هدية_ملفوفة:'},'\U0001F527':{'en':':wrench:','status':fully_qualified,'E':0.6,'de':':schraubenschlüssel:','es':':llave_inglesa:','fr':':clé_à_molette:','ja':':レンチ:','ko':':렌치:','pt':':chave_inglesa:','it':':chiave_inglese:','fa':':آچار:','id':':kunci_inggris:','zh':':扳手:','ru':':гаечный_ключ:','tr':':somun_anahtarı:','ar':':مفتاح_ربط:'},'\U0000270D\U0000FE0F':{'en':':writing_hand:','status':fully_qualified,'E':0.7,'variant':True,'de':':schreibende_hand:','es':':mano_escribiendo:','fr':':main_qui_écrit:','ja':':書いている手:','ko':':글을_쓰고_있는_손:','pt':':escrevendo_à_mão:','it':':mano_che_scrive:','fa':':نوشتن_با_دست:','id':':tangan_menulis:','zh':':写字:','ru':':пишущая_рука:','tr':':yazı_yazan_el:','ar':':يد_تكتب:'},'\U0000270D':{'en':':writing_hand:','status':unqualified,'E':0.7,'variant':True,'de':':schreibende_hand:','es':':mano_escribiendo:','fr':':main_qui_écrit:','ja':':書いている手:','ko':':글을_쓰고_있는_손:','pt':':escrevendo_à_mão:','it':':mano_che_scrive:','fa':':نوشتن_با_دست:','id':':tangan_menulis:','zh':':写字:','ru':':пишущая_рука:','tr':':yazı_yazan_el:','ar':':يد_تكتب:'},'\U0000270D\U0001F3FF':{'en':':writing_hand_dark_skin_tone:','status':fully_qualified,'E':1,'de':':schreibende_hand_dunkle_hautfarbe:','es':':mano_escribiendo_tono_de_piel_oscuro:','fr':':main_qui_écrit_peau_foncée:','ja':':書いている手_濃い肌色:','ko':':글을_쓰고_있는_손_검은색_피부:','pt':':escrevendo_à_mão_pele_escura:','it':':mano_che_scrive_carnagione_scura:','fa':':نوشتن_با_دست_پوست_آبنوسی:','id':':tangan_menulis_warna_kulit_gelap:','zh':':写字_较深肤色:','ru':':пишущая_рука_очень_темный_тон_кожи:','tr':':yazı_yazan_el_koyu_cilt_tonu:','ar':':يد_تكتب_بشرة_بلون_غامق:'},'\U0000270D\U0001F3FB':{'en':':writing_hand_light_skin_tone:','status':fully_qualified,'E':1,'de':':schreibende_hand_helle_hautfarbe:','es':':mano_escribiendo_tono_de_piel_claro:','fr':':main_qui_écrit_peau_claire:','ja':':書いている手_薄い肌色:','ko':':글을_쓰고_있는_손_하얀_피부:','pt':':escrevendo_à_mão_pele_clara:','it':':mano_che_scrive_carnagione_chiara:','fa':':نوشتن_با_دست_پوست_سفید:','id':':tangan_menulis_warna_kulit_cerah:','zh':':写字_较浅肤色:','ru':':пишущая_рука_очень_светлый_тон_кожи:','tr':':yazı_yazan_el_açık_cilt_tonu:','ar':':يد_تكتب_بشرة_بلون_فاتح:'},'\U0000270D\U0001F3FE':{'en':':writing_hand_medium-dark_skin_tone:','status':fully_qualified,'E':1,'de':':schreibende_hand_mitteldunkle_hautfarbe:','es':':mano_escribiendo_tono_de_piel_oscuro_medio:','fr':':main_qui_écrit_peau_mate:','ja':':書いている手_やや濃い肌色:','ko':':글을_쓰고_있는_손_진한_갈색_피부:','pt':':escrevendo_à_mão_pele_morena_escura:','it':':mano_che_scrive_carnagione_abbastanza_scura:','fa':':نوشتن_با_دست_پوست_گندمی:','id':':tangan_menulis_warna_kulit_gelap-sedang:','zh':':写字_中等深肤色:','ru':':пишущая_рука_темный_тон_кожи:','tr':':yazı_yazan_el_orta_koyu_cilt_tonu:','ar':':يد_تكتب_بشرة_بلون_معتدل_مائل_للغامق:'},'\U0000270D\U0001F3FC':{'en':':writing_hand_medium-light_skin_tone:','status':fully_qualified,'E':1,'de':':schreibende_hand_mittelhelle_hautfarbe:','es':':mano_escribiendo_tono_de_piel_claro_medio:','fr':':main_qui_écrit_peau_moyennement_claire:','ja':':書いている手_やや薄い肌色:','ko':':글을_쓰고_있는_손_연한_갈색_피부:','pt':':escrevendo_à_mão_pele_morena_clara:','it':':mano_che_scrive_carnagione_abbastanza_chiara:','fa':':نوشتن_با_دست_پوست_روشن:','id':':tangan_menulis_warna_kulit_cerah-sedang:','zh':':写字_中等浅肤色:','ru':':пишущая_рука_светлый_тон_кожи:','tr':':yazı_yazan_el_orta_açık_cilt_tonu:','ar':':يد_تكتب_بشرة_بلون_فاتح_ومعتدل:'},'\U0000270D\U0001F3FD':{'en':':writing_hand_medium_skin_tone:','status':fully_qualified,'E':1,'de':':schreibende_hand_mittlere_hautfarbe:','es':':mano_escribiendo_tono_de_piel_medio:','fr':':main_qui_écrit_peau_légèrement_mate:','ja':':書いている手_中間の肌色:','ko':':글을_쓰고_있는_손_갈색_피부:','pt':':escrevendo_à_mão_pele_morena:','it':':mano_che_scrive_carnagione_olivastra:','fa':':نوشتن_با_دست_پوست_طلایی:','id':':tangan_menulis_warna_kulit_sedang:','zh':':写字_中等肤色:','ru':':пишущая_рука_средний_тон_кожи:','tr':':yazı_yazan_el_orta_cilt_tonu:','ar':':يد_تكتب_بشرة_بلون_معتدل:'},'\U0001FA7B':{'en':':x-ray:','status':fully_qualified,'E':14,'alias':[':x_ray:'],'de':':röntgenbild:','es':':radiografía:','fr':':radiographie:','ja':':x線:','ko':':엑스레이:','pt':':raio_x:','it':':radiografia:','fa':':اشعه_ایکس:','id':':sinar-x:','zh':':X射线:','ru':':рентген:','tr':':x_ışını:','ar':':أشعة_سينية:'},'\U0001F9F6':{'en':':yarn:','status':fully_qualified,'E':11,'de':':wollknäuel:','es':':ovillo:','fr':':fil:','ja':':毛糸:','ko':':실뭉치:','pt':':novelo:','it':':gomitolo:','fa':':نخ_بافندگی:','id':':gulungan_benang:','zh':':毛线:','ru':':клубок:','tr':':iplik_yumağı:','ar':':خيط_حياكة:'},'\U0001F971':{'en':':yawning_face:','status':fully_qualified,'E':12,'de':':gähnendes_gesicht:','es':':cara_de_bostezo:','fr':':visage_bâillant:','ja':':あくびした顔:','ko':':하품하는_얼굴:','pt':':rosto_bocejando:','it':':faccina_che_sbadiglia:','fa':':صورت_خمیازه_کش:','id':':wajah_menguap:','zh':':打呵欠:','ru':':зевает:','tr':':esneyen_yüz:','ar':':وجه_متثائب:'},'\U0001F7E1':{'en':':yellow_circle:','status':fully_qualified,'E':12,'de':':gelber_punkt:','es':':círculo_amarillo:','fr':':disque_jaune:','ja':':黄色い丸:','ko':':노란색_원:','pt':':círculo_amarelo:','it':':cerchio_giallo:','fa':':دایره_زرد:','id':':lingkaran_kuning:','zh':':黄色圆:','ru':':желтый_круг:','tr':':sarı_daire:','ar':':دائرة_صفراء:'},'\U0001F49B':{'en':':yellow_heart:','status':fully_qualified,'E':0.6,'de':':gelbes_herz:','es':':corazón_amarillo:','fr':':cœur_jaune:','ja':':黄色のハート:','ko':':노란색_하트:','pt':':coração_amarelo:','it':':cuore_giallo:','fa':':قلب_زرد:','id':':hati_kuning:','zh':':黄心:','ru':':желтое_сердце:','tr':':sarı_kalp:','ar':':قلب_أصفر:'},'\U0001F7E8':{'en':':yellow_square:','status':fully_qualified,'E':12,'de':':gelbes_quadrat:','es':':cuadrado_amarillo:','fr':':carré_jaune:','ja':':黄色い四角:','ko':':노란색_사각형:','pt':':quadrado_amarelo:','it':':quadrato_giallo:','fa':':مربع_زرد:','id':':persegi_kuning:','zh':':黄色方块:','ru':':желтый_квадрат:','tr':':sarı_kare:','ar':':مربع_أصفر:'},'\U0001F4B4':{'en':':yen_banknote:','status':fully_qualified,'E':0.6,'alias':[':yen:'],'de':':yen-banknote:','es':':billete_de_yen:','fr':':billet_en_yens:','ja':':円札:','ko':':엔:','pt':':nota_de_iene:','it':':banconota_yen:','fa':':اسکناس_ین:','id':':uang_kertas_yen:','zh':':日元:','ru':':банкнота_иены:','tr':':yen_banknot:','ar':':ورقة_ين:'},'\U0000262F\U0000FE0F':{'en':':yin_yang:','status':fully_qualified,'E':0.7,'variant':True,'de':':yin_und_yang:','es':':yin_yang:','fr':':yin_yang:','ja':':陰陽:','ko':':음양:','pt':':yin_yang:','it':':yin_e_yang:','fa':':یین_و_ینگ:','id':':yin_yang:','zh':':阴阳:','ru':':инь-ян:','tr':':yin_yang:','ar':':نصفان_متكاملان:'},'\U0000262F':{'en':':yin_yang:','status':unqualified,'E':0.7,'variant':True,'de':':yin_und_yang:','es':':yin_yang:','fr':':yin_yang:','ja':':陰陽:','ko':':음양:','pt':':yin_yang:','it':':yin_e_yang:','fa':':یین_و_ینگ:','id':':yin_yang:','zh':':阴阳:','ru':':инь_ян:','tr':':yin_yang:','ar':':نصفان_متكاملان:'},'\U0001FA80':{'en':':yo-yo:','status':fully_qualified,'E':12,'alias':[':yo_yo:'],'de':':jo-jo:','es':':yoyó:','fr':':yoyo:','ja':':ヨーヨー:','ko':':요요:','pt':':ioiô:','it':':yo-yo:','fa':':یویو:','id':':yo-yo:','zh':':悠悠球:','ru':':йо_йо:','tr':':yoyo:','ar':':يويو:'},'\U0001F92A':{'en':':zany_face:','status':fully_qualified,'E':5,'de':':irres_gesicht:','es':':cara_de_loco:','fr':':tête_de_fou:','ja':':ふざけた顔:','ko':':괴짜_얼굴:','pt':':rosto_bizarro:','it':':faccina_impazzita:','fa':':صورتک_لوده:','id':':wajah_konyol:','zh':':滑稽:','ru':':сумасшедшее_лицо:','tr':':çılgın_surat:','ar':':وجه_غريب_الشكل:'},'\U0001F993':{'en':':zebra:','status':fully_qualified,'E':5,'de':':zebra:','es':':cebra:','fr':':zèbre:','ja':':シマウマ:','ko':':얼룩말:','pt':':zebra:','it':':zebra:','fa':':گورخر:','id':':zebra:','zh':':斑马:','ru':':зебра:','tr':':zebra:','ar':':حمار_وحشي:'},'\U0001F910':{'en':':zipper-mouth_face:','status':fully_qualified,'E':1,'alias':[':zipper_mouth_face:'],'de':':gesicht_mit_reißverschlussmund:','es':':cara_con_la_boca_cerrada_con_cremallera:','fr':':visage_avec_bouche_fermeture_éclair:','ja':':口チャック:','ko':':지퍼로_입을_잠근_얼굴:','pt':':rosto_com_boca_de_zíper:','it':':faccina_con_bocca_con_cerniera:','fa':':دهان_چفت:','id':':wajah_dengan_mulut_tertutup_rapat:','zh':':闭嘴:','ru':':рот_на_замке:','tr':':ağzı_fermuarlı_yüz:','ar':':وجه_بفم_مقفل:'},'\U0001F9DF':{'en':':zombie:','status':fully_qualified,'E':5,'de':':zombie:','es':':zombi:','fr':':zombie:','ja':':ゾンビ:','ko':':좀비:','pt':':zumbi:','it':':zombie:','fa':':مرده_متحرک:','id':':zombi:','zh':':僵尸:','ru':':зомби:','tr':':zombi:','ar':':زومبي:'},'\U0001F1E6\U0001F1FD':{'en':':Åland_Islands:','status':fully_qualified,'E':2,'alias':[':flag_for_Åland_Islands:', ':aland_islands:'],'de':':flagge_ålandinseln:','es':':bandera_islas_aland:','fr':':drapeau_îles_åland:','ja':':旗_オーランド諸島:','ko':':깃발_올란드_제도:','pt':':bandeira_ilhas_aland:','it':':bandiera_isole_åland:','fa':':پرچم_جزایر_آلاند:','id':':bendera_kepulauan_aland:','zh':':奥兰群岛:','ru':':флаг_Аландские_о_ва:','tr':':bayrak_åland_adaları:','ar':':علم_جزر_آلاند:'}}
